(* Content-type: application/mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 6.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       145,          7]
NotebookDataLength[    131729,       4686]
NotebookOptionsPosition[    104424,       3818]
NotebookOutlinePosition[    118749,       4249]
CellTagsIndexPosition[    118241,       4230]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{

Cell[CellGroupData[{
Cell["FeynCalc 8.0", "Title",
 CellChangeTimes->{3.500969808274276*^9},
 TextAlignment->Center],

Cell["Tools for quantum field theory calculations", "Subtitle",
 TextAlignment->Center],

Cell[TextData[StyleBox["Short manual",
 FontWeight->"Bold"]], "Subtitle",
 PageBreakBelow->True,
 TextAlignment->Center],

Cell["Rolf Mertig and Frederik Orellana", "Subsubtitle",
 TextAlignment->Center],

Cell[TextData[{
 "HTML version: ",
 ButtonBox["http://www.feyncalc.org/",
  BaseStyle->"Hyperlink",
  ButtonData:>{
    URL["http://www.feyncalc.org/"], None},
  ButtonNote->"http://www.feyncalc.org/"]
}], "Subsubtitle",
 TextAlignment->Center],

Cell[BoxData[
 ButtonBox[
  RowBox[{
  "Click", " ", "to", " ", "save", " ", "this", " ", "notebook", " ", 
   "without", " ", "cache", " ", "and", " ", "FrontEndVersion", " ", "info"}],
  
  ButtonFunction:>Module[{$CellContext`fn}, SetDirectory[
      DirectoryName[$CellContext`fn = ToFileName[
         ReplaceAll["FileName", 
          NotebookInformation[
           InputNotebook[]]]]]]; $CellContext`newfn = 
     StringReplace[$CellContext`fn, ".nb" -> "NoCache.nb"]; 
    If[FileNames[$CellContext`newfn] =!= {}, 
      DeleteFile[$CellContext`newfn]]; Put[
      ReplaceAll[
       NotebookGet[
        InputNotebook[]], {
       Condition[FrontEndVersion -> Pattern[$CellContext`v, 
            Blank[]], Head[$CellContext`v] == String] :> Sequence[], 
        Condition[CellLabel -> Pattern[$CellContext`v, 
            Blank[]], Head[$CellContext`v] == String] :> Sequence[]}], 
      "fcr.nb"]; CopyFile["fcr.nb", $CellContext`newfn]; DeleteFile["fcr.nb"]],
  Evaluator->Automatic]], NotebookDefault,
 Deployed->True,
 CellTags->"ClickSave"],

Cell[CellGroupData[{

Cell["Introduction", "Section",
 PageBreakAbove->True,
 CellTags->{"Short Overview", "intro"}],

Cell[TextData[{
 "FeynCalc is a ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " package for algebraic calculations in high energy physics. It provides \
tools for Lorentz structure manipulations, Dirac algebra manipulations, color \
factor calculations, Feynman rule derivation, Feynman loop integral \
calculations (e.g. Passarino-Veltman reduction) and noncommutative algebra \
calculations, as well as various tabulations of Feynman parameter integrals, \
Mellin ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["transforms", 
    ButtonBox["\[Dagger]",
     BaseStyle->"Hyperlink",
     ButtonData:>"introfoot"]], TraditionalForm]]],
 ", convolutions, lagrangians and Feynman rules. Also, translation facilities \
are provided to change the FeynCalc syntax to and from FORM syntax and to \
generate FORTRAN code."
}], "Text",
 CellTags->"intro"],

Cell[TextData[{
 "Some functions for input of objects like ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["p", "\[Mu]"], ",", 
    SuperscriptBox["\[Gamma]", "\[Nu]"], ",", 
    SuperscriptBox["g", "\[Mu]\[Nu]"], ",", " ", 
    SuperscriptBox["\[CurlyEpsilon]", "\[Mu]\[Nu]\[Rho]\[Sigma]"], ",", " ", 
    RowBox[{"etc", "."}]}], TraditionalForm]]],
 " (abbreviations in parentheses) are:"
}], "Text",
 CellChangeTimes->{{3.5009700642581778`*^9, 3.500970068297248*^9}},
 CellTags->"intro"],

Cell[TextData[{
 ButtonBox["DiracMatrix",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracMatrix"],
 " (",
 ButtonBox["GA",
  BaseStyle->"AddOnsLink",
  ButtonData:>"GA"],
 "), ",
 ButtonBox["DiracSlash",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracSlash"],
 " (",
 ButtonBox["GS",
  BaseStyle->"AddOnsLink",
  ButtonData:>"GS"],
 "), ",
 ButtonBox["FourVector",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FourVector",
  ButtonNote->"FourVector"],
 " (",
 ButtonBox["FV",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FV"],
 "), ",
 ButtonBox["LeviCivita",
  BaseStyle->"AddOnsLink",
  ButtonData:>"LeviCivita"],
 " (",
 ButtonBox["LC",
  BaseStyle->"AddOnsLink",
  ButtonData:>"LC"],
 "), ",
 ButtonBox["MetricTensor",
  BaseStyle->"AddOnsLink",
  ButtonData:>"MetricTensor"],
 " (",
 ButtonBox["MT",
  BaseStyle->"AddOnsLink",
  ButtonData:>"MT"],
 "), ",
 ButtonBox["Spinor",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Spinor"],
 ", ",
 ButtonBox["SUNDelta",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SUNDelta"],
 ", ",
 ButtonBox["SUNF",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SUNF"],
 ", ",
 ButtonBox["ScalarProduct",
  BaseStyle->"AddOnsLink",
  ButtonData:>"ScalarProduct"],
 " (",
 ButtonBox["SP",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SP"],
 "). "
}], "Text"],

Cell["Some functions for manipulations are:", "Text",
 CellTags->"intro"],

Cell[TextData[{
 ButtonBox["Contract",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Contract"],
 ", ",
 ButtonBox["Tr",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Tr"],
 ", ",
 ButtonBox["DiracSimplify",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracSimplify"],
 ", ",
 ButtonBox["DiracTrick",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracTrick"],
 ", ",
 ButtonBox["Calc",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Calc"],
 ", ",
 ButtonBox["Convolute",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Convolute"],
 ", ",
 ButtonBox["FeynRule",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FeynRule"],
 ", ",
 ButtonBox["Integrate2",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Integrate2"],
 ", ",
 ButtonBox["FeynCalc2FORM",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FeynCalc2FORM"],
 ", ",
 ButtonBox["PaVeReduce",
  BaseStyle->"AddOnsLink",
  ButtonData:>"PaVeReduce"],
 " ,",
 ButtonBox["SimplifyPolyLog",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SimplifyPolyLog"],
 " , ",
 ButtonBox["SUNSimplify",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SUNSimplify"],
 " , ",
 ButtonBox["Isolate",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Isolate"],
 " , ",
 ButtonBox["Collect2",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Collect2"],
 " , ",
 ButtonBox["OneLoopSimplify",
  BaseStyle->"AddOnsLink",
  ButtonData:>"OneLoopSimplify"],
 ", ",
 ButtonBox["Series2",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Series2",
  ButtonNote->"Series2"],
 " , ",
 ButtonBox["ScalarProductCancel",
  BaseStyle->"AddOnsLink",
  ButtonData:>"ScalarProductCancel"],
 "."
}], "Text",
 CellTags->"intro"],

Cell["Some functions for Feyman loop integral manipulations:", "Text",
 CellTags->"intro"],

Cell[TextData[{
 ButtonBox["OneLoop",
  BaseStyle->"AddOnsLink",
  ButtonData:>"OneLoop"],
 ", ",
 ButtonBox["OneLoopSimplify",
  BaseStyle->"AddOnsLink",
  ButtonData:>"OneLoopSimplify"],
 " and ",
 ButtonBox["PaVeReduce",
  BaseStyle->"AddOnsLink",
  ButtonData:>"PaVeReduce"],
 " ."
}], "Text",
 CellTags->"intro"],

Cell[TextData[{
 "FeynCalc (>4.2) is still compatible with FeynArts (>2.2) if the FeynArts \
function ToFA1Convention is used on the generated amplitudes. While ",
 ButtonBox["OneLoop",
  BaseStyle->"AddOnsLink",
  ButtonData:>"OneLoop"],
 " and ",
 ButtonBox["OneLoopSum",
  BaseStyle->"AddOnsLink",
  ButtonData:>"OneLoopSum"],
 "  work fine for not too complicated self-energy, triangle and box diagrams, \
the reader should be aware of the package ",
 ButtonBox["FormCalc",
  BaseStyle->"Hyperlink",
  ButtonData:>{
    URL["http://www.feynarts.de/formcalc/"], None}],
 " by Thomas Hahn, which is more efficient for the adventurous researcher \
calculating thousands of diagrams, e.g. involving four external gauge bosons. \
Notice however that FormCalc does not reduce the Passarino-Veltman integrals \
to scalar integrals. Also, FormCalc is limited to the 't Hooft Feynman gauge. \
FeynCalc provides algebraic redution of Passarino-Veltman integrals to scalar \
integrals via the function ",
 ButtonBox["PaVeReduce",
  BaseStyle->"AddOnsLink",
  ButtonData:>"PaVeReduce"],
 " ."
}], "Text",
 CellTags->"intro"],

Cell["\<\
For more detailed information on FeynCalc functions, use the help system or \
consult the source code in the file \"FeynCalc.m\" in the directory \
\"HighEnergyPhysics\" and the further packages in the sub-directories \
\"fctools\", \"fctables\" and others. A complete list of FeynCalc objects is \
stored in the variable $FeynCalcStuff. Not all functions are equally well \
documented in this notebook. Some are very special ones (e.g. for tools for \
2-loop QCD diagrams originating in twist-2 OPE), some are still experimental.\
\
\>", "Text",
 CellTags->"intro"],

Cell[TextData[{
 "The ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["\[Gamma]", "5"], TraditionalForm]]],
 "scheme used in OneLoop is the naive one, i.e., ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       SuperscriptBox["\[Gamma]", "5"], ",", 
       SuperscriptBox["\[Gamma]", "\[Mu]"]}]}], ")"}], "=", "0"}], 
   TraditionalForm]]],
 " in 4 and D dimensions. See also ",
 ButtonBox["ToLarin",
  BaseStyle->"AddOnsLink",
  ButtonData:>"ToLarin"]
}], "Text",
 CellTags->"intro"],

Cell["The metric used is the one from Bjorken and Drell (+---).", "Text",
 CellTags->"intro"],

Cell[TextData[{
 "For suggestions and bug reports please see ",
 ButtonBox["http://www.feyncalc.org/",
  BaseStyle->"Hyperlink",
  ButtonData:>{
    URL["http://www.feyncalc.org/"], None}],
 "."
}], "Text",
 CellTags->"intro"],

Cell[TextData[{
 "On a 2010 computer running ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " 8.0 for Linux, this noteboook needs less than half a minute to evaluate."
}], "Text",
 CellChangeTimes->{
  3.5009698219981823`*^9, {3.50097003301435*^9, 3.500970051196999*^9}},
 CellTags->"intro"],

Cell[TextData[{
 "\[Dagger] Mellin transforms (e.g. all integrals - except 57,58 and 59, and \
correcting a minor misprint in 14) of Appendix 7 from ",
 ButtonBox["hep-ph/9810241",
  BaseStyle->"Hyperlink",
  ButtonData:>{
    URL["http://xxx.lanl.gov/abs/hep-ph/9810241"], None}],
 " are tabulated in ",
 ButtonBox["Integrate2.",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Integrate2"]
}], "SmallText",
 CellTags->{"intro", "introfoot"}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Starting FeynCalc", "Section",
 CellTags->{"Load FeynCalc", "start"}],

Cell[TextData[{
 "If the directory \"HighEnergyPhysics\" has been put in the directory \
\"Applications\" in your ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " installation tree, you should be able to load FeynCalc from a ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " session with no further ado."
}], "Text",
 CellTags->"start"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"<<", "HighEnergyPhysics`FeynCalc`"}]], "Input",
 CellTags->"start",
 CellLabel->"In[2]:="],

Cell["Loading FeynCalc from /home/rolfm/HighEnergyPhysics", "Text",
 CellMargins->{{Inherited, Inherited}, {1, 0}},
 GeneratedCell->True,
 CellAutoOverwrite->True,
 CellChangeTimes->{3.500970003985324*^9},
 CellTags->"start"],

Cell[TextData[{
 StyleBox["FeynCalc",
  FontWeight->"Bold"],
 " ",
 "8.0.0.beta1",
 " For help, type ?FeynCalc, use the built-in ",
 ButtonBox["help system",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>(FrontEndExecute[
    FrontEnd`HelpBrowserLookup["AddOns", #]]& ),
  ButtonData:>{"Short Overview", "intro"},
  ButtonNote->"Open the help browser"],
 " or visit ",
 ButtonBox["www.feyncalc.org",
  BaseStyle->"Hyperlink",
  ButtonData:>{
    URL["http://www.feyncalc.org/"], None},
  ButtonNote->"http://www.feyncalc.org/"]
}], "Text",
 CellMargins->{{Inherited, Inherited}, {1, 0}},
 GeneratedCell->True,
 CellAutoOverwrite->True,
 CellChangeTimes->{3.500970005017955*^9},
 CellTags->"start"],

Cell[TextData[{
 "Loading FeynArts, see ",
 ButtonBox["www.feynarts.de",
  BaseStyle->"Hyperlink",
  ButtonData:>{
    URL["http://www.feynarts.de/"], None},
  ButtonNote->"http://www.feynarts.de/"],
 " for documentation"
}], "Text",
 CellMargins->{{Inherited, Inherited}, {1, 0}},
 GeneratedCell->True,
 CellAutoOverwrite->True,
 CellChangeTimes->{3.500970006989773*^9},
 CellTags->"start"],

Cell["FeynArts 3.4 patched for use with FeynCalc", "Text",
 CellMargins->{{Inherited, Inherited}, {1, 0}},
 GeneratedCell->True,
 CellAutoOverwrite->True,
 CellChangeTimes->{3.500970007180572*^9},
 CellTags->"start"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"Date", "[", "]"}], ",", "$Version", ",", "$FeynCalcVersion"}], 
  "}"}]], "Input",
 CellLabel->"In[3]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "2010", ",", "12", ",", "10", ",", "12", ",", "40", ",", 
      "7.702895`7.639228965180515"}], "}"}], 
    ",", "\<\"8.0 for Linux x86 (64-bit) (November 7, 2010)\"\>", 
    ",", "\<\"8.0.0.beta1\"\>"}], "}"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500969841603587*^9, 3.500970007709732*^9},
 CellLabel->"Out[3]="]
}, Open  ]],

Cell["This loads FeynCalc.", "MathCaption",
 CellTags->"start"],

Cell[TextData[{
 "You can also put this line into your \"init.m\" file. This will cause \
automatic loading of FeynCalc on every ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " startup."
}], "Text",
 CellTags->"start"],

Cell[TextData[{
 "By loading FeynCalc the setting of the default output format type is \
automatically set to TraditionalForm. You might want to reverse this after \
quitting FeynCalc by clicking on the ",
 StyleBox["Cell", "TextNormal",
  FontWeight->"Bold"],
 " menu, then go to the ",
 StyleBox["Default Output Format Type ", "TextNormal",
  FontWeight->"Bold"],
 "item and select ",
 StyleBox["StandardForm",
  FontWeight->"Bold"],
 ", or by evaluating"
}], "Text",
 CellTags->"start"],

Cell[BoxData[
 StyleBox[
  RowBox[{"SetOptions", "[", 
   RowBox[{"$FrontEnd", ",", 
    RowBox[{"\"\<CommonDefaultFormatTypes\>\"", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{"\"\<Output\>\"", "\[Rule]", "StandardForm"}], "}"}]}]}], "]"}],
   "Input"]], "Input",
 CellTags->"start",
 CellLabel->"In[4]:="],

Cell["This sets the default output form to StandardForm.", "MathCaption",
 CellTags->"start"],

Cell[CellGroupData[{

Cell[BoxData[
 StyleBox[
  RowBox[{"B0", "[", 
   RowBox[{"p", ",", 
    RowBox[{"m1", "^", "2"}], ",", 
    RowBox[{"m2", "^", "2"}]}], "]"}], "Input"]], "Input",
 CellTags->"start",
 CellLabel->"In[5]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["B", "0"], "(", "p", ",", 
   SuperscriptBox["m1", "2"], ",", 
   SuperscriptBox["m2", "2"], ")"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.50096984170325*^9, 3.500970007938715*^9},
 CellTags->"start",
 CellLabel->"Out[5]="]
}, Open  ]],

Cell["Now FeynCalc functions, like e.g. B0, are not typeset.", "MathCaption",
 CellTags->"start"],

Cell[BoxData[
 StyleBox[
  RowBox[{"SetOptions", "[", 
   RowBox[{"$FrontEnd", ",", 
    RowBox[{"\"\<CommonDefaultFormatTypes\>\"", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{"\"\<Output\>\"", "\[Rule]", "TraditionalForm"}], "}"}]}]}], 
   "]"}], "Input"]], "Input",
 CellTags->"start",
 CellLabel->"In[6]:="],

Cell["This sets the default output form to TraditionalForm.", "MathCaption",
 CellTags->"start"],

Cell[CellGroupData[{

Cell[BoxData[
 StyleBox[
  RowBox[{"B0", "[", 
   RowBox[{"p", ",", 
    RowBox[{"m1", "^", "2"}], ",", 
    RowBox[{"m2", "^", "2"}]}], "]"}], "Input"]], "Input",
 CellTags->"start",
 CellLabel->"In[7]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["B", "0"], "(", "p", ",", 
   SuperscriptBox["m1", "2"], ",", 
   SuperscriptBox["m2", "2"], ")"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.50096984178572*^9, 3.500970008116186*^9},
 CellTags->"start",
 CellLabel->"Out[7]="]
}, Open  ]],

Cell["Now FeynCalc functions, like e.g. B0, are again typeset.", "MathCaption",
 CellTags->"start"],

Cell[BoxData[{
 RowBox[{"m1", "/:", 
  RowBox[{"MakeBoxes", "[", 
   RowBox[{"m1", ",", "TraditionalForm"}], "]"}], ":=", 
  RowBox[{"InterpretationBox", "[", 
   RowBox[{
    RowBox[{"SubscriptBox", "[", 
     RowBox[{"m", ",", "1"}], "]"}], ",", "m1"}], "]"}]}], "\n", 
 RowBox[{"m2", "/:", 
  RowBox[{"MakeBoxes", "[", 
   RowBox[{"m2", ",", "TraditionalForm"}], "]"}], ":=", 
  RowBox[{"InterpretationBox", "[", 
   RowBox[{
    RowBox[{"SubscriptBox", "[", 
     RowBox[{"m", ",", "2"}], "]"}], ",", "m2"}], "]"}]}]}], "Input",
 CellTags->"start",
 CellLabel->"In[8]:="],

Cell[TextData[{
 "This defines additional typesetting rules for m",
 StyleBox["1",
  FontFamily->"Courier"],
 " and m",
 StyleBox["2",
  FontFamily->"Courier"],
 " for the output format type TraditionalForm."
}], "MathCaption",
 CellTags->"start"],

Cell[CellGroupData[{

Cell[BoxData[
 StyleBox[
  RowBox[{"B0", "[", 
   RowBox[{"p", ",", 
    RowBox[{"m1", "^", "2"}], ",", 
    RowBox[{"m2", "^", "2"}]}], "]"}], "Input"]], "Input",
 CellTags->"start",
 CellLabel->"In[10]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["B", "0"], "(", "p", ",", 
   SuperscriptBox[
    RowBox[{"(", 
     InterpretationBox[
      SubscriptBox["Global`m", "1"],
      $CellContext`m1], ")"}], "2"], ",", 
   SuperscriptBox[
    RowBox[{"(", 
     InterpretationBox[
      SubscriptBox["Global`m", "2"],
      $CellContext`m2], ")"}], "2"], ")"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500969841808113*^9, 3.500970008146549*^9},
 CellTags->"start",
 CellLabel->"Out[10]="]
}, Open  ]],

Cell[TextData[{
 "Evaluating m",
 StyleBox["1",
  FontFamily->"Courier"],
 " and m",
 StyleBox["2",
  FontFamily->"Courier"],
 " now returns formatted TraditionalForm output."
}], "MathCaption",
 CellTags->"start"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "StandardForm"}]], "Input",
 CellTags->"start",
 CellLabel->"In[11]:="],

Cell[BoxData[
 RowBox[{"B0", "[", 
  RowBox[{"p", ",", 
   SuperscriptBox["m1", "2"], ",", 
   SuperscriptBox["m2", "2"]}], "]"}]], "Output",
 CellChangeTimes->{3.500969841849895*^9, 3.5009700081660748`*^9},
 CellTags->"start",
 CellLabel->"Out[11]//StandardForm="]
}, Open  ]],

Cell["\<\
Enforcing StandardForm gives 'raw' output, usable for copying, pasting and \
re-evaluating.\
\>", "MathCaption",
 CellTags->"start"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Extra subpackages", "Section",
 PageBreakAbove->True,
 CellTags->"About Extras"],

Cell[TextData[{
 "The directory  \"HighEnergyPhysics\" contains various files and \
subdirectories belonging to third-party packages which are not necessary to \
run 'standard' FeynCalc calculations and which do not use the autoloading \
conventions of FeynCalc. These packages are not initialized by default when \
starting FeynCalc. Loading them can be switched on and off by setting \
appropiate variables to False or True before ",
 ButtonBox["loading FeynCalc",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Load FeynCalc",
  ButtonNote->"start"],
 ". "
}], "Text",
 CellTags->"About Extras"],

Cell[BoxData[
 FormBox[
  StyleBox[
   FrameBox[
    StyleBox[GridBox[{
       {
        StyleBox["$LoadTARCER",
         FontFamily->"Courier"], 
        RowBox[{
        "Specifies", " ", "if", " ", "TARCER", " ", "should", " ", "be", " ", 
         "loaded", " ", "by", " ", "FeynCalc"}]},
       {
        StyleBox["$LoadPhi",
         FontFamily->"Courier"], 
        RowBox[{
        "Specifies", " ", "if", " ", "PHI", " ", "should", " ", "be", " ", 
         "loaded", " ", "by", " ", "FeynCalc"}]},
       {
        StyleBox["$LoadFeynArts",
         FontFamily->"Courier"], 
        RowBox[{"Specifies", " ", "if", " ", 
         SuperscriptBox["FeynArts", 
          ButtonBox["\[Dagger]",
           BaseStyle->"Hyperlink",
           ButtonData:>"extrafoot"]], " ", "should", " ", "be", " ", "loaded",
          " ", "by", " ", "FeynCalc"}]}
      },
      GridBoxAlignment->{
       "Columns" -> {Right, {Left}}, "ColumnsIndexed" -> {}, 
        "Rows" -> {{Baseline}}, "RowsIndexed" -> {}}],
     GridBoxOptions->{
     GridBoxDividers->{
      "Columns" -> {{False}}, "ColumnsIndexed" -> {}, "Rows" -> {{False}}, 
       "RowsIndexed" -> {}}}]], "2ColumnBox"], TraditionalForm]], "Text",
 Background->None,
 CellTags->"About Extras"],

Cell["Variables switching loading of extra packages on and off. ", "Caption",
 CellTags->"About Extras"],

Cell[CellGroupData[{

Cell[TextData[{
 "TARCER -\n",
 "A ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " program for the reduction of two-loop propagator integrals"
}], "Subsection",
 CellTags->"tarcer"],

Cell["R. Mertig, R. Scharf", "InspectorLevel2",
 CellTags->"tarcer"],

Cell[TextData[{
 ButtonBox["hep-ph/9801383",
  BaseStyle->"Hyperlink",
  ButtonData:>{
    URL["http://xxx.lanl.gov/abs/hep-ph/9801383"], None}],
 " on the Los Alamos preprint server"
}], "Text",
 CellDingbat->"\[GrayCircle]",
 CellTags->"tarcer"],

Cell[TextData[{
 "published in ",
 StyleBox["Comp. Phys. Comm.",
  CellDingbat->"\[GrayCircle]"],
 " ",
 StyleBox["111",
  FontWeight->"Bold"],
 " (1998) 265-173"
}], "Text",
 CellDingbat->"\[GrayCircle]",
 CellTags->"tarcer"],

Cell["Loading TARCER", "Subsubsection",
 CellTags->"tarcer"],

Cell[TextData[{
 "If FeynCalc is not needed, simply evaluate the file \"TARCER.nb\". This \
produces a file of the form \"tarcer*.mx\". Later it is then only necessary \
to load this file (e.g. ",
 StyleBox["<<\"tarcer06.mx\"",
  FontFamily->"Courier"],
 "). You can later change parameters in \"TARCER.nb\" and reevaluate it. \
Alternatively you can get a preproduced .mx file at ",
 ButtonBox["http://www.feyncalc.org/tarcer/",
  BaseStyle->"Hyperlink",
  ButtonData:>{
    URL["http://www.feyncalc.org/tarcer/"], None},
  ButtonNote->"http://www.feyncalc.org/tarcer/"],
 "."
}], "Text",
 CellTags->"tarcer"],

Cell[TextData[{
 "If instructed to do so, FeynCalc automatically loads any tarcer*.mx file \
found the directory \"Tarcer\" (if several are present, the last one found by \
",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "'s FileNames is loaded). To have FeynCalc load TARCER, evaluate"
}], "Text",
 CellTags->"tarcer"],

Cell[BoxData[
 RowBox[{
  RowBox[{"$LoadTARCER", "=", "True"}], ";"}]], "Input",
 CellTags->"tarcer",
 CellLabel->"In[12]:="],

Cell[TextData[{
 "before ",
 ButtonBox["loading FeynCalc",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Load FeynCalc",
  ButtonNote->"start"],
 ". You can also put the last input line into your \"FCConfig.m\" file."
}], "Text",
 CellTags->"tarcer"],

Cell[TextData[{
 "An example notebook is also available at ",
 ButtonBox["http://www.feyncalc.org/tarcer/",
  BaseStyle->"Hyperlink",
  ButtonData:>{
    URL["http://www.feyncalc.org/tarcer/"], None},
  ButtonNote->"http://www.feyncalc.org/tarcer/"],
 "."
}], "Text",
 CellTags->"tarcer"]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
 "PHI -\nA ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " package for ChPT calculations"
}], "Subsection",
 CellTags->"phi"],

Cell["F. J. Orellana", "InspectorLevel2",
 CellTags->"phi"],

Cell[TextData[{
 "PHI is a ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " package for manipulating covariant derivatives etc. of effective theories \
like Chiral Perturbation Theory. Moreover it provides an alternative way of \
creating coupling definitions for FeynArts. This is convenient if one needs \
to generate a large number of Feynman rules from different lagrangians."
}], "Text",
 CellTags->"phi"],

Cell["To have FeynCalc load PHI, evaluate", "Text",
 CellTags->"phi"],

Cell[BoxData[
 RowBox[{
  RowBox[{"$LoadPhi", "=", "True"}], ";"}]], "Input",
 CellTags->"phi",
 CellLabel->"In[13]:="],

Cell[TextData[{
 "before ",
 ButtonBox["loading FeynCalc",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Load FeynCalc",
  ButtonNote->"start"],
 ". You can also put this line into your \"FCConfig.m\" file."
}], "Text",
 CellTags->"phi"],

Cell[TextData[{
 "More information is available at ",
 ButtonBox["http://www.feyncalc.org/phi/",
  BaseStyle->"Hyperlink",
  ButtonData:>{
    URL["http://www.feyncalc.org/phi/"], None},
  ButtonNote->"http://www.feyncalc.org/phi/"],
 "."
}], "Text",
 CellTags->"phi"]
}, Open  ]],

Cell[CellGroupData[{

Cell["FeynArts", "Subsection",
 CellTags->"fa"],

Cell["\<\
Hagen Eck, Sepp K\[UDoubleDot]blbeck and Thomas Hahn\
\>", "InspectorLevel2",
 CellTags->"fa"],

Cell[TextData[{
 ButtonBox["hep-ph/0012260",
  BaseStyle->"Hyperlink",
  ButtonData:>{
    URL["http://xxx.lanl.gov/abs/hep-ph/0012260"], None}],
 " on the Los Alamos preprint server"
}], "Text",
 CellDingbat->"\[GrayCircle]",
 CellTags->"fa"],

Cell[TextData[{
 "old version published in ",
 StyleBox["Comp. Phys. Comm. ",
  CellDingbat->"\[GrayCircle]"],
 StyleBox["60",
  CellDingbat->"\[GrayCircle]",
  FontWeight->"Bold"],
 StyleBox[" (1990) 165",
  CellDingbat->"\[GrayCircle]"]
}], "Text",
 CellDingbat->"\[GrayCircle]",
 CellTags->"fa"],

Cell["\<\
FeynArts is a Mathematica package written by Hagen Eck, Sepp \
K\[UDoubleDot]blbeck and Thomas Hahn for the generation and visualization of \
Feynman diagrams and amplitudes. The current version is being developed by \
Thomas Hahn independently of FeynCalc. As it turns out, it can, however, be \
used with FeynCalc with only a few modifications, which are done \
automatically when it is loaded by FeynCalc.\
\>", "Text",
 CellTags->"fa"],

Cell["\<\
One use of FeynCalc and FeynArts together is to use FeynCalc to process the \
output you get from FeynArts. Another use is to use FeynCalc and PHI to \
generate the many Feynman rules needed in effective theories like ChPT and \
feed them to FeynArts.\
\>", "Text",
 CellTags->"fa"],

Cell[TextData[{
 "To have FeynArts loaded by FeynCalc, you need first of all to get FeynArts \
at ",
 ButtonBox["http://www.feynarts.de/",
  BaseStyle->"Hyperlink",
  ButtonData:>{
    URL["http://www.feynarts.de/"], None},
  ButtonNote->"Download FeynArts"],
 " and put it in the directory \"HighEnergyPhysics\". Then evaluate"
}], "Text",
 CellTags->"fa"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"$LoadPhi", "=", "True"}], ";"}], "\n", 
 RowBox[{
  RowBox[{"$LoadFeynArts", "=", "True"}], ";"}]}], "Input",
 CellTags->"fa",
 CellLabel->"In[14]:="],

Cell[TextData[{
 "before ",
 ButtonBox["loading FeynCalc",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Load FeynCalc",
  ButtonNote->"start"],
 ". You can also put these lines into your \"FCConfig.m\" file."
}], "Text",
 CellTags->"fa"],

Cell[TextData[{
 "More information about FeynArts is available at ",
 ButtonBox["http://www.feynarts.de/",
  BaseStyle->"Hyperlink",
  ButtonData:>{
    URL["http://www.feynarts.de/"], None},
  ButtonNote->"Download FeynArts"],
 ". More information about using FeynArts, FeynCalc and PHI is available at \
",
 ButtonBox["http://www.feyncalc.org/phi/",
  BaseStyle->"Hyperlink",
  ButtonData:>{
    URL["http://www.feyncalc.org/phi/"], None},
  ButtonNote->"http://www.feyncalc.org/phi/"],
 "."
}], "Text",
 CellTags->"fa"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Example Calculations", "Section",
 PageBreakAbove->True,
 CellTags->{"Example Calculations", "ex"}],

Cell[TextData[{
 "This section is under development ... Please post suggestions on ",
 ButtonBox["www.feyncalc.org/forum/",
  BaseStyle->"Hyperlink",
  ButtonData:>{
    URL["http://www.feyncalc.org/forum/"], None},
  ButtonNote->"http://www.feyncalc.org/forum/"],
 "."
}], "Commentary",
 CellTags->"ex"],

Cell[CellGroupData[{

Cell[TextData[{
 "Tree level ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"\[Gamma]", " ", "\[Gamma]"}], " ", "\[Rule]", " ", 
    RowBox[{
     SuperscriptBox["e", "+"], 
     SuperscriptBox["e", "-"]}]}], TraditionalForm]]],
 " in the standard model"
}], "Subsection",
 CellTags->"Tree level"],

Cell["Define typesetting rules for p1, p2, k1, k2:", "Text",
 CellTags->"Tree level"],

Cell["\<\
p1/: MakeBoxes[p1, fmt_] := InterpretationBox[SubscriptBox[\"p\", 1], p1];
k1/: MakeBoxes[k1, fmt_] := InterpretationBox[SubscriptBox[\"k\", 1], k1];
p2/: MakeBoxes[p2, fmt_] := InterpretationBox[SubscriptBox[\"p\", 2], p2];
k2/: MakeBoxes[k2, fmt_] := InterpretationBox[SubscriptBox[\"k\", 2], k2];\
\>", "Input",
 CellTags->"Tree level",
 CellLabel->"In[16]:="],

Cell["Define slashes of external momenta to save future typing:", "Text",
 CellTags->"Tree level"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"{", 
   RowBox[{"p1sl", ",", "p2sl", ",", "k1sl", ",", "k2sl"}], "}"}], "=", 
  RowBox[{"DiracSlash", "/@", 
   RowBox[{"{", 
    RowBox[{"p1", ",", "p2", ",", "k1", ",", "k2"}], "}"}]}]}]], "Input",
 CellTags->"Tree level",
 CellLabel->"In[20]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     FormBox["\<\"\[Gamma]\"\>",
      TraditionalForm], "\[CenterDot]", 
     FormBox[
      InterpretationBox[
       SubscriptBox["p", "1"],
       $CellContext`p1],
      TraditionalForm]}], ",", 
    RowBox[{
     FormBox["\<\"\[Gamma]\"\>",
      TraditionalForm], "\[CenterDot]", 
     FormBox[
      InterpretationBox[
       SubscriptBox["p", "2"],
       $CellContext`p2],
      TraditionalForm]}], ",", 
    RowBox[{
     FormBox["\<\"\[Gamma]\"\>",
      TraditionalForm], "\[CenterDot]", 
     FormBox[
      InterpretationBox[
       SubscriptBox["k", "1"],
       $CellContext`k1],
      TraditionalForm]}], ",", 
    RowBox[{
     FormBox["\<\"\[Gamma]\"\>",
      TraditionalForm], "\[CenterDot]", 
     FormBox[
      InterpretationBox[
       SubscriptBox["k", "2"],
       $CellContext`k2],
      TraditionalForm]}]}], "}"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500969903151366*^9, 3.5009700089356937`*^9},
 CellTags->"Tree level",
 CellLabel->"Out[20]="]
}, Open  ]],

Cell[TextData[{
 "Kinematic definitions so that only ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["p", "1"], ".", 
    SubscriptBox["k", "1"]}], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["p", "1"], ".", 
    SubscriptBox["k", "2"]}], TraditionalForm]]],
 " appear in final results. I.e., define ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubsuperscriptBox["p", "1", "2"], "=", 
    RowBox[{
     SubsuperscriptBox["p", "2", "2"], "=", " ", 
     SuperscriptBox["m", "2"]}]}], TraditionalForm]]],
 " ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubsuperscriptBox["k", "1", "2"], "=", 
    RowBox[{
     SubsuperscriptBox["k", "2", "2"], "=", " ", "0"}]}], TraditionalForm]]],
 " ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SubscriptBox["k", "1"], "\[CenterDot]", 
     SubscriptBox["k", "2"]}], " ", "=", " ", 
    RowBox[{
     RowBox[{
      SubscriptBox["p", "1"], "\[CenterDot]", 
      SubscriptBox["p", "2"]}], "+", 
     SuperscriptBox["m", "2"]}]}], TraditionalForm]]],
 " ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SubscriptBox["k", "1"], "\[CenterDot]", 
     SubscriptBox["p", "2"]}], " ", "=", " ", 
    RowBox[{
     SubscriptBox["k", "2"], "\[CenterDot]", 
     SubscriptBox["p", "1"]}]}], TraditionalForm]]],
 " ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SubscriptBox["k", "2"], "\[CenterDot]", 
     SubscriptBox["p", "2"]}], " ", "=", " ", 
    RowBox[{
     SubscriptBox["k", "1"], "\[CenterDot]", 
     SubscriptBox["p", "1"]}]}], TraditionalForm]]],
 " ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SubscriptBox["p", "1"], "\[CenterDot]", 
     SubscriptBox["p", "2"]}], "=", " ", 
    RowBox[{
     RowBox[{
      SubscriptBox["k", "2"], "\[CenterDot]", 
      SubscriptBox["p", "1"]}], "-", 
     SuperscriptBox["m", "2"], "+", 
     RowBox[{
      SubscriptBox["k", "1"], "\[CenterDot]", 
      SubscriptBox["p", "1"]}]}]}], TraditionalForm]]],
 "."
}], "Text",
 CellTags->"Tree level"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"ScalarProduct", "[", 
    RowBox[{"p1", ",", "p1"}], "]"}], "=", 
   RowBox[{
    RowBox[{"ScalarProduct", "[", 
     RowBox[{"p2", ",", "p2"}], "]"}], "=", 
    SuperscriptBox["m", "2"]}]}], ";"}]], "Input",
 CellTags->"Tree level",
 CellLabel->"In[21]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"ScalarProduct", "[", 
    RowBox[{"k1", ",", "k1"}], "]"}], "=", 
   RowBox[{
    RowBox[{"ScalarProduct", "[", 
     RowBox[{"k2", ",", "k2"}], "]"}], "=", "0"}]}], ";"}]], "Input",
 CellTags->"Tree level",
 CellLabel->"In[22]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"ScalarProduct", "[", 
    RowBox[{"k1", ",", "k2"}], "]"}], "=", 
   RowBox[{
    RowBox[{"ScalarProduct", "[", 
     RowBox[{"p1", ",", "p2"}], "]"}], "+", 
    SuperscriptBox["m", "2"]}]}], ";"}]], "Input",
 CellTags->"Tree level",
 CellLabel->"In[23]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"ScalarProduct", "[", 
    RowBox[{"k1", ",", "p2"}], "]"}], "=", 
   RowBox[{"ScalarProduct", "[", 
    RowBox[{"k2", ",", "p1"}], "]"}]}], ";"}]], "Input",
 CellTags->"Tree level",
 CellLabel->"In[24]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"ScalarProduct", "[", 
    RowBox[{"k2", ",", "p2"}], "]"}], "=", 
   RowBox[{"ScalarProduct", "[", 
    RowBox[{"k1", ",", "p1"}], "]"}]}], ";"}]], "Input",
 CellTags->"Tree level",
 CellLabel->"In[25]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"ScalarProduct", "[", 
    RowBox[{"p1", ",", "p2"}], "]"}], "=", 
   RowBox[{
    RowBox[{"ScalarProduct", "[", 
     RowBox[{"k2", ",", "p1"}], "]"}], "-", 
    SuperscriptBox["m", "2"], "+", 
    RowBox[{"ScalarProduct", "[", 
     RowBox[{"k1", ",", "p1"}], "]"}]}]}], ";"}]], "Input",
 CellTags->"Tree level",
 CellLabel->"In[26]:="],

Cell["Define the amplitudes:", "Text",
 CellTags->"Tree level"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"f1", "=", 
  RowBox[{
   RowBox[{"DiracMatrix", "[", "\[Mu]", "]"}], ".", 
   RowBox[{"(", 
    RowBox[{"p1sl", "-", "k1sl", "+", "m"}], ")"}], ".", 
   RowBox[{"DiracMatrix", "[", "\[Nu]", "]"}]}]}]], "Input",
 CellTags->"Tree level",
 CellLabel->"In[27]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], ".", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      RowBox[{
       FormBox["\<\"\[Gamma]\"\>",
        TraditionalForm], "\[CenterDot]", 
       FormBox[
        InterpretationBox[
         SubscriptBox["k", "1"],
         $CellContext`k1],
        TraditionalForm]}]}], "+", "m", "+", 
     RowBox[{
      FormBox["\<\"\[Gamma]\"\>",
       TraditionalForm], "\[CenterDot]", 
      FormBox[
       InterpretationBox[
        SubscriptBox["p", "1"],
        $CellContext`p1],
       TraditionalForm]}]}], ")"}], ".", 
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Nu]",
      TraditionalForm],
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009699034473143`*^9, 3.500970009928458*^9},
 CellTags->"Tree level",
 CellLabel->"Out[27]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"f2", "=", 
  RowBox[{
   RowBox[{"DiracMatrix", "[", "\[Nu]", "]"}], ".", 
   RowBox[{"(", 
    RowBox[{"p1sl", "-", "k2sl", "+", "m"}], ")"}], ".", 
   RowBox[{"DiracMatrix", "[", "\[Mu]", "]"}]}]}]], "Input",
 CellTags->"Tree level",
 CellLabel->"In[28]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Nu]",
      TraditionalForm],
     TraditionalForm]], ".", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      RowBox[{
       FormBox["\<\"\[Gamma]\"\>",
        TraditionalForm], "\[CenterDot]", 
       FormBox[
        InterpretationBox[
         SubscriptBox["k", "2"],
         $CellContext`k2],
        TraditionalForm]}]}], "+", "m", "+", 
     RowBox[{
      FormBox["\<\"\[Gamma]\"\>",
       TraditionalForm], "\[CenterDot]", 
      FormBox[
       InterpretationBox[
        SubscriptBox["p", "1"],
        $CellContext`p1],
       TraditionalForm]}]}], ")"}], ".", 
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009699035433893`*^9, 3.5009700100506697`*^9},
 CellTags->"Tree level",
 CellLabel->"Out[28]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"f3", "=", 
  RowBox[{
   RowBox[{"DiracMatrix", "[", "\[Nu]", "]"}], ".", 
   RowBox[{"(", 
    RowBox[{"p1sl", "-", "k1sl", "+", "m"}], ")"}], ".", 
   RowBox[{"DiracMatrix", "[", "\[Mu]", "]"}]}]}]], "Input",
 CellTags->"Tree level",
 CellLabel->"In[29]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Nu]",
      TraditionalForm],
     TraditionalForm]], ".", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      RowBox[{
       FormBox["\<\"\[Gamma]\"\>",
        TraditionalForm], "\[CenterDot]", 
       FormBox[
        InterpretationBox[
         SubscriptBox["k", "1"],
         $CellContext`k1],
        TraditionalForm]}]}], "+", "m", "+", 
     RowBox[{
      FormBox["\<\"\[Gamma]\"\>",
       TraditionalForm], "\[CenterDot]", 
      FormBox[
       InterpretationBox[
        SubscriptBox["p", "1"],
        $CellContext`p1],
       TraditionalForm]}]}], ")"}], ".", 
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009699036593027`*^9, 3.500970010153281*^9},
 CellTags->"Tree level",
 CellLabel->"Out[29]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"f4", "=", 
  RowBox[{
   RowBox[{"DiracMatrix", "[", "\[Mu]", "]"}], ".", 
   RowBox[{"(", 
    RowBox[{"p1sl", "-", "k2sl", "+", "m"}], ")"}], ".", 
   RowBox[{"DiracMatrix", "[", "\[Nu]", "]"}]}]}]], "Input",
 CellTags->"Tree level",
 CellLabel->"In[30]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], ".", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      RowBox[{
       FormBox["\<\"\[Gamma]\"\>",
        TraditionalForm], "\[CenterDot]", 
       FormBox[
        InterpretationBox[
         SubscriptBox["k", "2"],
         $CellContext`k2],
        TraditionalForm]}]}], "+", "m", "+", 
     RowBox[{
      FormBox["\<\"\[Gamma]\"\>",
       TraditionalForm], "\[CenterDot]", 
      FormBox[
       InterpretationBox[
        SubscriptBox["p", "1"],
        $CellContext`p1],
       TraditionalForm]}]}], ")"}], ".", 
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Nu]",
      TraditionalForm],
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500969903751802*^9, 3.500970010309697*^9},
 CellTags->"Tree level",
 CellLabel->"Out[30]="]
}, Open  ]],

Cell[" Calculate the four traces:", "Text",
 CellTags->"Tree level"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"(", 
   RowBox[{"p1sl", " ", "+", " ", "m"}], ")"}], " ", ".", " ", "f1", " ", ".",
   " ", 
  RowBox[{"(", 
   RowBox[{"p2sl", " ", "-", " ", "m"}], ")"}], " ", ".", " ", 
  "f3"}]], "Input",
 CellLabel->"In[31]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"(", 
    RowBox[{"m", "+", 
     RowBox[{
      FormBox["\<\"\[Gamma]\"\>",
       TraditionalForm], "\[CenterDot]", 
      FormBox[
       InterpretationBox[
        SubscriptBox["p", "1"],
        $CellContext`p1],
       TraditionalForm]}]}], ")"}], ".", 
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], ".", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      RowBox[{
       FormBox["\<\"\[Gamma]\"\>",
        TraditionalForm], "\[CenterDot]", 
       FormBox[
        InterpretationBox[
         SubscriptBox["k", "1"],
         $CellContext`k1],
        TraditionalForm]}]}], "+", "m", "+", 
     RowBox[{
      FormBox["\<\"\[Gamma]\"\>",
       TraditionalForm], "\[CenterDot]", 
      FormBox[
       InterpretationBox[
        SubscriptBox["p", "1"],
        $CellContext`p1],
       TraditionalForm]}]}], ")"}], ".", 
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Nu]",
      TraditionalForm],
     TraditionalForm]], ".", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      FormBox["\<\"\[Gamma]\"\>",
       TraditionalForm], "\[CenterDot]", 
      FormBox[
       InterpretationBox[
        SubscriptBox["p", "2"],
        $CellContext`p2],
       TraditionalForm]}], "-", "m"}], ")"}], ".", 
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Nu]",
      TraditionalForm],
     TraditionalForm]], ".", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      RowBox[{
       FormBox["\<\"\[Gamma]\"\>",
        TraditionalForm], "\[CenterDot]", 
       FormBox[
        InterpretationBox[
         SubscriptBox["k", "1"],
         $CellContext`k1],
        TraditionalForm]}]}], "+", "m", "+", 
     RowBox[{
      FormBox["\<\"\[Gamma]\"\>",
       TraditionalForm], "\[CenterDot]", 
      FormBox[
       InterpretationBox[
        SubscriptBox["p", "1"],
        $CellContext`p1],
       TraditionalForm]}]}], ")"}], ".", 
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500969903877761*^9, 3.500970010382062*^9},
 CellLabel->"Out[31]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "InputForm"}]], "Input",
 CellLabel->"In[32]:="],

Cell["\<\
(m + DiracSlash[p1]) . DiracGamma[LorentzIndex[\[Mu]]] . (m - DiracSlash[k1] \
+ DiracSlash[p1]) . 
 DiracGamma[LorentzIndex[\[Nu]]] . (-m + DiracSlash[p2]) . \
DiracGamma[LorentzIndex[\[Nu]]] . 
 (m - DiracSlash[k1] + DiracSlash[p1]) . DiracGamma[LorentzIndex[\[Mu]]]\
\>", "Output",
 CellChangeTimes->{3.50096990400445*^9, 3.500970010786232*^9},
 CellLabel->"Out[32]//InputForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "DiracTrace", "]"}]], "Input",
 CellLabel->"In[33]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"EpsContract", "\[Rule]", "False"}], ",", 
    RowBox[{"Factoring", "\[Rule]", "False"}], ",", 
    RowBox[{"FeynCalcExternal", "\[Rule]", "False"}], ",", 
    RowBox[{"Mandelstam", "\[Rule]", 
     RowBox[{"{", "}"}]}], ",", 
    RowBox[{"PairCollect", "\[Rule]", "True"}], ",", 
    RowBox[{"DiracTraceEvaluate", "\[Rule]", "False"}], ",", 
    RowBox[{"Schouten", "\[Rule]", "0"}], ",", 
    RowBox[{"LeviCivitaSign", "\[RuleDelayed]", "$LeviCivitaSign"}], ",", 
    RowBox[{"TraceOfOne", "\[Rule]", "4"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500969904093853*^9, 3.500970011088866*^9},
 CellLabel->"Out[33]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracTrace", "//", "Context"}]], "Input",
 CellLabel->"In[34]:="],

Cell[BoxData[
 FormBox["\<\"HighEnergyPhysics`fctools`DiracTrace`\"\>", 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500969904531065*^9, 3.5009700112627373`*^9},
 CellLabel->"Out[34]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "SetOptions"}]], "Input",
 CellLabel->"In[35]:="],

Cell[BoxData[
 RowBox[{
  StyleBox["\<\"\!\(\*RowBox[{\\\"SetOptions\\\", \\\"[\\\", \
RowBox[{StyleBox[\\\"s\\\", \\\"TI\\\"], \\\",\\\", \
RowBox[{SubscriptBox[StyleBox[\\\"name\\\", \\\"TI\\\"], StyleBox[\\\"1\\\", \
\\\"TR\\\"]], \\\"->\\\", SubscriptBox[StyleBox[\\\"value\\\", \\\"TI\\\"], \
StyleBox[\\\"1\\\", \\\"TR\\\"]]}], \\\",\\\", RowBox[{SubscriptBox[StyleBox[\
\\\"name\\\", \\\"TI\\\"], StyleBox[\\\"2\\\", \\\"TR\\\"]], \\\"->\\\", \
SubscriptBox[StyleBox[\\\"value\\\", \\\"TI\\\"], StyleBox[\\\"2\\\", \
\\\"TR\\\"]]}], \\\",\\\", StyleBox[\\\"\[Ellipsis]\\\", \\\"TR\\\"]}], \\\"]\
\\\"}]\) sets the specified default options for a symbol \!\(\*StyleBox[\\\"s\
\\\", \\\"TI\\\"]\). \\n\!\(\*RowBox[{\\\"SetOptions\\\", \\\"[\\\", \
RowBox[{StyleBox[\\\"stream\\\", \\\"TI\\\"], \\\",\\\", StyleBox[\\\"\
\[Ellipsis]\\\", \\\"TR\\\"]}], \\\"]\\\"}]\) or \!\(\*RowBox[{\\\"SetOptions\
\\\", \\\"[\\\", \
RowBox[{StyleBox[\\\"\\\\\\\"\\\\!\\\\(\\\\*StyleBox[\\\\\\\"name\\\\\\\",\\\\\
\\\"TI\\\\\\\"]\\\\)\\\\\\\"\\\", ShowStringCharacters->True], \\\",\\\", \
StyleBox[\\\"\[Ellipsis]\\\", \\\"TR\\\"]}], \\\"]\\\"}]\) sets options \
associated with a particular stream. \\n\!\(\*RowBox[{\\\"SetOptions\\\", \
\\\"[\\\", RowBox[{StyleBox[\\\"object\\\", \\\"TI\\\"], \\\",\\\", StyleBox[\
\\\"\[Ellipsis]\\\", \\\"TR\\\"]}], \\\"]\\\"}]\) sets options associated \
with an external object such as a NotebookObject. \"\>", "MSG"], 
  "\[NonBreakingSpace]", 
  ButtonBox[
   StyleBox["\[RightSkeleton]", "SR"],
   Active->True,
   BaseStyle->"Link",
   ButtonData->"paclet:ref/SetOptions"]}]], "Print", "PrintUsage",
 CellChangeTimes->{3.500970011870399*^9},
 CellTags->"Info3500973611-8507579"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracTrace", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{"m", "+", 
      RowBox[{"DiracGamma", "[", 
       RowBox[{"Momentum", "[", "p1", "]"}], "]"}]}], ")"}], ".", 
    RowBox[{"DiracGamma", "[", 
     RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], "]"}]}], ",", 
   RowBox[{"DiracTraceEvaluate", "->", "True"}]}], "]"}]], "Input",
 CellLabel->"In[36]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"4", " ", 
   SuperscriptBox[
    FormBox[
     InterpretationBox[
      SubscriptBox["p", "1"],
      $CellContext`p1],
     TraditionalForm], 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.50096990608287*^9, 3.500970012330339*^9},
 CellLabel->"Out[36]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TR", "@", "%"}]], "Input",
 CellLabel->"In[37]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"16", " ", 
   SuperscriptBox[
    FormBox[
     InterpretationBox[
      SubscriptBox["p", "1"],
      $CellContext`p1],
     TraditionalForm], 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500969906142499*^9, 3.500970012440536*^9},
 CellLabel->"Out[37]="]
}, Open  ]],

Cell[CellGroupData[{

Cell["tr1 = TR[(p1sl + m) . f1 . (p2sl - m) . f3]", "Input",
 CellTags->"Tree level",
 CellLabel->"In[38]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"4", " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"8", " ", 
      SuperscriptBox["m", "2"], " ", 
      RowBox[{
       FormBox[
        InterpretationBox[
         SubscriptBox["k", "1"],
         $CellContext`k1],
        TraditionalForm], "\[CenterDot]", 
       FormBox[
        InterpretationBox[
         SubscriptBox["p", "1"],
         $CellContext`p1],
        TraditionalForm]}]}], "+", 
     RowBox[{"8", " ", 
      RowBox[{
       FormBox[
        InterpretationBox[
         SubscriptBox["k", "1"],
         $CellContext`k1],
        TraditionalForm], "\[CenterDot]", 
       FormBox[
        InterpretationBox[
         SubscriptBox["p", "1"],
         $CellContext`p1],
        TraditionalForm]}], " ", 
      RowBox[{
       FormBox[
        InterpretationBox[
         SubscriptBox["k", "2"],
         $CellContext`k2],
        TraditionalForm], "\[CenterDot]", 
       FormBox[
        InterpretationBox[
         SubscriptBox["p", "1"],
         $CellContext`p1],
        TraditionalForm]}]}], "-", 
     RowBox[{"8", " ", 
      SuperscriptBox["m", "4"]}]}], ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500969906420394*^9, 3.500970012716156*^9},
 CellTags->"Tree level",
 CellLabel->"Out[38]="]
}, Open  ]],

Cell[CellGroupData[{

Cell["tr2 = Tr[(p1sl + m) . f1 . (p2sl - m) . f4]", "Input",
 CellTags->"Tree level",
 CellLabel->"In[39]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"4", " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"4", " ", 
      SuperscriptBox["m", "2"], " ", 
      RowBox[{
       FormBox[
        InterpretationBox[
         SubscriptBox["k", "1"],
         $CellContext`k1],
        TraditionalForm], "\[CenterDot]", 
       FormBox[
        InterpretationBox[
         SubscriptBox["p", "1"],
         $CellContext`p1],
        TraditionalForm]}]}], "+", 
     RowBox[{"4", " ", 
      SuperscriptBox["m", "2"], " ", 
      RowBox[{
       FormBox[
        InterpretationBox[
         SubscriptBox["k", "2"],
         $CellContext`k2],
        TraditionalForm], "\[CenterDot]", 
       FormBox[
        InterpretationBox[
         SubscriptBox["p", "1"],
         $CellContext`p1],
        TraditionalForm]}]}], "-", 
     RowBox[{"8", " ", 
      SuperscriptBox["m", "4"]}]}], ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500969906607004*^9, 3.500970012911227*^9},
 CellTags->"Tree level",
 CellLabel->"Out[39]="]
}, Open  ]],

Cell[CellGroupData[{

Cell["tr3 = Tr[(p1sl + m) . f2 . (p2sl - m) . f3]", "Input",
 CellTags->"Tree level",
 CellLabel->"In[40]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"4", " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"4", " ", 
      SuperscriptBox["m", "2"], " ", 
      RowBox[{
       FormBox[
        InterpretationBox[
         SubscriptBox["k", "1"],
         $CellContext`k1],
        TraditionalForm], "\[CenterDot]", 
       FormBox[
        InterpretationBox[
         SubscriptBox["p", "1"],
         $CellContext`p1],
        TraditionalForm]}]}], "+", 
     RowBox[{"4", " ", 
      SuperscriptBox["m", "2"], " ", 
      RowBox[{
       FormBox[
        InterpretationBox[
         SubscriptBox["k", "2"],
         $CellContext`k2],
        TraditionalForm], "\[CenterDot]", 
       FormBox[
        InterpretationBox[
         SubscriptBox["p", "1"],
         $CellContext`p1],
        TraditionalForm]}]}], "-", 
     RowBox[{"8", " ", 
      SuperscriptBox["m", "4"]}]}], ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500969906923657*^9, 3.500970013231373*^9},
 CellTags->"Tree level",
 CellLabel->"Out[40]="]
}, Open  ]],

Cell[CellGroupData[{

Cell["tr4 = Tr[(p1sl + m) . f2 . (p2sl - m) . f4]", "Input",
 CellTags->"Tree level",
 CellLabel->"In[41]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"4", " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"8", " ", 
      SuperscriptBox["m", "2"], " ", 
      RowBox[{
       FormBox[
        InterpretationBox[
         SubscriptBox["k", "2"],
         $CellContext`k2],
        TraditionalForm], "\[CenterDot]", 
       FormBox[
        InterpretationBox[
         SubscriptBox["p", "1"],
         $CellContext`p1],
        TraditionalForm]}]}], "+", 
     RowBox[{"8", " ", 
      RowBox[{
       FormBox[
        InterpretationBox[
         SubscriptBox["k", "1"],
         $CellContext`k1],
        TraditionalForm], "\[CenterDot]", 
       FormBox[
        InterpretationBox[
         SubscriptBox["p", "1"],
         $CellContext`p1],
        TraditionalForm]}], " ", 
      RowBox[{
       FormBox[
        InterpretationBox[
         SubscriptBox["k", "2"],
         $CellContext`k2],
        TraditionalForm], "\[CenterDot]", 
       FormBox[
        InterpretationBox[
         SubscriptBox["p", "1"],
         $CellContext`p1],
        TraditionalForm]}]}], "-", 
     RowBox[{"8", " ", 
      SuperscriptBox["m", "4"]}]}], ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500969907328336*^9, 3.500970013578538*^9},
 CellTags->"Tree level",
 CellLabel->"Out[41]="]
}, Open  ]],

Cell["Divide by the denominators and add everything up:", "Text",
 CellTags->"Tree level"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"matrixsq", "=", 
  RowBox[{
   FractionBox["tr1", 
    SuperscriptBox[
     RowBox[{"ScalarProduct", "[", 
      RowBox[{
       RowBox[{"2", " ", "p1"}], ",", "k1"}], "]"}], "2"]], "+", 
   FractionBox["tr4", 
    SuperscriptBox[
     RowBox[{"ScalarProduct", "[", 
      RowBox[{
       RowBox[{"2", " ", "p1"}], ",", "k2"}], "]"}], "2"]], "+", 
   FractionBox[
    RowBox[{"tr2", "+", "tr3"}], 
    RowBox[{
     RowBox[{"ScalarProduct", "[", 
      RowBox[{
       RowBox[{"2", " ", "p1"}], ",", "k1"}], "]"}], " ", 
     RowBox[{"ScalarProduct", "[", 
      RowBox[{
       RowBox[{"2", " ", "p1"}], ",", "k2"}], "]"}]}]]}]}]], "Input",
 CellTags->"Tree level",
 CellLabel->"In[42]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{"2", " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"4", " ", 
        SuperscriptBox["m", "2"], " ", 
        RowBox[{
         FormBox[
          InterpretationBox[
           SubscriptBox["k", "1"],
           $CellContext`k1],
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          InterpretationBox[
           SubscriptBox["p", "1"],
           $CellContext`p1],
          TraditionalForm]}]}], "+", 
       RowBox[{"4", " ", 
        SuperscriptBox["m", "2"], " ", 
        RowBox[{
         FormBox[
          InterpretationBox[
           SubscriptBox["k", "2"],
           $CellContext`k2],
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          InterpretationBox[
           SubscriptBox["p", "1"],
           $CellContext`p1],
          TraditionalForm]}]}], "-", 
       RowBox[{"8", " ", 
        SuperscriptBox["m", "4"]}]}], ")"}]}], 
    RowBox[{
     RowBox[{
      FormBox[
       InterpretationBox[
        SubscriptBox["k", "1"],
        $CellContext`k1],
       TraditionalForm], "\[CenterDot]", 
      FormBox[
       InterpretationBox[
        SubscriptBox["p", "1"],
        $CellContext`p1],
       TraditionalForm]}], " ", 
     RowBox[{
      FormBox[
       InterpretationBox[
        SubscriptBox["k", "2"],
        $CellContext`k2],
       TraditionalForm], "\[CenterDot]", 
      FormBox[
       InterpretationBox[
        SubscriptBox["p", "1"],
        $CellContext`p1],
       TraditionalForm]}]}]], "+", 
   FractionBox[
    RowBox[{
     RowBox[{"8", " ", 
      SuperscriptBox["m", "2"], " ", 
      RowBox[{
       FormBox[
        InterpretationBox[
         SubscriptBox["k", "1"],
         $CellContext`k1],
        TraditionalForm], "\[CenterDot]", 
       FormBox[
        InterpretationBox[
         SubscriptBox["p", "1"],
         $CellContext`p1],
        TraditionalForm]}]}], "+", 
     RowBox[{"8", " ", 
      RowBox[{
       FormBox[
        InterpretationBox[
         SubscriptBox["k", "1"],
         $CellContext`k1],
        TraditionalForm], "\[CenterDot]", 
       FormBox[
        InterpretationBox[
         SubscriptBox["p", "1"],
         $CellContext`p1],
        TraditionalForm]}], " ", 
      RowBox[{
       FormBox[
        InterpretationBox[
         SubscriptBox["k", "2"],
         $CellContext`k2],
        TraditionalForm], "\[CenterDot]", 
       FormBox[
        InterpretationBox[
         SubscriptBox["p", "1"],
         $CellContext`p1],
        TraditionalForm]}]}], "-", 
     RowBox[{"8", " ", 
      SuperscriptBox["m", "4"]}]}], 
    SuperscriptBox[
     FormBox[
      RowBox[{
       FormBox[
        InterpretationBox[
         SubscriptBox["k", "1"],
         $CellContext`k1],
        TraditionalForm], "\[CenterDot]", 
       FormBox[
        InterpretationBox[
         SubscriptBox["p", "1"],
         $CellContext`p1],
        TraditionalForm]}],
      TraditionalForm], "2"]], "+", 
   FractionBox[
    RowBox[{
     RowBox[{"8", " ", 
      SuperscriptBox["m", "2"], " ", 
      RowBox[{
       FormBox[
        InterpretationBox[
         SubscriptBox["k", "2"],
         $CellContext`k2],
        TraditionalForm], "\[CenterDot]", 
       FormBox[
        InterpretationBox[
         SubscriptBox["p", "1"],
         $CellContext`p1],
        TraditionalForm]}]}], "+", 
     RowBox[{"8", " ", 
      RowBox[{
       FormBox[
        InterpretationBox[
         SubscriptBox["k", "1"],
         $CellContext`k1],
        TraditionalForm], "\[CenterDot]", 
       FormBox[
        InterpretationBox[
         SubscriptBox["p", "1"],
         $CellContext`p1],
        TraditionalForm]}], " ", 
      RowBox[{
       FormBox[
        InterpretationBox[
         SubscriptBox["k", "2"],
         $CellContext`k2],
        TraditionalForm], "\[CenterDot]", 
       FormBox[
        InterpretationBox[
         SubscriptBox["p", "1"],
         $CellContext`p1],
        TraditionalForm]}]}], "-", 
     RowBox[{"8", " ", 
      SuperscriptBox["m", "4"]}]}], 
    SuperscriptBox[
     FormBox[
      RowBox[{
       FormBox[
        InterpretationBox[
         SubscriptBox["k", "2"],
         $CellContext`k2],
        TraditionalForm], "\[CenterDot]", 
       FormBox[
        InterpretationBox[
         SubscriptBox["p", "1"],
         $CellContext`p1],
        TraditionalForm]}],
      TraditionalForm], "2"]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500969907392905*^9, 3.50097001367155*^9},
 CellTags->"Tree level",
 CellLabel->"Out[42]="]
}, Open  ]],

Cell[TextData[{
 "Collect with respect to powers of ",
 StyleBox["m",
  FontSlant->"Italic"],
 ":"
}], "Text",
 CellTags->"Tree level"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"r", "=", 
  RowBox[{"Collect2", "[", 
   RowBox[{"matrixsq", ",", "m"}], "]"}]}]], "Input",
 CellTags->"Tree level",
 CellLabel->"In[43]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    FractionBox[
     RowBox[{"8", " ", 
      SuperscriptBox["m", "4"], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          FormBox[
           InterpretationBox[
            SubscriptBox["k", "1"],
            $CellContext`k1],
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           InterpretationBox[
            SubscriptBox["p", "1"],
            $CellContext`p1],
           TraditionalForm]}], "+", 
         RowBox[{
          FormBox[
           InterpretationBox[
            SubscriptBox["k", "2"],
            $CellContext`k2],
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           InterpretationBox[
            SubscriptBox["p", "1"],
            $CellContext`p1],
           TraditionalForm]}]}], ")"}], "2"]}], 
     RowBox[{
      SuperscriptBox[
       FormBox[
        RowBox[{
         FormBox[
          InterpretationBox[
           SubscriptBox["k", "1"],
           $CellContext`k1],
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          InterpretationBox[
           SubscriptBox["p", "1"],
           $CellContext`p1],
          TraditionalForm]}],
        TraditionalForm], "2"], " ", 
      SuperscriptBox[
       FormBox[
        RowBox[{
         FormBox[
          InterpretationBox[
           SubscriptBox["k", "2"],
           $CellContext`k2],
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          InterpretationBox[
           SubscriptBox["p", "1"],
           $CellContext`p1],
          TraditionalForm]}],
        TraditionalForm], "2"]}]]}], "+", 
   FractionBox[
    RowBox[{"16", " ", 
     SuperscriptBox["m", "2"], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        FormBox[
         InterpretationBox[
          SubscriptBox["k", "1"],
          $CellContext`k1],
         TraditionalForm], "\[CenterDot]", 
        FormBox[
         InterpretationBox[
          SubscriptBox["p", "1"],
          $CellContext`p1],
         TraditionalForm]}], "+", 
       RowBox[{
        FormBox[
         InterpretationBox[
          SubscriptBox["k", "2"],
          $CellContext`k2],
         TraditionalForm], "\[CenterDot]", 
        FormBox[
         InterpretationBox[
          SubscriptBox["p", "1"],
          $CellContext`p1],
         TraditionalForm]}]}], ")"}]}], 
    RowBox[{
     RowBox[{
      FormBox[
       InterpretationBox[
        SubscriptBox["k", "1"],
        $CellContext`k1],
       TraditionalForm], "\[CenterDot]", 
      FormBox[
       InterpretationBox[
        SubscriptBox["p", "1"],
        $CellContext`p1],
       TraditionalForm]}], " ", 
     RowBox[{
      FormBox[
       InterpretationBox[
        SubscriptBox["k", "2"],
        $CellContext`k2],
       TraditionalForm], "\[CenterDot]", 
      FormBox[
       InterpretationBox[
        SubscriptBox["p", "1"],
        $CellContext`p1],
       TraditionalForm]}]}]], "+", 
   FractionBox[
    RowBox[{"8", " ", 
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox[
        FormBox[
         RowBox[{
          FormBox[
           InterpretationBox[
            SubscriptBox["k", "1"],
            $CellContext`k1],
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           InterpretationBox[
            SubscriptBox["p", "1"],
            $CellContext`p1],
           TraditionalForm]}],
         TraditionalForm], "2"], "+", 
       SuperscriptBox[
        FormBox[
         RowBox[{
          FormBox[
           InterpretationBox[
            SubscriptBox["k", "2"],
            $CellContext`k2],
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           InterpretationBox[
            SubscriptBox["p", "1"],
            $CellContext`p1],
           TraditionalForm]}],
         TraditionalForm], "2"]}], ")"}]}], 
    RowBox[{
     RowBox[{
      FormBox[
       InterpretationBox[
        SubscriptBox["k", "1"],
        $CellContext`k1],
       TraditionalForm], "\[CenterDot]", 
      FormBox[
       InterpretationBox[
        SubscriptBox["p", "1"],
        $CellContext`p1],
       TraditionalForm]}], " ", 
     RowBox[{
      FormBox[
       InterpretationBox[
        SubscriptBox["k", "2"],
        $CellContext`k2],
       TraditionalForm], "\[CenterDot]", 
      FormBox[
       InterpretationBox[
        SubscriptBox["p", "1"],
        $CellContext`p1],
       TraditionalForm]}]}]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500969907504218*^9, 3.50097001375911*^9},
 CellTags->"Tree level",
 CellLabel->"Out[43]="]
}, Open  ]],

Cell[TextData[{
 "The symmetry with respect to ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["k", "1"], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["k", "2"], TraditionalForm]]],
 " can be checked explicitly:"
}], "Text",
 CellTags->"Tree level"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"(", 
  RowBox[{"r", " ", "-", " ", 
   RowBox[{"(", 
    RowBox[{"r", " ", "/.", " ", 
     RowBox[{"{", " ", 
      RowBox[{"k1", "\[RuleDelayed]", "k2"}], ",", " ", 
      RowBox[{"k2", "\[RuleDelayed]", "k1"}], "}"}]}], ")"}]}], 
  ")"}]], "Input",
 CellTags->"Tree level",
 CellLabel->"In[44]:="],

Cell[BoxData[
 FormBox["0", TraditionalForm]], "Output",
 CellChangeTimes->{3.5009699076730223`*^9, 3.500970013771063*^9},
 CellTags->"Tree level",
 CellLabel->"Out[44]="]
}, Open  ]],

Cell["\<\
Remove the definitions for the scalar products and for the typesetting \
definitions:\
\>", "Text",
 CellTags->"Tree level"],

Cell[BoxData[
 RowBox[{"ClearScalarProducts", ";", " ", 
  RowBox[{"Clear", "[", 
   RowBox[{"p1", ",", "p2", ",", "k1", ",", "k2", ",", "r"}], "]"}], 
  ";"}]], "Input",
 CellTags->"Tree level",
 CellLabel->"In[45]:="],

Cell[TextData[{
 "See also: ",
 ButtonBox["ClearScalarProducts",
  BaseStyle->"AddOnsLink",
  ButtonData:>"ClearScalarProducts"],
 ", ",
 ButtonBox["Collect2",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Collect2"],
 ", ",
 ButtonBox["DiracMatrix",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracMatrix"],
 ", ",
 ButtonBox["DiracSlash",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracSlash"],
 ", ",
 ButtonBox["ScalarProduct",
  BaseStyle->"AddOnsLink",
  ButtonData:>"ScalarProduct"],
 ", ",
 ButtonBox["Tr",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Tr"],
 "."
}], "Text",
 CellTags->"Tree level"]
}, Open  ]],

Cell[CellGroupData[{

Cell["A moment integral", "Subsection",
 CellTags->"A moment integral"],

Cell[TextData[{
 "This section shows how to calculate the moments of ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox[
    StyleBox[
     OverscriptBox[
      StyleBox["A",
       FontSlant->"Plain"], "^"],
     FontSlant->"Plain"], "qq", 
    RowBox[{"NS", ",", "PHYS"}]], TraditionalForm]]],
 " from ",
 ButtonBox["hep-ph/9801224",
  BaseStyle->"Hyperlink",
  ButtonData:>{
    URL["http://xxx.lanl.gov/abs/hep-ph/9801224"], None}],
 ". Notice that due to current conservation the first moment of ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox[
    StyleBox[
     OverscriptBox[
      StyleBox["A",
       FontSlant->"Plain"], "^"],
     FontSlant->"Plain"], "qq", 
    RowBox[{"NS", ",", "PHYS"}]], TraditionalForm]]],
 "should vanish."
}], "Text",
 CellTags->"A moment integral"],

Cell["First e is defined to be \[CurlyEpsilon]:", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"e", " ", "=", " ", "Epsilon"}], ";"}]], "Input",
 CellTags->"A moment integral",
 CellLabel->"In[46]:="],

Cell[TextData[{
 "Next, FORM output from the file \"jqq12.log\" is translated to FeynCalc \
syntax. The FORM file \"jqq12.log\" was generated by one of the authors of ",
 ButtonBox["hep-ph/9801224",
  BaseStyle->"Hyperlink",
  ButtonData:>{
    URL["http://xxx.lanl.gov/abs/hep-ph/9801224"], None}],
 ". "
}], "Text",
 CellTags->"A moment integral"],

Cell["\<\
FORM convenctions like ln(1-x) can be translated automatically by \
FORM2FeynCalc, but they have to be listed once in the option Replace of \
FORM2FeynCalc :\
\>", "Text",
 CellTags->"A moment integral"],

Cell[BoxData[
 RowBox[{
  RowBox[{"SetOptions", "[", 
   RowBox[{"FORM2FeynCalc", ",", 
    RowBox[{"Replace", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"\"\<ln(x)\>\"", "\[Rule]", "\"\<Log[x]\>\""}], ",", 
       RowBox[{"\"\<ln(1+x)\>\"", "\[Rule]", "\"\<Log[1+x]\>\""}], ",", 
       RowBox[{"\"\<ln(1-x)\>\"", "\[Rule]", "\"\<Log[1-x]\>\""}], ",", 
       RowBox[{"\"\<Li2(-x)\>\"", "\[Rule]", "\"\<Li2[-x]\>\""}], ",", 
       RowBox[{"\"\<Li2(1-x)\>\"", "\[Rule]", "\"\<Li2[1-x]\>\""}], ",", 
       RowBox[{"\"\<Li3(1-x)\>\"", "\[Rule]", "\"\<Li3[1-x]\>\""}], ",", 
       RowBox[{"\"\<Li3(-x)\>\"", "\[Rule]", "\"\<Li3[-x]\>\""}], ",", 
       RowBox[{"\"\<S12(1-x)\>\"", "\[Rule]", "\"\<Nielsen[1,2,1-x]\>\""}], 
       ",", 
       RowBox[{"\"\<S12(-x)\>\"", "\[Rule]", "\"\<Nielsen[1,2,-x]\>\""}], ",", 
       RowBox[{"\"\<S12(x^2)\>\"", "\[Rule]", "\"\<Nielsen[1,2,x^2]\>\""}], 
       ",", 
       RowBox[{"\"\<Z2\>\"", "\[Rule]", "\"\<Zeta2\>\""}], ",", 
       RowBox[{"\"\<Z3\>\"", "\[Rule]", "\"\<Zeta[3]\>\""}], ",", 
       RowBox[{"\"\<[1+x]^-1\>\"", "\[Rule]", "\"\<(1+x)^-1\>\""}], ",", 
       RowBox[{"\"\<[(1-x)+]^-1\>\"", "\[Rule]", "\"\<(1-x)^-1\>\""}], ",", 
       RowBox[{"\"\<[1-x]^-1\>\"", "\[Rule]", "\"\<(1-x)^-1\>\""}], ",", 
       RowBox[{"\"\<delta\>\"", "\[Rule]", "\"\<DeltaFunction[1-x]\>\""}], 
       ",", 
       RowBox[{"\"\<Ca\>\"", "\[Rule]", "\"\<CA\>\""}], ",", 
       RowBox[{"\"\<Cf\>\"", "\[Rule]", "\"\<CF\>\""}]}], "}"}]}], ",", 
    RowBox[{"Dot", "\[Rule]", "Times"}], ",", 
    RowBox[{"HoldForm", "\[Rule]", "False"}]}], "]"}], ";"}]], "Input",
 CellTags->"A moment integral",
 CellLabel->"In[47]:="],

Cell["\<\
Copy the FORM output from jqq12.log into the next cell and put \
FORM2CFeynCalc[\" \"]//ReleaseHold around it.\
\>", "Text",
 CellTags->"A moment integral"],

Cell["\<\
aqq=FORM2FeynCalc[\"

          + [(-1)^m]*e^-1*O1*Ca*Cf * (
          - 8
          + 4*x*Z2
          + 8*x
          + 8*Z2*[1+x]^-1
          - 4*Z2
          - 4*ln(x)
          - 4*ln(x)*x
          - 8*ln(x)*ln(1 + x)
          + 8*ln(x)*ln(1 + x)*x
          + 16*ln(x)*ln(1 + x)*[1+x]^-1
          + 2*ln(x)*ln(x)
          - 2*ln(x)*ln(x)*x
          - 4*ln(x)*ln(x)*[1+x]^-1
          - 8*Li2( - x)
          + 8*Li2( - x)*x
          + 16*Li2( - x)*[1+x]^-1
          )

       + [(-1)^m]*e^-1*O1*Cf^2 * (
          + 16
          - 8*x*Z2
          - 16*x
          - 16*Z2*[1+x]^-1
          + 8*Z2
          + 8*ln(x)
          + 8*ln(x)*x
          + 16*ln(x)*ln(1 + x)
          - 16*ln(x)*ln(1 + x)*x
          - 32*ln(x)*ln(1 + x)*[1+x]^-1
          - 4*ln(x)*ln(x)
          + 4*ln(x)*ln(x)*x
          + 8*ln(x)*ln(x)*[1+x]^-1
          + 16*Li2( - x)
          - 16*Li2( - x)*x
          - 32*Li2( - x)*[1+x]^-1
          )

       + [(-1)^m]*O1*Ca*Cf * (
          - 41/3
          - 6*x*Z2
          - 2*x*Z3
          + 41/3*x
          + 16/3*x^2*Z2
          + 6*Z2
          + 4*Z3*[1+x]^-1
          - 6*Z3
          - 8*ln(1 - x)
          + 8*ln(1 - x)*x
          - 8*ln(1 - x)*Li2(1 - x)
          + 8*ln(1 - x)*Li2(1 - x)*x
          + 16*ln(1 - x)*Li2(1 - x)*[1+x]^-1
          + 8*ln(1 + x)*x*Z2
          + 8*ln(1 + x)*Z2*[1+x]^-1
          - 8*ln(1 + x)*Li2(1 - x)
          + 8*ln(1 + x)*Li2(1 - x)*x
          + 16*ln(1 + x)*Li2(1 - x)*[1+x]^-1
          + 16*ln(1 + x)*Li2( - x)*x
          + 16*ln(1 + x)*Li2( - x)*[1+x]^-1
          - 37/3*ln(x)
          - 25/3*ln(x)*x
          - 4*ln(x)*ln(1 - x)
          - 4*ln(x)*ln(1 - x)*x
          - 4*ln(x)*ln(1 - x)*ln(1 - x)
          + 4*ln(x)*ln(1 - x)*ln(1 - x)*x
          + 8*ln(x)*ln(1 - x)*ln(1 - x)*[1+x]^-1
          - 8*ln(x)*ln(1 - x)*ln(1 + x)
          + 8*ln(x)*ln(1 - x)*ln(1 + x)*x
          + 16*ln(x)*ln(1 - x)*ln(1 + x)*[1+x]^-1
          + 4/3*ln(x)*ln(1 + x)*x^-1
          + 4*ln(x)*ln(1 + x)*x
          + 16/3*ln(x)*ln(1 + x)*x^2
          + 8*ln(x)*ln(1 + x)*ln(1 + x)*x
          + 8*ln(x)*ln(1 + x)*ln(1 + x)*[1+x]^-1
          - 6*ln(x)*ln(x)
          - 8/3*ln(x)*ln(x)*x^2
          + 2*ln(x)*ln(x)*ln(1 - x)
          - 2*ln(x)*ln(x)*ln(1 - x)*x
          - 4*ln(x)*ln(x)*ln(1 - x)*[1+x]^-1
          - 6*ln(x)*ln(x)*ln(1 + x)
          + 2*ln(x)*ln(x)*ln(1 + x)*x
          + 8*ln(x)*ln(x)*ln(1 + x)*[1+x]^-1
          + ln(x)*ln(x)*ln(x)
          - ln(x)*ln(x)*ln(x)*x
          - 2*ln(x)*ln(x)*ln(x)*[1+x]^-1
          + 4*ln(x)*Li2(1 - x)
          - 4*ln(x)*Li2(1 - x)*x
          - 8*ln(x)*Li2(1 - x)*[1+x]^-1
          - 4*ln(x)*Li2( - x)
          + 4*ln(x)*Li2( - x)*x
          + 8*ln(x)*Li2( - x)*[1+x]^-1
          - 4*Li2(1 - x)
          - 4*Li2(1 - x)*x
          + 4/3*Li2( - x)*x^-1
          + 4*Li2( - x)*x
          + 16/3*Li2( - x)*x^2
          + 8*Li3(1 - x)
          - 8*Li3(1 - x)*x
          - 16*Li3(1 - x)*[1+x]^-1
          - 4*Li3( - x)
          - 4*Li3( - x)*x
          + 4*S12(1 - x)
          - 4*S12(1 - x)*x
          - 8*S12(1 - x)*[1+x]^-1
          - 8*S12( - x)
          + 24*S12( - x)*x
          + 32*S12( - x)*[1+x]^-1
          + 4*S12(x^2)
          - 4*S12(x^2)*x
          - 8*S12(x^2)*[1+x]^-1
          )

       + [(-1)^m]*O1*Cf^2 * (
          + 82/3
          + 12*x*Z2
          + 4*x*Z3
          - 82/3*x
          - 32/3*x^2*Z2
          - 12*Z2
          - 8*Z3*[1+x]^-1
          + 12*Z3
          + 16*ln(1 - x)
          - 16*ln(1 - x)*x
          + 16*ln(1 - x)*Li2(1 - x)
          - 16*ln(1 - x)*Li2(1 - x)*x
          - 32*ln(1 - x)*Li2(1 - x)*[1+x]^-1
          - 16*ln(1 + x)*x*Z2
          - 16*ln(1 + x)*Z2*[1+x]^-1
          + 16*ln(1 + x)*Li2(1 - x)
          - 16*ln(1 + x)*Li2(1 - x)*x
          - 32*ln(1 + x)*Li2(1 - x)*[1+x]^-1
          - 32*ln(1 + x)*Li2( - x)*x
          - 32*ln(1 + x)*Li2( - x)*[1+x]^-1
          + 74/3*ln(x)
          + 50/3*ln(x)*x
          + 8*ln(x)*ln(1 - x)
          + 8*ln(x)*ln(1 - x)*x
          + 8*ln(x)*ln(1 - x)*ln(1 - x)
          - 8*ln(x)*ln(1 - x)*ln(1 - x)*x
          - 16*ln(x)*ln(1 - x)*ln(1 - x)*[1+x]^-1
          + 16*ln(x)*ln(1 - x)*ln(1 + x)
          - 16*ln(x)*ln(1 - x)*ln(1 + x)*x
          - 32*ln(x)*ln(1 - x)*ln(1 + x)*[1+x]^-1
          - 8/3*ln(x)*ln(1 + x)*x^-1
          - 8*ln(x)*ln(1 + x)*x
          - 32/3*ln(x)*ln(1 + x)*x^2
          - 16*ln(x)*ln(1 + x)*ln(1 + x)*x
          - 16*ln(x)*ln(1 + x)*ln(1 + x)*[1+x]^-1
          + 12*ln(x)*ln(x)
          + 16/3*ln(x)*ln(x)*x^2
          - 4*ln(x)*ln(x)*ln(1 - x)
          + 4*ln(x)*ln(x)*ln(1 - x)*x
          + 8*ln(x)*ln(x)*ln(1 - x)*[1+x]^-1
          + 12*ln(x)*ln(x)*ln(1 + x)
          - 4*ln(x)*ln(x)*ln(1 + x)*x
          - 16*ln(x)*ln(x)*ln(1 + x)*[1+x]^-1
          - 2*ln(x)*ln(x)*ln(x)
          + 2*ln(x)*ln(x)*ln(x)*x
          + 4*ln(x)*ln(x)*ln(x)*[1+x]^-1
          - 8*ln(x)*Li2(1 - x)
          + 8*ln(x)*Li2(1 - x)*x
          + 16*ln(x)*Li2(1 - x)*[1+x]^-1
          + 8*ln(x)*Li2( - x)
          - 8*ln(x)*Li2( - x)*x
          - 16*ln(x)*Li2( - x)*[1+x]^-1
          + 8*Li2(1 - x)
          + 8*Li2(1 - x)*x
          - 8/3*Li2( - x)*x^-1
          - 8*Li2( - x)*x
          - 32/3*Li2( - x)*x^2
          - 16*Li3(1 - x)
          + 16*Li3(1 - x)*x
          + 32*Li3(1 - x)*[1+x]^-1
          + 8*Li3( - x)
          + 8*Li3( - x)*x
          - 8*S12(1 - x)
          + 8*S12(1 - x)*x
          + 16*S12(1 - x)*[1+x]^-1
          + 16*S12( - x)
          - 48*S12( - x)*x
          - 64*S12( - x)*[1+x]^-1
          - 8*S12(x^2)
          + 8*S12(x^2)*x
          + 16*S12(x^2)*[1+x]^-1
          )

       + [(-1)^m]*O2*Ca*Cf * (
          + 8/3
          - 8/3*x
          - 16/3*x^2*Z2
          - 8/3*ln(x)
          + 16/3*ln(x)*x
          + 8/3*ln(x)*ln(1 + x)*x^-1
          - 8*ln(x)*ln(1 + x)*x
          - 16/3*ln(x)*ln(1 + x)*x^2
          + 8/3*ln(x)*ln(x)*x^2
          + 8/3*Li2( - x)*x^-1
          - 8*Li2( - x)*x
          - 16/3*Li2( - x)*x^2
          )

       + [(-1)^m]*O2*Cf^2 * (
          - 16/3
          + 16/3*x
          + 32/3*x^2*Z2
          + 16/3*ln(x)
          - 32/3*ln(x)*x
          - 16/3*ln(x)*ln(1 + x)*x^-1
          + 16*ln(x)*ln(1 + x)*x
          + 32/3*ln(x)*ln(1 + x)*x^2
          - 16/3*ln(x)*ln(x)*x^2
          - 16/3*Li2( - x)*x^-1
          + 16*Li2( - x)*x
          + 32/3*Li2( - x)*x^2
          )

       + e^-2*delta*O1*Ca*Cf * (
          - 22
          )

       + e^-2*delta*O1*Cf*Tf * (
          + 8
          )

       + e^-2*delta*O1*Cf^2 * (
          - 18
          + 32*Z2
          )

       + e^-2*O1*Ca*Cf * (
          + 44/3
          + 44/3*x
          - 88/3*[(1-x)+]^-1
          )

       + e^-2*O1*Cf*Tf * (
          - 16/3
          - 16/3*x
          + 32/3*[(1-x)+]^-1
          )

       + e^-2*O1*Cf^2 * (
          + 40
          + 8*x
          - 48*[(1-x)+]^-1
          + 32*ln(1 - x)
          + 32*ln(1 - x)*x
          - 64*ln(1 - x)*[(1-x)+]^-1
          - 24*ln(x)
          - 24*ln(x)*x
          + 32*ln(x)*[1-x]^-1
          )

       + e^-1*delta*O1*Ca*Cf * (
          + 325/6
          - 44/3*Z2
          - 12*Z3
          )

       + e^-1*delta*O1*Cf*Tf * (
          - 58/3
          + 16/3*Z2
          )

       + e^-1*delta*O1*Cf^2 * (
          + 87/2
          - 36*Z2
          - 8*Z3
          )

       + e^-1*O1*Ca*Cf * (
          - 158/9
          + 4*x*Z2
          + 22/9*x
          - 8*Z2*[(1-x)+]^-1
          + 4*Z2
          + 238/9*[(1-x)+]^-1
          + 44/3*ln(1 - x)
          + 44/3*ln(1 - x)*x
          - 88/3*ln(1 - x)*[(1-x)+]^-1
          + 34/3*ln(x)
          + 34/3*ln(x)*x
          - 44/3*ln(x)*[1-x]^-1
          - 2*ln(x)*ln(x)
          - 2*ln(x)*ln(x)*x
          + 4*ln(x)*ln(x)*[1-x]^-1
          )

       + e^-1*O1*Cf*Tf * (
          + 88/9
          - 56/9*x
          - 56/9*[(1-x)+]^-1
          - 16/3*ln(1 - x)
          - 16/3*ln(1 - x)*x
          + 32/3*ln(1 - x)*[(1-x)+]^-1
          - 8/3*ln(x)
          - 8/3*ln(x)*x
          + 16/3*ln(x)*[1-x]^-1
          )

       + e^-1*O1*Cf^2 * (
          - 40
          - 4*x
          + 56*[(1-x)+]^-1
          - 4*ln(1 - x)
          + 44*ln(1 - x)*x
          - 24*ln(1 - x)*[(1-x)+]^-1
          + 24*ln(1 - x)*ln(1 - x)
          + 24*ln(1 - x)*ln(1 - x)*x
          - 48*ln(1 - x)*ln(1 - x)*[(1-x)+]^-1
          + 36*ln(x)
          - 20*ln(x)*x
          - 36*ln(x)*[1-x]^-1
          - 16*ln(x)*ln(1 - x)*[1-x]^-1
          - 14*ln(x)*ln(x)
          - 14*ln(x)*ln(x)*x
          + 16*ln(x)*ln(x)*[1-x]^-1
          - 8*Li2(1 - x)
          - 8*Li2(1 - x)*x
          )

       + e^-1*O2*Ca*Cf * (
          + 20/3
          - 88/3*x
          )

       + e^-1*O2*Cf*Tf * (
          - 16/3
          + 32/3*x
          )

       + e^-1*O2*Cf^2 * (
          - 16
          - 8*x
          - 32*ln(1 - x)*x
          + 16*ln(x)*x
          )

       + delta*O1*Ca*Cf * (
          - 7081/72
          + 301/18*Z2
          + 49/5*Z2^2
          + 28*Z3
          )

       + delta*O1*Cf*Tf * (
          + 569/18
          - 46/9*Z2
          - 8*Z3
          )

       + delta*O1*Cf^2 * (
          - 541/8
          + 97/2*Z2
          - 74/5*Z2^2
          + 54*Z3
          )

       + O1*Ca*Cf * (
          + 941/27
          + 17/3*x*Z2
          - 8*x*Z3
          - 580/27*x
          - 16/3*x^2*Z2
          - 24*x^2*Z3
          - 28/3*Z2*[(1-x)+]^-1
          - 1/3*Z2
          + 12*Z3*[1-x]^-1
          + 22*Z3*[(1-x)+]^-1
          - 20*Z3
          - 670/27*[(1-x)+]^-1
          - 122/9*ln(1 - x)
          + 6*ln(1 - x)*x*Z2
          - 32/9*ln(1 - x)*x
          + 8*ln(1 - x)*x^2*Z2
          - 10*ln(1 - x)*Z2*[(1-x)+]^-1
          + 2*ln(1 - x)*Z2
          + 238/9*ln(1 - x)*[(1-x)+]^-1
          + 22/3*ln(1 - x)*ln(1 - x)
          + 22/3*ln(1 - x)*ln(1 - x)*x
          - 44/3*ln(1 - x)*ln(1 - x)*[(1-x)+]^-1
          + 2*ln(1 - x)*Li2(1 - x)
          - 2*ln(1 - x)*Li2(1 - x)*x
          + 8*ln(1 - x)*Li2(1 - x)*x^2
          - 2*ln(1 - x)*Li2(1 - x)*[1-x]^-1
          + 14/9*ln(x)
          - 2*ln(x)*x*Z2
          + 176/9*ln(x)*x
          - 8*ln(x)*x^2*Z2
          + 10*ln(x)*Z2*[1-x]^-1
          - 6*ln(x)*Z2
          + 101/9*ln(x)*[1-x]^-1
          + 28/3*ln(x)*ln(1 - x)
          + 10/3*ln(x)*ln(1 - x)*x
          - 38/3*ln(x)*ln(1 - x)*[1-x]^-1
          - 12*ln(x)*ln(1 + x)
          - 4/3*ln(x)*ln(1 + x)*x^-1
          - 16*ln(x)*ln(1 + x)*x
          - 16/3*ln(x)*ln(1 + x)*x^2
          + 47/6*ln(x)*ln(x)
          + 47/6*ln(x)*ln(x)*x
          + 8/3*ln(x)*ln(x)*x^2
          - 11/3*ln(x)*ln(x)*[1-x]^-1
          + ln(x)*ln(x)*ln(1 - x)
          - ln(x)*ln(x)*ln(1 - x)*x
          + 4*ln(x)*ln(x)*ln(1 - x)*x^2
          - ln(x)*ln(x)*ln(1 - x)*[1-x]^-1
          - ln(x)*ln(x)*ln(x)
          - ln(x)*ln(x)*ln(x)*x
          + 2*ln(x)*ln(x)*ln(x)*[1-x]^-1
          + 8*ln(x)*Li2(1 - x)
          + 16*ln(x)*Li2(1 - x)*x^2
          - 12*ln(x)*Li2(1 - x)*[1-x]^-1
          + 8*ln(x)*Li2( - x)
          - 8*ln(x)*Li2( - x)*[1-x]^-1
          - 12*Li2(1 - x)*x
          + 4*Li2(1 - x)*[1-x]^-1
          - 12*Li2( - x)
          - 4/3*Li2( - x)*x^-1
          - 16*Li2( - x)*x
          - 16/3*Li2( - x)*x^2
          - 6*Li3(1 - x)
          + 6*Li3(1 - x)*x
          - 24*Li3(1 - x)*x^2
          + 6*Li3(1 - x)*[1-x]^-1
          - 16*Li3( - x)
          + 16*Li3( - x)*[1-x]^-1
          + 14*S12(1 - x)
          - 6*S12(1 - x)*x
          + 24*S12(1 - x)*x^2
          - 14*S12(1 - x)*[1-x]^-1
          )

       + O1*Cf*Tf * (
          - 88/27
          - 4/3*x*Z2
          + 20/27*x
          + 8/3*Z2*[(1-x)+]^-1
          - 4/3*Z2
          + 128/27*[(1-x)+]^-1
          + 88/9*ln(1 - x)
          - 56/9*ln(1 - x)*x
          - 56/9*ln(1 - x)*[(1-x)+]^-1
          - 8/3*ln(1 - x)*ln(1 - x)
          - 8/3*ln(1 - x)*ln(1 - x)*x
          + 16/3*ln(1 - x)*ln(1 - x)*[(1-x)+]^-1
          + 44/9*ln(x)
          - 28/9*ln(x)*x
          - 28/9*ln(x)*[1-x]^-1
          - 8/3*ln(x)*ln(1 - x)
          - 8/3*ln(x)*ln(1 - x)*x
          + 16/3*ln(x)*ln(1 - x)*[1-x]^-1
          - 2/3*ln(x)*ln(x)
          - 2/3*ln(x)*ln(x)*x
          + 4/3*ln(x)*ln(x)*[1-x]^-1
          )

       + O1*Cf^2 * (
          + 188/3
          - 4*x*Z2
          - 24*x*Z3
          + 4/3*x
          + 32/3*x^2*Z2
          + 48*x^2*Z3
          - 4*Z2*[(1-x)+]^-1
          + 20*Z2
          - 24*Z3*[1-x]^-1
          + 24*Z3
          - 56*[(1-x)+]^-1
          - 38*ln(1 - x)
          + 8*ln(1 - x)*x*Z2
          + 38*ln(1 - x)*x
          - 16*ln(1 - x)*x^2*Z2
          - 8*ln(1 - x)*Z2*[(1-x)+]^-1
          + 8*ln(1 - x)*Z2
          + 28*ln(1 - x)*[(1-x)+]^-1
          - 13*ln(1 - x)*ln(1 - x)
          + 31*ln(1 - x)*ln(1 - x)*x
          - 6*ln(1 - x)*ln(1 - x)*[(1-x)+]^-1
          + 28/3*ln(1 - x)*ln(1 - x)*ln(1 - x)
          + 28/3*ln(1 - x)*ln(1 - x)*ln(1 - x)*x
          - 56/3*ln(1 - x)*ln(1 - x)*ln(1 - x)*[(1-x)+]^-1
          - 12*ln(1 - x)*Li2(1 - x)
          + 4*ln(1 - x)*Li2(1 - x)*x
          - 16*ln(1 - x)*Li2(1 - x)*x^2
          - 68/3*ln(x)
          - 14*ln(x)*x*Z2
          - 68/3*ln(x)*x
          + 16*ln(x)*x^2*Z2
          + 2*ln(x)*Z2
          + 44*ln(x)*[1-x]^-1
          - 6*ln(x)*ln(1 - x)
          + 34*ln(x)*ln(1 - x)*x
          - 20*ln(x)*ln(1 - x)*[1-x]^-1
          + 6*ln(x)*ln(1 - x)*ln(1 - x)
          + 6*ln(x)*ln(1 - x)*ln(1 - x)*x
          - 20*ln(x)*ln(1 - x)*ln(1 - x)*[1-x]^-1
          + 24*ln(x)*ln(1 + x)
          + 8/3*ln(x)*ln(1 + x)*x^-1
          + 32*ln(x)*ln(1 + x)*x
          + 32/3*ln(x)*ln(1 + x)*x^2
          + 11*ln(x)*ln(x)
          - 23*ln(x)*ln(x)*x
          - 16/3*ln(x)*ln(x)*x^2
          - 15*ln(x)*ln(x)*[1-x]^-1
          - 6*ln(x)*ln(x)*ln(1 - x)
          + 2*ln(x)*ln(x)*ln(1 - x)*x
          - 8*ln(x)*ln(x)*ln(1 - x)*x^2
          - 4*ln(x)*ln(x)*ln(1 - x)*[1-x]^-1
          - 5*ln(x)*ln(x)*ln(x)
          - 5*ln(x)*ln(x)*ln(x)*x
          + 16/3*ln(x)*ln(x)*ln(x)*[1-x]^-1
          - 8*ln(x)*Li2(1 - x)
          + 24*ln(x)*Li2(1 - x)*x
          - 32*ln(x)*Li2(1 - x)*x^2
          - 16*ln(x)*Li2(1 - x)*[1-x]^-1
          - 16*ln(x)*Li2( - x)
          + 16*ln(x)*Li2( - x)*[1-x]^-1
          - 24*Li2(1 - x)
          + 48*Li2(1 - x)*x
          - 4*Li2(1 - x)*[1-x]^-1
          + 24*Li2( - x)
          + 8/3*Li2( - x)*x^-1
          + 32*Li2( - x)*x
          + 32/3*Li2( - x)*x^2
          + 20*Li3(1 - x)
          - 28*Li3(1 - x)*x
          + 48*Li3(1 - x)*x^2
          + 32*Li3( - x)
          - 32*Li3( - x)*[1-x]^-1
          - 8*S12(1 - x)
          + 56*S12(1 - x)*x
          - 48*S12(1 - x)*x^2
          - 32*S12(1 - x)*[1-x]^-1
          )

       + O2*Ca*Cf * (
          - 26/9
          - 8*x*Z2
          - 24*x*Z3
          + 232/9*x
          + 16/3*x^2*Z2
          + 24*x^2*Z3
          + 4*Z2
          - 16/3*ln(1 - x)
          + 8*ln(1 - x)*x*Z2
          - 64/3*ln(1 - x)*x
          - 8*ln(1 - x)*x^2*Z2
          + 8*ln(1 - x)*Li2(1 - x)*x
          - 8*ln(1 - x)*Li2(1 - x)*x^2
          + 6*ln(x)
          - 8*ln(x)*x*Z2
          - 28*ln(x)*x
          + 8*ln(x)*x^2*Z2
          - 4*ln(x)*ln(1 - x)
          + 8*ln(x)*ln(1 - x)*x
          - 8/3*ln(x)*ln(1 + x)*x^-1
          + 8*ln(x)*ln(1 + x)*x
          + 16/3*ln(x)*ln(1 + x)*x^2
          - 8/3*ln(x)*ln(x)*x^2
          + 4*ln(x)*ln(x)*ln(1 - x)*x
          - 4*ln(x)*ln(x)*ln(1 - x)*x^2
          + 16*ln(x)*Li2(1 - x)*x
          - 16*ln(x)*Li2(1 - x)*x^2
          - 8*Li2(1 - x)
          + 16*Li2(1 - x)*x
          - 8/3*Li2( - x)*x^-1
          + 8*Li2( - x)*x
          + 16/3*Li2( - x)*x^2
          - 24*Li3(1 - x)*x
          + 24*Li3(1 - x)*x^2
          + 24*S12(1 - x)*x
          - 24*S12(1 - x)*x^2
          )

       + O2*Cf*Tf * (
          - 8/9
          - 32/9*x
          - 16/3*ln(1 - x)
          + 32/3*ln(1 - x)*x
          - 8/3*ln(x)
          + 16/3*ln(x)*x
          )

       + O2*Cf^2 * (
          - 32/3
          + 16*x*Z2
          + 48*x*Z3
          - 16/3*x
          - 32/3*x^2*Z2
          - 48*x^2*Z3
          - 8*Z2
          - 16*ln(1 - x)*x*Z2
          - 44*ln(1 - x)*x
          + 16*ln(1 - x)*x^2*Z2
          - 24*ln(1 - x)*ln(1 - x)*x
          - 16*ln(1 - x)*Li2(1 - x)*x
          + 16*ln(1 - x)*Li2(1 - x)*x^2
          - 40/3*ln(x)
          + 16*ln(x)*x*Z2
          + 68/3*ln(x)*x
          - 16*ln(x)*x^2*Z2
          + 8*ln(x)*ln(1 - x)
          - 32*ln(x)*ln(1 - x)*x
          + 16/3*ln(x)*ln(1 + x)*x^-1
          - 16*ln(x)*ln(1 + x)*x
          - 32/3*ln(x)*ln(1 + x)*x^2
          + 12*ln(x)*ln(x)*x
          + 16/3*ln(x)*ln(x)*x^2
          - 8*ln(x)*ln(x)*ln(1 - x)*x
          + 8*ln(x)*ln(x)*ln(1 - x)*x^2
          - 32*ln(x)*Li2(1 - x)*x
          + 32*ln(x)*Li2(1 - x)*x^2
          + 16*Li2(1 - x)
          - 48*Li2(1 - x)*x
          + 16/3*Li2( - x)*x^-1
          - 16*Li2( - x)*x
          - 32/3*Li2( - x)*x^2
          + 48*Li3(1 - x)*x
          - 48*Li3(1 - x)*x^2
          - 48*S12(1 - x)*x
          + 48*S12(1 - x)*x^2
          )
          
          \"]//ReleaseHold;\
\>", "Input",
 CellTags->"A moment integral",
 CellLabel->"In[48]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Short", "[", "aqq", "]"}]], "Input",
 CellTags->"A moment integral",
 CellLabel->"In[49]:="],

Cell[BoxData[
 FormBox[
  TagBox[
   RowBox[{
    FractionBox[
     RowBox[{"O1", " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"32", " ", 
         RowBox[{"\[Zeta]", "(", "2", ")"}]}], "-", "18"}], ")"}], " ", 
      RowBox[{"\[Delta]", "(", 
       FormBox[
        RowBox[{"1", "-", "x"}],
        TraditionalForm], ")"}], " ", 
      SubsuperscriptBox["C", "F", "2"]}], 
     SuperscriptBox[
      TagBox["\[CurlyEpsilon]",
       TraditionalForm], "2"]], "+", 
    RowBox[{"\[LeftSkeleton]", "29", "\[RightSkeleton]"}], "+", 
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}], "m"], " ", 
     SubscriptBox["C", "A"], " ", "O1", " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"-", "x"}], " ", 
        RowBox[{
         SuperscriptBox["log", "3"], "(", "x", ")"}]}], "-", 
       FractionBox[
        RowBox[{"2", " ", 
         RowBox[{
          SuperscriptBox["log", "3"], "(", "x", ")"}]}], 
        RowBox[{"x", "+", "1"}]], "+", 
       RowBox[{"\[LeftSkeleton]", "96", "\[RightSkeleton]"}], "+", 
       FractionBox[
        RowBox[{"4", " ", 
         TemplateBox[{"3"},
          "Zeta"]}], 
        RowBox[{"x", "+", "1"}]], "-", 
       RowBox[{"6", " ", 
        TemplateBox[{"3"},
         "Zeta"]}], "-", 
       FractionBox["41", "3"]}], ")"}], " ", 
     SubscriptBox["C", "F"]}]}],
   Short], TraditionalForm]], "Output",
 CellChangeTimes->{{3.5009699086115627`*^9, 3.500969926090261*^9}, 
   3.500970014344095*^9},
 CellTags->"A moment integral",
 CellLabel->"Out[49]//Short="]
}, Open  ]],

Cell["\<\
By default all bracketed FORM expressions are translated to a Hold[] \
expression.\
\>", "Commentary",
 CellTags->"A moment integral"],

Cell[CellGroupData[{

Cell["FORM2FeynCalc[\"-[(-1)^m]\"]", "Input",
 CellTags->"A moment integral",
 CellLabel->"In[50]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   RowBox[{"Hold", "[", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"-", "1"}], ")"}], "m"], "]"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{{3.500969908684619*^9, 3.500969926187364*^9}, 
   3.5009700148358297`*^9},
 CellTags->"A moment integral",
 CellLabel->"Out[50]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "ReleaseHold"}]], "Input",
 CellTags->"A moment integral",
 CellLabel->"In[51]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"-", "1"}], ")"}], "m"]}], TraditionalForm]], "Output",
 CellChangeTimes->{{3.500969908740872*^9, 3.500969926205913*^9}, 
   3.5009700149957943`*^9},
 CellTags->"A moment integral",
 CellLabel->"Out[51]="]
}, Open  ]],

Cell["\<\
The first moment should be zero, which is verified by Integrate2 in a \
reasonable amount of time. The variable O2 can be set to zero since it is the \
coefficient of unphysical matrix elements.\
\>", "Text",
 CellTags->"A moment integral"],

Cell[TextData[{
 "This calculates for m=1 the integral ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubsuperscriptBox["\[Integral]", "0", 
     RowBox[{"1", " "}]], " ", 
    RowBox[{
     SubsuperscriptBox[
      StyleBox[
       OverscriptBox[
        StyleBox["A",
         FontSlant->"Plain"], "^"],
       FontSlant->"Plain"], "qq", 
      RowBox[{"NS", ",", "PHYS"}]], " ", 
     RowBox[{"\[DifferentialD]", "x"}]}], " "}], TraditionalForm]]],
 ":"
}], "Text",
 CellTags->"A moment integral"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Timing", "[", 
  RowBox[{"Integrate2", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"aqq", "/.", 
      RowBox[{"m", "\[Rule]", "1"}]}], "/.", 
     RowBox[{"O2", "\[Rule]", "0"}]}], ",", 
    RowBox[{"{", 
     RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}], "]"}]], "Input",
 CellTags->"A moment integral",
 CellLabel->"In[52]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"1.4399999999999995`", ",", "0"}], "}"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{{3.500969910000259*^9, 3.5009699270429993`*^9}, 
   3.500970016597961*^9},
 CellTags->"A moment integral",
 CellLabel->"Out[52]="]
}, Open  ]],

Cell["\<\
If you know that no partial fraction decomposition has to be done then \
Integrate3 will be much faster on an expanded or collected integral:\
\>", "Text",
 CellTags->"A moment integral"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Timing", "[", 
  RowBox[{"Expand", "[", 
   RowBox[{"Integrate3", "[", 
    RowBox[{
     RowBox[{"Expand", "[", 
      RowBox[{
       RowBox[{"aqq", "/.", 
        RowBox[{"m", "\[Rule]", "1"}]}], "/.", 
       RowBox[{"O2", "\[Rule]", "0"}]}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}], "]"}], 
  "]"}]], "Input",
 CellTags->"A moment integral",
 CellLabel->"In[53]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"0.2999999999999998`", ",", "0"}], "}"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{{3.50096991069394*^9, 3.500969927562147*^9}, 
   3.5009700172938547`*^9},
 CellTags->"A moment integral",
 CellLabel->"Out[53]="]
}, Open  ]],

Cell["Define an auxiliary typesetting function:", "Text",
 CellTags->"A moment integral"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"MakeBoxes", "[", 
    RowBox[{
     RowBox[{"big", "[", "a_", "]"}], ",", " ", "f_"}], "]"}], " ", ":=", " ", 
   RowBox[{"StyleBox", "[", 
    RowBox[{
     RowBox[{"MakeBoxes", "[", 
      RowBox[{"a", ",", " ", "f"}], "]"}], ",", " ", 
     RowBox[{"FontColor", " ", "\[Rule]", " ", 
      RowBox[{"RGBColor", "[", 
       RowBox[{"0", ",", " ", "0", ",", " ", "1"}], "]"}]}], ",", " ", " ", 
     " ", 
     RowBox[{"FontSize", " ", "\[Rule]", " ", "18"}]}], "]"}]}], 
  ";"}]], "Input",
 CellTags->"A moment integral",
 CellLabel->"In[54]:="],

Cell["The second moment takes a little bit longer.", "Text",
 CellTags->"A moment integral"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Timing", "[", 
   RowBox[{"Collect2", "[", 
    RowBox[{
     RowBox[{"Integrate2", "[", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{"x", " ", "aqq"}], "/.", 
          RowBox[{"m", "\[Rule]", "2"}]}], "/.", 
         RowBox[{"O2", "\[Rule]", "0"}]}], "/.", 
        RowBox[{"O1", "\[Rule]", "1"}]}], ",", 
       RowBox[{"{", 
        RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"CA", ",", "CF", ",", "Tf"}], "}"}], ",", 
     RowBox[{"Factoring", "\[Rule]", "False"}]}], "]"}], "]"}], "/.", " ", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     RowBox[{"CA", " ", "CF"}], " ", "\[RuleDelayed]", " ", 
     RowBox[{"big", "[", " ", 
      RowBox[{"CA", " ", "CF"}], "]"}]}], ",", 
    RowBox[{
     RowBox[{"CF", "^", "2"}], "\[RuleDelayed]", 
     RowBox[{"big", "[", 
      RowBox[{"CF", "^", "2"}], "]"}]}], ",", 
    RowBox[{
     RowBox[{"a_", " ", "CF", " ", "Tf"}], " ", "\[RuleDelayed]", 
     RowBox[{"HoldForm", "[", " ", 
      RowBox[{
       RowBox[{"big", "[", 
        RowBox[{"CF", " ", "Tf"}], "]"}], " ", "a"}], "]"}]}]}], 
   "}"}]}]], "Input",
 CellTags->"A moment integral",
 CellLabel->"In[55]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"1.6000000000000005`", ",", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       StyleBox[
        RowBox[{
         SubscriptBox["C", "A"], " ", 
         SubscriptBox["C", "F"]}],
        FontSize->18,
        FontColor->RGBColor[0, 0, 1]], ")"}], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", 
         FractionBox["1166", 
          RowBox[{"27", " ", 
           TagBox["\[CurlyEpsilon]",
            TraditionalForm]}]]}], "+", 
        FractionBox["176", 
         RowBox[{"9", " ", 
          SuperscriptBox[
           TagBox["\[CurlyEpsilon]",
            TraditionalForm], "2"]}]], "-", 
        FractionBox[
         RowBox[{"44", " ", 
          RowBox[{"\[Zeta]", "(", "2", ")"}]}], "9"], "-", 
        RowBox[{"12", " ", 
         TemplateBox[{"3"},
          "Zeta"]}], "+", 
        FractionBox["12133", "162"]}], ")"}]}], "+", 
     TagBox[
      RowBox[{
       RowBox[{"(", 
        StyleBox[
         RowBox[{
          SubscriptBox["C", "F"], " ", 
          SubscriptBox["T", "f"]}],
         FontSize->18,
         FontColor->RGBColor[0, 0, 1]], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", 
          FractionBox["2086", "81"]}], "-", 
         FractionBox["64", 
          RowBox[{"9", " ", 
           SuperscriptBox[
            TagBox["\[CurlyEpsilon]",
             TraditionalForm], "2"]}]], "+", 
         FractionBox["376", 
          RowBox[{"27", " ", 
           TagBox["\[CurlyEpsilon]",
            TraditionalForm]}]], "+", 
         FractionBox[
          RowBox[{"16", " ", 
           RowBox[{"\[Zeta]", "(", "2", ")"}]}], "9"]}], ")"}]}],
      HoldForm], "+", 
     RowBox[{
      StyleBox[
       SubsuperscriptBox["C", "F", "2"],
       FontSize->18,
       FontColor->RGBColor[0, 0, 1]], " ", 
      RowBox[{"(", 
       RowBox[{
        FractionBox["752", 
         RowBox[{"27", " ", 
          TagBox["\[CurlyEpsilon]",
           TraditionalForm]}]], "-", 
        FractionBox["128", 
         RowBox[{"9", " ", 
          SuperscriptBox[
           TagBox["\[CurlyEpsilon]",
            TraditionalForm], "2"]}]], "+", 
        FractionBox[
         RowBox[{"32", " ", 
          RowBox[{"\[Zeta]", "(", "2", ")"}]}], "9"], "+", 
        RowBox[{"8", " ", 
         TemplateBox[{"3"},
          "Zeta"]}], "-", 
        FractionBox["3452", "81"]}], ")"}]}]}]}], "}"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{{3.500969912836115*^9, 3.500969928792012*^9}, 
   3.500970018992548*^9},
 CellTags->"A moment integral",
 CellLabel->"Out[55]="]
}, Open  ]],

Cell["\<\
Try to do the general Mellin transform (replace m by OPEm, which has special \
properties):\
\>", "Text",
 CellTags->"A moment integral"],

Cell[BoxData[
 RowBox[{
  RowBox[{"gqq", " ", "=", " ", 
   RowBox[{"Collect2", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{
        SuperscriptBox["x", 
         RowBox[{"OPEm", "-", "1"}]], 
        RowBox[{"(", 
         RowBox[{"aqq", "/.", 
          RowBox[{"m", "\[Rule]", "OPEm"}]}], ")"}]}], "/.", 
       RowBox[{"O2", "\[Rule]", "0"}]}], "/.", 
      RowBox[{"O1", "\[Rule]", "1"}]}], ",", "x"}], "]"}]}], ";"}]], "Input",
 CellTags->"A moment integral",
 CellLabel->"In[56]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"gqq", "//", "Length"}]], "Input",
 CellTags->"A moment integral",
 CellLabel->"In[57]:="],

Cell[BoxData[
 FormBox["102", TraditionalForm]], "Output",
 CellChangeTimes->{{3.50096991366498*^9, 3.500969929719099*^9}, 
   3.500970020135365*^9},
 CellTags->"A moment integral",
 CellLabel->"Out[57]="]
}, Open  ]],

Cell["\<\
There are 102 integrals. However, some of the Mellin transforms cannot be \
done at the moment.\
\>", "Text",
 CellTags->"A moment integral"],

Cell[BoxData[
 RowBox[{
  RowBox[{"ww", "=", 
   RowBox[{"Integrate3", "[", 
    RowBox[{"gqq", ",", 
     RowBox[{"{", 
      RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]}], ";"}]], "Input",
 CellTags->"A moment integral",
 CellLabel->"In[58]:="],

Cell["\<\
If an integral cannot be performed the head Hold[Integrate3] is substituted \
for Integrate3. Thus, in order to find the list of integrals not done you can \
do:\
\>", "Text",
 CellTags->"A moment integral"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Cases2", "[", 
  RowBox[{"ww", ",", 
   RowBox[{"Hold", "[", "Integrate3", "]"}]}], "]"}]], "Input",
 CellTags->"A moment integral",
 CellLabel->"In[59]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     RowBox[{"Hold", "[", "Integrate3", "]"}], "[", 
     RowBox[{
      RowBox[{
       SuperscriptBox["x", 
        RowBox[{"m", "-", "1"}]], " ", 
       RowBox[{
        SubscriptBox["S", 
         RowBox[{
          FormBox["1",
           TraditionalForm], 
          FormBox["2",
           TraditionalForm]}]], "(", 
        FormBox[
         SuperscriptBox["x", "2"],
         TraditionalForm], ")"}]}], ",", 
      RowBox[{"{", 
       RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}], ",", 
    RowBox[{
     RowBox[{"Hold", "[", "Integrate3", "]"}], "[", 
     RowBox[{
      RowBox[{
       SuperscriptBox["x", "m"], " ", 
       RowBox[{
        SubscriptBox["S", 
         RowBox[{
          FormBox["1",
           TraditionalForm], 
          FormBox["2",
           TraditionalForm]}]], "(", 
        FormBox[
         SuperscriptBox["x", "2"],
         TraditionalForm], ")"}]}], ",", 
      RowBox[{"{", 
       RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}], ",", 
    RowBox[{
     RowBox[{"Hold", "[", "Integrate3", "]"}], "[", 
     RowBox[{
      FractionBox[
       RowBox[{
        SuperscriptBox["x", 
         RowBox[{"m", "-", "1"}]], " ", 
        RowBox[{
         SubscriptBox["S", 
          RowBox[{
           FormBox["1",
            TraditionalForm], 
           FormBox["2",
            TraditionalForm]}]], "(", 
         FormBox[
          SuperscriptBox["x", "2"],
          TraditionalForm], ")"}]}], 
       RowBox[{"x", "+", "1"}]], ",", 
      RowBox[{"{", 
       RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}], ",", 
    RowBox[{
     RowBox[{"Hold", "[", "Integrate3", "]"}], "[", 
     RowBox[{
      FractionBox[
       RowBox[{
        SuperscriptBox["x", 
         RowBox[{"m", "-", "1"}]], " ", 
        TemplateBox[{"2",RowBox[{"1", "-", "x"}]},
         "PolyLog"], " ", 
        RowBox[{"log", "(", 
         RowBox[{"1", "-", "x"}], ")"}]}], 
       RowBox[{"x", "+", "1"}]], ",", 
      RowBox[{"{", 
       RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}], ",", 
    RowBox[{
     RowBox[{"Hold", "[", "Integrate3", "]"}], "[", 
     RowBox[{
      RowBox[{
       SuperscriptBox["x", 
        RowBox[{"m", "-", "1"}]], " ", 
       TemplateBox[{"2",RowBox[{"1", "-", "x"}]},
        "PolyLog"], " ", 
       RowBox[{"log", "(", 
        RowBox[{"x", "+", "1"}], ")"}]}], ",", 
      RowBox[{"{", 
       RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}], ",", 
    RowBox[{
     RowBox[{"Hold", "[", "Integrate3", "]"}], "[", 
     RowBox[{
      RowBox[{
       SuperscriptBox["x", "m"], " ", 
       TemplateBox[{"2",RowBox[{"1", "-", "x"}]},
        "PolyLog"], " ", 
       RowBox[{"log", "(", 
        RowBox[{"x", "+", "1"}], ")"}]}], ",", 
      RowBox[{"{", 
       RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}], ",", 
    RowBox[{
     RowBox[{"Hold", "[", "Integrate3", "]"}], "[", 
     RowBox[{
      FractionBox[
       RowBox[{
        SuperscriptBox["x", 
         RowBox[{"m", "-", "1"}]], " ", 
        TemplateBox[{"2",RowBox[{"1", "-", "x"}]},
         "PolyLog"], " ", 
        RowBox[{"log", "(", 
         RowBox[{"x", "+", "1"}], ")"}]}], 
       RowBox[{"x", "+", "1"}]], ",", 
      RowBox[{"{", 
       RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{{3.5009699144058867`*^9, 3.500969930541212*^9}, 
   3.500970020489567*^9},
 CellTags->"A moment integral",
 CellLabel->"Out[59]="]
}, Open  ]],

Cell[TextData[{
 "Change the appearance and retain only those integrals with ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["x", 
     RowBox[{"m", "-", "1"}]], ":"}], TraditionalForm]]]
}], "Text",
 CellTags->"A moment integral"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Select2", "[", 
   RowBox[{"%", ",", 
    SuperscriptBox["x", 
     RowBox[{"OPEm", "-", "1"}]]}], "]"}], "/.", " ", 
  RowBox[{
   RowBox[{
    RowBox[{"Hold", "[", "Integrate3", "]"}], "[", "z__", "]"}], 
   "\[RuleDelayed]", " ", 
   RowBox[{"HoldForm", "[", 
    RowBox[{"Integrate", "[", "z", "]"}], "]"}]}]}]], "Input",
 CellTags->"A moment integral",
 CellLabel->"In[60]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    TagBox[
     RowBox[{
      SubsuperscriptBox["\[Integral]", "0", "1"], 
      RowBox[{
       RowBox[{
        SuperscriptBox["x", 
         RowBox[{
          RowBox[{"-", "1"}], "+", "m"}]], " ", 
        RowBox[{
         SubscriptBox["S", 
          RowBox[{
           FormBox["1",
            TraditionalForm], 
           FormBox["2",
            TraditionalForm]}]], "(", 
         FormBox[
          SuperscriptBox["x", "2"],
          TraditionalForm], ")"}]}], 
       RowBox[{"\[DifferentialD]", "x"}]}]}],
     HoldForm], ",", 
    TagBox[
     RowBox[{
      SubsuperscriptBox["\[Integral]", "0", "1"], 
      RowBox[{
       FractionBox[
        RowBox[{
         SuperscriptBox["x", 
          RowBox[{
           RowBox[{"-", "1"}], "+", "m"}]], " ", 
         RowBox[{
          SubscriptBox["S", 
           RowBox[{
            FormBox["1",
             TraditionalForm], 
            FormBox["2",
             TraditionalForm]}]], "(", 
          FormBox[
           SuperscriptBox["x", "2"],
           TraditionalForm], ")"}]}], 
        RowBox[{"1", "+", "x"}]], 
       RowBox[{"\[DifferentialD]", "x"}]}]}],
     HoldForm], ",", 
    TagBox[
     RowBox[{
      SubsuperscriptBox["\[Integral]", "0", "1"], 
      RowBox[{
       FractionBox[
        RowBox[{
         SuperscriptBox["x", 
          RowBox[{
           RowBox[{"-", "1"}], "+", "m"}]], " ", 
         RowBox[{"log", "(", 
          RowBox[{"1", "-", "x"}], ")"}], " ", 
         TemplateBox[{"2",RowBox[{"1", "-", "x"}]},
          "PolyLog"]}], 
        RowBox[{"1", "+", "x"}]], 
       RowBox[{"\[DifferentialD]", "x"}]}]}],
     HoldForm], ",", 
    TagBox[
     RowBox[{
      SubsuperscriptBox["\[Integral]", "0", "1"], 
      RowBox[{
       SuperscriptBox["x", 
        RowBox[{
         RowBox[{"-", "1"}], "+", "m"}]], " ", 
       RowBox[{"log", "(", 
        RowBox[{"1", "+", "x"}], ")"}], " ", 
       TemplateBox[{"2",RowBox[{"1", "-", "x"}]},
        "PolyLog"], 
       RowBox[{"\[DifferentialD]", "x"}]}]}],
     HoldForm], ",", 
    TagBox[
     RowBox[{
      SubsuperscriptBox["\[Integral]", "0", "1"], 
      RowBox[{
       FractionBox[
        RowBox[{
         SuperscriptBox["x", 
          RowBox[{
           RowBox[{"-", "1"}], "+", "m"}]], " ", 
         RowBox[{"log", "(", 
          RowBox[{"1", "+", "x"}], ")"}], " ", 
         TemplateBox[{"2",RowBox[{"1", "-", "x"}]},
          "PolyLog"]}], 
        RowBox[{"1", "+", "x"}]], 
       RowBox[{"\[DifferentialD]", "x"}]}]}],
     HoldForm]}], "}"}], TraditionalForm]], "Output",
 CellChangeTimes->{{3.500969914545759*^9, 3.500969930629973*^9}, 
   3.5009700206575203`*^9},
 CellTags->"A moment integral",
 CellLabel->"Out[60]="]
}, Open  ]],

Cell["\<\
If you know the answers for these integrals please share the knowledge.\
\>", "Text",
 CellTags->"A moment integral"],

Cell[BoxData[
 RowBox[{"Clear", "[", 
  RowBox[{"e", ",", "ww", ",", "gqq", ",", "big", ",", "aqq"}], 
  "]"}]], "Input",
 CellTags->"A moment integral",
 CellLabel->"In[61]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"SetOptions", "[", 
   RowBox[{"FORM2FeynCalc", ",", 
    RowBox[{"Replace", "\[Rule]", 
     RowBox[{"{", "}"}]}]}], "]"}], ";"}]], "Input",
 CellTags->"A moment integral",
 CellLabel->"In[62]:="],

Cell[TextData[{
 "See also: ",
 ButtonBox["CA",
  BaseStyle->"AddOnsLink",
  ButtonData:>"CA"],
 ", ",
 ButtonBox["CF",
  BaseStyle->"AddOnsLink",
  ButtonData:>"CF"],
 ", ",
 ButtonBox["Cases2",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Cases2"],
 ", ",
 ButtonBox["Collect2",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Collect2"],
 ", ",
 ButtonBox["DeltaFunction",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DeltaFunction"],
 ", ",
 ButtonBox["Epsilon",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Epsilon"],
 ", ",
 ButtonBox["FORM2FeynCalc",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FORM2FeynCalc"],
 ", ",
 ButtonBox["Integrate2",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Integrate3"],
 ", ",
 ButtonBox["Integrate3",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Integrate3"],
 ", ",
 ButtonBox["Li2",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Li2"],
 ", ",
 ButtonBox["Li3",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Li3"],
 ", ",
 ButtonBox["Nielsen",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Nielsen"],
 ", ",
 ButtonBox["Zeta2",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Zeta2"],
 "."
}], "Text",
 CellTags->"A moment integral"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
},
ScreenStyleEnvironment->"AllBrackets",
WindowToolbars->"EditBar",
EvaluationCompletionAction->{},
CellGrouping->Manual,
WindowSize->{1077, 896},
WindowMargins->{{196, Automatic}, {Automatic, 0}},
PrintingStartingPageNumber->1,
PrintingPageRange->{Automatic, Automatic},
PageHeaders->{{Inherited, 
   Cell[
    TextData[
     ValueBox["FileName"]], "Header"], Inherited}, {Inherited, 
   Cell[
    TextData[
     ValueBox["FileName"]], "Header"], Inherited}},
PageFooters->{{Inherited, 
   Cell[
    TextData[
     ValueBox["DateLong"]], "Footer"], Inherited}, {Inherited, 
   Cell[
    TextData[
     ValueBox["DateLong"]], "Footer"], Inherited}},
PrintingOptions->{"FacingPages"->True,
"FirstPageFace"->Left,
"FirstPageFooter"->True,
"FirstPageHeader"->False,
"GraphicsPrintingFormat"->"Automatic",
"Magnification"->1.,
"PaperOrientation"->"Portrait",
"PaperSize"->{597.5625, 842.375},
"PrintCellBrackets"->False,
"PrintMultipleHorizontalPages"->False,
"PrintRegistrationMarks"->True,
"PrintSelectionHighlighting"->False,
"PrintingMargins"->{{54., 54.}, {72., 72.}}},
PrivateNotebookOptions->{"ShortBoxForm"->False},
ShowSelection->True,
TaggingRules:>{
 "MakeContentsNonDefaultOptions" -> {$CellContext`AutomaticCellTagging -> \
$CellContext`FromContent, $CellContext`PageNumbers -> 
    False, $CellContext`SaveOptions -> 
    True, $CellContext`SelectCellStyles -> {
     "Section", "Subsection"}, $CellContext`TOCCellStyles -> {
     "Contents", "Index"}}},
CellLabelAutoDelete->True,
ShowCellTags->True,
Magnification->1.,
FrontEndVersion->"8.0 for Linux x86 (64-bit) (November 7, 2010)",
StyleDefinitions->"HelpBrowser.nb"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{
 "A moment integral"->{
  Cell[63374, 2349, 71, 1, 52, "Subsection",
   CellTags->"A moment integral"],
  Cell[63448, 2352, 787, 28, 44, "Text",
   CellTags->"A moment integral"],
  Cell[64298, 2384, 139, 4, 39, "Input",
   CellTags->"A moment integral"],
  Cell[64440, 2390, 349, 9, 37, "Text",
   CellTags->"A moment integral"],
  Cell[64792, 2401, 213, 5, 37, "Text",
   CellTags->"A moment integral"],
  Cell[65008, 2408, 1679, 31, 135, "Input",
   CellTags->"A moment integral"],
  Cell[66690, 2441, 166, 4, 37, "Text",
   CellTags->"A moment integral"],
  Cell[66859, 2447, 17159, 630, 11777, "Input",
   CellTags->"A moment integral"],
  Cell[84043, 3081, 116, 3, 39, "Input",
   CellTags->"A moment integral"],
  Cell[84162, 3086, 1575, 50, 72, "Output",
   CellTags->"A moment integral"],
  Cell[85752, 3139, 144, 4, 42, "Commentary",
   CellTags->"A moment integral"],
  Cell[85921, 3147, 101, 2, 39, "Input",
   CellTags->"A moment integral"],
  Cell[86025, 3151, 330, 10, 39, "Output",
   CellTags->"A moment integral"],
  Cell[86392, 3166, 116, 3, 39, "Input",
   CellTags->"A moment integral"],
  Cell[86511, 3171, 295, 9, 39, "Output",
   CellTags->"A moment integral"],
  Cell[86821, 3183, 250, 5, 54, "Text",
   CellTags->"A moment integral"],
  Cell[87074, 3190, 498, 18, 44, "Text",
   CellTags->"A moment integral"],
  Cell[87597, 3212, 358, 11, 39, "Input",
   CellTags->"A moment integral"],
  Cell[87958, 3225, 274, 8, 39, "Output",
   CellTags->"A moment integral"],
  Cell[88247, 3236, 197, 4, 37, "Text",
   CellTags->"A moment integral"],
  Cell[88469, 3244, 442, 14, 39, "Input",
   CellTags->"A moment integral"],
  Cell[88914, 3260, 273, 8, 39, "Output",
   CellTags->"A moment integral"],
  Cell[89202, 3271, 89, 1, 37, "Text",
   CellTags->"A moment integral"],
  Cell[89294, 3274, 594, 17, 39, "Input",
   CellTags->"A moment integral"],
  Cell[89891, 3293, 92, 1, 37, "Text",
   CellTags->"A moment integral"],
  Cell[90008, 3298, 1240, 37, 59, "Input",
   CellTags->"A moment integral"],
  Cell[91251, 3337, 2598, 85, 103, "Output",
   CellTags->"A moment integral"],
  Cell[93864, 3425, 147, 4, 37, "Text",
   CellTags->"A moment integral"],
  Cell[94014, 3431, 508, 16, 43, "Input",
   CellTags->"A moment integral"],
  Cell[94547, 3451, 113, 3, 39, "Input",
   CellTags->"A moment integral"],
  Cell[94663, 3456, 205, 5, 39, "Output",
   CellTags->"A moment integral"],
  Cell[94883, 3464, 151, 4, 37, "Text",
   CellTags->"A moment integral"],
  Cell[95037, 3470, 252, 8, 39, "Input",
   CellTags->"A moment integral"],
  Cell[95292, 3480, 217, 5, 37, "Text",
   CellTags->"A moment integral"],
  Cell[95534, 3489, 180, 5, 39, "Input",
   CellTags->"A moment integral"],
  Cell[95717, 3496, 3508, 114, 147, "Output",
   CellTags->"A moment integral"],
  Cell[99240, 3613, 243, 8, 37, "Text",
   CellTags->"A moment integral"],
  Cell[99508, 3625, 416, 13, 43, "Input",
   CellTags->"A moment integral"],
  Cell[99927, 3640, 2764, 94, 105, "Output",
   CellTags->"A moment integral"],
  Cell[102706, 3737, 127, 3, 37, "Text",
   CellTags->"A moment integral"],
  Cell[102836, 3742, 177, 5, 39, "Input",
   CellTags->"A moment integral"],
  Cell[103016, 3749, 231, 7, 39, "Input",
   CellTags->"A moment integral"],
  Cell[103250, 3758, 1134, 55, 37, "Text",
   CellTags->"A moment integral"]},
 "About Extras"->{
  Cell[18389, 680, 86, 2, 77, "Section",
   PageBreakAbove->True,
   CellTags->"About Extras"],
  Cell[18478, 684, 593, 13, 71, "Text",
   CellTags->"About Extras"],
  Cell[19074, 699, 1249, 35, 88, "Text",
   CellTags->"About Extras"],
  Cell[20326, 736, 104, 1, 28, "Caption",
   CellTags->"About Extras"]},
 "ClickSave"->{
  Cell[1208, 47, 1056, 24, 49, NotebookDefault,
   CellTags->"ClickSave"]},
 "ex"->{
  Cell[27076, 991, 105, 2, 77, "Section",
   PageBreakAbove->True,
   CellTags->{"Example Calculations", "ex"}],
  Cell[27184, 995, 304, 9, 42, "Commentary",
   CellTags->"ex"]},
 "Example Calculations"->{
  Cell[27076, 991, 105, 2, 77, "Section",
   PageBreakAbove->True,
   CellTags->{"Example Calculations", "ex"}]},
 "fa"->{
  Cell[24268, 890, 47, 1, 52, "Subsection",
   CellTags->"fa"],
  Cell[24318, 893, 104, 3, 44, "InspectorLevel2",
   CellTags->"fa"],
  Cell[24425, 898, 243, 8, 37, "Text",
   CellTags->"fa"],
  Cell[24671, 908, 298, 11, 37, "Text",
   CellTags->"fa"],
  Cell[24972, 921, 449, 8, 71, "Text",
   CellTags->"fa"],
  Cell[25424, 931, 292, 6, 54, "Text",
   CellTags->"fa"],
  Cell[25719, 939, 357, 10, 37, "Text",
   CellTags->"fa"],
  Cell[26079, 951, 186, 6, 59, "Input",
   CellTags->"fa"],
  Cell[26268, 959, 234, 8, 37, "Text",
   CellTags->"fa"],
  Cell[26505, 969, 522, 16, 54, "Text",
   CellTags->"fa"]},
 "Info3500973611-8507579"->{
  Cell[43029, 1597, 1712, 29, 93, "Print",
   CellTags->"Info3500973611-8507579"]},
 "intro"->{
  Cell[2289, 75, 94, 2, 77, "Section",
   PageBreakAbove->True,
   CellTags->{"Short Overview", "intro"}],
  Cell[2386, 79, 854, 20, 99, "Text",
   CellTags->"intro"],
  Cell[3243, 101, 502, 13, 39, "Text",
   CellTags->"intro"],
  Cell[5023, 180, 73, 1, 37, "Text",
   CellTags->"intro"],
  Cell[5099, 183, 1569, 71, 54, "Text",
   CellTags->"intro"],
  Cell[6671, 256, 90, 1, 37, "Text",
   CellTags->"intro"],
  Cell[6764, 259, 317, 14, 37, "Text",
   CellTags->"intro"],
  Cell[7084, 275, 1116, 27, 105, "Text",
   CellTags->"intro"],
  Cell[8203, 304, 576, 10, 71, "Text",
   CellTags->"intro"],
  Cell[8782, 316, 524, 20, 40, "Text",
   CellTags->"intro"],
  Cell[9309, 338, 93, 1, 37, "Text",
   CellTags->"intro"],
  Cell[9405, 341, 226, 8, 37, "Text",
   CellTags->"intro"],
  Cell[9634, 351, 297, 8, 37, "Text",
   CellTags->"intro"],
  Cell[9934, 361, 437, 12, 33, "SmallText",
   CellTags->{"intro", "introfoot"}]},
 "introfoot"->{
  Cell[9934, 361, 437, 12, 33, "SmallText",
   CellTags->{"intro", "introfoot"}]},
 "Load FeynCalc"->{
  Cell[10408, 378, 75, 1, 77, "Section",
   CellTags->{"Load FeynCalc", "start"}]},
 "phi"->{
  Cell[22901, 835, 148, 6, 74, "Subsection",
   CellTags->"phi"],
  Cell[23052, 843, 59, 1, 44, "InspectorLevel2",
   CellTags->"phi"],
  Cell[23114, 846, 416, 9, 54, "Text",
   CellTags->"phi"],
  Cell[23533, 857, 69, 1, 37, "Text",
   CellTags->"phi"],
  Cell[23605, 860, 119, 4, 39, "Input",
   CellTags->"phi"],
  Cell[23727, 866, 233, 8, 37, "Text",
   CellTags->"phi"],
  Cell[23963, 876, 268, 9, 37, "Text",
   CellTags->"phi"]},
 "Short Overview"->{
  Cell[2289, 75, 94, 2, 77, "Section",
   PageBreakAbove->True,
   CellTags->{"Short Overview", "intro"}]},
 "start"->{
  Cell[10408, 378, 75, 1, 77, "Section",
   CellTags->{"Load FeynCalc", "start"}],
  Cell[10486, 381, 347, 10, 54, "Text",
   CellTags->"start"],
  Cell[10858, 395, 114, 3, 39, "Input",
   CellTags->"start"],
  Cell[10975, 400, 225, 5, 30, "Text",
   CellTags->"start"],
  Cell[11203, 407, 696, 23, 30, "Text",
   CellTags->"start"],
  Cell[11902, 432, 391, 13, 30, "Text",
   CellTags->"start"],
  Cell[12296, 447, 216, 5, 30, "Text",
   CellTags->"start"],
  Cell[13140, 478, 63, 1, 53, "MathCaption",
   CellTags->"start"],
  Cell[13206, 481, 225, 7, 37, "Text",
   CellTags->"start"],
  Cell[13434, 490, 489, 14, 56, "Text",
   CellTags->"start"],
  Cell[13926, 506, 308, 9, 39, "Input",
   CellTags->"start"],
  Cell[14237, 517, 93, 1, 53, "MathCaption",
   CellTags->"start"],
  Cell[14355, 522, 206, 7, 39, "Input",
   CellTags->"start"],
  Cell[14564, 531, 285, 8, 42, "Output",
   CellTags->"start"],
  Cell[14864, 542, 97, 1, 53, "MathCaption",
   CellTags->"start"],
  Cell[14964, 545, 312, 9, 39, "Input",
   CellTags->"start"],
  Cell[15279, 556, 96, 1, 53, "MathCaption",
   CellTags->"start"],
  Cell[15400, 561, 206, 7, 39, "Input",
   CellTags->"start"],
  Cell[15609, 570, 285, 8, 42, "Output",
   CellTags->"start"],
  Cell[15909, 581, 99, 1, 53, "MathCaption",
   CellTags->"start"],
  Cell[16011, 584, 575, 16, 59, "Input",
   CellTags->"start"],
  Cell[16589, 602, 247, 9, 53, "MathCaption",
   CellTags->"start"],
  Cell[16861, 615, 207, 7, 39, "Input",
   CellTags->"start"],
  Cell[17071, 624, 497, 16, 42, "Output",
   CellTags->"start"],
  Cell[17583, 643, 214, 9, 53, "MathCaption",
   CellTags->"start"],
  Cell[17822, 656, 105, 3, 39, "Input",
   CellTags->"start"],
  Cell[17930, 661, 265, 7, 53, "Output",
   CellTags->"start"],
  Cell[18210, 671, 142, 4, 53, "MathCaption",
   CellTags->"start"]},
 "tarcer"->{
  Cell[20455, 741, 188, 7, 74, "Subsection",
   CellTags->"tarcer"],
  Cell[20646, 750, 68, 1, 44, "InspectorLevel2",
   CellTags->"tarcer"],
  Cell[20717, 753, 247, 8, 37, "Text",
   CellTags->"tarcer"],
  Cell[20967, 763, 226, 10, 37, "Text",
   CellTags->"tarcer"],
  Cell[21196, 775, 60, 1, 47, "Subsubsection",
   CellTags->"tarcer"],
  Cell[21259, 778, 610, 15, 50, "Text",
   CellTags->"tarcer"],
  Cell[21872, 795, 324, 8, 54, "Text",
   CellTags->"tarcer"],
  Cell[22199, 805, 125, 4, 39, "Input",
   CellTags->"tarcer"],
  Cell[22327, 811, 246, 8, 37, "Text",
   CellTags->"tarcer"],
  Cell[22576, 821, 288, 9, 37, "Text",
   CellTags->"tarcer"]},
 "Tree level"->{
  Cell[27513, 1008, 303, 11, 52, "Subsection",
   CellTags->"Tree level"],
  Cell[27819, 1021, 85, 1, 37, "Text",
   CellTags->"Tree level"],
  Cell[27907, 1024, 373, 7, 96, "Input",
   CellTags->"Tree level"],
  Cell[28283, 1033, 98, 1, 37, "Text",
   CellTags->"Tree level"],
  Cell[28406, 1038, 283, 8, 39, "Input",
   CellTags->"Tree level"],
  Cell[28692, 1048, 1054, 38, 41, "Output",
   CellTags->"Tree level"],
  Cell[29761, 1089, 2023, 77, 58, "Text",
   CellTags->"Tree level"],
  Cell[31787, 1168, 304, 10, 39, "Input",
   CellTags->"Tree level"],
  Cell[32094, 1180, 278, 9, 39, "Input",
   CellTags->"Tree level"],
  Cell[32375, 1191, 304, 10, 39, "Input",
   CellTags->"Tree level"],
  Cell[32682, 1203, 252, 8, 39, "Input",
   CellTags->"Tree level"],
  Cell[32937, 1213, 252, 8, 39, "Input",
   CellTags->"Tree level"],
  Cell[33192, 1223, 384, 12, 39, "Input",
   CellTags->"Tree level"],
  Cell[33579, 1237, 63, 1, 37, "Text",
   CellTags->"Tree level"],
  Cell[33667, 1242, 283, 8, 39, "Input",
   CellTags->"Tree level"],
  Cell[33953, 1252, 936, 34, 41, "Output",
   CellTags->"Tree level"],
  Cell[34926, 1291, 283, 8, 39, "Input",
   CellTags->"Tree level"],
  Cell[35212, 1301, 938, 34, 41, "Output",
   CellTags->"Tree level"],
  Cell[36187, 1340, 283, 8, 39, "Input",
   CellTags->"Tree level"],
  Cell[36473, 1350, 936, 34, 41, "Output",
   CellTags->"Tree level"],
  Cell[37446, 1389, 283, 8, 39, "Input",
   CellTags->"Tree level"],
  Cell[37732, 1399, 934, 34, 41, "Output",
   CellTags->"Tree level"],
  Cell[38681, 1436, 68, 1, 37, "Text",
   CellTags->"Tree level"],
  Cell[46089, 1686, 109, 2, 39, "Input",
   CellTags->"Tree level"],
  Cell[46201, 1690, 1269, 45, 42, "Output",
   CellTags->"Tree level"],
  Cell[47507, 1740, 109, 2, 39, "Input",
   CellTags->"Tree level"],
  Cell[47619, 1744, 1009, 35, 42, "Output",
   CellTags->"Tree level"],
  Cell[48665, 1784, 109, 2, 39, "Input",
   CellTags->"Tree level"],
  Cell[48777, 1788, 1009, 35, 42, "Output",
   CellTags->"Tree level"],
  Cell[49823, 1828, 109, 2, 39, "Input",
   CellTags->"Tree level"],
  Cell[49935, 1832, 1269, 45, 42, "Output",
   CellTags->"Tree level"],
  Cell[51219, 1880, 90, 1, 37, "Text",
   CellTags->"Tree level"],
  Cell[51334, 1885, 714, 23, 66, "Input",
   CellTags->"Tree level"],
  Cell[52051, 1910, 4558, 168, 66, "Output",
   CellTags->"Tree level"],
  Cell[56624, 2081, 135, 6, 37, "Text",
   CellTags->"Tree level"],
  Cell[56784, 2091, 164, 5, 39, "Input",
   CellTags->"Tree level"],
  Cell[56951, 2098, 4602, 168, 67, "Output",
   CellTags->"Tree level"],
  Cell[61568, 2269, 273, 11, 37, "Text",
   CellTags->"Tree level"],
  Cell[61866, 2284, 324, 10, 39, "Input",
   CellTags->"Tree level"],
  Cell[62193, 2296, 171, 4, 39, "Output",
   CellTags->"Tree level"],
  Cell[62379, 2303, 134, 4, 37, "Text",
   CellTags->"Tree level"],
  Cell[62516, 2309, 219, 6, 39, "Input",
   CellTags->"Tree level"],
  Cell[62738, 2317, 599, 27, 37, "Text",
   CellTags->"Tree level"]}
 }
*)
(*CellTagsIndex
CellTagsIndex->{
 {"A moment integral", 106177, 3873},
 {"About Extras", 109504, 3960},
 {"ClickSave", 109830, 3970},
 {"ex", 109915, 3973},
 {"Example Calculations", 110121, 3979},
 {"fa", 110244, 3983},
 {"Info3500973611-8507579", 110884, 4004},
 {"intro", 110980, 4007},
 {"introfoot", 111924, 4037},
 {"Load FeynCalc", 112027, 4040},
 {"phi", 112121, 4043},
 {"Short Overview", 112579, 4058},
 {"start", 112699, 4062},
 {"tarcer", 114543, 4119},
 {"Tree level", 115218, 4140}
 }
*)
(*NotebookFileOutline
Notebook[{
Cell[CellGroupData[{
Cell[567, 22, 95, 2, 103, "Title"],
Cell[665, 26, 87, 1, 64, "Subtitle"],
Cell[755, 29, 120, 3, 64, "Subtitle",
 PageBreakBelow->True],
Cell[878, 34, 80, 1, 52, "Subsubtitle"],
Cell[961, 37, 244, 8, 52, "Subsubtitle"],
Cell[1208, 47, 1056, 24, 49, NotebookDefault,
 CellTags->"ClickSave"],
Cell[CellGroupData[{
Cell[2289, 75, 94, 2, 77, "Section",
 PageBreakAbove->True,
 CellTags->{"Short Overview", "intro"}],
Cell[2386, 79, 854, 20, 99, "Text",
 CellTags->"intro"],
Cell[3243, 101, 502, 13, 39, "Text",
 CellTags->"intro"],
Cell[3748, 116, 1272, 62, 24, "Text"],
Cell[5023, 180, 73, 1, 37, "Text",
 CellTags->"intro"],
Cell[5099, 183, 1569, 71, 54, "Text",
 CellTags->"intro"],
Cell[6671, 256, 90, 1, 37, "Text",
 CellTags->"intro"],
Cell[6764, 259, 317, 14, 37, "Text",
 CellTags->"intro"],
Cell[7084, 275, 1116, 27, 105, "Text",
 CellTags->"intro"],
Cell[8203, 304, 576, 10, 71, "Text",
 CellTags->"intro"],
Cell[8782, 316, 524, 20, 40, "Text",
 CellTags->"intro"],
Cell[9309, 338, 93, 1, 37, "Text",
 CellTags->"intro"],
Cell[9405, 341, 226, 8, 37, "Text",
 CellTags->"intro"],
Cell[9634, 351, 297, 8, 37, "Text",
 CellTags->"intro"],
Cell[9934, 361, 437, 12, 33, "SmallText",
 CellTags->{"intro", "introfoot"}]
}, Open  ]],
Cell[CellGroupData[{
Cell[10408, 378, 75, 1, 77, "Section",
 CellTags->{"Load FeynCalc", "start"}],
Cell[10486, 381, 347, 10, 54, "Text",
 CellTags->"start"],
Cell[CellGroupData[{
Cell[10858, 395, 114, 3, 39, "Input",
 CellTags->"start"],
Cell[10975, 400, 225, 5, 30, "Text",
 CellTags->"start"],
Cell[11203, 407, 696, 23, 30, "Text",
 CellTags->"start"],
Cell[11902, 432, 391, 13, 30, "Text",
 CellTags->"start"],
Cell[12296, 447, 216, 5, 30, "Text",
 CellTags->"start"]
}, Open  ]],
Cell[CellGroupData[{
Cell[12549, 457, 157, 5, 26, "Input"],
Cell[12709, 464, 416, 11, 26, "Output"]
}, Open  ]],
Cell[13140, 478, 63, 1, 53, "MathCaption",
 CellTags->"start"],
Cell[13206, 481, 225, 7, 37, "Text",
 CellTags->"start"],
Cell[13434, 490, 489, 14, 56, "Text",
 CellTags->"start"],
Cell[13926, 506, 308, 9, 39, "Input",
 CellTags->"start"],
Cell[14237, 517, 93, 1, 53, "MathCaption",
 CellTags->"start"],
Cell[CellGroupData[{
Cell[14355, 522, 206, 7, 39, "Input",
 CellTags->"start"],
Cell[14564, 531, 285, 8, 42, "Output",
 CellTags->"start"]
}, Open  ]],
Cell[14864, 542, 97, 1, 53, "MathCaption",
 CellTags->"start"],
Cell[14964, 545, 312, 9, 39, "Input",
 CellTags->"start"],
Cell[15279, 556, 96, 1, 53, "MathCaption",
 CellTags->"start"],
Cell[CellGroupData[{
Cell[15400, 561, 206, 7, 39, "Input",
 CellTags->"start"],
Cell[15609, 570, 285, 8, 42, "Output",
 CellTags->"start"]
}, Open  ]],
Cell[15909, 581, 99, 1, 53, "MathCaption",
 CellTags->"start"],
Cell[16011, 584, 575, 16, 59, "Input",
 CellTags->"start"],
Cell[16589, 602, 247, 9, 53, "MathCaption",
 CellTags->"start"],
Cell[CellGroupData[{
Cell[16861, 615, 207, 7, 39, "Input",
 CellTags->"start"],
Cell[17071, 624, 497, 16, 42, "Output",
 CellTags->"start"]
}, Open  ]],
Cell[17583, 643, 214, 9, 53, "MathCaption",
 CellTags->"start"],
Cell[CellGroupData[{
Cell[17822, 656, 105, 3, 39, "Input",
 CellTags->"start"],
Cell[17930, 661, 265, 7, 53, "Output",
 CellTags->"start"]
}, Open  ]],
Cell[18210, 671, 142, 4, 53, "MathCaption",
 CellTags->"start"]
}, Open  ]],
Cell[CellGroupData[{
Cell[18389, 680, 86, 2, 77, "Section",
 PageBreakAbove->True,
 CellTags->"About Extras"],
Cell[18478, 684, 593, 13, 71, "Text",
 CellTags->"About Extras"],
Cell[19074, 699, 1249, 35, 88, "Text",
 CellTags->"About Extras"],
Cell[20326, 736, 104, 1, 28, "Caption",
 CellTags->"About Extras"],
Cell[CellGroupData[{
Cell[20455, 741, 188, 7, 74, "Subsection",
 CellTags->"tarcer"],
Cell[20646, 750, 68, 1, 44, "InspectorLevel2",
 CellTags->"tarcer"],
Cell[20717, 753, 247, 8, 37, "Text",
 CellTags->"tarcer"],
Cell[20967, 763, 226, 10, 37, "Text",
 CellTags->"tarcer"],
Cell[21196, 775, 60, 1, 47, "Subsubsection",
 CellTags->"tarcer"],
Cell[21259, 778, 610, 15, 50, "Text",
 CellTags->"tarcer"],
Cell[21872, 795, 324, 8, 54, "Text",
 CellTags->"tarcer"],
Cell[22199, 805, 125, 4, 39, "Input",
 CellTags->"tarcer"],
Cell[22327, 811, 246, 8, 37, "Text",
 CellTags->"tarcer"],
Cell[22576, 821, 288, 9, 37, "Text",
 CellTags->"tarcer"]
}, Open  ]],
Cell[CellGroupData[{
Cell[22901, 835, 148, 6, 74, "Subsection",
 CellTags->"phi"],
Cell[23052, 843, 59, 1, 44, "InspectorLevel2",
 CellTags->"phi"],
Cell[23114, 846, 416, 9, 54, "Text",
 CellTags->"phi"],
Cell[23533, 857, 69, 1, 37, "Text",
 CellTags->"phi"],
Cell[23605, 860, 119, 4, 39, "Input",
 CellTags->"phi"],
Cell[23727, 866, 233, 8, 37, "Text",
 CellTags->"phi"],
Cell[23963, 876, 268, 9, 37, "Text",
 CellTags->"phi"]
}, Open  ]],
Cell[CellGroupData[{
Cell[24268, 890, 47, 1, 52, "Subsection",
 CellTags->"fa"],
Cell[24318, 893, 104, 3, 44, "InspectorLevel2",
 CellTags->"fa"],
Cell[24425, 898, 243, 8, 37, "Text",
 CellTags->"fa"],
Cell[24671, 908, 298, 11, 37, "Text",
 CellTags->"fa"],
Cell[24972, 921, 449, 8, 71, "Text",
 CellTags->"fa"],
Cell[25424, 931, 292, 6, 54, "Text",
 CellTags->"fa"],
Cell[25719, 939, 357, 10, 37, "Text",
 CellTags->"fa"],
Cell[26079, 951, 186, 6, 59, "Input",
 CellTags->"fa"],
Cell[26268, 959, 234, 8, 37, "Text",
 CellTags->"fa"],
Cell[26505, 969, 522, 16, 54, "Text",
 CellTags->"fa"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[27076, 991, 105, 2, 77, "Section",
 PageBreakAbove->True,
 CellTags->{"Example Calculations", "ex"}],
Cell[27184, 995, 304, 9, 42, "Commentary",
 CellTags->"ex"],
Cell[CellGroupData[{
Cell[27513, 1008, 303, 11, 52, "Subsection",
 CellTags->"Tree level"],
Cell[27819, 1021, 85, 1, 37, "Text",
 CellTags->"Tree level"],
Cell[27907, 1024, 373, 7, 96, "Input",
 CellTags->"Tree level"],
Cell[28283, 1033, 98, 1, 37, "Text",
 CellTags->"Tree level"],
Cell[CellGroupData[{
Cell[28406, 1038, 283, 8, 39, "Input",
 CellTags->"Tree level"],
Cell[28692, 1048, 1054, 38, 41, "Output",
 CellTags->"Tree level"]
}, Open  ]],
Cell[29761, 1089, 2023, 77, 58, "Text",
 CellTags->"Tree level"],
Cell[31787, 1168, 304, 10, 39, "Input",
 CellTags->"Tree level"],
Cell[32094, 1180, 278, 9, 39, "Input",
 CellTags->"Tree level"],
Cell[32375, 1191, 304, 10, 39, "Input",
 CellTags->"Tree level"],
Cell[32682, 1203, 252, 8, 39, "Input",
 CellTags->"Tree level"],
Cell[32937, 1213, 252, 8, 39, "Input",
 CellTags->"Tree level"],
Cell[33192, 1223, 384, 12, 39, "Input",
 CellTags->"Tree level"],
Cell[33579, 1237, 63, 1, 37, "Text",
 CellTags->"Tree level"],
Cell[CellGroupData[{
Cell[33667, 1242, 283, 8, 39, "Input",
 CellTags->"Tree level"],
Cell[33953, 1252, 936, 34, 41, "Output",
 CellTags->"Tree level"]
}, Open  ]],
Cell[CellGroupData[{
Cell[34926, 1291, 283, 8, 39, "Input",
 CellTags->"Tree level"],
Cell[35212, 1301, 938, 34, 41, "Output",
 CellTags->"Tree level"]
}, Open  ]],
Cell[CellGroupData[{
Cell[36187, 1340, 283, 8, 39, "Input",
 CellTags->"Tree level"],
Cell[36473, 1350, 936, 34, 41, "Output",
 CellTags->"Tree level"]
}, Open  ]],
Cell[CellGroupData[{
Cell[37446, 1389, 283, 8, 39, "Input",
 CellTags->"Tree level"],
Cell[37732, 1399, 934, 34, 41, "Output",
 CellTags->"Tree level"]
}, Open  ]],
Cell[38681, 1436, 68, 1, 37, "Text",
 CellTags->"Tree level"],
Cell[CellGroupData[{
Cell[38774, 1441, 251, 8, 26, "Input"],
Cell[39028, 1451, 2213, 82, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[41278, 1538, 82, 2, 26, "Input"],
Cell[41363, 1542, 392, 8, 76, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[41792, 1555, 93, 2, 26, "Input"],
Cell[41888, 1559, 702, 16, 47, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[42627, 1580, 89, 2, 26, "Input"],
Cell[42719, 1584, 193, 4, 26, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[42949, 1593, 77, 2, 26, "Input"],
Cell[43029, 1597, 1712, 29, 93, "Print",
 CellTags->"Info3500973611-8507579"]
}, Open  ]],
Cell[CellGroupData[{
Cell[44778, 1631, 397, 11, 26, "Input"],
Cell[45178, 1644, 379, 14, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[45594, 1663, 74, 2, 26, "Input"],
Cell[45671, 1667, 381, 14, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[46089, 1686, 109, 2, 39, "Input",
 CellTags->"Tree level"],
Cell[46201, 1690, 1269, 45, 42, "Output",
 CellTags->"Tree level"]
}, Open  ]],
Cell[CellGroupData[{
Cell[47507, 1740, 109, 2, 39, "Input",
 CellTags->"Tree level"],
Cell[47619, 1744, 1009, 35, 42, "Output",
 CellTags->"Tree level"]
}, Open  ]],
Cell[CellGroupData[{
Cell[48665, 1784, 109, 2, 39, "Input",
 CellTags->"Tree level"],
Cell[48777, 1788, 1009, 35, 42, "Output",
 CellTags->"Tree level"]
}, Open  ]],
Cell[CellGroupData[{
Cell[49823, 1828, 109, 2, 39, "Input",
 CellTags->"Tree level"],
Cell[49935, 1832, 1269, 45, 42, "Output",
 CellTags->"Tree level"]
}, Open  ]],
Cell[51219, 1880, 90, 1, 37, "Text",
 CellTags->"Tree level"],
Cell[CellGroupData[{
Cell[51334, 1885, 714, 23, 66, "Input",
 CellTags->"Tree level"],
Cell[52051, 1910, 4558, 168, 66, "Output",
 CellTags->"Tree level"]
}, Open  ]],
Cell[56624, 2081, 135, 6, 37, "Text",
 CellTags->"Tree level"],
Cell[CellGroupData[{
Cell[56784, 2091, 164, 5, 39, "Input",
 CellTags->"Tree level"],
Cell[56951, 2098, 4602, 168, 67, "Output",
 CellTags->"Tree level"]
}, Open  ]],
Cell[61568, 2269, 273, 11, 37, "Text",
 CellTags->"Tree level"],
Cell[CellGroupData[{
Cell[61866, 2284, 324, 10, 39, "Input",
 CellTags->"Tree level"],
Cell[62193, 2296, 171, 4, 39, "Output",
 CellTags->"Tree level"]
}, Open  ]],
Cell[62379, 2303, 134, 4, 37, "Text",
 CellTags->"Tree level"],
Cell[62516, 2309, 219, 6, 39, "Input",
 CellTags->"Tree level"],
Cell[62738, 2317, 599, 27, 37, "Text",
 CellTags->"Tree level"]
}, Open  ]],
Cell[CellGroupData[{
Cell[63374, 2349, 71, 1, 52, "Subsection",
 CellTags->"A moment integral"],
Cell[63448, 2352, 787, 28, 44, "Text",
 CellTags->"A moment integral"],
Cell[64238, 2382, 57, 0, 24, "Text"],
Cell[64298, 2384, 139, 4, 39, "Input",
 CellTags->"A moment integral"],
Cell[64440, 2390, 349, 9, 37, "Text",
 CellTags->"A moment integral"],
Cell[64792, 2401, 213, 5, 37, "Text",
 CellTags->"A moment integral"],
Cell[65008, 2408, 1679, 31, 135, "Input",
 CellTags->"A moment integral"],
Cell[66690, 2441, 166, 4, 37, "Text",
 CellTags->"A moment integral"],
Cell[66859, 2447, 17159, 630, 11777, "Input",
 CellTags->"A moment integral"],
Cell[CellGroupData[{
Cell[84043, 3081, 116, 3, 39, "Input",
 CellTags->"A moment integral"],
Cell[84162, 3086, 1575, 50, 72, "Output",
 CellTags->"A moment integral"]
}, Open  ]],
Cell[85752, 3139, 144, 4, 42, "Commentary",
 CellTags->"A moment integral"],
Cell[CellGroupData[{
Cell[85921, 3147, 101, 2, 39, "Input",
 CellTags->"A moment integral"],
Cell[86025, 3151, 330, 10, 39, "Output",
 CellTags->"A moment integral"]
}, Open  ]],
Cell[CellGroupData[{
Cell[86392, 3166, 116, 3, 39, "Input",
 CellTags->"A moment integral"],
Cell[86511, 3171, 295, 9, 39, "Output",
 CellTags->"A moment integral"]
}, Open  ]],
Cell[86821, 3183, 250, 5, 54, "Text",
 CellTags->"A moment integral"],
Cell[87074, 3190, 498, 18, 44, "Text",
 CellTags->"A moment integral"],
Cell[CellGroupData[{
Cell[87597, 3212, 358, 11, 39, "Input",
 CellTags->"A moment integral"],
Cell[87958, 3225, 274, 8, 39, "Output",
 CellTags->"A moment integral"]
}, Open  ]],
Cell[88247, 3236, 197, 4, 37, "Text",
 CellTags->"A moment integral"],
Cell[CellGroupData[{
Cell[88469, 3244, 442, 14, 39, "Input",
 CellTags->"A moment integral"],
Cell[88914, 3260, 273, 8, 39, "Output",
 CellTags->"A moment integral"]
}, Open  ]],
Cell[89202, 3271, 89, 1, 37, "Text",
 CellTags->"A moment integral"],
Cell[89294, 3274, 594, 17, 39, "Input",
 CellTags->"A moment integral"],
Cell[89891, 3293, 92, 1, 37, "Text",
 CellTags->"A moment integral"],
Cell[CellGroupData[{
Cell[90008, 3298, 1240, 37, 59, "Input",
 CellTags->"A moment integral"],
Cell[91251, 3337, 2598, 85, 103, "Output",
 CellTags->"A moment integral"]
}, Open  ]],
Cell[93864, 3425, 147, 4, 37, "Text",
 CellTags->"A moment integral"],
Cell[94014, 3431, 508, 16, 43, "Input",
 CellTags->"A moment integral"],
Cell[CellGroupData[{
Cell[94547, 3451, 113, 3, 39, "Input",
 CellTags->"A moment integral"],
Cell[94663, 3456, 205, 5, 39, "Output",
 CellTags->"A moment integral"]
}, Open  ]],
Cell[94883, 3464, 151, 4, 37, "Text",
 CellTags->"A moment integral"],
Cell[95037, 3470, 252, 8, 39, "Input",
 CellTags->"A moment integral"],
Cell[95292, 3480, 217, 5, 37, "Text",
 CellTags->"A moment integral"],
Cell[CellGroupData[{
Cell[95534, 3489, 180, 5, 39, "Input",
 CellTags->"A moment integral"],
Cell[95717, 3496, 3508, 114, 147, "Output",
 CellTags->"A moment integral"]
}, Open  ]],
Cell[99240, 3613, 243, 8, 37, "Text",
 CellTags->"A moment integral"],
Cell[CellGroupData[{
Cell[99508, 3625, 416, 13, 43, "Input",
 CellTags->"A moment integral"],
Cell[99927, 3640, 2764, 94, 105, "Output",
 CellTags->"A moment integral"]
}, Open  ]],
Cell[102706, 3737, 127, 3, 37, "Text",
 CellTags->"A moment integral"],
Cell[102836, 3742, 177, 5, 39, "Input",
 CellTags->"A moment integral"],
Cell[103016, 3749, 231, 7, 39, "Input",
 CellTags->"A moment integral"],
Cell[103250, 3758, 1134, 55, 37, "Text",
 CellTags->"A moment integral"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}
]
*)

(* End of internal cache information *)
