(* Content-type: application/mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 6.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       145,          7]
NotebookDataLength[   3601165,     132934]
NotebookOptionsPosition[   2620037,     102619]
NotebookOutlinePosition[   3092680,     115691]
CellTagsIndexPosition[   3076315,     115213]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{

Cell[CellGroupData[{
Cell["FeynCalc 8.0", "Title",
 CellChangeTimes->{3.50097011067782*^9},
 TextAlignment->Center],

Cell["Tools for quantum field theory calculations", "Subtitle",
 TextAlignment->Center],

Cell[TextData[StyleBox["Reference Guide",
 FontWeight->"Bold"]], "Subtitle",
 PageBreakBelow->True,
 TextAlignment->Center],

Cell["Rolf Mertig and Frederik Orellana", "Subsubtitle",
 TextAlignment->Center],

Cell[TextData[{
 "HTML version: ",
 ButtonBox["http://www.feyncalc.org/",
  BaseStyle->"Hyperlink",
  ButtonData:>{
    URL["http://www.feyncalc.org/"], None},
  ButtonNote->"http://www.feyncalc.org/"]
}], "Subsubtitle",
 TextAlignment->Center],

Cell[BoxData[
 ButtonBox[
  RowBox[{
  "Click", " ", "to", " ", "save", " ", "this", " ", "notebook", " ", 
   "without", " ", "cache", " ", "and", " ", "FrontEndVersion", " ", "info"}],
  
  ButtonFunction:>Module[{fn}, SetDirectory[
      DirectoryName[fn = ToFileName[
         ReplaceAll["FileName", 
          NotebookInformation[
           InputNotebook[]]]]]]; 
    newfn = StringReplace[fn, ".nb" -> "NoCache.nb"]; 
    If[FileNames[newfn] =!= {}, 
      DeleteFile[newfn]]; Put[
      ReplaceAll[
       NotebookGet[
        InputNotebook[]], Condition[FrontEndVersion -> Pattern[v, 
           Blank[]], Head[v] == String] :> Sequence[]], "fcr.nb"]; 
    CopyFile["fcr.nb", newfn]; DeleteFile["fcr.nb"]],
  Evaluator->Automatic]], NotebookDefault,
 Deployed->True,
 CellTags->"ClickSave"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"$Version", ",", 
   RowBox[{"Date", "[", "]"}], ",", "$UserName"}], "}"}]], "Input",
 CellLabel->"In[1]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"\<\"8.0 for Linux x86 (64-bit) (November 7, 2010)\"\>", ",", 
    RowBox[{"{", 
     RowBox[{
     "2010", ",", "12", ",", "10", ",", "12", ",", "42", ",", 
      "1.702744`6.983724347510624"}], "}"}], ",", "\<\"rolfm\"\>"}], "}"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970121764854*^9},
 CellLabel->"Out[1]="]
}, Open  ]],

Cell[CellGroupData[{

Cell["Functions", "Section",
 CellTags->"Functions"],

Cell["Evaluation time and memory usage", "Commentary"],

Cell["\<\
This is only for timing purposes and can be omitted in general.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"absolutestarttime", "=", 
  RowBox[{"AbsoluteTime", "[", "]"}]}]], "Input",
 CellLabel->"In[2]:="],

Cell[BoxData[
 FormBox["3.500973721893899`16.29676384014798*^9", TraditionalForm]], "Output",\

 CellChangeTimes->{3.5009701219308233`*^9},
 CellLabel->"Out[2]="]
}, Open  ]],

Cell["<<HighEnergyPhysics`FeynCalc`", "Input",
 CellLabel->"In[3]:="],

Cell["Loading FeynCalc from /home/rolfm/HighEnergyPhysics", "Text",
 CellMargins->{{Inherited, Inherited}, {1, 0}},
 GeneratedCell->True,
 CellAutoOverwrite->True,
 CellChangeTimes->{3.500970122055299*^9}],

Cell[TextData[{
 StyleBox["FeynCalc",
  FontWeight->"Bold"],
 " ",
 "8.0.0.beta1",
 " For help, type ?FeynCalc, use the built-in ",
 ButtonBox["help system",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>(FrontEndExecute[
    FrontEnd`HelpBrowserLookup["AddOns", #]]& ),
  ButtonData:>{"Short Overview", "intro"},
  ButtonNote->"Open the help browser"],
 " or visit ",
 ButtonBox["www.feyncalc.org",
  BaseStyle->"Hyperlink",
  ButtonData:>{
    URL["http://www.feyncalc.org/"], None},
  ButtonNote->"http://www.feyncalc.org/"]
}], "Text",
 CellMargins->{{Inherited, Inherited}, {1, 0}},
 GeneratedCell->True,
 CellAutoOverwrite->True,
 CellChangeTimes->{3.500970122910754*^9}],

Cell[TextData[{
 "Loading FeynArts, see ",
 ButtonBox["www.feynarts.de",
  BaseStyle->"Hyperlink",
  ButtonData:>{
    URL["http://www.feynarts.de/"], None},
  ButtonNote->"http://www.feynarts.de/"],
 " for documentation"
}], "Text",
 CellMargins->{{Inherited, Inherited}, {1, 0}},
 GeneratedCell->True,
 CellAutoOverwrite->True,
 CellChangeTimes->{3.500970124822682*^9}],

Cell["FeynArts 3.4 patched for use with FeynCalc", "Text",
 CellMargins->{{Inherited, Inherited}, {1, 0}},
 GeneratedCell->True,
 CellAutoOverwrite->True,
 CellChangeTimes->{3.500970125097189*^9}],

Cell["Abbreviation", "ObjectName",
 PageBreakAbove->True,
 CellTags->"Abbreviation"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Abbreviation"],

Cell["\<\
Abbreviation is a function used by OneLoop and PaVeReduce for generating \
smaller files when saving results to the hard disk. The convention is that a \
definition like GP = GluonPropagator should be accompanied by the definition \
Abbreviation[GluonPropagator] = HoldForm[GP].\
\>", "Text",
 CellTags->"Abbreviation"],

Cell[TextData[{
 "See also:  ",
 ButtonBox["$Abbreviations",
  BaseStyle->"AddOnsLink",
  ButtonData:>"$Abbreviations",
  ButtonNote->"$Abbreviations"],
 ", ",
 ButtonBox["OneLoop",
  BaseStyle->"AddOnsLink",
  ButtonData:>"OneLoop",
  ButtonNote->"OneLoop"],
 ", ",
 ButtonBox["PaVeReduce",
  BaseStyle->"AddOnsLink",
  ButtonData:>"PaVeReduce",
  ButtonNote->"PaVeReduce"],
 ", ",
 ButtonBox["WriteOut",
  BaseStyle->"AddOnsLink",
  ButtonData:>"WriteOut",
  ButtonNote->"WriteOut"],
 ", ",
 ButtonBox["WriteOutPaVe",
  BaseStyle->"AddOnsLink",
  ButtonData:>"WriteOutPaVe",
  ButtonNote->"WriteOutPaVe"],
 ", ",
 ButtonBox["GluonPropagator",
  BaseStyle->"AddOnsLink",
  ButtonData:>"GluonPropagator",
  ButtonNote->"GluonPropagator"],
 ", ",
 ButtonBox["GluonVertex",
  BaseStyle->"AddOnsLink",
  ButtonData:>"GluonVertex",
  ButtonNote->"GluonVertex"],
 ", ",
 ButtonBox["QuarkPropagator",
  BaseStyle->"AddOnsLink",
  ButtonData:>"QuarkPropagator",
  ButtonNote->"QuarkPropagator"],
 "."
}], "Text",
 CellTags->"Abbreviation"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Abbreviation"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GP", "[", 
  RowBox[{"p", ",", " ", 
   RowBox[{"{", 
    RowBox[{"\[Mu]", ",", " ", "a"}], "}"}], ",", " ", 
   RowBox[{"{", 
    RowBox[{"\[Nu]", ",", " ", "b"}], "}"}]}], "]"}]], "Input",
 CellTags->"Abbreviation",
 CellLabel->"In[4]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubsuperscriptBox["\[CapitalPi]", 
    RowBox[{"a", "b"}], 
    RowBox[{"\[Mu]", "\[Nu]"}]], "(", "p", ")"}], TraditionalForm]], "Output",\

 CellChangeTimes->{3.500970126296341*^9},
 CellTags->"Abbreviation",
 CellLabel->"Out[4]="]
}, Open  ]]
}, Open  ]],

Cell["Amplitude", "ObjectName",
 CellTags->"Amplitude"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Amplitude"],

Cell["\<\
Amplitude is a database of Feynman amplitudes. Amplitude[\"name\"] returns \
the amplitude corresponding to the string \"name\". A list of all defined \
names is obtained with Amplitude[]. New amplitudes can be added to the file \
\"Amplitude.m\". It is strongly recommended to use names that reflect the \
process. The option Gauge \[Rule] 1 means `t Hooft Feynman gauge; \
Polarization \[Rule] 0 gives unpolarized OPE-type amplitudes, Polarization \
\[Rule] 1 the polarized ones.\
\>", "Text",
 CellTags->"Amplitude"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "Amplitude", "]"}]], "Input",
 CellTags->"Amplitude",
 CellLabel->"In[5]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
    RowBox[{"Gauge", "\[Rule]", "1"}], ",", 
    RowBox[{"QuarkMass", "\[Rule]", "0"}], ",", 
    RowBox[{"Polarization", "\[Rule]", "1"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.5009701266639433`*^9},
 CellTags->"Amplitude",
 CellLabel->"Out[5]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FeynAmp",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FeynAmp",
  ButtonNote->"FeynAmp"],
 "."
}], "Text",
 CellTags->"Amplitude"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Amplitude"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Amplitude", "[", "]"}], "//", "Length"}]], "Input",
 CellTags->"Amplitude",
 CellLabel->"In[6]:="],

Cell[BoxData[
 FormBox["98", TraditionalForm]], "Output",
 CellChangeTimes->{3.5009701267701263`*^9},
 CellTags->"Amplitude",
 CellLabel->"Out[6]="]
}, Open  ]],

Cell["This is the amplitude of a gluon self-energy diagram.", "Text",
 CellTags->"Amplitude"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Amplitude", "[", "\"\<se1g1\>\"", "]"}]], "Input",
 CellTags->"Amplitude",
 CellLabel->"In[7]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["f", 
    RowBox[{
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["d",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["f",
       TraditionalForm],
      TraditionalForm]}]], " ", 
   SubscriptBox["f", 
    RowBox[{
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["e",
       TraditionalForm],
      TraditionalForm]}]], " ", 
   RowBox[{
    SubsuperscriptBox["\[CapitalPi]", 
     RowBox[{"e", "f"}], 
     RowBox[{"\[Beta]", "\[Sigma]"}]], "(", "q", ")"}], " ", 
   RowBox[{
    SuperscriptBox["V", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Alpha]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Beta]",
        TraditionalForm],
       TraditionalForm]}]], "(", 
    RowBox[{
     FormBox["p",
      TraditionalForm], 
     RowBox[{",", " "}], 
     FormBox[
      RowBox[{"q", "-", "p"}],
      TraditionalForm], 
     RowBox[{",", " "}], 
     FormBox[
      RowBox[{"-", "q"}],
      TraditionalForm]}], ")"}], " ", 
   RowBox[{
    SuperscriptBox["V", 
     RowBox[{
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Rho]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm]}]], "(", 
    RowBox[{
     FormBox[
      RowBox[{"-", "p"}],
      TraditionalForm], 
     RowBox[{",", " "}], 
     FormBox[
      RowBox[{"p", "-", "q"}],
      TraditionalForm], 
     RowBox[{",", " "}], 
     FormBox["q",
      TraditionalForm]}], ")"}], " ", 
   RowBox[{
    SubsuperscriptBox["\[CapitalPi]", 
     RowBox[{"c", "d"}], 
     RowBox[{"\[Alpha]", "\[Rho]"}]], "(", 
    FormBox[
     RowBox[{"p", "-", "q"}],
     TraditionalForm], ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009701270724277`*^9},
 CellTags->"Amplitude",
 CellLabel->"Out[7]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Explicit", "[", "%", "]"}]], "Input",
 CellTags->"Amplitude",
 CellLabel->"In[8]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   RowBox[{
    FractionBox["1", 
     RowBox[{
      SuperscriptBox["q", "2"], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"p", "-", "q"}], ")"}], "2"]}]], 
    RowBox[{
     SubsuperscriptBox["g", "s", "2"], " ", 
     SubscriptBox["f", 
      RowBox[{
       FormBox[
        FormBox["b",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["d",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["f",
         TraditionalForm],
        TraditionalForm]}]], " ", 
     SubscriptBox["\[Delta]", 
      RowBox[{
       FormBox[
        FormBox["c",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["d",
         TraditionalForm],
        TraditionalForm]}]], " ", 
     SubscriptBox["\[Delta]", 
      RowBox[{
       FormBox[
        FormBox["e",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["f",
         TraditionalForm],
        TraditionalForm]}]], " ", 
     SuperscriptBox["g", 
      RowBox[{"\[Alpha]", "\[Rho]"}]], " ", 
     SuperscriptBox["g", 
      RowBox[{"\[Beta]", "\[Sigma]"}]], " ", 
     SubscriptBox["f", 
      RowBox[{
       FormBox[
        FormBox["a",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["c",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["e",
         TraditionalForm],
        TraditionalForm]}]], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        SuperscriptBox["g", 
         RowBox[{"\[Beta]", "\[Mu]"}]], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"-", 
           SuperscriptBox[
            FormBox["p",
             TraditionalForm], 
            FormBox[
             FormBox["\[Alpha]",
              TraditionalForm],
             TraditionalForm]]}], "-", 
          SuperscriptBox[
           FormBox["q",
            TraditionalForm], 
           FormBox[
            FormBox["\[Alpha]",
             TraditionalForm],
            TraditionalForm]]}], ")"}]}], "+", 
       RowBox[{
        SuperscriptBox["g", 
         RowBox[{"\[Alpha]", "\[Mu]"}]], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"2", " ", 
           SuperscriptBox[
            FormBox["p",
             TraditionalForm], 
            FormBox[
             FormBox["\[Beta]",
              TraditionalForm],
             TraditionalForm]]}], "-", 
          SuperscriptBox[
           FormBox["q",
            TraditionalForm], 
           FormBox[
            FormBox["\[Beta]",
             TraditionalForm],
            TraditionalForm]]}], ")"}]}], "+", 
       RowBox[{
        SuperscriptBox["g", 
         RowBox[{"\[Alpha]", "\[Beta]"}]], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"2", " ", 
           SuperscriptBox[
            FormBox["q",
             TraditionalForm], 
            FormBox[
             FormBox["\[Mu]",
              TraditionalForm],
             TraditionalForm]]}], "-", 
          SuperscriptBox[
           FormBox["p",
            TraditionalForm], 
           FormBox[
            FormBox["\[Mu]",
             TraditionalForm],
            TraditionalForm]]}], ")"}]}]}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        SuperscriptBox["g", 
         RowBox[{"\[Rho]", "\[Sigma]"}]], " ", 
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox[
           FormBox["p",
            TraditionalForm], 
           FormBox[
            FormBox["\[Nu]",
             TraditionalForm],
            TraditionalForm]], "-", 
          RowBox[{"2", " ", 
           SuperscriptBox[
            FormBox["q",
             TraditionalForm], 
            FormBox[
             FormBox["\[Nu]",
              TraditionalForm],
             TraditionalForm]]}]}], ")"}]}], "+", 
       RowBox[{
        SuperscriptBox["g", 
         RowBox[{"\[Nu]", "\[Sigma]"}]], " ", 
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox[
           FormBox["p",
            TraditionalForm], 
           FormBox[
            FormBox["\[Rho]",
             TraditionalForm],
            TraditionalForm]], "+", 
          SuperscriptBox[
           FormBox["q",
            TraditionalForm], 
           FormBox[
            FormBox["\[Rho]",
             TraditionalForm],
            TraditionalForm]]}], ")"}]}], "+", 
       RowBox[{
        SuperscriptBox["g", 
         RowBox[{"\[Nu]", "\[Rho]"}]], " ", 
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox[
           FormBox["q",
            TraditionalForm], 
           FormBox[
            FormBox["\[Sigma]",
             TraditionalForm],
            TraditionalForm]], "-", 
          RowBox[{"2", " ", 
           SuperscriptBox[
            FormBox["p",
             TraditionalForm], 
            FormBox[
             FormBox["\[Sigma]",
              TraditionalForm],
             TraditionalForm]]}]}], ")"}]}]}], ")"}]}]}]}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970127340906*^9},
 CellTags->"Amplitude",
 CellLabel->"Out[8]="]
}, Open  ]],

Cell[TextData[{
 "This is the amplitude for graph 6.2 from the paper Z.Phys C ",
 StyleBox["70:",
  FontWeight->"Bold"],
 "637-654, 1996",
 StyleBox[".",
  FontWeight->"Bold"]
}], "Text",
 CellTags->"Amplitude"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FeynAmp", "[", 
  RowBox[{"q1", ",", "q2", ",", 
   RowBox[{"EpsEvaluate", "[", 
    RowBox[{"Trick", "[", 
     RowBox[{"Explicit", "[", 
      RowBox[{"Amplitude", "[", "\"\<gg2\>\"", "]"}], "]"}], "]"}], "]"}]}], 
  "]"}]], "Input",
 CellTags->"Amplitude",
 CellLabel->"In[9]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"\[Integral]", 
   RowBox[{
    StyleBox[
     RowBox[{
      SuperscriptBox["\[DifferentialD]", "D"], "q1"}],
     ZeroWidthTimes->True], "\[Integral]", 
    RowBox[{
     SuperscriptBox["\[DifferentialD]", "D"], "q2"}]}], "(", 
   FormBox[
    RowBox[{
     RowBox[{"(", 
      RowBox[{"2", " ", "\[ImaginaryI]", " ", 
       RowBox[{"(", 
        RowBox[{"1", "-", 
         FormBox[
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{"-", "1"}], ")"}], "m"],
          TraditionalForm]}], ")"}], " ", 
       SubscriptBox["f", 
        RowBox[{
         FormBox[
          FormBox[
           SubscriptBox["\<\"c\"\>", "5"],
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox[
           SubscriptBox["\<\"c\"\>", "7"],
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox[
           SubscriptBox["\<\"c\"\>", "12"],
           TraditionalForm],
          TraditionalForm]}]], " ", 
       SubscriptBox["f", 
        RowBox[{
         FormBox[
          FormBox[
           SubscriptBox["\<\"c\"\>", "10"],
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox[
           SubscriptBox["\<\"c\"\>", "11"],
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox[
           SubscriptBox["\<\"c\"\>", "12"],
           TraditionalForm],
          TraditionalForm]}]], " ", 
       SubsuperscriptBox["g", "s", "4"], " ", 
       SubscriptBox["f", 
        RowBox[{
         FormBox[
          FormBox["a",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox[
           SubscriptBox["\<\"c\"\>", "5"],
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox[
           SubscriptBox["\<\"c\"\>", "11"],
           TraditionalForm],
          TraditionalForm]}]], " ", 
       SubscriptBox["f", 
        RowBox[{
         FormBox[
          FormBox["b",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox[
           SubscriptBox["\<\"c\"\>", "7"],
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox[
           SubscriptBox["\<\"c\"\>", "10"],
           TraditionalForm],
          TraditionalForm]}]], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          SuperscriptBox["g", 
           RowBox[{
            SubscriptBox["\[Lambda]", "7"], 
            SubscriptBox["\[Lambda]", "12"]}]], " ", 
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox[
             FormBox["q2",
              TraditionalForm], 
             FormBox[
              FormBox[
               SubscriptBox["\[Lambda]", "5"],
               TraditionalForm],
              TraditionalForm]], "-", 
            RowBox[{"2", " ", 
             SuperscriptBox[
              FormBox["q1",
               TraditionalForm], 
              FormBox[
               FormBox[
                SubscriptBox["\[Lambda]", "5"],
                TraditionalForm],
               TraditionalForm]]}]}], ")"}]}], "+", 
         RowBox[{
          SuperscriptBox["g", 
           RowBox[{
            SubscriptBox["\[Lambda]", "5"], 
            SubscriptBox["\[Lambda]", "12"]}]], " ", 
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox[
             FormBox["q1",
              TraditionalForm], 
             FormBox[
              FormBox[
               SubscriptBox["\[Lambda]", "7"],
               TraditionalForm],
              TraditionalForm]], "-", 
            RowBox[{"2", " ", 
             SuperscriptBox[
              FormBox["q2",
               TraditionalForm], 
              FormBox[
               FormBox[
                SubscriptBox["\[Lambda]", "7"],
                TraditionalForm],
               TraditionalForm]]}]}], ")"}]}], "+", 
         RowBox[{
          SuperscriptBox["g", 
           RowBox[{
            SubscriptBox["\[Lambda]", "5"], 
            SubscriptBox["\[Lambda]", "7"]}]], " ", 
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox[
             FormBox["q1",
              TraditionalForm], 
             FormBox[
              FormBox[
               SubscriptBox["\[Lambda]", "12"],
               TraditionalForm],
              TraditionalForm]], "+", 
            SuperscriptBox[
             FormBox["q2",
              TraditionalForm], 
             FormBox[
              FormBox[
               SubscriptBox["\[Lambda]", "12"],
               TraditionalForm],
              TraditionalForm]]}], ")"}]}]}], ")"}], " ", 
       FormBox[
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["q2",
            TraditionalForm]}], ")"}], 
         RowBox[{"m", "-", "1"}]],
        TraditionalForm], " ", 
       SuperscriptBox["\[Epsilon]", 
        RowBox[{
         FormBox[
          FormBox[
           SubscriptBox["\[Lambda]", "1"],
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox[
           SubscriptBox["\[Lambda]", "5"],
           TraditionalForm],
          TraditionalForm], 
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox["q2",
          TraditionalForm]}]], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          SuperscriptBox["g", 
           RowBox[{
            SubscriptBox["\[Lambda]", "7"], 
            SubscriptBox["\[Lambda]", "10"]}]], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"2", " ", 
             SuperscriptBox[
              FormBox["q1",
               TraditionalForm], 
              FormBox[
               FormBox["\[Nu]",
                TraditionalForm],
               TraditionalForm]]}], "-", 
            SuperscriptBox[
             FormBox["p",
              TraditionalForm], 
             FormBox[
              FormBox["\[Nu]",
               TraditionalForm],
              TraditionalForm]]}], ")"}]}], "+", 
         RowBox[{
          SuperscriptBox["g", 
           RowBox[{"\[Nu]", 
            SubscriptBox["\[Lambda]", "10"]}]], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"2", " ", 
             SuperscriptBox[
              FormBox["p",
               TraditionalForm], 
              FormBox[
               FormBox[
                SubscriptBox["\[Lambda]", "7"],
                TraditionalForm],
               TraditionalForm]]}], "-", 
            SuperscriptBox[
             FormBox["q1",
              TraditionalForm], 
             FormBox[
              FormBox[
               SubscriptBox["\[Lambda]", "7"],
               TraditionalForm],
              TraditionalForm]]}], ")"}]}], "+", 
         RowBox[{
          SuperscriptBox["g", 
           RowBox[{"\[Nu]", 
            SubscriptBox["\[Lambda]", "7"]}]], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"-", 
             SuperscriptBox[
              FormBox["p",
               TraditionalForm], 
              FormBox[
               FormBox[
                SubscriptBox["\[Lambda]", "10"],
                TraditionalForm],
               TraditionalForm]]}], "-", 
            SuperscriptBox[
             FormBox["q1",
              TraditionalForm], 
             FormBox[
              FormBox[
               SubscriptBox["\[Lambda]", "10"],
               TraditionalForm],
              TraditionalForm]]}], ")"}]}]}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          SuperscriptBox["g", 
           RowBox[{
            SubscriptBox["\[Lambda]", "11"], 
            SubscriptBox["\[Lambda]", "12"]}]], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"-", 
             SuperscriptBox[
              FormBox["p",
               TraditionalForm], 
              FormBox[
               FormBox[
                SubscriptBox["\[Lambda]", "10"],
                TraditionalForm],
               TraditionalForm]]}], "-", 
            SuperscriptBox[
             FormBox["q1",
              TraditionalForm], 
             FormBox[
              FormBox[
               SubscriptBox["\[Lambda]", "10"],
               TraditionalForm],
              TraditionalForm]], "+", 
            RowBox[{"2", " ", 
             SuperscriptBox[
              FormBox["q2",
               TraditionalForm], 
              FormBox[
               FormBox[
                SubscriptBox["\[Lambda]", "10"],
                TraditionalForm],
               TraditionalForm]]}]}], ")"}]}], "+", 
         RowBox[{
          SuperscriptBox["g", 
           RowBox[{
            SubscriptBox["\[Lambda]", "10"], 
            SubscriptBox["\[Lambda]", "12"]}]], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"-", 
             SuperscriptBox[
              FormBox["p",
               TraditionalForm], 
              FormBox[
               FormBox[
                SubscriptBox["\[Lambda]", "11"],
                TraditionalForm],
               TraditionalForm]]}], "+", 
            RowBox[{"2", " ", 
             SuperscriptBox[
              FormBox["q1",
               TraditionalForm], 
              FormBox[
               FormBox[
                SubscriptBox["\[Lambda]", "11"],
                TraditionalForm],
               TraditionalForm]]}], "-", 
            SuperscriptBox[
             FormBox["q2",
              TraditionalForm], 
             FormBox[
              FormBox[
               SubscriptBox["\[Lambda]", "11"],
               TraditionalForm],
              TraditionalForm]]}], ")"}]}], "+", 
         RowBox[{
          SuperscriptBox["g", 
           RowBox[{
            SubscriptBox["\[Lambda]", "10"], 
            SubscriptBox["\[Lambda]", "11"]}]], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"2", " ", 
             SuperscriptBox[
              FormBox["p",
               TraditionalForm], 
              FormBox[
               FormBox[
                SubscriptBox["\[Lambda]", "12"],
                TraditionalForm],
               TraditionalForm]]}], "-", 
            SuperscriptBox[
             FormBox["q1",
              TraditionalForm], 
             FormBox[
              FormBox[
               SubscriptBox["\[Lambda]", "12"],
               TraditionalForm],
              TraditionalForm]], "-", 
            SuperscriptBox[
             FormBox["q2",
              TraditionalForm], 
             FormBox[
              FormBox[
               SubscriptBox["\[Lambda]", "12"],
               TraditionalForm],
              TraditionalForm]]}], ")"}]}]}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          SuperscriptBox["g", 
           RowBox[{
            SubscriptBox["\[Lambda]", "1"], 
            SubscriptBox["\[Lambda]", "11"]}]], " ", 
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox[
             FormBox["p",
              TraditionalForm], 
             FormBox[
              FormBox["\[Mu]",
               TraditionalForm],
              TraditionalForm]], "-", 
            RowBox[{"2", " ", 
             SuperscriptBox[
              FormBox["q2",
               TraditionalForm], 
              FormBox[
               FormBox["\[Mu]",
                TraditionalForm],
               TraditionalForm]]}]}], ")"}]}], "+", 
         RowBox[{
          SuperscriptBox["g", 
           RowBox[{"\[Mu]", 
            SubscriptBox["\[Lambda]", "11"]}]], " ", 
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox[
             FormBox["q2",
              TraditionalForm], 
             FormBox[
              FormBox[
               SubscriptBox["\[Lambda]", "1"],
               TraditionalForm],
              TraditionalForm]], "-", 
            RowBox[{"2", " ", 
             SuperscriptBox[
              FormBox["p",
               TraditionalForm], 
              FormBox[
               FormBox[
                SubscriptBox["\[Lambda]", "1"],
                TraditionalForm],
               TraditionalForm]]}]}], ")"}]}], "+", 
         RowBox[{
          SuperscriptBox["g", 
           RowBox[{"\[Mu]", 
            SubscriptBox["\[Lambda]", "1"]}]], " ", 
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox[
             FormBox["p",
              TraditionalForm], 
             FormBox[
              FormBox[
               SubscriptBox["\[Lambda]", "11"],
               TraditionalForm],
              TraditionalForm]], "+", 
            SuperscriptBox[
             FormBox["q2",
              TraditionalForm], 
             FormBox[
              FormBox[
               SubscriptBox["\[Lambda]", "11"],
               TraditionalForm],
              TraditionalForm]]}], ")"}]}]}], ")"}]}], ")"}], "/", 
     RowBox[{
      SuperscriptBox["q1", "2"], ".", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"q1", "-", "p"}], ")"}], "2"], ".", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"q1", "-", "q2"}], ")"}], "2"], ".", 
      SuperscriptBox["q2", "2"], ".", 
      SuperscriptBox["q2", "2"], ".", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"q2", "-", "p"}], ")"}], "2"]}]}],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970127921139*^9},
 CellTags->"Amplitude",
 CellLabel->"Out[9]="]
}, Open  ]]
}, Open  ]],

Cell["Amputate", "ObjectName",
 CellTags->"Amputate"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Amputate"],

Cell["\<\
Amputate[exp,q1,q2,...] amputates Eps and DiracGamma. Amputate[exp,q1,q2,Pair\
\[Rule]{p}] amputates also p.q1 and p.q2; Pair\[Rule]All amputates all except \
OPEDelta.\
\>", "Text",
 CellTags->"Amputate"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "Amputate", "]"}]], "Input",
 CellTags->"Amputate",
 CellLabel->"In[10]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
    RowBox[{"Pair", "\[Rule]", 
     RowBox[{"{", "}"}]}], ",", 
    RowBox[{"Unique", "\[Rule]", "True"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970128113311*^9},
 CellTags->"Amputate",
 CellLabel->"Out[10]="]
}, Open  ]],

Cell[TextData[{
 "See also:  ",
 ButtonBox["DiracGamma",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracGamma",
  ButtonNote->"DiracGamma"],
 ", ",
 ButtonBox["DiracMatrix",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracMatrix",
  ButtonNote->"DiracMatrix"],
 ", ",
 ButtonBox["DiracSimplify",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracSimplify",
  ButtonNote->"DiracSimplify"],
 ", ",
 ButtonBox["DiracSlash",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracSlash",
  ButtonNote->"DiracSlash"],
 ", ",
 ButtonBox["DiracTrick",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracTrick",
  ButtonNote->"DiracTrick"],
 "."
}], "Text",
 CellTags->"Amputate"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Amputate"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"DiracSlash", "[", "p", "]"}], ".", 
  RowBox[{"DiracSlash", "[", "q", "]"}]}]], "Input",
 CellTags->"Amputate",
 CellLabel->"In[11]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     FormBox["\<\"\[Gamma]\"\>",
      TraditionalForm], "\[CenterDot]", 
     FormBox["p",
      TraditionalForm]}], ")"}], ".", 
   RowBox[{"(", 
    RowBox[{
     FormBox["\<\"\[Gamma]\"\>",
      TraditionalForm], "\[CenterDot]", 
     FormBox["q",
      TraditionalForm]}], ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009701285369673`*^9},
 CellTags->"Amputate",
 CellLabel->"Out[11]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Amputate", "[", 
  RowBox[{"%", ",", "q"}], "]"}]], "Input",
 CellTags->"Amputate",
 CellLabel->"In[12]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox[
    FormBox["q",
     TraditionalForm], 
    FormBox[
     FormBox[
      RowBox[{"$AL$708", "\[InvisibleApplication]", 
       RowBox[{"(", "1", ")"}]}],
      TraditionalForm],
     TraditionalForm]], " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      FormBox["\<\"\[Gamma]\"\>",
       TraditionalForm], "\[CenterDot]", 
      FormBox["p",
       TraditionalForm]}], ")"}], ".", 
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox[
       RowBox[{"$AL$708", "\[InvisibleApplication]", 
        RowBox[{"(", "1", ")"}]}],
       TraditionalForm],
      TraditionalForm]]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009701292725153`*^9},
 CellTags->"Amputate",
 CellLabel->"Out[12]="]
}, Open  ]]
}, Open  ]],

Cell["AnomalousDimension", "ObjectName",
 CellTags->"AnomalousDimension"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"AnomalousDimension"],

Cell["\<\
AnomalousDimension[name] is a database of anomalous dimensions of twist 2 \
operators.\
\>", "Text",
 CellTags->"AnomalousDimension"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "AnomalousDimension", "]"}]], "Input",
 CellTags->"AnomalousDimension",
 CellLabel->"In[13]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Polarization", "\[Rule]", "1"}], ",", 
    RowBox[{"Simplify", "\[Rule]", "FullSimplify"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970129433525*^9},
 CellTags->"AnomalousDimension",
 CellLabel->"Out[13]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 ButtonBox["SplittingFunction",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SplittingFunction",
  ButtonNote->"SplittingFunction"],
 ", ",
 ButtonBox["SumS",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SumS",
  ButtonNote->"SumS"],
 ", ",
 ButtonBox["SumT",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SumT",
  ButtonNote->"SumT"],
 "."
}], "Text",
 CellTags->"AnomalousDimension"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"AnomalousDimension"],

Cell["Polarized case", "Text",
 CellTags->"AnomalousDimension"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SetOptions", "[", " ", 
  RowBox[{"AnomalousDimension", ",", 
   RowBox[{"Polarization", "\[Rule]", "1"}]}], "]"}]], "Input",
 CellTags->"AnomalousDimension",
 CellLabel->"In[14]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Polarization", "\[Rule]", "1"}], ",", 
    RowBox[{"Simplify", "\[Rule]", "FullSimplify"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970129568454*^9},
 CellTags->"AnomalousDimension",
 CellLabel->"Out[14]="]
}, Open  ]],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["\[Gamma]", 
    RowBox[{"NS", ",", "qq", " "}], 
    RowBox[{
     RowBox[{"(", "0", ")"}], " "}]], TraditionalForm]]],
 "polarized"
}], "Text",
 CellTags->"AnomalousDimension"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"AnomalousDimension", "[", "gnsqq0", "]"}]], "Input",
 CellTags->"AnomalousDimension",
 CellLabel->"In[15]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["C", "F"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"8", " ", 
      RowBox[{
       SubscriptBox["S", "1"], "(", 
       FormBox[
        RowBox[{"m", "-", "1"}],
        TraditionalForm], ")"}]}], "+", 
     FractionBox["4", "m"], "+", 
     FractionBox["4", 
      RowBox[{"m", "+", "1"}]], "-", "6"}], ")"}]}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970129863584*^9},
 CellTags->"AnomalousDimension",
 CellLabel->"Out[15]="]
}, Open  ]],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["\[Gamma]", 
    RowBox[{"S", ",", "qg", " "}], 
    RowBox[{
     RowBox[{"(", "0", ")"}], " "}]], TraditionalForm]]],
 "polarized"
}], "Text",
 CellTags->"AnomalousDimension"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"AnomalousDimension", "[", "gsqg0", "]"}]], "Input",
 CellTags->"AnomalousDimension",
 CellLabel->"In[16]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     FractionBox["8", "m"], "-", 
     FractionBox["16", 
      RowBox[{"m", "+", "1"}]]}], ")"}], " ", 
   SubscriptBox["T", "f"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970130029182*^9},
 CellTags->"AnomalousDimension",
 CellLabel->"Out[16]="]
}, Open  ]],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["\[Gamma]", 
    RowBox[{"S", ",", "gq", " "}], 
    RowBox[{
     RowBox[{"(", "0", ")"}], " "}]], TraditionalForm]]],
 "polarized"
}], "Text",
 CellTags->"AnomalousDimension"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"AnomalousDimension", "[", "gsgq0", "]"}]], "Input",
 CellTags->"AnomalousDimension",
 CellLabel->"In[17]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     FractionBox["4", 
      RowBox[{"m", "+", "1"}]], "-", 
     FractionBox["8", "m"]}], ")"}], " ", 
   SubscriptBox["C", "F"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970130170414*^9},
 CellTags->"AnomalousDimension",
 CellLabel->"Out[17]="]
}, Open  ]],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["\[Gamma]", 
    RowBox[{"S", ",", "gg", " "}], 
    RowBox[{
     RowBox[{"(", "0", ")"}], " "}]], TraditionalForm]]],
 "polarized"
}], "Text",
 CellTags->"AnomalousDimension"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"AnomalousDimension", "[", "gsgg0", "]"}]], "Input",
 CellTags->"AnomalousDimension",
 CellLabel->"In[18]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox["C", "A"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"8", " ", 
       RowBox[{
        SubscriptBox["S", "1"], "(", 
        FormBox[
         RowBox[{"m", "-", "1"}],
         TraditionalForm], ")"}]}], "-", 
      FractionBox["8", "m"], "+", 
      FractionBox["16", 
       RowBox[{"m", "+", "1"}]], "-", 
      FractionBox["22", "3"]}], ")"}]}], "+", 
   FractionBox[
    RowBox[{"8", " ", 
     SubscriptBox["T", "f"]}], "3"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970130494033*^9},
 CellTags->"AnomalousDimension",
 CellLabel->"Out[18]="]
}, Open  ]],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["\[Gamma]", 
    RowBox[{"PS", ",", "qq", " "}], 
    RowBox[{
     RowBox[{"(", "0", ")"}], " "}]], TraditionalForm]]],
 "polarized"
}], "Text",
 CellTags->"AnomalousDimension"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"AnomalousDimension", "[", "gpsqq1", "]"}]], "Input",
 CellTags->"AnomalousDimension",
 CellLabel->"In[19]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"16", " ", 
   RowBox[{"(", 
    RowBox[{
     FractionBox["2", 
      SuperscriptBox["m", "3"]], "-", 
     FractionBox["1", 
      SuperscriptBox["m", "2"]], "+", 
     FractionBox["1", 
      RowBox[{"m", "+", "1"}]], "+", 
     FractionBox["3", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"m", "+", "1"}], ")"}], "2"]], "+", 
     FractionBox["2", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"m", "+", "1"}], ")"}], "3"]], "-", 
     FractionBox["1", "m"]}], ")"}], " ", 
   SubscriptBox["C", "F"], " ", 
   SubscriptBox["T", "f"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970130674858*^9},
 CellTags->"AnomalousDimension",
 CellLabel->"Out[19]="]
}, Open  ]],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["\[Gamma]", 
    RowBox[{"NS", ",", "qq", " "}], 
    RowBox[{
     RowBox[{"(", "1", ")"}], " "}]], TraditionalForm]]],
 "polarized"
}], "Text",
 CellTags->"AnomalousDimension"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"AnomalousDimension", "[", "gnsqq1", "]"}]], "Input",
 CellTags->"AnomalousDimension",
 CellLabel->"In[20]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    RowBox[{
     SubscriptBox["C", "A"], " ", 
     SubscriptBox["C", "F"], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"-", 
        FractionBox[
         RowBox[{"16", " ", 
          RowBox[{
           SubscriptBox[
            OverscriptBox["S", "~"], "2"], "(", 
           FormBox[
            RowBox[{"m", "-", "1"}],
            TraditionalForm], ")"}]}], "m"]}], "-", 
       FractionBox[
        RowBox[{"16", " ", 
         RowBox[{
          SubscriptBox[
           OverscriptBox["S", "~"], "2"], "(", 
          FormBox[
           RowBox[{"m", "-", "1"}],
           TraditionalForm], ")"}]}], 
        RowBox[{"m", "+", "1"}]], "+", 
       RowBox[{"16", " ", 
        RowBox[{
         SubscriptBox[
          OverscriptBox["S", "~"], "3"], "(", 
         FormBox[
          RowBox[{"m", "-", "1"}],
          TraditionalForm], ")"}]}], "-", 
       RowBox[{"32", " ", 
        RowBox[{
         SubscriptBox[
          OverscriptBox["S", "~"], "12"], "(", 
         FormBox[
          RowBox[{"m", "-", "1"}],
          TraditionalForm], ")"}]}], "+", 
       FractionBox["44", 
        RowBox[{"3", " ", 
         SuperscriptBox["m", "2"]}]], "-", 
       RowBox[{
        FractionBox["536", "9"], " ", 
        RowBox[{
         SubscriptBox["S", "1"], "(", 
         FormBox[
          RowBox[{"m", "-", "1"}],
          TraditionalForm], ")"}]}], "+", 
       RowBox[{
        FractionBox["88", "3"], " ", 
        RowBox[{
         SubscriptBox["S", "2"], "(", 
         FormBox[
          RowBox[{"m", "-", "1"}],
          TraditionalForm], ")"}]}], "-", 
       RowBox[{"16", " ", 
        RowBox[{
         SubscriptBox["S", "3"], "(", 
         FormBox[
          RowBox[{"m", "-", "1"}],
          TraditionalForm], ")"}]}], "+", 
       FractionBox["212", 
        RowBox[{"9", " ", "m"}]], "-", 
       FractionBox["748", 
        RowBox[{"9", " ", 
         RowBox[{"(", 
          RowBox[{"m", "+", "1"}], ")"}]}]], "-", 
       FractionBox["4", 
        RowBox[{"3", " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"m", "+", "1"}], ")"}], "2"]}]], "-", 
       FractionBox["16", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"m", "+", "1"}], ")"}], "3"]], "+", 
       FractionBox["17", "3"]}], ")"}]}]}], "+", 
   RowBox[{
    SubsuperscriptBox["C", "F", "2"], " ", 
    RowBox[{"(", 
     RowBox[{"-", 
      RowBox[{"(", 
       RowBox[{
        FractionBox[
         RowBox[{"32", " ", 
          RowBox[{
           SubscriptBox[
            OverscriptBox["S", "~"], "2"], "(", 
           FormBox[
            RowBox[{"m", "-", "1"}],
            TraditionalForm], ")"}]}], "m"], "+", 
        FractionBox[
         RowBox[{"32", " ", 
          RowBox[{
           SubscriptBox[
            OverscriptBox["S", "~"], "2"], "(", 
           FormBox[
            RowBox[{"m", "-", "1"}],
            TraditionalForm], ")"}]}], 
         RowBox[{"m", "+", "1"}]], "-", 
        RowBox[{"32", " ", 
         RowBox[{
          SubscriptBox[
           OverscriptBox["S", "~"], "3"], "(", 
          FormBox[
           RowBox[{"m", "-", "1"}],
           TraditionalForm], ")"}]}], "+", 
        RowBox[{"64", " ", 
         RowBox[{
          SubscriptBox[
           OverscriptBox["S", "~"], "12"], "(", 
          FormBox[
           RowBox[{"m", "-", "1"}],
           TraditionalForm], ")"}]}], "+", 
        FractionBox["8", 
         SuperscriptBox["m", "3"]], "+", 
        FractionBox[
         RowBox[{"16", " ", 
          RowBox[{
           SubscriptBox["S", "1"], "(", 
           FormBox[
            RowBox[{"m", "-", "1"}],
            TraditionalForm], ")"}]}], 
         SuperscriptBox["m", "2"]], "+", 
        FractionBox[
         RowBox[{"16", " ", 
          RowBox[{
           SubscriptBox["S", "1"], "(", 
           FormBox[
            RowBox[{"m", "-", "1"}],
            TraditionalForm], ")"}]}], 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"m", "+", "1"}], ")"}], "2"]], "+", 
        FractionBox[
         RowBox[{"16", " ", 
          RowBox[{
           SubscriptBox["S", "2"], "(", 
           FormBox[
            RowBox[{"m", "-", "1"}],
            TraditionalForm], ")"}]}], "m"], "+", 
        FractionBox[
         RowBox[{"16", " ", 
          RowBox[{
           SubscriptBox["S", "2"], "(", 
           FormBox[
            RowBox[{"m", "-", "1"}],
            TraditionalForm], ")"}]}], 
         RowBox[{"m", "+", "1"}]], "-", 
        RowBox[{"24", " ", 
         RowBox[{
          SubscriptBox["S", "2"], "(", 
          FormBox[
           RowBox[{"m", "-", "1"}],
           TraditionalForm], ")"}]}], "+", 
        RowBox[{"32", " ", 
         RowBox[{
          SubscriptBox["S", "12"], "(", 
          FormBox[
           RowBox[{"m", "-", "1"}],
           TraditionalForm], ")"}]}], "+", 
        RowBox[{"32", " ", 
         RowBox[{
          SubscriptBox["S", "21"], "(", 
          FormBox[
           RowBox[{"m", "-", "1"}],
           TraditionalForm], ")"}]}], "-", 
        FractionBox["40", "m"], "+", 
        FractionBox["40", 
         RowBox[{"m", "+", "1"}]], "+", 
        FractionBox["16", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"m", "+", "1"}], ")"}], "2"]], "+", 
        FractionBox["40", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"m", "+", "1"}], ")"}], "3"]], "+", "3"}], ")"}]}], 
     ")"}]}], "-", 
   RowBox[{
    SubscriptBox["C", "F"], " ", 
    SubscriptBox["N", "f"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", 
       FractionBox["8", 
        RowBox[{"3", " ", 
         SuperscriptBox["m", "2"]}]]}], "+", 
      RowBox[{
       FractionBox["80", "9"], " ", 
       RowBox[{
        SubscriptBox["S", "1"], "(", 
        FormBox[
         RowBox[{"m", "-", "1"}],
         TraditionalForm], ")"}]}], "-", 
      RowBox[{
       FractionBox["16", "3"], " ", 
       RowBox[{
        SubscriptBox["S", "2"], "(", 
        FormBox[
         RowBox[{"m", "-", "1"}],
         TraditionalForm], ")"}]}], "-", 
      FractionBox["8", 
       RowBox[{"9", " ", "m"}]], "+", 
      FractionBox["88", 
       RowBox[{"9", " ", 
        RowBox[{"(", 
         RowBox[{"m", "+", "1"}], ")"}]}]], "-", 
      FractionBox["8", 
       RowBox[{"3", " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"m", "+", "1"}], ")"}], "2"]}]], "-", 
      FractionBox["2", "3"]}], ")"}]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009701313442297`*^9},
 CellTags->"AnomalousDimension",
 CellLabel->"Out[20]="]
}, Open  ]],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["\[Gamma]", 
    RowBox[{"S", ",", "qg", " "}], 
    RowBox[{
     RowBox[{"(", "1", ")"}], " "}]], TraditionalForm]]],
 "polarized"
}], "Text",
 CellTags->"AnomalousDimension"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"AnomalousDimension", "[", "gsqg1", "]"}]], "Input",
 CellTags->"AnomalousDimension",
 CellLabel->"In[21]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"16", " ", 
    SubscriptBox["C", "A"], " ", 
    SubscriptBox["T", "f"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", 
       FractionBox[
        RowBox[{"2", " ", 
         RowBox[{
          SubscriptBox[
           OverscriptBox["S", "~"], "2"], "(", 
          FormBox[
           RowBox[{"m", "-", "1"}],
           TraditionalForm], ")"}]}], "m"]}], "+", 
      FractionBox[
       RowBox[{"4", " ", 
        RowBox[{
         SubscriptBox[
          OverscriptBox["S", "~"], "2"], "(", 
         FormBox[
          RowBox[{"m", "-", "1"}],
          TraditionalForm], ")"}]}], 
       RowBox[{"m", "+", "1"}]], "+", 
      FractionBox["2", 
       SuperscriptBox["m", "3"]], "-", 
      FractionBox[
       RowBox[{"2", " ", 
        RowBox[{
         SubscriptBox["S", "1"], "(", 
         FormBox[
          RowBox[{"m", "-", "1"}],
          TraditionalForm], ")"}]}], 
       SuperscriptBox["m", "2"]], "-", 
      FractionBox["3", 
       SuperscriptBox["m", "2"]], "-", 
      FractionBox[
       RowBox[{
        SubsuperscriptBox["S", "1", "2"], "(", 
        FormBox[
         RowBox[{"m", "-", "1"}],
         TraditionalForm], ")"}], "m"], "+", 
      FractionBox[
       RowBox[{"2", " ", 
        RowBox[{
         SubsuperscriptBox["S", "1", "2"], "(", 
         FormBox[
          RowBox[{"m", "-", "1"}],
          TraditionalForm], ")"}]}], 
       RowBox[{"m", "+", "1"}]], "+", 
      FractionBox[
       RowBox[{"4", " ", 
        RowBox[{
         SubscriptBox["S", "1"], "(", 
         FormBox[
          RowBox[{"m", "-", "1"}],
          TraditionalForm], ")"}]}], 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"m", "+", "1"}], ")"}], "2"]], "-", 
      FractionBox[
       RowBox[{
        SubscriptBox["S", "2"], "(", 
        FormBox[
         RowBox[{"m", "-", "1"}],
         TraditionalForm], ")"}], "m"], "+", 
      FractionBox[
       RowBox[{"2", " ", 
        RowBox[{
         SubscriptBox["S", "2"], "(", 
         FormBox[
          RowBox[{"m", "-", "1"}],
          TraditionalForm], ")"}]}], 
       RowBox[{"m", "+", "1"}]], "-", 
      FractionBox["4", "m"], "+", 
      FractionBox["3", 
       RowBox[{"m", "+", "1"}]], "+", 
      FractionBox["8", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"m", "+", "1"}], ")"}], "2"]], "+", 
      FractionBox["12", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"m", "+", "1"}], ")"}], "3"]]}], ")"}]}], "+", 
   RowBox[{"8", " ", 
    SubscriptBox["C", "F"], " ", 
    SubscriptBox["T", "f"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", 
       FractionBox["2", 
        SuperscriptBox["m", "3"]]}], "-", 
      FractionBox["1", 
       SuperscriptBox["m", "2"]], "+", 
      FractionBox[
       RowBox[{"2", " ", 
        RowBox[{
         SubsuperscriptBox["S", "1", "2"], "(", 
         FormBox[
          RowBox[{"m", "-", "1"}],
          TraditionalForm], ")"}]}], "m"], "-", 
      FractionBox[
       RowBox[{"4", " ", 
        RowBox[{
         SubsuperscriptBox["S", "1", "2"], "(", 
         FormBox[
          RowBox[{"m", "-", "1"}],
          TraditionalForm], ")"}]}], 
       RowBox[{"m", "+", "1"}]], "-", 
      FractionBox[
       RowBox[{"2", " ", 
        RowBox[{
         SubscriptBox["S", "2"], "(", 
         FormBox[
          RowBox[{"m", "-", "1"}],
          TraditionalForm], ")"}]}], "m"], "+", 
      FractionBox[
       RowBox[{"4", " ", 
        RowBox[{
         SubscriptBox["S", "2"], "(", 
         FormBox[
          RowBox[{"m", "-", "1"}],
          TraditionalForm], ")"}]}], 
       RowBox[{"m", "+", "1"}]], "+", 
      FractionBox["14", "m"], "-", 
      FractionBox["19", 
       RowBox[{"m", "+", "1"}]], "-", 
      FractionBox["8", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"m", "+", "1"}], ")"}], "2"]], "+", 
      FractionBox["4", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"m", "+", "1"}], ")"}], "3"]]}], ")"}]}]}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970131677845*^9},
 CellTags->"AnomalousDimension",
 CellLabel->"Out[21]="]
}, Open  ]],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["\[Gamma]", 
    RowBox[{"S", ",", "gq", " "}], 
    RowBox[{
     RowBox[{"(", "1", ")"}], " "}]], TraditionalForm]]],
 "polarized"
}], "Text",
 CellTags->"AnomalousDimension"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"AnomalousDimension", "[", "gsgq1", "]"}]], "Input",
 CellTags->"AnomalousDimension",
 CellLabel->"In[22]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"8", " ", 
    SubscriptBox["C", "A"], " ", 
    SubscriptBox["C", "F"], " ", 
    RowBox[{"(", 
     RowBox[{
      FractionBox[
       RowBox[{"4", " ", 
        RowBox[{
         SubscriptBox[
          OverscriptBox["S", "~"], "2"], "(", 
         FormBox[
          RowBox[{"m", "-", "1"}],
          TraditionalForm], ")"}]}], "m"], "-", 
      FractionBox[
       RowBox[{"2", " ", 
        RowBox[{
         SubscriptBox[
          OverscriptBox["S", "~"], "2"], "(", 
         FormBox[
          RowBox[{"m", "-", "1"}],
          TraditionalForm], ")"}]}], 
       RowBox[{"m", "+", "1"}]], "-", 
      FractionBox["4", 
       SuperscriptBox["m", "3"]], "+", 
      FractionBox["28", 
       RowBox[{"3", " ", 
        SuperscriptBox["m", "2"]}]], "-", 
      FractionBox[
       RowBox[{"2", " ", 
        RowBox[{
         SubsuperscriptBox["S", "1", "2"], "(", 
         FormBox[
          RowBox[{"m", "-", "1"}],
          TraditionalForm], ")"}]}], "m"], "+", 
      FractionBox[
       RowBox[{
        SubsuperscriptBox["S", "1", "2"], "(", 
        FormBox[
         RowBox[{"m", "-", "1"}],
         TraditionalForm], ")"}], 
       RowBox[{"m", "+", "1"}]], "+", 
      FractionBox[
       RowBox[{"16", " ", 
        RowBox[{
         SubscriptBox["S", "1"], "(", 
         FormBox[
          RowBox[{"m", "-", "1"}],
          TraditionalForm], ")"}]}], 
       RowBox[{"3", " ", "m"}]], "-", 
      FractionBox[
       RowBox[{"5", " ", 
        RowBox[{
         SubscriptBox["S", "1"], "(", 
         FormBox[
          RowBox[{"m", "-", "1"}],
          TraditionalForm], ")"}]}], 
       RowBox[{"3", " ", 
        RowBox[{"(", 
         RowBox[{"m", "+", "1"}], ")"}]}]], "+", 
      FractionBox[
       RowBox[{"2", " ", 
        RowBox[{
         SubscriptBox["S", "2"], "(", 
         FormBox[
          RowBox[{"m", "-", "1"}],
          TraditionalForm], ")"}]}], "m"], "-", 
      FractionBox[
       RowBox[{
        SubscriptBox["S", "2"], "(", 
        FormBox[
         RowBox[{"m", "-", "1"}],
         TraditionalForm], ")"}], 
       RowBox[{"m", "+", "1"}]], "-", 
      FractionBox["56", 
       RowBox[{"9", " ", "m"}]], "-", 
      FractionBox["20", 
       RowBox[{"9", " ", 
        RowBox[{"(", 
         RowBox[{"m", "+", "1"}], ")"}]}]], "-", 
      FractionBox["38", 
       RowBox[{"3", " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"m", "+", "1"}], ")"}], "2"]}]], "-", 
      FractionBox["6", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"m", "+", "1"}], ")"}], "3"]]}], ")"}]}], "+", 
   RowBox[{"32", " ", 
    SubscriptBox["C", "F"], " ", 
    SubscriptBox["T", "f"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", 
       FractionBox["2", 
        RowBox[{"3", " ", 
         SuperscriptBox["m", "2"]}]]}], "-", 
      FractionBox[
       RowBox[{"2", " ", 
        RowBox[{
         SubscriptBox["S", "1"], "(", 
         FormBox[
          RowBox[{"m", "-", "1"}],
          TraditionalForm], ")"}]}], 
       RowBox[{"3", " ", "m"}]], "+", 
      FractionBox[
       RowBox[{
        SubscriptBox["S", "1"], "(", 
        FormBox[
         RowBox[{"m", "-", "1"}],
         TraditionalForm], ")"}], 
       RowBox[{"3", " ", 
        RowBox[{"(", 
         RowBox[{"m", "+", "1"}], ")"}]}]], "+", 
      FractionBox["7", 
       RowBox[{"9", " ", "m"}]], "-", 
      FractionBox["2", 
       RowBox[{"9", " ", 
        RowBox[{"(", 
         RowBox[{"m", "+", "1"}], ")"}]}]], "+", 
      FractionBox["1", 
       RowBox[{"3", " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"m", "+", "1"}], ")"}], "2"]}]]}], ")"}]}], "+", 
   RowBox[{"4", " ", 
    SubsuperscriptBox["C", "F", "2"], " ", 
    RowBox[{"(", 
     RowBox[{
      FractionBox["4", 
       SuperscriptBox["m", "3"]], "+", 
      FractionBox[
       RowBox[{"8", " ", 
        RowBox[{
         SubscriptBox["S", "1"], "(", 
         FormBox[
          RowBox[{"m", "-", "1"}],
          TraditionalForm], ")"}]}], 
       SuperscriptBox["m", "2"]], "-", 
      FractionBox["12", 
       SuperscriptBox["m", "2"]], "+", 
      FractionBox[
       RowBox[{"4", " ", 
        RowBox[{
         SubsuperscriptBox["S", "1", "2"], "(", 
         FormBox[
          RowBox[{"m", "-", "1"}],
          TraditionalForm], ")"}]}], "m"], "-", 
      FractionBox[
       RowBox[{"2", " ", 
        RowBox[{
         SubsuperscriptBox["S", "1", "2"], "(", 
         FormBox[
          RowBox[{"m", "-", "1"}],
          TraditionalForm], ")"}]}], 
       RowBox[{"m", "+", "1"}]], "-", 
      FractionBox[
       RowBox[{"8", " ", 
        RowBox[{
         SubscriptBox["S", "1"], "(", 
         FormBox[
          RowBox[{"m", "-", "1"}],
          TraditionalForm], ")"}]}], "m"], "+", 
      FractionBox[
       RowBox[{"2", " ", 
        RowBox[{
         SubscriptBox["S", "1"], "(", 
         FormBox[
          RowBox[{"m", "-", "1"}],
          TraditionalForm], ")"}]}], 
       RowBox[{"m", "+", "1"}]], "-", 
      FractionBox[
       RowBox[{"4", " ", 
        RowBox[{
         SubscriptBox["S", "1"], "(", 
         FormBox[
          RowBox[{"m", "-", "1"}],
          TraditionalForm], ")"}]}], 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"m", "+", "1"}], ")"}], "2"]], "+", 
      FractionBox[
       RowBox[{"4", " ", 
        RowBox[{
         SubscriptBox["S", "2"], "(", 
         FormBox[
          RowBox[{"m", "-", "1"}],
          TraditionalForm], ")"}]}], "m"], "-", 
      FractionBox[
       RowBox[{"2", " ", 
        RowBox[{
         SubscriptBox["S", "2"], "(", 
         FormBox[
          RowBox[{"m", "-", "1"}],
          TraditionalForm], ")"}]}], 
       RowBox[{"m", "+", "1"}]], "+", 
      FractionBox["15", "m"], "-", 
      FractionBox["6", 
       RowBox[{"m", "+", "1"}]], "+", 
      FractionBox["3", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"m", "+", "1"}], ")"}], "2"]], "-", 
      FractionBox["2", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"m", "+", "1"}], ")"}], "3"]]}], ")"}]}]}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.5009701320062513`*^9},
 CellTags->"AnomalousDimension",
 CellLabel->"Out[22]="]
}, Open  ]],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["\[Gamma]", 
    RowBox[{"S", ",", "gg", " "}], 
    RowBox[{
     RowBox[{"(", "1", ")"}], " "}]], TraditionalForm]]],
 "polarized"
}], "Text",
 CellTags->"AnomalousDimension"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"AnomalousDimension", "[", "gsgg1", "]"}]], "Input",
 CellTags->"AnomalousDimension",
 CellLabel->"In[23]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"4", " ", 
    SubsuperscriptBox["C", "A", "2"], " ", 
    RowBox[{"(", 
     RowBox[{
      FractionBox[
       RowBox[{"8", " ", 
        RowBox[{
         SubscriptBox[
          OverscriptBox["S", "~"], "2"], "(", 
         FormBox[
          RowBox[{"m", "-", "1"}],
          TraditionalForm], ")"}]}], "m"], "-", 
      FractionBox[
       RowBox[{"16", " ", 
        RowBox[{
         SubscriptBox[
          OverscriptBox["S", "~"], "2"], "(", 
         FormBox[
          RowBox[{"m", "-", "1"}],
          TraditionalForm], ")"}]}], 
       RowBox[{"m", "+", "1"}]], "+", 
      RowBox[{"4", " ", 
       RowBox[{
        SubscriptBox[
         OverscriptBox["S", "~"], "3"], "(", 
        FormBox[
         RowBox[{"m", "-", "1"}],
         TraditionalForm], ")"}]}], "-", 
      RowBox[{"8", " ", 
       RowBox[{
        SubscriptBox[
         OverscriptBox["S", "~"], "12"], "(", 
        FormBox[
         RowBox[{"m", "-", "1"}],
         TraditionalForm], ")"}]}], "-", 
      FractionBox["8", 
       SuperscriptBox["m", "3"]], "+", 
      FractionBox[
       RowBox[{"8", " ", 
        RowBox[{
         SubscriptBox["S", "1"], "(", 
         FormBox[
          RowBox[{"m", "-", "1"}],
          TraditionalForm], ")"}]}], 
       SuperscriptBox["m", "2"]], "+", 
      FractionBox["58", 
       RowBox[{"3", " ", 
        SuperscriptBox["m", "2"]}]], "-", 
      FractionBox[
       RowBox[{"16", " ", 
        RowBox[{
         SubscriptBox["S", "1"], "(", 
         FormBox[
          RowBox[{"m", "-", "1"}],
          TraditionalForm], ")"}]}], 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"m", "+", "1"}], ")"}], "2"]], "+", 
      RowBox[{
       FractionBox["134", "9"], " ", 
       RowBox[{
        SubscriptBox["S", "1"], "(", 
        FormBox[
         RowBox[{"m", "-", "1"}],
         TraditionalForm], ")"}]}], "+", 
      FractionBox[
       RowBox[{"8", " ", 
        RowBox[{
         SubscriptBox["S", "2"], "(", 
         FormBox[
          RowBox[{"m", "-", "1"}],
          TraditionalForm], ")"}]}], "m"], "-", 
      FractionBox[
       RowBox[{"16", " ", 
        RowBox[{
         SubscriptBox["S", "2"], "(", 
         FormBox[
          RowBox[{"m", "-", "1"}],
          TraditionalForm], ")"}]}], 
       RowBox[{"m", "+", "1"}]], "+", 
      RowBox[{"4", " ", 
       RowBox[{
        SubscriptBox["S", "3"], "(", 
        FormBox[
         RowBox[{"m", "-", "1"}],
         TraditionalForm], ")"}]}], "-", 
      RowBox[{"8", " ", 
       RowBox[{
        SubscriptBox["S", "12"], "(", 
        FormBox[
         RowBox[{"m", "-", "1"}],
         TraditionalForm], ")"}]}], "-", 
      RowBox[{"8", " ", 
       RowBox[{
        SubscriptBox["S", "21"], "(", 
        FormBox[
         RowBox[{"m", "-", "1"}],
         TraditionalForm], ")"}]}], "-", 
      FractionBox["107", 
       RowBox[{"9", " ", "m"}]], "+", 
      FractionBox["241", 
       RowBox[{"9", " ", 
        RowBox[{"(", 
         RowBox[{"m", "+", "1"}], ")"}]}]], "-", 
      FractionBox["86", 
       RowBox[{"3", " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"m", "+", "1"}], ")"}], "2"]}]], "-", 
      FractionBox["48", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"m", "+", "1"}], ")"}], "3"]], "-", 
      FractionBox["16", "3"]}], ")"}]}], "+", 
   RowBox[{"32", " ", 
    SubscriptBox["C", "A"], " ", 
    SubscriptBox["T", "f"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", 
       FractionBox["1", 
        RowBox[{"3", " ", 
         SuperscriptBox["m", "2"]}]]}], "-", 
      RowBox[{
       FractionBox["5", "9"], " ", 
       RowBox[{
        SubscriptBox["S", "1"], "(", 
        FormBox[
         RowBox[{"m", "-", "1"}],
         TraditionalForm], ")"}]}], "+", 
      FractionBox["14", 
       RowBox[{"9", " ", "m"}]], "-", 
      FractionBox["19", 
       RowBox[{"9", " ", 
        RowBox[{"(", 
         RowBox[{"m", "+", "1"}], ")"}]}]], "-", 
      FractionBox["1", 
       RowBox[{"3", " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"m", "+", "1"}], ")"}], "2"]}]], "+", 
      FractionBox["1", "3"]}], ")"}]}], "+", 
   RowBox[{"8", " ", 
    RowBox[{"(", 
     RowBox[{
      FractionBox["4", 
       SuperscriptBox["m", "3"]], "-", 
      FractionBox["10", 
       SuperscriptBox["m", "2"]], "-", 
      FractionBox["10", 
       RowBox[{"m", "+", "1"}]], "+", 
      FractionBox["2", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"m", "+", "1"}], ")"}], "2"]], "+", 
      FractionBox["4", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"m", "+", "1"}], ")"}], "3"]], "+", 
      FractionBox["10", "m"], "+", "1"}], ")"}], " ", 
    SubscriptBox["C", "F"], " ", 
    SubscriptBox["T", "f"]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009701321657124`*^9},
 CellTags->"AnomalousDimension",
 CellLabel->"Out[23]="]
}, Open  ]],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["\[Gamma]", 
    RowBox[{"S", ",", "gg", " "}], 
    RowBox[{
     RowBox[{"(", "1", ")"}], " "}]], TraditionalForm]]],
 "polarized (different representation)"
}], "Text",
 CellTags->"AnomalousDimension"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"AnomalousDimension", "[", "GSGG1", "]"}]], "Input",
 CellTags->"AnomalousDimension",
 CellLabel->"In[24]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"4", " ", 
    SubsuperscriptBox["C", "A", "2"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"8", " ", 
       RowBox[{
        OverscriptBox["S", "~"], "(", 
        FormBox["m",
         TraditionalForm], ")"}]}], "+", 
      FractionBox[
       RowBox[{"8", " ", 
        RowBox[{
         SubsuperscriptBox["S", "2", "'"], "\[NoBreak]", "(", "\[NoBreak]", 
         FormBox[
          FractionBox["m", "2"],
          TraditionalForm], "\[NoBreak]", ")"}]}], 
       RowBox[{"m", " ", 
        RowBox[{"(", 
         RowBox[{"m", "+", "1"}], ")"}]}]], "-", 
      RowBox[{
       SubsuperscriptBox["S", "3", "'"], "\[NoBreak]", "(", "\[NoBreak]", 
       FormBox[
        FractionBox["m", "2"],
        TraditionalForm], "\[NoBreak]", ")"}], "-", 
      RowBox[{"4", " ", 
       RowBox[{
        SubscriptBox["S", "1"], "(", 
        FormBox["m",
         TraditionalForm], ")"}], " ", 
       RowBox[{
        SubsuperscriptBox["S", "2", "'"], "\[NoBreak]", "(", "\[NoBreak]", 
        FormBox[
         FractionBox["m", "2"],
         TraditionalForm], "\[NoBreak]", ")"}]}], "-", 
      FractionBox[
       RowBox[{
        RowBox[{"m", " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"m", " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{"m", " ", 
               RowBox[{"(", 
                RowBox[{
                 RowBox[{"m", " ", 
                  RowBox[{"(", 
                   RowBox[{
                    RowBox[{"48", " ", "m", " ", 
                    RowBox[{"(", 
                    RowBox[{"m", "+", "3"}], ")"}]}], "+", "469"}], ")"}]}], 
                 "+", "698"}], ")"}]}], "+", "7"}], ")"}]}], "+", "258"}], 
          ")"}]}], "+", "144"}], 
       RowBox[{"9", " ", 
        SuperscriptBox["m", "3"], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"m", "+", "1"}], ")"}], "3"]}]], "+", 
      FractionBox[
       RowBox[{"2", " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"m", " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"67", " ", "m", " ", 
              SuperscriptBox[
               RowBox[{"(", 
                RowBox[{"m", "+", "1"}], ")"}], "2"]}], "+", "144"}], ")"}]}],
           "+", "72"}], ")"}], " ", 
        RowBox[{
         SubscriptBox["S", "1"], "(", 
         FormBox["m",
          TraditionalForm], ")"}]}], 
       RowBox[{"9", " ", 
        SuperscriptBox["m", "2"], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"m", "+", "1"}], ")"}], "2"]}]]}], ")"}]}], "+", 
   RowBox[{"32", " ", 
    SubscriptBox["C", "A"], " ", 
    SubscriptBox["T", "f"], " ", 
    RowBox[{"(", 
     RowBox[{
      FractionBox[
       RowBox[{
        RowBox[{"m", " ", 
         RowBox[{"(", 
          RowBox[{"m", "+", "1"}], ")"}], " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"3", " ", "m", " ", 
            RowBox[{"(", 
             RowBox[{"m", "+", "1"}], ")"}]}], "+", "13"}], ")"}]}], "-", 
        "3"}], 
       RowBox[{"9", " ", 
        SuperscriptBox["m", "2"], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"m", "+", "1"}], ")"}], "2"]}]], "-", 
      FractionBox[
       RowBox[{"5", " ", 
        RowBox[{
         SubscriptBox["S", "1"], "(", 
         FormBox["m",
          TraditionalForm], ")"}]}], "9"]}], ")"}]}], "+", 
   FractionBox[
    RowBox[{"8", " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"m", " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"m", " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"m", " ", 
              RowBox[{"(", 
               RowBox[{
                RowBox[{"m", " ", 
                 RowBox[{"(", 
                  RowBox[{
                   RowBox[{"m", " ", 
                    RowBox[{"(", 
                    RowBox[{"m", "+", "3"}], ")"}]}], "+", "5"}], ")"}]}], 
                "+", "1"}], ")"}]}], "-", "8"}], ")"}]}], "+", "2"}], ")"}]}],
        "+", "4"}], ")"}], " ", 
     SubscriptBox["C", "F"], " ", 
     SubscriptBox["T", "f"]}], 
    RowBox[{
     SuperscriptBox["m", "3"], " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"m", "+", "1"}], ")"}], "3"]}]]}], TraditionalForm]], "Output",\

 CellChangeTimes->{3.500970132546471*^9},
 CellTags->"AnomalousDimension",
 CellLabel->"Out[24]="]
}, Open  ]],

Cell[TextData[{
 "Check that all odd moments give the same for the two representations of ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubsuperscriptBox["\[Gamma]", 
     RowBox[{"S", ",", "gg", " "}], 
     RowBox[{
      RowBox[{"(", "1", ")"}], " "}]], "."}], TraditionalForm]]]
}], "Text",
 CellTags->"AnomalousDimension"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Table", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"%", "-", "%%"}], "/.", 
    RowBox[{"OPEm", "\[Rule]", "ij"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"ij", ",", "1", ",", "17", ",", "2"}], "}"}]}], "]"}]], "Input",
 CellTags->"AnomalousDimension",
 CellLabel->"In[25]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
   "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", 
    ",", "0"}], "}"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970132705515*^9},
 CellTags->"AnomalousDimension",
 CellLabel->"Out[25]="]
}, Open  ]]
}, Open  ]],

Cell["AntiCommutator", "ObjectName",
 CellTags->"AntiCommutator"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"AntiCommutator"],

Cell["\<\
AntiCommutator[x, y] = c defines the anti-commutator of the non commuting \
objects x and y.\
\>", "Text",
 CellTags->"AntiCommutator"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Commutator",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Commutator",
  ButtonNote->"Commutator"],
 ", ",
 ButtonBox["CommutatorExplicit",
  BaseStyle->"AddOnsLink",
  ButtonData:>"CommutatorExplicit",
  ButtonNote->"CommutatorExplicit"],
 ", ",
 ButtonBox["DeclareNonCommutative",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DeclareNonCommutative",
  ButtonNote->"DeclareNonCommutative"],
 ", ",
 ButtonBox["DotSimplify",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DotSimplify",
  ButtonNote->"DotSimplify"],
 "."
}], "Text",
 CellTags->"AntiCommutator"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"AntiCommutator"],

Cell[TextData[{
 "This declares ",
 StyleBox["a",
  FontFamily->"Courier"],
 " and ",
 StyleBox["b",
  FontFamily->"Courier"],
 " as noncommutative variables."
}], "Text",
 CellTags->"AntiCommutator"],

Cell[BoxData[
 RowBox[{"DeclareNonCommutative", "[", 
  RowBox[{"a", ",", "b"}], "]"}]], "Input",
 CellTags->"AntiCommutator",
 CellLabel->"In[26]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"AntiCommutator", "[", 
  RowBox[{"a", ",", "b"}], "]"}]], "Input",
 CellTags->"AntiCommutator",
 CellLabel->"In[27]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", "a", ",", "\[MediumSpace]", "b", "}"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970132905449*^9},
 CellTags->"AntiCommutator",
 CellLabel->"Out[27]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CommutatorExplicit", "[", "%", "]"}]], "Input",
 CellTags->"AntiCommutator",
 CellLabel->"In[28]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"a", ".", "b"}], "+", 
   RowBox[{"b", ".", "a"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970132948254*^9},
 CellTags->"AntiCommutator",
 CellLabel->"Out[28]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CommutatorExplicit", "[", 
  RowBox[{"AntiCommutator", "[", 
   RowBox[{
    RowBox[{"a", "+", "b"}], ",", 
    RowBox[{"a", "-", 
     RowBox[{"2", "b"}]}]}], " ", "]"}], "]"}]], "Input",
 CellTags->"AntiCommutator",
 CellLabel->"In[29]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{"a", "-", 
      RowBox[{"2", " ", "b"}]}], ")"}], ".", 
    RowBox[{"(", 
     RowBox[{"a", "+", "b"}], ")"}]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"a", "+", "b"}], ")"}], ".", 
    RowBox[{"(", 
     RowBox[{"a", "-", 
      RowBox[{"2", " ", "b"}]}], ")"}]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970133054281*^9},
 CellTags->"AntiCommutator",
 CellLabel->"Out[29]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DotSimplify", "[", 
  RowBox[{"AntiCommutator", "[", 
   RowBox[{
    RowBox[{"a", "+", "b"}], ",", 
    RowBox[{"a", "-", 
     RowBox[{"2", "b"}]}]}], " ", "]"}], "]"}]], "Input",
 CellTags->"AntiCommutator",
 CellLabel->"In[30]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    RowBox[{"a", ".", "b"}]}], "-", 
   RowBox[{"b", ".", "a"}], "+", 
   RowBox[{"2", " ", 
    RowBox[{"a", ".", "a"}]}], "-", 
   RowBox[{"4", " ", 
    RowBox[{"b", ".", "b"}]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009701331601458`*^9},
 CellTags->"AntiCommutator",
 CellLabel->"Out[30]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"DeclareNonCommutative", "[", 
  RowBox[{"c", ",", "d", ",", 
   OverscriptBox["c", "~"], ",", 
   OverscriptBox["d", "~"]}], "]"}]], "Input",
 CellTags->"AntiCommutator",
 CellLabel->"In[31]:="],

Cell["Defining {c,d} = z results in replacements of c.d by z-d.c.", "Text",
 CellTags->"AntiCommutator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"AntiCommutator", "[", 
   RowBox[{"c", ",", "d"}], "]"}], " ", "=", " ", "z"}]], "Input",
 CellTags->"AntiCommutator",
 CellLabel->"In[32]:="],

Cell[BoxData[
 FormBox["z", TraditionalForm]], "Output",
 CellChangeTimes->{3.500970133314476*^9},
 CellTags->"AntiCommutator",
 CellLabel->"Out[32]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DotSimplify", "[", " ", 
  RowBox[{"d", " ", ".", " ", "c", " ", ".", " ", "d"}], " ", "]"}]], "Input",\

 CellTags->"AntiCommutator",
 CellLabel->"In[33]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"d", " ", "z"}], "-", 
   RowBox[{"d", ".", "d", ".", "c"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970133509025*^9},
 CellTags->"AntiCommutator",
 CellLabel->"Out[33]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"AntiCommutator", "[", 
   RowBox[{
    OverscriptBox["d", "~"], ",", 
    OverscriptBox["c", "~"]}], "]"}], " ", "=", " ", 
  OverscriptBox["z", "~"]}]], "Input",
 CellTags->"AntiCommutator",
 CellLabel->"In[34]:="],

Cell[BoxData[
 FormBox[
  OverscriptBox["z", "~"], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970133665078*^9},
 CellTags->"AntiCommutator",
 CellLabel->"Out[34]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DotSimplify", "[", " ", 
  RowBox[{
   OverscriptBox["d", "~"], " ", ".", " ", 
   OverscriptBox["c", "~"], " ", ".", " ", 
   OverscriptBox["d", "~"]}], " ", "]"}]], "Input",
 CellTags->"AntiCommutator",
 CellLabel->"In[35]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    OverscriptBox["d", "~"], " ", 
    OverscriptBox["z", "~"]}], "-", 
   RowBox[{
    OverscriptBox["c", "~"], ".", 
    OverscriptBox["d", "~"], ".", 
    OverscriptBox["d", "~"]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009701337613363`*^9},
 CellTags->"AntiCommutator",
 CellLabel->"Out[35]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"UnDeclareNonCommutative", "[", 
  RowBox[{"a", ",", "b", ",", "c", ",", "d", ",", 
   OverscriptBox["c", "~"], ",", 
   OverscriptBox["d", "~"]}], "]"}]], "Input",
 CellTags->"AntiCommutator",
 CellLabel->"In[36]:="],

Cell[BoxData[
 RowBox[{"Unset", "[", 
  RowBox[{"AntiCommutator", "[", 
   RowBox[{"c", ",", "d"}], "]"}], "]"}]], "Input",
 CellTags->"AntiCommutator",
 CellLabel->"In[37]:="],

Cell[BoxData[
 RowBox[{"Unset", "[", 
  RowBox[{"AntiCommutator", "[", 
   RowBox[{
    OverscriptBox["d", "~"], ",", 
    OverscriptBox["c", "~"]}], "]"}], "]"}]], "Input",
 CellTags->"AntiCommutator",
 CellLabel->"In[38]:="]
}, Open  ]],

Cell["AntiQuarkField", "ObjectName",
 CellTags->"AntiQuarkField"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"AntiQuarkField"],

Cell["\<\
AntiQuarkField is the name of a fermionic field. AntiQuarkField is just a \
name with no functional properties. Only typeset rules are attached.\
\>", "Text",
 CellTags->"AntiQuarkField"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["QuantumField",
  BaseStyle->"AddOnsLink",
  ButtonData:>"QuantumField",
  ButtonNote->"QuantumField"],
 ", ",
 ButtonBox["QuarkField",
  BaseStyle->"AddOnsLink",
  ButtonData:>"QuarkField",
  ButtonNote->"QuarkField"],
 "."
}], "Text",
 CellTags->"AntiQuarkField"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"AntiQuarkField"],

Cell[CellGroupData[{

Cell[BoxData["AntiQuarkField"], "Input",
 CellTags->"AntiQuarkField",
 CellLabel->"In[39]:="],

Cell[BoxData[
 FormBox[
  OverscriptBox["\[Psi]", "_"], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970133957492*^9},
 CellTags->"AntiQuarkField",
 CellLabel->"Out[39]="]
}, Open  ]]
}, Open  ]],

Cell["AntiSymmetrize", "ObjectName",
 CellTags->"AntiSymmetrize"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"AntiSymmetrize"],

Cell["\<\
AntiSymmetrize[expr, {a1, a2, ...}] antisymmetrizes expr with respect to the \
variables a1,a2, ... \
\>", "Text",
 CellTags->"AntiSymmetrize"],

Cell[TextData[{
 "See also: ",
 ButtonBox["Symmetrize",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Symmetrize",
  ButtonNote->"Symmetrize"],
 "."
}], "Text",
 CellTags->"AntiSymmetrize"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"AntiSymmetrize"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"AntiSymmetrize", "[", 
  RowBox[{
   RowBox[{"f", "[", 
    RowBox[{"a", ",", "b"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"a", ",", "b"}], "}"}]}], "]"}]], "Input",
 CellTags->"AntiSymmetrize",
 CellLabel->"In[40]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["1", "2"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"f", "(", 
      RowBox[{"a", ",", "b"}], ")"}], "-", 
     RowBox[{"f", "(", 
      RowBox[{"b", ",", "a"}], ")"}]}], ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970134056431*^9},
 CellTags->"AntiSymmetrize",
 CellLabel->"Out[40]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"AntiSymmetrize", "[", 
  RowBox[{
   RowBox[{"f", "[", 
    RowBox[{"x", ",", "y", ",", "z"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "y", ",", "z"}], "}"}]}], "]"}]], "Input",
 CellTags->"AntiSymmetrize",
 CellLabel->"In[41]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["1", "6"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"f", "(", 
      RowBox[{"x", ",", "y", ",", "z"}], ")"}], "-", 
     RowBox[{"f", "(", 
      RowBox[{"x", ",", "z", ",", "y"}], ")"}], "-", 
     RowBox[{"f", "(", 
      RowBox[{"y", ",", "x", ",", "z"}], ")"}], "+", 
     RowBox[{"f", "(", 
      RowBox[{"y", ",", "z", ",", "x"}], ")"}], "+", 
     RowBox[{"f", "(", 
      RowBox[{"z", ",", "x", ",", "y"}], ")"}], "-", 
     RowBox[{"f", "(", 
      RowBox[{"z", ",", "y", ",", "x"}], ")"}]}], ")"}]}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970134156501*^9},
 CellTags->"AntiSymmetrize",
 CellLabel->"Out[41]="]
}, Open  ]]
}, Open  ]],

Cell["Anti5", "ObjectName",
 CellTags->"Anti5"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Anti5"],

Cell[TextData[{
 "Anti5[exp] anticommutes all ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["\[Gamma]", "5"], TraditionalForm]]],
 "in exp to the right. Anti5[exp, n] anticommutes all ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["\[Gamma]", 
    RowBox[{"5", " "}]], TraditionalForm]]],
 "n times to the right. Anti5[exp, -n] anticommutes all ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["\[Gamma]", "5"], TraditionalForm]]],
 " n times to the left."
}], "Text",
 CellTags->"Anti5"],

Cell[TextData[{
 "The naive ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["\[Gamma]", "5"], TraditionalForm]]],
 "scheme is used."
}], "Text",
 CellTags->"Anti5"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["DiracOrder",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracOrder",
  ButtonNote->"DiracOrder"],
 ", ",
 ButtonBox["DiracSimplify",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracSimplify",
  ButtonNote->"DiracSimplify"],
 ", ",
 ButtonBox["DiracTrick",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracTrick",
  ButtonNote->"DiracTrick"],
 "."
}], "Text",
 CellTags->"Anti5"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Anti5"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"DiracMatrix", "[", 
   RowBox[{"5", ",", "\[Mu]"}], "]"}], " "}]], "Input",
 CellTags->"Anti5",
 CellLabel->"In[42]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", "5"], 
   SuperscriptBox["\[Gamma]", "\[Mu]"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970134242449*^9},
 CellTags->"Anti5",
 CellLabel->"Out[42]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Anti5", "[", "%", "]"}]], "Input",
 CellTags->"Anti5",
 CellLabel->"In[43]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   RowBox[{
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], ".", 
    SuperscriptBox["\[Gamma]", "5"]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970134349182*^9},
 CellTags->"Anti5",
 CellLabel->"Out[43]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Anti5", "[", 
  RowBox[{"%", ",", 
   RowBox[{"-", "1"}]}], "]"}]], "Input",
 CellTags->"Anti5",
 CellLabel->"In[44]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", "5"], ".", 
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970134452156*^9},
 CellTags->"Anti5",
 CellLabel->"Out[44]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracMatrix", "[", 
  RowBox[{
  "5", ",", "\[Alpha]", ",", "\[Beta]", ",", "\[Gamma]", ",", "\[Delta]"}], 
  "]"}]], "Input",
 CellTags->"Anti5",
 CellLabel->"In[45]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", "5"], 
   SuperscriptBox["\[Gamma]", "\[Alpha]"], 
   SuperscriptBox["\[Gamma]", "\[Beta]"], 
   SuperscriptBox["\[Gamma]", "\[Gamma]"], 
   SuperscriptBox["\[Gamma]", "\[Delta]"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970134528452*^9},
 CellTags->"Anti5",
 CellLabel->"Out[45]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Anti5", "[", 
  RowBox[{"%", ",", "2"}], "]"}]], "Input",
 CellTags->"Anti5",
 CellLabel->"In[46]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", "5"], 
   SuperscriptBox["\[Gamma]", "\[Alpha]"], 
   SuperscriptBox["\[Gamma]", "\[Beta]"], 
   SuperscriptBox["\[Gamma]", "\[Gamma]"], 
   SuperscriptBox["\[Gamma]", "\[Delta]"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970134604603*^9},
 CellTags->"Anti5",
 CellLabel->"Out[46]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Anti5", "[", 
  RowBox[{"%%", ",", "Infinity"}], "]"}]], "Input",
 CellTags->"Anti5",
 CellLabel->"In[47]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", "5"], 
   SuperscriptBox["\[Gamma]", "\[Alpha]"], 
   SuperscriptBox["\[Gamma]", "\[Beta]"], 
   SuperscriptBox["\[Gamma]", "\[Gamma]"], 
   SuperscriptBox["\[Gamma]", "\[Delta]"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970134703868*^9},
 CellTags->"Anti5",
 CellLabel->"Out[47]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Anti5", "[", 
  RowBox[{"%", ",", 
   RowBox[{"-", "Infinity"}]}], "]"}]], "Input",
 CellTags->"Anti5",
 CellLabel->"In[48]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", "5"], 
   SuperscriptBox["\[Gamma]", "\[Alpha]"], 
   SuperscriptBox["\[Gamma]", "\[Beta]"], 
   SuperscriptBox["\[Gamma]", "\[Gamma]"], 
   SuperscriptBox["\[Gamma]", "\[Delta]"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009701348379087`*^9},
 CellTags->"Anti5",
 CellLabel->"Out[48]="]
}, Open  ]],

Cell[TextData[{
 "In the naive ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["\[Gamma]", "5"], TraditionalForm]]],
 "- scheme D-dimensional ",
 Cell[BoxData[
  FormBox["\[Gamma]", TraditionalForm]]],
 "-matrices anticommute with ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["\[Gamma]", "5"], TraditionalForm]]],
 "."
}], "Text",
 CellTags->"Anti5"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GAD", "[", 
  RowBox[{"5", ",", "\[Mu]"}], "]"}]], "Input",
 CellTags->"Anti5",
 CellLabel->"In[49]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", "5"], ".", 
   SuperscriptBox["\[Gamma]", "\[Mu]"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970135354322*^9},
 CellTags->"Anti5",
 CellLabel->"Out[49]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Anti5", "[", "%", "]"}]], "Input",
 CellTags->"Anti5",
 CellLabel->"In[50]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   RowBox[{
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], ".", 
    SuperscriptBox["\[Gamma]", "5"]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970135424013*^9},
 CellTags->"Anti5",
 CellLabel->"Out[50]="]
}, Open  ]]
}, Open  ]],

Cell["Apart2", "ObjectName",
 CellTags->"Apart2"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Apart2"],

Cell["\<\
Apart2[expr] partial fractions FeynAmpDenominators (and FAD's).\
\>", "Text",
 CellTags->"Apart2"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FAD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FAD",
  ButtonNote->"FAD"],
 ", ",
 ButtonBox["FeynAmpDenominator",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FeynAmpDenominator",
  ButtonNote->"FeynAmpDenominator"],
 "."
}], "Text",
 CellTags->"Apart2"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Apart2"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FAD", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"q", ",", "m"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"q", ",", "M"}], "}"}], ",", 
   RowBox[{"q", "-", "p"}]}], "]"}]], "Input",
 CellTags->"Apart2",
 CellLabel->"In[51]:="],

Cell[BoxData[
 FormBox[
  FractionBox["1", 
   RowBox[{
    RowBox[{"(", 
     InterpretationBox[
      RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"q", "-", "p"}], ")"}], "2"], 
       "\[InvisibleSpace]", "\<\"]\"\>"}],
      SequenceForm["[", (-$CellContext`p + $CellContext`q)^2, "]"],
      Editable->False], ")"}], " ", 
    RowBox[{"(", 
     InterpretationBox[
      RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
       SuperscriptBox["q", "2"], "\[InvisibleSpace]", "\<\"-\"\>", 
       "\[InvisibleSpace]", 
       SuperscriptBox["m", "2"], "\[InvisibleSpace]", "\<\"]\"\>"}],
      SequenceForm["[", $CellContext`q^2, "-", $CellContext`m^2, "]"],
      Editable->False], ")"}], " ", 
    RowBox[{"(", 
     InterpretationBox[
      RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
       SuperscriptBox["q", "2"], "\[InvisibleSpace]", "\<\"-\"\>", 
       "\[InvisibleSpace]", 
       SuperscriptBox["M", "2"], "\[InvisibleSpace]", "\<\"]\"\>"}],
      SequenceForm["[", $CellContext`q^2, "-", $CellContext`M^2, "]"],
      Editable->False], ")"}]}]], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009701355101748`*^9},
 CellTags->"Apart2",
 CellLabel->"Out[51]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Apart2", "[", "%", "]"}]], "Input",
 CellTags->"Apart2",
 CellLabel->"In[52]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{
    FractionBox["1", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox["q", "2"], "-", 
        SuperscriptBox["m", "2"]}], ")"}], ".", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"q", "-", "p"}], ")"}], "2"]}]], "-", 
    FractionBox["1", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox["q", "2"], "-", 
        SuperscriptBox["M", "2"]}], ")"}], ".", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"q", "-", "p"}], ")"}], "2"]}]]}], 
   RowBox[{
    SuperscriptBox["m", "2"], "-", 
    SuperscriptBox["M", "2"]}]], TraditionalForm]], "Output",
 CellChangeTimes->{3.50097013560802*^9},
 CellTags->"Apart2",
 CellLabel->"Out[52]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StandardForm", "[", 
  RowBox[{"FCE", "[", "%", "]"}], "]"}]], "Input",
 CellTags->"Apart2",
 CellLabel->"In[53]:="],

Cell[BoxData[
 FractionBox[
  RowBox[{
   RowBox[{"FAD", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"q", ",", "m"}], "}"}], ",", 
     RowBox[{
      RowBox[{"-", "p"}], "+", "q"}]}], "]"}], "-", 
   RowBox[{"FAD", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"q", ",", "M"}], "}"}], ",", 
     RowBox[{
      RowBox[{"-", "p"}], "+", "q"}]}], "]"}]}], 
  RowBox[{
   SuperscriptBox["m", "2"], "-", 
   SuperscriptBox["M", "2"]}]]], "Output",
 CellChangeTimes->{3.5009701356924334`*^9},
 CellTags->"Apart2",
 CellLabel->"Out[53]//StandardForm="]
}, Open  ]]
}, Open  ]],

Cell["A0", "ObjectName",
 PageBreakAbove->True,
 CellTags->"A0"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"A0"],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   RowBox[{"A0", "[", 
    SuperscriptBox["m", "2"], "]"}], TraditionalForm]]],
 "is the Passarino-Veltman one-point integral ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["A", "0"], "."}], TraditionalForm]]]
}], "Text",
 CellTags->"A0"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", " ", "A0", "]"}]], "Input",
 CellTags->"A0",
 CellLabel->"In[54]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"A0ToB0", "\[Rule]", "False"}], "}"}], TraditionalForm]], "Output",\

 CellChangeTimes->{3.500970135781652*^9},
 CellTags->"A0",
 CellLabel->"Out[54]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["B0",
  BaseStyle->"AddOnsLink",
  ButtonData:>"B0",
  ButtonNote->"B0"],
 ", ",
 ButtonBox["C0",
  BaseStyle->"AddOnsLink",
  ButtonData:>"C0",
  ButtonNote->"C0"],
 ", ",
 ButtonBox["D0",
  BaseStyle->"AddOnsLink",
  ButtonData:>"D0",
  ButtonNote->"D0"],
 ", ",
 ButtonBox["PaVe",
  BaseStyle->"AddOnsLink",
  ButtonData:>"PaVe",
  ButtonNote->"PaVe"],
 "."
}], "Text",
 CellTags->"A0"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"A0"],

Cell[TextData[{
 "By default ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["A", "0"], TraditionalForm]]],
 " is not expressed in terms of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["B", "0"], TraditionalForm]]],
 "."
}], "Text",
 CellTags->"A0"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"A0", "[", 
  SuperscriptBox["m", "2"], "]"}]], "Input",
 CellTags->"A0",
 CellLabel->"In[55]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox[
    SubscriptBox["\<\"A\"\>", "\<\"0\"\>"],
    TraditionalForm], "(", 
   FormBox[
    SuperscriptBox["m", "2"],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970135866703*^9},
 CellTags->"A0",
 CellLabel->"Out[55]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"SetOptions", "[", 
   RowBox[{"A0", ",", 
    RowBox[{"A0ToB0", "\[Rule]", "True"}]}], "]"}], ";"}]], "Input",
 CellTags->"A0",
 CellLabel->"In[56]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"A0", "[", 
  SuperscriptBox["m", "2"], "]"}]], "Input",
 CellTags->"A0",
 CellLabel->"In[57]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SuperscriptBox["m", "2"], " ", 
    RowBox[{
     SubscriptBox["B", "0"], "(", "0", ",", 
     SuperscriptBox["m", "2"], ",", 
     SuperscriptBox["m", "2"], ")"}]}], "+", 
   SuperscriptBox["m", "2"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970136152454*^9},
 CellTags->"A0",
 CellLabel->"Out[57]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"SetOptions", "[", 
   RowBox[{"A0", ",", 
    RowBox[{"A0ToB0", "\[Rule]", "False"}]}], "]"}], ";"}]], "Input",
 CellTags->"A0",
 CellLabel->"In[58]:="],

Cell[TextData[{
 "According to the rules of dimensional regularization ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["A", "0"], "(", "0", ")"}], TraditionalForm]]],
 " is set to 0."
}], "Text",
 CellTags->"A0"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"A0", "[", "0", "]"}]], "Input",
 CellTags->"A0",
 CellLabel->"In[59]:="],

Cell[BoxData[
 FormBox["0", TraditionalForm]], "Output",
 CellChangeTimes->{3.500970136353944*^9},
 CellTags->"A0",
 CellLabel->"Out[59]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"A0", "[", 
  RowBox[{"SmallVariable", "[", 
   RowBox[{"M", "^", "2"}], "]"}], "]"}]], "Input",
 CellTags->"A0",
 CellLabel->"In[60]:="],

Cell[BoxData[
 FormBox["0", TraditionalForm]], "Output",
 CellChangeTimes->{3.500970136427803*^9},
 CellTags->"A0",
 CellLabel->"Out[60]="]
}, Open  ]]
}, Open  ]],

Cell["A0ToB0", "ObjectName",
 PageBreakAbove->True,
 CellTags->"A0ToB0"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"A0ToB0"],

Cell[TextData[{
 "A0ToB0 is an option for A0. If set to True, A0[",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    StyleBox["m",
     FontSlant->"Plain"], "2"], TraditionalForm]]],
 "] is expressed by (1 + B0[0, ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    StyleBox["m",
     FontSlant->"Plain"], "2"], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    StyleBox["m",
     FontSlant->"Plain"], "2"], TraditionalForm]]],
 "]) ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    StyleBox["m",
     FontSlant->"Plain"], "2"], TraditionalForm]]],
 "."
}], "Text",
 CellTags->"A0ToB0"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", " ", "A0", "]"}]], "Input",
 CellTags->"A0ToB0",
 CellLabel->"In[61]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"A0ToB0", "\[Rule]", "False"}], "}"}], TraditionalForm]], "Output",\

 CellChangeTimes->{3.5009701364712152`*^9},
 CellTags->"A0ToB0",
 CellLabel->"Out[61]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["A0",
  BaseStyle->"AddOnsLink",
  ButtonData:>"A0",
  ButtonNote->"A0"],
 ", ",
 ButtonBox["B0",
  BaseStyle->"AddOnsLink",
  ButtonData:>"B0",
  ButtonNote->"B0"],
 ", ",
 ButtonBox["C0",
  BaseStyle->"AddOnsLink",
  ButtonData:>"C0",
  ButtonNote->"C0"],
 ", ",
 ButtonBox["D0",
  BaseStyle->"AddOnsLink",
  ButtonData:>"D0",
  ButtonNote->"D0"],
 ", ",
 ButtonBox["PaVe",
  BaseStyle->"AddOnsLink",
  ButtonData:>"PaVe",
  ButtonNote->"PaVe"],
 "."
}], "Text",
 CellTags->"A0ToB0"]
}, Open  ]],

Cell["BackgroundGluonVertex", "ObjectName",
 CellTags->"BackgroundGluonVertex"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"BackgroundGluonVertex"],

Cell[TextData[{
 "BackgroundGluonVertex[{p,mu,a}, {q,nu,b}, {k,la,c}] or \
BackgroundGluonVertex[ p,mu,a , q,nu,b , k,la,c ] yields the 3-gluon vertex \
in the background field gauge, where the first set of arguments corresponds \
to the external background field. BackgroundGluonVertex[{p,mu,a}, {q,nu,b}, \
{k,la,c}, {s,si,d}] or BackgroundGluonVertex[{mu,a}, {nu,b}, {la,c}, {",
 "si",
 ",d}] or BackgroundGluonVertex[p,mu,a , q,nu,b , k,la,c , s,si,d] or \
BackgroundGluonVertex[ mu,a , nu,b , la,c , si,d ] yields the 4-gluon vertex, \
with {p,mu,a} and {k,la,c} denoting the external background fields. The \
gauge, dimension and the name of the coupling constant are determined by the \
options Gauge, Dimension and CouplingConstant. The Feynman rules are taken \
from L.Abbot ",
 StyleBox["NPB ",
  FontWeight->"Bold"],
 "185 (1981), 189-203; except that all momenta are incoming. Note that Abbots \
coupling constant convention is consistent with the default setting of \
GluonVertex."
}], "Text",
 CellTags->"BackgroundGluonVertex"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "BackgroundGluonVertex", "]"}]], "Input",
 CellTags->"BackgroundGluonVertex",
 CellLabel->"In[62]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
    RowBox[{"CouplingConstant", "\[Rule]", 
     SubscriptBox["g", "s"]}], ",", 
    RowBox[{"Gauge", "\[Rule]", "1"}]}], "}"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009701366348467`*^9},
 CellTags->"BackgroundGluonVertex",
 CellLabel->"Out[62]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["GluonVertex",
  BaseStyle->"AddOnsLink",
  ButtonData:>"GluonVertex",
  ButtonNote->"GluonVertex"],
 "."
}], "Text",
 CellTags->"BackgroundGluonVertex"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"BackgroundGluonVertex"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BackgroundGluonVertex", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"p", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"q", ",", "\[Nu]", ",", "b"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"k", ",", "\[Lambda]", ",", "c"}], "}"}]}], "]"}]], "Input",
 CellTags->"BackgroundGluonVertex",
 CellLabel->"In[63]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["g", "s"], " ", 
   SubscriptBox["f", 
    RowBox[{
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm]}]], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        FormBox[
         RowBox[{
          RowBox[{"-", "k"}], "+", "p", "-", "q"}],
         TraditionalForm], ")"}], 
       FormBox[
        FormBox["\[Lambda]",
         TraditionalForm],
        TraditionalForm]]}], "+", 
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Lambda]", "\[Mu]"}]], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        FormBox[
         RowBox[{"k", "-", "p", "+", "q"}],
         TraditionalForm], ")"}], 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]]}], "+", 
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Lambda]", "\[Nu]"}]], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        FormBox[
         RowBox[{"q", "-", "k"}],
         TraditionalForm], ")"}], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]]}]}], ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970136752479*^9},
 CellTags->"BackgroundGluonVertex",
 CellLabel->"Out[63]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BackgroundGluonVertex", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"p", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"q", ",", "\[Nu]", ",", "b"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"k", ",", "\[Lambda]", ",", "c"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"s", ",", "\[Sigma]", ",", "d"}], "}"}]}], "]"}]], "Input",
 CellTags->"BackgroundGluonVertex",
 CellLabel->"In[64]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", "\[ImaginaryI]"}], " ", 
   SubsuperscriptBox["g", "s", "2"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["a",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["d",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["u6",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["b",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["c",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["u6",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         SuperscriptBox["g", 
          RowBox[{"\[Lambda]", "\[Sigma]"}]], " ", 
         SuperscriptBox["g", 
          RowBox[{"\[Mu]", "\[Nu]"}]]}], "-", 
        RowBox[{
         SuperscriptBox["g", 
          RowBox[{"\[Lambda]", "\[Nu]"}]], " ", 
         SuperscriptBox["g", 
          RowBox[{"\[Mu]", "\[Sigma]"}]]}], "-", 
        RowBox[{
         SuperscriptBox["g", 
          RowBox[{"\[Lambda]", "\[Mu]"}]], " ", 
         SuperscriptBox["g", 
          RowBox[{"\[Nu]", "\[Sigma]"}]]}]}], ")"}]}], "+", 
     RowBox[{
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["a",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["c",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["u6",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["b",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["d",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["u6",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         SuperscriptBox["g", 
          RowBox[{"\[Lambda]", "\[Sigma]"}]], " ", 
         SuperscriptBox["g", 
          RowBox[{"\[Mu]", "\[Nu]"}]]}], "-", 
        RowBox[{
         SuperscriptBox["g", 
          RowBox[{"\[Lambda]", "\[Nu]"}]], " ", 
         SuperscriptBox["g", 
          RowBox[{"\[Mu]", "\[Sigma]"}]]}]}], ")"}]}], "+", 
     RowBox[{
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["a",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["b",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["u6",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["c",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["d",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["u6",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         SuperscriptBox["g", 
          RowBox[{"\[Lambda]", "\[Sigma]"}]], " ", 
         SuperscriptBox["g", 
          RowBox[{"\[Mu]", "\[Nu]"}]]}], "-", 
        RowBox[{
         SuperscriptBox["g", 
          RowBox[{"\[Lambda]", "\[Nu]"}]], " ", 
         SuperscriptBox["g", 
          RowBox[{"\[Mu]", "\[Sigma]"}]]}], "+", 
        RowBox[{
         SuperscriptBox["g", 
          RowBox[{"\[Lambda]", "\[Mu]"}]], " ", 
         SuperscriptBox["g", 
          RowBox[{"\[Nu]", "\[Sigma]"}]]}]}], ")"}]}]}], ")"}]}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970137098144*^9},
 CellTags->"BackgroundGluonVertex",
 CellLabel->"Out[64]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BackgroundGluonVertex", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"p", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"q", ",", "\[Nu]", ",", "b"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"k", ",", "\[Lambda]", ",", "c"}], "}"}], ",", 
   RowBox[{"Gauge", "\[Rule]", "\[Alpha]"}]}], "]"}]], "Input",
 CellTags->"BackgroundGluonVertex",
 CellLabel->"In[65]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["g", "s"], " ", 
   SubscriptBox["f", 
    RowBox[{
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm]}]], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        FormBox[
         RowBox[{
          RowBox[{"-", 
           FractionBox["k", "\[Alpha]"]}], "+", "p", "-", "q"}],
         TraditionalForm], ")"}], 
       FormBox[
        FormBox["\[Lambda]",
         TraditionalForm],
        TraditionalForm]]}], "+", 
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Lambda]", "\[Mu]"}]], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        FormBox[
         RowBox[{"k", "-", "p", "+", 
          FractionBox["q", "\[Alpha]"]}],
         TraditionalForm], ")"}], 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]]}], "+", 
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Lambda]", "\[Nu]"}]], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        FormBox[
         RowBox[{"q", "-", "k"}],
         TraditionalForm], ")"}], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]]}]}], ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009701374436607`*^9},
 CellTags->"BackgroundGluonVertex",
 CellLabel->"Out[65]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BackgroundGluonVertex", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"p", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"q", ",", "\[Nu]", ",", "b"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"k", ",", "\[Lambda]", ",", "c"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"s", ",", "\[Sigma]", ",", "d"}], "}"}], ",", 
   RowBox[{"Gauge", "\[Rule]", "\[Alpha]"}]}], "]"}]], "Input",
 CellTags->"BackgroundGluonVertex",
 CellLabel->"In[66]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", "\[ImaginaryI]"}], " ", 
   SubsuperscriptBox["g", "s", "2"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["a",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["d",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["u7",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["b",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["c",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["u7",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", 
         FractionBox[
          RowBox[{
           SuperscriptBox["g", 
            RowBox[{"\[Lambda]", "\[Nu]"}]], " ", 
           SuperscriptBox["g", 
            RowBox[{"\[Mu]", "\[Sigma]"}]]}], "\[Alpha]"]}], "+", 
        RowBox[{
         SuperscriptBox["g", 
          RowBox[{"\[Lambda]", "\[Sigma]"}]], " ", 
         SuperscriptBox["g", 
          RowBox[{"\[Mu]", "\[Nu]"}]]}], "-", 
        RowBox[{
         SuperscriptBox["g", 
          RowBox[{"\[Lambda]", "\[Mu]"}]], " ", 
         SuperscriptBox["g", 
          RowBox[{"\[Nu]", "\[Sigma]"}]]}]}], ")"}]}], "+", 
     RowBox[{
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["a",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["b",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["u7",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["c",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["d",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["u7",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      RowBox[{"(", 
       RowBox[{
        FractionBox[
         RowBox[{
          SuperscriptBox["g", 
           RowBox[{"\[Lambda]", "\[Sigma]"}]], " ", 
          SuperscriptBox["g", 
           RowBox[{"\[Mu]", "\[Nu]"}]]}], "\[Alpha]"], "-", 
        RowBox[{
         SuperscriptBox["g", 
          RowBox[{"\[Lambda]", "\[Nu]"}]], " ", 
         SuperscriptBox["g", 
          RowBox[{"\[Mu]", "\[Sigma]"}]]}], "+", 
        RowBox[{
         SuperscriptBox["g", 
          RowBox[{"\[Lambda]", "\[Mu]"}]], " ", 
         SuperscriptBox["g", 
          RowBox[{"\[Nu]", "\[Sigma]"}]]}]}], ")"}]}], "+", 
     RowBox[{
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["a",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["c",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["u7",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["b",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["d",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["u7",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         SuperscriptBox["g", 
          RowBox[{"\[Lambda]", "\[Sigma]"}]], " ", 
         SuperscriptBox["g", 
          RowBox[{"\[Mu]", "\[Nu]"}]]}], "-", 
        RowBox[{
         SuperscriptBox["g", 
          RowBox[{"\[Lambda]", "\[Nu]"}]], " ", 
         SuperscriptBox["g", 
          RowBox[{"\[Mu]", "\[Sigma]"}]]}]}], ")"}]}]}], ")"}]}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970137769499*^9},
 CellTags->"BackgroundGluonVertex",
 CellLabel->"Out[66]="]
}, Open  ]]
}, Open  ]],

Cell["Bracket", "ObjectName",
 CellTags->"Bracket"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Bracket"],

Cell["Bracket is an option of Convolute.", "Text",
 CellTags->"Bracket"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Convolute",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Convolute",
  ButtonNote->"Convolute"],
 "."
}], "Text",
 CellTags->"Bracket"]
}, Open  ]],

Cell["BReduce", "ObjectName",
 PageBreakAbove->True,
 CellTags->"BReduce"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"BReduce"],

Cell["\<\
BReduce is an option for B0, B00, B1, B11 determining whether reductions to \
A0 and B0 will be done. \
\>", "Text",
 CellTags->"BReduce"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["A0",
  BaseStyle->"AddOnsLink",
  ButtonData:>"A0",
  ButtonNote->"A0"],
 ", ",
 ButtonBox["B0",
  BaseStyle->"AddOnsLink",
  ButtonData:>"B0",
  ButtonNote->"B0"],
 ", ",
 ButtonBox["B00",
  BaseStyle->"AddOnsLink",
  ButtonData:>"B00",
  ButtonNote->"B00"],
 ", ",
 ButtonBox["B1",
  BaseStyle->"AddOnsLink",
  ButtonData:>"B1",
  ButtonNote->"B1"],
 ", ",
 ButtonBox["B11",
  BaseStyle->"AddOnsLink",
  ButtonData:>"B11",
  ButtonNote->"B11"],
 "."
}], "Text",
 CellTags->"BReduce"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"BReduce"],

Cell[TextData[{
 "By default ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["B", "0"], TraditionalForm]]],
 " is not expressed in terms of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["A", "0"], TraditionalForm]]],
 "."
}], "Text",
 CellTags->"BReduce"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"B0", "[", 
  RowBox[{"0", ",", "s", ",", "s"}], "]"}]], "Input",
 CellTags->"BReduce",
 CellLabel->"In[67]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["B", "0"], "(", "0", ",", "s", ",", "s", ")"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.5009701378848057`*^9},
 CellTags->"BReduce",
 CellLabel->"Out[67]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"B0", "//", "Options"}]], "Input",
 CellLabel->"In[68]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"BReduce", "\[Rule]", "False"}], ",", 
    RowBox[{"B0Unique", "\[Rule]", "False"}], ",", 
    RowBox[{"B0Real", "\[Rule]", "False"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970138001121*^9},
 CellLabel->"Out[68]="]
}, Open  ]],

Cell["\<\
With BReduce\[Rule]True, transformation is done.\
\>", "Text",
 CellTags->"BReduce"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"B0", "[", 
  RowBox[{"0", ",", "s", ",", "s", ",", 
   RowBox[{"BReduce", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"BReduce",
 CellLabel->"In[69]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{
     FormBox[
      SubscriptBox["\<\"A\"\>", "\<\"0\"\>"],
      TraditionalForm], "(", "s", ")"}], "s"], "-", "1"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970138109456*^9},
 CellTags->"BReduce",
 CellLabel->"Out[69]="]
}, Open  ]]
}, Open  ]],

Cell["B0", "ObjectName",
 CellTags->"B0"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"B0"],

Cell[TextData[{
 "B0[pp, ma^2, mb^2] is the Passarino-Veltman two-point integral ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["B", "0"], TraditionalForm]]],
 ". All arguments are scalars and have dimension mass^2. If the option \
BReduce is set to True certain B0's are reduced to A0's. Setting the option \
B0Unique to True simplifies B0[a,0,a] and B0[0,0,a]."
}], "Text",
 CellTags->"B0"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "B0", "]"}]], "Input",
 CellTags->"B0",
 CellLabel->"In[70]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"BReduce", "\[Rule]", "False"}], ",", 
    RowBox[{"B0Unique", "\[Rule]", "False"}], ",", 
    RowBox[{"B0Real", "\[Rule]", "False"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.5009701381903753`*^9},
 CellTags->"B0",
 CellLabel->"Out[70]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["B1",
  BaseStyle->"AddOnsLink",
  ButtonData:>"B1",
  ButtonNote->"B1"],
 ", ",
 ButtonBox["B00",
  BaseStyle->"AddOnsLink",
  ButtonData:>"B00",
  ButtonNote->"B00"],
 ", ",
 ButtonBox["B11",
  BaseStyle->"AddOnsLink",
  ButtonData:>"B11",
  ButtonNote->"B11"],
 ", ",
 ButtonBox["PaVe",
  BaseStyle->"AddOnsLink",
  ButtonData:>"PaVe",
  ButtonNote->"PaVe"],
 "."
}], "Text",
 CellTags->"B0"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"B0"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"B0", "[", 
  RowBox[{
   RowBox[{"SP", "[", 
    RowBox[{"p", ",", "p"}], "]"}], ",", 
   SuperscriptBox["m", "2"], ",", 
   SuperscriptBox["m", "2"]}], "]"}]], "Input",
 CellTags->"B0",
 CellLabel->"In[71]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["B", "0"], "(", 
   FormBox[
    SuperscriptBox[
     FormBox["p",
      TraditionalForm], "2"],
    TraditionalForm], ",", 
   SuperscriptBox["m", "2"], ",", 
   SuperscriptBox["m", "2"], ")"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970138264448*^9},
 CellTags->"B0",
 CellLabel->"Out[71]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"SetOptions", "[", 
   RowBox[{"B0", ",", 
    RowBox[{"B0Unique", "\[Rule]", "True"}], ",", 
    RowBox[{"B0Real", "\[Rule]", "True"}]}], "]"}], ";"}]], "Input",
 CellTags->"B0",
 CellLabel->"In[72]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"B0", "[", 
  RowBox[{"0", ",", "0", ",", 
   SuperscriptBox["m", "2"]}], "]"}]], "Input",
 CellTags->"B0",
 CellLabel->"In[73]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox["B", "0"], "(", "0", ",", 
    SuperscriptBox["m", "2"], ",", 
    SuperscriptBox["m", "2"], ")"}], "+", "1"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970138409875*^9},
 CellTags->"B0",
 CellLabel->"Out[73]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"B0", "[", 
  RowBox[{
   SuperscriptBox["m", "2"], ",", "0", ",", 
   SuperscriptBox["m", "2"]}], "]"}]], "Input",
 CellTags->"B0",
 CellLabel->"In[74]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox["B", "0"], "(", "0", ",", 
    SuperscriptBox["m", "2"], ",", 
    SuperscriptBox["m", "2"], ")"}], "+", "2"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009701384873133`*^9},
 CellTags->"B0",
 CellLabel->"Out[74]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"B0", "[", 
  RowBox[{"0", ",", 
   RowBox[{"m", "^", "2"}], ",", 
   RowBox[{"m", "^", "2"}]}], "]"}]], "Input",
 CellTags->"B0",
 CellLabel->"In[75]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["B", "0"], "(", "0", ",", 
   SuperscriptBox["m", "2"], ",", 
   SuperscriptBox["m", "2"], ")"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009701385324097`*^9},
 CellTags->"B0",
 CellLabel->"Out[75]="]
}, Open  ]]
}, Open  ]],

Cell["B0Real", "ObjectName",
 PageBreakAbove->True,
 CellTags->"B0Real"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"B0Real"],

Cell["\<\
B0Real is an option of B0 (default False). If set to True, B0 is assumed to \
be real and the relation B0[a,0,a] = 2 + B0[0,a,a]  is applied.\
\>", "Text",
 CellTags->"B0Real"],

Cell[TextData[{
 "See also: ",
 ButtonBox["B0",
  BaseStyle->"AddOnsLink",
  ButtonData:>"B0",
  ButtonNote->"B0"],
 "."
}], "Text",
 CellTags->"B0Real"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"B0Real"],

Cell["By default the arguments are not assumed real.", "Text",
 CellTags->"B0Real"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"B0", "[", 
  RowBox[{"s", ",", "0", ",", "s"}], "]"}]], "Input",
 CellTags->"B0Real",
 CellLabel->"In[76]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox["B", "0"], "(", "0", ",", "s", ",", "s", ")"}], "+", "2"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970138622401*^9},
 CellTags->"B0Real",
 CellLabel->"Out[76]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"B0", "//", "Options"}]], "Input",
 CellTags->"B0Real",
 CellLabel->"In[77]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"BReduce", "\[Rule]", "False"}], ",", 
    RowBox[{"B0Unique", "\[Rule]", "True"}], ",", 
    RowBox[{"B0Real", "\[Rule]", "True"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970138702117*^9},
 CellTags->"B0Real",
 CellLabel->"Out[77]="]
}, Open  ]],

Cell["\<\
With B0Real\[Rule]True, transformation is done.\
\>", "Text",
 CellTags->"B0Real"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"B0", "[", 
  RowBox[{"s", ",", "0", ",", "s", ",", 
   RowBox[{"B0Real", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"B0Real",
 CellLabel->"In[78]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox["B", "0"], "(", "0", ",", "s", ",", "s", ")"}], "+", "2"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970138747264*^9},
 CellTags->"B0Real",
 CellLabel->"Out[78]="]
}, Open  ]]
}, Open  ]],

Cell["B0Unique", "ObjectName",
 PageBreakAbove->True,
 CellTags->"B0Unique"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"B0Unique"],

Cell["\<\
B0Unique is an option of B0. If set to True, B0[0,0,m2] is replaced with \
(B0[0,m2,m2]+1) and B0[m2,0,m2] simplifies to (B0[0,m2,m2]+2).\
\>", "Text",
 CellTags->"B0Unique"],

Cell[TextData[{
 "See also: ",
 ButtonBox["B0",
  BaseStyle->"AddOnsLink",
  ButtonData:>"B0",
  ButtonNote->"B0"],
 "."
}], "Text",
 CellTags->"B0Unique"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"B0Unique"],

Cell["By default transformation is done.", "Text",
 CellTags->"B0Unique"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"B0", "[", 
  RowBox[{"0", ",", "0", ",", "s"}], "]"}]], "Input",
 CellTags->"B0Unique",
 CellLabel->"In[79]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox["B", "0"], "(", "0", ",", "s", ",", "s", ")"}], "+", "1"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970138790578*^9},
 CellTags->"B0Unique",
 CellLabel->"Out[79]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"B0", "//", "Options"}]], "Input",
 CellTags->"B0Unique",
 CellLabel->"In[80]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"BReduce", "\[Rule]", "False"}], ",", 
    RowBox[{"B0Unique", "\[Rule]", "True"}], ",", 
    RowBox[{"B0Real", "\[Rule]", "True"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.5009701388677883`*^9},
 CellTags->"B0Unique",
 CellLabel->"Out[80]="]
}, Open  ]],

Cell["\<\
With B0Real\[Rule]False, nothing happens.\
\>", "Text",
 CellTags->"B0Unique"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"B0", "[", 
  RowBox[{"0", ",", "0", ",", "s", ",", 
   RowBox[{"B0Unique", "\[Rule]", "False"}]}], "]"}]], "Input",
 CellTags->"B0Unique",
 CellLabel->"In[81]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["B", "0"], "(", "0", ",", "0", ",", "s", ")"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.50097013891451*^9},
 CellTags->"B0Unique",
 CellLabel->"Out[81]="]
}, Open  ]]
}, Open  ]],

Cell["B00", "ObjectName",
 CellTags->"B00"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"B00"],

Cell[TextData[{
 "B00[pp, ma^2,mb^2] is the Passarino-Veltman ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["B", "00"], TraditionalForm]]],
 "-function, i.e., the coefficient function of the metric tensor. All \
arguments are scalars and have dimension mass^2."
}], "Text",
 CellTags->"B00"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "B00", "]"}]], "Input",
 CellTags->"B00",
 CellLabel->"In[82]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"BReduce", "\[Rule]", "True"}], "}"}], TraditionalForm]], "Output",\

 CellChangeTimes->{3.500970138994022*^9},
 CellTags->"B00",
 CellLabel->"Out[82]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["B0",
  BaseStyle->"AddOnsLink",
  ButtonData:>"B0",
  ButtonNote->"B0"],
 ", ",
 ButtonBox["B1",
  BaseStyle->"AddOnsLink",
  ButtonData:>"B1",
  ButtonNote->"B1"],
 ", ",
 ButtonBox["PaVe",
  BaseStyle->"AddOnsLink",
  ButtonData:>"PaVe",
  ButtonNote->"PaVe"],
 "."
}], "Text",
 CellTags->"B00"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"B00"],

Cell[TextData[{
 "Remember that SP[p] is a short hand input for ScalarProduct[p,p], i.e., ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["p", "2"], TraditionalForm]]],
 "."
}], "Text",
 CellTags->"B00"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SP", "[", "p", "]"}]], "Input",
 CellTags->"B00",
 CellLabel->"In[83]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox[
   FormBox["p",
    TraditionalForm], "2"], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970139070822*^9},
 CellTags->"B00",
 CellLabel->"Out[83]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"B00", "[", 
  RowBox[{
   RowBox[{"SP", "[", "p", "]"}], ",", 
   SuperscriptBox["m", "2"], ",", 
   SuperscriptBox["M", "2"]}], "]"}]], "Input",
 CellTags->"B00",
 CellLabel->"In[84]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    FractionBox["1", "6"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox["m", "2"], "-", 
         SuperscriptBox["M", "2"], "+", 
         FormBox[
          SuperscriptBox[
           FormBox["p",
            TraditionalForm], "2"],
          TraditionalForm]}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{
         FractionBox[
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             SuperscriptBox["M", "2"], "-", 
             SuperscriptBox["m", "2"]}], ")"}], " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{
              SubscriptBox["B", "0"], "(", 
              FormBox[
               SuperscriptBox[
                FormBox["p",
                 TraditionalForm], "2"],
               TraditionalForm], ",", 
              SuperscriptBox["m", "2"], ",", 
              SuperscriptBox["M", "2"], ")"}], "-", 
             RowBox[{
              SubscriptBox["B", "0"], "(", "0", ",", 
              SuperscriptBox["m", "2"], ",", 
              SuperscriptBox["M", "2"], ")"}]}], ")"}]}], 
          RowBox[{"2", " ", 
           FormBox[
            SuperscriptBox[
             FormBox["p",
              TraditionalForm], "2"],
            TraditionalForm]}]], "-", 
         RowBox[{
          FractionBox["1", "2"], " ", 
          RowBox[{
           SubscriptBox["B", "0"], "(", 
           FormBox[
            SuperscriptBox[
             FormBox["p",
              TraditionalForm], "2"],
            TraditionalForm], ",", 
           SuperscriptBox["m", "2"], ",", 
           SuperscriptBox["M", "2"], ")"}]}]}], ")"}]}], "+", 
      RowBox[{
       FormBox[
        SubscriptBox["\<\"A\"\>", "\<\"0\"\>"],
        TraditionalForm], "(", 
       FormBox[
        SuperscriptBox["M", "2"],
        TraditionalForm], ")"}]}], ")"}]}], "+", 
   RowBox[{
    FractionBox["1", "3"], " ", 
    SuperscriptBox["m", "2"], " ", 
    RowBox[{
     SubscriptBox["B", "0"], "(", 
     FormBox[
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], "2"],
      TraditionalForm], ",", 
     SuperscriptBox["m", "2"], ",", 
     SuperscriptBox["M", "2"], ")"}]}], "+", 
   RowBox[{
    FractionBox["1", "18"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"3", " ", 
       SuperscriptBox["m", "2"]}], "+", 
      RowBox[{"3", " ", 
       SuperscriptBox["M", "2"]}], "-", 
      FormBox[
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], "2"],
       TraditionalForm]}], ")"}]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970139194018*^9},
 CellTags->"B00",
 CellLabel->"Out[84]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"B00", "[", 
  RowBox[{
   RowBox[{"SP", "[", "p", "]"}], ",", 
   SuperscriptBox["m", "2"], ",", 
   SuperscriptBox["m", "2"]}], "]"}]], "Input",
 CellTags->"B00",
 CellLabel->"In[85]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    FractionBox["1", "6"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       FormBox[
        SubscriptBox["\<\"A\"\>", "\<\"0\"\>"],
        TraditionalForm], "(", 
       FormBox[
        SuperscriptBox["m", "2"],
        TraditionalForm], ")"}], "-", 
      RowBox[{
       FractionBox["1", "2"], " ", 
       FormBox[
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], "2"],
        TraditionalForm], " ", 
       RowBox[{
        SubscriptBox["B", "0"], "(", 
        FormBox[
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], "2"],
         TraditionalForm], ",", 
        SuperscriptBox["m", "2"], ",", 
        SuperscriptBox["m", "2"], ")"}]}]}], ")"}]}], "+", 
   RowBox[{
    FractionBox["1", "3"], " ", 
    SuperscriptBox["m", "2"], " ", 
    RowBox[{
     SubscriptBox["B", "0"], "(", 
     FormBox[
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], "2"],
      TraditionalForm], ",", 
     SuperscriptBox["m", "2"], ",", 
     SuperscriptBox["m", "2"], ")"}]}], "+", 
   RowBox[{
    FractionBox["1", "18"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"6", " ", 
       SuperscriptBox["m", "2"]}], "-", 
      FormBox[
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], "2"],
       TraditionalForm]}], ")"}]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009701398851423`*^9},
 CellTags->"B00",
 CellLabel->"Out[85]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"B00", "[", 
  RowBox[{
   RowBox[{"SP", "[", "p", "]"}], ",", 
   SuperscriptBox["m", "2"], ",", 
   SuperscriptBox["M", "2"], ",", 
   RowBox[{"BReduce", "\[Rule]", "False"}]}], "]"}]], "Input",
 CellTags->"B00",
 CellLabel->"In[86]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox[
    SubscriptBox["\<\"B\"\>", "\<\"00\"\>"],
    TraditionalForm], "(", 
   FormBox[
    FormBox[
     SuperscriptBox[
      FormBox["p",
       TraditionalForm], "2"],
     TraditionalForm],
    TraditionalForm], 
   RowBox[{",", " "}], 
   FormBox[
    SuperscriptBox["m", "2"],
    TraditionalForm], 
   RowBox[{",", " "}], 
   FormBox[
    SuperscriptBox["M", "2"],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009701401440783`*^9},
 CellTags->"B00",
 CellLabel->"Out[86]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"B00", "[", 
  RowBox[{"0", ",", 
   SuperscriptBox["m", "2"], ",", 
   SuperscriptBox["m", "2"]}], "]"}]], "Input",
 CellTags->"B00",
 CellLabel->"In[87]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    FractionBox["1", "3"], " ", 
    SuperscriptBox["m", "2"], " ", 
    RowBox[{
     SubscriptBox["B", "0"], "(", "0", ",", 
     SuperscriptBox["m", "2"], ",", 
     SuperscriptBox["m", "2"], ")"}]}], "+", 
   RowBox[{
    FractionBox["1", "6"], " ", 
    RowBox[{
     FormBox[
      SubscriptBox["\<\"A\"\>", "\<\"0\"\>"],
      TraditionalForm], "(", 
     FormBox[
      SuperscriptBox["m", "2"],
      TraditionalForm], ")"}]}], "+", 
   FractionBox[
    SuperscriptBox["m", "2"], "3"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970140318282*^9},
 CellTags->"B00",
 CellLabel->"Out[87]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"B00", "[", 
  RowBox[{
   RowBox[{"SmallVariable", "[", 
    SuperscriptBox["M", "2"], "]"}], ",", 
   SuperscriptBox["m", "2"], ",", 
   SuperscriptBox["m", "2"]}], "]"}]], "Input",
 CellTags->"B00",
 CellLabel->"In[88]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    FractionBox["1", "3"], " ", 
    SuperscriptBox["m", "2"], " ", 
    RowBox[{
     SubscriptBox["B", "0"], "(", 
     SuperscriptBox["M", "2"], ",", 
     SuperscriptBox["m", "2"], ",", 
     SuperscriptBox["m", "2"], ")"}]}], "+", 
   RowBox[{
    FractionBox["1", "6"], " ", 
    RowBox[{
     FormBox[
      SubscriptBox["\<\"A\"\>", "\<\"0\"\>"],
      TraditionalForm], "(", 
     FormBox[
      SuperscriptBox["m", "2"],
      TraditionalForm], ")"}]}], "+", 
   FractionBox[
    SuperscriptBox["m", "2"], "3"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009701405232077`*^9},
 CellTags->"B00",
 CellLabel->"Out[88]="]
}, Open  ]]
}, Open  ]],

Cell["B1", "ObjectName",
 CellTags->"B1"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"B1"],

Cell[TextData[{
 "B1[pp, ma^2, mb^2] the Passarino-Veltman ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["B", "1"], TraditionalForm]]],
 "-function. All arguments are scalars and have dimension mass^2."
}], "Text",
 CellTags->"B1"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "B1", "]"}]], "Input",
 CellTags->"B1",
 CellLabel->"In[89]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"BReduce", "\[Rule]", "True"}], "}"}], TraditionalForm]], "Output",\

 CellChangeTimes->{3.500970140628902*^9},
 CellTags->"B1",
 CellLabel->"Out[89]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["B0",
  BaseStyle->"AddOnsLink",
  ButtonData:>"B0",
  ButtonNote->"B0"],
 ", ",
 ButtonBox["B00",
  BaseStyle->"AddOnsLink",
  ButtonData:>"B00",
  ButtonNote->"B00"],
 ", ",
 ButtonBox["B11",
  BaseStyle->"AddOnsLink",
  ButtonData:>"B11",
  ButtonNote->"B11"],
 ", ",
 ButtonBox["PaVe",
  BaseStyle->"AddOnsLink",
  ButtonData:>"PaVe",
  ButtonNote->"PaVe"],
 ", ",
 ButtonBox["PaVeReduce",
  BaseStyle->"AddOnsLink",
  ButtonData:>"PaVeReduce",
  ButtonNote->"PaVeReduce"],
 "."
}], "Text",
 CellTags->"B1"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"B1"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"B1", "[", 
  RowBox[{
   RowBox[{"SP", "[", "p", "]"}], ",", 
   SuperscriptBox["m", "2"], ",", 
   SuperscriptBox["M", "2"]}], "]"}]], "Input",
 CellTags->"B1",
 CellLabel->"In[90]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox["M", "2"], "-", 
       SuperscriptBox["m", "2"]}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        SubscriptBox["B", "0"], "(", 
        FormBox[
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], "2"],
         TraditionalForm], ",", 
        SuperscriptBox["m", "2"], ",", 
        SuperscriptBox["M", "2"], ")"}], "-", 
       RowBox[{
        SubscriptBox["B", "0"], "(", "0", ",", 
        SuperscriptBox["m", "2"], ",", 
        SuperscriptBox["M", "2"], ")"}]}], ")"}]}], 
    RowBox[{"2", " ", 
     FormBox[
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], "2"],
      TraditionalForm]}]], "-", 
   RowBox[{
    FractionBox["1", "2"], " ", 
    RowBox[{
     SubscriptBox["B", "0"], "(", 
     FormBox[
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], "2"],
      TraditionalForm], ",", 
     SuperscriptBox["m", "2"], ",", 
     SuperscriptBox["M", "2"], ")"}]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970140896916*^9},
 CellTags->"B1",
 CellLabel->"Out[90]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"B1", "[", 
  RowBox[{
   RowBox[{"SP", "[", "p", "]"}], ",", 
   SuperscriptBox["m", "2"], ",", 
   SuperscriptBox["M", "2"], ",", 
   RowBox[{"BReduce", "\[Rule]", "False"}]}], "]"}]], "Input",
 CellTags->"B1",
 CellLabel->"In[91]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["B", "1"], "(", 
   FormBox[
    SuperscriptBox[
     FormBox["p",
      TraditionalForm], "2"],
    TraditionalForm], ",", 
   SuperscriptBox["m", "2"], ",", 
   SuperscriptBox["M", "2"], ")"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970141052758*^9},
 CellTags->"B1",
 CellLabel->"Out[91]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"SetOptions", "[", 
   RowBox[{"B1", ",", 
    RowBox[{"BReduce", "\[Rule]", "True"}]}], "]"}], ";"}]], "Input",
 CellTags->"B1",
 CellLabel->"In[92]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"B1", "[", 
  RowBox[{
   RowBox[{"SP", "[", "p", "]"}], ",", 
   SuperscriptBox["m", "2"], ",", 
   SuperscriptBox["m", "2"]}], "]"}]], "Input",
 CellTags->"B1",
 CellLabel->"In[93]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    FractionBox["1", "2"]}], " ", 
   RowBox[{
    SubscriptBox["B", "0"], "(", 
    FormBox[
     SuperscriptBox[
      FormBox["p",
       TraditionalForm], "2"],
     TraditionalForm], ",", 
    SuperscriptBox["m", "2"], ",", 
    SuperscriptBox["m", "2"], ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970141244018*^9},
 CellTags->"B1",
 CellLabel->"Out[93]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"B1", "[", 
  RowBox[{
   SuperscriptBox["m", "2"], ",", 
   SuperscriptBox["m", "2"], ",", "0"}], "]"}]], "Input",
 CellTags->"B1",
 CellLabel->"In[94]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    FractionBox["1", "2"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", 
       RowBox[{
        SubscriptBox["B", "0"], "(", "0", ",", 
        SuperscriptBox["m", "2"], ",", 
        SuperscriptBox["m", "2"], ")"}]}], "-", "2"}], ")"}]}], "-", 
   FractionBox["1", "2"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970141390849*^9},
 CellTags->"B1",
 CellLabel->"Out[94]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"B1", "[", 
  RowBox[{"0", ",", "0", ",", 
   SuperscriptBox["m", "2"]}], "]"}]], "Input",
 CellTags->"B1",
 CellLabel->"In[95]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    FractionBox["1", "2"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", 
       RowBox[{
        SubscriptBox["B", "0"], "(", "0", ",", 
        SuperscriptBox["m", "2"], ",", 
        SuperscriptBox["m", "2"], ")"}]}], "-", "1"}], ")"}]}], "+", 
   FractionBox["1", "4"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009701415792923`*^9},
 CellTags->"B1",
 CellLabel->"Out[95]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"B1", "[", 
  RowBox[{"pp", ",", 
   RowBox[{"SmallVariable", "[", 
    SubsuperscriptBox["m", "e", "2"], "]"}], ",", 
   SubsuperscriptBox["m", "2", "2"]}], "]"}]], "Input",
 CellTags->"B1",
 CellLabel->"In[96]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{
     SubsuperscriptBox["m", "2", "2"], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        SubscriptBox["B", "0"], "(", "pp", ",", 
        SubsuperscriptBox["m", "e", "2"], ",", 
        SubsuperscriptBox["m", "2", "2"], ")"}], "-", 
       RowBox[{
        SubscriptBox["B", "0"], "(", "0", ",", 
        SubsuperscriptBox["m", "e", "2"], ",", 
        SubsuperscriptBox["m", "2", "2"], ")"}]}], ")"}]}], 
    RowBox[{"2", " ", "pp"}]], "-", 
   RowBox[{
    FractionBox["1", "2"], " ", 
    RowBox[{
     SubscriptBox["B", "0"], "(", "pp", ",", 
     SubsuperscriptBox["m", "e", "2"], ",", 
     SubsuperscriptBox["m", "2", "2"], ")"}]}]}], TraditionalForm]], "Output",\

 CellChangeTimes->{3.500970141747802*^9},
 CellTags->"B1",
 CellLabel->"Out[96]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"B1", "[", 
  RowBox[{
   RowBox[{"SmallVariable", "[", 
    SubsuperscriptBox["m", "e", "2"], "]"}], ",", 
   RowBox[{"SmallVariable", "[", 
    SubsuperscriptBox["m", "e", "2"], "]"}], ",", "0"}], "]"}]], "Input",
 CellTags->"B1",
 CellLabel->"In[97]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    FractionBox["1", "2"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", 
       RowBox[{
        SubscriptBox["B", "0"], "(", "0", ",", 
        SubsuperscriptBox["m", "e", "2"], ",", 
        SubsuperscriptBox["m", "e", "2"], ")"}]}], "-", "2"}], ")"}]}], "-", 
   FractionBox["1", "2"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970141904085*^9},
 CellTags->"B1",
 CellLabel->"Out[97]="]
}, Open  ]]
}, Open  ]],

Cell["B11", "ObjectName",
 CellTags->"B11"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"B11"],

Cell[TextData[{
 "B11[pp, ma^2, mb^2] is the Passarino-Veltman ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["B", "11"], TraditionalForm]]],
 "-function, i.e., the coefficient function of ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["p", "\[Mu]"], " ", 
    SuperscriptBox["p", "\[Nu]"]}], TraditionalForm]]],
 ". All arguments are scalars and have dimension mass",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["\[Null]", "2"], TraditionalForm]]],
 "."
}], "Text",
 CellTags->"B11"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "B11", "]"}]], "Input",
 CellTags->"B11",
 CellLabel->"In[98]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"BReduce", "\[Rule]", "True"}], "}"}], TraditionalForm]], "Output",\

 CellChangeTimes->{3.5009701420650682`*^9},
 CellTags->"B11",
 CellLabel->"Out[98]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["B0",
  BaseStyle->"AddOnsLink",
  ButtonData:>"B0",
  ButtonNote->"B0"],
 ", ",
 ButtonBox["B00",
  BaseStyle->"AddOnsLink",
  ButtonData:>"B00",
  ButtonNote->"B00"],
 ", ",
 ButtonBox["B1",
  BaseStyle->"AddOnsLink",
  ButtonData:>"B1",
  ButtonNote->"B1"],
 ", ",
 ButtonBox["PaVe",
  BaseStyle->"AddOnsLink",
  ButtonData:>"PaVe",
  ButtonNote->"PaVe"],
 "."
}], "Text",
 CellTags->"B11"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"B11"],

Cell[TextData[{
 "Remember that SP[p] is a short hand input for ScalarProduct[p,p], i.e. ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["p", "2"], "."}], TraditionalForm]]]
}], "Text",
 CellTags->"B11"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SP", "[", "p", "]"}]], "Input",
 CellTags->"B11",
 CellLabel->"In[99]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox[
   FormBox["p",
    TraditionalForm], "2"], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970142127284*^9},
 CellTags->"B11",
 CellLabel->"Out[99]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"B11", "[", 
  RowBox[{
   RowBox[{"SP", "[", "p", "]"}], ",", 
   SuperscriptBox["m", "2"], ",", 
   SuperscriptBox["M", "2"]}], "]"}]], "Input",
 CellTags->"B11",
 CellLabel->"In[100]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["1", 
    RowBox[{"3", " ", 
     FormBox[
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], "2"],
      TraditionalForm]}]], 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      SuperscriptBox["m", "2"], " ", 
      RowBox[{"(", 
       RowBox[{"-", 
        RowBox[{
         SubscriptBox["B", "0"], "(", 
         FormBox[
          SuperscriptBox[
           FormBox["p",
            TraditionalForm], "2"],
          TraditionalForm], ",", 
         SuperscriptBox["m", "2"], ",", 
         SuperscriptBox["M", "2"], ")"}]}], ")"}]}], "-", 
     RowBox[{"2", " ", 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox["m", "2"], "-", 
        SuperscriptBox["M", "2"], "+", 
        FormBox[
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], "2"],
         TraditionalForm]}], ")"}], " ", 
      RowBox[{"(", 
       RowBox[{
        FractionBox[
         RowBox[{
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox["M", "2"], "-", 
            SuperscriptBox["m", "2"]}], ")"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             SubscriptBox["B", "0"], "(", 
             FormBox[
              SuperscriptBox[
               FormBox["p",
                TraditionalForm], "2"],
              TraditionalForm], ",", 
             SuperscriptBox["m", "2"], ",", 
             SuperscriptBox["M", "2"], ")"}], "-", 
            RowBox[{
             SubscriptBox["B", "0"], "(", "0", ",", 
             SuperscriptBox["m", "2"], ",", 
             SuperscriptBox["M", "2"], ")"}]}], ")"}]}], 
         RowBox[{"2", " ", 
          FormBox[
           SuperscriptBox[
            FormBox["p",
             TraditionalForm], "2"],
           TraditionalForm]}]], "-", 
        RowBox[{
         FractionBox["1", "2"], " ", 
         RowBox[{
          SubscriptBox["B", "0"], "(", 
          FormBox[
           SuperscriptBox[
            FormBox["p",
             TraditionalForm], "2"],
           TraditionalForm], ",", 
          SuperscriptBox["m", "2"], ",", 
          SuperscriptBox["M", "2"], ")"}]}]}], ")"}]}], "+", 
     RowBox[{
      FormBox[
       SubscriptBox["\<\"A\"\>", "\<\"0\"\>"],
       TraditionalForm], "(", 
      FormBox[
       SuperscriptBox["M", "2"],
       TraditionalForm], ")"}], "+", 
     RowBox[{
      FractionBox["1", "6"], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", 
         RowBox[{"3", " ", 
          SuperscriptBox["m", "2"]}]}], "-", 
        RowBox[{"3", " ", 
         SuperscriptBox["M", "2"]}], "+", 
        FormBox[
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], "2"],
         TraditionalForm]}], ")"}]}]}], ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.50097014227173*^9},
 CellTags->"B11",
 CellLabel->"Out[100]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"B11", "[", 
  RowBox[{
   RowBox[{"SP", "[", "p", "]"}], ",", 
   SuperscriptBox["m", "2"], ",", 
   SuperscriptBox["m", "2"]}], "]"}]], "Input",
 CellTags->"B11",
 CellLabel->"In[101]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       FormBox[
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], "2"],
        TraditionalForm], "-", 
       SuperscriptBox["m", "2"]}], ")"}], " ", 
     RowBox[{
      SubscriptBox["B", "0"], "(", 
      FormBox[
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], "2"],
       TraditionalForm], ",", 
      SuperscriptBox["m", "2"], ",", 
      SuperscriptBox["m", "2"], ")"}]}], "+", 
    RowBox[{
     FormBox[
      SubscriptBox["\<\"A\"\>", "\<\"0\"\>"],
      TraditionalForm], "(", 
     FormBox[
      SuperscriptBox["m", "2"],
      TraditionalForm], ")"}], "+", 
    RowBox[{
     FractionBox["1", "6"], " ", 
     RowBox[{"(", 
      RowBox[{
       FormBox[
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], "2"],
        TraditionalForm], "-", 
       RowBox[{"6", " ", 
        SuperscriptBox["m", "2"]}]}], ")"}]}]}], 
   RowBox[{"3", " ", 
    FormBox[
     SuperscriptBox[
      FormBox["p",
       TraditionalForm], "2"],
     TraditionalForm]}]], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009701424100437`*^9},
 CellTags->"B11",
 CellLabel->"Out[101]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SetOptions", "[", 
  RowBox[{"B11", ",", 
   RowBox[{"BReduce", "\[Rule]", "False"}]}], "]"}]], "Input",
 CellTags->"B11",
 CellLabel->"In[102]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"BReduce", "\[Rule]", "False"}], "}"}], TraditionalForm]], "Output",\

 CellChangeTimes->{3.500970142462227*^9},
 CellTags->"B11",
 CellLabel->"Out[102]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"B11", "[", 
  RowBox[{
   RowBox[{"SP", "[", "p", "]"}], ",", 
   SuperscriptBox["m", "2"], ",", 
   SuperscriptBox["M", "2"]}], "]"}]], "Input",
 CellTags->"B11",
 CellLabel->"In[103]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox[
    SubscriptBox["\<\"B\"\>", "\<\"11\"\>"],
    TraditionalForm], "(", 
   FormBox[
    FormBox[
     SuperscriptBox[
      FormBox["p",
       TraditionalForm], "2"],
     TraditionalForm],
    TraditionalForm], 
   RowBox[{",", " "}], 
   FormBox[
    SuperscriptBox["m", "2"],
    TraditionalForm], 
   RowBox[{",", " "}], 
   FormBox[
    SuperscriptBox["M", "2"],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009701430084963`*^9},
 CellTags->"B11",
 CellLabel->"Out[103]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SetOptions", "[", 
  RowBox[{"B11", ",", 
   RowBox[{"BReduce", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"B11",
 CellLabel->"In[104]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"BReduce", "\[Rule]", "True"}], "}"}], TraditionalForm]], "Output",\

 CellChangeTimes->{3.500970143091317*^9},
 CellTags->"B11",
 CellLabel->"Out[104]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"B11", "[", 
  RowBox[{"0", ",", 
   SuperscriptBox["m", "2"], ",", 
   SuperscriptBox["m", "2"]}], "]"}]], "Input",
 CellTags->"B11",
 CellLabel->"In[105]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["1", "3"], " ", 
   RowBox[{
    SubscriptBox["B", "0"], "(", "0", ",", 
    SuperscriptBox["m", "2"], ",", 
    SuperscriptBox["m", "2"], ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970143179311*^9},
 CellTags->"B11",
 CellLabel->"Out[105]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"B11", "[", 
  RowBox[{
   RowBox[{"SmallVariable", "[", 
    SuperscriptBox["M", "2"], "]"}], ",", 
   SuperscriptBox["m", "2"], ",", 
   SuperscriptBox["m", "2"]}], "]"}]], "Input",
 CellTags->"B11",
 CellLabel->"In[106]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["1", "3"], " ", 
   RowBox[{
    SubscriptBox["B", "0"], "(", 
    SuperscriptBox["M", "2"], ",", 
    SuperscriptBox["m", "2"], ",", 
    SuperscriptBox["m", "2"], ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970143355213*^9},
 CellTags->"B11",
 CellLabel->"Out[106]="]
}, Open  ]]
}, Open  ]],

Cell["CA", "ObjectName",
 CellTags->"CA"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"CA"],

Cell[TextData[{
 "CA is one of the Casimir operator eigenvalues of SU(",
 StyleBox["N",
  FontSlant->"Italic"],
 ") (CA = ",
 StyleBox["N",
  FontSlant->"Italic"],
 ")."
}], "Text",
 CellTags->"CA"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["CF",
  BaseStyle->"AddOnsLink",
  ButtonData:>"CF",
  ButtonNote->"CF"],
 ", ",
 ButtonBox["SUNSimplify",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SUNSimplify",
  ButtonNote->"SUNSimplify"],
 "."
}], "Text",
 CellTags->"CA"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"CA"],

Cell[CellGroupData[{

Cell[BoxData["CA"], "Input",
 CellTags->"CA",
 CellLabel->"In[107]:="],

Cell[BoxData[
 FormBox[
  SubscriptBox["C", "A"], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970143440528*^9},
 CellTags->"CA",
 CellLabel->"Out[107]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNSimplify", "[", 
  RowBox[{"CA", ",", 
   RowBox[{"SUNNToCACF", "\[Rule]", "False"}]}], "]"}]], "Input",
 CellTags->"CA",
 CellLabel->"In[108]:="],

Cell[BoxData[
 FormBox["N", TraditionalForm]], "Output",
 CellChangeTimes->{3.500970143528762*^9},
 CellTags->"CA",
 CellLabel->"Out[108]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData["SUNN"], "Input",
 CellTags->"CA",
 CellLabel->"In[109]:="],

Cell[BoxData[
 FormBox["N", TraditionalForm]], "Output",
 CellChangeTimes->{3.5009701436266747`*^9},
 CellTags->"CA",
 CellLabel->"Out[109]="]
}, Open  ]]
}, Open  ]],

Cell["Calc", "ObjectName",
 CellTags->"Calc"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Calc"],

Cell["\<\
Calc[exp] performs several basic simplifications. Calc[exp] is the same as \
DotSimplify[DiracSimplify[Contract[DiracSimplify[Explicit[ \
SUNSimplify[Trick[exp], Explicit\[Rule]False] ]]]]].\
\>", "Text",
 CellTags->"Calc"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["DiracSimplify",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracSimplify",
  ButtonNote->"DiracSimplify"],
 ", ",
 ButtonBox["DiracTrick",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracTrick",
  ButtonNote->"DiracTrick"],
 ", ",
 ButtonBox["Trick",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Trick",
  ButtonNote->"Trick"],
 "."
}], "Text",
 CellTags->"Calc"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData["Examples"], "Subsubsection",
 CellTags->"Calc"],

Cell[TextData[{
 "This calculates ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["\[Gamma]", "\[Mu]"], 
    SubscriptBox["\[Gamma]", "\[Mu]"]}], TraditionalForm]]],
 " in 4 dimensions and ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["g", "\[Nu]", "\[Nu]"], TraditionalForm]]],
 " in D dimensions."
}], "Text",
 CellTags->"Calc"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Calc", "[", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"GA", "[", 
     RowBox[{"\[Mu]", ",", "\[Mu]"}], "]"}], ",", " ", 
    RowBox[{"MTD", "[", 
     RowBox[{"\[Nu]", ",", "\[Nu]"}], "]"}]}], "}"}], "]"}]], "Input",
 CellTags->"Calc",
 CellLabel->"In[110]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"4", ",", "D"}], "}"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970143801836*^9},
 CellTags->"Calc",
 CellLabel->"Out[110]="]
}, Open  ]],

Cell[TextData[{
 "This simplifies ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["f", "abc"], " ", 
    RowBox[{
     SubscriptBox["f", "abe"], "."}]}], TraditionalForm]]]
}], "Text",
 CellTags->"Calc"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Calc", "[", 
  RowBox[{
   RowBox[{"SUNF", "[", 
    RowBox[{"a", ",", "b", ",", "c"}], "]"}], " ", 
   RowBox[{"SUNF", "[", 
    RowBox[{"a", ",", "b", ",", "e"}], "]"}]}], "]"}]], "Input",
 CellTags->"Calc",
 CellLabel->"In[111]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["C", "A"], " ", 
   SubscriptBox["\[Delta]", 
    RowBox[{
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["e",
       TraditionalForm],
      TraditionalForm]}]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970143965877*^9},
 CellTags->"Calc",
 CellLabel->"Out[111]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"FV", "[", 
   RowBox[{
    RowBox[{"p", "+", "r"}], ",", "\[Mu]"}], "]"}], " ", 
  RowBox[{"MT", "[", 
   RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}], " ", 
  RowBox[{"FV", "[", 
   RowBox[{
    RowBox[{"q", "-", "p"}], ",", "\[Nu]"}], "]"}]}]], "Input",
 CellTags->"Calc",
 CellLabel->"In[112]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["g", 
    RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
   RowBox[{"(", 
    SuperscriptBox[
     FormBox[
      RowBox[{"q", "-", "p"}],
      TraditionalForm], "\[Nu]"], ")"}], " ", 
   RowBox[{"(", 
    SuperscriptBox[
     FormBox[
      RowBox[{"p", "+", "r"}],
      TraditionalForm], "\[Mu]"], ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970144070318*^9},
 CellTags->"Calc",
 CellLabel->"Out[112]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Calc", "[", "%", "]"}]], "Input",
 CellTags->"Calc",
 CellLabel->"In[113]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    FormBox["p",
     TraditionalForm], "\[CenterDot]", 
    FormBox["q",
     TraditionalForm]}], "-", 
   RowBox[{
    FormBox["p",
     TraditionalForm], "\[CenterDot]", 
    FormBox["r",
     TraditionalForm]}], "-", 
   SuperscriptBox[
    FormBox["p",
     TraditionalForm], "2"], "+", 
   RowBox[{
    FormBox["q",
     TraditionalForm], "\[CenterDot]", 
    FormBox["r",
     TraditionalForm]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009701442486773`*^9},
 CellTags->"Calc",
 CellLabel->"Out[113]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GluonVertex", "[", 
  RowBox[{"p", ",", "1", ",", "q", ",", "2", ",", 
   RowBox[{
    RowBox[{"-", "p"}], "-", "q"}], ",", "3"}], "]"}]], "Input",
 CellTags->"Calc",
 CellLabel->"In[114]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["f", 
    RowBox[{
     FormBox[
      FormBox["ci1",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["ci2",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["ci3",
       TraditionalForm],
      TraditionalForm]}]], " ", 
   RowBox[{
    SuperscriptBox["V", 
     RowBox[{
      FormBox[
       FormBox["li1",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["li2",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["li3",
        TraditionalForm],
       TraditionalForm]}]], "(", 
    RowBox[{
     FormBox["p",
      TraditionalForm], 
     RowBox[{",", " "}], 
     FormBox["q",
      TraditionalForm], 
     RowBox[{",", " "}], 
     FormBox[
      RowBox[{
       RowBox[{"-", "p"}], "-", "q"}],
      TraditionalForm]}], ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009701443697443`*^9},
 CellTags->"Calc",
 CellLabel->"Out[114]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Calc", "[", 
  RowBox[{
   RowBox[{"%", " ", 
    RowBox[{"FVD", "[", 
     RowBox[{"p", ",", "li1"}], "]"}]}], " ", 
   RowBox[{"FVD", "[", 
    RowBox[{"q", ",", "li2"}], "]"}], " ", 
   RowBox[{"FVD", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"-", "p"}], "-", "q"}], ",", "li3"}], "]"}]}], "]"}]], "Input",
 CellTags->"Calc",
 CellLabel->"In[115]:="],

Cell[BoxData[
 FormBox["0", TraditionalForm]], "Output",
 CellChangeTimes->{3.500970144564555*^9},
 CellTags->"Calc",
 CellLabel->"Out[115]="]
}, Open  ]]
}, Open  ]],

Cell["CalcColorFactor", "ObjectName",
 CellTags->"CalcColorFactor"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"CalcColorFactor"],

Cell["\<\
CalcColorFactor[expr] calculates the color factor of expr.  CalcColorFactor \
is useful for application on FeynArts produced amplitudes. CalcColorFactor is \
just a macro function for CalcColorFactor[x_] := 
If[FreeQ2[FeynCalcInternal[x], SUNIndex],  x, 
                    SUNSimplify[SUNSimplify[  (If[ !FreeQ[#1, DiracGamma], \
DiracTrick[#1], #1] & )[  SUNSimplify[x]], Explicit -> False], Explicit -> \
True]].\
\>", "Text",
 CellTags->"CalcColorFactor"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["SUNSimplify",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SUNSimplify",
  ButtonNote->"SUNSimplify"],
 "."
}], "Text",
 CellTags->"CalcColorFactor"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CalcColorFactor", "[", 
  RowBox[{
   RowBox[{"SUNF", "[", 
    RowBox[{"a", ",", "b", ",", "c"}], "]"}], " ", 
   RowBox[{"SUNF", "[", 
    RowBox[{"a", ",", "b", ",", "d"}], "]"}]}], "]"}]], "Input",
 CellLabel->"In[116]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["C", "A"], " ", 
   SubscriptBox["\[Delta]", 
    RowBox[{
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["d",
       TraditionalForm],
      TraditionalForm]}]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009701450000563`*^9},
 CellLabel->"Out[116]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Amplitude", "[", "\"\<q2se2\>\"", "]"}]], "Input",
 CellLabel->"In[117]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubsuperscriptBox["\[CapitalPi]", 
     RowBox[{
      SubscriptBox["c", "1"], 
      SubscriptBox["c", "3"]}], 
     RowBox[{
      SubscriptBox["\[Lambda]", "1"], 
      SubscriptBox["\[Lambda]", "3"]}]], "(", "q2", ")"}], " ", 
   RowBox[{
    SubsuperscriptBox["\[CapitalPi]", 
     RowBox[{
      SubscriptBox["c", "9"], 
      SubscriptBox["c", "11"]}], 
     RowBox[{
      SubscriptBox["\[Lambda]", "9"], 
      SubscriptBox["\[Lambda]", "11"]}]], "(", "q2", ")"}], " ", 
   RowBox[{
    SubsuperscriptBox["Q", 
     SubscriptBox["c", "11"], 
     SubscriptBox["\[Lambda]", "11"]], ".", 
    RowBox[{
     SubscriptBox["\[CapitalPi]", "q"], "(", 
     FormBox[
      RowBox[{"p", "-", "q2"}],
      TraditionalForm], ")"}], ".", 
    SubsuperscriptBox["Q", 
     SubscriptBox["c", "1"], 
     SubscriptBox["\[Lambda]", "1"]]}], " ", 
   RowBox[{"(", 
    RowBox[{"-", 
     RowBox[{"tr", "(", 
      FormBox[
       RowBox[{"2", " ", 
        SubscriptBox["T", "f"], " ", 
        RowBox[{
         RowBox[{
          SubscriptBox["\[CapitalPi]", "q"], "(", 
          FormBox[
           RowBox[{"-", "q1"}],
           TraditionalForm], ")"}], ".", 
         SubsuperscriptBox["Q", 
          SubscriptBox["c", "9"], 
          SubscriptBox["\[Lambda]", "9"]], ".", 
         RowBox[{
          SubscriptBox["\[CapitalPi]", "q"], "(", 
          FormBox[
           RowBox[{"q2", "-", "q1"}],
           TraditionalForm], ")"}], ".", 
         SubsuperscriptBox["Q", 
          SubscriptBox["c", "3"], 
          SubscriptBox["\[Lambda]", "3"]]}]}],
       TraditionalForm], ")"}]}], ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970145242673*^9},
 CellLabel->"Out[117]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CalcColorFactor", "@", 
  RowBox[{"Explicit", "@", "%"}]}]], "Input",
 CellLabel->"In[118]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{"\[ImaginaryI]", " ", 
    SuperscriptBox[
     RowBox[{"(", 
      FractionBox["1", 
       SuperscriptBox["q2", "2"]], ")"}], "2"], " ", 
    SubscriptBox["C", "F"], " ", 
    SubscriptBox["T", "f"], " ", 
    SuperscriptBox["g", 
     RowBox[{
      SubscriptBox["\[Lambda]", "1"], 
      SubscriptBox["\[Lambda]", "3"]}]], " ", 
    SuperscriptBox["g", 
     RowBox[{
      SubscriptBox["\[Lambda]", "9"], 
      SubscriptBox["\[Lambda]", "11"]}]], " ", 
    SubsuperscriptBox["g", "s", "4"], " ", 
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Lambda]", "11"],
        TraditionalForm],
       TraditionalForm]], ".", 
     RowBox[{"(", 
      RowBox[{
       FormBox["\<\"\[Gamma]\"\>",
        TraditionalForm], "\[CenterDot]", "(", 
       FormBox[
        RowBox[{"p", "-", "q2"}],
        TraditionalForm], ")"}], ")"}], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Lambda]", "1"],
        TraditionalForm],
       TraditionalForm]]}], " ", 
    RowBox[{"tr", "(", 
     FormBox[
      RowBox[{"-", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          FormBox["\<\"\[Gamma]\"\>",
           TraditionalForm], "\[CenterDot]", 
          FormBox["q1",
           TraditionalForm]}], ")"}], ".", 
        SuperscriptBox["\[Gamma]", 
         FormBox[
          FormBox[
           SubscriptBox["\[Lambda]", "9"],
           TraditionalForm],
          TraditionalForm]], ".", 
        RowBox[{"(", 
         RowBox[{
          FormBox["\<\"\[Gamma]\"\>",
           TraditionalForm], "\[CenterDot]", "(", 
          FormBox[
           RowBox[{"q1", "-", "q2"}],
           TraditionalForm], ")"}], ")"}], ".", 
        SuperscriptBox["\[Gamma]", 
         FormBox[
          FormBox[
           SubscriptBox["\[Lambda]", "3"],
           TraditionalForm],
          TraditionalForm]]}]}],
      TraditionalForm], ")"}]}], 
   RowBox[{
    SuperscriptBox["q1", "2"], ".", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"p", "-", "q2"}], ")"}], "2"], ".", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"q2", "-", "q1"}], ")"}], "2"]}]], TraditionalForm]], "Output",
 CellChangeTimes->{3.50097014548011*^9},
 CellLabel->"Out[118]="]
}, Open  ]],

Cell[TextData[{
 "See also:  ",
 ButtonBox["ScalarProductCancel",
  BaseStyle->"AddOnsLink",
  ButtonData:>"ScalarProductCancel",
  ButtonNote->"ScalarProductCancel"],
 ", ",
 ButtonBox["Series2",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Series2",
  ButtonNote->"Series2"],
 ", ",
 ButtonBox["TID",
  BaseStyle->"AddOnsLink",
  ButtonData:>"TID",
  ButtonNote->"TID"],
 "."
}], "Text",
 CellTags->"Collecting"]
}, Open  ]],

Cell["CalculateCounterTerm ***unfinished***", "ObjectName",
 CellTags->"CalculateCounterTerm"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"CalculateCounterTerm"],

Cell["CalculateCounterTerm[exp, k] calculates the residue of exp.", "Text",
 CellTags->"CalculateCounterTerm"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData["Examples"], "Subsubsection",
 CellTags->"CalculateCounterTerm"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "CalculateCounterTerm"}]], "Input",
 CellTags->"CalculateCounterTerm",
 CellLabel->"In[119]:="],

Cell[BoxData[
 StyleBox["\<\"CalculateCounterTerm[exp, k] calculates the residue of \
exp.\"\>", "MSG"]], "Print", "PrintUsage",
 CellChangeTimes->{3.5009701467294197`*^9},
 CellTags->{"CalculateCounterTerm", "Info3500973745-4982097"}]
}, Open  ]]
}, Open  ]],

Cell["CancelQP", "ObjectName",
 CellTags->"CancelQP"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"CancelQP"],

Cell[TextData[{
 "CancelQP is an option for OneLoop. If set to True, cancelation of all ",
 StyleBox["q",
  FontSlant->"Italic"],
 ".",
 StyleBox["p",
  FontSlant->"Italic"],
 "'s and ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["q", "2"], TraditionalForm]]],
 " is performed."
}], "Text",
 CellTags->"CancelQP"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData["Examples"], "Subsubsection",
 CellTags->"CancelQP"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t", "=", 
  RowBox[{
   RowBox[{
    RowBox[{"SPD", "[", 
     RowBox[{
      RowBox[{"p", "+", 
       RowBox[{"2", "q2"}]}], ",", 
      RowBox[{"p", "+", "q2"}]}], "]"}], " ", 
    RowBox[{"FAD", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{
        RowBox[{"q2", "-", "p"}], ",", "m"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"q2", ",", "0"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"q2", ",", "M"}], "}"}]}], "]"}]}], "//", "FCI"}]}]], "Input",
 CellTags->"CancelQP",
 CellLabel->"In[120]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{"(", 
    FormBox[
     RowBox[{"p", "+", "q2"}],
     TraditionalForm], ")", "\[CenterDot]", "(", 
    FormBox[
     RowBox[{"p", "+", 
      RowBox[{"2", " ", "q2"}]}],
     TraditionalForm], ")"}], 
   RowBox[{
    SuperscriptBox["q2", "2"], ".", 
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox["q2", "2"], "-", 
      SuperscriptBox["M", "2"]}], ")"}], ".", 
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"q2", "-", "p"}], ")"}], "2"], "-", 
      SuperscriptBox["m", "2"]}], ")"}]}]], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970147365191*^9},
 CellTags->"CancelQP",
 CellLabel->"Out[120]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"OneLoop", "[", 
  RowBox[{"q2", ",", "t", ",", 
   RowBox[{"CancelQP", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"CancelQP",
 CellLabel->"In[121]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"\[ImaginaryI]", " ", 
   SuperscriptBox["\[Pi]", "2"], " ", 
   RowBox[{"(", 
    RowBox[{
     FractionBox[
      RowBox[{"3", " ", 
       SuperscriptBox["m", "2"], " ", 
       RowBox[{
        SubscriptBox["B", "0"], "(", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], "2"], ",", "0", ",", 
        SuperscriptBox["m", "2"], ")"}]}], 
      RowBox[{"2", " ", 
       SuperscriptBox["M", "2"]}]], "-", 
     FractionBox[
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"3", " ", 
          SuperscriptBox["m", "2"]}], "-", 
         RowBox[{"7", " ", 
          SuperscriptBox["M", "2"]}]}], ")"}], " ", 
       RowBox[{
        SubscriptBox["B", "0"], "(", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], "2"], ",", 
        SuperscriptBox["m", "2"], ",", 
        SuperscriptBox["M", "2"], ")"}]}], 
      RowBox[{"2", " ", 
       SuperscriptBox["M", "2"]}]], "-", 
     FractionBox[
      RowBox[{"5", " ", 
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], "2"], " ", 
       RowBox[{
        SubscriptBox["B", "0"], "(", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], "2"], ",", "0", ",", 
        SuperscriptBox["m", "2"], ")"}]}], 
      RowBox[{"2", " ", 
       SuperscriptBox["M", "2"]}]], "+", 
     FractionBox[
      RowBox[{"5", " ", 
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], "2"], " ", 
       RowBox[{
        SubscriptBox["B", "0"], "(", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], "2"], ",", 
        SuperscriptBox["m", "2"], ",", 
        SuperscriptBox["M", "2"], ")"}]}], 
      RowBox[{"2", " ", 
       SuperscriptBox["M", "2"]}]], "-", 
     FractionBox[
      RowBox[{"3", " ", 
       RowBox[{
        FormBox[
         SubscriptBox["\<\"A\"\>", "\<\"0\"\>"],
         TraditionalForm], "(", 
        FormBox[
         SuperscriptBox["M", "2"],
         TraditionalForm], ")"}]}], 
      RowBox[{"2", " ", 
       SuperscriptBox["M", "2"]}]]}], ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009701484180813`*^9},
 CellTags->"CancelQP",
 CellLabel->"Out[121]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"OneLoop", "[", 
  RowBox[{"q2", ",", "t", ",", 
   RowBox[{"CancelQP", "\[Rule]", "False"}]}], "]"}]], "Input",
 CellTags->"CancelQP",
 CellLabel->"In[122]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"\[ImaginaryI]", " ", 
   SuperscriptBox["\[Pi]", "2"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      FractionBox[
       RowBox[{"8", " ", 
        RowBox[{"PaVe", "(", 
         RowBox[{"0", ",", "0", ",", 
          RowBox[{"{", 
           SuperscriptBox[
            FormBox["p",
             TraditionalForm], "2"], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"0", ",", 
            SuperscriptBox["m", "2"]}], "}"}]}], ")"}]}], 
       SuperscriptBox["M", "2"]]}], "+", 
     FractionBox[
      RowBox[{"8", " ", 
       RowBox[{"PaVe", "(", 
        RowBox[{"0", ",", "0", ",", 
         RowBox[{"{", 
          SuperscriptBox[
           FormBox["p",
            TraditionalForm], "2"], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{
           SuperscriptBox["m", "2"], ",", 
           SuperscriptBox["M", "2"]}], "}"}]}], ")"}]}], 
      SuperscriptBox["M", "2"]], "+", 
     FractionBox[
      RowBox[{"17", " ", 
       SuperscriptBox["m", "2"], " ", 
       RowBox[{
        SubscriptBox["B", "0"], "(", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], "2"], ",", "0", ",", 
        SuperscriptBox["m", "2"], ")"}]}], 
      RowBox[{"6", " ", 
       SuperscriptBox["M", "2"]}]], "+", 
     FractionBox[
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"13", " ", 
          SuperscriptBox["m", "2"]}], "-", 
         RowBox[{"17", " ", 
          SuperscriptBox["M", "2"]}]}], ")"}], " ", 
       RowBox[{
        SubscriptBox["B", "0"], "(", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], "2"], ",", 
        SuperscriptBox["m", "2"], ",", 
        SuperscriptBox["M", "2"], ")"}]}], 
      RowBox[{"6", " ", 
       SuperscriptBox["M", "2"]}]], "-", 
     FractionBox[
      RowBox[{"19", " ", 
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], "2"], " ", 
       RowBox[{
        SubscriptBox["B", "0"], "(", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], "2"], ",", "0", ",", 
        SuperscriptBox["m", "2"], ")"}]}], 
      RowBox[{"6", " ", 
       SuperscriptBox["M", "2"]}]], "+", 
     FractionBox[
      RowBox[{"19", " ", 
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], "2"], " ", 
       RowBox[{
        SubscriptBox["B", "0"], "(", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], "2"], ",", 
        SuperscriptBox["m", "2"], ",", 
        SuperscriptBox["M", "2"], ")"}]}], 
      RowBox[{"6", " ", 
       SuperscriptBox["M", "2"]}]], "-", 
     FractionBox[
      RowBox[{"2", " ", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox["m", "2"], "-", 
          SuperscriptBox["M", "2"]}], ")"}], "2"], " ", 
       RowBox[{
        SubscriptBox["B", "0"], "(", "0", ",", 
        SuperscriptBox["m", "2"], ",", 
        SuperscriptBox["M", "2"], ")"}]}], 
      RowBox[{"3", " ", 
       SuperscriptBox["M", "2"], " ", 
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], "2"]}]], "+", 
     FractionBox[
      RowBox[{"2", " ", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox["m", "2"], "-", 
          SuperscriptBox["M", "2"]}], ")"}], "2"], " ", 
       RowBox[{
        SubscriptBox["B", "0"], "(", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], "2"], ",", 
        SuperscriptBox["m", "2"], ",", 
        SuperscriptBox["M", "2"], ")"}]}], 
      RowBox[{"3", " ", 
       SuperscriptBox["M", "2"], " ", 
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], "2"]}]], "-", 
     FractionBox[
      RowBox[{"13", " ", 
       SuperscriptBox["m", "2"], " ", 
       RowBox[{
        SubscriptBox["B", "0"], "(", "0", ",", 
        SuperscriptBox["m", "2"], ",", 
        SuperscriptBox["m", "2"], ")"}]}], 
      RowBox[{"6", " ", 
       SuperscriptBox["M", "2"]}]], "-", 
     FractionBox[
      RowBox[{"13", " ", 
       SuperscriptBox["m", "2"], " ", 
       RowBox[{
        SubscriptBox["B", "0"], "(", "0", ",", 
        SuperscriptBox["m", "2"], ",", 
        SuperscriptBox["M", "2"], ")"}]}], 
      RowBox[{"6", " ", 
       SuperscriptBox["M", "2"]}]], "+", 
     RowBox[{
      FractionBox["13", "6"], " ", 
      RowBox[{
       SubscriptBox["B", "0"], "(", "0", ",", 
       SuperscriptBox["m", "2"], ",", 
       SuperscriptBox["M", "2"], ")"}]}], "+", 
     FractionBox[
      RowBox[{"2", " ", 
       SuperscriptBox["m", "4"], " ", 
       RowBox[{
        SubscriptBox["B", "0"], "(", "0", ",", 
        SuperscriptBox["m", "2"], ",", 
        SuperscriptBox["m", "2"], ")"}]}], 
      RowBox[{"3", " ", 
       SuperscriptBox["M", "2"], " ", 
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], "2"]}]], "-", 
     FractionBox[
      RowBox[{"2", " ", 
       SuperscriptBox["m", "4"], " ", 
       RowBox[{
        SubscriptBox["B", "0"], "(", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], "2"], ",", "0", ",", 
        SuperscriptBox["m", "2"], ")"}]}], 
      RowBox[{"3", " ", 
       SuperscriptBox["M", "2"], " ", 
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], "2"]}]], "-", 
     FractionBox[
      RowBox[{"2", " ", 
       RowBox[{
        FormBox[
         SubscriptBox["\<\"A\"\>", "\<\"0\"\>"],
         TraditionalForm], "(", 
        FormBox[
         SuperscriptBox["m", "2"],
         TraditionalForm], ")"}]}], 
      RowBox[{"3", " ", 
       SuperscriptBox["M", "2"]}]], "+", 
     FractionBox[
      RowBox[{"2", " ", 
       RowBox[{
        FormBox[
         SubscriptBox["\<\"A\"\>", "\<\"0\"\>"],
         TraditionalForm], "(", 
        FormBox[
         SuperscriptBox["M", "2"],
         TraditionalForm], ")"}]}], 
      RowBox[{"3", " ", 
       SuperscriptBox["M", "2"]}]], "+", 
     FractionBox[
      RowBox[{"2", " ", 
       SuperscriptBox["m", "4"]}], 
      RowBox[{"3", " ", 
       SuperscriptBox["M", "2"], " ", 
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], "2"]}]], "-", 
     FractionBox[
      RowBox[{"13", " ", 
       SuperscriptBox["m", "2"]}], 
      RowBox[{"6", " ", 
       SuperscriptBox["M", "2"]}]], "-", 
     FractionBox["4", "3"]}], ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.50097015025817*^9},
 CellTags->"CancelQP",
 CellLabel->"Out[122]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t", "=", 
  RowBox[{
   RowBox[{
    RowBox[{"SPD", "[", 
     RowBox[{"q2", ",", "p"}], "]"}], 
    RowBox[{"SPD", "[", 
     RowBox[{"q1", ",", "p"}], "]"}], " ", 
    RowBox[{"FAD", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"q1", ",", "m"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"q2", ",", "m"}], "}"}], ",", 
      RowBox[{"q1", "-", "p"}], ",", 
      RowBox[{"q2", "-", "p"}], ",", 
      RowBox[{"q2", "-", "q1"}]}], "]"}]}], "//", "FCI"}]}]], "Input",
 CellTags->"CancelQP",
 CellLabel->"In[123]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{
    RowBox[{
     FormBox["p",
      TraditionalForm], "\[CenterDot]", 
     FormBox["q1",
      TraditionalForm]}], " ", 
    RowBox[{
     FormBox["p",
      TraditionalForm], "\[CenterDot]", 
     FormBox["q2",
      TraditionalForm]}]}], 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox["q1", "2"], "-", 
      SuperscriptBox["m", "2"]}], ")"}], ".", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"q1", "-", "p"}], ")"}], "2"], ".", 
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox["q2", "2"], "-", 
      SuperscriptBox["m", "2"]}], ")"}], ".", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"q2", "-", "p"}], ")"}], "2"], ".", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"q2", "-", "q1"}], ")"}], "2"]}]], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009701506982393`*^9},
 CellTags->"CancelQP",
 CellLabel->"Out[123]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"OneLoop", "[", 
  RowBox[{"q2", ",", "t", ",", 
   RowBox[{"CancelQP", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"CancelQP",
 CellLabel->"In[124]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"\[ImaginaryI]", " ", 
   SuperscriptBox["\[Pi]", "2"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      FractionBox[
       RowBox[{
        SuperscriptBox["m", "2"], " ", 
        RowBox[{
         FormBox["p",
          TraditionalForm], "\[CenterDot]", 
         FormBox["q1",
          TraditionalForm]}], " ", 
        RowBox[{
         FormBox[
          SubscriptBox["\<\"C\"\>", "\<\"0\"\>"],
          TraditionalForm], "(", 
         FormBox[
          SuperscriptBox[
           FormBox["p",
            TraditionalForm], "2"],
          TraditionalForm], ",", 
         FormBox[
          SuperscriptBox[
           FormBox["q1",
            TraditionalForm], "2"],
          TraditionalForm], ",", 
         FormBox[
          RowBox[{
           RowBox[{"-", 
            RowBox[{"2", " ", 
             RowBox[{
              FormBox["p",
               TraditionalForm], "\[CenterDot]", 
              FormBox["q1",
               TraditionalForm]}]}]}], "+", 
           SuperscriptBox[
            FormBox["p",
             TraditionalForm], "2"], "+", 
           SuperscriptBox[
            FormBox["q1",
             TraditionalForm], "2"]}],
          TraditionalForm], ",", 
         FormBox["0",
          TraditionalForm], ",", 
         FormBox["0",
          TraditionalForm], ",", 
         FormBox[
          SuperscriptBox["m", "2"],
          TraditionalForm], ")"}]}], 
       RowBox[{"2", " ", 
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox["m", "2"], "-", 
          SuperscriptBox[
           FormBox["q1",
            TraditionalForm], "2"]}], ")"}], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"-", 
           RowBox[{"2", " ", 
            RowBox[{
             FormBox["p",
              TraditionalForm], "\[CenterDot]", 
             FormBox["q1",
              TraditionalForm]}]}]}], "+", 
          SuperscriptBox[
           FormBox["p",
            TraditionalForm], "2"], "+", 
          SuperscriptBox[
           FormBox["q1",
            TraditionalForm], "2"]}], ")"}]}]]}], "-", 
     FractionBox[
      RowBox[{
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], "2"], " ", 
       RowBox[{
        FormBox["p",
         TraditionalForm], "\[CenterDot]", 
        FormBox["q1",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox[
         SubscriptBox["\<\"C\"\>", "\<\"0\"\>"],
         TraditionalForm], "(", 
        FormBox[
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], "2"],
         TraditionalForm], ",", 
        FormBox[
         SuperscriptBox[
          FormBox["q1",
           TraditionalForm], "2"],
         TraditionalForm], ",", 
        FormBox[
         RowBox[{
          RowBox[{"-", 
           RowBox[{"2", " ", 
            RowBox[{
             FormBox["p",
              TraditionalForm], "\[CenterDot]", 
             FormBox["q1",
              TraditionalForm]}]}]}], "+", 
          SuperscriptBox[
           FormBox["p",
            TraditionalForm], "2"], "+", 
          SuperscriptBox[
           FormBox["q1",
            TraditionalForm], "2"]}],
         TraditionalForm], ",", 
        FormBox["0",
         TraditionalForm], ",", 
        FormBox["0",
         TraditionalForm], ",", 
        FormBox[
         SuperscriptBox["m", "2"],
         TraditionalForm], ")"}]}], 
      RowBox[{"2", " ", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox["m", "2"], "-", 
         SuperscriptBox[
          FormBox["q1",
           TraditionalForm], "2"]}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", 
          RowBox[{"2", " ", 
           RowBox[{
            FormBox["p",
             TraditionalForm], "\[CenterDot]", 
            FormBox["q1",
             TraditionalForm]}]}]}], "+", 
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], "2"], "+", 
         SuperscriptBox[
          FormBox["q1",
           TraditionalForm], "2"]}], ")"}]}]], "+", 
     FractionBox[
      RowBox[{
       RowBox[{
        FormBox["p",
         TraditionalForm], "\[CenterDot]", 
        FormBox["q1",
         TraditionalForm]}], " ", 
       RowBox[{
        SubscriptBox["B", "0"], "(", 
        SuperscriptBox[
         FormBox["q1",
          TraditionalForm], "2"], ",", "0", ",", 
        SuperscriptBox["m", "2"], ")"}]}], 
      RowBox[{"2", " ", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox["m", "2"], "-", 
         SuperscriptBox[
          FormBox["q1",
           TraditionalForm], "2"]}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", 
          RowBox[{"2", " ", 
           RowBox[{
            FormBox["p",
             TraditionalForm], "\[CenterDot]", 
            FormBox["q1",
             TraditionalForm]}]}]}], "+", 
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], "2"], "+", 
         SuperscriptBox[
          FormBox["q1",
           TraditionalForm], "2"]}], ")"}]}]], "-", 
     FractionBox[
      RowBox[{
       RowBox[{
        FormBox["p",
         TraditionalForm], "\[CenterDot]", 
        FormBox["q1",
         TraditionalForm]}], " ", 
       RowBox[{
        SubscriptBox["B", "0"], "(", 
        RowBox[{
         RowBox[{"-", 
          RowBox[{"2", " ", 
           RowBox[{
            FormBox["p",
             TraditionalForm], "\[CenterDot]", 
            FormBox["q1",
             TraditionalForm]}]}]}], "+", 
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], "2"], "+", 
         SuperscriptBox[
          FormBox["q1",
           TraditionalForm], "2"]}], ",", "0", ",", "0", ")"}]}], 
      RowBox[{"2", " ", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox["m", "2"], "-", 
         SuperscriptBox[
          FormBox["q1",
           TraditionalForm], "2"]}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", 
          RowBox[{"2", " ", 
           RowBox[{
            FormBox["p",
             TraditionalForm], "\[CenterDot]", 
            FormBox["q1",
             TraditionalForm]}]}]}], "+", 
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], "2"], "+", 
         SuperscriptBox[
          FormBox["q1",
           TraditionalForm], "2"]}], ")"}]}]]}], ")"}]}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970152137806*^9},
 CellTags->"CancelQP",
 CellLabel->"Out[124]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"OneLoop", "[", 
  RowBox[{"q2", ",", "t", ",", 
   RowBox[{"CancelQP", "\[Rule]", "False"}]}], "]"}]], "Input",
 CellTags->"CancelQP",
 CellLabel->"In[125]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"\[ImaginaryI]", " ", 
   SuperscriptBox["\[Pi]", "2"], " ", 
   RowBox[{"(", 
    RowBox[{
     FractionBox[
      RowBox[{
       SuperscriptBox[
        FormBox[
         RowBox[{
          FormBox["p",
           TraditionalForm], "\[CenterDot]", 
          FormBox["q1",
           TraditionalForm]}],
         TraditionalForm], "2"], " ", 
       RowBox[{"PaVe", "(", 
        RowBox[{"2", ",", 
         RowBox[{"{", 
          RowBox[{
           SuperscriptBox[
            FormBox["p",
             TraditionalForm], "2"], ",", 
           RowBox[{
            RowBox[{"-", 
             RowBox[{"2", " ", 
              RowBox[{
               FormBox["p",
                TraditionalForm], "\[CenterDot]", 
               FormBox["q1",
                TraditionalForm]}]}]}], "+", 
            SuperscriptBox[
             FormBox["p",
              TraditionalForm], "2"], "+", 
            SuperscriptBox[
             FormBox["q1",
              TraditionalForm], "2"]}], ",", 
           SuperscriptBox[
            FormBox["q1",
             TraditionalForm], "2"]}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"0", ",", "0", ",", 
           SuperscriptBox["m", "2"]}], "}"}]}], ")"}]}], 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox["m", "2"], "-", 
         SuperscriptBox[
          FormBox["q1",
           TraditionalForm], "2"]}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", 
          RowBox[{"2", " ", 
           RowBox[{
            FormBox["p",
             TraditionalForm], "\[CenterDot]", 
            FormBox["q1",
             TraditionalForm]}]}]}], "+", 
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], "2"], "+", 
         SuperscriptBox[
          FormBox["q1",
           TraditionalForm], "2"]}], ")"}]}]], "+", 
     FractionBox[
      RowBox[{
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], "2"], " ", 
       RowBox[{
        FormBox["p",
         TraditionalForm], "\[CenterDot]", 
        FormBox["q1",
         TraditionalForm]}], " ", 
       RowBox[{"PaVe", "(", 
        RowBox[{"1", ",", 
         RowBox[{"{", 
          RowBox[{
           SuperscriptBox[
            FormBox["p",
             TraditionalForm], "2"], ",", 
           RowBox[{
            RowBox[{"-", 
             RowBox[{"2", " ", 
              RowBox[{
               FormBox["p",
                TraditionalForm], "\[CenterDot]", 
               FormBox["q1",
                TraditionalForm]}]}]}], "+", 
            SuperscriptBox[
             FormBox["p",
              TraditionalForm], "2"], "+", 
            SuperscriptBox[
             FormBox["q1",
              TraditionalForm], "2"]}], ",", 
           SuperscriptBox[
            FormBox["q1",
             TraditionalForm], "2"]}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"0", ",", "0", ",", 
           SuperscriptBox["m", "2"]}], "}"}]}], ")"}]}], 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox["m", "2"], "-", 
         SuperscriptBox[
          FormBox["q1",
           TraditionalForm], "2"]}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", 
          RowBox[{"2", " ", 
           RowBox[{
            FormBox["p",
             TraditionalForm], "\[CenterDot]", 
            FormBox["q1",
             TraditionalForm]}]}]}], "+", 
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], "2"], "+", 
         SuperscriptBox[
          FormBox["q1",
           TraditionalForm], "2"]}], ")"}]}]]}], ")"}]}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.5009701534546547`*^9},
 CellTags->"CancelQP",
 CellLabel->"Out[125]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"Clear", "[", "t", "]"}]], "Input",
 CellTags->"CancelQP",
 CellLabel->"In[126]:="]
}, Open  ]],

Cell["Cases2", "ObjectName",
 CellTags->"Cases2"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Cases2"],

Cell["\<\
Cases2[expr, f] returns a list of all objects in expr with head f. \
Cases2[expr,f] is equivalent to Cases2[{expr},f[___],Infinity]//Union. \
Cases2[expr, f, g, ...] or Cases2[expr, {f,g, ...}] is equivalent to \
Cases[{expr},f[___] | g[___] ...] .\
\>", "Text",
 CellTags->"Cases2"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData["Examples"], "Subsubsection",
 CellTags->"Cases2"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Cases2", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"f", "[", "a", "]"}], "+", 
    SuperscriptBox[
     RowBox[{"f", "[", "b", "]"}], "2"], "+", 
    RowBox[{"f", "[", 
     RowBox[{"c", ",", "d"}], "]"}]}], ",", "f"}], "]"}]], "Input",
 CellTags->"Cases2",
 CellLabel->"In[127]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"f", "(", "a", ")"}], ",", 
    RowBox[{"f", "(", "b", ")"}], ",", 
    RowBox[{"f", "(", 
     RowBox[{"c", ",", "d"}], ")"}]}], "}"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009701536401243`*^9},
 CellTags->"Cases2",
 CellLabel->"Out[127]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Cases2", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"Sin", "[", "x", "]"}], " ", 
     RowBox[{"Sin", "[", 
      RowBox[{"y", "-", "z"}], "]"}]}], "+", 
    RowBox[{"g", "[", "y", "]"}]}], ",", "Sin", ",", "g"}], "]"}]], "Input",
 CellTags->"Cases2",
 CellLabel->"In[128]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"g", "(", "y", ")"}], ",", 
    RowBox[{"sin", "(", "x", ")"}], ",", 
    RowBox[{"sin", "(", 
     RowBox[{"y", "-", "z"}], ")"}]}], "}"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009701537494583`*^9},
 CellTags->"Cases2",
 CellLabel->"Out[128]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Cases2", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"Sin", "[", "x", "]"}], " ", 
     RowBox[{"Sin", "[", 
      RowBox[{"y", "-", "z"}], "]"}]}], "+", 
    RowBox[{"g", "[", "x", "]"}], "+", 
    RowBox[{"g", "[", 
     RowBox[{"a", ",", "b", ",", "c"}], "]"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"Sin", ",", "g"}], "}"}]}], "]"}]], "Input",
 CellTags->"Cases2",
 CellLabel->"In[129]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"g", "(", "x", ")"}], ",", 
    RowBox[{"g", "(", 
     RowBox[{"a", ",", "b", ",", "c"}], ")"}], ",", 
    RowBox[{"sin", "(", "x", ")"}], ",", 
    RowBox[{"sin", "(", 
     RowBox[{"y", "-", "z"}], ")"}]}], "}"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970153859394*^9},
 CellTags->"Cases2",
 CellLabel->"Out[129]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Cases2", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"DiracSlash", "[", "p", "]"}], ".", 
     RowBox[{"DiracSlash", "[", "q", "]"}]}], "+", 
    RowBox[{"ScalarProduct", "[", 
     RowBox[{"p", ",", "p"}], "]"}]}], ",", "Dot"}], "]"}]], "Input",
 CellTags->"Cases2",
 CellLabel->"In[130]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      FormBox["\<\"\[Gamma]\"\>",
       TraditionalForm], "\[CenterDot]", 
      FormBox["p",
       TraditionalForm]}], ")"}], ".", 
    RowBox[{"(", 
     RowBox[{
      FormBox["\<\"\[Gamma]\"\>",
       TraditionalForm], "\[CenterDot]", 
      FormBox["q",
       TraditionalForm]}], ")"}]}], "}"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970153974234*^9},
 CellTags->"Cases2",
 CellLabel->"Out[130]="]
}, Open  ]]
}, Open  ]],

Cell["CF", "ObjectName",
 CellTags->"CF"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"CF"],

Cell[TextData[{
 "CF is one of the Casimir operator eigenvalues of SU(",
 StyleBox["N",
  FontSlant->"Italic"],
 ") (CF = (",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["N", "2"], TraditionalForm]]],
 "-1)/(2 ",
 StyleBox["N",
  FontSlant->"Italic"],
 "))."
}], "Text",
 CellTags->"CF"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["CA",
  BaseStyle->"AddOnsLink",
  ButtonData:>"CA",
  ButtonNote->"CA"],
 ", ",
 ButtonBox["SUNSimplify",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SUNSimplify",
  ButtonNote->"SUNSimplify"],
 "."
}], "Text",
 CellTags->"CF"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"CF"],

Cell[CellGroupData[{

Cell[BoxData["CF"], "Input",
 CellTags->"CF",
 CellLabel->"In[131]:="],

Cell[BoxData[
 FormBox[
  SubscriptBox["C", "F"], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009701540766363`*^9},
 CellTags->"CF",
 CellLabel->"Out[131]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNSimplify", "[", 
  RowBox[{"CF", ",", " ", 
   RowBox[{"SUNNToCACF", "\[Rule]", "False"}]}], "]"}]], "Input",
 CellTags->"CF",
 CellLabel->"In[132]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{
    SuperscriptBox["N", "2"], "-", "1"}], 
   RowBox[{"2", " ", "N"}]], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970154186607*^9},
 CellTags->"CF",
 CellLabel->"Out[132]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "InputForm"}]], "Input",
 CellLabel->"In[133]:="],

Cell["(-1 + SUNN^2)/(2*SUNN)", "Output",
 CellChangeTimes->{3.500970154316777*^9},
 CellLabel->"Out[133]//InputForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData["SUNN"], "Input",
 CellTags->"CF",
 CellLabel->"In[134]:="],

Cell[BoxData[
 FormBox["N", TraditionalForm]], "Output",
 CellChangeTimes->{3.500970154438983*^9},
 CellTags->"CF",
 CellLabel->"Out[134]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"SUNN", "^", "2"}], "-", "1"}], ",", " ", 
   RowBox[{"SUNNToCACF", " ", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"CF",
 CellLabel->"In[135]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"2", " ", 
   SubscriptBox["C", "A"], " ", 
   SubscriptBox["C", "F"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970154578067*^9},
 CellTags->"CF",
 CellLabel->"Out[135]="]
}, Open  ]]
}, Open  ]],

Cell["ChangeDimension", "ObjectName",
 CellTags->"ChangeDimension"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"ChangeDimension"],

Cell["\<\
ChangeDimension[exp, dim] changes all LorentzIndex and Momenta in exp to \
dimension dim (and also Levi-Civita-tensors, Dirac slashes and Dirac \
matrices).\
\>", "Text",
 CellTags->"ChangeDimension"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["LorentzIndex",
  BaseStyle->"AddOnsLink",
  ButtonData:>"LorentzIndex",
  ButtonNote->"LorentzIndex"],
 ", ",
 ButtonBox["Momentum",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Momentum",
  ButtonNote->"Momentum"],
 ", ",
 ButtonBox["DiracGamma",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracGamma",
  ButtonNote->"DiracGamma"],
 ", ",
 ButtonBox["Eps",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Eps",
  ButtonNote->"Eps"],
 "."
}], "Text",
 CellTags->"ChangeDimension"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"ChangeDimension"],

Cell["\<\
Remember that LorentzIndex[mu, 4] is simplified to LorentzIndex[mu] and \
Momentum[p, 4] to Momentum[p]. Thus the fullowing objects are defined in four \
dimensions.\
\>", "Text",
 CellTags->"ChangeDimension"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", " ", 
   RowBox[{"Momentum", "[", "p", "]"}]}], "}"}]], "Input",
 CellTags->"ChangeDimension",
 CellLabel->"In[136]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    FormBox["\[Mu]",
     TraditionalForm], ",", "p"}], "}"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970154919737*^9},
 CellTags->"ChangeDimension",
 CellLabel->"Out[136]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ChangeDimension", "[", 
  RowBox[{"%", ",", " ", "D"}], "]"}]], "Input",
 CellTags->"ChangeDimension",
 CellLabel->"In[137]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    FormBox["\[Mu]",
     TraditionalForm], ",", "p"}], "}"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970155127243*^9},
 CellTags->"ChangeDimension",
 CellLabel->"Out[137]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "StandardForm"}]], "Input",
 CellTags->"ChangeDimension",
 CellLabel->"In[138]:="],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"LorentzIndex", "[", 
    RowBox[{"\[Mu]", ",", "D"}], "]"}], ",", 
   RowBox[{"Momentum", "[", 
    RowBox[{"p", ",", "D"}], "]"}]}], "}"}]], "Output",
 CellChangeTimes->{3.500970155282235*^9},
 CellTags->"ChangeDimension",
 CellLabel->"Out[138]//StandardForm="]
}, Open  ]],

Cell["\<\
This changes all non-4-dimensional objects to 4-dimensional ones.\
\>", "Text",
 CellTags->"ChangeDimension"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"ChangeDimension", "[", 
   RowBox[{"%%", ",", " ", "4"}], "]"}], " ", "//", " ", 
  "StandardForm"}]], "Input",
 CellTags->"ChangeDimension",
 CellLabel->"In[139]:="],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
   RowBox[{"Momentum", "[", "p", "]"}]}], "}"}]], "Output",
 CellChangeTimes->{3.5009701553935413`*^9},
 CellTags->"ChangeDimension",
 CellLabel->"Out[139]//StandardForm="]
}, Open  ]],

Cell["Consider the following list of 4- and D-dimensional object.", "Text",
 CellTags->"ChangeDimension"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"GA", "[", 
     RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}], " ", 
    RowBox[{"MT", "[", 
     RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}]}], ",", " ", 
   RowBox[{
    RowBox[{"GAD", "[", 
     RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}], " ", 
    RowBox[{"MTD", "[", 
     RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}], " ", 
    RowBox[{"f", "[", "D", "]"}]}]}], "}"}]], "Input",
 CellTags->"ChangeDimension",
 CellLabel->"In[140]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     RowBox[{
      SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
      SuperscriptBox["\[Gamma]", "\[Nu]"]}], " ", 
     SuperscriptBox["g", 
      RowBox[{"\[Mu]", "\[Nu]"}]]}], ",", 
    RowBox[{
     RowBox[{"f", "(", "D", ")"}], " ", 
     RowBox[{
      SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
      SuperscriptBox["\[Gamma]", "\[Nu]"]}], " ", 
     SuperscriptBox["g", 
      RowBox[{"\[Mu]", "\[Nu]"}]]}]}], "}"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970155511058*^9},
 CellTags->"ChangeDimension",
 CellLabel->"Out[140]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracTrick", "[", 
  RowBox[{"Contract", "[", "%", "]"}], "]"}]], "Input",
 CellTags->"ChangeDimension",
 CellLabel->"In[141]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm]]}], ",", 
    RowBox[{
     RowBox[{
      SuperscriptBox["\[Gamma]", 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]], ".", 
      SuperscriptBox["\[Gamma]", 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]]}], " ", 
     RowBox[{"f", "(", "D", ")"}]}]}], "}"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970155636015*^9},
 CellTags->"ChangeDimension",
 CellLabel->"Out[141]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracTrick", "[", 
  RowBox[{"Contract", "[", 
   RowBox[{"ChangeDimension", "[", 
    RowBox[{"%%", ",", "n"}], "]"}], "]"}], "]"}]], "Input",
 CellTags->"ChangeDimension",
 CellLabel->"In[142]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm]]}], ",", 
    RowBox[{
     RowBox[{
      SuperscriptBox["\[Gamma]", 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]], ".", 
      SuperscriptBox["\[Gamma]", 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]]}], " ", 
     RowBox[{"f", "(", "D", ")"}]}]}], "}"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970155853744*^9},
 CellTags->"ChangeDimension",
 CellLabel->"Out[142]="]
}, Open  ]],

Cell["\<\
Any explicit occurence of D (like in f(D)) is not replaced by ChangeDimension.\
\>", "Text",
 CellTags->"ChangeDimension"],

Cell[BoxData[
 RowBox[{
  RowBox[{"SetOptions", "[", 
   RowBox[{"LeviCivita", ",", 
    RowBox[{"Dimension", "\[Rule]", "D"}]}], "]"}], ";"}]], "Input",
 CellTags->"ChangeDimension",
 CellLabel->"In[143]:="],

Cell["\<\
The option Dimension of Eps must be changed too, since with the default \
setting Dimension\[Rule]4 the arguments of Eps are automatically changed to 4 \
dimensions.\
\>", "Text",
 CellTags->"ChangeDimension"],

Cell[BoxData[
 RowBox[{
  RowBox[{"SetOptions", "[", 
   RowBox[{"Eps", ",", 
    RowBox[{"Dimension", "\[Rule]", "D"}]}], "]"}], ";"}]], "Input",
 CellTags->"ChangeDimension",
 CellLabel->"In[144]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"a1", " ", "=", " ", 
  RowBox[{"LeviCivita", "[", 
   RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]", ",", "\[Sigma]"}], 
   "]"}]}]], "Input",
 CellTags->"ChangeDimension",
 CellLabel->"In[145]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["\[Epsilon]", 
   RowBox[{
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["\[Nu]",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["\[Rho]",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["\[Sigma]",
      TraditionalForm],
     TraditionalForm]}]], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970156087563*^9},
 CellTags->"ChangeDimension",
 CellLabel->"Out[145]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"a2", " ", "=", " ", 
  RowBox[{"ChangeDimension", "[", 
   RowBox[{"a1", ",", "4"}], "]"}]}]], "Input",
 CellTags->"ChangeDimension",
 CellLabel->"In[146]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["\[Epsilon]", 
   RowBox[{
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["\[Nu]",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["\[Rho]",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["\[Sigma]",
      TraditionalForm],
     TraditionalForm]}]], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970156193165*^9},
 CellTags->"ChangeDimension",
 CellLabel->"Out[146]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Factor2", "[", 
  RowBox[{"Contract", "[", 
   SuperscriptBox["a1", "2"], "]"}], "]"}]], "Input",
 CellTags->"ChangeDimension",
 CellLabel->"In[147]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"(", 
    RowBox[{"1", "-", "D"}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{"2", "-", "D"}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{"3", "-", "D"}], ")"}], " ", "D"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970156315621*^9},
 CellTags->"ChangeDimension",
 CellLabel->"Out[147]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Contract", "[", 
  SuperscriptBox["a2", "2"], "]"}]], "Input",
 CellTags->"ChangeDimension",
 CellLabel->"In[148]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", "24"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009701564838123`*^9},
 CellTags->"ChangeDimension",
 CellLabel->"Out[148]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"SetOptions", "[", 
   RowBox[{"Eps", ",", 
    RowBox[{"Dimension", "\[Rule]", "4"}]}], "]"}], ";"}]], "Input",
 CellTags->"ChangeDimension",
 CellLabel->"In[149]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", 
   RowBox[{"a1", ",", "a2"}], "]"}], ";"}]], "Input",
 CellTags->"ChangeDimension",
 CellLabel->"In[150]:="]
}, Open  ]],

Cell["ChargeConjugationMatrix", "ObjectName",
 CellTags->"ChargeConjugationMatrix"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"ChargeConjugationMatrix"],

Cell[TextData[{
 "ChargeConjugationMatrix denotes the charge conjugation matrix ",
 StyleBox["C",
  FontSlant->"Italic"],
 "."
}], "Text",
 CellTags->"ChargeConjugationMatrix"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["ChargeConjugationMatrixInv",
  BaseStyle->"AddOnsLink",
  ButtonData:>"ChargeConjugationMatrixInv",
  ButtonNote->"ChargeConjugationMatrixInv"],
 "."
}], "Text",
 CellTags->"ChargeConjugationMatrix"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"ChargeConjugationMatrix"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ChargeConjugationMatrix", ".", 
  RowBox[{"DiracMatrix", "[", "\[Mu]", "]"}], ".", 
  "ChargeConjugationMatrixInv"}]], "Input",
 CellTags->"ChargeConjugationMatrix",
 CellLabel->"In[151]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"C", ".", 
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], ".", 
   RowBox[{"(", 
    RowBox[{"-", "C"}], ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970156672904*^9},
 CellTags->"ChargeConjugationMatrix",
 CellLabel->"Out[151]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Calc", "[", "%", "]"}]], "Input",
 CellTags->"ChargeConjugationMatrix",
 CellLabel->"In[152]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   SubsuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm], "T"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009701570946617`*^9},
 CellTags->"ChargeConjugationMatrix",
 CellLabel->"Out[152]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ChargeConjugationMatrix", ".", 
  RowBox[{"DiracGamma", "[", "5", "]"}], ".", 
  "ChargeConjugationMatrixInv"}]], "Input",
 CellTags->"ChargeConjugationMatrix",
 CellLabel->"In[153]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"C", ".", 
   SuperscriptBox["\[Gamma]", "5"], ".", 
   RowBox[{"(", 
    RowBox[{"-", "C"}], ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970157199966*^9},
 CellTags->"ChargeConjugationMatrix",
 CellLabel->"Out[153]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Calc", "[", "%", "]"}]], "Input",
 CellTags->"ChargeConjugationMatrix",
 CellLabel->"In[154]:="],

Cell[BoxData[
 FormBox[
  SubsuperscriptBox["\[Gamma]", 
   FormBox["5",
    TraditionalForm], "T"], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970157327587*^9},
 CellTags->"ChargeConjugationMatrix",
 CellLabel->"Out[154]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ChargeConjugationMatrix", "^", "2"}]], "Input",
 CellTags->"ChargeConjugationMatrix",
 CellLabel->"In[155]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", "1"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970157430522*^9},
 CellTags->"ChargeConjugationMatrix",
 CellLabel->"Out[155]="]
}, Open  ]]
}, Open  ]],

Cell["ChargeConjugationMatrixInv", "ObjectName",
 CellTags->"ChargeConjugationMatrixInv"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"ChargeConjugationMatrixInv"],

Cell["\<\
ChargeConjugationMatrixInv is the inverse of ChargeConjugationMatrix. It is \
substituted immediately by -ChargeConjugationMatrix.\
\>", "Text",
 CellTags->"ChargeConjugationMatrixInv"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["ChargeConjugationMatrix",
  BaseStyle->"AddOnsLink",
  ButtonData:>"ChargeConjugationMatrix",
  ButtonNote->"ChargeConjugationMatrix"],
 "."
}], "Text",
 CellTags->"ChargeConjugationMatrixInv"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"ChargeConjugationMatrixInv"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ChargeConjugationMatrix", ".", 
  RowBox[{"DiracMatrix", "[", "\[Mu]", "]"}], ".", 
  "ChargeConjugationMatrixInv"}]], "Input",
 CellTags->"ChargeConjugationMatrixInv",
 CellLabel->"In[156]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"C", ".", 
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], ".", 
   RowBox[{"(", 
    RowBox[{"-", "C"}], ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970157536129*^9},
 CellTags->"ChargeConjugationMatrixInv",
 CellLabel->"Out[156]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Calc", "[", "%", "]"}]], "Input",
 CellTags->"ChargeConjugationMatrixInv",
 CellLabel->"In[157]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   SubsuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm], "T"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970157710026*^9},
 CellTags->"ChargeConjugationMatrixInv",
 CellLabel->"Out[157]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ChargeConjugationMatrix", ".", 
  RowBox[{"DiracGamma", "[", "5", "]"}], ".", 
  "ChargeConjugationMatrixInv"}]], "Input",
 CellTags->"ChargeConjugationMatrixInv",
 CellLabel->"In[158]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"C", ".", 
   SuperscriptBox["\[Gamma]", "5"], ".", 
   RowBox[{"(", 
    RowBox[{"-", "C"}], ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970157926896*^9},
 CellTags->"ChargeConjugationMatrixInv",
 CellLabel->"Out[158]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Calc", "[", "%", "]"}]], "Input",
 CellTags->"ChargeConjugationMatrixInv",
 CellLabel->"In[159]:="],

Cell[BoxData[
 FormBox[
  SubsuperscriptBox["\[Gamma]", 
   FormBox["5",
    TraditionalForm], "T"], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970158283781*^9},
 CellTags->"ChargeConjugationMatrixInv",
 CellLabel->"Out[159]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ChargeConjugationMatrix", ".", 
  "ChargeConjugationMatrixInv"}]], "Input",
 CellTags->"ChargeConjugationMatrixInv",
 CellLabel->"In[160]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"C", ".", 
   RowBox[{"(", 
    RowBox[{"-", "C"}], ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970158515489*^9},
 CellTags->"ChargeConjugationMatrixInv",
 CellLabel->"Out[160]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "Calc"}]], "Input",
 CellTags->"ChargeConjugationMatrixInv",
 CellLabel->"In[161]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   RowBox[{"C", ".", "C"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970158762826*^9},
 CellTags->"ChargeConjugationMatrixInv",
 CellLabel->"Out[161]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ChargeConjugationMatrixInv", "^", "2"}]], "Input",
 CellTags->"ChargeConjugationMatrixInv",
 CellLabel->"In[162]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", "1"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970158953204*^9},
 CellTags->"ChargeConjugationMatrixInv",
 CellLabel->"Out[162]="]
}, Open  ]]
}, Open  ]],

Cell["CheckContext", "ObjectName",
 CellTags->"CheckContext"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"CheckContext"],

Cell["\<\
CheckContext[string] yields True if the packaged associated with string is \
already loaded, and False otherwise.\
\>", "Text",
 CellTags->"CheckContext"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["MakeContext",
  BaseStyle->"AddOnsLink",
  ButtonData:>"MakeContext",
  ButtonNote->"MakeContext"],
 ", ",
 ButtonBox["$FeynCalcStuff",
  BaseStyle->"AddOnsLink",
  ButtonData:>"$FeynCalcStuff",
  ButtonNote->"$FeynCalcStuff"],
 "."
}], "Text",
 CellTags->"CheckContext"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"CheckContext"],

Cell["\<\
If a FeynCalc symbol has not been used, the corresponding subpackage has not \
been loaded:\
\>", "Text",
 CellTags->"CheckContext"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CheckContext", "[", "\"\<CompleteSquare\>\"", "]"}]], "Input",
 CellTags->"CheckContext",
 CellLabel->"In[163]:="],

Cell[BoxData[
 FormBox["False", TraditionalForm]], "Output",
 CellChangeTimes->{3.5009701591751957`*^9},
 CellTags->"CheckContext",
 CellLabel->"Out[163]="]
}, Open  ]],

Cell["Using it makes FeynCalc load the subpackage:", "Text",
 CellTags->"CheckContext"],

Cell[CellGroupData[{

Cell[BoxData["CompleteSquare"], "Input",
 CellTags->"CheckContext",
 CellLabel->"In[164]:="],

Cell[BoxData[
 FormBox["CompleteSquare", TraditionalForm]], "Output",
 CellChangeTimes->{3.500970159419361*^9},
 CellTags->"CheckContext",
 CellLabel->"Out[164]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CheckContext", "[", "\"\<CompleteSquare\>\"", "]"}]], "Input",
 CellTags->"CheckContext",
 CellLabel->"In[165]:="],

Cell[BoxData[
 FormBox["True", TraditionalForm]], "Output",
 CellChangeTimes->{3.500970159496594*^9},
 CellTags->"CheckContext",
 CellLabel->"Out[165]="]
}, Open  ]]
}, Open  ]],

Cell["CheckDB", "ObjectName",
 CellTags->"CheckDB"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 GeneratedCell->True,
 CellAutoOverwrite->True,
 CellTags->"CheckDB"],

Cell["\<\
CheckDB[exp, fil] saves [with Put] or retrieves [with Get] exp from a file \
fil. It checks if the setting of the option Directory is a valid directory \
name and if fil is a valid file name and does exist. If it does, Get[fil] is \
executed. If fil does not exist, exp gets evaluated and saved to  fil. Saving \
and evaluating can be further controlled with the options ForceSave and \
NoSave. If the option Check is set to False  the return value is what is \
evaluated [see above]. If Check is set to True the return value is True or \
False depending on whether the evaluation of exp agrees with what is loaded \
from fil or fil does not exist.  Default value of Check : False. NOTICE : If \
fil ends with \".Gen\" or \".Mod\", the   setting of Directory is ignored and \
fil is  saved in the \"CouplingVectors\"  subdirectory of \"Phi\". If fil \
ends with  \".Fac\", the setting of Directory is  ignored and fil is saved in \
the \"Factors\" subdirectory of \"Phi\". If fil is a file   name with full \
path, the setting of  Directory is also ignored.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "CheckDB", "]"}]], "Input",
 CellTags->"CheckDB",
 CellLabel->"In[166]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Directory", "\[RuleDelayed]", 
     RowBox[{"ToFileName", "[", 
      RowBox[{
       RowBox[{"{", "$FeynCalcDirectory", "}"}], ",", "\<\"fcdb\"\>"}], 
      "]"}]}], ",", 
    RowBox[{"ForceSave", "\[Rule]", "False"}], ",", 
    RowBox[{"NoSave", "\[Rule]", "False"}], ",", 
    RowBox[{"Check", "\[Rule]", "False"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.50097015980571*^9},
 CellTags->"CheckDB",
 CellLabel->"Out[166]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Attributes", "[", "CheckDB", "]"}]], "Input",
 CellTags->"CheckDB",
 CellLabel->"In[167]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", "HoldFirst", "}"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.50097016031078*^9},
 CellTags->"CheckDB",
 CellLabel->"Out[167]="]
}, Open  ]],

Cell["\<\
The first time the Table function is evaluated and the result saved into the \
test.s file.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CheckDB", "[", 
  RowBox[{
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"Print", "[", 
       RowBox[{"\"\<test \>\"", ",", "i"}], "]"}], ";", "i"}], ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", "2"}], "}"}]}], "]"}], ",", "\"\<test.s\>\""}], 
  "]"}]], "Input",
 CellLabel->"In[168]:="],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"test \"\>", "\[InvisibleSpace]", "1"}],
   SequenceForm["test ", 1],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.50097016046737*^9}],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"test \"\>", "\[InvisibleSpace]", "2"}],
   SequenceForm["test ", 2],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.5009701605341597`*^9}],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"1", ",", "2"}], "}"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009701605739613`*^9},
 CellLabel->"Out[168]="]
}, Open  ]],

Cell["\<\
Executing the same a second time will just load the result from test.s and \
not evaluate the Table function.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CheckDB", "[", 
  RowBox[{
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"WriteString", "[", 
       RowBox[{"\"\<stdout\>\"", ",", "\"\<test\>\""}], "]"}], ";", "i"}], 
     ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", "2"}], "}"}]}], "]"}], ",", "\"\<test.s\>\""}], 
  "]"}]], "Input",
 CellTags->"CheckDB",
 CellLabel->"In[169]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"1", ",", "2"}], "}"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970160660466*^9},
 CellTags->"CheckDB",
 CellLabel->"Out[169]="]
}, Open  ]],

Cell["This shows the actual saved value of test.s.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Import", "[", 
  RowBox[{
   RowBox[{"ToFileName", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"$FeynCalcDirectory", ",", "\"\<fcdb\>\""}], "}"}], ",", 
     "\"\<test.s\>\""}], "]"}], ",", "\"\<Text\>\""}], "]"}]], "Input",
 CellTags->"CheckDB",
 CellLabel->"In[170]:="],

Cell[BoxData[
 FormBox["\<\"{1, 2}\"\>", TraditionalForm]], "Output",
 CellChangeTimes->{3.500970160854603*^9},
 CellTags->"CheckDB",
 CellLabel->"Out[170]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"DeleteFile", "[", 
  RowBox[{"ToFileName", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"$FeynCalcDirectory", ",", "\"\<fcdb\>\""}], "}"}], ",", 
    "\"\<test.s\>\""}], "]"}], "]"}]], "Input",
 CellTags->"CheckDB",
 CellLabel->"In[171]:="]
}, Open  ]],

Cell["ChiralityProjector", "ObjectName",
 CellTags->"ChiralityProjector"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"ChiralityProjector"],

Cell[TextData[{
 "ChiralityProjector[+1] denotes ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"1", "/", "2"}], 
    RowBox[{"(", 
     RowBox[{"1", "+", 
      SuperscriptBox["\[Gamma]", "5"]}], ")"}]}], TraditionalForm]]],
 ". ChiralityProjector[-1] denotes ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"1", "/", "2"}], 
    RowBox[{"(", 
     RowBox[{"1", "+", 
      SuperscriptBox["\[Gamma]", "5"]}], ")"}]}], TraditionalForm]]],
 "."
}], "Text",
 CellTags->"ChiralityProjector"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["DiracGamma",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracGamma",
  ButtonNote->"DiracGamma"],
 ", ",
 ButtonBox["DiracMatrix",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracMatrix",
  ButtonNote->"DiracMatrix"],
 ", ",
 ButtonBox["FCI",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FCI",
  ButtonNote->"FCI"],
 "."
}], "Text",
 CellTags->"ChiralityProjector"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"ChiralityProjector"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"ChiralityProjector", "[", 
    RowBox[{"+", "1"}], "]"}], ",", 
   RowBox[{"ChiralityProjector", "[", 
    RowBox[{"-", "1"}], "]"}]}], "}"}]], "Input",
 CellTags->"ChiralityProjector",
 CellLabel->"In[172]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    SuperscriptBox["\[Gamma]", "6"], ",", 
    SuperscriptBox["\[Gamma]", "7"]}], "}"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.50097016174963*^9},
 CellTags->"ChiralityProjector",
 CellLabel->"Out[172]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FCI", "[", "%", "]"}]], "Input",
 CellTags->"ChiralityProjector",
 CellLabel->"In[173]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    SuperscriptBox["\[Gamma]", "6"], ",", 
    SuperscriptBox["\[Gamma]", "7"]}], "}"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970161842473*^9},
 CellTags->"ChiralityProjector",
 CellLabel->"Out[173]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracSimplify", "[", 
  RowBox[{"%", ",", 
   RowBox[{"DiracSubstitute67", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"ChiralityProjector",
 CellLabel->"In[174]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     FractionBox[
      SuperscriptBox["\[Gamma]", "5"], "2"], "+", 
     FractionBox["1", "2"]}], ",", 
    RowBox[{
     FractionBox["1", "2"], "-", 
     FractionBox[
      SuperscriptBox["\[Gamma]", "5"], "2"]}]}], "}"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970161928927*^9},
 CellTags->"ChiralityProjector",
 CellLabel->"Out[174]="]
}, Open  ]]
}, Open  ]],

Cell["Chisholm", "ObjectName",
 CellTags->"Chisholm"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Chisholm"],

Cell["\<\
Chisholm[x] substitutes products of three Dirac matrices or slashes by the \
Chisholm identity.\
\>", "Text",
 CellTags->"Chisholm"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["EpsChisholm",
  BaseStyle->"AddOnsLink",
  ButtonData:>"EpsChisholm",
  ButtonNote->"EpsChisholm"],
 ", ",
 ButtonBox["ChisholmSpinor",
  BaseStyle->"AddOnsLink",
  ButtonData:>"ChisholmSpinor",
  ButtonNote->"ChisholmSpinor"],
 "."
}], "Text",
 CellTags->"Chisholm"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Chisholm"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Chisholm", "[", 
  RowBox[{"GA", "[", 
   RowBox[{"\[Alpha]", ",", "\[Beta]", ",", "\[Mu]", ",", "\[Nu]"}], "]"}], 
  "]"}]], "Input",
 CellTags->"Chisholm",
 CellLabel->"In[175]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    RowBox[{"\[ImaginaryI]", " ", 
     RowBox[{
      SuperscriptBox["\[Gamma]", 
       FormBox[
        FormBox["$MU$2379",
         TraditionalForm],
        TraditionalForm]], ".", 
      SuperscriptBox["\[Gamma]", 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]], ".", 
      SuperscriptBox["\[Gamma]", "5"]}], " ", 
     SuperscriptBox["\[Epsilon]", 
      RowBox[{
       FormBox[
        FormBox["\[Alpha]",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["\[Beta]",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["$MU$2379",
         TraditionalForm],
        TraditionalForm]}]]}]}], "+", 
   RowBox[{
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Alpha]", "\[Beta]"}]]}], "-", 
   RowBox[{
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Beta]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Alpha]", "\[Mu]"}]]}], "+", 
   RowBox[{
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Alpha]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Beta]", "\[Mu]"}]]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970162512451*^9},
 CellTags->"Chisholm",
 CellLabel->"Out[175]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t1", "=", 
  RowBox[{"DiracMatrix", "[", 
   RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]"}], "]"}]}]], "Input",
 CellTags->"Chisholm",
 CellLabel->"In[176]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", "\[Mu]"], 
   SuperscriptBox["\[Gamma]", "\[Nu]"], 
   SuperscriptBox["\[Gamma]", "\[Rho]"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009701625563993`*^9},
 CellTags->"Chisholm",
 CellLabel->"Out[176]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t2", " ", "=", 
  RowBox[{"Chisholm", "[", "t1", "]"}]}]], "Input",
 CellTags->"Chisholm",
 CellLabel->"In[177]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"\[ImaginaryI]", " ", 
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["$MU$2414",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", "5"]}], " ", 
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Rho]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["$MU$2414",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox["\[Rho]",
       TraditionalForm],
      TraditionalForm]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Nu]"}]]}], "-", 
   RowBox[{
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Rho]"}]]}], "+", 
   RowBox[{
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Nu]", "\[Rho]"}]]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.50097016276114*^9},
 CellTags->"Chisholm",
 CellLabel->"Out[177]="]
}, Open  ]],

Cell["The $MU$ variables are unique indices.", "Text",
 CellTags->"Chisholm"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Calc", "[", 
  RowBox[{"t1", ".", "t1"}], "]"}]], "Input",
 CellTags->"Chisholm",
 CellLabel->"In[178]:="],

Cell[BoxData[
 FormBox["16", TraditionalForm]], "Output",
 CellChangeTimes->{3.500970162988903*^9},
 CellTags->"Chisholm",
 CellLabel->"Out[178]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t3", " ", "=", 
  RowBox[{"Chisholm", "[", "t1", "]"}]}]], "Input",
 CellTags->"Chisholm",
 CellLabel->"In[179]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"\[ImaginaryI]", " ", 
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["$MU$2446",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", "5"]}], " ", 
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Rho]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["$MU$2446",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox["\[Rho]",
       TraditionalForm],
      TraditionalForm]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Nu]"}]]}], "-", 
   RowBox[{
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Rho]"}]]}], "+", 
   RowBox[{
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Nu]", "\[Rho]"}]]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970163314596*^9},
 CellTags->"Chisholm",
 CellLabel->"Out[179]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Calc", "[", 
  RowBox[{"t2", ".", "t3"}], "]"}]], "Input",
 CellTags->"Chisholm",
 CellLabel->"In[180]:="],

Cell[BoxData[
 FormBox["16", TraditionalForm]], "Output",
 CellChangeTimes->{3.500970163668503*^9},
 CellTags->"Chisholm",
 CellLabel->"Out[180]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t4", "=", 
  RowBox[{"DiracSlash", "[", 
   RowBox[{"a", ",", "b", ",", "c"}], "]"}]}]], "Input",
 CellTags->"Chisholm",
 CellLabel->"In[181]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     FormBox["\<\"\[Gamma]\"\>",
      TraditionalForm], "\[CenterDot]", 
     FormBox["a",
      TraditionalForm]}], ")"}], ".", 
   RowBox[{"(", 
    RowBox[{
     FormBox["\<\"\[Gamma]\"\>",
      TraditionalForm], "\[CenterDot]", 
     FormBox["b",
      TraditionalForm]}], ")"}], ".", 
   RowBox[{"(", 
    RowBox[{
     FormBox["\<\"\[Gamma]\"\>",
      TraditionalForm], "\[CenterDot]", 
     FormBox["c",
      TraditionalForm]}], ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970163716227*^9},
 CellTags->"Chisholm",
 CellLabel->"Out[181]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Chisholm", "[", "t4", "]"}]], "Input",
 CellTags->"Chisholm",
 CellLabel->"In[182]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    RowBox[{
     FormBox["a",
      TraditionalForm], "\[CenterDot]", 
     FormBox["b",
      TraditionalForm]}], " ", 
    RowBox[{
     FormBox["\<\"\[Gamma]\"\>",
      TraditionalForm], "\[CenterDot]", 
     FormBox["c",
      TraditionalForm]}]}], "-", 
   RowBox[{
    RowBox[{
     FormBox["a",
      TraditionalForm], "\[CenterDot]", 
     FormBox["c",
      TraditionalForm]}], " ", 
    RowBox[{
     FormBox["\<\"\[Gamma]\"\>",
      TraditionalForm], "\[CenterDot]", 
     FormBox["b",
      TraditionalForm]}]}], "+", 
   RowBox[{
    RowBox[{
     FormBox["\<\"\[Gamma]\"\>",
      TraditionalForm], "\[CenterDot]", 
     FormBox["a",
      TraditionalForm]}], " ", 
    RowBox[{
     FormBox["b",
      TraditionalForm], "\[CenterDot]", 
     FormBox["c",
      TraditionalForm]}]}], "-", 
   RowBox[{"\[ImaginaryI]", " ", 
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["$MU$2485",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", "5"]}], " ", 
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["$MU$2485",
        TraditionalForm],
       TraditionalForm], 
      FormBox["a",
       TraditionalForm], 
      FormBox["b",
       TraditionalForm], 
      FormBox["c",
       TraditionalForm]}]]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970163921647*^9},
 CellTags->"Chisholm",
 CellLabel->"Out[182]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"a1", "=", 
  RowBox[{"GA", "[", 
   RowBox[{
   "\[Mu]", ",", "\[Nu]", ",", "\[Rho]", ",", "\[Sigma]", ",", "\[Tau]", ",", 
    "\[Kappa]"}], "]"}]}]], "Input",
 CellTags->"Chisholm",
 CellLabel->"In[183]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
   SuperscriptBox["\[Gamma]", "\[Nu]"], ".", 
   SuperscriptBox["\[Gamma]", "\[Rho]"], ".", 
   SuperscriptBox["\[Gamma]", "\[Sigma]"], ".", 
   SuperscriptBox["\[Gamma]", "\[Tau]"], ".", 
   SuperscriptBox["\[Gamma]", "\[Kappa]"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970164111683*^9},
 CellTags->"Chisholm",
 CellLabel->"Out[183]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"a2", " ", "=", 
  RowBox[{"Chisholm", "[", "a1", "]"}]}]], "Input",
 CellTags->"Chisholm",
 CellLabel->"In[184]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    RowBox[{"\[ImaginaryI]", " ", 
     RowBox[{
      SuperscriptBox["\[Gamma]", 
       FormBox[
        FormBox["\[Tau]",
         TraditionalForm],
        TraditionalForm]], ".", 
      SuperscriptBox["\[Gamma]", 
       FormBox[
        FormBox["\[Kappa]",
         TraditionalForm],
        TraditionalForm]], ".", 
      SuperscriptBox["\[Gamma]", "5"]}], " ", 
     SuperscriptBox["\[Epsilon]", 
      RowBox[{
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["\[Rho]",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["\[Sigma]",
         TraditionalForm],
        TraditionalForm]}]]}]}], "+", 
   RowBox[{"\[ImaginaryI]", " ", 
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Kappa]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", "5"]}], " ", 
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Rho]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Tau]",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Tau]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Kappa]",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Sigma]"}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Nu]", "\[Rho]"}]]}], "-", 
   RowBox[{
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Kappa]",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Tau]"}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Nu]", "\[Rho]"}]]}], "-", 
   RowBox[{
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Tau]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Kappa]",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Rho]"}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Nu]", "\[Sigma]"}]]}], "+", 
   RowBox[{
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Rho]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Kappa]",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Tau]"}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Nu]", "\[Sigma]"}]]}], "+", 
   RowBox[{
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Kappa]",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Rho]"}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Nu]", "\[Tau]"}]]}], "-", 
   RowBox[{
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Rho]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Kappa]",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Sigma]"}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Nu]", "\[Tau]"}]]}], "+", 
   RowBox[{
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Tau]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Kappa]",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Rho]", "\[Sigma]"}]]}], "-", 
   RowBox[{
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Kappa]",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Tau]"}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Rho]", "\[Sigma]"}]]}], "+", 
   RowBox[{
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Kappa]",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Nu]", "\[Tau]"}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Rho]", "\[Sigma]"}]]}], "-", 
   RowBox[{
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Kappa]",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Rho]", "\[Tau]"}]]}], "+", 
   RowBox[{
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Kappa]",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Sigma]"}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Rho]", "\[Tau]"}]]}], "-", 
   RowBox[{
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Kappa]",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Nu]", "\[Sigma]"}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Rho]", "\[Tau]"}]]}], "+", 
   RowBox[{
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Rho]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Kappa]",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Sigma]", "\[Tau]"}]]}], "-", 
   RowBox[{
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Kappa]",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Rho]"}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Sigma]", "\[Tau]"}]]}], "+", 
   RowBox[{
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Kappa]",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Nu]", "\[Rho]"}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Sigma]", "\[Tau]"}]]}], "-", 
   RowBox[{"\[ImaginaryI]", " ", 
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["$MU$2507",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Kappa]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", "5"]}], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Sigma]", "\[Tau]"}]], " ", 
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Rho]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["$MU$2507",
        TraditionalForm],
       TraditionalForm]}]]}], "-", 
   RowBox[{"\[ImaginaryI]", " ", 
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["$MU$2529",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Kappa]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", "5"]}], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["\[Rho]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Tau]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["$MU$2529",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   RowBox[{"\[ImaginaryI]", " ", 
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["$MU$2540",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Kappa]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", "5"]}], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Rho]"}]], " ", 
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Tau]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["$MU$2540",
        TraditionalForm],
       TraditionalForm]}]]}], "-", 
   RowBox[{"\[ImaginaryI]", " ", 
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["$MU$2551",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Kappa]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", "5"]}], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Nu]", "\[Rho]"}]], " ", 
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Tau]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["$MU$2551",
        TraditionalForm],
       TraditionalForm]}]]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970165330736*^9},
 CellTags->"Chisholm",
 CellLabel->"Out[184]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"a3", " ", "=", 
  RowBox[{"Chisholm", "[", "a1", "]"}]}]], "Input",
 CellTags->"Chisholm",
 CellLabel->"In[185]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    RowBox[{"\[ImaginaryI]", " ", 
     RowBox[{
      SuperscriptBox["\[Gamma]", 
       FormBox[
        FormBox["\[Tau]",
         TraditionalForm],
        TraditionalForm]], ".", 
      SuperscriptBox["\[Gamma]", 
       FormBox[
        FormBox["\[Kappa]",
         TraditionalForm],
        TraditionalForm]], ".", 
      SuperscriptBox["\[Gamma]", "5"]}], " ", 
     SuperscriptBox["\[Epsilon]", 
      RowBox[{
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["\[Rho]",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["\[Sigma]",
         TraditionalForm],
        TraditionalForm]}]]}]}], "+", 
   RowBox[{"\[ImaginaryI]", " ", 
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Kappa]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", "5"]}], " ", 
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Rho]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Tau]",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Tau]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Kappa]",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Sigma]"}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Nu]", "\[Rho]"}]]}], "-", 
   RowBox[{
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Kappa]",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Tau]"}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Nu]", "\[Rho]"}]]}], "-", 
   RowBox[{
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Tau]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Kappa]",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Rho]"}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Nu]", "\[Sigma]"}]]}], "+", 
   RowBox[{
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Rho]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Kappa]",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Tau]"}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Nu]", "\[Sigma]"}]]}], "+", 
   RowBox[{
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Kappa]",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Rho]"}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Nu]", "\[Tau]"}]]}], "-", 
   RowBox[{
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Rho]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Kappa]",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Sigma]"}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Nu]", "\[Tau]"}]]}], "+", 
   RowBox[{
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Tau]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Kappa]",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Rho]", "\[Sigma]"}]]}], "-", 
   RowBox[{
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Kappa]",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Tau]"}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Rho]", "\[Sigma]"}]]}], "+", 
   RowBox[{
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Kappa]",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Nu]", "\[Tau]"}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Rho]", "\[Sigma]"}]]}], "-", 
   RowBox[{
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Kappa]",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Rho]", "\[Tau]"}]]}], "+", 
   RowBox[{
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Kappa]",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Sigma]"}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Rho]", "\[Tau]"}]]}], "-", 
   RowBox[{
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Kappa]",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Nu]", "\[Sigma]"}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Rho]", "\[Tau]"}]]}], "+", 
   RowBox[{
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Rho]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Kappa]",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Sigma]", "\[Tau]"}]]}], "-", 
   RowBox[{
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Kappa]",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Rho]"}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Sigma]", "\[Tau]"}]]}], "+", 
   RowBox[{
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Kappa]",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Nu]", "\[Rho]"}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Sigma]", "\[Tau]"}]]}], "-", 
   RowBox[{"\[ImaginaryI]", " ", 
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["$MU$2635",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Kappa]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", "5"]}], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Sigma]", "\[Tau]"}]], " ", 
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Rho]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["$MU$2635",
        TraditionalForm],
       TraditionalForm]}]]}], "-", 
   RowBox[{"\[ImaginaryI]", " ", 
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["$MU$2657",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Kappa]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", "5"]}], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["\[Rho]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Tau]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["$MU$2657",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   RowBox[{"\[ImaginaryI]", " ", 
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["$MU$2668",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Kappa]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", "5"]}], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Rho]"}]], " ", 
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Tau]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["$MU$2668",
        TraditionalForm],
       TraditionalForm]}]]}], "-", 
   RowBox[{"\[ImaginaryI]", " ", 
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["$MU$2679",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Kappa]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", "5"]}], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Nu]", "\[Rho]"}]], " ", 
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Tau]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["$MU$2679",
        TraditionalForm],
       TraditionalForm]}]]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970166396491*^9},
 CellTags->"Chisholm",
 CellLabel->"Out[185]="]
}, Open  ]],

Cell["Check that both a1.a1 and a2.a3 give the same.", "Text",
 CellTags->"Chisholm"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Calc", "[", 
  RowBox[{"a1", ".", "a1"}], "]"}]], "Input",
 CellTags->"Chisholm",
 CellLabel->"In[186]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", "2048"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970166791564*^9},
 CellTags->"Chisholm",
 CellLabel->"Out[186]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Calc", "[", 
  RowBox[{"a2", ".", "a3"}], "]"}]], "Input",
 CellTags->"Chisholm",
 CellLabel->"In[187]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", "2048"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009701685727863`*^9},
 CellTags->"Chisholm",
 CellLabel->"Out[187]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"Clear", "[", 
  RowBox[{
  "t1", ",", "t2", ",", "t3", ",", "t4", ",", "a1", ",", "a2", ",", "a3"}], 
  "]"}]], "Input",
 CellTags->"Chisholm",
 CellLabel->"In[188]:="]
}, Open  ]],

Cell["ChisholmSpinor", "ObjectName",
 CellTags->"ChisholmSpinor"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"ChisholmSpinor"],

Cell["\<\
ChisholmSpinor[x] uses the Chisholm identity on a DiraGamma between spinors. \
As an optional second argument 1 or 2 may be given, indicating that \
ChisholmSpinor should only act on the first resp. second part of a product of \
spinor chains.\
\>", "Text",
 CellTags->"ChisholmSpinor"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["EpsChisholm",
  BaseStyle->"AddOnsLink",
  ButtonData:>"EpsChisholm",
  ButtonNote->"EpsChisholm"],
 ", ",
 ButtonBox["Chisholm",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Chisholm",
  ButtonNote->"Chisholm"],
 "."
}], "Text",
 CellTags->"ChisholmSpinor"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"ChisholmSpinor"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Spinor", "[", 
   RowBox[{"p1", ",", "m1"}], "]"}], ".", 
  RowBox[{"DiracGamma", "[", 
   RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], "]"}], ".", 
  RowBox[{"Spinor", "[", 
   RowBox[{"p2", ",", "m2"}], "]"}]}]], "Input",
 CellTags->"ChisholmSpinor",
 CellLabel->"In[189]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"\[CurlyPhi]", "(", 
    FormBox["p1",
     TraditionalForm], ",", "m1", ")"}], ".", 
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], ".", 
   RowBox[{"\[CurlyPhi]", "(", 
    FormBox["p2",
     TraditionalForm], ",", "m2", ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970168782687*^9},
 CellTags->"ChisholmSpinor",
 CellLabel->"Out[189]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "ChisholmSpinor"}]], "Input",
 CellTags->"ChisholmSpinor",
 CellLabel->"In[190]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{"\[ImaginaryI]", " ", 
     SuperscriptBox["\[Epsilon]", 
      RowBox[{
       FormBox[
        FormBox["alpha20",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm], 
       FormBox["p1",
        TraditionalForm], 
       FormBox["p2",
        TraditionalForm]}]], " ", 
     RowBox[{
      RowBox[{"\[CurlyPhi]", "(", 
       FormBox["p1",
        TraditionalForm], ",", "m1", ")"}], ".", 
      SuperscriptBox["\[Gamma]", 
       FormBox[
        FormBox["alpha20",
         TraditionalForm],
        TraditionalForm]], ".", 
      SuperscriptBox["\[Gamma]", "5"], ".", 
      RowBox[{"\[CurlyPhi]", "(", 
       FormBox["p2",
        TraditionalForm], ",", "m2", ")"}]}]}], 
    RowBox[{
     FormBox["p1",
      TraditionalForm], "\[CenterDot]", 
     FormBox["p2",
      TraditionalForm]}]], "-", 
   FractionBox[
    RowBox[{"m1", " ", "m2", " ", 
     RowBox[{
      RowBox[{"\[CurlyPhi]", "(", 
       FormBox["p1",
        TraditionalForm], ",", "m1", ")"}], ".", 
      SuperscriptBox["\[Gamma]", 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], ".", 
      RowBox[{"\[CurlyPhi]", "(", 
       FormBox["p2",
        TraditionalForm], ",", "m2", ")"}]}]}], 
    RowBox[{
     FormBox["p1",
      TraditionalForm], "\[CenterDot]", 
     FormBox["p2",
      TraditionalForm]}]], "+", 
   FractionBox[
    RowBox[{"m2", " ", 
     SuperscriptBox[
      FormBox["p1",
       TraditionalForm], 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], " ", 
     RowBox[{
      RowBox[{"\[CurlyPhi]", "(", 
       FormBox["p1",
        TraditionalForm], ",", "m1", ")"}], ".", 
      RowBox[{"\[CurlyPhi]", "(", 
       FormBox["p2",
        TraditionalForm], ",", "m2", ")"}]}]}], 
    RowBox[{
     FormBox["p1",
      TraditionalForm], "\[CenterDot]", 
     FormBox["p2",
      TraditionalForm]}]], "+", 
   FractionBox[
    RowBox[{"m1", " ", 
     SuperscriptBox[
      FormBox["p2",
       TraditionalForm], 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], " ", 
     RowBox[{
      RowBox[{"\[CurlyPhi]", "(", 
       FormBox["p1",
        TraditionalForm], ",", "m1", ")"}], ".", 
      RowBox[{"\[CurlyPhi]", "(", 
       FormBox["p2",
        TraditionalForm], ",", "m2", ")"}]}]}], 
    RowBox[{
     FormBox["p1",
      TraditionalForm], "\[CenterDot]", 
     FormBox["p2",
      TraditionalForm]}]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970169298902*^9},
 CellTags->"ChisholmSpinor",
 CellLabel->"Out[190]="]
}, Open  ]]
}, Open  ]],

Cell["ClearScalarProducts", "ObjectName",
 CellTags->"ClearScalarProducts"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"ClearScalarProducts"],

Cell["\<\
ClearScalarProducts removes all user-performed specific settings for \
ScalarProduct's.\
\>", "Text",
 CellTags->"ClearScalarProducts"],

Cell[TextData[{
 "See also: ",
 ButtonBox["ScalarProduct",
  BaseStyle->"AddOnsLink",
  ButtonData:>"ScalarProduct",
  ButtonNote->"ScalarProduct"],
 ", ",
 ButtonBox["Pair",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Pair",
  ButtonNote->"Pair"],
 ", ",
 ButtonBox["SP",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SP",
  ButtonNote->"SP"],
 ", ",
 ButtonBox["SPD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SPD",
  ButtonNote->"SPD"],
 "."
}], "Text",
 CellTags->"ClearScalarProducts"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"ClearScalarProducts"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"ScalarProduct", "[", 
   RowBox[{"p", ",", "p"}], "]"}], "=", 
  RowBox[{"m", "^", "2"}]}]], "Input",
 CellTags->"ClearScalarProducts",
 CellLabel->"In[191]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["m", "2"], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009701693960342`*^9},
 CellTags->"ClearScalarProducts",
 CellLabel->"Out[191]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Pair", "[", 
  RowBox[{
   RowBox[{"Momentum", "[", "p", "]"}], ",", 
   RowBox[{"Momentum", "[", "p", "]"}]}], "]"}]], "Input",
 CellTags->"ClearScalarProducts",
 CellLabel->"In[192]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["m", "2"], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009701695388947`*^9},
 CellTags->"ClearScalarProducts",
 CellLabel->"Out[192]="]
}, Open  ]],

Cell[BoxData["ClearScalarProducts"], "Input",
 CellTags->"ClearScalarProducts",
 CellLabel->"In[193]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Pair", "[", 
  RowBox[{
   RowBox[{"Momentum", "[", "p", "]"}], ",", 
   RowBox[{"Momentum", "[", "p", "]"}]}], "]"}]], "Input",
 CellTags->"ClearScalarProducts",
 CellLabel->"In[194]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox[
   FormBox["p",
    TraditionalForm], "2"], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009701699895067`*^9},
 CellTags->"ClearScalarProducts",
 CellLabel->"Out[194]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ScalarProduct", "[", 
  RowBox[{"p", ",", "p"}], "]"}]], "Input",
 CellTags->"ClearScalarProducts",
 CellLabel->"In[195]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox[
   FormBox["p",
    TraditionalForm], "2"], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970170023016*^9},
 CellTags->"ClearScalarProducts",
 CellLabel->"Out[195]="]
}, Open  ]]
}, Open  ]],

Cell["Collecting", "ObjectName",
 CellTags->"Collecting"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Collecting"],

Cell["\<\
Collecting is an option of ScalarProductCancel, Series2, TID and related \
functions. Setting it to True will trigger some kind of collecting of the \
result.\
\>", "Text",
 CellTags->"Collecting"],

Cell[TextData[{
 "See also:  ",
 ButtonBox["ScalarProductCancel",
  BaseStyle->"AddOnsLink",
  ButtonData:>"ScalarProductCancel",
  ButtonNote->"ScalarProductCancel"],
 ", ",
 ButtonBox["Series2",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Series2",
  ButtonNote->"Series2"],
 ", ",
 ButtonBox["TID",
  BaseStyle->"AddOnsLink",
  ButtonData:>"TID",
  ButtonNote->"TID"],
 "."
}], "Text",
 CellTags->"Collecting"]
}, Open  ]],

Cell["Collect2", "ObjectName",
 CellTags->"Collect2"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Collect2"],

Cell["\<\
Collect2[expr, x] collects together terms which are not free of any \
occurrence of x. Collect2[expr, {x1, x2, ...}] (or also Collect2[expr, x1, \
x2, ...]) collects together terms which are not free of any occurrence of x1, \
x2, .... The coefficients are put over a common denominator. If expr is \
expanded before collecting depends on the option Factoring, which may be set \
to Factor, Factor2, or any other function, which is applied to the \
coefficients. If expr is already expanded with respect to x (x1,x2, ...), the \
option Expanding can be set to False.\
\>", "Text",
 CellTags->"Collect2"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "Collect2", "]"}]], "Input",
 CellTags->"Collect2",
 CellLabel->"In[196]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Denominator", "\[Rule]", "False"}], ",", 
    RowBox[{"Dot", "\[Rule]", "False"}], ",", 
    RowBox[{"Expanding", "\[Rule]", "True"}], ",", 
    RowBox[{"Factoring", "\[Rule]", "Factor"}], ",", 
    RowBox[{"IsolateNames", "\[Rule]", "False"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970170070327*^9},
 CellTags->"Collect2",
 CellLabel->"Out[196]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Isolate",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Isolate",
  ButtonNote->"Isolate"],
 "."
}], "Text",
 CellTags->"Collect2"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Collect2"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Collect2", "[", 
  RowBox[{
   RowBox[{"t1", "=", 
    RowBox[{"a", "+", 
     RowBox[{"r", " ", "a"}], "+", 
     RowBox[{
      SuperscriptBox["k", "2"], " ", 
      RowBox[{"f", "[", "a", "]"}]}], "-", 
     RowBox[{"k", " ", 
      RowBox[{"f", "[", "a", "]"}]}], "+", 
     FractionBox["x", "2"], "-", 
     FractionBox["y", "w"]}]}], ",", "a"}], "]"}]], "Input",
 CellTags->"Collect2",
 CellLabel->"In[197]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{"k", "-", "1"}], ")"}], " ", "k", " ", 
    RowBox[{"f", "(", "a", ")"}]}], "+", 
   RowBox[{"a", " ", 
    RowBox[{"(", 
     RowBox[{"r", "+", "1"}], ")"}]}], "+", 
   FractionBox[
    RowBox[{
     RowBox[{"w", " ", "x"}], "-", 
     RowBox[{"2", " ", "y"}]}], 
    RowBox[{"2", " ", "w"}]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970170160516*^9},
 CellTags->"Collect2",
 CellLabel->"Out[197]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Collect2", "[", 
  RowBox[{"t1", ",", "a", ",", 
   RowBox[{"Factoring", "\[Rule]", "False"}]}], "]"}]], "Input",
 CellTags->"Collect2",
 CellLabel->"In[198]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox["k", "2"], "-", "k"}], ")"}], " ", 
    RowBox[{"f", "(", "a", ")"}]}], "+", 
   RowBox[{"a", " ", 
    RowBox[{"(", 
     RowBox[{"r", "+", "1"}], ")"}]}], "-", 
   FractionBox["y", "w"], "+", 
   FractionBox["x", "2"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970170242107*^9},
 CellTags->"Collect2",
 CellLabel->"Out[198]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Collect2", "[", 
  RowBox[{"t1", ",", "a", ",", 
   RowBox[{"Factoring", "\[Rule]", "Factor"}]}], "]"}]], "Input",
 CellTags->"Collect2",
 CellLabel->"In[199]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{"k", "-", "1"}], ")"}], " ", "k", " ", 
    RowBox[{"f", "(", "a", ")"}]}], "+", 
   RowBox[{"a", " ", 
    RowBox[{"(", 
     RowBox[{"r", "+", "1"}], ")"}]}], "+", 
   FractionBox[
    RowBox[{
     RowBox[{"w", " ", "x"}], "-", 
     RowBox[{"2", " ", "y"}]}], 
    RowBox[{"2", " ", "w"}]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009701703677673`*^9},
 CellTags->"Collect2",
 CellLabel->"Out[199]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Collect2", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"2", " ", "a", " ", 
     RowBox[{"(", 
      RowBox[{"b", "-", "a"}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{"h", "-", "1"}], ")"}]}], "-", 
    RowBox[{
     SuperscriptBox["b", "2"], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"e", " ", "a"}], "-", "c"}], ")"}]}], "+", 
    SuperscriptBox["b", "2"]}], ",", 
   RowBox[{"{", 
    RowBox[{"a", ",", "b"}], "}"}]}], "]"}]], "Input",
 CellTags->"Collect2",
 CellLabel->"In[200]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    RowBox[{"2", " ", 
     SuperscriptBox["a", "2"], " ", 
     RowBox[{"(", 
      RowBox[{"h", "-", "1"}], ")"}]}]}], "-", 
   RowBox[{"a", " ", 
    SuperscriptBox["b", "2"], " ", "e"}], "+", 
   RowBox[{"2", " ", "a", " ", "b", " ", 
    RowBox[{"(", 
     RowBox[{"h", "-", "1"}], ")"}]}], "+", 
   RowBox[{
    SuperscriptBox["b", "2"], " ", 
    RowBox[{"(", 
     RowBox[{"c", "+", "1"}], ")"}]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009701705706253`*^9},
 CellTags->"Collect2",
 CellLabel->"Out[200]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Collect2", "[", 
  RowBox[{
   RowBox[{"Expand", "[", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"a", "-", "b", "-", "c", "-", "d"}], ")"}], "5"], "]"}], ",", 
   "a", ",", 
   RowBox[{"IsolateNames", "\[Rule]", "L"}]}], "]"}]], "Input",
 CellTags->"Collect2",
 CellLabel->"In[201]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["a", "5"], "-", 
   RowBox[{"5", " ", 
    SuperscriptBox["a", "4"], " ", 
    TagBox[
     RowBox[{"L", "(", "21", ")"}],
     HoldForm]}], "+", 
   RowBox[{"10", " ", 
    SuperscriptBox["a", "3"], " ", 
    SuperscriptBox[
     TagBox[
      RowBox[{"L", "(", "21", ")"}],
      HoldForm], "2"]}], "-", 
   RowBox[{"10", " ", 
    SuperscriptBox["a", "2"], " ", 
    SuperscriptBox[
     TagBox[
      RowBox[{"L", "(", "21", ")"}],
      HoldForm], "3"]}], "+", 
   RowBox[{"5", " ", "a", " ", 
    SuperscriptBox[
     TagBox[
      RowBox[{"L", "(", "21", ")"}],
      HoldForm], "4"]}], "-", 
   SuperscriptBox[
    TagBox[
     RowBox[{"L", "(", "21", ")"}],
     HoldForm], "5"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970170736516*^9},
 CellTags->"Collect2",
 CellLabel->"Out[201]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ReleaseHold", "[", "%", "]"}]], "Input",
 CellTags->"Collect2",
 CellLabel->"In[202]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["a", "5"], "-", 
   RowBox[{"5", " ", 
    SuperscriptBox["a", "4"], " ", 
    RowBox[{"(", 
     RowBox[{"b", "+", "c", "+", "d"}], ")"}]}], "+", 
   RowBox[{"10", " ", 
    SuperscriptBox["a", "3"], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"b", "+", "c", "+", "d"}], ")"}], "2"]}], "-", 
   RowBox[{"10", " ", 
    SuperscriptBox["a", "2"], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"b", "+", "c", "+", "d"}], ")"}], "3"]}], "+", 
   RowBox[{"5", " ", "a", " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"b", "+", "c", "+", "d"}], ")"}], "4"]}], "-", 
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"b", "+", "c", "+", "d"}], ")"}], "5"]}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970170993279*^9},
 CellTags->"Collect2",
 CellLabel->"Out[202]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"Clear", "[", 
  RowBox[{"t1", ",", "L"}], "]"}]], "Input",
 CellTags->"Collect2",
 CellLabel->"In[203]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Collect2", "[", 
  RowBox[{
   RowBox[{"Expand", "[", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"a", "-", "b", "-", "c"}], ")"}], "3"], "]"}], ",", "a", ",", 
   RowBox[{"Factoring", "\[Rule]", "fun"}]}], "]"}]], "Input",
 CellTags->"Collect2",
 CellLabel->"In[204]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SuperscriptBox["a", "3"], " ", 
    RowBox[{"fun", "(", "1", ")"}]}], "+", 
   RowBox[{
    SuperscriptBox["a", "2"], " ", 
    RowBox[{"fun", "(", 
     RowBox[{
      RowBox[{"-", 
       RowBox[{"3", " ", "b"}]}], "-", 
      RowBox[{"3", " ", "c"}]}], ")"}]}], "+", 
   RowBox[{"a", " ", 
    RowBox[{"fun", "(", 
     RowBox[{
      RowBox[{"3", " ", 
       SuperscriptBox["b", "2"]}], "+", 
      RowBox[{"6", " ", "b", " ", "c"}], "+", 
      RowBox[{"3", " ", 
       SuperscriptBox["c", "2"]}]}], ")"}]}], "+", 
   RowBox[{"fun", "(", 
    RowBox[{
     RowBox[{"-", 
      SuperscriptBox["b", "3"]}], "-", 
     RowBox[{"3", " ", 
      SuperscriptBox["b", "2"], " ", "c"}], "-", 
     RowBox[{"3", " ", "b", " ", 
      SuperscriptBox["c", "2"]}], "-", 
     SuperscriptBox["c", "3"]}], ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009701711391363`*^9},
 CellTags->"Collect2",
 CellLabel->"Out[204]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", " ", "/.", " ", 
  RowBox[{"fun", "\[Rule]", "FactorTerms"}]}]], "Input",
 CellTags->"Collect2",
 CellLabel->"In[205]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["a", "3"], "-", 
   RowBox[{"3", " ", 
    SuperscriptBox["a", "2"], " ", 
    RowBox[{"(", 
     RowBox[{"b", "+", "c"}], ")"}]}], "+", 
   RowBox[{"3", " ", "a", " ", 
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox["b", "2"], "+", 
      RowBox[{"2", " ", "b", " ", "c"}], "+", 
      SuperscriptBox["c", "2"]}], ")"}]}], "-", 
   SuperscriptBox["b", "3"], "-", 
   RowBox[{"3", " ", 
    SuperscriptBox["b", "2"], " ", "c"}], "-", 
   RowBox[{"3", " ", "b", " ", 
    SuperscriptBox["c", "2"]}], "-", 
   SuperscriptBox["c", "3"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009701713460627`*^9},
 CellTags->"Collect2",
 CellLabel->"Out[205]="]
}, Open  ]]
}, Open  ]],

Cell["Collect3", "ObjectName",
 CellTags->"Collect3"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Collect3"],

Cell[TextData[{
 "Collect3[expr, {x, y, ...}] collects terms involving the same powers of \
monomials ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    StyleBox["x",
     FontSlant->"Plain"], 
    SubscriptBox[
     StyleBox["n",
      FontSlant->"Plain"], "1"]], TraditionalForm]]],
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    StyleBox["y",
     FontSlant->"Plain"], 
    SubscriptBox[
     StyleBox["n",
      FontSlant->"Plain"], "2"]], TraditionalForm]]],
 " ... An option Factor \[Rule] True/False can be  given, which factors the \
coefficients. The option Head (default Plus) determines the applied function \
to the list of monomials  mulitplied by their coefficients."
}], "Text",
 CellTags->"Collect3"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "Collect3", "]"}]], "Input",
 CellTags->"Collect3",
 CellLabel->"In[206]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Factoring", "\[Rule]", "False"}], ",", 
    RowBox[{"Head", "\[Rule]", "Plus"}]}], "}"}], TraditionalForm]], "Output",\

 CellChangeTimes->{3.500970171489023*^9},
 CellTags->"Collect3",
 CellLabel->"Out[206]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Collect2",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Collect2",
  ButtonNote->"Collect2"],
 ", ",
 ButtonBox["Isolate",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Isolate",
  ButtonNote->"Isolate"],
 "."
}], "Text",
 CellTags->"Collect3"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Collect3"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Collect3", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"2", " ", "a", " ", 
     RowBox[{"(", 
      RowBox[{"b", "-", "a"}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{"h", "-", "1"}], ")"}]}], "-", 
    RowBox[{
     SuperscriptBox["b", "2"], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"e", " ", "a"}], "-", "c"}], ")"}]}], "+", 
    SuperscriptBox["b", "2"]}], ",", 
   RowBox[{"{", 
    RowBox[{"a", ",", "b"}], "}"}]}], "]"}]], "Input",
 CellTags->"Collect3",
 CellLabel->"In[207]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   StyleBox[
    RowBox[{"MonomialList", "::", "optx"}], "MessageName"], 
   RowBox[{
   ":", " "}], "\<\"Unknown option \
\[NoBreak]\\!\\(TraditionalForm\\`CoefficientDomain\\)\[NoBreak] in \
\[NoBreak]\\!\\n\\(TraditionalForm\\`\\(MonomialList[\\(\\(\\(\\(b\\^2 - \
\\(\\(b\\^2\\\\ \\(\\((\\(\\(-c\\)\\) + \\(\\(a\\\\ e\\)\\))\\)\\)\\)\\) + \
\\(\\(2\\\\ a\\\\ \\(\\((\\(\\(-a\\)\\) + b)\\)\\)\\\\ \\(\\((\\(\\(-1\\)\\) \
+ h)\\)\\)\\)\\)\\)\\), \\(\\({}\\)\\), \\n        \\(\\(CoefficientDomain \
\[Rule] RationalFunctions\\)\\)\\)\\)]\\)\\)\\n\[NoBreak]. \
\\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", \
ButtonFrame->None, ButtonData:>\\\"paclet:ref/message/General/optx\\\", \
ButtonNote -> \\\"MonomialList::optx\\\"]\\)\"\>"}], 
  TraditionalForm]], "Message", "MSG",
 CellChangeTimes->{3.500970171774116*^9},
 CellTags->"Collect3"],

Cell[BoxData[
 FormBox[
  RowBox[{
   StyleBox[
    RowBox[{"MonomialList", "::", "optx"}], "MessageName"], 
   RowBox[{
   ":", " "}], "\<\"Unknown option \
\[NoBreak]\\!\\(TraditionalForm\\`CoefficientDomain\\)\[NoBreak] in \
\[NoBreak]\\!\\n\\(TraditionalForm\\`\\(MonomialList[\\(\\(\\(\\(b\\^2 - \
\\(\\(b\\^2\\\\ \\(\\((\\(\\(-c\\)\\) + \\(\\(a\\\\ e\\)\\))\\)\\)\\)\\) + \
\\(\\(2\\\\ a\\\\ \\(\\((\\(\\(-a\\)\\) + b)\\)\\)\\\\ \\(\\((\\(\\(-1\\)\\) \
+ h)\\)\\)\\)\\)\\)\\), \\(\\({}\\)\\), \\n        \\(\\(CoefficientDomain \
\[Rule] RationalFunctions\\)\\)\\)\\)]\\)\\)\\n\[NoBreak]. \
\\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", \
ButtonFrame->None, ButtonData:>\\\"paclet:ref/message/General/optx\\\", \
ButtonNote -> \\\"MonomialList::optx\\\"]\\)\"\>"}], 
  TraditionalForm]], "Message", "MSG",
 CellChangeTimes->{3.500970172267859*^9},
 CellTags->"Collect3"],

Cell[BoxData[
 FormBox[
  RowBox[{"{", "}"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970172311913*^9},
 CellTags->"Collect3",
 CellLabel->"Out[207]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Collect3", "[", 
  RowBox[{
   RowBox[{"Expand", "[", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"a", "-", "b", "-", "c", "-", "d"}], ")"}], "5"], "]"}], ",", 
   RowBox[{"{", "a", "}"}]}], "]"}]], "Input",
 CellTags->"Collect3",
 CellLabel->"In[208]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   StyleBox[
    RowBox[{"MonomialList", "::", "optx"}], "MessageName"], 
   RowBox[{
   ":", " "}], "\<\"Unknown option \
\[NoBreak]\\!\\(TraditionalForm\\`CoefficientDomain\\)\[NoBreak] in \
\[NoBreak]\\!\\(TraditionalForm\\`\\(MonomialList[\\(\\(\[LeftSkeleton] 1 \
\[RightSkeleton]\\)\\)]\\)\\)\[NoBreak]. \\!\\(\\*ButtonBox[\\\"\
\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", ButtonFrame->None, \
ButtonData:>\\\"paclet:ref/message/General/optx\\\", ButtonNote -> \
\\\"MonomialList::optx\\\"]\\)\"\>"}], TraditionalForm]], "Message", "MSG",
 CellChangeTimes->{3.500970173334149*^9},
 CellTags->"Collect3"],

Cell[BoxData[
 FormBox[
  RowBox[{"{", "}"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970173362101*^9},
 CellTags->"Collect3",
 CellLabel->"Out[208]="]
}, Open  ]]
}, Open  ]],

Cell["Combine", "ObjectName",
 CellTags->"Combine"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Combine"],

Cell["\<\
Combine[expr]puts terms in a sum over a common denominator,and cancels \
factors in the result. Combine is similar to Together, but accepts the option \
Expanding and works usually better than Together for polynomials involving \
rationals with sums in the denominator.\
\>", "Text",
 CellTags->"Combine"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "Combine", "]"}]], "Input",
 CellTags->"Combine",
 CellLabel->"In[209]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"Expanding", "\[Rule]", "False"}], "}"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970173576016*^9},
 CellTags->"Combine",
 CellLabel->"Out[209]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Factor2",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Factor2",
  ButtonNote->"Factor2"],
 "."
}], "Text",
 CellTags->"Combine"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Combine"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Combine", "[", 
  RowBox[{
   FractionBox[
    RowBox[{
     RowBox[{"(", 
      RowBox[{"a", "-", "b"}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{"c", "-", "d"}], ")"}]}], "e"], "+", "g"}], "]"}]], "Input",
 CellTags->"Combine",
 CellLabel->"In[210]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{"a", "-", "b"}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{"c", "-", "d"}], ")"}]}], "+", 
    RowBox[{"e", " ", "g"}]}], "e"], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970173753849*^9},
 CellTags->"Combine",
 CellLabel->"Out[210]="]
}, Open  ]],

Cell["\<\
Here the result from Together where the numerator is automatically expanded.\
\>", "Text",
 CellTags->"Combine"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Together", "[", 
  RowBox[{
   FractionBox[
    RowBox[{
     RowBox[{"(", 
      RowBox[{"a", "-", "b"}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{"c", "-", "d"}], ")"}]}], "e"], "+", "g"}], "]"}]], "Input",
 CellTags->"Combine",
 CellLabel->"In[211]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{
    RowBox[{"a", " ", "c"}], "-", 
    RowBox[{"a", " ", "d"}], "-", 
    RowBox[{"b", " ", "c"}], "+", 
    RowBox[{"b", " ", "d"}], "+", 
    RowBox[{"e", " ", "g"}]}], "e"], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970173918795*^9},
 CellTags->"Combine",
 CellLabel->"Out[211]="]
}, Open  ]],

Cell["\<\
If the option Expanding is set to True, the result of Combine is the same as \
Together, but uses a slightly different algorithm.\
\>", "Text",
 CellTags->"Combine"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Combine", "[", 
  RowBox[{
   RowBox[{
    FractionBox[
     RowBox[{
      RowBox[{"(", 
       RowBox[{"a", "-", "b"}], ")"}], " ", 
      RowBox[{"(", 
       RowBox[{"c", "-", "d"}], ")"}]}], "e"], "+", "g"}], ",", 
   RowBox[{"Expanding", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"Combine",
 CellLabel->"In[212]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{
    RowBox[{"a", " ", "c"}], "-", 
    RowBox[{"a", " ", "d"}], "-", 
    RowBox[{"b", " ", "c"}], "+", 
    RowBox[{"b", " ", "d"}], "+", 
    RowBox[{"e", " ", "g"}]}], "e"], TraditionalForm]], "Output",
 CellChangeTimes->{3.50097017407938*^9},
 CellTags->"Combine",
 CellLabel->"Out[212]="]
}, Open  ]]
}, Open  ]],

Cell["CombineGraphs", "ObjectName",
 CellTags->"CombineGraphs"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"CombineGraphs"],

Cell["CombineGraphs is an option for OneLoopSum.", "Text",
 CellTags->"CombineGraphs"],

Cell[TextData[{
 "See also:  ",
 ButtonBox["OneLoopSum",
  BaseStyle->"AddOnsLink",
  ButtonData:>"OneLoopSum",
  ButtonNote->"OneLoopSum"],
 "."
}], "Text",
 CellTags->"CombineGraphs"]
}, Open  ]],

Cell["Commutator", "ObjectName",
 CellTags->"Commutator"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Commutator"],

Cell["\<\
Commutator[x, y] = c defines the commutator between the non-commuting objects \
x and y. \
\>", "Text",
 CellTags->"Commutator"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["AntiCommutator",
  BaseStyle->"AddOnsLink",
  ButtonData:>"AntiCommutator",
  ButtonNote->"AntiCommutator"],
 ", ",
 ButtonBox["CommutatorExplicit",
  BaseStyle->"AddOnsLink",
  ButtonData:>"CommutatorExplicit",
  ButtonNote->"CommutatorExplicit"],
 ", ",
 ButtonBox["DeclareNonCommutative",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DeclareNonCommutative",
  ButtonNote->"DeclareNonCommutative"],
 ", ",
 ButtonBox["DotSimplify",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DotSimplify",
  ButtonNote->"DotSimplify"],
 "."
}], "Text",
 CellTags->"Commutator"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Commutator"],

Cell[BoxData[
 RowBox[{"DeclareNonCommutative", "[", 
  RowBox[{"a", ",", "b", ",", "c", ",", "d"}], "]"}]], "Input",
 CellTags->"Commutator",
 CellLabel->"In[213]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Commutator", "[", 
  RowBox[{"a", ",", "b"}], "]"}]], "Input",
 CellTags->"Commutator",
 CellLabel->"In[214]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"[", "\[NoBreak]", "a", "\[NoBreak]", ",", "b", "\[NoBreak]", "]"}],
   TraditionalForm]], "Output",
 CellChangeTimes->{3.500970174395273*^9},
 CellTags->"Commutator",
 CellLabel->"Out[214]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CommutatorExplicit", "[", "%", "]"}]], "Input",
 CellTags->"Commutator",
 CellLabel->"In[215]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"a", ".", "b"}], "-", 
   RowBox[{"b", ".", "a"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970174485813*^9},
 CellTags->"Commutator",
 CellLabel->"Out[215]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"DotSimplify", "[", 
   RowBox[{"Commutator", "[", 
    RowBox[{
     RowBox[{"a", "+", "b"}], ",", 
     RowBox[{"c", "+", "d"}]}], "]"}], "]"}], " "}]], "Input",
 CellTags->"Commutator",
 CellLabel->"In[216]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"a", ".", "c"}], "-", 
   RowBox[{"c", ".", "a"}], "+", 
   RowBox[{"a", ".", "d"}], "-", 
   RowBox[{"d", ".", "a"}], "+", 
   RowBox[{"b", ".", "c"}], "-", 
   RowBox[{"c", ".", "b"}], "+", 
   RowBox[{"b", ".", "d"}], "-", 
   RowBox[{"d", ".", "b"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970174574624*^9},
 CellTags->"Commutator",
 CellLabel->"Out[216]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"UnDeclareNonCommutative", "[", 
  RowBox[{"a", ",", "b", ",", "c", ",", "d"}], "]"}]], "Input",
 CellTags->"Commutator",
 CellLabel->"In[217]:="],

Cell["Verify the Jacobi identity.", "Text",
 CellTags->"Commutator"],

Cell[BoxData[
 RowBox[{
  RowBox[{"\[Chi]", "=", "Commutator"}], ";", " ", 
  RowBox[{"DeclareNonCommutative", "[", 
   RowBox[{"x", ",", "y", ",", "z"}], "]"}], ";"}]], "Input",
 CellTags->"Commutator",
 CellLabel->"In[218]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"\[Chi]", "[", 
   RowBox[{"x", ",", 
    RowBox[{"\[Chi]", "[", 
     RowBox[{"y", ",", "z"}], "]"}]}], "]"}], "+", 
  RowBox[{"\[Chi]", "[", 
   RowBox[{"y", ",", 
    RowBox[{"\[Chi]", "[", 
     RowBox[{"z", ",", "x"}], "]"}]}], "]"}], "+", 
  RowBox[{"\[Chi]", "[", 
   RowBox[{"z", ",", 
    RowBox[{"\[Chi]", "[", 
     RowBox[{"x", ",", "y"}], "]"}]}], "]"}]}]], "Input",
 CellTags->"Commutator",
 CellLabel->"In[219]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"[", "\[NoBreak]", "x", "\[NoBreak]", ",", 
    FormBox[
     RowBox[{
     "[", "\[NoBreak]", "y", "\[NoBreak]", ",", "z", "\[NoBreak]", "]"}],
     TraditionalForm], "\[NoBreak]", "]"}], "+", 
   RowBox[{"[", "\[NoBreak]", "y", "\[NoBreak]", ",", 
    FormBox[
     RowBox[{
     "[", "\[NoBreak]", "z", "\[NoBreak]", ",", "x", "\[NoBreak]", "]"}],
     TraditionalForm], "\[NoBreak]", "]"}], "+", 
   RowBox[{"[", "\[NoBreak]", "z", "\[NoBreak]", ",", 
    FormBox[
     RowBox[{
     "[", "\[NoBreak]", "x", "\[NoBreak]", ",", "y", "\[NoBreak]", "]"}],
     TraditionalForm], "\[NoBreak]", "]"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009701750189867`*^9},
 CellTags->"Commutator",
 CellLabel->"Out[219]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DotSimplify", "[", "%", "]"}]], "Input",
 CellTags->"Commutator",
 CellLabel->"In[220]:="],

Cell[BoxData[
 FormBox["0", TraditionalForm]], "Output",
 CellChangeTimes->{3.500970175133623*^9},
 CellTags->"Commutator",
 CellLabel->"Out[220]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"Clear", "[", "\[Chi]", "]"}]], "Input",
 CellTags->"Commutator",
 CellLabel->"In[221]:="],

Cell[BoxData[
 RowBox[{"UnDeclareNonCommutative", "[", 
  RowBox[{"x", ",", "y", ",", "z"}], "]"}]], "Input",
 CellTags->"Commutator",
 CellLabel->"In[222]:="]
}, Open  ]],

Cell["CommutatorExplicit", "ObjectName",
 CellTags->"CommutatorExplicit"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"CommutatorExplicit"],

Cell["\<\
CommutatorExplicit[exp] substitutes any Commutator and AntiCommutator in exp \
by their definitions.\
\>", "Text",
 CellTags->"CommutatorExplicit"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Calc",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Calc",
  ButtonNote->"Calc"],
 ", ",
 ButtonBox["DotSimplify",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DotSimplify",
  ButtonNote->"DotSimplify"],
 "."
}], "Text",
 CellTags->"CommutatorExplicit"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"CommutatorExplicit"],

Cell[BoxData[
 RowBox[{"DeclareNonCommutative", "[", 
  RowBox[{"a", ",", "b", ",", "c", ",", "d"}], "]"}]], "Input",
 CellTags->"CommutatorExplicit",
 CellLabel->"In[223]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Commutator", "[", 
  RowBox[{"a", ",", "b"}], "]"}]], "Input",
 CellTags->"CommutatorExplicit",
 CellLabel->"In[224]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"[", "\[NoBreak]", "a", "\[NoBreak]", ",", "b", "\[NoBreak]", "]"}],
   TraditionalForm]], "Output",
 CellChangeTimes->{3.500970175427294*^9},
 CellTags->"CommutatorExplicit",
 CellLabel->"Out[224]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CommutatorExplicit", "[", "%", "]"}]], "Input",
 CellTags->"CommutatorExplicit",
 CellLabel->"In[225]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"a", ".", "b"}], "-", 
   RowBox[{"b", ".", "a"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970175826198*^9},
 CellTags->"CommutatorExplicit",
 CellLabel->"Out[225]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"AntiCommutator", "[", 
  RowBox[{
   RowBox[{"a", "-", "c"}], ",", 
   RowBox[{"b", "-", "d"}]}], "]"}]], "Input",
 CellTags->"CommutatorExplicit",
 CellLabel->"In[226]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   FormBox[
    RowBox[{"a", "-", "c"}],
    TraditionalForm], ",", "\[MediumSpace]", 
   FormBox[
    RowBox[{"b", "-", "d"}],
    TraditionalForm], "}"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970175965725*^9},
 CellTags->"CommutatorExplicit",
 CellLabel->"Out[226]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CommutatorExplicit", "[", "%", "]"}]], "Input",
 CellTags->"CommutatorExplicit",
 CellLabel->"In[227]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{"a", "-", "c"}], ")"}], ".", 
    RowBox[{"(", 
     RowBox[{"b", "-", "d"}], ")"}]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"b", "-", "d"}], ")"}], ".", 
    RowBox[{"(", 
     RowBox[{"a", "-", "c"}], ")"}]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970176065855*^9},
 CellTags->"CommutatorExplicit",
 CellLabel->"Out[227]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"CommutatorExplicit", "[", "%%", "]"}], "//", 
  "DotSimplify"}]], "Input",
 CellTags->"CommutatorExplicit",
 CellLabel->"In[228]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"a", ".", "b"}], "+", 
   RowBox[{"b", ".", "a"}], "-", 
   RowBox[{"a", ".", "d"}], "-", 
   RowBox[{"d", ".", "a"}], "-", 
   RowBox[{"b", ".", "c"}], "-", 
   RowBox[{"c", ".", "b"}], "+", 
   RowBox[{"c", ".", "d"}], "+", 
   RowBox[{"d", ".", "c"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970176213162*^9},
 CellTags->"CommutatorExplicit",
 CellLabel->"Out[228]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"UnDeclareNonCommutative", "[", 
  RowBox[{"a", ",", "b", ",", "c", ",", "d"}], "]"}]], "Input",
 CellTags->"CommutatorExplicit",
 CellLabel->"In[229]:="]
}, Open  ]],

Cell["CompleteSquare", "ObjectName",
 CellTags->"CompleteSquare"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"CompleteSquare"],

Cell[TextData[{
 "Completes the square of a second order polynomial in the momentum x. \
CompleteSquare[",
 "a",
 " ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    StyleBox["p",
     FontSlant->"Plain"], "2"], TraditionalForm]]],
 "+",
 "b",
 " ",
 "p",
 "+",
 "c",
 ", ",
 "p",
 "] \[Rule] -",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    StyleBox["b",
     FontSlant->"Plain"], "2"], TraditionalForm]]],
 "/(4 ",
 "a",
 ")+",
 "c",
 "+",
 "a",
 " (",
 "b",
 "/(2 ",
 "a",
 ")+",
 "x",
 ")^2. CompleteSquare[",
 "a",
 " ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    StyleBox["p",
     FontSlant->"Plain"], "2"], TraditionalForm]]],
 "+",
 "b",
 " ",
 "p",
 "+",
 "c",
 ", ",
 "p",
 ", ",
 "q",
 "] \[Rule] {-",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    StyleBox["b",
     FontSlant->"Plain"], "2"], TraditionalForm]]],
 "/(4 ",
 "a",
 ")+",
 "c",
 "+",
 "a",
 " ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    StyleBox["q",
     FontSlant->"Plain"], "2"], TraditionalForm]]],
 ", ",
 "q",
 "\[Rule]",
 "b",
 "/(2 ",
 "a",
 ")+",
 "p",
 "}."
}], "Text",
 CellTags->"CompleteSquare"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"CompleteSquare"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t1", "=", 
  RowBox[{
   RowBox[{"5", 
    RowBox[{"SP", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"2", "p"}], "+", 
       RowBox[{"3", "r"}]}], ",", 
      RowBox[{"p", "+", "r"}]}], "]"}]}], "//", "FCI"}]}]], "Input",
 CellLabel->"In[230]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"5", " ", 
   RowBox[{"(", 
    FormBox[
     RowBox[{"p", "+", "r"}],
     TraditionalForm], ")", "\[CenterDot]", "(", 
    FormBox[
     RowBox[{
      RowBox[{"2", " ", "p"}], "+", 
      RowBox[{"3", " ", "r"}]}],
     TraditionalForm], ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970176521689*^9},
 CellLabel->"Out[230]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t2", "=", 
  RowBox[{"CompleteSquare", "[", 
   RowBox[{"t1", ",", "p"}], "]"}]}]], "Input",
 CellTags->"CompleteSquare",
 CellLabel->"In[231]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"10", " ", 
    RowBox[{"(", 
     TagBox[
      FormBox[
       RowBox[{"p", "+", 
        FractionBox[
         RowBox[{"5", " ", "r"}], "4"]}],
       TraditionalForm],
      TraditionalForm,
      Editable->True], ")", ".", "(", 
     TagBox[
      FormBox[
       RowBox[{"p", "+", 
        FractionBox[
         RowBox[{"5", " ", "r"}], "4"]}],
       TraditionalForm],
      TraditionalForm,
      Editable->True], ")"}]}], "-", 
   FractionBox[
    RowBox[{"5", " ", 
     SuperscriptBox[
      FormBox["r",
       TraditionalForm], "2"]}], "8"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970176868725*^9},
 CellTags->"CompleteSquare",
 CellLabel->"Out[231]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"t1", "-", "t2"}], "//", "ScalarProductExpand"}], "//", 
  "Expand"}]], "Input",
 CellTags->"CompleteSquare",
 CellLabel->"In[232]:="],

Cell[BoxData[
 FormBox["0", TraditionalForm]], "Output",
 CellChangeTimes->{3.500970176955389*^9},
 CellTags->"CompleteSquare",
 CellLabel->"Out[232]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CompleteSquare", "[", 
  RowBox[{"t1", ",", "p", ",", "q"}], "]"}]], "Input",
 CellTags->"CompleteSquare",
 CellLabel->"In[233]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     RowBox[{"10", " ", 
      SuperscriptBox[
       FormBox["q",
        TraditionalForm], "2"]}], "-", 
     FractionBox[
      RowBox[{"5", " ", 
       SuperscriptBox[
        FormBox["r",
         TraditionalForm], "2"]}], "8"]}], ",", 
    RowBox[{"q", "\[Rule]", 
     RowBox[{"p", "+", 
      FractionBox[
       RowBox[{"5", " ", "r"}], "4"]}]}]}], "}"}], TraditionalForm]], "Output",\

 CellChangeTimes->{3.500970177050817*^9},
 CellTags->"CompleteSquare",
 CellLabel->"Out[233]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"Clear", "[", 
  RowBox[{"t1", ",", "t2"}], "]"}]], "Input",
 CellTags->"CompleteSquare",
 CellLabel->"In[234]:="]
}, Open  ]],

Cell["ComplexConjugate", "ObjectName",
 CellTags->"ComplexConjugate"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"ComplexConjugate"],

Cell["\<\
ComplexConjugate[expr] complex conjugates expr. It operates on fermion lines, \
i.e., products of Spinor[..] .DiracMatrix[..] . Spinor[..], and changes all \
occuring LorentzIndex[mu] into LorentzIndex[ComplexIndex[mu]]. For taking the \
spin sum (i.e. constructing the traces) use FermionSpinSum. WARNING: In expr \
should be NO explicit I's in denominators!\
\>", "Text",
 CellTags->"ComplexConjugate"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["ComplexIndex",
  BaseStyle->"AddOnsLink",
  ButtonData:>"ComplexIndex",
  ButtonNote->"ComplexIndex"],
 ", ",
 ButtonBox["FermionSpinSum",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FermionSpinSum",
  ButtonNote->"FermionSpinSum"],
 ", ",
 ButtonBox["LorentzIndex",
  BaseStyle->"AddOnsLink",
  ButtonData:>"LorentzIndex",
  ButtonNote->"LorentzIndex"],
 "."
}], "Text",
 CellTags->"ComplexConjugate"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"ComplexConjugate"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ComplexConjugate", "[", 
  RowBox[{"MetricTensor", "[", 
   RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}], "]"}]], "Input",
 CellTags->"ComplexConjugate",
 CellLabel->"In[235]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["g", 
   RowBox[{"\[Mu]", "\[Nu]"}]], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009701771445627`*^9},
 CellTags->"ComplexConjugate",
 CellLabel->"Out[235]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StandardForm", "[", "%", "]"}]], "Input",
 CellTags->"ComplexConjugate",
 CellLabel->"In[236]:="],

Cell[BoxData[
 RowBox[{"Pair", "[", 
  RowBox[{
   RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
   RowBox[{"LorentzIndex", "[", "\[Nu]", "]"}]}], "]"}]], "Output",
 CellChangeTimes->{3.500970177189403*^9},
 CellTags->"ComplexConjugate",
 CellLabel->"Out[236]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GA", "[", 
  RowBox[{"\[Mu]", ",", "\[Nu]", ",", "5"}], "]"}]], "Input",
 CellTags->"ComplexConjugate",
 CellLabel->"In[237]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
   SuperscriptBox["\[Gamma]", "\[Nu]"], ".", 
   SuperscriptBox["\[Gamma]", "5"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970177234088*^9},
 CellTags->"ComplexConjugate",
 CellLabel->"Out[237]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ComplexConjugate", "[", "%", "]"}]], "Input",
 CellTags->"ComplexConjugate",
 CellLabel->"In[238]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   RowBox[{
    SuperscriptBox["\[Gamma]", "5"], ".", 
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm]], ".", 
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970177385975*^9},
 CellTags->"ComplexConjugate",
 CellLabel->"Out[238]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNTrace", "[", 
  RowBox[{"SUNT", "[", 
   RowBox[{"a", ",", "b", ",", "c"}], "]"}], "]"}]], "Input",
 CellTags->"ComplexConjugate",
 CellLabel->"In[239]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"tr", "(", 
   FormBox[
    RowBox[{
     SubscriptBox["T", 
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm]], ".", 
     SubscriptBox["T", 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]], ".", 
     SubscriptBox["T", 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm]]}],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970177486096*^9},
 CellTags->"ComplexConjugate",
 CellLabel->"Out[239]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ComplexConjugate", "[", "%", "]"}]], "Input",
 CellTags->"ComplexConjugate",
 CellLabel->"In[240]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"tr", "(", 
   FormBox[
    RowBox[{
     SubscriptBox["T", 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm]], ".", 
     SubscriptBox["T", 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]], ".", 
     SubscriptBox["T", 
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm]]}],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009701776310797`*^9},
 CellTags->"ComplexConjugate",
 CellLabel->"Out[240]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ComplexConjugate", "[", 
  RowBox[{"SUNF", "[", 
   RowBox[{"a", ",", "b", ",", "c"}], "]"}], "]"}]], "Input",
 CellTags->"ComplexConjugate",
 CellLabel->"In[241]:="],

Cell[BoxData[
 FormBox[
  SubscriptBox["f", 
   RowBox[{
    FormBox[
     FormBox["a",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["b",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["c",
      TraditionalForm],
     TraditionalForm]}]], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009701780359287`*^9},
 CellTags->"ComplexConjugate",
 CellLabel->"Out[241]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StandardForm", "[", 
  RowBox[{"FCE", "[", 
   RowBox[{"ComplexConjugate", "[", 
    RowBox[{"MetricTensor", "[", 
     RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}], "]"}], "]"}], "]"}]], "Input",
 CellTags->"ComplexConjugate",
 CellLabel->"In[242]:="],

Cell[BoxData[
 RowBox[{"MT", "[", 
  RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}]], "Output",
 CellChangeTimes->{3.500970178147587*^9},
 CellTags->"ComplexConjugate",
 CellLabel->"Out[242]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"SpinorUBar", "[", 
   RowBox[{"k1", ",", "m"}], "]"}], ".", 
  RowBox[{"GA", "[", "\[Lambda]", "]"}], ".", 
  RowBox[{"SpinorU", "[", 
   RowBox[{"p1", ",", "m"}], "]"}]}]], "Input",
 CellTags->"ComplexConjugate",
 CellLabel->"In[243]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    FormBox[
     OverscriptBox["\<\"u\"\>", "_"],
     TraditionalForm], "(", "k1", ",", "m", ")"}], ".", 
   SuperscriptBox["\[Gamma]", "\[Lambda]"], ".", 
   RowBox[{"u", "(", "p1", ",", "m", ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970178223349*^9},
 CellTags->"ComplexConjugate",
 CellLabel->"Out[243]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ComplexConjugate", "[", "%", "]"}]], "Input",
 CellTags->"ComplexConjugate",
 CellLabel->"In[244]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"\[CurlyPhi]", "(", 
    FormBox["p1",
     TraditionalForm], ",", "m", ")"}], ".", 
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Lambda]",
      TraditionalForm],
     TraditionalForm]], ".", 
   RowBox[{"\[CurlyPhi]", "(", 
    FormBox["k1",
     TraditionalForm], ",", "m", ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970178328273*^9},
 CellTags->"ComplexConjugate",
 CellLabel->"Out[244]="]
}, Open  ]],

Cell["\<\
Notice that SpinorUBar and SpinorU are only input functions. Internally they \
are converted to Spinor objects.\
\>", "Text",
 CellTags->"ComplexConjugate"]
}, Open  ]],

Cell["Contract", "ObjectName",
 CellTags->"Contract"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Contract"],

Cell["\<\
Contract[expr] contracts pairs of Lorentz indices of metric tensors, \
four-vectors and (depending on the optionEpsContract) of Levi-Civita tensors \
in expr. For the contraction of Dirac matrices with each other use \
DiracSimplify. Contract[exp1, exp2] contracts (exp1*exp2), where exp1 and \
exp2 may be larger products of sums of metric tensors and 4-vectors. \
\>", "Text",
 CellTags->"Contract"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "Contract", "]"}]], "Input",
 CellTags->"Contract",
 CellLabel->"In[245]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Collecting", "\[Rule]", "True"}], ",", 
    RowBox[{"Contract3", "\[Rule]", "True"}], ",", 
    RowBox[{"EpsContract", "\[Rule]", "True"}], ",", 
    RowBox[{"Expanding", "\[Rule]", "True"}], ",", 
    RowBox[{"Factoring", "\[Rule]", "False"}], ",", 
    RowBox[{"FeynCalcInternal", "\[Rule]", "True"}], ",", 
    RowBox[{"MomentumCombine", "\[Rule]", "True"}], ",", 
    RowBox[{"Rename", "\[Rule]", "False"}], ",", 
    RowBox[{"Schouten", "\[Rule]", "0"}]}], "}"}], TraditionalForm]], "Output",\

 CellChangeTimes->{3.500970178406336*^9},
 CellTags->"Contract",
 CellLabel->"Out[245]="]
}, Open  ]],

Cell["\<\
The option setting Contract3 can be set to True resulting in a faster \
algorithm to contract products of tensors.\
\>", "Text"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Pair",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Pair",
  ButtonNote->"Pair"],
 ", ",
 ButtonBox["DiracSimplify",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracSimplify",
  ButtonNote->"DiracSimplify"],
 ", ",
 ButtonBox["MomentumCombine",
  BaseStyle->"AddOnsLink",
  ButtonData:>"MomentumCombine",
  ButtonNote->"MomentumCombine"],
 "."
}], "Text",
 CellTags->"Contract"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Contract"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"MetricTensor", "[", 
   RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}], " ", 
  RowBox[{"FourVector", "[", 
   RowBox[{"p", ",", "\[Mu]"}], "]"}]}]], "Input",
 CellTags->"Contract",
 CellLabel->"In[246]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["p", "\[Mu]"], " ", 
   SuperscriptBox["g", 
    RowBox[{"\[Mu]", "\[Nu]"}]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970178480826*^9},
 CellTags->"Contract",
 CellLabel->"Out[246]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Contract", "[", "%", "]"}]], "Input",
 CellTags->"Contract",
 CellLabel->"In[247]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox[
   FormBox["p",
    TraditionalForm], 
   FormBox[
    FormBox["\[Nu]",
     TraditionalForm],
    TraditionalForm]], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009701785773087`*^9},
 CellTags->"Contract",
 CellLabel->"Out[247]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"FourVector", "[", 
   RowBox[{"p", ",", "\[Mu]"}], "]"}], 
  RowBox[{"DiracMatrix", "[", "\[Mu]", "]"}]}]], "Input",
 CellTags->"Contract",
 CellLabel->"In[248]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], " ", 
   SubscriptBox["p", "\[Mu]"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970178656209*^9},
 CellTags->"Contract",
 CellLabel->"Out[248]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Contract", "[", "%", "]"}]], "Input",
 CellTags->"Contract",
 CellLabel->"In[249]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox["\<\"\[Gamma]\"\>",
    TraditionalForm], "\[CenterDot]", 
   FormBox["p",
    TraditionalForm]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009701788361273`*^9},
 CellTags->"Contract",
 CellLabel->"Out[249]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"MetricTensor", "[", 
  RowBox[{"\[Mu]", ",", "\[Mu]"}], "]"}]], "Input",
 CellTags->"Contract",
 CellLabel->"In[250]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["g", 
   RowBox[{"\[Mu]", "\[Mu]"}]], TraditionalForm]], "Output",
 CellChangeTimes->{3.50097017891253*^9},
 CellTags->"Contract",
 CellLabel->"Out[250]="]
}, Open  ]],

Cell["The default dimension for MetricTensor is 4.", "Text",
 CellTags->"Contract"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Contract", "[", "%", "]"}]], "Input",
 CellTags->"Contract",
 CellLabel->"In[251]:="],

Cell[BoxData[
 FormBox["4", TraditionalForm]], "Output",
 CellChangeTimes->{3.500970179175264*^9},
 CellTags->"Contract",
 CellLabel->"Out[251]="]
}, Open  ]],

Cell["\<\
A short way to enter D-dimensional metric tensors is given by MTD. The \".\" \
as multiplication operator is not necessary but just convenient for \
typesetting.\
\>", "Text",
 CellTags->"Contract"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"MTD", "[", 
   RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}], " ", ".", " ", 
  RowBox[{"MTD", "[", 
   RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}]}]], "Input",
 CellTags->"Contract",
 CellLabel->"In[252]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["g", 
    RowBox[{"\[Mu]", "\[Nu]"}]], ".", 
   SuperscriptBox["g", 
    RowBox[{"\[Mu]", "\[Nu]"}]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970179258914*^9},
 CellTags->"Contract",
 CellLabel->"Out[252]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Contract", "[", "%", "]"}]], "Input",
 CellTags->"Contract",
 CellLabel->"In[253]:="],

Cell[BoxData[
 FormBox["D", TraditionalForm]], "Output",
 CellChangeTimes->{3.500970179413035*^9},
 CellTags->"Contract",
 CellLabel->"Out[253]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"MTD", "[", 
   RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}], " ", ".", " ", 
  RowBox[{"MTD", "[", 
   RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}]}]], "Input",
 CellTags->"Contract",
 CellLabel->"In[254]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["g", 
    RowBox[{"\[Mu]", "\[Nu]"}]], ".", 
   SuperscriptBox["g", 
    RowBox[{"\[Mu]", "\[Nu]"}]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009701796334333`*^9},
 CellTags->"Contract",
 CellLabel->"Out[254]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"FourVector", "[", 
   RowBox[{"p", ",", "\[Mu]"}], "]"}], " ", 
  RowBox[{"FourVector", "[", 
   RowBox[{"q", ",", "\[Mu]"}], "]"}]}]], "Input",
 CellTags->"Contract",
 CellLabel->"In[255]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["p", "\[Mu]"], " ", 
   SubscriptBox["q", "\[Mu]"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009701797843933`*^9},
 CellTags->"Contract",
 CellLabel->"Out[255]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Contract", "[", "%", " ", "]"}]], "Input",
 CellTags->"Contract",
 CellLabel->"In[256]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox["p",
    TraditionalForm], "\[CenterDot]", 
   FormBox["q",
    TraditionalForm]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970179889586*^9},
 CellTags->"Contract",
 CellLabel->"Out[256]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"FourVector", "[", 
   RowBox[{
    RowBox[{"p", "-", "q"}], ",", "\[Mu]"}], "]"}], " ", 
  RowBox[{"FourVector", "[", 
   RowBox[{
    RowBox[{"a", "-", "b"}], ",", "\[Mu]"}], "]"}]}]], "Input",
 CellTags->"Contract",
 CellLabel->"In[257]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox[
    RowBox[{"(", 
     FormBox[
      TagBox[
       RowBox[{"a", "-", "b"}],
       HoldForm],
      TraditionalForm], ")"}], "\[Mu]"], " ", 
   SubscriptBox[
    RowBox[{"(", 
     FormBox[
      TagBox[
       RowBox[{"p", "-", "q"}],
       HoldForm],
      TraditionalForm], ")"}], "\[Mu]"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970180147246*^9},
 CellTags->"Contract",
 CellLabel->"Out[257]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Contract", "[", "%", "]"}]], "Input",
 CellTags->"Contract",
 CellLabel->"In[258]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    FormBox["a",
     TraditionalForm], "\[CenterDot]", 
    FormBox["p",
     TraditionalForm]}], "-", 
   RowBox[{
    FormBox["a",
     TraditionalForm], "\[CenterDot]", 
    FormBox["q",
     TraditionalForm]}], "-", 
   RowBox[{
    FormBox["b",
     TraditionalForm], "\[CenterDot]", 
    FormBox["p",
     TraditionalForm]}], "+", 
   RowBox[{
    FormBox["b",
     TraditionalForm], "\[CenterDot]", 
    FormBox["q",
     TraditionalForm]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970180322514*^9},
 CellTags->"Contract",
 CellLabel->"Out[258]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"LeviCivita", "[", 
   RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Alpha]", ",", "\[Sigma]"}], "]"}], 
  " ", 
  RowBox[{"FourVector", "[", 
   RowBox[{"p", ",", "\[Sigma]"}], "]"}]}]], "Input",
 CellTags->"Contract",
 CellLabel->"In[259]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["p", "\[Sigma]"], " ", 
   SuperscriptBox["\[Epsilon]", 
    RowBox[{
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["\[Alpha]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["\[Sigma]",
       TraditionalForm],
      TraditionalForm]}]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.50097018047313*^9},
 CellTags->"Contract",
 CellLabel->"Out[259]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Contract", "[", "%", "]"}]], "Input",
 CellTags->"Contract",
 CellLabel->"In[260]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["\[Epsilon]", 
   RowBox[{
    FormBox[
     FormBox["\[Alpha]",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["\[Nu]",
      TraditionalForm],
     TraditionalForm], 
    FormBox["p",
     TraditionalForm]}]], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970180556088*^9},
 CellTags->"Contract",
 CellLabel->"Out[260]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"LeviCivita", "[", 
   RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Alpha]", ",", "\[Beta]"}], "]"}], 
  " ", 
  RowBox[{"LeviCivita", "[", 
   RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Alpha]", ",", "\[Sigma]"}], "]"}], 
  " "}]], "Input",
 CellTags->"Contract",
 CellLabel->"In[261]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Epsilon]", 
    RowBox[{
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["\[Alpha]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["\[Beta]",
       TraditionalForm],
      TraditionalForm]}]], " ", 
   SuperscriptBox["\[Epsilon]", 
    RowBox[{
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["\[Alpha]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["\[Sigma]",
       TraditionalForm],
      TraditionalForm]}]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970180629966*^9},
 CellTags->"Contract",
 CellLabel->"Out[261]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Contract", "[", "%", "]"}]], "Input",
 CellTags->"Contract",
 CellLabel->"In[262]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", "6"}], " ", 
   SuperscriptBox["g", 
    RowBox[{"\[Beta]", "\[Sigma]"}]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970180704706*^9},
 CellTags->"Contract",
 CellLabel->"Out[262]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"SetOptions", "[", 
   RowBox[{"Eps", ",", 
    RowBox[{"Dimension", "\[Rule]", "D"}]}], "]"}], ";", 
  RowBox[{
   RowBox[{"LCD", "[", 
    RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Alpha]", ",", "\[Beta]"}], "]"}], 
   " ", 
   RowBox[{"LCD", "[", 
    RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Alpha]", ",", "\[Sigma]"}], 
    "]"}]}]}]], "Input",
 CellTags->"Contract",
 CellLabel->"In[263]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Epsilon]", 
    RowBox[{"\[Mu]", "\[Nu]", "\[Alpha]", "\[Beta]"}]], " ", 
   SuperscriptBox["\[Epsilon]", 
    RowBox[{"\[Mu]", "\[Nu]", "\[Alpha]", "\[Sigma]"}]]}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.5009701807891397`*^9},
 CellTags->"Contract",
 CellLabel->"Out[263]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Contract", "[", "%", "]"}], "//", "Factor2"}]], "Input",
 CellTags->"Contract",
 CellLabel->"In[264]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"(", 
    RowBox[{"1", "-", "D"}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{"2", "-", "D"}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{"3", "-", "D"}], ")"}], " ", 
   SuperscriptBox["g", 
    RowBox[{"\[Beta]", "\[Sigma]"}]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.50097018092416*^9},
 CellTags->"Contract",
 CellLabel->"Out[264]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"SetOptions", "[", 
   RowBox[{"Eps", ",", 
    RowBox[{"Dimension", "\[Rule]", "4"}]}], "]"}], ";"}]], "Input",
 CellTags->"Contract",
 CellLabel->"In[265]:="]
}, Open  ]],

Cell["Contract1", "ObjectName",
 CellTags->"Contract1"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Contract1"],

Cell["\<\
For covariant calculations it is not necessary to distinguish between co- and \
contravariant indices (except for the Levi-Civita tensor).
Therefore normally no distinction is made between upper and lower indices.
Howere, if needed, one can use Upper and Lower and then a distuction is made:
Contract1[exp] contracts Upper and Lower indices. Neither Upper and Upper nor \
Lower and Lower indices are contracted.\
\>", "Text",
 CellTags->"Contract1"],

Cell[TextData[{
 "See also:  ",
 ButtonBox["LorentzIndex",
  BaseStyle->"AddOnsLink",
  ButtonData:>"LorentzIndex",
  ButtonNote->"LorentzIndex"],
 ", ",
 ButtonBox["Lower",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Lower",
  ButtonNote->"Lower"],
 ", ",
 ButtonBox["Upper",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Upper",
  ButtonNote->"Upper"],
 "."
}], "Text",
 CellTags->"Contract1"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Contract1"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"FV", "[", 
     RowBox[{"p", ",", 
      RowBox[{"Lower", "[", "\[Mu]", "]"}]}], "]"}], 
    RowBox[{"FV", "[", 
     RowBox[{"q", ",", 
      RowBox[{"Upper", "[", "\[Mu]", "]"}]}], "]"}]}], "//", "FCI"}], "//", 
  "Contract1"}]], "Input",
 CellTags->"Contract1",
 CellLabel->"In[266]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox["p",
    TraditionalForm], "\[CenterDot]", 
   FormBox["q",
    TraditionalForm]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009701810206738`*^9},
 CellTags->"Contract1",
 CellLabel->"Out[266]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"FV", "[", 
     RowBox[{"p", ",", 
      RowBox[{"Upper", "[", "\[Mu]", "]"}]}], "]"}], 
    RowBox[{"FV", "[", 
     RowBox[{"q", ",", 
      RowBox[{"Upper", "[", "\[Mu]", "]"}]}], "]"}]}], "//", "FCI"}], "//", 
  "Contract1"}]], "Input",
 CellTags->"Contract1",
 CellLabel->"In[267]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox[
    FormBox["p",
     TraditionalForm], 
    FormBox[
     FormBox[
      RowBox[{"Upper", "(", "\[Mu]", ")"}],
      TraditionalForm],
     TraditionalForm]], " ", 
   SuperscriptBox[
    FormBox["q",
     TraditionalForm], 
    FormBox[
     FormBox[
      RowBox[{"Upper", "(", "\[Mu]", ")"}],
      TraditionalForm],
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970181109994*^9},
 CellTags->"Contract1",
 CellLabel->"Out[267]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"MT", "[", 
    RowBox[{
     RowBox[{"Lower", "[", "\[Mu]", "]"}], ",", 
     RowBox[{"Upper", "[", "\[Mu]", "]"}]}], "]"}], "//", "FCI"}], "//", 
  "Contract1"}]], "Input",
 CellTags->"Contract1",
 CellLabel->"In[268]:="],

Cell[BoxData[
 FormBox["4", TraditionalForm]], "Output",
 CellChangeTimes->{3.500970181238673*^9},
 CellTags->"Contract1",
 CellLabel->"Out[268]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"MT", "[", 
    RowBox[{
     RowBox[{"Upper", "[", "\[Mu]", "]"}], ",", 
     RowBox[{"Upper", "[", "\[Mu]", "]"}]}], "]"}], "//", "FCI"}], "//", 
  "Contract1"}]], "Input",
 CellTags->"Contract1",
 CellLabel->"In[269]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["g", 
   RowBox[{
    FormBox[
     RowBox[{"Upper", "(", "\[Mu]", ")"}],
     TraditionalForm], 
    FormBox[
     RowBox[{"Upper", "(", "\[Mu]", ")"}],
     TraditionalForm]}]], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009701813385*^9},
 CellTags->"Contract1",
 CellLabel->"Out[269]="]
}, Open  ]]
}, Open  ]],

Cell["Convolute", "ObjectName",
 CellTags->"Convolute"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Convolute"],

Cell[TextData[{
 "Convolute[f, g, x] convolutes ",
 StyleBox["f",
  FontSlant->"Italic"],
 "(",
 StyleBox["x",
  FontSlant->"Italic"],
 ") and ",
 StyleBox["g",
  FontSlant->"Italic"],
 "(",
 StyleBox["x",
  FontSlant->"Italic"],
 "), i.e., ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubsuperscriptBox["\[Integral]", "0", "1"], 
    RowBox[{
     SubscriptBox[
      StyleBox[
       RowBox[{
        StyleBox["d",
         FontSlant->"Plain"], 
        StyleBox["x",
         FontSlant->"Italic"]}]], "1"], " ", 
     RowBox[{
      SubsuperscriptBox["\[Integral]", "0", "1"], 
      RowBox[{
       SubscriptBox[
        StyleBox[
         RowBox[{
          StyleBox["d",
           FontSlant->"Plain"], "x"}]], "2"], " ", 
       RowBox[{"\[Delta]", "(", 
        RowBox[{"x", " ", "-", " ", 
         RowBox[{
          SubscriptBox["x", "1"], " ", 
          SubscriptBox["x", "2"]}]}], ")"}], " ", 
       RowBox[{"f", "(", 
        SubscriptBox["x", "1"], ")"}], " ", 
       RowBox[{
        RowBox[{"g", "(", 
         SubscriptBox["x", "2"], ")"}], " ", "."}]}]}]}]}], TraditionalForm]]],
 " Convolute[f, g] is equivalent to Convolute[f, g, x]. Convolute[exp, {x1, \
x2}] assumes that exp is polynomial in x1 and x2. Convolute uses \
table-look-up and does not do any integral calculations, only linear \
algebra."
}], "Text",
 CellTags->"Convolute"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "Convolute", "]"}]], "Input",
 CellTags->"Convolute",
 CellLabel->"In[270]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Bracket", "\[Rule]", 
     RowBox[{"{", 
      TagBox["\[CurlyEpsilon]",
       TraditionalForm], "}"}]}], ",", 
    RowBox[{"FinalSubstitutions", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{"PlusDistribution", "\[Rule]", "Identity"}], "}"}]}]}], "}"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970181417213*^9},
 CellTags->"Convolute",
 CellLabel->"Out[270]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["PlusDistribution",
  BaseStyle->"AddOnsLink",
  ButtonData:>"PlusDistribution",
  ButtonNote->"PlusDistribution"],
 ", ",
 ButtonBox["ConvoluteTable",
  BaseStyle->"AddOnsLink",
  ButtonData:>"ConvoluteTable",
  ButtonNote->"ConvoluteTable"],
 "."
}], "Text",
 CellTags->"Convolute"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Convolute"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Convolute", "[", 
  RowBox[{"1", ",", "1"}], "]"}]], "Input",
 CellTags->"Convolute",
 CellLabel->"In[271]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   RowBox[{"log", "(", "x", ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.50097018178308*^9},
 CellTags->"Convolute",
 CellLabel->"Out[271]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Convolute", "[", 
  RowBox[{"x", ",", "x"}], "]"}]], "Input",
 CellTags->"Convolute",
 CellLabel->"In[272]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", "x"}], " ", 
   RowBox[{"log", "(", "x", ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.50097018189077*^9},
 CellTags->"Convolute",
 CellLabel->"Out[272]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Convolute", "[", 
  RowBox[{"1", ",", "x"}], "]"}]], "Input",
 CellTags->"Convolute",
 CellLabel->"In[273]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"1", "-", "x"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009701822862988`*^9},
 CellTags->"Convolute",
 CellLabel->"Out[273]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Convolute", "[", 
  RowBox[{"1", ",", 
   FractionBox["1", 
    RowBox[{"1", "-", "x"}]]}], "]"}]], "Input",
 CellTags->"Convolute",
 CellLabel->"In[274]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"log", "(", 
    RowBox[{"1", "-", "x"}], ")"}], "-", 
   RowBox[{"log", "(", "x", ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970182373221*^9},
 CellTags->"Convolute",
 CellLabel->"Out[274]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Convolute", "[", 
  RowBox[{"1", ",", 
   RowBox[{"PlusDistribution", "[", 
    FractionBox["1", 
     RowBox[{"1", "-", "x"}]], "]"}]}], "]"}]], "Input",
 CellTags->"Convolute",
 CellLabel->"In[275]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"log", "(", 
    RowBox[{"1", "-", "x"}], ")"}], "-", 
   RowBox[{"log", "(", "x", ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009701824701777`*^9},
 CellTags->"Convolute",
 CellLabel->"Out[275]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Convolute", "[", 
  RowBox[{
   FractionBox["1", 
    RowBox[{"1", "-", "x"}]], ",", "x"}], "]"}]], "Input",
 CellTags->"Convolute",
 CellLabel->"In[276]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", "x"}], "+", 
   RowBox[{"x", " ", 
    RowBox[{"log", "(", 
     RowBox[{"1", "-", "x"}], ")"}]}], "-", 
   RowBox[{"x", " ", 
    RowBox[{"log", "(", "x", ")"}]}], "+", "1"}], TraditionalForm]], "Output",\

 CellChangeTimes->{3.500970182845018*^9},
 CellTags->"Convolute",
 CellLabel->"Out[276]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Convolute", "[", 
  RowBox[{
   FractionBox["1", 
    RowBox[{"1", "-", "x"}]], ",", 
   FractionBox["1", 
    RowBox[{"1", "-", "x"}]]}], "]"}]], "Input",
 CellTags->"Convolute",
 CellLabel->"In[277]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    RowBox[{
     RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
     RowBox[{"\[Delta]", "(", 
      FormBox[
       RowBox[{"1", "-", "x"}],
       TraditionalForm], ")"}]}]}], "+", 
   FractionBox[
    RowBox[{"2", " ", 
     RowBox[{"log", "(", 
      RowBox[{"1", "-", "x"}], ")"}]}], 
    RowBox[{"1", "-", "x"}]], "-", 
   FractionBox[
    RowBox[{"log", "(", "x", ")"}], 
    RowBox[{"1", "-", "x"}]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970183026821*^9},
 CellTags->"Convolute",
 CellLabel->"Out[277]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Convolute", "[", 
  RowBox[{"1", ",", 
   RowBox[{"Log", "[", 
    RowBox[{"1", "-", "x"}], "]"}]}], "]"}]], "Input",
 CellTags->"Convolute",
 CellLabel->"In[278]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    TemplateBox[{"2",RowBox[{"1", "-", "x"}]},
     "PolyLog"]}], "-", 
   RowBox[{
    RowBox[{"log", "(", 
     RowBox[{"1", "-", "x"}], ")"}], " ", 
    RowBox[{"log", "(", "x", ")"}]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970183248098*^9},
 CellTags->"Convolute",
 CellLabel->"Out[278]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Convolute", "[", 
  RowBox[{"1", ",", 
   RowBox[{"x", " ", 
    RowBox[{"Log", "[", 
     RowBox[{"1", "-", "x"}], "]"}]}]}], "]"}]], "Input",
 CellTags->"Convolute",
 CellLabel->"In[279]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"x", "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"1", "-", "x"}], ")"}], " ", 
    RowBox[{"log", "(", 
     RowBox[{"1", "-", "x"}], ")"}]}], "-", "1"}], TraditionalForm]], "Output",\

 CellChangeTimes->{3.5009701833434057`*^9},
 CellTags->"Convolute",
 CellLabel->"Out[279]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Convolute", "[", 
  RowBox[{
   FractionBox["1", 
    RowBox[{"1", "-", "x"}]], ",", 
   RowBox[{"Log", "[", 
    RowBox[{"1", "-", "x"}], "]"}]}], "]"}]], "Input",
 CellTags->"Convolute",
 CellLabel->"In[280]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    RowBox[{"\[Zeta]", "(", "2", ")"}]}], "+", 
   RowBox[{
    SuperscriptBox["log", "2"], "(", 
    RowBox[{"1", "-", "x"}], ")"}], "-", 
   RowBox[{
    RowBox[{"log", "(", "x", ")"}], " ", 
    RowBox[{"log", "(", 
     RowBox[{"1", "-", "x"}], ")"}]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009701834510612`*^9},
 CellTags->"Convolute",
 CellLabel->"Out[280]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Convolute", "[", 
  RowBox[{
   FractionBox["1", 
    RowBox[{"1", "-", "x"}]], ",", 
   RowBox[{"x", " ", 
    RowBox[{"Log", "[", 
     RowBox[{"1", "-", "x"}], "]"}]}]}], "]"}]], "Input",
 CellTags->"Convolute",
 CellLabel->"In[281]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    RowBox[{
     RowBox[{"\[Zeta]", "(", "2", ")"}], " ", "x"}]}], "+", "x", "+", 
   RowBox[{"x", " ", 
    RowBox[{
     SuperscriptBox["log", "2"], "(", 
     RowBox[{"1", "-", "x"}], ")"}]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"1", "-", "x"}], ")"}], " ", 
    RowBox[{"log", "(", 
     RowBox[{"1", "-", "x"}], ")"}]}], "-", 
   RowBox[{"x", " ", 
    RowBox[{"log", "(", "x", ")"}], " ", 
    RowBox[{"log", "(", 
     RowBox[{"1", "-", "x"}], ")"}]}], "-", "1"}], TraditionalForm]], "Output",\

 CellChangeTimes->{3.5009701835504637`*^9},
 CellTags->"Convolute",
 CellLabel->"Out[281]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Convolute", "[", 
  RowBox[{
   FractionBox[
    RowBox[{"Log", "[", 
     RowBox[{"1", "-", "x"}], "]"}], 
    RowBox[{"1", "-", "x"}]], ",", "x"}], "]"}]], "Input",
 CellTags->"Convolute",
 CellLabel->"In[282]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    RowBox[{"-", "x"}], " ", 
    TemplateBox[{"2",RowBox[{"1", "-", "x"}]},
     "PolyLog"]}], "+", 
   RowBox[{
    FractionBox["1", "2"], " ", "x", " ", 
    RowBox[{
     SuperscriptBox["log", "2"], "(", 
     RowBox[{"1", "-", "x"}], ")"}]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"1", "-", "x"}], ")"}], " ", 
    RowBox[{"log", "(", 
     RowBox[{"1", "-", "x"}], ")"}]}], "-", 
   RowBox[{"x", " ", 
    RowBox[{"log", "(", "x", ")"}], " ", 
    RowBox[{"log", "(", 
     RowBox[{"1", "-", "x"}], ")"}]}], "+", 
   RowBox[{"x", " ", 
    RowBox[{"log", "(", "x", ")"}]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970183646153*^9},
 CellTags->"Convolute",
 CellLabel->"Out[282]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Convolute", "[", 
  RowBox[{"1", ",", 
   RowBox[{"x", " ", 
    RowBox[{"Log", "[", "x", "]"}]}]}], "]"}]], "Input",
 CellTags->"Convolute",
 CellLabel->"In[283]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"x", "+", 
   RowBox[{"x", " ", 
    RowBox[{"(", 
     RowBox[{"-", 
      RowBox[{"log", "(", "x", ")"}]}], ")"}]}], "-", "1"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.5009701837556133`*^9},
 CellTags->"Convolute",
 CellLabel->"Out[283]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Convolute", "[", 
  RowBox[{
   RowBox[{"Log", "[", 
    RowBox[{"1", "-", "x"}], "]"}], ",", "x"}], "]"}]], "Input",
 CellTags->"Convolute",
 CellLabel->"In[284]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{"1", "-", "x"}], ")"}], " ", 
    RowBox[{"log", "(", 
     RowBox[{"1", "-", "x"}], ")"}]}], "+", 
   RowBox[{"x", " ", 
    RowBox[{"log", "(", "x", ")"}]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.50097018406489*^9},
 CellTags->"Convolute",
 CellLabel->"Out[284]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Convolute", "[", 
  RowBox[{
   FractionBox["1", 
    RowBox[{"1", "-", "x"}]], ",", 
   FractionBox[
    RowBox[{"Log", "[", "x", "]"}], 
    RowBox[{"1", "-", "x"}]]}], "]"}]], "Input",
 CellTags->"Convolute",
 CellLabel->"In[285]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{
     RowBox[{"log", "(", 
      RowBox[{"1", "-", "x"}], ")"}], " ", 
     RowBox[{"log", "(", "x", ")"}]}], 
    RowBox[{"1", "-", "x"}]], "-", 
   FractionBox[
    RowBox[{
     SuperscriptBox["log", "2"], "(", "x", ")"}], 
    RowBox[{"2", " ", 
     RowBox[{"(", 
      RowBox[{"1", "-", "x"}], ")"}]}]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.50097018489505*^9},
 CellTags->"Convolute",
 CellLabel->"Out[285]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Convolute", "[", 
  RowBox[{"1", ",", 
   RowBox[{"Log", "[", "x", "]"}]}], "]"}]], "Input",
 CellTags->"Convolute",
 CellLabel->"In[286]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    FractionBox["1", "2"]}], " ", 
   RowBox[{
    SuperscriptBox["log", "2"], "(", "x", ")"}]}], TraditionalForm]], "Output",\

 CellChangeTimes->{3.500970184995907*^9},
 CellTags->"Convolute",
 CellLabel->"Out[286]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Convolute", "[", 
  RowBox[{"x", ",", " ", 
   RowBox[{"x", " ", 
    RowBox[{"Log", "[", "x", "]"}]}]}], "]"}]], "Input",
 CellTags->"Convolute",
 CellLabel->"In[287]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    FractionBox["1", "2"]}], " ", "x", " ", 
   RowBox[{
    SuperscriptBox["log", "2"], "(", "x", ")"}]}], TraditionalForm]], "Output",\

 CellChangeTimes->{3.500970185156683*^9},
 CellTags->"Convolute",
 CellLabel->"Out[287]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Convolute", "[", 
  RowBox[{
   FractionBox["1", 
    RowBox[{"1", "-", "x"}]], ",", 
   RowBox[{"Log", "[", "x", "]"}]}], "]"}]], "Input",
 CellTags->"Convolute",
 CellLabel->"In[288]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TemplateBox[{"2",RowBox[{"1", "-", "x"}]},
    "PolyLog"], "-", 
   RowBox[{
    FractionBox["1", "2"], " ", 
    RowBox[{
     SuperscriptBox["log", "2"], "(", "x", ")"}]}], "+", 
   RowBox[{
    RowBox[{"log", "(", 
     RowBox[{"1", "-", "x"}], ")"}], " ", 
    RowBox[{"log", "(", "x", ")"}]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009701852774343`*^9},
 CellTags->"Convolute",
 CellLabel->"Out[288]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Convolute", "[", 
  RowBox[{"1", ",", 
   FractionBox[
    RowBox[{"Log", "[", "x", "]"}], 
    RowBox[{"1", "-", "x"}]]}], "]"}]], "Input",
 CellTags->"Convolute",
 CellLabel->"In[289]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    TemplateBox[{"2",RowBox[{"1", "-", "x"}]},
     "PolyLog"]}], "-", 
   RowBox[{
    FractionBox["1", "2"], " ", 
    RowBox[{
     SuperscriptBox["log", "2"], "(", "x", ")"}]}]}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970185379174*^9},
 CellTags->"Convolute",
 CellLabel->"Out[289]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Convolute", "[", 
  RowBox[{
   FractionBox["1", 
    RowBox[{"1", "-", "x"}]], ",", 
   RowBox[{"x", " ", 
    RowBox[{"Log", "[", "x", "]"}]}]}], "]"}]], "Input",
 CellTags->"Convolute",
 CellLabel->"In[290]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"x", " ", 
    TemplateBox[{"2",RowBox[{"1", "-", "x"}]},
     "PolyLog"]}], "+", "x", "-", 
   RowBox[{
    FractionBox["1", "2"], " ", "x", " ", 
    RowBox[{
     SuperscriptBox["log", "2"], "(", "x", ")"}]}], "-", 
   RowBox[{"x", " ", 
    RowBox[{"log", "(", "x", ")"}]}], "+", 
   RowBox[{"x", " ", 
    RowBox[{"log", "(", 
     RowBox[{"1", "-", "x"}], ")"}], " ", 
    RowBox[{"log", "(", "x", ")"}]}], "-", "1"}], TraditionalForm]], "Output",\

 CellChangeTimes->{3.5009701855073767`*^9},
 CellTags->"Convolute",
 CellLabel->"Out[290]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Convolute", "[", 
  RowBox[{
   FractionBox[
    RowBox[{"Log", "[", "x", "]"}], 
    RowBox[{"1", "-", "x"}]], ",", "x"}], "]"}]], "Input",
 CellTags->"Convolute",
 CellLabel->"In[291]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    RowBox[{"-", "x"}], " ", 
    TemplateBox[{"2",RowBox[{"1", "-", "x"}]},
     "PolyLog"]}], "-", "x", "-", 
   RowBox[{
    FractionBox["1", "2"], " ", "x", " ", 
    RowBox[{
     SuperscriptBox["log", "2"], "(", "x", ")"}]}], "+", 
   RowBox[{"log", "(", "x", ")"}], "+", "1"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009701857453547`*^9},
 CellTags->"Convolute",
 CellLabel->"Out[291]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Convolute", "[", 
  RowBox[{"1", ",", 
   RowBox[{"x", " ", 
    RowBox[{"Log", "[", "x", "]"}]}]}], "]"}]], "Input",
 CellTags->"Convolute",
 CellLabel->"In[292]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"x", "+", 
   RowBox[{"x", " ", 
    RowBox[{"(", 
     RowBox[{"-", 
      RowBox[{"log", "(", "x", ")"}]}], ")"}]}], "-", "1"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970185923376*^9},
 CellTags->"Convolute",
 CellLabel->"Out[292]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Convolute", "[", 
  RowBox[{
   RowBox[{"Log", "[", "x", "]"}], ",", "x"}], "]"}]], "Input",
 CellTags->"Convolute",
 CellLabel->"In[293]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", "x"}], "+", 
   RowBox[{"log", "(", "x", ")"}], "+", "1"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970186088998*^9},
 CellTags->"Convolute",
 CellLabel->"Out[293]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Convolute", "[", 
  RowBox[{
   FractionBox["1", 
    RowBox[{"1", "-", "x"}]], ",", 
   FractionBox[
    RowBox[{"Log", "[", 
     RowBox[{"1", "-", "x"}], "]"}], 
    RowBox[{"1", "-", "x"}]]}], "]"}]], "Input",
 CellTags->"Convolute",
 CellLabel->"In[294]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    FractionBox[
     RowBox[{"\[Zeta]", "(", "2", ")"}], 
     RowBox[{"1", "-", "x"}]]}], "+", 
   RowBox[{
    TemplateBox[{"3"},
     "Zeta"], " ", 
    RowBox[{"\[Delta]", "(", 
     FormBox[
      RowBox[{"1", "-", "x"}],
      TraditionalForm], ")"}]}], "+", 
   FractionBox[
    RowBox[{"3", " ", 
     RowBox[{
      SuperscriptBox["log", "2"], "(", 
      RowBox[{"1", "-", "x"}], ")"}]}], 
    RowBox[{"2", " ", 
     RowBox[{"(", 
      RowBox[{"1", "-", "x"}], ")"}]}]], "-", 
   FractionBox[
    RowBox[{
     RowBox[{"log", "(", "x", ")"}], " ", 
     RowBox[{"log", "(", 
      RowBox[{"1", "-", "x"}], ")"}]}], 
    RowBox[{"1", "-", "x"}]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970186367654*^9},
 CellTags->"Convolute",
 CellLabel->"Out[294]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Convolute", "[", 
  RowBox[{
   FractionBox[
    RowBox[{"Log", "[", 
     RowBox[{"1", "-", "x"}], "]"}], 
    RowBox[{"1", "-", "x"}]], ",", 
   RowBox[{"Log", "[", 
    RowBox[{"1", "-", "x"}], "]"}]}], "]"}]], "Input",
 CellTags->"Convolute",
 CellLabel->"In[295]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TemplateBox[{"3",RowBox[{"1", "-", "x"}]},
    "PolyLog"], "+", 
   RowBox[{"2", " ", 
    TemplateBox[{"3","x"},
     "PolyLog"]}], "+", 
   RowBox[{
    TemplateBox[{"2",RowBox[{"1", "-", "x"}]},
     "PolyLog"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"log", "(", "x", ")"}], "-", 
      RowBox[{"log", "(", 
       RowBox[{"1", "-", "x"}], ")"}]}], ")"}]}], "-", 
   RowBox[{
    RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"log", "(", 
       RowBox[{"1", "-", "x"}], ")"}], "+", 
      RowBox[{"log", "(", "x", ")"}]}], ")"}]}], "+", 
   RowBox[{
    FractionBox["1", "2"], " ", 
    RowBox[{
     SuperscriptBox["log", "3"], "(", 
     RowBox[{"1", "-", "x"}], ")"}]}], "-", 
   RowBox[{
    RowBox[{"log", "(", "x", ")"}], " ", 
    RowBox[{
     SuperscriptBox["log", "2"], "(", 
     RowBox[{"1", "-", "x"}], ")"}]}], "+", 
   RowBox[{
    RowBox[{
     SuperscriptBox["log", "2"], "(", "x", ")"}], " ", 
    RowBox[{"log", "(", 
     RowBox[{"1", "-", "x"}], ")"}]}], "-", 
   TemplateBox[{"3"},
    "Zeta"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970186582007*^9},
 CellTags->"Convolute",
 CellLabel->"Out[295]="]
}, Open  ]]
}, Open  ]],

Cell["ConvoluteTable ", "ObjectName",
 CellTags->"ConvoluteTable"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"ConvoluteTable"],

Cell["\<\
ConvoluteTable[f, g, x] yields the convolution of f and g. ConvoluteTable is \
called by Convolute.\
\>", "Text",
 CellTags->"ConvoluteTable"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "Convolute", "]"}]], "Input",
 CellTags->"ConvoluteTable",
 CellLabel->"In[296]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Bracket", "\[Rule]", 
     RowBox[{"{", 
      TagBox["\[CurlyEpsilon]",
       TraditionalForm], "}"}]}], ",", 
    RowBox[{"FinalSubstitutions", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{"PlusDistribution", "\[Rule]", "Identity"}], "}"}]}]}], "}"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.5009701868082047`*^9},
 CellTags->"ConvoluteTable",
 CellLabel->"Out[296]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["PlusDistribution",
  BaseStyle->"AddOnsLink",
  ButtonData:>"PlusDistribution",
  ButtonNote->"PlusDistribution"],
 ", ",
 ButtonBox["Convolute",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Convolute",
  ButtonNote->"Convolute"],
 "."
}], "Text",
 CellTags->"ConvoluteTable"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"ConvoluteTable"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ConvoluteTable", "[", 
  RowBox[{"1", ",", "1"}], "]"}]], "Input",
 CellTags->"ConvoluteTable",
 CellLabel->"In[297]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   RowBox[{"log", "(", "x", ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009701869644737`*^9},
 CellTags->"ConvoluteTable",
 CellLabel->"Out[297]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ConvoluteTable", "[", 
  RowBox[{"x", ",", "x"}], "]"}]], "Input",
 CellTags->"ConvoluteTable",
 CellLabel->"In[298]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", "x"}], " ", 
   RowBox[{"log", "(", "x", ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970187041807*^9},
 CellTags->"ConvoluteTable",
 CellLabel->"Out[298]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ConvoluteTable", "[", 
  RowBox[{"1", ",", "x"}], "]"}]], "Input",
 CellTags->"ConvoluteTable",
 CellLabel->"In[299]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"1", "-", "x"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009701871128063`*^9},
 CellTags->"ConvoluteTable",
 CellLabel->"Out[299]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ConvoluteTable", "[", 
  RowBox[{
   FractionBox[
    RowBox[{"Log", "[", 
     RowBox[{"1", "-", "x"}], "]"}], 
    RowBox[{"1", "-", "x"}]], ",", "1"}], "]"}]], "Input",
 CellTags->"Convolute",
 CellLabel->"In[300]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    TemplateBox[{"2",RowBox[{"1", "-", "x"}]},
     "PolyLog"]}], "+", 
   RowBox[{
    FractionBox["1", "2"], " ", 
    RowBox[{
     SuperscriptBox["log", "2"], "(", 
     RowBox[{"1", "-", "x"}], ")"}]}], "-", 
   RowBox[{
    RowBox[{"log", "(", "x", ")"}], " ", 
    RowBox[{"log", "(", 
     RowBox[{"1", "-", "x"}], ")"}]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009701872449713`*^9},
 CellTags->"Convolute",
 CellLabel->"Out[300]="]
}, Open  ]]
}, Open  ]],

Cell["CovariantFieldDerivative", "ObjectName",
 CellTags->"CovariantFieldDerivative"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"CovariantFieldDerivative"],

Cell["\<\
CovariantFieldDerivative[f[x],x,{li1,li2,...},opts] is a covariant derivative \
of f[x] with respect to space-time variables x and with Lorentz indices li1, \
li2,... CovariantFieldDerivative has only typesetting definitions by default. \
The user is must supply his/her own definition of the actual function.\
\>", "Text",
 CellTags->"CovariantFieldDerivative"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["CovariantD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"CovariantD",
  ButtonNote->"CovariantD"],
 ", ",
 ButtonBox["ExpandPartialD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"ExpandPartialD",
  ButtonNote->"ExpandPartialD"],
 ", ",
 ButtonBox["FieldDerivative",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FieldDerivative",
  ButtonNote->"FieldDerivative"],
 "."
}], "Text",
 CellTags->"CovariantFieldDerivative"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"CovariantFieldDerivative"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CovariantFieldDerivative", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"QuantumField", "[", 
     RowBox[{"A", ",", 
      RowBox[{"{", "\[Mu]", "}"}]}], "]"}], "[", "x", "]"}], ",", "x", ",", 
   RowBox[{"{", "\[Mu]", "}"}]}], "]"}]], "Input",
 CellTags->"CovariantFieldDerivative",
 CellLabel->"In[301]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox[
    TagBox[
     StyleBox["\<\"\[ScriptCapitalD]\"\>",
      FontSlant->"Italic"],
     StyleForm[#, FontSlant -> "Italic"]& ], 
    TagBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm,
     Editable->True]], "(", 
   TagBox[
    FormBox[
     RowBox[{
      SubscriptBox["A", "\[Mu]"], "(", "x", ")"}],
     TraditionalForm],
    TraditionalForm,
    Editable->True], ")"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970187279873*^9},
 CellTags->"CovariantFieldDerivative",
 CellLabel->"Out[301]="]
}, Open  ]]
}, Open  ]],

Cell["CounterT", "ObjectName",
 CellTags->"CounterT"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"CounterT"],

Cell["\<\
CounterT is a factor used by GluonPropagator and QuarkPropagator when \
CounterTerms is set to All.\
\>", "Text",
 CellTags->"CounterT"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["CounterTerm",
  BaseStyle->"AddOnsLink",
  ButtonData:>"CounterTerm",
  ButtonNote->"CounterTerm"],
 ", ",
 ButtonBox["GluonPropagator",
  BaseStyle->"AddOnsLink",
  ButtonData:>"GluonPropagator",
  ButtonNote->"GluonPropagator"],
 ", ",
 ButtonBox["QuarkPropagator",
  BaseStyle->"AddOnsLink",
  ButtonData:>"QuarkPropagator",
  ButtonNote->"QuarkPropagator"],
 "."
}], "Text",
 CellTags->"CounterT"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"CounterT"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GluonPropagator", "[", 
  RowBox[{"p", ",", "\[Mu]", ",", "a", ",", "\[Nu]", ",", "b", ",", 
   RowBox[{"Explicit", "\[Rule]", "True"}], ",", 
   RowBox[{"CounterTerm", "\[Rule]", " ", "All"}]}], "]"}]], "Input",
 CellTags->"CounterT",
 CellLabel->"In[302]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"CounterT", " ", 
    RowBox[{"(", 
     RowBox[{
      FractionBox[
       RowBox[{"\[ImaginaryI]", " ", 
        SubscriptBox["C", "A"], " ", 
        SubsuperscriptBox["g", "s", "2"], " ", 
        SubscriptBox["S", "n"], " ", 
        SubscriptBox["\[Delta]", 
         RowBox[{
          FormBox[
           FormBox["a",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["b",
            TraditionalForm],
           TraditionalForm]}]], " ", 
        RowBox[{"(", 
         RowBox[{
          FractionBox[
           RowBox[{"10", " ", 
            SuperscriptBox[
             FormBox["p",
              TraditionalForm], 
             FormBox[
              FormBox["\[Mu]",
               TraditionalForm],
              TraditionalForm]], " ", 
            SuperscriptBox[
             FormBox["p",
              TraditionalForm], 
             FormBox[
              FormBox["\[Nu]",
               TraditionalForm],
              TraditionalForm]]}], "3"], "-", 
          RowBox[{
           FractionBox["10", "3"], " ", 
           SuperscriptBox[
            FormBox["p",
             TraditionalForm], "2"], " ", 
           SuperscriptBox["g", 
            RowBox[{"\[Mu]", "\[Nu]"}]]}]}], ")"}]}], 
       TagBox["\[CurlyEpsilon]",
        TraditionalForm]], "+", 
      FractionBox[
       RowBox[{"\[ImaginaryI]", " ", 
        SubscriptBox["T", "f"], " ", 
        SubsuperscriptBox["g", "s", "2"], " ", 
        SubscriptBox["S", "n"], " ", 
        SubscriptBox["\[Delta]", 
         RowBox[{
          FormBox[
           FormBox["a",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["b",
            TraditionalForm],
           TraditionalForm]}]], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           FractionBox["4", "3"], " ", 
           SuperscriptBox[
            FormBox["p",
             TraditionalForm], "2"], " ", 
           SuperscriptBox["g", 
            RowBox[{"\[Mu]", "\[Nu]"}]]}], "-", 
          FractionBox[
           RowBox[{"4", " ", 
            SuperscriptBox[
             FormBox["p",
              TraditionalForm], 
             FormBox[
              FormBox["\[Mu]",
               TraditionalForm],
              TraditionalForm]], " ", 
            SuperscriptBox[
             FormBox["p",
              TraditionalForm], 
             FormBox[
              FormBox["\[Nu]",
               TraditionalForm],
              TraditionalForm]]}], "3"]}], ")"}]}], 
       TagBox["\[CurlyEpsilon]",
        TraditionalForm]]}], ")"}]}], "-", 
   FractionBox[
    RowBox[{"\[ImaginaryI]", " ", 
     SubscriptBox["\[Delta]", 
      RowBox[{
       FormBox[
        FormBox["a",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["b",
         TraditionalForm],
        TraditionalForm]}]], " ", 
     SuperscriptBox["g", 
      RowBox[{"\[Mu]", "\[Nu]"}]]}], 
    SuperscriptBox["p", "2"]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970187390985*^9},
 CellTags->"CounterT",
 CellLabel->"Out[302]="]
}, Open  ]]
}, Open  ]],

Cell["CounterTerm", "ObjectName",
 CellTags->"CounterTerm"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"CounterTerm"],

Cell["\<\
CounterTerm[name] is a database of counter terms. CounterTerm is also an \
option for the Feynman rule functions QuarkGluonVertex, GluonPropagator, \
QuarkPropagator.\
\>", "Text",
 CellTags->"CounterTerm"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["CounterT",
  BaseStyle->"AddOnsLink",
  ButtonData:>"CounterT",
  ButtonNote->"CounterT"],
 ", ",
 ButtonBox["QuarkGluonVertex",
  BaseStyle->"AddOnsLink",
  ButtonData:>"QuarkGluonVertex",
  ButtonNote->"QuarkGluonVertex"],
 ", ",
 ButtonBox["GluonPropagator",
  BaseStyle->"AddOnsLink",
  ButtonData:>"GluonPropagator",
  ButtonNote->"GluonPropagator"],
 ", ",
 ButtonBox["QuarkPropagator",
  BaseStyle->"AddOnsLink",
  ButtonData:>"QuarkPropagator",
  ButtonNote->"QuarkPropagator"],
 "."
}], "Text",
 CellTags->"CounterTerm"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"CounterTerm"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CounterTerm", "[", "Zm", "]"}]], "Input",
 CellTags->"CounterTerm",
 CellLabel->"In[303]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{
     SubscriptBox["C", "F"], " ", 
     SubsuperscriptBox["g", "s", "4"], " ", 
     RowBox[{"(", 
      RowBox[{
       FractionBox[
        RowBox[{"4", " ", 
         RowBox[{"(", 
          RowBox[{
           FractionBox[
            RowBox[{"11", " ", 
             SubscriptBox["C", "A"]}], "2"], "+", 
           FractionBox[
            RowBox[{"9", " ", 
             SubscriptBox["C", "F"]}], "2"], "-", 
           RowBox[{"2", " ", 
            SubscriptBox["N", "f"], " ", 
            SubscriptBox["T", "f"]}]}], ")"}]}], 
        SuperscriptBox[
         TagBox["\[CurlyEpsilon]",
          TraditionalForm], "2"]], "+", 
       FractionBox[
        RowBox[{"2", " ", 
         RowBox[{"(", 
          RowBox[{
           FractionBox[
            RowBox[{"97", " ", 
             SubscriptBox["C", "A"]}], "12"], "+", 
           FractionBox[
            RowBox[{"3", " ", 
             SubscriptBox["C", "F"]}], "4"], "-", 
           FractionBox[
            RowBox[{"5", " ", 
             SubscriptBox["N", "f"], " ", 
             SubscriptBox["T", "f"]}], "3"]}], ")"}]}], 
        TagBox["\[CurlyEpsilon]",
         TraditionalForm]]}], ")"}]}], 
    RowBox[{"256", " ", 
     SuperscriptBox["\[Pi]", "4"]}]], "+", 
   FractionBox[
    RowBox[{"3", " ", 
     SubscriptBox["C", "F"], " ", 
     SubsuperscriptBox["g", "s", "2"]}], 
    RowBox[{"8", " ", 
     SuperscriptBox["\[Pi]", "2"], " ", 
     TagBox["\[CurlyEpsilon]",
      TraditionalForm]}]], "+", "1"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009701879494047`*^9},
 CellTags->"CounterTerm",
 CellLabel->"Out[303]="]
}, Open  ]]
}, Open  ]],

Cell["CouplingConstant", "ObjectName",
 CellTags->"CouplingConstant"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"CouplingConstant"],

Cell["\<\
In general, CouplingConstant is an option for several Feynman rule functions \
and for CovariantD and FieldStrength.
In the convention of the subpackage PHI, CouplingConstant is also the head of \
coupling constants.  CouplingConstant takes three extra optional arguments, \
with head RenormalizationState, RenormalizationScheme and ExpansionState \
respectively.  E.g. CouplingConstant[QED[1]] is the unit charge, \
CouplingConstant[ChPT2[4],1] is the first of the coupling constants of the \
lagrangian ChPT2[4].  CouplingConstant[a_,b_,c___][i_] := \
CouplingConstant[a,b,RenormalizationState[i],c].\
\>", "Text",
 CellTags->"CouplingConstant"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["CovariantD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"CovariantD",
  ButtonNote->"CovariantD"],
 ", ",
 ButtonBox["FieldStrength",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FieldStrength",
  ButtonNote->"FieldStrength"],
 "."
}], "Text",
 CellTags->"CouplingConstant"]
}, Open  ]],

Cell["CovariantD", "ObjectName",
 CellTags->"CovariantD"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"CovariantD"],

Cell["\<\
CovariantD[mu] is a generic covariant derivative with Lorentz index mu. With \
the option-setting Explicit \[Rule] True, an explicit expression for a \
fermionic field is returned, depending on the setting on the other options.
CovariantD[x, mu] is a generic covariant derivative with respect to x^mu.
CovariantD[mu, a, b] is a covariant derivative for a bosonic field; acting on \
QuantumField[f,{},{a,b}], where f is some field name and a and b are two \
SU(N) indices. Again, with the option-setting Explicit \[Rule] True, an \
explicit expression is returned, depending on the setting on the other options.
CovariantD[OPEDelta, a, b] is a short form for  \
CovariantD[mu,a,b]*FourVector[OPEDelta, mu]. CovariantD[{OPEDelta, a, b}, \
{n}] yields the product of n operators, where n is an integer.  \
CovariantD[OPEDelta, a, b, {m, n}]  gives the expanded form of \
CovariantD[OPEDelta, a, b]^m up to order g^n for the gluon, where n is an \
integer and g the coupling constant  indicated by the setting of the option \
CouplingConstant. CovariantD[OPEDelta, {m, n}] gives the expanded form of  \
CovariantD[OPEDelta]^m up to order g^n of the fermionic field.\
\>", "Text",
 CellTags->"CovariantD"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "CovariantD", "]"}]], "Input",
 CellTags->"CovariantD",
 CellLabel->"In[304]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"CouplingConstant", "\[Rule]", 
     SubscriptBox["g", "s"]}], ",", 
    RowBox[{"DummyIndex", "\[Rule]", "Automatic"}], ",", 
    RowBox[{"Explicit", "\[Rule]", "False"}], ",", 
    RowBox[{
    "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", 
     "\[Rule]", "RightPartialD"}], ",", 
    RowBox[{"QuantumField", "\[Rule]", "A"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.5009701880424547`*^9},
 CellTags->"CovariantD",
 CellLabel->"Out[304]="]
}, Open  ]],

Cell["\<\
 Possible settings of PartialD are: LeftPartialD, LeftRigthPartialD, \
RightPartialD. The default setting of QuantumField is GaugeField.\
\>", "Text",
 CellTags->"CovariantD"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["LeftPartialD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"LeftPartialD",
  ButtonNote->"LeftPartialD"],
 ", ",
 ButtonBox["LeftRightPartialD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"LeftRightPartialD",
  ButtonNote->"LeftRightPartialD"],
 ", ",
 ButtonBox["RightPartialD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"RightPartialD",
  ButtonNote->"RightPartialD"],
 "."
}], "Text",
 CellTags->"CovariantD"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"CovariantD"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CovariantD", "[", "\[Mu]", "]"}]], "Input",
 CellTags->"CovariantD",
 CellLabel->"In[305]:="],

Cell[BoxData[
 FormBox[
  SubscriptBox["D", "\[Mu]"], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970188101343*^9},
 CellTags->"CovariantD",
 CellLabel->"Out[305]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CovariantD", "[", 
  RowBox[{"\[Mu]", ",", "a", ",", "b"}], "]"}]], "Input",
 CellTags->"CovariantD",
 CellLabel->"In[306]:="],

Cell[BoxData[
 FormBox[
  SubsuperscriptBox["D", "\[Mu]", 
   RowBox[{"a", "b"}]], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970188148127*^9},
 CellTags->"CovariantD",
 CellLabel->"Out[306]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CovariantD", "[", 
  RowBox[{"\[Mu]", ",", 
   RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"CovariantD",
 CellLabel->"In[307]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox[
    OverscriptBox["\[PartialD]", "\[RightArrow]"], 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], "-", 
   RowBox[{"\[ImaginaryI]", " ", 
    SubscriptBox["g", "s"], " ", 
    RowBox[{
     SubscriptBox["T", 
      FormBox[
       FormBox[
        InterpretationBox[
         SubscriptBox["c", "1"],
         Subscripted[
          $CellContext`c[1]],
         Editable->False],
        TraditionalForm],
       TraditionalForm]], ".", 
     SubsuperscriptBox[
      FormBox["A",
       TraditionalForm], 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox[
        InterpretationBox[
         SubscriptBox["c", "1"],
         Subscripted[
          $CellContext`c[1]],
         Editable->False],
        TraditionalForm],
       TraditionalForm]]}]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970188309299*^9},
 CellTags->"CovariantD",
 CellLabel->"Out[307]="]
}, Open  ]],

Cell["\<\
The first argument of CovariantD is intepreted as type LorentzIndex, except \
for OPEDelta, which is type Momentum.\
\>", "Text",
 CellTags->"CovariantD"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CovariantD", "[", "OPEDelta", "]"}]], "Input",
 CellTags->"CovariantD",
 CellLabel->"In[308]:="],

Cell[BoxData[
 FormBox[
  SubscriptBox["D", 
   FormBox["\[CapitalDelta]",
    TraditionalForm]], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970188393202*^9},
 CellTags->"CovariantD",
 CellLabel->"Out[308]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CovariantD", "[", 
  RowBox[{"OPEDelta", ",", "a", ",", "b"}], "]"}]], "Input",
 CellTags->"CovariantD",
 CellLabel->"In[309]:="],

Cell[BoxData[
 FormBox[
  SubsuperscriptBox["D", 
   FormBox["\[CapitalDelta]",
    TraditionalForm], 
   RowBox[{"a", "b"}]], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970188441782*^9},
 CellTags->"CovariantD",
 CellLabel->"Out[309]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CovariantD", "[", 
  RowBox[{"OPEDelta", ",", "a", ",", "b", ",", 
   RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"CovariantD",
 CellLabel->"In[310]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox["\[Delta]", 
     RowBox[{"a", "b"}]], " ", 
    SubscriptBox[
     OverscriptBox["\[PartialD]", "\[RightArrow]"], 
     FormBox["\[CapitalDelta]",
      TraditionalForm]]}], "-", 
   RowBox[{
    SubscriptBox["g", "s"], " ", 
    SubsuperscriptBox[
     FormBox["A",
      TraditionalForm], 
     FormBox["\[CapitalDelta]",
      TraditionalForm], 
     FormBox[
      FormBox[
       InterpretationBox[
        SubscriptBox["c", "2"],
        Subscripted[
         $CellContext`c[2]],
        Editable->False],
       TraditionalForm],
      TraditionalForm]], " ", 
    SubscriptBox["f", 
     RowBox[{"a", "b", 
      FormBox[
       InterpretationBox[
        SubscriptBox["c", "2"],
        Subscripted[
         $CellContext`c[2]],
        Editable->False],
       TraditionalForm]}]]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970188733556*^9},
 CellTags->"CovariantD",
 CellLabel->"Out[310]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CovariantD", "[", 
  RowBox[{"OPEDelta", ",", 
   RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"CovariantD",
 CellLabel->"In[311]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox[
    OverscriptBox["\[PartialD]", "\[RightArrow]"], 
    FormBox["\[CapitalDelta]",
     TraditionalForm]], "-", 
   RowBox[{"\[ImaginaryI]", " ", 
    SubscriptBox["g", "s"], " ", 
    RowBox[{
     SubscriptBox["T", 
      FormBox[
       FormBox[
        InterpretationBox[
         SubscriptBox["c", "3"],
         Subscripted[
          $CellContext`c[3]],
         Editable->False],
        TraditionalForm],
       TraditionalForm]], ".", 
     SubsuperscriptBox[
      FormBox["A",
       TraditionalForm], 
      FormBox["\[CapitalDelta]",
       TraditionalForm], 
      FormBox[
       FormBox[
        InterpretationBox[
         SubscriptBox["c", "3"],
         Subscripted[
          $CellContext`c[3]],
         Editable->False],
        TraditionalForm],
       TraditionalForm]]}]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970188811976*^9},
 CellTags->"CovariantD",
 CellLabel->"Out[311]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CovariantD", "[", 
  RowBox[{"OPEDelta", ",", "a", ",", "b", ",", 
   RowBox[{"{", "2", "}"}]}], "]"}]], "Input",
 CellTags->"CovariantD",
 CellLabel->"In[312]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      SubscriptBox["\[Delta]", 
       RowBox[{"a", "c22"}]], " ", 
      SubscriptBox[
       OverscriptBox["\[PartialD]", "\[RightArrow]"], 
       FormBox["\[CapitalDelta]",
        TraditionalForm]]}], "-", 
     RowBox[{
      SubscriptBox["g", "s"], " ", 
      SubsuperscriptBox[
       FormBox["A",
        TraditionalForm], 
       FormBox["\[CapitalDelta]",
        TraditionalForm], 
       FormBox[
        FormBox["e23",
         TraditionalForm],
        TraditionalForm]], " ", 
      SubscriptBox["f", 
       RowBox[{"a", "c22", "e23"}]]}]}], ")"}], ".", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      SubscriptBox["\[Delta]", 
       RowBox[{"b", "c22"}]], " ", 
      SubscriptBox[
       OverscriptBox["\[PartialD]", "\[RightArrow]"], 
       FormBox["\[CapitalDelta]",
        TraditionalForm]]}], "-", 
     RowBox[{
      SubscriptBox["g", "s"], " ", 
      SubsuperscriptBox[
       FormBox["A",
        TraditionalForm], 
       FormBox["\[CapitalDelta]",
        TraditionalForm], 
       FormBox[
        FormBox["e24",
         TraditionalForm],
        TraditionalForm]], " ", 
      SubscriptBox["f", 
       RowBox[{"c22", "b", "e24"}]]}]}], ")"}]}], TraditionalForm]], "Output",\

 CellChangeTimes->{3.50097018891133*^9},
 CellTags->"CovariantD",
 CellLabel->"Out[312]="]
}, Open  ]],

Cell[TextData[{
 "This gives",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{"m", " ", "times", " ", 
      SubscriptBox[
       OverscriptBox["\[PartialD]", "\[RightArrow]"], "\[CapitalDelta]"]}], 
     ",", " "}]}], TraditionalForm]]],
 "the partial derivative ",
 Cell[BoxData[
  FormBox[
   SubscriptBox[
    OverscriptBox["\[PartialD]", "\[RightArrow]"], 
    RowBox[{"\[Mu]", " "}]], TraditionalForm]]],
 "contracted with ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["\[CapitalDelta]", "\[Mu]"], "."}], TraditionalForm]]]
}], "Text",
 CellTags->"CovariantD"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CovariantD", "[", 
  RowBox[{"OPEDelta", ",", "a", ",", "b", ",", 
   RowBox[{"{", 
    RowBox[{"OPEm", ",", "0"}], "}"}]}], "]"}]], "Input",
 CellTags->"CovariantD",
 CellLabel->"In[313]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["\[Delta]", 
    RowBox[{
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm]}]], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     SubscriptBox[
      OverscriptBox["\[PartialD]", "\[RightArrow]"], 
      FormBox["\[CapitalDelta]",
       TraditionalForm]], ")"}], "m"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.50097018924888*^9},
 CellTags->"CovariantD",
 CellLabel->"Out[313]="]
}, Open  ]],

Cell[TextData[{
 "The expansion up to first order in the coupling constant ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["g", "s"], " ", 
    RowBox[{
     RowBox[{"(", "thesumistheFeynCalcOPESum", ")"}], "."}]}], 
   TraditionalForm]]]
}], "Text",
 CellTags->"CovariantD"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CovariantD", "[", 
  RowBox[{"OPEDelta", ",", "a", ",", "b", ",", 
   RowBox[{"{", 
    RowBox[{"OPEm", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellTags->"CovariantD",
 CellLabel->"In[314]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      SubscriptBox[
       OverscriptBox["\[PartialD]", "\[RightArrow]"], 
       FormBox["\[CapitalDelta]",
        TraditionalForm]], ")"}], "m"]}], "-", 
   RowBox[{
    SubscriptBox["g", "s"], " ", 
    FormBox[
     TagBox[
      RowBox[{
       UnderoverscriptBox["\[Sum]", 
        RowBox[{"i", "=", "0"}], 
        RowBox[{
         RowBox[{"-", "1"}], "+", "m"}]], 
       RowBox[{
        RowBox[{
         SuperscriptBox[
          RowBox[{"(", 
           SubscriptBox[
            OverscriptBox["\[PartialD]", "\[RightArrow]"], 
            FormBox["\[CapitalDelta]",
             TraditionalForm]], ")"}], "i"], ".", 
         SubsuperscriptBox[
          FormBox["A",
           TraditionalForm], 
          FormBox["\[CapitalDelta]",
           TraditionalForm], 
          FormBox[
           FormBox[
            InterpretationBox[
             SubscriptBox["c", "1"],
             Subscripted[
              $CellContext`c[1]],
             Editable->False],
            TraditionalForm],
           TraditionalForm]], ".", 
         SuperscriptBox[
          RowBox[{"(", 
           SubscriptBox[
            OverscriptBox["\[PartialD]", "\[RightArrow]"], 
            FormBox["\[CapitalDelta]",
             TraditionalForm]], ")"}], 
          RowBox[{
           RowBox[{"-", "1"}], "-", "i", "+", "m"}]]}], " ", 
        SubscriptBox["f", 
         RowBox[{
          FormBox[
           FormBox["a",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["b",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox[
            InterpretationBox[
             SubscriptBox["c", "1"],
             Subscripted[
              $CellContext`c[1]],
             Editable->False],
            TraditionalForm],
           TraditionalForm]}]]}]}],
      HoldForm],
     TraditionalForm]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009701894162407`*^9},
 CellTags->"CovariantD",
 CellLabel->"Out[314]="]
}, Open  ]],

Cell[TextData[{
 "The expansion up to second order in the ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["g", "s"], "."}], TraditionalForm]]]
}], "Text",
 CellTags->"CovariantD"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CovariantD", "[", 
  RowBox[{"OPEDelta", ",", "a", ",", "b", ",", 
   RowBox[{"{", 
    RowBox[{"OPEm", ",", "2"}], "}"}]}], "]"}]], "Input",
 CellTags->"CovariantD",
 CellLabel->"In[315]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    RowBox[{
     SubsuperscriptBox["g", "s", "2"], " ", 
     FormBox[
      TagBox[
       RowBox[{
        UnderoverscriptBox["\[Sum]", 
         RowBox[{"j", "=", "0"}], 
         RowBox[{
          RowBox[{"-", "2"}], "+", "m"}]], 
        RowBox[{"(", 
         FormBox[
          TagBox[
           RowBox[{
            UnderoverscriptBox["\[Sum]", 
             RowBox[{"i", "=", "0"}], "j"], 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"(", 
                SubscriptBox[
                 OverscriptBox["\[PartialD]", "\[RightArrow]"], 
                 FormBox["\[CapitalDelta]",
                  TraditionalForm]], ")"}], "i"], ".", 
              SubsuperscriptBox[
               FormBox["A",
                TraditionalForm], 
               FormBox["\[CapitalDelta]",
                TraditionalForm], 
               FormBox[
                FormBox[
                 InterpretationBox[
                  SubscriptBox["c", "1"],
                  Subscripted[
                   $CellContext`c[1]],
                  Editable->False],
                 TraditionalForm],
                TraditionalForm]], ".", 
              SuperscriptBox[
               RowBox[{"(", 
                SubscriptBox[
                 OverscriptBox["\[PartialD]", "\[RightArrow]"], 
                 FormBox["\[CapitalDelta]",
                  TraditionalForm]], ")"}], 
               RowBox[{
                RowBox[{"-", "i"}], "+", "j"}]], ".", 
              SubsuperscriptBox[
               FormBox["A",
                TraditionalForm], 
               FormBox["\[CapitalDelta]",
                TraditionalForm], 
               FormBox[
                FormBox[
                 InterpretationBox[
                  SubscriptBox["c", "2"],
                  Subscripted[
                   $CellContext`c[2]],
                  Editable->False],
                 TraditionalForm],
                TraditionalForm]], ".", 
              SuperscriptBox[
               RowBox[{"(", 
                SubscriptBox[
                 OverscriptBox["\[PartialD]", "\[RightArrow]"], 
                 FormBox["\[CapitalDelta]",
                  TraditionalForm]], ")"}], 
               RowBox[{
                RowBox[{"-", "2"}], "-", "j", "+", "m"}]]}], " ", 
             SubscriptBox["f", 
              RowBox[{
               FormBox[
                FormBox["a",
                 TraditionalForm],
                TraditionalForm], 
               FormBox[
                FormBox[
                 InterpretationBox[
                  SubscriptBox["c", "1"],
                  Subscripted[
                   $CellContext`c[1]],
                  Editable->False],
                 TraditionalForm],
                TraditionalForm], 
               FormBox[
                FormBox[
                 InterpretationBox[
                  SubscriptBox["e", "1"],
                  Subscripted[
                   $CellContext`e[1]],
                  Editable->False],
                 TraditionalForm],
                TraditionalForm]}]], " ", 
             SubscriptBox["f", 
              RowBox[{
               FormBox[
                FormBox["b",
                 TraditionalForm],
                TraditionalForm], 
               FormBox[
                FormBox[
                 InterpretationBox[
                  SubscriptBox["c", "2"],
                  Subscripted[
                   $CellContext`c[2]],
                  Editable->False],
                 TraditionalForm],
                TraditionalForm], 
               FormBox[
                FormBox[
                 InterpretationBox[
                  SubscriptBox["e", "1"],
                  Subscripted[
                   $CellContext`e[1]],
                  Editable->False],
                 TraditionalForm],
                TraditionalForm]}]]}]}],
           HoldForm],
          TraditionalForm], ")"}]}],
       HoldForm],
      TraditionalForm]}]}], "-", 
   RowBox[{
    SubscriptBox["g", "s"], " ", 
    FormBox[
     TagBox[
      RowBox[{
       UnderoverscriptBox["\[Sum]", 
        RowBox[{"i", "=", "0"}], 
        RowBox[{
         RowBox[{"-", "1"}], "+", "m"}]], 
       RowBox[{
        RowBox[{
         SuperscriptBox[
          RowBox[{"(", 
           SubscriptBox[
            OverscriptBox["\[PartialD]", "\[RightArrow]"], 
            FormBox["\[CapitalDelta]",
             TraditionalForm]], ")"}], "i"], ".", 
         SubsuperscriptBox[
          FormBox["A",
           TraditionalForm], 
          FormBox["\[CapitalDelta]",
           TraditionalForm], 
          FormBox[
           FormBox[
            InterpretationBox[
             SubscriptBox["c", "1"],
             Subscripted[
              $CellContext`c[1]],
             Editable->False],
            TraditionalForm],
           TraditionalForm]], ".", 
         SuperscriptBox[
          RowBox[{"(", 
           SubscriptBox[
            OverscriptBox["\[PartialD]", "\[RightArrow]"], 
            FormBox["\[CapitalDelta]",
             TraditionalForm]], ")"}], 
          RowBox[{
           RowBox[{"-", "1"}], "-", "i", "+", "m"}]]}], " ", 
        SubscriptBox["f", 
         RowBox[{
          FormBox[
           FormBox["a",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["b",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox[
            InterpretationBox[
             SubscriptBox["c", "1"],
             Subscripted[
              $CellContext`c[1]],
             Editable->False],
            TraditionalForm],
           TraditionalForm]}]]}]}],
      HoldForm],
     TraditionalForm]}], "+", 
   RowBox[{
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      SubscriptBox[
       OverscriptBox["\[PartialD]", "\[RightArrow]"], 
       FormBox["\[CapitalDelta]",
        TraditionalForm]], ")"}], "m"]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970189610127*^9},
 CellTags->"CovariantD",
 CellLabel->"Out[315]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 SuperscriptBox[
  RowBox[{"CovariantD", "[", 
   RowBox[{"OPEDelta", ",", "a", ",", "b"}], "]"}], "OPEm"]], "Input",
 CellTags->"CovariantD",
 CellLabel->"In[316]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox[
   RowBox[{"(", 
    SubsuperscriptBox["D", 
     FormBox["\[CapitalDelta]",
      TraditionalForm], 
     RowBox[{"a", "b"}]], ")"}], "m"], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970190068985*^9},
 CellTags->"CovariantD",
 CellLabel->"Out[316]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CovariantD", "[", 
  RowBox[{"OPEDelta", ",", 
   RowBox[{"{", 
    RowBox[{"OPEm", ",", "2"}], "}"}]}], "]"}]], "Input",
 CellTags->"CovariantD",
 CellLabel->"In[317]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    RowBox[{
     SubsuperscriptBox["g", "s", "2"], " ", 
     FormBox[
      TagBox[
       RowBox[{
        UnderoverscriptBox["\[Sum]", 
         RowBox[{"j", "=", "0"}], 
         RowBox[{
          RowBox[{"-", "2"}], "+", "m"}]], 
        RowBox[{"(", 
         FormBox[
          TagBox[
           RowBox[{
            UnderoverscriptBox["\[Sum]", 
             RowBox[{"i", "=", "0"}], "j"], 
            RowBox[{
             SubscriptBox["T", 
              FormBox[
               FormBox[
                InterpretationBox[
                 SubscriptBox["c", "1"],
                 Subscripted[
                  $CellContext`c[1]],
                 Editable->False],
                TraditionalForm],
               TraditionalForm]], ".", 
             SubscriptBox["T", 
              FormBox[
               FormBox[
                InterpretationBox[
                 SubscriptBox["c", "2"],
                 Subscripted[
                  $CellContext`c[2]],
                 Editable->False],
                TraditionalForm],
               TraditionalForm]], ".", 
             SuperscriptBox[
              RowBox[{"(", 
               SubscriptBox[
                OverscriptBox["\[PartialD]", "\[RightArrow]"], 
                FormBox["\[CapitalDelta]",
                 TraditionalForm]], ")"}], "i"], ".", 
             SubsuperscriptBox[
              FormBox["A",
               TraditionalForm], 
              FormBox["\[CapitalDelta]",
               TraditionalForm], 
              FormBox[
               FormBox[
                InterpretationBox[
                 SubscriptBox["c", "1"],
                 Subscripted[
                  $CellContext`c[1]],
                 Editable->False],
                TraditionalForm],
               TraditionalForm]], ".", 
             SuperscriptBox[
              RowBox[{"(", 
               SubscriptBox[
                OverscriptBox["\[PartialD]", "\[RightArrow]"], 
                FormBox["\[CapitalDelta]",
                 TraditionalForm]], ")"}], 
              RowBox[{
               RowBox[{"-", "i"}], "+", "j"}]], ".", 
             SubsuperscriptBox[
              FormBox["A",
               TraditionalForm], 
              FormBox["\[CapitalDelta]",
               TraditionalForm], 
              FormBox[
               FormBox[
                InterpretationBox[
                 SubscriptBox["c", "2"],
                 Subscripted[
                  $CellContext`c[2]],
                 Editable->False],
                TraditionalForm],
               TraditionalForm]], ".", 
             SuperscriptBox[
              RowBox[{"(", 
               SubscriptBox[
                OverscriptBox["\[PartialD]", "\[RightArrow]"], 
                FormBox["\[CapitalDelta]",
                 TraditionalForm]], ")"}], 
              RowBox[{
               RowBox[{"-", "2"}], "-", "j", "+", "m"}]]}]}],
           HoldForm],
          TraditionalForm], ")"}]}],
       HoldForm],
      TraditionalForm]}]}], "-", 
   RowBox[{"\[ImaginaryI]", " ", 
    SubscriptBox["g", "s"], " ", 
    FormBox[
     TagBox[
      RowBox[{
       UnderoverscriptBox["\[Sum]", 
        RowBox[{"i", "=", "0"}], 
        RowBox[{
         RowBox[{"-", "1"}], "+", "m"}]], 
       RowBox[{
        SubscriptBox["T", 
         FormBox[
          FormBox[
           InterpretationBox[
            SubscriptBox["c", "1"],
            Subscripted[
             $CellContext`c[1]],
            Editable->False],
           TraditionalForm],
          TraditionalForm]], ".", 
        SuperscriptBox[
         RowBox[{"(", 
          SubscriptBox[
           OverscriptBox["\[PartialD]", "\[RightArrow]"], 
           FormBox["\[CapitalDelta]",
            TraditionalForm]], ")"}], "i"], ".", 
        SubsuperscriptBox[
         FormBox["A",
          TraditionalForm], 
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox[
          FormBox[
           InterpretationBox[
            SubscriptBox["c", "1"],
            Subscripted[
             $CellContext`c[1]],
            Editable->False],
           TraditionalForm],
          TraditionalForm]], ".", 
        SuperscriptBox[
         RowBox[{"(", 
          SubscriptBox[
           OverscriptBox["\[PartialD]", "\[RightArrow]"], 
           FormBox["\[CapitalDelta]",
            TraditionalForm]], ")"}], 
         RowBox[{
          RowBox[{"-", "1"}], "-", "i", "+", "m"}]]}]}],
      HoldForm],
     TraditionalForm]}], "+", 
   SuperscriptBox[
    RowBox[{"(", 
     SubscriptBox[
      OverscriptBox["\[PartialD]", "\[RightArrow]"], 
      FormBox["\[CapitalDelta]",
       TraditionalForm]], ")"}], "m"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.50097019027411*^9},
 CellTags->"CovariantD",
 CellLabel->"Out[317]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"CovariantD", "[", 
   RowBox[{"OPEDelta", ",", 
    RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"CovariantD",
 CellLabel->"In[318]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"-", "\[ImaginaryI]"}], " ", "Gstrong", " ", 
   RowBox[{
    RowBox[{"SUNT", "[", 
     RowBox[{"SUNIndex", "[", 
      InterpretationBox[
       SubscriptBox["c", "4"],
       Subscripted[
        $CellContext`c[4]],
       Editable->False], "]"}], "]"}], ".", 
    RowBox[{"QuantumField", "[", 
     RowBox[{"GaugeField", ",", 
      RowBox[{"Momentum", "[", "OPEDelta", "]"}], ",", 
      RowBox[{"SUNIndex", "[", 
       InterpretationBox[
        SubscriptBox["c", "4"],
        Subscripted[
         $CellContext`c[4]],
        Editable->False], "]"}]}], "]"}]}]}], "+", 
  RowBox[{"RightPartialD", "[", 
   RowBox[{"Momentum", "[", "OPEDelta", "]"}], "]"}]}]], "Output",
 CellChangeTimes->{3.5009701903430347`*^9},
 CellTags->"CovariantD",
 CellLabel->"Out[318]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"CovariantD", "[", 
   RowBox[{"\[Mu]", ",", "a", ",", "b", ",", 
    RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"CovariantD",
 CellLabel->"In[319]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"RightPartialD", "[", 
    RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], "]"}], " ", 
   RowBox[{"SUNDelta", "[", 
    RowBox[{"a", ",", "b"}], "]"}]}], "-", 
  RowBox[{"Gstrong", " ", 
   RowBox[{"QuantumField", "[", 
    RowBox[{"GaugeField", ",", 
     RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
     RowBox[{"SUNIndex", "[", 
      InterpretationBox[
       SubscriptBox["c", "5"],
       Subscripted[
        $CellContext`c[5]],
       Editable->False], "]"}]}], "]"}], " ", 
   RowBox[{"SUNF", "[", 
    RowBox[{"a", ",", "b", ",", 
     InterpretationBox[
      SubscriptBox["c", "5"],
      Subscripted[
       $CellContext`c[5]],
      Editable->False]}], "]"}]}]}]], "Output",
 CellChangeTimes->{3.5009701904119473`*^9},
 CellTags->"CovariantD",
 CellLabel->"Out[319]//StandardForm="]
}, Open  ]]
}, Open  ]],

Cell["CrossProduct", "ObjectName",
 CellTags->"CrossProduct"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"CrossProduct"],

Cell["\<\
CrossProduct[a, b] denotes the three-dimensional cross-product of the \
three-vectors a and b.\
\>", "Text",
 CellTags->"CrossProduct"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["DotProduct",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DotProduct",
  ButtonNote->"DotProduct"],
 ", ",
 ButtonBox["ThreeVector",
  BaseStyle->"AddOnsLink",
  ButtonData:>"ThreeVector",
  ButtonNote->"ThreeVector"],
 "."
}], "Text",
 CellTags->"CrossProduct"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"CrossProduct"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CrossProduct", "[", 
  RowBox[{
   RowBox[{"ThreeVector", "[", "a", "]"}], ",", 
   RowBox[{"CrossProduct", "[", 
    RowBox[{
     RowBox[{"ThreeVector", "[", "b", "]"}], ",", 
     RowBox[{"ThreeVector", "[", "c", "]"}]}], "]"}]}], "]"}]], "Input",
 CellTags->"CrossProduct",
 CellLabel->"In[320]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    OverscriptBox["b", "\[RightVector]"], " ", 
    RowBox[{
     FormBox[
      OverscriptBox["a", "\[RightVector]"],
      TraditionalForm], "\[CenterDot]", 
     FormBox[
      OverscriptBox["c", "\[RightVector]"],
      TraditionalForm]}]}], "-", 
   RowBox[{
    OverscriptBox["c", "\[RightVector]"], " ", 
    RowBox[{
     FormBox[
      OverscriptBox["a", "\[RightVector]"],
      TraditionalForm], "\[CenterDot]", 
     FormBox[
      OverscriptBox["b", "\[RightVector]"],
      TraditionalForm]}]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970190548152*^9},
 CellTags->"CrossProduct",
 CellLabel->"Out[320]="]
}, Open  ]]
}, Open  ]],

Cell["C0", "ObjectName",
 CellTags->"C0"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"C0"],

Cell[TextData[{
 "C0[p10, p12, p20, m1^2, m2^2, m3^2] is the scalar Passarino-Veltman ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["C", "0"], TraditionalForm]]],
 "function. The convention for the arguments is that if the denominator of \
the integrand has the form ([q^2-m1^2] [(q+p1)^2-m2^2] [(q+p2)^2-m3^2]), the \
first three arguments of C0 are the scalar products p10 = p1^2, p12 = \
(p1-p2).(p1-p2), p20 = p2^2."
}], "Text",
 CellTags->"C0"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["B0",
  BaseStyle->"AddOnsLink",
  ButtonData:>"B0",
  ButtonNote->"B0"],
 ", ",
 ButtonBox["D0",
  BaseStyle->"AddOnsLink",
  ButtonData:>"D0",
  ButtonNote->"D0"],
 ", ",
 ButtonBox["PaVe",
  BaseStyle->"AddOnsLink",
  ButtonData:>"PaVe",
  ButtonNote->"PaVe"],
 ", ",
 ButtonBox["PaVeOrder",
  BaseStyle->"AddOnsLink",
  ButtonData:>"PaVeOrder",
  ButtonNote->"PaVeOrder"],
 "."
}], "Text",
 CellTags->"C0"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"C0"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"C0", "[", 
  RowBox[{"a", ",", "b", ",", "c", ",", " ", "m12", ",", "m22", ",", "m32"}], 
  "]"}]], "Input",
 CellTags->"C0",
 CellLabel->"In[321]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox[
    SubscriptBox["\<\"C\"\>", "\<\"0\"\>"],
    TraditionalForm], "(", "a", ",", "b", ",", "c", ",", "m12", ",", "m22", 
   ",", "m32", ")"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970190591611*^9},
 CellTags->"C0",
 CellLabel->"Out[321]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"C0", "[", 
   RowBox[{"b", ",", "a", ",", "c", ",", "m32", ",", "m22", ",", "m12"}], 
   "]"}], "//", "PaVeOrder"}]], "Input",
 CellTags->"C0",
 CellLabel->"In[322]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox[
    SubscriptBox["\<\"C\"\>", "\<\"0\"\>"],
    TraditionalForm], "(", "a", ",", "b", ",", "c", ",", "m12", ",", "m22", 
   ",", "m32", ")"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970190678334*^9},
 CellTags->"C0",
 CellLabel->"Out[322]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PaVeOrder", "[", 
  RowBox[{
   RowBox[{"C0", "[", 
    RowBox[{"b", ",", "a", ",", "c", ",", "m32", ",", "m22", ",", "m12"}], 
    "]"}], ",", 
   RowBox[{"PaVeOrderList", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"c", ",", "a"}], "}"}]}]}], "]"}]], "Input",
 CellTags->"C0",
 CellLabel->"In[323]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox[
    SubscriptBox["\<\"C\"\>", "\<\"0\"\>"],
    TraditionalForm], "(", "b", ",", "c", ",", "a", ",", "m22", ",", "m32", 
   ",", "m12", ")"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.50097019115814*^9},
 CellTags->"C0",
 CellLabel->"Out[323]="]
}, Open  ]]
}, Open  ]],

Cell["DataType", "ObjectName",
 CellTags->"DataType"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"DataType"],

Cell["\<\
DataType[exp, type] = True defines the object exp to have data-type type. \
DataType[exp1, exp2, ..., type] defines the objects exp1, exp2, ...to have \
data-type type. The default setting is DataType[__, _] := False. To assign a \
certain data-type, do, e.g., DataType[x, PositiveInteger] = True.\
\>", "Text",
 CellTags->"DataType"],

Cell[TextData[{
 "Currently used DataTypes: ",
 ButtonBox["NonCommutative",
  BaseStyle->"AddOnsLink",
  ButtonData:>"NonCommutative",
  ButtonNote->"NonCommutative"],
 ", ",
 ButtonBox["PositiveInteger",
  BaseStyle->"AddOnsLink",
  ButtonData:>"PositiveInteger",
  ButtonNote->"PositiveInteger"],
 ", ",
 ButtonBox["NegativeInteger",
  BaseStyle->"AddOnsLink",
  ButtonData:>"NegativeInteger",
  ButtonNote->"NegativeInteger"],
 ", ",
 ButtonBox["PositiveNumber",
  BaseStyle->"AddOnsLink",
  ButtonData:>"PositiveNumber",
  ButtonNote->"PositiveNumber"],
 ", ",
 ButtonBox["FreeIndex",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FreeIndex",
  ButtonNote->"FreeIndex"],
 ", ",
 ButtonBox["GrassmannParity",
  BaseStyle->"AddOnsLink",
  ButtonData:>"GrassmannParity",
  ButtonNote->"GrassmannParity"]
}], "Text",
 CellTags->"DataType"],

Cell[TextData[{
 "PHI adds the DataTypes: ",
 ButtonBox["UMatrix",
  BaseStyle->"AddOnsLink",
  ButtonData:>"UMatrix",
  ButtonNote->"UMatrix"],
 ", ",
 ButtonBox["UScalar",
  BaseStyle->"AddOnsLink",
  ButtonData:>"UScalar",
  ButtonNote->"UScalar"],
 "."
}], "Text",
 CellTags->"DataType"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["DeclareNonCommutative",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DeclareNonCommutative",
  ButtonNote->"DeclareNonCommutative"],
 "."
}], "Text",
 CellTags->"DataType"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"DataType"],

Cell["NonCommutative is just a data-type.", "Text",
 CellTags->"DataType"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"DataType", "[", 
    RowBox[{"f", ",", "g", ",", " ", "NonCommutative"}], "]"}], " ", "=", " ",
    "True"}], ";"}]], "Input",
 CellTags->"DataType",
 CellLabel->"In[324]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t", "=", 
  RowBox[{
   RowBox[{"f", ".", "g"}], "-", 
   RowBox[{"g", ".", 
    RowBox[{"(", 
     RowBox[{"2", "a"}], ")"}], ".", "f"}]}]}]], "Input",
 CellTags->"DataType",
 CellLabel->"In[325]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"f", ".", "g"}], "-", 
   RowBox[{"g", ".", 
    RowBox[{"(", 
     RowBox[{"2", " ", "a"}], ")"}], ".", "f"}]}], TraditionalForm]], "Output",\

 CellChangeTimes->{3.500970191333096*^9},
 CellTags->"DataType",
 CellLabel->"Out[325]="]
}, Open  ]],

Cell["\<\
Since \"f \"and \"g\" have DataType NonCommutative the function DotSimplify \
extracts only \"a\" out of the noncommutative product.\
\>", "Text",
 CellTags->"DataType"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DotSimplify", "[", "t", "]"}]], "Input",
 CellTags->"DataType",
 CellLabel->"In[326]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"f", ".", "g"}], "-", 
   RowBox[{"2", " ", "a", " ", 
    RowBox[{"g", ".", "f"}]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970191433885*^9},
 CellTags->"DataType",
 CellLabel->"Out[326]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"DataType", "[", 
    RowBox[{"m", ",", "odd"}], "]"}], "=", 
   RowBox[{
    RowBox[{"DataType", "[", 
     RowBox[{"a", ",", "even"}], "]"}], "=", "True"}]}], ";"}]], "Input",
 CellTags->"DataType",
 CellLabel->"In[327]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"ptest1", "[", "x_", "]"}], ":=", 
   RowBox[{"x", "/.", 
    RowBox[{
     RowBox[{
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"-", "1"}], ")"}], "n_"], "/;", 
      RowBox[{"DataType", "[", 
       RowBox[{"n", ",", "odd"}], "]"}]}], "\[RuleDelayed]", 
     RowBox[{"-", "1"}]}]}]}], ";"}]], "Input",
 CellTags->"DataType",
 CellLabel->"In[328]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"ptest2", "[", "x_", "]"}], ":=", 
   RowBox[{"x", "/.", 
    RowBox[{
     RowBox[{
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"-", "1"}], ")"}], "n_"], "/;", 
      RowBox[{"DataType", "[", 
       RowBox[{"n", ",", "even"}], "]"}]}], "\[RuleDelayed]", "1"}]}]}], 
  ";"}]], "Input",
 CellTags->"DataType",
 CellLabel->"In[329]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t", "=", 
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{"-", "1"}], ")"}], "^", "m"}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"-", "1"}], ")"}], "^", "a"}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"-", "1"}], ")"}], "^", "z"}]}]}]], "Input",
 CellTags->"DataType",
 CellLabel->"In[330]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"-", "1"}], ")"}], "a"], "+", 
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"-", "1"}], ")"}], "m"], "+", 
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"-", "1"}], ")"}], "z"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970191568677*^9},
 CellTags->"DataType",
 CellLabel->"Out[330]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ptest1", "[", "t", "]"}]], "Input",
 CellTags->"DataType",
 CellLabel->"In[331]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"-", "1"}], ")"}], "a"], "+", 
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"-", "1"}], ")"}], "z"], "-", "1"}], TraditionalForm]], "Output",\

 CellChangeTimes->{3.500970191657963*^9},
 CellTags->"DataType",
 CellLabel->"Out[331]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ptest2", "[", "%", "]"}]], "Input",
 CellTags->"DataType",
 CellLabel->"In[332]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox[
   RowBox[{"(", 
    RowBox[{"-", "1"}], ")"}], "z"], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970191747169*^9},
 CellTags->"DataType",
 CellLabel->"Out[332]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", 
   RowBox[{"ptest1", ",", "ptest2", ",", "t", ",", "a", ",", "m"}], "]"}], 
  ";"}]], "Input",
 CellTags->"DataType",
 CellLabel->"In[333]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"DataType", "[", 
    RowBox[{"m", ",", "ganzeZahl"}], "]"}], "=", "True"}], ";"}]], "Input",
 CellTags->"DataType",
 CellLabel->"In[334]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"f", "[", "x_", "]"}], ":=", 
   RowBox[{"x", "/.", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{"-", "1"}], ")"}], "^", "p_"}], "/;", 
       RowBox[{"DataType", "[", 
        RowBox[{"p", ",", "ganzeZahl"}], "]"}]}], "\[RuleDelayed]", "1"}], 
     "}"}]}]}], ";"}]], "Input",
 CellTags->"DataType",
 CellLabel->"In[335]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"test", "=", 
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{"-", "1"}], ")"}], "^", "m"}], "+", 
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{"-", "1"}], ")"}], "^", "n"}], " ", "x"}]}]}]], "Input",
 CellTags->"DataType",
 CellLabel->"In[336]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"-", "1"}], ")"}], "m"], "+", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"-", "1"}], ")"}], "n"], " ", "x"}]}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.5009701919860067`*^9},
 CellTags->"DataType",
 CellLabel->"Out[336]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"f", "[", "test", "]"}]], "Input",
 CellTags->"DataType",
 CellLabel->"In[337]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"-", "1"}], ")"}], "n"], " ", "x"}], "+", "1"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.5009701921473494`*^9},
 CellTags->"DataType",
 CellLabel->"Out[337]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", 
   RowBox[{"f", ",", "test"}], "]"}], ";"}]], "Input",
 CellTags->"DataType",
 CellLabel->"In[338]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"DataType", "[", 
    RowBox[{"f", ",", "g", ",", " ", "NonCommutative"}], "]"}], " ", "=", " ",
    "False"}], ";"}]], "Input",
 CellTags->"DataType",
 CellLabel->"In[339]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"DataType", "[", 
    RowBox[{"m", ",", "odd"}], "]"}], "=", 
   RowBox[{
    RowBox[{"DataType", "[", 
     RowBox[{"a", ",", "even"}], "]"}], "=", "False"}]}], ";"}]], "Input",
 CellTags->"DataType",
 CellLabel->"In[340]:="],

Cell["\<\
Certain FeynCalc objects have DataType PositiveInteger set to True.\
\>", "Text",
 CellTags->"DataType"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DataType", "[", 
  RowBox[{"OPEm", ",", "PositiveInteger"}], "]"}]], "Input",
 CellTags->"DataType",
 CellLabel->"In[341]:="],

Cell[BoxData[
 FormBox["True", TraditionalForm]], "Output",
 CellChangeTimes->{3.500970192434671*^9},
 CellTags->"DataType",
 CellLabel->"Out[341]="]
}, Open  ]],

Cell["PowerSimplify uses the DataType information.", "Text",
 CellTags->"DataType"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PowerSimplify", "[", " ", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{"-", "1"}], ")"}], "^", 
   RowBox[{"(", 
    RowBox[{"2", "OPEm"}], ")"}]}], "]"}]], "Input",
 CellTags->"DataType",
 CellLabel->"In[342]:="],

Cell[BoxData[
 FormBox["1", TraditionalForm]], "Output",
 CellChangeTimes->{3.5009701924837303`*^9},
 CellTags->"DataType",
 CellLabel->"Out[342]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PowerSimplify", "[", " ", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{"-", " ", 
     RowBox[{"SO", "[", "q", "]"}]}], ")"}], "^", "OPEm"}], "]"}]], "Input",
 CellTags->"DataType",
 CellLabel->"In[343]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"-", "1"}], ")"}], "m"], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{
      FormBox["\<\"\[CapitalDelta]\"\>",
       TraditionalForm], 
      FormBox["\<\"\[CenterDot]\"\>",
       TraditionalForm], 
      FormBox["q",
       TraditionalForm]}], ")"}], "m"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009701925866632`*^9},
 CellTags->"DataType",
 CellLabel->"Out[343]="]
}, Open  ]]
}, Open  ]],

Cell["DB0", "ObjectName",
 CellTags->"DB0"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"DB0"],

Cell["\<\
DB0[p2, m1^2, m2^2] is the derivative of the two-point function B0[p2, m1^2, \
m2^2] with respect to p2.\
\>", "Text",
 CellTags->"DB0"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["B0",
  BaseStyle->"AddOnsLink",
  ButtonData:>"B0",
  ButtonNote->"B0"],
 "."
}], "Text",
 CellTags->"DB0"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"DB0"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"D", "[", 
  RowBox[{
   RowBox[{"B0", "[", 
    RowBox[{
     SubscriptBox["p", "2"], ",", 
     SuperscriptBox[
      SubscriptBox["m", "1"], "2"], ",", 
     SuperscriptBox[
      SubscriptBox["m", "2"], "2"]}], "]"}], ",", 
   SubscriptBox["p", "2"]}], "]"}]], "Input",
 CellTags->"DB0",
 CellLabel->"In[344]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"DB0", "(", 
   RowBox[{
    SubscriptBox["p", "2"], ",", 
    SubsuperscriptBox["m", "1", "2"], ",", 
    SubsuperscriptBox["m", "2", "2"]}], ")"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970192859435*^9},
 CellTags->"DB0",
 CellLabel->"Out[344]="]
}, Open  ]]
}, Open  ]],

Cell["DB1", "ObjectName",
 CellTags->"DB1"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"DB1"],

Cell["\<\
DB1[p2,m1^2,m2^2] is the derivative of B1[p2,m1^2,m2^2] with respect to p2.\
\>", "Text",
 CellTags->"DB1"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["B1",
  BaseStyle->"AddOnsLink",
  ButtonData:>"B1",
  ButtonNote->"B1"],
 "."
}], "Text",
 CellTags->"DB1"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"DB1"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"D", "[", 
  RowBox[{
   RowBox[{"B1", "[", 
    RowBox[{
     SubscriptBox["p", "2"], ",", 
     SuperscriptBox[
      SubscriptBox["m", "1"], "2"], ",", 
     SuperscriptBox[
      SubscriptBox["m", "2"], "2"]}], "]"}], ",", 
   SubscriptBox["p", "2"]}], "]"}]], "Input",
 CellTags->"DB1",
 CellLabel->"In[345]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    FractionBox[
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        SubsuperscriptBox["m", "2", "2"], "-", 
        SubsuperscriptBox["m", "1", "2"]}], ")"}], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         SubscriptBox["B", "0"], "(", 
         SubscriptBox["p", "2"], ",", 
         SubsuperscriptBox["m", "1", "2"], ",", 
         SubsuperscriptBox["m", "2", "2"], ")"}], "-", 
        RowBox[{
         SubscriptBox["B", "0"], "(", "0", ",", 
         SubsuperscriptBox["m", "1", "2"], ",", 
         SubsuperscriptBox["m", "2", "2"], ")"}]}], ")"}]}], 
     RowBox[{"2", " ", 
      SubsuperscriptBox["p", "2", "2"]}]]}], "+", 
   FractionBox[
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       SubsuperscriptBox["m", "2", "2"], "-", 
       SubsuperscriptBox["m", "1", "2"]}], ")"}], " ", 
     RowBox[{"DB0", "(", 
      RowBox[{
       SubscriptBox["p", "2"], ",", 
       SubsuperscriptBox["m", "1", "2"], ",", 
       SubsuperscriptBox["m", "2", "2"]}], ")"}]}], 
    RowBox[{"2", " ", 
     SubscriptBox["p", "2"]}]], "-", 
   RowBox[{
    FractionBox["1", "2"], " ", 
    RowBox[{"DB0", "(", 
     RowBox[{
      SubscriptBox["p", "2"], ",", 
      SubsuperscriptBox["m", "1", "2"], ",", 
      SubsuperscriptBox["m", "2", "2"]}], ")"}]}]}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970193204358*^9},
 CellTags->"DB1",
 CellLabel->"Out[345]="]
}, Open  ]]
}, Open  ]],

Cell["DeclareNonCommutative", "ObjectName",
 CellTags->"DeclareNonCommutative"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"DeclareNonCommutative"],

Cell["\<\
DeclareNonCommutative[a, b, ...] declares a,b, ... to be non-commutative, \
i.e., DataType[a,b, ..., NonCommutative] is set to True.\
\>", "Text",
 CellTags->"DeclareNonCommutative"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["DataType",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DataType",
  ButtonNote->"DataType"],
 ", ",
 ButtonBox["UnDeclareNonCommutative",
  BaseStyle->"AddOnsLink",
  ButtonData:>"UnDeclareNonCommutative",
  ButtonNote->"UnDeclareNonCommutative"],
 "."
}], "Text",
 CellTags->"DeclareNonCommutative"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"DeclareNonCommutative"],

Cell[BoxData[
 RowBox[{"DeclareNonCommutative", "[", "x", "]"}]], "Input",
 CellTags->"DeclareNonCommutative",
 CellLabel->"In[346]:="],

Cell["\<\
As a side effect of DeclareNonCommutative x is declared to be of data type \
NonCommutative.\
\>", "Text",
 CellTags->"DeclareNonCommutative"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DataType", "[", 
  RowBox[{"x", ",", "NonCommutative"}], "]"}]], "Input",
 CellTags->"DeclareNonCommutative",
 CellLabel->"In[347]:="],

Cell[BoxData[
 FormBox["True", TraditionalForm]], "Output",
 CellChangeTimes->{3.5009701935415707`*^9},
 CellTags->"DeclareNonCommutative",
 CellLabel->"Out[347]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"DeclareNonCommutative", "[", 
  RowBox[{"y", ",", "z"}], "]"}]], "Input",
 CellTags->"DeclareNonCommutative",
 CellLabel->"In[348]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DataType", "[", 
  RowBox[{"a", ",", "x", ",", "y", ",", "z", ",", "NonCommutative"}], 
  "]"}]], "Input",
 CellTags->"DeclareNonCommutative",
 CellLabel->"In[349]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"True", ",", "True", ",", "True", ",", "True"}], "}"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970193745144*^9},
 CellTags->"DeclareNonCommutative",
 CellLabel->"Out[349]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"UnDeclareNonCommutative", "[", 
  RowBox[{"x", ",", "y", ",", "z"}], "]"}]], "Input",
 CellTags->"DeclareNonCommutative",
 CellLabel->"In[350]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DataType", "[", 
  RowBox[{"a", ",", "x", ",", "y", ",", "z", ",", "NonCommutative"}], 
  "]"}]], "Input",
 CellTags->"DeclareNonCommutative",
 CellLabel->"In[351]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"True", ",", "True", ",", "True", ",", "True"}], "}"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970193915197*^9},
 CellTags->"DeclareNonCommutative",
 CellLabel->"Out[351]="]
}, Open  ]]
}, Open  ]],

Cell["DeltaFunction", "ObjectName",
 CellTags->"DeltaFunction"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"DeltaFunction"],

Cell[TextData[{
 " DeltaFunction[x] is the Dirac delta-function ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Delta]", "(", "x", ")"}], TraditionalForm]]],
 ".\n*** After version 4 of ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ", there is a built-in function, DiracDelta, with comparable properties. \
***"
}], "Text",
 CellTags->"DeltaFunction"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Convolute",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Convolute",
  ButtonNote->"Convolute"],
 ", ",
 ButtonBox["DeltaFunctionPrime",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DeltaFunctionPrime",
  ButtonNote->"DeltaFunctionPrime"],
 ", ",
 ButtonBox["Integrate2",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Integrate2",
  ButtonNote->"Integrate2"],
 ", ",
 ButtonBox["SimplifyDeltaFunction",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SimplifyDeltaFunction",
  ButtonNote->"SimplifyDeltaFunction"],
 "."
}], "Text",
 CellTags->"DeltaFunction"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"DeltaFunction"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DeltaFunction", "[", 
  RowBox[{"1", "-", "x"}], "]"}]], "Input",
 CellTags->"DeltaFunction",
 CellLabel->"In[352]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"\[Delta]", "(", 
   FormBox[
    RowBox[{"1", "-", "x"}],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970194197627*^9},
 CellTags->"DeltaFunction",
 CellLabel->"Out[352]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Integrate2", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"DeltaFunction", "[", 
     RowBox[{"1", "-", "x"}], "]"}], " ", 
    RowBox[{"f", "[", "x", "]"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellTags->"DeltaFunction",
 CellLabel->"In[353]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"f", "(", "1", ")"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970194548355*^9},
 CellTags->"DeltaFunction",
 CellLabel->"Out[353]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Integrate2", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"DeltaFunction", "[", "x", "]"}], " ", 
    RowBox[{"f", "[", "x", "]"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellTags->"DeltaFunction",
 CellLabel->"In[354]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"f", "(", "0", ")"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970194648967*^9},
 CellTags->"DeltaFunction",
 CellLabel->"Out[354]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Integrate2", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"DeltaFunction", "[", 
     RowBox[{"1", "-", "x"}], "]"}], " ", 
    RowBox[{"f", "[", "x", "]"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellTags->"DeltaFunction",
 CellLabel->"In[355]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"f", "(", "1", ")"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970194842451*^9},
 CellTags->"DeltaFunction",
 CellLabel->"Out[355]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Convolute", "[", 
  RowBox[{
   RowBox[{"DeltaFunction", "[", 
    RowBox[{"1", "-", "x"}], "]"}], ",", "x"}], "]"}]], "Input",
 CellTags->"DeltaFunction",
 CellLabel->"In[356]:="],

Cell[BoxData[
 FormBox["x", TraditionalForm]], "Output",
 CellChangeTimes->{3.5009701949425383`*^9},
 CellTags->"DeltaFunction",
 CellLabel->"Out[356]="]
}, Open  ]]
}, Open  ]],

Cell["DeltaFunctionDoublePrime", "ObjectName",
 CellTags->"DeltaFunctionDoublePrime"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"DeltaFunctionDoublePrime"],

Cell[TextData[{
 " DeltaFunctionDoublePrime[1-x] is the second derivative of the Dirac \
delta-function ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Delta]", "(", "x", ")"}], TraditionalForm]]],
 "."
}], "Text",
 CellTags->"DeltaFunctionDoublePrime"],

Cell[TextData[{
 "See also: ",
 ButtonBox["DeltaFunctionPrime",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DeltaFunctionPrime",
  ButtonNote->"DeltaFunctionPrime"],
 "."
}], "Text",
 CellTags->"DeltaFunctionDoublePrime"]
}, Open  ]],

Cell["DeltaFunctionPrime", "ObjectName",
 CellTags->"DeltaFunctionPrime"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"DeltaFunctionPrime"],

Cell[TextData[{
 " DeltaFunctionPrime[1-x] is the derivative of the Dirac delta-function ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Delta]", "(", "x", ")"}], TraditionalForm]]],
 "."
}], "Text",
 CellTags->"DeltaFunctionPrime"],

Cell[TextData[{
 "See also: ",
 ButtonBox["Convolute",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Convolute",
  ButtonNote->"Convolute"],
 ", ",
 ButtonBox["DeltaFunction",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DeltaFunction",
  ButtonNote->"DeltaFunction"],
 ", ",
 ButtonBox["Integrate2",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Integrate2",
  ButtonNote->"Integrate2"],
 ", ",
 ButtonBox["SimplifyDeltaFunction",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SimplifyDeltaFunction",
  ButtonNote->"SimplifyDeltaFunction"],
 "."
}], "Text",
 CellTags->"DeltaFunctionPrime"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"DeltaFunctionPrime"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DeltaFunctionPrime", "[", 
  RowBox[{"1", "-", "x"}], "]"}]], "Input",
 CellTags->"DeltaFunctionPrime",
 CellLabel->"In[357]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Delta]", "\[Prime]"], "(", 
   FormBox[
    RowBox[{"1", "-", "x"}],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970195424926*^9},
 CellTags->"DeltaFunctionPrime",
 CellLabel->"Out[357]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Integrate2", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"DeltaFunctionPrime", "[", 
     RowBox[{"1", "-", "x"}], "]"}], " ", 
    RowBox[{"f", "[", "x", "]"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellTags->"DeltaFunctionPrime",
 CellLabel->"In[358]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["f", "\[Prime]",
    MultilineFunction->None], "(", "1", ")"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009701958671293`*^9},
 CellTags->"DeltaFunctionPrime",
 CellLabel->"Out[358]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Integrate2", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"DeltaFunctionPrime", "[", 
     RowBox[{"1", "-", "x"}], "]"}], " ", 
    SuperscriptBox["x", "2"]}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellTags->"DeltaFunctionPrime",
 CellLabel->"In[359]:="],

Cell[BoxData[
 FormBox["2", TraditionalForm]], "Output",
 CellChangeTimes->{3.500970196055023*^9},
 CellTags->"DeltaFunctionPrime",
 CellLabel->"Out[359]="]
}, Open  ]]
}, Open  ]],

Cell["DenominatorOrder", "ObjectName",
 CellTags->"DenominatorOrder"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"DenominatorOrder"],

Cell["\<\
DenominatorOrder is an option for OneLoop, if set to True the \
PropagatorDenominator will be ordered in a standard way.\
\>", "Text",
 CellTags->"DenominatorOrder"],

Cell[TextData[{
 "See also: ",
 ButtonBox["OneLoop",
  BaseStyle->"AddOnsLink",
  ButtonData:>"OneLoop",
  ButtonNote->"OneLoop"],
 ", ",
 ButtonBox["PropagatorDenominator",
  BaseStyle->"AddOnsLink",
  ButtonData:>"PropagatorDenominator",
  ButtonNote->"PropagatorDenominator"],
 "."
}], "Text",
 CellTags->"DenominatorOrder"]
}, Open  ]],

Cell["Dimension", "ObjectName",
 CellTags->"Dimension"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Dimension"],

Cell["\<\
Dimension is an option of several functions and denotes the number of \
space-time dimensions. Possible settings are: 4, n, d, D, ... ,the variable \
does not matter, but it should have Head Symbol.\
\>", "Text",
 CellTags->"Dimension"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "MetricTensor", "]"}]], "Input",
 CellTags->"Dimension",
 CellLabel->"In[360]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Dimension", "\[Rule]", "4"}], ",", 
    RowBox[{"FeynCalcInternal", "\[Rule]", "False"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970196188274*^9},
 CellTags->"Dimension",
 CellLabel->"Out[360]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"MetricTensor", "[", 
   RowBox[{"m", ",", "n", ",", 
    RowBox[{"Dimension", "\[Rule]", "d"}]}], "]"}], 
  RowBox[{"DiracMatrix", "[", 
   RowBox[{"\[Alpha]", ",", 
    RowBox[{"Dimension", "\[Rule]", "d"}]}], "]"}]}]], "Input",
 CellTags->"Dimension",
 CellLabel->"In[361]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Alpha]",
      TraditionalForm],
     TraditionalForm]], " ", 
   SuperscriptBox["g", 
    RowBox[{"m", "n"}]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970196327963*^9},
 CellTags->"Dimension",
 CellLabel->"Out[361]="]
}, Open  ]],

Cell["\<\
The dimension of the indices is not shown by default but can be inspected \
easily.\
\>", "Text",
 CellTags->"Dimension"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"MetricTensor", "[", 
    RowBox[{"m", ",", "n", ",", 
     RowBox[{"Dimension", "\[Rule]", "d"}]}], "]"}], 
   RowBox[{"DiracMatrix", "[", 
    RowBox[{"\[Alpha]", ",", 
     RowBox[{"Dimension", "\[Rule]", "d"}]}], "]"}]}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"Dimension",
 CellLabel->"In[362]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"DiracGamma", "[", 
   RowBox[{
    RowBox[{"LorentzIndex", "[", 
     RowBox[{"\[Alpha]", ",", "d"}], "]"}], ",", "d"}], "]"}], " ", 
  RowBox[{"MetricTensor", "[", 
   RowBox[{"m", ",", "n", ",", 
    RowBox[{"Dimension", "\[Rule]", "d"}]}], "]"}]}]], "Output",
 CellChangeTimes->{3.500970196477008*^9},
 CellTags->"Dimension",
 CellLabel->"Out[362]//StandardForm="]
}, Open  ]],

Cell["\<\
Setting the global variable $LorentzIndices to True will display the \
dimension (if different from 4) as a subscript.\
\>", "Text",
 CellTags->"Dimension"],

Cell[BoxData[
 RowBox[{
  RowBox[{"$LorentzIndices", "=", "True"}], ";"}]], "Input",
 CellTags->"Dimension",
 CellLabel->"In[363]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"MetricTensor", "[", 
   RowBox[{"\[Alpha]", ",", "\[Beta]", ",", 
    RowBox[{"Dimension", "\[Rule]", "n"}]}], "]"}], 
  RowBox[{"DiracMatrix", "[", 
   RowBox[{"\[Alpha]", ",", 
    RowBox[{"Dimension", "\[Rule]", "n"}]}], "]"}]}]], "Input",
 CellTags->"Dimension",
 CellLabel->"In[364]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", 
    FormBox[
     SubscriptBox[
      FormBox["\[Alpha]",
       TraditionalForm], 
      FormBox["n",
       TraditionalForm]],
     TraditionalForm]], " ", 
   SuperscriptBox["g", 
    RowBox[{
     FormBox[
      SubscriptBox[
       FormBox["\[Alpha]",
        TraditionalForm], 
       FormBox["n",
        TraditionalForm]],
      TraditionalForm], 
     FormBox[
      SubscriptBox[
       FormBox["\[Beta]",
        TraditionalForm], 
       FormBox["n",
        TraditionalForm]],
      TraditionalForm]}]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009701966008387`*^9},
 CellTags->"Dimension",
 CellLabel->"Out[364]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "StandardForm"}]], "Input",
 CellTags->"Dimension",
 CellLabel->"In[365]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"DiracGamma", "[", 
   RowBox[{
    RowBox[{"LorentzIndex", "[", 
     RowBox[{"\[Alpha]", ",", "n"}], "]"}], ",", "n"}], "]"}], " ", 
  RowBox[{"MetricTensor", "[", 
   RowBox[{"\[Alpha]", ",", "\[Beta]", ",", 
    RowBox[{"Dimension", "\[Rule]", "n"}]}], "]"}]}]], "Output",
 CellChangeTimes->{3.500970196671088*^9},
 CellTags->"Dimension",
 CellLabel->"Out[365]//StandardForm="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"$LorentzIndices", "=", "False"}], ";"}]], "Input",
 CellTags->"Dimension",
 CellLabel->"In[366]:="]
}, Open  ]],

Cell["DimensionalReduction", "ObjectName",
 CellTags->"DimensionalReduction"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"DimensionalReduction"],

Cell["\<\
DimensionalReduction is an option for TID and OneLoopSimplify.\
\>", "Text",
 CellTags->"DimensionalReduction"],

Cell[TextData[{
 "See also: ",
 ButtonBox["TID",
  BaseStyle->"AddOnsLink",
  ButtonData:>"TID",
  ButtonNote->"TID"],
 ", ",
 ButtonBox["OneLoopSimplify",
  BaseStyle->"AddOnsLink",
  ButtonData:>"OneLoopSimplify",
  ButtonNote->"OneLoopSimplify"],
 "."
}], "Text",
 CellTags->"DimensionalReduction"]
}, Open  ]],

Cell["DiracBasis", "ObjectName",
 CellTags->"DiracBasis"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"DiracBasis"],

Cell["\<\
DiracBasis[any] is a head which is wrapped around Dirac structures (and the \
1) as a result of the function DiracReduce. Eventually you want to substitute \
DiracBasis by Identity (or set: DiracBasis[1] = S; \
DiracBasis[DiracMatrix[mu]] = P; etc.).\
\>", "Text",
 CellTags->"DiracBasis"],

Cell[TextData[{
 "See also: ",
 ButtonBox["DiracReduce",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracReduce",
  ButtonNote->"DiracReduce"],
 "."
}], "Text",
 CellTags->"DiracBasis"]
}, Open  ]],

Cell["DiracCanonical", "ObjectName",
 CellTags->"DiracCanonical"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"DiracCanonical"],

Cell["\<\
DiracCanonical is an option for DiracSimplify. If set to True DiracSimplify \
uses the function DiracOrder internally.\
\>", "Text",
 CellTags->"DiracCanonical"],

Cell[TextData[{
 "See also: ",
 ButtonBox["DiracSimplify",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracSimplify",
  ButtonNote->"DiracSimplify"],
 "."
}], "Text",
 CellTags->"DiracCanonical"]
}, Open  ]],

Cell["DiracEquation", "ObjectName",
 CellTags->"DiracEquation"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"DiracEquation"],

Cell["\<\
DiracEquation[exp] applies the Dirac equation without expanding exp. If that \
is needed, use DiracSimplify.\
\>", "Text",
 CellTags->"DiracEquation"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["DiracSimplify",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracSimplify",
  ButtonNote->"DiracSimplify"],
 "."
}], "Text",
 CellTags->"DiracEquation"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"DiracEquation"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Spinor", "[", 
   RowBox[{
    RowBox[{"Momentum", "[", "p", "]"}], ",", "m", ",", "1"}], "]"}], ".", 
  RowBox[{"DiracGamma", "[", 
   RowBox[{"Momentum", "[", "p", "]"}], "]"}]}]], "Input",
 CellTags->"DiracEquation",
 CellLabel->"In[367]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"\[CurlyPhi]", "(", 
    FormBox["p",
     TraditionalForm], ",", "m", ")"}], ".", 
   RowBox[{"(", 
    RowBox[{
     FormBox["\<\"\[Gamma]\"\>",
      TraditionalForm], "\[CenterDot]", 
     FormBox["p",
      TraditionalForm]}], ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970196811715*^9},
 CellTags->"DiracEquation",
 CellLabel->"Out[367]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracEquation", "[", "%", "]"}]], "Input",
 CellTags->"DiracEquation",
 CellLabel->"In[368]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"m", " ", 
   RowBox[{"\[CurlyPhi]", "(", 
    FormBox["p",
     TraditionalForm], ",", "m", ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009701968945417`*^9},
 CellTags->"DiracEquation",
 CellLabel->"Out[368]="]
}, Open  ]]
}, Open  ]],

Cell["DiracGamma", "ObjectName",
 CellTags->"DiracGamma"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"DiracGamma"],

Cell[TextData[{
 "DiracGamma[x, dim] is the head of all Dirac matrices and slashes (in the \
internal representation). Use DiracMatrix (or GA, GAD) and DiracSlash (or GS, \
GSD) for manual (short) input. DiracGamma[x, 4] simplifies to DiracGamma[x]. \
DiracGamma[5] is ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["\[Gamma]", "5"], TraditionalForm]]],
 ". DiracGamma[6] is ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"(", 
     RowBox[{"1", "+", 
      SuperscriptBox["\[Gamma]", "5"]}], ")"}], "/", "2."}], 
   TraditionalForm]]],
 " DiracGamma[7] is ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"(", 
     RowBox[{"1", "-", 
      SuperscriptBox["\[Gamma]", "5"]}], ")"}], "/", "2."}], TraditionalForm]]]
}], "Text",
 CellTags->"DiracGamma"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["DiracGammaExpand",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracGammaExpand",
  ButtonNote->"DiracGammaExpand"],
 ", ",
 ButtonBox["DiracMatrix",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracMatrix",
  ButtonNote->"DiracMatrix"],
 ", ",
 ButtonBox["DiracSimplify",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracSimplify",
  ButtonNote->"DiracSimplify"],
 ", ",
 ButtonBox["DiracSlash",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracSlash",
  ButtonNote->"DiracSlash"],
 ", ",
 ButtonBox["DiracTrick",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracTrick",
  ButtonNote->"DiracTrick"],
 "."
}], "Text",
 CellTags->"DiracGamma"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"DiracGamma"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracGamma", "[", "5", "]"}]], "Input",
 CellTags->"DiracGamma",
 CellLabel->"In[369]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["\[Gamma]", "5"], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970197034642*^9},
 CellTags->"DiracGamma",
 CellLabel->"Out[369]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracGamma", "[", 
  RowBox[{"LorentzIndex", "[", "\[Alpha]", "]"}], "]"}]], "Input",
 CellTags->"DiracGamma",
 CellLabel->"In[370]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["\[Gamma]", 
   FormBox[
    FormBox["\[Alpha]",
     TraditionalForm],
    TraditionalForm]], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970197115032*^9},
 CellTags->"DiracGamma",
 CellLabel->"Out[370]="]
}, Open  ]],

Cell[TextData[{
 "A Dirac-slash, i.e., ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["\[Gamma]", "\[Mu]"], 
    SubscriptBox["q", "\[Mu]"]}], TraditionalForm]]],
 ", is displayed as ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Gamma]", "\[CenterDot]", "q"}], TraditionalForm]]],
 "."
}], "Text",
 CellTags->"DiracGamma"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"DiracGamma", "[", 
   RowBox[{"Momentum", "[", "q", "]"}], "]"}], " "}]], "Input",
 CellTags->"DiracGamma",
 CellLabel->"In[371]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox["\<\"\[Gamma]\"\>",
    TraditionalForm], "\[CenterDot]", 
   FormBox["q",
    TraditionalForm]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009701973058577`*^9},
 CellTags->"DiracGamma",
 CellLabel->"Out[371]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"DiracGamma", "[", 
   RowBox[{"Momentum", "[", "q", "]"}], "]"}], " ", ".", " ", 
  RowBox[{"DiracGamma", "[", 
   RowBox[{"Momentum", "[", 
    RowBox[{"p", "-", "q"}], "]"}], "]"}]}]], "Input",
 CellTags->"DiracGamma",
 CellLabel->"In[372]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     FormBox["\<\"\[Gamma]\"\>",
      TraditionalForm], "\[CenterDot]", 
     FormBox["q",
      TraditionalForm]}], ")"}], ".", 
   RowBox[{"(", 
    RowBox[{
     FormBox["\<\"\[Gamma]\"\>",
      TraditionalForm], "\[CenterDot]", "(", 
     FormBox[
      RowBox[{"p", "-", "q"}],
      TraditionalForm], ")"}], ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009701974010677`*^9},
 CellTags->"DiracGamma",
 CellLabel->"Out[372]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"DiracGamma", "[", 
   RowBox[{
    RowBox[{"Momentum", "[", 
     RowBox[{"q", ",", "D"}], "]"}], ",", "D"}], "]"}], " "}]], "Input",
 CellTags->"DiracGamma",
 CellLabel->"In[373]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox["\<\"\[Gamma]\"\>",
    TraditionalForm], "\[CenterDot]", 
   FormBox["q",
    TraditionalForm]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970197453108*^9},
 CellTags->"DiracGamma",
 CellLabel->"Out[373]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"a1", "=", 
  RowBox[{
   RowBox[{"GS", "[", 
    RowBox[{"p", "-", "q"}], "]"}], ".", 
   RowBox[{"GS", "[", "p", "]"}]}]}]], "Input",
 CellTags->"DiracGamma",
 CellLabel->"In[374]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"(", 
    RowBox[{"\[Gamma]", "\[CenterDot]", "(", 
     FormBox[
      RowBox[{"p", "-", "q"}],
      TraditionalForm], ")"}], ")"}], ".", 
   RowBox[{"(", 
    RowBox[{"\[Gamma]", "\[CenterDot]", "p"}], ")"}]}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970197505131*^9},
 CellTags->"DiracGamma",
 CellLabel->"Out[374]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"a2", "=", 
  RowBox[{"DiracGammaExpand", "[", "a1", "]"}]}]], "Input",
 CellTags->"DiracGamma",
 CellLabel->"In[375]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      FormBox["\<\"\[Gamma]\"\>",
       TraditionalForm], "\[CenterDot]", 
      FormBox["p",
       TraditionalForm]}], "-", 
     RowBox[{
      FormBox["\<\"\[Gamma]\"\>",
       TraditionalForm], "\[CenterDot]", 
      FormBox["q",
       TraditionalForm]}]}], ")"}], ".", 
   RowBox[{"(", 
    RowBox[{
     FormBox["\<\"\[Gamma]\"\>",
      TraditionalForm], "\[CenterDot]", 
     FormBox["p",
      TraditionalForm]}], ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.50097019758916*^9},
 CellTags->"DiracGamma",
 CellLabel->"Out[375]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"a3", "=", 
  RowBox[{
   RowBox[{"GAD", "[", "\[Mu]", "]"}], ".", 
   RowBox[{"GSD", "[", 
    RowBox[{"p", "-", "q"}], "]"}], ".", 
   RowBox[{"GSD", "[", "q", "]"}], ".", 
   RowBox[{"GAD", "[", "\[Mu]", "]"}]}]}]], "Input",
 CellTags->"DiracGamma",
 CellLabel->"In[376]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
   RowBox[{"(", 
    RowBox[{"\[Gamma]", "\[CenterDot]", "(", 
     FormBox[
      RowBox[{"p", "-", "q"}],
      TraditionalForm], ")"}], ")"}], ".", 
   RowBox[{"(", 
    RowBox[{"\[Gamma]", "\[CenterDot]", "q"}], ")"}], ".", 
   SuperscriptBox["\[Gamma]", "\[Mu]"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970197700286*^9},
 CellTags->"DiracGamma",
 CellLabel->"Out[376]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"a4", "=", 
  RowBox[{"DiracTrick", "[", "a3", "]"}]}]], "Input",
 CellTags->"DiracGamma",
 CellLabel->"In[377]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
   RowBox[{"(", 
    RowBox[{"\[Gamma]", "\[CenterDot]", "(", 
     FormBox[
      RowBox[{"p", "-", "q"}],
      TraditionalForm], ")"}], ")"}], ".", 
   RowBox[{"(", 
    RowBox[{"\[Gamma]", "\[CenterDot]", "q"}], ")"}], ".", 
   SuperscriptBox["\[Gamma]", "\[Mu]"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009701977840977`*^9},
 CellTags->"DiracGamma",
 CellLabel->"Out[377]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"a5", "=", 
  RowBox[{"DiracSimplify", "[", "a4", "]"}]}]], "Input",
 CellTags->"DiracGamma",
 CellLabel->"In[378]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"D", " ", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       FormBox["\<\"\[Gamma]\"\>",
        TraditionalForm], "\[CenterDot]", 
       FormBox["p",
        TraditionalForm]}], ")"}], ".", 
     RowBox[{"(", 
      RowBox[{
       FormBox["\<\"\[Gamma]\"\>",
        TraditionalForm], "\[CenterDot]", 
       FormBox["q",
        TraditionalForm]}], ")"}]}]}], "-", 
   RowBox[{"D", " ", 
    SuperscriptBox[
     FormBox["q",
      TraditionalForm], "2"]}], "-", 
   RowBox[{"4", " ", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       FormBox["\<\"\[Gamma]\"\>",
        TraditionalForm], "\[CenterDot]", 
       FormBox["p",
        TraditionalForm]}], ")"}], ".", 
     RowBox[{"(", 
      RowBox[{
       FormBox["\<\"\[Gamma]\"\>",
        TraditionalForm], "\[CenterDot]", 
       FormBox["q",
        TraditionalForm]}], ")"}]}]}], "+", 
   RowBox[{"4", " ", 
    RowBox[{
     FormBox["p",
      TraditionalForm], "\[CenterDot]", 
     FormBox["q",
      TraditionalForm]}]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009701979601707`*^9},
 CellTags->"DiracGamma",
 CellLabel->"Out[378]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"Clear", "[", 
  RowBox[{"a1", ",", "a2", ",", "a3", ",", "a4", ",", "a5"}], "]"}]], "Input",\

 CellTags->"DiracGamma",
 CellLabel->"In[379]:="]
}, Open  ]],

Cell["DiracGammaCombine", "ObjectName",
 CellTags->"DiracGammaCombine"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"DiracGammaCombine"],

Cell["\<\
DiracGammaCombine[exp] is (nearly) the inverse operation to DiracGammaExpand.\
\
\>", "Text",
 CellTags->"DiracGammaCombine"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["DiracGamma",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracGamma",
  ButtonNote->"DiracGamma"],
 ", ",
 ButtonBox["DiracGammaExpand",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracGammaExpand",
  ButtonNote->"DiracGammaExpand"],
 ", ",
 ButtonBox["DiracMatrix",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracMatrix",
  ButtonNote->"DiracMatrix"],
 ", ",
 ButtonBox["DiracSimplify",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracSimplify",
  ButtonNote->"DiracSimplify"],
 ", ",
 ButtonBox["DiracSlash",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracSlash",
  ButtonNote->"DiracSlash"],
 ", ",
 ButtonBox["DiracTrick",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracTrick",
  ButtonNote->"DiracTrick"],
 "."
}], "Text",
 CellTags->"DiracGammaCombine"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"DiracGammaCombine"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracGammaCombine", "[", 
  RowBox[{
   RowBox[{"GS", "[", "p", "]"}], " ", "+", " ", 
   RowBox[{"GS", "[", "q", "]"}]}], "]"}]], "Input",
 CellTags->"DiracGammaCombine",
 CellLabel->"In[380]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox["\<\"\[Gamma]\"\>",
    TraditionalForm], "\[CenterDot]", "(", 
   FormBox[
    RowBox[{"p", "+", "q"}],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970198198633*^9},
 CellTags->"DiracGammaCombine",
 CellLabel->"Out[380]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StandardForm", "[", "%", "]"}]], "Input",
 CellTags->"DiracGammaCombine",
 CellLabel->"In[381]:="],

Cell[BoxData[
 RowBox[{"DiracGamma", "[", 
  RowBox[{"Momentum", "[", 
   RowBox[{"p", "+", "q"}], "]"}], "]"}]], "Output",
 CellChangeTimes->{3.5009701985363626`*^9},
 CellTags->"DiracGammaCombine",
 CellLabel->"Out[381]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracGammaCombine", "[", 
  RowBox[{
   RowBox[{"2", " ", 
    RowBox[{"GS", "[", "p", "]"}]}], " ", "-", " ", 
   RowBox[{"2", " ", 
    RowBox[{"GS", "[", "q", "]"}]}]}], "]"}]], "Input",
 CellTags->"DiracGammaCombine",
 CellLabel->"In[382]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox["\<\"\[Gamma]\"\>",
    TraditionalForm], "\[CenterDot]", "(", 
   FormBox[
    RowBox[{
     RowBox[{"2", " ", "p"}], "-", 
     RowBox[{"2", " ", "q"}]}],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970198700313*^9},
 CellTags->"DiracGammaCombine",
 CellLabel->"Out[382]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StandardForm", "[", "%", "]"}]], "Input",
 CellTags->"DiracGammaCombine",
 CellLabel->"In[383]:="],

Cell[BoxData[
 RowBox[{"DiracGamma", "[", 
  RowBox[{"Momentum", "[", 
   RowBox[{
    RowBox[{"2", " ", "p"}], "-", 
    RowBox[{"2", " ", "q"}]}], "]"}], "]"}]], "Output",
 CellChangeTimes->{3.500970198850799*^9},
 CellTags->"DiracGammaCombine",
 CellLabel->"Out[383]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracGammaExpand", "[", "%%", "]"}]], "Input",
 CellTags->"DiracGammaCombine",
 CellLabel->"In[384]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"2", " ", 
    RowBox[{
     FormBox["\<\"\[Gamma]\"\>",
      TraditionalForm], "\[CenterDot]", 
     FormBox["p",
      TraditionalForm]}]}], "-", 
   RowBox[{"2", " ", 
    RowBox[{
     FormBox["\<\"\[Gamma]\"\>",
      TraditionalForm], "\[CenterDot]", 
     FormBox["q",
      TraditionalForm]}]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009701990037107`*^9},
 CellTags->"DiracGammaCombine",
 CellLabel->"Out[384]="]
}, Open  ]]
}, Open  ]],

Cell["DiracGammaExpand", "ObjectName",
 CellTags->"DiracGammaExpand"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"DiracGammaExpand"],

Cell["\<\
DiracGammaExpand[exp] expands all DiracGamma[Momentum[a+b+..]] in exp into \
(DiracGamma[Momentum[a]] + DiracGamma[Momentum[b]] + ...).\
\>", "Text",
 CellTags->"DiracGammaExpand"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["DiracGamma",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracGamma",
  ButtonNote->"DiracGamma"],
 ", ",
 ButtonBox["DiracGammaCombine",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracGammaCombine",
  ButtonNote->"DiracGammaCombine"],
 ", ",
 ButtonBox["DiracMatrix",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracMatrix",
  ButtonNote->"DiracMatrix"],
 ", ",
 ButtonBox["DiracSimplify",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracSimplify",
  ButtonNote->"DiracSimplify"],
 ", ",
 ButtonBox["DiracSlash",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracSlash",
  ButtonNote->"DiracSlash"],
 ", ",
 ButtonBox["DiracTrick",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracTrick",
  ButtonNote->"DiracTrick"],
 "."
}], "Text",
 CellTags->"DiracGammaExpand"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"DiracGammaExpand"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t", "=", 
  RowBox[{
   RowBox[{"DiracGamma", "[", 
    RowBox[{"Momentum", "[", "q", "]"}], "]"}], " ", ".", " ", 
   RowBox[{"DiracGamma", "[", 
    RowBox[{"Momentum", "[", 
     RowBox[{"p", "-", "q"}], "]"}], "]"}]}]}]], "Input",
 CellTags->"DiracGammaExpand",
 CellLabel->"In[385]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     FormBox["\<\"\[Gamma]\"\>",
      TraditionalForm], "\[CenterDot]", 
     FormBox["q",
      TraditionalForm]}], ")"}], ".", 
   RowBox[{"(", 
    RowBox[{
     FormBox["\<\"\[Gamma]\"\>",
      TraditionalForm], "\[CenterDot]", "(", 
     FormBox[
      RowBox[{"p", "-", "q"}],
      TraditionalForm], ")"}], ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970199162677*^9},
 CellTags->"DiracGammaExpand",
 CellLabel->"Out[385]="]
}, Open  ]],

Cell["\<\
Momentum is the head of p-q, i.e., it is treated as one four-momentum.\
\>", "Text",
 CellTags->"DiracGammaExpand"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StandardForm", "[", "t", "]"}]], "Input",
 CellTags->"DiracGammaExpand",
 CellLabel->"In[386]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"DiracGamma", "[", 
   RowBox[{"Momentum", "[", "q", "]"}], "]"}], ".", 
  RowBox[{"DiracGamma", "[", 
   RowBox[{"Momentum", "[", 
    RowBox[{"p", "-", "q"}], "]"}], "]"}]}]], "Output",
 CellChangeTimes->{3.500970199211751*^9},
 CellTags->"DiracGammaExpand",
 CellLabel->"Out[386]//StandardForm="]
}, Open  ]],

Cell["With DiracGammaExpand the Momentum[p-q] gets expanded.", "Text",
 CellTags->"DiracGammaExpand"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracGammaExpand", "[", "t", "]"}]], "Input",
 CellTags->"DiracGammaExpand",
 CellLabel->"In[387]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     FormBox["\<\"\[Gamma]\"\>",
      TraditionalForm], "\[CenterDot]", 
     FormBox["q",
      TraditionalForm]}], ")"}], ".", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      FormBox["\<\"\[Gamma]\"\>",
       TraditionalForm], "\[CenterDot]", 
      FormBox["p",
       TraditionalForm]}], "-", 
     RowBox[{
      FormBox["\<\"\[Gamma]\"\>",
       TraditionalForm], "\[CenterDot]", 
      FormBox["q",
       TraditionalForm]}]}], ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970199293057*^9},
 CellTags->"DiracGammaExpand",
 CellLabel->"Out[387]="]
}, Open  ]],

Cell["The inverse operation is DiracGammaCombine.", "Text",
 CellTags->"DiracGammaExpand"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StandardForm", "[", 
  RowBox[{"DiracGammaCombine", "[", 
   RowBox[{"DiracGammaExpand", "[", "t", "]"}], "]"}], "]"}]], "Input",
 CellTags->"DiracGammaExpand",
 CellLabel->"In[388]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"DiracGamma", "[", 
   RowBox[{"Momentum", "[", "q", "]"}], "]"}], ".", 
  RowBox[{"DiracGamma", "[", 
   RowBox[{"Momentum", "[", 
    RowBox[{"p", "-", "q"}], "]"}], "]"}]}]], "Output",
 CellChangeTimes->{3.500970199460453*^9},
 CellTags->"DiracGammaExpand",
 CellLabel->"Out[388]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StandardForm", "[", 
  RowBox[{"DiracGammaExpand", "[", "t", "]"}], "]"}]], "Input",
 CellTags->"DiracGammaExpand",
 CellLabel->"In[389]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"DiracGamma", "[", 
   RowBox[{"Momentum", "[", "q", "]"}], "]"}], ".", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{"DiracGamma", "[", 
     RowBox[{"Momentum", "[", "p", "]"}], "]"}], "-", 
    RowBox[{"DiracGamma", "[", 
     RowBox[{"Momentum", "[", "q", "]"}], "]"}]}], ")"}]}]], "Output",
 CellChangeTimes->{3.500970199510544*^9},
 CellTags->"DiracGammaExpand",
 CellLabel->"Out[389]//StandardForm="]
}, Open  ]],

Cell["In order to do non-commutative expansion use DiracSimplify.", "Text",
 CellTags->"DiracGammaExpand"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracSimplify", "[", "t", "]"}]], "Input",
 CellTags->"DiracGammaExpand",
 CellLabel->"In[390]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      FormBox["\<\"\[Gamma]\"\>",
       TraditionalForm], "\[CenterDot]", 
      FormBox["q",
       TraditionalForm]}], ")"}], ".", 
    RowBox[{"(", 
     RowBox[{
      FormBox["\<\"\[Gamma]\"\>",
       TraditionalForm], "\[CenterDot]", 
      FormBox["p",
       TraditionalForm]}], ")"}]}], "-", 
   SuperscriptBox[
    FormBox["q",
     TraditionalForm], "2"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970199617811*^9},
 CellTags->"DiracGammaExpand",
 CellLabel->"Out[390]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"Clear", "[", "t", "]"}]], "Input",
 CellTags->"DiracGammaExpand",
 CellLabel->"In[391]:="]
}, Open  ]],

Cell["DiracGammaT", "ObjectName",
 CellTags->"DiracGammaT"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"DiracGammaT"],

Cell["\<\
DiracGammaT[x] denotes the transpose of DiracGamma[x]. \
Transpose[DiracGammaT[x]] gives DiracGamma[x]. Note that x must have Head \
LorentzIndex or Momentum.\
\>", "Text",
 CellTags->"DiracGammaT"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["DiracGamma",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracGamma",
  ButtonNote->"DiracGamma"],
 "."
}], "Text",
 CellTags->"DiracGammaT"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"DiracGammaT"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracGammaT", "[", 
  RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], "]"}]], "Input",
 CellTags->"DiracGammaT",
 CellLabel->"In[392]:="],

Cell[BoxData[
 FormBox[
  SubsuperscriptBox["\[Gamma]", 
   FormBox[
    FormBox["\[Mu]",
     TraditionalForm],
    TraditionalForm], "T"], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970199794036*^9},
 CellTags->"DiracGammaT",
 CellLabel->"Out[392]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Transpose", "[", "%", "]"}]], "Input",
 CellTags->"DiracGammaT",
 CellLabel->"In[393]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["\[Gamma]", 
   FormBox[
    FormBox["\[Mu]",
     TraditionalForm],
    TraditionalForm]], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009701998321533`*^9},
 CellTags->"DiracGammaT",
 CellLabel->"Out[393]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"GS", "[", "p", "]"}], "//", "FCI"}], "//", "Transpose"}]], "Input",\

 CellTags->"DiracGammaT",
 CellLabel->"In[394]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox[
   RowBox[{"(", "\[Gamma]", "\[CenterDot]", 
    FormBox["p",
     TraditionalForm], ")"}], "T"], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970199871594*^9},
 CellTags->"DiracGammaT",
 CellLabel->"Out[394]="]
}, Open  ]]
}, Open  ]],

Cell["DiracMatrix", "ObjectName",
 CellTags->"DiracMatrix"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"DiracMatrix"],

Cell[TextData[{
 "DiracMatrix[",
 Cell[BoxData[
  FormBox["\[Mu]", TraditionalForm]]],
 "] denotes a Dirac gamma matrix with Lorentz index ",
 Cell[BoxData[
  FormBox["\[Mu]", TraditionalForm]]],
 ". DiracMatrix[",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"\[Mu]", ",", "\[Nu]", ","}], " "}], TraditionalForm]]],
 "...] is a product of ",
 Cell[BoxData[
  FormBox["\[Gamma]", TraditionalForm]]],
 " matrices with Lorentz indices ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Mu]", ",", " ", "\[Nu]", ",", " ", "..."}], TraditionalForm]]],
 " DiracMatrix[5] is ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["\[Gamma]", "5"], TraditionalForm]]],
 ". DiracMatrix[6] is ",
 Cell[BoxData[
  FormBox[
   RowBox[{"1", "/", "2"}], TraditionalForm]]],
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{"+", " ", 
      SuperscriptBox["\[Gamma]", "5"]}], "/", "2"}]}], TraditionalForm]]],
 ". DiracMatrix[7] is",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{"1", "/", "2"}]}], TraditionalForm]]],
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{"-", " ", 
      SuperscriptBox["\[Gamma]", "5"]}], "/", "2"}]}], TraditionalForm]]],
 "."
}], "Text",
 CellTags->"DiracMatrix"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["DiracGammaExpand",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracGammaExpand",
  ButtonNote->"DiracGammaExpand"],
 ", ",
 ButtonBox["DiracGamma",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracGamma",
  ButtonNote->"DiracGamma"],
 ", ",
 ButtonBox["DiracSimplify",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracSimplify",
  ButtonNote->"DiracSimplify"],
 ", ",
 ButtonBox["DiracSlash",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracSlash",
  ButtonNote->"DiracSlash"],
 ", ",
 ButtonBox["DiracTrick",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracTrick",
  ButtonNote->"DiracTrick"],
 ", ",
 ButtonBox["GA",
  BaseStyle->"AddOnsLink",
  ButtonData:>"GA",
  ButtonNote->"GA"],
 ", ",
 ButtonBox["GAD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"GAD",
  ButtonNote->"GAD"],
 ", ",
 ButtonBox["GS",
  BaseStyle->"AddOnsLink",
  ButtonData:>"GS",
  ButtonNote->"GS"],
 ", ",
 ButtonBox["GSD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"GSD",
  ButtonNote->"GSD"],
 "."
}], "Text",
 CellTags->"DiracMatrix"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"DiracMatrix"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracMatrix", "[", "\[Mu]", "]"}]], "Input",
 CellTags->"DiracMatrix",
 CellLabel->"In[395]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["\[Gamma]", 
   FormBox[
    FormBox["\[Mu]",
     TraditionalForm],
    TraditionalForm]], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970199977964*^9},
 CellTags->"DiracMatrix",
 CellLabel->"Out[395]="]
}, Open  ]],

Cell[TextData[{
 "This is how to enter the non-commutative product of two ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], 
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm]]}], TraditionalForm]]],
 ". The ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " Dot \".\" is used as non-commutative multiplication operator."
}], "Text",
 CellTags->"DiracMatrix"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"DiracMatrix", "[", "\[Mu]", "]"}], ".", 
  RowBox[{"DiracMatrix", "[", "\[Nu]", "]"}]}]], "Input",
 CellTags->"DiracMatrix",
 CellLabel->"In[396]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], ".", 
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Nu]",
      TraditionalForm],
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009702000916157`*^9},
 CellTags->"DiracMatrix",
 CellLabel->"Out[396]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"DiracMatrix", "[", "\[Alpha]", "]"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"DiracMatrix",
 CellLabel->"In[397]:="],

Cell[BoxData[
 RowBox[{"DiracGamma", "[", 
  RowBox[{"LorentzIndex", "[", "\[Alpha]", "]"}], "]"}]], "Output",
 CellChangeTimes->{3.500970200173218*^9},
 CellTags->"DiracMatrix",
 CellLabel->"Out[397]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"DiracMatrix", "[", "\[Mu]", "]"}], "//", "FCE"}]], "Input",
 CellTags->"DiracMatrix",
 CellLabel->"In[398]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["\[Gamma]", "\[Mu]"], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009702002590847`*^9},
 CellTags->"DiracMatrix",
 CellLabel->"Out[398]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "StandardForm"}]], "Input",
 CellTags->"DiracMatrix",
 CellLabel->"In[399]:="],

Cell[BoxData[
 RowBox[{"GA", "[", "\[Mu]", "]"}]], "Output",
 CellChangeTimes->{3.500970200358519*^9},
 CellTags->"DiracMatrix",
 CellLabel->"Out[399]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GAD", "[", "\[Mu]", "]"}]], "Input",
 CellTags->"DiracMatrix",
 CellLabel->"In[400]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["\[Gamma]", "\[Mu]"], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970200838604*^9},
 CellTags->"DiracMatrix",
 CellLabel->"Out[400]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"%", "//", "FCI"}], "//", "StandardForm"}]], "Input",
 CellTags->"DiracMatrix",
 CellLabel->"In[401]:="],

Cell[BoxData[
 RowBox[{"DiracGamma", "[", 
  RowBox[{
   RowBox[{"LorentzIndex", "[", 
    RowBox[{"\[Mu]", ",", "D"}], "]"}], ",", "D"}], "]"}]], "Output",
 CellChangeTimes->{3.500970200932517*^9},
 CellTags->"DiracMatrix",
 CellLabel->"Out[401]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GA", "[", 
  RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]"}], "]"}]], "Input",
 CellTags->"DiracMatrix",
 CellLabel->"In[402]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
   SuperscriptBox["\[Gamma]", "\[Nu]"], ".", 
   SuperscriptBox["\[Gamma]", "\[Rho]"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970200977803*^9},
 CellTags->"DiracMatrix",
 CellLabel->"Out[402]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"GA", "[", 
   RowBox[{"a", " ", ".", " ", "b"}], "]"}], "//", "FCI"}]], "Input",
 CellTags->"DiracMatrix",
 CellLabel->"In[403]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["a",
      TraditionalForm],
     TraditionalForm]], ".", 
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["b",
      TraditionalForm],
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970201018735*^9},
 CellTags->"DiracMatrix",
 CellLabel->"Out[403]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "StandardForm"}]], "Input",
 CellTags->"DiracMatrix",
 CellLabel->"In[404]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"DiracGamma", "[", 
   RowBox[{"LorentzIndex", "[", "a", "]"}], "]"}], ".", 
  RowBox[{"DiracGamma", "[", 
   RowBox[{"LorentzIndex", "[", "b", "]"}], "]"}]}]], "Output",
 CellChangeTimes->{3.5009702010592003`*^9},
 CellTags->"DiracMatrix",
 CellLabel->"Out[404]//StandardForm="]
}, Open  ]]
}, Open  ]],

Cell["DiracOrder", "ObjectName",
 CellTags->"DiracOrder"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"DiracOrder"],

Cell["\<\
DiracOrder[expr] orders the Dirac matrices in expr alphabetically. \
DiracOrder[expr,orderlist] orders the Dirac matrices in expr according to \
orderlist.\
\>", "Text",
 CellTags->"DiracOrder"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["DiracSimplify",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracSimplify",
  ButtonNote->"DiracSimplify"],
 ", ",
 ButtonBox["DiracTrick",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracTrick",
  ButtonNote->"DiracTrick"],
 "."
}], "Text",
 CellTags->"DiracOrder"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"DiracOrder"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t1", " ", "=", " ", 
  RowBox[{"GA", "[", 
   RowBox[{"\[Beta]", ",", "\[Alpha]"}], "]"}]}]], "Input",
 CellTags->"DiracOrder",
 CellLabel->"In[405]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", "\[Beta]"], ".", 
   SuperscriptBox["\[Gamma]", "\[Alpha]"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009702010994577`*^9},
 CellTags->"DiracOrder",
 CellLabel->"Out[405]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracOrder", "[", "t1", "]"}]], "Input",
 CellTags->"DiracOrder",
 CellLabel->"In[406]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"2", " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Alpha]", "\[Beta]"}]]}], "-", 
   RowBox[{
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox["\[Alpha]",
       TraditionalForm],
      TraditionalForm]], ".", 
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox["\[Beta]",
       TraditionalForm],
      TraditionalForm]]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970201185754*^9},
 CellTags->"DiracOrder",
 CellLabel->"Out[406]="]
}, Open  ]],

Cell["This is a string of Dirac matrices in D dimensions. ", "Text",
 CellTags->"DiracOrder"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t2", "=", 
  RowBox[{"GAD", "[", 
   RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Mu]"}], "]"}]}]], "Input",
 CellTags->"DiracOrder",
 CellLabel->"In[407]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
   SuperscriptBox["\[Gamma]", "\[Nu]"], ".", 
   SuperscriptBox["\[Gamma]", "\[Mu]"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970201257368*^9},
 CellTags->"DiracOrder",
 CellLabel->"Out[407]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracOrder", "[", "t1", "]"}]], "Input",
 CellTags->"DiracOrder",
 CellLabel->"In[408]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"2", " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Alpha]", "\[Beta]"}]]}], "-", 
   RowBox[{
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox["\[Alpha]",
       TraditionalForm],
      TraditionalForm]], ".", 
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox["\[Beta]",
       TraditionalForm],
      TraditionalForm]]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970201330083*^9},
 CellTags->"DiracOrder",
 CellLabel->"Out[408]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t3", "=", 
  RowBox[{"GA", "[", 
   RowBox[{"5", ",", "\[Mu]", ",", "\[Nu]"}], "]"}]}]], "Input",
 CellTags->"DiracOrder",
 CellLabel->"In[409]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", "5"], ".", 
   SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
   SuperscriptBox["\[Gamma]", "\[Nu]"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970201401746*^9},
 CellTags->"DiracOrder",
 CellLabel->"Out[409]="]
}, Open  ]],

Cell[TextData[{
 "By default ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["\[Gamma]", 
    RowBox[{"5", " "}]], TraditionalForm]]],
 "is moved to the right."
}], "Text",
 CellTags->"DiracOrder"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracOrder", "[", "t3", "]"}]], "Input",
 CellTags->"DiracOrder",
 CellLabel->"In[410]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], ".", 
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Nu]",
      TraditionalForm],
     TraditionalForm]], ".", 
   SuperscriptBox["\[Gamma]", "5"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009702015804167`*^9},
 CellTags->"DiracOrder",
 CellLabel->"Out[410]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t4", "=", 
  RowBox[{"GA", "[", 
   RowBox[{"6", ",", "\[Mu]", ",", "7"}], "]"}]}]], "Input",
 CellTags->"DiracOrder",
 CellLabel->"In[411]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", "6"], ".", 
   SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
   SuperscriptBox["\[Gamma]", "7"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009702016613607`*^9},
 CellTags->"DiracOrder",
 CellLabel->"Out[411]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracOrder", "[", "t4", "]"}]], "Input",
 CellTags->"DiracOrder",
 CellLabel->"In[412]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], ".", 
   SuperscriptBox["\[Gamma]", "7"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009702017951813`*^9},
 CellTags->"DiracOrder",
 CellLabel->"Out[412]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t5", "=", 
  RowBox[{"GA", "[", 
   RowBox[{"\[Alpha]", ",", "\[Beta]", ",", "\[Delta]"}], "]"}]}]], "Input",
 CellTags->"DiracOrder",
 CellLabel->"In[413]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", "\[Alpha]"], ".", 
   SuperscriptBox["\[Gamma]", "\[Beta]"], ".", 
   SuperscriptBox["\[Gamma]", "\[Delta]"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970201897504*^9},
 CellTags->"DiracOrder",
 CellLabel->"Out[413]="]
}, Open  ]],

Cell[TextData[{
 "This orders the ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["\[Gamma]", "\[Alpha]"], 
    SuperscriptBox["\[Gamma]", "\[Beta]"], 
    SuperscriptBox["\[Gamma]", "\[Delta]"]}], TraditionalForm]]],
 " in reverse order."
}], "Text",
 CellTags->"DiracOrder"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracOrder", "[", 
  RowBox[{"t5", ",", 
   RowBox[{"{", 
    RowBox[{"\[Delta]", ",", "\[Beta]", ",", "\[Alpha]"}], "}"}]}], 
  "]"}]], "Input",
 CellTags->"DiracOrder",
 CellLabel->"In[414]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Delta]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Beta]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Alpha]",
        TraditionalForm],
       TraditionalForm]]}]}], "+", 
   RowBox[{"2", " ", 
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox["\[Delta]",
       TraditionalForm],
      TraditionalForm]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Alpha]", "\[Beta]"}]]}], "-", 
   RowBox[{"2", " ", 
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox["\[Beta]",
       TraditionalForm],
      TraditionalForm]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Alpha]", "\[Delta]"}]]}], "+", 
   RowBox[{"2", " ", 
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox["\[Alpha]",
       TraditionalForm],
      TraditionalForm]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Beta]", "\[Delta]"}]]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009702020303783`*^9},
 CellTags->"DiracOrder",
 CellLabel->"Out[414]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracOrder", "[", "%", "]"}]], "Input",
 CellTags->"DiracOrder",
 CellLabel->"In[415]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Alpha]",
      TraditionalForm],
     TraditionalForm]], ".", 
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Beta]",
      TraditionalForm],
     TraditionalForm]], ".", 
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Delta]",
      TraditionalForm],
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009702021495047`*^9},
 CellTags->"DiracOrder",
 CellLabel->"Out[415]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", 
   RowBox[{"t1", ",", "t2", ",", "t3", ",", "t4", ",", "t5"}], "]"}], 
  ";"}]], "Input",
 CellTags->"DiracOrder",
 CellLabel->"In[416]:="]
}, Open  ]],

Cell["DiracReduce", "ObjectName",
 CellTags->"DiracReduce"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"DiracReduce"],

Cell["\<\
DiracReduce[exp] reduces all four-dimensional Dirac matrices in exp to the \
standard basis (S,P,V,A,T) using the Chisholm identity (see Chisholm). In the \
result the basic Dirac structures are wrapped with a head DiracBasis. I.e., S \
corresponds to DiracBasis[1], P : DiracBasis[DiracMatrix[5]], V: \
DiracBasis[DiracMatrix[mu]], A: DiracBasis[DiracMatrix[mu, 5]], T: \
DiracBasis[DiracSigma[DiracMatrix[mu, nu]]]. By default DiracBasis is \
substituted to Identity. Notice that the result of DiracReduce is given in \
the FeynCalcExternal-way, i.e.,evtl. you may have to use FeynCalcInternal on \
the result.\
\>", "Text",
 CellTags->"DiracReduce"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "DiracReduce", "]"}]], "Input",
 CellTags->"DiracReduce",
 CellLabel->"In[417]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Factoring", "\[Rule]", "False"}], ",", 
    RowBox[{"FinalSubstitutions", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{"DiracBasis", "\[Rule]", "Identity"}], "}"}]}]}], "}"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970202227942*^9},
 CellTags->"DiracReduce",
 CellLabel->"Out[417]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["DiracSimplify",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracSimplify",
  ButtonNote->"DiracSimplify"],
 "."
}], "Text",
 CellTags->"DiracReduce"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"DiracReduce"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t1", " ", "=", " ", 
  RowBox[{"GA", "[", 
   RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}]}]], "Input",
 CellTags->"DiracReduce",
 CellLabel->"In[418]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
   SuperscriptBox["\[Gamma]", "\[Nu]"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009702022969427`*^9},
 CellTags->"DiracReduce",
 CellLabel->"Out[418]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracReduce", "[", "t1", "]"}]], "Input",
 CellTags->"DiracReduce",
 CellLabel->"In[419]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["g", 
    RowBox[{"\[Mu]", "\[Nu]"}]], "-", 
   RowBox[{"\[ImaginaryI]", " ", 
    SuperscriptBox["\[Sigma]", 
     RowBox[{"\[Mu]", "\[Nu]"}]]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970202468811*^9},
 CellTags->"DiracReduce",
 CellLabel->"Out[419]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t2", "=", 
  RowBox[{"DiracMatrix", "[", 
   RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]"}], "]"}]}]], "Input",
 CellTags->"DiracReduce",
 CellLabel->"In[420]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", "\[Mu]"], 
   SuperscriptBox["\[Gamma]", "\[Nu]"], 
   SuperscriptBox["\[Gamma]", "\[Rho]"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970203085369*^9},
 CellTags->"DiracReduce",
 CellLabel->"Out[420]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracReduce", "[", "t2", "]"}]], "Input",
 CellTags->"DiracReduce",
 CellLabel->"In[421]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"\[ImaginaryI]", " ", 
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      RowBox[{"$MU", "\[InvisibleApplication]", 
       RowBox[{"(", "1", ")"}]}]], ".", 
     SuperscriptBox["\[Gamma]", "5"]}], " ", 
    SuperscriptBox["\[Epsilon]", 
     RowBox[{"\[Mu]", "\[Nu]", "\[Rho]", 
      FormBox[
       RowBox[{"$MU", "\[InvisibleApplication]", 
        RowBox[{"(", "1", ")"}]}],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    SuperscriptBox["\[Gamma]", "\[Rho]"], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Nu]"}]]}], "-", 
   RowBox[{
    SuperscriptBox["\[Gamma]", "\[Nu]"], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Rho]"}]]}], "+", 
   RowBox[{
    SuperscriptBox["\[Gamma]", "\[Mu]"], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Nu]", "\[Rho]"}]]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970203374878*^9},
 CellTags->"DiracReduce",
 CellLabel->"Out[421]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t3", "=", 
  RowBox[{"DiracMatrix", "[", 
   RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]", ",", "\[Sigma]"}], 
   "]"}]}]], "Input",
 CellTags->"DiracReduce",
 CellLabel->"In[422]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", "\[Mu]"], 
   SuperscriptBox["\[Gamma]", "\[Nu]"], 
   SuperscriptBox["\[Gamma]", "\[Rho]"], 
   SuperscriptBox["\[Gamma]", "\[Sigma]"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970203413982*^9},
 CellTags->"DiracReduce",
 CellLabel->"Out[422]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t4", "=", 
  RowBox[{"DiracReduce", "[", "t3", "]"}]}]], "Input",
 CellTags->"DiracReduce",
 CellLabel->"In[423]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    RowBox[{"\[ImaginaryI]", " ", 
     SuperscriptBox["\[Gamma]", "5"], " ", 
     SuperscriptBox["\[Epsilon]", 
      RowBox[{"\[Mu]", "\[Nu]", "\[Rho]", "\[Sigma]"}]]}]}], "-", 
   RowBox[{"\[ImaginaryI]", " ", 
    SuperscriptBox["\[Sigma]", 
     RowBox[{"\[Rho]", "\[Sigma]"}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Nu]"}]]}], "+", 
   RowBox[{"\[ImaginaryI]", " ", 
    SuperscriptBox["\[Sigma]", 
     RowBox[{"\[Nu]", "\[Sigma]"}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Rho]"}]]}], "-", 
   RowBox[{"\[ImaginaryI]", " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Sigma]"}]], " ", 
    SuperscriptBox["\[Sigma]", 
     RowBox[{"\[Nu]", "\[Rho]"}]]}], "-", 
   RowBox[{"\[ImaginaryI]", " ", 
    SuperscriptBox["\[Sigma]", 
     RowBox[{"\[Mu]", "\[Sigma]"}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Nu]", "\[Rho]"}]]}], "+", 
   RowBox[{"\[ImaginaryI]", " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Nu]", "\[Sigma]"}]], " ", 
    SuperscriptBox["\[Sigma]", 
     RowBox[{"\[Mu]", "\[Rho]"}]]}], "-", 
   RowBox[{"\[ImaginaryI]", " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Rho]", "\[Sigma]"}]], " ", 
    SuperscriptBox["\[Sigma]", 
     RowBox[{"\[Mu]", "\[Nu]"}]]}], "+", 
   RowBox[{
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Sigma]"}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Nu]", "\[Rho]"}]]}], "-", 
   RowBox[{
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Rho]"}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Nu]", "\[Sigma]"}]]}], "+", 
   RowBox[{
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Rho]", "\[Sigma]"}]]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.50097020393863*^9},
 CellTags->"DiracReduce",
 CellLabel->"Out[423]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t5", " ", "=", 
  RowBox[{"Calc", "[", 
   RowBox[{"DiracSimplify", "[", 
    RowBox[{"DiracSigmaExplicit", "[", 
     RowBox[{"t4", ".", "t4"}], "]"}], "]"}], "]"}]}]], "Input",
 CellTags->"DiracReduce",
 CellLabel->"In[424]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", "128"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970204626143*^9},
 CellTags->"DiracReduce",
 CellLabel->"Out[424]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Calc", "[", 
  RowBox[{"t4", ".", "t4"}], "]"}]], "Input",
 CellTags->"DiracReduce",
 CellLabel->"In[425]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", "128"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970204944949*^9},
 CellTags->"DiracReduce",
 CellLabel->"Out[425]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"Clear", "[", 
  RowBox[{"t1", ",", "t2", ",", "t3", ",", "t4"}], "]"}]], "Input",
 CellTags->"DiracReduce",
 CellLabel->"In[426]:="]
}, Open  ]],

Cell["DiracSigma", "ObjectName",
 CellTags->"DiracSigma"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"DiracSigma"],

Cell["\<\
DiracSigma[a, b] stands for i/2*(a . b - b . a) in 4 dimensions. a and b must \
have Head DiracGamma, DiracMatrix or DiracSlash. Only antisymmetry is \
implemented.\
\>", "Text",
 CellTags->"DiracSigma"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["DiracSigmaExplicit",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracSigmaExplicit",
  ButtonNote->"DiracSigmaExplicit"],
 "."
}], "Text",
 CellTags->"DiracSigma"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"DiracSigma"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t1", "=", 
  RowBox[{"DiracSigma", "[", 
   RowBox[{
    RowBox[{"GA", "[", "\[Alpha]", "]"}], ",", 
    RowBox[{"GA", "[", "\[Beta]", "]"}]}], "]"}]}]], "Input",
 CellTags->"DiracSigma",
 CellLabel->"In[427]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["\[Sigma]", 
   RowBox[{"\[Alpha]", "\[Beta]"}]], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970205188114*^9},
 CellTags->"DiracSigma",
 CellLabel->"Out[427]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracSigmaExplicit", "[", "t1", "]"}]], "Input",
 CellTags->"DiracSigma",
 CellLabel->"In[428]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["1", "2"], " ", "\[ImaginaryI]", " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      SuperscriptBox["\[Gamma]", 
       FormBox[
        FormBox["\[Alpha]",
         TraditionalForm],
        TraditionalForm]], ".", 
      SuperscriptBox["\[Gamma]", 
       FormBox[
        FormBox["\[Beta]",
         TraditionalForm],
        TraditionalForm]]}], "-", 
     RowBox[{
      SuperscriptBox["\[Gamma]", 
       FormBox[
        FormBox["\[Beta]",
         TraditionalForm],
        TraditionalForm]], ".", 
      SuperscriptBox["\[Gamma]", 
       FormBox[
        FormBox["\[Alpha]",
         TraditionalForm],
        TraditionalForm]]}]}], ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970205260988*^9},
 CellTags->"DiracSigma",
 CellLabel->"Out[428]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t2", "=", 
  RowBox[{"DiracSigma", "[", 
   RowBox[{
    RowBox[{"GA", "[", "\[Beta]", "]"}], ",", 
    RowBox[{"GA", "[", "\[Alpha]", "]"}]}], "]"}]}]], "Input",
 CellTags->"DiracSigma",
 CellLabel->"In[429]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   SuperscriptBox["\[Sigma]", 
    RowBox[{"\[Alpha]", "\[Beta]"}]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970205370964*^9},
 CellTags->"DiracSigma",
 CellLabel->"Out[429]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t3", "=", 
  RowBox[{"DiracSigma", "[", 
   RowBox[{
    RowBox[{"GS", "[", "p", "]"}], ",", 
    RowBox[{"GS", "[", "q", "]"}]}], "]"}]}]], "Input",
 CellTags->"DiracSigma",
 CellLabel->"In[430]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["\[Sigma]", 
   RowBox[{"p", "q"}]], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009702054395943`*^9},
 CellTags->"DiracSigma",
 CellLabel->"Out[430]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracSigmaExplicit", "[", "t3", "]"}]], "Input",
 CellTags->"DiracSigma",
 CellLabel->"In[431]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["1", "2"], " ", "\[ImaginaryI]", " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        FormBox["\<\"\[Gamma]\"\>",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p",
         TraditionalForm]}], ")"}], ".", 
      RowBox[{"(", 
       RowBox[{
        FormBox["\<\"\[Gamma]\"\>",
         TraditionalForm], "\[CenterDot]", 
        FormBox["q",
         TraditionalForm]}], ")"}]}], "-", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        FormBox["\<\"\[Gamma]\"\>",
         TraditionalForm], "\[CenterDot]", 
        FormBox["q",
         TraditionalForm]}], ")"}], ".", 
      RowBox[{"(", 
       RowBox[{
        FormBox["\<\"\[Gamma]\"\>",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p",
         TraditionalForm]}], ")"}]}]}], ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009702055089283`*^9},
 CellTags->"DiracSigma",
 CellLabel->"Out[431]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"Clear", "[", 
  RowBox[{"t1", ",", "t2", ",", "t3"}], "]"}]], "Input",
 CellTags->"DiracSigma",
 CellLabel->"In[432]:="]
}, Open  ]],

Cell["DiracSigmaExplicit", "ObjectName",
 CellTags->"DiracSigmaExplicit"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"DiracSigmaExplicit"],

Cell["\<\
DiracSigmaExplicit[exp] inserts in exp for all DiracSigma its definition. \
DiracSigmaExplict is also an option of DiracSimplify.\
\>", "Text",
 CellTags->"DiracSigmaExplicit"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["DiracSigma",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracSigma",
  ButtonNote->"DiracSigma"],
 "."
}], "Text",
 CellTags->"DiracSigmaExplicit"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"DiracSigmaExplicit"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracSigma", "[", 
  RowBox[{
   RowBox[{"GA", "[", "\[Alpha]", "]"}], ",", 
   RowBox[{"GA", "[", "\[Beta]", "]"}]}], "]"}]], "Input",
 CellTags->"DiracSigmaExplicit",
 CellLabel->"In[433]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["\[Sigma]", 
   RowBox[{"\[Alpha]", "\[Beta]"}]], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970205552422*^9},
 CellTags->"DiracSigmaExplicit",
 CellLabel->"Out[433]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracSigmaExplicit", "[", "%", "]"}]], "Input",
 CellTags->"DiracSigmaExplicit",
 CellLabel->"In[434]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["1", "2"], " ", "\[ImaginaryI]", " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      SuperscriptBox["\[Gamma]", 
       FormBox[
        FormBox["\[Alpha]",
         TraditionalForm],
        TraditionalForm]], ".", 
      SuperscriptBox["\[Gamma]", 
       FormBox[
        FormBox["\[Beta]",
         TraditionalForm],
        TraditionalForm]]}], "-", 
     RowBox[{
      SuperscriptBox["\[Gamma]", 
       FormBox[
        FormBox["\[Beta]",
         TraditionalForm],
        TraditionalForm]], ".", 
      SuperscriptBox["\[Gamma]", 
       FormBox[
        FormBox["\[Alpha]",
         TraditionalForm],
        TraditionalForm]]}]}], ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970205617684*^9},
 CellTags->"DiracSigmaExplicit",
 CellLabel->"Out[434]="]
}, Open  ]]
}, Open  ]],

Cell["DiracSimpCombine", "ObjectName",
 CellTags->"DiracSimpCombine"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"DiracSimpCombine"],

Cell["\<\
DiracSimpCombine is an option for DiracSimplify. If set to True, sums of \
DiracGamma's will be merged as much as possible in DiracGamma[ .. + .. + ]'s.\
\
\>", "Text",
 CellTags->"DiracSimpCombine"],

Cell[TextData[{
 "See also: ",
 ButtonBox["DiracSimplify",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracSimplify",
  ButtonNote->"DiracSimplify"],
 "."
}], "Text",
 CellTags->"DiracSimpCombine"]
}, Open  ]],

Cell["DiracSimplify", "ObjectName",
 CellTags->"DiracSimplify"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"DiracSimplify"],

Cell["\<\
DiracSimplify[expr] simplifies products of Dirac matrices in expr and expands \
non-commutative products. Double Lorentz indices and four vectors are \
contracted. The Dirac equation is applied. All DiracMatrix[5], DiracMatrix[6] \
and DiracMatrix[7] are moved to the right. The order of the other Dirac \
matrices is not changed.\
\>", "Text",
 CellTags->"DiracSimplify"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "DiracSimplify", "]"}]], "Input",
 CellTags->"DiracSimplify",
 CellLabel->"In[435]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"DiracCanonical", "\[Rule]", "False"}], ",", 
    RowBox[{"DiracSigmaExplicit", "\[Rule]", "True"}], ",", 
    RowBox[{"DiracSimpCombine", "\[Rule]", "False"}], ",", 
    RowBox[{"DiracSubstitute67", "\[Rule]", "False"}], ",", 
    RowBox[{"Expanding", "\[Rule]", "True"}], ",", 
    RowBox[{"Factoring", "\[Rule]", "False"}], ",", 
    RowBox[{"FeynCalcInternal", "\[Rule]", "False"}], ",", 
    RowBox[{"InsideDiracTrace", "\[Rule]", "False"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970205682075*^9},
 CellTags->"DiracSimplify",
 CellLabel->"Out[435]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Calc",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Calc",
  ButtonNote->"Calc"],
 ", ",
 ButtonBox["DiracGammaExpand",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracGammaExpand",
  ButtonNote->"DiracGammaExpand"],
 ", ",
 ButtonBox["DiracTrick",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracTrick",
  ButtonNote->"DiracTrick"],
 "."
}], "Text",
 CellTags->"DiracSimplify"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"DiracSimplify"],

Cell["This is a string of Dirac matrices in four dimensions. ", "Text",
 CellTags->"DiracSimplify"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t1", "=", 
  RowBox[{"GA", "[", 
   RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Mu]"}], "]"}]}]], "Input",
 CellTags->"DiracSimplify",
 CellLabel->"In[436]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
   SuperscriptBox["\[Gamma]", "\[Nu]"], ".", 
   SuperscriptBox["\[Gamma]", "\[Mu]"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009702057152157`*^9},
 CellTags->"DiracSimplify",
 CellLabel->"Out[436]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracSimplify", "[", "t1", "]"}]], "Input",
 CellTags->"DiracSimplify",
 CellLabel->"In[437]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", "2"}], " ", 
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Nu]",
      TraditionalForm],
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.50097020578329*^9},
 CellTags->"DiracSimplify",
 CellLabel->"Out[437]="]
}, Open  ]],

Cell["This is a string of Dirac matrices in D dimensions. ", "Text",
 CellTags->"DiracSimplify"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t2", "=", 
  RowBox[{"GAD", "[", 
   RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Mu]"}], "]"}]}]], "Input",
 CellTags->"DiracSimplify",
 CellLabel->"In[438]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
   SuperscriptBox["\[Gamma]", "\[Nu]"], ".", 
   SuperscriptBox["\[Gamma]", "\[Mu]"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970205823612*^9},
 CellTags->"DiracSimplify",
 CellLabel->"Out[438]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracSimplify", "[", "t1", "]"}]], "Input",
 CellTags->"DiracSimplify",
 CellLabel->"In[439]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", "2"}], " ", 
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Nu]",
      TraditionalForm],
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970205891614*^9},
 CellTags->"DiracSimplify",
 CellLabel->"Out[439]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t3", "=", 
  RowBox[{"GA", "[", 
   RowBox[{"5", ",", "\[Mu]", ",", "\[Nu]"}], "]"}]}]], "Input",
 CellTags->"DiracSimplify",
 CellLabel->"In[440]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", "5"], ".", 
   SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
   SuperscriptBox["\[Gamma]", "\[Nu]"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970205933105*^9},
 CellTags->"DiracSimplify",
 CellLabel->"Out[440]="]
}, Open  ]],

Cell[TextData[{
 "By default ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["\[Gamma]", "5"], TraditionalForm]]],
 "is moved to the right."
}], "Text",
 CellTags->"DiracSimplify"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracSimplify", "[", "t3", "]"}]], "Input",
 CellTags->"DiracSimplify",
 CellLabel->"In[441]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], ".", 
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Nu]",
      TraditionalForm],
     TraditionalForm]], ".", 
   SuperscriptBox["\[Gamma]", "5"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009702060366163`*^9},
 CellTags->"DiracSimplify",
 CellLabel->"Out[441]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t4", "=", 
  RowBox[{"GA", "[", 
   RowBox[{"6", ",", "\[Mu]", ",", "7"}], "]"}]}]], "Input",
 CellTags->"DiracSimplify",
 CellLabel->"In[442]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", "6"], ".", 
   SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
   SuperscriptBox["\[Gamma]", "7"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009702064439707`*^9},
 CellTags->"DiracSimplify",
 CellLabel->"Out[442]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracSimplify", "[", "t4", "]"}]], "Input",
 CellTags->"DiracSimplify",
 CellLabel->"In[443]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], ".", 
   SuperscriptBox["\[Gamma]", "7"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970206533042*^9},
 CellTags->"DiracSimplify",
 CellLabel->"Out[443]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t5", "=", 
  RowBox[{
   RowBox[{"GS", "[", 
    RowBox[{"a", "+", "b"}], "]"}], " ", ".", " ", 
   RowBox[{"GS", "[", "p", "]"}], ".", 
   RowBox[{"GS", "[", "p", "]"}], ".", 
   RowBox[{"GS", "[", 
    RowBox[{"c", "+", "d"}], "]"}]}]}]], "Input",
 CellTags->"DiracSimplify",
 CellLabel->"In[444]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"(", 
    RowBox[{"\[Gamma]", "\[CenterDot]", "(", 
     FormBox[
      RowBox[{"a", "+", "b"}],
      TraditionalForm], ")"}], ")"}], ".", 
   RowBox[{"(", 
    RowBox[{"\[Gamma]", "\[CenterDot]", "p"}], ")"}], ".", 
   RowBox[{"(", 
    RowBox[{"\[Gamma]", "\[CenterDot]", "p"}], ")"}], ".", 
   RowBox[{"(", 
    RowBox[{"\[Gamma]", "\[CenterDot]", "(", 
     FormBox[
      RowBox[{"c", "+", "d"}],
      TraditionalForm], ")"}], ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970206672956*^9},
 CellTags->"DiracSimplify",
 CellLabel->"Out[444]="]
}, Open  ]],

Cell["\<\
Contrary to DiracTrick DiracSimplify does non-commutative expansion.\
\>", "Text",
 CellTags->"DiracSimplify"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracSimplify", "[", "t5", "]"}]], "Input",
 CellTags->"DiracSimplify",
 CellLabel->"In[445]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SuperscriptBox[
     FormBox["p",
      TraditionalForm], "2"], " ", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       FormBox["\<\"\[Gamma]\"\>",
        TraditionalForm], "\[CenterDot]", 
       FormBox["a",
        TraditionalForm]}], ")"}], ".", 
     RowBox[{"(", 
      RowBox[{
       FormBox["\<\"\[Gamma]\"\>",
        TraditionalForm], "\[CenterDot]", 
       FormBox["c",
        TraditionalForm]}], ")"}]}]}], "+", 
   RowBox[{
    SuperscriptBox[
     FormBox["p",
      TraditionalForm], "2"], " ", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       FormBox["\<\"\[Gamma]\"\>",
        TraditionalForm], "\[CenterDot]", 
       FormBox["a",
        TraditionalForm]}], ")"}], ".", 
     RowBox[{"(", 
      RowBox[{
       FormBox["\<\"\[Gamma]\"\>",
        TraditionalForm], "\[CenterDot]", 
       FormBox["d",
        TraditionalForm]}], ")"}]}]}], "+", 
   RowBox[{
    SuperscriptBox[
     FormBox["p",
      TraditionalForm], "2"], " ", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       FormBox["\<\"\[Gamma]\"\>",
        TraditionalForm], "\[CenterDot]", 
       FormBox["b",
        TraditionalForm]}], ")"}], ".", 
     RowBox[{"(", 
      RowBox[{
       FormBox["\<\"\[Gamma]\"\>",
        TraditionalForm], "\[CenterDot]", 
       FormBox["c",
        TraditionalForm]}], ")"}]}]}], "+", 
   RowBox[{
    SuperscriptBox[
     FormBox["p",
      TraditionalForm], "2"], " ", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       FormBox["\<\"\[Gamma]\"\>",
        TraditionalForm], "\[CenterDot]", 
       FormBox["b",
        TraditionalForm]}], ")"}], ".", 
     RowBox[{"(", 
      RowBox[{
       FormBox["\<\"\[Gamma]\"\>",
        TraditionalForm], "\[CenterDot]", 
       FormBox["d",
        TraditionalForm]}], ")"}]}]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009702067896957`*^9},
 CellTags->"DiracSimplify",
 CellLabel->"Out[445]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracTrick", "[", "t5", "]"}]], "Input",
 CellTags->"DiracSimplify",
 CellLabel->"In[446]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"(", 
    RowBox[{"\[Gamma]", "\[CenterDot]", "(", 
     FormBox[
      RowBox[{"a", "+", "b"}],
      TraditionalForm], ")"}], ")"}], ".", 
   RowBox[{"(", 
    RowBox[{"\[Gamma]", "\[CenterDot]", "p"}], ")"}], ".", 
   RowBox[{"(", 
    RowBox[{"\[Gamma]", "\[CenterDot]", "p"}], ")"}], ".", 
   RowBox[{"(", 
    RowBox[{"\[Gamma]", "\[CenterDot]", "(", 
     FormBox[
      RowBox[{"c", "+", "d"}],
      TraditionalForm], ")"}], ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970206895973*^9},
 CellTags->"DiracSimplify",
 CellLabel->"Out[446]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t6", " ", "=", " ", 
  RowBox[{
   RowBox[{"SpinorVBar", "[", "p", "]"}], " ", ".", " ", 
   RowBox[{"GS", "[", "p", "]"}], " ", ".", " ", 
   RowBox[{"SpinorUBar", "[", "q", "]"}]}]}]], "Input",
 CellTags->"DiracSimplify",
 CellLabel->"In[447]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    FormBox[
     OverscriptBox["\<\"v\"\>", "_"],
     TraditionalForm], "(", "p", ")"}], ".", 
   RowBox[{"(", 
    RowBox[{"\[Gamma]", "\[CenterDot]", "p"}], ")"}], ".", 
   RowBox[{
    FormBox[
     OverscriptBox["\<\"u\"\>", "_"],
     TraditionalForm], "(", "q", ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009702069491463`*^9},
 CellTags->"DiracSimplify",
 CellLabel->"Out[447]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracSimplify", "[", "t6", "]"}]], "Input",
 CellTags->"DiracSimplify",
 CellLabel->"In[448]:="],

Cell[BoxData[
 FormBox["0", TraditionalForm]], "Output",
 CellChangeTimes->{3.5009702071539583`*^9},
 CellTags->"DiracSimplify",
 CellLabel->"Out[448]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GAD", "@@", 
  RowBox[{"Join", "[", 
   RowBox[{
    RowBox[{"{", "\[Mu]", "}"}], ",", 
    RowBox[{"Table", "[", 
     RowBox[{
      SubscriptBox["\[Nu]", "i"], ",", 
      RowBox[{"{", 
       RowBox[{"i", ",", "6"}], "}"}]}], "]"}], ",", 
    RowBox[{"{", "\[Mu]", "}"}]}], "]"}]}]], "Input",
 CellTags->"DiracSimplify",
 CellLabel->"In[449]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
   SuperscriptBox["\[Gamma]", 
    SubscriptBox["\[Nu]", "1"]], ".", 
   SuperscriptBox["\[Gamma]", 
    SubscriptBox["\[Nu]", "2"]], ".", 
   SuperscriptBox["\[Gamma]", 
    SubscriptBox["\[Nu]", "3"]], ".", 
   SuperscriptBox["\[Gamma]", 
    SubscriptBox["\[Nu]", "4"]], ".", 
   SuperscriptBox["\[Gamma]", 
    SubscriptBox["\[Nu]", "5"]], ".", 
   SuperscriptBox["\[Gamma]", 
    SubscriptBox["\[Nu]", "6"]], ".", 
   SuperscriptBox["\[Gamma]", "\[Mu]"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970207375839*^9},
 CellTags->"DiracSimplify",
 CellLabel->"Out[449]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracSimplify", "[", "%", "]"}]], "Input",
 CellTags->"DiracSimplify",
 CellLabel->"In[450]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    RowBox[{"12", " ", 
     RowBox[{
      SuperscriptBox["\[Gamma]", 
       FormBox[
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm],
        TraditionalForm]], ".", 
      SuperscriptBox["\[Gamma]", 
       FormBox[
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm],
        TraditionalForm]], ".", 
      SuperscriptBox["\[Gamma]", 
       FormBox[
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm],
        TraditionalForm]], ".", 
      SuperscriptBox["\[Gamma]", 
       FormBox[
        FormBox[
         SubscriptBox["\[Nu]", "4"],
         TraditionalForm],
        TraditionalForm]], ".", 
      SuperscriptBox["\[Gamma]", 
       FormBox[
        FormBox[
         SubscriptBox["\[Nu]", "5"],
         TraditionalForm],
        TraditionalForm]], ".", 
      SuperscriptBox["\[Gamma]", 
       FormBox[
        FormBox[
         SubscriptBox["\[Nu]", "6"],
         TraditionalForm],
        TraditionalForm]]}]}]}], "+", 
   RowBox[{"D", " ", 
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "1"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "2"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "3"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "4"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "5"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "6"],
        TraditionalForm],
       TraditionalForm]]}]}], "+", 
   RowBox[{"4", " ", 
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "3"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "4"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "5"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "6"],
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{
      SubscriptBox["\[Nu]", "1"], 
      SubscriptBox["\[Nu]", "2"]}]]}], "-", 
   RowBox[{"4", " ", 
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "2"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "4"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "5"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "6"],
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{
      SubscriptBox["\[Nu]", "1"], 
      SubscriptBox["\[Nu]", "3"]}]]}], "+", 
   RowBox[{"4", " ", 
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "2"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "3"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "5"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "6"],
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{
      SubscriptBox["\[Nu]", "1"], 
      SubscriptBox["\[Nu]", "4"]}]]}], "-", 
   RowBox[{"4", " ", 
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "2"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "3"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "4"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "6"],
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{
      SubscriptBox["\[Nu]", "1"], 
      SubscriptBox["\[Nu]", "5"]}]]}], "+", 
   RowBox[{"4", " ", 
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "2"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "3"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "4"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "5"],
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{
      SubscriptBox["\[Nu]", "1"], 
      SubscriptBox["\[Nu]", "6"]}]]}], "+", 
   RowBox[{"4", " ", 
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "1"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "4"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "5"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "6"],
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{
      SubscriptBox["\[Nu]", "2"], 
      SubscriptBox["\[Nu]", "3"]}]]}], "-", 
   RowBox[{"4", " ", 
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "1"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "3"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "5"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "6"],
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{
      SubscriptBox["\[Nu]", "2"], 
      SubscriptBox["\[Nu]", "4"]}]]}], "+", 
   RowBox[{"4", " ", 
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "1"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "3"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "4"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "6"],
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{
      SubscriptBox["\[Nu]", "2"], 
      SubscriptBox["\[Nu]", "5"]}]]}], "-", 
   RowBox[{"4", " ", 
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "1"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "3"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "4"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "5"],
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{
      SubscriptBox["\[Nu]", "2"], 
      SubscriptBox["\[Nu]", "6"]}]]}], "+", 
   RowBox[{"4", " ", 
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "1"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "2"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "5"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "6"],
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{
      SubscriptBox["\[Nu]", "3"], 
      SubscriptBox["\[Nu]", "4"]}]]}], "-", 
   RowBox[{"4", " ", 
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "1"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "2"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "4"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "6"],
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{
      SubscriptBox["\[Nu]", "3"], 
      SubscriptBox["\[Nu]", "5"]}]]}], "+", 
   RowBox[{"4", " ", 
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "1"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "2"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "4"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "5"],
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{
      SubscriptBox["\[Nu]", "3"], 
      SubscriptBox["\[Nu]", "6"]}]]}], "+", 
   RowBox[{"4", " ", 
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "1"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "2"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "3"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "6"],
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{
      SubscriptBox["\[Nu]", "4"], 
      SubscriptBox["\[Nu]", "5"]}]]}], "-", 
   RowBox[{"4", " ", 
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "1"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "2"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "3"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "5"],
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{
      SubscriptBox["\[Nu]", "4"], 
      SubscriptBox["\[Nu]", "6"]}]]}], "+", 
   RowBox[{"4", " ", 
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "1"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "2"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "3"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "4"],
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{
      SubscriptBox["\[Nu]", "5"], 
      SubscriptBox["\[Nu]", "6"]}]]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970207756699*^9},
 CellTags->"DiracSimplify",
 CellLabel->"Out[450]="]
}, Open  ]],

Cell["\<\
 With the option DiracCanonical an alphabetic ordering is done.\
\>", "Text",
 CellTags->"DiracSimplify"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracSimplify", "[", 
  RowBox[{
   RowBox[{"GA", "[", 
    RowBox[{"\[Nu]", ",", "\[Mu]"}], "]"}], ",", 
   RowBox[{"DiracCanonical", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"DiracSimplify",
 CellLabel->"In[451]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"2", " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Nu]"}]]}], "-", 
   RowBox[{
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], ".", 
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm]]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970207922428*^9},
 CellTags->"DiracSimplify",
 CellLabel->"Out[451]="]
}, Open  ]],

Cell["\<\
Setting InsideDiracTrace\[Rule]True assumes that a trace is still to be taken \
later on.\
\>", "Text",
 CellTags->"DiracSimplify"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracSimplify", "[", 
  RowBox[{
   RowBox[{"GA", "[", 
    RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]", ",", "\[Sigma]"}], "]"}], 
   ",", 
   RowBox[{"InsideDiracTrace", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"DiracSimplify",
 CellLabel->"In[452]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Sigma]"}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Nu]", "\[Rho]"}]]}], "-", 
   RowBox[{
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Rho]"}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Nu]", "\[Sigma]"}]]}], "+", 
   RowBox[{
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Rho]", "\[Sigma]"}]]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970208145904*^9},
 CellTags->"DiracSimplify",
 CellLabel->"Out[452]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracSimplify", "[", 
  RowBox[{
   RowBox[{"GA", "[", 
    RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]"}], "]"}], ",", 
   RowBox[{"InsideDiracTrace", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"DiracSimplify",
 CellLabel->"In[453]:="],

Cell[BoxData[
 FormBox["0", TraditionalForm]], "Output",
 CellChangeTimes->{3.500970208324831*^9},
 CellTags->"DiracSimplify",
 CellLabel->"Out[453]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"Clear", "[", 
  RowBox[{"t1", ",", "t2", ",", "t3", ",", "t4", ",", "t5", ",", "t6"}], 
  "]"}]], "Input",
 CellTags->"DiracSimplify",
 CellLabel->"In[454]:="]
}, Open  ]],

Cell["DiracSimplify2", "ObjectName",
 CellTags->"DiracSimplify2"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"DiracSimplify2"],

Cell[TextData[{
 "DiracSimplify2[exp] simplifies the Dirac structure but leaves any ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["\[Gamma]", "5"], TraditionalForm]]],
 " untouched."
}], "Text",
 CellTags->"DiracSimplify2"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["DiracSimplify",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracSimplify",
  ButtonNote->"DiracSimplify"],
 "."
}], "Text",
 CellTags->"DiracSimplify2"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"DiracSimplify2"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GAD", "[", 
  RowBox[{
  "\[Mu]", ",", "\[Nu]", ",", "\[Mu]", ",", "5", ",", "\[Alpha]", ",", 
   "\[Beta]", ",", "\[Alpha]"}], "]"}]], "Input",
 CellTags->"DiracSimplify2",
 CellLabel->"In[455]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
   SuperscriptBox["\[Gamma]", "\[Nu]"], ".", 
   SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
   SuperscriptBox["\[Gamma]", "5"], ".", 
   SuperscriptBox["\[Gamma]", "\[Alpha]"], ".", 
   SuperscriptBox["\[Gamma]", "\[Beta]"], ".", 
   SuperscriptBox["\[Gamma]", "\[Alpha]"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970208451681*^9},
 CellTags->"DiracSimplify2",
 CellLabel->"Out[455]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracSimplify2", "[", "%", "]"}]], "Input",
 CellTags->"DiracSimplify2",
 CellLabel->"In[456]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], ".", 
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Nu]",
      TraditionalForm],
     TraditionalForm]], ".", 
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], ".", 
   SuperscriptBox["\[Gamma]", "5"], ".", 
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Alpha]",
      TraditionalForm],
     TraditionalForm]], ".", 
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Beta]",
      TraditionalForm],
     TraditionalForm]], ".", 
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Alpha]",
      TraditionalForm],
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.50097020856417*^9},
 CellTags->"DiracSimplify2",
 CellLabel->"Out[456]="]
}, Open  ]]
}, Open  ]],

Cell["DiracSlash", "ObjectName",
 CellTags->"DiracSlash"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"DiracSlash"],

Cell[TextData[{
 "DiracSlash[p] is the contraction ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["p", "\[Mu]"], 
    SubscriptBox["\[Gamma]", 
     RowBox[{"\[Mu]", " "}]]}], TraditionalForm]]],
 "(FourVector[p, ",
 Cell[BoxData[
  FormBox["\[Mu]", TraditionalForm]]],
 "] DiracMatrix[",
 Cell[BoxData[
  FormBox["\[Mu]", TraditionalForm]]],
 "]). Products of those can be entered in the form DiracSlash[p1, p2, ...]."
}], "Text",
 CellTags->"DiracSlash"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "DiracSlash", "]"}]], "Input",
 CellTags->"DiracSlash",
 CellLabel->"In[457]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Dimension", "\[Rule]", "4"}], ",", 
    RowBox[{"FeynCalcInternal", "\[Rule]", "False"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970208678238*^9},
 CellTags->"DiracSlash",
 CellLabel->"Out[457]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["DiracGammaExpand",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracGammaExpand",
  ButtonNote->"DiracGammaExpand"],
 ", ",
 ButtonBox["DiracGamma",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracGamma",
  ButtonNote->"DiracGamma"],
 ", ",
 ButtonBox["DiracMatrix",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracMatrix",
  ButtonNote->"DiracMatrix"],
 ", ",
 ButtonBox["DiracSimplify",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracSimplify",
  ButtonNote->"DiracSimplify"],
 ", ",
 ButtonBox["DiracTrick",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracTrick",
  ButtonNote->"DiracTrick"],
 ", ",
 ButtonBox["GS",
  BaseStyle->"AddOnsLink",
  ButtonData:>"GS",
  ButtonNote->"GS"],
 ", ",
 ButtonBox["GSD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"GSD",
  ButtonNote->"GSD"],
 "."
}], "Text",
 CellTags->"DiracSlash"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"DiracSlash"],

Cell[TextData[{
 "This is q-slash, i.e., ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["\[Gamma]", "\[Mu]"], 
    RowBox[{
     SubscriptBox["q", "\[Mu]"], "."}]}], TraditionalForm]]]
}], "Text",
 CellTags->"DiracSlash"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracSlash", "[", "q", "]"}]], "Input",
 CellTags->"DiracSlash",
 CellLabel->"In[458]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox["\<\"\[Gamma]\"\>",
    TraditionalForm], "\[CenterDot]", 
   FormBox["q",
    TraditionalForm]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009702087988863`*^9},
 CellTags->"DiracSlash",
 CellLabel->"Out[458]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"DiracSlash", "[", "p", "]"}], ".", 
  RowBox[{"DiracSlash", "[", "q", "]"}]}]], "Input",
 CellTags->"DiracSlash",
 CellLabel->"In[459]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     FormBox["\<\"\[Gamma]\"\>",
      TraditionalForm], "\[CenterDot]", 
     FormBox["p",
      TraditionalForm]}], ")"}], ".", 
   RowBox[{"(", 
    RowBox[{
     FormBox["\<\"\[Gamma]\"\>",
      TraditionalForm], "\[CenterDot]", 
     FormBox["q",
      TraditionalForm]}], ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970208996703*^9},
 CellTags->"DiracSlash",
 CellLabel->"Out[459]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracSlash", "[", 
  RowBox[{"p", ",", "q"}], "]"}]], "Input",
 CellTags->"DiracSlash",
 CellLabel->"In[460]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     FormBox["\<\"\[Gamma]\"\>",
      TraditionalForm], "\[CenterDot]", 
     FormBox["p",
      TraditionalForm]}], ")"}], ".", 
   RowBox[{"(", 
    RowBox[{
     FormBox["\<\"\[Gamma]\"\>",
      TraditionalForm], "\[CenterDot]", 
     FormBox["q",
      TraditionalForm]}], ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009702090955133`*^9},
 CellTags->"DiracSlash",
 CellLabel->"Out[460]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GS", "[", "p", "]"}]], "Input",
 CellTags->"DiracSlash",
 CellLabel->"In[461]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"\[Gamma]", "\[CenterDot]", "p"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009702091868277`*^9},
 CellTags->"DiracSlash",
 CellLabel->"Out[461]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"DiracSlash", "[", "q", "]"}], "//", "StandardForm"}]], "Input",
 CellTags->"DiracSlash",
 CellLabel->"In[462]:="],

Cell[BoxData[
 RowBox[{"DiracSlash", "[", "q", "]"}]], "Output",
 CellChangeTimes->{3.500970209250772*^9},
 CellTags->"DiracSlash",
 CellLabel->"Out[462]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"DiracSlash", "[", 
   RowBox[{"q", ",", 
    RowBox[{"Dimension", "\[Rule]", "n"}]}], "]"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"DiracSlash",
 CellLabel->"In[463]:="],

Cell[BoxData[
 RowBox[{"DiracSlash", "[", 
  RowBox[{"q", ",", 
   RowBox[{"Dimension", "\[Rule]", "n"}]}], "]"}]], "Output",
 CellChangeTimes->{3.500970209327012*^9},
 CellTags->"DiracSlash",
 CellLabel->"Out[463]//StandardForm="]
}, Open  ]]
}, Open  ]],

Cell["DiracSpinor", "ObjectName",
 CellTags->"DiracSpinor"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"DiracSpinor"],

Cell["\<\
DiracSpinor is simply a quantity defined as noncommutative (with \
DeclareNonCommutative[DiracSpinor]). The convention intended is that \
DiracSpinor[p, m, ind] is a Dirac spinor for a fermion with momentum p and \
mass m and indices ind.\
\>", "Text",
 CellTags->"DiracSpinor"],

Cell[TextData[{
 "See also: ",
 ButtonBox["Spinor",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Spinor",
  ButtonNote->"Spinor"],
 "."
}], "Text",
 CellTags->"DiracSpinor"]
}, Open  ]],

Cell["DiracSubstitute67", "ObjectName",
 CellTags->"DiracSubstitute67"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"DiracSubstitute67"],

Cell["\<\
DiracSubstitute67 is an option for DiracSimplify. If set to True the \
chirality-projectors DiracGamma[6] and DiracGamma[7] are substituted by their \
definitions.\
\>", "Text",
 CellTags->"DiracSubstitute67"],

Cell[TextData[{
 "See also: ",
 ButtonBox["DiracGamma",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracGamma",
  ButtonNote->"DiracGamma"],
 ", ",
 ButtonBox["DiracSimplify",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracSimplify",
  ButtonNote->"DiracSimplify"],
 "."
}], "Text",
 CellTags->"DiracSubstitute67"]
}, Open  ]],

Cell["DiracTrace", "ObjectName",
 CellTags->"DiracTrace"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"DiracTrace"],

Cell["\<\
DiracTrace[expr] is the head of Dirac traces. Whether the trace is evaluated \
depends on the option DiracTraceEvaluate. Direct trace evaluation should be \
performed with TR (or Tr). The argument expr may be a product of Dirac \
matrices or slashes separated by the Mathematica Dot (.).\
\>", "Text",
 CellTags->"DiracTrace"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "DiracTrace", "]"}]], "Input",
 CellTags->"DiracTrace",
 CellLabel->"In[464]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"EpsContract", "\[Rule]", "False"}], ",", 
    RowBox[{"Factoring", "\[Rule]", "False"}], ",", 
    RowBox[{"FeynCalcExternal", "\[Rule]", "False"}], ",", 
    RowBox[{"Mandelstam", "\[Rule]", 
     RowBox[{"{", "}"}]}], ",", 
    RowBox[{"PairCollect", "\[Rule]", "True"}], ",", 
    RowBox[{"DiracTraceEvaluate", "\[Rule]", "False"}], ",", 
    RowBox[{"Schouten", "\[Rule]", "0"}], ",", 
    RowBox[{"LeviCivitaSign", "\[RuleDelayed]", "$LeviCivitaSign"}], ",", 
    RowBox[{"TraceOfOne", "\[Rule]", "4"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970209943727*^9},
 CellTags->"DiracTrace",
 CellLabel->"Out[464]="]
}, Open  ]],

Cell[TextData[{
 "For comments regarding ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["\[Gamma]", "5"], TraditionalForm]]],
 "schemes see the notes for ",
 ButtonBox["TR",
  BaseStyle->"AddOnsLink",
  ButtonData:>"TR",
  ButtonNote->"TR"],
 "."
}], "Text",
 CellTags->"DiracTrace"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["TR",
  BaseStyle->"AddOnsLink",
  ButtonData:>"TR",
  ButtonNote->"TR"],
 "."
}], "Text",
 CellTags->"DiracTrace"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"DiracTrace"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracTrace", "[", 
  RowBox[{"GA", "[", 
   RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}], "]"}]], "Input",
 CellTags->"DiracTrace",
 CellLabel->"In[465]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"tr", "(", 
   FormBox[
    RowBox[{
     SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
     SuperscriptBox["\[Gamma]", "\[Nu]"]}],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970210020626*^9},
 CellTags->"DiracTrace",
 CellLabel->"Out[465]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracTrace", "[", 
  RowBox[{"GA", "[", 
   RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]", ",", "\[Sigma]"}], "]"}], 
  "]"}]], "Input",
 CellTags->"DiracTrace",
 CellLabel->"In[466]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"tr", "(", 
   FormBox[
    RowBox[{
     SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
     SuperscriptBox["\[Gamma]", "\[Nu]"], ".", 
     SuperscriptBox["\[Gamma]", "\[Rho]"], ".", 
     SuperscriptBox["\[Gamma]", "\[Sigma]"]}],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970210096796*^9},
 CellTags->"DiracTrace",
 CellLabel->"Out[466]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", " ", "/.", " ", 
  RowBox[{"DiracTrace", "\[Rule]", "Tr"}]}]], "Input",
 CellTags->"DiracTrace",
 CellLabel->"In[467]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"4", " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Sigma]"}]], " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Nu]", "\[Rho]"}]]}], "-", 
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Rho]"}]], " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Nu]", "\[Sigma]"}]]}], "+", 
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Rho]", "\[Sigma]"}]]}]}], ")"}]}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.50097021017828*^9},
 CellTags->"DiracTrace",
 CellLabel->"Out[467]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracTrace", "[", 
  RowBox[{
   RowBox[{"GA", "[", 
    RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]", ",", "\[Sigma]", ",", "5"}],
     "]"}], ",", 
   RowBox[{"DiracTraceEvaluate", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"DiracTrace",
 CellLabel->"In[468]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", "4"}], " ", "\[ImaginaryI]", " ", 
   SuperscriptBox["\[Epsilon]", 
    RowBox[{
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["\[Rho]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["\[Sigma]",
       TraditionalForm],
      TraditionalForm]}]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970210260747*^9},
 CellTags->"DiracTrace",
 CellLabel->"Out[468]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracTrace", "[", 
  RowBox[{
   RowBox[{"GA", "[", 
    RowBox[{
    "\[Mu]", ",", "\[Nu]", ",", "\[Rho]", ",", "\[Sigma]", ",", "\[Delta]", 
     ",", "\[Tau]", ",", "5"}], "]"}], ",", 
   RowBox[{"DiracTraceEvaluate", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"DiracTrace",
 CellLabel->"In[469]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"4", " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"\[ImaginaryI]", " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Delta]", "\[Mu]"}]], " ", 
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Sigma]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Tau]",
          TraditionalForm],
         TraditionalForm]}]]}], "-", 
     RowBox[{"\[ImaginaryI]", " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Delta]", "\[Nu]"}]], " ", 
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Sigma]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Tau]",
          TraditionalForm],
         TraditionalForm]}]]}], "+", 
     RowBox[{"\[ImaginaryI]", " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Delta]", "\[Rho]"}]], " ", 
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Sigma]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Tau]",
          TraditionalForm],
         TraditionalForm]}]]}], "-", 
     RowBox[{"\[ImaginaryI]", " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Delta]", "\[Sigma]"}]], " ", 
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Tau]",
          TraditionalForm],
         TraditionalForm]}]]}], "-", 
     RowBox[{"\[ImaginaryI]", " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Delta]", "\[Tau]"}]], " ", 
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Sigma]",
          TraditionalForm],
         TraditionalForm]}]]}], "-", 
     RowBox[{"\[ImaginaryI]", " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         FormBox["\[Delta]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Sigma]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Tau]",
          TraditionalForm],
         TraditionalForm]}]]}], "+", 
     RowBox[{"\[ImaginaryI]", " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Rho]"}]], " ", 
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         FormBox["\[Delta]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Sigma]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Tau]",
          TraditionalForm],
         TraditionalForm]}]]}], "-", 
     RowBox[{"\[ImaginaryI]", " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Sigma]"}]], " ", 
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         FormBox["\[Delta]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Tau]",
          TraditionalForm],
         TraditionalForm]}]]}], "+", 
     RowBox[{"\[ImaginaryI]", " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Tau]"}]], " ", 
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         FormBox["\[Delta]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Sigma]",
          TraditionalForm],
         TraditionalForm]}]]}], "-", 
     RowBox[{"\[ImaginaryI]", " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Nu]", "\[Rho]"}]], " ", 
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         FormBox["\[Delta]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Sigma]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Tau]",
          TraditionalForm],
         TraditionalForm]}]]}], "+", 
     RowBox[{"\[ImaginaryI]", " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Nu]", "\[Sigma]"}]], " ", 
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         FormBox["\[Delta]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Tau]",
          TraditionalForm],
         TraditionalForm]}]]}], "-", 
     RowBox[{"\[ImaginaryI]", " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Nu]", "\[Tau]"}]], " ", 
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         FormBox["\[Delta]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Sigma]",
          TraditionalForm],
         TraditionalForm]}]]}], "-", 
     RowBox[{"\[ImaginaryI]", " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Rho]", "\[Sigma]"}]], " ", 
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         FormBox["\[Delta]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Tau]",
          TraditionalForm],
         TraditionalForm]}]]}], "+", 
     RowBox[{"\[ImaginaryI]", " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Rho]", "\[Tau]"}]], " ", 
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         FormBox["\[Delta]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Sigma]",
          TraditionalForm],
         TraditionalForm]}]]}], "-", 
     RowBox[{"\[ImaginaryI]", " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Sigma]", "\[Tau]"}]], " ", 
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         FormBox["\[Delta]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm]}]]}]}], ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009702104478893`*^9},
 CellTags->"DiracTrace",
 CellLabel->"Out[469]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracTrace", "[", 
  RowBox[{"GS", "[", 
   RowBox[{"p", ",", "q", ",", "r", ",", "s"}], "]"}], "]"}]], "Input",
 CellTags->"DiracTrace",
 CellLabel->"In[470]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"tr", "(", 
   FormBox[
    RowBox[{
     RowBox[{"(", 
      RowBox[{"\[Gamma]", "\[CenterDot]", "p"}], ")"}], ".", 
     RowBox[{"(", 
      RowBox[{"\[Gamma]", "\[CenterDot]", "q"}], ")"}], ".", 
     RowBox[{"(", 
      RowBox[{"\[Gamma]", "\[CenterDot]", "r"}], ")"}], ".", 
     RowBox[{"(", 
      RowBox[{"\[Gamma]", "\[CenterDot]", "s"}], ")"}]}],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970210551824*^9},
 CellTags->"DiracTrace",
 CellLabel->"Out[470]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracTrace", "[", 
  RowBox[{
   RowBox[{"GA", "[", 
    RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}], ",", 
   RowBox[{"DiracTraceEvaluate", "\[Rule]", "True"}], ",", 
   RowBox[{"FCE", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"DiracTrace",
 CellLabel->"In[471]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"4", " ", 
   SuperscriptBox["g", 
    RowBox[{"\[Mu]", "\[Nu]"}]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970210693276*^9},
 CellTags->"DiracTrace",
 CellLabel->"Out[471]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "StandardForm"}]], "Input",
 CellTags->"DiracTrace",
 CellLabel->"In[472]:="],

Cell[BoxData[
 RowBox[{"4", " ", 
  RowBox[{"MT", "[", 
   RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}]}]], "Output",
 CellChangeTimes->{3.500970210843672*^9},
 CellTags->"DiracTrace",
 CellLabel->"Out[472]//StandardForm="]
}, Open  ]]
}, Open  ]],

Cell["DiracTraceEvaluate", "ObjectName",
 CellTags->"DiracTraceEvaluate"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"DiracTraceEvaluate"],

Cell["\<\
DiracTraceEvaluate is an option for DiracTrace and Tr. If set to False, \
DiracTrace remains unevaluated.\
\>", "Text",
 CellTags->"DiracTraceEvaluate"],

Cell[TextData[{
 "See also: ",
 ButtonBox["DiracTrace",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracTrace",
  ButtonNote->"DiracTrace"],
 ", ",
 ButtonBox["TR",
  BaseStyle->"AddOnsLink",
  ButtonData:>"TR",
  ButtonNote->"TR"],
 "."
}], "Text",
 CellTags->"DiracTraceEvaluate"]
}, Open  ]],

Cell["DiracTrick", "ObjectName",
 CellTags->"DiracTrick"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"DiracTrick"],

Cell["\<\
DiracTrick[exp] contracts gamma matrices with each other and performs several \
simplifications, but no expansion, use Calc or DiracSimplify for \
non-commutative expansion.\
\>", "Text",
 CellTags->"DiracTrick"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "DiracTrick", "]"}]], "Input",
 CellTags->"DiracTrick",
 CellLabel->"In[473]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"Expanding", "\[Rule]", "False"}], "}"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.5009702108949738`*^9},
 CellTags->"DiracTrick",
 CellLabel->"Out[473]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Calc",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Calc",
  ButtonNote->"Calc"],
 ", ",
 ButtonBox["DiracGammaExpand",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracGammaExpand",
  ButtonNote->"DiracGammaExpand"],
 ", ",
 ButtonBox["DiracSimplify",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracSimplify",
  ButtonNote->"DiracSimplify"],
 "."
}], "Text",
 CellTags->"DiracTrick"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"DiracTrick"],

Cell["This is a string of Dirac matrices in four dimensions. ", "Text",
 CellTags->"DiracTrick"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t1", "=", 
  RowBox[{"GA", "[", 
   RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Mu]"}], "]"}]}]], "Input",
 CellTags->"DiracTrick",
 CellLabel->"In[474]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
   SuperscriptBox["\[Gamma]", "\[Nu]"], ".", 
   SuperscriptBox["\[Gamma]", "\[Mu]"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970210932832*^9},
 CellTags->"DiracTrick",
 CellLabel->"Out[474]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracTrick", "[", "t1", "]"}]], "Input",
 CellTags->"DiracTrick",
 CellLabel->"In[475]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
   SuperscriptBox["\[Gamma]", "\[Nu]"], ".", 
   SuperscriptBox["\[Gamma]", "\[Mu]"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970211013039*^9},
 CellTags->"DiracTrick",
 CellLabel->"Out[475]="]
}, Open  ]],

Cell["This is a string of Dirac matrices in D dimensions. ", "Text",
 CellTags->"DiracTrick"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t2", "=", 
  RowBox[{"GAD", "[", 
   RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Mu]"}], "]"}]}]], "Input",
 CellTags->"DiracTrick",
 CellLabel->"In[476]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
   SuperscriptBox["\[Gamma]", "\[Nu]"], ".", 
   SuperscriptBox["\[Gamma]", "\[Mu]"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970211054741*^9},
 CellTags->"DiracTrick",
 CellLabel->"Out[476]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracTrick", "[", "t2", "]"}]], "Input",
 CellTags->"DiracTrick",
 CellLabel->"In[477]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
   SuperscriptBox["\[Gamma]", "\[Nu]"], ".", 
   SuperscriptBox["\[Gamma]", "\[Mu]"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009702110959387`*^9},
 CellTags->"DiracTrick",
 CellLabel->"Out[477]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t3", "=", 
  RowBox[{"GA", "[", 
   RowBox[{"5", ",", "\[Mu]", ",", "\[Nu]"}], "]"}]}]], "Input",
 CellTags->"DiracTrick",
 CellLabel->"In[478]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", "5"], ".", 
   SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
   SuperscriptBox["\[Gamma]", "\[Nu]"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970211177288*^9},
 CellTags->"DiracTrick",
 CellLabel->"Out[478]="]
}, Open  ]],

Cell[TextData[{
 "By default ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["\[Gamma]", "5"], TraditionalForm]]],
 "is moved to the right."
}], "Text",
 CellTags->"DiracTrick"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracTrick", "[", "t3", "]"}]], "Input",
 CellTags->"DiracTrick",
 CellLabel->"In[479]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", "5"], ".", 
   SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
   SuperscriptBox["\[Gamma]", "\[Nu]"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009702112190733`*^9},
 CellTags->"DiracTrick",
 CellLabel->"Out[479]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t4", "=", 
  RowBox[{"GA", "[", 
   RowBox[{"6", ",", "\[Mu]", ",", "7"}], "]"}]}]], "Input",
 CellTags->"DiracTrick",
 CellLabel->"In[480]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", "6"], ".", 
   SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
   SuperscriptBox["\[Gamma]", "7"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970211260401*^9},
 CellTags->"DiracTrick",
 CellLabel->"Out[480]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracTrick", "[", "t4", "]"}]], "Input",
 CellTags->"DiracTrick",
 CellLabel->"In[481]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", "6"], ".", 
   SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
   SuperscriptBox["\[Gamma]", "7"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970211302711*^9},
 CellTags->"DiracTrick",
 CellLabel->"Out[481]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t5", "=", 
  RowBox[{
   RowBox[{"GS", "[", 
    RowBox[{"a", "+", "b"}], "]"}], " ", ".", " ", 
   RowBox[{"GS", "[", "p", "]"}], ".", 
   RowBox[{"GS", "[", "p", "]"}], ".", 
   RowBox[{"GS", "[", 
    RowBox[{"c", "+", "d"}], "]"}]}]}]], "Input",
 CellTags->"DiracTrick",
 CellLabel->"In[482]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"(", 
    RowBox[{"\[Gamma]", "\[CenterDot]", "(", 
     FormBox[
      RowBox[{"a", "+", "b"}],
      TraditionalForm], ")"}], ")"}], ".", 
   RowBox[{"(", 
    RowBox[{"\[Gamma]", "\[CenterDot]", "p"}], ")"}], ".", 
   RowBox[{"(", 
    RowBox[{"\[Gamma]", "\[CenterDot]", "p"}], ")"}], ".", 
   RowBox[{"(", 
    RowBox[{"\[Gamma]", "\[CenterDot]", "(", 
     FormBox[
      RowBox[{"c", "+", "d"}],
      TraditionalForm], ")"}], ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970211393386*^9},
 CellTags->"DiracTrick",
 CellLabel->"Out[482]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracTrick", "[", "t5", "]"}]], "Input",
 CellTags->"DiracTrick",
 CellLabel->"In[483]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"(", 
    RowBox[{"\[Gamma]", "\[CenterDot]", "(", 
     FormBox[
      RowBox[{"a", "+", "b"}],
      TraditionalForm], ")"}], ")"}], ".", 
   RowBox[{"(", 
    RowBox[{"\[Gamma]", "\[CenterDot]", "p"}], ")"}], ".", 
   RowBox[{"(", 
    RowBox[{"\[Gamma]", "\[CenterDot]", "p"}], ")"}], ".", 
   RowBox[{"(", 
    RowBox[{"\[Gamma]", "\[CenterDot]", "(", 
     FormBox[
      RowBox[{"c", "+", "d"}],
      TraditionalForm], ")"}], ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970211495038*^9},
 CellTags->"DiracTrick",
 CellLabel->"Out[483]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Calc", "[", "t5", "]"}]], "Input",
 CellTags->"DiracTrick",
 CellLabel->"In[484]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SuperscriptBox[
     FormBox["p",
      TraditionalForm], "2"], " ", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       FormBox["\<\"\[Gamma]\"\>",
        TraditionalForm], "\[CenterDot]", 
       FormBox["a",
        TraditionalForm]}], ")"}], ".", 
     RowBox[{"(", 
      RowBox[{
       FormBox["\<\"\[Gamma]\"\>",
        TraditionalForm], "\[CenterDot]", 
       FormBox["c",
        TraditionalForm]}], ")"}]}]}], "+", 
   RowBox[{
    SuperscriptBox[
     FormBox["p",
      TraditionalForm], "2"], " ", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       FormBox["\<\"\[Gamma]\"\>",
        TraditionalForm], "\[CenterDot]", 
       FormBox["a",
        TraditionalForm]}], ")"}], ".", 
     RowBox[{"(", 
      RowBox[{
       FormBox["\<\"\[Gamma]\"\>",
        TraditionalForm], "\[CenterDot]", 
       FormBox["d",
        TraditionalForm]}], ")"}]}]}], "+", 
   RowBox[{
    SuperscriptBox[
     FormBox["p",
      TraditionalForm], "2"], " ", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       FormBox["\<\"\[Gamma]\"\>",
        TraditionalForm], "\[CenterDot]", 
       FormBox["b",
        TraditionalForm]}], ")"}], ".", 
     RowBox[{"(", 
      RowBox[{
       FormBox["\<\"\[Gamma]\"\>",
        TraditionalForm], "\[CenterDot]", 
       FormBox["c",
        TraditionalForm]}], ")"}]}]}], "+", 
   RowBox[{
    SuperscriptBox[
     FormBox["p",
      TraditionalForm], "2"], " ", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       FormBox["\<\"\[Gamma]\"\>",
        TraditionalForm], "\[CenterDot]", 
       FormBox["b",
        TraditionalForm]}], ")"}], ".", 
     RowBox[{"(", 
      RowBox[{
       FormBox["\<\"\[Gamma]\"\>",
        TraditionalForm], "\[CenterDot]", 
       FormBox["d",
        TraditionalForm]}], ")"}]}]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970211960903*^9},
 CellTags->"DiracTrick",
 CellLabel->"Out[484]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GAD", "@@", 
  RowBox[{"Join", "[", 
   RowBox[{
    RowBox[{"{", "\[Mu]", "}"}], ",", 
    RowBox[{"Table", "[", 
     RowBox[{
      SubscriptBox["\[Nu]", "i"], ",", 
      RowBox[{"{", 
       RowBox[{"i", ",", "6"}], "}"}]}], "]"}], ",", 
    RowBox[{"{", "\[Mu]", "}"}]}], "]"}]}]], "Input",
 CellTags->"DiracTrick",
 CellLabel->"In[485]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
   SuperscriptBox["\[Gamma]", 
    SubscriptBox["\[Nu]", "1"]], ".", 
   SuperscriptBox["\[Gamma]", 
    SubscriptBox["\[Nu]", "2"]], ".", 
   SuperscriptBox["\[Gamma]", 
    SubscriptBox["\[Nu]", "3"]], ".", 
   SuperscriptBox["\[Gamma]", 
    SubscriptBox["\[Nu]", "4"]], ".", 
   SuperscriptBox["\[Gamma]", 
    SubscriptBox["\[Nu]", "5"]], ".", 
   SuperscriptBox["\[Gamma]", 
    SubscriptBox["\[Nu]", "6"]], ".", 
   SuperscriptBox["\[Gamma]", "\[Mu]"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970212097728*^9},
 CellTags->"DiracTrick",
 CellLabel->"Out[485]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracTrick", "[", "%", "]"}]], "Input",
 CellTags->"DiracTrick",
 CellLabel->"In[486]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
   SuperscriptBox["\[Gamma]", 
    SubscriptBox["\[Nu]", "1"]], ".", 
   SuperscriptBox["\[Gamma]", 
    SubscriptBox["\[Nu]", "2"]], ".", 
   SuperscriptBox["\[Gamma]", 
    SubscriptBox["\[Nu]", "3"]], ".", 
   SuperscriptBox["\[Gamma]", 
    SubscriptBox["\[Nu]", "4"]], ".", 
   SuperscriptBox["\[Gamma]", 
    SubscriptBox["\[Nu]", "5"]], ".", 
   SuperscriptBox["\[Gamma]", 
    SubscriptBox["\[Nu]", "6"]], ".", 
   SuperscriptBox["\[Gamma]", "\[Mu]"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970212173347*^9},
 CellTags->"DiracTrick",
 CellLabel->"Out[486]="]
}, Open  ]]
}, Open  ]],

Cell["Divideout", "ObjectName",
 CellTags->"Divideout"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Divideout"],

Cell["\<\
Divideout is an option for OPEInt and OPEInsert. The setting is divided out \
at the end.\
\>", "Text",
 CellTags->"Divideout"],

Cell[TextData[{
 "See also: ",
 ButtonBox["OPEInt",
  BaseStyle->"AddOnsLink",
  ButtonData:>"OPEInt",
  ButtonNote->"OPEInt"],
 ", ",
 ButtonBox["OPEInsert",
  BaseStyle->"AddOnsLink",
  ButtonData:>"OPEInsert",
  ButtonNote->"OPEInsert"],
 "."
}], "Text",
 CellTags->"Divideout"]
}, Open  ]],

Cell["DOT", "ObjectName",
 CellTags->"DOT"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"DOT"],

Cell["\<\
DOT[a, b, ...] is the FeynCalc function for non-commutative multiplication. \
By default it is set to the Mathematica Dot functions. By setting
DOT=.
this can be disabled. Note that then non-commutative products should to be \
entered like DOT[ DiracMatrix[mu], m + DiracSlash[p], DiracMatrix[mu] ], etc.\
\
\>", "Text",
 CellTags->"DOT"],

Cell[TextData[{
 "See also: ",
 ButtonBox["DotSimplify",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DotSimplify",
  ButtonNote->"DotSimplify"],
 "."
}], "Text",
 CellTags->"DOT"]
}, Open  ]],

Cell["DotExpand", "ObjectName",
 CellTags->"DotExpand"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"DotExpand"],

Cell["DotExpand[expr] expands DOT products in expr.", "Text",
 CellTags->"DotExpand"],

Cell[TextData[{
 "See also: ",
 ButtonBox["DOT",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DOT",
  ButtonNote->"DOT"],
 ", ",
 ButtonBox["DotSimplify",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DotSimplify",
  ButtonNote->"DotSimplify"],
 ", ",
 ButtonBox["DeclareNonCommutative",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DeclareNonCommutative",
  ButtonNote->"DeclareNonCommutative"],
 ", ",
 ButtonBox["UnDeclareNonCommutative",
  BaseStyle->"AddOnsLink",
  ButtonData:>"UnDeclareNonCommutative",
  ButtonNote->"UnDeclareNonCommutative"],
 "."
}], "Text",
 CellTags->"DotExpand"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"DotExpand"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DotExpand", "[", 
  RowBox[{"DOT", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"a", " ", "x"}], "+", 
     RowBox[{"b", " ", "y"}], "+", 
     RowBox[{"c", " ", "z"}]}], ",", 
    RowBox[{"d", "+", "e", "+", "f"}]}], "]"}], "]"}]], "Input",
 CellTags->"DotExpand",
 CellLabel->"In[487]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"a", " ", "d", " ", "x"}], "+", 
   RowBox[{"a", " ", "e", " ", "x"}], "+", 
   RowBox[{"a", " ", "f", " ", "x"}], "+", 
   RowBox[{"b", " ", "d", " ", "y"}], "+", 
   RowBox[{"b", " ", "e", " ", "y"}], "+", 
   RowBox[{"b", " ", "f", " ", "y"}], "+", 
   RowBox[{"c", " ", "d", " ", "z"}], "+", 
   RowBox[{"c", " ", "e", " ", "z"}], "+", 
   RowBox[{"c", " ", "f", " ", "z"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970212295582*^9},
 CellTags->"DotExpand",
 CellLabel->"Out[487]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"DeclareNonCommutative", "/@", 
   RowBox[{"{", 
    RowBox[{"a", ",", "b", ",", "c", ",", "d", ",", "e", ",", "f"}], "}"}]}], 
  ";"}]], "Input",
 CellTags->"DotExpand",
 CellLabel->"In[488]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DotExpand", "[", 
  RowBox[{"DOT", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"a", " ", "x"}], "+", 
     RowBox[{"b", " ", "y"}], "+", 
     RowBox[{"c", " ", "z"}]}], ",", 
    RowBox[{"d", "+", "e", "+", "f"}]}], "]"}], "]"}]], "Input",
 CellTags->"DotExpand",
 CellLabel->"In[489]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"x", " ", 
    RowBox[{"a", ".", "d"}]}], "+", 
   RowBox[{"x", " ", 
    RowBox[{"a", ".", "e"}]}], "+", 
   RowBox[{"x", " ", 
    RowBox[{"a", ".", "f"}]}], "+", 
   RowBox[{"y", " ", 
    RowBox[{"b", ".", "d"}]}], "+", 
   RowBox[{"y", " ", 
    RowBox[{"b", ".", "e"}]}], "+", 
   RowBox[{"y", " ", 
    RowBox[{"b", ".", "f"}]}], "+", 
   RowBox[{"z", " ", 
    RowBox[{"c", ".", "d"}]}], "+", 
   RowBox[{"z", " ", 
    RowBox[{"c", ".", "e"}]}], "+", 
   RowBox[{"z", " ", 
    RowBox[{"c", ".", "f"}]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970213122171*^9},
 CellTags->"DotExpand",
 CellLabel->"Out[489]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"UnDeclareNonCommutative", "/@", 
   RowBox[{"{", 
    RowBox[{"a", ",", "b", ",", "c", ",", "d", ",", "e", ",", "f"}], "}"}]}], 
  ";"}]], "Input",
 CellTags->"DotExpand",
 CellLabel->"In[490]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DotExpand", "[", 
  RowBox[{"DOT", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"a", " ", "x"}], "+", 
     RowBox[{"b", " ", "y"}], "+", 
     RowBox[{"c", " ", "z"}]}], ",", 
    RowBox[{"d", "+", "e", "+", "f"}]}], "]"}], "]"}]], "Input",
 CellTags->"DotExpand",
 CellLabel->"In[491]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"a", " ", "d", " ", "x"}], "+", 
   RowBox[{"a", " ", "e", " ", "x"}], "+", 
   RowBox[{"a", " ", "f", " ", "x"}], "+", 
   RowBox[{"b", " ", "d", " ", "y"}], "+", 
   RowBox[{"b", " ", "e", " ", "y"}], "+", 
   RowBox[{"b", " ", "f", " ", "y"}], "+", 
   RowBox[{"c", " ", "d", " ", "z"}], "+", 
   RowBox[{"c", " ", "e", " ", "z"}], "+", 
   RowBox[{"c", " ", "f", " ", "z"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970213625683*^9},
 CellTags->"DotExpand",
 CellLabel->"Out[491]="]
}, Open  ]]
}, Open  ]],

Cell["DotPower", "ObjectName",
 CellTags->"DotPower"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"DotPower"],

Cell["\<\
DotPower is an option for DotSimplify. It determines whether non-commutative \
powers are represented by succesive multiplication or by Power.\
\>", "Text",
 CellTags->"DotPower"],

Cell[TextData[{
 "See also: ",
 ButtonBox["DotSimplify",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DotSimplify",
  ButtonNote->"DotSimplify"],
 "."
}], "Text",
 CellTags->"DotPower"]
}, Open  ]],

Cell["DotProduct", "ObjectName",
 CellTags->"DotProduct"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"DotProduct"],

Cell["\<\
DotProduct[x, y] denotes the three-dimensional dot-product. If x and y have \
Head List, DotProduct[x, a] (where a is a vector) performs Sum[ x[[k]] \
a[[k]], {k, 0, 3}].\
\>", "Text",
 CellTags->"DotProduct"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["CrossProduct",
  BaseStyle->"AddOnsLink",
  ButtonData:>"CrossProduct",
  ButtonNote->"CrossProduct"],
 ", ",
 ButtonBox["ThreeVector",
  BaseStyle->"AddOnsLink",
  ButtonData:>"ThreeVector",
  ButtonNote->"ThreeVector"],
 "."
}], "Text",
 CellTags->"DotProduct"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"DotProduct"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DotProduct", "[", 
  RowBox[{
   RowBox[{"ThreeVector", "[", "a", "]"}], ",", 
   RowBox[{"3", 
    RowBox[{"ThreeVector", "[", "b", "]"}]}]}], "]"}]], "Input",
 CellTags->"DotProduct",
 CellLabel->"In[492]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"3", " ", 
   RowBox[{
    FormBox[
     OverscriptBox["a", "\[RightVector]"],
     TraditionalForm], "\[CenterDot]", 
    FormBox[
     OverscriptBox["b", "\[RightVector]"],
     TraditionalForm]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970213769113*^9},
 CellTags->"DotProduct",
 CellLabel->"Out[492]="]
}, Open  ]]
}, Open  ]],

Cell["DotSimplifyRelations", "ObjectName",
 CellTags->"DotSimplifyRelations"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"DotSimplifyRelations"],

Cell["\<\
DotSimplifyRelations is an option for DotSimplify. Its setting may be a list \
of substitution rules of the form DotSimplifyRelations \[Rule] {a . b \[Rule] \
c, b^2 \[Rule] 0, ...}.\
\>", "Text",
 CellTags->"DotSimplifyRelations"],

Cell[TextData[{
 "See also: ",
 ButtonBox["DotSimplify",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DotSimplify",
  ButtonNote->"DotSimplify"],
 "."
}], "Text",
 CellTags->"DotSimplifyRelations"]
}, Open  ]],

Cell["DotSimplify", "ObjectName",
 CellTags->"DotSimplify"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"DotSimplify"],

Cell["\<\
DotSimplify[expr] expands and reorders noncommutative terms in expr. \
Simplifying relations may be specified by the option DotSimplifyRelations or \
by Commutator and AntiCommutator definitions. Whether expr is expanded \
noncommutatively depends on the option Expanding.\
\>", "Text",
 CellTags->"DotSimplify"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "DotSimplify", "]"}]], "Input",
 CellTags->"DotSimplify",
 CellLabel->"In[493]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Expanding", "\[Rule]", "True"}], ",", 
    RowBox[{"DotSimplifyRelations", "\[Rule]", 
     RowBox[{"{", "}"}]}], ",", 
    RowBox[{"DotPower", "\[Rule]", "False"}], ",", 
    RowBox[{"FeynCalcInternal", "\[Rule]", "True"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970213857963*^9},
 CellTags->"DotSimplify",
 CellLabel->"Out[493]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["AntiCommutator",
  BaseStyle->"AddOnsLink",
  ButtonData:>"AntiCommutator",
  ButtonNote->"AntiCommutator"],
 ", ",
 ButtonBox["Commutator",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Commutator",
  ButtonNote->"Commutator"],
 ", ",
 ButtonBox["Calc",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Calc",
  ButtonNote->"Calc"],
 "."
}], "Text",
 CellTags->"DotSimplify"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"DotSimplify"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t1", "=", 
  RowBox[{
   RowBox[{"GA", "[", "\[Mu]", "]"}], ".", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"2", " ", 
      RowBox[{"GS", "[", "p", "]"}]}], "-", 
     RowBox[{"GS", "[", "q", "]"}]}], ")"}], ".", 
   RowBox[{"GA", "[", "\[Nu]", "]"}]}]}]], "Input",
 CellTags->"DotSimplify",
 CellLabel->"In[494]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"2", " ", 
      RowBox[{"\[Gamma]", "\[CenterDot]", "p"}]}], "-", 
     RowBox[{"\[Gamma]", "\[CenterDot]", "q"}]}], ")"}], ".", 
   SuperscriptBox["\[Gamma]", "\[Nu]"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970213952342*^9},
 CellTags->"DotSimplify",
 CellLabel->"Out[494]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DotSimplify", "[", "t1", "]"}]], "Input",
 CellTags->"DotSimplify",
 CellLabel->"In[495]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"2", " ", 
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], ".", 
     RowBox[{"(", 
      RowBox[{
       FormBox["\<\"\[Gamma]\"\>",
        TraditionalForm], "\[CenterDot]", 
       FormBox["p",
        TraditionalForm]}], ")"}], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm]]}]}], "-", 
   RowBox[{
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], ".", 
    RowBox[{"(", 
     RowBox[{
      FormBox["\<\"\[Gamma]\"\>",
       TraditionalForm], "\[CenterDot]", 
      FormBox["q",
       TraditionalForm]}], ")"}], ".", 
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm]]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009702140779047`*^9},
 CellTags->"DotSimplify",
 CellLabel->"Out[495]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"DeclareNonCommutative", "[", 
  RowBox[{"a", ",", "b", ",", "c"}], "]"}]], "Input",
 CellTags->"DotSimplify",
 CellLabel->"In[496]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t2", "=", 
  RowBox[{"a", ".", 
   RowBox[{"(", 
    RowBox[{"b", "-", 
     RowBox[{"z", " ", "c"}]}], ")"}], ".", "a"}]}]], "Input",
 CellTags->"DotSimplify",
 CellLabel->"In[497]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"a", ".", 
   RowBox[{"(", 
    RowBox[{"b", "-", 
     RowBox[{"c", " ", "z"}]}], ")"}], ".", "a"}], TraditionalForm]], "Output",\

 CellChangeTimes->{3.500970214461557*^9},
 CellTags->"DotSimplify",
 CellLabel->"Out[497]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DotSimplify", "[", "t2", "]"}]], "Input",
 CellTags->"DotSimplify",
 CellLabel->"In[498]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"a", ".", "b", ".", "a"}], "-", 
   RowBox[{"z", " ", 
    RowBox[{"a", ".", "c", ".", "a"}]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970214597857*^9},
 CellTags->"DotSimplify",
 CellLabel->"Out[498]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Commutator", "[", 
   RowBox[{"a", ",", "c"}], "]"}], "=", "1"}]], "Input",
 CellTags->"DotSimplify",
 CellLabel->"In[499]:="],

Cell[BoxData[
 FormBox["1", TraditionalForm]], "Output",
 CellChangeTimes->{3.500970214772262*^9},
 CellTags->"DotSimplify",
 CellLabel->"Out[499]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DotSimplify", "[", "t2", "]"}]], "Input",
 CellTags->"DotSimplify",
 CellLabel->"In[500]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"a", ".", "b", ".", "a"}], "-", 
   RowBox[{"z", " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"c", ".", "a", ".", "a"}], "+", "a"}], ")"}]}]}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970215062153*^9},
 CellTags->"DotSimplify",
 CellLabel->"Out[500]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"Commutator", "[", 
   RowBox[{"a", ",", "c"}], "]"}], "=", "."}]], "Input",
 CellTags->"DotSimplify",
 CellLabel->"In[501]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DotSimplify", "[", "t2", "]"}]], "Input",
 CellTags->"DotSimplify",
 CellLabel->"In[502]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"a", ".", "b", ".", "a"}], "-", 
   RowBox[{"z", " ", 
    RowBox[{"a", ".", "c", ".", "a"}]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970215145084*^9},
 CellTags->"DotSimplify",
 CellLabel->"Out[502]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"AntiCommutator", "[", 
   RowBox[{"b", ",", "a"}], "]"}], "=", "c"}]], "Input",
 CellTags->"DotSimplify",
 CellLabel->"In[503]:="],

Cell[BoxData[
 FormBox["c", TraditionalForm]], "Output",
 CellChangeTimes->{3.5009702152671423`*^9},
 CellTags->"DotSimplify",
 CellLabel->"Out[503]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DotSimplify", "[", "t2", "]"}]], "Input",
 CellTags->"DotSimplify",
 CellLabel->"In[504]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    RowBox[{"a", ".", "a", ".", "b"}]}], "-", 
   RowBox[{"z", " ", 
    RowBox[{"a", ".", "c", ".", "a"}]}], "+", 
   RowBox[{"a", ".", "c"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970215402853*^9},
 CellTags->"DotSimplify",
 CellLabel->"Out[504]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"AntiCommutator", "[", 
   RowBox[{"b", ",", "a"}], "]"}], "=", "."}]], "Input",
 CellTags->"DotSimplify",
 CellLabel->"In[505]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DotSimplify", "[", 
  RowBox[{"t2", ",", 
   RowBox[{"DotSimplifyRelations", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"a", ".", "c"}], "\[Rule]", 
      RowBox[{"1", "/", "z"}]}], "}"}]}]}], "]"}]], "Input",
 CellTags->"DotSimplify",
 CellLabel->"In[506]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"a", ".", "b", ".", "a"}], "-", "a"}], TraditionalForm]], "Output",\

 CellChangeTimes->{3.500970215501441*^9},
 CellTags->"DotSimplify",
 CellLabel->"Out[506]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"DeclareNonCommutative", "[", "x", "]"}]], "Input",
 CellTags->"DotSimplify",
 CellLabel->"In[507]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DotSimplify", "[", 
  RowBox[{"x", ".", "x", ".", "x"}], "]"}]], "Input",
 CellTags->"DotSimplify",
 CellLabel->"In[508]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"x", ".", "x", ".", "x"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970215634953*^9},
 CellTags->"DotSimplify",
 CellLabel->"Out[508]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DotSimplify", "[", 
  RowBox[{
   RowBox[{"x", ".", "x", ".", "x"}], ",", 
   RowBox[{"DotPower", "\[Rule]", "False"}]}], "]"}]], "Input",
 CellTags->"DotSimplify",
 CellLabel->"In[509]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"x", ".", "x", ".", "x"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009702157407637`*^9},
 CellTags->"DotSimplify",
 CellLabel->"Out[509]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"UnDeclareNonCommutative", "[", 
  RowBox[{"a", ",", "b", ",", "c", ",", "x"}], "]"}]], "Input",
 CellTags->"DotSimplify",
 CellLabel->"In[510]:="]
}, Open  ]],

Cell["DummyIndex", "ObjectName",
 CellTags->"DummyIndex"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"DummyIndex"],

Cell["\<\
DummyIndex is an option of CovariantD specifying an index to use as dummy \
summation index. If set to Automatic, unique indices are generated\
\>", "Text",
 CellTags->"DummyIndex"],

Cell[TextData[{
 "See also: ",
 ButtonBox["CovariantD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"CovariantD",
  ButtonNote->"CovariantD"],
 "."
}], "Text",
 CellTags->"DummyIndex"]
}, Open  ]],

Cell["D0", "ObjectName",
 CellTags->"D0"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"D0"],

Cell[TextData[{
 "D0[ p10, p12, p23, p30, p20, p13, m1^2, m2^2, m3^2, m4^2 ] is the \
Passarino-Veltman ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["D", "0"], TraditionalForm]]],
 " function. The convention for the arguments is that if the denominator of \
the integrand has the form ([q^2-m1^2] [(q+p1)^2-m2^2] [(q+p2)^2-m3^2] \
[(q+p3)^2-m4^2] ), the first six arguments of D0 are the scalar products p10 \
= p1^2, p12 = (p1-p2)^2, p23 = (p2-p3)^2, p30 = p3^2, p20 = p2^2, p13 = \
(p1-p3)^2."
}], "Text",
 CellTags->"D0"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["B0",
  BaseStyle->"AddOnsLink",
  ButtonData:>"B0",
  ButtonNote->"B0"],
 ", ",
 ButtonBox["C0",
  BaseStyle->"AddOnsLink",
  ButtonData:>"C0",
  ButtonNote->"C0"],
 ", ",
 ButtonBox["PaVe",
  BaseStyle->"AddOnsLink",
  ButtonData:>"PaVe",
  ButtonNote->"PaVe"],
 ", ",
 ButtonBox["PaVeOrder",
  BaseStyle->"AddOnsLink",
  ButtonData:>"PaVeOrder",
  ButtonNote->"PaVeOrder"],
 "."
}], "Text",
 CellTags->"D0"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"D0"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"D0", "[", 
  RowBox[{
  "p10", ",", "p12", ",", "p23", ",", "p30", ",", "p20", ",", "p13", ",", 
   RowBox[{"m1", "^", "2"}], ",", 
   RowBox[{"m2", "^", "2"}], ",", 
   RowBox[{"m3", "^", "2"}], ",", 
   RowBox[{"m4", "^", "2"}]}], "]"}]], "Input",
 CellTags->"D0",
 CellLabel->"In[511]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox[
    SubscriptBox["\<\"D\"\>", "\<\"0\"\>"],
    TraditionalForm], "(", "p10", ",", "p12", ",", "p23", ",", "p30", ",", 
   "p20", ",", "p13", ",", 
   FormBox[
    SuperscriptBox["m1", "2"],
    TraditionalForm], ",", 
   FormBox[
    SuperscriptBox["m2", "2"],
    TraditionalForm], ",", 
   FormBox[
    SuperscriptBox["m3", "2"],
    TraditionalForm], ",", 
   FormBox[
    SuperscriptBox["m4", "2"],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970216008623*^9},
 CellTags->"D0",
 CellLabel->"Out[511]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PaVeOrder", "[", 
  RowBox[{
   RowBox[{"D0", "[", 
    RowBox[{
    "p10", ",", "p12", ",", "p23", ",", "p30", ",", "p20", ",", "p13", ",", 
     RowBox[{"m1", "^", "2"}], ",", 
     RowBox[{"m2", "^", "2"}], ",", 
     RowBox[{"m3", "^", "2"}], ",", 
     RowBox[{"m4", "^", "2"}]}], "]"}], ",", 
   RowBox[{"PaVeOrderList", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"p13", ",", "p20"}], "}"}]}]}], "]"}]], "Input",
 CellTags->"D0",
 CellLabel->"In[512]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox[
    SubscriptBox["\<\"D\"\>", "\<\"0\"\>"],
    TraditionalForm], "(", "p10", ",", "p30", ",", "p23", ",", "p12", ",", 
   "p13", ",", "p20", ",", 
   FormBox[
    SuperscriptBox["m2", "2"],
    TraditionalForm], ",", 
   FormBox[
    SuperscriptBox["m1", "2"],
    TraditionalForm], ",", 
   FormBox[
    SuperscriptBox["m4", "2"],
    TraditionalForm], ",", 
   FormBox[
    SuperscriptBox["m3", "2"],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970216100016*^9},
 CellTags->"D0",
 CellLabel->"Out[512]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PaVeOrder", "[", "%", "]"}]], "Input",
 CellTags->"D0",
 CellLabel->"In[513]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox[
    SubscriptBox["\<\"D\"\>", "\<\"0\"\>"],
    TraditionalForm], "(", "p10", ",", "p12", ",", "p23", ",", "p30", ",", 
   "p20", ",", "p13", ",", 
   FormBox[
    SuperscriptBox["m1", "2"],
    TraditionalForm], ",", 
   FormBox[
    SuperscriptBox["m2", "2"],
    TraditionalForm], ",", 
   FormBox[
    SuperscriptBox["m3", "2"],
    TraditionalForm], ",", 
   FormBox[
    SuperscriptBox["m4", "2"],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009702162035513`*^9},
 CellTags->"D0",
 CellLabel->"Out[513]="]
}, Open  ]]
}, Open  ]],

Cell["D0Convention", "ObjectName",
 CellTags->"D0Convention"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"D0Convention"],

Cell["\<\
D0Convention is an option for Write2. If set to 1, the convention for the \
arguments of D0 is changed when writing a Fortran file with Write2: The fifth \
and sixth argument of D0 are interchanged and the square root is taken of the \
last four arguments.\
\>", "Text",
 CellTags->"D0Convention"],

Cell[TextData[{
 "See also: ",
 ButtonBox["D0",
  BaseStyle->"AddOnsLink",
  ButtonData:>"D0",
  ButtonNote->"D0"],
 ", ",
 ButtonBox["Write2",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Write2",
  ButtonNote->"Write2"],
 "."
}], "Text",
 CellTags->"D0Convention"]
}, Open  ]],

Cell["Eps", "ObjectName",
 CellTags->"Eps"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Eps"],

Cell[TextData[{
 "Eps[a, b, c, d] is the head of the totally antisymmetric ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", "\[Epsilon]"}], TraditionalForm]]],
 " (Levi-Civita) tensor. The a,b, ... may have head LorentzIndex, Momentum or \
Integer. In case of integers the Levi-Civita tensor is evaluated immediately. "
}], "Text",
 CellTags->"Eps"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "Eps", "]"}]], "Input",
 CellTags->"Eps",
 CellLabel->"In[514]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"Dimension", "\[Rule]", "4"}], "}"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009702163074627`*^9},
 CellTags->"Eps",
 CellLabel->"Out[514]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["EpsEvaluate",
  BaseStyle->"AddOnsLink",
  ButtonData:>"EpsEvaluate",
  ButtonNote->"EpsEvaluate"],
 ", ",
 ButtonBox["LC",
  BaseStyle->"AddOnsLink",
  ButtonData:>"LC",
  ButtonNote->"LC"],
 ", ",
 ButtonBox["LCD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"LCD",
  ButtonNote->"LCD"],
 ", ",
 ButtonBox["LeviCivita",
  BaseStyle->"AddOnsLink",
  ButtonData:>"LeviCivita",
  ButtonNote->"LeviCivita"],
 "."
}], "Text",
 CellTags->"Eps"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Eps"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Eps", "[", 
  RowBox[{
   RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
   RowBox[{"LorentzIndex", "[", "\[Nu]", "]"}], ",", 
   RowBox[{"LorentzIndex", "[", "\[Rho]", "]"}], ",", 
   RowBox[{"LorentzIndex", "[", "\[Sigma]", "]"}]}], "]"}]], "Input",
 CellTags->"Eps",
 CellLabel->"In[515]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["\[Epsilon]", 
   RowBox[{
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["\[Nu]",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["\[Rho]",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["\[Sigma]",
      TraditionalForm],
     TraditionalForm]}]], TraditionalForm]], "Output",
 CellChangeTimes->{3.50097021636369*^9},
 CellTags->"Eps",
 CellLabel->"Out[515]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Eps", "[", 
  RowBox[{
   RowBox[{"Momentum", "[", "p", "]"}], ",", 
   RowBox[{"LorentzIndex", "[", "\[Nu]", "]"}], ",", 
   RowBox[{"LorentzIndex", "[", "\[Rho]", "]"}], ",", 
   RowBox[{"LorentzIndex", "[", "\[Sigma]", "]"}]}], "]"}]], "Input",
 CellTags->"Eps",
 CellLabel->"In[516]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["\[Epsilon]", 
   RowBox[{
    FormBox["p",
     TraditionalForm], 
    FormBox[
     FormBox["\[Nu]",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["\[Rho]",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["\[Sigma]",
      TraditionalForm],
     TraditionalForm]}]], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970216417161*^9},
 CellTags->"Eps",
 CellLabel->"Out[516]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Eps", "[", 
   RowBox[{"b", ",", "a", ",", "c", ",", "d"}], "]"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"Eps",
 CellLabel->"In[517]:="],

Cell[BoxData[
 RowBox[{"-", 
  RowBox[{"Eps", "[", 
   RowBox[{"a", ",", "b", ",", "c", ",", "d"}], "]"}]}]], "Output",
 CellChangeTimes->{3.500970216470199*^9},
 CellTags->"Eps",
 CellLabel->"Out[517]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Eps", "[", 
  RowBox[{"0", ",", "1", ",", "2", ",", "3"}], "]"}]], "Input",
 CellTags->"Eps",
 CellLabel->"In[518]:="],

Cell[BoxData[
 FormBox["1", TraditionalForm]], "Output",
 CellChangeTimes->{3.500970216519319*^9},
 CellTags->"Eps",
 CellLabel->"Out[518]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Eps", "[", 
  RowBox[{"1", ",", "0", ",", "2", ",", "3"}], "]"}]], "Input",
 CellTags->"Eps",
 CellLabel->"In[519]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", "1"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009702165995903`*^9},
 CellTags->"Eps",
 CellLabel->"Out[519]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"SetOptions", "[", 
   RowBox[{"Eps", ",", 
    RowBox[{"Dimension", "\[Rule]", "4"}]}], "]"}], ";"}]], "Input",
 CellTags->"Eps",
 CellLabel->"In[520]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"a1", " ", "=", 
  RowBox[{"Eps", "[", 
   RowBox[{
    RowBox[{"LorentzIndex", "[", 
     RowBox[{"\[Mu]", ",", "D"}], "]"}], ",", 
    RowBox[{"LorentzIndex", "[", 
     RowBox[{"\[Nu]", ",", "D"}], "]"}], ",", 
    RowBox[{"LorentzIndex", "[", 
     RowBox[{"\[Rho]", ",", "D"}], "]"}], ",", 
    RowBox[{"LorentzIndex", "[", 
     RowBox[{"\[Sigma]", ",", "D"}], "]"}]}], "]"}]}]], "Input",
 CellTags->"Eps",
 CellLabel->"In[521]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["\[Epsilon]", 
   RowBox[{
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["\[Nu]",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["\[Rho]",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["\[Sigma]",
      TraditionalForm],
     TraditionalForm]}]], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970216657036*^9},
 CellTags->"Eps",
 CellLabel->"Out[521]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Contract", "[", "a1a1", "]"}]], "Input",
 CellTags->"Eps",
 CellLabel->"In[522]:="],

Cell[BoxData[
 FormBox["a1a1", TraditionalForm]], "Output",
 CellChangeTimes->{3.500970216826532*^9},
 CellTags->"Eps",
 CellLabel->"Out[522]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"SetOptions", "[", 
   RowBox[{"Eps", ",", 
    RowBox[{"Dimension", "\[Rule]", "D"}]}], "]"}], ";"}]], "Input",
 CellTags->"Eps",
 CellLabel->"In[523]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"a2", " ", "=", 
  RowBox[{"Eps", "[", 
   RowBox[{
    RowBox[{"LorentzIndex", "[", 
     RowBox[{"\[Mu]", ",", "D"}], "]"}], ",", 
    RowBox[{"LorentzIndex", "[", 
     RowBox[{"\[Nu]", ",", "D"}], "]"}], ",", 
    RowBox[{"LorentzIndex", "[", 
     RowBox[{"\[Rho]", ",", "D"}], "]"}], ",", 
    RowBox[{"LorentzIndex", "[", 
     RowBox[{"\[Sigma]", ",", "D"}], "]"}]}], "]"}]}]], "Input",
 CellTags->"Eps",
 CellLabel->"In[524]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["\[Epsilon]", 
   RowBox[{
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["\[Nu]",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["\[Rho]",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["\[Sigma]",
      TraditionalForm],
     TraditionalForm]}]], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970216942718*^9},
 CellTags->"Eps",
 CellLabel->"Out[524]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Contract", "[", "a2a2", "]"}], "//", "Factor2"}]], "Input",
 CellTags->"Eps",
 CellLabel->"In[525]:="],

Cell[BoxData[
 FormBox["a2a2", TraditionalForm]], "Output",
 CellChangeTimes->{3.500970217035555*^9},
 CellTags->"Eps",
 CellLabel->"Out[525]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{" ", 
  RowBox[{
   RowBox[{"LCD", "[", 
    RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]", ",", "\[Alpha]"}], "]"}], 
   ".", 
   RowBox[{"GAD", "[", 
    RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]", ",", "\[Alpha]"}], 
    "]"}]}]}]], "Input",
 CellLabel->"In[526]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Epsilon]", 
    RowBox[{"\[Mu]", "\[Nu]", "\[Rho]", "\[Alpha]"}]], ".", 
   SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
   SuperscriptBox["\[Gamma]", "\[Nu]"], ".", 
   SuperscriptBox["\[Gamma]", "\[Rho]"], ".", 
   SuperscriptBox["\[Gamma]", "\[Alpha]"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009702170824823`*^9},
 CellLabel->"Out[526]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "FCI"}]], "Input",
 CellLabel->"In[527]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Epsilon]", 
    RowBox[{
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["\[Rho]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["\[Alpha]",
       TraditionalForm],
      TraditionalForm]}]], ".", 
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], ".", 
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Nu]",
      TraditionalForm],
     TraditionalForm]], ".", 
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Rho]",
      TraditionalForm],
     TraditionalForm]], ".", 
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Alpha]",
      TraditionalForm],
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009702171325197`*^9},
 CellLabel->"Out[527]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"g5", "=", 
  RowBox[{
   RowBox[{
    RowBox[{"-", 
     FractionBox["\[ImaginaryI]", "24"]}], 
    RowBox[{
     RowBox[{"LCD", "[", 
      RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]", ",", "\[Alpha]"}], "]"}], 
     ".", 
     RowBox[{"GAD", "[", 
      RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]", ",", "\[Alpha]"}], 
      "]"}]}]}], "//", "FCI"}]}]], "Input",
 CellTags->"Eps",
 CellLabel->"In[528]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    FractionBox["1", "24"]}], " ", "\[ImaginaryI]", " ", 
   RowBox[{
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Rho]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Alpha]",
        TraditionalForm],
       TraditionalForm]}]], ".", 
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], ".", 
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm]], ".", 
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox["\[Rho]",
       TraditionalForm],
      TraditionalForm]], ".", 
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox["\[Alpha]",
       TraditionalForm],
      TraditionalForm]]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009702174086742`*^9},
 CellTags->"Eps",
 CellLabel->"Out[528]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"g5p", "=", 
  RowBox[{
   RowBox[{
    RowBox[{"-", 
     FractionBox["\[ImaginaryI]", "24"]}], " ", 
    RowBox[{
     RowBox[{"LCD", "[", 
      RowBox[{
       SuperscriptBox["\[Mu]", "\[Prime]",
        MultilineFunction->None], ",", 
       SuperscriptBox["\[Nu]", "\[Prime]",
        MultilineFunction->None], ",", 
       SuperscriptBox["\[Rho]", "\[Prime]",
        MultilineFunction->None], ",", 
       SuperscriptBox["\[Alpha]", "\[Prime]",
        MultilineFunction->None]}], "]"}], ".", 
     RowBox[{"GAD", "[", 
      RowBox[{
       SuperscriptBox["\[Mu]", "\[Prime]",
        MultilineFunction->None], ",", 
       SuperscriptBox["\[Nu]", "\[Prime]",
        MultilineFunction->None], ",", 
       SuperscriptBox["\[Rho]", "\[Prime]",
        MultilineFunction->None], ",", 
       SuperscriptBox["\[Alpha]", "\[Prime]",
        MultilineFunction->None]}], "]"}]}]}], "//", "FCI"}]}]], "Input",
 CellTags->"Eps",
 CellLabel->"In[529]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    FractionBox["1", "24"]}], " ", "\[ImaginaryI]", " ", 
   RowBox[{
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox[
        SuperscriptBox["\[Mu]", "\[Prime]",
         MultilineFunction->None],
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox[
        SuperscriptBox["\[Nu]", "\[Prime]",
         MultilineFunction->None],
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox[
        SuperscriptBox["\[Rho]", "\[Prime]",
         MultilineFunction->None],
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox[
        SuperscriptBox["\[Alpha]", "\[Prime]",
         MultilineFunction->None],
        TraditionalForm],
       TraditionalForm]}]], ".", 
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox[
       SuperscriptBox["\[Mu]", "\[Prime]",
        MultilineFunction->None],
       TraditionalForm],
      TraditionalForm]], ".", 
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox[
       SuperscriptBox["\[Nu]", "\[Prime]",
        MultilineFunction->None],
       TraditionalForm],
      TraditionalForm]], ".", 
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox[
       SuperscriptBox["\[Rho]", "\[Prime]",
        MultilineFunction->None],
       TraditionalForm],
      TraditionalForm]], ".", 
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox[
       SuperscriptBox["\[Alpha]", "\[Prime]",
        MultilineFunction->None],
       TraditionalForm],
      TraditionalForm]]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970217645076*^9},
 CellTags->"Eps",
 CellLabel->"Out[529]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"g52", "=", 
  RowBox[{"Factor2", "[", 
   RowBox[{"Calc", "[", 
    RowBox[{"g5", ".", "g5p"}], "]"}], "]"}]}]], "Input",
 CellTags->"Eps",
 CellLabel->"In[530]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    FractionBox["1", "24"]}], " ", 
   RowBox[{"(", 
    RowBox[{"1", "-", "D"}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{"2", "-", "D"}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{"3", "-", "D"}], ")"}], " ", "D"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970218089423*^9},
 CellTags->"Eps",
 CellLabel->"Out[530]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"g52", "/.", 
  RowBox[{"D", "\[Rule]", "4"}]}]], "Input",
 CellTags->"Eps",
 CellLabel->"In[531]:="],

Cell[BoxData[
 FormBox["1", TraditionalForm]], "Output",
 CellChangeTimes->{3.500970218247573*^9},
 CellTags->"Eps",
 CellLabel->"Out[531]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"Clear", "[", 
  RowBox[{"a1", ",", "a2", ",", "g5", ",", "g5p", ",", "g52"}], 
  "]"}]], "Input",
 CellTags->"Eps",
 CellLabel->"In[532]:="]
}, Open  ]],

Cell["EpsChisholm", "ObjectName",
 CellTags->"EpsChisholm"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"EpsChisholm"],

Cell["\<\
EpsChisholm[expr] substitutes for a gamma matrix contracted with a \
Levi-Civita tensor (Eps) the Chisholm identity.\
\>", "Text",
 CellTags->"EpsChisholm"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Chisholm",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Chisholm",
  ButtonNote->"Chisholm"],
 "."
}], "Text",
 CellTags->"EpsChisholm"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"EpsChisholm"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Chisholm", "[", 
  RowBox[{"GA", "[", 
   RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]", ",", "\[Sigma]"}], "]"}], 
  "]"}]], "Input",
 CellTags->"EpsChisholm",
 CellLabel->"In[533]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    RowBox[{"\[ImaginaryI]", " ", 
     RowBox[{
      SuperscriptBox["\[Gamma]", 
       FormBox[
        FormBox["$MU$4872",
         TraditionalForm],
        TraditionalForm]], ".", 
      SuperscriptBox["\[Gamma]", 
       FormBox[
        FormBox["\[Sigma]",
         TraditionalForm],
        TraditionalForm]], ".", 
      SuperscriptBox["\[Gamma]", "5"]}], " ", 
     SuperscriptBox["\[Epsilon]", 
      RowBox[{
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["\[Rho]",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["$MU$4872",
         TraditionalForm],
        TraditionalForm]}]]}]}], "+", 
   RowBox[{
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Rho]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Nu]"}]]}], "-", 
   RowBox[{
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Rho]"}]]}], "+", 
   RowBox[{
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Nu]", "\[Rho]"}]]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970218545363*^9},
 CellTags->"EpsChisholm",
 CellLabel->"Out[533]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"EpsChisholm", "[", "%", "]"}]], "Input",
 CellTags->"EpsChisholm",
 CellLabel->"In[534]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], ".", 
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Nu]",
      TraditionalForm],
     TraditionalForm]], ".", 
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Rho]",
      TraditionalForm],
     TraditionalForm]], ".", 
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Sigma]",
      TraditionalForm],
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970218655767*^9},
 CellTags->"EpsChisholm",
 CellLabel->"Out[534]="]
}, Open  ]]
}, Open  ]],

Cell["EpsContract", "ObjectName",
 CellTags->"EpsContract"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"EpsContract"],

Cell["\<\
EpsContract is an option of Contract specifying whether Levi-Civita tensors \
Eps[...] will be contracted, i.e., products of two  Eps are replaced via the \
determinant formula.\
\>", "Text",
 CellTags->"EpsContract"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Eps",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Eps",
  ButtonNote->"Eps"],
 ", ",
 ButtonBox["Contract",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Contract",
  ButtonNote->"Contract"],
 "."
}], "Text",
 CellTags->"EpsContract"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"EpsContract"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"a1", " ", "=", 
  RowBox[{"Eps", "[", 
   RowBox[{
    RowBox[{"LorentzIndex", "[", 
     RowBox[{"\[Mu]", ",", "D"}], "]"}], ",", 
    RowBox[{"LorentzIndex", "[", 
     RowBox[{"\[Nu]", ",", "D"}], "]"}], ",", 
    RowBox[{"LorentzIndex", "[", 
     RowBox[{"\[Rho]", ",", "D"}], "]"}], ",", 
    RowBox[{"LorentzIndex", "[", 
     RowBox[{"\[Sigma]", ",", "D"}], "]"}]}], "]"}]}]], "Input",
 CellTags->"EpsContract",
 CellLabel->"In[535]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["\[Epsilon]", 
   RowBox[{
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["\[Nu]",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["\[Rho]",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["\[Sigma]",
      TraditionalForm],
     TraditionalForm]}]], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970218743188*^9},
 CellTags->"EpsContract",
 CellLabel->"Out[535]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Contract", "[", 
  RowBox[{
   RowBox[{"a1", " ", "a1"}], ",", 
   RowBox[{"EpsContract", "\[Rule]", "False"}]}], "]"}]], "Input",
 CellTags->"EpsContract",
 CellLabel->"In[536]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox[
   RowBox[{"(", 
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Rho]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm]}]], ")"}], "2"], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009702188454742`*^9},
 CellTags->"EpsContract",
 CellLabel->"Out[536]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Contract", "[", 
  RowBox[{
   RowBox[{"a1", " ", "a1"}], ",", 
   RowBox[{"EpsContract", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"EpsContract",
 CellLabel->"In[537]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    SuperscriptBox["D", "4"]}], "+", 
   RowBox[{"6", " ", 
    SuperscriptBox["D", "3"]}], "-", 
   RowBox[{"11", " ", 
    SuperscriptBox["D", "2"]}], "+", 
   RowBox[{"6", " ", "D"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970218941085*^9},
 CellTags->"EpsContract",
 CellLabel->"Out[537]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"Clear", "[", "a1", "]"}]], "Input",
 CellLabel->"In[538]:="]
}, Open  ]],

Cell["EpsDiscard", "ObjectName",
 CellTags->"EpsDiscard"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"EpsDiscard"],

Cell["\<\
EpsDiscard is an option for FeynCalc2FORM. If set to True all Levi-Civita \
tensors are replaced by 0 after contraction.\
\>", "Text",
 CellTags->"EpsDiscard"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FeynCalc2FORM",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FeynCalc2FORM",
  ButtonNote->"FeynCalc2FORM"],
 "."
}], "Text",
 CellTags->"EpsDiscard"]
}, Open  ]],

Cell["EpsEvaluate", "ObjectName",
 CellTags->"EpsEvaluate"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"EpsEvaluate"],

Cell["\<\
EpsEvaluate[expr] applies total antisymmetry and linearity (w.r.t. \
Momentum's) to all Levi-Civita tensors (Eps') in expr.\
\>", "Text",
 CellTags->"EpsEvaluate"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Contract",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Contract",
  ButtonNote->"Contract"],
 ", ",
 ButtonBox["Eps",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Eps",
  ButtonNote->"Eps"],
 ", ",
 ButtonBox["LeviCivita",
  BaseStyle->"AddOnsLink",
  ButtonData:>"LeviCivita",
  ButtonNote->"LeviCivita"],
 ", ",
 ButtonBox["Trick",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Trick",
  ButtonNote->"Trick"],
 "."
}], "Text",
 CellTags->"EpsEvaluate"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"EpsEvaluate"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Trick", "[", 
  RowBox[{
   RowBox[{"LeviCivita", "[", 
    RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]", ",", "\[Sigma]"}], "]"}], 
   " ", 
   RowBox[{"FourVector", "[", 
    RowBox[{
     RowBox[{"p", "+", "q"}], ",", "\[Sigma]"}], "]"}]}], "]"}]], "Input",
 CellTags->"EpsEvaluate",
 CellLabel->"In[539]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["\[Epsilon]", 
   RowBox[{
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["\[Nu]",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["\[Rho]",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     RowBox[{"p", "+", "q"}],
     TraditionalForm]}]], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970219063863*^9},
 CellTags->"EpsEvaluate",
 CellLabel->"Out[539]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"EpsEvaluate", "[", "%", "]"}]], "Input",
 CellTags->"EpsEvaluate",
 CellLabel->"In[540]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["\[Epsilon]", 
   RowBox[{
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["\[Nu]",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["\[Rho]",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     RowBox[{"p", "+", "q"}],
     TraditionalForm]}]], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970219149049*^9},
 CellTags->"EpsEvaluate",
 CellLabel->"Out[540]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StandardForm", "[", "%", "]"}]], "Input",
 CellTags->"EpsEvaluate",
 CellLabel->"In[541]:="],

Cell[BoxData[
 RowBox[{"Eps", "[", 
  RowBox[{
   RowBox[{"LorentzIndex", "[", 
    RowBox[{"\[Mu]", ",", "D"}], "]"}], ",", 
   RowBox[{"LorentzIndex", "[", 
    RowBox[{"\[Nu]", ",", "D"}], "]"}], ",", 
   RowBox[{"LorentzIndex", "[", 
    RowBox[{"\[Rho]", ",", "D"}], "]"}], ",", 
   RowBox[{"Momentum", "[", 
    RowBox[{"p", "+", "q"}], "]"}]}], "]"}]], "Output",
 CellChangeTimes->{3.500970219463649*^9},
 CellTags->"EpsEvaluate",
 CellLabel->"Out[541]//StandardForm="]
}, Open  ]]
}, Open  ]],

Cell["EpsUncontract", "ObjectName",
 CellTags->"EpsUncontract"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"EpsUncontract"],

Cell["\<\
EpsUncontract does Uncontract on scalar products involving Eps.\
\>", "Text",
 CellTags->"EpsUncontract"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Eps",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Eps",
  ButtonNote->"Eps"],
 ", ",
 ButtonBox["Uncontract",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Uncontract",
  ButtonNote->"Uncontract"],
 "."
}], "Text",
 CellTags->"EpsUncontract"]
}, Open  ]],

Cell["Epsilon", "ObjectName",
 CellTags->"Epsilon"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Epsilon"],

Cell[TextData[{
 "Epsilon is (",
 StyleBox["n",
  FontSlant->"Italic"],
 "-4), where ",
 StyleBox["n",
  FontSlant->"Italic"],
 " is the space-time dimension. Epsilon stands for a small positive number."
}], "Text",
 CellTags->"Epsilon"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Series2",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Series2",
  ButtonNote->"Series2"],
 "."
}], "Text",
 CellTags->"Epsilon"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Epsilon"],

Cell[CellGroupData[{

Cell[BoxData["Epsilon"], "Input",
 CellTags->"Epsilon",
 CellLabel->"In[542]:="],

Cell[BoxData[
 FormBox[
  TagBox["\[CurlyEpsilon]",
   TraditionalForm], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970219512917*^9},
 CellTags->"Epsilon",
 CellLabel->"Out[542]="]
}, Open  ]],

Cell["\<\
Epsilon has no functional properties, but some upvalues are changed:\
\>", "Text",
 CellTags->"Epsilon"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"Re", "[", "Epsilon", "]"}], " ", ">", " ", 
    RowBox[{"-", "4"}]}], ",", 
   RowBox[{
    RowBox[{"Re", "[", "Epsilon", "]"}], " ", ">", " ", 
    RowBox[{"-", "3"}]}], ",", 
   RowBox[{
    RowBox[{"Re", "[", "Epsilon", "]"}], " ", ">", " ", 
    RowBox[{"-", "2"}]}], ",", 
   RowBox[{
    RowBox[{"Re", "[", "Epsilon", "]"}], " ", ">", " ", 
    RowBox[{"-", "1"}]}], ",", 
   RowBox[{
    RowBox[{"Re", "[", "Epsilon", "]"}], " ", ">", " ", "0"}]}], 
  "}"}]], "Input",
 CellTags->"Epsilon",
 CellLabel->"In[543]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"True", ",", "True", ",", "True", ",", "True", ",", "True"}], 
   "}"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970219562889*^9},
 CellTags->"Epsilon",
 CellLabel->"Out[543]="]
}, Open  ]]
}, Open  ]],

Cell["EpsilonOrder", "ObjectName",
 CellTags->"EpsilonOrder"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"EpsilonOrder"],

Cell[TextData[{
 "EpsilonOrder is an option of OPEIntDelta and RHI. The setting determines \
the order ",
 "n",
 " (",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["Epsilon", 
    StyleBox["n",
     FontSlant->"Plain"]], TraditionalForm]]],
 ") which should be kept."
}], "Text",
 CellTags->"EpsilonOrder"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["OPEIntDelta",
  BaseStyle->"AddOnsLink",
  ButtonData:>"OPEIntDelta",
  ButtonNote->"OPEIntDelta"],
 ", ",
 ButtonBox["RHI",
  BaseStyle->"AddOnsLink",
  ButtonData:>"RHI",
  ButtonNote->"RHI"],
 "."
}], "Text",
 CellTags->"EpsilonOrder"]
}, Open  ]],

Cell["Expanding", "ObjectName",
 CellTags->"Expanding"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Expanding"],

Cell["\<\
Expanding is an option for Calc, Contract, DiracSimplify,DotSimplify, \
SUNSimplify, etc. As option for Contract it specifies whether expansion \
w.r.t. LorentzIndex is done BEFORE contraction. If set to False in \
DiracSimplify or SUNSimplify, only a limited set of simplifications \
(multiplicative linearity etc.) is performed.\
\>", "Text",
 CellTags->"Expanding"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Calc",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Calc",
  ButtonNote->"Calc"],
 ", ",
 ButtonBox["Contract",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Contract",
  ButtonNote->"Contract"],
 ", ",
 ButtonBox["DiracSimplify",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracSimplify",
  ButtonNote->"DiracSimplify"],
 ", ",
 ButtonBox["DotSimplify",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DotSimplify",
  ButtonNote->"DotSimplify"],
 ", ",
 ButtonBox["SUNSimplify",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SUNSimplify",
  ButtonNote->"SUNSimplify"],
 "."
}], "Text",
 CellTags->"Expanding"]
}, Open  ]],

Cell["ExpandPartialD", "ObjectName",
 CellTags->"ExpandPartialD"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"ExpandPartialD"],

Cell["\<\
ExpandPartialD[exp] expands all products of QuantumField's and partial \
differentiation operators in exp and applies the Leibniz rule.\
\>", "Text",
 CellTags->"ExpandPartialD"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "ExpandPartialD", "]"}]], "Input",
 CellLabel->"In[544]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"PartialDRelations", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"a$___", ".", 
       RowBox[{
       "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
        RowBox[{"x$_", ",", 
         FormBox["mu$_",
          TraditionalForm]}], ")"}], ".", "b$___"}], "\[RuleDelayed]", 
      RowBox[{"a$", ".", 
       RowBox[{
        SubscriptBox[
         TagBox[
          StyleBox["\<\"\[PartialD]\"\>",
           FontSlant->"Italic"],
          StyleForm[#, FontSlant -> "Italic"]& ], 
         TagBox[
          FormBox[
           FormBox["mu$",
            TraditionalForm],
           TraditionalForm],
          TraditionalForm,
          Editable->True]], "(", 
        TagBox[
         FormBox[
          RowBox[{"Dot", "[", "b$", "]"}],
          TraditionalForm],
         TraditionalForm,
         Editable->True], ")"}]}]}], "}"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970219639098*^9},
 CellLabel->"Out[544]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 ButtonBox["ExplicitPartialD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"ExplicitPartialD",
  ButtonNote->"ExplicitPartialD"],
 ", ",
 ButtonBox["LeftPartialD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"LeftPartialD",
  ButtonNote->"LeftPartialD"],
 ", ",
 ButtonBox["LeftRightPartialD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"LeftRightPartialD",
  ButtonNote->"LeftRightPartialD"],
 ", ",
 ButtonBox["PartialDRelations",
  BaseStyle->"AddOnsLink",
  ButtonData:>"PartialDRelations",
  ButtonNote->"PartialDRelations"],
 ", ",
 ButtonBox["RightPartialD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"RightPartialD",
  ButtonNote->"RightPartialD"],
 "."
}], "Text",
 CellTags->"ExpandPartialD"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"ExpandPartialD"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"RightPartialD", "[", "\[Mu]", "]"}], ".", 
  RowBox[{"QuantumField", "[", 
   RowBox[{"A", ",", 
    RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}]}], "]"}], ".", 
  RowBox[{"QuantumField", "[", 
   RowBox[{"A", ",", 
    RowBox[{"LorentzIndex", "[", "\[Nu]", "]"}]}], "]"}]}]], "Input",
 CellTags->"ExpandPartialD",
 CellLabel->"In[545]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox[
    OverscriptBox["\[PartialD]", "\[RightArrow]"], 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], ".", 
   SubscriptBox["A", "\[Mu]"], ".", 
   SubscriptBox["A", "\[Nu]"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970219726554*^9},
 CellTags->"ExpandPartialD",
 CellLabel->"Out[545]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ExpandPartialD", "[", "%", "]"}]], "Input",
 CellTags->"ExpandPartialD",
 CellLabel->"In[546]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox["A", "\[Mu]"], ".", 
    RowBox[{"QuantumField", "(", 
     RowBox[{
      RowBox[{
      "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
       FormBox["\[Mu]",
        TraditionalForm], ")"}], ",", "A", ",", 
      FormBox["\[Nu]",
       TraditionalForm]}], ")"}]}], "+", 
   RowBox[{
    RowBox[{"QuantumField", "(", 
     RowBox[{
      RowBox[{
      "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
       FormBox["\[Mu]",
        TraditionalForm], ")"}], ",", "A", ",", 
      FormBox["\[Mu]",
       TraditionalForm]}], ")"}], ".", 
    SubscriptBox["A", "\[Nu]"]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970219865541*^9},
 CellTags->"ExpandPartialD",
 CellLabel->"Out[546]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "StandardForm"}]], "Input",
 CellTags->"ExpandPartialD",
 CellLabel->"In[547]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"QuantumField", "[", 
    RowBox[{"A", ",", 
     RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}]}], "]"}], ".", 
   RowBox[{"QuantumField", "[", 
    RowBox[{
     RowBox[{
     "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "[", 
      RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], "]"}], ",", "A", ",", 
     RowBox[{"LorentzIndex", "[", "\[Nu]", "]"}]}], "]"}]}], "+", 
  RowBox[{
   RowBox[{"QuantumField", "[", 
    RowBox[{
     RowBox[{
     "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "[", 
      RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], "]"}], ",", "A", ",", 
     RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}]}], "]"}], ".", 
   RowBox[{"QuantumField", "[", 
    RowBox[{"A", ",", 
     RowBox[{"LorentzIndex", "[", "\[Nu]", "]"}]}], "]"}]}]}]], "Output",
 CellChangeTimes->{3.500970220123212*^9},
 CellTags->"ExpandPartialD",
 CellLabel->"Out[547]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"LeftRightPartialD", "[", "\[Mu]", "]"}], ".", 
  RowBox[{"QuantumField", "[", 
   RowBox[{"A", ",", 
    RowBox[{"LorentzIndex", "[", "\[Nu]", "]"}]}], "]"}]}]], "Input",
 CellTags->"ExpandPartialD",
 CellLabel->"In[548]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox[
    OverscriptBox["\[PartialD]", "\[LeftRightArrow]"], 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], ".", 
   SubscriptBox["A", "\[Nu]"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970220412313*^9},
 CellTags->"ExpandPartialD",
 CellLabel->"Out[548]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ExpandPartialD", "[", "%", "]"}]], "Input",
 CellTags->"ExpandPartialD",
 CellLabel->"In[549]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["1", "2"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"QuantumField", "(", 
      RowBox[{
       RowBox[{
       "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
        FormBox["\[Mu]",
         TraditionalForm], ")"}], ",", "A", ",", 
       FormBox["\[Nu]",
        TraditionalForm]}], ")"}], "-", 
     RowBox[{
      SubscriptBox[
       OverscriptBox["\[PartialD]", "\[LeftArrow]"], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], ".", 
      SubscriptBox["A", "\[Nu]"]}]}], ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970220634028*^9},
 CellTags->"ExpandPartialD",
 CellLabel->"Out[549]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"QuantumField", "[", 
   RowBox[{"A", ",", 
    RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}]}], "]"}], ".", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{"LeftRightPartialD", "[", "OPEDelta", "]"}], "^", "2"}], ")"}], 
  ".", 
  RowBox[{"QuantumField", "[", 
   RowBox[{"A", ",", 
    RowBox[{"LorentzIndex", "[", "\[Rho]", "]"}]}], "]"}]}]], "Input",
 CellTags->"ExpandPartialD",
 CellLabel->"In[550]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["A", "\[Mu]"], ".", 
   SubsuperscriptBox[
    OverscriptBox["\[PartialD]", "\[LeftRightArrow]"], 
    FormBox["\[CapitalDelta]",
     TraditionalForm], "2"], ".", 
   SubscriptBox["A", "\[Rho]"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009702207941227`*^9},
 CellTags->"ExpandPartialD",
 CellLabel->"Out[550]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ExpandPartialD", "[", "%", "]"}]], "Input",
 CellTags->"ExpandPartialD",
 CellLabel->"In[551]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["1", "4"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      SubscriptBox["A", "\[Mu]"], ".", 
      RowBox[{"QuantumField", "(", 
       RowBox[{
        RowBox[{
        "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(",
          "\[CapitalDelta]", ")"}], ",", 
        RowBox[{
        "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(",
          "\[CapitalDelta]", ")"}], ",", "A", ",", 
        FormBox["\[Rho]",
         TraditionalForm]}], ")"}]}], "-", 
     RowBox[{"2", " ", 
      RowBox[{
       RowBox[{"QuantumField", "(", 
        RowBox[{
         RowBox[{
         "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(",
           "\[CapitalDelta]", ")"}], ",", "A", ",", 
         FormBox["\[Mu]",
          TraditionalForm]}], ")"}], ".", 
       RowBox[{"QuantumField", "(", 
        RowBox[{
         RowBox[{
         "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(",
           "\[CapitalDelta]", ")"}], ",", "A", ",", 
         FormBox["\[Rho]",
          TraditionalForm]}], ")"}]}]}], "+", 
     RowBox[{
      RowBox[{"QuantumField", "(", 
       RowBox[{
        RowBox[{
        "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(",
          "\[CapitalDelta]", ")"}], ",", 
        RowBox[{
        "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(",
          "\[CapitalDelta]", ")"}], ",", "A", ",", 
        FormBox["\[Mu]",
         TraditionalForm]}], ")"}], ".", 
      SubscriptBox["A", "\[Rho]"]}]}], ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970221060945*^9},
 CellTags->"ExpandPartialD",
 CellLabel->"Out[551]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"8", " ", 
  RowBox[{
   RowBox[{"LeftRightPartialD", "[", "OPEDelta", "]"}], "^", "3"}]}]], "Input",\

 CellTags->"ExpandPartialD",
 CellLabel->"In[552]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"8", " ", 
   SubsuperscriptBox[
    OverscriptBox["\[PartialD]", "\[LeftRightArrow]"], 
    FormBox["\[CapitalDelta]",
     TraditionalForm], "3"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.50097022116042*^9},
 CellTags->"ExpandPartialD",
 CellLabel->"Out[552]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ExplicitPartialD", "[", "%", "]"}]], "Input",
 CellTags->"ExpandPartialD",
 CellLabel->"In[553]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox[
   RowBox[{"(", 
    RowBox[{
     SubscriptBox[
      OverscriptBox["\[PartialD]", "\[RightArrow]"], 
      FormBox["\[CapitalDelta]",
       TraditionalForm]], "-", 
     SubscriptBox[
      OverscriptBox["\[PartialD]", "\[LeftArrow]"], 
      FormBox["\[CapitalDelta]",
       TraditionalForm]]}], ")"}], "3"], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970221298671*^9},
 CellTags->"ExpandPartialD",
 CellLabel->"Out[553]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ExpandPartialD", "[", "%", "]"}]], "Input",
 CellTags->"ExpandPartialD",
 CellLabel->"In[554]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    RowBox[{
     SubscriptBox[
      OverscriptBox["\[PartialD]", "\[LeftArrow]"], 
      FormBox["\[CapitalDelta]",
       TraditionalForm]], ".", 
     SubscriptBox[
      OverscriptBox["\[PartialD]", "\[LeftArrow]"], 
      FormBox["\[CapitalDelta]",
       TraditionalForm]], ".", 
     SubscriptBox[
      OverscriptBox["\[PartialD]", "\[LeftArrow]"], 
      FormBox["\[CapitalDelta]",
       TraditionalForm]]}]}], "+", 
   RowBox[{"3", " ", 
    RowBox[{
     SubscriptBox[
      OverscriptBox["\[PartialD]", "\[LeftArrow]"], 
      FormBox["\[CapitalDelta]",
       TraditionalForm]], ".", 
     SubscriptBox[
      OverscriptBox["\[PartialD]", "\[LeftArrow]"], 
      FormBox["\[CapitalDelta]",
       TraditionalForm]], ".", 
     SubscriptBox[
      OverscriptBox["\[PartialD]", "\[RightArrow]"], 
      FormBox["\[CapitalDelta]",
       TraditionalForm]]}]}], "-", 
   RowBox[{"3", " ", 
    RowBox[{
     SubscriptBox[
      OverscriptBox["\[PartialD]", "\[LeftArrow]"], 
      FormBox["\[CapitalDelta]",
       TraditionalForm]], ".", 
     SubscriptBox[
      OverscriptBox["\[PartialD]", "\[RightArrow]"], 
      FormBox["\[CapitalDelta]",
       TraditionalForm]], ".", 
     SubscriptBox[
      OverscriptBox["\[PartialD]", "\[RightArrow]"], 
      FormBox["\[CapitalDelta]",
       TraditionalForm]]}]}], "+", 
   RowBox[{
    SubscriptBox[
     OverscriptBox["\[PartialD]", "\[RightArrow]"], 
     FormBox["\[CapitalDelta]",
      TraditionalForm]], ".", 
    SubscriptBox[
     OverscriptBox["\[PartialD]", "\[RightArrow]"], 
     FormBox["\[CapitalDelta]",
      TraditionalForm]], ".", 
    SubscriptBox[
     OverscriptBox["\[PartialD]", "\[RightArrow]"], 
     FormBox["\[CapitalDelta]",
      TraditionalForm]]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970221634675*^9},
 CellTags->"ExpandPartialD",
 CellLabel->"Out[554]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"LeviCivita", "[", 
   RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]", ",", "\[Tau]"}], "]"}], " ", 
  
  RowBox[{"RightPartialD", "[", 
   RowBox[{"\[Alpha]", ",", "\[Mu]", ",", "\[Beta]", ",", "\[Nu]"}], 
   "]"}]}]], "Input",
 CellTags->"ExpandPartialD",
 CellLabel->"In[555]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Epsilon]", 
    RowBox[{
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["\[Rho]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["\[Tau]",
       TraditionalForm],
      TraditionalForm]}]], " ", 
   RowBox[{
    SubscriptBox[
     OverscriptBox["\[PartialD]", "\[RightArrow]"], 
     FormBox[
      FormBox["\[Alpha]",
       TraditionalForm],
      TraditionalForm]], ".", 
    SubscriptBox[
     OverscriptBox["\[PartialD]", "\[RightArrow]"], 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], ".", 
    SubscriptBox[
     OverscriptBox["\[PartialD]", "\[RightArrow]"], 
     FormBox[
      FormBox["\[Beta]",
       TraditionalForm],
      TraditionalForm]], ".", 
    SubscriptBox[
     OverscriptBox["\[PartialD]", "\[RightArrow]"], 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm]]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970221826791*^9},
 CellTags->"ExpandPartialD",
 CellLabel->"Out[555]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ExpandPartialD", "[", "%", "]"}]], "Input",
 CellTags->"ExpandPartialD",
 CellLabel->"In[556]:="],

Cell[BoxData[
 FormBox["0", TraditionalForm]], "Output",
 CellChangeTimes->{3.500970221936844*^9},
 CellTags->"ExpandPartialD",
 CellLabel->"Out[556]="]
}, Open  ]]
}, Open  ]],

Cell["ExpandScalarProduct, ScalarProductExpand", "ObjectName",
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],

Cell["\<\
ExpandScalarProduct[expr] expands scalar products of sums of momenta in expr. \
ExpandScalarProduct does not use Expand on expr.
ScalarProductExpand=ExpandScalarProduct.\
\>", "Text",
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ExpandScalarProduct", "//", "Options"}]], "Input",
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"},
 CellLabel->"In[557]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"FeynCalcInternal", "\[Rule]", "True"}], "}"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970222050084*^9},
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"},
 CellLabel->"Out[557]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Calc",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Calc",
  ButtonNote->"Calc"],
 ", ",
 ButtonBox["MomentumExpand",
  BaseStyle->"AddOnsLink",
  ButtonData:>"MomentumExpand",
  ButtonNote->"MomentumExpand"],
 ", ",
 ButtonBox["MomentumCombine",
  BaseStyle->"AddOnsLink",
  ButtonData:>"MomentumCombine",
  ButtonNote->"MomentumCombine"],
 "."
}], "Text",
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{" ", 
  RowBox[{"SP", "[", 
   RowBox[{
    RowBox[{"p1", "+", "p2", "+", "p3"}], ",", 
    RowBox[{"p4", "+", "p5", "+", "p6"}]}], "]"}]}]], "Input",
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"},
 CellLabel->"In[558]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"(", 
   FormBox[
    RowBox[{"p1", "+", "p2", "+", "p3"}],
    TraditionalForm], ")", "\[CenterDot]", "(", 
   FormBox[
    RowBox[{"p4", "+", "p5", "+", "p6"}],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970222144238*^9},
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"},
 CellLabel->"Out[558]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "ScalarProductExpand"}]], "Input",
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"},
 CellLabel->"In[559]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"(", 
   FormBox[
    RowBox[{"p1", "+", "p2", "+", "p3"}],
    TraditionalForm], ")", "\[CenterDot]", "(", 
   FormBox[
    RowBox[{"p4", "+", "p5", "+", "p6"}],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009702222269573`*^9},
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"},
 CellLabel->"Out[559]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SP", "[", 
  RowBox[{"p", ",", 
   RowBox[{"p", "-", "q"}]}], "]"}]], "Input",
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"},
 CellLabel->"In[560]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox["p",
    TraditionalForm], "\[CenterDot]", "(", 
   FormBox[
    RowBox[{"p", "-", "q"}],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970222312406*^9},
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"},
 CellLabel->"Out[560]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ExpandScalarProduct", "[", "%", "]"}]], "Input",
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"},
 CellLabel->"In[561]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox["p",
    TraditionalForm], "\[CenterDot]", "(", 
   FormBox[
    RowBox[{"p", "-", "q"}],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970222398555*^9},
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"},
 CellLabel->"Out[561]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FV", "[", 
  RowBox[{
   RowBox[{"p", "-", "q"}], ",", "\[Mu]"}], "]"}]], "Input",
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"},
 CellLabel->"In[562]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox[
   FormBox[
    RowBox[{"p", "-", "q"}],
    TraditionalForm], "\[Mu]"], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009702224816093`*^9},
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"},
 CellLabel->"Out[562]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ExpandScalarProduct", "[", "%", "]"}]], "Input",
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"},
 CellLabel->"In[563]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox[
   FormBox[
    RowBox[{"p", "-", "q"}],
    TraditionalForm], "\[Mu]"], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970222564682*^9},
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"},
 CellLabel->"Out[563]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"SP", "[", 
   RowBox[{
    RowBox[{"p", "-", "q"}], ",", 
    RowBox[{"q", "-", "r"}]}], "]"}], "//", "FCI"}]], "Input",
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"},
 CellLabel->"In[564]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"(", 
   FormBox[
    RowBox[{"p", "-", "q"}],
    TraditionalForm], ")", "\[CenterDot]", "(", 
   FormBox[
    RowBox[{"q", "-", "r"}],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970222655943*^9},
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"},
 CellLabel->"Out[564]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "/.", 
  RowBox[{"Pair", "\[Rule]", "ExpandScalarProduct"}]}]], "Input",
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"},
 CellLabel->"In[565]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    FormBox["p",
     TraditionalForm], "\[CenterDot]", 
    FormBox["q",
     TraditionalForm]}], "-", 
   RowBox[{
    FormBox["p",
     TraditionalForm], "\[CenterDot]", 
    FormBox["r",
     TraditionalForm]}], "+", 
   RowBox[{
    FormBox["q",
     TraditionalForm], "\[CenterDot]", 
    FormBox["r",
     TraditionalForm]}], "-", 
   SuperscriptBox[
    FormBox["q",
     TraditionalForm], "2"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970222740246*^9},
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"},
 CellLabel->"Out[565]="]
}, Open  ]]
}, Open  ]],

Cell["Expand2", "ObjectName",
 CellTags->"Expand2"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Expand2"],

Cell["\<\
Expand2[exp, x] expands all sums containing x. Expand2[exp, {x1, x2, ...}]  \
expands all sums containing x1, x2, ....\
\>", "Text",
 CellTags->"Expand2"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Expand2"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Expand2", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{"x1", "+", "x2", "+", "x3"}], ")"}], 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"2", "x1"}], "+", 
       RowBox[{"3", "x2"}]}], ")"}]}], "+", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"y1", "+", "y2", "+", "y3"}], ")"}], 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"2", "y1"}], "+", 
       RowBox[{"3", "y2"}]}], ")"}]}]}], ",", 
   RowBox[{"{", 
    RowBox[{"x1", ",", "x2"}], "}"}]}], "]"}]], "Input",
 CellTags->"Expand2",
 CellLabel->"In[566]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"2", " ", 
    SuperscriptBox["x1", "2"]}], "+", 
   RowBox[{"5", " ", "x1", " ", "x2"}], "+", 
   RowBox[{"2", " ", "x1", " ", "x3"}], "+", 
   RowBox[{"3", " ", 
    SuperscriptBox["x2", "2"]}], "+", 
   RowBox[{"3", " ", "x2", " ", "x3"}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"2", " ", "y1"}], "+", 
      RowBox[{"3", " ", "y2"}]}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{"y1", "+", "y2", "+", "y3"}], ")"}]}]}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.5009702229439783`*^9},
 CellTags->"Expand2",
 CellLabel->"Out[566]="]
}, Open  ]]
}, Open  ]],

Cell["Explicit", "ObjectName",
 CellTags->"Explicit"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Explicit"],

Cell[TextData[{
 "Explicit is an option for FieldStrength, GluonVertex, SUNF, and \
Twist2GluonOperator. If set to True the full form of the operator is \
inserted. ",
 StyleBox["Explicit[exp]",
  FontVariations->{"CompatibilityType"->0}],
 " inserts explicit expressions of GluonVertex, Twist2GluonOperator, etc., in \
exp. SUNF's are replaced by SUNTrace objects."
}], "Text",
 CellTags->"Explicit"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["GluonVertex",
  BaseStyle->"AddOnsLink",
  ButtonData:>"GluonVertex",
  ButtonNote->"GluonVertex"],
 ", ",
 ButtonBox["Twist2GluonOperator",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Twist2GluonOperator",
  ButtonNote->"Twist2GluonOperator"],
 "."
}], "Text",
 CellTags->"Explicit"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Explicit", "//", "Options"}]], "Input",
 CellLabel->"In[567]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"CouplingConstant", "\[Rule]", 
     SubscriptBox["g", "s"]}], ",", 
    RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
    RowBox[{"Gauge", "\[Rule]", "1"}], ",", 
    RowBox[{"\[CapitalOmega]", "\[Rule]", "False"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.5009702230852537`*^9},
 CellLabel->"Out[567]="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Explicit"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GluonPropagator", "[", 
  RowBox[{"p", ",", "\[Mu]", ",", "\[Nu]"}], "]"}]], "Input",
 CellLabel->"In[568]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubsuperscriptBox["\[CapitalPi]", "g", 
    RowBox[{"\[Mu]", "\[Nu]"}]], "(", "p", ")"}], TraditionalForm]], "Output",\

 CellChangeTimes->{3.500970223157268*^9},
 CellLabel->"Out[568]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Explicit", "[", "%", "]"}]], "Input",
 CellLabel->"In[569]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   FractionBox[
    RowBox[{"\[ImaginaryI]", " ", 
     SuperscriptBox["g", 
      RowBox[{"\[Mu]", "\[Nu]"}]]}], 
    SuperscriptBox["p", "2"]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970223388063*^9},
 CellLabel->"Out[569]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Explicit", "[", 
  RowBox[{
   RowBox[{"GluonPropagator", "[", 
    RowBox[{"p", ",", "\[Mu]", ",", "\[Nu]"}], "]"}], ",", 
   RowBox[{"Gauge", "\[Rule]", "\[Xi]"}]}], "]"}]], "Input",
 CellLabel->"In[570]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{"\[ImaginaryI]", " ", 
    RowBox[{"(", 
     RowBox[{
      FractionBox[
       RowBox[{
        RowBox[{"(", 
         RowBox[{"1", "-", "\[Xi]"}], ")"}], " ", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], 
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm]], " ", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], 
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm]]}], 
       SuperscriptBox["p", "2"]], "-", 
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Nu]"}]]}], ")"}]}], 
   SuperscriptBox["p", "2"]], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009702235589437`*^9},
 CellLabel->"Out[570]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GluonVertex", "[", 
  RowBox[{
  "p", ",", "\[Mu]", ",", "a", ",", " ", "q", ",", "\[Nu]", ",", "b", ",", 
   "r", ",", "\[Rho]", ",", "c"}], "]"}]], "Input",
 CellTags->"Explicit",
 CellLabel->"In[571]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["f", 
    RowBox[{
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm]}]], " ", 
   RowBox[{
    SuperscriptBox["V", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Rho]",
        TraditionalForm],
       TraditionalForm]}]], "(", 
    RowBox[{
     FormBox["p",
      TraditionalForm], 
     RowBox[{",", " "}], 
     FormBox["q",
      TraditionalForm], 
     RowBox[{",", " "}], 
     FormBox["r",
      TraditionalForm]}], ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970223718136*^9},
 CellTags->"Explicit",
 CellLabel->"Out[571]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Explicit", "[", "%", "]"}]], "Input",
 CellTags->"Explicit",
 CellLabel->"In[572]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["g", "s"], " ", 
   SubscriptBox["f", 
    RowBox[{
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm]}]], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], 
         FormBox[
          FormBox["\[Rho]",
           TraditionalForm],
          TraditionalForm]], "-", 
        SuperscriptBox[
         FormBox["q",
          TraditionalForm], 
         FormBox[
          FormBox["\[Rho]",
           TraditionalForm],
          TraditionalForm]]}], ")"}]}], "+", 
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Rho]"}]], " ", 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox[
         FormBox["r",
          TraditionalForm], 
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm]], "-", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], 
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm]]}], ")"}]}], "+", 
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Nu]", "\[Rho]"}]], " ", 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox[
         FormBox["q",
          TraditionalForm], 
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm]], "-", 
        SuperscriptBox[
         FormBox["r",
          TraditionalForm], 
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm]]}], ")"}]}]}], ")"}]}], TraditionalForm]], "Output",\

 CellChangeTimes->{3.500970223863194*^9},
 CellTags->"Explicit",
 CellLabel->"Out[572]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Twist2GluonOperator", "[", 
  RowBox[{"p", ",", "\[Mu]", ",", "a", ",", "\[Nu]", ",", "b"}], 
  "]"}]], "Input",
 CellTags->"Explicit",
 CellLabel->"In[573]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["1", "2"], " ", 
   RowBox[{"(", 
    RowBox[{
     FormBox[
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"-", "1"}], ")"}], "m"],
      TraditionalForm], "+", "1"}], ")"}], " ", 
   SubscriptBox["\[Delta]", 
    RowBox[{
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm]}]], " ", 
   RowBox[{"(", 
    RowBox[{
     SubsuperscriptBox["O", 
      RowBox[{
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm], "\[VeryThinSpace]", 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]}], "G2"], "(", 
     FormBox["p",
      TraditionalForm], ")"}], ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009702241512814`*^9},
 CellTags->"Explicit",
 CellLabel->"Out[573]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Explicit", "[", "%", "]"}]], "Input",
 CellTags->"Explicit",
 CellLabel->"In[574]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["1", "2"], " ", 
   RowBox[{"(", 
    RowBox[{
     FormBox[
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"-", "1"}], ")"}], "m"],
      TraditionalForm], "+", "1"}], ")"}], " ", 
   SubscriptBox["\[Delta]", 
    RowBox[{
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm]}]], " ", 
   FormBox[
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{
       FormBox["\[CapitalDelta]",
        TraditionalForm], "\[CenterDot]", 
       FormBox["p",
        TraditionalForm]}], ")"}], 
     RowBox[{"m", "-", "2"}]],
    TraditionalForm], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
      SuperscriptBox[
       FormBox[
        RowBox[{
         FormBox["\[CapitalDelta]",
          TraditionalForm], "\[CenterDot]", 
         FormBox["p",
          TraditionalForm]}],
        TraditionalForm], "2"]}], "+", 
     RowBox[{
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], "2"], " ", 
      SuperscriptBox[
       FormBox["\[CapitalDelta]",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox[
       FormBox["\[CapitalDelta]",
        TraditionalForm], 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]]}], "-", 
     RowBox[{
      RowBox[{
       FormBox["\[CapitalDelta]",
        TraditionalForm], "\[CenterDot]", 
       FormBox["p",
        TraditionalForm]}], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         SuperscriptBox[
          FormBox["\[CapitalDelta]",
           TraditionalForm], 
          FormBox[
           FormBox["\[Nu]",
            TraditionalForm],
           TraditionalForm]], " ", 
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], 
          FormBox[
           FormBox["\[Mu]",
            TraditionalForm],
           TraditionalForm]]}], "+", 
        RowBox[{
         SuperscriptBox[
          FormBox["\[CapitalDelta]",
           TraditionalForm], 
          FormBox[
           FormBox["\[Mu]",
            TraditionalForm],
           TraditionalForm]], " ", 
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], 
          FormBox[
           FormBox["\[Nu]",
            TraditionalForm],
           TraditionalForm]]}]}], ")"}]}]}], ")"}]}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970224355205*^9},
 CellTags->"Explicit",
 CellLabel->"Out[574]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FieldStrength", "[", 
  RowBox[{"\[Mu]", ",", "\[Nu]", ",", "a"}], "]"}]], "Input",
 CellTags->"Explicit",
 CellLabel->"In[575]:="],

Cell[BoxData[
 FormBox[
  SubsuperscriptBox["F", 
   RowBox[{"\[Mu]", "\[Nu]"}], "a"], TraditionalForm]], "Output",
 CellChangeTimes->{3.50097022444002*^9},
 CellTags->"Explicit",
 CellLabel->"Out[575]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Explicit", "[", "%", "]"}]], "Input",
 CellTags->"Explicit",
 CellLabel->"In[576]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"QuantumField", "(", 
    RowBox[{
     RowBox[{
     "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
      FormBox["\[Mu]",
       TraditionalForm], ")"}], ",", "A", ",", 
     FormBox["\[Nu]",
      TraditionalForm], ",", 
     FormBox["a",
      TraditionalForm]}], ")"}], "-", 
   RowBox[{"QuantumField", "(", 
    RowBox[{
     RowBox[{
     "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
      FormBox["\[Nu]",
       TraditionalForm], ")"}], ",", "A", ",", 
     FormBox["\[Mu]",
      TraditionalForm], ",", 
     FormBox["a",
      TraditionalForm]}], ")"}], "+", 
   RowBox[{
    SubscriptBox["g", "s"], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b26",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c27",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    RowBox[{
     SubsuperscriptBox[
      FormBox["A",
       TraditionalForm], 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b26",
        TraditionalForm],
       TraditionalForm]], ".", 
     SubsuperscriptBox[
      FormBox["A",
       TraditionalForm], 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c27",
        TraditionalForm],
       TraditionalForm]]}]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970224548086*^9},
 CellTags->"Explicit",
 CellLabel->"Out[576]="]
}, Open  ]]
}, Open  ]],

Cell["ExplicitLorentzIndex", "ObjectName",
 CellTags->"ExplicitLorentzIndex"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"ExplicitLorentzIndex"],

Cell["\<\
ExplicitLorentzIndex[ind] is an explicit Lorentz index, i.e., ind is an \
integer.\
\>", "Text",
 CellTags->"ExplicitLorentzIndex"],

Cell[TextData[{
 "See also: ",
 ButtonBox["LorentzIndex",
  BaseStyle->"AddOnsLink",
  ButtonData:>"LorentzIndex",
  ButtonNote->"LorentzIndex"],
 ", ",
 ButtonBox["Pair",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Pair",
  ButtonNote->"Pair"],
 "."
}], "Text",
 CellTags->"ExplicitLorentzIndex"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"ExplicitLorentzIndex"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Pair", "[", 
  RowBox[{
   RowBox[{"LorentzIndex", "[", "1", "]"}], ",", 
   RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}]}], "]"}]], "Input",
 CellTags->"ExplicitLorentzIndex",
 CellLabel->"In[577]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["g", 
   RowBox[{
    FormBox["1",
     TraditionalForm], "\[Mu]"}]], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970224622487*^9},
 CellTags->"ExplicitLorentzIndex",
 CellLabel->"Out[577]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "StandardForm"}]], "Input",
 CellTags->"ExplicitLorentzIndex",
 CellLabel->"In[578]:="],

Cell[BoxData[
 RowBox[{"Pair", "[", 
  RowBox[{
   RowBox[{"ExplicitLorentzIndex", "[", "1", "]"}], ",", 
   RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}]}], "]"}]], "Output",
 CellChangeTimes->{3.500970224666483*^9},
 CellTags->"ExplicitLorentzIndex",
 CellLabel->"Out[578]//StandardForm="]
}, Open  ]]
}, Open  ]],

Cell["ExplicitPartialD", "ObjectName",
 CellTags->"ExplicitPartialD"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"ExplicitPartialD"],

Cell["\<\
ExplicitPartialD[exp] inserts in exp the definition for LeftRightPartialD[z] \
(and LeftRightPartialD2[z]).\
\>", "Text",
 CellTags->"ExplicitPartialD"],

Cell[TextData[{
 "See also: ",
 ButtonBox["ExpandPartialD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"ExpandPartialD",
  ButtonNote->"ExpandPartialD"],
 ", ",
 ButtonBox["LeftRightPartialD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"LeftRightPartialD",
  ButtonNote->"LeftRightPartialD"],
 ", ",
 ButtonBox["LeftRightPartialD2",
  BaseStyle->"AddOnsLink",
  ButtonData:>"LeftRightPartialD2",
  ButtonNote->"LeftRightPartialD2"],
 "."
}], "Text",
 CellTags->"ExplicitPartialD"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"ExplicitPartialD"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ExplicitPartialD", "[", " ", 
  RowBox[{"LeftRightPartialD", "[", "\[Mu]", "]"}], " ", "]"}]], "Input",
 CellTags->"ExplicitPartialD",
 CellLabel->"In[579]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["1", "2"], " ", 
   RowBox[{"(", 
    RowBox[{
     SubscriptBox[
      OverscriptBox["\[PartialD]", "\[RightArrow]"], 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], "-", 
     SubscriptBox[
      OverscriptBox["\[PartialD]", "\[LeftArrow]"], 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]]}], ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970224737801*^9},
 CellTags->"ExplicitPartialD",
 CellLabel->"Out[579]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ExplicitPartialD", "[", " ", 
  RowBox[{"LeftRightPartialD2", "[", "\[Mu]", "]"}], " ", "]"}]], "Input",
 CellTags->"ExplicitPartialD",
 CellLabel->"In[580]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox[
    OverscriptBox["\[PartialD]", "\[LeftArrow]"], 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], "+", 
   SubscriptBox[
    OverscriptBox["\[PartialD]", "\[RightArrow]"], 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009702248090754`*^9},
 CellTags->"ExplicitPartialD",
 CellLabel->"Out[580]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ExplicitPartialD", "[", " ", 
  RowBox[{"LeftRightPartialD", "[", "OPEDelta", "]"}], " ", "]"}]], "Input",
 CellTags->"ExplicitPartialD",
 CellLabel->"In[581]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["1", "2"], " ", 
   RowBox[{"(", 
    RowBox[{
     SubscriptBox[
      OverscriptBox["\[PartialD]", "\[RightArrow]"], 
      FormBox["\[CapitalDelta]",
       TraditionalForm]], "-", 
     SubscriptBox[
      OverscriptBox["\[PartialD]", "\[LeftArrow]"], 
      FormBox["\[CapitalDelta]",
       TraditionalForm]]}], ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.50097022488937*^9},
 CellTags->"ExplicitPartialD",
 CellLabel->"Out[581]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"16", " ", 
  RowBox[{
   RowBox[{"LeftRightPartialD", "[", "OPEDelta", "]"}], "^", "4"}]}]], "Input",\

 CellTags->"ExplicitPartialD",
 CellLabel->"In[582]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"16", " ", 
   SubsuperscriptBox[
    OverscriptBox["\[PartialD]", "\[LeftRightArrow]"], 
    FormBox["\[CapitalDelta]",
     TraditionalForm], "4"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970224960681*^9},
 CellTags->"ExplicitPartialD",
 CellLabel->"Out[582]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ExplicitPartialD", "[", "%", "]"}]], "Input",
 CellTags->"ExplicitPartialD",
 CellLabel->"In[583]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox[
   RowBox[{"(", 
    RowBox[{
     SubscriptBox[
      OverscriptBox["\[PartialD]", "\[RightArrow]"], 
      FormBox["\[CapitalDelta]",
       TraditionalForm]], "-", 
     SubscriptBox[
      OverscriptBox["\[PartialD]", "\[LeftArrow]"], 
      FormBox["\[CapitalDelta]",
       TraditionalForm]]}], ")"}], "4"], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970225094881*^9},
 CellTags->"ExplicitPartialD",
 CellLabel->"Out[583]="]
}, Open  ]]
}, Open  ]],

Cell["ExplicitSUNIndex", "ObjectName",
 CellTags->"ExplicitSUNIndex"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"ExplicitSUNIndex"],

Cell[TextData[{
 "ExplicitSUNIndex[ind] is a specific SU(",
 StyleBox["N",
  FontSlant->"Italic"],
 ") index, i.e., ind is an integer."
}], "Text",
 CellTags->"ExplicitSUNIndex"],

Cell[TextData[{
 "See also: ",
 ButtonBox["FCI",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FCI",
  ButtonNote->"FCI"],
 ", ",
 ButtonBox["SUNDelta",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SUNDelta",
  ButtonNote->"SUNDelta"],
 ", ",
 ButtonBox["SUNF",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SUNF",
  ButtonNote->"SUNF"],
 ", ",
 ButtonBox["SUNIndex",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SUNIndex",
  ButtonNote->"SUNIndex"],
 "."
}], "Text",
 CellTags->"ExplicitSUNIndex"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"ExplicitSUNIndex"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ExplicitSUNIndex", "[", "1", "]"}]], "Input",
 CellTags->"ExplicitSUNIndex",
 CellLabel->"In[584]:="],

Cell[BoxData[
 FormBox["1", TraditionalForm]], "Output",
 CellChangeTimes->{3.500970225230311*^9},
 CellTags->"ExplicitSUNIndex",
 CellLabel->"Out[584]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"SUNDelta", "[", 
    RowBox[{"1", ",", "a"}], "]"}], "//", "FCI"}], "//", 
  "StandardForm"}]], "Input",
 CellLabel->"In[585]:="],

Cell[BoxData[
 RowBox[{"SUNDelta", "[", 
  RowBox[{
   RowBox[{"ExplicitSUNIndex", "[", "1", "]"}], ",", 
   RowBox[{"SUNIndex", "[", "a", "]"}]}], "]"}]], "Output",
 CellChangeTimes->{3.500970225313472*^9},
 CellLabel->"Out[585]//StandardForm="]
}, Open  ]]
}, Open  ]],

Cell["ExtraFactor", "ObjectName",
 CellTags->"ExtraFactor"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"ExtraFactor"],

Cell["\<\
ExtraFactor is an option for FermionSpinSum. The setting ExtraFactor \[Rule] \
fa  multiplies the whole amplitude with the factor fa before squaring.\
\>", "Text",
 CellTags->"ExtraFactor"],

Cell[TextData[{
 "See also: ",
 ButtonBox["FermionSpinSum",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FermionSpinSum",
  ButtonNote->"FermionSpinSum"],
 "."
}], "Text",
 CellTags->"ExtraFactor"]
}, Open  ]],

Cell["ExtraVariables", "ObjectName",
 CellTags->"ExtraVariables"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"ExtraVariables"],

Cell["\<\
ExtraVariables is an option for OneLoopSum; it may be set to a list of \
variables which are also bracketed out in the result, just like B0, C0, D0 \
and  PaVe.\
\>", "Text",
 CellTags->"ExtraVariables"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["OneLoop",
  BaseStyle->"AddOnsLink",
  ButtonData:>"OneLoop",
  ButtonNote->"OneLoop"],
 ", ",
 ButtonBox["OneLoopSum",
  BaseStyle->"AddOnsLink",
  ButtonData:>"OneLoopSum",
  ButtonNote->"OneLoopSum"],
 "."
}], "Text",
 CellTags->"ExtraVariables"]
}, Open  ]],

Cell["FactorFull", "ObjectName",
 CellTags->"FactorFull"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FactorFull"],

Cell[TextData[{
 "FactorFull is an option of Factor2 (default False). If set to False, \
products like (",
 StyleBox["a",
  FontSlant->"Italic"],
 "-",
 StyleBox["b",
  FontSlant->"Italic"],
 ") (",
 StyleBox["a",
  FontSlant->"Italic"],
 "+",
 StyleBox["b",
  FontSlant->"Italic"],
 ") will be replaced by (",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["a", "2"], TraditionalForm]]],
 "-",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["b", "2"], TraditionalForm]]],
 ")."
}], "Text",
 CellTags->"FactorFull"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Factor2",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Factor2",
  ButtonNote->"Factor2"],
 "."
}], "Text",
 CellTags->"FactorFull"]
}, Open  ]],

Cell["Factoring", "ObjectName",
 CellTags->"Factoring"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Factoring"],

Cell["\<\
Factoring is an option for Collect2, Contract, Tr and more functions. If set \
to True, the result will be factored, using Factor2. If set to any function \
f, this function will be used.\
\>", "Text",
 CellTags->"Factoring"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Collect2",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Collect2",
  ButtonNote->"Collect2"],
 ", ",
 ButtonBox["Contract",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Contract",
  ButtonNote->"Contract"],
 ", ",
 ButtonBox["Tr",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Tr",
  ButtonNote->"Tr"],
 "."
}], "Text",
 CellTags->"Factoring"]
}, Open  ]],

Cell["Factorout", "ObjectName",
 CellTags->"Factorout"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Factorout"],

Cell["Factorout is an option for OPEInt.", "Text",
 CellTags->"Factorout"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["OPEInt",
  BaseStyle->"AddOnsLink",
  ButtonData:>"OPEInt",
  ButtonNote->"OPEInt"],
 "."
}], "Text",
 CellTags->"Factorout"]
}, Open  ]],

Cell["FactorTime", "ObjectName",
 CellTags->"FactorTime"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FactorTime"],

Cell["\<\
FactorTime is an option for Factor2. It denotes the maximum time (in seconds) \
during which Factor2 tries to factor.\
\>", "Text",
 CellTags->"FactorTime"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Factor2",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Factor2",
  ButtonNote->"Factor2"],
 "."
}], "Text",
 CellTags->"FactorTime"]
}, Open  ]],

Cell["Factor1", "ObjectName",
 CellTags->"Factor1"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Factor1"],

Cell["\<\
Factor1[poly] factorizes common terms  in the summands of poly. It uses \
basically PolynomialGCD.\
\>", "Text",
 CellTags->"Factor1"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Factor2",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Factor2",
  ButtonNote->"Factor2"],
 "."
}], "Text",
 CellTags->"Factor1"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Factor1"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t1", "=", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{"a", "-", "x"}], ")"}], 
   RowBox[{"(", 
    RowBox[{"b", "-", "x"}], ")"}]}]}]], "Input",
 CellTags->"Factor1",
 CellLabel->"In[586]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"(", 
    RowBox[{"a", "-", "x"}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{"b", "-", "x"}], ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009702254274807`*^9},
 CellTags->"Factor1",
 CellLabel->"Out[586]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t2", "=", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Factor1", "[", "t1", "]"}], ",", " ", 
    RowBox[{"Factor", "[", "t1", "]"}]}], "}"}]}]], "Input",
 CellTags->"Factor1",
 CellLabel->"In[587]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{"a", "-", "x"}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{"b", "-", "x"}], ")"}]}], ",", 
    RowBox[{
     RowBox[{"-", 
      RowBox[{"(", 
       RowBox[{"a", "-", "x"}], ")"}]}], " ", 
     RowBox[{"(", 
      RowBox[{"x", "-", "b"}], ")"}]}]}], "}"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970225499579*^9},
 CellTags->"Factor1",
 CellLabel->"Out[587]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t3", "=", 
  RowBox[{"Expand", "[", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"a", "-", "b"}], ")"}], 
    RowBox[{"(", 
     RowBox[{"a", "+", "b"}], ")"}]}], "]"}]}]], "Input",
 CellTags->"Factor1",
 CellLabel->"In[588]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["a", "2"], "-", 
   SuperscriptBox["b", "2"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970225576528*^9},
 CellTags->"Factor1",
 CellLabel->"Out[588]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Factor", "[", "t3", "]"}]], "Input",
 CellTags->"Factor1",
 CellLabel->"In[589]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"(", 
    RowBox[{"a", "-", "b"}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{"a", "+", "b"}], ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970225656722*^9},
 CellTags->"Factor1",
 CellLabel->"Out[589]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Factor1", "[", "t3", "]"}]], "Input",
 CellTags->"Factor1",
 CellLabel->"In[590]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["a", "2"], "-", 
   SuperscriptBox["b", "2"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970225737505*^9},
 CellTags->"Factor1",
 CellLabel->"Out[590]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"Clear", "[", 
  RowBox[{"t1", ",", "t2", ",", "t3"}], "]"}]], "Input",
 CellTags->"Factor1",
 CellLabel->"In[591]:="]
}, Open  ]],

Cell["Factor2", "ObjectName",
 CellTags->"Factor2"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Factor2"],

Cell["\<\
Factor2[poly] factors a polynomial in a standard way. Factor2 works sometimes \
better than Factor on polynomials involving rationals with sums in the \
denominator. Factor2 uses Factor internally and is in general slower than \
Factor. There are four possible settings of the option Method (0,1,2,3). In \
general Factor will work faster than Factor2.\
\>", "Text",
 CellTags->"Factor2"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "Factor2", "]"}]], "Input",
 CellTags->"Factor2",
 CellLabel->"In[592]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"FactorFull", "\[Rule]", "False"}], ",", 
    RowBox[{"Method", "\[Rule]", "3"}]}], "}"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009702258199*^9},
 CellTags->"Factor2",
 CellLabel->"Out[592]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Collect2",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Collect2",
  ButtonNote->"Collect2"],
 "."
}], "Text",
 CellTags->"Factor2"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Factor2"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t1", "=", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{"a", "-", "x"}], ")"}], 
   RowBox[{"(", 
    RowBox[{"b", "-", "x"}], ")"}]}]}]], "Input",
 CellTags->"Factor2",
 CellLabel->"In[593]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"(", 
    RowBox[{"a", "-", "x"}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{"b", "-", "x"}], ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009702258992777`*^9},
 CellTags->"Factor2",
 CellLabel->"Out[593]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t2", "=", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Factor2", "[", "t1", "]"}], ",", " ", 
    RowBox[{"Factor", "[", "t1", "]"}]}], "}"}]}]], "Input",
 CellTags->"Factor2",
 CellLabel->"In[594]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{"a", "-", "x"}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{"b", "-", "x"}], ")"}]}], ",", 
    RowBox[{
     RowBox[{"-", 
      RowBox[{"(", 
       RowBox[{"a", "-", "x"}], ")"}]}], " ", 
     RowBox[{"(", 
      RowBox[{"x", "-", "b"}], ")"}]}]}], "}"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970226006365*^9},
 CellTags->"Factor2",
 CellLabel->"Out[594]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t3", "=", 
  RowBox[{"Expand", "[", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"a", "-", "b"}], ")"}], 
    RowBox[{"(", 
     RowBox[{"a", "+", "b"}], ")"}]}], "]"}]}]], "Input",
 CellTags->"Factor2",
 CellLabel->"In[595]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["a", "2"], "-", 
   SuperscriptBox["b", "2"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970226271275*^9},
 CellTags->"Factor2",
 CellLabel->"Out[595]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Factor", "[", "t3", "]"}]], "Input",
 CellTags->"Factor2",
 CellLabel->"In[596]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"(", 
    RowBox[{"a", "-", "b"}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{"a", "+", "b"}], ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970226416388*^9},
 CellTags->"Factor2",
 CellLabel->"Out[596]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Factor2", "[", "t3", "]"}]], "Input",
 CellTags->"Factor2",
 CellLabel->"In[597]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["a", "2"], "-", 
   SuperscriptBox["b", "2"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009702265737133`*^9},
 CellTags->"Factor2",
 CellLabel->"Out[597]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Factor2", "[", 
  RowBox[{"t3", ",", 
   RowBox[{"FactorFull", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"Factor2",
 CellLabel->"In[598]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"(", 
    RowBox[{"a", "-", "b"}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{"a", "+", "b"}], ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970226731607*^9},
 CellTags->"Factor2",
 CellLabel->"Out[598]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"Clear", "[", 
  RowBox[{"t1", ",", "t2", ",", "t3"}], "]"}]], "Input",
 CellTags->"Factor2",
 CellLabel->"In[599]:="]
}, Open  ]],

Cell["FAD", "ObjectName",
 CellTags->"FAD"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FAD"],

Cell["\<\
FAD is the FeynCalc external form of FeynAmpDenominator and denotes an \
inverse propagator. FAD[q, q-p, ...] is 1/(q^2 (q-p)^2 ...). FAD[{q1,m}, \
{q1-p,m}, q2, ...] is 1/( (q1^2 - m^2) ( (q1-p)^2 - m^2 ) q2^2 ... ). \
Translation into FeynCalc internal form is performed by FeynCalcInternal.\
\>", "Text",
 CellTags->"FAD"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FAD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FAD",
  ButtonNote->"FAD"],
 ", ",
 ButtonBox["FCE",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FCE",
  ButtonNote->"FCE"],
 ", ",
 ButtonBox["FCI",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FCI",
  ButtonNote->"FCI"],
 ", ",
 ButtonBox["FeynAmpDenominator",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FeynAmpDenominator",
  ButtonNote->"FeynAmpDenominator"],
 ", ",
 ButtonBox["FeynAmpDenominatorSimplify",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FeynAmpDenominatorSimplify",
  ButtonNote->"FeynAmpDenominatorSimplify"],
 ", ",
 ButtonBox["PropagatorDenominator",
  BaseStyle->"AddOnsLink",
  ButtonData:>"PropagatorDenominator",
  ButtonNote->"PropagatorDenominator"],
 "."
}], "Text",
 CellTags->"FAD"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"FAD"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FAD", "[", 
  RowBox[{"q", ",", 
   RowBox[{"p", "-", "q"}]}], "]"}]], "Input",
 CellTags->"FAD",
 CellLabel->"In[600]:="],

Cell[BoxData[
 FormBox[
  FractionBox["1", 
   RowBox[{
    RowBox[{"(", 
     InterpretationBox[
      RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
       SuperscriptBox["q", "2"], "\[InvisibleSpace]", "\<\"]\"\>"}],
      SequenceForm["[", $CellContext`q^2, "]"],
      Editable->False], ")"}], " ", 
    RowBox[{"(", 
     InterpretationBox[
      RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"p", "-", "q"}], ")"}], "2"], 
       "\[InvisibleSpace]", "\<\"]\"\>"}],
      SequenceForm["[", ($CellContext`p - $CellContext`q)^2, "]"],
      Editable->False], ")"}]}]], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970226914711*^9},
 CellTags->"FAD",
 CellLabel->"Out[600]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FAD", "[", 
  RowBox[{"p", ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"p", "-", "q"}], ",", "m"}], "}"}]}], "]"}]], "Input",
 CellTags->"FAD",
 CellLabel->"In[601]:="],

Cell[BoxData[
 FormBox[
  FractionBox["1", 
   RowBox[{
    RowBox[{"(", 
     InterpretationBox[
      RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
       SuperscriptBox["p", "2"], "\[InvisibleSpace]", "\<\"]\"\>"}],
      SequenceForm["[", $CellContext`p^2, "]"],
      Editable->False], ")"}], " ", 
    RowBox[{"(", 
     InterpretationBox[
      RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"p", "-", "q"}], ")"}], "2"], 
       "\[InvisibleSpace]", "\<\"-\"\>", "\[InvisibleSpace]", 
       SuperscriptBox["m", "2"], "\[InvisibleSpace]", "\<\"]\"\>"}],
      SequenceForm[
      "[", ($CellContext`p - $CellContext`q)^2, "-", $CellContext`m^2, "]"],
      Editable->False], ")"}]}]], TraditionalForm]], "Output",
 CellChangeTimes->{3.50097022704869*^9},
 CellTags->"FAD",
 CellLabel->"Out[601]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"FAD", "[", 
     RowBox[{"q", ",", 
      RowBox[{"p", "-", "q"}]}], "]"}], "//", "FCI"}], "//", "FCE"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"FAD",
 CellLabel->"In[602]:="],

Cell[BoxData[
 RowBox[{"FAD", "[", 
  RowBox[{
   RowBox[{"p", "-", "q"}], ",", "q"}], "]"}]], "Output",
 CellChangeTimes->{3.5009702272149*^9},
 CellTags->"FAD",
 CellLabel->"Out[602]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"FAD", "[", 
    RowBox[{"q", ",", 
     RowBox[{"p", "-", "q"}]}], "]"}], "//", "FCI"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"FAD",
 CellLabel->"In[603]:="],

Cell[BoxData[
 RowBox[{"FeynAmpDenominator", "[", 
  RowBox[{
   RowBox[{"PropagatorDenominator", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"Momentum", "[", 
       RowBox[{"p", ",", "D"}], "]"}], "-", 
      RowBox[{"Momentum", "[", 
       RowBox[{"q", ",", "D"}], "]"}]}], ",", "0"}], "]"}], ",", 
   RowBox[{"PropagatorDenominator", "[", 
    RowBox[{
     RowBox[{"Momentum", "[", 
      RowBox[{"q", ",", "D"}], "]"}], ",", "0"}], "]"}]}], "]"}]], "Output",
 CellChangeTimes->{3.500970227358172*^9},
 CellTags->"FAD",
 CellLabel->"Out[603]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"FAD", "[", "p", "]"}], " ", 
    RowBox[{"FAD", "[", 
     RowBox[{"p", "-", "q"}], "]"}]}], " ", "//", " ", 
   "FeynAmpDenominatorCombine"}], "//", "StandardForm"}]], "Input",
 CellTags->"FAD",
 CellLabel->"In[604]:="],

Cell[BoxData[
 RowBox[{"FeynAmpDenominator", "[", 
  RowBox[{
   RowBox[{"PropagatorDenominator", "[", 
    RowBox[{
     RowBox[{"Momentum", "[", 
      RowBox[{"p", ",", "D"}], "]"}], ",", "0"}], "]"}], ",", 
   RowBox[{"PropagatorDenominator", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"Momentum", "[", 
       RowBox[{"p", ",", "D"}], "]"}], "-", 
      RowBox[{"Momentum", "[", 
       RowBox[{"q", ",", "D"}], "]"}]}], ",", "0"}], "]"}]}], "]"}]], "Output",\

 CellChangeTimes->{3.5009702274363813`*^9},
 CellTags->"FAD",
 CellLabel->"Out[604]//StandardForm="]
}, Open  ]]
}, Open  ]],

Cell["FC", "ObjectName",
 CellTags->"FC"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FC"],

Cell["\<\
FC changes the output format to FeynCalcForm. To change to InputForm use FI.\
\>", "Text",
 CellTags->"FC"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FeynCalcForm",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FeynCalcForm",
  ButtonNote->"FeynCalcForm"],
 ", ",
 ButtonBox["FI",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FI",
  ButtonNote->"FI"],
 ", ",
 ButtonBox["FeynCalcExternal",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FeynCalcExternal",
  ButtonNote->"FeynCalcExternal"],
 ", ",
 ButtonBox["FeynCalcInternal",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FeynCalcInternal",
  ButtonNote->"FeynCalcInternal"],
 "."
}], "Text",
 CellTags->"FC"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"FC"],

Cell[BoxData["FI"], "Input",
 CellTags->"FC",
 CellLabel->"In[605]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"DiracGamma", "[", "5", "]"}], ",", 
   RowBox[{"DiracGamma", "[", 
    RowBox[{"Momentum", "[", "p", "]"}], "]"}]}], "}"}]], "Input",
 CellTags->"FC",
 CellLabel->"In[606]:="],

Cell["{DiracGamma[5], DiracGamma[Momentum[p]]}", "Output",
 CellChangeTimes->{3.50097022755898*^9},
 CellTags->"FC",
 CellLabel->"Out[606]="]
}, Open  ]],

Cell[BoxData["FC"], "Input",
 CellTags->"FC",
 CellLabel->"In[607]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"DiracGamma", "[", "5", "]"}], ",", 
   RowBox[{"DiracGamma", "[", 
    RowBox[{"Momentum", "[", "p", "]"}], "]"}]}], "}"}]], "Input",
 CellTags->"FC",
 CellLabel->"In[608]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    SuperscriptBox["\[Gamma]", "5"], ",", 
    RowBox[{
     FormBox["\<\"\[Gamma]\"\>",
      TraditionalForm], "\[CenterDot]", 
     FormBox["p",
      TraditionalForm]}]}], "}"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970227660544*^9},
 CellTags->"FC",
 CellLabel->"Out[608]="]
}, Open  ]]
}, Open  ]],

Cell["FCE", "ObjectName",
 CellTags->"FCE"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FCE"],

Cell["\<\
FCE[exp] translates exp from the internal FeynCalc representation to a short \
form.\
\>", "Text",
 CellTags->"FCE"],

Cell["FCE is equivalent to FeynCalcExternal.", "Text",
 CellTags->"FCE"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FeynCalcExternal",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FeynCalcExternal",
  ButtonNote->"FeynCalcExternal"],
 ", ",
 ButtonBox["FCI",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FCI",
  ButtonNote->"FCI"],
 ", ",
 ButtonBox["FeynCalcInternal",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FeynCalcInternal",
  ButtonNote->"FeynCalcInternal"],
 "."
}], "Text",
 CellTags->"FCE"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"FCE"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FCE", "[", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"DiracGamma", "[", "5", "]"}], ",", 
    RowBox[{"DiracGamma", "[", 
     RowBox[{"Momentum", "[", "p", "]"}], "]"}]}], "}"}], "]"}]], "Input",
 CellTags->"FCE",
 CellLabel->"In[609]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    SuperscriptBox["\[Gamma]", "5"], ",", 
    RowBox[{"\[Gamma]", "\[CenterDot]", "p"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970227745013*^9},
 CellTags->"FCE",
 CellLabel->"Out[609]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "StandardForm"}]], "Input",
 CellTags->"FCE",
 CellLabel->"In[610]:="],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"GA", "[", "5", "]"}], ",", 
   RowBox[{"GS", "[", "p", "]"}]}], "}"}]], "Output",
 CellChangeTimes->{3.500970227823679*^9},
 CellTags->"FCE",
 CellLabel->"Out[610]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"GA", "[", "\[Mu]", "]"}], ",", 
   RowBox[{"GAD", "[", "\[Rho]", "]"}], ",", 
   RowBox[{"GS", "[", "p", "]"}], ",", 
   RowBox[{"SP", "[", 
    RowBox[{"p", ",", "q"}], "]"}], ",", 
   RowBox[{"MT", "[", 
    RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}], ",", 
   RowBox[{"FV", "[", 
    RowBox[{"p", ",", "\[Mu]"}], "]"}]}], "}"}]], "Input",
 CellTags->"FCE",
 CellLabel->"In[611]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    SuperscriptBox["\[Gamma]", "\[Mu]"], ",", 
    SuperscriptBox["\[Gamma]", "\[Rho]"], ",", 
    RowBox[{"\[Gamma]", "\[CenterDot]", "p"}], ",", 
    FormBox[
     RowBox[{
      FormBox["p",
       TraditionalForm], "\[CenterDot]", 
      FormBox["q",
       TraditionalForm]}],
     TraditionalForm], ",", 
    SuperscriptBox["g", 
     RowBox[{"\[Alpha]", "\[Beta]"}]], ",", 
    SuperscriptBox["p", "\[Mu]"]}], "}"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970227910688*^9},
 CellTags->"FCE",
 CellLabel->"Out[611]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "StandardForm"}]], "Input",
 CellTags->"FCE",
 CellLabel->"In[612]:="],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"GA", "[", "\[Mu]", "]"}], ",", 
   RowBox[{"GAD", "[", "\[Rho]", "]"}], ",", 
   RowBox[{"GS", "[", "p", "]"}], ",", 
   RowBox[{"SP", "[", 
    RowBox[{"p", ",", "q"}], "]"}], ",", 
   RowBox[{"MT", "[", 
    RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}], ",", 
   RowBox[{"FV", "[", 
    RowBox[{"p", ",", "\[Mu]"}], "]"}]}], "}"}]], "Output",
 CellChangeTimes->{3.500970227990769*^9},
 CellTags->"FCE",
 CellLabel->"Out[612]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "FCI"}]], "Input",
 CellTags->"FCE",
 CellLabel->"In[613]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], ",", 
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox["\[Rho]",
       TraditionalForm],
      TraditionalForm]], ",", 
    RowBox[{
     FormBox["\<\"\[Gamma]\"\>",
      TraditionalForm], "\[CenterDot]", 
     FormBox["p",
      TraditionalForm]}], ",", 
    RowBox[{
     FormBox["p",
      TraditionalForm], "\[CenterDot]", 
     FormBox["q",
      TraditionalForm]}], ",", 
    SuperscriptBox["g", 
     RowBox[{"\[Alpha]", "\[Beta]"}]], ",", 
    SuperscriptBox[
     FormBox["p",
      TraditionalForm], 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]]}], "}"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970228044455*^9},
 CellTags->"FCE",
 CellLabel->"Out[613]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "StandardForm"}]], "Input",
 CellTags->"FCE",
 CellLabel->"In[614]:="],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"DiracGamma", "[", 
    RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], "]"}], ",", 
   RowBox[{"DiracGamma", "[", 
    RowBox[{
     RowBox[{"LorentzIndex", "[", 
      RowBox[{"\[Rho]", ",", "D"}], "]"}], ",", "D"}], "]"}], ",", 
   RowBox[{"DiracGamma", "[", 
    RowBox[{"Momentum", "[", "p", "]"}], "]"}], ",", 
   RowBox[{"Pair", "[", 
    RowBox[{
     RowBox[{"Momentum", "[", "p", "]"}], ",", 
     RowBox[{"Momentum", "[", "q", "]"}]}], "]"}], ",", 
   RowBox[{"Pair", "[", 
    RowBox[{
     RowBox[{"LorentzIndex", "[", "\[Alpha]", "]"}], ",", 
     RowBox[{"LorentzIndex", "[", "\[Beta]", "]"}]}], "]"}], ",", 
   RowBox[{"Pair", "[", 
    RowBox[{
     RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
     RowBox[{"Momentum", "[", "p", "]"}]}], "]"}]}], "}"}]], "Output",
 CellChangeTimes->{3.500970228093855*^9},
 CellTags->"FCE",
 CellLabel->"Out[614]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"FCE", "[", "%", "]"}], "//", "StandardForm"}]], "Input",
 CellTags->"FCE",
 CellLabel->"In[615]:="],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"GA", "[", "\[Mu]", "]"}], ",", 
   RowBox[{"GAD", "[", "\[Rho]", "]"}], ",", 
   RowBox[{"GS", "[", "p", "]"}], ",", 
   RowBox[{"SP", "[", 
    RowBox[{"p", ",", "q"}], "]"}], ",", 
   RowBox[{"MT", "[", 
    RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}], ",", 
   RowBox[{"FV", "[", 
    RowBox[{"p", ",", "\[Mu]"}], "]"}]}], "}"}]], "Output",
 CellChangeTimes->{3.500970228182517*^9},
 CellTags->"FCE",
 CellLabel->"Out[615]//StandardForm="]
}, Open  ]]
}, Open  ]],

Cell["FCI", "ObjectName",
 CellTags->"FCI"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FCI"],

Cell["\<\
FCI[exp] translates exp into the internal FeynCalc \
(datatype-)representation.FCI is equivalent to FeynCalcInternal.\
\>", "Text",
 CellTags->"FCI"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FeynCalcExternal",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FeynCalcExternal",
  ButtonNote->"FeynCalcExternal"],
 ", ",
 ButtonBox["FeynCalcInternal",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FeynCalcInternal",
  ButtonNote->"FeynCalcInternal"],
 ", ",
 ButtonBox["FCE",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FCE",
  ButtonNote->"FCE"],
 "."
}], "Text",
 CellTags->"FCI"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"FCI"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"GA", "[", "\[Mu]", "]"}], ",", 
   RowBox[{"GAD", "[", "\[Rho]", "]"}], ",", 
   RowBox[{"GS", "[", "p", "]"}], ",", 
   RowBox[{"SP", "[", 
    RowBox[{"p", ",", "q"}], "]"}], ",", 
   RowBox[{"MT", "[", 
    RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}], ",", 
   RowBox[{"FV", "[", 
    RowBox[{"p", ",", "\[Mu]"}], "]"}]}], "}"}]], "Input",
 CellTags->"FCI",
 CellLabel->"In[616]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    SuperscriptBox["\[Gamma]", "\[Mu]"], ",", 
    SuperscriptBox["\[Gamma]", "\[Rho]"], ",", 
    RowBox[{"\[Gamma]", "\[CenterDot]", "p"}], ",", 
    FormBox[
     RowBox[{
      FormBox["p",
       TraditionalForm], "\[CenterDot]", 
      FormBox["q",
       TraditionalForm]}],
     TraditionalForm], ",", 
    SuperscriptBox["g", 
     RowBox[{"\[Alpha]", "\[Beta]"}]], ",", 
    SuperscriptBox["p", "\[Mu]"]}], "}"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009702282595673`*^9},
 CellTags->"FCI",
 CellLabel->"Out[616]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "StandardForm"}]], "Input",
 CellTags->"FCI",
 CellLabel->"In[617]:="],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"GA", "[", "\[Mu]", "]"}], ",", 
   RowBox[{"GAD", "[", "\[Rho]", "]"}], ",", 
   RowBox[{"GS", "[", "p", "]"}], ",", 
   RowBox[{"SP", "[", 
    RowBox[{"p", ",", "q"}], "]"}], ",", 
   RowBox[{"MT", "[", 
    RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}], ",", 
   RowBox[{"FV", "[", 
    RowBox[{"p", ",", "\[Mu]"}], "]"}]}], "}"}]], "Output",
 CellChangeTimes->{3.50097022833593*^9},
 CellTags->"FCI",
 CellLabel->"Out[617]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "FCI"}]], "Input",
 CellTags->"FCI",
 CellLabel->"In[618]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], ",", 
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox["\[Rho]",
       TraditionalForm],
      TraditionalForm]], ",", 
    RowBox[{
     FormBox["\<\"\[Gamma]\"\>",
      TraditionalForm], "\[CenterDot]", 
     FormBox["p",
      TraditionalForm]}], ",", 
    RowBox[{
     FormBox["p",
      TraditionalForm], "\[CenterDot]", 
     FormBox["q",
      TraditionalForm]}], ",", 
    SuperscriptBox["g", 
     RowBox[{"\[Alpha]", "\[Beta]"}]], ",", 
    SuperscriptBox[
     FormBox["p",
      TraditionalForm], 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]]}], "}"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009702283814363`*^9},
 CellTags->"FCI",
 CellLabel->"Out[618]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "StandardForm"}]], "Input",
 CellTags->"FCI",
 CellLabel->"In[619]:="],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"DiracGamma", "[", 
    RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], "]"}], ",", 
   RowBox[{"DiracGamma", "[", 
    RowBox[{
     RowBox[{"LorentzIndex", "[", 
      RowBox[{"\[Rho]", ",", "D"}], "]"}], ",", "D"}], "]"}], ",", 
   RowBox[{"DiracGamma", "[", 
    RowBox[{"Momentum", "[", "p", "]"}], "]"}], ",", 
   RowBox[{"Pair", "[", 
    RowBox[{
     RowBox[{"Momentum", "[", "p", "]"}], ",", 
     RowBox[{"Momentum", "[", "q", "]"}]}], "]"}], ",", 
   RowBox[{"Pair", "[", 
    RowBox[{
     RowBox[{"LorentzIndex", "[", "\[Alpha]", "]"}], ",", 
     RowBox[{"LorentzIndex", "[", "\[Beta]", "]"}]}], "]"}], ",", 
   RowBox[{"Pair", "[", 
    RowBox[{
     RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
     RowBox[{"Momentum", "[", "p", "]"}]}], "]"}]}], "}"}]], "Output",
 CellChangeTimes->{3.500970228427684*^9},
 CellTags->"FCI",
 CellLabel->"Out[619]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"FCE", "[", "%", "]"}], "//", "StandardForm"}]], "Input",
 CellTags->"FCI",
 CellLabel->"In[620]:="],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"GA", "[", "\[Mu]", "]"}], ",", 
   RowBox[{"GAD", "[", "\[Rho]", "]"}], ",", 
   RowBox[{"GS", "[", "p", "]"}], ",", 
   RowBox[{"SP", "[", 
    RowBox[{"p", ",", "q"}], "]"}], ",", 
   RowBox[{"MT", "[", 
    RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}], ",", 
   RowBox[{"FV", "[", 
    RowBox[{"p", ",", "\[Mu]"}], "]"}]}], "}"}]], "Output",
 CellChangeTimes->{3.50097022850348*^9},
 CellTags->"FCI",
 CellLabel->"Out[620]//StandardForm="]
}, Open  ]]
}, Open  ]],

Cell["FCIntegral", "ObjectName",
 CellTags->"FCIntegral"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FCIntegral"],

Cell["\<\
FCIntegral is the head of integrals in a setting of the option IntegralTable \
of FeynAmpDenominatorSimplify. Currently implemented only for 2-loop \
integrals.\
\>", "Text",
 CellTags->"FCIntegral"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["IntegralTable",
  BaseStyle->"AddOnsLink",
  ButtonData:>"IntegralTable",
  ButtonNote->"IntegralTable"],
 ", ",
 ButtonBox["FeynAmpDenominatorSimplify",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FeynAmpDenominatorSimplify",
  ButtonNote->"FeynAmpDenominatorSimplify"],
 "."
}], "Text",
 CellTags->"FCIntegral"]
}, Open  ]],

Cell["FCIntegrate", "ObjectName",
 CellTags->"FCIntegrate"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FCIntegrate"],

Cell["\<\
FCIntegrate is an option of certain Feynman integral related functions. It \
determines which integration function is used to evaluate analytic integrals. \
Possible settings include Integrate, NIntegrate, (Dot[Integratedx@@#2, #1] &).\
\>", "Text",
 CellTags->"FCIntegrate"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FCNIntegrate",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FCNIntegrate",
  ButtonNote->"FCNIntegrate"],
 "."
}], "Text",
 CellTags->"FCIntegrate"]
}, Open  ]],

Cell["FCNIntegrate", "ObjectName",
 CellTags->"FCNIntegrate"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FCNIntegrate"],

Cell["\<\
FCNIntegrate is an option of certain Feynman integral related functions which \
may return output containing both integrals that can be evaluated and \
integrals that can only be evaluated numerically. It then determines which \
integration function is used to evaluate numeric integrals. Possible settings \
include NIntegrate, (0*#1)&, (Dot[Integratedx@@#2, #1] &).\
\>", "Text",
 CellTags->"FCNIntegrate"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FCIntegrate",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FCIntegrate",
  ButtonNote->"FCIntegrate"],
 "."
}], "Text",
 CellTags->"FCNIntegrate"]
}, Open  ]],

Cell["FC2RHI ", "ObjectName",
 CellTags->"FC2RHI"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FC2RHI"],

Cell["\<\
FC2RHI[exp, k1, k2] transforms all 2-loop OPE-integrals in FeynAmpDenominator \
form to the RHI-integrals.  FC2RHI[exp] is equivalent to FC2RHI[exp,q1,q2]. \
The option IncludePair governs the inclusion  of scalar products p.k1, p.k2 \
and k1.k2 (setting True).\
\>", "Text",
 CellTags->"FC2RHI"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "FC2RHI", "]"}]], "Input",
 CellTags->"FC2RHI",
 CellLabel->"In[621]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
    RowBox[{"IncludePair", "\[Rule]", "True"}], ",", 
    RowBox[{"Do", "\[Rule]", "True"}]}], "}"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970228552112*^9},
 CellTags->"FC2RHI",
 CellLabel->"Out[621]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t", "=", 
  RowBox[{
   RowBox[{
    RowBox[{"FAD", "[", 
     RowBox[{"q1", ",", 
      RowBox[{"q1", "-", "q2"}], ",", 
      RowBox[{"q2", "-", "p"}]}], "]"}], " ", 
    RowBox[{
     RowBox[{"SP", "[", 
      RowBox[{"q1", ",", "OPEDelta"}], "]"}], "^", "OPEm"}]}], "//", 
   "FCI"}]}]], "Input",
 CellLabel->"In[622]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{
      FormBox["\[CapitalDelta]",
       TraditionalForm], "\[CenterDot]", 
      FormBox["q1",
       TraditionalForm]}], ")"}], "m"], 
   RowBox[{
    SuperscriptBox["q1", "2"], ".", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"q1", "-", "q2"}], ")"}], "2"], ".", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"q2", "-", "p"}], ")"}], "2"]}]], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970228645689*^9},
 CellLabel->"Out[622]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FC2RHI", "[", 
  RowBox[{"t", ",", "q1", ",", "q2"}], "]"}]], "Input",
 CellLabel->"In[623]:="],

Cell[BoxData[
 FormBox[
  SubsuperscriptBox[
   StyleBox["T",
    FontFamily->"Times",
    FontWeight->"Bold"], 
   StyleBox[
    RowBox[{"1", "0", "0", "1", "1"}],
    FontColor->RGBColor[0, 0, 1]], 
   RowBox[{
    FormBox["m",
     TraditionalForm], 
    FormBox["0",
     TraditionalForm], 
    FormBox["0",
     TraditionalForm], 
    FormBox["0",
     TraditionalForm], 
    FormBox["0",
     TraditionalForm]}]], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970228793119*^9},
 CellLabel->"Out[623]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "InputForm"}]], "Input",
 CellLabel->"In[624]:="],

Cell["RHI[{0, 0, OPEm, 0, 0}, {0, 1, 1, 0, 1}]", "Output",
 CellChangeTimes->{3.5009702288630342`*^9},
 CellLabel->"Out[624]//InputForm="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["RHI",
  BaseStyle->"AddOnsLink",
  ButtonData:>"RHI",
  ButtonNote->"RHI"],
 "."
}], "Text",
 CellTags->"FC2RHI"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"FC2RHI"],

Cell[BoxData[""], "Input",
 CellTags->"FC2RHI",
 CellLabel->"In[625]:="]
}, Open  ]],

Cell["FC2TLI ", "ObjectName",
 CellTags->"FC2TLI"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FC2TLI"],

Cell["\<\
FC2TLI[exp, k1, k2] transforms all 2-loop OPE-integrals in FeynAmpDenominator \
form to the TLI-integrals.  The option IncludePair governs the inclusion  of \
scalar products p.k1, p.k2 and k1.k2 (setting True).\
\>", "Text",
 CellTags->"FC2TLI"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "FC2TLI", "]"}]], "Input",
 CellTags->"FC2TLI",
 CellLabel->"In[626]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
    RowBox[{"IncludePair", "\[Rule]", "True"}], ",", 
    RowBox[{"Do", "\[Rule]", "True"}]}], "}"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970228975163*^9},
 CellTags->"FC2TLI",
 CellLabel->"Out[626]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "FAD"}]], "Input",
 CellLabel->"In[627]:="],

Cell[BoxData[
 StyleBox["\<\"FAD[q, q-p, ...] denotes 1/(q^2 (q-p)^2 ...).\\nFAD[{q1,m}, \
{q1-p,m}, q2, ...] is\\n1/( (q1^2 - m^2) ( (q1-p)^2 - m^2 ) q2^2 ... \
).\\n(Translation into FeynCalc internal form is performed \
by\\nFeynCalcInternal.)\"\>", "MSG"]], "Print", "PrintUsage",
 CellChangeTimes->{3.5009702298099413`*^9},
 CellTags->"Info3500973829-4982097"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t", "=", 
  RowBox[{
   RowBox[{"FAD", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"q1", ",", "m1"}], "}"}], ",", 
     RowBox[{"q2", "-", "q1"}], ",", 
     RowBox[{"q2", "-", "p"}]}], "]"}], " ", 
   RowBox[{
    RowBox[{"SP", "[", 
     RowBox[{"q1", ",", "OPEDelta"}], "]"}], "^", "OPEm"}]}]}]], "Input",
 CellLabel->"In[628]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox[
    RowBox[{"(", 
     FormBox[
      RowBox[{
       FormBox["\[CapitalDelta]",
        TraditionalForm], "\[CenterDot]", 
       FormBox["q1",
        TraditionalForm]}],
      TraditionalForm], ")"}], "m"], " ", 
   FormBox[
    FractionBox["1", 
     RowBox[{
      RowBox[{"(", 
       InterpretationBox[
        RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"q2", "-", "p"}], ")"}], "2"], 
         "\[InvisibleSpace]", "\<\"]\"\>"}],
        SequenceForm["[", (-$CellContext`p + $CellContext`q2)^2, "]"],
        Editable->False], ")"}], " ", 
      RowBox[{"(", 
       InterpretationBox[
        RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"q2", "-", "q1"}], ")"}], "2"], 
         "\[InvisibleSpace]", "\<\"]\"\>"}],
        SequenceForm["[", (-$CellContext`q1 + $CellContext`q2)^2, "]"],
        Editable->False], ")"}], " ", 
      RowBox[{"(", 
       InterpretationBox[
        RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
         SuperscriptBox["q1", "2"], "\[InvisibleSpace]", "\<\"-\"\>", 
         "\[InvisibleSpace]", 
         SuperscriptBox["m1", "2"], "\[InvisibleSpace]", "\<\"]\"\>"}],
        SequenceForm["[", $CellContext`q1^2, "-", $CellContext`m1^2, "]"],
        Editable->False], ")"}]}]],
    TraditionalForm]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009702306024933`*^9},
 CellLabel->"Out[628]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FC2TLI", "[", 
  RowBox[{"t", ",", "q1", ",", "q2"}], "]"}]], "Input",
 CellLabel->"In[629]:="],

Cell[BoxData[
 FormBox[
  SubsuperscriptBox[
   StyleBox["T",
    FontFamily->"Times",
    FontWeight->"Bold"], 
   RowBox[{
    StyleBox["1",
     FontWeight->"Bold",
     FontColor->RGBColor[1, 0, 0]], 
    StyleBox["0",
     FontColor->RGBColor[0, 0, 1]], 
    StyleBox["0",
     FontColor->RGBColor[0, 0, 1]], 
    StyleBox["1",
     FontColor->RGBColor[0, 0, 1]], 
    StyleBox["1",
     FontColor->RGBColor[0, 0, 1]]}], 
   RowBox[{
    FormBox["m",
     TraditionalForm], 
    FormBox["0",
     TraditionalForm], 
    FormBox["0",
     TraditionalForm], 
    FormBox["0",
     TraditionalForm], 
    FormBox["0",
     TraditionalForm]}]], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970230776005*^9},
 CellLabel->"Out[629]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "InputForm"}]], "Input",
 CellLabel->"In[630]:="],

Cell["TLI[{OPEm, 0, 0, 0, 0}, {{1, m1}, 0, 0, 1, 1}]", "Output",
 CellChangeTimes->{3.5009702308465137`*^9},
 CellLabel->"Out[630]//InputForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TLI2FC", "[", "%", "]"}]], "Input",
 CellLabel->"In[631]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{
      FormBox["\[CapitalDelta]",
       TraditionalForm], "\[CenterDot]", 
      FormBox["q1",
       TraditionalForm]}], ")"}], "m"], 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox["q1", "2"], "-", 
      SuperscriptBox["m1", "2"]}], ")"}], ".", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"q1", "-", "q2"}], ")"}], "2"], ".", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"q2", "-", "p"}], ")"}], "2"]}]], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970230943022*^9},
 CellLabel->"Out[631]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["TLI",
  BaseStyle->"AddOnsLink",
  ButtonData:>"TLI",
  ButtonNote->"TLI"],
 "."
}], "Text",
 CellTags->"FC2TLI"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"FC2TLI"],

Cell[BoxData[""], "Input",
 CellTags->"FC2TLI",
 CellLabel->"In[632]:="]
}, Open  ]],

Cell["FDS", "ObjectName",
 CellTags->"FDS"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FDS"],

Cell["FDS is shorthand for FeynAmpDenominatorSimplify.", "Text",
 CellTags->"FDS"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FeynAmpDenominatorSimplify",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FeynAmpDenominatorSimplify",
  ButtonNote->"FeynAmpDenominatorSimplify"],
 "."
}], "Text",
 CellTags->"FDS"]
}, Open  ]],

Cell["FermionSpinSum", "ObjectName",
 CellTags->"FermionSpinSum"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FermionSpinSum"],

Cell["\<\
FermionSpinSum[x] constructs the Traces out of squared ampliudes.\
\>", "Text",
 CellTags->"FermionSpinSum"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "FermionSpinSum", "]"}]], "Input",
 CellTags->"FermionSpinSum",
 CellLabel->"In[633]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"SpinPolarizationSum", "\[Rule]", "Identity"}], ",", 
    RowBox[{"SpinorCollect", "\[Rule]", "False"}], ",", 
    RowBox[{"ExtraFactor", "\[Rule]", "1"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970231024826*^9},
 CellTags->"FermionSpinSum",
 CellLabel->"Out[633]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Spinor",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Spinor",
  ButtonNote->"Spinor"],
 ", ",
 ButtonBox["ComplexConjugate",
  BaseStyle->"AddOnsLink",
  ButtonData:>"ComplexConjugate",
  ButtonNote->"ComplexConjugate"],
 ", ",
 ButtonBox["DiracTrace",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracTrace",
  ButtonNote->"DiracTrace"],
 ", ",
 ButtonBox["Tr",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Tr",
  ButtonNote->"Tr"],
 "."
}], "Text",
 CellTags->"FermionSpinSum"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"FermionSpinSum"],

Cell[TextData[{
 "Spinors of fermions of mass ",
 StyleBox["m",
  FontSlant->"Italic"],
 " are normalized to have square 2 ",
 StyleBox["m",
  FontSlant->"Italic"],
 " or -2 ",
 StyleBox["m",
  FontSlant->"Italic"],
 "."
}], "Text",
 CellTags->"FermionSpinSum"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"SpinorUBar", "[", 
   RowBox[{
    RowBox[{"Momentum", "[", "p", "]"}], ",", "m"}], "]"}], ".", 
  RowBox[{"SpinorU", "[", 
   RowBox[{
    RowBox[{"Momentum", "[", "p", "]"}], ",", "m"}], "]"}]}]], "Input",
 CellTags->"FermionSpinSum",
 CellLabel->"In[634]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    FormBox[
     OverscriptBox["\<\"u\"\>", "_"],
     TraditionalForm], "(", 
    FormBox["p",
     TraditionalForm], ",", "m", ")"}], ".", 
   RowBox[{"u", "(", 
    FormBox["p",
     TraditionalForm], ",", "m", ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970231071287*^9},
 CellTags->"FermionSpinSum",
 CellLabel->"Out[634]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"%", "//", "FCI"}], "//", "FermionSpinSum"}]], "Input",
 CellTags->"FermionSpinSum",
 CellLabel->"In[635]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"tr", "(", 
   FormBox[
    RowBox[{"m", "+", 
     RowBox[{
      FormBox["\<\"\[Gamma]\"\>",
       TraditionalForm], "\[CenterDot]", 
      FormBox["p",
       TraditionalForm]}]}],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970231170589*^9},
 CellTags->"FermionSpinSum",
 CellLabel->"Out[635]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "/.", 
  RowBox[{"DiracTrace", "\[Rule]", "TR"}]}]], "Input",
 CellTags->"FermionSpinSum",
 CellLabel->"In[636]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"4", " ", "m"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970231709279*^9},
 CellTags->"FermionSpinSum",
 CellLabel->"Out[636]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"SpinorVBar", "[", 
   RowBox[{
    RowBox[{"Momentum", "[", "p", "]"}], ",", "m"}], "]"}], ".", 
  RowBox[{"SpinorV", "[", 
   RowBox[{
    RowBox[{"Momentum", "[", "p", "]"}], ",", "m"}], "]"}]}]], "Input",
 CellTags->"FermionSpinSum",
 CellLabel->"In[637]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    FormBox[
     OverscriptBox["\<\"v\"\>", "_"],
     TraditionalForm], "(", 
    FormBox["p",
     TraditionalForm], ",", "m", ")"}], ".", 
   RowBox[{"v", "(", 
    FormBox["p",
     TraditionalForm], ",", "m", ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970231867214*^9},
 CellTags->"FermionSpinSum",
 CellLabel->"Out[637]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"%", "//", "FCI"}], "//", "FermionSpinSum"}]], "Input",
 CellTags->"FermionSpinSum",
 CellLabel->"In[638]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"tr", "(", 
   FormBox[
    RowBox[{
     RowBox[{
      FormBox["\<\"\[Gamma]\"\>",
       TraditionalForm], "\[CenterDot]", 
      FormBox["p",
       TraditionalForm]}], "-", "m"}],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970232023343*^9},
 CellTags->"FermionSpinSum",
 CellLabel->"Out[638]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "/.", 
  RowBox[{"DiracTrace", "\[Rule]", "TR"}]}]], "Input",
 CellTags->"FermionSpinSum",
 CellLabel->"In[639]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", "4"}], " ", "m"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009702321881227`*^9},
 CellTags->"FermionSpinSum",
 CellLabel->"Out[639]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t", "=", 
  RowBox[{
   RowBox[{"Spinor", "[", 
    RowBox[{"k1", ",", "m"}], "]"}], ".", 
   RowBox[{"DiracSlash", "[", "p", "]"}], ".", 
   RowBox[{"GA", "[", "5", "]"}], ".", 
   RowBox[{"Spinor", "[", 
    RowBox[{"p1", ",", "m"}], "]"}]}]}]], "Input",
 CellTags->"FermionSpinSum",
 CellLabel->"In[640]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"\[CurlyPhi]", "(", 
    FormBox["k1",
     TraditionalForm], ",", "m", ")"}], ".", 
   RowBox[{"(", 
    RowBox[{
     FormBox["\<\"\[Gamma]\"\>",
      TraditionalForm], "\[CenterDot]", 
     FormBox["p",
      TraditionalForm]}], ")"}], ".", 
   SuperscriptBox["\[Gamma]", "5"], ".", 
   RowBox[{"\[CurlyPhi]", "(", 
    FormBox["p1",
     TraditionalForm], ",", "m", ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970232393175*^9},
 CellTags->"FermionSpinSum",
 CellLabel->"Out[640]="]
}, Open  ]],

Cell["\<\
Notice that SpinorUBar and SpinorU are only input functions. Internally they \
are converted to Spinor objects.\
\>", "Text",
 CellTags->"FermionSpinSum"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ct", "=", 
  RowBox[{"ComplexConjugate", "[", "t", "]"}]}]], "Input",
 CellTags->"FermionSpinSum",
 CellLabel->"In[641]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   RowBox[{
    RowBox[{"\[CurlyPhi]", "(", 
     FormBox["p1",
      TraditionalForm], ",", "m", ")"}], ".", 
    SuperscriptBox["\[Gamma]", "5"], ".", 
    RowBox[{"(", 
     RowBox[{
      FormBox["\<\"\[Gamma]\"\>",
       TraditionalForm], "\[CenterDot]", 
      FormBox["p",
       TraditionalForm]}], ")"}], ".", 
    RowBox[{"\[CurlyPhi]", "(", 
     FormBox["k1",
      TraditionalForm], ",", "m", ")"}]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970232572721*^9},
 CellTags->"FermionSpinSum",
 CellLabel->"Out[641]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FermionSpinSum", "[", 
  RowBox[{"t", " ", "ct"}], "]"}]], "Input",
 CellTags->"FermionSpinSum",
 CellLabel->"In[642]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   RowBox[{"tr", "(", 
    FormBox[
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         FormBox["\<\"\[Gamma]\"\>",
          TraditionalForm], "\[CenterDot]", 
         FormBox["k1",
          TraditionalForm]}], "+", "m"}], ")"}], ".", 
      RowBox[{"(", 
       RowBox[{
        FormBox["\<\"\[Gamma]\"\>",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p",
         TraditionalForm]}], ")"}], ".", 
      SuperscriptBox["\[Gamma]", "5"], ".", 
      RowBox[{"(", 
       RowBox[{"m", "+", 
        RowBox[{
         FormBox["\<\"\[Gamma]\"\>",
          TraditionalForm], "\[CenterDot]", 
         FormBox["p1",
          TraditionalForm]}]}], ")"}], ".", 
      SuperscriptBox["\[Gamma]", "5"], ".", 
      RowBox[{"(", 
       RowBox[{
        FormBox["\<\"\[Gamma]\"\>",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p",
         TraditionalForm]}], ")"}]}],
     TraditionalForm], ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970232744256*^9},
 CellTags->"FermionSpinSum",
 CellLabel->"Out[642]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", " ", "/.", 
  RowBox[{"DiracTrace", "\[Rule]", "TR"}]}]], "Input",
 CellTags->"FermionSpinSum",
 CellLabel->"In[643]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", "4"}], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], "2"], " ", 
      RowBox[{
       FormBox["k1",
        TraditionalForm], "\[CenterDot]", 
       FormBox["p1",
        TraditionalForm]}]}], "-", 
     RowBox[{"2", " ", 
      RowBox[{
       FormBox["k1",
        TraditionalForm], "\[CenterDot]", 
       FormBox["p",
        TraditionalForm]}], " ", 
      RowBox[{
       FormBox["p",
        TraditionalForm], "\[CenterDot]", 
       FormBox["p1",
        TraditionalForm]}]}], "+", 
     RowBox[{
      SuperscriptBox["m", "2"], " ", 
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], "2"]}]}], ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970233050468*^9},
 CellTags->"FermionSpinSum",
 CellLabel->"Out[643]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"Clear", "[", 
  RowBox[{"t", ",", " ", "ct"}], "]"}]], "Input",
 CellTags->"FermionSpinSum",
 CellLabel->"In[644]:="]
}, Open  ]],

Cell["FeynAmp", "ObjectName",
 CellTags->"FeynAmp"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FeynAmp"],

Cell["\<\
FeynAmp[q, amp] is the head of a Feynman amplitude. amp denotes the \
analytical expression for the amplitude and q is the integration variable. \
FeynAmp[q1, q2, amp] denotes a two-loop amplitude.\
\>", "Text",
 CellTags->"FeynAmp"],

Cell["\<\
FeynAmp has no functional properties and serves just as a head. There are \
however special typesetting rules attached.\
\>", "Text",
 CellTags->"FeynAmp"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Amplitude",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Amplitude",
  ButtonNote->"Amplitude"],
 "."
}], "Text",
 CellTags->"FeynAmp"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"FeynAmp"],

Cell["\<\
This is a 1-loop gluon self-energy amplitude (ignoring factors of (2 \[Pi])).\
\
\>", "Text",
 CellTags->"FeynAmp"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FeynAmp", "[", 
  RowBox[{"q", ",", 
   RowBox[{
    RowBox[{"GV", "[", 
     RowBox[{"p", ",", "\[Mu]", ",", "a", ",", " ", 
      RowBox[{"q", "-", "p"}], ",", "\[Alpha]", ",", "c", ",", " ", 
      RowBox[{"-", "q"}], ",", "\[Beta]", ",", "e"}], "]"}], " ", 
    RowBox[{"GP", "[", 
     RowBox[{
      RowBox[{"p", "-", "q"}], ",", " ", "\[Alpha]", ",", "c", ",", " ", 
      "\[Rho]", ",", "d"}], "]"}], 
    RowBox[{"GV", "[", 
     RowBox[{
      RowBox[{"-", "p"}], ",", "\[Nu]", ",", "b", ",", " ", 
      RowBox[{"p", "-", "q"}], ",", "\[Rho]", ",", "d", ",", " ", "q", ",", 
      "\[Sigma]", ",", "f"}], "]"}], " ", 
    RowBox[{"GP", "[", 
     RowBox[{
     "q", ",", " ", "\[Beta]", ",", "e", ",", " ", "\[Sigma]", ",", "f"}], 
     "]"}]}]}], "]"}]], "Input",
 CellTags->"FeynAmp",
 CellLabel->"In[645]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"\[Integral]", 
   StyleBox[
    RowBox[{
     SuperscriptBox["\[DifferentialD]", "D"], "Global`q"}],
    ZeroWidthTimes->True], "(", 
   FormBox[
    RowBox[{
     SubscriptBox["f", 
      RowBox[{
       FormBox[
        FormBox["b",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["d",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["f",
         TraditionalForm],
        TraditionalForm]}]], " ", 
     SubscriptBox["f", 
      RowBox[{
       FormBox[
        FormBox["a",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["c",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["e",
         TraditionalForm],
        TraditionalForm]}]], " ", 
     RowBox[{
      SubsuperscriptBox["\[CapitalPi]", 
       RowBox[{"e", "f"}], 
       RowBox[{"\[Beta]", "\[Sigma]"}]], "(", "q", ")"}], " ", 
     RowBox[{
      SuperscriptBox["V", 
       RowBox[{
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Alpha]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Beta]",
          TraditionalForm],
         TraditionalForm]}]], "(", 
      RowBox[{
       FormBox["p",
        TraditionalForm], 
       RowBox[{",", " "}], 
       FormBox[
        RowBox[{"q", "-", "p"}],
        TraditionalForm], 
       RowBox[{",", " "}], 
       FormBox[
        RowBox[{"-", "q"}],
        TraditionalForm]}], ")"}], " ", 
     RowBox[{
      SuperscriptBox["V", 
       RowBox[{
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Sigma]",
          TraditionalForm],
         TraditionalForm]}]], "(", 
      RowBox[{
       FormBox[
        RowBox[{"-", "p"}],
        TraditionalForm], 
       RowBox[{",", " "}], 
       FormBox[
        RowBox[{"p", "-", "q"}],
        TraditionalForm], 
       RowBox[{",", " "}], 
       FormBox["q",
        TraditionalForm]}], ")"}], " ", 
     RowBox[{
      SubsuperscriptBox["\[CapitalPi]", 
       RowBox[{"c", "d"}], 
       RowBox[{"\[Alpha]", "\[Rho]"}]], "(", 
      FormBox[
       RowBox[{"p", "-", "q"}],
       TraditionalForm], ")"}]}],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009702333021193`*^9},
 CellTags->"FeynAmp",
 CellLabel->"Out[645]="]
}, Open  ]],

Cell["This is a generic 2-loop amplitude.", "Text",
 CellTags->"FeynAmp"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FeynAmp", "[", 
  RowBox[{
   SubscriptBox["q", "1"], ",", 
   SubscriptBox["q", "2"], ",", "anyexpression"}], "]"}]], "Input",
 CellTags->"FeynAmp",
 CellLabel->"In[646]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"\[Integral]", 
   RowBox[{
    StyleBox[
     RowBox[{
      SuperscriptBox["\[DifferentialD]", "D"], 
      SubscriptBox["q", "1"]}],
     ZeroWidthTimes->True], "\[Integral]", 
    RowBox[{
     SuperscriptBox["\[DifferentialD]", "D"], 
     SubscriptBox["q", "2"]}]}], "(", "anyexpression", ")"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970233854405*^9},
 CellTags->"FeynAmp",
 CellLabel->"Out[646]="]
}, Open  ]]
}, Open  ]],

Cell["FeynAmpDenominator", "ObjectName",
 CellTags->"FeynAmpDenominator"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FeynAmpDenominator"],

Cell["\<\
FeynAmpDenominator[ PropagatorDenominator[ ... ], PropagatorDenominator[ ... \
], ...] is the head of the denominators of the propagators, i.e., \
FeynAmpDenominator[x] is the representation of 1/x .\
\>", "Text",
 CellTags->"FeynAmpDenominator"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FAD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FAD",
  ButtonNote->"FAD"],
 ", ",
 ButtonBox["FeynAmpDenominatorSimplify",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FeynAmpDenominatorSimplify",
  ButtonNote->"FeynAmpDenominatorSimplify"],
 "."
}], "Text",
 CellTags->"FeynAmpDenominator"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"FeynAmpDenominator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FeynAmpDenominator", "[", 
  RowBox[{"PropagatorDenominator", "[", 
   RowBox[{"p", ",", "m"}], "]"}], "]"}]], "Input",
 CellTags->"FeynAmpDenominator",
 CellLabel->"In[647]:="],

Cell[BoxData[
 FormBox[
  FractionBox["1", 
   RowBox[{
    SuperscriptBox["p", "2"], "-", 
    SuperscriptBox["m", "2"]}]], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970233935955*^9},
 CellTags->"FeynAmpDenominator",
 CellLabel->"Out[647]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FeynAmpDenominator", "[", 
  RowBox[{
   RowBox[{"PropagatorDenominator", "[", 
    RowBox[{"p", ",", "m"}], "]"}], ",", 
   RowBox[{"PropagatorDenominator", "[", 
    RowBox[{
     RowBox[{"p", "-", "q"}], ",", "m"}], "]"}]}], "]"}]], "Input",
 CellTags->"FeynAmpDenominator",
 CellLabel->"In[648]:="],

Cell[BoxData[
 FormBox[
  FractionBox["1", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox["p", "2"], "-", 
      SuperscriptBox["m", "2"]}], ")"}], ".", 
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"p", "-", "q"}], ")"}], "2"], "-", 
      SuperscriptBox["m", "2"]}], ")"}]}]], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970234099381*^9},
 CellTags->"FeynAmpDenominator",
 CellLabel->"Out[648]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"t", "=", 
   RowBox[{"FeynAmpDenominator", "[", 
    RowBox[{"PropagatorDenominator", "[", 
     RowBox[{"p", ",", "m"}], "]"}], "]"}]}], ";"}]], "Input",
 CellTags->"FeynAmpDenominator",
 CellLabel->"In[649]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StandardForm", "[", 
  RowBox[{"t", "//", "FCI"}], "]"}]], "Input",
 CellTags->"FeynAmpDenominator",
 CellLabel->"In[650]:="],

Cell[BoxData[
 RowBox[{"FeynAmpDenominator", "[", 
  RowBox[{"PropagatorDenominator", "[", 
   RowBox[{
    RowBox[{"Momentum", "[", 
     RowBox[{"p", ",", "D"}], "]"}], ",", "m"}], "]"}], "]"}]], "Output",
 CellChangeTimes->{3.500970234395211*^9},
 CellTags->"FeynAmpDenominator",
 CellLabel->"Out[650]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StandardForm", "[", 
  RowBox[{"t", "//", "FCE"}], "]"}]], "Input",
 CellTags->"FeynAmpDenominator",
 CellLabel->"In[651]:="],

Cell[BoxData[
 RowBox[{"FAD", "[", 
  RowBox[{"{", 
   RowBox[{"p", ",", "m"}], "}"}], "]"}]], "Output",
 CellChangeTimes->{3.5009702346641808`*^9},
 CellTags->"FeynAmpDenominator",
 CellLabel->"Out[651]//StandardForm="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", "t", "]"}], ";"}]], "Input",
 CellTags->"FeynAmpDenominator",
 CellLabel->"In[652]:="]
}, Open  ]],

Cell["FeynAmpDenominatorCombine", "ObjectName",
 CellTags->"FeynAmpDenominatorCombine"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FeynAmpDenominatorCombine"],

Cell["\<\
FeynAmpDenominatorCombine[expr] expands expr with respect to \
FeynAmpDenominator and combines products of FeynAmpDenominator in expr into \
one FeynAmpDenominator.\
\>", "Text",
 CellTags->"FeynAmpDenominatorCombine"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FeynAmpDenominatorSplit",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FeynAmpDenominatorSplit",
  ButtonNote->"FeynAmpDenominatorSplit"],
 "."
}], "Text",
 CellTags->"FeynAmpDenominatorCombine"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"FeynAmpDenominatorCombine"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t", " ", "=", " ", 
  RowBox[{
   RowBox[{"FAD", "[", "q", "]"}], " ", 
   RowBox[{"FAD", "[", 
    RowBox[{"q", "-", "p"}], "]"}]}]}]], "Input",
 CellTags->"FeynAmpDenominatorCombine",
 CellLabel->"In[653]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox[
    FractionBox["1", 
     InterpretationBox[
      RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
       SuperscriptBox["q", "2"], "\[InvisibleSpace]", "\<\"]\"\>"}],
      SequenceForm["[", $CellContext`q^2, "]"],
      Editable->False]],
    TraditionalForm], " ", 
   FormBox[
    FractionBox["1", 
     InterpretationBox[
      RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"q", "-", "p"}], ")"}], "2"], 
       "\[InvisibleSpace]", "\<\"]\"\>"}],
      SequenceForm["[", (-$CellContext`p + $CellContext`q)^2, "]"],
      Editable->False]],
    TraditionalForm]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970234858974*^9},
 CellTags->"FeynAmpDenominatorCombine",
 CellLabel->"Out[653]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"FeynAmpDenominatorCombine", "[", "%", "]"}], "//", "FCE"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"FeynAmpDenominatorCombine",
 CellLabel->"In[654]:="],

Cell[BoxData[
 RowBox[{"FAD", "[", 
  RowBox[{"q", ",", 
   RowBox[{
    RowBox[{"-", "p"}], "+", "q"}]}], "]"}]], "Output",
 CellChangeTimes->{3.500970235012175*^9},
 CellTags->"FeynAmpDenominatorCombine",
 CellLabel->"Out[654]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"FeynAmpDenominatorSplit", "[", "%", "]"}], "//", "FCE"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"FeynAmpDenominatorCombine",
 CellLabel->"In[655]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"FAD", "[", "q", "]"}], " ", 
  RowBox[{"FAD", "[", 
   RowBox[{
    RowBox[{"-", "p"}], "+", "q"}], "]"}]}]], "Output",
 CellChangeTimes->{3.500970235165866*^9},
 CellTags->"FeynAmpDenominatorCombine",
 CellLabel->"Out[655]//StandardForm="]
}, Open  ]]
}, Open  ]],

Cell["FeynAmpDenominatorSimplify", "ObjectName",
 CellTags->"FeynAmpDenominatorSimplify"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FeynAmpDenominatorSimplify"],

Cell["\<\
FeynAmpDenominatorSimplify[exp] tries to simplify each PropagatorDenominator \
in a canonical way. FeynAmpDenominatorSimplify[exp, q1] simplifies all \
FeynAmpDenominator's in exp in a canonical way, including some translation of \
momenta. FeynAmpDenominatorSimplify[exp, q1, q2] additionally removes \
integrals with no mass scale.\
\>", "Text",
 CellTags->"FeynAmpDenominatorSimplify"],

Cell["FDS can be used as an alias.", "Text",
 CellTags->"FeynAmpDenominatorSimplify"],

Cell[CellGroupData[{

Cell[BoxData["FDS"], "Input",
 CellTags->"FeynAmpDenominatorSimplify",
 CellLabel->"In[656]:="],

Cell[BoxData[
 FormBox["FeynAmpDenominatorSimplify", TraditionalForm]], "Output",
 CellChangeTimes->{3.5009702353001947`*^9},
 CellTags->"FeynAmpDenominatorSimplify",
 CellLabel->"Out[656]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["OneLoopSimplify",
  BaseStyle->"AddOnsLink",
  ButtonData:>"OneLoopSimplify",
  ButtonNote->"OneLoopSimplify"],
 "."
}], "Text",
 CellTags->"FeynAmpDenominatorSimplify"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"FeynAmpDenominatorSimplify"],

Cell[TextData[{
 "The cornerstone of dimensional regularization is that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"\[Integral]", 
     RowBox[{
      SuperscriptBox["d", "n"], "k", " ", 
      RowBox[{
       RowBox[{"f", "(", "k", ")"}], "/", 
       SuperscriptBox["k", 
        RowBox[{"2", "m"}]]}]}]}], "=", " ", 
    RowBox[{"0", " ", "."}]}], TraditionalForm]]]
}], "Text",
 CellTags->"FeynAmpDenominatorSimplify"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FeynAmpDenominatorSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"f", "[", "k", "]"}], " ", 
    RowBox[{"FAD", "[", 
     RowBox[{"k", ",", "k"}], "]"}]}], ",", "k"}], "]"}]], "Input",
 CellTags->"FeynAmpDenominatorSimplify",
 CellLabel->"In[657]:="],

Cell[BoxData[
 FormBox["0", TraditionalForm]], "Output",
 CellChangeTimes->{3.50097023542299*^9},
 CellTags->"FeynAmpDenominatorSimplify",
 CellLabel->"Out[657]="]
}, Open  ]],

Cell[TextData[{
 "This brings ",
 Cell[BoxData[
  FormBox[
   RowBox[{"1", "/", 
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"k", "-", 
         SubscriptBox["p", "1"]}], ")"}], "2"], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"k", "-", 
         SubscriptBox["p", "2"]}], ")"}], "2"]}], ")"}]}], 
   TraditionalForm]]],
 " into a standard form."
}], "Text",
 CellTags->"FeynAmpDenominatorSimplify"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FeynAmpDenominatorSimplify", "[", 
  RowBox[{
   RowBox[{"FAD", "[", 
    RowBox[{
     RowBox[{"k", "-", 
      SubscriptBox["p", "1"]}], ",", 
     RowBox[{"k", "-", 
      SubscriptBox["p", "2"]}]}], "]"}], ",", "k"}], "]"}]], "Input",
 CellTags->"FeynAmpDenominatorSimplify",
 CellLabel->"In[658]:="],

Cell[BoxData[
 FormBox[
  FractionBox["1", 
   RowBox[{
    SuperscriptBox["k", "2"], ".", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"k", "-", 
       SubscriptBox["p", "1"], "+", 
       SubscriptBox["p", "2"]}], ")"}], "2"]}]], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970235520301*^9},
 CellTags->"FeynAmpDenominatorSimplify",
 CellLabel->"Out[658]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FeynAmpDenominatorSimplify", "[", 
  RowBox[{
   RowBox[{"FAD", "[", 
    RowBox[{"k", ",", "k", ",", 
     RowBox[{"k", "-", "q"}]}], "]"}], ",", "k"}], "]"}]], "Input",
 CellTags->"FeynAmpDenominatorSimplify",
 CellLabel->"In[659]:="],

Cell[BoxData[
 FormBox[
  FractionBox["1", 
   RowBox[{
    SuperscriptBox["k", "2"], ".", 
    SuperscriptBox["k", "2"], ".", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"k", "-", "q"}], ")"}], "2"]}]], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970235603445*^9},
 CellTags->"FeynAmpDenominatorSimplify",
 CellLabel->"Out[659]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FeynAmpDenominatorSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"f", "[", "k", "]"}], 
    RowBox[{"FAD", "[", 
     RowBox[{"k", ",", 
      RowBox[{"k", "-", "q"}], ",", 
      RowBox[{"k", "-", "q"}]}], "]"}]}], ",", "k"}], "]"}]], "Input",
 CellTags->"FeynAmpDenominatorSimplify",
 CellLabel->"In[660]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{"f", "(", 
    RowBox[{"q", "-", "k"}], ")"}], 
   RowBox[{
    SuperscriptBox["k", "2"], ".", 
    SuperscriptBox["k", "2"], ".", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"k", "-", "q"}], ")"}], "2"]}]], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970235693583*^9},
 CellTags->"FeynAmpDenominatorSimplify",
 CellLabel->"Out[660]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t", "=", 
  RowBox[{"FeynAmpDenominatorSimplify", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"FAD", "[", 
      RowBox[{
       RowBox[{"k", "-", 
        SubscriptBox["p", "1"]}], ",", 
       RowBox[{"k", "-", 
        SubscriptBox["p", "2"]}]}], "]"}], " ", 
     RowBox[{"SPD", "[", 
      RowBox[{"k", ",", "k"}], "]"}]}], ",", "k"}], "]"}]}]], "Input",
 CellTags->"FeynAmpDenominatorSimplify",
 CellLabel->"In[661]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    SuperscriptBox[
     FormBox["k",
      TraditionalForm], "2"], 
    RowBox[{
     SuperscriptBox["k", "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"k", "-", 
        SubscriptBox["p", "1"], "+", 
        SubscriptBox["p", "2"]}], ")"}], "2"]}]], "+", 
   FractionBox[
    SubsuperscriptBox["p", "1", "2"], 
    RowBox[{
     SuperscriptBox["k", "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"k", "-", 
        SubscriptBox["p", "1"], "+", 
        SubscriptBox["p", "2"]}], ")"}], "2"]}]], "-", 
   FractionBox[
    RowBox[{"2", " ", 
     RowBox[{
      FormBox["k",
       TraditionalForm], "\[CenterDot]", 
      FormBox[
       SubscriptBox["p", "1"],
       TraditionalForm]}]}], 
    RowBox[{
     SuperscriptBox["k", "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"k", "-", 
        SubscriptBox["p", "1"], "+", 
        SubscriptBox["p", "2"]}], ")"}], "2"]}]]}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.5009702358064623`*^9},
 CellTags->"FeynAmpDenominatorSimplify",
 CellLabel->"Out[661]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"r", "=", 
  RowBox[{"SPC", "[", 
   RowBox[{"t", ",", "k", ",", 
    RowBox[{"FDS", "\[Rule]", "True"}]}], "]"}]}]], "Input",
 CellTags->"FeynAmpDenominatorSimplify",
 CellLabel->"In[662]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{"2", " ", 
     RowBox[{
      FormBox["k",
       TraditionalForm], "\[CenterDot]", 
      FormBox[
       SubscriptBox["p", "2"],
       TraditionalForm]}]}], 
    RowBox[{
     SuperscriptBox["k", "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"k", "+", 
        SubscriptBox["p", "1"], "-", 
        SubscriptBox["p", "2"]}], ")"}], "2"]}]], "-", 
   FractionBox[
    RowBox[{
     SubsuperscriptBox["p", "1", "2"], "+", 
     SubsuperscriptBox["p", "2", "2"], "-", 
     RowBox[{"2", " ", 
      RowBox[{
       FormBox[
        SubscriptBox["p", "1"],
        TraditionalForm], "\[CenterDot]", 
       FormBox[
        SubscriptBox["p", "2"],
        TraditionalForm]}]}]}], 
    RowBox[{
     SuperscriptBox["k", "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"k", "+", 
        SubscriptBox["p", "1"], "-", 
        SubscriptBox["p", "2"]}], ")"}], "2"]}]], "+", 
   FractionBox[
    RowBox[{
     SubsuperscriptBox["p", "1", "2"], "+", 
     RowBox[{"2", " ", 
      SubsuperscriptBox["p", "2", "2"]}], "-", 
     RowBox[{"2", " ", 
      RowBox[{
       FormBox[
        SubscriptBox["p", "1"],
        TraditionalForm], "\[CenterDot]", 
       FormBox[
        SubscriptBox["p", "2"],
        TraditionalForm]}]}], "-", 
     RowBox[{"2", " ", 
      RowBox[{"(", 
       RowBox[{
        SubsuperscriptBox["p", "2", "2"], "-", 
        RowBox[{
         FormBox[
          SubscriptBox["p", "1"],
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "2"],
          TraditionalForm]}]}], ")"}]}]}], 
    RowBox[{
     SuperscriptBox["k", "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"k", "-", 
        SubscriptBox["p", "1"], "+", 
        SubscriptBox["p", "2"]}], ")"}], "2"]}]]}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.5009702359361963`*^9},
 CellTags->"FeynAmpDenominatorSimplify",
 CellLabel->"Out[662]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"OneLoopSimplify", "[", 
  RowBox[{"r", ",", "k"}], "]"}]], "Input",
 CellTags->"FeynAmpDenominatorSimplify",
 CellLabel->"In[663]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{
     RowBox[{"2", " ", 
      RowBox[{
       FormBox[
        SubscriptBox["p", "1"],
        TraditionalForm], "\[CenterDot]", 
       FormBox[
        SubscriptBox["p", "2"],
        TraditionalForm]}]}], "-", 
     SubsuperscriptBox["p", "2", "2"]}], 
    RowBox[{
     SuperscriptBox["k", "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"k", "-", 
        SubscriptBox["p", "1"], "+", 
        SubscriptBox["p", "2"]}], ")"}], "2"]}]], "-", 
   FractionBox[
    RowBox[{"2", " ", 
     RowBox[{
      FormBox["k",
       TraditionalForm], "\[CenterDot]", 
      FormBox[
       SubscriptBox["p", "2"],
       TraditionalForm]}]}], 
    RowBox[{
     SuperscriptBox["k", "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"k", "-", 
        SubscriptBox["p", "1"], "+", 
        SubscriptBox["p", "2"]}], ")"}], "2"]}]]}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.50097023627428*^9},
 CellTags->"FeynAmpDenominatorSimplify",
 CellLabel->"Out[663]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FDS", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"FAD", "[", 
     RowBox[{
      RowBox[{"k", "-", "p1"}], ",", 
      RowBox[{"k", "-", "p2"}]}], "]"}], 
    RowBox[{
     RowBox[{"SPD", "[", 
      RowBox[{"k", ",", "OPEDelta"}], "]"}], "^", "2"}]}], ",", "k"}], 
  "]"}]], "Input",
 CellTags->"FeynAmpDenominatorSimplify",
 CellLabel->"In[664]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    SuperscriptBox[
     FormBox[
      RowBox[{
       FormBox["k",
        TraditionalForm], "\[CenterDot]", 
       FormBox["\[CapitalDelta]",
        TraditionalForm]}],
      TraditionalForm], "2"], 
    RowBox[{
     SuperscriptBox["k", "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"k", "-", "p1", "+", "p2"}], ")"}], "2"]}]], "-", 
   FractionBox[
    RowBox[{"2", " ", 
     RowBox[{
      FormBox["k",
       TraditionalForm], "\[CenterDot]", 
      FormBox["\[CapitalDelta]",
       TraditionalForm]}], " ", 
     RowBox[{
      FormBox["\[CapitalDelta]",
       TraditionalForm], "\[CenterDot]", 
      FormBox["p1",
       TraditionalForm]}]}], 
    RowBox[{
     SuperscriptBox["k", "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"k", "-", "p1", "+", "p2"}], ")"}], "2"]}]], "+", 
   FractionBox[
    SuperscriptBox[
     FormBox[
      RowBox[{
       FormBox["\[CapitalDelta]",
        TraditionalForm], "\[CenterDot]", 
       FormBox["p1",
        TraditionalForm]}],
      TraditionalForm], "2"], 
    RowBox[{
     SuperscriptBox["k", "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"k", "-", "p1", "+", "p2"}], ")"}], "2"]}]]}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970236406466*^9},
 CellTags->"FeynAmpDenominatorSimplify",
 CellLabel->"Out[664]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"Clear", "[", 
  RowBox[{"t", ",", "r"}], "]"}]], "Input",
 CellTags->"FeynAmpDenominatorSimplify",
 CellLabel->"In[665]:="]
}, Open  ]],

Cell["FeynAmpDenominatorSplit", "ObjectName",
 CellTags->"FeynAmpDenominatorSplit"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FeynAmpDenominatorSplit"],

Cell["\<\
FeynAmpDenominatorSplit[expr] splits all FeynAmpDenominator[a,b, ...] in expr \
into FeynAmpDenominator[a]*FeynAmpDenominator[b] ... . \
FeynAmpDenominatorSplit[expr, q1] splits all FeynAmpDenominator in expr into \
a product of two, one containing q1 and other momenta, the second without q1.\
\
\>", "Text",
 CellTags->"FeynAmpDenominatorSplit"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FeynAmpDenominatorCombine",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FeynAmpDenominatorCombine",
  ButtonNote->"FeynAmpDenominatorCombine"],
 "."
}], "Text",
 CellTags->"FeynAmpDenominatorSplit"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"FeynAmpDenominatorSplit"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t", "=", 
  RowBox[{
   RowBox[{"FAD", "[", 
    RowBox[{"q1", ",", 
     RowBox[{"q1", "-", "p"}], ",", 
     RowBox[{"q1", "-", "q2"}], ",", "q2", ",", 
     RowBox[{"q2", "-", "p"}]}], "]"}], "//", "FCI"}]}]], "Input",
 CellTags->"FeynAmpDenominatorSplit",
 CellLabel->"In[666]:="],

Cell[BoxData[
 FormBox[
  FractionBox["1", 
   RowBox[{
    SuperscriptBox["q1", "2"], ".", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"q1", "-", "p"}], ")"}], "2"], ".", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"q1", "-", "q2"}], ")"}], "2"], ".", 
    SuperscriptBox["q2", "2"], ".", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"q2", "-", "p"}], ")"}], "2"]}]], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009702368839483`*^9},
 CellTags->"FeynAmpDenominatorSplit",
 CellLabel->"Out[666]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t", "//", "Head"}]], "Input",
 CellTags->"FeynAmpDenominatorSplit",
 CellLabel->"In[667]:="],

Cell[BoxData[
 FormBox["FeynAmpDenominator", TraditionalForm]], "Output",
 CellChangeTimes->{3.500970236963051*^9},
 CellTags->"FeynAmpDenominatorSplit",
 CellLabel->"Out[667]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FeynAmpDenominatorSplit", "[", "t", "]"}]], "Input",
 CellTags->"FeynAmpDenominatorSplit",
 CellLabel->"In[668]:="],

Cell[BoxData[
 FormBox[
  FractionBox["1", 
   RowBox[{
    SuperscriptBox["q1", "2"], " ", 
    SuperscriptBox["q2", "2"], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"q1", "-", "p"}], ")"}], "2"], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"q2", "-", "p"}], ")"}], "2"], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"q1", "-", "q2"}], ")"}], "2"]}]], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970237069602*^9},
 CellTags->"FeynAmpDenominatorSplit",
 CellLabel->"Out[668]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"%", "//", "FCE"}], "//", "StandardForm"}]], "Input",
 CellTags->"FeynAmpDenominatorSplit",
 CellLabel->"In[669]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"FAD", "[", "q1", "]"}], " ", 
  RowBox[{"FAD", "[", 
   RowBox[{
    RowBox[{"-", "p"}], "+", "q1"}], "]"}], " ", 
  RowBox[{"FAD", "[", 
   RowBox[{"q1", "-", "q2"}], "]"}], " ", 
  RowBox[{"FAD", "[", "q2", "]"}], " ", 
  RowBox[{"FAD", "[", 
   RowBox[{
    RowBox[{"-", "p"}], "+", "q2"}], "]"}]}]], "Output",
 CellChangeTimes->{3.5009702371468887`*^9},
 CellTags->"FeynAmpDenominatorSplit",
 CellLabel->"Out[669]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"FeynAmpDenominatorSplit", "[", 
    RowBox[{"t", ",", "q1"}], "]"}], "//", "FCE"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"FeynAmpDenominatorSplit",
 CellLabel->"In[670]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"FAD", "[", 
   RowBox[{"q2", ",", 
    RowBox[{
     RowBox[{"-", "p"}], "+", "q2"}]}], "]"}], " ", 
  RowBox[{"FAD", "[", 
   RowBox[{"q1", ",", 
    RowBox[{
     RowBox[{"-", "p"}], "+", "q1"}], ",", 
    RowBox[{"q1", "-", "q2"}]}], "]"}]}]], "Output",
 CellChangeTimes->{3.500970237252797*^9},
 CellTags->"FeynAmpDenominatorSplit",
 CellLabel->"Out[670]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"FeynAmpDenominatorCombine", "[", "%", "]"}], "//", "FCE"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"FeynAmpDenominatorSplit",
 CellLabel->"In[671]:="],

Cell[BoxData[
 RowBox[{"FAD", "[", 
  RowBox[{"q1", ",", 
   RowBox[{
    RowBox[{"-", "p"}], "+", "q1"}], ",", 
   RowBox[{"q1", "-", "q2"}], ",", "q2", ",", 
   RowBox[{
    RowBox[{"-", "p"}], "+", "q2"}]}], "]"}]], "Output",
 CellChangeTimes->{3.500970237404436*^9},
 CellTags->"FeynAmpDenominatorSplit",
 CellLabel->"Out[671]//StandardForm="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"Clear", "[", "t", "]"}]], "Input",
 CellTags->"FeynAmpDenominatorSplit",
 CellLabel->"In[672]:="]
}, Open  ]],

Cell["FeynAmpList", "ObjectName",
 CellTags->"FeynAmpList"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FeynAmpList"],

Cell["\<\
FeynAmpList[info][FeynAmp[...], FeynAmp[...], ...] is a head of a list of \
Feynman amplitudes.\
\>", "Text",
 CellTags->"FeynAmpList"],

Cell["\<\
FeynAmpList has no functional properties and serves just as a head.\
\>", "Text",
 CellTags->"FeynAmpList"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FeynAmp",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FeynAmp",
  ButtonNote->"FeynAmp"],
 "."
}], "Text",
 CellTags->"FeynAmpList"]
}, Open  ]],

Cell["FeynCalc", "ObjectName",
 CellTags->"FeynCalc"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FeynCalc"],

Cell["FeynCalc is simply a symbol with a usage definition.", "Text",
 CellTags->"FeynCalc"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "FeynCalc"}]], "Input",
 CellTags->"FeynCalc",
 CellLabel->"In[673]:="],

Cell[BoxData[
 StyleBox["\<\"For installation notes visit www.feyncalc.org\\n\\nFor a list \
of availabe objects type $FeynCalcStuff, which contains a list of all \
functions and options in StringForm. You can get on-line information by \
?function, e.g. ?Contract.\\n\\nThere are several useful functions for short \
input, type $FCS for a list of short commands. Then type, e.g., \
?GA.\\n\\n\\nTo enable/disable start-up messages, put the \
line\\n\\n$FeynCalcStartupMessages = \
True;\\n\\nor\\n\\n$FeynCalcStartupMessages = False;\\n\\n into your \
\\\"init.m\\\" file or into your \\\"FCConfig.m\\\" file.\"\>", 
  "MSG"]], "Print", "PrintUsage",
 CellChangeTimes->{3.500970238699658*^9},
 CellTags->{"FeynCalc", "Info3500973837-4982097"}]
}, Open  ]]
}, Open  ]],

Cell["FeynCalcExternal", "ObjectName",
 CellTags->"FeynCalcExternal"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FeynCalcExternal"],

Cell["\<\
FeynCalcExternal[exp] translates exp from the internal FeynCalc \
representation to a shorthand form.\
\>", "Text",
 CellTags->"FeynCalcExternal"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FeynCalcInternal",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FeynCalcInternal",
  ButtonNote->"FeynCalcInternal"],
 "."
}], "Text",
 CellTags->"FeynCalcExternal"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"FeynCalcExternal"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FeynCalcExternal", "[", 
  RowBox[{"DiracGamma", "[", "5", "]"}], "]"}]], "Input",
 CellTags->"FeynCalcExternal",
 CellLabel->"In[674]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["\[Gamma]", "5"], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970239062496*^9},
 CellTags->"FeynCalcExternal",
 CellLabel->"Out[674]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "StandardForm"}]], "Input",
 CellTags->"FeynCalcExternal",
 CellLabel->"In[675]:="],

Cell[BoxData[
 RowBox[{"GA", "[", "5", "]"}]], "Output",
 CellChangeTimes->{3.500970239103326*^9},
 CellTags->"FeynCalcExternal",
 CellLabel->"Out[675]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"GA", "[", "\[Mu]", "]"}], ",", 
   RowBox[{"GAD", "[", "\[Rho]", "]"}], ",", 
   RowBox[{"GS", "[", "p", "]"}], ",", 
   RowBox[{"SP", "[", 
    RowBox[{"p", ",", "q"}], "]"}], ",", 
   RowBox[{"MT", "[", 
    RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}], ",", 
   RowBox[{"FV", "[", 
    RowBox[{"p", ",", "\[Mu]"}], "]"}]}], "}"}]], "Input",
 CellTags->"FeynCalcExternal",
 CellLabel->"In[676]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    SuperscriptBox["\[Gamma]", "\[Mu]"], ",", 
    SuperscriptBox["\[Gamma]", "\[Rho]"], ",", 
    RowBox[{"\[Gamma]", "\[CenterDot]", "p"}], ",", 
    FormBox[
     RowBox[{
      FormBox["p",
       TraditionalForm], "\[CenterDot]", 
      FormBox["q",
       TraditionalForm]}],
     TraditionalForm], ",", 
    SuperscriptBox["g", 
     RowBox[{"\[Alpha]", "\[Beta]"}]], ",", 
    SuperscriptBox["p", "\[Mu]"]}], "}"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970239170945*^9},
 CellTags->"FeynCalcExternal",
 CellLabel->"Out[676]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "StandardForm"}]], "Input",
 CellTags->"FeynCalcExternal",
 CellLabel->"In[677]:="],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"GA", "[", "\[Mu]", "]"}], ",", 
   RowBox[{"GAD", "[", "\[Rho]", "]"}], ",", 
   RowBox[{"GS", "[", "p", "]"}], ",", 
   RowBox[{"SP", "[", 
    RowBox[{"p", ",", "q"}], "]"}], ",", 
   RowBox[{"MT", "[", 
    RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}], ",", 
   RowBox[{"FV", "[", 
    RowBox[{"p", ",", "\[Mu]"}], "]"}]}], "}"}]], "Output",
 CellChangeTimes->{3.500970239301091*^9},
 CellTags->"FeynCalcExternal",
 CellLabel->"Out[677]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "FeynCalcInternal"}]], "Input",
 CellTags->"FeynCalcExternal",
 CellLabel->"In[678]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], ",", 
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox["\[Rho]",
       TraditionalForm],
      TraditionalForm]], ",", 
    RowBox[{
     FormBox["\<\"\[Gamma]\"\>",
      TraditionalForm], "\[CenterDot]", 
     FormBox["p",
      TraditionalForm]}], ",", 
    RowBox[{
     FormBox["p",
      TraditionalForm], "\[CenterDot]", 
     FormBox["q",
      TraditionalForm]}], ",", 
    SuperscriptBox["g", 
     RowBox[{"\[Alpha]", "\[Beta]"}]], ",", 
    SuperscriptBox[
     FormBox["p",
      TraditionalForm], 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]]}], "}"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970239384769*^9},
 CellTags->"FeynCalcExternal",
 CellLabel->"Out[678]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "StandardForm"}]], "Input",
 CellTags->"FeynCalcExternal",
 CellLabel->"In[679]:="],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"DiracGamma", "[", 
    RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], "]"}], ",", 
   RowBox[{"DiracGamma", "[", 
    RowBox[{
     RowBox[{"LorentzIndex", "[", 
      RowBox[{"\[Rho]", ",", "D"}], "]"}], ",", "D"}], "]"}], ",", 
   RowBox[{"DiracGamma", "[", 
    RowBox[{"Momentum", "[", "p", "]"}], "]"}], ",", 
   RowBox[{"Pair", "[", 
    RowBox[{
     RowBox[{"Momentum", "[", "p", "]"}], ",", 
     RowBox[{"Momentum", "[", "q", "]"}]}], "]"}], ",", 
   RowBox[{"Pair", "[", 
    RowBox[{
     RowBox[{"LorentzIndex", "[", "\[Alpha]", "]"}], ",", 
     RowBox[{"LorentzIndex", "[", "\[Beta]", "]"}]}], "]"}], ",", 
   RowBox[{"Pair", "[", 
    RowBox[{
     RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
     RowBox[{"Momentum", "[", "p", "]"}]}], "]"}]}], "}"}]], "Output",
 CellChangeTimes->{3.5009702394665956`*^9},
 CellTags->"FeynCalcExternal",
 CellLabel->"Out[679]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"FeynCalcExternal", "[", "%", "]"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"FeynCalcExternal",
 CellLabel->"In[680]:="],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"GA", "[", "\[Mu]", "]"}], ",", 
   RowBox[{"GAD", "[", "\[Rho]", "]"}], ",", 
   RowBox[{"GS", "[", "p", "]"}], ",", 
   RowBox[{"SP", "[", 
    RowBox[{"p", ",", "q"}], "]"}], ",", 
   RowBox[{"MT", "[", 
    RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}], ",", 
   RowBox[{"FV", "[", 
    RowBox[{"p", ",", "\[Mu]"}], "]"}]}], "}"}]], "Output",
 CellChangeTimes->{3.500970239545916*^9},
 CellTags->"FeynCalcExternal",
 CellLabel->"Out[680]//StandardForm="]
}, Open  ]]
}, Open  ]],

Cell["FeynCalcForm", "ObjectName",
 CellTags->"FeynCalcForm"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FeynCalcForm"],

Cell[TextData[{
 "FeynCalcForm[expr] changes the printed output to a an easy-to-read form. It \
allows a readable output also when running a terminal based ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " session. Whether the result of FeynCalcForm[expr] is displayed or not, \
depends on the setting of $PrePrint. $PrePrint = FeynCalcForm forces \
displaying everything after applying FeynCalcForm. In order to change to the \
normal (internal) Mathematica OutputForm, do: ($PrePrint=.)."
}], "Text",
 CellTags->"FeynCalcForm"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FC",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FC",
  ButtonNote->"FC"],
 ", ",
 ButtonBox["FeynCalcExternal",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FeynCalcExternal",
  ButtonNote->"FeynCalcExternal"],
 ", ",
 ButtonBox["FeynCalcInternal",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FeynCalcInternal",
  ButtonNote->"FeynCalcInternal"],
 "."
}], "Text",
 CellTags->"FeynCalcForm"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"FeynCalcForm"],

Cell["This is the normal notebook display:", "Text",
 CellTags->"FeynCalcForm"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNTrace", "[", 
  RowBox[{
   RowBox[{"SUNT", "[", "a", "]"}], ".", 
   RowBox[{"SUNT", "[", "b", "]"}], ".", 
   RowBox[{"SUNT", "[", "c", "]"}]}], "]"}]], "Input",
 CellTags->"FeynCalcForm",
 CellLabel->"In[681]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"tr", "(", 
   FormBox[
    RowBox[{
     SubscriptBox["T", 
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm]], ".", 
     SubscriptBox["T", 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]], ".", 
     SubscriptBox["T", 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm]]}],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009702396363907`*^9},
 CellTags->"FeynCalcForm",
 CellLabel->"Out[681]="]
}, Open  ]],

Cell["\<\
This is the shorthand (terminal) display (easy-to-read form):\
\>", "Text",
 CellTags->"FeynCalcForm"],

Cell[BoxData[
 RowBox[{
  RowBox[{"$PrePrint", " ", "=", " ", "FeynCalcForm"}], ";"}]], "Input",
 CellTags->"FeynCalcForm",
 CellLabel->"In[682]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"SetOptions", "[", 
   RowBox[{"$FrontEnd", ",", 
    RowBox[{"Evaluate", "[", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"Options", "[", 
         RowBox[{"$FrontEnd", ",", "\"\<CommonDefaultFormatTypes\>\""}], 
         "]"}], "/.", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{"\"\<Output\>\"", "\[Rule]", "_"}], ")"}], "\[Rule]", 
         RowBox[{"(", 
          RowBox[{"\"\<Output\>\"", "\[Rule]", "OutputForm"}], ")"}]}]}], 
       ")"}], "[", 
      RowBox[{"[", "1", "]"}], "]"}], "]"}]}], "]"}], ";"}]], "Input",
 CellTags->"FeynCalcForm",
 CellLabel->"In[683]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNTrace", "[", 
  RowBox[{
   RowBox[{"SUNT", "[", "a", "]"}], ".", 
   RowBox[{"SUNT", "[", "b", "]"}], ".", 
   RowBox[{"SUNT", "[", "c", "]"}]}], "]"}]], "Input",
 CellTags->"FeynCalcForm",
 CellLabel->"In[684]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"\<\"tr\"\>", "(", 
   FormBox[
    InterpretationBox[
     RowBox[{
      RowBox[{"\<\"T\"\>", "(", "a", ")"}], "\<\" \"\>", 
      RowBox[{"\<\"T\"\>", "(", "b", ")"}], "\<\" \"\>", 
      RowBox[{"\<\"T\"\>", "(", "c", ")"}]}],
     Infix[
      HighEnergyPhysics`FeynCalc`FeynCalcForm`Private`fcdot2[
       "T"[$CellContext`a], 
       "T"[$CellContext`b], 
       "T"[$CellContext`c]], " "],
     Editable->False],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970239858429*^9},
 CellTags->"FeynCalcForm",
 CellLabel->"Out[684]="]
}, Open  ]],

Cell["Reset to normal notebook display:", "Text",
 CellTags->"FeynCalcForm"],

Cell[BoxData[
 RowBox[{
  RowBox[{"$PrePrint", "=."}], ";"}]], "Input",
 CellTags->"FeynCalcForm",
 CellLabel->"In[685]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"SetOptions", "[", 
   RowBox[{"$FrontEnd", ",", 
    RowBox[{"Evaluate", "[", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"Options", "[", 
         RowBox[{"$FrontEnd", ",", "\"\<CommonDefaultFormatTypes\>\""}], 
         "]"}], "/.", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{"\"\<Output\>\"", "\[Rule]", "_"}], ")"}], "\[Rule]", 
         RowBox[{"(", 
          RowBox[{"\"\<Output\>\"", "\[Rule]", "TraditionalForm"}], ")"}]}]}],
        ")"}], "[", 
      RowBox[{"[", "1", "]"}], "]"}], "]"}]}], "]"}], ";"}]], "Input",
 CellTags->"FeynCalcForm",
 CellLabel->"In[686]:="]
}, Open  ]],

Cell["FeynCalcInternal", "ObjectName",
 CellTags->"FeynCalcInternal"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FeynCalcInternal"],

Cell["\<\
FeynCalcInternal[exp] translates exp into the internal FeynCalc (abstract \
data-type) representation.\
\>", "Text",
 CellTags->"FeynCalcInternal"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FeynCalcExternal",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FeynCalcExternal",
  ButtonNote->"FeynCalcExternal"],
 ", ",
 ButtonBox["FCI",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FCI",
  ButtonNote->"FCI"],
 ", ",
 ButtonBox["FCE",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FCE",
  ButtonNote->"FCE"],
 "."
}], "Text",
 CellTags->"FeynCalcInternal"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"FeynCalcInternal"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"GA", "[", "\[Mu]", "]"}], ",", 
   RowBox[{"GAD", "[", "\[Rho]", "]"}], ",", 
   RowBox[{"GS", "[", "p", "]"}], ",", 
   RowBox[{"SP", "[", 
    RowBox[{"p", ",", "q"}], "]"}], ",", 
   RowBox[{"MT", "[", 
    RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}], ",", 
   RowBox[{"FV", "[", 
    RowBox[{"p", ",", "\[Mu]"}], "]"}]}], "}"}]], "Input",
 CellTags->"FeynCalcInternal",
 CellLabel->"In[687]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    SuperscriptBox["\[Gamma]", "\[Mu]"], ",", 
    SuperscriptBox["\[Gamma]", "\[Rho]"], ",", 
    RowBox[{"\[Gamma]", "\[CenterDot]", "p"}], ",", 
    FormBox[
     RowBox[{
      FormBox["p",
       TraditionalForm], "\[CenterDot]", 
      FormBox["q",
       TraditionalForm]}],
     TraditionalForm], ",", 
    SuperscriptBox["g", 
     RowBox[{"\[Alpha]", "\[Beta]"}]], ",", 
    SuperscriptBox["p", "\[Mu]"]}], "}"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.50097023999448*^9},
 CellTags->"FeynCalcInternal",
 CellLabel->"Out[687]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "StandardForm"}]], "Input",
 CellTags->"FeynCalcInternal",
 CellLabel->"In[688]:="],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"GA", "[", "\[Mu]", "]"}], ",", 
   RowBox[{"GAD", "[", "\[Rho]", "]"}], ",", 
   RowBox[{"GS", "[", "p", "]"}], ",", 
   RowBox[{"SP", "[", 
    RowBox[{"p", ",", "q"}], "]"}], ",", 
   RowBox[{"MT", "[", 
    RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}], ",", 
   RowBox[{"FV", "[", 
    RowBox[{"p", ",", "\[Mu]"}], "]"}]}], "}"}]], "Output",
 CellChangeTimes->{3.500970240197652*^9},
 CellTags->"FeynCalcInternal",
 CellLabel->"Out[688]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "FeynCalcInternal"}]], "Input",
 CellTags->"FeynCalcInternal",
 CellLabel->"In[689]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], ",", 
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox["\[Rho]",
       TraditionalForm],
      TraditionalForm]], ",", 
    RowBox[{
     FormBox["\<\"\[Gamma]\"\>",
      TraditionalForm], "\[CenterDot]", 
     FormBox["p",
      TraditionalForm]}], ",", 
    RowBox[{
     FormBox["p",
      TraditionalForm], "\[CenterDot]", 
     FormBox["q",
      TraditionalForm]}], ",", 
    SuperscriptBox["g", 
     RowBox[{"\[Alpha]", "\[Beta]"}]], ",", 
    SuperscriptBox[
     FormBox["p",
      TraditionalForm], 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]]}], "}"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009702403001623`*^9},
 CellTags->"FeynCalcInternal",
 CellLabel->"Out[689]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "StandardForm"}]], "Input",
 CellTags->"FeynCalcInternal",
 CellLabel->"In[690]:="],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"DiracGamma", "[", 
    RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], "]"}], ",", 
   RowBox[{"DiracGamma", "[", 
    RowBox[{
     RowBox[{"LorentzIndex", "[", 
      RowBox[{"\[Rho]", ",", "D"}], "]"}], ",", "D"}], "]"}], ",", 
   RowBox[{"DiracGamma", "[", 
    RowBox[{"Momentum", "[", "p", "]"}], "]"}], ",", 
   RowBox[{"Pair", "[", 
    RowBox[{
     RowBox[{"Momentum", "[", "p", "]"}], ",", 
     RowBox[{"Momentum", "[", "q", "]"}]}], "]"}], ",", 
   RowBox[{"Pair", "[", 
    RowBox[{
     RowBox[{"LorentzIndex", "[", "\[Alpha]", "]"}], ",", 
     RowBox[{"LorentzIndex", "[", "\[Beta]", "]"}]}], "]"}], ",", 
   RowBox[{"Pair", "[", 
    RowBox[{
     RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
     RowBox[{"Momentum", "[", "p", "]"}]}], "]"}]}], "}"}]], "Output",
 CellChangeTimes->{3.500970240376218*^9},
 CellTags->"FeynCalcInternal",
 CellLabel->"Out[690]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"FeynCalcExternal", "[", "%", "]"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"FeynCalcInternal",
 CellLabel->"In[691]:="],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"GA", "[", "\[Mu]", "]"}], ",", 
   RowBox[{"GAD", "[", "\[Rho]", "]"}], ",", 
   RowBox[{"GS", "[", "p", "]"}], ",", 
   RowBox[{"SP", "[", 
    RowBox[{"p", ",", "q"}], "]"}], ",", 
   RowBox[{"MT", "[", 
    RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}], ",", 
   RowBox[{"FV", "[", 
    RowBox[{"p", ",", "\[Mu]"}], "]"}]}], "}"}]], "Output",
 CellChangeTimes->{3.500970240538437*^9},
 CellTags->"FeynCalcInternal",
 CellLabel->"Out[691]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FCI", "[", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"SD", "[", 
     RowBox[{"a", ",", "b"}], "]"}], ",", 
    RowBox[{"SUND", "[", 
     RowBox[{"a", ",", "b", ",", "c"}], "]"}], ",", 
    RowBox[{"SUNF", "[", 
     RowBox[{"a", ",", "b", ",", "c"}], "]"}], ",", 
    RowBox[{"FAD", "[", "q", "]"}], ",", 
    RowBox[{"LC", "[", 
     RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]", ",", "\[Sigma]"}], "]"}]}],
    "}"}], "]"}]], "Input",
 CellTags->"FeynCalcInternal",
 CellLabel->"In[692]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]}]], ",", 
    SubscriptBox["d", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm]}]], ",", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm]}]], ",", 
    FractionBox["1", 
     SuperscriptBox["q", "2"]], ",", 
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Rho]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm]}]]}], "}"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009702406455307`*^9},
 CellTags->"FeynCalcInternal",
 CellLabel->"Out[692]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "StandardForm"}]], "Input",
 CellTags->"FeynCalcInternal",
 CellLabel->"In[693]:="],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"SUNDelta", "[", 
    RowBox[{
     RowBox[{"SUNIndex", "[", "a", "]"}], ",", 
     RowBox[{"SUNIndex", "[", "b", "]"}]}], "]"}], ",", 
   RowBox[{"SUND", "[", 
    RowBox[{
     RowBox[{"SUNIndex", "[", "a", "]"}], ",", 
     RowBox[{"SUNIndex", "[", "b", "]"}], ",", 
     RowBox[{"SUNIndex", "[", "c", "]"}]}], "]"}], ",", 
   RowBox[{"SUNF", "[", 
    RowBox[{
     RowBox[{"SUNIndex", "[", "a", "]"}], ",", 
     RowBox[{"SUNIndex", "[", "b", "]"}], ",", 
     RowBox[{"SUNIndex", "[", "c", "]"}]}], "]"}], ",", 
   RowBox[{"FeynAmpDenominator", "[", 
    RowBox[{"PropagatorDenominator", "[", 
     RowBox[{
      RowBox[{"Momentum", "[", 
       RowBox[{"q", ",", "D"}], "]"}], ",", "0"}], "]"}], "]"}], ",", 
   RowBox[{"Eps", "[", 
    RowBox[{
     RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
     RowBox[{"LorentzIndex", "[", "\[Nu]", "]"}], ",", 
     RowBox[{"LorentzIndex", "[", "\[Rho]", "]"}], ",", 
     RowBox[{"LorentzIndex", "[", "\[Sigma]", "]"}]}], "]"}]}], 
  "}"}]], "Output",
 CellChangeTimes->{3.5009702407411137`*^9},
 CellTags->"FeynCalcInternal",
 CellLabel->"Out[693]//StandardForm="]
}, Open  ]]
}, Open  ]],

Cell["FeynCalc2FORM", "ObjectName",
 CellTags->"FeynCalc2FORM"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FeynCalc2FORM"],

Cell["\<\
FeynCalc2FORM[expr] displays expr in FORM syntax. FeynCalc2FORM[file, x] \
writes x in FORM syntax to a file. FeynCalc2FORM[file, x==y] writes x=y to a \
file in FORM syntax.\
\>", "Text",
 CellTags->"FeynCalc2FORM"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FORM2FeynCalc",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FORM2FeynCalc",
  ButtonNote->"FORM2FeynCalc"],
 "."
}], "Text",
 CellTags->"FeynCalc2FORM"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "FeynCalc2FORM", "]"}]], "Input",
 CellTags->"FeynCalc2FORM",
 CellLabel->"In[694]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"EpsDiscard", "\[Rule]", "False"}], ",", 
    RowBox[{"FORMEpilog", "\[Rule]", "\<\"\"\>"}], ",", 
    RowBox[{"FORMProlog", "\[Rule]", "\<\"write statistics;\"\>"}], ",", 
    RowBox[{"Replace", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"\<\"\\\\[Alpha]\"\>", "\[Rule]", "\<\"al\"\>"}], ",", 
       RowBox[{"\<\"\\\\[Beta]\"\>", "\[Rule]", "\<\"be\"\>"}], ",", 
       RowBox[{"\<\"\\\\[Gamma]\"\>", "\[Rule]", "\<\"ga\"\>"}], ",", 
       RowBox[{"\<\"\\\\[Delta]\"\>", "\[Rule]", "\<\"de\"\>"}], ",", 
       RowBox[{"\<\"\\\\[Mu]\"\>", "\[Rule]", "\<\"mu\"\>"}], ",", 
       RowBox[{"\<\"\\\\[Nu]\"\>", "\[Rule]", "\<\"nu\"\>"}], ",", 
       RowBox[{"\<\"\\\\[Rho]\"\>", "\[Rule]", "\<\"ro\"\>"}], ",", 
       RowBox[{"\<\"\\\\[Sigma]\"\>", "\[Rule]", "\<\"si\"\>"}]}], "}"}]}], 
    ",", 
    RowBox[{"TraceDimension", "\[Rule]", "4"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.50097024079469*^9},
 CellTags->"FeynCalc2FORM",
 CellLabel->"Out[694]="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"FeynCalc2FORM"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"MT", "[", 
   RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}], 
  RowBox[{"FV", "[", 
   RowBox[{"p", ",", "\[Rho]"}], "]"}], " ", 
  RowBox[{
   RowBox[{"y", "^", "2"}], "/", "d"}]}]], "Input",
 CellTags->"FeynCalc2FORM",
 CellLabel->"In[695]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{
    SuperscriptBox["y", "2"], " ", 
    SuperscriptBox["p", "\[Rho]"], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Nu]"}]]}], "d"], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009702408914623`*^9},
 CellTags->"FeynCalc2FORM",
 CellLabel->"Out[695]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"FeynCalc2FORM", "[", "%", "]"}], ";"}]], "Input",
 CellTags->"FeynCalc2FORM",
 CellLabel->"In[696]:="],

Cell["(y^2*d_(mu,nu)*p(ro))/d", "Print",
 CellChangeTimes->{3.5009702410057783`*^9},
 CellTags->"FeynCalc2FORM"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{" ", 
  RowBox[{"LC", "[", 
   RowBox[{"\[Alpha]", ",", "\[Beta]", ",", "\[Delta]", ",", "\[Rho]"}], 
   "]"}]}]], "Input",
 CellTags->"FeynCalc2FORM",
 CellLabel->"In[697]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["\[Epsilon]", 
   RowBox[{"\[Alpha]", "\[Beta]", "\[Delta]", "\[Rho]"}]], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970241122656*^9},
 CellTags->"FeynCalc2FORM",
 CellLabel->"Out[697]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"FeynCalc2FORM", "[", "%", "]"}], ";"}]], "Input",
 CellTags->"FeynCalc2FORM",
 CellLabel->"In[698]:="],

Cell["(-i_)*e_(al,be,de,ro)", "Print",
 CellChangeTimes->{3.5009702412327147`*^9},
 CellTags->"FeynCalc2FORM"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracTrace", "[", 
  RowBox[{"GA", "[", 
   RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]", ",", "\[Sigma]"}], "]"}], 
  "]"}]], "Input",
 CellTags->"FeynCalc2FORM",
 CellLabel->"In[699]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"tr", "(", 
   FormBox[
    RowBox[{
     SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
     SuperscriptBox["\[Gamma]", "\[Nu]"], ".", 
     SuperscriptBox["\[Gamma]", "\[Rho]"], ".", 
     SuperscriptBox["\[Gamma]", "\[Sigma]"]}],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970241332633*^9},
 CellTags->"FeynCalc2FORM",
 CellLabel->"Out[699]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"FeynCalc2FORM", "[", "%", "]"}], ";"}]], "Input",
 CellTags->"FeynCalc2FORM",
 CellLabel->"In[700]:="],

Cell["\<\
g_(0,mu)*g_(0,nu)*g_(0,ro)*g_(0,si)\
\>", "Print",
 CellChangeTimes->{3.5009702414466677`*^9},
 CellTags->"FeynCalc2FORM"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"DiracTrace", "[", 
   RowBox[{"GA", "[", 
    RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}], "]"}], 
  RowBox[{"DiracTrace", "[", 
   RowBox[{"GA", "[", 
    RowBox[{"\[Mu]", ",", "\[Rho]"}], "]"}], "]"}]}]], "Input",
 CellTags->"FeynCalc2FORM",
 CellLabel->"In[701]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"tr", "(", 
    FormBox[
     RowBox[{
      SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
      SuperscriptBox["\[Gamma]", "\[Nu]"]}],
     TraditionalForm], ")"}], " ", 
   RowBox[{"tr", "(", 
    FormBox[
     RowBox[{
      SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
      SuperscriptBox["\[Gamma]", "\[Rho]"]}],
     TraditionalForm], ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009702415575542`*^9},
 CellTags->"FeynCalc2FORM",
 CellLabel->"Out[701]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"FeynCalc2FORM", "[", "%", "]"}], ";"}]], "Input",
 CellTags->"FeynCalc2FORM",
 CellLabel->"In[702]:="],

Cell["\<\
g_(0,mu)*g_(0,nu)*g_(1,mu)*g_(1,ro)\
\>", "Print",
 CellChangeTimes->{3.500970241680715*^9},
 CellTags->"FeynCalc2FORM"]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"FeynCalc2FORM", "[", 
   RowBox[{"\"\<fc2ftest.f\>\"", ",", 
    RowBox[{
     RowBox[{"MT", "[", 
      RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}], 
     RowBox[{"FV", "[", 
      RowBox[{"p", ",", "\[Mu]"}], "]"}]}]}], "]"}], ";"}]], "Input",
 CellTags->"FeynCalc2FORM",
 CellLabel->"In[703]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ReadList", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"If", "[", 
     RowBox[{
      RowBox[{"$OperatingSystem", "===", "\"\<MacOS\>\""}], ",", "\"\<:\>\"", 
      ",", "\"\<\>\""}], "]"}], "<>", "\"\<fc2ftest.f\>\""}], ",", "String"}],
   "]"}]], "Input",
 CellTags->"FeynCalc2FORM",
 CellLabel->"In[704]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", "\<\"d_(mu,nu)*p(mu)\"\>", "}"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970241807603*^9},
 CellTags->"FeynCalc2FORM",
 CellLabel->"Out[704]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t", "=", 
  RowBox[{"TR", "[", 
   RowBox[{
    RowBox[{"GA", "[", 
     RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]", ",", "\[Sigma]"}], "]"}], 
    ".", 
    RowBox[{"GS", "[", 
     RowBox[{"p", ",", "q"}], "]"}]}], "]"}]}]], "Input",
 CellTags->"FeynCalc2FORM",
 CellLabel->"In[705]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"4", " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox[
       FormBox["q",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Rho]", "\[Sigma]"}]]}], "-", 
     RowBox[{
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox[
       FormBox["q",
        TraditionalForm], 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Rho]", "\[Sigma]"}]]}], "-", 
     RowBox[{
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], 
       FormBox[
        FormBox["\[Rho]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox[
       FormBox["q",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Nu]", "\[Sigma]"}]]}], "+", 
     RowBox[{
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], 
       FormBox[
        FormBox["\[Rho]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox[
       FormBox["q",
        TraditionalForm], 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Sigma]"}]]}], "+", 
     RowBox[{
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox[
       FormBox["q",
        TraditionalForm], 
       FormBox[
        FormBox["\[Rho]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Nu]", "\[Sigma]"}]]}], "-", 
     RowBox[{
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox[
       FormBox["q",
        TraditionalForm], 
       FormBox[
        FormBox["\[Rho]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Sigma]"}]]}], "+", 
     RowBox[{
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], 
       FormBox[
        FormBox["\[Sigma]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox[
       FormBox["q",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Nu]", "\[Rho]"}]]}], "-", 
     RowBox[{
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], 
       FormBox[
        FormBox["\[Sigma]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox[
       FormBox["q",
        TraditionalForm], 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Rho]"}]]}], "+", 
     RowBox[{
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], 
       FormBox[
        FormBox["\[Sigma]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox[
       FormBox["q",
        TraditionalForm], 
       FormBox[
        FormBox["\[Rho]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Nu]"}]]}], "-", 
     RowBox[{
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox[
       FormBox["q",
        TraditionalForm], 
       FormBox[
        FormBox["\[Sigma]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Nu]", "\[Rho]"}]]}], "+", 
     RowBox[{
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox[
       FormBox["q",
        TraditionalForm], 
       FormBox[
        FormBox["\[Sigma]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Rho]"}]]}], "-", 
     RowBox[{
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], 
       FormBox[
        FormBox["\[Rho]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox[
       FormBox["q",
        TraditionalForm], 
       FormBox[
        FormBox["\[Sigma]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Nu]"}]]}], "+", 
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Rho]", "\[Sigma]"}]], " ", 
      RowBox[{
       FormBox["p",
        TraditionalForm], "\[CenterDot]", 
       FormBox["q",
        TraditionalForm]}]}], "+", 
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Sigma]"}]], " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Nu]", "\[Rho]"}]], " ", 
      RowBox[{
       FormBox["p",
        TraditionalForm], "\[CenterDot]", 
       FormBox["q",
        TraditionalForm]}]}], "-", 
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Rho]"}]], " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Nu]", "\[Sigma]"}]], " ", 
      RowBox[{
       FormBox["p",
        TraditionalForm], "\[CenterDot]", 
       FormBox["q",
        TraditionalForm]}]}]}], ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970242026597*^9},
 CellTags->"FeynCalc2FORM",
 CellLabel->"Out[705]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"FeynCalc2FORM", "[", 
   RowBox[{"\"\<fc2ftest.f\>\"", ",", 
    RowBox[{"L", " ", "==", "t"}]}], "]"}], ";"}]], "Input",
 CellTags->"FeynCalc2FORM",
 CellLabel->"In[706]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TableForm", "[", 
  RowBox[{"ReadList", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"$OperatingSystem", "===", "\"\<MacOS\>\""}], ",", "\"\<:\>\"",
        ",", "\"\<\>\""}], "]"}], "<>", "\"\<fc2ftest.f\>\""}], ",", 
    "String"}], "]"}], "]"}]], "Input",
 CellTags->"FeynCalc2FORM",
 CellLabel->"In[707]:="],

Cell[BoxData[
 FormBox[
  TagBox[
   TagBox[GridBox[{
      {"\<\"Indices \\\\[Mu],\\\\[Nu],\\\\[Rho],\\\\[Sigma];\"\>"},
      {"\<\"Vectors p,q;\"\>"},
      {"\<\"write statistics;\"\>"},
      {"\<\"Local L = ( \"\>"},
      {"\<\"4*(d_(mu,si)*d_(nu,ro)*q.p-d_(mu,ro)*d_(nu,si)*q.p+d_(mu,nu)*d_(\
ro,si)*q.p+\"\>"},
      {"\<\"d_(ro,si)*p(nu)*q(mu)-d_(nu,si)*p(ro)*q(mu)+d_(nu,ro)*p(si)*q(mu)-\
\"\>"},
      {"\<\"d_(ro,si)*p(mu)*q(nu)+d_(mu,si)*p(ro)*q(nu)-d_(mu,ro)*p(si)*q(nu)+\
\"\>"},
      {"\<\"d_(nu,si)*p(mu)*q(ro)-d_(mu,si)*p(nu)*q(ro)+d_(mu,nu)*p(si)*q(ro)-\
\"\>"},
      {"\<\"d_(nu,ro)*p(mu)*q(si)+d_(mu,ro)*p(nu)*q(si)-d_(mu,nu)*p(ro)*q(si))\
 ); \"\>"},
      {"\<\"   \"\>"},
      {"\<\"Print; \"\>"},
      {"\<\".end\"\>"}
     },
     GridBoxAlignment->{
      "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
       "RowsIndexed" -> {}},
     GridBoxSpacings->{"Columns" -> {
         Offset[0.27999999999999997`], {
          Offset[0.5599999999999999]}, 
         Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
         Offset[0.2], {
          Offset[0.4]}, 
         Offset[0.2]}, "RowsIndexed" -> {}}],
    Column],
   Function[BoxForm`e$, 
    TableForm[BoxForm`e$]]], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009702427564793`*^9},
 CellTags->"FeynCalc2FORM",
 CellLabel->"Out[707]//TableForm="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"If", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"FileNames", "[", "\"\<fc2ftest.f\>\"", "]"}], "=!=", 
     RowBox[{"{", "}"}]}], ",", 
    RowBox[{"DeleteFile", "[", "\"\<fc2ftest.f\>\"", "]"}]}], "]"}], 
  ";"}]], "Input",
 CellTags->"FeynCalc2FORM",
 CellLabel->"In[708]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", "t", "]"}], ";"}]], "Input",
 CellTags->"FeynCalc2FORM",
 CellLabel->"In[709]:="]
}, Open  ]],

Cell["FeynCalcToLaTeX", "ObjectName",
 CellTags->"FeynCalcToLaTeX"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FeynCalcToLaTeX"],

Cell["\<\
FeynCalcToLaTeX[expr] generates LaTeX with line-breaking  for expr. 
FeynCalcToLaTeX[expr, 500] generates LaTeX for expr where 500 is the Window \
width  setting for the Mathematica frontend. Increasing its value will \
generate less line breaks.\
\>", "Text",
 CellTags->"FeynCalcToLaTeX"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"GluonPropagator", "[", 
   RowBox[{"p", ",", "1", ",", "2"}], "]"}], "//", "Explicit"}]], "Input",
 CellLabel->"In[710]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   FractionBox[
    RowBox[{"\[ImaginaryI]", " ", 
     SubscriptBox["\[Delta]", 
      RowBox[{
       FormBox[
        FormBox["ci1",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["ci2",
         TraditionalForm],
        TraditionalForm]}]], " ", 
     SuperscriptBox["g", 
      RowBox[{"li1", "li2"}]]}], 
    SuperscriptBox["p", "2"]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009702429940357`*^9},
 CellLabel->"Out[710]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FeynCalcToLaTeX", "[", "%", "]"}]], "Input",
 CellLabel->"In[711]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   StyleBox[
    RowBox[{"TeXForm", "::", "unspt"}], "MessageName"], 
   RowBox[{
   ":", " "}], "\<\"TeXForm of \
\[NoBreak]\\!\\(TraditionalForm\\`\\(FormBox[\\(\\(FormBox[\\\"ci1\\\"]\\)\\)]\
\\)\\)\[NoBreak] is not supported. \
\\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", \
ButtonFrame->None, ButtonData:>\\\"paclet:ref/TeXForm\\\", ButtonNote -> \
\\\"TeXForm::unspt\\\"]\\)\"\>"}], TraditionalForm]], "Message", "MSG",
 CellChangeTimes->{3.500970243743477*^9}],

Cell[BoxData[
 FormBox[
  RowBox[{
   StyleBox[
    RowBox[{"TeXForm", "::", "unspt"}], "MessageName"], 
   RowBox[{
   ":", " "}], "\<\"TeXForm of \
\[NoBreak]\\!\\(TraditionalForm\\`\\(FormBox[\\(\\(FormBox[\\\"ci2\\\"]\\)\\)]\
\\)\\)\[NoBreak] is not supported. \
\\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", \
ButtonFrame->None, ButtonData:>\\\"paclet:ref/TeXForm\\\", ButtonNote -> \
\\\"TeXForm::unspt\\\"]\\)\"\>"}], TraditionalForm]], "Message", "MSG",
 CellChangeTimes->{3.500970243777568*^9}],

Cell[BoxData[
 FormBox["\<\"%% AMS-LaTeX Created by Wolfram Mathematica 8.0 : \
www.wolfram.com\\\\documentclass{article}\\n\\\\usepackage{amsmath, amssymb, \
graphics, \
setspace}\\\\newcommand{\\\\mathsym}[1]{{}}\\n\\\\newcommand{\\\\unicode}[1]{{\
}}\\\\newcounter{mathematicapage}\\n\\\\begin{document}\\\\begin{doublespace}\
\\n\\\\noindent\\\\(\\\\(-\\\\frac{i \\\\delta _{} \
g^{\\\\text{li1}\\\\text{li2}}}{p^2}\\\\)\\\\)\\n\\\\end{doublespace}\"\>", 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970243843014*^9},
 CellLabel->"Out[711]="]
}, Open  ]]
}, Open  ]],

Cell["FeynmanParameterNames", "ObjectName",
 CellTags->"FeynmanParameterNames"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FeynmanParameterNames"],

Cell["\<\
FeynmanParameterNames is an option for FeynmanParametrize and \
FeynmanParametrize.\
\>", "Text",
 CellTags->"FeynmanParameterNames"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FeynmanParametrize",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FeynmanParametrize",
  ButtonNote->"FeynmanParametrize"],
 ", ",
 ButtonBox["FeynmanParametrize",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FeynmanParametrize",
  ButtonNote->"FeynmanParametrize"],
 "."
}], "Text",
 CellTags->"FeynmanParameterNames"]
}, Open  ]],

Cell["FeynmanParametrize ***unfinished***", "ObjectName",
 CellTags->"FeynmanParametrize"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FeynmanParametrize"],

Cell["\<\
FeynmanParametrize[exp,k] introduces feynman parameters for all one-loop \
integrals in exp (k = integration momentum).\
\>", "Text",
 CellTags->"FeynmanParametrize"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "FeynmanParametrize", "]"}]], "Input",
 CellTags->"FeynmanParametrize",
 CellLabel->"In[712]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"FeynmanParameterNames", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"x", ",", "y", ",", "z"}], "}"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.50097024475511*^9},
 CellTags->"FeynmanParametrize",
 CellLabel->"Out[712]="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"FeynmanParametrize"],

Cell[BoxData[""], "Input",
 CellTags->"FeynmanParametrize",
 CellLabel->"In[713]:="]
}, Open  ]],

Cell["FeynRule", "ObjectName",
 CellTags->"FeynRule"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FeynRule"],

Cell["\<\
FeynRule[lag, {fields}] derives the Feynman rule corresponding to the field \
configuration fields of the lagrangian lag.\
\>", "Text",
 CellTags->"FeynRule"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "FeynRule", "]"}]], "Input",
 CellTags->"FeynRule",
 CellLabel->"In[714]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Anti5", "\[Rule]", 
     RowBox[{"-", "\[Infinity]"}]}], ",", 
    RowBox[{"Contract", "\[Rule]", "False"}], ",", 
    RowBox[{"Factor1", "\[Rule]", "False"}], ",", 
    RowBox[{"FinalSubstitutions", "\[Rule]", 
     RowBox[{"{", "}"}]}], ",", 
    RowBox[{
    "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", 
     "\[Rule]", "RightPartialD"}], ",", 
    RowBox[{"Schouten", "\[Rule]", "False"}], ",", 
    RowBox[{"ZeroMomentumInsertion", "\[Rule]", "True"}], ",", 
    RowBox[{"InitialFunction", "\[Rule]", "Identity"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970245308669*^9},
 CellTags->"FeynRule",
 CellLabel->"Out[714]="]
}, Open  ]],

Cell["FeynRule does not calculate propagator Feynman rules.", "Text",
 CellTags->"FeynRule"],

Cell["\<\
The option ZeroMomentumInsertion can be used for twist-2 and higher twist \
operators.\
\>", "Text",
 CellTags->"FeynRule"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Lagrangian",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Lagrangian",
  ButtonNote->"Lagrangian"],
 "."
}], "Text",
 CellTags->"FeynRule"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"FeynRule"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"gou", "=", 
  RowBox[{"Lagrangian", "[", "\"\<ogu\>\"", "]"}]}]], "Input",
 CellTags->"FeynRule",
 CellLabel->"In[715]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["1", "2"], " ", 
   SuperscriptBox["\[ImaginaryI]", 
    RowBox[{"m", "-", "1"}]], " ", 
   RowBox[{
    SubsuperscriptBox["F", 
     RowBox[{"\[Alpha]", 
      FormBox["\[CapitalDelta]",
       TraditionalForm]}], "a"], ".", 
    SuperscriptBox[
     RowBox[{"(", 
      SubsuperscriptBox["D", 
       FormBox["\[CapitalDelta]",
        TraditionalForm], 
       RowBox[{"a", "b"}]], ")"}], 
     RowBox[{"m", "-", "2"}]], ".", 
    SubsuperscriptBox["F", 
     RowBox[{"\[Alpha]", 
      FormBox["\[CapitalDelta]",
       TraditionalForm]}], "b"]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.50097024546894*^9},
 CellTags->"FeynRule",
 CellLabel->"Out[715]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"gop", "=", 
  RowBox[{"Lagrangian", "[", "\"\<ogp\>\"", "]"}]}]], "Input",
 CellTags->"FeynRule",
 CellLabel->"In[716]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["1", "2"], " ", 
   SuperscriptBox["\[ImaginaryI]", "m"], " ", 
   RowBox[{
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["\[Alpha]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Beta]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Gamma]",
        TraditionalForm],
       TraditionalForm], 
      FormBox["\[CapitalDelta]",
       TraditionalForm]}]], ".", 
    SubsuperscriptBox["F", 
     RowBox[{"\[Beta]", "\[Gamma]"}], "a"], ".", 
    SuperscriptBox[
     RowBox[{"(", 
      SubsuperscriptBox["D", 
       FormBox["\[CapitalDelta]",
        TraditionalForm], 
       RowBox[{"a", "b"}]], ")"}], 
     RowBox[{"m", "-", "2"}]], ".", 
    SubsuperscriptBox["F", 
     RowBox[{"\[Alpha]", 
      FormBox["\[CapitalDelta]",
       TraditionalForm]}], "b"]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970245630477*^9},
 CellTags->"FeynRule",
 CellLabel->"Out[716]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Explicit", "[", "gop", "]"}]], "Input",
 CellTags->"FeynRule",
 CellLabel->"In[717]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["1", "2"], " ", 
   SuperscriptBox["\[ImaginaryI]", "m"], " ", 
   RowBox[{
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["\[Alpha]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Beta]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Gamma]",
        TraditionalForm],
       TraditionalForm], 
      FormBox["\[CapitalDelta]",
       TraditionalForm]}]], ".", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"QuantumField", "(", 
       RowBox[{
        RowBox[{
        "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
         FormBox["\[Beta]",
          TraditionalForm], ")"}], ",", "A", ",", 
        FormBox["\[Gamma]",
         TraditionalForm], ",", 
        FormBox["a",
         TraditionalForm]}], ")"}], "-", 
      RowBox[{"QuantumField", "(", 
       RowBox[{
        RowBox[{
        "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
         FormBox["\[Gamma]",
          TraditionalForm], ")"}], ",", "A", ",", 
        FormBox["\[Beta]",
         TraditionalForm], ",", 
        FormBox["a",
         TraditionalForm]}], ")"}], "+", 
      RowBox[{
       SubscriptBox["g", "s"], " ", 
       SubscriptBox["f", 
        RowBox[{
         FormBox[
          FormBox["a",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["b28",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["c29",
           TraditionalForm],
          TraditionalForm]}]], " ", 
       RowBox[{
        SubsuperscriptBox[
         FormBox["A",
          TraditionalForm], 
         FormBox[
          FormBox["\[Beta]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["b28",
           TraditionalForm],
          TraditionalForm]], ".", 
        SubsuperscriptBox[
         FormBox["A",
          TraditionalForm], 
         FormBox[
          FormBox["\[Gamma]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["c29",
           TraditionalForm],
          TraditionalForm]]}]}]}], ")"}], ".", 
    SuperscriptBox[
     RowBox[{"(", 
      SubsuperscriptBox["D", 
       FormBox["\[CapitalDelta]",
        TraditionalForm], 
       RowBox[{"a", "b"}]], ")"}], 
     RowBox[{"m", "-", "2"}]], ".", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"QuantumField", "(", 
       RowBox[{
        RowBox[{
        "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
         FormBox["\[Alpha]",
          TraditionalForm], ")"}], ",", "A", ",", "\[CapitalDelta]", ",", 
        FormBox["b",
         TraditionalForm]}], ")"}], "-", 
      RowBox[{"QuantumField", "(", 
       RowBox[{
        RowBox[{
        "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(",
          "\[CapitalDelta]", ")"}], ",", "A", ",", 
        FormBox["\[Alpha]",
         TraditionalForm], ",", 
        FormBox["b",
         TraditionalForm]}], ")"}], "+", 
      RowBox[{
       SubscriptBox["g", "s"], " ", 
       SubscriptBox["f", 
        RowBox[{
         FormBox[
          FormBox["b",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["b30",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["c31",
           TraditionalForm],
          TraditionalForm]}]], " ", 
       RowBox[{
        SubsuperscriptBox[
         FormBox["A",
          TraditionalForm], 
         FormBox[
          FormBox["\[Alpha]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["b30",
           TraditionalForm],
          TraditionalForm]], ".", 
        SubsuperscriptBox[
         FormBox["A",
          TraditionalForm], 
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox[
          FormBox["c31",
           TraditionalForm],
          TraditionalForm]]}]}]}], ")"}]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970245868816*^9},
 CellTags->"FeynRule",
 CellLabel->"Out[717]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Cases2", "[", 
  RowBox[{"%", ",", "QuantumField"}], "]"}]], "Input",
 CellTags->"FeynRule",
 CellLabel->"In[718]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    SubsuperscriptBox[
     FormBox["A",
      TraditionalForm], 
     FormBox[
      FormBox["\[Alpha]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b30",
       TraditionalForm],
      TraditionalForm]], ",", 
    SubsuperscriptBox[
     FormBox["A",
      TraditionalForm], 
     FormBox[
      FormBox["\[Beta]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b28",
       TraditionalForm],
      TraditionalForm]], ",", 
    SubsuperscriptBox[
     FormBox["A",
      TraditionalForm], 
     FormBox[
      FormBox["\[Gamma]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c29",
       TraditionalForm],
      TraditionalForm]], ",", 
    SubsuperscriptBox[
     FormBox["A",
      TraditionalForm], 
     FormBox["\[CapitalDelta]",
      TraditionalForm], 
     FormBox[
      FormBox["c31",
       TraditionalForm],
      TraditionalForm]], ",", 
    RowBox[{"QuantumField", "(", 
     RowBox[{
      RowBox[{
      "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
       FormBox["\[Alpha]",
        TraditionalForm], ")"}], ",", "A", ",", "\[CapitalDelta]", ",", 
      FormBox["b",
       TraditionalForm]}], ")"}], ",", 
    RowBox[{"QuantumField", "(", 
     RowBox[{
      RowBox[{
      "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
       FormBox["\[Beta]",
        TraditionalForm], ")"}], ",", "A", ",", 
      FormBox["\[Gamma]",
       TraditionalForm], ",", 
      FormBox["a",
       TraditionalForm]}], ")"}], ",", 
    RowBox[{"QuantumField", "(", 
     RowBox[{
      RowBox[{
      "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
       FormBox["\[Gamma]",
        TraditionalForm], ")"}], ",", "A", ",", 
      FormBox["\[Beta]",
       TraditionalForm], ",", 
      FormBox["a",
       TraditionalForm]}], ")"}], ",", 
    RowBox[{"QuantumField", "(", 
     RowBox[{
      RowBox[{
      "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(",
        "\[CapitalDelta]", ")"}], ",", "A", ",", 
      FormBox["\[Alpha]",
       TraditionalForm], ",", 
      FormBox["b",
       TraditionalForm]}], ")"}]}], "}"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970246337365*^9},
 CellTags->"FeynRule",
 CellLabel->"Out[718]="]
}, Open  ]],

Cell["2-gluon Feynman rules (unpolarized)", "Text",
 CellTags->"FeynRule"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"fi", "=", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     RowBox[{"QuantumField", "[", 
      RowBox[{"GaugeField", ",", 
       RowBox[{"{", "\[Mu]", "}"}], ",", 
       RowBox[{"{", "a", "}"}]}], "]"}], "[", "p", "]"}], ",", 
    RowBox[{
     RowBox[{"QuantumField", "[", 
      RowBox[{"GaugeField", ",", 
       RowBox[{"{", "\[Nu]", "}"}], ",", 
       RowBox[{"{", "b", "}"}]}], "]"}], "[", "q", "]"}]}], "}"}]}]], "Input",\

 CellTags->"FeynRule",
 CellLabel->"In[719]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     SubsuperscriptBox[
      FormBox["A",
       TraditionalForm], 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm]], "(", "p", ")"}], ",", 
    RowBox[{
     SubsuperscriptBox[
      FormBox["A",
       TraditionalForm], 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]], "(", "q", ")"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970246545639*^9},
 CellTags->"FeynRule",
 CellLabel->"Out[719]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"f2u", "=", 
  RowBox[{"FullSimplify", "/@", 
   RowBox[{"Factor2", "[", 
    RowBox[{"FeynRule", "[", 
     RowBox[{"gou", ",", "fi", ",", 
      RowBox[{"ZeroMomentumInsertion", "\[Rule]", "False"}]}], "]"}], 
    "]"}]}]}]], "Input",
 CellTags->"FeynRule",
 CellLabel->"In[720]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox[
         RowBox[{
          FormBox["\[CapitalDelta]",
           TraditionalForm], "\[CenterDot]", 
          FormBox["q",
           TraditionalForm]}],
         TraditionalForm], "2"], " ", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{
          FormBox["\[CapitalDelta]",
           TraditionalForm], "\[CenterDot]", 
          FormBox["p",
           TraditionalForm]}], ")"}], "m"]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox[
         RowBox[{
          FormBox["\[CapitalDelta]",
           TraditionalForm], "\[CenterDot]", 
          FormBox["p",
           TraditionalForm]}],
         TraditionalForm], "2"], " ", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{
          FormBox["\[CapitalDelta]",
           TraditionalForm], "\[CenterDot]", 
          FormBox["q",
           TraditionalForm]}], ")"}], "m"]}]}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", 
       RowBox[{
        SuperscriptBox["g", 
         RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
        RowBox[{
         FormBox["\[CapitalDelta]",
          TraditionalForm], "\[CenterDot]", 
         FormBox["p",
          TraditionalForm]}], " ", 
        RowBox[{
         FormBox["\[CapitalDelta]",
          TraditionalForm], "\[CenterDot]", 
         FormBox["q",
          TraditionalForm]}]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["q",
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          SuperscriptBox[
           FormBox["p",
            TraditionalForm], 
           FormBox[
            FormBox["\[Nu]",
             TraditionalForm],
            TraditionalForm]], " ", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["q",
            TraditionalForm]}]}], "-", 
         RowBox[{
          SuperscriptBox[
           FormBox["\[CapitalDelta]",
            TraditionalForm], 
           FormBox[
            FormBox["\[Nu]",
             TraditionalForm],
            TraditionalForm]], " ", 
          RowBox[{
           FormBox["p",
            TraditionalForm], "\[CenterDot]", 
           FormBox["q",
            TraditionalForm]}]}]}], ")"}]}]}], ")"}]}], 
   RowBox[{"2", " ", 
    SuperscriptBox[
     FormBox[
      RowBox[{
       FormBox["\[CapitalDelta]",
        TraditionalForm], "\[CenterDot]", 
       FormBox["p",
        TraditionalForm]}],
      TraditionalForm], "2"], " ", 
    SuperscriptBox[
     FormBox[
      RowBox[{
       FormBox["\[CapitalDelta]",
        TraditionalForm], "\[CenterDot]", 
       FormBox["q",
        TraditionalForm]}],
      TraditionalForm], "2"]}]], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970248216243*^9},
 CellTags->"FeynRule",
 CellLabel->"Out[720]="]
}, Open  ]],

Cell["2-gluon Feynman rules (polarized)", "Text",
 CellTags->"FeynRule"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"fi", "=", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     RowBox[{"QuantumField", "[", 
      RowBox[{"GaugeField", ",", 
       RowBox[{"{", "\[Mu]", "}"}], ",", 
       RowBox[{"{", "a", "}"}]}], "]"}], "[", "p", "]"}], ",", 
    RowBox[{
     RowBox[{"QuantumField", "[", 
      RowBox[{"GaugeField", ",", 
       RowBox[{"{", "\[Nu]", "}"}], ",", 
       RowBox[{"{", "b", "}"}]}], "]"}], "[", "q", "]"}]}], "}"}]}]], "Input",\

 CellTags->"FeynRule",
 CellLabel->"In[721]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     SubsuperscriptBox[
      FormBox["A",
       TraditionalForm], 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm]], "(", "p", ")"}], ",", 
    RowBox[{
     SubsuperscriptBox[
      FormBox["A",
       TraditionalForm], 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]], "(", "q", ")"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970248597069*^9},
 CellTags->"FeynRule",
 CellLabel->"Out[721]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"f2p", "=", 
  RowBox[{"FullSimplify", "/@", 
   RowBox[{"Factor2", "[", 
    RowBox[{"FeynRule", "[", 
     RowBox[{"gop", ",", "fi", ",", 
      RowBox[{"ZeroMomentumInsertion", "\[Rule]", "False"}]}], "]"}], 
    "]"}]}]}]], "Input",
 CellTags->"FeynRule",
 CellLabel->"In[722]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   FractionBox[
    RowBox[{"\[ImaginaryI]", " ", 
     SubscriptBox["\[Delta]", 
      RowBox[{
       FormBox[
        FormBox["a",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["b",
         TraditionalForm],
        TraditionalForm]}]], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        SuperscriptBox[
         FormBox[
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["q",
            TraditionalForm]}],
          TraditionalForm], "2"], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["p",
            TraditionalForm]}], ")"}], "m"], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            FormBox["\[CapitalDelta]",
             TraditionalForm], 
            FormBox[
             FormBox["\[Mu]",
              TraditionalForm],
             TraditionalForm]], " ", 
           SuperscriptBox["\[Epsilon]", 
            RowBox[{
             FormBox[
              FormBox["\[Nu]",
               TraditionalForm],
              TraditionalForm], 
             FormBox["p",
              TraditionalForm], 
             FormBox["q",
              TraditionalForm], 
             FormBox["\[CapitalDelta]",
              TraditionalForm]}]]}], "+", 
          RowBox[{
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["p",
             TraditionalForm]}], " ", 
           SuperscriptBox["\[Epsilon]", 
            RowBox[{
             FormBox[
              FormBox["\[Mu]",
               TraditionalForm],
              TraditionalForm], 
             FormBox[
              FormBox["\[Nu]",
               TraditionalForm],
              TraditionalForm], 
             FormBox["q",
              TraditionalForm], 
             FormBox["\[CapitalDelta]",
              TraditionalForm]}]]}]}], ")"}]}], "-", 
       RowBox[{
        SuperscriptBox[
         FormBox[
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["p",
            TraditionalForm]}],
          TraditionalForm], "2"], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["q",
            TraditionalForm]}], ")"}], "m"], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            FormBox["\[CapitalDelta]",
             TraditionalForm], 
            FormBox[
             FormBox["\[Nu]",
              TraditionalForm],
             TraditionalForm]], " ", 
           SuperscriptBox["\[Epsilon]", 
            RowBox[{
             FormBox[
              FormBox["\[Mu]",
               TraditionalForm],
              TraditionalForm], 
             FormBox["p",
              TraditionalForm], 
             FormBox["q",
              TraditionalForm], 
             FormBox["\[CapitalDelta]",
              TraditionalForm]}]]}], "+", 
          RowBox[{
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["q",
             TraditionalForm]}], " ", 
           SuperscriptBox["\[Epsilon]", 
            RowBox[{
             FormBox[
              FormBox["\[Mu]",
               TraditionalForm],
              TraditionalForm], 
             FormBox[
              FormBox["\[Nu]",
               TraditionalForm],
              TraditionalForm], 
             FormBox["p",
              TraditionalForm], 
             FormBox["\[CapitalDelta]",
              TraditionalForm]}]]}]}], ")"}]}]}], ")"}]}], 
    RowBox[{
     SuperscriptBox[
      FormBox[
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p",
         TraditionalForm]}],
       TraditionalForm], "2"], " ", 
     SuperscriptBox[
      FormBox[
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["q",
         TraditionalForm]}],
       TraditionalForm], "2"]}]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009702499120283`*^9},
 CellTags->"FeynRule",
 CellLabel->"Out[722]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Factor2", "[", 
  RowBox[{"Calc", "[", 
   RowBox[{"f2p", "/.", 
    RowBox[{"p", "\[Rule]", 
     RowBox[{"-", "q"}]}]}], "]"}], "]"}]], "Input",
 CellTags->"FeynRule",
 CellLabel->"In[723]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", "\[ImaginaryI]"}], " ", 
   RowBox[{"(", 
    RowBox[{"1", "-", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}], "m"]}], ")"}], " ", 
   SubscriptBox["\[Delta]", 
    RowBox[{
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm]}]], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{
      FormBox["\[CapitalDelta]",
       TraditionalForm], "\[CenterDot]", 
      FormBox["q",
       TraditionalForm]}], ")"}], 
    RowBox[{"m", "-", "1"}]], " ", 
   SuperscriptBox["\[Epsilon]", 
    RowBox[{
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox["q",
      TraditionalForm], 
     FormBox["\[CapitalDelta]",
      TraditionalForm]}]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970250306391*^9},
 CellTags->"FeynRule",
 CellLabel->"Out[723]="]
}, Open  ]],

Cell["\<\
Compare with the Feynman rule tabulated in Twist2GluonOperator.\
\>", "Text",
 CellTags->"FeynRule"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Twist2GluonOperator", "[", 
  RowBox[{"q", ",", 
   RowBox[{"{", 
    RowBox[{"\[Mu]", ",", "a"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\[Nu]", ",", "b"}], "}"}], ",", 
   RowBox[{"Polarization", "\[Rule]", "1"}], ",", 
   RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"FeynRule",
 CellLabel->"In[724]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"\[ImaginaryI]", " ", 
   RowBox[{"(", 
    RowBox[{"1", "-", 
     FormBox[
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"-", "1"}], ")"}], "m"],
      TraditionalForm]}], ")"}], " ", 
   SubscriptBox["\[Delta]", 
    RowBox[{
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm]}]], " ", 
   FormBox[
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{
       FormBox["\[CapitalDelta]",
        TraditionalForm], "\[CenterDot]", 
       FormBox["q",
        TraditionalForm]}], ")"}], 
     RowBox[{"m", "-", "1"}]],
    TraditionalForm], " ", 
   SuperscriptBox["\[Epsilon]", 
    RowBox[{
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox["\[CapitalDelta]",
      TraditionalForm], 
     FormBox["q",
      TraditionalForm]}]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970250401679*^9},
 CellTags->"FeynRule",
 CellLabel->"Out[724]="]
}, Open  ]],

Cell["quark-quark Feynman rule (unpolarized)", "Text",
 CellTags->"FeynRule"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"qo", "=", 
  RowBox[{"Lagrangian", "[", "\"\<oqu\>\"", "]"}]}]], "Input",
 CellTags->"FeynRule",
 CellLabel->"In[725]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[ImaginaryI]", "m"], " ", 
   RowBox[{
    FormBox[
     OverscriptBox["\[Psi]", "_"],
     TraditionalForm], ".", 
    RowBox[{"(", 
     RowBox[{
      FormBox["\<\"\[Gamma]\"\>",
       TraditionalForm], "\[CenterDot]", 
      FormBox["\[CapitalDelta]",
       TraditionalForm]}], ")"}], ".", 
    SuperscriptBox[
     SubscriptBox["D", 
      FormBox["\[CapitalDelta]",
       TraditionalForm]], 
     RowBox[{"m", "-", "1"}]], ".", 
    FormBox["\[Psi]",
     TraditionalForm]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970250488524*^9},
 CellTags->"FeynRule",
 CellLabel->"Out[725]="]
}, Open  ]],

Cell["quark-quark -gluon-gluon Feynman rule (unpolarized)", "Text",
 CellTags->"FeynRule"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"qo", "=", 
  RowBox[{"Lagrangian", "[", "\"\<oqu\>\"", "]"}]}]], "Input",
 CellTags->"FeynRule",
 CellLabel->"In[726]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[ImaginaryI]", "m"], " ", 
   RowBox[{
    FormBox[
     OverscriptBox["\[Psi]", "_"],
     TraditionalForm], ".", 
    RowBox[{"(", 
     RowBox[{
      FormBox["\<\"\[Gamma]\"\>",
       TraditionalForm], "\[CenterDot]", 
      FormBox["\[CapitalDelta]",
       TraditionalForm]}], ")"}], ".", 
    SuperscriptBox[
     SubscriptBox["D", 
      FormBox["\[CapitalDelta]",
       TraditionalForm]], 
     RowBox[{"m", "-", "1"}]], ".", 
    FormBox["\[Psi]",
     TraditionalForm]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009702505912867`*^9},
 CellTags->"FeynRule",
 CellLabel->"Out[726]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"qggf", "=", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     RowBox[{"QuantumField", "[", "QuarkField", "]"}], "[", "p", "]"}], ",", 
    RowBox[{
     RowBox[{"QuantumField", "[", "AntiQuarkField", "]"}], "[", "q", "]"}], 
    ",", 
    RowBox[{
     RowBox[{"QuantumField", "[", 
      RowBox[{"GaugeField", ",", 
       RowBox[{"{", "\[Mu]", "}"}], ",", 
       RowBox[{"{", "a", "}"}]}], "]"}], "[", "r", "]"}], ",", 
    RowBox[{
     RowBox[{"QuantumField", "[", 
      RowBox[{"GaugeField", ",", 
       RowBox[{"{", "\[Nu]", "}"}], ",", 
       RowBox[{"{", "b", "}"}]}], "]"}], "[", "s", "]"}]}], "}"}]}]], "Input",\

 CellTags->"FeynRule",
 CellLabel->"In[727]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     FormBox["\[Psi]",
      TraditionalForm], "(", "p", ")"}], ",", 
    RowBox[{
     FormBox[
      OverscriptBox["\[Psi]", "_"],
      TraditionalForm], "(", "q", ")"}], ",", 
    RowBox[{
     SubsuperscriptBox[
      FormBox["A",
       TraditionalForm], 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm]], "(", "r", ")"}], ",", 
    RowBox[{
     SubsuperscriptBox[
      FormBox["A",
       TraditionalForm], 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]], "(", "s", ")"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.5009702506790047`*^9},
 CellTags->"FeynRule",
 CellLabel->"Out[727]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"n4", "=", 
  RowBox[{"FeynRule", "[", 
   RowBox[{"qo", ",", "qggf", ",", 
    RowBox[{"ZeroMomentumInsertion", "\[Rule]", "True"}]}], "]"}]}]], "Input",\

 CellTags->"FeynRule",
 CellLabel->"In[728]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SuperscriptBox[
     FormBox["\[CapitalDelta]",
      TraditionalForm], 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], " ", 
    SuperscriptBox[
     FormBox["\[CapitalDelta]",
      TraditionalForm], 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubsuperscriptBox["g", "s", "2"], " ", 
    RowBox[{
     SubscriptBox["T", 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]], ".", 
     SubscriptBox["T", 
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm]], ".", 
     RowBox[{"(", 
      RowBox[{
       FormBox["\<\"\[Gamma]\"\>",
        TraditionalForm], "\[CenterDot]", 
       FormBox["\[CapitalDelta]",
        TraditionalForm]}], ")"}]}], " ", 
    RowBox[{"(", 
     RowBox[{
      FormBox[
       TagBox[
        RowBox[{
         UnderoverscriptBox["\[Sum]", 
          RowBox[{"j", "=", "0"}], 
          RowBox[{
           RowBox[{"-", "3"}], "+", "m"}]], "\<\"\"\>"}],
        HoldForm],
       TraditionalForm], 
      FormBox[
       RowBox[{"\<\"\"\>", " ", 
        RowBox[{"(", 
         RowBox[{"j", "+", "1"}], ")"}]}],
       TraditionalForm], 
      FormBox[
       RowBox[{
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"-", "1"}], ")"}], "j"], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["q",
            TraditionalForm]}], ")"}], "i"], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["p",
            TraditionalForm]}], ")"}], 
         RowBox[{
          RowBox[{"-", "j"}], "+", "m", "-", "3"}]], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["q",
             TraditionalForm]}], "+", 
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["s",
             TraditionalForm]}]}], ")"}], 
         RowBox[{"j", "-", "i"}]]}],
       TraditionalForm]}], ")"}]}], "-", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"-", "1"}], ")"}], "m"], " ", 
    SuperscriptBox[
     FormBox["\[CapitalDelta]",
      TraditionalForm], 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], " ", 
    SuperscriptBox[
     FormBox["\[CapitalDelta]",
      TraditionalForm], 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubsuperscriptBox["g", "s", "2"], " ", 
    RowBox[{
     SubscriptBox["T", 
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm]], ".", 
     SubscriptBox["T", 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]], ".", 
     RowBox[{"(", 
      RowBox[{
       FormBox["\<\"\[Gamma]\"\>",
        TraditionalForm], "\[CenterDot]", 
       FormBox["\[CapitalDelta]",
        TraditionalForm]}], ")"}]}], " ", 
    RowBox[{"(", 
     RowBox[{
      FormBox[
       TagBox[
        RowBox[{
         UnderoverscriptBox["\[Sum]", 
          RowBox[{"j", "=", "0"}], 
          RowBox[{
           RowBox[{"-", "3"}], "+", "m"}]], "\<\"\"\>"}],
        HoldForm],
       TraditionalForm], 
      FormBox[
       RowBox[{"\<\"\"\>", " ", 
        RowBox[{"(", 
         RowBox[{"j", "+", "1"}], ")"}]}],
       TraditionalForm], 
      FormBox[
       RowBox[{
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"-", "1"}], ")"}], "j"], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["p",
            TraditionalForm]}], ")"}], "i"], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["q",
            TraditionalForm]}], ")"}], 
         RowBox[{
          RowBox[{"-", "j"}], "+", "m", "-", "3"}]], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["p",
             TraditionalForm]}], "+", 
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["s",
             TraditionalForm]}]}], ")"}], 
         RowBox[{"j", "-", "i"}]]}],
       TraditionalForm]}], ")"}]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009702519647837`*^9},
 CellTags->"FeynRule",
 CellLabel->"Out[728]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t4", "=", 
  RowBox[{"Twist2QuarkOperator", "[", 
   RowBox[{
    RowBox[{"{", "p", "}"}], ",", 
    RowBox[{"{", "q", "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"r", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"s", ",", "\[Nu]", ",", "b"}], "}"}], ",", 
    RowBox[{"Polarization", "\[Rule]", "0"}]}], "]"}]}]], "Input",
 CellTags->"FeynRule",
 CellLabel->"In[729]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"-", "1"}], ")"}], "m"]}], " ", 
   SuperscriptBox[
    FormBox["\[CapitalDelta]",
     TraditionalForm], 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], " ", 
   SuperscriptBox[
    FormBox["\[CapitalDelta]",
     TraditionalForm], 
    FormBox[
     FormBox["\[Nu]",
      TraditionalForm],
     TraditionalForm]], " ", 
   SubsuperscriptBox["g", "s", "2"], " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      FormBox["\<\"\[Gamma]\"\>",
       TraditionalForm], "\[CenterDot]", 
      FormBox["\[CapitalDelta]",
       TraditionalForm]}], ")"}], ".", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{
        SubscriptBox["T", 
         FormBox[
          FormBox["a",
           TraditionalForm],
          TraditionalForm]], ".", 
        SubscriptBox["T", 
         FormBox[
          FormBox["b",
           TraditionalForm],
          TraditionalForm]]}], " ", 
       RowBox[{"(", 
        RowBox[{
         FormBox[
          TagBox[
           RowBox[{
            UnderoverscriptBox["\[Sum]", 
             RowBox[{"i", "=", "0"}], 
             RowBox[{
              RowBox[{"-", "3"}], "+", "m"}]], "\<\"\"\>"}],
           HoldForm],
          TraditionalForm], 
         FormBox[
          RowBox[{"\<\"\"\>", " ", 
           RowBox[{"(", 
            RowBox[{"i", "+", "1"}], ")"}]}],
          TraditionalForm], 
         FormBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["q",
               TraditionalForm]}], ")"}], "j"], " ", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{"-", 
              RowBox[{"(", 
               RowBox[{
                FormBox["\[CapitalDelta]",
                 TraditionalForm], "\[CenterDot]", 
                FormBox["p",
                 TraditionalForm]}], ")"}]}], ")"}], 
            RowBox[{
             RowBox[{"-", "i"}], "+", "m", "-", "3"}]], " ", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               FormBox["\[CapitalDelta]",
                TraditionalForm], "\[CenterDot]", 
               FormBox["q",
                TraditionalForm]}], "+", 
              RowBox[{
               FormBox["\[CapitalDelta]",
                TraditionalForm], "\[CenterDot]", 
               FormBox["r",
                TraditionalForm]}]}], ")"}], 
            RowBox[{"i", "-", "j"}]]}],
          TraditionalForm]}], ")"}]}], "+", 
      RowBox[{
       RowBox[{
        SubscriptBox["T", 
         FormBox[
          FormBox["b",
           TraditionalForm],
          TraditionalForm]], ".", 
        SubscriptBox["T", 
         FormBox[
          FormBox["a",
           TraditionalForm],
          TraditionalForm]]}], " ", 
       RowBox[{"(", 
        RowBox[{
         FormBox[
          TagBox[
           RowBox[{
            UnderoverscriptBox["\[Sum]", 
             RowBox[{"i", "=", "0"}], 
             RowBox[{
              RowBox[{"-", "3"}], "+", "m"}]], "\<\"\"\>"}],
           HoldForm],
          TraditionalForm], 
         FormBox[
          RowBox[{"\<\"\"\>", " ", 
           RowBox[{"(", 
            RowBox[{"i", "+", "1"}], ")"}]}],
          TraditionalForm], 
         FormBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["q",
               TraditionalForm]}], ")"}], "j"], " ", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{"-", 
              RowBox[{"(", 
               RowBox[{
                FormBox["\[CapitalDelta]",
                 TraditionalForm], "\[CenterDot]", 
                FormBox["p",
                 TraditionalForm]}], ")"}]}], ")"}], 
            RowBox[{
             RowBox[{"-", "i"}], "+", "m", "-", "3"}]], " ", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               FormBox["\[CapitalDelta]",
                TraditionalForm], "\[CenterDot]", 
               FormBox["q",
                TraditionalForm]}], "+", 
              RowBox[{
               FormBox["\[CapitalDelta]",
                TraditionalForm], "\[CenterDot]", 
               FormBox["s",
                TraditionalForm]}]}], ")"}], 
            RowBox[{"i", "-", "j"}]]}],
          TraditionalForm]}], ")"}]}]}], ")"}]}]}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970252511416*^9},
 CellTags->"FeynRule",
 CellLabel->"Out[729]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Calc", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"n4", "-", "t4"}], "/.", 
     RowBox[{"OPEm", "\[Rule]", "5"}]}], "/.", 
    RowBox[{"s", "\[Rule]", 
     RowBox[{
      RowBox[{"-", "p"}], "-", "q", "-", "r"}]}]}], "/.", 
   RowBox[{"D", "\[Rule]", "4"}]}], "]"}]], "Input",
 CellTags->"FeynRule",
 CellLabel->"In[730]:="],

Cell[BoxData[
 FormBox["0", TraditionalForm]], "Output",
 CellChangeTimes->{3.5009702535338793`*^9},
 CellTags->"FeynRule",
 CellLabel->"Out[730]="]
}, Open  ]],

Cell["\<\
 In general equality can be shown by Timing[Factor2[ \
FCE[Calc[ChangeDimension[FCE[OPESumExplicit[n4-t4]],4]/.s\[Rule]-p-q-r]]]] \
but it is a little bit slow ...\
\>", "Text",
 CellTags->"FeynRule"],

Cell[BoxData[
 RowBox[{"Clear", "[", 
  RowBox[{"qggf", ",", "n2", ",", "n4"}], "]"}]], "Input",
 CellTags->"FeynRule",
 CellLabel->"In[731]:="],

Cell["QCD vertices", "Text",
 CellTags->"FeynRule"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"fii", "=", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     RowBox[{"QuantumField", "[", 
      RowBox[{"GaugeField", ",", 
       RowBox[{"{", "\[Mu]", "}"}], ",", 
       RowBox[{"{", "a", "}"}]}], "]"}], "[", "p", "]"}], ",", 
    RowBox[{
     RowBox[{"QuantumField", "[", 
      RowBox[{"GaugeField", ",", 
       RowBox[{"{", "\[Nu]", "}"}], ",", 
       RowBox[{"{", "b", "}"}]}], "]"}], "[", "q", "]"}], ",", 
    RowBox[{
     RowBox[{"QuantumField", "[", 
      RowBox[{"GaugeField", ",", 
       RowBox[{"{", "\[Rho]", "}"}], ",", 
       RowBox[{"{", "c", "}"}]}], "]"}], "[", "r", "]"}]}], "}"}]}]], "Input",\

 CellTags->"FeynRule",
 CellLabel->"In[732]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     SubsuperscriptBox[
      FormBox["A",
       TraditionalForm], 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm]], "(", "p", ")"}], ",", 
    RowBox[{
     SubsuperscriptBox[
      FormBox["A",
       TraditionalForm], 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]], "(", "q", ")"}], ",", 
    RowBox[{
     SubsuperscriptBox[
      FormBox["A",
       TraditionalForm], 
      FormBox[
       FormBox["\[Rho]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm]], "(", "r", ")"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.5009702536427183`*^9},
 CellTags->"FeynRule",
 CellLabel->"Out[732]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"g3", "=", 
  RowBox[{"FeynRule", "[", 
   RowBox[{
    RowBox[{"Lagrangian", "[", "\"\<QCD\>\"", "]"}], ",", "fii"}], 
   "]"}]}]], "Input",
 CellTags->"FeynRule",
 CellLabel->"In[733]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["g", "s"], " ", 
   SubscriptBox["f", 
    RowBox[{
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm]}]], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], 
         FormBox[
          FormBox["\[Rho]",
           TraditionalForm],
          TraditionalForm]], "-", 
        SuperscriptBox[
         FormBox["q",
          TraditionalForm], 
         FormBox[
          FormBox["\[Rho]",
           TraditionalForm],
          TraditionalForm]]}], ")"}]}], "-", 
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Rho]"}]], " ", 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], 
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm]], "-", 
        SuperscriptBox[
         FormBox["r",
          TraditionalForm], 
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm]]}], ")"}]}], "+", 
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Nu]", "\[Rho]"}]], " ", 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox[
         FormBox["q",
          TraditionalForm], 
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm]], "-", 
        SuperscriptBox[
         FormBox["r",
          TraditionalForm], 
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm]]}], ")"}]}]}], ")"}]}], TraditionalForm]], "Output",\

 CellChangeTimes->{3.5009702544945517`*^9},
 CellTags->"FeynRule",
 CellLabel->"Out[733]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GluonVertex", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"p", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"q", ",", "\[Nu]", ",", "b"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"r", ",", "\[Rho]", ",", "c"}], "}"}], ",", 
   RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"FeynRule",
 CellLabel->"In[734]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["g", "s"], " ", 
   SubscriptBox["f", 
    RowBox[{
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm]}]], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        FormBox[
         RowBox[{"p", "-", "q"}],
         TraditionalForm], ")"}], 
       FormBox[
        FormBox["\[Rho]",
         TraditionalForm],
        TraditionalForm]]}], "+", 
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Rho]"}]], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        FormBox[
         RowBox[{"r", "-", "p"}],
         TraditionalForm], ")"}], 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]]}], "+", 
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Nu]", "\[Rho]"}]], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        FormBox[
         RowBox[{"q", "-", "r"}],
         TraditionalForm], ")"}], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]]}]}], ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970254615179*^9},
 CellTags->"FeynRule",
 CellLabel->"Out[734]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Calc", "[", 
  RowBox[{"g3", "-", 
   RowBox[{"ChangeDimension", "[", 
    RowBox[{"%", ",", "4"}], "]"}]}], "]"}]], "Input",
 CellTags->"FeynRule",
 CellLabel->"In[735]:="],

Cell[BoxData[
 FormBox["0", TraditionalForm]], "Output",
 CellChangeTimes->{3.500970255070096*^9},
 CellTags->"FeynRule",
 CellLabel->"Out[735]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"fi4", "=", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     RowBox[{"QuantumField", "[", 
      RowBox[{"GaugeField", ",", 
       RowBox[{"{", "\[Mu]", "}"}], ",", 
       RowBox[{"{", "a", "}"}]}], "]"}], "[", "p", "]"}], ",", 
    RowBox[{
     RowBox[{"QuantumField", "[", 
      RowBox[{"GaugeField", ",", 
       RowBox[{"{", "\[Nu]", "}"}], ",", 
       RowBox[{"{", "b", "}"}]}], "]"}], "[", "q", "]"}], ",", 
    RowBox[{
     RowBox[{"QuantumField", "[", 
      RowBox[{"GaugeField", ",", 
       RowBox[{"{", "\[Rho]", "}"}], ",", 
       RowBox[{"{", "c", "}"}]}], "]"}], "[", "r", "]"}], ",", 
    RowBox[{
     RowBox[{"QuantumField", "[", 
      RowBox[{"GaugeField", ",", 
       RowBox[{"{", "\[Sigma]", "}"}], ",", 
       RowBox[{"{", "d", "}"}]}], "]"}], "[", "s", "]"}]}], "}"}]}]], "Input",\

 CellTags->"FeynRule",
 CellLabel->"In[736]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     SubsuperscriptBox[
      FormBox["A",
       TraditionalForm], 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm]], "(", "p", ")"}], ",", 
    RowBox[{
     SubsuperscriptBox[
      FormBox["A",
       TraditionalForm], 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]], "(", "q", ")"}], ",", 
    RowBox[{
     SubsuperscriptBox[
      FormBox["A",
       TraditionalForm], 
      FormBox[
       FormBox["\[Rho]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm]], "(", "r", ")"}], ",", 
    RowBox[{
     SubsuperscriptBox[
      FormBox["A",
       TraditionalForm], 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["d",
        TraditionalForm],
       TraditionalForm]], "(", "s", ")"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.5009702551634293`*^9},
 CellTags->"FeynRule",
 CellLabel->"Out[736]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"g4", "=", 
  RowBox[{"FeynRule", "[", 
   RowBox[{
    RowBox[{"Lagrangian", "[", "\"\<QCD\>\"", "]"}], ",", "fi4"}], 
   "]"}]}]], "Input",
 CellTags->"FeynRule",
 CellLabel->"In[737]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"\[ImaginaryI]", " ", 
    SubsuperscriptBox["g", "s", "2"], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["d",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["si37",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["si37",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"\[Mu]", "\[Rho]"}]], " ", 
       SuperscriptBox["g", 
        RowBox[{"\[Nu]", "\[Sigma]"}]]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
       SuperscriptBox["g", 
        RowBox[{"\[Rho]", "\[Sigma]"}]]}]}], ")"}]}], "+", 
   RowBox[{"\[ImaginaryI]", " ", 
    SubsuperscriptBox["g", "s", "2"], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["si37",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["d",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["si37",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"\[Mu]", "\[Sigma]"}]], " ", 
       SuperscriptBox["g", 
        RowBox[{"\[Nu]", "\[Rho]"}]]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
       SuperscriptBox["g", 
        RowBox[{"\[Rho]", "\[Sigma]"}]]}]}], ")"}]}], "+", 
   RowBox[{"\[ImaginaryI]", " ", 
    SubsuperscriptBox["g", "s", "2"], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["si37",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["d",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["si37",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"\[Mu]", "\[Sigma]"}]], " ", 
       SuperscriptBox["g", 
        RowBox[{"\[Nu]", "\[Rho]"}]]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"\[Mu]", "\[Rho]"}]], " ", 
       SuperscriptBox["g", 
        RowBox[{"\[Nu]", "\[Sigma]"}]]}]}], ")"}]}]}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970256159519*^9},
 CellTags->"FeynRule",
 CellLabel->"Out[737]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GluonVertex", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"p", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"q", ",", "\[Nu]", ",", "b"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"r", ",", "\[Rho]", ",", "c"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"s", ",", "\[Sigma]", ",", "d"}], "}"}], ",", 
   RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"FeynRule",
 CellLabel->"In[738]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", "\[ImaginaryI]"}], " ", 
   SubsuperscriptBox["g", "s", "2"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["a",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["d",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["u95",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["b",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["c",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["u95",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         SuperscriptBox["g", 
          RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
         SuperscriptBox["g", 
          RowBox[{"\[Rho]", "\[Sigma]"}]]}], "-", 
        RowBox[{
         SuperscriptBox["g", 
          RowBox[{"\[Mu]", "\[Rho]"}]], " ", 
         SuperscriptBox["g", 
          RowBox[{"\[Nu]", "\[Sigma]"}]]}]}], ")"}]}], "+", 
     RowBox[{
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["a",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["c",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["u95",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["b",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["d",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["u95",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         SuperscriptBox["g", 
          RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
         SuperscriptBox["g", 
          RowBox[{"\[Rho]", "\[Sigma]"}]]}], "-", 
        RowBox[{
         SuperscriptBox["g", 
          RowBox[{"\[Mu]", "\[Sigma]"}]], " ", 
         SuperscriptBox["g", 
          RowBox[{"\[Nu]", "\[Rho]"}]]}]}], ")"}]}], "+", 
     RowBox[{
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["a",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["b",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["u95",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["c",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["d",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["u95",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         SuperscriptBox["g", 
          RowBox[{"\[Mu]", "\[Rho]"}]], " ", 
         SuperscriptBox["g", 
          RowBox[{"\[Nu]", "\[Sigma]"}]]}], "-", 
        RowBox[{
         SuperscriptBox["g", 
          RowBox[{"\[Mu]", "\[Sigma]"}]], " ", 
         SuperscriptBox["g", 
          RowBox[{"\[Nu]", "\[Rho]"}]]}]}], ")"}]}]}], ")"}]}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970256554842*^9},
 CellTags->"FeynRule",
 CellLabel->"Out[738]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Calc", "[", 
  RowBox[{"g4", "-", 
   RowBox[{"ChangeDimension", "[", 
    RowBox[{"%", ",", "4"}], "]"}]}], "]"}]], "Input",
 CellTags->"FeynRule",
 CellLabel->"In[739]:="],

Cell[BoxData[
 FormBox["0", TraditionalForm]], "Output",
 CellChangeTimes->{3.5009702568689938`*^9},
 CellTags->"FeynRule",
 CellLabel->"Out[739]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"Clear", "[", 
  RowBox[{
  "f2p", ",", "f2u", ",", "f3", ",", "f32", ",", "fi", ",", "fi4", ",", "fii",
    ",", "g3", ",", "g4", ",", "gop", ",", "gou", ",", "n3", ",", "nf3", ",", 
   "n4", ",", "np2", ",", "npf3", ",", "p33", ",", "pf3", ",", "pn3", ",", 
   "pqo", ",", "qf", ",", "qp", ",", "qgf", ",", "qo", ",", "t4"}], 
  "]"}]], "Input",
 CellTags->"FeynRule",
 CellLabel->"In[740]:="]
}, Open  ]],

Cell["FI", "ObjectName",
 CellTags->"FI"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FI"],

Cell["\<\
FI changes the output format to InputForm. This is useful to see the internal \
representation of FeynCalc objects. To change back to FeynCalcForm use FC.\
\>", "Text",
 CellTags->"FI"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FeynCalcForm",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FeynCalcForm",
  ButtonNote->"FeynCalcForm"],
 ", ",
 ButtonBox["FC",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FC",
  ButtonNote->"FC"],
 ", ",
 ButtonBox["FeynCalcExternal",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FeynCalcExternal",
  ButtonNote->"FeynCalcExternal"],
 ", ",
 ButtonBox["FeynCalcInternal",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FeynCalcInternal",
  ButtonNote->"FeynCalcInternal"],
 "."
}], "Text",
 CellTags->"FI"]
}, Open  ]],

Cell["FieldDerivative", "ObjectName",
 CellTags->"FieldDerivative"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FieldDerivative"],

Cell["\<\
FieldDerivative[f[x],x,li1,li2,...] is the derivative of f[x] with respect to \
space-time variables x and with Lorentz indices li1, li2, ..., where li1, \
li2, ... have head LorentzIndex. FieldDerivative[f[x],x,li1,li2,...] can be \
given as FieldDerivative[f[x],x,{l1,l2,...}], where l1 is li1 without the \
head, ... FieldDerivative is defined only for objects with head \
QuantumField[...]. If the space-time derivative of other objects is wanted, \
the corresponding rule must be specified.\
\>", "Text",
 CellTags->"FieldDerivative"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["PartialD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"PartialD",
  ButtonNote->"PartialD"],
 ", ",
 ButtonBox["ExpandPartialD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"ExpandPartialD",
  ButtonNote->"ExpandPartialD"],
 "."
}], "Text",
 CellTags->"FieldDerivative"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"FieldDerivative"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"QuantumField", "[", 
    RowBox[{"A", ",", 
     RowBox[{"{", "\[Mu]", "}"}]}], "]"}], "[", "x", "]"}], ".", 
  RowBox[{
   RowBox[{"QuantumField", "[", 
    RowBox[{"B", ",", 
     RowBox[{"{", "\[Nu]", "}"}]}], "]"}], "[", "y", "]"}], ".", 
  RowBox[{
   RowBox[{"QuantumField", "[", 
    RowBox[{"C", ",", 
     RowBox[{"{", "\[Rho]", "}"}]}], "]"}], "[", "x", "]"}], ".", 
  RowBox[{
   RowBox[{"QuantumField", "[", 
    RowBox[{"D", ",", 
     RowBox[{"{", "\[Sigma]", "}"}]}], "]"}], "[", "y", "]"}]}]], "Input",
 CellTags->"FieldDerivative",
 CellLabel->"In[741]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox["A", "\[Mu]"], "(", "x", ")"}], ".", 
   RowBox[{
    SubscriptBox["B", "\[Nu]"], "(", "y", ")"}], ".", 
   RowBox[{
    SubscriptBox[
     FormBox["C",
      TraditionalForm], "\[Rho]"], "(", "x", ")"}], ".", 
   RowBox[{
    SubscriptBox[
     FormBox["D",
      TraditionalForm], "\[Sigma]"], "(", "y", ")"}]}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970256985818*^9},
 CellTags->"FieldDerivative",
 CellLabel->"Out[741]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"FieldDerivative", "[", 
   RowBox[{"%", ",", "x", ",", 
    RowBox[{"{", "\[Mu]", "}"}]}], "]"}], "//", "DotExpand"}]], "Input",
 CellTags->"FieldDerivative",
 CellLabel->"In[742]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    RowBox[{
     SubscriptBox["A", "\[Mu]"], "(", "x", ")"}], ".", 
    RowBox[{
     SubscriptBox["B", "\[Nu]"], "(", "y", ")"}], ".", 
    RowBox[{
     RowBox[{"QuantumField", "(", 
      RowBox[{
       RowBox[{
       "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
        FormBox["\[Mu]",
         TraditionalForm], ")"}], ",", "C", ",", 
       FormBox["\[Rho]",
        TraditionalForm]}], ")"}], "\[InvisibleApplication]", 
     RowBox[{"(", "x", ")"}]}], ".", 
    RowBox[{
     SubscriptBox[
      FormBox["D",
       TraditionalForm], "\[Sigma]"], "(", "y", ")"}]}], "+", 
   RowBox[{
    RowBox[{
     RowBox[{"QuantumField", "(", 
      RowBox[{
       RowBox[{
       "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
        FormBox["\[Mu]",
         TraditionalForm], ")"}], ",", "A", ",", 
       FormBox["\[Mu]",
        TraditionalForm]}], ")"}], "\[InvisibleApplication]", 
     RowBox[{"(", "x", ")"}]}], ".", 
    RowBox[{
     SubscriptBox["B", "\[Nu]"], "(", "y", ")"}], ".", 
    RowBox[{
     SubscriptBox[
      FormBox["C",
       TraditionalForm], "\[Rho]"], "(", "x", ")"}], ".", 
    RowBox[{
     SubscriptBox[
      FormBox["D",
       TraditionalForm], "\[Sigma]"], "(", "y", ")"}]}]}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970257141315*^9},
 CellTags->"FieldDerivative",
 CellLabel->"Out[742]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"FieldDerivative", "[", 
   RowBox[{"%", ",", "y", ",", 
    RowBox[{"{", "\[Nu]", "}"}]}], "]"}], "//", "DotExpand"}]], "Input",
 CellTags->"FieldDerivative",
 CellLabel->"In[743]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    RowBox[{
     SubscriptBox["A", "\[Mu]"], "(", "x", ")"}], ".", 
    RowBox[{
     SubscriptBox["B", "\[Nu]"], "(", "y", ")"}], ".", 
    RowBox[{
     RowBox[{"QuantumField", "(", 
      RowBox[{
       RowBox[{
       "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
        FormBox["\[Mu]",
         TraditionalForm], ")"}], ",", "C", ",", 
       FormBox["\[Rho]",
        TraditionalForm]}], ")"}], "\[InvisibleApplication]", 
     RowBox[{"(", "x", ")"}]}], ".", 
    RowBox[{
     RowBox[{"QuantumField", "(", 
      RowBox[{
       RowBox[{
       "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
        FormBox["\[Nu]",
         TraditionalForm], ")"}], ",", "D", ",", 
       FormBox["\[Sigma]",
        TraditionalForm]}], ")"}], "\[InvisibleApplication]", 
     RowBox[{"(", "y", ")"}]}]}], "+", 
   RowBox[{
    RowBox[{
     SubscriptBox["A", "\[Mu]"], "(", "x", ")"}], ".", 
    RowBox[{
     RowBox[{"QuantumField", "(", 
      RowBox[{
       RowBox[{
       "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
        FormBox["\[Nu]",
         TraditionalForm], ")"}], ",", "B", ",", 
       FormBox["\[Nu]",
        TraditionalForm]}], ")"}], "\[InvisibleApplication]", 
     RowBox[{"(", "y", ")"}]}], ".", 
    RowBox[{
     RowBox[{"QuantumField", "(", 
      RowBox[{
       RowBox[{
       "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
        FormBox["\[Mu]",
         TraditionalForm], ")"}], ",", "C", ",", 
       FormBox["\[Rho]",
        TraditionalForm]}], ")"}], "\[InvisibleApplication]", 
     RowBox[{"(", "x", ")"}]}], ".", 
    RowBox[{
     SubscriptBox[
      FormBox["D",
       TraditionalForm], "\[Sigma]"], "(", "y", ")"}]}], "+", 
   RowBox[{
    RowBox[{
     RowBox[{"QuantumField", "(", 
      RowBox[{
       RowBox[{
       "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
        FormBox["\[Mu]",
         TraditionalForm], ")"}], ",", "A", ",", 
       FormBox["\[Mu]",
        TraditionalForm]}], ")"}], "\[InvisibleApplication]", 
     RowBox[{"(", "x", ")"}]}], ".", 
    RowBox[{
     SubscriptBox["B", "\[Nu]"], "(", "y", ")"}], ".", 
    RowBox[{
     SubscriptBox[
      FormBox["C",
       TraditionalForm], "\[Rho]"], "(", "x", ")"}], ".", 
    RowBox[{
     RowBox[{"QuantumField", "(", 
      RowBox[{
       RowBox[{
       "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
        FormBox["\[Nu]",
         TraditionalForm], ")"}], ",", "D", ",", 
       FormBox["\[Sigma]",
        TraditionalForm]}], ")"}], "\[InvisibleApplication]", 
     RowBox[{"(", "y", ")"}]}]}], "+", 
   RowBox[{
    RowBox[{
     RowBox[{"QuantumField", "(", 
      RowBox[{
       RowBox[{
       "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
        FormBox["\[Mu]",
         TraditionalForm], ")"}], ",", "A", ",", 
       FormBox["\[Mu]",
        TraditionalForm]}], ")"}], "\[InvisibleApplication]", 
     RowBox[{"(", "x", ")"}]}], ".", 
    RowBox[{
     RowBox[{"QuantumField", "(", 
      RowBox[{
       RowBox[{
       "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
        FormBox["\[Nu]",
         TraditionalForm], ")"}], ",", "B", ",", 
       FormBox["\[Nu]",
        TraditionalForm]}], ")"}], "\[InvisibleApplication]", 
     RowBox[{"(", "y", ")"}]}], ".", 
    RowBox[{
     SubscriptBox[
      FormBox["C",
       TraditionalForm], "\[Rho]"], "(", "x", ")"}], ".", 
    RowBox[{
     SubscriptBox[
      FormBox["D",
       TraditionalForm], "\[Sigma]"], "(", "y", ")"}]}]}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.5009702573067102`*^9},
 CellTags->"FieldDerivative",
 CellLabel->"Out[743]="]
}, Open  ]]
}, Open  ]],

Cell["FieldStrength", "ObjectName",
 CellTags->"FieldStrength"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FieldStrength"],

Cell[TextData[{
 "FieldStrength[\[Mu], \[Nu], a] is the field strength tensor ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SubscriptBox["\[PartialD]", "\[Mu]"], 
     SubsuperscriptBox["A", "\[Nu]", "a"]}], "-", 
    RowBox[{
     SubscriptBox["\[PartialD]", "\[Nu]"], 
     SubsuperscriptBox["A", "\[Mu]", "a"]}], "+", 
    RowBox[{
     SubscriptBox["g", "s"], " ", 
     SubsuperscriptBox["A", "\[Mu]", "b1"], 
     SubsuperscriptBox["A", "\[Nu]", "c1"]}]}], TraditionalForm]]],
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    SuperscriptBox["f", 
     RowBox[{"a", " ", "b1", " ", "c1"}]]}], TraditionalForm]]],
 ". FieldStrength[\[Mu], \[Nu]] is the field strength tensor ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SubscriptBox["\[PartialD]", "\[Mu]"], 
     SubsuperscriptBox["A", "\[Nu]", "\[Null]"]}], "-", 
    RowBox[{
     RowBox[{
      SubscriptBox["\[PartialD]", "\[Nu]"], 
      SubsuperscriptBox["A", "\[Mu]", "\[Null]"]}], "."}]}], 
   TraditionalForm]]],
 " The name of the field (A) and the coupling constant (g) can be set through \
the options or by additional arguments. The first two indices are interpreted \
as type LorentzIndex, except OPEDelta, which is converted to \
Momentum[OPEDelta]."
}], "Text",
 CellTags->"FieldStrength"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "FieldStrength", "]"}]], "Input",
 CellTags->"FieldStrength",
 CellLabel->"In[744]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"CouplingConstant", "\[Rule]", 
     SubscriptBox["g", "s"]}], ",", 
    RowBox[{"Explicit", "\[Rule]", "False"}], ",", 
    RowBox[{
    "HighEnergyPhysics`fctools`IndexPosition`IndexPosition", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{"0", ",", "0"}], "}"}]}], ",", 
    RowBox[{"Symbol", "\[Rule]", "\<\"F\"\>"}], ",", 
    RowBox[{"QuantumField", "\[Rule]", "A"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.50097025764073*^9},
 CellTags->"FieldStrength",
 CellLabel->"Out[744]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["QuantumField",
  BaseStyle->"AddOnsLink",
  ButtonData:>"QuantumField",
  ButtonNote->"QuantumField"],
 "."
}], "Text",
 CellTags->"FieldStrength"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"FieldStrength"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FieldStrength", "[", 
  RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}]], "Input",
 CellTags->"FieldStrength",
 CellLabel->"In[745]:="],

Cell[BoxData[
 FormBox[
  SubsuperscriptBox["F", 
   RowBox[{"\[Mu]", "\[Nu]"}], "\[Null]"], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970257757003*^9},
 CellTags->"FieldStrength",
 CellLabel->"Out[745]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FieldStrength", "[", 
  RowBox[{"\[Mu]", ",", "\[Nu]", ",", "a"}], "]"}]], "Input",
 CellTags->"FieldStrength",
 CellLabel->"In[746]:="],

Cell[BoxData[
 FormBox[
  SubsuperscriptBox["F", 
   RowBox[{"\[Mu]", "\[Nu]"}], "a"], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009702578054543`*^9},
 CellTags->"FieldStrength",
 CellLabel->"Out[746]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FieldStrength", "[", 
  RowBox[{"\[Mu]", ",", "\[Nu]", ",", 
   RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"FieldStrength",
 CellLabel->"In[747]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"QuantumField", "(", 
    RowBox[{
     RowBox[{
     "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(",
       "\[Mu]", ")"}], ",", "A", ",", 
     FormBox["\[Nu]",
      TraditionalForm]}], ")"}], "-", 
   RowBox[{"QuantumField", "(", 
    RowBox[{
     RowBox[{
     "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(",
       "\[Nu]", ")"}], ",", "A", ",", 
     FormBox["\[Mu]",
      TraditionalForm]}], ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970257878474*^9},
 CellTags->"FieldStrength",
 CellLabel->"Out[747]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FieldStrength", "[", 
  RowBox[{"\[Mu]", ",", "\[Nu]", ",", "a", ",", 
   RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"FieldStrength",
 CellLabel->"In[748]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"QuantumField", "(", 
    RowBox[{
     RowBox[{
     "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
      FormBox["\[Mu]",
       TraditionalForm], ")"}], ",", "A", ",", 
     FormBox["\[Nu]",
      TraditionalForm], ",", 
     FormBox["a",
      TraditionalForm]}], ")"}], "-", 
   RowBox[{"QuantumField", "(", 
    RowBox[{
     RowBox[{
     "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
      FormBox["\[Nu]",
       TraditionalForm], ")"}], ",", "A", ",", 
     FormBox["\[Mu]",
      TraditionalForm], ",", 
     FormBox["a",
      TraditionalForm]}], ")"}], "+", 
   RowBox[{
    SubscriptBox["g", "s"], " ", 
    SubscriptBox["f", 
     RowBox[{"a", "b104", "c105"}]], " ", 
    RowBox[{
     SubsuperscriptBox[
      FormBox["A",
       TraditionalForm], 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b104",
        TraditionalForm],
       TraditionalForm]], ".", 
     SubsuperscriptBox[
      FormBox["A",
       TraditionalForm], 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c105",
        TraditionalForm],
       TraditionalForm]]}]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970257994623*^9},
 CellTags->"FieldStrength",
 CellLabel->"Out[748]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StandardForm", "[", 
  RowBox[{"FieldStrength", "[", 
   RowBox[{"\[Mu]", ",", "\[Nu]", ",", 
    RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}], "]"}]], "Input",
 CellTags->"FieldStrength",
 CellLabel->"In[749]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"QuantumField", "[", 
   RowBox[{
    RowBox[{
    "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "[",
      "\[Mu]", "]"}], ",", "GaugeField", ",", 
    RowBox[{"LorentzIndex", "[", "\[Nu]", "]"}]}], "]"}], "-", 
  RowBox[{"QuantumField", "[", 
   RowBox[{
    RowBox[{
    "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "[",
      "\[Nu]", "]"}], ",", "GaugeField", ",", 
    RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}]}], "]"}]}]], "Output",
 CellChangeTimes->{3.500970258137517*^9},
 CellTags->"FieldStrength",
 CellLabel->"Out[749]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StandardForm", "[", 
  RowBox[{"FieldStrength", "[", 
   RowBox[{"\[Mu]", ",", "OPEDelta", ",", 
    RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}], "]"}]], "Input",
 CellTags->"FieldStrength",
 CellLabel->"In[750]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"QuantumField", "[", 
   RowBox[{
    RowBox[{
    "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "[",
      "\[Mu]", "]"}], ",", "GaugeField", ",", 
    RowBox[{"Momentum", "[", "OPEDelta", "]"}]}], "]"}], "-", 
  RowBox[{"QuantumField", "[", 
   RowBox[{
    RowBox[{
    "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "[", 
     RowBox[{"Momentum", "[", "OPEDelta", "]"}], "]"}], ",", "GaugeField", 
    ",", 
    RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}]}], "]"}]}]], "Output",
 CellChangeTimes->{3.500970258192513*^9},
 CellTags->"FieldStrength",
 CellLabel->"Out[750]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FieldStrength", "[", 
  RowBox[{"\[Mu]", ",", "\[Nu]", ",", "a", ",", 
   RowBox[{"CouplingConstant", "\[Rule]", 
    RowBox[{"-", "Gstrong"}]}], ",", 
   RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"FieldStrength",
 CellLabel->"In[751]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"QuantumField", "(", 
    RowBox[{
     RowBox[{
     "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
      FormBox["\[Mu]",
       TraditionalForm], ")"}], ",", "A", ",", 
     FormBox["\[Nu]",
      TraditionalForm], ",", 
     FormBox["a",
      TraditionalForm]}], ")"}], "-", 
   RowBox[{"QuantumField", "(", 
    RowBox[{
     RowBox[{
     "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
      FormBox["\[Nu]",
       TraditionalForm], ")"}], ",", "A", ",", 
     FormBox["\[Mu]",
      TraditionalForm], ",", 
     FormBox["a",
      TraditionalForm]}], ")"}], "-", 
   RowBox[{
    SubscriptBox["g", "s"], " ", 
    SubscriptBox["f", 
     RowBox[{"a", "b106", "c107"}]], " ", 
    RowBox[{
     SubsuperscriptBox[
      FormBox["A",
       TraditionalForm], 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b106",
        TraditionalForm],
       TraditionalForm]], ".", 
     SubsuperscriptBox[
      FormBox["A",
       TraditionalForm], 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c107",
        TraditionalForm],
       TraditionalForm]]}]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970258298931*^9},
 CellTags->"FieldStrength",
 CellLabel->"Out[751]="]
}, Open  ]]
}, Open  ]],

Cell["FinalFunction", "ObjectName",
 CellTags->"FinalFunction"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FinalFunction"],

Cell["FinalFunction is an option for OneLoopSum.", "Text",
 CellTags->"FinalFunction"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["OneLoopSum",
  BaseStyle->"AddOnsLink",
  ButtonData:>"OneLoopSum",
  ButtonNote->"OneLoopSum"],
 "."
}], "Text",
 CellTags->"FinalFunction"]
}, Open  ]],

Cell["FinalSubstitutions", "ObjectName",
 CellTags->"FinalSubstitutions"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FinalSubstitutions"],

Cell["\<\
FinalSubstitutions is an option for OneLoop and OneLoopSum and Write2. All \
substitutions indicated hereby are done at the end of the calculation.\
\>", "Text",
 CellTags->"FinalSubstitutions"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["OneLoop",
  BaseStyle->"AddOnsLink",
  ButtonData:>"OneLoop",
  ButtonNote->"OneLoop"],
 ", ",
 ButtonBox["OneLoopSum",
  BaseStyle->"AddOnsLink",
  ButtonData:>"OneLoopSum",
  ButtonNote->"OneLoopSum"],
 ", ",
 ButtonBox["Write2",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Write2",
  ButtonNote->"Write2"],
 "."
}], "Text",
 CellTags->"FinalSubstitutions"]
}, Open  ]],

Cell["FORM", "ObjectName",
 CellTags->"FORM"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FORM"],

Cell["\<\
FORM is an option for RHI. If set to True a FORM file is generated and run \
from Mathematica (provided R. Hamberg's FORM-program is installed correctly \
... ).\
\>", "Text",
 CellTags->"FORM"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["RHI",
  BaseStyle->"AddOnsLink",
  ButtonData:>"RHI",
  ButtonNote->"RHI"],
 ", ",
 ButtonBox["FeynCalc2FORM",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FeynCalc2FORM",
  ButtonNote->"FeynCalc2FORM"],
 "."
}], "Text",
 CellTags->"FORM"]
}, Open  ]],

Cell["FORMEpilog", "ObjectName",
 CellTags->"FORMEpilog"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FORMEpilog"],

Cell["\<\
FORMEpilog is an option for FeynCalc2FORM. It may be set to a string which is \
put at the end of the FORM-file.\
\>", "Text",
 CellTags->"FORMEpilog"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FeynCalc2FORM",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FeynCalc2FORM",
  ButtonNote->"FeynCalc2FORM"],
 ", ",
 ButtonBox["FORMProlog",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FORMProlog",
  ButtonNote->"FORMProlog"],
 "."
}], "Text",
 CellTags->"FORMEpilog"]
}, Open  ]],

Cell["FORMProlog", "ObjectName",
 CellTags->"FORMProlog"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FORMProlog"],

Cell["\<\
FORMProlog is an option for FeynCalc2FORM. It may be set to a string which is \
put after the type declarations of the FORM-file.\
\>", "Text",
 CellTags->"FORMProlog"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FeynCalc2FORM",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FeynCalc2FORM",
  ButtonNote->"FeynCalc2FORM"],
 ", ",
 ButtonBox["FORMEpilog",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FORMEpilog",
  ButtonNote->"FORMEpilog"],
 "."
}], "Text",
 CellTags->"FORMProlog"]
}, Open  ]],

Cell["FORM2FeynCalc", "ObjectName",
 CellTags->"FORM2FeynCalc"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FORM2FeynCalc"],

Cell["\<\
FORM2FeynCalc[expr] translates the FORM expr into FeynCalc notation. \
FORM2FeynCalc[file] translates the FORM expresssions in file into \
FeynCalcnotation. FORM2FeynCalc[file, x1, x2, ...] reads in a file in \
FORM-format and translates the assignments for the variables a, b, ... into \
FeynCalc syntax. If the option Set is True, the variables x1, x2 are assigned \
to the right hand sides defined in the FORM-file.\
\>", "Text",
 CellTags->"FORM2FeynCalc"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FeynCalc2FORM",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FeynCalc2FORM",
  ButtonNote->"FeynCalc2FORM"],
 "."
}], "Text",
 CellTags->"FORM2FeynCalc"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "FORM2FeynCalc", "]"}]], "Input",
 CellTags->"FORM2FeynCalc",
 CellLabel->"In[752]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Dimension", "\[Rule]", "4"}], ",", 
    RowBox[{"FinalSubstitutions", "\[Rule]", 
     RowBox[{"{", "}"}]}], ",", 
    RowBox[{"Dot", "\[Rule]", "Dot"}], ",", 
    RowBox[{"HoldForm", "\[Rule]", "True"}], ",", 
    RowBox[{"LorentzIndex", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{"mu", ",", "nu", ",", "al", ",", "be"}], "}"}]}], ",", 
    RowBox[{"Set", "\[Rule]", "False"}], ",", 
    RowBox[{"Replace", "\[Rule]", 
     RowBox[{"{", "}"}]}], ",", 
    RowBox[{"Vectors", "\[Rule]", "Automatic"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970258430245*^9},
 CellTags->"FORM2FeynCalc",
 CellLabel->"Out[752]="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"FORM2FeynCalc"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FORM2FeynCalc", "[", "\"\<p.q + 2*x m^2\>\"", "]"}]], "Input",
 CellTags->"FORM2FeynCalc",
 CellLabel->"In[753]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"2", " ", 
    RowBox[{"x", ".", 
     SuperscriptBox["m", "2"]}]}], "+", 
   FormBox[
    RowBox[{
     FormBox["p",
      TraditionalForm], "\[CenterDot]", 
     FormBox["q",
      TraditionalForm]}],
    TraditionalForm]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.50097025854235*^9},
 CellTags->"FORM2FeynCalc",
 CellLabel->"Out[753]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "StandardForm"}]], "Input",
 CellTags->"FORM2FeynCalc",
 CellLabel->"In[754]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"2", " ", 
   RowBox[{"x", ".", 
    SuperscriptBox["m", "2"]}]}], "+", 
  RowBox[{"SP", "[", 
   RowBox[{"p", ",", "q"}], "]"}]}]], "Output",
 CellChangeTimes->{3.500970258676321*^9},
 CellTags->"FORM2FeynCalc",
 CellLabel->"Out[754]//StandardForm="]
}, Open  ]],

Cell["\<\
Functions are automatically converted right, but bracketed expressions need \
to be substituted explicitly.\
\>", "Text",
 CellTags->"FORM2FeynCalc"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FORM2FeynCalc", "[", 
  RowBox[{"\"\<x +f(z)+ log(x)^2+[li2(1-x)]\>\"", ",", 
   RowBox[{"Replace", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"\"\<[li2(1-x)]\>\"", "\[Rule]", "\"\<PolyLog[2,1-x]\>\""}], 
     "}"}]}]}], "]"}]], "Input",
 CellTags->"FORM2FeynCalc",
 CellLabel->"In[755]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"f", "(", "z", ")"}], "+", 
   TemplateBox[{"2",RowBox[{"1", "-", "x"}]},
    "PolyLog"], "+", "x", "+", 
   RowBox[{
    SuperscriptBox["log", "2"], "(", "x", ")"}]}], TraditionalForm]], "Output",\

 CellChangeTimes->{3.500970258766218*^9},
 CellTags->"FORM2FeynCalc",
 CellLabel->"Out[755]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "StandardForm"}]], "Input",
 CellTags->"FORM2FeynCalc",
 CellLabel->"In[756]:="],

Cell[BoxData[
 RowBox[{"x", "+", 
  RowBox[{"f", "[", "z", "]"}], "+", 
  SuperscriptBox[
   RowBox[{"Log", "[", "x", "]"}], "2"], "+", 
  RowBox[{"PolyLog", "[", 
   RowBox[{"2", ",", 
    RowBox[{"1", "-", "x"}]}], "]"}]}]], "Output",
 CellChangeTimes->{3.5009702588386583`*^9},
 CellTags->"FORM2FeynCalc",
 CellLabel->"Out[756]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FORM2FeynCalc", "[", "\"\<x + [(1)]*y -[(-1)^m]\>\"", 
  "]"}]], "Input",
 CellTags->"FORM2FeynCalc",
 CellLabel->"In[757]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    RowBox[{"Hold", "[", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}], "m"], "]"}]}], "+", 
   RowBox[{
    RowBox[{"Hold", "[", "1", "]"}], ".", "y"}], "+", "x"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.5009702589084682`*^9},
 CellTags->"FORM2FeynCalc",
 CellLabel->"Out[757]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ReleaseHold", "[", "%", "]"}]], "Input",
 CellTags->"FORM2FeynCalc",
 CellLabel->"In[758]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"-", "1"}], ")"}], "m"]}], "+", "x", "+", 
   RowBox[{"1", ".", "y"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009702589776154`*^9},
 CellTags->"FORM2FeynCalc",
 CellLabel->"Out[758]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FORM2FeynCalc", "[", "\"\<p(mu)*q(nu)+d_(mu,nu)\>\"", 
  "]"}]], "Input",
 CellTags->"FORM2FeynCalc",
 CellLabel->"In[759]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["g", 
    RowBox[{"mu", "nu"}]], "+", 
   RowBox[{
    SuperscriptBox["p", "mu"], ".", 
    SuperscriptBox["q", "nu"]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009702590442038`*^9},
 CellTags->"FORM2FeynCalc",
 CellLabel->"Out[759]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "StandardForm"}]], "Input",
 CellTags->"FORM2FeynCalc",
 CellLabel->"In[760]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"FV", "[", 
    RowBox[{"p", ",", "mu"}], "]"}], ".", 
   RowBox[{"FV", "[", 
    RowBox[{"q", ",", "nu"}], "]"}]}], "+", 
  RowBox[{"MT", "[", 
   RowBox[{"mu", ",", "nu"}], "]"}]}]], "Output",
 CellChangeTimes->{3.5009702591180153`*^9},
 CellTags->"FORM2FeynCalc",
 CellLabel->"Out[760]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FORM2FeynCalc", "[", 
  RowBox[{"\"\<p(mu)*q(nu)+d_(mu,nu)\>\"", ",", 
   RowBox[{"Replace", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"mu", "\[Rule]", "\[Mu]"}], ",", 
      RowBox[{"nu", "\[Rule]", "\[Nu]"}]}], "}"}]}]}], "]"}]], "Input",
 CellTags->"FORM2FeynCalc",
 CellLabel->"In[761]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["g", 
    RowBox[{"\[Mu]", "\[Nu]"}]], "+", 
   RowBox[{
    SuperscriptBox["p", "\[Mu]"], ".", 
    SuperscriptBox["q", "\[Nu]"]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009702592083693`*^9},
 CellTags->"FORM2FeynCalc",
 CellLabel->"Out[761]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FORM2FeynCalc", "[", 
  "\"\<i_*az*bz*aM^2*D1*[(1)]*b_G1 * ( 4*eperp(mu,nu)*avec.bvec*blam )\>\"", 
  "]"}]], "Input",
 CellTags->"FORM2FeynCalc",
 CellLabel->"In[762]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"(", 
    RowBox[{"4", " ", "\[ImaginaryI]"}], ")"}], ".", "az", ".", "bz", ".", 
   SuperscriptBox["aM", "2"], ".", "D1", ".", 
   RowBox[{"Hold", "[", "1", "]"}], ".", "b$G1", ".", 
   RowBox[{"eperp", "(", 
    RowBox[{"mu", ",", "nu"}], ")"}], ".", 
   RowBox[{"(", 
    FormBox[
     RowBox[{
      FormBox["avec",
       TraditionalForm], "\[CenterDot]", 
      FormBox["bvec",
       TraditionalForm]}],
     TraditionalForm], ")"}], ".", "blam"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970259285509*^9},
 CellTags->"FORM2FeynCalc",
 CellLabel->"Out[762]="]
}, Open  ]]
}, Open  ]],

Cell["FourDivergence", "ObjectName",
 CellTags->"FourDivergence"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FourDivergence"],

Cell["\<\
FourDivergence[exp, FourVector[p, mu]] calculates the partial derivative of \
exp w.r.t. p(mu). FourDivergence[exp, FourVector[p, mu], FourVector[p,nu], \
...]gives the multiple derivative.\
\>", "Text",
 CellTags->"FourDivergence"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["RussianTrick",
  BaseStyle->"AddOnsLink",
  ButtonData:>"RussianTrick",
  ButtonNote->"RussianTrick"],
 "."
}], "Text",
 CellTags->"FourDivergence"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"FourDivergence"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t", "=", 
  RowBox[{"ScalarProduct", "[", 
   RowBox[{"p", ",", "q"}], "]"}]}]], "Input",
 CellTags->"FourDivergence",
 CellLabel->"In[763]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox["p",
    TraditionalForm], "\[CenterDot]", 
   FormBox["q",
    TraditionalForm]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009702593607893`*^9},
 CellTags->"FourDivergence",
 CellLabel->"Out[763]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FourDivergence", "[", 
  RowBox[{"t", ",", 
   RowBox[{"FourVector", "[", 
    RowBox[{"q", ",", "\[Mu]"}], "]"}]}], "]"}]], "Input",
 CellTags->"FourDivergence",
 CellLabel->"In[764]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox[
   FormBox["p",
    TraditionalForm], 
   FormBox[
    FormBox["\[Mu]",
     TraditionalForm],
    TraditionalForm]], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970259436307*^9},
 CellTags->"FourDivergence",
 CellLabel->"Out[764]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t", "=", 
  RowBox[{"ScalarProduct", "[", 
   RowBox[{
    RowBox[{"p", "-", "k"}], ",", "q"}], "]"}]}]], "Input",
 CellTags->"FourDivergence",
 CellLabel->"In[765]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"(", 
   FormBox[
    RowBox[{"p", "-", "k"}],
    TraditionalForm], ")", "\[CenterDot]", 
   FormBox["q",
    TraditionalForm]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970259894912*^9},
 CellTags->"FourDivergence",
 CellLabel->"Out[765]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FourDivergence", "[", 
  RowBox[{"t", ",", 
   RowBox[{"FourVector", "[", 
    RowBox[{
     RowBox[{"k", "-", "p"}], ",", "\[Mu]"}], "]"}]}], "]"}]], "Input",
 CellTags->"FourDivergence",
 CellLabel->"In[766]:="],

Cell[BoxData[
 FormBox["0", TraditionalForm]], "Output",
 CellChangeTimes->{3.500970260064561*^9},
 CellTags->"FourDivergence",
 CellLabel->"Out[766]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"Clear", "[", "t", "]"}]], "Input",
 CellTags->"FourDivergence",
 CellLabel->"In[767]:="]
}, Open  ]],

Cell["FourLaplacian", "ObjectName",
 CellTags->"FourLaplacian"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FourLaplacian"],

Cell[TextData[{
 "FourLaplacian[exp, p, q] is ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[PartialD]", 
    RowBox[{"/", 
     RowBox[{"\[PartialD]", 
      SubscriptBox["p", "\[Mu]"]}]}], " "}], TraditionalForm]]],
 Cell[BoxData[
  FormBox[
   RowBox[{"\[PartialD]", 
    RowBox[{"/", 
     RowBox[{"\[PartialD]", 
      SubscriptBox["q", "\[Mu]"]}]}], " "}], TraditionalForm]]],
 "exp."
}], "Text",
 CellTags->"FourLaplacian"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "FourLaplacian", "]"}]], "Input",
 CellTags->"FourLaplacian",
 CellLabel->"In[768]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"Dimension", "\[Rule]", "D"}], "}"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970260225977*^9},
 CellTags->"FourLaplacian",
 CellLabel->"Out[768]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FourDivergence",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FourDivergence",
  ButtonNote->"FourDivergence"],
 ", ",
 ButtonBox["RussianTrick",
  BaseStyle->"AddOnsLink",
  ButtonData:>"RussianTrick",
  ButtonNote->"RussianTrick"],
 "."
}], "Text",
 CellTags->"FourLaplacian"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"FourLaplacian"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SP", "[", 
  RowBox[{"q", ",", "q"}], "]"}]], "Input",
 CellTags->"FourLaplacian",
 CellLabel->"In[769]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox[
   FormBox["q",
    TraditionalForm], "2"], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970260369718*^9},
 CellTags->"FourLaplacian",
 CellLabel->"Out[769]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FourLaplacian", "[", 
  RowBox[{"%", ",", "q", ",", "q"}], "]"}]], "Input",
 CellTags->"FourLaplacian",
 CellLabel->"In[770]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"2", " ", "D"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009702604763527`*^9},
 CellTags->"FourLaplacian",
 CellLabel->"Out[770]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"SOD", "[", "q", "]"}], "^", "OPEm"}], 
   RowBox[{"FAD", "[", 
    RowBox[{"q", ",", 
     RowBox[{"q", "-", "p"}]}], "]"}]}], "//", "FCI"}]], "Input",
 CellTags->"FourLaplacian",
 CellLabel->"In[771]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{
      FormBox["\[CapitalDelta]",
       TraditionalForm], "\[CenterDot]", 
      FormBox["q",
       TraditionalForm]}], ")"}], "m"], 
   RowBox[{
    SuperscriptBox["q", "2"], ".", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"q", "-", "p"}], ")"}], "2"]}]], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970260990803*^9},
 CellTags->"FourLaplacian",
 CellLabel->"Out[771]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FourLaplacian", "[", 
  RowBox[{"%", ",", "q", ",", " ", "q"}], "]"}]], "Input",
 CellTags->"FourLaplacian",
 CellLabel->"In[772]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    FractionBox[
     RowBox[{"2", " ", "D", " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{
         FormBox["\[CapitalDelta]",
          TraditionalForm], "\[CenterDot]", 
         FormBox["q",
          TraditionalForm]}], ")"}], "m"]}], 
     RowBox[{
      SuperscriptBox["q", "2"], ".", 
      SuperscriptBox["q", "2"], ".", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"q", "-", "p"}], ")"}], "2"]}]]}], "-", 
   FractionBox[
    RowBox[{"2", " ", "D", " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["q",
         TraditionalForm]}], ")"}], "m"]}], 
    RowBox[{
     SuperscriptBox["q", "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q", "-", "p"}], ")"}], "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q", "-", "p"}], ")"}], "2"]}]], "-", 
   FractionBox[
    RowBox[{"4", " ", "m", " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["q",
         TraditionalForm]}], ")"}], 
      RowBox[{"m", "-", "1"}]], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["q",
         TraditionalForm]}], "-", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p",
         TraditionalForm]}]}], ")"}]}], 
    RowBox[{
     SuperscriptBox["q", "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q", "-", "p"}], ")"}], "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q", "-", "p"}], ")"}], "2"]}]], "-", 
   FractionBox[
    RowBox[{"4", " ", "m", " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["q",
         TraditionalForm]}], ")"}], "m"]}], 
    RowBox[{
     SuperscriptBox["q", "2"], ".", 
     SuperscriptBox["q", "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q", "-", "p"}], ")"}], "2"]}]], "+", 
   FractionBox[
    RowBox[{"12", " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["q",
         TraditionalForm]}], ")"}], "m"]}], 
    RowBox[{
     SuperscriptBox["q", "2"], ".", 
     SuperscriptBox["q", "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q", "-", "p"}], ")"}], "2"]}]], "+", 
   FractionBox[
    RowBox[{"12", " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["q",
         TraditionalForm]}], ")"}], "m"]}], 
    RowBox[{
     SuperscriptBox["q", "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q", "-", "p"}], ")"}], "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q", "-", "p"}], ")"}], "2"]}]], "-", 
   FractionBox[
    RowBox[{"4", " ", 
     SuperscriptBox[
      FormBox["p",
       TraditionalForm], "2"], " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["q",
         TraditionalForm]}], ")"}], "m"]}], 
    RowBox[{
     SuperscriptBox["q", "2"], ".", 
     SuperscriptBox["q", "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q", "-", "p"}], ")"}], "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q", "-", "p"}], ")"}], "2"]}]]}], TraditionalForm]], "Output",\

 CellChangeTimes->{3.500970261244587*^9},
 CellTags->"FourLaplacian",
 CellLabel->"Out[772]="]
}, Open  ]]
}, Open  ]],

Cell["FourVector", "ObjectName",
 CellTags->"FourVector"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FourVector"],

Cell[TextData[{
 "FourVector[p, ",
 Cell[BoxData[
  FormBox["\[Mu]", TraditionalForm]]],
 "] is the four-dimensional vector p with Lorentz index ",
 Cell[BoxData[
  FormBox["\[Mu]", TraditionalForm]]],
 ". A vector with space-time Dimension D is obtained by supplying the option \
Dimension \[Rule] D."
}], "Text",
 CellTags->"FourVector"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FV",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FV",
  ButtonNote->"FV"],
 ", ",
 ButtonBox["FVD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FVD",
  ButtonNote->"FVD"],
 ", ",
 ButtonBox["Pair",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Pair",
  ButtonNote->"Pair"],
 "."
}], "Text",
 CellTags->"FourVector"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"FourVector"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FourVector", "[", 
  RowBox[{"p", ",", "\[Mu]"}], "]"}]], "Input",
 CellTags->"FourVector",
 CellLabel->"In[773]:="],

Cell[BoxData[
 FormBox[
  SubscriptBox["p", "\[Mu]"], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970261291292*^9},
 CellTags->"FourVector",
 CellLabel->"Out[773]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FourVector", "[", 
  RowBox[{
   RowBox[{"p", "-", "q"}], ",", "\[Mu]"}], "]"}]], "Input",
 CellTags->"FourVector",
 CellLabel->"In[774]:="],

Cell[BoxData[
 FormBox[
  SubscriptBox[
   RowBox[{"(", 
    FormBox[
     TagBox[
      RowBox[{"p", "-", "q"}],
      HoldForm],
     TraditionalForm], ")"}], "\[Mu]"], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970261334917*^9},
 CellTags->"FourVector",
 CellLabel->"Out[774]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StandardForm", "[", 
  RowBox[{"FourVector", "[", 
   RowBox[{"p", ",", "\[Mu]"}], "]"}], "]"}]], "Input",
 CellTags->"FourVector",
 CellLabel->"In[775]:="],

Cell[BoxData[
 RowBox[{"FourVector", "[", 
  RowBox[{"p", ",", "\[Mu]"}], "]"}]], "Output",
 CellChangeTimes->{3.500970261377803*^9},
 CellTags->"FourVector",
 CellLabel->"Out[775]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StandardForm", "[", 
  RowBox[{"FourVector", "[", 
   RowBox[{"p", ",", "\[Mu]", ",", 
    RowBox[{"Dimension", "\[Rule]", "D"}]}], "]"}], "]"}]], "Input",
 CellTags->"FourVector",
 CellLabel->"In[776]:="],

Cell[BoxData[
 RowBox[{"FourVector", "[", 
  RowBox[{"p", ",", "\[Mu]", ",", 
   RowBox[{"Dimension", "\[Rule]", "D"}]}], "]"}]], "Output",
 CellChangeTimes->{3.500970261419935*^9},
 CellTags->"FourVector",
 CellLabel->"Out[776]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StandardForm", "[", 
  RowBox[{"FCE", "[", 
   RowBox[{"FourVector", "[", 
    RowBox[{"p", ",", "\[Mu]"}], "]"}], "]"}], "]"}]], "Input",
 CellTags->"FourVector",
 CellLabel->"In[777]:="],

Cell[BoxData[
 RowBox[{"FourVector", "[", 
  RowBox[{"p", ",", "\[Mu]"}], "]"}]], "Output",
 CellChangeTimes->{3.5009702615005302`*^9},
 CellTags->"FourVector",
 CellLabel->"Out[777]//StandardForm="]
}, Open  ]],

Cell["\<\
There is no special function to expand momenta in FourVector. Since \
FourVector is turned into Pair internally ExpandScalarProduct may be used.\
\>", "Text",
 CellTags->"FourVector"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ExpandScalarProduct", "[", 
  RowBox[{"FourVector", "[", 
   RowBox[{
    RowBox[{"p", "-", "q"}], ",", "\[Mu]"}], "]"}], "]"}]], "Input",
 CellTags->"FourVector",
 CellLabel->"In[778]:="],

Cell[BoxData[
 FormBox[
  SubscriptBox[
   RowBox[{"(", 
    FormBox[
     TagBox[
      RowBox[{"p", "-", "q"}],
      HoldForm],
     TraditionalForm], ")"}], "\[Mu]"], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970261615526*^9},
 CellTags->"FourVector",
 CellLabel->"Out[778]="]
}, Open  ]]
}, Open  ]],

Cell["FreeIndex", "ObjectName",
 CellTags->"FreeIndex"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FreeIndex"],

Cell["\<\
FreeIndex is a datatype which is recognized by Contract. Possible use: \
DataType[mu, FreeIndex] = True.\
\>", "Text",
 CellTags->"FreeIndex"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Contract",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Contract",
  ButtonNote->"Contract"],
 ", ",
 ButtonBox["DataType",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DataType",
  ButtonNote->"DataType"],
 "."
}], "Text",
 CellTags->"FreeIndex"]
}, Open  ]],

Cell["FreeQ2", "ObjectName",
 CellTags->"FreeQ2"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FreeQ2"],

Cell["\<\
FreeQ2[expr, {form1, form2, ...}] yields True if expr does not contain any \
occurence of form1, form2, ... and False otherwise. FreeQ2[expr, form] is the \
same as FreeQ[expr, form].\
\>", "Text",
 CellTags->"FreeQ2"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["SelectFree",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SelectFree",
  ButtonNote->"SelectFree"],
 ", ",
 ButtonBox["SelectNotFree",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SelectNotFree",
  ButtonNote->"SelectNotFree"],
 "."
}], "Text",
 CellTags->"FreeQ2"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"FreeQ2"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FreeQ2", "[", 
  RowBox[{
   RowBox[{"x", "+", 
    RowBox[{"f", "[", "x", "]"}], "+", "y"}], ",", " ", 
   RowBox[{"{", 
    RowBox[{"a", ",", "x"}], "}"}]}], "]"}]], "Input",
 CellTags->"FreeQ2",
 CellLabel->"In[779]:="],

Cell[BoxData[
 FormBox["False", TraditionalForm]], "Output",
 CellChangeTimes->{3.500970261708275*^9},
 CellTags->"FreeQ2",
 CellLabel->"Out[779]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FreeQ2", "[", 
  RowBox[{
   RowBox[{"x", "+", 
    RowBox[{"f", "[", "x", "]"}], "+", "y"}], ",", 
   RowBox[{"{", 
    RowBox[{"a", ",", "b"}], "}"}]}], "]"}]], "Input",
 CellTags->"FreeQ2",
 CellLabel->"In[780]:="],

Cell[BoxData[
 FormBox["True", TraditionalForm]], "Output",
 CellChangeTimes->{3.500970261798047*^9},
 CellTags->"FreeQ2",
 CellLabel->"Out[780]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FreeQ2", "[", 
  RowBox[{"x", ",", " ", "y"}], "]"}]], "Input",
 CellTags->"FreeQ2",
 CellLabel->"In[781]:="],

Cell[BoxData[
 FormBox["True", TraditionalForm]], "Output",
 CellChangeTimes->{3.5009702618468*^9},
 CellTags->"FreeQ2",
 CellLabel->"Out[781]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FreeQ2", "[", 
  RowBox[{
   RowBox[{"f", "[", "x", "]"}], ",", " ", "f"}], "]"}]], "Input",
 CellTags->"FreeQ2",
 CellLabel->"In[782]:="],

Cell[BoxData[
 FormBox["False", TraditionalForm]], "Output",
 CellChangeTimes->{3.500970261921678*^9},
 CellTags->"FreeQ2",
 CellLabel->"Out[782]="]
}, Open  ]]
}, Open  ]],

Cell["FRH", "ObjectName",
 CellTags->"FRH"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FRH"],

Cell["\<\
FRH[exp_] := FixedPoint[ReleaseHold, exp], i.e., FRH removes all HoldForm and \
Hold in exp.\
\>", "Text",
 CellTags->"FRH"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Isolate",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Isolate",
  ButtonNote->"Isolate"],
 "."
}], "Text",
 CellTags->"FRH"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"FRH"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Hold", "[", 
  RowBox[{"1", "-", "1", " ", "-", " ", 
   RowBox[{"Hold", "[", 
    RowBox[{"2", "-", "2"}], "]"}]}], "]"}]], "Input",
 CellTags->"FRH",
 CellLabel->"In[783]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"Hold", "[", 
   RowBox[{
    RowBox[{"-", 
     RowBox[{"Hold", "[", 
      RowBox[{"2", "-", "2"}], "]"}]}], "+", "1", "-", "1"}], "]"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970261994589*^9},
 CellTags->"FRH",
 CellLabel->"Out[783]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FRH", "[", "%", "]"}]], "Input",
 CellTags->"FRH",
 CellLabel->"In[784]:="],

Cell[BoxData[
 FormBox["0", TraditionalForm]], "Output",
 CellChangeTimes->{3.500970262067672*^9},
 CellTags->"FRH",
 CellLabel->"Out[784]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Isolate", "[", 
  RowBox[{
   RowBox[{"Solve", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"x", "^", "3"}], "-", "x", "-", "1"}], "==", "0"}], "]"}], ",", 
   "x", ",", 
   RowBox[{"IsolateNames", "\[Rule]", "HH"}]}], "]"}]], "Input",
 CellTags->"FRH",
 CellLabel->"In[785]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"x", "\[Rule]", 
      TagBox[
       RowBox[{"HH", "(", "110", ")"}],
       HoldForm]}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"x", "\[Rule]", 
      TagBox[
       RowBox[{"HH", "(", "113", ")"}],
       HoldForm]}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"x", "\[Rule]", 
      TagBox[
       RowBox[{"HH", "(", "114", ")"}],
       HoldForm]}], "}"}]}], "}"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970262148705*^9},
 CellTags->"FRH",
 CellLabel->"Out[785]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FRH", "[", 
  RowBox[{"HH", "[", "3", "]"}], "]"}]], "Input",
 CellTags->"FRH",
 CellLabel->"In[786]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"HH", "(", "3", ")"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970262193712*^9},
 CellTags->"FRH",
 CellLabel->"Out[786]="]
}, Open  ]]
}, Open  ]],

Cell["FromTFI", "ObjectName",
 CellTags->"FromTFi"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FromTFi"],

Cell["\<\
FromTFi[expr, q1, q2, p] translates the TFi notatation from the TARCER \
package to the usual FeynCalc notation. See TFi for details on the \
conventions.\
\>", "Text",
 CellTags->"FromTFi"],

Cell[TextData[{
 "See also: ",
 ButtonBox["TFI",
  BaseStyle->"AddOnsLink",
  ButtonData:>"TFI",
  ButtonNote->"TFI"],
 ", ",
 ButtonBox["ToTFI",
  BaseStyle->"AddOnsLink",
  ButtonData:>"ToTFI",
  ButtonNote->"ToTFI"],
 "."
}], "Text",
 CellTags->"FromTFI"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"FromTFi"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ToTFI", "[", 
  RowBox[{
   RowBox[{"FAD", "[", 
    RowBox[{"q1", ",", 
     RowBox[{"q1", "-", "p"}], ",", 
     RowBox[{"{", 
      RowBox[{"q2", ",", "M"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"q2", "-", "p"}], ",", "m"}], "}"}], ",", 
     RowBox[{"q1", "-", "q2"}]}], "]"}], ",", "q1", ",", "q2", ",", "p"}], 
  "]"}]], "Input",
 CellTags->"FromTFi",
 CellLabel->"In[787]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"TFI", "(", 
   RowBox[{"D", ",", 
    SuperscriptBox["m", "2"], ",", 
    RowBox[{"(", "\[NoBreak]", GridBox[{
       {"1", "0"},
       {"1", "M"},
       {"1", "0"},
       {"1", "m"},
       {"1", "0"}
      },
      GridBoxAlignment->{
       "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
        "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
      GridBoxSpacings->{"Columns" -> {
          Offset[0.27999999999999997`], {
           Offset[0.7]}, 
          Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
          Offset[0.2], {
           Offset[0.4]}, 
          Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}]}], ")"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970262295122*^9},
 CellTags->"FromTFi",
 CellLabel->"Out[787]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FromTFI", "[", 
  RowBox[{
   RowBox[{"TFI", "[", 
    RowBox[{"D", ",", 
     RowBox[{"SPD", "[", 
      RowBox[{"p", ",", "p"}], "]"}], ",", 
     RowBox[{"SOD", "[", "p", "]"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"1", ",", "0"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"1", ",", "M"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"1", ",", "0"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"1", ",", "m"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"1", ",", "0"}], "}"}]}], "}"}]}], "]"}], ",", "q1", ",", 
   "q2", ",", "p"}], "]"}]], "Input",
 CellTags->"FromTFi",
 CellLabel->"In[788]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"TFI", "(", 
   RowBox[{"D", ",", 
    SuperscriptBox["m", "2"], ",", 
    RowBox[{
     FormBox["\<\"\[CapitalDelta]\"\>",
      TraditionalForm], 
     FormBox["\<\"\[CenterDot]\"\>",
      TraditionalForm], 
     FormBox["p",
      TraditionalForm]}], ",", 
    RowBox[{"(", "\[NoBreak]", GridBox[{
       {"1", "0"},
       {"1", "M"},
       {"1", "0"},
       {"1", "m"},
       {"1", "0"}
      },
      GridBoxAlignment->{
       "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
        "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
      GridBoxSpacings->{"Columns" -> {
          Offset[0.27999999999999997`], {
           Offset[0.7]}, 
          Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
          Offset[0.2], {
           Offset[0.4]}, 
          Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}]}], ")"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.5009702624084797`*^9},
 CellTags->"FromTFi",
 CellLabel->"Out[788]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FromTFI", "[", 
  RowBox[{
   RowBox[{"TFI", "[", 
    RowBox[{"D", ",", 
     RowBox[{"SPD", "[", 
      RowBox[{"p", ",", "p"}], "]"}], ",", 
     RowBox[{"SOD", "[", "p", "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"0", ",", "1"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"1", ",", "m"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"1", ",", "M"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"1", ",", "0"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"1", ",", "m"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"1", ",", "0"}], "}"}]}], "}"}]}], "]"}], ",", "q1", ",", 
   "q2", ",", "p"}], "]"}]], "Input",
 CellTags->"FromTFi",
 CellLabel->"In[789]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"TFI", "(", 
   RowBox[{"D", ",", 
    SuperscriptBox["m", "2"], ",", 
    RowBox[{
     FormBox["\<\"\[CapitalDelta]\"\>",
      TraditionalForm], 
     FormBox["\<\"\[CenterDot]\"\>",
      TraditionalForm], 
     FormBox["p",
      TraditionalForm]}], ",", 
    RowBox[{"{", 
     RowBox[{"0", ",", "1"}], "}"}], ",", 
    RowBox[{"(", "\[NoBreak]", GridBox[{
       {"1", "m"},
       {"1", "M"},
       {"1", "0"},
       {"1", "m"},
       {"1", "0"}
      },
      GridBoxAlignment->{
       "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
        "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
      GridBoxSpacings->{"Columns" -> {
          Offset[0.27999999999999997`], {
           Offset[0.7]}, 
          Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
          Offset[0.2], {
           Offset[0.4]}, 
          Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}]}], ")"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970262496376*^9},
 CellTags->"FromTFi",
 CellLabel->"Out[789]="]
}, Open  ]]
}, Open  ]],

Cell["FUNCTION", "ObjectName",
 CellTags->"FUNCTION"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FUNCTION"],

Cell["\<\
FUNCTION[exp, string] is a head of an expression to be declared a function \
(of type string), if used in Write2.\
\>", "Text",
 CellTags->"FUNCTION"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Write2",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Write2",
  ButtonNote->"Write2"],
 "."
}], "Text",
 CellTags->"FUNCTION"]
}, Open  ]],

Cell["FunctionalD", "ObjectName",
 CellTags->"FunctionalD"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FunctionalD"],

Cell["\<\
FunctionalD[expr, {QuantumField[name, {mu}, {a}][p], ...}] calculates the \
functional derivative of expr with respect to the field list (with incoming \
momenta p, etc.) and does the fourier transform. FunctionalD[expr, \
{QuantumField[name, {mu},{a}], ...}] calculates the functional derivative and \
does partial integration but omits the x-space delta functions.\
\>", "Text",
 CellTags->"FunctionalD"],

Cell["FunctionalD is a low level function used in FeynRule.", "Text",
 CellTags->"FunctionalD"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FeynRule",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FeynRule",
  ButtonNote->"FeynRule"],
 ", ",
 ButtonBox["QuantumField",
  BaseStyle->"AddOnsLink",
  ButtonData:>"QuantumField",
  ButtonNote->"QuantumField"],
 "."
}], "Text",
 CellTags->"FunctionalD"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"FunctionalD"],

Cell[TextData[{
 "Instead of the usual ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"\[Delta]\[Phi]", "(", "x", ")"}], "/", 
     RowBox[{"\[Delta]\[Phi]", "(", "y", ")"}]}], "=", 
    RowBox[{
     SuperscriptBox["\[Delta]", 
      RowBox[{"(", "D", ")"}]], "(", 
     RowBox[{"x", "-", "y"}], ")"}]}], TraditionalForm]]],
 " the arguments and the \[Delta] function are omitted, i.e., for the program \
for simplicity: ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"\[Delta]\[Phi]", "/", "\[Delta]\[Phi]"}], "=", "1"}], 
   TraditionalForm]]]
}], "Text",
 CellTags->"FunctionalD"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FunctionalD", "[", 
  RowBox[{
   RowBox[{"QuantumField", "[", "\[Phi]", "]"}], ",", 
   RowBox[{"QuantumField", "[", "\[Phi]", "]"}]}], "]"}]], "Input",
 CellTags->"FunctionalD",
 CellLabel->"In[790]:="],

Cell[BoxData[
 FormBox["1", TraditionalForm]], "Output",
 CellChangeTimes->{3.500970262585135*^9},
 CellTags->"FunctionalD",
 CellLabel->"Out[790]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FunctionalD", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"QuantumField", "[", "\[Phi]", "]"}], "^", "2"}], ",", 
   RowBox[{"QuantumField", "[", "\[Phi]", "]"}]}], "]"}]], "Input",
 CellTags->"FunctionalD",
 CellLabel->"In[791]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"2", " ", "\[Phi]"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009702630957317`*^9},
 CellTags->"FunctionalD",
 CellLabel->"Out[791]="]
}, Open  ]],

Cell[TextData[{
 "Instead of the usual ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{"\[Delta]", " ", 
       RowBox[{
        SubscriptBox["\[PartialD]", "\[Mu]"], 
        RowBox[{"\[Phi]", "(", "x", ")"}]}]}], ")"}], "/", 
     RowBox[{"\[Delta]\[Phi]", "(", "y", ")"}]}], "=", 
    RowBox[{
     SubscriptBox["\[PartialD]", "\[Mu]"], 
     RowBox[{
      SuperscriptBox["\[Delta]", 
       RowBox[{"(", "D", ")"}]], "(", 
      RowBox[{"x", "-", "y"}], ")"}]}]}], TraditionalForm]]],
 " the arguments are omitted, and the ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["\[PartialD]", "\[Mu]"], TraditionalForm]]],
 "operator is specified by default to be an integration by parts operator, \
i.e., the right hand side will be just ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"-", 
     FormBox[
      SubscriptBox["\[PartialD]", "\[Mu]"],
      TraditionalForm]}], ","}], TraditionalForm]]],
 " or, more precisely (by default) ",
 Cell[BoxData[
  FormBox[
   RowBox[{"-", 
    SubscriptBox[
     OverscriptBox["\[PartialD]", "\[RightArrow]"], 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]]}], TraditionalForm]]],
 "."
}], "Text",
 ZeroWidthTimes->True,
 CellTags->"FunctionalD"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FunctionalD", "[", 
  RowBox[{
   RowBox[{"QuantumField", "[", 
    RowBox[{
     RowBox[{"PartialD", "[", "\[Mu]", "]"}], ",", "\[Phi]"}], "]"}], ",", 
   RowBox[{"QuantumField", "[", "\[Phi]", "]"}]}], "]"}]], "Input",
 CellTags->"FunctionalD",
 CellLabel->"In[792]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   SubscriptBox[
    OverscriptBox["\[PartialD]", "\[RightArrow]"], 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970263266322*^9},
 CellTags->"FunctionalD",
 CellLabel->"Out[792]="]
}, Open  ]],

Cell[TextData[Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"S", "[", "\[Phi]", "]"}], " ", "=", 
   RowBox[{
    RowBox[{"1", "/", "2"}], 
    RowBox[{"\[Integral]", " ", 
     RowBox[{
      SuperscriptBox["d", "D"], 
      RowBox[{"x", " ", "[", " ", 
       RowBox[{
        RowBox[{
         RowBox[{
          SubscriptBox["\[PartialD]", "\[Mu]"], 
          RowBox[{"\[Phi]", "(", "x", ")"}]}], " ", 
         RowBox[{
          SuperscriptBox["\[PartialD]", "\[Mu]"], 
          RowBox[{"\[Phi]", "(", "x", ")"}]}]}], "-", 
        RowBox[{
         SuperscriptBox["m", "2"], 
         RowBox[{"\[Phi]", "(", "x", ")"}], " ", 
         RowBox[{"\[Phi]", "(", "y", ")"}]}]}], "]"}]}]}]}]}], 
  TraditionalForm]]]], "Text",
 ZeroWidthTimes->True,
 CellTags->"FunctionalD"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"s", "[", "\[Phi]", "]"}], "=", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"QuantumField", "[", 
       RowBox[{
        RowBox[{"PartialD", "[", "\[Mu]", "]"}], ",", "\[Phi]"}], "]"}], ".", 
      
      RowBox[{"QuantumField", "[", 
       RowBox[{
        RowBox[{"PartialD", "[", "\[Mu]", "]"}], ",", "\[Phi]"}], "]"}]}], 
     "-", 
     RowBox[{
      RowBox[{"m", "^", "2"}], " ", 
      RowBox[{
       RowBox[{"QuantumField", "[", "\[Phi]", "]"}], ".", 
       RowBox[{"QuantumField", "[", "\[Phi]", "]"}]}]}]}], ")"}], "/", 
   "2"}]}]], "Input",
 CellTags->"FunctionalD",
 CellLabel->"In[793]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["1", "2"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"QuantumField", "(", 
       RowBox[{
        RowBox[{
        "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(",
          "\[Mu]", ")"}], ",", "\[Phi]"}], ")"}], ".", 
      RowBox[{"QuantumField", "(", 
       RowBox[{
        RowBox[{
        "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(",
          "\[Mu]", ")"}], ",", "\[Phi]"}], ")"}]}], "-", 
     RowBox[{
      SuperscriptBox["m", "2"], " ", 
      RowBox[{"\[Phi]", ".", "\[Phi]"}]}]}], ")"}]}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970263446607*^9},
 CellTags->"FunctionalD",
 CellLabel->"Out[793]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FunctionalD", "[", 
  RowBox[{
   RowBox[{"s", "[", "\[Phi]", "]"}], ",", 
   RowBox[{"QuantumField", "[", "\[Phi]", "]"}]}], "]"}]], "Input",
 CellTags->"FunctionalD",
 CellLabel->"In[794]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SuperscriptBox["m", "2"], " ", 
    RowBox[{"(", 
     RowBox[{"-", "\[Phi]"}], ")"}]}], "-", 
   RowBox[{"QuantumField", "(", 
    RowBox[{
     RowBox[{
     "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(",
       "\[Mu]", ")"}], ",", 
     RowBox[{
     "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
      FormBox["\[Mu]",
       TraditionalForm], ")"}], ",", "\[Phi]"}], ")"}]}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970263659402*^9},
 CellTags->"FunctionalD",
 CellLabel->"Out[794]="]
}, Open  ]],

Cell[TextData[Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"S", "[", "A", "]"}], " ", "=", " ", 
   RowBox[{"-", 
    RowBox[{"\[Integral]", " ", 
     RowBox[{
      SuperscriptBox["d", "D"], "x", " ", 
      FractionBox["1", "4"], " ", 
      RowBox[{
       SubsuperscriptBox["F", "a", "\[Mu]\[Nu]"], "(", "x", ")"}], " ", 
      RowBox[{
       SubscriptBox["F", "\[Mu]\[Nu]a"], "(", "x", ")"}]}]}]}]}], 
  TraditionalForm]]]], "Text",
 ZeroWidthTimes->True,
 CellTags->"FunctionalD"],

Cell[TextData[StyleBox["First approach",
 FontWeight->"Bold"]], "Text",
 CellTags->"FunctionalD"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"F1", "=", 
  RowBox[{"FieldStrength", "[", 
   RowBox[{"\[Mu]", ",", "\[Nu]", ",", "a", ",", 
    RowBox[{"{", 
     RowBox[{"A", ",", "b", ",", "c"}], "}"}], ",", "1", ",", 
    RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]}]], "Input",
 CellTags->"FunctionalD",
 CellLabel->"In[795]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"QuantumField", "(", 
    RowBox[{
     RowBox[{
     "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
      FormBox["\[Mu]",
       TraditionalForm], ")"}], ",", "A", ",", 
     FormBox["\[Nu]",
      TraditionalForm], ",", 
     FormBox["a",
      TraditionalForm]}], ")"}], "-", 
   RowBox[{"QuantumField", "(", 
    RowBox[{
     RowBox[{
     "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
      FormBox["\[Nu]",
       TraditionalForm], ")"}], ",", "A", ",", 
     FormBox["\[Mu]",
      TraditionalForm], ",", 
     FormBox["a",
      TraditionalForm]}], ")"}], "+", 
   RowBox[{
    SubscriptBox["f", 
     RowBox[{"a", "b", "c"}]], " ", 
    RowBox[{
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]], ".", 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm]]}]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009702639749947`*^9},
 CellTags->"FunctionalD",
 CellLabel->"Out[795]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"F2", "=", 
  RowBox[{"FieldStrength", "[", 
   RowBox[{"\[Mu]", ",", "\[Nu]", ",", "a", ",", 
    RowBox[{"{", 
     RowBox[{"A", ",", "d", ",", "e"}], "}"}], ",", "1", ",", 
    RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]}]], "Input",
 CellTags->"FunctionalD",
 CellLabel->"In[796]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"QuantumField", "(", 
    RowBox[{
     RowBox[{
     "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
      FormBox["\[Mu]",
       TraditionalForm], ")"}], ",", "A", ",", 
     FormBox["\[Nu]",
      TraditionalForm], ",", 
     FormBox["a",
      TraditionalForm]}], ")"}], "-", 
   RowBox[{"QuantumField", "(", 
    RowBox[{
     RowBox[{
     "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
      FormBox["\[Nu]",
       TraditionalForm], ")"}], ",", "A", ",", 
     FormBox["\[Mu]",
      TraditionalForm], ",", 
     FormBox["a",
      TraditionalForm]}], ")"}], "+", 
   RowBox[{
    SubscriptBox["f", 
     RowBox[{"a", "d", "e"}]], " ", 
    RowBox[{
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["d",
        TraditionalForm],
       TraditionalForm]], ".", 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["e",
        TraditionalForm],
       TraditionalForm]]}]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970264130692*^9},
 CellTags->"FunctionalD",
 CellLabel->"Out[796]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"S", "[", "A", "]"}], "=", 
  RowBox[{
   RowBox[{"-", 
    RowBox[{"1", "/", "4"}]}], 
   RowBox[{"F1", ".", "F2"}]}]}]], "Input",
 CellTags->"FunctionalD",
 CellLabel->"In[797]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    FractionBox["1", "4"]}], " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"QuantumField", "(", 
       RowBox[{
        RowBox[{
        "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
         FormBox["\[Mu]",
          TraditionalForm], ")"}], ",", "A", ",", 
        FormBox["\[Nu]",
         TraditionalForm], ",", 
        FormBox["a",
         TraditionalForm]}], ")"}], "-", 
      RowBox[{"QuantumField", "(", 
       RowBox[{
        RowBox[{
        "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
         FormBox["\[Nu]",
          TraditionalForm], ")"}], ",", "A", ",", 
        FormBox["\[Mu]",
         TraditionalForm], ",", 
        FormBox["a",
         TraditionalForm]}], ")"}], "+", 
      RowBox[{
       SubscriptBox["f", 
        RowBox[{"a", "b", "c"}]], " ", 
       RowBox[{
        SubsuperscriptBox["A", 
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["b",
           TraditionalForm],
          TraditionalForm]], ".", 
        SubsuperscriptBox["A", 
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["c",
           TraditionalForm],
          TraditionalForm]]}]}]}], ")"}], ".", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"QuantumField", "(", 
       RowBox[{
        RowBox[{
        "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
         FormBox["\[Mu]",
          TraditionalForm], ")"}], ",", "A", ",", 
        FormBox["\[Nu]",
         TraditionalForm], ",", 
        FormBox["a",
         TraditionalForm]}], ")"}], "-", 
      RowBox[{"QuantumField", "(", 
       RowBox[{
        RowBox[{
        "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
         FormBox["\[Nu]",
          TraditionalForm], ")"}], ",", "A", ",", 
        FormBox["\[Mu]",
         TraditionalForm], ",", 
        FormBox["a",
         TraditionalForm]}], ")"}], "+", 
      RowBox[{
       SubscriptBox["f", 
        RowBox[{"a", "d", "e"}]], " ", 
       RowBox[{
        SubsuperscriptBox["A", 
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["d",
           TraditionalForm],
          TraditionalForm]], ".", 
        SubsuperscriptBox["A", 
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["e",
           TraditionalForm],
          TraditionalForm]]}]}]}], ")"}]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970264353796*^9},
 CellTags->"FunctionalD",
 CellLabel->"Out[797]="]
}, Open  ]],

Cell[TextData[{
 "In order to derive the equation of motion the functional derivative of ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 " with respect to ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["A", "\[Sigma]", "g"], TraditionalForm]]],
 "has to be set to zero. Bearing in mind that for FeynCalc we have to be \
precise as to where which operators (coming from the substitution of the \
derivative of the \[Delta] function) act:, act with the functional derivative \
operator on the first field strength:"
}], "Text",
 CellTags->"FunctionalD"],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     FormBox[
      RowBox[{
       FormBox[
        RowBox[{
         FormBox[
          RowBox[{"0", " ", "=", " ", 
           RowBox[{
            FormBox[
             RowBox[{
              RowBox[{"(", "\[Delta]S", ")"}], "/", 
              RowBox[{"(", 
               RowBox[{"\[Delta]", 
                RowBox[{
                 SubsuperscriptBox["A", "\[Sigma]", "g"], "(", "y", ")"}]}], 
               ")"}]}],
             TraditionalForm], "=", 
            RowBox[{"-", 
             RowBox[{"2", "/", "4"}]}]}]}],
          TraditionalForm], 
         RowBox[{"\[Integral]", 
          RowBox[{
           SuperscriptBox["d", "D"], "x"}]}]}],
        TraditionalForm], " ", 
       RowBox[{"(", 
        RowBox[{"\[Delta]", "/", 
         RowBox[{"(", 
          RowBox[{"\[Delta]", 
           RowBox[{
            SubsuperscriptBox["A", "\[Sigma]", "g"], "(", "y", ")"}]}], 
          ")"}]}]}]}],
      TraditionalForm], " ", 
     RowBox[{
      SubscriptBox["F", "\[Mu]\[Nu]a"], "(", "x", ")"}]}], ")"}], 
   TraditionalForm]]],
 Cell[BoxData[
  FormBox[
   RowBox[{
    FormBox[
     RowBox[{" ", 
      SubsuperscriptBox["F", "a", 
       RowBox[{",", "\[Mu]\[Nu]"}]]}],
     TraditionalForm], 
    RowBox[{"(", "x", ")"}]}], TraditionalForm]]]
}], "Text",
 ZeroWidthTimes->True,
 CellTags->"FunctionalD"],

Cell[TextData[Cell[BoxData[
 FormBox[
  RowBox[{"See", " ", "what", " ", "happens", " ", "with", " ", "just", 
   RowBox[{
    FormBox[
     RowBox[{
      RowBox[{"(", 
       RowBox[{"\[Delta]", 
        RowBox[{"S", "[", "A", "]"}]}], ")"}], "/", 
      RowBox[{"(", 
       RowBox[{"\[Delta]", 
        SubsuperscriptBox["A", "\[Sigma]", "g"]}], ")"}]}],
     TraditionalForm], "."}]}], TraditionalForm]]]], "Text",
 CellTags->"FunctionalD"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Ag", "=", 
  RowBox[{"QuantumField", "[", 
   RowBox[{"A", ",", 
    RowBox[{"{", "\[Sigma]", "}"}], ",", 
    RowBox[{"{", "g", "}"}]}], "]"}]}]], "Input",
 CellTags->"FunctionalD",
 CellLabel->"In[798]:="],

Cell[BoxData[
 FormBox[
  SubsuperscriptBox["A", 
   FormBox[
    FormBox["\[Sigma]",
     TraditionalForm],
    TraditionalForm], 
   FormBox[
    FormBox["g",
     TraditionalForm],
    TraditionalForm]], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009702644429893`*^9},
 CellTags->"FunctionalD",
 CellLabel->"Out[798]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t1", "=", 
  RowBox[{"FunctionalD", "[", 
   RowBox[{"F1", ",", "Ag"}], "]"}]}]], "Input",
 CellTags->"FunctionalD",
 CellLabel->"In[799]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Nu]", "\[Sigma]"}]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]]}], "-", 
   RowBox[{
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Sigma]"}]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    RowBox[{"(", 
     RowBox[{"-", 
      SuperscriptBox["g", 
       RowBox[{"\[Nu]", "\[Sigma]"}]]}], ")"}], " ", 
    SubscriptBox[
     OverscriptBox["\[PartialD]", "\[RightArrow]"], 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]]}], "+", 
   RowBox[{
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Sigma]"}]], " ", 
    SubscriptBox[
     OverscriptBox["\[PartialD]", "\[RightArrow]"], 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm]]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970264685483*^9},
 CellTags->"FunctionalD",
 CellLabel->"Out[799]="]
}, Open  ]],

Cell["\<\
In order to minimize the number of dummy indices, replace b \[Rule] c.\
\>", "Text",
 CellTags->"FunctionalD"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t1", " ", "=", " ", 
  RowBox[{"t1", " ", "/.", " ", 
   RowBox[{"b", "\[Rule]", "c"}]}]}]], "Input",
 CellTags->"FunctionalD",
 CellLabel->"In[800]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Nu]", "\[Sigma]"}]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]]}], "-", 
   RowBox[{
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Sigma]"}]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    RowBox[{"(", 
     RowBox[{"-", 
      SuperscriptBox["g", 
       RowBox[{"\[Nu]", "\[Sigma]"}]]}], ")"}], " ", 
    SubscriptBox[
     OverscriptBox["\[PartialD]", "\[RightArrow]"], 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]]}], "+", 
   RowBox[{
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Sigma]"}]], " ", 
    SubscriptBox[
     OverscriptBox["\[PartialD]", "\[RightArrow]"], 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm]]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970264998806*^9},
 CellTags->"FunctionalD",
 CellLabel->"Out[800]="]
}, Open  ]],

Cell[TextData[{
 "Instead of inserting the definition for the second ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["F", "a", "\[Mu]\[Nu]"], TraditionalForm]]],
 ", introduce a QuantumField object with antisymmetry built into the Lorentz \
indices:"
}], "Text",
 CellTags->"FunctionalD"],

Cell[BoxData[
 RowBox[{"F", "/:", " ", 
  RowBox[{"QuantumField", "[", 
   RowBox[{
   "pard___", ",", "F", ",", "\[Beta]_", ",", "\[Alpha]_", ",", "s_"}], "]"}],
   ":=", 
  RowBox[{
   RowBox[{"-", 
    RowBox[{"QuantumField", "[", 
     RowBox[{"pard", ",", "F", ",", "\[Alpha]", ",", "\[Beta]", ",", "s"}], 
     "]"}]}], "/;", 
   RowBox[{"!", 
    RowBox[{"OrderedQ", "[", 
     RowBox[{"{", 
      RowBox[{"\[Beta]", ",", "\[Alpha]"}], "}"}], "]"}]}]}]}]], "Input",
 CellTags->"FunctionalD",
 CellLabel->"In[801]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"QuantumField", "[", 
  RowBox[{"F", ",", 
   RowBox[{"{", 
    RowBox[{"\[Mu]", ",", "\[Nu]"}], "}"}], ",", 
   RowBox[{"{", "a", "}"}]}], "]"}]], "Input",
 CellTags->"FunctionalD",
 CellLabel->"In[802]:="],

Cell[BoxData[
 FormBox[
  SubsuperscriptBox["F", 
   RowBox[{
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["\[Nu]",
      TraditionalForm],
     TraditionalForm]}], 
   FormBox[
    FormBox["a",
     TraditionalForm],
    TraditionalForm]], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009702651140347`*^9},
 CellTags->"FunctionalD",
 CellLabel->"Out[802]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "/.", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"\[Mu]", "\[RuleDelayed]", "\[Nu]"}], ",", 
    RowBox[{"\[Nu]", "\[RuleDelayed]", "\[Mu]"}]}], "}"}]}]], "Input",
 CellTags->"FunctionalD",
 CellLabel->"In[803]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   SubsuperscriptBox["F", 
    RowBox[{
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm]}], 
    FormBox[
     FormBox["a",
      TraditionalForm],
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.50097026519189*^9},
 CellTags->"FunctionalD",
 CellLabel->"Out[803]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t2", "=", 
  RowBox[{
   RowBox[{"Contract", "[", 
    RowBox[{"ExpandPartialD", "[", 
     RowBox[{
      RowBox[{"-", 
       RowBox[{"1", "/", "2"}]}], " ", 
      RowBox[{"t1", ".", 
       RowBox[{"QuantumField", "[", 
        RowBox[{"F", ",", 
         RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
         RowBox[{"LorentzIndex", "[", "\[Nu]", "]"}], ",", 
         RowBox[{"SUNIndex", "[", "a", "]"}]}], "]"}]}]}], "]"}], "]"}], "/.", 
   RowBox[{"Dot", "\[Rule]", "Times"}]}]}]], "Input",
 CellTags->"FunctionalD",
 CellLabel->"In[804]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    FractionBox["1", "2"], " ", 
    RowBox[{"QuantumField", "(", 
     RowBox[{
      RowBox[{
      "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
       FormBox["\[Mu]",
        TraditionalForm], ")"}], ",", "F", ",", 
      FormBox["\[Mu]",
       TraditionalForm], ",", 
      FormBox["\[Sigma]",
       TraditionalForm], ",", 
      FormBox["g",
       TraditionalForm]}], ")"}]}], "+", 
   RowBox[{
    FractionBox["1", "2"], " ", 
    RowBox[{"QuantumField", "(", 
     RowBox[{
      RowBox[{
      "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
       FormBox["\[Nu]",
        TraditionalForm], ")"}], ",", "F", ",", 
      FormBox["\[Nu]",
       TraditionalForm], ",", 
      FormBox["\[Sigma]",
       TraditionalForm], ",", 
      FormBox["g",
       TraditionalForm]}], ")"}]}], "-", 
   RowBox[{
    FractionBox["1", "2"], " ", 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubsuperscriptBox["F", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm]}], 
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm]]}], "-", 
   RowBox[{
    FractionBox["1", "2"], " ", 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubsuperscriptBox["F", 
     RowBox[{
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm]}], 
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm]]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970265469644*^9},
 CellTags->"FunctionalD",
 CellLabel->"Out[804]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t3", " ", "=", " ", 
  RowBox[{"t2", " ", "/.", " ", 
   RowBox[{"\[Nu]", "\[Rule]", "\[Mu]"}]}]}]], "Input",
 CellTags->"FunctionalD",
 CellLabel->"In[805]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"QuantumField", "(", 
    RowBox[{
     RowBox[{
     "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
      FormBox["\[Mu]",
       TraditionalForm], ")"}], ",", "F", ",", 
     FormBox["\[Mu]",
      TraditionalForm], ",", 
     FormBox["\[Sigma]",
      TraditionalForm], ",", 
     FormBox["g",
      TraditionalForm]}], ")"}], "-", 
   RowBox[{
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubsuperscriptBox["F", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm]}], 
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm]]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009702655500793`*^9},
 CellTags->"FunctionalD",
 CellLabel->"Out[805]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t4", "=", 
  RowBox[{
   RowBox[{"FCE", "[", "t3", "]"}], "/.", " ", 
   RowBox[{
    RowBox[{"SUNF", "[", 
     RowBox[{"a", ",", "c", ",", "g"}], "]"}], "\[Rule]", 
    RowBox[{"-", 
     RowBox[{"SUNF", "[", 
      RowBox[{"g", ",", "c", ",", "a"}], "]"}]}]}]}]}]], "Input",
 CellTags->"FunctionalD",
 CellLabel->"In[806]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"QuantumField", "(", 
    RowBox[{
     RowBox[{
     "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
      FormBox["\[Mu]",
       TraditionalForm], ")"}], ",", "F", ",", 
     FormBox["\[Mu]",
      TraditionalForm], ",", 
     FormBox["\[Sigma]",
      TraditionalForm], ",", 
     FormBox["g",
      TraditionalForm]}], ")"}], "+", 
   RowBox[{
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubscriptBox["f", 
     RowBox[{"g", "c", "a"}]], " ", 
    SubsuperscriptBox["F", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm]}], 
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm]]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970265694339*^9},
 CellTags->"FunctionalD",
 CellLabel->"Out[806]="]
}, Open  ]],

Cell[TextData[{
 "Since the variational derivative vanishes t4 implies that 0 = ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["D", "\[Mu]"], 
    SubsuperscriptBox["F", "g", "\[Mu]\[Sigma]"]}], TraditionalForm]]],
 " ."
}], "Text",
 CellTags->"FunctionalD"],

Cell[TextData[StyleBox["Second approach",
 FontWeight->"Bold"]], "Text",
 CellTags->"FunctionalD"],

Cell["\<\
It is of course also possible to do the functional deriviate on the S[A] with \
both field strength tensors inserted.\
\>", "Text",
 CellTags->"FunctionalD"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"S", "[", "A", "]"}]], "Input",
 CellTags->"FunctionalD",
 CellLabel->"In[807]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    FractionBox["1", "4"]}], " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"QuantumField", "(", 
       RowBox[{
        RowBox[{
        "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
         FormBox["\[Mu]",
          TraditionalForm], ")"}], ",", "A", ",", 
        FormBox["\[Nu]",
         TraditionalForm], ",", 
        FormBox["a",
         TraditionalForm]}], ")"}], "-", 
      RowBox[{"QuantumField", "(", 
       RowBox[{
        RowBox[{
        "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
         FormBox["\[Nu]",
          TraditionalForm], ")"}], ",", "A", ",", 
        FormBox["\[Mu]",
         TraditionalForm], ",", 
        FormBox["a",
         TraditionalForm]}], ")"}], "+", 
      RowBox[{
       SubscriptBox["f", 
        RowBox[{"a", "b", "c"}]], " ", 
       RowBox[{
        SubsuperscriptBox["A", 
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["b",
           TraditionalForm],
          TraditionalForm]], ".", 
        SubsuperscriptBox["A", 
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["c",
           TraditionalForm],
          TraditionalForm]]}]}]}], ")"}], ".", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"QuantumField", "(", 
       RowBox[{
        RowBox[{
        "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
         FormBox["\[Mu]",
          TraditionalForm], ")"}], ",", "A", ",", 
        FormBox["\[Nu]",
         TraditionalForm], ",", 
        FormBox["a",
         TraditionalForm]}], ")"}], "-", 
      RowBox[{"QuantumField", "(", 
       RowBox[{
        RowBox[{
        "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
         FormBox["\[Nu]",
          TraditionalForm], ")"}], ",", "A", ",", 
        FormBox["\[Mu]",
         TraditionalForm], ",", 
        FormBox["a",
         TraditionalForm]}], ")"}], "+", 
      RowBox[{
       SubscriptBox["f", 
        RowBox[{"a", "d", "e"}]], " ", 
       RowBox[{
        SubsuperscriptBox["A", 
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["d",
           TraditionalForm],
          TraditionalForm]], ".", 
        SubsuperscriptBox["A", 
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["e",
           TraditionalForm],
          TraditionalForm]]}]}]}], ")"}]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970265851471*^9},
 CellTags->"FunctionalD",
 CellLabel->"Out[807]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"r1", "=", 
  RowBox[{"FunctionalD", "[", 
   RowBox[{
    RowBox[{"S", "[", "A", "]"}], ",", "Ag"}], "]"}]}]], "Input",
 CellTags->"FunctionalD",
 CellLabel->"In[808]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    RowBox[{
     FractionBox["1", "4"], " ", 
     SubscriptBox["f", 
      RowBox[{
       FormBox[
        FormBox["a",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["b",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["g",
         TraditionalForm],
        TraditionalForm]}]], " ", 
     RowBox[{
      SubsuperscriptBox["A", 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["b",
         TraditionalForm],
        TraditionalForm]], ".", 
      RowBox[{"QuantumField", "(", 
       RowBox[{
        RowBox[{
        "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
         FormBox["\[Mu]",
          TraditionalForm], ")"}], ",", "A", ",", 
        FormBox["\[Sigma]",
         TraditionalForm], ",", 
        FormBox["a",
         TraditionalForm]}], ")"}]}]}]}], "+", 
   RowBox[{
    FractionBox["1", "4"], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    RowBox[{
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]], ".", 
     RowBox[{"QuantumField", "(", 
      RowBox[{
       RowBox[{
       "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
        FormBox["\[Sigma]",
         TraditionalForm], ")"}], ",", "A", ",", 
       FormBox["\[Mu]",
        TraditionalForm], ",", 
       FormBox["a",
        TraditionalForm]}], ")"}]}]}], "-", 
   RowBox[{
    FractionBox["1", "4"], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    RowBox[{
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm]], ".", 
     RowBox[{"QuantumField", "(", 
      RowBox[{
       RowBox[{
       "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
        FormBox["\[Nu]",
         TraditionalForm], ")"}], ",", "A", ",", 
       FormBox["\[Sigma]",
        TraditionalForm], ",", 
       FormBox["a",
        TraditionalForm]}], ")"}]}]}], "+", 
   RowBox[{
    FractionBox["1", "4"], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    RowBox[{
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm]], ".", 
     RowBox[{"QuantumField", "(", 
      RowBox[{
       RowBox[{
       "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
        FormBox["\[Sigma]",
         TraditionalForm], ")"}], ",", "A", ",", 
       FormBox["\[Nu]",
        TraditionalForm], ",", 
       FormBox["a",
        TraditionalForm]}], ")"}]}]}], "-", 
   RowBox[{
    FractionBox["1", "4"], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["d",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    RowBox[{
     RowBox[{"QuantumField", "(", 
      RowBox[{
       RowBox[{
       "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
        FormBox["\[Mu]",
         TraditionalForm], ")"}], ",", "A", ",", 
       FormBox["\[Sigma]",
        TraditionalForm], ",", 
       FormBox["a",
        TraditionalForm]}], ")"}], ".", 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["d",
        TraditionalForm],
       TraditionalForm]]}]}], "+", 
   RowBox[{
    FractionBox["1", "4"], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["d",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    RowBox[{
     RowBox[{"QuantumField", "(", 
      RowBox[{
       RowBox[{
       "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
        FormBox["\[Sigma]",
         TraditionalForm], ")"}], ",", "A", ",", 
       FormBox["\[Mu]",
        TraditionalForm], ",", 
       FormBox["a",
        TraditionalForm]}], ")"}], ".", 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["d",
        TraditionalForm],
       TraditionalForm]]}]}], "-", 
   RowBox[{
    FractionBox["1", "4"], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["e",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    RowBox[{
     RowBox[{"QuantumField", "(", 
      RowBox[{
       RowBox[{
       "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
        FormBox["\[Nu]",
         TraditionalForm], ")"}], ",", "A", ",", 
       FormBox["\[Sigma]",
        TraditionalForm], ",", 
       FormBox["a",
        TraditionalForm]}], ")"}], ".", 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["e",
        TraditionalForm],
       TraditionalForm]]}]}], "+", 
   RowBox[{
    FractionBox["1", "4"], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["e",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    RowBox[{
     RowBox[{"QuantumField", "(", 
      RowBox[{
       RowBox[{
       "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
        FormBox["\[Sigma]",
         TraditionalForm], ")"}], ",", "A", ",", 
       FormBox["\[Nu]",
        TraditionalForm], ",", 
       FormBox["a",
        TraditionalForm]}], ")"}], ".", 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["e",
        TraditionalForm],
       TraditionalForm]]}]}], "+", 
   RowBox[{
    FractionBox["1", "4"], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    RowBox[{
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]], ".", 
     RowBox[{"QuantumField", "(", 
      RowBox[{
       RowBox[{
       "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
        FormBox["\[Mu]",
         TraditionalForm], ")"}], ",", "A", ",", 
       FormBox["\[Sigma]",
        TraditionalForm], ",", 
       FormBox["c",
        TraditionalForm]}], ")"}]}]}], "+", 
   RowBox[{
    FractionBox["1", "4"], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    RowBox[{
     RowBox[{"QuantumField", "(", 
      RowBox[{
       RowBox[{
       "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
        FormBox["\[Mu]",
         TraditionalForm], ")"}], ",", "A", ",", 
       FormBox["\[Mu]",
        TraditionalForm], ",", 
       FormBox["b",
        TraditionalForm]}], ")"}], ".", 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm]]}]}], "-", 
   RowBox[{
    FractionBox["1", "4"], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    RowBox[{
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]], ".", 
     RowBox[{"QuantumField", "(", 
      RowBox[{
       RowBox[{
       "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
        FormBox["\[Nu]",
         TraditionalForm], ")"}], ",", "A", ",", 
       FormBox["\[Nu]",
        TraditionalForm], ",", 
       FormBox["c",
        TraditionalForm]}], ")"}]}]}], "-", 
   RowBox[{
    FractionBox["1", "4"], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    RowBox[{
     RowBox[{"QuantumField", "(", 
      RowBox[{
       RowBox[{
       "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
        FormBox["\[Nu]",
         TraditionalForm], ")"}], ",", "A", ",", 
       FormBox["\[Sigma]",
        TraditionalForm], ",", 
       FormBox["b",
        TraditionalForm]}], ")"}], ".", 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm]]}]}], "+", 
   RowBox[{
    FractionBox["1", "4"], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["d",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["e",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    RowBox[{
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["d",
        TraditionalForm],
       TraditionalForm]], ".", 
     RowBox[{"QuantumField", "(", 
      RowBox[{
       RowBox[{
       "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
        FormBox["\[Mu]",
         TraditionalForm], ")"}], ",", "A", ",", 
       FormBox["\[Sigma]",
        TraditionalForm], ",", 
       FormBox["e",
        TraditionalForm]}], ")"}]}]}], "+", 
   RowBox[{
    FractionBox["1", "4"], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["d",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["e",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    RowBox[{
     RowBox[{"QuantumField", "(", 
      RowBox[{
       RowBox[{
       "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
        FormBox["\[Mu]",
         TraditionalForm], ")"}], ",", "A", ",", 
       FormBox["\[Mu]",
        TraditionalForm], ",", 
       FormBox["d",
        TraditionalForm]}], ")"}], ".", 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["e",
        TraditionalForm],
       TraditionalForm]]}]}], "-", 
   RowBox[{
    FractionBox["1", "4"], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["d",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["e",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    RowBox[{
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["d",
        TraditionalForm],
       TraditionalForm]], ".", 
     RowBox[{"QuantumField", "(", 
      RowBox[{
       RowBox[{
       "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
        FormBox["\[Nu]",
         TraditionalForm], ")"}], ",", "A", ",", 
       FormBox["\[Nu]",
        TraditionalForm], ",", 
       FormBox["e",
        TraditionalForm]}], ")"}]}]}], "-", 
   RowBox[{
    FractionBox["1", "4"], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["d",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["e",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    RowBox[{
     RowBox[{"QuantumField", "(", 
      RowBox[{
       RowBox[{
       "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
        FormBox["\[Nu]",
         TraditionalForm], ")"}], ",", "A", ",", 
       FormBox["\[Sigma]",
        TraditionalForm], ",", 
       FormBox["d",
        TraditionalForm]}], ")"}], ".", 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["e",
        TraditionalForm],
       TraditionalForm]]}]}], "+", 
   RowBox[{
    FractionBox["1", "2"], " ", 
    RowBox[{"QuantumField", "(", 
     RowBox[{
      RowBox[{
      "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
       FormBox["\[Mu]",
        TraditionalForm], ")"}], ",", 
      RowBox[{
      "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
       FormBox["\[Mu]",
        TraditionalForm], ")"}], ",", "A", ",", 
      FormBox["\[Sigma]",
       TraditionalForm], ",", 
      FormBox["g",
       TraditionalForm]}], ")"}]}], "-", 
   RowBox[{
    FractionBox["1", "2"], " ", 
    RowBox[{"QuantumField", "(", 
     RowBox[{
      RowBox[{
      "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
       FormBox["\[Mu]",
        TraditionalForm], ")"}], ",", 
      RowBox[{
      "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
       FormBox["\[Sigma]",
        TraditionalForm], ")"}], ",", "A", ",", 
      FormBox["\[Mu]",
       TraditionalForm], ",", 
      FormBox["g",
       TraditionalForm]}], ")"}]}], "+", 
   RowBox[{
    FractionBox["1", "2"], " ", 
    RowBox[{"QuantumField", "(", 
     RowBox[{
      RowBox[{
      "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
       FormBox["\[Nu]",
        TraditionalForm], ")"}], ",", 
      RowBox[{
      "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
       FormBox["\[Nu]",
        TraditionalForm], ")"}], ",", "A", ",", 
      FormBox["\[Sigma]",
       TraditionalForm], ",", 
      FormBox["g",
       TraditionalForm]}], ")"}]}], "-", 
   RowBox[{
    FractionBox["1", "2"], " ", 
    RowBox[{"QuantumField", "(", 
     RowBox[{
      RowBox[{
      "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
       FormBox["\[Nu]",
        TraditionalForm], ")"}], ",", 
      RowBox[{
      "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
       FormBox["\[Sigma]",
        TraditionalForm], ")"}], ",", "A", ",", 
      FormBox["\[Nu]",
       TraditionalForm], ",", 
      FormBox["g",
       TraditionalForm]}], ")"}]}], "-", 
   RowBox[{
    FractionBox["1", "4"], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["d",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    RowBox[{
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]], ".", 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm]], ".", 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["d",
        TraditionalForm],
       TraditionalForm]]}]}], "+", 
   RowBox[{
    FractionBox["1", "4"], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["e",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    RowBox[{
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]], ".", 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm]], ".", 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["e",
        TraditionalForm],
       TraditionalForm]]}]}], "-", 
   RowBox[{
    FractionBox["1", "4"], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["d",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["e",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    RowBox[{
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]], ".", 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["d",
        TraditionalForm],
       TraditionalForm]], ".", 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["e",
        TraditionalForm],
       TraditionalForm]]}]}], "+", 
   RowBox[{
    FractionBox["1", "4"], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["d",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["e",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    RowBox[{
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm]], ".", 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["d",
        TraditionalForm],
       TraditionalForm]], ".", 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["e",
        TraditionalForm],
       TraditionalForm]]}]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009702667534723`*^9},
 CellTags->"FunctionalD",
 CellLabel->"Out[808]="]
}, Open  ]],

Cell["\<\
This is just funcional derivation and partial integration and simple \
contraction of indices. No attempt is made to rename dummy indices (since \
this is difficult in general).\
\>", "Text",
 CellTags->"FunctionalD"],

Cell["\<\
With a general replacement rule only valid for commuting fields the color \
indices can be canonicalized a bit more. The idea is to use the commutative \
properties of the vector fields, and canonicalize the color indices by a \
trick. This function will work on any commuting product of fields.\
\>", "Text",
 CellTags->"FunctionalD"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", "symfun", "]"}], ";"}]], "Input",
 CellTags->"FunctionalD",
 CellLabel->"In[809]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"symfun", "[", 
   RowBox[{"z_", ",", "fieldname_Symbol"}], "]"}], ":=", 
  RowBox[{"Expand", "[", 
   RowBox[{"SUNSimplify", "[", 
    RowBox[{"FixedPoint", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"Collect2", "[", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{"DotSimplify", "[", 
            RowBox[{"#1", "/.", 
             RowBox[{"Times", "\[Rule]", "Dot"}]}], "]"}], "/.", 
           RowBox[{
            RowBox[{"(", 
             RowBox[{
              RowBox[{"(", 
               RowBox[{
                RowBox[{"(", "qi___", ")"}], ".", 
                RowBox[{"QuantumField", "[", 
                 RowBox[{
                 "par1___", ",", "fieldname", ",", "li1_", ",", "sui1_"}], 
                 "]"}], ".", 
                RowBox[{"QuantumField", "[", 
                 RowBox[{
                 "par2___", ",", "fieldname", ",", "li2_", ",", "sui2_"}], 
                 "]"}], ".", "qf___"}], " ", ")"}], "any_"}], ")"}], 
            "\[RuleDelayed]", 
            RowBox[{
             RowBox[{"(", 
              RowBox[{
               RowBox[{"(", 
                RowBox[{"qi", ".", 
                 RowBox[{"QuantumField", "[", 
                  RowBox[{"par1", ",", "A", ",", "li1", ",", "sui2"}], "]"}], 
                 ".", 
                 RowBox[{"QuantumField", "[", 
                  RowBox[{"par2", ",", "fieldname", ",", "li2", ",", "sui1"}],
                   "]"}], ".", "qf"}], " ", ")"}], 
               RowBox[{"(", 
                RowBox[{"any", "/.", 
                 RowBox[{"{", 
                  RowBox[{
                   RowBox[{"sui1", "\[RuleDelayed]", "sui2"}], ",", 
                   RowBox[{"sui2", "\[RuleDelayed]", "sui1"}]}], "}"}]}], 
                ")"}]}], ")"}], "/;", 
             RowBox[{
              RowBox[{"(", 
               RowBox[{"!", 
                RowBox[{"(", 
                 RowBox[{"FreeQ2", "[", 
                  RowBox[{"any", ",", 
                   RowBox[{"{", 
                    RowBox[{"sui1", ",", "su2"}], "}"}]}], "]"}], ")"}]}], 
               ")"}], "&&", 
              RowBox[{"!", 
               RowBox[{"(", 
                RowBox[{"OrderedQ", "[", 
                 RowBox[{"{", 
                  RowBox[{"sui1", ",", "sui2"}], "}"}], "]"}], 
                ")"}]}]}]}]}]}], "/.", 
          RowBox[{"Dot", "\[Rule]", "Times"}]}], ",", "QuantumField"}], "]"}],
        "&"}], ",", "z", ",", "42"}], "]"}], "]"}], "]"}]}]], "Input",
 CellTags->"FunctionalD",
 CellLabel->"In[810]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"r2", " ", "=", " ", 
  RowBox[{"symfun", "[", 
   RowBox[{"r1", ",", "A"}], "]"}]}]], "Input",
 CellTags->"FunctionalD",
 CellLabel->"In[811]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    RowBox[{
     FractionBox["1", "2"], " ", 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm]], " ", 
     SubscriptBox["f", 
      RowBox[{
       FormBox[
        FormBox["a",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["b",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["g",
         TraditionalForm],
        TraditionalForm]}]], " ", 
     RowBox[{"QuantumField", "(", 
      RowBox[{
       RowBox[{
       "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
        FormBox["\[Mu]",
         TraditionalForm], ")"}], ",", "A", ",", 
       FormBox["\[Mu]",
        TraditionalForm], ",", 
       FormBox["b",
        TraditionalForm]}], ")"}]}]}], "+", 
   RowBox[{
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    RowBox[{"QuantumField", "(", 
     RowBox[{
      RowBox[{
      "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
       FormBox["\[Mu]",
        TraditionalForm], ")"}], ",", "A", ",", 
      FormBox["\[Sigma]",
       TraditionalForm], ",", 
      FormBox["b",
       TraditionalForm]}], ")"}]}], "-", 
   RowBox[{
    FractionBox["1", "2"], " ", 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    RowBox[{"QuantumField", "(", 
     RowBox[{
      RowBox[{
      "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
       FormBox["\[Sigma]",
        TraditionalForm], ")"}], ",", "A", ",", 
      FormBox["\[Mu]",
       TraditionalForm], ",", 
      FormBox["b",
       TraditionalForm]}], ")"}]}], "-", 
   RowBox[{
    FractionBox["1", "2"], " ", 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Sigma]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    RowBox[{"QuantumField", "(", 
     RowBox[{
      RowBox[{
      "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
       FormBox["\[Nu]",
        TraditionalForm], ")"}], ",", "A", ",", 
      FormBox["\[Nu]",
       TraditionalForm], ",", 
      FormBox["b",
       TraditionalForm]}], ")"}]}], "+", 
   RowBox[{
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    RowBox[{"QuantumField", "(", 
     RowBox[{
      RowBox[{
      "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
       FormBox["\[Nu]",
        TraditionalForm], ")"}], ",", "A", ",", 
      FormBox["\[Sigma]",
       TraditionalForm], ",", 
      FormBox["b",
       TraditionalForm]}], ")"}]}], "-", 
   RowBox[{
    FractionBox["1", "2"], " ", 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    RowBox[{"QuantumField", "(", 
     RowBox[{
      RowBox[{
      "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
       FormBox["\[Sigma]",
        TraditionalForm], ")"}], ",", "A", ",", 
      FormBox["\[Nu]",
       TraditionalForm], ",", 
      FormBox["b",
       TraditionalForm]}], ")"}]}], "+", 
   RowBox[{
    FractionBox["1", "2"], " ", 
    RowBox[{"QuantumField", "(", 
     RowBox[{
      RowBox[{
      "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
       FormBox["\[Mu]",
        TraditionalForm], ")"}], ",", 
      RowBox[{
      "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
       FormBox["\[Mu]",
        TraditionalForm], ")"}], ",", "A", ",", 
      FormBox["\[Sigma]",
       TraditionalForm], ",", 
      FormBox["g",
       TraditionalForm]}], ")"}]}], "-", 
   RowBox[{
    FractionBox["1", "2"], " ", 
    RowBox[{"QuantumField", "(", 
     RowBox[{
      RowBox[{
      "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
       FormBox["\[Mu]",
        TraditionalForm], ")"}], ",", 
      RowBox[{
      "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
       FormBox["\[Sigma]",
        TraditionalForm], ")"}], ",", "A", ",", 
      FormBox["\[Mu]",
       TraditionalForm], ",", 
      FormBox["g",
       TraditionalForm]}], ")"}]}], "+", 
   RowBox[{
    FractionBox["1", "2"], " ", 
    RowBox[{"QuantumField", "(", 
     RowBox[{
      RowBox[{
      "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
       FormBox["\[Nu]",
        TraditionalForm], ")"}], ",", 
      RowBox[{
      "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
       FormBox["\[Nu]",
        TraditionalForm], ")"}], ",", "A", ",", 
      FormBox["\[Sigma]",
       TraditionalForm], ",", 
      FormBox["g",
       TraditionalForm]}], ")"}]}], "-", 
   RowBox[{
    FractionBox["1", "2"], " ", 
    RowBox[{"QuantumField", "(", 
     RowBox[{
      RowBox[{
      "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
       FormBox["\[Nu]",
        TraditionalForm], ")"}], ",", 
      RowBox[{
      "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
       FormBox["\[Sigma]",
        TraditionalForm], ")"}], ",", "A", ",", 
      FormBox["\[Nu]",
       TraditionalForm], ",", 
      FormBox["g",
       TraditionalForm]}], ")"}]}], "-", 
   RowBox[{
    FractionBox["1", "4"], " ", 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Sigma]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["e",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["e",
        TraditionalForm],
       TraditionalForm]}]]}], "-", 
   RowBox[{
    FractionBox["1", "4"], " ", 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Sigma]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["e",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["e",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]]}], "-", 
   RowBox[{
    FractionBox["1", "2"], " ", 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Sigma]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["e",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["e",
        TraditionalForm],
       TraditionalForm]}]]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009702673226357`*^9},
 CellTags->"FunctionalD",
 CellLabel->"Out[811]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"r3", "=", 
  RowBox[{"r2", "/.", 
   RowBox[{"\[Nu]", "\[Rule]", "\[Mu]"}]}]}]], "Input",
 CellTags->"FunctionalD",
 CellLabel->"In[812]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    RowBox[{
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm]], " ", 
     SubscriptBox["f", 
      RowBox[{
       FormBox[
        FormBox["a",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["b",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["g",
         TraditionalForm],
        TraditionalForm]}]], " ", 
     RowBox[{"QuantumField", "(", 
      RowBox[{
       RowBox[{
       "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
        FormBox["\[Mu]",
         TraditionalForm], ")"}], ",", "A", ",", 
       FormBox["\[Mu]",
        TraditionalForm], ",", 
       FormBox["b",
        TraditionalForm]}], ")"}]}]}], "+", 
   RowBox[{"2", " ", 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    RowBox[{"QuantumField", "(", 
     RowBox[{
      RowBox[{
      "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
       FormBox["\[Mu]",
        TraditionalForm], ")"}], ",", "A", ",", 
      FormBox["\[Sigma]",
       TraditionalForm], ",", 
      FormBox["b",
       TraditionalForm]}], ")"}]}], "-", 
   RowBox[{
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    RowBox[{"QuantumField", "(", 
     RowBox[{
      RowBox[{
      "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
       FormBox["\[Sigma]",
        TraditionalForm], ")"}], ",", "A", ",", 
      FormBox["\[Mu]",
       TraditionalForm], ",", 
      FormBox["b",
       TraditionalForm]}], ")"}]}], "+", 
   RowBox[{"QuantumField", "(", 
    RowBox[{
     RowBox[{
     "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
      FormBox["\[Mu]",
       TraditionalForm], ")"}], ",", 
     RowBox[{
     "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
      FormBox["\[Mu]",
       TraditionalForm], ")"}], ",", "A", ",", 
     FormBox["\[Sigma]",
      TraditionalForm], ",", 
     FormBox["g",
      TraditionalForm]}], ")"}], "-", 
   RowBox[{"QuantumField", "(", 
    RowBox[{
     RowBox[{
     "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
      FormBox["\[Mu]",
       TraditionalForm], ")"}], ",", 
     RowBox[{
     "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
      FormBox["\[Sigma]",
       TraditionalForm], ")"}], ",", "A", ",", 
     FormBox["\[Mu]",
      TraditionalForm], ",", 
     FormBox["g",
      TraditionalForm]}], ")"}], "-", 
   RowBox[{
    FractionBox["3", "4"], " ", 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Sigma]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["e",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["e",
        TraditionalForm],
       TraditionalForm]}]]}], "-", 
   RowBox[{
    FractionBox["1", "4"], " ", 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Sigma]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["e",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["e",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970267754949*^9},
 CellTags->"FunctionalD",
 CellLabel->"Out[812]="]
}, Open  ]],

Cell[TextData[{
 "Inspection reveals that still terms are the same. Gather the terms with two \
",
 Cell[BoxData[
  FormBox[
   RowBox[{"f", " "}], TraditionalForm]]],
 "'s:"
}], "Text",
 CellTags->"FunctionalD"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"twof", "=", 
  RowBox[{"Select", "[", 
   RowBox[{"r3", ",", 
    RowBox[{
     RowBox[{
      RowBox[{"Count", "[", 
       RowBox[{"#", ",", 
        RowBox[{"SUNF", "[", "__", "]"}]}], "]"}], "===", "2"}], "&"}]}], 
   "]"}]}]], "Input",
 CellTags->"FunctionalD",
 CellLabel->"In[813]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    RowBox[{
     FractionBox["3", "4"], " ", 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]], " ", 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm]], " ", 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["e",
        TraditionalForm],
       TraditionalForm]], " ", 
     SubscriptBox["f", 
      RowBox[{
       FormBox[
        FormBox["a",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["b",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["g",
         TraditionalForm],
        TraditionalForm]}]], " ", 
     SubscriptBox["f", 
      RowBox[{
       FormBox[
        FormBox["a",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["c",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["e",
         TraditionalForm],
        TraditionalForm]}]]}]}], "-", 
   RowBox[{
    FractionBox["1", "4"], " ", 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Sigma]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["e",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["e",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009702679450274`*^9},
 CellTags->"FunctionalD",
 CellLabel->"Out[813]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"twofnew", "=", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{"twof", "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}], 
     "+", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"twof", "[", 
        RowBox[{"[", "2", "]"}], "]"}], "/.", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"b", "\[RuleDelayed]", "c"}], ",", 
         RowBox[{"c", "\[RuleDelayed]", "b"}]}], "}"}]}], ")"}]}], ")"}], "/.", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"a", "\[RuleDelayed]", "c"}], ",", 
     RowBox[{"c", "\[RuleDelayed]", "a"}]}], "}"}]}]}]], "Input",
 CellTags->"FunctionalD",
 CellLabel->"In[814]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm]]}], " ", 
   SubsuperscriptBox["A", 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["b",
      TraditionalForm],
     TraditionalForm]], " ", 
   SubsuperscriptBox["A", 
    FormBox[
     FormBox["\[Sigma]",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["e",
      TraditionalForm],
     TraditionalForm]], " ", 
   SubscriptBox["f", 
    RowBox[{
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["e",
       TraditionalForm],
      TraditionalForm]}]], " ", 
   SubscriptBox["f", 
    RowBox[{
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["g",
       TraditionalForm],
      TraditionalForm]}]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970268093346*^9},
 CellTags->"FunctionalD",
 CellLabel->"Out[814]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"r4", " ", "=", " ", 
  RowBox[{"r3", "-", "twof", "+", "twofnew"}]}]], "Input",
 CellTags->"FunctionalD",
 CellLabel->"In[815]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    RowBox[{
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm]], " ", 
     SubscriptBox["f", 
      RowBox[{
       FormBox[
        FormBox["a",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["b",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["g",
         TraditionalForm],
        TraditionalForm]}]], " ", 
     RowBox[{"QuantumField", "(", 
      RowBox[{
       RowBox[{
       "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
        FormBox["\[Mu]",
         TraditionalForm], ")"}], ",", "A", ",", 
       FormBox["\[Mu]",
        TraditionalForm], ",", 
       FormBox["b",
        TraditionalForm]}], ")"}]}]}], "+", 
   RowBox[{"2", " ", 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    RowBox[{"QuantumField", "(", 
     RowBox[{
      RowBox[{
      "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
       FormBox["\[Mu]",
        TraditionalForm], ")"}], ",", "A", ",", 
      FormBox["\[Sigma]",
       TraditionalForm], ",", 
      FormBox["b",
       TraditionalForm]}], ")"}]}], "-", 
   RowBox[{
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    RowBox[{"QuantumField", "(", 
     RowBox[{
      RowBox[{
      "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
       FormBox["\[Sigma]",
        TraditionalForm], ")"}], ",", "A", ",", 
      FormBox["\[Mu]",
       TraditionalForm], ",", 
      FormBox["b",
       TraditionalForm]}], ")"}]}], "+", 
   RowBox[{"QuantumField", "(", 
    RowBox[{
     RowBox[{
     "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
      FormBox["\[Mu]",
       TraditionalForm], ")"}], ",", 
     RowBox[{
     "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
      FormBox["\[Mu]",
       TraditionalForm], ")"}], ",", "A", ",", 
     FormBox["\[Sigma]",
      TraditionalForm], ",", 
     FormBox["g",
      TraditionalForm]}], ")"}], "-", 
   RowBox[{"QuantumField", "(", 
    RowBox[{
     RowBox[{
     "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
      FormBox["\[Mu]",
       TraditionalForm], ")"}], ",", 
     RowBox[{
     "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
      FormBox["\[Sigma]",
       TraditionalForm], ")"}], ",", "A", ",", 
     FormBox["\[Mu]",
      TraditionalForm], ",", 
     FormBox["g",
      TraditionalForm]}], ")"}], "-", 
   RowBox[{
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Sigma]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["e",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["e",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009702682904263`*^9},
 CellTags->"FunctionalD",
 CellLabel->"Out[815]="]
}, Open  ]],

Cell["\<\
Check that this is now indeed the same as the t4 result from the first \
attempt.\
\>", "Text",
 CellTags->"FunctionalD"],

Cell[CellGroupData[{

Cell[BoxData["t4"], "Input",
 CellTags->"FunctionalD",
 CellLabel->"In[816]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"QuantumField", "(", 
    RowBox[{
     RowBox[{
     "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
      FormBox["\[Mu]",
       TraditionalForm], ")"}], ",", "F", ",", 
     FormBox["\[Mu]",
      TraditionalForm], ",", 
     FormBox["\[Sigma]",
      TraditionalForm], ",", 
     FormBox["g",
      TraditionalForm]}], ")"}], "+", 
   RowBox[{
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubscriptBox["f", 
     RowBox[{"g", "c", "a"}]], " ", 
    SubsuperscriptBox["F", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm]}], 
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm]]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970268420341*^9},
 CellTags->"FunctionalD",
 CellLabel->"Out[816]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"w0", "=", 
  RowBox[{
   RowBox[{
    RowBox[{"RightPartialD", "[", "\[Mu]", "]"}], ".", 
    RowBox[{"FieldStrength", "[", 
     RowBox[{"\[Mu]", ",", "\[Sigma]", ",", "g", ",", 
      RowBox[{"{", 
       RowBox[{"A", ",", "a", ",", "b"}], "}"}], ",", "1"}], "]"}]}], "+", 
   RowBox[{
    RowBox[{"QuantumField", "[", 
     RowBox[{"A", ",", 
      RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
      RowBox[{"SUNIndex", "[", "c", "]"}]}], "]"}], 
    RowBox[{"FieldStrength", "[", 
     RowBox[{"\[Mu]", ",", "\[Sigma]", ",", "a", ",", 
      RowBox[{"{", 
       RowBox[{"A", ",", "b", ",", "d"}], "}"}], ",", "1"}], "]"}], 
    RowBox[{"SUNF", "[", 
     RowBox[{"g", ",", "c", ",", "a"}], "]"}]}]}]}]], "Input",
 CellTags->"FunctionalD",
 CellLabel->"In[817]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubscriptBox["f", 
     RowBox[{"g", "c", "a"}]], " ", 
    SubsuperscriptBox["F", 
     RowBox[{"\[Mu]", "\[Sigma]"}], 
     RowBox[{"a", 
      FormBox[
       RowBox[{"{", 
        RowBox[{"A", ",", "b", ",", "d"}], "}"}],
       TraditionalForm], 
      FormBox["1",
       TraditionalForm]}]]}], "+", 
   RowBox[{
    SubscriptBox[
     OverscriptBox["\[PartialD]", "\[RightArrow]"], 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], ".", 
    SubsuperscriptBox["F", 
     RowBox[{"\[Mu]", "\[Sigma]"}], 
     RowBox[{"g", 
      FormBox[
       RowBox[{"{", 
        RowBox[{"A", ",", "a", ",", "b"}], "}"}],
       TraditionalForm], 
      FormBox["1",
       TraditionalForm]}]]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009702686338*^9},
 CellTags->"FunctionalD",
 CellLabel->"Out[817]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"w1", "=", 
  RowBox[{"Explicit", "[", "w0", "]"}]}]], "Input",
 CellTags->"FunctionalD",
 CellLabel->"In[818]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox[
     OverscriptBox["\[PartialD]", "\[RightArrow]"], 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], ".", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"QuantumField", "(", 
       RowBox[{
        RowBox[{
        "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
         FormBox["\[Mu]",
          TraditionalForm], ")"}], ",", "A", ",", 
        FormBox["\[Sigma]",
         TraditionalForm], ",", 
        FormBox["g",
         TraditionalForm]}], ")"}], "-", 
      RowBox[{"QuantumField", "(", 
       RowBox[{
        RowBox[{
        "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
         FormBox["\[Sigma]",
          TraditionalForm], ")"}], ",", "A", ",", 
        FormBox["\[Mu]",
         TraditionalForm], ",", 
        FormBox["g",
         TraditionalForm]}], ")"}], "+", 
      RowBox[{
       SubscriptBox["f", 
        RowBox[{
         FormBox[
          FormBox["a",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["b",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["g",
           TraditionalForm],
          TraditionalForm]}]], " ", 
       RowBox[{
        SubsuperscriptBox["A", 
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["a",
           TraditionalForm],
          TraditionalForm]], ".", 
        SubsuperscriptBox["A", 
         FormBox[
          FormBox["\[Sigma]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["b",
           TraditionalForm],
          TraditionalForm]]}]}]}], ")"}]}], "-", 
   RowBox[{
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"QuantumField", "(", 
       RowBox[{
        RowBox[{
        "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
         FormBox["\[Mu]",
          TraditionalForm], ")"}], ",", "A", ",", 
        FormBox["\[Sigma]",
         TraditionalForm], ",", 
        FormBox["a",
         TraditionalForm]}], ")"}], "-", 
      RowBox[{"QuantumField", "(", 
       RowBox[{
        RowBox[{
        "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
         FormBox["\[Sigma]",
          TraditionalForm], ")"}], ",", "A", ",", 
        FormBox["\[Mu]",
         TraditionalForm], ",", 
        FormBox["a",
         TraditionalForm]}], ")"}], "+", 
      RowBox[{
       SubscriptBox["f", 
        RowBox[{
         FormBox[
          FormBox["a",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["b",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["d",
           TraditionalForm],
          TraditionalForm]}]], " ", 
       RowBox[{
        SubsuperscriptBox["A", 
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["b",
           TraditionalForm],
          TraditionalForm]], ".", 
        SubsuperscriptBox["A", 
         FormBox[
          FormBox["\[Sigma]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["d",
           TraditionalForm],
          TraditionalForm]]}]}]}], ")"}]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009702688082867`*^9},
 CellTags->"FunctionalD",
 CellLabel->"Out[818]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"w2", "=", 
  RowBox[{
   RowBox[{"ExpandPartialD", "[", "w1", "]"}], "/.", 
   RowBox[{"Dot", "\[Rule]", "Times"}]}]}]], "Input",
 CellTags->"FunctionalD",
 CellLabel->"In[819]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Sigma]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    RowBox[{"QuantumField", "(", 
     RowBox[{
      RowBox[{
      "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
       FormBox["\[Mu]",
        TraditionalForm], ")"}], ",", "A", ",", 
      FormBox["\[Mu]",
       TraditionalForm], ",", 
      FormBox["a",
       TraditionalForm]}], ")"}]}], "+", 
   RowBox[{
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    RowBox[{"QuantumField", "(", 
     RowBox[{
      RowBox[{
      "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
       FormBox["\[Mu]",
        TraditionalForm], ")"}], ",", "A", ",", 
      FormBox["\[Sigma]",
       TraditionalForm], ",", 
      FormBox["b",
       TraditionalForm]}], ")"}]}], "-", 
   RowBox[{
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    RowBox[{"QuantumField", "(", 
     RowBox[{
      RowBox[{
      "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
       FormBox["\[Mu]",
        TraditionalForm], ")"}], ",", "A", ",", 
      FormBox["\[Sigma]",
       TraditionalForm], ",", 
      FormBox["a",
       TraditionalForm]}], ")"}]}], "+", 
   RowBox[{
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    RowBox[{"QuantumField", "(", 
     RowBox[{
      RowBox[{
      "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
       FormBox["\[Sigma]",
        TraditionalForm], ")"}], ",", "A", ",", 
      FormBox["\[Mu]",
       TraditionalForm], ",", 
      FormBox["a",
       TraditionalForm]}], ")"}]}], "+", 
   RowBox[{"QuantumField", "(", 
    RowBox[{
     RowBox[{
     "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
      FormBox["\[Mu]",
       TraditionalForm], ")"}], ",", 
     RowBox[{
     "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
      FormBox["\[Mu]",
       TraditionalForm], ")"}], ",", "A", ",", 
     FormBox["\[Sigma]",
      TraditionalForm], ",", 
     FormBox["g",
      TraditionalForm]}], ")"}], "-", 
   RowBox[{"QuantumField", "(", 
    RowBox[{
     RowBox[{
     "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
      FormBox["\[Mu]",
       TraditionalForm], ")"}], ",", 
     RowBox[{
     "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
      FormBox["\[Sigma]",
       TraditionalForm], ")"}], ",", "A", ",", 
     FormBox["\[Mu]",
      TraditionalForm], ",", 
     FormBox["g",
      TraditionalForm]}], ")"}], "-", 
   RowBox[{
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Sigma]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["d",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["d",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970269114056*^9},
 CellTags->"FunctionalD",
 CellLabel->"Out[819]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"dif1", " ", "=", " ", 
  RowBox[{"w2", "-", "r4"}]}]], "Input",
 CellTags->"FunctionalD",
 CellLabel->"In[820]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Sigma]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    RowBox[{"QuantumField", "(", 
     RowBox[{
      RowBox[{
      "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
       FormBox["\[Mu]",
        TraditionalForm], ")"}], ",", "A", ",", 
      FormBox["\[Mu]",
       TraditionalForm], ",", 
      FormBox["b",
       TraditionalForm]}], ")"}]}], "-", 
   RowBox[{"2", " ", 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    RowBox[{"QuantumField", "(", 
     RowBox[{
      RowBox[{
      "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
       FormBox["\[Mu]",
        TraditionalForm], ")"}], ",", "A", ",", 
      FormBox["\[Sigma]",
       TraditionalForm], ",", 
      FormBox["b",
       TraditionalForm]}], ")"}]}], "+", 
   RowBox[{
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    RowBox[{"QuantumField", "(", 
     RowBox[{
      RowBox[{
      "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
       FormBox["\[Sigma]",
        TraditionalForm], ")"}], ",", "A", ",", 
      FormBox["\[Mu]",
       TraditionalForm], ",", 
      FormBox["b",
       TraditionalForm]}], ")"}]}], "+", 
   RowBox[{
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Sigma]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    RowBox[{"QuantumField", "(", 
     RowBox[{
      RowBox[{
      "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
       FormBox["\[Mu]",
        TraditionalForm], ")"}], ",", "A", ",", 
      FormBox["\[Mu]",
       TraditionalForm], ",", 
      FormBox["a",
       TraditionalForm]}], ")"}]}], "+", 
   RowBox[{
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    RowBox[{"QuantumField", "(", 
     RowBox[{
      RowBox[{
      "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
       FormBox["\[Mu]",
        TraditionalForm], ")"}], ",", "A", ",", 
      FormBox["\[Sigma]",
       TraditionalForm], ",", 
      FormBox["b",
       TraditionalForm]}], ")"}]}], "-", 
   RowBox[{
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    RowBox[{"QuantumField", "(", 
     RowBox[{
      RowBox[{
      "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
       FormBox["\[Mu]",
        TraditionalForm], ")"}], ",", "A", ",", 
      FormBox["\[Sigma]",
       TraditionalForm], ",", 
      FormBox["a",
       TraditionalForm]}], ")"}]}], "+", 
   RowBox[{
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    RowBox[{"QuantumField", "(", 
     RowBox[{
      RowBox[{
      "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
       FormBox["\[Sigma]",
        TraditionalForm], ")"}], ",", "A", ",", 
      FormBox["\[Mu]",
       TraditionalForm], ",", 
      FormBox["a",
       TraditionalForm]}], ")"}]}], "-", 
   RowBox[{"QuantumField", "(", 
    RowBox[{
     RowBox[{
     "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
      FormBox["\[Mu]",
       TraditionalForm], ")"}], ",", 
     RowBox[{
     "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
      FormBox["\[Mu]",
       TraditionalForm], ")"}], ",", "A", ",", 
     FormBox["\[Sigma]",
      TraditionalForm], ",", 
     FormBox["g",
      TraditionalForm]}], ")"}], "+", 
   RowBox[{"QuantumField", "(", 
    RowBox[{
     RowBox[{
     "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
      FormBox["\[Mu]",
       TraditionalForm], ")"}], ",", 
     RowBox[{
     "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
      FormBox["\[Sigma]",
       TraditionalForm], ")"}], ",", "A", ",", 
     FormBox["\[Mu]",
      TraditionalForm], ",", 
     FormBox["g",
      TraditionalForm]}], ")"}], "+", 
   RowBox[{"QuantumField", "(", 
    RowBox[{
     RowBox[{
     "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
      FormBox["\[Mu]",
       TraditionalForm], ")"}], ",", 
     RowBox[{
     "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
      FormBox["\[Mu]",
       TraditionalForm], ")"}], ",", "A", ",", 
     FormBox["\[Sigma]",
      TraditionalForm], ",", 
     FormBox["g",
      TraditionalForm]}], ")"}], "-", 
   RowBox[{"QuantumField", "(", 
    RowBox[{
     RowBox[{
     "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
      FormBox["\[Mu]",
       TraditionalForm], ")"}], ",", 
     RowBox[{
     "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
      FormBox["\[Sigma]",
       TraditionalForm], ")"}], ",", "A", ",", 
     FormBox["\[Mu]",
      TraditionalForm], ",", 
     FormBox["g",
      TraditionalForm]}], ")"}], "-", 
   RowBox[{
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Sigma]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["d",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["d",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Sigma]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["e",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["e",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970269443068*^9},
 CellTags->"FunctionalD",
 CellLabel->"Out[820]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"dif2", "=", 
  RowBox[{"symfun", "[", 
   RowBox[{"dif1", ",", "A"}], "]"}]}]], "Input",
 CellTags->"FunctionalD",
 CellLabel->"In[821]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Sigma]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    RowBox[{"QuantumField", "(", 
     RowBox[{
      RowBox[{
      "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
       FormBox["\[Mu]",
        TraditionalForm], ")"}], ",", "A", ",", 
      FormBox["\[Mu]",
       TraditionalForm], ",", 
      FormBox["b",
       TraditionalForm]}], ")"}]}], "-", 
   RowBox[{"2", " ", 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    RowBox[{"QuantumField", "(", 
     RowBox[{
      RowBox[{
      "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
       FormBox["\[Mu]",
        TraditionalForm], ")"}], ",", "A", ",", 
      FormBox["\[Sigma]",
       TraditionalForm], ",", 
      FormBox["b",
       TraditionalForm]}], ")"}]}], "+", 
   RowBox[{
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    RowBox[{"QuantumField", "(", 
     RowBox[{
      RowBox[{
      "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
       FormBox["\[Sigma]",
        TraditionalForm], ")"}], ",", "A", ",", 
      FormBox["\[Mu]",
       TraditionalForm], ",", 
      FormBox["b",
       TraditionalForm]}], ")"}]}], "-", 
   RowBox[{
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Sigma]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    RowBox[{"QuantumField", "(", 
     RowBox[{
      RowBox[{
      "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
       FormBox["\[Mu]",
        TraditionalForm], ")"}], ",", "A", ",", 
      FormBox["\[Mu]",
       TraditionalForm], ",", 
      FormBox["b",
       TraditionalForm]}], ")"}]}], "+", 
   RowBox[{"2", " ", 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    RowBox[{"QuantumField", "(", 
     RowBox[{
      RowBox[{
      "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
       FormBox["\[Mu]",
        TraditionalForm], ")"}], ",", "A", ",", 
      FormBox["\[Sigma]",
       TraditionalForm], ",", 
      FormBox["b",
       TraditionalForm]}], ")"}]}], "-", 
   RowBox[{
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    RowBox[{"QuantumField", "(", 
     RowBox[{
      RowBox[{
      "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
       FormBox["\[Sigma]",
        TraditionalForm], ")"}], ",", "A", ",", 
      FormBox["\[Mu]",
       TraditionalForm], ",", 
      FormBox["b",
       TraditionalForm]}], ")"}]}], "-", 
   RowBox[{"QuantumField", "(", 
    RowBox[{
     RowBox[{
     "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
      FormBox["\[Mu]",
       TraditionalForm], ")"}], ",", 
     RowBox[{
     "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
      FormBox["\[Mu]",
       TraditionalForm], ")"}], ",", "A", ",", 
     FormBox["\[Sigma]",
      TraditionalForm], ",", 
     FormBox["g",
      TraditionalForm]}], ")"}], "+", 
   RowBox[{"QuantumField", "(", 
    RowBox[{
     RowBox[{
     "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
      FormBox["\[Mu]",
       TraditionalForm], ")"}], ",", 
     RowBox[{
     "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
      FormBox["\[Sigma]",
       TraditionalForm], ")"}], ",", "A", ",", 
     FormBox["\[Mu]",
      TraditionalForm], ",", 
     FormBox["g",
      TraditionalForm]}], ")"}], "+", 
   RowBox[{"QuantumField", "(", 
    RowBox[{
     RowBox[{
     "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
      FormBox["\[Mu]",
       TraditionalForm], ")"}], ",", 
     RowBox[{
     "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
      FormBox["\[Mu]",
       TraditionalForm], ")"}], ",", "A", ",", 
     FormBox["\[Sigma]",
      TraditionalForm], ",", 
     FormBox["g",
      TraditionalForm]}], ")"}], "-", 
   RowBox[{"QuantumField", "(", 
    RowBox[{
     RowBox[{
     "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(",
       
      FormBox["\[Mu]",
       TraditionalForm], ")"}], ",", 
     RowBox[{
     "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
      FormBox["\[Sigma]",
       TraditionalForm], ")"}], ",", "A", ",", 
     FormBox["\[Mu]",
      TraditionalForm], ",", 
     FormBox["g",
      TraditionalForm]}], ")"}], "+", 
   RowBox[{
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Sigma]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["d",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["d",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]]}], "-", 
   RowBox[{
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Sigma]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["d",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["d",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970270089498*^9},
 CellTags->"FunctionalD",
 CellLabel->"Out[821]="]
}, Open  ]],

Cell["quod erat demonstrandum.", "Text",
 CellTags->"FunctionalD"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Unset", "[", 
   RowBox[{"s", "[", "\[Phi]", "]"}], "]"}], ";", 
  RowBox[{"Unset", "[", 
   RowBox[{"S", "[", "A", "]"}], "]"}], ";", 
  RowBox[{"Clear", "[", 
   RowBox[{
   "Ag", ",", "F1", ",", "F2", ",", "t1", ",", "t2", ",", "t3", ",", "t4", 
    ",", "F", ",", "r1", ",", "r2", ",", "r3", ",", "r4", ",", "symfun", ",", 
    "twof", ",", "twofnew", ",", "w0", ",", "w1", ",", "w2", ",", "dif1", ",",
     "dif2", ",", "dif3"}], "]"}]}]], "Input",
 CellTags->"FunctionalD",
 CellLabel->"In[822]:="]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples of funcional differentiation as used in FeynRule", \
"Subsubsection",
 CellTags->"FunctionalD"],

Cell["This is a part of the QCD Lagrangian.", "Text",
 CellTags->"FunctionalD"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"(", 
   RowBox[{"Gstrong", "*", 
    RowBox[{
     RowBox[{"QuantumField", "[", 
      RowBox[{"GaugeField", ",", " ", 
       RowBox[{"LorentzIndex", "[", "li1", "]"}], ",", "  ", 
       RowBox[{"SUNIndex", "[", "si2", "]"}]}], "]"}], ".", "  ", 
     RowBox[{"QuantumField", "[", 
      RowBox[{"GaugeField", ",", " ", 
       RowBox[{"LorentzIndex", "[", "li2", "]"}], ",", "   ", 
       RowBox[{"SUNIndex", "[", "si4", "]"}]}], "]"}], " ", ".", "  ", 
     RowBox[{"QuantumField", "[", 
      RowBox[{
       RowBox[{"PartialD", "[", 
        RowBox[{"LorentzIndex", "[", "li1", "]"}], "]"}], ",", " ", 
       "GaugeField", ",", "   ", 
       RowBox[{"LorentzIndex", "[", "li2", "]"}], ",", " ", 
       RowBox[{"SUNIndex", "[", "si1", "]"}]}], "]"}]}], "*", "  ", 
    RowBox[{"SUNF", "[", 
     RowBox[{
      RowBox[{"SUNIndex", "[", "si1", "]"}], ",", " ", 
      RowBox[{"SUNIndex", "[", "si2", "]"}], ",", " ", 
      RowBox[{"SUNIndex", "[", "si4", "]"}]}], "]"}]}], ")"}], "/", 
  "4"}]], "Input",
 CellLabel->"In[823]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["1", "4"], " ", 
   SubscriptBox["g", "s"], " ", 
   SubscriptBox["f", 
    RowBox[{
     FormBox[
      FormBox["si1",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["si2",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["si4",
       TraditionalForm],
      TraditionalForm]}]], " ", 
   RowBox[{
    SubsuperscriptBox[
     FormBox["A",
      TraditionalForm], 
     FormBox[
      FormBox["li1",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["si2",
       TraditionalForm],
      TraditionalForm]], ".", 
    SubsuperscriptBox[
     FormBox["A",
      TraditionalForm], 
     FormBox[
      FormBox["li2",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["si4",
       TraditionalForm],
      TraditionalForm]], ".", 
    RowBox[{"QuantumField", "(", 
     RowBox[{
      RowBox[{
      "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
       FormBox["li1",
        TraditionalForm], ")"}], ",", "A", ",", 
      FormBox["li2",
       TraditionalForm], ",", 
      FormBox["si1",
       TraditionalForm]}], ")"}]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970270301622*^9},
 CellLabel->"Out[823]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FunctionalD", "[", 
  RowBox[{"%", ",", " ", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"QuantumField", "[", 
       RowBox[{"GaugeField", ",", 
        RowBox[{"{", "\[Mu]", "}"}], ",", 
        RowBox[{"{", "a", "}"}]}], "]"}], "[", "p", "]"}], ",", " ", 
     RowBox[{
      RowBox[{"QuantumField", "[", 
       RowBox[{"GaugeField", ",", 
        RowBox[{"{", "\[Nu]", "}"}], ",", 
        RowBox[{"{", "b", "}"}]}], "]"}], "[", "q", "]"}], ",", " ", 
     RowBox[{
      RowBox[{"QuantumField", "[", 
       RowBox[{"GaugeField", ",", 
        RowBox[{"{", "\[Rho]", "}"}], ",", 
        RowBox[{"{", "c", "}"}]}], "]"}], "[", "r", "]"}]}], "}"}]}], 
  "]"}]], "Input",
 CellTags->"FunctionalD",
 CellLabel->"In[824]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["1", "4"], " ", 
   SubscriptBox["g", "s"], " ", 
   SubscriptBox["f", 
    RowBox[{
     FormBox[
      FormBox["si1",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["si2",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["si4",
       TraditionalForm],
      TraditionalForm]}]], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        SubscriptBox["\[Delta]", 
         RowBox[{
          FormBox[
           FormBox["b",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["si2",
            TraditionalForm],
           TraditionalForm]}]], " ", 
        SuperscriptBox["g", 
         RowBox[{"li1", "\[Nu]"}]]}], ")"}], ".", 
      RowBox[{"(", 
       RowBox[{
        SubscriptBox["\[Delta]", 
         RowBox[{
          FormBox[
           FormBox["c",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["si4",
            TraditionalForm],
           TraditionalForm]}]], " ", 
        SuperscriptBox["g", 
         RowBox[{"li2", "\[Rho]"}]]}], ")"}], ".", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", "\[ImaginaryI]"}], " ", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], 
         FormBox[
          FormBox["li1",
           TraditionalForm],
          TraditionalForm]], " ", 
        SubscriptBox["\[Delta]", 
         RowBox[{
          FormBox[
           FormBox["a",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["si1",
            TraditionalForm],
           TraditionalForm]}]], " ", 
        SuperscriptBox["g", 
         RowBox[{"li2", "\[Mu]"}]]}], ")"}]}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        SubscriptBox["\[Delta]", 
         RowBox[{
          FormBox[
           FormBox["c",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["si2",
            TraditionalForm],
           TraditionalForm]}]], " ", 
        SuperscriptBox["g", 
         RowBox[{"li1", "\[Rho]"}]]}], ")"}], ".", 
      RowBox[{"(", 
       RowBox[{
        SubscriptBox["\[Delta]", 
         RowBox[{
          FormBox[
           FormBox["b",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["si4",
            TraditionalForm],
           TraditionalForm]}]], " ", 
        SuperscriptBox["g", 
         RowBox[{"li2", "\[Nu]"}]]}], ")"}], ".", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", "\[ImaginaryI]"}], " ", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], 
         FormBox[
          FormBox["li1",
           TraditionalForm],
          TraditionalForm]], " ", 
        SubscriptBox["\[Delta]", 
         RowBox[{
          FormBox[
           FormBox["a",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["si1",
            TraditionalForm],
           TraditionalForm]}]], " ", 
        SuperscriptBox["g", 
         RowBox[{"li2", "\[Mu]"}]]}], ")"}]}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        SubscriptBox["\[Delta]", 
         RowBox[{
          FormBox[
           FormBox["a",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["si2",
            TraditionalForm],
           TraditionalForm]}]], " ", 
        SuperscriptBox["g", 
         RowBox[{"li1", "\[Mu]"}]]}], ")"}], ".", 
      RowBox[{"(", 
       RowBox[{
        SubscriptBox["\[Delta]", 
         RowBox[{
          FormBox[
           FormBox["c",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["si4",
            TraditionalForm],
           TraditionalForm]}]], " ", 
        SuperscriptBox["g", 
         RowBox[{"li2", "\[Rho]"}]]}], ")"}], ".", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", "\[ImaginaryI]"}], " ", 
        SuperscriptBox[
         FormBox["q",
          TraditionalForm], 
         FormBox[
          FormBox["li1",
           TraditionalForm],
          TraditionalForm]], " ", 
        SubscriptBox["\[Delta]", 
         RowBox[{
          FormBox[
           FormBox["b",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["si1",
            TraditionalForm],
           TraditionalForm]}]], " ", 
        SuperscriptBox["g", 
         RowBox[{"li2", "\[Nu]"}]]}], ")"}]}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        SubscriptBox["\[Delta]", 
         RowBox[{
          FormBox[
           FormBox["c",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["si2",
            TraditionalForm],
           TraditionalForm]}]], " ", 
        SuperscriptBox["g", 
         RowBox[{"li1", "\[Rho]"}]]}], ")"}], ".", 
      RowBox[{"(", 
       RowBox[{
        SubscriptBox["\[Delta]", 
         RowBox[{
          FormBox[
           FormBox["a",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["si4",
            TraditionalForm],
           TraditionalForm]}]], " ", 
        SuperscriptBox["g", 
         RowBox[{"li2", "\[Mu]"}]]}], ")"}], ".", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", "\[ImaginaryI]"}], " ", 
        SuperscriptBox[
         FormBox["q",
          TraditionalForm], 
         FormBox[
          FormBox["li1",
           TraditionalForm],
          TraditionalForm]], " ", 
        SubscriptBox["\[Delta]", 
         RowBox[{
          FormBox[
           FormBox["b",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["si1",
            TraditionalForm],
           TraditionalForm]}]], " ", 
        SuperscriptBox["g", 
         RowBox[{"li2", "\[Nu]"}]]}], ")"}]}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        SubscriptBox["\[Delta]", 
         RowBox[{
          FormBox[
           FormBox["a",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["si2",
            TraditionalForm],
           TraditionalForm]}]], " ", 
        SuperscriptBox["g", 
         RowBox[{"li1", "\[Mu]"}]]}], ")"}], ".", 
      RowBox[{"(", 
       RowBox[{
        SubscriptBox["\[Delta]", 
         RowBox[{
          FormBox[
           FormBox["b",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["si4",
            TraditionalForm],
           TraditionalForm]}]], " ", 
        SuperscriptBox["g", 
         RowBox[{"li2", "\[Nu]"}]]}], ")"}], ".", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", "\[ImaginaryI]"}], " ", 
        SuperscriptBox[
         FormBox["r",
          TraditionalForm], 
         FormBox[
          FormBox["li1",
           TraditionalForm],
          TraditionalForm]], " ", 
        SubscriptBox["\[Delta]", 
         RowBox[{
          FormBox[
           FormBox["c",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["si1",
            TraditionalForm],
           TraditionalForm]}]], " ", 
        SuperscriptBox["g", 
         RowBox[{"li2", "\[Rho]"}]]}], ")"}]}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        SubscriptBox["\[Delta]", 
         RowBox[{
          FormBox[
           FormBox["b",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["si2",
            TraditionalForm],
           TraditionalForm]}]], " ", 
        SuperscriptBox["g", 
         RowBox[{"li1", "\[Nu]"}]]}], ")"}], ".", 
      RowBox[{"(", 
       RowBox[{
        SubscriptBox["\[Delta]", 
         RowBox[{
          FormBox[
           FormBox["a",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["si4",
            TraditionalForm],
           TraditionalForm]}]], " ", 
        SuperscriptBox["g", 
         RowBox[{"li2", "\[Mu]"}]]}], ")"}], ".", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", "\[ImaginaryI]"}], " ", 
        SuperscriptBox[
         FormBox["r",
          TraditionalForm], 
         FormBox[
          FormBox["li1",
           TraditionalForm],
          TraditionalForm]], " ", 
        SubscriptBox["\[Delta]", 
         RowBox[{
          FormBox[
           FormBox["c",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["si1",
            TraditionalForm],
           TraditionalForm]}]], " ", 
        SuperscriptBox["g", 
         RowBox[{"li2", "\[Rho]"}]]}], ")"}]}]}], ")"}]}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970270618402*^9},
 CellTags->"FunctionalD",
 CellLabel->"Out[824]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Calc", "[", 
   RowBox[{"%", "//", "Calc"}], "]"}], "//", "Factor"}]], "Input",
 CellTags->"FunctionalD",
 CellLabel->"In[825]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["1", "4"], " ", "\[ImaginaryI]", " ", 
   SubscriptBox["g", "s"], " ", 
   SubscriptBox["f", 
    RowBox[{
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm]}]], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      RowBox[{
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox["g", 
        RowBox[{"\[Mu]", "\[Rho]"}]]}]}], "+", 
     RowBox[{
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], 
       FormBox[
        FormBox["\[Rho]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Nu]"}]]}], "+", 
     RowBox[{
      SuperscriptBox[
       FormBox["q",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Nu]", "\[Rho]"}]]}], "-", 
     RowBox[{
      SuperscriptBox[
       FormBox["q",
        TraditionalForm], 
       FormBox[
        FormBox["\[Rho]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Nu]"}]]}], "-", 
     RowBox[{
      SuperscriptBox[
       FormBox["r",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Nu]", "\[Rho]"}]]}], "+", 
     RowBox[{
      SuperscriptBox[
       FormBox["r",
        TraditionalForm], 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Rho]"}]]}]}], ")"}]}], TraditionalForm]], "Output",\

 CellChangeTimes->{3.500970271685704*^9},
 CellTags->"FunctionalD",
 CellLabel->"Out[825]="]
}, Open  ]]
}, Open  ]],

Cell["FV", "ObjectName",
 CellTags->"FV"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FV"],

Cell[TextData[{
 "FV[",
 StyleBox["p",
  FontSlant->"Italic"],
 ", ",
 Cell[BoxData[
  FormBox["\[Mu]", TraditionalForm]]],
 "] is the four-dimensional vector ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["p", "\[Mu]"], TraditionalForm]]],
 ". "
}], "Text",
 CellTags->"FV"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FCE",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FCE",
  ButtonNote->"FCE"],
 ", ",
 ButtonBox["FCI",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FCI",
  ButtonNote->"FCI"],
 ", ",
 ButtonBox["FVD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FVD",
  ButtonNote->"FVD"],
 ", ",
 ButtonBox["FourVector",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FourVector",
  ButtonNote->"FourVector"],
 ", ",
 ButtonBox["Pair",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Pair",
  ButtonNote->"Pair"],
 "."
}], "Text",
 CellTags->"FV"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"FV"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FV", "[", 
  RowBox[{"p", ",", "\[Mu]"}], "]"}]], "Input",
 CellTags->"FV",
 CellLabel->"In[826]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["p", "\[Mu]"], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970271848628*^9},
 CellTags->"FV",
 CellLabel->"Out[826]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FV", "[", 
  RowBox[{
   RowBox[{"p", "-", "q"}], ",", "\[Mu]"}], "]"}]], "Input",
 CellTags->"FV",
 CellLabel->"In[827]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox[
   FormBox[
    RowBox[{"p", "-", "q"}],
    TraditionalForm], "\[Mu]"], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009702719285717`*^9},
 CellTags->"FV",
 CellLabel->"Out[827]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"FV", "[", 
   RowBox[{"p", ",", "\[Mu]"}], "]"}], "//", "StandardForm"}]], "Input",
 CellTags->"FV",
 CellLabel->"In[828]:="],

Cell[BoxData[
 RowBox[{"FV", "[", 
  RowBox[{"p", ",", "\[Mu]"}], "]"}]], "Output",
 CellChangeTimes->{3.500970272007146*^9},
 CellTags->"FV",
 CellLabel->"Out[828]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"FCI", "[", 
   RowBox[{"FV", "[", 
    RowBox[{"p", ",", "\[Mu]"}], "]"}], "]"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"FV",
 CellLabel->"In[829]:="],

Cell[BoxData[
 RowBox[{"Pair", "[", 
  RowBox[{
   RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
   RowBox[{"Momentum", "[", "p", "]"}]}], "]"}]], "Output",
 CellChangeTimes->{3.500970272109913*^9},
 CellTags->"FV",
 CellLabel->"Out[829]//StandardForm="]
}, Open  ]],

Cell["\<\
There is no special function to expand momenta in FV; ExpandScalarProduct \
does the job.\
\>", "Text",
 CellTags->"FV"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ExpandScalarProduct", "[", 
  RowBox[{"FV", "[", 
   RowBox[{
    RowBox[{"p", "-", "q"}], ",", "\[Mu]"}], "]"}], "]"}]], "Input",
 CellTags->"FV",
 CellLabel->"In[830]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox[
   FormBox[
    RowBox[{"p", "-", "q"}],
    TraditionalForm], "\[Mu]"], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970272196076*^9},
 CellTags->"FV",
 CellLabel->"Out[830]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StandardForm", "[", "%", "]"}]], "Input",
 CellTags->"FV",
 CellLabel->"In[831]:="],

Cell[BoxData[
 RowBox[{"FV", "[", 
  RowBox[{
   RowBox[{"p", "-", "q"}], ",", "\[Mu]"}], "]"}]], "Output",
 CellChangeTimes->{3.500970272269063*^9},
 CellTags->"FV",
 CellLabel->"Out[831]//StandardForm="]
}, Open  ]]
}, Open  ]],

Cell["FVD", "ObjectName",
 CellTags->"FVD"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FVD"],

Cell[TextData[{
 "FVD[",
 StyleBox["p",
  FontSlant->"Italic"],
 ", ",
 Cell[BoxData[
  FormBox["\[Mu]", TraditionalForm]]],
 "] is the D-dimensional vector ",
 StyleBox["p",
  FontSlant->"Italic"],
 " with Lorentz index ",
 Cell[BoxData[
  FormBox["\[Mu]", TraditionalForm]]],
 "."
}], "Text",
 CellTags->"FVD"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FCE",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FCE",
  ButtonNote->"FCE"],
 ", ",
 ButtonBox["FCI",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FCI",
  ButtonNote->"FCI"],
 ", ",
 ButtonBox["FV",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FV",
  ButtonNote->"FV"],
 ", ",
 ButtonBox["FourVector",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FourVector",
  ButtonNote->"FourVector"],
 ", ",
 ButtonBox["Pair",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Pair",
  ButtonNote->"Pair"],
 "."
}], "Text",
 CellTags->"FVD"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"FVD"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FVD", "[", 
  RowBox[{"p", ",", "\[Mu]"}], "]"}]], "Input",
 CellTags->"FVD",
 CellLabel->"In[832]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["p", "\[Mu]"], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970272314156*^9},
 CellTags->"FVD",
 CellLabel->"Out[832]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FVD", "[", 
  RowBox[{
   RowBox[{"p", "-", "q"}], ",", "\[Mu]"}], "]"}]], "Input",
 CellTags->"FVD",
 CellLabel->"In[833]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox[
   FormBox[
    RowBox[{"p", "-", "q"}],
    TraditionalForm], "\[Mu]"], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009702723885736`*^9},
 CellTags->"FVD",
 CellLabel->"Out[833]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"FVD", "[", 
   RowBox[{"p", ",", "\[Mu]"}], "]"}], "//", "StandardForm"}]], "Input",
 CellTags->"FVD",
 CellLabel->"In[834]:="],

Cell[BoxData[
 RowBox[{"FVD", "[", 
  RowBox[{"p", ",", "\[Mu]"}], "]"}]], "Output",
 CellChangeTimes->{3.500970272433949*^9},
 CellTags->"FVD",
 CellLabel->"Out[834]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"FCI", "[", 
   RowBox[{"FVD", "[", 
    RowBox[{"p", ",", "\[Mu]"}], "]"}], "]"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"FVD",
 CellLabel->"In[835]:="],

Cell[BoxData[
 RowBox[{"Pair", "[", 
  RowBox[{
   RowBox[{"LorentzIndex", "[", 
    RowBox[{"\[Mu]", ",", "D"}], "]"}], ",", 
   RowBox[{"Momentum", "[", 
    RowBox[{"p", ",", "D"}], "]"}]}], "]"}]], "Output",
 CellChangeTimes->{3.500970272497561*^9},
 CellTags->"FVD",
 CellLabel->"Out[835]//StandardForm="]
}, Open  ]],

Cell["There is no special function to expand momenta in FVD.", "Text",
 CellTags->"FVD"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ExpandScalarProduct", "[", 
  RowBox[{"FVD", "[", 
   RowBox[{
    RowBox[{"p", "-", "q"}], ",", "\[Mu]"}], "]"}], "]"}]], "Input",
 CellTags->"FVD",
 CellLabel->"In[836]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox[
   FormBox[
    RowBox[{"p", "-", "q"}],
    TraditionalForm], "\[Mu]"], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009702725790577`*^9},
 CellTags->"FVD",
 CellLabel->"Out[836]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StandardForm", "[", "%", "]"}]], "Input",
 CellTags->"FVD",
 CellLabel->"In[837]:="],

Cell[BoxData[
 RowBox[{"FVD", "[", 
  RowBox[{
   RowBox[{"p", "-", "q"}], ",", "\[Mu]"}], "]"}]], "Output",
 CellChangeTimes->{3.500970272624346*^9},
 CellTags->"FVD",
 CellLabel->"Out[837]//StandardForm="]
}, Open  ]]
}, Open  ]],

Cell["GA", "ObjectName",
 CellTags->"GA"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"GA"],

Cell[TextData[{
 "GA[",
 Cell[BoxData[
  FormBox["\[Mu]", TraditionalForm]]],
 "] can be used as input for a 4-dimensional ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["\[Gamma]", 
    RowBox[{"\[Mu]", " "}]], TraditionalForm]]],
 "and is transformed into DiracGamma[LorentzIndex[",
 Cell[BoxData[
  FormBox["\[Mu]", TraditionalForm]]],
 "]] by FeynCalcInternal (=FCI).GA[",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Mu]", ",", "\[Nu]", ","}], TraditionalForm]]],
 " ...] is a short form for GA[",
 Cell[BoxData[
  FormBox["\[Mu]", TraditionalForm]]],
 "].GA[",
 Cell[BoxData[
  FormBox["\[Nu]", TraditionalForm]]],
 "]. ... ."
}], "Text",
 CellTags->"GA"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["DiracMatrix",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracMatrix",
  ButtonNote->"DiracMatrix"],
 ", ",
 ButtonBox["GAD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"GAD",
  ButtonNote->"GAD"],
 ", ",
 ButtonBox["GS",
  BaseStyle->"AddOnsLink",
  ButtonData:>"GS",
  ButtonNote->"GS"],
 "."
}], "Text",
 CellTags->"GA"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"GA"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GA", "[", "\[Mu]", "]"}]], "Input",
 CellTags->"GA",
 CellLabel->"In[838]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["\[Gamma]", "\[Mu]"], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009702726721153`*^9},
 CellTags->"GA",
 CellLabel->"Out[838]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"GA", "[", 
   RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}], "-", 
  RowBox[{"GA", "[", 
   RowBox[{"\[Nu]", ",", "\[Mu]"}], "]"}]}]], "Input",
 CellTags->"GA",
 CellLabel->"In[839]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
    SuperscriptBox["\[Gamma]", "\[Nu]"]}], "-", 
   RowBox[{
    SuperscriptBox["\[Gamma]", "\[Nu]"], ".", 
    SuperscriptBox["\[Gamma]", "\[Mu]"]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970272769515*^9},
 CellTags->"GA",
 CellLabel->"Out[839]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StandardForm", "[", 
  RowBox[{"FCI", "[", 
   RowBox[{"GA", "[", "\[Mu]", "]"}], "]"}], "]"}]], "Input",
 CellTags->"GA",
 CellLabel->"In[840]:="],

Cell[BoxData[
 RowBox[{"DiracGamma", "[", 
  RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], "]"}]], "Output",
 CellChangeTimes->{3.500970272930216*^9},
 CellTags->"GA",
 CellLabel->"Out[840]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GA", "[", 
  RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]", ",", "\[Sigma]"}], 
  "]"}]], "Input",
 CellTags->"GA",
 CellLabel->"In[841]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
   SuperscriptBox["\[Gamma]", "\[Nu]"], ".", 
   SuperscriptBox["\[Gamma]", "\[Rho]"], ".", 
   SuperscriptBox["\[Gamma]", "\[Sigma]"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970273008616*^9},
 CellTags->"GA",
 CellLabel->"Out[841]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StandardForm", "[", 
  RowBox[{"GA", "[", 
   RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]", ",", "\[Sigma]"}], "]"}], 
  "]"}]], "Input",
 CellTags->"GA",
 CellLabel->"In[842]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"GA", "[", "\[Mu]", "]"}], ".", 
  RowBox[{"GA", "[", "\[Nu]", "]"}], ".", 
  RowBox[{"GA", "[", "\[Rho]", "]"}], ".", 
  RowBox[{"GA", "[", "\[Sigma]", "]"}]}]], "Output",
 CellChangeTimes->{3.5009702730867643`*^9},
 CellTags->"GA",
 CellLabel->"Out[842]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"GA", "[", "\[Alpha]", "]"}], ".", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{"GS", "[", "p", "]"}], "+", "m"}], ")"}], ".", 
  RowBox[{"GA", "[", "\[Beta]", "]"}]}]], "Input",
 CellTags->"GA",
 CellLabel->"In[843]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", "\[Alpha]"], ".", 
   RowBox[{"(", 
    RowBox[{"m", "+", 
     RowBox[{"\[Gamma]", "\[CenterDot]", "p"}]}], ")"}], ".", 
   SuperscriptBox["\[Gamma]", "\[Beta]"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970273232524*^9},
 CellTags->"GA",
 CellLabel->"Out[843]="]
}, Open  ]]
}, Open  ]],

Cell["GAD", "ObjectName",
 CellTags->"GAD"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"GAD"],

Cell[TextData[{
 "GAD[",
 Cell[BoxData[
  FormBox["\[Mu]", TraditionalForm]]],
 "] can be used as input for a D-dimensional ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["\[Gamma]", 
    RowBox[{"\[Mu]", " "}]], TraditionalForm]]],
 "and is transformed into DiracGamma[LorentzIndex[",
 Cell[BoxData[
  FormBox["\[Mu]", TraditionalForm]]],
 ",D],D] by FeynCalcInternal (=FCI).GAD[",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Mu]", ",", "\[Nu]", ",", " ", "..."}], TraditionalForm]]],
 "] is a short form for GAD[",
 Cell[BoxData[
  FormBox["\[Mu]", TraditionalForm]]],
 "].GAD[",
 Cell[BoxData[
  FormBox["\[Nu]", TraditionalForm]]],
 "]. ... ."
}], "Text",
 CellTags->"GAD"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["DiracMatrix",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracMatrix",
  ButtonNote->"DiracMatrix"],
 ", ",
 ButtonBox["GA",
  BaseStyle->"AddOnsLink",
  ButtonData:>"GA",
  ButtonNote->"GA"],
 ", ",
 ButtonBox["GS",
  BaseStyle->"AddOnsLink",
  ButtonData:>"GS",
  ButtonNote->"GS"],
 "."
}], "Text",
 CellTags->"GAD"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"GAD"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GAD", "[", "\[Mu]", "]"}]], "Input",
 CellTags->"GAD",
 CellLabel->"In[844]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["\[Gamma]", "\[Mu]"], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970273358634*^9},
 CellTags->"GAD",
 CellLabel->"Out[844]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"GAD", "[", 
   RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}], "-", 
  RowBox[{"GAD", "[", 
   RowBox[{"\[Nu]", ",", "\[Mu]"}], "]"}]}]], "Input",
 CellTags->"GAD",
 CellLabel->"In[845]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
    SuperscriptBox["\[Gamma]", "\[Nu]"]}], "-", 
   RowBox[{
    SuperscriptBox["\[Gamma]", "\[Nu]"], ".", 
    SuperscriptBox["\[Gamma]", "\[Mu]"]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970273482089*^9},
 CellTags->"GAD",
 CellLabel->"Out[845]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StandardForm", "[", 
  RowBox[{"FCI", "[", 
   RowBox[{"GAD", "[", "\[Mu]", "]"}], "]"}], "]"}]], "Input",
 CellTags->"GAD",
 CellLabel->"In[846]:="],

Cell[BoxData[
 RowBox[{"DiracGamma", "[", 
  RowBox[{
   RowBox[{"LorentzIndex", "[", 
    RowBox[{"\[Mu]", ",", "D"}], "]"}], ",", "D"}], "]"}]], "Output",
 CellChangeTimes->{3.500970273555429*^9},
 CellTags->"GAD",
 CellLabel->"Out[846]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GAD", "[", 
  RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]", ",", "\[Sigma]"}], 
  "]"}]], "Input",
 CellTags->"GAD",
 CellLabel->"In[847]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
   SuperscriptBox["\[Gamma]", "\[Nu]"], ".", 
   SuperscriptBox["\[Gamma]", "\[Rho]"], ".", 
   SuperscriptBox["\[Gamma]", "\[Sigma]"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970273674691*^9},
 CellTags->"GAD",
 CellLabel->"Out[847]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StandardForm", "[", 
  RowBox[{"GAD", "[", 
   RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]", ",", "\[Sigma]"}], "]"}], 
  "]"}]], "Input",
 CellTags->"GAD",
 CellLabel->"In[848]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"GAD", "[", "\[Mu]", "]"}], ".", 
  RowBox[{"GAD", "[", "\[Nu]", "]"}], ".", 
  RowBox[{"GAD", "[", "\[Rho]", "]"}], ".", 
  RowBox[{"GAD", "[", "\[Sigma]", "]"}]}]], "Output",
 CellChangeTimes->{3.5009702737558327`*^9},
 CellTags->"GAD",
 CellLabel->"Out[848]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"GAD", "[", "\[Alpha]", "]"}], ".", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{"GSD", "[", "p", "]"}], "+", "m"}], ")"}], ".", 
  RowBox[{"GAD", "[", "\[Beta]", "]"}]}]], "Input",
 CellTags->"GAD",
 CellLabel->"In[849]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", "\[Alpha]"], ".", 
   RowBox[{"(", 
    RowBox[{"m", "+", 
     RowBox[{"\[Gamma]", "\[CenterDot]", "p"}]}], ")"}], ".", 
   SuperscriptBox["\[Gamma]", "\[Beta]"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970273924788*^9},
 CellTags->"GAD",
 CellLabel->"Out[849]="]
}, Open  ]]
}, Open  ]],

Cell["GammaEpsilon", "ObjectName",
 CellTags->"GammaEpsilon"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"GammaEpsilon"],

Cell["\<\
GammaEpsilon[exp] gives a series expansion of Gamma[exp] in Epsilon up to \
order 6 (where EulerGamma is neglected).\
\>", "Text",
 CellTags->"GammaEpsilon"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["GammaExpand",
  BaseStyle->"AddOnsLink",
  ButtonData:>"GammaExpand",
  ButtonNote->"GammaExpand"],
 ", ",
 ButtonBox["Series2",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Series2",
  ButtonNote->"Series2"],
 "."
}], "Text",
 CellTags->"GammaEpsilon"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"GammaEpsilon"],

Cell["\<\
If the argument is of the form (1+a Epsilon) the result is not calculated but \
tabulated.\
\>", "Text",
 CellTags->"GammaEpsilon"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GammaEpsilon", "[", 
  RowBox[{"1", "+", 
   RowBox[{"a", " ", "Epsilon"}]}], "]"}]], "Input",
 CellTags->"GammaEpsilon",
 CellLabel->"In[850]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SuperscriptBox[
     TagBox["\[CurlyEpsilon]",
      TraditionalForm], "5"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", 
       FractionBox[
        RowBox[{
         SuperscriptBox["a", "5"], " ", 
         TemplateBox[{"5"},
          "Zeta"]}], "5"]}], "-", 
      RowBox[{
       FractionBox["1", "36"], " ", 
       SuperscriptBox["\[Pi]", "2"], " ", 
       SuperscriptBox["a", "5"], " ", 
       TemplateBox[{"3"},
        "Zeta"]}]}], ")"}]}], "+", 
   RowBox[{
    FractionBox["1", "160"], " ", 
    SuperscriptBox["\[Pi]", "4"], " ", 
    SuperscriptBox["a", "4"], " ", 
    SuperscriptBox[
     TagBox["\[CurlyEpsilon]",
      TraditionalForm], "4"]}], "-", 
   RowBox[{
    FractionBox["1", "3"], " ", 
    SuperscriptBox["a", "3"], " ", 
    SuperscriptBox[
     TagBox["\[CurlyEpsilon]",
      TraditionalForm], "3"], " ", 
    TemplateBox[{"3"},
     "Zeta"]}], "+", 
   RowBox[{
    FractionBox["1", "12"], " ", 
    SuperscriptBox["\[Pi]", "2"], " ", 
    SuperscriptBox["a", "2"], " ", 
    SuperscriptBox[
     TagBox["\[CurlyEpsilon]",
      TraditionalForm], "2"]}], "+", 
   RowBox[{"C$7432", " ", 
    SuperscriptBox[
     TagBox["\[CurlyEpsilon]",
      TraditionalForm], "6"]}], "+", "1"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970274136297*^9},
 CellTags->"GammaEpsilon",
 CellLabel->"Out[850]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GammaEpsilon", "[", 
  RowBox[{"1", "-", 
   RowBox[{"Epsilon", "/", "2"}]}], "]"}]], "Input",
 CellTags->"GammaEpsilon",
 CellLabel->"In[851]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"C$7452", " ", 
    SuperscriptBox[
     TagBox["\[CurlyEpsilon]",
      TraditionalForm], "6"]}], "+", 
   FractionBox[
    RowBox[{
     SuperscriptBox["\[Pi]", "4"], " ", 
     SuperscriptBox[
      TagBox["\[CurlyEpsilon]",
       TraditionalForm], "4"]}], "2560"], "+", 
   FractionBox[
    RowBox[{
     SuperscriptBox["\[Pi]", "2"], " ", 
     SuperscriptBox[
      TagBox["\[CurlyEpsilon]",
       TraditionalForm], "2"]}], "48"], "+", 
   RowBox[{
    SuperscriptBox[
     TagBox["\[CurlyEpsilon]",
      TraditionalForm], "5"], " ", 
    RowBox[{"(", 
     RowBox[{
      FractionBox[
       RowBox[{
        SuperscriptBox["\[Pi]", "2"], " ", 
        TemplateBox[{"3"},
         "Zeta"]}], "1152"], "+", 
      FractionBox[
       TemplateBox[{"5"},
        "Zeta"], "160"]}], ")"}]}], "+", 
   FractionBox[
    RowBox[{
     SuperscriptBox[
      TagBox["\[CurlyEpsilon]",
       TraditionalForm], "3"], " ", 
     TemplateBox[{"3"},
      "Zeta"]}], "24"], "+", "1"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970274292808*^9},
 CellTags->"GammaEpsilon",
 CellLabel->"Out[851]="]
}, Open  ]],

Cell["For other arguments the expansion is calculated.", "Text",
 CellTags->"GammaEpsilon"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GammaEpsilon", "[", "Epsilon", "]"}]], "Input",
 CellTags->"GammaEpsilon",
 CellLabel->"In[852]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"C$8546", " ", 
    SuperscriptBox[
     TagBox["\[CurlyEpsilon]",
      TraditionalForm], "6"]}], "+", 
   FractionBox[
    RowBox[{
     SuperscriptBox["\[Pi]", "4"], " ", 
     SuperscriptBox[
      TagBox["\[CurlyEpsilon]",
       TraditionalForm], "3"]}], "160"], "+", 
   FractionBox[
    RowBox[{
     SuperscriptBox["\[Pi]", "2"], " ", 
     TagBox["\[CurlyEpsilon]",
      TraditionalForm]}], "12"], "+", 
   FractionBox["1", 
    FormBox[
     TagBox["\[CurlyEpsilon]",
      TraditionalForm],
     TraditionalForm]], "+", 
   FractionBox[
    RowBox[{
     SuperscriptBox[
      TagBox["\[CurlyEpsilon]",
       TraditionalForm], "5"], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"61", " ", 
        SuperscriptBox["\[Pi]", "6"]}], "+", 
       RowBox[{"1680", " ", 
        SuperscriptBox[
         TemplateBox[{"2","1"},
          "PolyGamma2"], "2"]}]}], ")"}]}], "120960"], "+", 
   FractionBox[
    RowBox[{
     SuperscriptBox[
      TagBox["\[CurlyEpsilon]",
       TraditionalForm], "2"], " ", 
     TemplateBox[{"2","1"},
      "PolyGamma2"]}], "6"], "+", 
   FractionBox[
    RowBox[{
     SuperscriptBox[
      TagBox["\[CurlyEpsilon]",
       TraditionalForm], "6"], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"-", "2016"}], " ", 
        SuperscriptBox["\[Pi]", "2"], " ", 
        TemplateBox[{"5"},
         "Zeta"]}], "+", 
       RowBox[{"126", " ", 
        SuperscriptBox["\[Pi]", "4"], " ", 
        TemplateBox[{"2","1"},
         "PolyGamma2"]}], "+", 
       RowBox[{"24", " ", 
        TemplateBox[{"6","1"},
         "PolyGamma2"]}]}], ")"}]}], "120960"], "+", 
   FractionBox[
    RowBox[{
     SuperscriptBox[
      TagBox["\[CurlyEpsilon]",
       TraditionalForm], "4"], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"20", " ", 
        SuperscriptBox["\[Pi]", "2"], " ", 
        TemplateBox[{"2","1"},
         "PolyGamma2"]}], "-", 
       RowBox[{"288", " ", 
        TemplateBox[{"5"},
         "Zeta"]}]}], ")"}]}], "1440"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009702757603903`*^9},
 CellTags->"GammaEpsilon",
 CellLabel->"Out[852]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GammaEpsilon", "[", "x", "]"}]], "Input",
 CellTags->"GammaEpsilon",
 CellLabel->"In[853]:="],

Cell[BoxData[
 FormBox[
  TemplateBox[{"x"},
   "Gamma"], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970276219702*^9},
 CellTags->"GammaEpsilon",
 CellLabel->"Out[853]="]
}, Open  ]]
}, Open  ]],

Cell["GammaExpand", "ObjectName",
 CellTags->"GammaExpand"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"GammaExpand"],

Cell["\<\
GammaExpand[exp] rewrites Gamma[n + m] in exp (where n has Head Integer).\
\>", "Text",
 CellTags->"GammaExpand"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["GammaEpsilon",
  BaseStyle->"AddOnsLink",
  ButtonData:>"GammaEpsilon",
  ButtonNote->"GammaEpsilon"],
 "."
}], "Text",
 CellTags->"GammaExpand"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"GammaExpand"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GammaExpand", "[", 
  RowBox[{"Gamma", "[", 
   RowBox[{"2", " ", "+", " ", "Epsilon"}], "]"}], "]"}]], "Input",
 CellTags->"GammaExpand",
 CellLabel->"In[854]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     TagBox["\[CurlyEpsilon]",
      TraditionalForm], "+", "1"}], ")"}], " ", 
   TemplateBox[{RowBox[{
       TagBox["\[CurlyEpsilon]", TraditionalForm], "+", "1"}]},
    "Gamma"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970276374331*^9},
 CellTags->"GammaExpand",
 CellLabel->"Out[854]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GammaExpand", "[", 
  RowBox[{"Gamma", "[", 
   RowBox[{
    RowBox[{"-", "3"}], "+", "Epsilon"}], "]"}], "]"}]], "Input",
 CellTags->"GammaExpand",
 CellLabel->"In[855]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   TemplateBox[{RowBox[{
       TagBox["\[CurlyEpsilon]", TraditionalForm], "+", "1"}]},
    "Gamma"], 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      TagBox["\[CurlyEpsilon]",
       TraditionalForm], "-", "3"}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      TagBox["\[CurlyEpsilon]",
       TraditionalForm], "-", "2"}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      TagBox["\[CurlyEpsilon]",
       TraditionalForm], "-", "1"}], ")"}], " ", 
    TagBox["\[CurlyEpsilon]",
     TraditionalForm]}]], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970276514146*^9},
 CellTags->"GammaExpand",
 CellLabel->"Out[855]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GammaExpand", "[", 
  RowBox[{"Gamma", "[", 
   RowBox[{"1", " ", "+", " ", "Epsilon"}], "]"}], "]"}]], "Input",
 CellTags->"GammaExpand",
 CellLabel->"In[856]:="],

Cell[BoxData[
 FormBox[
  TemplateBox[{RowBox[{
      TagBox["\[CurlyEpsilon]", TraditionalForm], "+", "1"}]},
   "Gamma"], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970276644638*^9},
 CellTags->"GammaExpand",
 CellLabel->"Out[856]="]
}, Open  ]]
}, Open  ]],

Cell["Gamma1", "ObjectName",
 CellTags->"Gamma1"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Gamma1"],

Cell["\<\
Gamma1[al,ga, be,de] is a special product of Gamma functions expanded up to \
order Epsilon^2.\
\>", "Text",
 CellTags->"Gamma1"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Gamma2",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Gamma2",
  ButtonNote->"Gamma2"],
 ", ",
 ButtonBox["Gamma3",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Gamma3",
  ButtonNote->"Gamma3"],
 "."
}], "Text",
 CellTags->"Gamma1"]
}, Open  ]],

Cell["Gamma2", "ObjectName",
 CellTags->"Gamma2"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Gamma2"],

Cell["\<\
Gamma2[x,y] is a special product of Gamma functions expanded up to order \
Epsilon^3 when positive integer arguments are given.\
\>", "Text",
 CellTags->"Gamma2"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Gamma1",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Gamma1",
  ButtonNote->"Gamma1"],
 ", ",
 ButtonBox["Gamma3",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Gamma3",
  ButtonNote->"Gamma3"],
 "."
}], "Text",
 CellTags->"Gamma2"]
}, Open  ]],

Cell["Gamma3", "ObjectName",
 CellTags->"Gamma3"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Gamma3"],

Cell["\<\
Gamma3[al,be,ga,ep] is a special product of Gamma functions expanded up to \
order Epsilon^n when positive integer arguments are given (the order n is \
determined by the option EpsilonOrder).\
\>", "Text",
 CellTags->"Gamma3"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Gamma1",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Gamma1",
  ButtonNote->"Gamma1"],
 ", ",
 ButtonBox["Gamma2",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Gamma2",
  ButtonNote->"Gamma2"],
 "."
}], "Text",
 CellTags->"Gamma3"]
}, Open  ]],

Cell["Gauge", "ObjectName",
 CellTags->"Gauge"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Gauge"],

Cell["\<\
Gauge is an option for GluonProgagator. If set to 1 the 't Hooft Feynman \
gauge is used.\
\>", "Text",
 CellTags->"Gauge"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["$Gauge",
  BaseStyle->"AddOnsLink",
  ButtonData:>"$Gauge",
  ButtonNote->"$Gauge"],
 ", ",
 ButtonBox["GluonProgagator",
  BaseStyle->"AddOnsLink",
  ButtonData:>"GluonProgagator",
  ButtonNote->"GluonProgagator"],
 "."
}], "Text",
 CellTags->"Gauge"]
}, Open  ]],

Cell["GaugeField", "ObjectName",
 CellTags->"GaugeField"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"GaugeField"],

Cell["\<\
GaugeField is just a name. No functional properties are associated with it. \
GaugeField is used as default setting for the option QuantumField of \
FieldStrength.\
\>", "Text",
 CellTags->"GaugeField"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FieldStrength",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FieldStrength",
  ButtonNote->"FieldStrength"],
 ", ",
 ButtonBox["QuantumField",
  BaseStyle->"AddOnsLink",
  ButtonData:>"QuantumField",
  ButtonNote->"QuantumField"],
 "."
}], "Text",
 CellTags->"GaugeField"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"GaugeField"],

Cell[CellGroupData[{

Cell[BoxData["GaugeField"], "Input",
 CellTags->"GaugeField",
 CellLabel->"In[857]:="],

Cell[BoxData[
 FormBox["A", TraditionalForm]], "Output",
 CellChangeTimes->{3.500970276727873*^9},
 CellTags->"GaugeField",
 CellLabel->"Out[857]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"QuantumField", "[", 
  RowBox[{"GaugeField", ",", 
   RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
   RowBox[{"SUNIndex", "[", "a", "]"}]}], "]"}]], "Input",
 CellTags->"GaugeField",
 CellLabel->"In[858]:="],

Cell[BoxData[
 FormBox[
  SubsuperscriptBox[
   FormBox["A",
    TraditionalForm], 
   FormBox[
    FormBox["\[Mu]",
     TraditionalForm],
    TraditionalForm], 
   FormBox[
    FormBox["a",
     TraditionalForm],
    TraditionalForm]], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970276811262*^9},
 CellTags->"GaugeField",
 CellLabel->"Out[858]="]
}, Open  ]]
}, Open  ]],

Cell["GaugeXi", "ObjectName",
 CellTags->"GaugeXi"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"GaugeXi"],

Cell["GaugeXi is a head for gauge parameters.", "Text",
 CellTags->"GaugeXi"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Gauge",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Gauge",
  ButtonNote->"Gauge"],
 ", ",
 ButtonBox["GaugeField",
  BaseStyle->"AddOnsLink",
  ButtonData:>"GaugeField",
  ButtonNote->"GaugeField"],
 "."
}], "Text",
 CellTags->"GaugeXi"]
}, Open  ]],

Cell["GA5", "ObjectName",
 CellTags->"GA5"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"GA5"],

Cell["GA5 is equivalent to DiracGamma[5] and denotes gamma5.", "Text",
 CellTags->"GA5"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["DiracGamma",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracGamma",
  ButtonNote->"DiracGamma"],
 ", ",
 ButtonBox["GA",
  BaseStyle->"AddOnsLink",
  ButtonData:>"GA",
  ButtonNote->"GA"],
 ", ",
 ButtonBox["GS",
  BaseStyle->"AddOnsLink",
  ButtonData:>"GS",
  ButtonNote->"GS"],
 "."
}], "Text",
 CellTags->"GA5"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"GA5"],

Cell[CellGroupData[{

Cell[BoxData["GA5"], "Input",
 CellTags->"GA5",
 CellLabel->"In[859]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["\[Gamma]", "5"], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009702769555397`*^9},
 CellTags->"GA5",
 CellLabel->"Out[859]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "Tr"}]], "Input",
 CellTags->"GA5",
 CellLabel->"In[860]:="],

Cell[BoxData[
 FormBox["0", TraditionalForm]], "Output",
 CellChangeTimes->{3.5009702770372066`*^9},
 CellTags->"GA5",
 CellLabel->"Out[860]="]
}, Open  ]]
}, Open  ]],

Cell["GGV", "ObjectName",
 CellTags->"GGV"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"GGV"],

Cell["GGV is equivalent to GluonGhostVertex.", "Text",
 CellTags->"GGV"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["GluonGhostVertex",
  BaseStyle->"AddOnsLink",
  ButtonData:>"GluonGhostVertex",
  ButtonNote->"GluonGhostVertex"],
 "."
}], "Text",
 CellTags->"GGV"]
}, Open  ]],

Cell["GhostPropagator", "ObjectName",
 CellTags->"GhostPropagator"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"GhostPropagator"],

Cell[TextData[{
 "GhostPropagator[p, a, b] gives the ghost propagator where \"a\" and \"b\" \
are the color indices. GhostPropagator[p] omits the ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["\[Delta]", 
     RowBox[{"a", "\[NoBreak]", "b"}]], "."}], TraditionalForm]]]
}], "Text",
 CellTags->"GhostPropagator"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["GHP",
  BaseStyle->"AddOnsLink",
  ButtonData:>"GHP",
  ButtonNote->"GHP"],
 ", ",
 ButtonBox["GluonPropagator",
  BaseStyle->"AddOnsLink",
  ButtonData:>"GluonPropagator",
  ButtonNote->"GluonPropagator"],
 ", ",
 ButtonBox["GluonGhostVertex",
  BaseStyle->"AddOnsLink",
  ButtonData:>"GluonGhostVertex",
  ButtonNote->"GluonGhostVertex"],
 "."
}], "Text",
 CellTags->"GhostPropagator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "GhostPropagator", "]"}]], "Input",
 CellLabel->"In[861]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"Explicit", "\[Rule]", "False"}], "}"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.50097027716689*^9},
 CellLabel->"Out[861]="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"GhostPropagator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GhostPropagator", "[", 
  RowBox[{"p", ",", "a", ",", "b"}], "]"}]], "Input",
 CellTags->"GhostPropagator",
 CellLabel->"In[862]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["\[CapitalPi]", 
    RowBox[{"a", "b"}]], "(", "p", ")"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.50097027722073*^9},
 CellTags->"GhostPropagator",
 CellLabel->"Out[862]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GhostPropagator", "[", 
  RowBox[{"p", ",", "a", ",", "b", ",", 
   RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"GhostPropagator",
 CellLabel->"In[863]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{"\[ImaginaryI]", " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]}]]}], 
   SuperscriptBox["p", "2"]], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970277264295*^9},
 CellTags->"GhostPropagator",
 CellLabel->"Out[863]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GhostPropagator", "[", "p", "]"}]], "Input",
 CellTags->"GhostPropagator",
 CellLabel->"In[864]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["\[CapitalPi]", "u"], "(", "p", ")"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.5009702773344812`*^9},
 CellTags->"GhostPropagator",
 CellLabel->"Out[864]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"GhostPropagator", "[", 
   RowBox[{"p", ",", "1", ",", "2"}], "]"}], "//", "Explicit"}]], "Input",
 CellTags->"GhostPropagator",
 CellLabel->"In[865]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{"\[ImaginaryI]", " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["ci1",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["ci2",
        TraditionalForm],
       TraditionalForm]}]]}], 
   SuperscriptBox["p", "2"]], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970277405883*^9},
 CellTags->"GhostPropagator",
 CellLabel->"Out[865]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"GhostPropagator", "[", 
     RowBox[{
      RowBox[{"-", "k"}], ",", "3", ",", "4"}], "]"}], "//", "Explicit"}], "//",
    " ", "FCE"}], "//", "StandardForm"}]], "Input",
 CellTags->"GhostPropagator",
 CellLabel->"In[866]:="],

Cell[BoxData[
 RowBox[{"\[ImaginaryI]", " ", 
  RowBox[{"FAD", "[", "k", "]"}], " ", 
  RowBox[{"SD", "[", 
   RowBox[{"ci3", ",", "ci4"}], "]"}]}]], "Output",
 CellChangeTimes->{3.500970277489503*^9},
 CellTags->"GhostPropagator",
 CellLabel->"Out[866]//StandardForm="]
}, Open  ]]
}, Open  ]],

Cell["GHP", "ObjectName",
 CellTags->"GHP"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"GHP"],

Cell[TextData[{
 "GHP[p, a, b] gives the ghost propagator where \"a\" and \"b\" are the color \
indices. GHP[p] omits the ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["\[Delta]", 
     RowBox[{"a", "\[NoBreak]", "b"}]], "."}], TraditionalForm]]]
}], "Text",
 CellTags->"GHP"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["GhostPropagator",
  BaseStyle->"AddOnsLink",
  ButtonData:>"GhostPropagator",
  ButtonNote->"GhostPropagator"],
 ", ",
 ButtonBox["GluonPropagator",
  BaseStyle->"AddOnsLink",
  ButtonData:>"GluonPropagator",
  ButtonNote->"GluonPropagator"],
 ", ",
 ButtonBox["GluonGhostVertex",
  BaseStyle->"AddOnsLink",
  ButtonData:>"GluonGhostVertex",
  ButtonNote->"GluonGhostVertex"],
 "."
}], "Text",
 CellTags->"GHP"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"GHP"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GHP", "[", 
  RowBox[{"p", ",", "a", ",", "b"}], "]"}]], "Input",
 CellTags->"GHP",
 CellLabel->"In[867]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["\[CapitalPi]", 
    RowBox[{"a", "b"}]], "(", "p", ")"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970277533244*^9},
 CellTags->"GHP",
 CellLabel->"Out[867]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"GHP", "[", "p", "]"}], "//", "Explicit"}]], "Input",
 CellTags->"GHP",
 CellLabel->"In[868]:="],

Cell[BoxData[
 FormBox[
  FractionBox["\[ImaginaryI]", 
   SuperscriptBox["p", "2"]], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970277604892*^9},
 CellTags->"GHP",
 CellLabel->"Out[868]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GHP", "[", 
  RowBox[{"p", ",", "1", ",", "2"}], "]"}]], "Input",
 CellTags->"GHP",
 CellLabel->"In[869]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["\[CapitalPi]", 
    RowBox[{"ci1", "ci2"}]], "(", "p", ")"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970277689973*^9},
 CellTags->"GHP",
 CellLabel->"Out[869]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StandardForm", "[", 
  RowBox[{"FCE", "[", 
   RowBox[{
    RowBox[{"GHP", "[", 
     RowBox[{
      RowBox[{"-", "k"}], ",", "3", ",", "4"}], "]"}], "//", "Explicit"}], 
   "]"}], "]"}]], "Input",
 CellTags->"GHP",
 CellLabel->"In[870]:="],

Cell[BoxData[
 RowBox[{"\[ImaginaryI]", " ", 
  RowBox[{"FAD", "[", "k", "]"}], " ", 
  RowBox[{"SD", "[", 
   RowBox[{"ci3", ",", "ci4"}], "]"}]}]], "Output",
 CellChangeTimes->{3.5009702778215637`*^9},
 CellTags->"GHP",
 CellLabel->"Out[870]//StandardForm="]
}, Open  ]]
}, Open  ]],

Cell["GluonField", "ObjectName",
 CellTags->"GluonField"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"GluonField"],

Cell["GluonField is a name of a gauge field.", "Text",
 CellTags->"GluonField"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["GaugeField",
  BaseStyle->"AddOnsLink",
  ButtonData:>"GaugeField",
  ButtonNote->"GaugeField"],
 "."
}], "Text",
 CellTags->"GluonField"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"GluonField"],

Cell[CellGroupData[{

Cell[BoxData["GluonField"], "Input",
 CellTags->"GluonField",
 CellLabel->"In[871]:="],

Cell[BoxData[
 FormBox["A", TraditionalForm]], "Output",
 CellChangeTimes->{3.50097027792006*^9},
 CellTags->"GluonField",
 CellLabel->"Out[871]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"QuantumField", "[", 
  RowBox[{"GluonField", ",", 
   RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
   RowBox[{"SUNIndex", "[", "a", "]"}]}], "]"}]], "Input",
 CellTags->"GluonField",
 CellLabel->"In[872]:="],

Cell[BoxData[
 FormBox[
  SubsuperscriptBox[
   FormBox["A",
    TraditionalForm], 
   FormBox[
    FormBox["\[Mu]",
     TraditionalForm],
    TraditionalForm], 
   FormBox[
    FormBox["a",
     TraditionalForm],
    TraditionalForm]], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009702779892063`*^9},
 CellTags->"GluonField",
 CellLabel->"Out[872]="]
}, Open  ]]
}, Open  ]],

Cell["GluonGhostVertex", "ObjectName",
 CellTags->"GluonGhostVertex"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"GluonGhostVertex"],

Cell["\<\
GluonGhostVertex[{p,mu,a}, {q,nu,b}, {k,rho,c}] or GluonGhostVertex[ p,mu,a , \
q,nu,b , k,rho,c ] yields the Gluon-Ghost vertex. The first argument \
represents the gluon and the third argument the outgoing ghost field (but \
incoming four-momentum). The dimension and the name of the coupling constant \
are determined by the options Dimension and CouplingConstant.\
\>", "Text",
 CellTags->"GluonGhostVertex"],

Cell[TextData[{
 "See also: ",
 ButtonBox["GluonPropagator",
  BaseStyle->"AddOnsLink",
  ButtonData:>"GluonPropagator",
  ButtonNote->"GluonPropagator"],
 ", ",
 ButtonBox["GluonSelfEnergy",
  BaseStyle->"AddOnsLink",
  ButtonData:>"GluonSelfEnergy",
  ButtonNote->"GluonSelfEnergy"],
 ", ",
 ButtonBox["GhostPropagator",
  BaseStyle->"AddOnsLink",
  ButtonData:>"GhostPropagator",
  ButtonNote->"GhostPropagator"],
 ", ",
 ButtonBox["GluonVertex",
  BaseStyle->"AddOnsLink",
  ButtonData:>"GluonVertex",
  ButtonNote->"GluonVertex"],
 "."
}], "Text",
 CellTags->"GluonGhostVertex"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "GluonGhostVertex", "]"}]], "Input",
 CellTags->"GluonGhostVertex",
 CellLabel->"In[873]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"CouplingConstant", "\[Rule]", 
     SubscriptBox["g", "s"]}], ",", 
    RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
    RowBox[{"Explicit", "\[Rule]", "False"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.5009702780629683`*^9},
 CellTags->"GluonGhostVertex",
 CellLabel->"Out[873]="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"GluonGhostVertex"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GluonGhostVertex", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"p", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"q", ",", "\[Nu]", ",", "b"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"k", ",", "\[Rho]", ",", "c"}], "}"}]}], "]"}]], "Input",
 CellTags->"GluonGhostVertex",
 CellLabel->"In[874]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SuperscriptBox[
     OverscriptBox["\[CapitalLambda]", "~"], "\[Mu]"], "(", "k", ")"}], " ", 
   SubscriptBox["f", 
    RowBox[{
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm]}]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.50097027816818*^9},
 CellTags->"GluonGhostVertex",
 CellLabel->"Out[874]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Explicit", "[", "%", "]"}]], "Input",
 CellLabel->"In[875]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    SubscriptBox["g", "s"]}], " ", 
   SuperscriptBox[
    FormBox["k",
     TraditionalForm], 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], " ", 
   SubscriptBox["f", 
    RowBox[{
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm]}]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009702782399*^9},
 CellLabel->"Out[875]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GluonGhostVertex", "[", 
  RowBox[{"k", ",", "\[Mu]"}], "]"}]], "Input",
 CellLabel->"In[876]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox[
    OverscriptBox["\[CapitalLambda]", "~"], "\[Mu]"], "(", "k", ")"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.5009702783070917`*^9},
 CellLabel->"Out[876]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Explicit", "[", "%", "]"}]], "Input",
 CellLabel->"In[877]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    SubscriptBox["g", "s"]}], " ", 
   SuperscriptBox[
    FormBox["k",
     TraditionalForm], 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970278366561*^9},
 CellLabel->"Out[877]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"%", "//", "Explicit"}], "//", "StandardForm"}]], "Input",
 CellTags->"GluonGhostVertex",
 CellLabel->"In[878]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"-", "Gstrong"}], " ", 
  RowBox[{"Pair", "[", 
   RowBox[{
    RowBox[{"LorentzIndex", "[", 
     RowBox[{"\[Mu]", ",", "D"}], "]"}], ",", 
    RowBox[{"Momentum", "[", 
     RowBox[{"k", ",", "D"}], "]"}]}], "]"}]}]], "Output",
 CellChangeTimes->{3.5009702784343452`*^9},
 CellTags->"GluonGhostVertex",
 CellLabel->"Out[878]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GluonGhostVertex", "[", 
  RowBox[{"p", ",", "1", ",", "q", ",", "2", ",", "k", ",", "3"}], 
  "]"}]], "Input",
 CellTags->"GluonGhostVertex",
 CellLabel->"In[879]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SuperscriptBox[
     OverscriptBox["\[CapitalLambda]", "~"], "li1"], "(", "k", ")"}], " ", 
   SubscriptBox["f", 
    RowBox[{
     FormBox[
      FormBox["ci1",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["ci2",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["ci3",
       TraditionalForm],
      TraditionalForm]}]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970278588562*^9},
 CellTags->"GluonGhostVertex",
 CellLabel->"Out[879]="]
}, Open  ]]
}, Open  ]],

Cell["GluonPropagator", "ObjectName",
 CellTags->"GluonPropagator"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"GluonPropagator"],

Cell[TextData[{
 "GluonPropagator[p, {mu, a}, {nu, b}] or GluonPropagator[p, mu, a , nu, b ] \
yields the gluon propagator. GluonPropagator[p, {mu}, {nu}] or \
GluonPropagator[p, mu, nu] omits the SUNDelta. The gauge and the dimension is \
determined by the option Gauge and Dimension. The following settings of Gauge \
are possible: 1 for the Feynman gauge; ",
 Cell[BoxData[
  FormBox["\[Alpha]", TraditionalForm]]],
 " for the general covariant gauge; {Momentum[n] ,1} for the axial gauge."
}], "Text",
 CellTags->"GluonPropagator"],

Cell["GP can be used as an abbreviation of GluonPropagator.", "Text",
 CellTags->"GluonPropagator"],

Cell[CellGroupData[{

Cell[BoxData["GP"], "Input",
 CellTags->"GluonPropagator",
 CellLabel->"In[880]:="],

Cell[BoxData[
 FormBox["GluonPropagator", TraditionalForm]], "Output",
 CellChangeTimes->{3.500970278729764*^9},
 CellTags->"GluonPropagator",
 CellLabel->"Out[880]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "GluonPropagator", "]"}]], "Input",
 CellTags->"GluonPropagator",
 CellLabel->"In[881]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"CounterTerm", "\[Rule]", "False"}], ",", 
    RowBox[{"CouplingConstant", "\[Rule]", 
     SubscriptBox["g", "s"]}], ",", 
    RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
    RowBox[{"Explicit", "\[Rule]", "False"}], ",", 
    RowBox[{"Gauge", "\[Rule]", "1"}], ",", 
    RowBox[{"\[CapitalOmega]", "\[Rule]", "False"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970278810543*^9},
 CellTags->"GluonPropagator",
 CellLabel->"Out[881]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["GluonSelfEnergy",
  BaseStyle->"AddOnsLink",
  ButtonData:>"GluonSelfEnergy",
  ButtonNote->"GluonSelfEnergy"],
 ", ",
 ButtonBox["GluonVertex",
  BaseStyle->"AddOnsLink",
  ButtonData:>"GluonVertex",
  ButtonNote->"GluonVertex"],
 ", ",
 ButtonBox["GhostPropagator",
  BaseStyle->"AddOnsLink",
  ButtonData:>"GhostPropagator",
  ButtonNote->"GhostPropagator"],
 ", ",
 ButtonBox["GluonGhostVertex",
  BaseStyle->"AddOnsLink",
  ButtonData:>"GluonGhostVertex",
  ButtonNote->"GluonGhostVertex"],
 "."
}], "Text",
 CellTags->"GluonPropagator"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData["Examples"], "Subsubsection",
 CellTags->"GluonPropagator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GluonPropagator", "[", 
  RowBox[{"p", ",", "\[Mu]", ",", "a", ",", "\[Nu]", ",", "b"}], 
  "]"}]], "Input",
 CellTags->"GluonPropagator",
 CellLabel->"In[882]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubsuperscriptBox["\[CapitalPi]", 
    RowBox[{"a", "b"}], 
    RowBox[{"\[Mu]", "\[Nu]"}]], "(", "p", ")"}], TraditionalForm]], "Output",\

 CellChangeTimes->{3.500970278896676*^9},
 CellTags->"GluonPropagator",
 CellLabel->"Out[882]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"GluonPropagator", "[", 
   RowBox[{"p", ",", "\[Mu]", ",", "a", ",", "\[Nu]", ",", "b"}], "]"}], "//",
   "Explicit"}]], "Input",
 CellLabel->"In[883]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   FractionBox[
    RowBox[{"\[ImaginaryI]", " ", 
     SubscriptBox["\[Delta]", 
      RowBox[{
       FormBox[
        FormBox["a",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["b",
         TraditionalForm],
        TraditionalForm]}]], " ", 
     SuperscriptBox["g", 
      RowBox[{"\[Mu]", "\[Nu]"}]]}], 
    SuperscriptBox["p", "2"]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970279373806*^9},
 CellLabel->"Out[883]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"GluonPropagator", "[", 
   RowBox[{"p", ",", "\[Mu]", ",", "a", ",", "\[Nu]", ",", "b", ",", 
    RowBox[{"Gauge", "\[Rule]", "\[Alpha]"}]}], "]"}], "//", 
  "Explicit"}]], "Input",
 CellTags->"GluonPropagator",
 CellLabel->"In[884]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{"\[ImaginaryI]", " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    RowBox[{"(", 
     RowBox[{
      FractionBox[
       RowBox[{
        RowBox[{"(", 
         RowBox[{"1", "-", "\[Alpha]"}], ")"}], " ", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], 
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm]], " ", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], 
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm]]}], 
       SuperscriptBox["p", "2"]], "-", 
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Nu]"}]]}], ")"}]}], 
   SuperscriptBox["p", "2"]], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009702795598*^9},
 CellTags->"GluonPropagator",
 CellLabel->"Out[884]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"GluonPropagator", "[", 
   RowBox[{"p", ",", "\[Mu]", ",", "a", ",", "\[Nu]", ",", "b", ",", 
    RowBox[{"Gauge", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"Momentum", "[", "n", "]"}], ",", "1"}], "}"}]}]}], "]"}], "//",
   "Explicit"}]], "Input",
 CellTags->"GluonPropagator",
 CellLabel->"In[885]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{"\[ImaginaryI]", " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", 
       SuperscriptBox["g", 
        RowBox[{"\[Mu]", "\[Nu]"}]]}], "-", 
      FractionBox[
       RowBox[{
        RowBox[{
         SuperscriptBox[
          FormBox["n",
           TraditionalForm], "2"], " ", 
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], 
          FormBox[
           FormBox["\[Mu]",
            TraditionalForm],
           TraditionalForm]], " ", 
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], 
          FormBox[
           FormBox["\[Nu]",
            TraditionalForm],
           TraditionalForm]]}], "-", 
        RowBox[{
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], "2"], " ", 
         SuperscriptBox[
          FormBox["n",
           TraditionalForm], 
          FormBox[
           FormBox["\[Mu]",
            TraditionalForm],
           TraditionalForm]], " ", 
         SuperscriptBox[
          FormBox["n",
           TraditionalForm], 
          FormBox[
           FormBox["\[Nu]",
            TraditionalForm],
           TraditionalForm]]}]}], 
       SuperscriptBox[
        FormBox[
         RowBox[{
          FormBox["n",
           TraditionalForm], "\[CenterDot]", 
          FormBox["p",
           TraditionalForm]}],
         TraditionalForm], "2"]], "+", 
      FractionBox[
       RowBox[{
        RowBox[{
         SuperscriptBox[
          FormBox["n",
           TraditionalForm], 
          FormBox[
           FormBox["\[Nu]",
            TraditionalForm],
           TraditionalForm]], " ", 
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], 
          FormBox[
           FormBox["\[Mu]",
            TraditionalForm],
           TraditionalForm]]}], "+", 
        RowBox[{
         SuperscriptBox[
          FormBox["n",
           TraditionalForm], 
          FormBox[
           FormBox["\[Mu]",
            TraditionalForm],
           TraditionalForm]], " ", 
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], 
          FormBox[
           FormBox["\[Nu]",
            TraditionalForm],
           TraditionalForm]]}]}], 
       RowBox[{
        FormBox["n",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p",
         TraditionalForm]}]]}], ")"}]}], 
   SuperscriptBox["p", "2"]], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009702797190657`*^9},
 CellTags->"GluonPropagator",
 CellLabel->"Out[885]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"GluonPropagator", "[", 
   RowBox[{"p", ",", "\[Mu]", ",", "\[Nu]"}], "]"}], "//", 
  "Explicit"}]], "Input",
 CellTags->"GluonPropagator",
 CellLabel->"In[886]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   FractionBox[
    RowBox[{"\[ImaginaryI]", " ", 
     SuperscriptBox["g", 
      RowBox[{"\[Mu]", "\[Nu]"}]]}], 
    SuperscriptBox["p", "2"]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.50097027989377*^9},
 CellTags->"GluonPropagator",
 CellLabel->"Out[886]="]
}, Open  ]],

Cell["\<\
This is a convenient way to enter amplitudes by hand (GP is an abbreviation \
GluonPropagator).\
\>", "Text",
 CellTags->"GluonPropagator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Explicit", "[", 
  RowBox[{
   RowBox[{"GP", "[", 
    RowBox[{"p", ",", "1", ",", "2"}], "]"}], ",", 
   RowBox[{"Gauge", "\[Rule]", "\[Xi]"}]}], "]"}]], "Input",
 CellTags->"GluonPropagator",
 CellLabel->"In[887]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{"\[ImaginaryI]", " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["ci1",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["ci2",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    RowBox[{"(", 
     RowBox[{
      FractionBox[
       RowBox[{
        RowBox[{"(", 
         RowBox[{"1", "-", "\[Xi]"}], ")"}], " ", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], 
         FormBox[
          FormBox["li1",
           TraditionalForm],
          TraditionalForm]], " ", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], 
         FormBox[
          FormBox["li2",
           TraditionalForm],
          TraditionalForm]]}], 
       SuperscriptBox["p", "2"]], "-", 
      SuperscriptBox["g", 
       RowBox[{"li1", "li2"}]]}], ")"}]}], 
   SuperscriptBox["p", "2"]], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009702800812187`*^9},
 CellTags->"GluonPropagator",
 CellLabel->"Out[887]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"GP", "[", 
     RowBox[{
      RowBox[{"-", "k"}], ",", "3", ",", "4"}], "]"}], "//", "Explicit"}], "//",
    "FCE"}], "//", "StandardForm"}]], "Input",
 CellTags->"GluonPropagator",
 CellLabel->"In[888]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"-", "\[ImaginaryI]"}], " ", 
  RowBox[{"FAD", "[", "k", "]"}], " ", 
  RowBox[{"MTD", "[", 
   RowBox[{"li3", ",", "li4"}], "]"}], " ", 
  RowBox[{"SD", "[", 
   RowBox[{"ci3", ",", "ci4"}], "]"}]}]], "Output",
 CellChangeTimes->{3.5009702802210913`*^9},
 CellTags->"GluonPropagator",
 CellLabel->"Out[888]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"GluonPropagator", "[", 
   RowBox[{"p", ",", "\[Mu]", ",", "a", ",", "\[Nu]", ",", "b", ",", 
    RowBox[{"CounterTerm", "\[Rule]", " ", "1"}]}], "]"}], "//", 
  "Explicit"}]], "Input",
 CellTags->"GluonPropagator",
 CellLabel->"In[889]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   FractionBox[
    RowBox[{"\[ImaginaryI]", " ", 
     SubscriptBox["C", "A"], " ", 
     SubsuperscriptBox["g", "s", "2"], " ", 
     SubscriptBox["S", "n"], " ", 
     SubscriptBox["\[Delta]", 
      RowBox[{
       FormBox[
        FormBox["a",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["b",
         TraditionalForm],
        TraditionalForm]}]], " ", 
     RowBox[{"(", 
      RowBox[{
       FractionBox[
        RowBox[{"11", " ", 
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], 
          FormBox[
           FormBox["\[Mu]",
            TraditionalForm],
           TraditionalForm]], " ", 
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], 
          FormBox[
           FormBox["\[Nu]",
            TraditionalForm],
           TraditionalForm]]}], "3"], "-", 
       RowBox[{
        FractionBox["19", "6"], " ", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], "2"], " ", 
        SuperscriptBox["g", 
         RowBox[{"\[Mu]", "\[Nu]"}]]}]}], ")"}]}], 
    TagBox["\[CurlyEpsilon]",
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970280375289*^9},
 CellTags->"GluonPropagator",
 CellLabel->"Out[889]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"GluonPropagator", "[", 
   RowBox[{"p", ",", "\[Mu]", ",", "a", ",", "\[Nu]", ",", "b", ",", 
    RowBox[{"CounterTerm", "\[Rule]", " ", "2"}]}], "]"}], "//", 
  "Explicit"}]], "Input",
 CellTags->"GluonPropagator",
 CellLabel->"In[890]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   FractionBox[
    RowBox[{"\[ImaginaryI]", " ", 
     SubscriptBox["C", "A"], " ", 
     SubsuperscriptBox["g", "s", "2"], " ", 
     SubscriptBox["S", "n"], " ", 
     SubscriptBox["\[Delta]", 
      RowBox[{
       FormBox[
        FormBox["a",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["b",
         TraditionalForm],
        TraditionalForm]}]], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"-", 
        RowBox[{
         FractionBox["1", "6"], " ", 
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], "2"], " ", 
         SuperscriptBox["g", 
          RowBox[{"\[Mu]", "\[Nu]"}]]}]}], "-", 
       RowBox[{
        FractionBox["1", "3"], " ", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], 
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm]], " ", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], 
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm]]}]}], ")"}]}], 
    TagBox["\[CurlyEpsilon]",
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009702804733047`*^9},
 CellTags->"GluonPropagator",
 CellLabel->"Out[890]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"GluonPropagator", "[", 
   RowBox[{"p", ",", "\[Mu]", ",", "a", ",", "\[Nu]", ",", "b", ",", 
    RowBox[{"CounterTerm", "\[Rule]", " ", "3"}]}], "]"}], "//", 
  "Explicit"}]], "Input",
 CellTags->"GluonPropagator",
 CellLabel->"In[891]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   FractionBox[
    RowBox[{"2", " ", "\[ImaginaryI]", " ", 
     SubscriptBox["T", "f"], " ", 
     SubsuperscriptBox["g", "s", "2"], " ", 
     SubscriptBox["S", "n"], " ", 
     SubscriptBox["\[Delta]", 
      RowBox[{
       FormBox[
        FormBox["a",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["b",
         TraditionalForm],
        TraditionalForm]}]], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        FractionBox["4", "3"], " ", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], "2"], " ", 
        SuperscriptBox["g", 
         RowBox[{"\[Mu]", "\[Nu]"}]]}], "-", 
       FractionBox[
        RowBox[{"4", " ", 
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], 
          FormBox[
           FormBox["\[Mu]",
            TraditionalForm],
           TraditionalForm]], " ", 
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], 
          FormBox[
           FormBox["\[Nu]",
            TraditionalForm],
           TraditionalForm]]}], "3"]}], ")"}]}], 
    TagBox["\[CurlyEpsilon]",
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.50097028056145*^9},
 CellTags->"GluonPropagator",
 CellLabel->"Out[891]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"GluonPropagator", "[", 
   RowBox[{"p", ",", "\[Mu]", ",", "a", ",", "\[Nu]", ",", "b", ",", 
    RowBox[{"CounterTerm", "\[Rule]", " ", "4"}]}], "]"}], "//", 
  "Explicit"}]], "Input",
 CellTags->"GluonPropagator",
 CellLabel->"In[892]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   FractionBox[
    RowBox[{"\[ImaginaryI]", " ", 
     SubscriptBox["C", "A"], " ", 
     SubsuperscriptBox["g", "s", "2"], " ", 
     SubscriptBox["S", "n"], " ", 
     SubscriptBox["\[Delta]", 
      RowBox[{
       FormBox[
        FormBox["a",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["b",
         TraditionalForm],
        TraditionalForm]}]], " ", 
     RowBox[{"(", 
      RowBox[{
       FractionBox[
        RowBox[{"10", " ", 
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], 
          FormBox[
           FormBox["\[Mu]",
            TraditionalForm],
           TraditionalForm]], " ", 
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], 
          FormBox[
           FormBox["\[Nu]",
            TraditionalForm],
           TraditionalForm]]}], "3"], "-", 
       RowBox[{
        FractionBox["10", "3"], " ", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], "2"], " ", 
        SuperscriptBox["g", 
         RowBox[{"\[Mu]", "\[Nu]"}]]}]}], ")"}]}], 
    TagBox["\[CurlyEpsilon]",
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009702806751413`*^9},
 CellTags->"GluonPropagator",
 CellLabel->"Out[892]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"GluonPropagator", "[", 
   RowBox[{"p", ",", "\[Mu]", ",", "a", ",", "\[Nu]", ",", "b", ",", 
    RowBox[{"CounterTerm", "\[Rule]", " ", "5"}]}], "]"}], "//", 
  "Explicit"}]], "Input",
 CellTags->"GluonPropagator",
 CellLabel->"In[893]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{"\[ImaginaryI]", " ", 
     SubscriptBox["C", "A"], " ", 
     SubsuperscriptBox["g", "s", "2"], " ", 
     SubscriptBox["S", "n"], " ", 
     SubscriptBox["\[Delta]", 
      RowBox[{
       FormBox[
        FormBox["a",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["b",
         TraditionalForm],
        TraditionalForm]}]], " ", 
     RowBox[{"(", 
      RowBox[{
       FractionBox[
        RowBox[{"10", " ", 
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], 
          FormBox[
           FormBox["\[Mu]",
            TraditionalForm],
           TraditionalForm]], " ", 
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], 
          FormBox[
           FormBox["\[Nu]",
            TraditionalForm],
           TraditionalForm]]}], "3"], "-", 
       RowBox[{
        FractionBox["10", "3"], " ", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], "2"], " ", 
        SuperscriptBox["g", 
         RowBox[{"\[Mu]", "\[Nu]"}]]}]}], ")"}]}], 
    TagBox["\[CurlyEpsilon]",
     TraditionalForm]], "+", 
   FractionBox[
    RowBox[{"\[ImaginaryI]", " ", 
     SubscriptBox["T", "f"], " ", 
     SubsuperscriptBox["g", "s", "2"], " ", 
     SubscriptBox["S", "n"], " ", 
     SubscriptBox["\[Delta]", 
      RowBox[{
       FormBox[
        FormBox["a",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["b",
         TraditionalForm],
        TraditionalForm]}]], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        FractionBox["4", "3"], " ", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], "2"], " ", 
        SuperscriptBox["g", 
         RowBox[{"\[Mu]", "\[Nu]"}]]}], "-", 
       FractionBox[
        RowBox[{"4", " ", 
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], 
          FormBox[
           FormBox["\[Mu]",
            TraditionalForm],
           TraditionalForm]], " ", 
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], 
          FormBox[
           FormBox["\[Nu]",
            TraditionalForm],
           TraditionalForm]]}], "3"]}], ")"}]}], 
    TagBox["\[CurlyEpsilon]",
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970280887005*^9},
 CellTags->"GluonPropagator",
 CellLabel->"Out[893]="]
}, Open  ]]
}, Open  ]],

Cell["GluonSelfEnergy", "ObjectName",
 CellTags->"GluonSelfEnergy"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"GluonSelfEnergy"],

Cell["\<\
GluonSelfEnergy[{mu, a}, {nu,b}] yields the 1-loop Gluon selfenergy.\
\>", "Text",
 CellTags->"GluonSelfEnergy"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "GluonSelfEnergy", "]"}]], "Input",
 CellTags->"GluonSelfEnergy",
 CellLabel->"In[894]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
    RowBox[{"CouplingConstant", "\[Rule]", 
     SubscriptBox["g", "s"]}], ",", 
    RowBox[{"FinalSubstitutions", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{
        RowBox[{"log", "(", 
         RowBox[{"_", " ", 
          SuperscriptBox["\[Mu]", "2"]}], ")"}], "\[RuleDelayed]", "0"}], ",", 
       RowBox[{
        TagBox["\[DoubledGamma]",
         Function[{}, EulerGamma]], "\[RuleDelayed]", 
        RowBox[{"log", "(", 
         RowBox[{"4", " ", "\[Pi]"}], ")"}]}]}], "}"}]}], ",", 
    RowBox[{"Gauge", "\[Rule]", "1"}], ",", 
    RowBox[{"Momentum", "\[Rule]", "p"}]}], "}"}], TraditionalForm]], "Output",\

 CellChangeTimes->{3.50097028103833*^9},
 CellTags->"GluonSelfEnergy",
 CellLabel->"Out[894]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["GluonPropagator",
  BaseStyle->"AddOnsLink",
  ButtonData:>"GluonPropagator",
  ButtonNote->"GluonPropagator"],
 ", ",
 ButtonBox["GluonVertex",
  BaseStyle->"AddOnsLink",
  ButtonData:>"GluonVertex",
  ButtonNote->"GluonVertex"],
 ", ",
 ButtonBox["GhostPropagator",
  BaseStyle->"AddOnsLink",
  ButtonData:>"GhostPropagator",
  ButtonNote->"GhostPropagator"],
 ", ",
 ButtonBox["GluonGhostVertex",
  BaseStyle->"AddOnsLink",
  ButtonData:>"GluonGhostVertex",
  ButtonNote->"GluonGhostVertex"],
 "."
}], "Text",
 CellTags->"GluonSelfEnergy"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData["Examples"], "Subsubsection",
 CellTags->"GluonSelfEnergy"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GluonSelfEnergy", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"\[Mu]", ",", "a"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\[Nu]", ",", "b"}], "}"}]}], "]"}]], "Input",
 CellTags->"GluonSelfEnergy",
 CellLabel->"In[895]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    FractionBox["1", "2"], " ", "\[ImaginaryI]", " ", 
    RowBox[{"(", 
     RowBox[{
      FractionBox["20", 
       RowBox[{"3", " ", 
        TagBox["\[CurlyEpsilon]",
         TraditionalForm]}]], "-", 
      FractionBox["62", "9"]}], ")"}], " ", 
    SubscriptBox["C", "A"], " ", 
    SubsuperscriptBox["g", "s", "2"], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], "2"], " ", 
       SuperscriptBox["g", 
        RowBox[{"\[Mu]", "\[Nu]"}]]}]}], ")"}]}], "+", 
   RowBox[{"\[ImaginaryI]", " ", 
    RowBox[{"(", 
     RowBox[{
      FractionBox["20", "9"], "-", 
      FractionBox["8", 
       RowBox[{"3", " ", 
        TagBox["\[CurlyEpsilon]",
         TraditionalForm]}]]}], ")"}], " ", 
    SubscriptBox["T", "f"], " ", 
    SubsuperscriptBox["g", "s", "2"], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], "2"], " ", 
       SuperscriptBox["g", 
        RowBox[{"\[Mu]", "\[Nu]"}]]}]}], ")"}]}]}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970281558572*^9},
 CellTags->"GluonSelfEnergy",
 CellLabel->"Out[895]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GluonSelfEnergy", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"\[Mu]", ",", "a"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\[Nu]", ",", "b"}], "}"}], ",", 
   RowBox[{"Gauge", "\[Rule]", "\[Xi]"}]}], "]"}]], "Input",
 CellTags->"GluonSelfEnergy",
 CellLabel->"In[896]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    FractionBox["1", "2"], " ", "\[ImaginaryI]", " ", 
    SubscriptBox["C", "A"], " ", 
    RowBox[{"(", 
     RowBox[{
      FractionBox[
       RowBox[{"2", " ", 
        RowBox[{"(", 
         RowBox[{
          FractionBox["13", "3"], "-", "\[Xi]"}], ")"}]}], 
       TagBox["\[CurlyEpsilon]",
        TraditionalForm]], "-", 
      RowBox[{
       FractionBox["1", "2"], " ", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"1", "-", "\[Xi]"}], ")"}], "2"]}], "+", 
      RowBox[{"2", " ", 
       RowBox[{"(", 
        RowBox[{"1", "-", "\[Xi]"}], ")"}]}], "-", 
      FractionBox["62", "9"]}], ")"}], " ", 
    SubsuperscriptBox["g", "s", "2"], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], "2"], " ", 
       SuperscriptBox["g", 
        RowBox[{"\[Mu]", "\[Nu]"}]]}]}], ")"}]}], "+", 
   RowBox[{"\[ImaginaryI]", " ", 
    RowBox[{"(", 
     RowBox[{
      FractionBox["20", "9"], "-", 
      FractionBox["8", 
       RowBox[{"3", " ", 
        TagBox["\[CurlyEpsilon]",
         TraditionalForm]}]]}], ")"}], " ", 
    SubscriptBox["T", "f"], " ", 
    SubsuperscriptBox["g", "s", "2"], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], "2"], " ", 
       SuperscriptBox["g", 
        RowBox[{"\[Mu]", "\[Nu]"}]]}]}], ")"}]}]}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970281735392*^9},
 CellTags->"GluonSelfEnergy",
 CellLabel->"Out[896]="]
}, Open  ]]
}, Open  ]],

Cell["GluonVertex", "ObjectName",
 CellTags->"GluonVertex"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"GluonVertex"],

Cell["Note: All momenta are flowing into the vertex.", "Text",
 CellTags->"GluonVertex"],

Cell["\<\
GluonVertex[{p,mu,a}, {q,nu,b}, {k,la,c}] or GluonVertex[p,mu,a , q,nu,b , \
k,la,c ] yields the 3-gluon vertex. GluonVertex[{p,mu}, {q,nu}, {k,la}] \
yields the 3-gluon vertex without color structure and the coupling constant. \
GluonVertex[{p,mu,a}, {q,nu,b}, {k,la,c}, {s,si,d}] or GluonVertex[{mu,a}, \
{nu,b}, {la,c}, {si,d}] or GluonVertex[p,mu,a , q,nu,b , k,la,c , s,si,d] or \
GluonVertex[ mu,a , nu,b , la,c , si,d ] yields the 4-gluon vertex. The \
dimension and the name of the coupling constant are determined by the options \
Dimension and CouplingConstant.\
\>", "Text",
 CellTags->"GluonVertex"],

Cell["GV can be used as an abbreviation of GluonVertex.", "Text",
 CellTags->"GluonVertex"],

Cell[CellGroupData[{

Cell[BoxData["GV"], "Input",
 CellTags->"GluonVertex",
 CellLabel->"In[897]:="],

Cell[BoxData[
 FormBox["GluonVertex", TraditionalForm]], "Output",
 CellChangeTimes->{3.5009702817917*^9},
 CellTags->"GluonVertex",
 CellLabel->"Out[897]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "GluonVertex", "]"}]], "Input",
 CellTags->"GluonVertex",
 CellLabel->"In[898]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"CouplingConstant", "\[Rule]", 
     SubscriptBox["g", "s"]}], ",", 
    RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
    RowBox[{"Explicit", "\[Rule]", "False"}], ",", 
    RowBox[{"\[CapitalOmega]", "\[Rule]", "False"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.50097028184295*^9},
 CellTags->"GluonVertex",
 CellLabel->"Out[898]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["GluonGhostVertex",
  BaseStyle->"AddOnsLink",
  ButtonData:>"GluonGhostVertex",
  ButtonNote->"GluonGhostVertex"],
 ", ",
 ButtonBox["GluonPropagator",
  BaseStyle->"AddOnsLink",
  ButtonData:>"GluonPropagator",
  ButtonNote->"GluonPropagator"],
 "."
}], "Text",
 CellTags->"GluonVertex"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"GluonVertex"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GluonVertex", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"p", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"q", ",", "\[Nu]", ",", "b"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"r", ",", "\[Rho]", ",", "c"}], "}"}]}], "]"}]], "Input",
 CellTags->"GluonVertex",
 CellLabel->"In[899]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["f", 
    RowBox[{
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm]}]], " ", 
   RowBox[{
    SuperscriptBox["V", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Rho]",
        TraditionalForm],
       TraditionalForm]}]], "(", 
    RowBox[{
     FormBox["p",
      TraditionalForm], 
     RowBox[{",", " "}], 
     FormBox["q",
      TraditionalForm], 
     RowBox[{",", " "}], 
     FormBox["r",
      TraditionalForm]}], ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009702819279633`*^9},
 CellTags->"GluonVertex",
 CellLabel->"Out[899]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"GluonVertex", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"p", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"q", ",", "\[Nu]", ",", "b"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"r", ",", "\[Rho]", ",", "c"}], "}"}]}], "]"}], "//", 
  "Explicit"}]], "Input",
 CellTags->"GluonVertex",
 CellLabel->"In[900]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["g", "s"], " ", 
   SubscriptBox["f", 
    RowBox[{
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm]}]], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], 
         FormBox[
          FormBox["\[Rho]",
           TraditionalForm],
          TraditionalForm]], "-", 
        SuperscriptBox[
         FormBox["q",
          TraditionalForm], 
         FormBox[
          FormBox["\[Rho]",
           TraditionalForm],
          TraditionalForm]]}], ")"}]}], "+", 
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Rho]"}]], " ", 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox[
         FormBox["r",
          TraditionalForm], 
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm]], "-", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], 
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm]]}], ")"}]}], "+", 
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Nu]", "\[Rho]"}]], " ", 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox[
         FormBox["q",
          TraditionalForm], 
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm]], "-", 
        SuperscriptBox[
         FormBox["r",
          TraditionalForm], 
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm]]}], ")"}]}]}], ")"}]}], TraditionalForm]], "Output",\

 CellChangeTimes->{3.500970282041479*^9},
 CellTags->"GluonVertex",
 CellLabel->"Out[900]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GluonVertex", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"p", ",", "\[Mu]"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"q", ",", "\[Nu]"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"r", ",", "\[Rho]"}], "}"}]}], "]"}]], "Input",
 CellTags->"GluonVertex",
 CellLabel->"In[901]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["V", 
    RowBox[{"\[Mu]", "\[Nu]", "\[Rho]"}]], "(", 
   RowBox[{"p", 
    RowBox[{",", " "}], "q", 
    RowBox[{",", " "}], "r"}], ")"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970282126935*^9},
 CellTags->"GluonVertex",
 CellLabel->"Out[901]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"GluonVertex", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"p", ",", "\[Mu]"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"q", ",", "\[Nu]"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"r", ",", "\[Rho]"}], "}"}]}], "]"}], "//", 
  "Explicit"}]], "Input",
 CellTags->"GluonVertex",
 CellLabel->"In[902]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["g", "s"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], 
         FormBox[
          FormBox["\[Rho]",
           TraditionalForm],
          TraditionalForm]], "-", 
        SuperscriptBox[
         FormBox["q",
          TraditionalForm], 
         FormBox[
          FormBox["\[Rho]",
           TraditionalForm],
          TraditionalForm]]}], ")"}]}], "+", 
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Rho]"}]], " ", 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox[
         FormBox["r",
          TraditionalForm], 
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm]], "-", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], 
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm]]}], ")"}]}], "+", 
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Nu]", "\[Rho]"}]], " ", 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox[
         FormBox["q",
          TraditionalForm], 
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm]], "-", 
        SuperscriptBox[
         FormBox["r",
          TraditionalForm], 
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm]]}], ")"}]}]}], ")"}]}], TraditionalForm]], "Output",\

 CellChangeTimes->{3.500970282331934*^9},
 CellTags->"GluonVertex",
 CellLabel->"Out[902]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GluonVertex", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"p", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"q", ",", "\[Nu]", ",", "b"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"r", ",", "\[Rho]", ",", "c"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"s", ",", "\[Sigma]", ",", "d"}], "}"}]}], "]"}]], "Input",
 CellTags->"GluonVertex",
 CellLabel->"In[903]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubsuperscriptBox["V", 
    RowBox[{"a", "b", "c", "d"}], 
    RowBox[{"\[Mu]", "\[Nu]", "\[Rho]", "\[Sigma]"}]], "(", 
   RowBox[{"p", 
    RowBox[{",", " "}], "q", 
    RowBox[{",", " "}], "r", 
    RowBox[{",", " "}], "s"}], ")"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970282431205*^9},
 CellTags->"GluonVertex",
 CellLabel->"Out[903]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"GluonVertex", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"p", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"q", ",", "\[Nu]", ",", "b"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"r", ",", "\[Rho]", ",", "c"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"s", ",", "\[Sigma]", ",", "d"}], "}"}]}], "]"}], "//", 
  "Explicit"}]], "Input",
 CellLabel->"In[904]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", "\[ImaginaryI]"}], " ", 
   SubsuperscriptBox["g", "s", "2"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["a",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["d",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["u123",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["b",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["c",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["u123",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         SuperscriptBox["g", 
          RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
         SuperscriptBox["g", 
          RowBox[{"\[Rho]", "\[Sigma]"}]]}], "-", 
        RowBox[{
         SuperscriptBox["g", 
          RowBox[{"\[Mu]", "\[Rho]"}]], " ", 
         SuperscriptBox["g", 
          RowBox[{"\[Nu]", "\[Sigma]"}]]}]}], ")"}]}], "+", 
     RowBox[{
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["a",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["c",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["u123",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["b",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["d",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["u123",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         SuperscriptBox["g", 
          RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
         SuperscriptBox["g", 
          RowBox[{"\[Rho]", "\[Sigma]"}]]}], "-", 
        RowBox[{
         SuperscriptBox["g", 
          RowBox[{"\[Mu]", "\[Sigma]"}]], " ", 
         SuperscriptBox["g", 
          RowBox[{"\[Nu]", "\[Rho]"}]]}]}], ")"}]}], "+", 
     RowBox[{
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["a",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["b",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["u123",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["c",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["d",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["u123",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         SuperscriptBox["g", 
          RowBox[{"\[Mu]", "\[Rho]"}]], " ", 
         SuperscriptBox["g", 
          RowBox[{"\[Nu]", "\[Sigma]"}]]}], "-", 
        RowBox[{
         SuperscriptBox["g", 
          RowBox[{"\[Mu]", "\[Sigma]"}]], " ", 
         SuperscriptBox["g", 
          RowBox[{"\[Nu]", "\[Rho]"}]]}]}], ")"}]}]}], ")"}]}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970282623201*^9},
 CellLabel->"Out[904]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GluonVertex", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"\[Mu]", ",", "a"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\[Nu]", ",", "b"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\[Rho]", ",", "c"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\[Sigma]", ",", "d"}], "}"}]}], "]"}]], "Input",
 CellTags->"GluonVertex",
 CellLabel->"In[905]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["W", 
   RowBox[{"a", "b", "c", "d"}]], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970282712346*^9},
 CellTags->"GluonVertex",
 CellLabel->"Out[905]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"GluonVertex", "[", 
   RowBox[{
   "\[Mu]", ",", "a", ",", "\[Nu]", ",", "b", ",", "\[Rho]", ",", "c", ",", 
    "\[Sigma]", ",", "d"}], "]"}], "//", "Explicit"}]], "Input",
 CellTags->"GluonVertex",
 CellLabel->"In[906]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", "\[ImaginaryI]"}], " ", 
   SubsuperscriptBox["g", "s", "2"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["a",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["d",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["u124",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["b",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["c",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["u124",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         SuperscriptBox["g", 
          RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
         SuperscriptBox["g", 
          RowBox[{"\[Rho]", "\[Sigma]"}]]}], "-", 
        RowBox[{
         SuperscriptBox["g", 
          RowBox[{"\[Mu]", "\[Rho]"}]], " ", 
         SuperscriptBox["g", 
          RowBox[{"\[Nu]", "\[Sigma]"}]]}]}], ")"}]}], "+", 
     RowBox[{
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["a",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["c",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["u124",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["b",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["d",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["u124",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         SuperscriptBox["g", 
          RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
         SuperscriptBox["g", 
          RowBox[{"\[Rho]", "\[Sigma]"}]]}], "-", 
        RowBox[{
         SuperscriptBox["g", 
          RowBox[{"\[Mu]", "\[Sigma]"}]], " ", 
         SuperscriptBox["g", 
          RowBox[{"\[Nu]", "\[Rho]"}]]}]}], ")"}]}], "+", 
     RowBox[{
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["a",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["b",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["u124",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["c",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["d",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["u124",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         SuperscriptBox["g", 
          RowBox[{"\[Mu]", "\[Rho]"}]], " ", 
         SuperscriptBox["g", 
          RowBox[{"\[Nu]", "\[Sigma]"}]]}], "-", 
        RowBox[{
         SuperscriptBox["g", 
          RowBox[{"\[Mu]", "\[Sigma]"}]], " ", 
         SuperscriptBox["g", 
          RowBox[{"\[Nu]", "\[Rho]"}]]}]}], ")"}]}]}], ")"}]}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.5009702828535337`*^9},
 CellTags->"GluonVertex",
 CellLabel->"Out[906]="]
}, Open  ]],

Cell["\<\
A very convenient way to enter diagrams by hand is to label each line hitting \
a vertex by a number and put this number after the inflowing momentum.\
\>", "Text",
 CellTags->"GluonVertex"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GV", "[", 
  RowBox[{"p", ",", "1", ",", "q", ",", "2", ",", "r", ",", "3"}], 
  "]"}]], "Input",
 CellTags->"GluonVertex",
 CellLabel->"In[907]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["f", 
    RowBox[{
     FormBox[
      FormBox["ci1",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["ci2",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["ci3",
       TraditionalForm],
      TraditionalForm]}]], " ", 
   RowBox[{
    SuperscriptBox["V", 
     RowBox[{
      FormBox[
       FormBox["li1",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["li2",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["li3",
        TraditionalForm],
       TraditionalForm]}]], "(", 
    RowBox[{
     FormBox["p",
      TraditionalForm], 
     RowBox[{",", " "}], 
     FormBox["q",
      TraditionalForm], 
     RowBox[{",", " "}], 
     FormBox["r",
      TraditionalForm]}], ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970282942582*^9},
 CellTags->"GluonVertex",
 CellLabel->"Out[907]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"GV", "[", 
   RowBox[{
   "p", ",", "1", ",", "q", ",", "2", ",", "r", ",", "3", ",", "s", ",", 
    "4"}], "]"}], "//", "Explicit"}]], "Input",
 CellTags->"GluonVertex",
 CellLabel->"In[908]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", "\[ImaginaryI]"}], " ", 
   SubsuperscriptBox["g", "s", "2"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["ci1",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["ci4",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["u125",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["ci2",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["ci3",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["u125",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         SuperscriptBox["g", 
          RowBox[{"li1", "li2"}]], " ", 
         SuperscriptBox["g", 
          RowBox[{"li3", "li4"}]]}], "-", 
        RowBox[{
         SuperscriptBox["g", 
          RowBox[{"li1", "li3"}]], " ", 
         SuperscriptBox["g", 
          RowBox[{"li2", "li4"}]]}]}], ")"}]}], "+", 
     RowBox[{
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["ci1",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["ci3",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["u125",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["ci2",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["ci4",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["u125",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         SuperscriptBox["g", 
          RowBox[{"li1", "li2"}]], " ", 
         SuperscriptBox["g", 
          RowBox[{"li3", "li4"}]]}], "-", 
        RowBox[{
         SuperscriptBox["g", 
          RowBox[{"li1", "li4"}]], " ", 
         SuperscriptBox["g", 
          RowBox[{"li2", "li3"}]]}]}], ")"}]}], "+", 
     RowBox[{
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["ci1",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["ci2",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["u125",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["ci3",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["ci4",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["u125",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         SuperscriptBox["g", 
          RowBox[{"li1", "li3"}]], " ", 
         SuperscriptBox["g", 
          RowBox[{"li2", "li4"}]]}], "-", 
        RowBox[{
         SuperscriptBox["g", 
          RowBox[{"li1", "li4"}]], " ", 
         SuperscriptBox["g", 
          RowBox[{"li2", "li3"}]]}]}], ")"}]}]}], ")"}]}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.50097028315781*^9},
 CellTags->"GluonVertex",
 CellLabel->"Out[908]="]
}, Open  ]]
}, Open  ]],

Cell["GO", "ObjectName",
 CellTags->"GO"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"GO"],

Cell["GO is equivalent to Twist2GluonOperator.", "Text",
 CellTags->"GO"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Twist2GluonOperator",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Twist2GluonOperator",
  ButtonNote->"Twist2GluonOperator"],
 "."
}], "Text",
 CellTags->"GO"]
}, Open  ]],

Cell["GP", "ObjectName",
 CellTags->"GP"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"GP"],

Cell["GP is equivalent to GluonPropagator.", "Text",
 CellTags->"GP"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["GluonPropagator",
  BaseStyle->"AddOnsLink",
  ButtonData:>"GluonPropagator",
  ButtonNote->"GluonPropagator"],
 "."
}], "Text",
 CellTags->"GP"]
}, Open  ]],

Cell["GrassmannParity", "ObjectName",
 CellTags->"GrassmannParity"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"GrassmannParity"],

Cell["\<\
GrassmannParity is a data type. E.g. DataType[F, GrassmannParity] = 1 \
declares F to be of bosonic type and DataType[F, GrassmannParity] = -1 of \
fermionic one.\
\>", "Text",
 CellTags->"GrassmannParity"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["DataType",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DataType",
  ButtonNote->"DataType"],
 "."
}], "Text",
 CellTags->"GrassmannParity"]
}, Open  ]],

Cell["GS", "ObjectName",
 CellTags->"GS"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"GS"],

Cell[TextData[{
 "GS[p] can be used as input for a 4-dimensional ",
 Cell[BoxData[
  FormBox[
   RowBox[{
   "p", " \[NegativeMediumSpace]", 
    "\[NegativeMediumSpace]\[NegativeMediumSpace]", "\[NegativeMediumSpace]", 
    "/", " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"=", 
       RowBox[{"\[Gamma]", ".", "p"}]}], " ", "=", " ", 
      RowBox[{
       SubscriptBox["\[Gamma]", "\[Mu]"], 
       SuperscriptBox["p", "\[Mu]"]}]}], ")"}]}], TraditionalForm]]],
 " and is transformed into DiracGamma[Momentum[p]] by FeynCalcInternal \
(=FCI). GS[p,q, ...] is a short form for GS[p].GS[q]. ... ."
}], "Text",
 CellTags->"GS"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["DiracGamma",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracGamma",
  ButtonNote->"DiracGamma"],
 ", ",
 ButtonBox["DiracSlash",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracSlash",
  ButtonNote->"DiracSlash"],
 ", ",
 ButtonBox["GA",
  BaseStyle->"AddOnsLink",
  ButtonData:>"GA",
  ButtonNote->"GA"],
 ", ",
 ButtonBox["GAD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"GAD",
  ButtonNote->"GAD"],
 "."
}], "Text",
 CellTags->"GS"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"GS"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GS", "[", "p", "]"}]], "Input",
 CellTags->"GS",
 CellLabel->"In[909]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"\[Gamma]", "\[CenterDot]", "p"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009702833219767`*^9},
 CellTags->"GS",
 CellLabel->"Out[909]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"GS", "[", "p", "]"}], "//", "FCI"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"GS",
 CellLabel->"In[910]:="],

Cell[BoxData[
 RowBox[{"DiracGamma", "[", 
  RowBox[{"Momentum", "[", "p", "]"}], "]"}]], "Output",
 CellChangeTimes->{3.500970283400175*^9},
 CellTags->"GS",
 CellLabel->"Out[910]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GS", "[", 
  RowBox[{"p", ",", "q", ",", "r", ",", "s"}], "]"}]], "Input",
 CellTags->"GS",
 CellLabel->"In[911]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"(", 
    RowBox[{"\[Gamma]", "\[CenterDot]", "p"}], ")"}], ".", 
   RowBox[{"(", 
    RowBox[{"\[Gamma]", "\[CenterDot]", "q"}], ")"}], ".", 
   RowBox[{"(", 
    RowBox[{"\[Gamma]", "\[CenterDot]", "r"}], ")"}], ".", 
   RowBox[{"(", 
    RowBox[{"\[Gamma]", "\[CenterDot]", "s"}], ")"}]}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970283479463*^9},
 CellTags->"GS",
 CellLabel->"Out[911]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"GS", "[", 
   RowBox[{"p", ",", "q", ",", "r", ",", "s"}], "]"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"GS",
 CellLabel->"In[912]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"GS", "[", "p", "]"}], ".", 
  RowBox[{"GS", "[", "q", "]"}], ".", 
  RowBox[{"GS", "[", "r", "]"}], ".", 
  RowBox[{"GS", "[", "s", "]"}]}]], "Output",
 CellChangeTimes->{3.500970283584775*^9},
 CellTags->"GS",
 CellLabel->"Out[912]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"GS", "[", "q", "]"}], ".", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{"GS", "[", "p", "]"}], "+", "m"}], ")"}], ".", 
  RowBox[{"GS", "[", "q", "]"}]}]], "Input",
 CellTags->"GS",
 CellLabel->"In[913]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"(", 
    RowBox[{"\[Gamma]", "\[CenterDot]", "q"}], ")"}], ".", 
   RowBox[{"(", 
    RowBox[{"m", "+", 
     RowBox[{"\[Gamma]", "\[CenterDot]", "p"}]}], ")"}], ".", 
   RowBox[{"(", 
    RowBox[{"\[Gamma]", "\[CenterDot]", "q"}], ")"}]}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.5009702836604757`*^9},
 CellTags->"GS",
 CellLabel->"Out[913]="]
}, Open  ]]
}, Open  ]],

Cell["GSD", "ObjectName",
 CellTags->"GSD"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"GSD"],

Cell[TextData[{
 "GSD[p] can be used as input for a D-dimensional ",
 Cell[BoxData[
  FormBox[
   RowBox[{
   "p", " \[NegativeMediumSpace]", 
    "\[NegativeMediumSpace]\[NegativeMediumSpace]", "\[NegativeMediumSpace]", 
    "/", " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"=", 
       RowBox[{"\[Gamma]", ".", "p"}]}], " ", "=", " ", 
      RowBox[{
       SubscriptBox["\[Gamma]", "\[Mu]"], 
       SuperscriptBox["p", "\[Mu]"]}]}], ")"}]}], TraditionalForm]]],
 " and is transformed into DiracGamma[Momentum[p,D],D] by FeynCalcInternal \
(=FCI). GSD[p,q, ...] is a short form for GSD[p].GSD[q]. ... ."
}], "Text",
 CellTags->"GSD"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["DiracGamma",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracGamma",
  ButtonNote->"DiracGamma"],
 ", ",
 ButtonBox["DiracSlash",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracSlash",
  ButtonNote->"DiracSlash"],
 ", ",
 ButtonBox["GA",
  BaseStyle->"AddOnsLink",
  ButtonData:>"GA",
  ButtonNote->"GA"],
 ", ",
 ButtonBox["GAD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"GAD",
  ButtonNote->"GAD"],
 "."
}], "Text",
 CellTags->"GSD"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"GSD"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GSD", "[", "p", "]"}]], "Input",
 CellTags->"GSD",
 CellLabel->"In[914]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"\[Gamma]", "\[CenterDot]", "p"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970283739382*^9},
 CellTags->"GSD",
 CellLabel->"Out[914]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"GSD", "[", "p", "]"}], "//", "FCI"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"GSD",
 CellLabel->"In[915]:="],

Cell[BoxData[
 RowBox[{"DiracGamma", "[", 
  RowBox[{
   RowBox[{"Momentum", "[", 
    RowBox[{"p", ",", "D"}], "]"}], ",", "D"}], "]"}]], "Output",
 CellChangeTimes->{3.5009702838186398`*^9},
 CellTags->"GSD",
 CellLabel->"Out[915]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GSD", "[", 
  RowBox[{"p", ",", "q", ",", "r", ",", "s"}], "]"}]], "Input",
 CellTags->"GSD",
 CellLabel->"In[916]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"(", 
    RowBox[{"\[Gamma]", "\[CenterDot]", "p"}], ")"}], ".", 
   RowBox[{"(", 
    RowBox[{"\[Gamma]", "\[CenterDot]", "q"}], ")"}], ".", 
   RowBox[{"(", 
    RowBox[{"\[Gamma]", "\[CenterDot]", "r"}], ")"}], ".", 
   RowBox[{"(", 
    RowBox[{"\[Gamma]", "\[CenterDot]", "s"}], ")"}]}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970283865507*^9},
 CellTags->"GSD",
 CellLabel->"Out[916]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"GSD", "[", 
   RowBox[{"p", ",", "q", ",", "r", ",", "s"}], "]"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"GSD",
 CellLabel->"In[917]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"GSD", "[", "p", "]"}], ".", 
  RowBox[{"GSD", "[", "q", "]"}], ".", 
  RowBox[{"GSD", "[", "r", "]"}], ".", 
  RowBox[{"GSD", "[", "s", "]"}]}]], "Output",
 CellChangeTimes->{3.500970283912166*^9},
 CellTags->"GSD",
 CellLabel->"Out[917]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"GSD", "[", "q", "]"}], ".", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{"GSD", "[", "p", "]"}], "+", "m"}], ")"}], ".", 
  RowBox[{"GSD", "[", "q", "]"}]}]], "Input",
 CellTags->"GSD",
 CellLabel->"In[918]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"(", 
    RowBox[{"\[Gamma]", "\[CenterDot]", "q"}], ")"}], ".", 
   RowBox[{"(", 
    RowBox[{"m", "+", 
     RowBox[{"\[Gamma]", "\[CenterDot]", "p"}]}], ")"}], ".", 
   RowBox[{"(", 
    RowBox[{"\[Gamma]", "\[CenterDot]", "q"}], ")"}]}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.5009702839867983`*^9},
 CellTags->"GSD",
 CellLabel->"Out[918]="]
}, Open  ]]
}, Open  ]],

Cell["Gstrong", "ObjectName",
 CellTags->"Gstrong"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Gstrong"],

Cell["Gstrong denotes the strong coupling constant.", "Text",
 CellTags->"Gstrong"],

Cell[TextData[{
 "See also: ",
 ButtonBox["CovariantD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"CovariantD"],
 ", ",
 ButtonBox["FieldStrength",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FieldStrength"],
 ", ",
 ButtonBox["GluonVertex",
  BaseStyle->"AddOnsLink",
  ButtonData:>"GluonVertex"],
 "."
}], "Text",
 CellTags->"Gstrong"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Gstrong"],

Cell["\<\
Gstrong has no functional properties. Only a typesetting rule is defined.\
\>", "Text",
 CellTags->"Gstrong"],

Cell[CellGroupData[{

Cell[BoxData["Gstrong"], "Input",
 CellTags->"Gstrong",
 CellLabel->"In[919]:="],

Cell[BoxData[
 FormBox[
  SubscriptBox["g", "s"], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970284031008*^9},
 CellTags->"Gstrong",
 CellLabel->"Out[919]="]
}, Open  ]]
}, Open  ]],

Cell["GTI", "ObjectName",
 CellTags->"GTI"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"GTI"],

Cell["GTI is like RHI, but with no functional properties.", "Text",
 CellTags->"GTI"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["RHI",
  BaseStyle->"AddOnsLink",
  ButtonData:>"RHI",
  ButtonNote->"RHI"],
 "."
}], "Text",
 CellTags->"GTI"]
}, Open  ]],

Cell["GV", "ObjectName",
 CellTags->"GV"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"GV"],

Cell["GV is equivalent to GluonVertex.", "Text",
 CellTags->"GV"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["GluonVertex",
  BaseStyle->"AddOnsLink",
  ButtonData:>"GluonVertex",
  ButtonNote->"GluonVertex"],
 "."
}], "Text",
 CellTags->"GV"]
}, Open  ]],

Cell["Hill", "ObjectName",
 CellTags->"Hill"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Hill"],

Cell["\<\
Hill[x, y] gives the Hill identity with arguments x and y. The returned \
object is 0.\
\>", "Text",
 CellTags->"Hill"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["SimplifyPolyLog",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SimplifyPolyLog",
  ButtonNote->"SimplifyPolyLog"],
 "."
}], "Text",
 CellTags->"Hill"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Hill"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Hill", "[", 
  RowBox[{"a", ",", "b"}], "]"}]], "Input",
 CellTags->"Hill",
 CellLabel->"In[920]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TemplateBox[{"2",FractionBox[
      RowBox[{"1", "-", "a"}], 
      RowBox[{"1", "-", "b"}]]},
    "PolyLog"], "+", 
   TemplateBox[{"2",FractionBox["b", "a"]},
    "PolyLog"], "-", 
   TemplateBox[{"2",FractionBox[
      RowBox[{
        RowBox[{"(", 
          RowBox[{"1", "-", "a"}], ")"}], " ", "b"}], 
      RowBox[{"a", " ", 
        RowBox[{"(", 
          RowBox[{"1", "-", "b"}], ")"}]}]]},
    "PolyLog"], "+", 
   RowBox[{
    RowBox[{"log", "(", "a", ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"log", "(", 
       RowBox[{"1", "-", "a"}], ")"}], "-", 
      RowBox[{"log", "(", 
       RowBox[{"1", "-", "b"}], ")"}]}], ")"}]}], "+", 
   RowBox[{
    RowBox[{"log", "(", 
     FractionBox[
      RowBox[{"1", "-", "a"}], 
      RowBox[{"1", "-", "b"}]], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", 
       RowBox[{"log", "(", 
        FractionBox[
         RowBox[{"a", "-", "b"}], "a"], ")"}]}], "+", 
      RowBox[{"log", "(", 
       FractionBox[
        RowBox[{"a", "-", "b"}], 
        RowBox[{"1", "-", "b"}]], ")"}], "-", 
      RowBox[{"log", "(", "a", ")"}], "+", 
      RowBox[{"log", "(", 
       RowBox[{"1", "-", "b"}], ")"}]}], ")"}]}], "-", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", 
       RowBox[{"log", "(", 
        FractionBox[
         RowBox[{"a", "-", "b"}], "a"], ")"}]}], "+", 
      RowBox[{"log", "(", 
       FractionBox[
        RowBox[{"a", "-", "b"}], 
        RowBox[{"a", " ", 
         RowBox[{"(", 
          RowBox[{"1", "-", "b"}], ")"}]}]], ")"}], "+", 
      RowBox[{"log", "(", 
       RowBox[{"1", "-", "b"}], ")"}]}], ")"}], " ", 
    RowBox[{"log", "(", 
     FractionBox[
      RowBox[{
       RowBox[{"(", 
        RowBox[{"1", "-", "a"}], ")"}], " ", "b"}], 
      RowBox[{"a", " ", 
       RowBox[{"(", 
        RowBox[{"1", "-", "b"}], ")"}]}]], ")"}]}], "+", 
   TemplateBox[{"2","a"},
    "PolyLog"], "-", 
   TemplateBox[{"2","b"},
    "PolyLog"], "-", 
   FractionBox[
    SuperscriptBox["\[Pi]", "2"], "6"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970284205823*^9},
 CellTags->"Hill",
 CellLabel->"Out[920]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"%", " ", "/.", " ", 
    RowBox[{"a", "\[RuleDelayed]", " ", ".123"}]}], " ", "/.", " ", 
   RowBox[{"b", "\[RuleDelayed]", " ", ".656"}]}], " ", "//", " ", 
  "Chop"}]], "Input",
 CellTags->"Hill",
 CellLabel->"In[921]:="],

Cell[BoxData[
 FormBox["0", TraditionalForm]], "Output",
 CellChangeTimes->{3.500970284740097*^9},
 CellTags->"Hill",
 CellLabel->"Out[921]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Hill", "[", 
     RowBox[{"x", ",", 
      RowBox[{"x", " ", "y"}]}], "]"}], "//", "PowerExpand"}], "//", 
   "SimplifyPolyLog"}], "//", "Expand"}]], "Input",
 CellTags->"Hill",
 CellLabel->"In[922]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"\[Zeta]", "(", "2", ")"}], "-", 
   TemplateBox[{"2",RowBox[{"x", " ", "y"}]},
    "PolyLog"], "+", 
   TemplateBox[{"2",FractionBox[
      RowBox[{"1", "-", "x"}], 
      RowBox[{"1", "-", 
        RowBox[{"x", " ", "y"}]}]]},
    "PolyLog"], "-", 
   TemplateBox[{"2",FractionBox[
      RowBox[{
        RowBox[{"(", 
          RowBox[{"1", "-", "x"}], ")"}], " ", "y"}], 
      RowBox[{"1", "-", 
        RowBox[{"x", " ", "y"}]}]]},
    "PolyLog"], "-", 
   TemplateBox[{"2",RowBox[{"1", "-", "x"}]},
    "PolyLog"], "-", 
   TemplateBox[{"2",RowBox[{"1", "-", "y"}]},
    "PolyLog"], "-", 
   RowBox[{
    RowBox[{"log", "(", "x", ")"}], " ", 
    RowBox[{"log", "(", 
     RowBox[{"1", "-", 
      RowBox[{"x", " ", "y"}]}], ")"}]}], "-", 
   RowBox[{
    RowBox[{"log", "(", 
     RowBox[{"1", "-", "y"}], ")"}], " ", 
    RowBox[{"log", "(", "y", ")"}]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009702848424683`*^9},
 CellTags->"Hill",
 CellLabel->"Out[922]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"%", " ", "/.", " ", 
     RowBox[{"x", "\[RuleDelayed]", " ", ".34"}]}], "/.", " ", 
    RowBox[{"y", "\[Rule]", " ", ".6"}]}], "//", "N"}], "//", 
  "Chop"}]], "Input",
 CellTags->"Hill",
 CellLabel->"In[923]:="],

Cell[BoxData[
 FormBox["0", TraditionalForm]], "Output",
 CellChangeTimes->{3.500970284991948*^9},
 CellTags->"Hill",
 CellLabel->"Out[923]="]
}, Open  ]]
}, Open  ]],

Cell["HypergeometricAC", "ObjectName",
 CellTags->"HypergeometricAC"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"HypergeometricAC"],

Cell["\<\
HypergeometricAC[n][exp] analytically continues Hypergeometric2F1 functions \
in exp. The second argument n refers to the equation number (n) in chapter \
2.10 of \"Higher Transcendental Functions\" by Ergelyi, Magnus, \
Oberhettinger, Tricomi. In case of eq. (6) (p.109) the last line is returned \
for HypergeometricAC[6][exp], while the first equality is given by \
HypergeometricAC[61][exp]. ((2.10.1) is identical to eq. (9.5.7) of \"Special \
Functions & their Applications\" by N.N.Lebedev).\
\>", "Text",
 CellTags->"HypergeometricAC"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "HypergeometricAC", "]"}]], "Input",
 CellTags->"HypergeometricAC",
 CellLabel->"In[924]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"Collect2", "\[Rule]", "True"}], "}"}], TraditionalForm]], "Output",\

 CellChangeTimes->{3.500970285137616*^9},
 CellTags->"HypergeometricAC",
 CellLabel->"Out[924]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["HypExplicit",
  BaseStyle->"AddOnsLink",
  ButtonData:>"HypExplicit",
  ButtonNote->"HypExplicit"],
 ", ",
 ButtonBox["HypergeometricIR",
  BaseStyle->"AddOnsLink",
  ButtonData:>"HypergeometricIR",
  ButtonNote->"HypergeometricIR"],
 ", ",
 ButtonBox["HypergeometricSE",
  BaseStyle->"AddOnsLink",
  ButtonData:>"HypergeometricSE",
  ButtonNote->"HypergeometricSE"],
 ", ",
 ButtonBox["ToHypergeometric",
  BaseStyle->"AddOnsLink",
  ButtonData:>"ToHypergeometric",
  ButtonNote->"ToHypergeometric"],
 "."
}], "Text",
 CellTags->"HypergeometricAC"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"HypergeometricAC"],

Cell["These are all transformation rules currently built in.", "Text",
 CellTags->"HypergeometricAC"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"HypergeometricAC", "[", "1", "]"}], "[", 
  RowBox[{"Hypergeometric2F1", "[", 
   RowBox[{"\[Alpha]", ",", "\[Beta]", ",", "\[Gamma]", ",", "z"}], "]"}], 
  "]"}]], "Input",
 CellTags->"HypergeometricAC",
 CellLabel->"In[925]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{
     TemplateBox[{"\[Gamma]"},
      "Gamma"], " ", 
     TemplateBox[{RowBox[{"\[Alpha]", "+", "\[Beta]", "-", "\[Gamma]"}]},
      "Gamma"], " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"1", "-", "z"}], ")"}], 
      RowBox[{
       RowBox[{"-", "\[Alpha]"}], "-", "\[Beta]", "+", "\[Gamma]"}]], " ", 
     TemplateBox[{
      RowBox[{"\[Gamma]", "-", "\[Alpha]"}],
       RowBox[{"\[Gamma]", "-", "\[Beta]"}],RowBox[{
         RowBox[{"-", "\[Alpha]"}], "-", "\[Beta]", "+", "\[Gamma]", "+", 
         "1"}],RowBox[{"1", "-", "z"}]},
      "Hypergeometric2F1"]}], 
    RowBox[{
     TemplateBox[{"\[Alpha]"},
      "Gamma"], " ", 
     TemplateBox[{"\[Beta]"},
      "Gamma"]}]], "+", 
   FractionBox[
    RowBox[{
     TemplateBox[{"\[Gamma]"},
      "Gamma"], " ", 
     TemplateBox[{RowBox[{
         RowBox[{"-", "\[Alpha]"}], "-", "\[Beta]", "+", "\[Gamma]"}]},
      "Gamma"], " ", 
     TemplateBox[{
      "\[Alpha]","\[Beta]",
       RowBox[{"\[Alpha]", "+", "\[Beta]", "-", "\[Gamma]", "+", "1"}],
       RowBox[{"1", "-", "z"}]},
      "Hypergeometric2F1"]}], 
    RowBox[{
     TemplateBox[{RowBox[{"\[Gamma]", "-", "\[Alpha]"}]},
      "Gamma"], " ", 
     TemplateBox[{RowBox[{"\[Gamma]", "-", "\[Beta]"}]},
      "Gamma"]}]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009702855748167`*^9},
 CellTags->"HypergeometricAC",
 CellLabel->"Out[925]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"HypergeometricAC", "[", "2", "]"}], "[", 
  RowBox[{"Hypergeometric2F1", "[", 
   RowBox[{"\[Alpha]", ",", "\[Beta]", ",", "\[Gamma]", ",", "z"}], "]"}], 
  "]"}]], "Input",
 CellTags->"HypergeometricAC",
 CellLabel->"In[926]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{
     TemplateBox[{"\[Gamma]"},
      "Gamma"], " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"-", "z"}], ")"}], 
      RowBox[{"-", "\[Alpha]"}]], " ", 
     TemplateBox[{RowBox[{"\[Beta]", "-", "\[Alpha]"}]},
      "Gamma"], " ", 
     TemplateBox[{
      "\[Alpha]",RowBox[{"\[Alpha]", "-", "\[Gamma]", "+", "1"}],
       RowBox[{"\[Alpha]", "-", "\[Beta]", "+", "1"}],FractionBox["1", "z"]},
      "Hypergeometric2F1"]}], 
    RowBox[{
     TemplateBox[{"\[Beta]"},
      "Gamma"], " ", 
     TemplateBox[{RowBox[{"\[Gamma]", "-", "\[Alpha]"}]},
      "Gamma"]}]], "+", 
   FractionBox[
    RowBox[{
     TemplateBox[{"\[Gamma]"},
      "Gamma"], " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"-", "z"}], ")"}], 
      RowBox[{"-", "\[Beta]"}]], " ", 
     TemplateBox[{RowBox[{"\[Alpha]", "-", "\[Beta]"}]},
      "Gamma"], " ", 
     TemplateBox[{
      "\[Beta]",RowBox[{"\[Beta]", "-", "\[Gamma]", "+", "1"}],RowBox[{
         RowBox[{"-", "\[Alpha]"}], "+", "\[Beta]", "+", "1"}],FractionBox[
       "1", "z"]},
      "Hypergeometric2F1"]}], 
    RowBox[{
     TemplateBox[{"\[Alpha]"},
      "Gamma"], " ", 
     TemplateBox[{RowBox[{"\[Gamma]", "-", "\[Beta]"}]},
      "Gamma"]}]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.50097028615333*^9},
 CellTags->"HypergeometricAC",
 CellLabel->"Out[926]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"HypergeometricAC", "[", "3", "]"}], "[", 
  RowBox[{"Hypergeometric2F1", "[", 
   RowBox[{"\[Alpha]", ",", "\[Beta]", ",", "\[Gamma]", ",", "z"}], "]"}], 
  "]"}]], "Input",
 CellTags->"HypergeometricAC",
 CellLabel->"In[927]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{
     TemplateBox[{"\[Gamma]"},
      "Gamma"], " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"1", "-", "z"}], ")"}], 
      RowBox[{"-", "\[Alpha]"}]], " ", 
     TemplateBox[{RowBox[{"\[Beta]", "-", "\[Alpha]"}]},
      "Gamma"], " ", 
     TemplateBox[{
      "\[Alpha]",RowBox[{"\[Gamma]", "-", "\[Beta]"}],
       RowBox[{"\[Alpha]", "-", "\[Beta]", "+", "1"}],FractionBox["1", 
        RowBox[{"1", "-", "z"}]]},
      "Hypergeometric2F1"]}], 
    RowBox[{
     TemplateBox[{"\[Beta]"},
      "Gamma"], " ", 
     TemplateBox[{RowBox[{"\[Gamma]", "-", "\[Alpha]"}]},
      "Gamma"]}]], "+", 
   FractionBox[
    RowBox[{
     TemplateBox[{"\[Gamma]"},
      "Gamma"], " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"1", "-", "z"}], ")"}], 
      RowBox[{"-", "\[Beta]"}]], " ", 
     TemplateBox[{RowBox[{"\[Alpha]", "-", "\[Beta]"}]},
      "Gamma"], " ", 
     TemplateBox[{"\[Beta]",RowBox[{"\[Gamma]", "-", "\[Alpha]"}],RowBox[{
         RowBox[{"-", "\[Alpha]"}], "+", "\[Beta]", "+", "1"}],FractionBox[
       "1", 
        RowBox[{"1", "-", "z"}]]},
      "Hypergeometric2F1"]}], 
    RowBox[{
     TemplateBox[{"\[Alpha]"},
      "Gamma"], " ", 
     TemplateBox[{RowBox[{"\[Gamma]", "-", "\[Beta]"}]},
      "Gamma"]}]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009702864287367`*^9},
 CellTags->"HypergeometricAC",
 CellLabel->"Out[927]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"HypergeometricAC", "[", "4", "]"}], "[", 
  RowBox[{"Hypergeometric2F1", "[", 
   RowBox[{"\[Alpha]", ",", "\[Beta]", ",", "\[Gamma]", ",", "z"}], "]"}], 
  "]"}]], "Input",
 CellTags->"HypergeometricAC",
 CellLabel->"In[928]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{
     TemplateBox[{"\[Gamma]"},
      "Gamma"], " ", 
     SuperscriptBox["z", 
      RowBox[{"-", "\[Alpha]"}]], " ", 
     TemplateBox[{RowBox[{
         RowBox[{"-", "\[Alpha]"}], "-", "\[Beta]", "+", "\[Gamma]"}]},
      "Gamma"], " ", 
     TemplateBox[{
      "\[Alpha]",RowBox[{"\[Alpha]", "-", "\[Gamma]", "+", "1"}],
       RowBox[{"\[Alpha]", "+", "\[Beta]", "-", "\[Gamma]", "+", "1"}],
       RowBox[{"-", 
         FractionBox[
          RowBox[{"1", "-", "z"}], "z"]}]},
      "Hypergeometric2F1"]}], 
    RowBox[{
     TemplateBox[{RowBox[{"\[Gamma]", "-", "\[Alpha]"}]},
      "Gamma"], " ", 
     TemplateBox[{RowBox[{"\[Gamma]", "-", "\[Beta]"}]},
      "Gamma"]}]], "+", 
   FractionBox[
    RowBox[{
     TemplateBox[{"\[Gamma]"},
      "Gamma"], " ", 
     SuperscriptBox["z", 
      RowBox[{"\[Alpha]", "-", "\[Gamma]"}]], " ", 
     TemplateBox[{RowBox[{"\[Alpha]", "+", "\[Beta]", "-", "\[Gamma]"}]},
      "Gamma"], " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"1", "-", "z"}], ")"}], 
      RowBox[{
       RowBox[{"-", "\[Alpha]"}], "-", "\[Beta]", "+", "\[Gamma]"}]], " ", 
     TemplateBox[{
      RowBox[{"1", "-", "\[Alpha]"}],RowBox[{"\[Gamma]", "-", "\[Alpha]"}],
       RowBox[{
         RowBox[{"-", "\[Alpha]"}], "-", "\[Beta]", "+", "\[Gamma]", "+", 
         "1"}],RowBox[{"-", 
         FractionBox[
          RowBox[{"1", "-", "z"}], "z"]}]},
      "Hypergeometric2F1"]}], 
    RowBox[{
     TemplateBox[{"\[Alpha]"},
      "Gamma"], " ", 
     TemplateBox[{"\[Beta]"},
      "Gamma"]}]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970286714426*^9},
 CellTags->"HypergeometricAC",
 CellLabel->"Out[928]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"HypergeometricAC", "[", "6", "]"}], "[", 
  RowBox[{"Hypergeometric2F1", "[", 
   RowBox[{"\[Alpha]", ",", "\[Beta]", ",", "\[Gamma]", ",", "z"}], "]"}], 
  "]"}]], "Input",
 CellTags->"HypergeometricAC",
 CellLabel->"In[929]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"1", "-", "z"}], ")"}], 
    RowBox[{"-", "\[Beta]"}]], " ", 
   TemplateBox[{
    "\[Beta]",RowBox[{"\[Gamma]", "-", "\[Alpha]"}],"\[Gamma]",RowBox[{"-", 
       FractionBox["z", 
        RowBox[{"1", "-", "z"}]]}]},
    "Hypergeometric2F1"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970286932667*^9},
 CellTags->"HypergeometricAC",
 CellLabel->"Out[929]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"HypergeometricAC", "[", "61", "]"}], "[", 
  RowBox[{"Hypergeometric2F1", "[", 
   RowBox[{"\[Alpha]", ",", "\[Beta]", ",", "\[Gamma]", ",", "z"}], "]"}], 
  "]"}]], "Input",
 CellTags->"HypergeometricAC",
 CellLabel->"In[930]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"1", "-", "z"}], ")"}], 
    RowBox[{"-", "\[Alpha]"}]], " ", 
   TemplateBox[{
    "\[Alpha]",RowBox[{"\[Gamma]", "-", "\[Beta]"}],"\[Gamma]",RowBox[{"-", 
       FractionBox["z", 
        RowBox[{"1", "-", "z"}]]}]},
    "Hypergeometric2F1"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009702871349707`*^9},
 CellTags->"HypergeometricAC",
 CellLabel->"Out[930]="]
}, Open  ]]
}, Open  ]],

Cell["HypergeometricIR", "ObjectName",
 CellTags->"HypergeometricIR"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"HypergeometricIR"],

Cell["\<\
HypergeometricIR[exp, t] substitutes for all Hypergeometric2F1[a,b,c,z] in \
exp by its Euler integral reprentation. The factor Integratedx[t, 0, 1] can \
be omitted by setting the option Integratedx \[Rule] False.\
\>", "Text",
 CellTags->"HypergeometricIR"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["HypergeometricAC",
  BaseStyle->"AddOnsLink",
  ButtonData:>"HypergeometricAC",
  ButtonNote->"HypergeometricAC"],
 ", ",
 ButtonBox["HypergeometricSE",
  BaseStyle->"AddOnsLink",
  ButtonData:>"HypergeometricSE",
  ButtonNote->"HypergeometricSE"],
 ", ",
 ButtonBox["ToHypergeometric",
  BaseStyle->"AddOnsLink",
  ButtonData:>"ToHypergeometric",
  ButtonNote->"ToHypergeometric"],
 "."
}], "Text",
 CellTags->"HypergeometricIR"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "HypergeometricIR", "]"}]], "Input",
 CellTags->"HypergeometricIR",
 CellLabel->"In[931]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"Integratedx", "\[Rule]", "False"}], "}"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970287247726*^9},
 CellTags->"HypergeometricIR",
 CellLabel->"Out[931]="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"HypergeometricIR"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"HypergeometricIR", "[", 
  RowBox[{
   RowBox[{"Hypergeometric2F1", "[", 
    RowBox[{"a", ",", "b", ",", "c", ",", "z"}], "]"}], ",", "t"}], 
  "]"}]], "Input",
 CellTags->"HypergeometricIR",
 CellLabel->"In[932]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{
    SuperscriptBox["t", 
     RowBox[{"b", "-", "1"}]], " ", 
    TemplateBox[{"c"},
     "Gamma"], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "-", 
       RowBox[{"t", " ", "z"}]}], ")"}], 
     RowBox[{"-", "a"}]], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "-", "t"}], ")"}], 
     RowBox[{
      RowBox[{"-", "b"}], "+", "c", "-", "1"}]]}], 
   RowBox[{
    TemplateBox[{"b"},
     "Gamma"], " ", 
    TemplateBox[{RowBox[{"c", "-", "b"}]},
     "Gamma"]}]], TraditionalForm]], "Output",
 CellChangeTimes->{3.50097028738413*^9},
 CellTags->"HypergeometricIR",
 CellLabel->"Out[932]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ToHypergeometric", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"t", "^", "b"}], " ", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"1", "-", "t"}], ")"}], "^", "c"}], " ", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"1", "+", 
       RowBox[{"t", " ", "z"}]}], ")"}], "^", "a"}]}], ",", "t"}], 
  "]"}]], "Input",
 CellTags->"HypergeometricIR",
 CellLabel->"In[933]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{
    TemplateBox[{RowBox[{"b", "+", "1"}]},
     "Gamma"], " ", 
    TemplateBox[{RowBox[{"c", "+", "1"}]},
     "Gamma"], " ", 
    TemplateBox[{
     RowBox[{"-", "a"}],RowBox[{"b", "+", "1"}],
      RowBox[{"b", "+", "c", "+", "2"}],RowBox[{"-", "z"}]},
     "Hypergeometric2F1"]}], 
   TemplateBox[{RowBox[{"b", "+", "c", "+", "2"}]},
    "Gamma"]], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009702875529423`*^9},
 CellTags->"HypergeometricIR",
 CellLabel->"Out[933]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"HypergeometricIR", "[", 
  RowBox[{"%", ",", "t"}], "]"}]], "Input",
 CellTags->"HypergeometricIR",
 CellLabel->"In[934]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["t", "b"], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"1", "-", "t"}], ")"}], "c"], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{
      RowBox[{"t", " ", "z"}], "+", "1"}], ")"}], "a"]}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970287645713*^9},
 CellTags->"HypergeometricIR",
 CellLabel->"Out[934]="]
}, Open  ]]
}, Open  ]],

Cell["HypergeometricSE", "ObjectName",
 CellTags->"HypergeometricSE"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"HypergeometricSE"],

Cell[TextData[{
 "HypergeometricSE[exp, ",
 Cell[BoxData[
  FormBox["\[Nu]", TraditionalForm]]],
 "] expresses Hypergeometric functions by their series expansion in terms of \
a sum (the Sum is omitted and ",
 Cell[BoxData[
  FormBox["\[Nu]", TraditionalForm]]],
 ", running from 0 to ",
 Cell[BoxData[
  FormBox["\[Infinity]", TraditionalForm]]],
 ", is the summation index)."
}], "Text",
 CellTags->"HypergeometricSE"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "HypergeometricSE", "]"}]], "Input",
 CellTags->"HypergeometricSE",
 CellLabel->"In[935]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"Simplify", "\[Rule]", "FullSimplify"}], "}"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970287718883*^9},
 CellTags->"HypergeometricSE",
 CellLabel->"Out[935]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["HypergeometricIR",
  BaseStyle->"AddOnsLink",
  ButtonData:>"HypergeometricIR",
  ButtonNote->"HypergeometricIR"],
 "."
}], "Text",
 CellTags->"HypergeometricSE"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"HypergeometricSE"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"HypergeometricSE", "[", 
  RowBox[{
   RowBox[{"Hypergeometric2F1", "[", 
    RowBox[{"a", ",", "b", ",", "c", ",", "z"}], "]"}], ",", "\[Nu]"}], 
  "]"}]], "Input",
 CellTags->"HypergeometricSE",
 CellLabel->"In[936]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{
    TemplateBox[{"c"},
     "Gamma"], " ", 
    SuperscriptBox["z", "\[Nu]"], " ", 
    TemplateBox[{RowBox[{"a", "+", "\[Nu]"}]},
     "Gamma"], " ", 
    TemplateBox[{RowBox[{"b", "+", "\[Nu]"}]},
     "Gamma"]}], 
   RowBox[{
    TemplateBox[{"a"},
     "Gamma"], " ", 
    TemplateBox[{"b"},
     "Gamma"], " ", 
    TemplateBox[{RowBox[{"\[Nu]", "+", "1"}]},
     "Gamma"], " ", 
    TemplateBox[{RowBox[{"c", "+", "\[Nu]"}]},
     "Gamma"]}]], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970287846923*^9},
 CellTags->"HypergeometricSE",
 CellLabel->"Out[936]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"HypergeometricSE", "[", 
  RowBox[{
   RowBox[{"HypergeometricPFQ", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"a", ",", "b", ",", "c"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"d", ",", "e"}], "}"}], ",", "z"}], "]"}], ",", "\[Nu]"}], 
  "]"}]], "Input",
 CellTags->"HypergeometricSE",
 CellLabel->"In[937]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{
    TemplateBox[{"d"},
     "Gamma"], " ", 
    TemplateBox[{"e"},
     "Gamma"], " ", 
    SuperscriptBox["z", "\[Nu]"], " ", 
    TemplateBox[{RowBox[{"a", "+", "\[Nu]"}]},
     "Gamma"], " ", 
    TemplateBox[{RowBox[{"b", "+", "\[Nu]"}]},
     "Gamma"], " ", 
    TemplateBox[{RowBox[{"c", "+", "\[Nu]"}]},
     "Gamma"]}], 
   RowBox[{
    TemplateBox[{"a"},
     "Gamma"], " ", 
    TemplateBox[{"b"},
     "Gamma"], " ", 
    TemplateBox[{"c"},
     "Gamma"], " ", 
    TemplateBox[{RowBox[{"\[Nu]", "+", "1"}]},
     "Gamma"], " ", 
    TemplateBox[{RowBox[{"d", "+", "\[Nu]"}]},
     "Gamma"], " ", 
    TemplateBox[{RowBox[{"e", "+", "\[Nu]"}]},
     "Gamma"]}]], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970288560429*^9},
 CellTags->"HypergeometricSE",
 CellLabel->"Out[937]="]
}, Open  ]]
}, Open  ]],

Cell["HypExplicit", "ObjectName",
 CellTags->"HypExplicit"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"HypExplicit"],

Cell[TextData[{
 "HypExplicit[exp, ",
 Cell[BoxData[
  FormBox["\[Nu]", TraditionalForm]]],
 "] expresses Hypergeometric functions in exp by their definition in terms of \
a sum (the Sum is omitted and ",
 Cell[BoxData[
  FormBox["\[Nu]", TraditionalForm]]],
 " is the summation index)."
}], "Text",
 CellTags->"HypExplicit"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["HypergeometricIR",
  BaseStyle->"AddOnsLink",
  ButtonData:>"HypergeometricIR",
  ButtonNote->"HypergeometricIR"],
 "."
}], "Text",
 CellTags->"HypExplicit"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"HypExplicit"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Hypergeometric2F1", "[", 
  RowBox[{"a", ",", "b", ",", "c", ",", "z"}], "]"}]], "Input",
 CellTags->"HypExplicit",
 CellLabel->"In[938]:="],

Cell[BoxData[
 FormBox[
  TemplateBox[{"a","b","c","z"},
   "Hypergeometric2F1"], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970288610558*^9},
 CellTags->"HypExplicit",
 CellLabel->"Out[938]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"HypExplicit", "[", 
  RowBox[{"%", ",", "\[Nu]"}], "]"}]], "Input",
 CellTags->"HypExplicit",
 CellLabel->"In[939]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{
    TemplateBox[{"c"},
     "Gamma"], " ", 
    SuperscriptBox["z", "\[Nu]"], " ", 
    TemplateBox[{RowBox[{"a", "+", "\[Nu]"}]},
     "Gamma"], " ", 
    TemplateBox[{RowBox[{"b", "+", "\[Nu]"}]},
     "Gamma"]}], 
   RowBox[{
    TemplateBox[{"a"},
     "Gamma"], " ", 
    TemplateBox[{"b"},
     "Gamma"], " ", 
    TemplateBox[{RowBox[{"\[Nu]", "+", "1"}]},
     "Gamma"], " ", 
    TemplateBox[{RowBox[{"c", "+", "\[Nu]"}]},
     "Gamma"]}]], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970288696597*^9},
 CellTags->"HypExplicit",
 CellLabel->"Out[939]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"HypergeometricPFQ", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"a", ",", "b", ",", "c"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"d", ",", "e"}], "}"}], ",", "z"}], "]"}]], "Input",
 CellTags->"HypExplicit",
 CellLabel->"In[940]:="],

Cell[BoxData[
 FormBox[
  TagBox[
   TagBox[
    RowBox[{
     RowBox[{
      SubscriptBox["\[InvisiblePrefixScriptBase]", "3"], 
      SubscriptBox["F", "2"]}], "\[InvisibleApplication]", 
     RowBox[{"(", 
      RowBox[{
       TagBox[
        TagBox[
         RowBox[{
          TagBox["a",
           HypergeometricPFQ,
           Editable->True,
           Selectable->True], ",", 
          TagBox["b",
           HypergeometricPFQ,
           Editable->True,
           Selectable->True], ",", 
          TagBox["c",
           HypergeometricPFQ,
           Editable->True,
           Selectable->True]}],
         InterpretTemplate[{
           SlotSequence[1]}& ]],
        HypergeometricPFQ,
        Editable->False,
        Selectable->False], ";", 
       TagBox[
        TagBox[
         RowBox[{
          TagBox["d",
           HypergeometricPFQ,
           Editable->True,
           Selectable->True], ",", 
          TagBox["e",
           HypergeometricPFQ,
           Editable->True,
           Selectable->True]}],
         InterpretTemplate[{
           SlotSequence[1]}& ]],
        HypergeometricPFQ,
        Editable->False,
        Selectable->False], ";", 
       TagBox["z",
        HypergeometricPFQ,
        Editable->True,
        Selectable->True]}], ")"}]}],
    InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
    Editable->False,
    Selectable->False],
   HypergeometricPFQ], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970288782818*^9},
 CellTags->"HypExplicit",
 CellLabel->"Out[940]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"HypExplicit", "[", 
  RowBox[{"%", ",", "\[Nu]"}], "]"}]], "Input",
 CellTags->"HypExplicit",
 CellLabel->"In[941]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{
    TemplateBox[{"d"},
     "Gamma"], " ", 
    TemplateBox[{"e"},
     "Gamma"], " ", 
    SuperscriptBox["z", "\[Nu]"], " ", 
    TemplateBox[{RowBox[{"a", "+", "\[Nu]"}]},
     "Gamma"], " ", 
    TemplateBox[{RowBox[{"b", "+", "\[Nu]"}]},
     "Gamma"], " ", 
    TemplateBox[{RowBox[{"c", "+", "\[Nu]"}]},
     "Gamma"]}], 
   RowBox[{
    TemplateBox[{"a"},
     "Gamma"], " ", 
    TemplateBox[{"b"},
     "Gamma"], " ", 
    TemplateBox[{"c"},
     "Gamma"], " ", 
    TemplateBox[{RowBox[{"\[Nu]", "+", "1"}]},
     "Gamma"], " ", 
    TemplateBox[{RowBox[{"d", "+", "\[Nu]"}]},
     "Gamma"], " ", 
    TemplateBox[{RowBox[{"e", "+", "\[Nu]"}]},
     "Gamma"]}]], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970288936513*^9},
 CellTags->"HypExplicit",
 CellLabel->"Out[941]="]
}, Open  ]]
}, Open  ]],

Cell["HypInt", "ObjectName",
 CellTags->"HypInt"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"HypInt"],

Cell["\<\
HypInt[exp, t]  substitutes for all  Hypergeometric2F1[a,b,c,z] in exp \
Gamma[c]/(Gamma[b] Gamma[c-b]) Integratedx[t,0,1]  t^(b-1) (1-t)^(c-b-1) (1-t \
z)^(-a).\
\>", "Text",
 CellTags->"HypInt"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"HypInt"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Hypergeometric2F1", "[", 
  RowBox[{"a", ",", "b", ",", "c", ",", "z"}], "]"}]], "Input",
 CellTags->"HypInt",
 CellLabel->"In[942]:="],

Cell[BoxData[
 FormBox[
  TemplateBox[{"a","b","c","z"},
   "Hypergeometric2F1"], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970289505537*^9},
 CellTags->"HypInt",
 CellLabel->"Out[942]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"HypInt", "[", 
  RowBox[{"%", ",", "t"}], "]"}]], "Input",
 CellTags->"HypInt",
 CellLabel->"In[943]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{
    SuperscriptBox["t", 
     RowBox[{"b", "-", "1"}]], " ", 
    TemplateBox[{"c"},
     "Gamma"], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "-", 
       RowBox[{"t", " ", "z"}]}], ")"}], 
     RowBox[{"-", "a"}]], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "-", "t"}], ")"}], 
     RowBox[{
      RowBox[{"-", "b"}], "+", "c", "-", "1"}]], " ", 
    RowBox[{
     SubsuperscriptBox["\[Integral]", 
      FormBox["0",
       TraditionalForm], 
      FormBox["1",
       TraditionalForm]], 
     RowBox[{"\[DifferentialD]", 
      TagBox[
       FormBox["t",
        TraditionalForm],
       TraditionalForm,
       Editable->True], "\[VeryThinSpace]"}]}]}], 
   RowBox[{
    TemplateBox[{"b"},
     "Gamma"], " ", 
    TemplateBox[{RowBox[{"c", "-", "b"}]},
     "Gamma"]}]], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970289696095*^9},
 CellTags->"HypInt",
 CellLabel->"Out[943]="]
}, Open  ]]
}, Open  ]],

Cell["IFPD", "ObjectName",
 CellTags->"IFPD"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"IFPD"],

Cell["IFPD[p, m] denotes (p^2 - m^2).", "Text",
 CellTags->"IFPD"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["PropagatorDenominator",
  BaseStyle->"AddOnsLink",
  ButtonData:>"PropagatorDenominator",
  ButtonNote->"PropagatorDenominator"],
 "."
}], "Text",
 CellTags->"IFPD"]
}, Open  ]],

Cell["IFPDOff", "ObjectName",
 CellTags->"IFPDOff"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"IFPDOff"],

Cell["\<\
IFPDOff[exp_,q1_, q2_, ...] changes from IFPD representation to \
FeynAmpDenominator[ ...]. The q1, q2, ... are the integration momenta.\
\>", "Text",
 CellTags->"IFPDOff"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["IFPD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"IFPD",
  ButtonNote->"IFPD"],
 ", ",
 ButtonBox["IFPDOn",
  BaseStyle->"AddOnsLink",
  ButtonData:>"IFPDOn",
  ButtonNote->"IFPDOn"],
 "."
}], "Text",
 CellTags->"IFPDOff"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"IFPDOff"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"IFPD", "[", 
  RowBox[{
   RowBox[{"Momentum", "[", "p", "]"}], ",", "m"}], "]"}]], "Input",
 CellTags->"IFPDOff",
 CellLabel->"In[944]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox["\<\"(\"\>",
    TraditionalForm], 
   FormBox[
    SuperscriptBox["p", "2"],
    TraditionalForm], 
   FormBox["\<\" - \"\>",
    TraditionalForm], 
   FormBox[
    SuperscriptBox["m", "2"],
    TraditionalForm], 
   FormBox["\<\")\"\>",
    TraditionalForm]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970289843845*^9},
 CellTags->"IFPDOff",
 CellLabel->"Out[944]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "StandardForm"}]], "Input",
 CellTags->"IFPDOff",
 CellLabel->"In[945]:="],

Cell[BoxData[
 RowBox[{"IFPD", "[", 
  RowBox[{
   RowBox[{"Momentum", "[", "p", "]"}], ",", "m"}], "]"}]], "Output",
 CellChangeTimes->{3.50097028998322*^9},
 CellTags->"IFPDOff",
 CellLabel->"Out[945]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"IFPDOff", "[", 
  RowBox[{"%", ",", "p"}], "]"}]], "Input",
 CellTags->"IFPDOff",
 CellLabel->"In[946]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox[
    FormBox["p",
     TraditionalForm], "2"], "-", 
   SuperscriptBox["m", "2"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970290464684*^9},
 CellTags->"IFPDOff",
 CellLabel->"Out[946]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "StandardForm"}]], "Input",
 CellTags->"IFPDOff",
 CellLabel->"In[947]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"-", 
   SuperscriptBox["m", "2"]}], "+", 
  RowBox[{"Pair", "[", 
   RowBox[{
    RowBox[{"Momentum", "[", "p", "]"}], ",", 
    RowBox[{"Momentum", "[", "p", "]"}]}], "]"}]}]], "Output",
 CellChangeTimes->{3.500970290605256*^9},
 CellTags->"IFPDOff",
 CellLabel->"Out[947]//StandardForm="]
}, Open  ]]
}, Open  ]],

Cell["IFPDOn", "ObjectName",
 CellTags->"IFPDOn"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"IFPDOn"],

Cell["\<\
IFPDOn[exp_,q1_, q2_, ...] changes from FeynAmpDenominator[ ...] \
representation to the IFPD one (Inverse Feynman Propagator Denominator). \
I.e., FeynAmpDenominator[PropagatorDenominator[a,b]] is replaced by \
1/IFPD[a,b] and The q1, q2, ... are the integration momenta.\
\>", "Text",
 CellTags->"IFPDOn"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["IFPD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"IFPD",
  ButtonNote->"IFPD"],
 ", ",
 ButtonBox["IFPDOff",
  BaseStyle->"AddOnsLink",
  ButtonData:>"IFPDOff",
  ButtonNote->"IFPDOff"],
 "."
}], "Text",
 CellTags->"IFPDOn"]
}, Open  ]],

Cell["IncludePair", "ObjectName",
 CellTags->"IncludePair"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"IncludePair"],

Cell["\<\
IncludePair is an option for FC2RHI. Possible settings are True and False.\
\>", "Text",
 CellTags->"IncludePair"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FC2RHI",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FC2RHI",
  ButtonNote->"FC2RHI"],
 "."
}], "Text",
 CellTags->"IncludePair"]
}, Open  ]],

Cell["IndexPosition", "ObjectName",
 CellTags->"IndexPosition"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"IndexPosition"],

Cell["IndexPosition is an option for FieldStrength.", "Text",
 CellTags->"IndexPosition"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FieldStrength",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FieldStrength",
  ButtonNote->"FieldStrength"],
 "."
}], "Text",
 CellTags->"IndexPosition"]
}, Open  ]],

Cell["Indices", "ObjectName",
 CellTags->"Indices"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Indices"],

Cell["\<\
Indices is an option for FORM2FeynCalc. Its default setting is Automatic. It \
may be set to a list, if the FORM-file does not contain a I(ndices) statement.\
\>", "Text",
 CellTags->"Indices"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FORM2FeynCalc",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FORM2FeynCalc",
  ButtonNote->"FORM2FeynCalc"],
 "."
}], "Text",
 CellTags->"Indices"]
}, Open  ]],

Cell["InitialFunction", "ObjectName",
 CellTags->"InitialFunction"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"InitialFunction"],

Cell["\<\
InitialFunction is an option of FeynRule the setting of which is applied to \
the first argument of FeynRule before anything else\
\>", "Text",
 CellTags->"InitialFunction"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FeynRule",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FeynRule",
  ButtonNote->"FeynRule"],
 "."
}], "Text",
 CellTags->"InitialFunction"]
}, Open  ]],

Cell["InitialSubstitutions", "ObjectName",
 CellTags->"InitialSubstitutions"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"InitialSubstitutions"],

Cell["\<\
InitialSubstitutions is an option for OneLoop and OneLoopSum and Write2. All \
substitutions indicated hereby are done at the end of the calculation.\
\>", "Text",
 CellTags->"InitialSubstitutions"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["OneLoop",
  BaseStyle->"AddOnsLink",
  ButtonData:>"OneLoop",
  ButtonNote->"OneLoop"],
 ", ",
 ButtonBox["OneLoopSum",
  BaseStyle->"AddOnsLink",
  ButtonData:>"OneLoopSum",
  ButtonNote->"OneLoopSum"],
 ", ",
 ButtonBox["Write2",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Write2",
  ButtonNote->"Write2"],
 "."
}], "Text",
 CellTags->"InitialSubstitutions"]
}, Open  ]],

Cell["InsideDiracTrace", "ObjectName",
 CellTags->"InsideDiracTrace"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"InsideDiracTrace"],

Cell["\<\
InsideDiracTrace is and option of DiracSimplify. If set to True, \
DiracSimplify assumes to operate inside a DiracTrace, i.e., products of an \
odd number of Dirac matrices are discarded. Furthermore simple traces are \
calculated (but divided by a factor 4, i.e. :  \
DiracSimplify[DiracMatrix[a,b], InsideDiracTrace\[Rule]True]  yields \
ScalarProduct[a,b]) Traces involving more than four DiracGamma's and \
DiracGamma[5] are not performed.\
\>", "Text",
 CellTags->"InsideDiracTrace"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["DiracSimplify",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracSimplify",
  ButtonNote->"DiracSimplify"],
 ", ",
 ButtonBox["DiracTrace",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracTrace",
  ButtonNote->"DiracTrace"],
 "."
}], "Text",
 CellTags->"InsideDiracTrace"]
}, Open  ]],

Cell["IntegralTable", "ObjectName",
 CellTags->"IntegralTable"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"IntegralTable"],

Cell["\<\
IntegralTable is an option of OneLoopSimplify, TwoLoopSimplify and \
FeynAmpDenominatorSimplify. It may be set to a list of the form : \
{FCIntegral[ ... ] \[RuleDelayed] bla, ...}.\
\>", "Text",
 CellTags->"IntegralTable"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["OneLoopSimplify",
  BaseStyle->"AddOnsLink",
  ButtonData:>"OneLoopSimplify",
  ButtonNote->"OneLoopSimplify"],
 ", ",
 ButtonBox["TwoLoopSimplify",
  BaseStyle->"AddOnsLink",
  ButtonData:>"TwoLoopSimplify",
  ButtonNote->"TwoLoopSimplify"],
 ", ",
 ButtonBox["FeynAmpDenominatorSimplify",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FeynAmpDenominatorSimplify",
  ButtonNote->"FeynAmpDenominatorSimplify"],
 "."
}], "Text",
 CellTags->"IntegralTable"]
}, Open  ]],

Cell["IntegrateByParts", "ObjectName",
 CellTags->"IntegrateByParts"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"IntegrateByParts"],

Cell["\<\
IntegrateByParts[(1-t)^(a Epsilon -1) g[t], deriv, t] does an integration by \
parts of the definite integral over t from 0 to 1.\
\>", "Text",
 CellTags->"IntegrateByParts"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "IntegrateByParts", "]"}]], "Input",
 CellTags->"IntegrateByParts",
 CellLabel->"In[948]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"Hold", "\[Rule]", "False"}], "}"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970290750198*^9},
 CellTags->"IntegrateByParts",
 CellLabel->"Out[948]="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"IntegrateByParts"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"IntegrateByParts", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{"1", "-", "t"}], ")"}], "^", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"a", " ", "Epsilon"}], "-", "1"}], ")"}]}], " ", 
    RowBox[{"g", "[", "t", "]"}]}], ",", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"1", "-", "t"}], ")"}], "^", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"a", " ", "Epsilon"}], "-", "1"}], ")"}]}], ",", "t"}], 
  "]"}]], "Input",
 CellTags->"IntegrateByParts",
 CellLabel->"In[949]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"1", "-", "t"}], ")"}], 
      RowBox[{"a", " ", 
       TagBox["\[CurlyEpsilon]",
        TraditionalForm]}]], " ", 
     RowBox[{
      SuperscriptBox["g", "\[Prime]",
       MultilineFunction->None], "(", "t", ")"}]}], 
    RowBox[{"a", " ", 
     TagBox["\[CurlyEpsilon]",
      TraditionalForm]}]], "+", 
   FractionBox[
    RowBox[{"g", "(", "0", ")"}], 
    RowBox[{"a", " ", 
     TagBox["\[CurlyEpsilon]",
      TraditionalForm]}]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970290929255*^9},
 CellTags->"IntegrateByParts",
 CellLabel->"Out[949]="]
}, Open  ]]
}, Open  ]],

Cell["Integratedx", "ObjectName",
 CellTags->"Integratedx"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Integratedx"],

Cell["\<\
Integratedx[x, low, up] is a variable representing the integration operator \
Integrate[#, {x,low,up}]&.\
\>", "Text",
 CellTags->"Integratedx"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FeynmanParametrize",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FeynmanParametrize",
  ButtonNote->"FeynmanParametrize"],
 ", ",
 ButtonBox["CalculateCounterTerm",
  BaseStyle->"AddOnsLink",
  ButtonData:>"CalculateCounterTerm",
  ButtonNote->"CalculateCounterTerm"],
 ", ",
 ButtonBox["TimedIntegrate",
  BaseStyle->"AddOnsLink",
  ButtonData:>"TimedIntegrate",
  ButtonNote->"TimedIntegrate"],
 ", ",
 ButtonBox["HypergeometricIR",
  BaseStyle->"AddOnsLink",
  ButtonData:>"HypergeometricIR",
  ButtonNote->"HypergeometricIR"],
 ", ",
 ButtonBox["HypInt",
  BaseStyle->"AddOnsLink",
  ButtonData:>"HypInt",
  ButtonNote->"HypInt"],
 ", ",
 ButtonBox["TimedIntegrate",
  BaseStyle->"AddOnsLink",
  ButtonData:>"TimedIntegrate",
  ButtonNote->"TimedIntegrate"],
 "."
}], "Text",
 CellTags->"Integratedx"]
}, Open  ]],

Cell["Integrate2", "ObjectName",
 CellTags->"Integrate2"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Integrate2"],

Cell[TextData[{
 "Integrate2 is like Integrate, but : Integrate2[a_Plus, b__] := \
Map[Integrate2[#, b]&, a] ( more linear algebra and partial fraction \
decomposition is done) Integrate2[f[x] DeltaFunction[x], x] \[Rule] f[0] \
Integrate2[f[x] DeltaFunction[x0-x], x] \[Rule] f[x0]. Integrate2[f[x] \
DeltaFunction[a + b x], x] \[Rule] Integrate[f[x] (1/Abs[b]) \
DeltaFunction[a/b + x], x], where abs[b] \[Rule] b, if b is a Symbol, and if \
b = -c, then abs[-c] \[Rule] c, i.e., the variable contained in b is supposed \
to be positive. ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    SuperscriptBox["\[Pi]", "2"]}], TraditionalForm]]],
 " is replaced by 6 Zeta2. Integrate2[1/(1-y),{y,x,1}] is intepreted as \
distribution, i.e. as Integrate2[-1/(1-y)],{y, 0, x}] \[Rule] Log[1-y]. \
Integrate2[1/(1-x),{x,0,1}] \[Rule] 0."
}], "Text",
 CellTags->"Integrate2"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["DeltaFunction",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DeltaFunction",
  ButtonNote->"DeltaFunction"],
 ", ",
 ButtonBox["Integrate3",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Integrate3",
  ButtonNote->"Integrate3"],
 ", ",
 ButtonBox["SumS",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SumS",
  ButtonNote->"SumS"],
 ", ",
 ButtonBox["SumT",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SumT",
  ButtonNote->"SumT"],
 "."
}], "Text",
 CellTags->"Integrate2"],

Cell["\<\
NOTE: Since Integrate2 does do a reordering and partial fraction \
decomposition before calling the integral table of Integrate3 it will in \
general be slower compared to Integrate3 for sums of integrals. I.e., if the \
integrand has already an expanded form and if partial fraction decomposition \
is not necessary it is more effective to use Integrate3.\
\>", "Text",
 CellTags->"Integrate2"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Integrate2"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Integrate2", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"Log", "[", 
      RowBox[{"1", "+", "x"}], "]"}], 
     RowBox[{
      RowBox[{"Log", "[", "x", "]"}], "/", 
      RowBox[{"(", 
       RowBox[{"1", "-", "x"}], ")"}]}]}], ",", 
    RowBox[{"{", 
     RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}], "//", 
  "Timing"}]], "Input",
 CellTags->"Integrate2",
 CellLabel->"In[950]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"0.030000000000001137`", ",", 
    RowBox[{
     TemplateBox[{"3"},
      "Zeta"], "-", 
     RowBox[{
      FractionBox["3", "2"], " ", 
      RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
      RowBox[{"log", "(", "2", ")"}]}]}]}], "}"}], TraditionalForm]], "Output",\

 CellChangeTimes->{3.500970291047255*^9},
 CellTags->"Integrate2",
 CellLabel->"Out[950]="]
}, Open  ]],

Cell[TextData[{
 "Since Integrate2 uses table-look-up methods it is much faster than ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "'s Integrate."
}], "Text",
 CellTags->"Integrate2"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Integrate2", "[", 
  RowBox[{
   RowBox[{"PolyLog", "[", 
    RowBox[{"2", ",", 
     RowBox[{"x", "^", "2"}]}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellTags->"Integrate2",
 CellLabel->"In[951]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"\[Zeta]", "(", "2", ")"}], "-", "4", "+", 
   RowBox[{"4", " ", 
    RowBox[{"log", "(", "2", ")"}]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009702916565857`*^9},
 CellTags->"Integrate2",
 CellLabel->"Out[951]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Integrate2", "[", 
  RowBox[{
   RowBox[{"PolyLog", "[", 
    RowBox[{"3", ",", 
     RowBox[{"-", "x"}]}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellTags->"Integrate2",
 CellLabel->"In[952]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{"\[Zeta]", "(", "2", ")"}], "2"], "-", 
   FractionBox[
    RowBox[{"3", " ", 
     TemplateBox[{"3"},
      "Zeta"]}], "4"], "+", "1", "-", 
   RowBox[{"2", " ", 
    RowBox[{"log", "(", "2", ")"}]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970291827489*^9},
 CellTags->"Integrate2",
 CellLabel->"Out[952]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Integrate2", "[", 
  RowBox[{
   RowBox[{"PolyLog", "[", 
    RowBox[{"3", ",", 
     RowBox[{"1", "/", 
      RowBox[{"(", 
       RowBox[{"1", "+", "x"}], ")"}]}]}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellTags->"Integrate2",
 CellLabel->"In[953]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
    RowBox[{"(", 
     RowBox[{"-", 
      RowBox[{"log", "(", "2", ")"}]}], ")"}]}], "+", 
   FractionBox[
    RowBox[{"3", " ", 
     TemplateBox[{"3"},
      "Zeta"]}], "4"], "+", 
   FractionBox[
    RowBox[{
     SuperscriptBox["log", "3"], "(", "2", ")"}], "3"], "-", 
   RowBox[{
    SuperscriptBox["log", "2"], "(", "2", ")"}], "+", 
   RowBox[{"2", " ", 
    RowBox[{"log", "(", "2", ")"}]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970292010377*^9},
 CellTags->"Integrate2",
 CellLabel->"Out[953]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Integrate2", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"DeltaFunction", "[", 
     RowBox[{"1", "-", "x"}], "]"}], " ", 
    RowBox[{"f", "[", "x", "]"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellTags->"Integrate2",
 CellLabel->"In[954]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"f", "(", "1", ")"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970292170268*^9},
 CellTags->"Integrate2",
 CellLabel->"Out[954]="]
}, Open  ]],

Cell[TextData[{
 "Integrate2 does integration in a Hadamard sense, i.e., ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubsuperscriptBox["\[Integral]", "0", "1"], 
    RowBox[{
     RowBox[{"f", "(", "x", ")"}], 
     RowBox[{"\[DifferentialD]", "x"}]}], " "}], TraditionalForm]]],
 " means acutally expanding the result of ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubsuperscriptBox["\[Integral]", "\[Delta]", 
     RowBox[{"1", "-", "\[Delta]"}]], 
    RowBox[{
     RowBox[{"f", "(", "x", ")"}], 
     RowBox[{"\[DifferentialD]", "x"}]}], " "}], TraditionalForm]]],
 "up do ",
 Cell[BoxData[
  FormBox[
   RowBox[{"O", "(", "\[Delta]", ")"}], TraditionalForm]]],
 " and neglecting all ",
 Cell[BoxData[
  FormBox["\[Delta]", TraditionalForm]]],
 "-dependent terms. E.g. ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SubsuperscriptBox["\[Integral]", "\[Delta]", 
      RowBox[{"1", "-", "\[Delta]"}]], 
     RowBox[{
      RowBox[{"1", "/", 
       RowBox[{"(", 
        RowBox[{"1", "-", "x"}], ")"}]}], 
      RowBox[{"\[DifferentialD]", "x"}]}]}], " ", "=", " ", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"-", "log"}], 
       RowBox[{"(", 
        RowBox[{"1", "-", "x"}], ")"}]}], 
      SubsuperscriptBox["\[VerticalSeparator]", "\[Delta]", 
       RowBox[{"1", "-", "\[Delta]"}]]}], "=", 
     RowBox[{
      RowBox[{
       RowBox[{"-", 
        RowBox[{"log", "(", "\[Delta]", ")"}]}], "+", 
       RowBox[{"log", "(", "1", ")"}]}], "\[Implies]", "0."}]}], " "}], 
   TraditionalForm]]]
}], "Text",
 ZeroWidthTimes->False,
 CellTags->"Integrate2"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Integrate2", "[", 
  RowBox[{
   RowBox[{"1", "/", 
    RowBox[{"(", 
     RowBox[{"1", "-", "x"}], ")"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellTags->"Integrate2",
 CellLabel->"In[955]:="],

Cell[BoxData[
 FormBox["0", TraditionalForm]], "Output",
 CellChangeTimes->{3.500970292383875*^9},
 CellTags->"Integrate2",
 CellLabel->"Out[955]="]
}, Open  ]],

Cell[TextData[{
 "In the physics literature sometimes the \"+\" notation is used. In FeynCalc \
the ",
 Cell[BoxData[
  FormBox[
   SubscriptBox[
    RowBox[{"(", 
     FractionBox["1", 
      RowBox[{"1", "-", "x"}]], ")"}], "+"], TraditionalForm]]],
 " is represented by ",
 Cell[BoxData[
  FormBox[
   RowBox[{"PlusDistribution", "[", 
    RowBox[{"1", "/", 
     RowBox[{"(", 
      RowBox[{"1", "-", "x"}], ")"}]}], "]"}], TraditionalForm]]],
 " or just ",
 Cell[BoxData[
  FormBox[
   RowBox[{"1", "/", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"1", "-", "x"}], ")"}], " ", "."}]}], TraditionalForm]]]
}], "Text",
 CellTags->"Integrate2"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Integrate2", "[", 
  RowBox[{
   RowBox[{"PlusDistribution", "[", 
    RowBox[{"1", "/", 
     RowBox[{"(", 
      RowBox[{"1", "-", "x"}], ")"}]}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellTags->"Integrate2",
 CellLabel->"In[956]:="],

Cell[BoxData[
 FormBox["0", TraditionalForm]], "Output",
 CellChangeTimes->{3.500970292520746*^9},
 CellTags->"Integrate2",
 CellLabel->"Out[956]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Integrate2", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"PolyLog", "[", 
     RowBox[{"2", ",", 
      RowBox[{"1", "-", "x"}]}], "]"}], "/", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"1", "-", "x"}], ")"}], "^", "2"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellTags->"Integrate2",
 CellLabel->"In[957]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"2", "-", 
   RowBox[{"\[Zeta]", "(", "2", ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970292658667*^9},
 CellTags->"Integrate2",
 CellLabel->"Out[957]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Integrate2", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"Log", "[", "x", "]"}], " ", 
      RowBox[{"Log", "[", 
       RowBox[{"1", "+", "x"}], "]"}]}], ")"}], "/", 
    RowBox[{"(", 
     RowBox[{"1", "+", "x"}], ")"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellTags->"Integrate2",
 CellLabel->"In[958]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   FractionBox[
    TemplateBox[{"3"},
     "Zeta"], "8"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970292816907*^9},
 CellTags->"Integrate2",
 CellLabel->"Out[958]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Integrate2", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"Log", "[", "x", "]"}], "^", "2"}], "/", 
    RowBox[{"(", 
     RowBox[{"1", "-", "x"}], ")"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellTags->"Integrate2",
 CellLabel->"In[959]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"2", " ", 
   TemplateBox[{"3"},
    "Zeta"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970292924932*^9},
 CellTags->"Integrate2",
 CellLabel->"Out[959]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Integrate2", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"PolyLog", "[", 
     RowBox[{"2", ",", 
      RowBox[{"-", "x"}]}], "]"}], "/", 
    RowBox[{"(", 
     RowBox[{"1", "+", "x"}], ")"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellTags->"Integrate2",
 CellLabel->"In[960]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    TemplateBox[{"3"},
     "Zeta"], "4"], "-", 
   RowBox[{
    FractionBox["1", "2"], " ", 
    RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
    RowBox[{"log", "(", "2", ")"}]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970293043928*^9},
 CellTags->"Integrate2",
 CellLabel->"Out[960]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Integrate2", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"Log", "[", "x", "]"}], " ", 
    RowBox[{"PolyLog", "[", 
     RowBox[{"2", ",", "x"}], "]"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellTags->"Integrate2",
 CellLabel->"In[961]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"3", "-", 
   RowBox[{"2", " ", 
    RowBox[{"\[Zeta]", "(", "2", ")"}]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970293156776*^9},
 CellTags->"Integrate2",
 CellLabel->"Out[961]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Integrate2", "[", 
  RowBox[{
   RowBox[{"x", " ", 
    RowBox[{"PolyLog", "[", 
     RowBox[{"3", ",", "x"}], "]"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellTags->"Integrate2",
 CellLabel->"In[962]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    FractionBox[
     RowBox[{"\[Zeta]", "(", "2", ")"}], "4"]}], "+", 
   FractionBox[
    TemplateBox[{"3"},
     "Zeta"], "2"], "+", 
   FractionBox["3", "16"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970293274062*^9},
 CellTags->"Integrate2",
 CellLabel->"Out[962]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Integrate2", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"Log", "[", "x", "]"}], "^", "2"}], " ", 
      RowBox[{"Log", "[", 
       RowBox[{"1", "-", "x"}], "]"}]}], ")"}], "/", 
    RowBox[{"(", 
     RowBox[{"1", "+", "x"}], ")"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellTags->"Integrate2",
 CellLabel->"In[963]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
    RowBox[{
     SuperscriptBox["log", "2"], "(", "2", ")"}]}], "-", 
   RowBox[{"4", " ", 
    TemplateBox[{"4",FractionBox["1", "2"]},
     "PolyLog"]}], "+", 
   FractionBox[
    SuperscriptBox["\[Pi]", "4"], "90"], "-", 
   FractionBox[
    RowBox[{
     SuperscriptBox["log", "4"], "(", "2", ")"}], "6"]}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970293492095*^9},
 CellTags->"Integrate2",
 CellLabel->"Out[963]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Integrate2", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"PolyLog", "[", 
     RowBox[{"2", ",", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           RowBox[{"x", " ", 
            RowBox[{"(", 
             RowBox[{"1", "-", "z"}], ")"}]}], "+", "z"}], ")"}], " ", 
         RowBox[{"(", 
          RowBox[{"1", "-", "x", "+", 
           RowBox[{"x", " ", "z"}]}], ")"}]}], ")"}], "/", "z"}]}], "]"}], 
    "/", 
    RowBox[{"(", 
     RowBox[{"1", "-", "x", "+", 
      RowBox[{"x", " ", "z"}]}], ")"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellTags->"Integrate2",
 CellLabel->"In[964]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{"2", " ", "\[ImaginaryI]", " ", "\[Pi]", " ", 
     TemplateBox[{"2",RowBox[{"-", "z"}]},
      "PolyLog"]}], 
    RowBox[{"1", "-", "z"}]], "+", 
   FractionBox[
    RowBox[{"4", " ", 
     TemplateBox[{"3",RowBox[{"1", "-", "z"}]},
      "PolyLog"]}], 
    RowBox[{"1", "-", "z"}]], "+", 
   FractionBox[
    RowBox[{"2", " ", 
     TemplateBox[{"3",RowBox[{"-", "z"}]},
      "PolyLog"]}], 
    RowBox[{"1", "-", "z"}]], "+", 
   FractionBox[
    RowBox[{"4", " ", 
     TemplateBox[{"3",FractionBox["1", 
        RowBox[{"z", "+", "1"}]]},
      "PolyLog"]}], 
    RowBox[{"1", "-", "z"}]], "+", 
   FractionBox[
    RowBox[{"4", " ", 
     TemplateBox[{"3",RowBox[{"-", 
         FractionBox[
          RowBox[{"1", "-", "z"}], 
          RowBox[{"z", "+", "1"}]]}]},
      "PolyLog"]}], 
    RowBox[{"1", "-", "z"}]], "-", 
   FractionBox[
    RowBox[{"4", " ", 
     TemplateBox[{"3",FractionBox[
        RowBox[{"1", "-", "z"}], 
        RowBox[{"z", "+", "1"}]]},
      "PolyLog"]}], 
    RowBox[{"1", "-", "z"}]], "-", 
   FractionBox[
    RowBox[{"2", " ", 
     TemplateBox[{"2",RowBox[{"1", "-", "z"}]},
      "PolyLog"], " ", 
     RowBox[{"log", "(", "z", ")"}]}], 
    RowBox[{"1", "-", "z"}]], "-", 
   FractionBox[
    RowBox[{"2", " ", 
     TemplateBox[{"2",RowBox[{"-", "z"}]},
      "PolyLog"], " ", 
     RowBox[{"log", "(", "z", ")"}]}], 
    RowBox[{"1", "-", "z"}]], "+", 
   FractionBox[
    RowBox[{"4", " ", 
     TemplateBox[{"2",RowBox[{"-", "z"}]},
      "PolyLog"], " ", 
     RowBox[{"log", "(", 
      RowBox[{"1", "-", "z"}], ")"}]}], 
    RowBox[{"1", "-", "z"}]], "-", 
   FractionBox[
    RowBox[{"2", " ", 
     RowBox[{
      SubscriptBox["S", 
       RowBox[{
        FormBox["1",
         TraditionalForm], 
        FormBox["2",
         TraditionalForm]}]], "(", 
      FormBox[
       RowBox[{"1", "-", "z"}],
       TraditionalForm], ")"}]}], 
    RowBox[{"1", "-", "z"}]], "+", 
   FractionBox[
    RowBox[{"\[ImaginaryI]", " ", "\[Pi]", " ", 
     RowBox[{"\[Zeta]", "(", "2", ")"}]}], 
    RowBox[{"1", "-", "z"}]], "-", 
   FractionBox[
    RowBox[{
     RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
     RowBox[{"log", "(", "z", ")"}]}], 
    RowBox[{"1", "-", "z"}]], "+", 
   FractionBox[
    RowBox[{"2", " ", 
     RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
     RowBox[{"log", "(", 
      RowBox[{"1", "-", "z"}], ")"}]}], 
    RowBox[{"1", "-", "z"}]], "+", 
   FractionBox[
    RowBox[{"2", " ", 
     RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
     RowBox[{"log", "(", 
      RowBox[{"z", "+", "1"}], ")"}]}], 
    RowBox[{"1", "-", "z"}]], "-", 
   FractionBox[
    RowBox[{"2", " ", 
     TemplateBox[{"3"},
      "Zeta"]}], 
    RowBox[{"1", "-", "z"}]], "+", 
   FractionBox[
    RowBox[{
     SuperscriptBox["log", "3"], "(", "z", ")"}], 
    RowBox[{"6", " ", 
     RowBox[{"(", 
      RowBox[{"1", "-", "z"}], ")"}]}]], "-", 
   FractionBox[
    RowBox[{"2", " ", 
     RowBox[{
      SuperscriptBox["log", "3"], "(", 
      RowBox[{"z", "+", "1"}], ")"}]}], 
    RowBox[{"3", " ", 
     RowBox[{"(", 
      RowBox[{"1", "-", "z"}], ")"}]}]], "-", 
   FractionBox[
    RowBox[{
     RowBox[{"log", "(", 
      RowBox[{"1", "-", "z"}], ")"}], " ", 
     RowBox[{
      SuperscriptBox["log", "2"], "(", "z", ")"}]}], 
    RowBox[{"1", "-", "z"}]], "-", 
   FractionBox[
    RowBox[{
     RowBox[{"log", "(", 
      RowBox[{"z", "+", "1"}], ")"}], " ", 
     RowBox[{
      SuperscriptBox["log", "2"], "(", "z", ")"}]}], 
    RowBox[{"1", "-", "z"}]], "-", 
   FractionBox[
    RowBox[{"\[ImaginaryI]", " ", "\[Pi]", " ", 
     RowBox[{
      SuperscriptBox["log", "2"], "(", "z", ")"}]}], 
    RowBox[{"2", " ", 
     RowBox[{"(", 
      RowBox[{"1", "-", "z"}], ")"}]}]], "+", 
   FractionBox[
    RowBox[{"4", " ", 
     RowBox[{"log", "(", 
      RowBox[{"1", "-", "z"}], ")"}], " ", 
     RowBox[{"log", "(", 
      RowBox[{"z", "+", "1"}], ")"}], " ", 
     RowBox[{"log", "(", "z", ")"}]}], 
    RowBox[{"1", "-", "z"}]], "+", 
   FractionBox[
    RowBox[{"2", " ", "\[ImaginaryI]", " ", "\[Pi]", " ", 
     RowBox[{"log", "(", 
      RowBox[{"z", "+", "1"}], ")"}], " ", 
     RowBox[{"log", "(", "z", ")"}]}], 
    RowBox[{"1", "-", "z"}]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009702938449507`*^9},
 CellTags->"Integrate2",
 CellLabel->"Out[964]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Apart", "[", 
  RowBox[{
   RowBox[{"Integrate2", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"x", "^", 
       RowBox[{"(", 
        RowBox[{"OPEm", "-", "1"}], ")"}]}], " ", 
      RowBox[{"PolyLog", "[", 
       RowBox[{"3", ",", 
        RowBox[{"1", "-", "x"}]}], "]"}]}], ",", 
     RowBox[{"{", 
      RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}], ",", "OPEm"}], 
  "]"}]], "Input",
 CellTags->"Integrate2",
 CellLabel->"In[965]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    FractionBox[
     RowBox[{"\[Zeta]", "(", "2", ")"}], 
     SuperscriptBox["m", "2"]]}], "-", 
   FractionBox[
    RowBox[{"\[Zeta]", "(", "2", ")"}], 
    RowBox[{"m", "-", "1"}]], "+", 
   FractionBox[
    RowBox[{
     RowBox[{"\[Zeta]", "(", "2", ")"}], "+", 
     RowBox[{
      RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
      RowBox[{"(", 
       RowBox[{"-", 
        RowBox[{
         SubscriptBox["S", "1"], "(", 
         FormBox[
          RowBox[{"m", "-", "2"}],
          TraditionalForm], ")"}]}], ")"}]}], "+", 
     RowBox[{
      SubscriptBox["S", "12"], "(", 
      FormBox["m",
       TraditionalForm], ")"}], "+", 
     TemplateBox[{"3"},
      "Zeta"]}], "m"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970294029866*^9},
 CellTags->"Integrate2",
 CellLabel->"Out[965]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Integrate2", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"x", "^", 
      RowBox[{"(", 
       RowBox[{"OPEm", "-", "1"}], ")"}]}], " ", 
     RowBox[{"Log", "[", 
      RowBox[{"1", "-", "x"}], "]"}], " ", 
     RowBox[{"Log", "[", "x", "]"}], " ", 
     FractionBox[
      RowBox[{"Log", "[", 
       RowBox[{"1", "+", "x"}], "]"}], 
      RowBox[{"1", "+", "x"}]]}], ",", 
    RowBox[{"{", 
     RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}], "//", 
  "Simplify"}]], "Input",
 CellTags->"Integrate2",
 CellLabel->"In[966]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["1", "360"], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"-", "1"}], ")"}], "m"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"450", " ", 
      RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
      RowBox[{
       SuperscriptBox["log", "2"], "(", "2", ")"}]}], "+", 
     RowBox[{"90", " ", 
      RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
      RowBox[{
       SubsuperscriptBox["S", 
        RowBox[{"-", "1"}], "2"], "(", 
       FormBox[
        RowBox[{"m", "-", "1"}],
        TraditionalForm], ")"}]}], "+", 
     RowBox[{"270", " ", 
      RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
      RowBox[{
       SubscriptBox["S", "2"], "(", 
       FormBox[
        RowBox[{"m", "-", "1"}],
        TraditionalForm], ")"}]}], "-", 
     RowBox[{"360", " ", 
      RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
      RowBox[{
       SubscriptBox["S", 
        RowBox[{"1", "-", "1"}]], "(", 
       FormBox[
        RowBox[{"m", "-", "1"}],
        TraditionalForm], ")"}]}], "-", 
     RowBox[{"180", " ", 
      RowBox[{
       SubscriptBox["S", 
        RowBox[{"-", "2"}]], "(", 
       FormBox[
        RowBox[{"m", "-", "1"}],
        TraditionalForm], ")"}], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"\[Zeta]", "(", "2", ")"}], "-", 
        RowBox[{
         RowBox[{"log", "(", "4", ")"}], " ", 
         RowBox[{
          SubscriptBox["S", 
           RowBox[{"-", "1"}]], "(", 
          FormBox[
           RowBox[{"m", "-", "1"}],
           TraditionalForm], ")"}]}], "-", 
        RowBox[{
         SuperscriptBox["log", "2"], "(", "2", ")"}]}], ")"}]}], "-", 
     RowBox[{"540", " ", 
      RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
      RowBox[{"log", "(", "2", ")"}], " ", 
      RowBox[{
       SubscriptBox["S", "1"], "(", 
       FormBox[
        RowBox[{"m", "-", "1"}],
        TraditionalForm], ")"}]}], "+", 
     RowBox[{"180", " ", 
      RowBox[{
       SubscriptBox["S", 
        RowBox[{"-", "1"}]], "(", 
       FormBox[
        RowBox[{"m", "-", "1"}],
        TraditionalForm], ")"}], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
         RowBox[{"log", "(", "8", ")"}]}], "-", 
        RowBox[{"2", " ", 
         TemplateBox[{"3"},
          "Zeta"]}]}], ")"}]}], "+", 
     RowBox[{"585", " ", 
      TemplateBox[{"3"},
       "Zeta"], " ", 
      RowBox[{
       SubscriptBox["S", "1"], "(", 
       FormBox[
        RowBox[{"m", "-", "1"}],
        TraditionalForm], ")"}]}], "+", 
     RowBox[{"360", " ", 
      RowBox[{
       SubscriptBox["S", 
        RowBox[{
         RowBox[{"-", "2"}], "-", "1", "-", "1"}]], "(", 
       FormBox[
        RowBox[{"m", "-", "1"}],
        TraditionalForm], ")"}]}], "+", 
     RowBox[{"360", " ", 
      RowBox[{
       SubscriptBox["S", 
        RowBox[{
         RowBox[{"-", "1"}], "-", "2", "-", "1"}]], "(", 
       FormBox[
        RowBox[{"m", "-", "1"}],
        TraditionalForm], ")"}]}], "+", 
     RowBox[{"360", " ", 
      RowBox[{
       SubscriptBox["S", 
        RowBox[{
         RowBox[{"-", "1"}], "-", "1", "-", "2"}]], "(", 
       FormBox[
        RowBox[{"m", "-", "1"}],
        TraditionalForm], ")"}]}], "+", 
     RowBox[{"360", " ", 
      RowBox[{
       SubscriptBox["S", 
        RowBox[{"1", "-", "21"}]], "(", 
       FormBox[
        RowBox[{"m", "-", "1"}],
        TraditionalForm], ")"}]}], "+", 
     RowBox[{"360", " ", 
      RowBox[{
       SubscriptBox["S", 
        RowBox[{"1", "-", "12"}]], "(", 
       FormBox[
        RowBox[{"m", "-", "1"}],
        TraditionalForm], ")"}]}], "+", 
     RowBox[{"360", " ", 
      RowBox[{
       SubscriptBox["S", 
        RowBox[{"2", "-", "11"}]], "(", 
       FormBox[
        RowBox[{"m", "-", "1"}],
        TraditionalForm], ")"}]}], "-", 
     RowBox[{"180", " ", 
      RowBox[{
       SuperscriptBox["log", "2"], "(", "2", ")"}], " ", 
      RowBox[{
       SubscriptBox["S", "2"], "(", 
       FormBox[
        RowBox[{"m", "-", "1"}],
        TraditionalForm], ")"}]}], "+", 
     RowBox[{"360", " ", 
      RowBox[{"log", "(", "2", ")"}], " ", 
      RowBox[{
       SubscriptBox["S", "3"], "(", 
       FormBox[
        RowBox[{"m", "-", "1"}],
        TraditionalForm], ")"}]}], "-", 
     RowBox[{"360", " ", 
      RowBox[{"log", "(", "2", ")"}], " ", 
      RowBox[{
       SubscriptBox["S", 
        RowBox[{"-", "21"}]], "(", 
       FormBox[
        RowBox[{"m", "-", "1"}],
        TraditionalForm], ")"}]}], "-", 
     RowBox[{"360", " ", 
      RowBox[{"log", "(", "2", ")"}], " ", 
      RowBox[{
       SubscriptBox["S", 
        RowBox[{"-", "12"}]], "(", 
       FormBox[
        RowBox[{"m", "-", "1"}],
        TraditionalForm], ")"}]}], "-", 
     RowBox[{"720", " ", 
      TemplateBox[{"4",FractionBox["1", "2"]},
       "PolyLog"]}], "-", 
     RowBox[{"945", " ", 
      TemplateBox[{"3"},
       "Zeta"], " ", 
      RowBox[{"log", "(", "2", ")"}]}], "+", 
     RowBox[{"8", " ", 
      SuperscriptBox["\[Pi]", "4"]}], "-", 
     RowBox[{"30", " ", 
      RowBox[{
       SuperscriptBox["log", "4"], "(", "2", ")"}]}]}], ")"}]}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970294532989*^9},
 CellTags->"Integrate2",
 CellLabel->"Out[966]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", " ", "/.", " ", 
  RowBox[{"OPEm", "\[Rule]", "2"}]}]], "Input",
 CellTags->"Integrate2",
 CellLabel->"In[967]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["1", "360"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"720", " ", 
      RowBox[{"\[Zeta]", "(", "2", ")"}]}], "+", 
     RowBox[{"450", " ", 
      RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
      RowBox[{
       SuperscriptBox["log", "2"], "(", "2", ")"}]}], "+", 
     RowBox[{"180", " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"\[Zeta]", "(", "2", ")"}], "-", 
        RowBox[{
         SuperscriptBox["log", "2"], "(", "2", ")"}], "+", 
        RowBox[{"log", "(", "4", ")"}]}], ")"}]}], "-", 
     RowBox[{"540", " ", 
      RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
      RowBox[{"log", "(", "2", ")"}]}], "-", 
     RowBox[{"720", " ", 
      TemplateBox[{"4",FractionBox["1", "2"]},
       "PolyLog"]}], "-", 
     RowBox[{"180", " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
         RowBox[{"log", "(", "8", ")"}]}], "-", 
        RowBox[{"2", " ", 
         TemplateBox[{"3"},
          "Zeta"]}]}], ")"}]}], "+", 
     RowBox[{"585", " ", 
      TemplateBox[{"3"},
       "Zeta"]}], "-", 
     RowBox[{"945", " ", 
      TemplateBox[{"3"},
       "Zeta"], " ", 
      RowBox[{"log", "(", "2", ")"}]}], "+", 
     RowBox[{"8", " ", 
      SuperscriptBox["\[Pi]", "4"]}], "-", "2160", "-", 
     RowBox[{"30", " ", 
      RowBox[{
       SuperscriptBox["log", "4"], "(", "2", ")"}]}], "-", 
     RowBox[{"180", " ", 
      RowBox[{
       SuperscriptBox["log", "2"], "(", "2", ")"}]}], "+", 
     RowBox[{"1080", " ", 
      RowBox[{"log", "(", "2", ")"}]}]}], ")"}]}], TraditionalForm]], "Output",\

 CellChangeTimes->{3.5009702946981573`*^9},
 CellTags->"Integrate2",
 CellLabel->"Out[967]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"N", "[", "%", "]"}]], "Input",
 CellTags->"Integrate2",
 CellLabel->"In[968]:="],

Cell[BoxData[
 FormBox["0.05051383540274483`", TraditionalForm]], "Output",
 CellChangeTimes->{3.500970294870023*^9},
 CellTags->"Integrate2",
 CellLabel->"Out[968]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"NIntegrate", "[", " ", 
  RowBox[{
   RowBox[{"x", " ", 
    RowBox[{"Log", "[", 
     RowBox[{"1", "-", "x"}], "]"}], " ", 
    RowBox[{"Log", "[", "x", "]"}], " ", 
    FractionBox[
     RowBox[{"Log", "[", 
      RowBox[{"1", "+", "x"}], "]"}], 
     RowBox[{"1", "+", "x"}]]}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellLabel->"In[969]:="],

Cell[BoxData[
 FormBox["0.050513835399059064`", TraditionalForm]], "Output",
 CellChangeTimes->{3.50097029539507*^9},
 CellLabel->"Out[969]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Integrate2", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"x", "^", 
     RowBox[{"(", 
      RowBox[{"OPEm", "-", "1"}], ")"}]}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"PolyLog", "[", 
       RowBox[{"3", ",", 
        FractionBox[
         RowBox[{"1", "-", "x"}], 
         RowBox[{"1", "+", "x"}]]}], "]"}], "-", 
      RowBox[{"PolyLog", "[", 
       RowBox[{"3", ",", 
        RowBox[{"-", 
         FractionBox[
          RowBox[{"1", "-", "x"}], 
          RowBox[{"1", "+", "x"}]]}]}], "]"}]}], ")"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellTags->"Integrate2",
 CellLabel->"In[970]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{"3", " ", 
     RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}], "m"], " ", 
     RowBox[{"log", "(", "2", ")"}]}], 
    RowBox[{"2", " ", "m"}]], "-", 
   FractionBox[
    RowBox[{"3", " ", 
     RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
     RowBox[{"log", "(", "2", ")"}]}], 
    RowBox[{"2", " ", "m"}]], "+", 
   FractionBox[
    RowBox[{
     RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}], "m"], " ", 
     RowBox[{
      SubscriptBox["S", 
       RowBox[{"-", "1"}]], "(", 
      FormBox["m",
       TraditionalForm], ")"}]}], "m"], "-", 
   FractionBox[
    RowBox[{
     RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
     RowBox[{
      SubscriptBox["S", 
       RowBox[{"-", "1"}]], "(", 
      FormBox["m",
       TraditionalForm], ")"}]}], 
    RowBox[{"2", " ", "m"}]], "+", 
   FractionBox[
    RowBox[{
     RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}], "m"], " ", 
     RowBox[{
      SubscriptBox["S", "1"], "(", 
      FormBox["m",
       TraditionalForm], ")"}]}], 
    RowBox[{"2", " ", "m"}]], "-", 
   FractionBox[
    RowBox[{
     RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
     RowBox[{
      SubscriptBox["S", "1"], "(", 
      FormBox["m",
       TraditionalForm], ")"}]}], "m"], "+", 
   FractionBox[
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}], "m"], " ", 
     RowBox[{
      SubscriptBox["S", 
       RowBox[{"-", "3"}]], "(", 
      FormBox["m",
       TraditionalForm], ")"}]}], "m"], "+", 
   FractionBox[
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}], "m"], " ", 
     RowBox[{
      SubscriptBox["S", 
       RowBox[{"-", "2"}]], "(", 
      FormBox["m",
       TraditionalForm], ")"}], " ", 
     RowBox[{
      SubscriptBox["S", "1"], "(", 
      FormBox["m",
       TraditionalForm], ")"}]}], "m"], "+", 
   FractionBox[
    RowBox[{
     RowBox[{
      SubscriptBox["S", "1"], "(", 
      FormBox["m",
       TraditionalForm], ")"}], " ", 
     RowBox[{
      SubscriptBox["S", "2"], "(", 
      FormBox["m",
       TraditionalForm], ")"}]}], "m"], "+", 
   FractionBox[
    RowBox[{
     SubscriptBox["S", "3"], "(", 
     FormBox["m",
      TraditionalForm], ")"}], "m"], "-", 
   FractionBox[
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}], "m"], " ", 
     RowBox[{
      SubscriptBox["S", 
       RowBox[{"-", "21"}]], "(", 
      FormBox["m",
       TraditionalForm], ")"}]}], "m"], "-", 
   FractionBox[
    RowBox[{
     SubscriptBox["S", 
      RowBox[{
       RowBox[{"-", "1"}], "-", "2"}]], "(", 
     FormBox["m",
      TraditionalForm], ")"}], "m"], "-", 
   FractionBox[
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}], "m"], " ", 
     RowBox[{
      SubscriptBox["S", 
       RowBox[{"-", "12"}]], "(", 
      FormBox["m",
       TraditionalForm], ")"}]}], "m"], "-", 
   FractionBox[
    RowBox[{
     SubscriptBox["S", "21"], "(", 
     FormBox["m",
      TraditionalForm], ")"}], "m"], "-", 
   FractionBox[
    RowBox[{"7", " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}], "m"], " ", 
     TemplateBox[{"3"},
      "Zeta"]}], 
    RowBox[{"8", " ", "m"}]], "+", 
   FractionBox[
    RowBox[{"21", " ", 
     TemplateBox[{"3"},
      "Zeta"]}], 
    RowBox[{"8", " ", "m"}]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970295545147*^9},
 CellTags->"Integrate2",
 CellLabel->"Out[970]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "/.", 
  RowBox[{"OPEm", "\[Rule]", "3"}]}]], "Input",
 CellLabel->"In[971]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    FractionBox[
     RowBox[{"\[Zeta]", "(", "2", ")"}], "2"]}], "+", 
   RowBox[{
    RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
    RowBox[{"(", 
     RowBox[{"-", 
      RowBox[{"log", "(", "2", ")"}]}], ")"}]}], "+", 
   FractionBox[
    RowBox[{"7", " ", 
     TemplateBox[{"3"},
      "Zeta"]}], "6"], "+", 
   FractionBox["2", "3"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.50097029614258*^9},
 CellLabel->"Out[971]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"N", "[", "%", "]"}]], "Input",
 CellLabel->"In[972]:="],

Cell[BoxData[
 FormBox["0.10641794295256102`", TraditionalForm]], "Output",
 CellChangeTimes->{3.500970296278006*^9},
 CellLabel->"Out[972]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"NIntegrate", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"x", "^", 
     RowBox[{"(", 
      RowBox[{"3", "-", "1"}], ")"}]}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"PolyLog", "[", 
       RowBox[{"3", ",", 
        FractionBox[
         RowBox[{"1", "-", "x"}], 
         RowBox[{"1", "+", "x"}]]}], "]"}], "-", 
      RowBox[{"PolyLog", "[", 
       RowBox[{"3", ",", 
        RowBox[{"-", 
         FractionBox[
          RowBox[{"1", "-", "x"}], 
          RowBox[{"1", "+", "x"}]]}]}], "]"}]}], ")"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellLabel->"In[973]:="],

Cell[BoxData[
 FormBox["0.10641794295771256`", TraditionalForm]], "Output",
 CellChangeTimes->{3.5009702964262342`*^9},
 CellLabel->"Out[973]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DataType", "[", 
  RowBox[{"OPEm", ",", "PositiveInteger"}], "]"}]], "Input",
 CellTags->"Integrate2",
 CellLabel->"In[974]:="],

Cell[BoxData[
 FormBox["True", TraditionalForm]], "Output",
 CellChangeTimes->{3.500970296490127*^9},
 CellTags->"Integrate2",
 CellLabel->"Out[974]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Integrate2", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"x", "^", 
     RowBox[{"(", 
      RowBox[{"OPEm", "-", "1"}], ")"}]}], " ", 
    RowBox[{"DeltaFunction", "[", 
     RowBox[{"1", "-", "x"}], "]"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellTags->"Integrate2",
 CellLabel->"In[975]:="],

Cell[BoxData[
 FormBox["1", TraditionalForm]], "Output",
 CellChangeTimes->{3.500970296644927*^9},
 CellTags->"Integrate2",
 CellLabel->"Out[975]="]
}, Open  ]],

Cell["\<\
This is the polarized non-singlet spin splitting function whose first moment \
vanishes.\
\>", "Text",
 CellTags->"Integrate2"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t", "=", 
  RowBox[{"SplittingFunction", "[", "PQQNS", "]"}]}]], "Input",
 CellTags->"Integrate2",
 CellLabel->"In[976]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    RowBox[{"-", "8"}], " ", 
    SubscriptBox["C", "F"], " ", 
    RowBox[{"(", 
     RowBox[{
      SubscriptBox["C", "F"], "-", 
      FractionBox[
       SubscriptBox["C", "A"], "2"]}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      FractionBox[
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox["x", "2"], "+", "1"}], ")"}], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"-", 
           RowBox[{"2", " ", 
            RowBox[{"\[Zeta]", "(", "2", ")"}]}]}], "-", 
          RowBox[{"4", " ", 
           TemplateBox[{"2",RowBox[{"-", "x"}]},
            "PolyLog"]}], "+", 
          RowBox[{
           SuperscriptBox["log", "2"], "(", "x", ")"}], "-", 
          RowBox[{"4", " ", 
           RowBox[{"log", "(", 
            RowBox[{"x", "+", "1"}], ")"}], " ", 
           RowBox[{"log", "(", "x", ")"}]}]}], ")"}]}], 
       RowBox[{"x", "+", "1"}]], "+", 
      RowBox[{"4", " ", 
       RowBox[{"(", 
        RowBox[{"1", "-", "x"}], ")"}]}], "+", 
      RowBox[{"2", " ", 
       RowBox[{"(", 
        RowBox[{"x", "+", "1"}], ")"}], " ", 
       RowBox[{"log", "(", "x", ")"}]}]}], ")"}]}], "+", 
   RowBox[{
    SubscriptBox["C", "A"], " ", 
    SubscriptBox["C", "F"], " ", 
    RowBox[{"(", 
     RowBox[{
      FractionBox[
       RowBox[{"4", " ", 
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox["x", "2"], "+", "1"}], ")"}], " ", 
        RowBox[{
         SuperscriptBox["log", "2"], "(", "x", ")"}]}], 
       RowBox[{"1", "-", "x"}]], "+", 
      RowBox[{"8", " ", 
       RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
       RowBox[{"(", 
        RowBox[{"x", "+", "1"}], ")"}]}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         FractionBox["536", "9"], "-", 
         RowBox[{"16", " ", 
          RowBox[{"\[Zeta]", "(", "2", ")"}]}]}], ")"}], " ", 
       SubscriptBox[
        RowBox[{"(", 
         FractionBox["1", 
          RowBox[{"1", "-", "x"}]], ")"}], "+"]}], "+", 
      RowBox[{
       RowBox[{"\[Delta]", "(", 
        FormBox[
         RowBox[{"1", "-", "x"}],
         TraditionalForm], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{
         FractionBox[
          RowBox[{"88", " ", 
           RowBox[{"\[Zeta]", "(", "2", ")"}]}], "3"], "-", 
         RowBox[{"24", " ", 
          TemplateBox[{"3"},
           "Zeta"]}], "+", 
         FractionBox["17", "3"]}], ")"}]}], "+", 
      RowBox[{
       FractionBox["4", "9"], " ", 
       RowBox[{"(", 
        RowBox[{"53", "-", 
         RowBox[{"187", " ", "x"}]}], ")"}]}], "-", 
      RowBox[{
       FractionBox["4", "3"], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"5", " ", "x"}], "-", 
         FractionBox["22", 
          RowBox[{"1", "-", "x"}]], "+", "5"}], ")"}], " ", 
       RowBox[{"log", "(", "x", ")"}]}]}], ")"}]}], "+", 
   RowBox[{
    SubscriptBox["C", "F"], " ", 
    SubscriptBox["N", "f"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", 
       FractionBox[
        RowBox[{"8", " ", 
         RowBox[{"(", 
          RowBox[{
           SuperscriptBox["x", "2"], "+", "1"}], ")"}], " ", 
         RowBox[{"log", "(", "x", ")"}]}], 
        RowBox[{"3", " ", 
         RowBox[{"(", 
          RowBox[{"1", "-", "x"}], ")"}]}]]}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", 
          FractionBox[
           RowBox[{"16", " ", 
            RowBox[{"\[Zeta]", "(", "2", ")"}]}], "3"]}], "-", 
         FractionBox["2", "3"]}], ")"}], " ", 
       RowBox[{"\[Delta]", "(", 
        FormBox[
         RowBox[{"1", "-", "x"}],
         TraditionalForm], ")"}]}], "+", 
      FractionBox[
       RowBox[{"88", " ", "x"}], "9"], "-", 
      RowBox[{
       FractionBox["80", "9"], " ", 
       SubscriptBox[
        RowBox[{"(", 
         FractionBox["1", 
          RowBox[{"1", "-", "x"}]], ")"}], "+"]}], "-", 
      FractionBox["8", "9"]}], ")"}]}], "+", 
   RowBox[{
    SubsuperscriptBox["C", "F", "2"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", 
       FractionBox[
        RowBox[{"16", " ", 
         RowBox[{"(", 
          RowBox[{
           SuperscriptBox["x", "2"], "+", "1"}], ")"}], " ", 
         RowBox[{"log", "(", 
          RowBox[{"1", "-", "x"}], ")"}], " ", 
         RowBox[{"log", "(", "x", ")"}]}], 
        RowBox[{"1", "-", "x"}]]}], "+", 
      RowBox[{
       RowBox[{"\[Delta]", "(", 
        FormBox[
         RowBox[{"1", "-", "x"}],
         TraditionalForm], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", 
          RowBox[{"24", " ", 
           RowBox[{"\[Zeta]", "(", "2", ")"}]}]}], "+", 
         RowBox[{"48", " ", 
          TemplateBox[{"3"},
           "Zeta"]}], "+", "3"}], ")"}]}], "-", 
      RowBox[{"40", " ", 
       RowBox[{"(", 
        RowBox[{"1", "-", "x"}], ")"}]}], "-", 
      RowBox[{"4", " ", 
       RowBox[{"(", 
        RowBox[{"x", "+", "1"}], ")"}], " ", 
       RowBox[{
        SuperscriptBox["log", "2"], "(", "x", ")"}]}], "-", 
      RowBox[{"8", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"2", " ", "x"}], "+", 
         FractionBox["3", 
          RowBox[{"1", "-", "x"}]]}], ")"}], " ", 
       RowBox[{"log", "(", "x", ")"}]}]}], ")"}]}]}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.50097029723085*^9},
 CellTags->"Integrate2",
 CellLabel->"Out[976]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Integrate2", "[", 
   RowBox[{"t", ",", 
    RowBox[{"{", 
     RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}], "//", 
  "Timing"}]], "Input",
 CellTags->"Integrate2",
 CellLabel->"In[977]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"0.12000000000000455`", ",", "0"}], "}"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.50097029743515*^9},
 CellTags->"Integrate2",
 CellLabel->"Out[977]="]
}, Open  ]],

Cell["\<\
Expanding t with respect to x yields a form already suitable for Integrate3 \
and therefore the following is faster:\
\>", "Text",
 CellTags->"Integrate2"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Integrate3", "[", 
    RowBox[{
     RowBox[{"Expand", "[", 
      RowBox[{"t", ",", "x"}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}], "//", "Expand"}], "//",
   "Timing"}]], "Input",
 CellTags->"Integrate2",
 CellLabel->"In[978]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"0.03999999999999915`", ",", "0"}], "}"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970297591552*^9},
 CellTags->"Integrate2",
 CellLabel->"Out[978]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", "t", "]"}], ";"}]], "Input",
 CellTags->"Integrate2",
 CellLabel->"In[979]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Integrate2", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"DeltaFunction", "[", 
     RowBox[{"1", "-", "x"}], "]"}], " ", 
    RowBox[{"f", "[", "x", "]"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellTags->"Integrate2",
 CellLabel->"In[980]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"f", "(", "1", ")"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970298116803*^9},
 CellTags->"Integrate2",
 CellLabel->"Out[980]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Integrate2", "[", 
  RowBox[{
   RowBox[{
    SuperscriptBox["x", "5"], 
    RowBox[{
     RowBox[{"Log", "[", 
      RowBox[{"1", "+", "x"}], "]"}], "^", "2"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellTags->"Integrate2",
 CellLabel->"In[981]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{"46", " ", 
     RowBox[{"log", "(", "2", ")"}]}], "45"], "-", 
   FractionBox["6959", "10800"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970298211884*^9},
 CellTags->"Integrate2",
 CellLabel->"Out[981]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"N", "@", "%"}]], "Input",
 CellTags->"Integrate2",
 CellLabel->"In[982]:="],

Cell[BoxData[
 FormBox["0.06419859938720329`", TraditionalForm]], "Output",
 CellChangeTimes->{3.50097029828743*^9},
 CellTags->"Integrate2",
 CellLabel->"Out[982]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"NIntegrate", "[", 
  RowBox[{
   RowBox[{
    SuperscriptBox["x", "5"], 
    RowBox[{
     RowBox[{"Log", "[", 
      RowBox[{"1", "+", "x"}], "]"}], "^", "2"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellTags->"Integrate2",
 CellLabel->"In[983]:="],

Cell[BoxData[
 FormBox["0.06419859938720252`", TraditionalForm]], "Output",
 CellChangeTimes->{3.5009702984190273`*^9},
 CellTags->"Integrate2",
 CellLabel->"Out[983]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Integrate2", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"x", "^", 
     RowBox[{"(", 
      RowBox[{"OPEm", "-", "1"}], ")"}]}], 
    SuperscriptBox[
     RowBox[{"Log", "[", 
      RowBox[{"1", "+", "x"}], "]"}], "2"]}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellTags->"Integrate2",
 CellLabel->"In[984]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    FractionBox[
     RowBox[{"2", " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"-", "1"}], ")"}], "m"], " ", 
      RowBox[{
       SubsuperscriptBox["S", "1", "2"], "(", 
       FormBox["m",
        TraditionalForm], ")"}]}], "m"]}], "+", 
   FractionBox[
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}], "m"], " ", 
     RowBox[{
      SubscriptBox["S", "1"], "(", 
      FormBox[
       FractionBox[
        RowBox[{"m", "-", "1"}], "2"],
       TraditionalForm], ")"}], " ", 
     RowBox[{
      SubscriptBox["S", "1"], "(", 
      FormBox["m",
       TraditionalForm], ")"}]}], "m"], "-", 
   FractionBox[
    RowBox[{
     RowBox[{
      SubscriptBox["S", "1"], "(", 
      FormBox[
       FractionBox[
        RowBox[{"m", "-", "1"}], "2"],
       TraditionalForm], ")"}], " ", 
     RowBox[{
      SubscriptBox["S", "1"], "(", 
      FormBox["m",
       TraditionalForm], ")"}]}], "m"], "+", 
   FractionBox[
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}], "m"], " ", 
     RowBox[{
      SubscriptBox["S", "1"], "(", 
      FormBox[
       FractionBox["m", "2"],
       TraditionalForm], ")"}], " ", 
     RowBox[{
      SubscriptBox["S", "1"], "(", 
      FormBox["m",
       TraditionalForm], ")"}]}], "m"], "+", 
   FractionBox[
    RowBox[{
     RowBox[{
      SubscriptBox["S", "1"], "(", 
      FormBox[
       FractionBox["m", "2"],
       TraditionalForm], ")"}], " ", 
     RowBox[{
      SubscriptBox["S", "1"], "(", 
      FormBox["m",
       TraditionalForm], ")"}]}], "m"], "+", 
   FractionBox[
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}], "m"], " ", 
     RowBox[{
      SubscriptBox["S", "2"], "(", 
      FormBox[
       FractionBox[
        RowBox[{"m", "-", "1"}], "2"],
       TraditionalForm], ")"}]}], 
    RowBox[{"2", " ", "m"}]], "-", 
   FractionBox[
    RowBox[{
     SubscriptBox["S", "2"], "(", 
     FormBox[
      FractionBox[
       RowBox[{"m", "-", "1"}], "2"],
      TraditionalForm], ")"}], 
    RowBox[{"2", " ", "m"}]], "+", 
   FractionBox[
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}], "m"], " ", 
     RowBox[{
      SubscriptBox["S", "2"], "(", 
      FormBox[
       FractionBox["m", "2"],
       TraditionalForm], ")"}]}], 
    RowBox[{"2", " ", "m"}]], "+", 
   FractionBox[
    RowBox[{
     SubscriptBox["S", "2"], "(", 
     FormBox[
      FractionBox["m", "2"],
      TraditionalForm], ")"}], 
    RowBox[{"2", " ", "m"}]], "-", 
   FractionBox[
    RowBox[{"2", " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}], "m"], " ", 
     RowBox[{
      SubscriptBox["S", "2"], "(", 
      FormBox["m",
       TraditionalForm], ")"}]}], "m"], "-", 
   FractionBox[
    RowBox[{"2", " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}], "m"], " ", 
     RowBox[{
      SubscriptBox["S", 
       RowBox[{"-", "11"}]], "(", 
      FormBox["m",
       TraditionalForm], ")"}]}], "m"], "+", 
   FractionBox[
    RowBox[{"4", " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}], "m"], " ", 
     RowBox[{"log", "(", "2", ")"}], " ", 
     RowBox[{
      SubscriptBox["S", "1"], "(", 
      FormBox["m",
       TraditionalForm], ")"}]}], "m"], "-", 
   FractionBox[
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}], "m"], " ", 
     RowBox[{"log", "(", "2", ")"}], " ", 
     RowBox[{
      SubscriptBox["S", "1"], "(", 
      FormBox[
       FractionBox[
        RowBox[{"m", "-", "1"}], "2"],
       TraditionalForm], ")"}]}], "m"], "+", 
   FractionBox[
    RowBox[{
     RowBox[{"log", "(", "2", ")"}], " ", 
     RowBox[{
      SubscriptBox["S", "1"], "(", 
      FormBox[
       FractionBox[
        RowBox[{"m", "-", "1"}], "2"],
       TraditionalForm], ")"}]}], "m"], "-", 
   FractionBox[
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}], "m"], " ", 
     RowBox[{"log", "(", "2", ")"}], " ", 
     RowBox[{
      SubscriptBox["S", "1"], "(", 
      FormBox[
       FractionBox["m", "2"],
       TraditionalForm], ")"}]}], "m"], "-", 
   FractionBox[
    RowBox[{
     RowBox[{"log", "(", "2", ")"}], " ", 
     RowBox[{
      SubscriptBox["S", "1"], "(", 
      FormBox[
       FractionBox["m", "2"],
       TraditionalForm], ")"}]}], "m"], "-", 
   FractionBox[
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}], "m"], " ", 
     RowBox[{
      SuperscriptBox["log", "2"], "(", "2", ")"}]}], "m"], "+", 
   FractionBox[
    RowBox[{
     SuperscriptBox["log", "2"], "(", "2", ")"}], "m"]}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970298705364*^9},
 CellTags->"Integrate2",
 CellLabel->"Out[984]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "/.", 
  RowBox[{"OPEm", "\[Rule]", "4"}]}]], "Input",
 CellLabel->"In[985]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{"4", " ", 
     RowBox[{"log", "(", "2", ")"}]}], "3"], "-", 
   FractionBox["241", "288"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009702988325453`*^9},
 CellLabel->"Out[985]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"N", "[", "%", "]"}]], "Input",
 CellLabel->"In[986]:="],

Cell[BoxData[
 FormBox["0.0873906851910381`", TraditionalForm]], "Output",
 CellChangeTimes->{3.500970298961246*^9},
 CellLabel->"Out[986]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"NIntegrate", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"x", "^", 
     RowBox[{"(", 
      RowBox[{"4", "-", "1"}], ")"}]}], 
    SuperscriptBox[
     RowBox[{"Log", "[", 
      RowBox[{"1", "+", "x"}], "]"}], "2"]}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellLabel->"In[987]:="],

Cell[BoxData[
 FormBox["0.0873906851910375`", TraditionalForm]], "Output",
 CellChangeTimes->{3.5009702991110497`*^9},
 CellLabel->"Out[987]="]
}, Open  ]]
}, Open  ]],

Cell["Integrate3", "ObjectName",
 CellTags->"Integrate3"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Integrate3"],

Cell["\<\
Integrate3 contains the integral table used by Integrate2. Integration is \
performed in a distributional sense. Integrate3 works more effectively on a \
sum of expressions if they are expanded or collected with respect to the \
integration variable. See the examples in Integrate2.\
\>", "Text",
 CellTags->"Integrate3"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Integrate2",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Integrate2",
  ButtonNote->"Integrate2"],
 "."
}], "Text",
 CellTags->"Integrate3"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Integrate3"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Integrate3", "[", 
  RowBox[{
   RowBox[{
    SuperscriptBox["x", "OPEm"], " ", 
    RowBox[{"Log", "[", "x", "]"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellTags->"Integrate3",
 CellLabel->"In[988]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   FractionBox["1", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"m", "+", "1"}], ")"}], "2"]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970299336751*^9},
 CellTags->"Integrate3",
 CellLabel->"Out[988]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Integrate3", "[", 
  RowBox[{
   FractionBox[
    RowBox[{
     SuperscriptBox["x", "OPEm"], " ", 
     RowBox[{"Log", "[", "x", "]"}], " ", 
     RowBox[{"Log", "[", 
      RowBox[{"1", "-", "x"}], "]"}]}], 
    RowBox[{"1", "-", "x"}]], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellTags->"Integrate3",
 CellLabel->"In[989]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
    RowBox[{
     SubscriptBox["S", "1"], "(", 
     FormBox["m",
      TraditionalForm], ")"}]}], "-", 
   RowBox[{
    SubscriptBox["S", "12"], "(", 
    FormBox["m",
     TraditionalForm], ")"}], "-", 
   RowBox[{
    SubscriptBox["S", "21"], "(", 
    FormBox["m",
     TraditionalForm], ")"}], "+", 
   TemplateBox[{"3"},
    "Zeta"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970299527429*^9},
 CellTags->"Integrate3",
 CellLabel->"Out[989]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Integrate3", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"a", 
     FractionBox[
      RowBox[{
       SuperscriptBox["x", "OPEm"], " ", 
       RowBox[{"Log", "[", "x", "]"}], " ", 
       RowBox[{"Log", "[", 
        RowBox[{"1", "-", "x"}], "]"}]}], 
      RowBox[{"1", "-", "x"}]]}], "+", 
    RowBox[{"b", 
     FractionBox[
      RowBox[{
       SuperscriptBox["x", "OPEm"], 
       RowBox[{"PolyLog", "[", 
        RowBox[{"3", ",", 
         RowBox[{"-", "x"}]}], "]"}]}], 
      RowBox[{"1", "+", "x"}]]}]}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellTags->"Integrate3",
 CellLabel->"In[990]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"a", " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
       RowBox[{
        SubscriptBox["S", "1"], "(", 
        FormBox["m",
         TraditionalForm], ")"}]}], "-", 
      RowBox[{
       SubscriptBox["S", "12"], "(", 
       FormBox["m",
        TraditionalForm], ")"}], "-", 
      RowBox[{
       SubscriptBox["S", "21"], "(", 
       FormBox["m",
        TraditionalForm], ")"}], "+", 
      TemplateBox[{"3"},
       "Zeta"]}], ")"}]}], "+", 
   RowBox[{"b", " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"-", "1"}], ")"}], "m"], " ", 
    RowBox[{"(", 
     RowBox[{
      FractionBox[
       SuperscriptBox[
        RowBox[{"\[Zeta]", "(", "2", ")"}], "2"], "8"], "+", 
      RowBox[{
       FractionBox["1", "2"], " ", 
       RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
       RowBox[{
        SubscriptBox["S", 
         RowBox[{"-", "2"}]], "(", 
        FormBox["m",
         TraditionalForm], ")"}]}], "-", 
      RowBox[{
       FractionBox["3", "4"], " ", 
       TemplateBox[{"3"},
        "Zeta"], " ", 
       RowBox[{
        SubscriptBox["S", 
         RowBox[{"-", "1"}]], "(", 
        FormBox["m",
         TraditionalForm], ")"}]}], "+", 
      RowBox[{
       SubscriptBox["S", 
        RowBox[{"3", "-", "1"}]], "(", 
       FormBox["m",
        TraditionalForm], ")"}], "+", 
      RowBox[{
       RowBox[{"log", "(", "2", ")"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          SubscriptBox["S", "3"], "(", 
          FormBox["m",
           TraditionalForm], ")"}], "-", 
         RowBox[{
          SubscriptBox["S", 
           RowBox[{"-", "3"}]], "(", 
          FormBox["m",
           TraditionalForm], ")"}]}], ")"}]}], "-", 
      RowBox[{
       FractionBox["3", "4"], " ", 
       TemplateBox[{"3"},
        "Zeta"], " ", 
       RowBox[{"log", "(", "2", ")"}]}]}], ")"}]}]}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970299793692*^9},
 CellTags->"Integrate3",
 CellLabel->"Out[990]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Integrate3", "[", 
  RowBox[{
   RowBox[{"DeltaFunctionPrime", "[", 
    RowBox[{"1", "-", "x"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellTags->"Integrate3",
 CellLabel->"In[991]:="],

Cell[BoxData[
 FormBox["0", TraditionalForm]], "Output",
 CellChangeTimes->{3.500970300004347*^9},
 CellTags->"Integrate3",
 CellLabel->"Out[991]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Integrate3", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"f", "[", "x", "]"}], " ", 
    RowBox[{"DeltaFunctionPrime", "[", 
     RowBox[{"1", "-", "x"}], "]"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellTags->"Integrate3",
 CellLabel->"In[992]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["f", "\[Prime]",
    MultilineFunction->None], "(", "1", ")"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009703001927137`*^9},
 CellTags->"Integrate3",
 CellLabel->"Out[992]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Integrate3", "[", 
  RowBox[{
   RowBox[{"1", "/", 
    RowBox[{"(", 
     RowBox[{"1", "-", "x"}], ")"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellTags->"Integrate3",
 CellLabel->"In[993]:="],

Cell[BoxData[
 FormBox["0", TraditionalForm]], "Output",
 CellChangeTimes->{3.500970300509534*^9},
 CellTags->"Integrate3",
 CellLabel->"Out[993]="]
}, Open  ]]
}, Open  ]],

Cell["IntermediateSubstitutions", "ObjectName",
 CellTags->"IntermediateSubstitutions"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"IntermediateSubstitutions"],

Cell["\<\
IntermediateSubstitutions is an option for OneLoop. All substitutions \
indicated hereby are done at an intermediate stage of the calculation.\
\>", "Text",
 CellTags->"IntermediateSubstitutions"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["OneLoop",
  BaseStyle->"AddOnsLink",
  ButtonData:>"OneLoop",
  ButtonNote->"OneLoop"],
 "."
}], "Text",
 CellTags->"IntermediateSubstitutions"]
}, Open  ]],

Cell["InverseMellin", "ObjectName",
 CellTags->"InverseMellin"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"InverseMellin"],

Cell[TextData[{
 "InverseMellin[exp, y] performs the inverse Mellin transform of polynomials \
in OPEm. The inverse transforms are not calculated but a table-lookup is \
done. WARNING: do not \"trust\" the results for the inverse Mellin transform \
involving SumT's; there is an unresolved inconsistency here (related to ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}], "^", "m"}], ")"}], "."}], 
   TraditionalForm]]]
}], "Text",
 CellTags->"InverseMellin"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["DeltaFunction",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DeltaFunction",
  ButtonNote->"DeltaFunction"],
 ", ",
 ButtonBox["Integrate2",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Integrate2",
  ButtonNote->"Integrate2"],
 ", ",
 ButtonBox["OPEm",
  BaseStyle->"AddOnsLink",
  ButtonData:>"OPEi",
  ButtonNote->"SumS"],
 ", ",
 ButtonBox["SumS",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SumS",
  ButtonNote->"SumS"],
 ", ",
 ButtonBox["SumT",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SumT",
  ButtonNote->"SumT"],
 "."
}], "Text",
 CellTags->"InverseMellin"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"InverseMellin"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"InverseMellin", "[", 
  RowBox[{
   RowBox[{"1", "/", "OPEm"}], ",", "y"}], "]"}]], "Input",
 CellTags->"InverseMellin",
 CellLabel->"In[994]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["y", 
   RowBox[{"m", "-", "1"}]], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009703006463337`*^9},
 CellTags->"InverseMellin",
 CellLabel->"Out[994]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"InverseMellin", "[", 
  RowBox[{
   RowBox[{"1", "/", 
    RowBox[{"(", 
     RowBox[{"OPEm", "+", "3"}], ")"}]}], ",", "y"}], "]"}]], "Input",
 CellTags->"InverseMellin",
 CellLabel->"In[995]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["y", 
   RowBox[{"m", "+", "2"}]], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970300761024*^9},
 CellTags->"InverseMellin",
 CellLabel->"Out[995]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"InverseMellin", "[", 
  RowBox[{"1", ",", "y"}], "]"}]], "Input",
 CellTags->"InverseMellin",
 CellLabel->"In[996]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["y", 
    RowBox[{"m", "-", "1"}]], " ", 
   RowBox[{"\[Delta]", "(", 
    FormBox[
     RowBox[{"1", "-", "y"}],
     TraditionalForm], ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970300916089*^9},
 CellTags->"InverseMellin",
 CellLabel->"Out[996]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"InverseMellin", "[", 
  RowBox[{
   RowBox[{"1", "/", 
    RowBox[{"OPEm", "^", "4"}]}], ",", "y"}], "]"}]], "Input",
 CellTags->"InverseMellin",
 CellLabel->"In[997]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    FractionBox["1", "6"]}], " ", 
   SuperscriptBox["y", 
    RowBox[{"m", "-", "1"}]], " ", 
   RowBox[{
    SuperscriptBox["log", "3"], "(", "y", ")"}]}], TraditionalForm]], "Output",\

 CellChangeTimes->{3.5009703014643183`*^9},
 CellTags->"InverseMellin",
 CellLabel->"Out[997]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"InverseMellin", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"1", "/", "OPEm"}], "+", "1"}], ",", "y"}], "]"}]], "Input",
 CellTags->"InverseMellin",
 CellLabel->"In[998]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SuperscriptBox["y", 
     RowBox[{"m", "-", "1"}]], " ", 
    RowBox[{"\[Delta]", "(", 
     FormBox[
      RowBox[{"1", "-", "y"}],
      TraditionalForm], ")"}]}], "+", 
   SuperscriptBox["y", 
    RowBox[{"m", "-", "1"}]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009703016305323`*^9},
 CellTags->"InverseMellin",
 CellLabel->"Out[998]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"InverseMellin", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"1", "/", "i"}], "+", "1"}], ",", "y", ",", "i"}], "]"}]], "Input",\

 CellTags->"InverseMellin",
 CellLabel->"In[999]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SuperscriptBox["y", 
     RowBox[{"i", "-", "1"}]], " ", 
    RowBox[{"\[Delta]", "(", 
     FormBox[
      RowBox[{"1", "-", "y"}],
      TraditionalForm], ")"}]}], "+", 
   SuperscriptBox["y", 
    RowBox[{"i", "-", "1"}]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009703017767067`*^9},
 CellTags->"InverseMellin",
 CellLabel->"Out[999]="]
}, Open  ]],

Cell["\<\
The inverse operation to InverseMellin is done by Integrate2.\
\>", "Text",
 CellTags->"InverseMellin"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Integrate2", "[", 
  RowBox[{
   RowBox[{"InverseMellin", "[", 
    RowBox[{
     RowBox[{"1", "/", "OPEm"}], ",", "y"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"y", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellTags->"InverseMellin",
 CellLabel->"In[1000]:="],

Cell[BoxData[
 FormBox[
  FractionBox["1", "m"], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970301931012*^9},
 CellTags->"InverseMellin",
 CellLabel->"Out[1000]="]
}, Open  ]],

Cell["\<\
Below is a list of all built-in basic inverse Mellin transforms .\
\>", "Text",
 CellTags->"InverseMellin"],

Cell[BoxData[
 RowBox[{
  RowBox[{"list", "=", 
   RowBox[{"{", 
    RowBox[{"1", ",", 
     FractionBox["1", 
      RowBox[{"OPEm", "+", "n"}]], ",", 
     FractionBox["1", 
      RowBox[{
       RowBox[{"-", "OPEm"}], "+", "n"}]], ",", 
     RowBox[{"PolyGamma", "[", 
      RowBox[{"0", ",", "OPEm"}], "]"}], ",", 
     RowBox[{"SumS", "[", 
      RowBox[{"1", ",", 
       RowBox[{
        RowBox[{"-", "1"}], "+", "OPEm"}]}], "]"}], ",", 
     FractionBox[
      RowBox[{"SumS", "[", 
       RowBox[{"1", ",", 
        RowBox[{
         RowBox[{"-", "1"}], "+", "OPEm"}]}], "]"}], 
      RowBox[{"OPEm", "-", "1"}]], ",", 
     FractionBox[
      RowBox[{"SumS", "[", 
       RowBox[{"1", ",", 
        RowBox[{
         RowBox[{"-", "1"}], "+", "OPEm"}]}], "]"}], 
      RowBox[{"1", "-", "OPEm"}]], ",", 
     FractionBox[
      RowBox[{"SumS", "[", 
       RowBox[{"1", ",", 
        RowBox[{
         RowBox[{"-", "1"}], "+", "OPEm"}]}], "]"}], 
      RowBox[{"OPEm", "+", "1"}]], ",", 
     FractionBox[
      RowBox[{"SumS", "[", 
       RowBox[{"1", ",", 
        RowBox[{
         RowBox[{"-", "1"}], "+", "OPEm"}]}], "]"}], 
      SuperscriptBox["OPEm", "2"]], ",", 
     FractionBox[
      RowBox[{"SumS", "[", 
       RowBox[{"1", ",", 
        RowBox[{
         RowBox[{"-", "1"}], "+", "OPEm"}]}], "]"}], "OPEm"], ",", 
     FractionBox[
      SuperscriptBox[
       RowBox[{"SumS", "[", 
        RowBox[{"1", ",", 
         RowBox[{
          RowBox[{"-", "1"}], "+", "OPEm"}]}], "]"}], "2"], "OPEm"], ",", 
     RowBox[{"SumS", "[", 
      RowBox[{"2", ",", 
       RowBox[{
        RowBox[{"-", "1"}], "+", "OPEm"}]}], "]"}], ",", 
     FractionBox[
      RowBox[{"SumS", "[", 
       RowBox[{"2", ",", 
        RowBox[{
         RowBox[{"-", "1"}], "+", "OPEm"}]}], "]"}], "OPEm"], ",", 
     RowBox[{"SumS", "[", 
      RowBox[{"3", ",", 
       RowBox[{
        RowBox[{"-", "1"}], "+", "OPEm"}]}], "]"}], ",", 
     RowBox[{"SumS", "[", 
      RowBox[{"1", ",", "1", ",", 
       RowBox[{
        RowBox[{"-", "1"}], "+", "OPEm"}]}], "]"}], ",", 
     SuperscriptBox[
      RowBox[{"SumS", "[", 
       RowBox[{"1", ",", 
        RowBox[{"OPEm", "-", "1"}]}], "]"}], "2"], ",", 
     RowBox[{"SumS", "[", 
      RowBox[{"1", ",", "2", ",", 
       RowBox[{
        RowBox[{"-", "1"}], "+", "OPEm"}]}], "]"}], ",", 
     RowBox[{"SumS", "[", 
      RowBox[{"2", ",", "1", ",", 
       RowBox[{
        RowBox[{"-", "1"}], "+", "OPEm"}]}], "]"}], ",", 
     SuperscriptBox[
      RowBox[{"SumS", "[", 
       RowBox[{"1", ",", 
        RowBox[{
         RowBox[{"-", "1"}], "+", "OPEm"}]}], "]"}], "3"], ",", 
     RowBox[{
      RowBox[{"SumS", "[", 
       RowBox[{"1", ",", 
        RowBox[{
         RowBox[{"-", "1"}], "+", "OPEm"}]}], "]"}], " ", 
      RowBox[{"SumS", "[", 
       RowBox[{"2", ",", 
        RowBox[{
         RowBox[{"-", "1"}], "+", "OPEm"}]}], "]"}]}], ",", 
     RowBox[{"SumS", "[", 
      RowBox[{"1", ",", "1", ",", "1", ",", 
       RowBox[{
        RowBox[{"-", "1"}], "+", "OPEm"}]}], "]"}]}], "}"}]}], ";"}]], "Input",\

 CellTags->"InverseMellin",
 CellLabel->"In[1001]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"im", "[", "z_", "]"}], ":=", 
  RowBox[{"z", " ", "\[LongRightArrow]", 
   RowBox[{"InverseMellin", "[", 
    RowBox[{"z", ",", "y"}], "]"}]}]}]], "Input",
 CellTags->"InverseMellin",
 CellLabel->"In[1002]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"im", "[", 
  RowBox[{"OPEm", "^", 
   RowBox[{"(", 
    RowBox[{"-", "3"}], ")"}]}], "]"}]], "Input",
 CellTags->"InverseMellin",
 CellLabel->"In[1003]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["1", 
    SuperscriptBox["m", "3"]], "\[LongRightArrow]", 
   RowBox[{"(", 
    RowBox[{
     FractionBox["1", "2"], " ", 
     SuperscriptBox["y", 
      RowBox[{"m", "-", "1"}]], " ", 
     RowBox[{
      SuperscriptBox["log", "2"], "(", "y", ")"}]}], ")"}]}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970302218384*^9},
 CellTags->"InverseMellin",
 CellLabel->"Out[1003]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"im", "[", 
  RowBox[{"OPEm", "^", 
   RowBox[{"(", 
    RowBox[{"-", "2"}], ")"}]}], "]"}]], "Input",
 CellTags->"InverseMellin",
 CellLabel->"In[1004]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["1", 
    SuperscriptBox["m", "2"]], "\[LongRightArrow]", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      SuperscriptBox["y", 
       RowBox[{"m", "-", "1"}]]}], " ", 
     RowBox[{"log", "(", "y", ")"}]}], ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970302357243*^9},
 CellTags->"InverseMellin",
 CellLabel->"Out[1004]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"im", "[", 
  RowBox[{"PolyGamma", "[", 
   RowBox[{"0", ",", "OPEm"}], "]"}], "]"}]], "Input",
 CellTags->"InverseMellin",
 CellLabel->"In[1005]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TemplateBox[{"0","m"},
    "PolyGamma2"], "\[LongRightArrow]", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      RowBox[{
       TagBox["\[DoubledGamma]",
        Function[{}, EulerGamma]], " ", 
       SuperscriptBox["y", 
        RowBox[{"m", "-", "1"}]], " ", 
       RowBox[{"\[Delta]", "(", 
        FormBox[
         RowBox[{"1", "-", "y"}],
         TraditionalForm], ")"}]}]}], "-", 
     RowBox[{
      SubscriptBox[
       RowBox[{"(", 
        FractionBox["1", 
         RowBox[{"1", "-", "y"}]], ")"}], "+"], " ", 
      SuperscriptBox["y", 
       RowBox[{"m", "-", "1"}]]}]}], ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970302559854*^9},
 CellTags->"InverseMellin",
 CellLabel->"Out[1005]="]
}, Open  ]],

Cell[BoxData[" "], "Input",
 CellTags->"InverseMellin",
 CellLabel->"In[1006]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"im", "[", 
  RowBox[{"SumS", "[", 
   RowBox[{"1", ",", 
    RowBox[{"OPEm", "-", "1"}]}], "]"}], "]"}]], "Input",
 CellTags->"InverseMellin",
 CellLabel->"In[1007]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox["S", "1"], "(", 
    FormBox[
     RowBox[{"m", "-", "1"}],
     TraditionalForm], ")"}], "\[LongRightArrow]", 
   RowBox[{"(", 
    RowBox[{
     SubscriptBox[
      RowBox[{"(", 
       FractionBox["1", 
        RowBox[{"1", "-", "y"}]], ")"}], "+"], " ", 
     RowBox[{"(", 
      RowBox[{"-", 
       SuperscriptBox["y", 
        RowBox[{"m", "-", "1"}]]}], ")"}]}], ")"}]}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970302681383*^9},
 CellTags->"InverseMellin",
 CellLabel->"Out[1007]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"im", "[", 
  RowBox[{
   RowBox[{"SumS", "[", 
    RowBox[{"1", ",", 
     RowBox[{"OPEm", "-", "1"}]}], "]"}], "/", 
   RowBox[{"(", 
    RowBox[{"OPEm", "-", "1"}], ")"}]}], "]"}]], "Input",
 CellTags->"InverseMellin",
 CellLabel->"In[1008]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{
     SubscriptBox["S", "1"], "(", 
     FormBox[
      RowBox[{"m", "-", "1"}],
      TraditionalForm], ")"}], 
    RowBox[{"m", "-", "1"}]], "\[LongRightArrow]", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      SuperscriptBox["y", 
       RowBox[{"m", "-", "2"}]]}], " ", 
     RowBox[{"log", "(", 
      RowBox[{"1", "-", "y"}], ")"}]}], ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970302803019*^9},
 CellTags->"InverseMellin",
 CellLabel->"Out[1008]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"im", "[", 
  RowBox[{
   RowBox[{"SumS", "[", 
    RowBox[{"1", ",", 
     RowBox[{"OPEm", "-", "1"}]}], "]"}], "/", 
   RowBox[{"(", 
    RowBox[{"OPEm", "+", "1"}], ")"}]}], "]"}]], "Input",
 CellTags->"InverseMellin",
 CellLabel->"In[1009]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{
     SubscriptBox["S", "1"], "(", 
     FormBox[
      RowBox[{"m", "-", "1"}],
      TraditionalForm], ")"}], 
    RowBox[{"m", "+", "1"}]], "\[LongRightArrow]", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      SuperscriptBox["y", 
       RowBox[{"m", "-", "1"}]]}], "+", 
     SuperscriptBox["y", "m"], "-", 
     RowBox[{
      SuperscriptBox["y", "m"], " ", 
      RowBox[{"log", "(", 
       RowBox[{"1", "-", "y"}], ")"}]}], "+", 
     RowBox[{
      SuperscriptBox["y", "m"], " ", 
      RowBox[{"log", "(", "y", ")"}]}]}], ")"}]}], TraditionalForm]], "Output",\

 CellChangeTimes->{3.500970302915082*^9},
 CellTags->"InverseMellin",
 CellLabel->"Out[1009]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"im", "[", 
  RowBox[{
   RowBox[{"SumS", "[", 
    RowBox[{"1", ",", 
     RowBox[{
      RowBox[{"-", "1"}], "+", "OPEm"}]}], "]"}], "/", 
   RowBox[{"OPEm", "^", "2"}]}], "]"}]], "Input",
 CellTags->"InverseMellin",
 CellLabel->"In[1010]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{
     SubscriptBox["S", "1"], "(", 
     FormBox[
      RowBox[{"m", "-", "1"}],
      TraditionalForm], ")"}], 
    SuperscriptBox["m", "2"]], "\[LongRightArrow]", 
   RowBox[{"(", 
    RowBox[{
     SuperscriptBox["y", 
      RowBox[{"m", "-", "1"}]], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"\[Zeta]", "(", "2", ")"}], "-", 
       TemplateBox[{"2","y"},
        "PolyLog"], "-", 
       RowBox[{
        FractionBox["1", "2"], " ", 
        RowBox[{
         SuperscriptBox["log", "2"], "(", "y", ")"}]}]}], ")"}]}], ")"}]}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.50097030305092*^9},
 CellTags->"InverseMellin",
 CellLabel->"Out[1010]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"im", "[", 
  RowBox[{
   RowBox[{"SumS", "[", 
    RowBox[{"1", ",", 
     RowBox[{
      RowBox[{"-", "1"}], "+", "OPEm"}]}], "]"}], "/", "OPEm"}], 
  "]"}]], "Input",
 CellTags->"InverseMellin",
 CellLabel->"In[1011]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{
     SubscriptBox["S", "1"], "(", 
     FormBox[
      RowBox[{"m", "-", "1"}],
      TraditionalForm], ")"}], "m"], "\[LongRightArrow]", 
   RowBox[{"(", 
    RowBox[{
     SuperscriptBox["y", 
      RowBox[{"m", "-", "1"}]], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"log", "(", "y", ")"}], "-", 
       RowBox[{"log", "(", 
        RowBox[{"1", "-", "y"}], ")"}]}], ")"}]}], ")"}]}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970303244232*^9},
 CellTags->"InverseMellin",
 CellLabel->"Out[1011]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"im", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"SumS", "[", 
     RowBox[{"1", ",", 
      RowBox[{
       RowBox[{"-", "1"}], "+", "OPEm"}]}], "]"}], "^", "2"}], "/", "OPEm"}], 
  "]"}]], "Input",
 CellTags->"InverseMellin",
 CellLabel->"In[1012]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{
     SubsuperscriptBox["S", "1", "2"], "(", 
     FormBox[
      RowBox[{"m", "-", "1"}],
      TraditionalForm], ")"}], "m"], "\[LongRightArrow]", 
   RowBox[{"(", 
    RowBox[{
     SuperscriptBox["y", 
      RowBox[{"m", "-", "1"}]], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"-", 
        RowBox[{"3", " ", 
         RowBox[{"\[Zeta]", "(", "2", ")"}]}]}], "+", 
       TemplateBox[{"2",RowBox[{"1", "-", "y"}]},
        "PolyLog"], "+", 
       RowBox[{"2", " ", 
        TemplateBox[{"2","y"},
         "PolyLog"]}], "+", 
       RowBox[{
        SuperscriptBox["log", "2"], "(", 
        RowBox[{"1", "-", "y"}], ")"}], "+", 
       FractionBox[
        RowBox[{
         SuperscriptBox["log", "2"], "(", "y", ")"}], "2"]}], ")"}]}], 
    ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970303417944*^9},
 CellTags->"InverseMellin",
 CellLabel->"Out[1012]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"im", "[", 
  RowBox[{"SumS", "[", 
   RowBox[{"2", ",", 
    RowBox[{"OPEm", "-", "1"}]}], "]"}], "]"}]], "Input",
 CellTags->"InverseMellin",
 CellLabel->"In[1013]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox["S", "2"], "(", 
    FormBox[
     RowBox[{"m", "-", "1"}],
     TraditionalForm], ")"}], "\[LongRightArrow]", 
   RowBox[{"(", 
    RowBox[{
     SuperscriptBox["y", 
      RowBox[{"m", "-", "1"}]], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
        RowBox[{"\[Delta]", "(", 
         FormBox[
          RowBox[{"1", "-", "y"}],
          TraditionalForm], ")"}]}], "+", 
       FractionBox[
        RowBox[{"log", "(", "y", ")"}], 
        RowBox[{"1", "-", "y"}]]}], ")"}]}], ")"}]}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970303567341*^9},
 CellTags->"InverseMellin",
 CellLabel->"Out[1013]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"im", "[", 
  RowBox[{
   RowBox[{"SumS", "[", 
    RowBox[{"2", ",", 
     RowBox[{"OPEm", "-", "1"}]}], "]"}], "/", "OPEm"}], "]"}]], "Input",
 CellTags->"InverseMellin",
 CellLabel->"In[1014]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{
     SubscriptBox["S", "2"], "(", 
     FormBox[
      RowBox[{"m", "-", "1"}],
      TraditionalForm], ")"}], "m"], "\[LongRightArrow]", 
   RowBox[{"(", 
    RowBox[{
     SuperscriptBox["y", 
      RowBox[{"m", "-", "1"}]], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"\[Zeta]", "(", "2", ")"}], "-", 
       TemplateBox[{"2",RowBox[{"1", "-", "y"}]},
        "PolyLog"], "-", 
       RowBox[{
        FractionBox["1", "2"], " ", 
        RowBox[{
         SuperscriptBox["log", "2"], "(", "y", ")"}]}]}], ")"}]}], ")"}]}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970303720565*^9},
 CellTags->"InverseMellin",
 CellLabel->"Out[1014]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"im", "[", 
  RowBox[{"SumS", "[", 
   RowBox[{"3", ",", 
    RowBox[{"OPEm", "-", "1"}]}], "]"}], "]"}]], "Input",
 CellTags->"InverseMellin",
 CellLabel->"In[1015]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox["S", "3"], "(", 
    FormBox[
     RowBox[{"m", "-", "1"}],
     TraditionalForm], ")"}], "\[LongRightArrow]", 
   RowBox[{"(", 
    RowBox[{
     SuperscriptBox["y", 
      RowBox[{"m", "-", "1"}]], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        TemplateBox[{"3"},
         "Zeta"], " ", 
        RowBox[{"\[Delta]", "(", 
         FormBox[
          RowBox[{"1", "-", "y"}],
          TraditionalForm], ")"}]}], "-", 
       FractionBox[
        RowBox[{
         SuperscriptBox["log", "2"], "(", "y", ")"}], 
        RowBox[{"2", " ", 
         RowBox[{"(", 
          RowBox[{"1", "-", "y"}], ")"}]}]]}], ")"}]}], ")"}]}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.5009703038662767`*^9},
 CellTags->"InverseMellin",
 CellLabel->"Out[1015]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"im", "[", 
  RowBox[{"SumS", "[", 
   RowBox[{"1", ",", "1", ",", 
    RowBox[{"OPEm", "-", "1"}]}], "]"}], "]"}]], "Input",
 CellTags->"InverseMellin",
 CellLabel->"In[1016]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox["S", "11"], "(", 
    FormBox[
     RowBox[{"m", "-", "1"}],
     TraditionalForm], ")"}], "\[LongRightArrow]", 
   RowBox[{"(", 
    RowBox[{
     SuperscriptBox["y", 
      RowBox[{"m", "-", "1"}]], " ", 
     SubscriptBox[
      RowBox[{"(", 
       FractionBox[
        RowBox[{"log", "(", 
         RowBox[{"1", "-", "y"}], ")"}], 
        RowBox[{"1", "-", "y"}]], ")"}], "+"]}], ")"}]}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970304037127*^9},
 CellTags->"InverseMellin",
 CellLabel->"Out[1016]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{" ", 
  RowBox[{"im", "[", 
   RowBox[{"SumS", "[", 
    RowBox[{"1", ",", "2", ",", 
     RowBox[{"OPEm", "-", "1"}]}], "]"}], "]"}]}]], "Input",
 CellTags->"InverseMellin",
 CellLabel->"In[1017]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox["S", "12"], "(", 
    FormBox[
     RowBox[{"m", "-", "1"}],
     TraditionalForm], ")"}], "\[LongRightArrow]", 
   RowBox[{"(", 
    RowBox[{
     SuperscriptBox["y", 
      RowBox[{"m", "-", "1"}]], " ", 
     RowBox[{"(", 
      RowBox[{
       FractionBox[
        TemplateBox[{"2",RowBox[{"1", "-", "y"}]},
         "PolyLog"], 
        RowBox[{"1", "-", "y"}]], "-", 
       RowBox[{
        RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
        SubscriptBox[
         RowBox[{"(", 
          FractionBox["1", 
           RowBox[{"1", "-", "y"}]], ")"}], "+"]}], "-", 
       RowBox[{
        TemplateBox[{"3"},
         "Zeta"], " ", 
        RowBox[{"\[Delta]", "(", 
         FormBox[
          RowBox[{"1", "-", "y"}],
          TraditionalForm], ")"}]}]}], ")"}]}], ")"}]}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.5009703042437067`*^9},
 CellTags->"InverseMellin",
 CellLabel->"Out[1017]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{" ", 
  RowBox[{"im", "[", 
   RowBox[{"SumS", "[", 
    RowBox[{"2", ",", "1", ",", 
     RowBox[{"OPEm", "-", "1"}]}], "]"}], "]"}]}]], "Input",
 CellTags->"InverseMellin",
 CellLabel->"In[1018]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox["S", "21"], "(", 
    FormBox[
     RowBox[{"m", "-", "1"}],
     TraditionalForm], ")"}], "\[LongRightArrow]", 
   RowBox[{"(", 
    RowBox[{
     SuperscriptBox["y", 
      RowBox[{"m", "-", "1"}]], " ", 
     RowBox[{"(", 
      RowBox[{
       FractionBox[
        TemplateBox[{"2","y"},
         "PolyLog"], 
        RowBox[{"1", "-", "y"}]], "-", 
       RowBox[{
        RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
        SubscriptBox[
         RowBox[{"(", 
          FractionBox["1", 
           RowBox[{"1", "-", "y"}]], ")"}], "+"]}], "+", 
       RowBox[{"2", " ", 
        TemplateBox[{"3"},
         "Zeta"], " ", 
        RowBox[{"\[Delta]", "(", 
         FormBox[
          RowBox[{"1", "-", "y"}],
          TraditionalForm], ")"}]}]}], ")"}]}], ")"}]}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970304427321*^9},
 CellTags->"InverseMellin",
 CellLabel->"Out[1018]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"im", "[", 
  RowBox[{"SumS", "[", 
   RowBox[{"1", ",", "1", ",", "1", ",", 
    RowBox[{"OPEm", "-", "1"}]}], "]"}], "]"}]], "Input",
 CellTags->"InverseMellin",
 CellLabel->"In[1019]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox["S", "111"], "(", 
    FormBox[
     RowBox[{"m", "-", "1"}],
     TraditionalForm], ")"}], "\[LongRightArrow]", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      FractionBox["1", "2"]}], " ", 
     SuperscriptBox["y", 
      RowBox[{"m", "-", "1"}]], " ", 
     SubscriptBox[
      RowBox[{"(", 
       FractionBox[
        RowBox[{
         SuperscriptBox["log", "2"], "(", 
         RowBox[{"1", "-", "y"}], ")"}], 
        RowBox[{"1", "-", "y"}]], ")"}], "+"]}], ")"}]}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970304594678*^9},
 CellTags->"InverseMellin",
 CellLabel->"Out[1019]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", 
   RowBox[{"im", ",", "list"}], "]"}], ";"}]], "Input",
 CellTags->"InverseMellin",
 CellLabel->"In[1020]:="]
}, Open  ]],

Cell["Isolate", "ObjectName",
 CellTags->"Isolate"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Isolate"],

Cell["\<\
Isolate[expr] substitutes abbreviations KK[i] for all Plus[...] (sub-sums) in \
expr. The inserted KK[i] have head HoldForm. Isolate[expr, varlist] \
substitutes KK[i] for all subsums in expr which are free of any occurence of \
a member of the list varlist. Instead of KK any other head or a list of names \
of the abbreviations may be specified with the option IsolateNames.\
\>", "Text",
 CellTags->"Isolate"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "Isolate", "]"}]], "Input",
 CellTags->"Isolate",
 CellLabel->"In[1021]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"IsolateNames", "\[Rule]", "KK"}], ",", 
    RowBox[{
    "HighEnergyPhysics`FeynCalc`IsolatePrint`IsolatePrint", "\[Rule]", 
     "False"}], ",", 
    RowBox[{
    "HighEnergyPhysics`FeynCalc`IsolateSplit`IsolateSplit", "\[Rule]", 
     "\[Infinity]"}]}], "}"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970304727584*^9},
 CellTags->"Isolate",
 CellLabel->"Out[1021]="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Isolate"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t0", "=", 
  RowBox[{"Isolate", "[", 
   RowBox[{"a", "+", "b"}], "]"}]}]], "Input",
 CellTags->"Isolate",
 CellLabel->"In[1022]:="],

Cell[BoxData[
 FormBox[
  TagBox[
   RowBox[{"KK", "(", "126", ")"}],
   HoldForm], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970304772038*^9},
 CellTags->"Isolate",
 CellLabel->"Out[1022]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t1", "=", 
  RowBox[{"Isolate", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{"a", "+", "b"}], ")"}], " ", "f"}], " ", "+", " ", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"c", "+", "d"}], ")"}], " ", "f"}], " ", "+", " ", "e"}], ",", 
    "f"}], "]"}]}]], "Input",
 CellTags->"Isolate",
 CellLabel->"In[1023]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"e", "+", 
   RowBox[{"f", " ", 
    TagBox[
     RowBox[{"KK", "(", "126", ")"}],
     HoldForm]}], "+", 
   RowBox[{"f", " ", 
    TagBox[
     RowBox[{"KK", "(", "127", ")"}],
     HoldForm]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970304902705*^9},
 CellTags->"Isolate",
 CellLabel->"Out[1023]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StandardForm", "[", "t1", "]"}]], "Input",
 CellTags->"Isolate",
 CellLabel->"In[1024]:="],

Cell[BoxData[
 RowBox[{"e", "+", 
  RowBox[{"f", " ", 
   TagBox[
    RowBox[{"KK", "[", "126", "]"}],
    HoldForm]}], "+", 
  RowBox[{"f", " ", 
   TagBox[
    RowBox[{"KK", "[", "127", "]"}],
    HoldForm]}]}]], "Output",
 CellChangeTimes->{3.500970304994478*^9},
 CellTags->"Isolate",
 CellLabel->"Out[1024]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"t0", ",", " ", "t1", ",", " ", 
   RowBox[{"ReleaseHold", "[", "t1", "]"}]}], "}"}]], "Input",
 CellTags->"Isolate",
 CellLabel->"In[1025]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    TagBox[
     RowBox[{"KK", "(", "126", ")"}],
     HoldForm], ",", 
    RowBox[{"e", "+", 
     RowBox[{"f", " ", 
      TagBox[
       RowBox[{"KK", "(", "126", ")"}],
       HoldForm]}], "+", 
     RowBox[{"f", " ", 
      TagBox[
       RowBox[{"KK", "(", "127", ")"}],
       HoldForm]}]}], ",", 
    RowBox[{
     RowBox[{"f", " ", 
      RowBox[{"(", 
       RowBox[{"a", "+", "b"}], ")"}]}], "+", 
     RowBox[{"f", " ", 
      RowBox[{"(", 
       RowBox[{"c", "+", "d"}], ")"}]}], "+", "e"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970305082542*^9},
 CellTags->"Isolate",
 CellLabel->"Out[1025]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Isolate", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"a", "[", "z", "]"}], " ", 
     RowBox[{"(", 
      RowBox[{"b", "+", 
       RowBox[{"c", " ", 
        RowBox[{"(", 
         RowBox[{"y", "+", "z"}], ")"}]}]}], ")"}]}], "+", 
    RowBox[{
     RowBox[{"d", "[", "z", "]"}], " ", 
     RowBox[{"(", 
      RowBox[{"y", "+", "z"}], ")"}]}]}], ",", 
   RowBox[{"{", 
    RowBox[{"a", ",", "d"}], "}"}], ",", 
   RowBox[{"IsolateNames", "\[Rule]", "fF"}]}], "]"}]], "Input",
 CellTags->"Isolate",
 CellLabel->"In[1026]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    TagBox[
     RowBox[{"fF", "(", "129", ")"}],
     HoldForm], " ", 
    RowBox[{"a", "(", "z", ")"}]}], "+", 
   RowBox[{
    TagBox[
     RowBox[{"fF", "(", "128", ")"}],
     HoldForm], " ", 
    RowBox[{"d", "(", "z", ")"}]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009703056386766`*^9},
 CellTags->"Isolate",
 CellLabel->"Out[1026]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"??", "fF"}]], "Input",
 CellTags->"Isolate",
 CellLabel->"In[1027]:="],

Cell["Global`fF", "Print", "PrintUsage",
 CellChangeTimes->{3.50097030651427*^9},
 CellTags->{"Info3500973905-4982097", "Isolate"}],

Cell[BoxData[
 InterpretationBox[GridBox[{
    {GridBox[{
       {
        RowBox[{
         RowBox[{"fF", "[", "128", "]"}], "=", 
         RowBox[{"y", "+", "z"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"fF", "[", "129", "]"}], "=", 
         RowBox[{"b", "+", 
          RowBox[{"c", " ", 
           RowBox[{"\<\"HoldForm\"\>", "[", 
            RowBox[{"fF", "[", "128", "]"}], "]"}]}]}]}]}
      },
      BaselinePosition->{Baseline, {1, 1}},
      GridBoxAlignment->{
       "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
        "RowsIndexed" -> {}},
      GridBoxItemSize->{"Columns" -> {{
           Scaled[0.999]}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, 
        "RowsIndexed" -> {}}]}
   },
   BaselinePosition->{Baseline, {1, 1}},
   GridBoxAlignment->{
    "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
     "RowsIndexed" -> {}}],
  Definition[$CellContext`fF],
  Editable->False]], "Print",
 CellChangeTimes->{3.500970306606867*^9},
 CellTags->{"Info3500973905-4982097", "Isolate"}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Isolate", "[", 
  RowBox[{
   RowBox[{"a", "-", "b", "-", "c", "-", "d", "-", "e"}], ",", 
   RowBox[{"IsolateNames", "\[Rule]", "L"}], ",", 
   RowBox[{"IsolateSplit", "\[Rule]", "15"}]}], "]"}]], "Input",
 CellTags->"Isolate",
 CellLabel->"In[1028]:="],

Cell[BoxData[
 FormBox[
  TagBox[
   RowBox[{"L", "(", "131", ")"}],
   HoldForm], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970307152767*^9},
 CellTags->"Isolate",
 CellLabel->"Out[1028]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"L", "[", "2", "]"}], ",", 
   RowBox[{"L", "[", "1", "]"}]}], "}"}]], "Input",
 CellTags->"Isolate",
 CellLabel->"In[1029]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"L", "(", "2", ")"}], ",", 
    RowBox[{"L", "(", "1", ")"}]}], "}"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970307195737*^9},
 CellTags->"Isolate",
 CellLabel->"Out[1029]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"Clear", "[", 
  RowBox[{"t0", ",", "t1", ",", "L", ",", "fF"}], "]"}]], "Input",
 CellTags->"Isolate",
 CellLabel->"In[1030]:="]
}, Open  ]],

Cell["IsolateHead", "ObjectName",
 CellTags->"IsolateHead"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"IsolateHead"],

Cell["IsolateHead is equivalent to IsolateNames.", "Text",
 CellTags->"IsolateHead"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["IsolateNames",
  BaseStyle->"AddOnsLink",
  ButtonData:>"IsolateNames",
  ButtonNote->"IsolateNames"],
 "."
}], "Text",
 CellTags->"IsolateHead"]
}, Open  ]],

Cell["IsolateNames", "ObjectName",
 CellTags->"IsolateNames"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"IsolateNames"],

Cell["\<\
IsolateNames is an option for Isolate and Collect2. Its default setting is \
KK. Instead of a symbol the setting may also be a list with the names of the \
abbrevations.\
\>", "Text",
 CellTags->"IsolateNames"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Isolate",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Isolate",
  ButtonNote->"Isolate"],
 ", ",
 ButtonBox["Collect2",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Collect2",
  ButtonNote->"Collect2"],
 "."
}], "Text",
 CellTags->"IsolateNames"]
}, Open  ]],

Cell["IsolatePrint", "ObjectName",
 CellTags->"IsolatePrint"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"IsolatePrint"],

Cell["\<\
IsolatePrint is an option of Isolate. If it is set to OutputForm (or any \
other *Form) the definitions of the abbreviations are printed during the \
operation of Isolate. The setting IsolatePrint \[Rule] False suppresses \
printing.\
\>", "Text",
 CellTags->"IsolatePrint"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Isolate",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Isolate",
  ButtonNote->"Isolate"],
 "."
}], "Text",
 CellTags->"IsolatePrint"]
}, Open  ]],

Cell["IsolateSplit", "ObjectName",
 CellTags->"IsolateSplit"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"IsolateSplit"],

Cell["\<\
IsolateSplit is an option for Isolate. Its setting determines the maximum \
number of characters of FortranForm[expr] which are abbreviated by Isolate. \
If the expression is larger than the indicated number, it is split into \
smaller pieces and onto each subsum Isolate is applied. With the default \
setting IsolateSplit \[Rule] Infinity no splitting is done.\
\>", "Text",
 CellTags->"IsolateSplit"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Isolate",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Isolate",
  ButtonNote->"Isolate"],
 "."
}], "Text",
 CellTags->"IsolateSplit"]
}, Open  ]],

Cell["KeepOnly", "ObjectName",
 CellTags->"KeepOnly"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"KeepOnly"],

Cell["\<\
KeepOnly is an option of OneLoop. It may be set to B0, C0, D0 keeping only \
the corresponding coefficients. The default setting is False. If KeepOnly is \
set to {} then the part of the amplitude which is not coefficient of B0, C0, \
D0 is kept.\
\>", "Text",
 CellTags->"KeepOnly"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["OneLoop",
  BaseStyle->"AddOnsLink",
  ButtonData:>"OneLoop",
  ButtonNote->"OneLoop"],
 ", ",
 ButtonBox["B0",
  BaseStyle->"AddOnsLink",
  ButtonData:>"B0",
  ButtonNote->"B0"],
 ", ",
 ButtonBox["C0",
  BaseStyle->"AddOnsLink",
  ButtonData:>"C0",
  ButtonNote->"C0"],
 ", ",
 ButtonBox["D0",
  BaseStyle->"AddOnsLink",
  ButtonData:>"D0",
  ButtonNote->"D0"],
 "."
}], "Text",
 CellTags->"KeepOnly"]
}, Open  ]],

Cell["KK", "ObjectName",
 CellTags->"KK"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"KK"],

Cell["\<\
KK[i] is the default setting of IsolateNames, which is the head of \
abbreviations used by Isolate. A KK[i] returned by Isolate is given in \
HoldForm and can be recovered by ReleaseHold[KK[i]].\
\>", "Text",
 CellTags->"KK"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Isolate",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Isolate",
  ButtonNote->"Isolate"],
 ", ",
 ButtonBox["IsolateNames",
  BaseStyle->"AddOnsLink",
  ButtonData:>"IsolateNames",
  ButtonNote->"IsolateNames"],
 "."
}], "Text",
 CellTags->"KK"]
}, Open  ]],

Cell["Kummer", "ObjectName",
 CellTags->"Kummer"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Kummer"],

Cell["\<\
Kummer[i][exp] applies Kummer relation number i (i =1, ... 24, 94,95,96) to \
all Hypergeometric2F1 in exp. i = 94 corresponds to eq. 9.131.2, i = 95 to \
eq. 9.132.1 and i = 96 to eq. 9.132.2 in Gradsteyn & Ryzhik.\
\>", "Text",
 CellTags->"Kummer"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["HypergeometricAC",
  BaseStyle->"AddOnsLink",
  ButtonData:>"HypergeometricAC",
  ButtonNote->"HypergeometricAC"],
 "."
}], "Text",
 CellTags->"Kummer"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Kummer"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Hypergeometric2F1", "[", 
   RowBox[{"a", ",", "b", ",", "c", ",", "z"}], "]"}], "==", 
  RowBox[{
   RowBox[{"Kummer", "[", "2", "]"}], "[", 
   RowBox[{"Hypergeometric2F1", "[", 
    RowBox[{"a", ",", "b", ",", "c", ",", "z"}], "]"}], "]"}]}]], "Input",
 CellTags->"Kummer",
 CellLabel->"In[1031]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TemplateBox[{"a","b","c","z"},
    "Hypergeometric2F1"], "\[LongEqual]", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "-", "z"}], ")"}], 
     RowBox[{
      RowBox[{"-", "a"}], "-", "b", "+", "c"}]], " ", 
    TemplateBox[{RowBox[{"c", "-", "a"}],RowBox[{"c", "-", "b"}],"c","z"},
     "Hypergeometric2F1"]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970307311799*^9},
 CellTags->"Kummer",
 CellLabel->"Out[1031]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Hypergeometric2F1", "[", 
   RowBox[{"a", ",", "b", ",", "c", ",", "z"}], "]"}], "==", 
  RowBox[{
   RowBox[{"Kummer", "[", "3", "]"}], "[", 
   RowBox[{"Hypergeometric2F1", "[", 
    RowBox[{"a", ",", "b", ",", "c", ",", "z"}], "]"}], "]"}]}]], "Input",
 CellTags->"Kummer",
 CellLabel->"In[1032]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TemplateBox[{"a","b","c","z"},
    "Hypergeometric2F1"], "\[LongEqual]", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "-", "z"}], ")"}], 
     RowBox[{"-", "a"}]], " ", 
    TemplateBox[{"a",RowBox[{"c", "-", "b"}],"c",RowBox[{"-", 
        FractionBox["z", 
         RowBox[{"1", "-", "z"}]]}]},
     "Hypergeometric2F1"]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970307397079*^9},
 CellTags->"Kummer",
 CellLabel->"Out[1032]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Hypergeometric2F1", "[", 
   RowBox[{"a", ",", "b", ",", "c", ",", "z"}], "]"}], "==", 
  RowBox[{
   RowBox[{"Kummer", "[", "4", "]"}], "[", 
   RowBox[{"Hypergeometric2F1", "[", 
    RowBox[{"a", ",", "b", ",", "c", ",", "z"}], "]"}], "]"}]}]], "Input",
 CellTags->"Kummer",
 CellLabel->"In[1033]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TemplateBox[{"a","b","c","z"},
    "Hypergeometric2F1"], "\[LongEqual]", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "-", "z"}], ")"}], 
     RowBox[{"-", "b"}]], " ", 
    TemplateBox[{"b",RowBox[{"c", "-", "a"}],"c",RowBox[{"-", 
        FractionBox["z", 
         RowBox[{"1", "-", "z"}]]}]},
     "Hypergeometric2F1"]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970307505772*^9},
 CellTags->"Kummer",
 CellLabel->"Out[1033]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Hypergeometric2F1", "[", 
   RowBox[{"a", ",", "b", ",", "c", ",", 
    RowBox[{"1", "-", "z"}]}], "]"}], "==", 
  RowBox[{
   RowBox[{"Kummer", "[", "6", "]"}], "[", 
   RowBox[{"Hypergeometric2F1", "[", 
    RowBox[{"a", ",", "b", ",", "c", ",", 
     RowBox[{"1", "-", "z"}]}], "]"}], "]"}]}]], "Input",
 CellTags->"Kummer",
 CellLabel->"In[1034]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TemplateBox[{"a","b","c",RowBox[{"1", "-", "z"}]},
    "Hypergeometric2F1"], "\[LongEqual]", 
   RowBox[{
    SuperscriptBox["z", 
     RowBox[{
      RowBox[{"-", "a"}], "-", "b", "+", "c"}]], " ", 
    TemplateBox[{
     RowBox[{"c", "-", "a"}],RowBox[{"c", "-", "b"}],"c",
      RowBox[{"1", "-", "z"}]},
     "Hypergeometric2F1"]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970307701119*^9},
 CellTags->"Kummer",
 CellLabel->"Out[1034]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Hypergeometric2F1", "[", 
   RowBox[{"a", ",", "b", ",", 
    RowBox[{"a", "+", "b", "+", "1", "-", "c"}], ",", 
    RowBox[{"1", "-", "z"}]}], "]"}], "==", 
  RowBox[{
   RowBox[{"Kummer", "[", "6", "]"}], "[", 
   RowBox[{"Hypergeometric2F1", "[", 
    RowBox[{"a", ",", "b", ",", 
     RowBox[{"a", "+", "b", "+", "1", "-", "c"}], ",", 
     RowBox[{"1", "-", "z"}]}], "]"}], "]"}]}]], "Input",
 CellTags->"Kummer",
 CellLabel->"In[1035]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TemplateBox[{
    "a","b",RowBox[{"a", "+", "b", "-", "c", "+", "1"}],
     RowBox[{"1", "-", "z"}]},
    "Hypergeometric2F1"], "\[LongEqual]", 
   RowBox[{
    SuperscriptBox["z", 
     RowBox[{"1", "-", "c"}]], " ", 
    TemplateBox[{
     RowBox[{"a", "-", "c", "+", "1"}],RowBox[{"b", "-", "c", "+", "1"}],
      RowBox[{"a", "+", "b", "-", "c", "+", "1"}],RowBox[{"1", "-", "z"}]},
     "Hypergeometric2F1"]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970307848589*^9},
 CellTags->"Kummer",
 CellLabel->"Out[1035]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Hypergeometric2F1", "[", 
   RowBox[{"a", ",", "b", ",", "c", ",", 
    RowBox[{"1", "-", "z"}]}], "]"}], "==", 
  RowBox[{
   RowBox[{"Kummer", "[", "7", "]"}], "[", 
   RowBox[{"Hypergeometric2F1", "[", 
    RowBox[{"a", ",", "b", ",", "c", ",", 
     RowBox[{"1", "-", "z"}]}], "]"}], "]"}]}]], "Input",
 CellTags->"Kummer",
 CellLabel->"In[1036]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TemplateBox[{"a","b","c",RowBox[{"1", "-", "z"}]},
    "Hypergeometric2F1"], "\[LongEqual]", 
   RowBox[{
    SuperscriptBox["z", 
     RowBox[{"-", "a"}]], " ", 
    TemplateBox[{"a",RowBox[{"c", "-", "b"}],"c",RowBox[{"-", 
        FractionBox[
         RowBox[{"1", "-", "z"}], "z"]}]},
     "Hypergeometric2F1"]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970307958918*^9},
 CellTags->"Kummer",
 CellLabel->"Out[1036]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Hypergeometric2F1", "[", 
   RowBox[{"a", ",", "b", ",", 
    RowBox[{"a", "+", "b", "+", "1", "-", "c"}], ",", 
    RowBox[{"1", "-", "z"}]}], "]"}], "==", 
  RowBox[{
   RowBox[{"Kummer", "[", "7", "]"}], "[", 
   RowBox[{"Hypergeometric2F1", "[", 
    RowBox[{"a", ",", "b", ",", 
     RowBox[{"a", "+", "b", "+", "1", "-", "c"}], ",", 
     RowBox[{"1", "-", "z"}]}], "]"}], "]"}]}]], "Input",
 CellTags->"Kummer",
 CellLabel->"In[1037]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TemplateBox[{
    "a","b",RowBox[{"a", "+", "b", "-", "c", "+", "1"}],
     RowBox[{"1", "-", "z"}]},
    "Hypergeometric2F1"], "\[LongEqual]", 
   RowBox[{
    SuperscriptBox["z", 
     RowBox[{"-", "a"}]], " ", 
    TemplateBox[{
     "a",RowBox[{"a", "-", "c", "+", "1"}],
      RowBox[{"a", "+", "b", "-", "c", "+", "1"}],RowBox[{"-", 
        FractionBox[
         RowBox[{"1", "-", "z"}], "z"]}]},
     "Hypergeometric2F1"]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.50097030815277*^9},
 CellTags->"Kummer",
 CellLabel->"Out[1037]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Hypergeometric2F1", "[", 
   RowBox[{"a", ",", "b", ",", "c", ",", 
    RowBox[{"1", "-", "z"}]}], "]"}], "==", 
  RowBox[{
   RowBox[{"Kummer", "[", "8", "]"}], "[", 
   RowBox[{"Hypergeometric2F1", "[", 
    RowBox[{"a", ",", "b", ",", "c", ",", 
     RowBox[{"1", "-", "z"}]}], "]"}], "]"}]}]], "Input",
 CellTags->"Kummer",
 CellLabel->"In[1038]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TemplateBox[{"a","b","c",RowBox[{"1", "-", "z"}]},
    "Hypergeometric2F1"], "\[LongEqual]", 
   RowBox[{
    SuperscriptBox["z", 
     RowBox[{"-", "b"}]], " ", 
    TemplateBox[{"b",RowBox[{"c", "-", "a"}],"c",RowBox[{"-", 
        FractionBox[
         RowBox[{"1", "-", "z"}], "z"]}]},
     "Hypergeometric2F1"]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970308856097*^9},
 CellTags->"Kummer",
 CellLabel->"Out[1038]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Hypergeometric2F1", "[", 
   RowBox[{"a", ",", "b", ",", 
    RowBox[{"a", "+", "b", "+", "1", "-", "c"}], ",", 
    RowBox[{"1", "-", "z"}]}], "]"}], "==", 
  RowBox[{
   RowBox[{"Kummer", "[", "8", "]"}], "[", 
   RowBox[{"Hypergeometric2F1", "[", 
    RowBox[{"a", ",", "b", ",", 
     RowBox[{"a", "+", "b", "+", "1", "-", "c"}], ",", 
     RowBox[{"1", "-", "z"}]}], "]"}], "]"}]}]], "Input",
 CellTags->"Kummer",
 CellLabel->"In[1039]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TemplateBox[{
    "a","b",RowBox[{"a", "+", "b", "-", "c", "+", "1"}],
     RowBox[{"1", "-", "z"}]},
    "Hypergeometric2F1"], "\[LongEqual]", 
   RowBox[{
    SuperscriptBox["z", 
     RowBox[{"-", "b"}]], " ", 
    TemplateBox[{
     "b",RowBox[{"b", "-", "c", "+", "1"}],
      RowBox[{"a", "+", "b", "-", "c", "+", "1"}],RowBox[{"-", 
        FractionBox[
         RowBox[{"1", "-", "z"}], "z"]}]},
     "Hypergeometric2F1"]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009703090806417`*^9},
 CellTags->"Kummer",
 CellLabel->"Out[1039]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Hypergeometric2F1", "[", 
   RowBox[{"a", ",", "b", ",", "c", ",", 
    RowBox[{"z", "^", 
     RowBox[{"(", 
      RowBox[{"-", "1"}], ")"}]}]}], "]"}], "==", 
  RowBox[{
   RowBox[{"Kummer", "[", "10", "]"}], "[", 
   RowBox[{"Hypergeometric2F1", "[", 
    RowBox[{"a", ",", "b", ",", "c", ",", 
     RowBox[{"z", "^", 
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}]}]}], "]"}], "]"}]}]], "Input",
 CellTags->"Kummer",
 CellLabel->"In[1040]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TemplateBox[{"a","b","c",FractionBox["1", "z"]},
    "Hypergeometric2F1"], "\[LongEqual]", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "-", "z"}], ")"}], 
     RowBox[{
      RowBox[{"-", "a"}], "-", "b", "+", "c"}]], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"-", "z"}], ")"}], 
     RowBox[{"a", "+", "b", "-", "c"}]], " ", 
    TemplateBox[{
     RowBox[{"c", "-", "a"}],RowBox[{"c", "-", "b"}],"c",FractionBox[
      "1", "z"]},
     "Hypergeometric2F1"]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970309190786*^9},
 CellTags->"Kummer",
 CellLabel->"Out[1040]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Hypergeometric2F1", "[", 
   RowBox[{"a", ",", 
    RowBox[{"a", "+", "1", "-", "c"}], ",", 
    RowBox[{"a", "+", "1", "-", "b"}], ",", 
    RowBox[{"z", "^", 
     RowBox[{"(", 
      RowBox[{"-", "1"}], ")"}]}]}], "]"}], "==", 
  RowBox[{
   RowBox[{"Kummer", "[", "10", "]"}], "[", 
   RowBox[{"Hypergeometric2F1", "[", 
    RowBox[{"a", ",", 
     RowBox[{"a", "+", "1", "-", "c"}], ",", 
     RowBox[{"a", "+", "1", "-", "b"}], ",", 
     RowBox[{"z", "^", 
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}]}]}], "]"}], "]"}]}]], "Input",
 CellTags->"Kummer",
 CellLabel->"In[1041]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TemplateBox[{
    "a",RowBox[{"a", "-", "c", "+", "1"}],RowBox[{"a", "-", "b", "+", "1"}],
     FractionBox["1", "z"]},
    "Hypergeometric2F1"], "\[LongEqual]", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "-", "z"}], ")"}], 
     RowBox[{
      RowBox[{"-", "a"}], "-", "b", "+", "c"}]], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"-", "z"}], ")"}], 
     RowBox[{"a", "+", "b", "-", "c"}]], " ", 
    TemplateBox[{
     RowBox[{"1", "-", "b"}],RowBox[{"c", "-", "b"}],
      RowBox[{"a", "-", "b", "+", "1"}],FractionBox["1", "z"]},
     "Hypergeometric2F1"]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970309311872*^9},
 CellTags->"Kummer",
 CellLabel->"Out[1041]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Hypergeometric2F1", "[", 
   RowBox[{"a", ",", "b", ",", "c", ",", 
    RowBox[{"z", "^", 
     RowBox[{"(", 
      RowBox[{"-", "1"}], ")"}]}]}], "]"}], "==", 
  RowBox[{
   RowBox[{"Kummer", "[", "11", "]"}], "[", 
   RowBox[{"Hypergeometric2F1", "[", 
    RowBox[{"a", ",", "b", ",", "c", ",", 
     RowBox[{"z", "^", 
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}]}]}], "]"}], "]"}]}]], "Input",
 CellTags->"Kummer",
 CellLabel->"In[1042]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TemplateBox[{"a","b","c",FractionBox["1", "z"]},
    "Hypergeometric2F1"], "\[LongEqual]", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"-", 
       FractionBox[
        RowBox[{"1", "-", "z"}], "z"]}], ")"}], 
     RowBox[{"-", "a"}]], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"-", "z"}], ")"}], "a"], " ", 
    TemplateBox[{"a",RowBox[{"c", "-", "b"}],"c",FractionBox["1", 
       RowBox[{"1", "-", "z"}]]},
     "Hypergeometric2F1"]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009703094216833`*^9},
 CellTags->"Kummer",
 CellLabel->"Out[1042]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Hypergeometric2F1", "[", 
   RowBox[{"a", ",", 
    RowBox[{"a", "+", "1", "-", "c"}], ",", 
    RowBox[{"a", "+", "1", "-", "b"}], ",", 
    RowBox[{"z", "^", 
     RowBox[{"(", 
      RowBox[{"-", "1"}], ")"}]}]}], "]"}], "==", 
  RowBox[{
   RowBox[{"Kummer", "[", "11", "]"}], "[", 
   RowBox[{"Hypergeometric2F1", "[", 
    RowBox[{"a", ",", 
     RowBox[{"a", "+", "1", "-", "c"}], ",", 
     RowBox[{"a", "+", "1", "-", "b"}], ",", 
     RowBox[{"z", "^", 
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}]}]}], "]"}], "]"}]}]], "Input",
 CellTags->"Kummer",
 CellLabel->"In[1043]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TemplateBox[{
    "a",RowBox[{"a", "-", "c", "+", "1"}],RowBox[{"a", "-", "b", "+", "1"}],
     FractionBox["1", "z"]},
    "Hypergeometric2F1"], "\[LongEqual]", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"-", 
       FractionBox[
        RowBox[{"1", "-", "z"}], "z"]}], ")"}], 
     RowBox[{"-", "a"}]], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"-", "z"}], ")"}], "a"], " ", 
    TemplateBox[{
     "a",RowBox[{"c", "-", "b"}],RowBox[{"a", "-", "b", "+", "1"}],
      FractionBox["1", 
       RowBox[{"1", "-", "z"}]]},
     "Hypergeometric2F1"]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970309629614*^9},
 CellTags->"Kummer",
 CellLabel->"Out[1043]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Hypergeometric2F1", "[", 
   RowBox[{"a", ",", "b", ",", "c", ",", 
    RowBox[{"z", "^", 
     RowBox[{"(", 
      RowBox[{"-", "1"}], ")"}]}]}], "]"}], "==", 
  RowBox[{
   RowBox[{"Kummer", "[", "12", "]"}], "[", 
   RowBox[{"Hypergeometric2F1", "[", 
    RowBox[{"a", ",", "b", ",", "c", ",", 
     RowBox[{"z", "^", 
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}]}]}], "]"}], "]"}]}]], "Input",
 CellTags->"Kummer",
 CellLabel->"In[1044]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TemplateBox[{"a","b","c",FractionBox["1", "z"]},
    "Hypergeometric2F1"], "\[LongEqual]", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "-", "z"}], ")"}], 
     RowBox[{"-", "b"}]], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"-", "z"}], ")"}], 
     RowBox[{"b", "-", "a"}]], " ", 
    TemplateBox[{"b",RowBox[{"c", "-", "a"}],"c",FractionBox["1", 
       RowBox[{"1", "-", "z"}]]},
     "Hypergeometric2F1"]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970309734969*^9},
 CellTags->"Kummer",
 CellLabel->"Out[1044]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Hypergeometric2F1", "[", 
   RowBox[{"a", ",", 
    RowBox[{"a", "+", "1", "-", "c"}], ",", 
    RowBox[{"a", "+", "1", "-", "b"}], ",", 
    RowBox[{"z", "^", 
     RowBox[{"(", 
      RowBox[{"-", "1"}], ")"}]}]}], "]"}], "==", 
  RowBox[{
   RowBox[{"Kummer", "[", "12", "]"}], "[", 
   RowBox[{"Hypergeometric2F1", "[", 
    RowBox[{"a", ",", 
     RowBox[{"a", "+", "1", "-", "c"}], ",", 
     RowBox[{"a", "+", "1", "-", "b"}], ",", 
     RowBox[{"z", "^", 
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}]}]}], "]"}], "]"}]}]], "Input",
 CellTags->"Kummer",
 CellLabel->"In[1045]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TemplateBox[{
    "a",RowBox[{"a", "-", "c", "+", "1"}],RowBox[{"a", "-", "b", "+", "1"}],
     FractionBox["1", "z"]},
    "Hypergeometric2F1"], "\[LongEqual]", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"-", "z"}], ")"}], 
     RowBox[{"1", "-", "c"}]], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "-", "z"}], ")"}], 
     RowBox[{
      RowBox[{"-", "a"}], "+", "c", "-", "1"}]], " ", 
    TemplateBox[{
     RowBox[{"1", "-", "b"}],RowBox[{"a", "-", "c", "+", "1"}],
      RowBox[{"a", "-", "b", "+", "1"}],FractionBox["1", 
       RowBox[{"1", "-", "z"}]]},
     "Hypergeometric2F1"]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970309878215*^9},
 CellTags->"Kummer",
 CellLabel->"Out[1045]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Hypergeometric2F1", "[", 
   RowBox[{"a", ",", "b", ",", "c", ",", 
    RowBox[{"z", "^", 
     RowBox[{"(", 
      RowBox[{"-", "1"}], ")"}]}]}], "]"}], "==", 
  RowBox[{
   RowBox[{"Kummer", "[", "14", "]"}], "[", 
   RowBox[{"Hypergeometric2F1", "[", 
    RowBox[{"a", ",", "b", ",", "c", ",", 
     RowBox[{"z", "^", 
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}]}]}], "]"}], "]"}]}]], "Input",
 CellTags->"Kummer",
 CellLabel->"In[1046]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TemplateBox[{"a","b","c",FractionBox["1", "z"]},
    "Hypergeometric2F1"], "\[LongEqual]", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "-", "z"}], ")"}], 
     RowBox[{
      RowBox[{"-", "a"}], "-", "b", "+", "c"}]], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"-", "z"}], ")"}], 
     RowBox[{"a", "+", "b", "-", "c"}]], " ", 
    TemplateBox[{
     RowBox[{"c", "-", "a"}],RowBox[{"c", "-", "b"}],"c",FractionBox[
      "1", "z"]},
     "Hypergeometric2F1"]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970310002418*^9},
 CellTags->"Kummer",
 CellLabel->"Out[1046]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Hypergeometric2F1", "[", 
   RowBox[{
    RowBox[{"b", "+", "1", "-", "c"}], ",", "b", ",", 
    RowBox[{"b", "+", "1", "-", "a"}], ",", 
    RowBox[{"z", "^", 
     RowBox[{"(", 
      RowBox[{"-", "1"}], ")"}]}]}], "]"}], "==", 
  RowBox[{
   RowBox[{"Kummer", "[", "14", "]"}], "[", 
   RowBox[{"Hypergeometric2F1", "[", 
    RowBox[{
     RowBox[{"b", "+", "1", "-", "c"}], ",", "b", ",", 
     RowBox[{"b", "+", "1", "-", "a"}], ",", 
     RowBox[{"z", "^", 
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}]}]}], "]"}], "]"}]}]], "Input",
 CellTags->"Kummer",
 CellLabel->"In[1047]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TemplateBox[{"b",RowBox[{"b", "-", "c", "+", "1"}],RowBox[{
       RowBox[{"-", "a"}], "+", "b", "+", "1"}],FractionBox["1", "z"]},
    "Hypergeometric2F1"], "\[LongEqual]", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "-", "z"}], ")"}], 
     RowBox[{
      RowBox[{"-", "a"}], "-", "b", "+", "c"}]], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"-", "z"}], ")"}], 
     RowBox[{"a", "+", "b", "-", "c"}]], " ", 
    TemplateBox[{RowBox[{"1", "-", "a"}],RowBox[{"c", "-", "a"}],RowBox[{
        RowBox[{"-", "a"}], "+", "b", "+", "1"}],FractionBox["1", "z"]},
     "Hypergeometric2F1"]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970310171074*^9},
 CellTags->"Kummer",
 CellLabel->"Out[1047]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Hypergeometric2F1", "[", 
   RowBox[{"a", ",", "b", ",", "c", ",", 
    RowBox[{"z", "^", 
     RowBox[{"(", 
      RowBox[{"-", "1"}], ")"}]}]}], "]"}], "==", 
  RowBox[{
   RowBox[{"Kummer", "[", "15", "]"}], "[", 
   RowBox[{"Hypergeometric2F1", "[", 
    RowBox[{"a", ",", "b", ",", "c", ",", 
     RowBox[{"z", "^", 
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}]}]}], "]"}], "]"}]}]], "Input",
 CellTags->"Kummer",
 CellLabel->"In[1048]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TemplateBox[{"a","b","c",FractionBox["1", "z"]},
    "Hypergeometric2F1"], "\[LongEqual]", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "-", "z"}], ")"}], 
     RowBox[{"-", "b"}]], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"-", "z"}], ")"}], "b"], " ", 
    TemplateBox[{"b",RowBox[{"c", "-", "a"}],"c",FractionBox["1", 
       RowBox[{"1", "-", "z"}]]},
     "Hypergeometric2F1"]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970310326522*^9},
 CellTags->"Kummer",
 CellLabel->"Out[1048]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Hypergeometric2F1", "[", 
   RowBox[{
    RowBox[{"b", "+", "1", "-", "c"}], ",", "b", ",", 
    RowBox[{"b", "+", "1", "-", "a"}], ",", 
    RowBox[{"z", "^", 
     RowBox[{"(", 
      RowBox[{"-", "1"}], ")"}]}]}], "]"}], "==", 
  RowBox[{
   RowBox[{"Kummer", "[", "15", "]"}], "[", 
   RowBox[{"Hypergeometric2F1", "[", 
    RowBox[{
     RowBox[{"b", "+", "1", "-", "c"}], ",", "b", ",", 
     RowBox[{"b", "+", "1", "-", "a"}], ",", 
     RowBox[{"z", "^", 
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}]}]}], "]"}], "]"}]}]], "Input",
 CellTags->"Kummer",
 CellLabel->"In[1049]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TemplateBox[{"b",RowBox[{"b", "-", "c", "+", "1"}],RowBox[{
       RowBox[{"-", "a"}], "+", "b", "+", "1"}],FractionBox["1", "z"]},
    "Hypergeometric2F1"], "\[LongEqual]", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "-", "z"}], ")"}], 
     RowBox[{
      RowBox[{"-", "b"}], "+", "c", "-", "1"}]], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"-", "z"}], ")"}], 
     RowBox[{"b", "-", "c", "+", "1"}]], " ", 
    TemplateBox[{
     RowBox[{"1", "-", "a"}],RowBox[{"b", "-", "c", "+", "1"}],RowBox[{
        RowBox[{"-", "a"}], "+", "b", "+", "1"}],FractionBox["1", 
       RowBox[{"1", "-", "z"}]]},
     "Hypergeometric2F1"]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009703104541273`*^9},
 CellTags->"Kummer",
 CellLabel->"Out[1049]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Hypergeometric2F1", "[", 
   RowBox[{"a", ",", "b", ",", "c", ",", 
    RowBox[{"z", "^", 
     RowBox[{"(", 
      RowBox[{"-", "1"}], ")"}]}]}], "]"}], "==", 
  RowBox[{
   RowBox[{"Kummer", "[", "16", "]"}], "[", 
   RowBox[{"Hypergeometric2F1", "[", 
    RowBox[{"a", ",", "b", ",", "c", ",", 
     RowBox[{"z", "^", 
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}]}]}], "]"}], "]"}]}]], "Input",
 CellTags->"Kummer",
 CellLabel->"In[1050]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TemplateBox[{"a","b","c",FractionBox["1", "z"]},
    "Hypergeometric2F1"], "\[LongEqual]", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "-", "z"}], ")"}], 
     RowBox[{"-", "a"}]], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"-", "z"}], ")"}], "a"], " ", 
    TemplateBox[{"a",RowBox[{"c", "-", "b"}],"c",FractionBox["1", 
       RowBox[{"1", "-", "z"}]]},
     "Hypergeometric2F1"]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970310547176*^9},
 CellTags->"Kummer",
 CellLabel->"Out[1050]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Hypergeometric2F1", "[", 
   RowBox[{
    RowBox[{"b", "+", "1", "-", "c"}], ",", "b", ",", 
    RowBox[{"b", "+", "1", "-", "a"}], ",", 
    RowBox[{"z", "^", 
     RowBox[{"(", 
      RowBox[{"-", "1"}], ")"}]}]}], "]"}], "==", 
  RowBox[{
   RowBox[{"Kummer", "[", "16", "]"}], "[", 
   RowBox[{"Hypergeometric2F1", "[", 
    RowBox[{
     RowBox[{"b", "+", "1", "-", "c"}], ",", "b", ",", 
     RowBox[{"b", "+", "1", "-", "a"}], ",", 
     RowBox[{"z", "^", 
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}]}]}], "]"}], "]"}]}]], "Input",
 CellTags->"Kummer",
 CellLabel->"In[1051]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TemplateBox[{"b",RowBox[{"b", "-", "c", "+", "1"}],RowBox[{
       RowBox[{"-", "a"}], "+", "b", "+", "1"}],FractionBox["1", "z"]},
    "Hypergeometric2F1"], "\[LongEqual]", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "-", "z"}], ")"}], 
     RowBox[{"-", "b"}]], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"-", "z"}], ")"}], "b"], " ", 
    TemplateBox[{"b",RowBox[{"c", "-", "a"}],RowBox[{
        RowBox[{"-", "a"}], "+", "b", "+", "1"}],FractionBox["1", 
       RowBox[{"1", "-", "z"}]]},
     "Hypergeometric2F1"]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970310674469*^9},
 CellTags->"Kummer",
 CellLabel->"Out[1051]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Hypergeometric2F1", "[", 
   RowBox[{
    RowBox[{"a", "+", "1", "-", "c"}], ",", 
    RowBox[{"b", "+", "1", "-", "c"}], ",", 
    RowBox[{"2", "-", "c"}], ",", "z"}], "]"}], "==", 
  RowBox[{
   RowBox[{"Kummer", "[", "18", "]"}], "[", 
   RowBox[{"Hypergeometric2F1", "[", 
    RowBox[{
     RowBox[{"a", "+", "1", "-", "c"}], ",", 
     RowBox[{"b", "+", "1", "-", "c"}], ",", 
     RowBox[{"2", "-", "c"}], ",", "z"}], "]"}], "]"}]}]], "Input",
 CellTags->"Kummer",
 CellLabel->"In[1052]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TemplateBox[{
    RowBox[{"a", "-", "c", "+", "1"}],RowBox[{"b", "-", "c", "+", "1"}],
     RowBox[{"2", "-", "c"}],"z"},
    "Hypergeometric2F1"], "\[LongEqual]", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "-", "z"}], ")"}], 
     RowBox[{
      RowBox[{"-", "a"}], "-", "b", "+", "c"}]], " ", 
    TemplateBox[{
     RowBox[{"1", "-", "a"}],RowBox[{"1", "-", "b"}],RowBox[{"2", "-", "c"}],
      "z"},
     "Hypergeometric2F1"]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009703108672523`*^9},
 CellTags->"Kummer",
 CellLabel->"Out[1052]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Hypergeometric2F1", "[", 
   RowBox[{"a", ",", "b", ",", "c", ",", "z"}], "]"}], "==", 
  RowBox[{
   RowBox[{"Kummer", "[", "18", "]"}], "[", 
   RowBox[{"Hypergeometric2F1", "[", 
    RowBox[{"a", ",", "b", ",", "c", ",", "z"}], "]"}], "]"}]}]], "Input",
 CellTags->"Kummer",
 CellLabel->"In[1053]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TemplateBox[{"a","b","c","z"},
    "Hypergeometric2F1"], "\[LongEqual]", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "-", "z"}], ")"}], 
     RowBox[{
      RowBox[{"-", "a"}], "-", "b", "+", "c"}]], " ", 
    TemplateBox[{RowBox[{"c", "-", "a"}],RowBox[{"c", "-", "b"}],"c","z"},
     "Hypergeometric2F1"]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970310975625*^9},
 CellTags->"Kummer",
 CellLabel->"Out[1053]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Hypergeometric2F1", "[", 
   RowBox[{
    RowBox[{"a", "+", "1", "-", "c"}], ",", 
    RowBox[{"b", "+", "1", "-", "c"}], ",", 
    RowBox[{"2", "-", "c"}], ",", "z"}], "]"}], "==", 
  RowBox[{
   RowBox[{"Kummer", "[", "19", "]"}], "[", 
   RowBox[{"Hypergeometric2F1", "[", 
    RowBox[{
     RowBox[{"a", "+", "1", "-", "c"}], ",", 
     RowBox[{"b", "+", "1", "-", "c"}], ",", 
     RowBox[{"2", "-", "c"}], ",", "z"}], "]"}], "]"}]}]], "Input",
 CellTags->"Kummer",
 CellLabel->"In[1054]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TemplateBox[{
    RowBox[{"a", "-", "c", "+", "1"}],RowBox[{"b", "-", "c", "+", "1"}],
     RowBox[{"2", "-", "c"}],"z"},
    "Hypergeometric2F1"], "\[LongEqual]", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "-", "z"}], ")"}], 
     RowBox[{
      RowBox[{"-", "a"}], "+", "c", "-", "1"}]], " ", 
    TemplateBox[{
     RowBox[{"1", "-", "b"}],RowBox[{"a", "-", "c", "+", "1"}],
      RowBox[{"2", "-", "c"}],FractionBox["z", 
       RowBox[{"z", "-", "1"}]]},
     "Hypergeometric2F1"]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009703111507807`*^9},
 CellTags->"Kummer",
 CellLabel->"Out[1054]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Hypergeometric2F1", "[", 
   RowBox[{"a", ",", "b", ",", "c", ",", "z"}], "]"}], "==", 
  RowBox[{
   RowBox[{"Kummer", "[", "19", "]"}], "[", 
   RowBox[{"Hypergeometric2F1", "[", 
    RowBox[{"a", ",", "b", ",", "c", ",", "z"}], "]"}], "]"}]}]], "Input",
 CellTags->"Kummer",
 CellLabel->"In[1055]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TemplateBox[{"a","b","c","z"},
    "Hypergeometric2F1"], "\[LongEqual]", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "-", "z"}], ")"}], 
     RowBox[{"-", "a"}]], " ", 
    TemplateBox[{"a",RowBox[{"c", "-", "b"}],"c",FractionBox["z", 
       RowBox[{"z", "-", "1"}]]},
     "Hypergeometric2F1"]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970311271311*^9},
 CellTags->"Kummer",
 CellLabel->"Out[1055]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Hypergeometric2F1", "[", 
   RowBox[{
    RowBox[{"a", "+", "1", "-", "c"}], ",", 
    RowBox[{"b", "+", "1", "-", "c"}], ",", 
    RowBox[{"2", "-", "c"}], ",", "z"}], "]"}], "==", 
  RowBox[{
   RowBox[{"Kummer", "[", "20", "]"}], "[", 
   RowBox[{"Hypergeometric2F1", "[", 
    RowBox[{
     RowBox[{"a", "+", "1", "-", "c"}], ",", 
     RowBox[{"b", "+", "1", "-", "c"}], ",", 
     RowBox[{"2", "-", "c"}], ",", "z"}], "]"}], "]"}]}]], "Input",
 CellTags->"Kummer",
 CellLabel->"In[1056]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TemplateBox[{
    RowBox[{"a", "-", "c", "+", "1"}],RowBox[{"b", "-", "c", "+", "1"}],
     RowBox[{"2", "-", "c"}],"z"},
    "Hypergeometric2F1"], "\[LongEqual]", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "-", "z"}], ")"}], 
     RowBox[{
      RowBox[{"-", "b"}], "+", "c", "-", "1"}]], " ", 
    TemplateBox[{
     RowBox[{"1", "-", "a"}],RowBox[{"b", "-", "c", "+", "1"}],
      RowBox[{"2", "-", "c"}],FractionBox["z", 
       RowBox[{"z", "-", "1"}]]},
     "Hypergeometric2F1"]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970311496237*^9},
 CellTags->"Kummer",
 CellLabel->"Out[1056]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Hypergeometric2F1", "[", 
   RowBox[{"a", ",", "b", ",", "c", ",", "z"}], "]"}], "==", 
  RowBox[{
   RowBox[{"Kummer", "[", "20", "]"}], "[", 
   RowBox[{"Hypergeometric2F1", "[", 
    RowBox[{"a", ",", "b", ",", "c", ",", "z"}], "]"}], "]"}]}]], "Input",
 CellTags->"Kummer",
 CellLabel->"In[1057]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TemplateBox[{"a","b","c","z"},
    "Hypergeometric2F1"], "\[LongEqual]", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "-", "z"}], ")"}], 
     RowBox[{"-", "b"}]], " ", 
    TemplateBox[{"b",RowBox[{"c", "-", "a"}],"c",FractionBox["z", 
       RowBox[{"z", "-", "1"}]]},
     "Hypergeometric2F1"]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.50097031171789*^9},
 CellTags->"Kummer",
 CellLabel->"Out[1057]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Hypergeometric2F1", "[", 
   RowBox[{
    RowBox[{"c", "-", "a"}], ",", 
    RowBox[{"c", "-", "b"}], ",", 
    RowBox[{"c", "+", "1", "-", "a", "-", "b"}], ",", 
    RowBox[{"1", "-", "z"}]}], "]"}], "==", 
  RowBox[{
   RowBox[{"Kummer", "[", "22", "]"}], "[", 
   RowBox[{"Hypergeometric2F1", "[", 
    RowBox[{
     RowBox[{"c", "-", "a"}], ",", 
     RowBox[{"c", "-", "b"}], ",", 
     RowBox[{"c", "+", "1", "-", "a", "-", "b"}], ",", 
     RowBox[{"1", "-", "z"}]}], "]"}], "]"}]}]], "Input",
 CellTags->"Kummer",
 CellLabel->"In[1058]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TemplateBox[{RowBox[{"c", "-", "a"}],RowBox[{"c", "-", "b"}],RowBox[{
       RowBox[{"-", "a"}], "-", "b", "+", "c", "+", "1"}],
     RowBox[{"1", "-", "z"}]},
    "Hypergeometric2F1"], "\[LongEqual]", 
   RowBox[{
    SuperscriptBox["z", 
     RowBox[{"1", "-", "c"}]], " ", 
    TemplateBox[{RowBox[{"1", "-", "a"}],RowBox[{"1", "-", "b"}],RowBox[{
        RowBox[{"-", "a"}], "-", "b", "+", "c", "+", "1"}],
      RowBox[{"1", "-", "z"}]},
     "Hypergeometric2F1"]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009703119752274`*^9},
 CellTags->"Kummer",
 CellLabel->"Out[1058]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Hypergeometric2F1", "[", 
   RowBox[{"a", ",", "b", ",", "c", ",", 
    RowBox[{"1", "-", "z"}]}], "]"}], "==", 
  RowBox[{
   RowBox[{"Kummer", "[", "22", "]"}], "[", 
   RowBox[{"Hypergeometric2F1", "[", 
    RowBox[{"a", ",", "b", ",", "c", ",", 
     RowBox[{"1", "-", "z"}]}], "]"}], "]"}]}]], "Input",
 CellTags->"Kummer",
 CellLabel->"In[1059]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TemplateBox[{"a","b","c",RowBox[{"1", "-", "z"}]},
    "Hypergeometric2F1"], "\[LongEqual]", 
   RowBox[{
    SuperscriptBox["z", 
     RowBox[{
      RowBox[{"-", "a"}], "-", "b", "+", "c"}]], " ", 
    TemplateBox[{
     RowBox[{"c", "-", "a"}],RowBox[{"c", "-", "b"}],"c",
      RowBox[{"1", "-", "z"}]},
     "Hypergeometric2F1"]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970312148159*^9},
 CellTags->"Kummer",
 CellLabel->"Out[1059]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Hypergeometric2F1", "[", 
   RowBox[{
    RowBox[{"c", "-", "a"}], ",", 
    RowBox[{"c", "-", "b"}], ",", 
    RowBox[{"c", "+", "1", "-", "a", "-", "b"}], ",", 
    RowBox[{"1", "-", "z"}]}], "]"}], "==", 
  RowBox[{
   RowBox[{"Kummer", "[", "23", "]"}], "[", 
   RowBox[{"Hypergeometric2F1", "[", 
    RowBox[{
     RowBox[{"c", "-", "a"}], ",", 
     RowBox[{"c", "-", "b"}], ",", 
     RowBox[{"c", "+", "1", "-", "a", "-", "b"}], ",", 
     RowBox[{"1", "-", "z"}]}], "]"}], "]"}]}]], "Input",
 CellTags->"Kummer",
 CellLabel->"In[1060]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TemplateBox[{RowBox[{"c", "-", "a"}],RowBox[{"c", "-", "b"}],RowBox[{
       RowBox[{"-", "a"}], "-", "b", "+", "c", "+", "1"}],
     RowBox[{"1", "-", "z"}]},
    "Hypergeometric2F1"], "\[LongEqual]", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "-", "z"}], ")"}], 
     RowBox[{"a", "-", "c"}]], " ", 
    TemplateBox[{RowBox[{"1", "-", "a"}],RowBox[{"c", "-", "a"}],RowBox[{
        RowBox[{"-", "a"}], "-", "b", "+", "c", "+", "1"}],RowBox[{"1", "-", 
        FractionBox["1", 
         RowBox[{"1", "-", "z"}]]}]},
     "Hypergeometric2F1"]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.50097031229464*^9},
 CellTags->"Kummer",
 CellLabel->"Out[1060]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Hypergeometric2F1", "[", 
   RowBox[{"a", ",", "b", ",", "c", ",", 
    RowBox[{"1", "-", "z"}]}], "]"}], "==", 
  RowBox[{
   RowBox[{"Kummer", "[", "23", "]"}], "[", 
   RowBox[{"Hypergeometric2F1", "[", 
    RowBox[{"a", ",", "b", ",", "c", ",", 
     RowBox[{"1", "-", "z"}]}], "]"}], "]"}]}]], "Input",
 CellTags->"Kummer",
 CellLabel->"In[1061]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TemplateBox[{"a","b","c",RowBox[{"1", "-", "z"}]},
    "Hypergeometric2F1"], "\[LongEqual]", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "-", "z"}], ")"}], 
     RowBox[{"-", "a"}]], " ", 
    TemplateBox[{"a",RowBox[{"c", "-", "b"}],"c",RowBox[{"1", "-", 
        FractionBox["1", 
         RowBox[{"1", "-", "z"}]]}]},
     "Hypergeometric2F1"]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.50097031237859*^9},
 CellTags->"Kummer",
 CellLabel->"Out[1061]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Hypergeometric2F1", "[", 
   RowBox[{
    RowBox[{"c", "-", "a"}], ",", 
    RowBox[{"c", "-", "b"}], ",", 
    RowBox[{"c", "+", "1", "-", "a", "-", "b"}], ",", 
    RowBox[{"1", "-", "z"}]}], "]"}], "==", 
  RowBox[{
   RowBox[{"Kummer", "[", "24", "]"}], "[", 
   RowBox[{"Hypergeometric2F1", "[", 
    RowBox[{
     RowBox[{"c", "-", "a"}], ",", 
     RowBox[{"c", "-", "b"}], ",", 
     RowBox[{"c", "+", "1", "-", "a", "-", "b"}], ",", 
     RowBox[{"1", "-", "z"}]}], "]"}], "]"}]}]], "Input",
 CellTags->"Kummer",
 CellLabel->"In[1062]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TemplateBox[{RowBox[{"c", "-", "a"}],RowBox[{"c", "-", "b"}],RowBox[{
       RowBox[{"-", "a"}], "-", "b", "+", "c", "+", "1"}],
     RowBox[{"1", "-", "z"}]},
    "Hypergeometric2F1"], "\[LongEqual]", 
   RowBox[{
    SuperscriptBox["z", 
     RowBox[{"b", "-", "c"}]], " ", 
    TemplateBox[{RowBox[{"1", "-", "b"}],RowBox[{"c", "-", "b"}],RowBox[{
        RowBox[{"-", "a"}], "-", "b", "+", "c", "+", "1"}],RowBox[{"-", 
        FractionBox[
         RowBox[{"1", "-", "z"}], "z"]}]},
     "Hypergeometric2F1"]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970312524234*^9},
 CellTags->"Kummer",
 CellLabel->"Out[1062]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Hypergeometric2F1", "[", 
   RowBox[{"a", ",", "b", ",", "c", ",", 
    RowBox[{"1", "-", "z"}]}], "]"}], "==", 
  RowBox[{
   RowBox[{"Kummer", "[", "24", "]"}], "[", 
   RowBox[{"Hypergeometric2F1", "[", 
    RowBox[{"a", ",", "b", ",", "c", ",", 
     RowBox[{"1", "-", "z"}]}], "]"}], "]"}]}]], "Input",
 CellTags->"Kummer",
 CellLabel->"In[1063]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TemplateBox[{"a","b","c",RowBox[{"1", "-", "z"}]},
    "Hypergeometric2F1"], "\[LongEqual]", 
   RowBox[{
    SuperscriptBox["z", 
     RowBox[{"-", "b"}]], " ", 
    TemplateBox[{"b",RowBox[{"c", "-", "a"}],"c",RowBox[{"-", 
        FractionBox[
         RowBox[{"1", "-", "z"}], "z"]}]},
     "Hypergeometric2F1"]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009703127556467`*^9},
 CellTags->"Kummer",
 CellLabel->"Out[1063]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Hypergeometric2F1", "[", 
   RowBox[{"a", ",", "b", ",", "c", ",", "z"}], "]"}], "==", 
  RowBox[{
   RowBox[{"Kummer", "[", "94", "]"}], "[", 
   RowBox[{"Hypergeometric2F1", "[", 
    RowBox[{"a", ",", "b", ",", "c", ",", "z"}], "]"}], "]"}]}]], "Input",
 CellTags->"Kummer",
 CellLabel->"In[1064]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TemplateBox[{"a","b","c","z"},
    "Hypergeometric2F1"], "\[LongEqual]", 
   RowBox[{
    FractionBox[
     RowBox[{
      TemplateBox[{"c"},
       "Gamma"], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"1", "-", "z"}], ")"}], 
       RowBox[{
        RowBox[{"-", "a"}], "-", "b", "+", "c"}]], " ", 
      TemplateBox[{RowBox[{"a", "+", "b", "-", "c"}]},
       "Gamma"], " ", 
      TemplateBox[{RowBox[{"c", "-", "a"}],RowBox[{"c", "-", "b"}],RowBox[{
          RowBox[{"-", "a"}], "-", "b", "+", "c", "+", "1"}],
        RowBox[{"1", "-", "z"}]},
       "Hypergeometric2F1"]}], 
     RowBox[{
      TemplateBox[{"a"},
       "Gamma"], " ", 
      TemplateBox[{"b"},
       "Gamma"]}]], "+", 
    FractionBox[
     RowBox[{
      TemplateBox[{"c"},
       "Gamma"], " ", 
      TemplateBox[{RowBox[{
          RowBox[{"-", "a"}], "-", "b", "+", "c"}]},
       "Gamma"], " ", 
      TemplateBox[{
       "a","b",RowBox[{"a", "+", "b", "-", "c", "+", "1"}],
        RowBox[{"1", "-", "z"}]},
       "Hypergeometric2F1"]}], 
     RowBox[{
      TemplateBox[{RowBox[{"c", "-", "a"}]},
       "Gamma"], " ", 
      TemplateBox[{RowBox[{"c", "-", "b"}]},
       "Gamma"]}]]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970312984234*^9},
 CellTags->"Kummer",
 CellLabel->"Out[1064]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Hypergeometric2F1", "[", 
   RowBox[{"a", ",", "b", ",", "c", ",", "z"}], "]"}], "==", 
  RowBox[{
   RowBox[{"Kummer", "[", "95", "]"}], "[", 
   RowBox[{"Hypergeometric2F1", "[", 
    RowBox[{"a", ",", "b", ",", "c", ",", "z"}], "]"}], "]"}]}]], "Input",
 CellTags->"Kummer",
 CellLabel->"In[1065]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TemplateBox[{"a","b","c","z"},
    "Hypergeometric2F1"], "\[LongEqual]", 
   RowBox[{
    FractionBox[
     RowBox[{
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"1", "-", "z"}], ")"}], 
       RowBox[{"-", "a"}]], " ", 
      TemplateBox[{"c"},
       "Gamma"], " ", 
      TemplateBox[{RowBox[{"b", "-", "a"}]},
       "Gamma"], " ", 
      TemplateBox[{
       "a",RowBox[{"c", "-", "b"}],RowBox[{"a", "-", "b", "+", "1"}],
        FractionBox["1", 
         RowBox[{"1", "-", "z"}]]},
       "Hypergeometric2F1"]}], 
     RowBox[{
      TemplateBox[{"b"},
       "Gamma"], " ", 
      TemplateBox[{RowBox[{"c", "-", "a"}]},
       "Gamma"]}]], "+", 
    FractionBox[
     RowBox[{
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"1", "-", "z"}], ")"}], 
       RowBox[{"-", "b"}]], " ", 
      TemplateBox[{"c"},
       "Gamma"], " ", 
      TemplateBox[{RowBox[{"a", "-", "b"}]},
       "Gamma"], " ", 
      TemplateBox[{"b",RowBox[{"c", "-", "a"}],RowBox[{
          RowBox[{"-", "a"}], "+", "b", "+", "1"}],FractionBox["1", 
         RowBox[{"1", "-", "z"}]]},
       "Hypergeometric2F1"]}], 
     RowBox[{
      TemplateBox[{"a"},
       "Gamma"], " ", 
      TemplateBox[{RowBox[{"c", "-", "b"}]},
       "Gamma"]}]]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970313175654*^9},
 CellTags->"Kummer",
 CellLabel->"Out[1065]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Hypergeometric2F1", "[", 
   RowBox[{"a", ",", "b", ",", "c", ",", "z"}], "]"}], "==", 
  RowBox[{
   RowBox[{"Kummer", "[", "96", "]"}], "[", 
   RowBox[{"Hypergeometric2F1", "[", 
    RowBox[{"a", ",", "b", ",", "c", ",", "z"}], "]"}], "]"}]}]], "Input",
 CellTags->"Kummer",
 CellLabel->"In[1066]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TemplateBox[{"a","b","c","z"},
    "Hypergeometric2F1"], "\[LongEqual]", 
   RowBox[{
    FractionBox[
     RowBox[{
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"-", "1"}], ")"}], "a"], " ", 
      SuperscriptBox["z", 
       RowBox[{"-", "a"}]], " ", 
      TemplateBox[{"c"},
       "Gamma"], " ", 
      TemplateBox[{RowBox[{"b", "-", "a"}]},
       "Gamma"], " ", 
      TemplateBox[{
       "a",RowBox[{"a", "-", "c", "+", "1"}],
        RowBox[{"a", "-", "b", "+", "1"}],FractionBox["1", "z"]},
       "Hypergeometric2F1"]}], 
     RowBox[{
      TemplateBox[{"b"},
       "Gamma"], " ", 
      TemplateBox[{RowBox[{"c", "-", "a"}]},
       "Gamma"]}]], "+", 
    FractionBox[
     RowBox[{
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"-", "1"}], ")"}], "b"], " ", 
      SuperscriptBox["z", 
       RowBox[{"-", "b"}]], " ", 
      TemplateBox[{"c"},
       "Gamma"], " ", 
      TemplateBox[{RowBox[{"a", "-", "b"}]},
       "Gamma"], " ", 
      TemplateBox[{"b",RowBox[{"b", "-", "c", "+", "1"}],RowBox[{
          RowBox[{"-", "a"}], "+", "b", "+", "1"}],FractionBox["1", "z"]},
       "Hypergeometric2F1"]}], 
     RowBox[{
      TemplateBox[{"a"},
       "Gamma"], " ", 
      TemplateBox[{RowBox[{"c", "-", "b"}]},
       "Gamma"]}]]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009703134085913`*^9},
 CellTags->"Kummer",
 CellLabel->"Out[1066]="]
}, Open  ]]
}, Open  ]],

Cell["Lagrangian", "ObjectName",
 CellTags->"Lagrangian"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Lagrangian"],

Cell["\<\
Lagrangian[\"oqu\"] gives the unpolarized OPE quark operator. \
Lagrangian[\"oqp\"] gives the polarized quark OPE operator. \
Lagrangian[\"ogu\"] gives the unpolarized gluon OPE operator. \
Lagrangian[\"ogp\"] gives the polarized gluon OPE operator. \
Lagrangian[\"ogd\"] gives the sigma-term part of the QCD lagrangian. \
Lagrangian[\"QCD\"] gives the gluon self interaction part of the QCD \
lagrangian.\
\>", "Text",
 CellTags->"Lagrangian"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FeynRule",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FeynRule",
  ButtonNote->"FeynRule"],
 "."
}], "Text",
 CellTags->"Lagrangian"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Lagrangian"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Lagrangian", "[", "\"\<QCD\>\"", "]"}]], "Input",
 CellTags->"Lagrangian",
 CellLabel->"In[1067]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    FractionBox["1", "4"]}], " ", 
   RowBox[{
    SubsuperscriptBox["F", 
     RowBox[{"\[Alpha]", "\[Beta]"}], "a"], ".", 
    SubsuperscriptBox["F", 
     RowBox[{"\[Alpha]", "\[Beta]"}], "a"]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009703135721292`*^9},
 CellTags->"Lagrangian",
 CellLabel->"Out[1067]="]
}, Open  ]],

Cell["Twist-2 operator product expansion operators", "Text",
 CellTags->"Lagrangian"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Lagrangian", "[", "\"\<ogu\>\"", "]"}]], "Input",
 CellTags->"Lagrangian",
 CellLabel->"In[1068]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["1", "2"], " ", 
   SuperscriptBox["\[ImaginaryI]", 
    RowBox[{"m", "-", "1"}]], " ", 
   RowBox[{
    SubsuperscriptBox["F", 
     RowBox[{"\[Alpha]", 
      FormBox["\[CapitalDelta]",
       TraditionalForm]}], "a"], ".", 
    SuperscriptBox[
     RowBox[{"(", 
      SubsuperscriptBox["D", 
       FormBox["\[CapitalDelta]",
        TraditionalForm], 
       RowBox[{"a", "b"}]], ")"}], 
     RowBox[{"m", "-", "2"}]], ".", 
    SubsuperscriptBox["F", 
     RowBox[{"\[Alpha]", 
      FormBox["\[CapitalDelta]",
       TraditionalForm]}], "b"]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970313692506*^9},
 CellTags->"Lagrangian",
 CellLabel->"Out[1068]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Lagrangian", "[", "\"\<ogp\>\"", "]"}]], "Input",
 CellTags->"Lagrangian",
 CellLabel->"In[1069]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["1", "2"], " ", 
   SuperscriptBox["\[ImaginaryI]", "m"], " ", 
   RowBox[{
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["\[Alpha]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Beta]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Gamma]",
        TraditionalForm],
       TraditionalForm], 
      FormBox["\[CapitalDelta]",
       TraditionalForm]}]], ".", 
    SubsuperscriptBox["F", 
     RowBox[{"\[Beta]", "\[Gamma]"}], "a"], ".", 
    SuperscriptBox[
     RowBox[{"(", 
      SubsuperscriptBox["D", 
       FormBox["\[CapitalDelta]",
        TraditionalForm], 
       RowBox[{"a", "b"}]], ")"}], 
     RowBox[{"m", "-", "2"}]], ".", 
    SubsuperscriptBox["F", 
     RowBox[{"\[Alpha]", 
      FormBox["\[CapitalDelta]",
       TraditionalForm]}], "b"]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970313791171*^9},
 CellTags->"Lagrangian",
 CellLabel->"Out[1069]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Lagrangian", "[", "\"\<oqu\>\"", "]"}]], "Input",
 CellTags->"Lagrangian",
 CellLabel->"In[1070]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[ImaginaryI]", "m"], " ", 
   RowBox[{
    FormBox[
     OverscriptBox["\[Psi]", "_"],
     TraditionalForm], ".", 
    RowBox[{"(", 
     RowBox[{
      FormBox["\<\"\[Gamma]\"\>",
       TraditionalForm], "\[CenterDot]", 
      FormBox["\[CapitalDelta]",
       TraditionalForm]}], ")"}], ".", 
    SuperscriptBox[
     SubscriptBox["D", 
      FormBox["\[CapitalDelta]",
       TraditionalForm]], 
     RowBox[{"m", "-", "1"}]], ".", 
    FormBox["\[Psi]",
     TraditionalForm]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970313895207*^9},
 CellTags->"Lagrangian",
 CellLabel->"Out[1070]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Lagrangian", "[", "\"\<oqp\>\"", "]"}]], "Input",
 CellTags->"Lagrangian",
 CellLabel->"In[1071]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[ImaginaryI]", "m"], " ", 
   RowBox[{
    FormBox[
     OverscriptBox["\[Psi]", "_"],
     TraditionalForm], ".", 
    SuperscriptBox["\[Gamma]", "5"], ".", 
    RowBox[{"(", 
     RowBox[{
      FormBox["\<\"\[Gamma]\"\>",
       TraditionalForm], "\[CenterDot]", 
      FormBox["\[CapitalDelta]",
       TraditionalForm]}], ")"}], ".", 
    SuperscriptBox[
     SubscriptBox["D", 
      FormBox["\[CapitalDelta]",
       TraditionalForm]], 
     RowBox[{"m", "-", "1"}]], ".", 
    FormBox["\[Psi]",
     TraditionalForm]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970314002502*^9},
 CellTags->"Lagrangian",
 CellLabel->"Out[1071]="]
}, Open  ]]
}, Open  ]],

Cell["LC", "ObjectName",
 CellTags->"LC"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"LC"],

Cell["\<\
LC[m,n,r,s] evaluates to 4-dimensional LeviCivita[m,n,r,s] by virtue of \
applying FeynCalcInternal. LC[m,...][p, ...] evaluates to 4-dimensional \
LeviCivita[m,...][p,...] applying FeynCalcInternal.\
\>", "Text",
 CellTags->"LC"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["LeviCivita",
  BaseStyle->"AddOnsLink",
  ButtonData:>"LeviCivita",
  ButtonNote->"LeviCivita"],
 ", ",
 ButtonBox["LCD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"LCD",
  ButtonNote->"LCD"],
 "."
}], "Text",
 CellTags->"LC"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"LC"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"LC", "[", 
  RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]", ",", "\[Sigma]"}], 
  "]"}]], "Input",
 CellTags->"LC",
 CellLabel->"In[1072]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["\[Epsilon]", 
   RowBox[{"\[Mu]", "\[Nu]", "\[Rho]", "\[Sigma]"}]], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970314099575*^9},
 CellTags->"LC",
 CellLabel->"Out[1072]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "FCI"}]], "Input",
 CellTags->"LC",
 CellLabel->"In[1073]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["\[Epsilon]", 
   RowBox[{
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["\[Nu]",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["\[Rho]",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["\[Sigma]",
      TraditionalForm],
     TraditionalForm]}]], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970314188465*^9},
 CellTags->"LC",
 CellLabel->"Out[1073]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "StandardForm"}]], "Input",
 CellTags->"LC",
 CellLabel->"In[1074]:="],

Cell[BoxData[
 RowBox[{"Eps", "[", 
  RowBox[{
   RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
   RowBox[{"LorentzIndex", "[", "\[Nu]", "]"}], ",", 
   RowBox[{"LorentzIndex", "[", "\[Rho]", "]"}], ",", 
   RowBox[{"LorentzIndex", "[", "\[Sigma]", "]"}]}], "]"}]], "Output",
 CellChangeTimes->{3.500970314232777*^9},
 CellTags->"LC",
 CellLabel->"Out[1074]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"LC", "[", 
   RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}], "[", 
  RowBox[{"p", ",", "q"}], "]"}]], "Input",
 CellTags->"LC",
 CellLabel->"In[1075]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["\[Epsilon]", 
   RowBox[{"\[Mu]", "\[Nu]", "p", "q"}]], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970314277412*^9},
 CellTags->"LC",
 CellLabel->"Out[1075]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"%", "//", "FCI"}], "//", "StandardForm"}]], "Input",
 CellTags->"LC",
 CellLabel->"In[1076]:="],

Cell[BoxData[
 RowBox[{"Eps", "[", 
  RowBox[{
   RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
   RowBox[{"LorentzIndex", "[", "\[Nu]", "]"}], ",", 
   RowBox[{"Momentum", "[", 
    RowBox[{"p", ",", "D"}], "]"}], ",", 
   RowBox[{"Momentum", "[", 
    RowBox[{"q", ",", "D"}], "]"}]}], "]"}]], "Output",
 CellChangeTimes->{3.500970314356031*^9},
 CellTags->"LC",
 CellLabel->"Out[1076]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Contract", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"LC", "[", 
      RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]"}], "]"}], "[", "p", "]"}], 
    " ", 
    RowBox[{
     RowBox[{"LC", "[", 
      RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]"}], "]"}], "[", "q", 
     "]"}]}], "]"}], " "}]], "Input",
 CellTags->"LC",
 CellLabel->"In[1077]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"18", " ", 
    RowBox[{
     FormBox["p",
      TraditionalForm], "\[CenterDot]", 
     FormBox["q",
      TraditionalForm]}]}], "-", 
   RowBox[{"24", " ", 
    RowBox[{
     FormBox["p",
      TraditionalForm], "\[CenterDot]", 
     FormBox["q",
      TraditionalForm]}]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970314435821*^9},
 CellTags->"LC",
 CellLabel->"Out[1077]="]
}, Open  ]]
}, Open  ]],

Cell["LCD", "ObjectName",
 CellTags->"LCD"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"LCD"],

Cell["\<\
LCD[m,n,r,s] evaluates to D-dimensional LeviCivita[m,n,r,s] by virtue of \
FeynCalcInternal. LCD[m,...][p, ...] evaluates to D-dimensional \
LeviCivita[m,...][p,...] applying FeynCalcInternal.\
\>", "Text",
 CellTags->"LCD"],

Cell["\<\
You need to do SetOptions[Eps, Dimension\[Rule]D] before LCD can be used as \
D-dimensional Levi-Civita input function.\
\>", "Text",
 CellTags->"LCD"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["LeviCivita",
  BaseStyle->"AddOnsLink",
  ButtonData:>"LeviCivita",
  ButtonNote->"LeviCivita"],
 ", ",
 ButtonBox["LC",
  BaseStyle->"AddOnsLink",
  ButtonData:>"LC",
  ButtonNote->"LC"],
 "."
}], "Text",
 CellTags->"LCD"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"LCD"],

Cell[BoxData[
 RowBox[{
  RowBox[{"SetOptions", "[", 
   RowBox[{"Eps", ",", 
    RowBox[{"Dimension", "\[Rule]", "D"}]}], "]"}], ";"}]], "Input",
 CellTags->"LCD",
 CellLabel->"In[1078]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"LCD", "[", 
  RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]", ",", "\[Sigma]"}], 
  "]"}]], "Input",
 CellTags->"LCD",
 CellLabel->"In[1079]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["\[Epsilon]", 
   RowBox[{"\[Mu]", "\[Nu]", "\[Rho]", "\[Sigma]"}]], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.50097031453994*^9},
 CellTags->"LCD",
 CellLabel->"Out[1079]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "FCI"}]], "Input",
 CellTags->"LCD",
 CellLabel->"In[1080]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["\[Epsilon]", 
   RowBox[{
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["\[Nu]",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["\[Rho]",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["\[Sigma]",
      TraditionalForm],
     TraditionalForm]}]], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970314689602*^9},
 CellTags->"LCD",
 CellLabel->"Out[1080]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "StandardForm"}]], "Input",
 CellTags->"LCD",
 CellLabel->"In[1081]:="],

Cell[BoxData[
 RowBox[{"Eps", "[", 
  RowBox[{
   RowBox[{"LorentzIndex", "[", 
    RowBox[{"\[Mu]", ",", "D"}], "]"}], ",", 
   RowBox[{"LorentzIndex", "[", 
    RowBox[{"\[Nu]", ",", "D"}], "]"}], ",", 
   RowBox[{"LorentzIndex", "[", 
    RowBox[{"\[Rho]", ",", "D"}], "]"}], ",", 
   RowBox[{"LorentzIndex", "[", 
    RowBox[{"\[Sigma]", ",", "D"}], "]"}]}], "]"}]], "Output",
 CellChangeTimes->{3.500970314846794*^9},
 CellTags->"LCD",
 CellLabel->"Out[1081]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"LCD", "[", 
   RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}], "[", 
  RowBox[{"p", ",", "q"}], "]"}]], "Input",
 CellTags->"LCD",
 CellLabel->"In[1082]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["\[Epsilon]", 
   RowBox[{"\[Mu]", "\[Nu]", "p", "q"}]], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970314889463*^9},
 CellTags->"LCD",
 CellLabel->"Out[1082]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"%", "//", "FCI"}], "//", "StandardForm"}]], "Input",
 CellTags->"LCD",
 CellLabel->"In[1083]:="],

Cell[BoxData[
 RowBox[{"Eps", "[", 
  RowBox[{
   RowBox[{"LorentzIndex", "[", 
    RowBox[{"\[Mu]", ",", "D"}], "]"}], ",", 
   RowBox[{"LorentzIndex", "[", 
    RowBox[{"\[Nu]", ",", "D"}], "]"}], ",", 
   RowBox[{"Momentum", "[", 
    RowBox[{"p", ",", "D"}], "]"}], ",", 
   RowBox[{"Momentum", "[", 
    RowBox[{"q", ",", "D"}], "]"}]}], "]"}]], "Output",
 CellChangeTimes->{3.5009703149650917`*^9},
 CellTags->"LCD",
 CellLabel->"Out[1083]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Factor2", "[", 
  RowBox[{"Contract", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"LCD", "[", 
      RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]"}], "]"}], "[", "p", "]"}], 
    " ", 
    RowBox[{
     RowBox[{"LCD", "[", 
      RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]"}], "]"}], "[", "q", 
     "]"}]}], "]"}], "]"}]], "Input",
 CellTags->"LCD",
 CellLabel->"In[1084]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"(", 
    RowBox[{"1", "-", "D"}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{"2", "-", "D"}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{"3", "-", "D"}], ")"}], " ", 
   RowBox[{
    FormBox["p",
     TraditionalForm], "\[CenterDot]", 
    FormBox["q",
     TraditionalForm]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970315044696*^9},
 CellTags->"LCD",
 CellLabel->"Out[1084]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SetOptions", "[", 
  RowBox[{"Eps", ",", 
   RowBox[{"Dimension", "\[Rule]", "4"}]}], "]"}]], "Input",
 CellTags->"LCD",
 CellLabel->"In[1085]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"Dimension", "\[Rule]", "4"}], "}"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970315153421*^9},
 CellTags->"LCD",
 CellLabel->"Out[1085]="]
}, Open  ]]
}, Open  ]],

Cell["LeftPartialD", "ObjectName",
 CellTags->"LeftPartialD"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"LeftPartialD"],

Cell[TextData[{
 "LeftPartialD[\[Mu]] denotes ",
 Cell[BoxData[
  FormBox[
   SubscriptBox[
    OverscriptBox["\[PartialD]", "\[LeftArrow]"], 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], TraditionalForm]]],
 "acting to the left."
}], "Text",
 CellTags->"LeftPartialD"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["ExpandPartialD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"ExpandPartialD",
  ButtonNote->"ExpandPartialD"],
 ", ",
 ButtonBox["PartialD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"PartialD",
  ButtonNote->"PartialD"],
 ", ",
 ButtonBox["LeftRightPartialD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"LeftRightPartialD",
  ButtonNote->"LeftRightPartialD"],
 ", ",
 ButtonBox["RightPartialD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"RightPartialD",
  ButtonNote->"RightPartialD"],
 "."
}], "Text",
 CellTags->"LeftPartialD"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData["Examples"], "Subsubsection",
 CellTags->"LeftPartialD"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"QuantumField", "[", 
   RowBox[{"A", ",", 
    RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}]}], "]"}], ".", 
  RowBox[{"LeftPartialD", "[", "\[Nu]", "]"}]}]], "Input",
 CellTags->"LeftPartialD",
 CellLabel->"In[1086]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["A", "\[Mu]"], ".", 
   SubscriptBox[
    OverscriptBox["\[PartialD]", "\[LeftArrow]"], 
    FormBox[
     FormBox["\[Nu]",
      TraditionalForm],
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970315195592*^9},
 CellTags->"LeftPartialD",
 CellLabel->"Out[1086]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ExpandPartialD", "[", "%", "]"}]], "Input",
 CellTags->"LeftPartialD",
 CellLabel->"In[1087]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"QuantumField", "(", 
   RowBox[{
    RowBox[{
    "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
     FormBox["\[Nu]",
      TraditionalForm], ")"}], ",", "A", ",", 
    FormBox["\[Mu]",
     TraditionalForm]}], ")"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970315272558*^9},
 CellTags->"LeftPartialD",
 CellLabel->"Out[1087]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StandardForm", "[", "%", "]"}]], "Input",
 CellTags->"LeftPartialD",
 CellLabel->"In[1088]:="],

Cell[BoxData[
 RowBox[{"QuantumField", "[", 
  RowBox[{
   RowBox[{
   "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "[", 
    RowBox[{"LorentzIndex", "[", "\[Nu]", "]"}], "]"}], ",", "A", ",", 
   RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}]}], "]"}]], "Output",
 CellChangeTimes->{3.500970315319347*^9},
 CellTags->"LeftPartialD",
 CellLabel->"Out[1088]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StandardForm", "[", 
  RowBox[{"LeftPartialD", "[", "\[Mu]", "]"}], "]"}]], "Input",
 CellTags->"LeftPartialD",
 CellLabel->"In[1089]:="],

Cell[BoxData[
 RowBox[{"LeftPartialD", "[", 
  RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], "]"}]], "Output",
 CellChangeTimes->{3.500970315363104*^9},
 CellTags->"LeftPartialD",
 CellLabel->"Out[1089]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"QuantumField", "[", 
   RowBox[{"A", ",", 
    RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}]}], "]"}], ".", 
  RowBox[{"QuantumField", "[", 
   RowBox[{"A", ",", 
    RowBox[{"LorentzIndex", "[", "\[Nu]", "]"}]}], "]"}], ".", 
  RowBox[{"LeftPartialD", "[", "\[Rho]", "]"}]}]], "Input",
 CellTags->"LeftPartialD",
 CellLabel->"In[1090]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["A", "\[Mu]"], ".", 
   SubscriptBox["A", "\[Nu]"], ".", 
   SubscriptBox[
    OverscriptBox["\[PartialD]", "\[LeftArrow]"], 
    FormBox[
     FormBox["\[Rho]",
      TraditionalForm],
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.50097031544359*^9},
 CellTags->"LeftPartialD",
 CellLabel->"Out[1090]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ExpandPartialD", "[", "%", "]"}]], "Input",
 CellTags->"LeftPartialD",
 CellLabel->"In[1091]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox["A", "\[Mu]"], ".", 
    RowBox[{"QuantumField", "(", 
     RowBox[{
      RowBox[{
      "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
       FormBox["\[Rho]",
        TraditionalForm], ")"}], ",", "A", ",", 
      FormBox["\[Nu]",
       TraditionalForm]}], ")"}]}], "+", 
   RowBox[{
    RowBox[{"QuantumField", "(", 
     RowBox[{
      RowBox[{
      "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
       FormBox["\[Rho]",
        TraditionalForm], ")"}], ",", "A", ",", 
      FormBox["\[Mu]",
       TraditionalForm]}], ")"}], ".", 
    SubscriptBox["A", "\[Nu]"]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970315531125*^9},
 CellTags->"LeftPartialD",
 CellLabel->"Out[1091]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StandardForm", "[", "%", "]"}]], "Input",
 CellTags->"LeftPartialD",
 CellLabel->"In[1092]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"QuantumField", "[", 
    RowBox[{"A", ",", 
     RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}]}], "]"}], ".", 
   RowBox[{"QuantumField", "[", 
    RowBox[{
     RowBox[{
     "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "[", 
      RowBox[{"LorentzIndex", "[", "\[Rho]", "]"}], "]"}], ",", "A", ",", 
     RowBox[{"LorentzIndex", "[", "\[Nu]", "]"}]}], "]"}]}], "+", 
  RowBox[{
   RowBox[{"QuantumField", "[", 
    RowBox[{
     RowBox[{
     "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "[", 
      RowBox[{"LorentzIndex", "[", "\[Rho]", "]"}], "]"}], ",", "A", ",", 
     RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}]}], "]"}], ".", 
   RowBox[{"QuantumField", "[", 
    RowBox[{"A", ",", 
     RowBox[{"LorentzIndex", "[", "\[Nu]", "]"}]}], "]"}]}]}]], "Output",
 CellChangeTimes->{3.5009703156114817`*^9},
 CellTags->"LeftPartialD",
 CellLabel->"Out[1092]//StandardForm="]
}, Open  ]]
}, Open  ]],

Cell["LeftRightPartialD", "ObjectName",
 CellTags->"LeftRightPartialD"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"LeftRightPartialD"],

Cell[TextData[{
 "LeftRightPartialD[mu] denotes ",
 Cell[BoxData[
  FormBox[
   SubscriptBox[
    OverscriptBox["\[PartialD]", "\[LeftRightArrow]"], 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], TraditionalForm]]],
 ", acting to the left and right. ExplicitPartialD[LeftRightPartialD[",
 Cell[BoxData[
  FormBox["\[Mu]", TraditionalForm]]],
 "]] gives 1/2 (RightPartialD[",
 Cell[BoxData[
  FormBox["\[Mu]", TraditionalForm]]],
 "] - LeftPartialD[",
 Cell[BoxData[
  FormBox["\[Mu]", TraditionalForm]]],
 "])."
}], "Text",
 CellTags->"LeftRightPartialD"],

Cell[TextData[{
 "See also: ",
 ButtonBox["ExplicitPartialD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"ExplicitPartialD",
  ButtonNote->"ExplicitPartialD"],
 ", ",
 ButtonBox["ExpandPartialD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"ExpandPartialD",
  ButtonNote->"ExpandPartialD"],
 ", ",
 ButtonBox["PartialD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"PartialD",
  ButtonNote->"PartialD"],
 ", ",
 ButtonBox["LeftPartialD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"LeftPartialD",
  ButtonNote->"LeftPartialD"],
 ", ",
 ButtonBox["LeftRightPartialD2",
  BaseStyle->"AddOnsLink",
  ButtonData:>"LeftRightPartialD2",
  ButtonNote->"LeftRightPartialD2"],
 ", ",
 ButtonBox["RightPartialD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"RightPartialD",
  ButtonNote->"RightPartialD"],
 "."
}], "Text",
 CellTags->"LeftRightPartialD"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"LeftRightPartialD"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"LeftRightPartialD", "[", "\[Mu]", "]"}]], "Input",
 CellTags->"LeftRightPartialD",
 CellLabel->"In[1093]:="],

Cell[BoxData[
 FormBox[
  SubscriptBox[
   OverscriptBox["\[PartialD]", "\[LeftRightArrow]"], 
   FormBox[
    FormBox["\[Mu]",
     TraditionalForm],
    TraditionalForm]], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970315662957*^9},
 CellTags->"LeftRightPartialD",
 CellLabel->"Out[1093]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ExplicitPartialD", "[", "%", "]"}]], "Input",
 CellTags->"LeftRightPartialD",
 CellLabel->"In[1094]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["1", "2"], " ", 
   RowBox[{"(", 
    RowBox[{
     SubscriptBox[
      OverscriptBox["\[PartialD]", "\[RightArrow]"], 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], "-", 
     SubscriptBox[
      OverscriptBox["\[PartialD]", "\[LeftArrow]"], 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]]}], ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009703158146276`*^9},
 CellTags->"LeftRightPartialD",
 CellLabel->"Out[1094]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"LeftRightPartialD", "[", "\[Mu]", "]"}], ".", 
  RowBox[{"QuantumField", "[", 
   RowBox[{"A", ",", 
    RowBox[{"LorentzIndex", "[", "\[Nu]", "]"}]}], "]"}]}]], "Input",
 CellTags->"LeftRightPartialD",
 CellLabel->"In[1095]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox[
    OverscriptBox["\[PartialD]", "\[LeftRightArrow]"], 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], ".", 
   SubscriptBox["A", "\[Nu]"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970315985793*^9},
 CellTags->"LeftRightPartialD",
 CellLabel->"Out[1095]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ExpandPartialD", "[", "%", "]"}]], "Input",
 CellTags->"LeftRightPartialD",
 CellLabel->"In[1096]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["1", "2"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"QuantumField", "(", 
      RowBox[{
       RowBox[{
       "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
        FormBox["\[Mu]",
         TraditionalForm], ")"}], ",", "A", ",", 
       FormBox["\[Nu]",
        TraditionalForm]}], ")"}], "-", 
     RowBox[{
      SubscriptBox[
       OverscriptBox["\[PartialD]", "\[LeftArrow]"], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], ".", 
      SubscriptBox["A", "\[Nu]"]}]}], ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009703160949802`*^9},
 CellTags->"LeftRightPartialD",
 CellLabel->"Out[1096]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"QuantumField", "[", 
   RowBox[{"A", ",", 
    RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}]}], "]"}], ".", 
  RowBox[{"LeftRightPartialD", "[", "\[Nu]", "]"}], ".", 
  RowBox[{"QuantumField", "[", 
   RowBox[{"A", ",", 
    RowBox[{"LorentzIndex", "[", "\[Rho]", "]"}]}], "]"}]}]], "Input",
 CellTags->"LeftRightPartialD",
 CellLabel->"In[1097]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["A", "\[Mu]"], ".", 
   SubscriptBox[
    OverscriptBox["\[PartialD]", "\[LeftRightArrow]"], 
    FormBox[
     FormBox["\[Nu]",
      TraditionalForm],
     TraditionalForm]], ".", 
   SubscriptBox["A", "\[Rho]"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970316183282*^9},
 CellTags->"LeftRightPartialD",
 CellLabel->"Out[1097]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ExpandPartialD", "[", "%", "]"}]], "Input",
 CellTags->"LeftRightPartialD",
 CellLabel->"In[1098]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["1", "2"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      SubscriptBox["A", "\[Mu]"], ".", 
      RowBox[{"QuantumField", "(", 
       RowBox[{
        RowBox[{
        "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
         FormBox["\[Nu]",
          TraditionalForm], ")"}], ",", "A", ",", 
        FormBox["\[Rho]",
         TraditionalForm]}], ")"}]}], "-", 
     RowBox[{
      RowBox[{"QuantumField", "(", 
       RowBox[{
        RowBox[{
        "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
         FormBox["\[Nu]",
          TraditionalForm], ")"}], ",", "A", ",", 
        FormBox["\[Mu]",
         TraditionalForm]}], ")"}], ".", 
      SubscriptBox["A", "\[Rho]"]}]}], ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970316347316*^9},
 CellTags->"LeftRightPartialD",
 CellLabel->"Out[1098]="]
}, Open  ]]
}, Open  ]],

Cell["LeftRightPartialD2", "ObjectName",
 CellTags->"LeftRightPartialD2"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"LeftRightPartialD2"],

Cell[TextData[{
 "LeftRightPartialD2[mu] denotes ",
 Cell[BoxData[
  FormBox[
   SubscriptBox[
    OverscriptBox["\[PartialD]", "\[LeftRightArrow]"], 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], TraditionalForm]]],
 ", acting to the left and right. ExplicitPartialD[LeftRightPartialD2[",
 Cell[BoxData[
  FormBox["\[Mu]", TraditionalForm]]],
 "]] gives (RightPartialD[",
 Cell[BoxData[
  FormBox["\[Mu]", TraditionalForm]]],
 "] + LeftPartialD[",
 Cell[BoxData[
  FormBox["\[Mu]", TraditionalForm]]],
 "])."
}], "Text",
 CellTags->"LeftRightPartialD2"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["ExplicitPartialD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"ExplicitPartialD",
  ButtonNote->"ExplicitPartialD"],
 ", ",
 ButtonBox["ExpandPartialD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"ExpandPartialD",
  ButtonNote->"ExpandPartialD"],
 ", ",
 ButtonBox["PartialD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"PartialD",
  ButtonNote->"PartialD"],
 ", ",
 ButtonBox["LeftPartialD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"LeftPartialD",
  ButtonNote->"LeftPartialD"],
 ", ",
 ButtonBox["RightPartialD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"RightPartialD",
  ButtonNote->"RightPartialD"],
 "."
}], "Text",
 CellTags->"LeftRightPartialD2"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"LeftRightPartialD2"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"LeftRightPartialD2", "[", "\[Mu]", "]"}]], "Input",
 CellTags->"LeftRightPartialD2",
 CellLabel->"In[1099]:="],

Cell[BoxData[
 FormBox[
  SubscriptBox[
   OverscriptBox["\[PartialD]", "\[LeftRightArrow]"], 
   FormBox[
    FormBox["\[Mu]",
     TraditionalForm],
    TraditionalForm]], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970316488241*^9},
 CellTags->"LeftRightPartialD2",
 CellLabel->"Out[1099]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ExplicitPartialD", "[", "%", "]"}]], "Input",
 CellTags->"LeftRightPartialD2",
 CellLabel->"In[1100]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox[
    OverscriptBox["\[PartialD]", "\[LeftArrow]"], 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], "+", 
   SubscriptBox[
    OverscriptBox["\[PartialD]", "\[RightArrow]"], 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970316566457*^9},
 CellTags->"LeftRightPartialD2",
 CellLabel->"Out[1100]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"LeftRightPartialD2", "[", "\[Mu]", "]"}], ".", 
  RowBox[{"QuantumField", "[", 
   RowBox[{"A", ",", 
    RowBox[{"LorentzIndex", "[", "\[Nu]", "]"}]}], "]"}]}]], "Input",
 CellTags->"LeftRightPartialD2",
 CellLabel->"In[1101]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox[
    OverscriptBox["\[PartialD]", "\[LeftRightArrow]"], 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], ".", 
   SubscriptBox["A", "\[Nu]"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009703166480427`*^9},
 CellTags->"LeftRightPartialD2",
 CellLabel->"Out[1101]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ExpandPartialD", "[", "%", "]"}]], "Input",
 CellTags->"LeftRightPartialD2",
 CellLabel->"In[1102]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"QuantumField", "(", 
    RowBox[{
     RowBox[{
     "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
      FormBox["\[Mu]",
       TraditionalForm], ")"}], ",", "A", ",", 
     FormBox["\[Nu]",
      TraditionalForm]}], ")"}], "+", 
   RowBox[{
    SubscriptBox[
     OverscriptBox["\[PartialD]", "\[LeftArrow]"], 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], ".", 
    SubscriptBox["A", "\[Nu]"]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009703167364492`*^9},
 CellTags->"LeftRightPartialD2",
 CellLabel->"Out[1102]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"QuantumField", "[", 
   RowBox[{"A", ",", 
    RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}]}], "]"}], ".", 
  RowBox[{"LeftRightPartialD2", "[", "\[Nu]", "]"}], ".", 
  RowBox[{"QuantumField", "[", 
   RowBox[{"A", ",", 
    RowBox[{"LorentzIndex", "[", "\[Rho]", "]"}]}], "]"}]}]], "Input",
 CellTags->"LeftRightPartialD2",
 CellLabel->"In[1103]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["A", "\[Mu]"], ".", 
   SubscriptBox[
    OverscriptBox["\[PartialD]", "\[LeftRightArrow]"], 
    FormBox[
     FormBox["\[Nu]",
      TraditionalForm],
     TraditionalForm]], ".", 
   SubscriptBox["A", "\[Rho]"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970316791727*^9},
 CellTags->"LeftRightPartialD2",
 CellLabel->"Out[1103]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ExpandPartialD", "[", "%", "]"}]], "Input",
 CellTags->"LeftRightPartialD2",
 CellLabel->"In[1104]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox["A", "\[Mu]"], ".", 
    RowBox[{"QuantumField", "(", 
     RowBox[{
      RowBox[{
      "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
       FormBox["\[Nu]",
        TraditionalForm], ")"}], ",", "A", ",", 
      FormBox["\[Rho]",
       TraditionalForm]}], ")"}]}], "+", 
   RowBox[{
    RowBox[{"QuantumField", "(", 
     RowBox[{
      RowBox[{
      "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
       FormBox["\[Nu]",
        TraditionalForm], ")"}], ",", "A", ",", 
      FormBox["\[Mu]",
       TraditionalForm]}], ")"}], ".", 
    SubscriptBox["A", "\[Rho]"]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.50097031694127*^9},
 CellTags->"LeftRightPartialD2",
 CellLabel->"Out[1104]="]
}, Open  ]]
}, Open  ]],

Cell["LeviCivita ", "ObjectName",
 CellTags->"LeviCivita"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"LeviCivita"],

Cell[TextData[{
 "LeviCivita[",
 Cell[BoxData[
  FormBox[
   RowBox[{
   "\[Mu]", ",", " ", "\[Nu]", ",", " ", "\[Rho]", ",", " ", "\[Sigma]"}], 
   TraditionalForm]]],
 "] is an input function for the totally antisymmetric Levi-Civita tensor. It \
evaluates automatically to the internal representation Eps[LorentzIndex[",
 Cell[BoxData[
  FormBox["\[Mu]", TraditionalForm]]],
 "], LorentzIndex[",
 Cell[BoxData[
  FormBox["\[Nu]", TraditionalForm]]],
 "], LorentzIndex[",
 Cell[BoxData[
  FormBox[
   FormBox["\[Rho]",
    TraditionalForm], TraditionalForm]]],
 "], LorentzIndex[\[Sigma]]] (or with a second argument in LorentzIndex for \
the Dimension, if the option Dimension of LeviCivita is changed). LeviCivita[",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Mu]", ",", " ", 
    RowBox[{"\[Nu]", " ", "..."}]}], TraditionalForm]]],
 "][ ",
 Cell[BoxData[
  FormBox[
   RowBox[{"p", ",", " ", ".."}], TraditionalForm]]],
 ".] evaluates to Eps[LorentzIndex[",
 Cell[BoxData[
  FormBox["\[Mu]", TraditionalForm]]],
 "], LorentzIndex[",
 Cell[BoxData[
  FormBox["\[Nu]", TraditionalForm]]],
 "], ..., Momentum[",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 "], ...]."
}], "Text",
 CellTags->"LeviCivita"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "LeviCivita", "]"}]], "Input",
 CellTags->"LeviCivita",
 CellLabel->"In[1105]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"Dimension", "\[Rule]", "D"}], "}"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970317123549*^9},
 CellTags->"LeviCivita",
 CellLabel->"Out[1105]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Eps",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Eps",
  ButtonNote->"Eps"],
 ", ",
 ButtonBox["Contract",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Contract",
  ButtonNote->"Contract"],
 ", ",
 ButtonBox["LC",
  BaseStyle->"AddOnsLink",
  ButtonData:>"LC",
  ButtonNote->"LC"],
 ", ",
 ButtonBox["LCD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"LCD",
  ButtonNote->"LCD"],
 "."
}], "Text",
 CellTags->"LeviCivita"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"LeviCivita"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"LeviCivita", "[", 
  RowBox[{"\[Alpha]", ",", "\[Beta]", ",", "\[Gamma]", ",", "\[Delta]"}], 
  "]"}]], "Input",
 CellTags->"LeviCivita",
 CellLabel->"In[1106]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["\[Epsilon]", 
   RowBox[{
    FormBox[
     FormBox["\[Alpha]",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["\[Beta]",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["\[Gamma]",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["\[Delta]",
      TraditionalForm],
     TraditionalForm]}]], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970317188961*^9},
 CellTags->"LeviCivita",
 CellLabel->"Out[1106]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"LeviCivita", "[", 
   RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}], "[", 
  RowBox[{"OPEDelta", ",", "p"}], "]"}]], "Input",
 CellTags->"LeviCivita",
 CellLabel->"In[1107]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["\[Epsilon]", 
   RowBox[{
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["\[Nu]",
      TraditionalForm],
     TraditionalForm], 
    FormBox["\[CapitalDelta]",
     TraditionalForm], 
    FormBox["p",
     TraditionalForm]}]], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970317346385*^9},
 CellTags->"LeviCivita",
 CellLabel->"Out[1107]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"LeviCivita", "[", "]"}], "[", 
  RowBox[{"p", ",", "q", ",", "r", ",", "s"}], "]"}]], "Input",
 CellTags->"LeviCivita",
 CellLabel->"In[1108]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["\[Epsilon]", 
   RowBox[{
    FormBox["p",
     TraditionalForm], 
    FormBox["q",
     TraditionalForm], 
    FormBox["r",
     TraditionalForm], 
    FormBox["s",
     TraditionalForm]}]], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970317429298*^9},
 CellTags->"LeviCivita",
 CellLabel->"Out[1108]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"LeviCivita", "[", 
  RowBox[{"\[Alpha]", ",", "\[Beta]", ",", "\[Beta]", ",", "\[Delta]"}], 
  "]"}]], "Input",
 CellTags->"LeviCivita",
 CellLabel->"In[1109]:="],

Cell[BoxData[
 FormBox["0", TraditionalForm]], "Output",
 CellChangeTimes->{3.500970317492572*^9},
 CellTags->"LeviCivita",
 CellLabel->"Out[1109]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"LeviCivita", "[", 
    RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}], "[", 
   RowBox[{"OPEDelta", ",", "p"}], "]"}], "//", "StandardForm"}]], "Input",
 CellTags->"LeviCivita",
 CellLabel->"In[1110]:="],

Cell[BoxData[
 RowBox[{"Eps", "[", 
  RowBox[{
   RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
   RowBox[{"LorentzIndex", "[", "\[Nu]", "]"}], ",", 
   RowBox[{"Momentum", "[", "OPEDelta", "]"}], ",", 
   RowBox[{"Momentum", "[", "p", "]"}]}], "]"}]], "Output",
 CellChangeTimes->{3.500970317573045*^9},
 CellTags->"LeviCivita",
 CellLabel->"Out[1110]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t1", " ", "=", " ", 
  RowBox[{
   RowBox[{"LeviCivita", "[", 
    RowBox[{"\[Alpha]", ",", "\[Beta]", ",", "\[Gamma]", ",", "\[Rho]"}], 
    "]"}], ".", 
   RowBox[{"LeviCivita", "[", 
    RowBox[{"\[Alpha]", ",", "\[Beta]", ",", "\[Gamma]", ",", "\[Rho]"}], 
    "]"}]}], " "}]], "Input",
 CellTags->"LeviCivita",
 CellLabel->"In[1111]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Epsilon]", 
    RowBox[{
     FormBox[
      FormBox["\[Alpha]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["\[Beta]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["\[Gamma]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["\[Rho]",
       TraditionalForm],
      TraditionalForm]}]], ".", 
   SuperscriptBox["\[Epsilon]", 
    RowBox[{
     FormBox[
      FormBox["\[Alpha]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["\[Beta]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["\[Gamma]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["\[Rho]",
       TraditionalForm],
      TraditionalForm]}]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.50097031765366*^9},
 CellTags->"LeviCivita",
 CellLabel->"Out[1111]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Contract", "[", "t1", "]"}]], "Input",
 CellTags->"LeviCivita",
 CellLabel->"In[1112]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", "24"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.50097031776445*^9},
 CellTags->"LeviCivita",
 CellLabel->"Out[1112]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"SetOptions", "[", 
   RowBox[{"LeviCivita", ",", 
    RowBox[{"Dimension", "\[Rule]", "D"}]}], "]"}], ";", 
  RowBox[{"SetOptions", "[", 
   RowBox[{"Eps", ",", 
    RowBox[{"Dimension", "\[Rule]", "D"}]}], "]"}], ";"}]], "Input",
 CellTags->"LeviCivita",
 CellLabel->"In[1113]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Contract", "[", " ", 
   RowBox[{
    RowBox[{"LeviCivita", "[", 
     RowBox[{"\[Alpha]", ",", "\[Beta]", ",", "\[Gamma]", ",", "\[Rho]"}], 
     "]"}], ".", 
    RowBox[{"LeviCivita", "[", 
     RowBox[{"\[Alpha]", ",", "\[Beta]", ",", "\[Gamma]", ",", "\[Rho]"}], 
     "]"}]}], "]"}], "//", "Factor2"}]], "Input",
 CellTags->"LeviCivita",
 CellLabel->"In[1114]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"(", 
    RowBox[{"1", "-", "D"}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{"2", "-", "D"}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{"3", "-", "D"}], ")"}], " ", "D"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970317933887*^9},
 CellTags->"LeviCivita",
 CellLabel->"Out[1114]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"SetOptions", "[", 
   RowBox[{"LeviCivita", ",", 
    RowBox[{"Dimension", "\[Rule]", "4"}]}], "]"}], ";", 
  RowBox[{"SetOptions", "[", 
   RowBox[{"Eps", ",", 
    RowBox[{"Dimension", "\[Rule]", "4"}]}], "]"}], ";"}]], "Input",
 CellTags->"LeviCivita",
 CellLabel->"In[1115]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"LC", "[", 
  RowBox[{"\[Alpha]", ",", "\[Beta]", ",", "\[Gamma]", ",", "\[Delta]"}], 
  "]"}]], "Input",
 CellTags->"LeviCivita",
 CellLabel->"In[1116]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["\[Epsilon]", 
   RowBox[{"\[Alpha]", "\[Beta]", "\[Gamma]", "\[Delta]"}]], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970318356826*^9},
 CellTags->"LeviCivita",
 CellLabel->"Out[1116]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"Clear", "[", "t1", "]"}]], "Input",
 CellTags->"LeviCivita",
 CellLabel->"In[1117]:="]
}, Open  ]],

Cell["LeviCivitaSign", "ObjectName",
 CellTags->"LeviCivitaSign"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"LeviCivitaSign"],

Cell["\<\
LeviCivitaSign is an option for DiracTrace and EpsChisholm. It determines the \
sign in the result of a Dirac trace of four gamma matrices and gamma5.\
\>", "Text",
 CellTags->"LeviCivitaSign"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["DiracTrace",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracTrace",
  ButtonNote->"DiracTrace"],
 ", ",
 ButtonBox["EpsChisholm",
  BaseStyle->"AddOnsLink",
  ButtonData:>"EpsChisholm",
  ButtonNote->"EpsChisholm"],
 "."
}], "Text",
 CellTags->"LeviCivitaSign"]
}, Open  ]],

Cell["ListIntegrals  ***unfinished***", "ObjectName",
 CellTags->"ListIntegrals"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"ListIntegrals"],

Cell["\<\
ListIntegrals[exp, {q1, q2}] gives a list of basic integrals  \
(FeynAmpDenominator's multiplied by scalar products involving q1, q2).  Any \
non-integer exponent (e.g. the (OPEm-1) in  \
ScalarProduct[OPEDelta,q1]^(OPEm-1)) is replaced by OPEm.  If the options \
Pair is set to True only integrals involving   scalar products (not counting \
those with OPEDelta) are given; with Pair \[Rule] False those are excluded.\
\>", "Text",
 CellTags->"ListIntegrals"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"ListIntegrals"],

Cell[BoxData[""], "Input",
 CellTags->"ListIntegrals",
 CellLabel->"In[1118]:="]
}, Open  ]],

Cell["Li2 ", "ObjectName",
 CellTags->"Li2"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Li2"],

Cell["\<\
 Li2 is an abbreviation for the dilog function, i.e., Li2 = PolyLog[2, #]&.\
\>", "Text",
 CellTags->"Li2"],

Cell[TextData[{
 "See also: ",
 ButtonBox["Li3",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Li3",
  ButtonNote->"Li3"],
 "."
}], "Text",
 CellTags->"Li2"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Li2"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Li2", "[", "x", "]"}]], "Input",
 CellTags->"Li2",
 CellLabel->"In[1119]:="],

Cell[BoxData[
 FormBox[
  TemplateBox[{"2","x"},
   "PolyLog"], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970318487998*^9},
 CellTags->"Li2",
 CellLabel->"Out[1119]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Li2", "//", "StandardForm"}]], "Input",
 CellTags->"Li2",
 CellLabel->"In[1120]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"PolyLog", "[", 
   RowBox[{"2", ",", "#1"}], "]"}], "&"}]], "Output",
 CellChangeTimes->{3.500970318533296*^9},
 CellTags->"Li2",
 CellLabel->"Out[1120]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"-", 
  RowBox[{"\[Integral]", 
   RowBox[{
    FractionBox[
     RowBox[{"Log", "[", 
      RowBox[{"1", "-", "x"}], "]"}], "x"], 
    RowBox[{"\[DifferentialD]", "x"}]}]}]}]], "Input",
 CellTags->"Li2",
 CellLabel->"In[1121]:="],

Cell[BoxData[
 FormBox[
  TemplateBox[{"2","x"},
   "PolyLog"], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009703194448547`*^9},
 CellTags->"Li2",
 CellLabel->"Out[1121]="]
}, Open  ]]
}, Open  ]],

Cell["Li3 ", "ObjectName",
 CellTags->"Li3"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Li3"],

Cell["\<\
 Li3 is an abbreviation for the trilog function, i.e., Li3 = PolyLog[3, #]&.\
\>", "Text",
 CellTags->"Li3"],

Cell[TextData[{
 "See also: ",
 ButtonBox["Li2",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Li2",
  ButtonNote->"Li2"],
 "."
}], "Text",
 CellTags->"Li3"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Li3"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Li3", "[", "x", "]"}]], "Input",
 CellTags->"Li3",
 CellLabel->"In[1122]:="],

Cell[BoxData[
 FormBox[
  TemplateBox[{"3","x"},
   "PolyLog"], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970320019538*^9},
 CellTags->"Li3",
 CellLabel->"Out[1122]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Li3", "//", "StandardForm"}]], "Input",
 CellTags->"Li3",
 CellLabel->"In[1123]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"PolyLog", "[", 
   RowBox[{"3", ",", "#1"}], "]"}], "&"}]], "Output",
 CellChangeTimes->{3.500970320063252*^9},
 CellTags->"Li3",
 CellLabel->"Out[1123]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"D", "[", 
  RowBox[{
   RowBox[{"Li3", "[", "x", "]"}], ",", "x"}], "]"}]], "Input",
 CellTags->"Li3",
 CellLabel->"In[1124]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   TemplateBox[{"2","x"},
    "PolyLog"], "x"], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009703201125307`*^9},
 CellTags->"Li3",
 CellLabel->"Out[1124]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"\[Integral]", 
  RowBox[{
   FractionBox[
    RowBox[{"Li2", "[", "x", "]"}], "x"], 
   RowBox[{"\[DifferentialD]", "x"}]}]}]], "Input",
 CellTags->"Li3",
 CellLabel->"In[1125]:="],

Cell[BoxData[
 FormBox[
  TemplateBox[{"3","x"},
   "PolyLog"], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009703203201857`*^9},
 CellTags->"Li3",
 CellLabel->"Out[1125]="]
}, Open  ]]
}, Open  ]],

Cell["Loop", "ObjectName",
 CellTags->"Loop"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Loop"],

Cell["Loop is an option indicating the number of (virtual) loops.", "Text",
 CellTags->"Loop"]
}, Open  ]],

Cell["LorentzIndex", "ObjectName",
 CellTags->"LorentzIndex"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"LorentzIndex"],

Cell["\<\
LorentzIndex[mu] denotes a four dimensional Lorentz index. For other than \
four dimensions: LorentzIndex[mu, D] or LorentzIndex[mu], etc. \
LorentzIndex[mu, 4] simplifies to LorentzIndex[mu].\
\>", "Text",
 CellTags->"LorentzIndex"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["ChangeDimension",
  BaseStyle->"AddOnsLink",
  ButtonData:>"ChangeDimension",
  ButtonNote->"ChangeDimension"],
 ", ",
 ButtonBox["Momentum",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Momentum",
  ButtonNote->"Momentum"],
 "."
}], "Text",
 CellTags->"LorentzIndex"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"LorentzIndex"],

Cell["This denotes a four-dimensional Lorentz index.", "Text",
 CellTags->"LorentzIndex"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"LorentzIndex", "[", "\[Alpha]", "]"}]], "Input",
 CellTags->"LorentzIndex",
 CellLabel->"In[1126]:="],

Cell[BoxData[
 FormBox["\[Alpha]", TraditionalForm]], "Output",
 CellChangeTimes->{3.500970320460765*^9},
 CellTags->"LorentzIndex",
 CellLabel->"Out[1126]="]
}, Open  ]],

Cell["\<\
An optional second argument can be given for a dimension different from 4.\
\>", "Text",
 CellTags->"LorentzIndex"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"LorentzIndex", "[", 
  RowBox[{"\[Alpha]", ",", "n"}], "]"}]], "Input",
 CellTags->"LorentzIndex",
 CellLabel->"In[1127]:="],

Cell[BoxData[
 FormBox["\[Alpha]", TraditionalForm]], "Output",
 CellChangeTimes->{3.500970320503141*^9},
 CellTags->"LorentzIndex",
 CellLabel->"Out[1127]="]
}, Open  ]],

Cell[CellGroupData[{

Cell["LorentzIndex[ComplexIndex[\[Alpha]]]", "Input",
 CellTags->"LorentzIndex",
 CellLabel->"In[1128]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox[
   RowBox[{
    RowBox[{"\[UHat]", "\.03"}], "\[PlusMinus]"}], "*"], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970320590584*^9},
 CellTags->"LorentzIndex",
 CellLabel->"Out[1128]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ComplexConjugate", "[", 
  RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], "]"}]], "Input",
 CellTags->"LorentzIndex",
 CellLabel->"In[1129]:="],

Cell[BoxData[
 FormBox["\[Mu]", TraditionalForm]], "Output",
 CellChangeTimes->{3.50097032091199*^9},
 CellTags->"LorentzIndex",
 CellLabel->"Out[1129]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ComplexConjugate", "[", 
  RowBox[{"LorentzIndex", "[", 
   RowBox[{"ComplexIndex", "[", "\[Beta]", "]"}], "]"}], "]"}]], "Input",
 CellTags->"LorentzIndex",
 CellLabel->"In[1130]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["\[Beta]", "*"], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970321066553*^9},
 CellTags->"LorentzIndex",
 CellLabel->"Out[1130]="]
}, Open  ]],

Cell["\<\
Setting $LorentzIndices=True displays the dimension (if different from 4) as \
a subindex.\
\>", "Text",
 CellTags->"LorentzIndex"],

Cell[BoxData[
 RowBox[{
  RowBox[{"$LorentzIndices", "=", "True"}], ";"}]], "Input",
 CellTags->"LorentzIndex",
 CellLabel->"In[1131]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"LorentzIndex", "[", 
    RowBox[{"\[Alpha]", ",", "D"}], "]"}], ",", 
   RowBox[{"LorentzIndex", "[", 
    RowBox[{"\[Nu]", ",", "n"}], "]"}], ",", 
   RowBox[{"LorentzIndex", "[", 
    RowBox[{"\[Beta]", ",", "4"}], "]"}]}], "}"}]], "Input",
 CellTags->"LorentzIndex",
 CellLabel->"In[1132]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    SubscriptBox[
     FormBox["\[Alpha]",
      TraditionalForm], 
     FormBox["D",
      TraditionalForm]], ",", 
    SubscriptBox[
     FormBox["\[Nu]",
      TraditionalForm], 
     FormBox["n",
      TraditionalForm]], ",", "\[Beta]"}], "}"}], TraditionalForm]], "Output",\

 CellChangeTimes->{3.5009703212250643`*^9},
 CellTags->"LorentzIndex",
 CellLabel->"Out[1132]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"$LorentzIndices", " ", "=", " ", "False"}], ";"}]], "Input",
 CellTags->"LorentzIndex",
 CellLabel->"In[1133]:="]
}, Open  ]],

Cell["Lower", "ObjectName",
 CellTags->"Lower"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Lower"],

Cell["\<\
Lower may be used inside LorentzIndex to indicate an covariant LorentzIndex.\
\>", "Text",
 CellTags->"Lower"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["LorentzIndex",
  BaseStyle->"AddOnsLink",
  ButtonData:>"LorentzIndex",
  ButtonNote->"LorentzIndex"],
 ", ",
 ButtonBox["Upper",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Upper",
  ButtonNote->"Upper"],
 ", ",
 ButtonBox["Contract1",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Contract1",
  ButtonNote->"Contract1"],
 "."
}], "Text",
 CellTags->"Lower"]
}, Open  ]],

Cell["MakeContext", "ObjectName",
 CellTags->"MakeContext"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"MakeContext"],

Cell["\<\
MakeContext[string] constructs the context path of string. MakeContext is \
invoked at startup of FeynCalc. MakeContext[a, b] construct the context path \
of b defined in the context of a.\
\>", "Text",
 CellTags->"MakeContext"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FeynCalc",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FeynCalc",
  ButtonNote->"FeynCalc"],
 ", ",
 ButtonBox["CheckContext",
  BaseStyle->"AddOnsLink",
  ButtonData:>"CheckContext",
  ButtonNote->"CheckContext"],
 ", ",
 ButtonBox["$FeynCalcStuff",
  BaseStyle->"AddOnsLink",
  ButtonData:>"$FeynCalcStuff",
  ButtonNote->"$FeynCalcStuff"],
 ", ",
 ButtonBox["Load",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Load",
  ButtonNote->"Load"],
 "."
}], "Text",
 CellTags->"MakeContext"]
}, Open  ]],

Cell["Mandelstam", "ObjectName",
 CellTags->"Mandelstam"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Mandelstam"],

Cell[TextData[{
 "Mandelstam is an option for DiracTrace, OneLoop, OneLoopSum, Tr and \
TrickMandelstam.  A typical setting is Mandelstam \[Rule] {",
 "s",
 ", ",
 "t",
 ", ",
 "u",
 ", ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    SubscriptBox[
     StyleBox["m",
      FontSlant->"Plain"], "1"], "2"], TraditionalForm]]],
 " + ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    SubscriptBox[
     StyleBox["m",
      FontSlant->"Plain"], "2"], "2"], TraditionalForm]]],
 " + ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    SubscriptBox[
     StyleBox["m",
      FontSlant->"Plain"], "3"], "2"], TraditionalForm]]],
 " + ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    SubscriptBox[
     StyleBox["m",
      FontSlant->"Plain"], "4"], "2"], TraditionalForm]]],
 "}, which stands for  ",
 "s",
 " + ",
 "t",
 " + ",
 "u",
 " = ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    SubscriptBox[
     StyleBox["m",
      FontSlant->"Plain"], "1"], "2"], TraditionalForm]]],
 " + ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    SubscriptBox[
     StyleBox["m",
      FontSlant->"Plain"], "2"], "2"], TraditionalForm]]],
 " + ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    SubscriptBox[
     StyleBox["m",
      FontSlant->"Plain"], "3"], "2"], TraditionalForm]]],
 " + ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    SubscriptBox[
     StyleBox["m",
      FontSlant->"Plain"], "4"], "2"], TraditionalForm]]],
 ". If other than four-particle processes are calculated the setting should \
be: Mandelstam \[Rule] {}."
}], "Text",
 CellTags->"Mandelstam"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["DiracTrace",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracTrace",
  ButtonNote->"DiracTrace"],
 ", ",
 ButtonBox["OneLoop",
  BaseStyle->"AddOnsLink",
  ButtonData:>"OneLoop",
  ButtonNote->"OneLoop"],
 ", ",
 ButtonBox["OneLoopSum",
  BaseStyle->"AddOnsLink",
  ButtonData:>"OneLoopSum",
  ButtonNote->"OneLoopSum"],
 ", ",
 ButtonBox["Tr",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Tr",
  ButtonNote->"Tr"],
 ", ",
 ButtonBox["TrickMandelstam",
  BaseStyle->"AddOnsLink",
  ButtonData:>"TrickMandelstam",
  ButtonNote->"TrickMandelstam"],
 ", ",
 ButtonBox["SetMandelstam",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SetMandelstam",
  ButtonNote->"SetMandelstam"],
 "."
}], "Text",
 CellTags->"Mandelstam"]
}, Open  ]],

Cell["Map2", "ObjectName",
 CellTags->"Map2"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Map2"],

Cell["\<\
Map2[f, exp] is equivalent to Map if Nterms[exp] > 0, otherwise Map2[f, exp] \
gives f[exp].\
\>", "Text",
 CellTags->"Map2"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["NTerms",
  BaseStyle->"AddOnsLink",
  ButtonData:>"NTerms",
  ButtonNote->"NTerms"],
 "."
}], "Text",
 CellTags->"Map2"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Map2"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Map2", "[", 
  RowBox[{"f", ",", 
   RowBox[{"a", "-", "b"}]}], "]"}]], "Input",
 CellTags->"Map2",
 CellLabel->"In[1134]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"f", "(", "a", ")"}], "+", 
   RowBox[{"f", "(", 
    RowBox[{"-", "b"}], ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970321319818*^9},
 CellTags->"Map2",
 CellLabel->"Out[1134]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Map2", "[", 
  RowBox[{"f", ",", "x"}], "]"}]], "Input",
 CellTags->"Map2",
 CellLabel->"In[1135]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"f", "(", "x", ")"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970321364662*^9},
 CellTags->"Map2",
 CellLabel->"Out[1135]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Map2", "[", 
  RowBox[{"f", ",", 
   RowBox[{"{", 
    RowBox[{"a", ",", "b", ",", "c"}], "}"}]}], "]"}]], "Input",
 CellTags->"Map2",
 CellLabel->"In[1136]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"f", "(", 
   RowBox[{"{", 
    RowBox[{"a", ",", "b", ",", "c"}], "}"}], ")"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.5009703214129457`*^9},
 CellTags->"Map2",
 CellLabel->"Out[1136]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Map2", "[", 
  RowBox[{"f", ",", "1"}], "]"}]], "Input",
 CellTags->"Map2",
 CellLabel->"In[1137]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"f", "(", "1", ")"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970321456484*^9},
 CellTags->"Map2",
 CellLabel->"Out[1137]="]
}, Open  ]]
}, Open  ]],

Cell["MemoryAvailable", "ObjectName",
 CellTags->"MemoryAvailable"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"MemoryAvailable"],

Cell["\<\
MemoryAvailable is an option of MemSet. It can be set to an integer n, where \
n is the available amount of main memory in Mega Byte. The default setting is \
$MemoryAvailable.\
\>", "Text",
 CellTags->"MemoryAvailable"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["MemSet",
  BaseStyle->"AddOnsLink",
  ButtonData:>"MemSet",
  ButtonNote->"MemSet"],
 ", ",
 ButtonBox["$MemoryAvailable",
  BaseStyle->"AddOnsLink",
  ButtonData:>"$MemoryAvailable",
  ButtonNote->"$MemoryAvailable"],
 "."
}], "Text",
 CellTags->"MemoryAvailable"]
}, Open  ]],

Cell["MemSet", "ObjectName",
 CellTags->"MemSet"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"MemSet"],

Cell[TextData[{
 "MemSet[f[x_], body] is like f[x_] := f[x] = body, but dependend on the \
value of the setting of MemoryAvailable \[Rule] memorycut (memorycut - \
MemoryInUse[]/",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["10", "6"], TraditionalForm]]],
 ") MemSet[f[x_], body] may evaluate as f[x_] := body."
}], "Text",
 CellTags->"MemSet"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["MemoryAvailable",
  BaseStyle->"AddOnsLink",
  ButtonData:>"MemoryAvailable",
  ButtonNote->"MemoryAvailable"],
 "."
}], "Text",
 CellTags->"MemSet"]
}, Open  ]],

Cell["MetricTensor", "ObjectName",
 CellTags->"MetricTensor"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"MetricTensor"],

Cell[TextData[{
 "MetricTensor[",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Mu]", ",", " ", "\[Nu]"}], TraditionalForm]]],
 "] is the metric tensor. The default dimension is 4."
}], "Text",
 CellTags->"MetricTensor"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FeynCalcExternal",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FeynCalcExternal",
  ButtonNote->"FeynCalcExternal"],
 ", ",
 ButtonBox["FCE",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FCE",
  ButtonNote->"FCE"],
 ", ",
 ButtonBox["FCI",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FCI",
  ButtonNote->"FCI"],
 ", ",
 ButtonBox["MT",
  BaseStyle->"AddOnsLink",
  ButtonData:>"MT",
  ButtonNote->"MT"],
 ", ",
 ButtonBox["MTD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"MTD",
  ButtonNote->"MTD"],
 "."
}], "Text",
 CellTags->"MetricTensor"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"MetricTensor"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"MetricTensor", "[", 
  RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}]], "Input",
 CellTags->"MetricTensor",
 CellLabel->"In[1138]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["g", 
   RowBox[{"\[Alpha]", "\[Beta]"}]], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970321532753*^9},
 CellTags->"MetricTensor",
 CellLabel->"Out[1138]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Contract", "[", 
  RowBox[{"%", " ", "%"}], "]"}]], "Input",
 CellTags->"MetricTensor",
 CellLabel->"In[1139]:="],

Cell[BoxData[
 FormBox["4", TraditionalForm]], "Output",
 CellChangeTimes->{3.500970321910404*^9},
 CellTags->"MetricTensor",
 CellLabel->"Out[1139]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"MetricTensor", "[", 
  RowBox[{"\[Alpha]", ",", "\[Beta]", ",", 
   RowBox[{"Dimension", "\[Rule]", "n"}]}], "]"}]], "Input",
 CellTags->"MetricTensor",
 CellLabel->"In[1140]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["g", 
   RowBox[{"\[Alpha]", "\[Beta]"}]], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970321962736*^9},
 CellTags->"MetricTensor",
 CellLabel->"Out[1140]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Contract", "[", 
  RowBox[{"%", " ", "%"}], "]"}]], "Input",
 CellTags->"MetricTensor",
 CellLabel->"In[1141]:="],

Cell[BoxData[
 FormBox["n", TraditionalForm]], "Output",
 CellChangeTimes->{3.5009703220084867`*^9},
 CellTags->"MetricTensor",
 CellLabel->"Out[1141]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StandardForm", "[", 
  RowBox[{"MetricTensor", "[", 
   RowBox[{"a", ",", "b"}], "]"}], "]"}]], "Input",
 CellTags->"MetricTensor",
 CellLabel->"In[1142]:="],

Cell[BoxData[
 RowBox[{"MetricTensor", "[", 
  RowBox[{"a", ",", "b"}], "]"}]], "Output",
 CellChangeTimes->{3.500970322094965*^9},
 CellTags->"MetricTensor",
 CellLabel->"Out[1142]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StandardForm", "[", 
  RowBox[{"MetricTensor", "[", 
   RowBox[{"a", ",", "b", ",", 
    RowBox[{"Dimension", "\[Rule]", "D"}]}], "]"}], "]"}]], "Input",
 CellTags->"MetricTensor",
 CellLabel->"In[1143]:="],

Cell[BoxData[
 RowBox[{"MetricTensor", "[", 
  RowBox[{"a", ",", "b", ",", 
   RowBox[{"Dimension", "\[Rule]", "D"}]}], "]"}]], "Output",
 CellChangeTimes->{3.500970322143527*^9},
 CellTags->"MetricTensor",
 CellLabel->"Out[1143]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StandardForm", "[", 
  RowBox[{"FCE", "[", 
   RowBox[{"MetricTensor", "[", 
    RowBox[{"a", ",", "b"}], "]"}], "]"}], "]"}]], "Input",
 CellTags->"MetricTensor",
 CellLabel->"In[1144]:="],

Cell[BoxData[
 RowBox[{"HighEnergyPhysics`fctools`FeynCalcExternal`Private`metricmul", "[", 
  
  RowBox[{"a", ",", "b"}], "]"}]], "Output",
 CellChangeTimes->{3.5009703222331142`*^9},
 CellTags->"MetricTensor",
 CellLabel->"Out[1144]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StandardForm", "[", 
  RowBox[{"FCE", "[", 
   RowBox[{"MetricTensor", "[", 
    RowBox[{"a", ",", "b", ",", 
     RowBox[{"Dimension", "\[Rule]", "D"}]}], "]"}], "]"}], "]"}]], "Input",
 CellTags->"MetricTensor",
 CellLabel->"In[1145]:="],

Cell[BoxData[
 RowBox[{"HighEnergyPhysics`fctools`FeynCalcExternal`Private`metricmul", "[", 
  
  RowBox[{"a", ",", "b", ",", 
   RowBox[{"Dimension", "\[Rule]", "D"}]}], "]"}]], "Output",
 CellChangeTimes->{3.500970322289361*^9},
 CellTags->"MetricTensor",
 CellLabel->"Out[1145]//StandardForm="]
}, Open  ]]
}, Open  ]],

Cell["MLimit", "ObjectName",
 CellTags->"MLimit"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"MLimit"],

Cell["\<\
MLimit[expr, lims] takes multiple limits of expr using the limits lims.\
\>", "Text",
 CellTags->"MLimit"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"MLimit"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"MLimit", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"y", " ", 
     RowBox[{"Log", "[", "y", "]"}]}], "+", 
    RowBox[{
     RowBox[{"Sin", "[", 
      RowBox[{"x", "-", "1"}], "]"}], "/", 
     RowBox[{"(", 
      RowBox[{"x", "-", "1"}], ")"}]}]}], ",", 
   RowBox[{"x", "\[Rule]", "1"}], ",", 
   RowBox[{"y", "\[Rule]", "0"}]}], "]"}]], "Input",
 CellTags->"MLimit",
 CellLabel->"In[1146]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"MLimit", "(", 
   RowBox[{
    RowBox[{
     RowBox[{"y", " ", 
      RowBox[{"log", "(", "y", ")"}]}], "-", 
     FractionBox[
      RowBox[{"sin", "(", 
       RowBox[{"1", "-", "x"}], ")"}], 
      RowBox[{"x", "-", "1"}]]}], ",", 
    RowBox[{"x", "\[Rule]", "1"}], ",", 
    RowBox[{"y", "\[Rule]", "0"}]}], ")"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970322398903*^9},
 CellTags->"MLimit",
 CellLabel->"Out[1146]="]
}, Open  ]]
}, Open  ]],

Cell["Momentum", "ObjectName",
 CellTags->"Momentum"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Momentum"],

Cell["\<\
Momentum[p] is the head of a four momentum (p). The internal representation \
of a four-dimensional p is Momentum[p]. For other than four dimensions: \
Momentum[p, dim]. Momentum[p, 4] simplifies to Momentum[p].\
\>", "Text",
 CellTags->"Momentum"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["DiracGamma",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracGamma",
  ButtonNote->"DiracGamma"],
 ", ",
 ButtonBox["Eps",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Eps",
  ButtonNote->"Eps"],
 ", ",
 ButtonBox["LorentzIndex",
  BaseStyle->"AddOnsLink",
  ButtonData:>"LorentzIndex",
  ButtonNote->"LorentzIndex"],
 ", ",
 ButtonBox["MomentumExpand",
  BaseStyle->"AddOnsLink",
  ButtonData:>"MomentumExpand",
  ButtonNote->"MomentumExpand"],
 "."
}], "Text",
 CellTags->"Momentum"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Momentum"],

Cell["This is a four dimensional momentum.", "Text",
 CellTags->"Momentum"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Momentum", "[", "p", "]"}]], "Input",
 CellTags->"Momentum",
 CellLabel->"In[1147]:="],

Cell[BoxData[
 FormBox["p", TraditionalForm]], "Output",
 CellChangeTimes->{3.500970322449018*^9},
 CellTags->"Momentum",
 CellLabel->"Out[1147]="]
}, Open  ]],

Cell["\<\
As an optional second argument the dimension must be specified if it is \
different from 4.\
\>", "Text",
 CellTags->"Momentum"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Momentum", "[", 
  RowBox[{"p", ",", "D"}], "]"}]], "Input",
 CellTags->"Momentum",
 CellLabel->"In[1148]:="],

Cell[BoxData[
 FormBox["p", TraditionalForm]], "Output",
 CellChangeTimes->{3.500970322493381*^9},
 CellTags->"Momentum",
 CellLabel->"Out[1148]="]
}, Open  ]],

Cell["The dimension index is supressed in the output.", "Text",
 CellTags->"Momentum"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Momentum", "[", 
   RowBox[{"p", ",", "d"}], "]"}], "//", "StandardForm"}]], "Input",
 CellTags->"Momentum",
 CellLabel->"In[1149]:="],

Cell[BoxData[
 RowBox[{"Momentum", "[", 
  RowBox[{"p", ",", "d"}], "]"}]], "Output",
 CellChangeTimes->{3.500970322572586*^9},
 CellTags->"Momentum",
 CellLabel->"Out[1149]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"a1", "=", 
  RowBox[{"Momentum", "[", 
   RowBox[{"-", "q"}], "]"}]}]], "Input",
 CellTags->"Momentum",
 CellLabel->"In[1150]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", "q"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970322937718*^9},
 CellTags->"Momentum",
 CellLabel->"Out[1150]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"a1", "//", "StandardForm"}]], "Input",
 CellTags->"Momentum",
 CellLabel->"In[1151]:="],

Cell[BoxData[
 RowBox[{"-", 
  RowBox[{"Momentum", "[", "q", "]"}]}]], "Output",
 CellChangeTimes->{3.500970322981546*^9},
 CellTags->"Momentum",
 CellLabel->"Out[1151]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"a2", " ", "=", " ", 
  RowBox[{
   RowBox[{"Momentum", "[", 
    RowBox[{"p", "-", "q"}], "]"}], " ", "+", " ", 
   RowBox[{"Momentum", "[", 
    RowBox[{"2", "q"}], "]"}]}]}]], "Input",
 CellTags->"Momentum",
 CellLabel->"In[1152]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"(", 
    RowBox[{"p", "-", "q"}], ")"}], "+", 
   RowBox[{"2", " ", "q"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.50097032306804*^9},
 CellTags->"Momentum",
 CellLabel->"Out[1152]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"a2", "//", "StandardForm"}]], "Input",
 CellTags->"Momentum",
 CellLabel->"In[1153]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"Momentum", "[", 
   RowBox[{"p", "-", "q"}], "]"}], "+", 
  RowBox[{"2", " ", 
   RowBox[{"Momentum", "[", "q", "]"}]}]}]], "Output",
 CellChangeTimes->{3.500970323111817*^9},
 CellTags->"Momentum",
 CellLabel->"Out[1153]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"a2", "//", "MomentumExpand"}], "//", "StandardForm"}]], "Input",
 CellTags->"Momentum",
 CellLabel->"In[1154]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"Momentum", "[", "p", "]"}], "+", 
  RowBox[{"Momentum", "[", "q", "]"}]}]], "Output",
 CellChangeTimes->{3.500970323155884*^9},
 CellTags->"Momentum",
 CellLabel->"Out[1154]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"a2", "//", "MomentumCombine"}], "//", "StandardForm"}]], "Input",
 CellTags->"Momentum",
 CellLabel->"In[1155]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"Momentum", "[", 
   RowBox[{"p", "-", "q"}], "]"}], "+", 
  RowBox[{"2", " ", 
   RowBox[{"Momentum", "[", "q", "]"}]}]}]], "Output",
 CellChangeTimes->{3.500970323199641*^9},
 CellTags->"Momentum",
 CellLabel->"Out[1155]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"ChangeDimension", "[", 
   RowBox[{
    RowBox[{"Momentum", "[", "p", "]"}], ",", "d"}], "]"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"Momentum",
 CellLabel->"In[1156]:="],

Cell[BoxData[
 RowBox[{"Momentum", "[", 
  RowBox[{"p", ",", "d"}], "]"}]], "Output",
 CellChangeTimes->{3.500970323279142*^9},
 CellTags->"Momentum",
 CellLabel->"Out[1156]//StandardForm="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"Clear", "[", 
  RowBox[{"a1", ",", "a2"}], "]"}]], "Input",
 CellTags->"Momentum",
 CellLabel->"In[1157]:="]
}, Open  ]],

Cell["MomentumCombine", "ObjectName",
 CellTags->"MomentumCombine"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"MomentumCombine"],

Cell["\<\
MomentumCombine[expr] is the inverse operation to MomentumExpand and \
ExpandScalarProduct. MomentumCombine combines also Pair`s.\
\>", "Text",
 CellTags->"MomentumCombine"],

Cell[TextData[{
 "See also: ",
 ButtonBox["ExpandScalarProduct",
  BaseStyle->"AddOnsLink",
  ButtonData:>"ExpandScalarProduct",
  ButtonNote->"ExpandScalarProduct"],
 ", ",
 ButtonBox["Momentum",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Momentum",
  ButtonNote->"Momentum"],
 ", ",
 ButtonBox["MomentumExpand",
  BaseStyle->"AddOnsLink",
  ButtonData:>"MomentumExpand",
  ButtonNote->"MomentumExpand"],
 ", ",
 ButtonBox["MomentumCombine2",
  BaseStyle->"AddOnsLink",
  ButtonData:>"MomentumCombine2",
  ButtonNote->"MomentumCombine2"],
 "."
}], "Text",
 CellTags->"MomentumCombine"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"MomentumCombine"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Momentum", "[", "p", "]"}], "-", 
    RowBox[{"2", " ", 
     RowBox[{"Momentum", "[", "q", "]"}]}]}], " ", "//", " ", 
   "MomentumCombine"}], " ", "//", " ", "StandardForm"}]], "Input",
 CellTags->"MomentumCombine",
 CellLabel->"In[1158]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"Momentum", "[", "p", "]"}], "-", 
  RowBox[{"2", " ", 
   RowBox[{"Momentum", "[", "q", "]"}]}]}]], "Output",
 CellChangeTimes->{3.5009703233994207`*^9},
 CellTags->"MomentumCombine",
 CellLabel->"Out[1158]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t1", "=", 
  RowBox[{
   RowBox[{"FV", "[", 
    RowBox[{"p", ",", "\[Mu]"}], "]"}], " ", "+", " ", 
   RowBox[{"2", " ", 
    RowBox[{"FV", "[", 
     RowBox[{"q", ",", "\[Mu]"}], "]"}]}], " "}]}]], "Input",
 CellTags->"MomentumCombine",
 CellLabel->"In[1159]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["p", "\[Mu]"], "+", 
   RowBox[{"2", " ", 
    SuperscriptBox["q", "\[Mu]"]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970323486446*^9},
 CellTags->"MomentumCombine",
 CellLabel->"Out[1159]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"MomentumCombine", "[", "t1", "]"}]], "Input",
 CellTags->"MomentumCombine",
 CellLabel->"In[1160]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["p", "\[Mu]"], "+", 
   RowBox[{"2", " ", 
    SuperscriptBox["q", "\[Mu]"]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970323599244*^9},
 CellTags->"MomentumCombine",
 CellLabel->"Out[1160]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"MomentumCombine", "[", "t1", "]"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"MomentumCombine",
 CellLabel->"In[1161]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"FV", "[", 
   RowBox[{"p", ",", "\[Mu]"}], "]"}], "+", 
  RowBox[{"2", " ", 
   RowBox[{"FV", "[", 
    RowBox[{"q", ",", "\[Mu]"}], "]"}]}]}]], "Output",
 CellChangeTimes->{3.5009703238746147`*^9},
 CellTags->"MomentumCombine",
 CellLabel->"Out[1161]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"MomentumCombine", "[", "t1", "]"}], "//", 
  "ExpandScalarProduct"}]], "Input",
 CellTags->"MomentumCombine",
 CellLabel->"In[1162]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["p", "\[Mu]"], "+", 
   RowBox[{"2", " ", 
    SuperscriptBox["q", "\[Mu]"]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970324030452*^9},
 CellTags->"MomentumCombine",
 CellLabel->"Out[1162]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StandardForm", "[", "%", "]"}]], "Input",
 CellTags->"MomentumCombine",
 CellLabel->"In[1163]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"FV", "[", 
   RowBox[{"p", ",", "\[Mu]"}], "]"}], "+", 
  RowBox[{"2", " ", 
   RowBox[{"FV", "[", 
    RowBox[{"q", ",", "\[Mu]"}], "]"}]}]}]], "Output",
 CellChangeTimes->{3.500970324162573*^9},
 CellTags->"MomentumCombine",
 CellLabel->"Out[1163]//StandardForm="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"Clear", "[", "t1", "]"}]], "Input",
 CellTags->"MomentumCombine",
 CellLabel->"In[1164]:="]
}, Open  ]],

Cell["MomentumCombine2", "ObjectName",
 CellTags->"MomentumCombine2"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"MomentumCombine2"],

Cell["\<\
MomentumCombine2[expr]  is the inverse operation to MomentumExpand and \
ExpandScalarProduct. MomentumCombine2 combines also FourVectors.\
\>", "Text",
 CellTags->"MomentumCombine2"],

Cell[TextData[{
 "See also: ",
 ButtonBox["MomentumCombine",
  BaseStyle->"AddOnsLink",
  ButtonData:>"MomentumCombine",
  ButtonNote->"MomentumCombine"],
 ", ",
 ButtonBox["MomentumExpand",
  BaseStyle->"AddOnsLink",
  ButtonData:>"MomentumExpand",
  ButtonNote->"MomentumExpand"],
 ", ",
 ButtonBox["ExpandScalarProduct",
  BaseStyle->"AddOnsLink",
  ButtonData:>"ExpandScalarProduct",
  ButtonNote->"ExpandScalarProduct"],
 ", ",
 ButtonBox["FourVector",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FourVector",
  ButtonNote->"FourVector"],
 "."
}], "Text",
 CellTags->"MomentumCombine2"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"MomentumCombine2"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"3", " ", 
   RowBox[{"Pair", "[", 
    RowBox[{
     RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
     RowBox[{"Momentum", "[", "p", "]"}]}], "]"}]}], "+", 
  RowBox[{"2", " ", 
   RowBox[{"Pair", "[", 
    RowBox[{
     RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
     RowBox[{"Momentum", "[", "q", "]"}]}], "]"}]}]}]], "Input",
 CellTags->"MomentumCombine2",
 CellLabel->"In[1165]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"3", " ", 
    SuperscriptBox[
     FormBox["p",
      TraditionalForm], 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]]}], "+", 
   RowBox[{"2", " ", 
    SuperscriptBox[
     FormBox["q",
      TraditionalForm], 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.50097032425915*^9},
 CellTags->"MomentumCombine2",
 CellLabel->"Out[1165]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"MomentumCombine2", "[", "%", "]"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"MomentumCombine2",
 CellLabel->"In[1166]:="],

Cell[BoxData[
 RowBox[{"Pair", "[", 
  RowBox[{
   RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
   RowBox[{"Momentum", "[", 
    RowBox[{
     RowBox[{"3", " ", "p"}], "+", 
     RowBox[{"2", " ", "q"}]}], "]"}]}], "]"}]], "Output",
 CellChangeTimes->{3.500970324342741*^9},
 CellTags->"MomentumCombine2",
 CellLabel->"Out[1166]//StandardForm="]
}, Open  ]]
}, Open  ]],

Cell["MomentumExpand", "ObjectName",
 CellTags->"MomentumExpand"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"MomentumExpand"],

Cell["\<\
MomentumExpand[expr] expands Momentum[a+b+ ...] in expr into Momentum[a] + \
Momentum[b] + ... .\
\>", "Text",
 CellTags->"MomentumExpand"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["ExpandScalarProduct",
  BaseStyle->"AddOnsLink",
  ButtonData:>"ExpandScalarProduct",
  ButtonNote->"ExpandScalarProduct"],
 ", ",
 ButtonBox["MomentumCombine",
  BaseStyle->"AddOnsLink",
  ButtonData:>"MomentumCombine",
  ButtonNote->"MomentumCombine"],
 "."
}], "Text",
 CellTags->"MomentumExpand"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"MomentumExpand"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"MomentumExpand", "[", 
   RowBox[{"Momentum", "[", 
    RowBox[{"p", "+", "q"}], "]"}], "]"}], "//", "StandardForm"}]], "Input",
 CellTags->"MomentumExpand",
 CellLabel->"In[1167]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"Momentum", "[", "p", "]"}], "+", 
  RowBox[{"Momentum", "[", "q", "]"}]}]], "Output",
 CellChangeTimes->{3.500970324387044*^9},
 CellTags->"MomentumExpand",
 CellLabel->"Out[1167]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ScalarProduct", "[", 
  RowBox[{
   RowBox[{"p", "+", "q"}], ",", "r"}], "]"}]], "Input",
 CellTags->"MomentumExpand",
 CellLabel->"In[1168]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"(", 
   FormBox[
    RowBox[{"p", "+", "q"}],
    TraditionalForm], ")", "\[CenterDot]", 
   FormBox["r",
    TraditionalForm]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009703244638987`*^9},
 CellTags->"MomentumExpand",
 CellLabel->"Out[1168]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "StandardForm"}]], "Input",
 CellTags->"MomentumExpand",
 CellLabel->"In[1169]:="],

Cell[BoxData[
 RowBox[{"Pair", "[", 
  RowBox[{
   RowBox[{"Momentum", "[", 
    RowBox[{"p", "+", "q"}], "]"}], ",", 
   RowBox[{"Momentum", "[", "r", "]"}]}], "]"}]], "Output",
 CellChangeTimes->{3.500970324508828*^9},
 CellTags->"MomentumExpand",
 CellLabel->"Out[1169]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"MomentumExpand", "[", 
  RowBox[{"ScalarProduct", "[", 
   RowBox[{
    RowBox[{"p", "+", "q"}], ",", "r"}], "]"}], "]"}]], "Input",
 CellTags->"MomentumExpand",
 CellLabel->"In[1170]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"(", 
   TagBox[
    FormBox[
     RowBox[{"p", "+", "q"}],
     TraditionalForm],
    TraditionalForm,
    Editable->True], ")", ".", "(", 
   TagBox[
    FormBox["r",
     TraditionalForm],
    TraditionalForm,
    Editable->True], ")"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970324620229*^9},
 CellTags->"MomentumExpand",
 CellLabel->"Out[1170]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "StandardForm"}]], "Input",
 CellTags->"MomentumExpand",
 CellLabel->"In[1171]:="],

Cell[BoxData[
 RowBox[{"Pair", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"Momentum", "[", "p", "]"}], "+", 
    RowBox[{"Momentum", "[", "q", "]"}]}], ",", 
   RowBox[{"Momentum", "[", "r", "]"}]}], "]"}]], "Output",
 CellChangeTimes->{3.500970324906561*^9},
 CellTags->"MomentumExpand",
 CellLabel->"Out[1171]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"MomentumExpand", "[", 
  RowBox[{"ScalarProduct", "[", 
   RowBox[{
    RowBox[{"p", "+", "q"}], ",", 
    RowBox[{"r", "-", "p"}]}], "]"}], "]"}]], "Input",
 CellTags->"MomentumExpand",
 CellLabel->"In[1172]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"(", 
   TagBox[
    FormBox[
     RowBox[{"p", "+", "q"}],
     TraditionalForm],
    TraditionalForm,
    Editable->True], ")", ".", "(", 
   TagBox[
    FormBox[
     RowBox[{"r", "-", "p"}],
     TraditionalForm],
    TraditionalForm,
    Editable->True], ")"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009703249935837`*^9},
 CellTags->"MomentumExpand",
 CellLabel->"Out[1172]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "StandardForm"}]], "Input",
 CellTags->"MomentumExpand",
 CellLabel->"In[1173]:="],

Cell[BoxData[
 RowBox[{"Pair", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"Momentum", "[", "p", "]"}], "+", 
    RowBox[{"Momentum", "[", "q", "]"}]}], ",", 
   RowBox[{
    RowBox[{"-", 
     RowBox[{"Momentum", "[", "p", "]"}]}], "+", 
    RowBox[{"Momentum", "[", "r", "]"}]}]}], "]"}]], "Output",
 CellChangeTimes->{3.500970325037634*^9},
 CellTags->"MomentumExpand",
 CellLabel->"Out[1173]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Calc", "[", "%", "]"}]], "Input",
 CellTags->"MomentumExpand",
 CellLabel->"In[1174]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    RowBox[{
     FormBox["p",
      TraditionalForm], "\[CenterDot]", 
     FormBox["q",
      TraditionalForm]}]}], "+", 
   RowBox[{
    FormBox["p",
     TraditionalForm], "\[CenterDot]", 
    FormBox["r",
     TraditionalForm]}], "-", 
   SuperscriptBox[
    FormBox["p",
     TraditionalForm], "2"], "+", 
   RowBox[{
    FormBox["q",
     TraditionalForm], "\[CenterDot]", 
    FormBox["r",
     TraditionalForm]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970325152451*^9},
 CellTags->"MomentumExpand",
 CellLabel->"Out[1174]="]
}, Open  ]]
}, Open  ]],

Cell["MT", "ObjectName",
 CellTags->"MT"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"MT"],

Cell[TextData[{
 "MT[",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Mu]", ",", " ", "\[Nu]"}], TraditionalForm]]],
 "] is the metric tensor in 4 dimensions."
}], "Text",
 CellTags->"MT"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FeynCalcExternal",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FeynCalcExternal",
  ButtonNote->"FeynCalcExternal"],
 ", ",
 ButtonBox["FCE",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FCE",
  ButtonNote->"FCE"],
 ", ",
 ButtonBox["FCI",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FCI",
  ButtonNote->"FCI"],
 ", ",
 ButtonBox["MetricTensor",
  BaseStyle->"AddOnsLink",
  ButtonData:>"MetricTensor",
  ButtonNote->"MetricTensor"],
 ", ",
 ButtonBox["MTD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"MTD",
  ButtonNote->"MTD"],
 "."
}], "Text",
 CellTags->"MT"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"MT"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"MT", "[", 
  RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}]], "Input",
 CellTags->"MT",
 CellLabel->"In[1175]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["g", 
   RowBox[{"\[Alpha]", "\[Beta]"}]], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970325305566*^9},
 CellTags->"MT",
 CellLabel->"Out[1175]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Contract", "[", 
  RowBox[{
   RowBox[{"MT", "[", 
    RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}], " ", 
   RowBox[{"MT", "[", 
    RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}]}], "]"}]], "Input",
 CellTags->"MT",
 CellLabel->"In[1176]:="],

Cell[BoxData[
 FormBox["4", TraditionalForm]], "Output",
 CellChangeTimes->{3.500970325418046*^9},
 CellTags->"MT",
 CellLabel->"Out[1176]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"MT", "[", 
   RowBox[{"a", ",", "b"}], "]"}], "//", "StandardForm"}]], "Input",
 CellTags->"MT",
 CellLabel->"In[1177]:="],

Cell[BoxData[
 RowBox[{"MT", "[", 
  RowBox[{"a", ",", "b"}], "]"}]], "Output",
 CellChangeTimes->{3.5009703254614697`*^9},
 CellTags->"MT",
 CellLabel->"Out[1177]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"FCI", "[", 
   RowBox[{"MT", "[", 
    RowBox[{"a", ",", "b"}], "]"}], "]"}], "//", "StandardForm"}]], "Input",
 CellTags->"MT",
 CellLabel->"In[1178]:="],

Cell[BoxData[
 RowBox[{"Pair", "[", 
  RowBox[{
   RowBox[{"LorentzIndex", "[", "a", "]"}], ",", 
   RowBox[{"LorentzIndex", "[", "b", "]"}]}], "]"}]], "Output",
 CellChangeTimes->{3.500970325539385*^9},
 CellTags->"MT",
 CellLabel->"Out[1178]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"FCE", "[", 
   RowBox[{"FCI", "[", 
    RowBox[{"MT", "[", 
     RowBox[{"a", ",", "b"}], "]"}], "]"}], "]"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"MT",
 CellLabel->"In[1179]:="],

Cell[BoxData[
 RowBox[{"MT", "[", 
  RowBox[{"a", ",", "b"}], "]"}]], "Output",
 CellChangeTimes->{3.500970325630924*^9},
 CellTags->"MT",
 CellLabel->"Out[1179]//StandardForm="]
}, Open  ]]
}, Open  ]],

Cell["MTD", "ObjectName",
 CellTags->"MTD"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"MTD"],

Cell[TextData[{
 "MTD[",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Mu]", ",", " ", "\[Nu]"}], TraditionalForm]]],
 "] is the metric tensor in D dimensions."
}], "Text",
 CellTags->"MTD"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FeynCalcExternal",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FeynCalcExternal",
  ButtonNote->"FeynCalcExternal"],
 ", ",
 ButtonBox["FCE",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FCE",
  ButtonNote->"FCE"],
 ", ",
 ButtonBox["FCI",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FCI",
  ButtonNote->"FCI"],
 ", ",
 ButtonBox["MetricTensor",
  BaseStyle->"AddOnsLink",
  ButtonData:>"MetricTensor",
  ButtonNote->"MetricTensor"],
 ", ",
 ButtonBox["MT",
  BaseStyle->"AddOnsLink",
  ButtonData:>"MT",
  ButtonNote->"MT"],
 "."
}], "Text",
 CellTags->"MTD"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"MTD"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"MTD", "[", 
  RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}]], "Input",
 CellTags->"MTD",
 CellLabel->"In[1180]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["g", 
   RowBox[{"\[Alpha]", "\[Beta]"}]], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970325706506*^9},
 CellTags->"MTD",
 CellLabel->"Out[1180]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Contract", "[", 
  RowBox[{
   RowBox[{"MTD", "[", 
    RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}], " ", 
   RowBox[{"MTD", "[", 
    RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}]}], "]"}]], "Input",
 CellTags->"MTD",
 CellLabel->"In[1181]:="],

Cell[BoxData[
 FormBox["D", TraditionalForm]], "Output",
 CellChangeTimes->{3.5009703258006687`*^9},
 CellTags->"MTD",
 CellLabel->"Out[1181]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"MTD", "[", 
   RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"MTD",
 CellLabel->"In[1182]:="],

Cell[BoxData[
 RowBox[{"MTD", "[", 
  RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}]], "Output",
 CellChangeTimes->{3.500970325882538*^9},
 CellTags->"MTD",
 CellLabel->"Out[1182]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"FCI", "[", 
   RowBox[{"MTD", "[", 
    RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}], "]"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"MTD",
 CellLabel->"In[1183]:="],

Cell[BoxData[
 RowBox[{"Pair", "[", 
  RowBox[{
   RowBox[{"LorentzIndex", "[", 
    RowBox[{"\[Alpha]", ",", "D"}], "]"}], ",", 
   RowBox[{"LorentzIndex", "[", 
    RowBox[{"\[Beta]", ",", "D"}], "]"}]}], "]"}]], "Output",
 CellChangeTimes->{3.500970325963154*^9},
 CellTags->"MTD",
 CellLabel->"Out[1183]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"FCE", "[", 
   RowBox[{"FCI", "[", 
    RowBox[{"MTD", "[", 
     RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}], "]"}], "]"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"MTD",
 CellLabel->"In[1184]:="],

Cell[BoxData[
 RowBox[{"MTD", "[", 
  RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}]], "Output",
 CellChangeTimes->{3.500970326049754*^9},
 CellTags->"MTD",
 CellLabel->"Out[1184]//StandardForm="]
}, Open  ]]
}, Open  ]],

Cell["NegativeInteger", "ObjectName",
 CellTags->"NegativeInteger"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"NegativeInteger"],

Cell["\<\
NegativeInteger is a data type. E.g. DataType[n, NegativeInteger] can be set \
to True.\
\>", "Text",
 CellTags->"NegativeInteger"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["DataType",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DataType",
  ButtonNote->"DataType"],
 "."
}], "Text",
 CellTags->"NegativeInteger"]
}, Open  ]],

Cell["Nf", "ObjectName",
 CellTags->"Nf"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Nf"],

Cell["Nf denotes the number of flavors.", "Text",
 CellTags->"Nf"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Amplitude",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Amplitude",
  ButtonNote->"Amplitude"],
 ", ",
 ButtonBox["CounterTerm",
  BaseStyle->"AddOnsLink",
  ButtonData:>"CounterTerm",
  ButtonNote->"CounterTerm"],
 ", ",
 ButtonBox["Convolute",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Convolute",
  ButtonNote->"Convolute"],
 ", ",
 ButtonBox["AnomalousDimension",
  BaseStyle->"AddOnsLink",
  ButtonData:>"AnomalousDimension",
  ButtonNote->"AnomalousDimension"],
 ", ",
 ButtonBox["SplittingFunction",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SplittingFunction",
  ButtonNote->"SplittingFunction"],
 "."
}], "Text",
 CellTags->"Nf"]
}, Open  ]],

Cell["Nielsen", "ObjectName",
 CellTags->"Nielsen"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Nielsen"],

Cell["Nielsen[i,j, x] denotes Nielsen's polylogarithm.", "Text",
 CellTags->"Nielsen"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "Nielsen", "]"}]], "Input",
 CellTags->"Nielsen",
 CellLabel->"In[1185]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"PolyLog", "\[Rule]", "False"}], "}"}], TraditionalForm]], "Output",\

 CellChangeTimes->{3.500970326126276*^9},
 CellTags->"Nielsen",
 CellLabel->"Out[1185]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["SimplifyPolyLog",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SimplifyPolyLog",
  ButtonNote->"SimplifyPolyLog"],
 "."
}], "Text",
 CellTags->"Nielsen"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Nielsen"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Nielsen", "[", 
  RowBox[{"1", ",", "2", ",", "x"}], "]"}]], "Input",
 CellTags->"Nielsen",
 CellLabel->"In[1186]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["S", 
    RowBox[{
     FormBox["1",
      TraditionalForm], 
     FormBox["2",
      TraditionalForm]}]], "(", "x", ")"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970326176539*^9},
 CellTags->"Nielsen",
 CellLabel->"Out[1186]="]
}, Open  ]],

Cell["\<\
Numerical evaluation is done via N[Nielsen[n_,p_,x_]] := \
(-1)^(n+p-1)/(n-1)!/p! NIntegrate[Log[1-x t]^p Log[t]^(n-1)/t,{t,0,1}];\
\>", "Text",
 CellTags->"Nielsen"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"N", "[", 
  RowBox[{"Nielsen", "[", 
   RowBox[{"1", ",", "2", ",", ".45"}], "]"}], "]"}]], "Input",
 CellTags->"Nielsen",
 CellLabel->"In[1187]:="],

Cell[BoxData[
 FormBox["0.07287162612104658`", TraditionalForm]], "Output",
 CellChangeTimes->{3.5009703262840567`*^9},
 CellTags->"Nielsen",
 CellLabel->"Out[1187]="]
}, Open  ]],

Cell["Some special values are built in.", "Text",
 CellTags->"Nielsen"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"Nielsen", "[", 
    RowBox[{"1", ",", "2", ",", "0"}], "]"}], ",", 
   RowBox[{"Nielsen", "[", 
    RowBox[{"1", ",", "2", ",", 
     RowBox[{"-", "1"}]}], "]"}], ",", 
   RowBox[{"Nielsen", "[", 
    RowBox[{"1", ",", "2", ",", 
     RowBox[{"1", "/", "2"}]}], "]"}], ",", 
   RowBox[{"Nielsen", "[", 
    RowBox[{"1", ",", "2", ",", "1"}], "]"}]}], "}"}]], "Input",
 CellTags->"Nielsen",
 CellLabel->"In[1188]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"0", ",", 
    FractionBox[
     TemplateBox[{"3"},
      "Zeta"], "8"], ",", 
    FractionBox[
     TemplateBox[{"3"},
      "Zeta"], "8"], ",", 
    TemplateBox[{"3"},
     "Zeta"]}], "}"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970326433979*^9},
 CellTags->"Nielsen",
 CellLabel->"Out[1188]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Nielsen", "[", 
  RowBox[{"1", ",", "2", ",", "x", ",", 
   RowBox[{"PolyLog", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"Nielsen",
 CellLabel->"In[1189]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    TemplateBox[{"3",RowBox[{"1", "-", "x"}]},
     "PolyLog"]}], "+", 
   RowBox[{
    TemplateBox[{"2",RowBox[{"1", "-", "x"}]},
     "PolyLog"], " ", 
    RowBox[{"log", "(", 
     RowBox[{"1", "-", "x"}], ")"}]}], "+", 
   RowBox[{
    FractionBox["1", "2"], " ", 
    RowBox[{"log", "(", "x", ")"}], " ", 
    RowBox[{
     SuperscriptBox["log", "2"], "(", 
     RowBox[{"1", "-", "x"}], ")"}]}], "+", 
   TemplateBox[{"3"},
    "Zeta"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970326599015*^9},
 CellTags->"Nielsen",
 CellLabel->"Out[1189]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Nielsen", "[", 
  RowBox[{"1", ",", "3", ",", "x", ",", 
   RowBox[{"PolyLog", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"Nielsen",
 CellLabel->"In[1190]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    TemplateBox[{"4",RowBox[{"1", "-", "x"}]},
     "PolyLog"]}], "-", 
   RowBox[{
    FractionBox["1", "2"], " ", 
    TemplateBox[{"2",RowBox[{"1", "-", "x"}]},
     "PolyLog"], " ", 
    RowBox[{
     SuperscriptBox["log", "2"], "(", 
     RowBox[{"1", "-", "x"}], ")"}]}], "+", 
   RowBox[{
    TemplateBox[{"3",RowBox[{"1", "-", "x"}]},
     "PolyLog"], " ", 
    RowBox[{"log", "(", 
     RowBox[{"1", "-", "x"}], ")"}]}], "-", 
   RowBox[{
    FractionBox["1", "6"], " ", 
    RowBox[{"log", "(", "x", ")"}], " ", 
    RowBox[{
     SuperscriptBox["log", "3"], "(", 
     RowBox[{"1", "-", "x"}], ")"}]}], "+", 
   FractionBox[
    SuperscriptBox["\[Pi]", "4"], "90"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970326760972*^9},
 CellTags->"Nielsen",
 CellLabel->"Out[1190]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Nielsen", "[", 
  RowBox[{"3", ",", "1", ",", "x", ",", 
   RowBox[{"PolyLog", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"Nielsen",
 CellLabel->"In[1191]:="],

Cell[BoxData[
 FormBox[
  TemplateBox[{"4","x"},
   "PolyLog"], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970326954257*^9},
 CellTags->"Nielsen",
 CellLabel->"Out[1191]="]
}, Open  ]]
}, Open  ]],

Cell["NonCommFreeQ", "ObjectName",
 CellTags->"NonCommFreeQ"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"NonCommFreeQ"],

Cell["\<\
NonCommFreeQ[exp] yields True if exp contains no non-commutative objects \
(i.e. those objects which are listed in $NonComm) or only non-commutative \
objects inside DiracTrace's or SUNTrace's.\
\>", "Text",
 CellTags->"NonCommFreeQ"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["$NonComm",
  BaseStyle->"AddOnsLink",
  ButtonData:>"$NonComm",
  ButtonNote->"$NonComm"],
 ", ",
 ButtonBox["NonCommQ",
  BaseStyle->"AddOnsLink",
  ButtonData:>"NonCommQ",
  ButtonNote->"NonCommQ"],
 ", ",
 ButtonBox["DiracTrace",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracTrace",
  ButtonNote->"DiracTrace"],
 ", ",
 ButtonBox["SUNTrace",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SUNTrace",
  ButtonNote->"SUNTrace"],
 "."
}], "Text",
 CellTags->"NonCommFreeQ"]
}, Open  ]],

Cell["NonCommQ", "ObjectName",
 CellTags->"NonCommQ"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"NonCommQ"],

Cell["\<\
NonCommQ[exp] yields True if exp contains non-commutative objects (i.e. those \
objects which are listed in $NonComm) not inside DiracTrace's or SUNTrace's.\
\>", "Text",
 CellTags->"NonCommQ"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["$NonComm",
  BaseStyle->"AddOnsLink",
  ButtonData:>"$NonComm",
  ButtonNote->"$NonComm"],
 ", ",
 ButtonBox["NonCommFreeQ",
  BaseStyle->"AddOnsLink",
  ButtonData:>"NonCommFreeQ",
  ButtonNote->"NonCommFreeQ"],
 ", ",
 ButtonBox["DiracTrace",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracTrace",
  ButtonNote->"DiracTrace"],
 ", ",
 ButtonBox["SUNTrace",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SUNTrace",
  ButtonNote->"SUNTrace"],
 "."
}], "Text",
 CellTags->"NonCommQ"]
}, Open  ]],

Cell["NonCommutative", "ObjectName",
 CellTags->"NonCommutative"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"NonCommutative"],

Cell["\<\
NonCommutative is a data type which may be used, e.g.,  as: DataType[x, \
NonCommutative] = True.\
\>", "Text",
 CellTags->"NonCommutative"],

Cell[TextData[{
 "See also:",
 " ",
 ButtonBox["DataType",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DataType",
  ButtonNote->"DataType"],
 ", ",
 ButtonBox["DeclareNonCommutative",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DeclareNonCommutative",
  ButtonNote->"DeclareNonCommutative"],
 "."
}], "Text",
 CellTags->"NonCommutative"]
}, Open  ]],

Cell["NTerms", "ObjectName",
 CellTags->"NTerms"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"NTerms"],

Cell["\<\
NTerms[x] is equivalent to Length if x is a sum; otherwise NTerms[x] returns \
1, except NTerms[0] \[Rule] 0.\
\>", "Text",
 CellTags->"NTerms"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"NTerms"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"NTerms", "[", 
  RowBox[{"a", "-", "b"}], "]"}]], "Input",
 CellTags->"NTerms",
 CellLabel->"In[1192]:="],

Cell[BoxData[
 FormBox["2", TraditionalForm]], "Output",
 CellChangeTimes->{3.5009703270343037`*^9},
 CellTags->"NTerms",
 CellLabel->"Out[1192]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"NTerms", "[", 
  RowBox[{"a", " ", "b", " ", "c"}], "]"}]], "Input",
 CellTags->"NTerms",
 CellLabel->"In[1193]:="],

Cell[BoxData[
 FormBox["1", TraditionalForm]], "Output",
 CellChangeTimes->{3.500970327113619*^9},
 CellTags->"NTerms",
 CellLabel->"Out[1193]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"NTerms", "[", "9", "]"}]], "Input",
 CellTags->"NTerms",
 CellLabel->"In[1194]:="],

Cell[BoxData[
 FormBox["1", TraditionalForm]], "Output",
 CellChangeTimes->{3.5009703271546583`*^9},
 CellTags->"NTerms",
 CellLabel->"Out[1194]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"NTerms", "[", "0", "]"}]], "Input",
 CellTags->"NTerms",
 CellLabel->"In[1195]:="],

Cell[BoxData[
 FormBox["0", TraditionalForm]], "Output",
 CellChangeTimes->{3.500970327195912*^9},
 CellTags->"NTerms",
 CellLabel->"Out[1195]="]
}, Open  ]]
}, Open  ]],

Cell["NumberOfMetricTensors", "ObjectName",
 CellTags->"NumberOfMetricTensors"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"NumberOfMetricTensors"],

Cell["NumberOfMetricTensors is an option of Tdec.", "Text",
 CellTags->"NumberOfMetricTensors"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Tdec",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Tdec",
  ButtonNote->"Tdec"],
 "."
}], "Text",
 CellTags->"NonCommQ"]
}, Open  ]],

Cell["NumericalFactor", "ObjectName",
 CellTags->"NumericalFactor"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"NumericalFactor"],

Cell["\<\
NumericalFactor[expr] gives the overall numerical factor of expr.\
\>", "Text",
 CellTags->"NumericalFactor"]
}, Open  ]],

Cell["NumericQ1", "ObjectName",
 CellTags->"NumericQ1"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"NumericQ1"],

Cell["\<\
NumericQ1[x,{a,b,..}] is like NumericQ, but assumes that {a,b,..} are numeric \
quantities.\
\>", "Text",
 CellTags->"NumericQ1"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"NumericQ1"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"NumericQ", "[", 
  RowBox[{
   RowBox[{"3", " ", "a"}], "+", 
   RowBox[{"Log", "[", "b", "]"}], "+", 
   SuperscriptBox["c", "2"]}], "]"}]], "Input",
 CellTags->"NumericQ1",
 CellLabel->"In[1196]:="],

Cell[BoxData[
 FormBox["False", TraditionalForm]], "Output",
 CellChangeTimes->{3.500970327240448*^9},
 CellTags->"NumericQ1",
 CellLabel->"Out[1196]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"NumericQ1", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"3", " ", "a"}], "+", 
    RowBox[{"Log", "[", "b", "]"}], "+", 
    SuperscriptBox["c", "2"]}], ",", 
   RowBox[{"{", "}"}]}], "]"}]], "Input",
 CellTags->"NumericQ1",
 CellLabel->"In[1197]:="],

Cell[BoxData[
 FormBox["False", TraditionalForm]], "Output",
 CellChangeTimes->{3.5009703273391657`*^9},
 CellTags->"NumericQ1",
 CellLabel->"Out[1197]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"NumericQ1", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"3", " ", "a"}], "+", 
    RowBox[{"Log", "[", "b", "]"}], "+", 
    SuperscriptBox["c", "2"]}], ",", 
   RowBox[{"{", 
    RowBox[{"a", ",", "b", ",", "c"}], "}"}]}], "]"}]], "Input",
 CellTags->"NumericQ1",
 CellLabel->"In[1198]:="],

Cell[BoxData[
 FormBox["True", TraditionalForm]], "Output",
 CellChangeTimes->{3.500970327432764*^9},
 CellTags->"NumericQ1",
 CellLabel->"Out[1198]="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["OneLoop  ***unfinished***", "ObjectName",
 CellTags->"OneLoop"],

Cell[TextData[{
 "See also the old FeynCalc documentation at ",
 ButtonBox["http://www.mertig.com/oldfc/",
  BaseStyle->"Hyperlink",
  ButtonData:>{
    URL["http://www.mertig.com/oldfc/"], None}],
 ". NOTICE: While OneLoop is restricted to 't Hooft Feynman gauge the \
function ",
 ButtonBox["OneLoopSimplify",
  BaseStyle->"AddOnsLink",
  ButtonData:>"OneLoopSimplify",
  ButtonNote->"OneLoopSimplify"],
 " does not have this restriction (but is usually slower). OneLoop handles \
selfenergies, vertex and box-graphs (those only up to 3rd rank tensor in the \
integration variable)."
}], "Text",
 CellTags->"OneLoop"],

Cell[TextData[{
 "WARNING: If you encounter anomalies: ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
    "The", " ", "default", " ", "setting", " ", "of", " ", "$West", " ", "is",
      " ", "True"}], ",", " ", 
    RowBox[{"i", ".", "e", "."}], ",", " ", 
    RowBox[{"the", " ", "way", " ", 
     RowBox[{"tr", "(", 
      RowBox[{
       SuperscriptBox["\[Gamma]", "\[Mu]"], 
       SuperscriptBox["\[Gamma]", "\[Nu]"], 
       SuperscriptBox["\[Gamma]", "\[Rho]"], 
       SuperscriptBox["\[Gamma]", "\[Sigma]"], 
       SuperscriptBox["\[Gamma]", "\[Tau]"], 
       SuperscriptBox["\[Gamma]", "\[Lambda]"], 
       SuperscriptBox["\[Gamma]", "5"]}], ")"}]}]}], TraditionalForm]]],
 " is calculated in D dimensions has changed compared to the old FeynCalc \
version. Please keep in mind that the issue of ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["\[Gamma]", "5"], TraditionalForm]]],
 "schemes is inherintly tricky."
}], "Text",
 CellTags->"OneLoop"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"OneLoop"],

Cell[TextData[{
 "OneLoop[",
 "q",
 ", amplitude] calculates the one-loop Feynman diagram amplitude (",
 StyleBox["n",
  FontSlant->"Italic"],
 "-point, where ",
 StyleBox["n",
  FontSlant->"Italic"],
 "\[LessEqual]4 and the highest tensor rank of the integration momenta (after \
cancellation of scalar products) may be 3; unless OneLoopSimplify is used)."
}], "Text",
 CellTags->"OneLoop"],

Cell["\<\
The argument q denotes the integration variable, i.e., the loop momentum. \
OneLoop[name, q, amplitude] has as first argument a name of the amplitude. If \
the second argument has head FeynAmp then OneLoop[q, FeynAmp[name, k, expr]] \
and OneLoop[FeynAmp[name, k, expr]] tranform to OneLoop[name, k, expr].\
\>", "Text",
 CellTags->"OneLoop"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "OneLoop", "]"}]], "Input",
 CellTags->"OneLoop",
 CellLabel->"In[1199]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Apart2", "\[Rule]", "True"}], ",", 
    RowBox[{"CancelQP", "\[Rule]", "True"}], ",", 
    RowBox[{"DenominatorOrder", "\[Rule]", "False"}], ",", 
    RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
    RowBox[{"FinalSubstitutions", "\[Rule]", 
     RowBox[{"{", "}"}]}], ",", 
    RowBox[{"Factoring", "\[Rule]", "False"}], ",", 
    RowBox[{"FormatType", "\[Rule]", "InputForm"}], ",", 
    RowBox[{"InitialSubstitutions", "\[Rule]", 
     RowBox[{"{", "}"}]}], ",", 
    RowBox[{"IntermediateSubstitutions", "\[Rule]", 
     RowBox[{"{", "}"}]}], ",", 
    RowBox[{"IsolateNames", "\[Rule]", "False"}], ",", 
    RowBox[{"Mandelstam", "\[Rule]", 
     RowBox[{"{", "}"}]}], ",", 
    RowBox[{"OneLoopSimplify", "\[Rule]", "False"}], ",", 
    RowBox[{"Prefactor", "\[Rule]", "1"}], ",", 
    RowBox[{"ReduceGamma", "\[Rule]", "False"}], ",", 
    RowBox[{"ReduceToScalars", "\[Rule]", "False"}], ",", 
    RowBox[{"SmallVariables", "\[Rule]", 
     RowBox[{"{", "}"}]}], ",", 
    RowBox[{"WriteOut", "\[Rule]", "False"}], ",", 
    RowBox[{"WriteOutPaVe", "\[Rule]", "False"}], ",", 
    RowBox[{"Sum", "\[Rule]", "True"}]}], "}"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970327509407*^9},
 CellTags->"OneLoop",
 CellLabel->"Out[1199]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["B0",
  BaseStyle->"AddOnsLink",
  ButtonData:>"B0",
  ButtonNote->"B0"],
 ", ",
 ButtonBox["C0",
  BaseStyle->"AddOnsLink",
  ButtonData:>"C0",
  ButtonNote->"C0"],
 ", ",
 ButtonBox["D0",
  BaseStyle->"AddOnsLink",
  ButtonData:>"D0",
  ButtonNote->"D0"],
 ", ",
 ButtonBox["OneLoopSimplify",
  BaseStyle->"AddOnsLink",
  ButtonData:>"OneLoopSimplify",
  ButtonNote->"OneLoopSimplify"],
 ", ",
 ButtonBox["TID",
  BaseStyle->"AddOnsLink",
  ButtonData:>"TID",
  ButtonNote->"TID"],
 ", ",
 ButtonBox["TIDL",
  BaseStyle->"AddOnsLink",
  ButtonData:>"TIDL",
  ButtonNote->"TIDL"],
 ", ",
 ButtonBox["$LimitTo4",
  BaseStyle->"AddOnsLink",
  ButtonData:>"$LimitTo4",
  ButtonNote->"$LimitTo4"],
 "."
}], "Text",
 CellTags->"OneLoop"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"OneLoop"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"-", "I"}], "/", 
    RowBox[{"Pi", "^", "2"}]}], " ", 
   RowBox[{"FAD", "[", 
    RowBox[{"{", 
     RowBox[{"q", ",", "m"}], "}"}], "]"}]}], "//", "FCI"}]], "Input",
 CellTags->"OneLoop",
 CellLabel->"In[1200]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   FractionBox["\[ImaginaryI]", 
    RowBox[{
     SuperscriptBox["\[Pi]", "2"], " ", 
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox["q", "2"], "-", 
       SuperscriptBox["m", "2"]}], ")"}]}]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970327625476*^9},
 CellTags->"OneLoop",
 CellLabel->"Out[1200]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"OneLoop", "[", 
  RowBox[{"q", ",", "%"}], "]"}]], "Input",
 CellTags->"OneLoop",
 CellLabel->"In[1201]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox[
    SubscriptBox["\<\"A\"\>", "\<\"0\"\>"],
    TraditionalForm], "(", 
   FormBox[
    SuperscriptBox["m", "2"],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009703293790903`*^9},
 CellTags->"OneLoop",
 CellLabel->"Out[1201]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"mf", "/:", 
   RowBox[{"MakeBoxes", "[", 
    RowBox[{"mf", ",", "TraditionalForm"}], "]"}], "=", 
   RowBox[{"InterpretationBox", "[", 
    RowBox[{
     RowBox[{"SubscriptBox", "[", 
      RowBox[{"\"\<m\>\"", ",", "\"\<f\>\""}], "]"}], ",", "mf"}], "]"}]}], 
  ";"}]], "Input",
 CellTags->"OneLoop",
 CellLabel->"In[1202]:="],

Cell[TextData[{
 "Remember that FAD[{q,mf},{q-k,mf}] is a fast possibility to enter ",
 Cell[BoxData[
  FormBox[
   RowBox[{"1", "/", 
    RowBox[{
     RowBox[{"(", " ", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox["q", "2"], "-", 
         SuperscriptBox["mf", "2"]}], ")"}], " ", 
       RowBox[{"(", " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"q", "-", "k"}], ")"}], "2"], "-", 
         SuperscriptBox["mf", "2"]}], ")"}]}], " ", ")"}], "."}]}], 
   TraditionalForm]]]
}], "Text",
 CellTags->"OneLoop"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t", "=", 
  RowBox[{
   RowBox[{"I", " ", 
    FractionBox[
     SuperscriptBox["el", "2"], 
     RowBox[{"16", " ", 
      SuperscriptBox["Pi", "4"], 
      RowBox[{"(", 
       RowBox[{"1", "-", "D"}], ")"}]}]], 
    RowBox[{"FAD", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"q", ",", "mf"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"q", "-", "k"}], ",", "mf"}], "}"}]}], "]"}], 
    RowBox[{"DiracTrace", "[", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"mf", "+", 
        RowBox[{"GS", "[", 
         RowBox[{"q", "-", "k"}], "]"}]}], ")"}], ".", 
      RowBox[{"GA", "[", "\[Mu]", "]"}], ".", 
      RowBox[{"(", 
       RowBox[{"mf", "+", 
        RowBox[{"GS", "[", "q", "]"}]}], ")"}], ".", 
      RowBox[{"GA", "[", "\[Mu]", "]"}]}], "]"}]}], " ", "//", " ", 
   "FCI"}]}]], "Input",
 CellTags->"OneLoop",
 CellLabel->"In[1203]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{"\[ImaginaryI]", " ", 
    SuperscriptBox["el", "2"], " ", 
    RowBox[{"tr", "(", 
     FormBox[
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         InterpretationBox[
          SubscriptBox["m", "f"],
          $CellContext`mf], "+", 
         RowBox[{
          FormBox["\<\"\[Gamma]\"\>",
           TraditionalForm], "\[CenterDot]", "(", 
          FormBox[
           RowBox[{"q", "-", "k"}],
           TraditionalForm], ")"}]}], ")"}], ".", 
       SuperscriptBox["\[Gamma]", 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], ".", 
       RowBox[{"(", 
        RowBox[{
         InterpretationBox[
          SubscriptBox["m", "f"],
          $CellContext`mf], "+", 
         RowBox[{
          FormBox["\<\"\[Gamma]\"\>",
           TraditionalForm], "\[CenterDot]", 
          FormBox["q",
           TraditionalForm]}]}], ")"}], ".", 
       SuperscriptBox["\[Gamma]", 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]]}],
      TraditionalForm], ")"}]}], 
   RowBox[{"16", " ", 
    SuperscriptBox["\[Pi]", "4"], " ", 
    RowBox[{"(", 
     RowBox[{"1", "-", "D"}], ")"}], " ", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox["q", "2"], "-", 
       SuperscriptBox[
        RowBox[{"(", 
         InterpretationBox[
          SubscriptBox["m", "f"],
          $CellContext`mf], ")"}], "2"]}], ")"}], ".", 
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"q", "-", "k"}], ")"}], "2"], "-", 
       SuperscriptBox[
        RowBox[{"(", 
         InterpretationBox[
          SubscriptBox["m", "f"],
          $CellContext`mf], ")"}], "2"]}], ")"}]}]}]], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970329897017*^9},
 CellTags->"OneLoop",
 CellLabel->"Out[1203]="]
}, Open  ]],

Cell["\<\
The input to OneLoop may be in 4 dimensions, since the function changes the \
dimension of the objects automatically to the setting of the Dimension option \
(D by default).\
\>", "Text",
 CellTags->"OneLoop"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"OneLoop", "[", 
  RowBox[{"q", ",", "t"}], "]"}]], "Input",
 CellTags->"OneLoop",
 CellLabel->"In[1204]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   FractionBox[
    RowBox[{
     SuperscriptBox["el", "2"], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"-", 
        RowBox[{
         FractionBox["8", "3"], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           InterpretationBox[
            SubscriptBox["m", "f"],
            $CellContext`mf], ")"}], "2"], " ", 
         RowBox[{
          SubscriptBox["B", "0"], "(", 
          SuperscriptBox[
           FormBox["k",
            TraditionalForm], "2"], ",", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "f"],
             $CellContext`mf], ")"}], "2"], ",", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "f"],
             $CellContext`mf], ")"}], "2"], ")"}]}]}], "-", 
       RowBox[{
        FractionBox["4", "3"], " ", 
        SuperscriptBox[
         FormBox["k",
          TraditionalForm], "2"], " ", 
        RowBox[{
         SubscriptBox["B", "0"], "(", 
         SuperscriptBox[
          FormBox["k",
           TraditionalForm], "2"], ",", 
         SuperscriptBox[
          RowBox[{"(", 
           InterpretationBox[
            SubscriptBox["m", "f"],
            $CellContext`mf], ")"}], "2"], ",", 
         SuperscriptBox[
          RowBox[{"(", 
           InterpretationBox[
            SubscriptBox["m", "f"],
            $CellContext`mf], ")"}], "2"], ")"}]}], "+", 
       RowBox[{
        FractionBox["8", "3"], " ", 
        RowBox[{
         FormBox[
          SubscriptBox["\<\"A\"\>", "\<\"0\"\>"],
          TraditionalForm], "(", 
         FormBox[
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "f"],
             $CellContext`mf], ")"}], "2"],
          TraditionalForm], ")"}]}], "-", 
       FractionBox[
        RowBox[{"8", " ", 
         SuperscriptBox[
          RowBox[{"(", 
           InterpretationBox[
            SubscriptBox["m", "f"],
            $CellContext`mf], ")"}], "2"]}], "3"], "+", 
       FractionBox[
        RowBox[{"4", " ", 
         SuperscriptBox[
          FormBox["k",
           TraditionalForm], "2"]}], "9"]}], ")"}]}], 
    RowBox[{"16", " ", 
     SuperscriptBox["\[Pi]", "2"]}]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970332217388*^9},
 CellTags->"OneLoop",
 CellLabel->"Out[1204]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", "%", "]"}]], "Input",
 CellTags->"OneLoop",
 CellLabel->"In[1205]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{
    SuperscriptBox["el", "2"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"3", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"2", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "f"],
             $CellContext`mf], ")"}], "2"]}], "+", 
         SuperscriptBox[
          FormBox["k",
           TraditionalForm], "2"]}], ")"}], " ", 
       RowBox[{
        SubscriptBox["B", "0"], "(", 
        SuperscriptBox[
         FormBox["k",
          TraditionalForm], "2"], ",", 
        SuperscriptBox[
         RowBox[{"(", 
          InterpretationBox[
           SubscriptBox["m", "f"],
           $CellContext`mf], ")"}], "2"], ",", 
        SuperscriptBox[
         RowBox[{"(", 
          InterpretationBox[
           SubscriptBox["m", "f"],
           $CellContext`mf], ")"}], "2"], ")"}]}], "-", 
      RowBox[{"6", " ", 
       RowBox[{
        FormBox[
         SubscriptBox["\<\"A\"\>", "\<\"0\"\>"],
         TraditionalForm], "(", 
        FormBox[
         SuperscriptBox[
          RowBox[{"(", 
           InterpretationBox[
            SubscriptBox["m", "f"],
            $CellContext`mf], ")"}], "2"],
         TraditionalForm], ")"}]}], "+", 
      RowBox[{"6", " ", 
       SuperscriptBox[
        RowBox[{"(", 
         InterpretationBox[
          SubscriptBox["m", "f"],
          $CellContext`mf], ")"}], "2"]}], "-", 
      SuperscriptBox[
       FormBox["k",
        TraditionalForm], "2"]}], ")"}]}], 
   RowBox[{"36", " ", 
    SuperscriptBox["\[Pi]", "2"]}]], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970332582222*^9},
 CellTags->"OneLoop",
 CellLabel->"Out[1205]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"SP", "[", 
    RowBox[{"k", ",", "r"}], "]"}], " ", 
   RowBox[{"FAD", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"k", ",", "m"}], "}"}], " ", ",", " ", 
     RowBox[{"k", " ", "-", " ", "p"}]}], "]"}]}], "//", "FCI"}]], "Input",
 CellTags->"OneLoop",
 CellLabel->"In[1206]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{
    FormBox["k",
     TraditionalForm], "\[CenterDot]", 
    FormBox["r",
     TraditionalForm]}], 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox["k", "2"], "-", 
      SuperscriptBox["m", "2"]}], ")"}], ".", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"k", "-", "p"}], ")"}], "2"]}]], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009703326764727`*^9},
 CellTags->"OneLoop",
 CellLabel->"Out[1206]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"OneLoop", "[", 
  RowBox[{"k", ",", "%"}], "]"}]], "Input",
 CellTags->"OneLoop",
 CellLabel->"In[1207]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"\[ImaginaryI]", " ", 
   SuperscriptBox["\[Pi]", "2"], " ", 
   RowBox[{"(", 
    RowBox[{
     FractionBox[
      RowBox[{
       SuperscriptBox["m", "2"], " ", 
       RowBox[{
        SubscriptBox["B", "0"], "(", "0", ",", 
        SuperscriptBox["m", "2"], ",", 
        SuperscriptBox["m", "2"], ")"}], " ", 
       RowBox[{
        FormBox["p",
         TraditionalForm], "\[CenterDot]", 
        FormBox["r",
         TraditionalForm]}]}], 
      RowBox[{"2", " ", 
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], "2"]}]], "-", 
     FractionBox[
      RowBox[{
       SuperscriptBox["m", "2"], " ", 
       RowBox[{
        FormBox["p",
         TraditionalForm], "\[CenterDot]", 
        FormBox["r",
         TraditionalForm]}], " ", 
       RowBox[{
        SubscriptBox["B", "0"], "(", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], "2"], ",", "0", ",", 
        SuperscriptBox["m", "2"], ")"}]}], 
      RowBox[{"2", " ", 
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], "2"]}]], "+", 
     RowBox[{
      FractionBox["1", "2"], " ", 
      RowBox[{
       FormBox["p",
        TraditionalForm], "\[CenterDot]", 
       FormBox["r",
        TraditionalForm]}], " ", 
      RowBox[{
       SubscriptBox["B", "0"], "(", 
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], "2"], ",", "0", ",", 
       SuperscriptBox["m", "2"], ")"}]}], "+", 
     FractionBox[
      RowBox[{
       SuperscriptBox["m", "2"], " ", 
       RowBox[{
        FormBox["p",
         TraditionalForm], "\[CenterDot]", 
        FormBox["r",
         TraditionalForm]}]}], 
      RowBox[{"2", " ", 
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], "2"]}]]}], ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970334485516*^9},
 CellTags->"OneLoop",
 CellLabel->"Out[1207]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"Clear", "[", "t", "]"}]], "Input",
 CellTags->"OneLoop",
 CellLabel->"In[1208]:="]
}, Open  ]],

Cell["OneLoopSimplify", "ObjectName",
 CellTags->"OneLoopSimplify"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"OneLoopSimplify"],

Cell["\<\
OneLoopSimplify[amp, q] simplifies the one-loop amplitude amp. The second \
argument denotes the integration momentum.\
\>", "Text",
 CellTags->"OneLoopSimplify"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "OneLoopSimplify", "]"}]], "Input",
 CellTags->"OneLoopSimplify",
 CellLabel->"In[1209]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Collecting", "\[Rule]", "False"}], ",", 
    RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
    RowBox[{"DimensionalReduction", "\[Rule]", "False"}], ",", 
    RowBox[{"DiracSimplify", "\[Rule]", "True"}], ",", 
    RowBox[{"FinalSubstitutions", "\[Rule]", 
     RowBox[{"{", "}"}]}], ",", 
    RowBox[{"IntegralTable", "\[Rule]", 
     RowBox[{"{", "}"}]}], ",", 
    RowBox[{"OPE1Loop", "\[Rule]", "False"}], ",", 
    RowBox[{"ScalarProductCancel", "\[Rule]", "True"}], ",", 
    RowBox[{"SUNNToCACF", "\[Rule]", "True"}], ",", 
    RowBox[{"SUNTrace", "\[Rule]", "False"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970334685875*^9},
 CellTags->"OneLoopSimplify",
 CellLabel->"Out[1209]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["OneLoop",
  BaseStyle->"AddOnsLink",
  ButtonData:>"OneLoop",
  ButtonNote->"OneLoop"],
 ", ",
 ButtonBox["TID",
  BaseStyle->"AddOnsLink",
  ButtonData:>"TID",
  ButtonNote->"TID"],
 ", ",
 ButtonBox["TIDL",
  BaseStyle->"AddOnsLink",
  ButtonData:>"TIDL",
  ButtonNote->"TIDL"],
 "."
}], "Text",
 CellTags->"OneLoopSimplify"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"OneLoopSimplify"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"SP", "[", 
    RowBox[{"k", ",", "r"}], "]"}], " ", 
   RowBox[{"FAD", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"k", ",", "m"}], "}"}], " ", ",", " ", 
     RowBox[{"k", " ", "-", " ", "p"}]}], "]"}]}], "//", "FCI"}]], "Input",
 CellTags->"OneLoopSimplify",
 CellLabel->"In[1210]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{
    FormBox["k",
     TraditionalForm], "\[CenterDot]", 
    FormBox["r",
     TraditionalForm]}], 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox["k", "2"], "-", 
      SuperscriptBox["m", "2"]}], ")"}], ".", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"k", "-", "p"}], ")"}], "2"]}]], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009703348495293`*^9},
 CellTags->"OneLoopSimplify",
 CellLabel->"Out[1210]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"OneLoopSimplify", "[", 
  RowBox[{"%", ",", "k"}], "]"}]], "Input",
 CellTags->"OneLoopSimplify",
 CellLabel->"In[1211]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{
    FormBox["k",
     TraditionalForm], "\[CenterDot]", 
    FormBox["r",
     TraditionalForm]}], 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox["k", "2"], "-", 
      SuperscriptBox["m", "2"]}], ")"}], ".", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"k", "-", "p"}], ")"}], "2"]}]], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970335010521*^9},
 CellTags->"OneLoopSimplify",
 CellLabel->"Out[1211]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"OneLoopSimplify", "[", 
  RowBox[{
   RowBox[{"%", "/.", 
    RowBox[{"m", "\[Rule]", "0"}]}], ",", "k"}], "]"}]], "Input",
 CellTags->"OneLoopSimplify",
 CellLabel->"In[1212]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{
    FormBox["k",
     TraditionalForm], "\[CenterDot]", 
    FormBox["r",
     TraditionalForm]}], 
   RowBox[{
    SuperscriptBox["k", "2"], ".", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"k", "-", "p"}], ")"}], "2"]}]], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009703351790648`*^9},
 CellTags->"OneLoopSimplify",
 CellLabel->"Out[1212]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"FAD", "[", 
    RowBox[{"k", ",", "k", ",", " ", 
     RowBox[{"k", " ", "-", " ", 
      SubscriptBox["p", "1"]}], ",", " ", 
     RowBox[{"k", " ", "-", " ", 
      SubscriptBox["p", "2"]}]}], "]"}], " ", 
   RowBox[{"FVD", "[", 
    RowBox[{"k", ",", "\[Mu]"}], "]"}]}], "//", "FCI"}]], "Input",
 CellTags->"OneLoopSimplify",
 CellLabel->"In[1213]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   SuperscriptBox[
    FormBox["k",
     TraditionalForm], 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], 
   RowBox[{
    SuperscriptBox["k", "2"], ".", 
    SuperscriptBox["k", "2"], ".", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"k", "-", 
       SubscriptBox["p", "1"]}], ")"}], "2"], ".", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"k", "-", 
       SubscriptBox["p", "2"]}], ")"}], "2"]}]], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009703353432426`*^9},
 CellTags->"OneLoopSimplify",
 CellLabel->"Out[1213]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"OneLoopSimplify", "[", " ", 
  RowBox[{"%", ",", "k"}], "]"}]], "Input",
 CellTags->"OneLoopSimplify",
 CellLabel->"In[1214]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    FractionBox[
     RowBox[{
      RowBox[{
       SubsuperscriptBox["p", "1", "2"], " ", 
       SubsuperscriptBox["p", 
        FormBox["2",
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]]}], "-", 
      RowBox[{
       SubsuperscriptBox["p", 
        FormBox["1",
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox[
         SubscriptBox["p", "1"],
         TraditionalForm], "\[CenterDot]", 
        FormBox[
         SubscriptBox["p", "2"],
         TraditionalForm]}]}]}], 
     RowBox[{"2", " ", 
      RowBox[{
       SuperscriptBox["k", "2"], ".", 
       SuperscriptBox["k", "2"], ".", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"k", "-", 
          SubscriptBox["p", "1"]}], ")"}], "2"]}], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         SubsuperscriptBox["p", "1", "2"], " ", 
         SubsuperscriptBox["p", "2", "2"]}], "-", 
        SuperscriptBox[
         FormBox[
          RowBox[{
           FormBox[
            SubscriptBox["p", "1"],
            TraditionalForm], "\[CenterDot]", 
           FormBox[
            SubscriptBox["p", "2"],
            TraditionalForm]}],
          TraditionalForm], "2"]}], ")"}]}]]}], "-", 
   FractionBox[
    RowBox[{
     RowBox[{"-", 
      RowBox[{
       SubsuperscriptBox["p", "1", "2"], " ", 
       SubsuperscriptBox["p", "2", "2"], " ", 
       SubsuperscriptBox["p", 
        FormBox["1",
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]]}]}], "-", 
     RowBox[{
      SubsuperscriptBox["p", "1", "2"], " ", 
      SubsuperscriptBox["p", "2", "2"], " ", 
      SubsuperscriptBox["p", 
       FormBox["2",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]]}], "+", 
     RowBox[{
      SubsuperscriptBox["p", "1", "2"], " ", 
      SubsuperscriptBox["p", 
       FormBox["2",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      RowBox[{
       FormBox[
        SubscriptBox["p", "1"],
        TraditionalForm], "\[CenterDot]", 
       FormBox[
        SubscriptBox["p", "2"],
        TraditionalForm]}]}], "+", 
     RowBox[{
      SubsuperscriptBox["p", "2", "2"], " ", 
      SubsuperscriptBox["p", 
       FormBox["1",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      RowBox[{
       FormBox[
        SubscriptBox["p", "1"],
        TraditionalForm], "\[CenterDot]", 
       FormBox[
        SubscriptBox["p", "2"],
        TraditionalForm]}]}]}], 
    RowBox[{"2", " ", 
     RowBox[{
      SuperscriptBox["k", "2"], ".", 
      SuperscriptBox["k", "2"], ".", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"k", "-", 
         SubscriptBox["p", "1"]}], ")"}], "2"], ".", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"k", "-", 
         SubscriptBox["p", "2"]}], ")"}], "2"]}], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        SubsuperscriptBox["p", "1", "2"], " ", 
        SubsuperscriptBox["p", "2", "2"]}], "-", 
       SuperscriptBox[
        FormBox[
         RowBox[{
          FormBox[
           SubscriptBox["p", "1"],
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["p", "2"],
           TraditionalForm]}],
         TraditionalForm], "2"]}], ")"}]}]], "-", 
   FractionBox[
    RowBox[{
     RowBox[{
      SubsuperscriptBox["p", 
       FormBox["2",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      RowBox[{
       FormBox[
        SubscriptBox["p", "1"],
        TraditionalForm], "\[CenterDot]", 
       FormBox[
        SubscriptBox["p", "2"],
        TraditionalForm]}]}], "-", 
     RowBox[{
      SubsuperscriptBox["p", "2", "2"], " ", 
      SubsuperscriptBox["p", 
       FormBox["1",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]]}]}], 
    RowBox[{"2", " ", 
     RowBox[{
      SuperscriptBox["k", "2"], ".", 
      SuperscriptBox["k", "2"], ".", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"k", "-", 
         SubscriptBox["p", "2"]}], ")"}], "2"]}], " ", 
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox[
        FormBox[
         RowBox[{
          FormBox[
           SubscriptBox["p", "1"],
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["p", "2"],
           TraditionalForm]}],
         TraditionalForm], "2"], "-", 
       RowBox[{
        SubsuperscriptBox["p", "1", "2"], " ", 
        SubsuperscriptBox["p", "2", "2"]}]}], ")"}]}]], "+", 
   FractionBox[
    RowBox[{
     RowBox[{
      SubsuperscriptBox["p", "1", "2"], " ", 
      SubsuperscriptBox["p", 
       FormBox["2",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]]}], "+", 
     RowBox[{
      SubsuperscriptBox["p", "2", "2"], " ", 
      SubsuperscriptBox["p", 
       FormBox["1",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]]}], "-", 
     RowBox[{
      SubsuperscriptBox["p", 
       FormBox["1",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      RowBox[{
       FormBox[
        SubscriptBox["p", "1"],
        TraditionalForm], "\[CenterDot]", 
       FormBox[
        SubscriptBox["p", "2"],
        TraditionalForm]}]}], "-", 
     RowBox[{
      SubsuperscriptBox["p", 
       FormBox["2",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      RowBox[{
       FormBox[
        SubscriptBox["p", "1"],
        TraditionalForm], "\[CenterDot]", 
       FormBox[
        SubscriptBox["p", "2"],
        TraditionalForm]}]}]}], 
    RowBox[{"2", " ", 
     RowBox[{
      SuperscriptBox["k", "2"], ".", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"k", "-", 
         SubscriptBox["p", "1"]}], ")"}], "2"], ".", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"k", "-", 
         SubscriptBox["p", "2"]}], ")"}], "2"]}], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        SubsuperscriptBox["p", "1", "2"], " ", 
        SubsuperscriptBox["p", "2", "2"]}], "-", 
       SuperscriptBox[
        FormBox[
         RowBox[{
          FormBox[
           SubscriptBox["p", "1"],
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["p", "2"],
           TraditionalForm]}],
         TraditionalForm], "2"]}], ")"}]}]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970335996463*^9},
 CellTags->"OneLoopSimplify",
 CellLabel->"Out[1214]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"FCE", "[", "%", "]"}], "/.", 
   RowBox[{
    RowBox[{"SPD", "[", 
     SubscriptBox["p", "1"], "]"}], "\[Rule]", "0"}]}], "//", 
  "FCI"}]], "Input",
 CellTags->"OneLoopSimplify",
 CellLabel->"In[1215]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{
     SubsuperscriptBox["p", "2", "2"], " ", 
     SubsuperscriptBox["p", 
      FormBox["1",
       TraditionalForm], 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]]}], 
    RowBox[{"2", " ", 
     RowBox[{
      SuperscriptBox["k", "2"], ".", 
      SuperscriptBox["k", "2"], ".", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"k", "-", 
         SubscriptBox["p", "1"]}], ")"}], "2"], ".", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"k", "-", 
         SubscriptBox["p", "2"]}], ")"}], "2"]}], " ", 
     RowBox[{
      FormBox[
       SubscriptBox["p", "1"],
       TraditionalForm], "\[CenterDot]", 
      FormBox[
       SubscriptBox["p", "2"],
       TraditionalForm]}]}]], "-", 
   FractionBox[
    SubsuperscriptBox["p", 
     FormBox["1",
      TraditionalForm], 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], 
    RowBox[{"2", " ", 
     RowBox[{
      SuperscriptBox["k", "2"], ".", 
      SuperscriptBox["k", "2"], ".", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"k", "-", 
         SubscriptBox["p", "1"]}], ")"}], "2"]}], " ", 
     RowBox[{
      FormBox[
       SubscriptBox["p", "1"],
       TraditionalForm], "\[CenterDot]", 
      FormBox[
       SubscriptBox["p", "2"],
       TraditionalForm]}]}]], "-", 
   FractionBox[
    RowBox[{
     RowBox[{
      SubsuperscriptBox["p", 
       FormBox["2",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      RowBox[{
       FormBox[
        SubscriptBox["p", "1"],
        TraditionalForm], "\[CenterDot]", 
       FormBox[
        SubscriptBox["p", "2"],
        TraditionalForm]}]}], "-", 
     RowBox[{
      SubsuperscriptBox["p", "2", "2"], " ", 
      SubsuperscriptBox["p", 
       FormBox["1",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]]}]}], 
    RowBox[{"2", " ", 
     RowBox[{
      SuperscriptBox["k", "2"], ".", 
      SuperscriptBox["k", "2"], ".", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"k", "-", 
         SubscriptBox["p", "2"]}], ")"}], "2"]}], " ", 
     SuperscriptBox[
      FormBox[
       RowBox[{
        FormBox[
         SubscriptBox["p", "1"],
         TraditionalForm], "\[CenterDot]", 
        FormBox[
         SubscriptBox["p", "2"],
         TraditionalForm]}],
       TraditionalForm], "2"]}]], "-", 
   FractionBox[
    RowBox[{
     RowBox[{
      SubsuperscriptBox["p", "2", "2"], " ", 
      SubsuperscriptBox["p", 
       FormBox["1",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]]}], "-", 
     RowBox[{
      SubsuperscriptBox["p", 
       FormBox["1",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      RowBox[{
       FormBox[
        SubscriptBox["p", "1"],
        TraditionalForm], "\[CenterDot]", 
       FormBox[
        SubscriptBox["p", "2"],
        TraditionalForm]}]}], "-", 
     RowBox[{
      SubsuperscriptBox["p", 
       FormBox["2",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      RowBox[{
       FormBox[
        SubscriptBox["p", "1"],
        TraditionalForm], "\[CenterDot]", 
       FormBox[
        SubscriptBox["p", "2"],
        TraditionalForm]}]}]}], 
    RowBox[{"2", " ", 
     RowBox[{
      SuperscriptBox["k", "2"], ".", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"k", "-", 
         SubscriptBox["p", "1"]}], ")"}], "2"], ".", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"k", "-", 
         SubscriptBox["p", "2"]}], ")"}], "2"]}], " ", 
     SuperscriptBox[
      FormBox[
       RowBox[{
        FormBox[
         SubscriptBox["p", "1"],
         TraditionalForm], "\[CenterDot]", 
        FormBox[
         SubscriptBox["p", "2"],
         TraditionalForm]}],
       TraditionalForm], "2"]}]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009703366031437`*^9},
 CellTags->"OneLoopSimplify",
 CellLabel->"Out[1215]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"OneLoopSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"FAD", "[", 
     RowBox[{
      RowBox[{"k", "-", 
       SubscriptBox["p", "1"]}], ",", 
      RowBox[{"k", "-", 
       SubscriptBox["p", "2"]}]}], "]"}], " ", 
    RowBox[{
     RowBox[{"SP", "[", 
      RowBox[{"k", ",", "l"}], "]"}], "^", "2"}]}], ",", "k"}], 
  "]"}]], "Input",
 CellTags->"OneLoopSimplify",
 CellLabel->"In[1216]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    SuperscriptBox[
     FormBox[
      RowBox[{
       FormBox["k",
        TraditionalForm], "\[CenterDot]", 
       FormBox["l",
        TraditionalForm]}],
      TraditionalForm], "2"], 
    RowBox[{
     SuperscriptBox["k", "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"k", "-", 
        SubscriptBox["p", "1"], "+", 
        SubscriptBox["p", "2"]}], ")"}], "2"]}]], "-", 
   FractionBox[
    RowBox[{"2", " ", 
     RowBox[{
      FormBox["k",
       TraditionalForm], "\[CenterDot]", 
      FormBox["l",
       TraditionalForm]}], " ", 
     RowBox[{
      FormBox["l",
       TraditionalForm], "\[CenterDot]", 
      FormBox[
       SubscriptBox["p", "1"],
       TraditionalForm]}]}], 
    RowBox[{
     SuperscriptBox["k", "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"k", "-", 
        SubscriptBox["p", "1"], "+", 
        SubscriptBox["p", "2"]}], ")"}], "2"]}]], "+", 
   FractionBox[
    SuperscriptBox[
     FormBox[
      RowBox[{
       FormBox["l",
        TraditionalForm], "\[CenterDot]", 
       FormBox[
        SubscriptBox["p", "1"],
        TraditionalForm]}],
      TraditionalForm], "2"], 
    RowBox[{
     SuperscriptBox["k", "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"k", "-", 
        SubscriptBox["p", "1"], "+", 
        SubscriptBox["p", "2"]}], ")"}], "2"]}]]}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970336883359*^9},
 CellTags->"OneLoopSimplify",
 CellLabel->"Out[1216]="]
}, Open  ]]
}, Open  ]],

Cell["OneLoopSum", "ObjectName",
 CellTags->"OneLoopSum"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"OneLoopSum"],

Cell["\<\
OneLoopSum[ FeynAmp[ ... ], FeynAmp[ ... ] , ...] will calculate a list of \
Feynman amplitudes by replacing FeynAmp step by step by OneLoop.\
\>", "Text",
 CellTags->"OneLoopSum"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["OneLoop",
  BaseStyle->"AddOnsLink",
  ButtonData:>"OneLoop",
  ButtonNote->"OneLoop"],
 "."
}], "Text",
 CellTags->"OneLoopSum"]
}, Open  ]],

Cell["OPE", "ObjectName",
 CellTags->"OPE"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"OPE"],

Cell["\<\
OPE is a convenience variable to separate OPE insertions.
OPE is also an option of several input functions like GluonPropagator.\
\>", "Text",
 CellTags->"OPE"],

Cell["\<\
The OPE variable has the property to vanish for higher powers:\
\>", "Text"],

Cell[CellGroupData[{

Cell["{OPE,OPE^2,OPE^3}", "Input",
 CellLabel->"In[1217]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"\[CapitalOmega]", ",", "0", ",", "0"}], "}"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970336962059*^9},
 CellLabel->"Out[1217]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["OPE1Loop",
  BaseStyle->"AddOnsLink",
  ButtonData:>"OPE1Loop",
  ButtonNote->"OPE1Loop"],
 "."
}], "Text",
 CellTags->"OPE"]
}, Open  ]],

Cell["OPEDelta", "ObjectName",
 CellTags->"OPEDelta"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"OPEDelta"],

Cell["\<\
OPEDelta is a lightlike axial vector as used e.g. in the operator product \
expansion in QCD.\
\>", "Text",
 CellTags->"OPEDelta"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Twist2QuarkOperator",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Twist2QuarkOperator",
  ButtonNote->"Twist2QuarkOperator"],
 "."
}], "Text",
 CellTags->"OPEDelta"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"OPEDelta"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FourVector", "[", 
  RowBox[{"OPEDelta", ",", "\[Mu]"}], "]"}]], "Input",
 CellTags->"OPEDelta",
 CellLabel->"In[1218]:="],

Cell[BoxData[
 FormBox[
  SubscriptBox[
   FormBox["\[CapitalDelta]",
    TraditionalForm], "\[Mu]"], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970337031644*^9},
 CellTags->"OPEDelta",
 CellLabel->"Out[1218]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Contract", "[", 
  RowBox[{"%", " ", "%"}], "]"}]], "Input",
 CellTags->"OPEDelta",
 CellLabel->"In[1219]:="],

Cell[BoxData[
 FormBox["0", TraditionalForm]], "Output",
 CellChangeTimes->{3.5009703373249197`*^9},
 CellTags->"OPEDelta",
 CellLabel->"Out[1219]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SP", "[", 
  RowBox[{"OPEDelta", ",", "OPEDelta"}], "]"}]], "Input",
 CellTags->"OPEDelta",
 CellLabel->"In[1220]:="],

Cell[BoxData[
 FormBox["0", TraditionalForm]], "Output",
 CellChangeTimes->{3.5009703374197206`*^9},
 CellTags->"OPEDelta",
 CellLabel->"Out[1220]="]
}, Open  ]]
}, Open  ]],

Cell["OPEi, OPEj, OPEk, OPEl, OPEm, OPEn, OPEo", "ObjectName",
 CellTags->"OPEi"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"OPEi"],

Cell["\<\
OPEi, etc. are variables with DataType PositiveInteger which are used in \
functions relating to the operator product expansion.\
\>", "Text",
 CellTags->"OPEi"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"OPEi"],

Cell[CellGroupData[{

Cell[BoxData["OPEi"], "Input",
 CellTags->"OPEi",
 CellLabel->"In[1221]:="],

Cell[BoxData[
 FormBox["i", TraditionalForm]], "Output",
 CellChangeTimes->{3.500970337497903*^9},
 CellTags->"OPEi",
 CellLabel->"Out[1221]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DataType", "[", 
  RowBox[{
  "OPEi", ",", " ", "OPEj", ",", "OPEk", ",", "OPEl", ",", " ", "OPEm", ",", 
   " ", "OPEn", ",", " ", "OPEo", ",", " ", "PositiveInteger"}], 
  "]"}]], "Input",
 CellTags->"OPEi",
 CellLabel->"In[1222]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
   "True", ",", "True", ",", "True", ",", "True", ",", "True", ",", "True", 
    ",", "True"}], "}"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970337612411*^9},
 CellTags->"OPEi",
 CellLabel->"Out[1222]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PowerSimplify", "[", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{"-", "1"}], ")"}], "^", 
     RowBox[{"(", 
      RowBox[{"2", "OPEi"}], ")"}]}], ",", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"-", "1"}], ")"}], "^", 
     RowBox[{"(", 
      RowBox[{"2", "OPEj"}], ")"}]}], ",", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"-", "1"}], ")"}], "^", 
     RowBox[{"(", 
      RowBox[{"2", "OPEk"}], ")"}]}], ",", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"-", "1"}], ")"}], "^", 
     RowBox[{"(", 
      RowBox[{"2", "OPEl"}], ")"}]}], ",", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"-", "1"}], ")"}], "^", 
     RowBox[{"(", 
      RowBox[{"2", "OPEm"}], ")"}]}], ",", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"-", "1"}], ")"}], "^", 
     RowBox[{"(", 
      RowBox[{"2", "OPEn"}], ")"}]}], ",", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"-", "1"}], ")"}], "^", 
     RowBox[{"(", 
      RowBox[{"2", "OPEo"}], ")"}]}]}], "}"}], "]"}]], "Input",
 CellTags->"OPEi",
 CellLabel->"In[1223]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"1", ",", "1", ",", "1", ",", "1", ",", "1", ",", "1", ",", "1"}], 
   "}"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970337703239*^9},
 CellTags->"OPEi",
 CellLabel->"Out[1223]="]
}, Open  ]],

Cell["Re has been changed:", "Text",
 CellTags->"OPEi"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"Re", "[", "OPEi", "]"}], ">", 
    RowBox[{"-", "3"}]}], ",", " ", 
   RowBox[{
    RowBox[{"Re", "[", "OPEi", "]"}], ">", 
    RowBox[{"-", "2"}]}], ",", " ", 
   RowBox[{
    RowBox[{"Re", "[", "OPEi", "]"}], ">", 
    RowBox[{"-", "1"}]}], ",", "   ", 
   RowBox[{
    RowBox[{"Re", "[", "OPEi", "]"}], ">", "0"}], ",", " ", 
   RowBox[{
    RowBox[{"Re", "[", "OPEi", "]"}], ">", "1"}]}], "}"}]], "Input",
 CellTags->"OPEi",
 CellLabel->"In[1224]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     RowBox[{"Re", "(", "i", ")"}], ">", 
     RowBox[{"-", "3"}]}], ",", 
    RowBox[{
     RowBox[{"Re", "(", "i", ")"}], ">", 
     RowBox[{"-", "2"}]}], ",", 
    RowBox[{
     RowBox[{"Re", "(", "i", ")"}], ">", 
     RowBox[{"-", "1"}]}], ",", 
    RowBox[{
     RowBox[{"Re", "(", "i", ")"}], ">", "0"}], ",", 
    RowBox[{
     RowBox[{"Re", "(", "i", ")"}], ">", "1"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970337835216*^9},
 CellTags->"OPEi",
 CellLabel->"Out[1224]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"Re", "[", 
     RowBox[{
      RowBox[{"-", "OPEi"}], " ", "+", " ", "OPEm"}], "]"}], " ", ">", " ", 
    "0"}], ",", " ", 
   RowBox[{
    RowBox[{"Re", "[", 
     RowBox[{
      RowBox[{"-", "OPEi"}], " ", "+", " ", "OPEm"}], "]"}], " ", ">", " ", 
    "1"}], ",", 
   RowBox[{
    RowBox[{"Re", "[", 
     RowBox[{
      RowBox[{"-", "OPEi"}], " ", "+", " ", "OPEm"}], "]"}], " ", ">", " ", 
    "2"}]}], "}"}]], "Input",
 CellTags->"OPEi",
 CellLabel->"In[1225]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     RowBox[{"Re", "(", 
      RowBox[{"m", "-", "i"}], ")"}], ">", "0"}], ",", 
    RowBox[{
     RowBox[{"Re", "(", 
      RowBox[{"m", "-", "i"}], ")"}], ">", "1"}], ",", 
    RowBox[{
     RowBox[{"Re", "(", 
      RowBox[{"m", "-", "i"}], ")"}], ">", "2"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.50097033791523*^9},
 CellTags->"OPEi",
 CellLabel->"Out[1225]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"Re", "[", "OPEm", "]"}], ">", 
    RowBox[{"-", "3"}]}], ",", " ", 
   RowBox[{
    RowBox[{"Re", "[", "OPEm", "]"}], ">", 
    RowBox[{"-", "2"}]}], ",", " ", 
   RowBox[{
    RowBox[{"Re", "[", "OPEm", "]"}], ">", 
    RowBox[{"-", "1"}]}], ",", "   ", 
   RowBox[{
    RowBox[{"Re", "[", "OPEm", "]"}], ">", "0"}], ",", " ", 
   RowBox[{
    RowBox[{"Re", "[", "OPEm", "]"}], ">", "1"}]}], "}"}]], "Input",
 CellTags->"OPEi",
 CellLabel->"In[1226]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     RowBox[{"Re", "(", "m", ")"}], ">", 
     RowBox[{"-", "3"}]}], ",", 
    RowBox[{
     RowBox[{"Re", "(", "m", ")"}], ">", 
     RowBox[{"-", "2"}]}], ",", 
    RowBox[{
     RowBox[{"Re", "(", "m", ")"}], ">", 
     RowBox[{"-", "1"}]}], ",", 
    RowBox[{
     RowBox[{"Re", "(", "m", ")"}], ">", "0"}], ",", 
    RowBox[{
     RowBox[{"Re", "(", "m", ")"}], ">", "1"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970338012287*^9},
 CellTags->"OPEi",
 CellLabel->"Out[1226]="]
}, Open  ]]
}, Open  ]],

Cell["OPEInsert  ***unfinished***", "ObjectName",
 CellTags->"OPEInsert"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"OPEInsert"],

Cell["\<\
OPEInsert[diagram_String, name_, rhifile_String] or OPEInsert[diagram_String, \
name_]. The setting of the option EpsContract (4 or D) determines whether the \
Levi-Civita tensors are contracted in 4 or D dimensions.\
\>", "Text",
 CellTags->"OPEInsert"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["RHI",
  BaseStyle->"AddOnsLink",
  ButtonData:>"RHI",
  ButtonNote->"RHI"],
 "."
}], "Text",
 CellTags->"OPEInsert"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"OPEInsert"],

Cell[BoxData[""], "Input",
 CellTags->"OPEInsert",
 CellLabel->"In[1227]:="]
}, Open  ]],

Cell["OPEInt  ***unfinished***", "ObjectName",
 CellTags->"OPEInt"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"OPEInt"],

Cell["\<\
OPEInt[expr, q, p, x].  The dimension is changed to the one indicated by the \
option  Dimension. The setting of the option EpsContract determines  the \
dimension in which the Levi-Civita tensors are contracted.\
\>", "Text",
 CellTags->"OPEInt"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["RHI",
  BaseStyle->"AddOnsLink",
  ButtonData:>"RHI",
  ButtonNote->"RHI"],
 "."
}], "Text",
 CellTags->"OPEInt"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"OPEInt"],

Cell[BoxData[""], "Input",
 CellTags->"OPEInt",
 CellLabel->"In[1228]:="]
}, Open  ]],

Cell["OPEIntDelta  ***unfinished***", "ObjectName",
 CellTags->"OPEIntDelta"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"OPEIntDelta"],

Cell[TextData[{
 "OPEIntDelta[expr, x, m] introduces the delta(1-x) (DeltaFunction[1-x]). The \
",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " Integrate function is called and each integration  (from 0 to 1) is \
recorded for reference (and bug-checking) in the global list $MIntegrate. \n\
Notice that the dimension specified by the option should also be the \
dimension used in expr. It is replaced in OPEIntDelta by (4+Epsilon)."
}], "Text",
 CellTags->"OPEIntDelta"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["RHI",
  BaseStyle->"AddOnsLink",
  ButtonData:>"RHI",
  ButtonNote->"RHI"],
 "."
}], "Text",
 CellTags->"OPEIntDelta"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"OPEIntDelta"],

Cell[BoxData[""], "Input",
 CellTags->"OPEIntDelta",
 CellLabel->"In[1229]:="]
}, Open  ]],

Cell["OPEIntegrate  ***unfinished***", "ObjectName",
 CellTags->"OPEIntegrate"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"OPEIntegrate"],

Cell["\<\
OPEIntegrate[expr, q, x].  The dimension is changed to the one indicated by \
the option  Dimension. The setting of the option EpsContract determines  the \
dimension in which the Levi-Civita tensors are contracted.\
\>", "Text",
 CellTags->"OPEIntegrate"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["RHI",
  BaseStyle->"AddOnsLink",
  ButtonData:>"RHI",
  ButtonNote->"RHI"],
 "."
}], "Text",
 CellTags->"OPEIntegrate"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"OPEIntegrate"],

Cell[BoxData[""], "Input",
 CellTags->"OPEIntegrate",
 CellLabel->"In[1230]:="]
}, Open  ]],

Cell["OPEIntegrateDelta  ***unfinished***", "ObjectName",
 CellTags->"OPEIntegrateDelta"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"OPEIntegrateDelta"],

Cell[TextData[{
 "OPEIntegrateDelta[expr, x, m] introduces the \[Delta](1-x) \
(DeltaFunction[1-x]). The ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " Integrate function is called and each integration  (from 0 to 1) is \
recorded for reference (and bug-checking) in the global list $MIntegrate. \n\
Notice that the dimension specified by the option should also be the \
dimension used in expr. It is replaced in OPEIntegrateDelta by (4+Epsilon)."
}], "Text",
 CellTags->"OPEIntegrateDelta"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["RHI",
  BaseStyle->"AddOnsLink",
  ButtonData:>"RHI",
  ButtonNote->"RHI"],
 "."
}], "Text",
 CellTags->"OPEIntegrateDelta"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"OPEIntegrateDelta"],

Cell[BoxData[""], "Input",
 CellTags->"OPEIntegrateDelta",
 CellLabel->"In[1231]:="]
}, Open  ]],

Cell["OPESumExplicit", "ObjectName",
 CellTags->"OPESumExplicit"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"OPESumExplicit"],

Cell["OPESumExplicit[exp] calculates OPESum's.", "Text",
 CellTags->"OPESumExplicit"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["OPESum",
  BaseStyle->"AddOnsLink",
  ButtonData:>"OPESum",
  ButtonNote->"OPESum"],
 ", ",
 ButtonBox["OPESumSimplify",
  BaseStyle->"AddOnsLink",
  ButtonData:>"OPESumSimplify",
  ButtonNote->"OPESumSimplify"],
 "."
}], "Text",
 CellTags->"OPESumExplicit"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"OPESumExplicit"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t1", "=", 
  RowBox[{"OPESum", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"A", "^", "i"}], 
     RowBox[{"B", "^", 
      RowBox[{"(", 
       RowBox[{"m", "-", "i", "-", "3"}], ")"}]}]}], ",", 
    RowBox[{"{", 
     RowBox[{"i", ",", "0", ",", 
      RowBox[{"m", "-", "3"}]}], "}"}]}], "]"}]}]], "Input",
 CellTags->"OPESumExplicit",
 CellLabel->"In[1232]:="],

Cell[BoxData[
 FormBox[
  TagBox[
   RowBox[{
    UnderoverscriptBox["\[Sum]", 
     RowBox[{"i", "=", "0"}], 
     RowBox[{
      RowBox[{"-", "3"}], "+", "m"}]], 
    RowBox[{
     SuperscriptBox["A", "i"], " ", 
     SuperscriptBox["B", 
      RowBox[{
       RowBox[{"-", "3"}], "-", "i", "+", "m"}]]}]}],
   HoldForm], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970338605612*^9},
 CellTags->"OPESumExplicit",
 CellLabel->"Out[1232]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"OPESumExplicit", "[", "t1", "]"}]], "Input",
 CellTags->"OPESumExplicit",
 CellLabel->"In[1233]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    FormBox[
     SuperscriptBox["A", 
      RowBox[{"m", "-", "2"}]],
     TraditionalForm], 
    RowBox[{"A", "-", "B"}]], "-", 
   FractionBox[
    FormBox[
     SuperscriptBox["B", 
      RowBox[{"m", "-", "2"}]],
     TraditionalForm], 
    RowBox[{"A", "-", "B"}]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970339044011*^9},
 CellTags->"OPESumExplicit",
 CellLabel->"Out[1233]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t2", " ", "=", " ", 
  RowBox[{"OPESum", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"a", "^", "i"}], 
     RowBox[{"b", "^", 
      RowBox[{"(", 
       RowBox[{"j", "-", "i"}], ")"}]}], 
     RowBox[{"c", "^", 
      RowBox[{"(", 
       RowBox[{"m", "-", "j", "-", "4"}], ")"}]}]}], ",", 
    RowBox[{"{", 
     RowBox[{"i", ",", "0", ",", "j"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"j", ",", "0", ",", 
      RowBox[{"m", "-", "4"}]}], "}"}]}], "]"}]}]], "Input",
 CellTags->"OPESumExplicit",
 CellLabel->"In[1234]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox[
    TagBox[
     RowBox[{
      UnderoverscriptBox["\[Sum]", 
       RowBox[{"j", "=", "0"}], 
       RowBox[{
        RowBox[{"-", "4"}], "+", "m"}]], "\<\"\"\>"}],
     HoldForm],
    TraditionalForm], 
   FormBox[
    RowBox[{"\<\"\"\>", " ", 
     RowBox[{"(", 
      RowBox[{"j", "+", "1"}], ")"}]}],
    TraditionalForm], 
   FormBox[
    RowBox[{
     SuperscriptBox["a", "i"], " ", 
     SuperscriptBox["b", 
      RowBox[{"j", "-", "i"}]], " ", 
     SuperscriptBox["c", 
      RowBox[{
       RowBox[{"-", "j"}], "+", "m", "-", "4"}]]}],
    TraditionalForm]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970339175178*^9},
 CellTags->"OPESumExplicit",
 CellLabel->"Out[1234]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"OPESumExplicit", "[", "t2", "]"}]], "Input",
 CellTags->"OPESumExplicit",
 CellLabel->"In[1235]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    FormBox[
     SuperscriptBox["a", 
      RowBox[{"m", "-", "2"}]],
     TraditionalForm], 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"a", "-", "b"}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{"a", "-", "c"}], ")"}]}]], "+", 
   FractionBox[
    RowBox[{"c", " ", 
     FormBox[
      SuperscriptBox["b", 
       RowBox[{"m", "-", "2"}]],
      TraditionalForm]}], 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"a", "-", "b"}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{"a", "-", "c"}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{"b", "-", "c"}], ")"}]}]], "-", 
   FractionBox[
    RowBox[{"a", " ", 
     FormBox[
      SuperscriptBox["b", 
       RowBox[{"m", "-", "2"}]],
      TraditionalForm]}], 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"a", "-", "b"}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{"a", "-", "c"}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{"b", "-", "c"}], ")"}]}]], "-", 
   FractionBox[
    RowBox[{"b", " ", 
     FormBox[
      SuperscriptBox["c", 
       RowBox[{"m", "-", "2"}]],
      TraditionalForm]}], 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"a", "-", "b"}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{"a", "-", "c"}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{"b", "-", "c"}], ")"}]}]], "+", 
   FractionBox[
    RowBox[{"a", " ", 
     FormBox[
      SuperscriptBox["c", 
       RowBox[{"m", "-", "2"}]],
      TraditionalForm]}], 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"a", "-", "b"}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{"a", "-", "c"}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{"b", "-", "c"}], ")"}]}]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009703427713842`*^9},
 CellTags->"OPESumExplicit",
 CellLabel->"Out[1235]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", 
   RowBox[{"t1", ",", "t2"}], "]"}], ";"}]], "Input",
 CellTags->"OPESumExplicit",
 CellLabel->"In[1236]:="]
}, Open  ]],

Cell["OPESum", "ObjectName",
 CellTags->"OPESum"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"OPESum"],

Cell["\<\
OPESum[exp, {i, 0, m}] denotes a symbolic sum.The syntax is the same as for \
Sum.\
\>", "Text",
 CellTags->"OPESum"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["OPESumExplicit",
  BaseStyle->"AddOnsLink",
  ButtonData:>"OPESumExplicit",
  ButtonNote->"OPESumExplicit"],
 ", ",
 ButtonBox["OPESumSimplify",
  BaseStyle->"AddOnsLink",
  ButtonData:>"OPESumSimplify",
  ButtonNote->"OPESumSimplify"],
 "."
}], "Text",
 CellTags->"OPESum"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"OPESum"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t1", "=", 
  RowBox[{"OPESum", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"SO", "[", "p", "]"}], "^", "OPEi"}], 
     RowBox[{
      RowBox[{"SO", "[", "k", "]"}], "^", 
      RowBox[{"(", 
       RowBox[{"OPEm", "-", "OPEi", "-", "3"}], ")"}]}]}], ",", 
    RowBox[{"{", 
     RowBox[{"OPEi", ",", "0", ",", 
      RowBox[{"OPEm", "-", "3"}]}], "}"}]}], "]"}]}]], "Input",
 CellTags->"OPESum",
 CellLabel->"In[1237]:="],

Cell[BoxData[
 FormBox[
  TagBox[
   RowBox[{
    UnderoverscriptBox["\[Sum]", 
     RowBox[{"i", "=", "0"}], 
     RowBox[{
      RowBox[{"-", "3"}], "+", "m"}]], 
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        FormBox["\<\"\[CapitalDelta]\"\>",
         TraditionalForm], 
        FormBox["\<\"\[CenterDot]\"\>",
         TraditionalForm], 
        FormBox["k",
         TraditionalForm]}], ")"}], 
      RowBox[{
       RowBox[{"-", "3"}], "-", "i", "+", "m"}]], " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        FormBox["\<\"\[CapitalDelta]\"\>",
         TraditionalForm], 
        FormBox["\<\"\[CenterDot]\"\>",
         TraditionalForm], 
        FormBox["p",
         TraditionalForm]}], ")"}], "i"]}]}],
   HoldForm], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970343196699*^9},
 CellTags->"OPESum",
 CellLabel->"Out[1237]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"OPESumExplicit", "[", "t1", "]"}]], "Input",
 CellTags->"OPESum",
 CellLabel->"In[1238]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    FormBox[
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        FormBox["\<\"\[CapitalDelta]\"\>",
         TraditionalForm], 
        FormBox["\<\"\[CenterDot]\"\>",
         TraditionalForm], 
        FormBox["k",
         TraditionalForm]}], ")"}], 
      RowBox[{"m", "-", "2"}]],
     TraditionalForm], 
    RowBox[{
     RowBox[{
      FormBox["\<\"\[CapitalDelta]\"\>",
       TraditionalForm], 
      FormBox["\<\"\[CenterDot]\"\>",
       TraditionalForm], 
      FormBox["k",
       TraditionalForm]}], "-", 
     RowBox[{
      FormBox["\<\"\[CapitalDelta]\"\>",
       TraditionalForm], 
      FormBox["\<\"\[CenterDot]\"\>",
       TraditionalForm], 
      FormBox["p",
       TraditionalForm]}]}]], "-", 
   FractionBox[
    FormBox[
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        FormBox["\<\"\[CapitalDelta]\"\>",
         TraditionalForm], 
        FormBox["\<\"\[CenterDot]\"\>",
         TraditionalForm], 
        FormBox["p",
         TraditionalForm]}], ")"}], 
      RowBox[{"m", "-", "2"}]],
     TraditionalForm], 
    RowBox[{
     RowBox[{
      FormBox["\<\"\[CapitalDelta]\"\>",
       TraditionalForm], 
      FormBox["\<\"\[CenterDot]\"\>",
       TraditionalForm], 
      FormBox["k",
       TraditionalForm]}], "-", 
     RowBox[{
      FormBox["\<\"\[CapitalDelta]\"\>",
       TraditionalForm], 
      FormBox["\<\"\[CenterDot]\"\>",
       TraditionalForm], 
      FormBox["p",
       TraditionalForm]}]}]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009703432973137`*^9},
 CellTags->"OPESum",
 CellLabel->"Out[1238]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t2", " ", "=", " ", 
  RowBox[{"OPESum", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"a", "^", "i"}], 
     RowBox[{"b", "^", 
      RowBox[{"(", 
       RowBox[{"j", "-", "i"}], ")"}]}], 
     RowBox[{"c", "^", 
      RowBox[{"(", 
       RowBox[{"m", "-", "j", "-", "4"}], ")"}]}]}], ",", 
    RowBox[{"{", 
     RowBox[{"i", ",", "0", ",", "j"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"j", ",", "0", ",", 
      RowBox[{"m", "-", "4"}]}], "}"}]}], "]"}]}]], "Input",
 CellTags->"OPESum",
 CellLabel->"In[1239]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox[
    TagBox[
     RowBox[{
      UnderoverscriptBox["\[Sum]", 
       RowBox[{"j", "=", "0"}], 
       RowBox[{
        RowBox[{"-", "4"}], "+", "m"}]], "\<\"\"\>"}],
     HoldForm],
    TraditionalForm], 
   FormBox[
    RowBox[{"\<\"\"\>", " ", 
     RowBox[{"(", 
      RowBox[{"j", "+", "1"}], ")"}]}],
    TraditionalForm], 
   FormBox[
    RowBox[{
     SuperscriptBox["a", "i"], " ", 
     SuperscriptBox["b", 
      RowBox[{"j", "-", "i"}]], " ", 
     SuperscriptBox["c", 
      RowBox[{
       RowBox[{"-", "j"}], "+", "m", "-", "4"}]]}],
    TraditionalForm]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970343461033*^9},
 CellTags->"OPESum",
 CellLabel->"Out[1239]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"OPESumExplicit", "[", "t2", "]"}]], "Input",
 CellTags->"OPESum",
 CellLabel->"In[1240]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    FormBox[
     SuperscriptBox["a", 
      RowBox[{"m", "-", "2"}]],
     TraditionalForm], 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"a", "-", "b"}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{"a", "-", "c"}], ")"}]}]], "+", 
   FractionBox[
    RowBox[{"c", " ", 
     FormBox[
      SuperscriptBox["b", 
       RowBox[{"m", "-", "2"}]],
      TraditionalForm]}], 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"a", "-", "b"}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{"a", "-", "c"}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{"b", "-", "c"}], ")"}]}]], "-", 
   FractionBox[
    RowBox[{"a", " ", 
     FormBox[
      SuperscriptBox["b", 
       RowBox[{"m", "-", "2"}]],
      TraditionalForm]}], 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"a", "-", "b"}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{"a", "-", "c"}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{"b", "-", "c"}], ")"}]}]], "-", 
   FractionBox[
    RowBox[{"b", " ", 
     FormBox[
      SuperscriptBox["c", 
       RowBox[{"m", "-", "2"}]],
      TraditionalForm]}], 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"a", "-", "b"}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{"a", "-", "c"}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{"b", "-", "c"}], ")"}]}]], "+", 
   FractionBox[
    RowBox[{"a", " ", 
     FormBox[
      SuperscriptBox["c", 
       RowBox[{"m", "-", "2"}]],
      TraditionalForm]}], 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"a", "-", "b"}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{"a", "-", "c"}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{"b", "-", "c"}], ")"}]}]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970343578367*^9},
 CellTags->"OPESum",
 CellLabel->"Out[1240]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{" ", 
  RowBox[{"Clear", "[", 
   RowBox[{"t1", ",", "t2"}], "]"}]}]], "Input",
 CellTags->"OPESum",
 CellLabel->"In[1241]:="]
}, Open  ]],

Cell["OPESumSimplify", "ObjectName",
 CellTags->"OPESumSimplify"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"OPESumSimplify"],

Cell["OPESumSimplify[exp] simplifies OPESum's in exp.", "Text",
 CellTags->"OPESumSimplify"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["OPESum",
  BaseStyle->"AddOnsLink",
  ButtonData:>"OPESum",
  ButtonNote->"OPESum"],
 ", ",
 ButtonBox["OPESumExplicit",
  BaseStyle->"AddOnsLink",
  ButtonData:>"OPESumExplicit",
  ButtonNote->"OPESumExplicit"],
 "."
}], "Text",
 CellTags->"OPESumSimplify"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"OPESumSimplify"],

Cell[CellGroupData[{

Cell["\<\
OPESum[(-SOD[p])^(OPEi+1) SOD[p-q]^(OPEm-OPEi-2),{OPEi,0,OPEm}]\
\>", "Input",
 CellTags->"OPESumSimplify",
 CellLabel->"In[1242]:="],

Cell[BoxData[
 FormBox[
  TagBox[
   RowBox[{
    UnderoverscriptBox["\[Sum]", 
     RowBox[{"i", "=", "0"}], "m"], 
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"-", 
        RowBox[{"(", 
         RowBox[{
          FormBox["\<\"\[CapitalDelta]\"\>",
           TraditionalForm], 
          FormBox["\<\"\[CenterDot]\"\>",
           TraditionalForm], 
          FormBox["p",
           TraditionalForm]}], ")"}]}], ")"}], 
      RowBox[{"1", "+", "i"}]], " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        FormBox["\<\"\[CapitalDelta]\"\>",
         TraditionalForm], 
        FormBox["\<\"\[CenterDot]\"\>",
         TraditionalForm], 
        FormBox["\<\"(\"\>",
         TraditionalForm], 
        FormBox[
         RowBox[{"p", "-", "q"}],
         TraditionalForm], 
        FormBox["\<\")\"\>",
         TraditionalForm]}], ")"}], 
      RowBox[{
       RowBox[{"-", "2"}], "-", "i", "+", "m"}]]}]}],
   HoldForm], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009703436714373`*^9},
 CellTags->"OPESumSimplify",
 CellLabel->"Out[1242]="]
}, Open  ]],

Cell[CellGroupData[{

Cell["OPESumSimplify[%]", "Input",
 CellTags->"OPESumSimplify",
 CellLabel->"In[1243]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    FormBox["\[CapitalDelta]",
     TraditionalForm], "\[CenterDot]", 
    FormBox["p",
     TraditionalForm]}], " ", 
   RowBox[{"(", 
    RowBox[{"-", 
     FormBox[
      TagBox[
       RowBox[{
        UnderoverscriptBox["\[Sum]", 
         RowBox[{"i", "=", "0"}], "m"], 
        RowBox[{
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"-", "1"}], ")"}], "i"], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["p",
             TraditionalForm]}], ")"}], "i"], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", "(", 
            FormBox[
             RowBox[{"p", "-", "q"}],
             TraditionalForm], ")"}], ")"}], 
          RowBox[{
           RowBox[{"-", "2"}], "-", "i", "+", "m"}]]}]}],
       HoldForm],
      TraditionalForm]}], ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970343766663*^9},
 CellTags->"OPESumSimplify",
 CellLabel->"Out[1243]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"OPESumSimplify", "[", 
  RowBox[{
   RowBox[{"OPESum", "[", 
    RowBox[{"{", 
     RowBox[{"OPEi", ",", "0", ",", "OPEm"}], "}"}], "]"}], " ", 
   RowBox[{"a", "^", "OPEi"}]}], "]"}]], "Input",
 CellTags->"OPESumSimplify",
 CellLabel->"In[1244]:="],

Cell[BoxData[
 FormBox[
  TagBox[
   RowBox[{
    UnderoverscriptBox["\[Sum]", 
     RowBox[{"i", "=", "0"}], "m"], 
    FormBox[
     SuperscriptBox["a", "i"],
     TraditionalForm]}],
   HoldForm], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970343845517*^9},
 CellTags->"OPESumSimplify",
 CellLabel->"Out[1244]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"OPESumSimplify", "[", 
  RowBox[{
   RowBox[{"OPESum", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"j", ",", "0", ",", "i"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", "0", ",", "m"}], "}"}]}], "]"}], " ", 
   RowBox[{"a", "^", 
    RowBox[{"(", 
     RowBox[{"j", "-", "i"}], ")"}]}], " ", 
   RowBox[{"b", "^", "i"}]}], "]"}]], "Input",
 CellTags->"OPESumSimplify",
 CellLabel->"In[1245]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox[
    TagBox[
     RowBox[{
      UnderoverscriptBox["\[Sum]", 
       RowBox[{"i", "=", "0"}], "m"], "\<\"\"\>"}],
     HoldForm],
    TraditionalForm], 
   FormBox[
    RowBox[{"\<\"\"\>", " ", 
     RowBox[{"(", 
      RowBox[{"i", "+", "1"}], ")"}]}],
    TraditionalForm], 
   FormBox[
    RowBox[{
     SuperscriptBox["b", "i"], " ", 
     SuperscriptBox["a", 
      RowBox[{"j", "-", "i"}]]}],
    TraditionalForm]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970344143498*^9},
 CellTags->"OPESumSimplify",
 CellLabel->"Out[1245]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "StandardForm"}]], "Input",
 CellTags->"OPESumSimplify",
 CellLabel->"In[1246]:="],

Cell[BoxData[
 RowBox[{"OPESum", "[", 
  RowBox[{
   RowBox[{
    SuperscriptBox["a", 
     RowBox[{
      RowBox[{"-", "i"}], "+", "j"}]], " ", 
    SuperscriptBox["b", "i"]}], ",", 
   RowBox[{"{", 
    RowBox[{"i", ",", "0", ",", "m"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"j", ",", "0", ",", "i"}], "}"}]}], "]"}]], "Output",
 CellChangeTimes->{3.500970344219707*^9},
 CellTags->"OPESumSimplify",
 CellLabel->"Out[1246]//StandardForm="]
}, Open  ]]
}, Open  ]],

Cell["OPE1Loop ", "ObjectName",
 CellTags->"OPE1Loop"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"OPE1Loop"],

Cell["\<\
OPE1Loop[q1, amp].  OPE1Loop[{q1,q2}, amp] does sub-loop  decomposition.\
\>", "Text",
 CellTags->"OPE1Loop"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["OPESum",
  BaseStyle->"AddOnsLink",
  ButtonData:>"OPESum",
  ButtonNote->"OPESum"],
 "."
}], "Text",
 CellTags->"OPE1Loop"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"OPE1Loop"],

Cell[BoxData[""], "Input",
 CellTags->"OPE1Loop",
 CellLabel->"In[1247]:="]
}, Open  ]],

Cell["OPE2TID  ***unfinished***", "ObjectName",
 CellTags->"OPE2TID"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"OPE2TID"],

Cell["\<\
OPE2TID[exp, k1, k2, p]. The setting of the option EpsContract determines the \
dimension in which the Levi-Civita tensors are contracted.\
\>", "Text",
 CellTags->"OPE2TID"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "OPE2TID", "]"}]], "Input",
 CellTags->"OPE2TID",
 CellLabel->"In[1248]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Uncontract", "\[Rule]", "False"}], ",", 
    RowBox[{"Contract", "\[Rule]", "True"}], ",", 
    RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
    RowBox[{"EpsContract", "\[Rule]", "False"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.5009703444338837`*^9},
 CellTags->"OPE2TID",
 CellLabel->"Out[1248]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["OPESum",
  BaseStyle->"AddOnsLink",
  ButtonData:>"OPESum",
  ButtonNote->"OPESum"],
 "."
}], "Text",
 CellTags->"OPE2TID"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"OPE2TID"],

Cell[BoxData[""], "Input",
 CellTags->"OPE2TID",
 CellLabel->"In[1249]:="]
}, Open  ]],

Cell["OptionsSelect", "ObjectName",
 CellTags->"OptionsSelect"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"OptionsSelect"],

Cell["\<\
OptionsSelect[function,opts] returns the option settings of opts accepted by \
function.  When an option occurs several times in opts, the first setting is \
selected.\
\>", "Text",
 CellTags->"OptionsSelect"]
}, Open  ]],

Cell["Pair", "ObjectName",
 CellTags->"Pair"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Pair"],

Cell[TextData[{
 "Pair[x, y] is the head of a special pairing used in the internal \
representation: x and y may have heads LorentzIndex or Momentum. If both x \
and y have head LorentzIndex, the metric tensor is understood. If x and y \
have head Momentum, a scalar product is meant. If one of x and y has head \
LorentzIndex and the other Momentum, a Lorentz vector ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["p", "\[Mu]"], TraditionalForm]]],
 " is understood."
}], "Text",
 CellTags->"Pair"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FourVector",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FourVector",
  ButtonNote->"FourVector"],
 ", ",
 ButtonBox["FV",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FV",
  ButtonNote->"FV"],
 ", ",
 ButtonBox["FVD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FVD",
  ButtonNote->"FVD"],
 ", ",
 ButtonBox["MetricTensor",
  BaseStyle->"AddOnsLink",
  ButtonData:>"MetricTensor",
  ButtonNote->"MetricTensor"],
 ", ",
 ButtonBox["MT",
  BaseStyle->"AddOnsLink",
  ButtonData:>"MT",
  ButtonNote->"MT"],
 ", ",
 ButtonBox["MTD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"MTD",
  ButtonNote->"MTD"],
 ", ",
 ButtonBox["ScalarProduct",
  BaseStyle->"AddOnsLink",
  ButtonData:>"ScalarProduct",
  ButtonNote->"ScalarProduct"],
 ", ",
 ButtonBox["SP",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SP",
  ButtonNote->"SP"],
 ", ",
 ButtonBox["SPD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SPD",
  ButtonNote->"SPD"],
 "."
}], "Text",
 CellTags->"Pair"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Pair"],

Cell["This represents a four-dimensional metric tensor.", "Text",
 CellTags->"Pair"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Pair", "[", 
  RowBox[{
   RowBox[{"LorentzIndex", "[", "\[Alpha]", "]"}], ",", 
   RowBox[{"LorentzIndex", "[", "\[Beta]", "]"}]}], "]"}]], "Input",
 CellTags->"Pair",
 CellLabel->"In[1250]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["g", 
   RowBox[{"\[Alpha]", "\[Beta]"}]], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009703445259867`*^9},
 CellTags->"Pair",
 CellLabel->"Out[1250]="]
}, Open  ]],

Cell["This is a D-dimensional metric tensor.", "Text",
 CellTags->"Pair"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Pair", "[", 
  RowBox[{
   RowBox[{"LorentzIndex", "[", 
    RowBox[{"\[Alpha]", ",", "D"}], "]"}], ",", 
   RowBox[{"LorentzIndex", "[", 
    RowBox[{"\[Beta]", ",", "D"}], "]"}]}], "]"}]], "Input",
 CellTags->"Pair",
 CellLabel->"In[1251]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["g", 
   RowBox[{"\[Alpha]", "\[Beta]"}]], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970344570422*^9},
 CellTags->"Pair",
 CellLabel->"Out[1251]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Pair", "[", 
  RowBox[{
   RowBox[{"LorentzIndex", "[", "\[Alpha]", "]"}], ",", 
   RowBox[{"Momentum", "[", "p", "]"}]}], "]"}]], "Input",
 CellTags->"Pair",
 CellLabel->"In[1252]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox[
   FormBox["p",
    TraditionalForm], 
   FormBox[
    FormBox["\[Alpha]",
     TraditionalForm],
    TraditionalForm]], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970344661536*^9},
 CellTags->"Pair",
 CellLabel->"Out[1252]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Pair", "[", 
  RowBox[{
   RowBox[{"Momentum", "[", "q", "]"}], ",", 
   RowBox[{"Momentum", "[", "p", "]"}]}], "]"}]], "Input",
 CellTags->"Pair",
 CellLabel->"In[1253]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox["p",
    TraditionalForm], "\[CenterDot]", 
   FormBox["q",
    TraditionalForm]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970344753072*^9},
 CellTags->"Pair",
 CellLabel->"Out[1253]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Pair", "[", 
  RowBox[{
   RowBox[{"Momentum", "[", "p", "]"}], ",", 
   RowBox[{"Momentum", "[", "p", "]"}]}], "]"}]], "Input",
 CellTags->"Pair",
 CellLabel->"In[1254]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox[
   FormBox["p",
    TraditionalForm], "2"], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970344833468*^9},
 CellTags->"Pair",
 CellLabel->"Out[1254]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Pair", "[", 
  RowBox[{
   RowBox[{"Momentum", "[", 
    RowBox[{"p", "-", "q"}], "]"}], ",", 
   RowBox[{"Momentum", "[", "p", "]"}]}], "]"}]], "Input",
 CellTags->"Pair",
 CellLabel->"In[1255]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox["p",
    TraditionalForm], "\[CenterDot]", "(", 
   FormBox[
    RowBox[{"p", "-", "q"}],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970344914648*^9},
 CellTags->"Pair",
 CellLabel->"Out[1255]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Pair", "[", 
   RowBox[{
    RowBox[{"Momentum", "[", "p", "]"}], ",", 
    RowBox[{"Momentum", "[", "p", "]"}]}], "]"}], "^", "2"}]], "Input",
 CellTags->"Pair",
 CellLabel->"In[1256]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox[
   FormBox[
    SuperscriptBox[
     FormBox["p",
      TraditionalForm], "2"],
    TraditionalForm], "2"], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970345568534*^9},
 CellTags->"Pair",
 CellLabel->"Out[1256]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Pair", "[", 
   RowBox[{
    RowBox[{"Momentum", "[", "p", "]"}], ",", 
    RowBox[{"Momentum", "[", "p", "]"}]}], "]"}], "^", "3"}]], "Input",
 CellTags->"Pair",
 CellLabel->"In[1257]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox[
   FormBox[
    SuperscriptBox[
     FormBox["p",
      TraditionalForm], "2"],
    TraditionalForm], "3"], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009703456115923`*^9},
 CellTags->"Pair",
 CellLabel->"Out[1257]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Pair", "[", 
   RowBox[{
    RowBox[{"LorentzIndex", "[", 
     RowBox[{"\[Alpha]", ",", 
      RowBox[{"n", "-", "4"}]}], "]"}], ",", 
    RowBox[{"LorentzIndex", "[", "\[Beta]", "]"}]}], "]"}], "//", 
  "Contract"}]], "Input",
 CellTags->"Pair",
 CellLabel->"In[1258]:="],

Cell[BoxData[
 FormBox["0", TraditionalForm]], "Output",
 CellChangeTimes->{3.5009703456982527`*^9},
 CellTags->"Pair",
 CellLabel->"Out[1258]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ExpandScalarProduct", "[", 
  RowBox[{"Pair", "[", 
   RowBox[{
    RowBox[{"Momentum", "[", 
     RowBox[{"p", "-", "q"}], "]"}], ",", 
    RowBox[{"Momentum", "[", "p", "]"}]}], "]"}], "]"}]], "Input",
 CellTags->"Pair",
 CellLabel->"In[1259]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox[
    FormBox["p",
     TraditionalForm], "2"], "-", 
   RowBox[{
    FormBox["p",
     TraditionalForm], "\[CenterDot]", 
    FormBox["q",
     TraditionalForm]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970345783947*^9},
 CellTags->"Pair",
 CellLabel->"Out[1259]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Pair", "[", 
   RowBox[{
    RowBox[{"Momentum", "[", 
     RowBox[{"-", "q"}], "]"}], ",", 
    RowBox[{"Momentum", "[", "p", "]"}]}], "]"}], " ", "+", " ", 
  RowBox[{"Pair", "[", 
   RowBox[{
    RowBox[{"Momentum", "[", "q", "]"}], ",", 
    RowBox[{"Momentum", "[", "p", "]"}]}], "]"}]}]], "Input",
 CellTags->"Pair",
 CellLabel->"In[1260]:="],

Cell[BoxData[
 FormBox["0", TraditionalForm]], "Output",
 CellChangeTimes->{3.50097034587117*^9},
 CellTags->"Pair",
 CellLabel->"Out[1260]="]
}, Open  ]]
}, Open  ]],

Cell["PairCollect", "ObjectName",
 CellTags->"PairCollect"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"PairCollect"],

Cell["\<\
PairCollect is an option for DiracTrace specifying if the result is collected \
with respect to Pair's.\
\>", "Text",
 CellTags->"PairCollect"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["DiracTrace",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracTrace",
  ButtonNote->"DiracTrace"],
 ", ",
 ButtonBox["Pair",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Pair",
  ButtonNote->"Pair"],
 "."
}], "Text",
 CellTags->"PairCollect"]
}, Open  ]],

Cell["PairContract", "ObjectName",
 CellTags->"PairContract"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"PairContract"],

Cell["\<\
PairContract is like Pair, but with (local) contraction properties.\
\>", "Text",
 CellTags->"PairContract"],

Cell[TextData[{
 "See also: ",
 ButtonBox["Pair",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Pair",
  ButtonNote->"Pair"],
 ", ",
 ButtonBox["Contract",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Contract",
  ButtonNote->"Contract"],
 "."
}], "Text",
 CellTags->"PairContract"]
}, Open  ]],

Cell["PairContract2", "ObjectName",
 CellTags->"PairContract2"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"PairContract2"],

Cell["\<\
PairContract2 is like Pair, but with local contraction properties among \
PairContract2's.\
\>", "Text",
 CellTags->"PairContract2"],

Cell[TextData[{
 "See also: ",
 ButtonBox["Pair",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Pair",
  ButtonNote->"Pair"],
 ", ",
 ButtonBox["Contract",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Contract",
  ButtonNote->"Contract"],
 ", ",
 ButtonBox["PairContract",
  BaseStyle->"AddOnsLink",
  ButtonData:>"PairContract",
  ButtonNote->"PairContract"],
 "."
}], "Text",
 CellTags->"PairContract2"]
}, Open  ]],

Cell["PairContract3", "ObjectName",
 CellTags->"PairContract3"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"PairContract3"],

Cell["\<\
PairContract3 is like Pair, but with local contraction properties among \
PairContract3's.\
\>", "Text",
 CellTags->"PairContract3"],

Cell[TextData[{
 "See also: ",
 ButtonBox["Pair",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Pair",
  ButtonNote->"Pair"],
 ", ",
 ButtonBox["Contract",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Contract",
  ButtonNote->"Contract"],
 ", ",
 ButtonBox["PairContract",
  BaseStyle->"AddOnsLink",
  ButtonData:>"PairContract",
  ButtonNote->"PairContract"],
 "."
}], "Text",
 CellTags->"PairContract3"]
}, Open  ]],

Cell["PartialD", "ObjectName",
 CellTags->"PartialD"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"PartialD"],

Cell[TextData[{
 "PartialD[",
 Cell[BoxData[
  FormBox["\[Mu]", TraditionalForm]]],
 "] denotes the four-dimensional ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["\[PartialD]", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], "."}], TraditionalForm]]],
 " PartialD is used to denote derivative fields. PartialD[LorentzIndex[",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Mu]", ",", 
    StyleBox["D",
     FontSlant->"Plain"]}], TraditionalForm]]],
 "]] denotes the ",
 Cell[BoxData[
  FormBox[
   StyleBox["D",
    FontSlant->"Plain"], TraditionalForm]]],
 "-dimensional ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["\[PartialD]", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], "."}], TraditionalForm]]]
}], "Text",
 ZeroWidthTimes->True,
 CellTags->"PartialD"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["ExpandPartialD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"ExpandPartialD",
  ButtonNote->"ExpandPartialD"],
 ", ",
 ButtonBox["LeftPartialD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"LeftPartialD",
  ButtonNote->"LeftPartialD"],
 ", ",
 ButtonBox["LeftRightPartialD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"LeftRightPartialD",
  ButtonNote->"LeftRightPartialD"],
 ", ",
 ButtonBox["RightPartialD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"RightPartialD",
  ButtonNote->"RightPartialD"],
 "."
}], "Text",
 CellTags->"PartialD"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData["Examples"], "Subsubsection",
 CellTags->"PartialD"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"QuantumField", "[", 
   RowBox[{"A", ",", 
    RowBox[{"{", "\[Mu]", "}"}]}], "]"}], ".", 
  RowBox[{"LeftPartialD", "[", "\[Nu]", "]"}]}]], "Input",
 CellTags->"PartialD",
 CellLabel->"In[1261]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["A", "\[Mu]"], ".", 
   SubscriptBox[
    OverscriptBox["\[PartialD]", "\[LeftArrow]"], 
    FormBox[
     FormBox["\[Nu]",
      TraditionalForm],
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.50097034591782*^9},
 CellTags->"PartialD",
 CellLabel->"Out[1261]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ExpandPartialD", "[", "%", "]"}]], "Input",
 CellTags->"PartialD",
 CellLabel->"In[1262]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"QuantumField", "(", 
   RowBox[{
    RowBox[{
    "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
     FormBox["\[Nu]",
      TraditionalForm], ")"}], ",", "A", ",", 
    FormBox["\[Mu]",
     TraditionalForm]}], ")"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970346044388*^9},
 CellTags->"PartialD",
 CellLabel->"Out[1262]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StandardForm", "[", "%", "]"}]], "Input",
 CellTags->"PartialD",
 CellLabel->"In[1263]:="],

Cell[BoxData[
 RowBox[{"QuantumField", "[", 
  RowBox[{
   RowBox[{
   "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "[", 
    RowBox[{"LorentzIndex", "[", "\[Nu]", "]"}], "]"}], ",", "A", ",", 
   RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}]}], "]"}]], "Output",
 CellChangeTimes->{3.500970346120535*^9},
 CellTags->"PartialD",
 CellLabel->"Out[1263]//StandardForm="]
}, Open  ]]
}, Open  ]],

Cell["PartialDRelations", "ObjectName",
 CellTags->"PartialDRelations"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"PartialDRelations"],

Cell["\<\
PartialDRelations is an option for ExpandPartialD. It is a list of rules \
applied by ExpandPartialD at the end.\
\>", "Text",
 ZeroWidthTimes->True,
 CellTags->"PartialDRelations"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["PartialD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"PartialD",
  ButtonNote->"PartialD"],
 ", ",
 ButtonBox["ExpandPartialD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"ExpandPartialD",
  ButtonNote->"ExpandPartialD"],
 ", ",
 ButtonBox["LeftPartialD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"LeftPartialD",
  ButtonNote->"LeftPartialD"],
 ", ",
 ButtonBox["LeftRightPartialD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"LeftRightPartialD",
  ButtonNote->"LeftRightPartialD"],
 ", ",
 ButtonBox["RightPartialD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"RightPartialD",
  ButtonNote->"RightPartialD"],
 "."
}], "Text",
 CellTags->"PartialDRelations"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData["Examples"], "Subsubsection",
 CellTags->"PartialDRelations"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"QuantumField", "[", 
   RowBox[{"A", ",", 
    RowBox[{"{", "\[Mu]", "}"}]}], "]"}], ".", 
  RowBox[{"QuantumField", "[", 
   RowBox[{"B", ",", 
    RowBox[{"{", "\[Mu]", "}"}]}], "]"}], ".", 
  RowBox[{"LeftPartialD", "[", "\[Nu]", "]"}]}]], "Input",
 CellTags->"PartialDRelations",
 CellLabel->"In[1264]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["A", "\[Mu]"], ".", 
   SubscriptBox["B", "\[Mu]"], ".", 
   SubscriptBox[
    OverscriptBox["\[PartialD]", "\[LeftArrow]"], 
    FormBox[
     FormBox["\[Nu]",
      TraditionalForm],
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009703461972313`*^9},
 CellTags->"PartialDRelations",
 CellLabel->"Out[1264]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ExpandPartialD", "[", 
  RowBox[{"%", ",", " ", 
   RowBox[{"PartialDRelations", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"A", "\[Rule]", "C"}], "}"}]}]}], "]"}]], "Input",
 CellTags->"PartialDRelations",
 CellLabel->"In[1265]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox[
     FormBox["C",
      TraditionalForm], "\[Mu]"], ".", 
    RowBox[{"QuantumField", "(", 
     RowBox[{
      RowBox[{
      "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
       FormBox["\[Nu]",
        TraditionalForm], ")"}], ",", "B", ",", 
      FormBox["\[Mu]",
       TraditionalForm]}], ")"}]}], "+", 
   RowBox[{
    RowBox[{"QuantumField", "(", 
     RowBox[{
      RowBox[{
      "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
       FormBox["\[Nu]",
        TraditionalForm], ")"}], ",", "C", ",", 
      FormBox["\[Mu]",
       TraditionalForm]}], ")"}], ".", 
    SubscriptBox["B", "\[Mu]"]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970346334569*^9},
 CellTags->"PartialDRelations",
 CellLabel->"Out[1265]="]
}, Open  ]]
}, Open  ]],

Cell["PartialFourVector", "ObjectName",
 CellTags->"PartialFourVector"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"PartialFourVector"],

Cell["\<\
PartialFourVector[exp, FourVector[p, mu]]  calculates the partial derivative \
of exp w.r.t. p(mu). PartialFourVector[exp, FourVector[p, mu], \
FourVector[p,nu], ...]  gives the multiple derivative.\
\>", "Text",
 ZeroWidthTimes->True,
 CellTags->"PartialFourVector"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FourVector",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FourVector",
  ButtonNote->"FourVector"],
 ", ",
 ButtonBox["Pair",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Pair",
  ButtonNote->"Pair"],
 ", ",
 ButtonBox["Contract",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Contract",
  ButtonNote->"Contract"],
 "."
}], "Text",
 CellTags->"PartialFourVector"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData["Examples"], "Subsubsection",
 CellTags->"PartialFourVector"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PartialFourVector", "[", 
  RowBox[{
   RowBox[{"a", " ", 
    SuperscriptBox["\[ExponentialE]", 
     SuperscriptBox[
      RowBox[{"FourVector", "[", 
       RowBox[{"p", ",", "\[Nu]"}], "]"}], "2"]]}], ",", 
   RowBox[{"FourVector", "[", 
    RowBox[{"p", ",", "\[Mu]"}], "]"}]}], "]"}]], "Input",
 CellTags->"PartialFourVector",
 CellLabel->"In[1266]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"2", " ", "a", " ", 
   SuperscriptBox["\[ExponentialE]", 
    SuperscriptBox[
     FormBox["p",
      TraditionalForm], "2"]], " ", 
   SuperscriptBox[
    FormBox["p",
     TraditionalForm], 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970346422001*^9},
 CellTags->"PartialFourVector",
 CellLabel->"Out[1266]="]
}, Open  ]]
}, Open  ]],

Cell["PartialIntegrate", "ObjectName",
 CellTags->"PartialIntegrate"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"PartialIntegrate"],

Cell["\<\
 PartialIntegrate[exp, ap, t] does a partial integration of the definite \
integral Integrate[exp,{t,0,1}], with ap the factor that is to be integrated \
and exp/ap the factor that is to be differentiated.\
\>", "Text",
 ZeroWidthTimes->True,
 CellTags->"PartialIntegrate"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "PartialIntegrate", "]"}]], "Input",
 CellTags->"PartialIntegrate",
 CellLabel->"In[1267]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"Integrate", "\[Rule]", "Integrate"}], "}"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970346525943*^9},
 CellTags->"PartialIntegrate",
 CellLabel->"Out[1267]="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData["Examples"], "Subsubsection",
 CellTags->"PartialIntegrate"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PartialIntegrate", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"f", "[", "x", "]"}], 
    RowBox[{"g", "[", "x", "]"}]}], ",", 
   RowBox[{"g", "[", "x", "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellTags->"PartialIntegrate",
 CellLabel->"In[1268]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    RowBox[{"(", 
     TagBox[
      RowBox[{
       RowBox[{
        RowBox[{"f", "(", "x", ")"}], " ", 
        RowBox[{"\[Integral]", 
         RowBox[{
          RowBox[{"g", "(", "x", ")"}], 
          RowBox[{"\[DifferentialD]", "x"}]}]}]}], "/.", "\[InvisibleSpace]", 
       
       RowBox[{"x", "\[Rule]", "0"}]}],
      HoldForm], ")"}]}], "+", 
   RowBox[{"(", 
    TagBox[
     RowBox[{
      RowBox[{
       RowBox[{"f", "(", "x", ")"}], " ", 
       RowBox[{"\[Integral]", 
        RowBox[{
         RowBox[{"g", "(", "x", ")"}], 
         RowBox[{"\[DifferentialD]", "x"}]}]}]}], "/.", "\[InvisibleSpace]", 
      RowBox[{"x", "\[Rule]", "1"}]}],
     HoldForm], ")"}], "-", 
   RowBox[{
    SubsuperscriptBox["\[Integral]", "0", "1"], 
    RowBox[{
     RowBox[{
      RowBox[{
       SuperscriptBox["f", "\[Prime]",
        MultilineFunction->None], "(", "x", ")"}], " ", 
      RowBox[{"(", 
       RowBox[{"\[Integral]", 
        RowBox[{
         RowBox[{"g", "(", "x", ")"}], 
         RowBox[{"\[DifferentialD]", "x"}]}]}], ")"}]}], 
     RowBox[{"\[DifferentialD]", "x"}]}]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970347452742*^9},
 CellTags->"PartialIntegrate",
 CellLabel->"Out[1268]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "InputForm"}]], "Input",
 CellLabel->"In[1269]:="],

Cell["\<\
-HoldForm[f[x]*Integrate[g[x], x] /. x -> 0] + 
 HoldForm[f[x]*Integrate[g[x], x] /. x -> 1] - 
 Integrate[Integrate[g[x], x]*Derivative[1][f][x], {x, 0, 1}]\
\>", "Output",
 CellChangeTimes->{3.500970348049575*^9},
 CellLabel->"Out[1269]//InputForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"f", "[", "x_", "]"}], "=", 
  RowBox[{"Integrate", "[", 
   RowBox[{
    RowBox[{"Log", "[", 
     RowBox[{
      RowBox[{"3", "x"}], "+", "2"}], "]"}], ",", "x"}], "]"}]}]], "Input",
 CellTags->"PartialIntegrate",
 CellLabel->"In[1270]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["1", "3"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"3", " ", "x"}], "+", "2"}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"log", "(", 
      RowBox[{
       RowBox[{"3", " ", "x"}], "+", "2"}], ")"}], "-", "1"}], ")"}]}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970348227975*^9},
 CellTags->"PartialIntegrate",
 CellLabel->"Out[1270]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"g", "[", "x_", "]"}], "=", 
  RowBox[{"D", "[", 
   RowBox[{
    FractionBox["1", 
     RowBox[{"Log", "[", 
      RowBox[{
       RowBox[{"3", "x"}], "+", "2"}], "]"}]], ",", "x"}], "]"}]}]], "Input",
 CellTags->"PartialIntegrate",
 CellLabel->"In[1271]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   FractionBox["3", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"3", " ", "x"}], "+", "2"}], ")"}], " ", 
     RowBox[{
      SuperscriptBox["log", "2"], "(", 
      RowBox[{
       RowBox[{"3", " ", "x"}], "+", "2"}], ")"}]}]]}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970348336388*^9},
 CellTags->"PartialIntegrate",
 CellLabel->"Out[1271]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Integrate", "[", 
   RowBox[{
    RowBox[{"PartialIntegrate", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"f", "[", "x", "]"}], 
       RowBox[{"g", "[", "x", "]"}]}], ",", 
      RowBox[{"f", "[", "x", "]"}], ",", "x"}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}], "//", 
  "FullSimplify"}]], "Input",
 CellTags->"PartialIntegrate",
 CellLabel->"In[1272]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["2", 
    RowBox[{"log", "(", "8", ")"}]], "-", 
   FractionBox["5", 
    RowBox[{"log", "(", "125", ")"}]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970349928636*^9},
 CellTags->"PartialIntegrate",
 CellLabel->"Out[1272]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Integrate", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"f", "[", "x", "]"}], 
     RowBox[{"g", "[", "x", "]"}]}], ",", 
    RowBox[{"{", 
     RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}], "//", 
  "Simplify"}]], "Input",
 CellTags->"PartialIntegrate",
 CellLabel->"In[1273]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["2", 
    RowBox[{"log", "(", "8", ")"}]], "-", 
   FractionBox["5", 
    RowBox[{"log", "(", "125", ")"}]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009703505498857`*^9},
 CellTags->"PartialIntegrate",
 CellLabel->"Out[1273]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"Clear", "[", 
  RowBox[{"f", ",", "g"}], "]"}]], "Input",
 CellTags->"PartialIntegrate",
 CellLabel->"In[1274]:="]
}, Open  ]],

Cell["PartitHead", "ObjectName",
 CellTags->"PartitHead"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"PartitHead"],

Cell["\<\
PartitHead[expr, h] returns a list {ex1, h[ex2]} with ex1 free of expressions \
with head h, and h[ex2] having head h.\
\>", "Text",
 CellTags->"PartitHead"]
}, Open  ]],

Cell["PauliSigma", "ObjectName",
 CellTags->"PauliSigma"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"PauliSigma"],

Cell["\<\
PauliSigma denotes the vector of the 3 Pauli matrices. PauliSigma[1], \
PauliSigma[2], PauliSigma[3] give the explicit Pauli matrices. PauliSigma[] \
yields {PauliSigma[1], PauliSigma[2], PauliSigma[3]}.\
\>", "Text",
 CellTags->"PauliSigma"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"PauliSigma", "[", "]"}], "//", "TableForm"}]], "Input",
 CellTags->"PauliSigma",
 CellLabel->"In[1275]:="],

Cell[BoxData[
 FormBox[
  TagBox[GridBox[{
     {
      TagBox[GridBox[{
         {"0"},
         {"1"}
        },
        GridBoxAlignment->{
         "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
          "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
        GridBoxSpacings->{"Columns" -> {
            Offset[0.27999999999999997`], {
             Offset[0.5599999999999999]}, 
            Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {{
             Offset[0.2]}}, "RowsIndexed" -> {}}],
       Column], 
      TagBox[GridBox[{
         {"1"},
         {"0"}
        },
        GridBoxAlignment->{
         "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
          "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
        GridBoxSpacings->{"Columns" -> {
            Offset[0.27999999999999997`], {
             Offset[0.5599999999999999]}, 
            Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {{
             Offset[0.2]}}, "RowsIndexed" -> {}}],
       Column]},
     {
      TagBox[GridBox[{
         {"0"},
         {
          RowBox[{"-", "\[ImaginaryI]"}]}
        },
        GridBoxAlignment->{
         "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
          "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
        GridBoxSpacings->{"Columns" -> {
            Offset[0.27999999999999997`], {
             Offset[0.5599999999999999]}, 
            Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {{
             Offset[0.2]}}, "RowsIndexed" -> {}}],
       Column], 
      TagBox[GridBox[{
         {"\[ImaginaryI]"},
         {"0"}
        },
        GridBoxAlignment->{
         "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
          "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
        GridBoxSpacings->{"Columns" -> {
            Offset[0.27999999999999997`], {
             Offset[0.5599999999999999]}, 
            Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {{
             Offset[0.2]}}, "RowsIndexed" -> {}}],
       Column]},
     {
      TagBox[GridBox[{
         {"1"},
         {"0"}
        },
        GridBoxAlignment->{
         "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
          "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
        GridBoxSpacings->{"Columns" -> {
            Offset[0.27999999999999997`], {
             Offset[0.5599999999999999]}, 
            Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {{
             Offset[0.2]}}, "RowsIndexed" -> {}}],
       Column], 
      TagBox[GridBox[{
         {"0"},
         {
          RowBox[{"-", "1"}]}
        },
        GridBoxAlignment->{
         "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
          "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
        GridBoxSpacings->{"Columns" -> {
            Offset[0.27999999999999997`], {
             Offset[0.5599999999999999]}, 
            Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {{
             Offset[0.2]}}, "RowsIndexed" -> {}}],
       Column]}
    },
    GridBoxAlignment->{
     "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[2.0999999999999996`]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}],
   Function[BoxForm`e$, 
    TableForm[BoxForm`e$]]], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970350664432*^9},
 CellTags->"PauliSigma",
 CellLabel->"Out[1275]//TableForm="]
}, Open  ]]
}, Open  ]],

Cell["PaVe", "ObjectName",
 CellTags->"PaVe"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"PaVe"],

Cell["\<\
 PaVe[ i,j,... {p10,p12,...},{m1^2, mw^2, ...} ] denotes the invariant (and \
scalar) Passarino-Veltman integrals, i.e. the coefficient functions of the \
tensor integral decomposition. Joining plist and mlist gives the same \
conventions as for A0, B0, C0, D0. Automatic simlifications are performed for \
the coefficient functions of two-point integrals and for the scalar integrals.\
\>", "Text",
 CellTags->"PaVe"],

Cell[TextData[{
 "For a more detailed description see the manual (no changes from there \
basically): ",
 ButtonBox["http://www.mertig.com/oldfc",
  BaseStyle->"Hyperlink",
  ButtonData:>{
    URL["http://www.mertig.com/oldfc"], None}]
}], "Text",
 CellTags->"PaVe"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["PaVeReduce",
  BaseStyle->"AddOnsLink",
  ButtonData:>"PaVeReduce",
  ButtonNote->"PaVeReduce"],
 "."
}], "Text",
 CellTags->"PaVe"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"PaVe"],

Cell["Some of the PaVe's reduce to special cases.", "Text",
 CellTags->"PaVe"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PaVe", "[", 
  RowBox[{"0", ",", 
   RowBox[{"{", 
    RowBox[{"a", ",", "b", ",", "c", ",", "d", ",", "e", ",", "f"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"m1", ",", "m2", ",", "m3", ",", "m4"}], "}"}]}], "]"}]], "Input",\

 CellTags->"PaVe",
 CellLabel->"In[1276]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox[
    SubscriptBox["\<\"D\"\>", "\<\"0\"\>"],
    TraditionalForm], "(", "a", ",", "b", ",", "c", ",", "d", ",", "e", ",", 
   "f", ",", "m1", ",", "m2", ",", "m3", ",", "m4", ")"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.5009703507722607`*^9},
 CellTags->"PaVe",
 CellLabel->"Out[1276]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PaVe", "[", 
  RowBox[{"0", ",", "0", ",", 
   RowBox[{"{", "pp", "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"m", "^", "2"}], ",", 
     RowBox[{"M", "^", "2"}]}], "}"}]}], "]"}]], "Input",
 CellTags->"PaVe",
 CellLabel->"In[1277]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"PaVe", "(", 
   RowBox[{"0", ",", "0", ",", 
    RowBox[{"{", "pp", "}"}], ",", 
    RowBox[{"{", 
     RowBox[{
      SuperscriptBox["m", "2"], ",", 
      SuperscriptBox["M", "2"]}], "}"}]}], ")"}], TraditionalForm]], "Output",\

 CellChangeTimes->{3.500970350820159*^9},
 CellTags->"PaVe",
 CellLabel->"Out[1277]="]
}, Open  ]]
}, Open  ]],

Cell["PaVeOrderList", "ObjectName",
 CellTags->"PaVeOrderList"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"PaVeOrderList"],

Cell["\<\
PaVeOrderList is an option for PaVeOrder and PaVeReduce, specifying in which \
order the arguments of D0 are to be permuted.\
\>", "Text",
 CellTags->"PaVeOrderList"],

Cell[TextData[{
 "See also: ",
 ButtonBox["PaVeOrder",
  BaseStyle->"AddOnsLink",
  ButtonData:>"PaVeOrder",
  ButtonNote->"PaVeOrder"],
 ", ",
 ButtonBox["PaVeReduce",
  BaseStyle->"AddOnsLink",
  ButtonData:>"PaVeReduce",
  ButtonNote->"PaVeReduce"],
 ", ",
 ButtonBox["PaVe",
  BaseStyle->"AddOnsLink",
  ButtonData:>"PaVe",
  ButtonNote->"PaVe"],
 ", ",
 ButtonBox["D0",
  BaseStyle->"AddOnsLink",
  ButtonData:>"D0",
  ButtonNote->"D0"],
 "."
}], "Text",
 CellTags->"PaVeOrderList"]
}, Open  ]],

Cell["PaVeOrder", "ObjectName",
 CellTags->"PaVeOrder"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"PaVeOrder"],

Cell[TextData[{
 "PaVeOrder[expr] orders the arguments of all D0 in expr in a standard way. \
PaVeOrder[expr, PaVeOrderList \[Rule] { {..., s, u, ...}, {... ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    SubscriptBox["m", "1"], "2"], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    SubscriptBox["m", "2"], "2"], TraditionalForm]]],
 " ...}, ...}] orders the arguments of all D0 in expr according to the \
specified ordering lists. The lists may contain only a subsequence of the \
D0-variables."
}], "Text",
 CellTags->"PaVeOrder"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["PaVeReduce",
  BaseStyle->"AddOnsLink",
  ButtonData:>"PaVeReduce",
  ButtonNote->"PaVeReduce"],
 "."
}], "Text",
 CellTags->"PaVeOrder"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"PaVeOrder"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PaVeOrder", "[", 
  RowBox[{
   RowBox[{"D0", "[", 
    RowBox[{
    "me2", ",", "me2", ",", "mw2", ",", "mw2", ",", "t", ",", "s", ",", "me2",
      ",", "0", ",", "me2", ",", "0"}], "]"}], ",", 
   RowBox[{"PaVeOrderList", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"me2", ",", "me2", ",", "0", ",", "0"}], "}"}]}]}], 
  "]"}]], "Input",
 CellTags->"PaVeOrder",
 CellLabel->"In[1278]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox[
    SubscriptBox["\<\"D\"\>", "\<\"0\"\>"],
    TraditionalForm], "(", "me2", ",", "s", ",", "mw2", ",", "t", ",", "mw2", 
   ",", "me2", ",", "me2", ",", 
   FormBox["0",
    TraditionalForm], ",", 
   FormBox["0",
    TraditionalForm], ",", "me2", ")"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009703509016533`*^9},
 CellTags->"PaVeOrder",
 CellLabel->"Out[1278]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PaVeOrder", "[", 
  RowBox[{
   RowBox[{"D0", "[", 
    RowBox[{
    "me2", ",", "me2", ",", "mw2", ",", "mw2", ",", "t", ",", "s", ",", "me2",
      ",", "0", ",", "me2", ",", "0"}], "]"}], ",", 
   RowBox[{"PaVeOrderList", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"me2", ",", "me2", ",", "0", ",", "0"}], "}"}]}]}], 
  "]"}]], "Input",
 CellTags->"PaVeOrder",
 CellLabel->"In[1279]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox[
    SubscriptBox["\<\"D\"\>", "\<\"0\"\>"],
    TraditionalForm], "(", "me2", ",", "s", ",", "mw2", ",", "t", ",", "mw2", 
   ",", "me2", ",", "me2", ",", 
   FormBox["0",
    TraditionalForm], ",", 
   FormBox["0",
    TraditionalForm], ",", "me2", ")"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009703509817543`*^9},
 CellTags->"PaVeOrder",
 CellLabel->"Out[1279]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PaVeOrder", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"D0", "[", 
     RowBox[{
     "a", ",", "b", ",", "c", ",", "d", ",", "e", ",", "f", ",", "m12", ",", 
      "m22", ",", "m32", ",", "m42"}], "]"}], "+", 
    RowBox[{"D0", "[", 
     RowBox[{
     "me2", ",", "me2", ",", "mw2", ",", "mw2", ",", "t", ",", "s", ",", 
      "me2", ",", "0", ",", "me2", ",", "0"}], "]"}]}], ",", 
   RowBox[{"PaVeOrderList", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"me2", ",", "me2", ",", "0", ",", "0"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"f", ",", "e"}], "}"}]}], "}"}]}]}], "]"}]], "Input",
 CellTags->"PaVeOrder",
 CellLabel->"In[1280]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    FormBox[
     SubscriptBox["\<\"D\"\>", "\<\"0\"\>"],
     TraditionalForm], "(", "a", ",", "d", ",", "c", ",", "b", ",", "f", ",", 
    "e", ",", "m22", ",", "m12", ",", "m42", ",", "m32", ")"}], "+", 
   RowBox[{
    FormBox[
     SubscriptBox["\<\"D\"\>", "\<\"0\"\>"],
     TraditionalForm], "(", "me2", ",", "s", ",", "mw2", ",", "t", ",", "mw2",
     ",", "me2", ",", "me2", ",", 
    FormBox["0",
     TraditionalForm], ",", 
    FormBox["0",
     TraditionalForm], ",", "me2", ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970351326111*^9},
 CellTags->"PaVeOrder",
 CellLabel->"Out[1280]="]
}, Open  ]]
}, Open  ]],

Cell["PaVeReduce", "ObjectName",
 CellTags->"PaVeReduce"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"PaVeReduce"],

Cell["\<\
PaVeReduce[expr] reduces all Passarino-Veltman integrals (i.e. all PaVe's) in \
expr down to scalar A0, B0, C0 and D0.\
\>", "Text",
 CellTags->"PaVeReduce"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "PaVeReduce", "]"}]], "Input",
 CellTags->"PaVeReduce",
 CellLabel->"In[1281]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Dimension", "\[Rule]", "True"}], ",", 
    RowBox[{"IsolateNames", "\[Rule]", "False"}], ",", 
    RowBox[{"Mandelstam", "\[Rule]", 
     RowBox[{"{", "}"}]}], ",", 
    RowBox[{"PaVeOrderList", "\[Rule]", 
     RowBox[{"{", "}"}]}], ",", 
    RowBox[{"WriteOutPaVe", "\[Rule]", "False"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.50097035150079*^9},
 CellTags->"PaVeReduce",
 CellLabel->"Out[1281]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "WriteOutPaVe"}]], "Input",
 CellTags->"PaVeReduce",
 CellLabel->"In[1282]:="],

Cell[BoxData[
 StyleBox["\<\"WriteOutPaVe is an option for PaVeReduce and OneLoopSum. If \
set to a string, the results of all Passarino-Veltman PaVe's are stored in \
files with names generated from this string and the arguments of PaVe.\"\>", 
  "MSG"]], "Print", "PrintUsage",
 CellChangeTimes->{3.500970352069652*^9},
 CellTags->{"Info3500973951-4982097", "PaVeReduce"}]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FRH",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FRH",
  ButtonNote->"FRH"],
 ", ",
 ButtonBox["PaVeOrder",
  BaseStyle->"AddOnsLink",
  ButtonData:>"PaVeOrder",
  ButtonNote->"PaVeOrder"],
 "."
}], "Text",
 CellTags->"PaVeReduce"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"PaVeReduce"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PaVeReduce", "[", 
  RowBox[{
   RowBox[{"PaVe", "[", 
    RowBox[{"1", ",", "2", ",", 
     RowBox[{"{", 
      RowBox[{"s", ",", 
       RowBox[{"m", "^", "2"}], ",", 
       RowBox[{"m", "^", "2"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"m", "^", "2"}], ",", 
       RowBox[{"m", "^", "2"}], ",", 
       RowBox[{"M", "^", "2"}]}], "}"}]}], "]"}], ",", 
   RowBox[{"IsolateHead", "\[Rule]", "L"}]}], "]"}]], "Input",
 CellTags->"PaVeReduce",
 CellLabel->"In[1283]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{
     SuperscriptBox["M", "2"], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"8", " ", 
        SuperscriptBox["m", "2"]}], "-", 
       RowBox[{"3", " ", 
        SuperscriptBox["M", "2"]}], "-", 
       RowBox[{"2", " ", "s"}]}], ")"}], " ", 
     RowBox[{
      FormBox[
       SubscriptBox["\<\"C\"\>", "\<\"0\"\>"],
       TraditionalForm], "(", 
      FormBox[
       SuperscriptBox["m", "2"],
       TraditionalForm], ",", 
      FormBox[
       SuperscriptBox["m", "2"],
       TraditionalForm], ",", "s", ",", 
      FormBox[
       SuperscriptBox["m", "2"],
       TraditionalForm], ",", 
      FormBox[
       SuperscriptBox["M", "2"],
       TraditionalForm], ",", 
      FormBox[
       SuperscriptBox["m", "2"],
       TraditionalForm], ")"}]}], 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{
       RowBox[{"4", " ", 
        SuperscriptBox["m", "2"]}], "-", "s"}], ")"}], "2"]], "+", 
   FractionBox[
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox["m", "2"], "-", 
       SuperscriptBox["M", "2"]}], ")"}], " ", 
     RowBox[{
      SubscriptBox["B", "0"], "(", "0", ",", 
      SuperscriptBox["m", "2"], ",", 
      SuperscriptBox["M", "2"], ")"}]}], 
    RowBox[{"2", " ", 
     SuperscriptBox["m", "2"], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"4", " ", 
        SuperscriptBox["m", "2"]}], "-", "s"}], ")"}]}]], "+", 
   FractionBox[
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"4", " ", 
        SuperscriptBox["m", "2"]}], "-", 
       RowBox[{"6", " ", 
        SuperscriptBox["M", "2"]}], "-", "s"}], ")"}], " ", 
     RowBox[{
      SubscriptBox["B", "0"], "(", "s", ",", 
      SuperscriptBox["m", "2"], ",", 
      SuperscriptBox["m", "2"], ")"}]}], 
    RowBox[{"2", " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        RowBox[{"4", " ", 
         SuperscriptBox["m", "2"]}], "-", "s"}], ")"}], "2"]}]], "-", 
   FractionBox[
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"8", " ", 
        SuperscriptBox["m", "4"]}], "-", 
       RowBox[{"10", " ", 
        SuperscriptBox["m", "2"], " ", 
        SuperscriptBox["M", "2"]}], "-", 
       RowBox[{"2", " ", 
        SuperscriptBox["m", "2"], " ", "s"}], "+", 
       RowBox[{
        SuperscriptBox["M", "2"], " ", "s"}]}], ")"}], " ", 
     RowBox[{
      SubscriptBox["B", "0"], "(", 
      SuperscriptBox["m", "2"], ",", 
      SuperscriptBox["m", "2"], ",", 
      SuperscriptBox["M", "2"], ")"}]}], 
    RowBox[{"2", " ", 
     SuperscriptBox["m", "2"], " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        RowBox[{"4", " ", 
         SuperscriptBox["m", "2"]}], "-", "s"}], ")"}], "2"]}]], "+", 
   FractionBox["1", 
    RowBox[{"2", " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"4", " ", 
        SuperscriptBox["m", "2"]}], "-", "s"}], ")"}]}]]}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.5009703529114647`*^9},
 CellTags->"PaVeReduce",
 CellLabel->"Out[1283]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"L", "[", "10", "]"}]], "Input",
 CellTags->"PaVeReduce",
 CellLabel->"In[1284]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"L", "(", "10", ")"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970353077639*^9},
 CellTags->"PaVeReduce",
 CellLabel->"Out[1284]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FRH", "[", "%", "]"}]], "Input",
 CellTags->"PaVeReduce",
 CellLabel->"In[1285]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"L", "(", "10", ")"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970353153253*^9},
 CellTags->"PaVeReduce",
 CellLabel->"Out[1285]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PaVeReduce", "[", 
  RowBox[{
   RowBox[{"PaVe", "[", 
    RowBox[{"2", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"SmallVariable", "[", "me2", "]"}], ",", "mw2", ",", "t"}], 
      "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"SmallVariable", "[", "me2", "]"}], ",", "0", ",", "mw2"}], 
      "}"}]}], "]"}], ",", 
   RowBox[{"WriteOutPaVe", "\[Rule]", "\"\<p\>\""}]}], "]"}]], "Input",
 CellTags->"PaVeReduce",
 CellLabel->"In[1286]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    FractionBox[
     RowBox[{
      SubscriptBox["B", "0"], "(", "t", ",", "mw2", ",", "me2", ")"}], 
     RowBox[{"mw2", "-", "t"}]]}], "+", 
   FractionBox[
    RowBox[{
     SubscriptBox["B", "0"], "(", "0", ",", "mw2", ",", "mw2", ")"}], 
    RowBox[{"mw2", "-", "t"}]], "+", 
   FractionBox["2", 
    RowBox[{"mw2", "-", "t"}]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009703533174353`*^9},
 CellTags->"PaVeReduce",
 CellLabel->"Out[1286]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TableForm", "[", 
  RowBox[{"ReadList", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"$OperatingSystem", "===", "\"\<MacOS\>\""}], ",", "\"\<:\>\"",
        ",", "\"\<\>\""}], "]"}], "<>", 
     "\"\<pPaVe2Csmame2mw2tCsmame20mw2.s\>\""}], ",", "String"}], "]"}], 
  "]"}]], "Input",
 CellTags->"PaVeReduce",
 CellLabel->"In[1287]:="],

Cell[BoxData[
 FormBox[
  TagBox[
   TagBox[GridBox[{
      {"\<\"( 2/(mw2 - t) + B0[0, mw2, mw2]/(mw2 - t) - B0[t, mw2, \
SmallVariable[me2]]/\"\>"},
      {"\<\"  (mw2 - t)\"\>"},
      {"\<\"  ) \"\>"}
     },
     GridBoxAlignment->{
      "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
       "RowsIndexed" -> {}},
     GridBoxSpacings->{"Columns" -> {
         Offset[0.27999999999999997`], {
          Offset[0.5599999999999999]}, 
         Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
         Offset[0.2], {
          Offset[0.4]}, 
         Offset[0.2]}, "RowsIndexed" -> {}}],
    Column],
   Function[BoxForm`e$, 
    TableForm[BoxForm`e$]]], TraditionalForm]], "Output",
 CellChangeTimes->{3.50097035340245*^9},
 CellTags->"PaVeReduce",
 CellLabel->"Out[1287]//TableForm="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"DeleteFile", "/@", 
   RowBox[{
   "FileNames", "[", "\"\<pPaVe2Csmame2mw2tCsmame20mw2.s\>\"", "]"}]}], 
  ";"}]], "Input",
 CellTags->"PaVeReduce",
 CellLabel->"In[1288]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"se", "=", 
   RowBox[{"SmallVariable", "[", "ME2", "]"}]}], ";"}]], "Input",
 CellTags->"PaVeReduce",
 CellLabel->"In[1289]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"d122", "=", 
  RowBox[{"PaVeReduce", "[", 
   RowBox[{
    RowBox[{"PaVe", "[", 
     RowBox[{"1", ",", "2", ",", "2", ",", 
      RowBox[{"{", 
       RowBox[{"se", ",", "MW2", ",", "MW2", ",", "se", ",", "S", ",", "T"}], 
       "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"0", ",", "se", ",", "0", ",", "se"}], "}"}]}], "]"}], ",", 
    RowBox[{"Mandelstam", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{"S", ",", "T", ",", "U", ",", 
       RowBox[{"2", " ", "MW2"}]}], "}"}]}], ",", 
    RowBox[{"IsolateHead", "\[Rule]", "F"}]}], "]"}]}]], "Input",
 CellTags->"PaVeReduce",
 CellLabel->"In[1290]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    FractionBox[
     RowBox[{
      SuperscriptBox["S", "2"], " ", 
      SuperscriptBox["T", "2"], " ", 
      RowBox[{"(", 
       RowBox[{"MW2", "-", "S"}], ")"}], " ", 
      RowBox[{
       FormBox[
        SubscriptBox["\<\"C\"\>", "\<\"0\"\>"],
        TraditionalForm], "(", "T", ",", 
       FormBox["ME2",
        TraditionalForm], ",", 
       FormBox["ME2",
        TraditionalForm], ",", 
       FormBox["ME2",
        TraditionalForm], ",", 
       FormBox["ME2",
        TraditionalForm], ",", 
       FormBox["0",
        TraditionalForm], ")"}]}], 
     RowBox[{"2", " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox["MW2", "2"], "-", 
         RowBox[{"S", " ", "U"}]}], ")"}], "3"]}]]}], "+", 
   FractionBox[
    RowBox[{
     SuperscriptBox["S", "2"], " ", "T", " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"MW2", "-", "S"}], ")"}], "2"], " ", 
     RowBox[{
      FormBox[
       SubscriptBox["\<\"C\"\>", "\<\"0\"\>"],
       TraditionalForm], "(", "MW2", ",", "S", ",", 
      FormBox["ME2",
       TraditionalForm], ",", 
      FormBox["ME2",
       TraditionalForm], ",", 
      FormBox["0",
       TraditionalForm], ",", 
      FormBox["0",
       TraditionalForm], ")"}]}], 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox["MW2", "2"], "-", 
       RowBox[{"S", " ", "U"}]}], ")"}], "3"]], "-", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"MW2", "+", "S"}], ")"}], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"2", " ", 
         SuperscriptBox["MW2", "6"]}], "-", 
        RowBox[{"8", " ", 
         SuperscriptBox["MW2", "5"], " ", "S"}], "-", 
        RowBox[{"2", " ", 
         SuperscriptBox["MW2", "5"], " ", "T"}], "+", 
        RowBox[{"12", " ", 
         SuperscriptBox["MW2", "4"], " ", 
         SuperscriptBox["S", "2"]}], "+", 
        RowBox[{"20", " ", 
         SuperscriptBox["MW2", "4"], " ", "S", " ", "T"}], "-", 
        RowBox[{"8", " ", 
         SuperscriptBox["MW2", "3"], " ", 
         SuperscriptBox["S", "3"]}], "-", 
        RowBox[{"36", " ", 
         SuperscriptBox["MW2", "3"], " ", 
         SuperscriptBox["S", "2"], " ", "T"}], "-", 
        RowBox[{"6", " ", 
         SuperscriptBox["MW2", "3"], " ", "S", " ", 
         SuperscriptBox["T", "2"]}], "+", 
        RowBox[{"2", " ", 
         SuperscriptBox["MW2", "2"], " ", 
         SuperscriptBox["S", "4"]}], "+", 
        RowBox[{"20", " ", 
         SuperscriptBox["MW2", "2"], " ", 
         SuperscriptBox["S", "3"], " ", "T"}], "+", 
        RowBox[{"6", " ", 
         SuperscriptBox["MW2", "2"], " ", 
         SuperscriptBox["S", "2"], " ", 
         SuperscriptBox["T", "2"]}], "-", 
        RowBox[{"2", " ", "MW2", " ", 
         SuperscriptBox["S", "4"], " ", "T"}], "-", 
        RowBox[{"6", " ", "MW2", " ", 
         SuperscriptBox["S", "3"], " ", 
         SuperscriptBox["T", "2"]}], "+", 
        RowBox[{"4", " ", "MW2", " ", 
         SuperscriptBox["S", "2"], " ", 
         SuperscriptBox["T", "3"]}], "-", 
        RowBox[{
         SuperscriptBox["S", "2"], " ", 
         SuperscriptBox["T", "4"]}]}], ")"}], " ", 
      RowBox[{
       FormBox[
        SubscriptBox["\<\"C\"\>", "\<\"0\"\>"],
        TraditionalForm], "(", "MW2", ",", "MW2", ",", "T", ",", 
       FormBox["ME2",
        TraditionalForm], ",", 
       FormBox["0",
        TraditionalForm], ",", 
       FormBox["ME2",
        TraditionalForm], ")"}]}], ")"}], "/", 
    RowBox[{"(", 
     RowBox[{"2", " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{
         RowBox[{"4", " ", "MW2"}], "-", "T"}], ")"}], "2"], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox["MW2", "2"], "-", 
         RowBox[{"S", " ", "U"}]}], ")"}], "3"]}], ")"}]}], "+", 
   FractionBox[
    RowBox[{
     SuperscriptBox["S", "3"], " ", 
     SuperscriptBox["T", "2"], " ", 
     RowBox[{"(", 
      RowBox[{"MW2", "-", "S"}], ")"}], " ", 
     RowBox[{
      FormBox[
       SubscriptBox["\<\"D\"\>", "\<\"0\"\>"],
       TraditionalForm], "(", "MW2", ",", "MW2", ",", 
      FormBox["ME2",
       TraditionalForm], ",", 
      FormBox["ME2",
       TraditionalForm], ",", "T", ",", "S", ",", 
      FormBox["ME2",
       TraditionalForm], ",", 
      FormBox["0",
       TraditionalForm], ",", 
      FormBox["ME2",
       TraditionalForm], ",", 
      FormBox["0",
       TraditionalForm], ")"}]}], 
    RowBox[{"2", " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox["MW2", "2"], "-", 
        RowBox[{"S", " ", "U"}]}], ")"}], "3"]}]], "+", 
   FractionBox[
    RowBox[{
     RowBox[{"(", 
      RowBox[{"MW2", "+", "S"}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"4", " ", 
        SuperscriptBox["MW2", "3"]}], "-", 
       RowBox[{"9", " ", 
        SuperscriptBox["MW2", "2"], " ", "S"}], "-", 
       RowBox[{
        SuperscriptBox["MW2", "2"], " ", "U"}], "-", 
       RowBox[{"4", " ", "MW2", " ", "S", " ", "U"}], "+", 
       RowBox[{"2", " ", 
        SuperscriptBox["S", "3"]}], "+", 
       RowBox[{"5", " ", 
        SuperscriptBox["S", "2"], " ", "U"}], "+", 
       RowBox[{"3", " ", "S", " ", 
        SuperscriptBox["U", "2"]}]}], ")"}], " ", 
     RowBox[{
      SubscriptBox["B", "0"], "(", "T", ",", "ME2", ",", "ME2", ")"}]}], 
    RowBox[{"2", " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        RowBox[{"4", " ", "MW2"}], "-", "T"}], ")"}], "2"], " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox["MW2", "2"], "-", 
        RowBox[{"S", " ", "U"}]}], ")"}], "2"]}]], "-", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"4", " ", 
         SuperscriptBox["MW2", "5"]}], "-", 
        RowBox[{"5", " ", 
         SuperscriptBox["MW2", "4"], " ", "S"}], "-", 
        RowBox[{
         SuperscriptBox["MW2", "4"], " ", "U"}], "-", 
        RowBox[{"16", " ", 
         SuperscriptBox["MW2", "3"], " ", 
         SuperscriptBox["S", "2"]}], "+", 
        RowBox[{"4", " ", 
         SuperscriptBox["MW2", "2"], " ", 
         SuperscriptBox["S", "3"]}], "-", 
        RowBox[{"4", " ", 
         SuperscriptBox["MW2", "2"], " ", 
         SuperscriptBox["S", "2"], " ", "U"}], "+", 
        RowBox[{"4", " ", 
         SuperscriptBox["MW2", "2"], " ", "S", " ", 
         SuperscriptBox["U", "2"]}], "+", 
        RowBox[{"4", " ", "MW2", " ", 
         SuperscriptBox["S", "4"]}], "+", 
        RowBox[{"8", " ", "MW2", " ", 
         SuperscriptBox["S", "3"], " ", "U"}], "+", 
        RowBox[{
         SuperscriptBox["S", "3"], " ", 
         SuperscriptBox["U", "2"]}], "+", 
        RowBox[{
         SuperscriptBox["S", "2"], " ", 
         SuperscriptBox["U", "3"]}]}], ")"}], " ", 
      RowBox[{
       SubscriptBox["B", "0"], "(", "MW2", ",", "0", ",", "ME2", ")"}]}], 
     ")"}], "/", 
    RowBox[{"(", 
     RowBox[{"2", " ", 
      RowBox[{"(", 
       RowBox[{"MW2", "-", "S"}], ")"}], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{
         RowBox[{"4", " ", "MW2"}], "-", "T"}], ")"}], "2"], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox["MW2", "2"], "-", 
         RowBox[{"S", " ", "U"}]}], ")"}], "2"]}], ")"}]}], "+", 
   FractionBox[
    RowBox[{"S", " ", 
     RowBox[{
      SubscriptBox["B", "0"], "(", "S", ",", "0", ",", "0", ")"}], " ", 
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox["MW2", "2"], "-", 
       RowBox[{"4", " ", "MW2", " ", "S"}], "+", 
       RowBox[{"2", " ", 
        SuperscriptBox["S", "2"]}], "+", 
       RowBox[{"S", " ", "U"}]}], ")"}]}], 
    RowBox[{"2", " ", 
     RowBox[{"(", 
      RowBox[{"MW2", "-", "S"}], ")"}], " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox["MW2", "2"], "-", 
        RowBox[{"S", " ", "U"}]}], ")"}], "2"]}]], "-", 
   FractionBox[
    RowBox[{"MW2", "+", "S"}], 
    RowBox[{"2", " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"4", " ", "MW2"}], "-", "T"}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox["MW2", "2"], "-", 
       RowBox[{"S", " ", "U"}]}], ")"}]}]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970354112193*^9},
 CellTags->"PaVeReduce",
 CellLabel->"Out[1290]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"Write2", "[", 
   RowBox[{"\"\<fctd122.for\>\"", ",", 
    RowBox[{"d122res", "==", "d122"}], ",", 
    RowBox[{"FormatType", "\[Rule]", "FortranForm"}]}], "]"}], ";"}]], "Input",\

 CellTags->"PaVeReduce",
 CellLabel->"In[1291]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TableForm", "[", 
  RowBox[{"ReadList", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"$OperatingSystem", "===", "\"\<MacOS\>\""}], ",", "\"\<:\>\"",
        ",", "\"\<\>\""}], "]"}], "<>", "\"\<fctd122.for\>\""}], ",", 
    "String"}], "]"}], "]"}]], "Input",
 CellTags->"PaVeReduce",
 CellLabel->"In[1292]:="],

Cell[BoxData[
 FormBox[
  TagBox[
   TagBox[GridBox[{
      {"\<\"        d122res = (-5D0D-1*(MW2 + S))/\"\>"},
      {"\<\"     &   ((4D0*MW2 - 1D0*T)*(MW2**2 - 1D0*S*U)) - \"\>"},
      {"\<\"     &  (5.D-1*(4D0*MW2**5 - 5D0*MW2**4*S - \"\>"},
      {"\<\"     &       16D1*MW2**3*S**2 + 4D0*MW2**2*S**3 + \"\>"},
      {"\<\"     &       4D0*MW2*S**4 - 1D0*MW2**4*U - \"\>"},
      {"\<\"     &       4D0*MW2**2*S**2*U + 8D0*MW2*S**3*U + \"\>"},
      {"\<\"     &       4D0*MW2**2*S*U**2 + S**3*U**2 + S**2*U**3)*\"\>"},
      {"\<\"     &     B0(MW2,0D0,ME2))/\"\>"},
      {"\<\"     &   ((MW2 - 1D0*S)*(4D0*MW2 - 1D0*T)**2*\"\>"},
      {"\<\"     &     (MW2**2 - 1D0*S*U)**2) + \"\>"},
      {"\<\"     &  (5.D-1*S*(MW2**2 - 4D0*MW2*S + 2D0*S**2 + S*U)*\"\>"},
      {"\<\"     &     B0(S,0D0,0D0))/\"\>"},
      {"\<\"     &   ((MW2 - 1D0*S)*(MW2**2 - 1D0*S*U)**2) + \"\>"},
      {"\<\"     &  (5.D-1*(MW2 + S)*\"\>"},
      {"\<\"     &     (4D0*MW2**3 - 9D0*MW2**2*S + 2D0*S**3 - \"\>"},
      {"\<\"     &       1D0*MW2**2*U - 4D0*MW2*S*U + 5D0*S**2*U + \"\>"},
      {"\<\"     &       3D0*S*U**2)*B0(T,ME2,ME2))/\"\>"},
      {"\<\"     &   ((4D0*MW2 - 1D0*T)**2*(MW2**2 - 1D0*S*U)**2) - \"\>"},
      {"\<\"     &  (5.D-1*(MW2 + S)*\"\>"},
      {"\<\"     &     (2D0*MW2**6 - 8D0*MW2**5*S + 12D1*MW2**4*S**2 - \
\"\>"},
      {"\<\"     &       8D0*MW2**3*S**3 + 2D0*MW2**2*S**4 - \"\>"},
      {"\<\"     &       2D0*MW2**5*T + 20D1*MW2**4*S*T - \"\>"},
      {"\<\"     &       36D1*MW2**3*S**2*T + 20D1*MW2**2*S**3*T - \"\>"},
      {"\<\"     &       2D0*MW2*S**4*T - 6D0*MW2**3*S*T**2 + \"\>"},
      {"\<\"     &       6D0*MW2**2*S**2*T**2 - 6D0*MW2*S**3*T**2 + \"\>"},
      {"\<\"     &       4D0*MW2*S**2*T**3 - 1D0*S**2*T**4)*\"\>"},
      {"\<\"     &     C0(MW2,MW2,T,ME2,0D0,ME2))/\"\>"},
      {"\<\"     &   ((4D0*MW2 - 1D0*T)**2*(MW2**2 - 1D0*S*U)**3) + \"\>"},
      {"\<\"     &  ((MW2 - 1D0*S)**2*S**2*T*C0(MW2,S,ME2,ME2,0D0,0D0))/\"\>"},
      {"\<\"     &   (MW2**2 - 1D0*S*U)**3 - \"\>"},
      {"\<\"     &  (5.D-1*(MW2 - 1D0*S)*S**2*T**2*\"\>"},
      {"\<\"     &     C0(T,ME2,ME2,ME2,ME2,0D0))/(MW2**2 - 1D0*S*U)**3 + \
\"\>"},
      {"\<\"     &  (5.D-1*(MW2 - 1D0*S)*S**3*T**2*\"\>"},
      {"\<\"     &     D0(MW2,MW2,ME2,ME2,T,S,ME2,0D0,ME2,0D0))/\"\>"},
      {"\<\"     &   (MW2**2 - 1D0*S*U)**3\"\>"},
      {"\<\"                  \"\>"}
     },
     GridBoxAlignment->{
      "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
       "RowsIndexed" -> {}},
     GridBoxSpacings->{"Columns" -> {
         Offset[0.27999999999999997`], {
          Offset[0.5599999999999999]}, 
         Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
         Offset[0.2], {
          Offset[0.4]}, 
         Offset[0.2]}, "RowsIndexed" -> {}}],
    Column],
   Function[BoxForm`e$, 
    TableForm[BoxForm`e$]]], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009703542844343`*^9},
 CellTags->"PaVeReduce",
 CellLabel->"Out[1292]//TableForm="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"DeleteFile", "/@", 
   RowBox[{"FileNames", "[", "\"\<fctd122.for\>\"", "]"}]}], ";", 
  RowBox[{"Clear", "[", 
   RowBox[{"d122", ",", "se"}], "]"}], ";"}]], "Input",
 CellTags->"PaVeReduce",
 CellLabel->"In[1293]:="]
}, Open  ]],

Cell["PD", "ObjectName",
 CellTags->"PD"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"PD"],

Cell["PD is an abbreviation for PropagatorDenominator.", "Text",
 CellTags->"PD"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["PropagatorDenominator",
  BaseStyle->"AddOnsLink",
  ButtonData:>"PropagatorDenominator",
  ButtonNote->"PropagatorDenominator"],
 ", ",
 ButtonBox["IFPD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"IFPD",
  ButtonNote->"IFPD"],
 "."
}], "Text",
 CellTags->"PD"]
}, Open  ]],

Cell["PlusDistribution", "ObjectName",
 CellTags->"PlusDistribution"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"PlusDistribution"],

Cell["\<\
PlusDistribution[1/(1-x)] denotes a distribution (in the sense of the \"+\" \
prescription).\
\>", "Text",
 CellTags->"PlusDistribution"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Integrate2",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Integrate2",
  ButtonNote->"Integrate2"],
 "."
}], "Text",
 CellTags->"PlusDistribution"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"PlusDistribution"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PlusDistribution", "[", 
  RowBox[{"1", "/", 
   RowBox[{"(", 
    RowBox[{"1", "-", "x"}], ")"}]}], "]"}]], "Input",
 CellTags->"PlusDistribution",
 CellLabel->"In[1294]:="],

Cell[BoxData[
 FormBox[
  SubscriptBox[
   RowBox[{"(", 
    FractionBox["1", 
     RowBox[{"1", "-", "x"}]], ")"}], "+"], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970354404189*^9},
 CellTags->"PlusDistribution",
 CellLabel->"Out[1294]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PlusDistribution", "[", 
  RowBox[{
   RowBox[{"Log", "[", 
    RowBox[{"1", "-", "x"}], "]"}], "/", 
   RowBox[{"(", 
    RowBox[{"1", "-", "x"}], ")"}]}], "]"}]], "Input",
 CellTags->"PlusDistribution",
 CellLabel->"In[1295]:="],

Cell[BoxData[
 FormBox[
  SubscriptBox[
   RowBox[{"(", 
    FractionBox[
     RowBox[{"log", "(", 
      RowBox[{"1", "-", "x"}], ")"}], 
     RowBox[{"1", "-", "x"}]], ")"}], "+"], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970354504984*^9},
 CellTags->"PlusDistribution",
 CellLabel->"Out[1295]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Integrate2", "[", 
  RowBox[{
   RowBox[{"PlusDistribution", "[", 
    RowBox[{"1", "/", 
     RowBox[{"(", 
      RowBox[{"1", "-", "x"}], ")"}]}], "]"}], ",", " ", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellTags->"PlusDistribution",
 CellLabel->"In[1296]:="],

Cell[BoxData[
 FormBox["0", TraditionalForm]], "Output",
 CellChangeTimes->{3.5009703546879063`*^9},
 CellTags->"PlusDistribution",
 CellLabel->"Out[1296]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Integrate2", "[", 
  RowBox[{
   RowBox[{"PlusDistribution", "[", 
    RowBox[{
     RowBox[{"Log", "[", 
      RowBox[{"1", "-", "x"}], "]"}], "/", 
     RowBox[{"(", 
      RowBox[{"1", "-", "x"}], ")"}]}], "]"}], ",", " ", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellTags->"PlusDistribution",
 CellLabel->"In[1297]:="],

Cell[BoxData[
 FormBox["0", TraditionalForm]], "Output",
 CellChangeTimes->{3.500970354780402*^9},
 CellTags->"PlusDistribution",
 CellLabel->"Out[1297]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Integrate2", "[", 
  RowBox[{
   RowBox[{"PlusDistribution", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"Log", "[", 
       RowBox[{"1", "-", "x"}], "]"}], "^", "2"}], "/", 
     RowBox[{"(", 
      RowBox[{"1", "-", "x"}], ")"}]}], "]"}], ",", " ", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellTags->"PlusDistribution",
 CellLabel->"In[1298]:="],

Cell[BoxData[
 FormBox["0", TraditionalForm]], "Output",
 CellChangeTimes->{3.500970354891795*^9},
 CellTags->"PlusDistribution",
 CellLabel->"Out[1298]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PlusDistribution", "[", 
  RowBox[{
   RowBox[{"Log", "[", 
    RowBox[{"x", " ", 
     RowBox[{"(", 
      RowBox[{"1", "-", "x"}], ")"}]}], "]"}], "/", 
   RowBox[{"(", 
    RowBox[{"1", "-", "x"}], ")"}]}], "]"}]], "Input",
 CellTags->"PlusDistribution",
 CellLabel->"In[1299]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{"log", "(", "x", ")"}], 
    RowBox[{"1", "-", "x"}]], "+", 
   SubscriptBox[
    RowBox[{"(", 
     FractionBox[
      RowBox[{"log", "(", 
       RowBox[{"1", "-", "x"}], ")"}], 
      RowBox[{"1", "-", "x"}]], ")"}], "+"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.50097035508309*^9},
 CellTags->"PlusDistribution",
 CellLabel->"Out[1299]="]
}, Open  ]]
}, Open  ]],

Cell["Polarization", "ObjectName",
 CellTags->"Polarization"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Polarization"],

Cell["\<\
Polarization[k] = Polarization[k, I] is the head of a polarization momentum \
with (incoming) momentum k. A slashed polarization vector (e1(k) slash) has \
to be entered as DiracSlash[Polarization[k]]. The internal representation for \
a polarization vector e1 corresponding to a boson with four momentum k is: \
Momentum[ Polarization[ k, I ] ]. With this notation transversality of \
polarization vectors is provided, i.e. , Pair[ Momentum[k], Momentum[ \
Polarization[k, I] ] ] yields 0. Polarization[k,-I] denotes the complex \
conjugate polarization. Polarization is also an option of various functions \
related to the operator product expansion. The setting 0 denotes the \
unpolarized and 1 the polarized case.\
\>", "Text",
 CellTags->"Polarization"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["PolarizationVector",
  BaseStyle->"AddOnsLink",
  ButtonData:>"PolarizationVector",
  ButtonNote->"PolarizationVector"],
 ", ",
 ButtonBox["PolarizationSum",
  BaseStyle->"AddOnsLink",
  ButtonData:>"PolarizationSum",
  ButtonNote->"PolarizationSum"],
 ", ",
 ButtonBox["DoPolarizationSums",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DoPolarizationSums",
  ButtonNote->"DoPolarizationSums"],
 "."
}], "Text",
 CellTags->"Polarization"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Polarization"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Polarization", "[", "k", "]"}]], "Input",
 CellTags->"Polarization",
 CellLabel->"In[1300]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"\[CurlyEpsilon]", "(", "k", ")"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009703552416983`*^9},
 CellTags->"Polarization",
 CellLabel->"Out[1300]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Polarization", "[", "k", "]"}], "//", "StandardForm"}]], "Input",
 CellTags->"Polarization",
 CellLabel->"In[1301]:="],

Cell[BoxData[
 RowBox[{"Polarization", "[", 
  RowBox[{"k", ",", "\[ImaginaryI]"}], "]"}]], "Output",
 CellChangeTimes->{3.500970355401354*^9},
 CellTags->"Polarization",
 CellLabel->"Out[1301]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Polarization", "[", "k", "]"}], "//", 
  "ComplexConjugate"}]], "Input",
 CellTags->"Polarization",
 CellLabel->"In[1302]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox[
    TemplateBox[{"\"\[CurlyEpsilon]\"","\"*\""},
     "Superscript"],
    TraditionalForm], "(", "k", ")"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970355574913*^9},
 CellTags->"Polarization",
 CellLabel->"Out[1302]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Polarization", "[", "k", "]"}], "//", "ComplexConjugate"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"Polarization",
 CellLabel->"In[1303]:="],

Cell[BoxData[
 RowBox[{"Polarization", "[", 
  RowBox[{"k", ",", 
   RowBox[{"-", "\[ImaginaryI]"}]}], "]"}]], "Output",
 CellChangeTimes->{3.500970355797144*^9},
 CellTags->"Polarization",
 CellLabel->"Out[1303]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracSlash", "[", 
  RowBox[{"Polarization", "[", "k", "]"}], "]"}]], "Input",
 CellTags->"Polarization",
 CellLabel->"In[1304]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox["\<\"\[Gamma]\"\>",
    TraditionalForm], "\[CenterDot]", 
   FormBox[
    RowBox[{"\[CurlyEpsilon]", "(", "k", ")"}],
    TraditionalForm]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009703559541397`*^9},
 CellTags->"Polarization",
 CellLabel->"Out[1304]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"DiracSlash", "[", 
   RowBox[{"Polarization", "[", "k", "]"}], "]"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"Polarization",
 CellLabel->"In[1305]:="],

Cell[BoxData[
 RowBox[{"DiracSlash", "[", 
  RowBox[{"Polarization", "[", 
   RowBox[{"k", ",", "\[ImaginaryI]"}], "]"}], "]"}]], "Output",
 CellChangeTimes->{3.5009703560395117`*^9},
 CellTags->"Polarization",
 CellLabel->"Out[1305]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Pair", "[", " ", 
  RowBox[{
   RowBox[{"Momentum", "[", "k", "]"}], ",", " ", 
   RowBox[{"Momentum", "[", " ", 
    RowBox[{"Polarization", "[", 
     RowBox[{"k", ",", " ", "I"}], "]"}], " ", "]"}]}], " ", "]"}]], "Input",
 CellTags->"Polarization",
 CellLabel->"In[1306]:="],

Cell[BoxData[
 FormBox["0", TraditionalForm]], "Output",
 CellChangeTimes->{3.5009703562033987`*^9},
 CellTags->"Polarization",
 CellLabel->"Out[1306]="]
}, Open  ]]
}, Open  ]],

Cell["PolarizationSum", "ObjectName",
 CellTags->"PolarizationSum"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"PolarizationSum"],

Cell["\<\
PolarizationSum[ mu,nu, ... ] defines (as abbreviations) several polarization \
sums. The first two arguments are the interpreted as Lorentz indices, all \
further ones are momenta. PolarizationSum performs no calculations.\
\>", "Text",
 CellTags->"PolarizationSum"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "PolarizationSum", "]"}]], "Input",
 CellTags->"PolarizationSum",
 CellLabel->"In[1307]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"Dimension", "\[Rule]", "4"}], "}"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009703562866898`*^9},
 CellTags->"PolarizationSum",
 CellLabel->"Out[1307]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Polarization",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Polarization",
  ButtonNote->"Polarization"],
 "."
}], "Text",
 CellTags->"PolarizationSum"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"PolarizationSum"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PolarizationSum", "[", 
  RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}]], "Input",
 CellTags->"PolarizationSum",
 CellLabel->"In[1308]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   SuperscriptBox["g", 
    RowBox[{"\[Mu]", "\[Nu]"}]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009703564242077`*^9},
 CellTags->"PolarizationSum",
 CellLabel->"Out[1308]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PolarizationSum", "[", 
  RowBox[{"\[Mu]", ",", "\[Nu]", ",", "k"}], "]"}]], "Input",
 CellTags->"PolarizationSum",
 CellLabel->"In[1309]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{
     SuperscriptBox[
      FormBox["k",
       TraditionalForm], 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], " ", 
     SuperscriptBox[
      FormBox["k",
       TraditionalForm], 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm]]}], 
    SuperscriptBox[
     FormBox["k",
      TraditionalForm], "2"]], "-", 
   SuperscriptBox["g", 
    RowBox[{"\[Mu]", "\[Nu]"}]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009703565846167`*^9},
 CellTags->"PolarizationSum",
 CellLabel->"Out[1309]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PolarizationSum", "[", 
  RowBox[{"\[Mu]", ",", "\[Nu]", ",", "k", ",", "n"}], "]"}]], "Input",
 CellTags->"PolarizationSum",
 CellLabel->"In[1310]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Nu]"}]]}], "-", 
   FractionBox[
    RowBox[{
     SuperscriptBox[
      FormBox["n",
       TraditionalForm], "2"], " ", 
     SuperscriptBox[
      FormBox["k",
       TraditionalForm], 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], " ", 
     SuperscriptBox[
      FormBox["k",
       TraditionalForm], 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm]]}], 
    SuperscriptBox[
     FormBox[
      RowBox[{
       FormBox["k",
        TraditionalForm], "\[CenterDot]", 
       FormBox["n",
        TraditionalForm]}],
      TraditionalForm], "2"]], "+", 
   FractionBox[
    RowBox[{
     RowBox[{
      SuperscriptBox[
       FormBox["k",
        TraditionalForm], 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox[
       FormBox["n",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]]}], "+", 
     RowBox[{
      SuperscriptBox[
       FormBox["k",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox[
       FormBox["n",
        TraditionalForm], 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]]}]}], 
    RowBox[{
     FormBox["k",
      TraditionalForm], "\[CenterDot]", 
     FormBox["n",
      TraditionalForm]}]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970356727497*^9},
 CellTags->"PolarizationSum",
 CellLabel->"Out[1310]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PolarizationSum", "[", 
  RowBox[{"\[Mu]", ",", "\[Nu]", ",", "k", ",", 
   RowBox[{
    SubscriptBox["p", "1"], "-", 
    SubscriptBox["p", "2"]}]}], "]"}]], "Input",
 CellTags->"PolarizationSum",
 CellLabel->"In[1311]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Nu]"}]]}], "-", 
   FractionBox[
    RowBox[{
     SubsuperscriptBox["p", "1", "2"], " ", 
     SuperscriptBox[
      FormBox["k",
       TraditionalForm], 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], " ", 
     SuperscriptBox[
      FormBox["k",
       TraditionalForm], 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm]]}], 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        FormBox["k",
         TraditionalForm], "\[CenterDot]", 
        FormBox[
         SubscriptBox["p", "1"],
         TraditionalForm]}], "-", 
       RowBox[{
        FormBox["k",
         TraditionalForm], "\[CenterDot]", 
        FormBox[
         SubscriptBox["p", "2"],
         TraditionalForm]}]}], ")"}], "2"]], "+", 
   FractionBox[
    RowBox[{"2", " ", 
     SuperscriptBox[
      FormBox["k",
       TraditionalForm], 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], " ", 
     SuperscriptBox[
      FormBox["k",
       TraditionalForm], 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm]], " ", 
     RowBox[{
      FormBox[
       SubscriptBox["p", "1"],
       TraditionalForm], "\[CenterDot]", 
      FormBox[
       SubscriptBox["p", "2"],
       TraditionalForm]}]}], 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        FormBox["k",
         TraditionalForm], "\[CenterDot]", 
        FormBox[
         SubscriptBox["p", "1"],
         TraditionalForm]}], "-", 
       RowBox[{
        FormBox["k",
         TraditionalForm], "\[CenterDot]", 
        FormBox[
         SubscriptBox["p", "2"],
         TraditionalForm]}]}], ")"}], "2"]], "-", 
   FractionBox[
    RowBox[{
     SubsuperscriptBox["p", "2", "2"], " ", 
     SuperscriptBox[
      FormBox["k",
       TraditionalForm], 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], " ", 
     SuperscriptBox[
      FormBox["k",
       TraditionalForm], 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm]]}], 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        FormBox["k",
         TraditionalForm], "\[CenterDot]", 
        FormBox[
         SubscriptBox["p", "1"],
         TraditionalForm]}], "-", 
       RowBox[{
        FormBox["k",
         TraditionalForm], "\[CenterDot]", 
        FormBox[
         SubscriptBox["p", "2"],
         TraditionalForm]}]}], ")"}], "2"]], "+", 
   FractionBox[
    RowBox[{
     RowBox[{
      SuperscriptBox[
       FormBox["k",
        TraditionalForm], 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        FormBox[
         RowBox[{
          SubscriptBox["p", "1"], "-", 
          SubscriptBox["p", "2"]}],
         TraditionalForm], ")"}], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]]}], "+", 
     RowBox[{
      SuperscriptBox[
       FormBox["k",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        FormBox[
         RowBox[{
          SubscriptBox["p", "1"], "-", 
          SubscriptBox["p", "2"]}],
         TraditionalForm], ")"}], 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]]}]}], 
    RowBox[{
     RowBox[{
      FormBox["k",
       TraditionalForm], "\[CenterDot]", 
      FormBox[
       SubscriptBox["p", "1"],
       TraditionalForm]}], "-", 
     RowBox[{
      FormBox["k",
       TraditionalForm], "\[CenterDot]", 
      FormBox[
       SubscriptBox["p", "2"],
       TraditionalForm]}]}]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009703569844637`*^9},
 CellTags->"PolarizationSum",
 CellLabel->"Out[1311]="]
}, Open  ]]
}, Open  ]],

Cell["PolarizationUncontract", "ObjectName",
 CellTags->"PolarizationUncontract"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"PolarizationUncontract"],

Cell["\<\
PolarizationUncontract does Uncontract on scalar products involving \
polarization vectors.\
\>", "Text",
 CellTags->"PolarizationUncontract"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Polarization",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Polarization",
  ButtonNote->"Polarization"],
 ", ",
 ButtonBox["Uncontract",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Uncontract",
  ButtonNote->"Uncontract"],
 "."
}], "Text",
 CellTags->"PolarizationUncontract"]
}, Open  ]],

Cell["PolarizationVector", "ObjectName",
 CellTags->"PolarizationVector"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"PolarizationVector"],

Cell["PolarizationVector[p, mu] gives a polarization vector.", "Text",
 CellTags->"PolarizationVector"],

Cell[TextData[{
 "See also: ",
 ButtonBox["FourVector",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FourVector",
  ButtonNote->"FourVector"],
 ", ",
 ButtonBox["Pair",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Pair",
  ButtonNote->"Pair"],
 ", ",
 ButtonBox["Polarization",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Polarization",
  ButtonNote->"Polarization"],
 "."
}], "Text",
 CellTags->"PolarizationVector"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"PolarizationVector"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PolarizationVector", "[", 
  RowBox[{"k", ",", "\[Mu]"}], "]"}]], "Input",
 CellTags->"PolarizationVector",
 CellLabel->"In[1312]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["\[CurlyEpsilon]", 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], "(", "k", ")"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970357175892*^9},
 CellTags->"PolarizationVector",
 CellLabel->"Out[1312]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Conjugate", "[", 
  RowBox[{"PolarizationVector", "[", 
   RowBox[{"k", ",", "\[Mu]"}], "]"}], "]"}]], "Input",
 CellTags->"PolarizationVector",
 CellLabel->"In[1313]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubsuperscriptBox["\[CurlyEpsilon]", 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm], "*"], "(", "k", ")"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970357331181*^9},
 CellTags->"PolarizationVector",
 CellLabel->"Out[1313]="]
}, Open  ]],

Cell[TextData[{
 "A polarization vector ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["\[CurlyEpsilon]", "\[Mu]"], "(", "k", ")"}], 
   TraditionalForm]]],
 "is a special four-vector."
}], "Text",
 CellTags->"PolarizationVector"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"PolarizationVector", "[", 
   RowBox[{"k", ",", "mu"}], "]"}], "//", "StandardForm"}]], "Input",
 CellTags->"PolarizationVector",
 CellLabel->"In[1314]:="],

Cell[BoxData[
 RowBox[{"Pair", "[", 
  RowBox[{
   RowBox[{"LorentzIndex", "[", "mu", "]"}], ",", 
   RowBox[{"Momentum", "[", 
    RowBox[{"Polarization", "[", 
     RowBox[{"k", ",", "\[ImaginaryI]"}], "]"}], "]"}]}], "]"}]], "Output",
 CellChangeTimes->{3.50097035740195*^9},
 CellTags->"PolarizationVector",
 CellLabel->"Out[1314]//StandardForm="]
}, Open  ]],

Cell[TextData[{
 "The transverality property ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SuperscriptBox[
      FormBox["k",
       TraditionalForm], 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], " ", 
     RowBox[{
      SubscriptBox["\[CurlyEpsilon]", "\[Mu]"], "(", "k", ")"}]}], "=", "0"}],
    TraditionalForm]]],
 " is built in."
}], "Text",
 CellTags->"PolarizationVector"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"a1", " ", "=", " ", 
  RowBox[{
   RowBox[{"PolarizationVector", "[", 
    RowBox[{"k", ",", "\[Mu]"}], "]"}], " ", 
   RowBox[{"FourVector", "[", 
    RowBox[{"k", ",", "\[Mu]"}], "]"}]}]}]], "Input",
 CellTags->"PolarizationVector",
 CellLabel->"In[1315]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["k", "\[Mu]"], " ", 
   RowBox[{
    SubscriptBox["\[CurlyEpsilon]", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], "(", "k", ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970357551013*^9},
 CellTags->"PolarizationVector",
 CellLabel->"Out[1315]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Contract", "[", "a1", "]"}]], "Input",
 CellTags->"PolarizationVector",
 CellLabel->"In[1316]:="],

Cell[BoxData[
 FormBox["0", TraditionalForm]], "Output",
 CellChangeTimes->{3.500970357728305*^9},
 CellTags->"PolarizationVector",
 CellLabel->"Out[1316]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"a2", " ", "=", " ", 
  RowBox[{
   RowBox[{"PolarizationVector", "[", 
    RowBox[{
     RowBox[{"k", "-", "p"}], ",", "\[Mu]"}], "]"}], " ", 
   RowBox[{"FourVector", "[", 
    RowBox[{"k", ",", "\[Mu]"}], "]"}]}]}]], "Input",
 CellTags->"PolarizationVector",
 CellLabel->"In[1317]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["k", "\[Mu]"], " ", 
   RowBox[{
    SubscriptBox["\[CurlyEpsilon]", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], "(", 
    FormBox[
     RowBox[{"k", "-", "p"}],
     TraditionalForm], ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009703580793467`*^9},
 CellTags->"PolarizationVector",
 CellLabel->"Out[1317]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Contract", "[", "a2", "]"}]], "Input",
 CellTags->"PolarizationVector",
 CellLabel->"In[1318]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox["k",
    TraditionalForm], "\[CenterDot]", "(", 
   FormBox[
    RowBox[{"\[CurlyEpsilon]", "(", 
     FormBox[
      RowBox[{"k", "-", "p"}],
      TraditionalForm], ")"}],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970358261297*^9},
 CellTags->"PolarizationVector",
 CellLabel->"Out[1318]="]
}, Open  ]],

Cell[TextData[{
 "Depending on the alphabetical ordering of the momenta simplifcations are \
done, e.g., ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SuperscriptBox[
      FormBox["p",
       TraditionalForm], 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], " ", 
     RowBox[{
      SubscriptBox["\[CurlyEpsilon]", "\[Mu]"], "(", 
      FormBox[
       RowBox[{"k", "-", "p"}],
       TraditionalForm], ")"}]}], "=", 
    RowBox[{
     RowBox[{
      RowBox[{"-", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"k", "-", "p", "-", "k"}], ")"}], "\[Mu]"]}], 
      RowBox[{
       SubscriptBox["\[CurlyEpsilon]", "\[Mu]"], "(", 
       FormBox[
        RowBox[{"k", "-", "p"}],
        TraditionalForm], ")"}]}], "=", 
     RowBox[{
      RowBox[{
       SuperscriptBox["k", "\[Mu]"], 
       RowBox[{
        SubscriptBox["\[CurlyEpsilon]", "\[Mu]"], "(", 
        FormBox[
         RowBox[{"k", "-", "p"}],
         TraditionalForm], ")"}]}], "=", 
      RowBox[{
       FormBox["k",
        TraditionalForm], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
       RowBox[{
        RowBox[{"(", "\[NoBreak]", 
         FormBox[
          RowBox[{"\[CurlyEpsilon]", "\[NoBreak]", "(", "\[NoBreak]", 
           FormBox[
            RowBox[{"k", "-", "p"}],
            TraditionalForm], "\[NoBreak]", ")"}],
          TraditionalForm], "\[NoBreak]", ")"}], "."}]}]}]}]}], 
   TraditionalForm]]]
}], "Text",
 CellTags->"PolarizationVector"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"a3", " ", "=", " ", 
  RowBox[{
   RowBox[{"PolarizationVector", "[", 
    RowBox[{
     RowBox[{"k", "-", "p"}], ",", "\[Mu]"}], "]"}], " ", 
   RowBox[{"FourVector", "[", 
    RowBox[{"p", ",", "\[Mu]"}], "]"}]}]}]], "Input",
 CellTags->"PolarizationVector",
 CellLabel->"In[1319]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["p", "\[Mu]"], " ", 
   RowBox[{
    SubscriptBox["\[CurlyEpsilon]", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], "(", 
    FormBox[
     RowBox[{"k", "-", "p"}],
     TraditionalForm], ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970358357338*^9},
 CellTags->"PolarizationVector",
 CellLabel->"Out[1319]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Contract", "[", "a3", "]"}]], "Input",
 CellTags->"PolarizationVector",
 CellLabel->"In[1320]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox["k",
    TraditionalForm], "\[CenterDot]", "(", 
   FormBox[
    RowBox[{"\[CurlyEpsilon]", "(", 
     FormBox[
      RowBox[{"k", "-", "p"}],
      TraditionalForm], ")"}],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009703584496193`*^9},
 CellTags->"PolarizationVector",
 CellLabel->"Out[1320]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"Clear", "[", 
  RowBox[{"a1", ",", "a2", ",", "a3"}], "]"}]], "Input",
 CellTags->"PolarizationVector",
 CellLabel->"In[1321]:="]
}, Open  ]],

Cell["PositiveInteger", "ObjectName",
 CellTags->"PositiveInteger"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"PositiveInteger"],

Cell["\<\
PositiveInteger is a data type. E.g. DataType[OPEm, PositiveInteger] gives \
True.\
\>", "Text",
 CellTags->"PositiveInteger"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["DataType",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DataType",
  ButtonNote->"DataType"],
 "."
}], "Text",
 CellTags->"PositiveInteger"]
}, Open  ]],

Cell["PositiveNumber", "ObjectName",
 CellTags->"PositiveNumber"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"PositiveNumber"],

Cell["\<\
PositiveNumber is a data type. E.g. DataType[Epsilon, PositiveNumber] = True \
(by default). \
\>", "Text",
 CellTags->"PositiveNumber"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["DataType",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DataType",
  ButtonNote->"DataType"],
 "."
}], "Text",
 CellTags->"PositiveNumber"]
}, Open  ]],

Cell["PowerFactor", "ObjectName",
 CellTags->"PowerFactor"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"PowerFactor"],

Cell[TextData[{
 "PowerFactor[exp] replaces ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["x", "a"], TraditionalForm]]],
 Cell[BoxData[
  FormBox[
   SuperscriptBox["y", "a"], TraditionalForm]]],
 "with ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"x", " ", "y"}], ")"}], "a"], TraditionalForm]]],
 "."
}], "Text",
 CellTags->"PowerFactor"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["PowerSimplify",
  BaseStyle->"AddOnsLink",
  ButtonData:>"PowerSimplify",
  ButtonNote->"PowerSimplify"],
 "."
}], "Text",
 CellTags->"PowerFactor"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"PowerFactor"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  SuperscriptBox["x", "a"], 
  SuperscriptBox["y", "a"]}]], "Input",
 CellTags->"PowerFactor",
 CellLabel->"In[1322]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["x", "a"], " ", 
   SuperscriptBox["y", "a"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970358552066*^9},
 CellTags->"PowerFactor",
 CellLabel->"Out[1322]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PowerFactor", "[", "%", "]"}]], "Input",
 CellTags->"PowerFactor",
 CellLabel->"In[1323]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox[
   RowBox[{"(", 
    RowBox[{"x", " ", "y"}], ")"}], "a"], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009703587069387`*^9},
 CellTags->"PowerFactor",
 CellLabel->"Out[1323]="]
}, Open  ]]
}, Open  ]],

Cell["PowerSimplify", "ObjectName",
 CellTags->"PowerSimplify"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"PowerSimplify"],

Cell[TextData[{
 "PowerSimplify[exp] simplifies",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"-", "x"}], ")"}], "a"]}], TraditionalForm]]],
 " to ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"-", "1"}], ")"}], "a"], TraditionalForm]]],
 " ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["x", "a"], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"y", "-", "x"}], ")"}], "n"], TraditionalForm]]],
 " to ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"-", "1"}], ")"}], "n"], TraditionalForm]]],
 " ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"x", "-", "y"}], ")"}], "n"], TraditionalForm]]],
 "; thus assuming that the exponent is an integer (even if it is symbolic). \
Furthermore ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"-", "1"}], ")"}], 
    RowBox[{"a", "+", "n"}]], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["\[ImaginaryI]", 
    RowBox[{"a", "+", "n"}]], TraditionalForm]]],
 " are expanded and ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["\[ImaginaryI]", 
    RowBox[{"2", " ", "m"}]], TraditionalForm]]],
 " \[Rule] ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"-", "1"}], ")"}], "m"], TraditionalForm]]],
 " and (-1)^(",
 StyleBox["n",
  FontSlant->"Italic"],
 " ",
 StyleBox["m",
  FontSlant->"Italic"],
 ") \[Rule] 1 and (-1)^(",
 StyleBox["n",
  FontSlant->"Italic"],
 " ",
 StyleBox["m",
  FontSlant->"Italic"],
 ") \[Rule] ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    FormBox[
     RowBox[{"(", 
      RowBox[{"-", "1"}], ")"}],
     TraditionalForm], "m"], TraditionalForm]]],
 " for ",
 StyleBox["n",
  FontSlant->"Italic"],
 " even and odd respectively and ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"-", "1"}], ")"}], 
    RowBox[{"-", "n"}]], TraditionalForm]]],
 " \[Rule] ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"-", "1"}], ")"}], "n"], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["\[ExponentialE]", 
    StyleBox[
     RowBox[{"\[ImaginaryI]", 
      StyleBox["m",
       FontSlant->"Italic"], "\[Pi]"}]]], TraditionalForm]]],
 " \[Rule] ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"-", "1"}], ")"}], "m"], TraditionalForm]]],
 "."
}], "Text",
 CellTags->"PowerSimplify"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["DataType",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DataType",
  ButtonNote->"DataType"],
 ", ",
 ButtonBox["OPEm",
  BaseStyle->"AddOnsLink",
  ButtonData:>"OPEi",
  ButtonNote->"OPEm"],
 "."
}], "Text",
 CellTags->"PowerSimplify"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"PowerSimplify"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PowerSimplify", "[", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{"-", "1"}], ")"}], "^", 
   RowBox[{"(", 
    RowBox[{"2", "OPEm"}], ")"}]}], "]"}]], "Input",
 CellTags->"PowerSimplify",
 CellLabel->"In[1324]:="],

Cell[BoxData[
 FormBox["1", TraditionalForm]], "Output",
 CellChangeTimes->{3.5009703587804203`*^9},
 CellTags->"PowerSimplify",
 CellLabel->"Out[1324]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PowerSimplify", "[", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{"-", "1"}], ")"}], "^", 
   RowBox[{"(", 
    RowBox[{"OPEm", "+", "2"}], ")"}]}], "]"}]], "Input",
 CellTags->"PowerSimplify",
 CellLabel->"In[1325]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox[
   RowBox[{"(", 
    RowBox[{"-", "1"}], ")"}], "m"], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970359240827*^9},
 CellTags->"PowerSimplify",
 CellLabel->"Out[1325]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PowerSimplify", "[", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{"-", "1"}], ")"}], "^", 
   RowBox[{"(", 
    RowBox[{"OPEm", "-", "2"}], ")"}]}], "]"}]], "Input",
 CellTags->"PowerSimplify",
 CellLabel->"In[1326]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox[
   RowBox[{"(", 
    RowBox[{"-", "1"}], ")"}], "m"], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009703592909117`*^9},
 CellTags->"PowerSimplify",
 CellLabel->"Out[1326]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PowerSimplify", "[", 
  RowBox[{"I", "^", 
   RowBox[{"(", 
    RowBox[{"2", "OPEm"}], ")"}]}], "]"}]], "Input",
 CellTags->"PowerSimplify",
 CellLabel->"In[1327]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox[
   RowBox[{"(", 
    RowBox[{"-", "1"}], ")"}], "m"], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009703593415203`*^9},
 CellTags->"PowerSimplify",
 CellLabel->"Out[1327]="]
}, Open  ]]
}, Open  ]],

Cell["Power2", "ObjectName",
 CellTags->"Power2"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Power2"],

Cell["\<\
Power2[x, y] represents x^y.  Sometimes Power2 is more useful than the \
Mathematica Power. Power2[-a,b] simplifies to (-1)^b Power2[a,b] (if no \
Epsilon is in b ...).\
\>", "Text",
 CellTags->"Power2"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["PowerFactor",
  BaseStyle->"AddOnsLink",
  ButtonData:>"PowerFactor",
  ButtonNote->"PowerFactor"],
 "."
}], "Text",
 CellTags->"Power2"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Power2"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Power", "[", 
  RowBox[{
   RowBox[{"-", "a"}], ",", "b"}], "]"}]], "Input",
 CellTags->"Power2",
 CellLabel->"In[1328]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox[
   RowBox[{"(", 
    RowBox[{"-", "a"}], ")"}], "b"], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970359431675*^9},
 CellTags->"Power2",
 CellLabel->"Out[1328]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Power2", "[", 
  RowBox[{
   RowBox[{"-", "a"}], ",", "b"}], "]"}]], "Input",
 CellTags->"Power2",
 CellLabel->"In[1329]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"-", "1"}], ")"}], "b"], " ", 
   FormBox[
    SuperscriptBox["a", "b"],
    TraditionalForm]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.50097035947987*^9},
 CellTags->"Power2",
 CellLabel->"Out[1329]="]
}, Open  ]]
}, Open  ]],

Cell["PostFortranFile", "ObjectName",
 CellTags->"PostFortranFile"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"PostFortranFile"],

Cell["\<\
PostFortranFile is an option for Write2 which may be set to a file name (or a \
list of file names) or a string,  which will be put at the end of the \
generated Fortran file.\
\>", "Text",
 CellTags->"PostFortranFile"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Write2",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Write2",
  ButtonNote->"Write2"],
 ", ",
 ButtonBox["PreFortranFile",
  BaseStyle->"AddOnsLink",
  ButtonData:>"PreFortranFile",
  ButtonNote->"PreFortranFile"],
 "."
}], "Text",
 CellTags->"PostFortranFile"]
}, Open  ]],

Cell["Prefactor", "ObjectName",
 CellTags->"Prefactor"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Prefactor"],

Cell["\<\
Prefactor is an option for OneLoop and OneLoopSum. If set as option of \
OneLoop, the amplitude is multiplied by Prefactor before calculation; if \
specified as option of OneLoopSum, after calculation in the final result as a \
global factor.\
\>", "Text",
 CellTags->"Prefactor"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["OneLoop",
  BaseStyle->"AddOnsLink",
  ButtonData:>"OneLoop",
  ButtonNote->"OneLoop"],
 ", ",
 ButtonBox["OneLoopSum",
  BaseStyle->"AddOnsLink",
  ButtonData:>"OneLoopSum",
  ButtonNote->"OneLoopSum"],
 "."
}], "Text",
 CellTags->"Prefactor"]
}, Open  ]],

Cell["PreFortranFile", "ObjectName",
 CellTags->"PreFortranFile"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"PreFortranFile"],

Cell["\<\
PreFortranFile is an option for Write2 which may be set to a file name (or a \
list of file names) or a string, which will be put at the beginning of the \
generated Fortran file.\
\>", "Text",
 CellTags->"PreFortranFile"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Write2",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Write2",
  ButtonNote->"Write2"],
 ", ",
 ButtonBox["PostFortranFile",
  BaseStyle->"AddOnsLink",
  ButtonData:>"PostFortranFile",
  ButtonNote->"PostFortranFile"],
 "."
}], "Text",
 CellTags->"PreFortranFile"]
}, Open  ]],

Cell["PropagatorDenominator", "ObjectName",
 CellTags->"PropagatorDenominator"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"PropagatorDenominator"],

Cell[TextData[{
 "PropagatorDenominator[q, m] is a factor of the denominator of a propagator. \
If p is supposed to be D-dimensional enter: PropagatorDenominator[Momentum[q, \
D], m]. What is meant is ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{"1", "/", 
     RowBox[{"(", 
      RowBox[{"q", "^", "2"}]}]}]}], TraditionalForm]]],
 " - ",
 Cell[BoxData[
  FormBox[
   RowBox[{"m", "^", "2"}], TraditionalForm]]],
 "). PropagatorDenominator[p] evaluates to PropagatorDenominator[p, 0]. "
}], "Text",
 CellTags->"PropagatorDenominator"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FeynAmpDenominator",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FeynAmpDenominator",
  ButtonNote->"FeynAmpDenominator"],
 ", ",
 ButtonBox["PropagatorDenominatorExplicit",
  BaseStyle->"AddOnsLink",
  ButtonData:>"PropagatorDenominatorExplicit",
  ButtonNote->"PropagatorDenominatorExplicit"],
 ", ",
 ButtonBox["IFPD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"IFPD",
  ButtonNote->"IFPD"],
 "."
}], "Text",
 CellTags->"PropagatorDenominator"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"PropagatorDenominator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{" ", 
  RowBox[{"PropagatorDenominator", "[", 
   RowBox[{"p", ",", "m"}], "]"}]}]], "Input",
 CellTags->"PropagatorDenominator",
 CellLabel->"In[1330]:="],

Cell[BoxData[
 FormBox[
  FractionBox["1", 
   RowBox[{
    SuperscriptBox["p", "2"], "-", 
    SuperscriptBox["m", "2"]}]], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970359567528*^9},
 CellTags->"PropagatorDenominator",
 CellLabel->"Out[1330]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PropagatorDenominator", "[", "p", "]"}]], "Input",
 CellTags->"PropagatorDenominator",
 CellLabel->"In[1331]:="],

Cell[BoxData[
 FormBox[
  FractionBox["1", 
   SuperscriptBox["p", "2"]], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970359645504*^9},
 CellTags->"PropagatorDenominator",
 CellLabel->"Out[1331]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t1", "=", 
  RowBox[{"PropagatorDenominator", "[", 
   RowBox[{"q", ",", "m"}], "]"}], " "}]], "Input",
 CellTags->"PropagatorDenominator",
 CellLabel->"In[1332]:="],

Cell[BoxData[
 FormBox[
  FractionBox["1", 
   RowBox[{
    SuperscriptBox["q", "2"], "-", 
    SuperscriptBox["m", "2"]}]], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970359724105*^9},
 CellTags->"PropagatorDenominator",
 CellLabel->"Out[1332]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StandardForm", "[", 
  RowBox[{"FCI", "[", "t1", "]"}], "]"}]], "Input",
 CellTags->"PropagatorDenominator",
 CellLabel->"In[1333]:="],

Cell[BoxData[
 RowBox[{"PropagatorDenominator", "[", 
  RowBox[{
   RowBox[{"Momentum", "[", 
    RowBox[{"q", ",", "D"}], "]"}], ",", "m"}], "]"}]], "Output",
 CellChangeTimes->{3.500970359802268*^9},
 CellTags->"PropagatorDenominator",
 CellLabel->"Out[1333]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StandardForm", "[", 
  RowBox[{"ChangeDimension", "[", 
   RowBox[{"t1", ",", "D"}], "]"}], "]"}]], "Input",
 CellTags->"PropagatorDenominator",
 CellLabel->"In[1334]:="],

Cell[BoxData[
 RowBox[{"PropagatorDenominator", "[", 
  RowBox[{
   RowBox[{"Momentum", "[", 
    RowBox[{"q", ",", "D"}], "]"}], ",", "m"}], "]"}]], "Output",
 CellChangeTimes->{3.500970360052875*^9},
 CellTags->"PropagatorDenominator",
 CellLabel->"Out[1334]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PropagatorDenominatorExplicit", "[", "t1", "]"}]], "Input",
 CellTags->"PropagatorDenominator",
 CellLabel->"In[1335]:="],

Cell[BoxData[
 FormBox[
  FractionBox["1", 
   RowBox[{
    SuperscriptBox[
     FormBox["q",
      TraditionalForm], "2"], "-", 
    SuperscriptBox["m", "2"]}]], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009703602310133`*^9},
 CellTags->"PropagatorDenominator",
 CellLabel->"Out[1335]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StandardForm", "[", "%", "]"}]], "Input",
 CellTags->"PropagatorDenominator",
 CellLabel->"In[1336]:="],

Cell[BoxData[
 FractionBox["1", 
  RowBox[{
   RowBox[{"-", 
    SuperscriptBox["m", "2"]}], "+", 
   RowBox[{"Pair", "[", 
    RowBox[{
     RowBox[{"Momentum", "[", "q", "]"}], ",", 
     RowBox[{"Momentum", "[", "q", "]"}]}], "]"}]}]]], "Output",
 CellChangeTimes->{3.500970360318301*^9},
 CellTags->"PropagatorDenominator",
 CellLabel->"Out[1336]//StandardForm="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"Clear", "[", "t1", "]"}]], "Input",
 CellTags->"PropagatorDenominator",
 CellLabel->"In[1337]:="]
}, Open  ]],

Cell["PropagatorDenominatorExplicit", "ObjectName",
 CellTags->"PropagatorDenominatorExplicit"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"PropagatorDenominatorExplicit"],

Cell["\<\
PropagatorDenominatorExplicit[exp] changes each occurence of \
PropagatorDenominator[a,b] in exp into 1/(ScalarProduct[a,a]-b^2) and \
replaces FeynAmpDenominator by Identity.\
\>", "Text",
 CellTags->"PropagatorDenominatorExplicit"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FeynAmpDenominator",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FeynAmpDenominator",
  ButtonNote->"FeynAmpDenominator"],
 ", ",
 ButtonBox["PropagatorDenominator",
  BaseStyle->"AddOnsLink",
  ButtonData:>"PropagatorDenominator",
  ButtonNote->"PropagatorDenominator"],
 "."
}], "Text",
 CellTags->"PropagatorDenominatorExplicit"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"PropagatorDenominatorExplicit"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"FAD", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"q", ",", "m"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"q", "-", "p"}], ",", "0"}], "}"}]}], "]"}], "//", 
  "FCI"}]], "Input",
 CellTags->"PropagatorDenominatorExplicit",
 CellLabel->"In[1338]:="],

Cell[BoxData[
 FormBox[
  FractionBox["1", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox["q", "2"], "-", 
      SuperscriptBox["m", "2"]}], ")"}], ".", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"q", "-", "p"}], ")"}], "2"]}]], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970360492682*^9},
 CellTags->"PropagatorDenominatorExplicit",
 CellLabel->"Out[1338]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"PropagatorDenominatorExplicit", "[", "%", "]"}], "//", 
  "FCE"}]], "Input",
 CellTags->"PropagatorDenominatorExplicit",
 CellLabel->"In[1339]:="],

Cell[BoxData[
 FormBox[
  FractionBox["1", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      FormBox[
       SuperscriptBox[
        FormBox["q",
         TraditionalForm], "2"],
       TraditionalForm], "-", 
      SuperscriptBox["m", "2"]}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox["m", "2"], "-", 
      RowBox[{"2", " ", 
       FormBox[
        RowBox[{
         FormBox["p",
          TraditionalForm], "\[CenterDot]", 
         FormBox["q",
          TraditionalForm]}],
        TraditionalForm]}], "+", 
      FormBox[
       SuperscriptBox[
        FormBox["q",
         TraditionalForm], "2"],
       TraditionalForm]}], ")"}]}]], TraditionalForm]], "Output",
 CellChangeTimes->{3.50097036065175*^9},
 CellTags->"PropagatorDenominatorExplicit",
 CellLabel->"Out[1339]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "StandardForm"}]], "Input",
 CellTags->"PropagatorDenominatorExplicit",
 CellLabel->"In[1340]:="],

Cell[BoxData[
 FractionBox["1", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      SuperscriptBox["m", "2"]}], "+", 
     RowBox[{"SPD", "[", 
      RowBox[{"q", ",", "q"}], "]"}]}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{
     SuperscriptBox["m", "2"], "-", 
     RowBox[{"2", " ", 
      RowBox[{"SPD", "[", 
       RowBox[{"p", ",", "q"}], "]"}]}], "+", 
     RowBox[{"SPD", "[", 
      RowBox[{"q", ",", "q"}], "]"}]}], ")"}]}]]], "Output",
 CellChangeTimes->{3.500970360730987*^9},
 CellTags->"PropagatorDenominatorExplicit",
 CellLabel->"Out[1340]//StandardForm="]
}, Open  ]]
}, Open  ]],

Cell["QGV", "ObjectName",
 CellTags->"QGV"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"QGV"],

Cell["QGV is equivalent to QuarkGluonVertex.", "Text",
 CellTags->"QGV"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["QuarkGluonVertex",
  BaseStyle->"AddOnsLink",
  ButtonData:>"QuarkGluonVertex",
  ButtonNote->"QuarkGluonVertex"],
 "."
}], "Text",
 CellTags->"QGV"]
}, Open  ]],

Cell["QO", "ObjectName",
 CellTags->"QO"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"QO"],

Cell["QO is equivalent to Twist2QuarkOperator.", "Text",
 CellTags->"QO"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Twist2QuarkOperator",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Twist2QuarkOperator",
  ButtonNote->"Twist2QuarkOperator"],
 "."
}], "Text",
 CellTags->"QO"]
}, Open  ]],

Cell["QP", "ObjectName",
 CellTags->"QP"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"QP"],

Cell["\<\
QP is an alias for QuarkPropagator. QP[p] is the massless quark propagator. \
QP[{p,m}] gives the  quark propagator with mass m.\
\>", "Text",
 CellTags->"QP"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["QuarkPropagator",
  BaseStyle->"AddOnsLink",
  ButtonData:>"QuarkPropagator",
  ButtonNote->"QuarkPropagator"],
 "."
}], "Text",
 CellTags->"QP"]
}, Open  ]],

Cell["QuantumField", "ObjectName",
 CellTags->"QuantumField"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"QuantumField"],

Cell[TextData[{
 "QuantumField is the head of quantized fields and their derivatives. \
QuantumField[par, ftype, {lorind}, {sunind}] denotes a quantum field of type \
ftype with (possible) Lorentz-indices lorind and SU(",
 StyleBox["N",
  FontSlant->"Italic"],
 ") indices sunind. The optional first argument par denotes a partial \
derivative acting on the field."
}], "Text",
 CellTags->"QuantumField"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FeynRule",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FeynRule",
  ButtonNote->"FeynRule"],
 ", ",
 ButtonBox["PartialD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"PartialD",
  ButtonNote->"PartialD"],
 ", ",
 ButtonBox["ExpandPartialD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"ExpandPartialD",
  ButtonNote->"ExpandPartialD"],
 "."
}], "Text",
 CellTags->"QuantumField"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"QuantumField"],

Cell["This denotes a scalar field.", "Text",
 CellTags->"QuantumField"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"QuantumField", "[", "S", "]"}]], "Input",
 CellTags->"QuantumField",
 CellLabel->"In[1341]:="],

Cell[BoxData[
 FormBox["S", TraditionalForm]], "Output",
 CellChangeTimes->{3.500970360808578*^9},
 CellTags->"QuantumField",
 CellLabel->"Out[1341]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"QuantumField", "[", "AntiQuarkField", "]"}]], "Input",
 CellTags->"QuantumField",
 CellLabel->"In[1342]:="],

Cell[BoxData[
 FormBox[
  OverscriptBox["\[Psi]", "_"], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970361042523*^9},
 CellTags->"QuantumField",
 CellLabel->"Out[1342]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"QuantumField", "[", "QuarkField", "]"}]], "Input",
 CellTags->"QuantumField",
 CellLabel->"In[1343]:="],

Cell[BoxData[
 FormBox["\[Psi]", TraditionalForm]], "Output",
 CellChangeTimes->{3.500970361114728*^9},
 CellTags->"QuantumField",
 CellLabel->"Out[1343]="]
}, Open  ]],

Cell["This is a field with a Lorentz index.", "Text",
 CellTags->"QuantumField"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"QuantumField", "[", 
  RowBox[{"B", ",", 
   RowBox[{"{", "\[Mu]", "}"}]}], "]"}]], "Input",
 CellTags->"QuantumField",
 CellLabel->"In[1344]:="],

Cell[BoxData[
 FormBox[
  SubscriptBox["B", "\[Mu]"], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009703612809057`*^9},
 CellTags->"QuantumField",
 CellLabel->"Out[1344]="]
}, Open  ]],

Cell["Color indices should be put after the Lorentz ones.", "Text",
 CellTags->"QuantumField"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"QuantumField", "[", 
  RowBox[{"GaugeField", ",", 
   RowBox[{"{", "\[Mu]", "}"}], ",", 
   RowBox[{"{", "a", "}"}]}], "]"}]], "Input",
 CellTags->"QuantumField",
 CellLabel->"In[1345]:="],

Cell[BoxData[
 FormBox[
  SubsuperscriptBox[
   FormBox["A",
    TraditionalForm], 
   FormBox[
    FormBox["\[Mu]",
     TraditionalForm],
    TraditionalForm], 
   FormBox[
    FormBox["a",
     TraditionalForm],
    TraditionalForm]], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970361369835*^9},
 CellTags->"QuantumField",
 CellLabel->"Out[1345]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "StandardForm"}]], "Input",
 CellTags->"QuantumField",
 CellLabel->"In[1346]:="],

Cell[BoxData[
 RowBox[{"QuantumField", "[", 
  RowBox[{"GaugeField", ",", 
   RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
   RowBox[{"SUNIndex", "[", "a", "]"}]}], "]"}]], "Output",
 CellChangeTimes->{3.500970361531355*^9},
 CellTags->"QuantumField",
 CellLabel->"Out[1346]//StandardForm="]
}, Open  ]],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["A", "\[CapitalDelta]", "a"], TraditionalForm]]],
 " is a short form for ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"\[CapitalDelta]", "^", "\[Mu]"}], " ", 
    RowBox[{
     SubsuperscriptBox["A", "\[Mu]", "a"], "."}]}], TraditionalForm]]],
 " "
}], "Text",
 CellTags->"QuantumField"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"QuantumField", "[", 
  RowBox[{"A", ",", 
   RowBox[{"{", "OPEDelta", "}"}], ",", 
   RowBox[{"{", "a", "}"}]}], "]"}]], "Input",
 CellTags->"QuantumField",
 CellLabel->"In[1347]:="],

Cell[BoxData[
 FormBox[
  SubsuperscriptBox["A", 
   FormBox["\[CapitalDelta]",
    TraditionalForm], 
   FormBox[
    FormBox["a",
     TraditionalForm],
    TraditionalForm]], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970361593124*^9},
 CellTags->"QuantumField",
 CellLabel->"Out[1347]="]
}, Open  ]],

Cell["\<\
 The first list of indices is usually interpreted as type LorentzIndex, \
except for OPEDelta, which gets converted to type Momentum. \
\>", "Text",
 CellTags->"QuantumField"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"QuantumField", "[", 
   RowBox[{"A", ",", 
    RowBox[{"{", "OPEDelta", "}"}], ",", 
    RowBox[{"{", "a", "}"}]}], "]"}], "//", "StandardForm"}]], "Input",
 CellTags->"QuantumField",
 CellLabel->"In[1348]:="],

Cell[BoxData[
 RowBox[{"QuantumField", "[", 
  RowBox[{"A", ",", 
   RowBox[{"Momentum", "[", "OPEDelta", "]"}], ",", 
   RowBox[{"SUNIndex", "[", "a", "]"}]}], "]"}]], "Output",
 CellChangeTimes->{3.500970361639421*^9},
 CellTags->"QuantumField",
 CellLabel->"Out[1348]//StandardForm="]
}, Open  ]],

Cell["Derivatives of fields are denoted as follows.", "Text",
 CellTags->"QuantumField"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"QuantumField", "[", 
  RowBox[{
   RowBox[{"PartialD", "[", "\[Mu]", "]"}], ",", "A", ",", 
   RowBox[{"{", "\[Mu]", "}"}]}], "]"}]], "Input",
 CellTags->"QuantumField",
 CellLabel->"In[1349]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"QuantumField", "(", 
   RowBox[{
    RowBox[{
    "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(",
      "\[Mu]", ")"}], ",", "A", ",", 
    FormBox["\[Mu]",
     TraditionalForm]}], ")"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970361731248*^9},
 CellTags->"QuantumField",
 CellLabel->"Out[1349]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"QuantumField", "[", 
  RowBox[{
   RowBox[{"PartialD", "[", "OPEDelta", "]"}], ",", "S"}], "]"}]], "Input",
 CellTags->"QuantumField",
 CellLabel->"In[1350]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"QuantumField", "(", 
   RowBox[{
    RowBox[{
    "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(",
      "\[CapitalDelta]", ")"}], ",", "S"}], ")"}], TraditionalForm]], "Output",\

 CellChangeTimes->{3.500970361795765*^9},
 CellTags->"QuantumField",
 CellLabel->"Out[1350]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"QuantumField", "[", 
  RowBox[{
   RowBox[{"PartialD", "[", "OPEDelta", "]"}], ",", "A", ",", 
   RowBox[{"{", "OPEDelta", "}"}], ",", 
   RowBox[{"{", "a", "}"}]}], "]"}]], "Input",
 CellTags->"QuantumField",
 CellLabel->"In[1351]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"QuantumField", "(", 
   RowBox[{
    RowBox[{
    "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(",
      "\[CapitalDelta]", ")"}], ",", "A", ",", "\[CapitalDelta]", ",", 
    FormBox["a",
     TraditionalForm]}], ")"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009703618769503`*^9},
 CellTags->"QuantumField",
 CellLabel->"Out[1351]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"QuantumField", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"PartialD", "[", "OPEDelta", "]"}], "^", "OPEm"}], ",", "A", ",", 
   
   RowBox[{"{", "OPEDelta", "}"}], ",", 
   RowBox[{"{", "a", "}"}]}], "]"}]], "Input",
 CellTags->"QuantumField",
 CellLabel->"In[1352]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"QuantumField", "(", 
   RowBox[{
    SuperscriptBox[
     RowBox[{
     "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(",
       "\[CapitalDelta]", ")"}], "m"], ",", "A", ",", "\[CapitalDelta]", ",", 
    
    FormBox["a",
     TraditionalForm]}], ")"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970361987482*^9},
 CellTags->"QuantumField",
 CellLabel->"Out[1352]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"QuantumField", "[", 
   RowBox[{"QuantumField", "[", "A", "]"}], "]"}], " ", "===", " ", 
  RowBox[{"QuantumField", "[", "A", "]"}]}]], "Input",
 CellTags->"QuantumField",
 CellLabel->"In[1353]:="],

Cell[BoxData[
 FormBox["True", TraditionalForm]], "Output",
 CellChangeTimes->{3.5009703620615807`*^9},
 CellTags->"QuantumField",
 CellLabel->"Out[1353]="]
}, Open  ]]
}, Open  ]],

Cell["QuarkField", "ObjectName",
 CellTags->"QuarkField"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"QuarkField"],

Cell["\<\
QuarkField is the name of a fermionic field. QuarkField is just a name with \
no functional properties. Only typesetting rules are attached.\
\>", "Text",
 CellTags->"QuarkField"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["AntiQuarkField",
  BaseStyle->"AddOnsLink",
  ButtonData:>"AntiQuarkField",
  ButtonNote->"AntiQuarkField"],
 ", ",
 ButtonBox["QuantumField",
  BaseStyle->"AddOnsLink",
  ButtonData:>"QuantumField",
  ButtonNote->"QuantumField"],
 "."
}], "Text",
 CellTags->"QuarkField"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"QuarkField"],

Cell[CellGroupData[{

Cell[BoxData["QuarkField"], "Input",
 CellTags->"QuarkField",
 CellLabel->"In[1354]:="],

Cell[BoxData[
 FormBox["\[Psi]", TraditionalForm]], "Output",
 CellChangeTimes->{3.500970362126831*^9},
 CellTags->"QuarkField",
 CellLabel->"Out[1354]="]
}, Open  ]]
}, Open  ]],

Cell["QuarkGluonVertex", "ObjectName",
 CellTags->"QuarkGluonVertex"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"QuarkGluonVertex"],

Cell[TextData[{
 "QuarkGluonVertex[",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Mu]", ",", " ", 
    StyleBox["a",
     FontSlant->"Plain"]}], TraditionalForm]]],
 "] gives the Feynman rule for the quark-gluon vertex. "
}], "Text",
 CellTags->"QuarkGluonVertex"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "QuarkGluonVertex", "]"}]], "Input",
 CellTags->"QuarkGluonVertex",
 CellLabel->"In[1355]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"CounterTerm", "\[Rule]", "False"}], ",", 
    RowBox[{"CouplingConstant", "\[Rule]", 
     SubscriptBox["g", "s"]}], ",", 
    RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
    RowBox[{"Explicit", "\[Rule]", "False"}], ",", 
    RowBox[{"\[CapitalOmega]", "\[Rule]", "False"}], ",", 
    RowBox[{"Polarization", "\[Rule]", "0"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970362207735*^9},
 CellTags->"QuarkGluonVertex",
 CellLabel->"Out[1355]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["GluonVertex",
  BaseStyle->"AddOnsLink",
  ButtonData:>"GluonVertex",
  ButtonNote->"GluonVertex"],
 "."
}], "Text",
 CellTags->"QuarkGluonVertex"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"QuarkGluonVertex"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"QuarkGluonVertex", "[", 
  RowBox[{"\[Mu]", ",", "a"}], "]"}]], "Input",
 CellTags->"QuarkGluonVertex",
 CellLabel->"In[1356]:="],

Cell[BoxData[
 FormBox[
  SubsuperscriptBox["Q", "a", "\[Mu]"], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970362252324*^9},
 CellTags->"QuarkGluonVertex",
 CellLabel->"Out[1356]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"QuarkGluonVertex", "[", 
  RowBox[{"\[Mu]", ",", "a", ",", 
   RowBox[{"CounterTerm", " ", "\[Rule]", "1"}]}], "]"}]], "Input",
 CellTags->"QuarkGluonVertex",
 CellLabel->"In[1357]:="],

Cell[BoxData[
 FormBox[
  SubsuperscriptBox["Q", "a", "\[Mu]"], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970362296356*^9},
 CellTags->"QuarkGluonVertex",
 CellLabel->"Out[1357]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"QuarkGluonVertex", "[", 
  RowBox[{"\[Mu]", ",", "a", ",", 
   RowBox[{"CounterTerm", " ", "\[Rule]", "2"}]}], "]"}]], "Input",
 CellTags->"QuarkGluonVertex",
 CellLabel->"In[1358]:="],

Cell[BoxData[
 FormBox[
  SubsuperscriptBox["Q", "a", "\[Mu]"], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970362380445*^9},
 CellTags->"QuarkGluonVertex",
 CellLabel->"Out[1358]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"QuarkGluonVertex", "[", 
  RowBox[{"\[Mu]", ",", "a", ",", 
   RowBox[{"CounterTerm", " ", "\[Rule]", "3"}]}], "]"}]], "Input",
 CellTags->"QuarkGluonVertex",
 CellLabel->"In[1359]:="],

Cell[BoxData[
 FormBox[
  SubsuperscriptBox["Q", "a", "\[Mu]"], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970362427569*^9},
 CellTags->"QuarkGluonVertex",
 CellLabel->"Out[1359]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"QuarkGluonVertex", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"p", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
   RowBox[{"{", "q", "}"}], ",", 
   RowBox[{"{", "k", "}"}], ",", 
   RowBox[{"OPE", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"QuarkGluonVertex",
 CellLabel->"In[1360]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"QuarkGluonVertex", "(", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"p", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
    RowBox[{"{", "q", "}"}], ",", 
    RowBox[{"{", "k", "}"}], ",", 
    RowBox[{"\[CapitalOmega]", "\[Rule]", "True"}]}], ")"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970362511757*^9},
 CellTags->"QuarkGluonVertex",
 CellLabel->"Out[1360]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"QuarkGluonVertex", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"p", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
   RowBox[{"{", "q", "}"}], ",", 
   RowBox[{"{", "k", "}"}], ",", 
   RowBox[{"OPE", "\[Rule]", "False"}]}], "]"}]], "Input",
 CellTags->"QuarkGluonVertex",
 CellLabel->"In[1361]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"QuarkGluonVertex", "(", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"p", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
    RowBox[{"{", "q", "}"}], ",", 
    RowBox[{"{", "k", "}"}], ",", 
    RowBox[{"\[CapitalOmega]", "\[Rule]", "False"}]}], ")"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970362560108*^9},
 CellTags->"QuarkGluonVertex",
 CellLabel->"Out[1361]="]
}, Open  ]]
}, Open  ]],

Cell["QuarkMass", "ObjectName",
 CellTags->"QuarkMass"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"QuarkMass"],

Cell["QuarkMass is an option of Amplitude.", "Text",
 CellTags->"QuarkMass"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Amplitude",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Amplitude",
  ButtonNote->"Amplitude"],
 "."
}], "Text",
 CellTags->"QuarkMass"]
}, Open  ]],

Cell["QuarkPropagator", "ObjectName",
 CellTags->"QuarkPropagator"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"QuarkPropagator"],

Cell["\<\
QuarkPropagator[p] is the massless quark propagator. QuarkPropagator[{p,m}] \
or gives the quark propagator with mass m.\
\>", "Text",
 CellTags->"QuarkPropagator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "QuarkPropagator", "]"}]], "Input",
 CellTags->"QuarkPropagator",
 CellLabel->"In[1362]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"CounterTerm", "\[Rule]", "False"}], ",", 
    RowBox[{"CouplingConstant", "\[Rule]", 
     SubscriptBox["g", "s"]}], ",", 
    RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
    RowBox[{"Explicit", "\[Rule]", "False"}], ",", 
    RowBox[{"Loop", "\[Rule]", "0"}], ",", 
    RowBox[{"\[CapitalOmega]", "\[Rule]", "False"}], ",", 
    RowBox[{"Polarization", "\[Rule]", "0"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.50097036260355*^9},
 CellTags->"QuarkPropagator",
 CellLabel->"Out[1362]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["GluonPropagator",
  BaseStyle->"AddOnsLink",
  ButtonData:>"GluonPropagator",
  ButtonNote->"GluonPropagator"],
 ", ",
 ButtonBox["QuarkGluonVertex",
  BaseStyle->"AddOnsLink",
  ButtonData:>"QuarkGluonVertex",
  ButtonNote->"QuarkGluonVertex"],
 "."
}], "Text",
 CellTags->"QuarkPropagator"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"QuarkPropagator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"QuarkPropagator", "[", "p", "]"}]], "Input",
 CellTags->"QuarkPropagator",
 CellLabel->"In[1363]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["\[CapitalPi]", "q"], "(", "p", ")"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970362684177*^9},
 CellTags->"QuarkPropagator",
 CellLabel->"Out[1363]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"QuarkPropagator", "[", 
  RowBox[{"{", 
   RowBox[{"p", ",", "m"}], "}"}], "]"}]], "Input",
 CellTags->"QuarkPropagator",
 CellLabel->"In[1364]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["\[CapitalPi]", "q"], "(", "p", ")"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970362729389*^9},
 CellTags->"QuarkPropagator",
 CellLabel->"Out[1364]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"QuarkPropagator", "[", 
  RowBox[{"p", ",", "1", ",", "2"}], "]"}]], "Input",
 CellTags->"QuarkPropagator",
 CellLabel->"In[1365]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["\[CapitalPi]", "q"], "(", "p", ")"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970362772838*^9},
 CellTags->"QuarkPropagator",
 CellLabel->"Out[1365]="]
}, Open  ]]
}, Open  ]],

Cell["ReduceGamma", "ObjectName",
 CellTags->"ReduceGamma"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"ReduceGamma"],

Cell["\<\
ReduceGamma is an option of OneLoop. If set to True all DiracMatrix[6] and \
DiracMatrix[7] (i.e. all ChiralityProjector) are reduced to Gamma5.\
\>", "Text",
 CellTags->"ReduceGamma"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["OneLoop",
  BaseStyle->"AddOnsLink",
  ButtonData:>"OneLoop",
  ButtonNote->"OneLoop"],
 "."
}], "Text",
 CellTags->"ReduceGamma"]
}, Open  ]],

Cell["ReduceToScalars", "ObjectName",
 CellTags->"ReduceToScalars"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"ReduceToScalars"],

Cell["\<\
ReduceToScalars is an option for OneLoop  and OneLoopSum that specifies \
whether the result will be reduced to scalar A0, B0, C0 and D0 scalar \
integrals.\
\>", "Text",
 CellTags->"ReduceToScalars"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["OneLoop",
  BaseStyle->"AddOnsLink",
  ButtonData:>"OneLoop",
  ButtonNote->"OneLoop"],
 ", ",
 ButtonBox["OneLoopSum",
  BaseStyle->"AddOnsLink",
  ButtonData:>"OneLoopSum",
  ButtonNote->"OneLoopSum"],
 "."
}], "Text",
 CellTags->"ReduceToScalars"]
}, Open  ]],

Cell["Rename", "ObjectName",
 CellTags->"Rename"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Rename"],

Cell["\<\
Rename is an option for Contract. If set to True, dummy indices in Eps are \
renamed, using $MU[i].\
\>", "Text",
 CellTags->"Rename"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Contract",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Contract",
  ButtonNote->"Contract"],
 "."
}], "Text",
 CellTags->"Rename"]
}, Open  ]],

Cell["RHI  ***unfinished***", "ObjectName",
 CellTags->"RHI"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"RHI"],

Cell["\<\
RHI[{v,w,x,y,z},{a,b,c,d,e,f,g}, {al,be,ga,de,ep}]. (sn \[Rule] 1, mark1 \
\[Rule] 1, mark2 \[Rule] 1, mark3 \[Rule] 1, eph \[Rule] Epsilon/2 ). The \
exponents of the numerator scalar product are (dl = OPEDelta): 
v: k1.k1, w: k2.k2,  x: p.k1, y: p.k2, z: k1.k2. 
a: dl.k1, b: dl.k2,  c: dl.(p-k1), d: dl.(p-k2), e: dl.(k1-k2), f: \
dl.(p+k1-k2), g: dl.(p-k1-k2) 
RHI[any___,{a,b,c,d,e,0,0}, {al,be,ga,de,ep}] simplifies to  RHI[any, \
{a,b,c,d,e}, {al,be,ga,de,ep}]; 
RHI[{0,0,0,0,0},{a,b,c,d,e}, {al,be,ga,de,ep}] simplifies to  \
RHI[{a,b,c,d,e}, {al,be,ga,de,ep}].\
\>", "Text",
 CellTags->"RHI"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["RHI2FC",
  BaseStyle->"AddOnsLink",
  ButtonData:>"RHI2FC",
  ButtonNote->"RHI2FC"],
 "."
}], "Text",
 CellTags->"RHI"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"RHI"],

Cell[BoxData[""], "Input",
 CellTags->"RHI",
 CellLabel->"In[1366]:="]
}, Open  ]],

Cell["RHI2FC  ***unfinished***", "ObjectName",
 CellTags->"RHI2FC"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"RHI2FC"],

Cell["\<\
RHI[{v,w,x,y,z},{a,b,c,d,e,f,g}, {al,be,ga,de,ep}]. (sn \[Rule] 1, mark1 \
\[Rule] 1, mark2 \[Rule] 1, mark3 \[Rule] 1, eph \[Rule] Epsilon/2 ). The \
exponents of the numerator scalar product are (dl = OPEDelta): 
v: k1.k1, w: k2.k2,  x: p.k1, y: p.k2, z: k1.k2. 
a: dl.k1, b: dl.k2,  c: dl.(p-k1), d: dl.(p-k2), e: dl.(k1-k2), f: \
dl.(p+k1-k2), g: dl.(p-k1-k2) 
RHI[any___,{a,b,c,d,e,0,0}, {al,be,ga,de,ep}] simplifies to  RHI[any, \
{a,b,c,d,e}, {al,be,ga,de,ep}]; 
RHI[{0,0,0,0,0},{a,b,c,d,e}, {al,be,ga,de,ep}] simplifies to  \
RHI[{a,b,c,d,e}, {al,be,ga,de,ep}].\
\>", "Text",
 CellTags->"RHI2FC"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["RHI2FC",
  BaseStyle->"AddOnsLink",
  ButtonData:>"RHI2FC",
  ButtonNote->"RHI2FC"],
 "."
}], "Text",
 CellTags->"RHI2FC"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"RHI2FC"],

Cell[BoxData[""], "Input",
 CellTags->"RHI2FC",
 CellLabel->"In[1367]:="]
}, Open  ]],

Cell["RHM  ***unfinished***", "ObjectName",
 CellTags->"RHM"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"RHM"],

Cell["RHM[] is like RHI[], gives Gamma functions.", "Text",
 CellTags->"RHM"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["RHI",
  BaseStyle->"AddOnsLink",
  ButtonData:>"RHI",
  ButtonNote->"RHI"],
 "."
}], "Text",
 CellTags->"RHM"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"RHM"],

Cell[BoxData[""], "Input",
 CellTags->"RHM",
 CellLabel->"In[1368]:="]
}, Open  ]],

Cell["RHO  ***unfinished***", "ObjectName",
 CellTags->"RHO"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"RHO"],

Cell["\<\
RHO[i] with i from 1 to 4 is an abbreviation for the 4 operators (eq. \
(3.2.17) -- (3.2.20)) defined in R.Hambergs thesis. The Lorentz indices are \
suppressed. The explicit expressions may be recovered by RHO[i, mu, nu, p].\
\>", "Text",
 CellTags->"RHO"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["RHI",
  BaseStyle->"AddOnsLink",
  ButtonData:>"RHI",
  ButtonNote->"RHI"],
 "."
}], "Text",
 CellTags->"RHO"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"RHO"],

Cell[BoxData[""], "Input",
 CellTags->"RHO",
 CellLabel->"In[1369]:="]
}, Open  ]],

Cell["RHP  ", "ObjectName",
 CellTags->"RHP"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"RHP"],

Cell["\<\
RHP[i, mu, nu, p] with i from 1 to 4 gives the projectors for RHO[i]. RHP[mu, \
nu, p] gives Sum[RHP[i,mu, nu, p] RHO[i], {i, 4}] collected with respect to \
mu and nu. See the thesis of Roelof Hamberg (but correcting a misprint).\
\>", "Text",
 CellTags->"RHP"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["RHO",
  BaseStyle->"AddOnsLink",
  ButtonData:>"RHO",
  ButtonNote->"RHO"],
 "."
}], "Text",
 CellTags->"RHP"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"RHP"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"RHP", "[", 
  RowBox[{"1", ",", "\[Mu]", ",", "\[Nu]", ",", "p"}], "]"}]], "Input",
 CellLabel->"In[1370]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Nu]"}]], 
    RowBox[{"D", "-", "2"}]], "+", 
   FractionBox[
    RowBox[{
     SuperscriptBox[
      FormBox["p",
       TraditionalForm], "2"], " ", 
     SuperscriptBox[
      FormBox["\[CapitalDelta]",
       TraditionalForm], 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], " ", 
     SuperscriptBox[
      FormBox["\[CapitalDelta]",
       TraditionalForm], 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm]]}], 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"D", "-", "2"}], ")"}], " ", 
     SuperscriptBox[
      FormBox[
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p",
         TraditionalForm]}],
       TraditionalForm], "2"]}]], "+", 
   FractionBox[
    RowBox[{
     RowBox[{
      SuperscriptBox[
       FormBox["\[CapitalDelta]",
        TraditionalForm], 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]]}], "+", 
     RowBox[{
      SuperscriptBox[
       FormBox["\[CapitalDelta]",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]]}]}], 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"2", "-", "D"}], ")"}], " ", 
     RowBox[{
      FormBox["\[CapitalDelta]",
       TraditionalForm], "\[CenterDot]", 
      FormBox["p",
       TraditionalForm]}]}]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009703629755507`*^9},
 CellLabel->"Out[1370]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"RHP", "[", 
  RowBox[{"2", ",", "\[Mu]", ",", "\[Nu]", ",", "p"}], "]"}]], "Input",
 CellLabel->"In[1371]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{
    SuperscriptBox[
     FormBox["p",
      TraditionalForm], "2"], " ", 
    SuperscriptBox[
     FormBox["\[CapitalDelta]",
      TraditionalForm], 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], " ", 
    SuperscriptBox[
     FormBox["\[CapitalDelta]",
      TraditionalForm], 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm]]}], 
   SuperscriptBox[
    FormBox[
     RowBox[{
      FormBox["\[CapitalDelta]",
       TraditionalForm], "\[CenterDot]", 
      FormBox["p",
       TraditionalForm]}],
     TraditionalForm], "2"]], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970363827641*^9},
 CellLabel->"Out[1371]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"RHP", "[", 
  RowBox[{"3", ",", "\[Mu]", ",", "\[Nu]", ",", "p"}], "]"}]], "Input",
 CellLabel->"In[1372]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{
     SuperscriptBox[
      FormBox["p",
       TraditionalForm], 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], " ", 
     SuperscriptBox[
      FormBox["p",
       TraditionalForm], 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm]]}], 
    SuperscriptBox[
     FormBox["p",
      TraditionalForm], "2"]], "-", 
   FractionBox[
    RowBox[{
     RowBox[{
      SuperscriptBox[
       FormBox["\[CapitalDelta]",
        TraditionalForm], 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]]}], "+", 
     RowBox[{
      SuperscriptBox[
       FormBox["\[CapitalDelta]",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]]}]}], 
    RowBox[{
     FormBox["\[CapitalDelta]",
      TraditionalForm], "\[CenterDot]", 
     FormBox["p",
      TraditionalForm]}]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970363916501*^9},
 CellLabel->"Out[1372]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"RHP", "[", 
  RowBox[{"4", ",", "\[Mu]", ",", "\[Nu]", ",", "p"}], "]"}]], "Input",
 CellLabel->"In[1373]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{
    SuperscriptBox[
     FormBox["p",
      TraditionalForm], 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], " ", 
    SuperscriptBox[
     FormBox["p",
      TraditionalForm], 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm]]}], 
   RowBox[{"2", " ", 
    SuperscriptBox[
     FormBox["p",
      TraditionalForm], "2"]}]], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009703640085793`*^9},
 CellLabel->"Out[1373]="]
}, Open  ]],

Cell[BoxData[""], "Input",
 CellTags->"RHP",
 CellLabel->"In[1374]:="]
}, Open  ]],

Cell["RTL  ***unfinished***", "ObjectName",
 CellTags->"RTL"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"RTL"],

Cell["\<\
RTL[exp] inserts the list of known TLI integrals into exp, substitutes D\
\[RightArrow]4+Epsilon and expands around Epsilon up to the finite part.\
\>", "Text",
 CellTags->"RTL"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["TLI",
  BaseStyle->"AddOnsLink",
  ButtonData:>"TLI",
  ButtonNote->"TLI"],
 ", ",
 ButtonBox["Epsilon",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Epsilon",
  ButtonNote->"Epsilon"],
 "."
}], "Text",
 CellTags->"RTL"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"RTL"],

Cell[BoxData[""], "Input",
 CellTags->"RTL",
 CellLabel->"In[1375]:="]
}, Open  ]],

Cell["RightPartialD", "ObjectName",
 CellTags->"RightPartialD"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"RightPartialD"],

Cell[TextData[{
 "RightPartialD[",
 Cell[BoxData[
  FormBox["\[Mu]", TraditionalForm]]],
 "] denotes ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["\[PartialD]", "\[Mu]"], TraditionalForm]]],
 ", acting to the right."
}], "Text",
 CellTags->"RightPartialD"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["ExpandPartialD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"ExpandPartialD",
  ButtonNote->"ExpandPartialD"],
 ", ",
 ButtonBox["PartialD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"PartialD",
  ButtonNote->"PartialD"],
 ", ",
 ButtonBox["LeftPartialD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"LeftPartialD",
  ButtonNote->"LeftPartialD"],
 "."
}], "Text",
 CellTags->"RightPartialD"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"RightPartialD"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"RightPartialD", "[", "\[Mu]", "]"}]], "Input",
 CellTags->"RightPartialD",
 CellLabel->"In[1376]:="],

Cell[BoxData[
 FormBox[
  SubscriptBox[
   OverscriptBox["\[PartialD]", "\[RightArrow]"], 
   FormBox[
    FormBox["\[Mu]",
     TraditionalForm],
    TraditionalForm]], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970364222485*^9},
 CellTags->"RightPartialD",
 CellLabel->"Out[1376]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"RightPartialD", "[", "\[Mu]", "]"}], ".", 
  RowBox[{"QuantumField", "[", 
   RowBox[{"A", ",", 
    RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}]}], "]"}]}]], "Input",
 CellTags->"RightPartialD",
 CellLabel->"In[1377]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox[
    OverscriptBox["\[PartialD]", "\[RightArrow]"], 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], ".", 
   SubscriptBox["A", "\[Mu]"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970364385191*^9},
 CellTags->"RightPartialD",
 CellLabel->"Out[1377]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ExpandPartialD", "[", "%", "]"}]], "Input",
 CellTags->"RightPartialD",
 CellLabel->"In[1378]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"QuantumField", "(", 
   RowBox[{
    RowBox[{
    "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "(", 
     FormBox["\[Mu]",
      TraditionalForm], ")"}], ",", "A", ",", 
    FormBox["\[Mu]",
     TraditionalForm]}], ")"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970364494464*^9},
 CellTags->"RightPartialD",
 CellLabel->"Out[1378]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "StandardForm"}]], "Input",
 CellTags->"RightPartialD",
 CellLabel->"In[1379]:="],

Cell[BoxData[
 RowBox[{"QuantumField", "[", 
  RowBox[{
   RowBox[{
   "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", "[", 
    RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], "]"}], ",", "A", ",", 
   RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}]}], "]"}]], "Output",
 CellChangeTimes->{3.500970364572754*^9},
 CellTags->"RightPartialD",
 CellLabel->"Out[1379]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"RightPartialD", "[", "\[Mu]", "]"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"RightPartialD",
 CellLabel->"In[1380]:="],

Cell[BoxData[
 RowBox[{"RightPartialD", "[", 
  RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], "]"}]], "Output",
 CellChangeTimes->{3.500970364619945*^9},
 CellTags->"RightPartialD",
 CellLabel->"Out[1380]//StandardForm="]
}, Open  ]]
}, Open  ]],

Cell["RussianTrick", "ObjectName",
 CellTags->"RussianTrick"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"RussianTrick"],

Cell["\<\
RussianTrick[exp, k, {q1,q2,p}] (=RussianTrick[exp,p,p,{q1,q2,p}]) does the \
integration by parts where p is the external momentum. RussianTrick[exp, k,l, \
{q1,q2,p}] (=RussianTrick[exp,k,l]) does integration by parts where l is the \
momentum to be differentiated.\
\>", "Text",
 CellTags->"RussianTrick"],

Cell["The result is an expression which is vanishing.", "Text",
 CellTags->"RussianTrick"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FourDivergence",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FourDivergence",
  ButtonNote->"FourDivergence"],
 ", ",
 ButtonBox["FourLaplacian",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FourLaplacian",
  ButtonNote->"FourLaplacian"],
 "."
}], "Text",
 CellTags->"RussianTrick"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"RussianTrick"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t", "=", 
  RowBox[{"RHI", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"OPEm", ",", "0", ",", " ", "0", ",", "0", ",", " ", "0"}], 
     "}"}], ",", " ", 
    RowBox[{"{", 
     RowBox[{
     "1", ",", " ", "1", ",", " ", "1", ",", " ", "1", ",", " ", "1"}], 
     "}"}]}], "]"}]}]], "Input",
 CellTags->"RussianTrick",
 CellLabel->"In[1381]:="],

Cell[BoxData[
 FormBox[
  SubsuperscriptBox[
   StyleBox["T",
    FontFamily->"Times",
    FontWeight->"Bold"], 
   StyleBox[
    RowBox[{"1", "1", "1", "1", "1"}],
    FontColor->RGBColor[0, 0, 1]], 
   RowBox[{
    FormBox["m",
     TraditionalForm], 
    FormBox["0",
     TraditionalForm], 
    FormBox["0",
     TraditionalForm], 
    FormBox["0",
     TraditionalForm], 
    FormBox["0",
     TraditionalForm]}]], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009703646649857`*^9},
 CellTags->"RussianTrick",
 CellLabel->"Out[1381]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t", "//", "RHI2FC"}]], "Input",
 CellTags->"RussianTrick",
 CellLabel->"In[1382]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{
      FormBox["\[CapitalDelta]",
       TraditionalForm], "\[CenterDot]", 
      FormBox["q1",
       TraditionalForm]}], ")"}], "m"], 
   RowBox[{
    SuperscriptBox["q1", "2"], ".", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"q1", "-", "p"}], ")"}], "2"], ".", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"q1", "-", "q2"}], ")"}], "2"], ".", 
    SuperscriptBox["q2", "2"], ".", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"q2", "-", "p"}], ")"}], "2"]}]], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970364760779*^9},
 CellTags->"RussianTrick",
 CellLabel->"Out[1382]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"RussianTrick", "[", 
  RowBox[{
   RowBox[{"%", "//", "RHI2FC"}], ",", "q2"}], "]"}]], "Input",
 CellTags->"RussianTrick",
 CellLabel->"In[1383]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{
     RowBox[{"(", 
      RowBox[{"D", "-", "4"}], ")"}], " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["q1",
         TraditionalForm]}], ")"}], "m"]}], 
    RowBox[{
     SuperscriptBox["q1", "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q1", "-", "p"}], ")"}], "2"], ".", 
     SuperscriptBox["q2", "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q2", "-", "p"}], ")"}], "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q2", "-", "q1"}], ")"}], "2"]}]], "+", 
   FractionBox[
    RowBox[{
     SuperscriptBox[
      FormBox["p",
       TraditionalForm], "2"], " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["q1",
         TraditionalForm]}], ")"}], "m"]}], 
    RowBox[{
     SuperscriptBox["q1", "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q1", "-", "p"}], ")"}], "2"], ".", 
     SuperscriptBox["q2", "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q2", "-", "p"}], ")"}], "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q2", "-", "p"}], ")"}], "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q2", "-", "q1"}], ")"}], "2"]}]], "-", 
   FractionBox[
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{
       FormBox["\[CapitalDelta]",
        TraditionalForm], "\[CenterDot]", 
       FormBox["q1",
        TraditionalForm]}], ")"}], "m"], 
    RowBox[{
     SuperscriptBox["q1", "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q1", "-", "p"}], ")"}], "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q2", "-", "p"}], ")"}], "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q2", "-", "p"}], ")"}], "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q2", "-", "q1"}], ")"}], "2"]}]], "-", 
   FractionBox[
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{
       FormBox["\[CapitalDelta]",
        TraditionalForm], "\[CenterDot]", 
       FormBox["q1",
        TraditionalForm]}], ")"}], "m"], 
    RowBox[{
     SuperscriptBox["q1", "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q1", "-", "p"}], ")"}], "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q2", "-", "p"}], ")"}], "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q2", "-", "q1"}], ")"}], "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q2", "-", "q1"}], ")"}], "2"]}]], "+", 
   FractionBox[
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{
       FormBox["\[CapitalDelta]",
        TraditionalForm], "\[CenterDot]", 
       FormBox["q1",
        TraditionalForm]}], ")"}], "m"], 
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q1", "-", "p"}], ")"}], "2"], ".", 
     SuperscriptBox["q2", "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q2", "-", "p"}], ")"}], "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q2", "-", "q1"}], ")"}], "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q2", "-", "q1"}], ")"}], "2"]}]]}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.50097036527059*^9},
 CellTags->"RussianTrick",
 CellLabel->"Out[1383]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FC2RHI", "[", "%", "]"}]], "Input",
 CellTags->"RussianTrick",
 CellLabel->"In[1384]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{"D", "-", "4"}], ")"}], " ", 
    FormBox[
     SubsuperscriptBox[
      StyleBox["T",
       FontFamily->"Times",
       FontWeight->"Bold"], 
      StyleBox[
       RowBox[{"1", "1", "1", "1", "1"}],
       FontColor->RGBColor[0, 0, 1]], 
      RowBox[{
       FormBox["m",
        TraditionalForm], 
       FormBox["0",
        TraditionalForm], 
       FormBox["0",
        TraditionalForm], 
       FormBox["0",
        TraditionalForm], 
       FormBox["0",
        TraditionalForm]}]],
     TraditionalForm]}], "+", 
   RowBox[{
    SuperscriptBox[
     FormBox["p",
      TraditionalForm], "2"], " ", 
    FormBox[
     SubsuperscriptBox[
      StyleBox["T",
       FontFamily->"Times",
       FontWeight->"Bold"], 
      StyleBox[
       RowBox[{"1", "1", "1", "2", "1"}],
       FontColor->RGBColor[0, 0, 1]], 
      RowBox[{
       FormBox["m",
        TraditionalForm], 
       FormBox["0",
        TraditionalForm], 
       FormBox["0",
        TraditionalForm], 
       FormBox["0",
        TraditionalForm], 
       FormBox["0",
        TraditionalForm]}]],
     TraditionalForm]}], "+", 
   FormBox[
    SubsuperscriptBox[
     StyleBox["T",
      FontFamily->"Times",
      FontWeight->"Bold"], 
     StyleBox[
      RowBox[{"0", "1", "1", "1", "2"}],
      FontColor->RGBColor[0, 0, 1]], 
     RowBox[{
      FormBox["m",
       TraditionalForm], 
      FormBox["0",
       TraditionalForm], 
      FormBox["0",
       TraditionalForm], 
      FormBox["0",
       TraditionalForm], 
      FormBox["0",
       TraditionalForm]}]],
    TraditionalForm], "-", 
   FormBox[
    SubsuperscriptBox[
     StyleBox["T",
      FontFamily->"Times",
      FontWeight->"Bold"], 
     StyleBox[
      RowBox[{"1", "0", "1", "1", "2"}],
      FontColor->RGBColor[0, 0, 1]], 
     RowBox[{
      FormBox["m",
       TraditionalForm], 
      FormBox["0",
       TraditionalForm], 
      FormBox["0",
       TraditionalForm], 
      FormBox["0",
       TraditionalForm], 
      FormBox["0",
       TraditionalForm]}]],
    TraditionalForm], "-", 
   FormBox[
    SubsuperscriptBox[
     StyleBox["T",
      FontFamily->"Times",
      FontWeight->"Bold"], 
     StyleBox[
      RowBox[{"1", "0", "1", "2", "1"}],
      FontColor->RGBColor[0, 0, 1]], 
     RowBox[{
      FormBox["m",
       TraditionalForm], 
      FormBox["0",
       TraditionalForm], 
      FormBox["0",
       TraditionalForm], 
      FormBox["0",
       TraditionalForm], 
      FormBox["0",
       TraditionalForm]}]],
    TraditionalForm]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.50097036573997*^9},
 CellTags->"RussianTrick",
 CellLabel->"Out[1384]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Solve2", "[", 
  RowBox[{"%", ",", "t"}], "]"}]], "Input",
 CellTags->"RussianTrick",
 CellLabel->"In[1385]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    FormBox[
     SubsuperscriptBox[
      StyleBox["T",
       FontFamily->"Times",
       FontWeight->"Bold"], 
      StyleBox[
       RowBox[{"1", "1", "1", "1", "1"}],
       FontColor->RGBColor[0, 0, 1]], 
      RowBox[{
       FormBox["m",
        TraditionalForm], 
       FormBox["0",
        TraditionalForm], 
       FormBox["0",
        TraditionalForm], 
       FormBox["0",
        TraditionalForm], 
       FormBox["0",
        TraditionalForm]}]],
     TraditionalForm], "\[Rule]", 
    FractionBox[
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], "2"], " ", 
       FormBox[
        SubsuperscriptBox[
         StyleBox["T",
          FontFamily->"Times",
          FontWeight->"Bold"], 
         StyleBox[
          RowBox[{"1", "1", "1", "2", "1"}],
          FontColor->RGBColor[0, 0, 1]], 
         RowBox[{
          FormBox["m",
           TraditionalForm], 
          FormBox["0",
           TraditionalForm], 
          FormBox["0",
           TraditionalForm], 
          FormBox["0",
           TraditionalForm], 
          FormBox["0",
           TraditionalForm]}]],
        TraditionalForm]}], "+", 
      FormBox[
       SubsuperscriptBox[
        StyleBox["T",
         FontFamily->"Times",
         FontWeight->"Bold"], 
        StyleBox[
         RowBox[{"0", "1", "1", "1", "2"}],
         FontColor->RGBColor[0, 0, 1]], 
        RowBox[{
         FormBox["m",
          TraditionalForm], 
         FormBox["0",
          TraditionalForm], 
         FormBox["0",
          TraditionalForm], 
         FormBox["0",
          TraditionalForm], 
         FormBox["0",
          TraditionalForm]}]],
       TraditionalForm], "-", 
      FormBox[
       SubsuperscriptBox[
        StyleBox["T",
         FontFamily->"Times",
         FontWeight->"Bold"], 
        StyleBox[
         RowBox[{"1", "0", "1", "1", "2"}],
         FontColor->RGBColor[0, 0, 1]], 
        RowBox[{
         FormBox["m",
          TraditionalForm], 
         FormBox["0",
          TraditionalForm], 
         FormBox["0",
          TraditionalForm], 
         FormBox["0",
          TraditionalForm], 
         FormBox["0",
          TraditionalForm]}]],
       TraditionalForm], "-", 
      FormBox[
       SubsuperscriptBox[
        StyleBox["T",
         FontFamily->"Times",
         FontWeight->"Bold"], 
        StyleBox[
         RowBox[{"1", "0", "1", "2", "1"}],
         FontColor->RGBColor[0, 0, 1]], 
        RowBox[{
         FormBox["m",
          TraditionalForm], 
         FormBox["0",
          TraditionalForm], 
         FormBox["0",
          TraditionalForm], 
         FormBox["0",
          TraditionalForm], 
         FormBox["0",
          TraditionalForm]}]],
       TraditionalForm]}], 
     RowBox[{"4", "-", "D"}]]}], "}"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009703658533278`*^9},
 CellTags->"RussianTrick",
 CellLabel->"Out[1385]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"Clear", "[", "t", "]"}]], "Input",
 CellTags->"RussianTrick",
 CellLabel->"In[1386]:="]
}, Open  ]],

Cell["ScalarGluonVertex", "ObjectName",
 CellTags->"ScalarGluonVertex"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"ScalarGluonVertex"],

Cell["\<\
ScalarGluonVertex[{p}, {q}, {mu,a}] or ScalarGluonVertex[ p,  q,  mu, a ] \
yields the scalar-scalar-gluon vertex (p and q are incoming momenta).
ScalarGluonVertex[{mu,a}, {nu,b}] yields the scalar-scalar-gluon-gluon vertex \
(p and q are incoming momenta).
The dimension  and the name of the coupling constant are determined by the \
options Dimension and CouplingConstant.\
\>", "Text",
 CellTags->"ScalarGluonVertex"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "ScalarGluonVertex", "]"}]], "Input",
 CellTags->"ScalarGluonVertex",
 CellLabel->"In[1387]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
    RowBox[{"CouplingConstant", "\[Rule]", 
     SubscriptBox["g", "s"]}]}], "}"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009703659566708`*^9},
 CellTags->"ScalarGluonVertex",
 CellLabel->"Out[1387]="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"ScalarGluonVertex"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ScalarGluonVertex", "[", 
  RowBox[{
   RowBox[{"{", "p", "}"}], ",", " ", 
   RowBox[{"{", "q", "}"}], ",", " ", 
   RowBox[{"{", 
    RowBox[{"mu", ",", " ", "a"}], "}"}]}], "]"}]], "Input",
 CellTags->"ScalarGluonVertex",
 CellLabel->"In[1388]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"\[ImaginaryI]", " ", 
   SubscriptBox["T", 
    FormBox[
     FormBox["a",
      TraditionalForm],
     TraditionalForm]], " ", 
   SubscriptBox["g", "s"], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     FormBox[
      RowBox[{"p", "-", "q"}],
      TraditionalForm], ")"}], 
    FormBox[
     FormBox["mu",
      TraditionalForm],
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970366060712*^9},
 CellTags->"ScalarGluonVertex",
 CellLabel->"Out[1388]="]
}, Open  ]]
}, Open  ]],

Cell["ScalarProduct", "ObjectName",
 CellTags->"ScalarProduct"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"ScalarProduct"],

Cell[TextData[{
 "ScalarProduct[p, q] is the input for scalar product. ScalarProduct[p] is \
equivalent to ScalarProduct[p, p]. Expansion of sums of momenta in \
ScalarProduct is done with ExpandScalarProduct. Scalar product may be set, \
e.g., ScalarProduct[a, b] = ",
 Cell[BoxData[
  FormBox[
   RowBox[{"m", "^", "2"}], TraditionalForm]]],
 "; but a and b must not contain sums. Note that ScalarProduct[a, b] = ",
 Cell[BoxData[
  FormBox[
   RowBox[{"m", "^", "2"}], TraditionalForm]]],
 " actually sets also: Pair[Momentum[a, ___], Momentum[b, ___]] = ",
 Cell[BoxData[
  FormBox[
   RowBox[{"m", "^", "2"}], TraditionalForm]]],
 ". It is encouraged to always set ScalarProduct's ",
 StyleBox["before",
  FontWeight->"Bold"],
 " any calculation. This improves the performance of FeynCalc ."
}], "Text",
 CellTags->"ScalarProduct"],

Cell[TextData[{
 "See also: ",
 ButtonBox["Calc",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Calc",
  ButtonNote->"Calc"],
 ", ",
 ButtonBox["ClearScalarProducts",
  BaseStyle->"AddOnsLink",
  ButtonData:>"ClearScalarProducts",
  ButtonNote->"ClearScalarProducts"],
 ", ",
 ButtonBox["ExpandScalarProduct",
  BaseStyle->"AddOnsLink",
  ButtonData:>"ExpandScalarProduct",
  ButtonNote->"ExpandScalarProduct"],
 ", ",
 ButtonBox["ScalarProductCancel",
  BaseStyle->"AddOnsLink",
  ButtonData:>"ScalarProductCancel",
  ButtonNote->"ScalarProductCancel"],
 ", ",
 ButtonBox["Pair",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Pair",
  ButtonNote->"Pair"],
 ", ",
 ButtonBox["SP",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SP",
  ButtonNote->"SP"],
 ", ",
 ButtonBox["SPD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SPD",
  ButtonNote->"SPD"],
 "."
}], "Text",
 CellTags->"ScalarProduct"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"ScalarProduct"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ScalarProduct", "[", 
  RowBox[{"p", ",", "q"}], "]"}]], "Input",
 CellTags->"ScalarProduct",
 CellLabel->"In[1389]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox["p",
    TraditionalForm], "\[CenterDot]", 
   FormBox["q",
    TraditionalForm]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970366160563*^9},
 CellTags->"ScalarProduct",
 CellLabel->"Out[1389]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ScalarProduct", "[", 
  RowBox[{
   RowBox[{"p", "+", "q"}], ",", 
   RowBox[{"-", "q"}]}], "]"}]], "Input",
 CellTags->"ScalarProduct",
 CellLabel->"In[1390]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   RowBox[{"(", 
    RowBox[{
     FormBox["q",
      TraditionalForm], "\[CenterDot]", "(", 
     FormBox[
      RowBox[{"p", "+", "q"}],
      TraditionalForm], ")"}], ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970366269842*^9},
 CellTags->"ScalarProduct",
 CellLabel->"Out[1390]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ScalarProduct", "[", 
  RowBox[{"p", ",", "p"}], "]"}]], "Input",
 CellTags->"ScalarProduct",
 CellLabel->"In[1391]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox[
   FormBox["p",
    TraditionalForm], "2"], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970366322706*^9},
 CellTags->"ScalarProduct",
 CellLabel->"Out[1391]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ScalarProduct", "[", "q", "]"}]], "Input",
 CellTags->"ScalarProduct",
 CellLabel->"In[1392]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox[
   FormBox["q",
    TraditionalForm], "2"], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970366794561*^9},
 CellTags->"ScalarProduct",
 CellLabel->"Out[1392]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"ScalarProduct", "[", 
   RowBox[{"p", ",", "q"}], "]"}], "//", "StandardForm"}]], "Input",
 CellTags->"ScalarProduct",
 CellLabel->"In[1393]:="],

Cell[BoxData[
 RowBox[{"Pair", "[", 
  RowBox[{
   RowBox[{"Momentum", "[", "p", "]"}], ",", 
   RowBox[{"Momentum", "[", "q", "]"}]}], "]"}]], "Output",
 CellChangeTimes->{3.5009703668351917`*^9},
 CellTags->"ScalarProduct",
 CellLabel->"Out[1393]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"ScalarProduct", "[", 
   RowBox[{"p", ",", "q", ",", 
    RowBox[{"Dimension", "\[Rule]", "D"}]}], "]"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"ScalarProduct",
 CellLabel->"In[1394]:="],

Cell[BoxData[
 RowBox[{"Pair", "[", 
  RowBox[{
   RowBox[{"Momentum", "[", 
    RowBox[{"p", ",", "D"}], "]"}], ",", 
   RowBox[{"Momentum", "[", 
    RowBox[{"q", ",", "D"}], "]"}]}], "]"}]], "Output",
 CellChangeTimes->{3.500970366878243*^9},
 CellTags->"ScalarProduct",
 CellLabel->"Out[1394]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"ScalarProduct", "[", 
   RowBox[{
    SubscriptBox["p", "1"], ",", 
    SubscriptBox["p", "2"]}], "]"}], " ", "=", " ", 
  RowBox[{"s", "/", "2"}]}]], "Input",
 CellTags->"ScalarProduct",
 CellLabel->"In[1395]:="],

Cell[BoxData[
 FormBox[
  FractionBox["s", "2"], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970366974997*^9},
 CellTags->"ScalarProduct",
 CellLabel->"Out[1395]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ExpandScalarProduct", "[", " ", 
  RowBox[{"ScalarProduct", "[", 
   RowBox[{
    RowBox[{
     SubscriptBox["p", "1"], "-", "q"}], ",", 
    RowBox[{
     SubscriptBox["p", "2"], "-", "k"}]}], "]"}], "]"}]], "Input",
 CellTags->"ScalarProduct",
 CellLabel->"In[1396]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    RowBox[{
     FormBox["k",
      TraditionalForm], "\[CenterDot]", 
     FormBox[
      SubscriptBox["p", "1"],
      TraditionalForm]}]}], "+", 
   RowBox[{
    FormBox["k",
     TraditionalForm], "\[CenterDot]", 
    FormBox["q",
     TraditionalForm]}], "-", 
   RowBox[{
    FormBox["q",
     TraditionalForm], "\[CenterDot]", 
    FormBox[
     SubscriptBox["p", "2"],
     TraditionalForm]}], "+", 
   FractionBox["s", "2"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009703670725403`*^9},
 CellTags->"ScalarProduct",
 CellLabel->"Out[1396]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Calc", "[", " ", 
  RowBox[{"ScalarProduct", "[", 
   RowBox[{
    RowBox[{
     SubscriptBox["p", "1"], "-", "q"}], ",", 
    RowBox[{
     SubscriptBox["p", "2"], "-", "k"}]}], "]"}], "]"}]], "Input",
 CellTags->"ScalarProduct",
 CellLabel->"In[1397]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    RowBox[{
     FormBox["k",
      TraditionalForm], "\[CenterDot]", 
     FormBox[
      SubscriptBox["p", "1"],
      TraditionalForm]}]}], "+", 
   RowBox[{
    FormBox["k",
     TraditionalForm], "\[CenterDot]", 
    FormBox["q",
     TraditionalForm]}], "-", 
   RowBox[{
    FormBox["q",
     TraditionalForm], "\[CenterDot]", 
    FormBox[
     SubscriptBox["p", "2"],
     TraditionalForm]}], "+", 
   FractionBox["s", "2"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970367298194*^9},
 CellTags->"ScalarProduct",
 CellLabel->"Out[1397]="]
}, Open  ]],

Cell[BoxData["ClearScalarProducts"], "Input",
 CellTags->"ScalarProduct",
 CellLabel->"In[1398]:="]
}, Open  ]],

Cell["ScalarProductCancel", "ObjectName",
 CellTags->"ScalarProductCancel"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"ScalarProductCancel"],

Cell["\<\
ScalarProductCancel[exp, q1, q2, ...] cancels scalar products with \
propagators. ScalarProductCancel[exp] cancels simple cases.\
\>", "Text",
 CellTags->"ScalarProductCancel"],

Cell[CellGroupData[{

Cell[BoxData["SPC"], "Input",
 CellTags->"ScalarProductCancel",
 CellLabel->"In[1399]:="],

Cell[BoxData[
 FormBox["ScalarProductCancel", TraditionalForm]], "Output",
 CellChangeTimes->{3.5009703674509993`*^9},
 CellTags->"ScalarProductCancel",
 CellLabel->"Out[1399]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ScalarProductCancel", "//", "Options"}]], "Input",
 CellTags->"ScalarProductCancel",
 CellLabel->"In[1400]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"ChangeDimension", "\[Rule]", "D"}], ",", 
    RowBox[{"Collecting", "\[Rule]", "True"}], ",", 
    RowBox[{"FeynAmpDenominatorSimplify", "\[Rule]", "False"}], ",", 
    RowBox[{"FeynAmpDenominatorCombine", "\[Rule]", "True"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.5009703674933434`*^9},
 CellTags->"ScalarProductCancel",
 CellLabel->"Out[1400]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "SPC"}]], "Input",
 CellTags->"ScalarProductCancel",
 CellLabel->"In[1401]:="],

Cell[BoxData[
 StyleBox["\<\"SPC is an abbreviation for ScalarProductCancel.\"\>", 
  "MSG"]], "Print", "PrintUsage",
 CellChangeTimes->{3.500970368518517*^9},
 CellTags->{"Info3500973967-4982097", "ScalarProductCancel"}]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 ButtonBox["Calc",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Calc",
  ButtonNote->"Calc"],
 ", ",
 ButtonBox["ClearScalarProducts",
  BaseStyle->"AddOnsLink",
  ButtonData:>"ClearScalarProducts",
  ButtonNote->"ClearScalarProducts"],
 ", ",
 ButtonBox["ExpandScalarProduct",
  BaseStyle->"AddOnsLink",
  ButtonData:>"ExpandScalarProduct",
  ButtonNote->"ExpandScalarProduct"],
 ", ",
 ButtonBox["Pair",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Pair",
  ButtonNote->"Pair"],
 ", ",
 ButtonBox["SP",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SP",
  ButtonNote->"SP"],
 ", ",
 ButtonBox["SPD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SPD",
  ButtonNote->"SPD"],
 "."
}], "Text",
 CellTags->"ScalarProductCancel"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"ScalarProductCancel"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FeynAmpDenominatorSimplify",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FeynAmpDenominatorSimplify",
  ButtonNote->"FeynAmpDenominatorSimplify"],
 "."
}], "Text",
 CellTags->"ScalarProductCancel"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t1", " ", "=", " ", 
  RowBox[{
   RowBox[{
    RowBox[{"SPD", "[", 
     RowBox[{"q", ",", "p"}], "]"}], " ", 
    RowBox[{"FAD", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"q", ",", "m"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"q", "-", "p"}], ",", "0"}], "}"}]}], "]"}]}], "//", 
   "FCI"}]}]], "Input",
 CellTags->"ScalarProductCancel",
 CellLabel->"In[1402]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{
    FormBox["p",
     TraditionalForm], "\[CenterDot]", 
    FormBox["q",
     TraditionalForm]}], 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox["q", "2"], "-", 
      SuperscriptBox["m", "2"]}], ")"}], ".", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"q", "-", "p"}], ")"}], "2"]}]], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009703690224533`*^9},
 CellTags->"ScalarProductCancel",
 CellLabel->"Out[1402]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ScalarProductCancel", "[", 
  RowBox[{"t1", ",", "q"}], "]"}]], "Input",
 CellTags->"ScalarProductCancel",
 CellLabel->"In[1403]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{
     FractionBox[
      SuperscriptBox["m", "2"], "2"], "+", 
     FractionBox[
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], "2"], "2"]}], 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox["q", "2"], "-", 
       SuperscriptBox["m", "2"]}], ")"}], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q", "-", "p"}], ")"}], "2"]}]], "-", 
   FractionBox["1", 
    RowBox[{"2", " ", 
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox["q", "2"], "-", 
       SuperscriptBox["m", "2"]}], ")"}]}]], "+", 
   FractionBox["1", 
    RowBox[{"2", " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q", "-", "p"}], ")"}], "2"]}]]}], TraditionalForm]], "Output",\

 CellChangeTimes->{3.500970369145173*^9},
 CellTags->"ScalarProductCancel",
 CellLabel->"Out[1403]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FeynAmpDenominatorSimplify", "[", 
  RowBox[{"%", ",", "q"}], "]"}]], "Input",
 CellTags->"ScalarProductCancel",
 CellLabel->"In[1404]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{
     FractionBox[
      SuperscriptBox["m", "2"], "2"], "+", 
     FractionBox[
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], "2"], "2"]}], 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox["q", "2"], "-", 
       SuperscriptBox["m", "2"]}], ")"}], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q", "-", "p"}], ")"}], "2"]}]], "-", 
   FractionBox["1", 
    RowBox[{"2", " ", 
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox["q", "2"], "-", 
       SuperscriptBox["m", "2"]}], ")"}]}]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970369284923*^9},
 CellTags->"ScalarProductCancel",
 CellLabel->"Out[1404]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SPC", "[", 
  RowBox[{"t1", ",", "q", ",", 
   RowBox[{"FDS", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"ScalarProductCancel",
 CellLabel->"In[1405]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{
     FractionBox[
      SuperscriptBox["m", "2"], "2"], "+", 
     FractionBox[
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], "2"], "2"]}], 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox["q", "2"], "-", 
       SuperscriptBox["m", "2"]}], ")"}], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q", "-", "p"}], ")"}], "2"]}]], "-", 
   FractionBox["1", 
    RowBox[{"2", " ", 
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox["q", "2"], "-", 
       SuperscriptBox["m", "2"]}], ")"}]}]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009703694359293`*^9},
 CellTags->"ScalarProductCancel",
 CellLabel->"Out[1405]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t2", " ", "=", " ", 
  RowBox[{
   RowBox[{
    RowBox[{"SPD", "[", 
     RowBox[{"q2", ",", "p"}], "]"}], 
    RowBox[{"SPD", "[", 
     RowBox[{"q1", ",", "p"}], "]"}], " ", 
    RowBox[{"FAD", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"q1", ",", "m"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"q2", ",", "m"}], "}"}], ",", 
      RowBox[{"q1", "-", "p"}], ",", 
      RowBox[{"q2", "-", "p"}], ",", 
      RowBox[{"q2", "-", "q1"}]}], "]"}]}], "//", "FCI"}]}]], "Input",
 CellTags->"ScalarProductCancel",
 CellLabel->"In[1406]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{
    RowBox[{
     FormBox["p",
      TraditionalForm], "\[CenterDot]", 
     FormBox["q1",
      TraditionalForm]}], " ", 
    RowBox[{
     FormBox["p",
      TraditionalForm], "\[CenterDot]", 
     FormBox["q2",
      TraditionalForm]}]}], 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox["q1", "2"], "-", 
      SuperscriptBox["m", "2"]}], ")"}], ".", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"q1", "-", "p"}], ")"}], "2"], ".", 
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox["q2", "2"], "-", 
      SuperscriptBox["m", "2"]}], ")"}], ".", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"q2", "-", "p"}], ")"}], "2"], ".", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"q2", "-", "q1"}], ")"}], "2"]}]], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970369565503*^9},
 CellTags->"ScalarProductCancel",
 CellLabel->"Out[1406]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SPC", "[", 
  RowBox[{"t2", ",", "q1", ",", "q2", ",", 
   RowBox[{"FDS", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"ScalarProductCancel",
 CellLabel->"In[1407]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{
     RowBox[{"-", 
      FractionBox[
       SuperscriptBox["m", "2"], "2"]}], "-", 
     FractionBox[
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], "2"], "2"]}], 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox["q1", "2"], "-", 
       SuperscriptBox["m", "2"]}], ")"}], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q1", "-", "p"}], ")"}], "2"], ".", 
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox["q2", "2"], "-", 
       SuperscriptBox["m", "2"]}], ")"}], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q2", "-", "q1"}], ")"}], "2"]}]], "+", 
   FractionBox[
    RowBox[{
     FractionBox[
      SuperscriptBox["m", "2"], "2"], "+", 
     FractionBox[
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], "2"], "2"]}], 
    RowBox[{
     SuperscriptBox["q1", "2"], ".", 
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"q1", "-", "p"}], ")"}], "2"], "-", 
       SuperscriptBox["m", "2"]}], ")"}], ".", 
     SuperscriptBox["q2", "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q2", "-", "q1"}], ")"}], "2"]}]], "-", 
   FractionBox["1", 
    RowBox[{"2", " ", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"q1", "-", "p"}], ")"}], "2"], ".", 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox["q2", "2"], "-", 
        SuperscriptBox["m", "2"]}], ")"}], ".", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"q2", "-", "q1"}], ")"}], "2"]}]}]], "+", 
   FractionBox["1", 
    RowBox[{"4", " ", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox["q1", "2"], "-", 
        SuperscriptBox["m", "2"]}], ")"}], ".", 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox["q2", "2"], "-", 
        SuperscriptBox["m", "2"]}], ")"}], ".", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"q2", "-", "q1"}], ")"}], "2"]}]}]], "+", 
   FractionBox[
    RowBox[{
     FractionBox[
      SuperscriptBox["m", "4"], "4"], "+", 
     FractionBox[
      RowBox[{
       SuperscriptBox["m", "2"], " ", 
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], "2"]}], "2"], "+", 
     FractionBox[
      SuperscriptBox[
       FormBox[
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], "2"],
        TraditionalForm], "2"], "4"]}], 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox["q1", "2"], "-", 
       SuperscriptBox["m", "2"]}], ")"}], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q1", "-", "p"}], ")"}], "2"], ".", 
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox["q2", "2"], "-", 
       SuperscriptBox["m", "2"]}], ")"}], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q2", "-", "p"}], ")"}], "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q2", "-", "q1"}], ")"}], "2"]}]]}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970370091073*^9},
 CellTags->"ScalarProductCancel",
 CellLabel->"Out[1407]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", 
   RowBox[{"t1", ",", "t2"}], "]"}], ";"}]], "Input",
 CellTags->"ScalarProductCancel",
 CellLabel->"In[1408]:="]
}, Open  ]],

Cell["ScaleMu", "ObjectName",
 CellTags->"ScaleMu"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"ScaleMu"],

Cell["\<\
ScaleMu is the mass scale used for dimensional regularization of loop \
integrals.\
\>", "Text",
 CellTags->"ScaleMu"],

Cell[CellGroupData[{

Cell[BoxData["ScaleMu"], "Input",
 CellTags->"ScaleMu",
 CellLabel->"In[1409]:="],

Cell[BoxData[
 FormBox["\[Mu]", TraditionalForm]], "Output",
 CellChangeTimes->{3.50097037052782*^9},
 CellTags->"ScaleMu",
 CellLabel->"Out[1409]="]
}, Open  ]]
}, Open  ]],

Cell["Schouten", "ObjectName",
 CellTags->"Schouten"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Schouten"],

Cell["\<\
Schouten[expr] applies the Schouten identity on at most 42 terms in a sum. If \
Schouten should operate on larger expression you can give a second argument, \
e.g.: Schouten[expr, 4711] which will work on sums with less than 4711 terms.
Schouten is also an option of Contract and DiracTrace. It may be set to an \
integer indicating the maximum number of terms onto which the function \
Schouten will be applied .\
\>", "Text",
 CellTags->"Schouten"],

Cell[TextData[{
 "See also: ",
 ButtonBox["Contract",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Contract",
  ButtonNote->"Contract"],
 ", ",
 ButtonBox["DiracTrace",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracTrace",
  ButtonNote->"DiracTrace"],
 "."
}], "Text",
 CellTags->"Schouten"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Schouten"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t", "=", 
  RowBox[{
   RowBox[{"Sum", "[", 
    RowBox[{
     RowBox[{
      UnderoverscriptBox["\[Sum]", 
       RowBox[{"a", "=", "1"}], "4"], 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"SP", "[", 
         RowBox[{"k", ",", 
          RowBox[{"q", "[", "a", "]"}]}], "]"}], 
        RowBox[{"(", 
         RowBox[{
          FractionBox["1", "6"], 
          RowBox[{"Eps", "[", 
           RowBox[{
            RowBox[{"LorentzIndex", "[", "a", "]"}], ",", 
            RowBox[{"LorentzIndex", "[", "b", "]"}], ",", 
            RowBox[{"LorentzIndex", "[", "c", "]"}], ",", 
            RowBox[{"LorentzIndex", "[", "d", "]"}]}], "]"}], 
          RowBox[{"Eps", "[", 
           RowBox[{
            RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
            RowBox[{"Momentum", "[", 
             RowBox[{"q", "[", "b", "]"}], "]"}], ",", 
            RowBox[{"Momentum", "[", 
             RowBox[{"q", "[", "c", "]"}], "]"}], ",", 
            RowBox[{"Momentum", "[", 
             RowBox[{"q", "[", "d", "]"}], "]"}]}], "]"}]}], ")"}]}], ")"}]}],
      ",", 
     RowBox[{"{", 
      RowBox[{"b", ",", "1", ",", "4"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"c", ",", "1", ",", "4"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"d", ",", "1", ",", "4"}], "}"}]}], "]"}], "-", 
   RowBox[{
    RowBox[{"Eps", "[", 
     RowBox[{
      RowBox[{"Momentum", "[", 
       RowBox[{"q", "[", "1", "]"}], "]"}], ",", 
      RowBox[{"Momentum", "[", 
       RowBox[{"q", "[", "2", "]"}], "]"}], ",", 
      RowBox[{"Momentum", "[", 
       RowBox[{"q", "[", "3", "]"}], "]"}], ",", 
      RowBox[{"Momentum", "[", 
       RowBox[{"q", "[", "4", "]"}], "]"}]}], "]"}], "*", 
    RowBox[{"Pair", "[", 
     RowBox[{
      RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
      RowBox[{"Momentum", "[", "k", "]"}]}], "]"}]}]}]}]], "Input",
 CellTags->"Schouten",
 CellLabel->"In[1410]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SuperscriptBox[
     FormBox["k",
      TraditionalForm], 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], " ", 
    RowBox[{"(", 
     RowBox[{"-", 
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         RowBox[{"q", "(", "1", ")"}],
         TraditionalForm], 
        FormBox[
         RowBox[{"q", "(", "2", ")"}],
         TraditionalForm], 
        FormBox[
         RowBox[{"q", "(", "3", ")"}],
         TraditionalForm], 
        FormBox[
         RowBox[{"q", "(", "4", ")"}],
         TraditionalForm]}]]}], ")"}]}], "+", 
   RowBox[{
    FractionBox["1", "6"], " ", 
    FormBox[
     RowBox[{
      FormBox["k",
       TraditionalForm], "\[CenterDot]", 
      FormBox[
       RowBox[{"q", "(", "1", ")"}],
       TraditionalForm]}],
     TraditionalForm], " ", 
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "2", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "3", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "4", ")"}],
       TraditionalForm]}]]}], "-", 
   RowBox[{
    FractionBox["1", "6"], " ", 
    FormBox[
     RowBox[{
      FormBox["k",
       TraditionalForm], "\[CenterDot]", 
      FormBox[
       RowBox[{"q", "(", "1", ")"}],
       TraditionalForm]}],
     TraditionalForm], " ", 
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "2", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "4", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "3", ")"}],
       TraditionalForm]}]]}], "-", 
   RowBox[{
    FractionBox["1", "6"], " ", 
    FormBox[
     RowBox[{
      FormBox["k",
       TraditionalForm], "\[CenterDot]", 
      FormBox[
       RowBox[{"q", "(", "1", ")"}],
       TraditionalForm]}],
     TraditionalForm], " ", 
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "3", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "2", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "4", ")"}],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    FractionBox["1", "6"], " ", 
    FormBox[
     RowBox[{
      FormBox["k",
       TraditionalForm], "\[CenterDot]", 
      FormBox[
       RowBox[{"q", "(", "1", ")"}],
       TraditionalForm]}],
     TraditionalForm], " ", 
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "3", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "4", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "2", ")"}],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    FractionBox["1", "6"], " ", 
    FormBox[
     RowBox[{
      FormBox["k",
       TraditionalForm], "\[CenterDot]", 
      FormBox[
       RowBox[{"q", "(", "1", ")"}],
       TraditionalForm]}],
     TraditionalForm], " ", 
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "4", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "2", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "3", ")"}],
       TraditionalForm]}]]}], "-", 
   RowBox[{
    FractionBox["1", "6"], " ", 
    FormBox[
     RowBox[{
      FormBox["k",
       TraditionalForm], "\[CenterDot]", 
      FormBox[
       RowBox[{"q", "(", "1", ")"}],
       TraditionalForm]}],
     TraditionalForm], " ", 
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "4", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "3", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "2", ")"}],
       TraditionalForm]}]]}], "-", 
   RowBox[{
    FractionBox["1", "6"], " ", 
    FormBox[
     RowBox[{
      FormBox["k",
       TraditionalForm], "\[CenterDot]", 
      FormBox[
       RowBox[{"q", "(", "2", ")"}],
       TraditionalForm]}],
     TraditionalForm], " ", 
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "1", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "3", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "4", ")"}],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    FractionBox["1", "6"], " ", 
    FormBox[
     RowBox[{
      FormBox["k",
       TraditionalForm], "\[CenterDot]", 
      FormBox[
       RowBox[{"q", "(", "2", ")"}],
       TraditionalForm]}],
     TraditionalForm], " ", 
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "1", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "4", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "3", ")"}],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    FractionBox["1", "6"], " ", 
    FormBox[
     RowBox[{
      FormBox["k",
       TraditionalForm], "\[CenterDot]", 
      FormBox[
       RowBox[{"q", "(", "2", ")"}],
       TraditionalForm]}],
     TraditionalForm], " ", 
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "3", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "1", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "4", ")"}],
       TraditionalForm]}]]}], "-", 
   RowBox[{
    FractionBox["1", "6"], " ", 
    FormBox[
     RowBox[{
      FormBox["k",
       TraditionalForm], "\[CenterDot]", 
      FormBox[
       RowBox[{"q", "(", "2", ")"}],
       TraditionalForm]}],
     TraditionalForm], " ", 
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "3", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "4", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "1", ")"}],
       TraditionalForm]}]]}], "-", 
   RowBox[{
    FractionBox["1", "6"], " ", 
    FormBox[
     RowBox[{
      FormBox["k",
       TraditionalForm], "\[CenterDot]", 
      FormBox[
       RowBox[{"q", "(", "2", ")"}],
       TraditionalForm]}],
     TraditionalForm], " ", 
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "4", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "1", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "3", ")"}],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    FractionBox["1", "6"], " ", 
    FormBox[
     RowBox[{
      FormBox["k",
       TraditionalForm], "\[CenterDot]", 
      FormBox[
       RowBox[{"q", "(", "2", ")"}],
       TraditionalForm]}],
     TraditionalForm], " ", 
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "4", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "3", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "1", ")"}],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    FractionBox["1", "6"], " ", 
    FormBox[
     RowBox[{
      FormBox["k",
       TraditionalForm], "\[CenterDot]", 
      FormBox[
       RowBox[{"q", "(", "3", ")"}],
       TraditionalForm]}],
     TraditionalForm], " ", 
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "1", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "2", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "4", ")"}],
       TraditionalForm]}]]}], "-", 
   RowBox[{
    FractionBox["1", "6"], " ", 
    FormBox[
     RowBox[{
      FormBox["k",
       TraditionalForm], "\[CenterDot]", 
      FormBox[
       RowBox[{"q", "(", "3", ")"}],
       TraditionalForm]}],
     TraditionalForm], " ", 
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "1", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "4", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "2", ")"}],
       TraditionalForm]}]]}], "-", 
   RowBox[{
    FractionBox["1", "6"], " ", 
    FormBox[
     RowBox[{
      FormBox["k",
       TraditionalForm], "\[CenterDot]", 
      FormBox[
       RowBox[{"q", "(", "3", ")"}],
       TraditionalForm]}],
     TraditionalForm], " ", 
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "2", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "1", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "4", ")"}],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    FractionBox["1", "6"], " ", 
    FormBox[
     RowBox[{
      FormBox["k",
       TraditionalForm], "\[CenterDot]", 
      FormBox[
       RowBox[{"q", "(", "3", ")"}],
       TraditionalForm]}],
     TraditionalForm], " ", 
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "2", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "4", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "1", ")"}],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    FractionBox["1", "6"], " ", 
    FormBox[
     RowBox[{
      FormBox["k",
       TraditionalForm], "\[CenterDot]", 
      FormBox[
       RowBox[{"q", "(", "3", ")"}],
       TraditionalForm]}],
     TraditionalForm], " ", 
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "4", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "1", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "2", ")"}],
       TraditionalForm]}]]}], "-", 
   RowBox[{
    FractionBox["1", "6"], " ", 
    FormBox[
     RowBox[{
      FormBox["k",
       TraditionalForm], "\[CenterDot]", 
      FormBox[
       RowBox[{"q", "(", "3", ")"}],
       TraditionalForm]}],
     TraditionalForm], " ", 
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "4", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "2", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "1", ")"}],
       TraditionalForm]}]]}], "-", 
   RowBox[{
    FractionBox["1", "6"], " ", 
    FormBox[
     RowBox[{
      FormBox["k",
       TraditionalForm], "\[CenterDot]", 
      FormBox[
       RowBox[{"q", "(", "4", ")"}],
       TraditionalForm]}],
     TraditionalForm], " ", 
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "1", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "2", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "3", ")"}],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    FractionBox["1", "6"], " ", 
    FormBox[
     RowBox[{
      FormBox["k",
       TraditionalForm], "\[CenterDot]", 
      FormBox[
       RowBox[{"q", "(", "4", ")"}],
       TraditionalForm]}],
     TraditionalForm], " ", 
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "1", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "3", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "2", ")"}],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    FractionBox["1", "6"], " ", 
    FormBox[
     RowBox[{
      FormBox["k",
       TraditionalForm], "\[CenterDot]", 
      FormBox[
       RowBox[{"q", "(", "4", ")"}],
       TraditionalForm]}],
     TraditionalForm], " ", 
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "2", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "1", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "3", ")"}],
       TraditionalForm]}]]}], "-", 
   RowBox[{
    FractionBox["1", "6"], " ", 
    FormBox[
     RowBox[{
      FormBox["k",
       TraditionalForm], "\[CenterDot]", 
      FormBox[
       RowBox[{"q", "(", "4", ")"}],
       TraditionalForm]}],
     TraditionalForm], " ", 
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "2", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "3", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "1", ")"}],
       TraditionalForm]}]]}], "-", 
   RowBox[{
    FractionBox["1", "6"], " ", 
    FormBox[
     RowBox[{
      FormBox["k",
       TraditionalForm], "\[CenterDot]", 
      FormBox[
       RowBox[{"q", "(", "4", ")"}],
       TraditionalForm]}],
     TraditionalForm], " ", 
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "3", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "1", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "2", ")"}],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    FractionBox["1", "6"], " ", 
    FormBox[
     RowBox[{
      FormBox["k",
       TraditionalForm], "\[CenterDot]", 
      FormBox[
       RowBox[{"q", "(", "4", ")"}],
       TraditionalForm]}],
     TraditionalForm], " ", 
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "3", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "2", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "1", ")"}],
       TraditionalForm]}]]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970371116538*^9},
 CellTags->"Schouten",
 CellLabel->"Out[1410]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t", "//", "Schouten"}]], "Input",
 CellTags->"Schouten",
 CellLabel->"In[1411]:="],

Cell[BoxData[
 FormBox["0", TraditionalForm]], "Output",
 CellChangeTimes->{3.500970371601383*^9},
 CellTags->"Schouten",
 CellLabel->"Out[1411]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"Clear", "[", "t", "]"}]], "Input",
 CellLabel->"In[1412]:="]
}, Open  ]],

Cell["SD", "ObjectName",
 CellTags->"SD"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SD"],

Cell[TextData[{
 "SD[i, j] is the (FeynCalc-external) Kronecker-delta for SU(",
 StyleBox["N",
  FontSlant->"Italic"],
 ") with color indices i and j. SD[i,j] is transformed into \
SUNDelta[SUNIndex[i],SUNIndex[j]] by FeynCalcInternal."
}], "Text",
 CellTags->"SD"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["SUNDelta",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SUNDelta",
  ButtonNote->"SUNDelta"],
 "."
}], "Text",
 CellTags->"SD"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"SD"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SD", "[", 
  RowBox[{"a", ",", "b"}], "]"}]], "Input",
 CellTags->"SD",
 CellLabel->"In[1413]:="],

Cell[BoxData[
 FormBox[
  SubscriptBox["\[Delta]", 
   RowBox[{"a", "b"}]], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009703717328777`*^9},
 CellTags->"SD",
 CellLabel->"Out[1413]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"%", "//", "FCI"}], "//", "StandardForm"}]], "Input",
 CellTags->"SD",
 CellLabel->"In[1414]:="],

Cell[BoxData[
 RowBox[{"SUNDelta", "[", 
  RowBox[{
   RowBox[{"SUNIndex", "[", "a", "]"}], ",", 
   RowBox[{"SUNIndex", "[", "b", "]"}]}], "]"}]], "Output",
 CellChangeTimes->{3.500970371963155*^9},
 CellTags->"SD",
 CellLabel->"Out[1414]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"%", "//", "FCE"}], "//", "StandardForm"}]], "Input",
 CellTags->"SD",
 CellLabel->"In[1415]:="],

Cell[BoxData[
 RowBox[{"SD", "[", 
  RowBox[{"a", ",", "b"}], "]"}]], "Output",
 CellChangeTimes->{3.500970372245079*^9},
 CellTags->"SD",
 CellLabel->"Out[1415]//StandardForm="]
}, Open  ]]
}, Open  ]],

Cell["SelectFree", "ObjectName",
 CellTags->"SelectFree"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SelectFree"],

Cell["\<\
SelectFree[expr, a, b, ...] is equivalent to Select[expr, FreeQ2[#, {a,b, \
...}]&], except the special cases: SelectFree[a, b] returns a and \
SelectFree[a,a] returns 1 (where a is not a product or a sum).\
\>", "Text",
 CellTags->"SelectFree"],

Cell["SelectFree is equivalent to Select1.", "Text",
 CellTags->"SelectFree"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FreeQ2",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FreeQ2",
  ButtonNote->"FreeQ2"],
 ", ",
 ButtonBox["SelectNotFree",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SelectNotFree",
  ButtonNote->"SelectNotFree"],
 "."
}], "Text",
 CellTags->"SelectFree"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"SelectFree"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SelectFree", "[", 
  RowBox[{
   RowBox[{"a", "+", "b", "+", 
    RowBox[{"f", "[", "a", "]"}], "+", "d"}], ",", "a"}], "]"}]], "Input",
 CellTags->"SelectFree",
 CellLabel->"In[1416]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"b", "+", "d"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970372467226*^9},
 CellTags->"SelectFree",
 CellLabel->"Out[1416]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SelectFree", "[", 
  RowBox[{
   RowBox[{"x", " ", "y"}], ",", " ", "x"}], "]"}]], "Input",
 CellTags->"SelectFree",
 CellLabel->"In[1417]:="],

Cell[BoxData[
 FormBox["y", TraditionalForm]], "Output",
 CellChangeTimes->{3.5009703725136347`*^9},
 CellTags->"SelectFree",
 CellLabel->"Out[1417]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SelectFree", "[", 
  RowBox[{
   RowBox[{"2", " ", "x", " ", "y", " ", "z", " ", 
    RowBox[{"f", "[", "x", "]"}]}], ",", " ", 
   RowBox[{"{", 
    RowBox[{"x", ",", "y"}], "}"}]}], "]"}]], "Input",
 CellTags->"SelectFree",
 CellLabel->"In[1418]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"2", " ", "z"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.50097037256568*^9},
 CellTags->"SelectFree",
 CellLabel->"Out[1418]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SelectFree", "[", 
  RowBox[{"a", ",", "b"}], "]"}]], "Input",
 CellTags->"SelectFree",
 CellLabel->"In[1419]:="],

Cell[BoxData[
 FormBox["a", TraditionalForm]], "Output",
 CellChangeTimes->{3.500970372609707*^9},
 CellTags->"SelectFree",
 CellLabel->"Out[1419]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SelectFree", "[", 
  RowBox[{"a", ",", "a"}], "]"}]], "Input",
 CellTags->"SelectFree",
 CellLabel->"In[1420]:="],

Cell[BoxData[
 FormBox["1", TraditionalForm]], "Output",
 CellChangeTimes->{3.50097037267941*^9},
 CellTags->"SelectFree",
 CellLabel->"Out[1420]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SelectFree", "[", 
  RowBox[{"1", ",", "c"}], "]"}]], "Input",
 CellTags->"SelectFree",
 CellLabel->"In[1421]:="],

Cell[BoxData[
 FormBox["1", TraditionalForm]], "Output",
 CellChangeTimes->{3.5009703727251167`*^9},
 CellTags->"SelectFree",
 CellLabel->"Out[1421]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SelectFree", "[", 
  RowBox[{
   RowBox[{"f", "[", "x", "]"}], ",", "x"}], "]"}]], "Input",
 CellTags->"SelectFree",
 CellLabel->"In[1422]:="],

Cell[BoxData[
 FormBox["1", TraditionalForm]], "Output",
 CellChangeTimes->{3.5009703727699823`*^9},
 CellTags->"SelectFree",
 CellLabel->"Out[1422]="]
}, Open  ]]
}, Open  ]],

Cell["SelectGraphs", "ObjectName",
 CellTags->"SelectGraphs"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SelectGraphs"],

Cell["\<\
SelectGraphs is an option for OneLoopSum indicating that only a slected set \
of graphs of the list provided to OneLoopSum is to be calculated. Possible \
settings are: SelectGraphs\[RightArrow]{ i, j,  ... } or SelectGraphs\
\[RightArrow]{ a, {b, c}, ... } which indicates the graphs to be taken from \
the list provided to OneLoopSum. In the second setting the list {b, c} \
indicates that all amplitudes from b to c should be taken.\
\>", "Text",
 CellTags->"SelectGraphs"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["OneLoopSum",
  BaseStyle->"AddOnsLink",
  ButtonData:>"OneLoopSum",
  ButtonNote->"OneLoopSum"],
 "."
}], "Text",
 CellTags->"SelectGraphs"]
}, Open  ]],

Cell["SelectNotFree", "ObjectName",
 CellTags->"SelectNotFree"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SelectNotFree"],

Cell["\<\
SelectNotFree[expr, x] returns that part of expr which is not free of any \
occurance of x.\
\>", "Text",
 CellTags->"SelectNotFree"],

Cell["\<\
SelectNotFree[expr, a, b, ...] is equivalent to Select[expr, !FreeQ2[#, {a, \
b, ...}]&], except the special cases: SelectNotFree[a, b] returns 1 and \
SelectNotFree[a, a] returns a (where a is not a product or a sum).\
\>", "Text",
 CellTags->"SelectNotFree"],

Cell["SelectNotFree is equivalent to Select2.", "Text",
 CellTags->"SelectNotFree"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FreeQ2",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FreeQ2",
  ButtonNote->"FreeQ2"],
 ", ",
 ButtonBox["SelectFree",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SelectFree",
  ButtonNote->"SelectFree"],
 "."
}], "Text",
 CellTags->"SelectNotFree"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"SelectNotFree"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SelectNotFree", "[", 
  RowBox[{
   RowBox[{"a", "+", "b", "+", 
    RowBox[{"f", "[", "a", "]"}]}], ",", "a"}], "]"}]], "Input",
 CellTags->"SelectNotFree",
 CellLabel->"In[1423]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"f", "(", "a", ")"}], "+", "a"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009703729193583`*^9},
 CellTags->"SelectNotFree",
 CellLabel->"Out[1423]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SelectNotFree", "[", 
  RowBox[{
   RowBox[{"2", " ", "x", " ", "y", " ", 
    RowBox[{"f", "[", "x", "]"}], " ", "z"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "y"}], "}"}]}], "]"}]], "Input",
 CellTags->"SelectNotFree",
 CellLabel->"In[1424]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"x", " ", "y", " ", 
   RowBox[{"f", "(", "x", ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970372976244*^9},
 CellTags->"SelectNotFree",
 CellLabel->"Out[1424]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SelectNotFree", "[", 
  RowBox[{"a", ",", "b"}], "]"}]], "Input",
 CellTags->"SelectNotFree",
 CellLabel->"In[1425]:="],

Cell[BoxData[
 FormBox["1", TraditionalForm]], "Output",
 CellChangeTimes->{3.5009703730234137`*^9},
 CellTags->"SelectNotFree",
 CellLabel->"Out[1425]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SelectNotFree", "[", 
  RowBox[{
   RowBox[{"a", "+", "x"}], ",", "b"}], "]"}]], "Input",
 CellTags->"SelectNotFree",
 CellLabel->"In[1426]:="],

Cell[BoxData[
 FormBox["0", TraditionalForm]], "Output",
 CellChangeTimes->{3.5009703731049747`*^9},
 CellTags->"SelectNotFree",
 CellLabel->"Out[1426]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SelectNotFree", "[", 
  RowBox[{"a", ",", "a"}], "]"}]], "Input",
 CellTags->"SelectNotFree",
 CellLabel->"In[1427]:="],

Cell[BoxData[
 FormBox["a", TraditionalForm]], "Output",
 CellChangeTimes->{3.500970373172757*^9},
 CellTags->"SelectNotFree",
 CellLabel->"Out[1427]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SelectNotFree", "[", 
  RowBox[{"1", ",", "c"}], "]"}]], "Input",
 CellTags->"SelectNotFree",
 CellLabel->"In[1428]:="],

Cell[BoxData[
 FormBox["1", TraditionalForm]], "Output",
 CellChangeTimes->{3.500970373248744*^9},
 CellTags->"SelectNotFree",
 CellLabel->"Out[1428]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SelectNotFree", "[", 
  RowBox[{
   RowBox[{"f", "[", "x", "]"}], ",", "x"}], "]"}]], "Input",
 CellTags->"SelectNotFree",
 CellLabel->"In[1429]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"f", "(", "x", ")"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970373383452*^9},
 CellTags->"SelectNotFree",
 CellLabel->"Out[1429]="]
}, Open  ]]
}, Open  ]],

Cell["SelectSplit", "ObjectName",
 CellTags->"SelectSplit"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SelectSplit"],

Cell["\<\
SelectSplit[l, p] Construct list of mutually exclusive subsets from l in \
which every element li satisfies a criterium pj[li] with pj from p and \
appends the subset of remaining unmatched elements.\
\>", "Text",
 CellTags->"SelectSplit"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"SelectSplit"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SelectSplit", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     SuperscriptBox["a", "2"], ",", 
     SuperscriptBox["b", "3"], ",", 
     SuperscriptBox["c", "4"], ",", 
     SuperscriptBox["d", "5"], ",", 
     SuperscriptBox["e", "6"], ",", 
     RowBox[{"f", "+", "g"}], ",", 
     SuperscriptBox["h", "4"]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"MatchQ", "[", 
       RowBox[{"#", ",", 
        RowBox[{"_", "^", "2"}]}], "]"}], "&"}], ",", 
     RowBox[{
      RowBox[{"MatchQ", "[", 
       RowBox[{"#", ",", 
        RowBox[{"_", "^", "4"}]}], "]"}], "&"}], ",", 
     RowBox[{
      RowBox[{"FreeQ", "[", 
       RowBox[{"#", ",", "Power"}], "]"}], "&"}]}], "}"}]}], "]"}]], "Input",
 CellTags->"SelectSplit",
 CellLabel->"In[1430]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"{", 
     SuperscriptBox["a", "2"], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{
      SuperscriptBox["c", "4"], ",", 
      SuperscriptBox["h", "4"]}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"f", "+", "g"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{
      SuperscriptBox["b", "3"], ",", 
      SuperscriptBox["d", "5"], ",", 
      SuperscriptBox["e", "6"]}], "}"}]}], "}"}], TraditionalForm]], "Output",\

 CellChangeTimes->{3.500970373468548*^9},
 CellTags->"SelectSplit",
 CellLabel->"Out[1430]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SelectSplit", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     SuperscriptBox["a", "2"], ",", 
     SuperscriptBox["b", "3"], ",", 
     SuperscriptBox["c", "4"], ",", 
     SuperscriptBox["d", "5"], ",", 
     SuperscriptBox["e", "6"], ",", 
     RowBox[{"f", "+", "g"}], ",", 
     SuperscriptBox["h", "4"]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"FreeQ", "[", 
       RowBox[{"#", ",", "Plus"}], "]"}], "&"}], ",", 
     RowBox[{
      RowBox[{"FreeQ", "[", 
       RowBox[{"#", ",", "Power"}], "]"}], "&"}]}], "}"}]}], "]"}]], "Input",
 CellTags->"SelectSplit",
 CellLabel->"In[1431]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      SuperscriptBox["a", "2"], ",", 
      SuperscriptBox["b", "3"], ",", 
      SuperscriptBox["c", "4"], ",", 
      SuperscriptBox["d", "5"], ",", 
      SuperscriptBox["e", "6"], ",", 
      SuperscriptBox["h", "4"]}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"f", "+", "g"}], "}"}], ",", 
    RowBox[{"{", "}"}]}], "}"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970373549938*^9},
 CellTags->"SelectSplit",
 CellLabel->"Out[1431]="]
}, Open  ]]
}, Open  ]],

Cell["Select1", "ObjectName",
 CellTags->"Select1"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Select1"],

Cell["\<\
Select1[expr, a, b, ...] is equivalent to Select[expr, FreeQ2[#, {a,b, \
...}]&], except the special cases: Select1[a, b] returns a and Select1[a,a] \
returns 1 (where a is not a product or a sum).\
\>", "Text",
 CellTags->"Select1"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Select2",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Select2",
  ButtonNote->"Select2"],
 "."
}], "Text",
 CellTags->"Select1"]
}, Open  ]],

Cell["Select2", "ObjectName",
 CellTags->"Select2"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Select2"],

Cell["\<\
Select2[expr, a, b, ...] is equivalent to Select[expr, !FreeQ2[#, {a,b, \
...}]&], except the special cases: Select2[a, b] returns 1 and  Select2[a,a] \
returns a (where a is not a product or a sum).\
\>", "Text",
 CellTags->"Select2"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Select1",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Select1",
  ButtonNote->"Select1"],
 "."
}], "Text",
 CellTags->"Select2"]
}, Open  ]],

Cell["Series2", "ObjectName",
 CellTags->"Series2"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Series2"],

Cell[TextData[{
 " Series2 performs a series expansion around 0. Series2 is (up to the \
Gamma-bug in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " versions smaller than 5.0) equivalent to Series, except that it applies \
Normal on the result and has an option FinalSubstitutions. Series2[f, e, n] \
is equivalent to Series2[f, {e, 0, n}]."
}], "Text",
 CellTags->"Series2"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "Series2", "]"}]], "Input",
 CellTags->"Series2",
 CellLabel->"In[1432]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Collecting", "\[Rule]", "False"}], ",", 
    RowBox[{"Factoring", "\[Rule]", "True"}], ",", 
    RowBox[{"FinalSubstitutions", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{
       TagBox["\[DoubledGamma]",
        Function[{}, EulerGamma]], "\[Rule]", "0"}], "}"}]}], ",", 
    RowBox[{"Print", "\[Rule]", "False"}], ",", 
    RowBox[{"SimplifyPolyLog", "\[Rule]", "True"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970374155169*^9},
 CellTags->"Series2",
 CellLabel->"Out[1432]="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Series2"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Series2", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{"x", " ", 
      RowBox[{"(", 
       RowBox[{"1", "-", "x"}], ")"}]}], ")"}], "^", 
    RowBox[{"(", 
     RowBox[{"\[Delta]", "/", "2"}], ")"}]}], ",", "\[Delta]", ",", "1"}], 
  "]"}]], "Input",
 CellTags->"Series2",
 CellLabel->"In[1433]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    FractionBox["1", "2"], " ", "\[Delta]", " ", 
    RowBox[{"log", "(", 
     RowBox[{"1", "-", "x"}], ")"}]}], "+", 
   RowBox[{
    FractionBox["1", "2"], " ", "\[Delta]", " ", 
    RowBox[{"log", "(", "x", ")"}]}], "+", "1"}], TraditionalForm]], "Output",\

 CellChangeTimes->{3.500970374315703*^9},
 CellTags->"Series2",
 CellLabel->"Out[1433]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Series2", "[", 
  RowBox[{
   RowBox[{"Gamma", "[", "x", "]"}], ",", "x", ",", "1"}], "]"}]], "Input",
 CellTags->"Series2",
 CellLabel->"In[1434]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    FractionBox["1", "2"], " ", 
    RowBox[{"\[Zeta]", "(", "2", ")"}], " ", "x"}], "+", 
   FractionBox["1", "x"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.50097037495117*^9},
 CellTags->"Series2",
 CellLabel->"Out[1434]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Series", "[", 
  RowBox[{
   RowBox[{"Gamma", "[", "x", "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellTags->"Series2",
 CellLabel->"In[1435]:="],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{
    FractionBox["1", "x"], "-", 
    TagBox["\[DoubledGamma]",
     Function[{}, EulerGamma]], "+", 
    RowBox[{
     FractionBox["1", "12"], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"6", " ", 
        SuperscriptBox[
         TagBox["\[DoubledGamma]",
          Function[{}, EulerGamma]], "2"]}], "+", 
       SuperscriptBox["\[Pi]", "2"]}], ")"}], " ", "x"}], "+", 
    InterpretationBox[
     RowBox[{"O", "(", 
      SuperscriptBox["x", "2"], ")"}],
     SeriesData[$CellContext`x, 0, {}, -1, 2, 1],
     Editable->False]}],
   SeriesData[$CellContext`x, 0, {
    1, -EulerGamma, Rational[1, 12] (6 EulerGamma^2 + Pi^2)}, -1, 2, 1],
   Editable->False], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970375397635*^9},
 CellTags->"Series2",
 CellLabel->"Out[1435]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Series2", "[", 
  RowBox[{
   RowBox[{"Gamma", "[", "x", "]"}], ",", "x", ",", "2"}], "]"}]], "Input",
 CellTags->"Series2",
 CellLabel->"In[1436]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    FractionBox[
     RowBox[{
      SuperscriptBox["x", "2"], " ", 
      TemplateBox[{"3"},
       "Zeta"]}], "3"]}], "+", 
   RowBox[{
    FractionBox["1", "2"], " ", 
    RowBox[{"\[Zeta]", "(", "2", ")"}], " ", "x"}], "+", 
   FractionBox["1", "x"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970376201406*^9},
 CellTags->"Series2",
 CellLabel->"Out[1436]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Series2", "[", 
   RowBox[{
    RowBox[{"Gamma", "[", "x", "]"}], ",", "x", ",", "2", ",", 
    RowBox[{"FinalSubstitutions", "\[Rule]", 
     RowBox[{"{", "}"}]}]}], "]"}], "//", "FullSimplify"}]], "Input",
 CellTags->"Series2",
 CellLabel->"In[1437]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["1", "6"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      RowBox[{"3", " ", 
       TagBox["\[DoubledGamma]",
        Function[{}, EulerGamma]], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
          SuperscriptBox["x", "2"]}], "+", "2"}], ")"}]}]}], "-", 
     RowBox[{"2", " ", 
      SuperscriptBox["x", "2"], " ", 
      TemplateBox[{"3"},
       "Zeta"]}], "-", 
     RowBox[{
      SuperscriptBox[
       TagBox["\[DoubledGamma]",
        Function[{}, EulerGamma]], "3"], " ", 
      SuperscriptBox["x", "2"]}], "+", 
     RowBox[{"3", " ", 
      RowBox[{"\[Zeta]", "(", "2", ")"}], " ", "x"}], "+", 
     RowBox[{"3", " ", 
      SuperscriptBox[
       TagBox["\[DoubledGamma]",
        Function[{}, EulerGamma]], "2"], " ", "x"}], "+", 
     FractionBox["6", "x"]}], ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970376649684*^9},
 CellTags->"Series2",
 CellLabel->"Out[1437]="]
}, Open  ]],

Cell[TextData[{
 "In earlier ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " versions the Gamma functions was not expanded far enough, in 5.0 this got \
finally fixed:"
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Series", "[", 
     RowBox[{
      RowBox[{"Gamma", "[", "x", "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"x", ",", "0", ",", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{"$VersionNumber", "<", "5"}], ",", "4", ",", "2"}], "]"}]}],
        "}"}]}], "]"}], "//", "Normal"}], "//", "Expand"}], "//", 
  "FullSimplify"}]], "Input",
 CellTags->"Series2",
 CellLabel->"In[1438]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["1", "12"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      RowBox[{"2", " ", 
       SuperscriptBox[
        TagBox["\[DoubledGamma]",
         Function[{}, EulerGamma]], "3"], " ", 
       SuperscriptBox["x", "2"]}]}], "-", 
     RowBox[{
      TagBox["\[DoubledGamma]",
       Function[{}, EulerGamma]], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         SuperscriptBox["\[Pi]", "2"], " ", 
         SuperscriptBox["x", "2"]}], "+", "12"}], ")"}]}], "+", 
     RowBox[{"x", " ", 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox["\[Pi]", "2"], "-", 
        RowBox[{"4", " ", "x", " ", 
         TemplateBox[{"3"},
          "Zeta"]}]}], ")"}]}], "+", 
     RowBox[{"6", " ", 
      SuperscriptBox[
       TagBox["\[DoubledGamma]",
        Function[{}, EulerGamma]], "2"], " ", "x"}], "+", 
     FractionBox["12", "x"]}], ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970377956627*^9},
 CellTags->"Series2",
 CellLabel->"Out[1438]="]
}, Open  ]],

Cell["\<\
There is a table of expansions of special hypergeometric functions.\
\>", "Text",
 CellTags->"Series2"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Series2", "[", 
  RowBox[{
   RowBox[{"HypergeometricPFQ", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"1", ",", 
       RowBox[{"OPEm", "-", "1"}], ",", 
       RowBox[{
        RowBox[{"Epsilon", "/", "2"}], "+", "OPEm"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"OPEm", ",", 
       RowBox[{"OPEm", "+", "Epsilon"}]}], "}"}], ",", "1"}], "]"}], ",", 
   "Epsilon", ",", "1"}], "]"}]], "Input",
 CellTags->"Series2",
 CellLabel->"In[1439]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    FractionBox["2", 
     TagBox["\[CurlyEpsilon]",
      TraditionalForm]]}], "+", 
   FractionBox[
    RowBox[{"2", " ", "m"}], 
    TagBox["\[CurlyEpsilon]",
     TraditionalForm]], "+", 
   RowBox[{
    FractionBox["1", "2"], " ", 
    TagBox["\[CurlyEpsilon]",
     TraditionalForm], " ", "m", " ", 
    TemplateBox[{"1","m"},
     "PolyGamma2"]}], "-", 
   FractionBox[
    RowBox[{
     TagBox["\[CurlyEpsilon]",
      TraditionalForm], " ", 
     TemplateBox[{"1","m"},
      "PolyGamma2"]}], "2"], "+", "1"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970378497459*^9},
 CellTags->"Series2",
 CellLabel->"Out[1439]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{" ", 
  RowBox[{"Series2", "[", 
   RowBox[{
    RowBox[{"HypergeometricPFQ", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"1", ",", " ", "OPEm", ",", " ", 
        RowBox[{
         RowBox[{"Epsilon", "/", "2"}], " ", "+", " ", "OPEm"}]}], "}"}], ",",
       " ", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"1", " ", "+", " ", "OPEm"}], ",", " ", 
        RowBox[{"Epsilon", " ", "+", " ", "OPEm"}]}], "}"}], ",", "  ", "1"}],
      "]"}], ",", "Epsilon", ",", "1"}], "]"}]}]], "Input",
 CellTags->"Series2",
 CellLabel->"In[1440]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    FractionBox["1", "4"], " ", 
    TagBox["\[CurlyEpsilon]",
     TraditionalForm], " ", 
    RowBox[{"\[Zeta]", "(", "2", ")"}], " ", "m"}], "+", 
   FractionBox[
    RowBox[{"2", " ", "m"}], 
    TagBox["\[CurlyEpsilon]",
     TraditionalForm]], "+", 
   RowBox[{
    FractionBox["1", "4"], " ", 
    TagBox["\[CurlyEpsilon]",
     TraditionalForm], " ", "m", " ", 
    SuperscriptBox[
     TemplateBox[{"0","m"},
      "PolyGamma2"], "2"]}], "+", 
   RowBox[{
    FractionBox["3", "4"], " ", 
    TagBox["\[CurlyEpsilon]",
     TraditionalForm], " ", "m", " ", 
    TemplateBox[{"1","m"},
     "PolyGamma2"]}], "-", 
   RowBox[{
    FractionBox["1", "2"], " ", 
    TagBox["\[CurlyEpsilon]",
     TraditionalForm], " ", "m", " ", 
    RowBox[{
     SubscriptBox["S", "11"], "(", 
     FormBox[
      RowBox[{"m", "-", "1"}],
      TraditionalForm], ")"}]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970378870338*^9},
 CellTags->"Series2",
 CellLabel->"Out[1440]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Hypergeometric2F1", "[", 
  RowBox[{"1", ",", " ", "Epsilon", ",", " ", 
   RowBox[{"1", " ", "+", " ", 
    RowBox[{"2", " ", "Epsilon"}]}], ",", "x"}], "]"}]], "Input",
 CellTags->"Series2",
 CellLabel->"In[1441]:="],

Cell[BoxData[
 FormBox[
  TemplateBox[{"1",TagBox["\[CurlyEpsilon]", TraditionalForm],RowBox[{
      RowBox[{"2", " ", 
        TagBox["\[CurlyEpsilon]", TraditionalForm]}], "+", "1"}],"x"},
   "Hypergeometric2F1"], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009703794065437`*^9},
 CellTags->"Series2",
 CellLabel->"Out[1441]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Series2", "[", 
  RowBox[{"%", ",", "Epsilon", ",", "3"}], "]"}]], "Input",
 CellTags->"Series2",
 CellLabel->"In[1442]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    RowBox[{"2", " ", 
     SuperscriptBox[
      TagBox["\[CurlyEpsilon]",
       TraditionalForm], "2"], " ", 
     RowBox[{"\[Zeta]", "(", "2", ")"}]}]}], "+", 
   RowBox[{"2", " ", 
    SuperscriptBox[
     TagBox["\[CurlyEpsilon]",
      TraditionalForm], "3"], " ", 
    TemplateBox[{"3",RowBox[{"1", "-", "x"}]},
     "PolyLog"]}], "+", 
   RowBox[{"2", " ", 
    SuperscriptBox[
     TagBox["\[CurlyEpsilon]",
      TraditionalForm], "2"], " ", 
    TemplateBox[{"2",RowBox[{"1", "-", "x"}]},
     "PolyLog"]}], "-", 
   RowBox[{"4", " ", 
    SuperscriptBox[
     TagBox["\[CurlyEpsilon]",
      TraditionalForm], "3"], " ", 
    TemplateBox[{"2",RowBox[{"1", "-", "x"}]},
     "PolyLog"], " ", 
    RowBox[{"log", "(", "x", ")"}]}], "-", 
   RowBox[{"4", " ", 
    SuperscriptBox[
     TagBox["\[CurlyEpsilon]",
      TraditionalForm], "3"], " ", 
    RowBox[{
     SubscriptBox["S", 
      RowBox[{
       FormBox["1",
        TraditionalForm], 
       FormBox["2",
        TraditionalForm]}]], "(", 
     FormBox[
      RowBox[{"1", "-", "x"}],
      TraditionalForm], ")"}]}], "-", 
   RowBox[{"2", " ", 
    SuperscriptBox[
     TagBox["\[CurlyEpsilon]",
      TraditionalForm], "3"], " ", 
    RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
    RowBox[{"log", "(", 
     RowBox[{"1", "-", "x"}], ")"}]}], "+", 
   RowBox[{"4", " ", 
    SuperscriptBox[
     TagBox["\[CurlyEpsilon]",
      TraditionalForm], "3"], " ", 
    RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
    RowBox[{"log", "(", "x", ")"}]}], "-", 
   RowBox[{
    FractionBox["1", "6"], " ", 
    SuperscriptBox[
     TagBox["\[CurlyEpsilon]",
      TraditionalForm], "3"], " ", 
    RowBox[{
     SuperscriptBox["log", "3"], "(", 
     RowBox[{"1", "-", "x"}], ")"}]}], "-", 
   RowBox[{"2", " ", 
    SuperscriptBox[
     TagBox["\[CurlyEpsilon]",
      TraditionalForm], "3"], " ", 
    RowBox[{"log", "(", 
     RowBox[{"1", "-", "x"}], ")"}], " ", 
    RowBox[{
     SuperscriptBox["log", "2"], "(", "x", ")"}]}], "+", 
   RowBox[{
    SuperscriptBox[
     TagBox["\[CurlyEpsilon]",
      TraditionalForm], "3"], " ", 
    RowBox[{
     SuperscriptBox["log", "2"], "(", 
     RowBox[{"1", "-", "x"}], ")"}], " ", 
    RowBox[{"log", "(", "x", ")"}]}], "-", 
   RowBox[{
    FractionBox["1", "2"], " ", 
    SuperscriptBox[
     TagBox["\[CurlyEpsilon]",
      TraditionalForm], "2"], " ", 
    RowBox[{
     SuperscriptBox["log", "2"], "(", 
     RowBox[{"1", "-", "x"}], ")"}]}], "+", 
   RowBox[{"2", " ", 
    SuperscriptBox[
     TagBox["\[CurlyEpsilon]",
      TraditionalForm], "2"], " ", 
    RowBox[{"log", "(", 
     RowBox[{"1", "-", "x"}], ")"}], " ", 
    RowBox[{"log", "(", "x", ")"}]}], "-", 
   RowBox[{
    TagBox["\[CurlyEpsilon]",
     TraditionalForm], " ", 
    RowBox[{"log", "(", 
     RowBox[{"1", "-", "x"}], ")"}]}], "+", 
   RowBox[{"2", " ", 
    SuperscriptBox[
     TagBox["\[CurlyEpsilon]",
      TraditionalForm], "3"], " ", 
    TemplateBox[{"3"},
     "Zeta"]}], "+", "1"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970379852762*^9},
 CellTags->"Series2",
 CellLabel->"Out[1442]="]
}, Open  ]],

Cell[TextData[{
 "There are over 100 more special expansions of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[ThinSpace]", 
    RowBox[{
     SubscriptBox["", "2"], 
     SubscriptBox["F", "1"]}]}], TraditionalForm]]],
 " tabulated in Series2.m. The interested user can consult the source code \
(search for HYPERLIST)."
}], "Text",
 CellTags->"Series2"]
}, Open  ]],

Cell["Series3", "ObjectName",
 CellTags->"Series3"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Series3"],

Cell["\<\
Series3 performs a series expansion around 0. Series3 is equivalent to \
Series, except that it applies Normal on the result and that some Series bugs \
are fixed. Series3[f, e, n] is equivalent to  Series3[f, {e, 0, n}].\
\>", "Text",
 CellTags->"Series3"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "Series3", "]"}]], "Input",
 CellTags->"Series3",
 CellLabel->"In[1443]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Factoring", "\[Rule]", "True"}], ",", 
    RowBox[{"FinalSubstitutions", "\[Rule]", 
     RowBox[{"{", "}"}]}]}], "}"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009703800411177`*^9},
 CellTags->"Series3",
 CellLabel->"Out[1443]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Series2",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Series2",
  ButtonNote->"Series2"],
 "."
}], "Text",
 CellTags->"Series3"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Series3"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Series3", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{"x", " ", 
      RowBox[{"(", 
       RowBox[{"1", "-", "x"}], ")"}]}], ")"}], "^", 
    RowBox[{"(", 
     RowBox[{"\[Delta]", "/", "2"}], ")"}]}], ",", "\[Delta]", ",", "1"}], 
  "]"}]], "Input",
 CellTags->"Series3",
 CellLabel->"In[1444]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    FractionBox["1", "2"], " ", "\[Delta]", " ", 
    RowBox[{"log", "(", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"1", "-", "x"}], ")"}], " ", "x"}], ")"}]}], "+", "1"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970380128539*^9},
 CellTags->"Series3",
 CellLabel->"Out[1444]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Series3", "[", 
   RowBox[{
    RowBox[{"Gamma", "[", "x", "]"}], ",", "x", ",", "1"}], "]"}], "//", 
  "FullSimplify"}]], "Input",
 CellTags->"Series3",
 CellLabel->"In[1445]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["1", "x"], "-", 
   TagBox["\[DoubledGamma]",
    Function[{}, EulerGamma]], "+", "1"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970380317206*^9},
 CellTags->"Series3",
 CellLabel->"Out[1445]="]
}, Open  ]]
}, Open  ]],

Cell["SetMandelstam", "ObjectName",
 CellTags->"SetMandelstam"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SetMandelstam"],

Cell["\<\
SetMandelstam[s, t, u, p1, p2, p3, p4, m1, m2, m3, m4] defines the Mandelstam \
variables  s=(p1+p2)^2, t=(p1+p3)^2, u=(p1+p4)^2 and sets the pi on-shell: \
p1^2=m1^2, p2^2=m2^2, p3^2=m3^2, p4^2=m4^2. Note that p1 +  p2 + p3 + p4 = 0 \
is assumed.
SetMandelstam[x, {p1, p2, p3, p4, p5}, {m1, m2, m3, m4, m5}] defines x[i, j] \
= (pi+pj)^2 and sets the pi on-shell. The pi satisfy: p1 + p2 + p3 + p4 + p5 \
= 0.\
\>", "Text",
 CellTags->"SetMandelstam"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "SetMandelstam", "]"}]], "Input",
 CellTags->"SetMandelstam",
 CellLabel->"In[1446]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"Dimension", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"4", ",", "D", ",", "___"}], "}"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.5009703804057198`*^9},
 CellTags->"SetMandelstam",
 CellLabel->"Out[1446]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Mandelstam",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Mandelstam",
  ButtonNote->"Mandelstam"],
 "."
}], "Text",
 CellTags->"SetMandelstam"]
}, Open  ]],

Cell["SimplifyDeltaFunction", "ObjectName",
 CellTags->"SimplifyDeltaFunction"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SimplifyDeltaFunction"],

Cell["\<\
SimplifyDeltaFunction[exp, x] simplifies f[x]*DeltaFunction[1-x] \
toLimit[f[x],x\[Rule]1] DeltaFunction[1-x] and applies a list of \
transformation rules for DeltaFunctionPrime[1-x]*x^(OPEm-1)*f[x] where \
x^(OPEm-1) is suppressed in exp.\
\>", "Text",
 CellTags->"SimplifyDeltaFunction"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["DeltaFunction",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DeltaFunction",
  ButtonNote->"DeltaFunction"],
 ", ",
 ButtonBox["DeltaFunctionPrime",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DeltaFunctionPrime",
  ButtonNote->"DeltaFunctionPrime"],
 "."
}], "Text",
 CellTags->"SimplifyDeltaFunction"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"SimplifyDeltaFunction"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"g", "[", "x", "]"}], " ", 
  RowBox[{"DeltaFunction", "[", 
   RowBox[{"1", "-", "x"}], "]"}]}]], "Input",
 CellTags->"SimplifyDeltaFunction",
 CellLabel->"In[1447]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"g", "(", "x", ")"}], " ", 
   RowBox[{"\[Delta]", "(", 
    FormBox[
     RowBox[{"1", "-", "x"}],
     TraditionalForm], ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970380525051*^9},
 CellTags->"SimplifyDeltaFunction",
 CellLabel->"Out[1447]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SimplifyDeltaFunction", "[", " ", 
  RowBox[{"%", ",", "x"}], "]"}]], "Input",
 CellTags->"SimplifyDeltaFunction",
 CellLabel->"In[1448]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"\[Delta]", "(", 
    FormBox[
     RowBox[{"1", "-", "x"}],
     TraditionalForm], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{
     UnderscriptBox["lim", 
      RowBox[{"x", "\[Rule]", "1"}]], "\[ThinSpace]", 
     RowBox[{"g", "(", "x", ")"}]}], ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970381144664*^9},
 CellTags->"SimplifyDeltaFunction",
 CellLabel->"Out[1448]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"g", "[", "x", "]"}], 
  RowBox[{"DeltaFunctionPrime", "[", 
   RowBox[{"1", "-", "x"}], "]"}]}]], "Input",
 CellTags->"SimplifyDeltaFunction",
 CellLabel->"In[1449]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"g", "(", "x", ")"}], " ", 
   RowBox[{
    SuperscriptBox["\[Delta]", "\[Prime]"], "(", 
    FormBox[
     RowBox[{"1", "-", "x"}],
     TraditionalForm], ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009703812363653`*^9},
 CellTags->"SimplifyDeltaFunction",
 CellLabel->"Out[1449]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SimplifyDeltaFunction", "[", " ", 
  RowBox[{"%", ",", "x"}], "]"}]], "Input",
 CellTags->"SimplifyDeltaFunction",
 CellLabel->"In[1450]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    RowBox[{"\[Delta]", "(", 
     FormBox[
      RowBox[{"1", "-", "x"}],
      TraditionalForm], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      UnderscriptBox["lim", 
       RowBox[{"x", "\[Rule]", "1"}]], "\[ThinSpace]", 
      RowBox[{
       SuperscriptBox["g", "\[Prime]",
        MultilineFunction->None], "(", "x", ")"}]}], ")"}]}], "+", 
   RowBox[{
    RowBox[{
     SuperscriptBox["\[Delta]", "\[Prime]"], "(", 
     FormBox[
      RowBox[{"1", "-", "x"}],
      TraditionalForm], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      UnderscriptBox["lim", 
       RowBox[{"x", "\[Rule]", "1"}]], "\[ThinSpace]", 
      RowBox[{"g", "(", "x", ")"}]}], ")"}]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970381378201*^9},
 CellTags->"SimplifyDeltaFunction",
 CellLabel->"Out[1450]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"x", " ", 
  RowBox[{"Log", "[", "x", "]"}], " ", 
  RowBox[{"DeltaFunctionPrime", "[", 
   RowBox[{"1", "-", "x"}], "]"}]}]], "Input",
 CellTags->"SimplifyDeltaFunction",
 CellLabel->"In[1451]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"x", " ", 
   RowBox[{"log", "(", "x", ")"}], " ", 
   RowBox[{
    SuperscriptBox["\[Delta]", "\[Prime]"], "(", 
    FormBox[
     RowBox[{"1", "-", "x"}],
     TraditionalForm], ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970381537881*^9},
 CellTags->"SimplifyDeltaFunction",
 CellLabel->"Out[1451]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SimplifyDeltaFunction", "[", " ", 
  RowBox[{"%", ",", "x"}], "]"}]], "Input",
 CellTags->"SimplifyDeltaFunction",
 CellLabel->"In[1452]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"\[Delta]", "(", 
   FormBox[
    RowBox[{"1", "-", "x"}],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970382015794*^9},
 CellTags->"SimplifyDeltaFunction",
 CellLabel->"Out[1452]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"PolyLog", "[", 
   RowBox[{"2", ",", 
    RowBox[{"1", "-", "x"}]}], "]"}], " ", 
  RowBox[{"DeltaFunctionPrime", "[", 
   RowBox[{"1", "-", "x"}], "]"}]}]], "Input",
 CellTags->"SimplifyDeltaFunction",
 CellLabel->"In[1453]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TemplateBox[{"2",RowBox[{"1", "-", "x"}]},
    "PolyLog"], " ", 
   RowBox[{
    SuperscriptBox["\[Delta]", "\[Prime]"], "(", 
    FormBox[
     RowBox[{"1", "-", "x"}],
     TraditionalForm], ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970382174718*^9},
 CellTags->"SimplifyDeltaFunction",
 CellLabel->"Out[1453]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SimplifyDeltaFunction", "[", " ", 
  RowBox[{"%", ",", "x"}], "]"}]], "Input",
 CellTags->"SimplifyDeltaFunction",
 CellLabel->"In[1454]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   RowBox[{"\[Delta]", "(", 
    FormBox[
     RowBox[{"1", "-", "x"}],
     TraditionalForm], ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970382553945*^9},
 CellTags->"SimplifyDeltaFunction",
 CellLabel->"Out[1454]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Log", "[", "x", "]"}], 
  RowBox[{"PolyLog", "[", 
   RowBox[{"2", ",", 
    RowBox[{"1", "-", "x"}]}], "]"}], " ", 
  RowBox[{"DeltaFunctionPrime", "[", 
   RowBox[{"1", "-", "x"}], "]"}]}]], "Input",
 CellTags->"SimplifyDeltaFunction",
 CellLabel->"In[1455]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TemplateBox[{"2",RowBox[{"1", "-", "x"}]},
    "PolyLog"], " ", 
   RowBox[{"log", "(", "x", ")"}], " ", 
   RowBox[{
    SuperscriptBox["\[Delta]", "\[Prime]"], "(", 
    FormBox[
     RowBox[{"1", "-", "x"}],
     TraditionalForm], ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009703826417294`*^9},
 CellTags->"SimplifyDeltaFunction",
 CellLabel->"Out[1455]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SimplifyDeltaFunction", "[", " ", 
  RowBox[{"%", ",", "x"}], "]"}]], "Input",
 CellTags->"SimplifyDeltaFunction",
 CellLabel->"In[1456]:="],

Cell[BoxData[
 FormBox["0", TraditionalForm]], "Output",
 CellChangeTimes->{3.5009703831772346`*^9},
 CellTags->"SimplifyDeltaFunction",
 CellLabel->"Out[1456]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"PolyLog", "[", 
   RowBox[{"3", ",", 
    RowBox[{"1", "-", "x"}]}], "]"}], " ", 
  RowBox[{"DeltaFunctionPrime", "[", 
   RowBox[{"1", "-", "x"}], "]"}]}]], "Input",
 CellTags->"SimplifyDeltaFunction",
 CellLabel->"In[1457]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TemplateBox[{"3",RowBox[{"1", "-", "x"}]},
    "PolyLog"], " ", 
   RowBox[{
    SuperscriptBox["\[Delta]", "\[Prime]"], "(", 
    FormBox[
     RowBox[{"1", "-", "x"}],
     TraditionalForm], ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009703832795057`*^9},
 CellTags->"SimplifyDeltaFunction",
 CellLabel->"Out[1457]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SimplifyDeltaFunction", "[", " ", 
  RowBox[{"%", ",", "x"}], "]"}]], "Input",
 CellTags->"SimplifyDeltaFunction",
 CellLabel->"In[1458]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   RowBox[{"\[Delta]", "(", 
    FormBox[
     RowBox[{"1", "-", "x"}],
     TraditionalForm], ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970383396143*^9},
 CellTags->"SimplifyDeltaFunction",
 CellLabel->"Out[1458]="]
}, Open  ]]
}, Open  ]],

Cell["SimplifyGTI  ***unfinished***", "ObjectName",
 CellTags->"SimplifyGTI"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SimplifyGTI"],

Cell["SimplifyGTI simplifies GTI's.", "Text",
 CellTags->"SimplifyGTI"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["GTI",
  BaseStyle->"AddOnsLink",
  ButtonData:>"GTI",
  ButtonNote->"GTI"],
 "."
}], "Text",
 CellTags->"SimplifyGTI"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"SimplifyGTI"],

Cell[BoxData[""], "Input",
 CellTags->"SimplifyGTI",
 CellLabel->"In[1459]:="]
}, Open  ]],

Cell["SimplifyPolyLog", "ObjectName",
 CellTags->"SimplifyPolyLog"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SimplifyPolyLog"],

Cell["\<\
SimplifyPolyLog[y] performs several simplifications assuming that the \
variables occuring in the Log and PolyLog functions are between 0 and 1. \
\>", "Text",
 CellTags->"SimplifyPolyLog"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Nielsen",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Nielsen",
  ButtonNote->"Nielsen"],
 "."
}], "Text",
 CellTags->"SimplifyPolyLog"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"SimplifyPolyLog"],

Cell[BoxData[
 RowBox[{
  RowBox[{"sip", "[", "y_", "]"}], " ", ":=", " ", 
  RowBox[{"y", " ", "==", " ", 
   RowBox[{"SimplifyPolyLog", "[", "y", "]"}]}]}]], "Input",
 CellTags->"SimplifyPolyLog",
 CellLabel->"In[1460]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"sip", "[", 
  RowBox[{"PolyLog", "[", 
   RowBox[{"2", ",", 
    RowBox[{"1", "/", "x"}]}], "]"}], "]"}]], "Input",
 CellTags->"SimplifyPolyLog",
 CellLabel->"In[1461]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TemplateBox[{"2",FractionBox["1", "x"]},
    "PolyLog"], "\[LongEqual]", 
   RowBox[{
    RowBox[{"\[Zeta]", "(", "2", ")"}], "+", 
    TemplateBox[{"2",RowBox[{"1", "-", "x"}]},
     "PolyLog"], "-", 
    RowBox[{
     FractionBox["1", "2"], " ", 
     RowBox[{
      SuperscriptBox["log", "2"], "(", "x", ")"}]}], "+", 
    RowBox[{
     RowBox[{"log", "(", 
      RowBox[{"1", "-", "x"}], ")"}], " ", 
     RowBox[{"log", "(", "x", ")"}]}], "+", 
    RowBox[{"\[ImaginaryI]", " ", "\[Pi]", " ", 
     RowBox[{"log", "(", "x", ")"}]}]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009703835617723`*^9},
 CellTags->"SimplifyPolyLog",
 CellLabel->"Out[1461]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"sip", "[", 
  RowBox[{"PolyLog", "[", 
   RowBox[{"2", ",", "x"}], "]"}], "]"}]], "Input",
 CellTags->"SimplifyPolyLog",
 CellLabel->"In[1462]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TemplateBox[{"2","x"},
    "PolyLog"], "\[LongEqual]", 
   RowBox[{
    RowBox[{"\[Zeta]", "(", "2", ")"}], "-", 
    TemplateBox[{"2",RowBox[{"1", "-", "x"}]},
     "PolyLog"], "-", 
    RowBox[{
     RowBox[{"log", "(", 
      RowBox[{"1", "-", "x"}], ")"}], " ", 
     RowBox[{"log", "(", "x", ")"}]}]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970383695519*^9},
 CellTags->"SimplifyPolyLog",
 CellLabel->"Out[1462]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"sip", "[", 
  RowBox[{"PolyLog", "[", 
   RowBox[{"2", ",", 
    RowBox[{"1", "-", 
     RowBox[{"x", "^", "2"}]}]}], "]"}], "]"}]], "Input",
 CellTags->"SimplifyPolyLog",
 CellLabel->"In[1463]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TemplateBox[{"2",RowBox[{"1", "-", 
       SuperscriptBox["x", "2"]}]},
    "PolyLog"], "\[LongEqual]", 
   RowBox[{
    RowBox[{"-", 
     RowBox[{"\[Zeta]", "(", "2", ")"}]}], "+", 
    RowBox[{"2", " ", 
     TemplateBox[{"2",RowBox[{"1", "-", "x"}]},
      "PolyLog"]}], "-", 
    RowBox[{"2", " ", 
     TemplateBox[{"2",RowBox[{"-", "x"}]},
      "PolyLog"]}], "-", 
    RowBox[{"2", " ", 
     RowBox[{"log", "(", "x", ")"}], " ", 
     RowBox[{"log", "(", 
      RowBox[{"x", "+", "1"}], ")"}]}]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009703838725157`*^9},
 CellTags->"SimplifyPolyLog",
 CellLabel->"Out[1463]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"sip", "[", 
  RowBox[{"PolyLog", "[", 
   RowBox[{"2", ",", 
    RowBox[{"x", "^", "2"}]}], "]"}], "]"}]], "Input",
 CellTags->"SimplifyPolyLog",
 CellLabel->"In[1464]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TemplateBox[{"2",SuperscriptBox["x", "2"]},
    "PolyLog"], "\[LongEqual]", 
   RowBox[{
    RowBox[{"2", " ", 
     RowBox[{"\[Zeta]", "(", "2", ")"}]}], "-", 
    RowBox[{"2", " ", 
     TemplateBox[{"2",RowBox[{"1", "-", "x"}]},
      "PolyLog"]}], "+", 
    RowBox[{"2", " ", 
     TemplateBox[{"2",RowBox[{"-", "x"}]},
      "PolyLog"]}], "-", 
    RowBox[{"2", " ", 
     RowBox[{"log", "(", 
      RowBox[{"1", "-", "x"}], ")"}], " ", 
     RowBox[{"log", "(", "x", ")"}]}]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.50097038397536*^9},
 CellTags->"SimplifyPolyLog",
 CellLabel->"Out[1464]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"sip", "[", 
  RowBox[{"PolyLog", "[", 
   RowBox[{"2", ",", 
    RowBox[{"-", 
     RowBox[{"x", "/", 
      RowBox[{"(", 
       RowBox[{"1", "-", "x"}], ")"}]}]}]}], "]"}], "]"}]], "Input",
 CellTags->"SimplifyPolyLog",
 CellLabel->"In[1465]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TemplateBox[{"2",RowBox[{"-", 
       FractionBox["x", 
        RowBox[{"1", "-", "x"}]]}]},
    "PolyLog"], "\[LongEqual]", 
   RowBox[{
    RowBox[{"-", 
     RowBox[{"\[Zeta]", "(", "2", ")"}]}], "+", 
    TemplateBox[{"2",RowBox[{"1", "-", "x"}]},
     "PolyLog"], "-", 
    RowBox[{
     FractionBox["1", "2"], " ", 
     RowBox[{
      SuperscriptBox["log", "2"], "(", 
      RowBox[{"1", "-", "x"}], ")"}]}], "+", 
    RowBox[{
     RowBox[{"log", "(", "x", ")"}], " ", 
     RowBox[{"log", "(", 
      RowBox[{"1", "-", "x"}], ")"}]}]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009703840957603`*^9},
 CellTags->"SimplifyPolyLog",
 CellLabel->"Out[1465]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"sip", "[", 
  RowBox[{"PolyLog", "[", 
   RowBox[{"2", ",", 
    RowBox[{"x", "/", 
     RowBox[{"(", 
      RowBox[{"x", "-", "1"}], ")"}]}]}], "]"}], "]"}]], "Input",
 CellTags->"SimplifyPolyLog",
 CellLabel->"In[1466]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TemplateBox[{"2",FractionBox["x", 
      RowBox[{"x", "-", "1"}]]},
    "PolyLog"], "\[LongEqual]", 
   RowBox[{
    RowBox[{"-", 
     RowBox[{"\[Zeta]", "(", "2", ")"}]}], "+", 
    TemplateBox[{"2",RowBox[{"1", "-", "x"}]},
     "PolyLog"], "-", 
    RowBox[{
     FractionBox["1", "2"], " ", 
     RowBox[{
      SuperscriptBox["log", "2"], "(", 
      RowBox[{"1", "-", "x"}], ")"}]}], "+", 
    RowBox[{
     RowBox[{"log", "(", "x", ")"}], " ", 
     RowBox[{"log", "(", 
      RowBox[{"1", "-", "x"}], ")"}]}]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009703842941103`*^9},
 CellTags->"SimplifyPolyLog",
 CellLabel->"Out[1466]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"sip", "[", 
  RowBox[{"Nielsen", "[", 
   RowBox[{"1", ",", "2", ",", 
    RowBox[{"-", 
     RowBox[{"x", "/", 
      RowBox[{"(", 
       RowBox[{"1", "-", "x"}], ")"}]}]}]}], "]"}], "]"}]], "Input",
 CellTags->"SimplifyPolyLog",
 CellLabel->"In[1467]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox["S", 
     RowBox[{
      FormBox["1",
       TraditionalForm], 
      FormBox["2",
       TraditionalForm]}]], "(", 
    FormBox[
     RowBox[{"-", 
      FractionBox["x", 
       RowBox[{"1", "-", "x"}]]}],
     TraditionalForm], ")"}], "\[LongEqual]", 
   RowBox[{
    RowBox[{
     SubscriptBox["S", 
      RowBox[{
       FormBox["1",
        TraditionalForm], 
       FormBox["2",
        TraditionalForm]}]], "(", "x", ")"}], "-", 
    RowBox[{
     FractionBox["1", "6"], " ", 
     RowBox[{
      SuperscriptBox["log", "3"], "(", 
      RowBox[{"1", "-", "x"}], ")"}]}]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970384482889*^9},
 CellTags->"SimplifyPolyLog",
 CellLabel->"Out[1467]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"sip", "[", 
  RowBox[{"PolyLog", "[", 
   RowBox[{"3", ",", 
    RowBox[{"-", 
     RowBox[{"1", "/", "x"}]}]}], "]"}], "]"}]], "Input",
 CellTags->"SimplifyPolyLog",
 CellLabel->"In[1468]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TemplateBox[{"3",RowBox[{"-", 
       FractionBox["1", "x"]}]},
    "PolyLog"], "\[LongEqual]", 
   RowBox[{
    TemplateBox[{"3",RowBox[{"-", "x"}]},
     "PolyLog"], "+", 
    RowBox[{
     RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
     RowBox[{"log", "(", "x", ")"}]}], "+", 
    FractionBox[
     RowBox[{
      SuperscriptBox["log", "3"], "(", "x", ")"}], "6"]}]}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970384598011*^9},
 CellTags->"SimplifyPolyLog",
 CellLabel->"Out[1468]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"sip", "[", 
  RowBox[{"PolyLog", "[", 
   RowBox[{"3", ",", 
    RowBox[{"1", "-", "x"}]}], "]"}], "]"}]], "Input",
 CellTags->"SimplifyPolyLog",
 CellLabel->"In[1469]:="],

Cell[BoxData[
 FormBox["True", TraditionalForm]], "Output",
 CellChangeTimes->{3.500970384678566*^9},
 CellTags->"SimplifyPolyLog",
 CellLabel->"Out[1469]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"sip", "[", 
  RowBox[{"PolyLog", "[", 
   RowBox[{"3", ",", 
    RowBox[{"x", "^", "2"}]}], "]"}], "]"}]], "Input",
 CellTags->"SimplifyPolyLog",
 CellLabel->"In[1470]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TemplateBox[{"3",SuperscriptBox["x", "2"]},
    "PolyLog"], "\[LongEqual]", 
   RowBox[{
    RowBox[{"4", " ", 
     TemplateBox[{"3",RowBox[{"-", "x"}]},
      "PolyLog"]}], "-", 
    RowBox[{"4", " ", 
     TemplateBox[{"2",RowBox[{"1", "-", "x"}]},
      "PolyLog"], " ", 
     RowBox[{"log", "(", "x", ")"}]}], "-", 
    RowBox[{"4", " ", 
     RowBox[{
      SubscriptBox["S", 
       RowBox[{
        FormBox["1",
         TraditionalForm], 
        FormBox["2",
         TraditionalForm]}]], "(", 
      FormBox[
       RowBox[{"1", "-", "x"}],
       TraditionalForm], ")"}]}], "+", 
    RowBox[{"4", " ", 
     RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
     RowBox[{"log", "(", "x", ")"}]}], "-", 
    RowBox[{"2", " ", 
     RowBox[{"log", "(", 
      RowBox[{"1", "-", "x"}], ")"}], " ", 
     RowBox[{
      SuperscriptBox["log", "2"], "(", "x", ")"}]}], "+", 
    RowBox[{"4", " ", 
     TemplateBox[{"3"},
      "Zeta"]}]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009703847640676`*^9},
 CellTags->"SimplifyPolyLog",
 CellLabel->"Out[1470]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"sip", "[", 
  RowBox[{"PolyLog", "[", 
   RowBox[{"3", ",", 
    RowBox[{"-", 
     RowBox[{"x", "/", 
      RowBox[{"(", 
       RowBox[{"1", "-", "x"}], ")"}]}]}]}], "]"}], "]"}]], "Input",
 CellTags->"SimplifyPolyLog",
 CellLabel->"In[1471]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TemplateBox[{"3",RowBox[{"-", 
       FractionBox["x", 
        RowBox[{"1", "-", "x"}]]}]},
    "PolyLog"], "\[LongEqual]", 
   RowBox[{
    RowBox[{"-", 
     TemplateBox[{"3",RowBox[{"1", "-", "x"}]},
      "PolyLog"]}], "+", 
    RowBox[{
     TemplateBox[{"2",RowBox[{"1", "-", "x"}]},
      "PolyLog"], " ", 
     RowBox[{"log", "(", "x", ")"}]}], "+", 
    RowBox[{
     SubscriptBox["S", 
      RowBox[{
       FormBox["1",
        TraditionalForm], 
       FormBox["2",
        TraditionalForm]}]], "(", 
     FormBox[
      RowBox[{"1", "-", "x"}],
      TraditionalForm], ")"}], "+", 
    RowBox[{
     RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
     RowBox[{"log", "(", 
      RowBox[{"1", "-", "x"}], ")"}]}], "-", 
    RowBox[{
     RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
     RowBox[{"log", "(", "x", ")"}]}], "+", 
    RowBox[{
     FractionBox["1", "6"], " ", 
     RowBox[{
      SuperscriptBox["log", "3"], "(", 
      RowBox[{"1", "-", "x"}], ")"}]}], "-", 
    RowBox[{
     FractionBox["1", "2"], " ", 
     RowBox[{"log", "(", "x", ")"}], " ", 
     RowBox[{
      SuperscriptBox["log", "2"], "(", 
      RowBox[{"1", "-", "x"}], ")"}]}], "+", 
    RowBox[{
     FractionBox["1", "2"], " ", 
     RowBox[{
      SuperscriptBox["log", "2"], "(", "x", ")"}], " ", 
     RowBox[{"log", "(", 
      RowBox[{"1", "-", "x"}], ")"}]}]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970384983575*^9},
 CellTags->"SimplifyPolyLog",
 CellLabel->"Out[1471]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"sip", "[", 
  RowBox[{"PolyLog", "[", 
   RowBox[{"3", ",", 
    RowBox[{"1", "-", 
     RowBox[{"1", "/", "x"}]}]}], "]"}], "]"}]], "Input",
 CellTags->"SimplifyPolyLog",
 CellLabel->"In[1472]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TemplateBox[{"3",RowBox[{"1", "-", 
       FractionBox["1", "x"]}]},
    "PolyLog"], "\[LongEqual]", 
   RowBox[{
    RowBox[{
     TemplateBox[{"2",RowBox[{"1", "-", "x"}]},
      "PolyLog"], " ", 
     RowBox[{"log", "(", "x", ")"}]}], "-", 
    RowBox[{
     TemplateBox[{"2",RowBox[{"1", "-", "x"}]},
      "PolyLog"], " ", 
     RowBox[{"log", "(", 
      RowBox[{"1", "-", "x"}], ")"}]}], "+", 
    RowBox[{
     SubscriptBox["S", 
      RowBox[{
       FormBox["1",
        TraditionalForm], 
       FormBox["2",
        TraditionalForm]}]], "(", 
     FormBox[
      RowBox[{"1", "-", "x"}],
      TraditionalForm], ")"}], "+", 
    RowBox[{
     SubscriptBox["S", 
      RowBox[{
       FormBox["1",
        TraditionalForm], 
       FormBox["2",
        TraditionalForm]}]], "(", "x", ")"}], "+", 
    FractionBox[
     RowBox[{
      SuperscriptBox["log", "3"], "(", "x", ")"}], "6"], "-", 
    RowBox[{
     FractionBox["1", "2"], " ", 
     RowBox[{
      SuperscriptBox["log", "2"], "(", 
      RowBox[{"1", "-", "x"}], ")"}], " ", 
     RowBox[{"log", "(", "x", ")"}]}], "-", 
    TemplateBox[{"3"},
     "Zeta"]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970385200335*^9},
 CellTags->"SimplifyPolyLog",
 CellLabel->"Out[1472]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"sip", "[", 
  RowBox[{"PolyLog", "[", 
   RowBox[{"4", ",", 
    RowBox[{"-", 
     RowBox[{"x", "/", 
      RowBox[{"(", 
       RowBox[{"1", "-", "x"}], ")"}]}]}]}], "]"}], "]"}]], "Input",
 CellTags->"SimplifyPolyLog",
 CellLabel->"In[1473]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TemplateBox[{"4",RowBox[{"-", 
       FractionBox["x", 
        RowBox[{"1", "-", "x"}]]}]},
    "PolyLog"], "\[LongEqual]", 
   RowBox[{
    RowBox[{"-", 
     TemplateBox[{"4","x"},
      "PolyLog"]}], "+", 
    RowBox[{
     FractionBox["1", "2"], " ", 
     TemplateBox[{"2",RowBox[{"1", "-", "x"}]},
      "PolyLog"], " ", 
     RowBox[{
      SuperscriptBox["log", "2"], "(", 
      RowBox[{"1", "-", "x"}], ")"}]}], "-", 
    RowBox[{
     TemplateBox[{"2",RowBox[{"1", "-", "x"}]},
      "PolyLog"], " ", 
     RowBox[{"log", "(", "x", ")"}], " ", 
     RowBox[{"log", "(", 
      RowBox[{"1", "-", "x"}], ")"}]}], "-", 
    RowBox[{
     SubscriptBox["S", 
      RowBox[{
       FormBox["1",
        TraditionalForm], 
       FormBox["3",
        TraditionalForm]}]], "(", "x", ")"}], "+", 
    RowBox[{
     SubscriptBox["S", 
      RowBox[{
       FormBox["2",
        TraditionalForm], 
       FormBox["2",
        TraditionalForm]}]], "(", "x", ")"}], "-", 
    RowBox[{
     RowBox[{
      SubscriptBox["S", 
       RowBox[{
        FormBox["1",
         TraditionalForm], 
        FormBox["2",
         TraditionalForm]}]], "(", 
      FormBox[
       RowBox[{"1", "-", "x"}],
       TraditionalForm], ")"}], " ", 
     RowBox[{"log", "(", 
      RowBox[{"1", "-", "x"}], ")"}]}], "-", 
    RowBox[{
     RowBox[{
      SubscriptBox["S", 
       RowBox[{
        FormBox["1",
         TraditionalForm], 
        FormBox["2",
         TraditionalForm]}]], "(", "x", ")"}], " ", 
     RowBox[{"log", "(", 
      RowBox[{"1", "-", "x"}], ")"}]}], "-", 
    RowBox[{
     FractionBox["1", "2"], " ", 
     RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
     RowBox[{
      SuperscriptBox["log", "2"], "(", 
      RowBox[{"1", "-", "x"}], ")"}]}], "+", 
    RowBox[{
     RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
     RowBox[{"log", "(", "x", ")"}], " ", 
     RowBox[{"log", "(", 
      RowBox[{"1", "-", "x"}], ")"}]}], "+", 
    RowBox[{
     TemplateBox[{"3"},
      "Zeta"], " ", 
     RowBox[{"log", "(", 
      RowBox[{"1", "-", "x"}], ")"}]}], "-", 
    RowBox[{
     FractionBox["1", "24"], " ", 
     RowBox[{
      SuperscriptBox["log", "4"], "(", 
      RowBox[{"1", "-", "x"}], ")"}]}], "+", 
    RowBox[{
     FractionBox["1", "2"], " ", 
     RowBox[{"log", "(", "x", ")"}], " ", 
     RowBox[{
      SuperscriptBox["log", "3"], "(", 
      RowBox[{"1", "-", "x"}], ")"}]}], "-", 
    RowBox[{
     FractionBox["1", "2"], " ", 
     RowBox[{
      SuperscriptBox["log", "2"], "(", "x", ")"}], " ", 
     RowBox[{
      SuperscriptBox["log", "2"], "(", 
      RowBox[{"1", "-", "x"}], ")"}]}]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970385452857*^9},
 CellTags->"SimplifyPolyLog",
 CellLabel->"Out[1473]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"sip", "[", 
  RowBox[{"Log", "[", 
   RowBox[{"a", "+", 
    RowBox[{"b", "/", "c"}]}], "]"}], "]"}]], "Input",
 CellTags->"SimplifyPolyLog",
 CellLabel->"In[1474]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"log", "(", 
    RowBox[{"a", "+", 
     FractionBox["b", "c"]}], ")"}], "\[LongEqual]", 
   RowBox[{"log", "(", 
    FractionBox[
     RowBox[{
      RowBox[{"a", " ", "c"}], "+", "b"}], "c"], ")"}]}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970385540483*^9},
 CellTags->"SimplifyPolyLog",
 CellLabel->"Out[1474]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"sip", "[", 
  RowBox[{"Log", "[", 
   RowBox[{"1", "/", "x"}], "]"}], "]"}]], "Input",
 CellTags->"SimplifyPolyLog",
 CellLabel->"In[1475]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"log", "(", 
    FractionBox["1", "x"], ")"}], "\[LongEqual]", 
   RowBox[{"-", 
    RowBox[{"log", "(", "x", ")"}]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009703856189327`*^9},
 CellTags->"SimplifyPolyLog",
 CellLabel->"Out[1475]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"sip", "[", 
  RowBox[{"ArcTanh", "[", "x", "]"}], "]"}]], "Input",
 CellTags->"SimplifyPolyLog",
 CellLabel->"In[1476]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SuperscriptBox["tanh", 
     RowBox[{"-", "1"}]], "(", "x", ")"}], "\[LongEqual]", 
   RowBox[{
    FractionBox["1", "2"], " ", 
    RowBox[{"log", "(", 
     RowBox[{"-", 
      FractionBox[
       RowBox[{"x", "+", "1"}], 
       RowBox[{"1", "-", "x"}]]}], ")"}]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009703856966333`*^9},
 CellTags->"SimplifyPolyLog",
 CellLabel->"Out[1476]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"sip", "[", 
  RowBox[{"ArcSinh", "[", "x", "]"}], "]"}]], "Input",
 CellTags->"SimplifyPolyLog",
 CellLabel->"In[1477]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SuperscriptBox["sinh", 
     RowBox[{"-", "1"}]], "(", "x", ")"}], "\[LongEqual]", 
   RowBox[{"log", "(", 
    RowBox[{
     SqrtBox[
      RowBox[{
       SuperscriptBox["x", "2"], "+", "1"}]], "+", "x"}], ")"}]}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.5009703857843657`*^9},
 CellTags->"SimplifyPolyLog",
 CellLabel->"Out[1477]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"sip", "[", 
  RowBox[{"ArcCosh", "[", "x", "]"}], "]"}]], "Input",
 CellTags->"SimplifyPolyLog",
 CellLabel->"In[1478]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SuperscriptBox["cosh", 
     RowBox[{"-", "1"}]], "(", "x", ")"}], "\[LongEqual]", 
   RowBox[{"log", "(", 
    RowBox[{
     SqrtBox[
      RowBox[{
       SuperscriptBox["x", "2"], "-", "1"}]], "+", "x"}], ")"}]}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970385893758*^9},
 CellTags->"SimplifyPolyLog",
 CellLabel->"Out[1478]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"Clear", "[", "sip", "]"}]], "Input",
 CellTags->"SimplifyPolyLog",
 CellLabel->"In[1479]:="]
}, Open  ]],

Cell["Simplify2  ***unfinished***", "ObjectName",
 CellTags->"Simplify2"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Simplify2"],

Cell["Simplify2 is a special ordering function.", "Text",
 CellTags->"Simplify2"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Simplify2"],

Cell[BoxData[""], "Input",
 CellTags->"Simplify2",
 CellLabel->"In[1480]:="]
}, Open  ]],

Cell["SmallDelta", "ObjectName",
 CellTags->"SmallDelta"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SmallDelta"],

Cell["SmallDelta denotes some small positive number.", "Text",
 CellTags->"SmallDelta"]
}, Open  ]],

Cell["SmallEpsilon", "ObjectName",
 CellTags->"SmallEpsilon"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SmallEpsilon"],

Cell["SmallEpsilon denotes some small positive number.", "Text",
 CellTags->"SmallEpsilon"]
}, Open  ]],

Cell["SmallVariable", "ObjectName",
 CellTags->"SmallVariable"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SmallVariable"],

Cell["\<\
SmallVariable[me] is the head of small (negligible) variables. This means any \
mass with this head can be neglected if it appears in a sum, but not as an \
argument of Passarino-Veltman (PaVe) functions or PropagatorDenominator.\
\>", "Text",
 CellTags->"SmallVariable"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["PaVe",
  BaseStyle->"AddOnsLink",
  ButtonData:>"PaVe",
  ButtonNote->"PaVe"],
 ", ",
 ButtonBox["PaVeReduce",
  BaseStyle->"AddOnsLink",
  ButtonData:>"PaVeReduce",
  ButtonNote->"PaVeReduce"],
 ", ",
 ButtonBox["PropagatorDenominator",
  BaseStyle->"AddOnsLink",
  ButtonData:>"PropagatorDenominator",
  ButtonNote->"PropagatorDenominator"],
 "."
}], "Text",
 CellTags->"SmallVariable"]
}, Open  ]],

Cell["SmallVariables", "ObjectName",
 CellTags->"SmallVariables"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SmallVariables"],

Cell["\<\
SmallVariables is an option for OneLoop. \"SmallVariables\[Rule]{Melectron}\" \
i.e. will substitute \"SmallVariable[Melectron]\"  for all Melectron's in the \
calculation.\
\>", "Text",
 CellTags->"SmallVariables"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["OneLoop",
  BaseStyle->"AddOnsLink",
  ButtonData:>"OneLoop",
  ButtonNote->"OneLoop"],
 "."
}], "Text",
 CellTags->"SmallVariables"]
}, Open  ]],

Cell["SMP", "ObjectName",
 CellTags->"SMP"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SMP"],

Cell["\<\
SMP[par] substitutes a symbol for the  Standard Model parameter par.  SMP[] \
gives the list of substitutions. par should be a string; e.g., SMP[\"SW\"] \
gives  sw (in the Global` context).\
\>", "Text",
 CellTags->"SMP"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"SMP", "[", "]"}], "//", "InputForm"}]], "Input",
 CellTags->"SMP",
 CellLabel->"In[1481]:="],

Cell["\<\
{\"EL\" :> e, \"CW\" :> cw, \"ME\" :> me, \"MH\" :> mh, \"MW\" :> mw, \"SW\" \
:> sw}\
\>", "Output",
 CellChangeTimes->{3.500970386006748*^9},
 CellTags->"SMP",
 CellLabel->"Out[1481]//InputForm="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 ButtonBox["SMVertex",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SMVertex",
  ButtonNote->"SMVertex"],
 "."
}], "Text",
 CellTags->"SMP"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"SMP"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SMP", "[", "\"\<ME\>\"", "]"}]], "Input",
 CellTags->"SMP",
 CellLabel->"In[1482]:="],

Cell[BoxData[
 FormBox["me", TraditionalForm]], "Output",
 CellChangeTimes->{3.500970386088585*^9},
 CellTags->"SMP",
 CellLabel->"Out[1482]="]
}, Open  ]]
}, Open  ]],

Cell["SMVertex", "ObjectName",
 CellTags->"SMVertex"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SMVertex"],

Cell["\<\
SMVertex[\"AWW\", p,mu, q,nu, k,rho] gives the photon-W-W vertex (p,mu \
correspond to the photon, q,nu to the (incoming) W+ and k,rho to the \
(incoming) W-. All momenta are flowing into the vertex.  SMVertex[\"HHH\", \
___] give the three-higgs coupling. \
\>", "Text",
 CellTags->"SMVertex"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SMVertex", "//", "Options"}]], "Input",
 CellTags->"SMVertex",
 CellLabel->"In[1483]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Dimension", "\[Rule]", "4"}], ",", 
    RowBox[{"Explicit", "\[Rule]", "True"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.5009703861734257`*^9},
 CellTags->"SMVertex",
 CellLabel->"Out[1483]="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"SMVertex"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SMVertex", "[", 
  RowBox[{
  "\"\<AWW\>\"", ",", "p", ",", "\[Mu]", ",", " ", "q", ",", "\[Nu]", ",", 
   " ", "k", ",", "\[Rho]"}], "]"}]], "Input",
 CellTags->"SMVertex",
 CellLabel->"In[1484]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", "\[ImaginaryI]"}], " ", "e", " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"\[Mu]", "\[Rho]"}]], " ", 
       SuperscriptBox[
        RowBox[{"(", 
         FormBox[
          RowBox[{"k", "-", "p"}],
          TraditionalForm], ")"}], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]]}]}], "+", 
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Nu]", "\[Rho]"}]], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        FormBox[
         RowBox[{"k", "-", "q"}],
         TraditionalForm], ")"}], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]]}], "-", 
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        FormBox[
         RowBox[{"p", "-", "q"}],
         TraditionalForm], ")"}], 
       FormBox[
        FormBox["\[Rho]",
         TraditionalForm],
        TraditionalForm]]}]}], ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009703863240957`*^9},
 CellTags->"SMVertex",
 CellLabel->"Out[1484]="]
}, Open  ]]
}, Open  ]],

Cell["Smu  ***unfinished***", "ObjectName",
 CellTags->"Smu"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Smu"],

Cell["Smu  is ...", "Text",
 CellTags->"Smu"]
}, Open  ]],

Cell["Sn", "ObjectName",
 CellTags->"Sn"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Sn"],

Cell[TextData[{
 "Sn is ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["\[Pi]", 
    RowBox[{"n", "/", "2"}]], TraditionalForm]]],
 "/",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"2", " ", "\[Pi]"}], ")"}], "n"], TraditionalForm]]],
 "."
}], "Text",
 CellTags->"Sn"]
}, Open  ]],

Cell["SO", "ObjectName",
 CellTags->"SO"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SO"],

Cell["\<\
SO[q] is a four-dimensional scalar product of OPEDelta with q. It is \
transformed into Pair[Momentum[q], Momentum[OPEDelta] by FCI.\
\>", "Text",
 CellTags->"SO"],

Cell[TextData[{
 "See also: ",
 ButtonBox["FCI",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FCI",
  ButtonNote->"FCI"],
 ", ",
 ButtonBox["OPEDelta",
  BaseStyle->"AddOnsLink",
  ButtonData:>"OPEDelta",
  ButtonNote->"OPEDelta"],
 ", ",
 ButtonBox["Pair",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Pair",
  ButtonNote->"Pair"],
 ", ",
 ButtonBox["ScalarProduct",
  BaseStyle->"AddOnsLink",
  ButtonData:>"ScalarProduct",
  ButtonNote->"ScalarProduct"],
 ", ",
 ButtonBox["SOD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SOD",
  ButtonNote->"SOD"],
 "."
}], "Text",
 CellTags->"SO"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"SO"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SO", "[", "p", "]"}]], "Input",
 CellTags->"SO",
 CellLabel->"In[1485]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox["\<\"\[CapitalDelta]\"\>",
    TraditionalForm], 
   FormBox["\<\"\[CenterDot]\"\>",
    TraditionalForm], 
   FormBox["p",
    TraditionalForm]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970386426815*^9},
 CellTags->"SO",
 CellLabel->"Out[1485]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SO", "[", 
  RowBox[{"p", "-", "q"}], "]"}]], "Input",
 CellTags->"SO",
 CellLabel->"In[1486]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox["\<\"\[CapitalDelta]\"\>",
    TraditionalForm], 
   FormBox["\<\"\[CenterDot]\"\>",
    TraditionalForm], 
   FormBox["\<\"(\"\>",
    TraditionalForm], 
   FormBox[
    RowBox[{"p", "-", "q"}],
    TraditionalForm], 
   FormBox["\<\")\"\>",
    TraditionalForm]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970386530196*^9},
 CellTags->"SO",
 CellLabel->"Out[1486]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"SO", "[", "p", "]"}], "//", "FCI"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"SO",
 CellLabel->"In[1487]:="],

Cell[BoxData[
 RowBox[{"Pair", "[", 
  RowBox[{
   RowBox[{"Momentum", "[", "OPEDelta", "]"}], ",", 
   RowBox[{"Momentum", "[", "p", "]"}]}], "]"}]], "Output",
 CellChangeTimes->{3.500970386633793*^9},
 CellTags->"SO",
 CellLabel->"Out[1487]//StandardForm="]
}, Open  ]]
}, Open  ]],

Cell["SOD", "ObjectName",
 CellTags->"SOD"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SOD"],

Cell["\<\
SOD[q] is a D-dimensional scalar product of OPEDelta with q. It is \
transformed into Pair[Momentum[q,D], Momentum[OPEDelta,D] by FeynCalcInternal.\
\>", "Text",
 CellTags->"SOD"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["OPEDelta",
  BaseStyle->"AddOnsLink",
  ButtonData:>"OPEDelta",
  ButtonNote->"OPEDelta"],
 ", ",
 ButtonBox["Pair",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Pair",
  ButtonNote->"Pair"],
 ", ",
 ButtonBox["ScalarProduct",
  BaseStyle->"AddOnsLink",
  ButtonData:>"ScalarProduct",
  ButtonNote->"ScalarProduct"],
 ", ",
 ButtonBox["SOD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SOD",
  ButtonNote->"SOD"],
 "."
}], "Text",
 CellTags->"SOD"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"SOD"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SOD", "[", "p", "]"}]], "Input",
 CellTags->"SOD",
 CellLabel->"In[1488]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox["\<\"\[CapitalDelta]\"\>",
    TraditionalForm], 
   FormBox["\<\"\[CenterDot]\"\>",
    TraditionalForm], 
   FormBox["p",
    TraditionalForm]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.50097038668864*^9},
 CellTags->"SOD",
 CellLabel->"Out[1488]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SOD", "[", 
  RowBox[{"p", "-", "q"}], "]"}]], "Input",
 CellTags->"SOD",
 CellLabel->"In[1489]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox["\<\"\[CapitalDelta]\"\>",
    TraditionalForm], 
   FormBox["\<\"\[CenterDot]\"\>",
    TraditionalForm], 
   FormBox["\<\"(\"\>",
    TraditionalForm], 
   FormBox[
    RowBox[{"p", "-", "q"}],
    TraditionalForm], 
   FormBox["\<\")\"\>",
    TraditionalForm]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970386800749*^9},
 CellTags->"SOD",
 CellLabel->"Out[1489]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"SOD", "[", "p", "]"}], "//", "FCI"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"SOD",
 CellLabel->"In[1490]:="],

Cell[BoxData[
 RowBox[{"Pair", "[", 
  RowBox[{
   RowBox[{"Momentum", "[", 
    RowBox[{"OPEDelta", ",", "D"}], "]"}], ",", 
   RowBox[{"Momentum", "[", 
    RowBox[{"p", ",", "D"}], "]"}]}], "]"}]], "Output",
 CellChangeTimes->{3.500970387392346*^9},
 CellTags->"SOD",
 CellLabel->"Out[1490]//StandardForm="]
}, Open  ]]
}, Open  ]],

Cell["Solve2", "ObjectName",
 CellTags->"Solve2"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Solve2"],

Cell["\<\
Solve2 is equivalent to Solve, except that it works only for linear equations \
(and returns just a list) and accepts the options Factoring and \
FinalSubstitutions. Solve2 uses the \"high school algorithm\" and factors \
intermediate results. Therefore it can be drastically more useful than Solve.\
\
\>", "Text",
 CellTags->"Solve2"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "Solve2", "]"}]], "Input",
 CellTags->"Solve2",
 CellLabel->"In[1491]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Factoring", "\[Rule]", "Factor2"}], ",", 
    RowBox[{"FinalSubstitutions", "\[Rule]", 
     RowBox[{"{", "}"}]}]}], "}"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970387558983*^9},
 CellTags->"Solve2",
 CellLabel->"Out[1491]="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Solve2"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Solve2", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"2", " ", "x"}], "==", 
      RowBox[{"b", "-", 
       RowBox[{"w", "/", "2"}]}]}], ",", 
     RowBox[{
      RowBox[{"y", "-", "d"}], "==", "p"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "y"}], "}"}]}], "]"}]], "Input",
 CellTags->"Solve2",
 CellLabel->"In[1492]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"x", "\[Rule]", 
     RowBox[{
      FractionBox["1", "4"], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"2", " ", "b"}], "-", "w"}], ")"}]}]}], ",", 
    RowBox[{"y", "\[Rule]", 
     RowBox[{"d", "+", "p"}]}]}], "}"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970387749271*^9},
 CellTags->"Solve2",
 CellLabel->"Out[1492]="]
}, Open  ]],

Cell["\<\
If no equation sign is given the polynomials are supposed to be 0.\
\>", "Text",
 CellTags->"Solve2"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Solve2", "[", 
  RowBox[{
   RowBox[{"x", "+", "y"}], ",", "x"}], "]"}]], "Input",
 CellTags->"Solve2",
 CellLabel->"In[1493]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"x", "\[Rule]", 
    RowBox[{"-", "y"}]}], "}"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970387952674*^9},
 CellTags->"Solve2",
 CellLabel->"Out[1493]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Solve2", "[", 
  RowBox[{
   RowBox[{"x", "+", "y"}], ",", "x", ",", 
   RowBox[{"FinalSubstitutions", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"y", "\[Rule]", "h"}], "}"}]}]}], "]"}]], "Input",
 CellTags->"Solve2",
 CellLabel->"In[1494]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"x", "\[Rule]", 
    RowBox[{"-", "h"}]}], "}"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970388167481*^9},
 CellTags->"Solve2",
 CellLabel->"Out[1494]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Solve2", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"2", " ", "x"}], "==", 
      RowBox[{"b", "-", 
       RowBox[{"w", "/", "2"}]}]}], ",", 
     RowBox[{
      RowBox[{"y", "-", "d"}], "==", "p"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "y"}], "}"}], ",", 
   RowBox[{"Factoring", "\[Rule]", "Expand"}]}], "]"}]], "Input",
 CellTags->"Solve2",
 CellLabel->"In[1495]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"x", "\[Rule]", 
     RowBox[{
      FractionBox["b", "2"], "-", 
      FractionBox["w", "4"]}]}], ",", 
    RowBox[{"y", "\[Rule]", 
     RowBox[{"d", "+", "p"}]}]}], "}"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970388364687*^9},
 CellTags->"Solve2",
 CellLabel->"Out[1495]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Solve", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"2", " ", "x"}], "==", 
      RowBox[{"b", "-", 
       RowBox[{"w", "/", "2"}]}]}], ",", 
     RowBox[{
      RowBox[{"y", "-", "d"}], "==", "p"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "y"}], "}"}]}], "]"}]], "Input",
 CellTags->"Solve2",
 CellLabel->"In[1496]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"x", "\[Rule]", 
      RowBox[{
       FractionBox["1", "4"], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"2", " ", "b"}], "-", "w"}], ")"}]}]}], ",", 
     RowBox[{"y", "\[Rule]", 
      RowBox[{"d", "+", "p"}]}]}], "}"}], "}"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970388549954*^9},
 CellTags->"Solve2",
 CellLabel->"Out[1496]="]
}, Open  ]]
}, Open  ]],

Cell["Solve3", "ObjectName",
 CellTags->"Solve3"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Solve3"],

Cell["\<\
Solve3 is equivalent to Solve, except that it works only for linear equations \
(and returns just a list) and uses the \"high school algorithm\" and is \
sometimes better than Solve for systems involving rational polyonomials.\
\>", "Text",
 CellTags->"Solve3"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "Solve3", "]"}]], "Input",
 CellTags->"Solve3",
 CellLabel->"In[1497]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Factoring", "\[Rule]", "False"}], ",", 
    RowBox[{"FinalSubstitutions", "\[Rule]", 
     RowBox[{"{", "}"}]}], ",", 
    RowBox[{"ParallelMap", "\[Rule]", "True"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970388913176*^9},
 CellTags->"Solve3",
 CellLabel->"Out[1497]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 ButtonBox["Solve2",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Solve2",
  ButtonNote->"Solve2"],
 "."
}], "Text",
 CellTags->"SMP"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Solve3"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Solve3", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"2", " ", "x"}], "==", 
      RowBox[{"b", "-", 
       RowBox[{"w", "/", "2"}]}]}], ",", 
     RowBox[{
      RowBox[{"y", "-", "d"}], "==", "p"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "y"}], "}"}]}], "]"}]], "Input",
 CellTags->"Solve3",
 CellLabel->"In[1498]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   StyleBox[
    RowBox[{"Rule", "::", "rhs"}], "MessageName"], 
   RowBox[{
   ":", " "}], "\<\"Pattern \
\\!\\(TraditionalForm\\`HighEnergyPhysics`FeynCalc`FeynCalcForm`Private`v__\\)\
 appears on the right-hand side of rule \
\\!\\(TraditionalForm\\`\\(\\(\\(HighEnergyPhysics`FeynCalc`LorentzIndex`\
LorentzIndex(\\(\\(HighEnergyPhysics`FeynCalc`FeynCalcForm`Private`v__, \
4\\)\\))\\)\\) \[Rule] \
\\(\\(HighEnergyPhysics`FeynCalc`LorentzIndex`LorentzIndex(HighEnergyPhysics`\
FeynCalc`FeynCalcForm`Private`v__)\\)\\)\\)\\).\"\>"}], 
  TraditionalForm]], "Message", "MSG",
 ShowCellLabel->True,
 CellChangeTimes->{3.500970394690345*^9},
 CellTags->"Solve3"],

Cell[BoxData[
 FormBox[
  RowBox[{
   StyleBox[
    RowBox[{"Rule", "::", "rhs"}], "MessageName"], 
   RowBox[{
   ":", " "}], "\<\"Pattern \
\\!\\(TraditionalForm\\`HighEnergyPhysics`FeynCalc`FeynCalcForm`Private`v__\\)\
 appears on the right-hand side of rule \
\\!\\(TraditionalForm\\`\\(\\(\\(HighEnergyPhysics`FeynCalc`LorentzIndex`\
LorentzIndex(\\(\\(HighEnergyPhysics`FeynCalc`FeynCalcForm`Private`v__, \
4\\)\\))\\)\\) \[Rule] \
\\(\\(HighEnergyPhysics`FeynCalc`LorentzIndex`LorentzIndex(HighEnergyPhysics`\
FeynCalc`FeynCalcForm`Private`v__)\\)\\)\\)\\).\"\>"}], 
  TraditionalForm]], "Message", "MSG",
 ShowCellLabel->True,
 CellChangeTimes->{3.500970395360722*^9},
 CellTags->"Solve3"],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"x", "\[Rule]", 
     RowBox[{
      FractionBox["1", "4"], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"2", " ", "b"}], "-", "w"}], ")"}]}]}], ",", 
    RowBox[{"y", "\[Rule]", 
     RowBox[{"d", "+", "p"}]}]}], "}"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009703955273952`*^9},
 CellTags->"Solve3",
 CellLabel->"Out[1498]="]
}, Open  ]]
}, Open  ]],

Cell["SP", "ObjectName",
 CellTags->"SP"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SP"],

Cell[TextData[{
 "SP[a, b] denotes a four-dimensional scalar product. SP[a, b] is transformed \
into ScalarProduct[a, b] by FeynCalcInternal. SP[p] is the same as SP[p, p] \
",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{"=", 
     SuperscriptBox["p", "2"]}]}], TraditionalForm]]],
 ")."
}], "Text",
 CellTags->"SP"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Calc",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Calc",
  ButtonNote->"Calc"],
 ", ",
 ButtonBox["ExpandScalarProduct",
  BaseStyle->"AddOnsLink",
  ButtonData:>"ExpandScalarProduct",
  ButtonNote->"ExpandScalarProduct"],
 ", ",
 ButtonBox["ScalarProduct",
  BaseStyle->"AddOnsLink",
  ButtonData:>"ScalarProduct",
  ButtonNote->"ScalarProduct"],
 "."
}], "Text",
 CellTags->"SP"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"SP"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"SP", "[", 
   RowBox[{"p", ",", "q"}], "]"}], " ", "+", " ", 
  RowBox[{"SP", "[", "q", "]"}]}]], "Input",
 CellTags->"SP",
 CellLabel->"In[1499]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox[
    RowBox[{
     FormBox["p",
      TraditionalForm], "\[CenterDot]", 
     FormBox["q",
      TraditionalForm]}],
    TraditionalForm], "+", 
   FormBox[
    SuperscriptBox[
     FormBox["q",
      TraditionalForm], "2"],
    TraditionalForm]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970396368416*^9},
 CellTags->"SP",
 CellLabel->"Out[1499]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SP", "[", 
  RowBox[{
   RowBox[{"p", "-", "q"}], ",", 
   RowBox[{"q", "+", 
    RowBox[{"2", "p"}]}]}], "]"}]], "Input",
 CellTags->"SP",
 CellLabel->"In[1500]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"(", 
   FormBox[
    RowBox[{"p", "-", "q"}],
    TraditionalForm], ")", "\[CenterDot]", "(", 
   FormBox[
    RowBox[{
     RowBox[{"2", " ", "p"}], "+", "q"}],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970396487639*^9},
 CellTags->"SP",
 CellLabel->"Out[1500]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Calc", "[", " ", 
  RowBox[{"SP", "[", 
   RowBox[{
    RowBox[{"p", "-", "q"}], ",", 
    RowBox[{"q", "+", 
     RowBox[{"2", "p"}]}]}], "]"}], " ", "]"}]], "Input",
 CellTags->"SP",
 CellLabel->"In[1501]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    RowBox[{
     FormBox["p",
      TraditionalForm], "\[CenterDot]", 
     FormBox["q",
      TraditionalForm]}]}], "+", 
   RowBox[{"2", " ", 
    SuperscriptBox[
     FormBox["p",
      TraditionalForm], "2"]}], "-", 
   SuperscriptBox[
    FormBox["q",
     TraditionalForm], "2"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970396879381*^9},
 CellTags->"SP",
 CellLabel->"Out[1501]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ExpandScalarProduct", "[", 
  RowBox[{"SP", "[", 
   RowBox[{"p", "-", "q"}], "]"}], "]"}]], "Input",
 CellTags->"SP",
 CellLabel->"In[1502]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox[
   RowBox[{"(", 
    FormBox[
     RowBox[{"p", "-", "q"}],
     TraditionalForm], ")"}], "2"], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970397193116*^9},
 CellTags->"SP",
 CellLabel->"Out[1502]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"SP", "[", 
   RowBox[{"a", ",", "b"}], "]"}], "//", "StandardForm"}]], "Input",
 CellTags->"SP",
 CellLabel->"In[1503]:="],

Cell[BoxData[
 RowBox[{"SP", "[", 
  RowBox[{"a", ",", "b"}], "]"}]], "Output",
 CellChangeTimes->{3.500970397342803*^9},
 CellTags->"SP",
 CellLabel->"Out[1503]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"SP", "[", 
    RowBox[{"a", ",", "b"}], "]"}], "//", "FCI"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"SP",
 CellLabel->"In[1504]:="],

Cell[BoxData[
 RowBox[{"Pair", "[", 
  RowBox[{
   RowBox[{"Momentum", "[", "a", "]"}], ",", 
   RowBox[{"Momentum", "[", "b", "]"}]}], "]"}]], "Output",
 CellChangeTimes->{3.5009703975128937`*^9},
 CellTags->"SP",
 CellLabel->"Out[1504]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"SP", "[", 
     RowBox[{"a", ",", "b"}], "]"}], "//", "FCI"}], "//", "FCE"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"SP",
 CellLabel->"In[1505]:="],

Cell[BoxData[
 RowBox[{"SP", "[", 
  RowBox[{"a", ",", "b"}], "]"}]], "Output",
 CellChangeTimes->{3.500970397660102*^9},
 CellTags->"SP",
 CellLabel->"Out[1505]//StandardForm="]
}, Open  ]]
}, Open  ]],

Cell["SPC", "ObjectName",
 CellTags->"SPC"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SPC"],

Cell["SPC is an abbreviation for ScalarProductCancel.", "Text",
 CellTags->"SPC"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["ScalarProductCancel",
  BaseStyle->"AddOnsLink",
  ButtonData:>"ScalarProductCancel",
  ButtonNote->"ScalarProductCancel"],
 "."
}], "Text",
 CellTags->"SPC"]
}, Open  ]],

Cell["SPD", "ObjectName",
 CellTags->"SPD"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SPD"],

Cell[TextData[{
 "SPD[a, b] denotes a D-dimensional scalar product. SPD[a, b] is transformed \
into ScalarProduct[a, b,Dimension\[Rule]D] by FeynCalcInternal. SPD[p] is the \
same as SPD[p,p] ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{"=", 
     SuperscriptBox["p", "2"]}]}], TraditionalForm]]],
 ")."
}], "Text",
 CellTags->"SPD"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["PD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"PD",
  ButtonNote->"PD"],
 ", ",
 ButtonBox["Calc",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Calc",
  ButtonNote->"Calc"],
 ", ",
 ButtonBox["ExpandScalarProduct",
  BaseStyle->"AddOnsLink",
  ButtonData:>"ExpandScalarProduct",
  ButtonNote->"ExpandScalarProduct"],
 ", ",
 ButtonBox["ScalarProduct",
  BaseStyle->"AddOnsLink",
  ButtonData:>"ScalarProduct",
  ButtonNote->"ScalarProduct"],
 "."
}], "Text",
 CellTags->"SPD"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"SPD"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"SPD", "[", 
   RowBox[{"p", ",", "q"}], "]"}], " ", "+", " ", 
  RowBox[{"SPD", "[", "q", "]"}]}]], "Input",
 CellTags->"SPD",
 CellLabel->"In[1506]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox[
    RowBox[{
     FormBox["p",
      TraditionalForm], "\[CenterDot]", 
     FormBox["q",
      TraditionalForm]}],
    TraditionalForm], "+", 
   FormBox[
    SuperscriptBox[
     FormBox["q",
      TraditionalForm], "2"],
    TraditionalForm]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009703978105507`*^9},
 CellTags->"SPD",
 CellLabel->"Out[1506]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SPD", "[", 
  RowBox[{
   RowBox[{"p", "-", "q"}], ",", 
   RowBox[{"q", "+", 
    RowBox[{"2", "p"}]}]}], "]"}]], "Input",
 CellTags->"SPD",
 CellLabel->"In[1507]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"(", 
   FormBox[
    RowBox[{"p", "-", "q"}],
    TraditionalForm], ")", "\[CenterDot]", "(", 
   FormBox[
    RowBox[{
     RowBox[{"2", " ", "p"}], "+", "q"}],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970398039049*^9},
 CellTags->"SPD",
 CellLabel->"Out[1507]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Calc", "[", " ", 
  RowBox[{"SPD", "[", 
   RowBox[{
    RowBox[{"p", "-", "q"}], ",", 
    RowBox[{"q", "+", 
     RowBox[{"2", "p"}]}]}], "]"}], " ", "]"}]], "Input",
 CellTags->"SPD",
 CellLabel->"In[1508]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    RowBox[{
     FormBox["p",
      TraditionalForm], "\[CenterDot]", 
     FormBox["q",
      TraditionalForm]}]}], "+", 
   RowBox[{"2", " ", 
    SuperscriptBox[
     FormBox["p",
      TraditionalForm], "2"]}], "-", 
   SuperscriptBox[
    FormBox["q",
     TraditionalForm], "2"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.50097039839627*^9},
 CellTags->"SPD",
 CellLabel->"Out[1508]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ExpandScalarProduct", "[", 
  RowBox[{"SPD", "[", 
   RowBox[{"p", "-", "q"}], "]"}], "]"}]], "Input",
 CellTags->"SPD",
 CellLabel->"In[1509]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox[
   RowBox[{"(", 
    FormBox[
     RowBox[{"p", "-", "q"}],
     TraditionalForm], ")"}], "2"], TraditionalForm]], "Output",
 CellChangeTimes->{3.50097039854889*^9},
 CellTags->"SPD",
 CellLabel->"Out[1509]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"SPD", "[", 
   RowBox[{"a", ",", "b"}], "]"}], "//", "StandardForm"}]], "Input",
 CellTags->"SPD",
 CellLabel->"In[1510]:="],

Cell[BoxData[
 RowBox[{"SPD", "[", 
  RowBox[{"a", ",", "b"}], "]"}]], "Output",
 CellChangeTimes->{3.500970398801014*^9},
 CellTags->"SPD",
 CellLabel->"Out[1510]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"SPD", "[", 
    RowBox[{"a", ",", "b"}], "]"}], "//", "FCI"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"SPD",
 CellLabel->"In[1511]:="],

Cell[BoxData[
 RowBox[{"Pair", "[", 
  RowBox[{
   RowBox[{"Momentum", "[", 
    RowBox[{"a", ",", "D"}], "]"}], ",", 
   RowBox[{"Momentum", "[", 
    RowBox[{"b", ",", "D"}], "]"}]}], "]"}]], "Output",
 CellChangeTimes->{3.5009703989777184`*^9},
 CellTags->"SPD",
 CellLabel->"Out[1511]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"SPD", "[", 
     RowBox[{"a", ",", "b"}], "]"}], "//", "FCI"}], "//", "FCE"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"SPD",
 CellLabel->"In[1512]:="],

Cell[BoxData[
 RowBox[{"SPD", "[", 
  RowBox[{"a", ",", "b"}], "]"}]], "Output",
 CellChangeTimes->{3.500970399148987*^9},
 CellTags->"SPD",
 CellLabel->"Out[1512]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"FCE", "[", 
   RowBox[{"ChangeDimension", "[", 
    RowBox[{
     RowBox[{"SP", "[", 
      RowBox[{"p", ",", "q"}], "]"}], ",", " ", "D"}], "]"}], "]"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"SPD",
 CellLabel->"In[1513]:="],

Cell[BoxData[
 RowBox[{"SPD", "[", 
  RowBox[{"p", ",", "q"}], "]"}]], "Output",
 CellChangeTimes->{3.500970399349687*^9},
 CellTags->"SPD",
 CellLabel->"Out[1513]//StandardForm="]
}, Open  ]]
}, Open  ]],

Cell["Spinor", "ObjectName",
 CellTags->"Spinor"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Spinor"],

Cell[TextData[{
 "Spinor[p, m, o] is the head of Dirac spinors. Which of the spinors ",
 StyleBox["u",
  FontSlant->"Italic"],
 ", ",
 StyleBox["v",
  FontSlant->"Italic"],
 ", ",
 Cell[BoxData[
  FormBox[
   OverscriptBox["u", "_"], TraditionalForm]]],
 " ",
 Cell[BoxData[
  FormBox[
   RowBox[{",", " "}], TraditionalForm]]],
 "or ",
 Cell[BoxData[
  FormBox[
   OverscriptBox["v", 
    RowBox[{"_", " "}]], TraditionalForm]]],
 "is understood, depends on the sign of the momentum (p) argument and the \
relative position of DiracSlash[p]: Spinor[sign p, mass] is that spinor which \
yields: sign*mass*Spinor[p, mass] if the Dirac equation is applied (by \
DiracEquation or DiracSimplify).\nThe optional argument o can be used for \
additional degrees of freedom. If no optional argument o is supplied, a 1 is \
subsituted in.\nSpinors of fermions of mass ",
 StyleBox["m",
  FontSlant->"Italic"],
 " are normalized to have square ",
 Cell[BoxData[
  FormBox[
   OverscriptBox["u", "_"], TraditionalForm]]],
 StyleBox["u",
  FontSlant->"Italic"],
 "=2 ",
 StyleBox["m",
  FontSlant->"Italic"],
 " and  ",
 Cell[BoxData[
  FormBox[
   OverscriptBox["v", 
    RowBox[{"_", " "}]], TraditionalForm]]],
 StyleBox["v=",
  FontSlant->"Italic"],
 "-2 ",
 StyleBox["m",
  FontSlant->"Italic"],
 "."
}], "Text",
 CellTags->"Spinor"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FermionSpinSum",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FermionSpinSum",
  ButtonNote->"FermionSpinSum"],
 ", ",
 ButtonBox["DiracSimplify",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracSimplify",
  ButtonNote->"DiracSimplify"],
 ", ",
 ButtonBox["SpinorU",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SpinorU",
  ButtonNote->"SpinorU"],
 ", ",
 ButtonBox["SpinorV",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SpinorV",
  ButtonNote->"SpinorV"],
 ", ",
 ButtonBox["SpinorUBar",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SpinorUBar",
  ButtonNote->"SpinorUBar"],
 ", ",
 ButtonBox["SpinorVBar",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SpinorVBar",
  ButtonNote->"SpinorVBar"],
 "."
}], "Text",
 CellTags->"Spinor"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Spinor"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Spinor", "[", "p", "]"}]], "Input",
 CellTags->"Spinor",
 CellLabel->"In[1514]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"\[CurlyPhi]", "(", 
   FormBox["p",
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.50097039951775*^9},
 CellTags->"Spinor",
 CellLabel->"Out[1514]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Spinor", "[", 
  RowBox[{"p", ",", "m"}], "]"}]], "Input",
 CellTags->"Spinor",
 CellLabel->"In[1515]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"\[CurlyPhi]", "(", 
   FormBox["p",
    TraditionalForm], ",", "m", ")"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009703996909103`*^9},
 CellTags->"Spinor",
 CellLabel->"Out[1515]="]
}, Open  ]],

Cell["\<\
FeynCalc uses covariant normalization (as opposed to e.g. the normalization \
used in Bjorken&Drell).\
\>", "Text",
 CellTags->"Spinor"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Spinor", "[", 
    RowBox[{"p", ",", "m"}], "]"}], ".", 
   RowBox[{"Spinor", "[", 
    RowBox[{"p", ",", "m"}], "]"}]}], "//", "DiracSimplify"}]], "Input",
 CellTags->"Spinor",
 CellLabel->"In[1516]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"2", " ", "m"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970399865406*^9},
 CellTags->"Spinor",
 CellLabel->"Out[1516]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracSimplify", "[", 
  RowBox[{
   RowBox[{"Spinor", "[", 
    RowBox[{
     RowBox[{"-", "p"}], ",", "m"}], "]"}], ".", 
   RowBox[{"DiracSlash", "[", "p", "]"}]}], "]"}]], "Input",
 CellTags->"Spinor",
 CellLabel->"In[1517]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", "m"}], " ", 
   RowBox[{"\[CurlyPhi]", "(", 
    FormBox[
     RowBox[{"-", "p"}],
     TraditionalForm], ",", "m", ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970400082292*^9},
 CellTags->"Spinor",
 CellLabel->"Out[1517]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Spinor", "[", "p", "]"}], "//", "StandardForm"}]], "Input",
 CellTags->"Spinor",
 CellLabel->"In[1518]:="],

Cell[BoxData[
 RowBox[{"Spinor", "[", 
  RowBox[{
   RowBox[{"Momentum", "[", "p", "]"}], ",", "0", ",", "1"}], "]"}]], "Output",\

 CellChangeTimes->{3.500970400229981*^9},
 CellTags->"Spinor",
 CellLabel->"Out[1518]//StandardForm="]
}, Open  ]],

Cell["\<\
By convention, ChangeDimension does not operate on momenta in Spinor's (but \
on e.g.  DiracSlash[Momentum[p]]). \
\>", "Text",
 CellTags->"Spinor"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"ChangeDimension", "[", 
   RowBox[{
    RowBox[{"Spinor", "[", "p", "]"}], ",", "D"}], "]"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"Spinor",
 CellLabel->"In[1519]:="],

Cell[BoxData[
 RowBox[{"Spinor", "[", 
  RowBox[{
   RowBox[{"Momentum", "[", "p", "]"}], ",", "0", ",", "1"}], "]"}]], "Output",\

 CellChangeTimes->{3.500970400523074*^9},
 CellTags->"Spinor",
 CellLabel->"Out[1519]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Spinor", "[", 
   RowBox[{"p", ",", "m"}], "]"}], "//", "StandardForm"}]], "Input",
 CellTags->"Spinor",
 CellLabel->"In[1520]:="],

Cell[BoxData[
 RowBox[{"Spinor", "[", 
  RowBox[{
   RowBox[{"Momentum", "[", "p", "]"}], ",", "m", ",", "1"}], "]"}]], "Output",\

 CellChangeTimes->{3.500970400939363*^9},
 CellTags->"Spinor",
 CellLabel->"Out[1520]//StandardForm="]
}, Open  ]],

Cell["SmallVariable's are discarded by Spinor.", "Text",
 CellTags->"Spinor"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Spinor", "[", 
   RowBox[{"p", ",", 
    RowBox[{"SmallVariable", "[", "m", "]"}]}], "]"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"Spinor",
 CellLabel->"In[1521]:="],

Cell[BoxData[
 RowBox[{"Spinor", "[", 
  RowBox[{
   RowBox[{"Momentum", "[", "p", "]"}], ",", "0", ",", "1"}], "]"}]], "Output",\

 CellChangeTimes->{3.500970401110485*^9},
 CellTags->"Spinor",
 CellLabel->"Out[1521]//StandardForm="]
}, Open  ]]
}, Open  ]],

Cell["SpinorCollect", "ObjectName",
 CellTags->"SpinorCollect"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SpinorCollect"],

Cell["\<\
SpinorCollect is an option for FermionSpinSum. If set to False the  argument \
of FermionSpinSum has to be already collected w.r.t. Spinor.\
\>", "Text",
 CellTags->"SpinorCollect"],

Cell[TextData[{
 "See also:  ",
 ButtonBox["FermionSpinSum",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FermionSpinSum",
  ButtonNote->"FermionSpinSum"],
 ", ",
 ButtonBox["Spinor",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Spinor",
  ButtonNote->"Spinor"],
 "."
}], "Text",
 CellTags->"SpinorCollect"]
}, Open  ]],

Cell["SpinorUBar", "ObjectName",
 CellTags->"SpinorUBar"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SpinorUBar"],

Cell[TextData[{
 "SpinorUBar[p, m] denotes a ",
 Cell[BoxData[
  FormBox[
   OverscriptBox["u", "_"], TraditionalForm]]],
 "-spinor."
}], "Text",
 CellTags->"SpinorUBar"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Spinor",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Spinor",
  ButtonNote->"Spinor"],
 ", ",
 ButtonBox["SpinorU",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SpinorU",
  ButtonNote->"SpinorU"],
 ", ",
 ButtonBox["SpinorV",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SpinorV",
  ButtonNote->"SpinorV"],
 ", ",
 ButtonBox["SpinorVBar",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SpinorVBar",
  ButtonNote->"SpinorVBar"],
 "."
}], "Text",
 CellTags->"SpinorUBar"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"SpinorUBar"],

Cell["One argument only assumes a massless spinor.", "Text",
 CellTags->"SpinorUBar"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SpinorUBar", "[", "p", "]"}]], "Input",
 CellTags->"SpinorUBar",
 CellLabel->"In[1522]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox[
    OverscriptBox["\<\"u\"\>", "_"],
    TraditionalForm], "(", "p", ")"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009704011832*^9},
 CellTags->"SpinorUBar",
 CellLabel->"Out[1522]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SpinorUBar", "[", 
  RowBox[{"p", ",", "m"}], "]"}]], "Input",
 CellTags->"SpinorUBar",
 CellLabel->"In[1523]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox[
    OverscriptBox["\<\"u\"\>", "_"],
    TraditionalForm], "(", "p", ",", "m", ")"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009704012567577`*^9},
 CellTags->"SpinorUBar",
 CellLabel->"Out[1523]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"SpinorUBar", "[", 
   RowBox[{"p", ",", "m"}], "]"}], "//", "StandardForm"}]], "Input",
 CellTags->"SpinorUBar",
 CellLabel->"In[1524]:="],

Cell[BoxData[
 RowBox[{"SpinorUBar", "[", 
  RowBox[{"p", ",", "m"}], "]"}]], "Output",
 CellChangeTimes->{3.500970401451523*^9},
 CellTags->"SpinorUBar",
 CellLabel->"Out[1524]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"SpinorUBar", "[", 
    RowBox[{"p", ",", "m"}], "]"}], "//", "FCI"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"SpinorUBar",
 CellLabel->"In[1525]:="],

Cell[BoxData[
 RowBox[{"Spinor", "[", 
  RowBox[{
   RowBox[{"Momentum", "[", "p", "]"}], ",", "m", ",", "1"}], "]"}]], "Output",\

 CellChangeTimes->{3.5009704016268663`*^9},
 CellTags->"SpinorUBar",
 CellLabel->"Out[1525]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"SpinorUBar", "[", 
     RowBox[{"p", ",", "m"}], "]"}], "//", "FCI"}], "//", "FCE"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"SpinorUBar",
 CellLabel->"In[1526]:="],

Cell[BoxData[
 RowBox[{"Spinor", "[", 
  RowBox[{
   RowBox[{"Momentum", "[", "p", "]"}], ",", "m", ",", "1"}], "]"}]], "Output",\

 CellChangeTimes->{3.5009704018124027`*^9},
 CellTags->"SpinorUBar",
 CellLabel->"Out[1526]//StandardForm="]
}, Open  ]]
}, Open  ]],

Cell["SpinorU", "ObjectName",
 CellTags->"SpinorU"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SpinorU"],

Cell["SpinorU[p, m, optarg] denotes a u-spinor.", "Text",
 CellTags->"SpinorU"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Spinor",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Spinor",
  ButtonNote->"Spinor"],
 ", ",
 ButtonBox["SpinorUBar",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SpinorUBar",
  ButtonNote->"SpinorUBar"],
 ", ",
 ButtonBox["SpinorV",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SpinorV",
  ButtonNote->"SpinorV"],
 ", ",
 ButtonBox["SpinorVBar",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SpinorVBar",
  ButtonNote->"SpinorVBar"],
 "."
}], "Text",
 CellTags->"SpinorU"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"SpinorU"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SpinorU", "[", "p", "]"}]], "Input",
 CellTags->"SpinorU",
 CellLabel->"In[1527]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"u", "(", "p", ")"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970401980245*^9},
 CellTags->"SpinorU",
 CellLabel->"Out[1527]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SpinorU", "[", 
  RowBox[{"p", ",", "m"}], "]"}]], "Input",
 CellTags->"SpinorU",
 CellLabel->"In[1528]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"u", "(", "p", ",", "m", ")"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970402293388*^9},
 CellTags->"SpinorU",
 CellLabel->"Out[1528]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"SpinorU", "[", 
   RowBox[{"p", ",", "m"}], "]"}], "//", "StandardForm"}]], "Input",
 CellTags->"SpinorU",
 CellLabel->"In[1529]:="],

Cell[BoxData[
 RowBox[{"SpinorU", "[", 
  RowBox[{"p", ",", "m"}], "]"}]], "Output",
 CellChangeTimes->{3.500970402538982*^9},
 CellTags->"SpinorU",
 CellLabel->"Out[1529]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"SpinorU", "[", 
    RowBox[{"p", ",", "m"}], "]"}], "//", "FCI"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"SpinorU",
 CellLabel->"In[1530]:="],

Cell[BoxData[
 RowBox[{"Spinor", "[", 
  RowBox[{
   RowBox[{"Momentum", "[", "p", "]"}], ",", "m", ",", "1"}], "]"}]], "Output",\

 CellChangeTimes->{3.500970402896351*^9},
 CellTags->"SpinorU",
 CellLabel->"Out[1530]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"SpinorU", "[", 
     RowBox[{"p", ",", "m"}], "]"}], "//", "FCI"}], "//", "FCE"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"SpinorU",
 CellLabel->"In[1531]:="],

Cell[BoxData[
 RowBox[{"Spinor", "[", 
  RowBox[{
   RowBox[{"Momentum", "[", "p", "]"}], ",", "m", ",", "1"}], "]"}]], "Output",\

 CellChangeTimes->{3.500970403065014*^9},
 CellTags->"SpinorU",
 CellLabel->"Out[1531]//StandardForm="]
}, Open  ]]
}, Open  ]],

Cell["SpinorVBar", "ObjectName",
 CellTags->"SpinorVBar"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SpinorVBar"],

Cell[TextData[{
 "SpinorVBar[p, m] denotes a ",
 Cell[BoxData[
  FormBox[
   OverscriptBox["v", "_"], TraditionalForm]]],
 "(p,m)-spinor."
}], "Text",
 CellTags->"SpinorVBar"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Spinor",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Spinor",
  ButtonNote->"Spinor"],
 ", ",
 ButtonBox["SpinorU",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SpinorU",
  ButtonNote->"SpinorU"],
 ", ",
 ButtonBox["SpinorV",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SpinorV",
  ButtonNote->"SpinorV"],
 ", ",
 ButtonBox["SpinorUBar",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SpinorUBar",
  ButtonNote->"SpinorUBar"],
 "."
}], "Text",
 CellTags->"SpinorVBar"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"SpinorVBar"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SpinorVBar", "[", "p", "]"}]], "Input",
 CellTags->"SpinorVBar",
 CellLabel->"In[1532]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox[
    OverscriptBox["\<\"v\"\>", "_"],
    TraditionalForm], "(", "p", ")"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009704032866583`*^9},
 CellTags->"SpinorVBar",
 CellLabel->"Out[1532]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SpinorVBar", "[", 
  RowBox[{"p", ",", "m"}], "]"}]], "Input",
 CellTags->"SpinorVBar",
 CellLabel->"In[1533]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox[
    OverscriptBox["\<\"v\"\>", "_"],
    TraditionalForm], "(", "p", ",", "m", ")"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009704034948874`*^9},
 CellTags->"SpinorVBar",
 CellLabel->"Out[1533]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"SpinorVBar", "[", 
   RowBox[{"p", ",", "m"}], "]"}], "//", "StandardForm"}]], "Input",
 CellTags->"SpinorVBar",
 CellLabel->"In[1534]:="],

Cell[BoxData[
 RowBox[{"SpinorVBar", "[", 
  RowBox[{"p", ",", "m"}], "]"}]], "Output",
 CellChangeTimes->{3.500970403792244*^9},
 CellTags->"SpinorVBar",
 CellLabel->"Out[1534]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"SpinorVBar", "[", 
    RowBox[{"p", ",", "m"}], "]"}], "//", "FCI"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"SpinorVBar",
 CellLabel->"In[1535]:="],

Cell[BoxData[
 RowBox[{"Spinor", "[", 
  RowBox[{
   RowBox[{"-", 
    RowBox[{"Momentum", "[", "p", "]"}]}], ",", "m", ",", "1"}], 
  "]"}]], "Output",
 CellChangeTimes->{3.500970403960463*^9},
 CellTags->"SpinorVBar",
 CellLabel->"Out[1535]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"SpinorVBar", "[", 
     RowBox[{"p", ",", "m"}], "]"}], "//", "FCI"}], "//", "FCE"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"SpinorVBar",
 CellLabel->"In[1536]:="],

Cell[BoxData[
 RowBox[{"Spinor", "[", 
  RowBox[{
   RowBox[{"-", 
    RowBox[{"Momentum", "[", "p", "]"}]}], ",", "m", ",", "1"}], 
  "]"}]], "Output",
 CellChangeTimes->{3.500970404152513*^9},
 CellTags->"SpinorVBar",
 CellLabel->"Out[1536]//StandardForm="]
}, Open  ]]
}, Open  ]],

Cell["SpinorV", "ObjectName",
 CellTags->"SpinorV"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SpinorV"],

Cell["SpinorV[p, m, optarg] denotes a v-spinor.", "Text",
 CellTags->"SpinorV"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Spinor",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Spinor",
  ButtonNote->"Spinor"],
 ", ",
 ButtonBox["SpinorUBar",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SpinorUBar",
  ButtonNote->"SpinorUBar"],
 ", ",
 ButtonBox["SpinorU",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SpinorU",
  ButtonNote->"SpinorU"],
 ", ",
 ButtonBox["SpinorVBar",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SpinorVBar",
  ButtonNote->"SpinorVBar"],
 "."
}], "Text",
 CellTags->"SpinorV"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"SpinorV"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SpinorV", "[", "p", "]"}]], "Input",
 CellTags->"SpinorV",
 CellLabel->"In[1537]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"v", "(", "p", ")"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970404299901*^9},
 CellTags->"SpinorV",
 CellLabel->"Out[1537]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SpinorV", "[", 
  RowBox[{"p", ",", "m"}], "]"}]], "Input",
 CellTags->"SpinorV",
 CellLabel->"In[1538]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"v", "(", "p", ",", "m", ")"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970404440208*^9},
 CellTags->"SpinorV",
 CellLabel->"Out[1538]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"SpinorV", "[", 
   RowBox[{"p", ",", "m"}], "]"}], "//", "StandardForm"}]], "Input",
 CellTags->"SpinorV",
 CellLabel->"In[1539]:="],

Cell[BoxData[
 RowBox[{"SpinorV", "[", 
  RowBox[{"p", ",", "m"}], "]"}]], "Output",
 CellChangeTimes->{3.500970404583723*^9},
 CellTags->"SpinorV",
 CellLabel->"Out[1539]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"SpinorV", "[", 
    RowBox[{"p", ",", "m"}], "]"}], "//", "FCI"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"SpinorV",
 CellLabel->"In[1540]:="],

Cell[BoxData[
 RowBox[{"Spinor", "[", 
  RowBox[{
   RowBox[{"-", 
    RowBox[{"Momentum", "[", "p", "]"}]}], ",", "m", ",", "1"}], 
  "]"}]], "Output",
 CellChangeTimes->{3.500970404655121*^9},
 CellTags->"SpinorV",
 CellLabel->"Out[1540]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"SpinorV", "[", 
     RowBox[{"p", ",", "m"}], "]"}], "//", "FCI"}], "//", "FCE"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"SpinorV",
 CellLabel->"In[1541]:="],

Cell[BoxData[
 RowBox[{"Spinor", "[", 
  RowBox[{
   RowBox[{"-", 
    RowBox[{"Momentum", "[", "p", "]"}]}], ",", "m", ",", "1"}], 
  "]"}]], "Output",
 CellChangeTimes->{3.5009704047618437`*^9},
 CellTags->"SpinorV",
 CellLabel->"Out[1541]//StandardForm="]
}, Open  ]]
}, Open  ]],

Cell["SpinPolarizationSum", "ObjectName",
 CellTags->"SpinPolarizationSum"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SpinPolarizationSum"],

Cell["\<\
SpinPolarizationSum is an option for FermionSpinSum. The set (pure) function \
acts on the usual spin sum.\
\>", "Text",
 CellTags->"SpinPolarizationSum"],

Cell[TextData[{
 "See also:  ",
 ButtonBox["FermionSpinSum",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FermionSpinSum",
  ButtonNote->"FermionSpinSum"],
 "."
}], "Text",
 CellTags->"SpinPolarizationSum"]
}, Open  ]],

Cell["SPL", "ObjectName",
 CellTags->"SPL"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SPL"],

Cell["SPL is an abbreviation for SimplifyPolyLog.", "Text",
 CellTags->"SPL"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["SimplifyPolyLog",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SimplifyPolyLog",
  ButtonNote->"SimplifyPolyLog"],
 "."
}], "Text",
 CellTags->"SPL"]
}, Open  ]],

Cell["SplittingFunction", "ObjectName",
 CellTags->"SplittingFunction"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SplittingFunction"],

Cell[TextData[{
 "SplittingFunction[pxy] is a database of splitting functions in the ",
 Cell[BoxData[
  FormBox[
   OverscriptBox["MS", "_"], TraditionalForm]]],
 " scheme."
}], "Text",
 CellTags->"SplittingFunction"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "SplittingFunction", "]"}]], "Input",
 CellTags->"SplittingFunction",
 CellLabel->"In[1542]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"Polarization", "\[Rule]", "1"}], "}"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.50097040487084*^9},
 CellTags->"SplittingFunction",
 CellLabel->"Out[1542]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["AnomalousDimension",
  BaseStyle->"AddOnsLink",
  ButtonData:>"AnomalousDimension",
  ButtonNote->"AnomalousDimension"],
 "."
}], "Text",
 CellTags->"SplittingFunction"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"SplittingFunction"],

Cell["Unpolarized case:", "Text",
 CellTags->"SplittingFunction"],

Cell[BoxData[
 RowBox[{
  RowBox[{"SetOptions", "[", 
   RowBox[{"SplittingFunction", ",", 
    RowBox[{"Polarization", "\[Rule]", "0"}]}], "]"}], ";"}]], "Input",
 CellTags->"SplittingFunction",
 CellLabel->"In[1543]:="],

Cell["\<\
In general the argument should be a string, but if the variables Pqq, etc. \
have no value, you can omit the \"\".\
\>", "Text",
 CellTags->"SplittingFunction"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SplittingFunction", "[", "Pqq", "]"}]], "Input",
 CellTags->"SplittingFunction",
 CellLabel->"In[1544]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["C", "F"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"6", " ", 
      RowBox[{"\[Delta]", "(", 
       FormBox[
        RowBox[{"1", "-", "x"}],
        TraditionalForm], ")"}]}], "-", 
     RowBox[{"4", " ", "x"}], "+", 
     RowBox[{"8", " ", 
      SubscriptBox[
       RowBox[{"(", 
        FractionBox["1", 
         RowBox[{"1", "-", "x"}]], ")"}], "+"]}], "-", "4"}], ")"}]}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970405009254*^9},
 CellTags->"SplittingFunction",
 CellLabel->"Out[1544]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SplittingFunction", "[", "Pqg", "]"}]], "Input",
 CellTags->"SplittingFunction",
 CellLabel->"In[1545]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{"16", " ", 
      SuperscriptBox["x", "2"]}], "-", 
     RowBox[{"16", " ", "x"}], "+", "8"}], ")"}], " ", 
   SubscriptBox["T", "f"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.50097040511758*^9},
 CellTags->"SplittingFunction",
 CellLabel->"Out[1545]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SplittingFunction", "[", "Pgq", "]"}]], "Input",
 CellTags->"SplittingFunction",
 CellLabel->"In[1546]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{"4", " ", "x"}], "+", 
     FractionBox["8", "x"], "-", "8"}], ")"}], " ", 
   SubscriptBox["C", "F"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009704052247753`*^9},
 CellTags->"SplittingFunction",
 CellLabel->"Out[1546]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SplittingFunction", "[", "Pgg", "]"}]], "Input",
 CellTags->"SplittingFunction",
 CellLabel->"In[1547]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"8", " ", 
    SubscriptBox["C", "A"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", 
       SuperscriptBox["x", "2"]}], "+", 
      RowBox[{
       FractionBox["11", "12"], " ", 
       RowBox[{"\[Delta]", "(", 
        FormBox[
         RowBox[{"1", "-", "x"}],
         TraditionalForm], ")"}]}], "+", "x", "+", 
      SubscriptBox[
       RowBox[{"(", 
        FractionBox["1", 
         RowBox[{"1", "-", "x"}]], ")"}], "+"], "+", 
      FractionBox["1", "x"], "-", "2"}], ")"}]}], "-", 
   RowBox[{
    FractionBox["8", "3"], " ", 
    SubscriptBox["N", "f"], " ", 
    SubscriptBox["T", "f"], " ", 
    RowBox[{"\[Delta]", "(", 
     FormBox[
      RowBox[{"1", "-", "x"}],
      TraditionalForm], ")"}]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970405354897*^9},
 CellTags->"SplittingFunction",
 CellLabel->"Out[1547]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SplittingFunction", "[", "aqq", "]"}]], "Input",
 CellTags->"SplittingFunction",
 CellLabel->"In[1548]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["C", "F"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{"7", "-", 
        RowBox[{"4", " ", 
         RowBox[{"\[Zeta]", "(", "2", ")"}]}]}], ")"}], " ", 
      RowBox[{"\[Delta]", "(", 
       FormBox[
        RowBox[{"1", "-", "x"}],
        TraditionalForm], ")"}]}], "+", 
     RowBox[{"2", " ", "x"}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"2", " ", "x"}], "+", "2"}], ")"}], " ", 
      RowBox[{"log", "(", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"1", "-", "x"}], ")"}], " ", "x"}], ")"}]}], "-", 
     RowBox[{"4", " ", 
      RowBox[{"(", 
       RowBox[{
        FractionBox[
         RowBox[{"log", "(", "x", ")"}], 
         RowBox[{"1", "-", "x"}]], "+", 
        SubscriptBox[
         RowBox[{"(", 
          FractionBox[
           RowBox[{"log", "(", 
            RowBox[{"1", "-", "x"}], ")"}], 
           RowBox[{"1", "-", "x"}]], ")"}], "+"]}], ")"}]}], "-", "4"}], 
    ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970405523484*^9},
 CellTags->"SplittingFunction",
 CellLabel->"Out[1548]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SplittingFunction", "[", "agq", "]"}]], "Input",
 CellTags->"SplittingFunction",
 CellLabel->"In[1549]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["C", "F"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      RowBox[{"2", " ", "x"}]}], "-", 
     FractionBox["4", "x"], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", 
         RowBox[{"2", " ", "x"}]}], "-", 
        FractionBox["4", "x"], "+", "4"}], ")"}], " ", 
      RowBox[{"log", "(", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"1", "-", "x"}], ")"}], " ", "x"}], ")"}]}], "+", "2"}], 
    ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970405876767*^9},
 CellTags->"SplittingFunction",
 CellLabel->"Out[1549]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SplittingFunction", "[", "aqg", "]"}]], "Input",
 CellTags->"SplittingFunction",
 CellLabel->"In[1550]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["T", "f"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", 
         RowBox[{"8", " ", 
          SuperscriptBox["x", "2"]}]}], "+", 
        RowBox[{"8", " ", "x"}], "-", "4"}], ")"}], " ", 
      RowBox[{"log", "(", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"1", "-", "x"}], ")"}], " ", "x"}], ")"}]}], "-", "4"}], 
    ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970406044633*^9},
 CellTags->"SplittingFunction",
 CellLabel->"Out[1550]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"SplittingFunction", "[", "agg", "]"}]], "Input",
 CellTags->"SplittingFunction",
 CellLabel->"In[1551]:="],

Cell["Polarized case:", "Text",
 CellTags->"SplittingFunction"],

Cell[BoxData[
 RowBox[{
  RowBox[{"SetOptions", "[", 
   RowBox[{"SplittingFunction", ",", 
    RowBox[{"Polarization", "\[Rule]", "1"}]}], "]"}], ";"}]], "Input",
 CellTags->"SplittingFunction",
 CellLabel->"In[1552]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SplittingFunction", "[", "Pqq", "]"}]], "Input",
 CellTags->"SplittingFunction",
 CellLabel->"In[1553]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["C", "F"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"6", " ", 
      RowBox[{"\[Delta]", "(", 
       FormBox[
        RowBox[{"1", "-", "x"}],
        TraditionalForm], ")"}]}], "-", 
     RowBox[{"4", " ", "x"}], "+", 
     RowBox[{"8", " ", 
      SubscriptBox[
       RowBox[{"(", 
        FractionBox["1", 
         RowBox[{"1", "-", "x"}]], ")"}], "+"]}], "-", "4"}], ")"}]}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970406243676*^9},
 CellTags->"SplittingFunction",
 CellLabel->"Out[1553]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SplittingFunction", "[", "Pqg", "]"}]], "Input",
 CellTags->"SplittingFunction",
 CellLabel->"In[1554]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{"16", " ", "x"}], "-", "8"}], ")"}], " ", 
   SubscriptBox["T", "f"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970406323769*^9},
 CellTags->"SplittingFunction",
 CellLabel->"Out[1554]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SplittingFunction", "[", "Pgq", "]"}]], "Input",
 CellTags->"SplittingFunction",
 CellLabel->"In[1555]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"(", 
    RowBox[{"8", "-", 
     RowBox[{"4", " ", "x"}]}], ")"}], " ", 
   SubscriptBox["C", "F"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009704064581738`*^9},
 CellTags->"SplittingFunction",
 CellLabel->"Out[1555]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SplittingFunction", "[", "Pgg", "]"}]], "Input",
 CellTags->"SplittingFunction",
 CellLabel->"In[1556]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox["C", "A"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       FractionBox["22", "3"], " ", 
       RowBox[{"\[Delta]", "(", 
        FormBox[
         RowBox[{"1", "-", "x"}],
         TraditionalForm], ")"}]}], "-", 
      RowBox[{"16", " ", "x"}], "+", 
      RowBox[{"8", " ", 
       SubscriptBox[
        RowBox[{"(", 
         FractionBox["1", 
          RowBox[{"1", "-", "x"}]], ")"}], "+"]}], "+", "8"}], ")"}]}], "-", 
   RowBox[{
    FractionBox["8", "3"], " ", 
    SubscriptBox["N", "f"], " ", 
    SubscriptBox["T", "f"], " ", 
    RowBox[{"\[Delta]", "(", 
     FormBox[
      RowBox[{"1", "-", "x"}],
      TraditionalForm], ")"}]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009704065889673`*^9},
 CellTags->"SplittingFunction",
 CellLabel->"Out[1556]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SplittingFunction", "[", "aqq", "]"}]], "Input",
 CellTags->"SplittingFunction",
 CellLabel->"In[1557]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["C", "F"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{"7", "-", 
        RowBox[{"4", " ", 
         RowBox[{"\[Zeta]", "(", "2", ")"}]}]}], ")"}], " ", 
      RowBox[{"\[Delta]", "(", 
       FormBox[
        RowBox[{"1", "-", "x"}],
        TraditionalForm], ")"}]}], "+", 
     RowBox[{"8", " ", 
      RowBox[{"(", 
       RowBox[{"1", "-", "x"}], ")"}]}], "+", 
     RowBox[{"2", " ", "x"}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"2", " ", "x"}], "+", "2"}], ")"}], " ", 
      RowBox[{"log", "(", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"1", "-", "x"}], ")"}], " ", "x"}], ")"}]}], "-", 
     RowBox[{"4", " ", 
      SubscriptBox[
       RowBox[{"(", 
        FractionBox["1", 
         RowBox[{"1", "-", "x"}]], ")"}], "+"], " ", 
      RowBox[{"log", "(", "x", ")"}]}], "-", 
     RowBox[{"4", " ", 
      SubscriptBox[
       RowBox[{"(", 
        FractionBox[
         RowBox[{"log", "(", 
          RowBox[{"1", "-", "x"}], ")"}], 
         RowBox[{"1", "-", "x"}]], ")"}], "+"]}], "-", "4"}], ")"}]}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970406868925*^9},
 CellTags->"SplittingFunction",
 CellLabel->"Out[1557]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SplittingFunction", "[", "agq", "]"}]], "Input",
 CellTags->"SplittingFunction",
 CellLabel->"In[1558]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["C", "F"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      RowBox[{"4", " ", "x"}]}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"2", " ", "x"}], "-", "4"}], ")"}], " ", 
      RowBox[{"log", "(", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"1", "-", "x"}], ")"}], " ", "x"}], ")"}]}], "+", "2"}], 
    ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970407045814*^9},
 CellTags->"SplittingFunction",
 CellLabel->"Out[1558]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SplittingFunction", "[", "agqd", "]"}]], "Input",
 CellTags->"SplittingFunction",
 CellLabel->"In[1559]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["C", "F"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"2", " ", "x"}], "-", "4"}], ")"}], " ", 
      RowBox[{"log", "(", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"1", "-", "x"}], ")"}], " ", "x"}], ")"}]}], "-", "2"}], 
    ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970407189486*^9},
 CellTags->"SplittingFunction",
 CellLabel->"Out[1559]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SplittingFunction", "[", "aqg", "]"}]], "Input",
 CellTags->"SplittingFunction",
 CellLabel->"In[1560]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["T", "f"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{"4", "-", 
        RowBox[{"8", " ", "x"}]}], ")"}], " ", 
      RowBox[{"log", "(", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"1", "-", "x"}], ")"}], " ", "x"}], ")"}]}], "-", "4"}], 
    ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970407359391*^9},
 CellTags->"SplittingFunction",
 CellLabel->"Out[1560]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SplittingFunction", "[", "aqgd", "]"}]], "Input",
 CellTags->"SplittingFunction",
 CellLabel->"In[1561]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["T", "f"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{"4", "-", 
        RowBox[{"8", " ", "x"}]}], ")"}], " ", 
      RowBox[{"log", "(", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"1", "-", "x"}], ")"}], " ", "x"}], ")"}]}], "-", "4"}], 
    ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009704075241203`*^9},
 CellTags->"SplittingFunction",
 CellLabel->"Out[1561]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SplittingFunction", "[", "agg", "]"}]], "Input",
 CellTags->"SplittingFunction",
 CellLabel->"In[1562]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox["C", "A"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         FractionBox["67", "9"], "-", 
         RowBox[{"4", " ", 
          RowBox[{"\[Zeta]", "(", "2", ")"}]}]}], ")"}], " ", 
       RowBox[{"\[Delta]", "(", 
        FormBox[
         RowBox[{"1", "-", "x"}],
         TraditionalForm], ")"}]}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"8", " ", "x"}], "-", "4"}], ")"}], " ", 
       RowBox[{"log", "(", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{"1", "-", "x"}], ")"}], " ", "x"}], ")"}]}], "-", 
      RowBox[{"4", " ", 
       RowBox[{"(", 
        RowBox[{
         FractionBox[
          RowBox[{"log", "(", "x", ")"}], 
          RowBox[{"1", "-", "x"}]], "+", 
         SubscriptBox[
          RowBox[{"(", 
           FractionBox[
            RowBox[{"log", "(", 
             RowBox[{"1", "-", "x"}], ")"}], 
            RowBox[{"1", "-", "x"}]], ")"}], "+"]}], ")"}]}], "+", "2"}], 
     ")"}]}], "-", 
   RowBox[{
    FractionBox["20", "9"], " ", 
    SubscriptBox["T", "f"], " ", 
    RowBox[{"\[Delta]", "(", 
     FormBox[
      RowBox[{"1", "-", "x"}],
      TraditionalForm], ")"}]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970407899641*^9},
 CellTags->"SplittingFunction",
 CellLabel->"Out[1562]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SplittingFunction", "[", "aggd", "]"}]], "Input",
 CellTags->"SplittingFunction",
 CellLabel->"In[1563]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox["C", "A"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         FractionBox["67", "9"], "-", 
         RowBox[{"4", " ", 
          RowBox[{"\[Zeta]", "(", "2", ")"}]}]}], ")"}], " ", 
       RowBox[{"\[Delta]", "(", 
        FormBox[
         RowBox[{"1", "-", "x"}],
         TraditionalForm], ")"}]}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"8", " ", "x"}], "-", "4"}], ")"}], " ", 
       RowBox[{"log", "(", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{"1", "-", "x"}], ")"}], " ", "x"}], ")"}]}], "-", 
      RowBox[{"4", " ", 
       RowBox[{"(", 
        RowBox[{
         FractionBox[
          RowBox[{"log", "(", "x", ")"}], 
          RowBox[{"1", "-", "x"}]], "+", 
         SubscriptBox[
          RowBox[{"(", 
           FractionBox[
            RowBox[{"log", "(", 
             RowBox[{"1", "-", "x"}], ")"}], 
            RowBox[{"1", "-", "x"}]], ")"}], "+"]}], ")"}]}], "+", "2"}], 
     ")"}]}], "-", 
   RowBox[{
    FractionBox["20", "9"], " ", 
    SubscriptBox["T", "f"], " ", 
    RowBox[{"\[Delta]", "(", 
     FormBox[
      RowBox[{"1", "-", "x"}],
      TraditionalForm], ")"}]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970408289331*^9},
 CellTags->"SplittingFunction",
 CellLabel->"Out[1563]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SplittingFunction", "[", "PQQS", "]"}]], "Input",
 CellTags->"SplittingFunction",
 CellLabel->"In[1564]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["C", "F"], " ", 
   SubscriptBox["T", "f"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"16", " ", 
      RowBox[{"(", 
       RowBox[{"1", "-", "x"}], ")"}]}], "-", 
     RowBox[{"16", " ", 
      RowBox[{"(", 
       RowBox[{"x", "+", "1"}], ")"}], " ", 
      RowBox[{
       SuperscriptBox["log", "2"], "(", "x", ")"}]}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"48", " ", "x"}], "-", "16"}], ")"}], " ", 
      RowBox[{"log", "(", "x", ")"}]}]}], ")"}]}], TraditionalForm]], "Output",\

 CellChangeTimes->{3.5009704084644613`*^9},
 CellTags->"SplittingFunction",
 CellLabel->"Out[1564]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SplittingFunction", "[", "PQQNS", "]"}]], "Input",
 CellTags->"SplittingFunction",
 CellLabel->"In[1565]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    RowBox[{"-", "8"}], " ", 
    SubscriptBox["C", "F"], " ", 
    RowBox[{"(", 
     RowBox[{
      SubscriptBox["C", "F"], "-", 
      FractionBox[
       SubscriptBox["C", "A"], "2"]}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      FractionBox[
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox["x", "2"], "+", "1"}], ")"}], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"-", 
           RowBox[{"2", " ", 
            RowBox[{"\[Zeta]", "(", "2", ")"}]}]}], "-", 
          RowBox[{"4", " ", 
           TemplateBox[{"2",RowBox[{"-", "x"}]},
            "PolyLog"]}], "+", 
          RowBox[{
           SuperscriptBox["log", "2"], "(", "x", ")"}], "-", 
          RowBox[{"4", " ", 
           RowBox[{"log", "(", 
            RowBox[{"x", "+", "1"}], ")"}], " ", 
           RowBox[{"log", "(", "x", ")"}]}]}], ")"}]}], 
       RowBox[{"x", "+", "1"}]], "+", 
      RowBox[{"4", " ", 
       RowBox[{"(", 
        RowBox[{"1", "-", "x"}], ")"}]}], "+", 
      RowBox[{"2", " ", 
       RowBox[{"(", 
        RowBox[{"x", "+", "1"}], ")"}], " ", 
       RowBox[{"log", "(", "x", ")"}]}]}], ")"}]}], "+", 
   RowBox[{
    SubscriptBox["C", "A"], " ", 
    SubscriptBox["C", "F"], " ", 
    RowBox[{"(", 
     RowBox[{
      FractionBox[
       RowBox[{"4", " ", 
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox["x", "2"], "+", "1"}], ")"}], " ", 
        RowBox[{
         SuperscriptBox["log", "2"], "(", "x", ")"}]}], 
       RowBox[{"1", "-", "x"}]], "+", 
      RowBox[{"8", " ", 
       RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
       RowBox[{"(", 
        RowBox[{"x", "+", "1"}], ")"}]}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         FractionBox["536", "9"], "-", 
         RowBox[{"16", " ", 
          RowBox[{"\[Zeta]", "(", "2", ")"}]}]}], ")"}], " ", 
       SubscriptBox[
        RowBox[{"(", 
         FractionBox["1", 
          RowBox[{"1", "-", "x"}]], ")"}], "+"]}], "+", 
      RowBox[{
       RowBox[{"\[Delta]", "(", 
        FormBox[
         RowBox[{"1", "-", "x"}],
         TraditionalForm], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{
         FractionBox[
          RowBox[{"88", " ", 
           RowBox[{"\[Zeta]", "(", "2", ")"}]}], "3"], "-", 
         RowBox[{"24", " ", 
          TemplateBox[{"3"},
           "Zeta"]}], "+", 
         FractionBox["17", "3"]}], ")"}]}], "+", 
      RowBox[{
       FractionBox["4", "9"], " ", 
       RowBox[{"(", 
        RowBox[{"53", "-", 
         RowBox[{"187", " ", "x"}]}], ")"}]}], "-", 
      RowBox[{
       FractionBox["4", "3"], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"5", " ", "x"}], "-", 
         FractionBox["22", 
          RowBox[{"1", "-", "x"}]], "+", "5"}], ")"}], " ", 
       RowBox[{"log", "(", "x", ")"}]}]}], ")"}]}], "+", 
   RowBox[{
    SubscriptBox["C", "F"], " ", 
    SubscriptBox["N", "f"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", 
       FractionBox[
        RowBox[{"8", " ", 
         RowBox[{"(", 
          RowBox[{
           SuperscriptBox["x", "2"], "+", "1"}], ")"}], " ", 
         RowBox[{"log", "(", "x", ")"}]}], 
        RowBox[{"3", " ", 
         RowBox[{"(", 
          RowBox[{"1", "-", "x"}], ")"}]}]]}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", 
          FractionBox[
           RowBox[{"16", " ", 
            RowBox[{"\[Zeta]", "(", "2", ")"}]}], "3"]}], "-", 
         FractionBox["2", "3"]}], ")"}], " ", 
       RowBox[{"\[Delta]", "(", 
        FormBox[
         RowBox[{"1", "-", "x"}],
         TraditionalForm], ")"}]}], "+", 
      FractionBox[
       RowBox[{"88", " ", "x"}], "9"], "-", 
      RowBox[{
       FractionBox["80", "9"], " ", 
       SubscriptBox[
        RowBox[{"(", 
         FractionBox["1", 
          RowBox[{"1", "-", "x"}]], ")"}], "+"]}], "-", 
      FractionBox["8", "9"]}], ")"}]}], "+", 
   RowBox[{
    SubsuperscriptBox["C", "F", "2"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", 
       FractionBox[
        RowBox[{"16", " ", 
         RowBox[{"(", 
          RowBox[{
           SuperscriptBox["x", "2"], "+", "1"}], ")"}], " ", 
         RowBox[{"log", "(", 
          RowBox[{"1", "-", "x"}], ")"}], " ", 
         RowBox[{"log", "(", "x", ")"}]}], 
        RowBox[{"1", "-", "x"}]]}], "+", 
      RowBox[{
       RowBox[{"\[Delta]", "(", 
        FormBox[
         RowBox[{"1", "-", "x"}],
         TraditionalForm], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", 
          RowBox[{"24", " ", 
           RowBox[{"\[Zeta]", "(", "2", ")"}]}]}], "+", 
         RowBox[{"48", " ", 
          TemplateBox[{"3"},
           "Zeta"]}], "+", "3"}], ")"}]}], "-", 
      RowBox[{"40", " ", 
       RowBox[{"(", 
        RowBox[{"1", "-", "x"}], ")"}]}], "-", 
      RowBox[{"4", " ", 
       RowBox[{"(", 
        RowBox[{"x", "+", "1"}], ")"}], " ", 
       RowBox[{
        SuperscriptBox["log", "2"], "(", "x", ")"}]}], "-", 
      RowBox[{"8", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"2", " ", "x"}], "+", 
         FractionBox["3", 
          RowBox[{"1", "-", "x"}]]}], ")"}], " ", 
       RowBox[{"log", "(", "x", ")"}]}]}], ")"}]}]}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.5009704095161247`*^9},
 CellTags->"SplittingFunction",
 CellLabel->"Out[1565]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SplittingFunction", "[", "PQG", "]"}]], "Input",
 CellTags->"SplittingFunction",
 CellLabel->"In[1566]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"4", " ", 
    SubscriptBox["C", "A"], " ", 
    SubscriptBox["T", "f"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", 
       RowBox[{"8", " ", 
        RowBox[{"\[Zeta]", "(", "2", ")"}]}]}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", 
          RowBox[{"16", " ", "x"}]}], "-", "8"}], ")"}], " ", 
       TemplateBox[{"2",RowBox[{"-", "x"}]},
        "PolyLog"]}], "-", 
      RowBox[{"44", " ", "x"}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{"4", "-", 
         RowBox[{"8", " ", "x"}]}], ")"}], " ", 
       RowBox[{
        SuperscriptBox["log", "2"], "(", 
        RowBox[{"1", "-", "x"}], ")"}]}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", 
          RowBox[{"8", " ", "x"}]}], "-", "4"}], ")"}], " ", 
       RowBox[{
        SuperscriptBox["log", "2"], "(", "x", ")"}]}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"16", " ", "x"}], "-", "16"}], ")"}], " ", 
       RowBox[{"log", "(", 
        RowBox[{"1", "-", "x"}], ")"}]}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"32", " ", "x"}], "+", "4"}], ")"}], " ", 
       RowBox[{"log", "(", "x", ")"}]}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", 
          RowBox[{"16", " ", "x"}]}], "-", "8"}], ")"}], " ", 
       RowBox[{"log", "(", "x", ")"}], " ", 
       RowBox[{"log", "(", 
        RowBox[{"x", "+", "1"}], ")"}]}], "+", "48"}], ")"}]}], "+", 
   RowBox[{"4", " ", 
    SubscriptBox["C", "F"], " ", 
    SubscriptBox["T", "f"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"8", " ", 
       RowBox[{"\[Zeta]", "(", "2", ")"}]}], "-", 
      RowBox[{"16", " ", 
       RowBox[{"\[Zeta]", "(", "2", ")"}], " ", "x"}], "+", 
      RowBox[{"54", " ", "x"}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"8", " ", "x"}], "-", "4"}], ")"}], " ", 
       RowBox[{
        SuperscriptBox["log", "2"], "(", 
        RowBox[{"1", "-", "x"}], ")"}]}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"4", " ", "x"}], "-", "2"}], ")"}], " ", 
       RowBox[{
        SuperscriptBox["log", "2"], "(", "x", ")"}]}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{"16", "-", 
         RowBox[{"16", " ", "x"}]}], ")"}], " ", 
       RowBox[{"log", "(", 
        RowBox[{"1", "-", "x"}], ")"}]}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{"8", "-", 
         RowBox[{"16", " ", "x"}]}], ")"}], " ", 
       RowBox[{"log", "(", "x", ")"}], " ", 
       RowBox[{"log", "(", 
        RowBox[{"1", "-", "x"}], ")"}]}], "-", 
      RowBox[{"18", " ", 
       RowBox[{"log", "(", "x", ")"}]}], "-", "44"}], ")"}]}]}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970410615725*^9},
 CellTags->"SplittingFunction",
 CellLabel->"Out[1566]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SplittingFunction", "[", "PGQ", "]"}]], "Input",
 CellTags->"SplittingFunction",
 CellLabel->"In[1567]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      SubscriptBox["C", "A"], " ", 
      SubscriptBox["C", "F"]}], ")"}], ".", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"16", " ", "x"}], "+", "32"}], ")"}], " ", 
       TemplateBox[{"2",RowBox[{"-", "x"}]},
        "PolyLog"]}], "+", 
      RowBox[{"16", " ", 
       RowBox[{"\[Zeta]", "(", "2", ")"}], " ", "x"}], "+", 
      FractionBox[
       RowBox[{"280", " ", "x"}], "9"], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{"16", "-", 
         RowBox[{"8", " ", "x"}]}], ")"}], " ", 
       RowBox[{
        SuperscriptBox["log", "2"], "(", 
        RowBox[{"1", "-", "x"}], ")"}]}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"8", " ", "x"}], "+", "16"}], ")"}], " ", 
       RowBox[{
        SuperscriptBox["log", "2"], "(", "x", ")"}]}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         FractionBox[
          RowBox[{"8", " ", "x"}], "3"], "+", 
         FractionBox["80", "3"]}], ")"}], " ", 
       RowBox[{"log", "(", 
        RowBox[{"1", "-", "x"}], ")"}]}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"16", " ", "x"}], "-", "32"}], ")"}], " ", 
       RowBox[{"log", "(", "x", ")"}], " ", 
       RowBox[{"log", "(", 
        RowBox[{"1", "-", "x"}], ")"}]}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{"32", "-", 
         RowBox[{"104", " ", "x"}]}], ")"}], " ", 
       RowBox[{"log", "(", "x", ")"}]}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"16", " ", "x"}], "+", "32"}], ")"}], " ", 
       RowBox[{"log", "(", "x", ")"}], " ", 
       RowBox[{"log", "(", 
        RowBox[{"x", "+", "1"}], ")"}]}], "+", 
      FractionBox["328", "9"]}], ")"}]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      SubscriptBox["C", "F"], " ", 
      SubscriptBox["T", "f"]}], ")"}], ".", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", 
       FractionBox[
        RowBox[{"32", " ", "x"}], "9"]}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         FractionBox[
          RowBox[{"32", " ", "x"}], "3"], "-", 
         FractionBox["64", "3"]}], ")"}], " ", 
       RowBox[{"log", "(", 
        RowBox[{"1", "-", "x"}], ")"}]}], "-", 
      FractionBox["128", "9"]}], ")"}]}], "+", 
   RowBox[{
    SubsuperscriptBox["C", "F", "2"], ".", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"32", " ", "x"}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"8", " ", "x"}], "-", "16"}], ")"}], " ", 
       RowBox[{
        SuperscriptBox["log", "2"], "(", 
        RowBox[{"1", "-", "x"}], ")"}]}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{"8", "-", 
         RowBox[{"4", " ", "x"}]}], ")"}], " ", 
       RowBox[{
        SuperscriptBox["log", "2"], "(", "x", ")"}]}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", 
          RowBox[{"8", " ", "x"}]}], "-", "16"}], ")"}], " ", 
       RowBox[{"log", "(", 
        RowBox[{"1", "-", "x"}], ")"}]}], "-", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"32", " ", "x"}], "+", "64"}], ")"}], " ", 
       RowBox[{"log", "(", "x", ")"}]}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"36", " ", "x"}], "+", "48"}], ")"}], " ", 
       RowBox[{"log", "(", "x", ")"}]}], "-", "68"}], ")"}]}]}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970410791353*^9},
 CellTags->"SplittingFunction",
 CellLabel->"Out[1567]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SplittingFunction", "[", "PGG", "]"}]], "Input",
 CellTags->"SplittingFunction",
 CellLabel->"In[1568]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox["C", "A"], " ", 
    SubscriptBox["T", "f"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", 
       RowBox[{
        FractionBox["32", "3"], " ", 
        RowBox[{"\[Delta]", "(", 
         FormBox[
          RowBox[{"1", "-", "x"}],
          TraditionalForm], ")"}]}]}], "+", 
      FractionBox[
       RowBox[{"608", " ", "x"}], "9"], "-", 
      RowBox[{
       FractionBox["160", "9"], " ", 
       SubscriptBox[
        RowBox[{"(", 
         FractionBox["1", 
          RowBox[{"1", "-", "x"}]], ")"}], "+"]}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", 
          FractionBox[
           RowBox[{"32", " ", "x"}], "3"]}], "-", 
         FractionBox["32", "3"]}], ")"}], " ", 
       RowBox[{"log", "(", "x", ")"}]}], "-", 
      FractionBox["448", "9"]}], ")"}]}], "+", 
   RowBox[{
    SubsuperscriptBox["C", "A", "2"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"64", " ", "x"}], "+", 
         FractionBox["32", 
          RowBox[{"x", "+", "1"}]], "+", "32"}], ")"}], " ", 
       TemplateBox[{"2",RowBox[{"-", "x"}]},
        "PolyLog"]}], "+", 
      RowBox[{
       FractionBox["64", "3"], " ", 
       RowBox[{"\[Delta]", "(", 
        FormBox[
         RowBox[{"1", "-", "x"}],
         TraditionalForm], ")"}]}], "+", 
      RowBox[{
       RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"64", " ", "x"}], "-", 
         RowBox[{"16", " ", 
          SubscriptBox[
           RowBox[{"(", 
            FractionBox["1", 
             RowBox[{"1", "-", "x"}]], ")"}], "+"]}], "+", 
         FractionBox["16", 
          RowBox[{"x", "+", "1"}]]}], ")"}]}], "+", 
      RowBox[{"24", " ", 
       TemplateBox[{"3"},
        "Zeta"], " ", 
       RowBox[{"\[Delta]", "(", 
        FormBox[
         RowBox[{"1", "-", "x"}],
         TraditionalForm], ")"}]}], "-", 
      FractionBox[
       RowBox[{"388", " ", "x"}], "9"], "+", 
      RowBox[{
       FractionBox["536", "9"], " ", 
       SubscriptBox[
        RowBox[{"(", 
         FractionBox["1", 
          RowBox[{"1", "-", "x"}]], ")"}], "+"]}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", 
          FractionBox["8", 
           RowBox[{"x", "+", "1"}]]}], "+", 
         FractionBox["8", 
          RowBox[{"1", "-", "x"}]], "+", "32"}], ")"}], " ", 
       RowBox[{
        SuperscriptBox["log", "2"], "(", "x", ")"}]}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         FractionBox["232", "3"], "-", 
         FractionBox[
          RowBox[{"536", " ", "x"}], "3"]}], ")"}], " ", 
       RowBox[{"log", "(", "x", ")"}]}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"64", " ", "x"}], "-", 
         FractionBox["32", 
          RowBox[{"1", "-", "x"}]], "-", "32"}], ")"}], " ", 
       RowBox[{"log", "(", 
        RowBox[{"1", "-", "x"}], ")"}], " ", 
       RowBox[{"log", "(", "x", ")"}]}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"64", " ", "x"}], "+", 
         FractionBox["32", 
          RowBox[{"x", "+", "1"}]], "+", "32"}], ")"}], " ", 
       RowBox[{"log", "(", 
        RowBox[{"x", "+", "1"}], ")"}], " ", 
       RowBox[{"log", "(", "x", ")"}]}], "-", 
      FractionBox["148", "9"]}], ")"}]}], "+", 
   RowBox[{
    SubscriptBox["C", "F"], " ", 
    SubscriptBox["T", "f"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", 
       RowBox[{"8", " ", 
        RowBox[{"\[Delta]", "(", 
         FormBox[
          RowBox[{"1", "-", "x"}],
          TraditionalForm], ")"}]}]}], "+", 
      RowBox[{"80", " ", "x"}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", 
          RowBox[{"16", " ", "x"}]}], "-", "16"}], ")"}], " ", 
       RowBox[{
        SuperscriptBox["log", "2"], "(", "x", ")"}]}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"16", " ", "x"}], "-", "80"}], ")"}], " ", 
       RowBox[{"log", "(", "x", ")"}]}], "-", "80"}], ")"}]}]}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970410986806*^9},
 CellTags->"SplittingFunction",
 CellLabel->"Out[1568]="]
}, Open  ]]
}, Open  ]],

Cell["StandardMatrixElement", "ObjectName",
 CellTags->"StandardMatrixElement"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"StandardMatrixElement"],

Cell["\<\
StandardMatrixElement[ ... ] is the head for matrix element abbreviations.\
\>", "Text",
 CellTags->"StandardMatrixElement"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["OneLoop",
  BaseStyle->"AddOnsLink",
  ButtonData:>"OneLoop",
  ButtonNote->"OneLoop"],
 "."
}], "Text",
 CellTags->"StandardMatrixElement"]
}, Open  ]],

Cell["SubContext", "ObjectName",
 CellTags->"SubContext"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SubContext"],

Cell["\<\
SubContext[fun] gives the sub-directory (context) in HighEnergyPhysics.\
\>", "Text",
 CellTags->"SubContext"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FeynCalc",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FeynCalc",
  ButtonNote->"FeynCalc"],
 ", ",
 ButtonBox["Load",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Load",
  ButtonNote->"Load"],
 "."
}], "Text",
 CellTags->"SubContext"]
}, Open  ]],

Cell["SubLoop", "ObjectName",
 CellTags->"SubLoop"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SubLoop"],

Cell["\<\
SubLoop is an option for OPE1Loop. If set to True, sub 1-loop tensorintegral \
decomposition is performed.\
\>", "Text",
 CellTags->"SubLoop"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["OPE1Loop",
  BaseStyle->"AddOnsLink",
  ButtonData:>"OPE1Loop",
  ButtonNote->"OPE1Loop"],
 "."
}], "Text",
 CellTags->"SubLoop"]
}, Open  ]],

Cell["SumP", "ObjectName",
 CellTags->"SumP"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SumP"],

Cell[TextData[{
 "SumP[k, m] is ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["2", 
     RowBox[{
      StyleBox["k",
       FontSlant->"Plain"], "-", "1"}]], 
    RowBox[{
     SubsuperscriptBox["\[Sum]", 
      RowBox[{
       StyleBox["i",
        FontSlant->"Plain"], "=", "1"}], 
      RowBox[{"2", 
       StyleBox["m",
        FontSlant->"Plain"]}]], 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"1", "+", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"-", "1"}], ")"}], 
         StyleBox["i",
          FontSlant->"Plain"]]}], ")"}], "/", 
      SuperscriptBox[
       StyleBox["i",
        FontSlant->"Plain"], 
       StyleBox["k",
        FontSlant->"Plain"]]}]}]}], TraditionalForm]]]
}], "Text",
 CellTags->"SumP"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["SumS",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SumS",
  ButtonNote->"SumS"],
 ", ",
 ButtonBox["SumT",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SumT",
  ButtonNote->"SumT"],
 "."
}], "Text",
 CellTags->"SumP"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"SumP"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SumP", "[", 
  RowBox[{"1", ",", 
   RowBox[{"m", "-", "1"}]}], "]"}]], "Input",
 CellTags->"SumP",
 CellLabel->"In[1569]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubsuperscriptBox["S", "1", "'"], "\[NoBreak]", "(", "\[NoBreak]", 
   FormBox[
    RowBox[{"m", "-", "1"}],
    TraditionalForm], "\[NoBreak]", ")"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970411446875*^9},
 CellTags->"SumP",
 CellLabel->"Out[1569]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SumP", "[", 
  RowBox[{"2", ",", 
   RowBox[{"m", "-", "1"}]}], "]"}]], "Input",
 CellTags->"SumP",
 CellLabel->"In[1570]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubsuperscriptBox["S", "2", "'"], "\[NoBreak]", "(", "\[NoBreak]", 
   FormBox[
    RowBox[{"m", "-", "1"}],
    TraditionalForm], "\[NoBreak]", ")"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009704115558567`*^9},
 CellTags->"SumP",
 CellLabel->"Out[1570]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SumP", "[", 
  RowBox[{"1", ",", "m"}], "]"}]], "Input",
 CellTags->"SumP",
 CellLabel->"In[1571]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubsuperscriptBox["S", "1", "'"], "\[NoBreak]", "(", "\[NoBreak]", 
   FormBox["m",
    TraditionalForm], "\[NoBreak]", ")"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970411627223*^9},
 CellTags->"SumP",
 CellLabel->"Out[1571]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SumP", "[", 
  RowBox[{"1", ",", "4"}], "]"}]], "Input",
 CellTags->"SumP",
 CellLabel->"In[1572]:="],

Cell[BoxData[
 FormBox[
  FractionBox["25", "12"], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009704117821712`*^9},
 CellTags->"SumP",
 CellLabel->"Out[1572]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  UnderoverscriptBox["\[Sum]", 
   RowBox[{"i", "=", "1"}], "8"], 
  RowBox[{
   RowBox[{"(", 
    RowBox[{"1", "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}], "^", "i"}]}], ")"}], "/", "i"}]}]], "Input",\

 CellTags->"SumP",
 CellLabel->"In[1573]:="],

Cell[BoxData[
 FormBox[
  FractionBox["25", "12"], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970412029188*^9},
 CellTags->"SumP",
 CellLabel->"Out[1573]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Explicit", "[", 
  RowBox[{"SumP", "[", 
   RowBox[{"1", ",", 
    RowBox[{"n", "/", "2"}]}], "]"}], "]"}]], "Input",
 CellTags->"SumP",
 CellLabel->"In[1574]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    FractionBox["1", "2"], " ", 
    RowBox[{"(", 
     RowBox[{"1", "-", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"-", "1"}], ")"}], "n"]}], ")"}], " ", 
    RowBox[{
     SubscriptBox["S", "1"], "(", 
     FormBox[
      FractionBox[
       RowBox[{"n", "-", "1"}], "2"],
      TraditionalForm], ")"}]}], "+", 
   RowBox[{
    FractionBox["1", "2"], " ", 
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"-", "1"}], ")"}], "n"], "+", "1"}], ")"}], " ", 
    RowBox[{
     SubscriptBox["S", "1"], "(", 
     FormBox[
      FractionBox["n", "2"],
      TraditionalForm], ")"}]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970412252633*^9},
 CellTags->"SumP",
 CellLabel->"Out[1574]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "/.", 
  RowBox[{"n", "\[Rule]", "8"}]}]], "Input",
 CellTags->"SumP",
 CellLabel->"In[1575]:="],

Cell[BoxData[
 FormBox[
  FractionBox["25", "12"], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970412858739*^9},
 CellTags->"SumP",
 CellLabel->"Out[1575]="]
}, Open  ]]
}, Open  ]],

Cell["SumS", "ObjectName",
 CellTags->"SumS"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SumS"],

Cell[TextData[{
 "SumS[1, m] is the harmonic number ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubsuperscriptBox["\[Sum]", 
     RowBox[{
      StyleBox["i",
       FontSlant->"Italic"], "=", "1"}], 
     StyleBox["m",
      FontSlant->"Italic"]], 
    StyleBox[" ",
     FontSlant->"Italic"], 
    RowBox[{
     StyleBox["i",
      FontSlant->"Italic"], "^", 
     RowBox[{"(", 
      RowBox[{"-", "1"}], ")"}]}]}], TraditionalForm]]],
 " ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"(", 
     RowBox[{"=", 
      RowBox[{
       SubscriptBox["S", "1"], "(", "m", ")"}]}], " ", ")"}], "."}], 
   TraditionalForm]]],
 " SumS[1,1,m] is ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubsuperscriptBox["\[Sum]", 
     RowBox[{
      StyleBox["i",
       FontSlant->"Italic"], "=", "1"}], 
     StyleBox["m",
      FontSlant->"Italic"]], 
    SubscriptBox["S", "1"]}], TraditionalForm]]],
 "(",
 StyleBox["i",
  FontSlant->"Italic"],
 ")/",
 StyleBox["i",
  FontSlant->"Italic"],
 ". SumS[k,l,m] is ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubsuperscriptBox["\[Sum]", 
     RowBox[{"i", "=", "1"}], "m"], " ", 
    RowBox[{
     RowBox[{
      SubscriptBox["S", "l"], "(", "i", ")"}], "/", 
     SuperscriptBox["i", "k"]}]}], TraditionalForm]]],
 ". SumS[r, n] represents Sum[Sign[r]^i/i^Abs[r], {i, 1, n}]. SumS[r,s, n] is \
Sum[Sign[r]^k/k^Abs[r] Sign[s]^j/j^Abs[s], {k, 1, n}, {j, 1, k}], etc."
}], "Text",
 CellTags->"SumS"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "SumS", "]"}]], "Input",
 CellTags->"SumS",
 CellLabel->"In[1576]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"Reduce", "\[Rule]", "False"}], "}"}], TraditionalForm]], "Output",\

 CellChangeTimes->{3.500970412971087*^9},
 CellTags->"SumS",
 CellLabel->"Out[1576]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["SumP",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SumP",
  ButtonNote->"SumP"],
 ", ",
 ButtonBox["SumT",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SumT",
  ButtonNote->"SumT"],
 "."
}], "Text",
 CellTags->"SumS"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"SumS"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SumS", "[", 
  RowBox[{"1", ",", 
   RowBox[{"m", "-", "1"}]}], "]"}]], "Input",
 CellTags->"SumS",
 CellLabel->"In[1577]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["S", "1"], "(", 
   FormBox[
    RowBox[{"m", "-", "1"}],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970413087322*^9},
 CellTags->"SumS",
 CellLabel->"Out[1577]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SumS", "[", 
  RowBox[{"2", ",", 
   RowBox[{"m", "-", "1"}]}], "]"}]], "Input",
 CellTags->"SumS",
 CellLabel->"In[1578]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["S", "2"], "(", 
   FormBox[
    RowBox[{"m", "-", "1"}],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970413896649*^9},
 CellTags->"SumS",
 CellLabel->"Out[1578]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SumS", "[", 
  RowBox[{
   RowBox[{"-", "1"}], ",", "m"}], "]"}]], "Input",
 CellTags->"SumS",
 CellLabel->"In[1579]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["S", 
    RowBox[{"-", "1"}]], "(", "m", ")"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009704139407473`*^9},
 CellTags->"SumS",
 CellLabel->"Out[1579]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SumS", "[", 
  RowBox[{"1", ",", "m", ",", 
   RowBox[{"Reduce", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"SumS",
 CellLabel->"In[1580]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox["S", "1"], "(", 
    FormBox[
     RowBox[{"m", "-", "1"}],
     TraditionalForm], ")"}], "+", 
   FractionBox["1", "m"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009704144238033`*^9},
 CellTags->"SumS",
 CellLabel->"Out[1580]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SumS", "[", 
  RowBox[{"3", ",", 
   RowBox[{"m", "+", "2"}], ",", 
   RowBox[{"Reduce", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"SumS",
 CellLabel->"In[1581]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox["S", "3"], "(", 
    FormBox[
     RowBox[{"m", "+", "1"}],
     TraditionalForm], ")"}], "+", 
   FractionBox["1", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"m", "+", "2"}], ")"}], "3"]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009704145500937`*^9},
 CellTags->"SumS",
 CellLabel->"Out[1581]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"SetOptions", "[", 
   RowBox[{"SumS", ",", 
    RowBox[{"Reduce", "\[Rule]", "True"}]}], "]"}], ";"}]], "Input",
 CellTags->"SumS",
 CellLabel->"In[1582]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SumS", "[", 
  RowBox[{"3", ",", 
   RowBox[{"m", "+", "2"}]}], "]"}]], "Input",
 CellTags->"SumS",
 CellLabel->"In[1583]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["1", 
    SuperscriptBox["m", "3"]], "+", 
   RowBox[{
    SubscriptBox["S", "3"], "(", 
    FormBox[
     RowBox[{"m", "-", "1"}],
     TraditionalForm], ")"}], "+", 
   FractionBox["1", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"m", "+", "1"}], ")"}], "3"]], "+", 
   FractionBox["1", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"m", "+", "2"}], ")"}], "3"]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970414814166*^9},
 CellTags->"SumS",
 CellLabel->"Out[1583]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"SetOptions", "[", 
   RowBox[{"SumS", ",", 
    RowBox[{"Reduce", "\[Rule]", "False"}]}], "]"}], ";"}]], "Input",
 CellTags->"SumS",
 CellLabel->"In[1584]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SumS", "[", 
  RowBox[{"1", ",", "4"}], "]"}]], "Input",
 CellTags->"SumS",
 CellLabel->"In[1585]:="],

Cell[BoxData[
 FormBox[
  FractionBox["25", "12"], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970414983094*^9},
 CellTags->"SumS",
 CellLabel->"Out[1585]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  UnderoverscriptBox["\[Sum]", 
   RowBox[{"i", "=", "1"}], "4"], 
  RowBox[{"1", "/", "i"}]}]], "Input",
 CellTags->"SumS",
 CellLabel->"In[1586]:="],

Cell[BoxData[
 FormBox[
  FractionBox["25", "12"], TraditionalForm]], "Output",
 CellChangeTimes->{3.50097041504451*^9},
 CellTags->"SumS",
 CellLabel->"Out[1586]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SumS", "[", 
  RowBox[{"1", ",", "2", ",", 
   RowBox[{"m", "-", "1"}]}], "]"}]], "Input",
 CellTags->"SumS",
 CellLabel->"In[1587]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["S", "12"], "(", 
   FormBox[
    RowBox[{"m", "-", "1"}],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970415121756*^9},
 CellTags->"SumS",
 CellLabel->"Out[1587]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SumS", "[", 
  RowBox[{"1", ",", "1", ",", "1", ",", "11"}], "]"}]], "Input",
 CellTags->"SumS",
 CellLabel->"In[1588]:="],

Cell[BoxData[
 FormBox[
  FractionBox["31276937512951", "4260000729600"], TraditionalForm]], "Output",\

 CellChangeTimes->{3.500970415201149*^9},
 CellTags->"SumS",
 CellLabel->"Out[1588]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SumS", "[", 
  RowBox[{
   RowBox[{"-", "1"}], ",", "4"}], "]"}]], "Input",
 CellTags->"SumS",
 CellLabel->"In[1589]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   FractionBox["7", "12"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009704152784557`*^9},
 CellTags->"SumS",
 CellLabel->"Out[1589]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SumT", "[", 
  RowBox[{"1", ",", "4"}], "]"}]], "Input",
 CellTags->"SumS",
 CellLabel->"In[1590]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   FractionBox["7", "12"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970415322156*^9},
 CellTags->"SumS",
 CellLabel->"Out[1590]="]
}, Open  ]]
}, Open  ]],

Cell["SumT", "ObjectName",
 CellTags->"SumT"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SumT"],

Cell[TextData[{
 "SumT[1, m] is the alternative harmonic number ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubsuperscriptBox["\[Sum]", 
     RowBox[{"i", "=", "1"}], "m"], " ", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}], "^", "i"}], "/", "i", " "}]}], 
   TraditionalForm]]],
 " SumT[r, n] represents Sum[(-1)^i/i^r, {i,1,n}], SumT[r,s, n] is Sum[1/k^r \
(-1)^j/j^s, {k, 1, n}, {j, 1, k}]."
}], "Text",
 CellTags->"SumT"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["SumP",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SumP",
  ButtonNote->"SumP"],
 ", ",
 ButtonBox["SumS",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SumS",
  ButtonNote->"SumS"],
 "."
}], "Text",
 CellTags->"SumT"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"SumT"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SumT", "[", 
  RowBox[{"1", ",", 
   RowBox[{"m", "-", "1"}]}], "]"}]], "Input",
 CellTags->"SumT",
 CellLabel->"In[1591]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox[
    OverscriptBox["S", "~"], "1"], "(", 
   FormBox[
    RowBox[{"m", "-", "1"}],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970415376143*^9},
 CellTags->"SumT",
 CellLabel->"Out[1591]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SumT", "[", 
  RowBox[{"2", ",", 
   RowBox[{"m", "-", "1"}]}], "]"}]], "Input",
 CellTags->"SumT",
 CellLabel->"In[1592]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox[
    OverscriptBox["S", "~"], "2"], "(", 
   FormBox[
    RowBox[{"m", "-", "1"}],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970415610572*^9},
 CellTags->"SumT",
 CellLabel->"Out[1592]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SumT", "[", 
  RowBox[{"1", ",", "m"}], "]"}]], "Input",
 CellTags->"SumT",
 CellLabel->"In[1593]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox[
    OverscriptBox["S", "~"], "1"], "(", "m", ")"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970415733384*^9},
 CellTags->"SumT",
 CellLabel->"Out[1593]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SumT", "[", 
  RowBox[{"1", ",", "m", ",", 
   RowBox[{"Reduce", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"SumT",
 CellLabel->"In[1594]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox[
     OverscriptBox["S", "~"], "1"], "(", 
    FormBox[
     RowBox[{"m", "-", "1"}],
     TraditionalForm], ")"}], "+", 
   FractionBox[
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"-", "1"}], ")"}], "m"], "m"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970415808329*^9},
 CellTags->"SumT",
 CellLabel->"Out[1594]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SumT", "[", 
  RowBox[{"1", ",", "4"}], "]"}]], "Input",
 CellTags->"SumT",
 CellLabel->"In[1595]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   FractionBox["7", "12"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.50097041588472*^9},
 CellTags->"SumT",
 CellLabel->"Out[1595]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  UnderoverscriptBox["\[Sum]", 
   RowBox[{"i", "=", "1"}], "4"], 
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{"-", "1"}], ")"}], "^", "i"}], "/", "i"}]}]], "Input",
 CellTags->"SumT",
 CellLabel->"In[1596]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   FractionBox["7", "12"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970415936927*^9},
 CellTags->"SumT",
 CellLabel->"Out[1596]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SumT", "[", 
  RowBox[{"1", ",", "2", ",", 
   RowBox[{"m", "-", "1"}]}], "]"}]], "Input",
 CellTags->"SumT",
 CellLabel->"In[1597]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox[
    OverscriptBox["S", "~"], "12"], "(", 
   FormBox[
    RowBox[{"m", "-", "1"}],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970416024598*^9},
 CellTags->"SumT",
 CellLabel->"Out[1597]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SumT", "[", 
  RowBox[{"1", ",", "2", ",", "42"}], "]"}]], "Input",
 CellTags->"SumT",
 CellLabel->"In[1598]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   FractionBox["38987958697055013360489864298703621429610152138683927", 
    "10512121660702378405316004964483761080879190528000000"]}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.50097041614063*^9},
 CellTags->"SumT",
 CellLabel->"Out[1598]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SumT", "[", 
  RowBox[{"1", ",", "4"}], "]"}]], "Input",
 CellTags->"SumT",
 CellLabel->"In[1599]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   FractionBox["7", "12"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970416386285*^9},
 CellTags->"SumT",
 CellLabel->"Out[1599]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SumS", "[", 
  RowBox[{
   RowBox[{"-", "1"}], ",", "4"}], "]"}]], "Input",
 CellTags->"SumT",
 CellLabel->"In[1600]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   FractionBox["7", "12"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970416460154*^9},
 CellTags->"SumT",
 CellLabel->"Out[1600]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  UnderoverscriptBox["\[Sum]", 
   RowBox[{"i", "=", "1"}], 
   RowBox[{"m", "-", "1"}]], 
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{"-", "1"}], ")"}], "^", "i"}], "/", "i"}]}]], "Input",
 CellTags->"SumT",
 CellLabel->"In[1601]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    RowBox[{"-", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}], "m"]}], " ", 
    TemplateBox[{RowBox[{"-", "1"}],"1","m"},
     "LerchPhi"]}], "-", 
   RowBox[{"log", "(", "2", ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970417361703*^9},
 CellTags->"SumT",
 CellLabel->"Out[1601]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SumT", "[", 
  RowBox[{"1", ",", "2", ",", "12"}], "]"}]], "Input",
 CellTags->"SumT",
 CellLabel->"In[1602]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   FractionBox["57561743656913", "21300003648000"]}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970417871098*^9},
 CellTags->"SumT",
 CellLabel->"Out[1602]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SumS", "[", 
  RowBox[{"1", ",", 
   RowBox[{"-", "2"}], ",", "42"}], "]"}]], "Input",
 CellTags->"SumT",
 CellLabel->"In[1603]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   FractionBox["38987958697055013360489864298703621429610152138683927", 
    "10512121660702378405316004964483761080879190528000000"]}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.50097041795304*^9},
 CellTags->"SumT",
 CellLabel->"Out[1603]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Array", "[", 
  RowBox[{"SumT", ",", "6"}], "]"}]], "Input",
 CellTags->"SumT",
 CellLabel->"In[1604]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"-", "1"}], ",", 
    RowBox[{"-", 
     FractionBox["5", "8"]}], ",", 
    RowBox[{"-", 
     FractionBox["179", "216"]}], ",", 
    RowBox[{"-", 
     FractionBox["1207", "1728"]}], ",", 
    RowBox[{"-", 
     FractionBox["170603", "216000"]}], ",", 
    RowBox[{"-", 
     FractionBox["155903", "216000"]}]}], "}"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009704180330267`*^9},
 CellTags->"SumT",
 CellLabel->"Out[1604]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Array", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"SumS", "[", 
     RowBox[{
      RowBox[{"-", "2"}], ",", "1", ",", "#1"}], "]"}], "&"}], ",", "6"}], 
  "]"}]], "Input",
 CellTags->"SumT",
 CellLabel->"In[1605]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"-", "1"}], ",", 
    RowBox[{"-", 
     FractionBox["5", "8"]}], ",", 
    RowBox[{"-", 
     FractionBox["179", "216"]}], ",", 
    RowBox[{"-", 
     FractionBox["1207", "1728"]}], ",", 
    RowBox[{"-", 
     FractionBox["170603", "216000"]}], ",", 
    RowBox[{"-", 
     FractionBox["155903", "216000"]}]}], "}"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970418085615*^9},
 CellTags->"SumT",
 CellLabel->"Out[1605]="]
}, Open  ]]
}, Open  ]],

Cell["SUND", "ObjectName",
 CellTags->"SUND"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SUND"],

Cell[TextData[{
 "SUND[a,b,c] are the symmetric SU(",
 StyleBox["N",
  FontSlant->"Italic"],
 ") ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["d", "abc"], "."}], TraditionalForm]]]
}], "Text",
 CellTags->"SUND"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["SUNDelta",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SUNDelta",
  ButtonNote->"SUNDelta"],
 ", ",
 ButtonBox["SUNF",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SUNF",
  ButtonNote->"SUNF"],
 ", ",
 ButtonBox["SUNSimplify",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SUNSimplify",
  ButtonNote->"SUNSimplify"],
 "."
}], "Text",
 CellTags->"SUND"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"SUND"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUND", "[", 
  RowBox[{"a", ",", "b", ",", "c"}], "]"}]], "Input",
 CellTags->"SUND",
 CellLabel->"In[1606]:="],

Cell[BoxData[
 FormBox[
  SubscriptBox["d", 
   RowBox[{"a", "b", "c"}]], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009704181379347`*^9},
 CellTags->"SUND",
 CellLabel->"Out[1606]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"tt", "=", 
  RowBox[{"SUND", "[", 
   RowBox[{"a", ",", "b", ",", "c", ",", 
    RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]}]], "Input",
 CellTags->"SUND",
 CellLabel->"In[1607]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"2", " ", 
    RowBox[{"tr", "(", 
     FormBox[
      RowBox[{
       SubscriptBox["T", 
        FormBox[
         FormBox["a",
          TraditionalForm],
         TraditionalForm]], ".", 
       SubscriptBox["T", 
        FormBox[
         FormBox["b",
          TraditionalForm],
         TraditionalForm]], ".", 
       SubscriptBox["T", 
        FormBox[
         FormBox["c",
          TraditionalForm],
         TraditionalForm]]}],
      TraditionalForm], ")"}]}], "+", 
   RowBox[{"2", " ", 
    RowBox[{"tr", "(", 
     FormBox[
      RowBox[{
       SubscriptBox["T", 
        FormBox[
         FormBox["b",
          TraditionalForm],
         TraditionalForm]], ".", 
       SubscriptBox["T", 
        FormBox[
         FormBox["a",
          TraditionalForm],
         TraditionalForm]], ".", 
       SubscriptBox["T", 
        FormBox[
         FormBox["c",
          TraditionalForm],
         TraditionalForm]]}],
      TraditionalForm], ")"}]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970418271348*^9},
 CellTags->"SUND",
 CellLabel->"Out[1607]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUND", "[", 
  RowBox[{"c", ",", "a", ",", "b"}], "]"}]], "Input",
 CellTags->"SUND",
 CellLabel->"In[1608]:="],

Cell[BoxData[
 FormBox[
  SubscriptBox["d", 
   RowBox[{"a", "b", "c"}]], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970418437376*^9},
 CellTags->"SUND",
 CellLabel->"Out[1608]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUND", "[", 
  RowBox[{"a", ",", "b", ",", "b"}], "]"}]], "Input",
 CellTags->"SUND",
 CellLabel->"In[1609]:="],

Cell[BoxData[
 FormBox[
  SubscriptBox["d", 
   RowBox[{"a", "b", "b"}]], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009704186071377`*^9},
 CellTags->"SUND",
 CellLabel->"Out[1609]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNSimplify", "[", 
  RowBox[{
   RowBox[{"SUND", "[", 
    RowBox[{"a", ",", "b", ",", "c"}], "]"}], " ", 
   RowBox[{"SUND", "[", 
    RowBox[{"a", ",", "b", ",", "c"}], "]"}]}], "]"}]], "Input",
 CellTags->"SUND",
 CellLabel->"In[1610]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", "2"}], " ", 
   RowBox[{"(", 
    RowBox[{"4", "-", 
     SubsuperscriptBox["C", "A", "2"]}], ")"}], " ", 
   SubscriptBox["C", "F"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970418848465*^9},
 CellTags->"SUND",
 CellLabel->"Out[1610]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"SUNSimplify", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"SUND", "[", 
      RowBox[{"a", ",", "b", ",", "c"}], "]"}], " ", 
     RowBox[{"SUND", "[", 
      RowBox[{"a", ",", "b", ",", "c"}], "]"}]}], ",", 
    RowBox[{"SUNNToCACF", "\[Rule]", "False"}]}], "]"}], "//", 
  "Factor2"}]], "Input",
 CellTags->"SUND",
 CellLabel->"In[1611]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{
    RowBox[{"(", 
     RowBox[{"1", "-", 
      SuperscriptBox["N", "2"]}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{"4", "-", 
      SuperscriptBox["N", "2"]}], ")"}]}], "N"], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970418941587*^9},
 CellTags->"SUND",
 CellLabel->"Out[1611]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"SUNSimplify", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"SUND", "[", 
      RowBox[{"a", ",", "b", ",", "c"}], "]"}], " ", 
     RowBox[{"SUND", "[", 
      RowBox[{"e", ",", "b", ",", "c"}], "]"}]}], ",", 
    RowBox[{"SUNNToCACF", "\[Rule]", "False"}]}], "]"}], "//", 
  "Factor2"}]], "Input",
 CellTags->"SUND",
 CellLabel->"In[1612]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   FractionBox[
    RowBox[{
     RowBox[{"(", 
      RowBox[{"4", "-", 
       SuperscriptBox["N", "2"]}], ")"}], " ", 
     SubscriptBox["\[Delta]", 
      RowBox[{
       FormBox[
        FormBox["a",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["e",
         TraditionalForm],
        TraditionalForm]}]]}], "N"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970419045083*^9},
 CellTags->"SUND",
 CellLabel->"Out[1612]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"SUND", "[", 
   RowBox[{"a", ",", "b", ",", "c"}], "]"}], "//", "StandardForm"}]], "Input",\

 CellTags->"SUND",
 CellLabel->"In[1613]:="],

Cell[BoxData[
 RowBox[{"SUND", "[", 
  RowBox[{"a", ",", "b", ",", "c"}], "]"}]], "Output",
 CellChangeTimes->{3.500970419084148*^9},
 CellTags->"SUND",
 CellLabel->"Out[1613]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"SUND", "[", 
    RowBox[{"a", ",", "b", ",", "c"}], "]"}], "//", "FCI"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"SUND",
 CellLabel->"In[1614]:="],

Cell[BoxData[
 RowBox[{"SUND", "[", 
  RowBox[{
   RowBox[{"SUNIndex", "[", "a", "]"}], ",", 
   RowBox[{"SUNIndex", "[", "b", "]"}], ",", 
   RowBox[{"SUNIndex", "[", "c", "]"}]}], "]"}]], "Output",
 CellChangeTimes->{3.500970419123684*^9},
 CellTags->"SUND",
 CellLabel->"Out[1614]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"SUND", "[", 
     RowBox[{"a", ",", "b", ",", "c"}], "]"}], "//", "FCI"}], "//", "FCE"}], "//",
   "StandardForm"}]], "Input",
 CellTags->"SUND",
 CellLabel->"In[1615]:="],

Cell[BoxData[
 RowBox[{"SUND", "[", 
  RowBox[{"a", ",", "b", ",", "c"}], "]"}]], "Output",
 CellChangeTimes->{3.5009704191953287`*^9},
 CellTags->"SUND",
 CellLabel->"Out[1615]//StandardForm="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"Clear", "[", "tt", "]"}]], "Input",
 CellTags->"SUND",
 CellLabel->"In[1616]:="]
}, Open  ]],

Cell["SUNDelta", "ObjectName",
 CellTags->"SUNDelta"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SUNDelta"],

Cell[TextData[{
 "SUNDelta[a, b] is the Kronecker-delta for SU(",
 StyleBox["N",
  FontSlant->"Italic"],
 ") with color indices a and b."
}], "Text",
 CellTags->"SUNDelta"],

Cell[TextData[{
 "See also: ",
 ButtonBox["ExplicitSUNIndex",
  BaseStyle->"AddOnsLink",
  ButtonData:>"ExplicitSUNIndex",
  ButtonNote->"ExplicitSUNIndex"],
 ", ",
 ButtonBox["SD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SD",
  ButtonNote->"SD"],
 ", ",
 ButtonBox["SUNF",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SUNF",
  ButtonNote->"SUNF"],
 ", ",
 ButtonBox["SUNIndex",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SUNIndex",
  ButtonNote->"SUNIndex"],
 ", ",
 ButtonBox["SUNSimplify",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SUNSimplify",
  ButtonNote->"SUNSimplify"],
 ", ",
 ButtonBox["Trick",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Trick",
  ButtonNote->"Trick"],
 "."
}], "Text",
 CellTags->"SUNDelta"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"SUNDelta"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNDelta", "[", 
  RowBox[{"a", ",", "b"}], "]"}]], "Input",
 CellTags->"SUNDelta",
 CellLabel->"In[1617]:="],

Cell[BoxData[
 FormBox[
  SubscriptBox["\[Delta]", 
   RowBox[{"a", "b"}]], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970419269124*^9},
 CellTags->"SUNDelta",
 CellLabel->"Out[1617]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Trick", "[", " ", 
  RowBox[{
   RowBox[{"SUNDelta", "[", 
    RowBox[{"a", ",", "b"}], "]"}], " ", 
   RowBox[{"SUNDelta", "[", 
    RowBox[{"b", ",", "c"}], "]"}]}], " ", "]"}]], "Input",
 CellTags->"SUNDelta",
 CellLabel->"In[1618]:="],

Cell[BoxData[
 FormBox[
  SubscriptBox["\[Delta]", 
   RowBox[{
    FormBox[
     FormBox["a",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["c",
      TraditionalForm],
     TraditionalForm]}]], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970419366856*^9},
 CellTags->"SUNDelta",
 CellLabel->"Out[1618]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNDelta", "[", 
  RowBox[{
   RowBox[{"SUNIndex", "[", "a", "]"}], ",", 
   RowBox[{"SUNIndex", "[", "b", "]"}]}], "]"}]], "Input",
 CellTags->"SUNDelta",
 CellLabel->"In[1619]:="],

Cell[BoxData[
 FormBox[
  SubscriptBox["\[Delta]", 
   RowBox[{
    FormBox[
     FormBox["a",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["b",
      TraditionalForm],
     TraditionalForm]}]], TraditionalForm]], "Output",
 CellChangeTimes->{3.50097041947303*^9},
 CellTags->"SUNDelta",
 CellLabel->"Out[1619]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"SUNDelta", "[", 
   RowBox[{"a", ",", "b"}], "]"}], "//", "StandardForm"}]], "Input",
 CellTags->"SUNDelta",
 CellLabel->"In[1620]:="],

Cell[BoxData[
 RowBox[{"SUNDelta", "[", 
  RowBox[{"a", ",", "b"}], "]"}]], "Output",
 CellChangeTimes->{3.500970419817078*^9},
 CellTags->"SUNDelta",
 CellLabel->"Out[1620]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"SUNDelta", "[", 
    RowBox[{"a", ",", "b"}], "]"}], "//", "FCI"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"SUNDelta",
 CellLabel->"In[1621]:="],

Cell[BoxData[
 RowBox[{"SUNDelta", "[", 
  RowBox[{
   RowBox[{"SUNIndex", "[", "a", "]"}], ",", 
   RowBox[{"SUNIndex", "[", "b", "]"}]}], "]"}]], "Output",
 CellChangeTimes->{3.50097041987988*^9},
 CellTags->"SUNDelta",
 CellLabel->"Out[1621]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"SUNDelta", "[", 
     RowBox[{"a", ",", "b"}], "]"}], "//", "FCI"}], "//", "FCE"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"SUNDelta",
 CellLabel->"In[1622]:="],

Cell[BoxData[
 RowBox[{"SD", "[", 
  RowBox[{"a", ",", "b"}], "]"}]], "Output",
 CellChangeTimes->{3.500970420045053*^9},
 CellTags->"SUNDelta",
 CellLabel->"Out[1622]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"SD", "[", 
    RowBox[{"a", ",", "b"}], "]"}], "//", "FCI"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"SUNDelta",
 CellLabel->"In[1623]:="],

Cell[BoxData[
 RowBox[{"SUNDelta", "[", 
  RowBox[{
   RowBox[{"SUNIndex", "[", "a", "]"}], ",", 
   RowBox[{"SUNIndex", "[", "b", "]"}]}], "]"}]], "Output",
 CellChangeTimes->{3.500970420129467*^9},
 CellTags->"SUNDelta",
 CellLabel->"Out[1623]//StandardForm="]
}, Open  ]],

Cell["\<\
Symbolic arguments to SUNDelta are transformed into the data type SUNIndex \
and integer arguments are transformed to ExplicitSUNIndex. The difference is \
that SUNSimplify will only sum over symbolic indices.\
\>", "Text",
 CellTags->"SUNDelta"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"SUNDelta", "[", 
    RowBox[{"a", ",", "2"}], "]"}], 
   RowBox[{"SUNDelta", "[", 
    RowBox[{"a", ",", "b"}], "]"}], 
   RowBox[{"SUNDelta", "[", 
    RowBox[{"c", ",", "2"}], "]"}]}], "//", "SUNSimplify"}]], "Input",
 CellTags->"SUNDelta",
 CellLabel->"In[1624]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["\[Delta]", 
    RowBox[{
     FormBox["2",
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm]}]], " ", 
   SubscriptBox["\[Delta]", 
    RowBox[{
     FormBox["2",
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm]}]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970420345529*^9},
 CellTags->"SUNDelta",
 CellLabel->"Out[1624]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "StandardForm"}]], "Input",
 CellTags->"SUNDelta",
 CellLabel->"In[1625]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"SUNDelta", "[", 
   RowBox[{
    RowBox[{"ExplicitSUNIndex", "[", "2", "]"}], ",", 
    RowBox[{"SUNIndex", "[", "b", "]"}]}], "]"}], " ", 
  RowBox[{"SUNDelta", "[", 
   RowBox[{
    RowBox[{"ExplicitSUNIndex", "[", "2", "]"}], ",", 
    RowBox[{"SUNIndex", "[", "c", "]"}]}], "]"}]}]], "Output",
 CellChangeTimes->{3.500970420424706*^9},
 CellTags->"SUNDelta",
 CellLabel->"Out[1625]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"SUNDelta", "[", 
    RowBox[{"1", ",", "2"}], "]"}], "//", "FCI"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"SUNDelta",
 CellLabel->"In[1626]:="],

Cell[BoxData[
 RowBox[{"SUNDelta", "[", 
  RowBox[{
   RowBox[{"ExplicitSUNIndex", "[", "1", "]"}], ",", 
   RowBox[{"ExplicitSUNIndex", "[", "2", "]"}]}], "]"}]], "Output",
 CellChangeTimes->{3.5009704205000668`*^9},
 CellTags->"SUNDelta",
 CellLabel->"Out[1626]//StandardForm="]
}, Open  ]]
}, Open  ]],

Cell["SUNDeltaContract", "ObjectName",
 CellTags->"SUNDeltaContract"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SUNDeltaContract"],

Cell[TextData[{
 "SUNDeltaContract[expr] substitues for all SUNDelta in expr \
SUNDeltaContract, contracts the SUN(N) indices and resubstitutes SUNDelta. \
SUNDeltaContract[i, j] is the Kronecker-delta for SU(",
 StyleBox["N",
  FontSlant->"Italic"],
 ") with contraction properties. SUNDeltaContract wraps also the head \
SUNIndex around its arguments."
}], "Text",
 CellTags->"SUNDeltaContract"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["SUNDelta",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SUNDelta",
  ButtonNote->"SUNDelta"],
 ", ",
 ButtonBox["SUNIndex",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SUNIndex",
  ButtonNote->"SUNIndex"],
 "."
}], "Text",
 CellTags->"SUNDeltaContract"]
}, Open  ]],

Cell["SUNF", "ObjectName",
 CellTags->"SUNF"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SUNF"],

Cell[TextData[{
 "SUNF[a,b,c] are the structure constants of SU(",
 StyleBox["N",
  FontSlant->"Italic"],
 "). The arguments a,b,c should be of symbolic type."
}], "Text",
 CellTags->"SUNF"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "SUNF", "]"}]], "Input",
 CellTags->"SUNF",
 CellLabel->"In[1627]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"Explicit", "\[Rule]", "False"}], "}"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970420594005*^9},
 CellTags->"SUNF",
 CellLabel->"Out[1627]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["SUND",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SUND",
  ButtonNote->"SUND"],
 ", ",
 ButtonBox["SUNDelta",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SUNDelta",
  ButtonNote->"SUNDelta"],
 ", ",
 ButtonBox["SUNIndex",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SUNIndex",
  ButtonNote->"SUNIndex"],
 ", ",
 ButtonBox["SUNSimplify",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SUNSimplify",
  ButtonNote->"SUNSimplify"],
 ", ",
 ButtonBox["SUNT",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SUNT",
  ButtonNote->"SUNT"],
 ", ",
 ButtonBox["Trick",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Trick",
  ButtonNote->"Trick"],
 "."
}], "Text",
 CellTags->"SUNF"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"SUNF"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t1", "=", 
  RowBox[{
   RowBox[{
    RowBox[{"SUNF", "[", 
     RowBox[{"a", ",", "b", ",", "c"}], "]"}], "x"}], "+", 
   RowBox[{
    RowBox[{"SUNF", "[", 
     RowBox[{"b", ",", "a", ",", "c"}], "]"}], "y"}]}]}]], "Input",
 CellTags->"SUNF",
 CellLabel->"In[1628]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"x", " ", 
    SubscriptBox["f", 
     RowBox[{"a", "b", "c"}]]}], "+", 
   RowBox[{"y", " ", 
    SubscriptBox["f", 
     RowBox[{"b", "a", "c"}]]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970420742732*^9},
 CellTags->"SUNF",
 CellLabel->"Out[1628]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Calc", "[", "t1", "]"}]], "Input",
 CellTags->"SUNF",
 CellLabel->"In[1629]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"x", " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm]}]]}], "-", 
   RowBox[{"y", " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm]}]]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.50097042099436*^9},
 CellTags->"SUNF",
 CellLabel->"Out[1629]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNSimplify", "[", "t1", "]"}]], "Input",
 CellTags->"SUNF",
 CellLabel->"In[1630]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"(", 
    RowBox[{"x", "-", "y"}], ")"}], " ", 
   SubscriptBox["f", 
    RowBox[{
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm]}]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970421158595*^9},
 CellTags->"SUNF",
 CellLabel->"Out[1630]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNF", "[", 
  RowBox[{"a", ",", "a", ",", "b"}], "]"}]], "Input",
 CellTags->"SUNF",
 CellLabel->"In[1631]:="],

Cell[BoxData[
 FormBox[
  SubscriptBox["f", 
   RowBox[{"a", "a", "b"}]], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970421236178*^9},
 CellTags->"SUNF",
 CellLabel->"Out[1631]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"SUNF", "[", 
   RowBox[{"a", ",", "a", ",", "b"}], "]"}], "//", "Calc"}]], "Input",
 CellTags->"SUNF",
 CellLabel->"In[1632]:="],

Cell[BoxData[
 FormBox["0", TraditionalForm]], "Output",
 CellChangeTimes->{3.500970421342801*^9},
 CellTags->"SUNF",
 CellLabel->"Out[1632]="]
}, Open  ]],

Cell[TextData[{
 "This is a consequence of the usual choice for the normalization of the ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["T", "a"], " ", "generatorsin", " ", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"see", "  ", "SUNT"}], ")"}], "."}]}], TraditionalForm]]]
}], "Text",
 CellTags->"SUNF"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNF", "[", 
  RowBox[{"a", ",", "b", ",", "c", ",", 
   RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"SUNF",
 CellLabel->"In[1633]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"2", " ", "\[ImaginaryI]", " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"tr", "(", 
      FormBox[
       RowBox[{
        SubscriptBox["T", 
         FormBox[
          FormBox["a",
           TraditionalForm],
          TraditionalForm]], ".", 
        SubscriptBox["T", 
         FormBox[
          FormBox["c",
           TraditionalForm],
          TraditionalForm]], ".", 
        SubscriptBox["T", 
         FormBox[
          FormBox["b",
           TraditionalForm],
          TraditionalForm]]}],
       TraditionalForm], ")"}], "-", 
     RowBox[{"tr", "(", 
      FormBox[
       RowBox[{
        SubscriptBox["T", 
         FormBox[
          FormBox["a",
           TraditionalForm],
          TraditionalForm]], ".", 
        SubscriptBox["T", 
         FormBox[
          FormBox["b",
           TraditionalForm],
          TraditionalForm]], ".", 
        SubscriptBox["T", 
         FormBox[
          FormBox["c",
           TraditionalForm],
          TraditionalForm]]}],
       TraditionalForm], ")"}]}], ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970421501608*^9},
 CellTags->"SUNF",
 CellLabel->"Out[1633]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNSimplify", "[", 
  RowBox[{
   RowBox[{"SUNF", "[", 
    RowBox[{"a", ",", "b", ",", "c"}], "]"}], " ", 
   RowBox[{"SUNF", "[", 
    RowBox[{"a", ",", "b", ",", "d"}], "]"}]}], "]"}]], "Input",
 CellTags->"SUNF",
 CellLabel->"In[1634]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["C", "A"], " ", 
   SubscriptBox["\[Delta]", 
    RowBox[{
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["d",
       TraditionalForm],
      TraditionalForm]}]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970422071732*^9},
 CellTags->"SUNF",
 CellLabel->"Out[1634]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNSimplify", "[", 
  RowBox[{
   RowBox[{"SUNF", "[", 
    RowBox[{"a", ",", "b", ",", "c"}], "]"}], ",", 
   RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"SUNF",
 CellLabel->"In[1635]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", "2"}], " ", "\[ImaginaryI]", " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"tr", "(", 
      FormBox[
       RowBox[{
        SubscriptBox["T", 
         FormBox[
          FormBox["a",
           TraditionalForm],
          TraditionalForm]], ".", 
        SubscriptBox["T", 
         FormBox[
          FormBox["b",
           TraditionalForm],
          TraditionalForm]], ".", 
        SubscriptBox["T", 
         FormBox[
          FormBox["c",
           TraditionalForm],
          TraditionalForm]]}],
       TraditionalForm], ")"}], "-", 
     RowBox[{"tr", "(", 
      FormBox[
       RowBox[{
        SubscriptBox["T", 
         FormBox[
          FormBox["b",
           TraditionalForm],
          TraditionalForm]], ".", 
        SubscriptBox["T", 
         FormBox[
          FormBox["a",
           TraditionalForm],
          TraditionalForm]], ".", 
        SubscriptBox["T", 
         FormBox[
          FormBox["c",
           TraditionalForm],
          TraditionalForm]]}],
       TraditionalForm], ")"}]}], ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970422340914*^9},
 CellTags->"SUNF",
 CellLabel->"Out[1635]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"SUNF", "[", 
   RowBox[{"a", ",", "b", ",", "c"}], "]"}], "//", "StandardForm"}]], "Input",\

 CellTags->"SUNF",
 CellLabel->"In[1636]:="],

Cell[BoxData[
 RowBox[{"SUNF", "[", 
  RowBox[{"a", ",", "b", ",", "c"}], "]"}]], "Output",
 CellChangeTimes->{3.5009704224190283`*^9},
 CellTags->"SUNF",
 CellLabel->"Out[1636]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"SUNF", "[", 
    RowBox[{"a", ",", "b", ",", "c"}], "]"}], "//", "FCI"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"SUNF",
 CellLabel->"In[1637]:="],

Cell[BoxData[
 RowBox[{"SUNF", "[", 
  RowBox[{
   RowBox[{"SUNIndex", "[", "a", "]"}], ",", 
   RowBox[{"SUNIndex", "[", "b", "]"}], ",", 
   RowBox[{"SUNIndex", "[", "c", "]"}]}], "]"}]], "Output",
 CellChangeTimes->{3.500970422494177*^9},
 CellTags->"SUNF",
 CellLabel->"Out[1637]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"SUNF", "[", 
     RowBox[{"a", ",", "b", ",", "c"}], "]"}], "//", "FCI"}], "//", "FCE"}], "//",
   "StandardForm"}]], "Input",
 CellTags->"SUNF",
 CellLabel->"In[1638]:="],

Cell[BoxData[
 RowBox[{"SUNF", "[", 
  RowBox[{"a", ",", "b", ",", "c"}], "]"}]], "Output",
 CellChangeTimes->{3.500970422617675*^9},
 CellTags->"SUNF",
 CellLabel->"Out[1638]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNF", "[", 
  RowBox[{"b", ",", "a", ",", "c"}], "]"}]], "Input",
 CellTags->"SUNF",
 CellLabel->"In[1639]:="],

Cell[BoxData[
 FormBox[
  SubscriptBox["f", 
   RowBox[{"b", "a", "c"}]], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009704227783003`*^9},
 CellTags->"SUNF",
 CellLabel->"Out[1639]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"SUNF", "[", 
   RowBox[{"b", ",", "a", ",", "c"}], "]"}], "//", "FCI"}]], "Input",
 CellTags->"SUNF",
 CellLabel->"In[1640]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   SubscriptBox["f", 
    RowBox[{
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm]}]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970422913522*^9},
 CellTags->"SUNF",
 CellLabel->"Out[1640]="]
}, Open  ]]
}, Open  ]],

Cell["SUNFJacobi", "ObjectName",
 CellTags->"SUNFJacobi"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SUNFJacobi"],

Cell["\<\
SUNFJacobi is an option for SUNSimplify, indicating whether the Jacobi \
identity should be used.\
\>", "Text",
 CellTags->"SUNFJacobi"],

Cell[TextData[{
 "See also:  ",
 ButtonBox["SUNF",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SUNF",
  ButtonNote->"SUNF"],
 ", ",
 ButtonBox["SUNSimplify",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SUNSimplify",
  ButtonNote->"SUNSimplify"],
 "."
}], "Text",
 CellTags->"SUNFJacobi"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"SUNFJacobi"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"SUNF", "[", 
    RowBox[{"a", ",", "b", ",", "c"}], "]"}], 
   RowBox[{"SUNF", "[", 
    RowBox[{"e", ",", "f", ",", "c"}], "]"}]}], "//", 
  RowBox[{
   RowBox[{"SUNSimplify", "[", 
    RowBox[{"#", ",", 
     RowBox[{"SUNFJacobi", "\[Rule]", "False"}]}], "]"}], "&"}]}]], "Input",
 CellTags->"SUNFJacobi",
 CellLabel->"In[1641]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["f", 
    RowBox[{
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["e",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["f",
       TraditionalForm],
      TraditionalForm]}]], " ", 
   SubscriptBox["f", 
    RowBox[{
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm]}]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970423052745*^9},
 CellTags->"SUNFJacobi",
 CellLabel->"Out[1641]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"SUNF", "[", 
    RowBox[{"a", ",", "b", ",", "c"}], "]"}], 
   RowBox[{"SUNF", "[", 
    RowBox[{"e", ",", "f", ",", "c"}], "]"}]}], "//", 
  RowBox[{
   RowBox[{"SUNSimplify", "[", 
    RowBox[{"#", ",", 
     RowBox[{"SUNFJacobi", "\[Rule]", "True"}]}], "]"}], "&"}]}]], "Input",
 CellTags->"SUNFJacobi",
 CellLabel->"In[1642]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["f",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["e",
        TraditionalForm],
       TraditionalForm]}]]}], "-", 
   RowBox[{
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["f",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["e",
        TraditionalForm],
       TraditionalForm]}]]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970423258638*^9},
 CellTags->"SUNFJacobi",
 CellLabel->"Out[1642]="]
}, Open  ]]
}, Open  ]],

Cell["SUNIndex", "ObjectName",
 CellTags->"SUNIndex"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SUNIndex"],

Cell[TextData[{
 "SUNIndex is the head of SU(",
 StyleBox["N",
  FontSlant->"Italic"],
 ") indices."
}], "Text",
 CellTags->"SUNIndex"],

Cell[TextData[{
 "See also: ",
 ButtonBox["ExplicitSUNIndex",
  BaseStyle->"AddOnsLink",
  ButtonData:>"ExplicitSUNIndex",
  ButtonNote->"ExplicitSUNIndex"],
 ", ",
 ButtonBox["SUNDelta",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SUNDelta",
  ButtonNote->"SUNDelta"],
 ", ",
 ButtonBox["SUNF",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SUNF",
  ButtonNote->"SUNF"],
 "."
}], "Text",
 CellTags->"SUNIndex"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"SUNIndex"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNIndex", "[", "i", "]"}]], "Input",
 CellTags->"SUNIndex",
 CellLabel->"In[1643]:="],

Cell[BoxData[
 FormBox["i", TraditionalForm]], "Output",
 CellChangeTimes->{3.500970423327673*^9},
 CellTags->"SUNIndex",
 CellLabel->"Out[1643]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "StandardForm"}]], "Input",
 CellTags->"SUNIndex",
 CellLabel->"In[1644]:="],

Cell[BoxData[
 RowBox[{"SUNIndex", "[", "i", "]"}]], "Output",
 CellChangeTimes->{3.5009704233922033`*^9},
 CellTags->"SUNIndex",
 CellLabel->"Out[1644]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"SUNDelta", "[", 
    RowBox[{"i", ",", "j"}], "]"}], "//", "FCI"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"SUNIndex",
 CellLabel->"In[1645]:="],

Cell[BoxData[
 RowBox[{"SUNDelta", "[", 
  RowBox[{
   RowBox[{"SUNIndex", "[", "i", "]"}], ",", 
   RowBox[{"SUNIndex", "[", "j", "]"}]}], "]"}]], "Output",
 CellChangeTimes->{3.500970423429905*^9},
 CellTags->"SUNIndex",
 CellLabel->"Out[1645]//StandardForm="]
}, Open  ]]
}, Open  ]],

Cell["SUNIndexRename", "ObjectName",
 CellTags->"SUNIndexRename"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SUNIndexRename"],

Cell["\<\
SUNIndexRename is an option of SUNSimplify. If set to False, no automatic \
renaming of dummy indices is done.\
\>", "Text",
 CellTags->"SUNIndexRename"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["SUNSimplify",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SUNSimplify",
  ButtonNote->"SUNSimplify"],
 ", ",
 ButtonBox["SUNIndex",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SUNIndex",
  ButtonNote->"SUNIndex"],
 "."
}], "Text",
 CellTags->"SUNIndexRename"]
}, Open  ]],

Cell["SUNN", "ObjectName",
 CellTags->"SUNN"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SUNN"],

Cell[TextData[{
 "SUNN denotes the number of colors. Trick[SUNDelta[a, a]] yields (",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["SUNN", "2"], TraditionalForm]]],
 " -1)."
}], "Text",
 CellTags->"SUNN"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["SUNSimplify",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SUNSimplify",
  ButtonNote->"SUNSimplify"],
 ", ",
 ButtonBox["Trick",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Trick",
  ButtonNote->"Trick"],
 ", ",
 ButtonBox["SUNIndex",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SUNIndex",
  ButtonNote->"SUNIndex"],
 ", ",
 ButtonBox["CA",
  BaseStyle->"AddOnsLink",
  ButtonData:>"CA",
  ButtonNote->"CA"],
 ", ",
 ButtonBox["CF",
  BaseStyle->"AddOnsLink",
  ButtonData:>"CF",
  ButtonNote->"CF"],
 "."
}], "Text",
 CellTags->"SUNN"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"SUNN"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNSimplify", "[", 
  RowBox[{
   RowBox[{"SUNDelta", "[", 
    RowBox[{
     RowBox[{"SUNIndex", "[", "a", "]"}], ",", " ", 
     RowBox[{"SUNIndex", "[", "a", "]"}]}], "]"}], ",", 
   RowBox[{"SUNNToCACF", "\[Rule]", "False"}]}], "]"}]], "Input",
 CellTags->"SUNN",
 CellLabel->"In[1646]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["N", "2"], "-", "1"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970423523341*^9},
 CellTags->"SUNN",
 CellLabel->"Out[1646]="]
}, Open  ]]
}, Open  ]],

Cell["SUNNToCACF", "ObjectName",
 CellTags->"SUNNToCACF"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SUNNToCACF"],

Cell["\<\
SUNNToCACF is an option of SUNSimplify. If set to True, the Casimir operator \
eigenvalues CA (=N) and CF (=(N^2-1)/(2 N)) are introduced.\
\>", "Text",
 CellTags->"SUNNToCACF"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["SUNSimplify",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SUNSimplify",
  ButtonNote->"SUNSimplify"],
 ", ",
 ButtonBox["Trick",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Trick",
  ButtonNote->"Trick"],
 ", ",
 ButtonBox["SUNN",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SUNN",
  ButtonNote->"SUNN"],
 ", ",
 ButtonBox["CA",
  BaseStyle->"AddOnsLink",
  ButtonData:>"CA",
  ButtonNote->"CA"],
 ", ",
 ButtonBox["CF",
  BaseStyle->"AddOnsLink",
  ButtonData:>"CF",
  ButtonNote->"CF"],
 "."
}], "Text",
 CellTags->"SUNNToCACF"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"SUNNToCACF"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNSimplify", "[", 
  RowBox[{
   RowBox[{"SUNDelta", "[", 
    RowBox[{
     RowBox[{"SUNIndex", "[", "a", "]"}], ",", " ", 
     RowBox[{"SUNIndex", "[", "a", "]"}]}], "]"}], ",", 
   RowBox[{"SUNNToCACF", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"SUNNToCACF",
 CellLabel->"In[1647]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"2", " ", 
   SubscriptBox["C", "A"], " ", 
   SubscriptBox["C", "F"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970423647984*^9},
 CellTags->"SUNNToCACF",
 CellLabel->"Out[1647]="]
}, Open  ]]
}, Open  ]],

Cell["SUNSimplify", "ObjectName",
 CellTags->"SUNSimplify"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SUNSimplify"],

Cell["\<\
SUNSimplify simplifies products of SUNT (and complex conjugated) matrices. \
Basic renaming of dummy indices is done. If the option SUNTrace is set to \
False, then any SUNT-matrices are taken out of DiracTrace[...]; otherwise a \
color-trace is taken (by SUNTrace) before taking the SUN-objects in front of \
DiracTrace[...].\
\>", "Text",
 CellTags->"SUNSimplify"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "SUNSimplify", "]"}]], "Input",
 CellTags->"SUNSimplify",
 CellLabel->"In[1648]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Expanding", "\[Rule]", "False"}], ",", 
    RowBox[{"Explicit", "\[Rule]", "False"}], ",", 
    RowBox[{"Factoring", "\[Rule]", "False"}], ",", 
    RowBox[{"SUNIndexRename", "\[Rule]", "True"}], ",", 
    RowBox[{"SUNFJacobi", "\[Rule]", "False"}], ",", 
    RowBox[{"SUNNToCACF", "\[Rule]", "True"}], ",", 
    RowBox[{"SUNTrace", "\[Rule]", "False"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970423768125*^9},
 CellTags->"SUNSimplify",
 CellLabel->"Out[1648]="]
}, Open  ]],

Cell[TextData[{
 "See also:  ",
 ButtonBox["Trick",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Trick",
  ButtonNote->"Trick"],
 ", ",
 ButtonBox["CalcColorFactor",
  BaseStyle->"AddOnsLink",
  ButtonData:>"CalcColorFactor",
  ButtonNote->"CalcColorFactor"],
 "."
}], "Text",
 CellTags->"SUNSimplify"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"SUNSimplify"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t1", "=", 
  RowBox[{
   RowBox[{"SUNDelta", "[", 
    RowBox[{"a", ",", "b"}], "]"}], " ", 
   RowBox[{"SUNDelta", "[", 
    RowBox[{"b", ",", "c"}], "]"}]}]}]], "Input",
 CellTags->"SUNSimplify",
 CellLabel->"In[1649]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["\[Delta]", 
    RowBox[{"a", "b"}]], " ", 
   SubscriptBox["\[Delta]", 
    RowBox[{"b", "c"}]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970423814254*^9},
 CellTags->"SUNSimplify",
 CellLabel->"Out[1649]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNSimplify", "[", "t1", "]"}]], "Input",
 CellTags->"SUNSimplify",
 CellLabel->"In[1650]:="],

Cell[BoxData[
 FormBox[
  SubscriptBox["\[Delta]", 
   RowBox[{
    FormBox[
     FormBox["a",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["c",
      TraditionalForm],
     TraditionalForm]}]], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009704239127913`*^9},
 CellTags->"SUNSimplify",
 CellLabel->"Out[1650]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t2", "=", 
  RowBox[{
   RowBox[{"SUNT", "[", "a", "]"}], ".", 
   RowBox[{"SUNT", "[", "a", "]"}]}]}]], "Input",
 CellTags->"SUNSimplify",
 CellLabel->"In[1651]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["T", 
    FormBox["a",
     TraditionalForm]], ".", 
   SubscriptBox["T", 
    FormBox["a",
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970424133328*^9},
 CellTags->"SUNSimplify",
 CellLabel->"Out[1651]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNSimplify", "[", "t2", "]"}]], "Input",
 CellTags->"SUNSimplify",
 CellLabel->"In[1652]:="],

Cell[BoxData[
 FormBox[
  SubscriptBox["C", "F"], TraditionalForm]], "Output",
 CellChangeTimes->{3.50097042430081*^9},
 CellTags->"SUNSimplify",
 CellLabel->"Out[1652]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNSimplify", "[", 
  RowBox[{"t2", ",", 
   RowBox[{"SUNNToCACF", "\[Rule]", "False"}]}], "]"}]], "Input",
 CellTags->"SUNSimplify",
 CellLabel->"In[1653]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{
    SuperscriptBox["N", "2"], "-", "1"}], 
   RowBox[{"2", " ", "N"}]], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009704244193296`*^9},
 CellTags->"SUNSimplify",
 CellLabel->"Out[1653]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t3", "=", 
  RowBox[{
   RowBox[{"SUNF", "[", 
    RowBox[{"a", ",", "r", ",", "s"}], "]"}], 
   RowBox[{"SUNF", "[", 
    RowBox[{"b", ",", "r", ",", "s"}], "]"}]}]}]], "Input",
 CellTags->"SUNSimplify",
 CellLabel->"In[1654]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["f", 
    RowBox[{"a", "r", "s"}]], " ", 
   SubscriptBox["f", 
    RowBox[{"b", "r", "s"}]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.50097042449762*^9},
 CellTags->"SUNSimplify",
 CellLabel->"Out[1654]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNSimplify", "[", "t3", "]"}]], "Input",
 CellTags->"SUNSimplify",
 CellLabel->"In[1655]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["C", "A"], " ", 
   SubscriptBox["\[Delta]", 
    RowBox[{
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm]}]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970424588272*^9},
 CellTags->"SUNSimplify",
 CellLabel->"Out[1655]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t4", "=", 
  RowBox[{
   RowBox[{"SUNF", "[", 
    RowBox[{"a", ",", "b", ",", "c"}], "]"}], " ", ".", " ", 
   RowBox[{"SUNF", "[", 
    RowBox[{"a", ",", "b", ",", "c"}], "]"}]}]}]], "Input",
 CellTags->"SUNSimplify",
 CellLabel->"In[1656]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["f", 
    RowBox[{"a", "b", "c"}]], ".", 
   SubscriptBox["f", 
    RowBox[{"a", "b", "c"}]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970424733386*^9},
 CellTags->"SUNSimplify",
 CellLabel->"Out[1656]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNSimplify", "[", "t4", "]"}]], "Input",
 CellTags->"SUNSimplify",
 CellLabel->"In[1657]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"2", " ", 
   SubsuperscriptBox["C", "A", "2"], " ", 
   SubscriptBox["C", "F"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970424904858*^9},
 CellTags->"SUNSimplify",
 CellLabel->"Out[1657]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t5", "=", 
  RowBox[{
   RowBox[{"SUNF", "[", 
    RowBox[{"a", ",", "b", ",", "c"}], "]"}], " ", 
   RowBox[{"SUNF", "[", 
    RowBox[{"d", ",", "b", ",", "c"}], "]"}]}]}]], "Input",
 CellTags->"SUNSimplify",
 CellLabel->"In[1658]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["f", 
    RowBox[{"a", "b", "c"}]], " ", 
   SubscriptBox["f", 
    RowBox[{"d", "b", "c"}]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970424985692*^9},
 CellTags->"SUNSimplify",
 CellLabel->"Out[1658]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNSimplify", "[", "t5", "]"}]], "Input",
 CellTags->"SUNSimplify",
 CellLabel->"In[1659]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["C", "A"], " ", 
   SubscriptBox["\[Delta]", 
    RowBox[{
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["d",
       TraditionalForm],
      TraditionalForm]}]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970425103318*^9},
 CellTags->"SUNSimplify",
 CellLabel->"Out[1659]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t6", "=", 
  RowBox[{
   RowBox[{"SUNF", "[", 
    RowBox[{"a", ",", "b", ",", "c"}], "]"}], " ", 
   RowBox[{"SUND", "[", 
    RowBox[{"d", ",", "b", ",", "c"}], "]"}]}]}]], "Input",
 CellTags->"SUNSimplify",
 CellLabel->"In[1660]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["d", 
    RowBox[{"b", "c", "d"}]], " ", 
   SubscriptBox["f", 
    RowBox[{"a", "b", "c"}]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970425256611*^9},
 CellTags->"SUNSimplify",
 CellLabel->"Out[1660]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNSimplify", "[", 
  RowBox[{"t6", ",", 
   RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"SUNSimplify",
 CellLabel->"In[1661]:="],

Cell[BoxData[
 FormBox["0", TraditionalForm]], "Output",
 CellChangeTimes->{3.50097042534932*^9},
 CellTags->"SUNSimplify",
 CellLabel->"Out[1661]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"SUNSimplify", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"SUND", "[", 
      RowBox[{"a", ",", "b", ",", "c"}], "]"}], " ", 
     RowBox[{"SUND", "[", 
      RowBox[{"a", ",", "b", ",", "c"}], "]"}]}], ",", 
    RowBox[{"SUNNToCACF", "\[Rule]", "False"}]}], "]"}], "//", 
  "Factor2"}]], "Input",
 CellTags->"SUNSimplify",
 CellLabel->"In[1662]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{
    RowBox[{"(", 
     RowBox[{"1", "-", 
      SuperscriptBox["N", "2"]}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{"4", "-", 
      SuperscriptBox["N", "2"]}], ")"}]}], "N"], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970425446187*^9},
 CellTags->"SUNSimplify",
 CellLabel->"Out[1662]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"SUNSimplify", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"SUND", "[", 
      RowBox[{"a", ",", "b", ",", "c"}], "]"}], " ", 
     RowBox[{"SUND", "[", 
      RowBox[{"e", ",", "b", ",", "c"}], "]"}]}], ",", 
    RowBox[{"SUNNToCACF", "\[Rule]", "False"}]}], "]"}], "//", 
  "FullSimplify"}]], "Input",
 CellTags->"SUNSimplify",
 CellLabel->"In[1663]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox["N", "2"], "-", "4"}], ")"}], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["e",
        TraditionalForm],
       TraditionalForm]}]]}], "N"], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009704255391197`*^9},
 CellTags->"SUNSimplify",
 CellLabel->"Out[1663]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNSimplify", "[", 
  RowBox[{
   RowBox[{"SUNF", "[", 
    RowBox[{"a", ",", "b", ",", "c"}], "]"}], ",", 
   RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"SUNSimplify",
 CellLabel->"In[1664]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", "2"}], " ", "\[ImaginaryI]", " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"tr", "(", 
      FormBox[
       RowBox[{
        SubscriptBox["T", 
         FormBox[
          FormBox["a",
           TraditionalForm],
          TraditionalForm]], ".", 
        SubscriptBox["T", 
         FormBox[
          FormBox["b",
           TraditionalForm],
          TraditionalForm]], ".", 
        SubscriptBox["T", 
         FormBox[
          FormBox["c",
           TraditionalForm],
          TraditionalForm]]}],
       TraditionalForm], ")"}], "-", 
     RowBox[{"tr", "(", 
      FormBox[
       RowBox[{
        SubscriptBox["T", 
         FormBox[
          FormBox["b",
           TraditionalForm],
          TraditionalForm]], ".", 
        SubscriptBox["T", 
         FormBox[
          FormBox["a",
           TraditionalForm],
          TraditionalForm]], ".", 
        SubscriptBox["T", 
         FormBox[
          FormBox["c",
           TraditionalForm],
          TraditionalForm]]}],
       TraditionalForm], ")"}]}], ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970425614564*^9},
 CellTags->"SUNSimplify",
 CellLabel->"Out[1664]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNSimplify", "[", 
  RowBox[{
   RowBox[{"SUND", "[", 
    RowBox[{"a", ",", "b", ",", "c"}], "]"}], ",", 
   RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"SUNSimplify",
 CellLabel->"In[1665]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"2", " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"tr", "(", 
      FormBox[
       RowBox[{
        SubscriptBox["T", 
         FormBox[
          FormBox["a",
           TraditionalForm],
          TraditionalForm]], ".", 
        SubscriptBox["T", 
         FormBox[
          FormBox["b",
           TraditionalForm],
          TraditionalForm]], ".", 
        SubscriptBox["T", 
         FormBox[
          FormBox["c",
           TraditionalForm],
          TraditionalForm]]}],
       TraditionalForm], ")"}], "+", 
     RowBox[{"tr", "(", 
      FormBox[
       RowBox[{
        SubscriptBox["T", 
         FormBox[
          FormBox["b",
           TraditionalForm],
          TraditionalForm]], ".", 
        SubscriptBox["T", 
         FormBox[
          FormBox["a",
           TraditionalForm],
          TraditionalForm]], ".", 
        SubscriptBox["T", 
         FormBox[
          FormBox["c",
           TraditionalForm],
          TraditionalForm]]}],
       TraditionalForm], ")"}]}], ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.50097042573279*^9},
 CellTags->"SUNSimplify",
 CellLabel->"Out[1665]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNSimplify", "[", 
  RowBox[{
   RowBox[{"SUNF", "[", 
    RowBox[{"a", ",", "b", ",", "c"}], "]"}], " ", 
   RowBox[{"SUNT", "[", 
    RowBox[{"c", ",", "b", ",", "a"}], "]"}]}], "]"}]], "Input",
 CellTags->"SUNSimplify",
 CellLabel->"In[1666]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    FractionBox["1", "2"]}], " ", "\[ImaginaryI]", " ", 
   SubscriptBox["C", "A"], " ", 
   SubscriptBox["C", "F"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.50097042594168*^9},
 CellTags->"SUNSimplify",
 CellLabel->"Out[1666]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t7", "=", 
  RowBox[{
   RowBox[{
    RowBox[{"SUNF", "[", 
     RowBox[{"a", ",", "b", ",", "e"}], "]"}], 
    RowBox[{"SUNF", "[", 
     RowBox[{"c", ",", "d", ",", "e"}], "]"}]}], "+", 
   RowBox[{
    RowBox[{"SUNF", "[", 
     RowBox[{"a", ",", "b", ",", "z"}], "]"}], 
    RowBox[{"SUNF", "[", 
     RowBox[{"c", ",", "d", ",", "z"}], "]"}]}]}]}]], "Input",
 CellTags->"SUNSimplify",
 CellLabel->"In[1667]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox["f", 
     RowBox[{"a", "b", "e"}]], " ", 
    SubscriptBox["f", 
     RowBox[{"c", "d", "e"}]]}], "+", 
   RowBox[{
    SubscriptBox["f", 
     RowBox[{"a", "b", "z"}]], " ", 
    SubscriptBox["f", 
     RowBox[{"c", "d", "z"}]]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970426042099*^9},
 CellTags->"SUNSimplify",
 CellLabel->"Out[1667]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNSimplify", "[", 
  RowBox[{"t7", ",", 
   RowBox[{"Explicit", "\[Rule]", "False"}]}], "]"}]], "Input",
 CellTags->"SUNSimplify",
 CellLabel->"In[1668]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"2", " ", 
   SubscriptBox["f", 
    RowBox[{
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["e",
       TraditionalForm],
      TraditionalForm]}]], " ", 
   SubscriptBox["f", 
    RowBox[{
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["d",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["e",
       TraditionalForm],
      TraditionalForm]}]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970426148494*^9},
 CellTags->"SUNSimplify",
 CellLabel->"Out[1668]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNSimplify", "[", 
  RowBox[{"t7", ",", 
   RowBox[{"Explicit", "\[Rule]", "False"}], ",", 
   RowBox[{"SUNIndexRename", "\[Rule]", "False"}]}], "]"}]], "Input",
 CellTags->"SUNSimplify",
 CellLabel->"In[1669]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["e",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["d",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["e",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["z",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["d",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["z",
        TraditionalForm],
       TraditionalForm]}]]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.50097042635144*^9},
 CellTags->"SUNSimplify",
 CellLabel->"Out[1669]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNSimplify", "[", 
  RowBox[{"1", "-", 
   RowBox[{"SUNDelta", "[", 
    RowBox[{"i", ",", "i"}], "]"}]}], "]"}]], "Input",
 CellTags->"SUNSimplify",
 CellLabel->"In[1670]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"2", "-", 
   SubsuperscriptBox["C", "A", "2"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970426653536*^9},
 CellTags->"SUNSimplify",
 CellLabel->"Out[1670]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t8", "=", 
  RowBox[{"DiracTrace", "[", 
   RowBox[{
    RowBox[{"f", "[", 
     RowBox[{"SUNIndex", "[", "a", "]"}], "]"}], 
    RowBox[{
     RowBox[{"DiracMatrix", "[", "\[Mu]", "]"}], ".", 
     RowBox[{"DiracMatrix", "[", "\[Nu]", "]"}]}]}], "]"}]}]], "Input",
 CellTags->"SUNSimplify",
 CellLabel->"In[1671]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"tr", "(", 
   FormBox[
    RowBox[{
     RowBox[{"f", "(", 
      FormBox["a",
       TraditionalForm], ")"}], " ", 
     RowBox[{
      SuperscriptBox["\[Gamma]", 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], ".", 
      SuperscriptBox["\[Gamma]", 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]]}]}],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970426758799*^9},
 CellTags->"SUNSimplify",
 CellLabel->"Out[1671]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNSimplify", "[", 
  RowBox[{"t8", ",", 
   RowBox[{"SUNTrace", "\[Rule]", "False"}]}], "]"}]], "Input",
 CellTags->"SUNSimplify",
 CellLabel->"In[1672]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"tr", "(", 
   FormBox[
    RowBox[{
     RowBox[{"f", "(", 
      FormBox["a",
       TraditionalForm], ")"}], " ", 
     RowBox[{
      SuperscriptBox["\[Gamma]", 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], ".", 
      SuperscriptBox["\[Gamma]", 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]]}]}],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970427248094*^9},
 CellTags->"SUNSimplify",
 CellLabel->"Out[1672]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNSimplify", "[", 
  RowBox[{"t8", ",", 
   RowBox[{"SUNTrace", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"SUNSimplify",
 CellLabel->"In[1673]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"f", "(", 
    FormBox["a",
     TraditionalForm], ")"}], " ", 
   SubscriptBox["C", "A"], " ", 
   RowBox[{"tr", "(", 
    FormBox[
     RowBox[{
      SuperscriptBox["\[Gamma]", 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], ".", 
      SuperscriptBox["\[Gamma]", 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]]}],
     TraditionalForm], ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970427420506*^9},
 CellTags->"SUNSimplify",
 CellLabel->"Out[1673]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"Clear", "[", 
  RowBox[{
  "t1", ",", "t2", ",", "t3", ",", "t4", ",", "t5", ",", "t6", ",", "t7", ",",
    "t8"}], "]"}]], "Input",
 CellTags->"SUNSimplify",
 CellLabel->"In[1674]:="]
}, Open  ]],

Cell["SUNT", "ObjectName",
 CellTags->"SUNT"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SUNT"],

Cell[TextData[{
 "SUNT[a] is the SU(",
 StyleBox["N",
  FontSlant->"Italic"],
 ") ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["T", "a"], TraditionalForm]]],
 " generator in the fundamental representation."
}], "Text",
 CellTags->"SUNT"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["CA",
  BaseStyle->"AddOnsLink",
  ButtonData:>"CA",
  ButtonNote->"CA"],
 ", ",
 ButtonBox["CF",
  BaseStyle->"AddOnsLink",
  ButtonData:>"CF",
  ButtonNote->"CF"],
 ", ",
 ButtonBox["SUND",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SUND",
  ButtonNote->"SUND"],
 ", ",
 ButtonBox["SUNDelta",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SUNDelta",
  ButtonNote->"SUNDelta"],
 ", ",
 ButtonBox["SUNF",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SUNF",
  ButtonNote->"SUNF"],
 ", ",
 ButtonBox["SUNSimplify",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SUNSimplify",
  ButtonNote->"SUNSimplify"],
 "."
}], "Text",
 CellTags->"SUNT"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"SUNT"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNT", "[", "a", "]"}]], "Input",
 CellTags->"SUNT",
 CellLabel->"In[1675]:="],

Cell[BoxData[
 FormBox[
  SubscriptBox["T", 
   FormBox["a",
    TraditionalForm]], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970427518111*^9},
 CellTags->"SUNT",
 CellLabel->"Out[1675]="]
}, Open  ]],

Cell[TextData[{
 "Since ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["T", "a"], TraditionalForm]]],
 " is a noncommutative object, products have to separated by a dot (.)."
}], "Text",
 CellTags->"SUNT"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"SUNT", "[", "a", "]"}], ".", " ", 
  RowBox[{"SUNT", "[", "b", "]"}], ".", " ", 
  RowBox[{"SUNT", "[", "c", "]"}]}]], "Input",
 CellTags->"SUNT",
 CellLabel->"In[1676]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["T", 
    FormBox["a",
     TraditionalForm]], ".", 
   SubscriptBox["T", 
    FormBox["b",
     TraditionalForm]], ".", 
   SubscriptBox["T", 
    FormBox["c",
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009704275846357`*^9},
 CellTags->"SUNT",
 CellLabel->"Out[1676]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNT", "[", 
  RowBox[{"a", ",", "b", ",", "c", ",", "d"}], "]"}]], "Input",
 CellTags->"SUNT",
 CellLabel->"In[1677]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["T", 
    FormBox["a",
     TraditionalForm]], 
   SubscriptBox["T", 
    FormBox["b",
     TraditionalForm]], 
   SubscriptBox["T", 
    FormBox["c",
     TraditionalForm]], 
   SubscriptBox["T", 
    FormBox["d",
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970427730249*^9},
 CellTags->"SUNT",
 CellLabel->"Out[1677]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNSimplify", "[", 
  RowBox[{
   RowBox[{"SUNT", "[", 
    RowBox[{"a", ",", "b", ",", "a"}], "]"}], ",", 
   RowBox[{"SUNNToCACF", "\[Rule]", "False"}]}], "]"}]], "Input",
 CellTags->"SUNT",
 CellLabel->"In[1678]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   FractionBox[
    SubscriptBox["T", 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm]], 
    RowBox[{"2", " ", "N"}]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009704278684893`*^9},
 CellTags->"SUNT",
 CellLabel->"Out[1678]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNSimplify", "[", 
  RowBox[{"SUNT", "[", 
   RowBox[{"a", ",", "b", ",", "b", ",", "a"}], "]"}], "]"}]], "Input",
 CellTags->"SUNT",
 CellLabel->"In[1679]:="],

Cell[BoxData[
 FormBox[
  SubsuperscriptBox["C", "F", "2"], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970428735675*^9},
 CellTags->"SUNT",
 CellLabel->"Out[1679]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNSimplify", "[", 
  RowBox[{"SUNT", "[", 
   RowBox[{"a", ",", "b", ",", "a"}], "]"}], "]"}]], "Input",
 CellTags->"SUNT",
 CellLabel->"In[1680]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    FractionBox["1", "2"]}], " ", 
   SubscriptBox["T", 
    FormBox[
     FormBox["b",
      TraditionalForm],
     TraditionalForm]], " ", 
   RowBox[{"(", 
    RowBox[{
     SubscriptBox["C", "A"], "-", 
     RowBox[{"2", " ", 
      SubscriptBox["C", "F"]}]}], ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970428845997*^9},
 CellTags->"SUNT",
 CellLabel->"Out[1680]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNSimplify", "[", 
  RowBox[{
   RowBox[{"SUNT", "[", 
    RowBox[{"a", ",", "b", ",", "a"}], "]"}], ",", 
   RowBox[{"SUNNToCACF", "\[Rule]", "False"}]}], "]"}]], "Input",
 CellTags->"SUNT",
 CellLabel->"In[1681]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   FractionBox[
    SubscriptBox["T", 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm]], 
    RowBox[{"2", " ", "N"}]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970428930468*^9},
 CellTags->"SUNT",
 CellLabel->"Out[1681]="]
}, Open  ]],

Cell[TextData[{
 "The normalizaton of the generators is chosen in the standard way, therefore \
",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"tr", "(", 
     RowBox[{
      SubscriptBox["T", "a"], 
      SubscriptBox["T", "b"]}], ")"}], " ", "=", " ", 
    RowBox[{
     RowBox[{"1", "/", "2"}], " ", 
     RowBox[{
      SubscriptBox["\[Delta]", "ab"], "."}]}]}], TraditionalForm]]]
}], "Text",
 CellTags->"SUNT"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNTrace", "[", 
  RowBox[{"SUNT", "[", 
   RowBox[{"a", ",", "b"}], "]"}], "]"}]], "Input",
 CellTags->"SUNT",
 CellLabel->"In[1682]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   SubscriptBox["\[Delta]", 
    RowBox[{
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm]}]], "2"], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970429051312*^9},
 CellTags->"SUNT",
 CellLabel->"Out[1682]="]
}, Open  ]],

Cell[TextData[{
 "In case you want ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["T", "f"], TraditionalForm]]],
 ", you need to include a factor 2Tf inside the trace."
}], "Text",
 CellTags->"SUNT"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNTrace", "[", 
  RowBox[{"2", " ", "Tf", " ", 
   RowBox[{"SUNT", "[", 
    RowBox[{"a", ",", "b"}], "]"}]}], "]"}]], "Input",
 CellTags->"SUNT",
 CellLabel->"In[1683]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["T", "f"], " ", 
   SubscriptBox["\[Delta]", 
    RowBox[{
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm]}]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970429337778*^9},
 CellTags->"SUNT",
 CellLabel->"Out[1683]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"SUNTrace", "[", 
   RowBox[{"SUNT", "[", 
    RowBox[{"a", ",", "b"}], "]"}], "]"}], "//", "StandardForm"}]], "Input",
 CellTags->"SUNT",
 CellLabel->"In[1684]:="],

Cell[BoxData[
 RowBox[{
  FractionBox["1", "2"], " ", 
  RowBox[{"SUNDelta", "[", 
   RowBox[{
    RowBox[{"SUNIndex", "[", "a", "]"}], ",", 
    RowBox[{"SUNIndex", "[", "b", "]"}]}], "]"}]}]], "Output",
 CellChangeTimes->{3.50097042940655*^9},
 CellTags->"SUNT",
 CellLabel->"Out[1684]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"SUNT", "[", "a", "]"}], "//", "FCI"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"SUNT",
 CellLabel->"In[1685]:="],

Cell[BoxData[
 RowBox[{"SUNT", "[", 
  RowBox[{"SUNIndex", "[", "a", "]"}], "]"}]], "Output",
 CellChangeTimes->{3.500970429475638*^9},
 CellTags->"SUNT",
 CellLabel->"Out[1685]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"SUNT", "[", "a", "]"}], "//", "FCI"}], "//", "FCE"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"SUNT",
 CellLabel->"In[1686]:="],

Cell[BoxData[
 RowBox[{"SUNT", "[", "a", "]"}]], "Output",
 CellChangeTimes->{3.500970429582477*^9},
 CellTags->"SUNT",
 CellLabel->"Out[1686]//StandardForm="]
}, Open  ]]
}, Open  ]],

Cell["SUNTrace", "ObjectName",
 CellTags->"SUNTrace"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SUNTrace"],

Cell["SUNTrace[expr] calculates the color-trace.", "Text",
 CellTags->"SUNTrace"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "SUNTrace", "]"}]], "Input",
 CellTags->"SUNTrace",
 CellLabel->"In[1687]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"Explicit", "\[Rule]", "False"}], "}"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970429648118*^9},
 CellTags->"SUNTrace",
 CellLabel->"Out[1687]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["SUNSimplify",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SUNSimplify",
  ButtonNote->"SUNSimplify"],
 ", ",
 ButtonBox["Tr",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Tr",
  ButtonNote->"Tr"],
 "."
}], "Text",
 CellTags->"SUNTrace"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"SUNTrace"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNTrace", "[", 
  RowBox[{"SUNT", "[", 
   RowBox[{"a", ",", "b"}], "]"}], "]"}]], "Input",
 CellTags->"SUNTrace",
 CellLabel->"In[1688]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   SubscriptBox["\[Delta]", 
    RowBox[{
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm]}]], "2"], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970429688842*^9},
 CellTags->"SUNTrace",
 CellLabel->"Out[1688]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNTrace", "[", 
  RowBox[{"SUNT", "[", 
   RowBox[{"a", ",", "b", ",", "c"}], "]"}], "]"}]], "Input",
 CellTags->"SUNTrace",
 CellLabel->"In[1689]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"tr", "(", 
   FormBox[
    RowBox[{
     SubscriptBox["T", 
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm]], ".", 
     SubscriptBox["T", 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]], ".", 
     SubscriptBox["T", 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm]]}],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970429745332*^9},
 CellTags->"SUNTrace",
 CellLabel->"Out[1689]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNTrace", "[", 
  RowBox[{
   RowBox[{"SUNT", "[", 
    RowBox[{"a", ",", "b", ",", "c"}], "]"}], ",", 
   RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"SUNTrace",
 CellLabel->"In[1690]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    SubscriptBox["d", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm]}]], "4"], "+", 
   RowBox[{
    FractionBox["1", "4"], " ", "\[ImaginaryI]", " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm]}]]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970429882966*^9},
 CellTags->"SUNTrace",
 CellLabel->"Out[1690]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNTrace", "[", 
  RowBox[{"SUNT", "[", 
   RowBox[{"a", ",", "b", ",", "c", ",", "d"}], "]"}], "]"}]], "Input",
 CellTags->"SUNTrace",
 CellLabel->"In[1691]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"tr", "(", 
   FormBox[
    RowBox[{
     SubscriptBox["T", 
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm]], ".", 
     SubscriptBox["T", 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]], ".", 
     SubscriptBox["T", 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm]], ".", 
     SubscriptBox["T", 
      FormBox[
       FormBox["d",
        TraditionalForm],
       TraditionalForm]]}],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970430039404*^9},
 CellTags->"SUNTrace",
 CellLabel->"Out[1691]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t1", "=", 
  RowBox[{"SUNTrace", "[", 
   RowBox[{
    RowBox[{"SUNT", "[", 
     RowBox[{"a", ",", "b", ",", "c", ",", "d"}], "]"}], ",", 
    RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]}]], "Input",
 CellTags->"SUNTrace",
 CellLabel->"In[1692]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    RowBox[{
     FractionBox["1", "8"], " ", "\[ImaginaryI]", " ", 
     SubscriptBox["f", 
      RowBox[{
       FormBox[
        FormBox["a",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["d",
         TraditionalForm],
        TraditionalForm], "e"}]], " ", 
     SubscriptBox["d", 
      RowBox[{"e", 
       FormBox[
        FormBox["b",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["c",
         TraditionalForm],
        TraditionalForm]}]]}]}], "+", 
   RowBox[{
    FractionBox["1", "8"], " ", "\[ImaginaryI]", " ", 
    SubscriptBox["d", 
     RowBox[{"e", 
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["d",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm], "e"}]]}], "+", 
   RowBox[{
    FractionBox["1", "8"], " ", 
    SubscriptBox["d", 
     RowBox[{"e", 
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["d",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["d", 
     RowBox[{"e", 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm]}]]}], "-", 
   RowBox[{
    FractionBox["1", "8"], " ", 
    SubscriptBox["d", 
     RowBox[{"e", 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["d",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["d", 
     RowBox[{"e", 
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    FractionBox["1", "8"], " ", 
    SubscriptBox["d", 
     RowBox[{"e", 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["d",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["d", 
     RowBox[{"e", 
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   FractionBox[
    RowBox[{
     SubscriptBox["\[Delta]", 
      RowBox[{
       FormBox[
        FormBox["a",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["d",
         TraditionalForm],
        TraditionalForm]}]], " ", 
     SubscriptBox["\[Delta]", 
      RowBox[{
       FormBox[
        FormBox["b",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["c",
         TraditionalForm],
        TraditionalForm]}]]}], 
    RowBox[{"4", " ", "N"}]], "-", 
   FractionBox[
    RowBox[{
     SubscriptBox["\[Delta]", 
      RowBox[{
       FormBox[
        FormBox["a",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["c",
         TraditionalForm],
        TraditionalForm]}]], " ", 
     SubscriptBox["\[Delta]", 
      RowBox[{
       FormBox[
        FormBox["b",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["d",
         TraditionalForm],
        TraditionalForm]}]]}], 
    RowBox[{"4", " ", "N"}]], "+", 
   FractionBox[
    RowBox[{
     SubscriptBox["\[Delta]", 
      RowBox[{
       FormBox[
        FormBox["a",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["b",
         TraditionalForm],
        TraditionalForm]}]], " ", 
     SubscriptBox["\[Delta]", 
      RowBox[{
       FormBox[
        FormBox["c",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["d",
         TraditionalForm],
        TraditionalForm]}]]}], 
    RowBox[{"4", " ", "N"}]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970430342765*^9},
 CellTags->"SUNTrace",
 CellLabel->"Out[1692]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNSimplify", "[", 
  RowBox[{"t1", ",", 
   RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"SUNTrace",
 CellLabel->"In[1693]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"tr", "(", 
   FormBox[
    RowBox[{
     SubscriptBox["T", 
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm]], ".", 
     SubscriptBox["T", 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]], ".", 
     SubscriptBox["T", 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm]], ".", 
     SubscriptBox["T", 
      FormBox[
       FormBox["d",
        TraditionalForm],
       TraditionalForm]]}],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970430992708*^9},
 CellTags->"SUNTrace",
 CellLabel->"Out[1693]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t2", "=", 
  RowBox[{"SUNTrace", "[", 
   RowBox[{
    RowBox[{"SUNT", "[", 
     RowBox[{"a", ",", "b", ",", "c", ",", "d", ",", "e"}], "]"}], ",", 
    RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]}]], "Input",
 CellTags->"SUNTrace",
 CellLabel->"In[1694]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    FractionBox["1", "2"], " ", 
    SubscriptBox["d", 
     RowBox[{"c187", 
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       FractionBox["1", "8"], " ", "\[ImaginaryI]", " ", 
       SubscriptBox["f", 
        RowBox[{
         FormBox[
          FormBox["c",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["d",
           TraditionalForm],
          TraditionalForm], "c188"}]], " ", 
       SubscriptBox["d", 
        RowBox[{"c188", 
         FormBox[
          FormBox["c187",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["e",
           TraditionalForm],
          TraditionalForm]}]]}], "-", 
      RowBox[{
       FractionBox["1", "8"], " ", "\[ImaginaryI]", " ", 
       SubscriptBox["d", 
        RowBox[{"c188", 
         FormBox[
          FormBox["c",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["d",
           TraditionalForm],
          TraditionalForm]}]], " ", 
       SubscriptBox["f", 
        RowBox[{
         FormBox[
          FormBox["c187",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["e",
           TraditionalForm],
          TraditionalForm], "c188"}]]}], "-", 
      RowBox[{
       FractionBox["1", "8"], " ", 
       SubscriptBox["d", 
        RowBox[{"c188", 
         FormBox[
          FormBox["c187",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["d",
           TraditionalForm],
          TraditionalForm]}]], " ", 
       SubscriptBox["d", 
        RowBox[{"c188", 
         FormBox[
          FormBox["c",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["e",
           TraditionalForm],
          TraditionalForm]}]]}], "+", 
      RowBox[{
       FractionBox["1", "8"], " ", 
       SubscriptBox["d", 
        RowBox[{"c188", 
         FormBox[
          FormBox["c",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["d",
           TraditionalForm],
          TraditionalForm]}]], " ", 
       SubscriptBox["d", 
        RowBox[{"c188", 
         FormBox[
          FormBox["c187",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["e",
           TraditionalForm],
          TraditionalForm]}]]}], "+", 
      RowBox[{
       FractionBox["1", "8"], " ", 
       SubscriptBox["d", 
        RowBox[{"c188", 
         FormBox[
          FormBox["d",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["e",
           TraditionalForm],
          TraditionalForm]}]], " ", 
       SubscriptBox["d", 
        RowBox[{"c188", 
         FormBox[
          FormBox["c",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["c187",
           TraditionalForm],
          TraditionalForm]}]]}], "-", 
      FractionBox[
       RowBox[{
        SubscriptBox["\[Delta]", 
         RowBox[{
          FormBox[
           FormBox["c",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["e",
            TraditionalForm],
           TraditionalForm]}]], " ", 
        SubscriptBox["\[Delta]", 
         RowBox[{
          FormBox[
           FormBox["c187",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["d",
            TraditionalForm],
           TraditionalForm]}]]}], 
       RowBox[{"4", " ", "N"}]], "+", 
      FractionBox[
       RowBox[{
        SubscriptBox["\[Delta]", 
         RowBox[{
          FormBox[
           FormBox["c",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["d",
            TraditionalForm],
           TraditionalForm]}]], " ", 
        SubscriptBox["\[Delta]", 
         RowBox[{
          FormBox[
           FormBox["c187",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["e",
            TraditionalForm],
           TraditionalForm]}]]}], 
       RowBox[{"4", " ", "N"}]], "+", 
      FractionBox[
       RowBox[{
        SubscriptBox["\[Delta]", 
         RowBox[{
          FormBox[
           FormBox["c",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["c187",
            TraditionalForm],
           TraditionalForm]}]], " ", 
        SubscriptBox["\[Delta]", 
         RowBox[{
          FormBox[
           FormBox["d",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["e",
            TraditionalForm],
           TraditionalForm]}]]}], 
       RowBox[{"4", " ", "N"}]]}], ")"}]}], "+", 
   RowBox[{
    FractionBox["1", "2"], " ", "\[ImaginaryI]", " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], "c187"}]], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       FractionBox["1", "8"], " ", "\[ImaginaryI]", " ", 
       SubscriptBox["f", 
        RowBox[{
         FormBox[
          FormBox["c",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["d",
           TraditionalForm],
          TraditionalForm], "c189"}]], " ", 
       SubscriptBox["d", 
        RowBox[{"c189", 
         FormBox[
          FormBox["c187",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["e",
           TraditionalForm],
          TraditionalForm]}]]}], "-", 
      RowBox[{
       FractionBox["1", "8"], " ", "\[ImaginaryI]", " ", 
       SubscriptBox["d", 
        RowBox[{"c189", 
         FormBox[
          FormBox["c",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["d",
           TraditionalForm],
          TraditionalForm]}]], " ", 
       SubscriptBox["f", 
        RowBox[{
         FormBox[
          FormBox["c187",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["e",
           TraditionalForm],
          TraditionalForm], "c189"}]]}], "-", 
      RowBox[{
       FractionBox["1", "8"], " ", 
       SubscriptBox["d", 
        RowBox[{"c189", 
         FormBox[
          FormBox["c187",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["d",
           TraditionalForm],
          TraditionalForm]}]], " ", 
       SubscriptBox["d", 
        RowBox[{"c189", 
         FormBox[
          FormBox["c",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["e",
           TraditionalForm],
          TraditionalForm]}]]}], "+", 
      RowBox[{
       FractionBox["1", "8"], " ", 
       SubscriptBox["d", 
        RowBox[{"c189", 
         FormBox[
          FormBox["c",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["d",
           TraditionalForm],
          TraditionalForm]}]], " ", 
       SubscriptBox["d", 
        RowBox[{"c189", 
         FormBox[
          FormBox["c187",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["e",
           TraditionalForm],
          TraditionalForm]}]]}], "+", 
      RowBox[{
       FractionBox["1", "8"], " ", 
       SubscriptBox["d", 
        RowBox[{"c189", 
         FormBox[
          FormBox["d",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["e",
           TraditionalForm],
          TraditionalForm]}]], " ", 
       SubscriptBox["d", 
        RowBox[{"c189", 
         FormBox[
          FormBox["c",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["c187",
           TraditionalForm],
          TraditionalForm]}]]}], "-", 
      FractionBox[
       RowBox[{
        SubscriptBox["\[Delta]", 
         RowBox[{
          FormBox[
           FormBox["c",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["e",
            TraditionalForm],
           TraditionalForm]}]], " ", 
        SubscriptBox["\[Delta]", 
         RowBox[{
          FormBox[
           FormBox["c187",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["d",
            TraditionalForm],
           TraditionalForm]}]]}], 
       RowBox[{"4", " ", "N"}]], "+", 
      FractionBox[
       RowBox[{
        SubscriptBox["\[Delta]", 
         RowBox[{
          FormBox[
           FormBox["c",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["d",
            TraditionalForm],
           TraditionalForm]}]], " ", 
        SubscriptBox["\[Delta]", 
         RowBox[{
          FormBox[
           FormBox["c187",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["e",
            TraditionalForm],
           TraditionalForm]}]]}], 
       RowBox[{"4", " ", "N"}]], "+", 
      FractionBox[
       RowBox[{
        SubscriptBox["\[Delta]", 
         RowBox[{
          FormBox[
           FormBox["c",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["c187",
            TraditionalForm],
           TraditionalForm]}]], " ", 
        SubscriptBox["\[Delta]", 
         RowBox[{
          FormBox[
           FormBox["d",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["e",
            TraditionalForm],
           TraditionalForm]}]]}], 
       RowBox[{"4", " ", "N"}]]}], ")"}]}], "+", 
   FractionBox[
    RowBox[{
     SubscriptBox["\[Delta]", 
      RowBox[{
       FormBox[
        FormBox["a",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["b",
         TraditionalForm],
        TraditionalForm]}]], " ", 
     RowBox[{"(", 
      RowBox[{
       FractionBox[
        SubscriptBox["d", 
         RowBox[{
          FormBox[
           FormBox["c",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["d",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["e",
            TraditionalForm],
           TraditionalForm]}]], "4"], "+", 
       RowBox[{
        FractionBox["1", "4"], " ", "\[ImaginaryI]", " ", 
        SubscriptBox["f", 
         RowBox[{
          FormBox[
           FormBox["c",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["d",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["e",
            TraditionalForm],
           TraditionalForm]}]]}]}], ")"}]}], 
    RowBox[{"2", " ", "N"}]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009704311735983`*^9},
 CellTags->"SUNTrace",
 CellLabel->"Out[1694]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNSimplify", "[", 
  RowBox[{"t2", ",", 
   RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"SUNTrace",
 CellLabel->"In[1695]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"tr", "(", 
   FormBox[
    RowBox[{
     SubscriptBox["T", 
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm]], ".", 
     SubscriptBox["T", 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]], ".", 
     SubscriptBox["T", 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm]], ".", 
     SubscriptBox["T", 
      FormBox[
       FormBox["d",
        TraditionalForm],
       TraditionalForm]], ".", 
     SubscriptBox["T", 
      FormBox[
       FormBox["e",
        TraditionalForm],
       TraditionalForm]]}],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009704324164963`*^9},
 CellTags->"SUNTrace",
 CellLabel->"Out[1695]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNSimplify", "[", 
  RowBox[{
   RowBox[{"SUNF", "[", 
    RowBox[{"a", ",", "b", ",", "c"}], "]"}], " ", 
   RowBox[{"SUND", "[", 
    RowBox[{"d", ",", "b", ",", "c"}], "]"}]}], "]"}]], "Input",
 CellTags->"SUNTrace",
 CellLabel->"In[1696]:="],

Cell[BoxData[
 FormBox["0", TraditionalForm]], "Output",
 CellChangeTimes->{3.500970432534952*^9},
 CellTags->"SUNTrace",
 CellLabel->"Out[1696]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNSimplify", "[", 
  RowBox[{
   RowBox[{"SUNF", "[", 
    RowBox[{"a", ",", "b", ",", "c"}], "]"}], " ", 
   RowBox[{"SUND", "[", 
    RowBox[{"a", ",", "b", ",", "d"}], "]"}]}], "]"}]], "Input",
 CellTags->"SUNTrace",
 CellLabel->"In[1697]:="],

Cell[BoxData[
 FormBox["0", TraditionalForm]], "Output",
 CellChangeTimes->{3.500970432687027*^9},
 CellTags->"SUNTrace",
 CellLabel->"Out[1697]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNSimplify", "[", 
  RowBox[{
   RowBox[{"SUNF", "[", 
    RowBox[{"a", ",", "b", ",", "c"}], "]"}], " ", 
   RowBox[{"SUND", "[", 
    RowBox[{"a", ",", "d", ",", "c"}], "]"}]}], "]"}]], "Input",
 CellTags->"SUNTrace",
 CellLabel->"In[1698]:="],

Cell[BoxData[
 FormBox["0", TraditionalForm]], "Output",
 CellChangeTimes->{3.500970432772314*^9},
 CellTags->"SUNTrace",
 CellLabel->"Out[1698]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNSimplify", "[", 
  RowBox[{
   RowBox[{"SUND", "[", 
    RowBox[{"a", ",", "b", ",", "c"}], "]"}], " ", 
   RowBox[{"SUND", "[", 
    RowBox[{"d", ",", "b", ",", "c"}], "]"}]}], "]"}]], "Input",
 CellTags->"SUNTrace",
 CellLabel->"In[1699]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    RowBox[{"(", 
     RowBox[{"4", "-", 
      SubsuperscriptBox["C", "A", "2"]}], ")"}]}], " ", 
   SubscriptBox["\[Delta]", 
    RowBox[{
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["d",
       TraditionalForm],
      TraditionalForm]}]], " ", 
   RowBox[{"(", 
    RowBox[{
     SubscriptBox["C", "A"], "-", 
     RowBox[{"2", " ", 
      SubscriptBox["C", "F"]}]}], ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970432937364*^9},
 CellTags->"SUNTrace",
 CellLabel->"Out[1699]="]
}, Open  ]]
}, Open  ]],

Cell["SymbolicSum2", "ObjectName",
 CellTags->"SymbolicSum2"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SymbolicSum2"],

Cell["\<\
SymbolicSum2 is similar to SymbolicSum (Algegra`SymbolicSum`SymbolicSum was a \
function to do symbolic summation. It was obsolete from version 3 - all \
functionality is now autoloaded by Sum), but extended to several double sums.\
\
\>", "Text",
 CellTags->"SymbolicSum2"]
}, Open  ]],

Cell["SymbolicSum3", "ObjectName",
 CellTags->"SymbolicSum3"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SymbolicSum3"],

Cell["\<\
SymbolicSymbolicSum3 is similar to SymbolicSum \
(Algegra`SymbolicSum`SymbolicSum was a function to do symbolic summation. It \
was obsolete from version 3 - all functionality is now autoloaded by Sum), \
but extended to several double sums.\
\>", "Text",
 CellTags->"SymbolicSum3"]
}, Open  ]],

Cell["Symmetrize", "ObjectName",
 CellTags->"Symmetrize"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Symmetrize"],

Cell["\<\
Symmetrize[expr, {a1, a2, ...}] antisymmetrizes expr with respect to the \
variables a1,a2, ... \
\>", "Text",
 CellTags->"Symmetrize"],

Cell[TextData[{
 "See also: ",
 ButtonBox["AntiSymmetrize",
  BaseStyle->"AddOnsLink",
  ButtonData:>"AntiSymmetrize",
  ButtonNote->"AntiSymmetrize"],
 "."
}], "Text",
 CellTags->"Symmetrize"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Symmetrize"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Symmetrize", "[", 
  RowBox[{
   RowBox[{"f", "[", 
    RowBox[{"a", ",", "b"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"a", ",", "b"}], "}"}]}], "]"}]], "Input",
 CellTags->"Symmetrize",
 CellLabel->"In[1700]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["1", "2"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"f", "(", 
      RowBox[{"a", ",", "b"}], ")"}], "+", 
     RowBox[{"f", "(", 
      RowBox[{"b", ",", "a"}], ")"}]}], ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970433146077*^9},
 CellTags->"Symmetrize",
 CellLabel->"Out[1700]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Symmetrize", "[", 
  RowBox[{
   RowBox[{"f", "[", 
    RowBox[{"x", ",", "y", ",", "z"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "y", ",", "z"}], "}"}]}], "]"}]], "Input",
 CellTags->"Symmetrize",
 CellLabel->"In[1701]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["1", "6"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"f", "(", 
      RowBox[{"x", ",", "y", ",", "z"}], ")"}], "+", 
     RowBox[{"f", "(", 
      RowBox[{"x", ",", "z", ",", "y"}], ")"}], "+", 
     RowBox[{"f", "(", 
      RowBox[{"y", ",", "x", ",", "z"}], ")"}], "+", 
     RowBox[{"f", "(", 
      RowBox[{"y", ",", "z", ",", "x"}], ")"}], "+", 
     RowBox[{"f", "(", 
      RowBox[{"z", ",", "x", ",", "y"}], ")"}], "+", 
     RowBox[{"f", "(", 
      RowBox[{"z", ",", "y", ",", "x"}], ")"}]}], ")"}]}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970433369252*^9},
 CellTags->"Symmetrize",
 CellLabel->"Out[1701]="]
}, Open  ]]
}, Open  ]],

Cell["TBox, Tbox", "ObjectName",
 CellTags->{"TBox", "Tbox"}],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->{"TBox", "Tbox"}],

Cell["\<\
TBox[a, b, ...] or Tbox[a, b, ...] produces a RowBox[{a,b, ...}] where a, b, \
... are boxed in TraditionalForm. TBox and Tbox are used internally by \
FeynCalc to produce the typeset output in TraditionalForm\
\>", "Text",
 CellTags->{"TBox", "Tbox"}]
}, Open  ]],

Cell["Tdec", "ObjectName",
 CellTags->"Tdec"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Tdec"],

Cell["\<\
Tdec[{q,mu}, {p}]; Tdec[{{qi, mu}, {qj, nu}, ...}, {p1, p2, ...}] or \
Tdec[exp, {{qi, mu}, {qj, nu}, ...}, {p1, p2, ...}] calculates the tensorial \
decomposition formulas. The more common ones are saved in TIDL.\
\>", "Text",
 CellTags->"Tdec"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "Tdec", "]"}]], "Input",
 CellTags->"Tdec",
 CellLabel->"In[1702]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
    RowBox[{"Factoring", "\[Rule]", "Factor2"}], ",", 
    RowBox[{"FeynCalcExternal", "\[Rule]", "True"}], ",", 
    RowBox[{"List", "\[Rule]", "True"}], ",", 
    RowBox[{"NumberOfMetricTensors", "\[Rule]", "\[Infinity]"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970433448308*^9},
 CellTags->"Tdec",
 CellLabel->"Out[1702]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["TID",
  BaseStyle->"AddOnsLink",
  ButtonData:>"TID",
  ButtonNote->"TID"],
 ", ",
 ButtonBox["TIDL",
  BaseStyle->"AddOnsLink",
  ButtonData:>"TIDL",
  ButtonNote->"TIDL"],
 ", ",
 ButtonBox["OneLoopSimplify",
  BaseStyle->"AddOnsLink",
  ButtonData:>"OneLoopSimplify",
  ButtonNote->"OneLoopSimplify"],
 "."
}], "Text",
 CellTags->"Tdec"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Tdec"],

Cell[TextData[{
 "Check that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"\[Integral]", 
     RowBox[{
      SuperscriptBox["d", "D"], 
      RowBox[{"f", "(", 
       RowBox[{"p", ",", "q"}], ")"}], 
      SuperscriptBox["q", "\[Mu]"]}]}], "=", " ", 
    RowBox[{
     FractionBox[
      SuperscriptBox["p", "\[Mu]"], 
      SuperscriptBox["p", "2"]], 
     RowBox[{"\[Integral]", 
      RowBox[{
       SuperscriptBox["d", "D"], 
       RowBox[{"f", "(", 
        RowBox[{"p", ",", "q"}], ")"}], 
       RowBox[{"p", "\[CenterDot]", "q"}]}]}]}]}], TraditionalForm]]]
}], "Text",
 CellTags->"Tdec"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Tdec", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"q", ",", "\[Mu]"}], "}"}], ",", 
   RowBox[{"{", "p", "}"}]}], "]"}]], "Input",
 CellTags->"Tdec",
 CellLabel->"In[1703]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"X1", "\[Rule]", 
       SuperscriptBox["m", "2"]}], ",", 
      RowBox[{"X2", "\[Rule]", 
       FormBox[
        RowBox[{
         FormBox["p",
          TraditionalForm], "\[CenterDot]", 
         FormBox["q",
          TraditionalForm]}],
        TraditionalForm]}]}], "}"}], ",", 
    FractionBox[
     RowBox[{"X2", " ", 
      SuperscriptBox["p", "\[Mu]"]}], "X1"]}], "}"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.50097043398232*^9},
 CellTags->"Tdec",
 CellLabel->"Out[1703]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"%", "[", 
   RowBox[{"[", "2", "]"}], "]"}], "/.", 
  RowBox[{"%", "[", 
   RowBox[{"[", "1", "]"}], "]"}]}]], "Input",
 CellTags->"Tdec",
 CellLabel->"In[1704]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{
    SuperscriptBox["p", "\[Mu]"], " ", 
    FormBox[
     RowBox[{
      FormBox["p",
       TraditionalForm], "\[CenterDot]", 
      FormBox["q",
       TraditionalForm]}],
     TraditionalForm]}], 
   SuperscriptBox["m", "2"]], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009704340866833`*^9},
 CellTags->"Tdec",
 CellLabel->"Out[1704]="]
}, Open  ]],

Cell[TextData[{
 "This calculates integral transformation for any ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Integral]", 
    RowBox[{
     SuperscriptBox["d", "D"], 
     SubscriptBox["q", "1"], 
     SuperscriptBox["d", "D"], 
     SubscriptBox["q", "2"], 
     SuperscriptBox["d", "D"], 
     SubscriptBox["q", "3"]}]}], TraditionalForm]]],
 " ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"f", "(", 
     RowBox[{"p", ",", 
      RowBox[{
       SubscriptBox["q", 
        RowBox[{"1", ","}]], 
       SubscriptBox["q", "2"]}], ",", 
      SubscriptBox["q", "3"]}], ")"}], " ", 
    SubsuperscriptBox["q", "1", "\[Mu]"], 
    SubsuperscriptBox["q", "2", "\[Nu]"], 
    SubsuperscriptBox["q", "3", "\[Rho]"]}], TraditionalForm]]],
 "."
}], "Text",
 CellTags->"Tdec"],

Cell[BoxData[
 RowBox[{
  RowBox[{"t", "=", 
   RowBox[{"Tdec", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{
         SubscriptBox["q", "1"], ",", "\[Mu]"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{
         SubscriptBox["q", "2"], ",", "\[Nu]"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{
         SubscriptBox["q", "3"], ",", "\[Rho]"}], "}"}]}], "}"}], ",", 
     RowBox[{"{", "p", "}"}]}], "]"}]}], ";"}]], "Input",
 CellTags->"Tdec",
 CellLabel->"In[1705]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"t", "[", 
   RowBox[{"[", "2", "]"}], "]"}], "/.", 
  RowBox[{"t", "[", 
   RowBox[{"[", "1", "]"}], "]"}]}]], "Input",
 CellTags->"Tdec",
 CellLabel->"In[1706]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{
     SuperscriptBox["p", "\[Rho]"], " ", 
     SuperscriptBox["g", 
      RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
     FormBox[
      RowBox[{
       FormBox["p",
        TraditionalForm], "\[CenterDot]", 
       FormBox[
        SubscriptBox["q", "3"],
        TraditionalForm]}],
      TraditionalForm], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        FormBox[
         RowBox[{
          FormBox["p",
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["q", "1"],
           TraditionalForm]}],
         TraditionalForm], " ", 
        FormBox[
         RowBox[{
          FormBox["p",
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["q", "2"],
           TraditionalForm]}],
         TraditionalForm]}], "-", 
       RowBox[{
        SuperscriptBox["m", "2"], " ", 
        FormBox[
         RowBox[{
          FormBox[
           SubscriptBox["q", "1"],
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["q", "2"],
           TraditionalForm]}],
         TraditionalForm]}]}], ")"}]}], 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"1", "-", "D"}], ")"}], " ", 
     SuperscriptBox["m", "4"]}]], "+", 
   FractionBox[
    RowBox[{
     SuperscriptBox["p", "\[Nu]"], " ", 
     SuperscriptBox["g", 
      RowBox[{"\[Mu]", "\[Rho]"}]], " ", 
     FormBox[
      RowBox[{
       FormBox["p",
        TraditionalForm], "\[CenterDot]", 
       FormBox[
        SubscriptBox["q", "2"],
        TraditionalForm]}],
      TraditionalForm], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        FormBox[
         RowBox[{
          FormBox["p",
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["q", "1"],
           TraditionalForm]}],
         TraditionalForm], " ", 
        FormBox[
         RowBox[{
          FormBox["p",
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["q", "3"],
           TraditionalForm]}],
         TraditionalForm]}], "-", 
       RowBox[{
        SuperscriptBox["m", "2"], " ", 
        FormBox[
         RowBox[{
          FormBox[
           SubscriptBox["q", "1"],
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["q", "3"],
           TraditionalForm]}],
         TraditionalForm]}]}], ")"}]}], 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"1", "-", "D"}], ")"}], " ", 
     SuperscriptBox["m", "4"]}]], "+", 
   FractionBox[
    RowBox[{
     SuperscriptBox["p", "\[Mu]"], " ", 
     SuperscriptBox["g", 
      RowBox[{"\[Nu]", "\[Rho]"}]], " ", 
     FormBox[
      RowBox[{
       FormBox["p",
        TraditionalForm], "\[CenterDot]", 
       FormBox[
        SubscriptBox["q", "1"],
        TraditionalForm]}],
      TraditionalForm], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        FormBox[
         RowBox[{
          FormBox["p",
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["q", "2"],
           TraditionalForm]}],
         TraditionalForm], " ", 
        FormBox[
         RowBox[{
          FormBox["p",
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["q", "3"],
           TraditionalForm]}],
         TraditionalForm]}], "-", 
       RowBox[{
        SuperscriptBox["m", "2"], " ", 
        FormBox[
         RowBox[{
          FormBox[
           SubscriptBox["q", "2"],
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["q", "3"],
           TraditionalForm]}],
         TraditionalForm]}]}], ")"}]}], 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"1", "-", "D"}], ")"}], " ", 
     SuperscriptBox["m", "4"]}]], "-", 
   RowBox[{
    FractionBox["1", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"1", "-", "D"}], ")"}], " ", 
      SuperscriptBox["m", "6"]}]], 
    RowBox[{
     SuperscriptBox["p", "\[Mu]"], " ", 
     SuperscriptBox["p", "\[Nu]"], " ", 
     SuperscriptBox["p", "\[Rho]"], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"D", " ", 
        FormBox[
         RowBox[{
          FormBox["p",
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["q", "1"],
           TraditionalForm]}],
         TraditionalForm], " ", 
        FormBox[
         RowBox[{
          FormBox["p",
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["q", "2"],
           TraditionalForm]}],
         TraditionalForm], " ", 
        FormBox[
         RowBox[{
          FormBox["p",
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["q", "3"],
           TraditionalForm]}],
         TraditionalForm]}], "+", 
       RowBox[{
        SuperscriptBox["m", "2"], " ", 
        FormBox[
         RowBox[{
          FormBox[
           SubscriptBox["q", "1"],
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["q", "2"],
           TraditionalForm]}],
         TraditionalForm], " ", 
        RowBox[{"(", 
         RowBox[{"-", 
          RowBox[{"(", 
           FormBox[
            RowBox[{
             FormBox["p",
              TraditionalForm], "\[CenterDot]", 
             FormBox[
              SubscriptBox["q", "3"],
              TraditionalForm]}],
            TraditionalForm], ")"}]}], ")"}]}], "-", 
       RowBox[{
        SuperscriptBox["m", "2"], " ", 
        FormBox[
         RowBox[{
          FormBox[
           SubscriptBox["q", "1"],
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["q", "3"],
           TraditionalForm]}],
         TraditionalForm], " ", 
        FormBox[
         RowBox[{
          FormBox["p",
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["q", "2"],
           TraditionalForm]}],
         TraditionalForm]}], "-", 
       RowBox[{
        SuperscriptBox["m", "2"], " ", 
        FormBox[
         RowBox[{
          FormBox[
           SubscriptBox["q", "2"],
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["q", "3"],
           TraditionalForm]}],
         TraditionalForm], " ", 
        FormBox[
         RowBox[{
          FormBox["p",
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["q", "1"],
           TraditionalForm]}],
         TraditionalForm]}], "+", 
       RowBox[{"2", " ", 
        FormBox[
         RowBox[{
          FormBox["p",
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["q", "1"],
           TraditionalForm]}],
         TraditionalForm], " ", 
        FormBox[
         RowBox[{
          FormBox["p",
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["q", "2"],
           TraditionalForm]}],
         TraditionalForm], " ", 
        FormBox[
         RowBox[{
          FormBox["p",
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["q", "3"],
           TraditionalForm]}],
         TraditionalForm]}]}], ")"}]}]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970434739173*^9},
 CellTags->"Tdec",
 CellLabel->"Out[1706]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Contract", "[", 
   RowBox[{
    RowBox[{"%", " ", 
     RowBox[{"FVD", "[", 
      RowBox[{"p", ",", "\[Mu]"}], "]"}]}], 
    RowBox[{"FVD", "[", 
     RowBox[{"p", ",", "\[Nu]"}], "]"}], " ", 
    RowBox[{"FVD", "[", 
     RowBox[{"p", ",", "\[Rho]"}], "]"}]}], "]"}], "//", "Factor"}]], "Input",\

 CellTags->"Tdec",
 CellLabel->"In[1707]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["1", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"D", "-", "1"}], ")"}], " ", 
     SuperscriptBox["m", "6"]}]], 
   RowBox[{
    SuperscriptBox[
     FormBox[
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], "2"],
      TraditionalForm], "2"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"D", " ", 
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], "2"], " ", 
       RowBox[{
        FormBox["p",
         TraditionalForm], "\[CenterDot]", 
        FormBox[
         SubscriptBox["q", "1"],
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["p",
         TraditionalForm], "\[CenterDot]", 
        FormBox[
         SubscriptBox["q", "2"],
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["p",
         TraditionalForm], "\[CenterDot]", 
        FormBox[
         SubscriptBox["q", "3"],
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox["m", "4"], " ", 
       RowBox[{
        FormBox[
         SubscriptBox["q", "1"],
         TraditionalForm], "\[CenterDot]", 
        FormBox[
         SubscriptBox["q", "2"],
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["p",
         TraditionalForm], "\[CenterDot]", 
        FormBox[
         SubscriptBox["q", "3"],
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox["m", "4"], " ", 
       RowBox[{
        FormBox[
         SubscriptBox["q", "1"],
         TraditionalForm], "\[CenterDot]", 
        FormBox[
         SubscriptBox["q", "3"],
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["p",
         TraditionalForm], "\[CenterDot]", 
        FormBox[
         SubscriptBox["q", "2"],
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox["m", "4"], " ", 
       RowBox[{
        FormBox[
         SubscriptBox["q", "2"],
         TraditionalForm], "\[CenterDot]", 
        FormBox[
         SubscriptBox["q", "3"],
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["p",
         TraditionalForm], "\[CenterDot]", 
        FormBox[
         SubscriptBox["q", "1"],
         TraditionalForm]}]}], "-", 
      RowBox[{"3", " ", 
       SuperscriptBox["m", "2"], " ", 
       RowBox[{
        FormBox["p",
         TraditionalForm], "\[CenterDot]", 
        FormBox[
         SubscriptBox["q", "1"],
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["p",
         TraditionalForm], "\[CenterDot]", 
        FormBox[
         SubscriptBox["q", "2"],
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["p",
         TraditionalForm], "\[CenterDot]", 
        FormBox[
         SubscriptBox["q", "3"],
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["m", "2"], " ", 
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], "2"], " ", 
       RowBox[{
        FormBox[
         SubscriptBox["q", "1"],
         TraditionalForm], "\[CenterDot]", 
        FormBox[
         SubscriptBox["q", "2"],
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["p",
         TraditionalForm], "\[CenterDot]", 
        FormBox[
         SubscriptBox["q", "3"],
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["m", "2"], " ", 
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], "2"], " ", 
       RowBox[{
        FormBox[
         SubscriptBox["q", "1"],
         TraditionalForm], "\[CenterDot]", 
        FormBox[
         SubscriptBox["q", "3"],
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["p",
         TraditionalForm], "\[CenterDot]", 
        FormBox[
         SubscriptBox["q", "2"],
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["m", "2"], " ", 
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], "2"], " ", 
       RowBox[{
        FormBox[
         SubscriptBox["q", "2"],
         TraditionalForm], "\[CenterDot]", 
        FormBox[
         SubscriptBox["q", "3"],
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["p",
         TraditionalForm], "\[CenterDot]", 
        FormBox[
         SubscriptBox["q", "1"],
         TraditionalForm]}]}], "+", 
      RowBox[{"2", " ", 
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], "2"], " ", 
       RowBox[{
        FormBox["p",
         TraditionalForm], "\[CenterDot]", 
        FormBox[
         SubscriptBox["q", "1"],
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["p",
         TraditionalForm], "\[CenterDot]", 
        FormBox[
         SubscriptBox["q", "2"],
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["p",
         TraditionalForm], "\[CenterDot]", 
        FormBox[
         SubscriptBox["q", "3"],
         TraditionalForm]}]}]}], ")"}]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970435170312*^9},
 CellTags->"Tdec",
 CellLabel->"Out[1707]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", "t", "]"}], ";"}]], "Input",
 CellTags->"Tdec",
 CellLabel->"In[1708]:="]
}, Open  ]],

Cell["TensorFunction", "ObjectName",
 CellTags->"TensorFunction"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"TensorFunction"],

Cell["\<\
Tensorfunction[t, mu, nu, ...] transform into t[LorentzIndex[mu], \
LorentzIndex[nu], ...], i.e., it can be used as an unspecified tensoriell \
function t. A symmetric tensor can be obtained by Tensorfunction[{t, \"S\"}, \
mu, nu, ...], and an antisymmteric one by Tensorfunction[{t, \"A\"}, mu, nu, \
...].\
\>", "Text",
 CellTags->"TensorFunction"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Symmetrize",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Symmetrize",
  ButtonNote->"Symmetrize"],
 "."
}], "Text",
 CellTags->"TensorFunction"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"TensorFunction"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Tensorfunction", "[", 
  RowBox[{"t", ",", "\[Mu]", ",", "\[Nu]", ",", "\[Tau]"}], "]"}]], "Input",
 CellTags->"TensorFunction",
 CellLabel->"In[1709]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"Tensorfunction", "(", 
   RowBox[{"t", ",", "\[Mu]", ",", "\[Nu]", ",", "\[Tau]"}], ")"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.50097043529426*^9},
 CellTags->"TensorFunction",
 CellLabel->"Out[1709]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "StandardForm"}]], "Input",
 CellTags->"TensorFunction",
 CellLabel->"In[1710]:="],

Cell[BoxData[
 RowBox[{"Tensorfunction", "[", 
  RowBox[{"t", ",", "\[Mu]", ",", "\[Nu]", ",", "\[Tau]"}], "]"}]], "Output",
 CellChangeTimes->{3.500970435467876*^9},
 CellTags->"TensorFunction",
 CellLabel->"Out[1710]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Contract", "[", 
  RowBox[{
   RowBox[{"FV", "[", 
    RowBox[{"p", ",", "\[Mu]"}], "]"}], " ", "%"}], "]"}]], "Input",
 CellTags->"TensorFunction",
 CellLabel->"In[1711]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox[
    FormBox["p",
     TraditionalForm], 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], " ", 
   RowBox[{"Tensorfunction", "(", 
    RowBox[{"t", ",", "\[Mu]", ",", "\[Nu]", ",", "\[Tau]"}], ")"}]}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.50097043565991*^9},
 CellTags->"TensorFunction",
 CellLabel->"Out[1711]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "StandardForm"}]], "Input",
 CellTags->"TensorFunction",
 CellLabel->"In[1712]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"Pair", "[", 
   RowBox[{
    RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
    RowBox[{"Momentum", "[", "p", "]"}]}], "]"}], " ", 
  RowBox[{"Tensorfunction", "[", 
   RowBox[{"t", ",", "\[Mu]", ",", "\[Nu]", ",", "\[Tau]"}], 
   "]"}]}]], "Output",
 CellChangeTimes->{3.500970435738865*^9},
 CellTags->"TensorFunction",
 CellLabel->"Out[1712]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Tensorfunction", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"f", ",", "\"\<S\>\""}], "}"}], ",", "\[Alpha]", ",", "\[Beta]"}],
   "]"}]], "Input",
 CellTags->"TensorFunction",
 CellLabel->"In[1713]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"Tensorfunction", "(", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"f", ",", "\<\"S\"\>"}], "}"}], ",", "\[Alpha]", ",", 
    "\[Beta]"}], ")"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970435986767*^9},
 CellTags->"TensorFunction",
 CellLabel->"Out[1713]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Tensorfunction", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"f", ",", "\"\<S\>\""}], "}"}], ",", "\[Beta]", ",", "\[Alpha]"}],
   "]"}]], "Input",
 CellTags->"TensorFunction",
 CellLabel->"In[1714]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"Tensorfunction", "(", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"f", ",", "\<\"S\"\>"}], "}"}], ",", "\[Beta]", ",", 
    "\[Alpha]"}], ")"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.50097043643753*^9},
 CellTags->"TensorFunction",
 CellLabel->"Out[1714]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "StandardForm"}]], "Input",
 CellTags->"TensorFunction",
 CellLabel->"In[1715]:="],

Cell[BoxData[
 RowBox[{"Tensorfunction", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"f", ",", "\<\"S\"\>"}], "}"}], ",", "\[Beta]", ",", "\[Alpha]"}],
   "]"}]], "Output",
 CellChangeTimes->{3.500970436491486*^9},
 CellTags->"TensorFunction",
 CellLabel->"Out[1715]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Attributes", "[", "f", "]"}]], "Input",
 CellTags->"TensorFunction",
 CellLabel->"In[1716]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", "}"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970436529657*^9},
 CellTags->"TensorFunction",
 CellLabel->"Out[1716]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"ClearAttributes", "[", 
  RowBox[{"f", ",", "Orderless"}], "]"}]], "Input",
 CellTags->"TensorFunction",
 CellLabel->"In[1717]:="]
}, Open  ]],

Cell["Tf", "ObjectName",
 CellTags->"Tf"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Tf"],

Cell[TextData[{
 "Tf is the color factor ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["T", "f"], TraditionalForm]]],
 ". It is 1/2 for SU(",
 StyleBox["N",
  FontSlant->"Italic"],
 ")."
}], "Text",
 CellTags->"Tf"],

Cell[TextData[{
 "See also: ",
 ButtonBox["Tr2",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Tr2",
  ButtonNote->"Tr2"],
 ", ",
 ButtonBox["GluonPropagator",
  BaseStyle->"AddOnsLink",
  ButtonData:>"GluonPropagator",
  ButtonNote->"GluonPropagator"],
 ", ",
 ButtonBox["GluonSelfEnergy",
  BaseStyle->"AddOnsLink",
  ButtonData:>"GluonSelfEnergy",
  ButtonNote->"GluonSelfEnergy"],
 "."
}], "Text",
 CellTags->"Tf"]
}, Open  ]],

Cell["TFi", "ObjectName",
 CellTags->"TFi"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"TFi"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "TFi"}]], "Input",
 CellLabel->"In[1718]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   StyleBox[
    RowBox[{"Information", "::", "notfound"}], "MessageName"], 
   RowBox[{
   ":", " "}], "\<\"Symbol \[NoBreak]\\!\\(TraditionalForm\\`\\\"TFi\\\"\\)\
\[NoBreak] not found. \\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", \
ButtonStyle->\\\"Link\\\", ButtonFrame->None, \
ButtonData:>\\\"paclet:ref/message/General/notfound\\\", ButtonNote -> \
\\\"Information::notfound\\\"]\\)\"\>"}], TraditionalForm]], "Message", "MSG",
 
 CellChangeTimes->{3.500970437100914*^9}]
}, Open  ]],

Cell["\<\
TFi[d, pp, {{n1,m1},{n2,m2},{n3,m3},{n4,m4},{n5,m5}}] is the 2-loop \
d-dimensional integral 1/( (q1^2 - m1^2)^n1 (q2^2 - m2^2)^n2 ((q1-p)^2 - \
m3^2)^n3 *((q2-p)^2 - m4^2)^n4 * ((q1-q2)^2 - m5^2)^n5 ) . TFi[d, pp, \
{x,y,z,v,w}, {{n1,m1},{n2,m2},{n3,m3},{n4,m4},{n5,m5}}] has as additional \
factors in the numerator (q1^2)^x*(q2^2)^y*(q1.p)^z* (q2.p)^v*(q1.q2)^w. \
TFi[d, pp, dp, {a,b}, {{n1,m1},{n2,m2},{n3,m3},{n4,m4},{n5,m5}}] has as \
additional factors in the numerator (OPEDelta.q1)^a * (OPEDelta.q2)^b; dp is \
(OPEDelta.p).\
\>", "Text",
 CellTags->"TFi"],

Cell[TextData[{
 "TFi is similar to TFI from the TARCER package, see ",
 " ",
 ButtonBox["hep-ph/9801383.",
  BaseStyle->"Hyperlink",
  ButtonData:>{
    URL["http://xxx.lanl.gov/abs/hep-ph/9801383"], None}]
}], "Text",
 CellTags->"TFi"],

Cell["\<\
The function TarcerRecurse from the TARCER package recognize TFi (as well as \
TFI, which is defined in the HighEnergyPhysics`Tarcer` context).\
\>", "Text",
 CellTags->"TFi"],

Cell[TextData[{
 "See also: ",
 ButtonBox["ToTFi",
  BaseStyle->"AddOnsLink",
  ButtonData:>"ToTFi",
  ButtonNote->"ToTFi"],
 ", ",
 ButtonBox["FromTFi",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FromTFi",
  ButtonNote->"FromTFi"],
 "."
}], "Text",
 CellTags->"TFi"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"TFi"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TFi", "[", 
  RowBox[{"D", ",", 
   RowBox[{"M", "^", "2"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"2", ",", "m1"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"1", ",", "m2"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"3", ",", "m3"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"1", ",", "m4"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"1", ",", "m5"}], "}"}]}], "}"}]}], "]"}]], "Input",
 CellTags->"TFi",
 CellLabel->"In[1719]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"TFi", "(", 
   RowBox[{"D", ",", 
    SuperscriptBox["M", "2"], ",", 
    RowBox[{"(", "\[NoBreak]", GridBox[{
       {"2", "m1"},
       {"1", "m2"},
       {"3", "m3"},
       {"1", "m4"},
       {"1", "m5"}
      },
      GridBoxAlignment->{
       "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
        "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
      GridBoxSpacings->{"Columns" -> {
          Offset[0.27999999999999997`], {
           Offset[0.7]}, 
          Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
          Offset[0.2], {
           Offset[0.4]}, 
          Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}]}], ")"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.50097043753228*^9},
 CellTags->"TFi",
 CellLabel->"Out[1719]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TFi", "[", 
  RowBox[{"D", ",", 
   RowBox[{"M", "^", "2"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"2", ",", "M"}], "}"}], ",", "1", ",", "1", ",", "1", ",", 
     "1"}], "}"}]}], "]"}]], "Input",
 CellTags->"TFi",
 CellLabel->"In[1720]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"TFi", "(", 
   RowBox[{"D", ",", 
    SuperscriptBox["M", "2"], ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"2", ",", "M"}], "}"}], ",", "1", ",", "1", ",", "1", ",", 
      "1"}], "}"}]}], ")"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009704375917892`*^9},
 CellTags->"TFi",
 CellLabel->"Out[1720]="]
}, Open  ]]
}, Open  ]],

Cell["ThreeVector", "ObjectName",
 CellTags->"ThreeVector"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"ThreeVector"],

Cell["ThreeVector[p] is the three dimensional vector p.", "Text",
 CellTags->"ThreeVector"],

Cell[TextData[{
 "See also: ",
 ButtonBox["DotProduct",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DotProduct",
  ButtonNote->"DotProduct"],
 ", ",
 ButtonBox["CrossProduct",
  BaseStyle->"AddOnsLink",
  ButtonData:>"CrossProduct",
  ButtonNote->"CrossProduct"],
 "."
}], "Text",
 CellTags->"ThreeVector"]
}, Open  ]],

Cell["TID", "ObjectName",
 CellTags->"TID"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"TID"],

Cell["TID[amp,q] performs a tensorial decomposition.", "Text",
 CellTags->"TID"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "TID", "]"}]], "Input",
 CellTags->"TID",
 CellLabel->"In[1721]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Collecting", "\[Rule]", "True"}], ",", 
    RowBox[{"Contract", "\[Rule]", "False"}], ",", 
    RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
    RowBox[{"ChangeDimension", "\[Rule]", "D"}], ",", 
    RowBox[{"DimensionalReduction", "\[Rule]", "False"}], ",", 
    RowBox[{"FeynAmpDenominatorCombine", "\[Rule]", "True"}], ",", 
    RowBox[{"FeynAmpDenominatorSimplify", "\[Rule]", "False"}], ",", 
    RowBox[{"Isolate", "\[Rule]", "False"}], ",", 
    RowBox[{"ScalarProductCancel", "\[Rule]", "False"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970437631928*^9},
 CellTags->"TID",
 CellLabel->"Out[1721]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["OneLoopSimplify",
  BaseStyle->"AddOnsLink",
  ButtonData:>"OneLoopSimplify",
  ButtonNote->"OneLoopSimplify"],
 ", ",
 ButtonBox["TIDL",
  BaseStyle->"AddOnsLink",
  ButtonData:>"TIDL",
  ButtonNote->"TIDL"],
 "."
}], "Text",
 CellTags->"TID"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"TID"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"FAD", "[", 
    RowBox[{"k", ",", " ", 
     RowBox[{"k", " ", "-", " ", 
      SubscriptBox["p", "1"]}], ",", " ", 
     RowBox[{"k", " ", "-", " ", 
      SubscriptBox["p", "2"]}]}], "]"}], 
   RowBox[{"FVD", "[", 
    RowBox[{"k", ",", "\[Mu]"}], "]"}]}], "//", "FCI"}]], "Input",
 CellTags->"TID",
 CellLabel->"In[1722]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   SuperscriptBox[
    FormBox["k",
     TraditionalForm], 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], 
   RowBox[{
    SuperscriptBox["k", "2"], ".", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"k", "-", 
       SubscriptBox["p", "1"]}], ")"}], "2"], ".", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"k", "-", 
       SubscriptBox["p", "2"]}], ")"}], "2"]}]], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009704377063513`*^9},
 CellTags->"TID",
 CellLabel->"Out[1722]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Factor", "/@", 
   RowBox[{"TID", "[", 
    RowBox[{"%", ",", "k"}], "]"}]}], "//", "FCE"}]], "Input",
 CellTags->"TID",
 CellLabel->"In[1723]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{
     FormBox[
      RowBox[{
       FormBox["k",
        TraditionalForm], "\[CenterDot]", 
       FormBox[
        SubscriptBox["p", "2"],
        TraditionalForm]}],
      TraditionalForm], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        FormBox[
         SubsuperscriptBox["p", "1", "2"],
         TraditionalForm], " ", 
        SubsuperscriptBox["p", 
         FormBox["2",
          TraditionalForm], "\[Mu]"]}], "-", 
       FractionBox[
        RowBox[{"s", " ", 
         SubsuperscriptBox["p", 
          FormBox["1",
           TraditionalForm], "\[Mu]"]}], "2"]}], ")"}], " ", 
     FormBox[
      FractionBox["1", 
       RowBox[{
        RowBox[{"(", 
         InterpretationBox[
          RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
           SuperscriptBox["k", "2"], "\[InvisibleSpace]", "\<\"]\"\>"}],
          SequenceForm["[", $CellContext`k^2, "]"],
          Editable->False], ")"}], " ", 
        RowBox[{"(", 
         InterpretationBox[
          RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{"k", "-", 
              SubscriptBox["p", "1"]}], ")"}], "2"], 
           "\[InvisibleSpace]", "\<\"]\"\>"}],
          SequenceForm[
          "[", ($CellContext`k - Subscript[$CellContext`p, 1])^2, "]"],
          Editable->False], ")"}], " ", 
        RowBox[{"(", 
         InterpretationBox[
          RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{"k", "-", 
              SubscriptBox["p", "2"]}], ")"}], "2"], 
           "\[InvisibleSpace]", "\<\"]\"\>"}],
          SequenceForm[
          "[", ($CellContext`k - Subscript[$CellContext`p, 2])^2, "]"],
          Editable->False], ")"}]}]],
      TraditionalForm]}], 
    RowBox[{
     RowBox[{
      FormBox[
       SubsuperscriptBox["p", "1", "2"],
       TraditionalForm], " ", 
      FormBox[
       SubsuperscriptBox["p", "2", "2"],
       TraditionalForm]}], "-", 
     FractionBox[
      SuperscriptBox["s", "2"], "4"]}]], "+", 
   FractionBox[
    RowBox[{
     FormBox[
      RowBox[{
       FormBox["k",
        TraditionalForm], "\[CenterDot]", 
       FormBox[
        SubscriptBox["p", "1"],
        TraditionalForm]}],
      TraditionalForm], " ", 
     RowBox[{"(", 
      RowBox[{
       FractionBox[
        RowBox[{"s", " ", 
         SubsuperscriptBox["p", 
          FormBox["2",
           TraditionalForm], "\[Mu]"]}], "2"], "-", 
       RowBox[{
        FormBox[
         SubsuperscriptBox["p", "2", "2"],
         TraditionalForm], " ", 
        SubsuperscriptBox["p", 
         FormBox["1",
          TraditionalForm], "\[Mu]"]}]}], ")"}], " ", 
     FormBox[
      FractionBox["1", 
       RowBox[{
        RowBox[{"(", 
         InterpretationBox[
          RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
           SuperscriptBox["k", "2"], "\[InvisibleSpace]", "\<\"]\"\>"}],
          SequenceForm["[", $CellContext`k^2, "]"],
          Editable->False], ")"}], " ", 
        RowBox[{"(", 
         InterpretationBox[
          RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{"k", "-", 
              SubscriptBox["p", "1"]}], ")"}], "2"], 
           "\[InvisibleSpace]", "\<\"]\"\>"}],
          SequenceForm[
          "[", ($CellContext`k - Subscript[$CellContext`p, 1])^2, "]"],
          Editable->False], ")"}], " ", 
        RowBox[{"(", 
         InterpretationBox[
          RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{"k", "-", 
              SubscriptBox["p", "2"]}], ")"}], "2"], 
           "\[InvisibleSpace]", "\<\"]\"\>"}],
          SequenceForm[
          "[", ($CellContext`k - Subscript[$CellContext`p, 2])^2, "]"],
          Editable->False], ")"}]}]],
      TraditionalForm]}], 
    RowBox[{
     FractionBox[
      SuperscriptBox["s", "2"], "4"], "-", 
     RowBox[{
      FormBox[
       SubsuperscriptBox["p", "1", "2"],
       TraditionalForm], " ", 
      FormBox[
       SubsuperscriptBox["p", "2", "2"],
       TraditionalForm]}]}]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970437866062*^9},
 CellTags->"TID",
 CellLabel->"Out[1723]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"FAD", "[", 
    RowBox[{"k", ",", " ", 
     RowBox[{"k", " ", "-", " ", "p"}]}], "]"}], 
   RowBox[{"FVD", "[", 
    RowBox[{"k", ",", "\[Mu]"}], "]"}]}], "//", "FCI"}]], "Input",
 CellTags->"TID",
 CellLabel->"In[1724]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   SuperscriptBox[
    FormBox["k",
     TraditionalForm], 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], 
   RowBox[{
    SuperscriptBox["k", "2"], ".", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"k", "-", "p"}], ")"}], "2"]}]], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970438039885*^9},
 CellTags->"TID",
 CellLabel->"Out[1724]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TID", "[", 
  RowBox[{"%", ",", "k"}], "]"}]], "Input",
 CellTags->"TID",
 CellLabel->"In[1725]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{
    SuperscriptBox[
     FormBox["p",
      TraditionalForm], 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], " ", 
    RowBox[{
     FormBox["k",
      TraditionalForm], "\[CenterDot]", 
     FormBox["p",
      TraditionalForm]}]}], 
   RowBox[{
    SuperscriptBox[
     FormBox["p",
      TraditionalForm], "2"], " ", 
    RowBox[{
     SuperscriptBox["k", "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"k", "-", "p"}], ")"}], "2"]}]}]], TraditionalForm]], "Output",\

 CellChangeTimes->{3.500970438159384*^9},
 CellTags->"TID",
 CellLabel->"Out[1725]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TID", "[", 
  RowBox[{"%%", ",", "k", ",", 
   RowBox[{"FeynAmpDenominatorSimplify", "\[Rule]", "True"}]}], 
  "]"}]], "Input",
 CellTags->"TID",
 CellLabel->"In[1726]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{
    SuperscriptBox[
     FormBox["p",
      TraditionalForm], 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], " ", 
    RowBox[{
     FormBox["k",
      TraditionalForm], "\[CenterDot]", 
     FormBox["p",
      TraditionalForm]}]}], 
   RowBox[{
    SuperscriptBox[
     FormBox["p",
      TraditionalForm], "2"], " ", 
    RowBox[{
     SuperscriptBox["k", "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"k", "-", "p"}], ")"}], "2"]}]}]], TraditionalForm]], "Output",\

 CellChangeTimes->{3.500970438476099*^9},
 CellTags->"TID",
 CellLabel->"Out[1726]="]
}, Open  ]]
}, Open  ]],

Cell["TIDL", "ObjectName",
 CellTags->"TIDL"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"TIDL"],

Cell[" TIDL is a database of tensorial reduction formalae.", "Text",
 CellTags->"TIDL"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["TID",
  BaseStyle->"AddOnsLink",
  ButtonData:>"TID",
  ButtonNote->"TID"]
}], "Text",
 CellTags->"TIDL"]
}, Open  ]],

Cell["Examples", "Subsubsection",
 CellTags->"TIDL"],

Cell[CellGroupData[{

Cell[TextData[Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["B", "\[Mu]"], "-", "type"}], 
  TraditionalForm]]]], "Subsubsection",
 CellTags->"TIDL"],

Cell[TextData[{
 "In any n-dimensional integral ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Integral]", 
    RowBox[{
     SuperscriptBox["d", "n"], "\[NegativeVeryThinSpace]", 
     SuperscriptBox["q", "\[Mu]"], " ", 
     RowBox[{"f", "(", 
      RowBox[{"q", ",", "p"}], ")"}], " ", "the", " ", 
     SuperscriptBox["q", "\[Mu]"]}], " "}], TraditionalForm]]],
 "can be replaced by"
}], "Text",
 CellTags->"TIDL"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TIDL", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"q", ",", "\[Mu]"}], "}"}], ",", 
   RowBox[{"{", "p", "}"}]}], "]"}]], "Input",
 CellTags->"TIDL",
 CellLabel->"In[1727]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{
    SuperscriptBox[
     FormBox["p",
      TraditionalForm], 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], " ", 
    RowBox[{
     FormBox["p",
      TraditionalForm], "\[CenterDot]", 
     FormBox["q",
      TraditionalForm]}]}], 
   SuperscriptBox[
    FormBox["p",
     TraditionalForm], "2"]], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970438704732*^9},
 CellTags->"TIDL",
 CellLabel->"Out[1727]="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["B", "\[Mu]\[Nu]"], "-", "type"}], 
  TraditionalForm]]]], "Subsubsection",
 CellTags->"TIDL"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TIDL", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"q", ",", "\[Mu]"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"q", ",", "\[Nu]"}], "}"}]}], "}"}], ",", 
   RowBox[{"{", "p", "}"}]}], "]"}]], "Input",
 CellTags->"TIDL",
 CellLabel->"In[1728]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{
     SuperscriptBox["g", 
      RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox[
        FormBox[
         RowBox[{
          FormBox["p",
           TraditionalForm], "\[CenterDot]", 
          FormBox["q",
           TraditionalForm]}],
         TraditionalForm], "2"], "-", 
       RowBox[{
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], "2"], " ", 
        SuperscriptBox[
         FormBox["q",
          TraditionalForm], "2"]}]}], ")"}]}], 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"1", "-", "D"}], ")"}], " ", 
     SuperscriptBox[
      FormBox["p",
       TraditionalForm], "2"]}]], "-", 
   FractionBox[
    RowBox[{
     SuperscriptBox[
      FormBox["p",
       TraditionalForm], 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], " ", 
     SuperscriptBox[
      FormBox["p",
       TraditionalForm], 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm]], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"D", " ", 
        SuperscriptBox[
         FormBox[
          RowBox[{
           FormBox["p",
            TraditionalForm], "\[CenterDot]", 
           FormBox["q",
            TraditionalForm]}],
          TraditionalForm], "2"]}], "-", 
       RowBox[{
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], "2"], " ", 
        SuperscriptBox[
         FormBox["q",
          TraditionalForm], "2"]}]}], ")"}]}], 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"1", "-", "D"}], ")"}], " ", 
     SuperscriptBox[
      FormBox[
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], "2"],
       TraditionalForm], "2"]}]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009704389330807`*^9},
 CellTags->"TIDL",
 CellLabel->"Out[1728]="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["C", "\[Mu]"], "-", "type"}], 
  TraditionalForm]]]], "Subsubsection",
 CellTags->"TIDL"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TIDL", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"q", ",", "\[Mu]"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     SubscriptBox["p", "1"], ",", 
     SubscriptBox["p", "2"]}], "}"}]}], "]"}]], "Input",
 CellTags->"TIDL",
 CellLabel->"In[1729]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{
     RowBox[{
      SubsuperscriptBox["p", 
       FormBox["2",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      RowBox[{
       FormBox[
        SubscriptBox["p", "1"],
        TraditionalForm], "\[CenterDot]", 
       FormBox[
        SubscriptBox["p", "2"],
        TraditionalForm]}], " ", 
      RowBox[{
       FormBox["q",
        TraditionalForm], "\[CenterDot]", 
       FormBox[
        SubscriptBox["p", "1"],
        TraditionalForm]}]}], "-", 
     RowBox[{
      SubsuperscriptBox["p", "1", "2"], " ", 
      SubsuperscriptBox["p", 
       FormBox["2",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      RowBox[{
       FormBox["q",
        TraditionalForm], "\[CenterDot]", 
       FormBox[
        SubscriptBox["p", "2"],
        TraditionalForm]}]}]}], 
    RowBox[{
     SuperscriptBox[
      FormBox[
       RowBox[{
        FormBox[
         SubscriptBox["p", "1"],
         TraditionalForm], "\[CenterDot]", 
        FormBox[
         SubscriptBox["p", "2"],
         TraditionalForm]}],
       TraditionalForm], "2"], "-", 
     RowBox[{
      SubsuperscriptBox["p", "1", "2"], " ", 
      SubsuperscriptBox["p", "2", "2"]}]}]], "+", 
   FractionBox[
    RowBox[{
     RowBox[{
      SubsuperscriptBox["p", 
       FormBox["1",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      RowBox[{
       FormBox[
        SubscriptBox["p", "1"],
        TraditionalForm], "\[CenterDot]", 
       FormBox[
        SubscriptBox["p", "2"],
        TraditionalForm]}], " ", 
      RowBox[{
       FormBox["q",
        TraditionalForm], "\[CenterDot]", 
       FormBox[
        SubscriptBox["p", "2"],
        TraditionalForm]}]}], "-", 
     RowBox[{
      SubsuperscriptBox["p", "2", "2"], " ", 
      SubsuperscriptBox["p", 
       FormBox["1",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      RowBox[{
       FormBox["q",
        TraditionalForm], "\[CenterDot]", 
       FormBox[
        SubscriptBox["p", "1"],
        TraditionalForm]}]}]}], 
    RowBox[{
     SuperscriptBox[
      FormBox[
       RowBox[{
        FormBox[
         SubscriptBox["p", "1"],
         TraditionalForm], "\[CenterDot]", 
        FormBox[
         SubscriptBox["p", "2"],
         TraditionalForm]}],
       TraditionalForm], "2"], "-", 
     RowBox[{
      SubsuperscriptBox["p", "1", "2"], " ", 
      SubsuperscriptBox["p", "2", "2"]}]}]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970439166231*^9},
 CellTags->"TIDL",
 CellLabel->"Out[1729]="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["C", "\[Mu]\[Nu]"], "-", "type"}], 
  TraditionalForm]]]], "Subsubsection",
 CellTags->"TIDL"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TIDL", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"q", ",", "\[Mu]"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"q", ",", "\[Nu]"}], "}"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     SubscriptBox["p", "1"], ",", 
     SubscriptBox["p", "2"]}], "}"}]}], "]"}]], "Input",
 CellTags->"TIDL",
 CellLabel->"In[1730]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{
     SuperscriptBox["g", 
      RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        SubsuperscriptBox["p", "2", "2"], " ", 
        SuperscriptBox[
         FormBox[
          RowBox[{
           FormBox["q",
            TraditionalForm], "\[CenterDot]", 
           FormBox[
            SubscriptBox["p", "1"],
            TraditionalForm]}],
          TraditionalForm], "2"]}], "-", 
       RowBox[{"2", " ", 
        RowBox[{
         FormBox["q",
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "2"],
          TraditionalForm]}], " ", 
        RowBox[{
         FormBox[
          SubscriptBox["p", "1"],
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "2"],
          TraditionalForm]}], " ", 
        RowBox[{
         FormBox["q",
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "1"],
          TraditionalForm]}]}], "+", 
       RowBox[{
        SuperscriptBox[
         FormBox["q",
          TraditionalForm], "2"], " ", 
        SuperscriptBox[
         FormBox[
          RowBox[{
           FormBox[
            SubscriptBox["p", "1"],
            TraditionalForm], "\[CenterDot]", 
           FormBox[
            SubscriptBox["p", "2"],
            TraditionalForm]}],
          TraditionalForm], "2"]}], "+", 
       RowBox[{
        SuperscriptBox[
         FormBox[
          RowBox[{
           FormBox["q",
            TraditionalForm], "\[CenterDot]", 
           FormBox[
            SubscriptBox["p", "2"],
            TraditionalForm]}],
          TraditionalForm], "2"], " ", 
        SubsuperscriptBox["p", "1", "2"]}], "-", 
       RowBox[{
        SuperscriptBox[
         FormBox["q",
          TraditionalForm], "2"], " ", 
        SubsuperscriptBox["p", "1", "2"], " ", 
        SubsuperscriptBox["p", "2", "2"]}]}], ")"}]}], 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"D", "-", "2"}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox[
        FormBox[
         RowBox[{
          FormBox[
           SubscriptBox["p", "1"],
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["p", "2"],
           TraditionalForm]}],
         TraditionalForm], "2"], "-", 
       RowBox[{
        SubsuperscriptBox["p", "1", "2"], " ", 
        SubsuperscriptBox["p", "2", "2"]}]}], ")"}]}]], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      SubsuperscriptBox["p", 
       FormBox["2",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SubsuperscriptBox["p", 
       FormBox["2",
        TraditionalForm], 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"D", " ", 
         SuperscriptBox[
          FormBox[
           RowBox[{
            FormBox[
             SubscriptBox["p", "1"],
             TraditionalForm], "\[CenterDot]", 
            FormBox[
             SubscriptBox["p", "2"],
             TraditionalForm]}],
           TraditionalForm], "2"], " ", 
         SuperscriptBox[
          FormBox[
           RowBox[{
            FormBox["q",
             TraditionalForm], "\[CenterDot]", 
            FormBox[
             SubscriptBox["p", "1"],
             TraditionalForm]}],
           TraditionalForm], "2"]}], "-", 
        RowBox[{"2", " ", 
         SuperscriptBox[
          FormBox[
           RowBox[{
            FormBox[
             SubscriptBox["p", "1"],
             TraditionalForm], "\[CenterDot]", 
            FormBox[
             SubscriptBox["p", "2"],
             TraditionalForm]}],
           TraditionalForm], "2"], " ", 
         SuperscriptBox[
          FormBox[
           RowBox[{
            FormBox["q",
             TraditionalForm], "\[CenterDot]", 
            FormBox[
             SubscriptBox["p", "1"],
             TraditionalForm]}],
           TraditionalForm], "2"]}], "+", 
        RowBox[{
         SubsuperscriptBox["p", "1", "2"], " ", 
         SubsuperscriptBox["p", "2", "2"], " ", 
         SuperscriptBox[
          FormBox[
           RowBox[{
            FormBox["q",
             TraditionalForm], "\[CenterDot]", 
            FormBox[
             SubscriptBox["p", "1"],
             TraditionalForm]}],
           TraditionalForm], "2"]}], "-", 
        RowBox[{"2", " ", "D", " ", 
         RowBox[{
          FormBox["q",
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["p", "2"],
           TraditionalForm]}], " ", 
         SubsuperscriptBox["p", "1", "2"], " ", 
         RowBox[{
          FormBox[
           SubscriptBox["p", "1"],
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["p", "2"],
           TraditionalForm]}], " ", 
         RowBox[{
          FormBox["q",
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["p", "1"],
           TraditionalForm]}]}], "+", 
        RowBox[{"2", " ", 
         RowBox[{
          FormBox["q",
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["p", "2"],
           TraditionalForm]}], " ", 
         SubsuperscriptBox["p", "1", "2"], " ", 
         RowBox[{
          FormBox[
           SubscriptBox["p", "1"],
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["p", "2"],
           TraditionalForm]}], " ", 
         RowBox[{
          FormBox["q",
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["p", "1"],
           TraditionalForm]}]}], "+", 
        RowBox[{"D", " ", 
         SuperscriptBox[
          FormBox[
           RowBox[{
            FormBox["q",
             TraditionalForm], "\[CenterDot]", 
            FormBox[
             SubscriptBox["p", "2"],
             TraditionalForm]}],
           TraditionalForm], "2"], " ", 
         SuperscriptBox[
          FormBox[
           SubsuperscriptBox["p", "1", "2"],
           TraditionalForm], "2"]}], "-", 
        RowBox[{
         SuperscriptBox[
          FormBox[
           RowBox[{
            FormBox["q",
             TraditionalForm], "\[CenterDot]", 
            FormBox[
             SubscriptBox["p", "2"],
             TraditionalForm]}],
           TraditionalForm], "2"], " ", 
         SuperscriptBox[
          FormBox[
           SubsuperscriptBox["p", "1", "2"],
           TraditionalForm], "2"]}], "+", 
        RowBox[{
         SuperscriptBox[
          FormBox["q",
           TraditionalForm], "2"], " ", 
         SubsuperscriptBox["p", "1", "2"], " ", 
         SuperscriptBox[
          FormBox[
           RowBox[{
            FormBox[
             SubscriptBox["p", "1"],
             TraditionalForm], "\[CenterDot]", 
            FormBox[
             SubscriptBox["p", "2"],
             TraditionalForm]}],
           TraditionalForm], "2"]}], "-", 
        RowBox[{
         SuperscriptBox[
          FormBox["q",
           TraditionalForm], "2"], " ", 
         SuperscriptBox[
          FormBox[
           SubsuperscriptBox["p", "1", "2"],
           TraditionalForm], "2"], " ", 
         SubsuperscriptBox["p", "2", "2"]}]}], ")"}]}], ")"}], "/", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"D", "-", "2"}], ")"}], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          FormBox[
           RowBox[{
            FormBox[
             SubscriptBox["p", "1"],
             TraditionalForm], "\[CenterDot]", 
            FormBox[
             SubscriptBox["p", "2"],
             TraditionalForm]}],
           TraditionalForm], "2"], "-", 
         RowBox[{
          SubsuperscriptBox["p", "1", "2"], " ", 
          SubsuperscriptBox["p", "2", "2"]}]}], ")"}], "2"]}], ")"}]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      SubsuperscriptBox["p", 
       FormBox["2",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SubsuperscriptBox["p", 
       FormBox["1",
        TraditionalForm], 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{"-", 
          SuperscriptBox[
           FormBox["q",
            TraditionalForm], "2"]}], " ", 
         SuperscriptBox[
          FormBox[
           RowBox[{
            FormBox[
             SubscriptBox["p", "1"],
             TraditionalForm], "\[CenterDot]", 
            FormBox[
             SubscriptBox["p", "2"],
             TraditionalForm]}],
           TraditionalForm], "3"]}], "+", 
        RowBox[{"D", " ", 
         RowBox[{
          FormBox["q",
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["p", "1"],
           TraditionalForm]}], " ", 
         RowBox[{
          FormBox["q",
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["p", "2"],
           TraditionalForm]}], " ", 
         SuperscriptBox[
          FormBox[
           RowBox[{
            FormBox[
             SubscriptBox["p", "1"],
             TraditionalForm], "\[CenterDot]", 
            FormBox[
             SubscriptBox["p", "2"],
             TraditionalForm]}],
           TraditionalForm], "2"]}], "-", 
        RowBox[{"D", " ", 
         SuperscriptBox[
          FormBox[
           RowBox[{
            FormBox["q",
             TraditionalForm], "\[CenterDot]", 
            FormBox[
             SubscriptBox["p", "2"],
             TraditionalForm]}],
           TraditionalForm], "2"], " ", 
         SubsuperscriptBox["p", "1", "2"], " ", 
         RowBox[{
          FormBox[
           SubscriptBox["p", "1"],
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["p", "2"],
           TraditionalForm]}]}], "+", 
        RowBox[{
         SuperscriptBox[
          FormBox[
           RowBox[{
            FormBox["q",
             TraditionalForm], "\[CenterDot]", 
            FormBox[
             SubscriptBox["p", "2"],
             TraditionalForm]}],
           TraditionalForm], "2"], " ", 
         SubsuperscriptBox["p", "1", "2"], " ", 
         RowBox[{
          FormBox[
           SubscriptBox["p", "1"],
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["p", "2"],
           TraditionalForm]}]}], "-", 
        RowBox[{"D", " ", 
         SuperscriptBox[
          FormBox[
           RowBox[{
            FormBox["q",
             TraditionalForm], "\[CenterDot]", 
            FormBox[
             SubscriptBox["p", "1"],
             TraditionalForm]}],
           TraditionalForm], "2"], " ", 
         SubsuperscriptBox["p", "2", "2"], " ", 
         RowBox[{
          FormBox[
           SubscriptBox["p", "1"],
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["p", "2"],
           TraditionalForm]}]}], "+", 
        RowBox[{
         SuperscriptBox[
          FormBox[
           RowBox[{
            FormBox["q",
             TraditionalForm], "\[CenterDot]", 
            FormBox[
             SubscriptBox["p", "1"],
             TraditionalForm]}],
           TraditionalForm], "2"], " ", 
         SubsuperscriptBox["p", "2", "2"], " ", 
         RowBox[{
          FormBox[
           SubscriptBox["p", "1"],
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["p", "2"],
           TraditionalForm]}]}], "+", 
        RowBox[{
         SuperscriptBox[
          FormBox["q",
           TraditionalForm], "2"], " ", 
         SubsuperscriptBox["p", "1", "2"], " ", 
         SubsuperscriptBox["p", "2", "2"], " ", 
         RowBox[{
          FormBox[
           SubscriptBox["p", "1"],
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["p", "2"],
           TraditionalForm]}]}], "+", 
        RowBox[{"D", " ", 
         RowBox[{
          FormBox["q",
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["p", "1"],
           TraditionalForm]}], " ", 
         RowBox[{
          FormBox["q",
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["p", "2"],
           TraditionalForm]}], " ", 
         SubsuperscriptBox["p", "1", "2"], " ", 
         SubsuperscriptBox["p", "2", "2"]}], "-", 
        RowBox[{"2", " ", 
         RowBox[{
          FormBox["q",
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["p", "1"],
           TraditionalForm]}], " ", 
         RowBox[{
          FormBox["q",
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["p", "2"],
           TraditionalForm]}], " ", 
         SubsuperscriptBox["p", "1", "2"], " ", 
         SubsuperscriptBox["p", "2", "2"]}]}], ")"}]}], ")"}], "/", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"D", "-", "2"}], ")"}], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          FormBox[
           RowBox[{
            FormBox[
             SubscriptBox["p", "1"],
             TraditionalForm], "\[CenterDot]", 
            FormBox[
             SubscriptBox["p", "2"],
             TraditionalForm]}],
           TraditionalForm], "2"], "-", 
         RowBox[{
          SubsuperscriptBox["p", "1", "2"], " ", 
          SubsuperscriptBox["p", "2", "2"]}]}], ")"}], "2"]}], ")"}]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      SubsuperscriptBox["p", 
       FormBox["1",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SubsuperscriptBox["p", 
       FormBox["2",
        TraditionalForm], 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{"-", 
          SuperscriptBox[
           FormBox["q",
            TraditionalForm], "2"]}], " ", 
         SuperscriptBox[
          FormBox[
           RowBox[{
            FormBox[
             SubscriptBox["p", "1"],
             TraditionalForm], "\[CenterDot]", 
            FormBox[
             SubscriptBox["p", "2"],
             TraditionalForm]}],
           TraditionalForm], "3"]}], "+", 
        RowBox[{"D", " ", 
         RowBox[{
          FormBox["q",
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["p", "1"],
           TraditionalForm]}], " ", 
         RowBox[{
          FormBox["q",
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["p", "2"],
           TraditionalForm]}], " ", 
         SuperscriptBox[
          FormBox[
           RowBox[{
            FormBox[
             SubscriptBox["p", "1"],
             TraditionalForm], "\[CenterDot]", 
            FormBox[
             SubscriptBox["p", "2"],
             TraditionalForm]}],
           TraditionalForm], "2"]}], "-", 
        RowBox[{"D", " ", 
         SuperscriptBox[
          FormBox[
           RowBox[{
            FormBox["q",
             TraditionalForm], "\[CenterDot]", 
            FormBox[
             SubscriptBox["p", "2"],
             TraditionalForm]}],
           TraditionalForm], "2"], " ", 
         SubsuperscriptBox["p", "1", "2"], " ", 
         RowBox[{
          FormBox[
           SubscriptBox["p", "1"],
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["p", "2"],
           TraditionalForm]}]}], "+", 
        RowBox[{
         SuperscriptBox[
          FormBox[
           RowBox[{
            FormBox["q",
             TraditionalForm], "\[CenterDot]", 
            FormBox[
             SubscriptBox["p", "2"],
             TraditionalForm]}],
           TraditionalForm], "2"], " ", 
         SubsuperscriptBox["p", "1", "2"], " ", 
         RowBox[{
          FormBox[
           SubscriptBox["p", "1"],
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["p", "2"],
           TraditionalForm]}]}], "-", 
        RowBox[{"D", " ", 
         SuperscriptBox[
          FormBox[
           RowBox[{
            FormBox["q",
             TraditionalForm], "\[CenterDot]", 
            FormBox[
             SubscriptBox["p", "1"],
             TraditionalForm]}],
           TraditionalForm], "2"], " ", 
         SubsuperscriptBox["p", "2", "2"], " ", 
         RowBox[{
          FormBox[
           SubscriptBox["p", "1"],
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["p", "2"],
           TraditionalForm]}]}], "+", 
        RowBox[{
         SuperscriptBox[
          FormBox[
           RowBox[{
            FormBox["q",
             TraditionalForm], "\[CenterDot]", 
            FormBox[
             SubscriptBox["p", "1"],
             TraditionalForm]}],
           TraditionalForm], "2"], " ", 
         SubsuperscriptBox["p", "2", "2"], " ", 
         RowBox[{
          FormBox[
           SubscriptBox["p", "1"],
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["p", "2"],
           TraditionalForm]}]}], "+", 
        RowBox[{
         SuperscriptBox[
          FormBox["q",
           TraditionalForm], "2"], " ", 
         SubsuperscriptBox["p", "1", "2"], " ", 
         SubsuperscriptBox["p", "2", "2"], " ", 
         RowBox[{
          FormBox[
           SubscriptBox["p", "1"],
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["p", "2"],
           TraditionalForm]}]}], "+", 
        RowBox[{"D", " ", 
         RowBox[{
          FormBox["q",
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["p", "1"],
           TraditionalForm]}], " ", 
         RowBox[{
          FormBox["q",
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["p", "2"],
           TraditionalForm]}], " ", 
         SubsuperscriptBox["p", "1", "2"], " ", 
         SubsuperscriptBox["p", "2", "2"]}], "-", 
        RowBox[{"2", " ", 
         RowBox[{
          FormBox["q",
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["p", "1"],
           TraditionalForm]}], " ", 
         RowBox[{
          FormBox["q",
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["p", "2"],
           TraditionalForm]}], " ", 
         SubsuperscriptBox["p", "1", "2"], " ", 
         SubsuperscriptBox["p", "2", "2"]}]}], ")"}]}], ")"}], "/", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"D", "-", "2"}], ")"}], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          FormBox[
           RowBox[{
            FormBox[
             SubscriptBox["p", "1"],
             TraditionalForm], "\[CenterDot]", 
            FormBox[
             SubscriptBox["p", "2"],
             TraditionalForm]}],
           TraditionalForm], "2"], "-", 
         RowBox[{
          SubsuperscriptBox["p", "1", "2"], " ", 
          SubsuperscriptBox["p", "2", "2"]}]}], ")"}], "2"]}], ")"}]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      SubsuperscriptBox["p", 
       FormBox["1",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SubsuperscriptBox["p", 
       FormBox["1",
        TraditionalForm], 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"D", " ", 
         SuperscriptBox[
          FormBox[
           RowBox[{
            FormBox[
             SubscriptBox["p", "1"],
             TraditionalForm], "\[CenterDot]", 
            FormBox[
             SubscriptBox["p", "2"],
             TraditionalForm]}],
           TraditionalForm], "2"], " ", 
         SuperscriptBox[
          FormBox[
           RowBox[{
            FormBox["q",
             TraditionalForm], "\[CenterDot]", 
            FormBox[
             SubscriptBox["p", "2"],
             TraditionalForm]}],
           TraditionalForm], "2"]}], "-", 
        RowBox[{"2", " ", 
         SuperscriptBox[
          FormBox[
           RowBox[{
            FormBox[
             SubscriptBox["p", "1"],
             TraditionalForm], "\[CenterDot]", 
            FormBox[
             SubscriptBox["p", "2"],
             TraditionalForm]}],
           TraditionalForm], "2"], " ", 
         SuperscriptBox[
          FormBox[
           RowBox[{
            FormBox["q",
             TraditionalForm], "\[CenterDot]", 
            FormBox[
             SubscriptBox["p", "2"],
             TraditionalForm]}],
           TraditionalForm], "2"]}], "+", 
        RowBox[{
         SubsuperscriptBox["p", "1", "2"], " ", 
         SubsuperscriptBox["p", "2", "2"], " ", 
         SuperscriptBox[
          FormBox[
           RowBox[{
            FormBox["q",
             TraditionalForm], "\[CenterDot]", 
            FormBox[
             SubscriptBox["p", "2"],
             TraditionalForm]}],
           TraditionalForm], "2"]}], "-", 
        RowBox[{"2", " ", "D", " ", 
         RowBox[{
          FormBox["q",
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["p", "1"],
           TraditionalForm]}], " ", 
         RowBox[{
          FormBox[
           SubscriptBox["p", "1"],
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["p", "2"],
           TraditionalForm]}], " ", 
         SubsuperscriptBox["p", "2", "2"], " ", 
         RowBox[{
          FormBox["q",
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["p", "2"],
           TraditionalForm]}]}], "+", 
        RowBox[{"2", " ", 
         RowBox[{
          FormBox["q",
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["p", "1"],
           TraditionalForm]}], " ", 
         RowBox[{
          FormBox[
           SubscriptBox["p", "1"],
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["p", "2"],
           TraditionalForm]}], " ", 
         SubsuperscriptBox["p", "2", "2"], " ", 
         RowBox[{
          FormBox["q",
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["p", "2"],
           TraditionalForm]}]}], "+", 
        RowBox[{"D", " ", 
         SuperscriptBox[
          FormBox[
           RowBox[{
            FormBox["q",
             TraditionalForm], "\[CenterDot]", 
            FormBox[
             SubscriptBox["p", "1"],
             TraditionalForm]}],
           TraditionalForm], "2"], " ", 
         SuperscriptBox[
          FormBox[
           SubsuperscriptBox["p", "2", "2"],
           TraditionalForm], "2"]}], "-", 
        RowBox[{
         SuperscriptBox[
          FormBox[
           RowBox[{
            FormBox["q",
             TraditionalForm], "\[CenterDot]", 
            FormBox[
             SubscriptBox["p", "1"],
             TraditionalForm]}],
           TraditionalForm], "2"], " ", 
         SuperscriptBox[
          FormBox[
           SubsuperscriptBox["p", "2", "2"],
           TraditionalForm], "2"]}], "-", 
        RowBox[{
         SuperscriptBox[
          FormBox["q",
           TraditionalForm], "2"], " ", 
         SubsuperscriptBox["p", "1", "2"], " ", 
         SuperscriptBox[
          FormBox[
           SubsuperscriptBox["p", "2", "2"],
           TraditionalForm], "2"]}], "+", 
        RowBox[{
         SuperscriptBox[
          FormBox["q",
           TraditionalForm], "2"], " ", 
         SuperscriptBox[
          FormBox[
           RowBox[{
            FormBox[
             SubscriptBox["p", "1"],
             TraditionalForm], "\[CenterDot]", 
            FormBox[
             SubscriptBox["p", "2"],
             TraditionalForm]}],
           TraditionalForm], "2"], " ", 
         SubsuperscriptBox["p", "2", "2"]}]}], ")"}]}], ")"}], "/", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"D", "-", "2"}], ")"}], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          FormBox[
           RowBox[{
            FormBox[
             SubscriptBox["p", "1"],
             TraditionalForm], "\[CenterDot]", 
            FormBox[
             SubscriptBox["p", "2"],
             TraditionalForm]}],
           TraditionalForm], "2"], "-", 
         RowBox[{
          SubsuperscriptBox["p", "1", "2"], " ", 
          SubsuperscriptBox["p", "2", "2"]}]}], ")"}], "2"]}], ")"}]}]}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970439334076*^9},
 CellTags->"TIDL",
 CellLabel->"Out[1730]="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["C", "\[Mu]\[Nu]\[Rho]"], "-", "type"}], 
  TraditionalForm]]]], "Subsubsection",
 CellTags->"TIDL"],

Cell[BoxData[
 RowBox[{
  RowBox[{"TIDL", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"q", ",", "\[Mu]"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"q", ",", "\[Nu]"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"q", ",", "\[Rho]"}], "}"}]}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"p", ",", "k"}], "}"}]}], "]"}], ";"}]], "Input",
 CellTags->"TIDL",
 CellLabel->"In[1731]:="]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["C", "\[Mu]\[Nu]\[Rho]\[Sigma]"], "-", "type"}], 
  TraditionalForm]]]], "Subsubsection",
 CellTags->"TIDL"],

Cell[BoxData[
 RowBox[{
  RowBox[{"TIDL", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"q", ",", "\[Mu]"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"q", ",", "\[Nu]"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"q", ",", "\[Rho]"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"q", ",", "\[Sigma]"}], "}"}]}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"p", ",", "k"}], "}"}]}], "]"}], ";"}]], "Input",
 CellTags->"TIDL",
 CellLabel->"In[1732]:="]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["D", "\[Mu]"], "-", "type"}], 
  TraditionalForm]]]], "Subsubsection",
 CellTags->"TIDL"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TIDL", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"q", ",", "\[Mu]"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     SubscriptBox["p", "1"], ",", 
     SubscriptBox["p", "2"], ",", 
     SubscriptBox["p", "3"]}], "}"}]}], "]"}]], "Input",
 CellTags->"TIDL",
 CellLabel->"In[1733]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      SubsuperscriptBox["p", 
       FormBox["3",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{
          FormBox["q",
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["p", "3"],
           TraditionalForm]}], " ", 
         SuperscriptBox[
          FormBox[
           RowBox[{
            FormBox[
             SubscriptBox["p", "1"],
             TraditionalForm], "\[CenterDot]", 
            FormBox[
             SubscriptBox["p", "2"],
             TraditionalForm]}],
           TraditionalForm], "2"]}], "-", 
        RowBox[{
         RowBox[{
          FormBox["q",
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["p", "2"],
           TraditionalForm]}], " ", 
         RowBox[{
          FormBox[
           SubscriptBox["p", "1"],
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["p", "3"],
           TraditionalForm]}], " ", 
         RowBox[{
          FormBox[
           SubscriptBox["p", "1"],
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["p", "2"],
           TraditionalForm]}]}], "-", 
        RowBox[{
         RowBox[{
          FormBox["q",
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["p", "1"],
           TraditionalForm]}], " ", 
         RowBox[{
          FormBox[
           SubscriptBox["p", "2"],
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["p", "3"],
           TraditionalForm]}], " ", 
         RowBox[{
          FormBox[
           SubscriptBox["p", "1"],
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["p", "2"],
           TraditionalForm]}]}], "-", 
        RowBox[{
         RowBox[{
          FormBox["q",
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["p", "3"],
           TraditionalForm]}], " ", 
         SubsuperscriptBox["p", "1", "2"], " ", 
         SubsuperscriptBox["p", "2", "2"]}], "+", 
        RowBox[{
         RowBox[{
          FormBox["q",
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["p", "1"],
           TraditionalForm]}], " ", 
         RowBox[{
          FormBox[
           SubscriptBox["p", "1"],
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["p", "3"],
           TraditionalForm]}], " ", 
         SubsuperscriptBox["p", "2", "2"]}], "+", 
        RowBox[{
         RowBox[{
          FormBox["q",
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["p", "2"],
           TraditionalForm]}], " ", 
         SubsuperscriptBox["p", "1", "2"], " ", 
         RowBox[{
          FormBox[
           SubscriptBox["p", "2"],
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["p", "3"],
           TraditionalForm]}]}]}], ")"}]}], ")"}], "/", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SubsuperscriptBox["p", "3", "2"], " ", 
       SuperscriptBox[
        FormBox[
         RowBox[{
          FormBox[
           SubscriptBox["p", "1"],
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["p", "2"],
           TraditionalForm]}],
         TraditionalForm], "2"]}], "-", 
      RowBox[{"2", " ", 
       RowBox[{
        FormBox[
         SubscriptBox["p", "1"],
         TraditionalForm], "\[CenterDot]", 
        FormBox[
         SubscriptBox["p", "3"],
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox[
         SubscriptBox["p", "2"],
         TraditionalForm], "\[CenterDot]", 
        FormBox[
         SubscriptBox["p", "3"],
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox[
         SubscriptBox["p", "1"],
         TraditionalForm], "\[CenterDot]", 
        FormBox[
         SubscriptBox["p", "2"],
         TraditionalForm]}]}], "+", 
      RowBox[{
       SubsuperscriptBox["p", "1", "2"], " ", 
       SuperscriptBox[
        FormBox[
         RowBox[{
          FormBox[
           SubscriptBox["p", "2"],
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["p", "3"],
           TraditionalForm]}],
         TraditionalForm], "2"]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox[
         RowBox[{
          FormBox[
           SubscriptBox["p", "1"],
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["p", "3"],
           TraditionalForm]}],
         TraditionalForm], "2"], " ", 
       SubsuperscriptBox["p", "2", "2"]}], "-", 
      RowBox[{
       SubsuperscriptBox["p", "1", "2"], " ", 
       SubsuperscriptBox["p", "2", "2"], " ", 
       SubsuperscriptBox["p", "3", "2"]}]}], ")"}]}], "-", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      SubsuperscriptBox["p", 
       FormBox["2",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{"-", 
          RowBox[{"(", 
           RowBox[{
            FormBox["q",
             TraditionalForm], "\[CenterDot]", 
            FormBox[
             SubscriptBox["p", "2"],
             TraditionalForm]}], ")"}]}], " ", 
         SuperscriptBox[
          FormBox[
           RowBox[{
            FormBox[
             SubscriptBox["p", "1"],
             TraditionalForm], "\[CenterDot]", 
            FormBox[
             SubscriptBox["p", "3"],
             TraditionalForm]}],
           TraditionalForm], "2"]}], "+", 
        RowBox[{
         RowBox[{
          FormBox["q",
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["p", "3"],
           TraditionalForm]}], " ", 
         RowBox[{
          FormBox[
           SubscriptBox["p", "1"],
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["p", "2"],
           TraditionalForm]}], " ", 
         RowBox[{
          FormBox[
           SubscriptBox["p", "1"],
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["p", "3"],
           TraditionalForm]}]}], "+", 
        RowBox[{
         RowBox[{
          FormBox["q",
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["p", "1"],
           TraditionalForm]}], " ", 
         RowBox[{
          FormBox[
           SubscriptBox["p", "2"],
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["p", "3"],
           TraditionalForm]}], " ", 
         RowBox[{
          FormBox[
           SubscriptBox["p", "1"],
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["p", "3"],
           TraditionalForm]}]}], "-", 
        RowBox[{
         RowBox[{
          FormBox["q",
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["p", "3"],
           TraditionalForm]}], " ", 
         SubsuperscriptBox["p", "1", "2"], " ", 
         RowBox[{
          FormBox[
           SubscriptBox["p", "2"],
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["p", "3"],
           TraditionalForm]}]}], "+", 
        RowBox[{
         RowBox[{
          FormBox["q",
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["p", "2"],
           TraditionalForm]}], " ", 
         SubsuperscriptBox["p", "1", "2"], " ", 
         SubsuperscriptBox["p", "3", "2"]}], "-", 
        RowBox[{
         RowBox[{
          FormBox["q",
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["p", "1"],
           TraditionalForm]}], " ", 
         RowBox[{
          FormBox[
           SubscriptBox["p", "1"],
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["p", "2"],
           TraditionalForm]}], " ", 
         SubsuperscriptBox["p", "3", "2"]}]}], ")"}]}], ")"}], "/", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SubsuperscriptBox["p", "3", "2"], " ", 
       SuperscriptBox[
        FormBox[
         RowBox[{
          FormBox[
           SubscriptBox["p", "1"],
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["p", "2"],
           TraditionalForm]}],
         TraditionalForm], "2"]}], "-", 
      RowBox[{"2", " ", 
       RowBox[{
        FormBox[
         SubscriptBox["p", "1"],
         TraditionalForm], "\[CenterDot]", 
        FormBox[
         SubscriptBox["p", "3"],
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox[
         SubscriptBox["p", "2"],
         TraditionalForm], "\[CenterDot]", 
        FormBox[
         SubscriptBox["p", "3"],
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox[
         SubscriptBox["p", "1"],
         TraditionalForm], "\[CenterDot]", 
        FormBox[
         SubscriptBox["p", "2"],
         TraditionalForm]}]}], "+", 
      RowBox[{
       SubsuperscriptBox["p", "1", "2"], " ", 
       SuperscriptBox[
        FormBox[
         RowBox[{
          FormBox[
           SubscriptBox["p", "2"],
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["p", "3"],
           TraditionalForm]}],
         TraditionalForm], "2"]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox[
         RowBox[{
          FormBox[
           SubscriptBox["p", "1"],
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["p", "3"],
           TraditionalForm]}],
         TraditionalForm], "2"], " ", 
       SubsuperscriptBox["p", "2", "2"]}], "-", 
      RowBox[{
       SubsuperscriptBox["p", "1", "2"], " ", 
       SubsuperscriptBox["p", "2", "2"], " ", 
       SubsuperscriptBox["p", "3", "2"]}]}], ")"}]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      SubsuperscriptBox["p", 
       FormBox["1",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{
          FormBox["q",
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["p", "1"],
           TraditionalForm]}], " ", 
         SuperscriptBox[
          FormBox[
           RowBox[{
            FormBox[
             SubscriptBox["p", "2"],
             TraditionalForm], "\[CenterDot]", 
            FormBox[
             SubscriptBox["p", "3"],
             TraditionalForm]}],
           TraditionalForm], "2"]}], "-", 
        RowBox[{
         RowBox[{
          FormBox["q",
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["p", "3"],
           TraditionalForm]}], " ", 
         RowBox[{
          FormBox[
           SubscriptBox["p", "1"],
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["p", "2"],
           TraditionalForm]}], " ", 
         RowBox[{
          FormBox[
           SubscriptBox["p", "2"],
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["p", "3"],
           TraditionalForm]}]}], "-", 
        RowBox[{
         RowBox[{
          FormBox["q",
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["p", "2"],
           TraditionalForm]}], " ", 
         RowBox[{
          FormBox[
           SubscriptBox["p", "1"],
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["p", "3"],
           TraditionalForm]}], " ", 
         RowBox[{
          FormBox[
           SubscriptBox["p", "2"],
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["p", "3"],
           TraditionalForm]}]}], "+", 
        RowBox[{
         RowBox[{
          FormBox["q",
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["p", "3"],
           TraditionalForm]}], " ", 
         RowBox[{
          FormBox[
           SubscriptBox["p", "1"],
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["p", "3"],
           TraditionalForm]}], " ", 
         SubsuperscriptBox["p", "2", "2"]}], "+", 
        RowBox[{
         RowBox[{
          FormBox["q",
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["p", "2"],
           TraditionalForm]}], " ", 
         RowBox[{
          FormBox[
           SubscriptBox["p", "1"],
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["p", "2"],
           TraditionalForm]}], " ", 
         SubsuperscriptBox["p", "3", "2"]}], "-", 
        RowBox[{
         RowBox[{
          FormBox["q",
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["p", "1"],
           TraditionalForm]}], " ", 
         SubsuperscriptBox["p", "2", "2"], " ", 
         SubsuperscriptBox["p", "3", "2"]}]}], ")"}]}], ")"}], "/", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SubsuperscriptBox["p", "3", "2"], " ", 
       SuperscriptBox[
        FormBox[
         RowBox[{
          FormBox[
           SubscriptBox["p", "1"],
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["p", "2"],
           TraditionalForm]}],
         TraditionalForm], "2"]}], "-", 
      RowBox[{"2", " ", 
       RowBox[{
        FormBox[
         SubscriptBox["p", "1"],
         TraditionalForm], "\[CenterDot]", 
        FormBox[
         SubscriptBox["p", "3"],
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox[
         SubscriptBox["p", "2"],
         TraditionalForm], "\[CenterDot]", 
        FormBox[
         SubscriptBox["p", "3"],
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox[
         SubscriptBox["p", "1"],
         TraditionalForm], "\[CenterDot]", 
        FormBox[
         SubscriptBox["p", "2"],
         TraditionalForm]}]}], "+", 
      RowBox[{
       SubsuperscriptBox["p", "1", "2"], " ", 
       SuperscriptBox[
        FormBox[
         RowBox[{
          FormBox[
           SubscriptBox["p", "2"],
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["p", "3"],
           TraditionalForm]}],
         TraditionalForm], "2"]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox[
         RowBox[{
          FormBox[
           SubscriptBox["p", "1"],
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["p", "3"],
           TraditionalForm]}],
         TraditionalForm], "2"], " ", 
       SubsuperscriptBox["p", "2", "2"]}], "-", 
      RowBox[{
       SubsuperscriptBox["p", "1", "2"], " ", 
       SubsuperscriptBox["p", "2", "2"], " ", 
       SubsuperscriptBox["p", "3", "2"]}]}], ")"}]}]}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970439741548*^9},
 CellTags->"TIDL",
 CellLabel->"Out[1733]="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["D", "\[Mu]\[Nu]"], "-", "type"}], 
  TraditionalForm]]]], "Subsubsection",
 CellTags->"TIDL"],

Cell[BoxData[
 RowBox[{
  RowBox[{"TIDL", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"q", ",", "\[Mu]"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"q", ",", "\[Nu]"}], "}"}]}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"p", ",", "k", ",", "r"}], "}"}]}], "]"}], ";"}]], "Input",
 CellTags->"TIDL",
 CellLabel->"In[1734]:="]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["D", "\[Mu]\[Nu]\[Rho]"], "-", "type"}], 
  TraditionalForm]]]], "Subsubsection",
 CellTags->"TIDL"],

Cell[BoxData[
 RowBox[{
  RowBox[{"TIDL", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"q", ",", "\[Mu]"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"q", ",", "\[Nu]"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"q", ",", "\[Rho]"}], "}"}]}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"p", ",", "k", ",", "r"}], "}"}]}], "]"}], ";"}]], "Input",
 CellTags->"TIDL",
 CellLabel->"In[1735]:="]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["D", "\[Mu]\[Nu]\[Rho]\[Sigma]"], "-", "type"}], 
  TraditionalForm]]]], "Subsubsection",
 CellTags->"TIDL"],

Cell[BoxData[
 RowBox[{
  RowBox[{"TIDL", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"q", ",", "\[Mu]"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"q", ",", "\[Nu]"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"q", ",", "\[Rho]"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"q", ",", "\[Sigma]"}], "}"}]}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"p", ",", "k", ",", "r", ",", "s"}], "}"}]}], "]"}], 
  ";"}]], "Input",
 CellTags->"TIDL",
 CellLabel->"In[1736]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"Length", "[", "%", "]"}], ",", " ", 
   RowBox[{"LeafCount", "[", "%", "]"}], ",", 
   RowBox[{"ByteCount", "[", "%", "]"}]}], "}"}]], "Input",
 CellTags->"TIDL",
 CellLabel->"In[1737]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"4", ",", "29", ",", "744"}], "}"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009704400985413`*^9},
 CellTags->"TIDL",
 CellLabel->"Out[1737]="]
}, Open  ]]
}, Open  ]],

Cell["TimedIntegrate", "ObjectName",
 CellTags->"TimedIntegrate"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"TimedIntegrate"],

Cell["\<\
 TimedIntegrate[exp, vars] is like Integrate, but stops after the number of \
seconds specified by the option Timing. Options of Integrate can be given and \
are passed on.\
\>", "Text",
 CellTags->"TimedIntegrate"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "TimedIntegrate", "]"}]], "Input",
 CellTags->"TimedIntegrate",
 CellLabel->"In[1738]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Timing", "\[Rule]", "10"}], ",", 
    RowBox[{"Assumptions", "\[Rule]", 
     RowBox[{
      TagBox["\[CurlyEpsilon]",
       TraditionalForm], ">", "0"}]}], ",", 
    RowBox[{"Integrate", "\[Rule]", "Integrate"}], ",", 
    RowBox[{"Expand", "\[Rule]", "True"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970440223956*^9},
 CellTags->"TimedIntegrate",
 CellLabel->"Out[1738]="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"TimedIntegrate"],

Cell["This should reach to be done", "Text",
 CellTags->"TimedIntegrate"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TimedIntegrate", "[", 
  RowBox[{
   RowBox[{"Log", "[", 
    RowBox[{"x", "^", "5"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}], ",", 
   RowBox[{"Timing", "\[Rule]", "1"}]}], "]"}]], "Input",
 CellTags->"TimedIntegrate",
 CellLabel->"In[1739]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", "5"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970441017747*^9},
 CellTags->"TimedIntegrate",
 CellLabel->"Out[1739]="]
}, Open  ]],

Cell["This shouldn't", "Text",
 CellTags->"TimedIntegrate"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TimedIntegrate", "[", 
  RowBox[{
   RowBox[{"Log", "[", 
    RowBox[{"Cos", "[", 
     RowBox[{"x", "^", "5"}], "]"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}], ",", 
   RowBox[{"Timing", "\[Rule]", "10"}], ",", 
   RowBox[{"Integrate", "\[Rule]", "int"}]}], "]"}]], "Input",
 CellTags->"TimedIntegrate",
 CellLabel->"In[1740]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"int", "(", 
   RowBox[{
    RowBox[{"log", "(", 
     RowBox[{"cos", "(", 
      SuperscriptBox["x", "5"], ")"}], ")"}], ",", 
    RowBox[{"{", 
     RowBox[{"x", ",", "0", ",", "1"}], "}"}], ",", 
    RowBox[{"Assumptions", "\[Rule]", 
     RowBox[{
      TagBox["\[CurlyEpsilon]",
       TraditionalForm], ">", "0"}]}]}], ")"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970441256834*^9},
 CellTags->"TimedIntegrate",
 CellLabel->"Out[1740]="]
}, Open  ]]
}, Open  ]],

Cell["TLI  ***unfinished***", "ObjectName",
 CellTags->"TLI"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"TLI"],

Cell["\<\
TLI[{v,w,x,y,z},{a,b,c,d,e}, {al,be,ga,de,ep}].  The exponents of the \
numerator scalar product are (dl = OPEDelta):  v: k1.k1, w: k2.k2,  x: p.k1, \
y: p.k2, z: k1.k2.  a: dl.k1, b: dl.k2,  c: dl.(p-k1), d: dl.(p-k2), e: \
dl.(k1-k2).
TLI[{mu1, ...}, {nu1, ...}][{v,w,x,y,z},{a,b,c,d,e}, {al,be,ga,de,ep}] is a \
tensor integral, where mu1 belongs to k1 and nu1 to k2.
TLI[any___,{a,b,c,d,e,0,0}, {al,be,ga,de,ep}] simplifies to  TLI[any, \
{a,b,c,d,e}, {al,be,ga,de,ep}].
TLI[{0,0,0,0,0},{a,b,c,d,e}, {al,be,ga,de,ep}] simplifies to  \
TLI[{a,b,c,d,e}, {al,be,ga,de,ep}].\
\>", "Text",
 CellTags->"TLI"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "TLI", "]"}]], "Input",
 CellTags->"TLI",
 CellLabel->"In[1741]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"EpsilonOrder", "\[Rule]", "0"}], ",", 
    RowBox[{"Momentum", "\[Rule]", "p"}]}], "}"}], TraditionalForm]], "Output",\

 CellChangeTimes->{3.5009704418818274`*^9},
 CellTags->"TLI",
 CellLabel->"Out[1741]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["TLI2FC",
  BaseStyle->"AddOnsLink",
  ButtonData:>"TLI2FC",
  ButtonNote->"TLI2FC"],
 "."
}], "Text",
 CellTags->"TLI"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"TLI"],

Cell[BoxData[""], "Input",
 CellTags->"TLI",
 CellLabel->"In[1742]:="]
}, Open  ]],

Cell["TLIFP  ***unfinished***", "ObjectName",
 CellTags->"TLIFP"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"TLIFP"],

Cell["TLIFP[exp] does Feynman-Parametrizations of TLI's in exp.", "Text",
 CellTags->"TLIFP"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "TLIFP", "]"}]], "Input",
 CellTags->"TLIFP",
 CellLabel->"In[1743]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"FeynmanParameterNames", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{"x", ",", "t", ",", "u", ",", "s", ",", "y"}], "}"}]}], ",", 
    RowBox[{"GammaExpand", "\[Rule]", "True"}], ",", 
    RowBox[{"Momentum", "\[Rule]", "p"}], ",", 
    RowBox[{"Print", "\[Rule]", "True"}]}], "}"}], TraditionalForm]], "Output",\

 CellChangeTimes->{3.500970442075193*^9},
 CellTags->"TLIFP",
 CellLabel->"Out[1743]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["TLI",
  BaseStyle->"AddOnsLink",
  ButtonData:>"TLI",
  ButtonNote->"TLI"],
 "."
}], "Text",
 CellTags->"TLIFP"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"TLIFP"],

Cell[BoxData[""], "Input",
 CellTags->"TLIFP",
 CellLabel->"In[1744]:="]
}, Open  ]],

Cell["TLIHYP  ***unfinished***", "ObjectName",
 CellTags->"TLIHYP"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"TLIHYP"],

Cell["\<\
TLIHYP[exp] expresses TLI's in exp. in terms of hypergeometric functions, \
where possible.\
\>", "Text",
 CellTags->"TLIHYP"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "TLIHYP", "]"}]], "Input",
 CellTags->"TLIHYP",
 CellLabel->"In[1745]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"FeynmanParameterNames", "\[Rule]", "x"}], ",", 
    RowBox[{"Momentum", "\[Rule]", "p"}]}], "}"}], TraditionalForm]], "Output",\

 CellChangeTimes->{3.500970442210848*^9},
 CellTags->"TLIHYP",
 CellLabel->"Out[1745]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["TLI",
  BaseStyle->"AddOnsLink",
  ButtonData:>"TLI",
  ButtonNote->"TLI"],
 "."
}], "Text",
 CellTags->"TLIHYP"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"TLIHYP"],

Cell[BoxData[""], "Input",
 CellTags->"TLIHYP",
 CellLabel->"In[1746]:="]
}, Open  ]],

Cell["TLI2", "ObjectName",
 CellTags->"TLI2"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"TLI2"],

Cell["like TLI, but without any functional properties.", "Text",
 CellTags->"TLI2"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["TLI",
  BaseStyle->"AddOnsLink",
  ButtonData:>"TLI",
  ButtonNote->"TLI"],
 "."
}], "Text",
 CellTags->"TLI2"]
}, Open  ]],

Cell["TLI2FC  ***unfinished***", "ObjectName",
 CellTags->"TLI2FC"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"TLI2FC"],

Cell["\<\
TLI2FC[exp] transforms all TLI-integrals in exp to the FAD form.\
\>", "Text",
 CellTags->"TLI2FC"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "TLI2FC", "]"}]], "Input",
 CellTags->"TLI2FC",
 CellLabel->"In[1747]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
    RowBox[{"FeynCalcExternal", "\[Rule]", "False"}], ",", 
    RowBox[{"Momentum", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{"q1", ",", "q2", ",", "p"}], "}"}]}]}], "}"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970442724823*^9},
 CellTags->"TLI2FC",
 CellLabel->"Out[1747]="]
}, Open  ]],

Cell[TextData[{
 "See also:  ",
 ButtonBox["TLI",
  BaseStyle->"AddOnsLink",
  ButtonData:>"TLI",
  ButtonNote->"TLI"],
 ".  ",
 ButtonBox["FC2TLI",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FC2TLI",
  ButtonNote->"FC2TLI"],
 "."
}], "Text",
 CellTags->"TLI2FC"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"TLI2FC"],

Cell[BoxData[""], "Input",
 CellTags->"TLI2FC",
 CellLabel->"In[1748]:="]
}, Open  ]],

Cell["ToDistribution", "ObjectName",
 CellTags->"ToDistribution"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"ToDistribution"],

Cell["\<\
ToDistribution[exp,x] replaces (1-x)^(a Epsilon - 1) in exp by 1/(a Epsilon) \
DeltaFunction[1-x] + 1/(1-x) + a Epsilon Log[1-x]/(1-x) + 1/2 a^2 Epsilon^2 \
Log[1-x]^2/(1-x)] and (1-x)^(a Epsilon - 2) in exp by -1/(a Epsilon) \
DeltaFunctionPrime[1-x] + 1/(1-x)^2 + (a Epsilon) Log[1-x]/(1-x)^2 + a^2 \
Epsilon^2/2 Log[1-x]^2/(1-x)^2 + a^3 Epsilon^3/6 Log[1-x]^3/(1-x)^2.\
\>", "Text",
 CellTags->"ToDistribution"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "ToDistribution", "]"}]], "Input",
 CellTags->"ToDistribution",
 CellLabel->"In[1749]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"PlusDistribution", "\[Rule]", "Identity"}], "}"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.5009704428206873`*^9},
 CellTags->"ToDistribution",
 CellLabel->"Out[1749]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["PlusDistribution",
  BaseStyle->"AddOnsLink",
  ButtonData:>"PlusDistribution",
  ButtonNote->"PlusDistribution"],
 "."
}], "Text",
 CellTags->"ToDistribution"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"ToDistribution"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SetOptions", "[", 
  RowBox[{"ToDistribution", ",", 
   RowBox[{"PlusDistribution", "\[Rule]", "PlusDistribution"}]}], 
  "]"}]], "Input",
 CellTags->"ToDistribution",
 CellLabel->"In[1750]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"PlusDistribution", "\[Rule]", "PlusDistribution"}], "}"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970442867647*^9},
 CellTags->"ToDistribution",
 CellLabel->"Out[1750]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ToDistribution", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{"1", "-", "x"}], ")"}], "^", 
    RowBox[{"(", 
     RowBox[{"Epsilon", "-", "1"}], ")"}]}], ",", "x"}], "]"}]], "Input",
 CellTags->"ToDistribution",
 CellLabel->"In[1751]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{"\[Delta]", "(", 
     FormBox[
      RowBox[{"1", "-", "x"}],
      TraditionalForm], ")"}], 
    TagBox["\[CurlyEpsilon]",
     TraditionalForm]], "+", 
   RowBox[{
    FractionBox["1", "6"], " ", 
    SuperscriptBox[
     TagBox["\[CurlyEpsilon]",
      TraditionalForm], "3"], " ", 
    SubscriptBox[
     RowBox[{"(", 
      FractionBox[
       RowBox[{
        SuperscriptBox["log", "3"], "(", 
        RowBox[{"1", "-", "x"}], ")"}], 
       RowBox[{"1", "-", "x"}]], ")"}], "+"]}], "+", 
   RowBox[{
    FractionBox["1", "2"], " ", 
    SuperscriptBox[
     TagBox["\[CurlyEpsilon]",
      TraditionalForm], "2"], " ", 
    SubscriptBox[
     RowBox[{"(", 
      FractionBox[
       RowBox[{
        SuperscriptBox["log", "2"], "(", 
        RowBox[{"1", "-", "x"}], ")"}], 
       RowBox[{"1", "-", "x"}]], ")"}], "+"]}], "+", 
   RowBox[{
    TagBox["\[CurlyEpsilon]",
     TraditionalForm], " ", 
    SubscriptBox[
     RowBox[{"(", 
      FractionBox[
       RowBox[{"log", "(", 
        RowBox[{"1", "-", "x"}], ")"}], 
       RowBox[{"1", "-", "x"}]], ")"}], "+"]}], "+", 
   SubscriptBox[
    RowBox[{"(", 
     FractionBox["1", 
      RowBox[{"1", "-", "x"}]], ")"}], "+"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009704429798613`*^9},
 CellTags->"ToDistribution",
 CellLabel->"Out[1751]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SetOptions", "[", 
  RowBox[{"ToDistribution", ",", 
   RowBox[{"PlusDistribution", "\[Rule]", "Identity"}]}], "]"}]], "Input",
 CellTags->"ToDistribution",
 CellLabel->"In[1752]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"PlusDistribution", "\[Rule]", "Identity"}], "}"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970443031952*^9},
 CellTags->"ToDistribution",
 CellLabel->"Out[1752]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ToDistribution", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{"1", "-", "x"}], ")"}], "^", 
    RowBox[{"(", 
     RowBox[{"Epsilon", "-", "2"}], ")"}]}], ",", "x"}], "]"}]], "Input",
 CellTags->"ToDistribution",
 CellLabel->"In[1753]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    FractionBox[
     RowBox[{
      SuperscriptBox["\[Delta]", "\[Prime]"], "(", 
      FormBox[
       RowBox[{"1", "-", "x"}],
       TraditionalForm], ")"}], 
     TagBox["\[CurlyEpsilon]",
      TraditionalForm]]}], "+", 
   FractionBox[
    RowBox[{
     SuperscriptBox[
      TagBox["\[CurlyEpsilon]",
       TraditionalForm], "3"], " ", 
     RowBox[{
      SuperscriptBox["log", "3"], "(", 
      RowBox[{"1", "-", "x"}], ")"}]}], 
    RowBox[{"6", " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"1", "-", "x"}], ")"}], "2"]}]], "+", 
   FractionBox[
    RowBox[{
     SuperscriptBox[
      TagBox["\[CurlyEpsilon]",
       TraditionalForm], "2"], " ", 
     RowBox[{
      SuperscriptBox["log", "2"], "(", 
      RowBox[{"1", "-", "x"}], ")"}]}], 
    RowBox[{"2", " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"1", "-", "x"}], ")"}], "2"]}]], "+", 
   FractionBox[
    RowBox[{
     TagBox["\[CurlyEpsilon]",
      TraditionalForm], " ", 
     RowBox[{"log", "(", 
      RowBox[{"1", "-", "x"}], ")"}]}], 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "-", "x"}], ")"}], "2"]], "+", 
   FractionBox["1", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "-", "x"}], ")"}], "2"]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970443136469*^9},
 CellTags->"ToDistribution",
 CellLabel->"Out[1753]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Series2", "[", 
  RowBox[{
   RowBox[{"Integrate", "[", 
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"1", "-", "x"}], ")"}], 
      RowBox[{"Epsilon", "-", "2"}]], ",", 
     RowBox[{"{", 
      RowBox[{"x", ",", "0", ",", "1"}], "}"}], ",", 
     RowBox[{"GenerateConditions", "\[Rule]", "False"}]}], "]"}], ",", 
   "Epsilon", ",", "3"}], "]"}]], "Input",
 CellTags->"ToDistribution",
 CellLabel->"In[1754]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    SuperscriptBox[
     TagBox["\[CurlyEpsilon]",
      TraditionalForm], "3"]}], "-", 
   SuperscriptBox[
    TagBox["\[CurlyEpsilon]",
     TraditionalForm], "2"], "-", 
   TagBox["\[CurlyEpsilon]",
    TraditionalForm], "-", "1"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970443597746*^9},
 CellTags->"ToDistribution",
 CellLabel->"Out[1754]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "@", "Integrate"}]], "Input",
 CellTags->"ToDistribution",
 CellLabel->"In[1755]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Assumptions", "\[RuleDelayed]", "$Assumptions"}], ",", 
    RowBox[{"GenerateConditions", "\[Rule]", "False"}], ",", 
    RowBox[{"PrincipalValue", "\[Rule]", "False"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.5009704441014643`*^9},
 CellTags->"ToDistribution",
 CellLabel->"Out[1755]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Integrate2", "[", 
  RowBox[{
   RowBox[{"ToDistribution", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{"1", "-", "x"}], ")"}], "^", 
      RowBox[{"(", 
       RowBox[{"Epsilon", "-", "2"}], ")"}]}], ",", "x"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellTags->"ToDistribution",
 CellLabel->"In[1756]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    SuperscriptBox[
     TagBox["\[CurlyEpsilon]",
      TraditionalForm], "3"]}], "-", 
   SuperscriptBox[
    TagBox["\[CurlyEpsilon]",
     TraditionalForm], "2"], "-", 
   TagBox["\[CurlyEpsilon]",
    TraditionalForm], "-", "1"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970444302704*^9},
 CellTags->"ToDistribution",
 CellLabel->"Out[1756]="]
}, Open  ]]
}, Open  ]],

Cell["ToHypergeometric", "ObjectName",
 CellTags->"ToHypergeometric"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"ToHypergeometric"],

Cell[TextData[{
 "ToHypergeometric[",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox[
     StyleBox["t",
      FontSlant->"Plain"], 
     StyleBox["b",
      FontSlant->"Plain"]], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "-", 
       StyleBox["t",
        FontSlant->"Plain"]}], ")"}], 
     StyleBox["c",
      FontSlant->"Plain"]], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "+", 
       RowBox[{
        StyleBox["t",
         FontSlant->"Plain"], " ", 
        StyleBox["z",
         FontSlant->"Plain"]}]}], ")"}], 
     StyleBox["a",
      FontSlant->"Plain"]]}], TraditionalForm]],
  FontVariations->{"CompatibilityType"->0}],
 ", ",
 "t",
 "] returns ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"\[CapitalGamma]", "(", 
     RowBox[{"b", "+", "1"}], ")"}], " ", 
    RowBox[{
     RowBox[{"\[CapitalGamma]", "(", 
      RowBox[{"c", "+", "1"}], ")"}], "/", 
     RowBox[{"\[CapitalGamma]", "(", 
      RowBox[{"b", "+", "c", "+", "2"}], ")"}]}], " ", 
    FormBox[
     RowBox[{
      SubscriptBox["\[Null]", "2"], 
      RowBox[{
       SubscriptBox["F", "1"], "(", 
       RowBox[{
        RowBox[{
         RowBox[{"-", "a"}], ",", 
         RowBox[{"b", "+", "1"}]}], ";", 
        RowBox[{"b", "+", "c", "+", "2"}], ";", 
        RowBox[{"-", "z"}]}], ")"}]}],
     TraditionalForm]}], TraditionalForm]]],
 ". Remember that Re ",
 StyleBox["b",
  FontSlant->"Italic"],
 " >0 and Re (",
 StyleBox["c",
  FontSlant->"Italic"],
 "-",
 StyleBox["b",
  FontSlant->"Italic"],
 ") > 0 should hold (need not be set in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ")."
}], "Text",
 CellTags->"ToHypergeometric"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["HypergeometricAC",
  BaseStyle->"AddOnsLink",
  ButtonData:>"HypergeometricAC",
  ButtonNote->"HypergeometricAC"],
 ", ",
 ButtonBox["HypergeometricIR",
  BaseStyle->"AddOnsLink",
  ButtonData:>"HypergeometricIR",
  ButtonNote->"HypergeometricIR"],
 ", ",
 ButtonBox["HypergeometricSE",
  BaseStyle->"AddOnsLink",
  ButtonData:>"HypergeometricSE",
  ButtonNote->"HypergeometricSE"],
 "."
}], "Text",
 CellTags->"ToHypergeometric"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"ToHypergeometric"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ToHypergeometric", "[", 
  RowBox[{
   RowBox[{
    SuperscriptBox["t", "b"], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "-", "t"}], ")"}], "c"], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "+", 
       RowBox[{"t", " ", "z"}]}], ")"}], "a"]}], ",", "t"}], "]"}]], "Input",
 CellTags->"ToHypergeometric",
 CellLabel->"In[1757]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{
    TemplateBox[{RowBox[{"b", "+", "1"}]},
     "Gamma"], " ", 
    TemplateBox[{RowBox[{"c", "+", "1"}]},
     "Gamma"], " ", 
    TemplateBox[{
     RowBox[{"-", "a"}],RowBox[{"b", "+", "1"}],
      RowBox[{"b", "+", "c", "+", "2"}],RowBox[{"-", "z"}]},
     "Hypergeometric2F1"]}], 
   TemplateBox[{RowBox[{"b", "+", "c", "+", "2"}]},
    "Gamma"]], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970444409523*^9},
 CellTags->"ToHypergeometric",
 CellLabel->"Out[1757]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ToHypergeometric", "[", 
  RowBox[{
   RowBox[{"w", " ", 
    SuperscriptBox["t", 
     RowBox[{"b", "-", "1"}]], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "-", "t"}], ")"}], 
     RowBox[{"c", "-", "b", "-", "1"}]], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "-", 
       RowBox[{"t", " ", "z"}]}], ")"}], 
     RowBox[{"-", "a"}]]}], ",", "t"}], "]"}]], "Input",
 CellTags->"ToHypergeometric",
 CellLabel->"In[1758]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{"w", " ", 
    TemplateBox[{"b"},
     "Gamma"], " ", 
    TemplateBox[{RowBox[{"c", "-", "b"}]},
     "Gamma"], " ", 
    TemplateBox[{"a","b","c","z"},
     "Hypergeometric2F1"]}], 
   TemplateBox[{"c"},
    "Gamma"]], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009704445305967`*^9},
 CellTags->"ToHypergeometric",
 CellLabel->"Out[1758]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ToHypergeometric", "[", 
  RowBox[{
   RowBox[{
    SuperscriptBox["t", "b"], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "-", "t"}], ")"}], "c"], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"u", "+", 
       RowBox[{"t", " ", "z"}]}], ")"}], "a"]}], ",", "t"}], "]"}]], "Input",
 CellTags->"ToHypergeometric",
 CellLabel->"In[1759]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{
    SuperscriptBox["u", "a"], " ", 
    TemplateBox[{RowBox[{"b", "+", "1"}]},
     "Gamma"], " ", 
    TemplateBox[{RowBox[{"c", "+", "1"}]},
     "Gamma"], " ", 
    TemplateBox[{
     RowBox[{"-", "a"}],RowBox[{"b", "+", "1"}],
      RowBox[{"b", "+", "c", "+", "2"}],RowBox[{"-", 
        FractionBox["z", "u"]}]},
     "Hypergeometric2F1"]}], 
   TemplateBox[{RowBox[{"b", "+", "c", "+", "2"}]},
    "Gamma"]], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009704446756983`*^9},
 CellTags->"ToHypergeometric",
 CellLabel->"Out[1759]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ToHypergeometric", "[", 
  RowBox[{
   RowBox[{"w", " ", 
    SuperscriptBox["t", 
     RowBox[{"b", "-", "1"}]], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "-", "t"}], ")"}], 
     RowBox[{"c", "-", "b", "-", "1"}]], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"u", "-", 
       RowBox[{"t", " ", "z"}]}], ")"}], 
     RowBox[{"-", "a"}]]}], ",", "t"}], "]"}]], "Input",
 CellTags->"ToHypergeometric",
 CellLabel->"In[1760]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{"w", " ", 
    SuperscriptBox["u", 
     RowBox[{"-", "a"}]], " ", 
    TemplateBox[{"b"},
     "Gamma"], " ", 
    TemplateBox[{RowBox[{"c", "-", "b"}]},
     "Gamma"], " ", 
    TemplateBox[{"a","b","c",FractionBox["z", "u"]},
     "Hypergeometric2F1"]}], 
   TemplateBox[{"c"},
    "Gamma"]], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970444932664*^9},
 CellTags->"ToHypergeometric",
 CellLabel->"Out[1760]="]
}, Open  ]]
}, Open  ]],

Cell["ToLarin", "ObjectName",
 CellTags->"ToLarin"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"ToLarin"],

Cell[TextData[{
 "ToLarin[exp] translates all ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["\[Gamma]", "\[Mu]"], " ", 
    SuperscriptBox["\[Gamma]", "5"]}], TraditionalForm]]],
 " in exp into ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"-", "\[ImaginaryI]"}], "/", "6"}], " ", 
    SuperscriptBox["\[CurlyEpsilon]", "\[Mu]\[Nu]\[Lambda]\[Sigma]"], 
    SuperscriptBox["\[Gamma]", "\[Nu]"], " ", 
    SuperscriptBox["\[Gamma]", "\[Lambda]"], " ", 
    RowBox[{
     SuperscriptBox["\[Gamma]", "\[Sigma]"], "."}]}], TraditionalForm]]]
}], "Text",
 CellTags->"ToLarin"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Eps",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Eps",
  ButtonNote->"Eps"],
 "."
}], "Text",
 CellTags->"ToLarin"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"ToLarin"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"DiracMatrix", "[", "\[Nu]", "]"}], ".", 
  RowBox[{"DiracMatrix", "[", "\[Mu]", "]"}], ".", 
  RowBox[{"DiracMatrix", "[", "5", "]"}]}]], "Input",
 CellTags->"ToLarin",
 CellLabel->"In[1761]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Nu]",
      TraditionalForm],
     TraditionalForm]], ".", 
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], ".", 
   SuperscriptBox["\[Gamma]", "5"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970445112441*^9},
 CellTags->"ToLarin",
 CellLabel->"Out[1761]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ToLarin", "[", "%", "]"}]], "Input",
 CellTags->"ToLarin",
 CellLabel->"In[1762]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    FractionBox["1", "6"]}], " ", "\[ImaginaryI]", " ", 
   RowBox[{
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm]], ".", 
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox["du204",
       TraditionalForm],
      TraditionalForm]], ".", 
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox["du205",
       TraditionalForm],
      TraditionalForm]], ".", 
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox["du206",
       TraditionalForm],
      TraditionalForm]]}], " ", 
   SuperscriptBox["\[Epsilon]", 
    RowBox[{
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["du204",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["du205",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["du206",
       TraditionalForm],
      TraditionalForm]}]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009704453202553`*^9},
 CellTags->"ToLarin",
 CellLabel->"Out[1762]="]
}, Open  ]]
}, Open  ]],

Cell["ToTFi", "ObjectName",
 CellTags->"ToTFi"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"ToTFi"],

Cell["\<\
ToTFi[expr, q1, q2, p] translates FeynCalc 2-loop self energy type integrals \
into the TFi notatation, which can be used as input for the function \
TarcerRecurse from the TARCER package. See TFi for details on the conventions.\
\>", "Text",
 CellTags->"ToTFi"],

Cell[TextData[{
 "See also: ",
 ButtonBox["TFi",
  BaseStyle->"AddOnsLink",
  ButtonData:>"TFi",
  ButtonNote->"TFi"],
 ", ",
 ButtonBox["FromTFi",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FromTFi",
  ButtonNote->"FromTFi"],
 "."
}], "Text",
 CellTags->"ToTFi"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"ToTFi"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FAD", "[", 
  RowBox[{"q1", ",", 
   RowBox[{"q1", "-", "p"}], ",", 
   RowBox[{"{", 
    RowBox[{"q2", ",", "M"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"q2", "-", "p"}], ",", "m"}], "}"}], ",", 
   RowBox[{"q1", "-", "q2"}]}], "]"}]], "Input",
 CellTags->"ToTFi",
 CellLabel->"In[1763]:="],

Cell[BoxData[
 FormBox[
  FractionBox["1", 
   RowBox[{
    RowBox[{"(", 
     InterpretationBox[
      RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
       SuperscriptBox["q1", "2"], "\[InvisibleSpace]", "\<\"]\"\>"}],
      SequenceForm["[", $CellContext`q1^2, "]"],
      Editable->False], ")"}], " ", 
    RowBox[{"(", 
     InterpretationBox[
      RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"q1", "-", "p"}], ")"}], "2"], 
       "\[InvisibleSpace]", "\<\"]\"\>"}],
      SequenceForm["[", (-$CellContext`p + $CellContext`q1)^2, "]"],
      Editable->False], ")"}], " ", 
    RowBox[{"(", 
     InterpretationBox[
      RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"q1", "-", "q2"}], ")"}], "2"], 
       "\[InvisibleSpace]", "\<\"]\"\>"}],
      SequenceForm["[", ($CellContext`q1 - $CellContext`q2)^2, "]"],
      Editable->False], ")"}], " ", 
    RowBox[{"(", 
     InterpretationBox[
      RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
       SuperscriptBox["q2", "2"], "\[InvisibleSpace]", "\<\"-\"\>", 
       "\[InvisibleSpace]", 
       SuperscriptBox["M", "2"], "\[InvisibleSpace]", "\<\"]\"\>"}],
      SequenceForm["[", $CellContext`q2^2, "-", $CellContext`M^2, "]"],
      Editable->False], ")"}], " ", 
    RowBox[{"(", 
     InterpretationBox[
      RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"q2", "-", "p"}], ")"}], "2"], 
       "\[InvisibleSpace]", "\<\"-\"\>", "\[InvisibleSpace]", 
       SuperscriptBox["m", "2"], "\[InvisibleSpace]", "\<\"]\"\>"}],
      SequenceForm[
      "[", (-$CellContext`p + $CellContext`q2)^2, "-", $CellContext`m^2, "]"],
      
      Editable->False], ")"}]}]], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970445413986*^9},
 CellTags->"ToTFi",
 CellLabel->"Out[1763]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ToTFi", "[", 
  RowBox[{"%", ",", "q1", ",", "q2", ",", "p"}], "]"}]], "Input",
 CellTags->"ToTFi",
 CellLabel->"In[1764]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"ToTFi", "(", 
   RowBox[{
    FormBox[
     FractionBox["1", 
      RowBox[{
       RowBox[{"(", 
        InterpretationBox[
         RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
          SuperscriptBox["q1", "2"], "\[InvisibleSpace]", "\<\"]\"\>"}],
         SequenceForm["[", $CellContext`q1^2, "]"],
         Editable->False], ")"}], " ", 
       RowBox[{"(", 
        InterpretationBox[
         RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{"q1", "-", "p"}], ")"}], "2"], 
          "\[InvisibleSpace]", "\<\"]\"\>"}],
         SequenceForm["[", (-$CellContext`p + $CellContext`q1)^2, "]"],
         Editable->False], ")"}], " ", 
       RowBox[{"(", 
        InterpretationBox[
         RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{"q1", "-", "q2"}], ")"}], "2"], 
          "\[InvisibleSpace]", "\<\"]\"\>"}],
         SequenceForm["[", ($CellContext`q1 - $CellContext`q2)^2, "]"],
         Editable->False], ")"}], " ", 
       RowBox[{"(", 
        InterpretationBox[
         RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
          SuperscriptBox["q2", "2"], "\[InvisibleSpace]", "\<\"-\"\>", 
          "\[InvisibleSpace]", 
          SuperscriptBox["M", "2"], "\[InvisibleSpace]", "\<\"]\"\>"}],
         SequenceForm["[", $CellContext`q2^2, "-", $CellContext`M^2, "]"],
         Editable->False], ")"}], " ", 
       RowBox[{"(", 
        InterpretationBox[
         RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{"q2", "-", "p"}], ")"}], "2"], 
          "\[InvisibleSpace]", "\<\"-\"\>", "\[InvisibleSpace]", 
          SuperscriptBox["m", "2"], "\[InvisibleSpace]", "\<\"]\"\>"}],
         SequenceForm[
         "[", (-$CellContext`p + $CellContext`q2)^2, "-", $CellContext`m^2, 
          "]"],
         Editable->False], ")"}]}]],
     TraditionalForm], ",", "q1", ",", "q2", ",", "p"}], ")"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970445507415*^9},
 CellTags->"ToTFi",
 CellLabel->"Out[1764]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "StandardForm"}]], "Input",
 CellTags->"ToTFi",
 CellLabel->"In[1765]:="],

Cell[BoxData[
 RowBox[{"ToTFi", "[", 
  RowBox[{
   RowBox[{"FAD", "[", 
    RowBox[{"q1", ",", 
     RowBox[{
      RowBox[{"-", "p"}], "+", "q1"}], ",", 
     RowBox[{"{", 
      RowBox[{"q2", ",", "M"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{
        RowBox[{"-", "p"}], "+", "q2"}], ",", "m"}], "}"}], ",", 
     RowBox[{"q1", "-", "q2"}]}], "]"}], ",", "q1", ",", "q2", ",", "p"}], 
  "]"}]], "Output",
 CellChangeTimes->{3.5009704455972757`*^9},
 CellTags->"ToTFi",
 CellLabel->"Out[1765]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"SOD", "[", "q1", "]"}], " ", 
   RowBox[{"SOD", "[", "q2", "]"}], 
   RowBox[{"FAD", "[", 
    RowBox[{"q1", ",", 
     RowBox[{"q1", "-", "p"}], ",", 
     RowBox[{"{", 
      RowBox[{"q2", ",", "M"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"q2", "-", "p"}], ",", "m"}], "}"}], ",", 
     RowBox[{"q1", "-", "q2"}]}], "]"}]}], "//", "FCI"}]], "Input",
 CellTags->"ToTFi",
 CellLabel->"In[1766]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{
    RowBox[{
     FormBox["\[CapitalDelta]",
      TraditionalForm], "\[CenterDot]", 
     FormBox["q1",
      TraditionalForm]}], " ", 
    RowBox[{
     FormBox["\[CapitalDelta]",
      TraditionalForm], "\[CenterDot]", 
     FormBox["q2",
      TraditionalForm]}]}], 
   RowBox[{
    SuperscriptBox["q1", "2"], ".", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"q1", "-", "p"}], ")"}], "2"], ".", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"q1", "-", "q2"}], ")"}], "2"], ".", 
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox["q2", "2"], "-", 
      SuperscriptBox["M", "2"]}], ")"}], ".", 
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"q2", "-", "p"}], ")"}], "2"], "-", 
      SuperscriptBox["m", "2"]}], ")"}]}]], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970445748267*^9},
 CellTags->"ToTFi",
 CellLabel->"Out[1766]="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["TR", "ObjectName",
 CellTags->"TR"],

Cell[BoxData[
 FormBox["TR", TraditionalForm]], "Output",
 CellTags->"TR",
 CellLabel->"Out[1959]="]
}, Open  ]],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"TR"],

Cell[TextData[{
 "TR[exp] calculates the Dirac trace of exp. Depending on the setting of the \
option SUNTrace also a trace over SU(",
 StyleBox["N",
  FontSlant->"Italic"],
 ") objects is performed. The ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " build-in function Tr is overloaded to call TR if ",
 "any of DiracGamma, DiracSlash, GA, GAD, GS or GSD",
 " are in the expression."
}], "Text",
 CellTags->"TR"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "TR", "]"}]], "Input",
 CellTags->"TR",
 CellLabel->"In[1767]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"DiracTraceEvaluate", "\[Rule]", "True"}], ",", 
    RowBox[{"Explicit", "\[Rule]", "True"}], ",", 
    RowBox[{"Factoring", "\[Rule]", "False"}], ",", 
    RowBox[{"FeynCalcExternal", "\[Rule]", "False"}], ",", 
    RowBox[{"LeviCivitaSign", "\[RuleDelayed]", "$LeviCivitaSign"}], ",", 
    RowBox[{"Mandelstam", "\[Rule]", 
     RowBox[{"{", "}"}]}], ",", 
    RowBox[{"PairCollect", "\[Rule]", "False"}], ",", 
    RowBox[{"Schouten", "\[Rule]", "442"}], ",", 
    RowBox[{"SUNTrace", "\[Rule]", "False"}], ",", 
    RowBox[{"TraceOfOne", "\[Rule]", "4"}], ",", 
    RowBox[{"SUNNToCACF", "\[Rule]", "False"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970446627413*^9},
 CellTags->"TR",
 CellLabel->"Out[1767]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 ButtonBox["DiracSimplify",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracSimplify",
  ButtonNote->"DiracSimplify"],
 ", ",
 ButtonBox["DiracTrace",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracTrace",
  ButtonNote->"DiracTrace"],
 ", ",
 ButtonBox["FermionSpinSum",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FermionSpinSum",
  ButtonNote->"FermionSpinSum"],
 ", ",
 ButtonBox["SUNTrace",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SUNTrace",
  ButtonNote->"SUNTrace"],
 "."
}], "Text",
 CellTags->"TR"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"TR"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GA", "[", 
  RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}]], "Input",
 CellTags->"TR",
 CellLabel->"In[1768]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
   SuperscriptBox["\[Gamma]", "\[Nu]"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009704467105923`*^9},
 CellTags->"TR",
 CellLabel->"Out[1768]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TR", "[", "%", "]"}]], "Input",
 CellTags->"TR",
 CellLabel->"In[1769]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"4", " ", 
   SuperscriptBox["g", 
    RowBox[{"\[Mu]", "\[Nu]"}]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970446798711*^9},
 CellTags->"TR",
 CellLabel->"Out[1769]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TR", "[", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{"GSD", "[", "p", "]"}], "+", "m"}], ")"}], ".", 
   RowBox[{"GAD", "[", "\[Mu]", "]"}], ".", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"GSD", "[", "q", "]"}], "-", "m"}], ")"}], ".", 
   RowBox[{"GAD", "[", "\[Nu]", "]"}]}], "]"}]], "Input",
 CellTags->"TR",
 CellLabel->"In[1770]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"4", " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      SuperscriptBox["m", "2"], " ", 
      RowBox[{"(", 
       RowBox[{"-", 
        SuperscriptBox["g", 
         RowBox[{"\[Mu]", "\[Nu]"}]]}], ")"}]}], "-", 
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
      RowBox[{
       FormBox["p",
        TraditionalForm], "\[CenterDot]", 
       FormBox["q",
        TraditionalForm]}]}], "+", 
     RowBox[{
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox[
       FormBox["q",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]]}], "+", 
     RowBox[{
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox[
       FormBox["q",
        TraditionalForm], 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]]}]}], ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970447262014*^9},
 CellTags->"TR",
 CellLabel->"Out[1770]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TR", "[", 
  RowBox[{"GA", "[", 
   RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]", ",", "\[Sigma]", ",", "5"}], 
   "]"}], "]"}]], "Input",
 CellTags->"TR",
 CellLabel->"In[1771]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", "4"}], " ", "\[ImaginaryI]", " ", 
   SuperscriptBox["\[Epsilon]", 
    RowBox[{
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["\[Rho]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["\[Sigma]",
       TraditionalForm],
      TraditionalForm]}]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970447341941*^9},
 CellTags->"TR",
 CellLabel->"Out[1771]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TR", "[", 
  RowBox[{"GAD", "[", 
   RowBox[{
   "\[Mu]", ",", "\[Nu]", ",", "\[Rho]", ",", "\[Sigma]", ",", "\[Tau]", ",", 
    "\[Xi]", ",", "5"}], "]"}], "]"}]], "Input",
 CellTags->"TR",
 CellLabel->"In[1772]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"4", " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"\[ImaginaryI]", " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         FormBox["\[Xi]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Sigma]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Tau]",
          TraditionalForm],
         TraditionalForm]}]]}], "-", 
     RowBox[{"\[ImaginaryI]", " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Xi]"}]], " ", 
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Sigma]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Tau]",
          TraditionalForm],
         TraditionalForm]}]]}], "+", 
     RowBox[{"\[ImaginaryI]", " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Rho]"}]], " ", 
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Xi]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Sigma]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Tau]",
          TraditionalForm],
         TraditionalForm]}]]}], "-", 
     RowBox[{"\[ImaginaryI]", " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Sigma]"}]], " ", 
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Xi]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Tau]",
          TraditionalForm],
         TraditionalForm]}]]}], "+", 
     RowBox[{"\[ImaginaryI]", " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Tau]"}]], " ", 
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Xi]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Sigma]",
          TraditionalForm],
         TraditionalForm]}]]}], "+", 
     RowBox[{"\[ImaginaryI]", " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Nu]", "\[Xi]"}]], " ", 
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Sigma]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Tau]",
          TraditionalForm],
         TraditionalForm]}]]}], "-", 
     RowBox[{"\[ImaginaryI]", " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Nu]", "\[Rho]"}]], " ", 
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Xi]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Sigma]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Tau]",
          TraditionalForm],
         TraditionalForm]}]]}], "+", 
     RowBox[{"\[ImaginaryI]", " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Nu]", "\[Sigma]"}]], " ", 
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Xi]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Tau]",
          TraditionalForm],
         TraditionalForm]}]]}], "-", 
     RowBox[{"\[ImaginaryI]", " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Nu]", "\[Tau]"}]], " ", 
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Xi]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Sigma]",
          TraditionalForm],
         TraditionalForm]}]]}], "-", 
     RowBox[{"\[ImaginaryI]", " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Xi]", "\[Rho]"}]], " ", 
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Sigma]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Tau]",
          TraditionalForm],
         TraditionalForm]}]]}], "+", 
     RowBox[{"\[ImaginaryI]", " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Xi]", "\[Sigma]"}]], " ", 
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Tau]",
          TraditionalForm],
         TraditionalForm]}]]}], "-", 
     RowBox[{"\[ImaginaryI]", " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Xi]", "\[Tau]"}]], " ", 
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Sigma]",
          TraditionalForm],
         TraditionalForm]}]]}], "+", 
     RowBox[{"\[ImaginaryI]", " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Rho]", "\[Sigma]"}]], " ", 
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Xi]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Tau]",
          TraditionalForm],
         TraditionalForm]}]]}], "-", 
     RowBox[{"\[ImaginaryI]", " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Rho]", "\[Tau]"}]], " ", 
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Xi]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Sigma]",
          TraditionalForm],
         TraditionalForm]}]]}], "+", 
     RowBox[{"\[ImaginaryI]", " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Sigma]", "\[Tau]"}]], " ", 
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Xi]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm]}]]}]}], ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970447496655*^9},
 CellTags->"TR",
 CellLabel->"Out[1772]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData["$West"], "Input",
 CellTags->"TR",
 CellLabel->"In[1773]:="],

Cell[BoxData[
 FormBox["True", TraditionalForm]], "Output",
 CellChangeTimes->{3.5009704476212378`*^9},
 CellTags->"TR",
 CellLabel->"Out[1773]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "$West"}]], "Input",
 CellTags->"TR",
 CellLabel->"In[1774]:="],

Cell[BoxData[
 StyleBox["\<\"If $West is set to True (which is the default), traces \
involving more than 4 Dirac matrices and gamma5 are calculated recursively \
according to formula (A.5) from Comp. Phys. Comm 77 (1993) 286-298, which is \
based on the Breitenlohner Maison gamma5 - scheme.\"\>", "MSG"]], "Print", \
"PrintUsage",
 CellChangeTimes->{3.5009704481194277`*^9},
 CellTags->{"Info3500974047-4982097", "TR"}]
}, Open  ]],

Cell["\<\
$West=False should be set before any calculation is done (because \
intermediate results from trace calculations are cached).\
\>", "Text",
 CellTags->"TR"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"$West", "=", "False"}]], "Input",
 CellTags->"TR",
 CellLabel->"In[1775]:="],

Cell[BoxData[
 FormBox["False", TraditionalForm]], "Output",
 CellChangeTimes->{3.500970448576906*^9},
 CellTags->"TR",
 CellLabel->"Out[1775]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TR", "[", 
  RowBox[{
   RowBox[{"GAD", "[", 
    RowBox[{
     SubscriptBox["\[Nu]", "1"], ",", 
     SubscriptBox["\[Nu]", "2"], ",", 
     SubscriptBox["\[Nu]", "3"], ",", 
     SubscriptBox["\[Nu]", "4"], ",", 
     SubscriptBox["\[Nu]", "5"], ",", 
     SubscriptBox["\[Nu]", "6"]}], "]"}], ".", 
   RowBox[{"GA", "[", "5", "]"}]}], "]"}]], "Input",
 CellTags->"TR",
 CellLabel->"In[1776]:="],

Cell[BoxData[
 FormBox["0", TraditionalForm]], "Output",
 CellChangeTimes->{3.500970448755039*^9},
 CellTags->"TR",
 CellLabel->"Out[1776]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TR", "[", 
  RowBox[{"GS", "[", 
   RowBox[{"p", ",", "q", ",", "r", ",", "s"}], "]"}], "]"}]], "Input",
 CellTags->"TR",
 CellLabel->"In[1777]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"4", " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{
       FormBox["p",
        TraditionalForm], "\[CenterDot]", 
       FormBox["s",
        TraditionalForm]}], " ", 
      RowBox[{
       FormBox["q",
        TraditionalForm], "\[CenterDot]", 
       FormBox["r",
        TraditionalForm]}]}], "-", 
     RowBox[{
      RowBox[{
       FormBox["p",
        TraditionalForm], "\[CenterDot]", 
       FormBox["r",
        TraditionalForm]}], " ", 
      RowBox[{
       FormBox["q",
        TraditionalForm], "\[CenterDot]", 
       FormBox["s",
        TraditionalForm]}]}], "+", 
     RowBox[{
      RowBox[{
       FormBox["p",
        TraditionalForm], "\[CenterDot]", 
       FormBox["q",
        TraditionalForm]}], " ", 
      RowBox[{
       FormBox["r",
        TraditionalForm], "\[CenterDot]", 
       FormBox["s",
        TraditionalForm]}]}]}], ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009704488585253`*^9},
 CellTags->"TR",
 CellLabel->"Out[1777]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TR", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"GS", "[", "p", "]"}], "+", "m"}], ")"}], ".", 
    RowBox[{"GA", "[", "\[Mu]", "]"}], ".", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"GS", "[", "q", "]"}], "+", "m"}], ")"}], ".", 
    RowBox[{"GA", "[", "\[Mu]", "]"}]}], ",", 
   RowBox[{"Factoring", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"TR",
 CellLabel->"In[1778]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"8", " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"2", " ", 
      SuperscriptBox["m", "2"]}], "-", 
     RowBox[{
      FormBox["p",
       TraditionalForm], "\[CenterDot]", 
      FormBox["q",
       TraditionalForm]}]}], ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970448980496*^9},
 CellTags->"TR",
 CellLabel->"Out[1778]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TR", "[", 
  RowBox[{
   RowBox[{"GA", "[", 
    RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}], ",", 
   RowBox[{"FCE", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"TR",
 CellLabel->"In[1779]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"4", " ", 
   SuperscriptBox["g", 
    RowBox[{"\[Alpha]", "\[Beta]"}]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970449089345*^9},
 CellTags->"TR",
 CellLabel->"Out[1779]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "StandardForm"}]], "Input",
 CellTags->"TR",
 CellLabel->"In[1780]:="],

Cell[BoxData[
 RowBox[{"4", " ", 
  RowBox[{"MT", "[", 
   RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}]}]], "Output",
 CellChangeTimes->{3.500970449244205*^9},
 CellTags->"TR",
 CellLabel->"Out[1780]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TR", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"GAD", "[", "\[Mu]", "]"}], ".", 
    RowBox[{"GA", "[", "\[Alpha]", "]"}], ".", 
    RowBox[{"GA", "[", "\[Beta]", "]"}], ".", 
    RowBox[{"GAD", "[", "\[Mu]", "]"}]}], ",", 
   RowBox[{"FCE", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"TR",
 CellLabel->"In[1781]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"4", " ", "D", " ", 
   SuperscriptBox["g", 
    RowBox[{"\[Alpha]", "\[Beta]"}]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009704493839293`*^9},
 CellTags->"TR",
 CellLabel->"Out[1781]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "StandardForm"}]], "Input",
 CellTags->"TR",
 CellLabel->"In[1782]:="],

Cell[BoxData[
 RowBox[{"4", " ", "D", " ", 
  RowBox[{"MT", "[", 
   RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}]}]], "Output",
 CellChangeTimes->{3.500970449468088*^9},
 CellTags->"TR",
 CellLabel->"Out[1782]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t", "=", 
  RowBox[{
   RowBox[{"GA", "[", 
    RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}], " ", 
   RowBox[{
    RowBox[{"SUNT", "[", "b", "]"}], ".", 
    RowBox[{"SUNT", "[", "c", "]"}]}], " ", 
   RowBox[{"SUNDelta", "[", 
    RowBox[{"c", ",", "b"}], "]"}]}]}]], "Input",
 CellTags->"TR",
 CellLabel->"In[1783]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["\[Delta]", 
    RowBox[{"b", "c"}]], " ", 
   RowBox[{
    SubscriptBox["T", 
     FormBox["b",
      TraditionalForm]], ".", 
    SubscriptBox["T", 
     FormBox["c",
      TraditionalForm]]}], " ", 
   RowBox[{
    SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
    SuperscriptBox["\[Gamma]", "\[Nu]"]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970449570142*^9},
 CellTags->"TR",
 CellLabel->"Out[1783]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TR", "[", 
  RowBox[{"t", ",", 
   RowBox[{"SUNTrace", "\[Rule]", "False"}], ",", 
   RowBox[{"SUNNToCACF", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"TR",
 CellLabel->"In[1784]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"4", " ", 
   SubscriptBox["C", "F"], " ", 
   SuperscriptBox["g", 
    RowBox[{"\[Mu]", "\[Nu]"}]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009704497129593`*^9},
 CellTags->"TR",
 CellLabel->"Out[1784]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TR", "[", 
  RowBox[{"t", ",", 
   RowBox[{"SUNTrace", "\[Rule]", "True"}], ",", 
   RowBox[{"SUNNToCACF", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"TR",
 CellLabel->"In[1785]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"4", " ", 
   SubscriptBox["C", "F"], " ", 
   SuperscriptBox["g", 
    RowBox[{"\[Mu]", "\[Nu]"}]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009704499669437`*^9},
 CellTags->"TR",
 CellLabel->"Out[1785]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TR", "[", 
  RowBox[{"1", ",", 
   RowBox[{"SUNTrace", "\[Rule]", "False"}], ",", 
   RowBox[{"SUNNToCACF", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"TR",
 CellLabel->"In[1786]:="],

Cell[BoxData[
 FormBox["4", TraditionalForm]], "Output",
 CellChangeTimes->{3.500970450129458*^9},
 CellTags->"TR",
 CellLabel->"Out[1786]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TR", "[", 
  RowBox[{"1", ",", 
   RowBox[{"SUNTrace", "\[Rule]", "True"}], ",", 
   RowBox[{"SUNNToCACF", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"TR",
 CellLabel->"In[1787]:="],

Cell[BoxData[
 FormBox["4", TraditionalForm]], "Output",
 CellChangeTimes->{3.500970450353366*^9},
 CellTags->"TR",
 CellLabel->"Out[1787]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Tr", "[", " ", 
  RowBox[{"GA", "[", 
   RowBox[{"m", ",", "n"}], "]"}], "]"}]], "Input",
 CellLabel->"In[1788]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"4", " ", 
   SuperscriptBox["g", 
    RowBox[{"m", "n"}]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009704504757442`*^9},
 CellLabel->"Out[1788]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", "t", "]"}], ";"}]], "Input",
 CellTags->"TR",
 CellLabel->"In[1789]:="]
}, Open  ]],

Cell["TraceDimension", "ObjectName",
 CellTags->"TraceDimension"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"TraceDimension"],

Cell["\<\
TraceDimension is an option for FeynCalc2FORM. If set to 4: trace, if set to \
n: tracen.\
\>", "Text",
 CellTags->"TraceDimension"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FeynCalc2FORM",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FeynCalc2FORM",
  ButtonNote->"FeynCalc2FORM"],
 "."
}], "Text",
 CellTags->"TraceDimension"]
}, Open  ]],

Cell["TraceOfOne", "ObjectName",
 CellTags->"TraceOfOne"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"TraceOfOne"],

Cell["\<\
TraceOfOne is an option for Tr and DiracTrace. Its setting determines the \
value of the unit trace.\
\>", "Text",
 CellTags->"TraceOfOne"],

Cell[TextData[{
 "See also: ",
 ButtonBox["TR",
  BaseStyle->"AddOnsLink",
  ButtonData:>"TR",
  ButtonNote->"Tr"],
 ", ",
 ButtonBox["DiracTrace",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracTrace",
  ButtonNote->"DiracTrace"],
 "."
}], "Text",
 CellTags->"TraceOfOne"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"TraceOfOne"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TR", "[", 
  RowBox[{"1", ",", 
   RowBox[{"SUNTrace", "\[Rule]", "False"}], ",", 
   RowBox[{"SUNNToCACF", "\[Rule]", "True"}], ",", 
   RowBox[{"TraceOfOne", "\[Rule]", "tr1"}]}], "]"}]], "Input",
 CellTags->"TraceOfOne",
 CellLabel->"In[1790]:="],

Cell[BoxData[
 FormBox["tr1", TraditionalForm]], "Output",
 CellChangeTimes->{3.500970450624161*^9},
 CellTags->"TraceOfOne",
 CellLabel->"Out[1790]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TR", "[", 
  RowBox[{"1", ",", 
   RowBox[{"SUNTrace", "\[Rule]", "True"}], ",", 
   RowBox[{"SUNNToCACF", "\[Rule]", "True"}], ",", 
   RowBox[{"TraceOfOne", "\[Rule]", "tr2"}]}], "]"}]], "Input",
 CellTags->"TraceOfOne",
 CellLabel->"In[1791]:="],

Cell[BoxData[
 FormBox["tr2", TraditionalForm]], "Output",
 CellChangeTimes->{3.5009704507850847`*^9},
 CellTags->"TraceOfOne",
 CellLabel->"Out[1791]="]
}, Open  ]]
}, Open  ]],

Cell["Trick", "ObjectName",
 CellTags->"Trick"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Trick"],

Cell["\<\
Trick[exp] performs several basic simplifications without expansion. \
Trick[exp] uses Contract, DotSimplify and SUNDeltaContract.\
\>", "Text",
 CellTags->"Trick"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Calc",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Calc",
  ButtonNote->"Calc"],
 ", ",
 ButtonBox["Contract",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Contract",
  ButtonNote->"Contract"],
 ", ",
 ButtonBox["DiracTrick",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracTrick",
  ButtonNote->"DiracTrick"],
 ", ",
 ButtonBox["DotSimplify",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DotSimplify",
  ButtonNote->"DotSimplify"],
 ", ",
 ButtonBox["DiracTrick",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracTrick",
  ButtonNote->"DiracTrick"],
 "."
}], "Text",
 CellTags->"Trick"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData["Examples"], "Subsubsection",
 CellTags->"Trick"],

Cell[TextData[{
 "This calculates ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", " ", "\[Nu]"}]], 
    SubscriptBox["\[Gamma]", "\[Mu]"]}], TraditionalForm]]],
 "and ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["g", "\[Nu]", "\[Nu]"], TraditionalForm]]],
 " in D dimensions."
}], "Text",
 ZeroWidthTimes->True,
 CellTags->"Trick"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Trick", "[", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     RowBox[{"GA", "[", "\[Mu]", "]"}], " ", 
     RowBox[{"MT", "[", 
      RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}]}], ",", " ", 
    RowBox[{"MTD", "[", 
     RowBox[{"\[Nu]", ",", "\[Nu]"}], "]"}]}], "}"}], "]"}]], "Input",
 CellTags->"Trick",
 CellLabel->"In[1792]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm]], ",", "D"}], "}"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970450968565*^9},
 CellTags->"Trick",
 CellLabel->"Out[1792]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"FV", "[", 
   RowBox[{
    RowBox[{"p", "+", "r"}], ",", "\[Mu]"}], "]"}], " ", 
  RowBox[{"MT", "[", 
   RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}], " ", 
  RowBox[{"FV", "[", 
   RowBox[{
    RowBox[{"q", "-", "p"}], ",", "\[Nu]"}], "]"}]}]], "Input",
 CellTags->"Trick",
 CellLabel->"In[1793]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["g", 
    RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
   RowBox[{"(", 
    SuperscriptBox[
     FormBox[
      RowBox[{"q", "-", "p"}],
      TraditionalForm], "\[Nu]"], ")"}], " ", 
   RowBox[{"(", 
    SuperscriptBox[
     FormBox[
      RowBox[{"p", "+", "r"}],
      TraditionalForm], "\[Mu]"], ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970451070128*^9},
 CellTags->"Trick",
 CellLabel->"Out[1793]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Trick", "[", "%", "]"}]], "Input",
 CellTags->"Trick",
 CellLabel->"In[1794]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    FormBox["p",
     TraditionalForm], "\[CenterDot]", 
    FormBox["q",
     TraditionalForm]}], "-", 
   RowBox[{
    FormBox["p",
     TraditionalForm], "\[CenterDot]", 
    FormBox["r",
     TraditionalForm]}], "-", 
   SuperscriptBox[
    FormBox["p",
     TraditionalForm], "2"], "+", 
   RowBox[{
    FormBox["q",
     TraditionalForm], "\[CenterDot]", 
    FormBox["r",
     TraditionalForm]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970451202869*^9},
 CellTags->"Trick",
 CellLabel->"Out[1794]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Trick", "[", 
  RowBox[{"c", ".", "b", ".", "a", " ", ".", " ", 
   RowBox[{"GA", "[", "d", "]"}], ".", 
   RowBox[{"GA", "[", "e", "]"}]}], "]"}]], "Input",
 CellTags->"Trick",
 CellLabel->"In[1795]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"a", " ", "b", " ", "c", " ", 
   RowBox[{
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox["d",
       TraditionalForm],
      TraditionalForm]], ".", 
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox["e",
       TraditionalForm],
      TraditionalForm]]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970451409197*^9},
 CellTags->"Trick",
 CellLabel->"Out[1795]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"%", "//", "FCE"}], "//", "StandardForm"}]], "Input",
 CellTags->"Trick",
 CellLabel->"In[1796]:="],

Cell[BoxData[
 RowBox[{"a", " ", "b", " ", "c", " ", 
  RowBox[{
   RowBox[{"GA", "[", "d", "]"}], ".", 
   RowBox[{"GA", "[", "e", "]"}]}]}]], "Output",
 CellChangeTimes->{3.5009704515738087`*^9},
 CellTags->"Trick",
 CellLabel->"Out[1796]//StandardForm="]
}, Open  ]]
}, Open  ]],

Cell["TrickIntegrate", "ObjectName",
 CellTags->"TrickIntegrate"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"TrickIntegrate"],

Cell["\<\
 TrickIntegrate[(1-t)^(a Epsilon -1) g[t], t] does an integration trick for \
the definite integral of ((1-t)^(a Epsilon -1) g[t])  from 0 to 1. \
TrickIntegrate[(1-t)^(a Epsilon -1) g[t], t] gives  g[1]/a/Epsilon + \
Hold[Integrate][(1-t)^(a Epsilon-1) (g[t]-g[1]),{t,0,1}]. TrickIntegrate[t^(a \
Epsilon -1) f[t], t] gives f[0]/a/Epsilon + Hold[Integrate][t^(a Epsilon-1) \
(f[t]-f[0]),{t,0,1}], provided g[1] and f[0] do exist.\
\>", "Text",
 CellTags->"TrickIntegrate"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"TrickIntegrate"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TrickIntegrate", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{"1", "-", "t"}], ")"}], "^", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"a", " ", "Epsilon"}], "-", "1"}], ")"}]}], " ", 
    RowBox[{"g", "[", "t", "]"}]}], ",", "t"}], "]"}]], "Input",
 CellTags->"TrickIntegrate",
 CellLabel->"In[1797]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"Hold", "[", "Integrate", "]"}], "[", 
   RowBox[{
    RowBox[{
     RowBox[{"g", "(", "t", ")"}], " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"1", "-", "t"}], ")"}], 
      RowBox[{
       RowBox[{"a", " ", 
        TagBox["\[CurlyEpsilon]",
         TraditionalForm]}], "-", "1"}]]}], ",", 
    RowBox[{"{", 
     RowBox[{"t", ",", "0", ",", "1"}], "}"}]}], "]"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.5009704520026093`*^9},
 CellTags->"TrickIntegrate",
 CellLabel->"Out[1797]="]
}, Open  ]]
}, Open  ]],

Cell["TrickMandelstam", "ObjectName",
 CellTags->"TrickMandelstam"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"TrickMandelstam"],

Cell[TextData[{
 "TrickMandelstam[expr, {s, t, u, ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox[
    StyleBox["m",
     FontSlant->"Plain"], "1", "2"], TraditionalForm]]],
 " + ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox[
    StyleBox["m",
     FontSlant->"Plain"], "2", "2"], TraditionalForm]]],
 "+ ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox[
    StyleBox["m",
     FontSlant->"Plain"], "3", "2"], TraditionalForm]]],
 "+ ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox[
    StyleBox["m",
     FontSlant->"Plain"], "4", "2"], TraditionalForm]]],
 "}] simplifies all sums in expr so that one of the Mandelstam variables s, t \
or u is eliminated by the relation ",
 StyleBox["s",
  FontSlant->"Italic"],
 " + ",
 StyleBox["t",
  FontSlant->"Italic"],
 " + ",
 StyleBox["u",
  FontSlant->"Italic"],
 " = ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox[
    StyleBox["m",
     FontSlant->"Italic"], "1", "2"], TraditionalForm]]],
 " + ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["m", "2", "2"], TraditionalForm]]],
 "+ ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["m", "3", "2"], TraditionalForm]]],
 "+ ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["m", "4", "2"], TraditionalForm]]],
 " . The trick is that the resulting sum has the most short number of terms."
}], "Text",
 CellTags->"TrickMandelstam"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"TrickMandelstam"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"TrickMandelstam", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{"s", "+", "t", "-", "u"}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"2", 
        SubsuperscriptBox["M", "W", "2"]}], "-", "t", "-", "u"}], ")"}]}], 
    ",", 
    RowBox[{"{", 
     RowBox[{"s", ",", "t", ",", "u", ",", 
      RowBox[{"2", 
       SubsuperscriptBox["M", "W", "2"]}]}], "}"}]}], "]"}], "//", 
  "Factor2"}]], "Input",
 CellTags->"TrickMandelstam",
 CellLabel->"In[1798]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", "2"}], " ", "s", " ", 
   RowBox[{"(", 
    RowBox[{"u", "-", 
     SubsuperscriptBox["M", "W", "2"]}], ")"}]}], TraditionalForm]], "Output",\

 CellChangeTimes->{3.5009704523833942`*^9},
 CellTags->"TrickMandelstam",
 CellLabel->"Out[1798]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TrickMandelstam", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"M", "^", "2"}], " ", "s"}], " ", "-", " ", 
    RowBox[{"s", "^", "2"}], " ", "+", " ", 
    RowBox[{
     RowBox[{"M", "^", "2"}], " ", "t"}], " ", "-", " ", "st", " ", "+", " ", 
    
    RowBox[{
     RowBox[{"M", "^", "2"}], " ", "u"}], " ", "-", " ", "su"}], ",", " ", 
   RowBox[{"{", 
    RowBox[{"s", ",", "t", ",", "u", ",", 
     RowBox[{"2", 
      RowBox[{"M", "^", "2"}]}]}], "}"}]}], "]"}]], "Input",
 CellTags->"TrickMandelstam",
 CellLabel->"In[1799]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"2", " ", 
    SuperscriptBox["M", "4"]}], "-", 
   SuperscriptBox["s", "2"], "-", "st", "-", "su"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970452483961*^9},
 CellTags->"TrickMandelstam",
 CellLabel->"Out[1799]="]
}, Open  ]]
}, Open  ]],

Cell["Tr2", "ObjectName",
 CellTags->"Tr2"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Tr2"],

Cell["\<\
If exp contains DiracTrace's, Tr2[exp] simplifies exp and does the Dirac \
traces unless more that 4 gamma matrices and DiracGamma[5] occur. Tr2[exp] \
also separates the color-strucure, and takes the color trace if Tf occurs in \
exp. If exp does not contain DiracTrace's, Tr2[exp] takes the Dirac trace.\
\>", "Text",
 CellTags->"Tr2"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "Tr2", "]"}]], "Input",
 CellTags->"Tr2",
 CellLabel->"In[1800]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"Factoring", "\[Rule]", "False"}], "}"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970452564686*^9},
 CellTags->"Tr2",
 CellLabel->"Out[1800]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 ButtonBox["Tr",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Tr",
  ButtonNote->"Tr"],
 ", ",
 ButtonBox["Tf",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Tf",
  ButtonNote->"Tf"],
 ", ",
 ButtonBox["DiracTrace",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracTrace",
  ButtonNote->"DiracTrace"],
 ", ",
 ButtonBox["DiracSimplify",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracSimplify",
  ButtonNote->"DiracSimplify"],
 ", ",
 ButtonBox["SUNTrace",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SUNTrace",
  ButtonNote->"SUNTrace"],
 "."
}], "Text",
 CellTags->"Tr2"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Tr2"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Tr2", "[", 
  RowBox[{"DiracMatrix", "[", 
   RowBox[{"a", ",", "b", ",", "c", ",", "d"}], "]"}], "]"}]], "Input",
 CellLabel->"In[1801]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"4", " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"a", "d"}]], " ", 
      SuperscriptBox["g", 
       RowBox[{"b", "c"}]]}], "-", 
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"a", "c"}]], " ", 
      SuperscriptBox["g", 
       RowBox[{"b", "d"}]]}], "+", 
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"a", "b"}]], " ", 
      SuperscriptBox["g", 
       RowBox[{"c", "d"}]]}]}], ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970452684143*^9},
 CellLabel->"Out[1801]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TR", "[", 
  RowBox[{
   RowBox[{"a", " ", "Tf"}], ",", 
   RowBox[{"SUNTrace", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"Tr2",
 CellLabel->"In[1802]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"4", " ", "a", " ", 
   SubscriptBox["T", "f"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009704536176653`*^9},
 CellTags->"Tr2",
 CellLabel->"Out[1802]="]
}, Open  ]]
}, Open  ]],

Cell["Twist2AlienOperator  ***unfinished*** (unclear usage def.)", \
"ObjectName",
 CellTags->"Twist2AlienOperator"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Twist2AlienOperator"],

Cell["\<\
Twist2AlienOperator[p, 0] : (7);   Twist2AlienOperator[p1,p2,{p3,mu,a}, 0] \
(p1: incoming quark momentum, p3: incoming gluon (count1)).\
\>", "Text",
 CellTags->"Twist2AlienOperator"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Twist2AlienOperator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Twist2AlienOperator", "[", 
  RowBox[{"p", ",", " ", "0"}], "]"}]], "Input",
 CellTags->"Twist2AlienOperator",
 CellLabel->"In[1803]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{"2", " ", 
    RowBox[{"(", 
     RowBox[{
      FractionBox["2", "m"], "-", 
      FractionBox["1", 
       RowBox[{"m", "+", "1"}]], "-", 
      FractionBox["2", 
       RowBox[{"m", "-", "1"}]]}], ")"}], " ", 
    SubscriptBox["C", "F"], " ", 
    SubsuperscriptBox["g", "s", "2"], " ", 
    SubscriptBox["S", "n"], " ", 
    RowBox[{
     FormBox["\<\"\[Gamma]\"\>",
      TraditionalForm], "\[CenterDot]", 
     FormBox["\[CapitalDelta]",
      TraditionalForm]}], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{
       FormBox["\[CapitalDelta]",
        TraditionalForm], "\[CenterDot]", 
       FormBox["p",
        TraditionalForm]}], ")"}], 
     RowBox[{"m", "-", "1"}]]}], 
   TagBox["\[CurlyEpsilon]",
    TraditionalForm]], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970454027899*^9},
 CellTags->"Twist2AlienOperator",
 CellLabel->"Out[1803]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Twist2AlienOperator", "[", 
  RowBox[{"p1", ",", "p2", ",", 
   RowBox[{"{", 
    RowBox[{"p3", ",", "mu", ",", "a"}], "}"}], ",", " ", "0"}], 
  "]"}]], "Input",
 CellTags->"Twist2AlienOperator",
 CellLabel->"In[1804]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{"\[ImaginaryI]", " ", 
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"-", "1"}], ")"}], "m"], "+", "1"}], ")"}], " ", 
    SubsuperscriptBox["g", "s", "3"], " ", 
    SuperscriptBox[
     FormBox["\[CapitalDelta]",
      TraditionalForm], 
     FormBox[
      FormBox["mu",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubscriptBox["S", "n"], " ", 
    RowBox[{
     SubscriptBox["T", 
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm]], ".", 
     RowBox[{"(", 
      RowBox[{
       FormBox["\<\"\[Gamma]\"\>",
        TraditionalForm], "\[CenterDot]", 
       FormBox["\[CapitalDelta]",
        TraditionalForm]}], ")"}]}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         FractionBox["2", "m"], "-", 
         FractionBox["1", 
          RowBox[{"m", "+", "1"}]], "-", 
         FractionBox["2", 
          RowBox[{"m", "-", "1"}]]}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{
         FractionBox[
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["p1",
              TraditionalForm]}], ")"}], 
           RowBox[{"m", "-", "1"}]], 
          RowBox[{
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["p1",
             TraditionalForm]}], "+", 
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["p3",
             TraditionalForm]}]}]], "-", 
         FractionBox[
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{"-", 
             RowBox[{"(", 
              RowBox[{
               FormBox["\[CapitalDelta]",
                TraditionalForm], "\[CenterDot]", 
               FormBox["p3",
                TraditionalForm]}], ")"}]}], ")"}], 
           RowBox[{"m", "-", "1"}]], 
          RowBox[{
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["p1",
             TraditionalForm]}], "+", 
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["p3",
             TraditionalForm]}]}]]}], ")"}]}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         FractionBox["1", 
          RowBox[{"m", "-", "1"}]], "-", 
         FractionBox["1", "m"]}], ")"}], " ", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"-", 
          RowBox[{"(", 
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["p3",
             TraditionalForm]}], ")"}]}], ")"}], 
        RowBox[{"m", "-", "2"}]]}]}], ")"}]}], 
   TagBox["\[CurlyEpsilon]",
    TraditionalForm]], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970454581538*^9},
 CellTags->"Twist2AlienOperator",
 CellLabel->"Out[1804]="]
}, Open  ]]
}, Open  ]],

Cell["Twist2CounterOperator  ***unfinished*** (unclear usage def.)", \
"ObjectName",
 CellTags->"Twist2CounterOperator"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Twist2CounterOperator"],

Cell["\<\
Twist2CounterOperator[p,mu,nu,a,b,5]; Twist2CounterOperator[p, 7] : (7);  \
Twist2CounterOperator[p1,p2,{p3,mu,a}, 1] (p1: incoming quark momentum, p3: \
incoming gluon (count1)).\
\>", "Text",
 CellTags->"Twist2CounterOperator"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Twist2CounterOperator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Twist2CounterOperator", "[", 
  RowBox[{"p", ",", "mu", ",", "nu", ",", "a", ",", "b", ",", "5"}], 
  "]"}]], "Input",
 CellTags->"Twist2CounterOperator",
 CellLabel->"In[1805]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   RowBox[{
    FractionBox["1", 
     RowBox[{"2", " ", 
      TagBox["\[CurlyEpsilon]",
       TraditionalForm]}]], 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"-", "1"}], ")"}], "m"], "+", "1"}], ")"}], " ", 
     SubscriptBox["C", "A"], " ", 
     SubsuperscriptBox["g", "s", "2"], " ", 
     SubscriptBox["S", "n"], " ", 
     SubscriptBox["\[Delta]", 
      RowBox[{
       FormBox[
        FormBox["a",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["b",
         TraditionalForm],
        TraditionalForm]}]], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          FractionBox["8", "m"], "-", 
          FractionBox["12", 
           RowBox[{"m", "+", "1"}]], "+", 
          FractionBox["8", 
           RowBox[{"m", "+", "2"}]], "-", 
          FractionBox["8", 
           RowBox[{"m", "-", "1"}]]}], ")"}], " ", 
        SuperscriptBox["g", 
         RowBox[{"mu", "nu"}]], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["p",
            TraditionalForm]}], ")"}], "m"]}], "+", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          FractionBox["8", "m"], "-", 
          FractionBox["24", 
           RowBox[{"m", "+", "1"}]], "+", 
          FractionBox["24", 
           RowBox[{"m", "+", "2"}]], "-", 
          FractionBox["4", 
           RowBox[{"m", "-", "1"}]]}], ")"}], " ", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], "2"], " ", 
        SuperscriptBox[
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox[
          FormBox["mu",
           TraditionalForm],
          TraditionalForm]], " ", 
        SuperscriptBox[
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox[
          FormBox["nu",
           TraditionalForm],
          TraditionalForm]], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["p",
            TraditionalForm]}], ")"}], 
         RowBox[{"m", "-", "2"}]]}], "+", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{"-", 
           FractionBox["6", "m"]}], "+", 
          FractionBox["16", 
           RowBox[{"m", "+", "1"}]], "-", 
          FractionBox["16", 
           RowBox[{"m", "+", "2"}]], "+", 
          FractionBox["6", 
           RowBox[{"m", "-", "1"}]]}], ")"}], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["p",
            TraditionalForm]}], ")"}], 
         RowBox[{"m", "-", "1"}]], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            FormBox["p",
             TraditionalForm], 
            FormBox[
             FormBox["mu",
              TraditionalForm],
             TraditionalForm]], " ", 
           SuperscriptBox[
            FormBox["\[CapitalDelta]",
             TraditionalForm], 
            FormBox[
             FormBox["nu",
              TraditionalForm],
             TraditionalForm]]}], "+", 
          RowBox[{
           SuperscriptBox[
            FormBox["\[CapitalDelta]",
             TraditionalForm], 
            FormBox[
             FormBox["mu",
              TraditionalForm],
             TraditionalForm]], " ", 
           SuperscriptBox[
            FormBox["p",
             TraditionalForm], 
            FormBox[
             FormBox["nu",
              TraditionalForm],
             TraditionalForm]]}]}], ")"}]}]}], ")"}]}]}]}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970454933762*^9},
 CellTags->"Twist2CounterOperator",
 CellLabel->"Out[1805]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Twist2CounterOperator", "[", 
  RowBox[{"p", ",", "7"}], "]"}]], "Input",
 CellTags->"Twist2CounterOperator",
 CellLabel->"In[1806]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"-", "1"}], ")"}], "m"], "+", "1"}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      FractionBox["2", "m"], "-", 
      FractionBox["1", 
       RowBox[{"m", "+", "1"}]], "-", 
      FractionBox["2", 
       RowBox[{"m", "-", "1"}]]}], ")"}], " ", 
    SubscriptBox["C", "F"], " ", 
    SubsuperscriptBox["g", "s", "2"], " ", 
    SubscriptBox["S", "n"], " ", 
    RowBox[{
     FormBox["\<\"\[Gamma]\"\>",
      TraditionalForm], "\[CenterDot]", 
     FormBox["\[CapitalDelta]",
      TraditionalForm]}], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{
       FormBox["\[CapitalDelta]",
        TraditionalForm], "\[CenterDot]", 
       FormBox["p",
        TraditionalForm]}], ")"}], 
     RowBox[{"m", "-", "1"}]]}], 
   TagBox["\[CurlyEpsilon]",
    TraditionalForm]], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009704551155148`*^9},
 CellTags->"Twist2CounterOperator",
 CellLabel->"Out[1806]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Twist2CounterOperator", "[", 
  RowBox[{"p1", ",", "p2", ",", 
   RowBox[{"{", 
    RowBox[{"p3", ",", "mu", ",", "a"}], "}"}], ",", "1"}], "]"}]], "Input",
 CellTags->"Twist2CounterOperator",
 CellLabel->"In[1807]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   FractionBox[
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"-", "1"}], ")"}], "m"], "+", "1"}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{
       FractionBox["2", "m"], "-", 
       FractionBox["1", 
        RowBox[{"m", "+", "1"}]], "-", 
       FractionBox["2", 
        RowBox[{"m", "-", "1"}]]}], ")"}], " ", 
     SubscriptBox["T", 
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm]], " ", 
     SubsuperscriptBox["g", "s", "3"], " ", 
     SuperscriptBox[
      FormBox["\[CapitalDelta]",
       TraditionalForm], 
      FormBox[
       FormBox["mu",
        TraditionalForm],
       TraditionalForm]], " ", 
     SubscriptBox["S", "n"], " ", 
     RowBox[{"(", 
      RowBox[{
       SubscriptBox["C", "A"], "-", 
       RowBox[{"2", " ", 
        SubscriptBox["C", "F"]}]}], ")"}], " ", 
     RowBox[{
      FormBox["\<\"\[Gamma]\"\>",
       TraditionalForm], "\[CenterDot]", 
      FormBox["\[CapitalDelta]",
       TraditionalForm]}], " ", 
     RowBox[{"(", 
      RowBox[{
       FractionBox[
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["p1",
            TraditionalForm]}], ")"}], 
         RowBox[{"m", "-", "1"}]], 
        RowBox[{
         RowBox[{
          FormBox["\[CapitalDelta]",
           TraditionalForm], "\[CenterDot]", 
          FormBox["p1",
           TraditionalForm]}], "+", 
         RowBox[{
          FormBox["\[CapitalDelta]",
           TraditionalForm], "\[CenterDot]", 
          FormBox["p2",
           TraditionalForm]}]}]], "-", 
       FractionBox[
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"-", 
           RowBox[{"(", 
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["p2",
              TraditionalForm]}], ")"}]}], ")"}], 
         RowBox[{"m", "-", "1"}]], 
        RowBox[{
         RowBox[{
          FormBox["\[CapitalDelta]",
           TraditionalForm], "\[CenterDot]", 
          FormBox["p1",
           TraditionalForm]}], "+", 
         RowBox[{
          FormBox["\[CapitalDelta]",
           TraditionalForm], "\[CenterDot]", 
          FormBox["p2",
           TraditionalForm]}]}]]}], ")"}]}], 
    RowBox[{"2", " ", 
     TagBox["\[CurlyEpsilon]",
      TraditionalForm]}]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970455800061*^9},
 CellTags->"Twist2CounterOperator",
 CellLabel->"Out[1807]="]
}, Open  ]]
}, Open  ]],

Cell["Twist2GluonOperator", "ObjectName",
 CellTags->"Twist2GluonOperator"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Twist2GluonOperator"],

Cell["\<\
Twist2GluonOperator[{p, mu, a}, {nu, b}] or Twist2GluonOperator[p, {mu, a}, \
{nu, b}] or Twist2GluonOperator[p, mu,a, nu,b] yields the 2-gluon operator (p \
is ingoing momentum corresponding to Lorentz index mu). \
Twist2GluonOperator[{p,mu,a}, {q,nu,b}, {k,la,c}] or Twist2GluonOperator[ \
p,mu,a , q,nu,b , k,la,c ] gives the 3-gluon operator. \
Twist2GluonOperator[{p,mu,a}, {q,nu,b}, {k,la,c}, {s,si,d}] or \
Twist2GluonOperator[p,mu,a , q,nu,b , k,la,c , s,si,d] yields the 4-Gluon \
operator. The dimension is determined by the option and Dimension. The \
setting of the option Polarization (unpolarized: 0; polarized: 1) determines \
whether the unpolarized or polarized feynman rule is returned. With the \
setting Explicit to False the color-structure and the (1+(-1)^OPEm) (for \
polarized: (1-(-1)^OPEm)) is extracted and the color indices are omitted in \
the arguments of Twist2GluonOperator.\
\>", "Text",
 CellTags->"Twist2GluonOperator"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Twist2QuarkOperator",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Twist2QuarkOperator",
  ButtonNote->"Twist2QuarkOperator"],
 "."
}], "Text",
 CellTags->"Twist2GluonOperator"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Polarized case, zero-momentum insertion ", "Subsubsection",
 CellTags->"Twist2GluonOperator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SetOptions", "[", 
  RowBox[{"Twist2GluonOperator", ",", " ", 
   RowBox[{"Polarization", " ", "\[Rule]", " ", "1"}], ",", 
   RowBox[{"Explicit", "\[Rule]", "False"}], ",", " ", 
   RowBox[{"ZeroMomentumInsertion", " ", "\[Rule]", " ", "True"}]}], 
  "]"}]], "Input",
 CellTags->"Twist2GluonOperator",
 CellLabel->"In[1808]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"CouplingConstant", "\[Rule]", 
     SubscriptBox["g", "s"]}], ",", 
    RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
    RowBox[{"Polarization", "\[Rule]", "1"}], ",", 
    RowBox[{"Explicit", "\[Rule]", "False"}], ",", 
    RowBox[{"ZeroMomentumInsertion", "\[Rule]", "True"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970455881382*^9},
 CellTags->"Twist2GluonOperator",
 CellLabel->"Out[1808]="]
}, Open  ]],

Cell["2-gluon legs", "Text",
 CellTags->"Twist2GluonOperator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Twist2GluonOperator", "[", 
  RowBox[{"p", ",", 
   RowBox[{"{", 
    RowBox[{"\[Mu]", ",", "a"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\[Nu]", ",", "b"}], "}"}]}], "]"}]], "Input",
 CellTags->"Twist2GluonOperator",
 CellLabel->"In[1809]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"\[ImaginaryI]", " ", 
   RowBox[{"(", 
    RowBox[{"1", "-", 
     FormBox[
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"-", "1"}], ")"}], "m"],
      TraditionalForm]}], ")"}], " ", 
   SubscriptBox["\[Delta]", 
    RowBox[{
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm]}]], " ", 
   RowBox[{"(", 
    RowBox[{
     SubsuperscriptBox["O", 
      RowBox[{
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm], "\[VeryThinSpace]", 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]}], "G2"], "(", 
     FormBox["p",
      TraditionalForm], ")"}], ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970455964799*^9},
 CellTags->"Twist2GluonOperator",
 CellLabel->"Out[1809]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Twist2GluonOperator", "[", 
  RowBox[{"p", ",", 
   RowBox[{"{", 
    RowBox[{"\[Mu]", ",", "a"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\[Nu]", ",", "b"}], "}"}], ",", 
   RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"Twist2GluonOperator",
 CellLabel->"In[1810]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"\[ImaginaryI]", " ", 
   RowBox[{"(", 
    RowBox[{"1", "-", 
     FormBox[
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"-", "1"}], ")"}], "m"],
      TraditionalForm]}], ")"}], " ", 
   SubscriptBox["\[Delta]", 
    RowBox[{
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm]}]], " ", 
   FormBox[
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{
       FormBox["\[CapitalDelta]",
        TraditionalForm], "\[CenterDot]", 
       FormBox["p",
        TraditionalForm]}], ")"}], 
     RowBox[{"m", "-", "1"}]],
    TraditionalForm], " ", 
   SuperscriptBox["\[Epsilon]", 
    RowBox[{
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox["\[CapitalDelta]",
      TraditionalForm], 
     FormBox["p",
      TraditionalForm]}]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970456050006*^9},
 CellTags->"Twist2GluonOperator",
 CellLabel->"Out[1810]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Twist2GluonOperator", "[", 
  RowBox[{"p", ",", 
   RowBox[{"{", "\[Mu]", "}"}], ",", 
   RowBox[{"{", "\[Nu]", "}"}]}], "]"}]], "Input",
 CellTags->"Twist2GluonOperator",
 CellLabel->"In[1811]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubsuperscriptBox["O", 
    RowBox[{
     FormBox["\[Mu]",
      TraditionalForm], "\[VeryThinSpace]", 
     FormBox["\[Nu]",
      TraditionalForm]}], "G2"], "(", 
   FormBox["p",
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970456137594*^9},
 CellTags->"Twist2GluonOperator",
 CellLabel->"Out[1811]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Twist2GluonOperator", "[", 
  RowBox[{"p", ",", 
   RowBox[{"{", "\[Mu]", "}"}], ",", 
   RowBox[{"{", "\[Nu]", "}"}], ",", 
   RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"Twist2GluonOperator",
 CellLabel->"In[1812]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox[
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{
       FormBox["\[CapitalDelta]",
        TraditionalForm], "\[CenterDot]", 
       FormBox["p",
        TraditionalForm]}], ")"}], 
     RowBox[{"m", "-", "1"}]],
    TraditionalForm], " ", 
   SuperscriptBox["\[Epsilon]", 
    RowBox[{
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox["\[CapitalDelta]",
      TraditionalForm], 
     FormBox["p",
      TraditionalForm]}]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970456246578*^9},
 CellTags->"Twist2GluonOperator",
 CellLabel->"Out[1812]="]
}, Open  ]],

Cell["3-gluon legs", "Text",
 CellTags->"Twist2GluonOperator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Twist2GluonOperator", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"p", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"q", ",", "\[Nu]", ",", "b"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"r", ",", "\[Rho]", ",", "c"}], "}"}]}], "]"}]], "Input",
 CellTags->"Twist2GluonOperator",
 CellLabel->"In[1813]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"(", 
    RowBox[{"1", "-", 
     FormBox[
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"-", "1"}], ")"}], "m"],
      TraditionalForm]}], ")"}], " ", 
   SubscriptBox["g", "s"], " ", 
   SubscriptBox["f", 
    RowBox[{
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm]}]], " ", 
   RowBox[{"(", 
    RowBox[{
     SubsuperscriptBox["O", 
      RowBox[{
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm], "\[VeryThinSpace]", 
       FormBox[
        FormBox["\[Rho]",
         TraditionalForm],
        TraditionalForm], "\[VeryThinSpace]", 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]}], "G3"], "(", 
     FormBox["q",
      TraditionalForm], ",", 
     FormBox["r",
      TraditionalForm], ",", 
     FormBox["p",
      TraditionalForm], ")"}], ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970456423193*^9},
 CellTags->"Twist2GluonOperator",
 CellLabel->"Out[1813]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Twist2GluonOperator", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"p", ",", "\[Mu]"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"q", ",", "\[Nu]"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"r", ",", "\[Rho]"}], "}"}]}], "]"}]], "Input",
 CellTags->"Twist2GluonOperator",
 CellLabel->"In[1814]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubsuperscriptBox["O", 
    RowBox[{
     FormBox["\[Nu]",
      TraditionalForm], "\[VeryThinSpace]", 
     FormBox["\[Rho]",
      TraditionalForm], "\[VeryThinSpace]", 
     FormBox["\[Mu]",
      TraditionalForm]}], "G3"], "(", 
   FormBox["q",
    TraditionalForm], ",", 
   FormBox["r",
    TraditionalForm], ",", 
   FormBox["p",
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970456769051*^9},
 CellTags->"Twist2GluonOperator",
 CellLabel->"Out[1814]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Twist2GluonOperator", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"p", ",", "\[Mu]"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"q", ",", "\[Nu]"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"r", ",", "\[Rho]"}], "}"}], ",", 
   RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"Twist2GluonOperator",
 CellLabel->"In[1815]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SuperscriptBox[
     FormBox["\[CapitalDelta]",
      TraditionalForm], 
     FormBox[
      FormBox["\[Rho]",
       TraditionalForm],
      TraditionalForm]], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox["\[Epsilon]", 
        RowBox[{
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox["p",
          TraditionalForm], 
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox["q",
          TraditionalForm]}]]}], "+", 
      RowBox[{
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p",
         TraditionalForm]}], " ", 
       SuperscriptBox["\[Epsilon]", 
        RowBox[{
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox["q",
          TraditionalForm]}]]}]}], ")"}], " ", 
    RowBox[{"(", 
     FormBox[
      TagBox[
       RowBox[{
        UnderoverscriptBox["\[Sum]", 
         RowBox[{"i", "=", "0"}], 
         RowBox[{
          RowBox[{"-", "3"}], "+", "m"}]], 
        RowBox[{
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"-", "1"}], ")"}], "i"], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["p",
             TraditionalForm]}], ")"}], "i"], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["q",
             TraditionalForm]}], ")"}], 
          RowBox[{
           RowBox[{"-", "3"}], "-", "i", "+", "m"}]]}]}],
       HoldForm],
      TraditionalForm], ")"}]}], "-", 
   RowBox[{
    SuperscriptBox[
     FormBox["\[CapitalDelta]",
      TraditionalForm], 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm]], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox["\[Epsilon]", 
        RowBox[{
         FormBox[
          FormBox["\[Rho]",
           TraditionalForm],
          TraditionalForm], 
         FormBox["p",
          TraditionalForm], 
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox["r",
          TraditionalForm]}]]}], "+", 
      RowBox[{
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p",
         TraditionalForm]}], " ", 
       SuperscriptBox["\[Epsilon]", 
        RowBox[{
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["\[Rho]",
           TraditionalForm],
          TraditionalForm], 
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox["r",
          TraditionalForm]}]]}]}], ")"}], " ", 
    RowBox[{"(", 
     FormBox[
      TagBox[
       RowBox[{
        UnderoverscriptBox["\[Sum]", 
         RowBox[{"i", "=", "0"}], 
         RowBox[{
          RowBox[{"-", "3"}], "+", "m"}]], 
        RowBox[{
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"-", "1"}], ")"}], "i"], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["p",
             TraditionalForm]}], ")"}], "i"], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["r",
             TraditionalForm]}], ")"}], 
          RowBox[{
           RowBox[{"-", "3"}], "-", "i", "+", "m"}]]}]}],
       HoldForm],
      TraditionalForm], ")"}]}], "-", 
   RowBox[{
    SuperscriptBox[
     FormBox["\[CapitalDelta]",
      TraditionalForm], 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox["\[Epsilon]", 
        RowBox[{
         FormBox[
          FormBox["\[Rho]",
           TraditionalForm],
          TraditionalForm], 
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox["q",
          TraditionalForm], 
         FormBox["r",
          TraditionalForm]}]]}], "-", 
      RowBox[{
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["q",
         TraditionalForm]}], " ", 
       SuperscriptBox["\[Epsilon]", 
        RowBox[{
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["\[Rho]",
           TraditionalForm],
          TraditionalForm], 
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox["r",
          TraditionalForm]}]]}]}], ")"}], " ", 
    RowBox[{"(", 
     FormBox[
      TagBox[
       RowBox[{
        UnderoverscriptBox["\[Sum]", 
         RowBox[{"i", "=", "0"}], 
         RowBox[{
          RowBox[{"-", "3"}], "+", "m"}]], 
        RowBox[{
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"-", "1"}], ")"}], "i"], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["q",
             TraditionalForm]}], ")"}], "i"], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["r",
             TraditionalForm]}], ")"}], 
          RowBox[{
           RowBox[{"-", "3"}], "-", "i", "+", "m"}]]}]}],
       HoldForm],
      TraditionalForm], ")"}]}], "+", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{
       FormBox["\[CapitalDelta]",
        TraditionalForm], "\[CenterDot]", 
       FormBox["p",
        TraditionalForm]}], ")"}], 
     RowBox[{"m", "-", "2"}]], " ", 
    RowBox[{"(", 
     RowBox[{"-", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         SuperscriptBox[
          FormBox["\[CapitalDelta]",
           TraditionalForm], 
          FormBox[
           FormBox["\[Nu]",
            TraditionalForm],
           TraditionalForm]], " ", 
         SuperscriptBox["\[Epsilon]", 
          RowBox[{
           FormBox[
            FormBox["\[Mu]",
             TraditionalForm],
            TraditionalForm], 
           FormBox[
            FormBox["\[Rho]",
             TraditionalForm],
            TraditionalForm], 
           FormBox["p",
            TraditionalForm], 
           FormBox["\[CapitalDelta]",
            TraditionalForm]}]]}], "-", 
        RowBox[{
         SuperscriptBox[
          FormBox["\[CapitalDelta]",
           TraditionalForm], 
          FormBox[
           FormBox["\[Rho]",
            TraditionalForm],
           TraditionalForm]], " ", 
         SuperscriptBox["\[Epsilon]", 
          RowBox[{
           FormBox[
            FormBox["\[Mu]",
             TraditionalForm],
            TraditionalForm], 
           FormBox[
            FormBox["\[Nu]",
             TraditionalForm],
            TraditionalForm], 
           FormBox["p",
            TraditionalForm], 
           FormBox["\[CapitalDelta]",
            TraditionalForm]}]]}]}], ")"}]}], ")"}]}], "-", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{
       FormBox["\[CapitalDelta]",
        TraditionalForm], "\[CenterDot]", 
       FormBox["q",
        TraditionalForm]}], ")"}], 
     RowBox[{"m", "-", "2"}]], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox["\[Epsilon]", 
        RowBox[{
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["\[Rho]",
           TraditionalForm],
          TraditionalForm], 
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox["q",
          TraditionalForm]}]]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox["\[Epsilon]", 
        RowBox[{
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox["q",
          TraditionalForm]}]]}]}], ")"}]}], "-", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{
       FormBox["\[CapitalDelta]",
        TraditionalForm], "\[CenterDot]", 
       FormBox["r",
        TraditionalForm]}], ")"}], 
     RowBox[{"m", "-", "2"}]], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox["\[Epsilon]", 
        RowBox[{
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["\[Rho]",
           TraditionalForm],
          TraditionalForm], 
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox["r",
          TraditionalForm]}]]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox["\[Epsilon]", 
        RowBox[{
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["\[Rho]",
           TraditionalForm],
          TraditionalForm], 
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox["r",
          TraditionalForm]}]]}]}], ")"}]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970456939126*^9},
 CellTags->"Twist2GluonOperator",
 CellLabel->"Out[1815]="]
}, Open  ]],

Cell["4-gluon legs", "Text",
 CellTags->"Twist2GluonOperator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Twist2GluonOperator", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"p", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"q", ",", "\[Nu]", ",", "b"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"r", ",", "\[Rho]", ",", "c"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"s", ",", "\[Sigma]", ",", "d"}], "}"}]}], "]"}]], "Input",
 CellTags->"Twist2GluonOperator",
 CellLabel->"In[1816]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"\[ImaginaryI]", " ", 
   RowBox[{"(", 
    RowBox[{"1", "-", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}], "m"]}], ")"}], " ", 
   SubsuperscriptBox["g", "s", "2"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      RowBox[{
       SubscriptBox["f", 
        RowBox[{
         FormBox[
          FormBox["a",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["b",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["c213",
           TraditionalForm],
          TraditionalForm]}]], " ", 
       SubscriptBox["f", 
        RowBox[{
         FormBox[
          FormBox["c",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["c213",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["d",
           TraditionalForm],
          TraditionalForm]}]], " ", 
       RowBox[{"(", 
        RowBox[{
         SubsuperscriptBox["O", 
          RowBox[{
           FormBox[
            FormBox["\[Mu]",
             TraditionalForm],
            TraditionalForm], "\[VeryThinSpace]", 
           FormBox[
            FormBox["\[Nu]",
             TraditionalForm],
            TraditionalForm], "\[VeryThinSpace]", 
           FormBox[
            FormBox["\[Rho]",
             TraditionalForm],
            TraditionalForm], "\[VeryThinSpace]", 
           FormBox[
            FormBox["\[Sigma]",
             TraditionalForm],
            TraditionalForm]}], "G4"], "(", 
         FormBox["p",
          TraditionalForm], ",", 
         FormBox["q",
          TraditionalForm], ",", 
         FormBox["r",
          TraditionalForm], ",", 
         FormBox["s",
          TraditionalForm], ")"}], ")"}]}]}], "-", 
     RowBox[{
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["a",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["c",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["c213",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["b",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["c213",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["d",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      RowBox[{"(", 
       RowBox[{
        SubsuperscriptBox["O", 
         RowBox[{
          FormBox[
           FormBox["\[Mu]",
            TraditionalForm],
           TraditionalForm], "\[VeryThinSpace]", 
          FormBox[
           FormBox["\[Rho]",
            TraditionalForm],
           TraditionalForm], "\[VeryThinSpace]", 
          FormBox[
           FormBox["\[Nu]",
            TraditionalForm],
           TraditionalForm], "\[VeryThinSpace]", 
          FormBox[
           FormBox["\[Sigma]",
            TraditionalForm],
           TraditionalForm]}], "G4"], "(", 
        FormBox["p",
         TraditionalForm], ",", 
        FormBox["r",
         TraditionalForm], ",", 
        FormBox["q",
         TraditionalForm], ",", 
        FormBox["s",
         TraditionalForm], ")"}], ")"}]}], "+", 
     RowBox[{
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["a",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["c213",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["d",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["b",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["c",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["c213",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      RowBox[{"(", 
       RowBox[{
        SubsuperscriptBox["O", 
         RowBox[{
          FormBox[
           FormBox["\[Rho]",
            TraditionalForm],
           TraditionalForm], "\[VeryThinSpace]", 
          FormBox[
           FormBox["\[Nu]",
            TraditionalForm],
           TraditionalForm], "\[VeryThinSpace]", 
          FormBox[
           FormBox["\[Mu]",
            TraditionalForm],
           TraditionalForm], "\[VeryThinSpace]", 
          FormBox[
           FormBox["\[Sigma]",
            TraditionalForm],
           TraditionalForm]}], "G4"], "(", 
        FormBox["r",
         TraditionalForm], ",", 
        FormBox["q",
         TraditionalForm], ",", 
        FormBox["p",
         TraditionalForm], ",", 
        FormBox["s",
         TraditionalForm], ")"}], ")"}]}]}], ")"}]}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970457169689*^9},
 CellTags->"Twist2GluonOperator",
 CellLabel->"Out[1816]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Twist2GluonOperator", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"p", ",", "\[Mu]"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"q", ",", "\[Nu]"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"r", ",", "\[Rho]"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"s", ",", "\[Sigma]"}], "}"}]}], "]"}]], "Input",
 CellTags->"Twist2GluonOperator",
 CellLabel->"In[1817]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubsuperscriptBox["O", 
    RowBox[{
     FormBox["\[Mu]",
      TraditionalForm], "\[VeryThinSpace]", 
     FormBox["\[Nu]",
      TraditionalForm], "\[VeryThinSpace]", 
     FormBox["\[Rho]",
      TraditionalForm], "\[VeryThinSpace]", 
     FormBox["\[Sigma]",
      TraditionalForm]}], "G4"], "(", 
   FormBox["p",
    TraditionalForm], ",", 
   FormBox["q",
    TraditionalForm], ",", 
   FormBox["r",
    TraditionalForm], ",", 
   FormBox["s",
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970457341077*^9},
 CellTags->"Twist2GluonOperator",
 CellLabel->"Out[1817]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Twist2GluonOperator", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"p", ",", "\[Mu]"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"q", ",", "\[Nu]"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"r", ",", "\[Rho]"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"s", ",", "\[Sigma]"}], "}"}], ",", 
   RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"Twist2GluonOperator",
 CellLabel->"In[1818]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox["\[Epsilon]", 
        RowBox[{
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["\[Rho]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["\[Sigma]",
           TraditionalForm],
          TraditionalForm]}]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]]}], "-", 
      RowBox[{
       SuperscriptBox["\[Epsilon]", 
        RowBox[{
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["\[Rho]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["\[Sigma]",
           TraditionalForm],
          TraditionalForm]}]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]]}]}], ")"}], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["r",
         TraditionalForm]}], "+", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["s",
         TraditionalForm]}]}], ")"}], 
     RowBox[{"m", "-", "2"}]]}], "-", 
   RowBox[{
    RowBox[{"(", 
     FormBox[
      TagBox[
       RowBox[{
        UnderoverscriptBox["\[Sum]", 
         RowBox[{"i", "=", "0"}], 
         RowBox[{
          RowBox[{"-", "3"}], "+", "m"}]], 
        RowBox[{
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            RowBox[{"-", 
             RowBox[{"(", 
              RowBox[{
               FormBox["\[CapitalDelta]",
                TraditionalForm], "\[CenterDot]", 
               FormBox["p",
                TraditionalForm]}], ")"}]}], "-", 
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["q",
              TraditionalForm]}]}], ")"}], "i"], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["s",
             TraditionalForm]}], ")"}], 
          RowBox[{
           RowBox[{"-", "3"}], "-", "i", "+", "m"}]]}]}],
       HoldForm],
      TraditionalForm], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox["\[Epsilon]", 
        RowBox[{
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["\[Sigma]",
           TraditionalForm],
          TraditionalForm], 
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox["s",
          TraditionalForm]}]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]]}], "-", 
      RowBox[{
       SuperscriptBox["\[Epsilon]", 
        RowBox[{
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["\[Sigma]",
           TraditionalForm],
          TraditionalForm], 
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox["s",
          TraditionalForm]}]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]]}]}], ")"}], " ", 
    SuperscriptBox[
     FormBox["\[CapitalDelta]",
      TraditionalForm], 
     FormBox[
      FormBox["\[Rho]",
       TraditionalForm],
      TraditionalForm]]}], "+", 
   RowBox[{
    RowBox[{"(", 
     FormBox[
      TagBox[
       RowBox[{
        UnderoverscriptBox["\[Sum]", 
         RowBox[{"i", "=", "0"}], 
         RowBox[{
          RowBox[{"-", "3"}], "+", "m"}]], 
        RowBox[{
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["p",
              TraditionalForm]}], "+", 
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["q",
              TraditionalForm]}]}], ")"}], 
          RowBox[{
           RowBox[{"-", "3"}], "-", "i", "+", "m"}]], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"-", 
            RowBox[{"(", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["r",
               TraditionalForm]}], ")"}]}], ")"}], "i"]}]}],
       HoldForm],
      TraditionalForm], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox["\[Epsilon]", 
        RowBox[{
         FormBox[
          FormBox["\[Rho]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox["r",
          TraditionalForm], 
         FormBox["\[CapitalDelta]",
          TraditionalForm]}]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]]}], "-", 
      RowBox[{
       SuperscriptBox["\[Epsilon]", 
        RowBox[{
         FormBox[
          FormBox["\[Rho]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox["r",
          TraditionalForm], 
         FormBox["\[CapitalDelta]",
          TraditionalForm]}]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]]}]}], ")"}], " ", 
    SuperscriptBox[
     FormBox["\[CapitalDelta]",
      TraditionalForm], 
     FormBox[
      FormBox["\[Sigma]",
       TraditionalForm],
      TraditionalForm]]}], "-", 
   RowBox[{
    RowBox[{"(", 
     FormBox[
      TagBox[
       RowBox[{
        UnderoverscriptBox["\[Sum]", 
         RowBox[{"i", "=", "0"}], 
         RowBox[{
          RowBox[{"-", "3"}], "+", "m"}]], 
        RowBox[{
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"-", 
            RowBox[{"(", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["p",
               TraditionalForm]}], ")"}]}], ")"}], "i"], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["r",
              TraditionalForm]}], "+", 
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["s",
              TraditionalForm]}]}], ")"}], 
          RowBox[{
           RowBox[{"-", "3"}], "-", "i", "+", "m"}]]}]}],
       HoldForm],
      TraditionalForm], ")"}], " ", 
    SuperscriptBox[
     FormBox["\[CapitalDelta]",
      TraditionalForm], 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm]], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox["\[Epsilon]", 
        RowBox[{
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["\[Sigma]",
           TraditionalForm],
          TraditionalForm], 
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox["p",
          TraditionalForm]}]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm]]}], "-", 
      RowBox[{
       SuperscriptBox["\[Epsilon]", 
        RowBox[{
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["\[Rho]",
           TraditionalForm],
          TraditionalForm], 
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox["p",
          TraditionalForm]}]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Sigma]",
          TraditionalForm],
         TraditionalForm]]}]}], ")"}]}], "+", 
   RowBox[{
    RowBox[{"(", 
     FormBox[
      TagBox[
       RowBox[{
        UnderoverscriptBox["\[Sum]", 
         RowBox[{"i", "=", "0"}], 
         RowBox[{
          RowBox[{"-", "3"}], "+", "m"}]], 
        RowBox[{
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"-", 
            RowBox[{"(", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["q",
               TraditionalForm]}], ")"}]}], ")"}], "i"], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["r",
              TraditionalForm]}], "+", 
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["s",
              TraditionalForm]}]}], ")"}], 
          RowBox[{
           RowBox[{"-", "3"}], "-", "i", "+", "m"}]]}]}],
       HoldForm],
      TraditionalForm], ")"}], " ", 
    SuperscriptBox[
     FormBox["\[CapitalDelta]",
      TraditionalForm], 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox["\[Epsilon]", 
        RowBox[{
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["\[Sigma]",
           TraditionalForm],
          TraditionalForm], 
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox["q",
          TraditionalForm]}]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm]]}], "-", 
      RowBox[{
       SuperscriptBox["\[Epsilon]", 
        RowBox[{
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["\[Rho]",
           TraditionalForm],
          TraditionalForm], 
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox["q",
          TraditionalForm]}]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Sigma]",
          TraditionalForm],
         TraditionalForm]]}]}], ")"}]}], "-", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      FormBox[
       TagBox[
        RowBox[{
         UnderoverscriptBox["\[Sum]", 
          RowBox[{"j", "=", "0"}], 
          RowBox[{
           RowBox[{"-", "4"}], "+", "m"}]], "\<\"\"\>"}],
        HoldForm],
       TraditionalForm], 
      FormBox[
       RowBox[{"\<\"\"\>", " ", 
        RowBox[{"(", 
         RowBox[{"j", "+", "1"}], ")"}]}],
       TraditionalForm], 
      FormBox[
       RowBox[{
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["p",
            TraditionalForm]}], ")"}], 
         RowBox[{
          RowBox[{"-", "j"}], "+", "m", "-", "4"}]], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           RowBox[{"-", 
            RowBox[{"(", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["r",
               TraditionalForm]}], ")"}]}], "-", 
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["s",
             TraditionalForm]}]}], ")"}], 
         RowBox[{"j", "-", "i"}]], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"-", 
           RowBox[{"(", 
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["s",
              TraditionalForm]}], ")"}]}], ")"}], "i"]}],
       TraditionalForm]}], ")"}], " ", 
    SuperscriptBox[
     FormBox["\[CapitalDelta]",
      TraditionalForm], 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm]], " ", 
    SuperscriptBox[
     FormBox["\[CapitalDelta]",
      TraditionalForm], 
     FormBox[
      FormBox["\[Rho]",
       TraditionalForm],
      TraditionalForm]], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"-", 
        SuperscriptBox["\[Epsilon]", 
         RowBox[{
          FormBox["\[CapitalDelta]",
           TraditionalForm], 
          FormBox[
           FormBox["\[Sigma]",
            TraditionalForm],
           TraditionalForm], 
          FormBox["p",
           TraditionalForm], 
          FormBox["s",
           TraditionalForm]}]]}], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]]}], "-", 
      RowBox[{
       SuperscriptBox["\[Epsilon]", 
        RowBox[{
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["\[Sigma]",
           TraditionalForm],
          TraditionalForm], 
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox["s",
          TraditionalForm]}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p",
         TraditionalForm]}]}]}], ")"}]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      FormBox[
       TagBox[
        RowBox[{
         UnderoverscriptBox["\[Sum]", 
          RowBox[{"j", "=", "0"}], 
          RowBox[{
           RowBox[{"-", "4"}], "+", "m"}]], "\<\"\"\>"}],
        HoldForm],
       TraditionalForm], 
      FormBox[
       RowBox[{"\<\"\"\>", " ", 
        RowBox[{"(", 
         RowBox[{"j", "+", "1"}], ")"}]}],
       TraditionalForm], 
      FormBox[
       RowBox[{
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["q",
            TraditionalForm]}], ")"}], 
         RowBox[{
          RowBox[{"-", "j"}], "+", "m", "-", "4"}]], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           RowBox[{"-", 
            RowBox[{"(", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["r",
               TraditionalForm]}], ")"}]}], "-", 
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["s",
             TraditionalForm]}]}], ")"}], 
         RowBox[{"j", "-", "i"}]], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"-", 
           RowBox[{"(", 
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["s",
              TraditionalForm]}], ")"}]}], ")"}], "i"]}],
       TraditionalForm]}], ")"}], " ", 
    SuperscriptBox[
     FormBox["\[CapitalDelta]",
      TraditionalForm], 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], " ", 
    SuperscriptBox[
     FormBox["\[CapitalDelta]",
      TraditionalForm], 
     FormBox[
      FormBox["\[Rho]",
       TraditionalForm],
      TraditionalForm]], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"-", 
        SuperscriptBox["\[Epsilon]", 
         RowBox[{
          FormBox["\[CapitalDelta]",
           TraditionalForm], 
          FormBox[
           FormBox["\[Sigma]",
            TraditionalForm],
           TraditionalForm], 
          FormBox["q",
           TraditionalForm], 
          FormBox["s",
           TraditionalForm]}]]}], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]]}], "-", 
      RowBox[{
       SuperscriptBox["\[Epsilon]", 
        RowBox[{
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["\[Sigma]",
           TraditionalForm],
          TraditionalForm], 
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox["s",
          TraditionalForm]}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["q",
         TraditionalForm]}]}]}], ")"}]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      FormBox[
       TagBox[
        RowBox[{
         UnderoverscriptBox["\[Sum]", 
          RowBox[{"j", "=", "0"}], 
          RowBox[{
           RowBox[{"-", "4"}], "+", "m"}]], "\<\"\"\>"}],
        HoldForm],
       TraditionalForm], 
      FormBox[
       RowBox[{"\<\"\"\>", " ", 
        RowBox[{"(", 
         RowBox[{"j", "+", "1"}], ")"}]}],
       TraditionalForm], 
      FormBox[
       RowBox[{
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["p",
            TraditionalForm]}], ")"}], 
         RowBox[{
          RowBox[{"-", "j"}], "+", "m", "-", "4"}]], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"-", 
           RowBox[{"(", 
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["r",
              TraditionalForm]}], ")"}]}], ")"}], "i"], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           RowBox[{"-", 
            RowBox[{"(", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["r",
               TraditionalForm]}], ")"}]}], "-", 
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["s",
             TraditionalForm]}]}], ")"}], 
         RowBox[{"j", "-", "i"}]]}],
       TraditionalForm]}], ")"}], " ", 
    SuperscriptBox[
     FormBox["\[CapitalDelta]",
      TraditionalForm], 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm]], " ", 
    SuperscriptBox[
     FormBox["\[CapitalDelta]",
      TraditionalForm], 
     FormBox[
      FormBox["\[Sigma]",
       TraditionalForm],
      TraditionalForm]], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"-", 
        SuperscriptBox["\[Epsilon]", 
         RowBox[{
          FormBox["\[CapitalDelta]",
           TraditionalForm], 
          FormBox[
           FormBox["\[Mu]",
            TraditionalForm],
           TraditionalForm], 
          FormBox["p",
           TraditionalForm], 
          FormBox["r",
           TraditionalForm]}]]}], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm]]}], "-", 
      RowBox[{
       SuperscriptBox["\[Epsilon]", 
        RowBox[{
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["\[Rho]",
           TraditionalForm],
          TraditionalForm], 
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox["p",
          TraditionalForm]}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["r",
         TraditionalForm]}]}]}], ")"}]}], "-", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      FormBox[
       TagBox[
        RowBox[{
         UnderoverscriptBox["\[Sum]", 
          RowBox[{"j", "=", "0"}], 
          RowBox[{
           RowBox[{"-", "4"}], "+", "m"}]], "\<\"\"\>"}],
        HoldForm],
       TraditionalForm], 
      FormBox[
       RowBox[{"\<\"\"\>", " ", 
        RowBox[{"(", 
         RowBox[{"j", "+", "1"}], ")"}]}],
       TraditionalForm], 
      FormBox[
       RowBox[{
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["q",
            TraditionalForm]}], ")"}], 
         RowBox[{
          RowBox[{"-", "j"}], "+", "m", "-", "4"}]], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"-", 
           RowBox[{"(", 
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["r",
              TraditionalForm]}], ")"}]}], ")"}], "i"], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           RowBox[{"-", 
            RowBox[{"(", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["r",
               TraditionalForm]}], ")"}]}], "-", 
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["s",
             TraditionalForm]}]}], ")"}], 
         RowBox[{"j", "-", "i"}]]}],
       TraditionalForm]}], ")"}], " ", 
    SuperscriptBox[
     FormBox["\[CapitalDelta]",
      TraditionalForm], 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], " ", 
    SuperscriptBox[
     FormBox["\[CapitalDelta]",
      TraditionalForm], 
     FormBox[
      FormBox["\[Sigma]",
       TraditionalForm],
      TraditionalForm]], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"-", 
        SuperscriptBox["\[Epsilon]", 
         RowBox[{
          FormBox["\[CapitalDelta]",
           TraditionalForm], 
          FormBox[
           FormBox["\[Nu]",
            TraditionalForm],
           TraditionalForm], 
          FormBox["q",
           TraditionalForm], 
          FormBox["r",
           TraditionalForm]}]]}], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm]]}], "-", 
      RowBox[{
       SuperscriptBox["\[Epsilon]", 
        RowBox[{
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["\[Rho]",
           TraditionalForm],
          TraditionalForm], 
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox["q",
          TraditionalForm]}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["r",
         TraditionalForm]}]}]}], ")"}]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.50097045750261*^9},
 CellTags->"Twist2GluonOperator",
 CellLabel->"Out[1818]="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Unpolarized case, zero-momentum insertion", "Subsubsection",
 CellTags->"Twist2GluonOperator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SetOptions", "[", 
  RowBox[{"Twist2GluonOperator", ",", 
   RowBox[{"Polarization", "\[Rule]", "0"}], ",", 
   RowBox[{"ZeroMomentumInsertion", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"Twist2GluonOperator",
 CellLabel->"In[1819]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"CouplingConstant", "\[Rule]", 
     SubscriptBox["g", "s"]}], ",", 
    RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
    RowBox[{"Polarization", "\[Rule]", "0"}], ",", 
    RowBox[{"Explicit", "\[Rule]", "False"}], ",", 
    RowBox[{"ZeroMomentumInsertion", "\[Rule]", "True"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970457613934*^9},
 CellTags->"Twist2GluonOperator",
 CellLabel->"Out[1819]="]
}, Open  ]],

Cell["2-gluon legs", "Text",
 CellTags->"Twist2GluonOperator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Twist2GluonOperator", "[", 
  RowBox[{"p", ",", 
   RowBox[{"{", 
    RowBox[{"\[Mu]", ",", "a"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\[Nu]", ",", "b"}], "}"}]}], "]"}]], "Input",
 CellTags->"Twist2GluonOperator",
 CellLabel->"In[1820]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["1", "2"], " ", 
   RowBox[{"(", 
    RowBox[{
     FormBox[
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"-", "1"}], ")"}], "m"],
      TraditionalForm], "+", "1"}], ")"}], " ", 
   SubscriptBox["\[Delta]", 
    RowBox[{
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm]}]], " ", 
   RowBox[{"(", 
    RowBox[{
     SubsuperscriptBox["O", 
      RowBox[{
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm], "\[VeryThinSpace]", 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]}], "G2"], "(", 
     FormBox["p",
      TraditionalForm], ")"}], ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970457746477*^9},
 CellTags->"Twist2GluonOperator",
 CellLabel->"Out[1820]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Twist2GluonOperator", "[", 
  RowBox[{"p", ",", 
   RowBox[{"{", 
    RowBox[{"\[Mu]", ",", "a"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\[Nu]", ",", "b"}], "}"}], ",", 
   RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"Twist2GluonOperator",
 CellLabel->"In[1821]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["1", "2"], " ", 
   RowBox[{"(", 
    RowBox[{
     FormBox[
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"-", "1"}], ")"}], "m"],
      TraditionalForm], "+", "1"}], ")"}], " ", 
   SubscriptBox["\[Delta]", 
    RowBox[{
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm]}]], " ", 
   FormBox[
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{
       FormBox["\[CapitalDelta]",
        TraditionalForm], "\[CenterDot]", 
       FormBox["p",
        TraditionalForm]}], ")"}], 
     RowBox[{"m", "-", "2"}]],
    TraditionalForm], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
      SuperscriptBox[
       FormBox[
        RowBox[{
         FormBox["\[CapitalDelta]",
          TraditionalForm], "\[CenterDot]", 
         FormBox["p",
          TraditionalForm]}],
        TraditionalForm], "2"]}], "+", 
     RowBox[{
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], "2"], " ", 
      SuperscriptBox[
       FormBox["\[CapitalDelta]",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox[
       FormBox["\[CapitalDelta]",
        TraditionalForm], 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]]}], "-", 
     RowBox[{
      RowBox[{
       FormBox["\[CapitalDelta]",
        TraditionalForm], "\[CenterDot]", 
       FormBox["p",
        TraditionalForm]}], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         SuperscriptBox[
          FormBox["\[CapitalDelta]",
           TraditionalForm], 
          FormBox[
           FormBox["\[Nu]",
            TraditionalForm],
           TraditionalForm]], " ", 
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], 
          FormBox[
           FormBox["\[Mu]",
            TraditionalForm],
           TraditionalForm]]}], "+", 
        RowBox[{
         SuperscriptBox[
          FormBox["\[CapitalDelta]",
           TraditionalForm], 
          FormBox[
           FormBox["\[Mu]",
            TraditionalForm],
           TraditionalForm]], " ", 
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], 
          FormBox[
           FormBox["\[Nu]",
            TraditionalForm],
           TraditionalForm]]}]}], ")"}]}]}], ")"}]}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970458214603*^9},
 CellTags->"Twist2GluonOperator",
 CellLabel->"Out[1821]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Twist2GluonOperator", "[", 
  RowBox[{"p", ",", 
   RowBox[{"{", "\[Mu]", "}"}], ",", 
   RowBox[{"{", "\[Nu]", "}"}]}], "]"}]], "Input",
 CellTags->"Twist2GluonOperator",
 CellLabel->"In[1822]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubsuperscriptBox["O", 
    RowBox[{
     FormBox["\[Mu]",
      TraditionalForm], "\[VeryThinSpace]", 
     FormBox["\[Nu]",
      TraditionalForm]}], "G2"], "(", 
   FormBox["p",
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009704583690453`*^9},
 CellTags->"Twist2GluonOperator",
 CellLabel->"Out[1822]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Twist2GluonOperator", "[", 
  RowBox[{"p", ",", 
   RowBox[{"{", "\[Mu]", "}"}], ",", 
   RowBox[{"{", "\[Nu]", "}"}], ",", 
   RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"Twist2GluonOperator",
 CellLabel->"In[1823]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox[
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{
       FormBox["\[CapitalDelta]",
        TraditionalForm], "\[CenterDot]", 
       FormBox["p",
        TraditionalForm]}], ")"}], 
     RowBox[{"m", "-", "2"}]],
    TraditionalForm], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
      SuperscriptBox[
       FormBox[
        RowBox[{
         FormBox["\[CapitalDelta]",
          TraditionalForm], "\[CenterDot]", 
         FormBox["p",
          TraditionalForm]}],
        TraditionalForm], "2"]}], "+", 
     RowBox[{
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], "2"], " ", 
      SuperscriptBox[
       FormBox["\[CapitalDelta]",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox[
       FormBox["\[CapitalDelta]",
        TraditionalForm], 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]]}], "-", 
     RowBox[{
      RowBox[{
       FormBox["\[CapitalDelta]",
        TraditionalForm], "\[CenterDot]", 
       FormBox["p",
        TraditionalForm]}], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         SuperscriptBox[
          FormBox["\[CapitalDelta]",
           TraditionalForm], 
          FormBox[
           FormBox["\[Nu]",
            TraditionalForm],
           TraditionalForm]], " ", 
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], 
          FormBox[
           FormBox["\[Mu]",
            TraditionalForm],
           TraditionalForm]]}], "+", 
        RowBox[{
         SuperscriptBox[
          FormBox["\[CapitalDelta]",
           TraditionalForm], 
          FormBox[
           FormBox["\[Mu]",
            TraditionalForm],
           TraditionalForm]], " ", 
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], 
          FormBox[
           FormBox["\[Nu]",
            TraditionalForm],
           TraditionalForm]]}]}], ")"}]}]}], ")"}]}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.5009704585378313`*^9},
 CellTags->"Twist2GluonOperator",
 CellLabel->"Out[1823]="]
}, Open  ]],

Cell["3-gluon legs", "Text",
 CellTags->"Twist2GluonOperator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Twist2GluonOperator", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"p", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"q", ",", "\[Nu]", ",", "b"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"r", ",", "\[Rho]", ",", "c"}], "}"}]}], "]"}]], "Input",
 CellTags->"Twist2GluonOperator",
 CellLabel->"In[1824]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    FractionBox["1", "2"]}], " ", "\[ImaginaryI]", " ", 
   RowBox[{"(", 
    RowBox[{
     FormBox[
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"-", "1"}], ")"}], "m"],
      TraditionalForm], "+", "1"}], ")"}], " ", 
   SubscriptBox["g", "s"], " ", 
   SubscriptBox["f", 
    RowBox[{
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm]}]], " ", 
   RowBox[{"(", 
    RowBox[{
     SubsuperscriptBox["O", 
      RowBox[{
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm], "\[VeryThinSpace]", 
       FormBox[
        FormBox["\[Rho]",
         TraditionalForm],
        TraditionalForm], "\[VeryThinSpace]", 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]}], "G3"], "(", 
     FormBox["q",
      TraditionalForm], ",", 
     FormBox["r",
      TraditionalForm], ",", 
     FormBox["p",
      TraditionalForm], ")"}], ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009704586437607`*^9},
 CellTags->"Twist2GluonOperator",
 CellLabel->"Out[1824]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Twist2GluonOperator", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"p", ",", "\[Mu]"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"q", ",", "\[Nu]"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"r", ",", "\[Rho]"}], "}"}]}], "]"}]], "Input",
 CellTags->"Twist2GluonOperator",
 CellLabel->"In[1825]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubsuperscriptBox["O", 
    RowBox[{
     FormBox["\[Nu]",
      TraditionalForm], "\[VeryThinSpace]", 
     FormBox["\[Rho]",
      TraditionalForm], "\[VeryThinSpace]", 
     FormBox["\[Mu]",
      TraditionalForm]}], "G3"], "(", 
   FormBox["q",
    TraditionalForm], ",", 
   FormBox["r",
    TraditionalForm], ",", 
   FormBox["p",
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970458863352*^9},
 CellTags->"Twist2GluonOperator",
 CellLabel->"Out[1825]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Twist2GluonOperator", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"p", ",", "\[Mu]"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"q", ",", "\[Nu]"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"r", ",", "\[Rho]"}], "}"}], ",", 
   RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"Twist2GluonOperator",
 CellLabel->"In[1826]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SuperscriptBox[
     FormBox["\[CapitalDelta]",
      TraditionalForm], 
     FormBox[
      FormBox["\[Rho]",
       TraditionalForm],
      TraditionalForm]], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", 
       RowBox[{
        SuperscriptBox["g", 
         RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
        RowBox[{
         FormBox["\[CapitalDelta]",
          TraditionalForm], "\[CenterDot]", 
         FormBox["p",
          TraditionalForm]}], " ", 
        RowBox[{
         FormBox["\[CapitalDelta]",
          TraditionalForm], "\[CenterDot]", 
         FormBox["q",
          TraditionalForm]}]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["q",
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["q",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p",
         TraditionalForm], "\[CenterDot]", 
        FormBox["q",
         TraditionalForm]}]}]}], ")"}], " ", 
    RowBox[{"(", 
     FormBox[
      TagBox[
       RowBox[{
        UnderoverscriptBox["\[Sum]", 
         RowBox[{"i", "=", "0"}], 
         RowBox[{
          RowBox[{"-", "3"}], "+", "m"}]], 
        RowBox[{
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"-", 
            RowBox[{"(", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["p",
               TraditionalForm]}], ")"}]}], ")"}], "i"], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["q",
             TraditionalForm]}], ")"}], 
          RowBox[{
           RowBox[{"-", "3"}], "-", "i", "+", "m"}]]}]}],
       HoldForm],
      TraditionalForm], ")"}]}], "+", 
   RowBox[{
    SuperscriptBox[
     FormBox["\[CapitalDelta]",
      TraditionalForm], 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm]], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", 
       RowBox[{
        SuperscriptBox["g", 
         RowBox[{"\[Mu]", "\[Rho]"}]], " ", 
        RowBox[{
         FormBox["\[CapitalDelta]",
          TraditionalForm], "\[CenterDot]", 
         FormBox["p",
          TraditionalForm]}], " ", 
        RowBox[{
         FormBox["\[CapitalDelta]",
          TraditionalForm], "\[CenterDot]", 
         FormBox["r",
          TraditionalForm]}]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["r",
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["r",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p",
         TraditionalForm], "\[CenterDot]", 
        FormBox["r",
         TraditionalForm]}]}]}], ")"}], " ", 
    RowBox[{"(", 
     FormBox[
      TagBox[
       RowBox[{
        UnderoverscriptBox["\[Sum]", 
         RowBox[{"i", "=", "0"}], 
         RowBox[{
          RowBox[{"-", "3"}], "+", "m"}]], 
        RowBox[{
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["p",
             TraditionalForm]}], ")"}], 
          RowBox[{
           RowBox[{"-", "3"}], "-", "i", "+", "m"}]], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"-", 
            RowBox[{"(", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["r",
               TraditionalForm]}], ")"}]}], ")"}], "i"]}]}],
       HoldForm],
      TraditionalForm], ")"}]}], "+", 
   RowBox[{
    SuperscriptBox[
     FormBox["\[CapitalDelta]",
      TraditionalForm], 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", 
       RowBox[{
        SuperscriptBox["g", 
         RowBox[{"\[Nu]", "\[Rho]"}]], " ", 
        RowBox[{
         FormBox["\[CapitalDelta]",
          TraditionalForm], "\[CenterDot]", 
         FormBox["q",
          TraditionalForm]}], " ", 
        RowBox[{
         FormBox["\[CapitalDelta]",
          TraditionalForm], "\[CenterDot]", 
         FormBox["r",
          TraditionalForm]}]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["r",
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["q",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["q",
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["r",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["q",
         TraditionalForm], "\[CenterDot]", 
        FormBox["r",
         TraditionalForm]}]}]}], ")"}], " ", 
    RowBox[{"(", 
     FormBox[
      TagBox[
       RowBox[{
        UnderoverscriptBox["\[Sum]", 
         RowBox[{"i", "=", "0"}], 
         RowBox[{
          RowBox[{"-", "3"}], "+", "m"}]], 
        RowBox[{
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"-", 
            RowBox[{"(", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["q",
               TraditionalForm]}], ")"}]}], ")"}], "i"], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["r",
             TraditionalForm]}], ")"}], 
          RowBox[{
           RowBox[{"-", "3"}], "-", "i", "+", "m"}]]}]}],
       HoldForm],
      TraditionalForm], ")"}]}], "+", 
   RowBox[{
    FormBox[
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p",
         TraditionalForm]}], ")"}], 
      RowBox[{"m", "-", "2"}]],
     TraditionalForm], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p",
         TraditionalForm]}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          SuperscriptBox[
           FormBox["\[CapitalDelta]",
            TraditionalForm], 
           FormBox[
            FormBox["\[Nu]",
             TraditionalForm],
            TraditionalForm]], " ", 
          SuperscriptBox["g", 
           RowBox[{"\[Mu]", "\[Rho]"}]]}], "-", 
         RowBox[{
          SuperscriptBox[
           FormBox["\[CapitalDelta]",
            TraditionalForm], 
           FormBox[
            FormBox["\[Rho]",
             TraditionalForm],
            TraditionalForm]], " ", 
          SuperscriptBox["g", 
           RowBox[{"\[Mu]", "\[Nu]"}]]}]}], ")"}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          SuperscriptBox[
           FormBox["\[CapitalDelta]",
            TraditionalForm], 
           FormBox[
            FormBox["\[Rho]",
             TraditionalForm],
            TraditionalForm]], " ", 
          SuperscriptBox[
           FormBox["p",
            TraditionalForm], 
           FormBox[
            FormBox["\[Nu]",
             TraditionalForm],
            TraditionalForm]]}], "-", 
         RowBox[{
          SuperscriptBox[
           FormBox["\[CapitalDelta]",
            TraditionalForm], 
           FormBox[
            FormBox["\[Nu]",
             TraditionalForm],
            TraditionalForm]], " ", 
          SuperscriptBox[
           FormBox["p",
            TraditionalForm], 
           FormBox[
            FormBox["\[Rho]",
             TraditionalForm],
            TraditionalForm]]}]}], ")"}]}]}], ")"}]}], "+", 
   RowBox[{
    FormBox[
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["q",
         TraditionalForm]}], ")"}], 
      RowBox[{"m", "-", "2"}]],
     TraditionalForm], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["q",
         TraditionalForm]}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          SuperscriptBox[
           FormBox["\[CapitalDelta]",
            TraditionalForm], 
           FormBox[
            FormBox["\[Rho]",
             TraditionalForm],
            TraditionalForm]], " ", 
          SuperscriptBox["g", 
           RowBox[{"\[Mu]", "\[Nu]"}]]}], "-", 
         RowBox[{
          SuperscriptBox[
           FormBox["\[CapitalDelta]",
            TraditionalForm], 
           FormBox[
            FormBox["\[Mu]",
             TraditionalForm],
            TraditionalForm]], " ", 
          SuperscriptBox["g", 
           RowBox[{"\[Nu]", "\[Rho]"}]]}]}], ")"}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          SuperscriptBox[
           FormBox["\[CapitalDelta]",
            TraditionalForm], 
           FormBox[
            FormBox["\[Mu]",
             TraditionalForm],
            TraditionalForm]], " ", 
          SuperscriptBox[
           FormBox["q",
            TraditionalForm], 
           FormBox[
            FormBox["\[Rho]",
             TraditionalForm],
            TraditionalForm]]}], "-", 
         RowBox[{
          SuperscriptBox[
           FormBox["\[CapitalDelta]",
            TraditionalForm], 
           FormBox[
            FormBox["\[Rho]",
             TraditionalForm],
            TraditionalForm]], " ", 
          SuperscriptBox[
           FormBox["q",
            TraditionalForm], 
           FormBox[
            FormBox["\[Mu]",
             TraditionalForm],
            TraditionalForm]]}]}], ")"}]}]}], ")"}]}], "+", 
   RowBox[{
    FormBox[
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["r",
         TraditionalForm]}], ")"}], 
      RowBox[{"m", "-", "2"}]],
     TraditionalForm], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["r",
         TraditionalForm]}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          SuperscriptBox[
           FormBox["\[CapitalDelta]",
            TraditionalForm], 
           FormBox[
            FormBox["\[Mu]",
             TraditionalForm],
            TraditionalForm]], " ", 
          SuperscriptBox["g", 
           RowBox[{"\[Nu]", "\[Rho]"}]]}], "-", 
         RowBox[{
          SuperscriptBox[
           FormBox["\[CapitalDelta]",
            TraditionalForm], 
           FormBox[
            FormBox["\[Nu]",
             TraditionalForm],
            TraditionalForm]], " ", 
          SuperscriptBox["g", 
           RowBox[{"\[Mu]", "\[Rho]"}]]}]}], ")"}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          SuperscriptBox[
           FormBox["\[CapitalDelta]",
            TraditionalForm], 
           FormBox[
            FormBox["\[Nu]",
             TraditionalForm],
            TraditionalForm]], " ", 
          SuperscriptBox[
           FormBox["r",
            TraditionalForm], 
           FormBox[
            FormBox["\[Mu]",
             TraditionalForm],
            TraditionalForm]]}], "-", 
         RowBox[{
          SuperscriptBox[
           FormBox["\[CapitalDelta]",
            TraditionalForm], 
           FormBox[
            FormBox["\[Mu]",
             TraditionalForm],
            TraditionalForm]], " ", 
          SuperscriptBox[
           FormBox["r",
            TraditionalForm], 
           FormBox[
            FormBox["\[Nu]",
             TraditionalForm],
            TraditionalForm]]}]}], ")"}]}]}], ")"}]}]}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.5009704591243687`*^9},
 CellTags->"Twist2GluonOperator",
 CellLabel->"Out[1826]="]
}, Open  ]],

Cell["4-gluon legs", "Text",
 CellTags->"Twist2GluonOperator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Twist2GluonOperator", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"p", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"q", ",", "\[Nu]", ",", "b"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"r", ",", "\[Rho]", ",", "c"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"s", ",", "\[Sigma]", ",", "d"}], "}"}]}], "]"}]], "Input",
 CellTags->"Twist2GluonOperator",
 CellLabel->"In[1827]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["1", "2"], " ", 
   RowBox[{"(", 
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}], "m"], "+", "1"}], ")"}], " ", 
   SubsuperscriptBox["g", "s", "2"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      RowBox[{
       SubscriptBox["f", 
        RowBox[{
         FormBox[
          FormBox["a",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["b",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["c214",
           TraditionalForm],
          TraditionalForm]}]], " ", 
       SubscriptBox["f", 
        RowBox[{
         FormBox[
          FormBox["c",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["c214",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["d",
           TraditionalForm],
          TraditionalForm]}]], " ", 
       RowBox[{"(", 
        RowBox[{
         SubsuperscriptBox["O", 
          RowBox[{
           FormBox[
            FormBox["\[Mu]",
             TraditionalForm],
            TraditionalForm], "\[VeryThinSpace]", 
           FormBox[
            FormBox["\[Nu]",
             TraditionalForm],
            TraditionalForm], "\[VeryThinSpace]", 
           FormBox[
            FormBox["\[Rho]",
             TraditionalForm],
            TraditionalForm], "\[VeryThinSpace]", 
           FormBox[
            FormBox["\[Sigma]",
             TraditionalForm],
            TraditionalForm]}], "G4"], "(", 
         FormBox["p",
          TraditionalForm], ",", 
         FormBox["q",
          TraditionalForm], ",", 
         FormBox["r",
          TraditionalForm], ",", 
         FormBox["s",
          TraditionalForm], ")"}], ")"}]}]}], "-", 
     RowBox[{
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["a",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["c",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["c214",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["b",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["c214",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["d",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      RowBox[{"(", 
       RowBox[{
        SubsuperscriptBox["O", 
         RowBox[{
          FormBox[
           FormBox["\[Mu]",
            TraditionalForm],
           TraditionalForm], "\[VeryThinSpace]", 
          FormBox[
           FormBox["\[Rho]",
            TraditionalForm],
           TraditionalForm], "\[VeryThinSpace]", 
          FormBox[
           FormBox["\[Nu]",
            TraditionalForm],
           TraditionalForm], "\[VeryThinSpace]", 
          FormBox[
           FormBox["\[Sigma]",
            TraditionalForm],
           TraditionalForm]}], "G4"], "(", 
        FormBox["p",
         TraditionalForm], ",", 
        FormBox["r",
         TraditionalForm], ",", 
        FormBox["q",
         TraditionalForm], ",", 
        FormBox["s",
         TraditionalForm], ")"}], ")"}]}], "+", 
     RowBox[{
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["a",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["c214",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["d",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["b",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["c",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["c214",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      RowBox[{"(", 
       RowBox[{
        SubsuperscriptBox["O", 
         RowBox[{
          FormBox[
           FormBox["\[Rho]",
            TraditionalForm],
           TraditionalForm], "\[VeryThinSpace]", 
          FormBox[
           FormBox["\[Nu]",
            TraditionalForm],
           TraditionalForm], "\[VeryThinSpace]", 
          FormBox[
           FormBox["\[Mu]",
            TraditionalForm],
           TraditionalForm], "\[VeryThinSpace]", 
          FormBox[
           FormBox["\[Sigma]",
            TraditionalForm],
           TraditionalForm]}], "G4"], "(", 
        FormBox["r",
         TraditionalForm], ",", 
        FormBox["q",
         TraditionalForm], ",", 
        FormBox["p",
         TraditionalForm], ",", 
        FormBox["s",
         TraditionalForm], ")"}], ")"}]}]}], ")"}]}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.50097045926925*^9},
 CellTags->"Twist2GluonOperator",
 CellLabel->"Out[1827]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Twist2GluonOperator", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"p", ",", "\[Mu]"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"q", ",", "\[Nu]"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"r", ",", "\[Rho]"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"s", ",", "\[Sigma]"}], "}"}]}], "]"}]], "Input",
 CellTags->"Twist2GluonOperator",
 CellLabel->"In[1828]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubsuperscriptBox["O", 
    RowBox[{
     FormBox["\[Mu]",
      TraditionalForm], "\[VeryThinSpace]", 
     FormBox["\[Nu]",
      TraditionalForm], "\[VeryThinSpace]", 
     FormBox["\[Rho]",
      TraditionalForm], "\[VeryThinSpace]", 
     FormBox["\[Sigma]",
      TraditionalForm]}], "G4"], "(", 
   FormBox["p",
    TraditionalForm], ",", 
   FormBox["q",
    TraditionalForm], ",", 
   FormBox["r",
    TraditionalForm], ",", 
   FormBox["s",
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970459567546*^9},
 CellTags->"Twist2GluonOperator",
 CellLabel->"Out[1828]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Twist2GluonOperator", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"p", ",", "\[Mu]"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"q", ",", "\[Nu]"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"r", ",", "\[Rho]"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"s", ",", "\[Sigma]"}], "}"}], ",", 
   RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"Twist2GluonOperator",
 CellLabel->"In[1829]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox["g", 
        RowBox[{"\[Nu]", "\[Sigma]"}]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm]]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"\[Mu]", "\[Sigma]"}]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm]]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox["g", 
        RowBox[{"\[Nu]", "\[Rho]"}]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Sigma]",
          TraditionalForm],
         TraditionalForm]]}], "+", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"\[Mu]", "\[Rho]"}]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Sigma]",
          TraditionalForm],
         TraditionalForm]]}]}], ")"}], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["r",
         TraditionalForm]}], "+", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["s",
         TraditionalForm]}]}], ")"}], 
     RowBox[{"m", "-", "2"}]]}], "+", 
   RowBox[{
    RowBox[{"(", 
     FormBox[
      TagBox[
       RowBox[{
        UnderoverscriptBox["\[Sum]", 
         RowBox[{"i", "=", "0"}], 
         RowBox[{
          RowBox[{"-", "3"}], "+", "m"}]], 
        RowBox[{
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"-", 
            RowBox[{"(", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["p",
               TraditionalForm]}], ")"}]}], ")"}], "i"], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["r",
              TraditionalForm]}], "+", 
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["s",
              TraditionalForm]}]}], ")"}], 
          RowBox[{
           RowBox[{"-", "3"}], "-", "i", "+", "m"}]]}]}],
       HoldForm],
      TraditionalForm], ")"}], " ", 
    SuperscriptBox[
     FormBox["\[CapitalDelta]",
      TraditionalForm], 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm]], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Sigma]",
          TraditionalForm],
         TraditionalForm]]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"\[Mu]", "\[Rho]"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p",
         TraditionalForm]}], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Sigma]",
          TraditionalForm],
         TraditionalForm]]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], 
        FormBox[
         FormBox["\[Sigma]",
          TraditionalForm],
         TraditionalForm]]}], "+", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"\[Mu]", "\[Sigma]"}]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p",
         TraditionalForm]}]}]}], ")"}]}], "-", 
   RowBox[{
    RowBox[{"(", 
     FormBox[
      TagBox[
       RowBox[{
        UnderoverscriptBox["\[Sum]", 
         RowBox[{"i", "=", "0"}], 
         RowBox[{
          RowBox[{"-", "3"}], "+", "m"}]], 
        RowBox[{
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"-", 
            RowBox[{"(", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["q",
               TraditionalForm]}], ")"}]}], ")"}], "i"], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["r",
              TraditionalForm]}], "+", 
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["s",
              TraditionalForm]}]}], ")"}], 
          RowBox[{
           RowBox[{"-", "3"}], "-", "i", "+", "m"}]]}]}],
       HoldForm],
      TraditionalForm], ")"}], " ", 
    SuperscriptBox[
     FormBox["\[CapitalDelta]",
      TraditionalForm], 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["q",
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Sigma]",
          TraditionalForm],
         TraditionalForm]]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"\[Nu]", "\[Rho]"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["q",
         TraditionalForm]}], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Sigma]",
          TraditionalForm],
         TraditionalForm]]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["q",
         TraditionalForm], 
        FormBox[
         FormBox["\[Sigma]",
          TraditionalForm],
         TraditionalForm]]}], "+", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"\[Nu]", "\[Sigma]"}]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["q",
         TraditionalForm]}]}]}], ")"}]}], "+", 
   RowBox[{
    RowBox[{"(", 
     FormBox[
      TagBox[
       RowBox[{
        UnderoverscriptBox["\[Sum]", 
         RowBox[{"i", "=", "0"}], 
         RowBox[{
          RowBox[{"-", "3"}], "+", "m"}]], 
        RowBox[{
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["p",
              TraditionalForm]}], "+", 
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["q",
              TraditionalForm]}]}], ")"}], 
          RowBox[{
           RowBox[{"-", "3"}], "-", "i", "+", "m"}]], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"-", 
            RowBox[{"(", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["r",
               TraditionalForm]}], ")"}]}], ")"}], "i"]}]}],
       HoldForm],
      TraditionalForm], ")"}], " ", 
    SuperscriptBox[
     FormBox["\[CapitalDelta]",
      TraditionalForm], 
     FormBox[
      FormBox["\[Sigma]",
       TraditionalForm],
      TraditionalForm]], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["r",
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm]]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["r",
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm]]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox["g", 
        RowBox[{"\[Nu]", "\[Rho]"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["r",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"\[Mu]", "\[Rho]"}]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["r",
         TraditionalForm]}]}]}], ")"}]}], "+", 
   RowBox[{
    RowBox[{"(", 
     FormBox[
      TagBox[
       RowBox[{
        UnderoverscriptBox["\[Sum]", 
         RowBox[{"i", "=", "0"}], 
         RowBox[{
          RowBox[{"-", "3"}], "+", "m"}]], 
        RowBox[{
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            RowBox[{"-", 
             RowBox[{"(", 
              RowBox[{
               FormBox["\[CapitalDelta]",
                TraditionalForm], "\[CenterDot]", 
               FormBox["p",
                TraditionalForm]}], ")"}]}], "-", 
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["q",
              TraditionalForm]}]}], ")"}], "i"], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["s",
             TraditionalForm]}], ")"}], 
          RowBox[{
           RowBox[{"-", "3"}], "-", "i", "+", "m"}]]}]}],
       HoldForm],
      TraditionalForm], ")"}], " ", 
    SuperscriptBox[
     FormBox["\[CapitalDelta]",
      TraditionalForm], 
     FormBox[
      FormBox["\[Rho]",
       TraditionalForm],
      TraditionalForm]], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["s",
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Sigma]",
          TraditionalForm],
         TraditionalForm]]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["s",
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Sigma]",
          TraditionalForm],
         TraditionalForm]]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox["g", 
        RowBox[{"\[Nu]", "\[Sigma]"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["s",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"\[Mu]", "\[Sigma]"}]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["s",
         TraditionalForm]}]}]}], ")"}]}], "-", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      FormBox[
       TagBox[
        RowBox[{
         UnderoverscriptBox["\[Sum]", 
          RowBox[{"j", "=", "0"}], 
          RowBox[{
           RowBox[{"-", "4"}], "+", "m"}]], "\<\"\"\>"}],
        HoldForm],
       TraditionalForm], 
      FormBox[
       RowBox[{"\<\"\"\>", " ", 
        RowBox[{"(", 
         RowBox[{"j", "+", "1"}], ")"}]}],
       TraditionalForm], 
      FormBox[
       RowBox[{
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["p",
            TraditionalForm]}], ")"}], 
         RowBox[{
          RowBox[{"-", "j"}], "+", "m", "-", "4"}]], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"-", 
           RowBox[{"(", 
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["r",
              TraditionalForm]}], ")"}]}], ")"}], "i"], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           RowBox[{"-", 
            RowBox[{"(", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["r",
               TraditionalForm]}], ")"}]}], "-", 
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["s",
             TraditionalForm]}]}], ")"}], 
         RowBox[{"j", "-", "i"}]]}],
       TraditionalForm]}], ")"}], " ", 
    SuperscriptBox[
     FormBox["\[CapitalDelta]",
      TraditionalForm], 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm]], " ", 
    SuperscriptBox[
     FormBox["\[CapitalDelta]",
      TraditionalForm], 
     FormBox[
      FormBox["\[Sigma]",
       TraditionalForm],
      TraditionalForm]], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"-", 
        SuperscriptBox[
         FormBox["r",
          TraditionalForm], 
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm]]}], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"\[Mu]", "\[Rho]"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["r",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["r",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p",
         TraditionalForm], "\[CenterDot]", 
        FormBox["r",
         TraditionalForm]}]}]}], ")"}]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      FormBox[
       TagBox[
        RowBox[{
         UnderoverscriptBox["\[Sum]", 
          RowBox[{"j", "=", "0"}], 
          RowBox[{
           RowBox[{"-", "4"}], "+", "m"}]], "\<\"\"\>"}],
        HoldForm],
       TraditionalForm], 
      FormBox[
       RowBox[{"\<\"\"\>", " ", 
        RowBox[{"(", 
         RowBox[{"j", "+", "1"}], ")"}]}],
       TraditionalForm], 
      FormBox[
       RowBox[{
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["p",
            TraditionalForm]}], ")"}], 
         RowBox[{
          RowBox[{"-", "j"}], "+", "m", "-", "4"}]], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           RowBox[{"-", 
            RowBox[{"(", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["r",
               TraditionalForm]}], ")"}]}], "-", 
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["s",
             TraditionalForm]}]}], ")"}], 
         RowBox[{"j", "-", "i"}]], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"-", 
           RowBox[{"(", 
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["s",
              TraditionalForm]}], ")"}]}], ")"}], "i"]}],
       TraditionalForm]}], ")"}], " ", 
    SuperscriptBox[
     FormBox["\[CapitalDelta]",
      TraditionalForm], 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm]], " ", 
    SuperscriptBox[
     FormBox["\[CapitalDelta]",
      TraditionalForm], 
     FormBox[
      FormBox["\[Rho]",
       TraditionalForm],
      TraditionalForm]], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"-", 
        SuperscriptBox[
         FormBox["s",
          TraditionalForm], 
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm]]}], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Sigma]",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"\[Mu]", "\[Sigma]"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["s",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], 
        FormBox[
         FormBox["\[Sigma]",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["s",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Sigma]",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p",
         TraditionalForm], "\[CenterDot]", 
        FormBox["s",
         TraditionalForm]}]}]}], ")"}]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      FormBox[
       TagBox[
        RowBox[{
         UnderoverscriptBox["\[Sum]", 
          RowBox[{"j", "=", "0"}], 
          RowBox[{
           RowBox[{"-", "4"}], "+", "m"}]], "\<\"\"\>"}],
        HoldForm],
       TraditionalForm], 
      FormBox[
       RowBox[{"\<\"\"\>", " ", 
        RowBox[{"(", 
         RowBox[{"j", "+", "1"}], ")"}]}],
       TraditionalForm], 
      FormBox[
       RowBox[{
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["q",
            TraditionalForm]}], ")"}], 
         RowBox[{
          RowBox[{"-", "j"}], "+", "m", "-", "4"}]], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"-", 
           RowBox[{"(", 
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["r",
              TraditionalForm]}], ")"}]}], ")"}], "i"], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           RowBox[{"-", 
            RowBox[{"(", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["r",
               TraditionalForm]}], ")"}]}], "-", 
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["s",
             TraditionalForm]}]}], ")"}], 
         RowBox[{"j", "-", "i"}]]}],
       TraditionalForm]}], ")"}], " ", 
    SuperscriptBox[
     FormBox["\[CapitalDelta]",
      TraditionalForm], 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], " ", 
    SuperscriptBox[
     FormBox["\[CapitalDelta]",
      TraditionalForm], 
     FormBox[
      FormBox["\[Sigma]",
       TraditionalForm],
      TraditionalForm]], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"-", 
        SuperscriptBox[
         FormBox["r",
          TraditionalForm], 
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm]]}], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["q",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"\[Nu]", "\[Rho]"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["r",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["q",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["q",
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["r",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["q",
         TraditionalForm], "\[CenterDot]", 
        FormBox["r",
         TraditionalForm]}]}]}], ")"}]}], "-", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      FormBox[
       TagBox[
        RowBox[{
         UnderoverscriptBox["\[Sum]", 
          RowBox[{"j", "=", "0"}], 
          RowBox[{
           RowBox[{"-", "4"}], "+", "m"}]], "\<\"\"\>"}],
        HoldForm],
       TraditionalForm], 
      FormBox[
       RowBox[{"\<\"\"\>", " ", 
        RowBox[{"(", 
         RowBox[{"j", "+", "1"}], ")"}]}],
       TraditionalForm], 
      FormBox[
       RowBox[{
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["q",
            TraditionalForm]}], ")"}], 
         RowBox[{
          RowBox[{"-", "j"}], "+", "m", "-", "4"}]], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           RowBox[{"-", 
            RowBox[{"(", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["r",
               TraditionalForm]}], ")"}]}], "-", 
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["s",
             TraditionalForm]}]}], ")"}], 
         RowBox[{"j", "-", "i"}]], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"-", 
           RowBox[{"(", 
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["s",
              TraditionalForm]}], ")"}]}], ")"}], "i"]}],
       TraditionalForm]}], ")"}], " ", 
    SuperscriptBox[
     FormBox["\[CapitalDelta]",
      TraditionalForm], 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], " ", 
    SuperscriptBox[
     FormBox["\[CapitalDelta]",
      TraditionalForm], 
     FormBox[
      FormBox["\[Rho]",
       TraditionalForm],
      TraditionalForm]], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"-", 
        SuperscriptBox[
         FormBox["s",
          TraditionalForm], 
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm]]}], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Sigma]",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["q",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"\[Nu]", "\[Sigma]"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["s",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["q",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["q",
         TraditionalForm], 
        FormBox[
         FormBox["\[Sigma]",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["s",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Sigma]",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["q",
         TraditionalForm], "\[CenterDot]", 
        FormBox["s",
         TraditionalForm]}]}]}], ")"}]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970459772716*^9},
 CellTags->"Twist2GluonOperator",
 CellLabel->"Out[1829]="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Advanced Examples", "Subsubsection",
 CellTags->"Twist2GluonOperator"],

Cell["The setting All for Explicit performs the sums.", "Text",
 CellTags->"Twist2GluonOperator"],

Cell[BoxData[
 RowBox[{
  RowBox[{"SetOptions", "[", 
   RowBox[{"Twist2GluonOperator", ",", " ", 
    RowBox[{"Polarization", " ", "\[Rule]", " ", "1"}], ",", " ", 
    RowBox[{"Explicit", "\[Rule]", "All"}]}], "]"}], ";"}]], "Input",
 CellTags->"Twist2GluonOperator",
 CellLabel->"In[1830]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"Twist2GluonOperator", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"p", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"q", ",", "\[Nu]", ",", "b"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"r", ",", "\[Rho]", ",", "c"}], "}"}]}], "]"}], ";"}]], "Input",
 CellTags->"Twist2GluonOperator",
 CellLabel->"In[1831]:="]
}, Open  ]],

Cell[CellGroupData[{

Cell["Polarized case, non-zero-momentum insertion", "Subsubsection",
 CellTags->"Twist2GluonOperator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SetOptions", "[", 
  RowBox[{"Twist2GluonOperator", ",", " ", 
   RowBox[{"Polarization", " ", "\[Rule]", " ", "1"}], ",", 
   RowBox[{"ZeroMomentumInsertion", "\[Rule]", "False"}]}], "]"}]], "Input",
 CellTags->"Twist2GluonOperator",
 CellLabel->"In[1832]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"CouplingConstant", "\[Rule]", 
     SubscriptBox["g", "s"]}], ",", 
    RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
    RowBox[{"Polarization", "\[Rule]", "1"}], ",", 
    RowBox[{"Explicit", "\[Rule]", "All"}], ",", 
    RowBox[{"ZeroMomentumInsertion", "\[Rule]", "False"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970460226407*^9},
 CellTags->"Twist2GluonOperator",
 CellLabel->"Out[1832]="]
}, Open  ]],

Cell["2-gluon legs", "Text",
 CellTags->"Twist2GluonOperator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Twist2GluonOperator", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"p", ",", "q"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\[Mu]", ",", "a"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\[Nu]", ",", "b"}], "}"}]}], "]"}]], "Input",
 CellTags->"Twist2GluonOperator",
 CellLabel->"In[1833]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["1", "2"], " ", "\[ImaginaryI]", " ", 
   RowBox[{"(", 
    RowBox[{"1", "-", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}], "m"]}], ")"}], " ", 
   SubscriptBox["\[Delta]", 
    RowBox[{
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm]}]], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      FormBox[
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{
          FormBox["\[CapitalDelta]",
           TraditionalForm], "\[CenterDot]", 
          FormBox["p",
           TraditionalForm]}], ")"}], 
        RowBox[{"m", "-", "2"}]],
       TraditionalForm], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         SuperscriptBox[
          FormBox["\[CapitalDelta]",
           TraditionalForm], 
          FormBox[
           FormBox["\[Mu]",
            TraditionalForm],
           TraditionalForm]], " ", 
         SuperscriptBox["\[Epsilon]", 
          RowBox[{
           FormBox[
            FormBox["\[Nu]",
             TraditionalForm],
            TraditionalForm], 
           FormBox["\[CapitalDelta]",
            TraditionalForm], 
           FormBox["p",
            TraditionalForm], 
           FormBox["q",
            TraditionalForm]}]]}], "-", 
        RowBox[{
         RowBox[{
          FormBox["\[CapitalDelta]",
           TraditionalForm], "\[CenterDot]", 
          FormBox["p",
           TraditionalForm]}], " ", 
         SuperscriptBox["\[Epsilon]", 
          RowBox[{
           FormBox[
            FormBox["\[Mu]",
             TraditionalForm],
            TraditionalForm], 
           FormBox[
            FormBox["\[Nu]",
             TraditionalForm],
            TraditionalForm], 
           FormBox["\[CapitalDelta]",
            TraditionalForm], 
           FormBox["q",
            TraditionalForm]}]]}]}], ")"}]}], "+", 
     RowBox[{
      FormBox[
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{
          FormBox["\[CapitalDelta]",
           TraditionalForm], "\[CenterDot]", 
          FormBox["q",
           TraditionalForm]}], ")"}], 
        RowBox[{"m", "-", "2"}]],
       TraditionalForm], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{
          FormBox["\[CapitalDelta]",
           TraditionalForm], "\[CenterDot]", 
          FormBox["q",
           TraditionalForm]}], " ", 
         SuperscriptBox["\[Epsilon]", 
          RowBox[{
           FormBox[
            FormBox["\[Mu]",
             TraditionalForm],
            TraditionalForm], 
           FormBox[
            FormBox["\[Nu]",
             TraditionalForm],
            TraditionalForm], 
           FormBox["\[CapitalDelta]",
            TraditionalForm], 
           FormBox["p",
            TraditionalForm]}]]}], "-", 
        RowBox[{
         SuperscriptBox[
          FormBox["\[CapitalDelta]",
           TraditionalForm], 
          FormBox[
           FormBox["\[Nu]",
            TraditionalForm],
           TraditionalForm]], " ", 
         SuperscriptBox["\[Epsilon]", 
          RowBox[{
           FormBox[
            FormBox["\[Mu]",
             TraditionalForm],
            TraditionalForm], 
           FormBox["\[CapitalDelta]",
            TraditionalForm], 
           FormBox["p",
            TraditionalForm], 
           FormBox["q",
            TraditionalForm]}]]}]}], ")"}]}]}], ")"}]}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.5009704603896513`*^9},
 CellTags->"Twist2GluonOperator",
 CellLabel->"Out[1833]="]
}, Open  ]],

Cell["3-gluon legs", "Text",
 CellTags->"Twist2GluonOperator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Twist2GluonOperator", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"p", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"q", ",", "\[Nu]", ",", "b"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"r", ",", "\[Rho]", ",", "c"}], "}"}]}], "]"}]], "Input",
 CellTags->"Twist2GluonOperator",
 CellLabel->"In[1834]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["g", "s"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"-", 
       SuperscriptBox["\[Epsilon]", 
        RowBox[{
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["\[Rho]",
           TraditionalForm],
          TraditionalForm], 
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox["p",
          TraditionalForm]}]]}], " ", 
      SuperscriptBox[
       FormBox["\[CapitalDelta]",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{
         FormBox["\[CapitalDelta]",
          TraditionalForm], "\[CenterDot]", 
         FormBox["p",
          TraditionalForm]}], ")"}], 
       RowBox[{"m", "-", "2"}]]}], "+", 
     RowBox[{
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm], 
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox["q",
         TraditionalForm]}]], " ", 
      SuperscriptBox[
       FormBox["\[CapitalDelta]",
        TraditionalForm], 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{
         FormBox["\[CapitalDelta]",
          TraditionalForm], "\[CenterDot]", 
         FormBox["q",
          TraditionalForm]}], ")"}], 
       RowBox[{"m", "-", "2"}]]}], "-", 
     RowBox[{
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox["r",
         TraditionalForm]}]], " ", 
      SuperscriptBox[
       FormBox["\[CapitalDelta]",
        TraditionalForm], 
       FormBox[
        FormBox["\[Rho]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{
         FormBox["\[CapitalDelta]",
          TraditionalForm], "\[CenterDot]", 
         FormBox["r",
          TraditionalForm]}], ")"}], 
       RowBox[{"m", "-", "2"}]]}], "-", 
     RowBox[{
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm], 
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox["p",
         TraditionalForm], 
        FormBox["r",
         TraditionalForm]}]], " ", 
      RowBox[{"(", 
       FormBox[
        TagBox[
         RowBox[{
          UnderoverscriptBox["\[Sum]", 
           RowBox[{"i", "=", "0"}], 
           RowBox[{
            RowBox[{"-", "3"}], "+", "m"}]], 
          RowBox[{
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["p",
               TraditionalForm]}], ")"}], 
            RowBox[{
             RowBox[{"-", "3"}], "-", "i", "+", "m"}]], " ", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               FormBox["\[CapitalDelta]",
                TraditionalForm], "\[CenterDot]", 
               FormBox["p",
                TraditionalForm]}], "+", 
              RowBox[{
               FormBox["\[CapitalDelta]",
                TraditionalForm], "\[CenterDot]", 
               FormBox["q",
                TraditionalForm]}]}], ")"}], "i"]}]}],
         HoldForm],
        TraditionalForm], ")"}], " ", 
      SuperscriptBox[
       FormBox["\[CapitalDelta]",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox[
       FormBox["\[CapitalDelta]",
        TraditionalForm], 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]]}], "+", 
     RowBox[{
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm], 
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox["q",
         TraditionalForm], 
        FormBox["r",
         TraditionalForm]}]], " ", 
      RowBox[{"(", 
       FormBox[
        TagBox[
         RowBox[{
          UnderoverscriptBox["\[Sum]", 
           RowBox[{"i", "=", "0"}], 
           RowBox[{
            RowBox[{"-", "3"}], "+", "m"}]], 
          RowBox[{
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["q",
               TraditionalForm]}], ")"}], 
            RowBox[{
             RowBox[{"-", "3"}], "-", "i", "+", "m"}]], " ", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               FormBox["\[CapitalDelta]",
                TraditionalForm], "\[CenterDot]", 
               FormBox["p",
                TraditionalForm]}], "+", 
              RowBox[{
               FormBox["\[CapitalDelta]",
                TraditionalForm], "\[CenterDot]", 
               FormBox["q",
                TraditionalForm]}]}], ")"}], "i"]}]}],
         HoldForm],
        TraditionalForm], ")"}], " ", 
      SuperscriptBox[
       FormBox["\[CapitalDelta]",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox[
       FormBox["\[CapitalDelta]",
        TraditionalForm], 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]]}], "+", 
     RowBox[{
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox["p",
         TraditionalForm], 
        FormBox["q",
         TraditionalForm]}]], " ", 
      RowBox[{"(", 
       FormBox[
        TagBox[
         RowBox[{
          UnderoverscriptBox["\[Sum]", 
           RowBox[{"i", "=", "0"}], 
           RowBox[{
            RowBox[{"-", "3"}], "+", "m"}]], 
          RowBox[{
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["p",
               TraditionalForm]}], ")"}], 
            RowBox[{
             RowBox[{"-", "3"}], "-", "i", "+", "m"}]], " ", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               FormBox["\[CapitalDelta]",
                TraditionalForm], "\[CenterDot]", 
               FormBox["p",
                TraditionalForm]}], "+", 
              RowBox[{
               FormBox["\[CapitalDelta]",
                TraditionalForm], "\[CenterDot]", 
               FormBox["r",
                TraditionalForm]}]}], ")"}], "i"]}]}],
         HoldForm],
        TraditionalForm], ")"}], " ", 
      SuperscriptBox[
       FormBox["\[CapitalDelta]",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox[
       FormBox["\[CapitalDelta]",
        TraditionalForm], 
       FormBox[
        FormBox["\[Rho]",
         TraditionalForm],
        TraditionalForm]]}], "+", 
     RowBox[{
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox["q",
         TraditionalForm], 
        FormBox["r",
         TraditionalForm]}]], " ", 
      RowBox[{"(", 
       FormBox[
        TagBox[
         RowBox[{
          UnderoverscriptBox["\[Sum]", 
           RowBox[{"i", "=", "0"}], 
           RowBox[{
            RowBox[{"-", "3"}], "+", "m"}]], 
          RowBox[{
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["r",
               TraditionalForm]}], ")"}], 
            RowBox[{
             RowBox[{"-", "3"}], "-", "i", "+", "m"}]], " ", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               FormBox["\[CapitalDelta]",
                TraditionalForm], "\[CenterDot]", 
               FormBox["p",
                TraditionalForm]}], "+", 
              RowBox[{
               FormBox["\[CapitalDelta]",
                TraditionalForm], "\[CenterDot]", 
               FormBox["r",
                TraditionalForm]}]}], ")"}], "i"]}]}],
         HoldForm],
        TraditionalForm], ")"}], " ", 
      SuperscriptBox[
       FormBox["\[CapitalDelta]",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox[
       FormBox["\[CapitalDelta]",
        TraditionalForm], 
       FormBox[
        FormBox["\[Rho]",
         TraditionalForm],
        TraditionalForm]]}], "+", 
     RowBox[{
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox["p",
         TraditionalForm], 
        FormBox["q",
         TraditionalForm]}]], " ", 
      RowBox[{"(", 
       FormBox[
        TagBox[
         RowBox[{
          UnderoverscriptBox["\[Sum]", 
           RowBox[{"i", "=", "0"}], 
           RowBox[{
            RowBox[{"-", "3"}], "+", "m"}]], 
          RowBox[{
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["q",
               TraditionalForm]}], ")"}], 
            RowBox[{
             RowBox[{"-", "3"}], "-", "i", "+", "m"}]], " ", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               FormBox["\[CapitalDelta]",
                TraditionalForm], "\[CenterDot]", 
               FormBox["q",
                TraditionalForm]}], "+", 
              RowBox[{
               FormBox["\[CapitalDelta]",
                TraditionalForm], "\[CenterDot]", 
               FormBox["r",
                TraditionalForm]}]}], ")"}], "i"]}]}],
         HoldForm],
        TraditionalForm], ")"}], " ", 
      SuperscriptBox[
       FormBox["\[CapitalDelta]",
        TraditionalForm], 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox[
       FormBox["\[CapitalDelta]",
        TraditionalForm], 
       FormBox[
        FormBox["\[Rho]",
         TraditionalForm],
        TraditionalForm]]}], "-", 
     RowBox[{
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox["p",
         TraditionalForm], 
        FormBox["r",
         TraditionalForm]}]], " ", 
      RowBox[{"(", 
       FormBox[
        TagBox[
         RowBox[{
          UnderoverscriptBox["\[Sum]", 
           RowBox[{"i", "=", "0"}], 
           RowBox[{
            RowBox[{"-", "3"}], "+", "m"}]], 
          RowBox[{
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["r",
               TraditionalForm]}], ")"}], 
            RowBox[{
             RowBox[{"-", "3"}], "-", "i", "+", "m"}]], " ", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               FormBox["\[CapitalDelta]",
                TraditionalForm], "\[CenterDot]", 
               FormBox["q",
                TraditionalForm]}], "+", 
              RowBox[{
               FormBox["\[CapitalDelta]",
                TraditionalForm], "\[CenterDot]", 
               FormBox["r",
                TraditionalForm]}]}], ")"}], "i"]}]}],
         HoldForm],
        TraditionalForm], ")"}], " ", 
      SuperscriptBox[
       FormBox["\[CapitalDelta]",
        TraditionalForm], 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox[
       FormBox["\[CapitalDelta]",
        TraditionalForm], 
       FormBox[
        FormBox["\[Rho]",
         TraditionalForm],
        TraditionalForm]]}], "+", 
     RowBox[{
      FractionBox["1", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["p",
            TraditionalForm]}], "+", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["q",
            TraditionalForm]}]}], ")"}], "2"]], 
      RowBox[{
       SuperscriptBox["\[Epsilon]", 
        RowBox[{
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["\[Rho]",
           TraditionalForm],
          TraditionalForm], 
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox["r",
          TraditionalForm]}]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["p",
              TraditionalForm]}], "+", 
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["q",
              TraditionalForm]}]}], ")"}], "m"], "+", 
         RowBox[{
          RowBox[{"(", 
           FormBox[
            TagBox[
             RowBox[{
              UnderoverscriptBox["\[Sum]", 
               RowBox[{"i", "=", "0"}], 
               RowBox[{
                RowBox[{"-", "3"}], "+", "m"}]], 
              RowBox[{
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  FormBox["\[CapitalDelta]",
                   TraditionalForm], "\[CenterDot]", 
                  FormBox["p",
                   TraditionalForm]}], ")"}], 
                RowBox[{
                 RowBox[{"-", "3"}], "-", "i", "+", "m"}]], " ", 
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{
                   FormBox["\[CapitalDelta]",
                    TraditionalForm], "\[CenterDot]", 
                   FormBox["p",
                    TraditionalForm]}], "+", 
                  RowBox[{
                   FormBox["\[CapitalDelta]",
                    TraditionalForm], "\[CenterDot]", 
                   FormBox["q",
                    TraditionalForm]}]}], ")"}], "i"]}]}],
             HoldForm],
            TraditionalForm], ")"}], " ", 
          SuperscriptBox[
           FormBox[
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["p",
              TraditionalForm]}],
            TraditionalForm], "3"]}], "+", 
         RowBox[{
          RowBox[{"(", 
           FormBox[
            TagBox[
             RowBox[{
              UnderoverscriptBox["\[Sum]", 
               RowBox[{"i", "=", "0"}], 
               RowBox[{
                RowBox[{"-", "3"}], "+", "m"}]], 
              RowBox[{
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  FormBox["\[CapitalDelta]",
                   TraditionalForm], "\[CenterDot]", 
                  FormBox["p",
                   TraditionalForm]}], ")"}], 
                RowBox[{
                 RowBox[{"-", "3"}], "-", "i", "+", "m"}]], " ", 
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{
                   FormBox["\[CapitalDelta]",
                    TraditionalForm], "\[CenterDot]", 
                   FormBox["p",
                    TraditionalForm]}], "+", 
                  RowBox[{
                   FormBox["\[CapitalDelta]",
                    TraditionalForm], "\[CenterDot]", 
                   FormBox["q",
                    TraditionalForm]}]}], ")"}], "i"]}]}],
             HoldForm],
            TraditionalForm], ")"}], " ", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["p",
            TraditionalForm]}], " ", 
          SuperscriptBox[
           FormBox[
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["q",
              TraditionalForm]}],
            TraditionalForm], "2"]}], "+", 
         RowBox[{"2", " ", 
          RowBox[{"(", 
           FormBox[
            TagBox[
             RowBox[{
              UnderoverscriptBox["\[Sum]", 
               RowBox[{"i", "=", "0"}], 
               RowBox[{
                RowBox[{"-", "3"}], "+", "m"}]], 
              RowBox[{
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  FormBox["\[CapitalDelta]",
                   TraditionalForm], "\[CenterDot]", 
                  FormBox["p",
                   TraditionalForm]}], ")"}], 
                RowBox[{
                 RowBox[{"-", "3"}], "-", "i", "+", "m"}]], " ", 
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{
                   FormBox["\[CapitalDelta]",
                    TraditionalForm], "\[CenterDot]", 
                   FormBox["p",
                    TraditionalForm]}], "+", 
                  RowBox[{
                   FormBox["\[CapitalDelta]",
                    TraditionalForm], "\[CenterDot]", 
                   FormBox["q",
                    TraditionalForm]}]}], ")"}], "i"]}]}],
             HoldForm],
            TraditionalForm], ")"}], " ", 
          SuperscriptBox[
           FormBox[
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["p",
              TraditionalForm]}],
            TraditionalForm], "2"], " ", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["q",
            TraditionalForm]}]}]}], ")"}]}]}], "-", 
     RowBox[{
      FractionBox["1", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["p",
            TraditionalForm]}], "+", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["q",
            TraditionalForm]}]}], ")"}], "2"]], 
      RowBox[{
       SuperscriptBox["\[Epsilon]", 
        RowBox[{
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["\[Rho]",
           TraditionalForm],
          TraditionalForm], 
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox["r",
          TraditionalForm]}]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["p",
              TraditionalForm]}], "+", 
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["q",
              TraditionalForm]}]}], ")"}], "m"], "+", 
         RowBox[{
          RowBox[{"(", 
           FormBox[
            TagBox[
             RowBox[{
              UnderoverscriptBox["\[Sum]", 
               RowBox[{"i", "=", "0"}], 
               RowBox[{
                RowBox[{"-", "3"}], "+", "m"}]], 
              RowBox[{
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  FormBox["\[CapitalDelta]",
                   TraditionalForm], "\[CenterDot]", 
                  FormBox["q",
                   TraditionalForm]}], ")"}], 
                RowBox[{
                 RowBox[{"-", "3"}], "-", "i", "+", "m"}]], " ", 
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{
                   FormBox["\[CapitalDelta]",
                    TraditionalForm], "\[CenterDot]", 
                   FormBox["p",
                    TraditionalForm]}], "+", 
                  RowBox[{
                   FormBox["\[CapitalDelta]",
                    TraditionalForm], "\[CenterDot]", 
                   FormBox["q",
                    TraditionalForm]}]}], ")"}], "i"]}]}],
             HoldForm],
            TraditionalForm], ")"}], " ", 
          SuperscriptBox[
           FormBox[
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["q",
              TraditionalForm]}],
            TraditionalForm], "3"]}], "+", 
         RowBox[{"2", " ", 
          RowBox[{"(", 
           FormBox[
            TagBox[
             RowBox[{
              UnderoverscriptBox["\[Sum]", 
               RowBox[{"i", "=", "0"}], 
               RowBox[{
                RowBox[{"-", "3"}], "+", "m"}]], 
              RowBox[{
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  FormBox["\[CapitalDelta]",
                   TraditionalForm], "\[CenterDot]", 
                  FormBox["q",
                   TraditionalForm]}], ")"}], 
                RowBox[{
                 RowBox[{"-", "3"}], "-", "i", "+", "m"}]], " ", 
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{
                   FormBox["\[CapitalDelta]",
                    TraditionalForm], "\[CenterDot]", 
                   FormBox["p",
                    TraditionalForm]}], "+", 
                  RowBox[{
                   FormBox["\[CapitalDelta]",
                    TraditionalForm], "\[CenterDot]", 
                   FormBox["q",
                    TraditionalForm]}]}], ")"}], "i"]}]}],
             HoldForm],
            TraditionalForm], ")"}], " ", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["p",
            TraditionalForm]}], " ", 
          SuperscriptBox[
           FormBox[
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["q",
              TraditionalForm]}],
            TraditionalForm], "2"]}], "+", 
         RowBox[{
          RowBox[{"(", 
           FormBox[
            TagBox[
             RowBox[{
              UnderoverscriptBox["\[Sum]", 
               RowBox[{"i", "=", "0"}], 
               RowBox[{
                RowBox[{"-", "3"}], "+", "m"}]], 
              RowBox[{
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  FormBox["\[CapitalDelta]",
                   TraditionalForm], "\[CenterDot]", 
                  FormBox["q",
                   TraditionalForm]}], ")"}], 
                RowBox[{
                 RowBox[{"-", "3"}], "-", "i", "+", "m"}]], " ", 
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{
                   FormBox["\[CapitalDelta]",
                    TraditionalForm], "\[CenterDot]", 
                   FormBox["p",
                    TraditionalForm]}], "+", 
                  RowBox[{
                   FormBox["\[CapitalDelta]",
                    TraditionalForm], "\[CenterDot]", 
                   FormBox["q",
                    TraditionalForm]}]}], ")"}], "i"]}]}],
             HoldForm],
            TraditionalForm], ")"}], " ", 
          SuperscriptBox[
           FormBox[
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["p",
              TraditionalForm]}],
            TraditionalForm], "2"], " ", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["q",
            TraditionalForm]}]}]}], ")"}]}]}], "-", 
     RowBox[{
      FractionBox["1", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["p",
            TraditionalForm]}], "+", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["r",
            TraditionalForm]}]}], ")"}], "2"]], 
      RowBox[{
       SuperscriptBox["\[Epsilon]", 
        RowBox[{
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox["q",
          TraditionalForm]}]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["p",
              TraditionalForm]}], "+", 
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["r",
              TraditionalForm]}]}], ")"}], "m"], "+", 
         RowBox[{
          RowBox[{"(", 
           FormBox[
            TagBox[
             RowBox[{
              UnderoverscriptBox["\[Sum]", 
               RowBox[{"i", "=", "0"}], 
               RowBox[{
                RowBox[{"-", "3"}], "+", "m"}]], 
              RowBox[{
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  FormBox["\[CapitalDelta]",
                   TraditionalForm], "\[CenterDot]", 
                  FormBox["p",
                   TraditionalForm]}], ")"}], 
                RowBox[{
                 RowBox[{"-", "3"}], "-", "i", "+", "m"}]], " ", 
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{
                   FormBox["\[CapitalDelta]",
                    TraditionalForm], "\[CenterDot]", 
                   FormBox["p",
                    TraditionalForm]}], "+", 
                  RowBox[{
                   FormBox["\[CapitalDelta]",
                    TraditionalForm], "\[CenterDot]", 
                   FormBox["r",
                    TraditionalForm]}]}], ")"}], "i"]}]}],
             HoldForm],
            TraditionalForm], ")"}], " ", 
          SuperscriptBox[
           FormBox[
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["p",
              TraditionalForm]}],
            TraditionalForm], "3"]}], "+", 
         RowBox[{
          RowBox[{"(", 
           FormBox[
            TagBox[
             RowBox[{
              UnderoverscriptBox["\[Sum]", 
               RowBox[{"i", "=", "0"}], 
               RowBox[{
                RowBox[{"-", "3"}], "+", "m"}]], 
              RowBox[{
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  FormBox["\[CapitalDelta]",
                   TraditionalForm], "\[CenterDot]", 
                  FormBox["p",
                   TraditionalForm]}], ")"}], 
                RowBox[{
                 RowBox[{"-", "3"}], "-", "i", "+", "m"}]], " ", 
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{
                   FormBox["\[CapitalDelta]",
                    TraditionalForm], "\[CenterDot]", 
                   FormBox["p",
                    TraditionalForm]}], "+", 
                  RowBox[{
                   FormBox["\[CapitalDelta]",
                    TraditionalForm], "\[CenterDot]", 
                   FormBox["r",
                    TraditionalForm]}]}], ")"}], "i"]}]}],
             HoldForm],
            TraditionalForm], ")"}], " ", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["p",
            TraditionalForm]}], " ", 
          SuperscriptBox[
           FormBox[
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["r",
              TraditionalForm]}],
            TraditionalForm], "2"]}], "+", 
         RowBox[{"2", " ", 
          RowBox[{"(", 
           FormBox[
            TagBox[
             RowBox[{
              UnderoverscriptBox["\[Sum]", 
               RowBox[{"i", "=", "0"}], 
               RowBox[{
                RowBox[{"-", "3"}], "+", "m"}]], 
              RowBox[{
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  FormBox["\[CapitalDelta]",
                   TraditionalForm], "\[CenterDot]", 
                  FormBox["p",
                   TraditionalForm]}], ")"}], 
                RowBox[{
                 RowBox[{"-", "3"}], "-", "i", "+", "m"}]], " ", 
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{
                   FormBox["\[CapitalDelta]",
                    TraditionalForm], "\[CenterDot]", 
                   FormBox["p",
                    TraditionalForm]}], "+", 
                  RowBox[{
                   FormBox["\[CapitalDelta]",
                    TraditionalForm], "\[CenterDot]", 
                   FormBox["r",
                    TraditionalForm]}]}], ")"}], "i"]}]}],
             HoldForm],
            TraditionalForm], ")"}], " ", 
          SuperscriptBox[
           FormBox[
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["p",
              TraditionalForm]}],
            TraditionalForm], "2"], " ", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["r",
            TraditionalForm]}]}]}], ")"}]}]}], "-", 
     RowBox[{
      FractionBox["1", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["p",
            TraditionalForm]}], "+", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["r",
            TraditionalForm]}]}], ")"}], "2"]], 
      RowBox[{
       SuperscriptBox["\[Epsilon]", 
        RowBox[{
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["\[Rho]",
           TraditionalForm],
          TraditionalForm], 
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox["q",
          TraditionalForm]}]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["p",
              TraditionalForm]}], "+", 
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["r",
              TraditionalForm]}]}], ")"}], "m"], "+", 
         RowBox[{
          RowBox[{"(", 
           FormBox[
            TagBox[
             RowBox[{
              UnderoverscriptBox["\[Sum]", 
               RowBox[{"i", "=", "0"}], 
               RowBox[{
                RowBox[{"-", "3"}], "+", "m"}]], 
              RowBox[{
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  FormBox["\[CapitalDelta]",
                   TraditionalForm], "\[CenterDot]", 
                  FormBox["r",
                   TraditionalForm]}], ")"}], 
                RowBox[{
                 RowBox[{"-", "3"}], "-", "i", "+", "m"}]], " ", 
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{
                   FormBox["\[CapitalDelta]",
                    TraditionalForm], "\[CenterDot]", 
                   FormBox["p",
                    TraditionalForm]}], "+", 
                  RowBox[{
                   FormBox["\[CapitalDelta]",
                    TraditionalForm], "\[CenterDot]", 
                   FormBox["r",
                    TraditionalForm]}]}], ")"}], "i"]}]}],
             HoldForm],
            TraditionalForm], ")"}], " ", 
          SuperscriptBox[
           FormBox[
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["r",
              TraditionalForm]}],
            TraditionalForm], "3"]}], "+", 
         RowBox[{"2", " ", 
          RowBox[{"(", 
           FormBox[
            TagBox[
             RowBox[{
              UnderoverscriptBox["\[Sum]", 
               RowBox[{"i", "=", "0"}], 
               RowBox[{
                RowBox[{"-", "3"}], "+", "m"}]], 
              RowBox[{
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  FormBox["\[CapitalDelta]",
                   TraditionalForm], "\[CenterDot]", 
                  FormBox["r",
                   TraditionalForm]}], ")"}], 
                RowBox[{
                 RowBox[{"-", "3"}], "-", "i", "+", "m"}]], " ", 
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{
                   FormBox["\[CapitalDelta]",
                    TraditionalForm], "\[CenterDot]", 
                   FormBox["p",
                    TraditionalForm]}], "+", 
                  RowBox[{
                   FormBox["\[CapitalDelta]",
                    TraditionalForm], "\[CenterDot]", 
                   FormBox["r",
                    TraditionalForm]}]}], ")"}], "i"]}]}],
             HoldForm],
            TraditionalForm], ")"}], " ", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["p",
            TraditionalForm]}], " ", 
          SuperscriptBox[
           FormBox[
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["r",
              TraditionalForm]}],
            TraditionalForm], "2"]}], "+", 
         RowBox[{
          RowBox[{"(", 
           FormBox[
            TagBox[
             RowBox[{
              UnderoverscriptBox["\[Sum]", 
               RowBox[{"i", "=", "0"}], 
               RowBox[{
                RowBox[{"-", "3"}], "+", "m"}]], 
              RowBox[{
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  FormBox["\[CapitalDelta]",
                   TraditionalForm], "\[CenterDot]", 
                  FormBox["r",
                   TraditionalForm]}], ")"}], 
                RowBox[{
                 RowBox[{"-", "3"}], "-", "i", "+", "m"}]], " ", 
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{
                   FormBox["\[CapitalDelta]",
                    TraditionalForm], "\[CenterDot]", 
                   FormBox["p",
                    TraditionalForm]}], "+", 
                  RowBox[{
                   FormBox["\[CapitalDelta]",
                    TraditionalForm], "\[CenterDot]", 
                   FormBox["r",
                    TraditionalForm]}]}], ")"}], "i"]}]}],
             HoldForm],
            TraditionalForm], ")"}], " ", 
          SuperscriptBox[
           FormBox[
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["p",
              TraditionalForm]}],
            TraditionalForm], "2"], " ", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["r",
            TraditionalForm]}]}]}], ")"}]}]}], "-", 
     RowBox[{
      FractionBox["1", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["q",
            TraditionalForm]}], "+", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["r",
            TraditionalForm]}]}], ")"}], "2"]], 
      RowBox[{
       SuperscriptBox["\[Epsilon]", 
        RowBox[{
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox["p",
          TraditionalForm]}]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["q",
              TraditionalForm]}], "+", 
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["r",
              TraditionalForm]}]}], ")"}], "m"], "+", 
         RowBox[{
          RowBox[{"(", 
           FormBox[
            TagBox[
             RowBox[{
              UnderoverscriptBox["\[Sum]", 
               RowBox[{"i", "=", "0"}], 
               RowBox[{
                RowBox[{"-", "3"}], "+", "m"}]], 
              RowBox[{
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  FormBox["\[CapitalDelta]",
                   TraditionalForm], "\[CenterDot]", 
                  FormBox["q",
                   TraditionalForm]}], ")"}], 
                RowBox[{
                 RowBox[{"-", "3"}], "-", "i", "+", "m"}]], " ", 
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{
                   FormBox["\[CapitalDelta]",
                    TraditionalForm], "\[CenterDot]", 
                   FormBox["q",
                    TraditionalForm]}], "+", 
                  RowBox[{
                   FormBox["\[CapitalDelta]",
                    TraditionalForm], "\[CenterDot]", 
                   FormBox["r",
                    TraditionalForm]}]}], ")"}], "i"]}]}],
             HoldForm],
            TraditionalForm], ")"}], " ", 
          SuperscriptBox[
           FormBox[
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["q",
              TraditionalForm]}],
            TraditionalForm], "3"]}], "+", 
         RowBox[{
          RowBox[{"(", 
           FormBox[
            TagBox[
             RowBox[{
              UnderoverscriptBox["\[Sum]", 
               RowBox[{"i", "=", "0"}], 
               RowBox[{
                RowBox[{"-", "3"}], "+", "m"}]], 
              RowBox[{
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  FormBox["\[CapitalDelta]",
                   TraditionalForm], "\[CenterDot]", 
                  FormBox["q",
                   TraditionalForm]}], ")"}], 
                RowBox[{
                 RowBox[{"-", "3"}], "-", "i", "+", "m"}]], " ", 
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{
                   FormBox["\[CapitalDelta]",
                    TraditionalForm], "\[CenterDot]", 
                   FormBox["q",
                    TraditionalForm]}], "+", 
                  RowBox[{
                   FormBox["\[CapitalDelta]",
                    TraditionalForm], "\[CenterDot]", 
                   FormBox["r",
                    TraditionalForm]}]}], ")"}], "i"]}]}],
             HoldForm],
            TraditionalForm], ")"}], " ", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["q",
            TraditionalForm]}], " ", 
          SuperscriptBox[
           FormBox[
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["r",
              TraditionalForm]}],
            TraditionalForm], "2"]}], "+", 
         RowBox[{"2", " ", 
          RowBox[{"(", 
           FormBox[
            TagBox[
             RowBox[{
              UnderoverscriptBox["\[Sum]", 
               RowBox[{"i", "=", "0"}], 
               RowBox[{
                RowBox[{"-", "3"}], "+", "m"}]], 
              RowBox[{
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  FormBox["\[CapitalDelta]",
                   TraditionalForm], "\[CenterDot]", 
                  FormBox["q",
                   TraditionalForm]}], ")"}], 
                RowBox[{
                 RowBox[{"-", "3"}], "-", "i", "+", "m"}]], " ", 
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{
                   FormBox["\[CapitalDelta]",
                    TraditionalForm], "\[CenterDot]", 
                   FormBox["q",
                    TraditionalForm]}], "+", 
                  RowBox[{
                   FormBox["\[CapitalDelta]",
                    TraditionalForm], "\[CenterDot]", 
                   FormBox["r",
                    TraditionalForm]}]}], ")"}], "i"]}]}],
             HoldForm],
            TraditionalForm], ")"}], " ", 
          SuperscriptBox[
           FormBox[
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["q",
              TraditionalForm]}],
            TraditionalForm], "2"], " ", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["r",
            TraditionalForm]}]}]}], ")"}]}]}], "+", 
     RowBox[{
      FractionBox["1", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["q",
            TraditionalForm]}], "+", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["r",
            TraditionalForm]}]}], ")"}], "2"]], 
      RowBox[{
       SuperscriptBox["\[Epsilon]", 
        RowBox[{
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["\[Rho]",
           TraditionalForm],
          TraditionalForm], 
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox["p",
          TraditionalForm]}]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["q",
              TraditionalForm]}], "+", 
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["r",
              TraditionalForm]}]}], ")"}], "m"], "+", 
         RowBox[{
          RowBox[{"(", 
           FormBox[
            TagBox[
             RowBox[{
              UnderoverscriptBox["\[Sum]", 
               RowBox[{"i", "=", "0"}], 
               RowBox[{
                RowBox[{"-", "3"}], "+", "m"}]], 
              RowBox[{
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  FormBox["\[CapitalDelta]",
                   TraditionalForm], "\[CenterDot]", 
                  FormBox["r",
                   TraditionalForm]}], ")"}], 
                RowBox[{
                 RowBox[{"-", "3"}], "-", "i", "+", "m"}]], " ", 
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{
                   FormBox["\[CapitalDelta]",
                    TraditionalForm], "\[CenterDot]", 
                   FormBox["q",
                    TraditionalForm]}], "+", 
                  RowBox[{
                   FormBox["\[CapitalDelta]",
                    TraditionalForm], "\[CenterDot]", 
                   FormBox["r",
                    TraditionalForm]}]}], ")"}], "i"]}]}],
             HoldForm],
            TraditionalForm], ")"}], " ", 
          SuperscriptBox[
           FormBox[
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["r",
              TraditionalForm]}],
            TraditionalForm], "3"]}], "+", 
         RowBox[{"2", " ", 
          RowBox[{"(", 
           FormBox[
            TagBox[
             RowBox[{
              UnderoverscriptBox["\[Sum]", 
               RowBox[{"i", "=", "0"}], 
               RowBox[{
                RowBox[{"-", "3"}], "+", "m"}]], 
              RowBox[{
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  FormBox["\[CapitalDelta]",
                   TraditionalForm], "\[CenterDot]", 
                  FormBox["r",
                   TraditionalForm]}], ")"}], 
                RowBox[{
                 RowBox[{"-", "3"}], "-", "i", "+", "m"}]], " ", 
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{
                   FormBox["\[CapitalDelta]",
                    TraditionalForm], "\[CenterDot]", 
                   FormBox["q",
                    TraditionalForm]}], "+", 
                  RowBox[{
                   FormBox["\[CapitalDelta]",
                    TraditionalForm], "\[CenterDot]", 
                   FormBox["r",
                    TraditionalForm]}]}], ")"}], "i"]}]}],
             HoldForm],
            TraditionalForm], ")"}], " ", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["q",
            TraditionalForm]}], " ", 
          SuperscriptBox[
           FormBox[
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["r",
              TraditionalForm]}],
            TraditionalForm], "2"]}], "+", 
         RowBox[{
          RowBox[{"(", 
           FormBox[
            TagBox[
             RowBox[{
              UnderoverscriptBox["\[Sum]", 
               RowBox[{"i", "=", "0"}], 
               RowBox[{
                RowBox[{"-", "3"}], "+", "m"}]], 
              RowBox[{
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  FormBox["\[CapitalDelta]",
                   TraditionalForm], "\[CenterDot]", 
                  FormBox["r",
                   TraditionalForm]}], ")"}], 
                RowBox[{
                 RowBox[{"-", "3"}], "-", "i", "+", "m"}]], " ", 
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{
                   FormBox["\[CapitalDelta]",
                    TraditionalForm], "\[CenterDot]", 
                   FormBox["q",
                    TraditionalForm]}], "+", 
                  RowBox[{
                   FormBox["\[CapitalDelta]",
                    TraditionalForm], "\[CenterDot]", 
                   FormBox["r",
                    TraditionalForm]}]}], ")"}], "i"]}]}],
             HoldForm],
            TraditionalForm], ")"}], " ", 
          SuperscriptBox[
           FormBox[
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["q",
              TraditionalForm]}],
            TraditionalForm], "2"], " ", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["r",
            TraditionalForm]}]}]}], ")"}]}]}], "-", 
     FractionBox[
      RowBox[{
       SuperscriptBox["\[Epsilon]", 
        RowBox[{
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["\[Rho]",
           TraditionalForm],
          TraditionalForm], 
         FormBox["\[CapitalDelta]",
          TraditionalForm]}]], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["q",
            TraditionalForm]}], " ", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["r",
            TraditionalForm]}], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["p",
              TraditionalForm]}], ")"}], "m"]}], "+", 
         RowBox[{
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["q",
            TraditionalForm]}], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["r",
              TraditionalForm]}], ")"}], "m"], " ", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["p",
            TraditionalForm]}]}], "+", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["q",
              TraditionalForm]}], ")"}], "m"], " ", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["r",
            TraditionalForm]}], " ", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["p",
            TraditionalForm]}]}]}], ")"}]}], 
      RowBox[{
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["q",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["r",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p",
         TraditionalForm]}]}]]}], ")"}], " ", 
   SubscriptBox["f", 
    RowBox[{
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm]}]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970460694367*^9},
 CellTags->"Twist2GluonOperator",
 CellLabel->"Out[1834]="]
}, Open  ]],

Cell["4-gluon legs", "Text",
 CellTags->"Twist2GluonOperator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Twist2GluonOperator", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"p", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"q", ",", "\[Nu]", ",", "b"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"r", ",", "\[Rho]", ",", "c"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"s", ",", "\[Sigma]", ",", "d"}], "}"}], ",", 
   RowBox[{"Explicit", "\[Rule]", "False"}]}], "]"}]], "Input",
 CellTags->"Twist2GluonOperator",
 CellLabel->"In[1835]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"\[ImaginaryI]", " ", 
   RowBox[{"(", 
    RowBox[{"1", "-", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}], "m"]}], ")"}], " ", 
   SubsuperscriptBox["g", "s", "2"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      RowBox[{
       SubscriptBox["f", 
        RowBox[{
         FormBox[
          FormBox["a",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["b",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["c215",
           TraditionalForm],
          TraditionalForm]}]], " ", 
       SubscriptBox["f", 
        RowBox[{
         FormBox[
          FormBox["c",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["c215",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["d",
           TraditionalForm],
          TraditionalForm]}]], " ", 
       RowBox[{"(", 
        RowBox[{
         SubsuperscriptBox["O", 
          RowBox[{
           FormBox[
            FormBox["\[Mu]",
             TraditionalForm],
            TraditionalForm], "\[VeryThinSpace]", 
           FormBox[
            FormBox["\[Nu]",
             TraditionalForm],
            TraditionalForm], "\[VeryThinSpace]", 
           FormBox[
            FormBox["\[Rho]",
             TraditionalForm],
            TraditionalForm], "\[VeryThinSpace]", 
           FormBox[
            FormBox["\[Sigma]",
             TraditionalForm],
            TraditionalForm]}], "G4"], "(", 
         FormBox["p",
          TraditionalForm], ",", 
         FormBox["q",
          TraditionalForm], ",", 
         FormBox["r",
          TraditionalForm], ",", 
         FormBox["s",
          TraditionalForm], ")"}], ")"}]}]}], "-", 
     RowBox[{
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["a",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["c",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["c215",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["b",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["c215",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["d",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      RowBox[{"(", 
       RowBox[{
        SubsuperscriptBox["O", 
         RowBox[{
          FormBox[
           FormBox["\[Mu]",
            TraditionalForm],
           TraditionalForm], "\[VeryThinSpace]", 
          FormBox[
           FormBox["\[Rho]",
            TraditionalForm],
           TraditionalForm], "\[VeryThinSpace]", 
          FormBox[
           FormBox["\[Nu]",
            TraditionalForm],
           TraditionalForm], "\[VeryThinSpace]", 
          FormBox[
           FormBox["\[Sigma]",
            TraditionalForm],
           TraditionalForm]}], "G4"], "(", 
        FormBox["p",
         TraditionalForm], ",", 
        FormBox["r",
         TraditionalForm], ",", 
        FormBox["q",
         TraditionalForm], ",", 
        FormBox["s",
         TraditionalForm], ")"}], ")"}]}], "+", 
     RowBox[{
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["a",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["c215",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["d",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["b",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["c",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["c215",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      RowBox[{"(", 
       RowBox[{
        SubsuperscriptBox["O", 
         RowBox[{
          FormBox[
           FormBox["\[Rho]",
            TraditionalForm],
           TraditionalForm], "\[VeryThinSpace]", 
          FormBox[
           FormBox["\[Nu]",
            TraditionalForm],
           TraditionalForm], "\[VeryThinSpace]", 
          FormBox[
           FormBox["\[Mu]",
            TraditionalForm],
           TraditionalForm], "\[VeryThinSpace]", 
          FormBox[
           FormBox["\[Sigma]",
            TraditionalForm],
           TraditionalForm]}], "G4"], "(", 
        FormBox["r",
         TraditionalForm], ",", 
        FormBox["q",
         TraditionalForm], ",", 
        FormBox["p",
         TraditionalForm], ",", 
        FormBox["s",
         TraditionalForm], ")"}], ")"}]}]}], ")"}]}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970460984206*^9},
 CellTags->"Twist2GluonOperator",
 CellLabel->"Out[1835]="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Unpolarized case, non-zero-momentum insertion", "Subsubsection",
 CellTags->"Twist2GluonOperator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SetOptions", "[", 
  RowBox[{"Twist2GluonOperator", ",", " ", 
   RowBox[{"Polarization", " ", "\[Rule]", " ", "0"}], ",", 
   RowBox[{"ZeroMomentumInsertion", "\[Rule]", "False"}]}], "]"}]], "Input",
 CellTags->"Twist2GluonOperator",
 CellLabel->"In[1836]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"CouplingConstant", "\[Rule]", 
     SubscriptBox["g", "s"]}], ",", 
    RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
    RowBox[{"Polarization", "\[Rule]", "0"}], ",", 
    RowBox[{"Explicit", "\[Rule]", "All"}], ",", 
    RowBox[{"ZeroMomentumInsertion", "\[Rule]", "False"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970461159739*^9},
 CellTags->"Twist2GluonOperator",
 CellLabel->"Out[1836]="]
}, Open  ]],

Cell["2-gluon legs", "Text",
 CellTags->"Twist2GluonOperator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Twist2GluonOperator", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"p", ",", "q"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\[Mu]", ",", "a"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\[Nu]", ",", "b"}], "}"}]}], "]"}]], "Input",
 CellTags->"Twist2GluonOperator",
 CellLabel->"In[1837]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    FractionBox["1", "2"]}], " ", 
   RowBox[{"(", 
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}], "m"], "+", "1"}], ")"}], " ", 
   SubscriptBox["\[Delta]", 
    RowBox[{
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm]}]], " ", 
   RowBox[{"(", 
    RowBox[{
     FormBox[
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{
         FormBox["\[CapitalDelta]",
          TraditionalForm], "\[CenterDot]", 
         FormBox["p",
          TraditionalForm]}], ")"}], 
       RowBox[{"m", "-", "2"}]],
      TraditionalForm], "+", 
     FormBox[
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{
         FormBox["\[CapitalDelta]",
          TraditionalForm], "\[CenterDot]", 
         FormBox["q",
          TraditionalForm]}], ")"}], 
       RowBox[{"m", "-", "2"}]],
      TraditionalForm]}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["q",
         TraditionalForm]}]}]}], "+", 
     RowBox[{
      SuperscriptBox[
       FormBox["\[CapitalDelta]",
        TraditionalForm], 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox[
       FormBox["q",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      RowBox[{
       FormBox["\[CapitalDelta]",
        TraditionalForm], "\[CenterDot]", 
       FormBox["p",
        TraditionalForm]}]}], "+", 
     RowBox[{
      SuperscriptBox[
       FormBox["\[CapitalDelta]",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], 
          FormBox[
           FormBox["\[Nu]",
            TraditionalForm],
           TraditionalForm]], " ", 
         RowBox[{
          FormBox["\[CapitalDelta]",
           TraditionalForm], "\[CenterDot]", 
          FormBox["q",
           TraditionalForm]}]}], "-", 
        RowBox[{
         SuperscriptBox[
          FormBox["\[CapitalDelta]",
           TraditionalForm], 
          FormBox[
           FormBox["\[Nu]",
            TraditionalForm],
           TraditionalForm]], " ", 
         RowBox[{
          FormBox["p",
           TraditionalForm], "\[CenterDot]", 
          FormBox["q",
           TraditionalForm]}]}]}], ")"}]}]}], ")"}]}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970461364842*^9},
 CellTags->"Twist2GluonOperator",
 CellLabel->"Out[1837]="]
}, Open  ]],

Cell["3-gluon legs", "Text",
 CellTags->"Twist2GluonOperator"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Twist2GluonOperator", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"p", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"q", ",", "\[Nu]", ",", "b"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"r", ",", "\[Rho]", ",", "c"}], "}"}]}], "]"}], ";"}]], "Input",
 CellTags->"Twist2GluonOperator",
 CellLabel->"In[1838]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Twist2GluonOperator", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"p", ",", "\[Mu]"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"q", ",", "\[Nu]"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"r", ",", "\[Rho]"}], "}"}]}], "]"}]], "Input",
 CellTags->"Twist2GluonOperator",
 CellLabel->"In[1839]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    FractionBox["1", "2"]}], " ", 
   RowBox[{"(", 
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}], "m"], "+", "1"}], ")"}], " ", 
   SubscriptBox["\[Delta]", 
    RowBox[{
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["\[Rho]",
       TraditionalForm],
      TraditionalForm]}]], " ", 
   RowBox[{"(", 
    RowBox[{
     FormBox[
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{
         FormBox["\[CapitalDelta]",
          TraditionalForm], "\[CenterDot]", 
         FormBox["\[Mu]",
          TraditionalForm]}], ")"}], 
       RowBox[{"m", "-", "2"}]],
      TraditionalForm], "+", 
     FormBox[
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{
         FormBox["\[CapitalDelta]",
          TraditionalForm], "\[CenterDot]", 
         FormBox["p",
          TraditionalForm]}], ")"}], 
       RowBox[{"m", "-", "2"}]],
      TraditionalForm]}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      RowBox[{
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["\[Mu]",
         TraditionalForm]}], " ", 
       SuperscriptBox["g", 
        RowBox[{"q", "r"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p",
         TraditionalForm]}]}]}], "+", 
     RowBox[{
      SuperscriptBox[
       FormBox["\[Mu]",
        TraditionalForm], 
       FormBox[
        FormBox["q",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox[
       FormBox["\[CapitalDelta]",
        TraditionalForm], 
       FormBox[
        FormBox["r",
         TraditionalForm],
        TraditionalForm]], " ", 
      RowBox[{
       FormBox["\[CapitalDelta]",
        TraditionalForm], "\[CenterDot]", 
       FormBox["p",
        TraditionalForm]}]}], "+", 
     RowBox[{
      SuperscriptBox[
       FormBox["\[CapitalDelta]",
        TraditionalForm], 
       FormBox[
        FormBox["q",
         TraditionalForm],
        TraditionalForm]], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], 
          FormBox[
           FormBox["r",
            TraditionalForm],
           TraditionalForm]], " ", 
         RowBox[{
          FormBox["\[CapitalDelta]",
           TraditionalForm], "\[CenterDot]", 
          FormBox["\[Mu]",
           TraditionalForm]}]}], "-", 
        RowBox[{
         SuperscriptBox[
          FormBox["\[CapitalDelta]",
           TraditionalForm], 
          FormBox[
           FormBox["r",
            TraditionalForm],
           TraditionalForm]], " ", 
         RowBox[{
          FormBox["p",
           TraditionalForm], "\[CenterDot]", 
          FormBox["\[Mu]",
           TraditionalForm]}]}]}], ")"}]}]}], ")"}]}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.5009704618239117`*^9},
 CellTags->"Twist2GluonOperator",
 CellLabel->"Out[1839]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Twist2GluonOperator", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"p", ",", "\[Mu]"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"q", ",", "\[Nu]"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"r", ",", "\[Rho]"}], "}"}], ",", 
   RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"Twist2GluonOperator",
 CellLabel->"In[1840]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    FractionBox["1", "2"]}], " ", 
   RowBox[{"(", 
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}], "m"], "+", "1"}], ")"}], " ", 
   SubscriptBox["\[Delta]", 
    RowBox[{
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["\[Rho]",
       TraditionalForm],
      TraditionalForm]}]], " ", 
   RowBox[{"(", 
    RowBox[{
     FormBox[
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{
         FormBox["\[CapitalDelta]",
          TraditionalForm], "\[CenterDot]", 
         FormBox["\[Mu]",
          TraditionalForm]}], ")"}], 
       RowBox[{"m", "-", "2"}]],
      TraditionalForm], "+", 
     FormBox[
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{
         FormBox["\[CapitalDelta]",
          TraditionalForm], "\[CenterDot]", 
         FormBox["p",
          TraditionalForm]}], ")"}], 
       RowBox[{"m", "-", "2"}]],
      TraditionalForm]}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      RowBox[{
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["\[Mu]",
         TraditionalForm]}], " ", 
       SuperscriptBox["g", 
        RowBox[{"q", "r"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p",
         TraditionalForm]}]}]}], "+", 
     RowBox[{
      SuperscriptBox[
       FormBox["\[Mu]",
        TraditionalForm], 
       FormBox[
        FormBox["q",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox[
       FormBox["\[CapitalDelta]",
        TraditionalForm], 
       FormBox[
        FormBox["r",
         TraditionalForm],
        TraditionalForm]], " ", 
      RowBox[{
       FormBox["\[CapitalDelta]",
        TraditionalForm], "\[CenterDot]", 
       FormBox["p",
        TraditionalForm]}]}], "+", 
     RowBox[{
      SuperscriptBox[
       FormBox["\[CapitalDelta]",
        TraditionalForm], 
       FormBox[
        FormBox["q",
         TraditionalForm],
        TraditionalForm]], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], 
          FormBox[
           FormBox["r",
            TraditionalForm],
           TraditionalForm]], " ", 
         RowBox[{
          FormBox["\[CapitalDelta]",
           TraditionalForm], "\[CenterDot]", 
          FormBox["\[Mu]",
           TraditionalForm]}]}], "-", 
        RowBox[{
         SuperscriptBox[
          FormBox["\[CapitalDelta]",
           TraditionalForm], 
          FormBox[
           FormBox["r",
            TraditionalForm],
           TraditionalForm]], " ", 
         RowBox[{
          FormBox["p",
           TraditionalForm], "\[CenterDot]", 
          FormBox["\[Mu]",
           TraditionalForm]}]}]}], ")"}]}]}], ")"}]}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970462216435*^9},
 CellTags->"Twist2GluonOperator",
 CellLabel->"Out[1840]="]
}, Open  ]],

Cell["4-gluon legs", "Text",
 CellTags->"Twist2GluonOperator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Twist2GluonOperator", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"p", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"q", ",", "\[Nu]", ",", "b"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"r", ",", "\[Rho]", ",", "c"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"s", ",", "\[Sigma]", ",", "d"}], "}"}], ",", 
   RowBox[{"Explicit", "\[Rule]", "False"}]}], "]"}]], "Input",
 CellTags->"Twist2GluonOperator",
 CellLabel->"In[1841]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["1", "2"], " ", 
   RowBox[{"(", 
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}], "m"], "+", "1"}], ")"}], " ", 
   SubsuperscriptBox["g", "s", "2"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      RowBox[{
       SubscriptBox["f", 
        RowBox[{
         FormBox[
          FormBox["a",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["b",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["c216",
           TraditionalForm],
          TraditionalForm]}]], " ", 
       SubscriptBox["f", 
        RowBox[{
         FormBox[
          FormBox["c",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["c216",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["d",
           TraditionalForm],
          TraditionalForm]}]], " ", 
       RowBox[{"(", 
        RowBox[{
         SubsuperscriptBox["O", 
          RowBox[{
           FormBox[
            FormBox["\[Mu]",
             TraditionalForm],
            TraditionalForm], "\[VeryThinSpace]", 
           FormBox[
            FormBox["\[Nu]",
             TraditionalForm],
            TraditionalForm], "\[VeryThinSpace]", 
           FormBox[
            FormBox["\[Rho]",
             TraditionalForm],
            TraditionalForm], "\[VeryThinSpace]", 
           FormBox[
            FormBox["\[Sigma]",
             TraditionalForm],
            TraditionalForm]}], "G4"], "(", 
         FormBox["p",
          TraditionalForm], ",", 
         FormBox["q",
          TraditionalForm], ",", 
         FormBox["r",
          TraditionalForm], ",", 
         FormBox["s",
          TraditionalForm], ")"}], ")"}]}]}], "-", 
     RowBox[{
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["a",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["c",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["c216",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["b",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["c216",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["d",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      RowBox[{"(", 
       RowBox[{
        SubsuperscriptBox["O", 
         RowBox[{
          FormBox[
           FormBox["\[Mu]",
            TraditionalForm],
           TraditionalForm], "\[VeryThinSpace]", 
          FormBox[
           FormBox["\[Rho]",
            TraditionalForm],
           TraditionalForm], "\[VeryThinSpace]", 
          FormBox[
           FormBox["\[Nu]",
            TraditionalForm],
           TraditionalForm], "\[VeryThinSpace]", 
          FormBox[
           FormBox["\[Sigma]",
            TraditionalForm],
           TraditionalForm]}], "G4"], "(", 
        FormBox["p",
         TraditionalForm], ",", 
        FormBox["r",
         TraditionalForm], ",", 
        FormBox["q",
         TraditionalForm], ",", 
        FormBox["s",
         TraditionalForm], ")"}], ")"}]}], "+", 
     RowBox[{
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["a",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["c216",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["d",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["b",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["c",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["c216",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      RowBox[{"(", 
       RowBox[{
        SubsuperscriptBox["O", 
         RowBox[{
          FormBox[
           FormBox["\[Rho]",
            TraditionalForm],
           TraditionalForm], "\[VeryThinSpace]", 
          FormBox[
           FormBox["\[Nu]",
            TraditionalForm],
           TraditionalForm], "\[VeryThinSpace]", 
          FormBox[
           FormBox["\[Mu]",
            TraditionalForm],
           TraditionalForm], "\[VeryThinSpace]", 
          FormBox[
           FormBox["\[Sigma]",
            TraditionalForm],
           TraditionalForm]}], "G4"], "(", 
        FormBox["r",
         TraditionalForm], ",", 
        FormBox["q",
         TraditionalForm], ",", 
        FormBox["p",
         TraditionalForm], ",", 
        FormBox["s",
         TraditionalForm], ")"}], ")"}]}]}], ")"}]}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970462363084*^9},
 CellTags->"Twist2GluonOperator",
 CellLabel->"Out[1841]="]
}, Open  ]],

Cell["Suppress the lengthy output.", "Text",
 CellTags->"Twist2GluonOperator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Short", "[", 
  RowBox[{
   RowBox[{"Twist2GluonOperator", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"p", ",", "\[Mu]"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"q", ",", "\[Nu]"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"r", ",", "\[Rho]"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"s", ",", "\[Sigma]"}], "}"}], ",", 
     RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}], ",", "6"}], 
  "]"}]], "Input",
 CellTags->"Twist2GluonOperator",
 CellLabel->"In[1842]:="],

Cell[BoxData[
 FormBox[
  TagBox[
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        SuperscriptBox[
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm]], " ", 
        SuperscriptBox["g", 
         RowBox[{"\[Nu]", "\[Sigma]"}]], " ", 
        SuperscriptBox[
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox[
          FormBox["\[Rho]",
           TraditionalForm],
          TraditionalForm]]}], "-", 
       RowBox[{
        SuperscriptBox["g", 
         RowBox[{"\[Mu]", "\[Sigma]"}]], " ", 
        SuperscriptBox[
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm]], " ", 
        SuperscriptBox[
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox[
          FormBox["\[Rho]",
           TraditionalForm],
          TraditionalForm]]}], "-", 
       RowBox[{
        SuperscriptBox[
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm]], " ", 
        SuperscriptBox["g", 
         RowBox[{"\[Nu]", "\[Rho]"}]], " ", 
        SuperscriptBox[
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox[
          FormBox["\[Sigma]",
           TraditionalForm],
          TraditionalForm]]}], "+", 
       RowBox[{
        SuperscriptBox["g", 
         RowBox[{"\[Mu]", "\[Rho]"}]], " ", 
        SuperscriptBox[
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm]], " ", 
        SuperscriptBox[
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox[
          FormBox["\[Sigma]",
           TraditionalForm],
          TraditionalForm]]}]}], ")"}], " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         FormBox["\[CapitalDelta]",
          TraditionalForm], "\[CenterDot]", 
         FormBox["r",
          TraditionalForm]}], "+", 
        RowBox[{
         FormBox["\[CapitalDelta]",
          TraditionalForm], "\[CenterDot]", 
         FormBox["s",
          TraditionalForm]}]}], ")"}], 
      RowBox[{"m", "-", "2"}]]}], "+", 
    RowBox[{"\[LeftSkeleton]", "8", "\[RightSkeleton]"}], "+", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       FormBox[
        RowBox[{
         UnderoverscriptBox["\[Sum]", 
          RowBox[{"j", "=", "0"}], 
          RowBox[{
           RowBox[{"-", "4"}], "+", "m"}]], "\<\"\"\>"}],
        TraditionalForm], 
       FormBox[
        RowBox[{"\<\"\"\>", " ", 
         RowBox[{"(", 
          RowBox[{"j", "+", "1"}], ")"}]}],
        TraditionalForm], 
       FormBox[
        RowBox[{
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["q",
             TraditionalForm]}], ")"}], 
          RowBox[{
           RowBox[{"-", "j"}], "+", "m", "-", "4"}]], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"-", 
            RowBox[{"(", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["r",
               TraditionalForm]}], ")"}]}], ")"}], "i"], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            RowBox[{"-", 
             RowBox[{"(", 
              RowBox[{
               FormBox["\[CapitalDelta]",
                TraditionalForm], "\[CenterDot]", 
               FormBox["r",
                TraditionalForm]}], ")"}]}], "-", 
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["s",
              TraditionalForm]}]}], ")"}], 
          RowBox[{"j", "-", "i"}]]}],
        TraditionalForm]}], ")"}], " ", 
     SuperscriptBox[
      FormBox["\[CapitalDelta]",
       TraditionalForm], 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], " ", 
     SuperscriptBox[
      FormBox["\[CapitalDelta]",
       TraditionalForm], 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm]], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"-", 
         SuperscriptBox[
          FormBox["r",
           TraditionalForm], 
          FormBox[
           FormBox["\[Nu]",
            TraditionalForm],
           TraditionalForm]]}], " ", 
        SuperscriptBox[
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox[
          FormBox["\[Rho]",
           TraditionalForm],
          TraditionalForm]], " ", 
        RowBox[{
         FormBox["\[CapitalDelta]",
          TraditionalForm], "\[CenterDot]", 
         FormBox["q",
          TraditionalForm]}]}], "+", 
       RowBox[{
        SuperscriptBox["g", 
         RowBox[{"\[Nu]", "\[Rho]"}]], " ", 
        RowBox[{
         FormBox["\[CapitalDelta]",
          TraditionalForm], "\[CenterDot]", 
         FormBox["r",
          TraditionalForm]}], " ", 
        RowBox[{
         FormBox["\[CapitalDelta]",
          TraditionalForm], "\[CenterDot]", 
         FormBox["q",
          TraditionalForm]}]}], "-", 
       RowBox[{
        SuperscriptBox[
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm]], " ", 
        SuperscriptBox[
         FormBox["q",
          TraditionalForm], 
         FormBox[
          FormBox["\[Rho]",
           TraditionalForm],
          TraditionalForm]], " ", 
        RowBox[{
         FormBox["\[CapitalDelta]",
          TraditionalForm], "\[CenterDot]", 
         FormBox["r",
          TraditionalForm]}]}], "+", 
       RowBox[{
        SuperscriptBox[
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm]], " ", 
        SuperscriptBox[
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox[
          FormBox["\[Rho]",
           TraditionalForm],
          TraditionalForm]], " ", 
        RowBox[{
         FormBox["q",
          TraditionalForm], "\[CenterDot]", 
         FormBox["r",
          TraditionalForm]}]}]}], ")"}]}], "-", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       FormBox[
        RowBox[{
         UnderoverscriptBox["\[Sum]", 
          RowBox[{"j", "=", "0"}], 
          RowBox[{
           RowBox[{"-", "4"}], "+", "m"}]], "\<\"\"\>"}],
        TraditionalForm], 
       FormBox[
        RowBox[{"\<\"\"\>", " ", 
         RowBox[{"(", 
          RowBox[{"j", "+", "1"}], ")"}]}],
        TraditionalForm], 
       FormBox[
        RowBox[{
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["q",
             TraditionalForm]}], ")"}], 
          RowBox[{
           RowBox[{"-", "j"}], "+", "m", "-", "4"}]], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            RowBox[{"-", 
             RowBox[{"(", 
              RowBox[{
               FormBox["\[CapitalDelta]",
                TraditionalForm], "\[CenterDot]", 
               FormBox["r",
                TraditionalForm]}], ")"}]}], "-", 
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["s",
              TraditionalForm]}]}], ")"}], 
          RowBox[{"j", "-", "i"}]], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"-", 
            RowBox[{"(", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["s",
               TraditionalForm]}], ")"}]}], ")"}], "i"]}],
        TraditionalForm]}], ")"}], " ", 
     SuperscriptBox[
      FormBox["\[CapitalDelta]",
       TraditionalForm], 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], " ", 
     SuperscriptBox[
      FormBox["\[CapitalDelta]",
       TraditionalForm], 
      FormBox[
       FormBox["\[Rho]",
        TraditionalForm],
       TraditionalForm]], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"-", 
         SuperscriptBox[
          FormBox["s",
           TraditionalForm], 
          FormBox[
           FormBox["\[Nu]",
            TraditionalForm],
           TraditionalForm]]}], " ", 
        SuperscriptBox[
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox[
          FormBox["\[Sigma]",
           TraditionalForm],
          TraditionalForm]], " ", 
        RowBox[{
         FormBox["\[CapitalDelta]",
          TraditionalForm], "\[CenterDot]", 
         FormBox["q",
          TraditionalForm]}]}], "+", 
       RowBox[{
        SuperscriptBox["g", 
         RowBox[{"\[Nu]", "\[Sigma]"}]], " ", 
        RowBox[{
         FormBox["\[CapitalDelta]",
          TraditionalForm], "\[CenterDot]", 
         FormBox["s",
          TraditionalForm]}], " ", 
        RowBox[{
         FormBox["\[CapitalDelta]",
          TraditionalForm], "\[CenterDot]", 
         FormBox["q",
          TraditionalForm]}]}], "-", 
       RowBox[{
        SuperscriptBox[
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm]], " ", 
        SuperscriptBox[
         FormBox["q",
          TraditionalForm], 
         FormBox[
          FormBox["\[Sigma]",
           TraditionalForm],
          TraditionalForm]], " ", 
        RowBox[{
         FormBox["\[CapitalDelta]",
          TraditionalForm], "\[CenterDot]", 
         FormBox["s",
          TraditionalForm]}]}], "+", 
       RowBox[{
        SuperscriptBox[
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm]], " ", 
        SuperscriptBox[
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox[
          FormBox["\[Sigma]",
           TraditionalForm],
          TraditionalForm]], " ", 
        RowBox[{
         FormBox["q",
          TraditionalForm], "\[CenterDot]", 
         FormBox["s",
          TraditionalForm]}]}]}], ")"}]}]}],
   Short[#, 6]& ], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970462514502*^9},
 CellTags->"Twist2GluonOperator",
 CellLabel->"Out[1842]//Short="]
}, Open  ]],

Cell["\<\
The setting Explicit \[Rule] All performs the sums.\
\>", "Text",
 CellTags->"Twist2GluonOperator"],

Cell[BoxData[
 RowBox[{
  RowBox[{"SetOptions", "[", 
   RowBox[{"Twist2GluonOperator", ",", " ", 
    RowBox[{"Polarization", " ", "\[Rule]", " ", "1"}], ",", " ", 
    RowBox[{"Explicit", "\[Rule]", "All"}]}], "]"}], ";"}]], "Input",
 CellTags->"Twist2GluonOperator",
 CellLabel->"In[1843]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Short", "[", 
  RowBox[{
   RowBox[{"Twist2GluonOperator", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"p", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"q", ",", "\[Nu]", ",", "b"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"r", ",", "\[Rho]", ",", "c"}], "}"}]}], "]"}], ",", "5"}], 
  "]"}]], "Input",
 CellTags->"Twist2GluonOperator",
 CellLabel->"In[1844]:="],

Cell[BoxData[
 FormBox[
  TagBox[
   RowBox[{
    SubscriptBox["g", "s"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"-", 
        SuperscriptBox["\[Epsilon]", 
         RowBox[{
          FormBox[
           FormBox["\[Nu]",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["\[Rho]",
            TraditionalForm],
           TraditionalForm], 
          FormBox["\[CapitalDelta]",
           TraditionalForm], 
          FormBox["p",
           TraditionalForm]}]]}], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{
          FormBox["\[CapitalDelta]",
           TraditionalForm], "\[CenterDot]", 
          FormBox["p",
           TraditionalForm]}], ")"}], 
        RowBox[{"m", "-", "2"}]]}], "+", 
      RowBox[{
       SuperscriptBox["\[Epsilon]", 
        RowBox[{
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["\[Rho]",
           TraditionalForm],
          TraditionalForm], 
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox["q",
          TraditionalForm]}]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{
          FormBox["\[CapitalDelta]",
           TraditionalForm], "\[CenterDot]", 
          FormBox["q",
           TraditionalForm]}], ")"}], 
        RowBox[{"m", "-", "2"}]]}], "-", 
      RowBox[{
       SuperscriptBox["\[Epsilon]", 
        RowBox[{
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox["r",
          TraditionalForm]}]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{
          FormBox["\[CapitalDelta]",
           TraditionalForm], "\[CenterDot]", 
          FormBox["r",
           TraditionalForm]}], ")"}], 
        RowBox[{"m", "-", "2"}]]}], "-", 
      RowBox[{
       SuperscriptBox["\[Epsilon]", 
        RowBox[{
         FormBox[
          FormBox["\[Rho]",
           TraditionalForm],
          TraditionalForm], 
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox["p",
          TraditionalForm], 
         FormBox["r",
          TraditionalForm]}]], " ", 
       RowBox[{"(", 
        FormBox[
         RowBox[{
          UnderoverscriptBox["\[Sum]", 
           RowBox[{"i", "=", "0"}], 
           RowBox[{
            RowBox[{"-", "3"}], "+", "m"}]], 
          RowBox[{
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["p",
               TraditionalForm]}], ")"}], 
            RowBox[{
             RowBox[{"-", "3"}], "-", "i", "+", "m"}]], " ", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{"\[LeftSkeleton]", "1", "\[RightSkeleton]"}], ")"}], 
            "i"]}]}],
         TraditionalForm], ")"}], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         RowBox[{"\[LeftSkeleton]", "1", "\[RightSkeleton]"}],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]]}], "+", 
      RowBox[{"\[LeftSkeleton]", "15", "\[RightSkeleton]"}], "+", 
      RowBox[{
       FractionBox["1", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["q",
             TraditionalForm]}], "+", 
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["r",
             TraditionalForm]}]}], ")"}], "2"]], 
       RowBox[{
        SuperscriptBox["\[Epsilon]", 
         RowBox[{
          FormBox[
           FormBox["\[Mu]",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["\[Rho]",
            TraditionalForm],
           TraditionalForm], 
          FormBox["\[CapitalDelta]",
           TraditionalForm], 
          FormBox["p",
           TraditionalForm]}]], " ", 
        RowBox[{"\[LeftSkeleton]", "1", "\[RightSkeleton]"}], " ", 
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["q",
               TraditionalForm]}], "+", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["r",
               TraditionalForm]}]}], ")"}], "m"], "+", 
          RowBox[{
           RowBox[{"(", 
            FormBox[
             RowBox[{
              UnderoverscriptBox["\[Sum]", 
               RowBox[{"i", "=", "0"}], 
               RowBox[{
                RowBox[{"-", "3"}], "+", "m"}]], 
              RowBox[{
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  FormBox["\[CapitalDelta]",
                   TraditionalForm], "\[CenterDot]", 
                  FormBox["r",
                   TraditionalForm]}], ")"}], 
                RowBox[{
                 RowBox[{"-", "3"}], "-", "i", "+", "m"}]], " ", 
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{"\[LeftSkeleton]", "1", "\[RightSkeleton]"}], ")"}], 
                "i"]}]}],
             TraditionalForm], ")"}], " ", 
           SuperscriptBox[
            FormBox[
             RowBox[{"\[LeftSkeleton]", "1", "\[RightSkeleton]"}],
             TraditionalForm], "3"]}], "+", 
          RowBox[{"2", " ", 
           RowBox[{"\[LeftSkeleton]", "2", "\[RightSkeleton]"}], " ", 
           SuperscriptBox[
            RowBox[{"\[LeftSkeleton]", "1", "\[RightSkeleton]"}], "2"]}], "+", 
          RowBox[{
           RowBox[{"(", 
            FormBox[
             RowBox[{
              UnderoverscriptBox["\[Sum]", 
               RowBox[{"i", "=", "0"}], 
               RowBox[{
                RowBox[{"-", "3"}], "+", "m"}]], 
              RowBox[{
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  FormBox["\[CapitalDelta]",
                   TraditionalForm], "\[CenterDot]", 
                  FormBox["r",
                   TraditionalForm]}], ")"}], 
                RowBox[{
                 RowBox[{"-", "3"}], "-", "i", "+", "m"}]], " ", 
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{
                   FormBox["\[CapitalDelta]",
                    TraditionalForm], "\[CenterDot]", 
                   FormBox["q",
                    TraditionalForm]}], "+", 
                  RowBox[{
                   FormBox["\[CapitalDelta]",
                    TraditionalForm], "\[CenterDot]", 
                   FormBox["r",
                    TraditionalForm]}]}], ")"}], "i"]}]}],
             TraditionalForm], ")"}], " ", 
           SuperscriptBox[
            FormBox[
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["q",
               TraditionalForm]}],
             TraditionalForm], "2"], " ", 
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["r",
             TraditionalForm]}]}]}], ")"}]}]}], "-", 
      FractionBox[
       RowBox[{
        SuperscriptBox["\[Epsilon]", 
         RowBox[{
          FormBox[
           FormBox["\[Mu]",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["\[Nu]",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["\[Rho]",
            TraditionalForm],
           TraditionalForm], 
          FormBox["\[CapitalDelta]",
           TraditionalForm]}]], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["q",
             TraditionalForm]}], " ", 
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["r",
             TraditionalForm]}], " ", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["p",
               TraditionalForm]}], ")"}], "m"]}], "+", 
          RowBox[{
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["q",
             TraditionalForm]}], " ", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["r",
               TraditionalForm]}], ")"}], "m"], " ", 
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["p",
             TraditionalForm]}]}], "+", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["q",
               TraditionalForm]}], ")"}], "m"], " ", 
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["r",
             TraditionalForm]}], " ", 
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["p",
             TraditionalForm]}]}]}], ")"}]}], 
       RowBox[{
        RowBox[{
         FormBox["\[CapitalDelta]",
          TraditionalForm], "\[CenterDot]", 
         FormBox["q",
          TraditionalForm]}], " ", 
        RowBox[{
         FormBox["\[CapitalDelta]",
          TraditionalForm], "\[CenterDot]", 
         FormBox["r",
          TraditionalForm]}], " ", 
        RowBox[{
         FormBox["\[CapitalDelta]",
          TraditionalForm], "\[CenterDot]", 
         FormBox["p",
          TraditionalForm]}]}]]}], ")"}], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm]}]]}],
   Short[#, 5]& ], TraditionalForm]], "Output",
 CellChangeTimes->{3.50097046278925*^9},
 CellTags->"Twist2GluonOperator",
 CellLabel->"Out[1844]//Short="]
}, Open  ]]
}, Open  ]],

Cell["Twist2QuarkOperator", "ObjectName",
 CellTags->"Twist2QuarkOperator"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Twist2QuarkOperator"],

Cell["\<\
Twist2QuarkOperator[p] or Twist2QuarkOperator[p,_,_] yields the \
quark-antiquark operator (p is momentum in the direction of the incoming \
quark). Twist2QuarkOperator[{p,q}] yields the 2-quark operator for non-zero \
momentum insertion (p is momentum in the direction of the incoming quark). \
Twist2QuarkOperator[{p1,___}, {p2,___}, {p3, mu, a}] or \
Twist2QuarkOperator[p1,_,_, p2,_,_, p3,mu,a] is the quark-antiquark-gluon \
operator, where p1 is the incoming quark, p2 the incoming antiquark and p3 \
denotes the incoming gluon momentum. Twist2QuarkOperator[{p1}, {p2}, {p3, mu, \
a}, {p4, nu, b}] or Twist2QuarkOperator[{p1,___}, {p2,___}, {p3, mu, a}, {p4, \
nu, b}] or Twist2QuarkOperator[p1,_,_, p2,_,_, p3,mu,a, p4, nu, b] gives the \
Quark-Quark-Gluon-Gluon-operator. The setting of the option Polarization \
(unpolarized: 0; polarized: 1) determines whether the unpolarized or \
polarized operator is returned\
\>", "Text",
 CellTags->"Twist2QuarkOperator"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Twist2GluonOperator",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Twist2GluonOperator",
  ButtonNote->"Twist2GluonOperator"],
 "."
}], "Text",
 CellTags->"Twist2QuarkOperator"]
}, Open  ]],

Cell["Examples", "Subsubsection",
 CellTags->"Twist2QuarkOperator"],

Cell[CellGroupData[{

Cell["Polarized case, zero-momentum insertion", "Subsubsection",
 CellTags->"Twist2QuarkOperator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SetOptions", "[", 
  RowBox[{"Twist2QuarkOperator", ",", 
   RowBox[{"Polarization", "\[Rule]", "1"}], ",", 
   RowBox[{"ZeroMomentumInsertion", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"Twist2QuarkOperator",
 CellLabel->"In[1845]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"CouplingConstant", "\[Rule]", 
     SubscriptBox["g", "s"]}], ",", 
    RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
    RowBox[{"Explicit", "\[Rule]", "True"}], ",", 
    RowBox[{"Polarization", "\[Rule]", "1"}], ",", 
    RowBox[{"ZeroMomentumInsertion", "\[Rule]", "True"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.5009704630390368`*^9},
 CellTags->"Twist2QuarkOperator",
 CellLabel->"Out[1845]="]
}, Open  ]],

Cell["Quark-antiquark operator.", "Text",
 CellTags->"Twist2QuarkOperator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t1", "=", 
  RowBox[{"Twist2QuarkOperator", "[", "p", "]"}]}]], "Input",
 CellTags->"Twist2QuarkOperator",
 CellLabel->"In[1846]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       FormBox["\<\"\[Gamma]\"\>",
        TraditionalForm], "\[CenterDot]", 
       FormBox["\[CapitalDelta]",
        TraditionalForm]}], ")"}], ".", 
     SuperscriptBox["\[Gamma]", "5"]}]}], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{
      FormBox["\[CapitalDelta]",
       TraditionalForm], "\[CenterDot]", 
      FormBox["p",
       TraditionalForm]}], ")"}], 
    RowBox[{"m", "-", "1"}]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970463094718*^9},
 CellTags->"Twist2QuarkOperator",
 CellLabel->"Out[1846]="]
}, Open  ]],

Cell["Quark-antiquark-gluon operator.", "Text",
 CellTags->"Twist2QuarkOperator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t2", "=", 
  RowBox[{"Twist2QuarkOperator", "[", 
   RowBox[{
    RowBox[{"{", "p", "}"}], ",", 
    RowBox[{"{", "q", "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"k", ",", "\[Mu]", ",", "a"}], "}"}]}], "]"}]}]], "Input",
 CellTags->"Twist2QuarkOperator",
 CellLabel->"In[1847]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox[
    FormBox["\[CapitalDelta]",
     TraditionalForm], 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], " ", 
   RowBox[{"(", 
    RowBox[{"-", 
     SubscriptBox["g", "s"]}], ")"}], " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      FormBox["\<\"\[Gamma]\"\>",
       TraditionalForm], "\[CenterDot]", 
      FormBox["\[CapitalDelta]",
       TraditionalForm]}], ")"}], ".", 
    SuperscriptBox["\[Gamma]", "5"], ".", 
    SubscriptBox["T", 
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm]]}], " ", 
   RowBox[{"(", 
    FormBox[
     TagBox[
      RowBox[{
       UnderoverscriptBox["\[Sum]", 
        RowBox[{"i", "=", "0"}], 
        RowBox[{
         RowBox[{"-", "2"}], "+", "m"}]], 
       RowBox[{
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"-", "1"}], ")"}], "i"], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["p",
            TraditionalForm]}], ")"}], 
         RowBox[{
          RowBox[{"-", "2"}], "-", "i", "+", "m"}]], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["q",
            TraditionalForm]}], ")"}], "i"]}]}],
      HoldForm],
     TraditionalForm], ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009704632457123`*^9},
 CellTags->"Twist2QuarkOperator",
 CellLabel->"Out[1847]="]
}, Open  ]],

Cell["Quark-antiquark-gluon-gluon operator.", "Text",
 CellTags->"Twist2QuarkOperator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t3", "=", 
  RowBox[{"Twist2QuarkOperator", "[", 
   RowBox[{
    RowBox[{"{", "p", "}"}], ",", 
    RowBox[{"{", "q", "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"k", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"r", ",", "\[Nu]", ",", "b"}], "}"}]}], "]"}]}]], "Input",
 CellTags->"Twist2QuarkOperator",
 CellLabel->"In[1848]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox[
    FormBox["\[CapitalDelta]",
     TraditionalForm], 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], " ", 
   SuperscriptBox[
    FormBox["\[CapitalDelta]",
     TraditionalForm], 
    FormBox[
     FormBox["\[Nu]",
      TraditionalForm],
     TraditionalForm]], " ", 
   RowBox[{"(", 
    RowBox[{"-", 
     SubsuperscriptBox["g", "s", "2"]}], ")"}], " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      FormBox["\<\"\[Gamma]\"\>",
       TraditionalForm], "\[CenterDot]", 
      FormBox["\[CapitalDelta]",
       TraditionalForm]}], ")"}], ".", 
    SuperscriptBox["\[Gamma]", "5"], ".", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{
        SubscriptBox["T", 
         FormBox[
          FormBox["b",
           TraditionalForm],
          TraditionalForm]], ".", 
        SubscriptBox["T", 
         FormBox[
          FormBox["a",
           TraditionalForm],
          TraditionalForm]]}], " ", 
       RowBox[{"(", 
        RowBox[{
         FormBox[
          TagBox[
           RowBox[{
            UnderoverscriptBox["\[Sum]", 
             RowBox[{"j", "=", "0"}], 
             RowBox[{
              RowBox[{"-", "3"}], "+", "m"}]], "\<\"\"\>"}],
           HoldForm],
          TraditionalForm], 
         FormBox[
          RowBox[{"\<\"\"\>", " ", 
           RowBox[{"(", 
            RowBox[{"j", "+", "1"}], ")"}]}],
          TraditionalForm], 
         FormBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{"-", "1"}], ")"}], "j"], " ", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["p",
               TraditionalForm]}], ")"}], "i"], " ", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["q",
               TraditionalForm]}], ")"}], 
            RowBox[{
             RowBox[{"-", "j"}], "+", "m", "-", "3"}]], " ", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               FormBox["\[CapitalDelta]",
                TraditionalForm], "\[CenterDot]", 
               FormBox["p",
                TraditionalForm]}], "+", 
              RowBox[{
               FormBox["\[CapitalDelta]",
                TraditionalForm], "\[CenterDot]", 
               FormBox["r",
                TraditionalForm]}]}], ")"}], 
            RowBox[{"j", "-", "i"}]]}],
          TraditionalForm]}], ")"}]}], "-", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"-", "1"}], ")"}], "m"], " ", 
       RowBox[{
        SubscriptBox["T", 
         FormBox[
          FormBox["a",
           TraditionalForm],
          TraditionalForm]], ".", 
        SubscriptBox["T", 
         FormBox[
          FormBox["b",
           TraditionalForm],
          TraditionalForm]]}], " ", 
       RowBox[{"(", 
        RowBox[{
         FormBox[
          TagBox[
           RowBox[{
            UnderoverscriptBox["\[Sum]", 
             RowBox[{"j", "=", "0"}], 
             RowBox[{
              RowBox[{"-", "3"}], "+", "m"}]], "\<\"\"\>"}],
           HoldForm],
          TraditionalForm], 
         FormBox[
          RowBox[{"\<\"\"\>", " ", 
           RowBox[{"(", 
            RowBox[{"j", "+", "1"}], ")"}]}],
          TraditionalForm], 
         FormBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{"-", "1"}], ")"}], "j"], " ", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["q",
               TraditionalForm]}], ")"}], "i"], " ", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["p",
               TraditionalForm]}], ")"}], 
            RowBox[{
             RowBox[{"-", "j"}], "+", "m", "-", "3"}]], " ", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               FormBox["\[CapitalDelta]",
                TraditionalForm], "\[CenterDot]", 
               FormBox["q",
                TraditionalForm]}], "+", 
              RowBox[{
               FormBox["\[CapitalDelta]",
                TraditionalForm], "\[CenterDot]", 
               FormBox["r",
                TraditionalForm]}]}], ")"}], 
            RowBox[{"j", "-", "i"}]]}],
          TraditionalForm]}], ")"}]}]}], ")"}]}]}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970463623042*^9},
 CellTags->"Twist2QuarkOperator",
 CellLabel->"Out[1848]="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Unpolarized case, zero-momentum insertion", "Subsubsection",
 CellTags->"Twist2QuarkOperator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SetOptions", "[", 
  RowBox[{"Twist2QuarkOperator", ",", " ", 
   RowBox[{"Polarization", "\[Rule]", "0"}], ",", 
   RowBox[{"ZeroMomentumInsertion", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"Twist2QuarkOperator",
 CellLabel->"In[1849]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"CouplingConstant", "\[Rule]", 
     SubscriptBox["g", "s"]}], ",", 
    RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
    RowBox[{"Explicit", "\[Rule]", "True"}], ",", 
    RowBox[{"Polarization", "\[Rule]", "0"}], ",", 
    RowBox[{"ZeroMomentumInsertion", "\[Rule]", "True"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970463688986*^9},
 CellTags->"Twist2QuarkOperator",
 CellLabel->"Out[1849]="]
}, Open  ]],

Cell["Quark-antiquark operator.", "Text",
 CellTags->"Twist2QuarkOperator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t4", "=", 
  RowBox[{"Twist2QuarkOperator", "[", "p", "]"}]}]], "Input",
 CellTags->"Twist2QuarkOperator",
 CellLabel->"In[1850]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    FormBox["\<\"\[Gamma]\"\>",
     TraditionalForm], "\[CenterDot]", 
    FormBox["\[CapitalDelta]",
     TraditionalForm]}], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{
      FormBox["\[CapitalDelta]",
       TraditionalForm], "\[CenterDot]", 
      FormBox["p",
       TraditionalForm]}], ")"}], 
    RowBox[{"m", "-", "1"}]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970463755718*^9},
 CellTags->"Twist2QuarkOperator",
 CellLabel->"Out[1850]="]
}, Open  ]],

Cell["Quark-antiquark-gluon operator.", "Text",
 CellTags->"Twist2QuarkOperator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t5", "=", 
  RowBox[{"Twist2QuarkOperator", "[", 
   RowBox[{
    RowBox[{"{", "p", "}"}], ",", 
    RowBox[{"{", "q", "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"k", ",", "\[Mu]", ",", "a"}], "}"}]}], "]"}]}]], "Input",
 CellTags->"Twist2QuarkOperator",
 CellLabel->"In[1851]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox[
    FormBox["\[CapitalDelta]",
     TraditionalForm], 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], " ", 
   SubscriptBox["g", "s"], " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      FormBox["\<\"\[Gamma]\"\>",
       TraditionalForm], "\[CenterDot]", 
      FormBox["\[CapitalDelta]",
       TraditionalForm]}], ")"}], ".", 
    SubscriptBox["T", 
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm]]}], " ", 
   RowBox[{"(", 
    FormBox[
     TagBox[
      RowBox[{
       UnderoverscriptBox["\[Sum]", 
        RowBox[{"i", "=", "0"}], 
        RowBox[{
         RowBox[{"-", "2"}], "+", "m"}]], 
       RowBox[{
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"-", "1"}], ")"}], "i"], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["p",
            TraditionalForm]}], ")"}], 
         RowBox[{
          RowBox[{"-", "2"}], "-", "i", "+", "m"}]], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["q",
            TraditionalForm]}], ")"}], "i"]}]}],
      HoldForm],
     TraditionalForm], ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009704641324368`*^9},
 CellTags->"Twist2QuarkOperator",
 CellLabel->"Out[1851]="]
}, Open  ]],

Cell["Quark-antiquark-gluon-gluon operator.", "Text",
 CellTags->"Twist2QuarkOperator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t6", "=", 
  RowBox[{"Twist2QuarkOperator", "[", 
   RowBox[{
    RowBox[{"{", "p", "}"}], ",", 
    RowBox[{"{", "q", "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"k", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"r", ",", "\[Nu]", ",", "b"}], "}"}]}], "]"}]}]], "Input",
 CellTags->"Twist2QuarkOperator",
 CellLabel->"In[1852]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"-", "1"}], ")"}], "m"]}], " ", 
   SuperscriptBox[
    FormBox["\[CapitalDelta]",
     TraditionalForm], 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], " ", 
   SuperscriptBox[
    FormBox["\[CapitalDelta]",
     TraditionalForm], 
    FormBox[
     FormBox["\[Nu]",
      TraditionalForm],
     TraditionalForm]], " ", 
   SubsuperscriptBox["g", "s", "2"], " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      FormBox["\<\"\[Gamma]\"\>",
       TraditionalForm], "\[CenterDot]", 
      FormBox["\[CapitalDelta]",
       TraditionalForm]}], ")"}], ".", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{
        SubscriptBox["T", 
         FormBox[
          FormBox["a",
           TraditionalForm],
          TraditionalForm]], ".", 
        SubscriptBox["T", 
         FormBox[
          FormBox["b",
           TraditionalForm],
          TraditionalForm]]}], " ", 
       RowBox[{"(", 
        RowBox[{
         FormBox[
          TagBox[
           RowBox[{
            UnderoverscriptBox["\[Sum]", 
             RowBox[{"i", "=", "0"}], 
             RowBox[{
              RowBox[{"-", "3"}], "+", "m"}]], "\<\"\"\>"}],
           HoldForm],
          TraditionalForm], 
         FormBox[
          RowBox[{"\<\"\"\>", " ", 
           RowBox[{"(", 
            RowBox[{"i", "+", "1"}], ")"}]}],
          TraditionalForm], 
         FormBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["q",
               TraditionalForm]}], ")"}], "j"], " ", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{"-", 
              RowBox[{"(", 
               RowBox[{
                FormBox["\[CapitalDelta]",
                 TraditionalForm], "\[CenterDot]", 
                FormBox["p",
                 TraditionalForm]}], ")"}]}], ")"}], 
            RowBox[{
             RowBox[{"-", "i"}], "+", "m", "-", "3"}]], " ", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               FormBox["k",
                TraditionalForm], "\[CenterDot]", 
               FormBox["\[CapitalDelta]",
                TraditionalForm]}], "+", 
              RowBox[{
               FormBox["\[CapitalDelta]",
                TraditionalForm], "\[CenterDot]", 
               FormBox["q",
                TraditionalForm]}]}], ")"}], 
            RowBox[{"i", "-", "j"}]]}],
          TraditionalForm]}], ")"}]}], "+", 
      RowBox[{
       RowBox[{
        SubscriptBox["T", 
         FormBox[
          FormBox["b",
           TraditionalForm],
          TraditionalForm]], ".", 
        SubscriptBox["T", 
         FormBox[
          FormBox["a",
           TraditionalForm],
          TraditionalForm]]}], " ", 
       RowBox[{"(", 
        RowBox[{
         FormBox[
          TagBox[
           RowBox[{
            UnderoverscriptBox["\[Sum]", 
             RowBox[{"i", "=", "0"}], 
             RowBox[{
              RowBox[{"-", "3"}], "+", "m"}]], "\<\"\"\>"}],
           HoldForm],
          TraditionalForm], 
         FormBox[
          RowBox[{"\<\"\"\>", " ", 
           RowBox[{"(", 
            RowBox[{"i", "+", "1"}], ")"}]}],
          TraditionalForm], 
         FormBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["q",
               TraditionalForm]}], ")"}], "j"], " ", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{"-", 
              RowBox[{"(", 
               RowBox[{
                FormBox["\[CapitalDelta]",
                 TraditionalForm], "\[CenterDot]", 
                FormBox["p",
                 TraditionalForm]}], ")"}]}], ")"}], 
            RowBox[{
             RowBox[{"-", "i"}], "+", "m", "-", "3"}]], " ", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               FormBox["\[CapitalDelta]",
                TraditionalForm], "\[CenterDot]", 
               FormBox["q",
                TraditionalForm]}], "+", 
              RowBox[{
               FormBox["\[CapitalDelta]",
                TraditionalForm], "\[CenterDot]", 
               FormBox["r",
                TraditionalForm]}]}], ")"}], 
            RowBox[{"i", "-", "j"}]]}],
          TraditionalForm]}], ")"}]}]}], ")"}]}]}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.5009704644037*^9},
 CellTags->"Twist2QuarkOperator",
 CellLabel->"Out[1852]="]
}, Open  ]],

Cell["This shows the FeynCalcExternal (FCE) form.", "Text",
 CellTags->"Twist2QuarkOperator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Twist2QuarkOperator", "[", "p", "]"}], "//", "FCE"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"Twist2QuarkOperator",
 CellLabel->"In[1853]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"GSD", "[", "OPEDelta", "]"}], " ", 
  SuperscriptBox[
   RowBox[{"SOD", "[", "p", "]"}], 
   RowBox[{
    RowBox[{"-", "1"}], "+", "OPEm"}]]}]], "Output",
 CellChangeTimes->{3.500970464561555*^9},
 CellTags->"Twist2QuarkOperator",
 CellLabel->"Out[1853]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Twist2QuarkOperator", "[", 
  RowBox[{
   RowBox[{"{", "p", "}"}], ",", 
   RowBox[{"{", "q", "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"k", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
   RowBox[{"Explicit", "\[Rule]", "All"}], ",", 
   RowBox[{"Polarization", "\[Rule]", "0"}]}], "]"}]], "Input",
 CellTags->"Twist2QuarkOperator",
 CellLabel->"In[1854]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox[
    FormBox["\[CapitalDelta]",
     TraditionalForm], 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], " ", 
   RowBox[{"(", 
    RowBox[{"-", 
     SubscriptBox["g", "s"]}], ")"}], " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      FormBox["\<\"\[Gamma]\"\>",
       TraditionalForm], "\[CenterDot]", 
      FormBox["\[CapitalDelta]",
       TraditionalForm]}], ")"}], ".", 
    SubscriptBox["T", 
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm]]}], " ", 
   RowBox[{"(", 
    RowBox[{
     FractionBox[
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{
         FormBox["\[CapitalDelta]",
          TraditionalForm], "\[CenterDot]", 
         FormBox["p",
          TraditionalForm]}], ")"}], 
       RowBox[{"m", "-", "1"}]], 
      RowBox[{
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p",
         TraditionalForm]}], "+", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["q",
         TraditionalForm]}]}]], "+", 
     FractionBox[
      RowBox[{
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"-", "1"}], ")"}], "m"], " ", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{
          FormBox["\[CapitalDelta]",
           TraditionalForm], "\[CenterDot]", 
          FormBox["q",
           TraditionalForm]}], ")"}], 
        RowBox[{"m", "-", "1"}]]}], 
      RowBox[{
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p",
         TraditionalForm]}], "+", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["q",
         TraditionalForm]}]}]]}], ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970464691999*^9},
 CellTags->"Twist2QuarkOperator",
 CellLabel->"Out[1854]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Twist2QuarkOperator", "[", 
  RowBox[{
   RowBox[{"{", "p", "}"}], ",", 
   RowBox[{"{", "q", "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"k", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"r", ",", "\[Nu]", ",", "b"}], "}"}], ",", 
   RowBox[{"Explicit", "\[Rule]", "All"}], ",", 
   RowBox[{"Polarization", "\[Rule]", "0"}]}], "]"}]], "Input",
 CellTags->"Twist2QuarkOperator",
 CellLabel->"In[1855]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox[
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"-", "1"}], ")"}], "m"],
    TraditionalForm], " ", 
   SuperscriptBox[
    FormBox["\[CapitalDelta]",
     TraditionalForm], 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], " ", 
   SuperscriptBox[
    FormBox["\[CapitalDelta]",
     TraditionalForm], 
    FormBox[
     FormBox["\[Nu]",
      TraditionalForm],
     TraditionalForm]], " ", 
   RowBox[{"(", 
    RowBox[{"-", 
     SubsuperscriptBox["g", "s", "2"]}], ")"}], " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      FormBox["\<\"\[Gamma]\"\>",
       TraditionalForm], "\[CenterDot]", 
      FormBox["\[CapitalDelta]",
       TraditionalForm]}], ")"}], ".", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{
        SubscriptBox["T", 
         FormBox[
          FormBox["a",
           TraditionalForm],
          TraditionalForm]], ".", 
        SubscriptBox["T", 
         FormBox[
          FormBox["b",
           TraditionalForm],
          TraditionalForm]]}], " ", 
       RowBox[{"(", 
        RowBox[{
         FractionBox[
          RowBox[{
           FormBox[
            SuperscriptBox[
             RowBox[{"(", 
              RowBox[{"-", "1"}], ")"}], 
             RowBox[{"m", "-", "1"}]],
            TraditionalForm], " ", 
           FormBox[
            SuperscriptBox[
             RowBox[{"(", 
              RowBox[{
               FormBox["\[CapitalDelta]",
                TraditionalForm], "\[CenterDot]", 
               FormBox["p",
                TraditionalForm]}], ")"}], 
             RowBox[{"m", "-", "1"}]],
            TraditionalForm]}], 
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["p",
               TraditionalForm]}], "+", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["q",
               TraditionalForm]}]}], ")"}], " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{
              FormBox["k",
               TraditionalForm], "\[CenterDot]", 
              FormBox["\[CapitalDelta]",
               TraditionalForm]}], "+", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["p",
               TraditionalForm]}], "+", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["q",
               TraditionalForm]}]}], ")"}]}]], "-", 
         FractionBox[
          FormBox[
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["q",
               TraditionalForm]}], ")"}], 
            RowBox[{"m", "-", "1"}]],
           TraditionalForm], 
          RowBox[{
           RowBox[{
            FormBox["k",
             TraditionalForm], "\[CenterDot]", 
            FormBox["\[CapitalDelta]",
             TraditionalForm]}], " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["p",
               TraditionalForm]}], "+", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["q",
               TraditionalForm]}]}], ")"}]}]], "+", 
         FractionBox[
          FormBox[
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               FormBox["k",
                TraditionalForm], "\[CenterDot]", 
               FormBox["\[CapitalDelta]",
                TraditionalForm]}], "+", 
              RowBox[{
               FormBox["\[CapitalDelta]",
                TraditionalForm], "\[CenterDot]", 
               FormBox["q",
                TraditionalForm]}]}], ")"}], 
            RowBox[{"m", "-", "1"}]],
           TraditionalForm], 
          RowBox[{
           RowBox[{
            FormBox["k",
             TraditionalForm], "\[CenterDot]", 
            FormBox["\[CapitalDelta]",
             TraditionalForm]}], " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{
              FormBox["k",
               TraditionalForm], "\[CenterDot]", 
              FormBox["\[CapitalDelta]",
               TraditionalForm]}], "+", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["p",
               TraditionalForm]}], "+", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["q",
               TraditionalForm]}]}], ")"}]}]]}], ")"}]}], "+", 
      RowBox[{
       RowBox[{
        SubscriptBox["T", 
         FormBox[
          FormBox["b",
           TraditionalForm],
          TraditionalForm]], ".", 
        SubscriptBox["T", 
         FormBox[
          FormBox["a",
           TraditionalForm],
          TraditionalForm]]}], " ", 
       RowBox[{"(", 
        RowBox[{
         FractionBox[
          RowBox[{
           FormBox[
            SuperscriptBox[
             RowBox[{"(", 
              RowBox[{"-", "1"}], ")"}], 
             RowBox[{"m", "-", "1"}]],
            TraditionalForm], " ", 
           FormBox[
            SuperscriptBox[
             RowBox[{"(", 
              RowBox[{
               FormBox["\[CapitalDelta]",
                TraditionalForm], "\[CenterDot]", 
               FormBox["p",
                TraditionalForm]}], ")"}], 
             RowBox[{"m", "-", "1"}]],
            TraditionalForm]}], 
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["p",
               TraditionalForm]}], "+", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["q",
               TraditionalForm]}]}], ")"}], " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["p",
               TraditionalForm]}], "+", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["q",
               TraditionalForm]}], "+", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["r",
               TraditionalForm]}]}], ")"}]}]], "-", 
         FractionBox[
          FormBox[
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["q",
               TraditionalForm]}], ")"}], 
            RowBox[{"m", "-", "1"}]],
           TraditionalForm], 
          RowBox[{
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["r",
             TraditionalForm]}], " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["p",
               TraditionalForm]}], "+", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["q",
               TraditionalForm]}]}], ")"}]}]], "+", 
         FractionBox[
          FormBox[
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               FormBox["\[CapitalDelta]",
                TraditionalForm], "\[CenterDot]", 
               FormBox["q",
                TraditionalForm]}], "+", 
              RowBox[{
               FormBox["\[CapitalDelta]",
                TraditionalForm], "\[CenterDot]", 
               FormBox["r",
                TraditionalForm]}]}], ")"}], 
            RowBox[{"m", "-", "1"}]],
           TraditionalForm], 
          RowBox[{
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["r",
             TraditionalForm]}], " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["p",
               TraditionalForm]}], "+", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["q",
               TraditionalForm]}], "+", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["r",
               TraditionalForm]}]}], ")"}]}]]}], ")"}]}]}], ")"}]}]}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.5009704650104513`*^9},
 CellTags->"Twist2QuarkOperator",
 CellLabel->"Out[1855]="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Polarized case, non-zero momentum insertion", "Subsubsection",
 CellTags->"Twist2QuarkOperator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SetOptions", "[", 
  RowBox[{"Twist2QuarkOperator", ",", 
   RowBox[{"Polarization", "\[Rule]", "1"}], ",", 
   RowBox[{"ZeroMomentumInsertion", "\[Rule]", "False"}]}], "]"}]], "Input",
 CellTags->"Twist2QuarkOperator",
 CellLabel->"In[1856]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"CouplingConstant", "\[Rule]", 
     SubscriptBox["g", "s"]}], ",", 
    RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
    RowBox[{"Explicit", "\[Rule]", "True"}], ",", 
    RowBox[{"Polarization", "\[Rule]", "1"}], ",", 
    RowBox[{"ZeroMomentumInsertion", "\[Rule]", "False"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.5009704650914907`*^9},
 CellTags->"Twist2QuarkOperator",
 CellLabel->"Out[1856]="]
}, Open  ]],

Cell["\<\
With the setting ZeroMomentumInsertion \[Rule] False a non-zero momentum is \
assumed to flow into the operator vertex. \
\>", "Text",
 CellTags->"Twist2QuarkOperator"],

Cell["\<\
This is the Feynman rule associated with the quark-antiquark operator, where \
p is the momentum of the incoming quark and q the momentum of the antiquark. \
The momentum flowing into the operator vertex is thus -p-q.\
\>", "Text",
 CellTags->"Twist2QuarkOperator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t1", "=", 
  RowBox[{"Twist2QuarkOperator", "[", 
   RowBox[{"{", 
    RowBox[{"p", ",", "q"}], "}"}], "]"}]}]], "Input",
 CellTags->"Twist2QuarkOperator",
 CellLabel->"In[1857]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["2", 
    RowBox[{"1", "-", "m"}]], " ", 
   RowBox[{
    SuperscriptBox["\[Gamma]", "5"], ".", 
    RowBox[{"(", 
     RowBox[{
      FormBox["\<\"\[Gamma]\"\>",
       TraditionalForm], "\[CenterDot]", 
      FormBox["\[CapitalDelta]",
       TraditionalForm]}], ")"}]}], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       FormBox["\[CapitalDelta]",
        TraditionalForm], "\[CenterDot]", 
       FormBox["p",
        TraditionalForm]}], "-", 
      RowBox[{
       FormBox["\[CapitalDelta]",
        TraditionalForm], "\[CenterDot]", 
       FormBox["q",
        TraditionalForm]}]}], ")"}], 
    RowBox[{"m", "-", "1"}]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970465256983*^9},
 CellTags->"Twist2QuarkOperator",
 CellLabel->"Out[1857]="]
}, Open  ]],

Cell["This is the quark-antiquark-gluon operator vertex.", "Text",
 CellTags->"Twist2QuarkOperator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t2", "=", 
  RowBox[{"Twist2QuarkOperator", "[", 
   RowBox[{
    RowBox[{"{", "p", "}"}], ",", 
    RowBox[{"{", "q", "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"k", ",", "\[Mu]", ",", "a"}], "}"}]}], "]"}]}]], "Input",
 CellTags->"Twist2QuarkOperator",
 CellLabel->"In[1858]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["2", 
    RowBox[{"2", "-", "m"}]], " ", 
   SuperscriptBox[
    FormBox["\[CapitalDelta]",
     TraditionalForm], 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], " ", 
   SubscriptBox["g", "s"], " ", 
   RowBox[{
    SubscriptBox["T", 
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm]], ".", 
    SuperscriptBox["\[Gamma]", "5"], ".", 
    RowBox[{"(", 
     RowBox[{
      FormBox["\<\"\[Gamma]\"\>",
       TraditionalForm], "\[CenterDot]", 
      FormBox["\[CapitalDelta]",
       TraditionalForm]}], ")"}]}], " ", 
   RowBox[{"(", 
    FormBox[
     TagBox[
      RowBox[{
       UnderoverscriptBox["\[Sum]", 
        RowBox[{"i", "=", "0"}], 
        RowBox[{
         RowBox[{"-", "2"}], "+", "m"}]], 
       RowBox[{
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           RowBox[{"-", 
            RowBox[{"(", 
             RowBox[{
              FormBox["k",
               TraditionalForm], "\[CenterDot]", 
              FormBox["\[CapitalDelta]",
               TraditionalForm]}], ")"}]}], "+", 
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["p",
             TraditionalForm]}], "-", 
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["q",
             TraditionalForm]}]}], ")"}], 
         RowBox[{
          RowBox[{"-", "2"}], "-", "i", "+", "m"}]], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            FormBox["k",
             TraditionalForm], "\[CenterDot]", 
            FormBox["\[CapitalDelta]",
             TraditionalForm]}], "+", 
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["p",
             TraditionalForm]}], "-", 
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["q",
             TraditionalForm]}]}], ")"}], "i"]}]}],
      HoldForm],
     TraditionalForm], ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970465722743*^9},
 CellTags->"Twist2QuarkOperator",
 CellLabel->"Out[1858]="]
}, Open  ]],

Cell["This shows the FeynCalcExternal form.", "Text",
 CellTags->"Twist2QuarkOperator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StandardForm", "[", 
  RowBox[{"FCE", "[", "t2", "]"}], "]"}]], "Input",
 CellTags->"Twist2QuarkOperator",
 CellLabel->"In[1859]:="],

Cell[BoxData[
 RowBox[{
  SuperscriptBox["2", 
   RowBox[{"2", "-", "OPEm"}]], " ", "Gstrong", " ", 
  RowBox[{
   RowBox[{"SUNT", "[", "a", "]"}], ".", 
   RowBox[{"GA", "[", "5", "]"}], ".", 
   RowBox[{"GSD", "[", "OPEDelta", "]"}]}], " ", 
  RowBox[{"FVD", "[", 
   RowBox[{"OPEDelta", ",", "\[Mu]"}], "]"}], " ", 
  RowBox[{"OPESum", "[", 
   RowBox[{
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", 
         RowBox[{"SOD", "[", "k", "]"}]}], "+", 
        RowBox[{"SOD", "[", "p", "]"}], "-", 
        RowBox[{"SOD", "[", "q", "]"}]}], ")"}], 
      RowBox[{
       RowBox[{"-", "2"}], "-", "OPEi", "+", "OPEm"}]], " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        RowBox[{"SOD", "[", "k", "]"}], "+", 
        RowBox[{"SOD", "[", "p", "]"}], "-", 
        RowBox[{"SOD", "[", "q", "]"}]}], ")"}], "OPEi"]}], ",", 
    RowBox[{"{", 
     RowBox[{"OPEi", ",", "0", ",", 
      RowBox[{
       RowBox[{"-", "2"}], "+", "OPEm"}]}], "}"}]}], "]"}]}]], "Output",
 CellChangeTimes->{3.500970465879302*^9},
 CellTags->"Twist2QuarkOperator",
 CellLabel->"Out[1859]//StandardForm="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Unpolarized case, non-zero momentum insertion", "Subsubsection",
 CellTags->"Twist2QuarkOperator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SetOptions", "[", 
  RowBox[{"Twist2QuarkOperator", ",", 
   RowBox[{"Polarization", "\[Rule]", "1"}], ",", 
   RowBox[{"ZeroMomentumInsertion", "\[Rule]", "False"}]}], "]"}]], "Input",
 CellTags->"Twist2QuarkOperator",
 CellLabel->"In[1860]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"CouplingConstant", "\[Rule]", 
     SubscriptBox["g", "s"]}], ",", 
    RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
    RowBox[{"Explicit", "\[Rule]", "True"}], ",", 
    RowBox[{"Polarization", "\[Rule]", "1"}], ",", 
    RowBox[{"ZeroMomentumInsertion", "\[Rule]", "False"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970465950346*^9},
 CellTags->"Twist2QuarkOperator",
 CellLabel->"Out[1860]="]
}, Open  ]],

Cell["Quark-antiquark operator.", "Text",
 CellTags->"Twist2QuarkOperator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t4", "=", 
  RowBox[{"Twist2QuarkOperator", "[", 
   RowBox[{"{", 
    RowBox[{"p", ",", "q"}], "}"}], "]"}]}]], "Input",
 CellTags->"Twist2QuarkOperator",
 CellLabel->"In[1861]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["2", 
    RowBox[{"1", "-", "m"}]], " ", 
   RowBox[{
    SuperscriptBox["\[Gamma]", "5"], ".", 
    RowBox[{"(", 
     RowBox[{
      FormBox["\<\"\[Gamma]\"\>",
       TraditionalForm], "\[CenterDot]", 
      FormBox["\[CapitalDelta]",
       TraditionalForm]}], ")"}]}], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       FormBox["\[CapitalDelta]",
        TraditionalForm], "\[CenterDot]", 
       FormBox["p",
        TraditionalForm]}], "-", 
      RowBox[{
       FormBox["\[CapitalDelta]",
        TraditionalForm], "\[CenterDot]", 
       FormBox["q",
        TraditionalForm]}]}], ")"}], 
    RowBox[{"m", "-", "1"}]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970466085022*^9},
 CellTags->"Twist2QuarkOperator",
 CellLabel->"Out[1861]="]
}, Open  ]],

Cell["Quark-antiquark-gluon operator.", "Text",
 CellTags->"Twist2QuarkOperator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t5", "=", 
  RowBox[{"Twist2QuarkOperator", "[", 
   RowBox[{
    RowBox[{"{", "p", "}"}], ",", 
    RowBox[{"{", "q", "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"k", ",", "\[Mu]", ",", "a"}], "}"}]}], "]"}]}]], "Input",
 CellTags->"Twist2QuarkOperator",
 CellLabel->"In[1862]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["2", 
    RowBox[{"2", "-", "m"}]], " ", 
   SuperscriptBox[
    FormBox["\[CapitalDelta]",
     TraditionalForm], 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], " ", 
   SubscriptBox["g", "s"], " ", 
   RowBox[{
    SubscriptBox["T", 
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm]], ".", 
    SuperscriptBox["\[Gamma]", "5"], ".", 
    RowBox[{"(", 
     RowBox[{
      FormBox["\<\"\[Gamma]\"\>",
       TraditionalForm], "\[CenterDot]", 
      FormBox["\[CapitalDelta]",
       TraditionalForm]}], ")"}]}], " ", 
   RowBox[{"(", 
    FormBox[
     TagBox[
      RowBox[{
       UnderoverscriptBox["\[Sum]", 
        RowBox[{"i", "=", "0"}], 
        RowBox[{
         RowBox[{"-", "2"}], "+", "m"}]], 
       RowBox[{
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           RowBox[{"-", 
            RowBox[{"(", 
             RowBox[{
              FormBox["k",
               TraditionalForm], "\[CenterDot]", 
              FormBox["\[CapitalDelta]",
               TraditionalForm]}], ")"}]}], "+", 
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["p",
             TraditionalForm]}], "-", 
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["q",
             TraditionalForm]}]}], ")"}], 
         RowBox[{
          RowBox[{"-", "2"}], "-", "i", "+", "m"}]], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            FormBox["k",
             TraditionalForm], "\[CenterDot]", 
            FormBox["\[CapitalDelta]",
             TraditionalForm]}], "+", 
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["p",
             TraditionalForm]}], "-", 
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["q",
             TraditionalForm]}]}], ")"}], "i"]}]}],
      HoldForm],
     TraditionalForm], ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970466245133*^9},
 CellTags->"Twist2QuarkOperator",
 CellLabel->"Out[1862]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", 
   RowBox[{"t1", ",", "t2", ",", "t3", ",", "t4", ",", "t5", ",", "t6"}], 
   "]"}], ";"}]], "Input",
 CellTags->"Twist2QuarkOperator",
 CellLabel->"In[1863]:="]
}, Open  ]],

Cell["Twist3QuarkOperator", "ObjectName",
 CellTags->"Twist3QuarkOperator"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Twist3QuarkOperator"],

Cell["\<\
Twist3QuarkOperator[p] or  Twist3QuarkOperator[p,_,_]  yields the  2-quark \
operator (p is momentum in the direction of the fermion number flow). \
Twist3QuarkOperator[{p1,___}, {p2,___}, {p3, mu, a}] or \
Twist3QuarkOperator[p1,_,_,  p2,_,_,  p3,mu,a] Quark-Quark-Gluon-operator, \
where p1 is the incoming quark, p2 the incoming antiquark and p3 denotes the \
(incoming) gluon momentum.  Twist3QuarkOperator[{p1,___}, {p2,___}, {p3, mu, \
a}, {p4, nu, b}] or Twist3QuarkOperator[p1,_,_,  p2,_,_,  p3,mu,a, p4, nu, b] \
 gives the Quark-Quark-Gluon-Gluon-operator. The setting of the option \
Polarization (unpolarized: 0; polarized: 1) determines whether the \
uppolarized or polarized operator is returned.\
\>", "Text",
 CellTags->"Twist3QuarkOperator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Twist3QuarkOperator", "//", "Options"}]], "Input",
 CellTags->"Twist3QuarkOperator",
 CellLabel->"In[1864]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"CouplingConstant", "\[Rule]", 
     SubscriptBox["g", "s"]}], ",", 
    RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
    RowBox[{"Polarization", "\[Rule]", "1"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970466437613*^9},
 CellTags->"Twist3QuarkOperator",
 CellLabel->"Out[1864]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Twist2QuarkOperator",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Twist2QuarkOperator",
  ButtonNote->"Twist2QuarkOperator"],
 ", ",
 ButtonBox["Twist2GluonOperator",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Twist2GluonOperator",
  ButtonNote->"Twist2GluonOperator"],
 "."
}], "Text",
 CellTags->"Twist3QuarkOperator"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Twist3QuarkOperator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Twist3QuarkOperator", "[", "p", "]"}]], "Input",
 CellTags->"Twist3QuarkOperator",
 CellLabel->"In[1865]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"-", "1"}], ")"}], "m"], " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      FormBox["\<\"\[Gamma]\"\>",
       TraditionalForm], "\[CenterDot]", 
      FormBox["\[CapitalDelta]",
       TraditionalForm]}], ")"}], ".", 
    SuperscriptBox["\[Gamma]", "5"]}], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{
      FormBox["\[CapitalDelta]",
       TraditionalForm], "\[CenterDot]", 
      FormBox["p",
       TraditionalForm]}], ")"}], 
    RowBox[{"m", "-", "1"}]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970466594578*^9},
 CellTags->"Twist3QuarkOperator",
 CellLabel->"Out[1865]="]
}, Open  ]]
}, Open  ]],

Cell["Twist4GluonOperator", "ObjectName",
 CellTags->"Twist4GluonOperator"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Twist4GluonOperator"],

Cell["\<\
Twist4GluonOperator[{oa, ob, oc, od}, {p1,la1,a1}, {p2,la2,a2}, {p3,la3,a3}, \
{p4,la4,a4}].\
\>", "Text",
 CellTags->"Twist4GluonOperator"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Twist2QuarkOperator",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Twist2QuarkOperator",
  ButtonNote->"Twist2QuarkOperator"],
 ", ",
 ButtonBox["Twist3QuarkOperator",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Twist3QuarkOperator",
  ButtonNote->"Twist3QuarkOperator"],
 ", ",
 ButtonBox["Twist2GluonOperator",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Twist2GluonOperator",
  ButtonNote->"Twist2GluonOperator"],
 "."
}], "Text",
 CellTags->"Twist4GluonOperator"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Twist4GluonOperator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Twist4GluonOperator", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"oa", ",", " ", "ob", ",", " ", "oc", ",", " ", "od"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"p1", ",", "la1", ",", "a1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"p2", ",", "la2", ",", "a2"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"p3", ",", "la3", ",", "a3"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"p4", ",", "la4", ",", "a4"}], "}"}]}], "]"}]], "Input",
 CellTags->"Twist4GluonOperator",
 CellLabel->"In[1866]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p2",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la1", "la2"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p1",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p1",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}]}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p4",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la3", "la4"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p3",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p3",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}]}], ")"}], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a1",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["od",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a2",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oc",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a3",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["ob",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a4",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oa",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p2",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la1", "la2"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p1",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p1",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}]}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p4",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la3", "la4"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p3",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p3",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}]}], ")"}], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a1",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oc",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a2",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["od",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a3",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["ob",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a4",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oa",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p3",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la1", "la3"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p1",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p1",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}]}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p4",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la2", "la4"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p2",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p2",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}]}], ")"}], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a1",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["od",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a2",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["ob",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a3",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oc",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a4",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oa",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p3",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la1", "la3"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p1",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p1",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}]}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p4",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la2", "la4"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p2",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p2",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}]}], ")"}], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a1",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["ob",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a2",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["od",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a3",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oc",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a4",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oa",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p4",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la1", "la4"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p1",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p1",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}]}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p3",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la2", "la3"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p2",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p2",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}]}], ")"}], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a1",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oc",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a2",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["ob",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a3",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["od",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a4",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oa",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p4",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la1", "la4"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p1",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p1",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}]}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p3",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la2", "la3"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p2",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p2",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}]}], ")"}], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a1",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["ob",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a2",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oc",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a3",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["od",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a4",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oa",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p2",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la1", "la2"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p1",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p1",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}]}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p4",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la3", "la4"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p3",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p3",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}]}], ")"}], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a1",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["od",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a2",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oc",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a3",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oa",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a4",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["ob",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p2",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la1", "la2"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p1",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p1",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}]}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p4",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la3", "la4"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p3",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p3",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}]}], ")"}], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a1",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oc",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a2",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["od",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a3",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oa",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a4",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["ob",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p4",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la1", "la4"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p1",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p1",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}]}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p3",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la2", "la3"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p2",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p2",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}]}], ")"}], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a1",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["od",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a2",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oa",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a3",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oc",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a4",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["ob",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p4",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la1", "la4"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p1",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p1",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}]}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p3",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la2", "la3"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p2",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p2",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}]}], ")"}], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a1",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oa",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a2",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["od",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a3",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oc",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a4",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["ob",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p3",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la1", "la3"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p1",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p1",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}]}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p4",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la2", "la4"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p2",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p2",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}]}], ")"}], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a1",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oc",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a2",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oa",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a3",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["od",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a4",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["ob",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p3",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la1", "la3"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p1",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p1",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}]}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p4",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la2", "la4"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p2",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p2",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}]}], ")"}], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a1",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oa",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a2",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oc",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a3",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["od",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a4",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["ob",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p3",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la1", "la3"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p1",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p1",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}]}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p4",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la2", "la4"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p2",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p2",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}]}], ")"}], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a1",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["od",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a2",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["ob",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a3",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oa",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a4",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oc",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p3",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la1", "la3"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p1",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p1",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}]}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p4",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la2", "la4"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p2",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p2",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}]}], ")"}], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a1",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["ob",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a2",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["od",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a3",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oa",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a4",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oc",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p4",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la1", "la4"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p1",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p1",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}]}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p3",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la2", "la3"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p2",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p2",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}]}], ")"}], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a1",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["od",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a2",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oa",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a3",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["ob",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a4",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oc",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p4",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la1", "la4"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p1",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p1",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}]}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p3",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la2", "la3"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p2",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p2",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}]}], ")"}], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a1",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oa",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a2",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["od",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a3",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["ob",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a4",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oc",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p2",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la1", "la2"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p1",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p1",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}]}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p4",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la3", "la4"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p3",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p3",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}]}], ")"}], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a1",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["ob",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a2",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oa",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a3",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["od",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a4",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oc",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p2",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la1", "la2"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p1",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p1",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}]}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p4",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la3", "la4"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p3",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p3",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}]}], ")"}], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a1",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oa",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a2",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["ob",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a3",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["od",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a4",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oc",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p4",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la1", "la4"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p1",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p1",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}]}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p3",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la2", "la3"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p2",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p2",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}]}], ")"}], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a1",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oc",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a2",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["ob",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a3",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oa",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a4",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["od",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p4",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la1", "la4"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p1",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p1",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}]}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p3",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la2", "la3"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p2",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p2",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}]}], ")"}], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a1",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["ob",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a2",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oc",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a3",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oa",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a4",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["od",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p3",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la1", "la3"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p1",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p1",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}]}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p4",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la2", "la4"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p2",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p2",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}]}], ")"}], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a1",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oc",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a2",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oa",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a3",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["ob",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a4",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["od",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p3",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la1", "la3"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p1",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p1",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}]}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p4",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la2", "la4"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p2",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p2",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}]}], ")"}], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a1",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oa",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a2",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oc",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a3",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["ob",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a4",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["od",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p2",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la1", "la2"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p1",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p1",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}]}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p4",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la3", "la4"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p3",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p3",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}]}], ")"}], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a1",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["ob",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a2",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oa",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a3",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oc",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a4",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["od",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p2",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la1", "la2"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p1",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p1",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}]}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p4",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la3", "la4"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p3",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p3",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}]}], ")"}], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a1",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oa",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a2",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["ob",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a3",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oc",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a4",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["od",
        TraditionalForm],
       TraditionalForm]}]]}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970466898679*^9},
 CellTags->"Twist4GluonOperator",
 CellLabel->"Out[1866]="]
}, Open  ]]
}, Open  ]],

Cell["TwoLoopSimplify  ***unfinished*** (examples missing)", "ObjectName",
 CellTags->"TwoLoopSimplify"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"TwoLoopSimplify"],

Cell["\<\
TwoLoopSimplify[amplitude,{qu1,qu2}] simplifies the 2-loop amplitude (qu1 and \
qu2 denote the integration momenta). TwoLoopSimplify[amplitude] transforms to \
TwoLoopSimplify[amplitude, {Global`q1, Global`q2}], i.e., the integration \
momenta in amplitude must be named q1 and q2.\
\>", "Text",
 CellTags->"TwoLoopSimplify"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["OneLoopSimplify",
  BaseStyle->"AddOnsLink",
  ButtonData:>"OneLoopSimplify",
  ButtonNote->"OneLoopSimplify"],
 "."
}], "Text",
 CellTags->"TwoLoopSimplify"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"TwoLoopSimplify"],

Cell[BoxData[""], "Input",
 CellTags->"TwoLoopSimplify",
 CellLabel->"In[1867]:="]
}, Open  ]],

Cell["Uncontract", "ObjectName",
 CellTags->"Uncontract"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Uncontract"],

Cell["\<\
Uncontract[exp, q1, q2, ...] uncontracts Eps and DiracGamma. Uncontract[exp, \
q1, q2, Pair \[Rule] {p}] uncontracts also p.q1 and p.q2; the option Pair \
\[Rule] All uncontracts all momenta except OPEDelta.\
\>", "Text",
 CellTags->"Uncontract"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "Uncontract", "]"}]], "Input",
 CellTags->"Uncontract",
 CellLabel->"In[1868]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Dimension", "\[Rule]", "Automatic"}], ",", 
    RowBox[{"DimensionalReduction", "\[Rule]", "False"}], ",", 
    RowBox[{"Pair", "\[Rule]", 
     RowBox[{"{", "}"}]}], ",", 
    RowBox[{"Unique", "\[Rule]", "True"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970467113422*^9},
 CellTags->"Uncontract",
 CellLabel->"Out[1868]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Contract",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Contract",
  ButtonNote->"Contract"],
 "."
}], "Text",
 CellTags->"Uncontract"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Uncontract"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t1", " ", "=", " ", 
  RowBox[{
   RowBox[{"LeviCivita", "[", 
    RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}], "[", 
   RowBox[{"p", ",", "q"}], "]"}]}]], "Input",
 CellTags->"Uncontract",
 CellLabel->"In[1869]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["\[Epsilon]", 
   RowBox[{
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["\[Nu]",
      TraditionalForm],
     TraditionalForm], 
    FormBox["p",
     TraditionalForm], 
    FormBox["q",
     TraditionalForm]}]], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009704672123528`*^9},
 CellTags->"Uncontract",
 CellLabel->"Out[1869]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Uncontract", "[", 
  RowBox[{"t1", ",", " ", "p"}], "]"}]], "Input",
 CellTags->"Uncontract",
 CellLabel->"In[1870]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox[
    FormBox["p",
     TraditionalForm], 
    FormBox[
     FormBox[
      RowBox[{"$AL$24751", "\[InvisibleApplication]", 
       RowBox[{"(", "1", ")"}]}],
      TraditionalForm],
     TraditionalForm]], " ", 
   SuperscriptBox["\[Epsilon]", 
    RowBox[{
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox[
       RowBox[{"$AL$24751", "\[InvisibleApplication]", 
        RowBox[{"(", "1", ")"}]}],
       TraditionalForm],
      TraditionalForm], 
     FormBox["q",
      TraditionalForm]}]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970467331452*^9},
 CellTags->"Uncontract",
 CellLabel->"Out[1870]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t2", "=", 
  RowBox[{"DiracSlash", "[", "p", "]"}]}]], "Input",
 CellTags->"Uncontract",
 CellLabel->"In[1871]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox["\<\"\[Gamma]\"\>",
    TraditionalForm], "\[CenterDot]", 
   FormBox["p",
    TraditionalForm]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970467950371*^9},
 CellTags->"Uncontract",
 CellLabel->"Out[1871]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Uncontract", "[", 
  RowBox[{"t2", ",", " ", "p"}], "]"}]], "Input",
 CellTags->"Uncontract",
 CellLabel->"In[1872]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox[
      RowBox[{"$AL$24758", "\[InvisibleApplication]", 
       RowBox[{"(", "1", ")"}]}],
      TraditionalForm],
     TraditionalForm]], " ", 
   SuperscriptBox[
    FormBox["p",
     TraditionalForm], 
    FormBox[
     FormBox[
      RowBox[{"$AL$24758", "\[InvisibleApplication]", 
       RowBox[{"(", "1", ")"}]}],
      TraditionalForm],
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970468107951*^9},
 CellTags->"Uncontract",
 CellLabel->"Out[1872]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Uncontract", "[", 
  RowBox[{"t1", ",", " ", "p", ",", "q"}], "]"}]], "Input",
 CellTags->"Uncontract",
 CellLabel->"In[1873]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox[
    FormBox["p",
     TraditionalForm], 
    FormBox[
     FormBox[
      RowBox[{"$AL$24763", "\[InvisibleApplication]", 
       RowBox[{"(", "1", ")"}]}],
      TraditionalForm],
     TraditionalForm]], " ", 
   SuperscriptBox[
    FormBox["q",
     TraditionalForm], 
    FormBox[
     FormBox[
      RowBox[{"$AL$24762", "\[InvisibleApplication]", 
       RowBox[{"(", "1", ")"}]}],
      TraditionalForm],
     TraditionalForm]], " ", 
   SuperscriptBox["\[Epsilon]", 
    RowBox[{
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox[
       RowBox[{"$AL$24763", "\[InvisibleApplication]", 
        RowBox[{"(", "1", ")"}]}],
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox[
       RowBox[{"$AL$24762", "\[InvisibleApplication]", 
        RowBox[{"(", "1", ")"}]}],
       TraditionalForm],
      TraditionalForm]}]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009704682130413`*^9},
 CellTags->"Uncontract",
 CellLabel->"Out[1873]="]
}, Open  ]],

Cell["By default scalar products are not uncontracted.", "Text",
 CellTags->"Uncontract"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Uncontract", "[", 
  RowBox[{
   RowBox[{"ScalarProduct", "[", 
    RowBox[{"p", ",", "q"}], "]"}], ",", " ", "q"}], "]"}]], "Input",
 CellTags->"Uncontract",
 CellLabel->"In[1874]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox["p",
    TraditionalForm], "\[CenterDot]", 
   FormBox["q",
    TraditionalForm]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009704683253*^9},
 CellTags->"Uncontract",
 CellLabel->"Out[1874]="]
}, Open  ]],

Cell["\<\
With the option Pair\[Rule]All they are \[OpenCurlyDoubleQuote]uncontracted \
\[CloseCurlyDoubleQuote].\
\>", "Text",
 CellTags->"Uncontract"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Uncontract", "[", 
  RowBox[{
   RowBox[{"ScalarProduct", "[", 
    RowBox[{"p", ",", "q"}], "]"}], ",", "q", ",", 
   RowBox[{"Pair", "\[Rule]", "All"}]}], "]"}]], "Input",
 CellTags->"Uncontract",
 CellLabel->"In[1875]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox[
    FormBox["p",
     TraditionalForm], 
    FormBox[
     FormBox[
      RowBox[{"$AL$24770", "\[InvisibleApplication]", 
       RowBox[{"(", "1", ")"}]}],
      TraditionalForm],
     TraditionalForm]], " ", 
   SuperscriptBox[
    FormBox["q",
     TraditionalForm], 
    FormBox[
     FormBox[
      RowBox[{"$AL$24770", "\[InvisibleApplication]", 
       RowBox[{"(", "1", ")"}]}],
      TraditionalForm],
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970468439838*^9},
 CellTags->"Uncontract",
 CellLabel->"Out[1875]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Uncontract", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"ScalarProduct", "[", 
     RowBox[{"p", ",", "q"}], "]"}], "^", "2"}], ",", "q", ",", 
   RowBox[{"Pair", "\[Rule]", "All"}]}], "]"}]], "Input",
 CellTags->"Uncontract",
 CellLabel->"In[1876]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox[
    FormBox["p",
     TraditionalForm], 
    FormBox[
     FormBox[
      RowBox[{"$AL$24775", "\[InvisibleApplication]", 
       RowBox[{"(", "1", ")"}]}],
      TraditionalForm],
     TraditionalForm]], " ", 
   SuperscriptBox[
    FormBox["p",
     TraditionalForm], 
    FormBox[
     FormBox[
      RowBox[{"$AL$24775", "\[InvisibleApplication]", 
       RowBox[{"(", "2", ")"}]}],
      TraditionalForm],
     TraditionalForm]], " ", 
   SuperscriptBox[
    FormBox["q",
     TraditionalForm], 
    FormBox[
     FormBox[
      RowBox[{"$AL$24775", "\[InvisibleApplication]", 
       RowBox[{"(", "1", ")"}]}],
      TraditionalForm],
     TraditionalForm]], " ", 
   SuperscriptBox[
    FormBox["q",
     TraditionalForm], 
    FormBox[
     FormBox[
      RowBox[{"$AL$24775", "\[InvisibleApplication]", 
       RowBox[{"(", "2", ")"}]}],
      TraditionalForm],
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970468626499*^9},
 CellTags->"Uncontract",
 CellLabel->"Out[1876]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"Clear", "[", 
  RowBox[{"t1", ",", "t2"}], "]"}]], "Input",
 CellTags->"Uncontract",
 CellLabel->"In[1877]:="]
}, Open  ]],

Cell["UnDeclareNonCommutative", "ObjectName",
 CellTags->"UnDeclareNonCommutative"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"UnDeclareNonCommutative"],

Cell["\<\
UnDeclareNonCommutative[a, b, ...] undeclares a,b, ... to be noncommutative, \
i.e., DataType[a,b, ..., NonCommutative] is set to False.\
\>", "Text",
 CellTags->"UnDeclareNonCommutative"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["DataType",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DataType",
  ButtonNote->"DataType"],
 ", ",
 ButtonBox["DeclareNonCommutative",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DeclareNonCommutative",
  ButtonNote->"DeclareNonCommutative"],
 "."
}], "Text",
 CellTags->"UnDeclareNonCommutative"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"UnDeclareNonCommutative"],

Cell[BoxData[
 RowBox[{"DeclareNonCommutative", "[", "x", "]"}]], "Input",
 CellTags->"UnDeclareNonCommutative",
 CellLabel->"In[1878]:="],

Cell["\<\
As a side-effect of DeclareNonCommutative x is declared to be of DataType \
NonCommutative.\
\>", "Text",
 CellTags->"UnDeclareNonCommutative"],

Cell[CellGroupData[{

Cell["DataType[x,NonCommutative]", "Input",
 CellTags->"UnDeclareNonCommutative",
 CellLabel->"In[1879]:="],

Cell[BoxData[
 FormBox["True", TraditionalForm]], "Output",
 CellChangeTimes->{3.5009704691871443`*^9},
 CellTags->"UnDeclareNonCommutative",
 CellLabel->"Out[1879]="]
}, Open  ]],

Cell["The inverse operation is UnDeclareNonCommutative.", "Text",
 CellTags->"UnDeclareNonCommutative"],

Cell[BoxData[
 RowBox[{"UnDeclareNonCommutative", "[", "x", "]"}]], "Input",
 CellTags->"UnDeclareNonCommutative",
 CellLabel->"In[1880]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DataType", "[", 
  RowBox[{"x", ",", "NonCommutative"}], "]"}]], "Input",
 CellTags->"UnDeclareNonCommutative",
 CellLabel->"In[1881]:="],

Cell[BoxData[
 FormBox["True", TraditionalForm]], "Output",
 CellChangeTimes->{3.500970469428484*^9},
 CellTags->"UnDeclareNonCommutative",
 CellLabel->"Out[1881]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"DeclareNonCommutative", "[", 
  RowBox[{"y", ",", "z"}], "]"}]], "Input",
 CellTags->"UnDeclareNonCommutative",
 CellLabel->"In[1882]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DataType", "[", 
  RowBox[{"y", ",", "z", ",", "NonCommutative"}], "]"}]], "Input",
 CellTags->"UnDeclareNonCommutative",
 CellLabel->"In[1883]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"True", ",", "True"}], "}"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970469744516*^9},
 CellTags->"UnDeclareNonCommutative",
 CellLabel->"Out[1883]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"UnDeclareNonCommutative", "[", 
  RowBox[{"y", ",", "z"}], "]"}]], "Input",
 CellTags->"UnDeclareNonCommutative",
 CellLabel->"In[1884]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DataType", "[", 
  RowBox[{"y", ",", "z", ",", "NonCommutative"}], "]"}]], "Input",
 CellTags->"UnDeclareNonCommutative",
 CellLabel->"In[1885]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"True", ",", "True"}], "}"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009704699823723`*^9},
 CellTags->"UnDeclareNonCommutative",
 CellLabel->"Out[1885]="]
}, Open  ]]
}, Open  ]],

Cell["Upper", "ObjectName",
 CellTags->"Upper"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Upper"],

Cell["\<\
Upper may be used inside LorentzIndex to indicate an contravariant \
LorentzIndex.\
\>", "Text",
 CellTags->"Upper"],

Cell[TextData[{
 "See also:  ",
 ButtonBox["LorentzIndex",
  BaseStyle->"AddOnsLink",
  ButtonData:>"LorentzIndex",
  ButtonNote->"LorentzIndex"],
 ", ",
 ButtonBox["Lower",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Lower",
  ButtonNote->"Lower"],
 ", ",
 ButtonBox["Contract1",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Contract1",
  ButtonNote->"Contract1"],
 "."
}], "Text",
 CellTags->"Upper"]
}, Open  ]],

Cell["UVPart  ***unfinished*** (examples missing)", "ObjectName",
 CellTags->"UVPart"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"UVPart"],

Cell["\<\
UVPart[exp, q] discards ultraviolet finite integrals  (q = integration \
momentum).\
\>", "Text",
 CellTags->"UVPart"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"UVPart"],

Cell[BoxData[""], "Input",
 CellTags->"UVPart",
 CellLabel->"In[1886]:="]
}, Open  ]],

Cell["Vectors", "ObjectName",
 CellTags->"Vectors"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Vectors"],

Cell["\<\
Vectors is an option for FORM2FeynCalc. Its default setting is Automatic. It \
may be set to a list, if the FORM-file does not contain a V(ectors) statement.\
\>", "Text",
 CellTags->"Vectors"],

Cell[TextData[{
 "See also:  ",
 ButtonBox["FORM2FeynCalc",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FORM2FeynCalc",
  ButtonNote->"FORM2FeynCalc"],
 "."
}], "Text",
 CellTags->"Vectors"]
}, Open  ]],

Cell["WriteOut", "ObjectName",
 CellTags->"WriteOut"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"WriteOut"],

Cell["\<\
WriteOut is an option for OneLoop.  If set to True, the result of OneLoop \
will be written to a file called \"name.res\", where name is the first \
argument of OneLoop.\
\>", "Text",
 CellTags->"WriteOut"],

Cell[TextData[{
 "See also:  ",
 ButtonBox["OneLoop",
  BaseStyle->"AddOnsLink",
  ButtonData:>"OneLoop",
  ButtonNote->"OneLoop"],
 "."
}], "Text",
 CellTags->"WriteOut"]
}, Open  ]],

Cell["WriteOutPaVe", "ObjectName",
 CellTags->"WriteOutPaVe"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"WriteOutPaVe"],

Cell["\<\
WriteOutPaVe is an option for PaVeReduce and OneLoopSum. If set to a string, \
the results of all Passarino-Veltman PaVe's are stored in files with names \
generated from this string and the arguments of PaVe.\
\>", "Text",
 CellTags->"WriteOutPaVe"],

Cell[TextData[{
 "See also:  ",
 ButtonBox["PaVeReduce",
  BaseStyle->"AddOnsLink",
  ButtonData:>"PaVeReduce",
  ButtonNote->"PaVeReduce"],
 ", ",
 ButtonBox["PaVe",
  BaseStyle->"AddOnsLink",
  ButtonData:>"PaVe",
  ButtonNote->"PaVe"],
 ", ",
 ButtonBox["OneLoopSum",
  BaseStyle->"AddOnsLink",
  ButtonData:>"OneLoopSum",
  ButtonNote->"OneLoopSum"],
 "."
}], "Text",
 CellTags->"WriteOutPaVe"]
}, Open  ]],

Cell["Write2", "ObjectName",
 CellTags->"Write2"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Write2"],

Cell["\<\
Write2[file, val1 = expr1, val2 = expr2, ...] writes the settings val1 = \
expr1, val2 = expr2 in sequence followed by a newline, to the specified \
output file. Setting the option FormatType of Write2 to FortranForm results \
in FORTRAN syntax output.\
\>", "Text",
 CellTags->"Write2"],

Cell["\<\
The continuation character for FORTRAN output can be controlled by changing \
$FortranContinuationCharacter.\
\>", "Text",
 CellTags->"Write2"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullForm", "[", "$FortranContinuationCharacter", "]"}]], "Input",
 CellTags->"Write2",
 CellLabel->"In[1887]:="],

Cell[BoxData[
 FormBox[
  TagBox[
   StyleBox["\"\<&\>\"",
    ShowSpecialCharacters->False,
    ShowStringCharacters->True,
    NumberMarks->True],
   FullForm], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970470075959*^9},
 CellTags->"Write2",
 CellLabel->"Out[1887]//FullForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "Write2", "]"}]], "Input",
 CellTags->"Write2",
 CellLabel->"In[1888]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"D0Convention", "\[Rule]", "0"}], ",", 
    RowBox[{"FinalSubstitutions", "\[Rule]", 
     RowBox[{"{", "}"}]}], ",", 
    RowBox[{"FormatType", "\[Rule]", "InputForm"}], ",", 
    RowBox[{"FortranFormatDoublePrecision", "\[Rule]", "True"}], ",", 
    RowBox[{"PageWidth", "\[Rule]", "62"}], ",", 
    RowBox[{"PostFortranFile", "\[Rule]", "\<\"\"\>"}], ",", 
    RowBox[{"PreFortranFile", "\[Rule]", "\<\"\"\>"}], ",", 
    RowBox[{"StringReplace", "\[Rule]", 
     RowBox[{"{", "}"}]}]}], "}"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009704701460743`*^9},
 CellTags->"Write2",
 CellLabel->"Out[1888]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Attributes", "[", "Write2", "]"}]], "Input",
 CellTags->"Write2",
 CellLabel->"In[1889]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", "HoldRest", "}"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009704701889687`*^9},
 CellTags->"Write2",
 CellLabel->"Out[1889]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Isolate",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Isolate",
  ButtonNote->"Isolate"],
 ", ",
 ButtonBox["PaVeReduce",
  BaseStyle->"AddOnsLink",
  ButtonData:>"PaVeReduce",
  ButtonNote->"PaVeReduce"],
 "."
}], "Text",
 CellTags->"Write2"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Write2"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t", "=", 
  RowBox[{"Collect", "[", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"a", "-", "c"}], ")"}], "2"], "+", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"a", "-", "b"}], ")"}], "2"]}], ")"}], "2"], ",", "a", ",", 
    "Factor"}], "]"}]}]], "Input",
 CellTags->"Write2",
 CellLabel->"In[1890]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"4", " ", 
    SuperscriptBox["a", "4"]}], "-", 
   RowBox[{"8", " ", 
    SuperscriptBox["a", "3"], " ", 
    RowBox[{"(", 
     RowBox[{"b", "+", "c"}], ")"}]}], "+", 
   RowBox[{"8", " ", 
    SuperscriptBox["a", "2"], " ", 
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox["b", "2"], "+", 
      RowBox[{"b", " ", "c"}], "+", 
      SuperscriptBox["c", "2"]}], ")"}]}], "-", 
   RowBox[{"4", " ", "a", " ", 
    RowBox[{"(", 
     RowBox[{"b", "+", "c"}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox["b", "2"], "+", 
      SuperscriptBox["c", "2"]}], ")"}]}], "+", 
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox["b", "2"], "+", 
      SuperscriptBox["c", "2"]}], ")"}], "2"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.50097047031427*^9},
 CellTags->"Write2",
 CellLabel->"Out[1890]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"tempfilename", "=", 
   RowBox[{
    RowBox[{"ToString", "[", "$SessionID", "]"}], "<>", "\"\<.s\>\""}]}], 
  ";"}]], "Input",
 CellTags->"Write2",
 CellLabel->"In[1891]:="],

Cell["This writes the assignment r=t to a file.", "Text",
 CellTags->"Write2"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Write2", "[", 
   RowBox[{"tempfilename", ",", 
    RowBox[{"r", "=", "t"}]}], "]"}], ";"}]], "Input",
 CellTags->"Write2",
 CellLabel->"In[1892]:="],

Cell["This shows the contents of the file.", "Text",
 CellTags->"Write2"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TableForm", "[", 
  RowBox[{"ReadList", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"$OperatingSystem", "===", "\"\<MacOS\>\""}], ",", "\"\<:\>\"",
        ",", "\"\<\>\""}], "]"}], "<>", "tempfilename"}], ",", "String"}], 
   "]"}], "]"}]], "Input",
 CellTags->"Write2",
 CellLabel->"In[1893]:="],

Cell[BoxData[
 FormBox[
  TagBox[
   TagBox[GridBox[{
      {"\<\"r = ( 4*a^4 - 8*a^3*(b + c) - 4*a*(b + c)*(b^2 + c^2) + \"\>"},
      {"\<\" (b^2 + c^2)^2 + 8*a^2*(b^2 + b*c + c^2)\"\>"},
      {"\<\"       );\"\>"}
     },
     GridBoxAlignment->{
      "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
       "RowsIndexed" -> {}},
     GridBoxSpacings->{"Columns" -> {
         Offset[0.27999999999999997`], {
          Offset[0.5599999999999999]}, 
         Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
         Offset[0.2], {
          Offset[0.4]}, 
         Offset[0.2]}, "RowsIndexed" -> {}}],
    Column],
   Function[BoxForm`e$, 
    TableForm[BoxForm`e$]]], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970470435953*^9},
 CellTags->"Write2",
 CellLabel->"Out[1893]//TableForm="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"DeleteFile", "[", 
  RowBox[{
   RowBox[{"If", "[", 
    RowBox[{
     RowBox[{"$OperatingSystem", "===", "\"\<MacOS\>\""}], ",", "\"\<:\>\"", 
     ",", "\"\<\>\""}], "]"}], "<>", "tempfilename"}], "]"}]], "Input",
 CellTags->"Write2",
 CellLabel->"In[1894]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t2", "=", 
  RowBox[{"x", "+", 
   RowBox[{"Isolate", "[", 
    RowBox[{"t", ",", "a", ",", 
     RowBox[{"IsolateNames", "\[Rule]", "W"}]}], "]"}]}]}]], "Input",
 CellTags->"Write2",
 CellLabel->"In[1895]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"4", " ", 
    SuperscriptBox["a", "4"]}], "-", 
   RowBox[{"8", " ", 
    SuperscriptBox["a", "3"], " ", 
    TagBox[
     RowBox[{"W", "(", "217", ")"}],
     HoldForm]}], "+", 
   RowBox[{"8", " ", 
    SuperscriptBox["a", "2"], " ", 
    TagBox[
     RowBox[{"W", "(", "219", ")"}],
     HoldForm]}], "-", 
   RowBox[{"4", " ", "a", " ", 
    TagBox[
     RowBox[{"W", "(", "217", ")"}],
     HoldForm], " ", 
    TagBox[
     RowBox[{"W", "(", "218", ")"}],
     HoldForm]}], "+", 
   SuperscriptBox[
    TagBox[
     RowBox[{"W", "(", "218", ")"}],
     HoldForm], "2"], "+", "x"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970470542563*^9},
 CellTags->"Write2",
 CellLabel->"Out[1895]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"Write2", "[", 
   RowBox[{"tempfilename", ",", 
    RowBox[{"r", "=", "t2"}]}], "]"}], ";"}]], "Input",
 CellTags->"Write2",
 CellLabel->"In[1896]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TableForm", "[", 
  RowBox[{"ReadList", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"$OperatingSystem", "===", "\"\<MacOS\>\""}], ",", "\"\<:\>\"",
        ",", "\"\<\>\""}], "]"}], "<>", "tempfilename"}], ",", "String"}], 
   "]"}], "]"}]], "Input",
 CellTags->"Write2",
 CellLabel->"In[1897]:="],

Cell[BoxData[
 FormBox[
  TagBox[
   TagBox[GridBox[{
      {"\<\"W[217] = (b + c\"\>"},
      {"\<\"       );\"\>"},
      {"\<\"W[218] = (b^2 + c^2\"\>"},
      {"\<\"       );\"\>"},
      {"\<\"W[219] = (b^2 + b*c + c^2\"\>"},
      {"\<\"       );\"\>"},
      {"\<\"r = ( 4*a^4 + x - 8*a^3*HoldForm[W[217]] - \"\>"},
      {"\<\" 4*a*HoldForm[W[217]]*HoldForm[W[218]] + HoldForm[W[218]]^2 + \
\"\>"},
      {"\<\" 8*a^2*HoldForm[W[219]]\"\>"},
      {"\<\"       );\"\>"}
     },
     GridBoxAlignment->{
      "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
       "RowsIndexed" -> {}},
     GridBoxSpacings->{"Columns" -> {
         Offset[0.27999999999999997`], {
          Offset[0.5599999999999999]}, 
         Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
         Offset[0.2], {
          Offset[0.4]}, 
         Offset[0.2]}, "RowsIndexed" -> {}}],
    Column],
   Function[BoxForm`e$, 
    TableForm[BoxForm`e$]]], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970470629777*^9},
 CellTags->"Write2",
 CellLabel->"Out[1897]//TableForm="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"DeleteFile", "[", 
  RowBox[{
   RowBox[{"If", "[", 
    RowBox[{
     RowBox[{"$OperatingSystem", "===", "\"\<MacOS\>\""}], ",", "\"\<:\>\"", 
     ",", "\"\<\>\""}], "]"}], "<>", "tempfilename"}], "]"}]], "Input",
 CellTags->"Write2",
 CellLabel->"In[1898]:="],

Cell["\<\
This is how to write out the expression t2 in Fortran format.\
\>", "Text",
 CellTags->"Write2"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Write2", "[", 
   RowBox[{"tempfilename", ",", 
    RowBox[{"r", "=", "t2"}], ",", 
    RowBox[{"FormatType", "\[Rule]", "FortranForm"}]}], "]"}], ";"}]], "Input",\

 CellTags->"Write2",
 CellLabel->"In[1899]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TableForm", "[", 
  RowBox[{"ReadList", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"$OperatingSystem", "===", "\"\<MacOS\>\""}], ",", "\"\<:\>\"",
        ",", "\"\<\>\""}], "]"}], "<>", "tempfilename"}], ",", "String"}], 
   "]"}], "]"}]], "Input",
 CellTags->"Write2",
 CellLabel->"In[1900]:="],

Cell[BoxData[
 FormBox[
  TagBox[
   TagBox[GridBox[{
      {"\<\"        W(217D2)= W(217D2)\"\>"},
      {"\<\"        W(218D2)= W(218D2)\"\>"},
      {"\<\"        W(219D2)= W(219D2)\"\>"},
      {"\<\"        r = 4D0*a**4 + x - 8D0*a**3*W(217D2) - \"\>"},
      {"\<\"     &  4D0*a*W(217D2)*W(218D2) + W(218D2)**2 + \"\>"},
      {"\<\"     &  8D0*a**2*W(219D2)\"\>"}
     },
     GridBoxAlignment->{
      "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
       "RowsIndexed" -> {}},
     GridBoxSpacings->{"Columns" -> {
         Offset[0.27999999999999997`], {
          Offset[0.5599999999999999]}, 
         Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
         Offset[0.2], {
          Offset[0.4]}, 
         Offset[0.2]}, "RowsIndexed" -> {}}],
    Column],
   Function[BoxForm`e$, 
    TableForm[BoxForm`e$]]], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970470818913*^9},
 CellTags->"Write2",
 CellLabel->"Out[1900]//TableForm="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"DeleteFile", "[", 
   RowBox[{
    RowBox[{"If", "[", 
     RowBox[{
      RowBox[{"$OperatingSystem", "===", "\"\<MacOS\>\""}], ",", "\"\<:\>\"", 
      ",", "\"\<\>\""}], "]"}], "<>", "tempfilename"}], "]"}], ";", 
  RowBox[{"Clear", "[", 
   RowBox[{"t", ",", "t2", ",", "tempfilename"}], "]"}], ";"}]], "Input",
 CellTags->"Write2",
 CellLabel->"In[1901]:="]
}, Open  ]],

Cell["XYT", "ObjectName",
 CellTags->"XYT"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"XYT"],

Cell[TextData[{
 "XYT[exp, x,y] transforms  ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"x", " ", "y"}], ")"}], "m"], TraditionalForm]]],
 " away ..."
}], "Text",
 CellTags->"XYT"]
}, Open  ]],

Cell["ZeroMomentumInsertion", "ObjectName",
 CellTags->"ZeroMomentumInsertion"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"ZeroMomentumInsertion"],

Cell["\<\
ZeroMomentumInsertion is an option of FeynRule, Twist2GluonOperator and \
Twist2QuarkOperator.\
\>", "Text",
 CellTags->"ZeroMomentumInsertion"],

Cell[TextData[{
 "See also:  ",
 ButtonBox["FeynRule",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FeynRule",
  ButtonNote->"FeynRule"],
 ", ",
 ButtonBox["Twist2GluonOperator",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Twist2GluonOperator",
  ButtonNote->"Twist2GluonOperator"],
 ", ",
 ButtonBox["Twist2QuarkOperator",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Twist2QuarkOperator",
  ButtonNote->"Twist2QuarkOperator"],
 "."
}], "Text",
 CellTags->"ZeroMomentumInsertion"]
}, Open  ]],

Cell["Zeta2", "ObjectName",
 CellTags->"Zeta2"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Zeta2"],

Cell["Zeta2 denotes Zeta[2]. N[Zeta[2]] evaluates to N[Zeta[2]].", "Text",
 CellTags->"Zeta2"],

Cell[TextData[{
 "See also: ",
 ButtonBox["SimplifyPolyLog",
  BaseStyle->"AddOnsLink",
  ButtonData:>"SimplifyPolyLog",
  ButtonNote->"SimplifyPolyLog"],
 "."
}], "Text",
 CellTags->"Zeta2"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Zeta2"],

Cell[CellGroupData[{

Cell[BoxData["Zeta2"], "Input",
 CellTags->"Zeta2",
 CellLabel->"In[1902]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"\[Zeta]", "(", "2", ")"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970470983121*^9},
 CellTags->"Zeta2",
 CellLabel->"Out[1902]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"N", "[", "Zeta2", "]"}]], "Input",
 CellTags->"Zeta2",
 CellLabel->"In[1903]:="],

Cell[BoxData[
 FormBox["1.6449340668482262`", TraditionalForm]], "Output",
 CellChangeTimes->{3.5009704710583963`*^9},
 CellTags->"Zeta2",
 CellLabel->"Out[1903]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SimplifyPolyLog", "[", 
  RowBox[{"Pi", "^", "2"}], "]"}]], "Input",
 CellTags->"Zeta2",
 CellLabel->"In[1904]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"6", " ", 
   RowBox[{"\[Zeta]", "(", "2", ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.50097047115912*^9},
 CellTags->"Zeta2",
 CellLabel->"Out[1904]="]
}, Open  ]]
}, Open  ]],

Cell["$Abbreviations", "ObjectName",
 CellTags->"$Abbreviations"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"$Abbreviations"],

Cell["\<\
$Abbreviations are a list of string substitution rules used when generating \
names for storing intermediate results. It is used by OneLoop and \
PaVeReduce.The elements of the list should be of the form \"name\" \[Rule] \
\"abbreviation\".\
\>", "Text",
 CellTags->"$Abbreviations"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"$Abbreviations", "//", "StandardForm"}]], "Input",
 CellTags->"$Abbreviations",
 CellLabel->"In[1905]:="],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"\<\", \"\>", "\[Rule]", "\<\"\"\>"}], ",", 
   RowBox[{"\<\"^\"\>", "\[Rule]", "\<\"\"\>"}], ",", 
   RowBox[{"\<\"{\"\>", "\[Rule]", "\<\"\"\>"}], ",", 
   RowBox[{"\<\"/\"\>", "\[Rule]", "\<\"\"\>"}], ",", 
   RowBox[{"\<\"Subscript\"\>", "\[Rule]", "\<\"su\"\>"}], ",", 
   RowBox[{"\<\"SmallVariable\"\>", "\[Rule]", "\<\"sma\"\>"}], ",", 
   RowBox[{"\<\"}\"\>", "\[Rule]", "\<\"\"\>"}], ",", 
   RowBox[{"\<\"[\"\>", "\[Rule]", "\<\"\"\>"}], ",", 
   RowBox[{"\<\"]\"\>", "\[Rule]", "\<\"\"\>"}], ",", 
   RowBox[{"\<\"*\"\>", "\[Rule]", "\<\"\"\>"}], ",", 
   RowBox[{"\<\" \"\>", "\[Rule]", "\<\"\"\>"}], ",", 
   RowBox[{"\<\"\\n\"\>", "\[Rule]", "\<\"\"\>"}], ",", 
   RowBox[{"\<\"\\r\"\>", "\[Rule]", "\<\"\"\>"}]}], "}"}]], "Output",
 CellChangeTimes->{3.500970471281177*^9},
 CellTags->"$Abbreviations",
 CellLabel->"Out[1905]//StandardForm="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Abbreviation",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Abbreviation",
  ButtonNote->"Abbreviation"],
 ", ",
 ButtonBox["OneLoop",
  BaseStyle->"AddOnsLink",
  ButtonData:>"OneLoop",
  ButtonNote->"OneLoop"],
 ", ",
 ButtonBox["PaVeReduce",
  BaseStyle->"AddOnsLink",
  ButtonData:>"PaVeReduce",
  ButtonNote->"PaVeReduce"],
 ", ",
 ButtonBox["WriteOut",
  BaseStyle->"AddOnsLink",
  ButtonData:>"WriteOut",
  ButtonNote->"WriteOut"],
 ", ",
 ButtonBox["WriteOutPaVe",
  BaseStyle->"AddOnsLink",
  ButtonData:>"WriteOutPaVe",
  ButtonNote->"WriteOutPaVe"],
 "."
}], "Text",
 CellTags->"$Abbreviations"]
}, Open  ]],

Cell["$AL", "ObjectName",
 CellTags->"$AL"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"$AL"],

Cell["\<\
$AL is the head for dummy indices which may be introduced by Uncontract.\
\>", "Text",
 CellTags->"$AL"],

Cell[CellGroupData[{

Cell[BoxData["$AL"], "Input",
 CellTags->"$AL",
 CellLabel->"In[1906]:="],

Cell[BoxData[
 FormBox["$AL", TraditionalForm]], "Output",
 CellChangeTimes->{3.5009704713508387`*^9},
 CellTags->"$AL",
 CellLabel->"Out[1906]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Uncontract", "[", 
  RowBox[{
   RowBox[{"ScalarProduct", "[", 
    RowBox[{"p", ",", "q"}], "]"}], ",", "q", ",", 
   RowBox[{"Pair", "\[Rule]", "All"}]}], "]"}]], "Input",
 CellTags->"$AL",
 CellLabel->"In[1907]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox[
    FormBox["p",
     TraditionalForm], 
    FormBox[
     FormBox[
      RowBox[{"$AL$24855", "\[InvisibleApplication]", 
       RowBox[{"(", "1", ")"}]}],
      TraditionalForm],
     TraditionalForm]], " ", 
   SuperscriptBox[
    FormBox["q",
     TraditionalForm], 
    FormBox[
     FormBox[
      RowBox[{"$AL$24855", "\[InvisibleApplication]", 
       RowBox[{"(", "1", ")"}]}],
      TraditionalForm],
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970471431035*^9},
 CellTags->"$AL",
 CellLabel->"Out[1907]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"$AL", "=", "\[Mu]"}], ";"}]], "Input",
 CellTags->"$AL",
 CellLabel->"In[1908]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Uncontract", "[", 
  RowBox[{
   RowBox[{"ScalarProduct", "[", 
    RowBox[{"p", ",", "q"}], "]"}], ",", "q", ",", 
   RowBox[{"Pair", "\[Rule]", "All"}]}], "]"}]], "Input",
 CellTags->"$AL",
 CellLabel->"In[1909]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox[
    FormBox["p",
     TraditionalForm], 
    FormBox[
     FormBox[
      RowBox[{"\[Mu]$24859", "(", "1", ")"}],
      TraditionalForm],
     TraditionalForm]], " ", 
   SuperscriptBox[
    FormBox["q",
     TraditionalForm], 
    FormBox[
     FormBox[
      RowBox[{"\[Mu]$24859", "(", "1", ")"}],
      TraditionalForm],
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009704715339823`*^9},
 CellTags->"$AL",
 CellLabel->"Out[1909]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"$AL", "=."}], ";"}]], "Input",
 CellTags->"$AL",
 CellLabel->"In[1910]:="]
}, Open  ]],

Cell["$BreitMaison", "ObjectName",
 CellTags->"$BreitMaison"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"$BreitMaison"],

Cell[TextData[{
 "The Breitenlohner-Maison ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["\[Gamma]", "5"], TraditionalForm]]],
 " scheme is currently not supported by the Dirac algebra functions.Use \
Tracer if you need it (PHI supplies the functions FCToTracer and \
TracerToFC)."
}], "Text",
 CellTags->"$BreitMaison"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Tr",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Tr",
  ButtonNote->"Tr"],
 ", ",
 ButtonBox["$Larin",
  BaseStyle->"AddOnsLink",
  ButtonData:>"$Larin",
  ButtonNote->"$Larin"],
 ", ",
 ButtonBox["$West",
  BaseStyle->"AddOnsLink",
  ButtonData:>"$West",
  ButtonNote->"$West"],
 "."
}], "Text",
 CellTags->"$BreitMaison"]
}, Open  ]],

Cell["$Color", "ObjectName",
 CellTags->"$Color"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"$Color"],

Cell["\<\
$Color is False by default. If set to True, some special variables will be \
colored.\
\>", "Text",
 CellTags->"$Color"]
}, Open  ]],

Cell["$Covariant", "ObjectName",
 CellTags->"$Covariant"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"$Covariant"],

Cell["\<\
The boolean setting of $Covariant determines whether lorentz indices are \
displayed as lower indices (True) or as upper ones (False).\
\>", "Text",
 CellTags->"$Covariant"],

Cell[CellGroupData[{

Cell[BoxData["$Covariant"], "Input",
 CellTags->"$Covariant",
 CellLabel->"In[1911]:="],

Cell[BoxData[
 FormBox["False", TraditionalForm]], "Output",
 CellChangeTimes->{3.5009704718459578`*^9},
 CellTags->"$Covariant",
 CellLabel->"Out[1911]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GA", "[", "\[Mu]", "]"}]], "Input",
 CellTags->"$Covariant",
 CellLabel->"In[1912]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["\[Gamma]", "\[Mu]"], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970471900196*^9},
 CellTags->"$Covariant",
 CellLabel->"Out[1912]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"$Covariant", "=", "True"}], ";"}]], "Input",
 CellTags->"$Covariant",
 CellLabel->"In[1913]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GA", "[", "\[Mu]", "]"}]], "Input",
 CellTags->"$Covariant",
 CellLabel->"In[1914]:="],

Cell[BoxData[
 FormBox[
  SubscriptBox["\[Gamma]", "\[Mu]"], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970472081979*^9},
 CellTags->"$Covariant",
 CellLabel->"Out[1914]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"$Covariant", "=", "False"}], ";"}]], "Input",
 CellTags->"$Covariant",
 CellLabel->"In[1915]:="]
}, Open  ]],

Cell["$FCS", "ObjectName",
 CellTags->"$FCS"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"$FCS"],

Cell["\<\
$FCS is a list of functions with a short name. E.g. GA[nu] can be used \
instead of DiracGamma[nu].\
\>", "Text",
 CellTags->"$FCS"],

Cell[CellGroupData[{

Cell[BoxData["$FCS"], "Input",
 CellTags->"$FCS",
 CellLabel->"In[1916]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"\<\"FAD\"\>", ",", "\<\"FV\"\>", ",", "\<\"FVD\"\>", 
    ",", "\<\"GA\"\>", ",", "\<\"GA5\"\>", ",", "\<\"GS\"\>", 
    ",", "\<\"GSD\"\>", ",", "\<\"LC\"\>", ",", "\<\"LCD\"\>", 
    ",", "\<\"MT\"\>", ",", "\<\"MTD\"\>", ",", "\<\"SD\"\>", 
    ",", "\<\"SOD\"\>", ",", "\<\"SP\"\>", ",", "\<\"SPC\"\>", 
    ",", "\<\"SPD\"\>", ",", "\<\"SPL\"\>", ",", "\<\"FCI\"\>", 
    ",", "\<\"FCE\"\>", ",", "\<\"FI\"\>", ",", "\<\"FC\"\>", 
    ",", "\<\"GGV\"\>", ",", "\<\"GP\"\>", ",", "\<\"QGV\"\>", 
    ",", "\<\"QO\"\>", ",", "\<\"FDr\"\>", ",", "\<\"CDr\"\>"}], "}"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.50097047213542*^9},
 CellTags->"$FCS",
 CellLabel->"Out[1916]="]
}, Open  ]]
}, Open  ]],

Cell["$FCT", "ObjectName",
 CellTags->"$FCT"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"$FCT"],

Cell["\<\
Setting the global variable $FCT to True changes the default typesetting \
rules for the noncommutative multiplication operator Dot[].\
\>", "Text",
 CellTags->"$FCT"],

Cell[BoxData[
 RowBox[{
  RowBox[{"$FCT", " ", "=", " ", "True"}], ";"}]], "Input",
 CellTags->"$FCT",
 CellLabel->"In[1917]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"GA", "[", "\[Mu]", "]"}], " ", ".", " ", 
  RowBox[{"GA", "[", "\[Nu]", "]"}]}]], "Input",
 CellTags->"$FCT",
 CellLabel->"In[1918]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", "\[Mu]"], " ", 
   SuperscriptBox["\[Gamma]", "\[Nu]"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970472291071*^9},
 CellTags->"$FCT",
 CellLabel->"Out[1918]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"$FCT", " ", "=", " ", "False"}], ";"}]], "Input",
 CellTags->"$FCT",
 CellLabel->"In[1919]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"GA", "[", "\[Mu]", "]"}], " ", ".", " ", 
  RowBox[{"GA", "[", "\[Nu]", "]"}]}]], "Input",
 CellTags->"$FCT",
 CellLabel->"In[1920]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
   SuperscriptBox["\[Gamma]", "\[Nu]"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970472344816*^9},
 CellTags->"$FCT",
 CellLabel->"Out[1920]="]
}, Open  ]]
}, Open  ]],

Cell["$FeynCalcStuff", "ObjectName",
 CellTags->"$FeynCalcStuff"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"$FeynCalcStuff"],

Cell["$FeynCalcStuff is the list of availabe stuff in FeynCalc.", "Text",
 CellTags->"$FeynCalcStuff"]
}, Open  ]],

Cell["$FeynCalcVersion", "ObjectName",
 CellTags->"$FeynCalcVersion"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"$FeynCalcVersion"],

Cell["\<\
$FeynCalcVersion is a string that represents the version of FeynCalc.\
\>", "Text",
 CellTags->"$FeynCalcVersion"],

Cell[CellGroupData[{

Cell[BoxData["$FeynCalcVersion"], "Input",
 CellTags->"$FeynCalcVersion",
 CellLabel->"In[1921]:="],

Cell[BoxData[
 FormBox["\<\"8.0.0.beta1\"\>", TraditionalForm]], "Output",
 CellChangeTimes->{3.50097047238621*^9},
 CellTags->"$FeynCalcVersion",
 CellLabel->"Out[1921]="]
}, Open  ]]
}, Open  ]],

Cell["$FortranContinuationCharacter", "ObjectName",
 CellTags->"$FortranContinuationCharacter"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"$FortranContinuationCharacter"],

Cell["\<\
$FortranContinuationCharacter is the continuation character used in Write2.\
\>", "Text",
 CellTags->"$FortranContinuationCharacter"],

Cell[CellGroupData[{

Cell[BoxData["$FortranContinuationCharacter"], "Input",
 CellTags->"$FortranContinuationCharacter",
 CellLabel->"In[1922]:="],

Cell[BoxData[
 FormBox["\<\"&\"\>", TraditionalForm]], "Output",
 CellChangeTimes->{3.500970472507477*^9},
 CellTags->"$FortranContinuationCharacter",
 CellLabel->"Out[1922]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Write2",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Write2",
  ButtonNote->"Write2"],
 "."
}], "Text",
 CellTags->"$FortranContinuationCharacter"]
}, Open  ]],

Cell["$Gauge", "ObjectName",
 CellTags->"$Gauge"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"$Gauge"],

Cell["\<\
$Gauge(= 1/xi) is a constant specifying the gauge fixing parameter of QED in \
Lorentz gauge.  The usual choice is Feynman gauge, $Gauge=1. Notice that \
$Gauge is used by some functions, the option Gauge by others.\
\>", "Text",
 CellTags->"$Gauge"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Gauge",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Gauge",
  ButtonNote->"Gauge"],
 "."
}], "Text",
 CellTags->"$Gauge"]
}, Open  ]],

Cell["$Larin", "ObjectName",
 CellTags->"$Larin"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"$Larin"],

Cell[TextData[{
 "If set to True, the Larin-Gorishny-Atkyampo-DelBurgo-scheme for ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["\[Gamma]", "5"], TraditionalForm]]],
 " in ",
 StyleBox["D",
  FontSlant->"Italic"],
 "-dimensions is used, i.e. before evaluating traces (but after moving ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["\[Gamma]", "5"], TraditionalForm]]],
 " anticommuting in all dimensions to the right of the Dirac string) a \
product  gamma[mu].gamma5 is substituted to  -I/6 Eps[mu,al,be,si] \
gamma[al,be,si], where all indices live in ",
 StyleBox["D",
  FontSlant->"Italic"],
 "-dimensions now. Especially the Levic-Civita tensor is taken to be ",
 StyleBox["D",
  FontSlant->"Italic"],
 "-dimensional, i.e., contraction of two Eps's results in ",
 StyleBox["D",
  FontSlant->"Italic"],
 "'s. This has (FOR ONE AXIAL-VECTOR-CURRENT ONLY, it is not so clear if this \
scheme also works for more than one fermion line involving ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["\[Gamma]", "5"], TraditionalForm]]],
 ") the same effect as the Breitenlohner-Maison-'t Hooft-Veltman scheme."
}], "Text",
 CellTags->"$Larin"],

Cell[CellGroupData[{

Cell[BoxData["$Larin"], "Input",
 CellTags->"$Larin",
 CellLabel->"In[1923]:="],

Cell[BoxData[
 FormBox["False", TraditionalForm]], "Output",
 CellChangeTimes->{3.500970472572977*^9},
 CellTags->"$Larin",
 CellLabel->"Out[1923]="]
}, Open  ]],

Cell[TextData[{
 "See also:  ",
 ButtonBox["Tr",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Tr",
  ButtonNote->"Tr"],
 ", ",
 ButtonBox["$BreitMaison",
  BaseStyle->"AddOnsLink",
  ButtonData:>"$BreitMaison",
  ButtonNote->"$BreitMaison"],
 ", ",
 ButtonBox["$West",
  BaseStyle->"AddOnsLink",
  ButtonData:>"$West",
  ButtonNote->"$West"],
 "."
}], "Text",
 CellTags->"$Larin"]
}, Open  ]],

Cell["$LimitTo4", "ObjectName",
 CellTags->"$LimitTo4"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"$LimitTo4"],

Cell["\<\
$LimitTo4 is a global variable with default setting True. If set to False, no \
limit Dimension \[Rule] 4 is performed after tensor integral decomposition.\
\>", "Text",
 CellTags->"$LimitTo4"],

Cell[CellGroupData[{

Cell[BoxData["$LimitTo4"], "Input",
 CellTags->"$LimitTo4",
 CellLabel->"In[1924]:="],

Cell[BoxData[
 FormBox["True", TraditionalForm]], "Output",
 CellChangeTimes->{3.500970472613949*^9},
 CellTags->"$LimitTo4",
 CellLabel->"Out[1924]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["PaVe",
  BaseStyle->"AddOnsLink",
  ButtonData:>"PaVe",
  ButtonNote->"PaVe"],
 ", ",
 ButtonBox["PaVeReduce",
  BaseStyle->"AddOnsLink",
  ButtonData:>"PaVeReduce",
  ButtonNote->"PaVeReduce"],
 ", ",
 ButtonBox["OneLoop",
  BaseStyle->"AddOnsLink",
  ButtonData:>"OneLoop",
  ButtonNote->"OneLoop"],
 "."
}], "Text",
 CellTags->"$LimitTo4"]
}, Open  ]],

Cell["$LorentzIndices", "ObjectName",
 CellTags->"$LorentzIndices"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"$LorentzIndices"],

Cell["\<\
$LorentzIndices is a global variable. If set to True the dimension of \
LorentzIndex is displayed as an index.\
\>", "Text",
 CellTags->"$LorentzIndices"],

Cell[BoxData[
 RowBox[{
  RowBox[{"$LorentzIndices", "=", "True"}], ";"}]], "Input",
 CellTags->"$LorentzIndices",
 CellLabel->"In[1925]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"MetricTensor", "[", 
   RowBox[{"\[Alpha]", ",", "\[Beta]", ",", 
    RowBox[{"Dimension", "\[Rule]", "n"}]}], "]"}], 
  RowBox[{"DiracMatrix", "[", 
   RowBox[{"\[Alpha]", ",", 
    RowBox[{"Dimension", "\[Rule]", "n"}]}], "]"}]}]], "Input",
 CellTags->"$LorentzIndices",
 CellLabel->"In[1926]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", 
    FormBox[
     SubscriptBox[
      FormBox["\[Alpha]",
       TraditionalForm], 
      FormBox["n",
       TraditionalForm]],
     TraditionalForm]], " ", 
   SuperscriptBox["g", 
    RowBox[{
     FormBox[
      SubscriptBox[
       FormBox["\[Alpha]",
        TraditionalForm], 
       FormBox["n",
        TraditionalForm]],
      TraditionalForm], 
     FormBox[
      SubscriptBox[
       FormBox["\[Beta]",
        TraditionalForm], 
       FormBox["n",
        TraditionalForm]],
      TraditionalForm]}]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009704727730627`*^9},
 CellTags->"$LorentzIndices",
 CellLabel->"Out[1926]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "StandardForm"}]], "Input",
 CellTags->"$LorentzIndices",
 CellLabel->"In[1927]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"DiracGamma", "[", 
   RowBox[{
    RowBox[{"LorentzIndex", "[", 
     RowBox[{"\[Alpha]", ",", "n"}], "]"}], ",", "n"}], "]"}], " ", 
  RowBox[{"MetricTensor", "[", 
   RowBox[{"\[Alpha]", ",", "\[Beta]", ",", 
    RowBox[{"Dimension", "\[Rule]", "n"}]}], "]"}]}]], "Output",
 CellChangeTimes->{3.5009704728532867`*^9},
 CellTags->"$LorentzIndices",
 CellLabel->"Out[1927]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"MTD", "[", 
    RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}], " ", 
   RowBox[{"FVD", "[", 
    RowBox[{"p", ",", "\[Beta]"}], "]"}], 
   RowBox[{"GAD", "[", "\[Mu]", "]"}]}], "//", "FCI"}]], "Input",
 CellTags->"$LorentzIndices",
 CellLabel->"In[1928]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", 
    FormBox[
     SubscriptBox[
      FormBox["\[Mu]",
       TraditionalForm], 
      FormBox["D",
       TraditionalForm]],
     TraditionalForm]], " ", 
   SuperscriptBox[
    FormBox["p",
     TraditionalForm], 
    FormBox[
     SubscriptBox[
      FormBox["\[Beta]",
       TraditionalForm], 
      FormBox["D",
       TraditionalForm]],
     TraditionalForm]], " ", 
   SuperscriptBox["g", 
    RowBox[{
     FormBox[
      SubscriptBox[
       FormBox["\[Alpha]",
        TraditionalForm], 
       FormBox["D",
        TraditionalForm]],
      TraditionalForm], 
     FormBox[
      SubscriptBox[
       FormBox["\[Beta]",
        TraditionalForm], 
       FormBox["D",
        TraditionalForm]],
      TraditionalForm]}]]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970472917713*^9},
 CellTags->"$LorentzIndices",
 CellLabel->"Out[1928]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "StandardForm"}]], "Input",
 CellTags->"$LorentzIndices",
 CellLabel->"In[1929]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"DiracGamma", "[", 
   RowBox[{
    RowBox[{"LorentzIndex", "[", 
     RowBox[{"\[Mu]", ",", "D"}], "]"}], ",", "D"}], "]"}], " ", 
  RowBox[{"Pair", "[", 
   RowBox[{
    RowBox[{"LorentzIndex", "[", 
     RowBox[{"\[Alpha]", ",", "D"}], "]"}], ",", 
    RowBox[{"LorentzIndex", "[", 
     RowBox[{"\[Beta]", ",", "D"}], "]"}]}], "]"}], " ", 
  RowBox[{"Pair", "[", 
   RowBox[{
    RowBox[{"LorentzIndex", "[", 
     RowBox[{"\[Beta]", ",", "D"}], "]"}], ",", 
    RowBox[{"Momentum", "[", 
     RowBox[{"p", ",", "D"}], "]"}]}], "]"}]}]], "Output",
 CellChangeTimes->{3.500970473011181*^9},
 CellTags->"$LorentzIndices",
 CellLabel->"Out[1929]//StandardForm="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"$LorentzIndices", "=", "False"}], ";"}]], "Input",
 CellTags->"$LorentzIndices",
 CellLabel->"In[1930]:="]
}, Open  ]],

Cell["$MIntegrate", "ObjectName",
 CellTags->"$MIntegrate"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"$MIntegrate"],

Cell["\<\
$MIntegrate is a global list of integrations done by Mathematica inside \
OPEIntDelta.\
\>", "Text",
 CellTags->"$MIntegrate"],

Cell[CellGroupData[{

Cell[BoxData["$MIntegrate"], "Input",
 CellTags->"$MIntegrate",
 CellLabel->"In[1931]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", "}"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.50097047313678*^9},
 CellTags->"$MIntegrate",
 CellLabel->"Out[1931]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["OPEIntDelta",
  BaseStyle->"AddOnsLink",
  ButtonData:>"OPEIntDelta",
  ButtonNote->"OPEIntDelta"],
 "."
}], "Text",
 CellTags->"$MIntegrate"]
}, Open  ]],

Cell["$MU", "ObjectName",
 CellTags->"$MU"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"$MU"],

Cell["\<\
$MU is the head for dummy indices which may be introduced by Chisholm (and \
evtl. Contract and DiracReduce).\
\>", "Text",
 CellTags->"$MU"],

Cell[CellGroupData[{

Cell[BoxData["$MU"], "Input",
 CellTags->"$MU",
 CellLabel->"In[1932]:="],

Cell[BoxData[
 FormBox["$MU", TraditionalForm]], "Output",
 CellChangeTimes->{3.500970473177387*^9},
 CellTags->"$MU",
 CellLabel->"Out[1932]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Chisholm",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Chisholm",
  ButtonNote->"Chisholm"],
 "."
}], "Text",
 CellTags->"$MU"]
}, Open  ]],

Cell["$MemoryAvailable", "ObjectName",
 CellTags->"$MemoryAvailable"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"$MemoryAvailable"],

Cell[TextData[{
 "$MemoryAvailable is  a global variable which is set to an integer ",
 StyleBox["n",
  FontSlant->"Italic"],
 ", where ",
 StyleBox["n",
  FontSlant->"Italic"],
 " is the available amount of main memory in MB. The default is 128. It \
should be increased if possible. The higher $MemoryAvailable can be, the more \
intermediate steps do not have to be repeated by FeynCalc."
}], "Text",
 CellTags->"$MemoryAvailable"],

Cell[CellGroupData[{

Cell[BoxData["$MemoryAvailable"], "Input",
 CellTags->"$MemoryAvailable",
 CellLabel->"In[1933]:="],

Cell[BoxData[
 FormBox["256", TraditionalForm]], "Output",
 CellChangeTimes->{3.500970473217826*^9},
 CellTags->"$MemoryAvailable",
 CellLabel->"Out[1933]="]
}, Open  ]]
}, Open  ]],

Cell["$NonComm", "ObjectName",
 CellTags->"$NonComm"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"$NonComm"],

Cell["\<\
$NonComm contains a list of all non-commutative heads present.\
\>", "Text",
 CellTags->"$NonComm"],

Cell[CellGroupData[{

Cell[BoxData["$NonComm"], "Input",
 CellTags->"$NonComm",
 CellLabel->"In[1934]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
   "ChiralityProjector", ",", "DiracGamma", ",", "DiracGammaT", ",", 
    "DiracMatrix", ",", "DiracSigma", ",", "DiracSlash", ",", "Spinor", ",", 
    "GAD", ",", "GA", ",", "GSD", ",", "GS", ",", "LeftPartialD", ",", 
    "LeftRightPartialD2", ",", "LeftRightPartialD", ",", 
    "HighEnergyPhysics`FeynCalc`artialD`HighEnergyPhysics`FeynCalc`\
HighEnergyPhysics`FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`\
PartialD`PartialD`artialD`HighEnergyPhysics`FeynCalc`HighEnergyPhysics`\
FeynCalc`PartialD`PartialD`HighEnergyPhysics`FeynCalc`PartialD`PartialD", ",",
     "PauliSigma", ",", "QuantumField", ",", "RightPartialD", ",", 
    "SpinorUBar", ",", "SpinorU", ",", "SpinorVBar", ",", "SpinorV", ",", 
    "SUNT", ",", "FieldStrength", ",", "QuarkGluonVertex", ",", 
    "QuarkPropagator", ",", "C", ",", "Twist2QuarkOperator", ",", 
    "CovariantD", ",", "OPESum", ",", "Twist2AlienOperator", ",", 
    "Twist2CounterOperator"}], "}"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970473256143*^9},
 CellTags->"$NonComm",
 CellLabel->"Out[1934]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["NonCommQ",
  BaseStyle->"AddOnsLink",
  ButtonData:>"NonCommQ",
  ButtonNote->"NonCommQ"],
 ", ",
 ButtonBox["NonCommutative",
  BaseStyle->"AddOnsLink",
  ButtonData:>"NonCommutative",
  ButtonNote->"NonCommutative"],
 "."
}], "Text",
 CellTags->"$NonComm"]
}, Open  ]],

Cell["$PairBrackets", "ObjectName",
 CellTags->"$PairBrackets"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"$PairBrackets"],

Cell["\<\
$PairBrackets determines whether brackets are drawn around scalar products in \
the notebook interface.\
\>", "Text",
 CellTags->"$PairBrackets"],

Cell[CellGroupData[{

Cell[BoxData["$PairBrackets"], "Input",
 CellTags->"$PairBrackets",
 CellLabel->"In[1935]:="],

Cell[BoxData[
 FormBox["False", TraditionalForm]], "Output",
 CellChangeTimes->{3.500970473372106*^9},
 CellTags->"$PairBrackets",
 CellLabel->"Out[1935]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"SP", "[", 
   RowBox[{"p", ",", "q"}], "]"}], 
  RowBox[{"SP", "[", 
   RowBox[{"r", ",", "s"}], "]"}]}]], "Input",
 CellTags->"$PairBrackets",
 CellLabel->"In[1936]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox[
    RowBox[{
     FormBox["p",
      TraditionalForm], "\[CenterDot]", 
     FormBox["q",
      TraditionalForm]}],
    TraditionalForm], " ", 
   FormBox[
    RowBox[{
     FormBox["r",
      TraditionalForm], "\[CenterDot]", 
     FormBox["s",
      TraditionalForm]}],
    TraditionalForm]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.50097047351199*^9},
 CellTags->"$PairBrackets",
 CellLabel->"Out[1936]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"$PairBrackets", "=", "True"}]], "Input",
 CellTags->"$PairBrackets",
 CellLabel->"In[1937]:="],

Cell[BoxData[
 FormBox["True", TraditionalForm]], "Output",
 CellChangeTimes->{3.500970473560692*^9},
 CellTags->"$PairBrackets",
 CellLabel->"Out[1937]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"SP", "[", 
   RowBox[{"p", ",", "q"}], "]"}], 
  RowBox[{"SP", "[", 
   RowBox[{"r", ",", "s"}], "]"}]}]], "Input",
 CellTags->"$PairBrackets",
 CellLabel->"In[1938]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox[
    RowBox[{"(", 
     FormBox["p",
      TraditionalForm], "\[CenterDot]", 
     FormBox["q",
      TraditionalForm], ")"}],
    TraditionalForm], " ", 
   FormBox[
    RowBox[{"(", 
     FormBox["r",
      TraditionalForm], "\[CenterDot]", 
     FormBox["s",
      TraditionalForm], ")"}],
    TraditionalForm]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009704737984457`*^9},
 CellTags->"$PairBrackets",
 CellLabel->"Out[1938]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"$PairBrackets", "=", "False"}]], "Input",
 CellTags->"$PairBrackets",
 CellLabel->"In[1939]:="],

Cell[BoxData[
 FormBox["False", TraditionalForm]], "Output",
 CellChangeTimes->{3.5009704739107637`*^9},
 CellTags->"$PairBrackets",
 CellLabel->"Out[1939]="]
}, Open  ]]
}, Open  ]],

Cell["$SpinorMinimal", "ObjectName",
 CellTags->"$SpinorMinimal"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"$SpinorMinimal"],

Cell["\<\
$SpinorMinimal is a global switch for an additional simplification attempt in \
DiracSimplify for more than one Spinor-line. The default is False, since \
otherwise it costs too much time.\
\>", "Text",
 CellTags->"$SpinorMinimal"],

Cell[CellGroupData[{

Cell[BoxData["$SpinorMinimal"], "Input",
 CellTags->"$SpinorMinimal",
 CellLabel->"In[1940]:="],

Cell[BoxData[
 FormBox["False", TraditionalForm]], "Output",
 CellChangeTimes->{3.500970473954987*^9},
 CellTags->"$SpinorMinimal",
 CellLabel->"Out[1940]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["DiracSimplify",
  BaseStyle->"AddOnsLink",
  ButtonData:>"DiracSimplify",
  ButtonNote->"DiracSimplify"],
 "."
}], "Text",
 CellTags->"$SpinorMinimal"]
}, Open  ]],

Cell["$VeryVerbose", "ObjectName",
 CellTags->"$VeryVerbose"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"$VeryVerbose"],

Cell["\<\
$VeryVerbose is a global variable with default setting 0. If set to 1, 2, \
..., less and more intermediate comments and informations are displayed \
during calculations.\
\>", "Text",
 CellTags->"$VeryVerbose"],

Cell[CellGroupData[{

Cell[BoxData["$VeryVerbose"], "Input",
 CellTags->"$VeryVerbose",
 CellLabel->"In[1941]:="],

Cell[BoxData[
 FormBox["0", TraditionalForm]], "Output",
 CellChangeTimes->{3.50097047414876*^9},
 CellTags->"$VeryVerbose",
 CellLabel->"Out[1941]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"$VeryVerbose", " ", "=", " ", "3"}], ";"}]], "Input",
 CellTags->"$VeryVerbose",
 CellLabel->"In[1942]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Collect2", "[", 
   RowBox[{
    RowBox[{"Expand", "[", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"x", "-", "y", "-", "z"}], ")"}], "^", "6"}], "]"}], ",", 
    "x"}], "]"}], ";"}]], "Input",
 CellTags->"$VeryVerbose",
 CellLabel->"In[1943]:="],

Cell["\<\
expanding. inhomogeneous part; LeafCount = 43; factored. #1#2#3#4#5#6\
\>", "Print",
 CellChangeTimes->{3.5009704742513933`*^9},
 CellTags->"$VeryVerbose"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"length \"\>", "\[InvisibleSpace]", "21", 
    "\[InvisibleSpace]", "\<\".\"\>"}],
   SequenceForm["length ", 21, "."],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.500970474332724*^9},
 CellTags->"$VeryVerbose"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"collecting \"\>", "\[InvisibleSpace]", "6", 
    "\[InvisibleSpace]", "\<\" terms.\"\>"}],
   SequenceForm["collecting ", 6, " terms."],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.5009704743609333`*^9},
 CellTags->"$VeryVerbose"]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"$VeryVerbose", "=", "0"}], ";"}]], "Input",
 CellTags->"$VeryVerbose",
 CellLabel->"In[1944]:="],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["$V0",
  BaseStyle->"AddOnsLink",
  ButtonData:>"$V0",
  ButtonNote->"$V0"],
 "."
}], "Text",
 CellTags->"$V0"]
}, Open  ]],

Cell["$West", "ObjectName",
 CellTags->"$West"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"$West"],

Cell[TextData[{
 "If $West is set to True (which is the default), traces involving more than \
4 Dirac matrices and ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["\[Gamma]", "5"], TraditionalForm]]],
 " are calculated recursively according to formula (A.5) from Comp. Phys. \
Comm 77 (1993) 286-298, which is based on the Breitenlohner Maison ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["\[Gamma]", "5"], TraditionalForm]]],
 "-scheme."
}], "Text",
 CellTags->"$West"],

Cell[TextData[{
 "See examples in ",
 ButtonBox["Tr",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Tr",
  ButtonNote->"Tr"],
 "."
}], "Text",
 CellTags->"$West"],

Cell["Evaluation time and memory usage", "Commentary"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"AbsoluteTime", "[", "]"}], "-", "absolutestarttime"}]], "Input",
 CellLabel->"In[1945]:="],

Cell[BoxData[
 FormBox["352.556446`8.99877365304694", TraditionalForm]], "Output",
 CellChangeTimes->{3.5009704744685802`*^9},
 CellLabel->"Out[1945]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"MemoryInUse", "[", "]"}]], "Input",
 CellLabel->"In[1946]:="],

Cell[BoxData[
 FormBox["127326768", TraditionalForm]], "Output",
 CellChangeTimes->{3.500970474536861*^9},
 CellLabel->"Out[1946]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"MaxMemoryUsed", "[", "]"}]], "Input",
 CellLabel->"In[1947]:="],

Cell[BoxData[
 FormBox["127612088", TraditionalForm]], "Output",
 CellChangeTimes->{3.5009704746037397`*^9},
 CellLabel->"Out[1947]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TimeUsed", "[", "]"}]], "Input",
 CellLabel->"In[1948]:="],

Cell[BoxData[
 FormBox["112.68`", TraditionalForm]], "Output",
 CellChangeTimes->{3.500970474647911*^9},
 CellLabel->"Out[1948]="]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Functions (experimental)", "Section",
 CellTags->"Experimental"],

Cell["DoPolarizationSums", "ObjectName",
 CellTags->"DoPolarizationSums"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"DoPolarizationSums"],

Cell["\<\
***EXPERIMENTAL***
DoPolarizationSums[exp] sums over vector polarizations for expressions with a \
factor of the form
Pair[LorentzIndex[rho1_], Momentum[Polarization[p_, -I]]]*
Pair[LorentzIndex[rho2_], Momentum[Polarization[p_, I]]].\
\>", "Text",
 CellTags->"DoPolarizationSums"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Polarization",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Polarization",
  ButtonNote->"Polarization"],
 ", ",
 ButtonBox["Uncontract",
  BaseStyle->"AddOnsLink",
  ButtonData:>"Uncontract",
  ButtonNote->"Uncontract"],
 "."
}], "Text",
 CellTags->"DoPolarizationSums"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"DoPolarizationSums"],

Cell["\<\
In the following (and indeed everywhere else within FeynCalc), notice that \
Lorentz indices written as super or subscripts are not to be taken as such. \
Instead, by convention, when two indices are contracted one is always lower \
and the other upper.\
\>", "Text",
 CellTags->"DoPolarizationSums"],

Cell["\<\
FeynCalc uses the following normalization of Polarization vectors:\
\>", "Text",
 CellTags->"DoPolarizationSums"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Pair", "[", 
   RowBox[{
    RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
    RowBox[{"Momentum", "[", 
     RowBox[{"Polarization", "[", 
      RowBox[{"p4", ",", 
       RowBox[{"-", "\[ImaginaryI]"}]}], "]"}], "]"}]}], "]"}], " ", 
  RowBox[{"Pair", "[", 
   RowBox[{
    RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
    RowBox[{"Momentum", "[", 
     RowBox[{"Polarization", "[", 
      RowBox[{"p4", ",", "\[ImaginaryI]"}], "]"}], "]"}]}], "]"}]}]], "Input",\

 CellTags->"DoPolarizationSums",
 CellLabel->"In[1949]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubsuperscriptBox["\[CurlyEpsilon]", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], "*"], "(", "p4", ")"}], " ", 
   RowBox[{
    SubscriptBox["\[CurlyEpsilon]", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], "(", "p4", ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.5009704747114058`*^9},
 CellTags->"DoPolarizationSums",
 CellLabel->"Out[1949]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "Contract"}]], "Input",
 CellTags->"DoPolarizationSums",
 CellLabel->"In[1950]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", "1"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970474816437*^9},
 CellTags->"DoPolarizationSums",
 CellLabel->"Out[1950]="]
}, Open  ]],

Cell[TextData[{
 "DoPolarizationSums is chosen in such a way as to be consistent with this \
normalization and with ",
 Cell[BoxData[
  FormBox[
   UnderscriptBox["\[Sum]", "Polarizations"], TraditionalForm]]],
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SubscriptBox["\[CurlyEpsilon]", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], "(", 
     FormBox[
      SubscriptBox[
       TagBox[
        StyleBox["\<\"p\"\>",
         FontSlant->"Italic"],
        StyleForm[#, FontSlant -> "Italic"]& ], "4"],
      TraditionalForm], ")"}], " ", 
    RowBox[{
     SubscriptBox[
      RowBox[{"(", 
       SuperscriptBox["\[CurlyEpsilon]", "\[Nu]"], ")"}], 
      FormBox["",
       TraditionalForm]], "(", 
     FormBox[
      SubscriptBox[
       TagBox[
        StyleBox["\<\"p\"\>",
         FontSlant->"Italic"],
        StyleForm[#, FontSlant -> "Italic"]& ], "4"],
      TraditionalForm], ")"}]}], TraditionalForm]]],
 "=",
 Cell[BoxData[
  FormBox[
   RowBox[{"-", 
    FormBox[
     SuperscriptBox[
      SubscriptBox["g", "\[Mu]"], 
      RowBox[{"\[NoBreak]", 
       FormBox["\[Nu]",
        TraditionalForm]}]],
     TraditionalForm]}], TraditionalForm]]],
 ":"
}], "Text",
 CellTags->"DoPolarizationSums"],

Cell[TextData[{
 "We can choose e.g. the following polarization vectors, labeled with a \
subscript, {",
 Cell[BoxData[
  FormBox[
   SubscriptBox["e", "1"], TraditionalForm]]],
 ",",
 Cell[BoxData[
  FormBox[
   SubscriptBox["e", "2"], TraditionalForm]]],
 ",",
 Cell[BoxData[
  FormBox[
   SubscriptBox["e", "3"], TraditionalForm]]],
 ",",
 Cell[BoxData[
  FormBox[
   SubscriptBox["e", "4"], TraditionalForm]]],
 "} = {(\[ImaginaryI],0,0,0), (0,\[ImaginaryI],0,0), (0,0,\[ImaginaryI],0), \
(0,0,0,\[ImaginaryI])}."
}], "Text",
 CellTags->"DoPolarizationSums"],

Cell[BoxData[
 RowBox[{"Clear", "[", 
  RowBox[{"ee", ",", "tt"}], "]"}]], "Input",
 CellTags->"DoPolarizationSums",
 CellLabel->"In[1951]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"ev", "[", "i_", "]"}], ":=", 
   RowBox[{"ReplacePart", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"0", ",", "0", ",", "0", ",", "0"}], "}"}], ",", "I", ",", 
     "i"}], "]"}]}], ";"}]], "Input",
 CellTags->"DoPolarizationSums",
 CellLabel->"In[1952]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"ev", "[", "1", "]"}], ",", 
   RowBox[{"ev", "[", "2", "]"}], ",", 
   RowBox[{"ev", "[", "3", "]"}], ",", 
   RowBox[{"ev", "[", "4", "]"}]}], "}"}]], "Input",
 CellTags->"DoPolarizationSums",
 CellLabel->"In[1953]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"\[ImaginaryI]", "0", "0", "0"},
     {"0", "\[ImaginaryI]", "0", "0"},
     {"0", "0", "\[ImaginaryI]", "0"},
     {"0", "0", "0", "\[ImaginaryI]"}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.5009704749425898`*^9},
 CellTags->"DoPolarizationSums",
 CellLabel->"Out[1953]="]
}, Open  ]],

Cell[TextData[{
 StyleBox["ee[i][\[Mu]]", "Input"],
 " is ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    SubscriptBox["e", "i"], "\[Mu]"], TraditionalForm]]],
 ":"
}], "Text",
 CellTags->"DoPolarizationSums"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"ee", "[", "i_", "]"}], "[", "mu_", "]"}], ":=", 
   RowBox[{
    RowBox[{"ev", "[", "i", "]"}], "[", 
    RowBox[{"[", "mu", "]"}], "]"}]}], ";"}]], "Input",
 CellTags->"DoPolarizationSums",
 CellLabel->"In[1954]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"tt", "[", 
   RowBox[{"i_", ",", "j_", ",", "mu_", ",", "nu_"}], "]"}], ":=", 
  RowBox[{
   RowBox[{
    RowBox[{"ee", "[", "i", "]"}], "[", "mu", "]"}], 
   RowBox[{
    RowBox[{"ee", "[", "j", "]"}], "[", "nu", "]"}]}]}]], "Input",
 CellTags->"DoPolarizationSums",
 CellLabel->"In[1955]:="],

Cell[TextData[{
 StyleBox["tt[i,j,\[Mu],\[Nu]]", "Input"],
 " is ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    SubscriptBox["e", "i"], "\[Mu]"], TraditionalForm]]],
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    SubscriptBox["e", "j"], "\[Nu]"], TraditionalForm]]],
 ":"
}], "Text",
 CellTags->"DoPolarizationSums"],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   UnderscriptBox["\[Sum]", 
    RowBox[{"i", ",", "j"}]], TraditionalForm]]],
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     FormBox[
      SuperscriptBox[
       SubscriptBox["e", "i"], "\[Mu]"],
      TraditionalForm], 
     FormBox[
      SubscriptBox["e", 
       RowBox[{"j", ",", "\[Nu]"}]],
      TraditionalForm]}]}], TraditionalForm]]],
 "=",
 Cell[BoxData[
  FormBox[
   RowBox[{"-", 
    FormBox[
     SuperscriptBox[
      SubscriptBox["g", "\[Mu]"], 
      RowBox[{"\[NoBreak]", 
       FormBox["\[Nu]",
        TraditionalForm]}]],
     TraditionalForm]}], TraditionalForm]]],
 ":"
}], "Text",
 CellTags->"DoPolarizationSums"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Table", "[", 
  RowBox[{
   RowBox[{
    UnderoverscriptBox["\[Sum]", 
     RowBox[{"i", "=", "1"}], "4"], 
    RowBox[{"tt", "[", 
     RowBox[{"i", ",", "i", ",", "mu", ",", "nu"}], "]"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"mu", ",", "1", ",", "4"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"nu", ",", "1", ",", "4"}], "}"}]}], "]"}]], "Input",
 CellTags->"DoPolarizationSums",
 CellLabel->"In[1956]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      RowBox[{"-", "1"}], "0", "0", "0"},
     {"0", 
      RowBox[{"-", "1"}], "0", "0"},
     {"0", "0", 
      RowBox[{"-", "1"}], "0"},
     {"0", "0", "0", 
      RowBox[{"-", "1"}]}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970475028067*^9},
 CellTags->"DoPolarizationSums",
 CellLabel->"Out[1956]="]
}, Open  ]],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   UnderscriptBox["\[Sum]", 
    RowBox[{"\[Mu]", ",", "\[Nu]"}]], TraditionalForm]]],
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     FormBox[
      SuperscriptBox[
       SubscriptBox["e", "i"], "\[Mu]"],
      TraditionalForm], 
     FormBox[
      SubscriptBox["e", 
       RowBox[{"j", ",", "\[Nu]"}]],
      TraditionalForm]}]}], TraditionalForm]]],
 "=",
 Cell[BoxData[
  FormBox[
   RowBox[{"-", 
    FormBox[
     SuperscriptBox[
      SubscriptBox["g", "i"], "j"],
     TraditionalForm]}], TraditionalForm]]],
 ":"
}], "Text",
 CellTags->"DoPolarizationSums"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Table", "[", 
  RowBox[{
   RowBox[{
    UnderoverscriptBox["\[Sum]", 
     RowBox[{"mu", "=", "1"}], "4"], 
    RowBox[{"tt", "[", 
     RowBox[{"i", ",", "j", ",", "mu", ",", "mu"}], "]"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"i", ",", "1", ",", "4"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"j", ",", "1", ",", "4"}], "}"}]}], "]"}]], "Input",
 CellTags->"DoPolarizationSums",
 CellLabel->"In[1957]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      RowBox[{"-", "1"}], "0", "0", "0"},
     {"0", 
      RowBox[{"-", "1"}], "0", "0"},
     {"0", "0", 
      RowBox[{"-", "1"}], "0"},
     {"0", "0", "0", 
      RowBox[{"-", "1"}]}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970475133905*^9},
 CellTags->"DoPolarizationSums",
 CellLabel->"Out[1957]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ptest", "=", 
  RowBox[{
   RowBox[{"Pair", "[", 
    RowBox[{
     RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
     RowBox[{"LorentzIndex", "[", 
      RowBox[{"ComplexIndex", "[", "\[Mu]", "]"}], "]"}]}], "]"}], " ", 
   RowBox[{"Pair", "[", 
    RowBox[{
     RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
     RowBox[{"Momentum", "[", 
      RowBox[{"Polarization", "[", 
       RowBox[{"p4", ",", 
        RowBox[{"-", "\[ImaginaryI]"}]}], "]"}], "]"}]}], "]"}], " ", 
   RowBox[{"Pair", "[", 
    RowBox[{
     RowBox[{"LorentzIndex", "[", 
      RowBox[{"ComplexIndex", "[", "\[Mu]", "]"}], "]"}], ",", 
     RowBox[{"Momentum", "[", 
      RowBox[{"Polarization", "[", 
       RowBox[{"p4", ",", "\[ImaginaryI]"}], "]"}], "]"}]}], 
    "]"}]}]}]], "Input",
 CellTags->"DoPolarizationSums",
 CellLabel->"In[1958]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["g", 
    RowBox[{"\[Mu]", 
     FormBox[
      SuperscriptBox["\[Mu]", "*"],
      TraditionalForm]}]], " ", 
   RowBox[{
    SubsuperscriptBox["\[CurlyEpsilon]", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], "*"], "(", "p4", ")"}], " ", 
   RowBox[{
    SubscriptBox["\[CurlyEpsilon]", 
     FormBox[
      FormBox[
       SuperscriptBox["\[Mu]", "*"],
       TraditionalForm],
      TraditionalForm]], "(", "p4", ")"}]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970475235978*^9},
 CellTags->"DoPolarizationSums",
 CellLabel->"Out[1958]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ptest", "//", "DoPolarizationSums"}]], "Input",
 CellTags->"DoPolarizationSums",
 CellLabel->"In[1959]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   SuperscriptBox[
    FormBox[
     SuperscriptBox["g", 
      RowBox[{"\[Mu]", 
       FormBox[
        SuperscriptBox["\[Mu]", "*"],
        TraditionalForm]}]],
     TraditionalForm], "2"]}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970475351561*^9},
 CellTags->"DoPolarizationSums",
 CellLabel->"Out[1959]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "Contract"}]], "Input",
 CellTags->"DoPolarizationSums",
 CellLabel->"In[1960]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", "4"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970475476322*^9},
 CellTags->"DoPolarizationSums",
 CellLabel->"Out[1960]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ptest", "//", "Contract"}]], "Input",
 CellTags->"DoPolarizationSums",
 CellLabel->"In[1961]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", "1"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970475567568*^9},
 CellTags->"DoPolarizationSums",
 CellLabel->"Out[1961]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "DoPolarizationSums"}]], "Input",
 CellTags->"DoPolarizationSums",
 CellLabel->"In[1962]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", "4"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970475674899*^9},
 CellTags->"DoPolarizationSums",
 CellLabel->"Out[1962]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"Clear", "[", 
  RowBox[{"ee", ",", "tt", ",", "ev", ",", "ptest"}], "]"}]], "Input",
 CellTags->"DoPolarizationSums",
 CellLabel->"In[1963]:="]
}, Open  ]],

Cell["FeynmanDoIntegrals ***unfinished***", "ObjectName",
 CellTags->"FeynmanDoIntegrals"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FeynmanDoIntegrals"],

Cell["\<\
***EXPERIMENTAL***
 FeynmanDoIntegrals[exp, moms, vars] attempts to evaluate integrals over \
Feynman parameters vars in an expression exp as produced e.g. with \
FeynmanReduce. The variables given must be atomic quantities (AtomQ). If vars \
is omitted all variables in the integrals will be integrated. If vars is \
given, only the variables in vars will be integrated. moms is a list of all \
external momenta. The integrals in exp must be given in the form \
Integratedx[x, low, up].int, where x is the integration variable, low and up \
are the integration limits and int the integrand. The interval [low,up] is \
assumed to include integration bounds put by possible DeltaFunctions and \
moreover it is assumed that up \[GreaterEqual] 0 and that up > low. The two \
options FCIntegrate and FCNIntegrate determine which integration will be \
applied to integrals judged respectively analytically and numerically doable. \
This judging is a very rough one. Using (TimedIntegrate[##, \
Integrate\[Rule]Integrate]&) or (TimedIntegrate[##, \
Integrate\[Rule]NIntegrate]&) as the setting of one or both allows for finer \
judging. Those that are judged neither analytically nor numerically doable \
are left unevaluated, but can of course be attempted evaluated by a simple \
sustitution. Beside the explicit options of FeynmanDoIntegrals options of the \
integration functions specified (FCIntegrate and FCNIntegrate) may be given \
and are passed on to these.\
\>", "Text",
 CellTags->"FeynmanDoIntegrals"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FeynmanReduce",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FeynmanReduce",
  ButtonNote->"FeynmanReduce"],
 ", ",
 ButtonBox["FeynmanParametrize1",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FeynmanParametrize1",
  ButtonNote->"FeynmanParametrize1"],
 "."
}], "Text",
 CellTags->"FeynmanDoIntegrals"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "FeynmanDoIntegrals", "]"}]], "Input",
 CellTags->"FeynmanDoIntegrals",
 CellLabel->"In[1964]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"NIntegrate", "\[Rule]", "False"}], ",", 
    RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
    RowBox[{"EpsilonOrder", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "2"}], ",", "0"}], "}"}]}], ",", 
    RowBox[{"FCIntegrate", "\[Rule]", "TimedIntegrate"}], ",", 
    RowBox[{"Simplify", "\[Rule]", "True"}], ",", 
    RowBox[{"Expand", "\[Rule]", "True"}], ",", 
    RowBox[{"Series", "\[Rule]", "True"}], ",", 
    RowBox[{"FCNIntegrate", "\[Rule]", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"DOT", "(", 
        RowBox[{
         RowBox[{"Sequence", "@@", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             RowBox[{
             "HighEnergyPhysics`fcdevel`FeynmanDoIntegrals`Private`\
Integratedx", "@@", "#1"}], "&"}], ")"}], "/@", 
           RowBox[{"{", "##2", "}"}]}]}], ",", "#1"}], ")"}], "&"}], 
      ")"}]}]}], "}"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970475771927*^9},
 CellTags->"FeynmanDoIntegrals",
 CellLabel->"Out[1964]="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"FeynmanDoIntegrals"],

Cell[BoxData[""], "Input",
 CellTags->"FeynmanDoIntegrals",
 CellLabel->"In[1965]:="]
}, Open  ]],

Cell["FeynmanParametrize1 ***unfinished***", "ObjectName",
 CellTags->"FeynmanParametrize1"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FeynmanParametrize1"],

Cell["\<\
***EXPERIMENTAL***
 FeynmanParametrize1[exp,k,Method->Denominator] introduces Feynman parameters \
for all one-loop integrals in exp (k = integration momentum) using formula \
(11.A.1) from \"The Quantum Theory of Fields\" vol. 1 by Steven Weinberg.  \
FeynmanParametrize1[exp,k,Method->Exp] introduces Feynman parameters for all \
one-loop integrals in exp (k = integration momentum) using 1/(A-I eps) = I \
Integrate[Exp[-I x (A-I eps)],{x,0,Infinity}, \
Assumptions->{Arg[A]==0,Arg[eps]==0}]. In this case, when the option \
Integrate is set to True, odd factors of k-tensors are dropped and even \
factors are replaced according to Itzykson&Zuber (8-117).\
\>", "Text",
 CellTags->"FeynmanParametrize1"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "FeynmanParametrize1", "]"}]], "Input",
 CellTags->"FeynmanParametrize1",
 CellLabel->"In[1966]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"FeynmanParameterNames", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{"a", ",", "b", ",", "c", ",", "d", ",", "e"}], "}"}]}], ",", 
    RowBox[{"Method", "\[Rule]", "Denominator"}], ",", 
    RowBox[{"Integrate", "\[Rule]", "True"}], ",", 
    RowBox[{"Flatten", "\[Rule]", "True"}], ",", 
    RowBox[{"CompleteSquare", "\[Rule]", "True"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.500970475894513*^9},
 CellTags->"FeynmanParametrize1",
 CellLabel->"Out[1966]="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"FeynmanParametrize1"],

Cell[BoxData[""], "Input",
 CellTags->"FeynmanParametrize1",
 CellLabel->"In[1967]:="]
}, Open  ]],

Cell["FeynmanReduce ***unfinished***", "ObjectName",
 CellTags->"FeynmanReduce"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FeynmanReduce"],

Cell["\<\
***EXPERIMENTAL***
 FeynmanReduce[exp,params] takes a Feynman parameterized expression exp (as \
e.g. generated with FeynmanParametrize1) and a list of Feynman parameters as \
input and attempts to simplify the expression. If no parameters are given, \
Integratedx variables in the expression will be used. Currently, reduction of \
exponentials is implemented. This will work on terms of the form \
E^p1[a,b,c,...]*p2[a,b,c,...], where p1 and p2 are fractions of polynomials \
in the Feynman parameters a,b,c,... If the option Expand is set to True, \
FeynmanReduce will attempt to bring the expression exp into a sum of such \
terms and operate on the terms one by one.\
\>", "Text",
 CellTags->"FeynmanReduce"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "FeynmanReduce", "]"}]], "Input",
 CellTags->"FeynmanReduce",
 CellLabel->"In[1968]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
    RowBox[{"Flatten", "\[Rule]", "True"}], ",", 
    RowBox[{"Expand", "\[Rule]", "True"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.5009704767024*^9},
 CellTags->"FeynmanReduce",
 CellLabel->"Out[1968]="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"FeynmanReduce"],

Cell[BoxData[""], "Input",
 CellTags->"FeynmanReduce",
 CellLabel->"In[1969]:="],

Cell["\<\
Other objects that are considered to be ok or under active development:\
\>", "Text",
 CellTags->"Experimental"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "ChargeConjugationMatrix", ",", " ", "DiracGammaT", ",", " ", 
   "SquareAmplitude"}], "}"}]], "Input",
 CellTags->"Experimental",
 CellLabel->"In[1970]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"C", ",", "DiracGammaT", ",", "SquareAmplitude"}], "}"}], 
  TraditionalForm]], "Output",
 CellChangeTimes->{3.50097047676508*^9},
 CellTags->"Experimental",
 CellLabel->"Out[1970]="]
}, Open  ]],

Cell["Objects that are under consideration for being discarded:", "Text",
 CellTags->"Experimental"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "$OPEWard", ",", "$AchmedRoss", ",", "Contract2", ",", 
   "SpinSumExternalMomentum", ",", "SelectedGraphs"}], "}"}]], "Input",
 CellTags->"Experimental",
 CellLabel->"In[1971]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
   "False", ",", "False", ",", "Contract2", ",", "SpinSumExternalMomentum", 
    ",", "SelectedGraphs"}], "}"}], TraditionalForm]], "Output",
 CellChangeTimes->{3.500970476804213*^9},
 CellTags->"Experimental",
 CellLabel->"Out[1971]="]
}, Open  ]],

Cell[CellGroupData[{

Cell["AbsoluteTime[]-absolutestarttime", "Input",
 CellLabel->"In[1972]:="],

Cell[BoxData[
 FormBox["354.931787`9.001689889153813", TraditionalForm]], "Output",
 CellChangeTimes->{3.500970476840152*^9},
 CellLabel->"Out[1972]="]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Open  ]]
},
ScreenStyleEnvironment->"AllBrackets",
WindowToolbars->"EditBar",
EvaluationCompletionAction->{},
WindowSize->{880, 861},
WindowMargins->{{7, Automatic}, {28, Automatic}},
PrintingStartingPageNumber->1,
PrintingPageRange->{Automatic, Automatic},
PageHeaders->{{Inherited, 
   Cell[
    TextData[
     ValueBox["FileName"]], "Header"], Inherited}, {Inherited, 
   Cell[
    TextData[
     ValueBox["FileName"]], "Header"], Inherited}},
PageFooters->{{Inherited, 
   Cell[
    TextData[
     ValueBox["DateLong"]], "Footer"], Inherited}, {Inherited, 
   Cell[
    TextData[
     ValueBox["DateLong"]], "Footer"], Inherited}},
PrintingOptions->{"FacingPages"->True,
"FirstPageFace"->Left,
"FirstPageFooter"->True,
"FirstPageHeader"->False,
"GraphicsPrintingFormat"->"Automatic",
"Magnification"->1.,
"PaperOrientation"->"Portrait",
"PaperSize"->{597.5625, 842.375},
"PrintCellBrackets"->False,
"PrintMultipleHorizontalPages"->False,
"PrintRegistrationMarks"->True,
"PrintSelectionHighlighting"->False,
"PrintingMargins"->{{54., 54.}, {72., 72.}}},
PrivateNotebookOptions->{"ShortBoxForm"->False},
ShowSelection->True,
TaggingRules:>{
 "MakeContentsNonDefaultOptions" -> {
   AutomaticCellTagging -> FromContent, PageNumbers -> False, SaveOptions -> 
    True, SelectCellStyles -> {"Section", "Subsection"}, 
    TOCCellStyles -> {"Contents", "Index"}}},
CellLabelAutoDelete->True,
Magnification->1.,
FrontEndVersion->"8.0 for Linux x86 (64-bit) (November 7, 2010)",
StyleDefinitions->"HelpBrowser.nb"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{
 "ClickSave"->{
  Cell[1210, 47, 799, 21, 36, NotebookDefault,
   CellTags->"ClickSave"]},
 "Functions"->{
  Cell[2602, 93, 52, 1, 64, "Section",
   CellTags->"Functions"]},
 "Abbreviation"->{
  Cell[4660, 169, 84, 2, 39, "ObjectName",
   PageBreakAbove->True,
   CellTags->"Abbreviation"],
  Cell[4769, 175, 63, 1, 34, "Subsubsection",
   CellTags->"Abbreviation"],
  Cell[4835, 178, 329, 6, 58, "Text",
   CellTags->"Abbreviation"],
  Cell[5167, 186, 1032, 43, 41, "Text",
   CellTags->"Abbreviation"],
  Cell[6236, 234, 60, 1, 34, "Subsubsection",
   CellTags->"Abbreviation"],
  Cell[6321, 239, 264, 8, 26, "Input",
   CellTags->"Abbreviation"],
  Cell[6588, 249, 270, 9, 29, "Output",
   CellTags->"Abbreviation"]},
 "Amplitude"->{
  Cell[6885, 262, 55, 1, 39, "ObjectName",
   CellTags->"Amplitude"],
  Cell[6965, 267, 60, 1, 34, "Subsubsection",
   CellTags->"Amplitude"],
  Cell[7028, 270, 529, 9, 75, "Text",
   CellTags->"Amplitude"],
  Cell[7582, 283, 115, 3, 26, "Input",
   CellTags->"Amplitude"],
  Cell[7700, 288, 372, 11, 26, "Output",
   CellTags->"Amplitude"],
  Cell[8087, 302, 177, 9, 24, "Text",
   CellTags->"Amplitude"],
  Cell[8301, 316, 57, 1, 34, "Subsubsection",
   CellTags->"Amplitude"],
  Cell[8383, 321, 133, 4, 26, "Input",
   CellTags->"Amplitude"],
  Cell[8519, 327, 148, 4, 26, "Output",
   CellTags->"Amplitude"],
  Cell[8682, 334, 93, 1, 24, "Text",
   CellTags->"Amplitude"],
  Cell[8800, 339, 121, 3, 26, "Input",
   CellTags->"Amplitude"],
  Cell[8924, 344, 2260, 96, 30, "Output",
   CellTags->"Amplitude"],
  Cell[11221, 445, 108, 3, 26, "Input",
   CellTags->"Amplitude"],
  Cell[11332, 450, 5175, 190, 74, "Output",
   CellTags->"Amplitude"],
  Cell[16522, 643, 211, 8, 24, "Text",
   CellTags->"Amplitude"],
  Cell[16758, 655, 306, 9, 26, "Input",
   CellTags->"Amplitude"],
  Cell[17067, 666, 13571, 442, 159, "Output",
   CellTags->"Amplitude"]},
 "Amputate"->{
  Cell[30665, 1112, 53, 1, 39, "ObjectName",
   CellTags->"Amputate"],
  Cell[30743, 1117, 59, 1, 34, "Subsubsection",
   CellTags->"Amputate"],
  Cell[30805, 1120, 215, 5, 41, "Text",
   CellTags->"Amputate"],
  Cell[31045, 1129, 114, 3, 26, "Input",
   CellTags->"Amputate"],
  Cell[31162, 1134, 338, 11, 26, "Output",
   CellTags->"Amputate"],
  Cell[31515, 1148, 656, 28, 24, "Text",
   CellTags->"Amputate"],
  Cell[32208, 1181, 56, 1, 34, "Subsubsection",
   CellTags->"Amputate"],
  Cell[32289, 1186, 170, 5, 26, "Input",
   CellTags->"Amputate"],
  Cell[32462, 1193, 472, 17, 28, "Output",
   CellTags->"Amputate"],
  Cell[32971, 1215, 131, 4, 26, "Input",
   CellTags->"Amputate"],
  Cell[33105, 1221, 765, 28, 28, "Output",
   CellTags->"Amputate"]},
 "AnomalousDimension"->{
  Cell[33897, 1253, 73, 1, 39, "ObjectName",
   CellTags->"AnomalousDimension"],
  Cell[33995, 1258, 69, 1, 34, "Subsubsection",
   CellTags->"AnomalousDimension"],
  Cell[34067, 1261, 143, 4, 24, "Text",
   CellTags->"AnomalousDimension"],
  Cell[34235, 1269, 134, 3, 26, "Input",
   CellTags->"AnomalousDimension"],
  Cell[34372, 1274, 296, 9, 26, "Output",
   CellTags->"AnomalousDimension"],
  Cell[34683, 1286, 406, 18, 24, "Text",
   CellTags->"AnomalousDimension"],
  Cell[35126, 1309, 66, 1, 34, "Subsubsection",
   CellTags->"AnomalousDimension"],
  Cell[35195, 1312, 63, 1, 24, "Text",
   CellTags->"AnomalousDimension"],
  Cell[35283, 1317, 206, 5, 26, "Input",
   CellTags->"AnomalousDimension"],
  Cell[35492, 1324, 296, 9, 26, "Output",
   CellTags->"AnomalousDimension"],
  Cell[35803, 1336, 241, 9, 30, "Text",
   CellTags->"AnomalousDimension"],
  Cell[36069, 1349, 133, 3, 26, "Input",
   CellTags->"AnomalousDimension"],
  Cell[36205, 1354, 504, 18, 45, "Output",
   CellTags->"AnomalousDimension"],
  Cell[36724, 1375, 240, 9, 30, "Text",
   CellTags->"AnomalousDimension"],
  Cell[36989, 1388, 132, 3, 26, "Input",
   CellTags->"AnomalousDimension"],
  Cell[37124, 1393, 326, 11, 46, "Output",
   CellTags->"AnomalousDimension"],
  Cell[37465, 1407, 240, 9, 30, "Text",
   CellTags->"AnomalousDimension"],
  Cell[37730, 1420, 132, 3, 26, "Input",
   CellTags->"AnomalousDimension"],
  Cell[37865, 1425, 325, 11, 45, "Output",
   CellTags->"AnomalousDimension"],
  Cell[38205, 1439, 240, 9, 30, "Text",
   CellTags->"AnomalousDimension"],
  Cell[38470, 1452, 132, 3, 26, "Input",
   CellTags->"AnomalousDimension"],
  Cell[38605, 1457, 636, 22, 46, "Output",
   CellTags->"AnomalousDimension"],
  Cell[39256, 1482, 241, 9, 30, "Text",
   CellTags->"AnomalousDimension"],
  Cell[39522, 1495, 133, 3, 26, "Input",
   CellTags->"AnomalousDimension"],
  Cell[39658, 1500, 731, 24, 49, "Output",
   CellTags->"AnomalousDimension"],
  Cell[40404, 1527, 241, 9, 30, "Text",
   CellTags->"AnomalousDimension"],
  Cell[40670, 1540, 133, 3, 26, "Input",
   CellTags->"AnomalousDimension"],
  Cell[40806, 1545, 6650, 219, 305, "Output",
   CellTags->"AnomalousDimension"],
  Cell[47471, 1767, 240, 9, 30, "Text",
   CellTags->"AnomalousDimension"],
  Cell[47736, 1780, 132, 3, 26, "Input",
   CellTags->"AnomalousDimension"],
  Cell[47871, 1785, 4158, 141, 208, "Output",
   CellTags->"AnomalousDimension"],
  Cell[52044, 1929, 240, 9, 30, "Text",
   CellTags->"AnomalousDimension"],
  Cell[52309, 1942, 132, 3, 26, "Input",
   CellTags->"AnomalousDimension"],
  Cell[52444, 1947, 6266, 213, 253, "Output",
   CellTags->"AnomalousDimension"],
  Cell[58725, 2163, 240, 9, 30, "Text",
   CellTags->"AnomalousDimension"],
  Cell[58990, 2176, 132, 3, 26, "Input",
   CellTags->"AnomalousDimension"],
  Cell[59125, 2181, 4954, 167, 244, "Output",
   CellTags->"AnomalousDimension"],
  Cell[64094, 2351, 267, 9, 30, "Text",
   CellTags->"AnomalousDimension"],
  Cell[64386, 2364, 132, 3, 26, "Input",
   CellTags->"AnomalousDimension"],
  Cell[64521, 2369, 4421, 140, 200, "Output",
   CellTags->"AnomalousDimension"],
  Cell[68957, 2512, 326, 10, 30, "Text",
   CellTags->"AnomalousDimension"],
  Cell[69308, 2526, 295, 9, 26, "Input",
   CellTags->"AnomalousDimension"],
  Cell[69606, 2537, 281, 8, 26, "Output",
   CellTags->"AnomalousDimension"]},
 "AntiCommutator"->{
  Cell[69914, 2549, 65, 1, 39, "ObjectName",
   CellTags->"AntiCommutator"],
  Cell[70004, 2554, 65, 1, 34, "Subsubsection",
   CellTags->"AntiCommutator"],
  Cell[70072, 2557, 145, 4, 24, "Text",
   CellTags->"AntiCommutator"],
  Cell[70220, 2563, 599, 24, 24, "Text",
   CellTags->"AntiCommutator"],
  Cell[70856, 2592, 62, 1, 34, "Subsubsection",
   CellTags->"AntiCommutator"],
  Cell[70921, 2595, 200, 9, 24, "Text",
   CellTags->"AntiCommutator"],
  Cell[71124, 2606, 150, 4, 26, "Input",
   CellTags->"AntiCommutator"],
  Cell[71299, 2614, 143, 4, 26, "Input",
   CellTags->"AntiCommutator"],
  Cell[71445, 2620, 205, 6, 26, "Output",
   CellTags->"AntiCommutator"],
  Cell[71687, 2631, 124, 3, 26, "Input",
   CellTags->"AntiCommutator"],
  Cell[71814, 2636, 222, 7, 26, "Output",
   CellTags->"AntiCommutator"],
  Cell[72073, 2648, 265, 8, 26, "Input",
   CellTags->"AntiCommutator"],
  Cell[72341, 2658, 480, 17, 26, "Output",
   CellTags->"AntiCommutator"],
  Cell[72858, 2680, 258, 8, 26, "Input",
   CellTags->"AntiCommutator"],
  Cell[73119, 2690, 362, 12, 26, "Output",
   CellTags->"AntiCommutator"],
  Cell[73496, 2705, 218, 6, 38, "Input",
   CellTags->"AntiCommutator"],
  Cell[73717, 2713, 104, 1, 24, "Text",
   CellTags->"AntiCommutator"],
  Cell[73846, 2718, 177, 5, 26, "Input",
   CellTags->"AntiCommutator"],
  Cell[74026, 2725, 151, 4, 27, "Output",
   CellTags->"AntiCommutator"],
  Cell[74214, 2734, 182, 5, 26, "Input",
   CellTags->"AntiCommutator"],
  Cell[74399, 2741, 232, 7, 26, "Output",
   CellTags->"AntiCommutator"],
  Cell[74668, 2753, 250, 8, 38, "Input",
   CellTags->"AntiCommutator"],
  Cell[74921, 2763, 174, 5, 26, "Output",
   CellTags->"AntiCommutator"],
  Cell[75132, 2773, 252, 7, 38, "Input",
   CellTags->"AntiCommutator"],
  Cell[75387, 2782, 359, 12, 30, "Output",
   CellTags->"AntiCommutator"],
  Cell[75761, 2797, 240, 6, 38, "Input",
   CellTags->"AntiCommutator"],
  Cell[76004, 2805, 176, 5, 26, "Input",
   CellTags->"AntiCommutator"],
  Cell[76183, 2812, 226, 7, 38, "Input",
   CellTags->"AntiCommutator"]},
 "AntiQuarkField"->{
  Cell[76424, 2822, 65, 1, 39, "ObjectName",
   CellTags->"AntiQuarkField"],
  Cell[76514, 2827, 65, 1, 34, "Subsubsection",
   CellTags->"AntiQuarkField"],
  Cell[76582, 2830, 197, 4, 24, "Text",
   CellTags->"AntiQuarkField"],
  Cell[76782, 2836, 313, 14, 24, "Text",
   CellTags->"AntiQuarkField"],
  Cell[77132, 2855, 62, 1, 34, "Subsubsection",
   CellTags->"AntiQuarkField"],
  Cell[77219, 2860, 93, 2, 26, "Input",
   CellTags->"AntiQuarkField"],
  Cell[77315, 2864, 179, 5, 26, "Output",
   CellTags->"AntiQuarkField"]},
 "AntiSymmetrize"->{
  Cell[77521, 2873, 65, 1, 39, "ObjectName",
   CellTags->"AntiSymmetrize"],
  Cell[77611, 2878, 65, 1, 34, "Subsubsection",
   CellTags->"AntiSymmetrize"],
  Cell[77679, 2881, 153, 4, 24, "Text",
   CellTags->"AntiSymmetrize"],
  Cell[77835, 2887, 185, 8, 24, "Text",
   CellTags->"AntiSymmetrize"],
  Cell[78057, 2900, 62, 1, 34, "Subsubsection",
   CellTags->"AntiSymmetrize"],
  Cell[78144, 2905, 246, 8, 26, "Input",
   CellTags->"AntiSymmetrize"],
  Cell[78393, 2915, 361, 12, 44, "Output",
   CellTags->"AntiSymmetrize"],
  Cell[78791, 2932, 266, 8, 26, "Input",
   CellTags->"AntiSymmetrize"],
  Cell[79060, 2942, 696, 21, 44, "Output",
   CellTags->"AntiSymmetrize"]},
 "Anti5"->{
  Cell[79783, 2967, 47, 1, 39, "ObjectName",
   CellTags->"Anti5"],
  Cell[79855, 2972, 56, 1, 34, "Subsubsection",
   CellTags->"Anti5"],
  Cell[79914, 2975, 487, 16, 41, "Text",
   CellTags->"Anti5"],
  Cell[80404, 2993, 163, 7, 24, "Text",
   CellTags->"Anti5"],
  Cell[80570, 3002, 423, 19, 24, "Text",
   CellTags->"Anti5"],
  Cell[81030, 3026, 53, 1, 34, "Subsubsection",
   CellTags->"Anti5"],
  Cell[81108, 3031, 154, 5, 26, "Input",
   CellTags->"Anti5"],
  Cell[81265, 3038, 228, 7, 26, "Output",
   CellTags->"Anti5"],
  Cell[81530, 3050, 102, 3, 26, "Input",
   CellTags->"Anti5"],
  Cell[81635, 3055, 332, 12, 26, "Output",
   CellTags->"Anti5"],
  Cell[82004, 3072, 144, 5, 26, "Input",
   CellTags->"Anti5"],
  Cell[82151, 3079, 307, 11, 26, "Output",
   CellTags->"Anti5"],
  Cell[82495, 3095, 194, 6, 26, "Input",
   CellTags->"Anti5"],
  Cell[82692, 3103, 362, 10, 26, "Output",
   CellTags->"Anti5"],
  Cell[83091, 3118, 125, 4, 26, "Input",
   CellTags->"Anti5"],
  Cell[83219, 3124, 362, 10, 26, "Output",
   CellTags->"Anti5"],
  Cell[83618, 3139, 133, 4, 26, "Input",
   CellTags->"Anti5"],
  Cell[83754, 3145, 362, 10, 26, "Output",
   CellTags->"Anti5"],
  Cell[84153, 3160, 151, 5, 26, "Input",
   CellTags->"Anti5"],
  Cell[84307, 3167, 364, 10, 26, "Output",
   CellTags->"Anti5"],
  Cell[84686, 3180, 351, 14, 24, "Text",
   CellTags->"Anti5"],
  Cell[85062, 3198, 127, 4, 26, "Input",
   CellTags->"Anti5"],
  Cell[85192, 3204, 233, 7, 26, "Output",
   CellTags->"Anti5"],
  Cell[85462, 3216, 102, 3, 26, "Input",
   CellTags->"Anti5"],
  Cell[85567, 3221, 332, 12, 26, "Output",
   CellTags->"Anti5"]},
 "Apart2"->{
  Cell[85926, 3237, 49, 1, 39, "ObjectName",
   CellTags->"Apart2"],
  Cell[86000, 3242, 57, 1, 34, "Subsubsection",
   CellTags->"Apart2"],
  Cell[86060, 3245, 108, 3, 24, "Text",
   CellTags->"Apart2"],
  Cell[86171, 3250, 302, 14, 24, "Text",
   CellTags->"Apart2"],
  Cell[86510, 3269, 54, 1, 34, "Subsubsection",
   CellTags->"Apart2"],
  Cell[86589, 3274, 256, 9, 26, "Input",
   CellTags->"Apart2"],
  Cell[86848, 3285, 1232, 31, 51, "Output",
   CellTags->"Apart2"],
  Cell[88117, 3321, 104, 3, 26, "Input",
   CellTags->"Apart2"],
  Cell[88224, 3326, 761, 27, 56, "Output",
   CellTags->"Apart2"],
  Cell[89022, 3358, 140, 4, 26, "Input",
   CellTags->"Apart2"],
  Cell[89165, 3364, 561, 20, 54, "Output",
   CellTags->"Apart2"]},
 "A0"->{
  Cell[89753, 3388, 64, 2, 39, "ObjectName",
   PageBreakAbove->True,
   CellTags->"A0"],
  Cell[89842, 3394, 53, 1, 34, "Subsubsection",
   CellTags->"A0"],
  Cell[89898, 3397, 291, 11, 27, "Text",
   CellTags->"A0"],
  Cell[90214, 3412, 107, 3, 26, "Input",
   CellTags->"A0"],
  Cell[90324, 3417, 203, 7, 26, "Output",
   CellTags->"A0"],
  Cell[90542, 3427, 437, 24, 24, "Text",
   CellTags->"A0"],
  Cell[91016, 3456, 50, 1, 34, "Subsubsection",
   CellTags->"A0"],
  Cell[91069, 3459, 246, 11, 24, "Text",
   CellTags->"A0"],
  Cell[91340, 3474, 120, 4, 30, "Input",
   CellTags->"A0"],
  Cell[91463, 3480, 301, 11, 29, "Output",
   CellTags->"A0"],
  Cell[91779, 3494, 186, 6, 26, "Input",
   CellTags->"A0"],
  Cell[91990, 3504, 120, 4, 30, "Input",
   CellTags->"A0"],
  Cell[92113, 3510, 366, 12, 29, "Output",
   CellTags->"A0"],
  Cell[92494, 3525, 187, 6, 26, "Input",
   CellTags->"A0"],
  Cell[92684, 3533, 222, 8, 24, "Text",
   CellTags->"A0"],
  Cell[92931, 3545, 96, 3, 26, "Input",
   CellTags->"A0"],
  Cell[93030, 3550, 139, 4, 26, "Output",
   CellTags->"A0"],
  Cell[93206, 3559, 160, 5, 26, "Input",
   CellTags->"A0"],
  Cell[93369, 3566, 139, 4, 26, "Output",
   CellTags->"A0"]},
 "A0ToB0"->{
  Cell[93535, 3574, 72, 2, 39, "ObjectName",
   PageBreakAbove->True,
   CellTags->"A0ToB0"],
  Cell[93632, 3580, 57, 1, 34, "Subsubsection",
   CellTags->"A0ToB0"],
  Cell[93692, 3583, 614, 27, 24, "Text",
   CellTags->"A0ToB0"],
  Cell[94331, 3614, 111, 3, 26, "Input",
   CellTags->"A0ToB0"],
  Cell[94445, 3619, 209, 7, 26, "Output",
   CellTags->"A0ToB0"],
  Cell[94669, 3629, 533, 29, 24, "Text",
   CellTags->"A0ToB0"]},
 "BackgroundGluonVertex"->{
  Cell[95217, 3661, 79, 1, 39, "ObjectName",
   CellTags->"BackgroundGluonVertex"],
  Cell[95321, 3666, 72, 1, 34, "Subsubsection",
   CellTags->"BackgroundGluonVertex"],
  Cell[95396, 3669, 1042, 19, 126, "Text",
   CellTags->"BackgroundGluonVertex"],
  Cell[96463, 3692, 140, 3, 26, "Input",
   CellTags->"BackgroundGluonVertex"],
  Cell[96606, 3697, 362, 10, 26, "Output",
   CellTags->"BackgroundGluonVertex"],
  Cell[96983, 3710, 201, 9, 24, "Text",
   CellTags->"BackgroundGluonVertex"],
  Cell[97221, 3724, 69, 1, 34, "Subsubsection",
   CellTags->"BackgroundGluonVertex"],
  Cell[97315, 3729, 360, 10, 26, "Input",
   CellTags->"BackgroundGluonVertex"],
  Cell[97678, 3741, 1528, 59, 29, "Output",
   CellTags->"BackgroundGluonVertex"],
  Cell[99243, 3805, 436, 12, 26, "Input",
   CellTags->"BackgroundGluonVertex"],
  Cell[99682, 3819, 3888, 143, 52, "Output",
   CellTags->"BackgroundGluonVertex"],
  Cell[103607, 3967, 411, 11, 26, "Input",
   CellTags->"BackgroundGluonVertex"],
  Cell[104021, 3980, 1603, 61, 46, "Output",
   CellTags->"BackgroundGluonVertex"],
  Cell[105661, 4046, 487, 13, 26, "Input",
   CellTags->"BackgroundGluonVertex"],
  Cell[106151, 4061, 3996, 146, 90, "Output",
   CellTags->"BackgroundGluonVertex"]},
 "Bracket"->{
  Cell[110174, 4211, 51, 1, 39, "ObjectName",
   CellTags->"Bracket"],
  Cell[110250, 4216, 58, 1, 34, "Subsubsection",
   CellTags->"Bracket"],
  Cell[110311, 4219, 72, 1, 24, "Text",
   CellTags->"Bracket"],
  Cell[110386, 4222, 181, 9, 24, "Text",
   CellTags->"Bracket"]},
 "BReduce"->{
  Cell[110582, 4234, 74, 2, 39, "ObjectName",
   PageBreakAbove->True,
   CellTags->"BReduce"],
  Cell[110681, 4240, 58, 1, 34, "Subsubsection",
   CellTags->"BReduce"],
  Cell[110742, 4243, 148, 4, 24, "Text",
   CellTags->"BReduce"],
  Cell[110893, 4249, 534, 29, 24, "Text",
   CellTags->"BReduce"],
  Cell[111464, 4283, 55, 1, 34, "Subsubsection",
   CellTags->"BReduce"],
  Cell[111522, 4286, 251, 11, 24, "Text",
   CellTags->"BReduce"],
  Cell[111798, 4301, 134, 4, 26, "Input",
   CellTags->"BReduce"],
  Cell[111935, 4307, 220, 7, 26, "Output",
   CellTags->"BReduce"],
  Cell[112596, 4335, 94, 3, 24, "Text",
   CellTags->"BReduce"],
  Cell[112715, 4342, 183, 5, 26, "Input",
   CellTags->"BReduce"],
  Cell[112901, 4349, 302, 11, 45, "Output",
   CellTags->"BReduce"]},
 "B0"->{
  Cell[113230, 4364, 41, 1, 39, "ObjectName",
   CellTags->"B0"],
  Cell[113296, 4369, 53, 1, 34, "Subsubsection",
   CellTags->"B0"],
  Cell[113352, 4372, 390, 9, 41, "Text",
   CellTags->"B0"],
  Cell[113767, 4385, 102, 3, 26, "Input",
   CellTags->"B0"],
  Cell[113872, 4390, 324, 10, 26, "Output",
   CellTags->"B0"],
  Cell[114211, 4403, 443, 24, 24, "Text",
   CellTags->"B0"],
  Cell[114691, 4432, 50, 1, 34, "Subsubsection",
   CellTags->"B0"],
  Cell[114766, 4437, 234, 8, 30, "Input",
   CellTags->"B0"],
  Cell[115003, 4447, 358, 13, 29, "Output",
   CellTags->"B0"],
  Cell[115376, 4463, 237, 7, 26, "Input",
   CellTags->"B0"],
  Cell[115638, 4474, 154, 5, 30, "Input",
   CellTags->"B0"],
  Cell[115795, 4481, 287, 9, 29, "Output",
   CellTags->"B0"],
  Cell[116119, 4495, 179, 6, 30, "Input",
   CellTags->"B0"],
  Cell[116301, 4503, 289, 9, 29, "Output",
   CellTags->"B0"],
  Cell[116627, 4517, 177, 6, 26, "Input",
   CellTags->"B0"],
  Cell[116807, 4525, 262, 8, 29, "Output",
   CellTags->"B0"]},
 "B0Real"->{
  Cell[117096, 4537, 72, 2, 39, "ObjectName",
   PageBreakAbove->True,
   CellTags->"B0Real"],
  Cell[117193, 4543, 57, 1, 34, "Subsubsection",
   CellTags->"B0Real"],
  Cell[117253, 4546, 186, 4, 24, "Text",
   CellTags->"B0Real"],
  Cell[117442, 4552, 153, 8, 24, "Text",
   CellTags->"B0Real"],
  Cell[117632, 4565, 54, 1, 34, "Subsubsection",
   CellTags->"B0Real"],
  Cell[117689, 4568, 83, 1, 24, "Text",
   CellTags->"B0Real"],
  Cell[117797, 4573, 133, 4, 26, "Input",
   CellTags->"B0Real"],
  Cell[117933, 4579, 242, 8, 26, "Output",
   CellTags->"B0Real"],
  Cell[118212, 4592, 102, 3, 26, "Input",
   CellTags->"B0Real"],
  Cell[118317, 4597, 324, 10, 26, "Output",
   CellTags->"B0Real"],
  Cell[118656, 4610, 92, 3, 24, "Text",
   CellTags->"B0Real"],
  Cell[118773, 4617, 181, 5, 26, "Input",
   CellTags->"B0Real"],
  Cell[118957, 4624, 242, 8, 26, "Output",
   CellTags->"B0Real"]},
 "B0Unique"->{
  Cell[119226, 4636, 76, 2, 39, "ObjectName",
   PageBreakAbove->True,
   CellTags->"B0Unique"],
  Cell[119327, 4642, 59, 1, 34, "Subsubsection",
   CellTags->"B0Unique"],
  Cell[119389, 4645, 184, 4, 24, "Text",
   CellTags->"B0Unique"],
  Cell[119576, 4651, 155, 8, 24, "Text",
   CellTags->"B0Unique"],
  Cell[119768, 4664, 56, 1, 34, "Subsubsection",
   CellTags->"B0Unique"],
  Cell[119827, 4667, 73, 1, 24, "Text",
   CellTags->"B0Unique"],
  Cell[119925, 4672, 135, 4, 26, "Input",
   CellTags->"B0Unique"],
  Cell[120063, 4678, 244, 8, 26, "Output",
   CellTags->"B0Unique"],
  Cell[120344, 4691, 104, 3, 26, "Input",
   CellTags->"B0Unique"],
  Cell[120451, 4696, 328, 10, 26, "Output",
   CellTags->"B0Unique"],
  Cell[120794, 4709, 88, 3, 24, "Text",
   CellTags->"B0Unique"],
  Cell[120907, 4716, 186, 5, 26, "Input",
   CellTags->"B0Unique"],
  Cell[121096, 4723, 218, 7, 26, "Output",
   CellTags->"B0Unique"]},
 "B00"->{
  Cell[121341, 4734, 43, 1, 41, "ObjectName",
   CellTags->"B00"],
  Cell[121409, 4739, 54, 1, 36, "Subsubsection",
   CellTags->"B00"],
  Cell[121466, 4742, 290, 8, 41, "Text",
   CellTags->"B00"],
  Cell[121781, 4754, 104, 3, 26, "Input",
   CellTags->"B00"],
  Cell[121888, 4759, 204, 7, 26, "Output",
   CellTags->"B00"],
  Cell[122107, 4769, 346, 19, 25, "Text",
   CellTags->"B00"],
  Cell[122490, 4793, 51, 1, 36, "Subsubsection",
   CellTags->"B00"],
  Cell[122544, 4796, 202, 7, 26, "Text",
   CellTags->"B00"],
  Cell[122771, 4807, 97, 3, 26, "Input",
   CellTags->"B00"],
  Cell[122871, 4812, 198, 7, 28, "Output",
   CellTags->"B00"],
  Cell[123106, 4824, 211, 7, 30, "Input",
   CellTags->"B00"],
  Cell[123320, 4833, 2717, 91, 95, "Output",
   CellTags->"B00"],
  Cell[126074, 4929, 211, 7, 30, "Input",
   CellTags->"B00"],
  Cell[126288, 4938, 1496, 55, 46, "Output",
   CellTags->"B00"],
  Cell[127821, 4998, 261, 8, 30, "Input",
   CellTags->"B00"],
  Cell[128085, 5008, 560, 23, 29, "Output",
   CellTags->"B00"],
  Cell[128682, 5036, 181, 6, 30, "Input",
   CellTags->"B00"],
  Cell[128866, 5044, 656, 23, 46, "Output",
   CellTags->"B00"],
  Cell[129559, 5072, 248, 8, 30, "Input",
   CellTags->"B00"],
  Cell[129810, 5082, 685, 24, 46, "Output",
   CellTags->"B00"]},
 "B1"->{
  Cell[130522, 5110, 41, 1, 41, "ObjectName",
   CellTags->"B1"],
  Cell[130588, 5115, 53, 1, 36, "Subsubsection",
   CellTags->"B1"],
  Cell[130644, 5118, 230, 7, 24, "Text",
   CellTags->"B1"],
  Cell[130899, 5129, 102, 3, 26, "Input",
   CellTags->"B1"],
  Cell[131004, 5134, 203, 7, 26, "Output",
   CellTags->"B1"],
  Cell[131222, 5144, 559, 29, 25, "Text",
   CellTags->"B1"],
  Cell[131818, 5178, 50, 1, 34, "Subsubsection",
   CellTags->"B1"],
  Cell[131893, 5183, 209, 7, 30, "Input",
   CellTags->"B1"],
  Cell[132105, 5192, 1195, 43, 52, "Output",
   CellTags->"B1"],
  Cell[133337, 5240, 259, 8, 30, "Input",
   CellTags->"B1"],
  Cell[133599, 5250, 358, 13, 29, "Output",
   CellTags->"B1"],
  Cell[133972, 5266, 187, 6, 26, "Input",
   CellTags->"B1"],
  Cell[134184, 5276, 209, 7, 30, "Input",
   CellTags->"B1"],
  Cell[134396, 5285, 432, 16, 44, "Output",
   CellTags->"B1"],
  Cell[134865, 5306, 179, 6, 30, "Input",
   CellTags->"B1"],
  Cell[135047, 5314, 445, 15, 44, "Output",
   CellTags->"B1"],
  Cell[135529, 5334, 154, 5, 30, "Input",
   CellTags->"B1"],
  Cell[135686, 5341, 447, 15, 44, "Output",
   CellTags->"B1"],
  Cell[136170, 5361, 238, 7, 30, "Input",
   CellTags->"B1"],
  Cell[136411, 5370, 833, 26, 51, "Output",
   CellTags->"B1"],
  Cell[137281, 5401, 279, 8, 30, "Input",
   CellTags->"B1"],
  Cell[137563, 5411, 461, 15, 44, "Output",
   CellTags->"B1"]},
 "B11"->{
  Cell[138051, 5430, 43, 1, 57, "ObjectName",
   CellTags->"B11"],
  Cell[138119, 5435, 54, 1, 52, "Subsubsection",
   CellTags->"B11"],
  Cell[138176, 5438, 495, 17, 43, "Text",
   CellTags->"B11"],
  Cell[138696, 5459, 104, 3, 26, "Input",
   CellTags->"B11"],
  Cell[138803, 5464, 206, 7, 26, "Output",
   CellTags->"B11"],
  Cell[139024, 5474, 441, 24, 41, "Text",
   CellTags->"B11"],
  Cell[139502, 5503, 51, 1, 52, "Subsubsection",
   CellTags->"B11"],
  Cell[139556, 5506, 215, 7, 26, "Text",
   CellTags->"B11"],
  Cell[139796, 5517, 97, 3, 26, "Input",
   CellTags->"B11"],
  Cell[139896, 5522, 198, 7, 28, "Output",
   CellTags->"B11"],
  Cell[140131, 5534, 212, 7, 30, "Input",
   CellTags->"B11"],
  Cell[140346, 5543, 2889, 98, 154, "Output",
   CellTags->"B11"],
  Cell[143272, 5646, 212, 7, 30, "Input",
   CellTags->"B11"],
  Cell[143487, 5655, 1250, 48, 56, "Output",
   CellTags->"B11"],
  Cell[144774, 5708, 171, 5, 26, "Input",
   CellTags->"B11"],
  Cell[144948, 5715, 206, 7, 26, "Output",
   CellTags->"B11"],
  Cell[145191, 5727, 212, 7, 30, "Input",
   CellTags->"B11"],
  Cell[145406, 5736, 561, 23, 29, "Output",
   CellTags->"B11"],
  Cell[146004, 5764, 170, 5, 26, "Input",
   CellTags->"B11"],
  Cell[146177, 5771, 205, 7, 26, "Output",
   CellTags->"B11"],
  Cell[146419, 5783, 182, 6, 30, "Input",
   CellTags->"B11"],
  Cell[146604, 5791, 311, 10, 44, "Output",
   CellTags->"B11"],
  Cell[146952, 5806, 249, 8, 30, "Input",
   CellTags->"B11"],
  Cell[147204, 5816, 337, 11, 44, "Output",
   CellTags->"B11"]},
 "CA"->{
  Cell[147568, 5831, 41, 1, 39, "ObjectName",
   CellTags->"CA"],
  Cell[147634, 5836, 53, 1, 34, "Subsubsection",
   CellTags->"CA"],
  Cell[147690, 5839, 198, 9, 24, "Text",
   CellTags->"CA"],
  Cell[147891, 5850, 274, 14, 24, "Text",
   CellTags->"CA"],
  Cell[148202, 5869, 50, 1, 34, "Subsubsection",
   CellTags->"CA"],
  Cell[148277, 5874, 70, 2, 26, "Input",
   CellTags->"CA"],
  Cell[148350, 5878, 162, 5, 26, "Output",
   CellTags->"CA"],
  Cell[148549, 5888, 173, 5, 26, "Input",
   CellTags->"CA"],
  Cell[148725, 5895, 140, 4, 27, "Output",
   CellTags->"CA"],
  Cell[148902, 5904, 72, 2, 26, "Input",
   CellTags->"CA"],
  Cell[148977, 5908, 142, 4, 27, "Output",
   CellTags->"CA"]},
 "Calc"->{
  Cell[149146, 5916, 45, 1, 39, "ObjectName",
   CellTags->"Calc"],
  Cell[149216, 5921, 55, 1, 34, "Subsubsection",
   CellTags->"Calc"],
  Cell[149274, 5924, 233, 5, 41, "Text",
   CellTags->"Calc"],
  Cell[149510, 5931, 407, 19, 24, "Text",
   CellTags->"Calc"],
  Cell[149954, 5955, 61, 1, 33, "Subsubsection",
   CellTags->"Calc"],
  Cell[150018, 5958, 346, 13, 27, "Text",
   CellTags->"Calc"],
  Cell[150389, 5975, 284, 9, 26, "Input",
   CellTags->"Calc"],
  Cell[150676, 5986, 189, 6, 26, "Output",
   CellTags->"Calc"],
  Cell[150880, 5995, 213, 9, 26, "Text",
   CellTags->"Calc"],
  Cell[151118, 6008, 258, 8, 26, "Input",
   CellTags->"Calc"],
  Cell[151379, 6018, 394, 16, 26, "Output",
   CellTags->"Calc"],
  Cell[151810, 6039, 328, 11, 26, "Input",
   CellTags->"Calc"],
  Cell[152141, 6052, 466, 17, 28, "Output",
   CellTags->"Calc"],
  Cell[152644, 6074, 101, 3, 26, "Input",
   CellTags->"Calc"],
  Cell[152748, 6079, 569, 23, 28, "Output",
   CellTags->"Calc"],
  Cell[153354, 6107, 215, 6, 26, "Input",
   CellTags->"Calc"],
  Cell[153572, 6115, 1029, 45, 30, "Output",
   CellTags->"Calc"],
  Cell[154638, 6165, 382, 13, 26, "Input",
   CellTags->"Calc"],
  Cell[155023, 6180, 142, 4, 26, "Output",
   CellTags->"Calc"]},
 "CalcColorFactor"->{
  Cell[155192, 6188, 67, 1, 39, "ObjectName",
   CellTags->"CalcColorFactor"],
  Cell[155284, 6193, 66, 1, 34, "Subsubsection",
   CellTags->"CalcColorFactor"],
  Cell[155353, 6196, 470, 9, 108, "Text",
   CellTags->"CalcColorFactor"],
  Cell[155826, 6207, 195, 9, 24, "Text",
   CellTags->"CalcColorFactor"]},
 "Collecting"->{
  Cell[161057, 6400, 411, 18, 24, "Text",
   CellTags->"Collecting"],
  Cell[261119, 10342, 57, 1, 39, "ObjectName",
   CellTags->"Collecting"],
  Cell[261201, 10347, 61, 1, 34, "Subsubsection",
   CellTags->"Collecting"],
  Cell[261265, 10350, 207, 5, 41, "Text",
   CellTags->"Collecting"],
  Cell[261475, 10357, 411, 18, 24, "Text",
   CellTags->"Collecting"]},
 "CalculateCounterTerm"->{
  Cell[161483, 6421, 94, 1, 39, "ObjectName",
   CellTags->"CalculateCounterTerm"],
  Cell[161602, 6426, 71, 1, 34, "Subsubsection",
   CellTags->"CalculateCounterTerm"],
  Cell[161676, 6429, 110, 1, 24, "Text",
   CellTags->"CalculateCounterTerm"],
  Cell[161823, 6435, 77, 1, 33, "Subsubsection",
   CellTags->"CalculateCounterTerm"],
  Cell[161925, 6440, 123, 3, 26, "Input",
   CellTags->"CalculateCounterTerm"],
  Cell[162051, 6445, 235, 4, 41, "Print",
   CellTags->{"CalculateCounterTerm", "Info3500973745-4982097"}]},
 "Info3500973745-4982097"->{
  Cell[162051, 6445, 235, 4, 41, "Print",
   CellTags->{"CalculateCounterTerm", "Info3500973745-4982097"}]},
 "CancelQP"->{
  Cell[162313, 6453, 53, 1, 39, "ObjectName",
   CellTags->"CancelQP"],
  Cell[162391, 6458, 59, 1, 34, "Subsubsection",
   CellTags->"CancelQP"],
  Cell[162453, 6461, 314, 13, 26, "Text",
   CellTags->"CancelQP"],
  Cell[162804, 6479, 65, 1, 33, "Subsubsection",
   CellTags->"CancelQP"],
  Cell[162894, 6484, 549, 19, 26, "Input",
   CellTags->"CancelQP"],
  Cell[163446, 6505, 709, 25, 49, "Output",
   CellTags->"CancelQP"],
  Cell[164192, 6535, 182, 5, 26, "Input",
   CellTags->"CancelQP"],
  Cell[164377, 6542, 2227, 74, 105, "Output",
   CellTags->"CancelQP"],
  Cell[166641, 6621, 183, 5, 26, "Input",
   CellTags->"CancelQP"],
  Cell[166827, 6628, 6462, 210, 254, "Output",
   CellTags->"CancelQP"],
  Cell[173326, 6843, 555, 18, 26, "Input",
   CellTags->"CancelQP"],
  Cell[173884, 6863, 930, 34, 50, "Output",
   CellTags->"CancelQP"],
  Cell[174851, 6902, 182, 5, 26, "Input",
   CellTags->"CancelQP"],
  Cell[175036, 6909, 6513, 226, 157, "Output",
   CellTags->"CancelQP"],
  Cell[181586, 7140, 183, 5, 26, "Input",
   CellTags->"CancelQP"],
  Cell[181772, 7147, 3778, 127, 105, "Output",
   CellTags->"CancelQP"],
  Cell[185565, 7277, 106, 3, 26, "Input",
   CellTags->"CancelQP"]},
 "Cases2"->{
  Cell[185686, 7283, 49, 1, 39, "ObjectName",
   CellTags->"Cases2"],
  Cell[185760, 7288, 57, 1, 34, "Subsubsection",
   CellTags->"Cases2"],
  Cell[185820, 7291, 293, 6, 41, "Text",
   CellTags->"Cases2"],
  Cell[186150, 7302, 63, 1, 33, "Subsubsection",
   CellTags->"Cases2"],
  Cell[186238, 7307, 305, 10, 30, "Input",
   CellTags->"Cases2"],
  Cell[186546, 7319, 319, 10, 26, "Output",
   CellTags->"Cases2"],
  Cell[186902, 7334, 312, 10, 26, "Input",
   CellTags->"Cases2"],
  Cell[187217, 7346, 323, 10, 26, "Output",
   CellTags->"Cases2"],
  Cell[187577, 7361, 427, 14, 26, "Input",
   CellTags->"Cases2"],
  Cell[188007, 7377, 397, 12, 26, "Output",
   CellTags->"Cases2"],
  Cell[188441, 7394, 327, 10, 26, "Input",
   CellTags->"Cases2"],
  Cell[188771, 7406, 505, 18, 28, "Output",
   CellTags->"Cases2"]},
 "CF"->{
  Cell[189303, 7428, 41, 1, 39, "ObjectName",
   CellTags->"CF"],
  Cell[189369, 7433, 53, 1, 34, "Subsubsection",
   CellTags->"CF"],
  Cell[189425, 7436, 287, 13, 24, "Text",
   CellTags->"CF"],
  Cell[189715, 7451, 274, 14, 24, "Text",
   CellTags->"CF"],
  Cell[190026, 7470, 50, 1, 34, "Subsubsection",
   CellTags->"CF"],
  Cell[190101, 7475, 70, 2, 26, "Input",
   CellTags->"CF"],
  Cell[190174, 7479, 164, 5, 26, "Output",
   CellTags->"CF"],
  Cell[190375, 7489, 178, 5, 26, "Input",
   CellTags->"CF"],
  Cell[190556, 7496, 235, 8, 46, "Output",
   CellTags->"CF"],
  Cell[191069, 7520, 72, 2, 26, "Input",
   CellTags->"CF"],
  Cell[191144, 7524, 140, 4, 27, "Output",
   CellTags->"CF"],
  Cell[191321, 7533, 233, 7, 26, "Input",
   CellTags->"CF"],
  Cell[191557, 7542, 219, 7, 26, "Output",
   CellTags->"CF"]},
 "ChangeDimension"->{
  Cell[191803, 7553, 67, 1, 39, "ObjectName",
   CellTags->"ChangeDimension"],
  Cell[191895, 7558, 66, 1, 34, "Subsubsection",
   CellTags->"ChangeDimension"],
  Cell[191964, 7561, 210, 5, 41, "Text",
   CellTags->"ChangeDimension"],
  Cell[192177, 7568, 519, 24, 24, "Text",
   CellTags->"ChangeDimension"],
  Cell[192733, 7597, 63, 1, 34, "Subsubsection",
   CellTags->"ChangeDimension"],
  Cell[192799, 7600, 219, 5, 41, "Text",
   CellTags->"ChangeDimension"],
  Cell[193043, 7609, 212, 6, 26, "Input",
   CellTags->"ChangeDimension"],
  Cell[193258, 7617, 240, 8, 28, "Output",
   CellTags->"ChangeDimension"],
  Cell[193535, 7630, 151, 4, 26, "Input",
   CellTags->"ChangeDimension"],
  Cell[193689, 7636, 240, 8, 28, "Output",
   CellTags->"ChangeDimension"],
  Cell[193966, 7649, 116, 3, 26, "Input",
   CellTags->"ChangeDimension"],
  Cell[194085, 7654, 314, 9, 38, "Output",
   CellTags->"ChangeDimension"],
  Cell[194414, 7666, 119, 3, 24, "Text",
   CellTags->"ChangeDimension"],
  Cell[194558, 7673, 201, 6, 26, "Input",
   CellTags->"ChangeDimension"],
  Cell[194762, 7681, 266, 7, 38, "Output",
   CellTags->"ChangeDimension"],
  Cell[195043, 7691, 105, 1, 24, "Text",
   CellTags->"ChangeDimension"],
  Cell[195173, 7696, 482, 15, 26, "Input",
   CellTags->"ChangeDimension"],
  Cell[195658, 7713, 613, 19, 26, "Output",
   CellTags->"ChangeDimension"],
  Cell[196308, 7737, 153, 4, 26, "Input",
   CellTags->"ChangeDimension"],
  Cell[196464, 7743, 797, 30, 26, "Output",
   CellTags->"ChangeDimension"],
  Cell[197298, 7778, 222, 6, 26, "Input",
   CellTags->"ChangeDimension"],
  Cell[197523, 7786, 797, 30, 26, "Output",
   CellTags->"ChangeDimension"],
  Cell[198335, 7819, 132, 3, 24, "Text",
   CellTags->"ChangeDimension"],
  Cell[198470, 7824, 208, 6, 26, "Input",
   CellTags->"ChangeDimension"],
  Cell[198681, 7832, 219, 5, 41, "Text",
   CellTags->"ChangeDimension"],
  Cell[198903, 7839, 201, 6, 26, "Input",
   CellTags->"ChangeDimension"],
  Cell[199129, 7849, 220, 6, 26, "Input",
   CellTags->"ChangeDimension"],
  Cell[199352, 7857, 531, 22, 26, "Output",
   CellTags->"ChangeDimension"],
  Cell[199920, 7884, 182, 5, 26, "Input",
   CellTags->"ChangeDimension"],
  Cell[200105, 7891, 531, 22, 26, "Output",
   CellTags->"ChangeDimension"],
  Cell[200673, 7918, 176, 5, 30, "Input",
   CellTags->"ChangeDimension"],
  Cell[200852, 7925, 343, 11, 26, "Output",
   CellTags->"ChangeDimension"],
  Cell[201232, 7941, 141, 4, 30, "Input",
   CellTags->"ChangeDimension"],
  Cell[201376, 7947, 174, 5, 26, "Output",
   CellTags->"ChangeDimension"],
  Cell[201565, 7955, 201, 6, 26, "Input",
   CellTags->"ChangeDimension"],
  Cell[201769, 7963, 157, 5, 26, "Input",
   CellTags->"ChangeDimension"]},
 "ChargeConjugationMatrix"->{
  Cell[201941, 7971, 83, 1, 39, "ObjectName",
   CellTags->"ChargeConjugationMatrix"],
  Cell[202049, 7976, 74, 1, 34, "Subsubsection",
   CellTags->"ChargeConjugationMatrix"],
  Cell[202126, 7979, 176, 6, 24, "Text",
   CellTags->"ChargeConjugationMatrix"],
  Cell[202305, 7987, 248, 9, 24, "Text",
   CellTags->"ChargeConjugationMatrix"],
  Cell[202590, 8001, 71, 1, 34, "Subsubsection",
   CellTags->"ChargeConjugationMatrix"],
  Cell[202686, 8006, 214, 5, 26, "Input",
   CellTags->"ChargeConjugationMatrix"],
  Cell[202903, 8013, 348, 12, 26, "Output",
   CellTags->"ChargeConjugationMatrix"],
  Cell[203288, 8030, 120, 3, 26, "Input",
   CellTags->"ChargeConjugationMatrix"],
  Cell[203411, 8035, 299, 10, 30, "Output",
   CellTags->"ChargeConjugationMatrix"],
  Cell[203747, 8050, 209, 5, 26, "Input",
   CellTags->"ChargeConjugationMatrix"],
  Cell[203959, 8057, 270, 8, 26, "Output",
   CellTags->"ChargeConjugationMatrix"],
  Cell[204266, 8070, 120, 3, 26, "Input",
   CellTags->"ChargeConjugationMatrix"],
  Cell[204389, 8075, 234, 7, 28, "Output",
   CellTags->"ChargeConjugationMatrix"],
  Cell[204660, 8087, 134, 3, 26, "Input",
   CellTags->"ChargeConjugationMatrix"],
  Cell[204797, 8092, 179, 5, 26, "Output",
   CellTags->"ChargeConjugationMatrix"]},
 "ChargeConjugationMatrixInv"->{
  Cell[205003, 8101, 89, 1, 39, "ObjectName",
   CellTags->"ChargeConjugationMatrixInv"],
  Cell[205117, 8106, 77, 1, 34, "Subsubsection",
   CellTags->"ChargeConjugationMatrixInv"],
  Cell[205197, 8109, 195, 4, 24, "Text",
   CellTags->"ChargeConjugationMatrixInv"],
  Cell[205395, 8115, 242, 9, 24, "Text",
   CellTags->"ChargeConjugationMatrixInv"],
  Cell[205674, 8129, 74, 1, 34, "Subsubsection",
   CellTags->"ChargeConjugationMatrixInv"],
  Cell[205773, 8134, 217, 5, 26, "Input",
   CellTags->"ChargeConjugationMatrixInv"],
  Cell[205993, 8141, 351, 12, 26, "Output",
   CellTags->"ChargeConjugationMatrixInv"],
  Cell[206381, 8158, 123, 3, 26, "Input",
   CellTags->"ChargeConjugationMatrixInv"],
  Cell[206507, 8163, 300, 10, 30, "Output",
   CellTags->"ChargeConjugationMatrixInv"],
  Cell[206844, 8178, 212, 5, 26, "Input",
   CellTags->"ChargeConjugationMatrixInv"],
  Cell[207059, 8185, 273, 8, 26, "Output",
   CellTags->"ChargeConjugationMatrixInv"],
  Cell[207369, 8198, 123, 3, 26, "Input",
   CellTags->"ChargeConjugationMatrixInv"],
  Cell[207495, 8203, 237, 7, 28, "Output",
   CellTags->"ChargeConjugationMatrixInv"],
  Cell[207769, 8215, 165, 4, 26, "Input",
   CellTags->"ChargeConjugationMatrixInv"],
  Cell[207937, 8221, 231, 7, 26, "Output",
   CellTags->"ChargeConjugationMatrixInv"],
  Cell[208205, 8233, 119, 3, 26, "Input",
   CellTags->"ChargeConjugationMatrixInv"],
  Cell[208327, 8238, 206, 6, 26, "Output",
   CellTags->"ChargeConjugationMatrixInv"],
  Cell[208570, 8249, 140, 3, 26, "Input",
   CellTags->"ChargeConjugationMatrixInv"],
  Cell[208713, 8254, 182, 5, 26, "Output",
   CellTags->"ChargeConjugationMatrixInv"]},
 "CheckContext"->{
  Cell[208922, 8263, 61, 1, 39, "ObjectName",
   CellTags->"CheckContext"],
  Cell[209008, 8268, 63, 1, 34, "Subsubsection",
   CellTags->"CheckContext"],
  Cell[209074, 8271, 164, 4, 24, "Text",
   CellTags->"CheckContext"],
  Cell[209241, 8277, 320, 14, 24, "Text",
   CellTags->"CheckContext"],
  Cell[209598, 8296, 60, 1, 34, "Subsubsection",
   CellTags->"CheckContext"],
  Cell[209661, 8299, 142, 4, 24, "Text",
   CellTags->"CheckContext"],
  Cell[209828, 8307, 138, 3, 26, "Input",
   CellTags->"CheckContext"],
  Cell[209969, 8312, 156, 4, 26, "Output",
   CellTags->"CheckContext"],
  Cell[210140, 8319, 87, 1, 24, "Text",
   CellTags->"CheckContext"],
  Cell[210252, 8324, 92, 2, 26, "Input",
   CellTags->"CheckContext"],
  Cell[210347, 8328, 163, 4, 26, "Output",
   CellTags->"CheckContext"],
  Cell[210547, 8337, 138, 3, 26, "Input",
   CellTags->"CheckContext"],
  Cell[210688, 8342, 153, 4, 26, "Output",
   CellTags->"CheckContext"]},
 "CheckDB"->{
  Cell[210868, 8350, 51, 1, 39, "ObjectName",
   CellTags->"CheckDB"],
  Cell[210944, 8355, 106, 3, 34, "Subsubsection",
   CellTags->"CheckDB"],
  Cell[212159, 8379, 113, 3, 26, "Input",
   CellTags->"CheckDB"],
  Cell[212275, 8384, 512, 15, 26, "Output",
   CellTags->"CheckDB"],
  Cell[212824, 8404, 116, 3, 26, "Input",
   CellTags->"CheckDB"],
  Cell[212943, 8409, 175, 5, 26, "Output",
   CellTags->"CheckDB"],
  Cell[214405, 8468, 383, 13, 26, "Input",
   CellTags->"CheckDB"],
  Cell[214791, 8483, 192, 6, 26, "Output",
   CellTags->"CheckDB"],
  Cell[215083, 8496, 303, 9, 26, "Input",
   CellTags->"CheckDB"],
  Cell[215389, 8507, 158, 4, 26, "Output",
   CellTags->"CheckDB"],
  Cell[215562, 8514, 268, 8, 26, "Input",
   CellTags->"CheckDB"]},
 "ChiralityProjector"->{
  Cell[215845, 8525, 73, 1, 39, "ObjectName",
   CellTags->"ChiralityProjector"],
  Cell[215943, 8530, 69, 1, 34, "Subsubsection",
   CellTags->"ChiralityProjector"],
  Cell[216015, 8533, 499, 19, 27, "Text",
   CellTags->"ChiralityProjector"],
  Cell[216517, 8554, 409, 19, 24, "Text",
   CellTags->"ChiralityProjector"],
  Cell[216963, 8578, 66, 1, 34, "Subsubsection",
   CellTags->"ChiralityProjector"],
  Cell[217054, 8583, 262, 8, 26, "Input",
   CellTags->"ChiralityProjector"],
  Cell[217319, 8593, 268, 8, 29, "Output",
   CellTags->"ChiralityProjector"],
  Cell[217624, 8606, 114, 3, 26, "Input",
   CellTags->"ChiralityProjector"],
  Cell[217741, 8611, 269, 8, 29, "Output",
   CellTags->"ChiralityProjector"],
  Cell[218047, 8624, 196, 5, 26, "Input",
   CellTags->"ChiralityProjector"],
  Cell[218246, 8631, 422, 15, 46, "Output",
   CellTags->"ChiralityProjector"]},
 "Chisholm"->{
  Cell[218695, 8650, 53, 1, 39, "ObjectName",
   CellTags->"Chisholm"],
  Cell[218773, 8655, 59, 1, 34, "Subsubsection",
   CellTags->"Chisholm"],
  Cell[218835, 8658, 142, 4, 24, "Text",
   CellTags->"Chisholm"],
  Cell[218980, 8664, 316, 14, 24, "Text",
   CellTags->"Chisholm"],
  Cell[219333, 8683, 56, 1, 34, "Subsubsection",
   CellTags->"Chisholm"],
  Cell[219414, 8688, 206, 6, 26, "Input",
   CellTags->"Chisholm"],
  Cell[219623, 8696, 2062, 79, 26, "Output",
   CellTags->"Chisholm"],
  Cell[221722, 8780, 183, 5, 26, "Input",
   CellTags->"Chisholm"],
  Cell[221908, 8787, 280, 8, 26, "Output",
   CellTags->"Chisholm"],
  Cell[222225, 8800, 139, 4, 26, "Input",
   CellTags->"Chisholm"],
  Cell[222367, 8806, 1415, 55, 26, "Output",
   CellTags->"Chisholm"],
  Cell[223797, 8864, 77, 1, 24, "Text",
   CellTags->"Chisholm"],
  Cell[223899, 8869, 130, 4, 26, "Input",
   CellTags->"Chisholm"],
  Cell[224032, 8875, 147, 4, 26, "Output",
   CellTags->"Chisholm"],
  Cell[224216, 8884, 139, 4, 26, "Input",
   CellTags->"Chisholm"],
  Cell[224358, 8890, 1416, 55, 26, "Output",
   CellTags->"Chisholm"],
  Cell[225811, 8950, 130, 4, 26, "Input",
   CellTags->"Chisholm"],
  Cell[225944, 8956, 147, 4, 26, "Output",
   CellTags->"Chisholm"],
  Cell[226128, 8965, 169, 5, 26, "Input",
   CellTags->"Chisholm"],
  Cell[226300, 8972, 632, 23, 26, "Output",
   CellTags->"Chisholm"],
  Cell[226969, 9000, 110, 3, 26, "Input",
   CellTags->"Chisholm"],
  Cell[227082, 9005, 1481, 58, 28, "Output",
   CellTags->"Chisholm"],
  Cell[228600, 9068, 232, 7, 26, "Input",
   CellTags->"Chisholm"],
  Cell[228835, 9077, 433, 11, 26, "Output",
   CellTags->"Chisholm"],
  Cell[229305, 9093, 139, 4, 26, "Input",
   CellTags->"Chisholm"],
  Cell[229447, 9099, 11586, 440, 127, "Output",
   CellTags->"Chisholm"],
  Cell[241070, 9544, 139, 4, 26, "Input",
   CellTags->"Chisholm"],
  Cell[241212, 9550, 11586, 440, 127, "Output",
   CellTags->"Chisholm"],
  Cell[252813, 9993, 85, 1, 24, "Text",
   CellTags->"Chisholm"],
  Cell[252923, 9998, 130, 4, 26, "Input",
   CellTags->"Chisholm"],
  Cell[253056, 10004, 167, 5, 26, "Output",
   CellTags->"Chisholm"],
  Cell[253260, 10014, 130, 4, 26, "Input",
   CellTags->"Chisholm"],
  Cell[253393, 10020, 169, 5, 26, "Output",
   CellTags->"Chisholm"],
  Cell[253577, 10028, 192, 6, 26, "Input",
   CellTags->"Chisholm"]},
 "ChisholmSpinor"->{
  Cell[253784, 10037, 65, 1, 39, "ObjectName",
   CellTags->"ChisholmSpinor"],
  Cell[253874, 10042, 65, 1, 34, "Subsubsection",
   CellTags->"ChisholmSpinor"],
  Cell[253942, 10045, 296, 6, 41, "Text",
   CellTags->"ChisholmSpinor"],
  Cell[254241, 10053, 304, 14, 24, "Text",
   CellTags->"ChisholmSpinor"],
  Cell[254582, 10072, 62, 1, 34, "Subsubsection",
   CellTags->"ChisholmSpinor"],
  Cell[254669, 10077, 312, 9, 26, "Input",
   CellTags->"ChisholmSpinor"],
  Cell[254984, 10088, 469, 16, 26, "Output",
   CellTags->"ChisholmSpinor"],
  Cell[255490, 10109, 117, 3, 26, "Input",
   CellTags->"ChisholmSpinor"],
  Cell[255610, 10114, 2728, 100, 94, "Output",
   CellTags->"ChisholmSpinor"]},
 "ClearScalarProducts"->{
  Cell[258365, 10218, 75, 1, 39, "ObjectName",
   CellTags->"ClearScalarProducts"],
  Cell[258465, 10223, 70, 1, 34, "Subsubsection",
   CellTags->"ClearScalarProducts"],
  Cell[258538, 10226, 145, 4, 24, "Text",
   CellTags->"ClearScalarProducts"],
  Cell[258686, 10232, 484, 23, 24, "Text",
   CellTags->"ClearScalarProducts"],
  Cell[259207, 10260, 67, 1, 34, "Subsubsection",
   CellTags->"ClearScalarProducts"],
  Cell[259299, 10265, 195, 6, 26, "Input",
   CellTags->"ClearScalarProducts"],
  Cell[259497, 10273, 183, 5, 26, "Output",
   CellTags->"ClearScalarProducts"],
  Cell[259717, 10283, 211, 6, 26, "Input",
   CellTags->"ClearScalarProducts"],
  Cell[259931, 10291, 183, 5, 26, "Output",
   CellTags->"ClearScalarProducts"],
  Cell[260129, 10299, 104, 2, 26, "Input",
   CellTags->"ClearScalarProducts"],
  Cell[260258, 10305, 211, 6, 26, "Input",
   CellTags->"ClearScalarProducts"],
  Cell[260472, 10313, 217, 7, 28, "Output",
   CellTags->"ClearScalarProducts"],
  Cell[260726, 10325, 148, 4, 26, "Input",
   CellTags->"ClearScalarProducts"],
  Cell[260877, 10331, 215, 7, 28, "Output",
   CellTags->"ClearScalarProducts"]},
 "Collect2"->{
  Cell[261901, 10378, 53, 1, 39, "ObjectName",
   CellTags->"Collect2"],
  Cell[261979, 10383, 59, 1, 34, "Subsubsection",
   CellTags->"Collect2"],
  Cell[262041, 10386, 613, 10, 75, "Text",
   CellTags->"Collect2"],
  Cell[262679, 10400, 115, 3, 26, "Input",
   CellTags->"Collect2"],
  Cell[262797, 10405, 440, 12, 26, "Output",
   CellTags->"Collect2"],
  Cell[263252, 10420, 176, 9, 24, "Text",
   CellTags->"Collect2"],
  Cell[263465, 10434, 56, 1, 34, "Subsubsection",
   CellTags->"Collect2"],
  Cell[263546, 10439, 440, 14, 47, "Input",
   CellTags->"Collect2"],
  Cell[263989, 10455, 493, 17, 43, "Output",
   CellTags->"Collect2"],
  Cell[264519, 10477, 185, 5, 26, "Input",
   CellTags->"Collect2"],
  Cell[264707, 10484, 441, 15, 44, "Output",
   CellTags->"Collect2"],
  Cell[265185, 10504, 186, 5, 26, "Input",
   CellTags->"Collect2"],
  Cell[265374, 10511, 495, 17, 43, "Output",
   CellTags->"Collect2"],
  Cell[265906, 10533, 525, 18, 30, "Input",
   CellTags->"Collect2"],
  Cell[266434, 10553, 581, 19, 26, "Output",
   CellTags->"Collect2"],
  Cell[267052, 10577, 320, 10, 30, "Input",
   CellTags->"Collect2"],
  Cell[267375, 10589, 862, 32, 26, "Output",
   CellTags->"Collect2"],
  Cell[268274, 10626, 112, 3, 26, "Input",
   CellTags->"Collect2"],
  Cell[268389, 10631, 867, 28, 26, "Output",
   CellTags->"Collect2"],
  Cell[269271, 10662, 130, 4, 26, "Input",
   CellTags->"Collect2"],
  Cell[269426, 10670, 305, 9, 30, "Input",
   CellTags->"Collect2"],
  Cell[269734, 10681, 979, 32, 29, "Output",
   CellTags->"Collect2"],
  Cell[270750, 10718, 149, 4, 26, "Input",
   CellTags->"Collect2"],
  Cell[270902, 10724, 715, 22, 29, "Output",
   CellTags->"Collect2"]},
 "Collect3"->{
  Cell[271644, 10750, 53, 1, 39, "ObjectName",
   CellTags->"Collect3"],
  Cell[271722, 10755, 59, 1, 34, "Subsubsection",
   CellTags->"Collect3"],
  Cell[271784, 10758, 719, 23, 58, "Text",
   CellTags->"Collect3"],
  Cell[272528, 10785, 115, 3, 26, "Input",
   CellTags->"Collect3"],
  Cell[272646, 10790, 275, 9, 26, "Output",
   CellTags->"Collect3"],
  Cell[272936, 10802, 286, 14, 24, "Text",
   CellTags->"Collect3"],
  Cell[273259, 10821, 56, 1, 34, "Subsubsection",
   CellTags->"Collect3"],
  Cell[273340, 10826, 525, 18, 30, "Input",
   CellTags->"Collect3"],
  Cell[273868, 10846, 902, 18, 98, "Message",
   CellTags->"Collect3"],
  Cell[274773, 10866, 902, 18, 98, "Message",
   CellTags->"Collect3"],
  Cell[275678, 10886, 164, 5, 26, "Output",
   CellTags->"Collect3"],
  Cell[275879, 10896, 289, 9, 30, "Input",
   CellTags->"Collect3"],
  Cell[276171, 10907, 649, 14, 22, "Message",
   CellTags->"Collect3"],
  Cell[276823, 10923, 164, 5, 26, "Output",
   CellTags->"Collect3"]},
 "Combine"->{
  Cell[277014, 10932, 51, 1, 39, "ObjectName",
   CellTags->"Combine"],
  Cell[277090, 10937, 58, 1, 34, "Subsubsection",
   CellTags->"Combine"],
  Cell[277151, 10940, 315, 6, 41, "Text",
   CellTags->"Combine"],
  Cell[277491, 10950, 113, 3, 26, "Input",
   CellTags->"Combine"],
  Cell[277607, 10955, 213, 7, 26, "Output",
   CellTags->"Combine"],
  Cell[277835, 10965, 175, 9, 24, "Text",
   CellTags->"Combine"],
  Cell[278047, 10979, 55, 1, 34, "Subsubsection",
   CellTags->"Combine"],
  Cell[278127, 10984, 283, 10, 50, "Input",
   CellTags->"Combine"],
  Cell[278413, 10996, 346, 12, 47, "Output",
   CellTags->"Combine"],
  Cell[278774, 11011, 122, 3, 24, "Text",
   CellTags->"Combine"],
  Cell[278921, 11018, 284, 10, 50, "Input",
   CellTags->"Combine"],
  Cell[279208, 11030, 345, 11, 47, "Output",
   CellTags->"Combine"],
  Cell[279568, 11044, 175, 4, 24, "Text",
   CellTags->"Combine"],
  Cell[279768, 11052, 354, 12, 50, "Input",
   CellTags->"Combine"],
  Cell[280125, 11066, 344, 11, 47, "Output",
   CellTags->"Combine"]},
 "CombineGraphs"->{
  Cell[280496, 11081, 63, 1, 39, "ObjectName",
   CellTags->"CombineGraphs"],
  Cell[280584, 11086, 64, 1, 34, "Subsubsection",
   CellTags->"CombineGraphs"],
  Cell[280651, 11089, 86, 1, 24, "Text",
   CellTags->"CombineGraphs"],
  Cell[280740, 11092, 185, 8, 24, "Text",
   CellTags->"CombineGraphs"]},
 "Commutator"->{
  Cell[280940, 11103, 57, 1, 39, "ObjectName",
   CellTags->"Commutator"],
  Cell[281022, 11108, 61, 1, 34, "Subsubsection",
   CellTags->"Commutator"],
  Cell[281086, 11111, 138, 4, 24, "Text",
   CellTags->"Commutator"],
  Cell[281227, 11117, 607, 24, 24, "Text",
   CellTags->"Commutator"],
  Cell[281871, 11146, 58, 1, 34, "Subsubsection",
   CellTags->"Commutator"],
  Cell[281932, 11149, 167, 4, 26, "Input",
   CellTags->"Commutator"],
  Cell[282124, 11157, 136, 4, 26, "Input",
   CellTags->"Commutator"],
  Cell[282263, 11163, 226, 6, 26, "Output",
   CellTags->"Commutator"],
  Cell[282526, 11174, 121, 3, 26, "Input",
   CellTags->"Commutator"],
  Cell[282650, 11179, 219, 7, 26, "Output",
   CellTags->"Commutator"],
  Cell[282906, 11191, 247, 8, 26, "Input",
   CellTags->"Commutator"],
  Cell[283156, 11201, 423, 13, 26, "Output",
   CellTags->"Commutator"],
  Cell[283594, 11217, 169, 4, 26, "Input",
   CellTags->"Commutator"],
  Cell[283766, 11223, 68, 1, 24, "Text",
   CellTags->"Commutator"],
  Cell[283837, 11226, 228, 6, 26, "Input",
   CellTags->"Commutator"],
  Cell[284090, 11236, 463, 15, 26, "Input",
   CellTags->"Commutator"],
  Cell[284556, 11253, 771, 20, 26, "Output",
   CellTags->"Commutator"],
  Cell[285364, 11278, 114, 3, 26, "Input",
   CellTags->"Commutator"],
  Cell[285481, 11283, 148, 4, 26, "Output",
   CellTags->"Commutator"],
  Cell[285644, 11290, 113, 3, 26, "Input",
   CellTags->"Commutator"],
  Cell[285760, 11295, 159, 4, 26, "Input",
   CellTags->"Commutator"]},
 "CommutatorExplicit"->{
  Cell[285934, 11302, 73, 1, 39, "ObjectName",
   CellTags->"CommutatorExplicit"],
  Cell[286032, 11307, 69, 1, 34, "Subsubsection",
   CellTags->"CommutatorExplicit"],
  Cell[286104, 11310, 157, 4, 24, "Text",
   CellTags->"CommutatorExplicit"],
  Cell[286264, 11316, 296, 14, 24, "Text",
   CellTags->"CommutatorExplicit"],
  Cell[286597, 11335, 66, 1, 34, "Subsubsection",
   CellTags->"CommutatorExplicit"],
  Cell[286666, 11338, 175, 4, 26, "Input",
   CellTags->"CommutatorExplicit"],
  Cell[286866, 11346, 144, 4, 26, "Input",
   CellTags->"CommutatorExplicit"],
  Cell[287013, 11352, 234, 6, 26, "Output",
   CellTags->"CommutatorExplicit"],
  Cell[287284, 11363, 129, 3, 26, "Input",
   CellTags->"CommutatorExplicit"],
  Cell[287416, 11368, 227, 7, 26, "Output",
   CellTags->"CommutatorExplicit"],
  Cell[287680, 11380, 196, 6, 26, "Input",
   CellTags->"CommutatorExplicit"],
  Cell[287879, 11388, 325, 11, 26, "Output",
   CellTags->"CommutatorExplicit"],
  Cell[288241, 11404, 129, 3, 26, "Input",
   CellTags->"CommutatorExplicit"],
  Cell[288373, 11409, 431, 15, 26, "Output",
   CellTags->"CommutatorExplicit"],
  Cell[288841, 11429, 167, 5, 26, "Input",
   CellTags->"CommutatorExplicit"],
  Cell[289011, 11436, 431, 13, 26, "Output",
   CellTags->"CommutatorExplicit"],
  Cell[289457, 11452, 177, 4, 26, "Input",
   CellTags->"CommutatorExplicit"]},
 "CompleteSquare"->{
  Cell[289649, 11459, 65, 1, 39, "ObjectName",
   CellTags->"CompleteSquare"],
  Cell[289739, 11464, 65, 1, 34, "Subsubsection",
   CellTags->"CompleteSquare"],
  Cell[289807, 11467, 1107, 82, 41, "Text",
   CellTags->"CompleteSquare"],
  Cell[290951, 11554, 62, 1, 34, "Subsubsection",
   CellTags->"CompleteSquare"],
  Cell[291734, 11589, 170, 5, 26, "Input",
   CellTags->"CompleteSquare"],
  Cell[291907, 11596, 727, 28, 48, "Output",
   CellTags->"CompleteSquare"],
  Cell[292671, 11629, 180, 6, 26, "Input",
   CellTags->"CompleteSquare"],
  Cell[292854, 11637, 152, 4, 26, "Output",
   CellTags->"CompleteSquare"],
  Cell[293043, 11646, 155, 4, 26, "Input",
   CellTags->"CompleteSquare"],
  Cell[293201, 11652, 557, 21, 46, "Output",
   CellTags->"CompleteSquare"],
  Cell[293773, 11676, 137, 4, 26, "Input",
   CellTags->"CompleteSquare"]},
 "ComplexConjugate"->{
  Cell[293925, 11683, 69, 1, 39, "ObjectName",
   CellTags->"ComplexConjugate"],
  Cell[294019, 11688, 67, 1, 34, "Subsubsection",
   CellTags->"ComplexConjugate"],
  Cell[294089, 11691, 414, 7, 58, "Text",
   CellTags->"ComplexConjugate"],
  Cell[294506, 11700, 449, 19, 24, "Text",
   CellTags->"ComplexConjugate"],
  Cell[294992, 11724, 64, 1, 34, "Subsubsection",
   CellTags->"ComplexConjugate"],
  Cell[295081, 11729, 196, 5, 26, "Input",
   CellTags->"ComplexConjugate"],
  Cell[295280, 11736, 207, 6, 26, "Output",
   CellTags->"ComplexConjugate"],
  Cell[295524, 11747, 121, 3, 26, "Input",
   CellTags->"ComplexConjugate"],
  Cell[295648, 11752, 281, 7, 38, "Output",
   CellTags->"ComplexConjugate"],
  Cell[295966, 11764, 152, 4, 26, "Input",
   CellTags->"ComplexConjugate"],
  Cell[296121, 11770, 291, 8, 26, "Output",
   CellTags->"ComplexConjugate"],
  Cell[296449, 11783, 125, 3, 26, "Input",
   CellTags->"ComplexConjugate"],
  Cell[296577, 11788, 469, 17, 26, "Output",
   CellTags->"ComplexConjugate"],
  Cell[297083, 11810, 182, 5, 26, "Input",
   CellTags->"ComplexConjugate"],
  Cell[297268, 11817, 573, 23, 28, "Output",
   CellTags->"ComplexConjugate"],
  Cell[297878, 11845, 125, 3, 26, "Input",
   CellTags->"ComplexConjugate"],
  Cell[298006, 11850, 575, 23, 28, "Output",
   CellTags->"ComplexConjugate"],
  Cell[298618, 11878, 190, 5, 26, "Input",
   CellTags->"ComplexConjugate"],
  Cell[298811, 11885, 424, 18, 28, "Output",
   CellTags->"ComplexConjugate"],
  Cell[299272, 11908, 270, 7, 26, "Input",
   CellTags->"ComplexConjugate"],
  Cell[299545, 11917, 199, 5, 38, "Output",
   CellTags->"ComplexConjugate"],
  Cell[299781, 11927, 273, 8, 26, "Input",
   CellTags->"ComplexConjugate"],
  Cell[300057, 11937, 377, 11, 26, "Output",
   CellTags->"ComplexConjugate"],
  Cell[300471, 11953, 125, 3, 26, "Input",
   CellTags->"ComplexConjugate"],
  Cell[300599, 11958, 473, 16, 26, "Output",
   CellTags->"ComplexConjugate"],
  Cell[301087, 11977, 166, 4, 24, "Text",
   CellTags->"ComplexConjugate"]},
 "Contract"->{
  Cell[301268, 11984, 53, 1, 39, "ObjectName",
   CellTags->"Contract"],
  Cell[301346, 11989, 59, 1, 34, "Subsubsection",
   CellTags->"Contract"],
  Cell[301408, 11992, 411, 7, 58, "Text",
   CellTags->"Contract"],
  Cell[301844, 12003, 115, 3, 26, "Input",
   CellTags->"Contract"],
  Cell[301962, 12008, 654, 16, 47, "Output",
   CellTags->"Contract"],
  Cell[302772, 12032, 423, 19, 24, "Text",
   CellTags->"Contract"],
  Cell[303232, 12056, 56, 1, 34, "Subsubsection",
   CellTags->"Contract"],
  Cell[303313, 12061, 233, 7, 26, "Input",
   CellTags->"Contract"],
  Cell[303549, 12070, 249, 8, 29, "Output",
   CellTags->"Contract"],
  Cell[303835, 12083, 109, 3, 26, "Input",
   CellTags->"Contract"],
  Cell[303947, 12088, 280, 11, 28, "Output",
   CellTags->"Contract"],
  Cell[304264, 12104, 199, 6, 26, "Input",
   CellTags->"Contract"],
  Cell[304466, 12112, 306, 11, 29, "Output",
   CellTags->"Contract"],
  Cell[304809, 12128, 109, 3, 26, "Input",
   CellTags->"Contract"],
  Cell[304921, 12133, 265, 9, 28, "Output",
   CellTags->"Contract"],
  Cell[305223, 12147, 144, 4, 26, "Input",
   CellTags->"Contract"],
  Cell[305370, 12153, 196, 6, 26, "Output",
   CellTags->"Contract"],
  Cell[305581, 12162, 83, 1, 24, "Text",
   CellTags->"Contract"],
  Cell[305689, 12167, 109, 3, 26, "Input",
   CellTags->"Contract"],
  Cell[305801, 12172, 146, 4, 26, "Output",
   CellTags->"Contract"],
  Cell[305962, 12179, 208, 5, 41, "Text",
   CellTags->"Contract"],
  Cell[306195, 12188, 231, 7, 26, "Input",
   CellTags->"Contract"],
  Cell[306429, 12197, 275, 9, 26, "Output",
   CellTags->"Contract"],
  Cell[306741, 12211, 109, 3, 26, "Input",
   CellTags->"Contract"],
  Cell[306853, 12216, 146, 4, 27, "Output",
   CellTags->"Contract"],
  Cell[307036, 12225, 231, 7, 26, "Input",
   CellTags->"Contract"],
  Cell[307270, 12234, 277, 9, 26, "Output",
   CellTags->"Contract"],
  Cell[307584, 12248, 227, 7, 26, "Input",
   CellTags->"Contract"],
  Cell[307814, 12257, 225, 7, 29, "Output",
   CellTags->"Contract"],
  Cell[308076, 12269, 114, 3, 26, "Input",
   CellTags->"Contract"],
  Cell[308193, 12274, 248, 9, 28, "Output",
   CellTags->"Contract"],
  Cell[308478, 12288, 277, 9, 26, "Input",
   CellTags->"Contract"],
  Cell[308758, 12299, 469, 19, 30, "Output",
   CellTags->"Contract"],
  Cell[309264, 12323, 109, 3, 26, "Input",
   CellTags->"Contract"],
  Cell[309376, 12328, 617, 25, 28, "Output",
   CellTags->"Contract"],
  Cell[310030, 12358, 271, 8, 26, "Input",
   CellTags->"Contract"],
  Cell[310304, 12368, 596, 24, 28, "Output",
   CellTags->"Contract"],
  Cell[310937, 12397, 109, 3, 26, "Input",
   CellTags->"Contract"],
  Cell[311049, 12402, 481, 20, 26, "Output",
   CellTags->"Contract"],
  Cell[311567, 12427, 313, 9, 26, "Input",
   CellTags->"Contract"],
  Cell[311883, 12438, 964, 41, 26, "Output",
   CellTags->"Contract"],
  Cell[312884, 12484, 109, 3, 26, "Input",
   CellTags->"Contract"],
  Cell[312996, 12489, 246, 8, 26, "Output",
   CellTags->"Contract"],
  Cell[313279, 12502, 426, 13, 26, "Input",
   CellTags->"Contract"],
  Cell[313708, 12517, 345, 10, 26, "Output",
   CellTags->"Contract"],
  Cell[314090, 12532, 139, 4, 26, "Input",
   CellTags->"Contract"],
  Cell[314232, 12538, 393, 13, 26, "Output",
   CellTags->"Contract"],
  Cell[314640, 12554, 194, 6, 26, "Input",
   CellTags->"Contract"]},
 "Contract1"->{
  Cell[314849, 12563, 55, 1, 39, "ObjectName",
   CellTags->"Contract1"],
  Cell[314929, 12568, 60, 1, 34, "Subsubsection",
   CellTags->"Contract1"],
  Cell[314992, 12571, 459, 8, 99, "Text",
   CellTags->"Contract1"],
  Cell[315454, 12581, 389, 18, 24, "Text",
   CellTags->"Contract1"],
  Cell[315880, 12604, 57, 1, 34, "Subsubsection",
   CellTags->"Contract1"],
  Cell[315962, 12609, 349, 12, 26, "Input",
   CellTags->"Contract1"],
  Cell[316314, 12623, 251, 9, 28, "Output",
   CellTags->"Contract1"],
  Cell[316602, 12637, 349, 12, 26, "Input",
   CellTags->"Contract1"],
  Cell[316954, 12651, 520, 21, 28, "Output",
   CellTags->"Contract1"],
  Cell[317511, 12677, 269, 9, 26, "Input",
   CellTags->"Contract1"],
  Cell[317783, 12688, 147, 4, 26, "Output",
   CellTags->"Contract1"],
  Cell[317967, 12697, 269, 9, 26, "Input",
   CellTags->"Contract1"],
  Cell[318239, 12708, 338, 12, 26, "Output",
   CellTags->"Contract1"]},
 "Convolute"->{
  Cell[318604, 12724, 55, 1, 39, "ObjectName",
   CellTags->"Convolute"],
  Cell[318684, 12729, 60, 1, 34, "Subsubsection",
   CellTags->"Convolute"],
  Cell[318747, 12732, 1363, 49, 65, "Text",
   CellTags->"Convolute"],
  Cell[320135, 12785, 117, 3, 26, "Input",
   CellTags->"Convolute"],
  Cell[320255, 12790, 442, 14, 26, "Output",
   CellTags->"Convolute"],
  Cell[320712, 12807, 332, 14, 24, "Text",
   CellTags->"Convolute"],
  Cell[321081, 12826, 57, 1, 34, "Subsubsection",
   CellTags->"Convolute"],
  Cell[321163, 12831, 134, 4, 26, "Input",
   CellTags->"Convolute"],
  Cell[321300, 12837, 195, 6, 26, "Output",
   CellTags->"Convolute"],
  Cell[321532, 12848, 134, 4, 26, "Input",
   CellTags->"Convolute"],
  Cell[321669, 12854, 219, 7, 26, "Output",
   CellTags->"Convolute"],
  Cell[321925, 12866, 134, 4, 26, "Input",
   CellTags->"Convolute"],
  Cell[322062, 12872, 172, 5, 26, "Output",
   CellTags->"Convolute"],
  Cell[322271, 12882, 181, 6, 49, "Input",
   CellTags->"Convolute"],
  Cell[322455, 12890, 257, 8, 26, "Output",
   CellTags->"Convolute"],
  Cell[322749, 12903, 227, 7, 49, "Input",
   CellTags->"Convolute"],
  Cell[322979, 12912, 259, 8, 26, "Output",
   CellTags->"Convolute"],
  Cell[323275, 12925, 181, 6, 49, "Input",
   CellTags->"Convolute"],
  Cell[323459, 12933, 349, 12, 26, "Output",
   CellTags->"Convolute"],
  Cell[323845, 12950, 228, 8, 49, "Input",
   CellTags->"Convolute"],
  Cell[324076, 12960, 578, 20, 44, "Output",
   CellTags->"Convolute"],
  Cell[324691, 12985, 190, 6, 26, "Input",
   CellTags->"Convolute"],
  Cell[324884, 12993, 363, 12, 26, "Output",
   CellTags->"Convolute"],
  Cell[325284, 13010, 216, 7, 26, "Input",
   CellTags->"Convolute"],
  Cell[325503, 13019, 317, 11, 26, "Output",
   CellTags->"Convolute"],
  Cell[325857, 13035, 237, 8, 49, "Input",
   CellTags->"Convolute"],
  Cell[326097, 13045, 433, 14, 26, "Output",
   CellTags->"Convolute"],
  Cell[326567, 13064, 263, 9, 49, "Input",
   CellTags->"Convolute"],
  Cell[326833, 13075, 661, 22, 26, "Output",
   CellTags->"Convolute"],
  Cell[327531, 13102, 239, 8, 49, "Input",
   CellTags->"Convolute"],
  Cell[327773, 13112, 759, 25, 44, "Output",
   CellTags->"Convolute"],
  Cell[328569, 13142, 190, 6, 26, "Input",
   CellTags->"Convolute"],
  Cell[328762, 13150, 289, 10, 26, "Output",
   CellTags->"Convolute"],
  Cell[329088, 13165, 190, 6, 26, "Input",
   CellTags->"Convolute"],
  Cell[329281, 13173, 358, 12, 26, "Output",
   CellTags->"Convolute"],
  Cell[329676, 13190, 260, 9, 49, "Input",
   CellTags->"Convolute"],
  Cell[329939, 13201, 493, 17, 49, "Output",
   CellTags->"Convolute"],
  Cell[330469, 13223, 165, 5, 26, "Input",
   CellTags->"Convolute"],
  Cell[330637, 13230, 271, 10, 44, "Output",
   CellTags->"Convolute"],
  Cell[330945, 13245, 195, 6, 26, "Input",
   CellTags->"Convolute"],
  Cell[331143, 13253, 281, 10, 44, "Output",
   CellTags->"Convolute"],
  Cell[331461, 13268, 212, 7, 49, "Input",
   CellTags->"Convolute"],
  Cell[331676, 13277, 460, 15, 44, "Output",
   CellTags->"Convolute"],
  Cell[332173, 13297, 213, 7, 49, "Input",
   CellTags->"Convolute"],
  Cell[332389, 13306, 358, 13, 44, "Output",
   CellTags->"Convolute"],
  Cell[332784, 13324, 237, 8, 49, "Input",
   CellTags->"Convolute"],
  Cell[333024, 13334, 594, 19, 44, "Output",
   CellTags->"Convolute"],
  Cell[333655, 13358, 213, 7, 49, "Input",
   CellTags->"Convolute"],
  Cell[333871, 13367, 453, 14, 44, "Output",
   CellTags->"Convolute"],
  Cell[334361, 13386, 190, 6, 26, "Input",
   CellTags->"Convolute"],
  Cell[334554, 13394, 287, 10, 26, "Output",
   CellTags->"Convolute"],
  Cell[334878, 13409, 165, 5, 26, "Input",
   CellTags->"Convolute"],
  Cell[335046, 13416, 230, 7, 26, "Output",
   CellTags->"Convolute"],
  Cell[335313, 13428, 286, 10, 49, "Input",
   CellTags->"Convolute"],
  Cell[335602, 13440, 832, 30, 49, "Output",
   CellTags->"Convolute"],
  Cell[336471, 13475, 295, 10, 49, "Input",
   CellTags->"Convolute"],
  Cell[336769, 13487, 1235, 42, 66, "Output",
   CellTags->"Convolute"],
  Cell[340559, 13646, 244, 8, 49, "Input",
   CellTags->"Convolute"],
  Cell[340806, 13656, 507, 17, 44, "Output",
   CellTags->"Convolute"]},
 "ConvoluteTable"->{
  Cell[338031, 13533, 66, 1, 39, "ObjectName",
   CellTags->"ConvoluteTable"],
  Cell[338122, 13538, 65, 1, 34, "Subsubsection",
   CellTags->"ConvoluteTable"],
  Cell[338190, 13541, 152, 4, 24, "Text",
   CellTags->"ConvoluteTable"],
  Cell[338367, 13549, 122, 3, 26, "Input",
   CellTags->"ConvoluteTable"],
  Cell[338492, 13554, 449, 14, 26, "Output",
   CellTags->"ConvoluteTable"],
  Cell[338956, 13571, 322, 14, 24, "Text",
   CellTags->"ConvoluteTable"],
  Cell[339315, 13590, 62, 1, 34, "Subsubsection",
   CellTags->"ConvoluteTable"],
  Cell[339402, 13595, 144, 4, 26, "Input",
   CellTags->"ConvoluteTable"],
  Cell[339549, 13601, 203, 6, 26, "Output",
   CellTags->"ConvoluteTable"],
  Cell[339789, 13612, 144, 4, 26, "Input",
   CellTags->"ConvoluteTable"],
  Cell[339936, 13618, 225, 7, 26, "Output",
   CellTags->"ConvoluteTable"],
  Cell[340198, 13630, 144, 4, 26, "Input",
   CellTags->"ConvoluteTable"],
  Cell[340345, 13636, 177, 5, 26, "Output",
   CellTags->"ConvoluteTable"]},
 "CovariantFieldDerivative"->{
  Cell[341340, 13677, 85, 1, 39, "ObjectName",
   CellTags->"CovariantFieldDerivative"],
  Cell[341450, 13682, 75, 1, 34, "Subsubsection",
   CellTags->"CovariantFieldDerivative"],
  Cell[341528, 13685, 372, 6, 58, "Text",
   CellTags->"CovariantFieldDerivative"],
  Cell[341903, 13693, 460, 19, 24, "Text",
   CellTags->"CovariantFieldDerivative"],
  Cell[342400, 13717, 72, 1, 34, "Subsubsection",
   CellTags->"CovariantFieldDerivative"],
  Cell[342497, 13722, 329, 9, 26, "Input",
   CellTags->"CovariantFieldDerivative"],
  Cell[342829, 13733, 585, 22, 29, "Output",
   CellTags->"CovariantFieldDerivative"]},
 "CounterT"->{
  Cell[343441, 13759, 53, 1, 39, "ObjectName",
   CellTags->"CounterT"],
  Cell[343519, 13764, 59, 1, 34, "Subsubsection",
   CellTags->"CounterT"],
  Cell[343581, 13767, 146, 4, 24, "Text",
   CellTags->"CounterT"],
  Cell[343730, 13773, 450, 19, 24, "Text",
   CellTags->"CounterT"],
  Cell[344217, 13797, 56, 1, 34, "Subsubsection",
   CellTags->"CounterT"],
  Cell[344298, 13802, 284, 6, 26, "Input",
   CellTags->"CounterT"],
  Cell[344585, 13810, 3169, 107, 65, "Output",
   CellTags->"CounterT"]},
 "CounterTerm"->{
  Cell[347781, 13921, 59, 1, 39, "ObjectName",
   CellTags->"CounterTerm"],
  Cell[347865, 13926, 62, 1, 34, "Subsubsection",
   CellTags->"CounterTerm"],
  Cell[347930, 13929, 216, 5, 41, "Text",
   CellTags->"CounterTerm"],
  Cell[348149, 13936, 578, 24, 24, "Text",
   CellTags->"CounterTerm"],
  Cell[348764, 13965, 59, 1, 34, "Subsubsection",
   CellTags->"CounterTerm"],
  Cell[348848, 13970, 116, 3, 26, "Input",
   CellTags->"CounterTerm"],
  Cell[348967, 13975, 1676, 53, 65, "Output",
   CellTags->"CounterTerm"]},
 "CouplingConstant"->{
  Cell[350670, 14032, 69, 1, 39, "ObjectName",
   CellTags->"CouplingConstant"],
  Cell[350764, 14037, 67, 1, 34, "Subsubsection",
   CellTags->"CouplingConstant"],
  Cell[350834, 14040, 657, 11, 100, "Text",
   CellTags->"CouplingConstant"],
  Cell[351494, 14053, 318, 14, 24, "Text",
   CellTags->"CouplingConstant"]},
 "CovariantD"->{
  Cell[351827, 14070, 57, 1, 39, "ObjectName",
   CellTags->"CovariantD"],
  Cell[351909, 14075, 61, 1, 34, "Subsubsection",
   CellTags->"CovariantD"],
  Cell[351973, 14078, 1210, 18, 218, "Text",
   CellTags->"CovariantD"],
  Cell[353208, 14100, 119, 3, 26, "Input",
   CellTags->"CovariantD"],
  Cell[353330, 14105, 768, 18, 87, "Output",
   CellTags->"CovariantD"],
  Cell[354113, 14126, 185, 4, 24, "Text",
   CellTags->"CovariantD"],
  Cell[354301, 14132, 455, 19, 24, "Text",
   CellTags->"CovariantD"],
  Cell[354793, 14156, 58, 1, 34, "Subsubsection",
   CellTags->"CovariantD"],
  Cell[354876, 14161, 117, 3, 26, "Input",
   CellTags->"CovariantD"],
  Cell[354996, 14166, 174, 5, 29, "Output",
   CellTags->"CovariantD"],
  Cell[355207, 14176, 150, 4, 26, "Input",
   CellTags->"CovariantD"],
  Cell[355360, 14182, 203, 6, 32, "Output",
   CellTags->"CovariantD"],
  Cell[355600, 14193, 180, 5, 26, "Input",
   CellTags->"CovariantD"],
  Cell[355783, 14200, 1034, 40, 34, "Output",
   CellTags->"CovariantD"],
  Cell[356832, 14243, 164, 4, 24, "Text",
   CellTags->"CovariantD"],
  Cell[357021, 14251, 120, 3, 26, "Input",
   CellTags->"CovariantD"],
  Cell[357144, 14256, 218, 7, 26, "Output",
   CellTags->"CovariantD"],
  Cell[357399, 14268, 153, 4, 26, "Input",
   CellTags->"CovariantD"],
  Cell[357555, 14274, 247, 8, 30, "Output",
   CellTags->"CovariantD"],
  Cell[357839, 14287, 203, 5, 26, "Input",
   CellTags->"CovariantD"],
  Cell[358045, 14294, 981, 37, 34, "Output",
   CellTags->"CovariantD"],
  Cell[359063, 14336, 183, 5, 26, "Input",
   CellTags->"CovariantD"],
  Cell[359249, 14343, 974, 36, 33, "Output",
   CellTags->"CovariantD"],
  Cell[360260, 14384, 187, 5, 26, "Input",
   CellTags->"CovariantD"],
  Cell[360450, 14391, 1386, 50, 36, "Output",
   CellTags->"CovariantD"],
  Cell[361851, 14444, 602, 22, 31, "Text",
   CellTags->"CovariantD"],
  Cell[362478, 14470, 215, 6, 26, "Input",
   CellTags->"CovariantD"],
  Cell[362696, 14478, 554, 21, 32, "Output",
   CellTags->"CovariantD"],
  Cell[363265, 14502, 285, 10, 24, "Text",
   CellTags->"CovariantD"],
  Cell[363575, 14516, 215, 6, 26, "Input",
   CellTags->"CovariantD"],
  Cell[363793, 14524, 2331, 82, 57, "Output",
   CellTags->"CovariantD"],
  Cell[366139, 14609, 189, 7, 24, "Text",
   CellTags->"CovariantD"],
  Cell[366353, 14620, 215, 6, 26, "Input",
   CellTags->"CovariantD"],
  Cell[366571, 14628, 6358, 202, 116, "Output",
   CellTags->"CovariantD"],
  Cell[372966, 14835, 181, 5, 26, "Input",
   CellTags->"CovariantD"],
  Cell[373150, 14842, 303, 10, 31, "Output",
   CellTags->"CovariantD"],
  Cell[373490, 14857, 195, 6, 26, "Input",
   CellTags->"CovariantD"],
  Cell[373688, 14865, 4857, 152, 59, "Output",
   CellTags->"CovariantD"],
  Cell[378582, 15022, 223, 7, 26, "Input",
   CellTags->"CovariantD"],
  Cell[378808, 15031, 832, 25, 58, "Output",
   CellTags->"CovariantD"],
  Cell[379677, 15061, 240, 7, 26, "Input",
   CellTags->"CovariantD"],
  Cell[379920, 15070, 847, 26, 58, "Output",
   CellTags->"CovariantD"]},
 "CrossProduct"->{
  Cell[380794, 15100, 61, 1, 39, "ObjectName",
   CellTags->"CrossProduct"],
  Cell[380880, 15105, 63, 1, 34, "Subsubsection",
   CellTags->"CrossProduct"],
  Cell[380946, 15108, 145, 4, 24, "Text",
   CellTags->"CrossProduct"],
  Cell[381094, 15114, 308, 14, 24, "Text",
   CellTags->"CrossProduct"],
  Cell[381439, 15133, 60, 1, 34, "Subsubsection",
   CellTags->"CrossProduct"],
  Cell[381524, 15138, 326, 9, 26, "Input",
   CellTags->"CrossProduct"],
  Cell[381853, 15149, 681, 23, 30, "Output",
   CellTags->"CrossProduct"]},
 "C0"->{
  Cell[382561, 15176, 41, 1, 39, "ObjectName",
   CellTags->"C0"],
  Cell[382627, 15181, 53, 1, 34, "Subsubsection",
   CellTags->"C0"],
  Cell[382683, 15184, 448, 10, 58, "Text",
   CellTags->"C0"],
  Cell[383134, 15196, 458, 24, 24, "Text",
   CellTags->"C0"],
  Cell[383629, 15225, 50, 1, 34, "Subsubsection",
   CellTags->"C0"],
  Cell[383704, 15230, 174, 5, 26, "Input",
   CellTags->"C0"],
  Cell[383881, 15237, 302, 9, 26, "Output",
   CellTags->"C0"],
  Cell[384220, 15251, 203, 6, 26, "Input",
   CellTags->"C0"],
  Cell[384426, 15259, 302, 9, 26, "Output",
   CellTags->"C0"],
  Cell[384765, 15273, 326, 10, 26, "Input",
   CellTags->"C0"],
  Cell[385094, 15285, 301, 9, 26, "Output",
   CellTags->"C0"]},
 "DataType"->{
  Cell[385422, 15298, 53, 1, 39, "ObjectName",
   CellTags->"DataType"],
  Cell[385500, 15303, 59, 1, 34, "Subsubsection",
   CellTags->"DataType"],
  Cell[385562, 15306, 344, 6, 58, "Text",
   CellTags->"DataType"],
  Cell[385909, 15314, 835, 32, 40, "Text",
   CellTags->"DataType"],
  Cell[386747, 15348, 291, 13, 24, "Text",
   CellTags->"DataType"],
  Cell[387041, 15363, 218, 9, 24, "Text",
   CellTags->"DataType"],
  Cell[387296, 15377, 56, 1, 34, "Subsubsection",
   CellTags->"DataType"],
  Cell[387355, 15380, 74, 1, 24, "Text",
   CellTags->"DataType"],
  Cell[387432, 15383, 221, 7, 26, "Input",
   CellTags->"DataType"],
  Cell[387678, 15394, 224, 8, 26, "Input",
   CellTags->"DataType"],
  Cell[387905, 15404, 280, 10, 26, "Output",
   CellTags->"DataType"],
  Cell[388200, 15417, 179, 4, 24, "Text",
   CellTags->"DataType"],
  Cell[388404, 15425, 112, 3, 26, "Input",
   CellTags->"DataType"],
  Cell[388519, 15430, 252, 8, 26, "Output",
   CellTags->"DataType"],
  Cell[388786, 15441, 271, 9, 26, "Input",
   CellTags->"DataType"],
  Cell[389060, 15452, 412, 14, 26, "Input",
   CellTags->"DataType"],
  Cell[389475, 15468, 395, 14, 26, "Input",
   CellTags->"DataType"],
  Cell[389895, 15486, 340, 13, 26, "Input",
   CellTags->"DataType"],
  Cell[390238, 15501, 392, 14, 26, "Output",
   CellTags->"DataType"],
  Cell[390667, 15520, 107, 3, 26, "Input",
   CellTags->"DataType"],
  Cell[390777, 15525, 323, 12, 26, "Output",
   CellTags->"DataType"],
  Cell[391137, 15542, 107, 3, 26, "Input",
   CellTags->"DataType"],
  Cell[391247, 15547, 214, 7, 26, "Output",
   CellTags->"DataType"],
  Cell[391476, 15557, 191, 6, 26, "Input",
   CellTags->"DataType"],
  Cell[391670, 15565, 187, 6, 26, "Input",
   CellTags->"DataType"],
  Cell[391860, 15573, 429, 15, 26, "Input",
   CellTags->"DataType"],
  Cell[392314, 15592, 290, 11, 26, "Input",
   CellTags->"DataType"],
  Cell[392607, 15605, 343, 13, 26, "Output",
   CellTags->"DataType"],
  Cell[392987, 15623, 105, 3, 26, "Input",
   CellTags->"DataType"],
  Cell[393095, 15628, 272, 10, 26, "Output",
   CellTags->"DataType"],
  Cell[393382, 15641, 151, 5, 26, "Input",
   CellTags->"DataType"],
  Cell[393536, 15648, 222, 7, 26, "Input",
   CellTags->"DataType"],
  Cell[393761, 15657, 272, 9, 26, "Input",
   CellTags->"DataType"],
  Cell[394036, 15668, 114, 3, 24, "Text",
   CellTags->"DataType"],
  Cell[394175, 15675, 149, 4, 26, "Input",
   CellTags->"DataType"],
  Cell[394327, 15681, 149, 4, 26, "Output",
   CellTags->"DataType"],
  Cell[394491, 15688, 83, 1, 24, "Text",
   CellTags->"DataType"],
  Cell[394599, 15693, 233, 8, 26, "Input",
   CellTags->"DataType"],
  Cell[394835, 15703, 148, 4, 26, "Output",
   CellTags->"DataType"],
  Cell[395020, 15712, 226, 7, 26, "Input",
   CellTags->"DataType"],
  Cell[395249, 15721, 479, 17, 28, "Output",
   CellTags->"DataType"]},
 "DB0"->{
  Cell[395755, 15742, 43, 1, 39, "ObjectName",
   CellTags->"DB0"],
  Cell[395823, 15747, 54, 1, 34, "Subsubsection",
   CellTags->"DB0"],
  Cell[395880, 15750, 146, 4, 24, "Text",
   CellTags->"DB0"],
  Cell[396029, 15756, 156, 9, 24, "Text",
   CellTags->"DB0"],
  Cell[396222, 15770, 51, 1, 34, "Subsubsection",
   CellTags->"DB0"],
  Cell[396298, 15775, 339, 12, 30, "Input",
   CellTags->"DB0"],
  Cell[396640, 15789, 297, 9, 29, "Output",
   CellTags->"DB0"]},
 "DB1"->{
  Cell[396964, 15802, 43, 1, 39, "ObjectName",
   CellTags->"DB1"],
  Cell[397032, 15807, 54, 1, 34, "Subsubsection",
   CellTags->"DB1"],
  Cell[397089, 15810, 117, 3, 24, "Text",
   CellTags->"DB1"],
  Cell[397209, 15815, 156, 9, 24, "Text",
   CellTags->"DB1"],
  Cell[397402, 15829, 51, 1, 34, "Subsubsection",
   CellTags->"DB1"],
  Cell[397478, 15834, 339, 12, 30, "Input",
   CellTags->"DB1"],
  Cell[397820, 15848, 1452, 46, 52, "Output",
   CellTags->"DB1"]},
 "DeclareNonCommutative"->{
  Cell[399299, 15898, 79, 1, 39, "ObjectName",
   CellTags->"DeclareNonCommutative"],
  Cell[399403, 15903, 72, 1, 34, "Subsubsection",
   CellTags->"DeclareNonCommutative"],
  Cell[399478, 15906, 192, 4, 24, "Text",
   CellTags->"DeclareNonCommutative"],
  Cell[399673, 15912, 347, 14, 24, "Text",
   CellTags->"DeclareNonCommutative"],
  Cell[400057, 15931, 69, 1, 34, "Subsubsection",
   CellTags->"DeclareNonCommutative"],
  Cell[400129, 15934, 135, 3, 26, "Input",
   CellTags->"DeclareNonCommutative"],
  Cell[400267, 15939, 152, 4, 24, "Text",
   CellTags->"DeclareNonCommutative"],
  Cell[400444, 15947, 158, 4, 26, "Input",
   CellTags->"DeclareNonCommutative"],
  Cell[400605, 15953, 164, 4, 26, "Output",
   CellTags->"DeclareNonCommutative"],
  Cell[400784, 15960, 158, 4, 26, "Input",
   CellTags->"DeclareNonCommutative"],
  Cell[400967, 15968, 191, 5, 26, "Input",
   CellTags->"DeclareNonCommutative"],
  Cell[401161, 15975, 241, 7, 26, "Output",
   CellTags->"DeclareNonCommutative"],
  Cell[401417, 15985, 170, 4, 26, "Input",
   CellTags->"DeclareNonCommutative"],
  Cell[401612, 15993, 191, 5, 26, "Input",
   CellTags->"DeclareNonCommutative"],
  Cell[401806, 16000, 241, 7, 26, "Output",
   CellTags->"DeclareNonCommutative"]},
 "DeltaFunction"->{
  Cell[402074, 16011, 63, 1, 39, "ObjectName",
   CellTags->"DeltaFunction"],
  Cell[402162, 16016, 64, 1, 34, "Subsubsection",
   CellTags->"DeltaFunction"],
  Cell[402229, 16019, 353, 11, 49, "Text",
   CellTags->"DeltaFunction"],
  Cell[402585, 16032, 592, 24, 24, "Text",
   CellTags->"DeltaFunction"],
  Cell[403214, 16061, 61, 1, 34, "Subsubsection",
   CellTags->"DeltaFunction"],
  Cell[403300, 16066, 142, 4, 26, "Input",
   CellTags->"DeltaFunction"],
  Cell[403445, 16072, 245, 8, 26, "Output",
   CellTags->"DeltaFunction"],
  Cell[403727, 16085, 320, 10, 26, "Input",
   CellTags->"DeltaFunction"],
  Cell[404050, 16097, 179, 5, 26, "Output",
   CellTags->"DeltaFunction"],
  Cell[404266, 16107, 294, 9, 26, "Input",
   CellTags->"DeltaFunction"],
  Cell[404563, 16118, 179, 5, 26, "Output",
   CellTags->"DeltaFunction"],
  Cell[404779, 16128, 320, 10, 26, "Input",
   CellTags->"DeltaFunction"],
  Cell[405102, 16140, 179, 5, 26, "Output",
   CellTags->"DeltaFunction"],
  Cell[405318, 16150, 204, 6, 26, "Input",
   CellTags->"DeltaFunction"],
  Cell[405525, 16158, 153, 4, 27, "Output",
   CellTags->"DeltaFunction"]},
 "DeltaFunctionDoublePrime"->{
  Cell[405705, 16166, 85, 1, 39, "ObjectName",
   CellTags->"DeltaFunctionDoublePrime"],
  Cell[405815, 16171, 75, 1, 34, "Subsubsection",
   CellTags->"DeltaFunctionDoublePrime"],
  Cell[405893, 16174, 248, 8, 24, "Text",
   CellTags->"DeltaFunctionDoublePrime"],
  Cell[406144, 16184, 219, 8, 24, "Text",
   CellTags->"DeltaFunctionDoublePrime"]},
 "DeltaFunctionPrime"->{
  Cell[406378, 16195, 73, 1, 39, "ObjectName",
   CellTags->"DeltaFunctionPrime"],
  Cell[406476, 16200, 69, 1, 34, "Subsubsection",
   CellTags->"DeltaFunctionPrime"],
  Cell[406548, 16203, 227, 7, 24, "Text",
   CellTags->"DeltaFunctionPrime"],
  Cell[406778, 16212, 576, 23, 24, "Text",
   CellTags->"DeltaFunctionPrime"],
  Cell[407391, 16240, 66, 1, 34, "Subsubsection",
   CellTags->"DeltaFunctionPrime"],
  Cell[407482, 16245, 152, 4, 26, "Input",
   CellTags->"DeltaFunctionPrime"],
  Cell[407637, 16251, 282, 9, 26, "Output",
   CellTags->"DeltaFunctionPrime"],
  Cell[407956, 16265, 330, 10, 26, "Input",
   CellTags->"DeltaFunctionPrime"],
  Cell[408289, 16277, 247, 7, 26, "Output",
   CellTags->"DeltaFunctionPrime"],
  Cell[408573, 16289, 326, 10, 30, "Input",
   CellTags->"DeltaFunctionPrime"],
  Cell[408902, 16301, 156, 4, 26, "Output",
   CellTags->"DeltaFunctionPrime"]},
 "DenominatorOrder"->{
  Cell[409085, 16309, 69, 1, 39, "ObjectName",
   CellTags->"DenominatorOrder"],
  Cell[409179, 16314, 67, 1, 34, "Subsubsection",
   CellTags->"DenominatorOrder"],
  Cell[409249, 16317, 175, 4, 24, "Text",
   CellTags->"DenominatorOrder"],
  Cell[409427, 16323, 327, 13, 24, "Text",
   CellTags->"DenominatorOrder"]},
 "Dimension"->{
  Cell[409769, 16339, 55, 1, 39, "ObjectName",
   CellTags->"Dimension"],
  Cell[409849, 16344, 60, 1, 34, "Subsubsection",
   CellTags->"Dimension"],
  Cell[409912, 16347, 246, 5, 41, "Text",
   CellTags->"Dimension"],
  Cell[410183, 16356, 120, 3, 26, "Input",
   CellTags->"Dimension"],
  Cell[410306, 16361, 286, 9, 26, "Output",
   CellTags->"Dimension"],
  Cell[410629, 16375, 313, 9, 26, "Input",
   CellTags->"Dimension"],
  Cell[410945, 16386, 328, 12, 26, "Output",
   CellTags->"Dimension"],
  Cell[411288, 16401, 131, 4, 24, "Text",
   CellTags->"Dimension"],
  Cell[411444, 16409, 357, 11, 26, "Input",
   CellTags->"Dimension"],
  Cell[411804, 16422, 402, 11, 38, "Output",
   CellTags->"Dimension"],
  Cell[412221, 16436, 166, 4, 24, "Text",
   CellTags->"Dimension"],
  Cell[412390, 16442, 133, 4, 26, "Input",
   CellTags->"Dimension"],
  Cell[412548, 16450, 326, 9, 26, "Input",
   CellTags->"Dimension"],
  Cell[412877, 16461, 706, 29, 26, "Output",
   CellTags->"Dimension"],
  Cell[413620, 16495, 110, 3, 26, "Input",
   CellTags->"Dimension"],
  Cell[413733, 16500, 415, 11, 38, "Output",
   CellTags->"Dimension"],
  Cell[414163, 16514, 134, 4, 26, "Input",
   CellTags->"Dimension"]},
 "DimensionalReduction"->{
  Cell[414312, 16521, 77, 1, 39, "ObjectName",
   CellTags->"DimensionalReduction"],
  Cell[414414, 16526, 71, 1, 34, "Subsubsection",
   CellTags->"DimensionalReduction"],
  Cell[414488, 16529, 121, 3, 24, "Text",
   CellTags->"DimensionalReduction"],
  Cell[414612, 16534, 301, 13, 24, "Text",
   CellTags->"DimensionalReduction"]},
 "DiracBasis"->{
  Cell[414928, 16550, 57, 1, 39, "ObjectName",
   CellTags->"DiracBasis"],
  Cell[415010, 16555, 61, 1, 34, "Subsubsection",
   CellTags->"DiracBasis"],
  Cell[415074, 16558, 299, 6, 41, "Text",
   CellTags->"DiracBasis"],
  Cell[415376, 16566, 184, 8, 24, "Text",
   CellTags->"DiracBasis"]},
 "DiracCanonical"->{
  Cell[415575, 16577, 65, 1, 39, "ObjectName",
   CellTags->"DiracCanonical"],
  Cell[415665, 16582, 65, 1, 34, "Subsubsection",
   CellTags->"DiracCanonical"],
  Cell[415733, 16585, 171, 4, 24, "Text",
   CellTags->"DiracCanonical"],
  Cell[415907, 16591, 194, 8, 24, "Text",
   CellTags->"DiracCanonical"]},
 "DiracEquation"->{
  Cell[416116, 16602, 63, 1, 39, "ObjectName",
   CellTags->"DiracEquation"],
  Cell[416204, 16607, 64, 1, 34, "Subsubsection",
   CellTags->"DiracEquation"],
  Cell[416271, 16610, 160, 4, 24, "Text",
   CellTags->"DiracEquation"],
  Cell[416434, 16616, 199, 9, 24, "Text",
   CellTags->"DiracEquation"],
  Cell[416670, 16630, 61, 1, 34, "Subsubsection",
   CellTags->"DiracEquation"],
  Cell[416756, 16635, 279, 8, 26, "Input",
   CellTags->"DiracEquation"],
  Cell[417038, 16645, 410, 14, 28, "Output",
   CellTags->"DiracEquation"],
  Cell[417485, 16664, 119, 3, 26, "Input",
   CellTags->"DiracEquation"],
  Cell[417607, 16669, 261, 8, 28, "Output",
   CellTags->"DiracEquation"]},
 "DiracGamma"->{
  Cell[417895, 16681, 57, 1, 39, "ObjectName",
   CellTags->"DiracGamma"],
  Cell[417977, 16686, 61, 1, 34, "Subsubsection",
   CellTags->"DiracGamma"],
  Cell[418041, 16689, 762, 24, 61, "Text",
   CellTags->"DiracGamma"],
  Cell[418806, 16715, 681, 29, 24, "Text",
   CellTags->"DiracGamma"],
  Cell[419524, 16749, 58, 1, 34, "Subsubsection",
   CellTags->"DiracGamma"],
  Cell[419607, 16754, 113, 3, 26, "Input",
   CellTags->"DiracGamma"],
  Cell[419723, 16759, 179, 5, 26, "Output",
   CellTags->"DiracGamma"],
  Cell[419939, 16769, 159, 4, 26, "Input",
   CellTags->"DiracGamma"],
  Cell[420101, 16775, 256, 9, 26, "Output",
   CellTags->"DiracGamma"],
  Cell[420372, 16787, 332, 13, 27, "Text",
   CellTags->"DiracGamma"],
  Cell[420729, 16804, 167, 5, 26, "Input",
   CellTags->"DiracGamma"],
  Cell[420899, 16811, 267, 9, 28, "Output",
   CellTags->"DiracGamma"],
  Cell[421203, 16825, 280, 8, 26, "Input",
   CellTags->"DiracGamma"],
  Cell[421486, 16835, 512, 18, 28, "Output",
   CellTags->"DiracGamma"],
  Cell[422035, 16858, 218, 7, 26, "Input",
   CellTags->"DiracGamma"],
  Cell[422256, 16867, 265, 9, 28, "Output",
   CellTags->"DiracGamma"],
  Cell[422558, 16881, 208, 7, 26, "Input",
   CellTags->"DiracGamma"],
  Cell[422769, 16890, 383, 13, 28, "Output",
   CellTags->"DiracGamma"],
  Cell[423189, 16908, 144, 4, 26, "Input",
   CellTags->"DiracGamma"],
  Cell[423336, 16914, 634, 23, 28, "Output",
   CellTags->"DiracGamma"],
  Cell[424007, 16942, 300, 9, 26, "Input",
   CellTags->"DiracGamma"],
  Cell[424310, 16953, 472, 14, 28, "Output",
   CellTags->"DiracGamma"],
  Cell[424819, 16972, 138, 4, 26, "Input",
   CellTags->"DiracGamma"],
  Cell[424960, 16978, 474, 14, 28, "Output",
   CellTags->"DiracGamma"],
  Cell[425471, 16997, 141, 4, 26, "Input",
   CellTags->"DiracGamma"],
  Cell[425615, 17003, 1167, 43, 28, "Output",
   CellTags->"DiracGamma"],
  Cell[426797, 17049, 168, 5, 26, "Input",
   CellTags->"DiracGamma"]},
 "DiracGammaCombine"->{
  Cell[426980, 17057, 71, 1, 39, "ObjectName",
   CellTags->"DiracGammaCombine"],
  Cell[427076, 17062, 68, 1, 34, "Subsubsection",
   CellTags->"DiracGammaCombine"],
  Cell[427147, 17065, 135, 4, 24, "Text",
   CellTags->"DiracGammaCombine"],
  Cell[427285, 17071, 804, 34, 24, "Text",
   CellTags->"DiracGammaCombine"],
  Cell[428126, 17110, 65, 1, 34, "Subsubsection",
   CellTags->"DiracGammaCombine"],
  Cell[428216, 17115, 220, 6, 26, "Input",
   CellTags->"DiracGammaCombine"],
  Cell[428439, 17123, 307, 10, 28, "Output",
   CellTags->"DiracGammaCombine"],
  Cell[428783, 17138, 122, 3, 26, "Input",
   CellTags->"DiracGammaCombine"],
  Cell[428908, 17143, 238, 6, 38, "Output",
   CellTags->"DiracGammaCombine"],
  Cell[429183, 17154, 270, 8, 26, "Input",
   CellTags->"DiracGammaCombine"],
  Cell[429456, 17164, 359, 12, 28, "Output",
   CellTags->"DiracGammaCombine"],
  Cell[429852, 17181, 122, 3, 26, "Input",
   CellTags->"DiracGammaCombine"],
  Cell[429977, 17186, 286, 8, 38, "Output",
   CellTags->"DiracGammaCombine"],
  Cell[430300, 17199, 127, 3, 26, "Input",
   CellTags->"DiracGammaCombine"],
  Cell[430430, 17204, 482, 17, 28, "Output",
   CellTags->"DiracGammaCombine"]},
 "DiracGammaExpand"->{
  Cell[430939, 17225, 69, 1, 39, "ObjectName",
   CellTags->"DiracGammaExpand"],
  Cell[431033, 17230, 67, 1, 34, "Subsubsection",
   CellTags->"DiracGammaExpand"],
  Cell[431103, 17233, 190, 4, 41, "Text",
   CellTags->"DiracGammaExpand"],
  Cell[431296, 17239, 806, 34, 24, "Text",
   CellTags->"DiracGammaExpand"],
  Cell[432139, 17278, 64, 1, 34, "Subsubsection",
   CellTags->"DiracGammaExpand"],
  Cell[432228, 17283, 314, 9, 26, "Input",
   CellTags->"DiracGammaExpand"],
  Cell[432545, 17294, 516, 18, 28, "Output",
   CellTags->"DiracGammaExpand"],
  Cell[433076, 17315, 125, 3, 24, "Text",
   CellTags->"DiracGammaExpand"],
  Cell[433226, 17322, 121, 3, 26, "Input",
   CellTags->"DiracGammaExpand"],
  Cell[433350, 17327, 333, 9, 38, "Output",
   CellTags->"DiracGammaExpand"],
  Cell[433698, 17339, 101, 1, 24, "Text",
   CellTags->"DiracGammaExpand"],
  Cell[433824, 17344, 125, 3, 26, "Input",
   CellTags->"DiracGammaExpand"],
  Cell[433952, 17349, 641, 23, 28, "Output",
   CellTags->"DiracGammaExpand"],
  Cell[434608, 17375, 90, 1, 24, "Text",
   CellTags->"DiracGammaExpand"],
  Cell[434723, 17380, 209, 5, 26, "Input",
   CellTags->"DiracGammaExpand"],
  Cell[434935, 17387, 333, 9, 38, "Output",
   CellTags->"DiracGammaExpand"],
  Cell[435305, 17401, 164, 4, 26, "Input",
   CellTags->"DiracGammaExpand"],
  Cell[435472, 17407, 436, 12, 38, "Output",
   CellTags->"DiracGammaExpand"],
  Cell[435923, 17422, 106, 1, 24, "Text",
   CellTags->"DiracGammaExpand"],
  Cell[436054, 17427, 122, 3, 26, "Input",
   CellTags->"DiracGammaExpand"],
  Cell[436179, 17432, 576, 21, 28, "Output",
   CellTags->"DiracGammaExpand"],
  Cell[436770, 17456, 114, 3, 26, "Input",
   CellTags->"DiracGammaExpand"]},
 "DiracGammaT"->{
  Cell[436899, 17462, 59, 1, 39, "ObjectName",
   CellTags->"DiracGammaT"],
  Cell[436983, 17467, 62, 1, 34, "Subsubsection",
   CellTags->"DiracGammaT"],
  Cell[437048, 17470, 208, 5, 41, "Text",
   CellTags->"DiracGammaT"],
  Cell[437259, 17477, 188, 9, 24, "Text",
   CellTags->"DiracGammaT"],
  Cell[437484, 17491, 59, 1, 34, "Subsubsection",
   CellTags->"DiracGammaT"],
  Cell[437568, 17496, 158, 4, 26, "Input",
   CellTags->"DiracGammaT"],
  Cell[437729, 17502, 262, 9, 30, "Output",
   CellTags->"DiracGammaT"],
  Cell[438028, 17516, 113, 3, 26, "Input",
   CellTags->"DiracGammaT"],
  Cell[438144, 17521, 256, 9, 26, "Output",
   CellTags->"DiracGammaT"],
  Cell[438437, 17535, 167, 6, 26, "Input",
   CellTags->"DiracGammaT"],
  Cell[438607, 17543, 261, 8, 28, "Output",
   CellTags->"DiracGammaT"]},
 "DiracMatrix"->{
  Cell[438895, 17555, 59, 1, 39, "ObjectName",
   CellTags->"DiracMatrix"],
  Cell[438979, 17560, 62, 1, 34, "Subsubsection",
   CellTags->"DiracMatrix"],
  Cell[439044, 17563, 1220, 46, 44, "Text",
   CellTags->"DiracMatrix"],
  Cell[440267, 17611, 1053, 49, 24, "Text",
   CellTags->"DiracMatrix"],
  Cell[441357, 17665, 59, 1, 34, "Subsubsection",
   CellTags->"DiracMatrix"],
  Cell[441441, 17670, 119, 3, 26, "Input",
   CellTags->"DiracMatrix"],
  Cell[441563, 17675, 254, 9, 26, "Output",
   CellTags->"DiracMatrix"],
  Cell[441832, 17687, 539, 20, 41, "Text",
   CellTags->"DiracMatrix"],
  Cell[442396, 17711, 184, 5, 26, "Input",
   CellTags->"DiracMatrix"],
  Cell[442583, 17718, 394, 15, 26, "Output",
   CellTags->"DiracMatrix"],
  Cell[443014, 17738, 160, 5, 26, "Input",
   CellTags->"DiracMatrix"],
  Cell[443177, 17745, 217, 5, 38, "Output",
   CellTags->"DiracMatrix"],
  Cell[443431, 17755, 145, 4, 26, "Input",
   CellTags->"DiracMatrix"],
  Cell[443579, 17761, 186, 5, 26, "Output",
   CellTags->"DiracMatrix"],
  Cell[443802, 17771, 112, 3, 26, "Input",
   CellTags->"DiracMatrix"],
  Cell[443917, 17776, 167, 4, 38, "Output",
   CellTags->"DiracMatrix"],
  Cell[444121, 17785, 111, 3, 26, "Input",
   CellTags->"DiracMatrix"],
  Cell[444235, 17790, 184, 5, 26, "Output",
   CellTags->"DiracMatrix"],
  Cell[444456, 17800, 138, 4, 26, "Input",
   CellTags->"DiracMatrix"],
  Cell[444597, 17806, 263, 7, 38, "Output",
   CellTags->"DiracMatrix"],
  Cell[444897, 17818, 152, 4, 26, "Input",
   CellTags->"DiracMatrix"],
  Cell[445052, 17824, 291, 8, 26, "Output",
   CellTags->"DiracMatrix"],
  Cell[445380, 17837, 166, 5, 26, "Input",
   CellTags->"DiracMatrix"],
  Cell[445549, 17844, 384, 15, 28, "Output",
   CellTags->"DiracMatrix"],
  Cell[445970, 17864, 112, 3, 26, "Input",
   CellTags->"DiracMatrix"],
  Cell[446085, 17869, 313, 8, 38, "Output",
   CellTags->"DiracMatrix"]},
 "DiracOrder"->{
  Cell[446425, 17881, 57, 1, 39, "ObjectName",
   CellTags->"DiracOrder"],
  Cell[446507, 17886, 61, 1, 34, "Subsubsection",
   CellTags->"DiracOrder"],
  Cell[446571, 17889, 204, 5, 41, "Text",
   CellTags->"DiracOrder"],
  Cell[446778, 17896, 312, 14, 24, "Text",
   CellTags->"DiracOrder"],
  Cell[447127, 17915, 58, 1, 34, "Subsubsection",
   CellTags->"DiracOrder"],
  Cell[447210, 17920, 176, 5, 26, "Input",
   CellTags->"DiracOrder"],
  Cell[447389, 17927, 250, 7, 26, "Output",
   CellTags->"DiracOrder"],
  Cell[447676, 17939, 114, 3, 26, "Input",
   CellTags->"DiracOrder"],
  Cell[447793, 17944, 514, 19, 26, "Output",
   CellTags->"DiracOrder"],
  Cell[448322, 17966, 93, 1, 24, "Text",
   CellTags->"DiracOrder"],
  Cell[448440, 17971, 176, 5, 26, "Input",
   CellTags->"DiracOrder"],
  Cell[448619, 17978, 289, 8, 26, "Output",
   CellTags->"DiracOrder"],
  Cell[448945, 17991, 114, 3, 26, "Input",
   CellTags->"DiracOrder"],
  Cell[449062, 17996, 514, 19, 26, "Output",
   CellTags->"DiracOrder"],
  Cell[449613, 18020, 171, 5, 26, "Input",
   CellTags->"DiracOrder"],
  Cell[449787, 18027, 285, 8, 26, "Output",
   CellTags->"DiracOrder"],
  Cell[450087, 18038, 196, 8, 24, "Text",
   CellTags->"DiracOrder"],
  Cell[450308, 18050, 114, 3, 26, "Input",
   CellTags->"DiracOrder"],
  Cell[450425, 18055, 435, 16, 26, "Output",
   CellTags->"DiracOrder"],
  Cell[450897, 18076, 167, 5, 26, "Input",
   CellTags->"DiracOrder"],
  Cell[451067, 18083, 283, 8, 26, "Output",
   CellTags->"DiracOrder"],
  Cell[451387, 18096, 114, 3, 26, "Input",
   CellTags->"DiracOrder"],
  Cell[451504, 18101, 315, 11, 26, "Output",
   CellTags->"DiracOrder"],
  Cell[451856, 18117, 183, 5, 26, "Input",
   CellTags->"DiracOrder"],
  Cell[452042, 18124, 297, 8, 26, "Output",
   CellTags->"DiracOrder"],
  Cell[452354, 18135, 288, 10, 24, "Text",
   CellTags->"DiracOrder"],
  Cell[452667, 18149, 219, 7, 26, "Input",
   CellTags->"DiracOrder"],
  Cell[452889, 18158, 1251, 46, 26, "Output",
   CellTags->"DiracOrder"],
  Cell[454177, 18209, 113, 3, 26, "Input",
   CellTags->"DiracOrder"],
  Cell[454293, 18214, 521, 20, 26, "Output",
   CellTags->"DiracOrder"],
  Cell[454829, 18237, 188, 6, 26, "Input",
   CellTags->"DiracOrder"]},
 "DiracReduce"->{
  Cell[455032, 18246, 59, 1, 39, "ObjectName",
   CellTags->"DiracReduce"],
  Cell[455116, 18251, 62, 1, 34, "Subsubsection",
   CellTags->"DiracReduce"],
  Cell[455181, 18254, 662, 11, 92, "Text",
   CellTags->"DiracReduce"],
  Cell[455868, 18269, 121, 3, 26, "Input",
   CellTags->"DiracReduce"],
  Cell[455992, 18274, 365, 11, 26, "Output",
   CellTags->"DiracReduce"],
  Cell[456372, 18288, 197, 9, 24, "Text",
   CellTags->"DiracReduce"],
  Cell[456606, 18302, 59, 1, 34, "Subsubsection",
   CellTags->"DiracReduce"],
  Cell[456690, 18307, 172, 5, 26, "Input",
   CellTags->"DiracReduce"],
  Cell[456865, 18314, 246, 7, 26, "Output",
   CellTags->"DiracReduce"],
  Cell[457148, 18326, 116, 3, 26, "Input",
   CellTags->"DiracReduce"],
  Cell[457267, 18331, 323, 10, 26, "Output",
   CellTags->"DiracReduce"],
  Cell[457627, 18346, 186, 5, 26, "Input",
   CellTags->"DiracReduce"],
  Cell[457816, 18353, 281, 8, 26, "Output",
   CellTags->"DiracReduce"],
  Cell[458134, 18366, 116, 3, 26, "Input",
   CellTags->"DiracReduce"],
  Cell[458253, 18371, 961, 29, 26, "Output",
   CellTags->"DiracReduce"],
  Cell[459251, 18405, 207, 6, 26, "Input",
   CellTags->"DiracReduce"],
  Cell[459461, 18413, 325, 9, 26, "Output",
   CellTags->"DiracReduce"],
  Cell[459823, 18427, 140, 4, 26, "Input",
   CellTags->"DiracReduce"],
  Cell[459966, 18433, 1855, 55, 47, "Output",
   CellTags->"DiracReduce"],
  Cell[461858, 18493, 253, 7, 26, "Input",
   CellTags->"DiracReduce"],
  Cell[462114, 18502, 169, 5, 26, "Output",
   CellTags->"DiracReduce"],
  Cell[462320, 18512, 133, 4, 26, "Input",
   CellTags->"DiracReduce"],
  Cell[462456, 18518, 169, 5, 26, "Output",
   CellTags->"DiracReduce"],
  Cell[462640, 18526, 156, 4, 26, "Input",
   CellTags->"DiracReduce"]},
 "DiracSigma"->{
  Cell[462811, 18533, 57, 1, 39, "ObjectName",
   CellTags->"DiracSigma"],
  Cell[462893, 18538, 61, 1, 34, "Subsubsection",
   CellTags->"DiracSigma"],
  Cell[462957, 18541, 213, 5, 41, "Text",
   CellTags->"DiracSigma"],
  Cell[463173, 18548, 211, 9, 24, "Text",
   CellTags->"DiracSigma"],
  Cell[463421, 18562, 58, 1, 34, "Subsubsection",
   CellTags->"DiracSigma"],
  Cell[463504, 18567, 236, 7, 26, "Input",
   CellTags->"DiracSigma"],
  Cell[463743, 18576, 211, 6, 26, "Output",
   CellTags->"DiracSigma"],
  Cell[463991, 18587, 122, 3, 26, "Input",
   CellTags->"DiracSigma"],
  Cell[464116, 18592, 826, 30, 44, "Output",
   CellTags->"DiracSigma"],
  Cell[464979, 18627, 236, 7, 26, "Input",
   CellTags->"DiracSigma"],
  Cell[465218, 18636, 231, 7, 26, "Output",
   CellTags->"DiracSigma"],
  Cell[465486, 18648, 223, 7, 26, "Input",
   CellTags->"DiracSigma"],
  Cell[465712, 18657, 200, 6, 26, "Output",
   CellTags->"DiracSigma"],
  Cell[465949, 18668, 122, 3, 26, "Input",
   CellTags->"DiracSigma"],
  Cell[466074, 18673, 994, 34, 44, "Output",
   CellTags->"DiracSigma"],
  Cell[467083, 18710, 144, 4, 26, "Input",
   CellTags->"DiracSigma"]},
 "DiracSigmaExplicit"->{
  Cell[467242, 18717, 73, 1, 39, "ObjectName",
   CellTags->"DiracSigmaExplicit"],
  Cell[467340, 18722, 69, 1, 34, "Subsubsection",
   CellTags->"DiracSigmaExplicit"],
  Cell[467412, 18725, 186, 4, 24, "Text",
   CellTags->"DiracSigmaExplicit"],
  Cell[467601, 18731, 195, 9, 24, "Text",
   CellTags->"DiracSigmaExplicit"],
  Cell[467833, 18745, 66, 1, 34, "Subsubsection",
   CellTags->"DiracSigmaExplicit"],
  Cell[467924, 18750, 217, 6, 26, "Input",
   CellTags->"DiracSigmaExplicit"],
  Cell[468144, 18758, 219, 6, 26, "Output",
   CellTags->"DiracSigmaExplicit"],
  Cell[468400, 18769, 129, 3, 26, "Input",
   CellTags->"DiracSigmaExplicit"],
  Cell[468532, 18774, 834, 30, 44, "Output",
   CellTags->"DiracSigmaExplicit"]},
 "DiracSimpCombine"->{
  Cell[469393, 18808, 69, 1, 39, "ObjectName",
   CellTags->"DiracSimpCombine"],
  Cell[469487, 18813, 67, 1, 34, "Subsubsection",
   CellTags->"DiracSimpCombine"],
  Cell[469557, 18816, 209, 5, 41, "Text",
   CellTags->"DiracSimpCombine"],
  Cell[469769, 18823, 196, 8, 24, "Text",
   CellTags->"DiracSimpCombine"]},
 "DiracSimplify"->{
  Cell[469980, 18834, 63, 1, 39, "ObjectName",
   CellTags->"DiracSimplify"],
  Cell[470068, 18839, 64, 1, 34, "Subsubsection",
   CellTags->"DiracSimplify"],
  Cell[470135, 18842, 382, 7, 58, "Text",
   CellTags->"DiracSimplify"],
  Cell[470542, 18853, 125, 3, 26, "Input",
   CellTags->"DiracSimplify"],
  Cell[470670, 18858, 646, 15, 47, "Output",
   CellTags->"DiracSimplify"],
  Cell[471331, 18876, 422, 19, 24, "Text",
   CellTags->"DiracSimplify"],
  Cell[471790, 18900, 61, 1, 34, "Subsubsection",
   CellTags->"DiracSimplify"],
  Cell[471854, 18903, 99, 1, 24, "Text",
   CellTags->"DiracSimplify"],
  Cell[471978, 18908, 178, 5, 26, "Input",
   CellTags->"DiracSimplify"],
  Cell[472159, 18915, 294, 8, 26, "Output",
   CellTags->"DiracSimplify"],
  Cell[472490, 18928, 120, 3, 26, "Input",
   CellTags->"DiracSimplify"],
  Cell[472613, 18933, 302, 11, 26, "Output",
   CellTags->"DiracSimplify"],
  Cell[472930, 18947, 96, 1, 24, "Text",
   CellTags->"DiracSimplify"],
  Cell[473051, 18952, 179, 5, 26, "Input",
   CellTags->"DiracSimplify"],
  Cell[473233, 18959, 292, 8, 26, "Output",
   CellTags->"DiracSimplify"],
  Cell[473562, 18972, 120, 3, 26, "Input",
   CellTags->"DiracSimplify"],
  Cell[473685, 18977, 303, 11, 26, "Output",
   CellTags->"DiracSimplify"],
  Cell[474025, 18993, 174, 5, 26, "Input",
   CellTags->"DiracSimplify"],
  Cell[474202, 19000, 288, 8, 26, "Output",
   CellTags->"DiracSimplify"],
  Cell[474505, 19011, 179, 7, 24, "Text",
   CellTags->"DiracSimplify"],
  Cell[474709, 19022, 120, 3, 26, "Input",
   CellTags->"DiracSimplify"],
  Cell[474832, 19027, 438, 16, 26, "Output",
   CellTags->"DiracSimplify"],
  Cell[475307, 19048, 170, 5, 26, "Input",
   CellTags->"DiracSimplify"],
  Cell[475480, 19055, 286, 8, 26, "Output",
   CellTags->"DiracSimplify"],
  Cell[475803, 19068, 120, 3, 26, "Input",
   CellTags->"DiracSimplify"],
  Cell[475926, 19073, 316, 11, 26, "Output",
   CellTags->"DiracSimplify"],
  Cell[476279, 19089, 326, 10, 26, "Input",
   CellTags->"DiracSimplify"],
  Cell[476608, 19101, 612, 19, 28, "Output",
   CellTags->"DiracSimplify"],
  Cell[477235, 19123, 120, 3, 24, "Text",
   CellTags->"DiracSimplify"],
  Cell[477380, 19130, 120, 3, 26, "Input",
   CellTags->"DiracSimplify"],
  Cell[477503, 19135, 1952, 73, 28, "Output",
   CellTags->"DiracSimplify"],
  Cell[479492, 19213, 117, 3, 26, "Input",
   CellTags->"DiracSimplify"],
  Cell[479612, 19218, 612, 19, 28, "Output",
   CellTags->"DiracSimplify"],
  Cell[480261, 19242, 272, 7, 26, "Input",
   CellTags->"DiracSimplify"],
  Cell[480536, 19251, 451, 15, 28, "Output",
   CellTags->"DiracSimplify"],
  Cell[481024, 19271, 120, 3, 26, "Input",
   CellTags->"DiracSimplify"],
  Cell[481147, 19276, 153, 4, 26, "Output",
   CellTags->"DiracSimplify"],
  Cell[481337, 19285, 373, 12, 26, "Input",
   CellTags->"DiracSimplify"],
  Cell[481713, 19299, 666, 19, 26, "Output",
   CellTags->"DiracSimplify"],
  Cell[482416, 19323, 119, 3, 26, "Input",
   CellTags->"DiracSimplify"],
  Cell[482538, 19328, 14006, 532, 107, "Output",
   CellTags->"DiracSimplify"],
  Cell[496559, 19863, 115, 3, 24, "Text",
   CellTags->"DiracSimplify"],
  Cell[496699, 19870, 251, 7, 26, "Input",
   CellTags->"DiracSimplify"],
  Cell[496953, 19879, 507, 19, 26, "Output",
   CellTags->"DiracSimplify"],
  Cell[497475, 19901, 141, 4, 24, "Text",
   CellTags->"DiracSimplify"],
  Cell[497641, 19909, 289, 8, 26, "Input",
   CellTags->"DiracSimplify"],
  Cell[497933, 19919, 598, 20, 26, "Output",
   CellTags->"DiracSimplify"],
  Cell[498568, 19944, 268, 7, 26, "Input",
   CellTags->"DiracSimplify"],
  Cell[498839, 19953, 151, 4, 26, "Output",
   CellTags->"DiracSimplify"],
  Cell[499005, 19960, 183, 5, 26, "Input",
   CellTags->"DiracSimplify"]},
 "DiracSimplify2"->{
  Cell[499203, 19968, 65, 1, 39, "ObjectName",
   CellTags->"DiracSimplify2"],
  Cell[499293, 19973, 65, 1, 34, "Subsubsection",
   CellTags->"DiracSimplify2"],
  Cell[499361, 19976, 224, 7, 24, "Text",
   CellTags->"DiracSimplify2"],
  Cell[499588, 19985, 200, 9, 24, "Text",
   CellTags->"DiracSimplify2"],
  Cell[499825, 19999, 62, 1, 34, "Subsubsection",
   CellTags->"DiracSimplify2"],
  Cell[499912, 20004, 222, 6, 26, "Input",
   CellTags->"DiracSimplify2"],
  Cell[500137, 20012, 481, 12, 26, "Output",
   CellTags->"DiracSimplify2"],
  Cell[500655, 20029, 121, 3, 26, "Input",
   CellTags->"DiracSimplify2"],
  Cell[500779, 20034, 924, 36, 26, "Output",
   CellTags->"DiracSimplify2"]},
 "DiracSlash"->{
  Cell[501730, 20074, 57, 1, 39, "ObjectName",
   CellTags->"DiracSlash"],
  Cell[501812, 20079, 61, 1, 34, "Subsubsection",
   CellTags->"DiracSlash"],
  Cell[501876, 20082, 469, 16, 44, "Text",
   CellTags->"DiracSlash"],
  Cell[502370, 20102, 119, 3, 26, "Input",
   CellTags->"DiracSlash"],
  Cell[502492, 20107, 287, 9, 26, "Output",
   CellTags->"DiracSlash"],
  Cell[502794, 20119, 868, 39, 24, "Text",
   CellTags->"DiracSlash"],
  Cell[503699, 20163, 58, 1, 34, "Subsubsection",
   CellTags->"DiracSlash"],
  Cell[503760, 20166, 234, 9, 27, "Text",
   CellTags->"DiracSlash"],
  Cell[504019, 20179, 113, 3, 26, "Input",
   CellTags->"DiracSlash"],
  Cell[504135, 20184, 267, 9, 28, "Output",
   CellTags->"DiracSlash"],
  Cell[504439, 20198, 173, 5, 26, "Input",
   CellTags->"DiracSlash"],
  Cell[504615, 20205, 473, 17, 28, "Output",
   CellTags->"DiracSlash"],
  Cell[505125, 20227, 136, 4, 26, "Input",
   CellTags->"DiracSlash"],
  Cell[505264, 20233, 475, 17, 28, "Output",
   CellTags->"DiracSlash"],
  Cell[505776, 20255, 105, 3, 26, "Input",
   CellTags->"DiracSlash"],
  Cell[505884, 20260, 191, 5, 28, "Output",
   CellTags->"DiracSlash"],
  Cell[506112, 20270, 148, 4, 26, "Input",
   CellTags->"DiracSlash"],
  Cell[506263, 20276, 170, 4, 38, "Output",
   CellTags->"DiracSlash"],
  Cell[506470, 20285, 214, 7, 26, "Input",
   CellTags->"DiracSlash"],
  Cell[506687, 20294, 231, 6, 38, "Output",
   CellTags->"DiracSlash"]},
 "DiracSpinor"->{
  Cell[506945, 20304, 59, 1, 39, "ObjectName",
   CellTags->"DiracSpinor"],
  Cell[507029, 20309, 62, 1, 34, "Subsubsection",
   CellTags->"DiracSpinor"],
  Cell[507094, 20312, 288, 6, 41, "Text",
   CellTags->"DiracSpinor"],
  Cell[507385, 20320, 170, 8, 24, "Text",
   CellTags->"DiracSpinor"]},
 "DiracSubstitute67"->{
  Cell[507570, 20331, 71, 1, 39, "ObjectName",
   CellTags->"DiracSubstitute67"],
  Cell[507666, 20336, 68, 1, 34, "Subsubsection",
   CellTags->"DiracSubstitute67"],
  Cell[507737, 20339, 219, 5, 41, "Text",
   CellTags->"DiracSubstitute67"],
  Cell[507959, 20346, 313, 13, 24, "Text",
   CellTags->"DiracSubstitute67"]},
 "DiracTrace"->{
  Cell[508287, 20362, 57, 1, 39, "ObjectName",
   CellTags->"DiracTrace"],
  Cell[508369, 20367, 61, 1, 34, "Subsubsection",
   CellTags->"DiracTrace"],
  Cell[508433, 20370, 336, 6, 58, "Text",
   CellTags->"DiracTrace"],
  Cell[508794, 20380, 119, 3, 26, "Input",
   CellTags->"DiracTrace"],
  Cell[508916, 20385, 706, 17, 47, "Output",
   CellTags->"DiracTrace"],
  Cell[509637, 20405, 283, 12, 24, "Text",
   CellTags->"DiracTrace"],
  Cell[509923, 20419, 163, 9, 24, "Text",
   CellTags->"DiracTrace"],
  Cell[510123, 20433, 58, 1, 34, "Subsubsection",
   CellTags->"DiracTrace"],
  Cell[510206, 20438, 174, 5, 26, "Input",
   CellTags->"DiracTrace"],
  Cell[510383, 20445, 312, 10, 26, "Output",
   CellTags->"DiracTrace"],
  Cell[510732, 20460, 209, 6, 26, "Input",
   CellTags->"DiracTrace"],
  Cell[510944, 20468, 412, 12, 26, "Output",
   CellTags->"DiracTrace"],
  Cell[511393, 20485, 149, 4, 26, "Input",
   CellTags->"DiracTrace"],
  Cell[511545, 20491, 676, 23, 26, "Output",
   CellTags->"DiracTrace"],
  Cell[512258, 20519, 296, 8, 26, "Input",
   CellTags->"DiracTrace"],
  Cell[512557, 20529, 608, 24, 26, "Output",
   CellTags->"DiracTrace"],
  Cell[513202, 20558, 334, 9, 26, "Input",
   CellTags->"DiracTrace"],
  Cell[513539, 20569, 8767, 322, 73, "Output",
   CellTags->"DiracTrace"],
  Cell[522343, 20896, 186, 5, 26, "Input",
   CellTags->"DiracTrace"],
  Cell[522532, 20903, 540, 16, 28, "Output",
   CellTags->"DiracTrace"],
  Cell[523109, 20924, 294, 8, 26, "Input",
   CellTags->"DiracTrace"],
  Cell[523406, 20934, 224, 7, 26, "Output",
   CellTags->"DiracTrace"],
  Cell[523667, 20946, 111, 3, 26, "Input",
   CellTags->"DiracTrace"],
  Cell[523781, 20951, 217, 6, 38, "Output",
   CellTags->"DiracTrace"]},
 "DiracTraceEvaluate"->{
  Cell[524025, 20961, 73, 1, 39, "ObjectName",
   CellTags->"DiracTraceEvaluate"],
  Cell[524123, 20966, 69, 1, 34, "Subsubsection",
   CellTags->"DiracTraceEvaluate"],
  Cell[524195, 20969, 162, 4, 24, "Text",
   CellTags->"DiracTraceEvaluate"],
  Cell[524360, 20975, 281, 13, 24, "Text",
   CellTags->"DiracTraceEvaluate"]},
 "DiracTrick"->{
  Cell[524656, 20991, 57, 1, 39, "ObjectName",
   CellTags->"DiracTrick"],
  Cell[524738, 20996, 61, 1, 34, "Subsubsection",
   CellTags->"DiracTrick"],
  Cell[524802, 20999, 222, 5, 41, "Text",
   CellTags->"DiracTrick"],
  Cell[525049, 21008, 119, 3, 26, "Input",
   CellTags->"DiracTrick"],
  Cell[525171, 21013, 218, 7, 26, "Output",
   CellTags->"DiracTrick"],
  Cell[525404, 21023, 428, 19, 24, "Text",
   CellTags->"DiracTrick"],
  Cell[525869, 21047, 58, 1, 34, "Subsubsection",
   CellTags->"DiracTrick"],
  Cell[525930, 21050, 96, 1, 24, "Text",
   CellTags->"DiracTrick"],
  Cell[526051, 21055, 175, 5, 26, "Input",
   CellTags->"DiracTrick"],
  Cell[526229, 21062, 289, 8, 26, "Output",
   CellTags->"DiracTrick"],
  Cell[526555, 21075, 114, 3, 26, "Input",
   CellTags->"DiracTrick"],
  Cell[526672, 21080, 289, 8, 26, "Output",
   CellTags->"DiracTrick"],
  Cell[526976, 21091, 93, 1, 24, "Text",
   CellTags->"DiracTrick"],
  Cell[527094, 21096, 176, 5, 26, "Input",
   CellTags->"DiracTrick"],
  Cell[527273, 21103, 289, 8, 26, "Output",
   CellTags->"DiracTrick"],
  Cell[527599, 21116, 114, 3, 26, "Input",
   CellTags->"DiracTrick"],
  Cell[527716, 21121, 291, 8, 26, "Output",
   CellTags->"DiracTrick"],
  Cell[528044, 21134, 171, 5, 26, "Input",
   CellTags->"DiracTrick"],
  Cell[528218, 21141, 285, 8, 26, "Output",
   CellTags->"DiracTrick"],
  Cell[528518, 21152, 176, 7, 24, "Text",
   CellTags->"DiracTrick"],
  Cell[528719, 21163, 114, 3, 26, "Input",
   CellTags->"DiracTrick"],
  Cell[528836, 21168, 287, 8, 26, "Output",
   CellTags->"DiracTrick"],
  Cell[529160, 21181, 167, 5, 26, "Input",
   CellTags->"DiracTrick"],
  Cell[529330, 21188, 281, 8, 26, "Output",
   CellTags->"DiracTrick"],
  Cell[529648, 21201, 114, 3, 26, "Input",
   CellTags->"DiracTrick"],
  Cell[529765, 21206, 281, 8, 26, "Output",
   CellTags->"DiracTrick"],
  Cell[530083, 21219, 323, 10, 26, "Input",
   CellTags->"DiracTrick"],
  Cell[530409, 21231, 609, 19, 28, "Output",
   CellTags->"DiracTrick"],
  Cell[531055, 21255, 114, 3, 26, "Input",
   CellTags->"DiracTrick"],
  Cell[531172, 21260, 609, 19, 28, "Output",
   CellTags->"DiracTrick"],
  Cell[531818, 21284, 108, 3, 26, "Input",
   CellTags->"DiracTrick"],
  Cell[531929, 21289, 1947, 73, 28, "Output",
   CellTags->"DiracTrick"],
  Cell[533913, 21367, 370, 12, 26, "Input",
   CellTags->"DiracTrick"],
  Cell[534286, 21381, 663, 19, 26, "Output",
   CellTags->"DiracTrick"],
  Cell[534986, 21405, 113, 3, 26, "Input",
   CellTags->"DiracTrick"],
  Cell[535102, 21410, 663, 19, 26, "Output",
   CellTags->"DiracTrick"]},
 "Divideout"->{
  Cell[535792, 21433, 55, 1, 39, "ObjectName",
   CellTags->"Divideout"],
  Cell[535872, 21438, 60, 1, 34, "Subsubsection",
   CellTags->"Divideout"],
  Cell[535935, 21441, 137, 4, 24, "Text",
   CellTags->"Divideout"],
  Cell[536075, 21447, 281, 13, 24, "Text",
   CellTags->"Divideout"]},
 "DOT"->{
  Cell[536371, 21463, 43, 1, 39, "ObjectName",
   CellTags->"DOT"],
  Cell[536439, 21468, 54, 1, 34, "Subsubsection",
   CellTags->"DOT"],
  Cell[536496, 21471, 348, 8, 91, "Text",
   CellTags->"DOT"],
  Cell[536847, 21481, 177, 8, 24, "Text",
   CellTags->"DOT"]},
 "DotExpand"->{
  Cell[537039, 21492, 55, 1, 39, "ObjectName",
   CellTags->"DotExpand"],
  Cell[537119, 21497, 60, 1, 34, "Subsubsection",
   CellTags->"DotExpand"],
  Cell[537182, 21500, 85, 1, 24, "Text",
   CellTags->"DotExpand"],
  Cell[537270, 21503, 582, 23, 24, "Text",
   CellTags->"DotExpand"],
  Cell[537889, 21531, 57, 1, 34, "Subsubsection",
   CellTags->"DotExpand"],
  Cell[537971, 21536, 313, 10, 26, "Input",
   CellTags->"DotExpand"],
  Cell[538287, 21548, 546, 14, 26, "Output",
   CellTags->"DotExpand"],
  Cell[538848, 21565, 229, 7, 26, "Input",
   CellTags->"DotExpand"],
  Cell[539102, 21576, 313, 10, 26, "Input",
   CellTags->"DotExpand"],
  Cell[539418, 21588, 681, 23, 26, "Output",
   CellTags->"DotExpand"],
  Cell[540114, 21614, 231, 7, 26, "Input",
   CellTags->"DotExpand"],
  Cell[540370, 21625, 313, 10, 26, "Input",
   CellTags->"DotExpand"],
  Cell[540686, 21637, 546, 14, 26, "Output",
   CellTags->"DotExpand"]},
 "DotPower"->{
  Cell[541259, 21655, 53, 1, 39, "ObjectName",
   CellTags->"DotPower"],
  Cell[541337, 21660, 59, 1, 34, "Subsubsection",
   CellTags->"DotPower"],
  Cell[541399, 21663, 189, 4, 41, "Text",
   CellTags->"DotPower"],
  Cell[541591, 21669, 182, 8, 24, "Text",
   CellTags->"DotPower"]},
 "DotProduct"->{
  Cell[541788, 21680, 57, 1, 39, "ObjectName",
   CellTags->"DotProduct"],
  Cell[541870, 21685, 61, 1, 34, "Subsubsection",
   CellTags->"DotProduct"],
  Cell[541934, 21688, 219, 5, 41, "Text",
   CellTags->"DotProduct"],
  Cell[542156, 21695, 312, 14, 24, "Text",
   CellTags->"DotProduct"],
  Cell[542505, 21714, 58, 1, 34, "Subsubsection",
   CellTags->"DotProduct"],
  Cell[542588, 21719, 234, 7, 26, "Input",
   CellTags->"DotProduct"],
  Cell[542825, 21728, 356, 12, 30, "Output",
   CellTags->"DotProduct"]},
 "DotSimplifyRelations"->{
  Cell[543208, 21744, 77, 1, 39, "ObjectName",
   CellTags->"DotSimplifyRelations"],
  Cell[543310, 21749, 71, 1, 34, "Subsubsection",
   CellTags->"DotSimplifyRelations"],
  Cell[543384, 21752, 241, 5, 41, "Text",
   CellTags->"DotSimplifyRelations"],
  Cell[543628, 21759, 194, 8, 24, "Text",
   CellTags->"DotSimplifyRelations"]},
 "DotSimplify"->{
  Cell[543837, 21770, 59, 1, 39, "ObjectName",
   CellTags->"DotSimplify"],
  Cell[543921, 21775, 62, 1, 34, "Subsubsection",
   CellTags->"DotSimplify"],
  Cell[543986, 21778, 322, 6, 58, "Text",
   CellTags->"DotSimplify"],
  Cell[544333, 21788, 121, 3, 26, "Input",
   CellTags->"DotSimplify"],
  Cell[544457, 21793, 423, 12, 26, "Output",
   CellTags->"DotSimplify"],
  Cell[544895, 21808, 414, 19, 24, "Text",
   CellTags->"DotSimplify"],
  Cell[545346, 21832, 59, 1, 34, "Subsubsection",
   CellTags->"DotSimplify"],
  Cell[545430, 21837, 342, 11, 26, "Input",
   CellTags->"DotSimplify"],
  Cell[545775, 21850, 418, 12, 28, "Output",
   CellTags->"DotSimplify"],
  Cell[546230, 21867, 116, 3, 26, "Input",
   CellTags->"DotSimplify"],
  Cell[546349, 21872, 1057, 40, 28, "Output",
   CellTags->"DotSimplify"],
  Cell[547421, 21915, 158, 4, 26, "Input",
   CellTags->"DotSimplify"],
  Cell[547604, 21923, 209, 7, 26, "Input",
   CellTags->"DotSimplify"],
  Cell[547816, 21932, 259, 9, 26, "Output",
   CellTags->"DotSimplify"],
  Cell[548112, 21946, 116, 3, 26, "Input",
   CellTags->"DotSimplify"],
  Cell[548231, 21951, 265, 8, 26, "Output",
   CellTags->"DotSimplify"],
  Cell[548533, 21964, 161, 5, 26, "Input",
   CellTags->"DotSimplify"],
  Cell[548697, 21971, 149, 4, 26, "Output",
   CellTags->"DotSimplify"],
  Cell[548883, 21980, 116, 3, 26, "Input",
   CellTags->"DotSimplify"],
  Cell[549002, 21985, 321, 11, 26, "Output",
   CellTags->"DotSimplify"],
  Cell[549338, 21999, 161, 5, 26, "Input",
   CellTags->"DotSimplify"],
  Cell[549524, 22008, 116, 3, 26, "Input",
   CellTags->"DotSimplify"],
  Cell[549643, 22013, 265, 8, 26, "Output",
   CellTags->"DotSimplify"],
  Cell[549945, 22026, 165, 5, 26, "Input",
   CellTags->"DotSimplify"],
  Cell[550113, 22033, 151, 4, 27, "Output",
   CellTags->"DotSimplify"],
  Cell[550301, 22042, 116, 3, 26, "Input",
   CellTags->"DotSimplify"],
  Cell[550420, 22047, 319, 10, 26, "Output",
   CellTags->"DotSimplify"],
  Cell[550754, 22060, 165, 5, 26, "Input",
   CellTags->"DotSimplify"],
  Cell[550944, 22069, 299, 9, 26, "Input",
   CellTags->"DotSimplify"],
  Cell[551246, 22080, 208, 7, 26, "Output",
   CellTags->"DotSimplify"],
  Cell[551469, 22090, 125, 3, 26, "Input",
   CellTags->"DotSimplify"],
  Cell[551619, 22097, 148, 4, 26, "Input",
   CellTags->"DotSimplify"],
  Cell[551770, 22103, 182, 5, 26, "Output",
   CellTags->"DotSimplify"],
  Cell[551989, 22113, 213, 6, 26, "Input",
   CellTags->"DotSimplify"],
  Cell[552205, 22121, 184, 5, 26, "Output",
   CellTags->"DotSimplify"],
  Cell[552404, 22129, 170, 4, 26, "Input",
   CellTags->"DotSimplify"]},
 "DummyIndex"->{
  Cell[552589, 22136, 57, 1, 39, "ObjectName",
   CellTags->"DummyIndex"],
  Cell[552671, 22141, 61, 1, 34, "Subsubsection",
   CellTags->"DummyIndex"],
  Cell[552735, 22144, 191, 4, 41, "Text",
   CellTags->"DummyIndex"],
  Cell[552929, 22150, 181, 8, 24, "Text",
   CellTags->"DummyIndex"]},
 "D0"->{
  Cell[553125, 22161, 41, 1, 39, "ObjectName",
   CellTags->"D0"],
  Cell[553191, 22166, 53, 1, 34, "Subsubsection",
   CellTags->"D0"],
  Cell[553247, 22169, 524, 12, 58, "Text",
   CellTags->"D0"],
  Cell[553774, 22183, 458, 24, 24, "Text",
   CellTags->"D0"],
  Cell[554269, 22212, 50, 1, 34, "Subsubsection",
   CellTags->"D0"],
  Cell[554344, 22217, 315, 9, 26, "Input",
   CellTags->"D0"],
  Cell[554662, 22228, 592, 21, 29, "Output",
   CellTags->"D0"],
  Cell[555291, 22254, 484, 14, 46, "Input",
   CellTags->"D0"],
  Cell[555778, 22270, 592, 21, 29, "Output",
   CellTags->"D0"],
  Cell[556407, 22296, 104, 3, 26, "Input",
   CellTags->"D0"],
  Cell[556514, 22301, 594, 21, 29, "Output",
   CellTags->"D0"]},
 "D0Convention"->{
  Cell[557135, 22326, 61, 1, 39, "ObjectName",
   CellTags->"D0Convention"],
  Cell[557221, 22331, 63, 1, 34, "Subsubsection",
   CellTags->"D0Convention"],
  Cell[557287, 22334, 307, 6, 41, "Text",
   CellTags->"D0Convention"],
  Cell[557597, 22342, 263, 13, 24, "Text",
   CellTags->"D0Convention"]},
 "Eps"->{
  Cell[557875, 22358, 43, 1, 39, "ObjectName",
   CellTags->"Eps"],
  Cell[557943, 22363, 54, 1, 34, "Subsubsection",
   CellTags->"Eps"],
  Cell[558000, 22366, 345, 8, 41, "Text",
   CellTags->"Eps"],
  Cell[558370, 22378, 105, 3, 26, "Input",
   CellTags->"Eps"],
  Cell[558478, 22383, 204, 6, 26, "Output",
   CellTags->"Eps"],
  Cell[558697, 22392, 486, 24, 24, "Text",
   CellTags->"Eps"],
  Cell[559220, 22421, 51, 1, 34, "Subsubsection",
   CellTags->"Eps"],
  Cell[559296, 22426, 322, 8, 26, "Input",
   CellTags->"Eps"],
  Cell[559621, 22436, 518, 22, 26, "Output",
   CellTags->"Eps"],
  Cell[560176, 22463, 314, 8, 26, "Input",
   CellTags->"Eps"],
  Cell[560493, 22473, 477, 20, 26, "Output",
   CellTags->"Eps"],
  Cell[561007, 22498, 181, 6, 26, "Input",
   CellTags->"Eps"],
  Cell[561191, 22506, 218, 6, 38, "Output",
   CellTags->"Eps"],
  Cell[561446, 22517, 142, 4, 26, "Input",
   CellTags->"Eps"],
  Cell[561591, 22523, 141, 4, 26, "Output",
   CellTags->"Eps"],
  Cell[561769, 22532, 142, 4, 26, "Input",
   CellTags->"Eps"],
  Cell[561914, 22538, 161, 5, 26, "Output",
   CellTags->"Eps"],
  Cell[562090, 22546, 189, 6, 26, "Input",
   CellTags->"Eps"],
  Cell[562304, 22556, 460, 13, 26, "Input",
   CellTags->"Eps"],
  Cell[562767, 22571, 519, 22, 26, "Output",
   CellTags->"Eps"],
  Cell[563323, 22598, 107, 3, 26, "Input",
   CellTags->"Eps"],
  Cell[563433, 22603, 144, 4, 26, "Output",
   CellTags->"Eps"],
  Cell[563592, 22610, 189, 6, 26, "Input",
   CellTags->"Eps"],
  Cell[563806, 22620, 460, 13, 26, "Input",
   CellTags->"Eps"],
  Cell[564269, 22635, 519, 22, 26, "Output",
   CellTags->"Eps"],
  Cell[564825, 22662, 137, 4, 26, "Input",
   CellTags->"Eps"],
  Cell[564965, 22668, 144, 4, 26, "Output",
   CellTags->"Eps"],
  Cell[567024, 22754, 433, 14, 50, "Input",
   CellTags->"Eps"],
  Cell[567460, 22770, 1163, 46, 44, "Output",
   CellTags->"Eps"],
  Cell[568660, 22821, 978, 28, 50, "Input",
   CellTags->"Eps"],
  Cell[569641, 22851, 1721, 62, 44, "Output",
   CellTags->"Eps"],
  Cell[571399, 22918, 188, 6, 26, "Input",
   CellTags->"Eps"],
  Cell[571590, 22926, 384, 13, 44, "Output",
   CellTags->"Eps"],
  Cell[572011, 22944, 124, 4, 26, "Input",
   CellTags->"Eps"],
  Cell[572138, 22950, 141, 4, 26, "Output",
   CellTags->"Eps"],
  Cell[572294, 22957, 164, 5, 26, "Input",
   CellTags->"Eps"]},
 "EpsChisholm"->{
  Cell[572473, 22965, 59, 1, 39, "ObjectName",
   CellTags->"EpsChisholm"],
  Cell[572557, 22970, 62, 1, 34, "Subsubsection",
   CellTags->"EpsChisholm"],
  Cell[572622, 22973, 166, 4, 24, "Text",
   CellTags->"EpsChisholm"],
  Cell[572791, 22979, 182, 9, 24, "Text",
   CellTags->"EpsChisholm"],
  Cell[573010, 22993, 59, 1, 34, "Subsubsection",
   CellTags->"EpsChisholm"],
  Cell[573094, 22998, 208, 6, 26, "Input",
   CellTags->"EpsChisholm"],
  Cell[573305, 23006, 2061, 79, 26, "Output",
   CellTags->"EpsChisholm"],
  Cell[575403, 23090, 115, 3, 26, "Input",
   CellTags->"EpsChisholm"],
  Cell[575521, 23095, 636, 25, 26, "Output",
   CellTags->"EpsChisholm"]},
 "EpsContract"->{
  Cell[576184, 23124, 59, 1, 39, "ObjectName",
   CellTags->"EpsContract"],
  Cell[576268, 23129, 62, 1, 34, "Subsubsection",
   CellTags->"EpsContract"],
  Cell[576333, 23132, 227, 5, 41, "Text",
   CellTags->"EpsContract"],
  Cell[576563, 23139, 277, 14, 24, "Text",
   CellTags->"EpsContract"],
  Cell[576877, 23158, 59, 1, 34, "Subsubsection",
   CellTags->"EpsContract"],
  Cell[576961, 23163, 468, 13, 26, "Input",
   CellTags->"EpsContract"],
  Cell[577432, 23178, 527, 22, 26, "Output",
   CellTags->"EpsContract"],
  Cell[577996, 23205, 205, 6, 26, "Input",
   CellTags->"EpsContract"],
  Cell[578204, 23213, 613, 24, 26, "Output",
   CellTags->"EpsContract"],
  Cell[578854, 23242, 204, 6, 26, "Input",
   CellTags->"EpsContract"],
  Cell[579061, 23250, 362, 12, 26, "Output",
   CellTags->"EpsContract"]},
 "EpsDiscard"->{
  Cell[579537, 23270, 57, 1, 39, "ObjectName",
   CellTags->"EpsDiscard"],
  Cell[579619, 23275, 61, 1, 34, "Subsubsection",
   CellTags->"EpsDiscard"],
  Cell[579683, 23278, 169, 4, 24, "Text",
   CellTags->"EpsDiscard"],
  Cell[579855, 23284, 196, 9, 24, "Text",
   CellTags->"EpsDiscard"]},
 "EpsEvaluate"->{
  Cell[580066, 23296, 59, 1, 39, "ObjectName",
   CellTags->"EpsEvaluate"],
  Cell[580150, 23301, 62, 1, 34, "Subsubsection",
   CellTags->"EpsEvaluate"],
  Cell[580215, 23304, 173, 4, 24, "Text",
   CellTags->"EpsEvaluate"],
  Cell[580391, 23310, 494, 24, 24, "Text",
   CellTags->"EpsEvaluate"],
  Cell[580922, 23339, 59, 1, 34, "Subsubsection",
   CellTags->"EpsEvaluate"],
  Cell[581006, 23344, 335, 10, 26, "Input",
   CellTags->"EpsEvaluate"],
  Cell[581344, 23356, 508, 21, 26, "Output",
   CellTags->"EpsEvaluate"],
  Cell[581889, 23382, 115, 3, 26, "Input",
   CellTags->"EpsEvaluate"],
  Cell[582007, 23387, 508, 21, 26, "Output",
   CellTags->"EpsEvaluate"],
  Cell[582552, 23413, 116, 3, 26, "Input",
   CellTags->"EpsEvaluate"],
  Cell[582671, 23418, 476, 13, 38, "Output",
   CellTags->"EpsEvaluate"]},
 "EpsUncontract"->{
  Cell[583174, 23435, 63, 1, 39, "ObjectName",
   CellTags->"EpsUncontract"],
  Cell[583262, 23440, 64, 1, 34, "Subsubsection",
   CellTags->"EpsUncontract"],
  Cell[583329, 23443, 115, 3, 24, "Text",
   CellTags->"EpsUncontract"],
  Cell[583447, 23448, 285, 14, 24, "Text",
   CellTags->"EpsUncontract"]},
 "Epsilon"->{
  Cell[583747, 23465, 51, 1, 39, "ObjectName",
   CellTags->"Epsilon"],
  Cell[583823, 23470, 58, 1, 34, "Subsubsection",
   CellTags->"Epsilon"],
  Cell[583884, 23473, 237, 9, 24, "Text",
   CellTags->"Epsilon"],
  Cell[584124, 23484, 175, 9, 24, "Text",
   CellTags->"Epsilon"],
  Cell[584336, 23498, 55, 1, 34, "Subsubsection",
   CellTags->"Epsilon"],
  Cell[584416, 23503, 80, 2, 26, "Input",
   CellTags->"Epsilon"],
  Cell[584499, 23507, 190, 6, 26, "Output",
   CellTags->"Epsilon"],
  Cell[584704, 23516, 114, 3, 24, "Text",
   CellTags->"Epsilon"],
  Cell[584843, 23523, 587, 19, 46, "Input",
   CellTags->"Epsilon"],
  Cell[585433, 23544, 241, 7, 26, "Output",
   CellTags->"Epsilon"]},
 "EpsilonOrder"->{
  Cell[585701, 23555, 61, 1, 39, "ObjectName",
   CellTags->"EpsilonOrder"],
  Cell[585787, 23560, 63, 1, 34, "Subsubsection",
   CellTags->"EpsilonOrder"],
  Cell[585853, 23563, 305, 12, 24, "Text",
   CellTags->"EpsilonOrder"],
  Cell[586161, 23577, 287, 14, 24, "Text",
   CellTags->"EpsilonOrder"]},
 "Expanding"->{
  Cell[586463, 23594, 55, 1, 39, "ObjectName",
   CellTags->"Expanding"],
  Cell[586543, 23599, 60, 1, 34, "Subsubsection",
   CellTags->"Expanding"],
  Cell[586606, 23602, 378, 7, 58, "Text",
   CellTags->"Expanding"],
  Cell[586987, 23611, 641, 29, 24, "Text",
   CellTags->"Expanding"]},
 "ExpandPartialD"->{
  Cell[587643, 23643, 65, 1, 39, "ObjectName",
   CellTags->"ExpandPartialD"],
  Cell[587733, 23648, 65, 1, 34, "Subsubsection",
   CellTags->"ExpandPartialD"],
  Cell[587801, 23651, 188, 4, 24, "Text",
   CellTags->"ExpandPartialD"],
  Cell[589387, 23703, 724, 28, 24, "Text",
   CellTags->"ExpandPartialD"],
  Cell[590148, 23736, 62, 1, 34, "Subsubsection",
   CellTags->"ExpandPartialD"],
  Cell[590235, 23741, 373, 10, 26, "Input",
   CellTags->"ExpandPartialD"],
  Cell[590611, 23753, 387, 13, 33, "Output",
   CellTags->"ExpandPartialD"],
  Cell[591035, 23771, 121, 3, 26, "Input",
   CellTags->"ExpandPartialD"],
  Cell[591159, 23776, 1253, 31, 150, "Output",
   CellTags->"ExpandPartialD"],
  Cell[592449, 23812, 115, 3, 26, "Input",
   CellTags->"ExpandPartialD"],
  Cell[592567, 23817, 1400, 30, 210, "Output",
   CellTags->"ExpandPartialD"],
  Cell[594004, 23852, 259, 7, 26, "Input",
   CellTags->"ExpandPartialD"],
  Cell[594266, 23861, 354, 12, 33, "Output",
   CellTags->"ExpandPartialD"],
  Cell[594657, 23878, 121, 3, 26, "Input",
   CellTags->"ExpandPartialD"],
  Cell[594781, 23883, 962, 27, 117, "Output",
   CellTags->"ExpandPartialD"],
  Cell[595780, 23915, 433, 13, 46, "Input",
   CellTags->"ExpandPartialD"],
  Cell[596216, 23930, 376, 11, 35, "Output",
   CellTags->"ExpandPartialD"],
  Cell[596629, 23946, 121, 3, 26, "Input",
   CellTags->"ExpandPartialD"],
  Cell[596753, 23951, 3062, 66, 411, "Output",
   CellTags->"ExpandPartialD"],
  Cell[599852, 24022, 180, 6, 26, "Input",
   CellTags->"ExpandPartialD"],
  Cell[600035, 24030, 308, 9, 34, "Output",
   CellTags->"ExpandPartialD"],
  Cell[600380, 24044, 123, 3, 26, "Input",
   CellTags->"ExpandPartialD"],
  Cell[600506, 24049, 480, 15, 38, "Output",
   CellTags->"ExpandPartialD"],
  Cell[601023, 24069, 121, 3, 26, "Input",
   CellTags->"ExpandPartialD"],
  Cell[601147, 24074, 1919, 60, 30, "Output",
   CellTags->"ExpandPartialD"],
  Cell[603103, 24139, 314, 9, 26, "Input",
   CellTags->"ExpandPartialD"],
  Cell[603420, 24150, 1234, 48, 33, "Output",
   CellTags->"ExpandPartialD"],
  Cell[604691, 24203, 121, 3, 26, "Input",
   CellTags->"ExpandPartialD"],
  Cell[604815, 24208, 152, 4, 26, "Output",
   CellTags->"ExpandPartialD"]},
 "ScalarProductCancel"->{
  Cell[604994, 24216, 121, 1, 39, "ObjectName",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[605140, 24221, 95, 1, 34, "Subsubsection",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[605238, 24224, 252, 5, 49, "Text",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[605515, 24233, 158, 3, 26, "Input",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[605676, 24238, 256, 7, 26, "Output",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[605947, 24248, 462, 19, 24, "Text",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[606446, 24272, 92, 1, 34, "Subsubsection",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[606563, 24277, 257, 7, 26, "Input",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[606823, 24286, 380, 11, 26, "Output",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[607240, 24302, 152, 3, 26, "Input",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[607395, 24307, 382, 11, 26, "Output",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[607814, 24323, 186, 5, 26, "Input",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[608003, 24330, 319, 10, 28, "Output",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[608359, 24345, 156, 3, 26, "Input",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[608518, 24350, 319, 10, 28, "Output",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[608874, 24365, 190, 5, 26, "Input",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[609067, 24372, 271, 8, 28, "Output",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[609375, 24385, 156, 3, 26, "Input",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[609534, 24390, 269, 8, 28, "Output",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[609840, 24403, 239, 7, 26, "Input",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[610082, 24412, 354, 11, 28, "Output",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[610473, 24428, 184, 4, 26, "Input",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[610660, 24434, 607, 23, 28, "Output",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[1668121, 65832, 75, 1, 39, "ObjectName",
   CellTags->"ScalarProductCancel"],
  Cell[1668221, 65837, 70, 1, 34, "Subsubsection",
   CellTags->"ScalarProductCancel"],
  Cell[1668294, 65840, 186, 4, 24, "Text",
   CellTags->"ScalarProductCancel"],
  Cell[1668505, 65848, 89, 2, 26, "Input",
   CellTags->"ScalarProductCancel"],
  Cell[1668597, 65852, 178, 4, 26, "Output",
   CellTags->"ScalarProductCancel"],
  Cell[1668812, 65861, 134, 3, 26, "Input",
   CellTags->"ScalarProductCancel"],
  Cell[1668949, 65866, 436, 11, 26, "Output",
   CellTags->"ScalarProductCancel"],
  Cell[1669422, 65882, 106, 3, 26, "Input",
   CellTags->"ScalarProductCancel"],
  Cell[1669531, 65887, 221, 4, 39, "Print",
   CellTags->{"Info3500973967-4982097", "ScalarProductCancel"}],
  Cell[1669767, 65894, 743, 33, 24, "Text",
   CellTags->"ScalarProductCancel"],
  Cell[1670547, 65932, 67, 1, 34, "Subsubsection",
   CellTags->"ScalarProductCancel"],
  Cell[1670617, 65935, 244, 9, 24, "Text",
   CellTags->"ScalarProductCancel"],
  Cell[1670886, 65948, 429, 15, 26, "Input",
   CellTags->"ScalarProductCancel"],
  Cell[1671318, 65965, 498, 18, 47, "Output",
   CellTags->"ScalarProductCancel"],
  Cell[1671853, 65988, 156, 4, 26, "Input",
   CellTags->"ScalarProductCancel"],
  Cell[1672012, 65994, 891, 33, 59, "Output",
   CellTags->"ScalarProductCancel"],
  Cell[1672940, 66032, 162, 4, 26, "Input",
   CellTags->"ScalarProductCancel"],
  Cell[1673105, 66038, 750, 27, 59, "Output",
   CellTags->"ScalarProductCancel"],
  Cell[1673892, 66070, 185, 5, 26, "Input",
   CellTags->"ScalarProductCancel"],
  Cell[1674080, 66077, 752, 27, 59, "Output",
   CellTags->"ScalarProductCancel"],
  Cell[1674869, 66109, 578, 18, 26, "Input",
   CellTags->"ScalarProductCancel"],
  Cell[1675450, 66129, 940, 34, 50, "Output",
   CellTags->"ScalarProductCancel"],
  Cell[1676427, 66168, 197, 5, 26, "Input",
   CellTags->"ScalarProductCancel"],
  Cell[1676627, 66175, 3150, 112, 178, "Output",
   CellTags->"ScalarProductCancel"],
  Cell[1679792, 66290, 162, 5, 26, "Input",
   CellTags->"ScalarProductCancel"]},
 "ExpandScalarProduct"->{
  Cell[604994, 24216, 121, 1, 39, "ObjectName",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[605140, 24221, 95, 1, 34, "Subsubsection",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[605238, 24224, 252, 5, 49, "Text",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[605515, 24233, 158, 3, 26, "Input",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[605676, 24238, 256, 7, 26, "Output",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[605947, 24248, 462, 19, 24, "Text",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[606446, 24272, 92, 1, 34, "Subsubsection",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[606563, 24277, 257, 7, 26, "Input",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[606823, 24286, 380, 11, 26, "Output",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[607240, 24302, 152, 3, 26, "Input",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[607395, 24307, 382, 11, 26, "Output",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[607814, 24323, 186, 5, 26, "Input",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[608003, 24330, 319, 10, 28, "Output",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[608359, 24345, 156, 3, 26, "Input",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[608518, 24350, 319, 10, 28, "Output",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[608874, 24365, 190, 5, 26, "Input",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[609067, 24372, 271, 8, 28, "Output",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[609375, 24385, 156, 3, 26, "Input",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[609534, 24390, 269, 8, 28, "Output",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[609840, 24403, 239, 7, 26, "Input",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[610082, 24412, 354, 11, 28, "Output",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[610473, 24428, 184, 4, 26, "Input",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[610660, 24434, 607, 23, 28, "Output",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}]},
 "Expand2"->{
  Cell[611294, 24461, 51, 1, 39, "ObjectName",
   CellTags->"Expand2"],
  Cell[611370, 24466, 58, 1, 34, "Subsubsection",
   CellTags->"Expand2"],
  Cell[611431, 24469, 164, 4, 24, "Text",
   CellTags->"Expand2"],
  Cell[611632, 24478, 55, 1, 34, "Subsubsection",
   CellTags->"Expand2"],
  Cell[611712, 24483, 579, 21, 26, "Input",
   CellTags->"Expand2"],
  Cell[612294, 24506, 618, 20, 26, "Output",
   CellTags->"Expand2"]},
 "Explicit"->{
  Cell[612939, 24530, 53, 1, 39, "ObjectName",
   CellTags->"Explicit"],
  Cell[613017, 24535, 59, 1, 34, "Subsubsection",
   CellTags->"Explicit"],
  Cell[613079, 24538, 401, 9, 41, "Text",
   CellTags->"Explicit"],
  Cell[613483, 24549, 331, 14, 24, "Text",
   CellTags->"Explicit"],
  Cell[614368, 24588, 56, 1, 34, "Subsubsection",
   CellTags->"Explicit"],
  Cell[616350, 24670, 230, 6, 26, "Input",
   CellTags->"Explicit"],
  Cell[616583, 24678, 982, 43, 28, "Output",
   CellTags->"Explicit"],
  Cell[617602, 24726, 109, 3, 26, "Input",
   CellTags->"Explicit"],
  Cell[617714, 24731, 2013, 80, 28, "Output",
   CellTags->"Explicit"],
  Cell[619764, 24816, 184, 5, 26, "Input",
   CellTags->"Explicit"],
  Cell[619951, 24823, 936, 37, 44, "Output",
   CellTags->"Explicit"],
  Cell[620924, 24865, 109, 3, 26, "Input",
   CellTags->"Explicit"],
  Cell[621036, 24870, 2686, 103, 44, "Output",
   CellTags->"Explicit"],
  Cell[623759, 24978, 155, 4, 26, "Input",
   CellTags->"Explicit"],
  Cell[623917, 24984, 204, 6, 30, "Output",
   CellTags->"Explicit"],
  Cell[624158, 24995, 109, 3, 26, "Input",
   CellTags->"Explicit"],
  Cell[624270, 25000, 2106, 70, 152, "Output",
   CellTags->"Explicit"]},
 "ExplicitLorentzIndex"->{
  Cell[626403, 25074, 77, 1, 39, "ObjectName",
   CellTags->"ExplicitLorentzIndex"],
  Cell[626505, 25079, 71, 1, 34, "Subsubsection",
   CellTags->"ExplicitLorentzIndex"],
  Cell[626579, 25082, 141, 4, 24, "Text",
   CellTags->"ExplicitLorentzIndex"],
  Cell[626723, 25088, 295, 13, 24, "Text",
   CellTags->"ExplicitLorentzIndex"],
  Cell[627055, 25106, 68, 1, 34, "Subsubsection",
   CellTags->"ExplicitLorentzIndex"],
  Cell[627148, 25111, 224, 6, 26, "Input",
   CellTags->"ExplicitLorentzIndex"],
  Cell[627375, 25119, 241, 8, 26, "Output",
   CellTags->"ExplicitLorentzIndex"],
  Cell[627653, 25132, 121, 3, 26, "Input",
   CellTags->"ExplicitLorentzIndex"],
  Cell[627777, 25137, 289, 7, 38, "Output",
   CellTags->"ExplicitLorentzIndex"]},
 "ExplicitPartialD"->{
  Cell[628093, 25148, 69, 1, 39, "ObjectName",
   CellTags->"ExplicitPartialD"],
  Cell[628187, 25153, 67, 1, 34, "Subsubsection",
   CellTags->"ExplicitPartialD"],
  Cell[628257, 25156, 162, 4, 24, "Text",
   CellTags->"ExplicitPartialD"],
  Cell[628422, 25162, 476, 18, 24, "Text",
   CellTags->"ExplicitPartialD"],
  Cell[628935, 25185, 64, 1, 34, "Subsubsection",
   CellTags->"ExplicitPartialD"],
  Cell[629024, 25190, 183, 4, 26, "Input",
   CellTags->"ExplicitPartialD"],
  Cell[629210, 25196, 567, 20, 44, "Output",
   CellTags->"ExplicitPartialD"],
  Cell[629814, 25221, 184, 4, 26, "Input",
   CellTags->"ExplicitPartialD"],
  Cell[630001, 25227, 474, 17, 33, "Output",
   CellTags->"ExplicitPartialD"],
  Cell[630512, 25249, 186, 4, 26, "Input",
   CellTags->"ExplicitPartialD"],
  Cell[630701, 25255, 502, 16, 44, "Output",
   CellTags->"ExplicitPartialD"],
  Cell[631240, 25276, 183, 6, 26, "Input",
   CellTags->"ExplicitPartialD"],
  Cell[631426, 25284, 312, 9, 34, "Output",
   CellTags->"ExplicitPartialD"],
  Cell[631775, 25298, 125, 3, 26, "Input",
   CellTags->"ExplicitPartialD"],
  Cell[631903, 25303, 482, 15, 38, "Output",
   CellTags->"ExplicitPartialD"]},
 "ExplicitSUNIndex"->{
  Cell[632412, 25322, 69, 1, 39, "ObjectName",
   CellTags->"ExplicitSUNIndex"],
  Cell[632506, 25327, 67, 1, 34, "Subsubsection",
   CellTags->"ExplicitSUNIndex"],
  Cell[632576, 25330, 178, 6, 24, "Text",
   CellTags->"ExplicitSUNIndex"],
  Cell[632757, 25338, 484, 23, 24, "Text",
   CellTags->"ExplicitSUNIndex"],
  Cell[633278, 25366, 64, 1, 34, "Subsubsection",
   CellTags->"ExplicitSUNIndex"],
  Cell[633367, 25371, 125, 3, 26, "Input",
   CellTags->"ExplicitSUNIndex"],
  Cell[633495, 25376, 154, 4, 26, "Output",
   CellTags->"ExplicitSUNIndex"]},
 "ExtraFactor"->{
  Cell[634138, 25403, 59, 1, 39, "ObjectName",
   CellTags->"ExtraFactor"],
  Cell[634222, 25408, 62, 1, 34, "Subsubsection",
   CellTags->"ExtraFactor"],
  Cell[634287, 25411, 199, 4, 24, "Text",
   CellTags->"ExtraFactor"],
  Cell[634489, 25417, 194, 8, 24, "Text",
   CellTags->"ExtraFactor"]},
 "ExtraVariables"->{
  Cell[634698, 25428, 65, 1, 39, "ObjectName",
   CellTags->"ExtraVariables"],
  Cell[634788, 25433, 65, 1, 34, "Subsubsection",
   CellTags->"ExtraVariables"],
  Cell[634856, 25436, 213, 5, 41, "Text",
   CellTags->"ExtraVariables"],
  Cell[635072, 25443, 298, 14, 24, "Text",
   CellTags->"ExtraVariables"]},
 "FactorFull"->{
  Cell[635385, 25460, 57, 1, 39, "ObjectName",
   CellTags->"FactorFull"],
  Cell[635467, 25465, 61, 1, 36, "Subsubsection",
   CellTags->"FactorFull"],
  Cell[635531, 25468, 509, 24, 25, "Text",
   CellTags->"FactorFull"],
  Cell[636043, 25494, 178, 9, 25, "Text",
   CellTags->"FactorFull"]},
 "Factoring"->{
  Cell[636236, 25506, 55, 1, 41, "ObjectName",
   CellTags->"Factoring"],
  Cell[636316, 25511, 60, 1, 34, "Subsubsection",
   CellTags->"Factoring"],
  Cell[636379, 25514, 235, 5, 41, "Text",
   CellTags->"Factoring"],
  Cell[636617, 25521, 382, 19, 24, "Text",
   CellTags->"Factoring"]},
 "Factorout"->{
  Cell[637014, 25543, 55, 1, 39, "ObjectName",
   CellTags->"Factorout"],
  Cell[637094, 25548, 60, 1, 34, "Subsubsection",
   CellTags->"Factorout"],
  Cell[637157, 25551, 74, 1, 24, "Text",
   CellTags->"Factorout"],
  Cell[637234, 25554, 174, 9, 24, "Text",
   CellTags->"Factorout"]},
 "FactorTime"->{
  Cell[637423, 25566, 57, 1, 39, "ObjectName",
   CellTags->"FactorTime"],
  Cell[637505, 25571, 61, 1, 34, "Subsubsection",
   CellTags->"FactorTime"],
  Cell[637569, 25574, 166, 4, 24, "Text",
   CellTags->"FactorTime"],
  Cell[637738, 25580, 178, 9, 24, "Text",
   CellTags->"FactorTime"]},
 "Factor1"->{
  Cell[637931, 25592, 51, 1, 39, "ObjectName",
   CellTags->"Factor1"],
  Cell[638007, 25597, 58, 1, 34, "Subsubsection",
   CellTags->"Factor1"],
  Cell[638068, 25600, 144, 4, 24, "Text",
   CellTags->"Factor1"],
  Cell[638215, 25606, 175, 9, 24, "Text",
   CellTags->"Factor1"],
  Cell[638427, 25620, 55, 1, 34, "Subsubsection",
   CellTags->"Factor1"],
  Cell[638507, 25625, 213, 8, 26, "Input",
   CellTags->"Factor1"],
  Cell[638723, 25635, 268, 9, 26, "Output",
   CellTags->"Factor1"],
  Cell[639028, 25649, 222, 7, 26, "Input",
   CellTags->"Factor1"],
  Cell[639253, 25658, 477, 17, 26, "Output",
   CellTags->"Factor1"],
  Cell[639767, 25680, 251, 9, 26, "Input",
   CellTags->"Factor1"],
  Cell[640021, 25691, 218, 7, 26, "Output",
   CellTags->"Factor1"],
  Cell[640276, 25703, 107, 3, 26, "Input",
   CellTags->"Factor1"],
  Cell[640386, 25708, 266, 9, 26, "Output",
   CellTags->"Factor1"],
  Cell[640689, 25722, 108, 3, 26, "Input",
   CellTags->"Factor1"],
  Cell[640800, 25727, 218, 7, 26, "Output",
   CellTags->"Factor1"],
  Cell[641033, 25737, 141, 4, 26, "Input",
   CellTags->"Factor1"]},
 "Factor2"->{
  Cell[641189, 25744, 51, 1, 39, "ObjectName",
   CellTags->"Factor2"],
  Cell[641265, 25749, 58, 1, 36, "Subsubsection",
   CellTags->"Factor2"],
  Cell[641326, 25752, 398, 7, 58, "Text",
   CellTags->"Factor2"],
  Cell[641749, 25763, 113, 3, 26, "Input",
   CellTags->"Factor2"],
  Cell[641865, 25768, 270, 8, 26, "Output",
   CellTags->"Factor2"],
  Cell[642150, 25779, 178, 9, 25, "Text",
   CellTags->"Factor2"],
  Cell[642365, 25793, 55, 1, 34, "Subsubsection",
   CellTags->"Factor2"],
  Cell[642445, 25798, 213, 8, 26, "Input",
   CellTags->"Factor2"],
  Cell[642661, 25808, 268, 9, 26, "Output",
   CellTags->"Factor2"],
  Cell[642966, 25822, 222, 7, 26, "Input",
   CellTags->"Factor2"],
  Cell[643191, 25831, 477, 17, 26, "Output",
   CellTags->"Factor2"],
  Cell[643705, 25853, 251, 9, 26, "Input",
   CellTags->"Factor2"],
  Cell[643959, 25864, 218, 7, 26, "Output",
   CellTags->"Factor2"],
  Cell[644214, 25876, 107, 3, 26, "Input",
   CellTags->"Factor2"],
  Cell[644324, 25881, 266, 9, 26, "Output",
   CellTags->"Factor2"],
  Cell[644627, 25895, 108, 3, 26, "Input",
   CellTags->"Factor2"],
  Cell[644738, 25900, 220, 7, 26, "Output",
   CellTags->"Factor2"],
  Cell[644995, 25912, 173, 5, 26, "Input",
   CellTags->"Factor2"],
  Cell[645171, 25919, 266, 9, 26, "Output",
   CellTags->"Factor2"],
  Cell[645452, 25931, 141, 4, 26, "Input",
   CellTags->"Factor2"]},
 "FAD"->{
  Cell[645608, 25938, 43, 1, 70, "ObjectName",
   CellTags->"FAD"],
  Cell[645676, 25943, 54, 1, 70, "Subsubsection",
   CellTags->"FAD"],
  Cell[645733, 25946, 335, 6, 70, "Text",
   CellTags->"FAD"],
  Cell[646071, 25954, 802, 34, 70, "Text",
   CellTags->"FAD"],
  Cell[646910, 25993, 51, 1, 34, "Subsubsection",
   CellTags->"FAD"],
  Cell[646986, 25998, 146, 5, 26, "Input",
   CellTags->"FAD"],
  Cell[647135, 26005, 742, 21, 51, "Output",
   CellTags->"FAD"],
  Cell[647914, 26031, 197, 7, 26, "Input",
   CellTags->"FAD"],
  Cell[648114, 26040, 860, 23, 51, "Output",
   CellTags->"FAD"],
  Cell[649011, 26068, 245, 9, 26, "Input",
   CellTags->"FAD"],
  Cell[649259, 26079, 201, 6, 38, "Output",
   CellTags->"FAD"],
  Cell[649497, 26090, 215, 8, 26, "Input",
   CellTags->"FAD"],
  Cell[649715, 26100, 567, 16, 58, "Output",
   CellTags->"FAD"],
  Cell[650319, 26121, 280, 9, 26, "Input",
   CellTags->"FAD"],
  Cell[650602, 26132, 571, 17, 58, "Output",
   CellTags->"FAD"]},
 "FC"->{
  Cell[651200, 26153, 41, 1, 39, "ObjectName",
   CellTags->"FC"],
  Cell[651266, 26158, 53, 1, 34, "Subsubsection",
   CellTags->"FC"],
  Cell[651322, 26161, 117, 3, 24, "Text",
   CellTags->"FC"],
  Cell[651442, 26166, 545, 24, 24, "Text",
   CellTags->"FC"],
  Cell[652024, 26195, 50, 1, 34, "Subsubsection",
   CellTags->"FC"],
  Cell[652077, 26198, 70, 2, 26, "Input",
   CellTags->"FC"],
  Cell[652172, 26204, 227, 7, 26, "Input",
   CellTags->"FC"],
  Cell[652402, 26213, 141, 3, 25, "Output",
   CellTags->"FC"],
  Cell[652558, 26219, 70, 2, 26, "Input",
   CellTags->"FC"],
  Cell[652653, 26225, 227, 7, 26, "Input",
   CellTags->"FC"],
  Cell[652883, 26234, 347, 12, 29, "Output",
   CellTags->"FC"]},
 "FCE"->{
  Cell[653257, 26250, 43, 1, 39, "ObjectName",
   CellTags->"FCE"],
  Cell[653325, 26255, 54, 1, 34, "Subsubsection",
   CellTags->"FCE"],
  Cell[653382, 26258, 126, 4, 24, "Text",
   CellTags->"FCE"],
  Cell[653511, 26264, 72, 1, 24, "Text",
   CellTags->"FCE"],
  Cell[653586, 26267, 427, 19, 24, "Text",
   CellTags->"FCE"],
  Cell[654050, 26291, 51, 1, 34, "Subsubsection",
   CellTags->"FCE"],
  Cell[654126, 26296, 262, 8, 26, "Input",
   CellTags->"FCE"],
  Cell[654391, 26306, 267, 9, 29, "Output",
   CellTags->"FCE"],
  Cell[654695, 26320, 104, 3, 26, "Input",
   CellTags->"FCE"],
  Cell[654802, 26325, 232, 7, 38, "Output",
   CellTags->"FCE"],
  Cell[655071, 26337, 437, 13, 26, "Input",
   CellTags->"FCE"],
  Cell[655511, 26352, 589, 19, 29, "Output",
   CellTags->"FCE"],
  Cell[656137, 26376, 104, 3, 26, "Input",
   CellTags->"FCE"],
  Cell[656244, 26381, 494, 14, 38, "Output",
   CellTags->"FCE"],
  Cell[656775, 26400, 95, 3, 26, "Input",
   CellTags->"FCE"],
  Cell[656873, 26405, 904, 35, 29, "Output",
   CellTags->"FCE"],
  Cell[657814, 26445, 104, 3, 26, "Input",
   CellTags->"FCE"],
  Cell[657921, 26450, 935, 25, 77, "Output",
   CellTags->"FCE"],
  Cell[658893, 26480, 134, 4, 26, "Input",
   CellTags->"FCE"],
  Cell[659030, 26486, 494, 14, 38, "Output",
   CellTags->"FCE"]},
 "FCI"->{
  Cell[659551, 26504, 43, 1, 39, "ObjectName",
   CellTags->"FCI"],
  Cell[659619, 26509, 54, 1, 34, "Subsubsection",
   CellTags->"FCI"],
  Cell[659676, 26512, 159, 4, 24, "Text",
   CellTags->"FCI"],
  Cell[659838, 26518, 427, 19, 24, "Text",
   CellTags->"FCI"],
  Cell[660302, 26542, 51, 1, 34, "Subsubsection",
   CellTags->"FCI"],
  Cell[660378, 26547, 437, 13, 26, "Input",
   CellTags->"FCI"],
  Cell[660818, 26562, 591, 19, 29, "Output",
   CellTags->"FCI"],
  Cell[661446, 26586, 104, 3, 26, "Input",
   CellTags->"FCI"],
  Cell[661553, 26591, 493, 14, 38, "Output",
   CellTags->"FCI"],
  Cell[662083, 26610, 95, 3, 26, "Input",
   CellTags->"FCI"],
  Cell[662181, 26615, 906, 35, 29, "Output",
   CellTags->"FCI"],
  Cell[663124, 26655, 104, 3, 26, "Input",
   CellTags->"FCI"],
  Cell[663231, 26660, 935, 25, 77, "Output",
   CellTags->"FCI"],
  Cell[664203, 26690, 134, 4, 26, "Input",
   CellTags->"FCI"],
  Cell[664340, 26696, 493, 14, 38, "Output",
   CellTags->"FCI"]},
 "FCIntegral"->{
  Cell[664860, 26714, 57, 1, 39, "ObjectName",
   CellTags->"FCIntegral"],
  Cell[664942, 26719, 61, 1, 34, "Subsubsection",
   CellTags->"FCIntegral"],
  Cell[665006, 26722, 209, 5, 41, "Text",
   CellTags->"FCIntegral"],
  Cell[665218, 26729, 360, 14, 24, "Text",
   CellTags->"FCIntegral"]},
 "FCIntegrate"->{
  Cell[665593, 26746, 59, 1, 39, "ObjectName",
   CellTags->"FCIntegrate"],
  Cell[665677, 26751, 62, 1, 34, "Subsubsection",
   CellTags->"FCIntegrate"],
  Cell[665742, 26754, 285, 5, 41, "Text",
   CellTags->"FCIntegrate"],
  Cell[666030, 26761, 194, 9, 24, "Text",
   CellTags->"FCIntegrate"]},
 "FCNIntegrate"->{
  Cell[666239, 26773, 61, 1, 39, "ObjectName",
   CellTags->"FCNIntegrate"],
  Cell[666325, 26778, 63, 1, 34, "Subsubsection",
   CellTags->"FCNIntegrate"],
  Cell[666391, 26781, 418, 7, 58, "Text",
   CellTags->"FCNIntegrate"],
  Cell[666812, 26790, 192, 9, 24, "Text",
   CellTags->"FCNIntegrate"]},
 "FC2RHI"->{
  Cell[667019, 26802, 50, 1, 39, "ObjectName",
   CellTags->"FC2RHI"],
  Cell[667094, 26807, 57, 1, 34, "Subsubsection",
   CellTags->"FC2RHI"],
  Cell[667154, 26810, 306, 6, 41, "Text",
   CellTags->"FC2RHI"],
  Cell[667485, 26820, 111, 3, 26, "Input",
   CellTags->"FC2RHI"],
  Cell[667599, 26825, 319, 9, 26, "Output",
   CellTags->"FC2RHI"],
  Cell[669805, 26917, 162, 9, 24, "Text",
   CellTags->"FC2RHI"],
  Cell[670004, 26931, 54, 1, 34, "Subsubsection",
   CellTags->"FC2RHI"],
  Cell[670061, 26934, 72, 2, 26, "Input",
   CellTags->"FC2RHI"]},
 "FC2TLI"->{
  Cell[670148, 26939, 50, 1, 39, "ObjectName",
   CellTags->"FC2TLI"],
  Cell[670223, 26944, 57, 1, 34, "Subsubsection",
   CellTags->"FC2TLI"],
  Cell[670283, 26947, 256, 5, 41, "Text",
   CellTags->"FC2TLI"],
  Cell[670564, 26956, 111, 3, 26, "Input",
   CellTags->"FC2TLI"],
  Cell[670678, 26961, 319, 9, 26, "Output",
   CellTags->"FC2TLI"],
  Cell[675341, 27132, 162, 9, 24, "Text",
   CellTags->"FC2TLI"],
  Cell[675540, 27146, 54, 1, 34, "Subsubsection",
   CellTags->"FC2TLI"],
  Cell[675597, 27149, 72, 2, 26, "Input",
   CellTags->"FC2TLI"]},
 "Info3500973829-4982097"->{
  Cell[671108, 26979, 365, 6, 117, "Print",
   CellTags->"Info3500973829-4982097"]},
 "FDS"->{
  Cell[675684, 27154, 43, 1, 39, "ObjectName",
   CellTags->"FDS"],
  Cell[675752, 27159, 54, 1, 34, "Subsubsection",
   CellTags->"FDS"],
  Cell[675809, 27162, 82, 1, 24, "Text",
   CellTags->"FDS"],
  Cell[675894, 27165, 228, 9, 24, "Text",
   CellTags->"FDS"]},
 "FermionSpinSum"->{
  Cell[676137, 27177, 65, 1, 39, "ObjectName",
   CellTags->"FermionSpinSum"],
  Cell[676227, 27182, 65, 1, 34, "Subsubsection",
   CellTags->"FermionSpinSum"],
  Cell[676295, 27185, 118, 3, 24, "Text",
   CellTags->"FermionSpinSum"],
  Cell[676438, 27192, 127, 3, 26, "Input",
   CellTags->"FermionSpinSum"],
  Cell[676568, 27197, 356, 10, 26, "Output",
   CellTags->"FermionSpinSum"],
  Cell[676939, 27210, 521, 24, 25, "Text",
   CellTags->"FermionSpinSum"],
  Cell[677497, 27239, 62, 1, 36, "Subsubsection",
   CellTags->"FermionSpinSum"],
  Cell[677562, 27242, 261, 12, 24, "Text",
   CellTags->"FermionSpinSum"],
  Cell[677848, 27258, 296, 9, 26, "Input",
   CellTags->"FermionSpinSum"],
  Cell[678147, 27269, 395, 14, 28, "Output",
   CellTags->"FermionSpinSum"],
  Cell[678579, 27288, 143, 4, 26, "Input",
   CellTags->"FermionSpinSum"],
  Cell[678725, 27294, 372, 13, 28, "Output",
   CellTags->"FermionSpinSum"],
  Cell[679134, 27312, 143, 4, 26, "Input",
   CellTags->"FermionSpinSum"],
  Cell[679280, 27318, 175, 5, 26, "Output",
   CellTags->"FermionSpinSum"],
  Cell[679492, 27328, 296, 9, 26, "Input",
   CellTags->"FermionSpinSum"],
  Cell[679791, 27339, 395, 14, 28, "Output",
   CellTags->"FermionSpinSum"],
  Cell[680223, 27358, 143, 4, 26, "Input",
   CellTags->"FermionSpinSum"],
  Cell[680369, 27364, 372, 13, 28, "Output",
   CellTags->"FermionSpinSum"],
  Cell[680778, 27382, 143, 4, 26, "Input",
   CellTags->"FermionSpinSum"],
  Cell[680924, 27388, 196, 6, 26, "Output",
   CellTags->"FermionSpinSum"],
  Cell[681157, 27399, 334, 10, 26, "Input",
   CellTags->"FermionSpinSum"],
  Cell[681494, 27411, 550, 18, 28, "Output",
   CellTags->"FermionSpinSum"],
  Cell[682059, 27432, 164, 4, 24, "Text",
   CellTags->"FermionSpinSum"],
  Cell[682248, 27440, 147, 4, 26, "Input",
   CellTags->"FermionSpinSum"],
  Cell[682398, 27446, 582, 19, 28, "Output",
   CellTags->"FermionSpinSum"],
  Cell[683017, 27470, 145, 4, 26, "Input",
   CellTags->"FermionSpinSum"],
  Cell[683165, 27476, 1116, 37, 29, "Output",
   CellTags->"FermionSpinSum"],
  Cell[684318, 27518, 148, 4, 26, "Input",
   CellTags->"FermionSpinSum"],
  Cell[684469, 27524, 879, 33, 29, "Output",
   CellTags->"FermionSpinSum"],
  Cell[685363, 27560, 141, 4, 26, "Input",
   CellTags->"FermionSpinSum"]},
 "FeynAmp"->{
  Cell[685519, 27567, 51, 1, 41, "ObjectName",
   CellTags->"FeynAmp"],
  Cell[685595, 27572, 58, 1, 36, "Subsubsection",
   CellTags->"FeynAmp"],
  Cell[685656, 27575, 243, 5, 43, "Text",
   CellTags->"FeynAmp"],
  Cell[685902, 27582, 165, 4, 25, "Text",
   CellTags->"FeynAmp"],
  Cell[686070, 27588, 181, 9, 25, "Text",
   CellTags->"FeynAmp"],
  Cell[686288, 27602, 55, 1, 36, "Subsubsection",
   CellTags->"FeynAmp"],
  Cell[686346, 27605, 125, 4, 24, "Text",
   CellTags->"FeynAmp"],
  Cell[686496, 27613, 846, 22, 46, "Input",
   CellTags->"FeynAmp"],
  Cell[687345, 27637, 2630, 103, 42, "Output",
   CellTags->"FeynAmp"],
  Cell[689990, 27743, 73, 1, 24, "Text",
   CellTags->"FeynAmp"],
  Cell[690088, 27748, 198, 6, 26, "Input",
   CellTags->"FeynAmp"],
  Cell[690289, 27756, 456, 15, 42, "Output",
   CellTags->"FeynAmp"]},
 "FeynAmpDenominator"->{
  Cell[690772, 27775, 73, 1, 41, "ObjectName",
   CellTags->"FeynAmpDenominator"],
  Cell[690870, 27780, 69, 1, 36, "Subsubsection",
   CellTags->"FeynAmpDenominator"],
  Cell[690942, 27783, 256, 5, 43, "Text",
   CellTags->"FeynAmpDenominator"],
  Cell[691201, 27790, 338, 14, 25, "Text",
   CellTags->"FeynAmpDenominator"],
  Cell[691576, 27809, 66, 1, 34, "Subsubsection",
   CellTags->"FeynAmpDenominator"],
  Cell[691667, 27814, 201, 5, 26, "Input",
   CellTags->"FeynAmpDenominator"],
  Cell[691871, 27821, 253, 8, 50, "Output",
   CellTags->"FeynAmpDenominator"],
  Cell[692161, 27834, 326, 9, 26, "Input",
   CellTags->"FeynAmpDenominator"],
  Cell[692490, 27845, 472, 16, 51, "Output",
   CellTags->"FeynAmpDenominator"],
  Cell[692977, 27864, 247, 7, 26, "Input",
   CellTags->"FeynAmpDenominator"],
  Cell[693249, 27875, 149, 4, 26, "Input",
   CellTags->"FeynAmpDenominator"],
  Cell[693401, 27881, 321, 8, 38, "Output",
   CellTags->"FeynAmpDenominator"],
  Cell[693759, 27894, 149, 4, 26, "Input",
   CellTags->"FeynAmpDenominator"],
  Cell[693911, 27900, 220, 6, 38, "Output",
   CellTags->"FeynAmpDenominator"],
  Cell[694146, 27909, 134, 4, 26, "Input",
   CellTags->"FeynAmpDenominator"]},
 "FeynAmpDenominatorCombine"->{
  Cell[694295, 27916, 87, 1, 41, "ObjectName",
   CellTags->"FeynAmpDenominatorCombine"],
  Cell[694407, 27921, 76, 1, 36, "Subsubsection",
   CellTags->"FeynAmpDenominatorCombine"],
  Cell[694486, 27924, 228, 5, 43, "Text",
   CellTags->"FeynAmpDenominatorCombine"],
  Cell[694717, 27931, 241, 9, 25, "Text",
   CellTags->"FeynAmpDenominatorCombine"],
  Cell[694995, 27945, 73, 1, 34, "Subsubsection",
   CellTags->"FeynAmpDenominatorCombine"],
  Cell[695093, 27950, 234, 7, 26, "Input",
   CellTags->"FeynAmpDenominatorCombine"],
  Cell[695330, 27959, 807, 24, 50, "Output",
   CellTags->"FeynAmpDenominatorCombine"],
  Cell[696174, 27988, 208, 6, 26, "Input",
   CellTags->"FeynAmpDenominatorCombine"],
  Cell[696385, 27996, 245, 7, 38, "Output",
   CellTags->"FeynAmpDenominatorCombine"],
  Cell[696667, 28008, 206, 6, 26, "Input",
   CellTags->"FeynAmpDenominatorCombine"],
  Cell[696876, 28016, 275, 8, 38, "Output",
   CellTags->"FeynAmpDenominatorCombine"]},
 "FeynAmpDenominatorSimplify"->{
  Cell[697178, 28028, 89, 1, 41, "ObjectName",
   CellTags->"FeynAmpDenominatorSimplify"],
  Cell[697292, 28033, 77, 1, 36, "Subsubsection",
   CellTags->"FeynAmpDenominatorSimplify"],
  Cell[697372, 28036, 398, 7, 61, "Text",
   CellTags->"FeynAmpDenominatorSimplify"],
  Cell[697773, 28045, 85, 1, 24, "Text",
   CellTags->"FeynAmpDenominatorSimplify"],
  Cell[697883, 28050, 95, 2, 26, "Input",
   CellTags->"FeynAmpDenominatorSimplify"],
  Cell[697981, 28054, 191, 4, 26, "Output",
   CellTags->"FeynAmpDenominatorSimplify"],
  Cell[698187, 28061, 218, 9, 25, "Text",
   CellTags->"FeynAmpDenominatorSimplify"],
  Cell[698442, 28075, 74, 1, 36, "Subsubsection",
   CellTags->"FeynAmpDenominatorSimplify"],
  Cell[698519, 28078, 432, 14, 28, "Text",
   CellTags->"FeynAmpDenominatorSimplify"],
  Cell[698976, 28096, 280, 8, 26, "Input",
   CellTags->"FeynAmpDenominatorSimplify"],
  Cell[699259, 28106, 163, 4, 26, "Output",
   CellTags->"FeynAmpDenominatorSimplify"],
  Cell[699437, 28113, 468, 18, 27, "Text",
   CellTags->"FeynAmpDenominatorSimplify"],
  Cell[699930, 28135, 328, 10, 26, "Input",
   CellTags->"FeynAmpDenominatorSimplify"],
  Cell[700261, 28147, 378, 12, 50, "Output",
   CellTags->"FeynAmpDenominatorSimplify"],
  Cell[700676, 28164, 260, 7, 26, "Input",
   CellTags->"FeynAmpDenominatorSimplify"],
  Cell[700939, 28173, 350, 11, 50, "Output",
   CellTags->"FeynAmpDenominatorSimplify"],
  Cell[701326, 28189, 339, 10, 26, "Input",
   CellTags->"FeynAmpDenominatorSimplify"],
  Cell[701668, 28201, 404, 13, 50, "Output",
   CellTags->"FeynAmpDenominatorSimplify"],
  Cell[702109, 28219, 449, 14, 26, "Input",
   CellTags->"FeynAmpDenominatorSimplify"],
  Cell[702561, 28235, 1135, 41, 51, "Output",
   CellTags->"FeynAmpDenominatorSimplify"],
  Cell[703733, 28281, 215, 6, 26, "Input",
   CellTags->"FeynAmpDenominatorSimplify"],
  Cell[703951, 28289, 2003, 71, 52, "Output",
   CellTags->"FeynAmpDenominatorSimplify"],
  Cell[705991, 28365, 157, 4, 26, "Input",
   CellTags->"FeynAmpDenominatorSimplify"],
  Cell[706151, 28371, 1066, 39, 51, "Output",
   CellTags->"FeynAmpDenominatorSimplify"],
  Cell[707254, 28415, 373, 13, 26, "Input",
   CellTags->"FeynAmpDenominatorSimplify"],
  Cell[707630, 28430, 1403, 51, 52, "Output",
   CellTags->"FeynAmpDenominatorSimplify"],
  Cell[709048, 28484, 147, 4, 26, "Input",
   CellTags->"FeynAmpDenominatorSimplify"]},
 "FeynAmpDenominatorSplit"->{
  Cell[709210, 28491, 83, 1, 41, "ObjectName",
   CellTags->"FeynAmpDenominatorSplit"],
  Cell[709318, 28496, 74, 1, 34, "Subsubsection",
   CellTags->"FeynAmpDenominatorSplit"],
  Cell[709395, 28499, 357, 7, 58, "Text",
   CellTags->"FeynAmpDenominatorSplit"],
  Cell[709755, 28508, 245, 9, 24, "Text",
   CellTags->"FeynAmpDenominatorSplit"],
  Cell[710037, 28522, 71, 1, 34, "Subsubsection",
   CellTags->"FeynAmpDenominatorSplit"],
  Cell[710133, 28527, 308, 9, 26, "Input",
   CellTags->"FeynAmpDenominatorSplit"],
  Cell[710444, 28538, 533, 17, 50, "Output",
   CellTags->"FeynAmpDenominatorSplit"],
  Cell[711014, 28560, 116, 3, 26, "Input",
   CellTags->"FeynAmpDenominatorSplit"],
  Cell[711133, 28565, 178, 4, 26, "Output",
   CellTags->"FeynAmpDenominatorSplit"],
  Cell[711348, 28574, 139, 3, 26, "Input",
   CellTags->"FeynAmpDenominatorSplit"],
  Cell[711490, 28579, 531, 17, 50, "Output",
   CellTags->"FeynAmpDenominatorSplit"],
  Cell[712058, 28601, 150, 4, 26, "Input",
   CellTags->"FeynAmpDenominatorSplit"],
  Cell[712211, 28607, 469, 14, 38, "Output",
   CellTags->"FeynAmpDenominatorSplit"],
  Cell[712717, 28626, 230, 7, 26, "Input",
   CellTags->"FeynAmpDenominatorSplit"],
  Cell[712950, 28635, 410, 13, 38, "Output",
   CellTags->"FeynAmpDenominatorSplit"],
  Cell[713397, 28653, 206, 6, 26, "Input",
   CellTags->"FeynAmpDenominatorSplit"],
  Cell[713606, 28661, 347, 10, 38, "Output",
   CellTags->"FeynAmpDenominatorSplit"],
  Cell[713968, 28674, 121, 3, 26, "Input",
   CellTags->"FeynAmpDenominatorSplit"]},
 "FeynAmpList"->{
  Cell[714104, 28680, 59, 1, 39, "ObjectName",
   CellTags->"FeynAmpList"],
  Cell[714188, 28685, 62, 1, 34, "Subsubsection",
   CellTags->"FeynAmpList"],
  Cell[714253, 28688, 145, 4, 24, "Text",
   CellTags->"FeynAmpList"],
  Cell[714401, 28694, 117, 3, 24, "Text",
   CellTags->"FeynAmpList"],
  Cell[714521, 28699, 179, 9, 24, "Text",
   CellTags->"FeynAmpList"]},
 "FeynCalc"->{
  Cell[714715, 28711, 53, 1, 39, "ObjectName",
   CellTags->"FeynCalc"],
  Cell[714793, 28716, 59, 1, 34, "Subsubsection",
   CellTags->"FeynCalc"],
  Cell[714855, 28719, 91, 1, 24, "Text",
   CellTags->"FeynCalc"],
  Cell[714971, 28724, 99, 3, 26, "Input",
   CellTags->"FeynCalc"],
  Cell[715073, 28729, 745, 12, 354, "Print",
   CellTags->{"FeynCalc", "Info3500973837-4982097"}]},
 "Info3500973837-4982097"->{
  Cell[715073, 28729, 745, 12, 354, "Print",
   CellTags->{"FeynCalc", "Info3500973837-4982097"}]},
 "FeynCalcExternal"->{
  Cell[715845, 28745, 69, 1, 39, "ObjectName",
   CellTags->"FeynCalcExternal"],
  Cell[715939, 28750, 67, 1, 34, "Subsubsection",
   CellTags->"FeynCalcExternal"],
  Cell[716009, 28753, 156, 4, 24, "Text",
   CellTags->"FeynCalcExternal"],
  Cell[716168, 28759, 211, 9, 24, "Text",
   CellTags->"FeynCalcExternal"],
  Cell[716416, 28773, 64, 1, 34, "Subsubsection",
   CellTags->"FeynCalcExternal"],
  Cell[716505, 28778, 162, 4, 26, "Input",
   CellTags->"FeynCalcExternal"],
  Cell[716670, 28784, 185, 5, 26, "Output",
   CellTags->"FeynCalcExternal"],
  Cell[716892, 28794, 117, 3, 26, "Input",
   CellTags->"FeynCalcExternal"],
  Cell[717012, 28799, 168, 4, 38, "Output",
   CellTags->"FeynCalcExternal"],
  Cell[717217, 28808, 450, 13, 26, "Input",
   CellTags->"FeynCalcExternal"],
  Cell[717670, 28823, 602, 19, 29, "Output",
   CellTags->"FeynCalcExternal"],
  Cell[718309, 28847, 117, 3, 26, "Input",
   CellTags->"FeynCalcExternal"],
  Cell[718429, 28852, 507, 14, 38, "Output",
   CellTags->"FeynCalcExternal"],
  Cell[718973, 28871, 121, 3, 26, "Input",
   CellTags->"FeynCalcExternal"],
  Cell[719097, 28876, 917, 35, 29, "Output",
   CellTags->"FeynCalcExternal"],
  Cell[720051, 28916, 117, 3, 26, "Input",
   CellTags->"FeynCalcExternal"],
  Cell[720171, 28921, 950, 25, 77, "Output",
   CellTags->"FeynCalcExternal"],
  Cell[721158, 28951, 163, 5, 26, "Input",
   CellTags->"FeynCalcExternal"],
  Cell[721324, 28958, 507, 14, 38, "Output",
   CellTags->"FeynCalcExternal"]},
 "FeynCalcForm"->{
  Cell[721858, 28976, 61, 1, 41, "ObjectName",
   CellTags->"FeynCalcForm"],
  Cell[721944, 28981, 63, 1, 36, "Subsubsection",
   CellTags->"FeynCalcForm"],
  Cell[722010, 28984, 534, 10, 79, "Text",
   CellTags->"FeynCalcForm"],
  Cell[722547, 28996, 433, 19, 25, "Text",
   CellTags->"FeynCalcForm"],
  Cell[723017, 29020, 60, 1, 36, "Subsubsection",
   CellTags->"FeynCalcForm"],
  Cell[723080, 29023, 79, 1, 24, "Text",
   CellTags->"FeynCalcForm"],
  Cell[723184, 29028, 242, 7, 26, "Input",
   CellTags->"FeynCalcForm"],
  Cell[723429, 29037, 571, 23, 28, "Output",
   CellTags->"FeynCalcForm"],
  Cell[724015, 29063, 112, 3, 24, "Text",
   CellTags->"FeynCalcForm"],
  Cell[724130, 29068, 148, 4, 26, "Input",
   CellTags->"FeynCalcForm"],
  Cell[724281, 29074, 647, 19, 46, "Input",
   CellTags->"FeynCalcForm"],
  Cell[724953, 29097, 242, 7, 26, "Input",
   CellTags->"FeynCalcForm"],
  Cell[725198, 29106, 606, 18, 26, "Output",
   CellTags->"FeynCalcForm"],
  Cell[725819, 29127, 76, 1, 24, "Text",
   CellTags->"FeynCalcForm"],
  Cell[725898, 29130, 123, 4, 26, "Input",
   CellTags->"FeynCalcForm"],
  Cell[726024, 29136, 652, 19, 46, "Input",
   CellTags->"FeynCalcForm"]},
 "FeynCalcInternal"->{
  Cell[726691, 29158, 69, 1, 41, "ObjectName",
   CellTags->"FeynCalcInternal"],
  Cell[726785, 29163, 67, 1, 36, "Subsubsection",
   CellTags->"FeynCalcInternal"],
  Cell[726855, 29166, 157, 4, 25, "Text",
   CellTags->"FeynCalcInternal"],
  Cell[727015, 29172, 401, 19, 25, "Text",
   CellTags->"FeynCalcInternal"],
  Cell[727453, 29196, 64, 1, 34, "Subsubsection",
   CellTags->"FeynCalcInternal"],
  Cell[727542, 29201, 450, 13, 26, "Input",
   CellTags->"FeynCalcInternal"],
  Cell[727995, 29216, 601, 19, 29, "Output",
   CellTags->"FeynCalcInternal"],
  Cell[728633, 29240, 117, 3, 26, "Input",
   CellTags->"FeynCalcInternal"],
  Cell[728753, 29245, 507, 14, 38, "Output",
   CellTags->"FeynCalcInternal"],
  Cell[729297, 29264, 121, 3, 26, "Input",
   CellTags->"FeynCalcInternal"],
  Cell[729421, 29269, 919, 35, 29, "Output",
   CellTags->"FeynCalcInternal"],
  Cell[730377, 29309, 117, 3, 26, "Input",
   CellTags->"FeynCalcInternal"],
  Cell[730497, 29314, 948, 25, 77, "Output",
   CellTags->"FeynCalcInternal"],
  Cell[731482, 29344, 163, 5, 26, "Input",
   CellTags->"FeynCalcInternal"],
  Cell[731648, 29351, 507, 14, 38, "Output",
   CellTags->"FeynCalcInternal"],
  Cell[732192, 29370, 517, 15, 26, "Input",
   CellTags->"FeynCalcInternal"],
  Cell[732712, 29387, 1505, 64, 50, "Output",
   CellTags->"FeynCalcInternal"],
  Cell[734254, 29456, 117, 3, 26, "Input",
   CellTags->"FeynCalcInternal"],
  Cell[734374, 29461, 1172, 31, 96, "Output",
   CellTags->"FeynCalcInternal"]},
 "FeynCalc2FORM"->{
  Cell[735573, 29496, 63, 1, 41, "ObjectName",
   CellTags->"FeynCalc2FORM"],
  Cell[735661, 29501, 64, 1, 36, "Subsubsection",
   CellTags->"FeynCalc2FORM"],
  Cell[735728, 29504, 226, 5, 43, "Text",
   CellTags->"FeynCalc2FORM"],
  Cell[735957, 29511, 199, 9, 24, "Text",
   CellTags->"FeynCalc2FORM"],
  Cell[736181, 29524, 125, 3, 26, "Input",
   CellTags->"FeynCalc2FORM"],
  Cell[736309, 29529, 1063, 23, 67, "Output",
   CellTags->"FeynCalc2FORM"],
  Cell[737421, 29558, 61, 1, 34, "Subsubsection",
   CellTags->"FeynCalc2FORM"],
  Cell[737507, 29563, 273, 9, 26, "Input",
   CellTags->"FeynCalc2FORM"],
  Cell[737783, 29574, 320, 10, 47, "Output",
   CellTags->"FeynCalc2FORM"],
  Cell[738140, 29589, 137, 4, 26, "Input",
   CellTags->"FeynCalc2FORM"],
  Cell[738280, 29595, 112, 2, 21, "Print",
   CellTags->"FeynCalc2FORM"],
  Cell[738429, 29602, 199, 6, 26, "Input",
   CellTags->"FeynCalc2FORM"],
  Cell[738631, 29610, 241, 7, 26, "Output",
   CellTags->"FeynCalc2FORM"],
  Cell[738909, 29622, 137, 4, 26, "Input",
   CellTags->"FeynCalc2FORM"],
  Cell[739049, 29628, 110, 2, 21, "Print",
   CellTags->"FeynCalc2FORM"],
  Cell[739196, 29635, 212, 6, 26, "Input",
   CellTags->"FeynCalc2FORM"],
  Cell[739411, 29643, 415, 12, 26, "Output",
   CellTags->"FeynCalc2FORM"],
  Cell[739863, 29660, 137, 4, 26, "Input",
   CellTags->"FeynCalc2FORM"],
  Cell[740003, 29666, 132, 4, 21, "Print",
   CellTags->"FeynCalc2FORM"],
  Cell[740172, 29675, 298, 9, 26, "Input",
   CellTags->"FeynCalc2FORM"],
  Cell[740473, 29686, 517, 17, 26, "Output",
   CellTags->"FeynCalc2FORM"],
  Cell[741027, 29708, 137, 4, 26, "Input",
   CellTags->"FeynCalc2FORM"],
  Cell[741167, 29714, 130, 4, 21, "Print",
   CellTags->"FeynCalc2FORM"],
  Cell[741312, 29721, 328, 10, 26, "Input",
   CellTags->"FeynCalc2FORM"],
  Cell[741665, 29735, 332, 10, 26, "Input",
   CellTags->"FeynCalc2FORM"],
  Cell[742000, 29747, 196, 5, 26, "Output",
   CellTags->"FeynCalc2FORM"],
  Cell[742233, 29757, 314, 10, 26, "Input",
   CellTags->"FeynCalc2FORM"],
  Cell[742550, 29769, 6221, 241, 72, "Output",
   CellTags->"FeynCalc2FORM"],
  Cell[748786, 30013, 209, 6, 26, "Input",
   CellTags->"FeynCalc2FORM"],
  Cell[749020, 30023, 376, 11, 26, "Input",
   CellTags->"FeynCalc2FORM"],
  Cell[749399, 30036, 1382, 37, 231, "Output",
   CellTags->"FeynCalc2FORM"],
  Cell[750796, 30076, 313, 10, 26, "Input",
   CellTags->"FeynCalc2FORM"],
  Cell[751112, 30088, 129, 4, 26, "Input",
   CellTags->"FeynCalc2FORM"]},
 "FeynCalcToLaTeX"->{
  Cell[751256, 30095, 67, 1, 41, "ObjectName",
   CellTags->"FeynCalcToLaTeX"],
  Cell[751348, 30100, 66, 1, 36, "Subsubsection",
   CellTags->"FeynCalcToLaTeX"],
  Cell[751417, 30103, 300, 6, 66, "Text",
   CellTags->"FeynCalcToLaTeX"]},
 "FeynmanParameterNames"->{
  Cell[754205, 30189, 79, 1, 41, "ObjectName",
   CellTags->"FeynmanParameterNames"],
  Cell[754309, 30194, 72, 1, 36, "Subsubsection",
   CellTags->"FeynmanParameterNames"],
  Cell[754384, 30197, 143, 4, 25, "Text",
   CellTags->"FeynmanParameterNames"],
  Cell[754530, 30203, 362, 14, 25, "Text",
   CellTags->"FeynmanParameterNames"]},
 "FeynmanParametrize"->{
  Cell[754907, 30220, 90, 1, 41, "ObjectName",
   CellTags->"FeynmanParametrize"],
  Cell[755022, 30225, 69, 1, 36, "Subsubsection",
   CellTags->"FeynmanParametrize"],
  Cell[755094, 30228, 176, 4, 24, "Text",
   CellTags->"FeynmanParametrize"],
  Cell[755295, 30236, 135, 3, 26, "Input",
   CellTags->"FeynmanParametrize"],
  Cell[755433, 30241, 292, 9, 26, "Output",
   CellTags->"FeynmanParametrize"],
  Cell[755774, 30256, 66, 1, 34, "Subsubsection",
   CellTags->"FeynmanParametrize"],
  Cell[755843, 30259, 84, 2, 26, "Input",
   CellTags->"FeynmanParametrize"]},
 "FeynRule"->{
  Cell[755942, 30264, 53, 1, 41, "ObjectName",
   CellTags->"FeynRule"],
  Cell[756020, 30269, 59, 1, 36, "Subsubsection",
   CellTags->"FeynRule"],
  Cell[756082, 30272, 168, 4, 24, "Text",
   CellTags->"FeynRule"],
  Cell[756275, 30280, 115, 3, 26, "Input",
   CellTags->"FeynRule"],
  Cell[756393, 30285, 957, 22, 107, "Output",
   CellTags->"FeynRule"],
  Cell[757365, 30310, 92, 1, 25, "Text",
   CellTags->"FeynRule"],
  Cell[757460, 30313, 133, 4, 25, "Text",
   CellTags->"FeynRule"],
  Cell[757596, 30319, 185, 9, 25, "Text",
   CellTags->"FeynRule"],
  Cell[757818, 30333, 56, 1, 34, "Subsubsection",
   CellTags->"FeynRule"],
  Cell[757899, 30338, 146, 4, 26, "Input",
   CellTags->"FeynRule"],
  Cell[758048, 30344, 721, 24, 44, "Output",
   CellTags->"FeynRule"],
  Cell[758806, 30373, 146, 4, 26, "Input",
   CellTags->"FeynRule"],
  Cell[758955, 30379, 1045, 37, 44, "Output",
   CellTags->"FeynRule"],
  Cell[760037, 30421, 111, 3, 26, "Input",
   CellTags->"FeynRule"],
  Cell[760151, 30426, 5117, 160, 323, "Output",
   CellTags->"FeynRule"],
  Cell[765305, 30591, 141, 4, 26, "Input",
   CellTags->"FeynRule"],
  Cell[765449, 30597, 3277, 97, 276, "Output",
   CellTags->"FeynRule"],
  Cell[768741, 30697, 74, 1, 24, "Text",
   CellTags->"FeynRule"],
  Cell[768840, 30702, 503, 16, 26, "Input",
   CellTags->"FeynRule"],
  Cell[769346, 30720, 742, 31, 32, "Output",
   CellTags->"FeynRule"],
  Cell[770125, 30756, 307, 9, 26, "Input",
   CellTags->"FeynRule"],
  Cell[770435, 30767, 3837, 139, 52, "Output",
   CellTags->"FeynRule"],
  Cell[774287, 30909, 72, 1, 24, "Text",
   CellTags->"FeynRule"],
  Cell[774384, 30914, 503, 16, 26, "Input",
   CellTags->"FeynRule"],
  Cell[774890, 30932, 742, 31, 32, "Output",
   CellTags->"FeynRule"],
  Cell[775669, 30968, 307, 9, 26, "Input",
   CellTags->"FeynRule"],
  Cell[775979, 30979, 4509, 152, 52, "Output",
   CellTags->"FeynRule"],
  Cell[780525, 31136, 218, 7, 26, "Input",
   CellTags->"FeynRule"],
  Cell[780746, 31145, 1086, 43, 28, "Output",
   CellTags->"FeynRule"],
  Cell[781847, 31191, 110, 3, 24, "Text",
   CellTags->"FeynRule"],
  Cell[781982, 31198, 360, 10, 26, "Input",
   CellTags->"FeynRule"],
  Cell[782345, 31210, 1150, 46, 28, "Output",
   CellTags->"FeynRule"],
  Cell[783510, 31259, 77, 1, 24, "Text",
   CellTags->"FeynRule"],
  Cell[783612, 31264, 145, 4, 26, "Input",
   CellTags->"FeynRule"],
  Cell[783760, 31270, 660, 23, 26, "Output",
   CellTags->"FeynRule"],
  Cell[784435, 31296, 90, 1, 24, "Text",
   CellTags->"FeynRule"],
  Cell[784550, 31301, 145, 4, 26, "Input",
   CellTags->"FeynRule"],
  Cell[784698, 31307, 662, 23, 26, "Output",
   CellTags->"FeynRule"],
  Cell[785397, 31335, 696, 21, 46, "Input",
   CellTags->"FeynRule"],
  Cell[786096, 31358, 937, 38, 32, "Output",
   CellTags->"FeynRule"],
  Cell[787070, 31401, 227, 7, 26, "Input",
   CellTags->"FeynRule"],
  Cell[787300, 31410, 5001, 178, 118, "Output",
   CellTags->"FeynRule"],
  Cell[792338, 31593, 420, 12, 26, "Input",
   CellTags->"FeynRule"],
  Cell[792761, 31607, 4806, 160, 114, "Output",
   CellTags->"FeynRule"],
  Cell[797604, 31772, 362, 12, 26, "Input",
   CellTags->"FeynRule"],
  Cell[797969, 31786, 148, 4, 26, "Output",
   CellTags->"FeynRule"],
  Cell[798132, 31793, 210, 5, 41, "Text",
   CellTags->"FeynRule"],
  Cell[798345, 31800, 144, 4, 26, "Input",
   CellTags->"FeynRule"],
  Cell[798492, 31806, 51, 1, 24, "Text",
   CellTags->"FeynRule"],
  Cell[798568, 31811, 693, 21, 46, "Input",
   CellTags->"FeynRule"],
  Cell[799264, 31834, 1028, 43, 32, "Output",
   CellTags->"FeynRule"],
  Cell[800329, 31882, 212, 7, 26, "Input",
   CellTags->"FeynRule"],
  Cell[800544, 31891, 2015, 80, 28, "Output",
   CellTags->"FeynRule"],
  Cell[802596, 31976, 385, 11, 26, "Input",
   CellTags->"FeynRule"],
  Cell[802984, 31989, 1461, 58, 28, "Output",
   CellTags->"FeynRule"],
  Cell[804482, 32052, 197, 6, 26, "Input",
   CellTags->"FeynRule"],
  Cell[804682, 32060, 146, 4, 26, "Output",
   CellTags->"FeynRule"],
  Cell[804865, 32069, 884, 26, 46, "Input",
   CellTags->"FeynRule"],
  Cell[805752, 32097, 1314, 55, 32, "Output",
   CellTags->"FeynRule"],
  Cell[807103, 32157, 212, 7, 26, "Input",
   CellTags->"FeynRule"],
  Cell[807318, 32166, 3347, 132, 50, "Output",
   CellTags->"FeynRule"],
  Cell[810702, 32303, 461, 13, 26, "Input",
   CellTags->"FeynRule"],
  Cell[811166, 32318, 3512, 133, 50, "Output",
   CellTags->"FeynRule"],
  Cell[814715, 32456, 197, 6, 26, "Input",
   CellTags->"FeynRule"],
  Cell[814915, 32464, 148, 4, 26, "Output",
   CellTags->"FeynRule"],
  Cell[815078, 32471, 418, 9, 46, "Input",
   CellTags->"FeynRule"]},
 "FI"->{
  Cell[815511, 32483, 41, 1, 41, "ObjectName",
   CellTags->"FI"],
  Cell[815577, 32488, 53, 1, 36, "Subsubsection",
   CellTags->"FI"],
  Cell[815633, 32491, 195, 4, 43, "Text",
   CellTags->"FI"],
  Cell[815831, 32497, 545, 24, 25, "Text",
   CellTags->"FI"]},
 "FieldDerivative"->{
  Cell[816391, 32524, 67, 1, 41, "ObjectName",
   CellTags->"FieldDerivative"],
  Cell[816483, 32529, 66, 1, 36, "Subsubsection",
   CellTags->"FieldDerivative"],
  Cell[816552, 32532, 549, 9, 79, "Text",
   CellTags->"FieldDerivative"],
  Cell[817104, 32543, 314, 14, 25, "Text",
   CellTags->"FieldDerivative"],
  Cell[817455, 32562, 63, 1, 34, "Subsubsection",
   CellTags->"FieldDerivative"],
  Cell[817543, 32567, 620, 19, 46, "Input",
   CellTags->"FieldDerivative"],
  Cell[818166, 32588, 507, 18, 29, "Output",
   CellTags->"FieldDerivative"],
  Cell[818710, 32611, 218, 6, 26, "Input",
   CellTags->"FieldDerivative"],
  Cell[818931, 32619, 1885, 52, 153, "Output",
   CellTags->"FieldDerivative"],
  Cell[820853, 32676, 218, 6, 26, "Input",
   CellTags->"FieldDerivative"],
  Cell[821074, 32684, 5569, 138, 521, "Output",
   CellTags->"FieldDerivative"]},
 "FieldStrength"->{
  Cell[826670, 32826, 63, 1, 41, "ObjectName",
   CellTags->"FieldStrength"],
  Cell[826758, 32831, 64, 1, 36, "Subsubsection",
   CellTags->"FieldStrength"],
  Cell[826825, 32834, 1285, 37, 66, "Text",
   CellTags->"FieldStrength"],
  Cell[828135, 32875, 125, 3, 26, "Input",
   CellTags->"FieldStrength"],
  Cell[828263, 32880, 570, 16, 47, "Output",
   CellTags->"FieldStrength"],
  Cell[828848, 32899, 196, 9, 25, "Text",
   CellTags->"FieldStrength"],
  Cell[829081, 32913, 61, 1, 34, "Subsubsection",
   CellTags->"FieldStrength"],
  Cell[829167, 32918, 150, 4, 26, "Input",
   CellTags->"FieldStrength"],
  Cell[829320, 32924, 216, 6, 30, "Output",
   CellTags->"FieldStrength"],
  Cell[829573, 32935, 160, 4, 26, "Input",
   CellTags->"FieldStrength"],
  Cell[829736, 32941, 212, 6, 30, "Output",
   CellTags->"FieldStrength"],
  Cell[829985, 32952, 200, 5, 26, "Input",
   CellTags->"FieldStrength"],
  Cell[830188, 32959, 1066, 25, 147, "Output",
   CellTags->"FieldStrength"],
  Cell[831291, 32989, 210, 5, 26, "Input",
   CellTags->"FieldStrength"],
  Cell[831504, 32996, 1869, 58, 152, "Output",
   CellTags->"FieldStrength"],
  Cell[833410, 33059, 241, 6, 26, "Input",
   CellTags->"FieldStrength"],
  Cell[833654, 33067, 1065, 22, 191, "Output",
   CellTags->"FieldStrength"],
  Cell[834756, 33094, 244, 6, 26, "Input",
   CellTags->"FieldStrength"],
  Cell[835003, 33102, 1104, 23, 191, "Output",
   CellTags->"FieldStrength"],
  Cell[836144, 33130, 291, 7, 26, "Input",
   CellTags->"FieldStrength"],
  Cell[836438, 33139, 1869, 58, 152, "Output",
   CellTags->"FieldStrength"]},
 "FinalFunction"->{
  Cell[838334, 33201, 63, 1, 41, "ObjectName",
   CellTags->"FinalFunction"],
  Cell[838422, 33206, 64, 1, 36, "Subsubsection",
   CellTags->"FinalFunction"],
  Cell[838489, 33209, 86, 1, 25, "Text",
   CellTags->"FinalFunction"],
  Cell[838578, 33212, 190, 9, 25, "Text",
   CellTags->"FinalFunction"]},
 "FinalSubstitutions"->{
  Cell[838783, 33224, 73, 1, 41, "ObjectName",
   CellTags->"FinalSubstitutions"],
  Cell[838881, 33229, 69, 1, 36, "Subsubsection",
   CellTags->"FinalSubstitutions"],
  Cell[838953, 33232, 204, 4, 43, "Text",
   CellTags->"FinalSubstitutions"],
  Cell[839160, 33238, 406, 19, 25, "Text",
   CellTags->"FinalSubstitutions"]},
 "FORM"->{
  Cell[839581, 33260, 45, 1, 41, "ObjectName",
   CellTags->"FORM"],
  Cell[839651, 33265, 55, 1, 36, "Subsubsection",
   CellTags->"FORM"],
  Cell[839709, 33268, 204, 5, 43, "Text",
   CellTags->"FORM"],
  Cell[839916, 33275, 285, 14, 25, "Text",
   CellTags->"FORM"]},
 "FORMEpilog"->{
  Cell[840216, 33292, 57, 1, 41, "ObjectName",
   CellTags->"FORMEpilog"],
  Cell[840298, 33297, 61, 1, 36, "Subsubsection",
   CellTags->"FORMEpilog"],
  Cell[840362, 33300, 161, 4, 25, "Text",
   CellTags->"FORMEpilog"],
  Cell[840526, 33306, 312, 14, 25, "Text",
   CellTags->"FORMEpilog"]},
 "FORMProlog"->{
  Cell[840853, 33323, 57, 1, 41, "ObjectName",
   CellTags->"FORMProlog"],
  Cell[840935, 33328, 61, 1, 36, "Subsubsection",
   CellTags->"FORMProlog"],
  Cell[840999, 33331, 178, 4, 25, "Text",
   CellTags->"FORMProlog"],
  Cell[841180, 33337, 312, 14, 25, "Text",
   CellTags->"FORMProlog"]},
 "FORM2FeynCalc"->{
  Cell[841507, 33354, 63, 1, 41, "ObjectName",
   CellTags->"FORM2FeynCalc"],
  Cell[841595, 33359, 64, 1, 36, "Subsubsection",
   CellTags->"FORM2FeynCalc"],
  Cell[841662, 33362, 470, 8, 79, "Text",
   CellTags->"FORM2FeynCalc"],
  Cell[842135, 33372, 199, 9, 24, "Text",
   CellTags->"FORM2FeynCalc"],
  Cell[842359, 33385, 125, 3, 26, "Input",
   CellTags->"FORM2FeynCalc"],
  Cell[842487, 33390, 704, 19, 47, "Output",
   CellTags->"FORM2FeynCalc"],
  Cell[843240, 33415, 61, 1, 34, "Subsubsection",
   CellTags->"FORM2FeynCalc"],
  Cell[843326, 33420, 139, 3, 26, "Input",
   CellTags->"FORM2FeynCalc"],
  Cell[843468, 33425, 395, 15, 28, "Output",
   CellTags->"FORM2FeynCalc"],
  Cell[843900, 33445, 114, 3, 26, "Input",
   CellTags->"FORM2FeynCalc"],
  Cell[844017, 33450, 285, 9, 38, "Output",
   CellTags->"FORM2FeynCalc"],
  Cell[844317, 33462, 159, 4, 24, "Text",
   CellTags->"FORM2FeynCalc"],
  Cell[844501, 33470, 314, 8, 26, "Input",
   CellTags->"FORM2FeynCalc"],
  Cell[844818, 33480, 340, 11, 26, "Output",
   CellTags->"FORM2FeynCalc"],
  Cell[845195, 33496, 114, 3, 26, "Input",
   CellTags->"FORM2FeynCalc"],
  Cell[845312, 33501, 347, 10, 38, "Output",
   CellTags->"FORM2FeynCalc"],
  Cell[845696, 33516, 150, 4, 26, "Input",
   CellTags->"FORM2FeynCalc"],
  Cell[845849, 33522, 375, 13, 26, "Output",
   CellTags->"FORM2FeynCalc"],
  Cell[846261, 33540, 117, 3, 26, "Input",
   CellTags->"FORM2FeynCalc"],
  Cell[846381, 33545, 303, 10, 26, "Output",
   CellTags->"FORM2FeynCalc"],
  Cell[846721, 33560, 150, 4, 26, "Input",
   CellTags->"FORM2FeynCalc"],
  Cell[846874, 33566, 301, 10, 28, "Output",
   CellTags->"FORM2FeynCalc"],
  Cell[847212, 33581, 114, 3, 26, "Input",
   CellTags->"FORM2FeynCalc"],
  Cell[847329, 33586, 351, 11, 38, "Output",
   CellTags->"FORM2FeynCalc"],
  Cell[847717, 33602, 333, 9, 26, "Input",
   CellTags->"FORM2FeynCalc"],
  Cell[848053, 33613, 313, 10, 28, "Output",
   CellTags->"FORM2FeynCalc"],
  Cell[848403, 33628, 195, 5, 26, "Input",
   CellTags->"FORM2FeynCalc"],
  Cell[848601, 33635, 625, 19, 26, "Output",
   CellTags->"FORM2FeynCalc"]},
 "FourDivergence"->{
  Cell[849253, 33658, 65, 1, 41, "ObjectName",
   CellTags->"FourDivergence"],
  Cell[849343, 33663, 65, 1, 36, "Subsubsection",
   CellTags->"FourDivergence"],
  Cell[849411, 33666, 242, 5, 43, "Text",
   CellTags->"FourDivergence"],
  Cell[849656, 33673, 197, 9, 25, "Text",
   CellTags->"FourDivergence"],
  Cell[849890, 33687, 62, 1, 34, "Subsubsection",
   CellTags->"FourDivergence"],
  Cell[849977, 33692, 167, 5, 26, "Input",
   CellTags->"FourDivergence"],
  Cell[850147, 33699, 256, 9, 28, "Output",
   CellTags->"FourDivergence"],
  Cell[850440, 33713, 211, 6, 26, "Input",
   CellTags->"FourDivergence"],
  Cell[850654, 33721, 284, 11, 28, "Output",
   CellTags->"FourDivergence"],
  Cell[850975, 33737, 192, 6, 26, "Input",
   CellTags->"FourDivergence"],
  Cell[851170, 33745, 289, 10, 28, "Output",
   CellTags->"FourDivergence"],
  Cell[851496, 33760, 237, 7, 26, "Input",
   CellTags->"FourDivergence"],
  Cell[851736, 33769, 152, 4, 26, "Output",
   CellTags->"FourDivergence"],
  Cell[851903, 33776, 112, 3, 26, "Input",
   CellTags->"FourDivergence"]},
 "FourLaplacian"->{
  Cell[852030, 33782, 63, 1, 41, "ObjectName",
   CellTags->"FourLaplacian"],
  Cell[852118, 33787, 64, 1, 36, "Subsubsection",
   CellTags->"FourLaplacian"],
  Cell[852185, 33790, 426, 16, 27, "Text",
   CellTags->"FourLaplacian"],
  Cell[852636, 33810, 125, 3, 26, "Input",
   CellTags->"FourLaplacian"],
  Cell[852764, 33815, 212, 6, 26, "Output",
   CellTags->"FourLaplacian"],
  Cell[852991, 33824, 324, 14, 25, "Text",
   CellTags->"FourLaplacian"],
  Cell[853352, 33843, 61, 1, 34, "Subsubsection",
   CellTags->"FourLaplacian"],
  Cell[853438, 33848, 131, 4, 26, "Input",
   CellTags->"FourLaplacian"],
  Cell[853572, 33854, 209, 7, 28, "Output",
   CellTags->"FourLaplacian"],
  Cell[853818, 33866, 152, 4, 26, "Input",
   CellTags->"FourLaplacian"],
  Cell[853973, 33872, 176, 5, 26, "Output",
   CellTags->"FourLaplacian"],
  Cell[854186, 33882, 264, 9, 26, "Input",
   CellTags->"FourLaplacian"],
  Cell[854453, 33893, 482, 17, 49, "Output",
   CellTags->"FourLaplacian"],
  Cell[854972, 33915, 157, 4, 26, "Input",
   CellTags->"FourLaplacian"],
  Cell[855132, 33921, 3899, 137, 100, "Output",
   CellTags->"FourLaplacian"]},
 "FourVector"->{
  Cell[859058, 34062, 57, 1, 41, "ObjectName",
   CellTags->"FourVector"],
  Cell[859140, 34067, 61, 1, 36, "Subsubsection",
   CellTags->"FourVector"],
  Cell[859204, 34070, 339, 10, 43, "Text",
   CellTags->"FourVector"],
  Cell[859546, 34082, 356, 19, 25, "Text",
   CellTags->"FourVector"],
  Cell[859939, 34106, 58, 1, 34, "Subsubsection",
   CellTags->"FourVector"],
  Cell[860022, 34111, 140, 4, 26, "Input",
   CellTags->"FourVector"],
  Cell[860165, 34117, 174, 5, 29, "Output",
   CellTags->"FourVector"],
  Cell[860376, 34127, 164, 5, 26, "Input",
   CellTags->"FourVector"],
  Cell[860543, 34134, 291, 11, 30, "Output",
   CellTags->"FourVector"],
  Cell[860871, 34150, 180, 5, 26, "Input",
   CellTags->"FourVector"],
  Cell[861054, 34157, 197, 5, 38, "Output",
   CellTags->"FourVector"],
  Cell[861288, 34167, 229, 6, 26, "Input",
   CellTags->"FourVector"],
  Cell[861520, 34175, 245, 6, 38, "Output",
   CellTags->"FourVector"],
  Cell[861802, 34186, 212, 6, 26, "Input",
   CellTags->"FourVector"],
  Cell[862017, 34194, 199, 5, 38, "Output",
   CellTags->"FourVector"],
  Cell[862231, 34202, 193, 4, 41, "Text",
   CellTags->"FourVector"],
  Cell[862449, 34210, 212, 6, 26, "Input",
   CellTags->"FourVector"],
  Cell[862664, 34218, 291, 11, 30, "Output",
   CellTags->"FourVector"]},
 "FreeIndex"->{
  Cell[862982, 34233, 55, 1, 41, "ObjectName",
   CellTags->"FreeIndex"],
  Cell[863062, 34238, 60, 1, 36, "Subsubsection",
   CellTags->"FreeIndex"],
  Cell[863125, 34241, 152, 4, 25, "Text",
   CellTags->"FreeIndex"],
  Cell[863280, 34247, 290, 14, 25, "Text",
   CellTags->"FreeIndex"]},
 "FreeQ2"->{
  Cell[863585, 34264, 49, 1, 41, "ObjectName",
   CellTags->"FreeQ2"],
  Cell[863659, 34269, 57, 1, 36, "Subsubsection",
   CellTags->"FreeQ2"],
  Cell[863719, 34272, 228, 5, 43, "Text",
   CellTags->"FreeQ2"],
  Cell[863950, 34279, 308, 14, 25, "Text",
   CellTags->"FreeQ2"],
  Cell[864295, 34298, 54, 1, 34, "Subsubsection",
   CellTags->"FreeQ2"],
  Cell[864374, 34303, 246, 8, 26, "Input",
   CellTags->"FreeQ2"],
  Cell[864623, 34313, 148, 4, 26, "Output",
   CellTags->"FreeQ2"],
  Cell[864808, 34322, 241, 8, 26, "Input",
   CellTags->"FreeQ2"],
  Cell[865052, 34332, 147, 4, 26, "Output",
   CellTags->"FreeQ2"],
  Cell[865236, 34341, 133, 4, 26, "Input",
   CellTags->"FreeQ2"],
  Cell[865372, 34347, 145, 4, 26, "Output",
   CellTags->"FreeQ2"],
  Cell[865554, 34356, 162, 5, 26, "Input",
   CellTags->"FreeQ2"],
  Cell[865719, 34363, 148, 4, 26, "Output",
   CellTags->"FreeQ2"]},
 "FRH"->{
  Cell[865894, 34371, 43, 1, 41, "ObjectName",
   CellTags->"FRH"],
  Cell[865962, 34376, 54, 1, 36, "Subsubsection",
   CellTags->"FRH"],
  Cell[866019, 34379, 134, 4, 25, "Text",
   CellTags->"FRH"],
  Cell[866156, 34385, 171, 9, 25, "Text",
   CellTags->"FRH"],
  Cell[866364, 34399, 51, 1, 34, "Subsubsection",
   CellTags->"FRH"],
  Cell[866440, 34404, 200, 6, 26, "Input",
   CellTags->"FRH"],
  Cell[866643, 34412, 290, 10, 26, "Output",
   CellTags->"FRH"],
  Cell[866970, 34427, 99, 3, 26, "Input",
   CellTags->"FRH"],
  Cell[867072, 34432, 141, 4, 26, "Output",
   CellTags->"FRH"],
  Cell[867250, 34441, 304, 10, 26, "Input",
   CellTags->"FRH"],
  Cell[867557, 34453, 573, 21, 26, "Output",
   CellTags->"FRH"],
  Cell[868167, 34479, 128, 4, 26, "Input",
   CellTags->"FRH"],
  Cell[868298, 34485, 170, 5, 26, "Output",
   CellTags->"FRH"]},
 "FromTFi"->{
  Cell[868495, 34494, 51, 1, 41, "ObjectName",
   CellTags->"FromTFi"],
  Cell[868571, 34499, 58, 1, 36, "Subsubsection",
   CellTags->"FromTFi"],
  Cell[868632, 34502, 200, 5, 43, "Text",
   CellTags->"FromTFi"],
  Cell[869130, 34527, 55, 1, 34, "Subsubsection",
   CellTags->"FromTFi"],
  Cell[869210, 34532, 431, 14, 26, "Input",
   CellTags->"FromTFi"],
  Cell[869644, 34548, 818, 25, 97, "Output",
   CellTags->"FromTFi"],
  Cell[870499, 34578, 693, 22, 26, "Input",
   CellTags->"FromTFi"],
  Cell[871195, 34602, 1010, 32, 97, "Output",
   CellTags->"FromTFi"],
  Cell[872242, 34639, 756, 24, 26, "Input",
   CellTags->"FromTFi"],
  Cell[873001, 34665, 1069, 34, 97, "Output",
   CellTags->"FromTFi"]},
 "FromTFI"->{
  Cell[868835, 34509, 258, 13, 25, "Text",
   CellTags->"FromTFI"]},
 "FUNCTION"->{
  Cell[874097, 34703, 53, 1, 41, "ObjectName",
   CellTags->"FUNCTION"],
  Cell[874175, 34708, 59, 1, 36, "Subsubsection",
   CellTags->"FUNCTION"],
  Cell[874237, 34711, 160, 4, 25, "Text",
   CellTags->"FUNCTION"],
  Cell[874400, 34717, 173, 9, 25, "Text",
   CellTags->"FUNCTION"]},
 "FunctionalD"->{
  Cell[874588, 34729, 59, 1, 41, "ObjectName",
   CellTags->"FunctionalD"],
  Cell[874672, 34734, 62, 1, 36, "Subsubsection",
   CellTags->"FunctionalD"],
  Cell[874737, 34737, 416, 7, 61, "Text",
   CellTags->"FunctionalD"],
  Cell[875156, 34746, 95, 1, 25, "Text",
   CellTags->"FunctionalD"],
  Cell[875254, 34749, 304, 14, 25, "Text",
   CellTags->"FunctionalD"],
  Cell[875595, 34768, 59, 1, 36, "Subsubsection",
   CellTags->"FunctionalD"],
  Cell[875657, 34771, 610, 20, 40, "Text",
   CellTags->"FunctionalD"],
  Cell[876292, 34795, 228, 6, 26, "Input",
   CellTags->"FunctionalD"],
  Cell[876523, 34803, 149, 4, 26, "Output",
   CellTags->"FunctionalD"],
  Cell[876709, 34812, 253, 7, 26, "Input",
   CellTags->"FunctionalD"],
  Cell[876965, 34821, 179, 5, 26, "Output",
   CellTags->"FunctionalD"],
  Cell[877159, 34829, 1269, 44, 51, "Text",
   CellTags->"FunctionalD"],
  Cell[878453, 34877, 295, 8, 26, "Input",
   CellTags->"FunctionalD"],
  Cell[878751, 34887, 315, 11, 33, "Output",
   CellTags->"FunctionalD"],
  Cell[879081, 34901, 778, 24, 28, "Text",
   CellTags->"FunctionalD"],
  Cell[879884, 34929, 668, 22, 46, "Input",
   CellTags->"FunctionalD"],
  Cell[880555, 34953, 1191, 29, 171, "Output",
   CellTags->"FunctionalD"],
  Cell[881783, 34987, 217, 6, 26, "Input",
   CellTags->"FunctionalD"],
  Cell[882003, 34995, 1047, 25, 147, "Output",
   CellTags->"FunctionalD"],
  Cell[883065, 35023, 489, 15, 25, "Text",
   CellTags->"FunctionalD"],
  Cell[883557, 35040, 97, 2, 24, "Text",
   CellTags->"FunctionalD"],
  Cell[883679, 35046, 315, 8, 26, "Input",
   CellTags->"FunctionalD"],
  Cell[883997, 35056, 1743, 53, 152, "Output",
   CellTags->"FunctionalD"],
  Cell[885777, 35114, 315, 8, 26, "Input",
   CellTags->"FunctionalD"],
  Cell[886095, 35124, 1741, 53, 152, "Output",
   CellTags->"FunctionalD"],
  Cell[887873, 35182, 216, 8, 26, "Input",
   CellTags->"FunctionalD"],
  Cell[888092, 35192, 3730, 108, 299, "Output",
   CellTags->"FunctionalD"],
  Cell[891837, 35303, 560, 13, 61, "Text",
   CellTags->"FunctionalD"],
  Cell[892400, 35318, 1398, 50, 25, "Text",
   CellTags->"FunctionalD"],
  Cell[893801, 35370, 445, 13, 26, "Text",
   CellTags->"FunctionalD"],
  Cell[894271, 35387, 231, 7, 26, "Input",
   CellTags->"FunctionalD"],
  Cell[894505, 35396, 330, 13, 28, "Output",
   CellTags->"FunctionalD"],
  Cell[894872, 35414, 165, 5, 26, "Input",
   CellTags->"FunctionalD"],
  Cell[895040, 35421, 2330, 97, 34, "Output",
   CellTags->"FunctionalD"],
  Cell[897385, 35521, 120, 3, 24, "Text",
   CellTags->"FunctionalD"],
  Cell[897530, 35528, 176, 5, 26, "Input",
   CellTags->"FunctionalD"],
  Cell[897709, 35535, 2330, 97, 34, "Output",
   CellTags->"FunctionalD"],
  Cell[900054, 35635, 290, 8, 26, "Text",
   CellTags->"FunctionalD"],
  Cell[900347, 35645, 523, 16, 46, "Input",
   CellTags->"FunctionalD"],
  Cell[900895, 35665, 230, 7, 26, "Input",
   CellTags->"FunctionalD"],
  Cell[901128, 35674, 428, 18, 30, "Output",
   CellTags->"FunctionalD"],
  Cell[901593, 35697, 240, 7, 26, "Input",
   CellTags->"FunctionalD"],
  Cell[901836, 35706, 457, 19, 30, "Output",
   CellTags->"FunctionalD"],
  Cell[902330, 35730, 579, 16, 46, "Input",
   CellTags->"FunctionalD"],
  Cell[902912, 35748, 3204, 117, 206, "Output",
   CellTags->"FunctionalD"],
  Cell[906153, 35870, 184, 5, 26, "Input",
   CellTags->"FunctionalD"],
  Cell[906340, 35877, 1588, 58, 91, "Output",
   CellTags->"FunctionalD"],
  Cell[907965, 35940, 352, 11, 26, "Input",
   CellTags->"FunctionalD"],
  Cell[908320, 35953, 1340, 46, 91, "Output",
   CellTags->"FunctionalD"],
  Cell[909675, 36002, 269, 9, 27, "Text",
   CellTags->"FunctionalD"],
  Cell[909947, 36013, 98, 2, 24, "Text",
   CellTags->"FunctionalD"],
  Cell[910048, 36017, 167, 4, 24, "Text",
   CellTags->"FunctionalD"],
  Cell[910240, 36025, 105, 3, 26, "Input",
   CellTags->"FunctionalD"],
  Cell[910348, 36030, 3730, 108, 299, "Output",
   CellTags->"FunctionalD"],
  Cell[914115, 36143, 194, 6, 26, "Input",
   CellTags->"FunctionalD"],
  Cell[914312, 36151, 27621, 950, 1944, "Output",
   CellTags->"FunctionalD"],
  Cell[941948, 37104, 227, 5, 43, "Text",
   CellTags->"FunctionalD"],
  Cell[942178, 37111, 345, 6, 58, "Text",
   CellTags->"FunctionalD"],
  Cell[942526, 37119, 132, 4, 26, "Input",
   CellTags->"FunctionalD"],
  Cell[942661, 37125, 2583, 67, 141, "Input",
   CellTags->"FunctionalD"],
  Cell[945269, 37196, 169, 5, 26, "Input",
   CellTags->"FunctionalD"],
  Cell[945441, 37203, 14498, 491, 1082, "Output",
   CellTags->"FunctionalD"],
  Cell[959976, 37699, 164, 5, 26, "Input",
   CellTags->"FunctionalD"],
  Cell[960143, 37706, 7809, 271, 480, "Output",
   CellTags->"FunctionalD"],
  Cell[967967, 37980, 212, 8, 24, "Text",
   CellTags->"FunctionalD"],
  Cell[968204, 37992, 315, 11, 26, "Input",
   CellTags->"FunctionalD"],
  Cell[968522, 38005, 2778, 120, 44, "Output",
   CellTags->"FunctionalD"],
  Cell[971337, 38130, 647, 20, 26, "Input",
   CellTags->"FunctionalD"],
  Cell[971987, 38152, 1352, 61, 32, "Output",
   CellTags->"FunctionalD"],
  Cell[973376, 38218, 154, 4, 26, "Input",
   CellTags->"FunctionalD"],
  Cell[973533, 38224, 6504, 213, 466, "Output",
   CellTags->"FunctionalD"],
  Cell[980052, 38440, 131, 4, 24, "Text",
   CellTags->"FunctionalD"],
  Cell[980208, 38448, 79, 2, 26, "Input",
   CellTags->"FunctionalD"],
  Cell[980290, 38452, 1340, 46, 91, "Output",
   CellTags->"FunctionalD"],
  Cell[981667, 38503, 798, 21, 65, "Input",
   CellTags->"FunctionalD"],
  Cell[982468, 38526, 1104, 42, 34, "Output",
   CellTags->"FunctionalD"],
  Cell[983609, 38573, 137, 4, 26, "Input",
   CellTags->"FunctionalD"],
  Cell[983749, 38579, 4950, 160, 309, "Output",
   CellTags->"FunctionalD"],
  Cell[988736, 38744, 204, 6, 26, "Input",
   CellTags->"FunctionalD"],
  Cell[988943, 38752, 7513, 249, 533, "Output",
   CellTags->"FunctionalD"],
  Cell[996493, 39006, 138, 4, 26, "Input",
   CellTags->"FunctionalD"],
  Cell[996634, 39012, 13810, 456, 967, "Output",
   CellTags->"FunctionalD"],
  Cell[1010481, 39473, 163, 5, 26, "Input",
   CellTags->"FunctionalD"],
  Cell[1010647, 39480, 12753, 420, 901, "Output",
   CellTags->"FunctionalD"],
  Cell[1023415, 39903, 66, 1, 24, "Text",
   CellTags->"FunctionalD"],
  Cell[1023484, 39906, 539, 13, 46, "Input",
   CellTags->"FunctionalD"],
  Cell[1024060, 39924, 110, 2, 36, "Subsubsection",
   CellTags->"FunctionalD"],
  Cell[1024173, 39928, 79, 1, 24, "Text",
   CellTags->"FunctionalD"],
  Cell[1026922, 40022, 765, 22, 46, "Input",
   CellTags->"FunctionalD"],
  Cell[1027690, 40046, 8939, 330, 119, "Output",
   CellTags->"FunctionalD"],
  Cell[1036666, 40381, 165, 5, 26, "Input",
   CellTags->"FunctionalD"],
  Cell[1036834, 40388, 2139, 85, 44, "Output",
   CellTags->"FunctionalD"]},
 "FV"->{
  Cell[1039000, 40477, 41, 1, 41, "ObjectName",
   CellTags->"FV"],
  Cell[1039066, 40482, 53, 1, 36, "Subsubsection",
   CellTags->"FV"],
  Cell[1039122, 40485, 275, 13, 25, "Text",
   CellTags->"FV"],
  Cell[1039400, 40500, 562, 29, 25, "Text",
   CellTags->"FV"],
  Cell[1039999, 40534, 50, 1, 34, "Subsubsection",
   CellTags->"FV"],
  Cell[1040074, 40539, 124, 4, 26, "Input",
   CellTags->"FV"],
  Cell[1040201, 40545, 168, 5, 28, "Output",
   CellTags->"FV"],
  Cell[1040406, 40555, 148, 5, 26, "Input",
   CellTags->"FV"],
  Cell[1040557, 40562, 229, 8, 28, "Output",
   CellTags->"FV"],
  Cell[1040823, 40575, 160, 5, 26, "Input",
   CellTags->"FV"],
  Cell[1040986, 40582, 181, 5, 38, "Output",
   CellTags->"FV"],
  Cell[1041204, 40592, 195, 7, 26, "Input",
   CellTags->"FV"],
  Cell[1041402, 40601, 259, 7, 38, "Output",
   CellTags->"FV"],
  Cell[1041676, 40611, 130, 4, 24, "Text",
   CellTags->"FV"],
  Cell[1041831, 40619, 196, 6, 26, "Input",
   CellTags->"FV"],
  Cell[1042030, 40627, 227, 8, 28, "Output",
   CellTags->"FV"],
  Cell[1042294, 40640, 107, 3, 26, "Input",
   CellTags->"FV"],
  Cell[1042404, 40645, 205, 6, 38, "Output",
   CellTags->"FV"]},
 "FVD"->{
  Cell[1042636, 40655, 43, 1, 41, "ObjectName",
   CellTags->"FVD"],
  Cell[1042704, 40660, 54, 1, 36, "Subsubsection",
   CellTags->"FVD"],
  Cell[1042761, 40663, 312, 15, 25, "Text",
   CellTags->"FVD"],
  Cell[1043076, 40680, 560, 29, 25, "Text",
   CellTags->"FVD"],
  Cell[1043673, 40714, 51, 1, 34, "Subsubsection",
   CellTags->"FVD"],
  Cell[1043749, 40719, 126, 4, 26, "Input",
   CellTags->"FVD"],
  Cell[1043878, 40725, 169, 5, 28, "Output",
   CellTags->"FVD"],
  Cell[1044084, 40735, 150, 5, 26, "Input",
   CellTags->"FVD"],
  Cell[1044237, 40742, 230, 8, 28, "Output",
   CellTags->"FVD"],
  Cell[1044504, 40755, 162, 5, 26, "Input",
   CellTags->"FVD"],
  Cell[1044669, 40762, 183, 5, 38, "Output",
   CellTags->"FVD"],
  Cell[1044889, 40772, 197, 7, 26, "Input",
   CellTags->"FVD"],
  Cell[1045089, 40781, 310, 9, 38, "Output",
   CellTags->"FVD"],
  Cell[1045414, 40793, 88, 1, 24, "Text",
   CellTags->"FVD"],
  Cell[1045527, 40798, 198, 6, 26, "Input",
   CellTags->"FVD"],
  Cell[1045728, 40806, 230, 8, 28, "Output",
   CellTags->"FVD"],
  Cell[1045995, 40819, 108, 3, 26, "Input",
   CellTags->"FVD"],
  Cell[1046106, 40824, 207, 6, 38, "Output",
   CellTags->"FVD"]},
 "GA"->{
  Cell[1046340, 40834, 41, 1, 41, "ObjectName",
   CellTags->"GA"],
  Cell[1046406, 40839, 53, 1, 36, "Subsubsection",
   CellTags->"GA"],
  Cell[1046462, 40842, 654, 24, 43, "Text",
   CellTags->"GA"],
  Cell[1047119, 40868, 369, 19, 25, "Text",
   CellTags->"GA"],
  Cell[1047525, 40892, 50, 1, 34, "Subsubsection",
   CellTags->"GA"],
  Cell[1047600, 40897, 101, 3, 26, "Input",
   CellTags->"GA"],
  Cell[1047704, 40902, 177, 5, 26, "Output",
   CellTags->"GA"],
  Cell[1047918, 40912, 213, 7, 26, "Input",
   CellTags->"GA"],
  Cell[1048134, 40921, 359, 11, 26, "Output",
   CellTags->"GA"],
  Cell[1048530, 40937, 171, 5, 26, "Input",
   CellTags->"GA"],
  Cell[1048704, 40944, 205, 5, 38, "Output",
   CellTags->"GA"],
  Cell[1048946, 40954, 163, 5, 26, "Input",
   CellTags->"GA"],
  Cell[1049112, 40961, 331, 9, 26, "Output",
   CellTags->"GA"],
  Cell[1049480, 40975, 203, 6, 26, "Input",
   CellTags->"GA"],
  Cell[1049686, 40983, 306, 8, 38, "Output",
   CellTags->"GA"],
  Cell[1050029, 40996, 250, 8, 26, "Input",
   CellTags->"GA"],
  Cell[1050282, 41006, 343, 10, 28, "Output",
   CellTags->"GA"]},
 "GAD"->{
  Cell[1050652, 41020, 43, 1, 41, "ObjectName",
   CellTags->"GAD"],
  Cell[1050720, 41025, 54, 1, 36, "Subsubsection",
   CellTags->"GAD"],
  Cell[1050777, 41028, 671, 24, 43, "Text",
   CellTags->"GAD"],
  Cell[1051451, 41054, 367, 19, 25, "Text",
   CellTags->"GAD"],
  Cell[1051855, 41078, 51, 1, 34, "Subsubsection",
   CellTags->"GAD"],
  Cell[1051931, 41083, 103, 3, 26, "Input",
   CellTags->"GAD"],
  Cell[1052037, 41088, 176, 5, 26, "Output",
   CellTags->"GAD"],
  Cell[1052250, 41098, 216, 7, 26, "Input",
   CellTags->"GAD"],
  Cell[1052469, 41107, 360, 11, 26, "Output",
   CellTags->"GAD"],
  Cell[1052866, 41123, 173, 5, 26, "Input",
   CellTags->"GAD"],
  Cell[1053042, 41130, 255, 7, 38, "Output",
   CellTags->"GAD"],
  Cell[1053334, 41142, 165, 5, 26, "Input",
   CellTags->"GAD"],
  Cell[1053502, 41149, 332, 9, 26, "Output",
   CellTags->"GAD"],
  Cell[1053871, 41163, 205, 6, 26, "Input",
   CellTags->"GAD"],
  Cell[1054079, 41171, 311, 8, 38, "Output",
   CellTags->"GAD"],
  Cell[1054427, 41184, 254, 8, 26, "Input",
   CellTags->"GAD"],
  Cell[1054684, 41194, 344, 10, 28, "Output",
   CellTags->"GAD"]},
 "GammaEpsilon"->{
  Cell[1055055, 41208, 61, 1, 39, "ObjectName",
   CellTags->"GammaEpsilon"],
  Cell[1055141, 41213, 63, 1, 34, "Subsubsection",
   CellTags->"GammaEpsilon"],
  Cell[1055207, 41216, 167, 4, 24, "Text",
   CellTags->"GammaEpsilon"],
  Cell[1055377, 41222, 299, 14, 24, "Text",
   CellTags->"GammaEpsilon"],
  Cell[1055713, 41241, 60, 1, 34, "Subsubsection",
   CellTags->"GammaEpsilon"],
  Cell[1055776, 41244, 141, 4, 24, "Text",
   CellTags->"GammaEpsilon"],
  Cell[1055942, 41252, 170, 5, 26, "Input",
   CellTags->"GammaEpsilon"],
  Cell[1056115, 41259, 1403, 49, 48, "Output",
   CellTags->"GammaEpsilon"],
  Cell[1057555, 41313, 170, 5, 26, "Input",
   CellTags->"GammaEpsilon"],
  Cell[1057728, 41320, 1151, 42, 48, "Output",
   CellTags->"GammaEpsilon"],
  Cell[1058894, 41365, 91, 1, 24, "Text",
   CellTags->"GammaEpsilon"],
  Cell[1059010, 41370, 123, 3, 26, "Input",
   CellTags->"GammaEpsilon"],
  Cell[1059136, 41375, 2192, 78, 96, "Output",
   CellTags->"GammaEpsilon"],
  Cell[1061365, 41458, 117, 3, 26, "Input",
   CellTags->"GammaEpsilon"],
  Cell[1061485, 41463, 180, 6, 26, "Output",
   CellTags->"GammaEpsilon"]},
 "GammaExpand"->{
  Cell[1061692, 41473, 59, 1, 39, "ObjectName",
   CellTags->"GammaExpand"],
  Cell[1061776, 41478, 62, 1, 34, "Subsubsection",
   CellTags->"GammaExpand"],
  Cell[1061841, 41481, 123, 3, 24, "Text",
   CellTags->"GammaExpand"],
  Cell[1061967, 41486, 194, 9, 24, "Text",
   CellTags->"GammaExpand"],
  Cell[1062198, 41500, 59, 1, 34, "Subsubsection",
   CellTags->"GammaExpand"],
  Cell[1062282, 41505, 187, 5, 26, "Input",
   CellTags->"GammaExpand"],
  Cell[1062472, 41512, 371, 12, 26, "Output",
   CellTags->"GammaExpand"],
  Cell[1062880, 41529, 197, 6, 26, "Input",
   CellTags->"GammaExpand"],
  Cell[1063080, 41537, 674, 23, 47, "Output",
   CellTags->"GammaExpand"],
  Cell[1063791, 41565, 187, 5, 26, "Input",
   CellTags->"GammaExpand"],
  Cell[1063981, 41572, 245, 7, 26, "Output",
   CellTags->"GammaExpand"]},
 "Gamma1"->{
  Cell[1064253, 41583, 49, 1, 39, "ObjectName",
   CellTags->"Gamma1"],
  Cell[1064327, 41588, 57, 1, 34, "Subsubsection",
   CellTags->"Gamma1"],
  Cell[1064387, 41591, 139, 4, 24, "Text",
   CellTags->"Gamma1"],
  Cell[1064529, 41597, 275, 14, 24, "Text",
   CellTags->"Gamma1"]},
 "Gamma2"->{
  Cell[1064819, 41614, 49, 1, 39, "ObjectName",
   CellTags->"Gamma2"],
  Cell[1064893, 41619, 57, 1, 34, "Subsubsection",
   CellTags->"Gamma2"],
  Cell[1064953, 41622, 172, 4, 24, "Text",
   CellTags->"Gamma2"],
  Cell[1065128, 41628, 275, 14, 24, "Text",
   CellTags->"Gamma2"]},
 "Gamma3"->{
  Cell[1065418, 41645, 49, 1, 39, "ObjectName",
   CellTags->"Gamma3"],
  Cell[1065492, 41650, 57, 1, 34, "Subsubsection",
   CellTags->"Gamma3"],
  Cell[1065552, 41653, 237, 5, 41, "Text",
   CellTags->"Gamma3"],
  Cell[1065792, 41660, 275, 14, 24, "Text",
   CellTags->"Gamma3"]},
 "Gauge"->{
  Cell[1066082, 41677, 47, 1, 39, "ObjectName",
   CellTags->"Gauge"],
  Cell[1066154, 41682, 56, 1, 34, "Subsubsection",
   CellTags->"Gauge"],
  Cell[1066213, 41685, 133, 4, 24, "Text",
   CellTags->"Gauge"],
  Cell[1066349, 41691, 301, 14, 24, "Text",
   CellTags->"Gauge"]},
 "GaugeField"->{
  Cell[1066665, 41708, 57, 1, 39, "ObjectName",
   CellTags->"GaugeField"],
  Cell[1066747, 41713, 61, 1, 34, "Subsubsection",
   CellTags->"GaugeField"],
  Cell[1066811, 41716, 212, 5, 41, "Text",
   CellTags->"GaugeField"],
  Cell[1067026, 41723, 318, 14, 24, "Text",
   CellTags->"GaugeField"],
  Cell[1067381, 41742, 58, 1, 34, "Subsubsection",
   CellTags->"GaugeField"],
  Cell[1067464, 41747, 86, 2, 26, "Input",
   CellTags->"GaugeField"],
  Cell[1067553, 41751, 148, 4, 27, "Output",
   CellTags->"GaugeField"],
  Cell[1067738, 41760, 237, 6, 26, "Input",
   CellTags->"GaugeField"],
  Cell[1067978, 41768, 358, 15, 30, "Output",
   CellTags->"GaugeField"]},
 "GaugeXi"->{
  Cell[1068363, 41787, 51, 1, 39, "ObjectName",
   CellTags->"GaugeXi"],
  Cell[1068439, 41792, 58, 1, 34, "Subsubsection",
   CellTags->"GaugeXi"],
  Cell[1068500, 41795, 77, 1, 24, "Text",
   CellTags->"GaugeXi"],
  Cell[1068580, 41798, 285, 14, 24, "Text",
   CellTags->"GaugeXi"]},
 "GA5"->{
  Cell[1068880, 41815, 43, 1, 39, "ObjectName",
   CellTags->"GA5"],
  Cell[1068948, 41820, 54, 1, 34, "Subsubsection",
   CellTags->"GA5"],
  Cell[1069005, 41823, 88, 1, 24, "Text",
   CellTags->"GA5"],
  Cell[1069096, 41826, 364, 19, 24, "Text",
   CellTags->"GA5"],
  Cell[1069497, 41850, 51, 1, 34, "Subsubsection",
   CellTags->"GA5"],
  Cell[1069573, 41855, 72, 2, 26, "Input",
   CellTags->"GA5"],
  Cell[1069648, 41859, 174, 5, 26, "Output",
   CellTags->"GA5"],
  Cell[1069859, 41869, 94, 3, 26, "Input",
   CellTags->"GA5"],
  Cell[1069956, 41874, 143, 4, 26, "Output",
   CellTags->"GA5"]},
 "GGV"->{
  Cell[1070126, 41882, 43, 1, 39, "ObjectName",
   CellTags->"GGV"],
  Cell[1070194, 41887, 54, 1, 34, "Subsubsection",
   CellTags->"GGV"],
  Cell[1070251, 41890, 72, 1, 24, "Text",
   CellTags->"GGV"],
  Cell[1070326, 41893, 198, 9, 24, "Text",
   CellTags->"GGV"]},
 "GhostPropagator"->{
  Cell[1070539, 41905, 67, 1, 39, "ObjectName",
   CellTags->"GhostPropagator"],
  Cell[1070631, 41910, 66, 1, 34, "Subsubsection",
   CellTags->"GhostPropagator"],
  Cell[1070700, 41913, 324, 9, 26, "Text",
   CellTags->"GhostPropagator"],
  Cell[1071027, 41924, 436, 19, 24, "Text",
   CellTags->"GhostPropagator"],
  Cell[1071828, 41963, 63, 1, 34, "Subsubsection",
   CellTags->"GhostPropagator"],
  Cell[1071916, 41968, 156, 4, 26, "Input",
   CellTags->"GhostPropagator"],
  Cell[1072075, 41974, 234, 7, 28, "Output",
   CellTags->"GhostPropagator"],
  Cell[1072346, 41986, 206, 5, 26, "Input",
   CellTags->"GhostPropagator"],
  Cell[1072555, 41993, 451, 17, 48, "Output",
   CellTags->"GhostPropagator"],
  Cell[1073043, 42015, 123, 3, 26, "Input",
   CellTags->"GhostPropagator"],
  Cell[1073169, 42020, 220, 7, 28, "Output",
   CellTags->"GhostPropagator"],
  Cell[1073426, 42032, 188, 5, 26, "Input",
   CellTags->"GhostPropagator"],
  Cell[1073617, 42039, 455, 17, 49, "Output",
   CellTags->"GhostPropagator"],
  Cell[1074109, 42061, 285, 9, 26, "Input",
   CellTags->"GhostPropagator"],
  Cell[1074397, 42072, 270, 7, 38, "Output",
   CellTags->"GhostPropagator"]},
 "GHP"->{
  Cell[1074694, 42083, 43, 1, 39, "ObjectName",
   CellTags->"GHP"],
  Cell[1074762, 42088, 54, 1, 34, "Subsubsection",
   CellTags->"GHP"],
  Cell[1074819, 42091, 288, 9, 26, "Text",
   CellTags->"GHP"],
  Cell[1075110, 42102, 460, 19, 24, "Text",
   CellTags->"GHP"],
  Cell[1075607, 42126, 51, 1, 34, "Subsubsection",
   CellTags->"GHP"],
  Cell[1075683, 42131, 132, 4, 26, "Input",
   CellTags->"GHP"],
  Cell[1075818, 42137, 223, 7, 28, "Output",
   CellTags->"GHP"],
  Cell[1076078, 42149, 130, 4, 26, "Input",
   CellTags->"GHP"],
  Cell[1076211, 42155, 199, 6, 49, "Output",
   CellTags->"GHP"],
  Cell[1076447, 42166, 132, 4, 26, "Input",
   CellTags->"GHP"],
  Cell[1076582, 42172, 227, 7, 28, "Output",
   CellTags->"GHP"],
  Cell[1076846, 42184, 264, 9, 26, "Input",
   CellTags->"GHP"],
  Cell[1077113, 42195, 260, 7, 38, "Output",
   CellTags->"GHP"]},
 "GluonField"->{
  Cell[1077400, 42206, 57, 1, 41, "ObjectName",
   CellTags->"GluonField"],
  Cell[1077482, 42211, 61, 1, 36, "Subsubsection",
   CellTags->"GluonField"],
  Cell[1077546, 42214, 79, 1, 25, "Text",
   CellTags->"GluonField"],
  Cell[1077628, 42217, 187, 9, 25, "Text",
   CellTags->"GluonField"],
  Cell[1077852, 42231, 58, 1, 34, "Subsubsection",
   CellTags->"GluonField"],
  Cell[1077935, 42236, 86, 2, 26, "Input",
   CellTags->"GluonField"],
  Cell[1078024, 42240, 147, 4, 27, "Output",
   CellTags->"GluonField"],
  Cell[1078208, 42249, 237, 6, 26, "Input",
   CellTags->"GluonField"],
  Cell[1078448, 42257, 360, 15, 30, "Output",
   CellTags->"GluonField"]},
 "GluonGhostVertex"->{
  Cell[1078835, 42276, 69, 1, 41, "ObjectName",
   CellTags->"GluonGhostVertex"],
  Cell[1078929, 42281, 67, 1, 36, "Subsubsection",
   CellTags->"GluonGhostVertex"],
  Cell[1078999, 42284, 422, 7, 61, "Text",
   CellTags->"GluonGhostVertex"],
  Cell[1079424, 42293, 583, 23, 24, "Text",
   CellTags->"GluonGhostVertex"],
  Cell[1080032, 42320, 131, 3, 26, "Input",
   CellTags->"GluonGhostVertex"],
  Cell[1080166, 42325, 368, 11, 26, "Output",
   CellTags->"GluonGhostVertex"],
  Cell[1080583, 42342, 64, 1, 34, "Subsubsection",
   CellTags->"GluonGhostVertex"],
  Cell[1080672, 42347, 348, 10, 26, "Input",
   CellTags->"GluonGhostVertex"],
  Cell[1081023, 42359, 558, 22, 30, "Output",
   CellTags->"GluonGhostVertex"],
  Cell[1083202, 42461, 148, 4, 26, "Input",
   CellTags->"GluonGhostVertex"],
  Cell[1083353, 42467, 377, 11, 38, "Output",
   CellTags->"GluonGhostVertex"],
  Cell[1083767, 42483, 191, 5, 26, "Input",
   CellTags->"GluonGhostVertex"],
  Cell[1083961, 42490, 563, 22, 31, "Output",
   CellTags->"GluonGhostVertex"]},
 "GluonPropagator"->{
  Cell[1084551, 42516, 67, 1, 41, "ObjectName",
   CellTags->"GluonPropagator"],
  Cell[1084643, 42521, 66, 1, 36, "Subsubsection",
   CellTags->"GluonPropagator"],
  Cell[1084712, 42524, 535, 10, 70, "Text",
   CellTags->"GluonPropagator"],
  Cell[1085250, 42536, 99, 1, 24, "Text",
   CellTags->"GluonPropagator"],
  Cell[1085374, 42541, 83, 2, 26, "Input",
   CellTags->"GluonPropagator"],
  Cell[1085460, 42545, 167, 4, 26, "Output",
   CellTags->"GluonPropagator"],
  Cell[1085664, 42554, 129, 3, 26, "Input",
   CellTags->"GluonPropagator"],
  Cell[1085796, 42559, 524, 14, 26, "Output",
   CellTags->"GluonPropagator"],
  Cell[1086335, 42576, 591, 24, 70, "Text",
   CellTags->"GluonPropagator"],
  Cell[1086963, 42605, 72, 1, 33, "Subsubsection",
   CellTags->"GluonPropagator"],
  Cell[1087060, 42610, 187, 5, 26, "Input",
   CellTags->"GluonPropagator"],
  Cell[1087250, 42617, 275, 9, 29, "Output",
   CellTags->"GluonPropagator"],
  Cell[1088310, 42662, 271, 7, 26, "Input",
   CellTags->"GluonPropagator"],
  Cell[1088584, 42671, 1075, 40, 61, "Output",
   CellTags->"GluonPropagator"],
  Cell[1089696, 42716, 357, 10, 26, "Input",
   CellTags->"GluonPropagator"],
  Cell[1090056, 42728, 2792, 105, 61, "Output",
   CellTags->"GluonPropagator"],
  Cell[1092885, 42838, 199, 6, 26, "Input",
   CellTags->"GluonPropagator"],
  Cell[1093087, 42846, 312, 10, 49, "Output",
   CellTags->"GluonPropagator"],
  Cell[1093414, 42859, 149, 4, 24, "Text",
   CellTags->"GluonPropagator"],
  Cell[1093588, 42867, 242, 7, 26, "Input",
   CellTags->"GluonPropagator"],
  Cell[1093833, 42876, 1072, 40, 62, "Output",
   CellTags->"GluonPropagator"],
  Cell[1094942, 42921, 267, 9, 26, "Input",
   CellTags->"GluonPropagator"],
  Cell[1095212, 42932, 358, 10, 38, "Output",
   CellTags->"GluonPropagator"],
  Cell[1095607, 42947, 275, 7, 26, "Input",
   CellTags->"GluonPropagator"],
  Cell[1095885, 42956, 1316, 47, 55, "Output",
   CellTags->"GluonPropagator"],
  Cell[1097238, 43008, 275, 7, 26, "Input",
   CellTags->"GluonPropagator"],
  Cell[1097516, 43017, 1332, 48, 52, "Output",
   CellTags->"GluonPropagator"],
  Cell[1098885, 43070, 275, 7, 26, "Input",
   CellTags->"GluonPropagator"],
  Cell[1099163, 43079, 1323, 47, 55, "Output",
   CellTags->"GluonPropagator"],
  Cell[1100523, 43131, 275, 7, 26, "Input",
   CellTags->"GluonPropagator"],
  Cell[1100801, 43140, 1318, 47, 55, "Output",
   CellTags->"GluonPropagator"],
  Cell[1102156, 43192, 275, 7, 26, "Input",
   CellTags->"GluonPropagator"],
  Cell[1102434, 43201, 2464, 89, 55, "Output",
   CellTags->"GluonPropagator"]},
 "GluonSelfEnergy"->{
  Cell[1104925, 43294, 67, 1, 39, "ObjectName",
   CellTags->"GluonSelfEnergy"],
  Cell[1105017, 43299, 66, 1, 34, "Subsubsection",
   CellTags->"GluonSelfEnergy"],
  Cell[1105086, 43302, 122, 3, 24, "Text",
   CellTags->"GluonSelfEnergy"],
  Cell[1105233, 43309, 129, 3, 26, "Input",
   CellTags->"GluonSelfEnergy"],
  Cell[1105365, 43314, 835, 24, 52, "Output",
   CellTags->"GluonSelfEnergy"],
  Cell[1106215, 43341, 591, 24, 24, "Text",
   CellTags->"GluonSelfEnergy"],
  Cell[1106843, 43370, 72, 1, 33, "Subsubsection",
   CellTags->"GluonSelfEnergy"],
  Cell[1106940, 43375, 252, 8, 26, "Input",
   CellTags->"GluonSelfEnergy"],
  Cell[1107195, 43385, 2433, 93, 45, "Output",
   CellTags->"GluonSelfEnergy"],
  Cell[1109665, 43483, 300, 9, 26, "Input",
   CellTags->"GluonSelfEnergy"],
  Cell[1109968, 43494, 2776, 104, 101, "Output",
   CellTags->"GluonSelfEnergy"]},
 "GluonVertex"->{
  Cell[1112771, 43602, 59, 1, 39, "ObjectName",
   CellTags->"GluonVertex"],
  Cell[1112855, 43607, 62, 1, 34, "Subsubsection",
   CellTags->"GluonVertex"],
  Cell[1112920, 43610, 88, 1, 24, "Text",
   CellTags->"GluonVertex"],
  Cell[1113011, 43613, 621, 10, 75, "Text",
   CellTags->"GluonVertex"],
  Cell[1113635, 43625, 91, 1, 24, "Text",
   CellTags->"GluonVertex"],
  Cell[1113751, 43630, 79, 2, 26, "Input",
   CellTags->"GluonVertex"],
  Cell[1113833, 43634, 157, 4, 26, "Output",
   CellTags->"GluonVertex"],
  Cell[1114027, 43643, 121, 3, 26, "Input",
   CellTags->"GluonVertex"],
  Cell[1114151, 43648, 419, 12, 26, "Output",
   CellTags->"GluonVertex"],
  Cell[1114585, 43663, 337, 14, 24, "Text",
   CellTags->"GluonVertex"],
  Cell[1114959, 43682, 59, 1, 34, "Subsubsection",
   CellTags->"GluonVertex"],
  Cell[1115043, 43687, 338, 10, 26, "Input",
   CellTags->"GluonVertex"],
  Cell[1115384, 43699, 987, 43, 28, "Output",
   CellTags->"GluonVertex"],
  Cell[1116408, 43747, 379, 12, 26, "Input",
   CellTags->"GluonVertex"],
  Cell[1116790, 43761, 2016, 80, 28, "Output",
   CellTags->"GluonVertex"],
  Cell[1118843, 43846, 308, 10, 26, "Input",
   CellTags->"GluonVertex"],
  Cell[1119154, 43858, 314, 10, 28, "Output",
   CellTags->"GluonVertex"],
  Cell[1119505, 43873, 349, 12, 26, "Input",
   CellTags->"GluonVertex"],
  Cell[1119857, 43887, 1724, 66, 28, "Output",
   CellTags->"GluonVertex"],
  Cell[1121618, 43958, 414, 12, 26, "Input",
   CellTags->"GluonVertex"],
  Cell[1122035, 43972, 394, 12, 29, "Output",
   CellTags->"GluonVertex"],
  Cell[1126432, 44141, 374, 12, 26, "Input",
   CellTags->"GluonVertex"],
  Cell[1126809, 44155, 202, 6, 28, "Output",
   CellTags->"GluonVertex"],
  Cell[1127048, 44166, 259, 7, 26, "Input",
   CellTags->"GluonVertex"],
  Cell[1127310, 44175, 3523, 133, 50, "Output",
   CellTags->"GluonVertex"],
  Cell[1130848, 44311, 200, 4, 41, "Text",
   CellTags->"GluonVertex"],
  Cell[1131073, 44319, 172, 5, 26, "Input",
   CellTags->"GluonVertex"],
  Cell[1131248, 44326, 984, 43, 30, "Output",
   CellTags->"GluonVertex"],
  Cell[1132269, 44374, 230, 7, 26, "Input",
   CellTags->"GluonVertex"],
  Cell[1132502, 44383, 3472, 133, 55, "Output",
   CellTags->"GluonVertex"]},
 "GO"->{
  Cell[1136001, 44520, 41, 1, 39, "ObjectName",
   CellTags->"GO"],
  Cell[1136067, 44525, 53, 1, 34, "Subsubsection",
   CellTags->"GO"],
  Cell[1136123, 44528, 73, 1, 24, "Text",
   CellTags->"GO"],
  Cell[1136199, 44531, 206, 9, 24, "Text",
   CellTags->"GO"]},
 "GP"->{
  Cell[1136420, 44543, 41, 1, 39, "ObjectName",
   CellTags->"GP"],
  Cell[1136486, 44548, 53, 1, 34, "Subsubsection",
   CellTags->"GP"],
  Cell[1136542, 44551, 69, 1, 24, "Text",
   CellTags->"GP"],
  Cell[1136614, 44554, 194, 9, 24, "Text",
   CellTags->"GP"]},
 "GrassmannParity"->{
  Cell[1136823, 44566, 67, 1, 39, "ObjectName",
   CellTags->"GrassmannParity"],
  Cell[1136915, 44571, 66, 1, 34, "Subsubsection",
   CellTags->"GrassmannParity"],
  Cell[1136984, 44574, 216, 5, 41, "Text",
   CellTags->"GrassmannParity"],
  Cell[1137203, 44581, 186, 9, 24, "Text",
   CellTags->"GrassmannParity"]},
 "GS"->{
  Cell[1137404, 44593, 41, 1, 39, "ObjectName",
   CellTags->"GS"],
  Cell[1137470, 44598, 53, 1, 34, "Subsubsection",
   CellTags->"GS"],
  Cell[1137526, 44601, 635, 18, 44, "Text",
   CellTags->"GS"],
  Cell[1138164, 44621, 482, 24, 24, "Text",
   CellTags->"GS"],
  Cell[1138683, 44650, 50, 1, 34, "Subsubsection",
   CellTags->"GS"],
  Cell[1138758, 44655, 97, 3, 26, "Input",
   CellTags->"GS"],
  Cell[1138858, 44660, 183, 5, 28, "Output",
   CellTags->"GS"],
  Cell[1139078, 44670, 162, 6, 26, "Input",
   CellTags->"GS"],
  Cell[1139243, 44678, 197, 5, 38, "Output",
   CellTags->"GS"],
  Cell[1139477, 44688, 140, 4, 26, "Input",
   CellTags->"GS"],
  Cell[1139620, 44694, 454, 14, 28, "Output",
   CellTags->"GS"],
  Cell[1140111, 44713, 179, 6, 26, "Input",
   CellTags->"GS"],
  Cell[1140293, 44721, 284, 8, 38, "Output",
   CellTags->"GS"],
  Cell[1140614, 44734, 237, 8, 26, "Input",
   CellTags->"GS"],
  Cell[1140854, 44744, 405, 13, 28, "Output",
   CellTags->"GS"]},
 "GSD"->{
  Cell[1141286, 44761, 43, 1, 39, "ObjectName",
   CellTags->"GSD"],
  Cell[1141354, 44766, 54, 1, 34, "Subsubsection",
   CellTags->"GSD"],
  Cell[1141411, 44769, 644, 18, 44, "Text",
   CellTags->"GSD"],
  Cell[1142058, 44789, 483, 24, 24, "Text",
   CellTags->"GSD"],
  Cell[1142578, 44818, 51, 1, 34, "Subsubsection",
   CellTags->"GSD"],
  Cell[1142654, 44823, 99, 3, 26, "Input",
   CellTags->"GSD"],
  Cell[1142756, 44828, 182, 5, 28, "Output",
   CellTags->"GSD"],
  Cell[1142975, 44838, 164, 6, 26, "Input",
   CellTags->"GSD"],
  Cell[1143142, 44846, 249, 7, 38, "Output",
   CellTags->"GSD"],
  Cell[1143428, 44858, 142, 4, 26, "Input",
   CellTags->"GSD"],
  Cell[1143573, 44864, 455, 14, 28, "Output",
   CellTags->"GSD"],
  Cell[1144065, 44883, 181, 6, 26, "Input",
   CellTags->"GSD"],
  Cell[1144249, 44891, 289, 8, 38, "Output",
   CellTags->"GSD"],
  Cell[1144575, 44904, 241, 8, 26, "Input",
   CellTags->"GSD"],
  Cell[1144819, 44914, 406, 13, 28, "Output",
   CellTags->"GSD"]},
 "Gstrong"->{
  Cell[1145252, 44931, 51, 1, 39, "ObjectName",
   CellTags->"Gstrong"],
  Cell[1145328, 44936, 58, 1, 34, "Subsubsection",
   CellTags->"Gstrong"],
  Cell[1145389, 44939, 83, 1, 24, "Text",
   CellTags->"Gstrong"],
  Cell[1145475, 44942, 334, 15, 24, "Text",
   CellTags->"Gstrong"],
  Cell[1145846, 44962, 55, 1, 34, "Subsubsection",
   CellTags->"Gstrong"],
  Cell[1145904, 44965, 119, 3, 24, "Text",
   CellTags->"Gstrong"],
  Cell[1146048, 44972, 80, 2, 26, "Input",
   CellTags->"Gstrong"],
  Cell[1146131, 44976, 167, 5, 26, "Output",
   CellTags->"Gstrong"]},
 "GTI"->{
  Cell[1146325, 44985, 43, 1, 39, "ObjectName",
   CellTags->"GTI"],
  Cell[1146393, 44990, 54, 1, 34, "Subsubsection",
   CellTags->"GTI"],
  Cell[1146450, 44993, 85, 1, 24, "Text",
   CellTags->"GTI"],
  Cell[1146538, 44996, 159, 9, 24, "Text",
   CellTags->"GTI"]},
 "GV"->{
  Cell[1146712, 45008, 41, 1, 39, "ObjectName",
   CellTags->"GV"],
  Cell[1146778, 45013, 53, 1, 34, "Subsubsection",
   CellTags->"GV"],
  Cell[1146834, 45016, 65, 1, 24, "Text",
   CellTags->"GV"],
  Cell[1146902, 45019, 182, 9, 24, "Text",
   CellTags->"GV"]},
 "Hill"->{
  Cell[1147099, 45031, 45, 1, 39, "ObjectName",
   CellTags->"Hill"],
  Cell[1147169, 45036, 55, 1, 34, "Subsubsection",
   CellTags->"Hill"],
  Cell[1147227, 45039, 129, 4, 24, "Text",
   CellTags->"Hill"],
  Cell[1147359, 45045, 196, 9, 24, "Text",
   CellTags->"Hill"],
  Cell[1147592, 45059, 52, 1, 34, "Subsubsection",
   CellTags->"Hill"],
  Cell[1147669, 45064, 124, 4, 26, "Input",
   CellTags->"Hill"],
  Cell[1147796, 45070, 2187, 74, 137, "Output",
   CellTags->"Hill"],
  Cell[1150020, 45149, 270, 8, 26, "Input",
   CellTags->"Hill"],
  Cell[1150293, 45159, 142, 4, 26, "Output",
   CellTags->"Hill"],
  Cell[1150472, 45168, 262, 9, 26, "Input",
   CellTags->"Hill"],
  Cell[1150737, 45179, 1029, 33, 71, "Output",
   CellTags->"Hill"],
  Cell[1151803, 45217, 273, 9, 26, "Input",
   CellTags->"Hill"],
  Cell[1152079, 45228, 142, 4, 26, "Output",
   CellTags->"Hill"]},
 "HypergeometricAC"->{
  Cell[1152248, 45236, 69, 1, 39, "ObjectName",
   CellTags->"HypergeometricAC"],
  Cell[1152342, 45241, 67, 1, 34, "Subsubsection",
   CellTags->"HypergeometricAC"],
  Cell[1152412, 45244, 553, 9, 75, "Text",
   CellTags->"HypergeometricAC"],
  Cell[1152990, 45257, 131, 3, 26, "Input",
   CellTags->"HypergeometricAC"],
  Cell[1153124, 45262, 219, 7, 26, "Output",
   CellTags->"HypergeometricAC"],
  Cell[1153358, 45272, 598, 24, 25, "Text",
   CellTags->"HypergeometricAC"],
  Cell[1153993, 45301, 64, 1, 36, "Subsubsection",
   CellTags->"HypergeometricAC"],
  Cell[1154060, 45304, 101, 1, 24, "Text",
   CellTags->"HypergeometricAC"],
  Cell[1154186, 45309, 264, 7, 26, "Input",
   CellTags->"HypergeometricAC"],
  Cell[1154453, 45318, 1455, 44, 92, "Output",
   CellTags->"HypergeometricAC"],
  Cell[1155945, 45367, 264, 7, 26, "Input",
   CellTags->"HypergeometricAC"],
  Cell[1156212, 45376, 1416, 44, 55, "Output",
   CellTags->"HypergeometricAC"],
  Cell[1157665, 45425, 264, 7, 26, "Input",
   CellTags->"HypergeometricAC"],
  Cell[1157932, 45434, 1460, 45, 107, "Output",
   CellTags->"HypergeometricAC"],
  Cell[1159429, 45484, 264, 7, 26, "Input",
   CellTags->"HypergeometricAC"],
  Cell[1159696, 45493, 1737, 52, 107, "Output",
   CellTags->"HypergeometricAC"],
  Cell[1161470, 45550, 264, 7, 26, "Input",
   CellTags->"HypergeometricAC"],
  Cell[1161737, 45559, 457, 14, 42, "Output",
   CellTags->"HypergeometricAC"],
  Cell[1162231, 45578, 265, 7, 26, "Input",
   CellTags->"HypergeometricAC"],
  Cell[1162499, 45587, 460, 14, 42, "Output",
   CellTags->"HypergeometricAC"]},
 "HypergeometricIR"->{
  Cell[1162986, 45605, 69, 1, 41, "ObjectName",
   CellTags->"HypergeometricIR"],
  Cell[1163080, 45610, 67, 1, 36, "Subsubsection",
   CellTags->"HypergeometricIR"],
  Cell[1163150, 45613, 269, 5, 43, "Text",
   CellTags->"HypergeometricIR"],
  Cell[1163422, 45620, 479, 19, 24, "Text",
   CellTags->"HypergeometricIR"],
  Cell[1163926, 45643, 131, 3, 26, "Input",
   CellTags->"HypergeometricIR"],
  Cell[1164060, 45648, 224, 7, 26, "Output",
   CellTags->"HypergeometricIR"],
  Cell[1164333, 45661, 64, 1, 34, "Subsubsection",
   CellTags->"HypergeometricIR"],
  Cell[1164422, 45666, 241, 7, 26, "Input",
   CellTags->"HypergeometricIR"],
  Cell[1164666, 45675, 681, 25, 50, "Output",
   CellTags->"HypergeometricIR"],
  Cell[1165384, 45705, 394, 14, 26, "Input",
   CellTags->"HypergeometricIR"],
  Cell[1165781, 45721, 532, 16, 48, "Output",
   CellTags->"HypergeometricIR"],
  Cell[1166350, 45742, 148, 4, 26, "Input",
   CellTags->"HypergeometricIR"],
  Cell[1166501, 45748, 394, 14, 28, "Output",
   CellTags->"HypergeometricIR"]},
 "HypergeometricSE"->{
  Cell[1166922, 45766, 69, 1, 41, "ObjectName",
   CellTags->"HypergeometricSE"],
  Cell[1167016, 45771, 67, 1, 36, "Subsubsection",
   CellTags->"HypergeometricSE"],
  Cell[1167086, 45774, 420, 13, 41, "Text",
   CellTags->"HypergeometricSE"],
  Cell[1167531, 45791, 131, 3, 26, "Input",
   CellTags->"HypergeometricSE"],
  Cell[1167665, 45796, 228, 7, 26, "Output",
   CellTags->"HypergeometricSE"],
  Cell[1167908, 45806, 211, 9, 25, "Text",
   CellTags->"HypergeometricSE"],
  Cell[1168156, 45820, 64, 1, 34, "Subsubsection",
   CellTags->"HypergeometricSE"],
  Cell[1168245, 45825, 245, 7, 26, "Input",
   CellTags->"HypergeometricSE"],
  Cell[1168493, 45834, 627, 22, 48, "Output",
   CellTags->"HypergeometricSE"],
  Cell[1169157, 45861, 351, 11, 26, "Input",
   CellTags->"HypergeometricSE"],
  Cell[1169511, 45874, 851, 30, 48, "Output",
   CellTags->"HypergeometricSE"]},
 "HypExplicit"->{
  Cell[1170389, 45908, 59, 1, 41, "ObjectName",
   CellTags->"HypExplicit"],
  Cell[1170473, 45913, 62, 1, 36, "Subsubsection",
   CellTags->"HypExplicit"],
  Cell[1170538, 45916, 325, 10, 43, "Text",
   CellTags->"HypExplicit"],
  Cell[1170866, 45928, 206, 9, 25, "Text",
   CellTags->"HypExplicit"],
  Cell[1171109, 45942, 59, 1, 34, "Subsubsection",
   CellTags->"HypExplicit"],
  Cell[1171193, 45947, 164, 4, 26, "Input",
   CellTags->"HypExplicit"],
  Cell[1171360, 45953, 203, 6, 26, "Output",
   CellTags->"HypExplicit"],
  Cell[1171600, 45964, 142, 4, 26, "Input",
   CellTags->"HypExplicit"],
  Cell[1171745, 45970, 622, 22, 48, "Output",
   CellTags->"HypExplicit"],
  Cell[1172404, 45997, 262, 8, 26, "Input",
   CellTags->"HypExplicit"],
  Cell[1172669, 46007, 1537, 56, 26, "Output",
   CellTags->"HypExplicit"],
  Cell[1174243, 46068, 142, 4, 26, "Input",
   CellTags->"HypExplicit"],
  Cell[1174388, 46074, 846, 30, 48, "Output",
   CellTags->"HypExplicit"]},
 "HypInt"->{
  Cell[1175261, 46108, 49, 1, 41, "ObjectName",
   CellTags->"HypInt"],
  Cell[1175335, 46113, 57, 1, 36, "Subsubsection",
   CellTags->"HypInt"],
  Cell[1175395, 46116, 206, 5, 43, "Text",
   CellTags->"HypInt"],
  Cell[1175638, 46126, 54, 1, 34, "Subsubsection",
   CellTags->"HypInt"],
  Cell[1175717, 46131, 159, 4, 26, "Input",
   CellTags->"HypInt"],
  Cell[1175879, 46137, 198, 6, 26, "Output",
   CellTags->"HypInt"],
  Cell[1176114, 46148, 128, 4, 26, "Input",
   CellTags->"HypInt"],
  Cell[1176245, 46154, 987, 37, 55, "Output",
   CellTags->"HypInt"]},
 "IFPD"->{
  Cell[1177259, 46195, 45, 1, 41, "ObjectName",
   CellTags->"IFPD"],
  Cell[1177329, 46200, 55, 1, 36, "Subsubsection",
   CellTags->"IFPD"],
  Cell[1177387, 46203, 66, 1, 25, "Text",
   CellTags->"IFPD"],
  Cell[1177456, 46206, 214, 9, 25, "Text",
   CellTags->"IFPD"]},
 "IFPDOff"->{
  Cell[1177685, 46218, 51, 1, 41, "ObjectName",
   CellTags->"IFPDOff"],
  Cell[1177761, 46223, 58, 1, 36, "Subsubsection",
   CellTags->"IFPDOff"],
  Cell[1177822, 46226, 182, 4, 43, "Text",
   CellTags->"IFPDOff"],
  Cell[1178007, 46232, 270, 14, 25, "Text",
   CellTags->"IFPDOff"],
  Cell[1178314, 46251, 55, 1, 34, "Subsubsection",
   CellTags->"IFPDOff"],
  Cell[1178394, 46256, 163, 5, 26, "Input",
   CellTags->"IFPDOff"],
  Cell[1178560, 46263, 426, 17, 28, "Output",
   CellTags->"IFPDOff"],
  Cell[1179023, 46285, 108, 3, 26, "Input",
   CellTags->"IFPDOff"],
  Cell[1179134, 46290, 219, 6, 38, "Output",
   CellTags->"IFPDOff"],
  Cell[1179390, 46301, 130, 4, 26, "Input",
   CellTags->"IFPDOff"],
  Cell[1179523, 46307, 254, 9, 28, "Output",
   CellTags->"IFPDOff"],
  Cell[1179814, 46321, 108, 3, 26, "Input",
   CellTags->"IFPDOff"],
  Cell[1179925, 46326, 325, 10, 38, "Output",
   CellTags->"IFPDOff"]},
 "IFPDOn"->{
  Cell[1180277, 46340, 49, 1, 41, "ObjectName",
   CellTags->"IFPDOn"],
  Cell[1180351, 46345, 57, 1, 36, "Subsubsection",
   CellTags->"IFPDOn"],
  Cell[1180411, 46348, 317, 6, 61, "Text",
   CellTags->"IFPDOn"],
  Cell[1180731, 46356, 272, 14, 25, "Text",
   CellTags->"IFPDOn"]},
 "IncludePair"->{
  Cell[1181018, 46373, 59, 1, 41, "ObjectName",
   CellTags->"IncludePair"],
  Cell[1181102, 46378, 62, 1, 36, "Subsubsection",
   CellTags->"IncludePair"],
  Cell[1181167, 46381, 124, 3, 25, "Text",
   CellTags->"IncludePair"],
  Cell[1181294, 46386, 176, 9, 25, "Text",
   CellTags->"IncludePair"]},
 "IndexPosition"->{
  Cell[1181485, 46398, 63, 1, 41, "ObjectName",
   CellTags->"IndexPosition"],
  Cell[1181573, 46403, 64, 1, 36, "Subsubsection",
   CellTags->"IndexPosition"],
  Cell[1181640, 46406, 89, 1, 25, "Text",
   CellTags->"IndexPosition"],
  Cell[1181732, 46409, 199, 9, 25, "Text",
   CellTags->"IndexPosition"]},
 "Indices"->{
  Cell[1181946, 46421, 51, 1, 41, "ObjectName",
   CellTags->"Indices"],
  Cell[1182022, 46426, 58, 1, 36, "Subsubsection",
   CellTags->"Indices"],
  Cell[1182083, 46429, 203, 4, 43, "Text",
   CellTags->"Indices"],
  Cell[1182289, 46435, 193, 9, 25, "Text",
   CellTags->"Indices"]},
 "InitialFunction"->{
  Cell[1182497, 46447, 67, 1, 41, "ObjectName",
   CellTags->"InitialFunction"],
  Cell[1182589, 46452, 66, 1, 36, "Subsubsection",
   CellTags->"InitialFunction"],
  Cell[1182658, 46455, 183, 4, 25, "Text",
   CellTags->"InitialFunction"],
  Cell[1182844, 46461, 186, 9, 25, "Text",
   CellTags->"InitialFunction"]},
 "InitialSubstitutions"->{
  Cell[1183045, 46473, 77, 1, 41, "ObjectName",
   CellTags->"InitialSubstitutions"],
  Cell[1183147, 46478, 71, 1, 36, "Subsubsection",
   CellTags->"InitialSubstitutions"],
  Cell[1183221, 46481, 208, 4, 25, "Text",
   CellTags->"InitialSubstitutions"],
  Cell[1183432, 46487, 408, 19, 25, "Text",
   CellTags->"InitialSubstitutions"]},
 "InsideDiracTrace"->{
  Cell[1183855, 46509, 69, 1, 41, "ObjectName",
   CellTags->"InsideDiracTrace"],
  Cell[1183949, 46514, 67, 1, 36, "Subsubsection",
   CellTags->"InsideDiracTrace"],
  Cell[1184019, 46517, 498, 9, 61, "Text",
   CellTags->"InsideDiracTrace"],
  Cell[1184520, 46528, 318, 14, 25, "Text",
   CellTags->"InsideDiracTrace"]},
 "IntegralTable"->{
  Cell[1184853, 46545, 63, 1, 41, "ObjectName",
   CellTags->"IntegralTable"],
  Cell[1184941, 46550, 64, 1, 36, "Subsubsection",
   CellTags->"IntegralTable"],
  Cell[1185008, 46553, 233, 5, 25, "Text",
   CellTags->"IntegralTable"],
  Cell[1185244, 46560, 500, 19, 25, "Text",
   CellTags->"IntegralTable"]},
 "IntegrateByParts"->{
  Cell[1185759, 46582, 69, 1, 41, "ObjectName",
   CellTags->"IntegrateByParts"],
  Cell[1185853, 46587, 67, 1, 36, "Subsubsection",
   CellTags->"IntegrateByParts"],
  Cell[1185923, 46590, 184, 4, 24, "Text",
   CellTags->"IntegrateByParts"],
  Cell[1186132, 46598, 131, 3, 26, "Input",
   CellTags->"IntegrateByParts"],
  Cell[1186266, 46603, 214, 6, 26, "Output",
   CellTags->"IntegrateByParts"],
  Cell[1186529, 46615, 64, 1, 34, "Subsubsection",
   CellTags->"IntegrateByParts"],
  Cell[1186618, 46620, 539, 19, 26, "Input",
   CellTags->"IntegrateByParts"],
  Cell[1187160, 46641, 690, 24, 45, "Output",
   CellTags->"IntegrateByParts"]},
 "Integratedx"->{
  Cell[1187877, 46669, 59, 1, 41, "ObjectName",
   CellTags->"Integratedx"],
  Cell[1187961, 46674, 62, 1, 36, "Subsubsection",
   CellTags->"Integratedx"],
  Cell[1188026, 46677, 154, 4, 25, "Text",
   CellTags->"Integratedx"],
  Cell[1188183, 46683, 852, 34, 43, "Text",
   CellTags->"Integratedx"]},
 "Integrate2"->{
  Cell[1189050, 46720, 57, 1, 41, "ObjectName",
   CellTags->"Integrate2"],
  Cell[1189132, 46725, 61, 1, 36, "Subsubsection",
   CellTags->"Integrate2"],
  Cell[1189196, 46728, 866, 17, 97, "Text",
   CellTags->"Integrate2"],
  Cell[1190065, 46747, 508, 24, 25, "Text",
   CellTags->"Integrate2"],
  Cell[1190576, 46773, 405, 7, 61, "Text",
   CellTags->"Integrate2"],
  Cell[1191018, 46785, 58, 1, 34, "Subsubsection",
   CellTags->"Integrate2"],
  Cell[1191101, 46790, 425, 15, 26, "Input",
   CellTags->"Integrate2"],
  Cell[1191529, 46807, 410, 14, 43, "Output",
   CellTags->"Integrate2"],
  Cell[1191954, 46824, 190, 6, 24, "Text",
   CellTags->"Integrate2"],
  Cell[1192169, 46834, 281, 9, 26, "Input",
   CellTags->"Integrate2"],
  Cell[1192453, 46845, 274, 8, 26, "Output",
   CellTags->"Integrate2"],
  Cell[1192764, 46858, 276, 9, 26, "Input",
   CellTags->"Integrate2"],
  Cell[1193043, 46869, 387, 13, 43, "Output",
   CellTags->"Integrate2"],
  Cell[1193467, 46887, 336, 11, 26, "Input",
   CellTags->"Integrate2"],
  Cell[1193806, 46900, 617, 21, 46, "Output",
   CellTags->"Integrate2"],
  Cell[1194460, 46926, 317, 10, 26, "Input",
   CellTags->"Integrate2"],
  Cell[1194780, 46938, 176, 5, 26, "Output",
   CellTags->"Integrate2"],
  Cell[1194971, 46946, 1589, 53, 55, "Text",
   CellTags->"Integrate2"],
  Cell[1196585, 47003, 270, 9, 26, "Input",
   CellTags->"Integrate2"],
  Cell[1196858, 47014, 148, 4, 26, "Output",
   CellTags->"Integrate2"],
  Cell[1197021, 47021, 649, 24, 47, "Text",
   CellTags->"Integrate2"],
  Cell[1197695, 47049, 317, 10, 26, "Input",
   CellTags->"Integrate2"],
  Cell[1198015, 47061, 148, 4, 26, "Output",
   CellTags->"Integrate2"],
  Cell[1198200, 47070, 386, 13, 26, "Input",
   CellTags->"Integrate2"],
  Cell[1198589, 47085, 206, 6, 26, "Output",
   CellTags->"Integrate2"],
  Cell[1198832, 47096, 417, 14, 26, "Input",
   CellTags->"Integrate2"],
  Cell[1199252, 47112, 221, 8, 43, "Output",
   CellTags->"Integrate2"],
  Cell[1199510, 47125, 328, 11, 26, "Input",
   CellTags->"Integrate2"],
  Cell[1199841, 47138, 202, 7, 26, "Output",
   CellTags->"Integrate2"],
  Cell[1200080, 47150, 354, 12, 26, "Input",
   CellTags->"Integrate2"],
  Cell[1200437, 47164, 351, 12, 44, "Output",
   CellTags->"Integrate2"],
  Cell[1200825, 47181, 313, 10, 26, "Input",
   CellTags->"Integrate2"],
  Cell[1201141, 47193, 231, 7, 26, "Output",
   CellTags->"Integrate2"],
  Cell[1201409, 47205, 281, 9, 26, "Input",
   CellTags->"Integrate2"],
  Cell[1201693, 47216, 338, 12, 43, "Output",
   CellTags->"Integrate2"],
  Cell[1202068, 47233, 445, 15, 26, "Input",
   CellTags->"Integrate2"],
  Cell[1202516, 47250, 529, 18, 47, "Output",
   CellTags->"Integrate2"],
  Cell[1203082, 47273, 721, 24, 26, "Input",
   CellTags->"Integrate2"],
  Cell[1203806, 47299, 4395, 148, 185, "Output",
   CellTags->"Integrate2"],
  Cell[1208238, 47452, 471, 16, 26, "Input",
   CellTags->"Integrate2"],
  Cell[1208712, 47470, 861, 30, 46, "Output",
   CellTags->"Integrate2"],
  Cell[1209610, 47505, 567, 19, 49, "Input",
   CellTags->"Integrate2"],
  Cell[1210180, 47526, 5294, 178, 171, "Output",
   CellTags->"Integrate2"],
  Cell[1215511, 47709, 142, 4, 26, "Input",
   CellTags->"Integrate2"],
  Cell[1215656, 47715, 1740, 54, 127, "Output",
   CellTags->"Integrate2"],
  Cell[1217433, 47774, 104, 3, 26, "Input",
   CellTags->"Integrate2"],
  Cell[1217540, 47779, 167, 4, 26, "Output",
   CellTags->"Integrate2"],
  Cell[1218344, 47811, 684, 23, 49, "Input",
   CellTags->"Integrate2"],
  Cell[1219031, 47836, 3720, 140, 128, "Output",
   CellTags->"Integrate2"],
  Cell[1224524, 48052, 151, 4, 26, "Input",
   CellTags->"Integrate2"],
  Cell[1224678, 48058, 151, 4, 26, "Output",
   CellTags->"Integrate2"],
  Cell[1224866, 48067, 368, 12, 26, "Input",
   CellTags->"Integrate2"],
  Cell[1225237, 48081, 148, 4, 26, "Output",
   CellTags->"Integrate2"],
  Cell[1225400, 48088, 137, 4, 24, "Text",
   CellTags->"Integrate2"],
  Cell[1225562, 48096, 147, 4, 26, "Input",
   CellTags->"Integrate2"],
  Cell[1225712, 48102, 5418, 175, 353, "Output",
   CellTags->"Integrate2"],
  Cell[1231167, 48282, 230, 8, 26, "Input",
   CellTags->"Integrate2"],
  Cell[1231400, 48292, 216, 7, 26, "Output",
   CellTags->"Integrate2"],
  Cell[1231631, 48302, 165, 4, 24, "Text",
   CellTags->"Integrate2"],
  Cell[1231821, 48310, 326, 11, 26, "Input",
   CellTags->"Integrate2"],
  Cell[1232150, 48323, 217, 7, 26, "Output",
   CellTags->"Integrate2"],
  Cell[1232382, 48333, 126, 4, 26, "Input",
   CellTags->"Integrate2"],
  Cell[1232533, 48341, 317, 10, 26, "Input",
   CellTags->"Integrate2"],
  Cell[1232853, 48353, 176, 5, 26, "Output",
   CellTags->"Integrate2"],
  Cell[1233066, 48363, 325, 11, 30, "Input",
   CellTags->"Integrate2"],
  Cell[1233394, 48376, 282, 9, 44, "Output",
   CellTags->"Integrate2"],
  Cell[1233713, 48390, 99, 3, 26, "Input",
   CellTags->"Integrate2"],
  Cell[1233815, 48395, 166, 4, 26, "Output",
   CellTags->"Integrate2"],
  Cell[1234018, 48404, 325, 11, 30, "Input",
   CellTags->"Integrate2"],
  Cell[1234346, 48417, 169, 4, 26, "Output",
   CellTags->"Integrate2"],
  Cell[1234552, 48426, 381, 13, 30, "Input",
   CellTags->"Integrate2"],
  Cell[1234936, 48441, 4904, 185, 199, "Output",
   CellTags->"Integrate2"]},
 "Integrate3"->{
  Cell[1241064, 48682, 57, 1, 70, "ObjectName",
   CellTags->"Integrate3"],
  Cell[1241146, 48687, 61, 1, 70, "Subsubsection",
   CellTags->"Integrate3"],
  Cell[1241210, 48690, 331, 6, 70, "Text",
   CellTags->"Integrate3"],
  Cell[1241544, 48698, 187, 9, 70, "Text",
   CellTags->"Integrate3"],
  Cell[1241768, 48712, 58, 1, 34, "Subsubsection",
   CellTags->"Integrate3"],
  Cell[1241851, 48717, 280, 9, 26, "Input",
   CellTags->"Integrate3"],
  Cell[1242134, 48728, 267, 9, 49, "Output",
   CellTags->"Integrate3"],
  Cell[1242438, 48742, 400, 13, 51, "Input",
   CellTags->"Integrate3"],
  Cell[1242841, 48757, 555, 21, 26, "Output",
   CellTags->"Integrate3"],
  Cell[1243433, 48783, 676, 23, 51, "Input",
   CellTags->"Integrate3"],
  Cell[1244112, 48808, 2077, 74, 68, "Output",
   CellTags->"Integrate3"],
  Cell[1246226, 48887, 266, 8, 26, "Input",
   CellTags->"Integrate3"],
  Cell[1246495, 48897, 148, 4, 26, "Output",
   CellTags->"Integrate3"],
  Cell[1246680, 48906, 322, 10, 26, "Input",
   CellTags->"Integrate3"],
  Cell[1247005, 48918, 239, 7, 26, "Output",
   CellTags->"Integrate3"],
  Cell[1247281, 48930, 270, 9, 26, "Input",
   CellTags->"Integrate3"],
  Cell[1247554, 48941, 148, 4, 26, "Output",
   CellTags->"Integrate3"]},
 "IntermediateSubstitutions"->{
  Cell[1247729, 48949, 87, 1, 70, "ObjectName",
   CellTags->"IntermediateSubstitutions"],
  Cell[1247841, 48954, 76, 1, 34, "Subsubsection",
   CellTags->"IntermediateSubstitutions"],
  Cell[1247920, 48957, 206, 4, 24, "Text",
   CellTags->"IntermediateSubstitutions"],
  Cell[1248129, 48963, 193, 9, 24, "Text",
   CellTags->"IntermediateSubstitutions"]},
 "InverseMellin"->{
  Cell[1248337, 48975, 63, 1, 39, "ObjectName",
   CellTags->"InverseMellin"],
  Cell[1248425, 48980, 64, 1, 34, "Subsubsection",
   CellTags->"InverseMellin"],
  Cell[1248492, 48983, 531, 14, 58, "Text",
   CellTags->"InverseMellin"],
  Cell[1249026, 48999, 609, 29, 24, "Text",
   CellTags->"InverseMellin"],
  Cell[1249672, 49033, 61, 1, 34, "Subsubsection",
   CellTags->"InverseMellin"],
  Cell[1249758, 49038, 169, 5, 26, "Input",
   CellTags->"InverseMellin"],
  Cell[1249930, 49045, 201, 6, 26, "Output",
   CellTags->"InverseMellin"],
  Cell[1250168, 49056, 220, 7, 26, "Input",
   CellTags->"InverseMellin"],
  Cell[1250391, 49065, 199, 6, 26, "Output",
   CellTags->"InverseMellin"],
  Cell[1250627, 49076, 142, 4, 26, "Input",
   CellTags->"InverseMellin"],
  Cell[1250772, 49082, 322, 11, 26, "Output",
   CellTags->"InverseMellin"],
  Cell[1251131, 49098, 194, 6, 26, "Input",
   CellTags->"InverseMellin"],
  Cell[1251328, 49106, 337, 12, 44, "Output",
   CellTags->"InverseMellin"],
  Cell[1251702, 49123, 194, 6, 26, "Input",
   CellTags->"InverseMellin"],
  Cell[1251899, 49131, 404, 14, 26, "Output",
   CellTags->"InverseMellin"],
  Cell[1252340, 49150, 203, 7, 26, "Input",
   CellTags->"InverseMellin"],
  Cell[1252546, 49159, 404, 14, 26, "Output",
   CellTags->"InverseMellin"],
  Cell[1252965, 49176, 113, 3, 24, "Text",
   CellTags->"InverseMellin"],
  Cell[1253103, 49183, 294, 9, 26, "Input",
   CellTags->"InverseMellin"],
  Cell[1253400, 49194, 173, 5, 46, "Output",
   CellTags->"InverseMellin"],
  Cell[1253588, 49202, 117, 3, 24, "Text",
   CellTags->"InverseMellin"],
  Cell[1253708, 49207, 3132, 100, 193, "Input",
   CellTags->"InverseMellin"],
  Cell[1256843, 49309, 244, 7, 26, "Input",
   CellTags->"InverseMellin"],
  Cell[1257112, 49320, 179, 6, 26, "Input",
   CellTags->"InverseMellin"],
  Cell[1257294, 49328, 440, 15, 46, "Output",
   CellTags->"InverseMellin"],
  Cell[1257771, 49348, 179, 6, 26, "Input",
   CellTags->"InverseMellin"],
  Cell[1257953, 49356, 398, 13, 46, "Output",
   CellTags->"InverseMellin"],
  Cell[1258388, 49374, 172, 5, 26, "Input",
   CellTags->"InverseMellin"],
  Cell[1258563, 49381, 768, 26, 48, "Output",
   CellTags->"InverseMellin"],
  Cell[1259346, 49410, 81, 2, 26, "Input",
   CellTags->"InverseMellin"],
  Cell[1259452, 49416, 192, 6, 26, "Input",
   CellTags->"InverseMellin"],
  Cell[1259647, 49424, 571, 21, 48, "Output",
   CellTags->"InverseMellin"],
  Cell[1260255, 49450, 270, 9, 26, "Input",
   CellTags->"InverseMellin"],
  Cell[1260528, 49461, 544, 19, 44, "Output",
   CellTags->"InverseMellin"],
  Cell[1261109, 49485, 270, 9, 26, "Input",
   CellTags->"InverseMellin"],
  Cell[1261382, 49496, 737, 26, 44, "Output",
   CellTags->"InverseMellin"],
  Cell[1262156, 49527, 267, 9, 26, "Input",
   CellTags->"InverseMellin"],
  Cell[1262426, 49538, 734, 26, 46, "Output",
   CellTags->"InverseMellin"],
  Cell[1263197, 49569, 246, 9, 26, "Input",
   CellTags->"InverseMellin"],
  Cell[1263446, 49580, 591, 21, 46, "Output",
   CellTags->"InverseMellin"],
  Cell[1264074, 49606, 274, 10, 26, "Input",
   CellTags->"InverseMellin"],
  Cell[1264351, 49618, 956, 32, 47, "Output",
   CellTags->"InverseMellin"],
  Cell[1265344, 49655, 192, 6, 26, "Input",
   CellTags->"InverseMellin"],
  Cell[1265539, 49663, 742, 26, 46, "Output",
   CellTags->"InverseMellin"],
  Cell[1266318, 49694, 221, 7, 26, "Input",
   CellTags->"InverseMellin"],
  Cell[1266542, 49703, 729, 25, 46, "Output",
   CellTags->"InverseMellin"],
  Cell[1267308, 49733, 192, 6, 26, "Input",
   CellTags->"InverseMellin"],
  Cell[1267503, 49741, 837, 30, 49, "Output",
   CellTags->"InverseMellin"],
  Cell[1268377, 49776, 202, 6, 26, "Input",
   CellTags->"InverseMellin"],
  Cell[1268582, 49784, 586, 21, 48, "Output",
   CellTags->"InverseMellin"],
  Cell[1269205, 49810, 223, 7, 26, "Input",
   CellTags->"InverseMellin"],
  Cell[1269431, 49819, 975, 34, 48, "Output",
   CellTags->"InverseMellin"],
  Cell[1270443, 49858, 223, 7, 26, "Input",
   CellTags->"InverseMellin"],
  Cell[1270669, 49867, 963, 34, 48, "Output",
   CellTags->"InverseMellin"],
  Cell[1271669, 49906, 212, 6, 26, "Input",
   CellTags->"InverseMellin"],
  Cell[1271884, 49914, 674, 24, 52, "Output",
   CellTags->"InverseMellin"],
  Cell[1272573, 49941, 158, 5, 26, "Input",
   CellTags->"InverseMellin"]},
 "Isolate"->{
  Cell[1272746, 49949, 51, 1, 39, "ObjectName",
   CellTags->"Isolate"],
  Cell[1272822, 49954, 58, 1, 34, "Subsubsection",
   CellTags->"Isolate"],
  Cell[1272883, 49957, 422, 7, 58, "Text",
   CellTags->"Isolate"],
  Cell[1273330, 49968, 114, 3, 26, "Input",
   CellTags->"Isolate"],
  Cell[1273447, 49973, 446, 13, 47, "Output",
   CellTags->"Isolate"],
  Cell[1273942, 49992, 55, 1, 34, "Subsubsection",
   CellTags->"Isolate"],
  Cell[1274022, 49997, 156, 5, 26, "Input",
   CellTags->"Isolate"],
  Cell[1274181, 50004, 202, 7, 26, "Output",
   CellTags->"Isolate"],
  Cell[1274420, 50016, 378, 13, 26, "Input",
   CellTags->"Isolate"],
  Cell[1274801, 50031, 351, 13, 26, "Output",
   CellTags->"Isolate"],
  Cell[1275189, 50049, 114, 3, 26, "Input",
   CellTags->"Isolate"],
  Cell[1275306, 50054, 328, 12, 38, "Output",
   CellTags->"Isolate"],
  Cell[1275671, 50071, 182, 5, 26, "Input",
   CellTags->"Isolate"],
  Cell[1275856, 50078, 688, 26, 26, "Output",
   CellTags->"Isolate"],
  Cell[1276581, 50109, 561, 19, 26, "Input",
   CellTags->"Isolate"],
  Cell[1277145, 50130, 403, 15, 26, "Output",
   CellTags->"Isolate"],
  Cell[1277585, 50150, 94, 3, 26, "Input",
   CellTags->"Isolate"],
  Cell[1277682, 50155, 131, 2, 40, "Print",
   CellTags->{"Info3500973905-4982097", "Isolate"}],
  Cell[1277816, 50159, 1073, 31, 55, "Print",
   CellTags->{"Info3500973905-4982097", "Isolate"}],
  Cell[1278926, 50195, 278, 7, 26, "Input",
   CellTags->"Isolate"],
  Cell[1279207, 50204, 201, 7, 26, "Output",
   CellTags->"Isolate"],
  Cell[1279445, 50216, 178, 6, 26, "Input",
   CellTags->"Isolate"],
  Cell[1279626, 50224, 253, 8, 26, "Output",
   CellTags->"Isolate"],
  Cell[1279894, 50235, 152, 4, 26, "Input",
   CellTags->"Isolate"]},
 "Info3500973905-4982097"->{
  Cell[1277682, 50155, 131, 2, 40, "Print",
   CellTags->{"Info3500973905-4982097", "Isolate"}],
  Cell[1277816, 50159, 1073, 31, 55, "Print",
   CellTags->{"Info3500973905-4982097", "Isolate"}]},
 "IsolateHead"->{
  Cell[1280061, 50242, 59, 1, 39, "ObjectName",
   CellTags->"IsolateHead"],
  Cell[1280145, 50247, 62, 1, 34, "Subsubsection",
   CellTags->"IsolateHead"],
  Cell[1280210, 50250, 84, 1, 24, "Text",
   CellTags->"IsolateHead"],
  Cell[1280297, 50253, 194, 9, 24, "Text",
   CellTags->"IsolateHead"]},
 "IsolateNames"->{
  Cell[1280506, 50265, 61, 1, 39, "ObjectName",
   CellTags->"IsolateNames"],
  Cell[1280592, 50270, 63, 1, 34, "Subsubsection",
   CellTags->"IsolateNames"],
  Cell[1280658, 50273, 220, 5, 41, "Text",
   CellTags->"IsolateNames"],
  Cell[1280881, 50280, 290, 14, 24, "Text",
   CellTags->"IsolateNames"]},
 "IsolatePrint"->{
  Cell[1281186, 50297, 61, 1, 39, "ObjectName",
   CellTags->"IsolatePrint"],
  Cell[1281272, 50302, 63, 1, 34, "Subsubsection",
   CellTags->"IsolatePrint"],
  Cell[1281338, 50305, 284, 6, 41, "Text",
   CellTags->"IsolatePrint"],
  Cell[1281625, 50313, 180, 9, 24, "Text",
   CellTags->"IsolatePrint"]},
 "IsolateSplit"->{
  Cell[1281820, 50325, 61, 1, 39, "ObjectName",
   CellTags->"IsolateSplit"],
  Cell[1281906, 50330, 63, 1, 34, "Subsubsection",
   CellTags->"IsolateSplit"],
  Cell[1281972, 50333, 413, 7, 58, "Text",
   CellTags->"IsolateSplit"],
  Cell[1282388, 50342, 180, 9, 25, "Text",
   CellTags->"IsolateSplit"]},
 "KeepOnly"->{
  Cell[1282583, 50354, 53, 1, 41, "ObjectName",
   CellTags->"KeepOnly"],
  Cell[1282661, 50359, 59, 1, 36, "Subsubsection",
   CellTags->"KeepOnly"],
  Cell[1282723, 50362, 293, 6, 43, "Text",
   CellTags->"KeepOnly"],
  Cell[1283019, 50370, 452, 24, 25, "Text",
   CellTags->"KeepOnly"]},
 "KK"->{
  Cell[1283486, 50397, 41, 1, 41, "ObjectName",
   CellTags->"KK"],
  Cell[1283552, 50402, 53, 1, 36, "Subsubsection",
   CellTags->"KK"],
  Cell[1283608, 50405, 235, 5, 43, "Text",
   CellTags->"KK"],
  Cell[1283846, 50412, 292, 14, 25, "Text",
   CellTags->"KK"]},
 "Kummer"->{
  Cell[1284153, 50429, 49, 1, 41, "ObjectName",
   CellTags->"Kummer"],
  Cell[1284227, 50434, 57, 1, 34, "Subsubsection",
   CellTags->"Kummer"],
  Cell[1284287, 50437, 260, 5, 41, "Text",
   CellTags->"Kummer"],
  Cell[1284550, 50444, 201, 9, 24, "Text",
   CellTags->"Kummer"],
  Cell[1284788, 50458, 54, 1, 34, "Subsubsection",
   CellTags->"Kummer"],
  Cell[1284867, 50463, 337, 9, 26, "Input",
   CellTags->"Kummer"],
  Cell[1285207, 50474, 494, 15, 28, "Output",
   CellTags->"Kummer"],
  Cell[1285738, 50494, 337, 9, 26, "Input",
   CellTags->"Kummer"],
  Cell[1286078, 50505, 509, 16, 42, "Output",
   CellTags->"Kummer"],
  Cell[1286624, 50526, 337, 9, 26, "Input",
   CellTags->"Kummer"],
  Cell[1286964, 50537, 509, 16, 42, "Output",
   CellTags->"Kummer"],
  Cell[1287510, 50558, 388, 11, 26, "Input",
   CellTags->"Kummer"],
  Cell[1287901, 50571, 494, 15, 28, "Output",
   CellTags->"Kummer"],
  Cell[1288432, 50591, 479, 13, 46, "Input",
   CellTags->"Kummer"],
  Cell[1288914, 50606, 573, 16, 26, "Output",
   CellTags->"Kummer"],
  Cell[1289524, 50627, 388, 11, 26, "Input",
   CellTags->"Kummer"],
  Cell[1289915, 50640, 476, 14, 46, "Output",
   CellTags->"Kummer"],
  Cell[1290428, 50659, 479, 13, 46, "Input",
   CellTags->"Kummer"],
  Cell[1290910, 50674, 589, 18, 46, "Output",
   CellTags->"Kummer"],
  Cell[1291536, 50697, 388, 11, 26, "Input",
   CellTags->"Kummer"],
  Cell[1291927, 50710, 476, 14, 46, "Output",
   CellTags->"Kummer"],
  Cell[1292440, 50729, 479, 13, 46, "Input",
   CellTags->"Kummer"],
  Cell[1292922, 50744, 592, 18, 46, "Output",
   CellTags->"Kummer"],
  Cell[1293551, 50767, 487, 15, 26, "Input",
   CellTags->"Kummer"],
  Cell[1294041, 50784, 663, 21, 46, "Output",
   CellTags->"Kummer"],
  Cell[1294741, 50810, 629, 19, 46, "Input",
   CellTags->"Kummer"],
  Cell[1295373, 50831, 764, 23, 46, "Output",
   CellTags->"Kummer"],
  Cell[1296174, 50859, 487, 15, 26, "Input",
   CellTags->"Kummer"],
  Cell[1296664, 50876, 637, 20, 45, "Output",
   CellTags->"Kummer"],
  Cell[1297338, 50901, 629, 19, 46, "Input",
   CellTags->"Kummer"],
  Cell[1297970, 50922, 749, 24, 45, "Output",
   CellTags->"Kummer"],
  Cell[1298756, 50951, 487, 15, 26, "Input",
   CellTags->"Kummer"],
  Cell[1299246, 50968, 611, 19, 46, "Output",
   CellTags->"Kummer"],
  Cell[1299894, 50992, 629, 19, 46, "Input",
   CellTags->"Kummer"],
  Cell[1300526, 51013, 792, 24, 46, "Output",
   CellTags->"Kummer"],
  Cell[1301355, 51042, 487, 15, 26, "Input",
   CellTags->"Kummer"],
  Cell[1301845, 51059, 663, 21, 46, "Output",
   CellTags->"Kummer"],
  Cell[1302545, 51085, 629, 19, 46, "Input",
   CellTags->"Kummer"],
  Cell[1303177, 51106, 787, 21, 46, "Output",
   CellTags->"Kummer"],
  Cell[1304001, 51132, 487, 15, 26, "Input",
   CellTags->"Kummer"],
  Cell[1304491, 51149, 585, 18, 46, "Output",
   CellTags->"Kummer"],
  Cell[1305113, 51172, 629, 19, 46, "Input",
   CellTags->"Kummer"],
  Cell[1305745, 51193, 833, 23, 46, "Output",
   CellTags->"Kummer"],
  Cell[1306615, 51221, 487, 15, 26, "Input",
   CellTags->"Kummer"],
  Cell[1307105, 51238, 585, 18, 46, "Output",
   CellTags->"Kummer"],
  Cell[1307727, 51261, 629, 19, 46, "Input",
   CellTags->"Kummer"],
  Cell[1308359, 51282, 722, 20, 46, "Output",
   CellTags->"Kummer"],
  Cell[1309118, 51307, 531, 15, 46, "Input",
   CellTags->"Kummer"],
  Cell[1309652, 51324, 620, 19, 28, "Output",
   CellTags->"Kummer"],
  Cell[1310309, 51348, 338, 9, 26, "Input",
   CellTags->"Kummer"],
  Cell[1310650, 51359, 494, 15, 28, "Output",
   CellTags->"Kummer"],
  Cell[1311181, 51379, 531, 15, 46, "Input",
   CellTags->"Kummer"],
  Cell[1311715, 51396, 676, 20, 42, "Output",
   CellTags->"Kummer"],
  Cell[1312428, 51421, 338, 9, 26, "Input",
   CellTags->"Kummer"],
  Cell[1312769, 51432, 483, 15, 42, "Output",
   CellTags->"Kummer"],
  Cell[1313289, 51452, 531, 15, 46, "Input",
   CellTags->"Kummer"],
  Cell[1313823, 51469, 674, 20, 42, "Output",
   CellTags->"Kummer"],
  Cell[1314534, 51494, 338, 9, 26, "Input",
   CellTags->"Kummer"],
  Cell[1314875, 51505, 482, 15, 42, "Output",
   CellTags->"Kummer"],
  Cell[1315394, 51525, 582, 17, 46, "Input",
   CellTags->"Kummer"],
  Cell[1315979, 51544, 631, 16, 26, "Output",
   CellTags->"Kummer"],
  Cell[1316647, 51565, 389, 11, 26, "Input",
   CellTags->"Kummer"],
  Cell[1317039, 51578, 494, 15, 28, "Output",
   CellTags->"Kummer"],
  Cell[1317570, 51598, 582, 17, 46, "Input",
   CellTags->"Kummer"],
  Cell[1318155, 51617, 731, 19, 46, "Output",
   CellTags->"Kummer"],
  Cell[1318923, 51641, 389, 11, 26, "Input",
   CellTags->"Kummer"],
  Cell[1319315, 51654, 533, 16, 46, "Output",
   CellTags->"Kummer"],
  Cell[1319885, 51675, 582, 17, 46, "Input",
   CellTags->"Kummer"],
  Cell[1320470, 51694, 674, 17, 46, "Output",
   CellTags->"Kummer"],
  Cell[1321181, 51716, 389, 11, 26, "Input",
   CellTags->"Kummer"],
  Cell[1321573, 51729, 478, 14, 46, "Output",
   CellTags->"Kummer"],
  Cell[1322088, 51748, 338, 9, 26, "Input",
   CellTags->"Kummer"],
  Cell[1322429, 51759, 1352, 44, 95, "Output",
   CellTags->"Kummer"],
  Cell[1323818, 51808, 338, 9, 26, "Input",
   CellTags->"Kummer"],
  Cell[1324159, 51819, 1412, 47, 77, "Output",
   CellTags->"Kummer"],
  Cell[1325608, 51871, 338, 9, 26, "Input",
   CellTags->"Kummer"],
  Cell[1325949, 51882, 1440, 47, 107, "Output",
   CellTags->"Kummer"]},
 "Lagrangian"->{
  Cell[1327416, 51933, 57, 1, 70, "ObjectName",
   CellTags->"Lagrangian"],
  Cell[1327498, 51938, 61, 1, 36, "Subsubsection",
   CellTags->"Lagrangian"],
  Cell[1327562, 51941, 454, 9, 70, "Text",
   CellTags->"Lagrangian"],
  Cell[1328019, 51952, 181, 9, 25, "Text",
   CellTags->"Lagrangian"],
  Cell[1328237, 51966, 58, 1, 34, "Subsubsection",
   CellTags->"Lagrangian"],
  Cell[1328320, 51971, 124, 3, 26, "Input",
   CellTags->"Lagrangian"],
  Cell[1328447, 51976, 376, 12, 44, "Output",
   CellTags->"Lagrangian"],
  Cell[1328838, 51991, 85, 1, 24, "Text",
   CellTags->"Lagrangian"],
  Cell[1328948, 51996, 124, 3, 26, "Input",
   CellTags->"Lagrangian"],
  Cell[1329075, 52001, 725, 24, 44, "Output",
   CellTags->"Lagrangian"],
  Cell[1329837, 52030, 124, 3, 26, "Input",
   CellTags->"Lagrangian"],
  Cell[1329964, 52035, 1048, 37, 44, "Output",
   CellTags->"Lagrangian"],
  Cell[1331049, 52077, 124, 3, 26, "Input",
   CellTags->"Lagrangian"],
  Cell[1331176, 52082, 663, 23, 26, "Output",
   CellTags->"Lagrangian"],
  Cell[1331876, 52110, 124, 3, 26, "Input",
   CellTags->"Lagrangian"],
  Cell[1332003, 52115, 706, 24, 26, "Output",
   CellTags->"Lagrangian"]},
 "LC"->{
  Cell[1332736, 52143, 41, 1, 41, "ObjectName",
   CellTags->"LC"],
  Cell[1332802, 52148, 53, 1, 36, "Subsubsection",
   CellTags->"LC"],
  Cell[1332858, 52151, 240, 5, 43, "Text",
   CellTags->"LC"],
  Cell[1333101, 52158, 274, 14, 25, "Text",
   CellTags->"LC"],
  Cell[1333412, 52177, 50, 1, 34, "Subsubsection",
   CellTags->"LC"],
  Cell[1333487, 52182, 164, 5, 26, "Input",
   CellTags->"LC"],
  Cell[1333654, 52189, 226, 7, 26, "Output",
   CellTags->"LC"],
  Cell[1333917, 52201, 95, 3, 26, "Input",
   CellTags->"LC"],
  Cell[1334015, 52206, 519, 22, 26, "Output",
   CellTags->"LC"],
  Cell[1334571, 52233, 104, 3, 26, "Input",
   CellTags->"LC"],
  Cell[1334678, 52238, 379, 9, 38, "Output",
   CellTags->"LC"],
  Cell[1335094, 52252, 181, 6, 26, "Input",
   CellTags->"LC"],
  Cell[1335278, 52260, 211, 6, 26, "Output",
   CellTags->"LC"],
  Cell[1335526, 52271, 130, 4, 26, "Input",
   CellTags->"LC"],
  Cell[1335659, 52277, 409, 11, 38, "Output",
   CellTags->"LC"],
  Cell[1336105, 52293, 381, 13, 26, "Input",
   CellTags->"LC"],
  Cell[1336489, 52308, 438, 17, 28, "Output",
   CellTags->"LC"]},
 "LCD"->{
  Cell[1336954, 52329, 43, 1, 39, "ObjectName",
   CellTags->"LCD"],
  Cell[1337022, 52334, 54, 1, 34, "Subsubsection",
   CellTags->"LCD"],
  Cell[1337079, 52337, 234, 5, 41, "Text",
   CellTags->"LCD"],
  Cell[1337316, 52344, 161, 4, 24, "Text",
   CellTags->"LCD"],
  Cell[1337480, 52350, 272, 14, 24, "Text",
   CellTags->"LCD"],
  Cell[1337789, 52369, 51, 1, 34, "Subsubsection",
   CellTags->"LCD"],
  Cell[1337843, 52372, 190, 6, 26, "Input",
   CellTags->"LCD"],
  Cell[1338058, 52382, 166, 5, 26, "Input",
   CellTags->"LCD"],
  Cell[1338227, 52389, 226, 7, 26, "Output",
   CellTags->"LCD"],
  Cell[1338490, 52401, 96, 3, 26, "Input",
   CellTags->"LCD"],
  Cell[1338589, 52406, 520, 22, 26, "Output",
   CellTags->"LCD"],
  Cell[1339146, 52433, 105, 3, 26, "Input",
   CellTags->"LCD"],
  Cell[1339254, 52438, 480, 13, 38, "Output",
   CellTags->"LCD"],
  Cell[1339771, 52456, 183, 6, 26, "Input",
   CellTags->"LCD"],
  Cell[1339957, 52464, 212, 6, 26, "Output",
   CellTags->"LCD"],
  Cell[1340206, 52475, 131, 4, 26, "Input",
   CellTags->"LCD"],
  Cell[1340340, 52481, 462, 13, 38, "Output",
   CellTags->"LCD"],
  Cell[1340839, 52499, 400, 13, 26, "Input",
   CellTags->"LCD"],
  Cell[1341242, 52514, 439, 16, 28, "Output",
   CellTags->"LCD"],
  Cell[1341718, 52535, 170, 5, 26, "Input",
   CellTags->"LCD"],
  Cell[1341891, 52542, 203, 6, 26, "Output",
   CellTags->"LCD"]},
 "LeftPartialD"->{
  Cell[1342121, 52552, 61, 1, 41, "ObjectName",
   CellTags->"LeftPartialD"],
  Cell[1342207, 52557, 63, 1, 36, "Subsubsection",
   CellTags->"LeftPartialD"],
  Cell[1342273, 52560, 307, 12, 31, "Text",
   CellTags->"LeftPartialD"],
  Cell[1342583, 52574, 573, 24, 25, "Text",
   CellTags->"LeftPartialD"],
  Cell[1343193, 52603, 69, 1, 33, "Subsubsection",
   CellTags->"LeftPartialD"],
  Cell[1343287, 52608, 253, 7, 26, "Input",
   CellTags->"LeftPartialD"],
  Cell[1343543, 52617, 348, 12, 33, "Output",
   CellTags->"LeftPartialD"],
  Cell[1343928, 52634, 120, 3, 26, "Input",
   CellTags->"LeftPartialD"],
  Cell[1344051, 52639, 622, 15, 87, "Output",
   CellTags->"LeftPartialD"],
  Cell[1344710, 52659, 118, 3, 26, "Input",
   CellTags->"LeftPartialD"],
  Cell[1344831, 52664, 611, 12, 115, "Output",
   CellTags->"LeftPartialD"],
  Cell[1345479, 52681, 161, 4, 26, "Input",
   CellTags->"LeftPartialD"],
  Cell[1345643, 52687, 218, 5, 38, "Output",
   CellTags->"LeftPartialD"],
  Cell[1345898, 52697, 372, 10, 26, "Input",
   CellTags->"LeftPartialD"],
  Cell[1346273, 52709, 385, 13, 33, "Output",
   CellTags->"LeftPartialD"],
  Cell[1346695, 52727, 120, 3, 26, "Input",
   CellTags->"LeftPartialD"],
  Cell[1346818, 52732, 1254, 31, 150, "Output",
   CellTags->"LeftPartialD"],
  Cell[1348109, 52768, 118, 3, 26, "Input",
   CellTags->"LeftPartialD"],
  Cell[1348230, 52773, 1403, 30, 210, "Output",
   CellTags->"LeftPartialD"]},
 "LeftRightPartialD"->{
  Cell[1349660, 52807, 71, 1, 41, "ObjectName",
   CellTags->"LeftRightPartialD"],
  Cell[1349756, 52812, 68, 1, 36, "Subsubsection",
   CellTags->"LeftRightPartialD"],
  Cell[1349827, 52815, 592, 21, 49, "Text",
   CellTags->"LeftRightPartialD"],
  Cell[1350422, 52838, 831, 33, 25, "Text",
   CellTags->"LeftRightPartialD"],
  Cell[1351290, 52876, 65, 1, 34, "Subsubsection",
   CellTags->"LeftRightPartialD"],
  Cell[1351380, 52881, 132, 3, 26, "Input",
   CellTags->"LeftRightPartialD"],
  Cell[1351515, 52886, 302, 10, 33, "Output",
   CellTags->"LeftRightPartialD"],
  Cell[1351854, 52901, 127, 3, 26, "Input",
   CellTags->"LeftRightPartialD"],
  Cell[1351984, 52906, 571, 20, 44, "Output",
   CellTags->"LeftRightPartialD"],
  Cell[1352592, 52931, 263, 7, 26, "Input",
   CellTags->"LeftRightPartialD"],
  Cell[1352858, 52940, 358, 12, 33, "Output",
   CellTags->"LeftRightPartialD"],
  Cell[1353253, 52957, 125, 3, 26, "Input",
   CellTags->"LeftRightPartialD"],
  Cell[1353381, 52962, 968, 27, 117, "Output",
   CellTags->"LeftRightPartialD"],
  Cell[1354386, 52994, 382, 10, 26, "Input",
   CellTags->"LeftRightPartialD"],
  Cell[1354771, 53006, 396, 13, 33, "Output",
   CellTags->"LeftRightPartialD"],
  Cell[1355204, 53024, 125, 3, 26, "Input",
   CellTags->"LeftRightPartialD"],
  Cell[1355332, 53029, 1370, 34, 171, "Output",
   CellTags->"LeftRightPartialD"]},
 "LeftRightPartialD2"->{
  Cell[1356729, 53067, 73, 1, 70, "ObjectName",
   CellTags->"LeftRightPartialD2"],
  Cell[1356827, 53072, 69, 1, 70, "Subsubsection",
   CellTags->"LeftRightPartialD2"],
  Cell[1356899, 53075, 591, 21, 70, "Text",
   CellTags->"LeftRightPartialD2"],
  Cell[1357493, 53098, 698, 29, 70, "Text",
   CellTags->"LeftRightPartialD2"],
  Cell[1358228, 53132, 66, 1, 34, "Subsubsection",
   CellTags->"LeftRightPartialD2"],
  Cell[1358319, 53137, 134, 3, 26, "Input",
   CellTags->"LeftRightPartialD2"],
  Cell[1358456, 53142, 303, 10, 33, "Output",
   CellTags->"LeftRightPartialD2"],
  Cell[1358796, 53157, 128, 3, 26, "Input",
   CellTags->"LeftRightPartialD2"],
  Cell[1358927, 53162, 475, 17, 33, "Output",
   CellTags->"LeftRightPartialD2"],
  Cell[1359439, 53184, 265, 7, 26, "Input",
   CellTags->"LeftRightPartialD2"],
  Cell[1359707, 53193, 361, 12, 33, "Output",
   CellTags->"LeftRightPartialD2"],
  Cell[1360105, 53210, 126, 3, 26, "Input",
   CellTags->"LeftRightPartialD2"],
  Cell[1360234, 53215, 866, 24, 93, "Output",
   CellTags->"LeftRightPartialD2"],
  Cell[1361137, 53244, 384, 10, 26, "Input",
   CellTags->"LeftRightPartialD2"],
  Cell[1361524, 53256, 397, 13, 33, "Output",
   CellTags->"LeftRightPartialD2"],
  Cell[1361958, 53274, 126, 3, 26, "Input",
   CellTags->"LeftRightPartialD2"],
  Cell[1362087, 53279, 1259, 31, 153, "Output",
   CellTags->"LeftRightPartialD2"]},
 "LeviCivita"->{
  Cell[1363373, 53314, 58, 1, 39, "ObjectName",
   CellTags->"LeviCivita"],
  Cell[1363456, 53319, 61, 1, 34, "Subsubsection",
   CellTags->"LeviCivita"],
  Cell[1363520, 53322, 1210, 40, 80, "Text",
   CellTags->"LeviCivita"],
  Cell[1364755, 53366, 120, 3, 26, "Input",
   CellTags->"LeviCivita"],
  Cell[1364878, 53371, 210, 6, 26, "Output",
   CellTags->"LeviCivita"],
  Cell[1365103, 53380, 463, 24, 24, "Text",
   CellTags->"LeviCivita"],
  Cell[1365603, 53409, 58, 1, 34, "Subsubsection",
   CellTags->"LeviCivita"],
  Cell[1365686, 53414, 187, 5, 26, "Input",
   CellTags->"LeviCivita"],
  Cell[1365876, 53421, 534, 22, 26, "Output",
   CellTags->"LeviCivita"],
  Cell[1366447, 53448, 204, 6, 26, "Input",
   CellTags->"LeviCivita"],
  Cell[1366654, 53456, 453, 18, 26, "Output",
   CellTags->"LeviCivita"],
  Cell[1367144, 53479, 180, 5, 26, "Input",
   CellTags->"LeviCivita"],
  Cell[1367327, 53486, 355, 14, 26, "Output",
   CellTags->"LeviCivita"],
  Cell[1367719, 53505, 186, 5, 26, "Input",
   CellTags->"LeviCivita"],
  Cell[1367908, 53512, 149, 4, 26, "Output",
   CellTags->"LeviCivita"],
  Cell[1368094, 53521, 242, 7, 26, "Input",
   CellTags->"LeviCivita"],
  Cell[1368339, 53530, 374, 9, 38, "Output",
   CellTags->"LeviCivita"],
  Cell[1368750, 53544, 365, 10, 26, "Input",
   CellTags->"LeviCivita"],
  Cell[1369118, 53556, 973, 41, 26, "Output",
   CellTags->"LeviCivita"],
  Cell[1370128, 53602, 113, 3, 26, "Input",
   CellTags->"LeviCivita"],
  Cell[1370244, 53607, 167, 5, 26, "Output",
   CellTags->"LeviCivita"],
  Cell[1370426, 53615, 316, 9, 26, "Input",
   CellTags->"LeviCivita"],
  Cell[1370767, 53628, 403, 11, 26, "Input",
   CellTags->"LeviCivita"],
  Cell[1371173, 53641, 339, 11, 26, "Output",
   CellTags->"LeviCivita"],
  Cell[1371527, 53655, 316, 9, 26, "Input",
   CellTags->"LeviCivita"],
  Cell[1371868, 53668, 179, 5, 26, "Input",
   CellTags->"LeviCivita"],
  Cell[1372050, 53675, 241, 7, 26, "Output",
   CellTags->"LeviCivita"],
  Cell[1372306, 53685, 110, 3, 26, "Input",
   CellTags->"LeviCivita"]},
 "LeviCivitaSign"->{
  Cell[1372431, 53691, 65, 1, 41, "ObjectName",
   CellTags->"LeviCivitaSign"],
  Cell[1372521, 53696, 65, 1, 36, "Subsubsection",
   CellTags->"LeviCivitaSign"],
  Cell[1372589, 53699, 203, 4, 43, "Text",
   CellTags->"LeviCivitaSign"],
  Cell[1372795, 53705, 310, 14, 25, "Text",
   CellTags->"LeviCivitaSign"]},
 "ListIntegrals"->{
  Cell[1373120, 53722, 81, 1, 41, "ObjectName",
   CellTags->"ListIntegrals"],
  Cell[1373226, 53727, 64, 1, 36, "Subsubsection",
   CellTags->"ListIntegrals"],
  Cell[1373293, 53730, 470, 8, 79, "Text",
   CellTags->"ListIntegrals"],
  Cell[1373800, 53743, 61, 1, 34, "Subsubsection",
   CellTags->"ListIntegrals"],
  Cell[1373864, 53746, 80, 2, 26, "Input",
   CellTags->"ListIntegrals"]},
 "Li2"->{
  Cell[1373959, 53751, 44, 1, 41, "ObjectName",
   CellTags->"Li2"],
  Cell[1374028, 53756, 54, 1, 36, "Subsubsection",
   CellTags->"Li2"],
  Cell[1374085, 53759, 117, 3, 25, "Text",
   CellTags->"Li2"],
  Cell[1374205, 53764, 153, 8, 25, "Text",
   CellTags->"Li2"],
  Cell[1374395, 53777, 51, 1, 34, "Subsubsection",
   CellTags->"Li2"],
  Cell[1374471, 53782, 100, 3, 26, "Input",
   CellTags->"Li2"],
  Cell[1374574, 53787, 178, 6, 26, "Output",
   CellTags->"Li2"],
  Cell[1374789, 53798, 107, 3, 26, "Input",
   CellTags->"Li2"],
  Cell[1374899, 53803, 204, 6, 38, "Output",
   CellTags->"Li2"],
  Cell[1375140, 53814, 253, 9, 49, "Input",
   CellTags->"Li2"],
  Cell[1375396, 53825, 180, 6, 26, "Output",
   CellTags->"Li2"]},
 "Li3"->{
  Cell[1375603, 53835, 44, 1, 41, "ObjectName",
   CellTags->"Li3"],
  Cell[1375672, 53840, 54, 1, 36, "Subsubsection",
   CellTags->"Li3"],
  Cell[1375729, 53843, 118, 3, 25, "Text",
   CellTags->"Li3"],
  Cell[1375850, 53848, 153, 8, 25, "Text",
   CellTags->"Li3"],
  Cell[1376040, 53861, 51, 1, 34, "Subsubsection",
   CellTags->"Li3"],
  Cell[1376116, 53866, 100, 3, 26, "Input",
   CellTags->"Li3"],
  Cell[1376219, 53871, 178, 6, 26, "Output",
   CellTags->"Li3"],
  Cell[1376434, 53882, 107, 3, 26, "Input",
   CellTags->"Li3"],
  Cell[1376544, 53887, 204, 6, 38, "Output",
   CellTags->"Li3"],
  Cell[1376785, 53898, 152, 5, 26, "Input",
   CellTags->"Li3"],
  Cell[1376940, 53905, 203, 7, 45, "Output",
   CellTags->"Li3"],
  Cell[1377180, 53917, 204, 7, 50, "Input",
   CellTags->"Li3"],
  Cell[1377387, 53926, 180, 6, 26, "Output",
   CellTags->"Li3"]},
 "Loop"->{
  Cell[1377594, 53936, 45, 1, 41, "ObjectName",
   CellTags->"Loop"],
  Cell[1377664, 53941, 55, 1, 36, "Subsubsection",
   CellTags->"Loop"],
  Cell[1377722, 53944, 94, 1, 25, "Text",
   CellTags->"Loop"]},
 "LorentzIndex"->{
  Cell[1377831, 53948, 61, 1, 41, "ObjectName",
   CellTags->"LorentzIndex"],
  Cell[1377917, 53953, 63, 1, 36, "Subsubsection",
   CellTags->"LorentzIndex"],
  Cell[1377983, 53956, 243, 5, 43, "Text",
   CellTags->"LorentzIndex"],
  Cell[1378229, 53963, 314, 14, 25, "Text",
   CellTags->"LorentzIndex"],
  Cell[1378580, 53982, 60, 1, 36, "Subsubsection",
   CellTags->"LorentzIndex"],
  Cell[1378643, 53985, 89, 1, 24, "Text",
   CellTags->"LorentzIndex"],
  Cell[1378757, 53990, 125, 3, 26, "Input",
   CellTags->"LorentzIndex"],
  Cell[1378885, 53995, 158, 4, 26, "Output",
   CellTags->"LorentzIndex"],
  Cell[1379058, 54002, 125, 3, 24, "Text",
   CellTags->"LorentzIndex"],
  Cell[1379208, 54009, 148, 4, 26, "Input",
   CellTags->"LorentzIndex"],
  Cell[1379359, 54015, 158, 4, 26, "Output",
   CellTags->"LorentzIndex"],
  Cell[1379554, 54024, 106, 2, 26, "Input",
   CellTags->"LorentzIndex"],
  Cell[1379663, 54028, 237, 8, 26, "Output",
   CellTags->"LorentzIndex"],
  Cell[1379937, 54041, 165, 4, 26, "Input",
   CellTags->"LorentzIndex"],
  Cell[1380105, 54047, 154, 4, 26, "Output",
   CellTags->"LorentzIndex"],
  Cell[1380296, 54056, 207, 5, 26, "Input",
   CellTags->"LorentzIndex"],
  Cell[1380506, 54063, 181, 5, 26, "Output",
   CellTags->"LorentzIndex"],
  Cell[1380702, 54071, 141, 4, 24, "Text",
   CellTags->"LorentzIndex"],
  Cell[1380846, 54077, 137, 4, 26, "Input",
   CellTags->"LorentzIndex"],
  Cell[1381008, 54085, 347, 10, 26, "Input",
   CellTags->"LorentzIndex"],
  Cell[1381358, 54097, 429, 17, 26, "Output",
   CellTags->"LorentzIndex"],
  Cell[1381802, 54117, 148, 4, 26, "Input",
   CellTags->"LorentzIndex"]},
 "Lower"->{
  Cell[1381965, 54124, 47, 1, 41, "ObjectName",
   CellTags->"Lower"],
  Cell[1382037, 54129, 56, 1, 36, "Subsubsection",
   CellTags->"Lower"],
  Cell[1382096, 54132, 120, 3, 25, "Text",
   CellTags->"Lower"],
  Cell[1382219, 54137, 402, 19, 25, "Text",
   CellTags->"Lower"]},
 "MakeContext"->{
  Cell[1382636, 54159, 59, 1, 41, "ObjectName",
   CellTags->"MakeContext"],
  Cell[1382720, 54164, 62, 1, 36, "Subsubsection",
   CellTags->"MakeContext"],
  Cell[1382785, 54167, 238, 5, 43, "Text",
   CellTags->"MakeContext"],
  Cell[1383026, 54174, 530, 24, 25, "Text",
   CellTags->"MakeContext"]},
 "Mandelstam"->{
  Cell[1383571, 54201, 57, 1, 39, "ObjectName",
   CellTags->"Mandelstam"],
  Cell[1383653, 54206, 61, 1, 34, "Subsubsection",
   CellTags->"Mandelstam"],
  Cell[1383717, 54209, 1575, 73, 58, "Text",
   CellTags->"Mandelstam"],
  Cell[1385295, 54284, 758, 34, 24, "Text",
   CellTags->"Mandelstam"]},
 "Map2"->{
  Cell[1386068, 54321, 45, 1, 39, "ObjectName",
   CellTags->"Map2"],
  Cell[1386138, 54326, 55, 1, 34, "Subsubsection",
   CellTags->"Map2"],
  Cell[1386196, 54329, 135, 4, 24, "Text",
   CellTags->"Map2"],
  Cell[1386334, 54335, 169, 9, 24, "Text",
   CellTags->"Map2"],
  Cell[1386540, 54349, 52, 1, 34, "Subsubsection",
   CellTags->"Map2"],
  Cell[1386617, 54354, 149, 5, 26, "Input",
   CellTags->"Map2"],
  Cell[1386769, 54361, 244, 8, 26, "Output",
   CellTags->"Map2"],
  Cell[1387050, 54374, 125, 4, 26, "Input",
   CellTags->"Map2"],
  Cell[1387178, 54380, 171, 5, 26, "Output",
   CellTags->"Map2"],
  Cell[1387386, 54390, 184, 6, 26, "Input",
   CellTags->"Map2"],
  Cell[1387573, 54398, 235, 8, 26, "Output",
   CellTags->"Map2"],
  Cell[1387845, 54411, 125, 4, 26, "Input",
   CellTags->"Map2"],
  Cell[1387973, 54417, 171, 5, 26, "Output",
   CellTags->"Map2"]},
 "MemoryAvailable"->{
  Cell[1388171, 54426, 67, 1, 39, "ObjectName",
   CellTags->"MemoryAvailable"],
  Cell[1388263, 54431, 66, 1, 34, "Subsubsection",
   CellTags->"MemoryAvailable"],
  Cell[1388332, 54434, 230, 5, 41, "Text",
   CellTags->"MemoryAvailable"],
  Cell[1388565, 54441, 314, 14, 24, "Text",
   CellTags->"MemoryAvailable"]},
 "MemSet"->{
  Cell[1388894, 54458, 49, 1, 39, "ObjectName",
   CellTags->"MemSet"],
  Cell[1388968, 54463, 57, 1, 34, "Subsubsection",
   CellTags->"MemSet"],
  Cell[1389028, 54466, 345, 9, 41, "Text",
   CellTags->"MemSet"],
  Cell[1389376, 54477, 198, 9, 24, "Text",
   CellTags->"MemSet"]},
 "MetricTensor"->{
  Cell[1389589, 54489, 61, 1, 39, "ObjectName",
   CellTags->"MetricTensor"],
  Cell[1389675, 54494, 63, 1, 36, "Subsubsection",
   CellTags->"MetricTensor"],
  Cell[1389741, 54497, 214, 7, 25, "Text",
   CellTags->"MetricTensor"],
  Cell[1389958, 54506, 584, 29, 25, "Text",
   CellTags->"MetricTensor"],
  Cell[1390579, 54540, 60, 1, 34, "Subsubsection",
   CellTags->"MetricTensor"],
  Cell[1390664, 54545, 154, 4, 26, "Input",
   CellTags->"MetricTensor"],
  Cell[1390821, 54551, 207, 6, 26, "Output",
   CellTags->"MetricTensor"],
  Cell[1391065, 54562, 137, 4, 26, "Input",
   CellTags->"MetricTensor"],
  Cell[1391205, 54568, 151, 4, 26, "Output",
   CellTags->"MetricTensor"],
  Cell[1391393, 54577, 202, 5, 26, "Input",
   CellTags->"MetricTensor"],
  Cell[1391598, 54584, 207, 6, 26, "Output",
   CellTags->"MetricTensor"],
  Cell[1391842, 54595, 137, 4, 26, "Input",
   CellTags->"MetricTensor"],
  Cell[1391982, 54601, 153, 4, 27, "Output",
   CellTags->"MetricTensor"],
  Cell[1392172, 54610, 181, 5, 26, "Input",
   CellTags->"MetricTensor"],
  Cell[1392356, 54617, 198, 5, 38, "Output",
   CellTags->"MetricTensor"],
  Cell[1392591, 54627, 230, 6, 26, "Input",
   CellTags->"MetricTensor"],
  Cell[1392824, 54635, 246, 6, 38, "Output",
   CellTags->"MetricTensor"],
  Cell[1393107, 54646, 213, 6, 26, "Input",
   CellTags->"MetricTensor"],
  Cell[1393323, 54654, 251, 6, 38, "Output",
   CellTags->"MetricTensor"],
  Cell[1393611, 54665, 263, 7, 26, "Input",
   CellTags->"MetricTensor"],
  Cell[1393877, 54674, 297, 7, 38, "Output",
   CellTags->"MetricTensor"]},
 "MLimit"->{
  Cell[1394201, 54685, 49, 1, 41, "ObjectName",
   CellTags->"MLimit"],
  Cell[1394275, 54690, 57, 1, 36, "Subsubsection",
   CellTags->"MLimit"],
  Cell[1394335, 54693, 116, 3, 25, "Text",
   CellTags->"MLimit"],
  Cell[1394488, 54701, 54, 1, 34, "Subsubsection",
   CellTags->"MLimit"],
  Cell[1394567, 54706, 419, 14, 26, "Input",
   CellTags->"MLimit"],
  Cell[1394989, 54722, 472, 15, 46, "Output",
   CellTags->"MLimit"]},
 "Momentum"->{
  Cell[1395488, 54741, 53, 1, 41, "ObjectName",
   CellTags->"Momentum"],
  Cell[1395566, 54746, 59, 1, 36, "Subsubsection",
   CellTags->"Momentum"],
  Cell[1395628, 54749, 258, 5, 43, "Text",
   CellTags->"Momentum"],
  Cell[1395889, 54756, 530, 24, 25, "Text",
   CellTags->"Momentum"],
  Cell[1396456, 54785, 56, 1, 36, "Subsubsection",
   CellTags->"Momentum"],
  Cell[1396515, 54788, 75, 1, 24, "Text",
   CellTags->"Momentum"],
  Cell[1396615, 54793, 110, 3, 26, "Input",
   CellTags->"Momentum"],
  Cell[1396728, 54798, 147, 4, 29, "Output",
   CellTags->"Momentum"],
  Cell[1396890, 54805, 138, 4, 24, "Text",
   CellTags->"Momentum"],
  Cell[1397053, 54813, 133, 4, 26, "Input",
   CellTags->"Momentum"],
  Cell[1397189, 54819, 147, 4, 29, "Output",
   CellTags->"Momentum"],
  Cell[1397351, 54826, 86, 1, 24, "Text",
   CellTags->"Momentum"],
  Cell[1397462, 54831, 169, 5, 26, "Input",
   CellTags->"Momentum"],
  Cell[1397634, 54838, 190, 5, 38, "Output",
   CellTags->"Momentum"],
  Cell[1397861, 54848, 153, 5, 26, "Input",
   CellTags->"Momentum"],
  Cell[1398017, 54855, 165, 5, 28, "Output",
   CellTags->"Momentum"],
  Cell[1398219, 54865, 111, 3, 26, "Input",
   CellTags->"Momentum"],
  Cell[1398333, 54870, 185, 5, 38, "Output",
   CellTags->"Momentum"],
  Cell[1398555, 54880, 259, 8, 26, "Input",
   CellTags->"Momentum"],
  Cell[1398817, 54890, 242, 8, 28, "Output",
   CellTags->"Momentum"],
  Cell[1399096, 54903, 111, 3, 26, "Input",
   CellTags->"Momentum"],
  Cell[1399210, 54908, 273, 8, 38, "Output",
   CellTags->"Momentum"],
  Cell[1399520, 54921, 148, 4, 26, "Input",
   CellTags->"Momentum"],
  Cell[1399671, 54927, 225, 6, 38, "Output",
   CellTags->"Momentum"],
  Cell[1399933, 54938, 149, 4, 26, "Input",
   CellTags->"Momentum"],
  Cell[1400085, 54944, 273, 8, 38, "Output",
   CellTags->"Momentum"],
  Cell[1400395, 54957, 216, 7, 26, "Input",
   CellTags->"Momentum"],
  Cell[1400614, 54966, 190, 5, 38, "Output",
   CellTags->"Momentum"],
  Cell[1400819, 54974, 132, 4, 26, "Input",
   CellTags->"Momentum"]},
 "MomentumCombine"->{
  Cell[1400966, 54981, 67, 1, 39, "ObjectName",
   CellTags->"MomentumCombine"],
  Cell[1401058, 54986, 66, 1, 34, "Subsubsection",
   CellTags->"MomentumCombine"],
  Cell[1401127, 54989, 183, 4, 41, "Text",
   CellTags->"MomentumCombine"],
  Cell[1401313, 54995, 585, 23, 24, "Text",
   CellTags->"MomentumCombine"],
  Cell[1401935, 55023, 63, 1, 34, "Subsubsection",
   CellTags->"MomentumCombine"],
  Cell[1402023, 55028, 303, 9, 26, "Input",
   CellTags->"MomentumCombine"],
  Cell[1402329, 55039, 258, 7, 38, "Output",
   CellTags->"MomentumCombine"],
  Cell[1402624, 55051, 288, 9, 26, "Input",
   CellTags->"MomentumCombine"],
  Cell[1402915, 55062, 260, 8, 28, "Output",
   CellTags->"MomentumCombine"],
  Cell[1403212, 55075, 125, 3, 26, "Input",
   CellTags->"MomentumCombine"],
  Cell[1403340, 55080, 260, 8, 28, "Output",
   CellTags->"MomentumCombine"],
  Cell[1403637, 55093, 163, 5, 26, "Input",
   CellTags->"MomentumCombine"],
  Cell[1403803, 55100, 303, 9, 38, "Output",
   CellTags->"MomentumCombine"],
  Cell[1404143, 55114, 170, 5, 26, "Input",
   CellTags->"MomentumCombine"],
  Cell[1404316, 55121, 260, 8, 28, "Output",
   CellTags->"MomentumCombine"],
  Cell[1404613, 55134, 121, 3, 26, "Input",
   CellTags->"MomentumCombine"],
  Cell[1404737, 55139, 301, 9, 38, "Output",
   CellTags->"MomentumCombine"],
  Cell[1405053, 55151, 115, 3, 26, "Input",
   CellTags->"MomentumCombine"]},
 "MomentumCombine2"->{
  Cell[1405183, 55157, 69, 1, 39, "ObjectName",
   CellTags->"MomentumCombine2"],
  Cell[1405277, 55162, 67, 1, 34, "Subsubsection",
   CellTags->"MomentumCombine2"],
  Cell[1405347, 55165, 192, 4, 41, "Text",
   CellTags->"MomentumCombine2"],
  Cell[1405542, 55171, 589, 23, 24, "Text",
   CellTags->"MomentumCombine2"],
  Cell[1406168, 55199, 64, 1, 34, "Subsubsection",
   CellTags->"MomentumCombine2"],
  Cell[1406257, 55204, 434, 13, 26, "Input",
   CellTags->"MomentumCombine2"],
  Cell[1406694, 55219, 517, 21, 28, "Output",
   CellTags->"MomentumCombine2"],
  Cell[1407248, 55245, 164, 5, 26, "Input",
   CellTags->"MomentumCombine2"],
  Cell[1407415, 55252, 351, 10, 38, "Output",
   CellTags->"MomentumCombine2"]},
 "MomentumExpand"->{
  Cell[1407793, 55266, 65, 1, 39, "ObjectName",
   CellTags->"MomentumExpand"],
  Cell[1407883, 55271, 65, 1, 34, "Subsubsection",
   CellTags->"MomentumExpand"],
  Cell[1407951, 55274, 149, 4, 24, "Text",
   CellTags->"MomentumExpand"],
  Cell[1408103, 55280, 349, 14, 24, "Text",
   CellTags->"MomentumExpand"],
  Cell[1408489, 55299, 62, 1, 34, "Subsubsection",
   CellTags->"MomentumExpand"],
  Cell[1408576, 55304, 218, 6, 26, "Input",
   CellTags->"MomentumExpand"],
  Cell[1408797, 55312, 231, 6, 38, "Output",
   CellTags->"MomentumExpand"],
  Cell[1409065, 55323, 168, 5, 26, "Input",
   CellTags->"MomentumExpand"],
  Cell[1409236, 55330, 292, 10, 28, "Output",
   CellTags->"MomentumExpand"],
  Cell[1409565, 55345, 116, 3, 26, "Input",
   CellTags->"MomentumExpand"],
  Cell[1409684, 55350, 289, 8, 38, "Output",
   CellTags->"MomentumExpand"],
  Cell[1410010, 55363, 211, 6, 26, "Input",
   CellTags->"MomentumExpand"],
  Cell[1410224, 55371, 400, 16, 28, "Output",
   CellTags->"MomentumExpand"],
  Cell[1410661, 55392, 116, 3, 26, "Input",
   CellTags->"MomentumExpand"],
  Cell[1410780, 55397, 326, 9, 38, "Output",
   CellTags->"MomentumExpand"],
  Cell[1411143, 55411, 236, 7, 26, "Input",
   CellTags->"MomentumExpand"],
  Cell[1411382, 55420, 428, 17, 28, "Output",
   CellTags->"MomentumExpand"],
  Cell[1411847, 55442, 116, 3, 26, "Input",
   CellTags->"MomentumExpand"],
  Cell[1411966, 55447, 409, 12, 38, "Output",
   CellTags->"MomentumExpand"],
  Cell[1412412, 55464, 112, 3, 26, "Input",
   CellTags->"MomentumExpand"],
  Cell[1412527, 55469, 602, 24, 28, "Output",
   CellTags->"MomentumExpand"]},
 "MT"->{
  Cell[1413156, 55497, 41, 1, 41, "ObjectName",
   CellTags->"MT"],
  Cell[1413222, 55502, 53, 1, 36, "Subsubsection",
   CellTags->"MT"],
  Cell[1413278, 55505, 182, 7, 25, "Text",
   CellTags->"MT"],
  Cell[1413463, 55514, 604, 29, 25, "Text",
   CellTags->"MT"],
  Cell[1414104, 55548, 50, 1, 34, "Subsubsection",
   CellTags->"MT"],
  Cell[1414179, 55553, 134, 4, 26, "Input",
   CellTags->"MT"],
  Cell[1414316, 55559, 197, 6, 26, "Output",
   CellTags->"MT"],
  Cell[1414550, 55570, 263, 8, 26, "Input",
   CellTags->"MT"],
  Cell[1414816, 55580, 141, 4, 26, "Output",
   CellTags->"MT"],
  Cell[1414994, 55589, 157, 5, 26, "Input",
   CellTags->"MT"],
  Cell[1415154, 55596, 180, 5, 38, "Output",
   CellTags->"MT"],
  Cell[1415371, 55606, 189, 6, 26, "Input",
   CellTags->"MT"],
  Cell[1415563, 55614, 260, 7, 38, "Output",
   CellTags->"MT"],
  Cell[1415860, 55626, 225, 8, 26, "Input",
   CellTags->"MT"],
  Cell[1416088, 55636, 178, 5, 38, "Output",
   CellTags->"MT"]},
 "MTD"->{
  Cell[1416293, 55645, 43, 1, 39, "ObjectName",
   CellTags->"MTD"],
  Cell[1416361, 55650, 54, 1, 34, "Subsubsection",
   CellTags->"MTD"],
  Cell[1416418, 55653, 184, 7, 24, "Text",
   CellTags->"MTD"],
  Cell[1416605, 55662, 602, 29, 24, "Text",
   CellTags->"MTD"],
  Cell[1417244, 55696, 51, 1, 34, "Subsubsection",
   CellTags->"MTD"],
  Cell[1417320, 55701, 136, 4, 26, "Input",
   CellTags->"MTD"],
  Cell[1417459, 55707, 198, 6, 26, "Output",
   CellTags->"MTD"],
  Cell[1417694, 55718, 266, 8, 26, "Input",
   CellTags->"MTD"],
  Cell[1417963, 55728, 144, 4, 27, "Output",
   CellTags->"MTD"],
  Cell[1418144, 55737, 175, 6, 26, "Input",
   CellTags->"MTD"],
  Cell[1418322, 55745, 193, 5, 38, "Output",
   CellTags->"MTD"],
  Cell[1418552, 55755, 207, 7, 26, "Input",
   CellTags->"MTD"],
  Cell[1418762, 55764, 324, 9, 38, "Output",
   CellTags->"MTD"],
  Cell[1419123, 55778, 235, 8, 26, "Input",
   CellTags->"MTD"],
  Cell[1419361, 55788, 188, 5, 38, "Output",
   CellTags->"MTD"]},
 "NegativeInteger"->{
  Cell[1419576, 55797, 67, 1, 39, "ObjectName",
   CellTags->"NegativeInteger"],
  Cell[1419668, 55802, 66, 1, 34, "Subsubsection",
   CellTags->"NegativeInteger"],
  Cell[1419737, 55805, 141, 4, 24, "Text",
   CellTags->"NegativeInteger"],
  Cell[1419881, 55811, 186, 9, 24, "Text",
   CellTags->"NegativeInteger"]},
 "Nf"->{
  Cell[1420082, 55823, 41, 1, 39, "ObjectName",
   CellTags->"Nf"],
  Cell[1420148, 55828, 53, 1, 34, "Subsubsection",
   CellTags->"Nf"],
  Cell[1420204, 55831, 66, 1, 24, "Text",
   CellTags->"Nf"],
  Cell[1420273, 55834, 685, 29, 24, "Text",
   CellTags->"Nf"]},
 "Nielsen"->{
  Cell[1420973, 55866, 51, 1, 39, "ObjectName",
   CellTags->"Nielsen"],
  Cell[1421049, 55871, 58, 1, 34, "Subsubsection",
   CellTags->"Nielsen"],
  Cell[1421110, 55874, 86, 1, 24, "Text",
   CellTags->"Nielsen"],
  Cell[1421221, 55879, 114, 3, 26, "Input",
   CellTags->"Nielsen"],
  Cell[1421338, 55884, 211, 7, 26, "Output",
   CellTags->"Nielsen"],
  Cell[1421564, 55894, 199, 9, 25, "Text",
   CellTags->"Nielsen"],
  Cell[1421800, 55908, 55, 1, 34, "Subsubsection",
   CellTags->"Nielsen"],
  Cell[1421880, 55913, 141, 4, 26, "Input",
   CellTags->"Nielsen"],
  Cell[1422024, 55919, 293, 11, 26, "Output",
   CellTags->"Nielsen"],
  Cell[1422332, 55933, 176, 4, 24, "Text",
   CellTags->"Nielsen"],
  Cell[1422533, 55941, 172, 5, 26, "Input",
   CellTags->"Nielsen"],
  Cell[1422708, 55948, 167, 4, 26, "Output",
   CellTags->"Nielsen"],
  Cell[1422890, 55955, 71, 1, 24, "Text",
   CellTags->"Nielsen"],
  Cell[1422986, 55960, 467, 14, 26, "Input",
   CellTags->"Nielsen"],
  Cell[1423456, 55976, 362, 14, 43, "Output",
   CellTags->"Nielsen"],
  Cell[1423855, 55995, 190, 5, 26, "Input",
   CellTags->"Nielsen"],
  Cell[1424048, 56002, 615, 21, 44, "Output",
   CellTags->"Nielsen"],
  Cell[1424700, 56028, 190, 5, 26, "Input",
   CellTags->"Nielsen"],
  Cell[1424893, 56035, 849, 28, 47, "Output",
   CellTags->"Nielsen"],
  Cell[1425779, 56068, 190, 5, 26, "Input",
   CellTags->"Nielsen"],
  Cell[1425972, 56075, 182, 6, 26, "Output",
   CellTags->"Nielsen"]},
 "NonCommFreeQ"->{
  Cell[1426181, 56085, 61, 1, 70, "ObjectName",
   CellTags->"NonCommFreeQ"],
  Cell[1426267, 56090, 63, 1, 70, "Subsubsection",
   CellTags->"NonCommFreeQ"],
  Cell[1426333, 56093, 244, 5, 70, "Text",
   CellTags->"NonCommFreeQ"],
  Cell[1426580, 56100, 519, 24, 70, "Text",
   CellTags->"NonCommFreeQ"]},
 "NonCommQ"->{
  Cell[1427114, 56127, 53, 1, 70, "ObjectName",
   CellTags->"NonCommQ"],
  Cell[1427192, 56132, 59, 1, 70, "Subsubsection",
   CellTags->"NonCommQ"],
  Cell[1427254, 56135, 203, 4, 70, "Text",
   CellTags->"NonCommQ"],
  Cell[1427460, 56141, 527, 24, 70, "Text",
   CellTags->"NonCommQ"],
  Cell[1430556, 56289, 167, 9, 70, "Text",
   CellTags->"NonCommQ"]},
 "NonCommutative"->{
  Cell[1428002, 56168, 65, 1, 70, "ObjectName",
   CellTags->"NonCommutative"],
  Cell[1428092, 56173, 65, 1, 70, "Subsubsection",
   CellTags->"NonCommutative"],
  Cell[1428160, 56176, 150, 4, 70, "Text",
   CellTags->"NonCommutative"],
  Cell[1428313, 56182, 333, 14, 70, "Text",
   CellTags->"NonCommutative"]},
 "NTerms"->{
  Cell[1428661, 56199, 49, 1, 70, "ObjectName",
   CellTags->"NTerms"],
  Cell[1428735, 56204, 57, 1, 70, "Subsubsection",
   CellTags->"NTerms"],
  Cell[1428795, 56207, 154, 4, 70, "Text",
   CellTags->"NTerms"],
  Cell[1428986, 56216, 54, 1, 34, "Subsubsection",
   CellTags->"NTerms"],
  Cell[1429065, 56221, 129, 4, 26, "Input",
   CellTags->"NTerms"],
  Cell[1429197, 56227, 147, 4, 26, "Output",
   CellTags->"NTerms"],
  Cell[1429381, 56236, 139, 4, 26, "Input",
   CellTags->"NTerms"],
  Cell[1429523, 56242, 145, 4, 26, "Output",
   CellTags->"NTerms"],
  Cell[1429705, 56251, 106, 3, 26, "Input",
   CellTags->"NTerms"],
  Cell[1429814, 56256, 147, 4, 26, "Output",
   CellTags->"NTerms"],
  Cell[1429998, 56265, 106, 3, 26, "Input",
   CellTags->"NTerms"],
  Cell[1430107, 56270, 145, 4, 26, "Output",
   CellTags->"NTerms"]},
 "NumberOfMetricTensors"->{
  Cell[1430279, 56278, 79, 1, 70, "ObjectName",
   CellTags->"NumberOfMetricTensors"],
  Cell[1430383, 56283, 72, 1, 70, "Subsubsection",
   CellTags->"NumberOfMetricTensors"],
  Cell[1430458, 56286, 95, 1, 70, "Text",
   CellTags->"NumberOfMetricTensors"]},
 "NumericalFactor"->{
  Cell[1430738, 56301, 67, 1, 70, "ObjectName",
   CellTags->"NumericalFactor"],
  Cell[1430830, 56306, 66, 1, 52, "Subsubsection",
   CellTags->"NumericalFactor"],
  Cell[1430899, 56309, 119, 3, 70, "Text",
   CellTags->"NumericalFactor"]},
 "NumericQ1"->{
  Cell[1431033, 56315, 55, 1, 57, "ObjectName",
   CellTags->"NumericQ1"],
  Cell[1431113, 56320, 60, 1, 52, "Subsubsection",
   CellTags->"NumericQ1"],
  Cell[1431176, 56323, 139, 4, 25, "Text",
   CellTags->"NumericQ1"],
  Cell[1431352, 56332, 57, 1, 34, "Subsubsection",
   CellTags->"NumericQ1"],
  Cell[1431434, 56337, 224, 7, 30, "Input",
   CellTags->"NumericQ1"],
  Cell[1431661, 56346, 152, 4, 26, "Output",
   CellTags->"NumericQ1"],
  Cell[1431850, 56355, 271, 9, 30, "Input",
   CellTags->"NumericQ1"],
  Cell[1432124, 56366, 154, 4, 26, "Output",
   CellTags->"NumericQ1"],
  Cell[1432315, 56375, 311, 10, 30, "Input",
   CellTags->"NumericQ1"],
  Cell[1432629, 56387, 151, 4, 26, "Output",
   CellTags->"NumericQ1"]},
 "OneLoop"->{
  Cell[1432829, 56397, 69, 1, 41, "ObjectName",
   CellTags->"OneLoop"],
  Cell[1432901, 56400, 619, 16, 61, "Text",
   CellTags->"OneLoop"],
  Cell[1433523, 56418, 968, 26, 61, "Text",
   CellTags->"OneLoop"],
  Cell[1434528, 56449, 58, 1, 36, "Subsubsection",
   CellTags->"OneLoop"],
  Cell[1434589, 56452, 391, 12, 43, "Text",
   CellTags->"OneLoop"],
  Cell[1434983, 56466, 352, 6, 58, "Text",
   CellTags->"OneLoop"],
  Cell[1435360, 56476, 114, 3, 26, "Input",
   CellTags->"OneLoop"],
  Cell[1435477, 56481, 1317, 30, 87, "Output",
   CellTags->"OneLoop"],
  Cell[1436809, 56514, 781, 39, 25, "Text",
   CellTags->"OneLoop"],
  Cell[1437627, 56558, 55, 1, 34, "Subsubsection",
   CellTags->"OneLoop"],
  Cell[1437707, 56563, 275, 10, 26, "Input",
   CellTags->"OneLoop"],
  Cell[1437985, 56575, 365, 12, 50, "Output",
   CellTags->"OneLoop"],
  Cell[1438387, 56592, 131, 4, 26, "Input",
   CellTags->"OneLoop"],
  Cell[1438521, 56598, 310, 11, 29, "Output",
   CellTags->"OneLoop"],
  Cell[1438846, 56612, 363, 11, 26, "Input",
   CellTags->"OneLoop"],
  Cell[1439212, 56625, 588, 20, 27, "Text",
   CellTags->"OneLoop"],
  Cell[1439825, 56649, 908, 30, 80, "Input",
   CellTags->"OneLoop"],
  Cell[1440736, 56681, 1913, 66, 55, "Output",
   CellTags->"OneLoop"],
  Cell[1442664, 56750, 219, 5, 41, "Text",
   CellTags->"OneLoop"],
  Cell[1442908, 56759, 131, 4, 26, "Input",
   CellTags->"OneLoop"],
  Cell[1443042, 56765, 2448, 80, 56, "Output",
   CellTags->"OneLoop"],
  Cell[1445527, 56850, 113, 3, 26, "Input",
   CellTags->"OneLoop"],
  Cell[1445643, 56855, 1749, 59, 50, "Output",
   CellTags->"OneLoop"],
  Cell[1447429, 56919, 333, 11, 26, "Input",
   CellTags->"OneLoop"],
  Cell[1447765, 56932, 486, 18, 52, "Output",
   CellTags->"OneLoop"],
  Cell[1448288, 56955, 131, 4, 26, "Input",
   CellTags->"OneLoop"],
  Cell[1448422, 56961, 1922, 67, 53, "Output",
   CellTags->"OneLoop"],
  Cell[1450359, 57031, 106, 3, 26, "Input",
   CellTags->"OneLoop"]},
 "OneLoopSimplify"->{
  Cell[1450480, 57037, 67, 1, 41, "ObjectName",
   CellTags->"OneLoopSimplify"],
  Cell[1450572, 57042, 66, 1, 36, "Subsubsection",
   CellTags->"OneLoopSimplify"],
  Cell[1450641, 57045, 172, 4, 24, "Text",
   CellTags->"OneLoopSimplify"],
  Cell[1450838, 57053, 130, 3, 26, "Input",
   CellTags->"OneLoopSimplify"],
  Cell[1450971, 57058, 779, 19, 47, "Output",
   CellTags->"OneLoopSimplify"],
  Cell[1451765, 57080, 376, 19, 25, "Text",
   CellTags->"OneLoopSimplify"],
  Cell[1452178, 57104, 63, 1, 34, "Subsubsection",
   CellTags->"OneLoopSimplify"],
  Cell[1452266, 57109, 341, 11, 26, "Input",
   CellTags->"OneLoopSimplify"],
  Cell[1452610, 57122, 494, 18, 52, "Output",
   CellTags->"OneLoopSimplify"],
  Cell[1453141, 57145, 147, 4, 26, "Input",
   CellTags->"OneLoopSimplify"],
  Cell[1453291, 57151, 492, 18, 52, "Output",
   CellTags->"OneLoopSimplify"],
  Cell[1453820, 57174, 203, 6, 26, "Input",
   CellTags->"OneLoopSimplify"],
  Cell[1454026, 57182, 413, 15, 51, "Output",
   CellTags->"OneLoopSimplify"],
  Cell[1454476, 57202, 401, 12, 26, "Input",
   CellTags->"OneLoopSimplify"],
  Cell[1454880, 57216, 624, 23, 51, "Output",
   CellTags->"OneLoopSimplify"],
  Cell[1455541, 57244, 152, 4, 26, "Input",
   CellTags->"OneLoopSimplify"],
  Cell[1455696, 57250, 7210, 262, 102, "Output",
   CellTags->"OneLoopSimplify"],
  Cell[1462943, 57517, 253, 9, 26, "Input",
   CellTags->"OneLoopSimplify"],
  Cell[1463199, 57528, 4318, 163, 100, "Output",
   CellTags->"OneLoopSimplify"],
  Cell[1467554, 57696, 424, 15, 26, "Input",
   CellTags->"OneLoopSimplify"],
  Cell[1467981, 57713, 1552, 59, 52, "Output",
   CellTags->"OneLoopSimplify"]},
 "OneLoopSum"->{
  Cell[1469560, 57776, 57, 1, 41, "ObjectName",
   CellTags->"OneLoopSum"],
  Cell[1469642, 57781, 61, 1, 36, "Subsubsection",
   CellTags->"OneLoopSum"],
  Cell[1469706, 57784, 190, 4, 43, "Text",
   CellTags->"OneLoopSum"],
  Cell[1469899, 57790, 178, 9, 25, "Text",
   CellTags->"OneLoopSum"]},
 "OPE"->{
  Cell[1470092, 57802, 43, 1, 41, "ObjectName",
   CellTags->"OPE"],
  Cell[1470160, 57807, 54, 1, 36, "Subsubsection",
   CellTags->"OPE"],
  Cell[1470217, 57810, 170, 4, 51, "Text",
   CellTags->"OPE"],
  Cell[1470777, 57834, 174, 9, 25, "Text",
   CellTags->"OPE"]},
 "OPEDelta"->{
  Cell[1470966, 57846, 53, 1, 41, "ObjectName",
   CellTags->"OPEDelta"],
  Cell[1471044, 57851, 59, 1, 36, "Subsubsection",
   CellTags->"OPEDelta"],
  Cell[1471106, 57854, 140, 4, 25, "Text",
   CellTags->"OPEDelta"],
  Cell[1471249, 57860, 212, 9, 25, "Text",
   CellTags->"OPEDelta"],
  Cell[1471498, 57874, 56, 1, 34, "Subsubsection",
   CellTags->"OPEDelta"],
  Cell[1471579, 57879, 146, 4, 26, "Input",
   CellTags->"OPEDelta"],
  Cell[1471728, 57885, 221, 7, 29, "Output",
   CellTags->"OPEDelta"],
  Cell[1471986, 57897, 133, 4, 26, "Input",
   CellTags->"OPEDelta"],
  Cell[1472122, 57903, 149, 4, 26, "Output",
   CellTags->"OPEDelta"],
  Cell[1472308, 57912, 141, 4, 26, "Input",
   CellTags->"OPEDelta"],
  Cell[1472452, 57918, 149, 4, 26, "Output",
   CellTags->"OPEDelta"]},
 "OPEi"->{
  Cell[1472628, 57926, 81, 1, 41, "ObjectName",
   CellTags->"OPEi"],
  Cell[1472734, 57931, 55, 1, 36, "Subsubsection",
   CellTags->"OPEi"],
  Cell[1472792, 57934, 171, 4, 25, "Text",
   CellTags->"OPEi"],
  Cell[1473000, 57943, 52, 1, 34, "Subsubsection",
   CellTags->"OPEi"],
  Cell[1473077, 57948, 75, 2, 26, "Input",
   CellTags->"OPEi"],
  Cell[1473155, 57952, 143, 4, 27, "Output",
   CellTags->"OPEi"],
  Cell[1473335, 57961, 259, 7, 26, "Input",
   CellTags->"OPEi"],
  Cell[1473597, 57970, 270, 8, 26, "Output",
   CellTags->"OPEi"],
  Cell[1473904, 57983, 1076, 40, 46, "Input",
   CellTags->"OPEi"],
  Cell[1474983, 58025, 244, 7, 26, "Output",
   CellTags->"OPEi"],
  Cell[1475242, 58035, 55, 1, 24, "Text",
   CellTags->"OPEi"],
  Cell[1475322, 58040, 519, 17, 26, "Input",
   CellTags->"OPEi"],
  Cell[1475844, 58059, 571, 20, 26, "Output",
   CellTags->"OPEi"],
  Cell[1476452, 58084, 534, 19, 26, "Input",
   CellTags->"OPEi"],
  Cell[1476989, 58105, 454, 16, 26, "Output",
   CellTags->"OPEi"],
  Cell[1477480, 58126, 519, 17, 26, "Input",
   CellTags->"OPEi"],
  Cell[1478002, 58145, 571, 20, 26, "Output",
   CellTags->"OPEi"]},
 "OPEInsert"->{
  Cell[1478600, 58169, 73, 1, 41, "ObjectName",
   CellTags->"OPEInsert"],
  Cell[1478698, 58174, 60, 1, 36, "Subsubsection",
   CellTags->"OPEInsert"],
  Cell[1478761, 58177, 264, 5, 43, "Text",
   CellTags->"OPEInsert"],
  Cell[1479028, 58184, 165, 9, 25, "Text",
   CellTags->"OPEInsert"],
  Cell[1479230, 58198, 57, 1, 34, "Subsubsection",
   CellTags->"OPEInsert"],
  Cell[1479290, 58201, 76, 2, 26, "Input",
   CellTags->"OPEInsert"]},
 "OPEInt"->{
  Cell[1479381, 58206, 67, 1, 41, "ObjectName",
   CellTags->"OPEInt"],
  Cell[1479473, 58211, 57, 1, 36, "Subsubsection",
   CellTags->"OPEInt"],
  Cell[1479533, 58214, 257, 5, 43, "Text",
   CellTags->"OPEInt"],
  Cell[1479793, 58221, 162, 9, 25, "Text",
   CellTags->"OPEInt"],
  Cell[1479992, 58235, 54, 1, 34, "Subsubsection",
   CellTags->"OPEInt"],
  Cell[1480049, 58238, 73, 2, 26, "Input",
   CellTags->"OPEInt"]},
 "OPEIntDelta"->{
  Cell[1480137, 58243, 77, 1, 41, "ObjectName",
   CellTags->"OPEIntDelta"],
  Cell[1480239, 58248, 62, 1, 36, "Subsubsection",
   CellTags->"OPEIntDelta"],
  Cell[1480304, 58251, 478, 10, 87, "Text",
   CellTags->"OPEIntDelta"],
  Cell[1480785, 58263, 167, 9, 25, "Text",
   CellTags->"OPEIntDelta"],
  Cell[1480989, 58277, 59, 1, 34, "Subsubsection",
   CellTags->"OPEIntDelta"],
  Cell[1481051, 58280, 78, 2, 26, "Input",
   CellTags->"OPEIntDelta"]},
 "OPEIntegrate"->{
  Cell[1481144, 58285, 79, 1, 41, "ObjectName",
   CellTags->"OPEIntegrate"],
  Cell[1481248, 58290, 63, 1, 36, "Subsubsection",
   CellTags->"OPEIntegrate"],
  Cell[1481314, 58293, 266, 5, 43, "Text",
   CellTags->"OPEIntegrate"],
  Cell[1481583, 58300, 168, 9, 25, "Text",
   CellTags->"OPEIntegrate"],
  Cell[1481788, 58314, 60, 1, 34, "Subsubsection",
   CellTags->"OPEIntegrate"],
  Cell[1481851, 58317, 79, 2, 26, "Input",
   CellTags->"OPEIntegrate"]},
 "OPEIntegrateDelta"->{
  Cell[1481945, 58322, 89, 1, 41, "ObjectName",
   CellTags->"OPEIntegrateDelta"],
  Cell[1482059, 58327, 68, 1, 36, "Subsubsection",
   CellTags->"OPEIntegrateDelta"],
  Cell[1482130, 58330, 499, 10, 87, "Text",
   CellTags->"OPEIntegrateDelta"],
  Cell[1482632, 58342, 173, 9, 25, "Text",
   CellTags->"OPEIntegrateDelta"],
  Cell[1482842, 58356, 65, 1, 34, "Subsubsection",
   CellTags->"OPEIntegrateDelta"],
  Cell[1482910, 58359, 84, 2, 26, "Input",
   CellTags->"OPEIntegrateDelta"]},
 "OPESumExplicit"->{
  Cell[1483009, 58364, 65, 1, 41, "ObjectName",
   CellTags->"OPESumExplicit"],
  Cell[1483099, 58369, 65, 1, 36, "Subsubsection",
   CellTags->"OPESumExplicit"],
  Cell[1483167, 58372, 85, 1, 25, "Text",
   CellTags->"OPESumExplicit"],
  Cell[1483255, 58375, 307, 14, 25, "Text",
   CellTags->"OPESumExplicit"],
  Cell[1483599, 58394, 62, 1, 34, "Subsubsection",
   CellTags->"OPESumExplicit"],
  Cell[1483686, 58399, 387, 13, 26, "Input",
   CellTags->"OPESumExplicit"],
  Cell[1484076, 58414, 449, 16, 57, "Output",
   CellTags->"OPESumExplicit"],
  Cell[1484562, 58435, 123, 3, 26, "Input",
   CellTags->"OPESumExplicit"],
  Cell[1484688, 58440, 450, 17, 46, "Output",
   CellTags->"OPESumExplicit"],
  Cell[1485175, 58462, 554, 18, 26, "Input",
   CellTags->"OPESumExplicit"],
  Cell[1485732, 58482, 744, 28, 59, "Output",
   CellTags->"OPESumExplicit"],
  Cell[1486513, 58515, 123, 3, 26, "Input",
   CellTags->"OPESumExplicit"],
  Cell[1486639, 58520, 1791, 67, 95, "Output",
   CellTags->"OPESumExplicit"],
  Cell[1488445, 58590, 157, 5, 26, "Input",
   CellTags->"OPESumExplicit"]},
 "OPESum"->{
  Cell[1488617, 58598, 49, 1, 70, "ObjectName",
   CellTags->"OPESum"],
  Cell[1488691, 58603, 57, 1, 70, "Subsubsection",
   CellTags->"OPESum"],
  Cell[1488751, 58606, 127, 4, 70, "Text",
   CellTags->"OPESum"],
  Cell[1488881, 58612, 323, 14, 70, "Text",
   CellTags->"OPESum"],
  Cell[1489241, 58631, 54, 1, 34, "Subsubsection",
   CellTags->"OPESum"],
  Cell[1489320, 58636, 460, 15, 26, "Input",
   CellTags->"OPESum"],
  Cell[1489783, 58653, 897, 32, 57, "Output",
   CellTags->"OPESum"],
  Cell[1490717, 58690, 115, 3, 26, "Input",
   CellTags->"OPESum"],
  Cell[1490835, 58695, 1648, 61, 51, "Output",
   CellTags->"OPESum"],
  Cell[1492520, 58761, 546, 18, 26, "Input",
   CellTags->"OPESum"],
  Cell[1493069, 58781, 736, 28, 59, "Output",
   CellTags->"OPESum"],
  Cell[1493842, 58814, 115, 3, 26, "Input",
   CellTags->"OPESum"],
  Cell[1493960, 58819, 1781, 67, 95, "Output",
   CellTags->"OPESum"],
  Cell[1495756, 58889, 149, 5, 26, "Input",
   CellTags->"OPESum"]},
 "OPESumSimplify"->{
  Cell[1495920, 58897, 65, 1, 70, "ObjectName",
   CellTags->"OPESumSimplify"],
  Cell[1496010, 58902, 65, 1, 70, "Subsubsection",
   CellTags->"OPESumSimplify"],
  Cell[1496078, 58905, 92, 1, 70, "Text",
   CellTags->"OPESumSimplify"],
  Cell[1496173, 58908, 307, 14, 70, "Text",
   CellTags->"OPESumSimplify"],
  Cell[1496517, 58927, 62, 1, 34, "Subsubsection",
   CellTags->"OPESumSimplify"],
  Cell[1496604, 58932, 143, 4, 26, "Input",
   CellTags->"OPESumSimplify"],
  Cell[1496750, 58938, 1096, 38, 55, "Output",
   CellTags->"OPESumSimplify"],
  Cell[1497883, 58981, 89, 2, 26, "Input",
   CellTags->"OPESumSimplify"],
  Cell[1497975, 58985, 1194, 40, 55, "Output",
   CellTags->"OPESumSimplify"],
  Cell[1499206, 59030, 273, 8, 26, "Input",
   CellTags->"OPESumSimplify"],
  Cell[1499482, 59040, 325, 12, 55, "Output",
   CellTags->"OPESumSimplify"],
  Cell[1499844, 59057, 439, 14, 26, "Input",
   CellTags->"OPESumSimplify"],
  Cell[1500286, 59073, 595, 23, 55, "Output",
   CellTags->"OPESumSimplify"],
  Cell[1500918, 59101, 116, 3, 26, "Input",
   CellTags->"OPESumSimplify"],
  Cell[1501037, 59106, 446, 14, 41, "Output",
   CellTags->"OPESumSimplify"]},
 "OPE1Loop"->{
  Cell[1501510, 59124, 54, 1, 41, "ObjectName",
   CellTags->"OPE1Loop"],
  Cell[1501589, 59129, 59, 1, 36, "Subsubsection",
   CellTags->"OPE1Loop"],
  Cell[1501651, 59132, 119, 3, 25, "Text",
   CellTags->"OPE1Loop"],
  Cell[1501773, 59137, 173, 9, 25, "Text",
   CellTags->"OPE1Loop"],
  Cell[1501983, 59151, 56, 1, 34, "Subsubsection",
   CellTags->"OPE1Loop"],
  Cell[1502042, 59154, 75, 2, 26, "Input",
   CellTags->"OPE1Loop"]},
 "OPE2TID"->{
  Cell[1502132, 59159, 69, 1, 41, "ObjectName",
   CellTags->"OPE2TID"],
  Cell[1502226, 59164, 58, 1, 36, "Subsubsection",
   CellTags->"OPE2TID"],
  Cell[1502287, 59167, 184, 4, 24, "Text",
   CellTags->"OPE2TID"],
  Cell[1502496, 59175, 114, 3, 26, "Input",
   CellTags->"OPE2TID"],
  Cell[1502613, 59180, 387, 11, 26, "Output",
   CellTags->"OPE2TID"],
  Cell[1503015, 59194, 172, 9, 25, "Text",
   CellTags->"OPE2TID"],
  Cell[1503224, 59208, 55, 1, 34, "Subsubsection",
   CellTags->"OPE2TID"],
  Cell[1503282, 59211, 74, 2, 26, "Input",
   CellTags->"OPE2TID"]},
 "OptionsSelect"->{
  Cell[1503371, 59216, 63, 1, 41, "ObjectName",
   CellTags->"OptionsSelect"],
  Cell[1503459, 59221, 64, 1, 36, "Subsubsection",
   CellTags->"OptionsSelect"],
  Cell[1503526, 59224, 219, 5, 41, "Text",
   CellTags->"OptionsSelect"]},
 "Pair"->{
  Cell[1503760, 59232, 45, 1, 39, "ObjectName",
   CellTags->"Pair"],
  Cell[1503830, 59237, 55, 1, 34, "Subsubsection",
   CellTags->"Pair"],
  Cell[1503888, 59240, 499, 11, 60, "Text",
   CellTags->"Pair"],
  Cell[1504390, 59253, 989, 49, 24, "Text",
   CellTags->"Pair"],
  Cell[1505416, 59307, 52, 1, 34, "Subsubsection",
   CellTags->"Pair"],
  Cell[1505471, 59310, 84, 1, 24, "Text",
   CellTags->"Pair"],
  Cell[1505580, 59315, 218, 6, 26, "Input",
   CellTags->"Pair"],
  Cell[1505801, 59323, 201, 6, 26, "Output",
   CellTags->"Pair"],
  Cell[1506017, 59332, 73, 1, 24, "Text",
   CellTags->"Pair"],
  Cell[1506115, 59337, 268, 8, 26, "Input",
   CellTags->"Pair"],
  Cell[1506386, 59347, 199, 6, 26, "Output",
   CellTags->"Pair"],
  Cell[1506622, 59358, 208, 6, 26, "Input",
   CellTags->"Pair"],
  Cell[1506833, 59366, 278, 11, 28, "Output",
   CellTags->"Pair"],
  Cell[1507148, 59382, 197, 6, 26, "Input",
   CellTags->"Pair"],
  Cell[1507348, 59390, 245, 9, 28, "Output",
   CellTags->"Pair"],
  Cell[1507630, 59404, 197, 6, 26, "Input",
   CellTags->"Pair"],
  Cell[1507830, 59412, 201, 7, 28, "Output",
   CellTags->"Pair"],
  Cell[1508068, 59424, 222, 7, 26, "Input",
   CellTags->"Pair"],
  Cell[1508293, 59433, 280, 10, 28, "Output",
   CellTags->"Pair"],
  Cell[1508610, 59448, 223, 7, 26, "Input",
   CellTags->"Pair"],
  Cell[1508836, 59457, 265, 10, 31, "Output",
   CellTags->"Pair"],
  Cell[1509138, 59472, 223, 7, 26, "Input",
   CellTags->"Pair"],
  Cell[1509364, 59481, 267, 10, 31, "Output",
   CellTags->"Pair"],
  Cell[1509668, 59496, 308, 10, 26, "Input",
   CellTags->"Pair"],
  Cell[1509979, 59508, 145, 4, 26, "Output",
   CellTags->"Pair"],
  Cell[1510161, 59517, 272, 8, 26, "Input",
   CellTags->"Pair"],
  Cell[1510436, 59527, 334, 13, 28, "Output",
   CellTags->"Pair"],
  Cell[1510807, 59545, 383, 12, 26, "Input",
   CellTags->"Pair"],
  Cell[1511193, 59559, 142, 4, 26, "Output",
   CellTags->"Pair"]},
 "PairCollect"->{
  Cell[1511362, 59567, 59, 1, 39, "ObjectName",
   CellTags->"PairCollect"],
  Cell[1511446, 59572, 62, 1, 34, "Subsubsection",
   CellTags->"PairCollect"],
  Cell[1511511, 59575, 153, 4, 24, "Text",
   CellTags->"PairCollect"],
  Cell[1511667, 59581, 286, 14, 24, "Text",
   CellTags->"PairCollect"]},
 "PairContract"->{
  Cell[1511968, 59598, 61, 1, 39, "ObjectName",
   CellTags->"PairContract"],
  Cell[1512054, 59603, 63, 1, 34, "Subsubsection",
   CellTags->"PairContract"],
  Cell[1512120, 59606, 118, 3, 24, "Text",
   CellTags->"PairContract"],
  Cell[1512241, 59611, 275, 13, 24, "Text",
   CellTags->"PairContract"]},
 "PairContract2"->{
  Cell[1512531, 59627, 63, 1, 39, "ObjectName",
   CellTags->"PairContract2"],
  Cell[1512619, 59632, 64, 1, 34, "Subsubsection",
   CellTags->"PairContract2"],
  Cell[1512686, 59635, 142, 4, 24, "Text",
   CellTags->"PairContract2"],
  Cell[1512831, 59641, 398, 18, 24, "Text",
   CellTags->"PairContract2"]},
 "PairContract3"->{
  Cell[1513244, 59662, 63, 1, 39, "ObjectName",
   CellTags->"PairContract3"],
  Cell[1513332, 59667, 64, 1, 34, "Subsubsection",
   CellTags->"PairContract3"],
  Cell[1513399, 59670, 142, 4, 24, "Text",
   CellTags->"PairContract3"],
  Cell[1513544, 59676, 398, 18, 24, "Text",
   CellTags->"PairContract3"]},
 "PartialD"->{
  Cell[1513957, 59697, 53, 1, 39, "ObjectName",
   CellTags->"PartialD"],
  Cell[1514035, 59702, 59, 1, 34, "Subsubsection",
   CellTags->"PartialD"],
  Cell[1514097, 59705, 864, 35, 47, "Text",
   CellTags->"PartialD"],
  Cell[1514964, 59742, 581, 24, 24, "Text",
   CellTags->"PartialD"],
  Cell[1515582, 59771, 65, 1, 33, "Subsubsection",
   CellTags->"PartialD"],
  Cell[1515672, 59776, 233, 7, 26, "Input",
   CellTags->"PartialD"],
  Cell[1515908, 59785, 343, 12, 33, "Output",
   CellTags->"PartialD"],
  Cell[1516288, 59802, 116, 3, 26, "Input",
   CellTags->"PartialD"],
  Cell[1516407, 59807, 618, 15, 87, "Output",
   CellTags->"PartialD"],
  Cell[1517062, 59827, 114, 3, 26, "Input",
   CellTags->"PartialD"],
  Cell[1517179, 59832, 607, 12, 115, "Output",
   CellTags->"PartialD"]},
 "PartialDRelations"->{
  Cell[1517813, 59848, 71, 1, 39, "ObjectName",
   CellTags->"PartialDRelations"],
  Cell[1517909, 59853, 68, 1, 34, "Subsubsection",
   CellTags->"PartialDRelations"],
  Cell[1517980, 59856, 191, 5, 24, "Text",
   CellTags->"PartialDRelations"],
  Cell[1518174, 59863, 700, 29, 24, "Text",
   CellTags->"PartialDRelations"],
  Cell[1518911, 59897, 74, 1, 33, "Subsubsection",
   CellTags->"PartialDRelations"],
  Cell[1519010, 59902, 344, 10, 26, "Input",
   CellTags->"PartialDRelations"],
  Cell[1519357, 59914, 392, 13, 33, "Output",
   CellTags->"PartialDRelations"],
  Cell[1519786, 59932, 256, 7, 26, "Input",
   CellTags->"PartialDRelations"],
  Cell[1520045, 59941, 1295, 33, 153, "Output",
   CellTags->"PartialDRelations"]},
 "PartialFourVector"->{
  Cell[1521367, 59978, 71, 1, 39, "ObjectName",
   CellTags->"PartialFourVector"],
  Cell[1521463, 59983, 68, 1, 34, "Subsubsection",
   CellTags->"PartialFourVector"],
  Cell[1521534, 59986, 277, 6, 41, "Text",
   CellTags->"PartialFourVector"],
  Cell[1521814, 59994, 402, 19, 24, "Text",
   CellTags->"PartialFourVector"],
  Cell[1522253, 60018, 74, 1, 33, "Subsubsection",
   CellTags->"PartialFourVector"],
  Cell[1522352, 60023, 382, 11, 33, "Input",
   CellTags->"PartialFourVector"],
  Cell[1522737, 60036, 441, 16, 30, "Output",
   CellTags->"PartialFourVector"]},
 "PartialIntegrate"->{
  Cell[1523205, 60056, 69, 1, 39, "ObjectName",
   CellTags->"PartialIntegrate"],
  Cell[1523299, 60061, 67, 1, 34, "Subsubsection",
   CellTags->"PartialIntegrate"],
  Cell[1523369, 60064, 283, 6, 41, "Text",
   CellTags->"PartialIntegrate"],
  Cell[1523677, 60074, 132, 3, 26, "Input",
   CellTags->"PartialIntegrate"],
  Cell[1523812, 60079, 227, 7, 26, "Output",
   CellTags->"PartialIntegrate"],
  Cell[1524088, 60092, 73, 1, 33, "Subsubsection",
   CellTags->"PartialIntegrate"],
  Cell[1524186, 60097, 326, 10, 26, "Input",
   CellTags->"PartialIntegrate"],
  Cell[1524515, 60109, 1281, 42, 45, "Output",
   CellTags->"PartialIntegrate"],
  Cell[1526219, 60171, 275, 9, 26, "Input",
   CellTags->"PartialIntegrate"],
  Cell[1526497, 60182, 425, 15, 44, "Output",
   CellTags->"PartialIntegrate"],
  Cell[1526959, 60202, 293, 10, 52, "Input",
   CellTags->"PartialIntegrate"],
  Cell[1527255, 60214, 420, 15, 48, "Output",
   CellTags->"PartialIntegrate"],
  Cell[1527712, 60234, 443, 14, 26, "Input",
   CellTags->"PartialIntegrate"],
  Cell[1528158, 60250, 288, 9, 47, "Output",
   CellTags->"PartialIntegrate"],
  Cell[1528483, 60264, 320, 11, 26, "Input",
   CellTags->"PartialIntegrate"],
  Cell[1528806, 60277, 290, 9, 47, "Output",
   CellTags->"PartialIntegrate"],
  Cell[1529111, 60289, 138, 4, 26, "Input",
   CellTags->"PartialIntegrate"]},
 "PartitHead"->{
  Cell[1529264, 60296, 57, 1, 39, "ObjectName",
   CellTags->"PartitHead"],
  Cell[1529346, 60301, 61, 1, 34, "Subsubsection",
   CellTags->"PartitHead"],
  Cell[1529410, 60304, 167, 4, 24, "Text",
   CellTags->"PartitHead"]},
 "PauliSigma"->{
  Cell[1529592, 60311, 57, 1, 39, "ObjectName",
   CellTags->"PauliSigma"],
  Cell[1529674, 60316, 61, 1, 34, "Subsubsection",
   CellTags->"PauliSigma"],
  Cell[1529738, 60319, 252, 5, 41, "Text",
   CellTags->"PauliSigma"],
  Cell[1530015, 60328, 141, 4, 26, "Input",
   CellTags->"PauliSigma"],
  Cell[1530159, 60334, 3637, 101, 116, "Output",
   CellTags->"PauliSigma"]},
 "PaVe"->{
  Cell[1533823, 60439, 45, 1, 39, "ObjectName",
   CellTags->"PaVe"],
  Cell[1533893, 60444, 55, 1, 34, "Subsubsection",
   CellTags->"PaVe"],
  Cell[1533951, 60447, 428, 7, 58, "Text",
   CellTags->"PaVe"],
  Cell[1534382, 60456, 266, 8, 24, "Text",
   CellTags->"PaVe"],
  Cell[1534651, 60466, 181, 9, 24, "Text",
   CellTags->"PaVe"],
  Cell[1534869, 60480, 52, 1, 34, "Subsubsection",
   CellTags->"PaVe"],
  Cell[1534924, 60483, 78, 1, 24, "Text",
   CellTags->"PaVe"],
  Cell[1535027, 60488, 303, 10, 26, "Input",
   CellTags->"PaVe"],
  Cell[1535333, 60500, 348, 10, 26, "Output",
   CellTags->"PaVe"],
  Cell[1535718, 60515, 271, 9, 26, "Input",
   CellTags->"PaVe"],
  Cell[1535992, 60526, 353, 12, 29, "Output",
   CellTags->"PaVe"]},
 "PaVeOrderList"->{
  Cell[1536372, 60542, 63, 1, 39, "ObjectName",
   CellTags->"PaVeOrderList"],
  Cell[1536460, 60547, 64, 1, 34, "Subsubsection",
   CellTags->"PaVeOrderList"],
  Cell[1536527, 60550, 176, 4, 24, "Text",
   CellTags->"PaVeOrderList"],
  Cell[1536706, 60556, 487, 23, 24, "Text",
   CellTags->"PaVeOrderList"]},
 "PaVeOrder"->{
  Cell[1537208, 60582, 55, 1, 39, "ObjectName",
   CellTags->"PaVeOrder"],
  Cell[1537288, 60587, 60, 1, 34, "Subsubsection",
   CellTags->"PaVeOrder"],
  Cell[1537351, 60590, 564, 16, 58, "Text",
   CellTags->"PaVeOrder"],
  Cell[1537918, 60608, 186, 9, 24, "Text",
   CellTags->"PaVeOrder"],
  Cell[1538141, 60622, 57, 1, 34, "Subsubsection",
   CellTags->"PaVeOrder"],
  Cell[1538223, 60627, 413, 12, 26, "Input",
   CellTags->"PaVeOrder"],
  Cell[1538639, 60641, 426, 13, 26, "Output",
   CellTags->"PaVeOrder"],
  Cell[1539102, 60659, 413, 12, 26, "Input",
   CellTags->"PaVeOrder"],
  Cell[1539518, 60673, 426, 13, 26, "Output",
   CellTags->"PaVeOrder"],
  Cell[1539981, 60691, 706, 20, 65, "Input",
   CellTags->"PaVeOrder"],
  Cell[1540690, 60713, 665, 19, 26, "Output",
   CellTags->"PaVeOrder"]},
 "PaVeReduce"->{
  Cell[1541382, 60736, 57, 1, 39, "ObjectName",
   CellTags->"PaVeReduce"],
  Cell[1541464, 60741, 61, 1, 34, "Subsubsection",
   CellTags->"PaVeReduce"],
  Cell[1541528, 60744, 167, 4, 24, "Text",
   CellTags->"PaVeReduce"],
  Cell[1541720, 60752, 120, 3, 26, "Input",
   CellTags->"PaVeReduce"],
  Cell[1541843, 60757, 487, 14, 26, "Output",
   CellTags->"PaVeReduce"],
  Cell[1542367, 60776, 106, 3, 26, "Input",
   CellTags->"PaVeReduce"],
  Cell[1542476, 60781, 374, 6, 79, "Print",
   CellTags->{"Info3500973951-4982097", "PaVeReduce"}],
  Cell[1542865, 60790, 279, 14, 24, "Text",
   CellTags->"PaVeReduce"],
  Cell[1543181, 60809, 58, 1, 34, "Subsubsection",
   CellTags->"PaVeReduce"],
  Cell[1543264, 60814, 520, 16, 26, "Input",
   CellTags->"PaVeReduce"],
  Cell[1543787, 60832, 3060, 105, 109, "Output",
   CellTags->"PaVeReduce"],
  Cell[1546884, 60942, 106, 3, 26, "Input",
   CellTags->"PaVeReduce"],
  Cell[1546993, 60947, 178, 5, 26, "Output",
   CellTags->"PaVeReduce"],
  Cell[1547208, 60957, 107, 3, 26, "Input",
   CellTags->"PaVeReduce"],
  Cell[1547318, 60962, 178, 5, 26, "Output",
   CellTags->"PaVeReduce"],
  Cell[1547533, 60972, 491, 15, 46, "Input",
   CellTags->"PaVeReduce"],
  Cell[1548027, 60989, 512, 16, 44, "Output",
   CellTags->"PaVeReduce"],
  Cell[1548576, 61010, 398, 12, 46, "Input",
   CellTags->"PaVeReduce"],
  Cell[1548977, 61024, 833, 24, 72, "Output",
   CellTags->"PaVeReduce"],
  Cell[1549825, 61051, 209, 7, 26, "Input",
   CellTags->"PaVeReduce"],
  Cell[1550037, 61060, 162, 5, 26, "Input",
   CellTags->"PaVeReduce"],
  Cell[1550224, 61069, 633, 17, 46, "Input",
   CellTags->"PaVeReduce"],
  Cell[1550860, 61088, 8441, 261, 413, "Output",
   CellTags->"PaVeReduce"],
  Cell[1559316, 61352, 267, 8, 26, "Input",
   CellTags->"PaVeReduce"],
  Cell[1559608, 61364, 375, 11, 26, "Input",
   CellTags->"PaVeReduce"],
  Cell[1559986, 61377, 3020, 58, 663, "Output",
   CellTags->"PaVeReduce"],
  Cell[1563021, 61438, 253, 7, 26, "Input",
   CellTags->"PaVeReduce"]},
 "Info3500973951-4982097"->{
  Cell[1542476, 60781, 374, 6, 79, "Print",
   CellTags->{"Info3500973951-4982097", "PaVeReduce"}]},
 "PD"->{
  Cell[1563289, 61448, 41, 1, 39, "ObjectName",
   CellTags->"PD"],
  Cell[1563355, 61453, 53, 1, 34, "Subsubsection",
   CellTags->"PD"],
  Cell[1563411, 61456, 81, 1, 24, "Text",
   CellTags->"PD"],
  Cell[1563495, 61459, 310, 14, 24, "Text",
   CellTags->"PD"]},
 "PlusDistribution"->{
  Cell[1563820, 61476, 69, 1, 39, "ObjectName",
   CellTags->"PlusDistribution"],
  Cell[1563914, 61481, 67, 1, 34, "Subsubsection",
   CellTags->"PlusDistribution"],
  Cell[1563984, 61484, 147, 4, 24, "Text",
   CellTags->"PlusDistribution"],
  Cell[1564134, 61490, 193, 9, 24, "Text",
   CellTags->"PlusDistribution"],
  Cell[1564364, 61504, 64, 1, 34, "Subsubsection",
   CellTags->"PlusDistribution"],
  Cell[1564453, 61509, 198, 6, 26, "Input",
   CellTags->"PlusDistribution"],
  Cell[1564654, 61517, 250, 8, 46, "Output",
   CellTags->"PlusDistribution"],
  Cell[1564941, 61530, 254, 8, 26, "Input",
   CellTags->"PlusDistribution"],
  Cell[1565198, 61540, 310, 10, 46, "Output",
   CellTags->"PlusDistribution"],
  Cell[1565545, 61555, 329, 10, 26, "Input",
   CellTags->"PlusDistribution"],
  Cell[1565877, 61567, 157, 4, 26, "Output",
   CellTags->"PlusDistribution"],
  Cell[1566071, 61576, 389, 12, 26, "Input",
   CellTags->"PlusDistribution"],
  Cell[1566463, 61590, 155, 4, 26, "Output",
   CellTags->"PlusDistribution"],
  Cell[1566655, 61599, 417, 13, 26, "Input",
   CellTags->"PlusDistribution"],
  Cell[1567075, 61614, 155, 4, 26, "Output",
   CellTags->"PlusDistribution"],
  Cell[1567267, 61623, 307, 10, 26, "Input",
   CellTags->"PlusDistribution"],
  Cell[1567577, 61635, 417, 14, 46, "Output",
   CellTags->"PlusDistribution"]},
 "Polarization"->{
  Cell[1568021, 61653, 61, 1, 39, "ObjectName",
   CellTags->"Polarization"],
  Cell[1568107, 61658, 63, 1, 34, "Subsubsection",
   CellTags->"Polarization"],
  Cell[1568173, 61661, 769, 12, 92, "Text",
   CellTags->"Polarization"],
  Cell[1568945, 61675, 484, 19, 24, "Text",
   CellTags->"Polarization"],
  Cell[1569466, 61699, 60, 1, 34, "Subsubsection",
   CellTags->"Polarization"],
  Cell[1569551, 61704, 118, 3, 26, "Input",
   CellTags->"Polarization"],
  Cell[1569672, 61709, 195, 5, 26, "Output",
   CellTags->"Polarization"],
  Cell[1569904, 61719, 153, 4, 26, "Input",
   CellTags->"Polarization"],
  Cell[1570060, 61725, 210, 5, 38, "Output",
   CellTags->"Polarization"],
  Cell[1570307, 61735, 160, 5, 26, "Input",
   CellTags->"Polarization"],
  Cell[1570470, 61742, 279, 9, 26, "Output",
   CellTags->"Polarization"],
  Cell[1570786, 61756, 196, 6, 26, "Input",
   CellTags->"Polarization"],
  Cell[1570985, 61764, 229, 6, 38, "Output",
   CellTags->"Polarization"],
  Cell[1571251, 61775, 155, 4, 26, "Input",
   CellTags->"Polarization"],
  Cell[1571409, 61781, 314, 10, 26, "Output",
   CellTags->"Polarization"],
  Cell[1571760, 61796, 194, 6, 26, "Input",
   CellTags->"Polarization"],
  Cell[1571957, 61804, 250, 6, 38, "Output",
   CellTags->"Polarization"],
  Cell[1572244, 61815, 302, 8, 26, "Input",
   CellTags->"Polarization"],
  Cell[1572549, 61825, 153, 4, 26, "Output",
   CellTags->"Polarization"]},
 "PolarizationSum"->{
  Cell[1572729, 61833, 67, 1, 39, "ObjectName",
   CellTags->"PolarizationSum"],
  Cell[1572821, 61838, 66, 1, 34, "Subsubsection",
   CellTags->"PolarizationSum"],
  Cell[1572890, 61841, 277, 5, 41, "Text",
   CellTags->"PolarizationSum"],
  Cell[1573192, 61850, 130, 3, 26, "Input",
   CellTags->"PolarizationSum"],
  Cell[1573325, 61855, 217, 6, 26, "Output",
   CellTags->"PolarizationSum"],
  Cell[1573557, 61864, 198, 9, 24, "Text",
   CellTags->"PolarizationSum"],
  Cell[1573792, 61878, 63, 1, 34, "Subsubsection",
   CellTags->"PolarizationSum"],
  Cell[1573880, 61883, 155, 4, 26, "Input",
   CellTags->"PolarizationSum"],
  Cell[1574038, 61889, 227, 7, 26, "Output",
   CellTags->"PolarizationSum"],
  Cell[1574302, 61901, 165, 4, 26, "Input",
   CellTags->"PolarizationSum"],
  Cell[1574470, 61907, 649, 26, 47, "Output",
   CellTags->"PolarizationSum"],
  Cell[1575156, 61938, 175, 4, 26, "Input",
   CellTags->"PolarizationSum"],
  Cell[1575334, 61944, 1728, 72, 49, "Output",
   CellTags->"PolarizationSum"],
  Cell[1577099, 62021, 247, 7, 26, "Input",
   CellTags->"PolarizationSum"],
  Cell[1577349, 62030, 4088, 163, 51, "Output",
   CellTags->"PolarizationSum"]},
 "PolarizationUncontract"->{
  Cell[1581464, 62197, 81, 1, 41, "ObjectName",
   CellTags->"PolarizationUncontract"],
  Cell[1581570, 62202, 73, 1, 36, "Subsubsection",
   CellTags->"PolarizationUncontract"],
  Cell[1581646, 62205, 152, 4, 25, "Text",
   CellTags->"PolarizationUncontract"],
  Cell[1581801, 62211, 321, 14, 25, "Text",
   CellTags->"PolarizationUncontract"]},
 "PolarizationVector"->{
  Cell[1582137, 62228, 73, 1, 41, "ObjectName",
   CellTags->"PolarizationVector"],
  Cell[1582235, 62233, 69, 1, 36, "Subsubsection",
   CellTags->"PolarizationVector"],
  Cell[1582307, 62236, 103, 1, 25, "Text",
   CellTags->"PolarizationVector"],
  Cell[1582413, 62239, 409, 18, 25, "Text",
   CellTags->"PolarizationVector"],
  Cell[1582859, 62262, 66, 1, 34, "Subsubsection",
   CellTags->"PolarizationVector"],
  Cell[1582950, 62267, 157, 4, 26, "Input",
   CellTags->"PolarizationVector"],
  Cell[1583110, 62273, 300, 10, 29, "Output",
   CellTags->"PolarizationVector"],
  Cell[1583447, 62288, 194, 5, 26, "Input",
   CellTags->"PolarizationVector"],
  Cell[1583644, 62295, 310, 10, 30, "Output",
   CellTags->"PolarizationVector"],
  Cell[1583969, 62308, 241, 9, 27, "Text",
   CellTags->"PolarizationVector"],
  Cell[1584235, 62321, 190, 5, 26, "Input",
   CellTags->"PolarizationVector"],
  Cell[1584428, 62328, 351, 9, 38, "Output",
   CellTags->"PolarizationVector"],
  Cell[1584794, 62340, 440, 18, 27, "Text",
   CellTags->"PolarizationVector"],
  Cell[1585259, 62362, 284, 8, 26, "Input",
   CellTags->"PolarizationVector"],
  Cell[1585546, 62372, 356, 12, 29, "Output",
   CellTags->"PolarizationVector"],
  Cell[1585939, 62389, 121, 3, 26, "Input",
   CellTags->"PolarizationVector"],
  Cell[1586063, 62394, 157, 4, 26, "Output",
   CellTags->"PolarizationVector"],
  Cell[1586257, 62403, 310, 9, 26, "Input",
   CellTags->"PolarizationVector"],
  Cell[1586570, 62414, 420, 15, 29, "Output",
   CellTags->"PolarizationVector"],
  Cell[1587027, 62434, 121, 3, 26, "Input",
   CellTags->"PolarizationVector"],
  Cell[1587151, 62439, 378, 13, 28, "Output",
   CellTags->"PolarizationVector"],
  Cell[1587544, 62455, 1511, 51, 43, "Text",
   CellTags->"PolarizationVector"],
  Cell[1589080, 62510, 310, 9, 26, "Input",
   CellTags->"PolarizationVector"],
  Cell[1589393, 62521, 418, 15, 29, "Output",
   CellTags->"PolarizationVector"],
  Cell[1589848, 62541, 121, 3, 26, "Input",
   CellTags->"PolarizationVector"],
  Cell[1589972, 62546, 380, 13, 28, "Output",
   CellTags->"PolarizationVector"],
  Cell[1590367, 62562, 153, 4, 26, "Input",
   CellTags->"PolarizationVector"]},
 "PositiveInteger"->{
  Cell[1590535, 62569, 67, 1, 41, "ObjectName",
   CellTags->"PositiveInteger"],
  Cell[1590627, 62574, 66, 1, 36, "Subsubsection",
   CellTags->"PositiveInteger"],
  Cell[1590696, 62577, 136, 4, 25, "Text",
   CellTags->"PositiveInteger"],
  Cell[1590835, 62583, 186, 9, 25, "Text",
   CellTags->"PositiveInteger"]},
 "PositiveNumber"->{
  Cell[1591036, 62595, 65, 1, 41, "ObjectName",
   CellTags->"PositiveNumber"],
  Cell[1591126, 62600, 65, 1, 36, "Subsubsection",
   CellTags->"PositiveNumber"],
  Cell[1591194, 62603, 146, 4, 25, "Text",
   CellTags->"PositiveNumber"],
  Cell[1591343, 62609, 185, 9, 25, "Text",
   CellTags->"PositiveNumber"]},
 "PowerFactor"->{
  Cell[1591543, 62621, 59, 1, 41, "ObjectName",
   CellTags->"PowerFactor"],
  Cell[1591627, 62626, 62, 1, 36, "Subsubsection",
   CellTags->"PowerFactor"],
  Cell[1591692, 62629, 375, 16, 25, "Text",
   CellTags->"PowerFactor"],
  Cell[1592070, 62647, 197, 9, 25, "Text",
   CellTags->"PowerFactor"],
  Cell[1592304, 62661, 59, 1, 34, "Subsubsection",
   CellTags->"PowerFactor"],
  Cell[1592388, 62666, 144, 5, 26, "Input",
   CellTags->"PowerFactor"],
  Cell[1592535, 62673, 223, 7, 26, "Output",
   CellTags->"PowerFactor"],
  Cell[1592795, 62685, 116, 3, 26, "Input",
   CellTags->"PowerFactor"],
  Cell[1592914, 62690, 225, 7, 26, "Output",
   CellTags->"PowerFactor"]},
 "PowerSimplify"->{
  Cell[1593166, 62701, 63, 1, 41, "ObjectName",
   CellTags->"PowerSimplify"],
  Cell[1593254, 62706, 64, 1, 36, "Subsubsection",
   CellTags->"PowerSimplify"],
  Cell[1593321, 62709, 2559, 112, 63, "Text",
   CellTags->"PowerSimplify"],
  Cell[1595883, 62823, 282, 14, 25, "Text",
   CellTags->"PowerSimplify"],
  Cell[1596202, 62842, 61, 1, 34, "Subsubsection",
   CellTags->"PowerSimplify"],
  Cell[1596288, 62847, 234, 8, 26, "Input",
   CellTags->"PowerSimplify"],
  Cell[1596525, 62857, 154, 4, 26, "Output",
   CellTags->"PowerSimplify"],
  Cell[1596716, 62866, 239, 8, 26, "Input",
   CellTags->"PowerSimplify"],
  Cell[1596958, 62876, 220, 7, 26, "Output",
   CellTags->"PowerSimplify"],
  Cell[1597215, 62888, 239, 8, 26, "Input",
   CellTags->"PowerSimplify"],
  Cell[1597457, 62898, 222, 7, 26, "Output",
   CellTags->"PowerSimplify"],
  Cell[1597716, 62910, 190, 6, 26, "Input",
   CellTags->"PowerSimplify"],
  Cell[1597909, 62918, 222, 7, 26, "Output",
   CellTags->"PowerSimplify"]},
 "Power2"->{
  Cell[1598158, 62929, 49, 1, 41, "ObjectName",
   CellTags->"Power2"],
  Cell[1598232, 62934, 57, 1, 36, "Subsubsection",
   CellTags->"Power2"],
  Cell[1598292, 62937, 213, 5, 43, "Text",
   CellTags->"Power2"],
  Cell[1598508, 62944, 186, 9, 25, "Text",
   CellTags->"Power2"],
  Cell[1598731, 62958, 54, 1, 34, "Subsubsection",
   CellTags->"Power2"],
  Cell[1598810, 62963, 147, 5, 26, "Input",
   CellTags->"Power2"],
  Cell[1598960, 62970, 213, 7, 28, "Output",
   CellTags->"Power2"],
  Cell[1599210, 62982, 148, 5, 26, "Input",
   CellTags->"Power2"],
  Cell[1599361, 62989, 298, 11, 28, "Output",
   CellTags->"Power2"]},
 "PostFortranFile"->{
  Cell[1599686, 63004, 67, 1, 41, "ObjectName",
   CellTags->"PostFortranFile"],
  Cell[1599778, 63009, 66, 1, 36, "Subsubsection",
   CellTags->"PostFortranFile"],
  Cell[1599847, 63012, 229, 5, 43, "Text",
   CellTags->"PostFortranFile"],
  Cell[1600079, 63019, 308, 14, 25, "Text",
   CellTags->"PostFortranFile"]},
 "Prefactor"->{
  Cell[1600402, 63036, 55, 1, 41, "ObjectName",
   CellTags->"Prefactor"],
  Cell[1600482, 63041, 60, 1, 36, "Subsubsection",
   CellTags->"Prefactor"],
  Cell[1600545, 63044, 290, 6, 43, "Text",
   CellTags->"Prefactor"],
  Cell[1600838, 63052, 293, 14, 25, "Text",
   CellTags->"Prefactor"]},
 "PreFortranFile"->{
  Cell[1601146, 63069, 65, 1, 41, "ObjectName",
   CellTags->"PreFortranFile"],
  Cell[1601236, 63074, 65, 1, 36, "Subsubsection",
   CellTags->"PreFortranFile"],
  Cell[1601304, 63077, 232, 5, 43, "Text",
   CellTags->"PreFortranFile"],
  Cell[1601539, 63084, 310, 14, 25, "Text",
   CellTags->"PreFortranFile"]},
 "PropagatorDenominator"->{
  Cell[1601864, 63101, 79, 1, 41, "ObjectName",
   CellTags->"PropagatorDenominator"],
  Cell[1601968, 63106, 72, 1, 36, "Subsubsection",
   CellTags->"PropagatorDenominator"],
  Cell[1602043, 63109, 549, 16, 61, "Text",
   CellTags->"PropagatorDenominator"],
  Cell[1602595, 63127, 493, 19, 25, "Text",
   CellTags->"PropagatorDenominator"],
  Cell[1603125, 63151, 69, 1, 34, "Subsubsection",
   CellTags->"PropagatorDenominator"],
  Cell[1603219, 63156, 178, 5, 26, "Input",
   CellTags->"PropagatorDenominator"],
  Cell[1603400, 63163, 257, 8, 50, "Output",
   CellTags->"PropagatorDenominator"],
  Cell[1603694, 63176, 136, 3, 26, "Input",
   CellTags->"PropagatorDenominator"],
  Cell[1603833, 63181, 206, 6, 50, "Output",
   CellTags->"PropagatorDenominator"],
  Cell[1604076, 63192, 189, 5, 26, "Input",
   CellTags->"PropagatorDenominator"],
  Cell[1604268, 63199, 257, 8, 50, "Output",
   CellTags->"PropagatorDenominator"],
  Cell[1604562, 63212, 158, 4, 26, "Input",
   CellTags->"PropagatorDenominator"],
  Cell[1604723, 63218, 277, 7, 38, "Output",
   CellTags->"PropagatorDenominator"],
  Cell[1605037, 63230, 194, 5, 26, "Input",
   CellTags->"PropagatorDenominator"],
  Cell[1605234, 63237, 277, 7, 38, "Output",
   CellTags->"PropagatorDenominator"],
  Cell[1605548, 63249, 145, 3, 26, "Input",
   CellTags->"PropagatorDenominator"],
  Cell[1605696, 63254, 297, 10, 50, "Output",
   CellTags->"PropagatorDenominator"],
  Cell[1606030, 63269, 127, 3, 26, "Input",
   CellTags->"PropagatorDenominator"],
  Cell[1606160, 63274, 367, 11, 58, "Output",
   CellTags->"PropagatorDenominator"],
  Cell[1606542, 63288, 121, 3, 26, "Input",
   CellTags->"PropagatorDenominator"]},
 "PropagatorDenominatorExplicit"->{
  Cell[1606678, 63294, 95, 1, 39, "ObjectName",
   CellTags->"PropagatorDenominatorExplicit"],
  Cell[1606798, 63299, 80, 1, 34, "Subsubsection",
   CellTags->"PropagatorDenominatorExplicit"],
  Cell[1606881, 63302, 243, 5, 41, "Text",
   CellTags->"PropagatorDenominatorExplicit"],
  Cell[1607127, 63309, 379, 14, 24, "Text",
   CellTags->"PropagatorDenominatorExplicit"],
  Cell[1607543, 63328, 77, 1, 34, "Subsubsection",
   CellTags->"PropagatorDenominatorExplicit"],
  Cell[1607645, 63333, 308, 11, 26, "Input",
   CellTags->"PropagatorDenominatorExplicit"],
  Cell[1607956, 63346, 399, 13, 51, "Output",
   CellTags->"PropagatorDenominatorExplicit"],
  Cell[1608392, 63364, 181, 5, 26, "Input",
   CellTags->"PropagatorDenominatorExplicit"],
  Cell[1608576, 63371, 807, 30, 51, "Output",
   CellTags->"PropagatorDenominatorExplicit"],
  Cell[1609420, 63406, 131, 3, 26, "Input",
   CellTags->"PropagatorDenominatorExplicit"],
  Cell[1609554, 63411, 589, 19, 61, "Output",
   CellTags->"PropagatorDenominatorExplicit"]},
 "QGV"->{
  Cell[1610170, 63434, 43, 1, 39, "ObjectName",
   CellTags->"QGV"],
  Cell[1610238, 63439, 54, 1, 34, "Subsubsection",
   CellTags->"QGV"],
  Cell[1610295, 63442, 72, 1, 24, "Text",
   CellTags->"QGV"],
  Cell[1610370, 63445, 198, 9, 24, "Text",
   CellTags->"QGV"]},
 "QO"->{
  Cell[1610583, 63457, 41, 1, 39, "ObjectName",
   CellTags->"QO"],
  Cell[1610649, 63462, 53, 1, 34, "Subsubsection",
   CellTags->"QO"],
  Cell[1610705, 63465, 73, 1, 24, "Text",
   CellTags->"QO"],
  Cell[1610781, 63468, 206, 9, 24, "Text",
   CellTags->"QO"]},
 "QP"->{
  Cell[1611002, 63480, 41, 1, 39, "ObjectName",
   CellTags->"QP"],
  Cell[1611068, 63485, 53, 1, 34, "Subsubsection",
   CellTags->"QP"],
  Cell[1611124, 63488, 169, 4, 24, "Text",
   CellTags->"QP"],
  Cell[1611296, 63494, 194, 9, 24, "Text",
   CellTags->"QP"]},
 "QuantumField"->{
  Cell[1611505, 63506, 61, 1, 39, "ObjectName",
   CellTags->"QuantumField"],
  Cell[1611591, 63511, 63, 1, 34, "Subsubsection",
   CellTags->"QuantumField"],
  Cell[1611657, 63514, 404, 9, 58, "Text",
   CellTags->"QuantumField"],
  Cell[1612064, 63525, 421, 19, 24, "Text",
   CellTags->"QuantumField"],
  Cell[1612522, 63549, 60, 1, 34, "Subsubsection",
   CellTags->"QuantumField"],
  Cell[1612585, 63552, 71, 1, 24, "Text",
   CellTags->"QuantumField"],
  Cell[1612681, 63557, 118, 3, 26, "Input",
   CellTags->"QuantumField"],
  Cell[1612802, 63562, 151, 4, 27, "Output",
   CellTags->"QuantumField"],
  Cell[1612990, 63571, 131, 3, 26, "Input",
   CellTags->"QuantumField"],
  Cell[1613124, 63576, 179, 5, 26, "Output",
   CellTags->"QuantumField"],
  Cell[1613340, 63586, 127, 3, 26, "Input",
   CellTags->"QuantumField"],
  Cell[1613470, 63591, 156, 4, 26, "Output",
   CellTags->"QuantumField"],
  Cell[1613641, 63598, 80, 1, 24, "Text",
   CellTags->"QuantumField"],
  Cell[1613746, 63603, 169, 5, 26, "Input",
   CellTags->"QuantumField"],
  Cell[1613918, 63610, 179, 5, 29, "Output",
   CellTags->"QuantumField"],
  Cell[1614112, 63618, 94, 1, 24, "Text",
   CellTags->"QuantumField"],
  Cell[1614231, 63623, 212, 6, 26, "Input",
   CellTags->"QuantumField"],
  Cell[1614446, 63631, 361, 15, 30, "Output",
   CellTags->"QuantumField"],
  Cell[1614844, 63651, 114, 3, 26, "Input",
   CellTags->"QuantumField"],
  Cell[1614961, 63656, 297, 7, 38, "Output",
   CellTags->"QuantumField"],
  Cell[1615273, 63666, 358, 13, 28, "Text",
   CellTags->"QuantumField"],
  Cell[1615656, 63683, 206, 6, 26, "Input",
   CellTags->"QuantumField"],
  Cell[1615865, 63691, 301, 11, 28, "Output",
   CellTags->"QuantumField"],
  Cell[1616181, 63705, 185, 4, 24, "Text",
   CellTags->"QuantumField"],
  Cell[1616391, 63713, 244, 7, 26, "Input",
   CellTags->"QuantumField"],
  Cell[1616638, 63722, 287, 7, 38, "Output",
   CellTags->"QuantumField"],
  Cell[1616940, 63732, 88, 1, 24, "Text",
   CellTags->"QuantumField"],
  Cell[1617053, 63737, 219, 6, 26, "Input",
   CellTags->"QuantumField"],
  Cell[1617275, 63745, 590, 14, 87, "Output",
   CellTags->"QuantumField"],
  Cell[1617902, 63764, 184, 5, 26, "Input",
   CellTags->"QuantumField"],
  Cell[1618089, 63771, 552, 13, 87, "Output",
   CellTags->"QuantumField"],
  Cell[1618678, 63789, 259, 7, 26, "Input",
   CellTags->"QuantumField"],
  Cell[1618940, 63798, 622, 14, 87, "Output",
   CellTags->"QuantumField"],
  Cell[1619599, 63817, 291, 9, 26, "Input",
   CellTags->"QuantumField"],
  Cell[1619893, 63828, 654, 16, 87, "Output",
   CellTags->"QuantumField"],
  Cell[1620584, 63849, 232, 6, 26, "Input",
   CellTags->"QuantumField"],
  Cell[1620819, 63857, 156, 4, 26, "Output",
   CellTags->"QuantumField"]},
 "QuarkField"->{
  Cell[1621002, 63865, 57, 1, 57, "ObjectName",
   CellTags->"QuarkField"],
  Cell[1621084, 63870, 61, 1, 36, "Subsubsection",
   CellTags->"QuarkField"],
  Cell[1621148, 63873, 189, 4, 43, "Text",
   CellTags->"QuarkField"],
  Cell[1621340, 63879, 321, 14, 25, "Text",
   CellTags->"QuarkField"],
  Cell[1621698, 63898, 58, 1, 34, "Subsubsection",
   CellTags->"QuarkField"],
  Cell[1621781, 63903, 87, 2, 26, "Input",
   CellTags->"QuarkField"],
  Cell[1621871, 63907, 154, 4, 26, "Output",
   CellTags->"QuarkField"]},
 "QuarkGluonVertex"->{
  Cell[1622052, 63915, 69, 1, 41, "ObjectName",
   CellTags->"QuarkGluonVertex"],
  Cell[1622146, 63920, 67, 1, 36, "Subsubsection",
   CellTags->"QuarkGluonVertex"],
  Cell[1622216, 63923, 260, 9, 24, "Text",
   CellTags->"QuarkGluonVertex"],
  Cell[1622501, 63936, 132, 3, 26, "Input",
   CellTags->"QuarkGluonVertex"],
  Cell[1622636, 63941, 533, 14, 26, "Output",
   CellTags->"QuarkGluonVertex"],
  Cell[1623184, 63958, 196, 9, 25, "Text",
   CellTags->"QuarkGluonVertex"],
  Cell[1623417, 63972, 64, 1, 34, "Subsubsection",
   CellTags->"QuarkGluonVertex"],
  Cell[1623506, 63977, 153, 4, 26, "Input",
   CellTags->"QuarkGluonVertex"],
  Cell[1623662, 63983, 191, 5, 28, "Output",
   CellTags->"QuarkGluonVertex"],
  Cell[1623890, 63993, 208, 5, 26, "Input",
   CellTags->"QuarkGluonVertex"],
  Cell[1624101, 64000, 191, 5, 28, "Output",
   CellTags->"QuarkGluonVertex"],
  Cell[1624329, 64010, 208, 5, 26, "Input",
   CellTags->"QuarkGluonVertex"],
  Cell[1624540, 64017, 191, 5, 28, "Output",
   CellTags->"QuarkGluonVertex"],
  Cell[1624768, 64027, 208, 5, 26, "Input",
   CellTags->"QuarkGluonVertex"],
  Cell[1624979, 64034, 191, 5, 28, "Output",
   CellTags->"QuarkGluonVertex"],
  Cell[1625207, 64044, 315, 9, 26, "Input",
   CellTags->"QuarkGluonVertex"],
  Cell[1625525, 64055, 408, 12, 28, "Output",
   CellTags->"QuarkGluonVertex"],
  Cell[1625970, 64072, 316, 9, 26, "Input",
   CellTags->"QuarkGluonVertex"],
  Cell[1626289, 64083, 409, 12, 28, "Output",
   CellTags->"QuarkGluonVertex"]},
 "QuarkMass"->{
  Cell[1626725, 64099, 55, 1, 41, "ObjectName",
   CellTags->"QuarkMass"],
  Cell[1626805, 64104, 60, 1, 36, "Subsubsection",
   CellTags->"QuarkMass"],
  Cell[1626868, 64107, 76, 1, 25, "Text",
   CellTags->"QuarkMass"],
  Cell[1626947, 64110, 183, 9, 25, "Text",
   CellTags->"QuarkMass"]},
 "QuarkPropagator"->{
  Cell[1627145, 64122, 67, 1, 41, "ObjectName",
   CellTags->"QuarkPropagator"],
  Cell[1627237, 64127, 66, 1, 36, "Subsubsection",
   CellTags->"QuarkPropagator"],
  Cell[1627306, 64130, 174, 4, 24, "Text",
   CellTags->"QuarkPropagator"],
  Cell[1627505, 64138, 130, 3, 26, "Input",
   CellTags->"QuarkPropagator"],
  Cell[1627638, 64143, 575, 15, 26, "Output",
   CellTags->"QuarkPropagator"],
  Cell[1628228, 64161, 341, 14, 25, "Text",
   CellTags->"QuarkPropagator"],
  Cell[1628606, 64180, 63, 1, 34, "Subsubsection",
   CellTags->"QuarkPropagator"],
  Cell[1628694, 64185, 124, 3, 26, "Input",
   CellTags->"QuarkPropagator"],
  Cell[1628821, 64190, 219, 7, 30, "Output",
   CellTags->"QuarkPropagator"],
  Cell[1629077, 64202, 171, 5, 26, "Input",
   CellTags->"QuarkPropagator"],
  Cell[1629251, 64209, 219, 7, 30, "Output",
   CellTags->"QuarkPropagator"],
  Cell[1629507, 64221, 157, 4, 26, "Input",
   CellTags->"QuarkPropagator"],
  Cell[1629667, 64227, 219, 7, 30, "Output",
   CellTags->"QuarkPropagator"]},
 "ReduceGamma"->{
  Cell[1629913, 64238, 59, 1, 41, "ObjectName",
   CellTags->"ReduceGamma"],
  Cell[1629997, 64243, 62, 1, 36, "Subsubsection",
   CellTags->"ReduceGamma"],
  Cell[1630062, 64246, 194, 4, 43, "Text",
   CellTags->"ReduceGamma"],
  Cell[1630259, 64252, 179, 9, 25, "Text",
   CellTags->"ReduceGamma"]},
 "ReduceToScalars"->{
  Cell[1630453, 64264, 67, 1, 41, "ObjectName",
   CellTags->"ReduceToScalars"],
  Cell[1630545, 64269, 66, 1, 36, "Subsubsection",
   CellTags->"ReduceToScalars"],
  Cell[1630614, 64272, 210, 5, 43, "Text",
   CellTags->"ReduceToScalars"],
  Cell[1630827, 64279, 299, 14, 25, "Text",
   CellTags->"ReduceToScalars"]},
 "Rename"->{
  Cell[1631141, 64296, 49, 1, 41, "ObjectName",
   CellTags->"Rename"],
  Cell[1631215, 64301, 57, 1, 36, "Subsubsection",
   CellTags->"Rename"],
  Cell[1631275, 64304, 144, 4, 25, "Text",
   CellTags->"Rename"],
  Cell[1631422, 64310, 177, 9, 25, "Text",
   CellTags->"Rename"]},
 "RHI"->{
  Cell[1631614, 64322, 61, 1, 41, "ObjectName",
   CellTags->"RHI"],
  Cell[1631700, 64327, 54, 1, 36, "Subsubsection",
   CellTags->"RHI"],
  Cell[1631757, 64330, 611, 12, 147, "Text",
   CellTags->"RHI"],
  Cell[1632371, 64344, 168, 9, 25, "Text",
   CellTags->"RHI"],
  Cell[1632576, 64358, 51, 1, 34, "Subsubsection",
   CellTags->"RHI"],
  Cell[1632630, 64361, 70, 2, 26, "Input",
   CellTags->"RHI"]},
 "RHI2FC"->{
  Cell[1632715, 64366, 67, 1, 41, "ObjectName",
   CellTags->"RHI2FC"],
  Cell[1632807, 64371, 57, 1, 36, "Subsubsection",
   CellTags->"RHI2FC"],
  Cell[1632867, 64374, 614, 12, 147, "Text",
   CellTags->"RHI2FC"],
  Cell[1633484, 64388, 171, 9, 25, "Text",
   CellTags->"RHI2FC"],
  Cell[1633692, 64402, 54, 1, 34, "Subsubsection",
   CellTags->"RHI2FC"],
  Cell[1633749, 64405, 73, 2, 26, "Input",
   CellTags->"RHI2FC"]},
 "RHM"->{
  Cell[1633837, 64410, 61, 1, 41, "ObjectName",
   CellTags->"RHM"],
  Cell[1633923, 64415, 54, 1, 36, "Subsubsection",
   CellTags->"RHM"],
  Cell[1633980, 64418, 77, 1, 25, "Text",
   CellTags->"RHM"],
  Cell[1634060, 64421, 159, 9, 25, "Text",
   CellTags->"RHM"],
  Cell[1634256, 64435, 51, 1, 34, "Subsubsection",
   CellTags->"RHM"],
  Cell[1634310, 64438, 70, 2, 26, "Input",
   CellTags->"RHM"]},
 "RHO"->{
  Cell[1634395, 64443, 61, 1, 41, "ObjectName",
   CellTags->"RHO"],
  Cell[1634481, 64448, 54, 1, 36, "Subsubsection",
   CellTags->"RHO"],
  Cell[1634538, 64451, 267, 5, 43, "Text",
   CellTags->"RHO"],
  Cell[1634808, 64458, 159, 9, 25, "Text",
   CellTags->"RHO"],
  Cell[1635004, 64472, 51, 1, 34, "Subsubsection",
   CellTags->"RHO"],
  Cell[1635058, 64475, 70, 2, 26, "Input",
   CellTags->"RHO"]},
 "RHP"->{
  Cell[1635143, 64480, 45, 1, 41, "ObjectName",
   CellTags->"RHP"],
  Cell[1635213, 64485, 54, 1, 36, "Subsubsection",
   CellTags->"RHP"],
  Cell[1635270, 64488, 272, 5, 43, "Text",
   CellTags->"RHP"],
  Cell[1635545, 64495, 159, 9, 25, "Text",
   CellTags->"RHP"],
  Cell[1635741, 64509, 51, 1, 34, "Subsubsection",
   CellTags->"RHP"],
  Cell[1641231, 64743, 70, 2, 26, "Input",
   CellTags->"RHP"]},
 "RTL"->{
  Cell[1641316, 64748, 61, 1, 41, "ObjectName",
   CellTags->"RTL"],
  Cell[1641402, 64753, 54, 1, 36, "Subsubsection",
   CellTags->"RTL"],
  Cell[1641459, 64756, 188, 4, 43, "Text",
   CellTags->"RTL"],
  Cell[1641650, 64762, 266, 14, 25, "Text",
   CellTags->"RTL"],
  Cell[1641953, 64781, 51, 1, 34, "Subsubsection",
   CellTags->"RTL"],
  Cell[1642007, 64784, 70, 2, 26, "Input",
   CellTags->"RTL"]},
 "RightPartialD"->{
  Cell[1642092, 64789, 63, 1, 41, "ObjectName",
   CellTags->"RightPartialD"],
  Cell[1642180, 64794, 64, 1, 36, "Subsubsection",
   CellTags->"RightPartialD"],
  Cell[1642247, 64797, 256, 10, 25, "Text",
   CellTags->"RightPartialD"],
  Cell[1642506, 64809, 434, 19, 25, "Text",
   CellTags->"RightPartialD"],
  Cell[1642977, 64833, 61, 1, 34, "Subsubsection",
   CellTags->"RightPartialD"],
  Cell[1643063, 64838, 124, 3, 26, "Input",
   CellTags->"RightPartialD"],
  Cell[1643190, 64843, 294, 10, 33, "Output",
   CellTags->"RightPartialD"],
  Cell[1643521, 64858, 255, 7, 26, "Input",
   CellTags->"RightPartialD"],
  Cell[1643779, 64867, 350, 12, 33, "Output",
   CellTags->"RightPartialD"],
  Cell[1644166, 64884, 121, 3, 26, "Input",
   CellTags->"RightPartialD"],
  Cell[1644290, 64889, 623, 15, 87, "Output",
   CellTags->"RightPartialD"],
  Cell[1644950, 64909, 115, 3, 26, "Input",
   CellTags->"RightPartialD"],
  Cell[1645068, 64914, 612, 12, 115, "Output",
   CellTags->"RightPartialD"],
  Cell[1645717, 64931, 162, 5, 26, "Input",
   CellTags->"RightPartialD"],
  Cell[1645882, 64938, 220, 5, 38, "Output",
   CellTags->"RightPartialD"]},
 "RussianTrick"->{
  Cell[1646129, 64947, 61, 1, 41, "ObjectName",
   CellTags->"RussianTrick"],
  Cell[1646215, 64952, 63, 1, 36, "Subsubsection",
   CellTags->"RussianTrick"],
  Cell[1646281, 64955, 318, 6, 61, "Text",
   CellTags->"RussianTrick"],
  Cell[1646602, 64963, 90, 1, 25, "Text",
   CellTags->"RussianTrick"],
  Cell[1646695, 64966, 326, 14, 25, "Text",
   CellTags->"RussianTrick"],
  Cell[1647058, 64985, 60, 1, 34, "Subsubsection",
   CellTags->"RussianTrick"],
  Cell[1647143, 64990, 374, 12, 26, "Input",
   CellTags->"RussianTrick"],
  Cell[1647520, 65004, 545, 22, 29, "Output",
   CellTags->"RussianTrick"],
  Cell[1648102, 65031, 108, 3, 26, "Input",
   CellTags->"RussianTrick"],
  Cell[1648213, 65036, 703, 24, 50, "Output",
   CellTags->"RussianTrick"],
  Cell[1648953, 65065, 172, 5, 26, "Input",
   CellTags->"RussianTrick"],
  Cell[1649128, 65072, 3510, 122, 146, "Output",
   CellTags->"RussianTrick"],
  Cell[1652675, 65199, 112, 3, 26, "Input",
   CellTags->"RussianTrick"],
  Cell[1652790, 65204, 2716, 112, 29, "Output",
   CellTags->"RussianTrick"],
  Cell[1655543, 65321, 135, 4, 26, "Input",
   CellTags->"RussianTrick"],
  Cell[1655681, 65327, 2991, 113, 48, "Output",
   CellTags->"RussianTrick"],
  Cell[1658687, 65443, 111, 3, 26, "Input",
   CellTags->"RussianTrick"]},
 "ScalarGluonVertex"->{
  Cell[1658813, 65449, 71, 1, 41, "ObjectName",
   CellTags->"ScalarGluonVertex"],
  Cell[1658909, 65454, 68, 1, 36, "Subsubsection",
   CellTags->"ScalarGluonVertex"],
  Cell[1658980, 65457, 430, 8, 91, "Text",
   CellTags->"ScalarGluonVertex"],
  Cell[1659435, 65469, 134, 3, 26, "Input",
   CellTags->"ScalarGluonVertex"],
  Cell[1659572, 65474, 315, 9, 26, "Output",
   CellTags->"ScalarGluonVertex"],
  Cell[1659936, 65489, 65, 1, 34, "Subsubsection",
   CellTags->"ScalarGluonVertex"],
  Cell[1660026, 65494, 274, 8, 26, "Input",
   CellTags->"ScalarGluonVertex"],
  Cell[1660303, 65504, 521, 20, 28, "Output",
   CellTags->"ScalarGluonVertex"]},
 "ScalarProduct"->{
  Cell[1660851, 65528, 63, 1, 41, "ObjectName",
   CellTags->"ScalarProduct"],
  Cell[1660939, 65533, 64, 1, 36, "Subsubsection",
   CellTags->"ScalarProduct"],
  Cell[1661006, 65536, 836, 21, 79, "Text",
   CellTags->"ScalarProduct"],
  Cell[1661845, 65559, 880, 38, 25, "Text",
   CellTags->"ScalarProduct"],
  Cell[1662762, 65602, 61, 1, 34, "Subsubsection",
   CellTags->"ScalarProduct"],
  Cell[1662848, 65607, 143, 4, 26, "Input",
   CellTags->"ScalarProduct"],
  Cell[1662994, 65613, 254, 9, 28, "Output",
   CellTags->"ScalarProduct"],
  Cell[1663285, 65627, 186, 6, 26, "Input",
   CellTags->"ScalarProduct"],
  Cell[1663474, 65635, 343, 12, 28, "Output",
   CellTags->"ScalarProduct"],
  Cell[1663854, 65652, 143, 4, 26, "Input",
   CellTags->"ScalarProduct"],
  Cell[1664000, 65658, 210, 7, 28, "Output",
   CellTags->"ScalarProduct"],
  Cell[1664247, 65670, 120, 3, 26, "Input",
   CellTags->"ScalarProduct"],
  Cell[1664370, 65675, 210, 7, 28, "Output",
   CellTags->"ScalarProduct"],
  Cell[1664617, 65687, 179, 5, 26, "Input",
   CellTags->"ScalarProduct"],
  Cell[1664799, 65694, 265, 7, 38, "Output",
   CellTags->"ScalarProduct"],
  Cell[1665101, 65706, 231, 7, 26, "Input",
   CellTags->"ScalarProduct"],
  Cell[1665335, 65715, 313, 9, 38, "Output",
   CellTags->"ScalarProduct"],
  Cell[1665685, 65729, 248, 8, 26, "Input",
   CellTags->"ScalarProduct"],
  Cell[1665936, 65739, 173, 5, 41, "Output",
   CellTags->"ScalarProduct"],
  Cell[1666146, 65749, 295, 9, 26, "Input",
   CellTags->"ScalarProduct"],
  Cell[1666444, 65760, 615, 24, 41, "Output",
   CellTags->"ScalarProduct"],
  Cell[1667096, 65789, 280, 9, 26, "Input",
   CellTags->"ScalarProduct"],
  Cell[1667379, 65800, 613, 24, 41, "Output",
   CellTags->"ScalarProduct"],
  Cell[1668007, 65827, 99, 2, 26, "Input",
   CellTags->"ScalarProduct"]},
 "Info3500973967-4982097"->{
  Cell[1669531, 65887, 221, 4, 39, "Print",
   CellTags->{"Info3500973967-4982097", "ScalarProductCancel"}]},
 "ScaleMu"->{
  Cell[1679969, 66298, 51, 1, 41, "ObjectName",
   CellTags->"ScaleMu"],
  Cell[1680045, 66303, 58, 1, 36, "Subsubsection",
   CellTags->"ScaleMu"],
  Cell[1680106, 66306, 128, 4, 24, "Text",
   CellTags->"ScaleMu"],
  Cell[1680259, 66314, 81, 2, 26, "Input",
   CellTags->"ScaleMu"],
  Cell[1680343, 66318, 149, 4, 26, "Output",
   CellTags->"ScaleMu"]},
 "Schouten"->{
  Cell[1680519, 66326, 53, 1, 41, "ObjectName",
   CellTags->"Schouten"],
  Cell[1680597, 66331, 59, 1, 36, "Subsubsection",
   CellTags->"Schouten"],
  Cell[1680659, 66334, 460, 8, 87, "Text",
   CellTags->"Schouten"],
  Cell[1681122, 66344, 289, 13, 25, "Text",
   CellTags->"Schouten"],
  Cell[1681448, 66362, 56, 1, 34, "Subsubsection",
   CellTags->"Schouten"],
  Cell[1681529, 66367, 1938, 54, 184, "Input",
   CellTags->"Schouten"],
  Cell[1683470, 66423, 15738, 629, 324, "Output",
   CellTags->"Schouten"],
  Cell[1699245, 67057, 106, 3, 26, "Input",
   CellTags->"Schouten"],
  Cell[1699354, 67062, 147, 4, 26, "Output",
   CellTags->"Schouten"]},
 "SD"->{
  Cell[1699615, 67074, 41, 1, 41, "ObjectName",
   CellTags->"SD"],
  Cell[1699681, 67079, 53, 1, 36, "Subsubsection",
   CellTags->"SD"],
  Cell[1699737, 67082, 265, 7, 43, "Text",
   CellTags->"SD"],
  Cell[1700005, 67091, 173, 9, 25, "Text",
   CellTags->"SD"],
  Cell[1700215, 67105, 50, 1, 34, "Subsubsection",
   CellTags->"SD"],
  Cell[1700290, 67110, 121, 4, 26, "Input",
   CellTags->"SD"],
  Cell[1700414, 67116, 191, 6, 28, "Output",
   CellTags->"SD"],
  Cell[1700642, 67127, 130, 4, 26, "Input",
   CellTags->"SD"],
  Cell[1700775, 67133, 256, 7, 38, "Output",
   CellTags->"SD"],
  Cell[1701068, 67145, 130, 4, 26, "Input",
   CellTags->"SD"],
  Cell[1701201, 67151, 178, 5, 38, "Output",
   CellTags->"SD"]},
 "SelectFree"->{
  Cell[1701406, 67160, 57, 1, 41, "ObjectName",
   CellTags->"SelectFree"],
  Cell[1701488, 67165, 61, 1, 36, "Subsubsection",
   CellTags->"SelectFree"],
  Cell[1701552, 67168, 255, 5, 43, "Text",
   CellTags->"SelectFree"],
  Cell[1701810, 67175, 77, 1, 25, "Text",
   CellTags->"SelectFree"],
  Cell[1701890, 67178, 300, 14, 25, "Text",
   CellTags->"SelectFree"],
  Cell[1702227, 67197, 58, 1, 34, "Subsubsection",
   CellTags->"SelectFree"],
  Cell[1702310, 67202, 211, 6, 26, "Input",
   CellTags->"SelectFree"],
  Cell[1702524, 67210, 172, 5, 26, "Output",
   CellTags->"SelectFree"],
  Cell[1702733, 67220, 166, 5, 26, "Input",
   CellTags->"SelectFree"],
  Cell[1702902, 67227, 151, 4, 27, "Output",
   CellTags->"SelectFree"],
  Cell[1703090, 67236, 275, 8, 26, "Input",
   CellTags->"SelectFree"],
  Cell[1703368, 67246, 171, 5, 26, "Output",
   CellTags->"SelectFree"],
  Cell[1703576, 67256, 137, 4, 26, "Input",
   CellTags->"SelectFree"],
  Cell[1703716, 67262, 149, 4, 27, "Output",
   CellTags->"SelectFree"],
  Cell[1703902, 67271, 137, 4, 26, "Input",
   CellTags->"SelectFree"],
  Cell[1704042, 67277, 148, 4, 26, "Output",
   CellTags->"SelectFree"],
  Cell[1704227, 67286, 137, 4, 26, "Input",
   CellTags->"SelectFree"],
  Cell[1704367, 67292, 151, 4, 26, "Output",
   CellTags->"SelectFree"],
  Cell[1704555, 67301, 166, 5, 26, "Input",
   CellTags->"SelectFree"],
  Cell[1704724, 67308, 151, 4, 26, "Output",
   CellTags->"SelectFree"]},
 "SelectGraphs"->{
  Cell[1704902, 67316, 61, 1, 41, "ObjectName",
   CellTags->"SelectGraphs"],
  Cell[1704988, 67321, 63, 1, 36, "Subsubsection",
   CellTags->"SelectGraphs"],
  Cell[1705054, 67324, 486, 8, 79, "Text",
   CellTags->"SelectGraphs"],
  Cell[1705543, 67334, 189, 9, 25, "Text",
   CellTags->"SelectGraphs"]},
 "SelectNotFree"->{
  Cell[1705747, 67346, 63, 1, 41, "ObjectName",
   CellTags->"SelectNotFree"],
  Cell[1705835, 67351, 64, 1, 36, "Subsubsection",
   CellTags->"SelectNotFree"],
  Cell[1705902, 67354, 143, 4, 25, "Text",
   CellTags->"SelectNotFree"],
  Cell[1706048, 67360, 270, 5, 43, "Text",
   CellTags->"SelectNotFree"],
  Cell[1706321, 67367, 83, 1, 25, "Text",
   CellTags->"SelectNotFree"],
  Cell[1706407, 67370, 294, 14, 25, "Text",
   CellTags->"SelectNotFree"],
  Cell[1706738, 67389, 61, 1, 34, "Subsubsection",
   CellTags->"SelectNotFree"],
  Cell[1706824, 67394, 207, 6, 26, "Input",
   CellTags->"SelectNotFree"],
  Cell[1707034, 67402, 206, 6, 26, "Output",
   CellTags->"SelectNotFree"],
  Cell[1707277, 67413, 276, 8, 26, "Input",
   CellTags->"SelectNotFree"],
  Cell[1707556, 67423, 214, 6, 26, "Output",
   CellTags->"SelectNotFree"],
  Cell[1707807, 67434, 143, 4, 26, "Input",
   CellTags->"SelectNotFree"],
  Cell[1707953, 67440, 154, 4, 26, "Output",
   CellTags->"SelectNotFree"],
  Cell[1708144, 67449, 167, 5, 26, "Input",
   CellTags->"SelectNotFree"],
  Cell[1708314, 67456, 154, 4, 26, "Output",
   CellTags->"SelectNotFree"],
  Cell[1708505, 67465, 143, 4, 26, "Input",
   CellTags->"SelectNotFree"],
  Cell[1708651, 67471, 152, 4, 27, "Output",
   CellTags->"SelectNotFree"],
  Cell[1708840, 67480, 143, 4, 26, "Input",
   CellTags->"SelectNotFree"],
  Cell[1708986, 67486, 152, 4, 26, "Output",
   CellTags->"SelectNotFree"],
  Cell[1709175, 67495, 172, 5, 26, "Input",
   CellTags->"SelectNotFree"],
  Cell[1709350, 67502, 180, 5, 26, "Output",
   CellTags->"SelectNotFree"]},
 "SelectSplit"->{
  Cell[1709557, 67511, 59, 1, 41, "ObjectName",
   CellTags->"SelectSplit"],
  Cell[1709641, 67516, 62, 1, 36, "Subsubsection",
   CellTags->"SelectSplit"],
  Cell[1709706, 67519, 249, 5, 43, "Text",
   CellTags->"SelectSplit"],
  Cell[1709992, 67529, 59, 1, 34, "Subsubsection",
   CellTags->"SelectSplit"],
  Cell[1710076, 67534, 804, 26, 53, "Input",
   CellTags->"SelectSplit"],
  Cell[1710883, 67562, 574, 20, 29, "Output",
   CellTags->"SelectSplit"],
  Cell[1711494, 67587, 652, 21, 30, "Input",
   CellTags->"SelectSplit"],
  Cell[1712149, 67610, 537, 17, 29, "Output",
   CellTags->"SelectSplit"]},
 "Select1"->{
  Cell[1712713, 67631, 51, 1, 41, "ObjectName",
   CellTags->"Select1"],
  Cell[1712789, 67636, 58, 1, 36, "Subsubsection",
   CellTags->"Select1"],
  Cell[1712850, 67639, 243, 5, 43, "Text",
   CellTags->"Select1"],
  Cell[1713096, 67646, 175, 9, 25, "Text",
   CellTags->"Select1"]},
 "Select2"->{
  Cell[1713286, 67658, 51, 1, 41, "ObjectName",
   CellTags->"Select2"],
  Cell[1713362, 67663, 58, 1, 36, "Subsubsection",
   CellTags->"Select2"],
  Cell[1713423, 67666, 245, 5, 43, "Text",
   CellTags->"Select2"],
  Cell[1713671, 67673, 175, 9, 25, "Text",
   CellTags->"Select2"]},
 "Series2"->{
  Cell[1713861, 67685, 51, 1, 41, "ObjectName",
   CellTags->"Series2"],
  Cell[1713937, 67690, 58, 1, 36, "Subsubsection",
   CellTags->"Series2"],
  Cell[1713998, 67693, 383, 9, 41, "Text",
   CellTags->"Series2"],
  Cell[1714406, 67706, 114, 3, 26, "Input",
   CellTags->"Series2"],
  Cell[1714523, 67711, 570, 16, 28, "Output",
   CellTags->"Series2"],
  Cell[1715142, 67733, 55, 1, 34, "Subsubsection",
   CellTags->"Series2"],
  Cell[1715222, 67738, 339, 12, 26, "Input",
   CellTags->"Series2"],
  Cell[1715564, 67752, 399, 13, 44, "Output",
   CellTags->"Series2"],
  Cell[1716000, 67770, 174, 5, 26, "Input",
   CellTags->"Series2"],
  Cell[1716177, 67777, 283, 9, 46, "Output",
   CellTags->"Series2"],
  Cell[1716497, 67791, 222, 7, 26, "Input",
   CellTags->"Series2"],
  Cell[1716722, 67800, 848, 26, 46, "Output",
   CellTags->"Series2"],
  Cell[1717607, 67831, 174, 5, 26, "Input",
   CellTags->"Series2"],
  Cell[1717784, 67838, 427, 15, 48, "Output",
   CellTags->"Series2"],
  Cell[1718248, 67858, 290, 8, 26, "Input",
   CellTags->"Series2"],
  Cell[1718541, 67868, 1036, 33, 46, "Output",
   CellTags->"Series2"],
  Cell[1719804, 67914, 460, 15, 26, "Input",
   CellTags->"Series2"],
  Cell[1720267, 67931, 1042, 34, 46, "Output",
   CellTags->"Series2"],
  Cell[1721324, 67968, 113, 3, 24, "Text",
   CellTags->"Series2"],
  Cell[1721462, 67975, 484, 15, 46, "Input",
   CellTags->"Series2"],
  Cell[1721949, 67992, 690, 25, 46, "Output",
   CellTags->"Series2"],
  Cell[1722676, 68022, 578, 17, 46, "Input",
   CellTags->"Series2"],
  Cell[1723257, 68041, 1030, 36, 44, "Output",
   CellTags->"Series2"],
  Cell[1724324, 68082, 242, 6, 26, "Input",
   CellTags->"Series2"],
  Cell[1724569, 68090, 336, 8, 26, "Output",
   CellTags->"Series2"],
  Cell[1724942, 68103, 147, 4, 26, "Input",
   CellTags->"Series2"],
  Cell[1725092, 68109, 3156, 107, 106, "Output",
   CellTags->"Series2"],
  Cell[1728263, 68219, 351, 11, 41, "Text",
   CellTags->"Series2"]},
 "Series3"->{
  Cell[1728629, 68233, 51, 1, 39, "ObjectName",
   CellTags->"Series3"],
  Cell[1728705, 68238, 58, 1, 34, "Subsubsection",
   CellTags->"Series3"],
  Cell[1728766, 68241, 267, 5, 41, "Text",
   CellTags->"Series3"],
  Cell[1729058, 68250, 114, 3, 26, "Input",
   CellTags->"Series3"],
  Cell[1729175, 68255, 306, 9, 26, "Output",
   CellTags->"Series3"],
  Cell[1729496, 68267, 175, 9, 24, "Text",
   CellTags->"Series3"],
  Cell[1729708, 68281, 55, 1, 34, "Subsubsection",
   CellTags->"Series3"],
  Cell[1729788, 68286, 339, 12, 26, "Input",
   CellTags->"Series3"],
  Cell[1730130, 68300, 349, 12, 44, "Output",
   CellTags->"Series3"],
  Cell[1730516, 68317, 214, 7, 26, "Input",
   CellTags->"Series3"],
  Cell[1730733, 68326, 257, 8, 46, "Output",
   CellTags->"Series3"]},
 "SetMandelstam"->{
  Cell[1731017, 68338, 63, 1, 41, "ObjectName",
   CellTags->"SetMandelstam"],
  Cell[1731105, 68343, 64, 1, 36, "Subsubsection",
   CellTags->"SetMandelstam"],
  Cell[1731172, 68346, 462, 9, 83, "Text",
   CellTags->"SetMandelstam"],
  Cell[1731659, 68359, 126, 3, 26, "Input",
   CellTags->"SetMandelstam"],
  Cell[1731788, 68364, 281, 9, 26, "Output",
   CellTags->"SetMandelstam"],
  Cell[1732084, 68376, 190, 9, 25, "Text",
   CellTags->"SetMandelstam"]},
 "SimplifyDeltaFunction"->{
  Cell[1732289, 68388, 79, 1, 41, "ObjectName",
   CellTags->"SimplifyDeltaFunction"],
  Cell[1732393, 68393, 72, 1, 36, "Subsubsection",
   CellTags->"SimplifyDeltaFunction"],
  Cell[1732468, 68396, 299, 6, 43, "Text",
   CellTags->"SimplifyDeltaFunction"],
  Cell[1732770, 68404, 347, 14, 25, "Text",
   CellTags->"SimplifyDeltaFunction"],
  Cell[1733154, 68423, 69, 1, 34, "Subsubsection",
   CellTags->"SimplifyDeltaFunction"],
  Cell[1733248, 68428, 203, 6, 26, "Input",
   CellTags->"SimplifyDeltaFunction"],
  Cell[1733454, 68436, 310, 10, 26, "Output",
   CellTags->"SimplifyDeltaFunction"],
  Cell[1733801, 68451, 164, 4, 26, "Input",
   CellTags->"SimplifyDeltaFunction"],
  Cell[1733968, 68457, 434, 14, 41, "Output",
   CellTags->"SimplifyDeltaFunction"],
  Cell[1734439, 68476, 203, 6, 26, "Input",
   CellTags->"SimplifyDeltaFunction"],
  Cell[1734645, 68484, 345, 11, 26, "Output",
   CellTags->"SimplifyDeltaFunction"],
  Cell[1735027, 68500, 164, 4, 26, "Input",
   CellTags->"SimplifyDeltaFunction"],
  Cell[1735194, 68506, 854, 28, 41, "Output",
   CellTags->"SimplifyDeltaFunction"],
  Cell[1736085, 68539, 220, 6, 26, "Input",
   CellTags->"SimplifyDeltaFunction"],
  Cell[1736308, 68547, 355, 11, 26, "Output",
   CellTags->"SimplifyDeltaFunction"],
  Cell[1736700, 68563, 164, 4, 26, "Input",
   CellTags->"SimplifyDeltaFunction"],
  Cell[1736867, 68569, 254, 8, 26, "Output",
   CellTags->"SimplifyDeltaFunction"],
  Cell[1737158, 68582, 263, 8, 26, "Input",
   CellTags->"SimplifyDeltaFunction"],
  Cell[1737424, 68592, 372, 12, 26, "Output",
   CellTags->"SimplifyDeltaFunction"],
  Cell[1737833, 68609, 164, 4, 26, "Input",
   CellTags->"SimplifyDeltaFunction"],
  Cell[1738000, 68615, 276, 9, 26, "Output",
   CellTags->"SimplifyDeltaFunction"],
  Cell[1738313, 68629, 298, 9, 26, "Input",
   CellTags->"SimplifyDeltaFunction"],
  Cell[1738614, 68640, 415, 13, 26, "Output",
   CellTags->"SimplifyDeltaFunction"],
  Cell[1739066, 68658, 164, 4, 26, "Input",
   CellTags->"SimplifyDeltaFunction"],
  Cell[1739233, 68664, 162, 4, 26, "Output",
   CellTags->"SimplifyDeltaFunction"],
  Cell[1739432, 68673, 263, 8, 26, "Input",
   CellTags->"SimplifyDeltaFunction"],
  Cell[1739698, 68683, 374, 12, 26, "Output",
   CellTags->"SimplifyDeltaFunction"],
  Cell[1740109, 68700, 164, 4, 26, "Input",
   CellTags->"SimplifyDeltaFunction"],
  Cell[1740276, 68706, 276, 9, 26, "Output",
   CellTags->"SimplifyDeltaFunction"]},
 "SimplifyGTI"->{
  Cell[1740579, 68719, 77, 1, 41, "ObjectName",
   CellTags->"SimplifyGTI"],
  Cell[1740681, 68724, 62, 1, 36, "Subsubsection",
   CellTags->"SimplifyGTI"],
  Cell[1740746, 68727, 71, 1, 25, "Text",
   CellTags->"SimplifyGTI"],
  Cell[1740820, 68730, 167, 9, 25, "Text",
   CellTags->"SimplifyGTI"],
  Cell[1741024, 68744, 59, 1, 34, "Subsubsection",
   CellTags->"SimplifyGTI"],
  Cell[1741086, 68747, 78, 2, 26, "Input",
   CellTags->"SimplifyGTI"]},
 "SimplifyPolyLog"->{
  Cell[1741179, 68752, 67, 1, 41, "ObjectName",
   CellTags->"SimplifyPolyLog"],
  Cell[1741271, 68757, 66, 1, 36, "Subsubsection",
   CellTags->"SimplifyPolyLog"],
  Cell[1741340, 68760, 199, 4, 25, "Text",
   CellTags->"SimplifyPolyLog"],
  Cell[1741542, 68766, 183, 9, 25, "Text",
   CellTags->"SimplifyPolyLog"],
  Cell[1741762, 68780, 63, 1, 34, "Subsubsection",
   CellTags->"SimplifyPolyLog"],
  Cell[1741828, 68783, 224, 6, 26, "Input",
   CellTags->"SimplifyPolyLog"],
  Cell[1742077, 68793, 195, 6, 26, "Input",
   CellTags->"SimplifyPolyLog"],
  Cell[1742275, 68801, 708, 21, 46, "Output",
   CellTags->"SimplifyPolyLog"],
  Cell[1743020, 68827, 170, 5, 26, "Input",
   CellTags->"SimplifyPolyLog"],
  Cell[1743193, 68834, 474, 15, 26, "Output",
   CellTags->"SimplifyPolyLog"],
  Cell[1743704, 68854, 221, 7, 26, "Input",
   CellTags->"SimplifyPolyLog"],
  Cell[1743928, 68863, 675, 21, 29, "Output",
   CellTags->"SimplifyPolyLog"],
  Cell[1744640, 68889, 195, 6, 26, "Input",
   CellTags->"SimplifyPolyLog"],
  Cell[1744838, 68897, 649, 20, 29, "Output",
   CellTags->"SimplifyPolyLog"],
  Cell[1745524, 68922, 271, 9, 26, "Input",
   CellTags->"SimplifyPolyLog"],
  Cell[1745798, 68933, 714, 23, 44, "Output",
   CellTags->"SimplifyPolyLog"],
  Cell[1746549, 68961, 248, 8, 26, "Input",
   CellTags->"SimplifyPolyLog"],
  Cell[1746800, 68971, 689, 22, 44, "Output",
   CellTags->"SimplifyPolyLog"],
  Cell[1747526, 68998, 281, 9, 26, "Input",
   CellTags->"SimplifyPolyLog"],
  Cell[1747810, 69009, 774, 30, 44, "Output",
   CellTags->"SimplifyPolyLog"],
  Cell[1748621, 69044, 216, 7, 26, "Input",
   CellTags->"SimplifyPolyLog"],
  Cell[1748840, 69053, 538, 18, 48, "Output",
   CellTags->"SimplifyPolyLog"],
  Cell[1749415, 69076, 195, 6, 26, "Input",
   CellTags->"SimplifyPolyLog"],
  Cell[1749613, 69084, 157, 4, 26, "Output",
   CellTags->"SimplifyPolyLog"],
  Cell[1749807, 69093, 195, 6, 26, "Input",
   CellTags->"SimplifyPolyLog"],
  Cell[1750005, 69101, 1107, 37, 29, "Output",
   CellTags->"SimplifyPolyLog"],
  Cell[1751149, 69143, 271, 9, 26, "Input",
   CellTags->"SimplifyPolyLog"],
  Cell[1751423, 69154, 1522, 51, 82, "Output",
   CellTags->"SimplifyPolyLog"],
  Cell[1752982, 69210, 221, 7, 26, "Input",
   CellTags->"SimplifyPolyLog"],
  Cell[1753206, 69219, 1297, 46, 87, "Output",
   CellTags->"SimplifyPolyLog"],
  Cell[1754540, 69270, 271, 9, 26, "Input",
   CellTags->"SimplifyPolyLog"],
  Cell[1754814, 69281, 2782, 97, 124, "Output",
   CellTags->"SimplifyPolyLog"],
  Cell[1757633, 69383, 191, 6, 26, "Input",
   CellTags->"SimplifyPolyLog"],
  Cell[1757827, 69391, 378, 13, 47, "Output",
   CellTags->"SimplifyPolyLog"],
  Cell[1758242, 69409, 166, 5, 26, "Input",
   CellTags->"SimplifyPolyLog"],
  Cell[1758411, 69416, 295, 9, 46, "Output",
   CellTags->"SimplifyPolyLog"],
  Cell[1758743, 69430, 146, 4, 26, "Input",
   CellTags->"SimplifyPolyLog"],
  Cell[1758892, 69436, 450, 15, 46, "Output",
   CellTags->"SimplifyPolyLog"],
  Cell[1759379, 69456, 146, 4, 26, "Input",
   CellTags->"SimplifyPolyLog"],
  Cell[1759528, 69462, 399, 14, 44, "Output",
   CellTags->"SimplifyPolyLog"],
  Cell[1759964, 69481, 146, 4, 26, "Input",
   CellTags->"SimplifyPolyLog"],
  Cell[1760113, 69487, 397, 14, 44, "Output",
   CellTags->"SimplifyPolyLog"],
  Cell[1760525, 69504, 116, 3, 26, "Input",
   CellTags->"SimplifyPolyLog"]},
 "Simplify2"->{
  Cell[1760656, 69510, 73, 1, 39, "ObjectName",
   CellTags->"Simplify2"],
  Cell[1760754, 69515, 60, 1, 34, "Subsubsection",
   CellTags->"Simplify2"],
  Cell[1760817, 69518, 81, 1, 24, "Text",
   CellTags->"Simplify2"],
  Cell[1760935, 69524, 57, 1, 34, "Subsubsection",
   CellTags->"Simplify2"],
  Cell[1760995, 69527, 76, 2, 26, "Input",
   CellTags->"Simplify2"]},
 "SmallDelta"->{
  Cell[1761086, 69532, 57, 1, 39, "ObjectName",
   CellTags->"SmallDelta"],
  Cell[1761168, 69537, 61, 1, 34, "Subsubsection",
   CellTags->"SmallDelta"],
  Cell[1761232, 69540, 87, 1, 24, "Text",
   CellTags->"SmallDelta"]},
 "SmallEpsilon"->{
  Cell[1761334, 69544, 61, 1, 39, "ObjectName",
   CellTags->"SmallEpsilon"],
  Cell[1761420, 69549, 63, 1, 34, "Subsubsection",
   CellTags->"SmallEpsilon"],
  Cell[1761486, 69552, 91, 1, 24, "Text",
   CellTags->"SmallEpsilon"]},
 "SmallVariable"->{
  Cell[1761592, 69556, 63, 1, 39, "ObjectName",
   CellTags->"SmallVariable"],
  Cell[1761680, 69561, 64, 1, 34, "Subsubsection",
   CellTags->"SmallVariable"],
  Cell[1761747, 69564, 281, 5, 41, "Text",
   CellTags->"SmallVariable"],
  Cell[1762031, 69571, 437, 19, 24, "Text",
   CellTags->"SmallVariable"]},
 "SmallVariables"->{
  Cell[1762483, 69593, 65, 1, 39, "ObjectName",
   CellTags->"SmallVariables"],
  Cell[1762573, 69598, 65, 1, 36, "Subsubsection",
   CellTags->"SmallVariables"],
  Cell[1762641, 69601, 225, 5, 43, "Text",
   CellTags->"SmallVariables"],
  Cell[1762869, 69608, 182, 9, 25, "Text",
   CellTags->"SmallVariables"]},
 "SMP"->{
  Cell[1763066, 69620, 43, 1, 41, "ObjectName",
   CellTags->"SMP"],
  Cell[1763134, 69625, 54, 1, 36, "Subsubsection",
   CellTags->"SMP"],
  Cell[1763191, 69628, 232, 5, 41, "Text",
   CellTags->"SMP"],
  Cell[1763448, 69637, 127, 4, 26, "Input",
   CellTags->"SMP"],
  Cell[1763578, 69643, 208, 6, 38, "Output",
   CellTags->"SMP"],
  Cell[1763801, 69652, 168, 8, 25, "Text",
   CellTags->"SMP"],
  Cell[1764006, 69665, 51, 1, 34, "Subsubsection",
   CellTags->"SMP"],
  Cell[1764082, 69670, 109, 3, 26, "Input",
   CellTags->"SMP"],
  Cell[1764194, 69675, 143, 4, 26, "Output",
   CellTags->"SMP"],
  Cell[1777987, 70286, 162, 8, 70, "Text",
   CellTags->"SMP"]},
 "SMVertex"->{
  Cell[1764364, 69683, 53, 1, 41, "ObjectName",
   CellTags->"SMVertex"],
  Cell[1764442, 69688, 59, 1, 36, "Subsubsection",
   CellTags->"SMVertex"],
  Cell[1764504, 69691, 304, 6, 41, "Text",
   CellTags->"SMVertex"],
  Cell[1764833, 69701, 112, 3, 26, "Input",
   CellTags->"SMVertex"],
  Cell[1764948, 69706, 279, 9, 26, "Output",
   CellTags->"SMVertex"],
  Cell[1765276, 69721, 56, 1, 34, "Subsubsection",
   CellTags->"SMVertex"],
  Cell[1765357, 69726, 223, 6, 26, "Input",
   CellTags->"SMVertex"],
  Cell[1765583, 69734, 1223, 45, 28, "Output",
   CellTags->"SMVertex"]},
 "Smu"->{
  Cell[1766833, 69783, 61, 1, 41, "ObjectName",
   CellTags->"Smu"],
  Cell[1766919, 69788, 54, 1, 36, "Subsubsection",
   CellTags->"Smu"],
  Cell[1766976, 69791, 45, 1, 25, "Text",
   CellTags->"Smu"]},
 "Sn"->{
  Cell[1767036, 69795, 41, 1, 41, "ObjectName",
   CellTags->"Sn"],
  Cell[1767102, 69800, 53, 1, 36, "Subsubsection",
   CellTags->"Sn"],
  Cell[1767158, 69803, 300, 14, 25, "Text",
   CellTags->"Sn"]},
 "SO"->{
  Cell[1767473, 69820, 41, 1, 41, "ObjectName",
   CellTags->"SO"],
  Cell[1767539, 69825, 53, 1, 36, "Subsubsection",
   CellTags->"SO"],
  Cell[1767595, 69828, 173, 4, 43, "Text",
   CellTags->"SO"],
  Cell[1767771, 69834, 580, 28, 25, "Text",
   CellTags->"SO"],
  Cell[1768388, 69867, 50, 1, 34, "Subsubsection",
   CellTags->"SO"],
  Cell[1768463, 69872, 98, 3, 26, "Input",
   CellTags->"SO"],
  Cell[1768564, 69877, 307, 11, 28, "Output",
   CellTags->"SO"],
  Cell[1768908, 69893, 121, 4, 26, "Input",
   CellTags->"SO"],
  Cell[1769032, 69899, 426, 16, 28, "Output",
   CellTags->"SO"],
  Cell[1769495, 69920, 163, 6, 26, "Input",
   CellTags->"SO"],
  Cell[1769661, 69928, 259, 7, 38, "Output",
   CellTags->"SO"]},
 "SOD"->{
  Cell[1769947, 69939, 43, 1, 70, "ObjectName",
   CellTags->"SOD"],
  Cell[1770015, 69944, 54, 1, 70, "Subsubsection",
   CellTags->"SOD"],
  Cell[1770072, 69947, 189, 4, 70, "Text",
   CellTags->"SOD"],
  Cell[1770264, 69953, 492, 24, 70, "Text",
   CellTags->"SOD"],
  Cell[1770793, 69982, 51, 1, 34, "Subsubsection",
   CellTags->"SOD"],
  Cell[1770869, 69987, 100, 3, 26, "Input",
   CellTags->"SOD"],
  Cell[1770972, 69992, 307, 11, 28, "Output",
   CellTags->"SOD"],
  Cell[1771316, 70008, 123, 4, 26, "Input",
   CellTags->"SOD"],
  Cell[1771442, 70014, 427, 16, 28, "Output",
   CellTags->"SOD"],
  Cell[1771906, 70035, 165, 6, 26, "Input",
   CellTags->"SOD"],
  Cell[1772074, 70043, 310, 9, 38, "Output",
   CellTags->"SOD"]},
 "Solve2"->{
  Cell[1772411, 70056, 49, 1, 70, "ObjectName",
   CellTags->"Solve2"],
  Cell[1772485, 70061, 57, 1, 70, "Subsubsection",
   CellTags->"Solve2"],
  Cell[1772545, 70064, 346, 7, 58, "Text",
   CellTags->"Solve2"],
  Cell[1772916, 70075, 112, 3, 26, "Input",
   CellTags->"Solve2"],
  Cell[1773031, 70080, 306, 9, 26, "Output",
   CellTags->"Solve2"],
  Cell[1773386, 70095, 54, 1, 34, "Subsubsection",
   CellTags->"Solve2"],
  Cell[1773465, 70100, 390, 14, 26, "Input",
   CellTags->"Solve2"],
  Cell[1773858, 70116, 415, 14, 44, "Output",
   CellTags->"Solve2"],
  Cell[1774288, 70133, 111, 3, 24, "Text",
   CellTags->"Solve2"],
  Cell[1774424, 70140, 153, 5, 26, "Input",
   CellTags->"Solve2"],
  Cell[1774580, 70147, 218, 7, 26, "Output",
   CellTags->"Solve2"],
  Cell[1774835, 70159, 267, 8, 26, "Input",
   CellTags->"Solve2"],
  Cell[1775105, 70169, 218, 7, 26, "Output",
   CellTags->"Solve2"],
  Cell[1775360, 70181, 443, 15, 26, "Input",
   CellTags->"Solve2"],
  Cell[1775806, 70198, 356, 12, 45, "Output",
   CellTags->"Solve2"],
  Cell[1776199, 70215, 389, 14, 26, "Input",
   CellTags->"Solve2"],
  Cell[1776591, 70231, 448, 15, 44, "Output",
   CellTags->"Solve2"]},
 "Solve3"->{
  Cell[1777066, 70250, 49, 1, 70, "ObjectName",
   CellTags->"Solve3"],
  Cell[1777140, 70255, 57, 1, 70, "Subsubsection",
   CellTags->"Solve3"],
  Cell[1777200, 70258, 271, 5, 41, "Text",
   CellTags->"Solve3"],
  Cell[1777496, 70267, 112, 3, 26, "Input",
   CellTags->"Solve3"],
  Cell[1777611, 70272, 361, 11, 26, "Output",
   CellTags->"Solve3"],
  Cell[1778186, 70299, 54, 1, 34, "Subsubsection",
   CellTags->"Solve3"],
  Cell[1778265, 70304, 390, 14, 26, "Input",
   CellTags->"Solve3"],
  Cell[1778658, 70320, 697, 17, 109, "Message",
   CellTags->"Solve3"],
  Cell[1779358, 70339, 697, 17, 109, "Message",
   CellTags->"Solve3"],
  Cell[1780058, 70358, 417, 14, 44, "Output",
   CellTags->"Solve3"]},
 "SP"->{
  Cell[1780502, 70376, 41, 1, 70, "ObjectName",
   CellTags->"SP"],
  Cell[1780568, 70381, 53, 1, 70, "Subsubsection",
   CellTags->"SP"],
  Cell[1780624, 70384, 327, 11, 70, "Text",
   CellTags->"SP"],
  Cell[1780954, 70397, 429, 19, 70, "Text",
   CellTags->"SP"],
  Cell[1781420, 70421, 50, 1, 34, "Subsubsection",
   CellTags->"SP"],
  Cell[1781495, 70426, 184, 6, 26, "Input",
   CellTags->"SP"],
  Cell[1781682, 70434, 408, 17, 28, "Output",
   CellTags->"SP"],
  Cell[1782127, 70456, 189, 7, 26, "Input",
   CellTags->"SP"],
  Cell[1782319, 70465, 339, 12, 28, "Output",
   CellTags->"SP"],
  Cell[1782695, 70482, 234, 8, 26, "Input",
   CellTags->"SP"],
  Cell[1782932, 70492, 454, 18, 28, "Output",
   CellTags->"SP"],
  Cell[1783423, 70515, 168, 5, 26, "Input",
   CellTags->"SP"],
  Cell[1783594, 70522, 251, 9, 28, "Output",
   CellTags->"SP"],
  Cell[1783882, 70536, 157, 5, 26, "Input",
   CellTags->"SP"],
  Cell[1784042, 70543, 178, 5, 38, "Output",
   CellTags->"SP"],
  Cell[1784257, 70553, 188, 7, 26, "Input",
   CellTags->"SP"],
  Cell[1784448, 70562, 254, 7, 38, "Output",
   CellTags->"SP"],
  Cell[1784739, 70574, 217, 8, 26, "Input",
   CellTags->"SP"],
  Cell[1784959, 70584, 178, 5, 38, "Output",
   CellTags->"SP"]},
 "SPC"->{
  Cell[1785164, 70593, 43, 1, 41, "ObjectName",
   CellTags->"SPC"],
  Cell[1785232, 70598, 54, 1, 36, "Subsubsection",
   CellTags->"SPC"],
  Cell[1785289, 70601, 81, 1, 25, "Text",
   CellTags->"SPC"],
  Cell[1785373, 70604, 207, 9, 25, "Text",
   CellTags->"SPC"]},
 "SPD"->{
  Cell[1785595, 70616, 43, 1, 41, "ObjectName",
   CellTags->"SPD"],
  Cell[1785663, 70621, 54, 1, 36, "Subsubsection",
   CellTags->"SPD"],
  Cell[1785720, 70624, 346, 11, 43, "Text",
   CellTags->"SPD"],
  Cell[1786069, 70637, 522, 24, 25, "Text",
   CellTags->"SPD"],
  Cell[1786628, 70666, 51, 1, 34, "Subsubsection",
   CellTags->"SPD"],
  Cell[1786704, 70671, 187, 6, 26, "Input",
   CellTags->"SPD"],
  Cell[1786894, 70679, 411, 17, 28, "Output",
   CellTags->"SPD"],
  Cell[1787342, 70701, 191, 7, 26, "Input",
   CellTags->"SPD"],
  Cell[1787536, 70710, 340, 12, 28, "Output",
   CellTags->"SPD"],
  Cell[1787913, 70727, 236, 8, 26, "Input",
   CellTags->"SPD"],
  Cell[1788152, 70737, 454, 18, 28, "Output",
   CellTags->"SPD"],
  Cell[1788643, 70760, 170, 5, 26, "Input",
   CellTags->"SPD"],
  Cell[1788816, 70767, 251, 9, 28, "Output",
   CellTags->"SPD"],
  Cell[1789104, 70781, 159, 5, 26, "Input",
   CellTags->"SPD"],
  Cell[1789266, 70788, 180, 5, 38, "Output",
   CellTags->"SPD"],
  Cell[1789483, 70798, 190, 7, 26, "Input",
   CellTags->"SPD"],
  Cell[1789676, 70807, 305, 9, 38, "Output",
   CellTags->"SPD"],
  Cell[1790018, 70821, 219, 8, 26, "Input",
   CellTags->"SPD"],
  Cell[1790240, 70831, 180, 5, 38, "Output",
   CellTags->"SPD"],
  Cell[1790457, 70841, 270, 9, 26, "Input",
   CellTags->"SPD"],
  Cell[1790730, 70852, 180, 5, 38, "Output",
   CellTags->"SPD"]},
 "Spinor"->{
  Cell[1790937, 70861, 49, 1, 41, "ObjectName",
   CellTags->"Spinor"],
  Cell[1791011, 70866, 57, 1, 36, "Subsubsection",
   CellTags->"Spinor"],
  Cell[1791071, 70869, 1326, 49, 131, "Text",
   CellTags->"Spinor"],
  Cell[1792400, 70920, 766, 34, 25, "Text",
   CellTags->"Spinor"],
  Cell[1793203, 70959, 54, 1, 34, "Subsubsection",
   CellTags->"Spinor"],
  Cell[1793282, 70964, 106, 3, 26, "Input",
   CellTags->"Spinor"],
  Cell[1793391, 70969, 216, 7, 28, "Output",
   CellTags->"Spinor"],
  Cell[1793644, 70981, 129, 4, 26, "Input",
   CellTags->"Spinor"],
  Cell[1793776, 70987, 229, 7, 28, "Output",
   CellTags->"Spinor"],
  Cell[1794020, 70997, 146, 4, 24, "Text",
   CellTags->"Spinor"],
  Cell[1794191, 71005, 250, 8, 26, "Input",
   CellTags->"Spinor"],
  Cell[1794444, 71015, 168, 5, 26, "Output",
   CellTags->"Spinor"],
  Cell[1794649, 71025, 254, 8, 26, "Input",
   CellTags->"Spinor"],
  Cell[1794906, 71035, 293, 10, 28, "Output",
   CellTags->"Spinor"],
  Cell[1795236, 71050, 141, 4, 26, "Input",
   CellTags->"Spinor"],
  Cell[1795380, 71056, 234, 7, 38, "Output",
   CellTags->"Spinor"],
  Cell[1795629, 71066, 158, 4, 24, "Text",
   CellTags->"Spinor"],
  Cell[1795812, 71074, 212, 7, 26, "Input",
   CellTags->"Spinor"],
  Cell[1796027, 71083, 234, 7, 38, "Output",
   CellTags->"Spinor"],
  Cell[1796298, 71095, 165, 5, 26, "Input",
   CellTags->"Spinor"],
  Cell[1796466, 71102, 234, 7, 38, "Output",
   CellTags->"Spinor"],
  Cell[1796715, 71112, 77, 1, 24, "Text",
   CellTags->"Spinor"],
  Cell[1796817, 71117, 210, 7, 26, "Input",
   CellTags->"Spinor"],
  Cell[1797030, 71126, 234, 7, 38, "Output",
   CellTags->"Spinor"]},
 "SpinorCollect"->{
  Cell[1797291, 71137, 63, 1, 41, "ObjectName",
   CellTags->"SpinorCollect"],
  Cell[1797379, 71142, 64, 1, 36, "Subsubsection",
   CellTags->"SpinorCollect"],
  Cell[1797446, 71145, 191, 4, 43, "Text",
   CellTags->"SpinorCollect"],
  Cell[1797640, 71151, 301, 13, 25, "Text",
   CellTags->"SpinorCollect"]},
 "SpinorUBar"->{
  Cell[1797956, 71167, 57, 1, 41, "ObjectName",
   CellTags->"SpinorUBar"],
  Cell[1798038, 71172, 61, 1, 36, "Subsubsection",
   CellTags->"SpinorUBar"],
  Cell[1798102, 71175, 170, 7, 25, "Text",
   CellTags->"SpinorUBar"],
  Cell[1798275, 71184, 505, 24, 25, "Text",
   CellTags->"SpinorUBar"],
  Cell[1798817, 71213, 58, 1, 36, "Subsubsection",
   CellTags->"SpinorUBar"],
  Cell[1798878, 71216, 85, 1, 24, "Text",
   CellTags->"SpinorUBar"],
  Cell[1798988, 71221, 114, 3, 26, "Input",
   CellTags->"SpinorUBar"],
  Cell[1799105, 71226, 242, 8, 28, "Output",
   CellTags->"SpinorUBar"],
  Cell[1799384, 71239, 137, 4, 26, "Input",
   CellTags->"SpinorUBar"],
  Cell[1799524, 71245, 256, 8, 28, "Output",
   CellTags->"SpinorUBar"],
  Cell[1799817, 71258, 173, 5, 26, "Input",
   CellTags->"SpinorUBar"],
  Cell[1799993, 71265, 194, 5, 38, "Output",
   CellTags->"SpinorUBar"],
  Cell[1800224, 71275, 204, 7, 26, "Input",
   CellTags->"SpinorUBar"],
  Cell[1800431, 71284, 240, 7, 38, "Output",
   CellTags->"SpinorUBar"],
  Cell[1800708, 71296, 233, 8, 26, "Input",
   CellTags->"SpinorUBar"],
  Cell[1800944, 71306, 240, 7, 38, "Output",
   CellTags->"SpinorUBar"]},
 "SpinorU"->{
  Cell[1801211, 71317, 51, 1, 41, "ObjectName",
   CellTags->"SpinorU"],
  Cell[1801287, 71322, 58, 1, 70, "Subsubsection",
   CellTags->"SpinorU"],
  Cell[1801348, 71325, 79, 1, 70, "Text",
   CellTags->"SpinorU"],
  Cell[1801430, 71328, 511, 24, 70, "Text",
   CellTags->"SpinorU"],
  Cell[1801978, 71357, 55, 1, 34, "Subsubsection",
   CellTags->"SpinorU"],
  Cell[1802058, 71362, 108, 3, 26, "Input",
   CellTags->"SpinorU"],
  Cell[1802169, 71367, 174, 5, 28, "Output",
   CellTags->"SpinorU"],
  Cell[1802380, 71377, 131, 4, 26, "Input",
   CellTags->"SpinorU"],
  Cell[1802514, 71383, 184, 5, 28, "Output",
   CellTags->"SpinorU"],
  Cell[1802735, 71393, 167, 5, 26, "Input",
   CellTags->"SpinorU"],
  Cell[1802905, 71400, 188, 5, 38, "Output",
   CellTags->"SpinorU"],
  Cell[1803130, 71410, 198, 7, 26, "Input",
   CellTags->"SpinorU"],
  Cell[1803331, 71419, 235, 7, 38, "Output",
   CellTags->"SpinorU"],
  Cell[1803603, 71431, 227, 8, 26, "Input",
   CellTags->"SpinorU"],
  Cell[1803833, 71441, 235, 7, 38, "Output",
   CellTags->"SpinorU"]},
 "SpinorVBar"->{
  Cell[1804095, 71452, 57, 1, 70, "ObjectName",
   CellTags->"SpinorVBar"],
  Cell[1804177, 71457, 61, 1, 70, "Subsubsection",
   CellTags->"SpinorVBar"],
  Cell[1804241, 71460, 175, 7, 70, "Text",
   CellTags->"SpinorVBar"],
  Cell[1804419, 71469, 505, 24, 70, "Text",
   CellTags->"SpinorVBar"],
  Cell[1804961, 71498, 58, 1, 34, "Subsubsection",
   CellTags->"SpinorVBar"],
  Cell[1805044, 71503, 114, 3, 26, "Input",
   CellTags->"SpinorVBar"],
  Cell[1805161, 71508, 246, 8, 28, "Output",
   CellTags->"SpinorVBar"],
  Cell[1805444, 71521, 137, 4, 26, "Input",
   CellTags->"SpinorVBar"],
  Cell[1805584, 71527, 256, 8, 28, "Output",
   CellTags->"SpinorVBar"],
  Cell[1805877, 71540, 173, 5, 26, "Input",
   CellTags->"SpinorVBar"],
  Cell[1806053, 71547, 194, 5, 38, "Output",
   CellTags->"SpinorVBar"],
  Cell[1806284, 71557, 204, 7, 26, "Input",
   CellTags->"SpinorVBar"],
  Cell[1806491, 71566, 259, 8, 38, "Output",
   CellTags->"SpinorVBar"],
  Cell[1806787, 71579, 233, 8, 26, "Input",
   CellTags->"SpinorVBar"],
  Cell[1807023, 71589, 259, 8, 38, "Output",
   CellTags->"SpinorVBar"]},
 "SpinorV"->{
  Cell[1807309, 71601, 51, 1, 70, "ObjectName",
   CellTags->"SpinorV"],
  Cell[1807385, 71606, 58, 1, 70, "Subsubsection",
   CellTags->"SpinorV"],
  Cell[1807446, 71609, 79, 1, 70, "Text",
   CellTags->"SpinorV"],
  Cell[1807528, 71612, 511, 24, 70, "Text",
   CellTags->"SpinorV"],
  Cell[1808076, 71641, 55, 1, 34, "Subsubsection",
   CellTags->"SpinorV"],
  Cell[1808156, 71646, 108, 3, 26, "Input",
   CellTags->"SpinorV"],
  Cell[1808267, 71651, 174, 5, 28, "Output",
   CellTags->"SpinorV"],
  Cell[1808478, 71661, 131, 4, 26, "Input",
   CellTags->"SpinorV"],
  Cell[1808612, 71667, 184, 5, 28, "Output",
   CellTags->"SpinorV"],
  Cell[1808833, 71677, 167, 5, 26, "Input",
   CellTags->"SpinorV"],
  Cell[1809003, 71684, 188, 5, 38, "Output",
   CellTags->"SpinorV"],
  Cell[1809228, 71694, 198, 7, 26, "Input",
   CellTags->"SpinorV"],
  Cell[1809429, 71703, 256, 8, 38, "Output",
   CellTags->"SpinorV"],
  Cell[1809722, 71716, 227, 8, 26, "Input",
   CellTags->"SpinorV"],
  Cell[1809952, 71726, 258, 8, 38, "Output",
   CellTags->"SpinorV"]},
 "SpinPolarizationSum"->{
  Cell[1810237, 71738, 75, 1, 41, "ObjectName",
   CellTags->"SpinPolarizationSum"],
  Cell[1810337, 71743, 70, 1, 36, "Subsubsection",
   CellTags->"SpinPolarizationSum"],
  Cell[1810410, 71746, 164, 4, 25, "Text",
   CellTags->"SpinPolarizationSum"],
  Cell[1810577, 71752, 203, 8, 25, "Text",
   CellTags->"SpinPolarizationSum"]},
 "SPL"->{
  Cell[1810795, 71763, 43, 1, 41, "ObjectName",
   CellTags->"SPL"],
  Cell[1810863, 71768, 54, 1, 36, "Subsubsection",
   CellTags->"SPL"],
  Cell[1810920, 71771, 77, 1, 25, "Text",
   CellTags->"SPL"],
  Cell[1811000, 71774, 195, 9, 25, "Text",
   CellTags->"SPL"]},
 "SplittingFunction"->{
  Cell[1811210, 71786, 71, 1, 41, "ObjectName",
   CellTags->"SplittingFunction"],
  Cell[1811306, 71791, 68, 1, 36, "Subsubsection",
   CellTags->"SplittingFunction"],
  Cell[1811377, 71794, 218, 7, 24, "Text",
   CellTags->"SplittingFunction"],
  Cell[1811620, 71805, 134, 3, 26, "Input",
   CellTags->"SplittingFunction"],
  Cell[1811757, 71810, 222, 7, 26, "Output",
   CellTags->"SplittingFunction"],
  Cell[1811994, 71820, 218, 9, 25, "Text",
   CellTags->"SplittingFunction"],
  Cell[1812249, 71834, 65, 1, 36, "Subsubsection",
   CellTags->"SplittingFunction"],
  Cell[1812317, 71837, 65, 1, 24, "Text",
   CellTags->"SplittingFunction"],
  Cell[1812385, 71840, 221, 6, 26, "Input",
   CellTags->"SplittingFunction"],
  Cell[1812609, 71848, 170, 4, 24, "Text",
   CellTags->"SplittingFunction"],
  Cell[1812804, 71856, 130, 3, 26, "Input",
   CellTags->"SplittingFunction"],
  Cell[1812937, 71861, 573, 20, 46, "Output",
   CellTags->"SplittingFunction"],
  Cell[1813547, 71886, 130, 3, 26, "Input",
   CellTags->"SplittingFunction"],
  Cell[1813680, 71891, 342, 11, 29, "Output",
   CellTags->"SplittingFunction"],
  Cell[1814059, 71907, 130, 3, 26, "Input",
   CellTags->"SplittingFunction"],
  Cell[1814192, 71912, 313, 10, 45, "Output",
   CellTags->"SplittingFunction"],
  Cell[1814542, 71927, 130, 3, 26, "Input",
   CellTags->"SplittingFunction"],
  Cell[1814675, 71932, 901, 30, 46, "Output",
   CellTags->"SplittingFunction"],
  Cell[1815613, 71967, 130, 3, 26, "Input",
   CellTags->"SplittingFunction"],
  Cell[1815746, 71972, 1170, 39, 46, "Output",
   CellTags->"SplittingFunction"],
  Cell[1816953, 72016, 130, 3, 26, "Input",
   CellTags->"SplittingFunction"],
  Cell[1817086, 72021, 641, 22, 45, "Output",
   CellTags->"SplittingFunction"],
  Cell[1817764, 72048, 130, 3, 26, "Input",
   CellTags->"SplittingFunction"],
  Cell[1817897, 72053, 583, 20, 29, "Output",
   CellTags->"SplittingFunction"],
  Cell[1818495, 72076, 130, 3, 26, "Input",
   CellTags->"SplittingFunction"],
  Cell[1818628, 72081, 63, 1, 24, "Text",
   CellTags->"SplittingFunction"],
  Cell[1818694, 72084, 221, 6, 26, "Input",
   CellTags->"SplittingFunction"],
  Cell[1818940, 72094, 130, 3, 26, "Input",
   CellTags->"SplittingFunction"],
  Cell[1819073, 72099, 573, 20, 46, "Output",
   CellTags->"SplittingFunction"],
  Cell[1819683, 72124, 130, 3, 26, "Input",
   CellTags->"SplittingFunction"],
  Cell[1819816, 72129, 278, 9, 29, "Output",
   CellTags->"SplittingFunction"],
  Cell[1820131, 72143, 130, 3, 26, "Input",
   CellTags->"SplittingFunction"],
  Cell[1820264, 72148, 279, 9, 26, "Output",
   CellTags->"SplittingFunction"],
  Cell[1820580, 72162, 130, 3, 26, "Input",
   CellTags->"SplittingFunction"],
  Cell[1820713, 72167, 855, 29, 46, "Output",
   CellTags->"SplittingFunction"],
  Cell[1821605, 72201, 130, 3, 26, "Input",
   CellTags->"SplittingFunction"],
  Cell[1821738, 72206, 1289, 43, 46, "Output",
   CellTags->"SplittingFunction"],
  Cell[1823064, 72254, 130, 3, 26, "Input",
   CellTags->"SplittingFunction"],
  Cell[1823197, 72259, 545, 19, 26, "Output",
   CellTags->"SplittingFunction"],
  Cell[1823779, 72283, 131, 3, 26, "Input",
   CellTags->"SplittingFunction"],
  Cell[1823913, 72288, 487, 17, 26, "Output",
   CellTags->"SplittingFunction"],
  Cell[1824437, 72310, 130, 3, 26, "Input",
   CellTags->"SplittingFunction"],
  Cell[1824570, 72315, 487, 17, 29, "Output",
   CellTags->"SplittingFunction"],
  Cell[1825094, 72337, 131, 3, 26, "Input",
   CellTags->"SplittingFunction"],
  Cell[1825228, 72342, 489, 17, 29, "Output",
   CellTags->"SplittingFunction"],
  Cell[1825754, 72364, 130, 3, 26, "Input",
   CellTags->"SplittingFunction"],
  Cell[1825887, 72369, 1404, 47, 46, "Output",
   CellTags->"SplittingFunction"],
  Cell[1827328, 72421, 131, 3, 26, "Input",
   CellTags->"SplittingFunction"],
  Cell[1827462, 72426, 1404, 47, 46, "Output",
   CellTags->"SplittingFunction"],
  Cell[1828903, 72478, 131, 3, 26, "Input",
   CellTags->"SplittingFunction"],
  Cell[1829037, 72483, 681, 23, 29, "Output",
   CellTags->"SplittingFunction"],
  Cell[1829755, 72511, 132, 3, 26, "Input",
   CellTags->"SplittingFunction"],
  Cell[1829890, 72516, 5429, 175, 353, "Output",
   CellTags->"SplittingFunction"],
  Cell[1835356, 72696, 130, 3, 26, "Input",
   CellTags->"SplittingFunction"],
  Cell[1835489, 72701, 2948, 93, 98, "Output",
   CellTags->"SplittingFunction"],
  Cell[1838474, 72799, 130, 3, 26, "Input",
   CellTags->"SplittingFunction"],
  Cell[1838607, 72804, 3626, 118, 151, "Output",
   CellTags->"SplittingFunction"],
  Cell[1842270, 72927, 130, 3, 26, "Input",
   CellTags->"SplittingFunction"],
  Cell[1842403, 72932, 4300, 140, 198, "Output",
   CellTags->"SplittingFunction"]},
 "StandardMatrixElement"->{
  Cell[1846730, 73076, 79, 1, 39, "ObjectName",
   CellTags->"StandardMatrixElement"],
  Cell[1846834, 73081, 72, 1, 34, "Subsubsection",
   CellTags->"StandardMatrixElement"],
  Cell[1846909, 73084, 134, 3, 24, "Text",
   CellTags->"StandardMatrixElement"],
  Cell[1847046, 73089, 189, 9, 24, "Text",
   CellTags->"StandardMatrixElement"]},
 "SubContext"->{
  Cell[1847250, 73101, 57, 1, 39, "ObjectName",
   CellTags->"SubContext"],
  Cell[1847332, 73106, 61, 1, 34, "Subsubsection",
   CellTags->"SubContext"],
  Cell[1847396, 73109, 120, 3, 24, "Text",
   CellTags->"SubContext"],
  Cell[1847519, 73114, 279, 14, 24, "Text",
   CellTags->"SubContext"]},
 "SubLoop"->{
  Cell[1847813, 73131, 51, 1, 39, "ObjectName",
   CellTags->"SubLoop"],
  Cell[1847889, 73136, 58, 1, 34, "Subsubsection",
   CellTags->"SubLoop"],
  Cell[1847950, 73139, 152, 4, 24, "Text",
   CellTags->"SubLoop"],
  Cell[1848105, 73145, 178, 9, 24, "Text",
   CellTags->"SubLoop"]},
 "SumP"->{
  Cell[1848298, 73157, 45, 1, 39, "ObjectName",
   CellTags->"SumP"],
  Cell[1848368, 73162, 55, 1, 34, "Subsubsection",
   CellTags->"SumP"],
  Cell[1848426, 73165, 767, 31, 27, "Text",
   CellTags->"SumP"],
  Cell[1849196, 73198, 261, 14, 24, "Text",
   CellTags->"SumP"],
  Cell[1849494, 73217, 52, 1, 34, "Subsubsection",
   CellTags->"SumP"],
  Cell[1849571, 73222, 149, 5, 26, "Input",
   CellTags->"SumP"],
  Cell[1849723, 73229, 305, 9, 28, "Output",
   CellTags->"SumP"],
  Cell[1850065, 73243, 149, 5, 26, "Input",
   CellTags->"SumP"],
  Cell[1850217, 73250, 307, 9, 28, "Output",
   CellTags->"SumP"],
  Cell[1850561, 73264, 125, 4, 26, "Input",
   CellTags->"SumP"],
  Cell[1850689, 73270, 280, 8, 28, "Output",
   CellTags->"SumP"],
  Cell[1851006, 73283, 125, 4, 26, "Input",
   CellTags->"SumP"],
  Cell[1851134, 73289, 168, 5, 44, "Output",
   CellTags->"SumP"],
  Cell[1851339, 73299, 301, 12, 56, "Input",
   CellTags->"SumP"],
  Cell[1851643, 73313, 166, 5, 44, "Output",
   CellTags->"SumP"],
  Cell[1851846, 73323, 186, 6, 26, "Input",
   CellTags->"SumP"],
  Cell[1852035, 73331, 806, 30, 46, "Output",
   CellTags->"SumP"],
  Cell[1852878, 73366, 124, 4, 26, "Input",
   CellTags->"SumP"],
  Cell[1853005, 73372, 166, 5, 44, "Output",
   CellTags->"SumP"]},
 "SumS"->{
  Cell[1853198, 73381, 45, 1, 39, "ObjectName",
   CellTags->"SumS"],
  Cell[1853268, 73386, 55, 1, 34, "Subsubsection",
   CellTags->"SumS"],
  Cell[1853326, 73389, 1438, 57, 44, "Text",
   CellTags->"SumS"],
  Cell[1854789, 73450, 108, 3, 26, "Input",
   CellTags->"SumS"],
  Cell[1854900, 73455, 207, 7, 26, "Output",
   CellTags->"SumS"],
  Cell[1855122, 73465, 261, 14, 24, "Text",
   CellTags->"SumS"],
  Cell[1855420, 73484, 52, 1, 34, "Subsubsection",
   CellTags->"SumS"],
  Cell[1855497, 73489, 149, 5, 26, "Input",
   CellTags->"SumS"],
  Cell[1855649, 73496, 253, 9, 26, "Output",
   CellTags->"SumS"],
  Cell[1855939, 73510, 149, 5, 26, "Input",
   CellTags->"SumS"],
  Cell[1856091, 73517, 253, 9, 26, "Output",
   CellTags->"SumS"],
  Cell[1856381, 73531, 144, 5, 26, "Input",
   CellTags->"SumS"],
  Cell[1856528, 73538, 216, 7, 26, "Output",
   CellTags->"SumS"],
  Cell[1856781, 73550, 173, 5, 26, "Input",
   CellTags->"SumS"],
  Cell[1856957, 73557, 305, 11, 46, "Output",
   CellTags->"SumS"],
  Cell[1857299, 73573, 197, 6, 26, "Input",
   CellTags->"SumS"],
  Cell[1857499, 73581, 384, 14, 49, "Output",
   CellTags->"SumS"],
  Cell[1857898, 73598, 192, 6, 26, "Input",
   CellTags->"SumS"],
  Cell[1858115, 73608, 149, 5, 26, "Input",
   CellTags->"SumS"],
  Cell[1858267, 73615, 551, 20, 49, "Output",
   CellTags->"SumS"],
  Cell[1858833, 73638, 193, 6, 26, "Input",
   CellTags->"SumS"],
  Cell[1859051, 73648, 125, 4, 26, "Input",
   CellTags->"SumS"],
  Cell[1859179, 73654, 166, 5, 44, "Output",
   CellTags->"SumS"],
  Cell[1859382, 73664, 174, 6, 56, "Input",
   CellTags->"SumS"],
  Cell[1859559, 73672, 165, 5, 44, "Output",
   CellTags->"SumS"],
  Cell[1859761, 73682, 159, 5, 26, "Input",
   CellTags->"SumS"],
  Cell[1859923, 73689, 254, 9, 26, "Output",
   CellTags->"SumS"],
  Cell[1860214, 73703, 146, 4, 26, "Input",
   CellTags->"SumS"],
  Cell[1860363, 73709, 191, 6, 45, "Output",
   CellTags->"SumS"],
  Cell[1860591, 73720, 144, 5, 26, "Input",
   CellTags->"SumS"],
  Cell[1860738, 73727, 186, 6, 43, "Output",
   CellTags->"SumS"],
  Cell[1860961, 73738, 125, 4, 26, "Input",
   CellTags->"SumS"],
  Cell[1861089, 73744, 184, 6, 43, "Output",
   CellTags->"SumS"]},
 "SumT"->{
  Cell[1861300, 73754, 45, 1, 41, "ObjectName",
   CellTags->"SumT"],
  Cell[1861370, 73759, 55, 1, 36, "Subsubsection",
   CellTags->"SumT"],
  Cell[1861428, 73762, 461, 15, 43, "Text",
   CellTags->"SumT"],
  Cell[1861892, 73779, 261, 14, 25, "Text",
   CellTags->"SumT"],
  Cell[1862190, 73798, 52, 1, 34, "Subsubsection",
   CellTags->"SumT"],
  Cell[1862267, 73803, 149, 5, 26, "Input",
   CellTags->"SumT"],
  Cell[1862419, 73810, 278, 10, 29, "Output",
   CellTags->"SumT"],
  Cell[1862734, 73825, 149, 5, 26, "Input",
   CellTags->"SumT"],
  Cell[1862886, 73832, 278, 10, 29, "Output",
   CellTags->"SumT"],
  Cell[1863201, 73847, 125, 4, 26, "Input",
   CellTags->"SumT"],
  Cell[1863329, 73853, 222, 8, 29, "Output",
   CellTags->"SumT"],
  Cell[1863588, 73866, 173, 5, 26, "Input",
   CellTags->"SumT"],
  Cell[1863764, 73873, 403, 15, 46, "Output",
   CellTags->"SumT"],
  Cell[1864204, 73893, 125, 4, 26, "Input",
   CellTags->"SumT"],
  Cell[1864332, 73899, 183, 6, 43, "Output",
   CellTags->"SumT"],
  Cell[1864552, 73910, 244, 9, 56, "Input",
   CellTags->"SumT"],
  Cell[1864799, 73921, 184, 6, 43, "Output",
   CellTags->"SumT"],
  Cell[1865020, 73932, 159, 5, 26, "Input",
   CellTags->"SumT"],
  Cell[1865182, 73939, 279, 10, 29, "Output",
   CellTags->"SumT"],
  Cell[1865498, 73954, 136, 4, 26, "Input",
   CellTags->"SumT"],
  Cell[1865637, 73960, 294, 8, 45, "Output",
   CellTags->"SumT"],
  Cell[1865968, 73973, 125, 4, 26, "Input",
   CellTags->"SumT"],
  Cell[1866096, 73979, 184, 6, 43, "Output",
   CellTags->"SumT"],
  Cell[1866317, 73990, 144, 5, 26, "Input",
   CellTags->"SumT"],
  Cell[1866464, 73997, 184, 6, 43, "Output",
   CellTags->"SumT"],
  Cell[1866685, 74008, 268, 10, 56, "Input",
   CellTags->"SumT"],
  Cell[1866956, 74020, 378, 13, 26, "Output",
   CellTags->"SumT"],
  Cell[1867371, 74038, 136, 4, 26, "Input",
   CellTags->"SumT"],
  Cell[1867510, 74044, 212, 7, 44, "Output",
   CellTags->"SumT"],
  Cell[1867759, 74056, 155, 5, 26, "Input",
   CellTags->"SumT"],
  Cell[1867917, 74063, 294, 8, 45, "Output",
   CellTags->"SumT"],
  Cell[1868248, 74076, 129, 4, 26, "Input",
   CellTags->"SumT"],
  Cell[1868380, 74082, 502, 17, 44, "Output",
   CellTags->"SumT"],
  Cell[1868919, 74104, 240, 9, 26, "Input",
   CellTags->"SumT"],
  Cell[1869162, 74115, 500, 17, 44, "Output",
   CellTags->"SumT"]},
 "SUND"->{
  Cell[1869689, 74136, 45, 1, 39, "ObjectName",
   CellTags->"SUND"],
  Cell[1869759, 74141, 55, 1, 34, "Subsubsection",
   CellTags->"SUND"],
  Cell[1869817, 74144, 224, 10, 26, "Text",
   CellTags->"SUND"],
  Cell[1870044, 74156, 392, 19, 24, "Text",
   CellTags->"SUND"],
  Cell[1870473, 74180, 52, 1, 34, "Subsubsection",
   CellTags->"SUND"],
  Cell[1870550, 74185, 135, 4, 26, "Input",
   CellTags->"SUND"],
  Cell[1870688, 74191, 191, 6, 28, "Output",
   CellTags->"SUND"],
  Cell[1870916, 74202, 211, 6, 26, "Input",
   CellTags->"SUND"],
  Cell[1871130, 74210, 1128, 45, 28, "Output",
   CellTags->"SUND"],
  Cell[1872295, 74260, 135, 4, 26, "Input",
   CellTags->"SUND"],
  Cell[1872433, 74266, 189, 6, 28, "Output",
   CellTags->"SUND"],
  Cell[1872659, 74277, 135, 4, 26, "Input",
   CellTags->"SUND"],
  Cell[1872797, 74283, 191, 6, 28, "Output",
   CellTags->"SUND"],
  Cell[1873025, 74294, 266, 8, 26, "Input",
   CellTags->"SUND"],
  Cell[1873294, 74304, 302, 10, 29, "Output",
   CellTags->"SUND"],
  Cell[1873633, 74319, 377, 12, 26, "Input",
   CellTags->"SUND"],
  Cell[1874013, 74333, 346, 12, 48, "Output",
   CellTags->"SUND"],
  Cell[1874396, 74350, 377, 12, 26, "Input",
   CellTags->"SUND"],
  Cell[1874776, 74364, 516, 20, 48, "Output",
   CellTags->"SUND"],
  Cell[1875329, 74389, 173, 6, 26, "Input",
   CellTags->"SUND"],
  Cell[1875505, 74397, 192, 5, 38, "Output",
   CellTags->"SUND"],
  Cell[1875734, 74407, 202, 7, 26, "Input",
   CellTags->"SUND"],
  Cell[1875939, 74416, 300, 8, 38, "Output",
   CellTags->"SUND"],
  Cell[1876276, 74429, 231, 8, 26, "Input",
   CellTags->"SUND"],
  Cell[1876510, 74439, 194, 5, 38, "Output",
   CellTags->"SUND"],
  Cell[1876719, 74447, 104, 3, 26, "Input",
   CellTags->"SUND"]},
 "SUNDelta"->{
  Cell[1876838, 74453, 53, 1, 39, "ObjectName",
   CellTags->"SUNDelta"],
  Cell[1876916, 74458, 59, 1, 34, "Subsubsection",
   CellTags->"SUNDelta"],
  Cell[1876978, 74461, 172, 6, 24, "Text",
   CellTags->"SUNDelta"],
  Cell[1877153, 74469, 717, 33, 24, "Text",
   CellTags->"SUNDelta"],
  Cell[1877907, 74507, 56, 1, 34, "Subsubsection",
   CellTags->"SUNDelta"],
  Cell[1877988, 74512, 133, 4, 26, "Input",
   CellTags->"SUNDelta"],
  Cell[1878124, 74518, 195, 6, 28, "Output",
   CellTags->"SUNDelta"],
  Cell[1878356, 74529, 262, 8, 26, "Input",
   CellTags->"SUNDelta"],
  Cell[1878621, 74539, 343, 14, 26, "Output",
   CellTags->"SUNDelta"],
  Cell[1879001, 74558, 205, 6, 26, "Input",
   CellTags->"SUNDelta"],
  Cell[1879209, 74566, 342, 14, 28, "Output",
   CellTags->"SUNDelta"],
  Cell[1879588, 74585, 169, 5, 26, "Input",
   CellTags->"SUNDelta"],
  Cell[1879760, 74592, 190, 5, 38, "Output",
   CellTags->"SUNDelta"],
  Cell[1879987, 74602, 200, 7, 26, "Input",
   CellTags->"SUNDelta"],
  Cell[1880190, 74611, 261, 7, 38, "Output",
   CellTags->"SUNDelta"],
  Cell[1880488, 74623, 229, 8, 26, "Input",
   CellTags->"SUNDelta"],
  Cell[1880720, 74633, 184, 5, 38, "Output",
   CellTags->"SUNDelta"],
  Cell[1880941, 74643, 194, 7, 26, "Input",
   CellTags->"SUNDelta"],
  Cell[1881138, 74652, 262, 7, 38, "Output",
   CellTags->"SUNDelta"],
  Cell[1881415, 74662, 256, 5, 41, "Text",
   CellTags->"SUNDelta"],
  Cell[1881696, 74671, 315, 10, 26, "Input",
   CellTags->"SUNDelta"],
  Cell[1882014, 74683, 502, 21, 28, "Output",
   CellTags->"SUNDelta"],
  Cell[1882553, 74709, 110, 3, 26, "Input",
   CellTags->"SUNDelta"],
  Cell[1882666, 74714, 437, 12, 38, "Output",
   CellTags->"SUNDelta"],
  Cell[1883140, 74731, 200, 7, 26, "Input",
   CellTags->"SUNDelta"],
  Cell[1883343, 74740, 280, 7, 38, "Output",
   CellTags->"SUNDelta"]},
 "SUNDeltaContract"->{
  Cell[1883650, 74751, 69, 1, 39, "ObjectName",
   CellTags->"SUNDeltaContract"],
  Cell[1883744, 74756, 67, 1, 34, "Subsubsection",
   CellTags->"SUNDeltaContract"],
  Cell[1883814, 74759, 397, 9, 58, "Text",
   CellTags->"SUNDeltaContract"],
  Cell[1884214, 74770, 297, 14, 24, "Text",
   CellTags->"SUNDeltaContract"]},
 "SUNF"->{
  Cell[1884526, 74787, 45, 1, 39, "ObjectName",
   CellTags->"SUNF"],
  Cell[1884596, 74792, 55, 1, 34, "Subsubsection",
   CellTags->"SUNF"],
  Cell[1884654, 74795, 190, 6, 24, "Text",
   CellTags->"SUNF"],
  Cell[1884869, 74805, 108, 3, 26, "Input",
   CellTags->"SUNF"],
  Cell[1884980, 74810, 210, 7, 26, "Output",
   CellTags->"SUNF"],
  Cell[1885205, 74820, 701, 34, 24, "Text",
   CellTags->"SUNF"],
  Cell[1885943, 74859, 52, 1, 34, "Subsubsection",
   CellTags->"SUNF"],
  Cell[1886020, 74864, 294, 10, 26, "Input",
   CellTags->"SUNF"],
  Cell[1886317, 74876, 314, 11, 28, "Output",
   CellTags->"SUNF"],
  Cell[1886668, 74892, 103, 3, 26, "Input",
   CellTags->"SUNF"],
  Cell[1886774, 74897, 805, 35, 28, "Output",
   CellTags->"SUNF"],
  Cell[1887616, 74937, 110, 3, 26, "Input",
   CellTags->"SUNF"],
  Cell[1887729, 74942, 497, 21, 28, "Output",
   CellTags->"SUNF"],
  Cell[1888263, 74968, 135, 4, 26, "Input",
   CellTags->"SUNF"],
  Cell[1888401, 74974, 189, 6, 28, "Output",
   CellTags->"SUNF"],
  Cell[1888627, 74985, 163, 5, 26, "Input",
   CellTags->"SUNF"],
  Cell[1888793, 74992, 143, 4, 26, "Output",
   CellTags->"SUNF"],
  Cell[1888951, 74999, 319, 10, 24, "Text",
   CellTags->"SUNF"],
  Cell[1889295, 75013, 185, 5, 26, "Input",
   CellTags->"SUNF"],
  Cell[1889483, 75020, 1189, 45, 28, "Output",
   CellTags->"SUNF"],
  Cell[1890709, 75070, 266, 8, 26, "Input",
   CellTags->"SUNF"],
  Cell[1890978, 75080, 395, 16, 28, "Output",
   CellTags->"SUNF"],
  Cell[1891410, 75101, 239, 7, 26, "Input",
   CellTags->"SUNF"],
  Cell[1891652, 75110, 1208, 46, 28, "Output",
   CellTags->"SUNF"],
  Cell[1892897, 75161, 173, 6, 26, "Input",
   CellTags->"SUNF"],
  Cell[1893073, 75169, 194, 5, 38, "Output",
   CellTags->"SUNF"],
  Cell[1893304, 75179, 202, 7, 26, "Input",
   CellTags->"SUNF"],
  Cell[1893509, 75188, 300, 8, 38, "Output",
   CellTags->"SUNF"],
  Cell[1893846, 75201, 231, 8, 26, "Input",
   CellTags->"SUNF"],
  Cell[1894080, 75211, 192, 5, 38, "Output",
   CellTags->"SUNF"],
  Cell[1894309, 75221, 135, 4, 26, "Input",
   CellTags->"SUNF"],
  Cell[1894447, 75227, 191, 6, 28, "Output",
   CellTags->"SUNF"],
  Cell[1894675, 75238, 162, 5, 26, "Input",
   CellTags->"SUNF"],
  Cell[1894840, 75245, 443, 19, 28, "Output",
   CellTags->"SUNF"]},
 "SUNFJacobi"->{
  Cell[1895310, 75268, 57, 1, 39, "ObjectName",
   CellTags->"SUNFJacobi"],
  Cell[1895392, 75273, 61, 1, 34, "Subsubsection",
   CellTags->"SUNFJacobi"],
  Cell[1895456, 75276, 146, 4, 24, "Text",
   CellTags->"SUNFJacobi"],
  Cell[1895605, 75282, 283, 13, 24, "Text",
   CellTags->"SUNFJacobi"],
  Cell[1895925, 75300, 58, 1, 34, "Subsubsection",
   CellTags->"SUNFJacobi"],
  Cell[1896008, 75305, 380, 12, 26, "Input",
   CellTags->"SUNFJacobi"],
  Cell[1896391, 75319, 736, 33, 29, "Output",
   CellTags->"SUNFJacobi"],
  Cell[1897164, 75357, 379, 12, 26, "Input",
   CellTags->"SUNFJacobi"],
  Cell[1897546, 75371, 1404, 63, 30, "Output",
   CellTags->"SUNFJacobi"]},
 "SUNIndex"->{
  Cell[1898977, 75438, 53, 1, 39, "ObjectName",
   CellTags->"SUNIndex"],
  Cell[1899055, 75443, 59, 1, 34, "Subsubsection",
   CellTags->"SUNIndex"],
  Cell[1899117, 75446, 135, 6, 24, "Text",
   CellTags->"SUNIndex"],
  Cell[1899255, 75454, 405, 18, 24, "Text",
   CellTags->"SUNIndex"],
  Cell[1899697, 75477, 56, 1, 34, "Subsubsection",
   CellTags->"SUNIndex"],
  Cell[1899778, 75482, 110, 3, 26, "Input",
   CellTags->"SUNIndex"],
  Cell[1899891, 75487, 147, 4, 27, "Output",
   CellTags->"SUNIndex"],
  Cell[1900075, 75496, 110, 3, 26, "Input",
   CellTags->"SUNIndex"],
  Cell[1900188, 75501, 169, 4, 38, "Output",
   CellTags->"SUNIndex"],
  Cell[1900394, 75510, 200, 7, 26, "Input",
   CellTags->"SUNIndex"],
  Cell[1900597, 75519, 262, 7, 38, "Output",
   CellTags->"SUNIndex"]},
 "SUNIndexRename"->{
  Cell[1900886, 75530, 65, 1, 39, "ObjectName",
   CellTags->"SUNIndexRename"],
  Cell[1900976, 75535, 65, 1, 34, "Subsubsection",
   CellTags->"SUNIndexRename"],
  Cell[1901044, 75538, 163, 4, 24, "Text",
   CellTags->"SUNIndexRename"],
  Cell[1901210, 75544, 304, 14, 24, "Text",
   CellTags->"SUNIndexRename"]},
 "SUNN"->{
  Cell[1901529, 75561, 45, 1, 39, "ObjectName",
   CellTags->"SUNN"],
  Cell[1901599, 75566, 55, 1, 34, "Subsubsection",
   CellTags->"SUNN"],
  Cell[1901657, 75569, 203, 7, 24, "Text",
   CellTags->"SUNN"],
  Cell[1901863, 75578, 579, 29, 24, "Text",
   CellTags->"SUNN"],
  Cell[1902479, 75612, 52, 1, 34, "Subsubsection",
   CellTags->"SUNN"],
  Cell[1902556, 75617, 317, 9, 26, "Input",
   CellTags->"SUNN"],
  Cell[1902876, 75628, 191, 6, 26, "Output",
   CellTags->"SUNN"]},
 "SUNNToCACF"->{
  Cell[1903094, 75638, 57, 1, 39, "ObjectName",
   CellTags->"SUNNToCACF"],
  Cell[1903176, 75643, 61, 1, 34, "Subsubsection",
   CellTags->"SUNNToCACF"],
  Cell[1903240, 75646, 187, 4, 41, "Text",
   CellTags->"SUNNToCACF"],
  Cell[1903430, 75652, 573, 29, 24, "Text",
   CellTags->"SUNNToCACF"],
  Cell[1904040, 75686, 58, 1, 34, "Subsubsection",
   CellTags->"SUNNToCACF"],
  Cell[1904123, 75691, 322, 9, 26, "Input",
   CellTags->"SUNNToCACF"],
  Cell[1904448, 75702, 228, 7, 26, "Output",
   CellTags->"SUNNToCACF"]},
 "SUNSimplify"->{
  Cell[1904703, 75713, 59, 1, 39, "ObjectName",
   CellTags->"SUNSimplify"],
  Cell[1904787, 75718, 62, 1, 34, "Subsubsection",
   CellTags->"SUNSimplify"],
  Cell[1904852, 75721, 376, 7, 58, "Text",
   CellTags->"SUNSimplify"],
  Cell[1905253, 75732, 122, 3, 26, "Input",
   CellTags->"SUNSimplify"],
  Cell[1905378, 75737, 554, 14, 47, "Output",
   CellTags->"SUNSimplify"],
  Cell[1905947, 75754, 299, 13, 24, "Text",
   CellTags->"SUNSimplify"],
  Cell[1906283, 75772, 59, 1, 34, "Subsubsection",
   CellTags->"SUNSimplify"],
  Cell[1906367, 75777, 247, 8, 26, "Input",
   CellTags->"SUNSimplify"],
  Cell[1906617, 75787, 273, 9, 28, "Output",
   CellTags->"SUNSimplify"],
  Cell[1906927, 75801, 117, 3, 26, "Input",
   CellTags->"SUNSimplify"],
  Cell[1907047, 75806, 348, 14, 26, "Output",
   CellTags->"SUNSimplify"],
  Cell[1907432, 75825, 189, 6, 26, "Input",
   CellTags->"SUNSimplify"],
  Cell[1907624, 75833, 291, 11, 26, "Output",
   CellTags->"SUNSimplify"],
  Cell[1907952, 75849, 117, 3, 26, "Input",
   CellTags->"SUNSimplify"],
  Cell[1908072, 75854, 171, 5, 26, "Output",
   CellTags->"SUNSimplify"],
  Cell[1908280, 75864, 183, 5, 26, "Input",
   CellTags->"SUNSimplify"],
  Cell[1908466, 75871, 247, 8, 46, "Output",
   CellTags->"SUNSimplify"],
  Cell[1908750, 75884, 254, 8, 26, "Input",
   CellTags->"SUNSimplify"],
  Cell[1909007, 75894, 268, 9, 28, "Output",
   CellTags->"SUNSimplify"],
  Cell[1909312, 75908, 117, 3, 26, "Input",
   CellTags->"SUNSimplify"],
  Cell[1909432, 75913, 402, 16, 28, "Output",
   CellTags->"SUNSimplify"],
  Cell[1909871, 75934, 269, 8, 26, "Input",
   CellTags->"SUNSimplify"],
  Cell[1910143, 75944, 269, 9, 28, "Output",
   CellTags->"SUNSimplify"],
  Cell[1910449, 75958, 117, 3, 26, "Input",
   CellTags->"SUNSimplify"],
  Cell[1910569, 75963, 239, 7, 28, "Output",
   CellTags->"SUNSimplify"],
  Cell[1910845, 75975, 259, 8, 26, "Input",
   CellTags->"SUNSimplify"],
  Cell[1911107, 75985, 269, 9, 28, "Output",
   CellTags->"SUNSimplify"],
  Cell[1911413, 75999, 117, 3, 26, "Input",
   CellTags->"SUNSimplify"],
  Cell[1911533, 76004, 402, 16, 28, "Output",
   CellTags->"SUNSimplify"],
  Cell[1911972, 76025, 259, 8, 26, "Input",
   CellTags->"SUNSimplify"],
  Cell[1912234, 76035, 269, 9, 28, "Output",
   CellTags->"SUNSimplify"],
  Cell[1912540, 76049, 180, 5, 26, "Input",
   CellTags->"SUNSimplify"],
  Cell[1912723, 76056, 149, 4, 26, "Output",
   CellTags->"SUNSimplify"],
  Cell[1912909, 76065, 384, 12, 26, "Input",
   CellTags->"SUNSimplify"],
  Cell[1913296, 76079, 353, 12, 48, "Output",
   CellTags->"SUNSimplify"],
  Cell[1913686, 76096, 389, 12, 26, "Input",
   CellTags->"SUNSimplify"],
  Cell[1914078, 76110, 492, 19, 48, "Output",
   CellTags->"SUNSimplify"],
  Cell[1914607, 76134, 246, 7, 26, "Input",
   CellTags->"SUNSimplify"],
  Cell[1914856, 76143, 1215, 46, 28, "Output",
   CellTags->"SUNSimplify"],
  Cell[1916108, 76194, 246, 7, 26, "Input",
   CellTags->"SUNSimplify"],
  Cell[1916357, 76203, 1173, 45, 28, "Output",
   CellTags->"SUNSimplify"],
  Cell[1917567, 76253, 273, 8, 26, "Input",
   CellTags->"SUNSimplify"],
  Cell[1917843, 76263, 292, 9, 44, "Output",
   CellTags->"SUNSimplify"],
  Cell[1918172, 76277, 439, 14, 26, "Input",
   CellTags->"SUNSimplify"],
  Cell[1918614, 76293, 421, 15, 28, "Output",
   CellTags->"SUNSimplify"],
  Cell[1919072, 76313, 181, 5, 26, "Input",
   CellTags->"SUNSimplify"],
  Cell[1919256, 76320, 747, 33, 28, "Output",
   CellTags->"SUNSimplify"],
  Cell[1920040, 76358, 238, 6, 26, "Input",
   CellTags->"SUNSimplify"],
  Cell[1920281, 76366, 1404, 63, 28, "Output",
   CellTags->"SUNSimplify"],
  Cell[1921722, 76434, 200, 6, 26, "Input",
   CellTags->"SUNSimplify"],
  Cell[1921925, 76442, 206, 6, 28, "Output",
   CellTags->"SUNSimplify"],
  Cell[1922168, 76453, 341, 10, 26, "Input",
   CellTags->"SUNSimplify"],
  Cell[1922512, 76465, 585, 22, 26, "Output",
   CellTags->"SUNSimplify"],
  Cell[1923134, 76492, 181, 5, 26, "Input",
   CellTags->"SUNSimplify"],
  Cell[1923318, 76499, 585, 22, 26, "Output",
   CellTags->"SUNSimplify"],
  Cell[1923940, 76526, 180, 5, 26, "Input",
   CellTags->"SUNSimplify"],
  Cell[1924123, 76533, 613, 23, 26, "Output",
   CellTags->"SUNSimplify"],
  Cell[1924751, 76559, 208, 6, 26, "Input",
   CellTags->"SUNSimplify"]},
 "SUNT"->{
  Cell[1924974, 76568, 45, 1, 39, "ObjectName",
   CellTags->"SUNT"],
  Cell[1925044, 76573, 55, 1, 34, "Subsubsection",
   CellTags->"SUNT"],
  Cell[1925102, 76576, 237, 10, 24, "Text",
   CellTags->"SUNT"],
  Cell[1925342, 76588, 674, 34, 24, "Text",
   CellTags->"SUNT"],
  Cell[1926053, 76627, 52, 1, 34, "Subsubsection",
   CellTags->"SUNT"],
  Cell[1926130, 76632, 102, 3, 26, "Input",
   CellTags->"SUNT"],
  Cell[1926235, 76637, 199, 7, 26, "Output",
   CellTags->"SUNT"],
  Cell[1926449, 76647, 203, 7, 24, "Text",
   CellTags->"SUNT"],
  Cell[1926677, 76658, 207, 6, 26, "Input",
   CellTags->"SUNT"],
  Cell[1926887, 76666, 355, 14, 28, "Output",
   CellTags->"SUNT"],
  Cell[1927279, 76685, 145, 4, 26, "Input",
   CellTags->"SUNT"],
  Cell[1927427, 76691, 407, 17, 28, "Output",
   CellTags->"SUNT"],
  Cell[1927871, 76713, 242, 7, 26, "Input",
   CellTags->"SUNT"],
  Cell[1928116, 76722, 312, 12, 45, "Output",
   CellTags->"SUNT"],
  Cell[1928465, 76739, 184, 5, 26, "Input",
   CellTags->"SUNT"],
  Cell[1928652, 76746, 175, 5, 28, "Output",
   CellTags->"SUNT"],
  Cell[1928864, 76756, 174, 5, 26, "Input",
   CellTags->"SUNT"],
  Cell[1929041, 76763, 441, 17, 44, "Output",
   CellTags->"SUNT"],
  Cell[1929519, 76785, 242, 7, 26, "Input",
   CellTags->"SUNT"],
  Cell[1929764, 76794, 310, 12, 45, "Output",
   CellTags->"SUNT"],
  Cell[1930089, 76809, 422, 15, 26, "Text",
   CellTags->"SUNT"],
  Cell[1930536, 76828, 161, 5, 26, "Input",
   CellTags->"SUNT"],
  Cell[1930700, 76835, 370, 15, 44, "Output",
   CellTags->"SUNT"],
  Cell[1931085, 76853, 197, 7, 27, "Text",
   CellTags->"SUNT"],
  Cell[1931307, 76864, 197, 6, 26, "Input",
   CellTags->"SUNT"],
  Cell[1931507, 76872, 395, 16, 29, "Output",
   CellTags->"SUNT"],
  Cell[1931939, 76893, 198, 6, 26, "Input",
   CellTags->"SUNT"],
  Cell[1932140, 76901, 304, 9, 54, "Output",
   CellTags->"SUNT"],
  Cell[1932481, 76915, 167, 6, 26, "Input",
   CellTags->"SUNT"],
  Cell[1932651, 76923, 194, 5, 38, "Output",
   CellTags->"SUNT"],
  Cell[1932882, 76933, 195, 7, 26, "Input",
   CellTags->"SUNT"],
  Cell[1933080, 76942, 159, 4, 38, "Output",
   CellTags->"SUNT"]},
 "SUNTrace"->{
  Cell[1933266, 76950, 53, 1, 39, "ObjectName",
   CellTags->"SUNTrace"],
  Cell[1933344, 76955, 59, 1, 34, "Subsubsection",
   CellTags->"SUNTrace"],
  Cell[1933406, 76958, 81, 1, 24, "Text",
   CellTags->"SUNTrace"],
  Cell[1933512, 76963, 116, 3, 26, "Input",
   CellTags->"SUNTrace"],
  Cell[1933631, 76968, 214, 7, 26, "Output",
   CellTags->"SUNTrace"],
  Cell[1933860, 76978, 280, 14, 24, "Text",
   CellTags->"SUNTrace"],
  Cell[1934177, 76997, 56, 1, 34, "Subsubsection",
   CellTags->"SUNTrace"],
  Cell[1934258, 77002, 165, 5, 26, "Input",
   CellTags->"SUNTrace"],
  Cell[1934426, 77009, 374, 15, 44, "Output",
   CellTags->"SUNTrace"],
  Cell[1934837, 77029, 175, 5, 26, "Input",
   CellTags->"SUNTrace"],
  Cell[1935015, 77036, 566, 23, 28, "Output",
   CellTags->"SUNTrace"],
  Cell[1935618, 77064, 240, 7, 26, "Input",
   CellTags->"SUNTrace"],
  Cell[1935861, 77073, 853, 36, 46, "Output",
   CellTags->"SUNTrace"],
  Cell[1936751, 77114, 185, 5, 26, "Input",
   CellTags->"SUNTrace"],
  Cell[1936939, 77121, 683, 28, 28, "Output",
   CellTags->"SUNTrace"],
  Cell[1937659, 77154, 278, 8, 26, "Input",
   CellTags->"SUNTrace"],
  Cell[1937940, 77164, 4312, 185, 84, "Output",
   CellTags->"SUNTrace"],
  Cell[1942289, 77354, 177, 5, 26, "Input",
   CellTags->"SUNTrace"],
  Cell[1942469, 77361, 683, 28, 28, "Output",
   CellTags->"SUNTrace"],
  Cell[1943189, 77394, 288, 8, 26, "Input",
   CellTags->"SUNTrace"],
  Cell[1943480, 77404, 11396, 437, 180, "Output",
   CellTags->"SUNTrace"],
  Cell[1954913, 77846, 177, 5, 26, "Input",
   CellTags->"SUNTrace"],
  Cell[1955093, 77853, 802, 33, 28, "Output",
   CellTags->"SUNTrace"],
  Cell[1955932, 77891, 270, 8, 26, "Input",
   CellTags->"SUNTrace"],
  Cell[1956205, 77901, 147, 4, 26, "Output",
   CellTags->"SUNTrace"],
  Cell[1956389, 77910, 270, 8, 26, "Input",
   CellTags->"SUNTrace"],
  Cell[1956662, 77920, 147, 4, 26, "Output",
   CellTags->"SUNTrace"],
  Cell[1956846, 77929, 270, 8, 26, "Input",
   CellTags->"SUNTrace"],
  Cell[1957119, 77939, 147, 4, 26, "Output",
   CellTags->"SUNTrace"],
  Cell[1957303, 77948, 270, 8, 26, "Input",
   CellTags->"SUNTrace"],
  Cell[1957576, 77958, 618, 24, 29, "Output",
   CellTags->"SUNTrace"]},
 "SymbolicSum2"->{
  Cell[1958221, 77986, 61, 1, 39, "ObjectName",
   CellTags->"SymbolicSum2"],
  Cell[1958307, 77991, 63, 1, 34, "Subsubsection",
   CellTags->"SymbolicSum2"],
  Cell[1958373, 77994, 284, 6, 41, "Text",
   CellTags->"SymbolicSum2"]},
 "SymbolicSum3"->{
  Cell[1958672, 78003, 61, 1, 39, "ObjectName",
   CellTags->"SymbolicSum3"],
  Cell[1958758, 78008, 63, 1, 34, "Subsubsection",
   CellTags->"SymbolicSum3"],
  Cell[1958824, 78011, 292, 6, 41, "Text",
   CellTags->"SymbolicSum3"]},
 "Symmetrize"->{
  Cell[1959131, 78020, 57, 1, 41, "ObjectName",
   CellTags->"Symmetrize"],
  Cell[1959213, 78025, 61, 1, 34, "Subsubsection",
   CellTags->"Symmetrize"],
  Cell[1959277, 78028, 145, 4, 24, "Text",
   CellTags->"Symmetrize"],
  Cell[1959425, 78034, 193, 8, 24, "Text",
   CellTags->"Symmetrize"],
  Cell[1959655, 78047, 58, 1, 34, "Subsubsection",
   CellTags->"Symmetrize"],
  Cell[1959738, 78052, 240, 8, 26, "Input",
   CellTags->"Symmetrize"],
  Cell[1959981, 78062, 359, 12, 44, "Output",
   CellTags->"Symmetrize"],
  Cell[1960377, 78079, 260, 8, 26, "Input",
   CellTags->"Symmetrize"],
  Cell[1960640, 78089, 694, 21, 44, "Output",
   CellTags->"Symmetrize"]},
 "TBox"->{
  Cell[1961361, 78114, 61, 1, 39, "ObjectName",
   CellTags->{"TBox", "Tbox"}],
  Cell[1961447, 78119, 65, 1, 34, "Subsubsection",
   CellTags->{"TBox", "Tbox"}],
  Cell[1961515, 78122, 262, 5, 41, "Text",
   CellTags->{"TBox", "Tbox"}]},
 "Tbox"->{
  Cell[1961361, 78114, 61, 1, 39, "ObjectName",
   CellTags->{"TBox", "Tbox"}],
  Cell[1961447, 78119, 65, 1, 34, "Subsubsection",
   CellTags->{"TBox", "Tbox"}],
  Cell[1961515, 78122, 262, 5, 41, "Text",
   CellTags->{"TBox", "Tbox"}]},
 "Tdec"->{
  Cell[1961792, 78130, 45, 1, 39, "ObjectName",
   CellTags->"Tdec"],
  Cell[1961862, 78135, 55, 1, 34, "Subsubsection",
   CellTags->"Tdec"],
  Cell[1961920, 78138, 256, 5, 41, "Text",
   CellTags->"Tdec"],
  Cell[1962201, 78147, 108, 3, 26, "Input",
   CellTags->"Tdec"],
  Cell[1962312, 78152, 454, 12, 26, "Output",
   CellTags->"Tdec"],
  Cell[1962781, 78167, 389, 19, 24, "Text",
   CellTags->"Tdec"],
  Cell[1963207, 78191, 52, 1, 34, "Subsubsection",
   CellTags->"Tdec"],
  Cell[1963262, 78194, 608, 22, 36, "Text",
   CellTags->"Tdec"],
  Cell[1963895, 78220, 202, 7, 26, "Input",
   CellTags->"Tdec"],
  Cell[1964100, 78229, 597, 22, 44, "Output",
   CellTags->"Tdec"],
  Cell[1964734, 78256, 199, 7, 26, "Input",
   CellTags->"Tdec"],
  Cell[1964936, 78265, 398, 15, 44, "Output",
   CellTags->"Tdec"],
  Cell[1965349, 78283, 776, 28, 28, "Text",
   CellTags->"Tdec"],
  Cell[1966128, 78313, 529, 18, 26, "Input",
   CellTags->"Tdec"],
  Cell[1966682, 78335, 199, 7, 26, "Input",
   CellTags->"Tdec"],
  Cell[1966884, 78344, 7370, 268, 126, "Output",
   CellTags->"Tdec"],
  Cell[1974291, 78617, 380, 13, 26, "Input",
   CellTags->"Tdec"],
  Cell[1974674, 78632, 5008, 182, 97, "Output",
   CellTags->"Tdec"],
  Cell[1979697, 78817, 121, 4, 26, "Input",
   CellTags->"Tdec"]},
 "TensorFunction"->{
  Cell[1979833, 78824, 65, 1, 39, "ObjectName",
   CellTags->"TensorFunction"],
  Cell[1979923, 78829, 65, 1, 34, "Subsubsection",
   CellTags->"TensorFunction"],
  Cell[1979991, 78832, 360, 7, 58, "Text",
   CellTags->"TensorFunction"],
  Cell[1980354, 78841, 191, 9, 24, "Text",
   CellTags->"TensorFunction"],
  Cell[1980582, 78855, 62, 1, 34, "Subsubsection",
   CellTags->"TensorFunction"],
  Cell[1980669, 78860, 178, 4, 26, "Input",
   CellTags->"TensorFunction"],
  Cell[1980850, 78866, 253, 7, 26, "Output",
   CellTags->"TensorFunction"],
  Cell[1981140, 78878, 116, 3, 26, "Input",
   CellTags->"TensorFunction"],
  Cell[1981259, 78883, 235, 5, 38, "Output",
   CellTags->"TensorFunction"],
  Cell[1981531, 78893, 198, 6, 26, "Input",
   CellTags->"TensorFunction"],
  Cell[1981732, 78901, 417, 15, 28, "Output",
   CellTags->"TensorFunction"],
  Cell[1982186, 78921, 116, 3, 26, "Input",
   CellTags->"TensorFunction"],
  Cell[1982305, 78926, 400, 11, 38, "Output",
   CellTags->"TensorFunction"],
  Cell[1982742, 78942, 228, 7, 26, "Input",
   CellTags->"TensorFunction"],
  Cell[1982973, 78951, 305, 9, 26, "Output",
   CellTags->"TensorFunction"],
  Cell[1983315, 78965, 228, 7, 26, "Input",
   CellTags->"TensorFunction"],
  Cell[1983546, 78974, 304, 9, 26, "Output",
   CellTags->"TensorFunction"],
  Cell[1983887, 78988, 116, 3, 26, "Input",
   CellTags->"TensorFunction"],
  Cell[1984006, 78993, 285, 8, 38, "Output",
   CellTags->"TensorFunction"],
  Cell[1984328, 79006, 118, 3, 26, "Input",
   CellTags->"TensorFunction"],
  Cell[1984449, 79011, 171, 5, 26, "Output",
   CellTags->"TensorFunction"],
  Cell[1984635, 79019, 154, 4, 26, "Input",
   CellTags->"TensorFunction"]},
 "Tf"->{
  Cell[1984804, 79026, 41, 1, 39, "ObjectName",
   CellTags->"Tf"],
  Cell[1984870, 79031, 53, 1, 34, "Subsubsection",
   CellTags->"Tf"],
  Cell[1984926, 79034, 214, 10, 27, "Text",
   CellTags->"Tf"],
  Cell[1985143, 79046, 414, 18, 24, "Text",
   CellTags->"Tf"]},
 "TFi"->{
  Cell[1985572, 79067, 43, 1, 39, "ObjectName",
   CellTags->"TFi"],
  Cell[1985640, 79072, 54, 1, 34, "Subsubsection",
   CellTags->"TFi"],
  Cell[1986322, 79096, 576, 10, 92, "Text",
   CellTags->"TFi"],
  Cell[1986901, 79108, 237, 8, 24, "Text",
   CellTags->"TFi"],
  Cell[1987141, 79118, 185, 4, 41, "Text",
   CellTags->"TFi"],
  Cell[1987329, 79124, 266, 13, 24, "Text",
   CellTags->"TFi"],
  Cell[1987632, 79142, 51, 1, 34, "Subsubsection",
   CellTags->"TFi"],
  Cell[1987708, 79147, 506, 17, 26, "Input",
   CellTags->"TFi"],
  Cell[1988217, 79166, 819, 25, 96, "Output",
   CellTags->"TFi"],
  Cell[1989073, 79196, 295, 10, 26, "Input",
   CellTags->"TFi"],
  Cell[1989371, 79208, 377, 12, 29, "Output",
   CellTags->"TFi"]},
 "ThreeVector"->{
  Cell[1989775, 79224, 59, 1, 39, "ObjectName",
   CellTags->"ThreeVector"],
  Cell[1989859, 79229, 62, 1, 34, "Subsubsection",
   CellTags->"ThreeVector"],
  Cell[1989924, 79232, 91, 1, 24, "Text",
   CellTags->"ThreeVector"],
  Cell[1990018, 79235, 304, 13, 24, "Text",
   CellTags->"ThreeVector"]},
 "TID"->{
  Cell[1990337, 79251, 43, 1, 39, "ObjectName",
   CellTags->"TID"],
  Cell[1990405, 79256, 54, 1, 34, "Subsubsection",
   CellTags->"TID"],
  Cell[1990462, 79259, 80, 1, 24, "Text",
   CellTags->"TID"],
  Cell[1990567, 79264, 106, 3, 26, "Input",
   CellTags->"TID"],
  Cell[1990676, 79269, 697, 16, 67, "Output",
   CellTags->"TID"],
  Cell[1991388, 79288, 293, 14, 24, "Text",
   CellTags->"TID"],
  Cell[1991718, 79307, 51, 1, 34, "Subsubsection",
   CellTags->"TID"],
  Cell[1991794, 79312, 374, 12, 26, "Input",
   CellTags->"TID"],
  Cell[1992171, 79326, 576, 22, 51, "Output",
   CellTags->"TID"],
  Cell[1992784, 79353, 181, 6, 26, "Input",
   CellTags->"TID"],
  Cell[1992968, 79361, 4349, 137, 68, "Output",
   CellTags->"TID"],
  Cell[1997354, 79503, 271, 9, 26, "Input",
   CellTags->"TID"],
  Cell[1997628, 79514, 431, 17, 51, "Output",
   CellTags->"TID"],
  Cell[1998096, 79536, 123, 4, 26, "Input",
   CellTags->"TID"],
  Cell[1998222, 79542, 669, 28, 50, "Output",
   CellTags->"TID"],
  Cell[1998928, 79575, 195, 6, 26, "Input",
   CellTags->"TID"],
  Cell[1999126, 79583, 669, 28, 50, "Output",
   CellTags->"TID"]},
 "TIDL"->{
  Cell[1999822, 79615, 45, 1, 39, "ObjectName",
   CellTags->"TIDL"],
  Cell[1999892, 79620, 55, 1, 34, "Subsubsection",
   CellTags->"TIDL"],
  Cell[1999950, 79623, 87, 1, 24, "Text",
   CellTags->"TIDL"],
  Cell[2000040, 79626, 154, 8, 24, "Text",
   CellTags->"TIDL"],
  Cell[2000209, 79637, 52, 1, 34, "Subsubsection",
   CellTags->"TIDL"],
  Cell[2000286, 79642, 154, 5, 37, "Subsubsection",
   CellTags->"TIDL"],
  Cell[2000443, 79649, 414, 13, 28, "Text",
   CellTags->"TIDL"],
  Cell[2000882, 79666, 202, 7, 26, "Input",
   CellTags->"TIDL"],
  Cell[2001087, 79675, 507, 21, 48, "Output",
   CellTags->"TIDL"],
  Cell[2001643, 79702, 159, 5, 37, "Subsubsection",
   CellTags->"TIDL"],
  Cell[2001827, 79711, 312, 11, 26, "Input",
   CellTags->"TIDL"],
  Cell[2002142, 79724, 1920, 75, 54, "Output",
   CellTags->"TIDL"],
  Cell[2004111, 79805, 154, 5, 37, "Subsubsection",
   CellTags->"TIDL"],
  Cell[2004290, 79814, 277, 10, 26, "Input",
   CellTags->"TIDL"],
  Cell[2004570, 79826, 2854, 109, 51, "Output",
   CellTags->"TIDL"],
  Cell[2007473, 79941, 159, 5, 37, "Subsubsection",
   CellTags->"TIDL"],
  Cell[2007657, 79950, 387, 14, 26, "Input",
   CellTags->"TIDL"],
  Cell[2008047, 79966, 25229, 846, 328, "Output",
   CellTags->"TIDL"],
  Cell[2033325, 80818, 165, 5, 37, "Subsubsection",
   CellTags->"TIDL"],
  Cell[2033493, 80825, 434, 15, 26, "Input",
   CellTags->"TIDL"],
  Cell[2033964, 80845, 173, 5, 37, "Subsubsection",
   CellTags->"TIDL"],
  Cell[2034140, 80852, 506, 17, 26, "Input",
   CellTags->"TIDL"],
  Cell[2034683, 80874, 154, 5, 37, "Subsubsection",
   CellTags->"TIDL"],
  Cell[2034862, 80883, 312, 11, 26, "Input",
   CellTags->"TIDL"],
  Cell[2035177, 80896, 15400, 530, 213, "Output",
   CellTags->"TIDL"],
  Cell[2050626, 81432, 159, 5, 37, "Subsubsection",
   CellTags->"TIDL"],
  Cell[2050788, 81439, 374, 13, 26, "Input",
   CellTags->"TIDL"],
  Cell[2051199, 81457, 165, 5, 37, "Subsubsection",
   CellTags->"TIDL"],
  Cell[2051367, 81464, 444, 15, 26, "Input",
   CellTags->"TIDL"],
  Cell[2051848, 81484, 173, 5, 37, "Subsubsection",
   CellTags->"TIDL"],
  Cell[2052024, 81491, 529, 18, 26, "Input",
   CellTags->"TIDL"],
  Cell[2052578, 81513, 240, 7, 26, "Input",
   CellTags->"TIDL"],
  Cell[2052821, 81522, 205, 6, 26, "Output",
   CellTags->"TIDL"]},
 "TimedIntegrate"->{
  Cell[2053053, 81532, 65, 1, 39, "ObjectName",
   CellTags->"TimedIntegrate"],
  Cell[2053143, 81537, 65, 1, 34, "Subsubsection",
   CellTags->"TimedIntegrate"],
  Cell[2053211, 81540, 225, 5, 41, "Text",
   CellTags->"TimedIntegrate"],
  Cell[2053461, 81549, 128, 3, 26, "Input",
   CellTags->"TimedIntegrate"],
  Cell[2053592, 81554, 466, 14, 26, "Output",
   CellTags->"TimedIntegrate"],
  Cell[2054107, 81574, 62, 1, 34, "Subsubsection",
   CellTags->"TimedIntegrate"],
  Cell[2054172, 81577, 73, 1, 24, "Text",
   CellTags->"TimedIntegrate"],
  Cell[2054270, 81582, 305, 9, 26, "Input",
   CellTags->"TimedIntegrate"],
  Cell[2054578, 81593, 171, 5, 26, "Output",
   CellTags->"TimedIntegrate"],
  Cell[2054764, 81601, 59, 1, 24, "Text",
   CellTags->"TimedIntegrate"],
  Cell[2054848, 81606, 389, 11, 26, "Input",
   CellTags->"TimedIntegrate"],
  Cell[2055240, 81619, 491, 15, 29, "Output",
   CellTags->"TimedIntegrate"]},
 "TLI"->{
  Cell[2055758, 81638, 61, 1, 39, "ObjectName",
   CellTags->"TLI"],
  Cell[2055844, 81643, 54, 1, 34, "Subsubsection",
   CellTags->"TLI"],
  Cell[2055901, 81646, 616, 12, 116, "Text",
   CellTags->"TLI"],
  Cell[2056542, 81662, 106, 3, 26, "Input",
   CellTags->"TLI"],
  Cell[2056651, 81667, 273, 9, 28, "Output",
   CellTags->"TLI"],
  Cell[2056939, 81679, 168, 9, 24, "Text",
   CellTags->"TLI"],
  Cell[2057144, 81693, 51, 1, 34, "Subsubsection",
   CellTags->"TLI"],
  Cell[2057198, 81696, 70, 2, 26, "Input",
   CellTags->"TLI"]},
 "TLIFP"->{
  Cell[2057283, 81701, 65, 1, 39, "ObjectName",
   CellTags->"TLIFP"],
  Cell[2057373, 81706, 56, 1, 34, "Subsubsection",
   CellTags->"TLIFP"],
  Cell[2057432, 81709, 93, 1, 24, "Text",
   CellTags->"TLIFP"],
  Cell[2057550, 81714, 110, 3, 26, "Input",
   CellTags->"TLIFP"],
  Cell[2057663, 81719, 467, 13, 28, "Output",
   CellTags->"TLIFP"],
  Cell[2058145, 81735, 161, 9, 24, "Text",
   CellTags->"TLIFP"],
  Cell[2058343, 81749, 53, 1, 34, "Subsubsection",
   CellTags->"TLIFP"],
  Cell[2058399, 81752, 72, 2, 26, "Input",
   CellTags->"TLIFP"]},
 "TLIHYP"->{
  Cell[2058486, 81757, 67, 1, 39, "ObjectName",
   CellTags->"TLIHYP"],
  Cell[2058578, 81762, 57, 1, 34, "Subsubsection",
   CellTags->"TLIHYP"],
  Cell[2058638, 81765, 136, 4, 24, "Text",
   CellTags->"TLIHYP"],
  Cell[2058799, 81773, 112, 3, 26, "Input",
   CellTags->"TLIHYP"],
  Cell[2058914, 81778, 283, 9, 28, "Output",
   CellTags->"TLIHYP"],
  Cell[2059212, 81790, 162, 9, 24, "Text",
   CellTags->"TLIHYP"],
  Cell[2059411, 81804, 54, 1, 34, "Subsubsection",
   CellTags->"TLIHYP"],
  Cell[2059468, 81807, 73, 2, 26, "Input",
   CellTags->"TLIHYP"]},
 "TLI2"->{
  Cell[2059556, 81812, 45, 1, 39, "ObjectName",
   CellTags->"TLI2"],
  Cell[2059626, 81817, 55, 1, 34, "Subsubsection",
   CellTags->"TLI2"],
  Cell[2059684, 81820, 83, 1, 24, "Text",
   CellTags->"TLI2"],
  Cell[2059770, 81823, 160, 9, 24, "Text",
   CellTags->"TLI2"]},
 "TLI2FC"->{
  Cell[2059945, 81835, 67, 1, 39, "ObjectName",
   CellTags->"TLI2FC"],
  Cell[2060037, 81840, 57, 1, 34, "Subsubsection",
   CellTags->"TLI2FC"],
  Cell[2060097, 81843, 109, 3, 24, "Text",
   CellTags->"TLI2FC"],
  Cell[2060231, 81850, 112, 3, 26, "Input",
   CellTags->"TLI2FC"],
  Cell[2060346, 81855, 397, 12, 28, "Output",
   CellTags->"TLI2FC"],
  Cell[2060758, 81870, 262, 13, 24, "Text",
   CellTags->"TLI2FC"],
  Cell[2061057, 81888, 54, 1, 34, "Subsubsection",
   CellTags->"TLI2FC"],
  Cell[2061114, 81891, 73, 2, 26, "Input",
   CellTags->"TLI2FC"]},
 "ToDistribution"->{
  Cell[2061202, 81896, 65, 1, 39, "ObjectName",
   CellTags->"ToDistribution"],
  Cell[2061292, 81901, 65, 1, 34, "Subsubsection",
   CellTags->"ToDistribution"],
  Cell[2061360, 81904, 424, 7, 58, "Text",
   CellTags->"ToDistribution"],
  Cell[2061809, 81915, 128, 3, 26, "Input",
   CellTags->"ToDistribution"],
  Cell[2061940, 81920, 233, 7, 26, "Output",
   CellTags->"ToDistribution"],
  Cell[2062188, 81930, 209, 9, 24, "Text",
   CellTags->"ToDistribution"],
  Cell[2062434, 81944, 62, 1, 34, "Subsubsection",
   CellTags->"ToDistribution"],
  Cell[2062521, 81949, 217, 6, 26, "Input",
   CellTags->"ToDistribution"],
  Cell[2062741, 81957, 239, 7, 26, "Output",
   CellTags->"ToDistribution"],
  Cell[2063017, 81969, 277, 9, 26, "Input",
   CellTags->"ToDistribution"],
  Cell[2063297, 81980, 1382, 49, 48, "Output",
   CellTags->"ToDistribution"],
  Cell[2064716, 82034, 206, 5, 26, "Input",
   CellTags->"ToDistribution"],
  Cell[2064925, 82041, 231, 7, 26, "Output",
   CellTags->"ToDistribution"],
  Cell[2065193, 82053, 277, 9, 26, "Input",
   CellTags->"ToDistribution"],
  Cell[2065473, 82064, 1422, 51, 51, "Output",
   CellTags->"ToDistribution"],
  Cell[2066932, 82120, 461, 14, 30, "Input",
   CellTags->"ToDistribution"],
  Cell[2067396, 82136, 413, 14, 26, "Output",
   CellTags->"ToDistribution"],
  Cell[2067846, 82155, 118, 3, 26, "Input",
   CellTags->"ToDistribution"],
  Cell[2067967, 82160, 374, 10, 26, "Output",
   CellTags->"ToDistribution"],
  Cell[2068378, 82175, 409, 13, 26, "Input",
   CellTags->"ToDistribution"],
  Cell[2068790, 82190, 413, 14, 26, "Output",
   CellTags->"ToDistribution"]},
 "ToHypergeometric"->{
  Cell[2069230, 82208, 69, 1, 39, "ObjectName",
   CellTags->"ToHypergeometric"],
  Cell[2069324, 82213, 67, 1, 34, "Subsubsection",
   CellTags->"ToHypergeometric"],
  Cell[2069394, 82216, 1686, 67, 43, "Text",
   CellTags->"ToHypergeometric"],
  Cell[2071083, 82285, 479, 19, 24, "Text",
   CellTags->"ToHypergeometric"],
  Cell[2071599, 82309, 64, 1, 34, "Subsubsection",
   CellTags->"ToHypergeometric"],
  Cell[2071688, 82314, 395, 13, 30, "Input",
   CellTags->"ToHypergeometric"],
  Cell[2072086, 82329, 531, 16, 48, "Output",
   CellTags->"ToHypergeometric"],
  Cell[2072654, 82350, 488, 16, 30, "Input",
   CellTags->"ToHypergeometric"],
  Cell[2073145, 82368, 400, 14, 48, "Output",
   CellTags->"ToHypergeometric"],
  Cell[2073582, 82387, 395, 13, 30, "Input",
   CellTags->"ToHypergeometric"],
  Cell[2073980, 82402, 596, 18, 52, "Output",
   CellTags->"ToHypergeometric"],
  Cell[2074613, 82425, 488, 16, 30, "Input",
   CellTags->"ToHypergeometric"],
  Cell[2075104, 82443, 473, 16, 52, "Output",
   CellTags->"ToHypergeometric"]},
 "ToLarin"->{
  Cell[2075604, 82463, 51, 1, 39, "ObjectName",
   CellTags->"ToLarin"],
  Cell[2075680, 82468, 58, 1, 34, "Subsubsection",
   CellTags->"ToLarin"],
  Cell[2075741, 82471, 605, 19, 24, "Text",
   CellTags->"ToLarin"],
  Cell[2076349, 82492, 163, 9, 24, "Text",
   CellTags->"ToLarin"],
  Cell[2076549, 82506, 55, 1, 34, "Subsubsection",
   CellTags->"ToLarin"],
  Cell[2076629, 82511, 229, 6, 26, "Input",
   CellTags->"ToLarin"],
  Cell[2076861, 82519, 431, 16, 26, "Output",
   CellTags->"ToLarin"],
  Cell[2077329, 82540, 108, 3, 26, "Input",
   CellTags->"ToLarin"],
  Cell[2077440, 82545, 1141, 46, 44, "Output",
   CellTags->"ToLarin"]},
 "ToTFi"->{
  Cell[2078608, 82595, 47, 1, 39, "ObjectName",
   CellTags->"ToTFi"],
  Cell[2078680, 82600, 56, 1, 34, "Subsubsection",
   CellTags->"ToTFi"],
  Cell[2078739, 82603, 272, 5, 41, "Text",
   CellTags->"ToTFi"],
  Cell[2079014, 82610, 262, 13, 24, "Text",
   CellTags->"ToTFi"],
  Cell[2079313, 82628, 53, 1, 34, "Subsubsection",
   CellTags->"ToTFi"],
  Cell[2079391, 82633, 333, 11, 26, "Input",
   CellTags->"ToTFi"],
  Cell[2079727, 82646, 1912, 50, 51, "Output",
   CellTags->"ToTFi"],
  Cell[2081676, 82701, 149, 4, 26, "Input",
   CellTags->"ToTFi"],
  Cell[2081828, 82707, 2168, 55, 52, "Output",
   CellTags->"ToTFi"],
  Cell[2084033, 82767, 107, 3, 26, "Input",
   CellTags->"ToTFi"],
  Cell[2084143, 82772, 535, 17, 38, "Output",
   CellTags->"ToTFi"],
  Cell[2084715, 82794, 468, 15, 26, "Input",
   CellTags->"ToTFi"],
  Cell[2085186, 82811, 958, 34, 51, "Output",
   CellTags->"ToTFi"]},
 "TR"->{
  Cell[2086193, 82851, 41, 1, 39, "ObjectName",
   CellTags->"TR"],
  Cell[2086237, 82854, 100, 3, 26, "Output",
   CellTags->"TR"],
  Cell[2086374, 82862, 53, 1, 34, "Subsubsection",
   CellTags->"TR"],
  Cell[2086430, 82865, 420, 12, 58, "Text",
   CellTags->"TR"],
  Cell[2086875, 82881, 104, 3, 26, "Input",
   CellTags->"TR"],
  Cell[2086982, 82886, 803, 19, 67, "Output",
   CellTags->"TR"],
  Cell[2087800, 82908, 536, 23, 24, "Text",
   CellTags->"TR"],
  Cell[2088373, 82936, 50, 1, 34, "Subsubsection",
   CellTags->"TR"],
  Cell[2088448, 82941, 129, 4, 26, "Input",
   CellTags->"TR"],
  Cell[2088580, 82947, 238, 7, 26, "Output",
   CellTags->"TR"],
  Cell[2088855, 82959, 98, 3, 26, "Input",
   CellTags->"TR"],
  Cell[2088956, 82964, 217, 7, 26, "Output",
   CellTags->"TR"],
  Cell[2089210, 82976, 375, 12, 26, "Input",
   CellTags->"TR"],
  Cell[2089588, 82990, 1299, 51, 29, "Output",
   CellTags->"TR"],
  Cell[2090924, 83046, 205, 6, 26, "Input",
   CellTags->"TR"],
  Cell[2091132, 83054, 601, 24, 26, "Output",
   CellTags->"TR"],
  Cell[2091770, 83083, 240, 7, 26, "Input",
   CellTags->"TR"],
  Cell[2092013, 83092, 8713, 322, 73, "Output",
   CellTags->"TR"],
  Cell[2100763, 83419, 74, 2, 26, "Input",
   CellTags->"TR"],
  Cell[2100840, 83423, 146, 4, 26, "Output",
   CellTags->"TR"],
  Cell[2101023, 83432, 91, 3, 26, "Input",
   CellTags->"TR"],
  Cell[2101117, 83437, 421, 7, 79, "Print",
   CellTags->{"Info3500974047-4982097", "TR"}],
  Cell[2101553, 83447, 166, 4, 24, "Text",
   CellTags->"TR"],
  Cell[2101744, 83455, 100, 3, 26, "Input",
   CellTags->"TR"],
  Cell[2101847, 83460, 145, 4, 26, "Output",
   CellTags->"TR"],
  Cell[2102029, 83469, 421, 13, 26, "Input",
   CellTags->"TR"],
  Cell[2102453, 83484, 141, 4, 26, "Output",
   CellTags->"TR"],
  Cell[2102631, 83493, 171, 5, 26, "Input",
   CellTags->"TR"],
  Cell[2102805, 83500, 1035, 40, 28, "Output",
   CellTags->"TR"],
  Cell[2103877, 83545, 444, 14, 26, "Input",
   CellTags->"TR"],
  Cell[2104324, 83561, 384, 14, 29, "Output",
   CellTags->"TR"],
  Cell[2104745, 83580, 224, 7, 26, "Input",
   CellTags->"TR"],
  Cell[2104972, 83589, 222, 7, 26, "Output",
   CellTags->"TR"],
  Cell[2105231, 83601, 104, 3, 26, "Input",
   CellTags->"TR"],
  Cell[2105338, 83606, 215, 6, 38, "Output",
   CellTags->"TR"],
  Cell[2105590, 83617, 347, 10, 26, "Input",
   CellTags->"TR"],
  Cell[2105940, 83629, 234, 7, 26, "Output",
   CellTags->"TR"],
  Cell[2106211, 83641, 104, 3, 26, "Input",
   CellTags->"TR"],
  Cell[2106318, 83646, 225, 6, 38, "Output",
   CellTags->"TR"],
  Cell[2106580, 83657, 339, 11, 26, "Input",
   CellTags->"TR"],
  Cell[2106922, 83670, 470, 17, 28, "Output",
   CellTags->"TR"],
  Cell[2107429, 83692, 214, 6, 26, "Input",
   CellTags->"TR"],
  Cell[2107646, 83700, 252, 8, 26, "Output",
   CellTags->"TR"],
  Cell[2107935, 83713, 213, 6, 26, "Input",
   CellTags->"TR"],
  Cell[2108151, 83721, 252, 8, 26, "Output",
   CellTags->"TR"],
  Cell[2108440, 83734, 214, 6, 26, "Input",
   CellTags->"TR"],
  Cell[2108657, 83742, 141, 4, 26, "Output",
   CellTags->"TR"],
  Cell[2108835, 83751, 213, 6, 26, "Input",
   CellTags->"TR"],
  Cell[2109051, 83759, 141, 4, 26, "Output",
   CellTags->"TR"],
  Cell[2109580, 83783, 119, 4, 26, "Input",
   CellTags->"TR"]},
 "Info3500974047-4982097"->{
  Cell[2101117, 83437, 421, 7, 79, "Print",
   CellTags->{"Info3500974047-4982097", "TR"}]},
 "TraceDimension"->{
  Cell[2109714, 83790, 65, 1, 39, "ObjectName",
   CellTags->"TraceDimension"],
  Cell[2109804, 83795, 65, 1, 34, "Subsubsection",
   CellTags->"TraceDimension"],
  Cell[2109872, 83798, 142, 4, 24, "Text",
   CellTags->"TraceDimension"],
  Cell[2110017, 83804, 200, 9, 24, "Text",
   CellTags->"TraceDimension"]},
 "TraceOfOne"->{
  Cell[2110232, 83816, 57, 1, 39, "ObjectName",
   CellTags->"TraceOfOne"],
  Cell[2110314, 83821, 61, 1, 34, "Subsubsection",
   CellTags->"TraceOfOne"],
  Cell[2110378, 83824, 149, 4, 24, "Text",
   CellTags->"TraceOfOne"],
  Cell[2110530, 83830, 273, 13, 24, "Text",
   CellTags->"TraceOfOne"],
  Cell[2110840, 83848, 58, 1, 34, "Subsubsection",
   CellTags->"TraceOfOne"],
  Cell[2110923, 83853, 273, 7, 26, "Input",
   CellTags->"TraceOfOne"],
  Cell[2111199, 83862, 151, 4, 26, "Output",
   CellTags->"TraceOfOne"],
  Cell[2111387, 83871, 272, 7, 26, "Input",
   CellTags->"TraceOfOne"],
  Cell[2111662, 83880, 153, 4, 26, "Output",
   CellTags->"TraceOfOne"]},
 "Trick"->{
  Cell[2111842, 83888, 47, 1, 39, "ObjectName",
   CellTags->"Trick"],
  Cell[2111914, 83893, 56, 1, 34, "Subsubsection",
   CellTags->"Trick"],
  Cell[2111973, 83896, 174, 4, 24, "Text",
   CellTags->"Trick"],
  Cell[2112150, 83902, 625, 29, 24, "Text",
   CellTags->"Trick"],
  Cell[2112812, 83936, 62, 1, 33, "Subsubsection",
   CellTags->"Trick"],
  Cell[2112877, 83939, 376, 15, 27, "Text",
   CellTags->"Trick"],
  Cell[2113278, 83958, 350, 11, 26, "Input",
   CellTags->"Trick"],
  Cell[2113631, 83971, 306, 11, 26, "Output",
   CellTags->"Trick"],
  Cell[2113974, 83987, 330, 11, 26, "Input",
   CellTags->"Trick"],
  Cell[2114307, 84000, 468, 17, 28, "Output",
   CellTags->"Trick"],
  Cell[2114812, 84022, 104, 3, 26, "Input",
   CellTags->"Trick"],
  Cell[2114919, 84027, 569, 23, 28, "Output",
   CellTags->"Trick"],
  Cell[2115525, 84055, 227, 6, 26, "Input",
   CellTags->"Trick"],
  Cell[2115755, 84063, 433, 16, 28, "Output",
   CellTags->"Trick"],
  Cell[2116225, 84084, 133, 4, 26, "Input",
   CellTags->"Trick"],
  Cell[2116361, 84090, 257, 7, 38, "Output",
   CellTags->"Trick"]},
 "TrickIntegrate"->{
  Cell[2116645, 84101, 65, 1, 39, "ObjectName",
   CellTags->"TrickIntegrate"],
  Cell[2116735, 84106, 65, 1, 34, "Subsubsection",
   CellTags->"TrickIntegrate"],
  Cell[2116803, 84109, 483, 8, 58, "Text",
   CellTags->"TrickIntegrate"],
  Cell[2117323, 84122, 62, 1, 34, "Subsubsection",
   CellTags->"TrickIntegrate"],
  Cell[2117410, 84127, 364, 12, 26, "Input",
   CellTags->"TrickIntegrate"],
  Cell[2117777, 84141, 561, 19, 29, "Output",
   CellTags->"TrickIntegrate"]},
 "TrickMandelstam"->{
  Cell[2118365, 84164, 67, 1, 39, "ObjectName",
   CellTags->"TrickMandelstam"],
  Cell[2118457, 84169, 66, 1, 34, "Subsubsection",
   CellTags->"TrickMandelstam"],
  Cell[2118526, 84172, 1340, 55, 45, "Text",
   CellTags->"TrickMandelstam"],
  Cell[2119903, 84232, 63, 1, 34, "Subsubsection",
   CellTags->"TrickMandelstam"],
  Cell[2119991, 84237, 528, 18, 30, "Input",
   CellTags->"TrickMandelstam"],
  Cell[2120522, 84257, 294, 10, 29, "Output",
   CellTags->"TrickMandelstam"],
  Cell[2120853, 84272, 568, 17, 26, "Input",
   CellTags->"TrickMandelstam"],
  Cell[2121424, 84291, 277, 9, 26, "Output",
   CellTags->"TrickMandelstam"]},
 "Tr2"->{
  Cell[2121728, 84304, 43, 1, 39, "ObjectName",
   CellTags->"Tr2"],
  Cell[2121796, 84309, 54, 1, 34, "Subsubsection",
   CellTags->"Tr2"],
  Cell[2121853, 84312, 347, 6, 58, "Text",
   CellTags->"Tr2"],
  Cell[2122225, 84322, 106, 3, 26, "Input",
   CellTags->"Tr2"],
  Cell[2122334, 84327, 210, 7, 26, "Output",
   CellTags->"Tr2"],
  Cell[2122559, 84337, 593, 28, 24, "Text",
   CellTags->"Tr2"],
  Cell[2123189, 84370, 51, 1, 34, "Subsubsection",
   CellTags->"Tr2"],
  Cell[2124059, 84407, 187, 6, 26, "Input",
   CellTags->"Tr2"],
  Cell[2124249, 84415, 200, 6, 29, "Output",
   CellTags->"Tr2"]},
 "Twist2AlienOperator"->{
  Cell[2124476, 84425, 116, 2, 39, "ObjectName",
   CellTags->"Twist2AlienOperator"],
  Cell[2124617, 84431, 70, 1, 34, "Subsubsection",
   CellTags->"Twist2AlienOperator"],
  Cell[2124690, 84434, 194, 4, 24, "Text",
   CellTags->"Twist2AlienOperator"],
  Cell[2124921, 84443, 67, 1, 34, "Subsubsection",
   CellTags->"Twist2AlienOperator"],
  Cell[2125013, 84448, 160, 4, 26, "Input",
   CellTags->"Twist2AlienOperator"],
  Cell[2125176, 84454, 929, 31, 52, "Output",
   CellTags->"Twist2AlienOperator"],
  Cell[2126142, 84490, 246, 7, 26, "Input",
   CellTags->"Twist2AlienOperator"],
  Cell[2126391, 84499, 3155, 104, 55, "Output",
   CellTags->"Twist2AlienOperator"]},
 "Twist2CounterOperator"->{
  Cell[2129573, 84607, 120, 2, 39, "ObjectName",
   CellTags->"Twist2CounterOperator"],
  Cell[2129718, 84613, 72, 1, 34, "Subsubsection",
   CellTags->"Twist2CounterOperator"],
  Cell[2129793, 84616, 239, 5, 41, "Text",
   CellTags->"Twist2CounterOperator"],
  Cell[2130069, 84626, 69, 1, 34, "Subsubsection",
   CellTags->"Twist2CounterOperator"],
  Cell[2130163, 84631, 204, 5, 26, "Input",
   CellTags->"Twist2CounterOperator"],
  Cell[2130370, 84638, 4044, 137, 127, "Output",
   CellTags->"Twist2CounterOperator"],
  Cell[2134451, 84780, 159, 4, 26, "Input",
   CellTags->"Twist2CounterOperator"],
  Cell[2134613, 84786, 1064, 36, 52, "Output",
   CellTags->"Twist2CounterOperator"],
  Cell[2135714, 84827, 242, 6, 26, "Input",
   CellTags->"Twist2CounterOperator"],
  Cell[2135959, 84835, 2652, 90, 55, "Output",
   CellTags->"Twist2CounterOperator"]},
 "Twist2GluonOperator"->{
  Cell[2138638, 84929, 75, 1, 39, "ObjectName",
   CellTags->"Twist2GluonOperator"],
  Cell[2138738, 84934, 70, 1, 34, "Subsubsection",
   CellTags->"Twist2GluonOperator"],
  Cell[2138811, 84937, 964, 15, 126, "Text",
   CellTags->"Twist2GluonOperator"],
  Cell[2139778, 84954, 223, 9, 24, "Text",
   CellTags->"Twist2GluonOperator"],
  Cell[2140038, 84968, 99, 1, 34, "Subsubsection",
   CellTags->"Twist2GluonOperator"],
  Cell[2140162, 84973, 352, 8, 46, "Input",
   CellTags->"Twist2GluonOperator"],
  Cell[2140517, 84983, 486, 13, 26, "Output",
   CellTags->"Twist2GluonOperator"],
  Cell[2141018, 84999, 62, 1, 24, "Text",
   CellTags->"Twist2GluonOperator"],
  Cell[2141105, 85004, 271, 8, 26, "Input",
   CellTags->"Twist2GluonOperator"],
  Cell[2141379, 85014, 936, 36, 30, "Output",
   CellTags->"Twist2GluonOperator"],
  Cell[2142352, 85055, 321, 9, 26, "Input",
   CellTags->"Twist2GluonOperator"],
  Cell[2142676, 85066, 1162, 46, 28, "Output",
   CellTags->"Twist2GluonOperator"],
  Cell[2143875, 85117, 221, 6, 26, "Input",
   CellTags->"Twist2GluonOperator"],
  Cell[2144099, 85125, 378, 13, 30, "Output",
   CellTags->"Twist2GluonOperator"],
  Cell[2144514, 85143, 271, 7, 26, "Input",
   CellTags->"Twist2GluonOperator"],
  Cell[2144788, 85152, 747, 29, 28, "Output",
   CellTags->"Twist2GluonOperator"],
  Cell[2145550, 85184, 62, 1, 24, "Text",
   CellTags->"Twist2GluonOperator"],
  Cell[2145637, 85189, 355, 10, 26, "Input",
   CellTags->"Twist2GluonOperator"],
  Cell[2145995, 85201, 1235, 49, 30, "Output",
   CellTags->"Twist2GluonOperator"],
  Cell[2147267, 85255, 325, 10, 26, "Input",
   CellTags->"Twist2GluonOperator"],
  Cell[2147595, 85267, 534, 19, 30, "Output",
   CellTags->"Twist2GluonOperator"],
  Cell[2148166, 85291, 375, 11, 26, "Input",
   CellTags->"Twist2GluonOperator"],
  Cell[2148544, 85304, 11350, 412, 220, "Output",
   CellTags->"Twist2GluonOperator"],
  Cell[2159909, 85719, 62, 1, 24, "Text",
   CellTags->"Twist2GluonOperator"],
  Cell[2159996, 85724, 431, 12, 26, "Input",
   CellTags->"Twist2GluonOperator"],
  Cell[2160430, 85738, 5018, 186, 54, "Output",
   CellTags->"Twist2GluonOperator"],
  Cell[2165485, 85929, 391, 12, 26, "Input",
   CellTags->"Twist2GluonOperator"],
  Cell[2165879, 85943, 648, 23, 30, "Output",
   CellTags->"Twist2GluonOperator"],
  Cell[2166564, 85971, 441, 13, 26, "Input",
   CellTags->"Twist2GluonOperator"],
  Cell[2167008, 85986, 24346, 870, 489, "Output",
   CellTags->"Twist2GluonOperator"],
  Cell[2191403, 86862, 100, 1, 34, "Subsubsection",
   CellTags->"Twist2GluonOperator"],
  Cell[2191528, 86867, 268, 6, 26, "Input",
   CellTags->"Twist2GluonOperator"],
  Cell[2191799, 86875, 486, 13, 26, "Output",
   CellTags->"Twist2GluonOperator"],
  Cell[2192300, 86891, 62, 1, 24, "Text",
   CellTags->"Twist2GluonOperator"],
  Cell[2192387, 86896, 271, 8, 26, "Input",
   CellTags->"Twist2GluonOperator"],
  Cell[2192661, 86906, 946, 37, 44, "Output",
   CellTags->"Twist2GluonOperator"],
  Cell[2193644, 86948, 321, 9, 26, "Input",
   CellTags->"Twist2GluonOperator"],
  Cell[2193968, 86959, 2698, 103, 44, "Output",
   CellTags->"Twist2GluonOperator"],
  Cell[2196703, 87067, 221, 6, 26, "Input",
   CellTags->"Twist2GluonOperator"],
  Cell[2196927, 87075, 380, 13, 30, "Output",
   CellTags->"Twist2GluonOperator"],
  Cell[2197344, 87093, 271, 7, 26, "Input",
   CellTags->"Twist2GluonOperator"],
  Cell[2197618, 87102, 2275, 85, 29, "Output",
   CellTags->"Twist2GluonOperator"],
  Cell[2199908, 87190, 62, 1, 24, "Text",
   CellTags->"Twist2GluonOperator"],
  Cell[2199995, 87195, 355, 10, 26, "Input",
   CellTags->"Twist2GluonOperator"],
  Cell[2200353, 87207, 1311, 51, 44, "Output",
   CellTags->"Twist2GluonOperator"],
  Cell[2201701, 87263, 325, 10, 26, "Input",
   CellTags->"Twist2GluonOperator"],
  Cell[2202029, 87275, 534, 19, 30, "Output",
   CellTags->"Twist2GluonOperator"],
  Cell[2202600, 87299, 375, 11, 26, "Input",
   CellTags->"Twist2GluonOperator"],
  Cell[2202978, 87312, 16386, 588, 239, "Output",
   CellTags->"Twist2GluonOperator"],
  Cell[2219379, 87903, 62, 1, 24, "Text",
   CellTags->"Twist2GluonOperator"],
  Cell[2219466, 87908, 431, 12, 26, "Input",
   CellTags->"Twist2GluonOperator"],
  Cell[2219900, 87922, 5027, 187, 71, "Output",
   CellTags->"Twist2GluonOperator"],
  Cell[2224964, 88114, 391, 12, 26, "Input",
   CellTags->"Twist2GluonOperator"],
  Cell[2225358, 88128, 648, 23, 30, "Output",
   CellTags->"Twist2GluonOperator"],
  Cell[2226043, 88156, 441, 13, 26, "Input",
   CellTags->"Twist2GluonOperator"],
  Cell[2226487, 88171, 31903, 1142, 575, "Output",
   CellTags->"Twist2GluonOperator"],
  Cell[2258439, 89319, 76, 1, 34, "Subsubsection",
   CellTags->"Twist2GluonOperator"],
  Cell[2258518, 89322, 97, 1, 24, "Text",
   CellTags->"Twist2GluonOperator"],
  Cell[2258618, 89325, 295, 7, 26, "Input",
   CellTags->"Twist2GluonOperator"],
  Cell[2258916, 89334, 380, 11, 26, "Input",
   CellTags->"Twist2GluonOperator"],
  Cell[2259333, 89350, 102, 1, 34, "Subsubsection",
   CellTags->"Twist2GluonOperator"],
  Cell[2259460, 89355, 284, 6, 26, "Input",
   CellTags->"Twist2GluonOperator"],
  Cell[2259747, 89363, 485, 13, 26, "Output",
   CellTags->"Twist2GluonOperator"],
  Cell[2260247, 89379, 62, 1, 24, "Text",
   CellTags->"Twist2GluonOperator"],
  Cell[2260334, 89384, 320, 10, 26, "Input",
   CellTags->"Twist2GluonOperator"],
  Cell[2260657, 89396, 3653, 130, 44, "Output",
   CellTags->"Twist2GluonOperator"],
  Cell[2264325, 89529, 62, 1, 24, "Text",
   CellTags->"Twist2GluonOperator"],
  Cell[2264412, 89534, 355, 10, 26, "Input",
   CellTags->"Twist2GluonOperator"],
  Cell[2264770, 89546, 54229, 1691, 1130, "Output",
   CellTags->"Twist2GluonOperator"],
  Cell[2319014, 91240, 62, 1, 24, "Text",
   CellTags->"Twist2GluonOperator"],
  Cell[2319101, 91245, 482, 13, 26, "Input",
   CellTags->"Twist2GluonOperator"],
  Cell[2319586, 91260, 5018, 186, 54, "Output",
   CellTags->"Twist2GluonOperator"],
  Cell[2324653, 91452, 104, 1, 34, "Subsubsection",
   CellTags->"Twist2GluonOperator"],
  Cell[2324782, 91457, 284, 6, 26, "Input",
   CellTags->"Twist2GluonOperator"],
  Cell[2325069, 91465, 485, 13, 26, "Output",
   CellTags->"Twist2GluonOperator"],
  Cell[2325569, 91481, 62, 1, 24, "Text",
   CellTags->"Twist2GluonOperator"],
  Cell[2325656, 91486, 320, 10, 26, "Input",
   CellTags->"Twist2GluonOperator"],
  Cell[2325979, 91498, 3116, 117, 44, "Output",
   CellTags->"Twist2GluonOperator"],
  Cell[2329110, 91618, 62, 1, 24, "Text",
   CellTags->"Twist2GluonOperator"],
  Cell[2329175, 91621, 380, 11, 26, "Input",
   CellTags->"Twist2GluonOperator"],
  Cell[2329580, 91636, 325, 10, 26, "Input",
   CellTags->"Twist2GluonOperator"],
  Cell[2329908, 91648, 3119, 117, 44, "Output",
   CellTags->"Twist2GluonOperator"],
  Cell[2333064, 91770, 375, 11, 26, "Input",
   CellTags->"Twist2GluonOperator"],
  Cell[2333442, 91783, 3117, 117, 44, "Output",
   CellTags->"Twist2GluonOperator"],
  Cell[2336574, 91903, 62, 1, 24, "Text",
   CellTags->"Twist2GluonOperator"],
  Cell[2336661, 91908, 482, 13, 26, "Input",
   CellTags->"Twist2GluonOperator"],
  Cell[2337146, 91923, 5028, 187, 71, "Output",
   CellTags->"Twist2GluonOperator"],
  Cell[2342189, 92113, 78, 1, 24, "Text",
   CellTags->"Twist2GluonOperator"],
  Cell[2342292, 92118, 520, 16, 26, "Input",
   CellTags->"Twist2GluonOperator"],
  Cell[2342815, 92136, 10949, 377, 200, "Output",
   CellTags->"Twist2GluonOperator"],
  Cell[2353779, 92516, 109, 3, 24, "Text",
   CellTags->"Twist2GluonOperator"],
  Cell[2353891, 92521, 295, 7, 26, "Input",
   CellTags->"Twist2GluonOperator"],
  Cell[2354211, 92532, 428, 13, 26, "Input",
   CellTags->"Twist2GluonOperator"],
  Cell[2354642, 92547, 11788, 376, 288, "Output",
   CellTags->"Twist2GluonOperator"]},
 "Twist2QuarkOperator"->{
  Cell[2366457, 92927, 75, 1, 39, "ObjectName",
   CellTags->"Twist2QuarkOperator"],
  Cell[2366557, 92932, 70, 1, 34, "Subsubsection",
   CellTags->"Twist2QuarkOperator"],
  Cell[2366630, 92935, 980, 15, 126, "Text",
   CellTags->"Twist2QuarkOperator"],
  Cell[2367613, 92952, 223, 9, 24, "Text",
   CellTags->"Twist2QuarkOperator"],
  Cell[2367851, 92964, 67, 1, 34, "Subsubsection",
   CellTags->"Twist2QuarkOperator"],
  Cell[2367943, 92969, 98, 1, 34, "Subsubsection",
   CellTags->"Twist2QuarkOperator"],
  Cell[2368066, 92974, 268, 6, 26, "Input",
   CellTags->"Twist2QuarkOperator"],
  Cell[2368337, 92982, 487, 13, 26, "Output",
   CellTags->"Twist2QuarkOperator"],
  Cell[2368839, 92998, 75, 1, 24, "Text",
   CellTags->"Twist2QuarkOperator"],
  Cell[2368939, 93003, 156, 4, 26, "Input",
   CellTags->"Twist2QuarkOperator"],
  Cell[2369098, 93009, 642, 22, 28, "Output",
   CellTags->"Twist2QuarkOperator"],
  Cell[2369755, 93034, 81, 1, 24, "Text",
   CellTags->"Twist2QuarkOperator"],
  Cell[2369861, 93039, 305, 9, 26, "Input",
   CellTags->"Twist2QuarkOperator"],
  Cell[2370169, 93050, 1625, 58, 57, "Output",
   CellTags->"Twist2QuarkOperator"],
  Cell[2371809, 93111, 87, 1, 24, "Text",
   CellTags->"Twist2QuarkOperator"],
  Cell[2371921, 93116, 380, 11, 26, "Input",
   CellTags->"Twist2QuarkOperator"],
  Cell[2372304, 93129, 4959, 164, 118, "Output",
   CellTags->"Twist2QuarkOperator"],
  Cell[2377312, 93299, 100, 1, 34, "Subsubsection",
   CellTags->"Twist2QuarkOperator"],
  Cell[2377437, 93304, 273, 6, 26, "Input",
   CellTags->"Twist2QuarkOperator"],
  Cell[2377713, 93312, 485, 13, 26, "Output",
   CellTags->"Twist2QuarkOperator"],
  Cell[2378213, 93328, 75, 1, 24, "Text",
   CellTags->"Twist2QuarkOperator"],
  Cell[2378313, 93333, 156, 4, 26, "Input",
   CellTags->"Twist2QuarkOperator"],
  Cell[2378472, 93339, 523, 18, 28, "Output",
   CellTags->"Twist2QuarkOperator"],
  Cell[2379010, 93360, 81, 1, 24, "Text",
   CellTags->"Twist2QuarkOperator"],
  Cell[2379116, 93365, 305, 9, 26, "Input",
   CellTags->"Twist2QuarkOperator"],
  Cell[2379424, 93376, 1536, 55, 57, "Output",
   CellTags->"Twist2QuarkOperator"],
  Cell[2380975, 93434, 87, 1, 24, "Text",
   CellTags->"Twist2QuarkOperator"],
  Cell[2381087, 93439, 380, 11, 26, "Input",
   CellTags->"Twist2QuarkOperator"],
  Cell[2381470, 93452, 4816, 160, 114, "Output",
   CellTags->"Twist2QuarkOperator"],
  Cell[2386301, 93615, 93, 1, 24, "Text",
   CellTags->"Twist2QuarkOperator"],
  Cell[2386419, 93620, 197, 6, 26, "Input",
   CellTags->"Twist2QuarkOperator"],
  Cell[2386619, 93628, 305, 9, 38, "Output",
   CellTags->"Twist2QuarkOperator"],
  Cell[2386961, 93642, 376, 10, 26, "Input",
   CellTags->"Twist2QuarkOperator"],
  Cell[2387340, 93654, 2013, 73, 52, "Output",
   CellTags->"Twist2QuarkOperator"],
  Cell[2389390, 93732, 449, 12, 26, "Input",
   CellTags->"Twist2QuarkOperator"],
  Cell[2389842, 93746, 9449, 299, 127, "Output",
   CellTags->"Twist2QuarkOperator"],
  Cell[2399340, 94051, 102, 1, 34, "Subsubsection",
   CellTags->"Twist2QuarkOperator"],
  Cell[2399467, 94056, 269, 6, 26, "Input",
   CellTags->"Twist2QuarkOperator"],
  Cell[2399739, 94064, 488, 13, 26, "Output",
   CellTags->"Twist2QuarkOperator"],
  Cell[2400242, 94080, 178, 4, 24, "Text",
   CellTags->"Twist2QuarkOperator"],
  Cell[2400423, 94086, 275, 5, 41, "Text",
   CellTags->"Twist2QuarkOperator"],
  Cell[2400723, 94095, 205, 6, 26, "Input",
   CellTags->"Twist2QuarkOperator"],
  Cell[2400931, 94103, 842, 29, 28, "Output",
   CellTags->"Twist2QuarkOperator"],
  Cell[2401788, 94135, 100, 1, 24, "Text",
   CellTags->"Twist2QuarkOperator"],
  Cell[2401913, 94140, 305, 9, 26, "Input",
   CellTags->"Twist2QuarkOperator"],
  Cell[2402221, 94151, 2347, 79, 57, "Output",
   CellTags->"Twist2QuarkOperator"],
  Cell[2404583, 94233, 87, 1, 24, "Text",
   CellTags->"Twist2QuarkOperator"],
  Cell[2404695, 94238, 156, 4, 26, "Input",
   CellTags->"Twist2QuarkOperator"],
  Cell[2404854, 94244, 1144, 34, 63, "Output",
   CellTags->"Twist2QuarkOperator"],
  Cell[2406047, 94284, 104, 1, 34, "Subsubsection",
   CellTags->"Twist2QuarkOperator"],
  Cell[2406176, 94289, 269, 6, 26, "Input",
   CellTags->"Twist2QuarkOperator"],
  Cell[2406448, 94297, 486, 13, 26, "Output",
   CellTags->"Twist2QuarkOperator"],
  Cell[2406949, 94313, 75, 1, 24, "Text",
   CellTags->"Twist2QuarkOperator"],
  Cell[2407049, 94318, 205, 6, 26, "Input",
   CellTags->"Twist2QuarkOperator"],
  Cell[2407257, 94326, 842, 29, 28, "Output",
   CellTags->"Twist2QuarkOperator"],
  Cell[2408114, 94358, 81, 1, 24, "Text",
   CellTags->"Twist2QuarkOperator"],
  Cell[2408220, 94363, 305, 9, 26, "Input",
   CellTags->"Twist2QuarkOperator"],
  Cell[2408528, 94374, 2347, 79, 57, "Output",
   CellTags->"Twist2QuarkOperator"],
  Cell[2410890, 94456, 210, 6, 26, "Input",
   CellTags->"Twist2QuarkOperator"]},
 "Twist3QuarkOperator"->{
  Cell[2411115, 94465, 75, 1, 39, "ObjectName",
   CellTags->"Twist3QuarkOperator"],
  Cell[2411215, 94470, 70, 1, 34, "Subsubsection",
   CellTags->"Twist3QuarkOperator"],
  Cell[2411288, 94473, 768, 12, 109, "Text",
   CellTags->"Twist3QuarkOperator"],
  Cell[2412081, 94489, 134, 3, 26, "Input",
   CellTags->"Twist3QuarkOperator"],
  Cell[2412218, 94494, 370, 11, 26, "Output",
   CellTags->"Twist3QuarkOperator"],
  Cell[2412603, 94508, 366, 14, 24, "Text",
   CellTags->"Twist3QuarkOperator"],
  Cell[2413006, 94527, 67, 1, 34, "Subsubsection",
   CellTags->"Twist3QuarkOperator"],
  Cell[2413098, 94532, 132, 3, 26, "Input",
   CellTags->"Twist3QuarkOperator"],
  Cell[2413233, 94537, 696, 24, 28, "Output",
   CellTags->"Twist3QuarkOperator"]},
 "Twist4GluonOperator"->{
  Cell[2413956, 94565, 75, 1, 39, "ObjectName",
   CellTags->"Twist4GluonOperator"],
  Cell[2414056, 94570, 70, 1, 34, "Subsubsection",
   CellTags->"Twist4GluonOperator"],
  Cell[2414129, 94573, 150, 4, 24, "Text",
   CellTags->"Twist4GluonOperator"],
  Cell[2414282, 94579, 509, 19, 24, "Text",
   CellTags->"Twist4GluonOperator"],
  Cell[2414828, 94603, 67, 1, 34, "Subsubsection",
   CellTags->"Twist4GluonOperator"],
  Cell[2414920, 94608, 529, 15, 46, "Input",
   CellTags->"Twist4GluonOperator"],
  Cell[2415452, 94625, 119873, 4589, 1159, "Output",
   CellTags->"Twist4GluonOperator"]},
 "TwoLoopSimplify"->{
  Cell[2535352, 99218, 104, 1, 39, "ObjectName",
   CellTags->"TwoLoopSimplify"],
  Cell[2535481, 99223, 66, 1, 34, "Subsubsection",
   CellTags->"TwoLoopSimplify"],
  Cell[2535550, 99226, 335, 6, 58, "Text",
   CellTags->"TwoLoopSimplify"],
  Cell[2535888, 99234, 207, 9, 24, "Text",
   CellTags->"TwoLoopSimplify"],
  Cell[2536132, 99248, 63, 1, 34, "Subsubsection",
   CellTags->"TwoLoopSimplify"],
  Cell[2536198, 99251, 82, 2, 26, "Input",
   CellTags->"TwoLoopSimplify"]},
 "Uncontract"->{
  Cell[2536295, 99256, 57, 1, 39, "ObjectName",
   CellTags->"Uncontract"],
  Cell[2536377, 99261, 61, 1, 34, "Subsubsection",
   CellTags->"Uncontract"],
  Cell[2536441, 99264, 256, 5, 41, "Text",
   CellTags->"Uncontract"],
  Cell[2536722, 99273, 120, 3, 26, "Input",
   CellTags->"Uncontract"],
  Cell[2536845, 99278, 414, 12, 26, "Output",
   CellTags->"Uncontract"],
  Cell[2537274, 99293, 181, 9, 24, "Text",
   CellTags->"Uncontract"],
  Cell[2537492, 99307, 58, 1, 34, "Subsubsection",
   CellTags->"Uncontract"],
  Cell[2537575, 99312, 234, 7, 26, "Input",
   CellTags->"Uncontract"],
  Cell[2537812, 99321, 441, 18, 26, "Output",
   CellTags->"Uncontract"],
  Cell[2538290, 99344, 143, 4, 26, "Input",
   CellTags->"Uncontract"],
  Cell[2538436, 99350, 823, 32, 28, "Output",
   CellTags->"Uncontract"],
  Cell[2539296, 99387, 138, 4, 26, "Input",
   CellTags->"Uncontract"],
  Cell[2539437, 99393, 266, 9, 28, "Output",
   CellTags->"Uncontract"],
  Cell[2539740, 99407, 143, 4, 26, "Input",
   CellTags->"Uncontract"],
  Cell[2539886, 99413, 583, 21, 28, "Output",
   CellTags->"Uncontract"],
  Cell[2540506, 99439, 153, 4, 26, "Input",
   CellTags->"Uncontract"],
  Cell[2540662, 99445, 1182, 45, 28, "Output",
   CellTags->"Uncontract"],
  Cell[2541859, 99493, 89, 1, 24, "Text",
   CellTags->"Uncontract"],
  Cell[2541973, 99498, 208, 6, 26, "Input",
   CellTags->"Uncontract"],
  Cell[2542184, 99506, 249, 9, 28, "Output",
   CellTags->"Uncontract"],
  Cell[2542448, 99518, 152, 4, 24, "Text",
   CellTags->"Uncontract"],
  Cell[2542625, 99526, 248, 7, 26, "Input",
   CellTags->"Uncontract"],
  Cell[2542876, 99535, 612, 23, 28, "Output",
   CellTags->"Uncontract"],
  Cell[2543525, 99563, 274, 8, 26, "Input",
   CellTags->"Uncontract"],
  Cell[2543802, 99573, 1072, 41, 28, "Output",
   CellTags->"Uncontract"],
  Cell[2544889, 99617, 134, 4, 26, "Input",
   CellTags->"Uncontract"]},
 "UnDeclareNonCommutative"->{
  Cell[2545038, 99624, 83, 1, 39, "ObjectName",
   CellTags->"UnDeclareNonCommutative"],
  Cell[2545146, 99629, 74, 1, 34, "Subsubsection",
   CellTags->"UnDeclareNonCommutative"],
  Cell[2545223, 99632, 198, 4, 24, "Text",
   CellTags->"UnDeclareNonCommutative"],
  Cell[2545424, 99638, 343, 14, 24, "Text",
   CellTags->"UnDeclareNonCommutative"],
  Cell[2545804, 99657, 71, 1, 34, "Subsubsection",
   CellTags->"UnDeclareNonCommutative"],
  Cell[2545878, 99660, 138, 3, 26, "Input",
   CellTags->"UnDeclareNonCommutative"],
  Cell[2546019, 99665, 153, 4, 24, "Text",
   CellTags->"UnDeclareNonCommutative"],
  Cell[2546197, 99673, 107, 2, 26, "Input",
   CellTags->"UnDeclareNonCommutative"],
  Cell[2546307, 99677, 167, 4, 26, "Output",
   CellTags->"UnDeclareNonCommutative"],
  Cell[2546489, 99684, 103, 1, 24, "Text",
   CellTags->"UnDeclareNonCommutative"],
  Cell[2546595, 99687, 140, 3, 26, "Input",
   CellTags->"UnDeclareNonCommutative"],
  Cell[2546760, 99694, 161, 4, 26, "Input",
   CellTags->"UnDeclareNonCommutative"],
  Cell[2546924, 99700, 165, 4, 26, "Output",
   CellTags->"UnDeclareNonCommutative"],
  Cell[2547104, 99707, 161, 4, 26, "Input",
   CellTags->"UnDeclareNonCommutative"],
  Cell[2547290, 99715, 171, 4, 26, "Input",
   CellTags->"UnDeclareNonCommutative"],
  Cell[2547464, 99721, 215, 6, 26, "Output",
   CellTags->"UnDeclareNonCommutative"],
  Cell[2547694, 99730, 163, 4, 26, "Input",
   CellTags->"UnDeclareNonCommutative"],
  Cell[2547882, 99738, 171, 4, 26, "Input",
   CellTags->"UnDeclareNonCommutative"],
  Cell[2548056, 99744, 217, 6, 26, "Output",
   CellTags->"UnDeclareNonCommutative"]},
 "Upper"->{
  Cell[2548300, 99754, 47, 1, 39, "ObjectName",
   CellTags->"Upper"],
  Cell[2548372, 99759, 56, 1, 34, "Subsubsection",
   CellTags->"Upper"],
  Cell[2548431, 99762, 126, 4, 24, "Text",
   CellTags->"Upper"],
  Cell[2548560, 99768, 397, 18, 24, "Text",
   CellTags->"Upper"]},
 "UVPart"->{
  Cell[2548972, 99789, 86, 1, 39, "ObjectName",
   CellTags->"UVPart"],
  Cell[2549083, 99794, 57, 1, 34, "Subsubsection",
   CellTags->"UVPart"],
  Cell[2549143, 99797, 128, 4, 24, "Text",
   CellTags->"UVPart"],
  Cell[2549308, 99806, 54, 1, 34, "Subsubsection",
   CellTags->"UVPart"],
  Cell[2549365, 99809, 73, 2, 26, "Input",
   CellTags->"UVPart"]},
 "Vectors"->{
  Cell[2549453, 99814, 51, 1, 39, "ObjectName",
   CellTags->"Vectors"],
  Cell[2549529, 99819, 58, 1, 34, "Subsubsection",
   CellTags->"Vectors"],
  Cell[2549590, 99822, 203, 4, 41, "Text",
   CellTags->"Vectors"],
  Cell[2549796, 99828, 188, 8, 24, "Text",
   CellTags->"Vectors"]},
 "WriteOut"->{
  Cell[2549999, 99839, 53, 1, 39, "ObjectName",
   CellTags->"WriteOut"],
  Cell[2550077, 99844, 59, 1, 34, "Subsubsection",
   CellTags->"WriteOut"],
  Cell[2550139, 99847, 216, 5, 41, "Text",
   CellTags->"WriteOut"],
  Cell[2550358, 99854, 171, 8, 24, "Text",
   CellTags->"WriteOut"]},
 "WriteOutPaVe"->{
  Cell[2550544, 99865, 61, 1, 39, "ObjectName",
   CellTags->"WriteOutPaVe"],
  Cell[2550630, 99870, 63, 1, 34, "Subsubsection",
   CellTags->"WriteOutPaVe"],
  Cell[2550696, 99873, 260, 5, 41, "Text",
   CellTags->"WriteOutPaVe"],
  Cell[2550959, 99880, 398, 18, 24, "Text",
   CellTags->"WriteOutPaVe"]},
 "Write2"->{
  Cell[2551372, 99901, 49, 1, 39, "ObjectName",
   CellTags->"Write2"],
  Cell[2551446, 99906, 57, 1, 34, "Subsubsection",
   CellTags->"Write2"],
  Cell[2551506, 99909, 297, 6, 41, "Text",
   CellTags->"Write2"],
  Cell[2551806, 99917, 153, 4, 24, "Text",
   CellTags->"Write2"],
  Cell[2551984, 99925, 136, 3, 26, "Input",
   CellTags->"Write2"],
  Cell[2552123, 99930, 290, 10, 39, "Output",
   CellTags->"Write2"],
  Cell[2552450, 99945, 112, 3, 26, "Input",
   CellTags->"Write2"],
  Cell[2552565, 99950, 681, 16, 47, "Output",
   CellTags->"Write2"],
  Cell[2553283, 99971, 115, 3, 26, "Input",
   CellTags->"Write2"],
  Cell[2553401, 99976, 177, 5, 26, "Output",
   CellTags->"Write2"],
  Cell[2553593, 99984, 290, 14, 24, "Text",
   CellTags->"Write2"],
  Cell[2553920, 100003, 54, 1, 34, "Subsubsection",
   CellTags->"Write2"],
  Cell[2553999, 100008, 429, 15, 38, "Input",
   CellTags->"Write2"],
  Cell[2554431, 100025, 896, 30, 32, "Output",
   CellTags->"Write2"],
  Cell[2555342, 100058, 208, 7, 26, "Input",
   CellTags->"Write2"],
  Cell[2555553, 100067, 78, 1, 24, "Text",
   CellTags->"Write2"],
  Cell[2555634, 100070, 184, 6, 26, "Input",
   CellTags->"Write2"],
  Cell[2555821, 100078, 73, 1, 24, "Text",
   CellTags->"Write2"],
  Cell[2555919, 100083, 363, 11, 26, "Input",
   CellTags->"Write2"],
  Cell[2556285, 100096, 843, 23, 73, "Output",
   CellTags->"Write2"],
  Cell[2557143, 100122, 286, 8, 26, "Input",
   CellTags->"Write2"],
  Cell[2557454, 100134, 233, 7, 26, "Input",
   CellTags->"Write2"],
  Cell[2557690, 100143, 752, 28, 26, "Output",
   CellTags->"Write2"],
  Cell[2558457, 100174, 185, 6, 26, "Input",
   CellTags->"Write2"],
  Cell[2558667, 100184, 363, 11, 26, "Input",
   CellTags->"Write2"],
  Cell[2559033, 100197, 1103, 31, 196, "Output",
   CellTags->"Write2"],
  Cell[2560151, 100231, 286, 8, 26, "Input",
   CellTags->"Write2"],
  Cell[2560440, 100241, 106, 3, 24, "Text",
   CellTags->"Write2"],
  Cell[2560549, 100246, 247, 8, 26, "Input",
   CellTags->"Write2"],
  Cell[2560821, 100258, 363, 11, 26, "Input",
   CellTags->"Write2"],
  Cell[2561187, 100271, 996, 26, 128, "Output",
   CellTags->"Write2"],
  Cell[2562198, 100300, 397, 11, 46, "Input",
   CellTags->"Write2"]},
 "XYT"->{
  Cell[2562610, 100314, 43, 1, 39, "ObjectName",
   CellTags->"XYT"],
  Cell[2562678, 100319, 54, 1, 34, "Subsubsection",
   CellTags->"XYT"],
  Cell[2562735, 100322, 215, 9, 41, "Text",
   CellTags->"XYT"]},
 "ZeroMomentumInsertion"->{
  Cell[2562965, 100334, 79, 1, 57, "ObjectName",
   CellTags->"ZeroMomentumInsertion"],
  Cell[2563069, 100339, 72, 1, 34, "Subsubsection",
   CellTags->"ZeroMomentumInsertion"],
  Cell[2563144, 100342, 154, 4, 24, "Text",
   CellTags->"ZeroMomentumInsertion"],
  Cell[2563301, 100348, 473, 18, 24, "Text",
   CellTags->"ZeroMomentumInsertion"]},
 "Zeta2"->{
  Cell[2563789, 100369, 47, 1, 39, "ObjectName",
   CellTags->"Zeta2"],
  Cell[2563861, 100374, 56, 1, 34, "Subsubsection",
   CellTags->"Zeta2"],
  Cell[2563920, 100377, 94, 1, 24, "Text",
   CellTags->"Zeta2"],
  Cell[2564017, 100380, 191, 8, 24, "Text",
   CellTags->"Zeta2"],
  Cell[2564245, 100393, 53, 1, 34, "Subsubsection",
   CellTags->"Zeta2"],
  Cell[2564323, 100398, 77, 2, 26, "Input",
   CellTags->"Zeta2"],
  Cell[2564403, 100402, 178, 5, 26, "Output",
   CellTags->"Zeta2"],
  Cell[2564618, 100412, 104, 3, 26, "Input",
   CellTags->"Zeta2"],
  Cell[2564725, 100417, 164, 4, 26, "Output",
   CellTags->"Zeta2"],
  Cell[2564926, 100426, 138, 4, 26, "Input",
   CellTags->"Zeta2"],
  Cell[2565067, 100432, 201, 6, 26, "Output",
   CellTags->"Zeta2"]},
 "$Abbreviations"->{
  Cell[2565295, 100442, 65, 1, 39, "ObjectName",
   CellTags->"$Abbreviations"],
  Cell[2565385, 100447, 65, 1, 34, "Subsubsection",
   CellTags->"$Abbreviations"],
  Cell[2565453, 100450, 293, 6, 41, "Text",
   CellTags->"$Abbreviations"],
  Cell[2565771, 100460, 129, 3, 26, "Input",
   CellTags->"$Abbreviations"],
  Cell[2565903, 100465, 909, 18, 77, "Output",
   CellTags->"$Abbreviations"],
  Cell[2566827, 100486, 652, 29, 24, "Text",
   CellTags->"$Abbreviations"]},
 "$AL"->{
  Cell[2567494, 100518, 43, 1, 39, "ObjectName",
   CellTags->"$AL"],
  Cell[2567562, 100523, 54, 1, 34, "Subsubsection",
   CellTags->"$AL"],
  Cell[2567619, 100526, 114, 3, 24, "Text",
   CellTags->"$AL"],
  Cell[2567758, 100533, 73, 2, 26, "Input",
   CellTags->"$AL"],
  Cell[2567834, 100537, 146, 4, 26, "Output",
   CellTags->"$AL"],
  Cell[2568017, 100546, 241, 7, 26, "Input",
   CellTags->"$AL"],
  Cell[2568261, 100555, 605, 23, 28, "Output",
   CellTags->"$AL"],
  Cell[2568881, 100581, 117, 4, 26, "Input",
   CellTags->"$AL"],
  Cell[2569023, 100589, 241, 7, 26, "Input",
   CellTags->"$AL"],
  Cell[2569267, 100598, 521, 21, 28, "Output",
   CellTags->"$AL"],
  Cell[2569803, 100622, 109, 4, 26, "Input",
   CellTags->"$AL"]},
 "$BreitMaison"->{
  Cell[2569927, 100629, 61, 1, 39, "ObjectName",
   CellTags->"$BreitMaison"],
  Cell[2570013, 100634, 63, 1, 34, "Subsubsection",
   CellTags->"$BreitMaison"],
  Cell[2570079, 100637, 319, 9, 41, "Text",
   CellTags->"$BreitMaison"],
  Cell[2570401, 100648, 370, 19, 24, "Text",
   CellTags->"$BreitMaison"]},
 "$Color"->{
  Cell[2570786, 100670, 49, 1, 39, "ObjectName",
   CellTags->"$Color"],
  Cell[2570860, 100675, 57, 1, 34, "Subsubsection",
   CellTags->"$Color"],
  Cell[2570920, 100678, 130, 4, 24, "Text",
   CellTags->"$Color"]},
 "$Covariant"->{
  Cell[2571065, 100685, 57, 1, 39, "ObjectName",
   CellTags->"$Covariant"],
  Cell[2571147, 100690, 61, 1, 34, "Subsubsection",
   CellTags->"$Covariant"],
  Cell[2571211, 100693, 183, 4, 24, "Text",
   CellTags->"$Covariant"],
  Cell[2571419, 100701, 87, 2, 26, "Input",
   CellTags->"$Covariant"],
  Cell[2571509, 100705, 155, 4, 26, "Output",
   CellTags->"$Covariant"],
  Cell[2571701, 100714, 110, 3, 26, "Input",
   CellTags->"$Covariant"],
  Cell[2571814, 100719, 184, 5, 26, "Output",
   CellTags->"$Covariant"],
  Cell[2572013, 100727, 130, 4, 26, "Input",
   CellTags->"$Covariant"],
  Cell[2572168, 100735, 110, 3, 26, "Input",
   CellTags->"$Covariant"],
  Cell[2572281, 100740, 182, 5, 29, "Output",
   CellTags->"$Covariant"],
  Cell[2572478, 100748, 131, 4, 26, "Input",
   CellTags->"$Covariant"]},
 "$FCS"->{
  Cell[2572624, 100755, 45, 1, 39, "ObjectName",
   CellTags->"$FCS"],
  Cell[2572694, 100760, 55, 1, 34, "Subsubsection",
   CellTags->"$FCS"],
  Cell[2572752, 100763, 142, 4, 24, "Text",
   CellTags->"$FCS"],
  Cell[2572919, 100771, 75, 2, 26, "Input",
   CellTags->"$FCS"],
  Cell[2572997, 100775, 741, 15, 47, "Output",
   CellTags->"$FCS"]},
 "$FCT"->{
  Cell[2573765, 100794, 45, 1, 39, "ObjectName",
   CellTags->"$FCT"],
  Cell[2573835, 100799, 55, 1, 34, "Subsubsection",
   CellTags->"$FCT"],
  Cell[2573893, 100802, 177, 4, 24, "Text",
   CellTags->"$FCT"],
  Cell[2574073, 100808, 128, 4, 26, "Input",
   CellTags->"$FCT"],
  Cell[2574226, 100816, 170, 5, 26, "Input",
   CellTags->"$FCT"],
  Cell[2574399, 100823, 238, 7, 26, "Output",
   CellTags->"$FCT"],
  Cell[2574652, 100833, 129, 4, 26, "Input",
   CellTags->"$FCT"],
  Cell[2574806, 100841, 170, 5, 26, "Input",
   CellTags->"$FCT"],
  Cell[2574979, 100848, 238, 7, 26, "Output",
   CellTags->"$FCT"]},
 "$FeynCalcStuff"->{
  Cell[2575244, 100859, 65, 1, 39, "ObjectName",
   CellTags->"$FeynCalcStuff"],
  Cell[2575334, 100864, 65, 1, 34, "Subsubsection",
   CellTags->"$FeynCalcStuff"],
  Cell[2575402, 100867, 102, 1, 24, "Text",
   CellTags->"$FeynCalcStuff"]},
 "$FeynCalcVersion"->{
  Cell[2575519, 100871, 69, 1, 39, "ObjectName",
   CellTags->"$FeynCalcVersion"],
  Cell[2575613, 100876, 67, 1, 34, "Subsubsection",
   CellTags->"$FeynCalcVersion"],
  Cell[2575683, 100879, 124, 3, 24, "Text",
   CellTags->"$FeynCalcVersion"],
  Cell[2575832, 100886, 99, 2, 26, "Input",
   CellTags->"$FeynCalcVersion"],
  Cell[2575934, 100890, 172, 4, 26, "Output",
   CellTags->"$FeynCalcVersion"]},
 "$FortranContinuationCharacter"->{
  Cell[2576133, 100898, 95, 1, 39, "ObjectName",
   CellTags->"$FortranContinuationCharacter"],
  Cell[2576253, 100903, 80, 1, 34, "Subsubsection",
   CellTags->"$FortranContinuationCharacter"],
  Cell[2576336, 100906, 143, 3, 24, "Text",
   CellTags->"$FortranContinuationCharacter"],
  Cell[2576504, 100913, 125, 2, 26, "Input",
   CellTags->"$FortranContinuationCharacter"],
  Cell[2576632, 100917, 176, 4, 26, "Output",
   CellTags->"$FortranContinuationCharacter"],
  Cell[2576823, 100924, 194, 9, 24, "Text",
   CellTags->"$FortranContinuationCharacter"]},
 "$Gauge"->{
  Cell[2577032, 100936, 49, 1, 39, "ObjectName",
   CellTags->"$Gauge"],
  Cell[2577106, 100941, 57, 1, 34, "Subsubsection",
   CellTags->"$Gauge"],
  Cell[2577166, 100944, 260, 5, 41, "Text",
   CellTags->"$Gauge"],
  Cell[2577429, 100951, 168, 9, 24, "Text",
   CellTags->"$Gauge"]},
 "$Larin"->{
  Cell[2577612, 100963, 49, 1, 39, "ObjectName",
   CellTags->"$Larin"],
  Cell[2577686, 100968, 57, 1, 34, "Subsubsection",
   CellTags->"$Larin"],
  Cell[2577746, 100971, 1133, 30, 92, "Text",
   CellTags->"$Larin"],
  Cell[2578904, 101005, 79, 2, 26, "Input",
   CellTags->"$Larin"],
  Cell[2578986, 101009, 149, 4, 26, "Output",
   CellTags->"$Larin"],
  Cell[2579150, 101016, 377, 18, 24, "Text",
   CellTags->"$Larin"]},
 "$LimitTo4"->{
  Cell[2579542, 101037, 55, 1, 39, "ObjectName",
   CellTags->"$LimitTo4"],
  Cell[2579622, 101042, 60, 1, 34, "Subsubsection",
   CellTags->"$LimitTo4"],
  Cell[2579685, 101045, 203, 4, 41, "Text",
   CellTags->"$LimitTo4"],
  Cell[2579913, 101053, 85, 2, 26, "Input",
   CellTags->"$LimitTo4"],
  Cell[2580001, 101057, 151, 4, 26, "Output",
   CellTags->"$LimitTo4"],
  Cell[2580167, 101064, 391, 19, 24, "Text",
   CellTags->"$LimitTo4"]},
 "$LorentzIndices"->{
  Cell[2580573, 101086, 67, 1, 39, "ObjectName",
   CellTags->"$LorentzIndices"],
  Cell[2580665, 101091, 66, 1, 34, "Subsubsection",
   CellTags->"$LorentzIndices"],
  Cell[2580734, 101094, 164, 4, 24, "Text",
   CellTags->"$LorentzIndices"],
  Cell[2580901, 101100, 140, 4, 26, "Input",
   CellTags->"$LorentzIndices"],
  Cell[2581066, 101108, 333, 9, 26, "Input",
   CellTags->"$LorentzIndices"],
  Cell[2581402, 101119, 713, 29, 26, "Output",
   CellTags->"$LorentzIndices"],
  Cell[2582152, 101153, 117, 3, 26, "Input",
   CellTags->"$LorentzIndices"],
  Cell[2582272, 101158, 424, 11, 38, "Output",
   CellTags->"$LorentzIndices"],
  Cell[2582733, 101174, 302, 9, 26, "Input",
   CellTags->"$LorentzIndices"],
  Cell[2583038, 101185, 926, 39, 28, "Output",
   CellTags->"$LorentzIndices"],
  Cell[2584001, 101229, 117, 3, 26, "Input",
   CellTags->"$LorentzIndices"],
  Cell[2584121, 101234, 697, 20, 58, "Output",
   CellTags->"$LorentzIndices"],
  Cell[2584833, 101257, 141, 4, 26, "Input",
   CellTags->"$LorentzIndices"]},
 "$MIntegrate"->{
  Cell[2584989, 101264, 59, 1, 39, "ObjectName",
   CellTags->"$MIntegrate"],
  Cell[2585073, 101269, 62, 1, 34, "Subsubsection",
   CellTags->"$MIntegrate"],
  Cell[2585138, 101272, 136, 4, 24, "Text",
   CellTags->"$MIntegrate"],
  Cell[2585299, 101280, 89, 2, 26, "Input",
   CellTags->"$MIntegrate"],
  Cell[2585391, 101284, 167, 5, 26, "Output",
   CellTags->"$MIntegrate"],
  Cell[2585573, 101292, 191, 9, 24, "Text",
   CellTags->"$MIntegrate"]},
 "$MU"->{
  Cell[2585779, 101304, 43, 1, 39, "ObjectName",
   CellTags->"$MU"],
  Cell[2585847, 101309, 54, 1, 34, "Subsubsection",
   CellTags->"$MU"],
  Cell[2585904, 101312, 151, 4, 24, "Text",
   CellTags->"$MU"],
  Cell[2586080, 101320, 73, 2, 26, "Input",
   CellTags->"$MU"],
  Cell[2586156, 101324, 144, 4, 26, "Output",
   CellTags->"$MU"],
  Cell[2586315, 101331, 174, 9, 24, "Text",
   CellTags->"$MU"]},
 "$MemoryAvailable"->{
  Cell[2586504, 101343, 69, 1, 39, "ObjectName",
   CellTags->"$MemoryAvailable"],
  Cell[2586598, 101348, 67, 1, 34, "Subsubsection",
   CellTags->"$MemoryAvailable"],
  Cell[2586668, 101351, 434, 11, 58, "Text",
   CellTags->"$MemoryAvailable"],
  Cell[2587127, 101366, 99, 2, 26, "Input",
   CellTags->"$MemoryAvailable"],
  Cell[2587229, 101370, 157, 4, 26, "Output",
   CellTags->"$MemoryAvailable"]},
 "$NonComm"->{
  Cell[2587413, 101378, 53, 1, 39, "ObjectName",
   CellTags->"$NonComm"],
  Cell[2587491, 101383, 59, 1, 34, "Subsubsection",
   CellTags->"$NonComm"],
  Cell[2587553, 101386, 109, 3, 24, "Text",
   CellTags->"$NonComm"],
  Cell[2587687, 101393, 83, 2, 26, "Input",
   CellTags->"$NonComm"],
  Cell[2587773, 101397, 1124, 20, 147, "Output",
   CellTags->"$NonComm"],
  Cell[2588912, 101420, 307, 14, 24, "Text",
   CellTags->"$NonComm"]},
 "$PairBrackets"->{
  Cell[2589234, 101437, 63, 1, 39, "ObjectName",
   CellTags->"$PairBrackets"],
  Cell[2589322, 101442, 64, 1, 34, "Subsubsection",
   CellTags->"$PairBrackets"],
  Cell[2589389, 101445, 155, 4, 24, "Text",
   CellTags->"$PairBrackets"],
  Cell[2589569, 101453, 93, 2, 26, "Input",
   CellTags->"$PairBrackets"],
  Cell[2589665, 101457, 156, 4, 26, "Output",
   CellTags->"$PairBrackets"],
  Cell[2589858, 101466, 204, 7, 26, "Input",
   CellTags->"$PairBrackets"],
  Cell[2590065, 101475, 466, 19, 28, "Output",
   CellTags->"$PairBrackets"],
  Cell[2590568, 101499, 118, 3, 26, "Input",
   CellTags->"$PairBrackets"],
  Cell[2590689, 101504, 155, 4, 26, "Output",
   CellTags->"$PairBrackets"],
  Cell[2590881, 101513, 204, 7, 26, "Input",
   CellTags->"$PairBrackets"],
  Cell[2591088, 101522, 489, 19, 28, "Output",
   CellTags->"$PairBrackets"],
  Cell[2591614, 101546, 119, 3, 26, "Input",
   CellTags->"$PairBrackets"],
  Cell[2591736, 101551, 158, 4, 26, "Output",
   CellTags->"$PairBrackets"]},
 "$SpinorMinimal"->{
  Cell[2591921, 101559, 65, 1, 39, "ObjectName",
   CellTags->"$SpinorMinimal"],
  Cell[2592011, 101564, 65, 1, 34, "Subsubsection",
   CellTags->"$SpinorMinimal"],
  Cell[2592079, 101567, 241, 5, 41, "Text",
   CellTags->"$SpinorMinimal"],
  Cell[2592345, 101576, 95, 2, 26, "Input",
   CellTags->"$SpinorMinimal"],
  Cell[2592443, 101580, 157, 4, 26, "Output",
   CellTags->"$SpinorMinimal"],
  Cell[2592615, 101587, 200, 9, 24, "Text",
   CellTags->"$SpinorMinimal"]},
 "$VeryVerbose"->{
  Cell[2592830, 101599, 61, 1, 39, "ObjectName",
   CellTags->"$VeryVerbose"],
  Cell[2592916, 101604, 63, 1, 34, "Subsubsection",
   CellTags->"$VeryVerbose"],
  Cell[2592982, 101607, 221, 5, 41, "Text",
   CellTags->"$VeryVerbose"],
  Cell[2593228, 101616, 91, 2, 26, "Input",
   CellTags->"$VeryVerbose"],
  Cell[2593322, 101620, 150, 4, 26, "Output",
   CellTags->"$VeryVerbose"],
  Cell[2593487, 101627, 141, 4, 26, "Input",
   CellTags->"$VeryVerbose"],
  Cell[2593653, 101635, 287, 10, 26, "Input",
   CellTags->"$VeryVerbose"],
  Cell[2593943, 101647, 165, 4, 21, "Print",
   CellTags->"$VeryVerbose"],
  Cell[2594111, 101653, 297, 8, 22, "Print",
   CellTags->"$VeryVerbose"],
  Cell[2594411, 101663, 317, 8, 22, "Print",
   CellTags->"$VeryVerbose"],
  Cell[2594743, 101674, 131, 4, 26, "Input",
   CellTags->"$VeryVerbose"]},
 "$V0"->{
  Cell[2594877, 101680, 159, 9, 24, "Text",
   CellTags->"$V0"]},
 "$West"->{
  Cell[2595051, 101692, 47, 1, 39, "ObjectName",
   CellTags->"$West"],
  Cell[2595123, 101697, 56, 1, 34, "Subsubsection",
   CellTags->"$West"],
  Cell[2595182, 101700, 468, 13, 41, "Text",
   CellTags->"$West"],
  Cell[2595653, 101715, 158, 8, 24, "Text",
   CellTags->"$West"]},
 "Experimental"->{
  Cell[2597004, 101780, 70, 1, 64, "Section",
   CellTags->"Experimental"],
  Cell[2618484, 102556, 122, 3, 24, "Text",
   CellTags->"Experimental"],
  Cell[2618631, 102563, 198, 6, 26, "Input",
   CellTags->"Experimental"],
  Cell[2618832, 102571, 234, 7, 26, "Output",
   CellTags->"Experimental"],
  Cell[2619081, 102581, 100, 1, 24, "Text",
   CellTags->"Experimental"],
  Cell[2619206, 102586, 222, 6, 26, "Input",
   CellTags->"Experimental"],
  Cell[2619431, 102594, 288, 8, 26, "Output",
   CellTags->"Experimental"]},
 "DoPolarizationSums"->{
  Cell[2597077, 101783, 73, 1, 39, "ObjectName",
   CellTags->"DoPolarizationSums"],
  Cell[2597175, 101788, 69, 1, 34, "Subsubsection",
   CellTags->"DoPolarizationSums"],
  Cell[2597247, 101791, 291, 7, 99, "Text",
   CellTags->"DoPolarizationSums"],
  Cell[2597541, 101800, 317, 14, 24, "Text",
   CellTags->"DoPolarizationSums"],
  Cell[2597895, 101819, 66, 1, 34, "Subsubsection",
   CellTags->"DoPolarizationSums"],
  Cell[2597964, 101822, 310, 6, 41, "Text",
   CellTags->"DoPolarizationSums"],
  Cell[2598277, 101830, 123, 3, 24, "Text",
   CellTags->"DoPolarizationSums"],
  Cell[2598425, 101837, 571, 17, 46, "Input",
   CellTags->"DoPolarizationSums"],
  Cell[2598999, 101856, 492, 17, 30, "Output",
   CellTags->"DoPolarizationSums"],
  Cell[2599528, 101878, 116, 3, 26, "Input",
   CellTags->"DoPolarizationSums"],
  Cell[2599647, 101883, 175, 5, 26, "Output",
   CellTags->"DoPolarizationSums"],
  Cell[2599837, 101891, 1270, 48, 27, "Text",
   CellTags->"DoPolarizationSums"],
  Cell[2601110, 101941, 562, 21, 24, "Text",
   CellTags->"DoPolarizationSums"],
  Cell[2601675, 101964, 142, 4, 26, "Input",
   CellTags->"DoPolarizationSums"],
  Cell[2601820, 101970, 311, 10, 26, "Input",
   CellTags->"DoPolarizationSums"],
  Cell[2602156, 101984, 271, 8, 26, "Input",
   CellTags->"DoPolarizationSums"],
  Cell[2602430, 101994, 779, 21, 75, "Output",
   CellTags->"DoPolarizationSums"],
  Cell[2603224, 102018, 213, 9, 24, "Text",
   CellTags->"DoPolarizationSums"],
  Cell[2603440, 102029, 276, 9, 26, "Input",
   CellTags->"DoPolarizationSums"],
  Cell[2603719, 102040, 328, 10, 26, "Input",
   CellTags->"DoPolarizationSums"],
  Cell[2604050, 102052, 323, 13, 27, "Text",
   CellTags->"DoPolarizationSums"],
  Cell[2604376, 102067, 697, 30, 27, "Text",
   CellTags->"DoPolarizationSums"],
  Cell[2605098, 102101, 434, 13, 56, "Input",
   CellTags->"DoPolarizationSums"],
  Cell[2605535, 102116, 817, 25, 79, "Output",
   CellTags->"DoPolarizationSums"],
  Cell[2606367, 102144, 624, 27, 27, "Text",
   CellTags->"DoPolarizationSums"],
  Cell[2607016, 102175, 433, 13, 56, "Input",
   CellTags->"DoPolarizationSums"],
  Cell[2607452, 102190, 817, 25, 79, "Output",
   CellTags->"DoPolarizationSums"],
  Cell[2608306, 102220, 859, 24, 65, "Input",
   CellTags->"DoPolarizationSums"],
  Cell[2609168, 102246, 648, 23, 30, "Output",
   CellTags->"DoPolarizationSums"],
  Cell[2609853, 102274, 130, 3, 26, "Input",
   CellTags->"DoPolarizationSums"],
  Cell[2609986, 102279, 365, 13, 29, "Output",
   CellTags->"DoPolarizationSums"],
  Cell[2610388, 102297, 116, 3, 26, "Input",
   CellTags->"DoPolarizationSums"],
  Cell[2610507, 102302, 175, 5, 26, "Output",
   CellTags->"DoPolarizationSums"],
  Cell[2610719, 102312, 120, 3, 26, "Input",
   CellTags->"DoPolarizationSums"],
  Cell[2610842, 102317, 175, 5, 26, "Output",
   CellTags->"DoPolarizationSums"],
  Cell[2611054, 102327, 126, 3, 26, "Input",
   CellTags->"DoPolarizationSums"],
  Cell[2611183, 102332, 175, 5, 26, "Output",
   CellTags->"DoPolarizationSums"],
  Cell[2611373, 102340, 167, 4, 26, "Input",
   CellTags->"DoPolarizationSums"]},
 "FeynmanDoIntegrals"->{
  Cell[2611555, 102347, 90, 1, 39, "ObjectName",
   CellTags->"FeynmanDoIntegrals"],
  Cell[2611670, 102352, 69, 1, 34, "Subsubsection",
   CellTags->"FeynmanDoIntegrals"],
  Cell[2611742, 102355, 1518, 23, 202, "Text",
   CellTags->"FeynmanDoIntegrals"],
  Cell[2613263, 102380, 347, 14, 24, "Text",
   CellTags->"FeynmanDoIntegrals"],
  Cell[2613635, 102398, 136, 3, 26, "Input",
   CellTags->"FeynmanDoIntegrals"],
  Cell[2613774, 102403, 1079, 30, 67, "Output",
   CellTags->"FeynmanDoIntegrals"],
  Cell[2614902, 102439, 66, 1, 34, "Subsubsection",
   CellTags->"FeynmanDoIntegrals"],
  Cell[2614971, 102442, 85, 2, 26, "Input",
   CellTags->"FeynmanDoIntegrals"]},
 "FeynmanParametrize1"->{
  Cell[2615071, 102447, 92, 1, 39, "ObjectName",
   CellTags->"FeynmanParametrize1"],
  Cell[2615188, 102452, 70, 1, 34, "Subsubsection",
   CellTags->"FeynmanParametrize1"],
  Cell[2615261, 102455, 717, 12, 117, "Text",
   CellTags->"FeynmanParametrize1"],
  Cell[2616003, 102471, 138, 3, 26, "Input",
   CellTags->"FeynmanParametrize1"],
  Cell[2616144, 102476, 547, 14, 47, "Output",
   CellTags->"FeynmanParametrize1"],
  Cell[2616740, 102496, 67, 1, 34, "Subsubsection",
   CellTags->"FeynmanParametrize1"],
  Cell[2616810, 102499, 86, 2, 26, "Input",
   CellTags->"FeynmanParametrize1"]},
 "FeynmanReduce"->{
  Cell[2616911, 102504, 80, 1, 39, "ObjectName",
   CellTags->"FeynmanReduce"],
  Cell[2617016, 102509, 64, 1, 34, "Subsubsection",
   CellTags->"FeynmanReduce"],
  Cell[2617083, 102512, 723, 12, 117, "Text",
   CellTags->"FeynmanReduce"],
  Cell[2617831, 102528, 126, 3, 26, "Input",
   CellTags->"FeynmanReduce"],
  Cell[2617960, 102533, 328, 10, 26, "Output",
   CellTags->"FeynmanReduce"],
  Cell[2618337, 102549, 61, 1, 34, "Subsubsection",
   CellTags->"FeynmanReduce"],
  Cell[2618401, 102552, 80, 2, 26, "Input",
   CellTags->"FeynmanReduce"]}
 }
*)
(*CellTagsIndex
CellTagsIndex->{
 {"ClickSave", 2621650, 102670},
 {"Functions", 2621741, 102673},
 {"Abbreviation", 2621827, 102676},
 {"Amplitude", 2622371, 102692},
 {"Amputate", 2623543, 102727},
 {"AnomalousDimension", 2624329, 102750},
 {"AntiCommutator", 2627705, 102837},
 {"AntiQuarkField", 2629797, 102894},
 {"AntiSymmetrize", 2630347, 102909},
 {"Anti5", 2631037, 102928},
 {"Apart2", 2632684, 102979},
 {"A0", 2633433, 103002},
 {"A0ToB0", 2634654, 103042},
 {"BackgroundGluonVertex", 2635110, 103056},
 {"Bracket", 2636368, 103087},
 {"BReduce", 2636659, 103096},
 {"B0", 2637449, 103120},
 {"B0Real", 2638479, 103153},
 {"B0Unique", 2639394, 103181},
 {"B00", 2640330, 103209},
 {"B1", 2641629, 103250},
 {"B11", 2643037, 103295},
 {"CA", 2644592, 103344},
 {"Calc", 2645304, 103367},
 {"CalcColorFactor", 2646572, 103406},
 {"Collecting", 2646900, 103415},
 {"CalculateCounterTerm", 2647292, 103426},
 {"Info3500973745-4982097", 2647847, 103439},
 {"CancelQP", 2647971, 103442},
 {"Cases2", 2649184, 103477},
 {"CF", 2650015, 103502},
 {"ChangeDimension", 2650864, 103529},
 {"ChargeConjugationMatrix", 2653625, 103602},
 {"ChargeConjugationMatrixInv", 2654927, 103633},
 {"CheckContext", 2656606, 103672},
 {"CheckDB", 2657573, 103699},
 {"ChiralityProjector", 2658354, 103722},
 {"Chisholm", 2659250, 103745},
 {"ChisholmSpinor", 2661634, 103814},
 {"ClearScalarProducts", 2662359, 103833},
 {"Collect2", 2663516, 103862},
 {"Collect3", 2665226, 103911},
 {"Combine", 2666238, 103940},
 {"CombineGraphs", 2667303, 103971},
 {"Commutator", 2667625, 103980},
 {"CommutatorExplicit", 2669165, 104023},
 {"CompleteSquare", 2670555, 104058},
 {"ComplexConjugate", 2671428, 104081},
 {"Contract", 2673478, 104134},
 {"Contract1", 2676852, 104231},
 {"Convolute", 2677804, 104258},
 {"ConvoluteTable", 2682026, 104377},
 {"CovariantFieldDerivative", 2683055, 104404},
 {"CounterT", 2683685, 104419},
 {"CounterTerm", 2684208, 104434},
 {"CouplingConstant", 2684756, 104449},
 {"CovariantD", 2685094, 104458},
 {"CrossProduct", 2688149, 104543},
 {"C0", 2688689, 104558},
 {"DataType", 2689420, 104581},
 {"DB0", 2692291, 104664},
 {"DB1", 2692768, 104679},
 {"DeclareNonCommutative", 2693265, 104694},
 {"DeltaFunction", 2694535, 104725},
 {"DeltaFunctionDoublePrime", 2695702, 104756},
 {"DeltaFunctionPrime", 2696077, 104765},
 {"DenominatorOrder", 2696992, 104788},
 {"Dimension", 2697327, 104797},
 {"DimensionalReduction", 2698563, 104832},
 {"DiracBasis", 2698915, 104841},
 {"DiracCanonical", 2699230, 104850},
 {"DiracEquation", 2699560, 104859},
 {"DiracGamma", 2700263, 104878},
 {"DiracGammaCombine", 2702240, 104933},
 {"DiracGammaExpand", 2703459, 104964},
 {"DiracGammaT", 2705194, 105009},
 {"DiracMatrix", 2706024, 105032},
 {"DiracOrder", 2707946, 105085},
 {"DiracReduce", 2710198, 105148},
 {"DiracSigma", 2711975, 105197},
 {"DiracSigmaExplicit", 2713160, 105230},
 {"DiracSimpCombine", 2713914, 105249},
 {"DiracSimplify", 2714252, 105258},
 {"DiracSimplify2", 2718022, 105359},
 {"DiracSlash", 2718735, 105378},
 {"DiracSpinor", 2720200, 105419},
 {"DiracSubstitute67", 2720522, 105428},
 {"DiracTrace", 2720862, 105437},
 {"DiracTraceEvaluate", 2722626, 105486},
 {"DiracTrick", 2722970, 105495},
 {"Divideout", 2725577, 105568},
 {"DOT", 2725878, 105577},
 {"DotExpand", 2726160, 105586},
 {"DotPower", 2727111, 105613},
 {"DotProduct", 2727414, 105622},
 {"DotSimplifyRelations", 2727958, 105637},
 {"DotSimplify", 2728310, 105646},
 {"DummyIndex", 2730957, 105719},
 {"D0", 2731260, 105728},
 {"D0Convention", 2731998, 105751},
 {"Eps", 2732311, 105760},
 {"EpsChisholm", 2734678, 105833},
 {"EpsContract", 2735366, 105852},
 {"EpsDiscard", 2736200, 105875},
 {"EpsEvaluate", 2736512, 105884},
 {"EpsUncontract", 2737349, 105907},
 {"Epsilon", 2737670, 105916},
 {"EpsilonOrder", 2738387, 105937},
 {"Expanding", 2738707, 105946},
 {"ExpandPartialD", 2739019, 105955},
 {"ScalarProductCancel", 2741271, 106014},
 {"ExpandScalarProduct", 2745834, 106111},
 {"Expand2", 2748297, 106158},
 {"Explicit", 2748741, 106171},
 {"ExplicitLorentzIndex", 2749975, 106206},
 {"ExplicitPartialD", 2750747, 106225},
 {"ExplicitSUNIndex", 2751952, 106256},
 {"ExtraFactor", 2752528, 106271},
 {"ExtraVariables", 2752847, 106280},
 {"FactorFull", 2753175, 106289},
 {"Factoring", 2753486, 106298},
 {"Factorout", 2753793, 106307},
 {"FactorTime", 2754099, 106316},
 {"Factor1", 2754407, 106325},
 {"Factor2", 2755532, 106358},
 {"FAD", 2756927, 106399},
 {"FC", 2757923, 106430},
 {"FCE", 2758645, 106453},
 {"FCI", 2759963, 106494},
 {"FCIntegral", 2760968, 106525},
 {"FCIntegrate", 2761281, 106534},
 {"FCNIntegrate", 2761598, 106543},
 {"FC2RHI", 2761913, 106552},
 {"FC2TLI", 2762479, 106569},
 {"Info3500973829-4982097", 2763061, 106586},
 {"FDS", 2763156, 106589},
 {"FermionSpinSum", 2763442, 106598},
 {"FeynAmp", 2765745, 106659},
 {"FeynAmpDenominator", 2766606, 106684},
 {"FeynAmpDenominatorCombine", 2767846, 106715},
 {"FeynAmpDenominatorSimplify", 2768846, 106738},
 {"FeynAmpDenominatorSplit", 2771263, 106793},
 {"FeynAmpList", 2772821, 106830},
 {"FeynCalc", 2773205, 106841},
 {"Info3500973837-4982097", 2773618, 106852},
 {"FeynCalcExternal", 2773741, 106855},
 {"FeynCalcForm", 2775253, 106894},
 {"FeynCalcInternal", 2776464, 106927},
 {"FeynCalc2FORM", 2777983, 106966},
 {"FeynCalcToLaTeX", 2780488, 107033},
 {"FeynmanParameterNames", 2780755, 107040},
 {"FeynmanParametrize", 2781119, 107049},
 {"FeynRule", 2781705, 107064},
 {"FI", 2786354, 107197},
 {"FieldDerivative", 2786639, 107206},
 {"FieldStrength", 2787525, 107229},
 {"FinalFunction", 2789142, 107272},
 {"FinalSubstitutions", 2789472, 107281},
 {"FORM", 2789810, 107290},
 {"FORMEpilog", 2790098, 107299},
 {"FORMProlog", 2790410, 107308},
 {"FORM2FeynCalc", 2790725, 107317},
 {"FourDivergence", 2792855, 107374},
 {"FourLaplacian", 2793948, 107403},
 {"FourVector", 2795104, 107434},
 {"FreeIndex", 2796424, 107471},
 {"FreeQ2", 2796728, 107480},
 {"FRH", 2797631, 107507},
 {"FromTFi", 2798500, 107534},
 {"FromTFI", 2799223, 107555},
 {"FUNCTION", 2799307, 107558},
 {"FunctionalD", 2799611, 107567},
 {"FV", 2806395, 107752},
 {"FVD", 2807581, 107789},
 {"GA", 2808783, 107826},
 {"GAD", 2809908, 107861},
 {"GammaEpsilon", 2811059, 107896},
 {"GammaExpand", 2812212, 107927},
 {"Gamma1", 2813050, 107950},
 {"Gamma2", 2813346, 107959},
 {"Gamma3", 2813642, 107968},
 {"Gauge", 2813937, 107977},
 {"GaugeField", 2814233, 107986},
 {"GaugeXi", 2814915, 108005},
 {"GA5", 2815211, 108014},
 {"GGV", 2815823, 108033},
 {"GhostPropagator", 2816114, 108042},
 {"GHP", 2817304, 108073},
 {"GluonField", 2818188, 108100},
 {"GluonGhostVertex", 2818877, 108119},
 {"GluonPropagator", 2819940, 108146},
 {"GluonSelfEnergy", 2822552, 108213},
 {"GluonVertex", 2823445, 108236},
 {"GO", 2825689, 108297},
 {"GP", 2825963, 108306},
 {"GrassmannParity", 2826250, 108315},
 {"GS", 2826577, 108324},
 {"GSD", 2827572, 108355},
 {"Gstrong", 2828586, 108386},
 {"GTI", 2829164, 108403},
 {"GV", 2829442, 108412},
 {"Hill", 2829718, 108421},
 {"HypergeometricAC", 2830625, 108448},
 {"HypergeometricIR", 2832244, 108489},
 {"HypergeometricSE", 2833307, 108516},
 {"HypExplicit", 2834207, 108539},
 {"HypInt", 2835195, 108566},
 {"IFPD", 2835771, 108583},
 {"IFPDOff", 2836058, 108592},
 {"IFPDOn", 2836992, 108619},
 {"IncludePair", 2837293, 108628},
 {"IndexPosition", 2837615, 108637},
 {"Indices", 2837938, 108646},
 {"InitialFunction", 2838246, 108655},
 {"InitialSubstitutions", 2838591, 108664},
 {"InsideDiracTrace", 2838953, 108673},
 {"IntegralTable", 2839296, 108682},
 {"IntegrateByParts", 2839630, 108691},
 {"Integratedx", 2840294, 108708},
 {"Integrate2", 2840614, 108717},
 {"Integrate3", 2845910, 108862},
 {"IntermediateSubstitutions", 2847195, 108897},
 {"InverseMellin", 2847573, 108906},
 {"Isolate", 2851926, 109021},
 {"Info3500973905-4982097", 2853701, 109070},
 {"IsolateHead", 2853916, 109075},
 {"IsolateNames", 2854236, 109084},
 {"IsolatePrint", 2854562, 109093},
 {"IsolateSplit", 2854887, 109102},
 {"KeepOnly", 2855208, 109111},
 {"KK", 2855508, 109120},
 {"Kummer", 2855788, 109129},
 {"Lagrangian", 2861161, 109284},
 {"LC", 2862348, 109317},
 {"LCD", 2863473, 109352},
 {"LeftPartialD", 2864885, 109395},
 {"LeftRightPartialD", 2866353, 109434},
 {"LeftRightPartialD2", 2867756, 109469},
 {"LeviCivita", 2869167, 109504},
 {"LeviCivitaSign", 2871241, 109561},
 {"ListIntegrals", 2871576, 109570},
 {"Li2", 2871978, 109581},
 {"Li3", 2872723, 109604},
 {"Loop", 2873600, 109631},
 {"LorentzIndex", 2873827, 109638},
 {"Lower", 2875489, 109683},
 {"MakeContext", 2875786, 109692},
 {"Mandelstam", 2876106, 109701},
 {"Map2", 2876418, 109710},
 {"MemoryAvailable", 2877319, 109737},
 {"MemSet", 2877651, 109746},
 {"MetricTensor", 2877952, 109755},
 {"MLimit", 2879545, 109798},
 {"Momentum", 2879989, 109811},
 {"MomentumCombine", 2882064, 109870},
 {"MomentumCombine2", 2883497, 109907},
 {"MomentumExpand", 2884243, 109926},
 {"MT", 2885879, 109969},
 {"MTD", 2886872, 110000},
 {"NegativeInteger", 2887892, 110031},
 {"Nf", 2888219, 110040},
 {"Nielsen", 2888499, 110049},
 {"NonCommFreeQ", 2889992, 110092},
 {"NonCommQ", 2890314, 110101},
 {"NonCommutative", 2890695, 110112},
 {"NTerms", 2891023, 110121},
 {"NumberOfMetricTensors", 2891889, 110146},
 {"NumericalFactor", 2892170, 110153},
 {"NumericQ1", 2892428, 110160},
 {"OneLoop", 2893174, 110181},
 {"OneLoopSimplify", 2895170, 110238},
 {"OneLoopSum", 2896848, 110281},
 {"OPE", 2897156, 110290},
 {"OPEDelta", 2897441, 110299},
 {"OPEi", 2898242, 110322},
 {"OPEInsert", 2899409, 110357},
 {"OPEInt", 2899864, 110370},
 {"OPEIntDelta", 2900306, 110383},
 {"OPEIntegrate", 2900780, 110396},
 {"OPEIntegrateDelta", 2901264, 110409},
 {"OPESumExplicit", 2901776, 110422},
 {"OPESum", 2902881, 110451},
 {"OPESumSimplify", 2903884, 110480},
 {"OPE1Loop", 2905068, 110511},
 {"OPE2TID", 2905518, 110524},
 {"OptionsSelect", 2906108, 110541},
 {"Pair", 2906355, 110548},
 {"PairCollect", 2908320, 110607},
 {"PairContract", 2908642, 110616},
 {"PairContract2", 2908969, 110625},
 {"PairContract3", 2909300, 110634},
 {"PartialD", 2909626, 110643},
 {"PartialDRelations", 2910446, 110666},
 {"PartialFourVector", 2911206, 110685},
 {"PartialIntegrate", 2911803, 110700},
 {"PartitHead", 2913178, 110735},
 {"PauliSigma", 2913422, 110742},
 {"PaVe", 2913809, 110753},
 {"PaVeOrderList", 2914574, 110776},
 {"PaVeOrder", 2914901, 110785},
 {"PaVeReduce", 2915726, 110808},
 {"Info3500973951-4982097", 2917772, 110863},
 {"PD", 2917882, 110866},
 {"PlusDistribution", 2918171, 110875},
 {"Polarization", 2919545, 110910},
 {"PolarizationSum", 2921000, 110949},
 {"PolarizationUncontract", 2922212, 110980},
 {"PolarizationVector", 2922584, 110989},
 {"PositiveInteger", 2924800, 111044},
 {"PositiveNumber", 2925139, 111053},
 {"PowerFactor", 2925471, 111062},
 {"PowerSimplify", 2926168, 111081},
 {"Power2", 2927181, 111108},
 {"PostFortranFile", 2927835, 111127},
 {"Prefactor", 2928170, 111136},
 {"PreFortranFile", 2928486, 111145},
 {"PropagatorDenominator", 2928829, 111154},
 {"PropagatorDenominatorExplicit", 2930553, 111195},
 {"QGV", 2931589, 111218},
 {"QO", 2931867, 111227},
 {"QP", 2932141, 111236},
 {"QuantumField", 2932426, 111245},
 {"QuarkField", 2935209, 111320},
 {"QuarkGluonVertex", 2935754, 111335},
 {"QuarkMass", 2937279, 111374},
 {"QuarkPropagator", 2937594, 111383},
 {"ReduceGamma", 2938636, 111410},
 {"ReduceToScalars", 2938960, 111419},
 {"Rename", 2939292, 111428},
 {"RHI", 2939584, 111437},
 {"RHI2FC", 2940005, 111450},
 {"RHM", 2940441, 111463},
 {"RHO", 2940856, 111476},
 {"RHP", 2941272, 111489},
 {"RTL", 2941688, 111502},
 {"RightPartialD", 2942115, 111515},
 {"RussianTrick", 2943288, 111546},
 {"ScalarGluonVertex", 2944604, 111581},
 {"ScalarProduct", 2945277, 111598},
 {"Info3500973967-4982097", 2947136, 111647},
 {"ScaleMu", 2947260, 111650},
 {"Schouten", 2947631, 111661},
 {"SD", 2948298, 111680},
 {"SelectFree", 2949039, 111703},
 {"SelectGraphs", 2950521, 111744},
 {"SelectNotFree", 2950847, 111753},
 {"SelectSplit", 2952462, 111796},
 {"Select1", 2953084, 111813},
 {"Select2", 2953384, 111822},
 {"Series2", 2953684, 111831},
 {"Series3", 2955677, 111888},
 {"SetMandelstam", 2956478, 111911},
 {"SimplifyDeltaFunction", 2956967, 111924},
 {"SimplifyGTI", 2959430, 111983},
 {"SimplifyPolyLog", 2959905, 111996},
 {"Simplify2", 2963288, 112083},
 {"SmallDelta", 2963676, 112094},
 {"SmallEpsilon", 2963921, 112101},
 {"SmallVariable", 2964173, 112108},
 {"SmallVariables", 2964505, 112117},
 {"SMP", 2964829, 112126},
 {"SMVertex", 2965512, 112147},
 {"Smu", 2966104, 112164},
 {"Sn", 2966318, 112171},
 {"SO", 2966531, 112178},
 {"SOD", 2967267, 112201},
 {"Solve2", 2968018, 112224},
 {"Solve3", 2969216, 112259},
 {"SP", 2969936, 112280},
 {"SPC", 2971191, 112319},
 {"SPD", 2971470, 112328},
 {"Spinor", 2972878, 112371},
 {"SpinorCollect", 2974556, 112420},
 {"SpinorUBar", 2974884, 112429},
 {"SpinorU", 2976071, 112462},
 {"SpinorVBar", 2977145, 112493},
 {"SpinorV", 2978262, 112524},
 {"SpinPolarizationSum", 2979345, 112555},
 {"SPL", 2979689, 112564},
 {"SplittingFunction", 2979982, 112573},
 {"StandardMatrixElement", 2984742, 112692},
 {"SubContext", 2985101, 112701},
 {"SubLoop", 2985414, 112710},
 {"SumP", 2985711, 112719},
 {"SumS", 2987004, 112758},
 {"SumT", 2989229, 112825},
 {"SUND", 2991592, 112896},
 {"SUNDelta", 2993359, 112949},
 {"SUNDeltaContract", 2995234, 113002},
 {"SUNF", 2995568, 113011},
 {"SUNFJacobi", 2997871, 113080},
 {"SUNIndex", 2998559, 113099},
 {"SUNIndexRename", 2999371, 113122},
 {"SUNN", 2999697, 113131},
 {"SUNNToCACF", 3000195, 113146},
 {"SUNSimplify", 3000736, 113161},
 {"SUNT", 3005047, 113278},
 {"SUNTrace", 3007210, 113343},
 {"SymbolicSum2", 3009444, 113406},
 {"SymbolicSum3", 3009696, 113413},
 {"Symmetrize", 3009946, 113420},
 {"TBox", 3010626, 113439},
 {"Tbox", 3010876, 113446},
 {"Tdec", 3011126, 113453},
 {"TensorFunction", 3012436, 113492},
 {"Tf", 3014144, 113537},
 {"TFi", 3014422, 113546},
 {"ThreeVector", 3015178, 113569},
 {"TID", 3015490, 113578},
 {"TIDL", 3016638, 113613},
 {"TimedIntegrate", 3018964, 113680},
 {"TLI", 3019906, 113705},
 {"TLIFP", 3020457, 113722},
 {"TLIHYP", 3021023, 113739},
 {"TLI2", 3021595, 113756},
 {"TLI2FC", 3021881, 113765},
 {"ToDistribution", 3022465, 113782},
 {"ToHypergeometric", 3024118, 113825},
 {"ToLarin", 3025178, 113852},
 {"ToTFi", 3025834, 113871},
 {"TR", 3026744, 113898},
 {"Info3500974047-4982097", 3030053, 113999},
 {"TraceDimension", 3030167, 114002},
 {"TraceOfOne", 3030498, 114011},
 {"Trick", 3031178, 114030},
 {"TrickIntegrate", 3032300, 114063},
 {"TrickMandelstam", 3032799, 114076},
 {"Tr2", 3033450, 114093},
 {"Twist2AlienOperator", 3034081, 114112},
 {"Twist2CounterOperator", 3034782, 114129},
 {"Twist2GluonOperator", 3035668, 114150},
 {"Twist2QuarkOperator", 3043471, 114339},
 {"Twist3QuarkOperator", 3048357, 114458},
 {"Twist4GluonOperator", 3049136, 114477},
 {"TwoLoopSimplify", 3049753, 114492},
 {"Uncontract", 3050249, 114505},
 {"UnDeclareNonCommutative", 3052183, 114558},
 {"Upper", 3053830, 114597},
 {"UVPart", 3054122, 114606},
 {"Vectors", 3054493, 114617},
 {"WriteOut", 3054794, 114626},
 {"WriteOutPaVe", 3055103, 114635},
 {"Write2", 3055423, 114644},
 {"XYT", 3057665, 114709},
 {"ZeroMomentumInsertion", 3057902, 114716},
 {"Zeta2", 3058261, 114725},
 {"$Abbreviations", 3059048, 114748},
 {"$AL", 3059533, 114761},
 {"$BreitMaison", 3060293, 114784},
 {"$Color", 3060617, 114793},
 {"$Covariant", 3060852, 114800},
 {"$FCS", 3061679, 114823},
 {"$FCT", 3062037, 114834},
 {"$FeynCalcStuff", 3062674, 114853},
 {"$FeynCalcVersion", 3062939, 114860},
 {"$FortranContinuationCharacter", 3063381, 114871},
 {"$Gauge", 3063957, 114884},
 {"$Larin", 3064256, 114893},
 {"$LimitTo4", 3064699, 114906},
 {"$LorentzIndices", 3065164, 114919},
 {"$MIntegrate", 3066215, 114946},
 {"$MU", 3066679, 114959},
 {"$MemoryAvailable", 3067108, 114972},
 {"$NonComm", 3067530, 114983},
 {"$PairBrackets", 3067990, 114996},
 {"$SpinorMinimal", 3069016, 115023},
 {"$VeryVerbose", 3069507, 115036},
 {"$V0", 3070354, 115059},
 {"$West", 3070432, 115062},
 {"Experimental", 3070734, 115071},
 {"DoPolarizationSums", 3071286, 115086},
 {"FeynmanDoIntegrals", 3074427, 115163},
 {"FeynmanParametrize1", 3075124, 115180},
 {"FeynmanReduce", 3075739, 115195}
 }
*)
(*NotebookFileOutline
Notebook[{
Cell[CellGroupData[{
Cell[567, 22, 94, 2, 103, "Title"],
Cell[664, 26, 87, 1, 64, "Subtitle"],
Cell[754, 29, 123, 3, 64, "Subtitle",
 PageBreakBelow->True],
Cell[880, 34, 80, 1, 52, "Subsubtitle"],
Cell[963, 37, 244, 8, 52, "Subsubtitle"],
Cell[1210, 47, 799, 21, 36, NotebookDefault,
 CellTags->"ClickSave"],
Cell[CellGroupData[{
Cell[2034, 72, 147, 4, 26, "Input"],
Cell[2184, 78, 381, 10, 26, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2602, 93, 52, 1, 64, "Section",
 CellTags->"Functions"],
Cell[2657, 96, 54, 0, 29, "Commentary"],
Cell[2714, 98, 87, 2, 24, "Text"],
Cell[CellGroupData[{
Cell[2826, 104, 122, 3, 26, "Input"],
Cell[2951, 109, 162, 4, 26, "Output"]
}, Open  ]],
Cell[3128, 116, 69, 1, 26, "Input"],
Cell[3200, 119, 205, 4, 17, "Text"],
Cell[3408, 125, 676, 22, 17, "Text"],
Cell[4087, 149, 371, 12, 17, "Text"],
Cell[4461, 163, 196, 4, 17, "Text"],
Cell[4660, 169, 84, 2, 39, "ObjectName",
 PageBreakAbove->True,
 CellTags->"Abbreviation"],
Cell[CellGroupData[{
Cell[4769, 175, 63, 1, 34, "Subsubsection",
 CellTags->"Abbreviation"],
Cell[4835, 178, 329, 6, 58, "Text",
 CellTags->"Abbreviation"],
Cell[5167, 186, 1032, 43, 41, "Text",
 CellTags->"Abbreviation"]
}, Open  ]],
Cell[CellGroupData[{
Cell[6236, 234, 60, 1, 34, "Subsubsection",
 CellTags->"Abbreviation"],
Cell[CellGroupData[{
Cell[6321, 239, 264, 8, 26, "Input",
 CellTags->"Abbreviation"],
Cell[6588, 249, 270, 9, 29, "Output",
 CellTags->"Abbreviation"]
}, Open  ]]
}, Open  ]],
Cell[6885, 262, 55, 1, 39, "ObjectName",
 CellTags->"Amplitude"],
Cell[CellGroupData[{
Cell[6965, 267, 60, 1, 34, "Subsubsection",
 CellTags->"Amplitude"],
Cell[7028, 270, 529, 9, 75, "Text",
 CellTags->"Amplitude"],
Cell[CellGroupData[{
Cell[7582, 283, 115, 3, 26, "Input",
 CellTags->"Amplitude"],
Cell[7700, 288, 372, 11, 26, "Output",
 CellTags->"Amplitude"]
}, Open  ]],
Cell[8087, 302, 177, 9, 24, "Text",
 CellTags->"Amplitude"]
}, Open  ]],
Cell[CellGroupData[{
Cell[8301, 316, 57, 1, 34, "Subsubsection",
 CellTags->"Amplitude"],
Cell[CellGroupData[{
Cell[8383, 321, 133, 4, 26, "Input",
 CellTags->"Amplitude"],
Cell[8519, 327, 148, 4, 26, "Output",
 CellTags->"Amplitude"]
}, Open  ]],
Cell[8682, 334, 93, 1, 24, "Text",
 CellTags->"Amplitude"],
Cell[CellGroupData[{
Cell[8800, 339, 121, 3, 26, "Input",
 CellTags->"Amplitude"],
Cell[8924, 344, 2260, 96, 30, "Output",
 CellTags->"Amplitude"]
}, Open  ]],
Cell[CellGroupData[{
Cell[11221, 445, 108, 3, 26, "Input",
 CellTags->"Amplitude"],
Cell[11332, 450, 5175, 190, 74, "Output",
 CellTags->"Amplitude"]
}, Open  ]],
Cell[16522, 643, 211, 8, 24, "Text",
 CellTags->"Amplitude"],
Cell[CellGroupData[{
Cell[16758, 655, 306, 9, 26, "Input",
 CellTags->"Amplitude"],
Cell[17067, 666, 13571, 442, 159, "Output",
 CellTags->"Amplitude"]
}, Open  ]]
}, Open  ]],
Cell[30665, 1112, 53, 1, 39, "ObjectName",
 CellTags->"Amputate"],
Cell[CellGroupData[{
Cell[30743, 1117, 59, 1, 34, "Subsubsection",
 CellTags->"Amputate"],
Cell[30805, 1120, 215, 5, 41, "Text",
 CellTags->"Amputate"],
Cell[CellGroupData[{
Cell[31045, 1129, 114, 3, 26, "Input",
 CellTags->"Amputate"],
Cell[31162, 1134, 338, 11, 26, "Output",
 CellTags->"Amputate"]
}, Open  ]],
Cell[31515, 1148, 656, 28, 24, "Text",
 CellTags->"Amputate"]
}, Open  ]],
Cell[CellGroupData[{
Cell[32208, 1181, 56, 1, 34, "Subsubsection",
 CellTags->"Amputate"],
Cell[CellGroupData[{
Cell[32289, 1186, 170, 5, 26, "Input",
 CellTags->"Amputate"],
Cell[32462, 1193, 472, 17, 28, "Output",
 CellTags->"Amputate"]
}, Open  ]],
Cell[CellGroupData[{
Cell[32971, 1215, 131, 4, 26, "Input",
 CellTags->"Amputate"],
Cell[33105, 1221, 765, 28, 28, "Output",
 CellTags->"Amputate"]
}, Open  ]]
}, Open  ]],
Cell[33897, 1253, 73, 1, 39, "ObjectName",
 CellTags->"AnomalousDimension"],
Cell[CellGroupData[{
Cell[33995, 1258, 69, 1, 34, "Subsubsection",
 CellTags->"AnomalousDimension"],
Cell[34067, 1261, 143, 4, 24, "Text",
 CellTags->"AnomalousDimension"],
Cell[CellGroupData[{
Cell[34235, 1269, 134, 3, 26, "Input",
 CellTags->"AnomalousDimension"],
Cell[34372, 1274, 296, 9, 26, "Output",
 CellTags->"AnomalousDimension"]
}, Open  ]],
Cell[34683, 1286, 406, 18, 24, "Text",
 CellTags->"AnomalousDimension"]
}, Open  ]],
Cell[CellGroupData[{
Cell[35126, 1309, 66, 1, 34, "Subsubsection",
 CellTags->"AnomalousDimension"],
Cell[35195, 1312, 63, 1, 24, "Text",
 CellTags->"AnomalousDimension"],
Cell[CellGroupData[{
Cell[35283, 1317, 206, 5, 26, "Input",
 CellTags->"AnomalousDimension"],
Cell[35492, 1324, 296, 9, 26, "Output",
 CellTags->"AnomalousDimension"]
}, Open  ]],
Cell[35803, 1336, 241, 9, 30, "Text",
 CellTags->"AnomalousDimension"],
Cell[CellGroupData[{
Cell[36069, 1349, 133, 3, 26, "Input",
 CellTags->"AnomalousDimension"],
Cell[36205, 1354, 504, 18, 45, "Output",
 CellTags->"AnomalousDimension"]
}, Open  ]],
Cell[36724, 1375, 240, 9, 30, "Text",
 CellTags->"AnomalousDimension"],
Cell[CellGroupData[{
Cell[36989, 1388, 132, 3, 26, "Input",
 CellTags->"AnomalousDimension"],
Cell[37124, 1393, 326, 11, 46, "Output",
 CellTags->"AnomalousDimension"]
}, Open  ]],
Cell[37465, 1407, 240, 9, 30, "Text",
 CellTags->"AnomalousDimension"],
Cell[CellGroupData[{
Cell[37730, 1420, 132, 3, 26, "Input",
 CellTags->"AnomalousDimension"],
Cell[37865, 1425, 325, 11, 45, "Output",
 CellTags->"AnomalousDimension"]
}, Open  ]],
Cell[38205, 1439, 240, 9, 30, "Text",
 CellTags->"AnomalousDimension"],
Cell[CellGroupData[{
Cell[38470, 1452, 132, 3, 26, "Input",
 CellTags->"AnomalousDimension"],
Cell[38605, 1457, 636, 22, 46, "Output",
 CellTags->"AnomalousDimension"]
}, Open  ]],
Cell[39256, 1482, 241, 9, 30, "Text",
 CellTags->"AnomalousDimension"],
Cell[CellGroupData[{
Cell[39522, 1495, 133, 3, 26, "Input",
 CellTags->"AnomalousDimension"],
Cell[39658, 1500, 731, 24, 49, "Output",
 CellTags->"AnomalousDimension"]
}, Open  ]],
Cell[40404, 1527, 241, 9, 30, "Text",
 CellTags->"AnomalousDimension"],
Cell[CellGroupData[{
Cell[40670, 1540, 133, 3, 26, "Input",
 CellTags->"AnomalousDimension"],
Cell[40806, 1545, 6650, 219, 305, "Output",
 CellTags->"AnomalousDimension"]
}, Open  ]],
Cell[47471, 1767, 240, 9, 30, "Text",
 CellTags->"AnomalousDimension"],
Cell[CellGroupData[{
Cell[47736, 1780, 132, 3, 26, "Input",
 CellTags->"AnomalousDimension"],
Cell[47871, 1785, 4158, 141, 208, "Output",
 CellTags->"AnomalousDimension"]
}, Open  ]],
Cell[52044, 1929, 240, 9, 30, "Text",
 CellTags->"AnomalousDimension"],
Cell[CellGroupData[{
Cell[52309, 1942, 132, 3, 26, "Input",
 CellTags->"AnomalousDimension"],
Cell[52444, 1947, 6266, 213, 253, "Output",
 CellTags->"AnomalousDimension"]
}, Open  ]],
Cell[58725, 2163, 240, 9, 30, "Text",
 CellTags->"AnomalousDimension"],
Cell[CellGroupData[{
Cell[58990, 2176, 132, 3, 26, "Input",
 CellTags->"AnomalousDimension"],
Cell[59125, 2181, 4954, 167, 244, "Output",
 CellTags->"AnomalousDimension"]
}, Open  ]],
Cell[64094, 2351, 267, 9, 30, "Text",
 CellTags->"AnomalousDimension"],
Cell[CellGroupData[{
Cell[64386, 2364, 132, 3, 26, "Input",
 CellTags->"AnomalousDimension"],
Cell[64521, 2369, 4421, 140, 200, "Output",
 CellTags->"AnomalousDimension"]
}, Open  ]],
Cell[68957, 2512, 326, 10, 30, "Text",
 CellTags->"AnomalousDimension"],
Cell[CellGroupData[{
Cell[69308, 2526, 295, 9, 26, "Input",
 CellTags->"AnomalousDimension"],
Cell[69606, 2537, 281, 8, 26, "Output",
 CellTags->"AnomalousDimension"]
}, Open  ]]
}, Open  ]],
Cell[69914, 2549, 65, 1, 39, "ObjectName",
 CellTags->"AntiCommutator"],
Cell[CellGroupData[{
Cell[70004, 2554, 65, 1, 34, "Subsubsection",
 CellTags->"AntiCommutator"],
Cell[70072, 2557, 145, 4, 24, "Text",
 CellTags->"AntiCommutator"],
Cell[70220, 2563, 599, 24, 24, "Text",
 CellTags->"AntiCommutator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[70856, 2592, 62, 1, 34, "Subsubsection",
 CellTags->"AntiCommutator"],
Cell[70921, 2595, 200, 9, 24, "Text",
 CellTags->"AntiCommutator"],
Cell[71124, 2606, 150, 4, 26, "Input",
 CellTags->"AntiCommutator"],
Cell[CellGroupData[{
Cell[71299, 2614, 143, 4, 26, "Input",
 CellTags->"AntiCommutator"],
Cell[71445, 2620, 205, 6, 26, "Output",
 CellTags->"AntiCommutator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[71687, 2631, 124, 3, 26, "Input",
 CellTags->"AntiCommutator"],
Cell[71814, 2636, 222, 7, 26, "Output",
 CellTags->"AntiCommutator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[72073, 2648, 265, 8, 26, "Input",
 CellTags->"AntiCommutator"],
Cell[72341, 2658, 480, 17, 26, "Output",
 CellTags->"AntiCommutator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[72858, 2680, 258, 8, 26, "Input",
 CellTags->"AntiCommutator"],
Cell[73119, 2690, 362, 12, 26, "Output",
 CellTags->"AntiCommutator"]
}, Open  ]],
Cell[73496, 2705, 218, 6, 38, "Input",
 CellTags->"AntiCommutator"],
Cell[73717, 2713, 104, 1, 24, "Text",
 CellTags->"AntiCommutator"],
Cell[CellGroupData[{
Cell[73846, 2718, 177, 5, 26, "Input",
 CellTags->"AntiCommutator"],
Cell[74026, 2725, 151, 4, 27, "Output",
 CellTags->"AntiCommutator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[74214, 2734, 182, 5, 26, "Input",
 CellTags->"AntiCommutator"],
Cell[74399, 2741, 232, 7, 26, "Output",
 CellTags->"AntiCommutator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[74668, 2753, 250, 8, 38, "Input",
 CellTags->"AntiCommutator"],
Cell[74921, 2763, 174, 5, 26, "Output",
 CellTags->"AntiCommutator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[75132, 2773, 252, 7, 38, "Input",
 CellTags->"AntiCommutator"],
Cell[75387, 2782, 359, 12, 30, "Output",
 CellTags->"AntiCommutator"]
}, Open  ]],
Cell[75761, 2797, 240, 6, 38, "Input",
 CellTags->"AntiCommutator"],
Cell[76004, 2805, 176, 5, 26, "Input",
 CellTags->"AntiCommutator"],
Cell[76183, 2812, 226, 7, 38, "Input",
 CellTags->"AntiCommutator"]
}, Open  ]],
Cell[76424, 2822, 65, 1, 39, "ObjectName",
 CellTags->"AntiQuarkField"],
Cell[CellGroupData[{
Cell[76514, 2827, 65, 1, 34, "Subsubsection",
 CellTags->"AntiQuarkField"],
Cell[76582, 2830, 197, 4, 24, "Text",
 CellTags->"AntiQuarkField"],
Cell[76782, 2836, 313, 14, 24, "Text",
 CellTags->"AntiQuarkField"]
}, Open  ]],
Cell[CellGroupData[{
Cell[77132, 2855, 62, 1, 34, "Subsubsection",
 CellTags->"AntiQuarkField"],
Cell[CellGroupData[{
Cell[77219, 2860, 93, 2, 26, "Input",
 CellTags->"AntiQuarkField"],
Cell[77315, 2864, 179, 5, 26, "Output",
 CellTags->"AntiQuarkField"]
}, Open  ]]
}, Open  ]],
Cell[77521, 2873, 65, 1, 39, "ObjectName",
 CellTags->"AntiSymmetrize"],
Cell[CellGroupData[{
Cell[77611, 2878, 65, 1, 34, "Subsubsection",
 CellTags->"AntiSymmetrize"],
Cell[77679, 2881, 153, 4, 24, "Text",
 CellTags->"AntiSymmetrize"],
Cell[77835, 2887, 185, 8, 24, "Text",
 CellTags->"AntiSymmetrize"]
}, Open  ]],
Cell[CellGroupData[{
Cell[78057, 2900, 62, 1, 34, "Subsubsection",
 CellTags->"AntiSymmetrize"],
Cell[CellGroupData[{
Cell[78144, 2905, 246, 8, 26, "Input",
 CellTags->"AntiSymmetrize"],
Cell[78393, 2915, 361, 12, 44, "Output",
 CellTags->"AntiSymmetrize"]
}, Open  ]],
Cell[CellGroupData[{
Cell[78791, 2932, 266, 8, 26, "Input",
 CellTags->"AntiSymmetrize"],
Cell[79060, 2942, 696, 21, 44, "Output",
 CellTags->"AntiSymmetrize"]
}, Open  ]]
}, Open  ]],
Cell[79783, 2967, 47, 1, 39, "ObjectName",
 CellTags->"Anti5"],
Cell[CellGroupData[{
Cell[79855, 2972, 56, 1, 34, "Subsubsection",
 CellTags->"Anti5"],
Cell[79914, 2975, 487, 16, 41, "Text",
 CellTags->"Anti5"],
Cell[80404, 2993, 163, 7, 24, "Text",
 CellTags->"Anti5"],
Cell[80570, 3002, 423, 19, 24, "Text",
 CellTags->"Anti5"]
}, Open  ]],
Cell[CellGroupData[{
Cell[81030, 3026, 53, 1, 34, "Subsubsection",
 CellTags->"Anti5"],
Cell[CellGroupData[{
Cell[81108, 3031, 154, 5, 26, "Input",
 CellTags->"Anti5"],
Cell[81265, 3038, 228, 7, 26, "Output",
 CellTags->"Anti5"]
}, Open  ]],
Cell[CellGroupData[{
Cell[81530, 3050, 102, 3, 26, "Input",
 CellTags->"Anti5"],
Cell[81635, 3055, 332, 12, 26, "Output",
 CellTags->"Anti5"]
}, Open  ]],
Cell[CellGroupData[{
Cell[82004, 3072, 144, 5, 26, "Input",
 CellTags->"Anti5"],
Cell[82151, 3079, 307, 11, 26, "Output",
 CellTags->"Anti5"]
}, Open  ]],
Cell[CellGroupData[{
Cell[82495, 3095, 194, 6, 26, "Input",
 CellTags->"Anti5"],
Cell[82692, 3103, 362, 10, 26, "Output",
 CellTags->"Anti5"]
}, Open  ]],
Cell[CellGroupData[{
Cell[83091, 3118, 125, 4, 26, "Input",
 CellTags->"Anti5"],
Cell[83219, 3124, 362, 10, 26, "Output",
 CellTags->"Anti5"]
}, Open  ]],
Cell[CellGroupData[{
Cell[83618, 3139, 133, 4, 26, "Input",
 CellTags->"Anti5"],
Cell[83754, 3145, 362, 10, 26, "Output",
 CellTags->"Anti5"]
}, Open  ]],
Cell[CellGroupData[{
Cell[84153, 3160, 151, 5, 26, "Input",
 CellTags->"Anti5"],
Cell[84307, 3167, 364, 10, 26, "Output",
 CellTags->"Anti5"]
}, Open  ]],
Cell[84686, 3180, 351, 14, 24, "Text",
 CellTags->"Anti5"],
Cell[CellGroupData[{
Cell[85062, 3198, 127, 4, 26, "Input",
 CellTags->"Anti5"],
Cell[85192, 3204, 233, 7, 26, "Output",
 CellTags->"Anti5"]
}, Open  ]],
Cell[CellGroupData[{
Cell[85462, 3216, 102, 3, 26, "Input",
 CellTags->"Anti5"],
Cell[85567, 3221, 332, 12, 26, "Output",
 CellTags->"Anti5"]
}, Open  ]]
}, Open  ]],
Cell[85926, 3237, 49, 1, 39, "ObjectName",
 CellTags->"Apart2"],
Cell[CellGroupData[{
Cell[86000, 3242, 57, 1, 34, "Subsubsection",
 CellTags->"Apart2"],
Cell[86060, 3245, 108, 3, 24, "Text",
 CellTags->"Apart2"],
Cell[86171, 3250, 302, 14, 24, "Text",
 CellTags->"Apart2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[86510, 3269, 54, 1, 34, "Subsubsection",
 CellTags->"Apart2"],
Cell[CellGroupData[{
Cell[86589, 3274, 256, 9, 26, "Input",
 CellTags->"Apart2"],
Cell[86848, 3285, 1232, 31, 51, "Output",
 CellTags->"Apart2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[88117, 3321, 104, 3, 26, "Input",
 CellTags->"Apart2"],
Cell[88224, 3326, 761, 27, 56, "Output",
 CellTags->"Apart2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[89022, 3358, 140, 4, 26, "Input",
 CellTags->"Apart2"],
Cell[89165, 3364, 561, 20, 54, "Output",
 CellTags->"Apart2"]
}, Open  ]]
}, Open  ]],
Cell[89753, 3388, 64, 2, 39, "ObjectName",
 PageBreakAbove->True,
 CellTags->"A0"],
Cell[CellGroupData[{
Cell[89842, 3394, 53, 1, 34, "Subsubsection",
 CellTags->"A0"],
Cell[89898, 3397, 291, 11, 27, "Text",
 CellTags->"A0"],
Cell[CellGroupData[{
Cell[90214, 3412, 107, 3, 26, "Input",
 CellTags->"A0"],
Cell[90324, 3417, 203, 7, 26, "Output",
 CellTags->"A0"]
}, Open  ]],
Cell[90542, 3427, 437, 24, 24, "Text",
 CellTags->"A0"]
}, Open  ]],
Cell[CellGroupData[{
Cell[91016, 3456, 50, 1, 34, "Subsubsection",
 CellTags->"A0"],
Cell[91069, 3459, 246, 11, 24, "Text",
 CellTags->"A0"],
Cell[CellGroupData[{
Cell[91340, 3474, 120, 4, 30, "Input",
 CellTags->"A0"],
Cell[91463, 3480, 301, 11, 29, "Output",
 CellTags->"A0"]
}, Open  ]],
Cell[91779, 3494, 186, 6, 26, "Input",
 CellTags->"A0"],
Cell[CellGroupData[{
Cell[91990, 3504, 120, 4, 30, "Input",
 CellTags->"A0"],
Cell[92113, 3510, 366, 12, 29, "Output",
 CellTags->"A0"]
}, Open  ]],
Cell[92494, 3525, 187, 6, 26, "Input",
 CellTags->"A0"],
Cell[92684, 3533, 222, 8, 24, "Text",
 CellTags->"A0"],
Cell[CellGroupData[{
Cell[92931, 3545, 96, 3, 26, "Input",
 CellTags->"A0"],
Cell[93030, 3550, 139, 4, 26, "Output",
 CellTags->"A0"]
}, Open  ]],
Cell[CellGroupData[{
Cell[93206, 3559, 160, 5, 26, "Input",
 CellTags->"A0"],
Cell[93369, 3566, 139, 4, 26, "Output",
 CellTags->"A0"]
}, Open  ]]
}, Open  ]],
Cell[93535, 3574, 72, 2, 39, "ObjectName",
 PageBreakAbove->True,
 CellTags->"A0ToB0"],
Cell[CellGroupData[{
Cell[93632, 3580, 57, 1, 34, "Subsubsection",
 CellTags->"A0ToB0"],
Cell[93692, 3583, 614, 27, 24, "Text",
 CellTags->"A0ToB0"],
Cell[CellGroupData[{
Cell[94331, 3614, 111, 3, 26, "Input",
 CellTags->"A0ToB0"],
Cell[94445, 3619, 209, 7, 26, "Output",
 CellTags->"A0ToB0"]
}, Open  ]],
Cell[94669, 3629, 533, 29, 24, "Text",
 CellTags->"A0ToB0"]
}, Open  ]],
Cell[95217, 3661, 79, 1, 39, "ObjectName",
 CellTags->"BackgroundGluonVertex"],
Cell[CellGroupData[{
Cell[95321, 3666, 72, 1, 34, "Subsubsection",
 CellTags->"BackgroundGluonVertex"],
Cell[95396, 3669, 1042, 19, 126, "Text",
 CellTags->"BackgroundGluonVertex"],
Cell[CellGroupData[{
Cell[96463, 3692, 140, 3, 26, "Input",
 CellTags->"BackgroundGluonVertex"],
Cell[96606, 3697, 362, 10, 26, "Output",
 CellTags->"BackgroundGluonVertex"]
}, Open  ]],
Cell[96983, 3710, 201, 9, 24, "Text",
 CellTags->"BackgroundGluonVertex"]
}, Open  ]],
Cell[CellGroupData[{
Cell[97221, 3724, 69, 1, 34, "Subsubsection",
 CellTags->"BackgroundGluonVertex"],
Cell[CellGroupData[{
Cell[97315, 3729, 360, 10, 26, "Input",
 CellTags->"BackgroundGluonVertex"],
Cell[97678, 3741, 1528, 59, 29, "Output",
 CellTags->"BackgroundGluonVertex"]
}, Open  ]],
Cell[CellGroupData[{
Cell[99243, 3805, 436, 12, 26, "Input",
 CellTags->"BackgroundGluonVertex"],
Cell[99682, 3819, 3888, 143, 52, "Output",
 CellTags->"BackgroundGluonVertex"]
}, Open  ]],
Cell[CellGroupData[{
Cell[103607, 3967, 411, 11, 26, "Input",
 CellTags->"BackgroundGluonVertex"],
Cell[104021, 3980, 1603, 61, 46, "Output",
 CellTags->"BackgroundGluonVertex"]
}, Open  ]],
Cell[CellGroupData[{
Cell[105661, 4046, 487, 13, 26, "Input",
 CellTags->"BackgroundGluonVertex"],
Cell[106151, 4061, 3996, 146, 90, "Output",
 CellTags->"BackgroundGluonVertex"]
}, Open  ]]
}, Open  ]],
Cell[110174, 4211, 51, 1, 39, "ObjectName",
 CellTags->"Bracket"],
Cell[CellGroupData[{
Cell[110250, 4216, 58, 1, 34, "Subsubsection",
 CellTags->"Bracket"],
Cell[110311, 4219, 72, 1, 24, "Text",
 CellTags->"Bracket"],
Cell[110386, 4222, 181, 9, 24, "Text",
 CellTags->"Bracket"]
}, Open  ]],
Cell[110582, 4234, 74, 2, 39, "ObjectName",
 PageBreakAbove->True,
 CellTags->"BReduce"],
Cell[CellGroupData[{
Cell[110681, 4240, 58, 1, 34, "Subsubsection",
 CellTags->"BReduce"],
Cell[110742, 4243, 148, 4, 24, "Text",
 CellTags->"BReduce"],
Cell[110893, 4249, 534, 29, 24, "Text",
 CellTags->"BReduce"]
}, Open  ]],
Cell[CellGroupData[{
Cell[111464, 4283, 55, 1, 34, "Subsubsection",
 CellTags->"BReduce"],
Cell[111522, 4286, 251, 11, 24, "Text",
 CellTags->"BReduce"],
Cell[CellGroupData[{
Cell[111798, 4301, 134, 4, 26, "Input",
 CellTags->"BReduce"],
Cell[111935, 4307, 220, 7, 26, "Output",
 CellTags->"BReduce"]
}, Open  ]],
Cell[CellGroupData[{
Cell[112192, 4319, 81, 2, 26, "Input"],
Cell[112276, 4323, 305, 9, 26, "Output"]
}, Open  ]],
Cell[112596, 4335, 94, 3, 24, "Text",
 CellTags->"BReduce"],
Cell[CellGroupData[{
Cell[112715, 4342, 183, 5, 26, "Input",
 CellTags->"BReduce"],
Cell[112901, 4349, 302, 11, 45, "Output",
 CellTags->"BReduce"]
}, Open  ]]
}, Open  ]],
Cell[113230, 4364, 41, 1, 39, "ObjectName",
 CellTags->"B0"],
Cell[CellGroupData[{
Cell[113296, 4369, 53, 1, 34, "Subsubsection",
 CellTags->"B0"],
Cell[113352, 4372, 390, 9, 41, "Text",
 CellTags->"B0"],
Cell[CellGroupData[{
Cell[113767, 4385, 102, 3, 26, "Input",
 CellTags->"B0"],
Cell[113872, 4390, 324, 10, 26, "Output",
 CellTags->"B0"]
}, Open  ]],
Cell[114211, 4403, 443, 24, 24, "Text",
 CellTags->"B0"]
}, Open  ]],
Cell[CellGroupData[{
Cell[114691, 4432, 50, 1, 34, "Subsubsection",
 CellTags->"B0"],
Cell[CellGroupData[{
Cell[114766, 4437, 234, 8, 30, "Input",
 CellTags->"B0"],
Cell[115003, 4447, 358, 13, 29, "Output",
 CellTags->"B0"]
}, Open  ]],
Cell[115376, 4463, 237, 7, 26, "Input",
 CellTags->"B0"],
Cell[CellGroupData[{
Cell[115638, 4474, 154, 5, 30, "Input",
 CellTags->"B0"],
Cell[115795, 4481, 287, 9, 29, "Output",
 CellTags->"B0"]
}, Open  ]],
Cell[CellGroupData[{
Cell[116119, 4495, 179, 6, 30, "Input",
 CellTags->"B0"],
Cell[116301, 4503, 289, 9, 29, "Output",
 CellTags->"B0"]
}, Open  ]],
Cell[CellGroupData[{
Cell[116627, 4517, 177, 6, 26, "Input",
 CellTags->"B0"],
Cell[116807, 4525, 262, 8, 29, "Output",
 CellTags->"B0"]
}, Open  ]]
}, Open  ]],
Cell[117096, 4537, 72, 2, 39, "ObjectName",
 PageBreakAbove->True,
 CellTags->"B0Real"],
Cell[CellGroupData[{
Cell[117193, 4543, 57, 1, 34, "Subsubsection",
 CellTags->"B0Real"],
Cell[117253, 4546, 186, 4, 24, "Text",
 CellTags->"B0Real"],
Cell[117442, 4552, 153, 8, 24, "Text",
 CellTags->"B0Real"]
}, Open  ]],
Cell[CellGroupData[{
Cell[117632, 4565, 54, 1, 34, "Subsubsection",
 CellTags->"B0Real"],
Cell[117689, 4568, 83, 1, 24, "Text",
 CellTags->"B0Real"],
Cell[CellGroupData[{
Cell[117797, 4573, 133, 4, 26, "Input",
 CellTags->"B0Real"],
Cell[117933, 4579, 242, 8, 26, "Output",
 CellTags->"B0Real"]
}, Open  ]],
Cell[CellGroupData[{
Cell[118212, 4592, 102, 3, 26, "Input",
 CellTags->"B0Real"],
Cell[118317, 4597, 324, 10, 26, "Output",
 CellTags->"B0Real"]
}, Open  ]],
Cell[118656, 4610, 92, 3, 24, "Text",
 CellTags->"B0Real"],
Cell[CellGroupData[{
Cell[118773, 4617, 181, 5, 26, "Input",
 CellTags->"B0Real"],
Cell[118957, 4624, 242, 8, 26, "Output",
 CellTags->"B0Real"]
}, Open  ]]
}, Open  ]],
Cell[119226, 4636, 76, 2, 39, "ObjectName",
 PageBreakAbove->True,
 CellTags->"B0Unique"],
Cell[CellGroupData[{
Cell[119327, 4642, 59, 1, 34, "Subsubsection",
 CellTags->"B0Unique"],
Cell[119389, 4645, 184, 4, 24, "Text",
 CellTags->"B0Unique"],
Cell[119576, 4651, 155, 8, 24, "Text",
 CellTags->"B0Unique"]
}, Open  ]],
Cell[CellGroupData[{
Cell[119768, 4664, 56, 1, 34, "Subsubsection",
 CellTags->"B0Unique"],
Cell[119827, 4667, 73, 1, 24, "Text",
 CellTags->"B0Unique"],
Cell[CellGroupData[{
Cell[119925, 4672, 135, 4, 26, "Input",
 CellTags->"B0Unique"],
Cell[120063, 4678, 244, 8, 26, "Output",
 CellTags->"B0Unique"]
}, Open  ]],
Cell[CellGroupData[{
Cell[120344, 4691, 104, 3, 26, "Input",
 CellTags->"B0Unique"],
Cell[120451, 4696, 328, 10, 26, "Output",
 CellTags->"B0Unique"]
}, Open  ]],
Cell[120794, 4709, 88, 3, 24, "Text",
 CellTags->"B0Unique"],
Cell[CellGroupData[{
Cell[120907, 4716, 186, 5, 26, "Input",
 CellTags->"B0Unique"],
Cell[121096, 4723, 218, 7, 26, "Output",
 CellTags->"B0Unique"]
}, Open  ]]
}, Open  ]],
Cell[121341, 4734, 43, 1, 41, "ObjectName",
 CellTags->"B00"],
Cell[CellGroupData[{
Cell[121409, 4739, 54, 1, 36, "Subsubsection",
 CellTags->"B00"],
Cell[121466, 4742, 290, 8, 41, "Text",
 CellTags->"B00"],
Cell[CellGroupData[{
Cell[121781, 4754, 104, 3, 26, "Input",
 CellTags->"B00"],
Cell[121888, 4759, 204, 7, 26, "Output",
 CellTags->"B00"]
}, Open  ]],
Cell[122107, 4769, 346, 19, 25, "Text",
 CellTags->"B00"]
}, Open  ]],
Cell[CellGroupData[{
Cell[122490, 4793, 51, 1, 36, "Subsubsection",
 CellTags->"B00"],
Cell[122544, 4796, 202, 7, 26, "Text",
 CellTags->"B00"],
Cell[CellGroupData[{
Cell[122771, 4807, 97, 3, 26, "Input",
 CellTags->"B00"],
Cell[122871, 4812, 198, 7, 28, "Output",
 CellTags->"B00"]
}, Open  ]],
Cell[CellGroupData[{
Cell[123106, 4824, 211, 7, 30, "Input",
 CellTags->"B00"],
Cell[123320, 4833, 2717, 91, 95, "Output",
 CellTags->"B00"]
}, Open  ]],
Cell[CellGroupData[{
Cell[126074, 4929, 211, 7, 30, "Input",
 CellTags->"B00"],
Cell[126288, 4938, 1496, 55, 46, "Output",
 CellTags->"B00"]
}, Open  ]],
Cell[CellGroupData[{
Cell[127821, 4998, 261, 8, 30, "Input",
 CellTags->"B00"],
Cell[128085, 5008, 560, 23, 29, "Output",
 CellTags->"B00"]
}, Open  ]],
Cell[CellGroupData[{
Cell[128682, 5036, 181, 6, 30, "Input",
 CellTags->"B00"],
Cell[128866, 5044, 656, 23, 46, "Output",
 CellTags->"B00"]
}, Open  ]],
Cell[CellGroupData[{
Cell[129559, 5072, 248, 8, 30, "Input",
 CellTags->"B00"],
Cell[129810, 5082, 685, 24, 46, "Output",
 CellTags->"B00"]
}, Open  ]]
}, Open  ]],
Cell[130522, 5110, 41, 1, 41, "ObjectName",
 CellTags->"B1"],
Cell[CellGroupData[{
Cell[130588, 5115, 53, 1, 36, "Subsubsection",
 CellTags->"B1"],
Cell[130644, 5118, 230, 7, 24, "Text",
 CellTags->"B1"],
Cell[CellGroupData[{
Cell[130899, 5129, 102, 3, 26, "Input",
 CellTags->"B1"],
Cell[131004, 5134, 203, 7, 26, "Output",
 CellTags->"B1"]
}, Open  ]],
Cell[131222, 5144, 559, 29, 25, "Text",
 CellTags->"B1"]
}, Open  ]],
Cell[CellGroupData[{
Cell[131818, 5178, 50, 1, 34, "Subsubsection",
 CellTags->"B1"],
Cell[CellGroupData[{
Cell[131893, 5183, 209, 7, 30, "Input",
 CellTags->"B1"],
Cell[132105, 5192, 1195, 43, 52, "Output",
 CellTags->"B1"]
}, Open  ]],
Cell[CellGroupData[{
Cell[133337, 5240, 259, 8, 30, "Input",
 CellTags->"B1"],
Cell[133599, 5250, 358, 13, 29, "Output",
 CellTags->"B1"]
}, Open  ]],
Cell[133972, 5266, 187, 6, 26, "Input",
 CellTags->"B1"],
Cell[CellGroupData[{
Cell[134184, 5276, 209, 7, 30, "Input",
 CellTags->"B1"],
Cell[134396, 5285, 432, 16, 44, "Output",
 CellTags->"B1"]
}, Open  ]],
Cell[CellGroupData[{
Cell[134865, 5306, 179, 6, 30, "Input",
 CellTags->"B1"],
Cell[135047, 5314, 445, 15, 44, "Output",
 CellTags->"B1"]
}, Open  ]],
Cell[CellGroupData[{
Cell[135529, 5334, 154, 5, 30, "Input",
 CellTags->"B1"],
Cell[135686, 5341, 447, 15, 44, "Output",
 CellTags->"B1"]
}, Open  ]],
Cell[CellGroupData[{
Cell[136170, 5361, 238, 7, 30, "Input",
 CellTags->"B1"],
Cell[136411, 5370, 833, 26, 51, "Output",
 CellTags->"B1"]
}, Open  ]],
Cell[CellGroupData[{
Cell[137281, 5401, 279, 8, 30, "Input",
 CellTags->"B1"],
Cell[137563, 5411, 461, 15, 44, "Output",
 CellTags->"B1"]
}, Open  ]]
}, Open  ]],
Cell[138051, 5430, 43, 1, 57, "ObjectName",
 CellTags->"B11"],
Cell[CellGroupData[{
Cell[138119, 5435, 54, 1, 52, "Subsubsection",
 CellTags->"B11"],
Cell[138176, 5438, 495, 17, 43, "Text",
 CellTags->"B11"],
Cell[CellGroupData[{
Cell[138696, 5459, 104, 3, 26, "Input",
 CellTags->"B11"],
Cell[138803, 5464, 206, 7, 26, "Output",
 CellTags->"B11"]
}, Open  ]],
Cell[139024, 5474, 441, 24, 41, "Text",
 CellTags->"B11"]
}, Open  ]],
Cell[CellGroupData[{
Cell[139502, 5503, 51, 1, 52, "Subsubsection",
 CellTags->"B11"],
Cell[139556, 5506, 215, 7, 26, "Text",
 CellTags->"B11"],
Cell[CellGroupData[{
Cell[139796, 5517, 97, 3, 26, "Input",
 CellTags->"B11"],
Cell[139896, 5522, 198, 7, 28, "Output",
 CellTags->"B11"]
}, Open  ]],
Cell[CellGroupData[{
Cell[140131, 5534, 212, 7, 30, "Input",
 CellTags->"B11"],
Cell[140346, 5543, 2889, 98, 154, "Output",
 CellTags->"B11"]
}, Open  ]],
Cell[CellGroupData[{
Cell[143272, 5646, 212, 7, 30, "Input",
 CellTags->"B11"],
Cell[143487, 5655, 1250, 48, 56, "Output",
 CellTags->"B11"]
}, Open  ]],
Cell[CellGroupData[{
Cell[144774, 5708, 171, 5, 26, "Input",
 CellTags->"B11"],
Cell[144948, 5715, 206, 7, 26, "Output",
 CellTags->"B11"]
}, Open  ]],
Cell[CellGroupData[{
Cell[145191, 5727, 212, 7, 30, "Input",
 CellTags->"B11"],
Cell[145406, 5736, 561, 23, 29, "Output",
 CellTags->"B11"]
}, Open  ]],
Cell[CellGroupData[{
Cell[146004, 5764, 170, 5, 26, "Input",
 CellTags->"B11"],
Cell[146177, 5771, 205, 7, 26, "Output",
 CellTags->"B11"]
}, Open  ]],
Cell[CellGroupData[{
Cell[146419, 5783, 182, 6, 30, "Input",
 CellTags->"B11"],
Cell[146604, 5791, 311, 10, 44, "Output",
 CellTags->"B11"]
}, Open  ]],
Cell[CellGroupData[{
Cell[146952, 5806, 249, 8, 30, "Input",
 CellTags->"B11"],
Cell[147204, 5816, 337, 11, 44, "Output",
 CellTags->"B11"]
}, Open  ]]
}, Open  ]],
Cell[147568, 5831, 41, 1, 39, "ObjectName",
 CellTags->"CA"],
Cell[CellGroupData[{
Cell[147634, 5836, 53, 1, 34, "Subsubsection",
 CellTags->"CA"],
Cell[147690, 5839, 198, 9, 24, "Text",
 CellTags->"CA"],
Cell[147891, 5850, 274, 14, 24, "Text",
 CellTags->"CA"]
}, Open  ]],
Cell[CellGroupData[{
Cell[148202, 5869, 50, 1, 34, "Subsubsection",
 CellTags->"CA"],
Cell[CellGroupData[{
Cell[148277, 5874, 70, 2, 26, "Input",
 CellTags->"CA"],
Cell[148350, 5878, 162, 5, 26, "Output",
 CellTags->"CA"]
}, Open  ]],
Cell[CellGroupData[{
Cell[148549, 5888, 173, 5, 26, "Input",
 CellTags->"CA"],
Cell[148725, 5895, 140, 4, 27, "Output",
 CellTags->"CA"]
}, Open  ]],
Cell[CellGroupData[{
Cell[148902, 5904, 72, 2, 26, "Input",
 CellTags->"CA"],
Cell[148977, 5908, 142, 4, 27, "Output",
 CellTags->"CA"]
}, Open  ]]
}, Open  ]],
Cell[149146, 5916, 45, 1, 39, "ObjectName",
 CellTags->"Calc"],
Cell[CellGroupData[{
Cell[149216, 5921, 55, 1, 34, "Subsubsection",
 CellTags->"Calc"],
Cell[149274, 5924, 233, 5, 41, "Text",
 CellTags->"Calc"],
Cell[149510, 5931, 407, 19, 24, "Text",
 CellTags->"Calc"]
}, Open  ]],
Cell[CellGroupData[{
Cell[149954, 5955, 61, 1, 33, "Subsubsection",
 CellTags->"Calc"],
Cell[150018, 5958, 346, 13, 27, "Text",
 CellTags->"Calc"],
Cell[CellGroupData[{
Cell[150389, 5975, 284, 9, 26, "Input",
 CellTags->"Calc"],
Cell[150676, 5986, 189, 6, 26, "Output",
 CellTags->"Calc"]
}, Open  ]],
Cell[150880, 5995, 213, 9, 26, "Text",
 CellTags->"Calc"],
Cell[CellGroupData[{
Cell[151118, 6008, 258, 8, 26, "Input",
 CellTags->"Calc"],
Cell[151379, 6018, 394, 16, 26, "Output",
 CellTags->"Calc"]
}, Open  ]],
Cell[CellGroupData[{
Cell[151810, 6039, 328, 11, 26, "Input",
 CellTags->"Calc"],
Cell[152141, 6052, 466, 17, 28, "Output",
 CellTags->"Calc"]
}, Open  ]],
Cell[CellGroupData[{
Cell[152644, 6074, 101, 3, 26, "Input",
 CellTags->"Calc"],
Cell[152748, 6079, 569, 23, 28, "Output",
 CellTags->"Calc"]
}, Open  ]],
Cell[CellGroupData[{
Cell[153354, 6107, 215, 6, 26, "Input",
 CellTags->"Calc"],
Cell[153572, 6115, 1029, 45, 30, "Output",
 CellTags->"Calc"]
}, Open  ]],
Cell[CellGroupData[{
Cell[154638, 6165, 382, 13, 26, "Input",
 CellTags->"Calc"],
Cell[155023, 6180, 142, 4, 26, "Output",
 CellTags->"Calc"]
}, Open  ]]
}, Open  ]],
Cell[155192, 6188, 67, 1, 39, "ObjectName",
 CellTags->"CalcColorFactor"],
Cell[CellGroupData[{
Cell[155284, 6193, 66, 1, 34, "Subsubsection",
 CellTags->"CalcColorFactor"],
Cell[155353, 6196, 470, 9, 108, "Text",
 CellTags->"CalcColorFactor"],
Cell[155826, 6207, 195, 9, 24, "Text",
 CellTags->"CalcColorFactor"],
Cell[CellGroupData[{
Cell[156046, 6220, 250, 7, 26, "Input"],
Cell[156299, 6229, 377, 15, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[156713, 6249, 99, 2, 26, "Input"],
Cell[156815, 6253, 1749, 56, 30, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[158601, 6314, 118, 3, 26, "Input"],
Cell[158722, 6319, 2320, 78, 61, "Output"]
}, Open  ]],
Cell[161057, 6400, 411, 18, 24, "Text",
 CellTags->"Collecting"]
}, Open  ]],
Cell[161483, 6421, 94, 1, 39, "ObjectName",
 CellTags->"CalculateCounterTerm"],
Cell[CellGroupData[{
Cell[161602, 6426, 71, 1, 34, "Subsubsection",
 CellTags->"CalculateCounterTerm"],
Cell[161676, 6429, 110, 1, 24, "Text",
 CellTags->"CalculateCounterTerm"]
}, Open  ]],
Cell[CellGroupData[{
Cell[161823, 6435, 77, 1, 33, "Subsubsection",
 CellTags->"CalculateCounterTerm"],
Cell[CellGroupData[{
Cell[161925, 6440, 123, 3, 26, "Input",
 CellTags->"CalculateCounterTerm"],
Cell[162051, 6445, 235, 4, 41, "Print",
 CellTags->{"CalculateCounterTerm", "Info3500973745-4982097"}]
}, Open  ]]
}, Open  ]],
Cell[162313, 6453, 53, 1, 39, "ObjectName",
 CellTags->"CancelQP"],
Cell[CellGroupData[{
Cell[162391, 6458, 59, 1, 34, "Subsubsection",
 CellTags->"CancelQP"],
Cell[162453, 6461, 314, 13, 26, "Text",
 CellTags->"CancelQP"]
}, Open  ]],
Cell[CellGroupData[{
Cell[162804, 6479, 65, 1, 33, "Subsubsection",
 CellTags->"CancelQP"],
Cell[CellGroupData[{
Cell[162894, 6484, 549, 19, 26, "Input",
 CellTags->"CancelQP"],
Cell[163446, 6505, 709, 25, 49, "Output",
 CellTags->"CancelQP"]
}, Open  ]],
Cell[CellGroupData[{
Cell[164192, 6535, 182, 5, 26, "Input",
 CellTags->"CancelQP"],
Cell[164377, 6542, 2227, 74, 105, "Output",
 CellTags->"CancelQP"]
}, Open  ]],
Cell[CellGroupData[{
Cell[166641, 6621, 183, 5, 26, "Input",
 CellTags->"CancelQP"],
Cell[166827, 6628, 6462, 210, 254, "Output",
 CellTags->"CancelQP"]
}, Open  ]],
Cell[CellGroupData[{
Cell[173326, 6843, 555, 18, 26, "Input",
 CellTags->"CancelQP"],
Cell[173884, 6863, 930, 34, 50, "Output",
 CellTags->"CancelQP"]
}, Open  ]],
Cell[CellGroupData[{
Cell[174851, 6902, 182, 5, 26, "Input",
 CellTags->"CancelQP"],
Cell[175036, 6909, 6513, 226, 157, "Output",
 CellTags->"CancelQP"]
}, Open  ]],
Cell[CellGroupData[{
Cell[181586, 7140, 183, 5, 26, "Input",
 CellTags->"CancelQP"],
Cell[181772, 7147, 3778, 127, 105, "Output",
 CellTags->"CancelQP"]
}, Open  ]],
Cell[185565, 7277, 106, 3, 26, "Input",
 CellTags->"CancelQP"]
}, Open  ]],
Cell[185686, 7283, 49, 1, 39, "ObjectName",
 CellTags->"Cases2"],
Cell[CellGroupData[{
Cell[185760, 7288, 57, 1, 34, "Subsubsection",
 CellTags->"Cases2"],
Cell[185820, 7291, 293, 6, 41, "Text",
 CellTags->"Cases2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[186150, 7302, 63, 1, 33, "Subsubsection",
 CellTags->"Cases2"],
Cell[CellGroupData[{
Cell[186238, 7307, 305, 10, 30, "Input",
 CellTags->"Cases2"],
Cell[186546, 7319, 319, 10, 26, "Output",
 CellTags->"Cases2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[186902, 7334, 312, 10, 26, "Input",
 CellTags->"Cases2"],
Cell[187217, 7346, 323, 10, 26, "Output",
 CellTags->"Cases2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[187577, 7361, 427, 14, 26, "Input",
 CellTags->"Cases2"],
Cell[188007, 7377, 397, 12, 26, "Output",
 CellTags->"Cases2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[188441, 7394, 327, 10, 26, "Input",
 CellTags->"Cases2"],
Cell[188771, 7406, 505, 18, 28, "Output",
 CellTags->"Cases2"]
}, Open  ]]
}, Open  ]],
Cell[189303, 7428, 41, 1, 39, "ObjectName",
 CellTags->"CF"],
Cell[CellGroupData[{
Cell[189369, 7433, 53, 1, 34, "Subsubsection",
 CellTags->"CF"],
Cell[189425, 7436, 287, 13, 24, "Text",
 CellTags->"CF"],
Cell[189715, 7451, 274, 14, 24, "Text",
 CellTags->"CF"]
}, Open  ]],
Cell[CellGroupData[{
Cell[190026, 7470, 50, 1, 34, "Subsubsection",
 CellTags->"CF"],
Cell[CellGroupData[{
Cell[190101, 7475, 70, 2, 26, "Input",
 CellTags->"CF"],
Cell[190174, 7479, 164, 5, 26, "Output",
 CellTags->"CF"]
}, Open  ]],
Cell[CellGroupData[{
Cell[190375, 7489, 178, 5, 26, "Input",
 CellTags->"CF"],
Cell[190556, 7496, 235, 8, 46, "Output",
 CellTags->"CF"]
}, Open  ]],
Cell[CellGroupData[{
Cell[190828, 7509, 83, 2, 26, "Input"],
Cell[190914, 7513, 118, 2, 38, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[191069, 7520, 72, 2, 26, "Input",
 CellTags->"CF"],
Cell[191144, 7524, 140, 4, 27, "Output",
 CellTags->"CF"]
}, Open  ]],
Cell[CellGroupData[{
Cell[191321, 7533, 233, 7, 26, "Input",
 CellTags->"CF"],
Cell[191557, 7542, 219, 7, 26, "Output",
 CellTags->"CF"]
}, Open  ]]
}, Open  ]],
Cell[191803, 7553, 67, 1, 39, "ObjectName",
 CellTags->"ChangeDimension"],
Cell[CellGroupData[{
Cell[191895, 7558, 66, 1, 34, "Subsubsection",
 CellTags->"ChangeDimension"],
Cell[191964, 7561, 210, 5, 41, "Text",
 CellTags->"ChangeDimension"],
Cell[192177, 7568, 519, 24, 24, "Text",
 CellTags->"ChangeDimension"]
}, Open  ]],
Cell[CellGroupData[{
Cell[192733, 7597, 63, 1, 34, "Subsubsection",
 CellTags->"ChangeDimension"],
Cell[192799, 7600, 219, 5, 41, "Text",
 CellTags->"ChangeDimension"],
Cell[CellGroupData[{
Cell[193043, 7609, 212, 6, 26, "Input",
 CellTags->"ChangeDimension"],
Cell[193258, 7617, 240, 8, 28, "Output",
 CellTags->"ChangeDimension"]
}, Open  ]],
Cell[CellGroupData[{
Cell[193535, 7630, 151, 4, 26, "Input",
 CellTags->"ChangeDimension"],
Cell[193689, 7636, 240, 8, 28, "Output",
 CellTags->"ChangeDimension"]
}, Open  ]],
Cell[CellGroupData[{
Cell[193966, 7649, 116, 3, 26, "Input",
 CellTags->"ChangeDimension"],
Cell[194085, 7654, 314, 9, 38, "Output",
 CellTags->"ChangeDimension"]
}, Open  ]],
Cell[194414, 7666, 119, 3, 24, "Text",
 CellTags->"ChangeDimension"],
Cell[CellGroupData[{
Cell[194558, 7673, 201, 6, 26, "Input",
 CellTags->"ChangeDimension"],
Cell[194762, 7681, 266, 7, 38, "Output",
 CellTags->"ChangeDimension"]
}, Open  ]],
Cell[195043, 7691, 105, 1, 24, "Text",
 CellTags->"ChangeDimension"],
Cell[CellGroupData[{
Cell[195173, 7696, 482, 15, 26, "Input",
 CellTags->"ChangeDimension"],
Cell[195658, 7713, 613, 19, 26, "Output",
 CellTags->"ChangeDimension"]
}, Open  ]],
Cell[CellGroupData[{
Cell[196308, 7737, 153, 4, 26, "Input",
 CellTags->"ChangeDimension"],
Cell[196464, 7743, 797, 30, 26, "Output",
 CellTags->"ChangeDimension"]
}, Open  ]],
Cell[CellGroupData[{
Cell[197298, 7778, 222, 6, 26, "Input",
 CellTags->"ChangeDimension"],
Cell[197523, 7786, 797, 30, 26, "Output",
 CellTags->"ChangeDimension"]
}, Open  ]],
Cell[198335, 7819, 132, 3, 24, "Text",
 CellTags->"ChangeDimension"],
Cell[198470, 7824, 208, 6, 26, "Input",
 CellTags->"ChangeDimension"],
Cell[198681, 7832, 219, 5, 41, "Text",
 CellTags->"ChangeDimension"],
Cell[198903, 7839, 201, 6, 26, "Input",
 CellTags->"ChangeDimension"],
Cell[CellGroupData[{
Cell[199129, 7849, 220, 6, 26, "Input",
 CellTags->"ChangeDimension"],
Cell[199352, 7857, 531, 22, 26, "Output",
 CellTags->"ChangeDimension"]
}, Open  ]],
Cell[CellGroupData[{
Cell[199920, 7884, 182, 5, 26, "Input",
 CellTags->"ChangeDimension"],
Cell[200105, 7891, 531, 22, 26, "Output",
 CellTags->"ChangeDimension"]
}, Open  ]],
Cell[CellGroupData[{
Cell[200673, 7918, 176, 5, 30, "Input",
 CellTags->"ChangeDimension"],
Cell[200852, 7925, 343, 11, 26, "Output",
 CellTags->"ChangeDimension"]
}, Open  ]],
Cell[CellGroupData[{
Cell[201232, 7941, 141, 4, 30, "Input",
 CellTags->"ChangeDimension"],
Cell[201376, 7947, 174, 5, 26, "Output",
 CellTags->"ChangeDimension"]
}, Open  ]],
Cell[201565, 7955, 201, 6, 26, "Input",
 CellTags->"ChangeDimension"],
Cell[201769, 7963, 157, 5, 26, "Input",
 CellTags->"ChangeDimension"]
}, Open  ]],
Cell[201941, 7971, 83, 1, 39, "ObjectName",
 CellTags->"ChargeConjugationMatrix"],
Cell[CellGroupData[{
Cell[202049, 7976, 74, 1, 34, "Subsubsection",
 CellTags->"ChargeConjugationMatrix"],
Cell[202126, 7979, 176, 6, 24, "Text",
 CellTags->"ChargeConjugationMatrix"],
Cell[202305, 7987, 248, 9, 24, "Text",
 CellTags->"ChargeConjugationMatrix"]
}, Open  ]],
Cell[CellGroupData[{
Cell[202590, 8001, 71, 1, 34, "Subsubsection",
 CellTags->"ChargeConjugationMatrix"],
Cell[CellGroupData[{
Cell[202686, 8006, 214, 5, 26, "Input",
 CellTags->"ChargeConjugationMatrix"],
Cell[202903, 8013, 348, 12, 26, "Output",
 CellTags->"ChargeConjugationMatrix"]
}, Open  ]],
Cell[CellGroupData[{
Cell[203288, 8030, 120, 3, 26, "Input",
 CellTags->"ChargeConjugationMatrix"],
Cell[203411, 8035, 299, 10, 30, "Output",
 CellTags->"ChargeConjugationMatrix"]
}, Open  ]],
Cell[CellGroupData[{
Cell[203747, 8050, 209, 5, 26, "Input",
 CellTags->"ChargeConjugationMatrix"],
Cell[203959, 8057, 270, 8, 26, "Output",
 CellTags->"ChargeConjugationMatrix"]
}, Open  ]],
Cell[CellGroupData[{
Cell[204266, 8070, 120, 3, 26, "Input",
 CellTags->"ChargeConjugationMatrix"],
Cell[204389, 8075, 234, 7, 28, "Output",
 CellTags->"ChargeConjugationMatrix"]
}, Open  ]],
Cell[CellGroupData[{
Cell[204660, 8087, 134, 3, 26, "Input",
 CellTags->"ChargeConjugationMatrix"],
Cell[204797, 8092, 179, 5, 26, "Output",
 CellTags->"ChargeConjugationMatrix"]
}, Open  ]]
}, Open  ]],
Cell[205003, 8101, 89, 1, 39, "ObjectName",
 CellTags->"ChargeConjugationMatrixInv"],
Cell[CellGroupData[{
Cell[205117, 8106, 77, 1, 34, "Subsubsection",
 CellTags->"ChargeConjugationMatrixInv"],
Cell[205197, 8109, 195, 4, 24, "Text",
 CellTags->"ChargeConjugationMatrixInv"],
Cell[205395, 8115, 242, 9, 24, "Text",
 CellTags->"ChargeConjugationMatrixInv"]
}, Open  ]],
Cell[CellGroupData[{
Cell[205674, 8129, 74, 1, 34, "Subsubsection",
 CellTags->"ChargeConjugationMatrixInv"],
Cell[CellGroupData[{
Cell[205773, 8134, 217, 5, 26, "Input",
 CellTags->"ChargeConjugationMatrixInv"],
Cell[205993, 8141, 351, 12, 26, "Output",
 CellTags->"ChargeConjugationMatrixInv"]
}, Open  ]],
Cell[CellGroupData[{
Cell[206381, 8158, 123, 3, 26, "Input",
 CellTags->"ChargeConjugationMatrixInv"],
Cell[206507, 8163, 300, 10, 30, "Output",
 CellTags->"ChargeConjugationMatrixInv"]
}, Open  ]],
Cell[CellGroupData[{
Cell[206844, 8178, 212, 5, 26, "Input",
 CellTags->"ChargeConjugationMatrixInv"],
Cell[207059, 8185, 273, 8, 26, "Output",
 CellTags->"ChargeConjugationMatrixInv"]
}, Open  ]],
Cell[CellGroupData[{
Cell[207369, 8198, 123, 3, 26, "Input",
 CellTags->"ChargeConjugationMatrixInv"],
Cell[207495, 8203, 237, 7, 28, "Output",
 CellTags->"ChargeConjugationMatrixInv"]
}, Open  ]],
Cell[CellGroupData[{
Cell[207769, 8215, 165, 4, 26, "Input",
 CellTags->"ChargeConjugationMatrixInv"],
Cell[207937, 8221, 231, 7, 26, "Output",
 CellTags->"ChargeConjugationMatrixInv"]
}, Open  ]],
Cell[CellGroupData[{
Cell[208205, 8233, 119, 3, 26, "Input",
 CellTags->"ChargeConjugationMatrixInv"],
Cell[208327, 8238, 206, 6, 26, "Output",
 CellTags->"ChargeConjugationMatrixInv"]
}, Open  ]],
Cell[CellGroupData[{
Cell[208570, 8249, 140, 3, 26, "Input",
 CellTags->"ChargeConjugationMatrixInv"],
Cell[208713, 8254, 182, 5, 26, "Output",
 CellTags->"ChargeConjugationMatrixInv"]
}, Open  ]]
}, Open  ]],
Cell[208922, 8263, 61, 1, 39, "ObjectName",
 CellTags->"CheckContext"],
Cell[CellGroupData[{
Cell[209008, 8268, 63, 1, 34, "Subsubsection",
 CellTags->"CheckContext"],
Cell[209074, 8271, 164, 4, 24, "Text",
 CellTags->"CheckContext"],
Cell[209241, 8277, 320, 14, 24, "Text",
 CellTags->"CheckContext"]
}, Open  ]],
Cell[CellGroupData[{
Cell[209598, 8296, 60, 1, 34, "Subsubsection",
 CellTags->"CheckContext"],
Cell[209661, 8299, 142, 4, 24, "Text",
 CellTags->"CheckContext"],
Cell[CellGroupData[{
Cell[209828, 8307, 138, 3, 26, "Input",
 CellTags->"CheckContext"],
Cell[209969, 8312, 156, 4, 26, "Output",
 CellTags->"CheckContext"]
}, Open  ]],
Cell[210140, 8319, 87, 1, 24, "Text",
 CellTags->"CheckContext"],
Cell[CellGroupData[{
Cell[210252, 8324, 92, 2, 26, "Input",
 CellTags->"CheckContext"],
Cell[210347, 8328, 163, 4, 26, "Output",
 CellTags->"CheckContext"]
}, Open  ]],
Cell[CellGroupData[{
Cell[210547, 8337, 138, 3, 26, "Input",
 CellTags->"CheckContext"],
Cell[210688, 8342, 153, 4, 26, "Output",
 CellTags->"CheckContext"]
}, Open  ]]
}, Open  ]],
Cell[210868, 8350, 51, 1, 39, "ObjectName",
 CellTags->"CheckDB"],
Cell[CellGroupData[{
Cell[210944, 8355, 106, 3, 34, "Subsubsection",
 CellTags->"CheckDB"],
Cell[211053, 8360, 1081, 15, 126, "Text"],
Cell[CellGroupData[{
Cell[212159, 8379, 113, 3, 26, "Input",
 CellTags->"CheckDB"],
Cell[212275, 8384, 512, 15, 26, "Output",
 CellTags->"CheckDB"]
}, Open  ]],
Cell[CellGroupData[{
Cell[212824, 8404, 116, 3, 26, "Input",
 CellTags->"CheckDB"],
Cell[212943, 8409, 175, 5, 26, "Output",
 CellTags->"CheckDB"]
}, Open  ]],
Cell[213133, 8417, 115, 3, 24, "Text"],
Cell[CellGroupData[{
Cell[213273, 8424, 337, 11, 26, "Input"],
Cell[213613, 8437, 219, 6, 22, "Print"],
Cell[213835, 8445, 222, 6, 22, "Print"],
Cell[214060, 8453, 172, 5, 26, "Output"]
}, Open  ]],
Cell[214247, 8461, 133, 3, 24, "Text"],
Cell[CellGroupData[{
Cell[214405, 8468, 383, 13, 26, "Input",
 CellTags->"CheckDB"],
Cell[214791, 8483, 192, 6, 26, "Output",
 CellTags->"CheckDB"]
}, Open  ]],
Cell[214998, 8492, 60, 0, 24, "Text"],
Cell[CellGroupData[{
Cell[215083, 8496, 303, 9, 26, "Input",
 CellTags->"CheckDB"],
Cell[215389, 8507, 158, 4, 26, "Output",
 CellTags->"CheckDB"]
}, Open  ]],
Cell[215562, 8514, 268, 8, 26, "Input",
 CellTags->"CheckDB"]
}, Open  ]],
Cell[215845, 8525, 73, 1, 39, "ObjectName",
 CellTags->"ChiralityProjector"],
Cell[CellGroupData[{
Cell[215943, 8530, 69, 1, 34, "Subsubsection",
 CellTags->"ChiralityProjector"],
Cell[216015, 8533, 499, 19, 27, "Text",
 CellTags->"ChiralityProjector"],
Cell[216517, 8554, 409, 19, 24, "Text",
 CellTags->"ChiralityProjector"]
}, Open  ]],
Cell[CellGroupData[{
Cell[216963, 8578, 66, 1, 34, "Subsubsection",
 CellTags->"ChiralityProjector"],
Cell[CellGroupData[{
Cell[217054, 8583, 262, 8, 26, "Input",
 CellTags->"ChiralityProjector"],
Cell[217319, 8593, 268, 8, 29, "Output",
 CellTags->"ChiralityProjector"]
}, Open  ]],
Cell[CellGroupData[{
Cell[217624, 8606, 114, 3, 26, "Input",
 CellTags->"ChiralityProjector"],
Cell[217741, 8611, 269, 8, 29, "Output",
 CellTags->"ChiralityProjector"]
}, Open  ]],
Cell[CellGroupData[{
Cell[218047, 8624, 196, 5, 26, "Input",
 CellTags->"ChiralityProjector"],
Cell[218246, 8631, 422, 15, 46, "Output",
 CellTags->"ChiralityProjector"]
}, Open  ]]
}, Open  ]],
Cell[218695, 8650, 53, 1, 39, "ObjectName",
 CellTags->"Chisholm"],
Cell[CellGroupData[{
Cell[218773, 8655, 59, 1, 34, "Subsubsection",
 CellTags->"Chisholm"],
Cell[218835, 8658, 142, 4, 24, "Text",
 CellTags->"Chisholm"],
Cell[218980, 8664, 316, 14, 24, "Text",
 CellTags->"Chisholm"]
}, Open  ]],
Cell[CellGroupData[{
Cell[219333, 8683, 56, 1, 34, "Subsubsection",
 CellTags->"Chisholm"],
Cell[CellGroupData[{
Cell[219414, 8688, 206, 6, 26, "Input",
 CellTags->"Chisholm"],
Cell[219623, 8696, 2062, 79, 26, "Output",
 CellTags->"Chisholm"]
}, Open  ]],
Cell[CellGroupData[{
Cell[221722, 8780, 183, 5, 26, "Input",
 CellTags->"Chisholm"],
Cell[221908, 8787, 280, 8, 26, "Output",
 CellTags->"Chisholm"]
}, Open  ]],
Cell[CellGroupData[{
Cell[222225, 8800, 139, 4, 26, "Input",
 CellTags->"Chisholm"],
Cell[222367, 8806, 1415, 55, 26, "Output",
 CellTags->"Chisholm"]
}, Open  ]],
Cell[223797, 8864, 77, 1, 24, "Text",
 CellTags->"Chisholm"],
Cell[CellGroupData[{
Cell[223899, 8869, 130, 4, 26, "Input",
 CellTags->"Chisholm"],
Cell[224032, 8875, 147, 4, 26, "Output",
 CellTags->"Chisholm"]
}, Open  ]],
Cell[CellGroupData[{
Cell[224216, 8884, 139, 4, 26, "Input",
 CellTags->"Chisholm"],
Cell[224358, 8890, 1416, 55, 26, "Output",
 CellTags->"Chisholm"]
}, Open  ]],
Cell[CellGroupData[{
Cell[225811, 8950, 130, 4, 26, "Input",
 CellTags->"Chisholm"],
Cell[225944, 8956, 147, 4, 26, "Output",
 CellTags->"Chisholm"]
}, Open  ]],
Cell[CellGroupData[{
Cell[226128, 8965, 169, 5, 26, "Input",
 CellTags->"Chisholm"],
Cell[226300, 8972, 632, 23, 26, "Output",
 CellTags->"Chisholm"]
}, Open  ]],
Cell[CellGroupData[{
Cell[226969, 9000, 110, 3, 26, "Input",
 CellTags->"Chisholm"],
Cell[227082, 9005, 1481, 58, 28, "Output",
 CellTags->"Chisholm"]
}, Open  ]],
Cell[CellGroupData[{
Cell[228600, 9068, 232, 7, 26, "Input",
 CellTags->"Chisholm"],
Cell[228835, 9077, 433, 11, 26, "Output",
 CellTags->"Chisholm"]
}, Open  ]],
Cell[CellGroupData[{
Cell[229305, 9093, 139, 4, 26, "Input",
 CellTags->"Chisholm"],
Cell[229447, 9099, 11586, 440, 127, "Output",
 CellTags->"Chisholm"]
}, Open  ]],
Cell[CellGroupData[{
Cell[241070, 9544, 139, 4, 26, "Input",
 CellTags->"Chisholm"],
Cell[241212, 9550, 11586, 440, 127, "Output",
 CellTags->"Chisholm"]
}, Open  ]],
Cell[252813, 9993, 85, 1, 24, "Text",
 CellTags->"Chisholm"],
Cell[CellGroupData[{
Cell[252923, 9998, 130, 4, 26, "Input",
 CellTags->"Chisholm"],
Cell[253056, 10004, 167, 5, 26, "Output",
 CellTags->"Chisholm"]
}, Open  ]],
Cell[CellGroupData[{
Cell[253260, 10014, 130, 4, 26, "Input",
 CellTags->"Chisholm"],
Cell[253393, 10020, 169, 5, 26, "Output",
 CellTags->"Chisholm"]
}, Open  ]],
Cell[253577, 10028, 192, 6, 26, "Input",
 CellTags->"Chisholm"]
}, Open  ]],
Cell[253784, 10037, 65, 1, 39, "ObjectName",
 CellTags->"ChisholmSpinor"],
Cell[CellGroupData[{
Cell[253874, 10042, 65, 1, 34, "Subsubsection",
 CellTags->"ChisholmSpinor"],
Cell[253942, 10045, 296, 6, 41, "Text",
 CellTags->"ChisholmSpinor"],
Cell[254241, 10053, 304, 14, 24, "Text",
 CellTags->"ChisholmSpinor"]
}, Open  ]],
Cell[CellGroupData[{
Cell[254582, 10072, 62, 1, 34, "Subsubsection",
 CellTags->"ChisholmSpinor"],
Cell[CellGroupData[{
Cell[254669, 10077, 312, 9, 26, "Input",
 CellTags->"ChisholmSpinor"],
Cell[254984, 10088, 469, 16, 26, "Output",
 CellTags->"ChisholmSpinor"]
}, Open  ]],
Cell[CellGroupData[{
Cell[255490, 10109, 117, 3, 26, "Input",
 CellTags->"ChisholmSpinor"],
Cell[255610, 10114, 2728, 100, 94, "Output",
 CellTags->"ChisholmSpinor"]
}, Open  ]]
}, Open  ]],
Cell[258365, 10218, 75, 1, 39, "ObjectName",
 CellTags->"ClearScalarProducts"],
Cell[CellGroupData[{
Cell[258465, 10223, 70, 1, 34, "Subsubsection",
 CellTags->"ClearScalarProducts"],
Cell[258538, 10226, 145, 4, 24, "Text",
 CellTags->"ClearScalarProducts"],
Cell[258686, 10232, 484, 23, 24, "Text",
 CellTags->"ClearScalarProducts"]
}, Open  ]],
Cell[CellGroupData[{
Cell[259207, 10260, 67, 1, 34, "Subsubsection",
 CellTags->"ClearScalarProducts"],
Cell[CellGroupData[{
Cell[259299, 10265, 195, 6, 26, "Input",
 CellTags->"ClearScalarProducts"],
Cell[259497, 10273, 183, 5, 26, "Output",
 CellTags->"ClearScalarProducts"]
}, Open  ]],
Cell[CellGroupData[{
Cell[259717, 10283, 211, 6, 26, "Input",
 CellTags->"ClearScalarProducts"],
Cell[259931, 10291, 183, 5, 26, "Output",
 CellTags->"ClearScalarProducts"]
}, Open  ]],
Cell[260129, 10299, 104, 2, 26, "Input",
 CellTags->"ClearScalarProducts"],
Cell[CellGroupData[{
Cell[260258, 10305, 211, 6, 26, "Input",
 CellTags->"ClearScalarProducts"],
Cell[260472, 10313, 217, 7, 28, "Output",
 CellTags->"ClearScalarProducts"]
}, Open  ]],
Cell[CellGroupData[{
Cell[260726, 10325, 148, 4, 26, "Input",
 CellTags->"ClearScalarProducts"],
Cell[260877, 10331, 215, 7, 28, "Output",
 CellTags->"ClearScalarProducts"]
}, Open  ]]
}, Open  ]],
Cell[261119, 10342, 57, 1, 39, "ObjectName",
 CellTags->"Collecting"],
Cell[CellGroupData[{
Cell[261201, 10347, 61, 1, 34, "Subsubsection",
 CellTags->"Collecting"],
Cell[261265, 10350, 207, 5, 41, "Text",
 CellTags->"Collecting"],
Cell[261475, 10357, 411, 18, 24, "Text",
 CellTags->"Collecting"]
}, Open  ]],
Cell[261901, 10378, 53, 1, 39, "ObjectName",
 CellTags->"Collect2"],
Cell[CellGroupData[{
Cell[261979, 10383, 59, 1, 34, "Subsubsection",
 CellTags->"Collect2"],
Cell[262041, 10386, 613, 10, 75, "Text",
 CellTags->"Collect2"],
Cell[CellGroupData[{
Cell[262679, 10400, 115, 3, 26, "Input",
 CellTags->"Collect2"],
Cell[262797, 10405, 440, 12, 26, "Output",
 CellTags->"Collect2"]
}, Open  ]],
Cell[263252, 10420, 176, 9, 24, "Text",
 CellTags->"Collect2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[263465, 10434, 56, 1, 34, "Subsubsection",
 CellTags->"Collect2"],
Cell[CellGroupData[{
Cell[263546, 10439, 440, 14, 47, "Input",
 CellTags->"Collect2"],
Cell[263989, 10455, 493, 17, 43, "Output",
 CellTags->"Collect2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[264519, 10477, 185, 5, 26, "Input",
 CellTags->"Collect2"],
Cell[264707, 10484, 441, 15, 44, "Output",
 CellTags->"Collect2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[265185, 10504, 186, 5, 26, "Input",
 CellTags->"Collect2"],
Cell[265374, 10511, 495, 17, 43, "Output",
 CellTags->"Collect2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[265906, 10533, 525, 18, 30, "Input",
 CellTags->"Collect2"],
Cell[266434, 10553, 581, 19, 26, "Output",
 CellTags->"Collect2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[267052, 10577, 320, 10, 30, "Input",
 CellTags->"Collect2"],
Cell[267375, 10589, 862, 32, 26, "Output",
 CellTags->"Collect2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[268274, 10626, 112, 3, 26, "Input",
 CellTags->"Collect2"],
Cell[268389, 10631, 867, 28, 26, "Output",
 CellTags->"Collect2"]
}, Open  ]],
Cell[269271, 10662, 130, 4, 26, "Input",
 CellTags->"Collect2"],
Cell[CellGroupData[{
Cell[269426, 10670, 305, 9, 30, "Input",
 CellTags->"Collect2"],
Cell[269734, 10681, 979, 32, 29, "Output",
 CellTags->"Collect2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[270750, 10718, 149, 4, 26, "Input",
 CellTags->"Collect2"],
Cell[270902, 10724, 715, 22, 29, "Output",
 CellTags->"Collect2"]
}, Open  ]]
}, Open  ]],
Cell[271644, 10750, 53, 1, 39, "ObjectName",
 CellTags->"Collect3"],
Cell[CellGroupData[{
Cell[271722, 10755, 59, 1, 34, "Subsubsection",
 CellTags->"Collect3"],
Cell[271784, 10758, 719, 23, 58, "Text",
 CellTags->"Collect3"],
Cell[CellGroupData[{
Cell[272528, 10785, 115, 3, 26, "Input",
 CellTags->"Collect3"],
Cell[272646, 10790, 275, 9, 26, "Output",
 CellTags->"Collect3"]
}, Open  ]],
Cell[272936, 10802, 286, 14, 24, "Text",
 CellTags->"Collect3"]
}, Open  ]],
Cell[CellGroupData[{
Cell[273259, 10821, 56, 1, 34, "Subsubsection",
 CellTags->"Collect3"],
Cell[CellGroupData[{
Cell[273340, 10826, 525, 18, 30, "Input",
 CellTags->"Collect3"],
Cell[273868, 10846, 902, 18, 98, "Message",
 CellTags->"Collect3"],
Cell[274773, 10866, 902, 18, 98, "Message",
 CellTags->"Collect3"],
Cell[275678, 10886, 164, 5, 26, "Output",
 CellTags->"Collect3"]
}, Open  ]],
Cell[CellGroupData[{
Cell[275879, 10896, 289, 9, 30, "Input",
 CellTags->"Collect3"],
Cell[276171, 10907, 649, 14, 22, "Message",
 CellTags->"Collect3"],
Cell[276823, 10923, 164, 5, 26, "Output",
 CellTags->"Collect3"]
}, Open  ]]
}, Open  ]],
Cell[277014, 10932, 51, 1, 39, "ObjectName",
 CellTags->"Combine"],
Cell[CellGroupData[{
Cell[277090, 10937, 58, 1, 34, "Subsubsection",
 CellTags->"Combine"],
Cell[277151, 10940, 315, 6, 41, "Text",
 CellTags->"Combine"],
Cell[CellGroupData[{
Cell[277491, 10950, 113, 3, 26, "Input",
 CellTags->"Combine"],
Cell[277607, 10955, 213, 7, 26, "Output",
 CellTags->"Combine"]
}, Open  ]],
Cell[277835, 10965, 175, 9, 24, "Text",
 CellTags->"Combine"]
}, Open  ]],
Cell[CellGroupData[{
Cell[278047, 10979, 55, 1, 34, "Subsubsection",
 CellTags->"Combine"],
Cell[CellGroupData[{
Cell[278127, 10984, 283, 10, 50, "Input",
 CellTags->"Combine"],
Cell[278413, 10996, 346, 12, 47, "Output",
 CellTags->"Combine"]
}, Open  ]],
Cell[278774, 11011, 122, 3, 24, "Text",
 CellTags->"Combine"],
Cell[CellGroupData[{
Cell[278921, 11018, 284, 10, 50, "Input",
 CellTags->"Combine"],
Cell[279208, 11030, 345, 11, 47, "Output",
 CellTags->"Combine"]
}, Open  ]],
Cell[279568, 11044, 175, 4, 24, "Text",
 CellTags->"Combine"],
Cell[CellGroupData[{
Cell[279768, 11052, 354, 12, 50, "Input",
 CellTags->"Combine"],
Cell[280125, 11066, 344, 11, 47, "Output",
 CellTags->"Combine"]
}, Open  ]]
}, Open  ]],
Cell[280496, 11081, 63, 1, 39, "ObjectName",
 CellTags->"CombineGraphs"],
Cell[CellGroupData[{
Cell[280584, 11086, 64, 1, 34, "Subsubsection",
 CellTags->"CombineGraphs"],
Cell[280651, 11089, 86, 1, 24, "Text",
 CellTags->"CombineGraphs"],
Cell[280740, 11092, 185, 8, 24, "Text",
 CellTags->"CombineGraphs"]
}, Open  ]],
Cell[280940, 11103, 57, 1, 39, "ObjectName",
 CellTags->"Commutator"],
Cell[CellGroupData[{
Cell[281022, 11108, 61, 1, 34, "Subsubsection",
 CellTags->"Commutator"],
Cell[281086, 11111, 138, 4, 24, "Text",
 CellTags->"Commutator"],
Cell[281227, 11117, 607, 24, 24, "Text",
 CellTags->"Commutator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[281871, 11146, 58, 1, 34, "Subsubsection",
 CellTags->"Commutator"],
Cell[281932, 11149, 167, 4, 26, "Input",
 CellTags->"Commutator"],
Cell[CellGroupData[{
Cell[282124, 11157, 136, 4, 26, "Input",
 CellTags->"Commutator"],
Cell[282263, 11163, 226, 6, 26, "Output",
 CellTags->"Commutator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[282526, 11174, 121, 3, 26, "Input",
 CellTags->"Commutator"],
Cell[282650, 11179, 219, 7, 26, "Output",
 CellTags->"Commutator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[282906, 11191, 247, 8, 26, "Input",
 CellTags->"Commutator"],
Cell[283156, 11201, 423, 13, 26, "Output",
 CellTags->"Commutator"]
}, Open  ]],
Cell[283594, 11217, 169, 4, 26, "Input",
 CellTags->"Commutator"],
Cell[283766, 11223, 68, 1, 24, "Text",
 CellTags->"Commutator"],
Cell[283837, 11226, 228, 6, 26, "Input",
 CellTags->"Commutator"],
Cell[CellGroupData[{
Cell[284090, 11236, 463, 15, 26, "Input",
 CellTags->"Commutator"],
Cell[284556, 11253, 771, 20, 26, "Output",
 CellTags->"Commutator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[285364, 11278, 114, 3, 26, "Input",
 CellTags->"Commutator"],
Cell[285481, 11283, 148, 4, 26, "Output",
 CellTags->"Commutator"]
}, Open  ]],
Cell[285644, 11290, 113, 3, 26, "Input",
 CellTags->"Commutator"],
Cell[285760, 11295, 159, 4, 26, "Input",
 CellTags->"Commutator"]
}, Open  ]],
Cell[285934, 11302, 73, 1, 39, "ObjectName",
 CellTags->"CommutatorExplicit"],
Cell[CellGroupData[{
Cell[286032, 11307, 69, 1, 34, "Subsubsection",
 CellTags->"CommutatorExplicit"],
Cell[286104, 11310, 157, 4, 24, "Text",
 CellTags->"CommutatorExplicit"],
Cell[286264, 11316, 296, 14, 24, "Text",
 CellTags->"CommutatorExplicit"]
}, Open  ]],
Cell[CellGroupData[{
Cell[286597, 11335, 66, 1, 34, "Subsubsection",
 CellTags->"CommutatorExplicit"],
Cell[286666, 11338, 175, 4, 26, "Input",
 CellTags->"CommutatorExplicit"],
Cell[CellGroupData[{
Cell[286866, 11346, 144, 4, 26, "Input",
 CellTags->"CommutatorExplicit"],
Cell[287013, 11352, 234, 6, 26, "Output",
 CellTags->"CommutatorExplicit"]
}, Open  ]],
Cell[CellGroupData[{
Cell[287284, 11363, 129, 3, 26, "Input",
 CellTags->"CommutatorExplicit"],
Cell[287416, 11368, 227, 7, 26, "Output",
 CellTags->"CommutatorExplicit"]
}, Open  ]],
Cell[CellGroupData[{
Cell[287680, 11380, 196, 6, 26, "Input",
 CellTags->"CommutatorExplicit"],
Cell[287879, 11388, 325, 11, 26, "Output",
 CellTags->"CommutatorExplicit"]
}, Open  ]],
Cell[CellGroupData[{
Cell[288241, 11404, 129, 3, 26, "Input",
 CellTags->"CommutatorExplicit"],
Cell[288373, 11409, 431, 15, 26, "Output",
 CellTags->"CommutatorExplicit"]
}, Open  ]],
Cell[CellGroupData[{
Cell[288841, 11429, 167, 5, 26, "Input",
 CellTags->"CommutatorExplicit"],
Cell[289011, 11436, 431, 13, 26, "Output",
 CellTags->"CommutatorExplicit"]
}, Open  ]],
Cell[289457, 11452, 177, 4, 26, "Input",
 CellTags->"CommutatorExplicit"]
}, Open  ]],
Cell[289649, 11459, 65, 1, 39, "ObjectName",
 CellTags->"CompleteSquare"],
Cell[CellGroupData[{
Cell[289739, 11464, 65, 1, 34, "Subsubsection",
 CellTags->"CompleteSquare"],
Cell[289807, 11467, 1107, 82, 41, "Text",
 CellTags->"CompleteSquare"]
}, Open  ]],
Cell[CellGroupData[{
Cell[290951, 11554, 62, 1, 34, "Subsubsection",
 CellTags->"CompleteSquare"],
Cell[CellGroupData[{
Cell[291038, 11559, 277, 10, 26, "Input"],
Cell[291318, 11571, 379, 13, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[291734, 11589, 170, 5, 26, "Input",
 CellTags->"CompleteSquare"],
Cell[291907, 11596, 727, 28, 48, "Output",
 CellTags->"CompleteSquare"]
}, Open  ]],
Cell[CellGroupData[{
Cell[292671, 11629, 180, 6, 26, "Input",
 CellTags->"CompleteSquare"],
Cell[292854, 11637, 152, 4, 26, "Output",
 CellTags->"CompleteSquare"]
}, Open  ]],
Cell[CellGroupData[{
Cell[293043, 11646, 155, 4, 26, "Input",
 CellTags->"CompleteSquare"],
Cell[293201, 11652, 557, 21, 46, "Output",
 CellTags->"CompleteSquare"]
}, Open  ]],
Cell[293773, 11676, 137, 4, 26, "Input",
 CellTags->"CompleteSquare"]
}, Open  ]],
Cell[293925, 11683, 69, 1, 39, "ObjectName",
 CellTags->"ComplexConjugate"],
Cell[CellGroupData[{
Cell[294019, 11688, 67, 1, 34, "Subsubsection",
 CellTags->"ComplexConjugate"],
Cell[294089, 11691, 414, 7, 58, "Text",
 CellTags->"ComplexConjugate"],
Cell[294506, 11700, 449, 19, 24, "Text",
 CellTags->"ComplexConjugate"]
}, Open  ]],
Cell[CellGroupData[{
Cell[294992, 11724, 64, 1, 34, "Subsubsection",
 CellTags->"ComplexConjugate"],
Cell[CellGroupData[{
Cell[295081, 11729, 196, 5, 26, "Input",
 CellTags->"ComplexConjugate"],
Cell[295280, 11736, 207, 6, 26, "Output",
 CellTags->"ComplexConjugate"]
}, Open  ]],
Cell[CellGroupData[{
Cell[295524, 11747, 121, 3, 26, "Input",
 CellTags->"ComplexConjugate"],
Cell[295648, 11752, 281, 7, 38, "Output",
 CellTags->"ComplexConjugate"]
}, Open  ]],
Cell[CellGroupData[{
Cell[295966, 11764, 152, 4, 26, "Input",
 CellTags->"ComplexConjugate"],
Cell[296121, 11770, 291, 8, 26, "Output",
 CellTags->"ComplexConjugate"]
}, Open  ]],
Cell[CellGroupData[{
Cell[296449, 11783, 125, 3, 26, "Input",
 CellTags->"ComplexConjugate"],
Cell[296577, 11788, 469, 17, 26, "Output",
 CellTags->"ComplexConjugate"]
}, Open  ]],
Cell[CellGroupData[{
Cell[297083, 11810, 182, 5, 26, "Input",
 CellTags->"ComplexConjugate"],
Cell[297268, 11817, 573, 23, 28, "Output",
 CellTags->"ComplexConjugate"]
}, Open  ]],
Cell[CellGroupData[{
Cell[297878, 11845, 125, 3, 26, "Input",
 CellTags->"ComplexConjugate"],
Cell[298006, 11850, 575, 23, 28, "Output",
 CellTags->"ComplexConjugate"]
}, Open  ]],
Cell[CellGroupData[{
Cell[298618, 11878, 190, 5, 26, "Input",
 CellTags->"ComplexConjugate"],
Cell[298811, 11885, 424, 18, 28, "Output",
 CellTags->"ComplexConjugate"]
}, Open  ]],
Cell[CellGroupData[{
Cell[299272, 11908, 270, 7, 26, "Input",
 CellTags->"ComplexConjugate"],
Cell[299545, 11917, 199, 5, 38, "Output",
 CellTags->"ComplexConjugate"]
}, Open  ]],
Cell[CellGroupData[{
Cell[299781, 11927, 273, 8, 26, "Input",
 CellTags->"ComplexConjugate"],
Cell[300057, 11937, 377, 11, 26, "Output",
 CellTags->"ComplexConjugate"]
}, Open  ]],
Cell[CellGroupData[{
Cell[300471, 11953, 125, 3, 26, "Input",
 CellTags->"ComplexConjugate"],
Cell[300599, 11958, 473, 16, 26, "Output",
 CellTags->"ComplexConjugate"]
}, Open  ]],
Cell[301087, 11977, 166, 4, 24, "Text",
 CellTags->"ComplexConjugate"]
}, Open  ]],
Cell[301268, 11984, 53, 1, 39, "ObjectName",
 CellTags->"Contract"],
Cell[CellGroupData[{
Cell[301346, 11989, 59, 1, 34, "Subsubsection",
 CellTags->"Contract"],
Cell[301408, 11992, 411, 7, 58, "Text",
 CellTags->"Contract"],
Cell[CellGroupData[{
Cell[301844, 12003, 115, 3, 26, "Input",
 CellTags->"Contract"],
Cell[301962, 12008, 654, 16, 47, "Output",
 CellTags->"Contract"]
}, Open  ]],
Cell[302631, 12027, 138, 3, 24, "Text"],
Cell[302772, 12032, 423, 19, 24, "Text",
 CellTags->"Contract"]
}, Open  ]],
Cell[CellGroupData[{
Cell[303232, 12056, 56, 1, 34, "Subsubsection",
 CellTags->"Contract"],
Cell[CellGroupData[{
Cell[303313, 12061, 233, 7, 26, "Input",
 CellTags->"Contract"],
Cell[303549, 12070, 249, 8, 29, "Output",
 CellTags->"Contract"]
}, Open  ]],
Cell[CellGroupData[{
Cell[303835, 12083, 109, 3, 26, "Input",
 CellTags->"Contract"],
Cell[303947, 12088, 280, 11, 28, "Output",
 CellTags->"Contract"]
}, Open  ]],
Cell[CellGroupData[{
Cell[304264, 12104, 199, 6, 26, "Input",
 CellTags->"Contract"],
Cell[304466, 12112, 306, 11, 29, "Output",
 CellTags->"Contract"]
}, Open  ]],
Cell[CellGroupData[{
Cell[304809, 12128, 109, 3, 26, "Input",
 CellTags->"Contract"],
Cell[304921, 12133, 265, 9, 28, "Output",
 CellTags->"Contract"]
}, Open  ]],
Cell[CellGroupData[{
Cell[305223, 12147, 144, 4, 26, "Input",
 CellTags->"Contract"],
Cell[305370, 12153, 196, 6, 26, "Output",
 CellTags->"Contract"]
}, Open  ]],
Cell[305581, 12162, 83, 1, 24, "Text",
 CellTags->"Contract"],
Cell[CellGroupData[{
Cell[305689, 12167, 109, 3, 26, "Input",
 CellTags->"Contract"],
Cell[305801, 12172, 146, 4, 26, "Output",
 CellTags->"Contract"]
}, Open  ]],
Cell[305962, 12179, 208, 5, 41, "Text",
 CellTags->"Contract"],
Cell[CellGroupData[{
Cell[306195, 12188, 231, 7, 26, "Input",
 CellTags->"Contract"],
Cell[306429, 12197, 275, 9, 26, "Output",
 CellTags->"Contract"]
}, Open  ]],
Cell[CellGroupData[{
Cell[306741, 12211, 109, 3, 26, "Input",
 CellTags->"Contract"],
Cell[306853, 12216, 146, 4, 27, "Output",
 CellTags->"Contract"]
}, Open  ]],
Cell[CellGroupData[{
Cell[307036, 12225, 231, 7, 26, "Input",
 CellTags->"Contract"],
Cell[307270, 12234, 277, 9, 26, "Output",
 CellTags->"Contract"]
}, Open  ]],
Cell[CellGroupData[{
Cell[307584, 12248, 227, 7, 26, "Input",
 CellTags->"Contract"],
Cell[307814, 12257, 225, 7, 29, "Output",
 CellTags->"Contract"]
}, Open  ]],
Cell[CellGroupData[{
Cell[308076, 12269, 114, 3, 26, "Input",
 CellTags->"Contract"],
Cell[308193, 12274, 248, 9, 28, "Output",
 CellTags->"Contract"]
}, Open  ]],
Cell[CellGroupData[{
Cell[308478, 12288, 277, 9, 26, "Input",
 CellTags->"Contract"],
Cell[308758, 12299, 469, 19, 30, "Output",
 CellTags->"Contract"]
}, Open  ]],
Cell[CellGroupData[{
Cell[309264, 12323, 109, 3, 26, "Input",
 CellTags->"Contract"],
Cell[309376, 12328, 617, 25, 28, "Output",
 CellTags->"Contract"]
}, Open  ]],
Cell[CellGroupData[{
Cell[310030, 12358, 271, 8, 26, "Input",
 CellTags->"Contract"],
Cell[310304, 12368, 596, 24, 28, "Output",
 CellTags->"Contract"]
}, Open  ]],
Cell[CellGroupData[{
Cell[310937, 12397, 109, 3, 26, "Input",
 CellTags->"Contract"],
Cell[311049, 12402, 481, 20, 26, "Output",
 CellTags->"Contract"]
}, Open  ]],
Cell[CellGroupData[{
Cell[311567, 12427, 313, 9, 26, "Input",
 CellTags->"Contract"],
Cell[311883, 12438, 964, 41, 26, "Output",
 CellTags->"Contract"]
}, Open  ]],
Cell[CellGroupData[{
Cell[312884, 12484, 109, 3, 26, "Input",
 CellTags->"Contract"],
Cell[312996, 12489, 246, 8, 26, "Output",
 CellTags->"Contract"]
}, Open  ]],
Cell[CellGroupData[{
Cell[313279, 12502, 426, 13, 26, "Input",
 CellTags->"Contract"],
Cell[313708, 12517, 345, 10, 26, "Output",
 CellTags->"Contract"]
}, Open  ]],
Cell[CellGroupData[{
Cell[314090, 12532, 139, 4, 26, "Input",
 CellTags->"Contract"],
Cell[314232, 12538, 393, 13, 26, "Output",
 CellTags->"Contract"]
}, Open  ]],
Cell[314640, 12554, 194, 6, 26, "Input",
 CellTags->"Contract"]
}, Open  ]],
Cell[314849, 12563, 55, 1, 39, "ObjectName",
 CellTags->"Contract1"],
Cell[CellGroupData[{
Cell[314929, 12568, 60, 1, 34, "Subsubsection",
 CellTags->"Contract1"],
Cell[314992, 12571, 459, 8, 99, "Text",
 CellTags->"Contract1"],
Cell[315454, 12581, 389, 18, 24, "Text",
 CellTags->"Contract1"]
}, Open  ]],
Cell[CellGroupData[{
Cell[315880, 12604, 57, 1, 34, "Subsubsection",
 CellTags->"Contract1"],
Cell[CellGroupData[{
Cell[315962, 12609, 349, 12, 26, "Input",
 CellTags->"Contract1"],
Cell[316314, 12623, 251, 9, 28, "Output",
 CellTags->"Contract1"]
}, Open  ]],
Cell[CellGroupData[{
Cell[316602, 12637, 349, 12, 26, "Input",
 CellTags->"Contract1"],
Cell[316954, 12651, 520, 21, 28, "Output",
 CellTags->"Contract1"]
}, Open  ]],
Cell[CellGroupData[{
Cell[317511, 12677, 269, 9, 26, "Input",
 CellTags->"Contract1"],
Cell[317783, 12688, 147, 4, 26, "Output",
 CellTags->"Contract1"]
}, Open  ]],
Cell[CellGroupData[{
Cell[317967, 12697, 269, 9, 26, "Input",
 CellTags->"Contract1"],
Cell[318239, 12708, 338, 12, 26, "Output",
 CellTags->"Contract1"]
}, Open  ]]
}, Open  ]],
Cell[318604, 12724, 55, 1, 39, "ObjectName",
 CellTags->"Convolute"],
Cell[CellGroupData[{
Cell[318684, 12729, 60, 1, 34, "Subsubsection",
 CellTags->"Convolute"],
Cell[318747, 12732, 1363, 49, 65, "Text",
 CellTags->"Convolute"],
Cell[CellGroupData[{
Cell[320135, 12785, 117, 3, 26, "Input",
 CellTags->"Convolute"],
Cell[320255, 12790, 442, 14, 26, "Output",
 CellTags->"Convolute"]
}, Open  ]],
Cell[320712, 12807, 332, 14, 24, "Text",
 CellTags->"Convolute"]
}, Open  ]],
Cell[CellGroupData[{
Cell[321081, 12826, 57, 1, 34, "Subsubsection",
 CellTags->"Convolute"],
Cell[CellGroupData[{
Cell[321163, 12831, 134, 4, 26, "Input",
 CellTags->"Convolute"],
Cell[321300, 12837, 195, 6, 26, "Output",
 CellTags->"Convolute"]
}, Open  ]],
Cell[CellGroupData[{
Cell[321532, 12848, 134, 4, 26, "Input",
 CellTags->"Convolute"],
Cell[321669, 12854, 219, 7, 26, "Output",
 CellTags->"Convolute"]
}, Open  ]],
Cell[CellGroupData[{
Cell[321925, 12866, 134, 4, 26, "Input",
 CellTags->"Convolute"],
Cell[322062, 12872, 172, 5, 26, "Output",
 CellTags->"Convolute"]
}, Open  ]],
Cell[CellGroupData[{
Cell[322271, 12882, 181, 6, 49, "Input",
 CellTags->"Convolute"],
Cell[322455, 12890, 257, 8, 26, "Output",
 CellTags->"Convolute"]
}, Open  ]],
Cell[CellGroupData[{
Cell[322749, 12903, 227, 7, 49, "Input",
 CellTags->"Convolute"],
Cell[322979, 12912, 259, 8, 26, "Output",
 CellTags->"Convolute"]
}, Open  ]],
Cell[CellGroupData[{
Cell[323275, 12925, 181, 6, 49, "Input",
 CellTags->"Convolute"],
Cell[323459, 12933, 349, 12, 26, "Output",
 CellTags->"Convolute"]
}, Open  ]],
Cell[CellGroupData[{
Cell[323845, 12950, 228, 8, 49, "Input",
 CellTags->"Convolute"],
Cell[324076, 12960, 578, 20, 44, "Output",
 CellTags->"Convolute"]
}, Open  ]],
Cell[CellGroupData[{
Cell[324691, 12985, 190, 6, 26, "Input",
 CellTags->"Convolute"],
Cell[324884, 12993, 363, 12, 26, "Output",
 CellTags->"Convolute"]
}, Open  ]],
Cell[CellGroupData[{
Cell[325284, 13010, 216, 7, 26, "Input",
 CellTags->"Convolute"],
Cell[325503, 13019, 317, 11, 26, "Output",
 CellTags->"Convolute"]
}, Open  ]],
Cell[CellGroupData[{
Cell[325857, 13035, 237, 8, 49, "Input",
 CellTags->"Convolute"],
Cell[326097, 13045, 433, 14, 26, "Output",
 CellTags->"Convolute"]
}, Open  ]],
Cell[CellGroupData[{
Cell[326567, 13064, 263, 9, 49, "Input",
 CellTags->"Convolute"],
Cell[326833, 13075, 661, 22, 26, "Output",
 CellTags->"Convolute"]
}, Open  ]],
Cell[CellGroupData[{
Cell[327531, 13102, 239, 8, 49, "Input",
 CellTags->"Convolute"],
Cell[327773, 13112, 759, 25, 44, "Output",
 CellTags->"Convolute"]
}, Open  ]],
Cell[CellGroupData[{
Cell[328569, 13142, 190, 6, 26, "Input",
 CellTags->"Convolute"],
Cell[328762, 13150, 289, 10, 26, "Output",
 CellTags->"Convolute"]
}, Open  ]],
Cell[CellGroupData[{
Cell[329088, 13165, 190, 6, 26, "Input",
 CellTags->"Convolute"],
Cell[329281, 13173, 358, 12, 26, "Output",
 CellTags->"Convolute"]
}, Open  ]],
Cell[CellGroupData[{
Cell[329676, 13190, 260, 9, 49, "Input",
 CellTags->"Convolute"],
Cell[329939, 13201, 493, 17, 49, "Output",
 CellTags->"Convolute"]
}, Open  ]],
Cell[CellGroupData[{
Cell[330469, 13223, 165, 5, 26, "Input",
 CellTags->"Convolute"],
Cell[330637, 13230, 271, 10, 44, "Output",
 CellTags->"Convolute"]
}, Open  ]],
Cell[CellGroupData[{
Cell[330945, 13245, 195, 6, 26, "Input",
 CellTags->"Convolute"],
Cell[331143, 13253, 281, 10, 44, "Output",
 CellTags->"Convolute"]
}, Open  ]],
Cell[CellGroupData[{
Cell[331461, 13268, 212, 7, 49, "Input",
 CellTags->"Convolute"],
Cell[331676, 13277, 460, 15, 44, "Output",
 CellTags->"Convolute"]
}, Open  ]],
Cell[CellGroupData[{
Cell[332173, 13297, 213, 7, 49, "Input",
 CellTags->"Convolute"],
Cell[332389, 13306, 358, 13, 44, "Output",
 CellTags->"Convolute"]
}, Open  ]],
Cell[CellGroupData[{
Cell[332784, 13324, 237, 8, 49, "Input",
 CellTags->"Convolute"],
Cell[333024, 13334, 594, 19, 44, "Output",
 CellTags->"Convolute"]
}, Open  ]],
Cell[CellGroupData[{
Cell[333655, 13358, 213, 7, 49, "Input",
 CellTags->"Convolute"],
Cell[333871, 13367, 453, 14, 44, "Output",
 CellTags->"Convolute"]
}, Open  ]],
Cell[CellGroupData[{
Cell[334361, 13386, 190, 6, 26, "Input",
 CellTags->"Convolute"],
Cell[334554, 13394, 287, 10, 26, "Output",
 CellTags->"Convolute"]
}, Open  ]],
Cell[CellGroupData[{
Cell[334878, 13409, 165, 5, 26, "Input",
 CellTags->"Convolute"],
Cell[335046, 13416, 230, 7, 26, "Output",
 CellTags->"Convolute"]
}, Open  ]],
Cell[CellGroupData[{
Cell[335313, 13428, 286, 10, 49, "Input",
 CellTags->"Convolute"],
Cell[335602, 13440, 832, 30, 49, "Output",
 CellTags->"Convolute"]
}, Open  ]],
Cell[CellGroupData[{
Cell[336471, 13475, 295, 10, 49, "Input",
 CellTags->"Convolute"],
Cell[336769, 13487, 1235, 42, 66, "Output",
 CellTags->"Convolute"]
}, Open  ]]
}, Open  ]],
Cell[338031, 13533, 66, 1, 39, "ObjectName",
 CellTags->"ConvoluteTable"],
Cell[CellGroupData[{
Cell[338122, 13538, 65, 1, 34, "Subsubsection",
 CellTags->"ConvoluteTable"],
Cell[338190, 13541, 152, 4, 24, "Text",
 CellTags->"ConvoluteTable"],
Cell[CellGroupData[{
Cell[338367, 13549, 122, 3, 26, "Input",
 CellTags->"ConvoluteTable"],
Cell[338492, 13554, 449, 14, 26, "Output",
 CellTags->"ConvoluteTable"]
}, Open  ]],
Cell[338956, 13571, 322, 14, 24, "Text",
 CellTags->"ConvoluteTable"]
}, Open  ]],
Cell[CellGroupData[{
Cell[339315, 13590, 62, 1, 34, "Subsubsection",
 CellTags->"ConvoluteTable"],
Cell[CellGroupData[{
Cell[339402, 13595, 144, 4, 26, "Input",
 CellTags->"ConvoluteTable"],
Cell[339549, 13601, 203, 6, 26, "Output",
 CellTags->"ConvoluteTable"]
}, Open  ]],
Cell[CellGroupData[{
Cell[339789, 13612, 144, 4, 26, "Input",
 CellTags->"ConvoluteTable"],
Cell[339936, 13618, 225, 7, 26, "Output",
 CellTags->"ConvoluteTable"]
}, Open  ]],
Cell[CellGroupData[{
Cell[340198, 13630, 144, 4, 26, "Input",
 CellTags->"ConvoluteTable"],
Cell[340345, 13636, 177, 5, 26, "Output",
 CellTags->"ConvoluteTable"]
}, Open  ]],
Cell[CellGroupData[{
Cell[340559, 13646, 244, 8, 49, "Input",
 CellTags->"Convolute"],
Cell[340806, 13656, 507, 17, 44, "Output",
 CellTags->"Convolute"]
}, Open  ]]
}, Open  ]],
Cell[341340, 13677, 85, 1, 39, "ObjectName",
 CellTags->"CovariantFieldDerivative"],
Cell[CellGroupData[{
Cell[341450, 13682, 75, 1, 34, "Subsubsection",
 CellTags->"CovariantFieldDerivative"],
Cell[341528, 13685, 372, 6, 58, "Text",
 CellTags->"CovariantFieldDerivative"],
Cell[341903, 13693, 460, 19, 24, "Text",
 CellTags->"CovariantFieldDerivative"]
}, Open  ]],
Cell[CellGroupData[{
Cell[342400, 13717, 72, 1, 34, "Subsubsection",
 CellTags->"CovariantFieldDerivative"],
Cell[CellGroupData[{
Cell[342497, 13722, 329, 9, 26, "Input",
 CellTags->"CovariantFieldDerivative"],
Cell[342829, 13733, 585, 22, 29, "Output",
 CellTags->"CovariantFieldDerivative"]
}, Open  ]]
}, Open  ]],
Cell[343441, 13759, 53, 1, 39, "ObjectName",
 CellTags->"CounterT"],
Cell[CellGroupData[{
Cell[343519, 13764, 59, 1, 34, "Subsubsection",
 CellTags->"CounterT"],
Cell[343581, 13767, 146, 4, 24, "Text",
 CellTags->"CounterT"],
Cell[343730, 13773, 450, 19, 24, "Text",
 CellTags->"CounterT"]
}, Open  ]],
Cell[CellGroupData[{
Cell[344217, 13797, 56, 1, 34, "Subsubsection",
 CellTags->"CounterT"],
Cell[CellGroupData[{
Cell[344298, 13802, 284, 6, 26, "Input",
 CellTags->"CounterT"],
Cell[344585, 13810, 3169, 107, 65, "Output",
 CellTags->"CounterT"]
}, Open  ]]
}, Open  ]],
Cell[347781, 13921, 59, 1, 39, "ObjectName",
 CellTags->"CounterTerm"],
Cell[CellGroupData[{
Cell[347865, 13926, 62, 1, 34, "Subsubsection",
 CellTags->"CounterTerm"],
Cell[347930, 13929, 216, 5, 41, "Text",
 CellTags->"CounterTerm"],
Cell[348149, 13936, 578, 24, 24, "Text",
 CellTags->"CounterTerm"]
}, Open  ]],
Cell[CellGroupData[{
Cell[348764, 13965, 59, 1, 34, "Subsubsection",
 CellTags->"CounterTerm"],
Cell[CellGroupData[{
Cell[348848, 13970, 116, 3, 26, "Input",
 CellTags->"CounterTerm"],
Cell[348967, 13975, 1676, 53, 65, "Output",
 CellTags->"CounterTerm"]
}, Open  ]]
}, Open  ]],
Cell[350670, 14032, 69, 1, 39, "ObjectName",
 CellTags->"CouplingConstant"],
Cell[CellGroupData[{
Cell[350764, 14037, 67, 1, 34, "Subsubsection",
 CellTags->"CouplingConstant"],
Cell[350834, 14040, 657, 11, 100, "Text",
 CellTags->"CouplingConstant"],
Cell[351494, 14053, 318, 14, 24, "Text",
 CellTags->"CouplingConstant"]
}, Open  ]],
Cell[351827, 14070, 57, 1, 39, "ObjectName",
 CellTags->"CovariantD"],
Cell[CellGroupData[{
Cell[351909, 14075, 61, 1, 34, "Subsubsection",
 CellTags->"CovariantD"],
Cell[351973, 14078, 1210, 18, 218, "Text",
 CellTags->"CovariantD"],
Cell[CellGroupData[{
Cell[353208, 14100, 119, 3, 26, "Input",
 CellTags->"CovariantD"],
Cell[353330, 14105, 768, 18, 87, "Output",
 CellTags->"CovariantD"]
}, Open  ]],
Cell[354113, 14126, 185, 4, 24, "Text",
 CellTags->"CovariantD"],
Cell[354301, 14132, 455, 19, 24, "Text",
 CellTags->"CovariantD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[354793, 14156, 58, 1, 34, "Subsubsection",
 CellTags->"CovariantD"],
Cell[CellGroupData[{
Cell[354876, 14161, 117, 3, 26, "Input",
 CellTags->"CovariantD"],
Cell[354996, 14166, 174, 5, 29, "Output",
 CellTags->"CovariantD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[355207, 14176, 150, 4, 26, "Input",
 CellTags->"CovariantD"],
Cell[355360, 14182, 203, 6, 32, "Output",
 CellTags->"CovariantD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[355600, 14193, 180, 5, 26, "Input",
 CellTags->"CovariantD"],
Cell[355783, 14200, 1034, 40, 34, "Output",
 CellTags->"CovariantD"]
}, Open  ]],
Cell[356832, 14243, 164, 4, 24, "Text",
 CellTags->"CovariantD"],
Cell[CellGroupData[{
Cell[357021, 14251, 120, 3, 26, "Input",
 CellTags->"CovariantD"],
Cell[357144, 14256, 218, 7, 26, "Output",
 CellTags->"CovariantD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[357399, 14268, 153, 4, 26, "Input",
 CellTags->"CovariantD"],
Cell[357555, 14274, 247, 8, 30, "Output",
 CellTags->"CovariantD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[357839, 14287, 203, 5, 26, "Input",
 CellTags->"CovariantD"],
Cell[358045, 14294, 981, 37, 34, "Output",
 CellTags->"CovariantD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[359063, 14336, 183, 5, 26, "Input",
 CellTags->"CovariantD"],
Cell[359249, 14343, 974, 36, 33, "Output",
 CellTags->"CovariantD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[360260, 14384, 187, 5, 26, "Input",
 CellTags->"CovariantD"],
Cell[360450, 14391, 1386, 50, 36, "Output",
 CellTags->"CovariantD"]
}, Open  ]],
Cell[361851, 14444, 602, 22, 31, "Text",
 CellTags->"CovariantD"],
Cell[CellGroupData[{
Cell[362478, 14470, 215, 6, 26, "Input",
 CellTags->"CovariantD"],
Cell[362696, 14478, 554, 21, 32, "Output",
 CellTags->"CovariantD"]
}, Open  ]],
Cell[363265, 14502, 285, 10, 24, "Text",
 CellTags->"CovariantD"],
Cell[CellGroupData[{
Cell[363575, 14516, 215, 6, 26, "Input",
 CellTags->"CovariantD"],
Cell[363793, 14524, 2331, 82, 57, "Output",
 CellTags->"CovariantD"]
}, Open  ]],
Cell[366139, 14609, 189, 7, 24, "Text",
 CellTags->"CovariantD"],
Cell[CellGroupData[{
Cell[366353, 14620, 215, 6, 26, "Input",
 CellTags->"CovariantD"],
Cell[366571, 14628, 6358, 202, 116, "Output",
 CellTags->"CovariantD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[372966, 14835, 181, 5, 26, "Input",
 CellTags->"CovariantD"],
Cell[373150, 14842, 303, 10, 31, "Output",
 CellTags->"CovariantD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[373490, 14857, 195, 6, 26, "Input",
 CellTags->"CovariantD"],
Cell[373688, 14865, 4857, 152, 59, "Output",
 CellTags->"CovariantD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[378582, 15022, 223, 7, 26, "Input",
 CellTags->"CovariantD"],
Cell[378808, 15031, 832, 25, 58, "Output",
 CellTags->"CovariantD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[379677, 15061, 240, 7, 26, "Input",
 CellTags->"CovariantD"],
Cell[379920, 15070, 847, 26, 58, "Output",
 CellTags->"CovariantD"]
}, Open  ]]
}, Open  ]],
Cell[380794, 15100, 61, 1, 39, "ObjectName",
 CellTags->"CrossProduct"],
Cell[CellGroupData[{
Cell[380880, 15105, 63, 1, 34, "Subsubsection",
 CellTags->"CrossProduct"],
Cell[380946, 15108, 145, 4, 24, "Text",
 CellTags->"CrossProduct"],
Cell[381094, 15114, 308, 14, 24, "Text",
 CellTags->"CrossProduct"]
}, Open  ]],
Cell[CellGroupData[{
Cell[381439, 15133, 60, 1, 34, "Subsubsection",
 CellTags->"CrossProduct"],
Cell[CellGroupData[{
Cell[381524, 15138, 326, 9, 26, "Input",
 CellTags->"CrossProduct"],
Cell[381853, 15149, 681, 23, 30, "Output",
 CellTags->"CrossProduct"]
}, Open  ]]
}, Open  ]],
Cell[382561, 15176, 41, 1, 39, "ObjectName",
 CellTags->"C0"],
Cell[CellGroupData[{
Cell[382627, 15181, 53, 1, 34, "Subsubsection",
 CellTags->"C0"],
Cell[382683, 15184, 448, 10, 58, "Text",
 CellTags->"C0"],
Cell[383134, 15196, 458, 24, 24, "Text",
 CellTags->"C0"]
}, Open  ]],
Cell[CellGroupData[{
Cell[383629, 15225, 50, 1, 34, "Subsubsection",
 CellTags->"C0"],
Cell[CellGroupData[{
Cell[383704, 15230, 174, 5, 26, "Input",
 CellTags->"C0"],
Cell[383881, 15237, 302, 9, 26, "Output",
 CellTags->"C0"]
}, Open  ]],
Cell[CellGroupData[{
Cell[384220, 15251, 203, 6, 26, "Input",
 CellTags->"C0"],
Cell[384426, 15259, 302, 9, 26, "Output",
 CellTags->"C0"]
}, Open  ]],
Cell[CellGroupData[{
Cell[384765, 15273, 326, 10, 26, "Input",
 CellTags->"C0"],
Cell[385094, 15285, 301, 9, 26, "Output",
 CellTags->"C0"]
}, Open  ]]
}, Open  ]],
Cell[385422, 15298, 53, 1, 39, "ObjectName",
 CellTags->"DataType"],
Cell[CellGroupData[{
Cell[385500, 15303, 59, 1, 34, "Subsubsection",
 CellTags->"DataType"],
Cell[385562, 15306, 344, 6, 58, "Text",
 CellTags->"DataType"],
Cell[385909, 15314, 835, 32, 40, "Text",
 CellTags->"DataType"],
Cell[386747, 15348, 291, 13, 24, "Text",
 CellTags->"DataType"],
Cell[387041, 15363, 218, 9, 24, "Text",
 CellTags->"DataType"]
}, Open  ]],
Cell[CellGroupData[{
Cell[387296, 15377, 56, 1, 34, "Subsubsection",
 CellTags->"DataType"],
Cell[387355, 15380, 74, 1, 24, "Text",
 CellTags->"DataType"],
Cell[387432, 15383, 221, 7, 26, "Input",
 CellTags->"DataType"],
Cell[CellGroupData[{
Cell[387678, 15394, 224, 8, 26, "Input",
 CellTags->"DataType"],
Cell[387905, 15404, 280, 10, 26, "Output",
 CellTags->"DataType"]
}, Open  ]],
Cell[388200, 15417, 179, 4, 24, "Text",
 CellTags->"DataType"],
Cell[CellGroupData[{
Cell[388404, 15425, 112, 3, 26, "Input",
 CellTags->"DataType"],
Cell[388519, 15430, 252, 8, 26, "Output",
 CellTags->"DataType"]
}, Open  ]],
Cell[388786, 15441, 271, 9, 26, "Input",
 CellTags->"DataType"],
Cell[389060, 15452, 412, 14, 26, "Input",
 CellTags->"DataType"],
Cell[389475, 15468, 395, 14, 26, "Input",
 CellTags->"DataType"],
Cell[CellGroupData[{
Cell[389895, 15486, 340, 13, 26, "Input",
 CellTags->"DataType"],
Cell[390238, 15501, 392, 14, 26, "Output",
 CellTags->"DataType"]
}, Open  ]],
Cell[CellGroupData[{
Cell[390667, 15520, 107, 3, 26, "Input",
 CellTags->"DataType"],
Cell[390777, 15525, 323, 12, 26, "Output",
 CellTags->"DataType"]
}, Open  ]],
Cell[CellGroupData[{
Cell[391137, 15542, 107, 3, 26, "Input",
 CellTags->"DataType"],
Cell[391247, 15547, 214, 7, 26, "Output",
 CellTags->"DataType"]
}, Open  ]],
Cell[391476, 15557, 191, 6, 26, "Input",
 CellTags->"DataType"],
Cell[391670, 15565, 187, 6, 26, "Input",
 CellTags->"DataType"],
Cell[391860, 15573, 429, 15, 26, "Input",
 CellTags->"DataType"],
Cell[CellGroupData[{
Cell[392314, 15592, 290, 11, 26, "Input",
 CellTags->"DataType"],
Cell[392607, 15605, 343, 13, 26, "Output",
 CellTags->"DataType"]
}, Open  ]],
Cell[CellGroupData[{
Cell[392987, 15623, 105, 3, 26, "Input",
 CellTags->"DataType"],
Cell[393095, 15628, 272, 10, 26, "Output",
 CellTags->"DataType"]
}, Open  ]],
Cell[393382, 15641, 151, 5, 26, "Input",
 CellTags->"DataType"],
Cell[393536, 15648, 222, 7, 26, "Input",
 CellTags->"DataType"],
Cell[393761, 15657, 272, 9, 26, "Input",
 CellTags->"DataType"],
Cell[394036, 15668, 114, 3, 24, "Text",
 CellTags->"DataType"],
Cell[CellGroupData[{
Cell[394175, 15675, 149, 4, 26, "Input",
 CellTags->"DataType"],
Cell[394327, 15681, 149, 4, 26, "Output",
 CellTags->"DataType"]
}, Open  ]],
Cell[394491, 15688, 83, 1, 24, "Text",
 CellTags->"DataType"],
Cell[CellGroupData[{
Cell[394599, 15693, 233, 8, 26, "Input",
 CellTags->"DataType"],
Cell[394835, 15703, 148, 4, 26, "Output",
 CellTags->"DataType"]
}, Open  ]],
Cell[CellGroupData[{
Cell[395020, 15712, 226, 7, 26, "Input",
 CellTags->"DataType"],
Cell[395249, 15721, 479, 17, 28, "Output",
 CellTags->"DataType"]
}, Open  ]]
}, Open  ]],
Cell[395755, 15742, 43, 1, 39, "ObjectName",
 CellTags->"DB0"],
Cell[CellGroupData[{
Cell[395823, 15747, 54, 1, 34, "Subsubsection",
 CellTags->"DB0"],
Cell[395880, 15750, 146, 4, 24, "Text",
 CellTags->"DB0"],
Cell[396029, 15756, 156, 9, 24, "Text",
 CellTags->"DB0"]
}, Open  ]],
Cell[CellGroupData[{
Cell[396222, 15770, 51, 1, 34, "Subsubsection",
 CellTags->"DB0"],
Cell[CellGroupData[{
Cell[396298, 15775, 339, 12, 30, "Input",
 CellTags->"DB0"],
Cell[396640, 15789, 297, 9, 29, "Output",
 CellTags->"DB0"]
}, Open  ]]
}, Open  ]],
Cell[396964, 15802, 43, 1, 39, "ObjectName",
 CellTags->"DB1"],
Cell[CellGroupData[{
Cell[397032, 15807, 54, 1, 34, "Subsubsection",
 CellTags->"DB1"],
Cell[397089, 15810, 117, 3, 24, "Text",
 CellTags->"DB1"],
Cell[397209, 15815, 156, 9, 24, "Text",
 CellTags->"DB1"]
}, Open  ]],
Cell[CellGroupData[{
Cell[397402, 15829, 51, 1, 34, "Subsubsection",
 CellTags->"DB1"],
Cell[CellGroupData[{
Cell[397478, 15834, 339, 12, 30, "Input",
 CellTags->"DB1"],
Cell[397820, 15848, 1452, 46, 52, "Output",
 CellTags->"DB1"]
}, Open  ]]
}, Open  ]],
Cell[399299, 15898, 79, 1, 39, "ObjectName",
 CellTags->"DeclareNonCommutative"],
Cell[CellGroupData[{
Cell[399403, 15903, 72, 1, 34, "Subsubsection",
 CellTags->"DeclareNonCommutative"],
Cell[399478, 15906, 192, 4, 24, "Text",
 CellTags->"DeclareNonCommutative"],
Cell[399673, 15912, 347, 14, 24, "Text",
 CellTags->"DeclareNonCommutative"]
}, Open  ]],
Cell[CellGroupData[{
Cell[400057, 15931, 69, 1, 34, "Subsubsection",
 CellTags->"DeclareNonCommutative"],
Cell[400129, 15934, 135, 3, 26, "Input",
 CellTags->"DeclareNonCommutative"],
Cell[400267, 15939, 152, 4, 24, "Text",
 CellTags->"DeclareNonCommutative"],
Cell[CellGroupData[{
Cell[400444, 15947, 158, 4, 26, "Input",
 CellTags->"DeclareNonCommutative"],
Cell[400605, 15953, 164, 4, 26, "Output",
 CellTags->"DeclareNonCommutative"]
}, Open  ]],
Cell[400784, 15960, 158, 4, 26, "Input",
 CellTags->"DeclareNonCommutative"],
Cell[CellGroupData[{
Cell[400967, 15968, 191, 5, 26, "Input",
 CellTags->"DeclareNonCommutative"],
Cell[401161, 15975, 241, 7, 26, "Output",
 CellTags->"DeclareNonCommutative"]
}, Open  ]],
Cell[401417, 15985, 170, 4, 26, "Input",
 CellTags->"DeclareNonCommutative"],
Cell[CellGroupData[{
Cell[401612, 15993, 191, 5, 26, "Input",
 CellTags->"DeclareNonCommutative"],
Cell[401806, 16000, 241, 7, 26, "Output",
 CellTags->"DeclareNonCommutative"]
}, Open  ]]
}, Open  ]],
Cell[402074, 16011, 63, 1, 39, "ObjectName",
 CellTags->"DeltaFunction"],
Cell[CellGroupData[{
Cell[402162, 16016, 64, 1, 34, "Subsubsection",
 CellTags->"DeltaFunction"],
Cell[402229, 16019, 353, 11, 49, "Text",
 CellTags->"DeltaFunction"],
Cell[402585, 16032, 592, 24, 24, "Text",
 CellTags->"DeltaFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[403214, 16061, 61, 1, 34, "Subsubsection",
 CellTags->"DeltaFunction"],
Cell[CellGroupData[{
Cell[403300, 16066, 142, 4, 26, "Input",
 CellTags->"DeltaFunction"],
Cell[403445, 16072, 245, 8, 26, "Output",
 CellTags->"DeltaFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[403727, 16085, 320, 10, 26, "Input",
 CellTags->"DeltaFunction"],
Cell[404050, 16097, 179, 5, 26, "Output",
 CellTags->"DeltaFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[404266, 16107, 294, 9, 26, "Input",
 CellTags->"DeltaFunction"],
Cell[404563, 16118, 179, 5, 26, "Output",
 CellTags->"DeltaFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[404779, 16128, 320, 10, 26, "Input",
 CellTags->"DeltaFunction"],
Cell[405102, 16140, 179, 5, 26, "Output",
 CellTags->"DeltaFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[405318, 16150, 204, 6, 26, "Input",
 CellTags->"DeltaFunction"],
Cell[405525, 16158, 153, 4, 27, "Output",
 CellTags->"DeltaFunction"]
}, Open  ]]
}, Open  ]],
Cell[405705, 16166, 85, 1, 39, "ObjectName",
 CellTags->"DeltaFunctionDoublePrime"],
Cell[CellGroupData[{
Cell[405815, 16171, 75, 1, 34, "Subsubsection",
 CellTags->"DeltaFunctionDoublePrime"],
Cell[405893, 16174, 248, 8, 24, "Text",
 CellTags->"DeltaFunctionDoublePrime"],
Cell[406144, 16184, 219, 8, 24, "Text",
 CellTags->"DeltaFunctionDoublePrime"]
}, Open  ]],
Cell[406378, 16195, 73, 1, 39, "ObjectName",
 CellTags->"DeltaFunctionPrime"],
Cell[CellGroupData[{
Cell[406476, 16200, 69, 1, 34, "Subsubsection",
 CellTags->"DeltaFunctionPrime"],
Cell[406548, 16203, 227, 7, 24, "Text",
 CellTags->"DeltaFunctionPrime"],
Cell[406778, 16212, 576, 23, 24, "Text",
 CellTags->"DeltaFunctionPrime"]
}, Open  ]],
Cell[CellGroupData[{
Cell[407391, 16240, 66, 1, 34, "Subsubsection",
 CellTags->"DeltaFunctionPrime"],
Cell[CellGroupData[{
Cell[407482, 16245, 152, 4, 26, "Input",
 CellTags->"DeltaFunctionPrime"],
Cell[407637, 16251, 282, 9, 26, "Output",
 CellTags->"DeltaFunctionPrime"]
}, Open  ]],
Cell[CellGroupData[{
Cell[407956, 16265, 330, 10, 26, "Input",
 CellTags->"DeltaFunctionPrime"],
Cell[408289, 16277, 247, 7, 26, "Output",
 CellTags->"DeltaFunctionPrime"]
}, Open  ]],
Cell[CellGroupData[{
Cell[408573, 16289, 326, 10, 30, "Input",
 CellTags->"DeltaFunctionPrime"],
Cell[408902, 16301, 156, 4, 26, "Output",
 CellTags->"DeltaFunctionPrime"]
}, Open  ]]
}, Open  ]],
Cell[409085, 16309, 69, 1, 39, "ObjectName",
 CellTags->"DenominatorOrder"],
Cell[CellGroupData[{
Cell[409179, 16314, 67, 1, 34, "Subsubsection",
 CellTags->"DenominatorOrder"],
Cell[409249, 16317, 175, 4, 24, "Text",
 CellTags->"DenominatorOrder"],
Cell[409427, 16323, 327, 13, 24, "Text",
 CellTags->"DenominatorOrder"]
}, Open  ]],
Cell[409769, 16339, 55, 1, 39, "ObjectName",
 CellTags->"Dimension"],
Cell[CellGroupData[{
Cell[409849, 16344, 60, 1, 34, "Subsubsection",
 CellTags->"Dimension"],
Cell[409912, 16347, 246, 5, 41, "Text",
 CellTags->"Dimension"],
Cell[CellGroupData[{
Cell[410183, 16356, 120, 3, 26, "Input",
 CellTags->"Dimension"],
Cell[410306, 16361, 286, 9, 26, "Output",
 CellTags->"Dimension"]
}, Open  ]],
Cell[CellGroupData[{
Cell[410629, 16375, 313, 9, 26, "Input",
 CellTags->"Dimension"],
Cell[410945, 16386, 328, 12, 26, "Output",
 CellTags->"Dimension"]
}, Open  ]],
Cell[411288, 16401, 131, 4, 24, "Text",
 CellTags->"Dimension"],
Cell[CellGroupData[{
Cell[411444, 16409, 357, 11, 26, "Input",
 CellTags->"Dimension"],
Cell[411804, 16422, 402, 11, 38, "Output",
 CellTags->"Dimension"]
}, Open  ]],
Cell[412221, 16436, 166, 4, 24, "Text",
 CellTags->"Dimension"],
Cell[412390, 16442, 133, 4, 26, "Input",
 CellTags->"Dimension"],
Cell[CellGroupData[{
Cell[412548, 16450, 326, 9, 26, "Input",
 CellTags->"Dimension"],
Cell[412877, 16461, 706, 29, 26, "Output",
 CellTags->"Dimension"]
}, Open  ]],
Cell[CellGroupData[{
Cell[413620, 16495, 110, 3, 26, "Input",
 CellTags->"Dimension"],
Cell[413733, 16500, 415, 11, 38, "Output",
 CellTags->"Dimension"]
}, Open  ]],
Cell[414163, 16514, 134, 4, 26, "Input",
 CellTags->"Dimension"]
}, Open  ]],
Cell[414312, 16521, 77, 1, 39, "ObjectName",
 CellTags->"DimensionalReduction"],
Cell[CellGroupData[{
Cell[414414, 16526, 71, 1, 34, "Subsubsection",
 CellTags->"DimensionalReduction"],
Cell[414488, 16529, 121, 3, 24, "Text",
 CellTags->"DimensionalReduction"],
Cell[414612, 16534, 301, 13, 24, "Text",
 CellTags->"DimensionalReduction"]
}, Open  ]],
Cell[414928, 16550, 57, 1, 39, "ObjectName",
 CellTags->"DiracBasis"],
Cell[CellGroupData[{
Cell[415010, 16555, 61, 1, 34, "Subsubsection",
 CellTags->"DiracBasis"],
Cell[415074, 16558, 299, 6, 41, "Text",
 CellTags->"DiracBasis"],
Cell[415376, 16566, 184, 8, 24, "Text",
 CellTags->"DiracBasis"]
}, Open  ]],
Cell[415575, 16577, 65, 1, 39, "ObjectName",
 CellTags->"DiracCanonical"],
Cell[CellGroupData[{
Cell[415665, 16582, 65, 1, 34, "Subsubsection",
 CellTags->"DiracCanonical"],
Cell[415733, 16585, 171, 4, 24, "Text",
 CellTags->"DiracCanonical"],
Cell[415907, 16591, 194, 8, 24, "Text",
 CellTags->"DiracCanonical"]
}, Open  ]],
Cell[416116, 16602, 63, 1, 39, "ObjectName",
 CellTags->"DiracEquation"],
Cell[CellGroupData[{
Cell[416204, 16607, 64, 1, 34, "Subsubsection",
 CellTags->"DiracEquation"],
Cell[416271, 16610, 160, 4, 24, "Text",
 CellTags->"DiracEquation"],
Cell[416434, 16616, 199, 9, 24, "Text",
 CellTags->"DiracEquation"]
}, Open  ]],
Cell[CellGroupData[{
Cell[416670, 16630, 61, 1, 34, "Subsubsection",
 CellTags->"DiracEquation"],
Cell[CellGroupData[{
Cell[416756, 16635, 279, 8, 26, "Input",
 CellTags->"DiracEquation"],
Cell[417038, 16645, 410, 14, 28, "Output",
 CellTags->"DiracEquation"]
}, Open  ]],
Cell[CellGroupData[{
Cell[417485, 16664, 119, 3, 26, "Input",
 CellTags->"DiracEquation"],
Cell[417607, 16669, 261, 8, 28, "Output",
 CellTags->"DiracEquation"]
}, Open  ]]
}, Open  ]],
Cell[417895, 16681, 57, 1, 39, "ObjectName",
 CellTags->"DiracGamma"],
Cell[CellGroupData[{
Cell[417977, 16686, 61, 1, 34, "Subsubsection",
 CellTags->"DiracGamma"],
Cell[418041, 16689, 762, 24, 61, "Text",
 CellTags->"DiracGamma"],
Cell[418806, 16715, 681, 29, 24, "Text",
 CellTags->"DiracGamma"]
}, Open  ]],
Cell[CellGroupData[{
Cell[419524, 16749, 58, 1, 34, "Subsubsection",
 CellTags->"DiracGamma"],
Cell[CellGroupData[{
Cell[419607, 16754, 113, 3, 26, "Input",
 CellTags->"DiracGamma"],
Cell[419723, 16759, 179, 5, 26, "Output",
 CellTags->"DiracGamma"]
}, Open  ]],
Cell[CellGroupData[{
Cell[419939, 16769, 159, 4, 26, "Input",
 CellTags->"DiracGamma"],
Cell[420101, 16775, 256, 9, 26, "Output",
 CellTags->"DiracGamma"]
}, Open  ]],
Cell[420372, 16787, 332, 13, 27, "Text",
 CellTags->"DiracGamma"],
Cell[CellGroupData[{
Cell[420729, 16804, 167, 5, 26, "Input",
 CellTags->"DiracGamma"],
Cell[420899, 16811, 267, 9, 28, "Output",
 CellTags->"DiracGamma"]
}, Open  ]],
Cell[CellGroupData[{
Cell[421203, 16825, 280, 8, 26, "Input",
 CellTags->"DiracGamma"],
Cell[421486, 16835, 512, 18, 28, "Output",
 CellTags->"DiracGamma"]
}, Open  ]],
Cell[CellGroupData[{
Cell[422035, 16858, 218, 7, 26, "Input",
 CellTags->"DiracGamma"],
Cell[422256, 16867, 265, 9, 28, "Output",
 CellTags->"DiracGamma"]
}, Open  ]],
Cell[CellGroupData[{
Cell[422558, 16881, 208, 7, 26, "Input",
 CellTags->"DiracGamma"],
Cell[422769, 16890, 383, 13, 28, "Output",
 CellTags->"DiracGamma"]
}, Open  ]],
Cell[CellGroupData[{
Cell[423189, 16908, 144, 4, 26, "Input",
 CellTags->"DiracGamma"],
Cell[423336, 16914, 634, 23, 28, "Output",
 CellTags->"DiracGamma"]
}, Open  ]],
Cell[CellGroupData[{
Cell[424007, 16942, 300, 9, 26, "Input",
 CellTags->"DiracGamma"],
Cell[424310, 16953, 472, 14, 28, "Output",
 CellTags->"DiracGamma"]
}, Open  ]],
Cell[CellGroupData[{
Cell[424819, 16972, 138, 4, 26, "Input",
 CellTags->"DiracGamma"],
Cell[424960, 16978, 474, 14, 28, "Output",
 CellTags->"DiracGamma"]
}, Open  ]],
Cell[CellGroupData[{
Cell[425471, 16997, 141, 4, 26, "Input",
 CellTags->"DiracGamma"],
Cell[425615, 17003, 1167, 43, 28, "Output",
 CellTags->"DiracGamma"]
}, Open  ]],
Cell[426797, 17049, 168, 5, 26, "Input",
 CellTags->"DiracGamma"]
}, Open  ]],
Cell[426980, 17057, 71, 1, 39, "ObjectName",
 CellTags->"DiracGammaCombine"],
Cell[CellGroupData[{
Cell[427076, 17062, 68, 1, 34, "Subsubsection",
 CellTags->"DiracGammaCombine"],
Cell[427147, 17065, 135, 4, 24, "Text",
 CellTags->"DiracGammaCombine"],
Cell[427285, 17071, 804, 34, 24, "Text",
 CellTags->"DiracGammaCombine"]
}, Open  ]],
Cell[CellGroupData[{
Cell[428126, 17110, 65, 1, 34, "Subsubsection",
 CellTags->"DiracGammaCombine"],
Cell[CellGroupData[{
Cell[428216, 17115, 220, 6, 26, "Input",
 CellTags->"DiracGammaCombine"],
Cell[428439, 17123, 307, 10, 28, "Output",
 CellTags->"DiracGammaCombine"]
}, Open  ]],
Cell[CellGroupData[{
Cell[428783, 17138, 122, 3, 26, "Input",
 CellTags->"DiracGammaCombine"],
Cell[428908, 17143, 238, 6, 38, "Output",
 CellTags->"DiracGammaCombine"]
}, Open  ]],
Cell[CellGroupData[{
Cell[429183, 17154, 270, 8, 26, "Input",
 CellTags->"DiracGammaCombine"],
Cell[429456, 17164, 359, 12, 28, "Output",
 CellTags->"DiracGammaCombine"]
}, Open  ]],
Cell[CellGroupData[{
Cell[429852, 17181, 122, 3, 26, "Input",
 CellTags->"DiracGammaCombine"],
Cell[429977, 17186, 286, 8, 38, "Output",
 CellTags->"DiracGammaCombine"]
}, Open  ]],
Cell[CellGroupData[{
Cell[430300, 17199, 127, 3, 26, "Input",
 CellTags->"DiracGammaCombine"],
Cell[430430, 17204, 482, 17, 28, "Output",
 CellTags->"DiracGammaCombine"]
}, Open  ]]
}, Open  ]],
Cell[430939, 17225, 69, 1, 39, "ObjectName",
 CellTags->"DiracGammaExpand"],
Cell[CellGroupData[{
Cell[431033, 17230, 67, 1, 34, "Subsubsection",
 CellTags->"DiracGammaExpand"],
Cell[431103, 17233, 190, 4, 41, "Text",
 CellTags->"DiracGammaExpand"],
Cell[431296, 17239, 806, 34, 24, "Text",
 CellTags->"DiracGammaExpand"]
}, Open  ]],
Cell[CellGroupData[{
Cell[432139, 17278, 64, 1, 34, "Subsubsection",
 CellTags->"DiracGammaExpand"],
Cell[CellGroupData[{
Cell[432228, 17283, 314, 9, 26, "Input",
 CellTags->"DiracGammaExpand"],
Cell[432545, 17294, 516, 18, 28, "Output",
 CellTags->"DiracGammaExpand"]
}, Open  ]],
Cell[433076, 17315, 125, 3, 24, "Text",
 CellTags->"DiracGammaExpand"],
Cell[CellGroupData[{
Cell[433226, 17322, 121, 3, 26, "Input",
 CellTags->"DiracGammaExpand"],
Cell[433350, 17327, 333, 9, 38, "Output",
 CellTags->"DiracGammaExpand"]
}, Open  ]],
Cell[433698, 17339, 101, 1, 24, "Text",
 CellTags->"DiracGammaExpand"],
Cell[CellGroupData[{
Cell[433824, 17344, 125, 3, 26, "Input",
 CellTags->"DiracGammaExpand"],
Cell[433952, 17349, 641, 23, 28, "Output",
 CellTags->"DiracGammaExpand"]
}, Open  ]],
Cell[434608, 17375, 90, 1, 24, "Text",
 CellTags->"DiracGammaExpand"],
Cell[CellGroupData[{
Cell[434723, 17380, 209, 5, 26, "Input",
 CellTags->"DiracGammaExpand"],
Cell[434935, 17387, 333, 9, 38, "Output",
 CellTags->"DiracGammaExpand"]
}, Open  ]],
Cell[CellGroupData[{
Cell[435305, 17401, 164, 4, 26, "Input",
 CellTags->"DiracGammaExpand"],
Cell[435472, 17407, 436, 12, 38, "Output",
 CellTags->"DiracGammaExpand"]
}, Open  ]],
Cell[435923, 17422, 106, 1, 24, "Text",
 CellTags->"DiracGammaExpand"],
Cell[CellGroupData[{
Cell[436054, 17427, 122, 3, 26, "Input",
 CellTags->"DiracGammaExpand"],
Cell[436179, 17432, 576, 21, 28, "Output",
 CellTags->"DiracGammaExpand"]
}, Open  ]],
Cell[436770, 17456, 114, 3, 26, "Input",
 CellTags->"DiracGammaExpand"]
}, Open  ]],
Cell[436899, 17462, 59, 1, 39, "ObjectName",
 CellTags->"DiracGammaT"],
Cell[CellGroupData[{
Cell[436983, 17467, 62, 1, 34, "Subsubsection",
 CellTags->"DiracGammaT"],
Cell[437048, 17470, 208, 5, 41, "Text",
 CellTags->"DiracGammaT"],
Cell[437259, 17477, 188, 9, 24, "Text",
 CellTags->"DiracGammaT"]
}, Open  ]],
Cell[CellGroupData[{
Cell[437484, 17491, 59, 1, 34, "Subsubsection",
 CellTags->"DiracGammaT"],
Cell[CellGroupData[{
Cell[437568, 17496, 158, 4, 26, "Input",
 CellTags->"DiracGammaT"],
Cell[437729, 17502, 262, 9, 30, "Output",
 CellTags->"DiracGammaT"]
}, Open  ]],
Cell[CellGroupData[{
Cell[438028, 17516, 113, 3, 26, "Input",
 CellTags->"DiracGammaT"],
Cell[438144, 17521, 256, 9, 26, "Output",
 CellTags->"DiracGammaT"]
}, Open  ]],
Cell[CellGroupData[{
Cell[438437, 17535, 167, 6, 26, "Input",
 CellTags->"DiracGammaT"],
Cell[438607, 17543, 261, 8, 28, "Output",
 CellTags->"DiracGammaT"]
}, Open  ]]
}, Open  ]],
Cell[438895, 17555, 59, 1, 39, "ObjectName",
 CellTags->"DiracMatrix"],
Cell[CellGroupData[{
Cell[438979, 17560, 62, 1, 34, "Subsubsection",
 CellTags->"DiracMatrix"],
Cell[439044, 17563, 1220, 46, 44, "Text",
 CellTags->"DiracMatrix"],
Cell[440267, 17611, 1053, 49, 24, "Text",
 CellTags->"DiracMatrix"]
}, Open  ]],
Cell[CellGroupData[{
Cell[441357, 17665, 59, 1, 34, "Subsubsection",
 CellTags->"DiracMatrix"],
Cell[CellGroupData[{
Cell[441441, 17670, 119, 3, 26, "Input",
 CellTags->"DiracMatrix"],
Cell[441563, 17675, 254, 9, 26, "Output",
 CellTags->"DiracMatrix"]
}, Open  ]],
Cell[441832, 17687, 539, 20, 41, "Text",
 CellTags->"DiracMatrix"],
Cell[CellGroupData[{
Cell[442396, 17711, 184, 5, 26, "Input",
 CellTags->"DiracMatrix"],
Cell[442583, 17718, 394, 15, 26, "Output",
 CellTags->"DiracMatrix"]
}, Open  ]],
Cell[CellGroupData[{
Cell[443014, 17738, 160, 5, 26, "Input",
 CellTags->"DiracMatrix"],
Cell[443177, 17745, 217, 5, 38, "Output",
 CellTags->"DiracMatrix"]
}, Open  ]],
Cell[CellGroupData[{
Cell[443431, 17755, 145, 4, 26, "Input",
 CellTags->"DiracMatrix"],
Cell[443579, 17761, 186, 5, 26, "Output",
 CellTags->"DiracMatrix"]
}, Open  ]],
Cell[CellGroupData[{
Cell[443802, 17771, 112, 3, 26, "Input",
 CellTags->"DiracMatrix"],
Cell[443917, 17776, 167, 4, 38, "Output",
 CellTags->"DiracMatrix"]
}, Open  ]],
Cell[CellGroupData[{
Cell[444121, 17785, 111, 3, 26, "Input",
 CellTags->"DiracMatrix"],
Cell[444235, 17790, 184, 5, 26, "Output",
 CellTags->"DiracMatrix"]
}, Open  ]],
Cell[CellGroupData[{
Cell[444456, 17800, 138, 4, 26, "Input",
 CellTags->"DiracMatrix"],
Cell[444597, 17806, 263, 7, 38, "Output",
 CellTags->"DiracMatrix"]
}, Open  ]],
Cell[CellGroupData[{
Cell[444897, 17818, 152, 4, 26, "Input",
 CellTags->"DiracMatrix"],
Cell[445052, 17824, 291, 8, 26, "Output",
 CellTags->"DiracMatrix"]
}, Open  ]],
Cell[CellGroupData[{
Cell[445380, 17837, 166, 5, 26, "Input",
 CellTags->"DiracMatrix"],
Cell[445549, 17844, 384, 15, 28, "Output",
 CellTags->"DiracMatrix"]
}, Open  ]],
Cell[CellGroupData[{
Cell[445970, 17864, 112, 3, 26, "Input",
 CellTags->"DiracMatrix"],
Cell[446085, 17869, 313, 8, 38, "Output",
 CellTags->"DiracMatrix"]
}, Open  ]]
}, Open  ]],
Cell[446425, 17881, 57, 1, 39, "ObjectName",
 CellTags->"DiracOrder"],
Cell[CellGroupData[{
Cell[446507, 17886, 61, 1, 34, "Subsubsection",
 CellTags->"DiracOrder"],
Cell[446571, 17889, 204, 5, 41, "Text",
 CellTags->"DiracOrder"],
Cell[446778, 17896, 312, 14, 24, "Text",
 CellTags->"DiracOrder"]
}, Open  ]],
Cell[CellGroupData[{
Cell[447127, 17915, 58, 1, 34, "Subsubsection",
 CellTags->"DiracOrder"],
Cell[CellGroupData[{
Cell[447210, 17920, 176, 5, 26, "Input",
 CellTags->"DiracOrder"],
Cell[447389, 17927, 250, 7, 26, "Output",
 CellTags->"DiracOrder"]
}, Open  ]],
Cell[CellGroupData[{
Cell[447676, 17939, 114, 3, 26, "Input",
 CellTags->"DiracOrder"],
Cell[447793, 17944, 514, 19, 26, "Output",
 CellTags->"DiracOrder"]
}, Open  ]],
Cell[448322, 17966, 93, 1, 24, "Text",
 CellTags->"DiracOrder"],
Cell[CellGroupData[{
Cell[448440, 17971, 176, 5, 26, "Input",
 CellTags->"DiracOrder"],
Cell[448619, 17978, 289, 8, 26, "Output",
 CellTags->"DiracOrder"]
}, Open  ]],
Cell[CellGroupData[{
Cell[448945, 17991, 114, 3, 26, "Input",
 CellTags->"DiracOrder"],
Cell[449062, 17996, 514, 19, 26, "Output",
 CellTags->"DiracOrder"]
}, Open  ]],
Cell[CellGroupData[{
Cell[449613, 18020, 171, 5, 26, "Input",
 CellTags->"DiracOrder"],
Cell[449787, 18027, 285, 8, 26, "Output",
 CellTags->"DiracOrder"]
}, Open  ]],
Cell[450087, 18038, 196, 8, 24, "Text",
 CellTags->"DiracOrder"],
Cell[CellGroupData[{
Cell[450308, 18050, 114, 3, 26, "Input",
 CellTags->"DiracOrder"],
Cell[450425, 18055, 435, 16, 26, "Output",
 CellTags->"DiracOrder"]
}, Open  ]],
Cell[CellGroupData[{
Cell[450897, 18076, 167, 5, 26, "Input",
 CellTags->"DiracOrder"],
Cell[451067, 18083, 283, 8, 26, "Output",
 CellTags->"DiracOrder"]
}, Open  ]],
Cell[CellGroupData[{
Cell[451387, 18096, 114, 3, 26, "Input",
 CellTags->"DiracOrder"],
Cell[451504, 18101, 315, 11, 26, "Output",
 CellTags->"DiracOrder"]
}, Open  ]],
Cell[CellGroupData[{
Cell[451856, 18117, 183, 5, 26, "Input",
 CellTags->"DiracOrder"],
Cell[452042, 18124, 297, 8, 26, "Output",
 CellTags->"DiracOrder"]
}, Open  ]],
Cell[452354, 18135, 288, 10, 24, "Text",
 CellTags->"DiracOrder"],
Cell[CellGroupData[{
Cell[452667, 18149, 219, 7, 26, "Input",
 CellTags->"DiracOrder"],
Cell[452889, 18158, 1251, 46, 26, "Output",
 CellTags->"DiracOrder"]
}, Open  ]],
Cell[CellGroupData[{
Cell[454177, 18209, 113, 3, 26, "Input",
 CellTags->"DiracOrder"],
Cell[454293, 18214, 521, 20, 26, "Output",
 CellTags->"DiracOrder"]
}, Open  ]],
Cell[454829, 18237, 188, 6, 26, "Input",
 CellTags->"DiracOrder"]
}, Open  ]],
Cell[455032, 18246, 59, 1, 39, "ObjectName",
 CellTags->"DiracReduce"],
Cell[CellGroupData[{
Cell[455116, 18251, 62, 1, 34, "Subsubsection",
 CellTags->"DiracReduce"],
Cell[455181, 18254, 662, 11, 92, "Text",
 CellTags->"DiracReduce"],
Cell[CellGroupData[{
Cell[455868, 18269, 121, 3, 26, "Input",
 CellTags->"DiracReduce"],
Cell[455992, 18274, 365, 11, 26, "Output",
 CellTags->"DiracReduce"]
}, Open  ]],
Cell[456372, 18288, 197, 9, 24, "Text",
 CellTags->"DiracReduce"]
}, Open  ]],
Cell[CellGroupData[{
Cell[456606, 18302, 59, 1, 34, "Subsubsection",
 CellTags->"DiracReduce"],
Cell[CellGroupData[{
Cell[456690, 18307, 172, 5, 26, "Input",
 CellTags->"DiracReduce"],
Cell[456865, 18314, 246, 7, 26, "Output",
 CellTags->"DiracReduce"]
}, Open  ]],
Cell[CellGroupData[{
Cell[457148, 18326, 116, 3, 26, "Input",
 CellTags->"DiracReduce"],
Cell[457267, 18331, 323, 10, 26, "Output",
 CellTags->"DiracReduce"]
}, Open  ]],
Cell[CellGroupData[{
Cell[457627, 18346, 186, 5, 26, "Input",
 CellTags->"DiracReduce"],
Cell[457816, 18353, 281, 8, 26, "Output",
 CellTags->"DiracReduce"]
}, Open  ]],
Cell[CellGroupData[{
Cell[458134, 18366, 116, 3, 26, "Input",
 CellTags->"DiracReduce"],
Cell[458253, 18371, 961, 29, 26, "Output",
 CellTags->"DiracReduce"]
}, Open  ]],
Cell[CellGroupData[{
Cell[459251, 18405, 207, 6, 26, "Input",
 CellTags->"DiracReduce"],
Cell[459461, 18413, 325, 9, 26, "Output",
 CellTags->"DiracReduce"]
}, Open  ]],
Cell[CellGroupData[{
Cell[459823, 18427, 140, 4, 26, "Input",
 CellTags->"DiracReduce"],
Cell[459966, 18433, 1855, 55, 47, "Output",
 CellTags->"DiracReduce"]
}, Open  ]],
Cell[CellGroupData[{
Cell[461858, 18493, 253, 7, 26, "Input",
 CellTags->"DiracReduce"],
Cell[462114, 18502, 169, 5, 26, "Output",
 CellTags->"DiracReduce"]
}, Open  ]],
Cell[CellGroupData[{
Cell[462320, 18512, 133, 4, 26, "Input",
 CellTags->"DiracReduce"],
Cell[462456, 18518, 169, 5, 26, "Output",
 CellTags->"DiracReduce"]
}, Open  ]],
Cell[462640, 18526, 156, 4, 26, "Input",
 CellTags->"DiracReduce"]
}, Open  ]],
Cell[462811, 18533, 57, 1, 39, "ObjectName",
 CellTags->"DiracSigma"],
Cell[CellGroupData[{
Cell[462893, 18538, 61, 1, 34, "Subsubsection",
 CellTags->"DiracSigma"],
Cell[462957, 18541, 213, 5, 41, "Text",
 CellTags->"DiracSigma"],
Cell[463173, 18548, 211, 9, 24, "Text",
 CellTags->"DiracSigma"]
}, Open  ]],
Cell[CellGroupData[{
Cell[463421, 18562, 58, 1, 34, "Subsubsection",
 CellTags->"DiracSigma"],
Cell[CellGroupData[{
Cell[463504, 18567, 236, 7, 26, "Input",
 CellTags->"DiracSigma"],
Cell[463743, 18576, 211, 6, 26, "Output",
 CellTags->"DiracSigma"]
}, Open  ]],
Cell[CellGroupData[{
Cell[463991, 18587, 122, 3, 26, "Input",
 CellTags->"DiracSigma"],
Cell[464116, 18592, 826, 30, 44, "Output",
 CellTags->"DiracSigma"]
}, Open  ]],
Cell[CellGroupData[{
Cell[464979, 18627, 236, 7, 26, "Input",
 CellTags->"DiracSigma"],
Cell[465218, 18636, 231, 7, 26, "Output",
 CellTags->"DiracSigma"]
}, Open  ]],
Cell[CellGroupData[{
Cell[465486, 18648, 223, 7, 26, "Input",
 CellTags->"DiracSigma"],
Cell[465712, 18657, 200, 6, 26, "Output",
 CellTags->"DiracSigma"]
}, Open  ]],
Cell[CellGroupData[{
Cell[465949, 18668, 122, 3, 26, "Input",
 CellTags->"DiracSigma"],
Cell[466074, 18673, 994, 34, 44, "Output",
 CellTags->"DiracSigma"]
}, Open  ]],
Cell[467083, 18710, 144, 4, 26, "Input",
 CellTags->"DiracSigma"]
}, Open  ]],
Cell[467242, 18717, 73, 1, 39, "ObjectName",
 CellTags->"DiracSigmaExplicit"],
Cell[CellGroupData[{
Cell[467340, 18722, 69, 1, 34, "Subsubsection",
 CellTags->"DiracSigmaExplicit"],
Cell[467412, 18725, 186, 4, 24, "Text",
 CellTags->"DiracSigmaExplicit"],
Cell[467601, 18731, 195, 9, 24, "Text",
 CellTags->"DiracSigmaExplicit"]
}, Open  ]],
Cell[CellGroupData[{
Cell[467833, 18745, 66, 1, 34, "Subsubsection",
 CellTags->"DiracSigmaExplicit"],
Cell[CellGroupData[{
Cell[467924, 18750, 217, 6, 26, "Input",
 CellTags->"DiracSigmaExplicit"],
Cell[468144, 18758, 219, 6, 26, "Output",
 CellTags->"DiracSigmaExplicit"]
}, Open  ]],
Cell[CellGroupData[{
Cell[468400, 18769, 129, 3, 26, "Input",
 CellTags->"DiracSigmaExplicit"],
Cell[468532, 18774, 834, 30, 44, "Output",
 CellTags->"DiracSigmaExplicit"]
}, Open  ]]
}, Open  ]],
Cell[469393, 18808, 69, 1, 39, "ObjectName",
 CellTags->"DiracSimpCombine"],
Cell[CellGroupData[{
Cell[469487, 18813, 67, 1, 34, "Subsubsection",
 CellTags->"DiracSimpCombine"],
Cell[469557, 18816, 209, 5, 41, "Text",
 CellTags->"DiracSimpCombine"],
Cell[469769, 18823, 196, 8, 24, "Text",
 CellTags->"DiracSimpCombine"]
}, Open  ]],
Cell[469980, 18834, 63, 1, 39, "ObjectName",
 CellTags->"DiracSimplify"],
Cell[CellGroupData[{
Cell[470068, 18839, 64, 1, 34, "Subsubsection",
 CellTags->"DiracSimplify"],
Cell[470135, 18842, 382, 7, 58, "Text",
 CellTags->"DiracSimplify"],
Cell[CellGroupData[{
Cell[470542, 18853, 125, 3, 26, "Input",
 CellTags->"DiracSimplify"],
Cell[470670, 18858, 646, 15, 47, "Output",
 CellTags->"DiracSimplify"]
}, Open  ]],
Cell[471331, 18876, 422, 19, 24, "Text",
 CellTags->"DiracSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[471790, 18900, 61, 1, 34, "Subsubsection",
 CellTags->"DiracSimplify"],
Cell[471854, 18903, 99, 1, 24, "Text",
 CellTags->"DiracSimplify"],
Cell[CellGroupData[{
Cell[471978, 18908, 178, 5, 26, "Input",
 CellTags->"DiracSimplify"],
Cell[472159, 18915, 294, 8, 26, "Output",
 CellTags->"DiracSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[472490, 18928, 120, 3, 26, "Input",
 CellTags->"DiracSimplify"],
Cell[472613, 18933, 302, 11, 26, "Output",
 CellTags->"DiracSimplify"]
}, Open  ]],
Cell[472930, 18947, 96, 1, 24, "Text",
 CellTags->"DiracSimplify"],
Cell[CellGroupData[{
Cell[473051, 18952, 179, 5, 26, "Input",
 CellTags->"DiracSimplify"],
Cell[473233, 18959, 292, 8, 26, "Output",
 CellTags->"DiracSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[473562, 18972, 120, 3, 26, "Input",
 CellTags->"DiracSimplify"],
Cell[473685, 18977, 303, 11, 26, "Output",
 CellTags->"DiracSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[474025, 18993, 174, 5, 26, "Input",
 CellTags->"DiracSimplify"],
Cell[474202, 19000, 288, 8, 26, "Output",
 CellTags->"DiracSimplify"]
}, Open  ]],
Cell[474505, 19011, 179, 7, 24, "Text",
 CellTags->"DiracSimplify"],
Cell[CellGroupData[{
Cell[474709, 19022, 120, 3, 26, "Input",
 CellTags->"DiracSimplify"],
Cell[474832, 19027, 438, 16, 26, "Output",
 CellTags->"DiracSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[475307, 19048, 170, 5, 26, "Input",
 CellTags->"DiracSimplify"],
Cell[475480, 19055, 286, 8, 26, "Output",
 CellTags->"DiracSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[475803, 19068, 120, 3, 26, "Input",
 CellTags->"DiracSimplify"],
Cell[475926, 19073, 316, 11, 26, "Output",
 CellTags->"DiracSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[476279, 19089, 326, 10, 26, "Input",
 CellTags->"DiracSimplify"],
Cell[476608, 19101, 612, 19, 28, "Output",
 CellTags->"DiracSimplify"]
}, Open  ]],
Cell[477235, 19123, 120, 3, 24, "Text",
 CellTags->"DiracSimplify"],
Cell[CellGroupData[{
Cell[477380, 19130, 120, 3, 26, "Input",
 CellTags->"DiracSimplify"],
Cell[477503, 19135, 1952, 73, 28, "Output",
 CellTags->"DiracSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[479492, 19213, 117, 3, 26, "Input",
 CellTags->"DiracSimplify"],
Cell[479612, 19218, 612, 19, 28, "Output",
 CellTags->"DiracSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[480261, 19242, 272, 7, 26, "Input",
 CellTags->"DiracSimplify"],
Cell[480536, 19251, 451, 15, 28, "Output",
 CellTags->"DiracSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[481024, 19271, 120, 3, 26, "Input",
 CellTags->"DiracSimplify"],
Cell[481147, 19276, 153, 4, 26, "Output",
 CellTags->"DiracSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[481337, 19285, 373, 12, 26, "Input",
 CellTags->"DiracSimplify"],
Cell[481713, 19299, 666, 19, 26, "Output",
 CellTags->"DiracSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[482416, 19323, 119, 3, 26, "Input",
 CellTags->"DiracSimplify"],
Cell[482538, 19328, 14006, 532, 107, "Output",
 CellTags->"DiracSimplify"]
}, Open  ]],
Cell[496559, 19863, 115, 3, 24, "Text",
 CellTags->"DiracSimplify"],
Cell[CellGroupData[{
Cell[496699, 19870, 251, 7, 26, "Input",
 CellTags->"DiracSimplify"],
Cell[496953, 19879, 507, 19, 26, "Output",
 CellTags->"DiracSimplify"]
}, Open  ]],
Cell[497475, 19901, 141, 4, 24, "Text",
 CellTags->"DiracSimplify"],
Cell[CellGroupData[{
Cell[497641, 19909, 289, 8, 26, "Input",
 CellTags->"DiracSimplify"],
Cell[497933, 19919, 598, 20, 26, "Output",
 CellTags->"DiracSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[498568, 19944, 268, 7, 26, "Input",
 CellTags->"DiracSimplify"],
Cell[498839, 19953, 151, 4, 26, "Output",
 CellTags->"DiracSimplify"]
}, Open  ]],
Cell[499005, 19960, 183, 5, 26, "Input",
 CellTags->"DiracSimplify"]
}, Open  ]],
Cell[499203, 19968, 65, 1, 39, "ObjectName",
 CellTags->"DiracSimplify2"],
Cell[CellGroupData[{
Cell[499293, 19973, 65, 1, 34, "Subsubsection",
 CellTags->"DiracSimplify2"],
Cell[499361, 19976, 224, 7, 24, "Text",
 CellTags->"DiracSimplify2"],
Cell[499588, 19985, 200, 9, 24, "Text",
 CellTags->"DiracSimplify2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[499825, 19999, 62, 1, 34, "Subsubsection",
 CellTags->"DiracSimplify2"],
Cell[CellGroupData[{
Cell[499912, 20004, 222, 6, 26, "Input",
 CellTags->"DiracSimplify2"],
Cell[500137, 20012, 481, 12, 26, "Output",
 CellTags->"DiracSimplify2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[500655, 20029, 121, 3, 26, "Input",
 CellTags->"DiracSimplify2"],
Cell[500779, 20034, 924, 36, 26, "Output",
 CellTags->"DiracSimplify2"]
}, Open  ]]
}, Open  ]],
Cell[501730, 20074, 57, 1, 39, "ObjectName",
 CellTags->"DiracSlash"],
Cell[CellGroupData[{
Cell[501812, 20079, 61, 1, 34, "Subsubsection",
 CellTags->"DiracSlash"],
Cell[501876, 20082, 469, 16, 44, "Text",
 CellTags->"DiracSlash"],
Cell[CellGroupData[{
Cell[502370, 20102, 119, 3, 26, "Input",
 CellTags->"DiracSlash"],
Cell[502492, 20107, 287, 9, 26, "Output",
 CellTags->"DiracSlash"]
}, Open  ]],
Cell[502794, 20119, 868, 39, 24, "Text",
 CellTags->"DiracSlash"]
}, Open  ]],
Cell[CellGroupData[{
Cell[503699, 20163, 58, 1, 34, "Subsubsection",
 CellTags->"DiracSlash"],
Cell[503760, 20166, 234, 9, 27, "Text",
 CellTags->"DiracSlash"],
Cell[CellGroupData[{
Cell[504019, 20179, 113, 3, 26, "Input",
 CellTags->"DiracSlash"],
Cell[504135, 20184, 267, 9, 28, "Output",
 CellTags->"DiracSlash"]
}, Open  ]],
Cell[CellGroupData[{
Cell[504439, 20198, 173, 5, 26, "Input",
 CellTags->"DiracSlash"],
Cell[504615, 20205, 473, 17, 28, "Output",
 CellTags->"DiracSlash"]
}, Open  ]],
Cell[CellGroupData[{
Cell[505125, 20227, 136, 4, 26, "Input",
 CellTags->"DiracSlash"],
Cell[505264, 20233, 475, 17, 28, "Output",
 CellTags->"DiracSlash"]
}, Open  ]],
Cell[CellGroupData[{
Cell[505776, 20255, 105, 3, 26, "Input",
 CellTags->"DiracSlash"],
Cell[505884, 20260, 191, 5, 28, "Output",
 CellTags->"DiracSlash"]
}, Open  ]],
Cell[CellGroupData[{
Cell[506112, 20270, 148, 4, 26, "Input",
 CellTags->"DiracSlash"],
Cell[506263, 20276, 170, 4, 38, "Output",
 CellTags->"DiracSlash"]
}, Open  ]],
Cell[CellGroupData[{
Cell[506470, 20285, 214, 7, 26, "Input",
 CellTags->"DiracSlash"],
Cell[506687, 20294, 231, 6, 38, "Output",
 CellTags->"DiracSlash"]
}, Open  ]]
}, Open  ]],
Cell[506945, 20304, 59, 1, 39, "ObjectName",
 CellTags->"DiracSpinor"],
Cell[CellGroupData[{
Cell[507029, 20309, 62, 1, 34, "Subsubsection",
 CellTags->"DiracSpinor"],
Cell[507094, 20312, 288, 6, 41, "Text",
 CellTags->"DiracSpinor"],
Cell[507385, 20320, 170, 8, 24, "Text",
 CellTags->"DiracSpinor"]
}, Open  ]],
Cell[507570, 20331, 71, 1, 39, "ObjectName",
 CellTags->"DiracSubstitute67"],
Cell[CellGroupData[{
Cell[507666, 20336, 68, 1, 34, "Subsubsection",
 CellTags->"DiracSubstitute67"],
Cell[507737, 20339, 219, 5, 41, "Text",
 CellTags->"DiracSubstitute67"],
Cell[507959, 20346, 313, 13, 24, "Text",
 CellTags->"DiracSubstitute67"]
}, Open  ]],
Cell[508287, 20362, 57, 1, 39, "ObjectName",
 CellTags->"DiracTrace"],
Cell[CellGroupData[{
Cell[508369, 20367, 61, 1, 34, "Subsubsection",
 CellTags->"DiracTrace"],
Cell[508433, 20370, 336, 6, 58, "Text",
 CellTags->"DiracTrace"],
Cell[CellGroupData[{
Cell[508794, 20380, 119, 3, 26, "Input",
 CellTags->"DiracTrace"],
Cell[508916, 20385, 706, 17, 47, "Output",
 CellTags->"DiracTrace"]
}, Open  ]],
Cell[509637, 20405, 283, 12, 24, "Text",
 CellTags->"DiracTrace"],
Cell[509923, 20419, 163, 9, 24, "Text",
 CellTags->"DiracTrace"]
}, Open  ]],
Cell[CellGroupData[{
Cell[510123, 20433, 58, 1, 34, "Subsubsection",
 CellTags->"DiracTrace"],
Cell[CellGroupData[{
Cell[510206, 20438, 174, 5, 26, "Input",
 CellTags->"DiracTrace"],
Cell[510383, 20445, 312, 10, 26, "Output",
 CellTags->"DiracTrace"]
}, Open  ]],
Cell[CellGroupData[{
Cell[510732, 20460, 209, 6, 26, "Input",
 CellTags->"DiracTrace"],
Cell[510944, 20468, 412, 12, 26, "Output",
 CellTags->"DiracTrace"]
}, Open  ]],
Cell[CellGroupData[{
Cell[511393, 20485, 149, 4, 26, "Input",
 CellTags->"DiracTrace"],
Cell[511545, 20491, 676, 23, 26, "Output",
 CellTags->"DiracTrace"]
}, Open  ]],
Cell[CellGroupData[{
Cell[512258, 20519, 296, 8, 26, "Input",
 CellTags->"DiracTrace"],
Cell[512557, 20529, 608, 24, 26, "Output",
 CellTags->"DiracTrace"]
}, Open  ]],
Cell[CellGroupData[{
Cell[513202, 20558, 334, 9, 26, "Input",
 CellTags->"DiracTrace"],
Cell[513539, 20569, 8767, 322, 73, "Output",
 CellTags->"DiracTrace"]
}, Open  ]],
Cell[CellGroupData[{
Cell[522343, 20896, 186, 5, 26, "Input",
 CellTags->"DiracTrace"],
Cell[522532, 20903, 540, 16, 28, "Output",
 CellTags->"DiracTrace"]
}, Open  ]],
Cell[CellGroupData[{
Cell[523109, 20924, 294, 8, 26, "Input",
 CellTags->"DiracTrace"],
Cell[523406, 20934, 224, 7, 26, "Output",
 CellTags->"DiracTrace"]
}, Open  ]],
Cell[CellGroupData[{
Cell[523667, 20946, 111, 3, 26, "Input",
 CellTags->"DiracTrace"],
Cell[523781, 20951, 217, 6, 38, "Output",
 CellTags->"DiracTrace"]
}, Open  ]]
}, Open  ]],
Cell[524025, 20961, 73, 1, 39, "ObjectName",
 CellTags->"DiracTraceEvaluate"],
Cell[CellGroupData[{
Cell[524123, 20966, 69, 1, 34, "Subsubsection",
 CellTags->"DiracTraceEvaluate"],
Cell[524195, 20969, 162, 4, 24, "Text",
 CellTags->"DiracTraceEvaluate"],
Cell[524360, 20975, 281, 13, 24, "Text",
 CellTags->"DiracTraceEvaluate"]
}, Open  ]],
Cell[524656, 20991, 57, 1, 39, "ObjectName",
 CellTags->"DiracTrick"],
Cell[CellGroupData[{
Cell[524738, 20996, 61, 1, 34, "Subsubsection",
 CellTags->"DiracTrick"],
Cell[524802, 20999, 222, 5, 41, "Text",
 CellTags->"DiracTrick"],
Cell[CellGroupData[{
Cell[525049, 21008, 119, 3, 26, "Input",
 CellTags->"DiracTrick"],
Cell[525171, 21013, 218, 7, 26, "Output",
 CellTags->"DiracTrick"]
}, Open  ]],
Cell[525404, 21023, 428, 19, 24, "Text",
 CellTags->"DiracTrick"]
}, Open  ]],
Cell[CellGroupData[{
Cell[525869, 21047, 58, 1, 34, "Subsubsection",
 CellTags->"DiracTrick"],
Cell[525930, 21050, 96, 1, 24, "Text",
 CellTags->"DiracTrick"],
Cell[CellGroupData[{
Cell[526051, 21055, 175, 5, 26, "Input",
 CellTags->"DiracTrick"],
Cell[526229, 21062, 289, 8, 26, "Output",
 CellTags->"DiracTrick"]
}, Open  ]],
Cell[CellGroupData[{
Cell[526555, 21075, 114, 3, 26, "Input",
 CellTags->"DiracTrick"],
Cell[526672, 21080, 289, 8, 26, "Output",
 CellTags->"DiracTrick"]
}, Open  ]],
Cell[526976, 21091, 93, 1, 24, "Text",
 CellTags->"DiracTrick"],
Cell[CellGroupData[{
Cell[527094, 21096, 176, 5, 26, "Input",
 CellTags->"DiracTrick"],
Cell[527273, 21103, 289, 8, 26, "Output",
 CellTags->"DiracTrick"]
}, Open  ]],
Cell[CellGroupData[{
Cell[527599, 21116, 114, 3, 26, "Input",
 CellTags->"DiracTrick"],
Cell[527716, 21121, 291, 8, 26, "Output",
 CellTags->"DiracTrick"]
}, Open  ]],
Cell[CellGroupData[{
Cell[528044, 21134, 171, 5, 26, "Input",
 CellTags->"DiracTrick"],
Cell[528218, 21141, 285, 8, 26, "Output",
 CellTags->"DiracTrick"]
}, Open  ]],
Cell[528518, 21152, 176, 7, 24, "Text",
 CellTags->"DiracTrick"],
Cell[CellGroupData[{
Cell[528719, 21163, 114, 3, 26, "Input",
 CellTags->"DiracTrick"],
Cell[528836, 21168, 287, 8, 26, "Output",
 CellTags->"DiracTrick"]
}, Open  ]],
Cell[CellGroupData[{
Cell[529160, 21181, 167, 5, 26, "Input",
 CellTags->"DiracTrick"],
Cell[529330, 21188, 281, 8, 26, "Output",
 CellTags->"DiracTrick"]
}, Open  ]],
Cell[CellGroupData[{
Cell[529648, 21201, 114, 3, 26, "Input",
 CellTags->"DiracTrick"],
Cell[529765, 21206, 281, 8, 26, "Output",
 CellTags->"DiracTrick"]
}, Open  ]],
Cell[CellGroupData[{
Cell[530083, 21219, 323, 10, 26, "Input",
 CellTags->"DiracTrick"],
Cell[530409, 21231, 609, 19, 28, "Output",
 CellTags->"DiracTrick"]
}, Open  ]],
Cell[CellGroupData[{
Cell[531055, 21255, 114, 3, 26, "Input",
 CellTags->"DiracTrick"],
Cell[531172, 21260, 609, 19, 28, "Output",
 CellTags->"DiracTrick"]
}, Open  ]],
Cell[CellGroupData[{
Cell[531818, 21284, 108, 3, 26, "Input",
 CellTags->"DiracTrick"],
Cell[531929, 21289, 1947, 73, 28, "Output",
 CellTags->"DiracTrick"]
}, Open  ]],
Cell[CellGroupData[{
Cell[533913, 21367, 370, 12, 26, "Input",
 CellTags->"DiracTrick"],
Cell[534286, 21381, 663, 19, 26, "Output",
 CellTags->"DiracTrick"]
}, Open  ]],
Cell[CellGroupData[{
Cell[534986, 21405, 113, 3, 26, "Input",
 CellTags->"DiracTrick"],
Cell[535102, 21410, 663, 19, 26, "Output",
 CellTags->"DiracTrick"]
}, Open  ]]
}, Open  ]],
Cell[535792, 21433, 55, 1, 39, "ObjectName",
 CellTags->"Divideout"],
Cell[CellGroupData[{
Cell[535872, 21438, 60, 1, 34, "Subsubsection",
 CellTags->"Divideout"],
Cell[535935, 21441, 137, 4, 24, "Text",
 CellTags->"Divideout"],
Cell[536075, 21447, 281, 13, 24, "Text",
 CellTags->"Divideout"]
}, Open  ]],
Cell[536371, 21463, 43, 1, 39, "ObjectName",
 CellTags->"DOT"],
Cell[CellGroupData[{
Cell[536439, 21468, 54, 1, 34, "Subsubsection",
 CellTags->"DOT"],
Cell[536496, 21471, 348, 8, 91, "Text",
 CellTags->"DOT"],
Cell[536847, 21481, 177, 8, 24, "Text",
 CellTags->"DOT"]
}, Open  ]],
Cell[537039, 21492, 55, 1, 39, "ObjectName",
 CellTags->"DotExpand"],
Cell[CellGroupData[{
Cell[537119, 21497, 60, 1, 34, "Subsubsection",
 CellTags->"DotExpand"],
Cell[537182, 21500, 85, 1, 24, "Text",
 CellTags->"DotExpand"],
Cell[537270, 21503, 582, 23, 24, "Text",
 CellTags->"DotExpand"]
}, Open  ]],
Cell[CellGroupData[{
Cell[537889, 21531, 57, 1, 34, "Subsubsection",
 CellTags->"DotExpand"],
Cell[CellGroupData[{
Cell[537971, 21536, 313, 10, 26, "Input",
 CellTags->"DotExpand"],
Cell[538287, 21548, 546, 14, 26, "Output",
 CellTags->"DotExpand"]
}, Open  ]],
Cell[538848, 21565, 229, 7, 26, "Input",
 CellTags->"DotExpand"],
Cell[CellGroupData[{
Cell[539102, 21576, 313, 10, 26, "Input",
 CellTags->"DotExpand"],
Cell[539418, 21588, 681, 23, 26, "Output",
 CellTags->"DotExpand"]
}, Open  ]],
Cell[540114, 21614, 231, 7, 26, "Input",
 CellTags->"DotExpand"],
Cell[CellGroupData[{
Cell[540370, 21625, 313, 10, 26, "Input",
 CellTags->"DotExpand"],
Cell[540686, 21637, 546, 14, 26, "Output",
 CellTags->"DotExpand"]
}, Open  ]]
}, Open  ]],
Cell[541259, 21655, 53, 1, 39, "ObjectName",
 CellTags->"DotPower"],
Cell[CellGroupData[{
Cell[541337, 21660, 59, 1, 34, "Subsubsection",
 CellTags->"DotPower"],
Cell[541399, 21663, 189, 4, 41, "Text",
 CellTags->"DotPower"],
Cell[541591, 21669, 182, 8, 24, "Text",
 CellTags->"DotPower"]
}, Open  ]],
Cell[541788, 21680, 57, 1, 39, "ObjectName",
 CellTags->"DotProduct"],
Cell[CellGroupData[{
Cell[541870, 21685, 61, 1, 34, "Subsubsection",
 CellTags->"DotProduct"],
Cell[541934, 21688, 219, 5, 41, "Text",
 CellTags->"DotProduct"],
Cell[542156, 21695, 312, 14, 24, "Text",
 CellTags->"DotProduct"]
}, Open  ]],
Cell[CellGroupData[{
Cell[542505, 21714, 58, 1, 34, "Subsubsection",
 CellTags->"DotProduct"],
Cell[CellGroupData[{
Cell[542588, 21719, 234, 7, 26, "Input",
 CellTags->"DotProduct"],
Cell[542825, 21728, 356, 12, 30, "Output",
 CellTags->"DotProduct"]
}, Open  ]]
}, Open  ]],
Cell[543208, 21744, 77, 1, 39, "ObjectName",
 CellTags->"DotSimplifyRelations"],
Cell[CellGroupData[{
Cell[543310, 21749, 71, 1, 34, "Subsubsection",
 CellTags->"DotSimplifyRelations"],
Cell[543384, 21752, 241, 5, 41, "Text",
 CellTags->"DotSimplifyRelations"],
Cell[543628, 21759, 194, 8, 24, "Text",
 CellTags->"DotSimplifyRelations"]
}, Open  ]],
Cell[543837, 21770, 59, 1, 39, "ObjectName",
 CellTags->"DotSimplify"],
Cell[CellGroupData[{
Cell[543921, 21775, 62, 1, 34, "Subsubsection",
 CellTags->"DotSimplify"],
Cell[543986, 21778, 322, 6, 58, "Text",
 CellTags->"DotSimplify"],
Cell[CellGroupData[{
Cell[544333, 21788, 121, 3, 26, "Input",
 CellTags->"DotSimplify"],
Cell[544457, 21793, 423, 12, 26, "Output",
 CellTags->"DotSimplify"]
}, Open  ]],
Cell[544895, 21808, 414, 19, 24, "Text",
 CellTags->"DotSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[545346, 21832, 59, 1, 34, "Subsubsection",
 CellTags->"DotSimplify"],
Cell[CellGroupData[{
Cell[545430, 21837, 342, 11, 26, "Input",
 CellTags->"DotSimplify"],
Cell[545775, 21850, 418, 12, 28, "Output",
 CellTags->"DotSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[546230, 21867, 116, 3, 26, "Input",
 CellTags->"DotSimplify"],
Cell[546349, 21872, 1057, 40, 28, "Output",
 CellTags->"DotSimplify"]
}, Open  ]],
Cell[547421, 21915, 158, 4, 26, "Input",
 CellTags->"DotSimplify"],
Cell[CellGroupData[{
Cell[547604, 21923, 209, 7, 26, "Input",
 CellTags->"DotSimplify"],
Cell[547816, 21932, 259, 9, 26, "Output",
 CellTags->"DotSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[548112, 21946, 116, 3, 26, "Input",
 CellTags->"DotSimplify"],
Cell[548231, 21951, 265, 8, 26, "Output",
 CellTags->"DotSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[548533, 21964, 161, 5, 26, "Input",
 CellTags->"DotSimplify"],
Cell[548697, 21971, 149, 4, 26, "Output",
 CellTags->"DotSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[548883, 21980, 116, 3, 26, "Input",
 CellTags->"DotSimplify"],
Cell[549002, 21985, 321, 11, 26, "Output",
 CellTags->"DotSimplify"]
}, Open  ]],
Cell[549338, 21999, 161, 5, 26, "Input",
 CellTags->"DotSimplify"],
Cell[CellGroupData[{
Cell[549524, 22008, 116, 3, 26, "Input",
 CellTags->"DotSimplify"],
Cell[549643, 22013, 265, 8, 26, "Output",
 CellTags->"DotSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[549945, 22026, 165, 5, 26, "Input",
 CellTags->"DotSimplify"],
Cell[550113, 22033, 151, 4, 27, "Output",
 CellTags->"DotSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[550301, 22042, 116, 3, 26, "Input",
 CellTags->"DotSimplify"],
Cell[550420, 22047, 319, 10, 26, "Output",
 CellTags->"DotSimplify"]
}, Open  ]],
Cell[550754, 22060, 165, 5, 26, "Input",
 CellTags->"DotSimplify"],
Cell[CellGroupData[{
Cell[550944, 22069, 299, 9, 26, "Input",
 CellTags->"DotSimplify"],
Cell[551246, 22080, 208, 7, 26, "Output",
 CellTags->"DotSimplify"]
}, Open  ]],
Cell[551469, 22090, 125, 3, 26, "Input",
 CellTags->"DotSimplify"],
Cell[CellGroupData[{
Cell[551619, 22097, 148, 4, 26, "Input",
 CellTags->"DotSimplify"],
Cell[551770, 22103, 182, 5, 26, "Output",
 CellTags->"DotSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[551989, 22113, 213, 6, 26, "Input",
 CellTags->"DotSimplify"],
Cell[552205, 22121, 184, 5, 26, "Output",
 CellTags->"DotSimplify"]
}, Open  ]],
Cell[552404, 22129, 170, 4, 26, "Input",
 CellTags->"DotSimplify"]
}, Open  ]],
Cell[552589, 22136, 57, 1, 39, "ObjectName",
 CellTags->"DummyIndex"],
Cell[CellGroupData[{
Cell[552671, 22141, 61, 1, 34, "Subsubsection",
 CellTags->"DummyIndex"],
Cell[552735, 22144, 191, 4, 41, "Text",
 CellTags->"DummyIndex"],
Cell[552929, 22150, 181, 8, 24, "Text",
 CellTags->"DummyIndex"]
}, Open  ]],
Cell[553125, 22161, 41, 1, 39, "ObjectName",
 CellTags->"D0"],
Cell[CellGroupData[{
Cell[553191, 22166, 53, 1, 34, "Subsubsection",
 CellTags->"D0"],
Cell[553247, 22169, 524, 12, 58, "Text",
 CellTags->"D0"],
Cell[553774, 22183, 458, 24, 24, "Text",
 CellTags->"D0"]
}, Open  ]],
Cell[CellGroupData[{
Cell[554269, 22212, 50, 1, 34, "Subsubsection",
 CellTags->"D0"],
Cell[CellGroupData[{
Cell[554344, 22217, 315, 9, 26, "Input",
 CellTags->"D0"],
Cell[554662, 22228, 592, 21, 29, "Output",
 CellTags->"D0"]
}, Open  ]],
Cell[CellGroupData[{
Cell[555291, 22254, 484, 14, 46, "Input",
 CellTags->"D0"],
Cell[555778, 22270, 592, 21, 29, "Output",
 CellTags->"D0"]
}, Open  ]],
Cell[CellGroupData[{
Cell[556407, 22296, 104, 3, 26, "Input",
 CellTags->"D0"],
Cell[556514, 22301, 594, 21, 29, "Output",
 CellTags->"D0"]
}, Open  ]]
}, Open  ]],
Cell[557135, 22326, 61, 1, 39, "ObjectName",
 CellTags->"D0Convention"],
Cell[CellGroupData[{
Cell[557221, 22331, 63, 1, 34, "Subsubsection",
 CellTags->"D0Convention"],
Cell[557287, 22334, 307, 6, 41, "Text",
 CellTags->"D0Convention"],
Cell[557597, 22342, 263, 13, 24, "Text",
 CellTags->"D0Convention"]
}, Open  ]],
Cell[557875, 22358, 43, 1, 39, "ObjectName",
 CellTags->"Eps"],
Cell[CellGroupData[{
Cell[557943, 22363, 54, 1, 34, "Subsubsection",
 CellTags->"Eps"],
Cell[558000, 22366, 345, 8, 41, "Text",
 CellTags->"Eps"],
Cell[CellGroupData[{
Cell[558370, 22378, 105, 3, 26, "Input",
 CellTags->"Eps"],
Cell[558478, 22383, 204, 6, 26, "Output",
 CellTags->"Eps"]
}, Open  ]],
Cell[558697, 22392, 486, 24, 24, "Text",
 CellTags->"Eps"]
}, Open  ]],
Cell[CellGroupData[{
Cell[559220, 22421, 51, 1, 34, "Subsubsection",
 CellTags->"Eps"],
Cell[CellGroupData[{
Cell[559296, 22426, 322, 8, 26, "Input",
 CellTags->"Eps"],
Cell[559621, 22436, 518, 22, 26, "Output",
 CellTags->"Eps"]
}, Open  ]],
Cell[CellGroupData[{
Cell[560176, 22463, 314, 8, 26, "Input",
 CellTags->"Eps"],
Cell[560493, 22473, 477, 20, 26, "Output",
 CellTags->"Eps"]
}, Open  ]],
Cell[CellGroupData[{
Cell[561007, 22498, 181, 6, 26, "Input",
 CellTags->"Eps"],
Cell[561191, 22506, 218, 6, 38, "Output",
 CellTags->"Eps"]
}, Open  ]],
Cell[CellGroupData[{
Cell[561446, 22517, 142, 4, 26, "Input",
 CellTags->"Eps"],
Cell[561591, 22523, 141, 4, 26, "Output",
 CellTags->"Eps"]
}, Open  ]],
Cell[CellGroupData[{
Cell[561769, 22532, 142, 4, 26, "Input",
 CellTags->"Eps"],
Cell[561914, 22538, 161, 5, 26, "Output",
 CellTags->"Eps"]
}, Open  ]],
Cell[562090, 22546, 189, 6, 26, "Input",
 CellTags->"Eps"],
Cell[CellGroupData[{
Cell[562304, 22556, 460, 13, 26, "Input",
 CellTags->"Eps"],
Cell[562767, 22571, 519, 22, 26, "Output",
 CellTags->"Eps"]
}, Open  ]],
Cell[CellGroupData[{
Cell[563323, 22598, 107, 3, 26, "Input",
 CellTags->"Eps"],
Cell[563433, 22603, 144, 4, 26, "Output",
 CellTags->"Eps"]
}, Open  ]],
Cell[563592, 22610, 189, 6, 26, "Input",
 CellTags->"Eps"],
Cell[CellGroupData[{
Cell[563806, 22620, 460, 13, 26, "Input",
 CellTags->"Eps"],
Cell[564269, 22635, 519, 22, 26, "Output",
 CellTags->"Eps"]
}, Open  ]],
Cell[CellGroupData[{
Cell[564825, 22662, 137, 4, 26, "Input",
 CellTags->"Eps"],
Cell[564965, 22668, 144, 4, 26, "Output",
 CellTags->"Eps"]
}, Open  ]],
Cell[CellGroupData[{
Cell[565146, 22677, 293, 9, 26, "Input"],
Cell[565442, 22688, 410, 10, 26, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[565889, 22703, 77, 2, 26, "Input"],
Cell[565969, 22707, 1018, 42, 26, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[567024, 22754, 433, 14, 50, "Input",
 CellTags->"Eps"],
Cell[567460, 22770, 1163, 46, 44, "Output",
 CellTags->"Eps"]
}, Open  ]],
Cell[CellGroupData[{
Cell[568660, 22821, 978, 28, 50, "Input",
 CellTags->"Eps"],
Cell[569641, 22851, 1721, 62, 44, "Output",
 CellTags->"Eps"]
}, Open  ]],
Cell[CellGroupData[{
Cell[571399, 22918, 188, 6, 26, "Input",
 CellTags->"Eps"],
Cell[571590, 22926, 384, 13, 44, "Output",
 CellTags->"Eps"]
}, Open  ]],
Cell[CellGroupData[{
Cell[572011, 22944, 124, 4, 26, "Input",
 CellTags->"Eps"],
Cell[572138, 22950, 141, 4, 26, "Output",
 CellTags->"Eps"]
}, Open  ]],
Cell[572294, 22957, 164, 5, 26, "Input",
 CellTags->"Eps"]
}, Open  ]],
Cell[572473, 22965, 59, 1, 39, "ObjectName",
 CellTags->"EpsChisholm"],
Cell[CellGroupData[{
Cell[572557, 22970, 62, 1, 34, "Subsubsection",
 CellTags->"EpsChisholm"],
Cell[572622, 22973, 166, 4, 24, "Text",
 CellTags->"EpsChisholm"],
Cell[572791, 22979, 182, 9, 24, "Text",
 CellTags->"EpsChisholm"]
}, Open  ]],
Cell[CellGroupData[{
Cell[573010, 22993, 59, 1, 34, "Subsubsection",
 CellTags->"EpsChisholm"],
Cell[CellGroupData[{
Cell[573094, 22998, 208, 6, 26, "Input",
 CellTags->"EpsChisholm"],
Cell[573305, 23006, 2061, 79, 26, "Output",
 CellTags->"EpsChisholm"]
}, Open  ]],
Cell[CellGroupData[{
Cell[575403, 23090, 115, 3, 26, "Input",
 CellTags->"EpsChisholm"],
Cell[575521, 23095, 636, 25, 26, "Output",
 CellTags->"EpsChisholm"]
}, Open  ]]
}, Open  ]],
Cell[576184, 23124, 59, 1, 39, "ObjectName",
 CellTags->"EpsContract"],
Cell[CellGroupData[{
Cell[576268, 23129, 62, 1, 34, "Subsubsection",
 CellTags->"EpsContract"],
Cell[576333, 23132, 227, 5, 41, "Text",
 CellTags->"EpsContract"],
Cell[576563, 23139, 277, 14, 24, "Text",
 CellTags->"EpsContract"]
}, Open  ]],
Cell[CellGroupData[{
Cell[576877, 23158, 59, 1, 34, "Subsubsection",
 CellTags->"EpsContract"],
Cell[CellGroupData[{
Cell[576961, 23163, 468, 13, 26, "Input",
 CellTags->"EpsContract"],
Cell[577432, 23178, 527, 22, 26, "Output",
 CellTags->"EpsContract"]
}, Open  ]],
Cell[CellGroupData[{
Cell[577996, 23205, 205, 6, 26, "Input",
 CellTags->"EpsContract"],
Cell[578204, 23213, 613, 24, 26, "Output",
 CellTags->"EpsContract"]
}, Open  ]],
Cell[CellGroupData[{
Cell[578854, 23242, 204, 6, 26, "Input",
 CellTags->"EpsContract"],
Cell[579061, 23250, 362, 12, 26, "Output",
 CellTags->"EpsContract"]
}, Open  ]],
Cell[579438, 23265, 84, 2, 26, "Input"]
}, Open  ]],
Cell[579537, 23270, 57, 1, 39, "ObjectName",
 CellTags->"EpsDiscard"],
Cell[CellGroupData[{
Cell[579619, 23275, 61, 1, 34, "Subsubsection",
 CellTags->"EpsDiscard"],
Cell[579683, 23278, 169, 4, 24, "Text",
 CellTags->"EpsDiscard"],
Cell[579855, 23284, 196, 9, 24, "Text",
 CellTags->"EpsDiscard"]
}, Open  ]],
Cell[580066, 23296, 59, 1, 39, "ObjectName",
 CellTags->"EpsEvaluate"],
Cell[CellGroupData[{
Cell[580150, 23301, 62, 1, 34, "Subsubsection",
 CellTags->"EpsEvaluate"],
Cell[580215, 23304, 173, 4, 24, "Text",
 CellTags->"EpsEvaluate"],
Cell[580391, 23310, 494, 24, 24, "Text",
 CellTags->"EpsEvaluate"]
}, Open  ]],
Cell[CellGroupData[{
Cell[580922, 23339, 59, 1, 34, "Subsubsection",
 CellTags->"EpsEvaluate"],
Cell[CellGroupData[{
Cell[581006, 23344, 335, 10, 26, "Input",
 CellTags->"EpsEvaluate"],
Cell[581344, 23356, 508, 21, 26, "Output",
 CellTags->"EpsEvaluate"]
}, Open  ]],
Cell[CellGroupData[{
Cell[581889, 23382, 115, 3, 26, "Input",
 CellTags->"EpsEvaluate"],
Cell[582007, 23387, 508, 21, 26, "Output",
 CellTags->"EpsEvaluate"]
}, Open  ]],
Cell[CellGroupData[{
Cell[582552, 23413, 116, 3, 26, "Input",
 CellTags->"EpsEvaluate"],
Cell[582671, 23418, 476, 13, 38, "Output",
 CellTags->"EpsEvaluate"]
}, Open  ]]
}, Open  ]],
Cell[583174, 23435, 63, 1, 39, "ObjectName",
 CellTags->"EpsUncontract"],
Cell[CellGroupData[{
Cell[583262, 23440, 64, 1, 34, "Subsubsection",
 CellTags->"EpsUncontract"],
Cell[583329, 23443, 115, 3, 24, "Text",
 CellTags->"EpsUncontract"],
Cell[583447, 23448, 285, 14, 24, "Text",
 CellTags->"EpsUncontract"]
}, Open  ]],
Cell[583747, 23465, 51, 1, 39, "ObjectName",
 CellTags->"Epsilon"],
Cell[CellGroupData[{
Cell[583823, 23470, 58, 1, 34, "Subsubsection",
 CellTags->"Epsilon"],
Cell[583884, 23473, 237, 9, 24, "Text",
 CellTags->"Epsilon"],
Cell[584124, 23484, 175, 9, 24, "Text",
 CellTags->"Epsilon"]
}, Open  ]],
Cell[CellGroupData[{
Cell[584336, 23498, 55, 1, 34, "Subsubsection",
 CellTags->"Epsilon"],
Cell[CellGroupData[{
Cell[584416, 23503, 80, 2, 26, "Input",
 CellTags->"Epsilon"],
Cell[584499, 23507, 190, 6, 26, "Output",
 CellTags->"Epsilon"]
}, Open  ]],
Cell[584704, 23516, 114, 3, 24, "Text",
 CellTags->"Epsilon"],
Cell[CellGroupData[{
Cell[584843, 23523, 587, 19, 46, "Input",
 CellTags->"Epsilon"],
Cell[585433, 23544, 241, 7, 26, "Output",
 CellTags->"Epsilon"]
}, Open  ]]
}, Open  ]],
Cell[585701, 23555, 61, 1, 39, "ObjectName",
 CellTags->"EpsilonOrder"],
Cell[CellGroupData[{
Cell[585787, 23560, 63, 1, 34, "Subsubsection",
 CellTags->"EpsilonOrder"],
Cell[585853, 23563, 305, 12, 24, "Text",
 CellTags->"EpsilonOrder"],
Cell[586161, 23577, 287, 14, 24, "Text",
 CellTags->"EpsilonOrder"]
}, Open  ]],
Cell[586463, 23594, 55, 1, 39, "ObjectName",
 CellTags->"Expanding"],
Cell[CellGroupData[{
Cell[586543, 23599, 60, 1, 34, "Subsubsection",
 CellTags->"Expanding"],
Cell[586606, 23602, 378, 7, 58, "Text",
 CellTags->"Expanding"],
Cell[586987, 23611, 641, 29, 24, "Text",
 CellTags->"Expanding"]
}, Open  ]],
Cell[587643, 23643, 65, 1, 39, "ObjectName",
 CellTags->"ExpandPartialD"],
Cell[CellGroupData[{
Cell[587733, 23648, 65, 1, 34, "Subsubsection",
 CellTags->"ExpandPartialD"],
Cell[587801, 23651, 188, 4, 24, "Text",
 CellTags->"ExpandPartialD"],
Cell[CellGroupData[{
Cell[588014, 23659, 98, 2, 26, "Input"],
Cell[588115, 23663, 1257, 37, 93, "Output"]
}, Open  ]],
Cell[589387, 23703, 724, 28, 24, "Text",
 CellTags->"ExpandPartialD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[590148, 23736, 62, 1, 34, "Subsubsection",
 CellTags->"ExpandPartialD"],
Cell[CellGroupData[{
Cell[590235, 23741, 373, 10, 26, "Input",
 CellTags->"ExpandPartialD"],
Cell[590611, 23753, 387, 13, 33, "Output",
 CellTags->"ExpandPartialD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[591035, 23771, 121, 3, 26, "Input",
 CellTags->"ExpandPartialD"],
Cell[591159, 23776, 1253, 31, 150, "Output",
 CellTags->"ExpandPartialD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[592449, 23812, 115, 3, 26, "Input",
 CellTags->"ExpandPartialD"],
Cell[592567, 23817, 1400, 30, 210, "Output",
 CellTags->"ExpandPartialD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[594004, 23852, 259, 7, 26, "Input",
 CellTags->"ExpandPartialD"],
Cell[594266, 23861, 354, 12, 33, "Output",
 CellTags->"ExpandPartialD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[594657, 23878, 121, 3, 26, "Input",
 CellTags->"ExpandPartialD"],
Cell[594781, 23883, 962, 27, 117, "Output",
 CellTags->"ExpandPartialD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[595780, 23915, 433, 13, 46, "Input",
 CellTags->"ExpandPartialD"],
Cell[596216, 23930, 376, 11, 35, "Output",
 CellTags->"ExpandPartialD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[596629, 23946, 121, 3, 26, "Input",
 CellTags->"ExpandPartialD"],
Cell[596753, 23951, 3062, 66, 411, "Output",
 CellTags->"ExpandPartialD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[599852, 24022, 180, 6, 26, "Input",
 CellTags->"ExpandPartialD"],
Cell[600035, 24030, 308, 9, 34, "Output",
 CellTags->"ExpandPartialD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[600380, 24044, 123, 3, 26, "Input",
 CellTags->"ExpandPartialD"],
Cell[600506, 24049, 480, 15, 38, "Output",
 CellTags->"ExpandPartialD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[601023, 24069, 121, 3, 26, "Input",
 CellTags->"ExpandPartialD"],
Cell[601147, 24074, 1919, 60, 30, "Output",
 CellTags->"ExpandPartialD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[603103, 24139, 314, 9, 26, "Input",
 CellTags->"ExpandPartialD"],
Cell[603420, 24150, 1234, 48, 33, "Output",
 CellTags->"ExpandPartialD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[604691, 24203, 121, 3, 26, "Input",
 CellTags->"ExpandPartialD"],
Cell[604815, 24208, 152, 4, 26, "Output",
 CellTags->"ExpandPartialD"]
}, Open  ]]
}, Open  ]],
Cell[604994, 24216, 121, 1, 39, "ObjectName",
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
Cell[CellGroupData[{
Cell[605140, 24221, 95, 1, 34, "Subsubsection",
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
Cell[605238, 24224, 252, 5, 49, "Text",
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
Cell[CellGroupData[{
Cell[605515, 24233, 158, 3, 26, "Input",
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
Cell[605676, 24238, 256, 7, 26, "Output",
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}]
}, Open  ]],
Cell[605947, 24248, 462, 19, 24, "Text",
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}]
}, Open  ]],
Cell[CellGroupData[{
Cell[606446, 24272, 92, 1, 34, "Subsubsection",
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
Cell[CellGroupData[{
Cell[606563, 24277, 257, 7, 26, "Input",
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
Cell[606823, 24286, 380, 11, 26, "Output",
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}]
}, Open  ]],
Cell[CellGroupData[{
Cell[607240, 24302, 152, 3, 26, "Input",
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
Cell[607395, 24307, 382, 11, 26, "Output",
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}]
}, Open  ]],
Cell[CellGroupData[{
Cell[607814, 24323, 186, 5, 26, "Input",
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
Cell[608003, 24330, 319, 10, 28, "Output",
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}]
}, Open  ]],
Cell[CellGroupData[{
Cell[608359, 24345, 156, 3, 26, "Input",
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
Cell[608518, 24350, 319, 10, 28, "Output",
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}]
}, Open  ]],
Cell[CellGroupData[{
Cell[608874, 24365, 190, 5, 26, "Input",
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
Cell[609067, 24372, 271, 8, 28, "Output",
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}]
}, Open  ]],
Cell[CellGroupData[{
Cell[609375, 24385, 156, 3, 26, "Input",
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
Cell[609534, 24390, 269, 8, 28, "Output",
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}]
}, Open  ]],
Cell[CellGroupData[{
Cell[609840, 24403, 239, 7, 26, "Input",
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
Cell[610082, 24412, 354, 11, 28, "Output",
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}]
}, Open  ]],
Cell[CellGroupData[{
Cell[610473, 24428, 184, 4, 26, "Input",
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
Cell[610660, 24434, 607, 23, 28, "Output",
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}]
}, Open  ]]
}, Open  ]],
Cell[611294, 24461, 51, 1, 39, "ObjectName",
 CellTags->"Expand2"],
Cell[CellGroupData[{
Cell[611370, 24466, 58, 1, 34, "Subsubsection",
 CellTags->"Expand2"],
Cell[611431, 24469, 164, 4, 24, "Text",
 CellTags->"Expand2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[611632, 24478, 55, 1, 34, "Subsubsection",
 CellTags->"Expand2"],
Cell[CellGroupData[{
Cell[611712, 24483, 579, 21, 26, "Input",
 CellTags->"Expand2"],
Cell[612294, 24506, 618, 20, 26, "Output",
 CellTags->"Expand2"]
}, Open  ]]
}, Open  ]],
Cell[612939, 24530, 53, 1, 39, "ObjectName",
 CellTags->"Explicit"],
Cell[CellGroupData[{
Cell[613017, 24535, 59, 1, 34, "Subsubsection",
 CellTags->"Explicit"],
Cell[613079, 24538, 401, 9, 41, "Text",
 CellTags->"Explicit"],
Cell[613483, 24549, 331, 14, 24, "Text",
 CellTags->"Explicit"],
Cell[CellGroupData[{
Cell[613839, 24567, 88, 2, 26, "Input"],
Cell[613930, 24571, 389, 11, 26, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[614368, 24588, 56, 1, 34, "Subsubsection",
 CellTags->"Explicit"],
Cell[CellGroupData[{
Cell[614449, 24593, 134, 3, 26, "Input"],
Cell[614586, 24598, 225, 7, 29, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[614848, 24610, 86, 2, 26, "Input"],
Cell[614937, 24614, 283, 9, 49, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[615257, 24628, 233, 6, 26, "Input"],
Cell[615493, 24636, 820, 29, 61, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[616350, 24670, 230, 6, 26, "Input",
 CellTags->"Explicit"],
Cell[616583, 24678, 982, 43, 28, "Output",
 CellTags->"Explicit"]
}, Open  ]],
Cell[CellGroupData[{
Cell[617602, 24726, 109, 3, 26, "Input",
 CellTags->"Explicit"],
Cell[617714, 24731, 2013, 80, 28, "Output",
 CellTags->"Explicit"]
}, Open  ]],
Cell[CellGroupData[{
Cell[619764, 24816, 184, 5, 26, "Input",
 CellTags->"Explicit"],
Cell[619951, 24823, 936, 37, 44, "Output",
 CellTags->"Explicit"]
}, Open  ]],
Cell[CellGroupData[{
Cell[620924, 24865, 109, 3, 26, "Input",
 CellTags->"Explicit"],
Cell[621036, 24870, 2686, 103, 44, "Output",
 CellTags->"Explicit"]
}, Open  ]],
Cell[CellGroupData[{
Cell[623759, 24978, 155, 4, 26, "Input",
 CellTags->"Explicit"],
Cell[623917, 24984, 204, 6, 30, "Output",
 CellTags->"Explicit"]
}, Open  ]],
Cell[CellGroupData[{
Cell[624158, 24995, 109, 3, 26, "Input",
 CellTags->"Explicit"],
Cell[624270, 25000, 2106, 70, 152, "Output",
 CellTags->"Explicit"]
}, Open  ]]
}, Open  ]],
Cell[626403, 25074, 77, 1, 39, "ObjectName",
 CellTags->"ExplicitLorentzIndex"],
Cell[CellGroupData[{
Cell[626505, 25079, 71, 1, 34, "Subsubsection",
 CellTags->"ExplicitLorentzIndex"],
Cell[626579, 25082, 141, 4, 24, "Text",
 CellTags->"ExplicitLorentzIndex"],
Cell[626723, 25088, 295, 13, 24, "Text",
 CellTags->"ExplicitLorentzIndex"]
}, Open  ]],
Cell[CellGroupData[{
Cell[627055, 25106, 68, 1, 34, "Subsubsection",
 CellTags->"ExplicitLorentzIndex"],
Cell[CellGroupData[{
Cell[627148, 25111, 224, 6, 26, "Input",
 CellTags->"ExplicitLorentzIndex"],
Cell[627375, 25119, 241, 8, 26, "Output",
 CellTags->"ExplicitLorentzIndex"]
}, Open  ]],
Cell[CellGroupData[{
Cell[627653, 25132, 121, 3, 26, "Input",
 CellTags->"ExplicitLorentzIndex"],
Cell[627777, 25137, 289, 7, 38, "Output",
 CellTags->"ExplicitLorentzIndex"]
}, Open  ]]
}, Open  ]],
Cell[628093, 25148, 69, 1, 39, "ObjectName",
 CellTags->"ExplicitPartialD"],
Cell[CellGroupData[{
Cell[628187, 25153, 67, 1, 34, "Subsubsection",
 CellTags->"ExplicitPartialD"],
Cell[628257, 25156, 162, 4, 24, "Text",
 CellTags->"ExplicitPartialD"],
Cell[628422, 25162, 476, 18, 24, "Text",
 CellTags->"ExplicitPartialD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[628935, 25185, 64, 1, 34, "Subsubsection",
 CellTags->"ExplicitPartialD"],
Cell[CellGroupData[{
Cell[629024, 25190, 183, 4, 26, "Input",
 CellTags->"ExplicitPartialD"],
Cell[629210, 25196, 567, 20, 44, "Output",
 CellTags->"ExplicitPartialD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[629814, 25221, 184, 4, 26, "Input",
 CellTags->"ExplicitPartialD"],
Cell[630001, 25227, 474, 17, 33, "Output",
 CellTags->"ExplicitPartialD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[630512, 25249, 186, 4, 26, "Input",
 CellTags->"ExplicitPartialD"],
Cell[630701, 25255, 502, 16, 44, "Output",
 CellTags->"ExplicitPartialD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[631240, 25276, 183, 6, 26, "Input",
 CellTags->"ExplicitPartialD"],
Cell[631426, 25284, 312, 9, 34, "Output",
 CellTags->"ExplicitPartialD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[631775, 25298, 125, 3, 26, "Input",
 CellTags->"ExplicitPartialD"],
Cell[631903, 25303, 482, 15, 38, "Output",
 CellTags->"ExplicitPartialD"]
}, Open  ]]
}, Open  ]],
Cell[632412, 25322, 69, 1, 39, "ObjectName",
 CellTags->"ExplicitSUNIndex"],
Cell[CellGroupData[{
Cell[632506, 25327, 67, 1, 34, "Subsubsection",
 CellTags->"ExplicitSUNIndex"],
Cell[632576, 25330, 178, 6, 24, "Text",
 CellTags->"ExplicitSUNIndex"],
Cell[632757, 25338, 484, 23, 24, "Text",
 CellTags->"ExplicitSUNIndex"]
}, Open  ]],
Cell[CellGroupData[{
Cell[633278, 25366, 64, 1, 34, "Subsubsection",
 CellTags->"ExplicitSUNIndex"],
Cell[CellGroupData[{
Cell[633367, 25371, 125, 3, 26, "Input",
 CellTags->"ExplicitSUNIndex"],
Cell[633495, 25376, 154, 4, 26, "Output",
 CellTags->"ExplicitSUNIndex"]
}, Open  ]],
Cell[CellGroupData[{
Cell[633686, 25385, 176, 6, 26, "Input"],
Cell[633865, 25393, 246, 6, 38, "Output"]
}, Open  ]]
}, Open  ]],
Cell[634138, 25403, 59, 1, 39, "ObjectName",
 CellTags->"ExtraFactor"],
Cell[CellGroupData[{
Cell[634222, 25408, 62, 1, 34, "Subsubsection",
 CellTags->"ExtraFactor"],
Cell[634287, 25411, 199, 4, 24, "Text",
 CellTags->"ExtraFactor"],
Cell[634489, 25417, 194, 8, 24, "Text",
 CellTags->"ExtraFactor"]
}, Open  ]],
Cell[634698, 25428, 65, 1, 39, "ObjectName",
 CellTags->"ExtraVariables"],
Cell[CellGroupData[{
Cell[634788, 25433, 65, 1, 34, "Subsubsection",
 CellTags->"ExtraVariables"],
Cell[634856, 25436, 213, 5, 41, "Text",
 CellTags->"ExtraVariables"],
Cell[635072, 25443, 298, 14, 24, "Text",
 CellTags->"ExtraVariables"]
}, Open  ]],
Cell[635385, 25460, 57, 1, 39, "ObjectName",
 CellTags->"FactorFull"],
Cell[CellGroupData[{
Cell[635467, 25465, 61, 1, 36, "Subsubsection",
 CellTags->"FactorFull"],
Cell[635531, 25468, 509, 24, 25, "Text",
 CellTags->"FactorFull"],
Cell[636043, 25494, 178, 9, 25, "Text",
 CellTags->"FactorFull"]
}, Open  ]],
Cell[636236, 25506, 55, 1, 41, "ObjectName",
 CellTags->"Factoring"],
Cell[CellGroupData[{
Cell[636316, 25511, 60, 1, 34, "Subsubsection",
 CellTags->"Factoring"],
Cell[636379, 25514, 235, 5, 41, "Text",
 CellTags->"Factoring"],
Cell[636617, 25521, 382, 19, 24, "Text",
 CellTags->"Factoring"]
}, Open  ]],
Cell[637014, 25543, 55, 1, 39, "ObjectName",
 CellTags->"Factorout"],
Cell[CellGroupData[{
Cell[637094, 25548, 60, 1, 34, "Subsubsection",
 CellTags->"Factorout"],
Cell[637157, 25551, 74, 1, 24, "Text",
 CellTags->"Factorout"],
Cell[637234, 25554, 174, 9, 24, "Text",
 CellTags->"Factorout"]
}, Open  ]],
Cell[637423, 25566, 57, 1, 39, "ObjectName",
 CellTags->"FactorTime"],
Cell[CellGroupData[{
Cell[637505, 25571, 61, 1, 34, "Subsubsection",
 CellTags->"FactorTime"],
Cell[637569, 25574, 166, 4, 24, "Text",
 CellTags->"FactorTime"],
Cell[637738, 25580, 178, 9, 24, "Text",
 CellTags->"FactorTime"]
}, Open  ]],
Cell[637931, 25592, 51, 1, 39, "ObjectName",
 CellTags->"Factor1"],
Cell[CellGroupData[{
Cell[638007, 25597, 58, 1, 34, "Subsubsection",
 CellTags->"Factor1"],
Cell[638068, 25600, 144, 4, 24, "Text",
 CellTags->"Factor1"],
Cell[638215, 25606, 175, 9, 24, "Text",
 CellTags->"Factor1"]
}, Open  ]],
Cell[CellGroupData[{
Cell[638427, 25620, 55, 1, 34, "Subsubsection",
 CellTags->"Factor1"],
Cell[CellGroupData[{
Cell[638507, 25625, 213, 8, 26, "Input",
 CellTags->"Factor1"],
Cell[638723, 25635, 268, 9, 26, "Output",
 CellTags->"Factor1"]
}, Open  ]],
Cell[CellGroupData[{
Cell[639028, 25649, 222, 7, 26, "Input",
 CellTags->"Factor1"],
Cell[639253, 25658, 477, 17, 26, "Output",
 CellTags->"Factor1"]
}, Open  ]],
Cell[CellGroupData[{
Cell[639767, 25680, 251, 9, 26, "Input",
 CellTags->"Factor1"],
Cell[640021, 25691, 218, 7, 26, "Output",
 CellTags->"Factor1"]
}, Open  ]],
Cell[CellGroupData[{
Cell[640276, 25703, 107, 3, 26, "Input",
 CellTags->"Factor1"],
Cell[640386, 25708, 266, 9, 26, "Output",
 CellTags->"Factor1"]
}, Open  ]],
Cell[CellGroupData[{
Cell[640689, 25722, 108, 3, 26, "Input",
 CellTags->"Factor1"],
Cell[640800, 25727, 218, 7, 26, "Output",
 CellTags->"Factor1"]
}, Open  ]],
Cell[641033, 25737, 141, 4, 26, "Input",
 CellTags->"Factor1"]
}, Open  ]],
Cell[641189, 25744, 51, 1, 39, "ObjectName",
 CellTags->"Factor2"],
Cell[CellGroupData[{
Cell[641265, 25749, 58, 1, 36, "Subsubsection",
 CellTags->"Factor2"],
Cell[641326, 25752, 398, 7, 58, "Text",
 CellTags->"Factor2"],
Cell[CellGroupData[{
Cell[641749, 25763, 113, 3, 26, "Input",
 CellTags->"Factor2"],
Cell[641865, 25768, 270, 8, 26, "Output",
 CellTags->"Factor2"]
}, Open  ]],
Cell[642150, 25779, 178, 9, 25, "Text",
 CellTags->"Factor2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[642365, 25793, 55, 1, 34, "Subsubsection",
 CellTags->"Factor2"],
Cell[CellGroupData[{
Cell[642445, 25798, 213, 8, 26, "Input",
 CellTags->"Factor2"],
Cell[642661, 25808, 268, 9, 26, "Output",
 CellTags->"Factor2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[642966, 25822, 222, 7, 26, "Input",
 CellTags->"Factor2"],
Cell[643191, 25831, 477, 17, 26, "Output",
 CellTags->"Factor2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[643705, 25853, 251, 9, 26, "Input",
 CellTags->"Factor2"],
Cell[643959, 25864, 218, 7, 26, "Output",
 CellTags->"Factor2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[644214, 25876, 107, 3, 26, "Input",
 CellTags->"Factor2"],
Cell[644324, 25881, 266, 9, 26, "Output",
 CellTags->"Factor2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[644627, 25895, 108, 3, 26, "Input",
 CellTags->"Factor2"],
Cell[644738, 25900, 220, 7, 26, "Output",
 CellTags->"Factor2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[644995, 25912, 173, 5, 26, "Input",
 CellTags->"Factor2"],
Cell[645171, 25919, 266, 9, 26, "Output",
 CellTags->"Factor2"]
}, Open  ]],
Cell[645452, 25931, 141, 4, 26, "Input",
 CellTags->"Factor2"]
}, Open  ]],
Cell[645608, 25938, 43, 1, 70, "ObjectName",
 CellTags->"FAD"],
Cell[CellGroupData[{
Cell[645676, 25943, 54, 1, 70, "Subsubsection",
 CellTags->"FAD"],
Cell[645733, 25946, 335, 6, 70, "Text",
 CellTags->"FAD"],
Cell[646071, 25954, 802, 34, 70, "Text",
 CellTags->"FAD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[646910, 25993, 51, 1, 34, "Subsubsection",
 CellTags->"FAD"],
Cell[CellGroupData[{
Cell[646986, 25998, 146, 5, 26, "Input",
 CellTags->"FAD"],
Cell[647135, 26005, 742, 21, 51, "Output",
 CellTags->"FAD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[647914, 26031, 197, 7, 26, "Input",
 CellTags->"FAD"],
Cell[648114, 26040, 860, 23, 51, "Output",
 CellTags->"FAD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[649011, 26068, 245, 9, 26, "Input",
 CellTags->"FAD"],
Cell[649259, 26079, 201, 6, 38, "Output",
 CellTags->"FAD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[649497, 26090, 215, 8, 26, "Input",
 CellTags->"FAD"],
Cell[649715, 26100, 567, 16, 58, "Output",
 CellTags->"FAD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[650319, 26121, 280, 9, 26, "Input",
 CellTags->"FAD"],
Cell[650602, 26132, 571, 17, 58, "Output",
 CellTags->"FAD"]
}, Open  ]]
}, Open  ]],
Cell[651200, 26153, 41, 1, 39, "ObjectName",
 CellTags->"FC"],
Cell[CellGroupData[{
Cell[651266, 26158, 53, 1, 34, "Subsubsection",
 CellTags->"FC"],
Cell[651322, 26161, 117, 3, 24, "Text",
 CellTags->"FC"],
Cell[651442, 26166, 545, 24, 24, "Text",
 CellTags->"FC"]
}, Open  ]],
Cell[CellGroupData[{
Cell[652024, 26195, 50, 1, 34, "Subsubsection",
 CellTags->"FC"],
Cell[652077, 26198, 70, 2, 26, "Input",
 CellTags->"FC"],
Cell[CellGroupData[{
Cell[652172, 26204, 227, 7, 26, "Input",
 CellTags->"FC"],
Cell[652402, 26213, 141, 3, 25, "Output",
 CellTags->"FC"]
}, Open  ]],
Cell[652558, 26219, 70, 2, 26, "Input",
 CellTags->"FC"],
Cell[CellGroupData[{
Cell[652653, 26225, 227, 7, 26, "Input",
 CellTags->"FC"],
Cell[652883, 26234, 347, 12, 29, "Output",
 CellTags->"FC"]
}, Open  ]]
}, Open  ]],
Cell[653257, 26250, 43, 1, 39, "ObjectName",
 CellTags->"FCE"],
Cell[CellGroupData[{
Cell[653325, 26255, 54, 1, 34, "Subsubsection",
 CellTags->"FCE"],
Cell[653382, 26258, 126, 4, 24, "Text",
 CellTags->"FCE"],
Cell[653511, 26264, 72, 1, 24, "Text",
 CellTags->"FCE"],
Cell[653586, 26267, 427, 19, 24, "Text",
 CellTags->"FCE"]
}, Open  ]],
Cell[CellGroupData[{
Cell[654050, 26291, 51, 1, 34, "Subsubsection",
 CellTags->"FCE"],
Cell[CellGroupData[{
Cell[654126, 26296, 262, 8, 26, "Input",
 CellTags->"FCE"],
Cell[654391, 26306, 267, 9, 29, "Output",
 CellTags->"FCE"]
}, Open  ]],
Cell[CellGroupData[{
Cell[654695, 26320, 104, 3, 26, "Input",
 CellTags->"FCE"],
Cell[654802, 26325, 232, 7, 38, "Output",
 CellTags->"FCE"]
}, Open  ]],
Cell[CellGroupData[{
Cell[655071, 26337, 437, 13, 26, "Input",
 CellTags->"FCE"],
Cell[655511, 26352, 589, 19, 29, "Output",
 CellTags->"FCE"]
}, Open  ]],
Cell[CellGroupData[{
Cell[656137, 26376, 104, 3, 26, "Input",
 CellTags->"FCE"],
Cell[656244, 26381, 494, 14, 38, "Output",
 CellTags->"FCE"]
}, Open  ]],
Cell[CellGroupData[{
Cell[656775, 26400, 95, 3, 26, "Input",
 CellTags->"FCE"],
Cell[656873, 26405, 904, 35, 29, "Output",
 CellTags->"FCE"]
}, Open  ]],
Cell[CellGroupData[{
Cell[657814, 26445, 104, 3, 26, "Input",
 CellTags->"FCE"],
Cell[657921, 26450, 935, 25, 77, "Output",
 CellTags->"FCE"]
}, Open  ]],
Cell[CellGroupData[{
Cell[658893, 26480, 134, 4, 26, "Input",
 CellTags->"FCE"],
Cell[659030, 26486, 494, 14, 38, "Output",
 CellTags->"FCE"]
}, Open  ]]
}, Open  ]],
Cell[659551, 26504, 43, 1, 39, "ObjectName",
 CellTags->"FCI"],
Cell[CellGroupData[{
Cell[659619, 26509, 54, 1, 34, "Subsubsection",
 CellTags->"FCI"],
Cell[659676, 26512, 159, 4, 24, "Text",
 CellTags->"FCI"],
Cell[659838, 26518, 427, 19, 24, "Text",
 CellTags->"FCI"]
}, Open  ]],
Cell[CellGroupData[{
Cell[660302, 26542, 51, 1, 34, "Subsubsection",
 CellTags->"FCI"],
Cell[CellGroupData[{
Cell[660378, 26547, 437, 13, 26, "Input",
 CellTags->"FCI"],
Cell[660818, 26562, 591, 19, 29, "Output",
 CellTags->"FCI"]
}, Open  ]],
Cell[CellGroupData[{
Cell[661446, 26586, 104, 3, 26, "Input",
 CellTags->"FCI"],
Cell[661553, 26591, 493, 14, 38, "Output",
 CellTags->"FCI"]
}, Open  ]],
Cell[CellGroupData[{
Cell[662083, 26610, 95, 3, 26, "Input",
 CellTags->"FCI"],
Cell[662181, 26615, 906, 35, 29, "Output",
 CellTags->"FCI"]
}, Open  ]],
Cell[CellGroupData[{
Cell[663124, 26655, 104, 3, 26, "Input",
 CellTags->"FCI"],
Cell[663231, 26660, 935, 25, 77, "Output",
 CellTags->"FCI"]
}, Open  ]],
Cell[CellGroupData[{
Cell[664203, 26690, 134, 4, 26, "Input",
 CellTags->"FCI"],
Cell[664340, 26696, 493, 14, 38, "Output",
 CellTags->"FCI"]
}, Open  ]]
}, Open  ]],
Cell[664860, 26714, 57, 1, 39, "ObjectName",
 CellTags->"FCIntegral"],
Cell[CellGroupData[{
Cell[664942, 26719, 61, 1, 34, "Subsubsection",
 CellTags->"FCIntegral"],
Cell[665006, 26722, 209, 5, 41, "Text",
 CellTags->"FCIntegral"],
Cell[665218, 26729, 360, 14, 24, "Text",
 CellTags->"FCIntegral"]
}, Open  ]],
Cell[665593, 26746, 59, 1, 39, "ObjectName",
 CellTags->"FCIntegrate"],
Cell[CellGroupData[{
Cell[665677, 26751, 62, 1, 34, "Subsubsection",
 CellTags->"FCIntegrate"],
Cell[665742, 26754, 285, 5, 41, "Text",
 CellTags->"FCIntegrate"],
Cell[666030, 26761, 194, 9, 24, "Text",
 CellTags->"FCIntegrate"]
}, Open  ]],
Cell[666239, 26773, 61, 1, 39, "ObjectName",
 CellTags->"FCNIntegrate"],
Cell[CellGroupData[{
Cell[666325, 26778, 63, 1, 34, "Subsubsection",
 CellTags->"FCNIntegrate"],
Cell[666391, 26781, 418, 7, 58, "Text",
 CellTags->"FCNIntegrate"],
Cell[666812, 26790, 192, 9, 24, "Text",
 CellTags->"FCNIntegrate"]
}, Open  ]],
Cell[667019, 26802, 50, 1, 39, "ObjectName",
 CellTags->"FC2RHI"],
Cell[CellGroupData[{
Cell[667094, 26807, 57, 1, 34, "Subsubsection",
 CellTags->"FC2RHI"],
Cell[667154, 26810, 306, 6, 41, "Text",
 CellTags->"FC2RHI"],
Cell[CellGroupData[{
Cell[667485, 26820, 111, 3, 26, "Input",
 CellTags->"FC2RHI"],
Cell[667599, 26825, 319, 9, 26, "Output",
 CellTags->"FC2RHI"]
}, Open  ]],
Cell[CellGroupData[{
Cell[667955, 26839, 349, 12, 26, "Input"],
Cell[668307, 26853, 548, 19, 50, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[668892, 26877, 119, 3, 26, "Input"],
Cell[669014, 26882, 515, 21, 29, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[669566, 26908, 83, 2, 26, "Input"],
Cell[669652, 26912, 138, 2, 38, "Output"]
}, Open  ]],
Cell[669805, 26917, 162, 9, 24, "Text",
 CellTags->"FC2RHI"]
}, Open  ]],
Cell[CellGroupData[{
Cell[670004, 26931, 54, 1, 34, "Subsubsection",
 CellTags->"FC2RHI"],
Cell[670061, 26934, 72, 2, 26, "Input",
 CellTags->"FC2RHI"]
}, Open  ]],
Cell[670148, 26939, 50, 1, 39, "ObjectName",
 CellTags->"FC2TLI"],
Cell[CellGroupData[{
Cell[670223, 26944, 57, 1, 34, "Subsubsection",
 CellTags->"FC2TLI"],
Cell[670283, 26947, 256, 5, 41, "Text",
 CellTags->"FC2TLI"],
Cell[CellGroupData[{
Cell[670564, 26956, 111, 3, 26, "Input",
 CellTags->"FC2TLI"],
Cell[670678, 26961, 319, 9, 26, "Output",
 CellTags->"FC2TLI"]
}, Open  ]],
Cell[CellGroupData[{
Cell[671034, 26975, 71, 2, 26, "Input"],
Cell[671108, 26979, 365, 6, 117, "Print",
 CellTags->"Info3500973829-4982097"]
}, Open  ]],
Cell[CellGroupData[{
Cell[671510, 26990, 365, 12, 26, "Input"],
Cell[671878, 27004, 1527, 43, 51, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[673442, 27052, 119, 3, 26, "Input"],
Cell[673564, 27057, 741, 30, 29, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[674342, 27092, 83, 2, 26, "Input"],
Cell[674428, 27096, 144, 2, 38, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[674609, 27103, 84, 2, 26, "Input"],
Cell[674696, 27107, 630, 22, 51, "Output"]
}, Open  ]],
Cell[675341, 27132, 162, 9, 24, "Text",
 CellTags->"FC2TLI"]
}, Open  ]],
Cell[CellGroupData[{
Cell[675540, 27146, 54, 1, 34, "Subsubsection",
 CellTags->"FC2TLI"],
Cell[675597, 27149, 72, 2, 26, "Input",
 CellTags->"FC2TLI"]
}, Open  ]],
Cell[675684, 27154, 43, 1, 39, "ObjectName",
 CellTags->"FDS"],
Cell[CellGroupData[{
Cell[675752, 27159, 54, 1, 34, "Subsubsection",
 CellTags->"FDS"],
Cell[675809, 27162, 82, 1, 24, "Text",
 CellTags->"FDS"],
Cell[675894, 27165, 228, 9, 24, "Text",
 CellTags->"FDS"]
}, Open  ]],
Cell[676137, 27177, 65, 1, 39, "ObjectName",
 CellTags->"FermionSpinSum"],
Cell[CellGroupData[{
Cell[676227, 27182, 65, 1, 34, "Subsubsection",
 CellTags->"FermionSpinSum"],
Cell[676295, 27185, 118, 3, 24, "Text",
 CellTags->"FermionSpinSum"],
Cell[CellGroupData[{
Cell[676438, 27192, 127, 3, 26, "Input",
 CellTags->"FermionSpinSum"],
Cell[676568, 27197, 356, 10, 26, "Output",
 CellTags->"FermionSpinSum"]
}, Open  ]],
Cell[676939, 27210, 521, 24, 25, "Text",
 CellTags->"FermionSpinSum"]
}, Open  ]],
Cell[CellGroupData[{
Cell[677497, 27239, 62, 1, 36, "Subsubsection",
 CellTags->"FermionSpinSum"],
Cell[677562, 27242, 261, 12, 24, "Text",
 CellTags->"FermionSpinSum"],
Cell[CellGroupData[{
Cell[677848, 27258, 296, 9, 26, "Input",
 CellTags->"FermionSpinSum"],
Cell[678147, 27269, 395, 14, 28, "Output",
 CellTags->"FermionSpinSum"]
}, Open  ]],
Cell[CellGroupData[{
Cell[678579, 27288, 143, 4, 26, "Input",
 CellTags->"FermionSpinSum"],
Cell[678725, 27294, 372, 13, 28, "Output",
 CellTags->"FermionSpinSum"]
}, Open  ]],
Cell[CellGroupData[{
Cell[679134, 27312, 143, 4, 26, "Input",
 CellTags->"FermionSpinSum"],
Cell[679280, 27318, 175, 5, 26, "Output",
 CellTags->"FermionSpinSum"]
}, Open  ]],
Cell[CellGroupData[{
Cell[679492, 27328, 296, 9, 26, "Input",
 CellTags->"FermionSpinSum"],
Cell[679791, 27339, 395, 14, 28, "Output",
 CellTags->"FermionSpinSum"]
}, Open  ]],
Cell[CellGroupData[{
Cell[680223, 27358, 143, 4, 26, "Input",
 CellTags->"FermionSpinSum"],
Cell[680369, 27364, 372, 13, 28, "Output",
 CellTags->"FermionSpinSum"]
}, Open  ]],
Cell[CellGroupData[{
Cell[680778, 27382, 143, 4, 26, "Input",
 CellTags->"FermionSpinSum"],
Cell[680924, 27388, 196, 6, 26, "Output",
 CellTags->"FermionSpinSum"]
}, Open  ]],
Cell[CellGroupData[{
Cell[681157, 27399, 334, 10, 26, "Input",
 CellTags->"FermionSpinSum"],
Cell[681494, 27411, 550, 18, 28, "Output",
 CellTags->"FermionSpinSum"]
}, Open  ]],
Cell[682059, 27432, 164, 4, 24, "Text",
 CellTags->"FermionSpinSum"],
Cell[CellGroupData[{
Cell[682248, 27440, 147, 4, 26, "Input",
 CellTags->"FermionSpinSum"],
Cell[682398, 27446, 582, 19, 28, "Output",
 CellTags->"FermionSpinSum"]
}, Open  ]],
Cell[CellGroupData[{
Cell[683017, 27470, 145, 4, 26, "Input",
 CellTags->"FermionSpinSum"],
Cell[683165, 27476, 1116, 37, 29, "Output",
 CellTags->"FermionSpinSum"]
}, Open  ]],
Cell[CellGroupData[{
Cell[684318, 27518, 148, 4, 26, "Input",
 CellTags->"FermionSpinSum"],
Cell[684469, 27524, 879, 33, 29, "Output",
 CellTags->"FermionSpinSum"]
}, Open  ]],
Cell[685363, 27560, 141, 4, 26, "Input",
 CellTags->"FermionSpinSum"]
}, Open  ]],
Cell[685519, 27567, 51, 1, 41, "ObjectName",
 CellTags->"FeynAmp"],
Cell[CellGroupData[{
Cell[685595, 27572, 58, 1, 36, "Subsubsection",
 CellTags->"FeynAmp"],
Cell[685656, 27575, 243, 5, 43, "Text",
 CellTags->"FeynAmp"],
Cell[685902, 27582, 165, 4, 25, "Text",
 CellTags->"FeynAmp"],
Cell[686070, 27588, 181, 9, 25, "Text",
 CellTags->"FeynAmp"]
}, Open  ]],
Cell[CellGroupData[{
Cell[686288, 27602, 55, 1, 36, "Subsubsection",
 CellTags->"FeynAmp"],
Cell[686346, 27605, 125, 4, 24, "Text",
 CellTags->"FeynAmp"],
Cell[CellGroupData[{
Cell[686496, 27613, 846, 22, 46, "Input",
 CellTags->"FeynAmp"],
Cell[687345, 27637, 2630, 103, 42, "Output",
 CellTags->"FeynAmp"]
}, Open  ]],
Cell[689990, 27743, 73, 1, 24, "Text",
 CellTags->"FeynAmp"],
Cell[CellGroupData[{
Cell[690088, 27748, 198, 6, 26, "Input",
 CellTags->"FeynAmp"],
Cell[690289, 27756, 456, 15, 42, "Output",
 CellTags->"FeynAmp"]
}, Open  ]]
}, Open  ]],
Cell[690772, 27775, 73, 1, 41, "ObjectName",
 CellTags->"FeynAmpDenominator"],
Cell[CellGroupData[{
Cell[690870, 27780, 69, 1, 36, "Subsubsection",
 CellTags->"FeynAmpDenominator"],
Cell[690942, 27783, 256, 5, 43, "Text",
 CellTags->"FeynAmpDenominator"],
Cell[691201, 27790, 338, 14, 25, "Text",
 CellTags->"FeynAmpDenominator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[691576, 27809, 66, 1, 34, "Subsubsection",
 CellTags->"FeynAmpDenominator"],
Cell[CellGroupData[{
Cell[691667, 27814, 201, 5, 26, "Input",
 CellTags->"FeynAmpDenominator"],
Cell[691871, 27821, 253, 8, 50, "Output",
 CellTags->"FeynAmpDenominator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[692161, 27834, 326, 9, 26, "Input",
 CellTags->"FeynAmpDenominator"],
Cell[692490, 27845, 472, 16, 51, "Output",
 CellTags->"FeynAmpDenominator"]
}, Open  ]],
Cell[692977, 27864, 247, 7, 26, "Input",
 CellTags->"FeynAmpDenominator"],
Cell[CellGroupData[{
Cell[693249, 27875, 149, 4, 26, "Input",
 CellTags->"FeynAmpDenominator"],
Cell[693401, 27881, 321, 8, 38, "Output",
 CellTags->"FeynAmpDenominator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[693759, 27894, 149, 4, 26, "Input",
 CellTags->"FeynAmpDenominator"],
Cell[693911, 27900, 220, 6, 38, "Output",
 CellTags->"FeynAmpDenominator"]
}, Open  ]],
Cell[694146, 27909, 134, 4, 26, "Input",
 CellTags->"FeynAmpDenominator"]
}, Open  ]],
Cell[694295, 27916, 87, 1, 41, "ObjectName",
 CellTags->"FeynAmpDenominatorCombine"],
Cell[CellGroupData[{
Cell[694407, 27921, 76, 1, 36, "Subsubsection",
 CellTags->"FeynAmpDenominatorCombine"],
Cell[694486, 27924, 228, 5, 43, "Text",
 CellTags->"FeynAmpDenominatorCombine"],
Cell[694717, 27931, 241, 9, 25, "Text",
 CellTags->"FeynAmpDenominatorCombine"]
}, Open  ]],
Cell[CellGroupData[{
Cell[694995, 27945, 73, 1, 34, "Subsubsection",
 CellTags->"FeynAmpDenominatorCombine"],
Cell[CellGroupData[{
Cell[695093, 27950, 234, 7, 26, "Input",
 CellTags->"FeynAmpDenominatorCombine"],
Cell[695330, 27959, 807, 24, 50, "Output",
 CellTags->"FeynAmpDenominatorCombine"]
}, Open  ]],
Cell[CellGroupData[{
Cell[696174, 27988, 208, 6, 26, "Input",
 CellTags->"FeynAmpDenominatorCombine"],
Cell[696385, 27996, 245, 7, 38, "Output",
 CellTags->"FeynAmpDenominatorCombine"]
}, Open  ]],
Cell[CellGroupData[{
Cell[696667, 28008, 206, 6, 26, "Input",
 CellTags->"FeynAmpDenominatorCombine"],
Cell[696876, 28016, 275, 8, 38, "Output",
 CellTags->"FeynAmpDenominatorCombine"]
}, Open  ]]
}, Open  ]],
Cell[697178, 28028, 89, 1, 41, "ObjectName",
 CellTags->"FeynAmpDenominatorSimplify"],
Cell[CellGroupData[{
Cell[697292, 28033, 77, 1, 36, "Subsubsection",
 CellTags->"FeynAmpDenominatorSimplify"],
Cell[697372, 28036, 398, 7, 61, "Text",
 CellTags->"FeynAmpDenominatorSimplify"],
Cell[697773, 28045, 85, 1, 24, "Text",
 CellTags->"FeynAmpDenominatorSimplify"],
Cell[CellGroupData[{
Cell[697883, 28050, 95, 2, 26, "Input",
 CellTags->"FeynAmpDenominatorSimplify"],
Cell[697981, 28054, 191, 4, 26, "Output",
 CellTags->"FeynAmpDenominatorSimplify"]
}, Open  ]],
Cell[698187, 28061, 218, 9, 25, "Text",
 CellTags->"FeynAmpDenominatorSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[698442, 28075, 74, 1, 36, "Subsubsection",
 CellTags->"FeynAmpDenominatorSimplify"],
Cell[698519, 28078, 432, 14, 28, "Text",
 CellTags->"FeynAmpDenominatorSimplify"],
Cell[CellGroupData[{
Cell[698976, 28096, 280, 8, 26, "Input",
 CellTags->"FeynAmpDenominatorSimplify"],
Cell[699259, 28106, 163, 4, 26, "Output",
 CellTags->"FeynAmpDenominatorSimplify"]
}, Open  ]],
Cell[699437, 28113, 468, 18, 27, "Text",
 CellTags->"FeynAmpDenominatorSimplify"],
Cell[CellGroupData[{
Cell[699930, 28135, 328, 10, 26, "Input",
 CellTags->"FeynAmpDenominatorSimplify"],
Cell[700261, 28147, 378, 12, 50, "Output",
 CellTags->"FeynAmpDenominatorSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[700676, 28164, 260, 7, 26, "Input",
 CellTags->"FeynAmpDenominatorSimplify"],
Cell[700939, 28173, 350, 11, 50, "Output",
 CellTags->"FeynAmpDenominatorSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[701326, 28189, 339, 10, 26, "Input",
 CellTags->"FeynAmpDenominatorSimplify"],
Cell[701668, 28201, 404, 13, 50, "Output",
 CellTags->"FeynAmpDenominatorSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[702109, 28219, 449, 14, 26, "Input",
 CellTags->"FeynAmpDenominatorSimplify"],
Cell[702561, 28235, 1135, 41, 51, "Output",
 CellTags->"FeynAmpDenominatorSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[703733, 28281, 215, 6, 26, "Input",
 CellTags->"FeynAmpDenominatorSimplify"],
Cell[703951, 28289, 2003, 71, 52, "Output",
 CellTags->"FeynAmpDenominatorSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[705991, 28365, 157, 4, 26, "Input",
 CellTags->"FeynAmpDenominatorSimplify"],
Cell[706151, 28371, 1066, 39, 51, "Output",
 CellTags->"FeynAmpDenominatorSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[707254, 28415, 373, 13, 26, "Input",
 CellTags->"FeynAmpDenominatorSimplify"],
Cell[707630, 28430, 1403, 51, 52, "Output",
 CellTags->"FeynAmpDenominatorSimplify"]
}, Open  ]],
Cell[709048, 28484, 147, 4, 26, "Input",
 CellTags->"FeynAmpDenominatorSimplify"]
}, Open  ]],
Cell[709210, 28491, 83, 1, 41, "ObjectName",
 CellTags->"FeynAmpDenominatorSplit"],
Cell[CellGroupData[{
Cell[709318, 28496, 74, 1, 34, "Subsubsection",
 CellTags->"FeynAmpDenominatorSplit"],
Cell[709395, 28499, 357, 7, 58, "Text",
 CellTags->"FeynAmpDenominatorSplit"],
Cell[709755, 28508, 245, 9, 24, "Text",
 CellTags->"FeynAmpDenominatorSplit"]
}, Open  ]],
Cell[CellGroupData[{
Cell[710037, 28522, 71, 1, 34, "Subsubsection",
 CellTags->"FeynAmpDenominatorSplit"],
Cell[CellGroupData[{
Cell[710133, 28527, 308, 9, 26, "Input",
 CellTags->"FeynAmpDenominatorSplit"],
Cell[710444, 28538, 533, 17, 50, "Output",
 CellTags->"FeynAmpDenominatorSplit"]
}, Open  ]],
Cell[CellGroupData[{
Cell[711014, 28560, 116, 3, 26, "Input",
 CellTags->"FeynAmpDenominatorSplit"],
Cell[711133, 28565, 178, 4, 26, "Output",
 CellTags->"FeynAmpDenominatorSplit"]
}, Open  ]],
Cell[CellGroupData[{
Cell[711348, 28574, 139, 3, 26, "Input",
 CellTags->"FeynAmpDenominatorSplit"],
Cell[711490, 28579, 531, 17, 50, "Output",
 CellTags->"FeynAmpDenominatorSplit"]
}, Open  ]],
Cell[CellGroupData[{
Cell[712058, 28601, 150, 4, 26, "Input",
 CellTags->"FeynAmpDenominatorSplit"],
Cell[712211, 28607, 469, 14, 38, "Output",
 CellTags->"FeynAmpDenominatorSplit"]
}, Open  ]],
Cell[CellGroupData[{
Cell[712717, 28626, 230, 7, 26, "Input",
 CellTags->"FeynAmpDenominatorSplit"],
Cell[712950, 28635, 410, 13, 38, "Output",
 CellTags->"FeynAmpDenominatorSplit"]
}, Open  ]],
Cell[CellGroupData[{
Cell[713397, 28653, 206, 6, 26, "Input",
 CellTags->"FeynAmpDenominatorSplit"],
Cell[713606, 28661, 347, 10, 38, "Output",
 CellTags->"FeynAmpDenominatorSplit"]
}, Open  ]],
Cell[713968, 28674, 121, 3, 26, "Input",
 CellTags->"FeynAmpDenominatorSplit"]
}, Open  ]],
Cell[714104, 28680, 59, 1, 39, "ObjectName",
 CellTags->"FeynAmpList"],
Cell[CellGroupData[{
Cell[714188, 28685, 62, 1, 34, "Subsubsection",
 CellTags->"FeynAmpList"],
Cell[714253, 28688, 145, 4, 24, "Text",
 CellTags->"FeynAmpList"],
Cell[714401, 28694, 117, 3, 24, "Text",
 CellTags->"FeynAmpList"],
Cell[714521, 28699, 179, 9, 24, "Text",
 CellTags->"FeynAmpList"]
}, Open  ]],
Cell[714715, 28711, 53, 1, 39, "ObjectName",
 CellTags->"FeynCalc"],
Cell[CellGroupData[{
Cell[714793, 28716, 59, 1, 34, "Subsubsection",
 CellTags->"FeynCalc"],
Cell[714855, 28719, 91, 1, 24, "Text",
 CellTags->"FeynCalc"],
Cell[CellGroupData[{
Cell[714971, 28724, 99, 3, 26, "Input",
 CellTags->"FeynCalc"],
Cell[715073, 28729, 745, 12, 354, "Print",
 CellTags->{"FeynCalc", "Info3500973837-4982097"}]
}, Open  ]]
}, Open  ]],
Cell[715845, 28745, 69, 1, 39, "ObjectName",
 CellTags->"FeynCalcExternal"],
Cell[CellGroupData[{
Cell[715939, 28750, 67, 1, 34, "Subsubsection",
 CellTags->"FeynCalcExternal"],
Cell[716009, 28753, 156, 4, 24, "Text",
 CellTags->"FeynCalcExternal"],
Cell[716168, 28759, 211, 9, 24, "Text",
 CellTags->"FeynCalcExternal"]
}, Open  ]],
Cell[CellGroupData[{
Cell[716416, 28773, 64, 1, 34, "Subsubsection",
 CellTags->"FeynCalcExternal"],
Cell[CellGroupData[{
Cell[716505, 28778, 162, 4, 26, "Input",
 CellTags->"FeynCalcExternal"],
Cell[716670, 28784, 185, 5, 26, "Output",
 CellTags->"FeynCalcExternal"]
}, Open  ]],
Cell[CellGroupData[{
Cell[716892, 28794, 117, 3, 26, "Input",
 CellTags->"FeynCalcExternal"],
Cell[717012, 28799, 168, 4, 38, "Output",
 CellTags->"FeynCalcExternal"]
}, Open  ]],
Cell[CellGroupData[{
Cell[717217, 28808, 450, 13, 26, "Input",
 CellTags->"FeynCalcExternal"],
Cell[717670, 28823, 602, 19, 29, "Output",
 CellTags->"FeynCalcExternal"]
}, Open  ]],
Cell[CellGroupData[{
Cell[718309, 28847, 117, 3, 26, "Input",
 CellTags->"FeynCalcExternal"],
Cell[718429, 28852, 507, 14, 38, "Output",
 CellTags->"FeynCalcExternal"]
}, Open  ]],
Cell[CellGroupData[{
Cell[718973, 28871, 121, 3, 26, "Input",
 CellTags->"FeynCalcExternal"],
Cell[719097, 28876, 917, 35, 29, "Output",
 CellTags->"FeynCalcExternal"]
}, Open  ]],
Cell[CellGroupData[{
Cell[720051, 28916, 117, 3, 26, "Input",
 CellTags->"FeynCalcExternal"],
Cell[720171, 28921, 950, 25, 77, "Output",
 CellTags->"FeynCalcExternal"]
}, Open  ]],
Cell[CellGroupData[{
Cell[721158, 28951, 163, 5, 26, "Input",
 CellTags->"FeynCalcExternal"],
Cell[721324, 28958, 507, 14, 38, "Output",
 CellTags->"FeynCalcExternal"]
}, Open  ]]
}, Open  ]],
Cell[721858, 28976, 61, 1, 41, "ObjectName",
 CellTags->"FeynCalcForm"],
Cell[CellGroupData[{
Cell[721944, 28981, 63, 1, 36, "Subsubsection",
 CellTags->"FeynCalcForm"],
Cell[722010, 28984, 534, 10, 79, "Text",
 CellTags->"FeynCalcForm"],
Cell[722547, 28996, 433, 19, 25, "Text",
 CellTags->"FeynCalcForm"]
}, Open  ]],
Cell[CellGroupData[{
Cell[723017, 29020, 60, 1, 36, "Subsubsection",
 CellTags->"FeynCalcForm"],
Cell[723080, 29023, 79, 1, 24, "Text",
 CellTags->"FeynCalcForm"],
Cell[CellGroupData[{
Cell[723184, 29028, 242, 7, 26, "Input",
 CellTags->"FeynCalcForm"],
Cell[723429, 29037, 571, 23, 28, "Output",
 CellTags->"FeynCalcForm"]
}, Open  ]],
Cell[724015, 29063, 112, 3, 24, "Text",
 CellTags->"FeynCalcForm"],
Cell[724130, 29068, 148, 4, 26, "Input",
 CellTags->"FeynCalcForm"],
Cell[724281, 29074, 647, 19, 46, "Input",
 CellTags->"FeynCalcForm"],
Cell[CellGroupData[{
Cell[724953, 29097, 242, 7, 26, "Input",
 CellTags->"FeynCalcForm"],
Cell[725198, 29106, 606, 18, 26, "Output",
 CellTags->"FeynCalcForm"]
}, Open  ]],
Cell[725819, 29127, 76, 1, 24, "Text",
 CellTags->"FeynCalcForm"],
Cell[725898, 29130, 123, 4, 26, "Input",
 CellTags->"FeynCalcForm"],
Cell[726024, 29136, 652, 19, 46, "Input",
 CellTags->"FeynCalcForm"]
}, Open  ]],
Cell[726691, 29158, 69, 1, 41, "ObjectName",
 CellTags->"FeynCalcInternal"],
Cell[CellGroupData[{
Cell[726785, 29163, 67, 1, 36, "Subsubsection",
 CellTags->"FeynCalcInternal"],
Cell[726855, 29166, 157, 4, 25, "Text",
 CellTags->"FeynCalcInternal"],
Cell[727015, 29172, 401, 19, 25, "Text",
 CellTags->"FeynCalcInternal"]
}, Open  ]],
Cell[CellGroupData[{
Cell[727453, 29196, 64, 1, 34, "Subsubsection",
 CellTags->"FeynCalcInternal"],
Cell[CellGroupData[{
Cell[727542, 29201, 450, 13, 26, "Input",
 CellTags->"FeynCalcInternal"],
Cell[727995, 29216, 601, 19, 29, "Output",
 CellTags->"FeynCalcInternal"]
}, Open  ]],
Cell[CellGroupData[{
Cell[728633, 29240, 117, 3, 26, "Input",
 CellTags->"FeynCalcInternal"],
Cell[728753, 29245, 507, 14, 38, "Output",
 CellTags->"FeynCalcInternal"]
}, Open  ]],
Cell[CellGroupData[{
Cell[729297, 29264, 121, 3, 26, "Input",
 CellTags->"FeynCalcInternal"],
Cell[729421, 29269, 919, 35, 29, "Output",
 CellTags->"FeynCalcInternal"]
}, Open  ]],
Cell[CellGroupData[{
Cell[730377, 29309, 117, 3, 26, "Input",
 CellTags->"FeynCalcInternal"],
Cell[730497, 29314, 948, 25, 77, "Output",
 CellTags->"FeynCalcInternal"]
}, Open  ]],
Cell[CellGroupData[{
Cell[731482, 29344, 163, 5, 26, "Input",
 CellTags->"FeynCalcInternal"],
Cell[731648, 29351, 507, 14, 38, "Output",
 CellTags->"FeynCalcInternal"]
}, Open  ]],
Cell[CellGroupData[{
Cell[732192, 29370, 517, 15, 26, "Input",
 CellTags->"FeynCalcInternal"],
Cell[732712, 29387, 1505, 64, 50, "Output",
 CellTags->"FeynCalcInternal"]
}, Open  ]],
Cell[CellGroupData[{
Cell[734254, 29456, 117, 3, 26, "Input",
 CellTags->"FeynCalcInternal"],
Cell[734374, 29461, 1172, 31, 96, "Output",
 CellTags->"FeynCalcInternal"]
}, Open  ]]
}, Open  ]],
Cell[735573, 29496, 63, 1, 41, "ObjectName",
 CellTags->"FeynCalc2FORM"],
Cell[CellGroupData[{
Cell[735661, 29501, 64, 1, 36, "Subsubsection",
 CellTags->"FeynCalc2FORM"],
Cell[735728, 29504, 226, 5, 43, "Text",
 CellTags->"FeynCalc2FORM"],
Cell[735957, 29511, 199, 9, 24, "Text",
 CellTags->"FeynCalc2FORM"],
Cell[CellGroupData[{
Cell[736181, 29524, 125, 3, 26, "Input",
 CellTags->"FeynCalc2FORM"],
Cell[736309, 29529, 1063, 23, 67, "Output",
 CellTags->"FeynCalc2FORM"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[737421, 29558, 61, 1, 34, "Subsubsection",
 CellTags->"FeynCalc2FORM"],
Cell[CellGroupData[{
Cell[737507, 29563, 273, 9, 26, "Input",
 CellTags->"FeynCalc2FORM"],
Cell[737783, 29574, 320, 10, 47, "Output",
 CellTags->"FeynCalc2FORM"]
}, Open  ]],
Cell[CellGroupData[{
Cell[738140, 29589, 137, 4, 26, "Input",
 CellTags->"FeynCalc2FORM"],
Cell[738280, 29595, 112, 2, 21, "Print",
 CellTags->"FeynCalc2FORM"]
}, Open  ]],
Cell[CellGroupData[{
Cell[738429, 29602, 199, 6, 26, "Input",
 CellTags->"FeynCalc2FORM"],
Cell[738631, 29610, 241, 7, 26, "Output",
 CellTags->"FeynCalc2FORM"]
}, Open  ]],
Cell[CellGroupData[{
Cell[738909, 29622, 137, 4, 26, "Input",
 CellTags->"FeynCalc2FORM"],
Cell[739049, 29628, 110, 2, 21, "Print",
 CellTags->"FeynCalc2FORM"]
}, Open  ]],
Cell[CellGroupData[{
Cell[739196, 29635, 212, 6, 26, "Input",
 CellTags->"FeynCalc2FORM"],
Cell[739411, 29643, 415, 12, 26, "Output",
 CellTags->"FeynCalc2FORM"]
}, Open  ]],
Cell[CellGroupData[{
Cell[739863, 29660, 137, 4, 26, "Input",
 CellTags->"FeynCalc2FORM"],
Cell[740003, 29666, 132, 4, 21, "Print",
 CellTags->"FeynCalc2FORM"]
}, Open  ]],
Cell[CellGroupData[{
Cell[740172, 29675, 298, 9, 26, "Input",
 CellTags->"FeynCalc2FORM"],
Cell[740473, 29686, 517, 17, 26, "Output",
 CellTags->"FeynCalc2FORM"]
}, Open  ]],
Cell[CellGroupData[{
Cell[741027, 29708, 137, 4, 26, "Input",
 CellTags->"FeynCalc2FORM"],
Cell[741167, 29714, 130, 4, 21, "Print",
 CellTags->"FeynCalc2FORM"]
}, Open  ]],
Cell[741312, 29721, 328, 10, 26, "Input",
 CellTags->"FeynCalc2FORM"],
Cell[CellGroupData[{
Cell[741665, 29735, 332, 10, 26, "Input",
 CellTags->"FeynCalc2FORM"],
Cell[742000, 29747, 196, 5, 26, "Output",
 CellTags->"FeynCalc2FORM"]
}, Open  ]],
Cell[CellGroupData[{
Cell[742233, 29757, 314, 10, 26, "Input",
 CellTags->"FeynCalc2FORM"],
Cell[742550, 29769, 6221, 241, 72, "Output",
 CellTags->"FeynCalc2FORM"]
}, Open  ]],
Cell[748786, 30013, 209, 6, 26, "Input",
 CellTags->"FeynCalc2FORM"],
Cell[CellGroupData[{
Cell[749020, 30023, 376, 11, 26, "Input",
 CellTags->"FeynCalc2FORM"],
Cell[749399, 30036, 1382, 37, 231, "Output",
 CellTags->"FeynCalc2FORM"]
}, Open  ]],
Cell[750796, 30076, 313, 10, 26, "Input",
 CellTags->"FeynCalc2FORM"],
Cell[751112, 30088, 129, 4, 26, "Input",
 CellTags->"FeynCalc2FORM"]
}, Open  ]],
Cell[751256, 30095, 67, 1, 41, "ObjectName",
 CellTags->"FeynCalcToLaTeX"],
Cell[CellGroupData[{
Cell[751348, 30100, 66, 1, 36, "Subsubsection",
 CellTags->"FeynCalcToLaTeX"],
Cell[751417, 30103, 300, 6, 66, "Text",
 CellTags->"FeynCalcToLaTeX"],
Cell[CellGroupData[{
Cell[751742, 30113, 158, 4, 26, "Input"],
Cell[751903, 30119, 521, 19, 53, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[752461, 30143, 93, 2, 26, "Input"],
Cell[752557, 30147, 529, 12, 22, "Message"],
Cell[753089, 30161, 529, 12, 22, "Message"],
Cell[753621, 30175, 557, 10, 127, "Output"]
}, Open  ]]
}, Open  ]],
Cell[754205, 30189, 79, 1, 41, "ObjectName",
 CellTags->"FeynmanParameterNames"],
Cell[CellGroupData[{
Cell[754309, 30194, 72, 1, 36, "Subsubsection",
 CellTags->"FeynmanParameterNames"],
Cell[754384, 30197, 143, 4, 25, "Text",
 CellTags->"FeynmanParameterNames"],
Cell[754530, 30203, 362, 14, 25, "Text",
 CellTags->"FeynmanParameterNames"]
}, Open  ]],
Cell[754907, 30220, 90, 1, 41, "ObjectName",
 CellTags->"FeynmanParametrize"],
Cell[CellGroupData[{
Cell[755022, 30225, 69, 1, 36, "Subsubsection",
 CellTags->"FeynmanParametrize"],
Cell[755094, 30228, 176, 4, 24, "Text",
 CellTags->"FeynmanParametrize"],
Cell[CellGroupData[{
Cell[755295, 30236, 135, 3, 26, "Input",
 CellTags->"FeynmanParametrize"],
Cell[755433, 30241, 292, 9, 26, "Output",
 CellTags->"FeynmanParametrize"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[755774, 30256, 66, 1, 34, "Subsubsection",
 CellTags->"FeynmanParametrize"],
Cell[755843, 30259, 84, 2, 26, "Input",
 CellTags->"FeynmanParametrize"]
}, Open  ]],
Cell[755942, 30264, 53, 1, 41, "ObjectName",
 CellTags->"FeynRule"],
Cell[CellGroupData[{
Cell[756020, 30269, 59, 1, 36, "Subsubsection",
 CellTags->"FeynRule"],
Cell[756082, 30272, 168, 4, 24, "Text",
 CellTags->"FeynRule"],
Cell[CellGroupData[{
Cell[756275, 30280, 115, 3, 26, "Input",
 CellTags->"FeynRule"],
Cell[756393, 30285, 957, 22, 107, "Output",
 CellTags->"FeynRule"]
}, Open  ]],
Cell[757365, 30310, 92, 1, 25, "Text",
 CellTags->"FeynRule"],
Cell[757460, 30313, 133, 4, 25, "Text",
 CellTags->"FeynRule"],
Cell[757596, 30319, 185, 9, 25, "Text",
 CellTags->"FeynRule"]
}, Open  ]],
Cell[CellGroupData[{
Cell[757818, 30333, 56, 1, 34, "Subsubsection",
 CellTags->"FeynRule"],
Cell[CellGroupData[{
Cell[757899, 30338, 146, 4, 26, "Input",
 CellTags->"FeynRule"],
Cell[758048, 30344, 721, 24, 44, "Output",
 CellTags->"FeynRule"]
}, Open  ]],
Cell[CellGroupData[{
Cell[758806, 30373, 146, 4, 26, "Input",
 CellTags->"FeynRule"],
Cell[758955, 30379, 1045, 37, 44, "Output",
 CellTags->"FeynRule"]
}, Open  ]],
Cell[CellGroupData[{
Cell[760037, 30421, 111, 3, 26, "Input",
 CellTags->"FeynRule"],
Cell[760151, 30426, 5117, 160, 323, "Output",
 CellTags->"FeynRule"]
}, Open  ]],
Cell[CellGroupData[{
Cell[765305, 30591, 141, 4, 26, "Input",
 CellTags->"FeynRule"],
Cell[765449, 30597, 3277, 97, 276, "Output",
 CellTags->"FeynRule"]
}, Open  ]],
Cell[768741, 30697, 74, 1, 24, "Text",
 CellTags->"FeynRule"],
Cell[CellGroupData[{
Cell[768840, 30702, 503, 16, 26, "Input",
 CellTags->"FeynRule"],
Cell[769346, 30720, 742, 31, 32, "Output",
 CellTags->"FeynRule"]
}, Open  ]],
Cell[CellGroupData[{
Cell[770125, 30756, 307, 9, 26, "Input",
 CellTags->"FeynRule"],
Cell[770435, 30767, 3837, 139, 52, "Output",
 CellTags->"FeynRule"]
}, Open  ]],
Cell[774287, 30909, 72, 1, 24, "Text",
 CellTags->"FeynRule"],
Cell[CellGroupData[{
Cell[774384, 30914, 503, 16, 26, "Input",
 CellTags->"FeynRule"],
Cell[774890, 30932, 742, 31, 32, "Output",
 CellTags->"FeynRule"]
}, Open  ]],
Cell[CellGroupData[{
Cell[775669, 30968, 307, 9, 26, "Input",
 CellTags->"FeynRule"],
Cell[775979, 30979, 4509, 152, 52, "Output",
 CellTags->"FeynRule"]
}, Open  ]],
Cell[CellGroupData[{
Cell[780525, 31136, 218, 7, 26, "Input",
 CellTags->"FeynRule"],
Cell[780746, 31145, 1086, 43, 28, "Output",
 CellTags->"FeynRule"]
}, Open  ]],
Cell[781847, 31191, 110, 3, 24, "Text",
 CellTags->"FeynRule"],
Cell[CellGroupData[{
Cell[781982, 31198, 360, 10, 26, "Input",
 CellTags->"FeynRule"],
Cell[782345, 31210, 1150, 46, 28, "Output",
 CellTags->"FeynRule"]
}, Open  ]],
Cell[783510, 31259, 77, 1, 24, "Text",
 CellTags->"FeynRule"],
Cell[CellGroupData[{
Cell[783612, 31264, 145, 4, 26, "Input",
 CellTags->"FeynRule"],
Cell[783760, 31270, 660, 23, 26, "Output",
 CellTags->"FeynRule"]
}, Open  ]],
Cell[784435, 31296, 90, 1, 24, "Text",
 CellTags->"FeynRule"],
Cell[CellGroupData[{
Cell[784550, 31301, 145, 4, 26, "Input",
 CellTags->"FeynRule"],
Cell[784698, 31307, 662, 23, 26, "Output",
 CellTags->"FeynRule"]
}, Open  ]],
Cell[CellGroupData[{
Cell[785397, 31335, 696, 21, 46, "Input",
 CellTags->"FeynRule"],
Cell[786096, 31358, 937, 38, 32, "Output",
 CellTags->"FeynRule"]
}, Open  ]],
Cell[CellGroupData[{
Cell[787070, 31401, 227, 7, 26, "Input",
 CellTags->"FeynRule"],
Cell[787300, 31410, 5001, 178, 118, "Output",
 CellTags->"FeynRule"]
}, Open  ]],
Cell[CellGroupData[{
Cell[792338, 31593, 420, 12, 26, "Input",
 CellTags->"FeynRule"],
Cell[792761, 31607, 4806, 160, 114, "Output",
 CellTags->"FeynRule"]
}, Open  ]],
Cell[CellGroupData[{
Cell[797604, 31772, 362, 12, 26, "Input",
 CellTags->"FeynRule"],
Cell[797969, 31786, 148, 4, 26, "Output",
 CellTags->"FeynRule"]
}, Open  ]],
Cell[798132, 31793, 210, 5, 41, "Text",
 CellTags->"FeynRule"],
Cell[798345, 31800, 144, 4, 26, "Input",
 CellTags->"FeynRule"],
Cell[798492, 31806, 51, 1, 24, "Text",
 CellTags->"FeynRule"],
Cell[CellGroupData[{
Cell[798568, 31811, 693, 21, 46, "Input",
 CellTags->"FeynRule"],
Cell[799264, 31834, 1028, 43, 32, "Output",
 CellTags->"FeynRule"]
}, Open  ]],
Cell[CellGroupData[{
Cell[800329, 31882, 212, 7, 26, "Input",
 CellTags->"FeynRule"],
Cell[800544, 31891, 2015, 80, 28, "Output",
 CellTags->"FeynRule"]
}, Open  ]],
Cell[CellGroupData[{
Cell[802596, 31976, 385, 11, 26, "Input",
 CellTags->"FeynRule"],
Cell[802984, 31989, 1461, 58, 28, "Output",
 CellTags->"FeynRule"]
}, Open  ]],
Cell[CellGroupData[{
Cell[804482, 32052, 197, 6, 26, "Input",
 CellTags->"FeynRule"],
Cell[804682, 32060, 146, 4, 26, "Output",
 CellTags->"FeynRule"]
}, Open  ]],
Cell[CellGroupData[{
Cell[804865, 32069, 884, 26, 46, "Input",
 CellTags->"FeynRule"],
Cell[805752, 32097, 1314, 55, 32, "Output",
 CellTags->"FeynRule"]
}, Open  ]],
Cell[CellGroupData[{
Cell[807103, 32157, 212, 7, 26, "Input",
 CellTags->"FeynRule"],
Cell[807318, 32166, 3347, 132, 50, "Output",
 CellTags->"FeynRule"]
}, Open  ]],
Cell[CellGroupData[{
Cell[810702, 32303, 461, 13, 26, "Input",
 CellTags->"FeynRule"],
Cell[811166, 32318, 3512, 133, 50, "Output",
 CellTags->"FeynRule"]
}, Open  ]],
Cell[CellGroupData[{
Cell[814715, 32456, 197, 6, 26, "Input",
 CellTags->"FeynRule"],
Cell[814915, 32464, 148, 4, 26, "Output",
 CellTags->"FeynRule"]
}, Open  ]],
Cell[815078, 32471, 418, 9, 46, "Input",
 CellTags->"FeynRule"]
}, Open  ]],
Cell[815511, 32483, 41, 1, 41, "ObjectName",
 CellTags->"FI"],
Cell[CellGroupData[{
Cell[815577, 32488, 53, 1, 36, "Subsubsection",
 CellTags->"FI"],
Cell[815633, 32491, 195, 4, 43, "Text",
 CellTags->"FI"],
Cell[815831, 32497, 545, 24, 25, "Text",
 CellTags->"FI"]
}, Open  ]],
Cell[816391, 32524, 67, 1, 41, "ObjectName",
 CellTags->"FieldDerivative"],
Cell[CellGroupData[{
Cell[816483, 32529, 66, 1, 36, "Subsubsection",
 CellTags->"FieldDerivative"],
Cell[816552, 32532, 549, 9, 79, "Text",
 CellTags->"FieldDerivative"],
Cell[817104, 32543, 314, 14, 25, "Text",
 CellTags->"FieldDerivative"]
}, Open  ]],
Cell[CellGroupData[{
Cell[817455, 32562, 63, 1, 34, "Subsubsection",
 CellTags->"FieldDerivative"],
Cell[CellGroupData[{
Cell[817543, 32567, 620, 19, 46, "Input",
 CellTags->"FieldDerivative"],
Cell[818166, 32588, 507, 18, 29, "Output",
 CellTags->"FieldDerivative"]
}, Open  ]],
Cell[CellGroupData[{
Cell[818710, 32611, 218, 6, 26, "Input",
 CellTags->"FieldDerivative"],
Cell[818931, 32619, 1885, 52, 153, "Output",
 CellTags->"FieldDerivative"]
}, Open  ]],
Cell[CellGroupData[{
Cell[820853, 32676, 218, 6, 26, "Input",
 CellTags->"FieldDerivative"],
Cell[821074, 32684, 5569, 138, 521, "Output",
 CellTags->"FieldDerivative"]
}, Open  ]]
}, Open  ]],
Cell[826670, 32826, 63, 1, 41, "ObjectName",
 CellTags->"FieldStrength"],
Cell[CellGroupData[{
Cell[826758, 32831, 64, 1, 36, "Subsubsection",
 CellTags->"FieldStrength"],
Cell[826825, 32834, 1285, 37, 66, "Text",
 CellTags->"FieldStrength"],
Cell[CellGroupData[{
Cell[828135, 32875, 125, 3, 26, "Input",
 CellTags->"FieldStrength"],
Cell[828263, 32880, 570, 16, 47, "Output",
 CellTags->"FieldStrength"]
}, Open  ]],
Cell[828848, 32899, 196, 9, 25, "Text",
 CellTags->"FieldStrength"]
}, Open  ]],
Cell[CellGroupData[{
Cell[829081, 32913, 61, 1, 34, "Subsubsection",
 CellTags->"FieldStrength"],
Cell[CellGroupData[{
Cell[829167, 32918, 150, 4, 26, "Input",
 CellTags->"FieldStrength"],
Cell[829320, 32924, 216, 6, 30, "Output",
 CellTags->"FieldStrength"]
}, Open  ]],
Cell[CellGroupData[{
Cell[829573, 32935, 160, 4, 26, "Input",
 CellTags->"FieldStrength"],
Cell[829736, 32941, 212, 6, 30, "Output",
 CellTags->"FieldStrength"]
}, Open  ]],
Cell[CellGroupData[{
Cell[829985, 32952, 200, 5, 26, "Input",
 CellTags->"FieldStrength"],
Cell[830188, 32959, 1066, 25, 147, "Output",
 CellTags->"FieldStrength"]
}, Open  ]],
Cell[CellGroupData[{
Cell[831291, 32989, 210, 5, 26, "Input",
 CellTags->"FieldStrength"],
Cell[831504, 32996, 1869, 58, 152, "Output",
 CellTags->"FieldStrength"]
}, Open  ]],
Cell[CellGroupData[{
Cell[833410, 33059, 241, 6, 26, "Input",
 CellTags->"FieldStrength"],
Cell[833654, 33067, 1065, 22, 191, "Output",
 CellTags->"FieldStrength"]
}, Open  ]],
Cell[CellGroupData[{
Cell[834756, 33094, 244, 6, 26, "Input",
 CellTags->"FieldStrength"],
Cell[835003, 33102, 1104, 23, 191, "Output",
 CellTags->"FieldStrength"]
}, Open  ]],
Cell[CellGroupData[{
Cell[836144, 33130, 291, 7, 26, "Input",
 CellTags->"FieldStrength"],
Cell[836438, 33139, 1869, 58, 152, "Output",
 CellTags->"FieldStrength"]
}, Open  ]]
}, Open  ]],
Cell[838334, 33201, 63, 1, 41, "ObjectName",
 CellTags->"FinalFunction"],
Cell[CellGroupData[{
Cell[838422, 33206, 64, 1, 36, "Subsubsection",
 CellTags->"FinalFunction"],
Cell[838489, 33209, 86, 1, 25, "Text",
 CellTags->"FinalFunction"],
Cell[838578, 33212, 190, 9, 25, "Text",
 CellTags->"FinalFunction"]
}, Open  ]],
Cell[838783, 33224, 73, 1, 41, "ObjectName",
 CellTags->"FinalSubstitutions"],
Cell[CellGroupData[{
Cell[838881, 33229, 69, 1, 36, "Subsubsection",
 CellTags->"FinalSubstitutions"],
Cell[838953, 33232, 204, 4, 43, "Text",
 CellTags->"FinalSubstitutions"],
Cell[839160, 33238, 406, 19, 25, "Text",
 CellTags->"FinalSubstitutions"]
}, Open  ]],
Cell[839581, 33260, 45, 1, 41, "ObjectName",
 CellTags->"FORM"],
Cell[CellGroupData[{
Cell[839651, 33265, 55, 1, 36, "Subsubsection",
 CellTags->"FORM"],
Cell[839709, 33268, 204, 5, 43, "Text",
 CellTags->"FORM"],
Cell[839916, 33275, 285, 14, 25, "Text",
 CellTags->"FORM"]
}, Open  ]],
Cell[840216, 33292, 57, 1, 41, "ObjectName",
 CellTags->"FORMEpilog"],
Cell[CellGroupData[{
Cell[840298, 33297, 61, 1, 36, "Subsubsection",
 CellTags->"FORMEpilog"],
Cell[840362, 33300, 161, 4, 25, "Text",
 CellTags->"FORMEpilog"],
Cell[840526, 33306, 312, 14, 25, "Text",
 CellTags->"FORMEpilog"]
}, Open  ]],
Cell[840853, 33323, 57, 1, 41, "ObjectName",
 CellTags->"FORMProlog"],
Cell[CellGroupData[{
Cell[840935, 33328, 61, 1, 36, "Subsubsection",
 CellTags->"FORMProlog"],
Cell[840999, 33331, 178, 4, 25, "Text",
 CellTags->"FORMProlog"],
Cell[841180, 33337, 312, 14, 25, "Text",
 CellTags->"FORMProlog"]
}, Open  ]],
Cell[841507, 33354, 63, 1, 41, "ObjectName",
 CellTags->"FORM2FeynCalc"],
Cell[CellGroupData[{
Cell[841595, 33359, 64, 1, 36, "Subsubsection",
 CellTags->"FORM2FeynCalc"],
Cell[841662, 33362, 470, 8, 79, "Text",
 CellTags->"FORM2FeynCalc"],
Cell[842135, 33372, 199, 9, 24, "Text",
 CellTags->"FORM2FeynCalc"],
Cell[CellGroupData[{
Cell[842359, 33385, 125, 3, 26, "Input",
 CellTags->"FORM2FeynCalc"],
Cell[842487, 33390, 704, 19, 47, "Output",
 CellTags->"FORM2FeynCalc"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[843240, 33415, 61, 1, 34, "Subsubsection",
 CellTags->"FORM2FeynCalc"],
Cell[CellGroupData[{
Cell[843326, 33420, 139, 3, 26, "Input",
 CellTags->"FORM2FeynCalc"],
Cell[843468, 33425, 395, 15, 28, "Output",
 CellTags->"FORM2FeynCalc"]
}, Open  ]],
Cell[CellGroupData[{
Cell[843900, 33445, 114, 3, 26, "Input",
 CellTags->"FORM2FeynCalc"],
Cell[844017, 33450, 285, 9, 38, "Output",
 CellTags->"FORM2FeynCalc"]
}, Open  ]],
Cell[844317, 33462, 159, 4, 24, "Text",
 CellTags->"FORM2FeynCalc"],
Cell[CellGroupData[{
Cell[844501, 33470, 314, 8, 26, "Input",
 CellTags->"FORM2FeynCalc"],
Cell[844818, 33480, 340, 11, 26, "Output",
 CellTags->"FORM2FeynCalc"]
}, Open  ]],
Cell[CellGroupData[{
Cell[845195, 33496, 114, 3, 26, "Input",
 CellTags->"FORM2FeynCalc"],
Cell[845312, 33501, 347, 10, 38, "Output",
 CellTags->"FORM2FeynCalc"]
}, Open  ]],
Cell[CellGroupData[{
Cell[845696, 33516, 150, 4, 26, "Input",
 CellTags->"FORM2FeynCalc"],
Cell[845849, 33522, 375, 13, 26, "Output",
 CellTags->"FORM2FeynCalc"]
}, Open  ]],
Cell[CellGroupData[{
Cell[846261, 33540, 117, 3, 26, "Input",
 CellTags->"FORM2FeynCalc"],
Cell[846381, 33545, 303, 10, 26, "Output",
 CellTags->"FORM2FeynCalc"]
}, Open  ]],
Cell[CellGroupData[{
Cell[846721, 33560, 150, 4, 26, "Input",
 CellTags->"FORM2FeynCalc"],
Cell[846874, 33566, 301, 10, 28, "Output",
 CellTags->"FORM2FeynCalc"]
}, Open  ]],
Cell[CellGroupData[{
Cell[847212, 33581, 114, 3, 26, "Input",
 CellTags->"FORM2FeynCalc"],
Cell[847329, 33586, 351, 11, 38, "Output",
 CellTags->"FORM2FeynCalc"]
}, Open  ]],
Cell[CellGroupData[{
Cell[847717, 33602, 333, 9, 26, "Input",
 CellTags->"FORM2FeynCalc"],
Cell[848053, 33613, 313, 10, 28, "Output",
 CellTags->"FORM2FeynCalc"]
}, Open  ]],
Cell[CellGroupData[{
Cell[848403, 33628, 195, 5, 26, "Input",
 CellTags->"FORM2FeynCalc"],
Cell[848601, 33635, 625, 19, 26, "Output",
 CellTags->"FORM2FeynCalc"]
}, Open  ]]
}, Open  ]],
Cell[849253, 33658, 65, 1, 41, "ObjectName",
 CellTags->"FourDivergence"],
Cell[CellGroupData[{
Cell[849343, 33663, 65, 1, 36, "Subsubsection",
 CellTags->"FourDivergence"],
Cell[849411, 33666, 242, 5, 43, "Text",
 CellTags->"FourDivergence"],
Cell[849656, 33673, 197, 9, 25, "Text",
 CellTags->"FourDivergence"]
}, Open  ]],
Cell[CellGroupData[{
Cell[849890, 33687, 62, 1, 34, "Subsubsection",
 CellTags->"FourDivergence"],
Cell[CellGroupData[{
Cell[849977, 33692, 167, 5, 26, "Input",
 CellTags->"FourDivergence"],
Cell[850147, 33699, 256, 9, 28, "Output",
 CellTags->"FourDivergence"]
}, Open  ]],
Cell[CellGroupData[{
Cell[850440, 33713, 211, 6, 26, "Input",
 CellTags->"FourDivergence"],
Cell[850654, 33721, 284, 11, 28, "Output",
 CellTags->"FourDivergence"]
}, Open  ]],
Cell[CellGroupData[{
Cell[850975, 33737, 192, 6, 26, "Input",
 CellTags->"FourDivergence"],
Cell[851170, 33745, 289, 10, 28, "Output",
 CellTags->"FourDivergence"]
}, Open  ]],
Cell[CellGroupData[{
Cell[851496, 33760, 237, 7, 26, "Input",
 CellTags->"FourDivergence"],
Cell[851736, 33769, 152, 4, 26, "Output",
 CellTags->"FourDivergence"]
}, Open  ]],
Cell[851903, 33776, 112, 3, 26, "Input",
 CellTags->"FourDivergence"]
}, Open  ]],
Cell[852030, 33782, 63, 1, 41, "ObjectName",
 CellTags->"FourLaplacian"],
Cell[CellGroupData[{
Cell[852118, 33787, 64, 1, 36, "Subsubsection",
 CellTags->"FourLaplacian"],
Cell[852185, 33790, 426, 16, 27, "Text",
 CellTags->"FourLaplacian"],
Cell[CellGroupData[{
Cell[852636, 33810, 125, 3, 26, "Input",
 CellTags->"FourLaplacian"],
Cell[852764, 33815, 212, 6, 26, "Output",
 CellTags->"FourLaplacian"]
}, Open  ]],
Cell[852991, 33824, 324, 14, 25, "Text",
 CellTags->"FourLaplacian"]
}, Open  ]],
Cell[CellGroupData[{
Cell[853352, 33843, 61, 1, 34, "Subsubsection",
 CellTags->"FourLaplacian"],
Cell[CellGroupData[{
Cell[853438, 33848, 131, 4, 26, "Input",
 CellTags->"FourLaplacian"],
Cell[853572, 33854, 209, 7, 28, "Output",
 CellTags->"FourLaplacian"]
}, Open  ]],
Cell[CellGroupData[{
Cell[853818, 33866, 152, 4, 26, "Input",
 CellTags->"FourLaplacian"],
Cell[853973, 33872, 176, 5, 26, "Output",
 CellTags->"FourLaplacian"]
}, Open  ]],
Cell[CellGroupData[{
Cell[854186, 33882, 264, 9, 26, "Input",
 CellTags->"FourLaplacian"],
Cell[854453, 33893, 482, 17, 49, "Output",
 CellTags->"FourLaplacian"]
}, Open  ]],
Cell[CellGroupData[{
Cell[854972, 33915, 157, 4, 26, "Input",
 CellTags->"FourLaplacian"],
Cell[855132, 33921, 3899, 137, 100, "Output",
 CellTags->"FourLaplacian"]
}, Open  ]]
}, Open  ]],
Cell[859058, 34062, 57, 1, 41, "ObjectName",
 CellTags->"FourVector"],
Cell[CellGroupData[{
Cell[859140, 34067, 61, 1, 36, "Subsubsection",
 CellTags->"FourVector"],
Cell[859204, 34070, 339, 10, 43, "Text",
 CellTags->"FourVector"],
Cell[859546, 34082, 356, 19, 25, "Text",
 CellTags->"FourVector"]
}, Open  ]],
Cell[CellGroupData[{
Cell[859939, 34106, 58, 1, 34, "Subsubsection",
 CellTags->"FourVector"],
Cell[CellGroupData[{
Cell[860022, 34111, 140, 4, 26, "Input",
 CellTags->"FourVector"],
Cell[860165, 34117, 174, 5, 29, "Output",
 CellTags->"FourVector"]
}, Open  ]],
Cell[CellGroupData[{
Cell[860376, 34127, 164, 5, 26, "Input",
 CellTags->"FourVector"],
Cell[860543, 34134, 291, 11, 30, "Output",
 CellTags->"FourVector"]
}, Open  ]],
Cell[CellGroupData[{
Cell[860871, 34150, 180, 5, 26, "Input",
 CellTags->"FourVector"],
Cell[861054, 34157, 197, 5, 38, "Output",
 CellTags->"FourVector"]
}, Open  ]],
Cell[CellGroupData[{
Cell[861288, 34167, 229, 6, 26, "Input",
 CellTags->"FourVector"],
Cell[861520, 34175, 245, 6, 38, "Output",
 CellTags->"FourVector"]
}, Open  ]],
Cell[CellGroupData[{
Cell[861802, 34186, 212, 6, 26, "Input",
 CellTags->"FourVector"],
Cell[862017, 34194, 199, 5, 38, "Output",
 CellTags->"FourVector"]
}, Open  ]],
Cell[862231, 34202, 193, 4, 41, "Text",
 CellTags->"FourVector"],
Cell[CellGroupData[{
Cell[862449, 34210, 212, 6, 26, "Input",
 CellTags->"FourVector"],
Cell[862664, 34218, 291, 11, 30, "Output",
 CellTags->"FourVector"]
}, Open  ]]
}, Open  ]],
Cell[862982, 34233, 55, 1, 41, "ObjectName",
 CellTags->"FreeIndex"],
Cell[CellGroupData[{
Cell[863062, 34238, 60, 1, 36, "Subsubsection",
 CellTags->"FreeIndex"],
Cell[863125, 34241, 152, 4, 25, "Text",
 CellTags->"FreeIndex"],
Cell[863280, 34247, 290, 14, 25, "Text",
 CellTags->"FreeIndex"]
}, Open  ]],
Cell[863585, 34264, 49, 1, 41, "ObjectName",
 CellTags->"FreeQ2"],
Cell[CellGroupData[{
Cell[863659, 34269, 57, 1, 36, "Subsubsection",
 CellTags->"FreeQ2"],
Cell[863719, 34272, 228, 5, 43, "Text",
 CellTags->"FreeQ2"],
Cell[863950, 34279, 308, 14, 25, "Text",
 CellTags->"FreeQ2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[864295, 34298, 54, 1, 34, "Subsubsection",
 CellTags->"FreeQ2"],
Cell[CellGroupData[{
Cell[864374, 34303, 246, 8, 26, "Input",
 CellTags->"FreeQ2"],
Cell[864623, 34313, 148, 4, 26, "Output",
 CellTags->"FreeQ2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[864808, 34322, 241, 8, 26, "Input",
 CellTags->"FreeQ2"],
Cell[865052, 34332, 147, 4, 26, "Output",
 CellTags->"FreeQ2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[865236, 34341, 133, 4, 26, "Input",
 CellTags->"FreeQ2"],
Cell[865372, 34347, 145, 4, 26, "Output",
 CellTags->"FreeQ2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[865554, 34356, 162, 5, 26, "Input",
 CellTags->"FreeQ2"],
Cell[865719, 34363, 148, 4, 26, "Output",
 CellTags->"FreeQ2"]
}, Open  ]]
}, Open  ]],
Cell[865894, 34371, 43, 1, 41, "ObjectName",
 CellTags->"FRH"],
Cell[CellGroupData[{
Cell[865962, 34376, 54, 1, 36, "Subsubsection",
 CellTags->"FRH"],
Cell[866019, 34379, 134, 4, 25, "Text",
 CellTags->"FRH"],
Cell[866156, 34385, 171, 9, 25, "Text",
 CellTags->"FRH"]
}, Open  ]],
Cell[CellGroupData[{
Cell[866364, 34399, 51, 1, 34, "Subsubsection",
 CellTags->"FRH"],
Cell[CellGroupData[{
Cell[866440, 34404, 200, 6, 26, "Input",
 CellTags->"FRH"],
Cell[866643, 34412, 290, 10, 26, "Output",
 CellTags->"FRH"]
}, Open  ]],
Cell[CellGroupData[{
Cell[866970, 34427, 99, 3, 26, "Input",
 CellTags->"FRH"],
Cell[867072, 34432, 141, 4, 26, "Output",
 CellTags->"FRH"]
}, Open  ]],
Cell[CellGroupData[{
Cell[867250, 34441, 304, 10, 26, "Input",
 CellTags->"FRH"],
Cell[867557, 34453, 573, 21, 26, "Output",
 CellTags->"FRH"]
}, Open  ]],
Cell[CellGroupData[{
Cell[868167, 34479, 128, 4, 26, "Input",
 CellTags->"FRH"],
Cell[868298, 34485, 170, 5, 26, "Output",
 CellTags->"FRH"]
}, Open  ]]
}, Open  ]],
Cell[868495, 34494, 51, 1, 41, "ObjectName",
 CellTags->"FromTFi"],
Cell[CellGroupData[{
Cell[868571, 34499, 58, 1, 36, "Subsubsection",
 CellTags->"FromTFi"],
Cell[868632, 34502, 200, 5, 43, "Text",
 CellTags->"FromTFi"],
Cell[868835, 34509, 258, 13, 25, "Text",
 CellTags->"FromTFI"]
}, Open  ]],
Cell[CellGroupData[{
Cell[869130, 34527, 55, 1, 34, "Subsubsection",
 CellTags->"FromTFi"],
Cell[CellGroupData[{
Cell[869210, 34532, 431, 14, 26, "Input",
 CellTags->"FromTFi"],
Cell[869644, 34548, 818, 25, 97, "Output",
 CellTags->"FromTFi"]
}, Open  ]],
Cell[CellGroupData[{
Cell[870499, 34578, 693, 22, 26, "Input",
 CellTags->"FromTFi"],
Cell[871195, 34602, 1010, 32, 97, "Output",
 CellTags->"FromTFi"]
}, Open  ]],
Cell[CellGroupData[{
Cell[872242, 34639, 756, 24, 26, "Input",
 CellTags->"FromTFi"],
Cell[873001, 34665, 1069, 34, 97, "Output",
 CellTags->"FromTFi"]
}, Open  ]]
}, Open  ]],
Cell[874097, 34703, 53, 1, 41, "ObjectName",
 CellTags->"FUNCTION"],
Cell[CellGroupData[{
Cell[874175, 34708, 59, 1, 36, "Subsubsection",
 CellTags->"FUNCTION"],
Cell[874237, 34711, 160, 4, 25, "Text",
 CellTags->"FUNCTION"],
Cell[874400, 34717, 173, 9, 25, "Text",
 CellTags->"FUNCTION"]
}, Open  ]],
Cell[874588, 34729, 59, 1, 41, "ObjectName",
 CellTags->"FunctionalD"],
Cell[CellGroupData[{
Cell[874672, 34734, 62, 1, 36, "Subsubsection",
 CellTags->"FunctionalD"],
Cell[874737, 34737, 416, 7, 61, "Text",
 CellTags->"FunctionalD"],
Cell[875156, 34746, 95, 1, 25, "Text",
 CellTags->"FunctionalD"],
Cell[875254, 34749, 304, 14, 25, "Text",
 CellTags->"FunctionalD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[875595, 34768, 59, 1, 36, "Subsubsection",
 CellTags->"FunctionalD"],
Cell[875657, 34771, 610, 20, 40, "Text",
 CellTags->"FunctionalD"],
Cell[CellGroupData[{
Cell[876292, 34795, 228, 6, 26, "Input",
 CellTags->"FunctionalD"],
Cell[876523, 34803, 149, 4, 26, "Output",
 CellTags->"FunctionalD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[876709, 34812, 253, 7, 26, "Input",
 CellTags->"FunctionalD"],
Cell[876965, 34821, 179, 5, 26, "Output",
 CellTags->"FunctionalD"]
}, Open  ]],
Cell[877159, 34829, 1269, 44, 51, "Text",
 CellTags->"FunctionalD"],
Cell[CellGroupData[{
Cell[878453, 34877, 295, 8, 26, "Input",
 CellTags->"FunctionalD"],
Cell[878751, 34887, 315, 11, 33, "Output",
 CellTags->"FunctionalD"]
}, Open  ]],
Cell[879081, 34901, 778, 24, 28, "Text",
 CellTags->"FunctionalD"],
Cell[CellGroupData[{
Cell[879884, 34929, 668, 22, 46, "Input",
 CellTags->"FunctionalD"],
Cell[880555, 34953, 1191, 29, 171, "Output",
 CellTags->"FunctionalD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[881783, 34987, 217, 6, 26, "Input",
 CellTags->"FunctionalD"],
Cell[882003, 34995, 1047, 25, 147, "Output",
 CellTags->"FunctionalD"]
}, Open  ]],
Cell[883065, 35023, 489, 15, 25, "Text",
 CellTags->"FunctionalD"],
Cell[883557, 35040, 97, 2, 24, "Text",
 CellTags->"FunctionalD"],
Cell[CellGroupData[{
Cell[883679, 35046, 315, 8, 26, "Input",
 CellTags->"FunctionalD"],
Cell[883997, 35056, 1743, 53, 152, "Output",
 CellTags->"FunctionalD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[885777, 35114, 315, 8, 26, "Input",
 CellTags->"FunctionalD"],
Cell[886095, 35124, 1741, 53, 152, "Output",
 CellTags->"FunctionalD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[887873, 35182, 216, 8, 26, "Input",
 CellTags->"FunctionalD"],
Cell[888092, 35192, 3730, 108, 299, "Output",
 CellTags->"FunctionalD"]
}, Open  ]],
Cell[891837, 35303, 560, 13, 61, "Text",
 CellTags->"FunctionalD"],
Cell[892400, 35318, 1398, 50, 25, "Text",
 CellTags->"FunctionalD"],
Cell[893801, 35370, 445, 13, 26, "Text",
 CellTags->"FunctionalD"],
Cell[CellGroupData[{
Cell[894271, 35387, 231, 7, 26, "Input",
 CellTags->"FunctionalD"],
Cell[894505, 35396, 330, 13, 28, "Output",
 CellTags->"FunctionalD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[894872, 35414, 165, 5, 26, "Input",
 CellTags->"FunctionalD"],
Cell[895040, 35421, 2330, 97, 34, "Output",
 CellTags->"FunctionalD"]
}, Open  ]],
Cell[897385, 35521, 120, 3, 24, "Text",
 CellTags->"FunctionalD"],
Cell[CellGroupData[{
Cell[897530, 35528, 176, 5, 26, "Input",
 CellTags->"FunctionalD"],
Cell[897709, 35535, 2330, 97, 34, "Output",
 CellTags->"FunctionalD"]
}, Open  ]],
Cell[900054, 35635, 290, 8, 26, "Text",
 CellTags->"FunctionalD"],
Cell[900347, 35645, 523, 16, 46, "Input",
 CellTags->"FunctionalD"],
Cell[CellGroupData[{
Cell[900895, 35665, 230, 7, 26, "Input",
 CellTags->"FunctionalD"],
Cell[901128, 35674, 428, 18, 30, "Output",
 CellTags->"FunctionalD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[901593, 35697, 240, 7, 26, "Input",
 CellTags->"FunctionalD"],
Cell[901836, 35706, 457, 19, 30, "Output",
 CellTags->"FunctionalD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[902330, 35730, 579, 16, 46, "Input",
 CellTags->"FunctionalD"],
Cell[902912, 35748, 3204, 117, 206, "Output",
 CellTags->"FunctionalD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[906153, 35870, 184, 5, 26, "Input",
 CellTags->"FunctionalD"],
Cell[906340, 35877, 1588, 58, 91, "Output",
 CellTags->"FunctionalD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[907965, 35940, 352, 11, 26, "Input",
 CellTags->"FunctionalD"],
Cell[908320, 35953, 1340, 46, 91, "Output",
 CellTags->"FunctionalD"]
}, Open  ]],
Cell[909675, 36002, 269, 9, 27, "Text",
 CellTags->"FunctionalD"],
Cell[909947, 36013, 98, 2, 24, "Text",
 CellTags->"FunctionalD"],
Cell[910048, 36017, 167, 4, 24, "Text",
 CellTags->"FunctionalD"],
Cell[CellGroupData[{
Cell[910240, 36025, 105, 3, 26, "Input",
 CellTags->"FunctionalD"],
Cell[910348, 36030, 3730, 108, 299, "Output",
 CellTags->"FunctionalD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[914115, 36143, 194, 6, 26, "Input",
 CellTags->"FunctionalD"],
Cell[914312, 36151, 27621, 950, 1944, "Output",
 CellTags->"FunctionalD"]
}, Open  ]],
Cell[941948, 37104, 227, 5, 43, "Text",
 CellTags->"FunctionalD"],
Cell[942178, 37111, 345, 6, 58, "Text",
 CellTags->"FunctionalD"],
Cell[942526, 37119, 132, 4, 26, "Input",
 CellTags->"FunctionalD"],
Cell[942661, 37125, 2583, 67, 141, "Input",
 CellTags->"FunctionalD"],
Cell[CellGroupData[{
Cell[945269, 37196, 169, 5, 26, "Input",
 CellTags->"FunctionalD"],
Cell[945441, 37203, 14498, 491, 1082, "Output",
 CellTags->"FunctionalD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[959976, 37699, 164, 5, 26, "Input",
 CellTags->"FunctionalD"],
Cell[960143, 37706, 7809, 271, 480, "Output",
 CellTags->"FunctionalD"]
}, Open  ]],
Cell[967967, 37980, 212, 8, 24, "Text",
 CellTags->"FunctionalD"],
Cell[CellGroupData[{
Cell[968204, 37992, 315, 11, 26, "Input",
 CellTags->"FunctionalD"],
Cell[968522, 38005, 2778, 120, 44, "Output",
 CellTags->"FunctionalD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[971337, 38130, 647, 20, 26, "Input",
 CellTags->"FunctionalD"],
Cell[971987, 38152, 1352, 61, 32, "Output",
 CellTags->"FunctionalD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[973376, 38218, 154, 4, 26, "Input",
 CellTags->"FunctionalD"],
Cell[973533, 38224, 6504, 213, 466, "Output",
 CellTags->"FunctionalD"]
}, Open  ]],
Cell[980052, 38440, 131, 4, 24, "Text",
 CellTags->"FunctionalD"],
Cell[CellGroupData[{
Cell[980208, 38448, 79, 2, 26, "Input",
 CellTags->"FunctionalD"],
Cell[980290, 38452, 1340, 46, 91, "Output",
 CellTags->"FunctionalD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[981667, 38503, 798, 21, 65, "Input",
 CellTags->"FunctionalD"],
Cell[982468, 38526, 1104, 42, 34, "Output",
 CellTags->"FunctionalD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[983609, 38573, 137, 4, 26, "Input",
 CellTags->"FunctionalD"],
Cell[983749, 38579, 4950, 160, 309, "Output",
 CellTags->"FunctionalD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[988736, 38744, 204, 6, 26, "Input",
 CellTags->"FunctionalD"],
Cell[988943, 38752, 7513, 249, 533, "Output",
 CellTags->"FunctionalD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[996493, 39006, 138, 4, 26, "Input",
 CellTags->"FunctionalD"],
Cell[996634, 39012, 13810, 456, 967, "Output",
 CellTags->"FunctionalD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1010481, 39473, 163, 5, 26, "Input",
 CellTags->"FunctionalD"],
Cell[1010647, 39480, 12753, 420, 901, "Output",
 CellTags->"FunctionalD"]
}, Open  ]],
Cell[1023415, 39903, 66, 1, 24, "Text",
 CellTags->"FunctionalD"],
Cell[1023484, 39906, 539, 13, 46, "Input",
 CellTags->"FunctionalD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1024060, 39924, 110, 2, 36, "Subsubsection",
 CellTags->"FunctionalD"],
Cell[1024173, 39928, 79, 1, 24, "Text",
 CellTags->"FunctionalD"],
Cell[CellGroupData[{
Cell[1024277, 39933, 1073, 26, 84, "Input"],
Cell[1025353, 39961, 1532, 56, 108, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1026922, 40022, 765, 22, 46, "Input",
 CellTags->"FunctionalD"],
Cell[1027690, 40046, 8939, 330, 119, "Output",
 CellTags->"FunctionalD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1036666, 40381, 165, 5, 26, "Input",
 CellTags->"FunctionalD"],
Cell[1036834, 40388, 2139, 85, 44, "Output",
 CellTags->"FunctionalD"]
}, Open  ]]
}, Open  ]],
Cell[1039000, 40477, 41, 1, 41, "ObjectName",
 CellTags->"FV"],
Cell[CellGroupData[{
Cell[1039066, 40482, 53, 1, 36, "Subsubsection",
 CellTags->"FV"],
Cell[1039122, 40485, 275, 13, 25, "Text",
 CellTags->"FV"],
Cell[1039400, 40500, 562, 29, 25, "Text",
 CellTags->"FV"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1039999, 40534, 50, 1, 34, "Subsubsection",
 CellTags->"FV"],
Cell[CellGroupData[{
Cell[1040074, 40539, 124, 4, 26, "Input",
 CellTags->"FV"],
Cell[1040201, 40545, 168, 5, 28, "Output",
 CellTags->"FV"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1040406, 40555, 148, 5, 26, "Input",
 CellTags->"FV"],
Cell[1040557, 40562, 229, 8, 28, "Output",
 CellTags->"FV"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1040823, 40575, 160, 5, 26, "Input",
 CellTags->"FV"],
Cell[1040986, 40582, 181, 5, 38, "Output",
 CellTags->"FV"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1041204, 40592, 195, 7, 26, "Input",
 CellTags->"FV"],
Cell[1041402, 40601, 259, 7, 38, "Output",
 CellTags->"FV"]
}, Open  ]],
Cell[1041676, 40611, 130, 4, 24, "Text",
 CellTags->"FV"],
Cell[CellGroupData[{
Cell[1041831, 40619, 196, 6, 26, "Input",
 CellTags->"FV"],
Cell[1042030, 40627, 227, 8, 28, "Output",
 CellTags->"FV"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1042294, 40640, 107, 3, 26, "Input",
 CellTags->"FV"],
Cell[1042404, 40645, 205, 6, 38, "Output",
 CellTags->"FV"]
}, Open  ]]
}, Open  ]],
Cell[1042636, 40655, 43, 1, 41, "ObjectName",
 CellTags->"FVD"],
Cell[CellGroupData[{
Cell[1042704, 40660, 54, 1, 36, "Subsubsection",
 CellTags->"FVD"],
Cell[1042761, 40663, 312, 15, 25, "Text",
 CellTags->"FVD"],
Cell[1043076, 40680, 560, 29, 25, "Text",
 CellTags->"FVD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1043673, 40714, 51, 1, 34, "Subsubsection",
 CellTags->"FVD"],
Cell[CellGroupData[{
Cell[1043749, 40719, 126, 4, 26, "Input",
 CellTags->"FVD"],
Cell[1043878, 40725, 169, 5, 28, "Output",
 CellTags->"FVD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1044084, 40735, 150, 5, 26, "Input",
 CellTags->"FVD"],
Cell[1044237, 40742, 230, 8, 28, "Output",
 CellTags->"FVD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1044504, 40755, 162, 5, 26, "Input",
 CellTags->"FVD"],
Cell[1044669, 40762, 183, 5, 38, "Output",
 CellTags->"FVD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1044889, 40772, 197, 7, 26, "Input",
 CellTags->"FVD"],
Cell[1045089, 40781, 310, 9, 38, "Output",
 CellTags->"FVD"]
}, Open  ]],
Cell[1045414, 40793, 88, 1, 24, "Text",
 CellTags->"FVD"],
Cell[CellGroupData[{
Cell[1045527, 40798, 198, 6, 26, "Input",
 CellTags->"FVD"],
Cell[1045728, 40806, 230, 8, 28, "Output",
 CellTags->"FVD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1045995, 40819, 108, 3, 26, "Input",
 CellTags->"FVD"],
Cell[1046106, 40824, 207, 6, 38, "Output",
 CellTags->"FVD"]
}, Open  ]]
}, Open  ]],
Cell[1046340, 40834, 41, 1, 41, "ObjectName",
 CellTags->"GA"],
Cell[CellGroupData[{
Cell[1046406, 40839, 53, 1, 36, "Subsubsection",
 CellTags->"GA"],
Cell[1046462, 40842, 654, 24, 43, "Text",
 CellTags->"GA"],
Cell[1047119, 40868, 369, 19, 25, "Text",
 CellTags->"GA"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1047525, 40892, 50, 1, 34, "Subsubsection",
 CellTags->"GA"],
Cell[CellGroupData[{
Cell[1047600, 40897, 101, 3, 26, "Input",
 CellTags->"GA"],
Cell[1047704, 40902, 177, 5, 26, "Output",
 CellTags->"GA"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1047918, 40912, 213, 7, 26, "Input",
 CellTags->"GA"],
Cell[1048134, 40921, 359, 11, 26, "Output",
 CellTags->"GA"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1048530, 40937, 171, 5, 26, "Input",
 CellTags->"GA"],
Cell[1048704, 40944, 205, 5, 38, "Output",
 CellTags->"GA"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1048946, 40954, 163, 5, 26, "Input",
 CellTags->"GA"],
Cell[1049112, 40961, 331, 9, 26, "Output",
 CellTags->"GA"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1049480, 40975, 203, 6, 26, "Input",
 CellTags->"GA"],
Cell[1049686, 40983, 306, 8, 38, "Output",
 CellTags->"GA"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1050029, 40996, 250, 8, 26, "Input",
 CellTags->"GA"],
Cell[1050282, 41006, 343, 10, 28, "Output",
 CellTags->"GA"]
}, Open  ]]
}, Open  ]],
Cell[1050652, 41020, 43, 1, 41, "ObjectName",
 CellTags->"GAD"],
Cell[CellGroupData[{
Cell[1050720, 41025, 54, 1, 36, "Subsubsection",
 CellTags->"GAD"],
Cell[1050777, 41028, 671, 24, 43, "Text",
 CellTags->"GAD"],
Cell[1051451, 41054, 367, 19, 25, "Text",
 CellTags->"GAD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1051855, 41078, 51, 1, 34, "Subsubsection",
 CellTags->"GAD"],
Cell[CellGroupData[{
Cell[1051931, 41083, 103, 3, 26, "Input",
 CellTags->"GAD"],
Cell[1052037, 41088, 176, 5, 26, "Output",
 CellTags->"GAD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1052250, 41098, 216, 7, 26, "Input",
 CellTags->"GAD"],
Cell[1052469, 41107, 360, 11, 26, "Output",
 CellTags->"GAD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1052866, 41123, 173, 5, 26, "Input",
 CellTags->"GAD"],
Cell[1053042, 41130, 255, 7, 38, "Output",
 CellTags->"GAD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1053334, 41142, 165, 5, 26, "Input",
 CellTags->"GAD"],
Cell[1053502, 41149, 332, 9, 26, "Output",
 CellTags->"GAD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1053871, 41163, 205, 6, 26, "Input",
 CellTags->"GAD"],
Cell[1054079, 41171, 311, 8, 38, "Output",
 CellTags->"GAD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1054427, 41184, 254, 8, 26, "Input",
 CellTags->"GAD"],
Cell[1054684, 41194, 344, 10, 28, "Output",
 CellTags->"GAD"]
}, Open  ]]
}, Open  ]],
Cell[1055055, 41208, 61, 1, 39, "ObjectName",
 CellTags->"GammaEpsilon"],
Cell[CellGroupData[{
Cell[1055141, 41213, 63, 1, 34, "Subsubsection",
 CellTags->"GammaEpsilon"],
Cell[1055207, 41216, 167, 4, 24, "Text",
 CellTags->"GammaEpsilon"],
Cell[1055377, 41222, 299, 14, 24, "Text",
 CellTags->"GammaEpsilon"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1055713, 41241, 60, 1, 34, "Subsubsection",
 CellTags->"GammaEpsilon"],
Cell[1055776, 41244, 141, 4, 24, "Text",
 CellTags->"GammaEpsilon"],
Cell[CellGroupData[{
Cell[1055942, 41252, 170, 5, 26, "Input",
 CellTags->"GammaEpsilon"],
Cell[1056115, 41259, 1403, 49, 48, "Output",
 CellTags->"GammaEpsilon"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1057555, 41313, 170, 5, 26, "Input",
 CellTags->"GammaEpsilon"],
Cell[1057728, 41320, 1151, 42, 48, "Output",
 CellTags->"GammaEpsilon"]
}, Open  ]],
Cell[1058894, 41365, 91, 1, 24, "Text",
 CellTags->"GammaEpsilon"],
Cell[CellGroupData[{
Cell[1059010, 41370, 123, 3, 26, "Input",
 CellTags->"GammaEpsilon"],
Cell[1059136, 41375, 2192, 78, 96, "Output",
 CellTags->"GammaEpsilon"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1061365, 41458, 117, 3, 26, "Input",
 CellTags->"GammaEpsilon"],
Cell[1061485, 41463, 180, 6, 26, "Output",
 CellTags->"GammaEpsilon"]
}, Open  ]]
}, Open  ]],
Cell[1061692, 41473, 59, 1, 39, "ObjectName",
 CellTags->"GammaExpand"],
Cell[CellGroupData[{
Cell[1061776, 41478, 62, 1, 34, "Subsubsection",
 CellTags->"GammaExpand"],
Cell[1061841, 41481, 123, 3, 24, "Text",
 CellTags->"GammaExpand"],
Cell[1061967, 41486, 194, 9, 24, "Text",
 CellTags->"GammaExpand"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1062198, 41500, 59, 1, 34, "Subsubsection",
 CellTags->"GammaExpand"],
Cell[CellGroupData[{
Cell[1062282, 41505, 187, 5, 26, "Input",
 CellTags->"GammaExpand"],
Cell[1062472, 41512, 371, 12, 26, "Output",
 CellTags->"GammaExpand"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1062880, 41529, 197, 6, 26, "Input",
 CellTags->"GammaExpand"],
Cell[1063080, 41537, 674, 23, 47, "Output",
 CellTags->"GammaExpand"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1063791, 41565, 187, 5, 26, "Input",
 CellTags->"GammaExpand"],
Cell[1063981, 41572, 245, 7, 26, "Output",
 CellTags->"GammaExpand"]
}, Open  ]]
}, Open  ]],
Cell[1064253, 41583, 49, 1, 39, "ObjectName",
 CellTags->"Gamma1"],
Cell[CellGroupData[{
Cell[1064327, 41588, 57, 1, 34, "Subsubsection",
 CellTags->"Gamma1"],
Cell[1064387, 41591, 139, 4, 24, "Text",
 CellTags->"Gamma1"],
Cell[1064529, 41597, 275, 14, 24, "Text",
 CellTags->"Gamma1"]
}, Open  ]],
Cell[1064819, 41614, 49, 1, 39, "ObjectName",
 CellTags->"Gamma2"],
Cell[CellGroupData[{
Cell[1064893, 41619, 57, 1, 34, "Subsubsection",
 CellTags->"Gamma2"],
Cell[1064953, 41622, 172, 4, 24, "Text",
 CellTags->"Gamma2"],
Cell[1065128, 41628, 275, 14, 24, "Text",
 CellTags->"Gamma2"]
}, Open  ]],
Cell[1065418, 41645, 49, 1, 39, "ObjectName",
 CellTags->"Gamma3"],
Cell[CellGroupData[{
Cell[1065492, 41650, 57, 1, 34, "Subsubsection",
 CellTags->"Gamma3"],
Cell[1065552, 41653, 237, 5, 41, "Text",
 CellTags->"Gamma3"],
Cell[1065792, 41660, 275, 14, 24, "Text",
 CellTags->"Gamma3"]
}, Open  ]],
Cell[1066082, 41677, 47, 1, 39, "ObjectName",
 CellTags->"Gauge"],
Cell[CellGroupData[{
Cell[1066154, 41682, 56, 1, 34, "Subsubsection",
 CellTags->"Gauge"],
Cell[1066213, 41685, 133, 4, 24, "Text",
 CellTags->"Gauge"],
Cell[1066349, 41691, 301, 14, 24, "Text",
 CellTags->"Gauge"]
}, Open  ]],
Cell[1066665, 41708, 57, 1, 39, "ObjectName",
 CellTags->"GaugeField"],
Cell[CellGroupData[{
Cell[1066747, 41713, 61, 1, 34, "Subsubsection",
 CellTags->"GaugeField"],
Cell[1066811, 41716, 212, 5, 41, "Text",
 CellTags->"GaugeField"],
Cell[1067026, 41723, 318, 14, 24, "Text",
 CellTags->"GaugeField"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1067381, 41742, 58, 1, 34, "Subsubsection",
 CellTags->"GaugeField"],
Cell[CellGroupData[{
Cell[1067464, 41747, 86, 2, 26, "Input",
 CellTags->"GaugeField"],
Cell[1067553, 41751, 148, 4, 27, "Output",
 CellTags->"GaugeField"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1067738, 41760, 237, 6, 26, "Input",
 CellTags->"GaugeField"],
Cell[1067978, 41768, 358, 15, 30, "Output",
 CellTags->"GaugeField"]
}, Open  ]]
}, Open  ]],
Cell[1068363, 41787, 51, 1, 39, "ObjectName",
 CellTags->"GaugeXi"],
Cell[CellGroupData[{
Cell[1068439, 41792, 58, 1, 34, "Subsubsection",
 CellTags->"GaugeXi"],
Cell[1068500, 41795, 77, 1, 24, "Text",
 CellTags->"GaugeXi"],
Cell[1068580, 41798, 285, 14, 24, "Text",
 CellTags->"GaugeXi"]
}, Open  ]],
Cell[1068880, 41815, 43, 1, 39, "ObjectName",
 CellTags->"GA5"],
Cell[CellGroupData[{
Cell[1068948, 41820, 54, 1, 34, "Subsubsection",
 CellTags->"GA5"],
Cell[1069005, 41823, 88, 1, 24, "Text",
 CellTags->"GA5"],
Cell[1069096, 41826, 364, 19, 24, "Text",
 CellTags->"GA5"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1069497, 41850, 51, 1, 34, "Subsubsection",
 CellTags->"GA5"],
Cell[CellGroupData[{
Cell[1069573, 41855, 72, 2, 26, "Input",
 CellTags->"GA5"],
Cell[1069648, 41859, 174, 5, 26, "Output",
 CellTags->"GA5"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1069859, 41869, 94, 3, 26, "Input",
 CellTags->"GA5"],
Cell[1069956, 41874, 143, 4, 26, "Output",
 CellTags->"GA5"]
}, Open  ]]
}, Open  ]],
Cell[1070126, 41882, 43, 1, 39, "ObjectName",
 CellTags->"GGV"],
Cell[CellGroupData[{
Cell[1070194, 41887, 54, 1, 34, "Subsubsection",
 CellTags->"GGV"],
Cell[1070251, 41890, 72, 1, 24, "Text",
 CellTags->"GGV"],
Cell[1070326, 41893, 198, 9, 24, "Text",
 CellTags->"GGV"]
}, Open  ]],
Cell[1070539, 41905, 67, 1, 39, "ObjectName",
 CellTags->"GhostPropagator"],
Cell[CellGroupData[{
Cell[1070631, 41910, 66, 1, 34, "Subsubsection",
 CellTags->"GhostPropagator"],
Cell[1070700, 41913, 324, 9, 26, "Text",
 CellTags->"GhostPropagator"],
Cell[1071027, 41924, 436, 19, 24, "Text",
 CellTags->"GhostPropagator"],
Cell[CellGroupData[{
Cell[1071488, 41947, 99, 2, 26, "Input"],
Cell[1071590, 41951, 189, 6, 26, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[1071828, 41963, 63, 1, 34, "Subsubsection",
 CellTags->"GhostPropagator"],
Cell[CellGroupData[{
Cell[1071916, 41968, 156, 4, 26, "Input",
 CellTags->"GhostPropagator"],
Cell[1072075, 41974, 234, 7, 28, "Output",
 CellTags->"GhostPropagator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1072346, 41986, 206, 5, 26, "Input",
 CellTags->"GhostPropagator"],
Cell[1072555, 41993, 451, 17, 48, "Output",
 CellTags->"GhostPropagator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1073043, 42015, 123, 3, 26, "Input",
 CellTags->"GhostPropagator"],
Cell[1073169, 42020, 220, 7, 28, "Output",
 CellTags->"GhostPropagator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1073426, 42032, 188, 5, 26, "Input",
 CellTags->"GhostPropagator"],
Cell[1073617, 42039, 455, 17, 49, "Output",
 CellTags->"GhostPropagator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1074109, 42061, 285, 9, 26, "Input",
 CellTags->"GhostPropagator"],
Cell[1074397, 42072, 270, 7, 38, "Output",
 CellTags->"GhostPropagator"]
}, Open  ]]
}, Open  ]],
Cell[1074694, 42083, 43, 1, 39, "ObjectName",
 CellTags->"GHP"],
Cell[CellGroupData[{
Cell[1074762, 42088, 54, 1, 34, "Subsubsection",
 CellTags->"GHP"],
Cell[1074819, 42091, 288, 9, 26, "Text",
 CellTags->"GHP"],
Cell[1075110, 42102, 460, 19, 24, "Text",
 CellTags->"GHP"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1075607, 42126, 51, 1, 34, "Subsubsection",
 CellTags->"GHP"],
Cell[CellGroupData[{
Cell[1075683, 42131, 132, 4, 26, "Input",
 CellTags->"GHP"],
Cell[1075818, 42137, 223, 7, 28, "Output",
 CellTags->"GHP"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1076078, 42149, 130, 4, 26, "Input",
 CellTags->"GHP"],
Cell[1076211, 42155, 199, 6, 49, "Output",
 CellTags->"GHP"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1076447, 42166, 132, 4, 26, "Input",
 CellTags->"GHP"],
Cell[1076582, 42172, 227, 7, 28, "Output",
 CellTags->"GHP"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1076846, 42184, 264, 9, 26, "Input",
 CellTags->"GHP"],
Cell[1077113, 42195, 260, 7, 38, "Output",
 CellTags->"GHP"]
}, Open  ]]
}, Open  ]],
Cell[1077400, 42206, 57, 1, 41, "ObjectName",
 CellTags->"GluonField"],
Cell[CellGroupData[{
Cell[1077482, 42211, 61, 1, 36, "Subsubsection",
 CellTags->"GluonField"],
Cell[1077546, 42214, 79, 1, 25, "Text",
 CellTags->"GluonField"],
Cell[1077628, 42217, 187, 9, 25, "Text",
 CellTags->"GluonField"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1077852, 42231, 58, 1, 34, "Subsubsection",
 CellTags->"GluonField"],
Cell[CellGroupData[{
Cell[1077935, 42236, 86, 2, 26, "Input",
 CellTags->"GluonField"],
Cell[1078024, 42240, 147, 4, 27, "Output",
 CellTags->"GluonField"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1078208, 42249, 237, 6, 26, "Input",
 CellTags->"GluonField"],
Cell[1078448, 42257, 360, 15, 30, "Output",
 CellTags->"GluonField"]
}, Open  ]]
}, Open  ]],
Cell[1078835, 42276, 69, 1, 41, "ObjectName",
 CellTags->"GluonGhostVertex"],
Cell[CellGroupData[{
Cell[1078929, 42281, 67, 1, 36, "Subsubsection",
 CellTags->"GluonGhostVertex"],
Cell[1078999, 42284, 422, 7, 61, "Text",
 CellTags->"GluonGhostVertex"],
Cell[1079424, 42293, 583, 23, 24, "Text",
 CellTags->"GluonGhostVertex"],
Cell[CellGroupData[{
Cell[1080032, 42320, 131, 3, 26, "Input",
 CellTags->"GluonGhostVertex"],
Cell[1080166, 42325, 368, 11, 26, "Output",
 CellTags->"GluonGhostVertex"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[1080583, 42342, 64, 1, 34, "Subsubsection",
 CellTags->"GluonGhostVertex"],
Cell[CellGroupData[{
Cell[1080672, 42347, 348, 10, 26, "Input",
 CellTags->"GluonGhostVertex"],
Cell[1081023, 42359, 558, 22, 30, "Output",
 CellTags->"GluonGhostVertex"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1081618, 42386, 86, 2, 26, "Input"],
Cell[1081707, 42390, 618, 27, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1082362, 42422, 121, 3, 26, "Input"],
Cell[1082486, 42427, 225, 7, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1082748, 42439, 86, 2, 26, "Input"],
Cell[1082837, 42443, 328, 13, 26, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1083202, 42461, 148, 4, 26, "Input",
 CellTags->"GluonGhostVertex"],
Cell[1083353, 42467, 377, 11, 38, "Output",
 CellTags->"GluonGhostVertex"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1083767, 42483, 191, 5, 26, "Input",
 CellTags->"GluonGhostVertex"],
Cell[1083961, 42490, 563, 22, 31, "Output",
 CellTags->"GluonGhostVertex"]
}, Open  ]]
}, Open  ]],
Cell[1084551, 42516, 67, 1, 41, "ObjectName",
 CellTags->"GluonPropagator"],
Cell[CellGroupData[{
Cell[1084643, 42521, 66, 1, 36, "Subsubsection",
 CellTags->"GluonPropagator"],
Cell[1084712, 42524, 535, 10, 70, "Text",
 CellTags->"GluonPropagator"],
Cell[1085250, 42536, 99, 1, 24, "Text",
 CellTags->"GluonPropagator"],
Cell[CellGroupData[{
Cell[1085374, 42541, 83, 2, 26, "Input",
 CellTags->"GluonPropagator"],
Cell[1085460, 42545, 167, 4, 26, "Output",
 CellTags->"GluonPropagator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1085664, 42554, 129, 3, 26, "Input",
 CellTags->"GluonPropagator"],
Cell[1085796, 42559, 524, 14, 26, "Output",
 CellTags->"GluonPropagator"]
}, Open  ]],
Cell[1086335, 42576, 591, 24, 70, "Text",
 CellTags->"GluonPropagator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1086963, 42605, 72, 1, 33, "Subsubsection",
 CellTags->"GluonPropagator"],
Cell[CellGroupData[{
Cell[1087060, 42610, 187, 5, 26, "Input",
 CellTags->"GluonPropagator"],
Cell[1087250, 42617, 275, 9, 29, "Output",
 CellTags->"GluonPropagator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1087562, 42631, 189, 5, 26, "Input"],
Cell[1087754, 42638, 519, 19, 48, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1088310, 42662, 271, 7, 26, "Input",
 CellTags->"GluonPropagator"],
Cell[1088584, 42671, 1075, 40, 61, "Output",
 CellTags->"GluonPropagator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1089696, 42716, 357, 10, 26, "Input",
 CellTags->"GluonPropagator"],
Cell[1090056, 42728, 2792, 105, 61, "Output",
 CellTags->"GluonPropagator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1092885, 42838, 199, 6, 26, "Input",
 CellTags->"GluonPropagator"],
Cell[1093087, 42846, 312, 10, 49, "Output",
 CellTags->"GluonPropagator"]
}, Open  ]],
Cell[1093414, 42859, 149, 4, 24, "Text",
 CellTags->"GluonPropagator"],
Cell[CellGroupData[{
Cell[1093588, 42867, 242, 7, 26, "Input",
 CellTags->"GluonPropagator"],
Cell[1093833, 42876, 1072, 40, 62, "Output",
 CellTags->"GluonPropagator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1094942, 42921, 267, 9, 26, "Input",
 CellTags->"GluonPropagator"],
Cell[1095212, 42932, 358, 10, 38, "Output",
 CellTags->"GluonPropagator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1095607, 42947, 275, 7, 26, "Input",
 CellTags->"GluonPropagator"],
Cell[1095885, 42956, 1316, 47, 55, "Output",
 CellTags->"GluonPropagator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1097238, 43008, 275, 7, 26, "Input",
 CellTags->"GluonPropagator"],
Cell[1097516, 43017, 1332, 48, 52, "Output",
 CellTags->"GluonPropagator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1098885, 43070, 275, 7, 26, "Input",
 CellTags->"GluonPropagator"],
Cell[1099163, 43079, 1323, 47, 55, "Output",
 CellTags->"GluonPropagator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1100523, 43131, 275, 7, 26, "Input",
 CellTags->"GluonPropagator"],
Cell[1100801, 43140, 1318, 47, 55, "Output",
 CellTags->"GluonPropagator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1102156, 43192, 275, 7, 26, "Input",
 CellTags->"GluonPropagator"],
Cell[1102434, 43201, 2464, 89, 55, "Output",
 CellTags->"GluonPropagator"]
}, Open  ]]
}, Open  ]],
Cell[1104925, 43294, 67, 1, 39, "ObjectName",
 CellTags->"GluonSelfEnergy"],
Cell[CellGroupData[{
Cell[1105017, 43299, 66, 1, 34, "Subsubsection",
 CellTags->"GluonSelfEnergy"],
Cell[1105086, 43302, 122, 3, 24, "Text",
 CellTags->"GluonSelfEnergy"],
Cell[CellGroupData[{
Cell[1105233, 43309, 129, 3, 26, "Input",
 CellTags->"GluonSelfEnergy"],
Cell[1105365, 43314, 835, 24, 52, "Output",
 CellTags->"GluonSelfEnergy"]
}, Open  ]],
Cell[1106215, 43341, 591, 24, 24, "Text",
 CellTags->"GluonSelfEnergy"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1106843, 43370, 72, 1, 33, "Subsubsection",
 CellTags->"GluonSelfEnergy"],
Cell[CellGroupData[{
Cell[1106940, 43375, 252, 8, 26, "Input",
 CellTags->"GluonSelfEnergy"],
Cell[1107195, 43385, 2433, 93, 45, "Output",
 CellTags->"GluonSelfEnergy"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1109665, 43483, 300, 9, 26, "Input",
 CellTags->"GluonSelfEnergy"],
Cell[1109968, 43494, 2776, 104, 101, "Output",
 CellTags->"GluonSelfEnergy"]
}, Open  ]]
}, Open  ]],
Cell[1112771, 43602, 59, 1, 39, "ObjectName",
 CellTags->"GluonVertex"],
Cell[CellGroupData[{
Cell[1112855, 43607, 62, 1, 34, "Subsubsection",
 CellTags->"GluonVertex"],
Cell[1112920, 43610, 88, 1, 24, "Text",
 CellTags->"GluonVertex"],
Cell[1113011, 43613, 621, 10, 75, "Text",
 CellTags->"GluonVertex"],
Cell[1113635, 43625, 91, 1, 24, "Text",
 CellTags->"GluonVertex"],
Cell[CellGroupData[{
Cell[1113751, 43630, 79, 2, 26, "Input",
 CellTags->"GluonVertex"],
Cell[1113833, 43634, 157, 4, 26, "Output",
 CellTags->"GluonVertex"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1114027, 43643, 121, 3, 26, "Input",
 CellTags->"GluonVertex"],
Cell[1114151, 43648, 419, 12, 26, "Output",
 CellTags->"GluonVertex"]
}, Open  ]],
Cell[1114585, 43663, 337, 14, 24, "Text",
 CellTags->"GluonVertex"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1114959, 43682, 59, 1, 34, "Subsubsection",
 CellTags->"GluonVertex"],
Cell[CellGroupData[{
Cell[1115043, 43687, 338, 10, 26, "Input",
 CellTags->"GluonVertex"],
Cell[1115384, 43699, 987, 43, 28, "Output",
 CellTags->"GluonVertex"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1116408, 43747, 379, 12, 26, "Input",
 CellTags->"GluonVertex"],
Cell[1116790, 43761, 2016, 80, 28, "Output",
 CellTags->"GluonVertex"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1118843, 43846, 308, 10, 26, "Input",
 CellTags->"GluonVertex"],
Cell[1119154, 43858, 314, 10, 28, "Output",
 CellTags->"GluonVertex"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1119505, 43873, 349, 12, 26, "Input",
 CellTags->"GluonVertex"],
Cell[1119857, 43887, 1724, 66, 28, "Output",
 CellTags->"GluonVertex"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1121618, 43958, 414, 12, 26, "Input",
 CellTags->"GluonVertex"],
Cell[1122035, 43972, 394, 12, 29, "Output",
 CellTags->"GluonVertex"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1122466, 43989, 431, 13, 26, "Input"],
Cell[1122900, 44004, 3495, 132, 50, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1126432, 44141, 374, 12, 26, "Input",
 CellTags->"GluonVertex"],
Cell[1126809, 44155, 202, 6, 28, "Output",
 CellTags->"GluonVertex"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1127048, 44166, 259, 7, 26, "Input",
 CellTags->"GluonVertex"],
Cell[1127310, 44175, 3523, 133, 50, "Output",
 CellTags->"GluonVertex"]
}, Open  ]],
Cell[1130848, 44311, 200, 4, 41, "Text",
 CellTags->"GluonVertex"],
Cell[CellGroupData[{
Cell[1131073, 44319, 172, 5, 26, "Input",
 CellTags->"GluonVertex"],
Cell[1131248, 44326, 984, 43, 30, "Output",
 CellTags->"GluonVertex"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1132269, 44374, 230, 7, 26, "Input",
 CellTags->"GluonVertex"],
Cell[1132502, 44383, 3472, 133, 55, "Output",
 CellTags->"GluonVertex"]
}, Open  ]]
}, Open  ]],
Cell[1136001, 44520, 41, 1, 39, "ObjectName",
 CellTags->"GO"],
Cell[CellGroupData[{
Cell[1136067, 44525, 53, 1, 34, "Subsubsection",
 CellTags->"GO"],
Cell[1136123, 44528, 73, 1, 24, "Text",
 CellTags->"GO"],
Cell[1136199, 44531, 206, 9, 24, "Text",
 CellTags->"GO"]
}, Open  ]],
Cell[1136420, 44543, 41, 1, 39, "ObjectName",
 CellTags->"GP"],
Cell[CellGroupData[{
Cell[1136486, 44548, 53, 1, 34, "Subsubsection",
 CellTags->"GP"],
Cell[1136542, 44551, 69, 1, 24, "Text",
 CellTags->"GP"],
Cell[1136614, 44554, 194, 9, 24, "Text",
 CellTags->"GP"]
}, Open  ]],
Cell[1136823, 44566, 67, 1, 39, "ObjectName",
 CellTags->"GrassmannParity"],
Cell[CellGroupData[{
Cell[1136915, 44571, 66, 1, 34, "Subsubsection",
 CellTags->"GrassmannParity"],
Cell[1136984, 44574, 216, 5, 41, "Text",
 CellTags->"GrassmannParity"],
Cell[1137203, 44581, 186, 9, 24, "Text",
 CellTags->"GrassmannParity"]
}, Open  ]],
Cell[1137404, 44593, 41, 1, 39, "ObjectName",
 CellTags->"GS"],
Cell[CellGroupData[{
Cell[1137470, 44598, 53, 1, 34, "Subsubsection",
 CellTags->"GS"],
Cell[1137526, 44601, 635, 18, 44, "Text",
 CellTags->"GS"],
Cell[1138164, 44621, 482, 24, 24, "Text",
 CellTags->"GS"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1138683, 44650, 50, 1, 34, "Subsubsection",
 CellTags->"GS"],
Cell[CellGroupData[{
Cell[1138758, 44655, 97, 3, 26, "Input",
 CellTags->"GS"],
Cell[1138858, 44660, 183, 5, 28, "Output",
 CellTags->"GS"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1139078, 44670, 162, 6, 26, "Input",
 CellTags->"GS"],
Cell[1139243, 44678, 197, 5, 38, "Output",
 CellTags->"GS"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1139477, 44688, 140, 4, 26, "Input",
 CellTags->"GS"],
Cell[1139620, 44694, 454, 14, 28, "Output",
 CellTags->"GS"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1140111, 44713, 179, 6, 26, "Input",
 CellTags->"GS"],
Cell[1140293, 44721, 284, 8, 38, "Output",
 CellTags->"GS"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1140614, 44734, 237, 8, 26, "Input",
 CellTags->"GS"],
Cell[1140854, 44744, 405, 13, 28, "Output",
 CellTags->"GS"]
}, Open  ]]
}, Open  ]],
Cell[1141286, 44761, 43, 1, 39, "ObjectName",
 CellTags->"GSD"],
Cell[CellGroupData[{
Cell[1141354, 44766, 54, 1, 34, "Subsubsection",
 CellTags->"GSD"],
Cell[1141411, 44769, 644, 18, 44, "Text",
 CellTags->"GSD"],
Cell[1142058, 44789, 483, 24, 24, "Text",
 CellTags->"GSD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1142578, 44818, 51, 1, 34, "Subsubsection",
 CellTags->"GSD"],
Cell[CellGroupData[{
Cell[1142654, 44823, 99, 3, 26, "Input",
 CellTags->"GSD"],
Cell[1142756, 44828, 182, 5, 28, "Output",
 CellTags->"GSD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1142975, 44838, 164, 6, 26, "Input",
 CellTags->"GSD"],
Cell[1143142, 44846, 249, 7, 38, "Output",
 CellTags->"GSD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1143428, 44858, 142, 4, 26, "Input",
 CellTags->"GSD"],
Cell[1143573, 44864, 455, 14, 28, "Output",
 CellTags->"GSD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1144065, 44883, 181, 6, 26, "Input",
 CellTags->"GSD"],
Cell[1144249, 44891, 289, 8, 38, "Output",
 CellTags->"GSD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1144575, 44904, 241, 8, 26, "Input",
 CellTags->"GSD"],
Cell[1144819, 44914, 406, 13, 28, "Output",
 CellTags->"GSD"]
}, Open  ]]
}, Open  ]],
Cell[1145252, 44931, 51, 1, 39, "ObjectName",
 CellTags->"Gstrong"],
Cell[CellGroupData[{
Cell[1145328, 44936, 58, 1, 34, "Subsubsection",
 CellTags->"Gstrong"],
Cell[1145389, 44939, 83, 1, 24, "Text",
 CellTags->"Gstrong"],
Cell[1145475, 44942, 334, 15, 24, "Text",
 CellTags->"Gstrong"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1145846, 44962, 55, 1, 34, "Subsubsection",
 CellTags->"Gstrong"],
Cell[1145904, 44965, 119, 3, 24, "Text",
 CellTags->"Gstrong"],
Cell[CellGroupData[{
Cell[1146048, 44972, 80, 2, 26, "Input",
 CellTags->"Gstrong"],
Cell[1146131, 44976, 167, 5, 26, "Output",
 CellTags->"Gstrong"]
}, Open  ]]
}, Open  ]],
Cell[1146325, 44985, 43, 1, 39, "ObjectName",
 CellTags->"GTI"],
Cell[CellGroupData[{
Cell[1146393, 44990, 54, 1, 34, "Subsubsection",
 CellTags->"GTI"],
Cell[1146450, 44993, 85, 1, 24, "Text",
 CellTags->"GTI"],
Cell[1146538, 44996, 159, 9, 24, "Text",
 CellTags->"GTI"]
}, Open  ]],
Cell[1146712, 45008, 41, 1, 39, "ObjectName",
 CellTags->"GV"],
Cell[CellGroupData[{
Cell[1146778, 45013, 53, 1, 34, "Subsubsection",
 CellTags->"GV"],
Cell[1146834, 45016, 65, 1, 24, "Text",
 CellTags->"GV"],
Cell[1146902, 45019, 182, 9, 24, "Text",
 CellTags->"GV"]
}, Open  ]],
Cell[1147099, 45031, 45, 1, 39, "ObjectName",
 CellTags->"Hill"],
Cell[CellGroupData[{
Cell[1147169, 45036, 55, 1, 34, "Subsubsection",
 CellTags->"Hill"],
Cell[1147227, 45039, 129, 4, 24, "Text",
 CellTags->"Hill"],
Cell[1147359, 45045, 196, 9, 24, "Text",
 CellTags->"Hill"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1147592, 45059, 52, 1, 34, "Subsubsection",
 CellTags->"Hill"],
Cell[CellGroupData[{
Cell[1147669, 45064, 124, 4, 26, "Input",
 CellTags->"Hill"],
Cell[1147796, 45070, 2187, 74, 137, "Output",
 CellTags->"Hill"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1150020, 45149, 270, 8, 26, "Input",
 CellTags->"Hill"],
Cell[1150293, 45159, 142, 4, 26, "Output",
 CellTags->"Hill"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1150472, 45168, 262, 9, 26, "Input",
 CellTags->"Hill"],
Cell[1150737, 45179, 1029, 33, 71, "Output",
 CellTags->"Hill"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1151803, 45217, 273, 9, 26, "Input",
 CellTags->"Hill"],
Cell[1152079, 45228, 142, 4, 26, "Output",
 CellTags->"Hill"]
}, Open  ]]
}, Open  ]],
Cell[1152248, 45236, 69, 1, 39, "ObjectName",
 CellTags->"HypergeometricAC"],
Cell[CellGroupData[{
Cell[1152342, 45241, 67, 1, 34, "Subsubsection",
 CellTags->"HypergeometricAC"],
Cell[1152412, 45244, 553, 9, 75, "Text",
 CellTags->"HypergeometricAC"],
Cell[CellGroupData[{
Cell[1152990, 45257, 131, 3, 26, "Input",
 CellTags->"HypergeometricAC"],
Cell[1153124, 45262, 219, 7, 26, "Output",
 CellTags->"HypergeometricAC"]
}, Open  ]],
Cell[1153358, 45272, 598, 24, 25, "Text",
 CellTags->"HypergeometricAC"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1153993, 45301, 64, 1, 36, "Subsubsection",
 CellTags->"HypergeometricAC"],
Cell[1154060, 45304, 101, 1, 24, "Text",
 CellTags->"HypergeometricAC"],
Cell[CellGroupData[{
Cell[1154186, 45309, 264, 7, 26, "Input",
 CellTags->"HypergeometricAC"],
Cell[1154453, 45318, 1455, 44, 92, "Output",
 CellTags->"HypergeometricAC"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1155945, 45367, 264, 7, 26, "Input",
 CellTags->"HypergeometricAC"],
Cell[1156212, 45376, 1416, 44, 55, "Output",
 CellTags->"HypergeometricAC"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1157665, 45425, 264, 7, 26, "Input",
 CellTags->"HypergeometricAC"],
Cell[1157932, 45434, 1460, 45, 107, "Output",
 CellTags->"HypergeometricAC"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1159429, 45484, 264, 7, 26, "Input",
 CellTags->"HypergeometricAC"],
Cell[1159696, 45493, 1737, 52, 107, "Output",
 CellTags->"HypergeometricAC"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1161470, 45550, 264, 7, 26, "Input",
 CellTags->"HypergeometricAC"],
Cell[1161737, 45559, 457, 14, 42, "Output",
 CellTags->"HypergeometricAC"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1162231, 45578, 265, 7, 26, "Input",
 CellTags->"HypergeometricAC"],
Cell[1162499, 45587, 460, 14, 42, "Output",
 CellTags->"HypergeometricAC"]
}, Open  ]]
}, Open  ]],
Cell[1162986, 45605, 69, 1, 41, "ObjectName",
 CellTags->"HypergeometricIR"],
Cell[CellGroupData[{
Cell[1163080, 45610, 67, 1, 36, "Subsubsection",
 CellTags->"HypergeometricIR"],
Cell[1163150, 45613, 269, 5, 43, "Text",
 CellTags->"HypergeometricIR"],
Cell[1163422, 45620, 479, 19, 24, "Text",
 CellTags->"HypergeometricIR"],
Cell[CellGroupData[{
Cell[1163926, 45643, 131, 3, 26, "Input",
 CellTags->"HypergeometricIR"],
Cell[1164060, 45648, 224, 7, 26, "Output",
 CellTags->"HypergeometricIR"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[1164333, 45661, 64, 1, 34, "Subsubsection",
 CellTags->"HypergeometricIR"],
Cell[CellGroupData[{
Cell[1164422, 45666, 241, 7, 26, "Input",
 CellTags->"HypergeometricIR"],
Cell[1164666, 45675, 681, 25, 50, "Output",
 CellTags->"HypergeometricIR"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1165384, 45705, 394, 14, 26, "Input",
 CellTags->"HypergeometricIR"],
Cell[1165781, 45721, 532, 16, 48, "Output",
 CellTags->"HypergeometricIR"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1166350, 45742, 148, 4, 26, "Input",
 CellTags->"HypergeometricIR"],
Cell[1166501, 45748, 394, 14, 28, "Output",
 CellTags->"HypergeometricIR"]
}, Open  ]]
}, Open  ]],
Cell[1166922, 45766, 69, 1, 41, "ObjectName",
 CellTags->"HypergeometricSE"],
Cell[CellGroupData[{
Cell[1167016, 45771, 67, 1, 36, "Subsubsection",
 CellTags->"HypergeometricSE"],
Cell[1167086, 45774, 420, 13, 41, "Text",
 CellTags->"HypergeometricSE"],
Cell[CellGroupData[{
Cell[1167531, 45791, 131, 3, 26, "Input",
 CellTags->"HypergeometricSE"],
Cell[1167665, 45796, 228, 7, 26, "Output",
 CellTags->"HypergeometricSE"]
}, Open  ]],
Cell[1167908, 45806, 211, 9, 25, "Text",
 CellTags->"HypergeometricSE"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1168156, 45820, 64, 1, 34, "Subsubsection",
 CellTags->"HypergeometricSE"],
Cell[CellGroupData[{
Cell[1168245, 45825, 245, 7, 26, "Input",
 CellTags->"HypergeometricSE"],
Cell[1168493, 45834, 627, 22, 48, "Output",
 CellTags->"HypergeometricSE"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1169157, 45861, 351, 11, 26, "Input",
 CellTags->"HypergeometricSE"],
Cell[1169511, 45874, 851, 30, 48, "Output",
 CellTags->"HypergeometricSE"]
}, Open  ]]
}, Open  ]],
Cell[1170389, 45908, 59, 1, 41, "ObjectName",
 CellTags->"HypExplicit"],
Cell[CellGroupData[{
Cell[1170473, 45913, 62, 1, 36, "Subsubsection",
 CellTags->"HypExplicit"],
Cell[1170538, 45916, 325, 10, 43, "Text",
 CellTags->"HypExplicit"],
Cell[1170866, 45928, 206, 9, 25, "Text",
 CellTags->"HypExplicit"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1171109, 45942, 59, 1, 34, "Subsubsection",
 CellTags->"HypExplicit"],
Cell[CellGroupData[{
Cell[1171193, 45947, 164, 4, 26, "Input",
 CellTags->"HypExplicit"],
Cell[1171360, 45953, 203, 6, 26, "Output",
 CellTags->"HypExplicit"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1171600, 45964, 142, 4, 26, "Input",
 CellTags->"HypExplicit"],
Cell[1171745, 45970, 622, 22, 48, "Output",
 CellTags->"HypExplicit"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1172404, 45997, 262, 8, 26, "Input",
 CellTags->"HypExplicit"],
Cell[1172669, 46007, 1537, 56, 26, "Output",
 CellTags->"HypExplicit"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1174243, 46068, 142, 4, 26, "Input",
 CellTags->"HypExplicit"],
Cell[1174388, 46074, 846, 30, 48, "Output",
 CellTags->"HypExplicit"]
}, Open  ]]
}, Open  ]],
Cell[1175261, 46108, 49, 1, 41, "ObjectName",
 CellTags->"HypInt"],
Cell[CellGroupData[{
Cell[1175335, 46113, 57, 1, 36, "Subsubsection",
 CellTags->"HypInt"],
Cell[1175395, 46116, 206, 5, 43, "Text",
 CellTags->"HypInt"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1175638, 46126, 54, 1, 34, "Subsubsection",
 CellTags->"HypInt"],
Cell[CellGroupData[{
Cell[1175717, 46131, 159, 4, 26, "Input",
 CellTags->"HypInt"],
Cell[1175879, 46137, 198, 6, 26, "Output",
 CellTags->"HypInt"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1176114, 46148, 128, 4, 26, "Input",
 CellTags->"HypInt"],
Cell[1176245, 46154, 987, 37, 55, "Output",
 CellTags->"HypInt"]
}, Open  ]]
}, Open  ]],
Cell[1177259, 46195, 45, 1, 41, "ObjectName",
 CellTags->"IFPD"],
Cell[CellGroupData[{
Cell[1177329, 46200, 55, 1, 36, "Subsubsection",
 CellTags->"IFPD"],
Cell[1177387, 46203, 66, 1, 25, "Text",
 CellTags->"IFPD"],
Cell[1177456, 46206, 214, 9, 25, "Text",
 CellTags->"IFPD"]
}, Open  ]],
Cell[1177685, 46218, 51, 1, 41, "ObjectName",
 CellTags->"IFPDOff"],
Cell[CellGroupData[{
Cell[1177761, 46223, 58, 1, 36, "Subsubsection",
 CellTags->"IFPDOff"],
Cell[1177822, 46226, 182, 4, 43, "Text",
 CellTags->"IFPDOff"],
Cell[1178007, 46232, 270, 14, 25, "Text",
 CellTags->"IFPDOff"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1178314, 46251, 55, 1, 34, "Subsubsection",
 CellTags->"IFPDOff"],
Cell[CellGroupData[{
Cell[1178394, 46256, 163, 5, 26, "Input",
 CellTags->"IFPDOff"],
Cell[1178560, 46263, 426, 17, 28, "Output",
 CellTags->"IFPDOff"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1179023, 46285, 108, 3, 26, "Input",
 CellTags->"IFPDOff"],
Cell[1179134, 46290, 219, 6, 38, "Output",
 CellTags->"IFPDOff"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1179390, 46301, 130, 4, 26, "Input",
 CellTags->"IFPDOff"],
Cell[1179523, 46307, 254, 9, 28, "Output",
 CellTags->"IFPDOff"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1179814, 46321, 108, 3, 26, "Input",
 CellTags->"IFPDOff"],
Cell[1179925, 46326, 325, 10, 38, "Output",
 CellTags->"IFPDOff"]
}, Open  ]]
}, Open  ]],
Cell[1180277, 46340, 49, 1, 41, "ObjectName",
 CellTags->"IFPDOn"],
Cell[CellGroupData[{
Cell[1180351, 46345, 57, 1, 36, "Subsubsection",
 CellTags->"IFPDOn"],
Cell[1180411, 46348, 317, 6, 61, "Text",
 CellTags->"IFPDOn"],
Cell[1180731, 46356, 272, 14, 25, "Text",
 CellTags->"IFPDOn"]
}, Open  ]],
Cell[1181018, 46373, 59, 1, 41, "ObjectName",
 CellTags->"IncludePair"],
Cell[CellGroupData[{
Cell[1181102, 46378, 62, 1, 36, "Subsubsection",
 CellTags->"IncludePair"],
Cell[1181167, 46381, 124, 3, 25, "Text",
 CellTags->"IncludePair"],
Cell[1181294, 46386, 176, 9, 25, "Text",
 CellTags->"IncludePair"]
}, Open  ]],
Cell[1181485, 46398, 63, 1, 41, "ObjectName",
 CellTags->"IndexPosition"],
Cell[CellGroupData[{
Cell[1181573, 46403, 64, 1, 36, "Subsubsection",
 CellTags->"IndexPosition"],
Cell[1181640, 46406, 89, 1, 25, "Text",
 CellTags->"IndexPosition"],
Cell[1181732, 46409, 199, 9, 25, "Text",
 CellTags->"IndexPosition"]
}, Open  ]],
Cell[1181946, 46421, 51, 1, 41, "ObjectName",
 CellTags->"Indices"],
Cell[CellGroupData[{
Cell[1182022, 46426, 58, 1, 36, "Subsubsection",
 CellTags->"Indices"],
Cell[1182083, 46429, 203, 4, 43, "Text",
 CellTags->"Indices"],
Cell[1182289, 46435, 193, 9, 25, "Text",
 CellTags->"Indices"]
}, Open  ]],
Cell[1182497, 46447, 67, 1, 41, "ObjectName",
 CellTags->"InitialFunction"],
Cell[CellGroupData[{
Cell[1182589, 46452, 66, 1, 36, "Subsubsection",
 CellTags->"InitialFunction"],
Cell[1182658, 46455, 183, 4, 25, "Text",
 CellTags->"InitialFunction"],
Cell[1182844, 46461, 186, 9, 25, "Text",
 CellTags->"InitialFunction"]
}, Open  ]],
Cell[1183045, 46473, 77, 1, 41, "ObjectName",
 CellTags->"InitialSubstitutions"],
Cell[CellGroupData[{
Cell[1183147, 46478, 71, 1, 36, "Subsubsection",
 CellTags->"InitialSubstitutions"],
Cell[1183221, 46481, 208, 4, 25, "Text",
 CellTags->"InitialSubstitutions"],
Cell[1183432, 46487, 408, 19, 25, "Text",
 CellTags->"InitialSubstitutions"]
}, Open  ]],
Cell[1183855, 46509, 69, 1, 41, "ObjectName",
 CellTags->"InsideDiracTrace"],
Cell[CellGroupData[{
Cell[1183949, 46514, 67, 1, 36, "Subsubsection",
 CellTags->"InsideDiracTrace"],
Cell[1184019, 46517, 498, 9, 61, "Text",
 CellTags->"InsideDiracTrace"],
Cell[1184520, 46528, 318, 14, 25, "Text",
 CellTags->"InsideDiracTrace"]
}, Open  ]],
Cell[1184853, 46545, 63, 1, 41, "ObjectName",
 CellTags->"IntegralTable"],
Cell[CellGroupData[{
Cell[1184941, 46550, 64, 1, 36, "Subsubsection",
 CellTags->"IntegralTable"],
Cell[1185008, 46553, 233, 5, 25, "Text",
 CellTags->"IntegralTable"],
Cell[1185244, 46560, 500, 19, 25, "Text",
 CellTags->"IntegralTable"]
}, Open  ]],
Cell[1185759, 46582, 69, 1, 41, "ObjectName",
 CellTags->"IntegrateByParts"],
Cell[CellGroupData[{
Cell[1185853, 46587, 67, 1, 36, "Subsubsection",
 CellTags->"IntegrateByParts"],
Cell[1185923, 46590, 184, 4, 24, "Text",
 CellTags->"IntegrateByParts"],
Cell[CellGroupData[{
Cell[1186132, 46598, 131, 3, 26, "Input",
 CellTags->"IntegrateByParts"],
Cell[1186266, 46603, 214, 6, 26, "Output",
 CellTags->"IntegrateByParts"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[1186529, 46615, 64, 1, 34, "Subsubsection",
 CellTags->"IntegrateByParts"],
Cell[CellGroupData[{
Cell[1186618, 46620, 539, 19, 26, "Input",
 CellTags->"IntegrateByParts"],
Cell[1187160, 46641, 690, 24, 45, "Output",
 CellTags->"IntegrateByParts"]
}, Open  ]]
}, Open  ]],
Cell[1187877, 46669, 59, 1, 41, "ObjectName",
 CellTags->"Integratedx"],
Cell[CellGroupData[{
Cell[1187961, 46674, 62, 1, 36, "Subsubsection",
 CellTags->"Integratedx"],
Cell[1188026, 46677, 154, 4, 25, "Text",
 CellTags->"Integratedx"],
Cell[1188183, 46683, 852, 34, 43, "Text",
 CellTags->"Integratedx"]
}, Open  ]],
Cell[1189050, 46720, 57, 1, 41, "ObjectName",
 CellTags->"Integrate2"],
Cell[CellGroupData[{
Cell[1189132, 46725, 61, 1, 36, "Subsubsection",
 CellTags->"Integrate2"],
Cell[1189196, 46728, 866, 17, 97, "Text",
 CellTags->"Integrate2"],
Cell[1190065, 46747, 508, 24, 25, "Text",
 CellTags->"Integrate2"],
Cell[1190576, 46773, 405, 7, 61, "Text",
 CellTags->"Integrate2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1191018, 46785, 58, 1, 34, "Subsubsection",
 CellTags->"Integrate2"],
Cell[CellGroupData[{
Cell[1191101, 46790, 425, 15, 26, "Input",
 CellTags->"Integrate2"],
Cell[1191529, 46807, 410, 14, 43, "Output",
 CellTags->"Integrate2"]
}, Open  ]],
Cell[1191954, 46824, 190, 6, 24, "Text",
 CellTags->"Integrate2"],
Cell[CellGroupData[{
Cell[1192169, 46834, 281, 9, 26, "Input",
 CellTags->"Integrate2"],
Cell[1192453, 46845, 274, 8, 26, "Output",
 CellTags->"Integrate2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1192764, 46858, 276, 9, 26, "Input",
 CellTags->"Integrate2"],
Cell[1193043, 46869, 387, 13, 43, "Output",
 CellTags->"Integrate2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1193467, 46887, 336, 11, 26, "Input",
 CellTags->"Integrate2"],
Cell[1193806, 46900, 617, 21, 46, "Output",
 CellTags->"Integrate2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1194460, 46926, 317, 10, 26, "Input",
 CellTags->"Integrate2"],
Cell[1194780, 46938, 176, 5, 26, "Output",
 CellTags->"Integrate2"]
}, Open  ]],
Cell[1194971, 46946, 1589, 53, 55, "Text",
 CellTags->"Integrate2"],
Cell[CellGroupData[{
Cell[1196585, 47003, 270, 9, 26, "Input",
 CellTags->"Integrate2"],
Cell[1196858, 47014, 148, 4, 26, "Output",
 CellTags->"Integrate2"]
}, Open  ]],
Cell[1197021, 47021, 649, 24, 47, "Text",
 CellTags->"Integrate2"],
Cell[CellGroupData[{
Cell[1197695, 47049, 317, 10, 26, "Input",
 CellTags->"Integrate2"],
Cell[1198015, 47061, 148, 4, 26, "Output",
 CellTags->"Integrate2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1198200, 47070, 386, 13, 26, "Input",
 CellTags->"Integrate2"],
Cell[1198589, 47085, 206, 6, 26, "Output",
 CellTags->"Integrate2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1198832, 47096, 417, 14, 26, "Input",
 CellTags->"Integrate2"],
Cell[1199252, 47112, 221, 8, 43, "Output",
 CellTags->"Integrate2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1199510, 47125, 328, 11, 26, "Input",
 CellTags->"Integrate2"],
Cell[1199841, 47138, 202, 7, 26, "Output",
 CellTags->"Integrate2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1200080, 47150, 354, 12, 26, "Input",
 CellTags->"Integrate2"],
Cell[1200437, 47164, 351, 12, 44, "Output",
 CellTags->"Integrate2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1200825, 47181, 313, 10, 26, "Input",
 CellTags->"Integrate2"],
Cell[1201141, 47193, 231, 7, 26, "Output",
 CellTags->"Integrate2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1201409, 47205, 281, 9, 26, "Input",
 CellTags->"Integrate2"],
Cell[1201693, 47216, 338, 12, 43, "Output",
 CellTags->"Integrate2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1202068, 47233, 445, 15, 26, "Input",
 CellTags->"Integrate2"],
Cell[1202516, 47250, 529, 18, 47, "Output",
 CellTags->"Integrate2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1203082, 47273, 721, 24, 26, "Input",
 CellTags->"Integrate2"],
Cell[1203806, 47299, 4395, 148, 185, "Output",
 CellTags->"Integrate2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1208238, 47452, 471, 16, 26, "Input",
 CellTags->"Integrate2"],
Cell[1208712, 47470, 861, 30, 46, "Output",
 CellTags->"Integrate2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1209610, 47505, 567, 19, 49, "Input",
 CellTags->"Integrate2"],
Cell[1210180, 47526, 5294, 178, 171, "Output",
 CellTags->"Integrate2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1215511, 47709, 142, 4, 26, "Input",
 CellTags->"Integrate2"],
Cell[1215656, 47715, 1740, 54, 127, "Output",
 CellTags->"Integrate2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1217433, 47774, 104, 3, 26, "Input",
 CellTags->"Integrate2"],
Cell[1217540, 47779, 167, 4, 26, "Output",
 CellTags->"Integrate2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1217744, 47788, 418, 13, 49, "Input"],
Cell[1218165, 47803, 142, 3, 26, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1218344, 47811, 684, 23, 49, "Input",
 CellTags->"Integrate2"],
Cell[1219031, 47836, 3720, 140, 128, "Output",
 CellTags->"Integrate2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1222788, 47981, 107, 3, 26, "Input"],
Cell[1222898, 47986, 488, 17, 43, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1223423, 48008, 79, 2, 26, "Input"],
Cell[1223505, 48012, 142, 3, 26, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1223684, 48020, 656, 22, 49, "Input"],
Cell[1224343, 48044, 144, 3, 26, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1224524, 48052, 151, 4, 26, "Input",
 CellTags->"Integrate2"],
Cell[1224678, 48058, 151, 4, 26, "Output",
 CellTags->"Integrate2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1224866, 48067, 368, 12, 26, "Input",
 CellTags->"Integrate2"],
Cell[1225237, 48081, 148, 4, 26, "Output",
 CellTags->"Integrate2"]
}, Open  ]],
Cell[1225400, 48088, 137, 4, 24, "Text",
 CellTags->"Integrate2"],
Cell[CellGroupData[{
Cell[1225562, 48096, 147, 4, 26, "Input",
 CellTags->"Integrate2"],
Cell[1225712, 48102, 5418, 175, 353, "Output",
 CellTags->"Integrate2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1231167, 48282, 230, 8, 26, "Input",
 CellTags->"Integrate2"],
Cell[1231400, 48292, 216, 7, 26, "Output",
 CellTags->"Integrate2"]
}, Open  ]],
Cell[1231631, 48302, 165, 4, 24, "Text",
 CellTags->"Integrate2"],
Cell[CellGroupData[{
Cell[1231821, 48310, 326, 11, 26, "Input",
 CellTags->"Integrate2"],
Cell[1232150, 48323, 217, 7, 26, "Output",
 CellTags->"Integrate2"]
}, Open  ]],
Cell[1232382, 48333, 126, 4, 26, "Input",
 CellTags->"Integrate2"],
Cell[CellGroupData[{
Cell[1232533, 48341, 317, 10, 26, "Input",
 CellTags->"Integrate2"],
Cell[1232853, 48353, 176, 5, 26, "Output",
 CellTags->"Integrate2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1233066, 48363, 325, 11, 30, "Input",
 CellTags->"Integrate2"],
Cell[1233394, 48376, 282, 9, 44, "Output",
 CellTags->"Integrate2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1233713, 48390, 99, 3, 26, "Input",
 CellTags->"Integrate2"],
Cell[1233815, 48395, 166, 4, 26, "Output",
 CellTags->"Integrate2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1234018, 48404, 325, 11, 30, "Input",
 CellTags->"Integrate2"],
Cell[1234346, 48417, 169, 4, 26, "Output",
 CellTags->"Integrate2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1234552, 48426, 381, 13, 30, "Input",
 CellTags->"Integrate2"],
Cell[1234936, 48441, 4904, 185, 199, "Output",
 CellTags->"Integrate2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1239877, 48631, 107, 3, 26, "Input"],
Cell[1239987, 48636, 254, 8, 44, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1240278, 48649, 79, 2, 26, "Input"],
Cell[1240360, 48653, 141, 3, 26, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1240538, 48661, 353, 12, 30, "Input"],
Cell[1240894, 48675, 143, 3, 26, "Output"]
}, Open  ]]
}, Open  ]],
Cell[1241064, 48682, 57, 1, 70, "ObjectName",
 CellTags->"Integrate3"],
Cell[CellGroupData[{
Cell[1241146, 48687, 61, 1, 70, "Subsubsection",
 CellTags->"Integrate3"],
Cell[1241210, 48690, 331, 6, 70, "Text",
 CellTags->"Integrate3"],
Cell[1241544, 48698, 187, 9, 70, "Text",
 CellTags->"Integrate3"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1241768, 48712, 58, 1, 34, "Subsubsection",
 CellTags->"Integrate3"],
Cell[CellGroupData[{
Cell[1241851, 48717, 280, 9, 26, "Input",
 CellTags->"Integrate3"],
Cell[1242134, 48728, 267, 9, 49, "Output",
 CellTags->"Integrate3"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1242438, 48742, 400, 13, 51, "Input",
 CellTags->"Integrate3"],
Cell[1242841, 48757, 555, 21, 26, "Output",
 CellTags->"Integrate3"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1243433, 48783, 676, 23, 51, "Input",
 CellTags->"Integrate3"],
Cell[1244112, 48808, 2077, 74, 68, "Output",
 CellTags->"Integrate3"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1246226, 48887, 266, 8, 26, "Input",
 CellTags->"Integrate3"],
Cell[1246495, 48897, 148, 4, 26, "Output",
 CellTags->"Integrate3"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1246680, 48906, 322, 10, 26, "Input",
 CellTags->"Integrate3"],
Cell[1247005, 48918, 239, 7, 26, "Output",
 CellTags->"Integrate3"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1247281, 48930, 270, 9, 26, "Input",
 CellTags->"Integrate3"],
Cell[1247554, 48941, 148, 4, 26, "Output",
 CellTags->"Integrate3"]
}, Open  ]]
}, Open  ]],
Cell[1247729, 48949, 87, 1, 70, "ObjectName",
 CellTags->"IntermediateSubstitutions"],
Cell[CellGroupData[{
Cell[1247841, 48954, 76, 1, 34, "Subsubsection",
 CellTags->"IntermediateSubstitutions"],
Cell[1247920, 48957, 206, 4, 24, "Text",
 CellTags->"IntermediateSubstitutions"],
Cell[1248129, 48963, 193, 9, 24, "Text",
 CellTags->"IntermediateSubstitutions"]
}, Open  ]],
Cell[1248337, 48975, 63, 1, 39, "ObjectName",
 CellTags->"InverseMellin"],
Cell[CellGroupData[{
Cell[1248425, 48980, 64, 1, 34, "Subsubsection",
 CellTags->"InverseMellin"],
Cell[1248492, 48983, 531, 14, 58, "Text",
 CellTags->"InverseMellin"],
Cell[1249026, 48999, 609, 29, 24, "Text",
 CellTags->"InverseMellin"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1249672, 49033, 61, 1, 34, "Subsubsection",
 CellTags->"InverseMellin"],
Cell[CellGroupData[{
Cell[1249758, 49038, 169, 5, 26, "Input",
 CellTags->"InverseMellin"],
Cell[1249930, 49045, 201, 6, 26, "Output",
 CellTags->"InverseMellin"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1250168, 49056, 220, 7, 26, "Input",
 CellTags->"InverseMellin"],
Cell[1250391, 49065, 199, 6, 26, "Output",
 CellTags->"InverseMellin"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1250627, 49076, 142, 4, 26, "Input",
 CellTags->"InverseMellin"],
Cell[1250772, 49082, 322, 11, 26, "Output",
 CellTags->"InverseMellin"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1251131, 49098, 194, 6, 26, "Input",
 CellTags->"InverseMellin"],
Cell[1251328, 49106, 337, 12, 44, "Output",
 CellTags->"InverseMellin"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1251702, 49123, 194, 6, 26, "Input",
 CellTags->"InverseMellin"],
Cell[1251899, 49131, 404, 14, 26, "Output",
 CellTags->"InverseMellin"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1252340, 49150, 203, 7, 26, "Input",
 CellTags->"InverseMellin"],
Cell[1252546, 49159, 404, 14, 26, "Output",
 CellTags->"InverseMellin"]
}, Open  ]],
Cell[1252965, 49176, 113, 3, 24, "Text",
 CellTags->"InverseMellin"],
Cell[CellGroupData[{
Cell[1253103, 49183, 294, 9, 26, "Input",
 CellTags->"InverseMellin"],
Cell[1253400, 49194, 173, 5, 46, "Output",
 CellTags->"InverseMellin"]
}, Open  ]],
Cell[1253588, 49202, 117, 3, 24, "Text",
 CellTags->"InverseMellin"],
Cell[1253708, 49207, 3132, 100, 193, "Input",
 CellTags->"InverseMellin"],
Cell[1256843, 49309, 244, 7, 26, "Input",
 CellTags->"InverseMellin"],
Cell[CellGroupData[{
Cell[1257112, 49320, 179, 6, 26, "Input",
 CellTags->"InverseMellin"],
Cell[1257294, 49328, 440, 15, 46, "Output",
 CellTags->"InverseMellin"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1257771, 49348, 179, 6, 26, "Input",
 CellTags->"InverseMellin"],
Cell[1257953, 49356, 398, 13, 46, "Output",
 CellTags->"InverseMellin"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1258388, 49374, 172, 5, 26, "Input",
 CellTags->"InverseMellin"],
Cell[1258563, 49381, 768, 26, 48, "Output",
 CellTags->"InverseMellin"]
}, Open  ]],
Cell[1259346, 49410, 81, 2, 26, "Input",
 CellTags->"InverseMellin"],
Cell[CellGroupData[{
Cell[1259452, 49416, 192, 6, 26, "Input",
 CellTags->"InverseMellin"],
Cell[1259647, 49424, 571, 21, 48, "Output",
 CellTags->"InverseMellin"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1260255, 49450, 270, 9, 26, "Input",
 CellTags->"InverseMellin"],
Cell[1260528, 49461, 544, 19, 44, "Output",
 CellTags->"InverseMellin"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1261109, 49485, 270, 9, 26, "Input",
 CellTags->"InverseMellin"],
Cell[1261382, 49496, 737, 26, 44, "Output",
 CellTags->"InverseMellin"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1262156, 49527, 267, 9, 26, "Input",
 CellTags->"InverseMellin"],
Cell[1262426, 49538, 734, 26, 46, "Output",
 CellTags->"InverseMellin"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1263197, 49569, 246, 9, 26, "Input",
 CellTags->"InverseMellin"],
Cell[1263446, 49580, 591, 21, 46, "Output",
 CellTags->"InverseMellin"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1264074, 49606, 274, 10, 26, "Input",
 CellTags->"InverseMellin"],
Cell[1264351, 49618, 956, 32, 47, "Output",
 CellTags->"InverseMellin"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1265344, 49655, 192, 6, 26, "Input",
 CellTags->"InverseMellin"],
Cell[1265539, 49663, 742, 26, 46, "Output",
 CellTags->"InverseMellin"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1266318, 49694, 221, 7, 26, "Input",
 CellTags->"InverseMellin"],
Cell[1266542, 49703, 729, 25, 46, "Output",
 CellTags->"InverseMellin"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1267308, 49733, 192, 6, 26, "Input",
 CellTags->"InverseMellin"],
Cell[1267503, 49741, 837, 30, 49, "Output",
 CellTags->"InverseMellin"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1268377, 49776, 202, 6, 26, "Input",
 CellTags->"InverseMellin"],
Cell[1268582, 49784, 586, 21, 48, "Output",
 CellTags->"InverseMellin"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1269205, 49810, 223, 7, 26, "Input",
 CellTags->"InverseMellin"],
Cell[1269431, 49819, 975, 34, 48, "Output",
 CellTags->"InverseMellin"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1270443, 49858, 223, 7, 26, "Input",
 CellTags->"InverseMellin"],
Cell[1270669, 49867, 963, 34, 48, "Output",
 CellTags->"InverseMellin"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1271669, 49906, 212, 6, 26, "Input",
 CellTags->"InverseMellin"],
Cell[1271884, 49914, 674, 24, 52, "Output",
 CellTags->"InverseMellin"]
}, Open  ]],
Cell[1272573, 49941, 158, 5, 26, "Input",
 CellTags->"InverseMellin"]
}, Open  ]],
Cell[1272746, 49949, 51, 1, 39, "ObjectName",
 CellTags->"Isolate"],
Cell[CellGroupData[{
Cell[1272822, 49954, 58, 1, 34, "Subsubsection",
 CellTags->"Isolate"],
Cell[1272883, 49957, 422, 7, 58, "Text",
 CellTags->"Isolate"],
Cell[CellGroupData[{
Cell[1273330, 49968, 114, 3, 26, "Input",
 CellTags->"Isolate"],
Cell[1273447, 49973, 446, 13, 47, "Output",
 CellTags->"Isolate"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[1273942, 49992, 55, 1, 34, "Subsubsection",
 CellTags->"Isolate"],
Cell[CellGroupData[{
Cell[1274022, 49997, 156, 5, 26, "Input",
 CellTags->"Isolate"],
Cell[1274181, 50004, 202, 7, 26, "Output",
 CellTags->"Isolate"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1274420, 50016, 378, 13, 26, "Input",
 CellTags->"Isolate"],
Cell[1274801, 50031, 351, 13, 26, "Output",
 CellTags->"Isolate"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1275189, 50049, 114, 3, 26, "Input",
 CellTags->"Isolate"],
Cell[1275306, 50054, 328, 12, 38, "Output",
 CellTags->"Isolate"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1275671, 50071, 182, 5, 26, "Input",
 CellTags->"Isolate"],
Cell[1275856, 50078, 688, 26, 26, "Output",
 CellTags->"Isolate"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1276581, 50109, 561, 19, 26, "Input",
 CellTags->"Isolate"],
Cell[1277145, 50130, 403, 15, 26, "Output",
 CellTags->"Isolate"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1277585, 50150, 94, 3, 26, "Input",
 CellTags->"Isolate"],
Cell[1277682, 50155, 131, 2, 40, "Print",
 CellTags->{"Info3500973905-4982097", "Isolate"}],
Cell[1277816, 50159, 1073, 31, 55, "Print",
 CellTags->{"Info3500973905-4982097", "Isolate"}]
}, Open  ]],
Cell[CellGroupData[{
Cell[1278926, 50195, 278, 7, 26, "Input",
 CellTags->"Isolate"],
Cell[1279207, 50204, 201, 7, 26, "Output",
 CellTags->"Isolate"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1279445, 50216, 178, 6, 26, "Input",
 CellTags->"Isolate"],
Cell[1279626, 50224, 253, 8, 26, "Output",
 CellTags->"Isolate"]
}, Open  ]],
Cell[1279894, 50235, 152, 4, 26, "Input",
 CellTags->"Isolate"]
}, Open  ]],
Cell[1280061, 50242, 59, 1, 39, "ObjectName",
 CellTags->"IsolateHead"],
Cell[CellGroupData[{
Cell[1280145, 50247, 62, 1, 34, "Subsubsection",
 CellTags->"IsolateHead"],
Cell[1280210, 50250, 84, 1, 24, "Text",
 CellTags->"IsolateHead"],
Cell[1280297, 50253, 194, 9, 24, "Text",
 CellTags->"IsolateHead"]
}, Open  ]],
Cell[1280506, 50265, 61, 1, 39, "ObjectName",
 CellTags->"IsolateNames"],
Cell[CellGroupData[{
Cell[1280592, 50270, 63, 1, 34, "Subsubsection",
 CellTags->"IsolateNames"],
Cell[1280658, 50273, 220, 5, 41, "Text",
 CellTags->"IsolateNames"],
Cell[1280881, 50280, 290, 14, 24, "Text",
 CellTags->"IsolateNames"]
}, Open  ]],
Cell[1281186, 50297, 61, 1, 39, "ObjectName",
 CellTags->"IsolatePrint"],
Cell[CellGroupData[{
Cell[1281272, 50302, 63, 1, 34, "Subsubsection",
 CellTags->"IsolatePrint"],
Cell[1281338, 50305, 284, 6, 41, "Text",
 CellTags->"IsolatePrint"],
Cell[1281625, 50313, 180, 9, 24, "Text",
 CellTags->"IsolatePrint"]
}, Open  ]],
Cell[1281820, 50325, 61, 1, 39, "ObjectName",
 CellTags->"IsolateSplit"],
Cell[CellGroupData[{
Cell[1281906, 50330, 63, 1, 34, "Subsubsection",
 CellTags->"IsolateSplit"],
Cell[1281972, 50333, 413, 7, 58, "Text",
 CellTags->"IsolateSplit"],
Cell[1282388, 50342, 180, 9, 25, "Text",
 CellTags->"IsolateSplit"]
}, Open  ]],
Cell[1282583, 50354, 53, 1, 41, "ObjectName",
 CellTags->"KeepOnly"],
Cell[CellGroupData[{
Cell[1282661, 50359, 59, 1, 36, "Subsubsection",
 CellTags->"KeepOnly"],
Cell[1282723, 50362, 293, 6, 43, "Text",
 CellTags->"KeepOnly"],
Cell[1283019, 50370, 452, 24, 25, "Text",
 CellTags->"KeepOnly"]
}, Open  ]],
Cell[1283486, 50397, 41, 1, 41, "ObjectName",
 CellTags->"KK"],
Cell[CellGroupData[{
Cell[1283552, 50402, 53, 1, 36, "Subsubsection",
 CellTags->"KK"],
Cell[1283608, 50405, 235, 5, 43, "Text",
 CellTags->"KK"],
Cell[1283846, 50412, 292, 14, 25, "Text",
 CellTags->"KK"]
}, Open  ]],
Cell[1284153, 50429, 49, 1, 41, "ObjectName",
 CellTags->"Kummer"],
Cell[CellGroupData[{
Cell[1284227, 50434, 57, 1, 34, "Subsubsection",
 CellTags->"Kummer"],
Cell[1284287, 50437, 260, 5, 41, "Text",
 CellTags->"Kummer"],
Cell[1284550, 50444, 201, 9, 24, "Text",
 CellTags->"Kummer"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1284788, 50458, 54, 1, 34, "Subsubsection",
 CellTags->"Kummer"],
Cell[CellGroupData[{
Cell[1284867, 50463, 337, 9, 26, "Input",
 CellTags->"Kummer"],
Cell[1285207, 50474, 494, 15, 28, "Output",
 CellTags->"Kummer"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1285738, 50494, 337, 9, 26, "Input",
 CellTags->"Kummer"],
Cell[1286078, 50505, 509, 16, 42, "Output",
 CellTags->"Kummer"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1286624, 50526, 337, 9, 26, "Input",
 CellTags->"Kummer"],
Cell[1286964, 50537, 509, 16, 42, "Output",
 CellTags->"Kummer"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1287510, 50558, 388, 11, 26, "Input",
 CellTags->"Kummer"],
Cell[1287901, 50571, 494, 15, 28, "Output",
 CellTags->"Kummer"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1288432, 50591, 479, 13, 46, "Input",
 CellTags->"Kummer"],
Cell[1288914, 50606, 573, 16, 26, "Output",
 CellTags->"Kummer"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1289524, 50627, 388, 11, 26, "Input",
 CellTags->"Kummer"],
Cell[1289915, 50640, 476, 14, 46, "Output",
 CellTags->"Kummer"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1290428, 50659, 479, 13, 46, "Input",
 CellTags->"Kummer"],
Cell[1290910, 50674, 589, 18, 46, "Output",
 CellTags->"Kummer"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1291536, 50697, 388, 11, 26, "Input",
 CellTags->"Kummer"],
Cell[1291927, 50710, 476, 14, 46, "Output",
 CellTags->"Kummer"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1292440, 50729, 479, 13, 46, "Input",
 CellTags->"Kummer"],
Cell[1292922, 50744, 592, 18, 46, "Output",
 CellTags->"Kummer"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1293551, 50767, 487, 15, 26, "Input",
 CellTags->"Kummer"],
Cell[1294041, 50784, 663, 21, 46, "Output",
 CellTags->"Kummer"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1294741, 50810, 629, 19, 46, "Input",
 CellTags->"Kummer"],
Cell[1295373, 50831, 764, 23, 46, "Output",
 CellTags->"Kummer"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1296174, 50859, 487, 15, 26, "Input",
 CellTags->"Kummer"],
Cell[1296664, 50876, 637, 20, 45, "Output",
 CellTags->"Kummer"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1297338, 50901, 629, 19, 46, "Input",
 CellTags->"Kummer"],
Cell[1297970, 50922, 749, 24, 45, "Output",
 CellTags->"Kummer"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1298756, 50951, 487, 15, 26, "Input",
 CellTags->"Kummer"],
Cell[1299246, 50968, 611, 19, 46, "Output",
 CellTags->"Kummer"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1299894, 50992, 629, 19, 46, "Input",
 CellTags->"Kummer"],
Cell[1300526, 51013, 792, 24, 46, "Output",
 CellTags->"Kummer"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1301355, 51042, 487, 15, 26, "Input",
 CellTags->"Kummer"],
Cell[1301845, 51059, 663, 21, 46, "Output",
 CellTags->"Kummer"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1302545, 51085, 629, 19, 46, "Input",
 CellTags->"Kummer"],
Cell[1303177, 51106, 787, 21, 46, "Output",
 CellTags->"Kummer"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1304001, 51132, 487, 15, 26, "Input",
 CellTags->"Kummer"],
Cell[1304491, 51149, 585, 18, 46, "Output",
 CellTags->"Kummer"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1305113, 51172, 629, 19, 46, "Input",
 CellTags->"Kummer"],
Cell[1305745, 51193, 833, 23, 46, "Output",
 CellTags->"Kummer"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1306615, 51221, 487, 15, 26, "Input",
 CellTags->"Kummer"],
Cell[1307105, 51238, 585, 18, 46, "Output",
 CellTags->"Kummer"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1307727, 51261, 629, 19, 46, "Input",
 CellTags->"Kummer"],
Cell[1308359, 51282, 722, 20, 46, "Output",
 CellTags->"Kummer"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1309118, 51307, 531, 15, 46, "Input",
 CellTags->"Kummer"],
Cell[1309652, 51324, 620, 19, 28, "Output",
 CellTags->"Kummer"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1310309, 51348, 338, 9, 26, "Input",
 CellTags->"Kummer"],
Cell[1310650, 51359, 494, 15, 28, "Output",
 CellTags->"Kummer"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1311181, 51379, 531, 15, 46, "Input",
 CellTags->"Kummer"],
Cell[1311715, 51396, 676, 20, 42, "Output",
 CellTags->"Kummer"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1312428, 51421, 338, 9, 26, "Input",
 CellTags->"Kummer"],
Cell[1312769, 51432, 483, 15, 42, "Output",
 CellTags->"Kummer"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1313289, 51452, 531, 15, 46, "Input",
 CellTags->"Kummer"],
Cell[1313823, 51469, 674, 20, 42, "Output",
 CellTags->"Kummer"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1314534, 51494, 338, 9, 26, "Input",
 CellTags->"Kummer"],
Cell[1314875, 51505, 482, 15, 42, "Output",
 CellTags->"Kummer"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1315394, 51525, 582, 17, 46, "Input",
 CellTags->"Kummer"],
Cell[1315979, 51544, 631, 16, 26, "Output",
 CellTags->"Kummer"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1316647, 51565, 389, 11, 26, "Input",
 CellTags->"Kummer"],
Cell[1317039, 51578, 494, 15, 28, "Output",
 CellTags->"Kummer"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1317570, 51598, 582, 17, 46, "Input",
 CellTags->"Kummer"],
Cell[1318155, 51617, 731, 19, 46, "Output",
 CellTags->"Kummer"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1318923, 51641, 389, 11, 26, "Input",
 CellTags->"Kummer"],
Cell[1319315, 51654, 533, 16, 46, "Output",
 CellTags->"Kummer"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1319885, 51675, 582, 17, 46, "Input",
 CellTags->"Kummer"],
Cell[1320470, 51694, 674, 17, 46, "Output",
 CellTags->"Kummer"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1321181, 51716, 389, 11, 26, "Input",
 CellTags->"Kummer"],
Cell[1321573, 51729, 478, 14, 46, "Output",
 CellTags->"Kummer"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1322088, 51748, 338, 9, 26, "Input",
 CellTags->"Kummer"],
Cell[1322429, 51759, 1352, 44, 95, "Output",
 CellTags->"Kummer"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1323818, 51808, 338, 9, 26, "Input",
 CellTags->"Kummer"],
Cell[1324159, 51819, 1412, 47, 77, "Output",
 CellTags->"Kummer"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1325608, 51871, 338, 9, 26, "Input",
 CellTags->"Kummer"],
Cell[1325949, 51882, 1440, 47, 107, "Output",
 CellTags->"Kummer"]
}, Open  ]]
}, Open  ]],
Cell[1327416, 51933, 57, 1, 70, "ObjectName",
 CellTags->"Lagrangian"],
Cell[CellGroupData[{
Cell[1327498, 51938, 61, 1, 36, "Subsubsection",
 CellTags->"Lagrangian"],
Cell[1327562, 51941, 454, 9, 70, "Text",
 CellTags->"Lagrangian"],
Cell[1328019, 51952, 181, 9, 25, "Text",
 CellTags->"Lagrangian"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1328237, 51966, 58, 1, 34, "Subsubsection",
 CellTags->"Lagrangian"],
Cell[CellGroupData[{
Cell[1328320, 51971, 124, 3, 26, "Input",
 CellTags->"Lagrangian"],
Cell[1328447, 51976, 376, 12, 44, "Output",
 CellTags->"Lagrangian"]
}, Open  ]],
Cell[1328838, 51991, 85, 1, 24, "Text",
 CellTags->"Lagrangian"],
Cell[CellGroupData[{
Cell[1328948, 51996, 124, 3, 26, "Input",
 CellTags->"Lagrangian"],
Cell[1329075, 52001, 725, 24, 44, "Output",
 CellTags->"Lagrangian"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1329837, 52030, 124, 3, 26, "Input",
 CellTags->"Lagrangian"],
Cell[1329964, 52035, 1048, 37, 44, "Output",
 CellTags->"Lagrangian"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1331049, 52077, 124, 3, 26, "Input",
 CellTags->"Lagrangian"],
Cell[1331176, 52082, 663, 23, 26, "Output",
 CellTags->"Lagrangian"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1331876, 52110, 124, 3, 26, "Input",
 CellTags->"Lagrangian"],
Cell[1332003, 52115, 706, 24, 26, "Output",
 CellTags->"Lagrangian"]
}, Open  ]]
}, Open  ]],
Cell[1332736, 52143, 41, 1, 41, "ObjectName",
 CellTags->"LC"],
Cell[CellGroupData[{
Cell[1332802, 52148, 53, 1, 36, "Subsubsection",
 CellTags->"LC"],
Cell[1332858, 52151, 240, 5, 43, "Text",
 CellTags->"LC"],
Cell[1333101, 52158, 274, 14, 25, "Text",
 CellTags->"LC"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1333412, 52177, 50, 1, 34, "Subsubsection",
 CellTags->"LC"],
Cell[CellGroupData[{
Cell[1333487, 52182, 164, 5, 26, "Input",
 CellTags->"LC"],
Cell[1333654, 52189, 226, 7, 26, "Output",
 CellTags->"LC"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1333917, 52201, 95, 3, 26, "Input",
 CellTags->"LC"],
Cell[1334015, 52206, 519, 22, 26, "Output",
 CellTags->"LC"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1334571, 52233, 104, 3, 26, "Input",
 CellTags->"LC"],
Cell[1334678, 52238, 379, 9, 38, "Output",
 CellTags->"LC"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1335094, 52252, 181, 6, 26, "Input",
 CellTags->"LC"],
Cell[1335278, 52260, 211, 6, 26, "Output",
 CellTags->"LC"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1335526, 52271, 130, 4, 26, "Input",
 CellTags->"LC"],
Cell[1335659, 52277, 409, 11, 38, "Output",
 CellTags->"LC"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1336105, 52293, 381, 13, 26, "Input",
 CellTags->"LC"],
Cell[1336489, 52308, 438, 17, 28, "Output",
 CellTags->"LC"]
}, Open  ]]
}, Open  ]],
Cell[1336954, 52329, 43, 1, 39, "ObjectName",
 CellTags->"LCD"],
Cell[CellGroupData[{
Cell[1337022, 52334, 54, 1, 34, "Subsubsection",
 CellTags->"LCD"],
Cell[1337079, 52337, 234, 5, 41, "Text",
 CellTags->"LCD"],
Cell[1337316, 52344, 161, 4, 24, "Text",
 CellTags->"LCD"],
Cell[1337480, 52350, 272, 14, 24, "Text",
 CellTags->"LCD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1337789, 52369, 51, 1, 34, "Subsubsection",
 CellTags->"LCD"],
Cell[1337843, 52372, 190, 6, 26, "Input",
 CellTags->"LCD"],
Cell[CellGroupData[{
Cell[1338058, 52382, 166, 5, 26, "Input",
 CellTags->"LCD"],
Cell[1338227, 52389, 226, 7, 26, "Output",
 CellTags->"LCD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1338490, 52401, 96, 3, 26, "Input",
 CellTags->"LCD"],
Cell[1338589, 52406, 520, 22, 26, "Output",
 CellTags->"LCD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1339146, 52433, 105, 3, 26, "Input",
 CellTags->"LCD"],
Cell[1339254, 52438, 480, 13, 38, "Output",
 CellTags->"LCD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1339771, 52456, 183, 6, 26, "Input",
 CellTags->"LCD"],
Cell[1339957, 52464, 212, 6, 26, "Output",
 CellTags->"LCD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1340206, 52475, 131, 4, 26, "Input",
 CellTags->"LCD"],
Cell[1340340, 52481, 462, 13, 38, "Output",
 CellTags->"LCD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1340839, 52499, 400, 13, 26, "Input",
 CellTags->"LCD"],
Cell[1341242, 52514, 439, 16, 28, "Output",
 CellTags->"LCD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1341718, 52535, 170, 5, 26, "Input",
 CellTags->"LCD"],
Cell[1341891, 52542, 203, 6, 26, "Output",
 CellTags->"LCD"]
}, Open  ]]
}, Open  ]],
Cell[1342121, 52552, 61, 1, 41, "ObjectName",
 CellTags->"LeftPartialD"],
Cell[CellGroupData[{
Cell[1342207, 52557, 63, 1, 36, "Subsubsection",
 CellTags->"LeftPartialD"],
Cell[1342273, 52560, 307, 12, 31, "Text",
 CellTags->"LeftPartialD"],
Cell[1342583, 52574, 573, 24, 25, "Text",
 CellTags->"LeftPartialD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1343193, 52603, 69, 1, 33, "Subsubsection",
 CellTags->"LeftPartialD"],
Cell[CellGroupData[{
Cell[1343287, 52608, 253, 7, 26, "Input",
 CellTags->"LeftPartialD"],
Cell[1343543, 52617, 348, 12, 33, "Output",
 CellTags->"LeftPartialD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1343928, 52634, 120, 3, 26, "Input",
 CellTags->"LeftPartialD"],
Cell[1344051, 52639, 622, 15, 87, "Output",
 CellTags->"LeftPartialD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1344710, 52659, 118, 3, 26, "Input",
 CellTags->"LeftPartialD"],
Cell[1344831, 52664, 611, 12, 115, "Output",
 CellTags->"LeftPartialD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1345479, 52681, 161, 4, 26, "Input",
 CellTags->"LeftPartialD"],
Cell[1345643, 52687, 218, 5, 38, "Output",
 CellTags->"LeftPartialD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1345898, 52697, 372, 10, 26, "Input",
 CellTags->"LeftPartialD"],
Cell[1346273, 52709, 385, 13, 33, "Output",
 CellTags->"LeftPartialD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1346695, 52727, 120, 3, 26, "Input",
 CellTags->"LeftPartialD"],
Cell[1346818, 52732, 1254, 31, 150, "Output",
 CellTags->"LeftPartialD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1348109, 52768, 118, 3, 26, "Input",
 CellTags->"LeftPartialD"],
Cell[1348230, 52773, 1403, 30, 210, "Output",
 CellTags->"LeftPartialD"]
}, Open  ]]
}, Open  ]],
Cell[1349660, 52807, 71, 1, 41, "ObjectName",
 CellTags->"LeftRightPartialD"],
Cell[CellGroupData[{
Cell[1349756, 52812, 68, 1, 36, "Subsubsection",
 CellTags->"LeftRightPartialD"],
Cell[1349827, 52815, 592, 21, 49, "Text",
 CellTags->"LeftRightPartialD"],
Cell[1350422, 52838, 831, 33, 25, "Text",
 CellTags->"LeftRightPartialD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1351290, 52876, 65, 1, 34, "Subsubsection",
 CellTags->"LeftRightPartialD"],
Cell[CellGroupData[{
Cell[1351380, 52881, 132, 3, 26, "Input",
 CellTags->"LeftRightPartialD"],
Cell[1351515, 52886, 302, 10, 33, "Output",
 CellTags->"LeftRightPartialD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1351854, 52901, 127, 3, 26, "Input",
 CellTags->"LeftRightPartialD"],
Cell[1351984, 52906, 571, 20, 44, "Output",
 CellTags->"LeftRightPartialD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1352592, 52931, 263, 7, 26, "Input",
 CellTags->"LeftRightPartialD"],
Cell[1352858, 52940, 358, 12, 33, "Output",
 CellTags->"LeftRightPartialD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1353253, 52957, 125, 3, 26, "Input",
 CellTags->"LeftRightPartialD"],
Cell[1353381, 52962, 968, 27, 117, "Output",
 CellTags->"LeftRightPartialD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1354386, 52994, 382, 10, 26, "Input",
 CellTags->"LeftRightPartialD"],
Cell[1354771, 53006, 396, 13, 33, "Output",
 CellTags->"LeftRightPartialD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1355204, 53024, 125, 3, 26, "Input",
 CellTags->"LeftRightPartialD"],
Cell[1355332, 53029, 1370, 34, 171, "Output",
 CellTags->"LeftRightPartialD"]
}, Open  ]]
}, Open  ]],
Cell[1356729, 53067, 73, 1, 70, "ObjectName",
 CellTags->"LeftRightPartialD2"],
Cell[CellGroupData[{
Cell[1356827, 53072, 69, 1, 70, "Subsubsection",
 CellTags->"LeftRightPartialD2"],
Cell[1356899, 53075, 591, 21, 70, "Text",
 CellTags->"LeftRightPartialD2"],
Cell[1357493, 53098, 698, 29, 70, "Text",
 CellTags->"LeftRightPartialD2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1358228, 53132, 66, 1, 34, "Subsubsection",
 CellTags->"LeftRightPartialD2"],
Cell[CellGroupData[{
Cell[1358319, 53137, 134, 3, 26, "Input",
 CellTags->"LeftRightPartialD2"],
Cell[1358456, 53142, 303, 10, 33, "Output",
 CellTags->"LeftRightPartialD2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1358796, 53157, 128, 3, 26, "Input",
 CellTags->"LeftRightPartialD2"],
Cell[1358927, 53162, 475, 17, 33, "Output",
 CellTags->"LeftRightPartialD2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1359439, 53184, 265, 7, 26, "Input",
 CellTags->"LeftRightPartialD2"],
Cell[1359707, 53193, 361, 12, 33, "Output",
 CellTags->"LeftRightPartialD2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1360105, 53210, 126, 3, 26, "Input",
 CellTags->"LeftRightPartialD2"],
Cell[1360234, 53215, 866, 24, 93, "Output",
 CellTags->"LeftRightPartialD2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1361137, 53244, 384, 10, 26, "Input",
 CellTags->"LeftRightPartialD2"],
Cell[1361524, 53256, 397, 13, 33, "Output",
 CellTags->"LeftRightPartialD2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1361958, 53274, 126, 3, 26, "Input",
 CellTags->"LeftRightPartialD2"],
Cell[1362087, 53279, 1259, 31, 153, "Output",
 CellTags->"LeftRightPartialD2"]
}, Open  ]]
}, Open  ]],
Cell[1363373, 53314, 58, 1, 39, "ObjectName",
 CellTags->"LeviCivita"],
Cell[CellGroupData[{
Cell[1363456, 53319, 61, 1, 34, "Subsubsection",
 CellTags->"LeviCivita"],
Cell[1363520, 53322, 1210, 40, 80, "Text",
 CellTags->"LeviCivita"],
Cell[CellGroupData[{
Cell[1364755, 53366, 120, 3, 26, "Input",
 CellTags->"LeviCivita"],
Cell[1364878, 53371, 210, 6, 26, "Output",
 CellTags->"LeviCivita"]
}, Open  ]],
Cell[1365103, 53380, 463, 24, 24, "Text",
 CellTags->"LeviCivita"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1365603, 53409, 58, 1, 34, "Subsubsection",
 CellTags->"LeviCivita"],
Cell[CellGroupData[{
Cell[1365686, 53414, 187, 5, 26, "Input",
 CellTags->"LeviCivita"],
Cell[1365876, 53421, 534, 22, 26, "Output",
 CellTags->"LeviCivita"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1366447, 53448, 204, 6, 26, "Input",
 CellTags->"LeviCivita"],
Cell[1366654, 53456, 453, 18, 26, "Output",
 CellTags->"LeviCivita"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1367144, 53479, 180, 5, 26, "Input",
 CellTags->"LeviCivita"],
Cell[1367327, 53486, 355, 14, 26, "Output",
 CellTags->"LeviCivita"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1367719, 53505, 186, 5, 26, "Input",
 CellTags->"LeviCivita"],
Cell[1367908, 53512, 149, 4, 26, "Output",
 CellTags->"LeviCivita"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1368094, 53521, 242, 7, 26, "Input",
 CellTags->"LeviCivita"],
Cell[1368339, 53530, 374, 9, 38, "Output",
 CellTags->"LeviCivita"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1368750, 53544, 365, 10, 26, "Input",
 CellTags->"LeviCivita"],
Cell[1369118, 53556, 973, 41, 26, "Output",
 CellTags->"LeviCivita"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1370128, 53602, 113, 3, 26, "Input",
 CellTags->"LeviCivita"],
Cell[1370244, 53607, 167, 5, 26, "Output",
 CellTags->"LeviCivita"]
}, Open  ]],
Cell[1370426, 53615, 316, 9, 26, "Input",
 CellTags->"LeviCivita"],
Cell[CellGroupData[{
Cell[1370767, 53628, 403, 11, 26, "Input",
 CellTags->"LeviCivita"],
Cell[1371173, 53641, 339, 11, 26, "Output",
 CellTags->"LeviCivita"]
}, Open  ]],
Cell[1371527, 53655, 316, 9, 26, "Input",
 CellTags->"LeviCivita"],
Cell[CellGroupData[{
Cell[1371868, 53668, 179, 5, 26, "Input",
 CellTags->"LeviCivita"],
Cell[1372050, 53675, 241, 7, 26, "Output",
 CellTags->"LeviCivita"]
}, Open  ]],
Cell[1372306, 53685, 110, 3, 26, "Input",
 CellTags->"LeviCivita"]
}, Open  ]],
Cell[1372431, 53691, 65, 1, 41, "ObjectName",
 CellTags->"LeviCivitaSign"],
Cell[CellGroupData[{
Cell[1372521, 53696, 65, 1, 36, "Subsubsection",
 CellTags->"LeviCivitaSign"],
Cell[1372589, 53699, 203, 4, 43, "Text",
 CellTags->"LeviCivitaSign"],
Cell[1372795, 53705, 310, 14, 25, "Text",
 CellTags->"LeviCivitaSign"]
}, Open  ]],
Cell[1373120, 53722, 81, 1, 41, "ObjectName",
 CellTags->"ListIntegrals"],
Cell[CellGroupData[{
Cell[1373226, 53727, 64, 1, 36, "Subsubsection",
 CellTags->"ListIntegrals"],
Cell[1373293, 53730, 470, 8, 79, "Text",
 CellTags->"ListIntegrals"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1373800, 53743, 61, 1, 34, "Subsubsection",
 CellTags->"ListIntegrals"],
Cell[1373864, 53746, 80, 2, 26, "Input",
 CellTags->"ListIntegrals"]
}, Open  ]],
Cell[1373959, 53751, 44, 1, 41, "ObjectName",
 CellTags->"Li2"],
Cell[CellGroupData[{
Cell[1374028, 53756, 54, 1, 36, "Subsubsection",
 CellTags->"Li2"],
Cell[1374085, 53759, 117, 3, 25, "Text",
 CellTags->"Li2"],
Cell[1374205, 53764, 153, 8, 25, "Text",
 CellTags->"Li2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1374395, 53777, 51, 1, 34, "Subsubsection",
 CellTags->"Li2"],
Cell[CellGroupData[{
Cell[1374471, 53782, 100, 3, 26, "Input",
 CellTags->"Li2"],
Cell[1374574, 53787, 178, 6, 26, "Output",
 CellTags->"Li2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1374789, 53798, 107, 3, 26, "Input",
 CellTags->"Li2"],
Cell[1374899, 53803, 204, 6, 38, "Output",
 CellTags->"Li2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1375140, 53814, 253, 9, 49, "Input",
 CellTags->"Li2"],
Cell[1375396, 53825, 180, 6, 26, "Output",
 CellTags->"Li2"]
}, Open  ]]
}, Open  ]],
Cell[1375603, 53835, 44, 1, 41, "ObjectName",
 CellTags->"Li3"],
Cell[CellGroupData[{
Cell[1375672, 53840, 54, 1, 36, "Subsubsection",
 CellTags->"Li3"],
Cell[1375729, 53843, 118, 3, 25, "Text",
 CellTags->"Li3"],
Cell[1375850, 53848, 153, 8, 25, "Text",
 CellTags->"Li3"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1376040, 53861, 51, 1, 34, "Subsubsection",
 CellTags->"Li3"],
Cell[CellGroupData[{
Cell[1376116, 53866, 100, 3, 26, "Input",
 CellTags->"Li3"],
Cell[1376219, 53871, 178, 6, 26, "Output",
 CellTags->"Li3"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1376434, 53882, 107, 3, 26, "Input",
 CellTags->"Li3"],
Cell[1376544, 53887, 204, 6, 38, "Output",
 CellTags->"Li3"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1376785, 53898, 152, 5, 26, "Input",
 CellTags->"Li3"],
Cell[1376940, 53905, 203, 7, 45, "Output",
 CellTags->"Li3"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1377180, 53917, 204, 7, 50, "Input",
 CellTags->"Li3"],
Cell[1377387, 53926, 180, 6, 26, "Output",
 CellTags->"Li3"]
}, Open  ]]
}, Open  ]],
Cell[1377594, 53936, 45, 1, 41, "ObjectName",
 CellTags->"Loop"],
Cell[CellGroupData[{
Cell[1377664, 53941, 55, 1, 36, "Subsubsection",
 CellTags->"Loop"],
Cell[1377722, 53944, 94, 1, 25, "Text",
 CellTags->"Loop"]
}, Open  ]],
Cell[1377831, 53948, 61, 1, 41, "ObjectName",
 CellTags->"LorentzIndex"],
Cell[CellGroupData[{
Cell[1377917, 53953, 63, 1, 36, "Subsubsection",
 CellTags->"LorentzIndex"],
Cell[1377983, 53956, 243, 5, 43, "Text",
 CellTags->"LorentzIndex"],
Cell[1378229, 53963, 314, 14, 25, "Text",
 CellTags->"LorentzIndex"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1378580, 53982, 60, 1, 36, "Subsubsection",
 CellTags->"LorentzIndex"],
Cell[1378643, 53985, 89, 1, 24, "Text",
 CellTags->"LorentzIndex"],
Cell[CellGroupData[{
Cell[1378757, 53990, 125, 3, 26, "Input",
 CellTags->"LorentzIndex"],
Cell[1378885, 53995, 158, 4, 26, "Output",
 CellTags->"LorentzIndex"]
}, Open  ]],
Cell[1379058, 54002, 125, 3, 24, "Text",
 CellTags->"LorentzIndex"],
Cell[CellGroupData[{
Cell[1379208, 54009, 148, 4, 26, "Input",
 CellTags->"LorentzIndex"],
Cell[1379359, 54015, 158, 4, 26, "Output",
 CellTags->"LorentzIndex"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1379554, 54024, 106, 2, 26, "Input",
 CellTags->"LorentzIndex"],
Cell[1379663, 54028, 237, 8, 26, "Output",
 CellTags->"LorentzIndex"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1379937, 54041, 165, 4, 26, "Input",
 CellTags->"LorentzIndex"],
Cell[1380105, 54047, 154, 4, 26, "Output",
 CellTags->"LorentzIndex"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1380296, 54056, 207, 5, 26, "Input",
 CellTags->"LorentzIndex"],
Cell[1380506, 54063, 181, 5, 26, "Output",
 CellTags->"LorentzIndex"]
}, Open  ]],
Cell[1380702, 54071, 141, 4, 24, "Text",
 CellTags->"LorentzIndex"],
Cell[1380846, 54077, 137, 4, 26, "Input",
 CellTags->"LorentzIndex"],
Cell[CellGroupData[{
Cell[1381008, 54085, 347, 10, 26, "Input",
 CellTags->"LorentzIndex"],
Cell[1381358, 54097, 429, 17, 26, "Output",
 CellTags->"LorentzIndex"]
}, Open  ]],
Cell[1381802, 54117, 148, 4, 26, "Input",
 CellTags->"LorentzIndex"]
}, Open  ]],
Cell[1381965, 54124, 47, 1, 41, "ObjectName",
 CellTags->"Lower"],
Cell[CellGroupData[{
Cell[1382037, 54129, 56, 1, 36, "Subsubsection",
 CellTags->"Lower"],
Cell[1382096, 54132, 120, 3, 25, "Text",
 CellTags->"Lower"],
Cell[1382219, 54137, 402, 19, 25, "Text",
 CellTags->"Lower"]
}, Open  ]],
Cell[1382636, 54159, 59, 1, 41, "ObjectName",
 CellTags->"MakeContext"],
Cell[CellGroupData[{
Cell[1382720, 54164, 62, 1, 36, "Subsubsection",
 CellTags->"MakeContext"],
Cell[1382785, 54167, 238, 5, 43, "Text",
 CellTags->"MakeContext"],
Cell[1383026, 54174, 530, 24, 25, "Text",
 CellTags->"MakeContext"]
}, Open  ]],
Cell[1383571, 54201, 57, 1, 39, "ObjectName",
 CellTags->"Mandelstam"],
Cell[CellGroupData[{
Cell[1383653, 54206, 61, 1, 34, "Subsubsection",
 CellTags->"Mandelstam"],
Cell[1383717, 54209, 1575, 73, 58, "Text",
 CellTags->"Mandelstam"],
Cell[1385295, 54284, 758, 34, 24, "Text",
 CellTags->"Mandelstam"]
}, Open  ]],
Cell[1386068, 54321, 45, 1, 39, "ObjectName",
 CellTags->"Map2"],
Cell[CellGroupData[{
Cell[1386138, 54326, 55, 1, 34, "Subsubsection",
 CellTags->"Map2"],
Cell[1386196, 54329, 135, 4, 24, "Text",
 CellTags->"Map2"],
Cell[1386334, 54335, 169, 9, 24, "Text",
 CellTags->"Map2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1386540, 54349, 52, 1, 34, "Subsubsection",
 CellTags->"Map2"],
Cell[CellGroupData[{
Cell[1386617, 54354, 149, 5, 26, "Input",
 CellTags->"Map2"],
Cell[1386769, 54361, 244, 8, 26, "Output",
 CellTags->"Map2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1387050, 54374, 125, 4, 26, "Input",
 CellTags->"Map2"],
Cell[1387178, 54380, 171, 5, 26, "Output",
 CellTags->"Map2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1387386, 54390, 184, 6, 26, "Input",
 CellTags->"Map2"],
Cell[1387573, 54398, 235, 8, 26, "Output",
 CellTags->"Map2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1387845, 54411, 125, 4, 26, "Input",
 CellTags->"Map2"],
Cell[1387973, 54417, 171, 5, 26, "Output",
 CellTags->"Map2"]
}, Open  ]]
}, Open  ]],
Cell[1388171, 54426, 67, 1, 39, "ObjectName",
 CellTags->"MemoryAvailable"],
Cell[CellGroupData[{
Cell[1388263, 54431, 66, 1, 34, "Subsubsection",
 CellTags->"MemoryAvailable"],
Cell[1388332, 54434, 230, 5, 41, "Text",
 CellTags->"MemoryAvailable"],
Cell[1388565, 54441, 314, 14, 24, "Text",
 CellTags->"MemoryAvailable"]
}, Open  ]],
Cell[1388894, 54458, 49, 1, 39, "ObjectName",
 CellTags->"MemSet"],
Cell[CellGroupData[{
Cell[1388968, 54463, 57, 1, 34, "Subsubsection",
 CellTags->"MemSet"],
Cell[1389028, 54466, 345, 9, 41, "Text",
 CellTags->"MemSet"],
Cell[1389376, 54477, 198, 9, 24, "Text",
 CellTags->"MemSet"]
}, Open  ]],
Cell[1389589, 54489, 61, 1, 39, "ObjectName",
 CellTags->"MetricTensor"],
Cell[CellGroupData[{
Cell[1389675, 54494, 63, 1, 36, "Subsubsection",
 CellTags->"MetricTensor"],
Cell[1389741, 54497, 214, 7, 25, "Text",
 CellTags->"MetricTensor"],
Cell[1389958, 54506, 584, 29, 25, "Text",
 CellTags->"MetricTensor"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1390579, 54540, 60, 1, 34, "Subsubsection",
 CellTags->"MetricTensor"],
Cell[CellGroupData[{
Cell[1390664, 54545, 154, 4, 26, "Input",
 CellTags->"MetricTensor"],
Cell[1390821, 54551, 207, 6, 26, "Output",
 CellTags->"MetricTensor"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1391065, 54562, 137, 4, 26, "Input",
 CellTags->"MetricTensor"],
Cell[1391205, 54568, 151, 4, 26, "Output",
 CellTags->"MetricTensor"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1391393, 54577, 202, 5, 26, "Input",
 CellTags->"MetricTensor"],
Cell[1391598, 54584, 207, 6, 26, "Output",
 CellTags->"MetricTensor"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1391842, 54595, 137, 4, 26, "Input",
 CellTags->"MetricTensor"],
Cell[1391982, 54601, 153, 4, 27, "Output",
 CellTags->"MetricTensor"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1392172, 54610, 181, 5, 26, "Input",
 CellTags->"MetricTensor"],
Cell[1392356, 54617, 198, 5, 38, "Output",
 CellTags->"MetricTensor"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1392591, 54627, 230, 6, 26, "Input",
 CellTags->"MetricTensor"],
Cell[1392824, 54635, 246, 6, 38, "Output",
 CellTags->"MetricTensor"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1393107, 54646, 213, 6, 26, "Input",
 CellTags->"MetricTensor"],
Cell[1393323, 54654, 251, 6, 38, "Output",
 CellTags->"MetricTensor"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1393611, 54665, 263, 7, 26, "Input",
 CellTags->"MetricTensor"],
Cell[1393877, 54674, 297, 7, 38, "Output",
 CellTags->"MetricTensor"]
}, Open  ]]
}, Open  ]],
Cell[1394201, 54685, 49, 1, 41, "ObjectName",
 CellTags->"MLimit"],
Cell[CellGroupData[{
Cell[1394275, 54690, 57, 1, 36, "Subsubsection",
 CellTags->"MLimit"],
Cell[1394335, 54693, 116, 3, 25, "Text",
 CellTags->"MLimit"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1394488, 54701, 54, 1, 34, "Subsubsection",
 CellTags->"MLimit"],
Cell[CellGroupData[{
Cell[1394567, 54706, 419, 14, 26, "Input",
 CellTags->"MLimit"],
Cell[1394989, 54722, 472, 15, 46, "Output",
 CellTags->"MLimit"]
}, Open  ]]
}, Open  ]],
Cell[1395488, 54741, 53, 1, 41, "ObjectName",
 CellTags->"Momentum"],
Cell[CellGroupData[{
Cell[1395566, 54746, 59, 1, 36, "Subsubsection",
 CellTags->"Momentum"],
Cell[1395628, 54749, 258, 5, 43, "Text",
 CellTags->"Momentum"],
Cell[1395889, 54756, 530, 24, 25, "Text",
 CellTags->"Momentum"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1396456, 54785, 56, 1, 36, "Subsubsection",
 CellTags->"Momentum"],
Cell[1396515, 54788, 75, 1, 24, "Text",
 CellTags->"Momentum"],
Cell[CellGroupData[{
Cell[1396615, 54793, 110, 3, 26, "Input",
 CellTags->"Momentum"],
Cell[1396728, 54798, 147, 4, 29, "Output",
 CellTags->"Momentum"]
}, Open  ]],
Cell[1396890, 54805, 138, 4, 24, "Text",
 CellTags->"Momentum"],
Cell[CellGroupData[{
Cell[1397053, 54813, 133, 4, 26, "Input",
 CellTags->"Momentum"],
Cell[1397189, 54819, 147, 4, 29, "Output",
 CellTags->"Momentum"]
}, Open  ]],
Cell[1397351, 54826, 86, 1, 24, "Text",
 CellTags->"Momentum"],
Cell[CellGroupData[{
Cell[1397462, 54831, 169, 5, 26, "Input",
 CellTags->"Momentum"],
Cell[1397634, 54838, 190, 5, 38, "Output",
 CellTags->"Momentum"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1397861, 54848, 153, 5, 26, "Input",
 CellTags->"Momentum"],
Cell[1398017, 54855, 165, 5, 28, "Output",
 CellTags->"Momentum"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1398219, 54865, 111, 3, 26, "Input",
 CellTags->"Momentum"],
Cell[1398333, 54870, 185, 5, 38, "Output",
 CellTags->"Momentum"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1398555, 54880, 259, 8, 26, "Input",
 CellTags->"Momentum"],
Cell[1398817, 54890, 242, 8, 28, "Output",
 CellTags->"Momentum"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1399096, 54903, 111, 3, 26, "Input",
 CellTags->"Momentum"],
Cell[1399210, 54908, 273, 8, 38, "Output",
 CellTags->"Momentum"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1399520, 54921, 148, 4, 26, "Input",
 CellTags->"Momentum"],
Cell[1399671, 54927, 225, 6, 38, "Output",
 CellTags->"Momentum"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1399933, 54938, 149, 4, 26, "Input",
 CellTags->"Momentum"],
Cell[1400085, 54944, 273, 8, 38, "Output",
 CellTags->"Momentum"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1400395, 54957, 216, 7, 26, "Input",
 CellTags->"Momentum"],
Cell[1400614, 54966, 190, 5, 38, "Output",
 CellTags->"Momentum"]
}, Open  ]],
Cell[1400819, 54974, 132, 4, 26, "Input",
 CellTags->"Momentum"]
}, Open  ]],
Cell[1400966, 54981, 67, 1, 39, "ObjectName",
 CellTags->"MomentumCombine"],
Cell[CellGroupData[{
Cell[1401058, 54986, 66, 1, 34, "Subsubsection",
 CellTags->"MomentumCombine"],
Cell[1401127, 54989, 183, 4, 41, "Text",
 CellTags->"MomentumCombine"],
Cell[1401313, 54995, 585, 23, 24, "Text",
 CellTags->"MomentumCombine"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1401935, 55023, 63, 1, 34, "Subsubsection",
 CellTags->"MomentumCombine"],
Cell[CellGroupData[{
Cell[1402023, 55028, 303, 9, 26, "Input",
 CellTags->"MomentumCombine"],
Cell[1402329, 55039, 258, 7, 38, "Output",
 CellTags->"MomentumCombine"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1402624, 55051, 288, 9, 26, "Input",
 CellTags->"MomentumCombine"],
Cell[1402915, 55062, 260, 8, 28, "Output",
 CellTags->"MomentumCombine"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1403212, 55075, 125, 3, 26, "Input",
 CellTags->"MomentumCombine"],
Cell[1403340, 55080, 260, 8, 28, "Output",
 CellTags->"MomentumCombine"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1403637, 55093, 163, 5, 26, "Input",
 CellTags->"MomentumCombine"],
Cell[1403803, 55100, 303, 9, 38, "Output",
 CellTags->"MomentumCombine"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1404143, 55114, 170, 5, 26, "Input",
 CellTags->"MomentumCombine"],
Cell[1404316, 55121, 260, 8, 28, "Output",
 CellTags->"MomentumCombine"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1404613, 55134, 121, 3, 26, "Input",
 CellTags->"MomentumCombine"],
Cell[1404737, 55139, 301, 9, 38, "Output",
 CellTags->"MomentumCombine"]
}, Open  ]],
Cell[1405053, 55151, 115, 3, 26, "Input",
 CellTags->"MomentumCombine"]
}, Open  ]],
Cell[1405183, 55157, 69, 1, 39, "ObjectName",
 CellTags->"MomentumCombine2"],
Cell[CellGroupData[{
Cell[1405277, 55162, 67, 1, 34, "Subsubsection",
 CellTags->"MomentumCombine2"],
Cell[1405347, 55165, 192, 4, 41, "Text",
 CellTags->"MomentumCombine2"],
Cell[1405542, 55171, 589, 23, 24, "Text",
 CellTags->"MomentumCombine2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1406168, 55199, 64, 1, 34, "Subsubsection",
 CellTags->"MomentumCombine2"],
Cell[CellGroupData[{
Cell[1406257, 55204, 434, 13, 26, "Input",
 CellTags->"MomentumCombine2"],
Cell[1406694, 55219, 517, 21, 28, "Output",
 CellTags->"MomentumCombine2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1407248, 55245, 164, 5, 26, "Input",
 CellTags->"MomentumCombine2"],
Cell[1407415, 55252, 351, 10, 38, "Output",
 CellTags->"MomentumCombine2"]
}, Open  ]]
}, Open  ]],
Cell[1407793, 55266, 65, 1, 39, "ObjectName",
 CellTags->"MomentumExpand"],
Cell[CellGroupData[{
Cell[1407883, 55271, 65, 1, 34, "Subsubsection",
 CellTags->"MomentumExpand"],
Cell[1407951, 55274, 149, 4, 24, "Text",
 CellTags->"MomentumExpand"],
Cell[1408103, 55280, 349, 14, 24, "Text",
 CellTags->"MomentumExpand"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1408489, 55299, 62, 1, 34, "Subsubsection",
 CellTags->"MomentumExpand"],
Cell[CellGroupData[{
Cell[1408576, 55304, 218, 6, 26, "Input",
 CellTags->"MomentumExpand"],
Cell[1408797, 55312, 231, 6, 38, "Output",
 CellTags->"MomentumExpand"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1409065, 55323, 168, 5, 26, "Input",
 CellTags->"MomentumExpand"],
Cell[1409236, 55330, 292, 10, 28, "Output",
 CellTags->"MomentumExpand"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1409565, 55345, 116, 3, 26, "Input",
 CellTags->"MomentumExpand"],
Cell[1409684, 55350, 289, 8, 38, "Output",
 CellTags->"MomentumExpand"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1410010, 55363, 211, 6, 26, "Input",
 CellTags->"MomentumExpand"],
Cell[1410224, 55371, 400, 16, 28, "Output",
 CellTags->"MomentumExpand"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1410661, 55392, 116, 3, 26, "Input",
 CellTags->"MomentumExpand"],
Cell[1410780, 55397, 326, 9, 38, "Output",
 CellTags->"MomentumExpand"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1411143, 55411, 236, 7, 26, "Input",
 CellTags->"MomentumExpand"],
Cell[1411382, 55420, 428, 17, 28, "Output",
 CellTags->"MomentumExpand"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1411847, 55442, 116, 3, 26, "Input",
 CellTags->"MomentumExpand"],
Cell[1411966, 55447, 409, 12, 38, "Output",
 CellTags->"MomentumExpand"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1412412, 55464, 112, 3, 26, "Input",
 CellTags->"MomentumExpand"],
Cell[1412527, 55469, 602, 24, 28, "Output",
 CellTags->"MomentumExpand"]
}, Open  ]]
}, Open  ]],
Cell[1413156, 55497, 41, 1, 41, "ObjectName",
 CellTags->"MT"],
Cell[CellGroupData[{
Cell[1413222, 55502, 53, 1, 36, "Subsubsection",
 CellTags->"MT"],
Cell[1413278, 55505, 182, 7, 25, "Text",
 CellTags->"MT"],
Cell[1413463, 55514, 604, 29, 25, "Text",
 CellTags->"MT"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1414104, 55548, 50, 1, 34, "Subsubsection",
 CellTags->"MT"],
Cell[CellGroupData[{
Cell[1414179, 55553, 134, 4, 26, "Input",
 CellTags->"MT"],
Cell[1414316, 55559, 197, 6, 26, "Output",
 CellTags->"MT"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1414550, 55570, 263, 8, 26, "Input",
 CellTags->"MT"],
Cell[1414816, 55580, 141, 4, 26, "Output",
 CellTags->"MT"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1414994, 55589, 157, 5, 26, "Input",
 CellTags->"MT"],
Cell[1415154, 55596, 180, 5, 38, "Output",
 CellTags->"MT"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1415371, 55606, 189, 6, 26, "Input",
 CellTags->"MT"],
Cell[1415563, 55614, 260, 7, 38, "Output",
 CellTags->"MT"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1415860, 55626, 225, 8, 26, "Input",
 CellTags->"MT"],
Cell[1416088, 55636, 178, 5, 38, "Output",
 CellTags->"MT"]
}, Open  ]]
}, Open  ]],
Cell[1416293, 55645, 43, 1, 39, "ObjectName",
 CellTags->"MTD"],
Cell[CellGroupData[{
Cell[1416361, 55650, 54, 1, 34, "Subsubsection",
 CellTags->"MTD"],
Cell[1416418, 55653, 184, 7, 24, "Text",
 CellTags->"MTD"],
Cell[1416605, 55662, 602, 29, 24, "Text",
 CellTags->"MTD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1417244, 55696, 51, 1, 34, "Subsubsection",
 CellTags->"MTD"],
Cell[CellGroupData[{
Cell[1417320, 55701, 136, 4, 26, "Input",
 CellTags->"MTD"],
Cell[1417459, 55707, 198, 6, 26, "Output",
 CellTags->"MTD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1417694, 55718, 266, 8, 26, "Input",
 CellTags->"MTD"],
Cell[1417963, 55728, 144, 4, 27, "Output",
 CellTags->"MTD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1418144, 55737, 175, 6, 26, "Input",
 CellTags->"MTD"],
Cell[1418322, 55745, 193, 5, 38, "Output",
 CellTags->"MTD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1418552, 55755, 207, 7, 26, "Input",
 CellTags->"MTD"],
Cell[1418762, 55764, 324, 9, 38, "Output",
 CellTags->"MTD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1419123, 55778, 235, 8, 26, "Input",
 CellTags->"MTD"],
Cell[1419361, 55788, 188, 5, 38, "Output",
 CellTags->"MTD"]
}, Open  ]]
}, Open  ]],
Cell[1419576, 55797, 67, 1, 39, "ObjectName",
 CellTags->"NegativeInteger"],
Cell[CellGroupData[{
Cell[1419668, 55802, 66, 1, 34, "Subsubsection",
 CellTags->"NegativeInteger"],
Cell[1419737, 55805, 141, 4, 24, "Text",
 CellTags->"NegativeInteger"],
Cell[1419881, 55811, 186, 9, 24, "Text",
 CellTags->"NegativeInteger"]
}, Open  ]],
Cell[1420082, 55823, 41, 1, 39, "ObjectName",
 CellTags->"Nf"],
Cell[CellGroupData[{
Cell[1420148, 55828, 53, 1, 34, "Subsubsection",
 CellTags->"Nf"],
Cell[1420204, 55831, 66, 1, 24, "Text",
 CellTags->"Nf"],
Cell[1420273, 55834, 685, 29, 24, "Text",
 CellTags->"Nf"]
}, Open  ]],
Cell[1420973, 55866, 51, 1, 39, "ObjectName",
 CellTags->"Nielsen"],
Cell[CellGroupData[{
Cell[1421049, 55871, 58, 1, 34, "Subsubsection",
 CellTags->"Nielsen"],
Cell[1421110, 55874, 86, 1, 24, "Text",
 CellTags->"Nielsen"],
Cell[CellGroupData[{
Cell[1421221, 55879, 114, 3, 26, "Input",
 CellTags->"Nielsen"],
Cell[1421338, 55884, 211, 7, 26, "Output",
 CellTags->"Nielsen"]
}, Open  ]],
Cell[1421564, 55894, 199, 9, 25, "Text",
 CellTags->"Nielsen"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1421800, 55908, 55, 1, 34, "Subsubsection",
 CellTags->"Nielsen"],
Cell[CellGroupData[{
Cell[1421880, 55913, 141, 4, 26, "Input",
 CellTags->"Nielsen"],
Cell[1422024, 55919, 293, 11, 26, "Output",
 CellTags->"Nielsen"]
}, Open  ]],
Cell[1422332, 55933, 176, 4, 24, "Text",
 CellTags->"Nielsen"],
Cell[CellGroupData[{
Cell[1422533, 55941, 172, 5, 26, "Input",
 CellTags->"Nielsen"],
Cell[1422708, 55948, 167, 4, 26, "Output",
 CellTags->"Nielsen"]
}, Open  ]],
Cell[1422890, 55955, 71, 1, 24, "Text",
 CellTags->"Nielsen"],
Cell[CellGroupData[{
Cell[1422986, 55960, 467, 14, 26, "Input",
 CellTags->"Nielsen"],
Cell[1423456, 55976, 362, 14, 43, "Output",
 CellTags->"Nielsen"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1423855, 55995, 190, 5, 26, "Input",
 CellTags->"Nielsen"],
Cell[1424048, 56002, 615, 21, 44, "Output",
 CellTags->"Nielsen"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1424700, 56028, 190, 5, 26, "Input",
 CellTags->"Nielsen"],
Cell[1424893, 56035, 849, 28, 47, "Output",
 CellTags->"Nielsen"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1425779, 56068, 190, 5, 26, "Input",
 CellTags->"Nielsen"],
Cell[1425972, 56075, 182, 6, 26, "Output",
 CellTags->"Nielsen"]
}, Open  ]]
}, Open  ]],
Cell[1426181, 56085, 61, 1, 70, "ObjectName",
 CellTags->"NonCommFreeQ"],
Cell[CellGroupData[{
Cell[1426267, 56090, 63, 1, 70, "Subsubsection",
 CellTags->"NonCommFreeQ"],
Cell[1426333, 56093, 244, 5, 70, "Text",
 CellTags->"NonCommFreeQ"],
Cell[1426580, 56100, 519, 24, 70, "Text",
 CellTags->"NonCommFreeQ"]
}, Open  ]],
Cell[1427114, 56127, 53, 1, 70, "ObjectName",
 CellTags->"NonCommQ"],
Cell[CellGroupData[{
Cell[1427192, 56132, 59, 1, 70, "Subsubsection",
 CellTags->"NonCommQ"],
Cell[1427254, 56135, 203, 4, 70, "Text",
 CellTags->"NonCommQ"],
Cell[1427460, 56141, 527, 24, 70, "Text",
 CellTags->"NonCommQ"]
}, Open  ]],
Cell[1428002, 56168, 65, 1, 70, "ObjectName",
 CellTags->"NonCommutative"],
Cell[CellGroupData[{
Cell[1428092, 56173, 65, 1, 70, "Subsubsection",
 CellTags->"NonCommutative"],
Cell[1428160, 56176, 150, 4, 70, "Text",
 CellTags->"NonCommutative"],
Cell[1428313, 56182, 333, 14, 70, "Text",
 CellTags->"NonCommutative"]
}, Open  ]],
Cell[1428661, 56199, 49, 1, 70, "ObjectName",
 CellTags->"NTerms"],
Cell[CellGroupData[{
Cell[1428735, 56204, 57, 1, 70, "Subsubsection",
 CellTags->"NTerms"],
Cell[1428795, 56207, 154, 4, 70, "Text",
 CellTags->"NTerms"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1428986, 56216, 54, 1, 34, "Subsubsection",
 CellTags->"NTerms"],
Cell[CellGroupData[{
Cell[1429065, 56221, 129, 4, 26, "Input",
 CellTags->"NTerms"],
Cell[1429197, 56227, 147, 4, 26, "Output",
 CellTags->"NTerms"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1429381, 56236, 139, 4, 26, "Input",
 CellTags->"NTerms"],
Cell[1429523, 56242, 145, 4, 26, "Output",
 CellTags->"NTerms"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1429705, 56251, 106, 3, 26, "Input",
 CellTags->"NTerms"],
Cell[1429814, 56256, 147, 4, 26, "Output",
 CellTags->"NTerms"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1429998, 56265, 106, 3, 26, "Input",
 CellTags->"NTerms"],
Cell[1430107, 56270, 145, 4, 26, "Output",
 CellTags->"NTerms"]
}, Open  ]]
}, Open  ]],
Cell[1430279, 56278, 79, 1, 70, "ObjectName",
 CellTags->"NumberOfMetricTensors"],
Cell[CellGroupData[{
Cell[1430383, 56283, 72, 1, 70, "Subsubsection",
 CellTags->"NumberOfMetricTensors"],
Cell[1430458, 56286, 95, 1, 70, "Text",
 CellTags->"NumberOfMetricTensors"],
Cell[1430556, 56289, 167, 9, 70, "Text",
 CellTags->"NonCommQ"]
}, Open  ]],
Cell[1430738, 56301, 67, 1, 70, "ObjectName",
 CellTags->"NumericalFactor"],
Cell[CellGroupData[{
Cell[1430830, 56306, 66, 1, 52, "Subsubsection",
 CellTags->"NumericalFactor"],
Cell[1430899, 56309, 119, 3, 70, "Text",
 CellTags->"NumericalFactor"]
}, Open  ]],
Cell[1431033, 56315, 55, 1, 57, "ObjectName",
 CellTags->"NumericQ1"],
Cell[CellGroupData[{
Cell[1431113, 56320, 60, 1, 52, "Subsubsection",
 CellTags->"NumericQ1"],
Cell[1431176, 56323, 139, 4, 25, "Text",
 CellTags->"NumericQ1"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1431352, 56332, 57, 1, 34, "Subsubsection",
 CellTags->"NumericQ1"],
Cell[CellGroupData[{
Cell[1431434, 56337, 224, 7, 30, "Input",
 CellTags->"NumericQ1"],
Cell[1431661, 56346, 152, 4, 26, "Output",
 CellTags->"NumericQ1"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1431850, 56355, 271, 9, 30, "Input",
 CellTags->"NumericQ1"],
Cell[1432124, 56366, 154, 4, 26, "Output",
 CellTags->"NumericQ1"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1432315, 56375, 311, 10, 30, "Input",
 CellTags->"NumericQ1"],
Cell[1432629, 56387, 151, 4, 26, "Output",
 CellTags->"NumericQ1"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[1432829, 56397, 69, 1, 41, "ObjectName",
 CellTags->"OneLoop"],
Cell[1432901, 56400, 619, 16, 61, "Text",
 CellTags->"OneLoop"],
Cell[1433523, 56418, 968, 26, 61, "Text",
 CellTags->"OneLoop"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1434528, 56449, 58, 1, 36, "Subsubsection",
 CellTags->"OneLoop"],
Cell[1434589, 56452, 391, 12, 43, "Text",
 CellTags->"OneLoop"],
Cell[1434983, 56466, 352, 6, 58, "Text",
 CellTags->"OneLoop"],
Cell[CellGroupData[{
Cell[1435360, 56476, 114, 3, 26, "Input",
 CellTags->"OneLoop"],
Cell[1435477, 56481, 1317, 30, 87, "Output",
 CellTags->"OneLoop"]
}, Open  ]],
Cell[1436809, 56514, 781, 39, 25, "Text",
 CellTags->"OneLoop"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1437627, 56558, 55, 1, 34, "Subsubsection",
 CellTags->"OneLoop"],
Cell[CellGroupData[{
Cell[1437707, 56563, 275, 10, 26, "Input",
 CellTags->"OneLoop"],
Cell[1437985, 56575, 365, 12, 50, "Output",
 CellTags->"OneLoop"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1438387, 56592, 131, 4, 26, "Input",
 CellTags->"OneLoop"],
Cell[1438521, 56598, 310, 11, 29, "Output",
 CellTags->"OneLoop"]
}, Open  ]],
Cell[1438846, 56612, 363, 11, 26, "Input",
 CellTags->"OneLoop"],
Cell[1439212, 56625, 588, 20, 27, "Text",
 CellTags->"OneLoop"],
Cell[CellGroupData[{
Cell[1439825, 56649, 908, 30, 80, "Input",
 CellTags->"OneLoop"],
Cell[1440736, 56681, 1913, 66, 55, "Output",
 CellTags->"OneLoop"]
}, Open  ]],
Cell[1442664, 56750, 219, 5, 41, "Text",
 CellTags->"OneLoop"],
Cell[CellGroupData[{
Cell[1442908, 56759, 131, 4, 26, "Input",
 CellTags->"OneLoop"],
Cell[1443042, 56765, 2448, 80, 56, "Output",
 CellTags->"OneLoop"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1445527, 56850, 113, 3, 26, "Input",
 CellTags->"OneLoop"],
Cell[1445643, 56855, 1749, 59, 50, "Output",
 CellTags->"OneLoop"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1447429, 56919, 333, 11, 26, "Input",
 CellTags->"OneLoop"],
Cell[1447765, 56932, 486, 18, 52, "Output",
 CellTags->"OneLoop"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1448288, 56955, 131, 4, 26, "Input",
 CellTags->"OneLoop"],
Cell[1448422, 56961, 1922, 67, 53, "Output",
 CellTags->"OneLoop"]
}, Open  ]],
Cell[1450359, 57031, 106, 3, 26, "Input",
 CellTags->"OneLoop"]
}, Open  ]],
Cell[1450480, 57037, 67, 1, 41, "ObjectName",
 CellTags->"OneLoopSimplify"],
Cell[CellGroupData[{
Cell[1450572, 57042, 66, 1, 36, "Subsubsection",
 CellTags->"OneLoopSimplify"],
Cell[1450641, 57045, 172, 4, 24, "Text",
 CellTags->"OneLoopSimplify"],
Cell[CellGroupData[{
Cell[1450838, 57053, 130, 3, 26, "Input",
 CellTags->"OneLoopSimplify"],
Cell[1450971, 57058, 779, 19, 47, "Output",
 CellTags->"OneLoopSimplify"]
}, Open  ]],
Cell[1451765, 57080, 376, 19, 25, "Text",
 CellTags->"OneLoopSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1452178, 57104, 63, 1, 34, "Subsubsection",
 CellTags->"OneLoopSimplify"],
Cell[CellGroupData[{
Cell[1452266, 57109, 341, 11, 26, "Input",
 CellTags->"OneLoopSimplify"],
Cell[1452610, 57122, 494, 18, 52, "Output",
 CellTags->"OneLoopSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1453141, 57145, 147, 4, 26, "Input",
 CellTags->"OneLoopSimplify"],
Cell[1453291, 57151, 492, 18, 52, "Output",
 CellTags->"OneLoopSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1453820, 57174, 203, 6, 26, "Input",
 CellTags->"OneLoopSimplify"],
Cell[1454026, 57182, 413, 15, 51, "Output",
 CellTags->"OneLoopSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1454476, 57202, 401, 12, 26, "Input",
 CellTags->"OneLoopSimplify"],
Cell[1454880, 57216, 624, 23, 51, "Output",
 CellTags->"OneLoopSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1455541, 57244, 152, 4, 26, "Input",
 CellTags->"OneLoopSimplify"],
Cell[1455696, 57250, 7210, 262, 102, "Output",
 CellTags->"OneLoopSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1462943, 57517, 253, 9, 26, "Input",
 CellTags->"OneLoopSimplify"],
Cell[1463199, 57528, 4318, 163, 100, "Output",
 CellTags->"OneLoopSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1467554, 57696, 424, 15, 26, "Input",
 CellTags->"OneLoopSimplify"],
Cell[1467981, 57713, 1552, 59, 52, "Output",
 CellTags->"OneLoopSimplify"]
}, Open  ]]
}, Open  ]],
Cell[1469560, 57776, 57, 1, 41, "ObjectName",
 CellTags->"OneLoopSum"],
Cell[CellGroupData[{
Cell[1469642, 57781, 61, 1, 36, "Subsubsection",
 CellTags->"OneLoopSum"],
Cell[1469706, 57784, 190, 4, 43, "Text",
 CellTags->"OneLoopSum"],
Cell[1469899, 57790, 178, 9, 25, "Text",
 CellTags->"OneLoopSum"]
}, Open  ]],
Cell[1470092, 57802, 43, 1, 41, "ObjectName",
 CellTags->"OPE"],
Cell[CellGroupData[{
Cell[1470160, 57807, 54, 1, 36, "Subsubsection",
 CellTags->"OPE"],
Cell[1470217, 57810, 170, 4, 51, "Text",
 CellTags->"OPE"],
Cell[1470390, 57816, 86, 2, 24, "Text"],
Cell[CellGroupData[{
Cell[1470501, 57822, 60, 1, 26, "Input"],
Cell[1470564, 57825, 198, 6, 26, "Output"]
}, Open  ]],
Cell[1470777, 57834, 174, 9, 25, "Text",
 CellTags->"OPE"]
}, Open  ]],
Cell[1470966, 57846, 53, 1, 41, "ObjectName",
 CellTags->"OPEDelta"],
Cell[CellGroupData[{
Cell[1471044, 57851, 59, 1, 36, "Subsubsection",
 CellTags->"OPEDelta"],
Cell[1471106, 57854, 140, 4, 25, "Text",
 CellTags->"OPEDelta"],
Cell[1471249, 57860, 212, 9, 25, "Text",
 CellTags->"OPEDelta"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1471498, 57874, 56, 1, 34, "Subsubsection",
 CellTags->"OPEDelta"],
Cell[CellGroupData[{
Cell[1471579, 57879, 146, 4, 26, "Input",
 CellTags->"OPEDelta"],
Cell[1471728, 57885, 221, 7, 29, "Output",
 CellTags->"OPEDelta"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1471986, 57897, 133, 4, 26, "Input",
 CellTags->"OPEDelta"],
Cell[1472122, 57903, 149, 4, 26, "Output",
 CellTags->"OPEDelta"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1472308, 57912, 141, 4, 26, "Input",
 CellTags->"OPEDelta"],
Cell[1472452, 57918, 149, 4, 26, "Output",
 CellTags->"OPEDelta"]
}, Open  ]]
}, Open  ]],
Cell[1472628, 57926, 81, 1, 41, "ObjectName",
 CellTags->"OPEi"],
Cell[CellGroupData[{
Cell[1472734, 57931, 55, 1, 36, "Subsubsection",
 CellTags->"OPEi"],
Cell[1472792, 57934, 171, 4, 25, "Text",
 CellTags->"OPEi"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1473000, 57943, 52, 1, 34, "Subsubsection",
 CellTags->"OPEi"],
Cell[CellGroupData[{
Cell[1473077, 57948, 75, 2, 26, "Input",
 CellTags->"OPEi"],
Cell[1473155, 57952, 143, 4, 27, "Output",
 CellTags->"OPEi"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1473335, 57961, 259, 7, 26, "Input",
 CellTags->"OPEi"],
Cell[1473597, 57970, 270, 8, 26, "Output",
 CellTags->"OPEi"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1473904, 57983, 1076, 40, 46, "Input",
 CellTags->"OPEi"],
Cell[1474983, 58025, 244, 7, 26, "Output",
 CellTags->"OPEi"]
}, Open  ]],
Cell[1475242, 58035, 55, 1, 24, "Text",
 CellTags->"OPEi"],
Cell[CellGroupData[{
Cell[1475322, 58040, 519, 17, 26, "Input",
 CellTags->"OPEi"],
Cell[1475844, 58059, 571, 20, 26, "Output",
 CellTags->"OPEi"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1476452, 58084, 534, 19, 26, "Input",
 CellTags->"OPEi"],
Cell[1476989, 58105, 454, 16, 26, "Output",
 CellTags->"OPEi"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1477480, 58126, 519, 17, 26, "Input",
 CellTags->"OPEi"],
Cell[1478002, 58145, 571, 20, 26, "Output",
 CellTags->"OPEi"]
}, Open  ]]
}, Open  ]],
Cell[1478600, 58169, 73, 1, 41, "ObjectName",
 CellTags->"OPEInsert"],
Cell[CellGroupData[{
Cell[1478698, 58174, 60, 1, 36, "Subsubsection",
 CellTags->"OPEInsert"],
Cell[1478761, 58177, 264, 5, 43, "Text",
 CellTags->"OPEInsert"],
Cell[1479028, 58184, 165, 9, 25, "Text",
 CellTags->"OPEInsert"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1479230, 58198, 57, 1, 34, "Subsubsection",
 CellTags->"OPEInsert"],
Cell[1479290, 58201, 76, 2, 26, "Input",
 CellTags->"OPEInsert"]
}, Open  ]],
Cell[1479381, 58206, 67, 1, 41, "ObjectName",
 CellTags->"OPEInt"],
Cell[CellGroupData[{
Cell[1479473, 58211, 57, 1, 36, "Subsubsection",
 CellTags->"OPEInt"],
Cell[1479533, 58214, 257, 5, 43, "Text",
 CellTags->"OPEInt"],
Cell[1479793, 58221, 162, 9, 25, "Text",
 CellTags->"OPEInt"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1479992, 58235, 54, 1, 34, "Subsubsection",
 CellTags->"OPEInt"],
Cell[1480049, 58238, 73, 2, 26, "Input",
 CellTags->"OPEInt"]
}, Open  ]],
Cell[1480137, 58243, 77, 1, 41, "ObjectName",
 CellTags->"OPEIntDelta"],
Cell[CellGroupData[{
Cell[1480239, 58248, 62, 1, 36, "Subsubsection",
 CellTags->"OPEIntDelta"],
Cell[1480304, 58251, 478, 10, 87, "Text",
 CellTags->"OPEIntDelta"],
Cell[1480785, 58263, 167, 9, 25, "Text",
 CellTags->"OPEIntDelta"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1480989, 58277, 59, 1, 34, "Subsubsection",
 CellTags->"OPEIntDelta"],
Cell[1481051, 58280, 78, 2, 26, "Input",
 CellTags->"OPEIntDelta"]
}, Open  ]],
Cell[1481144, 58285, 79, 1, 41, "ObjectName",
 CellTags->"OPEIntegrate"],
Cell[CellGroupData[{
Cell[1481248, 58290, 63, 1, 36, "Subsubsection",
 CellTags->"OPEIntegrate"],
Cell[1481314, 58293, 266, 5, 43, "Text",
 CellTags->"OPEIntegrate"],
Cell[1481583, 58300, 168, 9, 25, "Text",
 CellTags->"OPEIntegrate"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1481788, 58314, 60, 1, 34, "Subsubsection",
 CellTags->"OPEIntegrate"],
Cell[1481851, 58317, 79, 2, 26, "Input",
 CellTags->"OPEIntegrate"]
}, Open  ]],
Cell[1481945, 58322, 89, 1, 41, "ObjectName",
 CellTags->"OPEIntegrateDelta"],
Cell[CellGroupData[{
Cell[1482059, 58327, 68, 1, 36, "Subsubsection",
 CellTags->"OPEIntegrateDelta"],
Cell[1482130, 58330, 499, 10, 87, "Text",
 CellTags->"OPEIntegrateDelta"],
Cell[1482632, 58342, 173, 9, 25, "Text",
 CellTags->"OPEIntegrateDelta"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1482842, 58356, 65, 1, 34, "Subsubsection",
 CellTags->"OPEIntegrateDelta"],
Cell[1482910, 58359, 84, 2, 26, "Input",
 CellTags->"OPEIntegrateDelta"]
}, Open  ]],
Cell[1483009, 58364, 65, 1, 41, "ObjectName",
 CellTags->"OPESumExplicit"],
Cell[CellGroupData[{
Cell[1483099, 58369, 65, 1, 36, "Subsubsection",
 CellTags->"OPESumExplicit"],
Cell[1483167, 58372, 85, 1, 25, "Text",
 CellTags->"OPESumExplicit"],
Cell[1483255, 58375, 307, 14, 25, "Text",
 CellTags->"OPESumExplicit"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1483599, 58394, 62, 1, 34, "Subsubsection",
 CellTags->"OPESumExplicit"],
Cell[CellGroupData[{
Cell[1483686, 58399, 387, 13, 26, "Input",
 CellTags->"OPESumExplicit"],
Cell[1484076, 58414, 449, 16, 57, "Output",
 CellTags->"OPESumExplicit"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1484562, 58435, 123, 3, 26, "Input",
 CellTags->"OPESumExplicit"],
Cell[1484688, 58440, 450, 17, 46, "Output",
 CellTags->"OPESumExplicit"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1485175, 58462, 554, 18, 26, "Input",
 CellTags->"OPESumExplicit"],
Cell[1485732, 58482, 744, 28, 59, "Output",
 CellTags->"OPESumExplicit"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1486513, 58515, 123, 3, 26, "Input",
 CellTags->"OPESumExplicit"],
Cell[1486639, 58520, 1791, 67, 95, "Output",
 CellTags->"OPESumExplicit"]
}, Open  ]],
Cell[1488445, 58590, 157, 5, 26, "Input",
 CellTags->"OPESumExplicit"]
}, Open  ]],
Cell[1488617, 58598, 49, 1, 70, "ObjectName",
 CellTags->"OPESum"],
Cell[CellGroupData[{
Cell[1488691, 58603, 57, 1, 70, "Subsubsection",
 CellTags->"OPESum"],
Cell[1488751, 58606, 127, 4, 70, "Text",
 CellTags->"OPESum"],
Cell[1488881, 58612, 323, 14, 70, "Text",
 CellTags->"OPESum"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1489241, 58631, 54, 1, 34, "Subsubsection",
 CellTags->"OPESum"],
Cell[CellGroupData[{
Cell[1489320, 58636, 460, 15, 26, "Input",
 CellTags->"OPESum"],
Cell[1489783, 58653, 897, 32, 57, "Output",
 CellTags->"OPESum"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1490717, 58690, 115, 3, 26, "Input",
 CellTags->"OPESum"],
Cell[1490835, 58695, 1648, 61, 51, "Output",
 CellTags->"OPESum"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1492520, 58761, 546, 18, 26, "Input",
 CellTags->"OPESum"],
Cell[1493069, 58781, 736, 28, 59, "Output",
 CellTags->"OPESum"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1493842, 58814, 115, 3, 26, "Input",
 CellTags->"OPESum"],
Cell[1493960, 58819, 1781, 67, 95, "Output",
 CellTags->"OPESum"]
}, Open  ]],
Cell[1495756, 58889, 149, 5, 26, "Input",
 CellTags->"OPESum"]
}, Open  ]],
Cell[1495920, 58897, 65, 1, 70, "ObjectName",
 CellTags->"OPESumSimplify"],
Cell[CellGroupData[{
Cell[1496010, 58902, 65, 1, 70, "Subsubsection",
 CellTags->"OPESumSimplify"],
Cell[1496078, 58905, 92, 1, 70, "Text",
 CellTags->"OPESumSimplify"],
Cell[1496173, 58908, 307, 14, 70, "Text",
 CellTags->"OPESumSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1496517, 58927, 62, 1, 34, "Subsubsection",
 CellTags->"OPESumSimplify"],
Cell[CellGroupData[{
Cell[1496604, 58932, 143, 4, 26, "Input",
 CellTags->"OPESumSimplify"],
Cell[1496750, 58938, 1096, 38, 55, "Output",
 CellTags->"OPESumSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1497883, 58981, 89, 2, 26, "Input",
 CellTags->"OPESumSimplify"],
Cell[1497975, 58985, 1194, 40, 55, "Output",
 CellTags->"OPESumSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1499206, 59030, 273, 8, 26, "Input",
 CellTags->"OPESumSimplify"],
Cell[1499482, 59040, 325, 12, 55, "Output",
 CellTags->"OPESumSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1499844, 59057, 439, 14, 26, "Input",
 CellTags->"OPESumSimplify"],
Cell[1500286, 59073, 595, 23, 55, "Output",
 CellTags->"OPESumSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1500918, 59101, 116, 3, 26, "Input",
 CellTags->"OPESumSimplify"],
Cell[1501037, 59106, 446, 14, 41, "Output",
 CellTags->"OPESumSimplify"]
}, Open  ]]
}, Open  ]],
Cell[1501510, 59124, 54, 1, 41, "ObjectName",
 CellTags->"OPE1Loop"],
Cell[CellGroupData[{
Cell[1501589, 59129, 59, 1, 36, "Subsubsection",
 CellTags->"OPE1Loop"],
Cell[1501651, 59132, 119, 3, 25, "Text",
 CellTags->"OPE1Loop"],
Cell[1501773, 59137, 173, 9, 25, "Text",
 CellTags->"OPE1Loop"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1501983, 59151, 56, 1, 34, "Subsubsection",
 CellTags->"OPE1Loop"],
Cell[1502042, 59154, 75, 2, 26, "Input",
 CellTags->"OPE1Loop"]
}, Open  ]],
Cell[1502132, 59159, 69, 1, 41, "ObjectName",
 CellTags->"OPE2TID"],
Cell[CellGroupData[{
Cell[1502226, 59164, 58, 1, 36, "Subsubsection",
 CellTags->"OPE2TID"],
Cell[1502287, 59167, 184, 4, 24, "Text",
 CellTags->"OPE2TID"],
Cell[CellGroupData[{
Cell[1502496, 59175, 114, 3, 26, "Input",
 CellTags->"OPE2TID"],
Cell[1502613, 59180, 387, 11, 26, "Output",
 CellTags->"OPE2TID"]
}, Open  ]],
Cell[1503015, 59194, 172, 9, 25, "Text",
 CellTags->"OPE2TID"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1503224, 59208, 55, 1, 34, "Subsubsection",
 CellTags->"OPE2TID"],
Cell[1503282, 59211, 74, 2, 26, "Input",
 CellTags->"OPE2TID"]
}, Open  ]],
Cell[1503371, 59216, 63, 1, 41, "ObjectName",
 CellTags->"OptionsSelect"],
Cell[CellGroupData[{
Cell[1503459, 59221, 64, 1, 36, "Subsubsection",
 CellTags->"OptionsSelect"],
Cell[1503526, 59224, 219, 5, 41, "Text",
 CellTags->"OptionsSelect"]
}, Open  ]],
Cell[1503760, 59232, 45, 1, 39, "ObjectName",
 CellTags->"Pair"],
Cell[CellGroupData[{
Cell[1503830, 59237, 55, 1, 34, "Subsubsection",
 CellTags->"Pair"],
Cell[1503888, 59240, 499, 11, 60, "Text",
 CellTags->"Pair"],
Cell[1504390, 59253, 989, 49, 24, "Text",
 CellTags->"Pair"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1505416, 59307, 52, 1, 34, "Subsubsection",
 CellTags->"Pair"],
Cell[1505471, 59310, 84, 1, 24, "Text",
 CellTags->"Pair"],
Cell[CellGroupData[{
Cell[1505580, 59315, 218, 6, 26, "Input",
 CellTags->"Pair"],
Cell[1505801, 59323, 201, 6, 26, "Output",
 CellTags->"Pair"]
}, Open  ]],
Cell[1506017, 59332, 73, 1, 24, "Text",
 CellTags->"Pair"],
Cell[CellGroupData[{
Cell[1506115, 59337, 268, 8, 26, "Input",
 CellTags->"Pair"],
Cell[1506386, 59347, 199, 6, 26, "Output",
 CellTags->"Pair"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1506622, 59358, 208, 6, 26, "Input",
 CellTags->"Pair"],
Cell[1506833, 59366, 278, 11, 28, "Output",
 CellTags->"Pair"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1507148, 59382, 197, 6, 26, "Input",
 CellTags->"Pair"],
Cell[1507348, 59390, 245, 9, 28, "Output",
 CellTags->"Pair"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1507630, 59404, 197, 6, 26, "Input",
 CellTags->"Pair"],
Cell[1507830, 59412, 201, 7, 28, "Output",
 CellTags->"Pair"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1508068, 59424, 222, 7, 26, "Input",
 CellTags->"Pair"],
Cell[1508293, 59433, 280, 10, 28, "Output",
 CellTags->"Pair"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1508610, 59448, 223, 7, 26, "Input",
 CellTags->"Pair"],
Cell[1508836, 59457, 265, 10, 31, "Output",
 CellTags->"Pair"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1509138, 59472, 223, 7, 26, "Input",
 CellTags->"Pair"],
Cell[1509364, 59481, 267, 10, 31, "Output",
 CellTags->"Pair"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1509668, 59496, 308, 10, 26, "Input",
 CellTags->"Pair"],
Cell[1509979, 59508, 145, 4, 26, "Output",
 CellTags->"Pair"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1510161, 59517, 272, 8, 26, "Input",
 CellTags->"Pair"],
Cell[1510436, 59527, 334, 13, 28, "Output",
 CellTags->"Pair"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1510807, 59545, 383, 12, 26, "Input",
 CellTags->"Pair"],
Cell[1511193, 59559, 142, 4, 26, "Output",
 CellTags->"Pair"]
}, Open  ]]
}, Open  ]],
Cell[1511362, 59567, 59, 1, 39, "ObjectName",
 CellTags->"PairCollect"],
Cell[CellGroupData[{
Cell[1511446, 59572, 62, 1, 34, "Subsubsection",
 CellTags->"PairCollect"],
Cell[1511511, 59575, 153, 4, 24, "Text",
 CellTags->"PairCollect"],
Cell[1511667, 59581, 286, 14, 24, "Text",
 CellTags->"PairCollect"]
}, Open  ]],
Cell[1511968, 59598, 61, 1, 39, "ObjectName",
 CellTags->"PairContract"],
Cell[CellGroupData[{
Cell[1512054, 59603, 63, 1, 34, "Subsubsection",
 CellTags->"PairContract"],
Cell[1512120, 59606, 118, 3, 24, "Text",
 CellTags->"PairContract"],
Cell[1512241, 59611, 275, 13, 24, "Text",
 CellTags->"PairContract"]
}, Open  ]],
Cell[1512531, 59627, 63, 1, 39, "ObjectName",
 CellTags->"PairContract2"],
Cell[CellGroupData[{
Cell[1512619, 59632, 64, 1, 34, "Subsubsection",
 CellTags->"PairContract2"],
Cell[1512686, 59635, 142, 4, 24, "Text",
 CellTags->"PairContract2"],
Cell[1512831, 59641, 398, 18, 24, "Text",
 CellTags->"PairContract2"]
}, Open  ]],
Cell[1513244, 59662, 63, 1, 39, "ObjectName",
 CellTags->"PairContract3"],
Cell[CellGroupData[{
Cell[1513332, 59667, 64, 1, 34, "Subsubsection",
 CellTags->"PairContract3"],
Cell[1513399, 59670, 142, 4, 24, "Text",
 CellTags->"PairContract3"],
Cell[1513544, 59676, 398, 18, 24, "Text",
 CellTags->"PairContract3"]
}, Open  ]],
Cell[1513957, 59697, 53, 1, 39, "ObjectName",
 CellTags->"PartialD"],
Cell[CellGroupData[{
Cell[1514035, 59702, 59, 1, 34, "Subsubsection",
 CellTags->"PartialD"],
Cell[1514097, 59705, 864, 35, 47, "Text",
 CellTags->"PartialD"],
Cell[1514964, 59742, 581, 24, 24, "Text",
 CellTags->"PartialD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1515582, 59771, 65, 1, 33, "Subsubsection",
 CellTags->"PartialD"],
Cell[CellGroupData[{
Cell[1515672, 59776, 233, 7, 26, "Input",
 CellTags->"PartialD"],
Cell[1515908, 59785, 343, 12, 33, "Output",
 CellTags->"PartialD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1516288, 59802, 116, 3, 26, "Input",
 CellTags->"PartialD"],
Cell[1516407, 59807, 618, 15, 87, "Output",
 CellTags->"PartialD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1517062, 59827, 114, 3, 26, "Input",
 CellTags->"PartialD"],
Cell[1517179, 59832, 607, 12, 115, "Output",
 CellTags->"PartialD"]
}, Open  ]]
}, Open  ]],
Cell[1517813, 59848, 71, 1, 39, "ObjectName",
 CellTags->"PartialDRelations"],
Cell[CellGroupData[{
Cell[1517909, 59853, 68, 1, 34, "Subsubsection",
 CellTags->"PartialDRelations"],
Cell[1517980, 59856, 191, 5, 24, "Text",
 CellTags->"PartialDRelations"],
Cell[1518174, 59863, 700, 29, 24, "Text",
 CellTags->"PartialDRelations"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1518911, 59897, 74, 1, 33, "Subsubsection",
 CellTags->"PartialDRelations"],
Cell[CellGroupData[{
Cell[1519010, 59902, 344, 10, 26, "Input",
 CellTags->"PartialDRelations"],
Cell[1519357, 59914, 392, 13, 33, "Output",
 CellTags->"PartialDRelations"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1519786, 59932, 256, 7, 26, "Input",
 CellTags->"PartialDRelations"],
Cell[1520045, 59941, 1295, 33, 153, "Output",
 CellTags->"PartialDRelations"]
}, Open  ]]
}, Open  ]],
Cell[1521367, 59978, 71, 1, 39, "ObjectName",
 CellTags->"PartialFourVector"],
Cell[CellGroupData[{
Cell[1521463, 59983, 68, 1, 34, "Subsubsection",
 CellTags->"PartialFourVector"],
Cell[1521534, 59986, 277, 6, 41, "Text",
 CellTags->"PartialFourVector"],
Cell[1521814, 59994, 402, 19, 24, "Text",
 CellTags->"PartialFourVector"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1522253, 60018, 74, 1, 33, "Subsubsection",
 CellTags->"PartialFourVector"],
Cell[CellGroupData[{
Cell[1522352, 60023, 382, 11, 33, "Input",
 CellTags->"PartialFourVector"],
Cell[1522737, 60036, 441, 16, 30, "Output",
 CellTags->"PartialFourVector"]
}, Open  ]]
}, Open  ]],
Cell[1523205, 60056, 69, 1, 39, "ObjectName",
 CellTags->"PartialIntegrate"],
Cell[CellGroupData[{
Cell[1523299, 60061, 67, 1, 34, "Subsubsection",
 CellTags->"PartialIntegrate"],
Cell[1523369, 60064, 283, 6, 41, "Text",
 CellTags->"PartialIntegrate"],
Cell[CellGroupData[{
Cell[1523677, 60074, 132, 3, 26, "Input",
 CellTags->"PartialIntegrate"],
Cell[1523812, 60079, 227, 7, 26, "Output",
 CellTags->"PartialIntegrate"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[1524088, 60092, 73, 1, 33, "Subsubsection",
 CellTags->"PartialIntegrate"],
Cell[CellGroupData[{
Cell[1524186, 60097, 326, 10, 26, "Input",
 CellTags->"PartialIntegrate"],
Cell[1524515, 60109, 1281, 42, 45, "Output",
 CellTags->"PartialIntegrate"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1525833, 60156, 84, 2, 26, "Input"],
Cell[1525920, 60160, 262, 6, 76, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1526219, 60171, 275, 9, 26, "Input",
 CellTags->"PartialIntegrate"],
Cell[1526497, 60182, 425, 15, 44, "Output",
 CellTags->"PartialIntegrate"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1526959, 60202, 293, 10, 52, "Input",
 CellTags->"PartialIntegrate"],
Cell[1527255, 60214, 420, 15, 48, "Output",
 CellTags->"PartialIntegrate"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1527712, 60234, 443, 14, 26, "Input",
 CellTags->"PartialIntegrate"],
Cell[1528158, 60250, 288, 9, 47, "Output",
 CellTags->"PartialIntegrate"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1528483, 60264, 320, 11, 26, "Input",
 CellTags->"PartialIntegrate"],
Cell[1528806, 60277, 290, 9, 47, "Output",
 CellTags->"PartialIntegrate"]
}, Open  ]],
Cell[1529111, 60289, 138, 4, 26, "Input",
 CellTags->"PartialIntegrate"]
}, Open  ]],
Cell[1529264, 60296, 57, 1, 39, "ObjectName",
 CellTags->"PartitHead"],
Cell[CellGroupData[{
Cell[1529346, 60301, 61, 1, 34, "Subsubsection",
 CellTags->"PartitHead"],
Cell[1529410, 60304, 167, 4, 24, "Text",
 CellTags->"PartitHead"]
}, Open  ]],
Cell[1529592, 60311, 57, 1, 39, "ObjectName",
 CellTags->"PauliSigma"],
Cell[CellGroupData[{
Cell[1529674, 60316, 61, 1, 34, "Subsubsection",
 CellTags->"PauliSigma"],
Cell[1529738, 60319, 252, 5, 41, "Text",
 CellTags->"PauliSigma"],
Cell[CellGroupData[{
Cell[1530015, 60328, 141, 4, 26, "Input",
 CellTags->"PauliSigma"],
Cell[1530159, 60334, 3637, 101, 116, "Output",
 CellTags->"PauliSigma"]
}, Open  ]]
}, Open  ]],
Cell[1533823, 60439, 45, 1, 39, "ObjectName",
 CellTags->"PaVe"],
Cell[CellGroupData[{
Cell[1533893, 60444, 55, 1, 34, "Subsubsection",
 CellTags->"PaVe"],
Cell[1533951, 60447, 428, 7, 58, "Text",
 CellTags->"PaVe"],
Cell[1534382, 60456, 266, 8, 24, "Text",
 CellTags->"PaVe"],
Cell[1534651, 60466, 181, 9, 24, "Text",
 CellTags->"PaVe"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1534869, 60480, 52, 1, 34, "Subsubsection",
 CellTags->"PaVe"],
Cell[1534924, 60483, 78, 1, 24, "Text",
 CellTags->"PaVe"],
Cell[CellGroupData[{
Cell[1535027, 60488, 303, 10, 26, "Input",
 CellTags->"PaVe"],
Cell[1535333, 60500, 348, 10, 26, "Output",
 CellTags->"PaVe"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1535718, 60515, 271, 9, 26, "Input",
 CellTags->"PaVe"],
Cell[1535992, 60526, 353, 12, 29, "Output",
 CellTags->"PaVe"]
}, Open  ]]
}, Open  ]],
Cell[1536372, 60542, 63, 1, 39, "ObjectName",
 CellTags->"PaVeOrderList"],
Cell[CellGroupData[{
Cell[1536460, 60547, 64, 1, 34, "Subsubsection",
 CellTags->"PaVeOrderList"],
Cell[1536527, 60550, 176, 4, 24, "Text",
 CellTags->"PaVeOrderList"],
Cell[1536706, 60556, 487, 23, 24, "Text",
 CellTags->"PaVeOrderList"]
}, Open  ]],
Cell[1537208, 60582, 55, 1, 39, "ObjectName",
 CellTags->"PaVeOrder"],
Cell[CellGroupData[{
Cell[1537288, 60587, 60, 1, 34, "Subsubsection",
 CellTags->"PaVeOrder"],
Cell[1537351, 60590, 564, 16, 58, "Text",
 CellTags->"PaVeOrder"],
Cell[1537918, 60608, 186, 9, 24, "Text",
 CellTags->"PaVeOrder"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1538141, 60622, 57, 1, 34, "Subsubsection",
 CellTags->"PaVeOrder"],
Cell[CellGroupData[{
Cell[1538223, 60627, 413, 12, 26, "Input",
 CellTags->"PaVeOrder"],
Cell[1538639, 60641, 426, 13, 26, "Output",
 CellTags->"PaVeOrder"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1539102, 60659, 413, 12, 26, "Input",
 CellTags->"PaVeOrder"],
Cell[1539518, 60673, 426, 13, 26, "Output",
 CellTags->"PaVeOrder"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1539981, 60691, 706, 20, 65, "Input",
 CellTags->"PaVeOrder"],
Cell[1540690, 60713, 665, 19, 26, "Output",
 CellTags->"PaVeOrder"]
}, Open  ]]
}, Open  ]],
Cell[1541382, 60736, 57, 1, 39, "ObjectName",
 CellTags->"PaVeReduce"],
Cell[CellGroupData[{
Cell[1541464, 60741, 61, 1, 34, "Subsubsection",
 CellTags->"PaVeReduce"],
Cell[1541528, 60744, 167, 4, 24, "Text",
 CellTags->"PaVeReduce"],
Cell[CellGroupData[{
Cell[1541720, 60752, 120, 3, 26, "Input",
 CellTags->"PaVeReduce"],
Cell[1541843, 60757, 487, 14, 26, "Output",
 CellTags->"PaVeReduce"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1542367, 60776, 106, 3, 26, "Input",
 CellTags->"PaVeReduce"],
Cell[1542476, 60781, 374, 6, 79, "Print",
 CellTags->{"Info3500973951-4982097", "PaVeReduce"}]
}, Open  ]],
Cell[1542865, 60790, 279, 14, 24, "Text",
 CellTags->"PaVeReduce"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1543181, 60809, 58, 1, 34, "Subsubsection",
 CellTags->"PaVeReduce"],
Cell[CellGroupData[{
Cell[1543264, 60814, 520, 16, 26, "Input",
 CellTags->"PaVeReduce"],
Cell[1543787, 60832, 3060, 105, 109, "Output",
 CellTags->"PaVeReduce"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1546884, 60942, 106, 3, 26, "Input",
 CellTags->"PaVeReduce"],
Cell[1546993, 60947, 178, 5, 26, "Output",
 CellTags->"PaVeReduce"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1547208, 60957, 107, 3, 26, "Input",
 CellTags->"PaVeReduce"],
Cell[1547318, 60962, 178, 5, 26, "Output",
 CellTags->"PaVeReduce"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1547533, 60972, 491, 15, 46, "Input",
 CellTags->"PaVeReduce"],
Cell[1548027, 60989, 512, 16, 44, "Output",
 CellTags->"PaVeReduce"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1548576, 61010, 398, 12, 46, "Input",
 CellTags->"PaVeReduce"],
Cell[1548977, 61024, 833, 24, 72, "Output",
 CellTags->"PaVeReduce"]
}, Open  ]],
Cell[1549825, 61051, 209, 7, 26, "Input",
 CellTags->"PaVeReduce"],
Cell[1550037, 61060, 162, 5, 26, "Input",
 CellTags->"PaVeReduce"],
Cell[CellGroupData[{
Cell[1550224, 61069, 633, 17, 46, "Input",
 CellTags->"PaVeReduce"],
Cell[1550860, 61088, 8441, 261, 413, "Output",
 CellTags->"PaVeReduce"]
}, Open  ]],
Cell[1559316, 61352, 267, 8, 26, "Input",
 CellTags->"PaVeReduce"],
Cell[CellGroupData[{
Cell[1559608, 61364, 375, 11, 26, "Input",
 CellTags->"PaVeReduce"],
Cell[1559986, 61377, 3020, 58, 663, "Output",
 CellTags->"PaVeReduce"]
}, Open  ]],
Cell[1563021, 61438, 253, 7, 26, "Input",
 CellTags->"PaVeReduce"]
}, Open  ]],
Cell[1563289, 61448, 41, 1, 39, "ObjectName",
 CellTags->"PD"],
Cell[CellGroupData[{
Cell[1563355, 61453, 53, 1, 34, "Subsubsection",
 CellTags->"PD"],
Cell[1563411, 61456, 81, 1, 24, "Text",
 CellTags->"PD"],
Cell[1563495, 61459, 310, 14, 24, "Text",
 CellTags->"PD"]
}, Open  ]],
Cell[1563820, 61476, 69, 1, 39, "ObjectName",
 CellTags->"PlusDistribution"],
Cell[CellGroupData[{
Cell[1563914, 61481, 67, 1, 34, "Subsubsection",
 CellTags->"PlusDistribution"],
Cell[1563984, 61484, 147, 4, 24, "Text",
 CellTags->"PlusDistribution"],
Cell[1564134, 61490, 193, 9, 24, "Text",
 CellTags->"PlusDistribution"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1564364, 61504, 64, 1, 34, "Subsubsection",
 CellTags->"PlusDistribution"],
Cell[CellGroupData[{
Cell[1564453, 61509, 198, 6, 26, "Input",
 CellTags->"PlusDistribution"],
Cell[1564654, 61517, 250, 8, 46, "Output",
 CellTags->"PlusDistribution"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1564941, 61530, 254, 8, 26, "Input",
 CellTags->"PlusDistribution"],
Cell[1565198, 61540, 310, 10, 46, "Output",
 CellTags->"PlusDistribution"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1565545, 61555, 329, 10, 26, "Input",
 CellTags->"PlusDistribution"],
Cell[1565877, 61567, 157, 4, 26, "Output",
 CellTags->"PlusDistribution"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1566071, 61576, 389, 12, 26, "Input",
 CellTags->"PlusDistribution"],
Cell[1566463, 61590, 155, 4, 26, "Output",
 CellTags->"PlusDistribution"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1566655, 61599, 417, 13, 26, "Input",
 CellTags->"PlusDistribution"],
Cell[1567075, 61614, 155, 4, 26, "Output",
 CellTags->"PlusDistribution"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1567267, 61623, 307, 10, 26, "Input",
 CellTags->"PlusDistribution"],
Cell[1567577, 61635, 417, 14, 46, "Output",
 CellTags->"PlusDistribution"]
}, Open  ]]
}, Open  ]],
Cell[1568021, 61653, 61, 1, 39, "ObjectName",
 CellTags->"Polarization"],
Cell[CellGroupData[{
Cell[1568107, 61658, 63, 1, 34, "Subsubsection",
 CellTags->"Polarization"],
Cell[1568173, 61661, 769, 12, 92, "Text",
 CellTags->"Polarization"],
Cell[1568945, 61675, 484, 19, 24, "Text",
 CellTags->"Polarization"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1569466, 61699, 60, 1, 34, "Subsubsection",
 CellTags->"Polarization"],
Cell[CellGroupData[{
Cell[1569551, 61704, 118, 3, 26, "Input",
 CellTags->"Polarization"],
Cell[1569672, 61709, 195, 5, 26, "Output",
 CellTags->"Polarization"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1569904, 61719, 153, 4, 26, "Input",
 CellTags->"Polarization"],
Cell[1570060, 61725, 210, 5, 38, "Output",
 CellTags->"Polarization"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1570307, 61735, 160, 5, 26, "Input",
 CellTags->"Polarization"],
Cell[1570470, 61742, 279, 9, 26, "Output",
 CellTags->"Polarization"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1570786, 61756, 196, 6, 26, "Input",
 CellTags->"Polarization"],
Cell[1570985, 61764, 229, 6, 38, "Output",
 CellTags->"Polarization"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1571251, 61775, 155, 4, 26, "Input",
 CellTags->"Polarization"],
Cell[1571409, 61781, 314, 10, 26, "Output",
 CellTags->"Polarization"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1571760, 61796, 194, 6, 26, "Input",
 CellTags->"Polarization"],
Cell[1571957, 61804, 250, 6, 38, "Output",
 CellTags->"Polarization"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1572244, 61815, 302, 8, 26, "Input",
 CellTags->"Polarization"],
Cell[1572549, 61825, 153, 4, 26, "Output",
 CellTags->"Polarization"]
}, Open  ]]
}, Open  ]],
Cell[1572729, 61833, 67, 1, 39, "ObjectName",
 CellTags->"PolarizationSum"],
Cell[CellGroupData[{
Cell[1572821, 61838, 66, 1, 34, "Subsubsection",
 CellTags->"PolarizationSum"],
Cell[1572890, 61841, 277, 5, 41, "Text",
 CellTags->"PolarizationSum"],
Cell[CellGroupData[{
Cell[1573192, 61850, 130, 3, 26, "Input",
 CellTags->"PolarizationSum"],
Cell[1573325, 61855, 217, 6, 26, "Output",
 CellTags->"PolarizationSum"]
}, Open  ]],
Cell[1573557, 61864, 198, 9, 24, "Text",
 CellTags->"PolarizationSum"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1573792, 61878, 63, 1, 34, "Subsubsection",
 CellTags->"PolarizationSum"],
Cell[CellGroupData[{
Cell[1573880, 61883, 155, 4, 26, "Input",
 CellTags->"PolarizationSum"],
Cell[1574038, 61889, 227, 7, 26, "Output",
 CellTags->"PolarizationSum"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1574302, 61901, 165, 4, 26, "Input",
 CellTags->"PolarizationSum"],
Cell[1574470, 61907, 649, 26, 47, "Output",
 CellTags->"PolarizationSum"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1575156, 61938, 175, 4, 26, "Input",
 CellTags->"PolarizationSum"],
Cell[1575334, 61944, 1728, 72, 49, "Output",
 CellTags->"PolarizationSum"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1577099, 62021, 247, 7, 26, "Input",
 CellTags->"PolarizationSum"],
Cell[1577349, 62030, 4088, 163, 51, "Output",
 CellTags->"PolarizationSum"]
}, Open  ]]
}, Open  ]],
Cell[1581464, 62197, 81, 1, 41, "ObjectName",
 CellTags->"PolarizationUncontract"],
Cell[CellGroupData[{
Cell[1581570, 62202, 73, 1, 36, "Subsubsection",
 CellTags->"PolarizationUncontract"],
Cell[1581646, 62205, 152, 4, 25, "Text",
 CellTags->"PolarizationUncontract"],
Cell[1581801, 62211, 321, 14, 25, "Text",
 CellTags->"PolarizationUncontract"]
}, Open  ]],
Cell[1582137, 62228, 73, 1, 41, "ObjectName",
 CellTags->"PolarizationVector"],
Cell[CellGroupData[{
Cell[1582235, 62233, 69, 1, 36, "Subsubsection",
 CellTags->"PolarizationVector"],
Cell[1582307, 62236, 103, 1, 25, "Text",
 CellTags->"PolarizationVector"],
Cell[1582413, 62239, 409, 18, 25, "Text",
 CellTags->"PolarizationVector"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1582859, 62262, 66, 1, 34, "Subsubsection",
 CellTags->"PolarizationVector"],
Cell[CellGroupData[{
Cell[1582950, 62267, 157, 4, 26, "Input",
 CellTags->"PolarizationVector"],
Cell[1583110, 62273, 300, 10, 29, "Output",
 CellTags->"PolarizationVector"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1583447, 62288, 194, 5, 26, "Input",
 CellTags->"PolarizationVector"],
Cell[1583644, 62295, 310, 10, 30, "Output",
 CellTags->"PolarizationVector"]
}, Open  ]],
Cell[1583969, 62308, 241, 9, 27, "Text",
 CellTags->"PolarizationVector"],
Cell[CellGroupData[{
Cell[1584235, 62321, 190, 5, 26, "Input",
 CellTags->"PolarizationVector"],
Cell[1584428, 62328, 351, 9, 38, "Output",
 CellTags->"PolarizationVector"]
}, Open  ]],
Cell[1584794, 62340, 440, 18, 27, "Text",
 CellTags->"PolarizationVector"],
Cell[CellGroupData[{
Cell[1585259, 62362, 284, 8, 26, "Input",
 CellTags->"PolarizationVector"],
Cell[1585546, 62372, 356, 12, 29, "Output",
 CellTags->"PolarizationVector"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1585939, 62389, 121, 3, 26, "Input",
 CellTags->"PolarizationVector"],
Cell[1586063, 62394, 157, 4, 26, "Output",
 CellTags->"PolarizationVector"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1586257, 62403, 310, 9, 26, "Input",
 CellTags->"PolarizationVector"],
Cell[1586570, 62414, 420, 15, 29, "Output",
 CellTags->"PolarizationVector"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1587027, 62434, 121, 3, 26, "Input",
 CellTags->"PolarizationVector"],
Cell[1587151, 62439, 378, 13, 28, "Output",
 CellTags->"PolarizationVector"]
}, Open  ]],
Cell[1587544, 62455, 1511, 51, 43, "Text",
 CellTags->"PolarizationVector"],
Cell[CellGroupData[{
Cell[1589080, 62510, 310, 9, 26, "Input",
 CellTags->"PolarizationVector"],
Cell[1589393, 62521, 418, 15, 29, "Output",
 CellTags->"PolarizationVector"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1589848, 62541, 121, 3, 26, "Input",
 CellTags->"PolarizationVector"],
Cell[1589972, 62546, 380, 13, 28, "Output",
 CellTags->"PolarizationVector"]
}, Open  ]],
Cell[1590367, 62562, 153, 4, 26, "Input",
 CellTags->"PolarizationVector"]
}, Open  ]],
Cell[1590535, 62569, 67, 1, 41, "ObjectName",
 CellTags->"PositiveInteger"],
Cell[CellGroupData[{
Cell[1590627, 62574, 66, 1, 36, "Subsubsection",
 CellTags->"PositiveInteger"],
Cell[1590696, 62577, 136, 4, 25, "Text",
 CellTags->"PositiveInteger"],
Cell[1590835, 62583, 186, 9, 25, "Text",
 CellTags->"PositiveInteger"]
}, Open  ]],
Cell[1591036, 62595, 65, 1, 41, "ObjectName",
 CellTags->"PositiveNumber"],
Cell[CellGroupData[{
Cell[1591126, 62600, 65, 1, 36, "Subsubsection",
 CellTags->"PositiveNumber"],
Cell[1591194, 62603, 146, 4, 25, "Text",
 CellTags->"PositiveNumber"],
Cell[1591343, 62609, 185, 9, 25, "Text",
 CellTags->"PositiveNumber"]
}, Open  ]],
Cell[1591543, 62621, 59, 1, 41, "ObjectName",
 CellTags->"PowerFactor"],
Cell[CellGroupData[{
Cell[1591627, 62626, 62, 1, 36, "Subsubsection",
 CellTags->"PowerFactor"],
Cell[1591692, 62629, 375, 16, 25, "Text",
 CellTags->"PowerFactor"],
Cell[1592070, 62647, 197, 9, 25, "Text",
 CellTags->"PowerFactor"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1592304, 62661, 59, 1, 34, "Subsubsection",
 CellTags->"PowerFactor"],
Cell[CellGroupData[{
Cell[1592388, 62666, 144, 5, 26, "Input",
 CellTags->"PowerFactor"],
Cell[1592535, 62673, 223, 7, 26, "Output",
 CellTags->"PowerFactor"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1592795, 62685, 116, 3, 26, "Input",
 CellTags->"PowerFactor"],
Cell[1592914, 62690, 225, 7, 26, "Output",
 CellTags->"PowerFactor"]
}, Open  ]]
}, Open  ]],
Cell[1593166, 62701, 63, 1, 41, "ObjectName",
 CellTags->"PowerSimplify"],
Cell[CellGroupData[{
Cell[1593254, 62706, 64, 1, 36, "Subsubsection",
 CellTags->"PowerSimplify"],
Cell[1593321, 62709, 2559, 112, 63, "Text",
 CellTags->"PowerSimplify"],
Cell[1595883, 62823, 282, 14, 25, "Text",
 CellTags->"PowerSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1596202, 62842, 61, 1, 34, "Subsubsection",
 CellTags->"PowerSimplify"],
Cell[CellGroupData[{
Cell[1596288, 62847, 234, 8, 26, "Input",
 CellTags->"PowerSimplify"],
Cell[1596525, 62857, 154, 4, 26, "Output",
 CellTags->"PowerSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1596716, 62866, 239, 8, 26, "Input",
 CellTags->"PowerSimplify"],
Cell[1596958, 62876, 220, 7, 26, "Output",
 CellTags->"PowerSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1597215, 62888, 239, 8, 26, "Input",
 CellTags->"PowerSimplify"],
Cell[1597457, 62898, 222, 7, 26, "Output",
 CellTags->"PowerSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1597716, 62910, 190, 6, 26, "Input",
 CellTags->"PowerSimplify"],
Cell[1597909, 62918, 222, 7, 26, "Output",
 CellTags->"PowerSimplify"]
}, Open  ]]
}, Open  ]],
Cell[1598158, 62929, 49, 1, 41, "ObjectName",
 CellTags->"Power2"],
Cell[CellGroupData[{
Cell[1598232, 62934, 57, 1, 36, "Subsubsection",
 CellTags->"Power2"],
Cell[1598292, 62937, 213, 5, 43, "Text",
 CellTags->"Power2"],
Cell[1598508, 62944, 186, 9, 25, "Text",
 CellTags->"Power2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1598731, 62958, 54, 1, 34, "Subsubsection",
 CellTags->"Power2"],
Cell[CellGroupData[{
Cell[1598810, 62963, 147, 5, 26, "Input",
 CellTags->"Power2"],
Cell[1598960, 62970, 213, 7, 28, "Output",
 CellTags->"Power2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1599210, 62982, 148, 5, 26, "Input",
 CellTags->"Power2"],
Cell[1599361, 62989, 298, 11, 28, "Output",
 CellTags->"Power2"]
}, Open  ]]
}, Open  ]],
Cell[1599686, 63004, 67, 1, 41, "ObjectName",
 CellTags->"PostFortranFile"],
Cell[CellGroupData[{
Cell[1599778, 63009, 66, 1, 36, "Subsubsection",
 CellTags->"PostFortranFile"],
Cell[1599847, 63012, 229, 5, 43, "Text",
 CellTags->"PostFortranFile"],
Cell[1600079, 63019, 308, 14, 25, "Text",
 CellTags->"PostFortranFile"]
}, Open  ]],
Cell[1600402, 63036, 55, 1, 41, "ObjectName",
 CellTags->"Prefactor"],
Cell[CellGroupData[{
Cell[1600482, 63041, 60, 1, 36, "Subsubsection",
 CellTags->"Prefactor"],
Cell[1600545, 63044, 290, 6, 43, "Text",
 CellTags->"Prefactor"],
Cell[1600838, 63052, 293, 14, 25, "Text",
 CellTags->"Prefactor"]
}, Open  ]],
Cell[1601146, 63069, 65, 1, 41, "ObjectName",
 CellTags->"PreFortranFile"],
Cell[CellGroupData[{
Cell[1601236, 63074, 65, 1, 36, "Subsubsection",
 CellTags->"PreFortranFile"],
Cell[1601304, 63077, 232, 5, 43, "Text",
 CellTags->"PreFortranFile"],
Cell[1601539, 63084, 310, 14, 25, "Text",
 CellTags->"PreFortranFile"]
}, Open  ]],
Cell[1601864, 63101, 79, 1, 41, "ObjectName",
 CellTags->"PropagatorDenominator"],
Cell[CellGroupData[{
Cell[1601968, 63106, 72, 1, 36, "Subsubsection",
 CellTags->"PropagatorDenominator"],
Cell[1602043, 63109, 549, 16, 61, "Text",
 CellTags->"PropagatorDenominator"],
Cell[1602595, 63127, 493, 19, 25, "Text",
 CellTags->"PropagatorDenominator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1603125, 63151, 69, 1, 34, "Subsubsection",
 CellTags->"PropagatorDenominator"],
Cell[CellGroupData[{
Cell[1603219, 63156, 178, 5, 26, "Input",
 CellTags->"PropagatorDenominator"],
Cell[1603400, 63163, 257, 8, 50, "Output",
 CellTags->"PropagatorDenominator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1603694, 63176, 136, 3, 26, "Input",
 CellTags->"PropagatorDenominator"],
Cell[1603833, 63181, 206, 6, 50, "Output",
 CellTags->"PropagatorDenominator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1604076, 63192, 189, 5, 26, "Input",
 CellTags->"PropagatorDenominator"],
Cell[1604268, 63199, 257, 8, 50, "Output",
 CellTags->"PropagatorDenominator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1604562, 63212, 158, 4, 26, "Input",
 CellTags->"PropagatorDenominator"],
Cell[1604723, 63218, 277, 7, 38, "Output",
 CellTags->"PropagatorDenominator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1605037, 63230, 194, 5, 26, "Input",
 CellTags->"PropagatorDenominator"],
Cell[1605234, 63237, 277, 7, 38, "Output",
 CellTags->"PropagatorDenominator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1605548, 63249, 145, 3, 26, "Input",
 CellTags->"PropagatorDenominator"],
Cell[1605696, 63254, 297, 10, 50, "Output",
 CellTags->"PropagatorDenominator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1606030, 63269, 127, 3, 26, "Input",
 CellTags->"PropagatorDenominator"],
Cell[1606160, 63274, 367, 11, 58, "Output",
 CellTags->"PropagatorDenominator"]
}, Open  ]],
Cell[1606542, 63288, 121, 3, 26, "Input",
 CellTags->"PropagatorDenominator"]
}, Open  ]],
Cell[1606678, 63294, 95, 1, 39, "ObjectName",
 CellTags->"PropagatorDenominatorExplicit"],
Cell[CellGroupData[{
Cell[1606798, 63299, 80, 1, 34, "Subsubsection",
 CellTags->"PropagatorDenominatorExplicit"],
Cell[1606881, 63302, 243, 5, 41, "Text",
 CellTags->"PropagatorDenominatorExplicit"],
Cell[1607127, 63309, 379, 14, 24, "Text",
 CellTags->"PropagatorDenominatorExplicit"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1607543, 63328, 77, 1, 34, "Subsubsection",
 CellTags->"PropagatorDenominatorExplicit"],
Cell[CellGroupData[{
Cell[1607645, 63333, 308, 11, 26, "Input",
 CellTags->"PropagatorDenominatorExplicit"],
Cell[1607956, 63346, 399, 13, 51, "Output",
 CellTags->"PropagatorDenominatorExplicit"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1608392, 63364, 181, 5, 26, "Input",
 CellTags->"PropagatorDenominatorExplicit"],
Cell[1608576, 63371, 807, 30, 51, "Output",
 CellTags->"PropagatorDenominatorExplicit"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1609420, 63406, 131, 3, 26, "Input",
 CellTags->"PropagatorDenominatorExplicit"],
Cell[1609554, 63411, 589, 19, 61, "Output",
 CellTags->"PropagatorDenominatorExplicit"]
}, Open  ]]
}, Open  ]],
Cell[1610170, 63434, 43, 1, 39, "ObjectName",
 CellTags->"QGV"],
Cell[CellGroupData[{
Cell[1610238, 63439, 54, 1, 34, "Subsubsection",
 CellTags->"QGV"],
Cell[1610295, 63442, 72, 1, 24, "Text",
 CellTags->"QGV"],
Cell[1610370, 63445, 198, 9, 24, "Text",
 CellTags->"QGV"]
}, Open  ]],
Cell[1610583, 63457, 41, 1, 39, "ObjectName",
 CellTags->"QO"],
Cell[CellGroupData[{
Cell[1610649, 63462, 53, 1, 34, "Subsubsection",
 CellTags->"QO"],
Cell[1610705, 63465, 73, 1, 24, "Text",
 CellTags->"QO"],
Cell[1610781, 63468, 206, 9, 24, "Text",
 CellTags->"QO"]
}, Open  ]],
Cell[1611002, 63480, 41, 1, 39, "ObjectName",
 CellTags->"QP"],
Cell[CellGroupData[{
Cell[1611068, 63485, 53, 1, 34, "Subsubsection",
 CellTags->"QP"],
Cell[1611124, 63488, 169, 4, 24, "Text",
 CellTags->"QP"],
Cell[1611296, 63494, 194, 9, 24, "Text",
 CellTags->"QP"]
}, Open  ]],
Cell[1611505, 63506, 61, 1, 39, "ObjectName",
 CellTags->"QuantumField"],
Cell[CellGroupData[{
Cell[1611591, 63511, 63, 1, 34, "Subsubsection",
 CellTags->"QuantumField"],
Cell[1611657, 63514, 404, 9, 58, "Text",
 CellTags->"QuantumField"],
Cell[1612064, 63525, 421, 19, 24, "Text",
 CellTags->"QuantumField"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1612522, 63549, 60, 1, 34, "Subsubsection",
 CellTags->"QuantumField"],
Cell[1612585, 63552, 71, 1, 24, "Text",
 CellTags->"QuantumField"],
Cell[CellGroupData[{
Cell[1612681, 63557, 118, 3, 26, "Input",
 CellTags->"QuantumField"],
Cell[1612802, 63562, 151, 4, 27, "Output",
 CellTags->"QuantumField"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1612990, 63571, 131, 3, 26, "Input",
 CellTags->"QuantumField"],
Cell[1613124, 63576, 179, 5, 26, "Output",
 CellTags->"QuantumField"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1613340, 63586, 127, 3, 26, "Input",
 CellTags->"QuantumField"],
Cell[1613470, 63591, 156, 4, 26, "Output",
 CellTags->"QuantumField"]
}, Open  ]],
Cell[1613641, 63598, 80, 1, 24, "Text",
 CellTags->"QuantumField"],
Cell[CellGroupData[{
Cell[1613746, 63603, 169, 5, 26, "Input",
 CellTags->"QuantumField"],
Cell[1613918, 63610, 179, 5, 29, "Output",
 CellTags->"QuantumField"]
}, Open  ]],
Cell[1614112, 63618, 94, 1, 24, "Text",
 CellTags->"QuantumField"],
Cell[CellGroupData[{
Cell[1614231, 63623, 212, 6, 26, "Input",
 CellTags->"QuantumField"],
Cell[1614446, 63631, 361, 15, 30, "Output",
 CellTags->"QuantumField"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1614844, 63651, 114, 3, 26, "Input",
 CellTags->"QuantumField"],
Cell[1614961, 63656, 297, 7, 38, "Output",
 CellTags->"QuantumField"]
}, Open  ]],
Cell[1615273, 63666, 358, 13, 28, "Text",
 CellTags->"QuantumField"],
Cell[CellGroupData[{
Cell[1615656, 63683, 206, 6, 26, "Input",
 CellTags->"QuantumField"],
Cell[1615865, 63691, 301, 11, 28, "Output",
 CellTags->"QuantumField"]
}, Open  ]],
Cell[1616181, 63705, 185, 4, 24, "Text",
 CellTags->"QuantumField"],
Cell[CellGroupData[{
Cell[1616391, 63713, 244, 7, 26, "Input",
 CellTags->"QuantumField"],
Cell[1616638, 63722, 287, 7, 38, "Output",
 CellTags->"QuantumField"]
}, Open  ]],
Cell[1616940, 63732, 88, 1, 24, "Text",
 CellTags->"QuantumField"],
Cell[CellGroupData[{
Cell[1617053, 63737, 219, 6, 26, "Input",
 CellTags->"QuantumField"],
Cell[1617275, 63745, 590, 14, 87, "Output",
 CellTags->"QuantumField"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1617902, 63764, 184, 5, 26, "Input",
 CellTags->"QuantumField"],
Cell[1618089, 63771, 552, 13, 87, "Output",
 CellTags->"QuantumField"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1618678, 63789, 259, 7, 26, "Input",
 CellTags->"QuantumField"],
Cell[1618940, 63798, 622, 14, 87, "Output",
 CellTags->"QuantumField"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1619599, 63817, 291, 9, 26, "Input",
 CellTags->"QuantumField"],
Cell[1619893, 63828, 654, 16, 87, "Output",
 CellTags->"QuantumField"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1620584, 63849, 232, 6, 26, "Input",
 CellTags->"QuantumField"],
Cell[1620819, 63857, 156, 4, 26, "Output",
 CellTags->"QuantumField"]
}, Open  ]]
}, Open  ]],
Cell[1621002, 63865, 57, 1, 57, "ObjectName",
 CellTags->"QuarkField"],
Cell[CellGroupData[{
Cell[1621084, 63870, 61, 1, 36, "Subsubsection",
 CellTags->"QuarkField"],
Cell[1621148, 63873, 189, 4, 43, "Text",
 CellTags->"QuarkField"],
Cell[1621340, 63879, 321, 14, 25, "Text",
 CellTags->"QuarkField"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1621698, 63898, 58, 1, 34, "Subsubsection",
 CellTags->"QuarkField"],
Cell[CellGroupData[{
Cell[1621781, 63903, 87, 2, 26, "Input",
 CellTags->"QuarkField"],
Cell[1621871, 63907, 154, 4, 26, "Output",
 CellTags->"QuarkField"]
}, Open  ]]
}, Open  ]],
Cell[1622052, 63915, 69, 1, 41, "ObjectName",
 CellTags->"QuarkGluonVertex"],
Cell[CellGroupData[{
Cell[1622146, 63920, 67, 1, 36, "Subsubsection",
 CellTags->"QuarkGluonVertex"],
Cell[1622216, 63923, 260, 9, 24, "Text",
 CellTags->"QuarkGluonVertex"],
Cell[CellGroupData[{
Cell[1622501, 63936, 132, 3, 26, "Input",
 CellTags->"QuarkGluonVertex"],
Cell[1622636, 63941, 533, 14, 26, "Output",
 CellTags->"QuarkGluonVertex"]
}, Open  ]],
Cell[1623184, 63958, 196, 9, 25, "Text",
 CellTags->"QuarkGluonVertex"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1623417, 63972, 64, 1, 34, "Subsubsection",
 CellTags->"QuarkGluonVertex"],
Cell[CellGroupData[{
Cell[1623506, 63977, 153, 4, 26, "Input",
 CellTags->"QuarkGluonVertex"],
Cell[1623662, 63983, 191, 5, 28, "Output",
 CellTags->"QuarkGluonVertex"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1623890, 63993, 208, 5, 26, "Input",
 CellTags->"QuarkGluonVertex"],
Cell[1624101, 64000, 191, 5, 28, "Output",
 CellTags->"QuarkGluonVertex"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1624329, 64010, 208, 5, 26, "Input",
 CellTags->"QuarkGluonVertex"],
Cell[1624540, 64017, 191, 5, 28, "Output",
 CellTags->"QuarkGluonVertex"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1624768, 64027, 208, 5, 26, "Input",
 CellTags->"QuarkGluonVertex"],
Cell[1624979, 64034, 191, 5, 28, "Output",
 CellTags->"QuarkGluonVertex"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1625207, 64044, 315, 9, 26, "Input",
 CellTags->"QuarkGluonVertex"],
Cell[1625525, 64055, 408, 12, 28, "Output",
 CellTags->"QuarkGluonVertex"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1625970, 64072, 316, 9, 26, "Input",
 CellTags->"QuarkGluonVertex"],
Cell[1626289, 64083, 409, 12, 28, "Output",
 CellTags->"QuarkGluonVertex"]
}, Open  ]]
}, Open  ]],
Cell[1626725, 64099, 55, 1, 41, "ObjectName",
 CellTags->"QuarkMass"],
Cell[CellGroupData[{
Cell[1626805, 64104, 60, 1, 36, "Subsubsection",
 CellTags->"QuarkMass"],
Cell[1626868, 64107, 76, 1, 25, "Text",
 CellTags->"QuarkMass"],
Cell[1626947, 64110, 183, 9, 25, "Text",
 CellTags->"QuarkMass"]
}, Open  ]],
Cell[1627145, 64122, 67, 1, 41, "ObjectName",
 CellTags->"QuarkPropagator"],
Cell[CellGroupData[{
Cell[1627237, 64127, 66, 1, 36, "Subsubsection",
 CellTags->"QuarkPropagator"],
Cell[1627306, 64130, 174, 4, 24, "Text",
 CellTags->"QuarkPropagator"],
Cell[CellGroupData[{
Cell[1627505, 64138, 130, 3, 26, "Input",
 CellTags->"QuarkPropagator"],
Cell[1627638, 64143, 575, 15, 26, "Output",
 CellTags->"QuarkPropagator"]
}, Open  ]],
Cell[1628228, 64161, 341, 14, 25, "Text",
 CellTags->"QuarkPropagator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1628606, 64180, 63, 1, 34, "Subsubsection",
 CellTags->"QuarkPropagator"],
Cell[CellGroupData[{
Cell[1628694, 64185, 124, 3, 26, "Input",
 CellTags->"QuarkPropagator"],
Cell[1628821, 64190, 219, 7, 30, "Output",
 CellTags->"QuarkPropagator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1629077, 64202, 171, 5, 26, "Input",
 CellTags->"QuarkPropagator"],
Cell[1629251, 64209, 219, 7, 30, "Output",
 CellTags->"QuarkPropagator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1629507, 64221, 157, 4, 26, "Input",
 CellTags->"QuarkPropagator"],
Cell[1629667, 64227, 219, 7, 30, "Output",
 CellTags->"QuarkPropagator"]
}, Open  ]]
}, Open  ]],
Cell[1629913, 64238, 59, 1, 41, "ObjectName",
 CellTags->"ReduceGamma"],
Cell[CellGroupData[{
Cell[1629997, 64243, 62, 1, 36, "Subsubsection",
 CellTags->"ReduceGamma"],
Cell[1630062, 64246, 194, 4, 43, "Text",
 CellTags->"ReduceGamma"],
Cell[1630259, 64252, 179, 9, 25, "Text",
 CellTags->"ReduceGamma"]
}, Open  ]],
Cell[1630453, 64264, 67, 1, 41, "ObjectName",
 CellTags->"ReduceToScalars"],
Cell[CellGroupData[{
Cell[1630545, 64269, 66, 1, 36, "Subsubsection",
 CellTags->"ReduceToScalars"],
Cell[1630614, 64272, 210, 5, 43, "Text",
 CellTags->"ReduceToScalars"],
Cell[1630827, 64279, 299, 14, 25, "Text",
 CellTags->"ReduceToScalars"]
}, Open  ]],
Cell[1631141, 64296, 49, 1, 41, "ObjectName",
 CellTags->"Rename"],
Cell[CellGroupData[{
Cell[1631215, 64301, 57, 1, 36, "Subsubsection",
 CellTags->"Rename"],
Cell[1631275, 64304, 144, 4, 25, "Text",
 CellTags->"Rename"],
Cell[1631422, 64310, 177, 9, 25, "Text",
 CellTags->"Rename"]
}, Open  ]],
Cell[1631614, 64322, 61, 1, 41, "ObjectName",
 CellTags->"RHI"],
Cell[CellGroupData[{
Cell[1631700, 64327, 54, 1, 36, "Subsubsection",
 CellTags->"RHI"],
Cell[1631757, 64330, 611, 12, 147, "Text",
 CellTags->"RHI"],
Cell[1632371, 64344, 168, 9, 25, "Text",
 CellTags->"RHI"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1632576, 64358, 51, 1, 34, "Subsubsection",
 CellTags->"RHI"],
Cell[1632630, 64361, 70, 2, 26, "Input",
 CellTags->"RHI"]
}, Open  ]],
Cell[1632715, 64366, 67, 1, 41, "ObjectName",
 CellTags->"RHI2FC"],
Cell[CellGroupData[{
Cell[1632807, 64371, 57, 1, 36, "Subsubsection",
 CellTags->"RHI2FC"],
Cell[1632867, 64374, 614, 12, 147, "Text",
 CellTags->"RHI2FC"],
Cell[1633484, 64388, 171, 9, 25, "Text",
 CellTags->"RHI2FC"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1633692, 64402, 54, 1, 34, "Subsubsection",
 CellTags->"RHI2FC"],
Cell[1633749, 64405, 73, 2, 26, "Input",
 CellTags->"RHI2FC"]
}, Open  ]],
Cell[1633837, 64410, 61, 1, 41, "ObjectName",
 CellTags->"RHM"],
Cell[CellGroupData[{
Cell[1633923, 64415, 54, 1, 36, "Subsubsection",
 CellTags->"RHM"],
Cell[1633980, 64418, 77, 1, 25, "Text",
 CellTags->"RHM"],
Cell[1634060, 64421, 159, 9, 25, "Text",
 CellTags->"RHM"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1634256, 64435, 51, 1, 34, "Subsubsection",
 CellTags->"RHM"],
Cell[1634310, 64438, 70, 2, 26, "Input",
 CellTags->"RHM"]
}, Open  ]],
Cell[1634395, 64443, 61, 1, 41, "ObjectName",
 CellTags->"RHO"],
Cell[CellGroupData[{
Cell[1634481, 64448, 54, 1, 36, "Subsubsection",
 CellTags->"RHO"],
Cell[1634538, 64451, 267, 5, 43, "Text",
 CellTags->"RHO"],
Cell[1634808, 64458, 159, 9, 25, "Text",
 CellTags->"RHO"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1635004, 64472, 51, 1, 34, "Subsubsection",
 CellTags->"RHO"],
Cell[1635058, 64475, 70, 2, 26, "Input",
 CellTags->"RHO"]
}, Open  ]],
Cell[1635143, 64480, 45, 1, 41, "ObjectName",
 CellTags->"RHP"],
Cell[CellGroupData[{
Cell[1635213, 64485, 54, 1, 36, "Subsubsection",
 CellTags->"RHP"],
Cell[1635270, 64488, 272, 5, 43, "Text",
 CellTags->"RHP"],
Cell[1635545, 64495, 159, 9, 25, "Text",
 CellTags->"RHP"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1635741, 64509, 51, 1, 34, "Subsubsection",
 CellTags->"RHP"],
Cell[CellGroupData[{
Cell[1635817, 64514, 133, 3, 26, "Input"],
Cell[1635953, 64519, 1981, 78, 51, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1637971, 64602, 133, 3, 26, "Input"],
Cell[1638107, 64607, 755, 30, 51, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1638899, 64642, 133, 3, 26, "Input"],
Cell[1639035, 64647, 1457, 60, 48, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1640529, 64712, 133, 3, 26, "Input"],
Cell[1640665, 64717, 551, 23, 48, "Output"]
}, Open  ]],
Cell[1641231, 64743, 70, 2, 26, "Input",
 CellTags->"RHP"]
}, Open  ]],
Cell[1641316, 64748, 61, 1, 41, "ObjectName",
 CellTags->"RTL"],
Cell[CellGroupData[{
Cell[1641402, 64753, 54, 1, 36, "Subsubsection",
 CellTags->"RTL"],
Cell[1641459, 64756, 188, 4, 43, "Text",
 CellTags->"RTL"],
Cell[1641650, 64762, 266, 14, 25, "Text",
 CellTags->"RTL"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1641953, 64781, 51, 1, 34, "Subsubsection",
 CellTags->"RTL"],
Cell[1642007, 64784, 70, 2, 26, "Input",
 CellTags->"RTL"]
}, Open  ]],
Cell[1642092, 64789, 63, 1, 41, "ObjectName",
 CellTags->"RightPartialD"],
Cell[CellGroupData[{
Cell[1642180, 64794, 64, 1, 36, "Subsubsection",
 CellTags->"RightPartialD"],
Cell[1642247, 64797, 256, 10, 25, "Text",
 CellTags->"RightPartialD"],
Cell[1642506, 64809, 434, 19, 25, "Text",
 CellTags->"RightPartialD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1642977, 64833, 61, 1, 34, "Subsubsection",
 CellTags->"RightPartialD"],
Cell[CellGroupData[{
Cell[1643063, 64838, 124, 3, 26, "Input",
 CellTags->"RightPartialD"],
Cell[1643190, 64843, 294, 10, 33, "Output",
 CellTags->"RightPartialD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1643521, 64858, 255, 7, 26, "Input",
 CellTags->"RightPartialD"],
Cell[1643779, 64867, 350, 12, 33, "Output",
 CellTags->"RightPartialD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1644166, 64884, 121, 3, 26, "Input",
 CellTags->"RightPartialD"],
Cell[1644290, 64889, 623, 15, 87, "Output",
 CellTags->"RightPartialD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1644950, 64909, 115, 3, 26, "Input",
 CellTags->"RightPartialD"],
Cell[1645068, 64914, 612, 12, 115, "Output",
 CellTags->"RightPartialD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1645717, 64931, 162, 5, 26, "Input",
 CellTags->"RightPartialD"],
Cell[1645882, 64938, 220, 5, 38, "Output",
 CellTags->"RightPartialD"]
}, Open  ]]
}, Open  ]],
Cell[1646129, 64947, 61, 1, 41, "ObjectName",
 CellTags->"RussianTrick"],
Cell[CellGroupData[{
Cell[1646215, 64952, 63, 1, 36, "Subsubsection",
 CellTags->"RussianTrick"],
Cell[1646281, 64955, 318, 6, 61, "Text",
 CellTags->"RussianTrick"],
Cell[1646602, 64963, 90, 1, 25, "Text",
 CellTags->"RussianTrick"],
Cell[1646695, 64966, 326, 14, 25, "Text",
 CellTags->"RussianTrick"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1647058, 64985, 60, 1, 34, "Subsubsection",
 CellTags->"RussianTrick"],
Cell[CellGroupData[{
Cell[1647143, 64990, 374, 12, 26, "Input",
 CellTags->"RussianTrick"],
Cell[1647520, 65004, 545, 22, 29, "Output",
 CellTags->"RussianTrick"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1648102, 65031, 108, 3, 26, "Input",
 CellTags->"RussianTrick"],
Cell[1648213, 65036, 703, 24, 50, "Output",
 CellTags->"RussianTrick"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1648953, 65065, 172, 5, 26, "Input",
 CellTags->"RussianTrick"],
Cell[1649128, 65072, 3510, 122, 146, "Output",
 CellTags->"RussianTrick"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1652675, 65199, 112, 3, 26, "Input",
 CellTags->"RussianTrick"],
Cell[1652790, 65204, 2716, 112, 29, "Output",
 CellTags->"RussianTrick"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1655543, 65321, 135, 4, 26, "Input",
 CellTags->"RussianTrick"],
Cell[1655681, 65327, 2991, 113, 48, "Output",
 CellTags->"RussianTrick"]
}, Open  ]],
Cell[1658687, 65443, 111, 3, 26, "Input",
 CellTags->"RussianTrick"]
}, Open  ]],
Cell[1658813, 65449, 71, 1, 41, "ObjectName",
 CellTags->"ScalarGluonVertex"],
Cell[CellGroupData[{
Cell[1658909, 65454, 68, 1, 36, "Subsubsection",
 CellTags->"ScalarGluonVertex"],
Cell[1658980, 65457, 430, 8, 91, "Text",
 CellTags->"ScalarGluonVertex"],
Cell[CellGroupData[{
Cell[1659435, 65469, 134, 3, 26, "Input",
 CellTags->"ScalarGluonVertex"],
Cell[1659572, 65474, 315, 9, 26, "Output",
 CellTags->"ScalarGluonVertex"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[1659936, 65489, 65, 1, 34, "Subsubsection",
 CellTags->"ScalarGluonVertex"],
Cell[CellGroupData[{
Cell[1660026, 65494, 274, 8, 26, "Input",
 CellTags->"ScalarGluonVertex"],
Cell[1660303, 65504, 521, 20, 28, "Output",
 CellTags->"ScalarGluonVertex"]
}, Open  ]]
}, Open  ]],
Cell[1660851, 65528, 63, 1, 41, "ObjectName",
 CellTags->"ScalarProduct"],
Cell[CellGroupData[{
Cell[1660939, 65533, 64, 1, 36, "Subsubsection",
 CellTags->"ScalarProduct"],
Cell[1661006, 65536, 836, 21, 79, "Text",
 CellTags->"ScalarProduct"],
Cell[1661845, 65559, 880, 38, 25, "Text",
 CellTags->"ScalarProduct"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1662762, 65602, 61, 1, 34, "Subsubsection",
 CellTags->"ScalarProduct"],
Cell[CellGroupData[{
Cell[1662848, 65607, 143, 4, 26, "Input",
 CellTags->"ScalarProduct"],
Cell[1662994, 65613, 254, 9, 28, "Output",
 CellTags->"ScalarProduct"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1663285, 65627, 186, 6, 26, "Input",
 CellTags->"ScalarProduct"],
Cell[1663474, 65635, 343, 12, 28, "Output",
 CellTags->"ScalarProduct"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1663854, 65652, 143, 4, 26, "Input",
 CellTags->"ScalarProduct"],
Cell[1664000, 65658, 210, 7, 28, "Output",
 CellTags->"ScalarProduct"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1664247, 65670, 120, 3, 26, "Input",
 CellTags->"ScalarProduct"],
Cell[1664370, 65675, 210, 7, 28, "Output",
 CellTags->"ScalarProduct"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1664617, 65687, 179, 5, 26, "Input",
 CellTags->"ScalarProduct"],
Cell[1664799, 65694, 265, 7, 38, "Output",
 CellTags->"ScalarProduct"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1665101, 65706, 231, 7, 26, "Input",
 CellTags->"ScalarProduct"],
Cell[1665335, 65715, 313, 9, 38, "Output",
 CellTags->"ScalarProduct"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1665685, 65729, 248, 8, 26, "Input",
 CellTags->"ScalarProduct"],
Cell[1665936, 65739, 173, 5, 41, "Output",
 CellTags->"ScalarProduct"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1666146, 65749, 295, 9, 26, "Input",
 CellTags->"ScalarProduct"],
Cell[1666444, 65760, 615, 24, 41, "Output",
 CellTags->"ScalarProduct"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1667096, 65789, 280, 9, 26, "Input",
 CellTags->"ScalarProduct"],
Cell[1667379, 65800, 613, 24, 41, "Output",
 CellTags->"ScalarProduct"]
}, Open  ]],
Cell[1668007, 65827, 99, 2, 26, "Input",
 CellTags->"ScalarProduct"]
}, Open  ]],
Cell[1668121, 65832, 75, 1, 39, "ObjectName",
 CellTags->"ScalarProductCancel"],
Cell[CellGroupData[{
Cell[1668221, 65837, 70, 1, 34, "Subsubsection",
 CellTags->"ScalarProductCancel"],
Cell[1668294, 65840, 186, 4, 24, "Text",
 CellTags->"ScalarProductCancel"],
Cell[CellGroupData[{
Cell[1668505, 65848, 89, 2, 26, "Input",
 CellTags->"ScalarProductCancel"],
Cell[1668597, 65852, 178, 4, 26, "Output",
 CellTags->"ScalarProductCancel"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1668812, 65861, 134, 3, 26, "Input",
 CellTags->"ScalarProductCancel"],
Cell[1668949, 65866, 436, 11, 26, "Output",
 CellTags->"ScalarProductCancel"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1669422, 65882, 106, 3, 26, "Input",
 CellTags->"ScalarProductCancel"],
Cell[1669531, 65887, 221, 4, 39, "Print",
 CellTags->{"Info3500973967-4982097", "ScalarProductCancel"}]
}, Open  ]],
Cell[1669767, 65894, 743, 33, 24, "Text",
 CellTags->"ScalarProductCancel"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1670547, 65932, 67, 1, 34, "Subsubsection",
 CellTags->"ScalarProductCancel"],
Cell[1670617, 65935, 244, 9, 24, "Text",
 CellTags->"ScalarProductCancel"],
Cell[CellGroupData[{
Cell[1670886, 65948, 429, 15, 26, "Input",
 CellTags->"ScalarProductCancel"],
Cell[1671318, 65965, 498, 18, 47, "Output",
 CellTags->"ScalarProductCancel"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1671853, 65988, 156, 4, 26, "Input",
 CellTags->"ScalarProductCancel"],
Cell[1672012, 65994, 891, 33, 59, "Output",
 CellTags->"ScalarProductCancel"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1672940, 66032, 162, 4, 26, "Input",
 CellTags->"ScalarProductCancel"],
Cell[1673105, 66038, 750, 27, 59, "Output",
 CellTags->"ScalarProductCancel"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1673892, 66070, 185, 5, 26, "Input",
 CellTags->"ScalarProductCancel"],
Cell[1674080, 66077, 752, 27, 59, "Output",
 CellTags->"ScalarProductCancel"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1674869, 66109, 578, 18, 26, "Input",
 CellTags->"ScalarProductCancel"],
Cell[1675450, 66129, 940, 34, 50, "Output",
 CellTags->"ScalarProductCancel"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1676427, 66168, 197, 5, 26, "Input",
 CellTags->"ScalarProductCancel"],
Cell[1676627, 66175, 3150, 112, 178, "Output",
 CellTags->"ScalarProductCancel"]
}, Open  ]],
Cell[1679792, 66290, 162, 5, 26, "Input",
 CellTags->"ScalarProductCancel"]
}, Open  ]],
Cell[1679969, 66298, 51, 1, 41, "ObjectName",
 CellTags->"ScaleMu"],
Cell[CellGroupData[{
Cell[1680045, 66303, 58, 1, 36, "Subsubsection",
 CellTags->"ScaleMu"],
Cell[1680106, 66306, 128, 4, 24, "Text",
 CellTags->"ScaleMu"],
Cell[CellGroupData[{
Cell[1680259, 66314, 81, 2, 26, "Input",
 CellTags->"ScaleMu"],
Cell[1680343, 66318, 149, 4, 26, "Output",
 CellTags->"ScaleMu"]
}, Open  ]]
}, Open  ]],
Cell[1680519, 66326, 53, 1, 41, "ObjectName",
 CellTags->"Schouten"],
Cell[CellGroupData[{
Cell[1680597, 66331, 59, 1, 36, "Subsubsection",
 CellTags->"Schouten"],
Cell[1680659, 66334, 460, 8, 87, "Text",
 CellTags->"Schouten"],
Cell[1681122, 66344, 289, 13, 25, "Text",
 CellTags->"Schouten"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1681448, 66362, 56, 1, 34, "Subsubsection",
 CellTags->"Schouten"],
Cell[CellGroupData[{
Cell[1681529, 66367, 1938, 54, 184, "Input",
 CellTags->"Schouten"],
Cell[1683470, 66423, 15738, 629, 324, "Output",
 CellTags->"Schouten"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1699245, 67057, 106, 3, 26, "Input",
 CellTags->"Schouten"],
Cell[1699354, 67062, 147, 4, 26, "Output",
 CellTags->"Schouten"]
}, Open  ]],
Cell[1699516, 67069, 84, 2, 26, "Input"]
}, Open  ]],
Cell[1699615, 67074, 41, 1, 41, "ObjectName",
 CellTags->"SD"],
Cell[CellGroupData[{
Cell[1699681, 67079, 53, 1, 36, "Subsubsection",
 CellTags->"SD"],
Cell[1699737, 67082, 265, 7, 43, "Text",
 CellTags->"SD"],
Cell[1700005, 67091, 173, 9, 25, "Text",
 CellTags->"SD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1700215, 67105, 50, 1, 34, "Subsubsection",
 CellTags->"SD"],
Cell[CellGroupData[{
Cell[1700290, 67110, 121, 4, 26, "Input",
 CellTags->"SD"],
Cell[1700414, 67116, 191, 6, 28, "Output",
 CellTags->"SD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1700642, 67127, 130, 4, 26, "Input",
 CellTags->"SD"],
Cell[1700775, 67133, 256, 7, 38, "Output",
 CellTags->"SD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1701068, 67145, 130, 4, 26, "Input",
 CellTags->"SD"],
Cell[1701201, 67151, 178, 5, 38, "Output",
 CellTags->"SD"]
}, Open  ]]
}, Open  ]],
Cell[1701406, 67160, 57, 1, 41, "ObjectName",
 CellTags->"SelectFree"],
Cell[CellGroupData[{
Cell[1701488, 67165, 61, 1, 36, "Subsubsection",
 CellTags->"SelectFree"],
Cell[1701552, 67168, 255, 5, 43, "Text",
 CellTags->"SelectFree"],
Cell[1701810, 67175, 77, 1, 25, "Text",
 CellTags->"SelectFree"],
Cell[1701890, 67178, 300, 14, 25, "Text",
 CellTags->"SelectFree"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1702227, 67197, 58, 1, 34, "Subsubsection",
 CellTags->"SelectFree"],
Cell[CellGroupData[{
Cell[1702310, 67202, 211, 6, 26, "Input",
 CellTags->"SelectFree"],
Cell[1702524, 67210, 172, 5, 26, "Output",
 CellTags->"SelectFree"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1702733, 67220, 166, 5, 26, "Input",
 CellTags->"SelectFree"],
Cell[1702902, 67227, 151, 4, 27, "Output",
 CellTags->"SelectFree"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1703090, 67236, 275, 8, 26, "Input",
 CellTags->"SelectFree"],
Cell[1703368, 67246, 171, 5, 26, "Output",
 CellTags->"SelectFree"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1703576, 67256, 137, 4, 26, "Input",
 CellTags->"SelectFree"],
Cell[1703716, 67262, 149, 4, 27, "Output",
 CellTags->"SelectFree"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1703902, 67271, 137, 4, 26, "Input",
 CellTags->"SelectFree"],
Cell[1704042, 67277, 148, 4, 26, "Output",
 CellTags->"SelectFree"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1704227, 67286, 137, 4, 26, "Input",
 CellTags->"SelectFree"],
Cell[1704367, 67292, 151, 4, 26, "Output",
 CellTags->"SelectFree"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1704555, 67301, 166, 5, 26, "Input",
 CellTags->"SelectFree"],
Cell[1704724, 67308, 151, 4, 26, "Output",
 CellTags->"SelectFree"]
}, Open  ]]
}, Open  ]],
Cell[1704902, 67316, 61, 1, 41, "ObjectName",
 CellTags->"SelectGraphs"],
Cell[CellGroupData[{
Cell[1704988, 67321, 63, 1, 36, "Subsubsection",
 CellTags->"SelectGraphs"],
Cell[1705054, 67324, 486, 8, 79, "Text",
 CellTags->"SelectGraphs"],
Cell[1705543, 67334, 189, 9, 25, "Text",
 CellTags->"SelectGraphs"]
}, Open  ]],
Cell[1705747, 67346, 63, 1, 41, "ObjectName",
 CellTags->"SelectNotFree"],
Cell[CellGroupData[{
Cell[1705835, 67351, 64, 1, 36, "Subsubsection",
 CellTags->"SelectNotFree"],
Cell[1705902, 67354, 143, 4, 25, "Text",
 CellTags->"SelectNotFree"],
Cell[1706048, 67360, 270, 5, 43, "Text",
 CellTags->"SelectNotFree"],
Cell[1706321, 67367, 83, 1, 25, "Text",
 CellTags->"SelectNotFree"],
Cell[1706407, 67370, 294, 14, 25, "Text",
 CellTags->"SelectNotFree"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1706738, 67389, 61, 1, 34, "Subsubsection",
 CellTags->"SelectNotFree"],
Cell[CellGroupData[{
Cell[1706824, 67394, 207, 6, 26, "Input",
 CellTags->"SelectNotFree"],
Cell[1707034, 67402, 206, 6, 26, "Output",
 CellTags->"SelectNotFree"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1707277, 67413, 276, 8, 26, "Input",
 CellTags->"SelectNotFree"],
Cell[1707556, 67423, 214, 6, 26, "Output",
 CellTags->"SelectNotFree"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1707807, 67434, 143, 4, 26, "Input",
 CellTags->"SelectNotFree"],
Cell[1707953, 67440, 154, 4, 26, "Output",
 CellTags->"SelectNotFree"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1708144, 67449, 167, 5, 26, "Input",
 CellTags->"SelectNotFree"],
Cell[1708314, 67456, 154, 4, 26, "Output",
 CellTags->"SelectNotFree"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1708505, 67465, 143, 4, 26, "Input",
 CellTags->"SelectNotFree"],
Cell[1708651, 67471, 152, 4, 27, "Output",
 CellTags->"SelectNotFree"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1708840, 67480, 143, 4, 26, "Input",
 CellTags->"SelectNotFree"],
Cell[1708986, 67486, 152, 4, 26, "Output",
 CellTags->"SelectNotFree"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1709175, 67495, 172, 5, 26, "Input",
 CellTags->"SelectNotFree"],
Cell[1709350, 67502, 180, 5, 26, "Output",
 CellTags->"SelectNotFree"]
}, Open  ]]
}, Open  ]],
Cell[1709557, 67511, 59, 1, 41, "ObjectName",
 CellTags->"SelectSplit"],
Cell[CellGroupData[{
Cell[1709641, 67516, 62, 1, 36, "Subsubsection",
 CellTags->"SelectSplit"],
Cell[1709706, 67519, 249, 5, 43, "Text",
 CellTags->"SelectSplit"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1709992, 67529, 59, 1, 34, "Subsubsection",
 CellTags->"SelectSplit"],
Cell[CellGroupData[{
Cell[1710076, 67534, 804, 26, 53, "Input",
 CellTags->"SelectSplit"],
Cell[1710883, 67562, 574, 20, 29, "Output",
 CellTags->"SelectSplit"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1711494, 67587, 652, 21, 30, "Input",
 CellTags->"SelectSplit"],
Cell[1712149, 67610, 537, 17, 29, "Output",
 CellTags->"SelectSplit"]
}, Open  ]]
}, Open  ]],
Cell[1712713, 67631, 51, 1, 41, "ObjectName",
 CellTags->"Select1"],
Cell[CellGroupData[{
Cell[1712789, 67636, 58, 1, 36, "Subsubsection",
 CellTags->"Select1"],
Cell[1712850, 67639, 243, 5, 43, "Text",
 CellTags->"Select1"],
Cell[1713096, 67646, 175, 9, 25, "Text",
 CellTags->"Select1"]
}, Open  ]],
Cell[1713286, 67658, 51, 1, 41, "ObjectName",
 CellTags->"Select2"],
Cell[CellGroupData[{
Cell[1713362, 67663, 58, 1, 36, "Subsubsection",
 CellTags->"Select2"],
Cell[1713423, 67666, 245, 5, 43, "Text",
 CellTags->"Select2"],
Cell[1713671, 67673, 175, 9, 25, "Text",
 CellTags->"Select2"]
}, Open  ]],
Cell[1713861, 67685, 51, 1, 41, "ObjectName",
 CellTags->"Series2"],
Cell[CellGroupData[{
Cell[1713937, 67690, 58, 1, 36, "Subsubsection",
 CellTags->"Series2"],
Cell[1713998, 67693, 383, 9, 41, "Text",
 CellTags->"Series2"],
Cell[CellGroupData[{
Cell[1714406, 67706, 114, 3, 26, "Input",
 CellTags->"Series2"],
Cell[1714523, 67711, 570, 16, 28, "Output",
 CellTags->"Series2"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[1715142, 67733, 55, 1, 34, "Subsubsection",
 CellTags->"Series2"],
Cell[CellGroupData[{
Cell[1715222, 67738, 339, 12, 26, "Input",
 CellTags->"Series2"],
Cell[1715564, 67752, 399, 13, 44, "Output",
 CellTags->"Series2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1716000, 67770, 174, 5, 26, "Input",
 CellTags->"Series2"],
Cell[1716177, 67777, 283, 9, 46, "Output",
 CellTags->"Series2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1716497, 67791, 222, 7, 26, "Input",
 CellTags->"Series2"],
Cell[1716722, 67800, 848, 26, 46, "Output",
 CellTags->"Series2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1717607, 67831, 174, 5, 26, "Input",
 CellTags->"Series2"],
Cell[1717784, 67838, 427, 15, 48, "Output",
 CellTags->"Series2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1718248, 67858, 290, 8, 26, "Input",
 CellTags->"Series2"],
Cell[1718541, 67868, 1036, 33, 46, "Output",
 CellTags->"Series2"]
}, Open  ]],
Cell[1719592, 67904, 187, 6, 24, "Text"],
Cell[CellGroupData[{
Cell[1719804, 67914, 460, 15, 26, "Input",
 CellTags->"Series2"],
Cell[1720267, 67931, 1042, 34, 46, "Output",
 CellTags->"Series2"]
}, Open  ]],
Cell[1721324, 67968, 113, 3, 24, "Text",
 CellTags->"Series2"],
Cell[CellGroupData[{
Cell[1721462, 67975, 484, 15, 46, "Input",
 CellTags->"Series2"],
Cell[1721949, 67992, 690, 25, 46, "Output",
 CellTags->"Series2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1722676, 68022, 578, 17, 46, "Input",
 CellTags->"Series2"],
Cell[1723257, 68041, 1030, 36, 44, "Output",
 CellTags->"Series2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1724324, 68082, 242, 6, 26, "Input",
 CellTags->"Series2"],
Cell[1724569, 68090, 336, 8, 26, "Output",
 CellTags->"Series2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1724942, 68103, 147, 4, 26, "Input",
 CellTags->"Series2"],
Cell[1725092, 68109, 3156, 107, 106, "Output",
 CellTags->"Series2"]
}, Open  ]],
Cell[1728263, 68219, 351, 11, 41, "Text",
 CellTags->"Series2"]
}, Open  ]],
Cell[1728629, 68233, 51, 1, 39, "ObjectName",
 CellTags->"Series3"],
Cell[CellGroupData[{
Cell[1728705, 68238, 58, 1, 34, "Subsubsection",
 CellTags->"Series3"],
Cell[1728766, 68241, 267, 5, 41, "Text",
 CellTags->"Series3"],
Cell[CellGroupData[{
Cell[1729058, 68250, 114, 3, 26, "Input",
 CellTags->"Series3"],
Cell[1729175, 68255, 306, 9, 26, "Output",
 CellTags->"Series3"]
}, Open  ]],
Cell[1729496, 68267, 175, 9, 24, "Text",
 CellTags->"Series3"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1729708, 68281, 55, 1, 34, "Subsubsection",
 CellTags->"Series3"],
Cell[CellGroupData[{
Cell[1729788, 68286, 339, 12, 26, "Input",
 CellTags->"Series3"],
Cell[1730130, 68300, 349, 12, 44, "Output",
 CellTags->"Series3"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1730516, 68317, 214, 7, 26, "Input",
 CellTags->"Series3"],
Cell[1730733, 68326, 257, 8, 46, "Output",
 CellTags->"Series3"]
}, Open  ]]
}, Open  ]],
Cell[1731017, 68338, 63, 1, 41, "ObjectName",
 CellTags->"SetMandelstam"],
Cell[CellGroupData[{
Cell[1731105, 68343, 64, 1, 36, "Subsubsection",
 CellTags->"SetMandelstam"],
Cell[1731172, 68346, 462, 9, 83, "Text",
 CellTags->"SetMandelstam"],
Cell[CellGroupData[{
Cell[1731659, 68359, 126, 3, 26, "Input",
 CellTags->"SetMandelstam"],
Cell[1731788, 68364, 281, 9, 26, "Output",
 CellTags->"SetMandelstam"]
}, Open  ]],
Cell[1732084, 68376, 190, 9, 25, "Text",
 CellTags->"SetMandelstam"]
}, Open  ]],
Cell[1732289, 68388, 79, 1, 41, "ObjectName",
 CellTags->"SimplifyDeltaFunction"],
Cell[CellGroupData[{
Cell[1732393, 68393, 72, 1, 36, "Subsubsection",
 CellTags->"SimplifyDeltaFunction"],
Cell[1732468, 68396, 299, 6, 43, "Text",
 CellTags->"SimplifyDeltaFunction"],
Cell[1732770, 68404, 347, 14, 25, "Text",
 CellTags->"SimplifyDeltaFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1733154, 68423, 69, 1, 34, "Subsubsection",
 CellTags->"SimplifyDeltaFunction"],
Cell[CellGroupData[{
Cell[1733248, 68428, 203, 6, 26, "Input",
 CellTags->"SimplifyDeltaFunction"],
Cell[1733454, 68436, 310, 10, 26, "Output",
 CellTags->"SimplifyDeltaFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1733801, 68451, 164, 4, 26, "Input",
 CellTags->"SimplifyDeltaFunction"],
Cell[1733968, 68457, 434, 14, 41, "Output",
 CellTags->"SimplifyDeltaFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1734439, 68476, 203, 6, 26, "Input",
 CellTags->"SimplifyDeltaFunction"],
Cell[1734645, 68484, 345, 11, 26, "Output",
 CellTags->"SimplifyDeltaFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1735027, 68500, 164, 4, 26, "Input",
 CellTags->"SimplifyDeltaFunction"],
Cell[1735194, 68506, 854, 28, 41, "Output",
 CellTags->"SimplifyDeltaFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1736085, 68539, 220, 6, 26, "Input",
 CellTags->"SimplifyDeltaFunction"],
Cell[1736308, 68547, 355, 11, 26, "Output",
 CellTags->"SimplifyDeltaFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1736700, 68563, 164, 4, 26, "Input",
 CellTags->"SimplifyDeltaFunction"],
Cell[1736867, 68569, 254, 8, 26, "Output",
 CellTags->"SimplifyDeltaFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1737158, 68582, 263, 8, 26, "Input",
 CellTags->"SimplifyDeltaFunction"],
Cell[1737424, 68592, 372, 12, 26, "Output",
 CellTags->"SimplifyDeltaFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1737833, 68609, 164, 4, 26, "Input",
 CellTags->"SimplifyDeltaFunction"],
Cell[1738000, 68615, 276, 9, 26, "Output",
 CellTags->"SimplifyDeltaFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1738313, 68629, 298, 9, 26, "Input",
 CellTags->"SimplifyDeltaFunction"],
Cell[1738614, 68640, 415, 13, 26, "Output",
 CellTags->"SimplifyDeltaFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1739066, 68658, 164, 4, 26, "Input",
 CellTags->"SimplifyDeltaFunction"],
Cell[1739233, 68664, 162, 4, 26, "Output",
 CellTags->"SimplifyDeltaFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1739432, 68673, 263, 8, 26, "Input",
 CellTags->"SimplifyDeltaFunction"],
Cell[1739698, 68683, 374, 12, 26, "Output",
 CellTags->"SimplifyDeltaFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1740109, 68700, 164, 4, 26, "Input",
 CellTags->"SimplifyDeltaFunction"],
Cell[1740276, 68706, 276, 9, 26, "Output",
 CellTags->"SimplifyDeltaFunction"]
}, Open  ]]
}, Open  ]],
Cell[1740579, 68719, 77, 1, 41, "ObjectName",
 CellTags->"SimplifyGTI"],
Cell[CellGroupData[{
Cell[1740681, 68724, 62, 1, 36, "Subsubsection",
 CellTags->"SimplifyGTI"],
Cell[1740746, 68727, 71, 1, 25, "Text",
 CellTags->"SimplifyGTI"],
Cell[1740820, 68730, 167, 9, 25, "Text",
 CellTags->"SimplifyGTI"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1741024, 68744, 59, 1, 34, "Subsubsection",
 CellTags->"SimplifyGTI"],
Cell[1741086, 68747, 78, 2, 26, "Input",
 CellTags->"SimplifyGTI"]
}, Open  ]],
Cell[1741179, 68752, 67, 1, 41, "ObjectName",
 CellTags->"SimplifyPolyLog"],
Cell[CellGroupData[{
Cell[1741271, 68757, 66, 1, 36, "Subsubsection",
 CellTags->"SimplifyPolyLog"],
Cell[1741340, 68760, 199, 4, 25, "Text",
 CellTags->"SimplifyPolyLog"],
Cell[1741542, 68766, 183, 9, 25, "Text",
 CellTags->"SimplifyPolyLog"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1741762, 68780, 63, 1, 34, "Subsubsection",
 CellTags->"SimplifyPolyLog"],
Cell[1741828, 68783, 224, 6, 26, "Input",
 CellTags->"SimplifyPolyLog"],
Cell[CellGroupData[{
Cell[1742077, 68793, 195, 6, 26, "Input",
 CellTags->"SimplifyPolyLog"],
Cell[1742275, 68801, 708, 21, 46, "Output",
 CellTags->"SimplifyPolyLog"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1743020, 68827, 170, 5, 26, "Input",
 CellTags->"SimplifyPolyLog"],
Cell[1743193, 68834, 474, 15, 26, "Output",
 CellTags->"SimplifyPolyLog"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1743704, 68854, 221, 7, 26, "Input",
 CellTags->"SimplifyPolyLog"],
Cell[1743928, 68863, 675, 21, 29, "Output",
 CellTags->"SimplifyPolyLog"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1744640, 68889, 195, 6, 26, "Input",
 CellTags->"SimplifyPolyLog"],
Cell[1744838, 68897, 649, 20, 29, "Output",
 CellTags->"SimplifyPolyLog"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1745524, 68922, 271, 9, 26, "Input",
 CellTags->"SimplifyPolyLog"],
Cell[1745798, 68933, 714, 23, 44, "Output",
 CellTags->"SimplifyPolyLog"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1746549, 68961, 248, 8, 26, "Input",
 CellTags->"SimplifyPolyLog"],
Cell[1746800, 68971, 689, 22, 44, "Output",
 CellTags->"SimplifyPolyLog"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1747526, 68998, 281, 9, 26, "Input",
 CellTags->"SimplifyPolyLog"],
Cell[1747810, 69009, 774, 30, 44, "Output",
 CellTags->"SimplifyPolyLog"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1748621, 69044, 216, 7, 26, "Input",
 CellTags->"SimplifyPolyLog"],
Cell[1748840, 69053, 538, 18, 48, "Output",
 CellTags->"SimplifyPolyLog"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1749415, 69076, 195, 6, 26, "Input",
 CellTags->"SimplifyPolyLog"],
Cell[1749613, 69084, 157, 4, 26, "Output",
 CellTags->"SimplifyPolyLog"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1749807, 69093, 195, 6, 26, "Input",
 CellTags->"SimplifyPolyLog"],
Cell[1750005, 69101, 1107, 37, 29, "Output",
 CellTags->"SimplifyPolyLog"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1751149, 69143, 271, 9, 26, "Input",
 CellTags->"SimplifyPolyLog"],
Cell[1751423, 69154, 1522, 51, 82, "Output",
 CellTags->"SimplifyPolyLog"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1752982, 69210, 221, 7, 26, "Input",
 CellTags->"SimplifyPolyLog"],
Cell[1753206, 69219, 1297, 46, 87, "Output",
 CellTags->"SimplifyPolyLog"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1754540, 69270, 271, 9, 26, "Input",
 CellTags->"SimplifyPolyLog"],
Cell[1754814, 69281, 2782, 97, 124, "Output",
 CellTags->"SimplifyPolyLog"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1757633, 69383, 191, 6, 26, "Input",
 CellTags->"SimplifyPolyLog"],
Cell[1757827, 69391, 378, 13, 47, "Output",
 CellTags->"SimplifyPolyLog"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1758242, 69409, 166, 5, 26, "Input",
 CellTags->"SimplifyPolyLog"],
Cell[1758411, 69416, 295, 9, 46, "Output",
 CellTags->"SimplifyPolyLog"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1758743, 69430, 146, 4, 26, "Input",
 CellTags->"SimplifyPolyLog"],
Cell[1758892, 69436, 450, 15, 46, "Output",
 CellTags->"SimplifyPolyLog"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1759379, 69456, 146, 4, 26, "Input",
 CellTags->"SimplifyPolyLog"],
Cell[1759528, 69462, 399, 14, 44, "Output",
 CellTags->"SimplifyPolyLog"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1759964, 69481, 146, 4, 26, "Input",
 CellTags->"SimplifyPolyLog"],
Cell[1760113, 69487, 397, 14, 44, "Output",
 CellTags->"SimplifyPolyLog"]
}, Open  ]],
Cell[1760525, 69504, 116, 3, 26, "Input",
 CellTags->"SimplifyPolyLog"]
}, Open  ]],
Cell[1760656, 69510, 73, 1, 39, "ObjectName",
 CellTags->"Simplify2"],
Cell[CellGroupData[{
Cell[1760754, 69515, 60, 1, 34, "Subsubsection",
 CellTags->"Simplify2"],
Cell[1760817, 69518, 81, 1, 24, "Text",
 CellTags->"Simplify2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1760935, 69524, 57, 1, 34, "Subsubsection",
 CellTags->"Simplify2"],
Cell[1760995, 69527, 76, 2, 26, "Input",
 CellTags->"Simplify2"]
}, Open  ]],
Cell[1761086, 69532, 57, 1, 39, "ObjectName",
 CellTags->"SmallDelta"],
Cell[CellGroupData[{
Cell[1761168, 69537, 61, 1, 34, "Subsubsection",
 CellTags->"SmallDelta"],
Cell[1761232, 69540, 87, 1, 24, "Text",
 CellTags->"SmallDelta"]
}, Open  ]],
Cell[1761334, 69544, 61, 1, 39, "ObjectName",
 CellTags->"SmallEpsilon"],
Cell[CellGroupData[{
Cell[1761420, 69549, 63, 1, 34, "Subsubsection",
 CellTags->"SmallEpsilon"],
Cell[1761486, 69552, 91, 1, 24, "Text",
 CellTags->"SmallEpsilon"]
}, Open  ]],
Cell[1761592, 69556, 63, 1, 39, "ObjectName",
 CellTags->"SmallVariable"],
Cell[CellGroupData[{
Cell[1761680, 69561, 64, 1, 34, "Subsubsection",
 CellTags->"SmallVariable"],
Cell[1761747, 69564, 281, 5, 41, "Text",
 CellTags->"SmallVariable"],
Cell[1762031, 69571, 437, 19, 24, "Text",
 CellTags->"SmallVariable"]
}, Open  ]],
Cell[1762483, 69593, 65, 1, 39, "ObjectName",
 CellTags->"SmallVariables"],
Cell[CellGroupData[{
Cell[1762573, 69598, 65, 1, 36, "Subsubsection",
 CellTags->"SmallVariables"],
Cell[1762641, 69601, 225, 5, 43, "Text",
 CellTags->"SmallVariables"],
Cell[1762869, 69608, 182, 9, 25, "Text",
 CellTags->"SmallVariables"]
}, Open  ]],
Cell[1763066, 69620, 43, 1, 41, "ObjectName",
 CellTags->"SMP"],
Cell[CellGroupData[{
Cell[1763134, 69625, 54, 1, 36, "Subsubsection",
 CellTags->"SMP"],
Cell[1763191, 69628, 232, 5, 41, "Text",
 CellTags->"SMP"],
Cell[CellGroupData[{
Cell[1763448, 69637, 127, 4, 26, "Input",
 CellTags->"SMP"],
Cell[1763578, 69643, 208, 6, 38, "Output",
 CellTags->"SMP"]
}, Open  ]],
Cell[1763801, 69652, 168, 8, 25, "Text",
 CellTags->"SMP"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1764006, 69665, 51, 1, 34, "Subsubsection",
 CellTags->"SMP"],
Cell[CellGroupData[{
Cell[1764082, 69670, 109, 3, 26, "Input",
 CellTags->"SMP"],
Cell[1764194, 69675, 143, 4, 26, "Output",
 CellTags->"SMP"]
}, Open  ]]
}, Open  ]],
Cell[1764364, 69683, 53, 1, 41, "ObjectName",
 CellTags->"SMVertex"],
Cell[CellGroupData[{
Cell[1764442, 69688, 59, 1, 36, "Subsubsection",
 CellTags->"SMVertex"],
Cell[1764504, 69691, 304, 6, 41, "Text",
 CellTags->"SMVertex"],
Cell[CellGroupData[{
Cell[1764833, 69701, 112, 3, 26, "Input",
 CellTags->"SMVertex"],
Cell[1764948, 69706, 279, 9, 26, "Output",
 CellTags->"SMVertex"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[1765276, 69721, 56, 1, 34, "Subsubsection",
 CellTags->"SMVertex"],
Cell[CellGroupData[{
Cell[1765357, 69726, 223, 6, 26, "Input",
 CellTags->"SMVertex"],
Cell[1765583, 69734, 1223, 45, 28, "Output",
 CellTags->"SMVertex"]
}, Open  ]]
}, Open  ]],
Cell[1766833, 69783, 61, 1, 41, "ObjectName",
 CellTags->"Smu"],
Cell[CellGroupData[{
Cell[1766919, 69788, 54, 1, 36, "Subsubsection",
 CellTags->"Smu"],
Cell[1766976, 69791, 45, 1, 25, "Text",
 CellTags->"Smu"]
}, Open  ]],
Cell[1767036, 69795, 41, 1, 41, "ObjectName",
 CellTags->"Sn"],
Cell[CellGroupData[{
Cell[1767102, 69800, 53, 1, 36, "Subsubsection",
 CellTags->"Sn"],
Cell[1767158, 69803, 300, 14, 25, "Text",
 CellTags->"Sn"]
}, Open  ]],
Cell[1767473, 69820, 41, 1, 41, "ObjectName",
 CellTags->"SO"],
Cell[CellGroupData[{
Cell[1767539, 69825, 53, 1, 36, "Subsubsection",
 CellTags->"SO"],
Cell[1767595, 69828, 173, 4, 43, "Text",
 CellTags->"SO"],
Cell[1767771, 69834, 580, 28, 25, "Text",
 CellTags->"SO"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1768388, 69867, 50, 1, 34, "Subsubsection",
 CellTags->"SO"],
Cell[CellGroupData[{
Cell[1768463, 69872, 98, 3, 26, "Input",
 CellTags->"SO"],
Cell[1768564, 69877, 307, 11, 28, "Output",
 CellTags->"SO"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1768908, 69893, 121, 4, 26, "Input",
 CellTags->"SO"],
Cell[1769032, 69899, 426, 16, 28, "Output",
 CellTags->"SO"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1769495, 69920, 163, 6, 26, "Input",
 CellTags->"SO"],
Cell[1769661, 69928, 259, 7, 38, "Output",
 CellTags->"SO"]
}, Open  ]]
}, Open  ]],
Cell[1769947, 69939, 43, 1, 70, "ObjectName",
 CellTags->"SOD"],
Cell[CellGroupData[{
Cell[1770015, 69944, 54, 1, 70, "Subsubsection",
 CellTags->"SOD"],
Cell[1770072, 69947, 189, 4, 70, "Text",
 CellTags->"SOD"],
Cell[1770264, 69953, 492, 24, 70, "Text",
 CellTags->"SOD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1770793, 69982, 51, 1, 34, "Subsubsection",
 CellTags->"SOD"],
Cell[CellGroupData[{
Cell[1770869, 69987, 100, 3, 26, "Input",
 CellTags->"SOD"],
Cell[1770972, 69992, 307, 11, 28, "Output",
 CellTags->"SOD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1771316, 70008, 123, 4, 26, "Input",
 CellTags->"SOD"],
Cell[1771442, 70014, 427, 16, 28, "Output",
 CellTags->"SOD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1771906, 70035, 165, 6, 26, "Input",
 CellTags->"SOD"],
Cell[1772074, 70043, 310, 9, 38, "Output",
 CellTags->"SOD"]
}, Open  ]]
}, Open  ]],
Cell[1772411, 70056, 49, 1, 70, "ObjectName",
 CellTags->"Solve2"],
Cell[CellGroupData[{
Cell[1772485, 70061, 57, 1, 70, "Subsubsection",
 CellTags->"Solve2"],
Cell[1772545, 70064, 346, 7, 58, "Text",
 CellTags->"Solve2"],
Cell[CellGroupData[{
Cell[1772916, 70075, 112, 3, 26, "Input",
 CellTags->"Solve2"],
Cell[1773031, 70080, 306, 9, 26, "Output",
 CellTags->"Solve2"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[1773386, 70095, 54, 1, 34, "Subsubsection",
 CellTags->"Solve2"],
Cell[CellGroupData[{
Cell[1773465, 70100, 390, 14, 26, "Input",
 CellTags->"Solve2"],
Cell[1773858, 70116, 415, 14, 44, "Output",
 CellTags->"Solve2"]
}, Open  ]],
Cell[1774288, 70133, 111, 3, 24, "Text",
 CellTags->"Solve2"],
Cell[CellGroupData[{
Cell[1774424, 70140, 153, 5, 26, "Input",
 CellTags->"Solve2"],
Cell[1774580, 70147, 218, 7, 26, "Output",
 CellTags->"Solve2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1774835, 70159, 267, 8, 26, "Input",
 CellTags->"Solve2"],
Cell[1775105, 70169, 218, 7, 26, "Output",
 CellTags->"Solve2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1775360, 70181, 443, 15, 26, "Input",
 CellTags->"Solve2"],
Cell[1775806, 70198, 356, 12, 45, "Output",
 CellTags->"Solve2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1776199, 70215, 389, 14, 26, "Input",
 CellTags->"Solve2"],
Cell[1776591, 70231, 448, 15, 44, "Output",
 CellTags->"Solve2"]
}, Open  ]]
}, Open  ]],
Cell[1777066, 70250, 49, 1, 70, "ObjectName",
 CellTags->"Solve3"],
Cell[CellGroupData[{
Cell[1777140, 70255, 57, 1, 70, "Subsubsection",
 CellTags->"Solve3"],
Cell[1777200, 70258, 271, 5, 41, "Text",
 CellTags->"Solve3"],
Cell[CellGroupData[{
Cell[1777496, 70267, 112, 3, 26, "Input",
 CellTags->"Solve3"],
Cell[1777611, 70272, 361, 11, 26, "Output",
 CellTags->"Solve3"]
}, Open  ]],
Cell[1777987, 70286, 162, 8, 70, "Text",
 CellTags->"SMP"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1778186, 70299, 54, 1, 34, "Subsubsection",
 CellTags->"Solve3"],
Cell[CellGroupData[{
Cell[1778265, 70304, 390, 14, 26, "Input",
 CellTags->"Solve3"],
Cell[1778658, 70320, 697, 17, 109, "Message",
 CellTags->"Solve3"],
Cell[1779358, 70339, 697, 17, 109, "Message",
 CellTags->"Solve3"],
Cell[1780058, 70358, 417, 14, 44, "Output",
 CellTags->"Solve3"]
}, Open  ]]
}, Open  ]],
Cell[1780502, 70376, 41, 1, 70, "ObjectName",
 CellTags->"SP"],
Cell[CellGroupData[{
Cell[1780568, 70381, 53, 1, 70, "Subsubsection",
 CellTags->"SP"],
Cell[1780624, 70384, 327, 11, 70, "Text",
 CellTags->"SP"],
Cell[1780954, 70397, 429, 19, 70, "Text",
 CellTags->"SP"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1781420, 70421, 50, 1, 34, "Subsubsection",
 CellTags->"SP"],
Cell[CellGroupData[{
Cell[1781495, 70426, 184, 6, 26, "Input",
 CellTags->"SP"],
Cell[1781682, 70434, 408, 17, 28, "Output",
 CellTags->"SP"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1782127, 70456, 189, 7, 26, "Input",
 CellTags->"SP"],
Cell[1782319, 70465, 339, 12, 28, "Output",
 CellTags->"SP"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1782695, 70482, 234, 8, 26, "Input",
 CellTags->"SP"],
Cell[1782932, 70492, 454, 18, 28, "Output",
 CellTags->"SP"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1783423, 70515, 168, 5, 26, "Input",
 CellTags->"SP"],
Cell[1783594, 70522, 251, 9, 28, "Output",
 CellTags->"SP"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1783882, 70536, 157, 5, 26, "Input",
 CellTags->"SP"],
Cell[1784042, 70543, 178, 5, 38, "Output",
 CellTags->"SP"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1784257, 70553, 188, 7, 26, "Input",
 CellTags->"SP"],
Cell[1784448, 70562, 254, 7, 38, "Output",
 CellTags->"SP"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1784739, 70574, 217, 8, 26, "Input",
 CellTags->"SP"],
Cell[1784959, 70584, 178, 5, 38, "Output",
 CellTags->"SP"]
}, Open  ]]
}, Open  ]],
Cell[1785164, 70593, 43, 1, 41, "ObjectName",
 CellTags->"SPC"],
Cell[CellGroupData[{
Cell[1785232, 70598, 54, 1, 36, "Subsubsection",
 CellTags->"SPC"],
Cell[1785289, 70601, 81, 1, 25, "Text",
 CellTags->"SPC"],
Cell[1785373, 70604, 207, 9, 25, "Text",
 CellTags->"SPC"]
}, Open  ]],
Cell[1785595, 70616, 43, 1, 41, "ObjectName",
 CellTags->"SPD"],
Cell[CellGroupData[{
Cell[1785663, 70621, 54, 1, 36, "Subsubsection",
 CellTags->"SPD"],
Cell[1785720, 70624, 346, 11, 43, "Text",
 CellTags->"SPD"],
Cell[1786069, 70637, 522, 24, 25, "Text",
 CellTags->"SPD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1786628, 70666, 51, 1, 34, "Subsubsection",
 CellTags->"SPD"],
Cell[CellGroupData[{
Cell[1786704, 70671, 187, 6, 26, "Input",
 CellTags->"SPD"],
Cell[1786894, 70679, 411, 17, 28, "Output",
 CellTags->"SPD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1787342, 70701, 191, 7, 26, "Input",
 CellTags->"SPD"],
Cell[1787536, 70710, 340, 12, 28, "Output",
 CellTags->"SPD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1787913, 70727, 236, 8, 26, "Input",
 CellTags->"SPD"],
Cell[1788152, 70737, 454, 18, 28, "Output",
 CellTags->"SPD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1788643, 70760, 170, 5, 26, "Input",
 CellTags->"SPD"],
Cell[1788816, 70767, 251, 9, 28, "Output",
 CellTags->"SPD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1789104, 70781, 159, 5, 26, "Input",
 CellTags->"SPD"],
Cell[1789266, 70788, 180, 5, 38, "Output",
 CellTags->"SPD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1789483, 70798, 190, 7, 26, "Input",
 CellTags->"SPD"],
Cell[1789676, 70807, 305, 9, 38, "Output",
 CellTags->"SPD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1790018, 70821, 219, 8, 26, "Input",
 CellTags->"SPD"],
Cell[1790240, 70831, 180, 5, 38, "Output",
 CellTags->"SPD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1790457, 70841, 270, 9, 26, "Input",
 CellTags->"SPD"],
Cell[1790730, 70852, 180, 5, 38, "Output",
 CellTags->"SPD"]
}, Open  ]]
}, Open  ]],
Cell[1790937, 70861, 49, 1, 41, "ObjectName",
 CellTags->"Spinor"],
Cell[CellGroupData[{
Cell[1791011, 70866, 57, 1, 36, "Subsubsection",
 CellTags->"Spinor"],
Cell[1791071, 70869, 1326, 49, 131, "Text",
 CellTags->"Spinor"],
Cell[1792400, 70920, 766, 34, 25, "Text",
 CellTags->"Spinor"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1793203, 70959, 54, 1, 34, "Subsubsection",
 CellTags->"Spinor"],
Cell[CellGroupData[{
Cell[1793282, 70964, 106, 3, 26, "Input",
 CellTags->"Spinor"],
Cell[1793391, 70969, 216, 7, 28, "Output",
 CellTags->"Spinor"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1793644, 70981, 129, 4, 26, "Input",
 CellTags->"Spinor"],
Cell[1793776, 70987, 229, 7, 28, "Output",
 CellTags->"Spinor"]
}, Open  ]],
Cell[1794020, 70997, 146, 4, 24, "Text",
 CellTags->"Spinor"],
Cell[CellGroupData[{
Cell[1794191, 71005, 250, 8, 26, "Input",
 CellTags->"Spinor"],
Cell[1794444, 71015, 168, 5, 26, "Output",
 CellTags->"Spinor"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1794649, 71025, 254, 8, 26, "Input",
 CellTags->"Spinor"],
Cell[1794906, 71035, 293, 10, 28, "Output",
 CellTags->"Spinor"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1795236, 71050, 141, 4, 26, "Input",
 CellTags->"Spinor"],
Cell[1795380, 71056, 234, 7, 38, "Output",
 CellTags->"Spinor"]
}, Open  ]],
Cell[1795629, 71066, 158, 4, 24, "Text",
 CellTags->"Spinor"],
Cell[CellGroupData[{
Cell[1795812, 71074, 212, 7, 26, "Input",
 CellTags->"Spinor"],
Cell[1796027, 71083, 234, 7, 38, "Output",
 CellTags->"Spinor"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1796298, 71095, 165, 5, 26, "Input",
 CellTags->"Spinor"],
Cell[1796466, 71102, 234, 7, 38, "Output",
 CellTags->"Spinor"]
}, Open  ]],
Cell[1796715, 71112, 77, 1, 24, "Text",
 CellTags->"Spinor"],
Cell[CellGroupData[{
Cell[1796817, 71117, 210, 7, 26, "Input",
 CellTags->"Spinor"],
Cell[1797030, 71126, 234, 7, 38, "Output",
 CellTags->"Spinor"]
}, Open  ]]
}, Open  ]],
Cell[1797291, 71137, 63, 1, 41, "ObjectName",
 CellTags->"SpinorCollect"],
Cell[CellGroupData[{
Cell[1797379, 71142, 64, 1, 36, "Subsubsection",
 CellTags->"SpinorCollect"],
Cell[1797446, 71145, 191, 4, 43, "Text",
 CellTags->"SpinorCollect"],
Cell[1797640, 71151, 301, 13, 25, "Text",
 CellTags->"SpinorCollect"]
}, Open  ]],
Cell[1797956, 71167, 57, 1, 41, "ObjectName",
 CellTags->"SpinorUBar"],
Cell[CellGroupData[{
Cell[1798038, 71172, 61, 1, 36, "Subsubsection",
 CellTags->"SpinorUBar"],
Cell[1798102, 71175, 170, 7, 25, "Text",
 CellTags->"SpinorUBar"],
Cell[1798275, 71184, 505, 24, 25, "Text",
 CellTags->"SpinorUBar"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1798817, 71213, 58, 1, 36, "Subsubsection",
 CellTags->"SpinorUBar"],
Cell[1798878, 71216, 85, 1, 24, "Text",
 CellTags->"SpinorUBar"],
Cell[CellGroupData[{
Cell[1798988, 71221, 114, 3, 26, "Input",
 CellTags->"SpinorUBar"],
Cell[1799105, 71226, 242, 8, 28, "Output",
 CellTags->"SpinorUBar"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1799384, 71239, 137, 4, 26, "Input",
 CellTags->"SpinorUBar"],
Cell[1799524, 71245, 256, 8, 28, "Output",
 CellTags->"SpinorUBar"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1799817, 71258, 173, 5, 26, "Input",
 CellTags->"SpinorUBar"],
Cell[1799993, 71265, 194, 5, 38, "Output",
 CellTags->"SpinorUBar"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1800224, 71275, 204, 7, 26, "Input",
 CellTags->"SpinorUBar"],
Cell[1800431, 71284, 240, 7, 38, "Output",
 CellTags->"SpinorUBar"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1800708, 71296, 233, 8, 26, "Input",
 CellTags->"SpinorUBar"],
Cell[1800944, 71306, 240, 7, 38, "Output",
 CellTags->"SpinorUBar"]
}, Open  ]]
}, Open  ]],
Cell[1801211, 71317, 51, 1, 41, "ObjectName",
 CellTags->"SpinorU"],
Cell[CellGroupData[{
Cell[1801287, 71322, 58, 1, 70, "Subsubsection",
 CellTags->"SpinorU"],
Cell[1801348, 71325, 79, 1, 70, "Text",
 CellTags->"SpinorU"],
Cell[1801430, 71328, 511, 24, 70, "Text",
 CellTags->"SpinorU"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1801978, 71357, 55, 1, 34, "Subsubsection",
 CellTags->"SpinorU"],
Cell[CellGroupData[{
Cell[1802058, 71362, 108, 3, 26, "Input",
 CellTags->"SpinorU"],
Cell[1802169, 71367, 174, 5, 28, "Output",
 CellTags->"SpinorU"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1802380, 71377, 131, 4, 26, "Input",
 CellTags->"SpinorU"],
Cell[1802514, 71383, 184, 5, 28, "Output",
 CellTags->"SpinorU"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1802735, 71393, 167, 5, 26, "Input",
 CellTags->"SpinorU"],
Cell[1802905, 71400, 188, 5, 38, "Output",
 CellTags->"SpinorU"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1803130, 71410, 198, 7, 26, "Input",
 CellTags->"SpinorU"],
Cell[1803331, 71419, 235, 7, 38, "Output",
 CellTags->"SpinorU"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1803603, 71431, 227, 8, 26, "Input",
 CellTags->"SpinorU"],
Cell[1803833, 71441, 235, 7, 38, "Output",
 CellTags->"SpinorU"]
}, Open  ]]
}, Open  ]],
Cell[1804095, 71452, 57, 1, 70, "ObjectName",
 CellTags->"SpinorVBar"],
Cell[CellGroupData[{
Cell[1804177, 71457, 61, 1, 70, "Subsubsection",
 CellTags->"SpinorVBar"],
Cell[1804241, 71460, 175, 7, 70, "Text",
 CellTags->"SpinorVBar"],
Cell[1804419, 71469, 505, 24, 70, "Text",
 CellTags->"SpinorVBar"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1804961, 71498, 58, 1, 34, "Subsubsection",
 CellTags->"SpinorVBar"],
Cell[CellGroupData[{
Cell[1805044, 71503, 114, 3, 26, "Input",
 CellTags->"SpinorVBar"],
Cell[1805161, 71508, 246, 8, 28, "Output",
 CellTags->"SpinorVBar"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1805444, 71521, 137, 4, 26, "Input",
 CellTags->"SpinorVBar"],
Cell[1805584, 71527, 256, 8, 28, "Output",
 CellTags->"SpinorVBar"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1805877, 71540, 173, 5, 26, "Input",
 CellTags->"SpinorVBar"],
Cell[1806053, 71547, 194, 5, 38, "Output",
 CellTags->"SpinorVBar"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1806284, 71557, 204, 7, 26, "Input",
 CellTags->"SpinorVBar"],
Cell[1806491, 71566, 259, 8, 38, "Output",
 CellTags->"SpinorVBar"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1806787, 71579, 233, 8, 26, "Input",
 CellTags->"SpinorVBar"],
Cell[1807023, 71589, 259, 8, 38, "Output",
 CellTags->"SpinorVBar"]
}, Open  ]]
}, Open  ]],
Cell[1807309, 71601, 51, 1, 70, "ObjectName",
 CellTags->"SpinorV"],
Cell[CellGroupData[{
Cell[1807385, 71606, 58, 1, 70, "Subsubsection",
 CellTags->"SpinorV"],
Cell[1807446, 71609, 79, 1, 70, "Text",
 CellTags->"SpinorV"],
Cell[1807528, 71612, 511, 24, 70, "Text",
 CellTags->"SpinorV"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1808076, 71641, 55, 1, 34, "Subsubsection",
 CellTags->"SpinorV"],
Cell[CellGroupData[{
Cell[1808156, 71646, 108, 3, 26, "Input",
 CellTags->"SpinorV"],
Cell[1808267, 71651, 174, 5, 28, "Output",
 CellTags->"SpinorV"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1808478, 71661, 131, 4, 26, "Input",
 CellTags->"SpinorV"],
Cell[1808612, 71667, 184, 5, 28, "Output",
 CellTags->"SpinorV"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1808833, 71677, 167, 5, 26, "Input",
 CellTags->"SpinorV"],
Cell[1809003, 71684, 188, 5, 38, "Output",
 CellTags->"SpinorV"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1809228, 71694, 198, 7, 26, "Input",
 CellTags->"SpinorV"],
Cell[1809429, 71703, 256, 8, 38, "Output",
 CellTags->"SpinorV"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1809722, 71716, 227, 8, 26, "Input",
 CellTags->"SpinorV"],
Cell[1809952, 71726, 258, 8, 38, "Output",
 CellTags->"SpinorV"]
}, Open  ]]
}, Open  ]],
Cell[1810237, 71738, 75, 1, 41, "ObjectName",
 CellTags->"SpinPolarizationSum"],
Cell[CellGroupData[{
Cell[1810337, 71743, 70, 1, 36, "Subsubsection",
 CellTags->"SpinPolarizationSum"],
Cell[1810410, 71746, 164, 4, 25, "Text",
 CellTags->"SpinPolarizationSum"],
Cell[1810577, 71752, 203, 8, 25, "Text",
 CellTags->"SpinPolarizationSum"]
}, Open  ]],
Cell[1810795, 71763, 43, 1, 41, "ObjectName",
 CellTags->"SPL"],
Cell[CellGroupData[{
Cell[1810863, 71768, 54, 1, 36, "Subsubsection",
 CellTags->"SPL"],
Cell[1810920, 71771, 77, 1, 25, "Text",
 CellTags->"SPL"],
Cell[1811000, 71774, 195, 9, 25, "Text",
 CellTags->"SPL"]
}, Open  ]],
Cell[1811210, 71786, 71, 1, 41, "ObjectName",
 CellTags->"SplittingFunction"],
Cell[CellGroupData[{
Cell[1811306, 71791, 68, 1, 36, "Subsubsection",
 CellTags->"SplittingFunction"],
Cell[1811377, 71794, 218, 7, 24, "Text",
 CellTags->"SplittingFunction"],
Cell[CellGroupData[{
Cell[1811620, 71805, 134, 3, 26, "Input",
 CellTags->"SplittingFunction"],
Cell[1811757, 71810, 222, 7, 26, "Output",
 CellTags->"SplittingFunction"]
}, Open  ]],
Cell[1811994, 71820, 218, 9, 25, "Text",
 CellTags->"SplittingFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1812249, 71834, 65, 1, 36, "Subsubsection",
 CellTags->"SplittingFunction"],
Cell[1812317, 71837, 65, 1, 24, "Text",
 CellTags->"SplittingFunction"],
Cell[1812385, 71840, 221, 6, 26, "Input",
 CellTags->"SplittingFunction"],
Cell[1812609, 71848, 170, 4, 24, "Text",
 CellTags->"SplittingFunction"],
Cell[CellGroupData[{
Cell[1812804, 71856, 130, 3, 26, "Input",
 CellTags->"SplittingFunction"],
Cell[1812937, 71861, 573, 20, 46, "Output",
 CellTags->"SplittingFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1813547, 71886, 130, 3, 26, "Input",
 CellTags->"SplittingFunction"],
Cell[1813680, 71891, 342, 11, 29, "Output",
 CellTags->"SplittingFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1814059, 71907, 130, 3, 26, "Input",
 CellTags->"SplittingFunction"],
Cell[1814192, 71912, 313, 10, 45, "Output",
 CellTags->"SplittingFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1814542, 71927, 130, 3, 26, "Input",
 CellTags->"SplittingFunction"],
Cell[1814675, 71932, 901, 30, 46, "Output",
 CellTags->"SplittingFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1815613, 71967, 130, 3, 26, "Input",
 CellTags->"SplittingFunction"],
Cell[1815746, 71972, 1170, 39, 46, "Output",
 CellTags->"SplittingFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1816953, 72016, 130, 3, 26, "Input",
 CellTags->"SplittingFunction"],
Cell[1817086, 72021, 641, 22, 45, "Output",
 CellTags->"SplittingFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1817764, 72048, 130, 3, 26, "Input",
 CellTags->"SplittingFunction"],
Cell[1817897, 72053, 583, 20, 29, "Output",
 CellTags->"SplittingFunction"]
}, Open  ]],
Cell[1818495, 72076, 130, 3, 26, "Input",
 CellTags->"SplittingFunction"],
Cell[1818628, 72081, 63, 1, 24, "Text",
 CellTags->"SplittingFunction"],
Cell[1818694, 72084, 221, 6, 26, "Input",
 CellTags->"SplittingFunction"],
Cell[CellGroupData[{
Cell[1818940, 72094, 130, 3, 26, "Input",
 CellTags->"SplittingFunction"],
Cell[1819073, 72099, 573, 20, 46, "Output",
 CellTags->"SplittingFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1819683, 72124, 130, 3, 26, "Input",
 CellTags->"SplittingFunction"],
Cell[1819816, 72129, 278, 9, 29, "Output",
 CellTags->"SplittingFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1820131, 72143, 130, 3, 26, "Input",
 CellTags->"SplittingFunction"],
Cell[1820264, 72148, 279, 9, 26, "Output",
 CellTags->"SplittingFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1820580, 72162, 130, 3, 26, "Input",
 CellTags->"SplittingFunction"],
Cell[1820713, 72167, 855, 29, 46, "Output",
 CellTags->"SplittingFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1821605, 72201, 130, 3, 26, "Input",
 CellTags->"SplittingFunction"],
Cell[1821738, 72206, 1289, 43, 46, "Output",
 CellTags->"SplittingFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1823064, 72254, 130, 3, 26, "Input",
 CellTags->"SplittingFunction"],
Cell[1823197, 72259, 545, 19, 26, "Output",
 CellTags->"SplittingFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1823779, 72283, 131, 3, 26, "Input",
 CellTags->"SplittingFunction"],
Cell[1823913, 72288, 487, 17, 26, "Output",
 CellTags->"SplittingFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1824437, 72310, 130, 3, 26, "Input",
 CellTags->"SplittingFunction"],
Cell[1824570, 72315, 487, 17, 29, "Output",
 CellTags->"SplittingFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1825094, 72337, 131, 3, 26, "Input",
 CellTags->"SplittingFunction"],
Cell[1825228, 72342, 489, 17, 29, "Output",
 CellTags->"SplittingFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1825754, 72364, 130, 3, 26, "Input",
 CellTags->"SplittingFunction"],
Cell[1825887, 72369, 1404, 47, 46, "Output",
 CellTags->"SplittingFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1827328, 72421, 131, 3, 26, "Input",
 CellTags->"SplittingFunction"],
Cell[1827462, 72426, 1404, 47, 46, "Output",
 CellTags->"SplittingFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1828903, 72478, 131, 3, 26, "Input",
 CellTags->"SplittingFunction"],
Cell[1829037, 72483, 681, 23, 29, "Output",
 CellTags->"SplittingFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1829755, 72511, 132, 3, 26, "Input",
 CellTags->"SplittingFunction"],
Cell[1829890, 72516, 5429, 175, 353, "Output",
 CellTags->"SplittingFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1835356, 72696, 130, 3, 26, "Input",
 CellTags->"SplittingFunction"],
Cell[1835489, 72701, 2948, 93, 98, "Output",
 CellTags->"SplittingFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1838474, 72799, 130, 3, 26, "Input",
 CellTags->"SplittingFunction"],
Cell[1838607, 72804, 3626, 118, 151, "Output",
 CellTags->"SplittingFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1842270, 72927, 130, 3, 26, "Input",
 CellTags->"SplittingFunction"],
Cell[1842403, 72932, 4300, 140, 198, "Output",
 CellTags->"SplittingFunction"]
}, Open  ]]
}, Open  ]],
Cell[1846730, 73076, 79, 1, 39, "ObjectName",
 CellTags->"StandardMatrixElement"],
Cell[CellGroupData[{
Cell[1846834, 73081, 72, 1, 34, "Subsubsection",
 CellTags->"StandardMatrixElement"],
Cell[1846909, 73084, 134, 3, 24, "Text",
 CellTags->"StandardMatrixElement"],
Cell[1847046, 73089, 189, 9, 24, "Text",
 CellTags->"StandardMatrixElement"]
}, Open  ]],
Cell[1847250, 73101, 57, 1, 39, "ObjectName",
 CellTags->"SubContext"],
Cell[CellGroupData[{
Cell[1847332, 73106, 61, 1, 34, "Subsubsection",
 CellTags->"SubContext"],
Cell[1847396, 73109, 120, 3, 24, "Text",
 CellTags->"SubContext"],
Cell[1847519, 73114, 279, 14, 24, "Text",
 CellTags->"SubContext"]
}, Open  ]],
Cell[1847813, 73131, 51, 1, 39, "ObjectName",
 CellTags->"SubLoop"],
Cell[CellGroupData[{
Cell[1847889, 73136, 58, 1, 34, "Subsubsection",
 CellTags->"SubLoop"],
Cell[1847950, 73139, 152, 4, 24, "Text",
 CellTags->"SubLoop"],
Cell[1848105, 73145, 178, 9, 24, "Text",
 CellTags->"SubLoop"]
}, Open  ]],
Cell[1848298, 73157, 45, 1, 39, "ObjectName",
 CellTags->"SumP"],
Cell[CellGroupData[{
Cell[1848368, 73162, 55, 1, 34, "Subsubsection",
 CellTags->"SumP"],
Cell[1848426, 73165, 767, 31, 27, "Text",
 CellTags->"SumP"],
Cell[1849196, 73198, 261, 14, 24, "Text",
 CellTags->"SumP"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1849494, 73217, 52, 1, 34, "Subsubsection",
 CellTags->"SumP"],
Cell[CellGroupData[{
Cell[1849571, 73222, 149, 5, 26, "Input",
 CellTags->"SumP"],
Cell[1849723, 73229, 305, 9, 28, "Output",
 CellTags->"SumP"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1850065, 73243, 149, 5, 26, "Input",
 CellTags->"SumP"],
Cell[1850217, 73250, 307, 9, 28, "Output",
 CellTags->"SumP"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1850561, 73264, 125, 4, 26, "Input",
 CellTags->"SumP"],
Cell[1850689, 73270, 280, 8, 28, "Output",
 CellTags->"SumP"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1851006, 73283, 125, 4, 26, "Input",
 CellTags->"SumP"],
Cell[1851134, 73289, 168, 5, 44, "Output",
 CellTags->"SumP"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1851339, 73299, 301, 12, 56, "Input",
 CellTags->"SumP"],
Cell[1851643, 73313, 166, 5, 44, "Output",
 CellTags->"SumP"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1851846, 73323, 186, 6, 26, "Input",
 CellTags->"SumP"],
Cell[1852035, 73331, 806, 30, 46, "Output",
 CellTags->"SumP"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1852878, 73366, 124, 4, 26, "Input",
 CellTags->"SumP"],
Cell[1853005, 73372, 166, 5, 44, "Output",
 CellTags->"SumP"]
}, Open  ]]
}, Open  ]],
Cell[1853198, 73381, 45, 1, 39, "ObjectName",
 CellTags->"SumS"],
Cell[CellGroupData[{
Cell[1853268, 73386, 55, 1, 34, "Subsubsection",
 CellTags->"SumS"],
Cell[1853326, 73389, 1438, 57, 44, "Text",
 CellTags->"SumS"],
Cell[CellGroupData[{
Cell[1854789, 73450, 108, 3, 26, "Input",
 CellTags->"SumS"],
Cell[1854900, 73455, 207, 7, 26, "Output",
 CellTags->"SumS"]
}, Open  ]],
Cell[1855122, 73465, 261, 14, 24, "Text",
 CellTags->"SumS"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1855420, 73484, 52, 1, 34, "Subsubsection",
 CellTags->"SumS"],
Cell[CellGroupData[{
Cell[1855497, 73489, 149, 5, 26, "Input",
 CellTags->"SumS"],
Cell[1855649, 73496, 253, 9, 26, "Output",
 CellTags->"SumS"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1855939, 73510, 149, 5, 26, "Input",
 CellTags->"SumS"],
Cell[1856091, 73517, 253, 9, 26, "Output",
 CellTags->"SumS"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1856381, 73531, 144, 5, 26, "Input",
 CellTags->"SumS"],
Cell[1856528, 73538, 216, 7, 26, "Output",
 CellTags->"SumS"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1856781, 73550, 173, 5, 26, "Input",
 CellTags->"SumS"],
Cell[1856957, 73557, 305, 11, 46, "Output",
 CellTags->"SumS"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1857299, 73573, 197, 6, 26, "Input",
 CellTags->"SumS"],
Cell[1857499, 73581, 384, 14, 49, "Output",
 CellTags->"SumS"]
}, Open  ]],
Cell[1857898, 73598, 192, 6, 26, "Input",
 CellTags->"SumS"],
Cell[CellGroupData[{
Cell[1858115, 73608, 149, 5, 26, "Input",
 CellTags->"SumS"],
Cell[1858267, 73615, 551, 20, 49, "Output",
 CellTags->"SumS"]
}, Open  ]],
Cell[1858833, 73638, 193, 6, 26, "Input",
 CellTags->"SumS"],
Cell[CellGroupData[{
Cell[1859051, 73648, 125, 4, 26, "Input",
 CellTags->"SumS"],
Cell[1859179, 73654, 166, 5, 44, "Output",
 CellTags->"SumS"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1859382, 73664, 174, 6, 56, "Input",
 CellTags->"SumS"],
Cell[1859559, 73672, 165, 5, 44, "Output",
 CellTags->"SumS"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1859761, 73682, 159, 5, 26, "Input",
 CellTags->"SumS"],
Cell[1859923, 73689, 254, 9, 26, "Output",
 CellTags->"SumS"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1860214, 73703, 146, 4, 26, "Input",
 CellTags->"SumS"],
Cell[1860363, 73709, 191, 6, 45, "Output",
 CellTags->"SumS"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1860591, 73720, 144, 5, 26, "Input",
 CellTags->"SumS"],
Cell[1860738, 73727, 186, 6, 43, "Output",
 CellTags->"SumS"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1860961, 73738, 125, 4, 26, "Input",
 CellTags->"SumS"],
Cell[1861089, 73744, 184, 6, 43, "Output",
 CellTags->"SumS"]
}, Open  ]]
}, Open  ]],
Cell[1861300, 73754, 45, 1, 41, "ObjectName",
 CellTags->"SumT"],
Cell[CellGroupData[{
Cell[1861370, 73759, 55, 1, 36, "Subsubsection",
 CellTags->"SumT"],
Cell[1861428, 73762, 461, 15, 43, "Text",
 CellTags->"SumT"],
Cell[1861892, 73779, 261, 14, 25, "Text",
 CellTags->"SumT"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1862190, 73798, 52, 1, 34, "Subsubsection",
 CellTags->"SumT"],
Cell[CellGroupData[{
Cell[1862267, 73803, 149, 5, 26, "Input",
 CellTags->"SumT"],
Cell[1862419, 73810, 278, 10, 29, "Output",
 CellTags->"SumT"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1862734, 73825, 149, 5, 26, "Input",
 CellTags->"SumT"],
Cell[1862886, 73832, 278, 10, 29, "Output",
 CellTags->"SumT"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1863201, 73847, 125, 4, 26, "Input",
 CellTags->"SumT"],
Cell[1863329, 73853, 222, 8, 29, "Output",
 CellTags->"SumT"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1863588, 73866, 173, 5, 26, "Input",
 CellTags->"SumT"],
Cell[1863764, 73873, 403, 15, 46, "Output",
 CellTags->"SumT"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1864204, 73893, 125, 4, 26, "Input",
 CellTags->"SumT"],
Cell[1864332, 73899, 183, 6, 43, "Output",
 CellTags->"SumT"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1864552, 73910, 244, 9, 56, "Input",
 CellTags->"SumT"],
Cell[1864799, 73921, 184, 6, 43, "Output",
 CellTags->"SumT"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1865020, 73932, 159, 5, 26, "Input",
 CellTags->"SumT"],
Cell[1865182, 73939, 279, 10, 29, "Output",
 CellTags->"SumT"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1865498, 73954, 136, 4, 26, "Input",
 CellTags->"SumT"],
Cell[1865637, 73960, 294, 8, 45, "Output",
 CellTags->"SumT"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1865968, 73973, 125, 4, 26, "Input",
 CellTags->"SumT"],
Cell[1866096, 73979, 184, 6, 43, "Output",
 CellTags->"SumT"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1866317, 73990, 144, 5, 26, "Input",
 CellTags->"SumT"],
Cell[1866464, 73997, 184, 6, 43, "Output",
 CellTags->"SumT"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1866685, 74008, 268, 10, 56, "Input",
 CellTags->"SumT"],
Cell[1866956, 74020, 378, 13, 26, "Output",
 CellTags->"SumT"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1867371, 74038, 136, 4, 26, "Input",
 CellTags->"SumT"],
Cell[1867510, 74044, 212, 7, 44, "Output",
 CellTags->"SumT"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1867759, 74056, 155, 5, 26, "Input",
 CellTags->"SumT"],
Cell[1867917, 74063, 294, 8, 45, "Output",
 CellTags->"SumT"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1868248, 74076, 129, 4, 26, "Input",
 CellTags->"SumT"],
Cell[1868380, 74082, 502, 17, 44, "Output",
 CellTags->"SumT"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1868919, 74104, 240, 9, 26, "Input",
 CellTags->"SumT"],
Cell[1869162, 74115, 500, 17, 44, "Output",
 CellTags->"SumT"]
}, Open  ]]
}, Open  ]],
Cell[1869689, 74136, 45, 1, 39, "ObjectName",
 CellTags->"SUND"],
Cell[CellGroupData[{
Cell[1869759, 74141, 55, 1, 34, "Subsubsection",
 CellTags->"SUND"],
Cell[1869817, 74144, 224, 10, 26, "Text",
 CellTags->"SUND"],
Cell[1870044, 74156, 392, 19, 24, "Text",
 CellTags->"SUND"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1870473, 74180, 52, 1, 34, "Subsubsection",
 CellTags->"SUND"],
Cell[CellGroupData[{
Cell[1870550, 74185, 135, 4, 26, "Input",
 CellTags->"SUND"],
Cell[1870688, 74191, 191, 6, 28, "Output",
 CellTags->"SUND"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1870916, 74202, 211, 6, 26, "Input",
 CellTags->"SUND"],
Cell[1871130, 74210, 1128, 45, 28, "Output",
 CellTags->"SUND"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1872295, 74260, 135, 4, 26, "Input",
 CellTags->"SUND"],
Cell[1872433, 74266, 189, 6, 28, "Output",
 CellTags->"SUND"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1872659, 74277, 135, 4, 26, "Input",
 CellTags->"SUND"],
Cell[1872797, 74283, 191, 6, 28, "Output",
 CellTags->"SUND"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1873025, 74294, 266, 8, 26, "Input",
 CellTags->"SUND"],
Cell[1873294, 74304, 302, 10, 29, "Output",
 CellTags->"SUND"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1873633, 74319, 377, 12, 26, "Input",
 CellTags->"SUND"],
Cell[1874013, 74333, 346, 12, 48, "Output",
 CellTags->"SUND"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1874396, 74350, 377, 12, 26, "Input",
 CellTags->"SUND"],
Cell[1874776, 74364, 516, 20, 48, "Output",
 CellTags->"SUND"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1875329, 74389, 173, 6, 26, "Input",
 CellTags->"SUND"],
Cell[1875505, 74397, 192, 5, 38, "Output",
 CellTags->"SUND"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1875734, 74407, 202, 7, 26, "Input",
 CellTags->"SUND"],
Cell[1875939, 74416, 300, 8, 38, "Output",
 CellTags->"SUND"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1876276, 74429, 231, 8, 26, "Input",
 CellTags->"SUND"],
Cell[1876510, 74439, 194, 5, 38, "Output",
 CellTags->"SUND"]
}, Open  ]],
Cell[1876719, 74447, 104, 3, 26, "Input",
 CellTags->"SUND"]
}, Open  ]],
Cell[1876838, 74453, 53, 1, 39, "ObjectName",
 CellTags->"SUNDelta"],
Cell[CellGroupData[{
Cell[1876916, 74458, 59, 1, 34, "Subsubsection",
 CellTags->"SUNDelta"],
Cell[1876978, 74461, 172, 6, 24, "Text",
 CellTags->"SUNDelta"],
Cell[1877153, 74469, 717, 33, 24, "Text",
 CellTags->"SUNDelta"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1877907, 74507, 56, 1, 34, "Subsubsection",
 CellTags->"SUNDelta"],
Cell[CellGroupData[{
Cell[1877988, 74512, 133, 4, 26, "Input",
 CellTags->"SUNDelta"],
Cell[1878124, 74518, 195, 6, 28, "Output",
 CellTags->"SUNDelta"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1878356, 74529, 262, 8, 26, "Input",
 CellTags->"SUNDelta"],
Cell[1878621, 74539, 343, 14, 26, "Output",
 CellTags->"SUNDelta"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1879001, 74558, 205, 6, 26, "Input",
 CellTags->"SUNDelta"],
Cell[1879209, 74566, 342, 14, 28, "Output",
 CellTags->"SUNDelta"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1879588, 74585, 169, 5, 26, "Input",
 CellTags->"SUNDelta"],
Cell[1879760, 74592, 190, 5, 38, "Output",
 CellTags->"SUNDelta"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1879987, 74602, 200, 7, 26, "Input",
 CellTags->"SUNDelta"],
Cell[1880190, 74611, 261, 7, 38, "Output",
 CellTags->"SUNDelta"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1880488, 74623, 229, 8, 26, "Input",
 CellTags->"SUNDelta"],
Cell[1880720, 74633, 184, 5, 38, "Output",
 CellTags->"SUNDelta"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1880941, 74643, 194, 7, 26, "Input",
 CellTags->"SUNDelta"],
Cell[1881138, 74652, 262, 7, 38, "Output",
 CellTags->"SUNDelta"]
}, Open  ]],
Cell[1881415, 74662, 256, 5, 41, "Text",
 CellTags->"SUNDelta"],
Cell[CellGroupData[{
Cell[1881696, 74671, 315, 10, 26, "Input",
 CellTags->"SUNDelta"],
Cell[1882014, 74683, 502, 21, 28, "Output",
 CellTags->"SUNDelta"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1882553, 74709, 110, 3, 26, "Input",
 CellTags->"SUNDelta"],
Cell[1882666, 74714, 437, 12, 38, "Output",
 CellTags->"SUNDelta"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1883140, 74731, 200, 7, 26, "Input",
 CellTags->"SUNDelta"],
Cell[1883343, 74740, 280, 7, 38, "Output",
 CellTags->"SUNDelta"]
}, Open  ]]
}, Open  ]],
Cell[1883650, 74751, 69, 1, 39, "ObjectName",
 CellTags->"SUNDeltaContract"],
Cell[CellGroupData[{
Cell[1883744, 74756, 67, 1, 34, "Subsubsection",
 CellTags->"SUNDeltaContract"],
Cell[1883814, 74759, 397, 9, 58, "Text",
 CellTags->"SUNDeltaContract"],
Cell[1884214, 74770, 297, 14, 24, "Text",
 CellTags->"SUNDeltaContract"]
}, Open  ]],
Cell[1884526, 74787, 45, 1, 39, "ObjectName",
 CellTags->"SUNF"],
Cell[CellGroupData[{
Cell[1884596, 74792, 55, 1, 34, "Subsubsection",
 CellTags->"SUNF"],
Cell[1884654, 74795, 190, 6, 24, "Text",
 CellTags->"SUNF"],
Cell[CellGroupData[{
Cell[1884869, 74805, 108, 3, 26, "Input",
 CellTags->"SUNF"],
Cell[1884980, 74810, 210, 7, 26, "Output",
 CellTags->"SUNF"]
}, Open  ]],
Cell[1885205, 74820, 701, 34, 24, "Text",
 CellTags->"SUNF"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1885943, 74859, 52, 1, 34, "Subsubsection",
 CellTags->"SUNF"],
Cell[CellGroupData[{
Cell[1886020, 74864, 294, 10, 26, "Input",
 CellTags->"SUNF"],
Cell[1886317, 74876, 314, 11, 28, "Output",
 CellTags->"SUNF"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1886668, 74892, 103, 3, 26, "Input",
 CellTags->"SUNF"],
Cell[1886774, 74897, 805, 35, 28, "Output",
 CellTags->"SUNF"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1887616, 74937, 110, 3, 26, "Input",
 CellTags->"SUNF"],
Cell[1887729, 74942, 497, 21, 28, "Output",
 CellTags->"SUNF"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1888263, 74968, 135, 4, 26, "Input",
 CellTags->"SUNF"],
Cell[1888401, 74974, 189, 6, 28, "Output",
 CellTags->"SUNF"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1888627, 74985, 163, 5, 26, "Input",
 CellTags->"SUNF"],
Cell[1888793, 74992, 143, 4, 26, "Output",
 CellTags->"SUNF"]
}, Open  ]],
Cell[1888951, 74999, 319, 10, 24, "Text",
 CellTags->"SUNF"],
Cell[CellGroupData[{
Cell[1889295, 75013, 185, 5, 26, "Input",
 CellTags->"SUNF"],
Cell[1889483, 75020, 1189, 45, 28, "Output",
 CellTags->"SUNF"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1890709, 75070, 266, 8, 26, "Input",
 CellTags->"SUNF"],
Cell[1890978, 75080, 395, 16, 28, "Output",
 CellTags->"SUNF"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1891410, 75101, 239, 7, 26, "Input",
 CellTags->"SUNF"],
Cell[1891652, 75110, 1208, 46, 28, "Output",
 CellTags->"SUNF"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1892897, 75161, 173, 6, 26, "Input",
 CellTags->"SUNF"],
Cell[1893073, 75169, 194, 5, 38, "Output",
 CellTags->"SUNF"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1893304, 75179, 202, 7, 26, "Input",
 CellTags->"SUNF"],
Cell[1893509, 75188, 300, 8, 38, "Output",
 CellTags->"SUNF"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1893846, 75201, 231, 8, 26, "Input",
 CellTags->"SUNF"],
Cell[1894080, 75211, 192, 5, 38, "Output",
 CellTags->"SUNF"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1894309, 75221, 135, 4, 26, "Input",
 CellTags->"SUNF"],
Cell[1894447, 75227, 191, 6, 28, "Output",
 CellTags->"SUNF"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1894675, 75238, 162, 5, 26, "Input",
 CellTags->"SUNF"],
Cell[1894840, 75245, 443, 19, 28, "Output",
 CellTags->"SUNF"]
}, Open  ]]
}, Open  ]],
Cell[1895310, 75268, 57, 1, 39, "ObjectName",
 CellTags->"SUNFJacobi"],
Cell[CellGroupData[{
Cell[1895392, 75273, 61, 1, 34, "Subsubsection",
 CellTags->"SUNFJacobi"],
Cell[1895456, 75276, 146, 4, 24, "Text",
 CellTags->"SUNFJacobi"],
Cell[1895605, 75282, 283, 13, 24, "Text",
 CellTags->"SUNFJacobi"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1895925, 75300, 58, 1, 34, "Subsubsection",
 CellTags->"SUNFJacobi"],
Cell[CellGroupData[{
Cell[1896008, 75305, 380, 12, 26, "Input",
 CellTags->"SUNFJacobi"],
Cell[1896391, 75319, 736, 33, 29, "Output",
 CellTags->"SUNFJacobi"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1897164, 75357, 379, 12, 26, "Input",
 CellTags->"SUNFJacobi"],
Cell[1897546, 75371, 1404, 63, 30, "Output",
 CellTags->"SUNFJacobi"]
}, Open  ]]
}, Open  ]],
Cell[1898977, 75438, 53, 1, 39, "ObjectName",
 CellTags->"SUNIndex"],
Cell[CellGroupData[{
Cell[1899055, 75443, 59, 1, 34, "Subsubsection",
 CellTags->"SUNIndex"],
Cell[1899117, 75446, 135, 6, 24, "Text",
 CellTags->"SUNIndex"],
Cell[1899255, 75454, 405, 18, 24, "Text",
 CellTags->"SUNIndex"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1899697, 75477, 56, 1, 34, "Subsubsection",
 CellTags->"SUNIndex"],
Cell[CellGroupData[{
Cell[1899778, 75482, 110, 3, 26, "Input",
 CellTags->"SUNIndex"],
Cell[1899891, 75487, 147, 4, 27, "Output",
 CellTags->"SUNIndex"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1900075, 75496, 110, 3, 26, "Input",
 CellTags->"SUNIndex"],
Cell[1900188, 75501, 169, 4, 38, "Output",
 CellTags->"SUNIndex"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1900394, 75510, 200, 7, 26, "Input",
 CellTags->"SUNIndex"],
Cell[1900597, 75519, 262, 7, 38, "Output",
 CellTags->"SUNIndex"]
}, Open  ]]
}, Open  ]],
Cell[1900886, 75530, 65, 1, 39, "ObjectName",
 CellTags->"SUNIndexRename"],
Cell[CellGroupData[{
Cell[1900976, 75535, 65, 1, 34, "Subsubsection",
 CellTags->"SUNIndexRename"],
Cell[1901044, 75538, 163, 4, 24, "Text",
 CellTags->"SUNIndexRename"],
Cell[1901210, 75544, 304, 14, 24, "Text",
 CellTags->"SUNIndexRename"]
}, Open  ]],
Cell[1901529, 75561, 45, 1, 39, "ObjectName",
 CellTags->"SUNN"],
Cell[CellGroupData[{
Cell[1901599, 75566, 55, 1, 34, "Subsubsection",
 CellTags->"SUNN"],
Cell[1901657, 75569, 203, 7, 24, "Text",
 CellTags->"SUNN"],
Cell[1901863, 75578, 579, 29, 24, "Text",
 CellTags->"SUNN"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1902479, 75612, 52, 1, 34, "Subsubsection",
 CellTags->"SUNN"],
Cell[CellGroupData[{
Cell[1902556, 75617, 317, 9, 26, "Input",
 CellTags->"SUNN"],
Cell[1902876, 75628, 191, 6, 26, "Output",
 CellTags->"SUNN"]
}, Open  ]]
}, Open  ]],
Cell[1903094, 75638, 57, 1, 39, "ObjectName",
 CellTags->"SUNNToCACF"],
Cell[CellGroupData[{
Cell[1903176, 75643, 61, 1, 34, "Subsubsection",
 CellTags->"SUNNToCACF"],
Cell[1903240, 75646, 187, 4, 41, "Text",
 CellTags->"SUNNToCACF"],
Cell[1903430, 75652, 573, 29, 24, "Text",
 CellTags->"SUNNToCACF"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1904040, 75686, 58, 1, 34, "Subsubsection",
 CellTags->"SUNNToCACF"],
Cell[CellGroupData[{
Cell[1904123, 75691, 322, 9, 26, "Input",
 CellTags->"SUNNToCACF"],
Cell[1904448, 75702, 228, 7, 26, "Output",
 CellTags->"SUNNToCACF"]
}, Open  ]]
}, Open  ]],
Cell[1904703, 75713, 59, 1, 39, "ObjectName",
 CellTags->"SUNSimplify"],
Cell[CellGroupData[{
Cell[1904787, 75718, 62, 1, 34, "Subsubsection",
 CellTags->"SUNSimplify"],
Cell[1904852, 75721, 376, 7, 58, "Text",
 CellTags->"SUNSimplify"],
Cell[CellGroupData[{
Cell[1905253, 75732, 122, 3, 26, "Input",
 CellTags->"SUNSimplify"],
Cell[1905378, 75737, 554, 14, 47, "Output",
 CellTags->"SUNSimplify"]
}, Open  ]],
Cell[1905947, 75754, 299, 13, 24, "Text",
 CellTags->"SUNSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1906283, 75772, 59, 1, 34, "Subsubsection",
 CellTags->"SUNSimplify"],
Cell[CellGroupData[{
Cell[1906367, 75777, 247, 8, 26, "Input",
 CellTags->"SUNSimplify"],
Cell[1906617, 75787, 273, 9, 28, "Output",
 CellTags->"SUNSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1906927, 75801, 117, 3, 26, "Input",
 CellTags->"SUNSimplify"],
Cell[1907047, 75806, 348, 14, 26, "Output",
 CellTags->"SUNSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1907432, 75825, 189, 6, 26, "Input",
 CellTags->"SUNSimplify"],
Cell[1907624, 75833, 291, 11, 26, "Output",
 CellTags->"SUNSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1907952, 75849, 117, 3, 26, "Input",
 CellTags->"SUNSimplify"],
Cell[1908072, 75854, 171, 5, 26, "Output",
 CellTags->"SUNSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1908280, 75864, 183, 5, 26, "Input",
 CellTags->"SUNSimplify"],
Cell[1908466, 75871, 247, 8, 46, "Output",
 CellTags->"SUNSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1908750, 75884, 254, 8, 26, "Input",
 CellTags->"SUNSimplify"],
Cell[1909007, 75894, 268, 9, 28, "Output",
 CellTags->"SUNSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1909312, 75908, 117, 3, 26, "Input",
 CellTags->"SUNSimplify"],
Cell[1909432, 75913, 402, 16, 28, "Output",
 CellTags->"SUNSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1909871, 75934, 269, 8, 26, "Input",
 CellTags->"SUNSimplify"],
Cell[1910143, 75944, 269, 9, 28, "Output",
 CellTags->"SUNSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1910449, 75958, 117, 3, 26, "Input",
 CellTags->"SUNSimplify"],
Cell[1910569, 75963, 239, 7, 28, "Output",
 CellTags->"SUNSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1910845, 75975, 259, 8, 26, "Input",
 CellTags->"SUNSimplify"],
Cell[1911107, 75985, 269, 9, 28, "Output",
 CellTags->"SUNSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1911413, 75999, 117, 3, 26, "Input",
 CellTags->"SUNSimplify"],
Cell[1911533, 76004, 402, 16, 28, "Output",
 CellTags->"SUNSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1911972, 76025, 259, 8, 26, "Input",
 CellTags->"SUNSimplify"],
Cell[1912234, 76035, 269, 9, 28, "Output",
 CellTags->"SUNSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1912540, 76049, 180, 5, 26, "Input",
 CellTags->"SUNSimplify"],
Cell[1912723, 76056, 149, 4, 26, "Output",
 CellTags->"SUNSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1912909, 76065, 384, 12, 26, "Input",
 CellTags->"SUNSimplify"],
Cell[1913296, 76079, 353, 12, 48, "Output",
 CellTags->"SUNSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1913686, 76096, 389, 12, 26, "Input",
 CellTags->"SUNSimplify"],
Cell[1914078, 76110, 492, 19, 48, "Output",
 CellTags->"SUNSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1914607, 76134, 246, 7, 26, "Input",
 CellTags->"SUNSimplify"],
Cell[1914856, 76143, 1215, 46, 28, "Output",
 CellTags->"SUNSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1916108, 76194, 246, 7, 26, "Input",
 CellTags->"SUNSimplify"],
Cell[1916357, 76203, 1173, 45, 28, "Output",
 CellTags->"SUNSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1917567, 76253, 273, 8, 26, "Input",
 CellTags->"SUNSimplify"],
Cell[1917843, 76263, 292, 9, 44, "Output",
 CellTags->"SUNSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1918172, 76277, 439, 14, 26, "Input",
 CellTags->"SUNSimplify"],
Cell[1918614, 76293, 421, 15, 28, "Output",
 CellTags->"SUNSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1919072, 76313, 181, 5, 26, "Input",
 CellTags->"SUNSimplify"],
Cell[1919256, 76320, 747, 33, 28, "Output",
 CellTags->"SUNSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1920040, 76358, 238, 6, 26, "Input",
 CellTags->"SUNSimplify"],
Cell[1920281, 76366, 1404, 63, 28, "Output",
 CellTags->"SUNSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1921722, 76434, 200, 6, 26, "Input",
 CellTags->"SUNSimplify"],
Cell[1921925, 76442, 206, 6, 28, "Output",
 CellTags->"SUNSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1922168, 76453, 341, 10, 26, "Input",
 CellTags->"SUNSimplify"],
Cell[1922512, 76465, 585, 22, 26, "Output",
 CellTags->"SUNSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1923134, 76492, 181, 5, 26, "Input",
 CellTags->"SUNSimplify"],
Cell[1923318, 76499, 585, 22, 26, "Output",
 CellTags->"SUNSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1923940, 76526, 180, 5, 26, "Input",
 CellTags->"SUNSimplify"],
Cell[1924123, 76533, 613, 23, 26, "Output",
 CellTags->"SUNSimplify"]
}, Open  ]],
Cell[1924751, 76559, 208, 6, 26, "Input",
 CellTags->"SUNSimplify"]
}, Open  ]],
Cell[1924974, 76568, 45, 1, 39, "ObjectName",
 CellTags->"SUNT"],
Cell[CellGroupData[{
Cell[1925044, 76573, 55, 1, 34, "Subsubsection",
 CellTags->"SUNT"],
Cell[1925102, 76576, 237, 10, 24, "Text",
 CellTags->"SUNT"],
Cell[1925342, 76588, 674, 34, 24, "Text",
 CellTags->"SUNT"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1926053, 76627, 52, 1, 34, "Subsubsection",
 CellTags->"SUNT"],
Cell[CellGroupData[{
Cell[1926130, 76632, 102, 3, 26, "Input",
 CellTags->"SUNT"],
Cell[1926235, 76637, 199, 7, 26, "Output",
 CellTags->"SUNT"]
}, Open  ]],
Cell[1926449, 76647, 203, 7, 24, "Text",
 CellTags->"SUNT"],
Cell[CellGroupData[{
Cell[1926677, 76658, 207, 6, 26, "Input",
 CellTags->"SUNT"],
Cell[1926887, 76666, 355, 14, 28, "Output",
 CellTags->"SUNT"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1927279, 76685, 145, 4, 26, "Input",
 CellTags->"SUNT"],
Cell[1927427, 76691, 407, 17, 28, "Output",
 CellTags->"SUNT"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1927871, 76713, 242, 7, 26, "Input",
 CellTags->"SUNT"],
Cell[1928116, 76722, 312, 12, 45, "Output",
 CellTags->"SUNT"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1928465, 76739, 184, 5, 26, "Input",
 CellTags->"SUNT"],
Cell[1928652, 76746, 175, 5, 28, "Output",
 CellTags->"SUNT"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1928864, 76756, 174, 5, 26, "Input",
 CellTags->"SUNT"],
Cell[1929041, 76763, 441, 17, 44, "Output",
 CellTags->"SUNT"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1929519, 76785, 242, 7, 26, "Input",
 CellTags->"SUNT"],
Cell[1929764, 76794, 310, 12, 45, "Output",
 CellTags->"SUNT"]
}, Open  ]],
Cell[1930089, 76809, 422, 15, 26, "Text",
 CellTags->"SUNT"],
Cell[CellGroupData[{
Cell[1930536, 76828, 161, 5, 26, "Input",
 CellTags->"SUNT"],
Cell[1930700, 76835, 370, 15, 44, "Output",
 CellTags->"SUNT"]
}, Open  ]],
Cell[1931085, 76853, 197, 7, 27, "Text",
 CellTags->"SUNT"],
Cell[CellGroupData[{
Cell[1931307, 76864, 197, 6, 26, "Input",
 CellTags->"SUNT"],
Cell[1931507, 76872, 395, 16, 29, "Output",
 CellTags->"SUNT"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1931939, 76893, 198, 6, 26, "Input",
 CellTags->"SUNT"],
Cell[1932140, 76901, 304, 9, 54, "Output",
 CellTags->"SUNT"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1932481, 76915, 167, 6, 26, "Input",
 CellTags->"SUNT"],
Cell[1932651, 76923, 194, 5, 38, "Output",
 CellTags->"SUNT"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1932882, 76933, 195, 7, 26, "Input",
 CellTags->"SUNT"],
Cell[1933080, 76942, 159, 4, 38, "Output",
 CellTags->"SUNT"]
}, Open  ]]
}, Open  ]],
Cell[1933266, 76950, 53, 1, 39, "ObjectName",
 CellTags->"SUNTrace"],
Cell[CellGroupData[{
Cell[1933344, 76955, 59, 1, 34, "Subsubsection",
 CellTags->"SUNTrace"],
Cell[1933406, 76958, 81, 1, 24, "Text",
 CellTags->"SUNTrace"],
Cell[CellGroupData[{
Cell[1933512, 76963, 116, 3, 26, "Input",
 CellTags->"SUNTrace"],
Cell[1933631, 76968, 214, 7, 26, "Output",
 CellTags->"SUNTrace"]
}, Open  ]],
Cell[1933860, 76978, 280, 14, 24, "Text",
 CellTags->"SUNTrace"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1934177, 76997, 56, 1, 34, "Subsubsection",
 CellTags->"SUNTrace"],
Cell[CellGroupData[{
Cell[1934258, 77002, 165, 5, 26, "Input",
 CellTags->"SUNTrace"],
Cell[1934426, 77009, 374, 15, 44, "Output",
 CellTags->"SUNTrace"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1934837, 77029, 175, 5, 26, "Input",
 CellTags->"SUNTrace"],
Cell[1935015, 77036, 566, 23, 28, "Output",
 CellTags->"SUNTrace"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1935618, 77064, 240, 7, 26, "Input",
 CellTags->"SUNTrace"],
Cell[1935861, 77073, 853, 36, 46, "Output",
 CellTags->"SUNTrace"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1936751, 77114, 185, 5, 26, "Input",
 CellTags->"SUNTrace"],
Cell[1936939, 77121, 683, 28, 28, "Output",
 CellTags->"SUNTrace"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1937659, 77154, 278, 8, 26, "Input",
 CellTags->"SUNTrace"],
Cell[1937940, 77164, 4312, 185, 84, "Output",
 CellTags->"SUNTrace"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1942289, 77354, 177, 5, 26, "Input",
 CellTags->"SUNTrace"],
Cell[1942469, 77361, 683, 28, 28, "Output",
 CellTags->"SUNTrace"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1943189, 77394, 288, 8, 26, "Input",
 CellTags->"SUNTrace"],
Cell[1943480, 77404, 11396, 437, 180, "Output",
 CellTags->"SUNTrace"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1954913, 77846, 177, 5, 26, "Input",
 CellTags->"SUNTrace"],
Cell[1955093, 77853, 802, 33, 28, "Output",
 CellTags->"SUNTrace"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1955932, 77891, 270, 8, 26, "Input",
 CellTags->"SUNTrace"],
Cell[1956205, 77901, 147, 4, 26, "Output",
 CellTags->"SUNTrace"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1956389, 77910, 270, 8, 26, "Input",
 CellTags->"SUNTrace"],
Cell[1956662, 77920, 147, 4, 26, "Output",
 CellTags->"SUNTrace"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1956846, 77929, 270, 8, 26, "Input",
 CellTags->"SUNTrace"],
Cell[1957119, 77939, 147, 4, 26, "Output",
 CellTags->"SUNTrace"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1957303, 77948, 270, 8, 26, "Input",
 CellTags->"SUNTrace"],
Cell[1957576, 77958, 618, 24, 29, "Output",
 CellTags->"SUNTrace"]
}, Open  ]]
}, Open  ]],
Cell[1958221, 77986, 61, 1, 39, "ObjectName",
 CellTags->"SymbolicSum2"],
Cell[CellGroupData[{
Cell[1958307, 77991, 63, 1, 34, "Subsubsection",
 CellTags->"SymbolicSum2"],
Cell[1958373, 77994, 284, 6, 41, "Text",
 CellTags->"SymbolicSum2"]
}, Open  ]],
Cell[1958672, 78003, 61, 1, 39, "ObjectName",
 CellTags->"SymbolicSum3"],
Cell[CellGroupData[{
Cell[1958758, 78008, 63, 1, 34, "Subsubsection",
 CellTags->"SymbolicSum3"],
Cell[1958824, 78011, 292, 6, 41, "Text",
 CellTags->"SymbolicSum3"]
}, Open  ]],
Cell[1959131, 78020, 57, 1, 41, "ObjectName",
 CellTags->"Symmetrize"],
Cell[CellGroupData[{
Cell[1959213, 78025, 61, 1, 34, "Subsubsection",
 CellTags->"Symmetrize"],
Cell[1959277, 78028, 145, 4, 24, "Text",
 CellTags->"Symmetrize"],
Cell[1959425, 78034, 193, 8, 24, "Text",
 CellTags->"Symmetrize"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1959655, 78047, 58, 1, 34, "Subsubsection",
 CellTags->"Symmetrize"],
Cell[CellGroupData[{
Cell[1959738, 78052, 240, 8, 26, "Input",
 CellTags->"Symmetrize"],
Cell[1959981, 78062, 359, 12, 44, "Output",
 CellTags->"Symmetrize"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1960377, 78079, 260, 8, 26, "Input",
 CellTags->"Symmetrize"],
Cell[1960640, 78089, 694, 21, 44, "Output",
 CellTags->"Symmetrize"]
}, Open  ]]
}, Open  ]],
Cell[1961361, 78114, 61, 1, 39, "ObjectName",
 CellTags->{"TBox", "Tbox"}],
Cell[CellGroupData[{
Cell[1961447, 78119, 65, 1, 34, "Subsubsection",
 CellTags->{"TBox", "Tbox"}],
Cell[1961515, 78122, 262, 5, 41, "Text",
 CellTags->{"TBox", "Tbox"}]
}, Open  ]],
Cell[1961792, 78130, 45, 1, 39, "ObjectName",
 CellTags->"Tdec"],
Cell[CellGroupData[{
Cell[1961862, 78135, 55, 1, 34, "Subsubsection",
 CellTags->"Tdec"],
Cell[1961920, 78138, 256, 5, 41, "Text",
 CellTags->"Tdec"],
Cell[CellGroupData[{
Cell[1962201, 78147, 108, 3, 26, "Input",
 CellTags->"Tdec"],
Cell[1962312, 78152, 454, 12, 26, "Output",
 CellTags->"Tdec"]
}, Open  ]],
Cell[1962781, 78167, 389, 19, 24, "Text",
 CellTags->"Tdec"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1963207, 78191, 52, 1, 34, "Subsubsection",
 CellTags->"Tdec"],
Cell[1963262, 78194, 608, 22, 36, "Text",
 CellTags->"Tdec"],
Cell[CellGroupData[{
Cell[1963895, 78220, 202, 7, 26, "Input",
 CellTags->"Tdec"],
Cell[1964100, 78229, 597, 22, 44, "Output",
 CellTags->"Tdec"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1964734, 78256, 199, 7, 26, "Input",
 CellTags->"Tdec"],
Cell[1964936, 78265, 398, 15, 44, "Output",
 CellTags->"Tdec"]
}, Open  ]],
Cell[1965349, 78283, 776, 28, 28, "Text",
 CellTags->"Tdec"],
Cell[1966128, 78313, 529, 18, 26, "Input",
 CellTags->"Tdec"],
Cell[CellGroupData[{
Cell[1966682, 78335, 199, 7, 26, "Input",
 CellTags->"Tdec"],
Cell[1966884, 78344, 7370, 268, 126, "Output",
 CellTags->"Tdec"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1974291, 78617, 380, 13, 26, "Input",
 CellTags->"Tdec"],
Cell[1974674, 78632, 5008, 182, 97, "Output",
 CellTags->"Tdec"]
}, Open  ]],
Cell[1979697, 78817, 121, 4, 26, "Input",
 CellTags->"Tdec"]
}, Open  ]],
Cell[1979833, 78824, 65, 1, 39, "ObjectName",
 CellTags->"TensorFunction"],
Cell[CellGroupData[{
Cell[1979923, 78829, 65, 1, 34, "Subsubsection",
 CellTags->"TensorFunction"],
Cell[1979991, 78832, 360, 7, 58, "Text",
 CellTags->"TensorFunction"],
Cell[1980354, 78841, 191, 9, 24, "Text",
 CellTags->"TensorFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1980582, 78855, 62, 1, 34, "Subsubsection",
 CellTags->"TensorFunction"],
Cell[CellGroupData[{
Cell[1980669, 78860, 178, 4, 26, "Input",
 CellTags->"TensorFunction"],
Cell[1980850, 78866, 253, 7, 26, "Output",
 CellTags->"TensorFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1981140, 78878, 116, 3, 26, "Input",
 CellTags->"TensorFunction"],
Cell[1981259, 78883, 235, 5, 38, "Output",
 CellTags->"TensorFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1981531, 78893, 198, 6, 26, "Input",
 CellTags->"TensorFunction"],
Cell[1981732, 78901, 417, 15, 28, "Output",
 CellTags->"TensorFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1982186, 78921, 116, 3, 26, "Input",
 CellTags->"TensorFunction"],
Cell[1982305, 78926, 400, 11, 38, "Output",
 CellTags->"TensorFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1982742, 78942, 228, 7, 26, "Input",
 CellTags->"TensorFunction"],
Cell[1982973, 78951, 305, 9, 26, "Output",
 CellTags->"TensorFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1983315, 78965, 228, 7, 26, "Input",
 CellTags->"TensorFunction"],
Cell[1983546, 78974, 304, 9, 26, "Output",
 CellTags->"TensorFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1983887, 78988, 116, 3, 26, "Input",
 CellTags->"TensorFunction"],
Cell[1984006, 78993, 285, 8, 38, "Output",
 CellTags->"TensorFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1984328, 79006, 118, 3, 26, "Input",
 CellTags->"TensorFunction"],
Cell[1984449, 79011, 171, 5, 26, "Output",
 CellTags->"TensorFunction"]
}, Open  ]],
Cell[1984635, 79019, 154, 4, 26, "Input",
 CellTags->"TensorFunction"]
}, Open  ]],
Cell[1984804, 79026, 41, 1, 39, "ObjectName",
 CellTags->"Tf"],
Cell[CellGroupData[{
Cell[1984870, 79031, 53, 1, 34, "Subsubsection",
 CellTags->"Tf"],
Cell[1984926, 79034, 214, 10, 27, "Text",
 CellTags->"Tf"],
Cell[1985143, 79046, 414, 18, 24, "Text",
 CellTags->"Tf"]
}, Open  ]],
Cell[1985572, 79067, 43, 1, 39, "ObjectName",
 CellTags->"TFi"],
Cell[CellGroupData[{
Cell[1985640, 79072, 54, 1, 34, "Subsubsection",
 CellTags->"TFi"],
Cell[CellGroupData[{
Cell[1985719, 79077, 72, 2, 26, "Input"],
Cell[1985794, 79081, 513, 12, 22, "Message"]
}, Open  ]],
Cell[1986322, 79096, 576, 10, 92, "Text",
 CellTags->"TFi"],
Cell[1986901, 79108, 237, 8, 24, "Text",
 CellTags->"TFi"],
Cell[1987141, 79118, 185, 4, 41, "Text",
 CellTags->"TFi"],
Cell[1987329, 79124, 266, 13, 24, "Text",
 CellTags->"TFi"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1987632, 79142, 51, 1, 34, "Subsubsection",
 CellTags->"TFi"],
Cell[CellGroupData[{
Cell[1987708, 79147, 506, 17, 26, "Input",
 CellTags->"TFi"],
Cell[1988217, 79166, 819, 25, 96, "Output",
 CellTags->"TFi"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1989073, 79196, 295, 10, 26, "Input",
 CellTags->"TFi"],
Cell[1989371, 79208, 377, 12, 29, "Output",
 CellTags->"TFi"]
}, Open  ]]
}, Open  ]],
Cell[1989775, 79224, 59, 1, 39, "ObjectName",
 CellTags->"ThreeVector"],
Cell[CellGroupData[{
Cell[1989859, 79229, 62, 1, 34, "Subsubsection",
 CellTags->"ThreeVector"],
Cell[1989924, 79232, 91, 1, 24, "Text",
 CellTags->"ThreeVector"],
Cell[1990018, 79235, 304, 13, 24, "Text",
 CellTags->"ThreeVector"]
}, Open  ]],
Cell[1990337, 79251, 43, 1, 39, "ObjectName",
 CellTags->"TID"],
Cell[CellGroupData[{
Cell[1990405, 79256, 54, 1, 34, "Subsubsection",
 CellTags->"TID"],
Cell[1990462, 79259, 80, 1, 24, "Text",
 CellTags->"TID"],
Cell[CellGroupData[{
Cell[1990567, 79264, 106, 3, 26, "Input",
 CellTags->"TID"],
Cell[1990676, 79269, 697, 16, 67, "Output",
 CellTags->"TID"]
}, Open  ]],
Cell[1991388, 79288, 293, 14, 24, "Text",
 CellTags->"TID"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1991718, 79307, 51, 1, 34, "Subsubsection",
 CellTags->"TID"],
Cell[CellGroupData[{
Cell[1991794, 79312, 374, 12, 26, "Input",
 CellTags->"TID"],
Cell[1992171, 79326, 576, 22, 51, "Output",
 CellTags->"TID"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1992784, 79353, 181, 6, 26, "Input",
 CellTags->"TID"],
Cell[1992968, 79361, 4349, 137, 68, "Output",
 CellTags->"TID"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1997354, 79503, 271, 9, 26, "Input",
 CellTags->"TID"],
Cell[1997628, 79514, 431, 17, 51, "Output",
 CellTags->"TID"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1998096, 79536, 123, 4, 26, "Input",
 CellTags->"TID"],
Cell[1998222, 79542, 669, 28, 50, "Output",
 CellTags->"TID"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1998928, 79575, 195, 6, 26, "Input",
 CellTags->"TID"],
Cell[1999126, 79583, 669, 28, 50, "Output",
 CellTags->"TID"]
}, Open  ]]
}, Open  ]],
Cell[1999822, 79615, 45, 1, 39, "ObjectName",
 CellTags->"TIDL"],
Cell[CellGroupData[{
Cell[1999892, 79620, 55, 1, 34, "Subsubsection",
 CellTags->"TIDL"],
Cell[1999950, 79623, 87, 1, 24, "Text",
 CellTags->"TIDL"],
Cell[2000040, 79626, 154, 8, 24, "Text",
 CellTags->"TIDL"]
}, Open  ]],
Cell[2000209, 79637, 52, 1, 34, "Subsubsection",
 CellTags->"TIDL"],
Cell[CellGroupData[{
Cell[2000286, 79642, 154, 5, 37, "Subsubsection",
 CellTags->"TIDL"],
Cell[2000443, 79649, 414, 13, 28, "Text",
 CellTags->"TIDL"],
Cell[CellGroupData[{
Cell[2000882, 79666, 202, 7, 26, "Input",
 CellTags->"TIDL"],
Cell[2001087, 79675, 507, 21, 48, "Output",
 CellTags->"TIDL"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[2001643, 79702, 159, 5, 37, "Subsubsection",
 CellTags->"TIDL"],
Cell[CellGroupData[{
Cell[2001827, 79711, 312, 11, 26, "Input",
 CellTags->"TIDL"],
Cell[2002142, 79724, 1920, 75, 54, "Output",
 CellTags->"TIDL"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[2004111, 79805, 154, 5, 37, "Subsubsection",
 CellTags->"TIDL"],
Cell[CellGroupData[{
Cell[2004290, 79814, 277, 10, 26, "Input",
 CellTags->"TIDL"],
Cell[2004570, 79826, 2854, 109, 51, "Output",
 CellTags->"TIDL"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[2007473, 79941, 159, 5, 37, "Subsubsection",
 CellTags->"TIDL"],
Cell[CellGroupData[{
Cell[2007657, 79950, 387, 14, 26, "Input",
 CellTags->"TIDL"],
Cell[2008047, 79966, 25229, 846, 328, "Output",
 CellTags->"TIDL"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[2033325, 80818, 165, 5, 37, "Subsubsection",
 CellTags->"TIDL"],
Cell[2033493, 80825, 434, 15, 26, "Input",
 CellTags->"TIDL"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2033964, 80845, 173, 5, 37, "Subsubsection",
 CellTags->"TIDL"],
Cell[2034140, 80852, 506, 17, 26, "Input",
 CellTags->"TIDL"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2034683, 80874, 154, 5, 37, "Subsubsection",
 CellTags->"TIDL"],
Cell[CellGroupData[{
Cell[2034862, 80883, 312, 11, 26, "Input",
 CellTags->"TIDL"],
Cell[2035177, 80896, 15400, 530, 213, "Output",
 CellTags->"TIDL"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[2050626, 81432, 159, 5, 37, "Subsubsection",
 CellTags->"TIDL"],
Cell[2050788, 81439, 374, 13, 26, "Input",
 CellTags->"TIDL"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2051199, 81457, 165, 5, 37, "Subsubsection",
 CellTags->"TIDL"],
Cell[2051367, 81464, 444, 15, 26, "Input",
 CellTags->"TIDL"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2051848, 81484, 173, 5, 37, "Subsubsection",
 CellTags->"TIDL"],
Cell[2052024, 81491, 529, 18, 26, "Input",
 CellTags->"TIDL"],
Cell[CellGroupData[{
Cell[2052578, 81513, 240, 7, 26, "Input",
 CellTags->"TIDL"],
Cell[2052821, 81522, 205, 6, 26, "Output",
 CellTags->"TIDL"]
}, Open  ]]
}, Open  ]],
Cell[2053053, 81532, 65, 1, 39, "ObjectName",
 CellTags->"TimedIntegrate"],
Cell[CellGroupData[{
Cell[2053143, 81537, 65, 1, 34, "Subsubsection",
 CellTags->"TimedIntegrate"],
Cell[2053211, 81540, 225, 5, 41, "Text",
 CellTags->"TimedIntegrate"],
Cell[CellGroupData[{
Cell[2053461, 81549, 128, 3, 26, "Input",
 CellTags->"TimedIntegrate"],
Cell[2053592, 81554, 466, 14, 26, "Output",
 CellTags->"TimedIntegrate"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[2054107, 81574, 62, 1, 34, "Subsubsection",
 CellTags->"TimedIntegrate"],
Cell[2054172, 81577, 73, 1, 24, "Text",
 CellTags->"TimedIntegrate"],
Cell[CellGroupData[{
Cell[2054270, 81582, 305, 9, 26, "Input",
 CellTags->"TimedIntegrate"],
Cell[2054578, 81593, 171, 5, 26, "Output",
 CellTags->"TimedIntegrate"]
}, Open  ]],
Cell[2054764, 81601, 59, 1, 24, "Text",
 CellTags->"TimedIntegrate"],
Cell[CellGroupData[{
Cell[2054848, 81606, 389, 11, 26, "Input",
 CellTags->"TimedIntegrate"],
Cell[2055240, 81619, 491, 15, 29, "Output",
 CellTags->"TimedIntegrate"]
}, Open  ]]
}, Open  ]],
Cell[2055758, 81638, 61, 1, 39, "ObjectName",
 CellTags->"TLI"],
Cell[CellGroupData[{
Cell[2055844, 81643, 54, 1, 34, "Subsubsection",
 CellTags->"TLI"],
Cell[2055901, 81646, 616, 12, 116, "Text",
 CellTags->"TLI"],
Cell[CellGroupData[{
Cell[2056542, 81662, 106, 3, 26, "Input",
 CellTags->"TLI"],
Cell[2056651, 81667, 273, 9, 28, "Output",
 CellTags->"TLI"]
}, Open  ]],
Cell[2056939, 81679, 168, 9, 24, "Text",
 CellTags->"TLI"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2057144, 81693, 51, 1, 34, "Subsubsection",
 CellTags->"TLI"],
Cell[2057198, 81696, 70, 2, 26, "Input",
 CellTags->"TLI"]
}, Open  ]],
Cell[2057283, 81701, 65, 1, 39, "ObjectName",
 CellTags->"TLIFP"],
Cell[CellGroupData[{
Cell[2057373, 81706, 56, 1, 34, "Subsubsection",
 CellTags->"TLIFP"],
Cell[2057432, 81709, 93, 1, 24, "Text",
 CellTags->"TLIFP"],
Cell[CellGroupData[{
Cell[2057550, 81714, 110, 3, 26, "Input",
 CellTags->"TLIFP"],
Cell[2057663, 81719, 467, 13, 28, "Output",
 CellTags->"TLIFP"]
}, Open  ]],
Cell[2058145, 81735, 161, 9, 24, "Text",
 CellTags->"TLIFP"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2058343, 81749, 53, 1, 34, "Subsubsection",
 CellTags->"TLIFP"],
Cell[2058399, 81752, 72, 2, 26, "Input",
 CellTags->"TLIFP"]
}, Open  ]],
Cell[2058486, 81757, 67, 1, 39, "ObjectName",
 CellTags->"TLIHYP"],
Cell[CellGroupData[{
Cell[2058578, 81762, 57, 1, 34, "Subsubsection",
 CellTags->"TLIHYP"],
Cell[2058638, 81765, 136, 4, 24, "Text",
 CellTags->"TLIHYP"],
Cell[CellGroupData[{
Cell[2058799, 81773, 112, 3, 26, "Input",
 CellTags->"TLIHYP"],
Cell[2058914, 81778, 283, 9, 28, "Output",
 CellTags->"TLIHYP"]
}, Open  ]],
Cell[2059212, 81790, 162, 9, 24, "Text",
 CellTags->"TLIHYP"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2059411, 81804, 54, 1, 34, "Subsubsection",
 CellTags->"TLIHYP"],
Cell[2059468, 81807, 73, 2, 26, "Input",
 CellTags->"TLIHYP"]
}, Open  ]],
Cell[2059556, 81812, 45, 1, 39, "ObjectName",
 CellTags->"TLI2"],
Cell[CellGroupData[{
Cell[2059626, 81817, 55, 1, 34, "Subsubsection",
 CellTags->"TLI2"],
Cell[2059684, 81820, 83, 1, 24, "Text",
 CellTags->"TLI2"],
Cell[2059770, 81823, 160, 9, 24, "Text",
 CellTags->"TLI2"]
}, Open  ]],
Cell[2059945, 81835, 67, 1, 39, "ObjectName",
 CellTags->"TLI2FC"],
Cell[CellGroupData[{
Cell[2060037, 81840, 57, 1, 34, "Subsubsection",
 CellTags->"TLI2FC"],
Cell[2060097, 81843, 109, 3, 24, "Text",
 CellTags->"TLI2FC"],
Cell[CellGroupData[{
Cell[2060231, 81850, 112, 3, 26, "Input",
 CellTags->"TLI2FC"],
Cell[2060346, 81855, 397, 12, 28, "Output",
 CellTags->"TLI2FC"]
}, Open  ]],
Cell[2060758, 81870, 262, 13, 24, "Text",
 CellTags->"TLI2FC"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2061057, 81888, 54, 1, 34, "Subsubsection",
 CellTags->"TLI2FC"],
Cell[2061114, 81891, 73, 2, 26, "Input",
 CellTags->"TLI2FC"]
}, Open  ]],
Cell[2061202, 81896, 65, 1, 39, "ObjectName",
 CellTags->"ToDistribution"],
Cell[CellGroupData[{
Cell[2061292, 81901, 65, 1, 34, "Subsubsection",
 CellTags->"ToDistribution"],
Cell[2061360, 81904, 424, 7, 58, "Text",
 CellTags->"ToDistribution"],
Cell[CellGroupData[{
Cell[2061809, 81915, 128, 3, 26, "Input",
 CellTags->"ToDistribution"],
Cell[2061940, 81920, 233, 7, 26, "Output",
 CellTags->"ToDistribution"]
}, Open  ]],
Cell[2062188, 81930, 209, 9, 24, "Text",
 CellTags->"ToDistribution"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2062434, 81944, 62, 1, 34, "Subsubsection",
 CellTags->"ToDistribution"],
Cell[CellGroupData[{
Cell[2062521, 81949, 217, 6, 26, "Input",
 CellTags->"ToDistribution"],
Cell[2062741, 81957, 239, 7, 26, "Output",
 CellTags->"ToDistribution"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2063017, 81969, 277, 9, 26, "Input",
 CellTags->"ToDistribution"],
Cell[2063297, 81980, 1382, 49, 48, "Output",
 CellTags->"ToDistribution"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2064716, 82034, 206, 5, 26, "Input",
 CellTags->"ToDistribution"],
Cell[2064925, 82041, 231, 7, 26, "Output",
 CellTags->"ToDistribution"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2065193, 82053, 277, 9, 26, "Input",
 CellTags->"ToDistribution"],
Cell[2065473, 82064, 1422, 51, 51, "Output",
 CellTags->"ToDistribution"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2066932, 82120, 461, 14, 30, "Input",
 CellTags->"ToDistribution"],
Cell[2067396, 82136, 413, 14, 26, "Output",
 CellTags->"ToDistribution"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2067846, 82155, 118, 3, 26, "Input",
 CellTags->"ToDistribution"],
Cell[2067967, 82160, 374, 10, 26, "Output",
 CellTags->"ToDistribution"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2068378, 82175, 409, 13, 26, "Input",
 CellTags->"ToDistribution"],
Cell[2068790, 82190, 413, 14, 26, "Output",
 CellTags->"ToDistribution"]
}, Open  ]]
}, Open  ]],
Cell[2069230, 82208, 69, 1, 39, "ObjectName",
 CellTags->"ToHypergeometric"],
Cell[CellGroupData[{
Cell[2069324, 82213, 67, 1, 34, "Subsubsection",
 CellTags->"ToHypergeometric"],
Cell[2069394, 82216, 1686, 67, 43, "Text",
 CellTags->"ToHypergeometric"],
Cell[2071083, 82285, 479, 19, 24, "Text",
 CellTags->"ToHypergeometric"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2071599, 82309, 64, 1, 34, "Subsubsection",
 CellTags->"ToHypergeometric"],
Cell[CellGroupData[{
Cell[2071688, 82314, 395, 13, 30, "Input",
 CellTags->"ToHypergeometric"],
Cell[2072086, 82329, 531, 16, 48, "Output",
 CellTags->"ToHypergeometric"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2072654, 82350, 488, 16, 30, "Input",
 CellTags->"ToHypergeometric"],
Cell[2073145, 82368, 400, 14, 48, "Output",
 CellTags->"ToHypergeometric"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2073582, 82387, 395, 13, 30, "Input",
 CellTags->"ToHypergeometric"],
Cell[2073980, 82402, 596, 18, 52, "Output",
 CellTags->"ToHypergeometric"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2074613, 82425, 488, 16, 30, "Input",
 CellTags->"ToHypergeometric"],
Cell[2075104, 82443, 473, 16, 52, "Output",
 CellTags->"ToHypergeometric"]
}, Open  ]]
}, Open  ]],
Cell[2075604, 82463, 51, 1, 39, "ObjectName",
 CellTags->"ToLarin"],
Cell[CellGroupData[{
Cell[2075680, 82468, 58, 1, 34, "Subsubsection",
 CellTags->"ToLarin"],
Cell[2075741, 82471, 605, 19, 24, "Text",
 CellTags->"ToLarin"],
Cell[2076349, 82492, 163, 9, 24, "Text",
 CellTags->"ToLarin"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2076549, 82506, 55, 1, 34, "Subsubsection",
 CellTags->"ToLarin"],
Cell[CellGroupData[{
Cell[2076629, 82511, 229, 6, 26, "Input",
 CellTags->"ToLarin"],
Cell[2076861, 82519, 431, 16, 26, "Output",
 CellTags->"ToLarin"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2077329, 82540, 108, 3, 26, "Input",
 CellTags->"ToLarin"],
Cell[2077440, 82545, 1141, 46, 44, "Output",
 CellTags->"ToLarin"]
}, Open  ]]
}, Open  ]],
Cell[2078608, 82595, 47, 1, 39, "ObjectName",
 CellTags->"ToTFi"],
Cell[CellGroupData[{
Cell[2078680, 82600, 56, 1, 34, "Subsubsection",
 CellTags->"ToTFi"],
Cell[2078739, 82603, 272, 5, 41, "Text",
 CellTags->"ToTFi"],
Cell[2079014, 82610, 262, 13, 24, "Text",
 CellTags->"ToTFi"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2079313, 82628, 53, 1, 34, "Subsubsection",
 CellTags->"ToTFi"],
Cell[CellGroupData[{
Cell[2079391, 82633, 333, 11, 26, "Input",
 CellTags->"ToTFi"],
Cell[2079727, 82646, 1912, 50, 51, "Output",
 CellTags->"ToTFi"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2081676, 82701, 149, 4, 26, "Input",
 CellTags->"ToTFi"],
Cell[2081828, 82707, 2168, 55, 52, "Output",
 CellTags->"ToTFi"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2084033, 82767, 107, 3, 26, "Input",
 CellTags->"ToTFi"],
Cell[2084143, 82772, 535, 17, 38, "Output",
 CellTags->"ToTFi"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2084715, 82794, 468, 15, 26, "Input",
 CellTags->"ToTFi"],
Cell[2085186, 82811, 958, 34, 51, "Output",
 CellTags->"ToTFi"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[2086193, 82851, 41, 1, 39, "ObjectName",
 CellTags->"TR"],
Cell[2086237, 82854, 100, 3, 26, "Output",
 CellTags->"TR"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2086374, 82862, 53, 1, 34, "Subsubsection",
 CellTags->"TR"],
Cell[2086430, 82865, 420, 12, 58, "Text",
 CellTags->"TR"],
Cell[CellGroupData[{
Cell[2086875, 82881, 104, 3, 26, "Input",
 CellTags->"TR"],
Cell[2086982, 82886, 803, 19, 67, "Output",
 CellTags->"TR"]
}, Open  ]],
Cell[2087800, 82908, 536, 23, 24, "Text",
 CellTags->"TR"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2088373, 82936, 50, 1, 34, "Subsubsection",
 CellTags->"TR"],
Cell[CellGroupData[{
Cell[2088448, 82941, 129, 4, 26, "Input",
 CellTags->"TR"],
Cell[2088580, 82947, 238, 7, 26, "Output",
 CellTags->"TR"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2088855, 82959, 98, 3, 26, "Input",
 CellTags->"TR"],
Cell[2088956, 82964, 217, 7, 26, "Output",
 CellTags->"TR"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2089210, 82976, 375, 12, 26, "Input",
 CellTags->"TR"],
Cell[2089588, 82990, 1299, 51, 29, "Output",
 CellTags->"TR"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2090924, 83046, 205, 6, 26, "Input",
 CellTags->"TR"],
Cell[2091132, 83054, 601, 24, 26, "Output",
 CellTags->"TR"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2091770, 83083, 240, 7, 26, "Input",
 CellTags->"TR"],
Cell[2092013, 83092, 8713, 322, 73, "Output",
 CellTags->"TR"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2100763, 83419, 74, 2, 26, "Input",
 CellTags->"TR"],
Cell[2100840, 83423, 146, 4, 26, "Output",
 CellTags->"TR"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2101023, 83432, 91, 3, 26, "Input",
 CellTags->"TR"],
Cell[2101117, 83437, 421, 7, 79, "Print",
 CellTags->{"Info3500974047-4982097", "TR"}]
}, Open  ]],
Cell[2101553, 83447, 166, 4, 24, "Text",
 CellTags->"TR"],
Cell[CellGroupData[{
Cell[2101744, 83455, 100, 3, 26, "Input",
 CellTags->"TR"],
Cell[2101847, 83460, 145, 4, 26, "Output",
 CellTags->"TR"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2102029, 83469, 421, 13, 26, "Input",
 CellTags->"TR"],
Cell[2102453, 83484, 141, 4, 26, "Output",
 CellTags->"TR"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2102631, 83493, 171, 5, 26, "Input",
 CellTags->"TR"],
Cell[2102805, 83500, 1035, 40, 28, "Output",
 CellTags->"TR"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2103877, 83545, 444, 14, 26, "Input",
 CellTags->"TR"],
Cell[2104324, 83561, 384, 14, 29, "Output",
 CellTags->"TR"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2104745, 83580, 224, 7, 26, "Input",
 CellTags->"TR"],
Cell[2104972, 83589, 222, 7, 26, "Output",
 CellTags->"TR"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2105231, 83601, 104, 3, 26, "Input",
 CellTags->"TR"],
Cell[2105338, 83606, 215, 6, 38, "Output",
 CellTags->"TR"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2105590, 83617, 347, 10, 26, "Input",
 CellTags->"TR"],
Cell[2105940, 83629, 234, 7, 26, "Output",
 CellTags->"TR"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2106211, 83641, 104, 3, 26, "Input",
 CellTags->"TR"],
Cell[2106318, 83646, 225, 6, 38, "Output",
 CellTags->"TR"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2106580, 83657, 339, 11, 26, "Input",
 CellTags->"TR"],
Cell[2106922, 83670, 470, 17, 28, "Output",
 CellTags->"TR"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2107429, 83692, 214, 6, 26, "Input",
 CellTags->"TR"],
Cell[2107646, 83700, 252, 8, 26, "Output",
 CellTags->"TR"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2107935, 83713, 213, 6, 26, "Input",
 CellTags->"TR"],
Cell[2108151, 83721, 252, 8, 26, "Output",
 CellTags->"TR"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2108440, 83734, 214, 6, 26, "Input",
 CellTags->"TR"],
Cell[2108657, 83742, 141, 4, 26, "Output",
 CellTags->"TR"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2108835, 83751, 213, 6, 26, "Input",
 CellTags->"TR"],
Cell[2109051, 83759, 141, 4, 26, "Output",
 CellTags->"TR"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2109229, 83768, 139, 4, 26, "Input"],
Cell[2109371, 83774, 194, 6, 26, "Output"]
}, Open  ]],
Cell[2109580, 83783, 119, 4, 26, "Input",
 CellTags->"TR"]
}, Open  ]],
Cell[2109714, 83790, 65, 1, 39, "ObjectName",
 CellTags->"TraceDimension"],
Cell[CellGroupData[{
Cell[2109804, 83795, 65, 1, 34, "Subsubsection",
 CellTags->"TraceDimension"],
Cell[2109872, 83798, 142, 4, 24, "Text",
 CellTags->"TraceDimension"],
Cell[2110017, 83804, 200, 9, 24, "Text",
 CellTags->"TraceDimension"]
}, Open  ]],
Cell[2110232, 83816, 57, 1, 39, "ObjectName",
 CellTags->"TraceOfOne"],
Cell[CellGroupData[{
Cell[2110314, 83821, 61, 1, 34, "Subsubsection",
 CellTags->"TraceOfOne"],
Cell[2110378, 83824, 149, 4, 24, "Text",
 CellTags->"TraceOfOne"],
Cell[2110530, 83830, 273, 13, 24, "Text",
 CellTags->"TraceOfOne"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2110840, 83848, 58, 1, 34, "Subsubsection",
 CellTags->"TraceOfOne"],
Cell[CellGroupData[{
Cell[2110923, 83853, 273, 7, 26, "Input",
 CellTags->"TraceOfOne"],
Cell[2111199, 83862, 151, 4, 26, "Output",
 CellTags->"TraceOfOne"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2111387, 83871, 272, 7, 26, "Input",
 CellTags->"TraceOfOne"],
Cell[2111662, 83880, 153, 4, 26, "Output",
 CellTags->"TraceOfOne"]
}, Open  ]]
}, Open  ]],
Cell[2111842, 83888, 47, 1, 39, "ObjectName",
 CellTags->"Trick"],
Cell[CellGroupData[{
Cell[2111914, 83893, 56, 1, 34, "Subsubsection",
 CellTags->"Trick"],
Cell[2111973, 83896, 174, 4, 24, "Text",
 CellTags->"Trick"],
Cell[2112150, 83902, 625, 29, 24, "Text",
 CellTags->"Trick"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2112812, 83936, 62, 1, 33, "Subsubsection",
 CellTags->"Trick"],
Cell[2112877, 83939, 376, 15, 27, "Text",
 CellTags->"Trick"],
Cell[CellGroupData[{
Cell[2113278, 83958, 350, 11, 26, "Input",
 CellTags->"Trick"],
Cell[2113631, 83971, 306, 11, 26, "Output",
 CellTags->"Trick"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2113974, 83987, 330, 11, 26, "Input",
 CellTags->"Trick"],
Cell[2114307, 84000, 468, 17, 28, "Output",
 CellTags->"Trick"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2114812, 84022, 104, 3, 26, "Input",
 CellTags->"Trick"],
Cell[2114919, 84027, 569, 23, 28, "Output",
 CellTags->"Trick"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2115525, 84055, 227, 6, 26, "Input",
 CellTags->"Trick"],
Cell[2115755, 84063, 433, 16, 28, "Output",
 CellTags->"Trick"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2116225, 84084, 133, 4, 26, "Input",
 CellTags->"Trick"],
Cell[2116361, 84090, 257, 7, 38, "Output",
 CellTags->"Trick"]
}, Open  ]]
}, Open  ]],
Cell[2116645, 84101, 65, 1, 39, "ObjectName",
 CellTags->"TrickIntegrate"],
Cell[CellGroupData[{
Cell[2116735, 84106, 65, 1, 34, "Subsubsection",
 CellTags->"TrickIntegrate"],
Cell[2116803, 84109, 483, 8, 58, "Text",
 CellTags->"TrickIntegrate"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2117323, 84122, 62, 1, 34, "Subsubsection",
 CellTags->"TrickIntegrate"],
Cell[CellGroupData[{
Cell[2117410, 84127, 364, 12, 26, "Input",
 CellTags->"TrickIntegrate"],
Cell[2117777, 84141, 561, 19, 29, "Output",
 CellTags->"TrickIntegrate"]
}, Open  ]]
}, Open  ]],
Cell[2118365, 84164, 67, 1, 39, "ObjectName",
 CellTags->"TrickMandelstam"],
Cell[CellGroupData[{
Cell[2118457, 84169, 66, 1, 34, "Subsubsection",
 CellTags->"TrickMandelstam"],
Cell[2118526, 84172, 1340, 55, 45, "Text",
 CellTags->"TrickMandelstam"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2119903, 84232, 63, 1, 34, "Subsubsection",
 CellTags->"TrickMandelstam"],
Cell[CellGroupData[{
Cell[2119991, 84237, 528, 18, 30, "Input",
 CellTags->"TrickMandelstam"],
Cell[2120522, 84257, 294, 10, 29, "Output",
 CellTags->"TrickMandelstam"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2120853, 84272, 568, 17, 26, "Input",
 CellTags->"TrickMandelstam"],
Cell[2121424, 84291, 277, 9, 26, "Output",
 CellTags->"TrickMandelstam"]
}, Open  ]]
}, Open  ]],
Cell[2121728, 84304, 43, 1, 39, "ObjectName",
 CellTags->"Tr2"],
Cell[CellGroupData[{
Cell[2121796, 84309, 54, 1, 34, "Subsubsection",
 CellTags->"Tr2"],
Cell[2121853, 84312, 347, 6, 58, "Text",
 CellTags->"Tr2"],
Cell[CellGroupData[{
Cell[2122225, 84322, 106, 3, 26, "Input",
 CellTags->"Tr2"],
Cell[2122334, 84327, 210, 7, 26, "Output",
 CellTags->"Tr2"]
}, Open  ]],
Cell[2122559, 84337, 593, 28, 24, "Text",
 CellTags->"Tr2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2123189, 84370, 51, 1, 34, "Subsubsection",
 CellTags->"Tr2"],
Cell[CellGroupData[{
Cell[2123265, 84375, 164, 4, 26, "Input"],
Cell[2123432, 84381, 590, 21, 31, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2124059, 84407, 187, 6, 26, "Input",
 CellTags->"Tr2"],
Cell[2124249, 84415, 200, 6, 29, "Output",
 CellTags->"Tr2"]
}, Open  ]]
}, Open  ]],
Cell[2124476, 84425, 116, 2, 39, "ObjectName",
 CellTags->"Twist2AlienOperator"],
Cell[CellGroupData[{
Cell[2124617, 84431, 70, 1, 34, "Subsubsection",
 CellTags->"Twist2AlienOperator"],
Cell[2124690, 84434, 194, 4, 24, "Text",
 CellTags->"Twist2AlienOperator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2124921, 84443, 67, 1, 34, "Subsubsection",
 CellTags->"Twist2AlienOperator"],
Cell[CellGroupData[{
Cell[2125013, 84448, 160, 4, 26, "Input",
 CellTags->"Twist2AlienOperator"],
Cell[2125176, 84454, 929, 31, 52, "Output",
 CellTags->"Twist2AlienOperator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2126142, 84490, 246, 7, 26, "Input",
 CellTags->"Twist2AlienOperator"],
Cell[2126391, 84499, 3155, 104, 55, "Output",
 CellTags->"Twist2AlienOperator"]
}, Open  ]]
}, Open  ]],
Cell[2129573, 84607, 120, 2, 39, "ObjectName",
 CellTags->"Twist2CounterOperator"],
Cell[CellGroupData[{
Cell[2129718, 84613, 72, 1, 34, "Subsubsection",
 CellTags->"Twist2CounterOperator"],
Cell[2129793, 84616, 239, 5, 41, "Text",
 CellTags->"Twist2CounterOperator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2130069, 84626, 69, 1, 34, "Subsubsection",
 CellTags->"Twist2CounterOperator"],
Cell[CellGroupData[{
Cell[2130163, 84631, 204, 5, 26, "Input",
 CellTags->"Twist2CounterOperator"],
Cell[2130370, 84638, 4044, 137, 127, "Output",
 CellTags->"Twist2CounterOperator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2134451, 84780, 159, 4, 26, "Input",
 CellTags->"Twist2CounterOperator"],
Cell[2134613, 84786, 1064, 36, 52, "Output",
 CellTags->"Twist2CounterOperator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2135714, 84827, 242, 6, 26, "Input",
 CellTags->"Twist2CounterOperator"],
Cell[2135959, 84835, 2652, 90, 55, "Output",
 CellTags->"Twist2CounterOperator"]
}, Open  ]]
}, Open  ]],
Cell[2138638, 84929, 75, 1, 39, "ObjectName",
 CellTags->"Twist2GluonOperator"],
Cell[CellGroupData[{
Cell[2138738, 84934, 70, 1, 34, "Subsubsection",
 CellTags->"Twist2GluonOperator"],
Cell[2138811, 84937, 964, 15, 126, "Text",
 CellTags->"Twist2GluonOperator"],
Cell[2139778, 84954, 223, 9, 24, "Text",
 CellTags->"Twist2GluonOperator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2140038, 84968, 99, 1, 34, "Subsubsection",
 CellTags->"Twist2GluonOperator"],
Cell[CellGroupData[{
Cell[2140162, 84973, 352, 8, 46, "Input",
 CellTags->"Twist2GluonOperator"],
Cell[2140517, 84983, 486, 13, 26, "Output",
 CellTags->"Twist2GluonOperator"]
}, Open  ]],
Cell[2141018, 84999, 62, 1, 24, "Text",
 CellTags->"Twist2GluonOperator"],
Cell[CellGroupData[{
Cell[2141105, 85004, 271, 8, 26, "Input",
 CellTags->"Twist2GluonOperator"],
Cell[2141379, 85014, 936, 36, 30, "Output",
 CellTags->"Twist2GluonOperator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2142352, 85055, 321, 9, 26, "Input",
 CellTags->"Twist2GluonOperator"],
Cell[2142676, 85066, 1162, 46, 28, "Output",
 CellTags->"Twist2GluonOperator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2143875, 85117, 221, 6, 26, "Input",
 CellTags->"Twist2GluonOperator"],
Cell[2144099, 85125, 378, 13, 30, "Output",
 CellTags->"Twist2GluonOperator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2144514, 85143, 271, 7, 26, "Input",
 CellTags->"Twist2GluonOperator"],
Cell[2144788, 85152, 747, 29, 28, "Output",
 CellTags->"Twist2GluonOperator"]
}, Open  ]],
Cell[2145550, 85184, 62, 1, 24, "Text",
 CellTags->"Twist2GluonOperator"],
Cell[CellGroupData[{
Cell[2145637, 85189, 355, 10, 26, "Input",
 CellTags->"Twist2GluonOperator"],
Cell[2145995, 85201, 1235, 49, 30, "Output",
 CellTags->"Twist2GluonOperator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2147267, 85255, 325, 10, 26, "Input",
 CellTags->"Twist2GluonOperator"],
Cell[2147595, 85267, 534, 19, 30, "Output",
 CellTags->"Twist2GluonOperator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2148166, 85291, 375, 11, 26, "Input",
 CellTags->"Twist2GluonOperator"],
Cell[2148544, 85304, 11350, 412, 220, "Output",
 CellTags->"Twist2GluonOperator"]
}, Open  ]],
Cell[2159909, 85719, 62, 1, 24, "Text",
 CellTags->"Twist2GluonOperator"],
Cell[CellGroupData[{
Cell[2159996, 85724, 431, 12, 26, "Input",
 CellTags->"Twist2GluonOperator"],
Cell[2160430, 85738, 5018, 186, 54, "Output",
 CellTags->"Twist2GluonOperator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2165485, 85929, 391, 12, 26, "Input",
 CellTags->"Twist2GluonOperator"],
Cell[2165879, 85943, 648, 23, 30, "Output",
 CellTags->"Twist2GluonOperator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2166564, 85971, 441, 13, 26, "Input",
 CellTags->"Twist2GluonOperator"],
Cell[2167008, 85986, 24346, 870, 489, "Output",
 CellTags->"Twist2GluonOperator"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[2191403, 86862, 100, 1, 34, "Subsubsection",
 CellTags->"Twist2GluonOperator"],
Cell[CellGroupData[{
Cell[2191528, 86867, 268, 6, 26, "Input",
 CellTags->"Twist2GluonOperator"],
Cell[2191799, 86875, 486, 13, 26, "Output",
 CellTags->"Twist2GluonOperator"]
}, Open  ]],
Cell[2192300, 86891, 62, 1, 24, "Text",
 CellTags->"Twist2GluonOperator"],
Cell[CellGroupData[{
Cell[2192387, 86896, 271, 8, 26, "Input",
 CellTags->"Twist2GluonOperator"],
Cell[2192661, 86906, 946, 37, 44, "Output",
 CellTags->"Twist2GluonOperator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2193644, 86948, 321, 9, 26, "Input",
 CellTags->"Twist2GluonOperator"],
Cell[2193968, 86959, 2698, 103, 44, "Output",
 CellTags->"Twist2GluonOperator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2196703, 87067, 221, 6, 26, "Input",
 CellTags->"Twist2GluonOperator"],
Cell[2196927, 87075, 380, 13, 30, "Output",
 CellTags->"Twist2GluonOperator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2197344, 87093, 271, 7, 26, "Input",
 CellTags->"Twist2GluonOperator"],
Cell[2197618, 87102, 2275, 85, 29, "Output",
 CellTags->"Twist2GluonOperator"]
}, Open  ]],
Cell[2199908, 87190, 62, 1, 24, "Text",
 CellTags->"Twist2GluonOperator"],
Cell[CellGroupData[{
Cell[2199995, 87195, 355, 10, 26, "Input",
 CellTags->"Twist2GluonOperator"],
Cell[2200353, 87207, 1311, 51, 44, "Output",
 CellTags->"Twist2GluonOperator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2201701, 87263, 325, 10, 26, "Input",
 CellTags->"Twist2GluonOperator"],
Cell[2202029, 87275, 534, 19, 30, "Output",
 CellTags->"Twist2GluonOperator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2202600, 87299, 375, 11, 26, "Input",
 CellTags->"Twist2GluonOperator"],
Cell[2202978, 87312, 16386, 588, 239, "Output",
 CellTags->"Twist2GluonOperator"]
}, Open  ]],
Cell[2219379, 87903, 62, 1, 24, "Text",
 CellTags->"Twist2GluonOperator"],
Cell[CellGroupData[{
Cell[2219466, 87908, 431, 12, 26, "Input",
 CellTags->"Twist2GluonOperator"],
Cell[2219900, 87922, 5027, 187, 71, "Output",
 CellTags->"Twist2GluonOperator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2224964, 88114, 391, 12, 26, "Input",
 CellTags->"Twist2GluonOperator"],
Cell[2225358, 88128, 648, 23, 30, "Output",
 CellTags->"Twist2GluonOperator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2226043, 88156, 441, 13, 26, "Input",
 CellTags->"Twist2GluonOperator"],
Cell[2226487, 88171, 31903, 1142, 575, "Output",
 CellTags->"Twist2GluonOperator"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[2258439, 89319, 76, 1, 34, "Subsubsection",
 CellTags->"Twist2GluonOperator"],
Cell[2258518, 89322, 97, 1, 24, "Text",
 CellTags->"Twist2GluonOperator"],
Cell[2258618, 89325, 295, 7, 26, "Input",
 CellTags->"Twist2GluonOperator"],
Cell[2258916, 89334, 380, 11, 26, "Input",
 CellTags->"Twist2GluonOperator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2259333, 89350, 102, 1, 34, "Subsubsection",
 CellTags->"Twist2GluonOperator"],
Cell[CellGroupData[{
Cell[2259460, 89355, 284, 6, 26, "Input",
 CellTags->"Twist2GluonOperator"],
Cell[2259747, 89363, 485, 13, 26, "Output",
 CellTags->"Twist2GluonOperator"]
}, Open  ]],
Cell[2260247, 89379, 62, 1, 24, "Text",
 CellTags->"Twist2GluonOperator"],
Cell[CellGroupData[{
Cell[2260334, 89384, 320, 10, 26, "Input",
 CellTags->"Twist2GluonOperator"],
Cell[2260657, 89396, 3653, 130, 44, "Output",
 CellTags->"Twist2GluonOperator"]
}, Open  ]],
Cell[2264325, 89529, 62, 1, 24, "Text",
 CellTags->"Twist2GluonOperator"],
Cell[CellGroupData[{
Cell[2264412, 89534, 355, 10, 26, "Input",
 CellTags->"Twist2GluonOperator"],
Cell[2264770, 89546, 54229, 1691, 1130, "Output",
 CellTags->"Twist2GluonOperator"]
}, Open  ]],
Cell[2319014, 91240, 62, 1, 24, "Text",
 CellTags->"Twist2GluonOperator"],
Cell[CellGroupData[{
Cell[2319101, 91245, 482, 13, 26, "Input",
 CellTags->"Twist2GluonOperator"],
Cell[2319586, 91260, 5018, 186, 54, "Output",
 CellTags->"Twist2GluonOperator"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[2324653, 91452, 104, 1, 34, "Subsubsection",
 CellTags->"Twist2GluonOperator"],
Cell[CellGroupData[{
Cell[2324782, 91457, 284, 6, 26, "Input",
 CellTags->"Twist2GluonOperator"],
Cell[2325069, 91465, 485, 13, 26, "Output",
 CellTags->"Twist2GluonOperator"]
}, Open  ]],
Cell[2325569, 91481, 62, 1, 24, "Text",
 CellTags->"Twist2GluonOperator"],
Cell[CellGroupData[{
Cell[2325656, 91486, 320, 10, 26, "Input",
 CellTags->"Twist2GluonOperator"],
Cell[2325979, 91498, 3116, 117, 44, "Output",
 CellTags->"Twist2GluonOperator"]
}, Open  ]],
Cell[2329110, 91618, 62, 1, 24, "Text",
 CellTags->"Twist2GluonOperator"],
Cell[2329175, 91621, 380, 11, 26, "Input",
 CellTags->"Twist2GluonOperator"],
Cell[CellGroupData[{
Cell[2329580, 91636, 325, 10, 26, "Input",
 CellTags->"Twist2GluonOperator"],
Cell[2329908, 91648, 3119, 117, 44, "Output",
 CellTags->"Twist2GluonOperator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2333064, 91770, 375, 11, 26, "Input",
 CellTags->"Twist2GluonOperator"],
Cell[2333442, 91783, 3117, 117, 44, "Output",
 CellTags->"Twist2GluonOperator"]
}, Open  ]],
Cell[2336574, 91903, 62, 1, 24, "Text",
 CellTags->"Twist2GluonOperator"],
Cell[CellGroupData[{
Cell[2336661, 91908, 482, 13, 26, "Input",
 CellTags->"Twist2GluonOperator"],
Cell[2337146, 91923, 5028, 187, 71, "Output",
 CellTags->"Twist2GluonOperator"]
}, Open  ]],
Cell[2342189, 92113, 78, 1, 24, "Text",
 CellTags->"Twist2GluonOperator"],
Cell[CellGroupData[{
Cell[2342292, 92118, 520, 16, 26, "Input",
 CellTags->"Twist2GluonOperator"],
Cell[2342815, 92136, 10949, 377, 200, "Output",
 CellTags->"Twist2GluonOperator"]
}, Open  ]],
Cell[2353779, 92516, 109, 3, 24, "Text",
 CellTags->"Twist2GluonOperator"],
Cell[2353891, 92521, 295, 7, 26, "Input",
 CellTags->"Twist2GluonOperator"],
Cell[CellGroupData[{
Cell[2354211, 92532, 428, 13, 26, "Input",
 CellTags->"Twist2GluonOperator"],
Cell[2354642, 92547, 11788, 376, 288, "Output",
 CellTags->"Twist2GluonOperator"]
}, Open  ]]
}, Open  ]],
Cell[2366457, 92927, 75, 1, 39, "ObjectName",
 CellTags->"Twist2QuarkOperator"],
Cell[CellGroupData[{
Cell[2366557, 92932, 70, 1, 34, "Subsubsection",
 CellTags->"Twist2QuarkOperator"],
Cell[2366630, 92935, 980, 15, 126, "Text",
 CellTags->"Twist2QuarkOperator"],
Cell[2367613, 92952, 223, 9, 24, "Text",
 CellTags->"Twist2QuarkOperator"]
}, Open  ]],
Cell[2367851, 92964, 67, 1, 34, "Subsubsection",
 CellTags->"Twist2QuarkOperator"],
Cell[CellGroupData[{
Cell[2367943, 92969, 98, 1, 34, "Subsubsection",
 CellTags->"Twist2QuarkOperator"],
Cell[CellGroupData[{
Cell[2368066, 92974, 268, 6, 26, "Input",
 CellTags->"Twist2QuarkOperator"],
Cell[2368337, 92982, 487, 13, 26, "Output",
 CellTags->"Twist2QuarkOperator"]
}, Open  ]],
Cell[2368839, 92998, 75, 1, 24, "Text",
 CellTags->"Twist2QuarkOperator"],
Cell[CellGroupData[{
Cell[2368939, 93003, 156, 4, 26, "Input",
 CellTags->"Twist2QuarkOperator"],
Cell[2369098, 93009, 642, 22, 28, "Output",
 CellTags->"Twist2QuarkOperator"]
}, Open  ]],
Cell[2369755, 93034, 81, 1, 24, "Text",
 CellTags->"Twist2QuarkOperator"],
Cell[CellGroupData[{
Cell[2369861, 93039, 305, 9, 26, "Input",
 CellTags->"Twist2QuarkOperator"],
Cell[2370169, 93050, 1625, 58, 57, "Output",
 CellTags->"Twist2QuarkOperator"]
}, Open  ]],
Cell[2371809, 93111, 87, 1, 24, "Text",
 CellTags->"Twist2QuarkOperator"],
Cell[CellGroupData[{
Cell[2371921, 93116, 380, 11, 26, "Input",
 CellTags->"Twist2QuarkOperator"],
Cell[2372304, 93129, 4959, 164, 118, "Output",
 CellTags->"Twist2QuarkOperator"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[2377312, 93299, 100, 1, 34, "Subsubsection",
 CellTags->"Twist2QuarkOperator"],
Cell[CellGroupData[{
Cell[2377437, 93304, 273, 6, 26, "Input",
 CellTags->"Twist2QuarkOperator"],
Cell[2377713, 93312, 485, 13, 26, "Output",
 CellTags->"Twist2QuarkOperator"]
}, Open  ]],
Cell[2378213, 93328, 75, 1, 24, "Text",
 CellTags->"Twist2QuarkOperator"],
Cell[CellGroupData[{
Cell[2378313, 93333, 156, 4, 26, "Input",
 CellTags->"Twist2QuarkOperator"],
Cell[2378472, 93339, 523, 18, 28, "Output",
 CellTags->"Twist2QuarkOperator"]
}, Open  ]],
Cell[2379010, 93360, 81, 1, 24, "Text",
 CellTags->"Twist2QuarkOperator"],
Cell[CellGroupData[{
Cell[2379116, 93365, 305, 9, 26, "Input",
 CellTags->"Twist2QuarkOperator"],
Cell[2379424, 93376, 1536, 55, 57, "Output",
 CellTags->"Twist2QuarkOperator"]
}, Open  ]],
Cell[2380975, 93434, 87, 1, 24, "Text",
 CellTags->"Twist2QuarkOperator"],
Cell[CellGroupData[{
Cell[2381087, 93439, 380, 11, 26, "Input",
 CellTags->"Twist2QuarkOperator"],
Cell[2381470, 93452, 4816, 160, 114, "Output",
 CellTags->"Twist2QuarkOperator"]
}, Open  ]],
Cell[2386301, 93615, 93, 1, 24, "Text",
 CellTags->"Twist2QuarkOperator"],
Cell[CellGroupData[{
Cell[2386419, 93620, 197, 6, 26, "Input",
 CellTags->"Twist2QuarkOperator"],
Cell[2386619, 93628, 305, 9, 38, "Output",
 CellTags->"Twist2QuarkOperator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2386961, 93642, 376, 10, 26, "Input",
 CellTags->"Twist2QuarkOperator"],
Cell[2387340, 93654, 2013, 73, 52, "Output",
 CellTags->"Twist2QuarkOperator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2389390, 93732, 449, 12, 26, "Input",
 CellTags->"Twist2QuarkOperator"],
Cell[2389842, 93746, 9449, 299, 127, "Output",
 CellTags->"Twist2QuarkOperator"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[2399340, 94051, 102, 1, 34, "Subsubsection",
 CellTags->"Twist2QuarkOperator"],
Cell[CellGroupData[{
Cell[2399467, 94056, 269, 6, 26, "Input",
 CellTags->"Twist2QuarkOperator"],
Cell[2399739, 94064, 488, 13, 26, "Output",
 CellTags->"Twist2QuarkOperator"]
}, Open  ]],
Cell[2400242, 94080, 178, 4, 24, "Text",
 CellTags->"Twist2QuarkOperator"],
Cell[2400423, 94086, 275, 5, 41, "Text",
 CellTags->"Twist2QuarkOperator"],
Cell[CellGroupData[{
Cell[2400723, 94095, 205, 6, 26, "Input",
 CellTags->"Twist2QuarkOperator"],
Cell[2400931, 94103, 842, 29, 28, "Output",
 CellTags->"Twist2QuarkOperator"]
}, Open  ]],
Cell[2401788, 94135, 100, 1, 24, "Text",
 CellTags->"Twist2QuarkOperator"],
Cell[CellGroupData[{
Cell[2401913, 94140, 305, 9, 26, "Input",
 CellTags->"Twist2QuarkOperator"],
Cell[2402221, 94151, 2347, 79, 57, "Output",
 CellTags->"Twist2QuarkOperator"]
}, Open  ]],
Cell[2404583, 94233, 87, 1, 24, "Text",
 CellTags->"Twist2QuarkOperator"],
Cell[CellGroupData[{
Cell[2404695, 94238, 156, 4, 26, "Input",
 CellTags->"Twist2QuarkOperator"],
Cell[2404854, 94244, 1144, 34, 63, "Output",
 CellTags->"Twist2QuarkOperator"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[2406047, 94284, 104, 1, 34, "Subsubsection",
 CellTags->"Twist2QuarkOperator"],
Cell[CellGroupData[{
Cell[2406176, 94289, 269, 6, 26, "Input",
 CellTags->"Twist2QuarkOperator"],
Cell[2406448, 94297, 486, 13, 26, "Output",
 CellTags->"Twist2QuarkOperator"]
}, Open  ]],
Cell[2406949, 94313, 75, 1, 24, "Text",
 CellTags->"Twist2QuarkOperator"],
Cell[CellGroupData[{
Cell[2407049, 94318, 205, 6, 26, "Input",
 CellTags->"Twist2QuarkOperator"],
Cell[2407257, 94326, 842, 29, 28, "Output",
 CellTags->"Twist2QuarkOperator"]
}, Open  ]],
Cell[2408114, 94358, 81, 1, 24, "Text",
 CellTags->"Twist2QuarkOperator"],
Cell[CellGroupData[{
Cell[2408220, 94363, 305, 9, 26, "Input",
 CellTags->"Twist2QuarkOperator"],
Cell[2408528, 94374, 2347, 79, 57, "Output",
 CellTags->"Twist2QuarkOperator"]
}, Open  ]],
Cell[2410890, 94456, 210, 6, 26, "Input",
 CellTags->"Twist2QuarkOperator"]
}, Open  ]],
Cell[2411115, 94465, 75, 1, 39, "ObjectName",
 CellTags->"Twist3QuarkOperator"],
Cell[CellGroupData[{
Cell[2411215, 94470, 70, 1, 34, "Subsubsection",
 CellTags->"Twist3QuarkOperator"],
Cell[2411288, 94473, 768, 12, 109, "Text",
 CellTags->"Twist3QuarkOperator"],
Cell[CellGroupData[{
Cell[2412081, 94489, 134, 3, 26, "Input",
 CellTags->"Twist3QuarkOperator"],
Cell[2412218, 94494, 370, 11, 26, "Output",
 CellTags->"Twist3QuarkOperator"]
}, Open  ]],
Cell[2412603, 94508, 366, 14, 24, "Text",
 CellTags->"Twist3QuarkOperator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2413006, 94527, 67, 1, 34, "Subsubsection",
 CellTags->"Twist3QuarkOperator"],
Cell[CellGroupData[{
Cell[2413098, 94532, 132, 3, 26, "Input",
 CellTags->"Twist3QuarkOperator"],
Cell[2413233, 94537, 696, 24, 28, "Output",
 CellTags->"Twist3QuarkOperator"]
}, Open  ]]
}, Open  ]],
Cell[2413956, 94565, 75, 1, 39, "ObjectName",
 CellTags->"Twist4GluonOperator"],
Cell[CellGroupData[{
Cell[2414056, 94570, 70, 1, 34, "Subsubsection",
 CellTags->"Twist4GluonOperator"],
Cell[2414129, 94573, 150, 4, 24, "Text",
 CellTags->"Twist4GluonOperator"],
Cell[2414282, 94579, 509, 19, 24, "Text",
 CellTags->"Twist4GluonOperator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2414828, 94603, 67, 1, 34, "Subsubsection",
 CellTags->"Twist4GluonOperator"],
Cell[CellGroupData[{
Cell[2414920, 94608, 529, 15, 46, "Input",
 CellTags->"Twist4GluonOperator"],
Cell[2415452, 94625, 119873, 4589, 1159, "Output",
 CellTags->"Twist4GluonOperator"]
}, Open  ]]
}, Open  ]],
Cell[2535352, 99218, 104, 1, 39, "ObjectName",
 CellTags->"TwoLoopSimplify"],
Cell[CellGroupData[{
Cell[2535481, 99223, 66, 1, 34, "Subsubsection",
 CellTags->"TwoLoopSimplify"],
Cell[2535550, 99226, 335, 6, 58, "Text",
 CellTags->"TwoLoopSimplify"],
Cell[2535888, 99234, 207, 9, 24, "Text",
 CellTags->"TwoLoopSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2536132, 99248, 63, 1, 34, "Subsubsection",
 CellTags->"TwoLoopSimplify"],
Cell[2536198, 99251, 82, 2, 26, "Input",
 CellTags->"TwoLoopSimplify"]
}, Open  ]],
Cell[2536295, 99256, 57, 1, 39, "ObjectName",
 CellTags->"Uncontract"],
Cell[CellGroupData[{
Cell[2536377, 99261, 61, 1, 34, "Subsubsection",
 CellTags->"Uncontract"],
Cell[2536441, 99264, 256, 5, 41, "Text",
 CellTags->"Uncontract"],
Cell[CellGroupData[{
Cell[2536722, 99273, 120, 3, 26, "Input",
 CellTags->"Uncontract"],
Cell[2536845, 99278, 414, 12, 26, "Output",
 CellTags->"Uncontract"]
}, Open  ]],
Cell[2537274, 99293, 181, 9, 24, "Text",
 CellTags->"Uncontract"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2537492, 99307, 58, 1, 34, "Subsubsection",
 CellTags->"Uncontract"],
Cell[CellGroupData[{
Cell[2537575, 99312, 234, 7, 26, "Input",
 CellTags->"Uncontract"],
Cell[2537812, 99321, 441, 18, 26, "Output",
 CellTags->"Uncontract"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2538290, 99344, 143, 4, 26, "Input",
 CellTags->"Uncontract"],
Cell[2538436, 99350, 823, 32, 28, "Output",
 CellTags->"Uncontract"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2539296, 99387, 138, 4, 26, "Input",
 CellTags->"Uncontract"],
Cell[2539437, 99393, 266, 9, 28, "Output",
 CellTags->"Uncontract"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2539740, 99407, 143, 4, 26, "Input",
 CellTags->"Uncontract"],
Cell[2539886, 99413, 583, 21, 28, "Output",
 CellTags->"Uncontract"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2540506, 99439, 153, 4, 26, "Input",
 CellTags->"Uncontract"],
Cell[2540662, 99445, 1182, 45, 28, "Output",
 CellTags->"Uncontract"]
}, Open  ]],
Cell[2541859, 99493, 89, 1, 24, "Text",
 CellTags->"Uncontract"],
Cell[CellGroupData[{
Cell[2541973, 99498, 208, 6, 26, "Input",
 CellTags->"Uncontract"],
Cell[2542184, 99506, 249, 9, 28, "Output",
 CellTags->"Uncontract"]
}, Open  ]],
Cell[2542448, 99518, 152, 4, 24, "Text",
 CellTags->"Uncontract"],
Cell[CellGroupData[{
Cell[2542625, 99526, 248, 7, 26, "Input",
 CellTags->"Uncontract"],
Cell[2542876, 99535, 612, 23, 28, "Output",
 CellTags->"Uncontract"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2543525, 99563, 274, 8, 26, "Input",
 CellTags->"Uncontract"],
Cell[2543802, 99573, 1072, 41, 28, "Output",
 CellTags->"Uncontract"]
}, Open  ]],
Cell[2544889, 99617, 134, 4, 26, "Input",
 CellTags->"Uncontract"]
}, Open  ]],
Cell[2545038, 99624, 83, 1, 39, "ObjectName",
 CellTags->"UnDeclareNonCommutative"],
Cell[CellGroupData[{
Cell[2545146, 99629, 74, 1, 34, "Subsubsection",
 CellTags->"UnDeclareNonCommutative"],
Cell[2545223, 99632, 198, 4, 24, "Text",
 CellTags->"UnDeclareNonCommutative"],
Cell[2545424, 99638, 343, 14, 24, "Text",
 CellTags->"UnDeclareNonCommutative"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2545804, 99657, 71, 1, 34, "Subsubsection",
 CellTags->"UnDeclareNonCommutative"],
Cell[2545878, 99660, 138, 3, 26, "Input",
 CellTags->"UnDeclareNonCommutative"],
Cell[2546019, 99665, 153, 4, 24, "Text",
 CellTags->"UnDeclareNonCommutative"],
Cell[CellGroupData[{
Cell[2546197, 99673, 107, 2, 26, "Input",
 CellTags->"UnDeclareNonCommutative"],
Cell[2546307, 99677, 167, 4, 26, "Output",
 CellTags->"UnDeclareNonCommutative"]
}, Open  ]],
Cell[2546489, 99684, 103, 1, 24, "Text",
 CellTags->"UnDeclareNonCommutative"],
Cell[2546595, 99687, 140, 3, 26, "Input",
 CellTags->"UnDeclareNonCommutative"],
Cell[CellGroupData[{
Cell[2546760, 99694, 161, 4, 26, "Input",
 CellTags->"UnDeclareNonCommutative"],
Cell[2546924, 99700, 165, 4, 26, "Output",
 CellTags->"UnDeclareNonCommutative"]
}, Open  ]],
Cell[2547104, 99707, 161, 4, 26, "Input",
 CellTags->"UnDeclareNonCommutative"],
Cell[CellGroupData[{
Cell[2547290, 99715, 171, 4, 26, "Input",
 CellTags->"UnDeclareNonCommutative"],
Cell[2547464, 99721, 215, 6, 26, "Output",
 CellTags->"UnDeclareNonCommutative"]
}, Open  ]],
Cell[2547694, 99730, 163, 4, 26, "Input",
 CellTags->"UnDeclareNonCommutative"],
Cell[CellGroupData[{
Cell[2547882, 99738, 171, 4, 26, "Input",
 CellTags->"UnDeclareNonCommutative"],
Cell[2548056, 99744, 217, 6, 26, "Output",
 CellTags->"UnDeclareNonCommutative"]
}, Open  ]]
}, Open  ]],
Cell[2548300, 99754, 47, 1, 39, "ObjectName",
 CellTags->"Upper"],
Cell[CellGroupData[{
Cell[2548372, 99759, 56, 1, 34, "Subsubsection",
 CellTags->"Upper"],
Cell[2548431, 99762, 126, 4, 24, "Text",
 CellTags->"Upper"],
Cell[2548560, 99768, 397, 18, 24, "Text",
 CellTags->"Upper"]
}, Open  ]],
Cell[2548972, 99789, 86, 1, 39, "ObjectName",
 CellTags->"UVPart"],
Cell[CellGroupData[{
Cell[2549083, 99794, 57, 1, 34, "Subsubsection",
 CellTags->"UVPart"],
Cell[2549143, 99797, 128, 4, 24, "Text",
 CellTags->"UVPart"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2549308, 99806, 54, 1, 34, "Subsubsection",
 CellTags->"UVPart"],
Cell[2549365, 99809, 73, 2, 26, "Input",
 CellTags->"UVPart"]
}, Open  ]],
Cell[2549453, 99814, 51, 1, 39, "ObjectName",
 CellTags->"Vectors"],
Cell[CellGroupData[{
Cell[2549529, 99819, 58, 1, 34, "Subsubsection",
 CellTags->"Vectors"],
Cell[2549590, 99822, 203, 4, 41, "Text",
 CellTags->"Vectors"],
Cell[2549796, 99828, 188, 8, 24, "Text",
 CellTags->"Vectors"]
}, Open  ]],
Cell[2549999, 99839, 53, 1, 39, "ObjectName",
 CellTags->"WriteOut"],
Cell[CellGroupData[{
Cell[2550077, 99844, 59, 1, 34, "Subsubsection",
 CellTags->"WriteOut"],
Cell[2550139, 99847, 216, 5, 41, "Text",
 CellTags->"WriteOut"],
Cell[2550358, 99854, 171, 8, 24, "Text",
 CellTags->"WriteOut"]
}, Open  ]],
Cell[2550544, 99865, 61, 1, 39, "ObjectName",
 CellTags->"WriteOutPaVe"],
Cell[CellGroupData[{
Cell[2550630, 99870, 63, 1, 34, "Subsubsection",
 CellTags->"WriteOutPaVe"],
Cell[2550696, 99873, 260, 5, 41, "Text",
 CellTags->"WriteOutPaVe"],
Cell[2550959, 99880, 398, 18, 24, "Text",
 CellTags->"WriteOutPaVe"]
}, Open  ]],
Cell[2551372, 99901, 49, 1, 39, "ObjectName",
 CellTags->"Write2"],
Cell[CellGroupData[{
Cell[2551446, 99906, 57, 1, 34, "Subsubsection",
 CellTags->"Write2"],
Cell[2551506, 99909, 297, 6, 41, "Text",
 CellTags->"Write2"],
Cell[2551806, 99917, 153, 4, 24, "Text",
 CellTags->"Write2"],
Cell[CellGroupData[{
Cell[2551984, 99925, 136, 3, 26, "Input",
 CellTags->"Write2"],
Cell[2552123, 99930, 290, 10, 39, "Output",
 CellTags->"Write2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2552450, 99945, 112, 3, 26, "Input",
 CellTags->"Write2"],
Cell[2552565, 99950, 681, 16, 47, "Output",
 CellTags->"Write2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2553283, 99971, 115, 3, 26, "Input",
 CellTags->"Write2"],
Cell[2553401, 99976, 177, 5, 26, "Output",
 CellTags->"Write2"]
}, Open  ]],
Cell[2553593, 99984, 290, 14, 24, "Text",
 CellTags->"Write2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2553920, 100003, 54, 1, 34, "Subsubsection",
 CellTags->"Write2"],
Cell[CellGroupData[{
Cell[2553999, 100008, 429, 15, 38, "Input",
 CellTags->"Write2"],
Cell[2554431, 100025, 896, 30, 32, "Output",
 CellTags->"Write2"]
}, Open  ]],
Cell[2555342, 100058, 208, 7, 26, "Input",
 CellTags->"Write2"],
Cell[2555553, 100067, 78, 1, 24, "Text",
 CellTags->"Write2"],
Cell[2555634, 100070, 184, 6, 26, "Input",
 CellTags->"Write2"],
Cell[2555821, 100078, 73, 1, 24, "Text",
 CellTags->"Write2"],
Cell[CellGroupData[{
Cell[2555919, 100083, 363, 11, 26, "Input",
 CellTags->"Write2"],
Cell[2556285, 100096, 843, 23, 73, "Output",
 CellTags->"Write2"]
}, Open  ]],
Cell[2557143, 100122, 286, 8, 26, "Input",
 CellTags->"Write2"],
Cell[CellGroupData[{
Cell[2557454, 100134, 233, 7, 26, "Input",
 CellTags->"Write2"],
Cell[2557690, 100143, 752, 28, 26, "Output",
 CellTags->"Write2"]
}, Open  ]],
Cell[2558457, 100174, 185, 6, 26, "Input",
 CellTags->"Write2"],
Cell[CellGroupData[{
Cell[2558667, 100184, 363, 11, 26, "Input",
 CellTags->"Write2"],
Cell[2559033, 100197, 1103, 31, 196, "Output",
 CellTags->"Write2"]
}, Open  ]],
Cell[2560151, 100231, 286, 8, 26, "Input",
 CellTags->"Write2"],
Cell[2560440, 100241, 106, 3, 24, "Text",
 CellTags->"Write2"],
Cell[2560549, 100246, 247, 8, 26, "Input",
 CellTags->"Write2"],
Cell[CellGroupData[{
Cell[2560821, 100258, 363, 11, 26, "Input",
 CellTags->"Write2"],
Cell[2561187, 100271, 996, 26, 128, "Output",
 CellTags->"Write2"]
}, Open  ]],
Cell[2562198, 100300, 397, 11, 46, "Input",
 CellTags->"Write2"]
}, Open  ]],
Cell[2562610, 100314, 43, 1, 39, "ObjectName",
 CellTags->"XYT"],
Cell[CellGroupData[{
Cell[2562678, 100319, 54, 1, 34, "Subsubsection",
 CellTags->"XYT"],
Cell[2562735, 100322, 215, 9, 41, "Text",
 CellTags->"XYT"]
}, Open  ]],
Cell[2562965, 100334, 79, 1, 57, "ObjectName",
 CellTags->"ZeroMomentumInsertion"],
Cell[CellGroupData[{
Cell[2563069, 100339, 72, 1, 34, "Subsubsection",
 CellTags->"ZeroMomentumInsertion"],
Cell[2563144, 100342, 154, 4, 24, "Text",
 CellTags->"ZeroMomentumInsertion"],
Cell[2563301, 100348, 473, 18, 24, "Text",
 CellTags->"ZeroMomentumInsertion"]
}, Open  ]],
Cell[2563789, 100369, 47, 1, 39, "ObjectName",
 CellTags->"Zeta2"],
Cell[CellGroupData[{
Cell[2563861, 100374, 56, 1, 34, "Subsubsection",
 CellTags->"Zeta2"],
Cell[2563920, 100377, 94, 1, 24, "Text",
 CellTags->"Zeta2"],
Cell[2564017, 100380, 191, 8, 24, "Text",
 CellTags->"Zeta2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2564245, 100393, 53, 1, 34, "Subsubsection",
 CellTags->"Zeta2"],
Cell[CellGroupData[{
Cell[2564323, 100398, 77, 2, 26, "Input",
 CellTags->"Zeta2"],
Cell[2564403, 100402, 178, 5, 26, "Output",
 CellTags->"Zeta2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2564618, 100412, 104, 3, 26, "Input",
 CellTags->"Zeta2"],
Cell[2564725, 100417, 164, 4, 26, "Output",
 CellTags->"Zeta2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2564926, 100426, 138, 4, 26, "Input",
 CellTags->"Zeta2"],
Cell[2565067, 100432, 201, 6, 26, "Output",
 CellTags->"Zeta2"]
}, Open  ]]
}, Open  ]],
Cell[2565295, 100442, 65, 1, 39, "ObjectName",
 CellTags->"$Abbreviations"],
Cell[CellGroupData[{
Cell[2565385, 100447, 65, 1, 34, "Subsubsection",
 CellTags->"$Abbreviations"],
Cell[2565453, 100450, 293, 6, 41, "Text",
 CellTags->"$Abbreviations"],
Cell[CellGroupData[{
Cell[2565771, 100460, 129, 3, 26, "Input",
 CellTags->"$Abbreviations"],
Cell[2565903, 100465, 909, 18, 77, "Output",
 CellTags->"$Abbreviations"]
}, Open  ]],
Cell[2566827, 100486, 652, 29, 24, "Text",
 CellTags->"$Abbreviations"]
}, Open  ]],
Cell[2567494, 100518, 43, 1, 39, "ObjectName",
 CellTags->"$AL"],
Cell[CellGroupData[{
Cell[2567562, 100523, 54, 1, 34, "Subsubsection",
 CellTags->"$AL"],
Cell[2567619, 100526, 114, 3, 24, "Text",
 CellTags->"$AL"],
Cell[CellGroupData[{
Cell[2567758, 100533, 73, 2, 26, "Input",
 CellTags->"$AL"],
Cell[2567834, 100537, 146, 4, 26, "Output",
 CellTags->"$AL"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2568017, 100546, 241, 7, 26, "Input",
 CellTags->"$AL"],
Cell[2568261, 100555, 605, 23, 28, "Output",
 CellTags->"$AL"]
}, Open  ]],
Cell[2568881, 100581, 117, 4, 26, "Input",
 CellTags->"$AL"],
Cell[CellGroupData[{
Cell[2569023, 100589, 241, 7, 26, "Input",
 CellTags->"$AL"],
Cell[2569267, 100598, 521, 21, 28, "Output",
 CellTags->"$AL"]
}, Open  ]],
Cell[2569803, 100622, 109, 4, 26, "Input",
 CellTags->"$AL"]
}, Open  ]],
Cell[2569927, 100629, 61, 1, 39, "ObjectName",
 CellTags->"$BreitMaison"],
Cell[CellGroupData[{
Cell[2570013, 100634, 63, 1, 34, "Subsubsection",
 CellTags->"$BreitMaison"],
Cell[2570079, 100637, 319, 9, 41, "Text",
 CellTags->"$BreitMaison"],
Cell[2570401, 100648, 370, 19, 24, "Text",
 CellTags->"$BreitMaison"]
}, Open  ]],
Cell[2570786, 100670, 49, 1, 39, "ObjectName",
 CellTags->"$Color"],
Cell[CellGroupData[{
Cell[2570860, 100675, 57, 1, 34, "Subsubsection",
 CellTags->"$Color"],
Cell[2570920, 100678, 130, 4, 24, "Text",
 CellTags->"$Color"]
}, Open  ]],
Cell[2571065, 100685, 57, 1, 39, "ObjectName",
 CellTags->"$Covariant"],
Cell[CellGroupData[{
Cell[2571147, 100690, 61, 1, 34, "Subsubsection",
 CellTags->"$Covariant"],
Cell[2571211, 100693, 183, 4, 24, "Text",
 CellTags->"$Covariant"],
Cell[CellGroupData[{
Cell[2571419, 100701, 87, 2, 26, "Input",
 CellTags->"$Covariant"],
Cell[2571509, 100705, 155, 4, 26, "Output",
 CellTags->"$Covariant"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2571701, 100714, 110, 3, 26, "Input",
 CellTags->"$Covariant"],
Cell[2571814, 100719, 184, 5, 26, "Output",
 CellTags->"$Covariant"]
}, Open  ]],
Cell[2572013, 100727, 130, 4, 26, "Input",
 CellTags->"$Covariant"],
Cell[CellGroupData[{
Cell[2572168, 100735, 110, 3, 26, "Input",
 CellTags->"$Covariant"],
Cell[2572281, 100740, 182, 5, 29, "Output",
 CellTags->"$Covariant"]
}, Open  ]],
Cell[2572478, 100748, 131, 4, 26, "Input",
 CellTags->"$Covariant"]
}, Open  ]],
Cell[2572624, 100755, 45, 1, 39, "ObjectName",
 CellTags->"$FCS"],
Cell[CellGroupData[{
Cell[2572694, 100760, 55, 1, 34, "Subsubsection",
 CellTags->"$FCS"],
Cell[2572752, 100763, 142, 4, 24, "Text",
 CellTags->"$FCS"],
Cell[CellGroupData[{
Cell[2572919, 100771, 75, 2, 26, "Input",
 CellTags->"$FCS"],
Cell[2572997, 100775, 741, 15, 47, "Output",
 CellTags->"$FCS"]
}, Open  ]]
}, Open  ]],
Cell[2573765, 100794, 45, 1, 39, "ObjectName",
 CellTags->"$FCT"],
Cell[CellGroupData[{
Cell[2573835, 100799, 55, 1, 34, "Subsubsection",
 CellTags->"$FCT"],
Cell[2573893, 100802, 177, 4, 24, "Text",
 CellTags->"$FCT"],
Cell[2574073, 100808, 128, 4, 26, "Input",
 CellTags->"$FCT"],
Cell[CellGroupData[{
Cell[2574226, 100816, 170, 5, 26, "Input",
 CellTags->"$FCT"],
Cell[2574399, 100823, 238, 7, 26, "Output",
 CellTags->"$FCT"]
}, Open  ]],
Cell[2574652, 100833, 129, 4, 26, "Input",
 CellTags->"$FCT"],
Cell[CellGroupData[{
Cell[2574806, 100841, 170, 5, 26, "Input",
 CellTags->"$FCT"],
Cell[2574979, 100848, 238, 7, 26, "Output",
 CellTags->"$FCT"]
}, Open  ]]
}, Open  ]],
Cell[2575244, 100859, 65, 1, 39, "ObjectName",
 CellTags->"$FeynCalcStuff"],
Cell[CellGroupData[{
Cell[2575334, 100864, 65, 1, 34, "Subsubsection",
 CellTags->"$FeynCalcStuff"],
Cell[2575402, 100867, 102, 1, 24, "Text",
 CellTags->"$FeynCalcStuff"]
}, Open  ]],
Cell[2575519, 100871, 69, 1, 39, "ObjectName",
 CellTags->"$FeynCalcVersion"],
Cell[CellGroupData[{
Cell[2575613, 100876, 67, 1, 34, "Subsubsection",
 CellTags->"$FeynCalcVersion"],
Cell[2575683, 100879, 124, 3, 24, "Text",
 CellTags->"$FeynCalcVersion"],
Cell[CellGroupData[{
Cell[2575832, 100886, 99, 2, 26, "Input",
 CellTags->"$FeynCalcVersion"],
Cell[2575934, 100890, 172, 4, 26, "Output",
 CellTags->"$FeynCalcVersion"]
}, Open  ]]
}, Open  ]],
Cell[2576133, 100898, 95, 1, 39, "ObjectName",
 CellTags->"$FortranContinuationCharacter"],
Cell[CellGroupData[{
Cell[2576253, 100903, 80, 1, 34, "Subsubsection",
 CellTags->"$FortranContinuationCharacter"],
Cell[2576336, 100906, 143, 3, 24, "Text",
 CellTags->"$FortranContinuationCharacter"],
Cell[CellGroupData[{
Cell[2576504, 100913, 125, 2, 26, "Input",
 CellTags->"$FortranContinuationCharacter"],
Cell[2576632, 100917, 176, 4, 26, "Output",
 CellTags->"$FortranContinuationCharacter"]
}, Open  ]],
Cell[2576823, 100924, 194, 9, 24, "Text",
 CellTags->"$FortranContinuationCharacter"]
}, Open  ]],
Cell[2577032, 100936, 49, 1, 39, "ObjectName",
 CellTags->"$Gauge"],
Cell[CellGroupData[{
Cell[2577106, 100941, 57, 1, 34, "Subsubsection",
 CellTags->"$Gauge"],
Cell[2577166, 100944, 260, 5, 41, "Text",
 CellTags->"$Gauge"],
Cell[2577429, 100951, 168, 9, 24, "Text",
 CellTags->"$Gauge"]
}, Open  ]],
Cell[2577612, 100963, 49, 1, 39, "ObjectName",
 CellTags->"$Larin"],
Cell[CellGroupData[{
Cell[2577686, 100968, 57, 1, 34, "Subsubsection",
 CellTags->"$Larin"],
Cell[2577746, 100971, 1133, 30, 92, "Text",
 CellTags->"$Larin"],
Cell[CellGroupData[{
Cell[2578904, 101005, 79, 2, 26, "Input",
 CellTags->"$Larin"],
Cell[2578986, 101009, 149, 4, 26, "Output",
 CellTags->"$Larin"]
}, Open  ]],
Cell[2579150, 101016, 377, 18, 24, "Text",
 CellTags->"$Larin"]
}, Open  ]],
Cell[2579542, 101037, 55, 1, 39, "ObjectName",
 CellTags->"$LimitTo4"],
Cell[CellGroupData[{
Cell[2579622, 101042, 60, 1, 34, "Subsubsection",
 CellTags->"$LimitTo4"],
Cell[2579685, 101045, 203, 4, 41, "Text",
 CellTags->"$LimitTo4"],
Cell[CellGroupData[{
Cell[2579913, 101053, 85, 2, 26, "Input",
 CellTags->"$LimitTo4"],
Cell[2580001, 101057, 151, 4, 26, "Output",
 CellTags->"$LimitTo4"]
}, Open  ]],
Cell[2580167, 101064, 391, 19, 24, "Text",
 CellTags->"$LimitTo4"]
}, Open  ]],
Cell[2580573, 101086, 67, 1, 39, "ObjectName",
 CellTags->"$LorentzIndices"],
Cell[CellGroupData[{
Cell[2580665, 101091, 66, 1, 34, "Subsubsection",
 CellTags->"$LorentzIndices"],
Cell[2580734, 101094, 164, 4, 24, "Text",
 CellTags->"$LorentzIndices"],
Cell[2580901, 101100, 140, 4, 26, "Input",
 CellTags->"$LorentzIndices"],
Cell[CellGroupData[{
Cell[2581066, 101108, 333, 9, 26, "Input",
 CellTags->"$LorentzIndices"],
Cell[2581402, 101119, 713, 29, 26, "Output",
 CellTags->"$LorentzIndices"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2582152, 101153, 117, 3, 26, "Input",
 CellTags->"$LorentzIndices"],
Cell[2582272, 101158, 424, 11, 38, "Output",
 CellTags->"$LorentzIndices"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2582733, 101174, 302, 9, 26, "Input",
 CellTags->"$LorentzIndices"],
Cell[2583038, 101185, 926, 39, 28, "Output",
 CellTags->"$LorentzIndices"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2584001, 101229, 117, 3, 26, "Input",
 CellTags->"$LorentzIndices"],
Cell[2584121, 101234, 697, 20, 58, "Output",
 CellTags->"$LorentzIndices"]
}, Open  ]],
Cell[2584833, 101257, 141, 4, 26, "Input",
 CellTags->"$LorentzIndices"]
}, Open  ]],
Cell[2584989, 101264, 59, 1, 39, "ObjectName",
 CellTags->"$MIntegrate"],
Cell[CellGroupData[{
Cell[2585073, 101269, 62, 1, 34, "Subsubsection",
 CellTags->"$MIntegrate"],
Cell[2585138, 101272, 136, 4, 24, "Text",
 CellTags->"$MIntegrate"],
Cell[CellGroupData[{
Cell[2585299, 101280, 89, 2, 26, "Input",
 CellTags->"$MIntegrate"],
Cell[2585391, 101284, 167, 5, 26, "Output",
 CellTags->"$MIntegrate"]
}, Open  ]],
Cell[2585573, 101292, 191, 9, 24, "Text",
 CellTags->"$MIntegrate"]
}, Open  ]],
Cell[2585779, 101304, 43, 1, 39, "ObjectName",
 CellTags->"$MU"],
Cell[CellGroupData[{
Cell[2585847, 101309, 54, 1, 34, "Subsubsection",
 CellTags->"$MU"],
Cell[2585904, 101312, 151, 4, 24, "Text",
 CellTags->"$MU"],
Cell[CellGroupData[{
Cell[2586080, 101320, 73, 2, 26, "Input",
 CellTags->"$MU"],
Cell[2586156, 101324, 144, 4, 26, "Output",
 CellTags->"$MU"]
}, Open  ]],
Cell[2586315, 101331, 174, 9, 24, "Text",
 CellTags->"$MU"]
}, Open  ]],
Cell[2586504, 101343, 69, 1, 39, "ObjectName",
 CellTags->"$MemoryAvailable"],
Cell[CellGroupData[{
Cell[2586598, 101348, 67, 1, 34, "Subsubsection",
 CellTags->"$MemoryAvailable"],
Cell[2586668, 101351, 434, 11, 58, "Text",
 CellTags->"$MemoryAvailable"],
Cell[CellGroupData[{
Cell[2587127, 101366, 99, 2, 26, "Input",
 CellTags->"$MemoryAvailable"],
Cell[2587229, 101370, 157, 4, 26, "Output",
 CellTags->"$MemoryAvailable"]
}, Open  ]]
}, Open  ]],
Cell[2587413, 101378, 53, 1, 39, "ObjectName",
 CellTags->"$NonComm"],
Cell[CellGroupData[{
Cell[2587491, 101383, 59, 1, 34, "Subsubsection",
 CellTags->"$NonComm"],
Cell[2587553, 101386, 109, 3, 24, "Text",
 CellTags->"$NonComm"],
Cell[CellGroupData[{
Cell[2587687, 101393, 83, 2, 26, "Input",
 CellTags->"$NonComm"],
Cell[2587773, 101397, 1124, 20, 147, "Output",
 CellTags->"$NonComm"]
}, Open  ]],
Cell[2588912, 101420, 307, 14, 24, "Text",
 CellTags->"$NonComm"]
}, Open  ]],
Cell[2589234, 101437, 63, 1, 39, "ObjectName",
 CellTags->"$PairBrackets"],
Cell[CellGroupData[{
Cell[2589322, 101442, 64, 1, 34, "Subsubsection",
 CellTags->"$PairBrackets"],
Cell[2589389, 101445, 155, 4, 24, "Text",
 CellTags->"$PairBrackets"],
Cell[CellGroupData[{
Cell[2589569, 101453, 93, 2, 26, "Input",
 CellTags->"$PairBrackets"],
Cell[2589665, 101457, 156, 4, 26, "Output",
 CellTags->"$PairBrackets"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2589858, 101466, 204, 7, 26, "Input",
 CellTags->"$PairBrackets"],
Cell[2590065, 101475, 466, 19, 28, "Output",
 CellTags->"$PairBrackets"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2590568, 101499, 118, 3, 26, "Input",
 CellTags->"$PairBrackets"],
Cell[2590689, 101504, 155, 4, 26, "Output",
 CellTags->"$PairBrackets"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2590881, 101513, 204, 7, 26, "Input",
 CellTags->"$PairBrackets"],
Cell[2591088, 101522, 489, 19, 28, "Output",
 CellTags->"$PairBrackets"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2591614, 101546, 119, 3, 26, "Input",
 CellTags->"$PairBrackets"],
Cell[2591736, 101551, 158, 4, 26, "Output",
 CellTags->"$PairBrackets"]
}, Open  ]]
}, Open  ]],
Cell[2591921, 101559, 65, 1, 39, "ObjectName",
 CellTags->"$SpinorMinimal"],
Cell[CellGroupData[{
Cell[2592011, 101564, 65, 1, 34, "Subsubsection",
 CellTags->"$SpinorMinimal"],
Cell[2592079, 101567, 241, 5, 41, "Text",
 CellTags->"$SpinorMinimal"],
Cell[CellGroupData[{
Cell[2592345, 101576, 95, 2, 26, "Input",
 CellTags->"$SpinorMinimal"],
Cell[2592443, 101580, 157, 4, 26, "Output",
 CellTags->"$SpinorMinimal"]
}, Open  ]],
Cell[2592615, 101587, 200, 9, 24, "Text",
 CellTags->"$SpinorMinimal"]
}, Open  ]],
Cell[2592830, 101599, 61, 1, 39, "ObjectName",
 CellTags->"$VeryVerbose"],
Cell[CellGroupData[{
Cell[2592916, 101604, 63, 1, 34, "Subsubsection",
 CellTags->"$VeryVerbose"],
Cell[2592982, 101607, 221, 5, 41, "Text",
 CellTags->"$VeryVerbose"],
Cell[CellGroupData[{
Cell[2593228, 101616, 91, 2, 26, "Input",
 CellTags->"$VeryVerbose"],
Cell[2593322, 101620, 150, 4, 26, "Output",
 CellTags->"$VeryVerbose"]
}, Open  ]],
Cell[2593487, 101627, 141, 4, 26, "Input",
 CellTags->"$VeryVerbose"],
Cell[CellGroupData[{
Cell[2593653, 101635, 287, 10, 26, "Input",
 CellTags->"$VeryVerbose"],
Cell[2593943, 101647, 165, 4, 21, "Print",
 CellTags->"$VeryVerbose"],
Cell[2594111, 101653, 297, 8, 22, "Print",
 CellTags->"$VeryVerbose"],
Cell[2594411, 101663, 317, 8, 22, "Print",
 CellTags->"$VeryVerbose"]
}, Open  ]],
Cell[2594743, 101674, 131, 4, 26, "Input",
 CellTags->"$VeryVerbose"],
Cell[2594877, 101680, 159, 9, 24, "Text",
 CellTags->"$V0"]
}, Open  ]],
Cell[2595051, 101692, 47, 1, 39, "ObjectName",
 CellTags->"$West"],
Cell[CellGroupData[{
Cell[2595123, 101697, 56, 1, 34, "Subsubsection",
 CellTags->"$West"],
Cell[2595182, 101700, 468, 13, 41, "Text",
 CellTags->"$West"],
Cell[2595653, 101715, 158, 8, 24, "Text",
 CellTags->"$West"],
Cell[2595814, 101725, 54, 0, 29, "Commentary"],
Cell[CellGroupData[{
Cell[2595893, 101729, 125, 3, 26, "Input"],
Cell[2596021, 101734, 152, 3, 26, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2596210, 101742, 85, 2, 26, "Input"],
Cell[2596298, 101746, 132, 3, 26, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2596467, 101754, 87, 2, 26, "Input"],
Cell[2596557, 101758, 134, 3, 26, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2596728, 101766, 82, 2, 26, "Input"],
Cell[2596813, 101770, 130, 3, 26, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[2597004, 101780, 70, 1, 64, "Section",
 CellTags->"Experimental"],
Cell[2597077, 101783, 73, 1, 39, "ObjectName",
 CellTags->"DoPolarizationSums"],
Cell[CellGroupData[{
Cell[2597175, 101788, 69, 1, 34, "Subsubsection",
 CellTags->"DoPolarizationSums"],
Cell[2597247, 101791, 291, 7, 99, "Text",
 CellTags->"DoPolarizationSums"],
Cell[2597541, 101800, 317, 14, 24, "Text",
 CellTags->"DoPolarizationSums"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2597895, 101819, 66, 1, 34, "Subsubsection",
 CellTags->"DoPolarizationSums"],
Cell[2597964, 101822, 310, 6, 41, "Text",
 CellTags->"DoPolarizationSums"],
Cell[2598277, 101830, 123, 3, 24, "Text",
 CellTags->"DoPolarizationSums"],
Cell[CellGroupData[{
Cell[2598425, 101837, 571, 17, 46, "Input",
 CellTags->"DoPolarizationSums"],
Cell[2598999, 101856, 492, 17, 30, "Output",
 CellTags->"DoPolarizationSums"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2599528, 101878, 116, 3, 26, "Input",
 CellTags->"DoPolarizationSums"],
Cell[2599647, 101883, 175, 5, 26, "Output",
 CellTags->"DoPolarizationSums"]
}, Open  ]],
Cell[2599837, 101891, 1270, 48, 27, "Text",
 CellTags->"DoPolarizationSums"],
Cell[2601110, 101941, 562, 21, 24, "Text",
 CellTags->"DoPolarizationSums"],
Cell[2601675, 101964, 142, 4, 26, "Input",
 CellTags->"DoPolarizationSums"],
Cell[2601820, 101970, 311, 10, 26, "Input",
 CellTags->"DoPolarizationSums"],
Cell[CellGroupData[{
Cell[2602156, 101984, 271, 8, 26, "Input",
 CellTags->"DoPolarizationSums"],
Cell[2602430, 101994, 779, 21, 75, "Output",
 CellTags->"DoPolarizationSums"]
}, Open  ]],
Cell[2603224, 102018, 213, 9, 24, "Text",
 CellTags->"DoPolarizationSums"],
Cell[2603440, 102029, 276, 9, 26, "Input",
 CellTags->"DoPolarizationSums"],
Cell[2603719, 102040, 328, 10, 26, "Input",
 CellTags->"DoPolarizationSums"],
Cell[2604050, 102052, 323, 13, 27, "Text",
 CellTags->"DoPolarizationSums"],
Cell[2604376, 102067, 697, 30, 27, "Text",
 CellTags->"DoPolarizationSums"],
Cell[CellGroupData[{
Cell[2605098, 102101, 434, 13, 56, "Input",
 CellTags->"DoPolarizationSums"],
Cell[2605535, 102116, 817, 25, 79, "Output",
 CellTags->"DoPolarizationSums"]
}, Open  ]],
Cell[2606367, 102144, 624, 27, 27, "Text",
 CellTags->"DoPolarizationSums"],
Cell[CellGroupData[{
Cell[2607016, 102175, 433, 13, 56, "Input",
 CellTags->"DoPolarizationSums"],
Cell[2607452, 102190, 817, 25, 79, "Output",
 CellTags->"DoPolarizationSums"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2608306, 102220, 859, 24, 65, "Input",
 CellTags->"DoPolarizationSums"],
Cell[2609168, 102246, 648, 23, 30, "Output",
 CellTags->"DoPolarizationSums"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2609853, 102274, 130, 3, 26, "Input",
 CellTags->"DoPolarizationSums"],
Cell[2609986, 102279, 365, 13, 29, "Output",
 CellTags->"DoPolarizationSums"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2610388, 102297, 116, 3, 26, "Input",
 CellTags->"DoPolarizationSums"],
Cell[2610507, 102302, 175, 5, 26, "Output",
 CellTags->"DoPolarizationSums"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2610719, 102312, 120, 3, 26, "Input",
 CellTags->"DoPolarizationSums"],
Cell[2610842, 102317, 175, 5, 26, "Output",
 CellTags->"DoPolarizationSums"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2611054, 102327, 126, 3, 26, "Input",
 CellTags->"DoPolarizationSums"],
Cell[2611183, 102332, 175, 5, 26, "Output",
 CellTags->"DoPolarizationSums"]
}, Open  ]],
Cell[2611373, 102340, 167, 4, 26, "Input",
 CellTags->"DoPolarizationSums"]
}, Open  ]],
Cell[2611555, 102347, 90, 1, 39, "ObjectName",
 CellTags->"FeynmanDoIntegrals"],
Cell[CellGroupData[{
Cell[2611670, 102352, 69, 1, 34, "Subsubsection",
 CellTags->"FeynmanDoIntegrals"],
Cell[2611742, 102355, 1518, 23, 202, "Text",
 CellTags->"FeynmanDoIntegrals"],
Cell[2613263, 102380, 347, 14, 24, "Text",
 CellTags->"FeynmanDoIntegrals"],
Cell[CellGroupData[{
Cell[2613635, 102398, 136, 3, 26, "Input",
 CellTags->"FeynmanDoIntegrals"],
Cell[2613774, 102403, 1079, 30, 67, "Output",
 CellTags->"FeynmanDoIntegrals"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[2614902, 102439, 66, 1, 34, "Subsubsection",
 CellTags->"FeynmanDoIntegrals"],
Cell[2614971, 102442, 85, 2, 26, "Input",
 CellTags->"FeynmanDoIntegrals"]
}, Open  ]],
Cell[2615071, 102447, 92, 1, 39, "ObjectName",
 CellTags->"FeynmanParametrize1"],
Cell[CellGroupData[{
Cell[2615188, 102452, 70, 1, 34, "Subsubsection",
 CellTags->"FeynmanParametrize1"],
Cell[2615261, 102455, 717, 12, 117, "Text",
 CellTags->"FeynmanParametrize1"],
Cell[CellGroupData[{
Cell[2616003, 102471, 138, 3, 26, "Input",
 CellTags->"FeynmanParametrize1"],
Cell[2616144, 102476, 547, 14, 47, "Output",
 CellTags->"FeynmanParametrize1"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[2616740, 102496, 67, 1, 34, "Subsubsection",
 CellTags->"FeynmanParametrize1"],
Cell[2616810, 102499, 86, 2, 26, "Input",
 CellTags->"FeynmanParametrize1"]
}, Open  ]],
Cell[2616911, 102504, 80, 1, 39, "ObjectName",
 CellTags->"FeynmanReduce"],
Cell[CellGroupData[{
Cell[2617016, 102509, 64, 1, 34, "Subsubsection",
 CellTags->"FeynmanReduce"],
Cell[2617083, 102512, 723, 12, 117, "Text",
 CellTags->"FeynmanReduce"],
Cell[CellGroupData[{
Cell[2617831, 102528, 126, 3, 26, "Input",
 CellTags->"FeynmanReduce"],
Cell[2617960, 102533, 328, 10, 26, "Output",
 CellTags->"FeynmanReduce"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[2618337, 102549, 61, 1, 34, "Subsubsection",
 CellTags->"FeynmanReduce"],
Cell[2618401, 102552, 80, 2, 26, "Input",
 CellTags->"FeynmanReduce"],
Cell[2618484, 102556, 122, 3, 24, "Text",
 CellTags->"Experimental"],
Cell[CellGroupData[{
Cell[2618631, 102563, 198, 6, 26, "Input",
 CellTags->"Experimental"],
Cell[2618832, 102571, 234, 7, 26, "Output",
 CellTags->"Experimental"]
}, Open  ]],
Cell[2619081, 102581, 100, 1, 24, "Text",
 CellTags->"Experimental"],
Cell[CellGroupData[{
Cell[2619206, 102586, 222, 6, 26, "Input",
 CellTags->"Experimental"],
Cell[2619431, 102594, 288, 8, 26, "Output",
 CellTags->"Experimental"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2619756, 102607, 75, 1, 26, "Input"],
Cell[2619834, 102610, 151, 3, 26, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}
]
*)

(* End of internal cache information *)
